
##############################################################################
#
# Copyright (c) 2003-2018 by The University of Queensland
# http://www.uq.edu.au
#
# Primary Business: Queensland, Australia
# Licensed under the Apache License, version 2.0
# http://www.apache.org/licenses/LICENSE-2.0
#
# Development until 2012 by Earth Systems Science Computational Center (ESSCC)
# Development 2012-2013 by School of Earth Sciences
# Development from 2014 by Centre for Geoscience Computing (GeoComp)
#
##############################################################################

from __future__ import print_function, division

__copyright__="""Copyright (c) 2003-2018 by The University of Queensland
http://www.uq.edu.au
Primary Business: Queensland, Australia"""
__license__="""Licensed under the Apache License, version 2.0
http://www.apache.org/licenses/LICENSE-2.0"""
__url__="https://launchpad.net/escript-finley"

"""
test for util operations for slicing operations without tagged data

:remark: use see `test_util`
:var __author__: name of author
:var __copyright__: copyrights
:var __license__: licence agreement
:var __url__: url entry point on documentation
:var __version__: version
:var __date__: date of the version
"""

__author__="Lutz Gross, l.gross@uq.edu.au"

import esys.escriptcore.utestselect as unittest
import numpy
from esys.escript import *
from test_util_base import Test_util_base

class Test_util_slicing_no_tagged_data(Test_util_base):
   """
   test for slicing operations. No tagged data are tested.
   """
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank1_L(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      res=arg[2:]
      ref=Data(numpy.array([2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg=Data(numpy.array([0.0j, 1.0j, 2.0j]),self.functionspace)
      res=arg[2:]
      ref=Data(numpy.array([2.0j]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank1_R(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      res=arg[1:2]
      ref=Data(numpy.array([1.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank1_U(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      res=arg[:2]
      ref=Data(numpy.array([0.0, 1.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank1_C(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      res=arg[:]
      ref=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg=Data(numpy.array([0.0j, 1.0j, 2.0j]),self.functionspace)
      res=arg[:]
      ref=Data(numpy.array([0.0j, 1.0j, 2.0j]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")      
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank1_P(self):
      arg=Data(numpy.array([0.0, 1.0, 2.0]),self.functionspace)
      res=arg[1]
      ref=Data(1.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_RC(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1:2,:]
      ref=Data(numpy.array([[4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_UN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:2]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_RR(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1:2,1:3]
      ref=Data(numpy.array([[5.0, 6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_UP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:2,2]
      ref=Data(numpy.array([2.0, 6.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_PL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1,2:]
      ref=Data(numpy.array([6.0, 7.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_PP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1,2]
      ref=Data(6.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_LN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[2:]
      ref=Data(numpy.array([[8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_RL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1:2,2:]
      ref=Data(numpy.array([[6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_UC(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:2,:]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_UL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:2,2:]
      ref=Data(numpy.array([[2.0, 3.0], [6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_UU(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:2,:3]
      ref=Data(numpy.array([[0.0, 1.0, 2.0], [4.0, 5.0, 6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_CC(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:,:]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_CL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:,2:]
      ref=Data(numpy.array([[2.0, 3.0], [6.0, 7.0], [10.0, 11.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_CP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:,2]
      ref=Data(numpy.array([2.0, 6.0, 10.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_RP(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1:2,2]
      ref=Data(numpy.array([6.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_RN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1:2]
      ref=Data(numpy.array([[4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0j, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1:2]
      ref=Data(numpy.array([[4.0, 5.0, 6.0j, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")      
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_CN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:]
      ref=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_UR(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[:2,1:3]
      ref=Data(numpy.array([[1.0, 2.0], [5.0, 6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_PN(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[1]
      ref=Data(numpy.array([4.0, 5.0, 6.0, 7.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank2_LL(self):
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]]),self.functionspace)
      res=arg[2:,2:]
      ref=Data(numpy.array([[10.0, 11.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UUL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,:3,2:]
      ref=Data(numpy.array([[[2.0], [5.0], [8.0]], [[14.0], [17.0], [20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RLL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,2:,2:]
      ref=Data(numpy.array([[[20.0], [23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UPR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,2,1:2]
      ref=Data(numpy.array([[7.0], [19.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_ULC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,2:,:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CUU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,:3,:2]
      ref=Data(numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]], [[24.0, 25.0], 
[27.0, 28.0], [30.0, 31.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LLU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,2:,:2]
      ref=Data(numpy.array([[[30.0, 31.0], [33.0, 34.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PPL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2,2:]
      ref=Data(numpy.array([20.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PLR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2:,1:2]
      ref=Data(numpy.array([[19.0], [22.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PPP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2,1]
      ref=Data(19.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg=Data(numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0j]]),self.functionspace)
      res=arg[1:2]
      ref=Data(numpy.array([[4.0, 5.0, 6.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")      
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,1:3]
      ref=Data(numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]], [[27.0, 28.0, 29.0], 
[30.0, 31.0, 32.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:]
      ref=Data(numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,2]
      ref=Data(numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PLC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2:,:]
      ref=Data(numpy.array([[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RRR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,1:3,1:2]
      ref=Data(numpy.array([[[16.0], [19.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PPR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2,1:2]
      ref=Data(numpy.array([19.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RRU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,1:3,:2]
      ref=Data(numpy.array([[[15.0, 16.0], [18.0, 19.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RRL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,1:3,2:]
      ref=Data(numpy.array([[[17.0], [20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CLC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,2:,:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[30.0, 31.0, 
32.0], [33.0, 34.0, 35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RCL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:,2:]
      ref=Data(numpy.array([[[14.0], [17.0], [20.0], [23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UCC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,:,:]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,:]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RCU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:,:2]
      ref=Data(numpy.array([[[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], [21.0, 22.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,1:3]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RPC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,2,:]
      ref=Data(numpy.array([[18.0, 19.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CRP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,1:3,1]
      ref=Data(numpy.array([[4.0, 7.0], [16.0, 19.0], [28.0, 31.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RRP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,1:3,1]
      ref=Data(numpy.array([[16.0, 19.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UUP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,:3,1]
      ref=Data(numpy.array([[1.0, 4.0, 7.0], [13.0, 16.0, 19.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_URC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,1:3,:]
      ref=Data(numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,:]
      ref=Data(numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PLU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2:,:2]
      ref=Data(numpy.array([[18.0, 19.0], [21.0, 22.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UPL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,2,2:]
      ref=Data(numpy.array([[8.0], [20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LLP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,2:,1]
      ref=Data(numpy.array([[31.0, 34.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LUR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,:3,1:2]
      ref=Data(numpy.array([[[25.0], [28.0], [31.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,2:]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PPU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2,:2]
      ref=Data(numpy.array([18.0, 19.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CCP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,:,1]
      ref=Data(numpy.array([[1.0, 4.0, 7.0, 10.0], [13.0, 16.0, 19.0, 22.0], [25.0, 28.0, 31.0, 34.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RPL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,2,2:]
      ref=Data(numpy.array([[20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,2]
      ref=Data(numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0], [30.0, 31.0, 32.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2:]
      ref=Data(numpy.array([[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PCU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,:,:2]
      ref=Data(numpy.array([[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], [21.0, 22.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CPP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,2,1]
      ref=Data(numpy.array([7.0, 19.0, 31.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RUP(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:3,1]
      ref=Data(numpy.array([[13.0, 16.0, 19.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CRR(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,1:3,1:2]
      ref=Data(numpy.array([[[4.0], [7.0]], [[16.0], [19.0]], [[28.0], [31.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,:]
      ref=Data(numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_URN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,1:3]
      ref=Data(numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,2]
      ref=Data(numpy.array([18.0, 19.0, 20.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,:3]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LRC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,1:3,:]
      ref=Data(numpy.array([[[27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LLL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,2:,2:]
      ref=Data(numpy.array([[[32.0], [35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UUU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,:3,:2]
      ref=Data(numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 
19.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LCL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,:,2:]
      ref=Data(numpy.array([[[26.0], [29.0], [32.0], [35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,2]
      ref=Data(numpy.array([[30.0, 31.0, 32.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CUL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,:3,2:]
      ref=Data(numpy.array([[[2.0], [5.0], [8.0]], [[14.0], [17.0], [20.0]], [[26.0], [29.0], [32.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:3]
      ref=Data(numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_ULN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,2:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RPN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,2]
      ref=Data(numpy.array([[18.0, 19.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RUU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:3,:2]
      ref=Data(numpy.array([[[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_UPC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:2,2,:]
      ref=Data(numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RLU(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,2:,:2]
      ref=Data(numpy.array([[[18.0, 19.0], [21.0, 22.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_RCC(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1:2,:,:]
      ref=Data(numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,1:3]
      ref=Data(numpy.array([[[27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,:3]
      ref=Data(numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_LLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[2:,2:]
      ref=Data(numpy.array([[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,:3]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CPL(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,2,2:]
      ref=Data(numpy.array([[8.0], [20.0], [32.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CLN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,2:]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[30.0, 31.0, 
32.0], [33.0, 34.0, 35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_CCN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[:,:]
      ref=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PUN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,:3]
      ref=Data(numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank3_PRN(self):
      arg=Data(numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]]),self.functionspace)
      res=arg[1,1:3]
      ref=Data(numpy.array([[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1:2,2]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]], [[156.0, 157.0, 
158.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,2:,:]
      ref=Data(numpy.array([[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,:4,2:]
      ref=Data(numpy.array([[[[122.0], [125.0], [128.0], [131.0]]], [[[167.0], [170.0], [173.0], [176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUPR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,2,1:2]
      ref=Data(numpy.array([[[7.0], [22.0]], [[52.0], [67.0]], [[97.0], [112.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,:4,:2]
      ref=Data(numpy.array([[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 
67.0], [69.0, 70.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]], [[150.0, 151.0], [153.0, 154.0], 
[156.0, 157.0], [159.0, 160.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PUCL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,:,2:]
      ref=Data(numpy.array([[[92.0], [95.0], [98.0], [101.0], [104.0]], [[107.0], [110.0], [113.0], [116.0], 
[119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CRUR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1:2,:4,1:2]
      ref=Data(numpy.array([[[[16.0], [19.0], [22.0], [25.0]]], [[[61.0], [64.0], [67.0], [70.0]]], [[[106.0], [109.0], 
[112.0], [115.0]]], [[[151.0], [154.0], [157.0], [160.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCRL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,1:4,2:]
      ref=Data(numpy.array([[[[95.0], [98.0], [101.0]], [[110.0], [113.0], [116.0]], [[125.0], [128.0], [131.0]]], [[[140.0], 
[143.0], [146.0]], [[155.0], [158.0], [161.0]], [[170.0], [173.0], [176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPLR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,2:,1:2]
      ref=Data(numpy.array([[[67.0], [70.0], [73.0]], [[112.0], [115.0], [118.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,1:4,1]
      ref=Data(numpy.array([[19.0, 22.0, 25.0], [64.0, 67.0, 70.0], [109.0, 112.0, 115.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,1:4,1]
      ref=Data(numpy.array([[[4.0, 7.0, 10.0], [19.0, 22.0, 25.0], [34.0, 37.0, 40.0]], [[49.0, 52.0, 55.0], [64.0, 67.0, 
70.0], [79.0, 82.0, 85.0]], [[94.0, 97.0, 100.0], [109.0, 112.0, 115.0], [124.0, 127.0, 130.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_ULPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,2:,2,1]
      ref=Data(numpy.array([[37.0], [82.0], [127.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRUR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,:4,1:2]
      ref=Data(numpy.array([[[[106.0], [109.0], [112.0], [115.0]]], [[[151.0], [154.0], [157.0], [160.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LPLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1,2:,2:]
      ref=Data(numpy.array([[[113.0], [116.0], [119.0]], [[158.0], [161.0], [164.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,:,:2]
      ref=Data(numpy.array([[[[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0], [132.0, 133.0]]], [[[165.0, 
166.0], [168.0, 169.0], [171.0, 172.0], [174.0, 175.0], [177.0, 178.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PULR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,2:,1:2]
      ref=Data(numpy.array([[[97.0], [100.0], [103.0]], [[112.0], [115.0], [118.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,:,:2]
      ref=Data(numpy.array([[[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0], [57.0, 58.0]], [[60.0, 61.0], [63.0, 
64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]], [[75.0, 76.0], [78.0, 79.0], [81.0, 82.0], [84.0, 85.0], [87.0, 88.0]]], 
[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0], [102.0, 103.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], 
[114.0, 115.0], [117.0, 118.0]], [[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0], [132.0, 
133.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPPC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,2,:]
      ref=Data(numpy.array([111.0, 112.0, 113.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRCR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,:,1:2]
      ref=Data(numpy.array([[[106.0], [109.0], [112.0], [115.0], [118.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,:4,2:]
      ref=Data(numpy.array([[[[2.0], [5.0], [8.0], [11.0]], [[17.0], [20.0], [23.0], [26.0]]], [[[47.0], [50.0], [53.0], 
[56.0]], [[62.0], [65.0], [68.0], [71.0]]], [[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], 
[116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,:]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], 
[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRRC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,1:4,:]
      ref=Data(numpy.array([[[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LULP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,2:,1]
      ref=Data(numpy.array([[[97.0, 100.0, 103.0], [112.0, 115.0, 118.0]], [[142.0, 145.0, 148.0], [157.0, 160.0, 
163.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRUP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,:4,1]
      ref=Data(numpy.array([[[61.0, 64.0, 67.0, 70.0]], [[106.0, 109.0, 112.0, 115.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,:,:]
      ref=Data(numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCUP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,:4,1]
      ref=Data(numpy.array([[[1.0, 4.0, 7.0, 10.0], [16.0, 19.0, 22.0, 25.0], [31.0, 34.0, 37.0, 40.0]], [[46.0, 49.0, 52.0, 
55.0], [61.0, 64.0, 67.0, 70.0], [76.0, 79.0, 82.0, 85.0]], [[91.0, 94.0, 97.0, 100.0], [106.0, 109.0, 112.0, 115.0], [121.0, 
124.0, 127.0, 130.0]], [[136.0, 139.0, 142.0, 145.0], [151.0, 154.0, 157.0, 160.0], [166.0, 169.0, 172.0, 
175.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,2:]
      ref=Data(numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,:4,:2]
      ref=Data(numpy.array([[[[75.0, 76.0], [78.0, 79.0], [81.0, 82.0], [84.0, 85.0]]], [[[120.0, 121.0], [123.0, 124.0], 
[126.0, 127.0], [129.0, 130.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,2:]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,:4,:2]
      ref=Data(numpy.array([[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], 
[111.0, 112.0], [114.0, 115.0]], [[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0]]], [[[135.0, 136.0], [138.0, 
139.0], [141.0, 142.0], [144.0, 145.0]], [[150.0, 151.0], [153.0, 154.0], [156.0, 157.0], [159.0, 160.0]], [[165.0, 166.0], 
[168.0, 169.0], [171.0, 172.0], [174.0, 175.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPRL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,1:4,2:]
      ref=Data(numpy.array([[110.0], [113.0], [116.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRUC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,:4,:]
      ref=Data(numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], 
[[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLLP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,2:,1]
      ref=Data(numpy.array([[[37.0, 40.0, 43.0]], [[82.0, 85.0, 88.0]], [[127.0, 130.0, 133.0]], [[172.0, 175.0, 
178.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLPC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,2,:]
      ref=Data(numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]], [[171.0, 172.0, 
173.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,1:4]
      ref=Data(numpy.array([[[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,:,:]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], 
[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LUCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,:,:]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], 
[[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 
151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,2,1]
      ref=Data(numpy.array([112.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRRP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,1:4,1]
      ref=Data(numpy.array([[109.0, 112.0, 115.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCPC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,2,:]
      ref=Data(numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0], [126.0, 127.0, 128.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRPL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,2,2:]
      ref=Data(numpy.array([[[113.0]], [[158.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_URCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1:2,:,:2]
      ref=Data(numpy.array([[[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0], [27.0, 28.0]]], [[[60.0, 61.0], [63.0, 
64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 
118.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,2:]
      ref=Data(numpy.array([[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRRL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,1:4,2:]
      ref=Data(numpy.array([[[[110.0], [113.0], [116.0]]], [[[155.0], [158.0], [161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,:4]
      ref=Data(numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLPL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,2,2:]
      ref=Data(numpy.array([[128.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:,:]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,:4,:2]
      ref=Data(numpy.array([[[[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0]]], [[[105.0, 106.0], [108.0, 109.0], 
[111.0, 112.0], [114.0, 115.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRPR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,2,1:2]
      ref=Data(numpy.array([[[67.0]], [[112.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PUUL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,:4,2:]
      ref=Data(numpy.array([[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], [116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 
145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRCR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,:,1:2]
      ref=Data(numpy.array([[[[106.0], [109.0], [112.0], [115.0], [118.0]]], [[[151.0], [154.0], [157.0], [160.0], 
[163.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPUP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,:4,1]
      ref=Data(numpy.array([[16.0, 19.0, 22.0, 25.0], [61.0, 64.0, 67.0, 70.0], [106.0, 109.0, 112.0, 115.0], [151.0, 154.0, 
157.0, 160.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,2]
      ref=Data(numpy.array([[66.0, 67.0, 68.0], [111.0, 112.0, 113.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,2:,:]
      ref=Data(numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,2]
      ref=Data(numpy.array([[126.0, 127.0, 128.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,2]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RULL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,2:,2:]
      ref=Data(numpy.array([[[[53.0], [56.0], [59.0]], [[68.0], [71.0], [74.0]]], [[[98.0], [101.0], [104.0]], [[113.0], 
[116.0], [119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,2,1]
      ref=Data(numpy.array([[52.0, 67.0, 82.0], [97.0, 112.0, 127.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLPC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,2,:]
      ref=Data(numpy.array([[126.0, 127.0, 128.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,2]
      ref=Data(numpy.array([[[111.0, 112.0, 113.0]], [[156.0, 157.0, 158.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,:4]
      ref=Data(numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,:]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRRR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,1:4,1:2]
      ref=Data(numpy.array([[[[109.0], [112.0], [115.0]]], [[[154.0], [157.0], [160.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LULU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,2:,:2]
      ref=Data(numpy.array([[[[96.0, 97.0], [99.0, 100.0], [102.0, 103.0]], [[111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]], 
[[[141.0, 142.0], [144.0, 145.0], [147.0, 148.0]], [[156.0, 157.0], [159.0, 160.0], [162.0, 163.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,2]
      ref=Data(numpy.array([[[126.0, 127.0, 128.0]], [[171.0, 172.0, 173.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCCR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,:,1:2]
      ref=Data(numpy.array([[[[46.0], [49.0], [52.0], [55.0], [58.0]], [[61.0], [64.0], [67.0], [70.0], [73.0]], [[76.0], 
[79.0], [82.0], [85.0], [88.0]]], [[[91.0], [94.0], [97.0], [100.0], [103.0]], [[106.0], [109.0], [112.0], [115.0], [118.0]], 
[[121.0], [124.0], [127.0], [130.0], [133.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,1:4]
      ref=Data(numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPPU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,2,:2]
      ref=Data(numpy.array([[21.0, 22.0], [66.0, 67.0], [111.0, 112.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,:,:]
      ref=Data(numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,:4]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1:2,:4]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:,2]
      ref=Data(numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1,:]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_ULLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,2:,2:]
      ref=Data(numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,:4]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,2:]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,1:4]
      ref=Data(numpy.array([[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:,2:]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RURR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,1:4,1:2]
      ref=Data(numpy.array([[[[49.0], [52.0], [55.0]], [[64.0], [67.0], [70.0]]], [[[94.0], [97.0], [100.0]], [[109.0], 
[112.0], [115.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,:]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 
92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 
109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,2]
      ref=Data(numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0], [126.0, 127.0, 128.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,1:4]
      ref=Data(numpy.array([[[[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCCC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,:,:]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 
95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCUC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,:4,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,2:]
      ref=Data(numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUUR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,:4,1:2]
      ref=Data(numpy.array([[[[1.0], [4.0], [7.0], [10.0]], [[16.0], [19.0], [22.0], [25.0]]], [[[46.0], [49.0], [52.0], 
[55.0]], [[61.0], [64.0], [67.0], [70.0]]], [[[91.0], [94.0], [97.0], [100.0]], [[106.0], [109.0], [112.0], 
[115.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,:]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,1:4]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,:]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,2:,:]
      ref=Data(numpy.array([[[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPLP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,2:,1]
      ref=Data(numpy.array([112.0, 115.0, 118.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_URLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1:2,2:]
      ref=Data(numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,:4,:2]
      ref=Data(numpy.array([[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 
67.0], [69.0, 70.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,2]
      ref=Data(numpy.array([[[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1:2,:]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]]], [[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,:4]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], 
[144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 
166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,1:4]
      ref=Data(numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,:]
      ref=Data(numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,1:4]
      ref=Data(numpy.array([[[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,1:4]
      ref=Data(numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,2]
      ref=Data(numpy.array([111.0, 112.0, 113.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PUPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,2,1]
      ref=Data(numpy.array([97.0, 112.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,1:4]
      ref=Data(numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,:]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 
95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,:]
      ref=Data(numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 
178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,1:4]
      ref=Data(numpy.array([[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:2,:]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], 
[[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 
151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,2:,2:]
      ref=Data(numpy.array([[[[83.0], [86.0], [89.0]]], [[[128.0], [131.0], [134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1,1:4]
      ref=Data(numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0], [171.0, 172.0, 173.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRRR(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,1:4,1:2]
      ref=Data(numpy.array([[[[64.0], [67.0], [70.0]]], [[[109.0], [112.0], [115.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,:4]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,:]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 
44.0]]], [[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,:]
      ref=Data(numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,:]
      ref=Data(numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,:4]
      ref=Data(numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCLC(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,2:,:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,2:]
      ref=Data(numpy.array([[[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,:]
      ref=Data(numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 
74.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:2,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 
98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 158.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_URUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1:2,:4]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1:2,2:]
      ref=Data(numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,:]
      ref=Data(numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,:4]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_ULUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,2:,:4]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:,1:4]
      ref=Data(numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:2,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,2:]
      ref=Data(numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,2:]
      ref=Data(numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,1:4]
      ref=Data(numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:2,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], 
[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], 
[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 
146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], 
[162.0, 163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,:4]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,1:4]
      ref=Data(numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,2]
      ref=Data(numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]], [[171.0, 172.0, 
173.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,2]
      ref=Data(numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,:4]
      ref=Data(numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PPPP(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1,2,1]
      ref=Data(112.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_URPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1:2,2]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1,:4]
      ref=Data(numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,:4]
      ref=Data(numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,2:]
      ref=Data(numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,2:]
      ref=Data(numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,2:]
      ref=Data(numpy.array([[[[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 
130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUCU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,:,:2]
      ref=Data(numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0], [12.0, 13.0]], [[15.0, 16.0], [18.0, 19.0], 
[21.0, 22.0], [24.0, 25.0], [27.0, 28.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0], [57.0, 58.0]], [[60.0, 
61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0], 
[102.0, 103.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,:]
      ref=Data(numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,:4]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 
176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1,2:]
      ref=Data(numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_URRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1:2,1:4]
      ref=Data(numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUUU(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,:4,:2]
      ref=Data(numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], 
[24.0, 25.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 
70.0]]], [[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 
115.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,2:]
      ref=Data(numpy.array([[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,2]
      ref=Data(numpy.array([[[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:2,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,2:,1:4]
      ref=Data(numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RPRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1,1:4]
      ref=Data(numpy.array([[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PCUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:,:4]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_ULCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,2:,:]
      ref=Data(numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 
44.0]]], [[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1,2]
      ref=Data(numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0], [156.0, 157.0, 
158.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,:4]
      ref=Data(numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], 
[[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,2:,:4]
      ref=Data(numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,:]
      ref=Data(numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 
74.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PURN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,1:4]
      ref=Data(numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:2,2]
      ref=Data(numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLLL(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,2:,2:]
      ref=Data(numpy.array([[[[128.0], [131.0], [134.0]]], [[[173.0], [176.0], [179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,2]
      ref=Data(numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 
98.0], [111.0, 112.0, 113.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_URCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1:2,:]
      ref=Data(numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]]], [[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_ULRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,2:,1:4]
      ref=Data(numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1:2,2:]
      ref=Data(numpy.array([[[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LCPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,:,2]
      ref=Data(numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 
157.0, 158.0], [171.0, 172.0, 173.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,1:4]
      ref=Data(numpy.array([[[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PRPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,1:2,2]
      ref=Data(numpy.array([[111.0, 112.0, 113.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UCRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:,1:4]
      ref=Data(numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CULN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:2,2:]
      ref=Data(numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RRLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,1:2,2:]
      ref=Data(numpy.array([[[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CCCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,:,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,1,2]
      ref=Data(numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,:4]
      ref=Data(numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_CRRN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:,1:2,1:4]
      ref=Data(numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PUUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,:2,:4]
      ref=Data(numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LLUN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,2:,:4]
      ref=Data(numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], 
[[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_PLLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2,2:,2:]
      ref=Data(numpy.array([[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_ULPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,2:,2]
      ref=Data(numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_RCLN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[1:3,:,2:]
      ref=Data(numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], 
[129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_LPPN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[2:,1,2]
      ref=Data(numpy.array([[111.0, 112.0, 113.0], [156.0, 157.0, 158.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_constData_rank4_UUCN(self):
      arg=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]]),self.functionspace)
      res=arg[:3,:2,:]
      ref=Data(numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], 
[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], 
[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank1_C(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([0.0, 1.0, 2.0])+(1.-msk_arg)*numpy.array([-0.0, -1.0, -2.0])
      res=arg[:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0, 1.0, 2.0])+(1.-msk_ref)*numpy.array([-0.0, -1.0, -2.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank1_P(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([0.0, 1.0, 2.0])+(1.-msk_arg)*numpy.array([-0.0, -1.0, -2.0])
      res=arg[1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(1.0)+(1.-msk_ref)*(-1.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank1_R(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([0.0, 1.0, 2.0])+(1.-msk_arg)*numpy.array([-0.0, -1.0, -2.0])
      res=arg[1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.0])+(1.-msk_ref)*numpy.array([-1.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank1_L(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([0.0, 1.0, 2.0])+(1.-msk_arg)*numpy.array([-0.0, -1.0, -2.0])
      res=arg[2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.0])+(1.-msk_ref)*numpy.array([-2.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank1_U(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([0.0, 1.0, 2.0])+(1.-msk_arg)*numpy.array([-0.0, -1.0, -2.0])
      res=arg[:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0, 1.0])+(1.-msk_ref)*numpy.array([-0.0, -1.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_PU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.0, 5.0, 6.0])+(1.-msk_ref)*numpy.array([-4.0, -5.0, -6.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_PL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([6.0, 7.0])+(1.-msk_ref)*numpy.array([-6.0, -7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_RR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1:2,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 6.0]])+(1.-msk_ref)*numpy.array([[-5.0, -6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_RL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 7.0]])+(1.-msk_ref)*numpy.array([[-6.0, -7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_CU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.0, 1.0, 2.0], [4.0, 5.0, 6.0], [8.0, 9.0, 10.0]])+(1.-msk_ref)*numpy.array([[-0.0, -1.0, 
-2.0], [-4.0, -5.0, -6.0], [-8.0, -9.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_LU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[2:,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 9.0, 10.0]])+(1.-msk_ref)*numpy.array([[-8.0, -9.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_LC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 9.0, 10.0, 11.0]])+(1.-msk_ref)*numpy.array([[-8.0, -9.0, -10.0, -11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_UR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:2,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 2.0], [5.0, 6.0]])+(1.-msk_ref)*numpy.array([[-1.0, -2.0], [-5.0, -6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_UN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0]])+(1.-msk_ref)*numpy.array([[-0.0, -1.0, -2.0, -3.0], 
[-4.0, -5.0, -6.0, -7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_PR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([5.0, 6.0])+(1.-msk_ref)*numpy.array([-5.0, -6.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_RN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 5.0, 6.0, 7.0]])+(1.-msk_ref)*numpy.array([[-4.0, -5.0, -6.0, -7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_LL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[10.0, 11.0]])+(1.-msk_ref)*numpy.array([[-10.0, -11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_CN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_ref)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_PC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.0, 5.0, 6.0, 7.0])+(1.-msk_ref)*numpy.array([-4.0, -5.0, -6.0, -7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_CR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 2.0], [5.0, 6.0], [9.0, 10.0]])+(1.-msk_ref)*numpy.array([[-1.0, -2.0], [-5.0, -6.0], 
[-9.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_CC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_ref)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_PN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.0, 5.0, 6.0, 7.0])+(1.-msk_ref)*numpy.array([-4.0, -5.0, -6.0, -7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_UU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[:2,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.0, 1.0, 2.0], [4.0, 5.0, 6.0]])+(1.-msk_ref)*numpy.array([[-0.0, -1.0, -2.0], [-4.0, -5.0, 
-6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_PP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(6.0)+(1.-msk_ref)*(-6.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank2_LN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 
11.0]])+(1.-msk_arg)*numpy.array([[-0.0, -1.0, -2.0, -3.0], [-4.0, -5.0, -6.0, -7.0], [-8.0, -9.0, -10.0, -11.0]])
      res=arg[2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 9.0, 10.0, 11.0]])+(1.-msk_ref)*numpy.array([[-8.0, -9.0, -10.0, -11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 4.0, 7.0], [13.0, 16.0, 19.0]])+(1.-msk_ref)*numpy.array([[-1.0, -4.0, -7.0], [-13.0, 
-16.0, -19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[32.0], [35.0]]])+(1.-msk_ref)*numpy.array([[[-32.0], [-35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,1:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[28.0, 31.0]])+(1.-msk_ref)*numpy.array([[-28.0, -31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 
32.0]]])+(1.-msk_ref)*numpy.array([[[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 4.0, 7.0, 10.0], [13.0, 16.0, 19.0, 22.0], [25.0, 28.0, 31.0, 
34.0]])+(1.-msk_ref)*numpy.array([[-1.0, -4.0, -7.0, -10.0], [-13.0, -16.0, -19.0, -22.0], [-25.0, -28.0, -31.0, -34.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], 
[21.0, 22.0]], [[24.0, 25.0], [27.0, 28.0], [30.0, 31.0], [33.0, 34.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0], [-3.0, 
-4.0], [-6.0, -7.0], [-9.0, -10.0]], [[-12.0, -13.0], [-15.0, -16.0], [-18.0, -19.0], [-21.0, -22.0]], [[-24.0, -25.0], [-27.0, 
-28.0], [-30.0, -31.0], [-33.0, -34.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0], [20.0]])+(1.-msk_ref)*numpy.array([[-8.0], [-20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0], [30.0, 31.0, 32.0]])+(1.-msk_ref)*numpy.array([[-6.0, -7.0, 
-8.0], [-18.0, -19.0, -20.0], [-30.0, -31.0, -32.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-12.0, -13.0, 
-14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,2,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[18.0, 19.0]])+(1.-msk_ref)*numpy.array([[-18.0, -19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_URN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 
20.0]]])+(1.-msk_ref)*numpy.array([[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,1:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 7.0], [16.0, 19.0], [28.0, 31.0]])+(1.-msk_ref)*numpy.array([[-4.0, -7.0], [-16.0, -19.0], 
[-28.0, -31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0], [5.0], [8.0], [11.0]], [[14.0], [17.0], [20.0], [23.0]], [[26.0], [29.0], [32.0], 
[35.0]]])+(1.-msk_ref)*numpy.array([[[-2.0], [-5.0], [-8.0], [-11.0]], [[-14.0], [-17.0], [-20.0], [-23.0]], [[-26.0], [-29.0], 
[-32.0], [-35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_ULC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:3,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0], [5.0], [8.0]], [[14.0], [17.0], [20.0]], [[26.0], [29.0], 
[32.0]]])+(1.-msk_ref)*numpy.array([[[-2.0], [-5.0], [-8.0]], [[-14.0], [-17.0], [-20.0]], [[-26.0], [-29.0], [-32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], 
[-18.0, -19.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.0, 19.0])+(1.-msk_ref)*numpy.array([-7.0, -19.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,1:3,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[29.0], [32.0]]])+(1.-msk_ref)*numpy.array([[[-29.0], [-32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 4.0, 7.0], [13.0, 16.0, 19.0], [25.0, 28.0, 31.0]])+(1.-msk_ref)*numpy.array([[-1.0, -4.0, 
-7.0], [-13.0, -16.0, -19.0], [-25.0, -28.0, -31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:3,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]], [[24.0, 25.0], 
[27.0, 28.0], [30.0, 31.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0], [-3.0, -4.0], [-6.0, -7.0]], [[-12.0, -13.0], [-15.0, 
-16.0], [-18.0, -19.0]], [[-24.0, -25.0], [-27.0, -28.0], [-30.0, -31.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([20.0])+(1.-msk_ref)*numpy.array([-20.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, -19.0, -20.0], 
[-21.0, -22.0, -23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 
20.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,:3,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[24.0, 25.0], [27.0, 28.0], [30.0, 31.0]]])+(1.-msk_ref)*numpy.array([[[-24.0, -25.0], [-27.0, 
-28.0], [-30.0, -31.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]], [[27.0, 28.0, 
29.0], [30.0, 31.0, 32.0]]])+(1.-msk_ref)*numpy.array([[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-15.0, -16.0, -17.0], 
[-18.0, -19.0, -20.0]], [[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,1:3,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0], [8.0]], [[17.0], [20.0]], [[29.0], [32.0]]])+(1.-msk_ref)*numpy.array([[[-5.0], [-8.0]], 
[[-17.0], [-20.0]], [[-29.0], [-32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,:3,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, -13.0], [-15.0, 
-16.0], [-18.0, -19.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,2:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[19.0], [22.0]]])+(1.-msk_ref)*numpy.array([[[-19.0], [-22.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,2:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[19.0], [22.0]])+(1.-msk_ref)*numpy.array([[-19.0], [-22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], [21.0, 22.0]])+(1.-msk_ref)*numpy.array([[-12.0, 
-13.0], [-15.0, -16.0], [-18.0, -19.0], [-21.0, -22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_ULR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0], [10.0]], [[19.0], [22.0]]])+(1.-msk_ref)*numpy.array([[[-7.0], [-10.0]], [[-19.0], 
[-22.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0], [20.0], [32.0]])+(1.-msk_ref)*numpy.array([[-8.0], [-20.0], [-32.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_URC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,1:3,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 
20.0]]])+(1.-msk_ref)*numpy.array([[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, -31.0, -32.0], 
[-33.0, -34.0, -35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,1:3,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[16.0], [19.0]])+(1.-msk_ref)*numpy.array([[-16.0], [-19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[25.0, 28.0, 31.0]])+(1.-msk_ref)*numpy.array([[-25.0, -28.0, -31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_ULL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0], [11.0]], [[20.0], [23.0]]])+(1.-msk_ref)*numpy.array([[[-8.0], [-11.0]], [[-20.0], 
[-23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]])+(1.-msk_ref)*numpy.array([[-18.0, -19.0, -20.0], 
[-21.0, -22.0, -23.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[25.0, 28.0, 31.0, 34.0]])+(1.-msk_ref)*numpy.array([[-25.0, -28.0, -31.0, -34.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:3,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0], [4.0], [7.0]], [[13.0], [16.0], [19.0]], [[25.0], [28.0], 
[31.0]]])+(1.-msk_ref)*numpy.array([[[-1.0], [-4.0], [-7.0]], [[-13.0], [-16.0], [-19.0]], [[-25.0], [-28.0], [-31.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]])+(1.-msk_ref)*numpy.array([[[-27.0, -28.0, -29.0], 
[-30.0, -31.0, -32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0]])+(1.-msk_ref)*numpy.array([[-6.0, -7.0, -8.0], [-18.0, 
-19.0, -20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.0, 19.0, 31.0])+(1.-msk_ref)*numpy.array([-7.0, -19.0, -31.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]])+(1.-msk_ref)*numpy.array([[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,1:3,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], 
[-18.0, -19.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[30.0, 31.0, 
32.0], [33.0, 34.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], 
[-21.0, -22.0, -23.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,:3,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[14.0], [17.0], [20.0]]])+(1.-msk_ref)*numpy.array([[[-14.0], [-17.0], [-20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,1:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]])+(1.-msk_ref)*numpy.array([[-15.0, -16.0, -17.0], 
[-18.0, -19.0, -20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,:3,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 
19.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0], [-3.0, -4.0], [-6.0, -7.0]], [[-12.0, -13.0], [-15.0, -16.0], [-18.0, 
-19.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,1:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([16.0, 19.0])+(1.-msk_ref)*numpy.array([-16.0, -19.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[18.0, 19.0, 20.0]])+(1.-msk_ref)*numpy.array([[-18.0, -19.0, -20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_URP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,1:3,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 7.0], [16.0, 19.0]])+(1.-msk_ref)*numpy.array([[-4.0, -7.0], [-16.0, -19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_RCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1:2,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_UCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, 
-5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, 
-22.0, -23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_ULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 
23.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]])+(1.-msk_ref)*numpy.array([[-12.0, 
-13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:3]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0, 
-2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0]], [[-24.0, 
-25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_URR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:2,1:3,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0], [7.0]], [[16.0], [19.0]]])+(1.-msk_ref)*numpy.array([[[-4.0], [-7.0]], [[-16.0], 
[-19.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[30.0, 31.0, 32.0]])+(1.-msk_ref)*numpy.array([[-30.0, -31.0, -32.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[13.0], [16.0], [19.0], [22.0]])+(1.-msk_ref)*numpy.array([[-13.0], [-16.0], [-19.0], [-22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([18.0, 19.0, 20.0])+(1.-msk_ref)*numpy.array([-18.0, -19.0, -20.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])+(1.-msk_ref)*numpy.array([[[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, 
-35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, -31.0, -32.0], 
[-33.0, -34.0, -35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_PLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[1,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[20.0], [23.0]])+(1.-msk_ref)*numpy.array([[-20.0], [-23.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_LCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[2:,:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[25.0], [28.0], [31.0], [34.0]]])+(1.-msk_ref)*numpy.array([[[-25.0], [-28.0], [-31.0], 
[-34.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank3_CCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_arg)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      res=arg[:,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], 
[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0]], [[-12.0, -13.0, -14.0], [-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0]], [[-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0], [-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RULP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,2:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[52.0, 55.0, 58.0], [67.0, 70.0, 73.0]], [[97.0, 100.0, 103.0], [112.0, 115.0, 
118.0]]])+(1.-msk_ref)*numpy.array([[[-52.0, -55.0, -58.0], [-67.0, -70.0, -73.0]], [[-97.0, -100.0, -103.0], [-112.0, -115.0, 
-118.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,:4,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], 
[[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]])+(1.-msk_ref)*numpy.array([[[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]])+(1.-msk_ref)*numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[79.0], [82.0], [85.0]]], [[[124.0], [127.0], [130.0]]]])+(1.-msk_ref)*numpy.array([[[[-79.0], 
[-82.0], [-85.0]]], [[[-124.0], [-127.0], [-130.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], 
[-9.0, -10.0, -11.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-45.0, 
-46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], 
[-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[122.0], [125.0], [128.0], [131.0], [134.0]]], [[[167.0], [170.0], [173.0], [176.0], 
[179.0]]]])+(1.-msk_ref)*numpy.array([[[[-122.0], [-125.0], [-128.0], [-131.0], [-134.0]]], [[[-167.0], [-170.0], [-173.0], 
[-176.0], [-179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PUCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[91.0, 94.0, 97.0, 100.0, 103.0], [106.0, 109.0, 112.0, 115.0, 
118.0]])+(1.-msk_ref)*numpy.array([[-91.0, -94.0, -97.0, -100.0, -103.0], [-106.0, -109.0, -112.0, -115.0, -118.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[16.0, 19.0, 22.0, 25.0, 28.0], [61.0, 64.0, 67.0, 70.0, 73.0], [106.0, 109.0, 112.0, 115.0, 
118.0], [151.0, 154.0, 157.0, 160.0, 163.0]])+(1.-msk_ref)*numpy.array([[-16.0, -19.0, -22.0, -25.0, -28.0], [-61.0, -64.0, 
-67.0, -70.0, -73.0], [-106.0, -109.0, -112.0, -115.0, -118.0], [-151.0, -154.0, -157.0, -160.0, -163.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], 
[24.0, 25.0]], [[30.0, 31.0], [33.0, 34.0], [36.0, 37.0], [39.0, 40.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 
55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0]], [[75.0, 76.0], [78.0, 79.0], [81.0, 82.0], [84.0, 85.0]]], 
[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]], 
[[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0], [-3.0, -4.0], 
[-6.0, -7.0], [-9.0, -10.0]], [[-15.0, -16.0], [-18.0, -19.0], [-21.0, -22.0], [-24.0, -25.0]], [[-30.0, -31.0], [-33.0, 
-34.0], [-36.0, -37.0], [-39.0, -40.0]]], [[[-45.0, -46.0], [-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0]], [[-60.0, -61.0], 
[-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]], [[-75.0, -76.0], [-78.0, -79.0], [-81.0, -82.0], [-84.0, -85.0]]], [[[-90.0, 
-91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0]], [[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, 
-115.0]], [[-120.0, -121.0], [-123.0, -124.0], [-126.0, -127.0], [-129.0, -130.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 
67.0], [69.0, 70.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, 
-16.0], [-18.0, -19.0], [-21.0, -22.0], [-24.0, -25.0]], [[-60.0, -61.0], [-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]], 
[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[109.0], [112.0], [115.0]]])+(1.-msk_ref)*numpy.array([[[-109.0], [-112.0], [-115.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[23.0], [26.0], [29.0]], [[68.0], [71.0], [74.0]], [[113.0], [116.0], 
[119.0]]])+(1.-msk_ref)*numpy.array([[[-23.0], [-26.0], [-29.0]], [[-68.0], [-71.0], [-74.0]], [[-113.0], [-116.0], [-119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UUPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,2,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0], [22.0]], [[52.0], [67.0]], [[97.0], [112.0]]])+(1.-msk_ref)*numpy.array([[[-7.0], 
[-22.0]], [[-52.0], [-67.0]], [[-97.0], [-112.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 
26.0], [27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 
56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], 
[[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]], [[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, 
-7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, 
-29.0]], [[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], [[[-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-81.0, -82.0, -83.0], 
[-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], 
[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 
103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]])+(1.-msk_ref)*numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[124.0], [127.0], [130.0]]], [[[169.0], [172.0], 
[175.0]]]])+(1.-msk_ref)*numpy.array([[[[-124.0], [-127.0], [-130.0]]], [[[-169.0], [-172.0], [-175.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[94.0], [97.0], [100.0]], [[109.0], [112.0], [115.0]]])+(1.-msk_ref)*numpy.array([[[-94.0], 
[-97.0], [-100.0]], [[-109.0], [-112.0], [-115.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[113.0]], [[158.0]]])+(1.-msk_ref)*numpy.array([[[-113.0]], [[-158.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[61.0, 64.0, 67.0, 70.0, 73.0]], [[106.0, 109.0, 112.0, 115.0, 
118.0]]])+(1.-msk_ref)*numpy.array([[[-61.0, -64.0, -67.0, -70.0, -73.0]], [[-106.0, -109.0, -112.0, -115.0, -118.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([97.0, 112.0, 127.0])+(1.-msk_ref)*numpy.array([-97.0, -112.0, -127.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[105.0, 106.0], 
[108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]]])+(1.-msk_ref)*numpy.array([[[[-60.0, -61.0], [-63.0, -64.0], 
[-66.0, -67.0], [-69.0, -70.0], [-72.0, -73.0]]], [[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0], 
[-117.0, -118.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,1:4,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[65.0], [68.0], [71.0]], [[110.0], [113.0], [116.0]]])+(1.-msk_ref)*numpy.array([[[-65.0], 
[-68.0], [-71.0]], [[-110.0], [-113.0], [-116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UULL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0], [11.0], [14.0]], [[23.0], [26.0], [29.0]]], [[[53.0], [56.0], [59.0]], [[68.0], [71.0], 
[74.0]]], [[[98.0], [101.0], [104.0]], [[113.0], [116.0], [119.0]]]])+(1.-msk_ref)*numpy.array([[[[-8.0], [-11.0], [-14.0]], 
[[-23.0], [-26.0], [-29.0]]], [[[-53.0], [-56.0], [-59.0]], [[-68.0], [-71.0], [-74.0]]], [[[-98.0], [-101.0], [-104.0]], 
[[-113.0], [-116.0], [-119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PUCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[91.0], [94.0], [97.0], [100.0], [103.0]], [[106.0], [109.0], [112.0], [115.0], 
[118.0]]])+(1.-msk_ref)*numpy.array([[[-91.0], [-94.0], [-97.0], [-100.0], [-103.0]], [[-106.0], [-109.0], [-112.0], [-115.0], 
[-118.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,2,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[67.0], [112.0]])+(1.-msk_ref)*numpy.array([[-67.0], [-112.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPLP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,2:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([112.0, 115.0, 118.0])+(1.-msk_ref)*numpy.array([-112.0, -115.0, -118.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LUCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 
103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]])+(1.-msk_ref)*numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], 
[-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], 
[24.0, 25.0]], [[30.0, 31.0], [33.0, 34.0], [36.0, 37.0], [39.0, 40.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 
55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0]], [[75.0, 76.0], [78.0, 79.0], [81.0, 82.0], [84.0, 85.0]]], 
[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]], 
[[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0]]], [[[135.0, 136.0], [138.0, 139.0], [141.0, 142.0], [144.0, 
145.0]], [[150.0, 151.0], [153.0, 154.0], [156.0, 157.0], [159.0, 160.0]], [[165.0, 166.0], [168.0, 169.0], [171.0, 172.0], 
[174.0, 175.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0], [-3.0, -4.0], [-6.0, -7.0], [-9.0, -10.0]], [[-15.0, -16.0], 
[-18.0, -19.0], [-21.0, -22.0], [-24.0, -25.0]], [[-30.0, -31.0], [-33.0, -34.0], [-36.0, -37.0], [-39.0, -40.0]]], [[[-45.0, 
-46.0], [-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0]], [[-60.0, -61.0], [-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]], 
[[-75.0, -76.0], [-78.0, -79.0], [-81.0, -82.0], [-84.0, -85.0]]], [[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, 
-100.0]], [[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]], [[-120.0, -121.0], [-123.0, -124.0], 
[-126.0, -127.0], [-129.0, -130.0]]], [[[-135.0, -136.0], [-138.0, -139.0], [-141.0, -142.0], [-144.0, -145.0]], [[-150.0, 
-151.0], [-153.0, -154.0], [-156.0, -157.0], [-159.0, -160.0]], [[-165.0, -166.0], [-168.0, -169.0], [-171.0, -172.0], [-174.0, 
-175.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PUPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[98.0], [113.0]])+(1.-msk_ref)*numpy.array([[-98.0], [-113.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[111.0, 112.0, 113.0]], [[156.0, 157.0, 158.0]]])+(1.-msk_ref)*numpy.array([[[-111.0, -112.0, 
-113.0]], [[-156.0, -157.0, -158.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 
98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])+(1.-msk_ref)*numpy.array([[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], 
[-54.0, -55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, 
-82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_ULRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,2:,1:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[33.0, 34.0], [36.0, 37.0], [39.0, 40.0]]], [[[78.0, 79.0], [81.0, 82.0], [84.0, 85.0]]], 
[[[123.0, 124.0], [126.0, 127.0], [129.0, 130.0]]]])+(1.-msk_ref)*numpy.array([[[[-33.0, -34.0], [-36.0, -37.0], [-39.0, 
-40.0]]], [[[-78.0, -79.0], [-81.0, -82.0], [-84.0, -85.0]]], [[[-123.0, -124.0], [-126.0, -127.0], [-129.0, -130.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,2:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[126.0, 127.0, 128.0]])+(1.-msk_ref)*numpy.array([[-126.0, -127.0, -128.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_URPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 
113.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, -22.0, -23.0]], [[-66.0, -67.0, -68.0]], [[-111.0, -112.0, -113.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[82.0], [127.0]])+(1.-msk_ref)*numpy.array([[-82.0], [-127.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,1:4,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, 
-7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-33.0, -34.0, 
-35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], 
[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, 
-85.0, -86.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], 
[[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0]], [[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], 
[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0], [-162.0, -163.0, -164.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0], [7.0], [10.0]], [[19.0], [22.0], [25.0]], [[34.0], [37.0], [40.0]]], [[[49.0], [52.0], 
[55.0]], [[64.0], [67.0], [70.0]], [[79.0], [82.0], [85.0]]], [[[94.0], [97.0], [100.0]], [[109.0], [112.0], [115.0]], 
[[124.0], [127.0], [130.0]]], [[[139.0], [142.0], [145.0]], [[154.0], [157.0], [160.0]], [[169.0], [172.0], 
[175.0]]]])+(1.-msk_ref)*numpy.array([[[[-4.0], [-7.0], [-10.0]], [[-19.0], [-22.0], [-25.0]], [[-34.0], [-37.0], [-40.0]]], 
[[[-49.0], [-52.0], [-55.0]], [[-64.0], [-67.0], [-70.0]], [[-79.0], [-82.0], [-85.0]]], [[[-94.0], [-97.0], [-100.0]], 
[[-109.0], [-112.0], [-115.0]], [[-124.0], [-127.0], [-130.0]]], [[[-139.0], [-142.0], [-145.0]], [[-154.0], [-157.0], 
[-160.0]], [[-169.0], [-172.0], [-175.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 
130.0, 131.0], [132.0, 133.0, 134.0]]]])+(1.-msk_ref)*numpy.array([[[[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, 
-88.0, -89.0]]], [[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UULP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,2:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 10.0, 13.0], [22.0, 25.0, 28.0]], [[52.0, 55.0, 58.0], [67.0, 70.0, 73.0]], [[97.0, 
100.0, 103.0], [112.0, 115.0, 118.0]]])+(1.-msk_ref)*numpy.array([[[-7.0, -10.0, -13.0], [-22.0, -25.0, -28.0]], [[-52.0, 
-55.0, -58.0], [-67.0, -70.0, -73.0]], [[-97.0, -100.0, -103.0], [-112.0, -115.0, -118.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0], [102.0, 103.0]], [[105.0, 106.0], 
[108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]], [[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 
130.0], [132.0, 133.0]]])+(1.-msk_ref)*numpy.array([[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0], [-102.0, 
-103.0]], [[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0], [-117.0, -118.0]], [[-120.0, -121.0], 
[-123.0, -124.0], [-126.0, -127.0], [-129.0, -130.0], [-132.0, -133.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[107.0], [110.0], [113.0], [116.0], [119.0]]])+(1.-msk_ref)*numpy.array([[[-107.0], [-110.0], 
[-113.0], [-116.0], [-119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CURC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:2,1:4,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], 
[-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-48.0, -49.0, -50.0], [-51.0, 
-52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, 
-95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]], [[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_URUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 
61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], 
[111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], 
[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, 
-70.0, -71.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PURP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,1:4,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[94.0, 97.0, 100.0], [109.0, 112.0, 115.0]])+(1.-msk_ref)*numpy.array([[-94.0, -97.0, -100.0], 
[-109.0, -112.0, -115.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[46.0, 49.0, 52.0, 55.0, 58.0], [61.0, 64.0, 67.0, 70.0, 73.0], [76.0, 79.0, 82.0, 85.0, 
88.0]], [[91.0, 94.0, 97.0, 100.0, 103.0], [106.0, 109.0, 112.0, 115.0, 118.0], [121.0, 124.0, 127.0, 130.0, 
133.0]]])+(1.-msk_ref)*numpy.array([[[-46.0, -49.0, -52.0, -55.0, -58.0], [-61.0, -64.0, -67.0, -70.0, -73.0], [-76.0, -79.0, 
-82.0, -85.0, -88.0]], [[-91.0, -94.0, -97.0, -100.0, -103.0], [-106.0, -109.0, -112.0, -115.0, -118.0], [-121.0, -124.0, 
-127.0, -130.0, -133.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], 
[[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]], [[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-96.0, 
-97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], 
[-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, 
-143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 
67.0], [69.0, 70.0]]], [[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 
112.0], [114.0, 115.0]]]])+(1.-msk_ref)*numpy.array([[[[-45.0, -46.0], [-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0]], 
[[-60.0, -61.0], [-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]]], [[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, 
-100.0]], [[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,1:4,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[19.0, 22.0, 25.0], [64.0, 67.0, 70.0], [109.0, 112.0, 115.0], [154.0, 157.0, 
160.0]])+(1.-msk_ref)*numpy.array([[-19.0, -22.0, -25.0], [-64.0, -67.0, -70.0], [-109.0, -112.0, -115.0], [-154.0, -157.0, 
-160.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,:4,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 7.0, 10.0], [16.0, 19.0, 22.0, 25.0], [31.0, 34.0, 37.0, 40.0]], [[46.0, 49.0, 52.0, 
55.0], [61.0, 64.0, 67.0, 70.0], [76.0, 79.0, 82.0, 85.0]], [[91.0, 94.0, 97.0, 100.0], [106.0, 109.0, 112.0, 115.0], [121.0, 
124.0, 127.0, 130.0]]])+(1.-msk_ref)*numpy.array([[[-1.0, -4.0, -7.0, -10.0], [-16.0, -19.0, -22.0, -25.0], [-31.0, -34.0, 
-37.0, -40.0]], [[-46.0, -49.0, -52.0, -55.0], [-61.0, -64.0, -67.0, -70.0], [-76.0, -79.0, -82.0, -85.0]], [[-91.0, -94.0, 
-97.0, -100.0], [-106.0, -109.0, -112.0, -115.0], [-121.0, -124.0, -127.0, -130.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,2:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]], [[[156.0, 157.0], [159.0, 160.0], [162.0, 
163.0]]]])+(1.-msk_ref)*numpy.array([[[[-111.0, -112.0], [-114.0, -115.0], [-117.0, -118.0]]], [[[-156.0, -157.0], [-159.0, 
-160.0], [-162.0, -163.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[113.0], [116.0], [119.0]])+(1.-msk_ref)*numpy.array([[-113.0], [-116.0], [-119.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]])+(1.-msk_ref)*numpy.array([[[-66.0, -67.0, 
-68.0]], [[-111.0, -112.0, -113.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,2:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[126.0, 127.0], [129.0, 130.0], [132.0, 133.0]]], [[[171.0, 172.0], [174.0, 175.0], [177.0, 
178.0]]]])+(1.-msk_ref)*numpy.array([[[[-126.0, -127.0], [-129.0, -130.0], [-132.0, -133.0]]], [[[-171.0, -172.0], [-174.0, 
-175.0], [-177.0, -178.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RURU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,1:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[63.0, 64.0], [66.0, 67.0], [69.0, 70.0]]], 
[[[93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[108.0, 109.0], [111.0, 112.0], [114.0, 
115.0]]]])+(1.-msk_ref)*numpy.array([[[[-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0]], [[-63.0, -64.0], [-66.0, -67.0], 
[-69.0, -70.0]]], [[[-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0]], [[-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, 
-164.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,:,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[106.0], [109.0], [112.0], [115.0], [118.0]]], [[[151.0], [154.0], [157.0], [160.0], 
[163.0]]]])+(1.-msk_ref)*numpy.array([[[[-106.0], [-109.0], [-112.0], [-115.0], [-118.0]]], [[[-151.0], [-154.0], [-157.0], 
[-160.0], [-163.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 
115.0]])+(1.-msk_ref)*numpy.array([[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]])+(1.-msk_ref)*numpy.array([[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], 
[-132.0, -133.0, -134.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PLCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,2:,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]])+(1.-msk_ref)*numpy.array([[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, 
-128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[61.0], [64.0], [67.0], [70.0]]], [[[106.0], [109.0], [112.0], 
[115.0]]]])+(1.-msk_ref)*numpy.array([[[[-61.0], [-64.0], [-67.0], [-70.0]]], [[[-106.0], [-109.0], [-112.0], [-115.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_ULPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,2:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 
128.0]]])+(1.-msk_ref)*numpy.array([[[-36.0, -37.0, -38.0]], [[-81.0, -82.0, -83.0]], [[-126.0, -127.0, -128.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,:4,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], 
[[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 
176.0]]]])+(1.-msk_ref)*numpy.array([[[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]])+(1.-msk_ref)*numpy.array([[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,2:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[111.0, 112.0], [114.0, 115.0], [117.0, 
118.0]]]])+(1.-msk_ref)*numpy.array([[[[-66.0, -67.0], [-69.0, -70.0], [-72.0, -73.0]]], [[[-111.0, -112.0], [-114.0, -115.0], 
[-117.0, -118.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[37.0], [82.0], [127.0], [172.0]])+(1.-msk_ref)*numpy.array([[-37.0], [-82.0], [-127.0], 
[-172.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,1:4,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[95.0], [98.0], [101.0]], [[110.0], [113.0], [116.0]], [[125.0], [128.0], [131.0]]], 
[[[140.0], [143.0], [146.0]], [[155.0], [158.0], [161.0]], [[170.0], [173.0], [176.0]]]])+(1.-msk_ref)*numpy.array([[[[-95.0], 
[-98.0], [-101.0]], [[-110.0], [-113.0], [-116.0]], [[-125.0], [-128.0], [-131.0]]], [[[-140.0], [-143.0], [-146.0]], 
[[-155.0], [-158.0], [-161.0]], [[-170.0], [-173.0], [-176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,2,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([111.0, 112.0])+(1.-msk_ref)*numpy.array([-111.0, -112.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LUCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0], [102.0, 103.0]], [[105.0, 106.0], 
[108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]], [[[135.0, 136.0], [138.0, 139.0], [141.0, 142.0], [144.0, 
145.0], [147.0, 148.0]], [[150.0, 151.0], [153.0, 154.0], [156.0, 157.0], [159.0, 160.0], [162.0, 
163.0]]]])+(1.-msk_ref)*numpy.array([[[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0], [-102.0, -103.0]], 
[[-105.0, -106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0], [-117.0, -118.0]]], [[[-135.0, -136.0], [-138.0, 
-139.0], [-141.0, -142.0], [-144.0, -145.0], [-147.0, -148.0]], [[-150.0, -151.0], [-153.0, -154.0], [-156.0, -157.0], [-159.0, 
-160.0], [-162.0, -163.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]])+(1.-msk_ref)*numpy.array([[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]])+(1.-msk_ref)*numpy.array([[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], 
[129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])+(1.-msk_ref)*numpy.array([[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], 
[-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-81.0, -82.0, -83.0], [-84.0, 
-85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], 
[-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], 
[[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]])+(1.-msk_ref)*numpy.array([[[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]]], [[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0], [156.0, 157.0, 
158.0]])+(1.-msk_ref)*numpy.array([[-21.0, -22.0, -23.0], [-66.0, -67.0, -68.0], [-111.0, -112.0, -113.0], [-156.0, -157.0, 
-158.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLLP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,2:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[37.0, 40.0, 43.0]], [[82.0, 85.0, 88.0]], [[127.0, 130.0, 133.0]], [[172.0, 175.0, 
178.0]]])+(1.-msk_ref)*numpy.array([[[-37.0, -40.0, -43.0]], [[-82.0, -85.0, -88.0]], [[-127.0, -130.0, -133.0]], [[-172.0, 
-175.0, -178.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, 
-55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 
82.0, 83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-33.0, -34.0, -35.0], [-36.0, -37.0, 
-38.0], [-39.0, -40.0, -41.0]]], [[[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-123.0, -124.0, 
-125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, 
-175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 
68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 
157.0, 158.0], [171.0, 172.0, 173.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0], [-36.0, -37.0, 
-38.0]], [[-51.0, -52.0, -53.0], [-66.0, -67.0, -68.0], [-81.0, -82.0, -83.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, 
-113.0], [-126.0, -127.0, -128.0]], [[-141.0, -142.0, -143.0], [-156.0, -157.0, -158.0], [-171.0, -172.0, -173.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], 
[144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 
166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-90.0, 
-91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], 
[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, 
-143.0], [-144.0, -145.0, -146.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]])+(1.-msk_ref)*numpy.array([[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], 
[-117.0, -118.0, -119.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 
173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]], [[[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]])+(1.-msk_ref)*numpy.array([[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[49.0], [52.0], [55.0]], [[64.0], [67.0], [70.0]], [[79.0], [82.0], [85.0]]], [[[94.0], 
[97.0], [100.0]], [[109.0], [112.0], [115.0]], [[124.0], [127.0], [130.0]]]])+(1.-msk_ref)*numpy.array([[[[-49.0], [-52.0], 
[-55.0]], [[-64.0], [-67.0], [-70.0]], [[-79.0], [-82.0], [-85.0]]], [[[-94.0], [-97.0], [-100.0]], [[-109.0], [-112.0], 
[-115.0]], [[-124.0], [-127.0], [-130.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 
98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 158.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], 
[-21.0, -22.0, -23.0]], [[-51.0, -52.0, -53.0], [-66.0, -67.0, -68.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]], 
[[-141.0, -142.0, -143.0], [-156.0, -157.0, -158.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 
26.0], [27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, 
-13.0, -14.0]], [[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, 
-118.0, -119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RUCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,:,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0], [57.0, 58.0]], [[60.0, 61.0], [63.0, 
64.0], [66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0], [102.0, 103.0]], 
[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]]])+(1.-msk_ref)*numpy.array([[[[-45.0, -46.0], 
[-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0], [-57.0, -58.0]], [[-60.0, -61.0], [-63.0, -64.0], [-66.0, -67.0], [-69.0, 
-70.0], [-72.0, -73.0]]], [[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0], [-102.0, -103.0]], [[-105.0, 
-106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0], [-117.0, -118.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UUUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,:4,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0], [5.0], [8.0], [11.0]], [[17.0], [20.0], [23.0], [26.0]]], [[[47.0], [50.0], [53.0], 
[56.0]], [[62.0], [65.0], [68.0], [71.0]]], [[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], 
[116.0]]]])+(1.-msk_ref)*numpy.array([[[[-2.0], [-5.0], [-8.0], [-11.0]], [[-17.0], [-20.0], [-23.0], [-26.0]]], [[[-47.0], 
[-50.0], [-53.0], [-56.0]], [[-62.0], [-65.0], [-68.0], [-71.0]]], [[[-92.0], [-95.0], [-98.0], [-101.0]], [[-107.0], [-110.0], 
[-113.0], [-116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_URCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]]], [[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]])+(1.-msk_ref)*numpy.array([[[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, 
-73.0, -74.0]]], [[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLLP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,2:,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[127.0, 130.0, 133.0]], [[172.0, 175.0, 178.0]]])+(1.-msk_ref)*numpy.array([[[-127.0, -130.0, 
-133.0]], [[-172.0, -175.0, -178.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([112.0])+(1.-msk_ref)*numpy.array([-112.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]])+(1.-msk_ref)*numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, 
-95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, 
-113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 
44.0]]], [[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_ref)*numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0], [-42.0, -43.0, -44.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], 
[-87.0, -88.0, -89.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, 
-175.0, -176.0], [-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]], [[156.0, 157.0, 
158.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, -22.0, -23.0]], [[-66.0, -67.0, -68.0]], [[-111.0, -112.0, -113.0]], [[-156.0, 
-157.0, -158.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]])+(1.-msk_ref)*numpy.array([[[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, 
-86.0], [-87.0, -88.0, -89.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[113.0], [116.0], [119.0]]], [[[158.0], [161.0], 
[164.0]]]])+(1.-msk_ref)*numpy.array([[[[-113.0], [-116.0], [-119.0]]], [[[-158.0], [-161.0], [-164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]])+(1.-msk_ref)*numpy.array([[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]])+(1.-msk_ref)*numpy.array([[[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, 
-70.0, -71.0]]], [[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 
113.0]])+(1.-msk_ref)*numpy.array([[-21.0, -22.0, -23.0], [-66.0, -67.0, -68.0], [-111.0, -112.0, -113.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], 
[[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-96.0, -97.0, -98.0], 
[-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-138.0, 
-139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], 
[-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]])+(1.-msk_ref)*numpy.array([[-96.0, -97.0, -98.0], 
[-111.0, -112.0, -113.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]])+(1.-msk_ref)*numpy.array([[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])+(1.-msk_ref)*numpy.array([[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], 
[-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-96.0, -97.0, -98.0], 
[-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([111.0, 112.0, 113.0])+(1.-msk_ref)*numpy.array([-111.0, -112.0, -113.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]])+(1.-msk_ref)*numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,2:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]])+(1.-msk_ref)*numpy.array([[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], 
[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], 
[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, 
-8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], 
[-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, 
-55.0, -56.0], [-57.0, -58.0, -59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0], [-72.0, -73.0, -74.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]], [[[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]])+(1.-msk_ref)*numpy.array([[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]])+(1.-msk_ref)*numpy.array([[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_ULUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,2:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 
76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])+(1.-msk_ref)*numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], 
[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, 
-85.0, -86.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], 
[-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-150.0, 
-151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 
26.0], [27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 
56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], 
[[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]], [[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, 
-7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, 
-29.0]], [[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], [[[-51.0, -52.0, -53.0], [-54.0, -55.0, 
-56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-81.0, -82.0, -83.0], 
[-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], 
[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]])+(1.-msk_ref)*numpy.array([[[[-60.0, 
-61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-105.0, -106.0, -107.0], [-108.0, 
-109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 
76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 
175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, 
-40.0, -41.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-165.0, -166.0, -167.0], 
[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0]]])+(1.-msk_ref)*numpy.array([[[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]], [[-141.0, -142.0, -143.0], [-156.0, 
-157.0, -158.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])+(1.-msk_ref)*numpy.array([[[-60.0, -61.0, 
-62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], 
[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], 
[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, 
-70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, 
-124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_URLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 
70.0, 71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, 
-119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 
68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 
128.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0], [-36.0, -37.0, -38.0]], [[-51.0, -52.0, -53.0], 
[-66.0, -67.0, -68.0], [-81.0, -82.0, -83.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, -128.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 
70.0, 71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]]], [[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-111.0, -112.0, 
-113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,1:4,1:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[64.0], [67.0], [70.0]]], [[[109.0], [112.0], [115.0]]]])+(1.-msk_ref)*numpy.array([[[[-64.0], 
[-67.0], [-70.0]]], [[[-109.0], [-112.0], [-115.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], 
[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 
67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0]]], [[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 
145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], 
[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], 
[[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, 
-64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, 
-83.0], [-84.0, -85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-135.0, -136.0, -137.0], 
[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, 
-155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, 
-172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 
29.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], 
[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0], [-27.0, -28.0, -29.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], 
[-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, 
-160.0, -161.0], [-162.0, -163.0, -164.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 
103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], 
[[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 
151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_ref)*numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, 
-143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], 
[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[126.0, 127.0, 128.0]], [[171.0, 172.0, 173.0]]])+(1.-msk_ref)*numpy.array([[[-126.0, -127.0, 
-128.0]], [[-171.0, -172.0, -173.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]])+(1.-msk_ref)*numpy.array([[[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, 
-55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, 
-7.0, -8.0], [-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]], [[-33.0, -34.0, 
-35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], 
[[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, 
-85.0, -86.0]]], [[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], 
[[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0]], [[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-150.0, -151.0, -152.0], [-153.0, 
-154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 
128.0]])+(1.-msk_ref)*numpy.array([[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, -128.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]])+(1.-msk_ref)*numpy.array([[[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, 
-74.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 
178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], 
[-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, 
-173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]], [[171.0, 172.0, 
173.0]]])+(1.-msk_ref)*numpy.array([[[-36.0, -37.0, -38.0]], [[-81.0, -82.0, -83.0]], [[-126.0, -127.0, -128.0]], [[-171.0, 
-172.0, -173.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]])+(1.-msk_ref)*numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, 
-113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_ULRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,2:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 
82.0, 83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]]])+(1.-msk_ref)*numpy.array([[[[-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-78.0, -79.0, 
-80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-123.0, -124.0, -125.0], [-126.0, -127.0, 
-128.0], [-129.0, -130.0, -131.0]]], [[[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]])+(1.-msk_ref)*numpy.array([[[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], 
[39.0, 40.0, 41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, 
-7.0, -8.0], [-9.0, -10.0, -11.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, 
-26.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0]]], [[[-45.0, -46.0, 
-47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], 
[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, 
-85.0, -86.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, 
-106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], 
[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[66.0, 67.0, 68.0], [111.0, 112.0, 113.0]])+(1.-msk_ref)*numpy.array([[-66.0, -67.0, -68.0], 
[-111.0, -112.0, -113.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-108.0, -109.0, -110.0], [-111.0, -112.0, 
-113.0], [-114.0, -115.0, -116.0]]], [[[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[128.0], [131.0], [134.0]]], [[[173.0], [176.0], 
[179.0]]]])+(1.-msk_ref)*numpy.array([[[[-128.0], [-131.0], [-134.0]]], [[[-173.0], [-176.0], [-179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], 
[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], 
[54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0]], 
[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-45.0, -46.0, -47.0], [-48.0, 
-49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, 
-68.0], [-69.0, -70.0, -71.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-135.0, 
-136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-150.0, -151.0, -152.0], 
[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 
26.0], [27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], 
[[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, 
-10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]]], [[[-51.0, -52.0, 
-53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], 
[[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]], [[[-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[111.0, 112.0, 113.0], [156.0, 157.0, 158.0]])+(1.-msk_ref)*numpy.array([[-111.0, -112.0, 
-113.0], [-156.0, -157.0, -158.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0]]])+(1.-msk_ref)*numpy.array([[[-51.0, -52.0, -53.0], [-66.0, -67.0, -68.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, 
-113.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], 
[156.0, 157.0, 158.0], [171.0, 172.0, 173.0]]])+(1.-msk_ref)*numpy.array([[[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], 
[-126.0, -127.0, -128.0]], [[-141.0, -142.0, -143.0], [-156.0, -157.0, -158.0], [-171.0, -172.0, -173.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,2:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], 
[[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 
176.0]]]])+(1.-msk_ref)*numpy.array([[[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, 
-130.0, -131.0]]], [[[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 
26.0], [27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 
56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], [-12.0, -13.0, -14.0]], [[-21.0, -22.0, 
-23.0], [-24.0, -25.0, -26.0], [-27.0, -28.0, -29.0]], [[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], 
[[[-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, 
-73.0, -74.0]], [[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-96.0, -97.0, -98.0], [-99.0, 
-100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], 
[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])+(1.-msk_ref)*numpy.array([[[[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], [[[-120.0, -121.0, -122.0], [-123.0, 
-124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], 
[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], 
[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 
146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], 
[162.0, 163.0, 164.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, 
-11.0], [-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], 
[-27.0, -28.0, -29.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, 
-58.0, -59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, 
-74.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, 
-104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, 
-118.0, -119.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], 
[-147.0, -148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, 
-161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_URRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,1:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 
67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 
116.0]]]])+(1.-msk_ref)*numpy.array([[[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-63.0, -64.0, 
-65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])+(1.-msk_ref)*numpy.array([[[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], 
[-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_ULLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 
85.0, 86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]])+(1.-msk_ref)*numpy.array([[[[-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, -44.0]]], [[[-81.0, -82.0, 
-83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, 
-134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,:4,:2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], 
[24.0, 25.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 
70.0]]], [[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 
115.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -1.0], [-3.0, -4.0], [-6.0, -7.0], [-9.0, -10.0]], [[-15.0, -16.0], [-18.0, 
-19.0], [-21.0, -22.0], [-24.0, -25.0]]], [[[-45.0, -46.0], [-48.0, -49.0], [-51.0, -52.0], [-54.0, -55.0]], [[-60.0, -61.0], 
[-63.0, -64.0], [-66.0, -67.0], [-69.0, -70.0]]], [[[-90.0, -91.0], [-93.0, -94.0], [-96.0, -97.0], [-99.0, -100.0]], [[-105.0, 
-106.0], [-108.0, -109.0], [-111.0, -112.0], [-114.0, -115.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]]])+(1.-msk_ref)*numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], 
[[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 
163.0, 164.0]], [[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-96.0, 
-97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], 
[-117.0, -118.0, -119.0]], [[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-141.0, -142.0, 
-143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, -178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 
103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]])+(1.-msk_ref)*numpy.array([[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 
125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])+(1.-msk_ref)*numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, 
-50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], 
[-69.0, -70.0, -71.0]], [[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0]]], 
[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], 
[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, 
-125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]])+(1.-msk_ref)*numpy.array([[[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], 
[-114.0, -115.0, -116.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 
61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], 
[111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 
160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, 
-25.0, -26.0]]], [[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-105.0, 
-106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]], [[[-150.0, -151.0, -152.0], 
[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,2:,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 
85.0, 86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 
173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])+(1.-msk_ref)*numpy.array([[[[-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0], [-42.0, -43.0, -44.0]]], [[[-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-126.0, -127.0, 
-128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], [[[-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], [-177.0, 
-178.0, -179.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])+(1.-msk_ref)*numpy.array([[[[-93.0, -94.0, -95.0], 
[-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]], [[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]], [[[-138.0, -139.0, -140.0], [-141.0, 
-142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]], 
[[-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[111.0, 112.0, 113.0]])+(1.-msk_ref)*numpy.array([[-111.0, -112.0, -113.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,1,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], 
[-24.0, -25.0, -26.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]], [[-108.0, -109.0, -110.0], 
[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]], [[-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, 
-161.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 
74.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]])+(1.-msk_ref)*numpy.array([[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0], [-72.0, -73.0, -74.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0], [-117.0, -118.0, -119.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_CURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:,:2,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], 
[24.0, 25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], 
[-9.0, -10.0, -11.0]], [[-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0]]], [[[-48.0, -49.0, -50.0], [-51.0, 
-52.0, -53.0], [-54.0, -55.0, -56.0]], [[-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0]]], [[[-93.0, -94.0, 
-95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, 
-116.0]]], [[[-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0]], [[-153.0, -154.0, -155.0], [-156.0, 
-157.0, -158.0], [-159.0, -160.0, -161.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 
92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 
109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])+(1.-msk_ref)*numpy.array([[[[-45.0, 
-46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-60.0, -61.0, 
-62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]]], [[[-90.0, -91.0, -92.0], 
[-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], 
[-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,1,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]])+(1.-msk_ref)*numpy.array([[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 
59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 
95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])+(1.-msk_ref)*numpy.array([[[[-45.0, -46.0, -47.0], [-48.0, -49.0, 
-50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], [[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], 
[-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, 
-82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, 
-98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, 
-112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], 
[-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_LUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2:,:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 
103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]])+(1.-msk_ref)*numpy.array([[[[-90.0, -91.0, -92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, 
-101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]], [[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], 
[-144.0, -145.0, -146.0], [-147.0, -148.0, -149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, 
-158.0], [-159.0, -160.0, -161.0], [-162.0, -163.0, -164.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]])+(1.-msk_ref)*numpy.array([[[-81.0, -82.0, 
-83.0]], [[-126.0, -127.0, -128.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,:2,:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])+(1.-msk_ref)*numpy.array([[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0]], [[-105.0, -106.0, -107.0], [-108.0, -109.0, 
-110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], 
[132.0, 133.0, 134.0]]])+(1.-msk_ref)*numpy.array([[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, 
-128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_ULCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,2:,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 
44.0]]], [[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 
121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 
134.0]]]])+(1.-msk_ref)*numpy.array([[[[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, 
-41.0], [-42.0, -43.0, -44.0]]], [[[-75.0, -76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], 
[-87.0, -88.0, -89.0]]], [[[-120.0, -121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, 
-131.0], [-132.0, -133.0, -134.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,2:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]])+(1.-msk_ref)*numpy.array([[[[-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, 
-85.0, -86.0]]], [[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,1:2,:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 
74.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]]]])+(1.-msk_ref)*numpy.array([[[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, 
-71.0], [-72.0, -73.0, -74.0]]], [[[-105.0, -106.0, -107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, 
-115.0, -116.0], [-117.0, -118.0, -119.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_UUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[:3,:2,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 
98.0], [111.0, 112.0, 113.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -7.0, -8.0], [-21.0, -22.0, -23.0]], [[-51.0, -52.0, -53.0], 
[-66.0, -67.0, -68.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,2:,1:4]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 
131.0]]])+(1.-msk_ref)*numpy.array([[[-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,2,1]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(112.0)+(1.-msk_ref)*(-112.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_PPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[2,1,2:]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 
119.0]])+(1.-msk_ref)*numpy.array([[-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_expandedData_rank4_RCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], 
[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], 
[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])+(1.-msk_arg)*numpy.array([[[[-0.0, -1.0, -2.0], [-3.0, -4.0, -5.0], [-6.0, -7.0, -8.0], [-9.0, -10.0, -11.0], 
[-12.0, -13.0, -14.0]], [[-15.0, -16.0, -17.0], [-18.0, -19.0, -20.0], [-21.0, -22.0, -23.0], [-24.0, -25.0, -26.0], [-27.0, 
-28.0, -29.0]], [[-30.0, -31.0, -32.0], [-33.0, -34.0, -35.0], [-36.0, -37.0, -38.0], [-39.0, -40.0, -41.0], [-42.0, -43.0, 
-44.0]]], [[[-45.0, -46.0, -47.0], [-48.0, -49.0, -50.0], [-51.0, -52.0, -53.0], [-54.0, -55.0, -56.0], [-57.0, -58.0, -59.0]], 
[[-60.0, -61.0, -62.0], [-63.0, -64.0, -65.0], [-66.0, -67.0, -68.0], [-69.0, -70.0, -71.0], [-72.0, -73.0, -74.0]], [[-75.0, 
-76.0, -77.0], [-78.0, -79.0, -80.0], [-81.0, -82.0, -83.0], [-84.0, -85.0, -86.0], [-87.0, -88.0, -89.0]]], [[[-90.0, -91.0, 
-92.0], [-93.0, -94.0, -95.0], [-96.0, -97.0, -98.0], [-99.0, -100.0, -101.0], [-102.0, -103.0, -104.0]], [[-105.0, -106.0, 
-107.0], [-108.0, -109.0, -110.0], [-111.0, -112.0, -113.0], [-114.0, -115.0, -116.0], [-117.0, -118.0, -119.0]], [[-120.0, 
-121.0, -122.0], [-123.0, -124.0, -125.0], [-126.0, -127.0, -128.0], [-129.0, -130.0, -131.0], [-132.0, -133.0, -134.0]]], 
[[[-135.0, -136.0, -137.0], [-138.0, -139.0, -140.0], [-141.0, -142.0, -143.0], [-144.0, -145.0, -146.0], [-147.0, -148.0, 
-149.0]], [[-150.0, -151.0, -152.0], [-153.0, -154.0, -155.0], [-156.0, -157.0, -158.0], [-159.0, -160.0, -161.0], [-162.0, 
-163.0, -164.0]], [[-165.0, -166.0, -167.0], [-168.0, -169.0, -170.0], [-171.0, -172.0, -173.0], [-174.0, -175.0, -176.0], 
[-177.0, -178.0, -179.0]]]])
      res=arg[1:3,:,2]
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 
112.0, 113.0], [126.0, 127.0, 128.0]]])+(1.-msk_ref)*numpy.array([[[-51.0, -52.0, -53.0], [-66.0, -67.0, -68.0], [-81.0, -82.0, 
-83.0]], [[-96.0, -97.0, -98.0], [-111.0, -112.0, -113.0], [-126.0, -127.0, -128.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank1_R(self):
      arg=Symbol(shape=(3,))
      res=arg[1:2]
      s=numpy.array([0.0, 1.0, 2.0])
      sub=res.substitute({arg:s})
      ref=numpy.array([1.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank1_L(self):
      arg=Symbol(shape=(3,))
      res=arg[2:]
      s=numpy.array([0.0, 1.0, 2.0])
      sub=res.substitute({arg:s})
      ref=numpy.array([2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank1_U(self):
      arg=Symbol(shape=(3,))
      res=arg[:2]
      s=numpy.array([0.0, 1.0, 2.0])
      sub=res.substitute({arg:s})
      ref=numpy.array([0.0, 1.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank1_P(self):
      arg=Symbol(shape=(3,))
      res=arg[1]
      s=numpy.array([0.0, 1.0, 2.0])
      sub=res.substitute({arg:s})
      ref=numpy.array(1.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank1_C(self):
      arg=Symbol(shape=(3,))
      res=arg[:]
      s=numpy.array([0.0, 1.0, 2.0])
      sub=res.substitute({arg:s})
      ref=numpy.array([0.0, 1.0, 2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_LU(self):
      arg=Symbol(shape=(3, 4))
      res=arg[2:,:3]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[8.0, 9.0, 10.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_PL(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1,2:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([6.0, 7.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_RC(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1:2,:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[4.0, 5.0, 6.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_UP(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:2,2]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([2.0, 6.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_UC(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:2,:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_UN(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:2]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_CN(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_CP(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:,2]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([2.0, 6.0, 10.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_RR(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1:2,1:3]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[5.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_UR(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:2,1:3]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[1.0, 2.0], [5.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_RN(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1:2]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[4.0, 5.0, 6.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_PR(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1,1:3]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([5.0, 6.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_UU(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:2,:3]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[0.0, 1.0, 2.0], [4.0, 5.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_CC(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:,:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_CL(self):
      arg=Symbol(shape=(3, 4))
      res=arg[:,2:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[2.0, 3.0], [6.0, 7.0], [10.0, 11.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_RL(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1:2,2:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[6.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_PN(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([4.0, 5.0, 6.0, 7.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_LN(self):
      arg=Symbol(shape=(3, 4))
      res=arg[2:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[8.0, 9.0, 10.0, 11.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_LL(self):
      arg=Symbol(shape=(3, 4))
      res=arg[2:,2:]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[10.0, 11.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank2_PP(self):
      arg=Symbol(shape=(3, 4))
      res=arg[1,2]
      s=numpy.array([[0.0, 1.0, 2.0, 3.0], [4.0, 5.0, 6.0, 7.0], [8.0, 9.0, 10.0, 11.0]])
      sub=res.substitute({arg:s})
      ref=numpy.array(6.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LUP(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,:3,1]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[25.0, 28.0, 31.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CPP(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2,1]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([7.0, 19.0, 31.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CUU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]], [[24.0, 25.0], [27.0, 
28.0], [30.0, 31.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LRU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,1:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[27.0, 28.0], [30.0, 31.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CUR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:3,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[1.0], [4.0], [7.0]], [[13.0], [16.0], [19.0]], [[25.0], [28.0], [31.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CPR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[7.0], [19.0], [31.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RRC(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,1:3,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LRN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,1:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CLL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[8.0], [11.0]], [[20.0], [23.0]], [[32.0], [35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PRL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,1:3,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[17.0], [20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PUU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PLL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[20.0], [23.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RUU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CCP(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:,1]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[1.0, 4.0, 7.0, 10.0], [13.0, 16.0, 19.0, 22.0], [25.0, 28.0, 31.0, 34.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PPR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([19.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RPN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[18.0, 19.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_UCL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[2.0], [5.0], [8.0], [11.0]], [[14.0], [17.0], [20.0], [23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LRL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,1:3,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[29.0], [32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PUR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,:3,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[13.0], [16.0], [19.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RCN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PCU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,:,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], [21.0, 22.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LLN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RRL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,1:3,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[17.0], [20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LUC(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,:3,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PRN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,1:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CLR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2:,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[7.0], [10.0]], [[19.0], [22.0]], [[31.0], [34.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_UCN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_ULN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RUN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CCU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], [21.0, 
22.0]], [[24.0, 25.0], [27.0, 28.0], [30.0, 31.0], [33.0, 34.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PCN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PPP(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2,1]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array(19.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RRN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,1:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CLP(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2:,1]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[7.0, 10.0], [19.0, 22.0], [31.0, 34.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RCL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[14.0], [17.0], [20.0], [23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LPN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[30.0, 31.0, 32.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_URN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,1:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_ULR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,2:,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[7.0], [10.0]], [[19.0], [22.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RRU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,1:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0], [18.0, 19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LUU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[24.0, 25.0], [27.0, 28.0], [30.0, 31.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LUN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RCU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[12.0, 13.0], [15.0, 16.0], [18.0, 19.0], [21.0, 22.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_UPN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RPU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,2,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[18.0, 19.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PRR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,1:3,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[16.0], [19.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PPL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([20.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CUN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 
19.0, 20.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PPU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([18.0, 19.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CCN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RRR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,1:3,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[16.0], [19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_ULL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,2:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[8.0], [11.0]], [[20.0], [23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CRN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,1:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[15.0, 16.0, 17.0], [18.0, 19.0, 20.0]], [[27.0, 28.0, 29.0], 
[30.0, 31.0, 32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PPN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([18.0, 19.0, 20.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_UUU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,:3,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0]], [[12.0, 13.0], [15.0, 16.0], [18.0, 19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PLU(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2:,:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[18.0, 19.0], [21.0, 22.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CPN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0], [30.0, 31.0, 32.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PCL(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[14.0], [17.0], [20.0], [23.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PLN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RCP(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:,1]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[13.0, 16.0, 19.0, 22.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LCN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RCC(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,:,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CCC(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,:,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_UPC(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,2,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[6.0, 7.0, 8.0], [18.0, 19.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_PUN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1,:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 19.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LCC(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,:,:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_UUN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:2,:3]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 17.0], [18.0, 
19.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_CLN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[:,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[30.0, 31.0, 32.0], 
[33.0, 34.0, 35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_LPR(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[2:,2,1:2]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[31.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank3_RLN(self):
      arg=Symbol(shape=(3, 4, 3))
      res=arg[1:2,2:]
      s=numpy.array([[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[12.0, 13.0, 14.0], [15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0]], [[24.0, 25.0, 26.0], [27.0, 28.0, 29.0], [30.0, 31.0, 32.0], [33.0, 34.0, 
35.0]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RCPR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:,2,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[52.0], [67.0], [82.0]], [[97.0], [112.0], [127.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUUL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[2.0], [5.0], [8.0], [11.0]], [[17.0], [20.0], [23.0], [26.0]]], [[[47.0], [50.0], [53.0], [56.0]], 
[[62.0], [65.0], [68.0], [71.0]]], [[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], [116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URLC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RURC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,1:4,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRPU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,2,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[111.0, 112.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUPP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,2,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[7.0, 22.0], [52.0, 67.0], [97.0, 112.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUPL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[8.0], [23.0]], [[53.0], [68.0]], [[98.0], [113.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URCP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,:,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[16.0, 19.0, 22.0, 25.0, 28.0]], [[61.0, 64.0, 67.0, 70.0, 73.0]], [[106.0, 109.0, 112.0, 115.0, 
118.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[111.0, 112.0, 113.0], [156.0, 157.0, 158.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLLU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,2:,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[126.0, 127.0], [129.0, 130.0], [132.0, 133.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLUL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[32.0], [35.0], [38.0], [41.0]]], [[[77.0], [80.0], [83.0], [86.0]]], [[[122.0], [125.0], [128.0], 
[131.0]]], [[[167.0], [170.0], [173.0], [176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCRC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,1:4,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLLC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUPU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,2,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0], [21.0, 22.0]], [[51.0, 52.0], [66.0, 67.0]], [[96.0, 97.0], [111.0, 112.0]], [[141.0, 
142.0], [156.0, 157.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 
145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 166.0, 
167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPRL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,1:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[20.0], [23.0], [26.0]], [[65.0], [68.0], [71.0]], [[110.0], [113.0], [116.0]], [[155.0], [158.0], 
[161.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPRL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,1:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[110.0], [113.0], [116.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRPR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,2,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[67.0]], [[112.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLLR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,2:,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[127.0], [130.0], [133.0]]], [[[172.0], [175.0], [178.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUCL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[2.0], [5.0], [8.0], [11.0], [14.0]], [[17.0], [20.0], [23.0], [26.0], [29.0]]], [[[47.0], [50.0], 
[53.0], [56.0], [59.0]], [[62.0], [65.0], [68.0], [71.0], [74.0]]], [[[92.0], [95.0], [98.0], [101.0], [104.0]], [[107.0], 
[110.0], [113.0], [116.0], [119.0]]], [[[137.0], [140.0], [143.0], [146.0], [149.0]], [[152.0], [155.0], [158.0], [161.0], 
[164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RULU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,2:,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[51.0, 52.0], [54.0, 55.0], [57.0, 58.0]], [[66.0, 67.0], [69.0, 70.0], [72.0, 73.0]]], [[[96.0, 
97.0], [99.0, 100.0], [102.0, 103.0]], [[111.0, 112.0], [114.0, 115.0], [117.0, 118.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RLRL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,2:,1:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[80.0], [83.0], [86.0]]], [[[125.0], [128.0], [131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCCR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,:,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[91.0], [94.0], [97.0], [100.0], [103.0]], [[106.0], [109.0], [112.0], [115.0], [118.0]], [[121.0], 
[124.0], [127.0], [130.0], [133.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UCRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 
25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 
55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 
85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULCU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,:,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[30.0, 31.0], [33.0, 34.0], [36.0, 37.0], [39.0, 40.0], [42.0, 43.0]]], [[[75.0, 76.0], [78.0, 79.0], 
[81.0, 82.0], [84.0, 85.0], [87.0, 88.0]]], [[[120.0, 121.0], [123.0, 124.0], [126.0, 127.0], [129.0, 130.0], [132.0, 133.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPRL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,1:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[65.0], [68.0], [71.0]], [[110.0], [113.0], [116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RUPL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[53.0], [68.0]], [[98.0], [113.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 
116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRUL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[107.0], [110.0], [113.0], [116.0]]], [[[152.0], [155.0], [158.0], [161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCCC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPUC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,:4,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URUU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,:4,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0]]], [[[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], 
[69.0, 70.0]]], [[[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRUU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,:4,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0]]], [[[105.0, 106.0], [108.0, 109.0], [111.0, 
112.0], [114.0, 115.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UCRC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:,1:4,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 
25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 
55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 
85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRPP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,2,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([112.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PURN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UCUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], [18.0, 
19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 
41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 
65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 
86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 
109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URRP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,1:4,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[19.0, 22.0, 25.0]], [[64.0, 67.0, 70.0]], [[109.0, 112.0, 115.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], 
[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 107.0], 
[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRLR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,2:,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[67.0], [70.0], [73.0]]], [[[112.0], [115.0], [118.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LURN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 
155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCRR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,1:4,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[94.0], [97.0], [100.0]], [[109.0], [112.0], [115.0]], [[124.0], [127.0], [130.0]]], [[[139.0], 
[142.0], [145.0]], [[154.0], [157.0], [160.0]], [[169.0], [172.0], [175.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPLC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LUUL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:2,:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[92.0], [95.0], [98.0], [101.0]], [[107.0], [110.0], [113.0], [116.0]]], [[[137.0], [140.0], [143.0], 
[146.0]], [[152.0], [155.0], [158.0], [161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPPC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[111.0, 112.0, 113.0], [156.0, 157.0, 158.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPPC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0], [156.0, 157.0, 158.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCLC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRRR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,1:4,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[64.0], [67.0], [70.0]]], [[[109.0], [112.0], [115.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRUL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[62.0], [65.0], [68.0], [71.0]]], [[[107.0], [110.0], [113.0], [116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPPL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([113.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRPP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,2,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[22.0], [67.0], [112.0], [157.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 98.0], 
[111.0, 112.0, 113.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUCC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], [48.0, 
49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 
148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRRR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,1:4,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[19.0], [22.0], [25.0]]], [[[64.0], [67.0], [70.0]]], [[[109.0], [112.0], [115.0]]], [[[154.0], 
[157.0], [160.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLCC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 
133.0, 134.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPLL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[113.0], [116.0], [119.0]], [[158.0], [161.0], [164.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPPU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,2,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([111.0, 112.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[111.0, 112.0, 113.0]], [[156.0, 157.0, 158.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RUUP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,:4,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[46.0, 49.0, 52.0, 55.0], [61.0, 64.0, 67.0, 70.0]], [[91.0, 94.0, 97.0, 100.0], [106.0, 109.0, 112.0, 
115.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UURC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,1:4,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 
25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 
70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCRR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,1:4,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[4.0], [7.0], [10.0]], [[19.0], [22.0], [25.0]], [[34.0], [37.0], [40.0]]], [[[49.0], [52.0], [55.0]], 
[[64.0], [67.0], [70.0]], [[79.0], [82.0], [85.0]]], [[[94.0], [97.0], [100.0]], [[109.0], [112.0], [115.0]], [[124.0], 
[127.0], [130.0]]], [[[139.0], [142.0], [145.0]], [[154.0], [157.0], [160.0]], [[169.0], [172.0], [175.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], 
[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCPU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,2,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0], [21.0, 22.0], [36.0, 37.0]], [[51.0, 52.0], [66.0, 67.0], [81.0, 82.0]], [[96.0, 97.0], 
[111.0, 112.0], [126.0, 127.0]], [[141.0, 142.0], [156.0, 157.0], [171.0, 172.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPLL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[113.0], [116.0], [119.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RCPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], [81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 
113.0], [126.0, 127.0, 128.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 
140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], 
[[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RUUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPUU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,:4,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 25.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], 
[69.0, 70.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RCUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], 
[84.0, 85.0, 86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUCC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], [48.0, 
49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRPL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[113.0]], [[158.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RCLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], 
[102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 
130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLUL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,:4,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[122.0], [125.0], [128.0], [131.0]]], [[[167.0], [170.0], [173.0], [176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULPP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,2,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[37.0], [82.0], [127.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RLRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,2:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRCL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[107.0], [110.0], [113.0], [116.0], [119.0]]], [[[152.0], [155.0], [158.0], [161.0], [164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], [18.0, 
19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 
55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 
94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[171.0, 172.0, 173.0], 
[174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], 
[[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]], [[156.0, 157.0, 158.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PUCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], 
[126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 
133.0, 134.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 
179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UULN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLCC(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], 
[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RCRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], 
[99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], [159.0, 
160.0, 161.0], [162.0, 163.0, 164.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCCR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,:,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[1.0], [4.0], [7.0], [10.0], [13.0]], [[16.0], [19.0], [22.0], [25.0], [28.0]], [[31.0], [34.0], 
[37.0], [40.0], [43.0]]], [[[46.0], [49.0], [52.0], [55.0], [58.0]], [[61.0], [64.0], [67.0], [70.0], [73.0]], [[76.0], [79.0], 
[82.0], [85.0], [88.0]]], [[[91.0], [94.0], [97.0], [100.0], [103.0]], [[106.0], [109.0], [112.0], [115.0], [118.0]], [[121.0], 
[124.0], [127.0], [130.0], [133.0]]], [[[136.0], [139.0], [142.0], [145.0], [148.0]], [[151.0], [154.0], [157.0], [160.0], 
[163.0]], [[166.0], [169.0], [172.0], [175.0], [178.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCPP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,2,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([97.0, 112.0, 127.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[156.0, 157.0, 158.0], [159.0, 
160.0, 161.0], [162.0, 163.0, 164.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[21.0, 22.0, 23.0]], [[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLCL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[122.0], [125.0], [128.0], [131.0], [134.0]]], [[[167.0], [170.0], [173.0], [176.0], [179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPPR(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,2,1:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[67.0], [112.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 
133.0, 134.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PULN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[150.0, 
151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPPP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,2,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array(112.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 98.0], 
[111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 158.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([111.0, 112.0, 113.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RLUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,2:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], 
[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CULN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRRU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,1:4,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[63.0, 64.0], [66.0, 67.0], [69.0, 70.0]]], [[[108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RURN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], [48.0, 
49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 
116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCCP(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,:,1]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[1.0, 4.0, 7.0, 10.0, 13.0], [16.0, 19.0, 22.0, 25.0, 28.0], [31.0, 34.0, 37.0, 40.0, 43.0]], [[46.0, 
49.0, 52.0, 55.0, 58.0], [61.0, 64.0, 67.0, 70.0, 73.0], [76.0, 79.0, 82.0, 85.0, 88.0]], [[91.0, 94.0, 97.0, 100.0, 103.0], 
[106.0, 109.0, 112.0, 115.0, 118.0], [121.0, 124.0, 127.0, 130.0, 133.0]], [[136.0, 139.0, 142.0, 145.0, 148.0], [151.0, 154.0, 
157.0, 160.0, 163.0], [166.0, 169.0, 172.0, 175.0, 178.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], 
[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 
136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 
152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 
25.0, 26.0]], [[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 
55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 
85.0, 86.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]], [[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[138.0, 139.0, 140.0], 
[141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[168.0, 
169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[45.0, 46.0, 47.0], [48.0, 
49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 
101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], 
[117.0, 118.0, 119.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 
148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RUPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[51.0, 52.0, 53.0], [66.0, 67.0, 68.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RLCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], 
[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], [[[66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]]], [[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[156.0, 157.0, 158.0], 
[159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PUPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LUUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], 
[141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 
160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[75.0, 76.0, 
77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0]]], [[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 
127.0, 128.0], [129.0, 130.0, 131.0]]], [[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 
176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], 
[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0], [156.0, 157.0, 158.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 
157.0, 158.0], [171.0, 172.0, 173.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], 
[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 
158.0], [171.0, 172.0, 173.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[60.0, 61.0, 
62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 
112.0, 113.0], [114.0, 115.0, 116.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 
161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]], [[[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[66.0, 67.0, 68.0], [111.0, 112.0, 113.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RCCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], 
[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], 
[78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], 
[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], 
[129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PUUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RULN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CURN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 
25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 
70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]]], [[[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], [[153.0, 154.0, 155.0], 
[156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_URRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[63.0, 64.0, 65.0], [66.0, 67.0, 
68.0], [69.0, 70.0, 71.0]]], [[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], [18.0, 
19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 
41.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 
65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 
86.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 
109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 
128.0], [129.0, 130.0, 131.0]]], [[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0]], 
[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]], [[165.0, 166.0, 167.0], [168.0, 
169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[36.0, 37.0, 38.0]], [[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]], [[171.0, 172.0, 173.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CUUU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:2,:4,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 
25.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0]]], 
[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]], 
[[[135.0, 136.0], [138.0, 139.0], [141.0, 142.0], [144.0, 145.0]], [[150.0, 151.0], [153.0, 154.0], [156.0, 157.0], [159.0, 
160.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 2, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LRCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 
118.0, 119.0]]], [[[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 
164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LULN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]]], [[[141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 
158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[60.0, 61.0, 62.0], 
[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]], [[150.0, 151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UCCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RLLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 
131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 
113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[111.0, 112.0, 113.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], 
[69.0, 70.0, 71.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], [156.0, 
157.0, 158.0], [159.0, 160.0, 161.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CCCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 
16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UCPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[6.0, 7.0, 8.0], [21.0, 22.0, 23.0], [36.0, 37.0, 38.0]], [[51.0, 52.0, 53.0], [66.0, 67.0, 68.0], 
[81.0, 82.0, 83.0]], [[96.0, 97.0, 98.0], [111.0, 112.0, 113.0], [126.0, 127.0, 128.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], 
[[[165.0, 166.0, 167.0], [168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUUU(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,:4,:2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0], [3.0, 4.0], [6.0, 7.0], [9.0, 10.0]], [[15.0, 16.0], [18.0, 19.0], [21.0, 22.0], [24.0, 
25.0]]], [[[45.0, 46.0], [48.0, 49.0], [51.0, 52.0], [54.0, 55.0]], [[60.0, 61.0], [63.0, 64.0], [66.0, 67.0], [69.0, 70.0]]], 
[[[90.0, 91.0], [93.0, 94.0], [96.0, 97.0], [99.0, 100.0]], [[105.0, 106.0], [108.0, 109.0], [111.0, 112.0], [114.0, 115.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UURN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 
25.0, 26.0]]], [[[48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0]], [[63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 
70.0, 71.0]]], [[[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PCLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLLL(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[128.0], [131.0], [134.0]]], [[[173.0], [176.0], [179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3, 1),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]], [[[168.0, 169.0, 170.0], 
[171.0, 172.0, 173.0], [174.0, 175.0, 176.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RPUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]], [[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UUUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0]], [[15.0, 16.0, 17.0], [18.0, 
19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 
55.0, 56.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[90.0, 91.0, 92.0], [93.0, 
94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], 
[114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CLCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,2:,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[30.0, 31.0, 32.0], [33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], 
[[[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 89.0]]], [[[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[165.0, 166.0, 167.0], 
[168.0, 169.0, 170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RUCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], 
[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[90.0, 91.0, 92.0], 
[93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 
110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CPCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], 
[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[105.0, 106.0, 107.0], 
[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[126.0, 127.0, 128.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LLPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,2:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[126.0, 127.0, 128.0]], [[171.0, 172.0, 173.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[33.0, 34.0, 35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0]]], [[[78.0, 79.0, 80.0], [81.0, 82.0, 
83.0], [84.0, 85.0, 86.0]]], [[[123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LUCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 
104.0]], [[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], 
[[[135.0, 136.0, 137.0], [138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 
151.0, 152.0], [153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UCLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], 
[27.0, 28.0, 29.0]], [[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[51.0, 52.0, 53.0], [54.0, 55.0, 56.0], 
[57.0, 58.0, 59.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]], [[81.0, 82.0, 83.0], [84.0, 85.0, 86.0], 
[87.0, 88.0, 89.0]]], [[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LUPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[96.0, 97.0, 98.0], [111.0, 112.0, 113.0]], [[141.0, 142.0, 143.0], [156.0, 157.0, 158.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RLPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,2:,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[81.0, 82.0, 83.0]], [[126.0, 127.0, 128.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LCLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[111.0, 112.0, 113.0], [114.0, 
115.0, 116.0], [117.0, 118.0, 119.0]], [[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[141.0, 142.0, 
143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], 
[[171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[66.0, 67.0, 68.0], [69.0, 70.0, 71.0], 
[72.0, 73.0, 74.0]], [[111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0]]], [[[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_LPRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2:,1,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]], [[153.0, 154.0, 155.0], [156.0, 
157.0, 158.0], [159.0, 160.0, 161.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_UPPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,1,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[21.0, 22.0, 23.0], [66.0, 67.0, 68.0], [111.0, 112.0, 113.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_ULLN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:3,2:,2:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[81.0, 82.0, 83.0], [84.0, 85.0, 
86.0], [87.0, 88.0, 89.0]]], [[[126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_RRPN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[1:3,1:2,2]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[66.0, 67.0, 68.0]], [[111.0, 112.0, 113.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 1, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PPUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[105.0, 106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PRRN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,1:2,1:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 3, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_PLUN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[2,2:,:4]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[120.0, 121.0, 122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(1, 4, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_getslice_Symbol_rank4_CRCN(self):
      arg=Symbol(shape=(4, 3, 5, 3))
      res=arg[:,1:2,:]
      s=numpy.array([[[[0.0, 1.0, 2.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0], [9.0, 10.0, 11.0], [12.0, 13.0, 14.0]], [[15.0, 16.0, 
17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]], [[30.0, 31.0, 32.0], [33.0, 34.0, 
35.0], [36.0, 37.0, 38.0], [39.0, 40.0, 41.0], [42.0, 43.0, 44.0]]], [[[45.0, 46.0, 47.0], [48.0, 49.0, 50.0], [51.0, 52.0, 
53.0], [54.0, 55.0, 56.0], [57.0, 58.0, 59.0]], [[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 
71.0], [72.0, 73.0, 74.0]], [[75.0, 76.0, 77.0], [78.0, 79.0, 80.0], [81.0, 82.0, 83.0], [84.0, 85.0, 86.0], [87.0, 88.0, 
89.0]]], [[[90.0, 91.0, 92.0], [93.0, 94.0, 95.0], [96.0, 97.0, 98.0], [99.0, 100.0, 101.0], [102.0, 103.0, 104.0]], [[105.0, 
106.0, 107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]], [[120.0, 121.0, 
122.0], [123.0, 124.0, 125.0], [126.0, 127.0, 128.0], [129.0, 130.0, 131.0], [132.0, 133.0, 134.0]]], [[[135.0, 136.0, 137.0], 
[138.0, 139.0, 140.0], [141.0, 142.0, 143.0], [144.0, 145.0, 146.0], [147.0, 148.0, 149.0]], [[150.0, 151.0, 152.0], [153.0, 
154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]], [[165.0, 166.0, 167.0], [168.0, 169.0, 
170.0], [171.0, 172.0, 173.0], [174.0, 175.0, 176.0], [177.0, 178.0, 179.0]]]])
      sub=res.substitute({arg:s})
      ref=numpy.array([[[[15.0, 16.0, 17.0], [18.0, 19.0, 20.0], [21.0, 22.0, 23.0], [24.0, 25.0, 26.0], [27.0, 28.0, 29.0]]], 
[[[60.0, 61.0, 62.0], [63.0, 64.0, 65.0], [66.0, 67.0, 68.0], [69.0, 70.0, 71.0], [72.0, 73.0, 74.0]]], [[[105.0, 106.0, 
107.0], [108.0, 109.0, 110.0], [111.0, 112.0, 113.0], [114.0, 115.0, 116.0], [117.0, 118.0, 119.0]]], [[[150.0, 151.0, 152.0], 
[153.0, 154.0, 155.0], [156.0, 157.0, 158.0], [159.0, 160.0, 161.0], [162.0, 163.0, 164.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 1, 5, 3),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_array_L(self):
      arg=Data(numpy.array([6.0, 2.0, 7.0]),self.functionspace)
      val=numpy.array([7.0])
      arg[2:]=val
      ref=Data(numpy.array([6.0, 2.0, 7.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg=Data(numpy.array([6.0j, 2.0, 7.0]),self.functionspace)
      val=numpy.array([7.0])
      arg[2:]=val
      ref=Data(numpy.array([6.0j, 2.0, 7.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_constData_L(self):
      arg=Data(numpy.array([2.0, 8.0, 8.0]),self.functionspace)
      val=Data(numpy.array([7.0]),self.functionspace)
      arg[2:]=val
      ref=Data(numpy.array([2.0, 8.0, 7.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_expandedData_L(self):
      arg=Data(numpy.array([7.0, 2.0, 4.0]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([1.0])+(1.-msk_val)*numpy.array([3.0])
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.0, 2.0, 1.0])+(1.-msk_ref)*numpy.array([7.0, 2.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_array_P(self):
      arg=Data(numpy.array([7.0, 4.0, 2.0]),self.functionspace)
      val=numpy.array(1.0)
      arg[1]=val
      ref=Data(numpy.array([7.0, 1.0, 2.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_constData_P(self):
      arg=Data(numpy.array([8.0, 1.0, 6.0]),self.functionspace)
      val=Data(1.0,self.functionspace)
      arg[1]=val
      ref=Data(numpy.array([8.0, 1.0, 6.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_expandedData_P(self):
      arg=Data(numpy.array([5.0, 4.0, 2.0]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*(1.0)+(1.-msk_val)*(1.0)
      arg[1]=val
      ref=Data(numpy.array([5.0, 1.0, 2.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_array_C(self):
      arg=Data(numpy.array([2.0, 5.0, 2.0]),self.functionspace)
      val=numpy.array([3.0, 6.0, 8.0])
      arg[:]=val
      ref=Data(numpy.array([3.0, 6.0, 8.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_constData_C(self):
      arg=Data(numpy.array([8.0, 2.0, 5.0]),self.functionspace)
      val=Data(numpy.array([2.0, 1.0, 2.0]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([2.0, 1.0, 2.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg=Data(numpy.array([8.0j, 2.0, 5.0j]),self.functionspace)
      val=Data(numpy.array([2.0, 1.0, 2.0j]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([2.0, 1.0, 2.0j]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")      
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_expandedData_C(self):
      arg=Data(numpy.array([4.0, 2.0, 5.0]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([7.0, 7.0, 2.0])+(1.-msk_val)*numpy.array([1.0, 7.0, 3.0])
      arg[:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.0, 7.0, 2.0])+(1.-msk_ref)*numpy.array([1.0, 7.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_array_U(self):
      arg=Data(numpy.array([8.0, 4.0, 3.0]),self.functionspace)
      val=numpy.array([3.0, 2.0])
      arg[:2]=val
      ref=Data(numpy.array([3.0, 2.0, 3.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_constData_U(self):
      arg=Data(numpy.array([5.0, 5.0, 5.0]),self.functionspace)
      val=Data(numpy.array([7.0, 5.0]),self.functionspace)
      arg[:2]=val
      ref=Data(numpy.array([7.0, 5.0, 5.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_expandedData_U(self):
      arg=Data(numpy.array([4.0, 5.0, 4.0]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([4.0, 8.0])+(1.-msk_val)*numpy.array([1.0, 8.0])
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.0, 8.0, 4.0])+(1.-msk_ref)*numpy.array([1.0, 8.0, 4.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_array_R(self):
      arg=Data(numpy.array([7.0, 1.0, 3.0]),self.functionspace)
      val=numpy.array([8.0])
      arg[1:2]=val
      ref=Data(numpy.array([7.0, 8.0, 3.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_constData_R(self):
      arg=Data(numpy.array([7.0, 7.0, 7.0]),self.functionspace)
      val=Data(numpy.array([6.0]),self.functionspace)
      arg[1:2]=val
      ref=Data(numpy.array([7.0, 6.0, 7.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank1_expandedData_R(self):
      arg=Data(numpy.array([8.0, 3.0, 5.0]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([8.0])+(1.-msk_val)*numpy.array([7.0])
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([8.0, 8.0, 5.0])+(1.-msk_ref)*numpy.array([8.0, 7.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_PC(self):
      arg=Data(numpy.array([[5.0, 2.0, 5.0, 7.0], [3.0, 5.0, 2.0, 3.0], [5.0, 2.0, 5.0, 1.0]]),self.functionspace)
      val=numpy.array([4.0, 7.0, 5.0, 8.0])
      arg[1,:]=val
      ref=Data(numpy.array([[5.0, 2.0, 5.0, 7.0], [4.0, 7.0, 5.0, 8.0], [5.0, 2.0, 5.0, 1.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_PC(self):
      arg=Data(numpy.array([[4.0, 8.0, 8.0, 5.0], [5.0, 6.0, 7.0, 4.0], [7.0, 8.0, 6.0, 4.0]]),self.functionspace)
      val=Data(numpy.array([5.0, 7.0, 7.0, 8.0]),self.functionspace)
      arg[1,:]=val
      ref=Data(numpy.array([[4.0, 8.0, 8.0, 5.0], [5.0, 7.0, 7.0, 8.0], [7.0, 8.0, 6.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_PC(self):
      arg=Data(numpy.array([[5.0, 4.0, 1.0, 1.0], [5.0, 8.0, 8.0, 1.0], [3.0, 1.0, 6.0, 7.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([6.0, 6.0, 4.0, 1.0])+(1.-msk_val)*numpy.array([4.0, 6.0, 5.0, 1.0])
      arg[1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 4.0, 1.0, 1.0], [6.0, 6.0, 4.0, 1.0], [3.0, 1.0, 6.0, 
7.0]])+(1.-msk_ref)*numpy.array([[5.0, 4.0, 1.0, 1.0], [4.0, 6.0, 5.0, 1.0], [3.0, 1.0, 6.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_CR(self):
      arg=Data(numpy.array([[8.0, 2.0, 3.0, 6.0], [1.0, 8.0, 2.0, 5.0], [2.0, 7.0, 6.0, 4.0]]),self.functionspace)
      val=numpy.array([[1.0, 3.0], [2.0, 5.0], [2.0, 1.0]])
      arg[:,1:3]=val
      ref=Data(numpy.array([[8.0, 1.0, 3.0, 6.0], [1.0, 2.0, 5.0, 5.0], [2.0, 2.0, 1.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_CR(self):
      arg=Data(numpy.array([[7.0, 3.0, 3.0, 3.0], [3.0, 3.0, 1.0, 4.0], [7.0, 2.0, 4.0, 3.0]]),self.functionspace)
      val=Data(numpy.array([[2.0, 7.0], [1.0, 5.0], [1.0, 6.0]]),self.functionspace)
      arg[:,1:3]=val
      ref=Data(numpy.array([[7.0, 2.0, 7.0, 3.0], [3.0, 1.0, 5.0, 4.0], [7.0, 1.0, 6.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_CR(self):
      arg=Data(numpy.array([[3.0, 2.0, 5.0, 2.0], [8.0, 4.0, 5.0, 2.0], [7.0, 8.0, 1.0, 4.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 5.0], [3.0, 4.0], [8.0, 3.0]])+(1.-msk_val)*numpy.array([[1.0, 1.0], [3.0, 4.0], [8.0, 
1.0]])
      arg[:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 6.0, 5.0, 2.0], [8.0, 3.0, 4.0, 2.0], [7.0, 8.0, 3.0, 
4.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 1.0, 2.0], [8.0, 3.0, 4.0, 2.0], [7.0, 8.0, 1.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_PL(self):
      arg=Data(numpy.array([[4.0, 4.0, 6.0, 8.0], [5.0, 2.0, 5.0, 4.0], [7.0, 5.0, 6.0, 1.0]]),self.functionspace)
      val=numpy.array([5.0, 1.0])
      arg[1,2:]=val
      ref=Data(numpy.array([[4.0, 4.0, 6.0, 8.0], [5.0, 2.0, 5.0, 1.0], [7.0, 5.0, 6.0, 1.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_PL(self):
      arg=Data(numpy.array([[2.0, 6.0, 5.0, 4.0], [7.0, 7.0, 2.0, 3.0], [5.0, 4.0, 4.0, 2.0]]),self.functionspace)
      val=Data(numpy.array([1.0, 4.0]),self.functionspace)
      arg[1,2:]=val
      ref=Data(numpy.array([[2.0, 6.0, 5.0, 4.0], [7.0, 7.0, 1.0, 4.0], [5.0, 4.0, 4.0, 2.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_PL(self):
      arg=Data(numpy.array([[2.0, 7.0, 4.0, 7.0], [1.0, 2.0, 2.0, 3.0], [5.0, 4.0, 7.0, 5.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([7.0, 5.0])+(1.-msk_val)*numpy.array([6.0, 6.0])
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 7.0, 4.0, 7.0], [1.0, 2.0, 7.0, 5.0], [5.0, 4.0, 7.0, 
5.0]])+(1.-msk_ref)*numpy.array([[2.0, 7.0, 4.0, 7.0], [1.0, 2.0, 6.0, 6.0], [5.0, 4.0, 7.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_UN(self):
      arg=Data(numpy.array([[8.0, 3.0, 5.0, 5.0], [1.0, 6.0, 1.0, 4.0], [2.0, 7.0, 5.0, 6.0]]),self.functionspace)
      val=numpy.array([[2.0, 2.0, 7.0, 7.0], [1.0, 5.0, 7.0, 8.0]])
      arg[:2]=val
      ref=Data(numpy.array([[2.0, 2.0, 7.0, 7.0], [1.0, 5.0, 7.0, 8.0], [2.0, 7.0, 5.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_UN(self):
      arg=Data(numpy.array([[7.0, 1.0, 1.0, 3.0], [7.0, 2.0, 7.0, 8.0], [8.0, 3.0, 1.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([[3.0, 8.0, 6.0, 8.0], [2.0, 6.0, 7.0, 1.0]]),self.functionspace)
      arg[:2]=val
      ref=Data(numpy.array([[3.0, 8.0, 6.0, 8.0], [2.0, 6.0, 7.0, 1.0], [8.0, 3.0, 1.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_UN(self):
      arg=Data(numpy.array([[8.0, 7.0, 2.0, 7.0], [5.0, 7.0, 3.0, 2.0], [8.0, 5.0, 7.0, 3.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 8.0, 8.0, 7.0], [5.0, 6.0, 8.0, 4.0]])+(1.-msk_val)*numpy.array([[2.0, 4.0, 2.0, 4.0], 
[5.0, 1.0, 5.0, 1.0]])
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 8.0, 8.0, 7.0], [5.0, 6.0, 8.0, 4.0], [8.0, 5.0, 7.0, 
3.0]])+(1.-msk_ref)*numpy.array([[2.0, 4.0, 2.0, 4.0], [5.0, 1.0, 5.0, 1.0], [8.0, 5.0, 7.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_PP(self):
      arg=Data(numpy.array([[5.0, 4.0, 2.0, 8.0], [5.0, 4.0, 4.0, 4.0], [4.0, 3.0, 8.0, 3.0]]),self.functionspace)
      val=numpy.array(1.0)
      arg[1,2]=val
      ref=Data(numpy.array([[5.0, 4.0, 2.0, 8.0], [5.0, 4.0, 1.0, 4.0], [4.0, 3.0, 8.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_PP(self):
      arg=Data(numpy.array([[8.0, 8.0, 1.0, 7.0], [5.0, 8.0, 2.0, 5.0], [4.0, 8.0, 5.0, 4.0]]),self.functionspace)
      val=Data(1.0,self.functionspace)
      arg[1,2]=val
      ref=Data(numpy.array([[8.0, 8.0, 1.0, 7.0], [5.0, 8.0, 1.0, 5.0], [4.0, 8.0, 5.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_PP(self):
      arg=Data(numpy.array([[2.0, 6.0, 7.0, 4.0], [3.0, 8.0, 2.0, 6.0], [7.0, 2.0, 5.0, 8.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*(1.0)+(1.-msk_val)*(1.0)
      arg[1,2]=val
      ref=Data(numpy.array([[2.0, 6.0, 7.0, 4.0], [3.0, 8.0, 1.0, 6.0], [7.0, 2.0, 5.0, 8.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_RN(self):
      arg=Data(numpy.array([[8.0, 8.0, 6.0, 7.0], [8.0, 1.0, 7.0, 6.0], [2.0, 6.0, 5.0, 7.0]]),self.functionspace)
      val=numpy.array([[4.0, 4.0, 3.0, 8.0]])
      arg[1:2]=val
      ref=Data(numpy.array([[8.0, 8.0, 6.0, 7.0], [4.0, 4.0, 3.0, 8.0], [2.0, 6.0, 5.0, 7.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_RN(self):
      arg=Data(numpy.array([[8.0, 4.0, 6.0, 4.0], [8.0, 1.0, 1.0, 1.0], [3.0, 1.0, 3.0, 8.0]]),self.functionspace)
      val=Data(numpy.array([[5.0, 5.0, 4.0, 2.0]]),self.functionspace)
      arg[1:2]=val
      ref=Data(numpy.array([[8.0, 4.0, 6.0, 4.0], [5.0, 5.0, 4.0, 2.0], [3.0, 1.0, 3.0, 8.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_RN(self):
      arg=Data(numpy.array([[1.0, 6.0, 7.0, 4.0], [4.0, 5.0, 5.0, 6.0], [5.0, 6.0, 8.0, 3.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 4.0, 1.0, 7.0]])+(1.-msk_val)*numpy.array([[2.0, 2.0, 7.0, 1.0]])
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 6.0, 7.0, 4.0], [8.0, 4.0, 1.0, 7.0], [5.0, 6.0, 8.0, 
3.0]])+(1.-msk_ref)*numpy.array([[1.0, 6.0, 7.0, 4.0], [2.0, 2.0, 7.0, 1.0], [5.0, 6.0, 8.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_UC(self):
      arg=Data(numpy.array([[6.0, 2.0, 1.0, 1.0], [5.0, 8.0, 3.0, 4.0], [5.0, 4.0, 8.0, 5.0]]),self.functionspace)
      val=numpy.array([[1.0, 4.0, 8.0, 6.0], [2.0, 8.0, 7.0, 3.0]])
      arg[:2,:]=val
      ref=Data(numpy.array([[1.0, 4.0, 8.0, 6.0], [2.0, 8.0, 7.0, 3.0], [5.0, 4.0, 8.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_UC(self):
      arg=Data(numpy.array([[4.0, 1.0, 1.0, 7.0], [5.0, 7.0, 1.0, 8.0], [8.0, 1.0, 2.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([[4.0, 1.0, 1.0, 2.0], [5.0, 6.0, 8.0, 4.0]]),self.functionspace)
      arg[:2,:]=val
      ref=Data(numpy.array([[4.0, 1.0, 1.0, 2.0], [5.0, 6.0, 8.0, 4.0], [8.0, 1.0, 2.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_UC(self):
      arg=Data(numpy.array([[1.0, 1.0, 7.0, 3.0], [5.0, 7.0, 1.0, 7.0], [3.0, 8.0, 3.0, 4.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 6.0, 1.0, 2.0], [8.0, 5.0, 8.0, 4.0]])+(1.-msk_val)*numpy.array([[5.0, 5.0, 3.0, 4.0], 
[3.0, 3.0, 2.0, 5.0]])
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 6.0, 1.0, 2.0], [8.0, 5.0, 8.0, 4.0], [3.0, 8.0, 3.0, 
4.0]])+(1.-msk_ref)*numpy.array([[5.0, 5.0, 3.0, 4.0], [3.0, 3.0, 2.0, 5.0], [3.0, 8.0, 3.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_LC(self):
      arg=Data(numpy.array([[1.0, 5.0, 2.0, 2.0], [1.0, 1.0, 3.0, 7.0], [8.0, 7.0, 8.0, 2.0]]),self.functionspace)
      val=numpy.array([[5.0, 2.0, 5.0, 5.0]])
      arg[2:,:]=val
      ref=Data(numpy.array([[1.0, 5.0, 2.0, 2.0], [1.0, 1.0, 3.0, 7.0], [5.0, 2.0, 5.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_LC(self):
      arg=Data(numpy.array([[4.0, 4.0, 5.0, 5.0], [2.0, 8.0, 2.0, 8.0], [6.0, 5.0, 5.0, 7.0]]),self.functionspace)
      val=Data(numpy.array([[7.0, 1.0, 8.0, 5.0]]),self.functionspace)
      arg[2:,:]=val
      ref=Data(numpy.array([[4.0, 4.0, 5.0, 5.0], [2.0, 8.0, 2.0, 8.0], [7.0, 1.0, 8.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_LC(self):
      arg=Data(numpy.array([[8.0, 2.0, 5.0, 2.0], [6.0, 3.0, 1.0, 6.0], [4.0, 5.0, 2.0, 5.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 2.0, 7.0, 6.0]])+(1.-msk_val)*numpy.array([[5.0, 4.0, 4.0, 4.0]])
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 2.0, 5.0, 2.0], [6.0, 3.0, 1.0, 6.0], [1.0, 2.0, 7.0, 
6.0]])+(1.-msk_ref)*numpy.array([[8.0, 2.0, 5.0, 2.0], [6.0, 3.0, 1.0, 6.0], [5.0, 4.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_LL(self):
      arg=Data(numpy.array([[3.0, 3.0, 6.0, 5.0], [4.0, 1.0, 3.0, 4.0], [5.0, 7.0, 3.0, 6.0]]),self.functionspace)
      val=numpy.array([[4.0, 5.0]])
      arg[2:,2:]=val
      ref=Data(numpy.array([[3.0, 3.0, 6.0, 5.0], [4.0, 1.0, 3.0, 4.0], [5.0, 7.0, 4.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_LL(self):
      arg=Data(numpy.array([[7.0, 1.0, 5.0, 1.0], [3.0, 1.0, 5.0, 8.0], [1.0, 3.0, 3.0, 4.0]]),self.functionspace)
      val=Data(numpy.array([[2.0, 6.0]]),self.functionspace)
      arg[2:,2:]=val
      ref=Data(numpy.array([[7.0, 1.0, 5.0, 1.0], [3.0, 1.0, 5.0, 8.0], [1.0, 3.0, 2.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_LL(self):
      arg=Data(numpy.array([[1.0, 3.0, 1.0, 2.0], [5.0, 3.0, 2.0, 3.0], [3.0, 1.0, 1.0, 5.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 7.0]])+(1.-msk_val)*numpy.array([[4.0, 4.0]])
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 3.0, 1.0, 2.0], [5.0, 3.0, 2.0, 3.0], [3.0, 1.0, 7.0, 
7.0]])+(1.-msk_ref)*numpy.array([[1.0, 3.0, 1.0, 2.0], [5.0, 3.0, 2.0, 3.0], [3.0, 1.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_UL(self):
      arg=Data(numpy.array([[4.0, 2.0, 2.0, 2.0], [8.0, 5.0, 3.0, 8.0], [1.0, 3.0, 8.0, 5.0]]),self.functionspace)
      val=numpy.array([[2.0, 6.0], [8.0, 1.0]])
      arg[:2,2:]=val
      ref=Data(numpy.array([[4.0, 2.0, 2.0, 6.0], [8.0, 5.0, 8.0, 1.0], [1.0, 3.0, 8.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_UL(self):
      arg=Data(numpy.array([[3.0, 5.0, 2.0, 8.0], [3.0, 4.0, 8.0, 2.0], [8.0, 1.0, 2.0, 7.0]]),self.functionspace)
      val=Data(numpy.array([[6.0, 6.0], [8.0, 6.0]]),self.functionspace)
      arg[:2,2:]=val
      ref=Data(numpy.array([[3.0, 5.0, 6.0, 6.0], [3.0, 4.0, 8.0, 6.0], [8.0, 1.0, 2.0, 7.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_UL(self):
      arg=Data(numpy.array([[8.0, 1.0, 2.0, 7.0], [1.0, 5.0, 8.0, 6.0], [4.0, 7.0, 2.0, 2.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 2.0], [6.0, 2.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0], [6.0, 8.0]])
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 1.0, 8.0, 2.0], [1.0, 5.0, 6.0, 2.0], [4.0, 7.0, 2.0, 
2.0]])+(1.-msk_ref)*numpy.array([[8.0, 1.0, 3.0, 4.0], [1.0, 5.0, 6.0, 8.0], [4.0, 7.0, 2.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_CC(self):
      arg=Data(numpy.array([[6.0, 2.0, 7.0, 6.0], [6.0, 4.0, 8.0, 4.0], [3.0, 1.0, 6.0, 1.0]]),self.functionspace)
      val=numpy.array([[3.0, 7.0, 6.0, 1.0], [4.0, 3.0, 1.0, 5.0], [6.0, 6.0, 2.0, 3.0]])
      arg[:,:]=val
      ref=Data(numpy.array([[3.0, 7.0, 6.0, 1.0], [4.0, 3.0, 1.0, 5.0], [6.0, 6.0, 2.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_CC(self):
      arg=Data(numpy.array([[6.0, 7.0, 3.0, 1.0], [5.0, 2.0, 5.0, 5.0], [6.0, 6.0, 5.0, 8.0]]),self.functionspace)
      val=Data(numpy.array([[8.0, 5.0, 3.0, 8.0], [2.0, 7.0, 6.0, 3.0], [6.0, 8.0, 1.0, 3.0]]),self.functionspace)
      arg[:,:]=val
      ref=Data(numpy.array([[8.0, 5.0, 3.0, 8.0], [2.0, 7.0, 6.0, 3.0], [6.0, 8.0, 1.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_CC(self):
      arg=Data(numpy.array([[4.0, 6.0, 3.0, 1.0], [8.0, 2.0, 1.0, 3.0], [2.0, 4.0, 8.0, 6.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 6.0, 3.0, 7.0], [8.0, 1.0, 1.0, 4.0], [5.0, 8.0, 2.0, 
3.0]])+(1.-msk_val)*numpy.array([[3.0, 1.0, 7.0, 3.0], [2.0, 1.0, 6.0, 2.0], [4.0, 8.0, 6.0, 5.0]])
      arg[:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 6.0, 3.0, 7.0], [8.0, 1.0, 1.0, 4.0], [5.0, 8.0, 2.0, 
3.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 7.0, 3.0], [2.0, 1.0, 6.0, 2.0], [4.0, 8.0, 6.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_CN(self):
      arg=Data(numpy.array([[1.0, 3.0, 3.0, 8.0], [4.0, 7.0, 6.0, 7.0], [3.0, 3.0, 3.0, 1.0]]),self.functionspace)
      val=numpy.array([[5.0, 4.0, 2.0, 8.0], [3.0, 6.0, 4.0, 2.0], [7.0, 7.0, 2.0, 4.0]])
      arg[:]=val
      ref=Data(numpy.array([[5.0, 4.0, 2.0, 8.0], [3.0, 6.0, 4.0, 2.0], [7.0, 7.0, 2.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_CN(self):
      arg=Data(numpy.array([[5.0, 8.0, 6.0, 1.0], [6.0, 5.0, 2.0, 2.0], [5.0, 7.0, 5.0, 4.0]]),self.functionspace)
      val=Data(numpy.array([[6.0, 2.0, 8.0, 1.0], [1.0, 4.0, 3.0, 8.0], [1.0, 1.0, 2.0, 5.0]]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([[6.0, 2.0, 8.0, 1.0], [1.0, 4.0, 3.0, 8.0], [1.0, 1.0, 2.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_CN(self):
      arg=Data(numpy.array([[4.0, 5.0, 6.0, 2.0], [6.0, 8.0, 5.0, 1.0], [6.0, 3.0, 4.0, 6.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 2.0, 3.0, 6.0], [8.0, 7.0, 4.0, 7.0], [6.0, 8.0, 1.0, 
3.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0, 6.0, 2.0], [3.0, 4.0, 2.0, 1.0], [7.0, 8.0, 3.0, 1.0]])
      arg[:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 2.0, 3.0, 6.0], [8.0, 7.0, 4.0, 7.0], [6.0, 8.0, 1.0, 
3.0]])+(1.-msk_ref)*numpy.array([[3.0, 4.0, 6.0, 2.0], [3.0, 4.0, 2.0, 1.0], [7.0, 8.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_RP(self):
      arg=Data(numpy.array([[5.0, 8.0, 4.0, 7.0], [2.0, 3.0, 7.0, 3.0], [6.0, 7.0, 5.0, 7.0]]),self.functionspace)
      val=numpy.array([2.0])
      arg[1:2,2]=val
      ref=Data(numpy.array([[5.0, 8.0, 4.0, 7.0], [2.0, 3.0, 2.0, 3.0], [6.0, 7.0, 5.0, 7.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_RP(self):
      arg=Data(numpy.array([[4.0, 1.0, 2.0, 2.0], [6.0, 2.0, 2.0, 3.0], [3.0, 4.0, 7.0, 2.0]]),self.functionspace)
      val=Data(numpy.array([2.0]),self.functionspace)
      arg[1:2,2]=val
      ref=Data(numpy.array([[4.0, 1.0, 2.0, 2.0], [6.0, 2.0, 2.0, 3.0], [3.0, 4.0, 7.0, 2.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_RP(self):
      arg=Data(numpy.array([[5.0, 7.0, 4.0, 2.0], [7.0, 8.0, 5.0, 4.0], [4.0, 1.0, 5.0, 4.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([1.0])+(1.-msk_val)*numpy.array([5.0])
      arg[1:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 7.0, 4.0, 2.0], [7.0, 8.0, 1.0, 4.0], [4.0, 1.0, 5.0, 
4.0]])+(1.-msk_ref)*numpy.array([[5.0, 7.0, 4.0, 2.0], [7.0, 8.0, 5.0, 4.0], [4.0, 1.0, 5.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_PN(self):
      arg=Data(numpy.array([[4.0, 2.0, 6.0, 3.0], [8.0, 4.0, 4.0, 4.0], [4.0, 7.0, 5.0, 5.0]]),self.functionspace)
      val=numpy.array([4.0, 7.0, 1.0, 6.0])
      arg[1]=val
      ref=Data(numpy.array([[4.0, 2.0, 6.0, 3.0], [4.0, 7.0, 1.0, 6.0], [4.0, 7.0, 5.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_PN(self):
      arg=Data(numpy.array([[3.0, 4.0, 8.0, 3.0], [7.0, 2.0, 8.0, 1.0], [3.0, 7.0, 4.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([3.0, 5.0, 3.0, 3.0]),self.functionspace)
      arg[1]=val
      ref=Data(numpy.array([[3.0, 4.0, 8.0, 3.0], [3.0, 5.0, 3.0, 3.0], [3.0, 7.0, 4.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_PN(self):
      arg=Data(numpy.array([[3.0, 5.0, 4.0, 5.0], [5.0, 7.0, 2.0, 8.0], [3.0, 7.0, 2.0, 7.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([2.0, 8.0, 1.0, 6.0])+(1.-msk_val)*numpy.array([2.0, 4.0, 5.0, 3.0])
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 5.0, 4.0, 5.0], [2.0, 8.0, 1.0, 6.0], [3.0, 7.0, 2.0, 
7.0]])+(1.-msk_ref)*numpy.array([[3.0, 5.0, 4.0, 5.0], [2.0, 4.0, 5.0, 3.0], [3.0, 7.0, 2.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_RR(self):
      arg=Data(numpy.array([[4.0, 6.0, 8.0, 6.0], [5.0, 4.0, 7.0, 6.0], [1.0, 6.0, 5.0, 5.0]]),self.functionspace)
      val=numpy.array([[1.0, 6.0]])
      arg[1:2,1:3]=val
      ref=Data(numpy.array([[4.0, 6.0, 8.0, 6.0], [5.0, 1.0, 6.0, 6.0], [1.0, 6.0, 5.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_RR(self):
      arg=Data(numpy.array([[3.0, 8.0, 7.0, 3.0], [8.0, 5.0, 8.0, 4.0], [8.0, 2.0, 2.0, 3.0]]),self.functionspace)
      val=Data(numpy.array([[5.0, 1.0]]),self.functionspace)
      arg[1:2,1:3]=val
      ref=Data(numpy.array([[3.0, 8.0, 7.0, 3.0], [8.0, 5.0, 1.0, 4.0], [8.0, 2.0, 2.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_RR(self):
      arg=Data(numpy.array([[3.0, 5.0, 3.0, 4.0], [7.0, 1.0, 3.0, 5.0], [8.0, 3.0, 5.0, 2.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 2.0]])+(1.-msk_val)*numpy.array([[7.0, 7.0]])
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 5.0, 3.0, 4.0], [7.0, 4.0, 2.0, 5.0], [8.0, 3.0, 5.0, 
2.0]])+(1.-msk_ref)*numpy.array([[3.0, 5.0, 3.0, 4.0], [7.0, 7.0, 7.0, 5.0], [8.0, 3.0, 5.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_UP(self):
      arg=Data(numpy.array([[7.0, 7.0, 1.0, 5.0], [7.0, 6.0, 8.0, 5.0], [8.0, 8.0, 4.0, 4.0]]),self.functionspace)
      val=numpy.array([3.0, 7.0])
      arg[:2,2]=val
      ref=Data(numpy.array([[7.0, 7.0, 3.0, 5.0], [7.0, 6.0, 7.0, 5.0], [8.0, 8.0, 4.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_UP(self):
      arg=Data(numpy.array([[5.0, 6.0, 6.0, 3.0], [7.0, 7.0, 6.0, 6.0], [8.0, 2.0, 2.0, 1.0]]),self.functionspace)
      val=Data(numpy.array([2.0, 7.0]),self.functionspace)
      arg[:2,2]=val
      ref=Data(numpy.array([[5.0, 6.0, 2.0, 3.0], [7.0, 7.0, 7.0, 6.0], [8.0, 2.0, 2.0, 1.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_UP(self):
      arg=Data(numpy.array([[7.0, 6.0, 8.0, 1.0], [2.0, 5.0, 1.0, 1.0], [4.0, 5.0, 6.0, 8.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([7.0, 3.0])+(1.-msk_val)*numpy.array([6.0, 5.0])
      arg[:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 6.0, 7.0, 1.0], [2.0, 5.0, 3.0, 1.0], [4.0, 5.0, 6.0, 
8.0]])+(1.-msk_ref)*numpy.array([[7.0, 6.0, 6.0, 1.0], [2.0, 5.0, 5.0, 1.0], [4.0, 5.0, 6.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_UR(self):
      arg=Data(numpy.array([[5.0, 2.0, 3.0, 2.0], [4.0, 4.0, 6.0, 2.0], [4.0, 3.0, 7.0, 4.0]]),self.functionspace)
      val=numpy.array([[8.0, 4.0], [3.0, 3.0]])
      arg[:2,1:3]=val
      ref=Data(numpy.array([[5.0, 8.0, 4.0, 2.0], [4.0, 3.0, 3.0, 2.0], [4.0, 3.0, 7.0, 4.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_UR(self):
      arg=Data(numpy.array([[8.0, 8.0, 1.0, 2.0], [2.0, 7.0, 5.0, 6.0], [7.0, 4.0, 5.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([[8.0, 5.0], [2.0, 7.0]]),self.functionspace)
      arg[:2,1:3]=val
      ref=Data(numpy.array([[8.0, 8.0, 5.0, 2.0], [2.0, 2.0, 7.0, 6.0], [7.0, 4.0, 5.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_UR(self):
      arg=Data(numpy.array([[4.0, 3.0, 5.0, 8.0], [5.0, 8.0, 3.0, 5.0], [6.0, 8.0, 3.0, 1.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 5.0], [1.0, 2.0]])+(1.-msk_val)*numpy.array([[2.0, 3.0], [7.0, 4.0]])
      arg[:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 6.0, 5.0, 8.0], [5.0, 1.0, 2.0, 5.0], [6.0, 8.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[4.0, 2.0, 3.0, 8.0], [5.0, 7.0, 4.0, 5.0], [6.0, 8.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_LR(self):
      arg=Data(numpy.array([[7.0, 3.0, 3.0, 5.0], [3.0, 5.0, 3.0, 3.0], [8.0, 8.0, 1.0, 3.0]]),self.functionspace)
      val=numpy.array([[1.0, 6.0]])
      arg[2:,1:3]=val
      ref=Data(numpy.array([[7.0, 3.0, 3.0, 5.0], [3.0, 5.0, 3.0, 3.0], [8.0, 1.0, 6.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_LR(self):
      arg=Data(numpy.array([[3.0, 1.0, 3.0, 3.0], [5.0, 5.0, 3.0, 4.0], [6.0, 3.0, 5.0, 5.0]]),self.functionspace)
      val=Data(numpy.array([[4.0, 8.0]]),self.functionspace)
      arg[2:,1:3]=val
      ref=Data(numpy.array([[3.0, 1.0, 3.0, 3.0], [5.0, 5.0, 3.0, 4.0], [6.0, 4.0, 8.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_LR(self):
      arg=Data(numpy.array([[6.0, 2.0, 7.0, 6.0], [4.0, 8.0, 2.0, 1.0], [4.0, 1.0, 5.0, 1.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 8.0]])+(1.-msk_val)*numpy.array([[8.0, 1.0]])
      arg[2:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 2.0, 7.0, 6.0], [4.0, 8.0, 2.0, 1.0], [4.0, 2.0, 8.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 2.0, 7.0, 6.0], [4.0, 8.0, 2.0, 1.0], [4.0, 8.0, 1.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_LN(self):
      arg=Data(numpy.array([[8.0, 2.0, 5.0, 3.0], [1.0, 1.0, 5.0, 5.0], [7.0, 7.0, 5.0, 5.0]]),self.functionspace)
      val=numpy.array([[8.0, 7.0, 6.0, 1.0]])
      arg[2:]=val
      ref=Data(numpy.array([[8.0, 2.0, 5.0, 3.0], [1.0, 1.0, 5.0, 5.0], [8.0, 7.0, 6.0, 1.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_LN(self):
      arg=Data(numpy.array([[7.0, 1.0, 1.0, 2.0], [5.0, 1.0, 3.0, 2.0], [4.0, 7.0, 6.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([[5.0, 4.0, 4.0, 5.0]]),self.functionspace)
      arg[2:]=val
      ref=Data(numpy.array([[7.0, 1.0, 1.0, 2.0], [5.0, 1.0, 3.0, 2.0], [5.0, 4.0, 4.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_LN(self):
      arg=Data(numpy.array([[6.0, 4.0, 1.0, 1.0], [2.0, 4.0, 5.0, 4.0], [3.0, 2.0, 5.0, 5.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 4.0, 4.0, 7.0]])+(1.-msk_val)*numpy.array([[4.0, 5.0, 2.0, 7.0]])
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 4.0, 1.0, 1.0], [2.0, 4.0, 5.0, 4.0], [1.0, 4.0, 4.0, 
7.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 1.0, 1.0], [2.0, 4.0, 5.0, 4.0], [4.0, 5.0, 2.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_array_UU(self):
      arg=Data(numpy.array([[2.0, 6.0, 6.0, 2.0], [2.0, 5.0, 7.0, 4.0], [6.0, 4.0, 3.0, 2.0]]),self.functionspace)
      val=numpy.array([[2.0, 5.0, 1.0], [6.0, 3.0, 5.0]])
      arg[:2,:3]=val
      ref=Data(numpy.array([[2.0, 5.0, 1.0, 2.0], [6.0, 3.0, 5.0, 4.0], [6.0, 4.0, 3.0, 2.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_constData_UU(self):
      arg=Data(numpy.array([[8.0, 1.0, 3.0, 8.0], [2.0, 8.0, 2.0, 3.0], [8.0, 6.0, 8.0, 6.0]]),self.functionspace)
      val=Data(numpy.array([[3.0, 4.0, 5.0], [3.0, 4.0, 8.0]]),self.functionspace)
      arg[:2,:3]=val
      ref=Data(numpy.array([[3.0, 4.0, 5.0, 8.0], [3.0, 4.0, 8.0, 3.0], [8.0, 6.0, 8.0, 6.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank2_expandedData_UU(self):
      arg=Data(numpy.array([[5.0, 1.0, 5.0, 1.0], [8.0, 4.0, 5.0, 4.0], [5.0, 1.0, 3.0, 1.0]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 6.0, 8.0], [4.0, 5.0, 4.0]])+(1.-msk_val)*numpy.array([[2.0, 5.0, 4.0], [3.0, 5.0, 5.0]])
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 6.0, 8.0, 1.0], [4.0, 5.0, 4.0, 4.0], [5.0, 1.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[2.0, 5.0, 4.0, 1.0], [3.0, 5.0, 5.0, 4.0], [5.0, 1.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RLP(self):
      arg=Data(numpy.array([[[3.0, 1.0, 7.0], [5.0, 8.0, 2.0], [7.0, 4.0, 5.0], [6.0, 7.0, 4.0]], [[1.0, 6.0, 2.0], [5.0, 7.0, 
5.0], [8.0, 8.0, 2.0], [3.0, 1.0, 2.0]], [[5.0, 2.0, 4.0], [5.0, 3.0, 5.0], [8.0, 8.0, 5.0], [7.0, 7.0, 
8.0]]]),self.functionspace)
      val=numpy.array([[2.0, 6.0]])
      arg[1:2,2:,1]=val
      ref=Data(numpy.array([[[3.0, 1.0, 7.0], [5.0, 8.0, 2.0], [7.0, 4.0, 5.0], [6.0, 7.0, 4.0]], [[1.0, 6.0, 2.0], [5.0, 7.0, 
5.0], [8.0, 2.0, 2.0], [3.0, 6.0, 2.0]], [[5.0, 2.0, 4.0], [5.0, 3.0, 5.0], [8.0, 8.0, 5.0], [7.0, 7.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RLP(self):
      arg=Data(numpy.array([[[8.0, 3.0, 7.0], [8.0, 7.0, 8.0], [7.0, 1.0, 2.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 3.0, 
4.0], [1.0, 2.0, 3.0], [8.0, 1.0, 1.0]], [[3.0, 8.0, 1.0], [4.0, 3.0, 3.0], [5.0, 2.0, 3.0], [5.0, 4.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 1.0]]),self.functionspace)
      arg[1:2,2:,1]=val
      ref=Data(numpy.array([[[8.0, 3.0, 7.0], [8.0, 7.0, 8.0], [7.0, 1.0, 2.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 3.0, 
4.0], [1.0, 1.0, 3.0], [8.0, 1.0, 1.0]], [[3.0, 8.0, 1.0], [4.0, 3.0, 3.0], [5.0, 2.0, 3.0], [5.0, 4.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RLP(self):
      arg=Data(numpy.array([[[3.0, 1.0, 4.0], [7.0, 2.0, 8.0], [6.0, 1.0, 8.0], [8.0, 1.0, 2.0]], [[3.0, 7.0, 1.0], [3.0, 8.0, 
2.0], [6.0, 7.0, 5.0], [1.0, 1.0, 8.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 1.0], [1.0, 6.0, 8.0], [1.0, 4.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 4.0]])+(1.-msk_val)*numpy.array([[7.0, 3.0]])
      arg[1:2,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 4.0], [7.0, 2.0, 8.0], [6.0, 1.0, 8.0], [8.0, 1.0, 2.0]], [[3.0, 7.0, 1.0], [3.0, 
8.0, 2.0], [6.0, 2.0, 5.0], [1.0, 4.0, 8.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 1.0], [1.0, 6.0, 8.0], [1.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 4.0], [7.0, 2.0, 8.0], [6.0, 1.0, 8.0], [8.0, 1.0, 2.0]], [[3.0, 7.0, 1.0], [3.0, 
8.0, 2.0], [6.0, 7.0, 5.0], [1.0, 3.0, 8.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 1.0], [1.0, 6.0, 8.0], [1.0, 4.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_URC(self):
      arg=Data(numpy.array([[[2.0, 5.0, 2.0], [8.0, 4.0, 5.0], [3.0, 5.0, 5.0], [7.0, 1.0, 2.0]], [[1.0, 5.0, 7.0], [3.0, 6.0, 
6.0], [1.0, 5.0, 4.0], [5.0, 8.0, 8.0]], [[5.0, 6.0, 5.0], [8.0, 6.0, 3.0], [8.0, 5.0, 7.0], [6.0, 4.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[1.0, 6.0, 3.0], [5.0, 6.0, 2.0]], [[2.0, 4.0, 7.0], [4.0, 1.0, 6.0]]])
      arg[:2,1:3,:]=val
      ref=Data(numpy.array([[[2.0, 5.0, 2.0], [1.0, 6.0, 3.0], [5.0, 6.0, 2.0], [7.0, 1.0, 2.0]], [[1.0, 5.0, 7.0], [2.0, 4.0, 
7.0], [4.0, 1.0, 6.0], [5.0, 8.0, 8.0]], [[5.0, 6.0, 5.0], [8.0, 6.0, 3.0], [8.0, 5.0, 7.0], [6.0, 4.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_URC(self):
      arg=Data(numpy.array([[[4.0, 6.0, 7.0], [3.0, 7.0, 7.0], [5.0, 2.0, 6.0], [8.0, 3.0, 4.0]], [[2.0, 5.0, 8.0], [1.0, 6.0, 
2.0], [4.0, 4.0, 7.0], [4.0, 4.0, 8.0]], [[6.0, 3.0, 4.0], [6.0, 2.0, 7.0], [5.0, 5.0, 2.0], [4.0, 1.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 1.0], [7.0, 3.0, 7.0]], [[3.0, 3.0, 5.0], [4.0, 5.0, 8.0]]]),self.functionspace)
      arg[:2,1:3,:]=val
      ref=Data(numpy.array([[[4.0, 6.0, 7.0], [4.0, 7.0, 1.0], [7.0, 3.0, 7.0], [8.0, 3.0, 4.0]], [[2.0, 5.0, 8.0], [3.0, 3.0, 
5.0], [4.0, 5.0, 8.0], [4.0, 4.0, 8.0]], [[6.0, 3.0, 4.0], [6.0, 2.0, 7.0], [5.0, 5.0, 2.0], [4.0, 1.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_URC(self):
      arg=Data(numpy.array([[[5.0, 3.0, 2.0], [3.0, 3.0, 4.0], [2.0, 5.0, 8.0], [7.0, 2.0, 2.0]], [[7.0, 8.0, 4.0], [3.0, 2.0, 
8.0], [6.0, 7.0, 8.0], [8.0, 5.0, 3.0]], [[2.0, 8.0, 2.0], [7.0, 6.0, 3.0], [2.0, 2.0, 2.0], [4.0, 8.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 4.0, 5.0], [1.0, 3.0, 3.0]], [[7.0, 7.0, 2.0], [6.0, 7.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[8.0, 6.0, 7.0], [4.0, 5.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 2.0, 1.0]]])
      arg[:2,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 2.0], [1.0, 4.0, 5.0], [1.0, 3.0, 3.0], [7.0, 2.0, 2.0]], [[7.0, 8.0, 4.0], [7.0, 
7.0, 2.0], [6.0, 7.0, 6.0], [8.0, 5.0, 3.0]], [[2.0, 8.0, 2.0], [7.0, 6.0, 3.0], [2.0, 2.0, 2.0], [4.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 2.0], [8.0, 6.0, 7.0], [4.0, 5.0, 2.0], [7.0, 2.0, 2.0]], [[7.0, 8.0, 4.0], [2.0, 
4.0, 1.0], [3.0, 2.0, 1.0], [8.0, 5.0, 3.0]], [[2.0, 8.0, 2.0], [7.0, 6.0, 3.0], [2.0, 2.0, 2.0], [4.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PUP(self):
      arg=Data(numpy.array([[[7.0, 3.0, 6.0], [4.0, 8.0, 2.0], [4.0, 5.0, 5.0], [1.0, 4.0, 7.0]], [[2.0, 7.0, 1.0], [1.0, 1.0, 
6.0], [1.0, 7.0, 7.0], [4.0, 5.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 6.0, 7.0], [3.0, 5.0, 5.0], [1.0, 6.0, 
8.0]]]),self.functionspace)
      val=numpy.array([6.0, 5.0, 2.0])
      arg[1,:3,1]=val
      ref=Data(numpy.array([[[7.0, 3.0, 6.0], [4.0, 8.0, 2.0], [4.0, 5.0, 5.0], [1.0, 4.0, 7.0]], [[2.0, 6.0, 1.0], [1.0, 5.0, 
6.0], [1.0, 2.0, 7.0], [4.0, 5.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 6.0, 7.0], [3.0, 5.0, 5.0], [1.0, 6.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PUP(self):
      arg=Data(numpy.array([[[6.0, 4.0, 5.0], [5.0, 3.0, 2.0], [8.0, 5.0, 7.0], [4.0, 7.0, 2.0]], [[8.0, 6.0, 7.0], [5.0, 5.0, 
7.0], [2.0, 4.0, 3.0], [1.0, 5.0, 3.0]], [[2.0, 6.0, 8.0], [4.0, 8.0, 7.0], [5.0, 8.0, 2.0], [7.0, 7.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([6.0, 7.0, 8.0]),self.functionspace)
      arg[1,:3,1]=val
      ref=Data(numpy.array([[[6.0, 4.0, 5.0], [5.0, 3.0, 2.0], [8.0, 5.0, 7.0], [4.0, 7.0, 2.0]], [[8.0, 6.0, 7.0], [5.0, 7.0, 
7.0], [2.0, 8.0, 3.0], [1.0, 5.0, 3.0]], [[2.0, 6.0, 8.0], [4.0, 8.0, 7.0], [5.0, 8.0, 2.0], [7.0, 7.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PUP(self):
      arg=Data(numpy.array([[[2.0, 1.0, 1.0], [2.0, 5.0, 1.0], [7.0, 3.0, 3.0], [4.0, 4.0, 2.0]], [[8.0, 4.0, 3.0], [6.0, 7.0, 
3.0], [2.0, 5.0, 2.0], [7.0, 8.0, 5.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 5.0], [1.0, 2.0, 3.0], [8.0, 4.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([2.0, 3.0, 2.0])+(1.-msk_val)*numpy.array([6.0, 6.0, 1.0])
      arg[1,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 1.0], [2.0, 5.0, 1.0], [7.0, 3.0, 3.0], [4.0, 4.0, 2.0]], [[8.0, 2.0, 3.0], [6.0, 
3.0, 3.0], [2.0, 2.0, 2.0], [7.0, 8.0, 5.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 5.0], [1.0, 2.0, 3.0], [8.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 1.0], [2.0, 5.0, 1.0], [7.0, 3.0, 3.0], [4.0, 4.0, 2.0]], [[8.0, 6.0, 3.0], [6.0, 
6.0, 3.0], [2.0, 1.0, 2.0], [7.0, 8.0, 5.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 5.0], [1.0, 2.0, 3.0], [8.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RLR(self):
      arg=Data(numpy.array([[[6.0, 2.0, 5.0], [6.0, 6.0, 5.0], [6.0, 2.0, 4.0], [2.0, 8.0, 2.0]], [[7.0, 7.0, 8.0], [5.0, 3.0, 
7.0], [2.0, 8.0, 3.0], [8.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 6.0, 2.0], [7.0, 4.0, 2.0], [7.0, 6.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[5.0], [6.0]]])
      arg[1:2,2:,1:2]=val
      ref=Data(numpy.array([[[6.0, 2.0, 5.0], [6.0, 6.0, 5.0], [6.0, 2.0, 4.0], [2.0, 8.0, 2.0]], [[7.0, 7.0, 8.0], [5.0, 3.0, 
7.0], [2.0, 5.0, 3.0], [8.0, 6.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 6.0, 2.0], [7.0, 4.0, 2.0], [7.0, 6.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RLR(self):
      arg=Data(numpy.array([[[6.0, 2.0, 6.0], [4.0, 5.0, 7.0], [3.0, 1.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 7.0, 7.0], [1.0, 7.0, 
6.0], [1.0, 2.0, 7.0], [1.0, 7.0, 3.0]], [[4.0, 2.0, 7.0], [6.0, 6.0, 7.0], [4.0, 1.0, 6.0], [6.0, 1.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [6.0]]]),self.functionspace)
      arg[1:2,2:,1:2]=val
      ref=Data(numpy.array([[[6.0, 2.0, 6.0], [4.0, 5.0, 7.0], [3.0, 1.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 7.0, 7.0], [1.0, 7.0, 
6.0], [1.0, 4.0, 7.0], [1.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [6.0, 6.0, 7.0], [4.0, 1.0, 6.0], [6.0, 1.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RLR(self):
      arg=Data(numpy.array([[[3.0, 8.0, 7.0], [4.0, 6.0, 5.0], [2.0, 4.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 4.0, 5.0], [1.0, 8.0, 
4.0], [4.0, 7.0, 6.0], [8.0, 7.0, 1.0]], [[6.0, 7.0, 1.0], [2.0, 5.0, 4.0], [2.0, 5.0, 3.0], [1.0, 8.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0], [3.0]]])+(1.-msk_val)*numpy.array([[[7.0], [6.0]]])
      arg[1:2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 7.0], [4.0, 6.0, 5.0], [2.0, 4.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 4.0, 5.0], [1.0, 
8.0, 4.0], [4.0, 7.0, 6.0], [8.0, 3.0, 1.0]], [[6.0, 7.0, 1.0], [2.0, 5.0, 4.0], [2.0, 5.0, 3.0], [1.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 7.0], [4.0, 6.0, 5.0], [2.0, 4.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 4.0, 5.0], [1.0, 
8.0, 4.0], [4.0, 7.0, 6.0], [8.0, 6.0, 1.0]], [[6.0, 7.0, 1.0], [2.0, 5.0, 4.0], [2.0, 5.0, 3.0], [1.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PLN(self):
      arg=Data(numpy.array([[[5.0, 5.0, 4.0], [8.0, 8.0, 8.0], [4.0, 6.0, 5.0], [2.0, 3.0, 2.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 
1.0], [8.0, 6.0, 1.0], [2.0, 2.0, 2.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 2.0], [3.0, 7.0, 7.0], [4.0, 3.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[8.0, 7.0, 5.0], [6.0, 6.0, 3.0]])
      arg[1,2:]=val
      ref=Data(numpy.array([[[5.0, 5.0, 4.0], [8.0, 8.0, 8.0], [4.0, 6.0, 5.0], [2.0, 3.0, 2.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 
1.0], [8.0, 7.0, 5.0], [6.0, 6.0, 3.0]], [[1.0, 5.0, 3.0], [2.0, 2.0, 2.0], [3.0, 7.0, 7.0], [4.0, 3.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PLN(self):
      arg=Data(numpy.array([[[4.0, 6.0, 4.0], [7.0, 3.0, 3.0], [5.0, 8.0, 7.0], [6.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [4.0, 4.0, 
3.0], [8.0, 5.0, 1.0], [3.0, 3.0, 3.0]], [[3.0, 1.0, 8.0], [8.0, 4.0, 1.0], [1.0, 2.0, 7.0], [7.0, 7.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 2.0, 2.0], [2.0, 3.0, 2.0]]),self.functionspace)
      arg[1,2:]=val
      ref=Data(numpy.array([[[4.0, 6.0, 4.0], [7.0, 3.0, 3.0], [5.0, 8.0, 7.0], [6.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [4.0, 4.0, 
3.0], [8.0, 2.0, 2.0], [2.0, 3.0, 2.0]], [[3.0, 1.0, 8.0], [8.0, 4.0, 1.0], [1.0, 2.0, 7.0], [7.0, 7.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PLN(self):
      arg=Data(numpy.array([[[8.0, 4.0, 3.0], [2.0, 8.0, 3.0], [7.0, 5.0, 7.0], [5.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], [5.0, 8.0, 
1.0], [2.0, 3.0, 2.0], [6.0, 7.0, 3.0]], [[2.0, 1.0, 2.0], [1.0, 6.0, 6.0], [4.0, 7.0, 5.0], [5.0, 5.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 7.0, 1.0], [4.0, 7.0, 5.0]])+(1.-msk_val)*numpy.array([[5.0, 1.0, 3.0], [1.0, 4.0, 2.0]])
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 3.0], [2.0, 8.0, 3.0], [7.0, 5.0, 7.0], [5.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], [5.0, 
8.0, 1.0], [5.0, 7.0, 1.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 2.0], [1.0, 6.0, 6.0], [4.0, 7.0, 5.0], [5.0, 5.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 3.0], [2.0, 8.0, 3.0], [7.0, 5.0, 7.0], [5.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], [5.0, 
8.0, 1.0], [5.0, 1.0, 3.0], [1.0, 4.0, 2.0]], [[2.0, 1.0, 2.0], [1.0, 6.0, 6.0], [4.0, 7.0, 5.0], [5.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CLL(self):
      arg=Data(numpy.array([[[6.0, 7.0, 7.0], [8.0, 2.0, 2.0], [2.0, 8.0, 7.0], [1.0, 5.0, 4.0]], [[7.0, 6.0, 8.0], [7.0, 4.0, 
6.0], [2.0, 4.0, 5.0], [4.0, 6.0, 7.0]], [[5.0, 7.0, 6.0], [6.0, 1.0, 7.0], [1.0, 1.0, 5.0], [1.0, 6.0, 
6.0]]]),self.functionspace)
      val=numpy.array([[[5.0], [3.0]], [[6.0], [6.0]], [[5.0], [3.0]]])
      arg[:,2:,2:]=val
      ref=Data(numpy.array([[[6.0, 7.0, 7.0], [8.0, 2.0, 2.0], [2.0, 8.0, 5.0], [1.0, 5.0, 3.0]], [[7.0, 6.0, 8.0], [7.0, 4.0, 
6.0], [2.0, 4.0, 6.0], [4.0, 6.0, 6.0]], [[5.0, 7.0, 6.0], [6.0, 1.0, 7.0], [1.0, 1.0, 5.0], [1.0, 6.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CLL(self):
      arg=Data(numpy.array([[[1.0, 3.0, 5.0], [1.0, 4.0, 2.0], [6.0, 7.0, 8.0], [6.0, 4.0, 6.0]], [[3.0, 5.0, 1.0], [6.0, 7.0, 
6.0], [6.0, 2.0, 7.0], [2.0, 8.0, 3.0]], [[6.0, 1.0, 6.0], [4.0, 6.0, 3.0], [1.0, 1.0, 4.0], [7.0, 2.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [1.0]], [[6.0], [3.0]], [[1.0], [4.0]]]),self.functionspace)
      arg[:,2:,2:]=val
      ref=Data(numpy.array([[[1.0, 3.0, 5.0], [1.0, 4.0, 2.0], [6.0, 7.0, 4.0], [6.0, 4.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 7.0, 
6.0], [6.0, 2.0, 6.0], [2.0, 8.0, 3.0]], [[6.0, 1.0, 6.0], [4.0, 6.0, 3.0], [1.0, 1.0, 1.0], [7.0, 2.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CLL(self):
      arg=Data(numpy.array([[[1.0, 6.0, 8.0], [6.0, 1.0, 7.0], [1.0, 1.0, 2.0], [5.0, 3.0, 4.0]], [[8.0, 4.0, 2.0], [8.0, 1.0, 
2.0], [7.0, 2.0, 3.0], [2.0, 6.0, 4.0]], [[3.0, 8.0, 6.0], [1.0, 3.0, 1.0], [7.0, 5.0, 4.0], [4.0, 8.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0], [7.0]], [[1.0], [2.0]], [[8.0], [1.0]]])+(1.-msk_val)*numpy.array([[[5.0], [1.0]], 
[[2.0], [3.0]], [[4.0], [2.0]]])
      arg[:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 8.0], [6.0, 1.0, 7.0], [1.0, 1.0, 1.0], [5.0, 3.0, 7.0]], [[8.0, 4.0, 2.0], [8.0, 
1.0, 2.0], [7.0, 2.0, 1.0], [2.0, 6.0, 2.0]], [[3.0, 8.0, 6.0], [1.0, 3.0, 1.0], [7.0, 5.0, 8.0], [4.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 8.0], [6.0, 1.0, 7.0], [1.0, 1.0, 5.0], [5.0, 3.0, 1.0]], [[8.0, 4.0, 2.0], [8.0, 
1.0, 2.0], [7.0, 2.0, 2.0], [2.0, 6.0, 3.0]], [[3.0, 8.0, 6.0], [1.0, 3.0, 1.0], [7.0, 5.0, 4.0], [4.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RCR(self):
      arg=Data(numpy.array([[[3.0, 4.0, 5.0], [7.0, 3.0, 4.0], [6.0, 7.0, 6.0], [4.0, 2.0, 6.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 
1.0], [4.0, 2.0, 8.0], [6.0, 7.0, 4.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 1.0], [6.0, 1.0, 4.0], [1.0, 7.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[5.0], [5.0], [2.0], [6.0]]])
      arg[1:2,:,1:2]=val
      ref=Data(numpy.array([[[3.0, 4.0, 5.0], [7.0, 3.0, 4.0], [6.0, 7.0, 6.0], [4.0, 2.0, 6.0]], [[8.0, 5.0, 7.0], [3.0, 5.0, 
1.0], [4.0, 2.0, 8.0], [6.0, 6.0, 4.0]], [[2.0, 2.0, 1.0], [3.0, 7.0, 1.0], [6.0, 1.0, 4.0], [1.0, 7.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RCR(self):
      arg=Data(numpy.array([[[5.0, 2.0, 4.0], [8.0, 5.0, 1.0], [8.0, 2.0, 6.0], [4.0, 3.0, 1.0]], [[1.0, 6.0, 4.0], [8.0, 7.0, 
7.0], [2.0, 3.0, 7.0], [4.0, 3.0, 3.0]], [[4.0, 6.0, 2.0], [3.0, 4.0, 7.0], [1.0, 8.0, 8.0], [2.0, 3.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0], [5.0], [7.0], [4.0]]]),self.functionspace)
      arg[1:2,:,1:2]=val
      ref=Data(numpy.array([[[5.0, 2.0, 4.0], [8.0, 5.0, 1.0], [8.0, 2.0, 6.0], [4.0, 3.0, 1.0]], [[1.0, 3.0, 4.0], [8.0, 5.0, 
7.0], [2.0, 7.0, 7.0], [4.0, 4.0, 3.0]], [[4.0, 6.0, 2.0], [3.0, 4.0, 7.0], [1.0, 8.0, 8.0], [2.0, 3.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RCR(self):
      arg=Data(numpy.array([[[8.0, 3.0, 8.0], [2.0, 8.0, 6.0], [7.0, 4.0, 2.0], [2.0, 1.0, 3.0]], [[5.0, 2.0, 1.0], [1.0, 3.0, 
3.0], [3.0, 4.0, 8.0], [6.0, 7.0, 4.0]], [[1.0, 5.0, 1.0], [1.0, 7.0, 6.0], [1.0, 7.0, 8.0], [8.0, 7.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0], [7.0], [8.0], [1.0]]])+(1.-msk_val)*numpy.array([[[5.0], [3.0], [4.0], [2.0]]])
      arg[1:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 8.0], [2.0, 8.0, 6.0], [7.0, 4.0, 2.0], [2.0, 1.0, 3.0]], [[5.0, 3.0, 1.0], [1.0, 
7.0, 3.0], [3.0, 8.0, 8.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 1.0], [1.0, 7.0, 6.0], [1.0, 7.0, 8.0], [8.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 8.0], [2.0, 8.0, 6.0], [7.0, 4.0, 2.0], [2.0, 1.0, 3.0]], [[5.0, 5.0, 1.0], [1.0, 
3.0, 3.0], [3.0, 4.0, 8.0], [6.0, 2.0, 4.0]], [[1.0, 5.0, 1.0], [1.0, 7.0, 6.0], [1.0, 7.0, 8.0], [8.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LLL(self):
      arg=Data(numpy.array([[[8.0, 5.0, 4.0], [8.0, 3.0, 8.0], [4.0, 8.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 8.0, 4.0], [7.0, 2.0, 
1.0], [7.0, 6.0, 5.0], [8.0, 1.0, 3.0]], [[5.0, 4.0, 4.0], [2.0, 2.0, 2.0], [8.0, 5.0, 5.0], [1.0, 8.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[4.0], [6.0]]])
      arg[2:,2:,2:]=val
      ref=Data(numpy.array([[[8.0, 5.0, 4.0], [8.0, 3.0, 8.0], [4.0, 8.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 8.0, 4.0], [7.0, 2.0, 
1.0], [7.0, 6.0, 5.0], [8.0, 1.0, 3.0]], [[5.0, 4.0, 4.0], [2.0, 2.0, 2.0], [8.0, 5.0, 4.0], [1.0, 8.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LLL(self):
      arg=Data(numpy.array([[[5.0, 7.0, 4.0], [3.0, 5.0, 7.0], [5.0, 4.0, 5.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 7.0], [4.0, 3.0, 
4.0], [1.0, 6.0, 3.0], [1.0, 5.0, 2.0]], [[8.0, 2.0, 3.0], [5.0, 5.0, 7.0], [5.0, 3.0, 7.0], [8.0, 6.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0], [6.0]]]),self.functionspace)
      arg[2:,2:,2:]=val
      ref=Data(numpy.array([[[5.0, 7.0, 4.0], [3.0, 5.0, 7.0], [5.0, 4.0, 5.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 7.0], [4.0, 3.0, 
4.0], [1.0, 6.0, 3.0], [1.0, 5.0, 2.0]], [[8.0, 2.0, 3.0], [5.0, 5.0, 7.0], [5.0, 3.0, 3.0], [8.0, 6.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LLL(self):
      arg=Data(numpy.array([[[3.0, 8.0, 2.0], [1.0, 3.0, 1.0], [1.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 6.0, 
4.0], [5.0, 8.0, 2.0], [6.0, 7.0, 2.0]], [[6.0, 7.0, 8.0], [7.0, 2.0, 3.0], [5.0, 1.0, 7.0], [4.0, 2.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0], [2.0]]])+(1.-msk_val)*numpy.array([[[8.0], [5.0]]])
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 8.0, 2.0], [1.0, 3.0, 1.0], [1.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 
6.0, 4.0], [5.0, 8.0, 2.0], [6.0, 7.0, 2.0]], [[6.0, 7.0, 8.0], [7.0, 2.0, 3.0], [5.0, 1.0, 8.0], [4.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 2.0], [1.0, 3.0, 1.0], [1.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 
6.0, 4.0], [5.0, 8.0, 2.0], [6.0, 7.0, 2.0]], [[6.0, 7.0, 8.0], [7.0, 2.0, 3.0], [5.0, 1.0, 8.0], [4.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LPC(self):
      arg=Data(numpy.array([[[8.0, 3.0, 3.0], [8.0, 1.0, 2.0], [7.0, 5.0, 7.0], [5.0, 8.0, 5.0]], [[2.0, 1.0, 5.0], [2.0, 6.0, 
1.0], [8.0, 6.0, 5.0], [8.0, 6.0, 8.0]], [[8.0, 3.0, 1.0], [5.0, 3.0, 6.0], [3.0, 4.0, 6.0], [2.0, 7.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[3.0, 3.0, 1.0]])
      arg[2:,2,:]=val
      ref=Data(numpy.array([[[8.0, 3.0, 3.0], [8.0, 1.0, 2.0], [7.0, 5.0, 7.0], [5.0, 8.0, 5.0]], [[2.0, 1.0, 5.0], [2.0, 6.0, 
1.0], [8.0, 6.0, 5.0], [8.0, 6.0, 8.0]], [[8.0, 3.0, 1.0], [5.0, 3.0, 6.0], [3.0, 3.0, 1.0], [2.0, 7.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LPC(self):
      arg=Data(numpy.array([[[4.0, 1.0, 5.0], [2.0, 5.0, 5.0], [1.0, 3.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 1.0, 7.0], [1.0, 7.0, 
1.0], [3.0, 7.0, 3.0], [3.0, 1.0, 8.0]], [[4.0, 3.0, 8.0], [7.0, 8.0, 8.0], [4.0, 8.0, 2.0], [4.0, 1.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 6.0, 6.0]]),self.functionspace)
      arg[2:,2,:]=val
      ref=Data(numpy.array([[[4.0, 1.0, 5.0], [2.0, 5.0, 5.0], [1.0, 3.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 1.0, 7.0], [1.0, 7.0, 
1.0], [3.0, 7.0, 3.0], [3.0, 1.0, 8.0]], [[4.0, 3.0, 8.0], [7.0, 8.0, 8.0], [2.0, 6.0, 6.0], [4.0, 1.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LPC(self):
      arg=Data(numpy.array([[[4.0, 4.0, 1.0], [7.0, 4.0, 4.0], [8.0, 5.0, 4.0], [2.0, 2.0, 8.0]], [[8.0, 5.0, 5.0], [6.0, 6.0, 
1.0], [2.0, 7.0, 8.0], [4.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 3.0], [3.0, 5.0, 3.0], [5.0, 8.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 2.0, 5.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0, 7.0]])
      arg[2:,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 1.0], [7.0, 4.0, 4.0], [8.0, 5.0, 4.0], [2.0, 2.0, 8.0]], [[8.0, 5.0, 5.0], [6.0, 
6.0, 1.0], [2.0, 7.0, 8.0], [4.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 3.0], [4.0, 2.0, 5.0], [5.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 1.0], [7.0, 4.0, 4.0], [8.0, 5.0, 4.0], [2.0, 2.0, 8.0]], [[8.0, 5.0, 5.0], [6.0, 
6.0, 1.0], [2.0, 7.0, 8.0], [4.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 3.0], [3.0, 4.0, 7.0], [5.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_URU(self):
      arg=Data(numpy.array([[[1.0, 4.0, 3.0], [2.0, 7.0, 6.0], [2.0, 8.0, 5.0], [6.0, 6.0, 5.0]], [[4.0, 6.0, 1.0], [7.0, 2.0, 
5.0], [1.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[7.0, 1.0, 7.0], [4.0, 4.0, 6.0], [3.0, 7.0, 1.0], [2.0, 4.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[7.0, 4.0], [5.0, 5.0]], [[3.0, 2.0], [7.0, 4.0]]])
      arg[:2,1:3,:2]=val
      ref=Data(numpy.array([[[1.0, 4.0, 3.0], [7.0, 4.0, 6.0], [5.0, 5.0, 5.0], [6.0, 6.0, 5.0]], [[4.0, 6.0, 1.0], [3.0, 2.0, 
5.0], [7.0, 4.0, 3.0], [4.0, 4.0, 3.0]], [[7.0, 1.0, 7.0], [4.0, 4.0, 6.0], [3.0, 7.0, 1.0], [2.0, 4.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_URU(self):
      arg=Data(numpy.array([[[3.0, 5.0, 6.0], [8.0, 8.0, 5.0], [7.0, 2.0, 7.0], [8.0, 4.0, 4.0]], [[5.0, 3.0, 1.0], [8.0, 2.0, 
2.0], [1.0, 5.0, 6.0], [6.0, 6.0, 2.0]], [[2.0, 7.0, 4.0], [4.0, 4.0, 7.0], [3.0, 5.0, 3.0], [7.0, 6.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 4.0], [8.0, 4.0]], [[8.0, 3.0], [8.0, 4.0]]]),self.functionspace)
      arg[:2,1:3,:2]=val
      ref=Data(numpy.array([[[3.0, 5.0, 6.0], [7.0, 4.0, 5.0], [8.0, 4.0, 7.0], [8.0, 4.0, 4.0]], [[5.0, 3.0, 1.0], [8.0, 3.0, 
2.0], [8.0, 4.0, 6.0], [6.0, 6.0, 2.0]], [[2.0, 7.0, 4.0], [4.0, 4.0, 7.0], [3.0, 5.0, 3.0], [7.0, 6.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_URU(self):
      arg=Data(numpy.array([[[2.0, 3.0, 7.0], [2.0, 4.0, 3.0], [6.0, 5.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 3.0, 1.0], [1.0, 1.0, 
5.0], [6.0, 4.0, 4.0], [8.0, 8.0, 3.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 5.0], [4.0, 4.0, 3.0], [7.0, 2.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 8.0], [3.0, 8.0]], [[8.0, 4.0], [7.0, 6.0]]])+(1.-msk_val)*numpy.array([[[1.0, 5.0], 
[6.0, 3.0]], [[8.0, 5.0], [4.0, 2.0]]])
      arg[:2,1:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 7.0], [1.0, 8.0, 3.0], [3.0, 8.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 3.0, 1.0], [8.0, 
4.0, 5.0], [7.0, 6.0, 4.0], [8.0, 8.0, 3.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 5.0], [4.0, 4.0, 3.0], [7.0, 2.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 7.0], [1.0, 5.0, 3.0], [6.0, 3.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 3.0, 1.0], [8.0, 
5.0, 5.0], [4.0, 2.0, 4.0], [8.0, 8.0, 3.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 5.0], [4.0, 4.0, 3.0], [7.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CUN(self):
      arg=Data(numpy.array([[[6.0, 5.0, 2.0], [2.0, 4.0, 4.0], [4.0, 5.0, 5.0], [7.0, 6.0, 5.0]], [[6.0, 6.0, 8.0], [1.0, 8.0, 
6.0], [5.0, 1.0, 6.0], [4.0, 6.0, 7.0]], [[2.0, 3.0, 2.0], [7.0, 7.0, 8.0], [6.0, 1.0, 6.0], [7.0, 1.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[5.0, 8.0, 3.0], [2.0, 4.0, 3.0], [6.0, 3.0, 2.0]], [[7.0, 2.0, 7.0], [7.0, 1.0, 2.0], [4.0, 8.0, 
7.0]], [[2.0, 3.0, 5.0], [5.0, 3.0, 7.0], [1.0, 6.0, 2.0]]])
      arg[:,:3]=val
      ref=Data(numpy.array([[[5.0, 8.0, 3.0], [2.0, 4.0, 3.0], [6.0, 3.0, 2.0], [7.0, 6.0, 5.0]], [[7.0, 2.0, 7.0], [7.0, 1.0, 
2.0], [4.0, 8.0, 7.0], [4.0, 6.0, 7.0]], [[2.0, 3.0, 5.0], [5.0, 3.0, 7.0], [1.0, 6.0, 2.0], [7.0, 1.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CUN(self):
      arg=Data(numpy.array([[[8.0, 2.0, 3.0], [3.0, 5.0, 5.0], [4.0, 7.0, 1.0], [4.0, 2.0, 3.0]], [[8.0, 3.0, 4.0], [3.0, 6.0, 
8.0], [1.0, 8.0, 4.0], [1.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [8.0, 8.0, 3.0], [1.0, 7.0, 5.0], [4.0, 7.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 8.0, 8.0], [4.0, 4.0, 3.0], [7.0, 6.0, 7.0]], [[8.0, 8.0, 1.0], [5.0, 6.0, 1.0], [1.0, 4.0, 
4.0]], [[8.0, 7.0, 4.0], [7.0, 3.0, 4.0], [3.0, 7.0, 5.0]]]),self.functionspace)
      arg[:,:3]=val
      ref=Data(numpy.array([[[7.0, 8.0, 8.0], [4.0, 4.0, 3.0], [7.0, 6.0, 7.0], [4.0, 2.0, 3.0]], [[8.0, 8.0, 1.0], [5.0, 6.0, 
1.0], [1.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[8.0, 7.0, 4.0], [7.0, 3.0, 4.0], [3.0, 7.0, 5.0], [4.0, 7.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CUN(self):
      arg=Data(numpy.array([[[3.0, 3.0, 3.0], [2.0, 7.0, 3.0], [1.0, 5.0, 6.0], [4.0, 7.0, 1.0]], [[4.0, 4.0, 1.0], [1.0, 5.0, 
4.0], [3.0, 1.0, 4.0], [2.0, 5.0, 4.0]], [[6.0, 1.0, 1.0], [3.0, 7.0, 1.0], [4.0, 5.0, 1.0], [7.0, 8.0, 
7.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 6.0, 8.0], [3.0, 5.0, 6.0], [2.0, 5.0, 6.0]], [[1.0, 2.0, 5.0], [2.0, 5.0, 3.0], [2.0, 
2.0, 1.0]], [[8.0, 4.0, 1.0], [2.0, 8.0, 8.0], [2.0, 3.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 6.0, 8.0], [5.0, 4.0, 7.0], 
[7.0, 4.0, 5.0]], [[7.0, 1.0, 2.0], [6.0, 6.0, 3.0], [7.0, 2.0, 6.0]], [[8.0, 7.0, 2.0], [7.0, 3.0, 3.0], [4.0, 2.0, 8.0]]])
      arg[:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 8.0], [3.0, 5.0, 6.0], [2.0, 5.0, 6.0], [4.0, 7.0, 1.0]], [[1.0, 2.0, 5.0], [2.0, 
5.0, 3.0], [2.0, 2.0, 1.0], [2.0, 5.0, 4.0]], [[8.0, 4.0, 1.0], [2.0, 8.0, 8.0], [2.0, 3.0, 3.0], [7.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 8.0], [5.0, 4.0, 7.0], [7.0, 4.0, 5.0], [4.0, 7.0, 1.0]], [[7.0, 1.0, 2.0], [6.0, 
6.0, 3.0], [7.0, 2.0, 6.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 2.0], [7.0, 3.0, 3.0], [4.0, 2.0, 8.0], [7.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PPC(self):
      arg=Data(numpy.array([[[2.0, 7.0, 1.0], [5.0, 1.0, 6.0], [1.0, 3.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 4.0, 7.0], [6.0, 5.0, 
3.0], [1.0, 6.0, 2.0], [3.0, 2.0, 4.0]], [[1.0, 3.0, 7.0], [3.0, 4.0, 3.0], [1.0, 4.0, 7.0], [5.0, 3.0, 
2.0]]]),self.functionspace)
      val=numpy.array([6.0, 6.0, 3.0])
      arg[1,2,:]=val
      ref=Data(numpy.array([[[2.0, 7.0, 1.0], [5.0, 1.0, 6.0], [1.0, 3.0, 2.0], [6.0, 7.0, 3.0]], [[3.0, 4.0, 7.0], [6.0, 5.0, 
3.0], [6.0, 6.0, 3.0], [3.0, 2.0, 4.0]], [[1.0, 3.0, 7.0], [3.0, 4.0, 3.0], [1.0, 4.0, 7.0], [5.0, 3.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PPC(self):
      arg=Data(numpy.array([[[5.0, 1.0, 5.0], [3.0, 4.0, 4.0], [6.0, 8.0, 8.0], [1.0, 4.0, 8.0]], [[6.0, 3.0, 4.0], [8.0, 3.0, 
2.0], [3.0, 1.0, 1.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 3.0], [3.0, 7.0, 8.0], [3.0, 1.0, 7.0], [3.0, 1.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([3.0, 7.0, 1.0]),self.functionspace)
      arg[1,2,:]=val
      ref=Data(numpy.array([[[5.0, 1.0, 5.0], [3.0, 4.0, 4.0], [6.0, 8.0, 8.0], [1.0, 4.0, 8.0]], [[6.0, 3.0, 4.0], [8.0, 3.0, 
2.0], [3.0, 7.0, 1.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 3.0], [3.0, 7.0, 8.0], [3.0, 1.0, 7.0], [3.0, 1.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PPC(self):
      arg=Data(numpy.array([[[5.0, 8.0, 8.0], [6.0, 8.0, 5.0], [4.0, 5.0, 5.0], [2.0, 8.0, 7.0]], [[8.0, 8.0, 5.0], [1.0, 1.0, 
1.0], [7.0, 4.0, 7.0], [6.0, 1.0, 3.0]], [[1.0, 4.0, 1.0], [2.0, 1.0, 7.0], [5.0, 7.0, 3.0], [1.0, 7.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([2.0, 5.0, 8.0])+(1.-msk_val)*numpy.array([6.0, 1.0, 1.0])
      arg[1,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 8.0], [6.0, 8.0, 5.0], [4.0, 5.0, 5.0], [2.0, 8.0, 7.0]], [[8.0, 8.0, 5.0], [1.0, 
1.0, 1.0], [2.0, 5.0, 8.0], [6.0, 1.0, 3.0]], [[1.0, 4.0, 1.0], [2.0, 1.0, 7.0], [5.0, 7.0, 3.0], [1.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 8.0], [6.0, 8.0, 5.0], [4.0, 5.0, 5.0], [2.0, 8.0, 7.0]], [[8.0, 8.0, 5.0], [1.0, 
1.0, 1.0], [6.0, 1.0, 1.0], [6.0, 1.0, 3.0]], [[1.0, 4.0, 1.0], [2.0, 1.0, 7.0], [5.0, 7.0, 3.0], [1.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CCU(self):
      arg=Data(numpy.array([[[2.0, 5.0, 5.0], [1.0, 8.0, 5.0], [2.0, 8.0, 3.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 1.0], [8.0, 1.0, 
6.0], [7.0, 1.0, 3.0], [1.0, 3.0, 2.0]], [[7.0, 2.0, 6.0], [7.0, 7.0, 3.0], [7.0, 8.0, 6.0], [1.0, 2.0, 
6.0]]]),self.functionspace)
      val=numpy.array([[[2.0, 2.0], [5.0, 5.0], [2.0, 8.0], [5.0, 5.0]], [[1.0, 6.0], [4.0, 5.0], [4.0, 5.0], [6.0, 8.0]], 
[[8.0, 2.0], [1.0, 4.0], [7.0, 4.0], [1.0, 2.0]]])
      arg[:,:,:2]=val
      ref=Data(numpy.array([[[2.0, 2.0, 5.0], [5.0, 5.0, 5.0], [2.0, 8.0, 3.0], [5.0, 5.0, 6.0]], [[1.0, 6.0, 1.0], [4.0, 5.0, 
6.0], [4.0, 5.0, 3.0], [6.0, 8.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 4.0, 3.0], [7.0, 4.0, 6.0], [1.0, 2.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CCU(self):
      arg=Data(numpy.array([[[1.0, 7.0, 1.0], [2.0, 1.0, 7.0], [4.0, 6.0, 7.0], [1.0, 1.0, 6.0]], [[4.0, 6.0, 2.0], [2.0, 7.0, 
5.0], [8.0, 5.0, 4.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 5.0], [5.0, 2.0, 3.0], [1.0, 4.0, 5.0], [6.0, 6.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 8.0], [8.0, 7.0], [1.0, 5.0], [6.0, 4.0]], [[5.0, 3.0], [8.0, 7.0], [1.0, 2.0], [1.0, 2.0]], 
[[7.0, 7.0], [7.0, 8.0], [7.0, 3.0], [4.0, 1.0]]]),self.functionspace)
      arg[:,:,:2]=val
      ref=Data(numpy.array([[[8.0, 8.0, 1.0], [8.0, 7.0, 7.0], [1.0, 5.0, 7.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 2.0], [8.0, 7.0, 
5.0], [1.0, 2.0, 4.0], [1.0, 2.0, 5.0]], [[7.0, 7.0, 5.0], [7.0, 8.0, 3.0], [7.0, 3.0, 5.0], [4.0, 1.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CCU(self):
      arg=Data(numpy.array([[[2.0, 2.0, 8.0], [5.0, 4.0, 3.0], [2.0, 3.0, 4.0], [4.0, 2.0, 3.0]], [[3.0, 7.0, 2.0], [8.0, 2.0, 
1.0], [7.0, 3.0, 5.0], [3.0, 8.0, 3.0]], [[7.0, 8.0, 5.0], [7.0, 5.0, 6.0], [5.0, 8.0, 5.0], [5.0, 4.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 6.0], [2.0, 7.0], [5.0, 6.0], [8.0, 1.0]], [[1.0, 1.0], [7.0, 5.0], [8.0, 6.0], [2.0, 
6.0]], [[2.0, 7.0], [7.0, 6.0], [2.0, 8.0], [6.0, 1.0]]])+(1.-msk_val)*numpy.array([[[3.0, 3.0], [2.0, 1.0], [3.0, 7.0], [5.0, 
7.0]], [[5.0, 6.0], [7.0, 7.0], [3.0, 2.0], [6.0, 3.0]], [[7.0, 6.0], [8.0, 6.0], [5.0, 6.0], [5.0, 4.0]]])
      arg[:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 8.0], [2.0, 7.0, 3.0], [5.0, 6.0, 4.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 2.0], [7.0, 
5.0, 1.0], [8.0, 6.0, 5.0], [2.0, 6.0, 3.0]], [[2.0, 7.0, 5.0], [7.0, 6.0, 6.0], [2.0, 8.0, 5.0], [6.0, 1.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 8.0], [2.0, 1.0, 3.0], [3.0, 7.0, 4.0], [5.0, 7.0, 3.0]], [[5.0, 6.0, 2.0], [7.0, 
7.0, 1.0], [3.0, 2.0, 5.0], [6.0, 3.0, 3.0]], [[7.0, 6.0, 5.0], [8.0, 6.0, 6.0], [5.0, 6.0, 5.0], [5.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LLU(self):
      arg=Data(numpy.array([[[6.0, 8.0, 5.0], [3.0, 4.0, 2.0], [8.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[4.0, 2.0, 1.0], [2.0, 7.0, 
5.0], [6.0, 2.0, 8.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 2.0], [7.0, 1.0, 3.0], [8.0, 7.0, 4.0], [1.0, 5.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[4.0, 6.0], [7.0, 5.0]]])
      arg[2:,2:,:2]=val
      ref=Data(numpy.array([[[6.0, 8.0, 5.0], [3.0, 4.0, 2.0], [8.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[4.0, 2.0, 1.0], [2.0, 7.0, 
5.0], [6.0, 2.0, 8.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 2.0], [7.0, 1.0, 3.0], [4.0, 6.0, 4.0], [7.0, 5.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LLU(self):
      arg=Data(numpy.array([[[8.0, 3.0, 3.0], [2.0, 5.0, 4.0], [1.0, 4.0, 2.0], [6.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [6.0, 5.0, 
3.0], [8.0, 7.0, 8.0], [7.0, 4.0, 8.0]], [[3.0, 7.0, 4.0], [6.0, 7.0, 5.0], [8.0, 4.0, 7.0], [3.0, 1.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 4.0], [6.0, 1.0]]]),self.functionspace)
      arg[2:,2:,:2]=val
      ref=Data(numpy.array([[[8.0, 3.0, 3.0], [2.0, 5.0, 4.0], [1.0, 4.0, 2.0], [6.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [6.0, 5.0, 
3.0], [8.0, 7.0, 8.0], [7.0, 4.0, 8.0]], [[3.0, 7.0, 4.0], [6.0, 7.0, 5.0], [3.0, 4.0, 7.0], [6.0, 1.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LLU(self):
      arg=Data(numpy.array([[[8.0, 2.0, 2.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 4.0], [3.0, 8.0, 
1.0], [6.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[7.0, 2.0, 8.0], [7.0, 6.0, 6.0], [3.0, 2.0, 3.0], [8.0, 3.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 5.0], [4.0, 1.0]]])+(1.-msk_val)*numpy.array([[[1.0, 8.0], [4.0, 8.0]]])
      arg[2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 2.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 4.0], [3.0, 
8.0, 1.0], [6.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[7.0, 2.0, 8.0], [7.0, 6.0, 6.0], [3.0, 5.0, 3.0], [4.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 2.0], [4.0, 1.0, 6.0], [2.0, 2.0, 2.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 4.0], [3.0, 
8.0, 1.0], [6.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[7.0, 2.0, 8.0], [7.0, 6.0, 6.0], [1.0, 8.0, 3.0], [4.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LUU(self):
      arg=Data(numpy.array([[[4.0, 7.0, 6.0], [8.0, 2.0, 5.0], [7.0, 6.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 
3.0], [3.0, 4.0, 2.0], [4.0, 2.0, 6.0]], [[4.0, 8.0, 6.0], [6.0, 4.0, 1.0], [5.0, 8.0, 2.0], [7.0, 1.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[6.0, 7.0], [8.0, 6.0], [7.0, 8.0]]])
      arg[2:,:3,:2]=val
      ref=Data(numpy.array([[[4.0, 7.0, 6.0], [8.0, 2.0, 5.0], [7.0, 6.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 
3.0], [3.0, 4.0, 2.0], [4.0, 2.0, 6.0]], [[6.0, 7.0, 6.0], [8.0, 6.0, 1.0], [7.0, 8.0, 2.0], [7.0, 1.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LUU(self):
      arg=Data(numpy.array([[[3.0, 2.0, 8.0], [7.0, 2.0, 8.0], [2.0, 5.0, 2.0], [7.0, 4.0, 1.0]], [[4.0, 8.0, 8.0], [6.0, 6.0, 
2.0], [7.0, 1.0, 2.0], [1.0, 1.0, 3.0]], [[7.0, 5.0, 7.0], [1.0, 4.0, 8.0], [3.0, 3.0, 4.0], [4.0, 3.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 7.0], [3.0, 5.0], [5.0, 6.0]]]),self.functionspace)
      arg[2:,:3,:2]=val
      ref=Data(numpy.array([[[3.0, 2.0, 8.0], [7.0, 2.0, 8.0], [2.0, 5.0, 2.0], [7.0, 4.0, 1.0]], [[4.0, 8.0, 8.0], [6.0, 6.0, 
2.0], [7.0, 1.0, 2.0], [1.0, 1.0, 3.0]], [[2.0, 7.0, 7.0], [3.0, 5.0, 8.0], [5.0, 6.0, 4.0], [4.0, 3.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LUU(self):
      arg=Data(numpy.array([[[5.0, 1.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 1.0], [2.0, 4.0, 2.0]], [[6.0, 6.0, 1.0], [6.0, 2.0, 
2.0], [6.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[6.0, 4.0, 6.0], [5.0, 8.0, 3.0], [2.0, 1.0, 4.0], [5.0, 6.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 1.0], [5.0, 7.0], [6.0, 3.0]]])+(1.-msk_val)*numpy.array([[[7.0, 5.0], [4.0, 5.0], [8.0, 
4.0]]])
      arg[2:,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 1.0], [2.0, 4.0, 2.0]], [[6.0, 6.0, 1.0], [6.0, 
2.0, 2.0], [6.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 6.0], [5.0, 7.0, 3.0], [6.0, 3.0, 4.0], [5.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 1.0], [2.0, 4.0, 2.0]], [[6.0, 6.0, 1.0], [6.0, 
2.0, 2.0], [6.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[7.0, 5.0, 6.0], [4.0, 5.0, 3.0], [8.0, 4.0, 4.0], [5.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CLR(self):
      arg=Data(numpy.array([[[5.0, 5.0, 6.0], [4.0, 4.0, 8.0], [5.0, 4.0, 2.0], [3.0, 7.0, 4.0]], [[3.0, 4.0, 3.0], [4.0, 6.0, 
7.0], [2.0, 3.0, 6.0], [1.0, 1.0, 2.0]], [[6.0, 7.0, 6.0], [1.0, 5.0, 4.0], [2.0, 4.0, 4.0], [8.0, 1.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[[4.0], [5.0]], [[1.0], [8.0]], [[8.0], [5.0]]])
      arg[:,2:,1:2]=val
      ref=Data(numpy.array([[[5.0, 5.0, 6.0], [4.0, 4.0, 8.0], [5.0, 4.0, 2.0], [3.0, 5.0, 4.0]], [[3.0, 4.0, 3.0], [4.0, 6.0, 
7.0], [2.0, 1.0, 6.0], [1.0, 8.0, 2.0]], [[6.0, 7.0, 6.0], [1.0, 5.0, 4.0], [2.0, 8.0, 4.0], [8.0, 5.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CLR(self):
      arg=Data(numpy.array([[[2.0, 3.0, 1.0], [4.0, 2.0, 6.0], [2.0, 1.0, 4.0], [3.0, 5.0, 8.0]], [[3.0, 8.0, 2.0], [4.0, 1.0, 
5.0], [4.0, 5.0, 8.0], [4.0, 8.0, 6.0]], [[1.0, 7.0, 5.0], [2.0, 8.0, 1.0], [4.0, 5.0, 6.0], [1.0, 1.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[3.0], [7.0]], [[7.0], [8.0]], [[7.0], [2.0]]]),self.functionspace)
      arg[:,2:,1:2]=val
      ref=Data(numpy.array([[[2.0, 3.0, 1.0], [4.0, 2.0, 6.0], [2.0, 3.0, 4.0], [3.0, 7.0, 8.0]], [[3.0, 8.0, 2.0], [4.0, 1.0, 
5.0], [4.0, 7.0, 8.0], [4.0, 8.0, 6.0]], [[1.0, 7.0, 5.0], [2.0, 8.0, 1.0], [4.0, 7.0, 6.0], [1.0, 2.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CLR(self):
      arg=Data(numpy.array([[[4.0, 3.0, 1.0], [4.0, 5.0, 4.0], [2.0, 7.0, 6.0], [4.0, 5.0, 5.0]], [[1.0, 5.0, 6.0], [7.0, 7.0, 
7.0], [2.0, 5.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 3.0], [2.0, 8.0, 2.0], [1.0, 1.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0], [4.0]], [[2.0], [1.0]], [[6.0], [2.0]]])+(1.-msk_val)*numpy.array([[[1.0], [5.0]], 
[[2.0], [4.0]], [[4.0], [8.0]]])
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 1.0], [4.0, 5.0, 4.0], [2.0, 5.0, 6.0], [4.0, 4.0, 5.0]], [[1.0, 5.0, 6.0], [7.0, 
7.0, 7.0], [2.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 3.0], [2.0, 6.0, 2.0], [1.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 1.0], [4.0, 5.0, 4.0], [2.0, 1.0, 6.0], [4.0, 5.0, 5.0]], [[1.0, 5.0, 6.0], [7.0, 
7.0, 7.0], [2.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 3.0], [2.0, 4.0, 2.0], [1.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CCN(self):
      arg=Data(numpy.array([[[8.0, 2.0, 4.0], [3.0, 3.0, 4.0], [3.0, 8.0, 8.0], [8.0, 2.0, 8.0]], [[3.0, 8.0, 1.0], [6.0, 8.0, 
1.0], [8.0, 2.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 3.0, 7.0], [7.0, 6.0, 4.0], [3.0, 5.0, 4.0], [5.0, 7.0, 
8.0]]]),self.functionspace)
      val=numpy.array([[[5.0, 1.0, 6.0], [2.0, 8.0, 2.0], [3.0, 4.0, 7.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 5.0], [6.0, 4.0, 3.0], 
[3.0, 7.0, 1.0], [6.0, 2.0, 5.0]], [[1.0, 7.0, 4.0], [6.0, 7.0, 1.0], [7.0, 2.0, 4.0], [4.0, 2.0, 1.0]]])
      arg[:,:]=val
      ref=Data(numpy.array([[[5.0, 1.0, 6.0], [2.0, 8.0, 2.0], [3.0, 4.0, 7.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 5.0], [6.0, 4.0, 
3.0], [3.0, 7.0, 1.0], [6.0, 2.0, 5.0]], [[1.0, 7.0, 4.0], [6.0, 7.0, 1.0], [7.0, 2.0, 4.0], [4.0, 2.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CCN(self):
      arg=Data(numpy.array([[[7.0, 7.0, 4.0], [5.0, 7.0, 1.0], [1.0, 7.0, 4.0], [6.0, 5.0, 7.0]], [[5.0, 8.0, 2.0], [7.0, 1.0, 
5.0], [3.0, 5.0, 8.0], [6.0, 3.0, 3.0]], [[7.0, 5.0, 6.0], [8.0, 8.0, 4.0], [7.0, 3.0, 6.0], [2.0, 5.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 2.0, 6.0], [2.0, 3.0, 6.0], [8.0, 7.0, 5.0], [3.0, 4.0, 3.0]], [[1.0, 3.0, 7.0], [5.0, 6.0, 
7.0], [3.0, 5.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [8.0, 6.0, 5.0], [5.0, 5.0, 4.0], [4.0, 1.0, 
6.0]]]),self.functionspace)
      arg[:,:]=val
      ref=Data(numpy.array([[[7.0, 2.0, 6.0], [2.0, 3.0, 6.0], [8.0, 7.0, 5.0], [3.0, 4.0, 3.0]], [[1.0, 3.0, 7.0], [5.0, 6.0, 
7.0], [3.0, 5.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [8.0, 6.0, 5.0], [5.0, 5.0, 4.0], [4.0, 1.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CCN(self):
      arg=Data(numpy.array([[[8.0, 5.0, 2.0], [5.0, 2.0, 1.0], [3.0, 4.0, 7.0], [7.0, 4.0, 1.0]], [[1.0, 4.0, 4.0], [4.0, 3.0, 
4.0], [2.0, 7.0, 6.0], [8.0, 4.0, 5.0]], [[4.0, 2.0, 3.0], [8.0, 4.0, 2.0], [3.0, 4.0, 6.0], [7.0, 1.0, 
6.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 8.0, 8.0], [5.0, 7.0, 1.0], [6.0, 7.0, 6.0], [8.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [4.0, 
4.0, 4.0], [4.0, 3.0, 5.0], [1.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [3.0, 2.0, 2.0], [7.0, 3.0, 1.0], [5.0, 6.0, 
3.0]]])+(1.-msk_val)*numpy.array([[[2.0, 4.0, 3.0], [4.0, 6.0, 4.0], [7.0, 5.0, 7.0], [8.0, 1.0, 8.0]], [[5.0, 8.0, 8.0], [6.0, 
8.0, 2.0], [1.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[5.0, 8.0, 1.0], [2.0, 6.0, 7.0], [4.0, 5.0, 5.0], [5.0, 5.0, 8.0]]])
      arg[:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 8.0], [5.0, 7.0, 1.0], [6.0, 7.0, 6.0], [8.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [4.0, 
4.0, 4.0], [4.0, 3.0, 5.0], [1.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [3.0, 2.0, 2.0], [7.0, 3.0, 1.0], [5.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 4.0, 3.0], [4.0, 6.0, 4.0], [7.0, 5.0, 7.0], [8.0, 1.0, 8.0]], [[5.0, 8.0, 8.0], [6.0, 
8.0, 2.0], [1.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[5.0, 8.0, 1.0], [2.0, 6.0, 7.0], [4.0, 5.0, 5.0], [5.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LUR(self):
      arg=Data(numpy.array([[[4.0, 7.0, 2.0], [4.0, 5.0, 6.0], [3.0, 4.0, 6.0], [1.0, 4.0, 4.0]], [[6.0, 5.0, 2.0], [8.0, 6.0, 
8.0], [7.0, 1.0, 6.0], [1.0, 5.0, 8.0]], [[6.0, 2.0, 4.0], [1.0, 5.0, 3.0], [2.0, 5.0, 8.0], [1.0, 8.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[1.0], [7.0], [6.0]]])
      arg[2:,:3,1:2]=val
      ref=Data(numpy.array([[[4.0, 7.0, 2.0], [4.0, 5.0, 6.0], [3.0, 4.0, 6.0], [1.0, 4.0, 4.0]], [[6.0, 5.0, 2.0], [8.0, 6.0, 
8.0], [7.0, 1.0, 6.0], [1.0, 5.0, 8.0]], [[6.0, 1.0, 4.0], [1.0, 7.0, 3.0], [2.0, 6.0, 8.0], [1.0, 8.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LUR(self):
      arg=Data(numpy.array([[[6.0, 8.0, 6.0], [8.0, 1.0, 1.0], [2.0, 2.0, 3.0], [4.0, 4.0, 3.0]], [[1.0, 6.0, 7.0], [4.0, 3.0, 
3.0], [6.0, 5.0, 8.0], [7.0, 8.0, 2.0]], [[6.0, 7.0, 8.0], [4.0, 3.0, 5.0], [6.0, 7.0, 6.0], [7.0, 4.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0], [3.0], [5.0]]]),self.functionspace)
      arg[2:,:3,1:2]=val
      ref=Data(numpy.array([[[6.0, 8.0, 6.0], [8.0, 1.0, 1.0], [2.0, 2.0, 3.0], [4.0, 4.0, 3.0]], [[1.0, 6.0, 7.0], [4.0, 3.0, 
3.0], [6.0, 5.0, 8.0], [7.0, 8.0, 2.0]], [[6.0, 1.0, 8.0], [4.0, 3.0, 5.0], [6.0, 5.0, 6.0], [7.0, 4.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LUR(self):
      arg=Data(numpy.array([[[3.0, 1.0, 4.0], [1.0, 5.0, 6.0], [3.0, 5.0, 7.0], [6.0, 8.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 2.0, 
6.0], [1.0, 6.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 8.0, 1.0], [4.0, 4.0, 2.0], [2.0, 7.0, 6.0], [4.0, 6.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0], [2.0], [1.0]]])+(1.-msk_val)*numpy.array([[[1.0], [6.0], [5.0]]])
      arg[2:,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 4.0], [1.0, 5.0, 6.0], [3.0, 5.0, 7.0], [6.0, 8.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 
2.0, 6.0], [1.0, 6.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 7.0, 1.0], [4.0, 2.0, 2.0], [2.0, 1.0, 6.0], [4.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 4.0], [1.0, 5.0, 6.0], [3.0, 5.0, 7.0], [6.0, 8.0, 2.0]], [[3.0, 8.0, 4.0], [3.0, 
2.0, 6.0], [1.0, 6.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 6.0, 2.0], [2.0, 5.0, 6.0], [4.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CCL(self):
      arg=Data(numpy.array([[[1.0, 2.0, 2.0], [4.0, 6.0, 3.0], [1.0, 4.0, 7.0], [8.0, 7.0, 1.0]], [[1.0, 6.0, 7.0], [1.0, 1.0, 
1.0], [7.0, 3.0, 4.0], [7.0, 5.0, 3.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 6.0], [5.0, 7.0, 4.0], [1.0, 3.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[6.0], [1.0], [7.0], [4.0]], [[5.0], [5.0], [2.0], [2.0]], [[2.0], [1.0], [7.0], [3.0]]])
      arg[:,:,2:]=val
      ref=Data(numpy.array([[[1.0, 2.0, 6.0], [4.0, 6.0, 1.0], [1.0, 4.0, 7.0], [8.0, 7.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 1.0, 
5.0], [7.0, 3.0, 2.0], [7.0, 5.0, 2.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 1.0], [5.0, 7.0, 7.0], [1.0, 3.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CCL(self):
      arg=Data(numpy.array([[[4.0, 1.0, 8.0], [2.0, 4.0, 3.0], [2.0, 1.0, 7.0], [1.0, 5.0, 4.0]], [[2.0, 4.0, 8.0], [5.0, 3.0, 
5.0], [1.0, 6.0, 1.0], [5.0, 7.0, 8.0]], [[8.0, 2.0, 7.0], [4.0, 7.0, 1.0], [6.0, 6.0, 3.0], [5.0, 5.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0], [5.0], [5.0], [8.0]], [[8.0], [2.0], [5.0], [1.0]], [[7.0], [5.0], [4.0], 
[2.0]]]),self.functionspace)
      arg[:,:,2:]=val
      ref=Data(numpy.array([[[4.0, 1.0, 6.0], [2.0, 4.0, 5.0], [2.0, 1.0, 5.0], [1.0, 5.0, 8.0]], [[2.0, 4.0, 8.0], [5.0, 3.0, 
2.0], [1.0, 6.0, 5.0], [5.0, 7.0, 1.0]], [[8.0, 2.0, 7.0], [4.0, 7.0, 5.0], [6.0, 6.0, 4.0], [5.0, 5.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CCL(self):
      arg=Data(numpy.array([[[2.0, 7.0, 8.0], [4.0, 6.0, 8.0], [6.0, 2.0, 3.0], [5.0, 5.0, 6.0]], [[3.0, 8.0, 8.0], [3.0, 3.0, 
4.0], [3.0, 6.0, 3.0], [8.0, 4.0, 7.0]], [[5.0, 3.0, 2.0], [5.0, 7.0, 1.0], [6.0, 6.0, 8.0], [3.0, 2.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0], [6.0], [6.0], [1.0]], [[5.0], [5.0], [6.0], [7.0]], [[2.0], [3.0], [7.0], 
[6.0]]])+(1.-msk_val)*numpy.array([[[6.0], [5.0], [5.0], [8.0]], [[5.0], [2.0], [1.0], [7.0]], [[3.0], [2.0], [7.0], [2.0]]])
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 8.0], [4.0, 6.0, 6.0], [6.0, 2.0, 6.0], [5.0, 5.0, 1.0]], [[3.0, 8.0, 5.0], [3.0, 
3.0, 5.0], [3.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[5.0, 3.0, 2.0], [5.0, 7.0, 3.0], [6.0, 6.0, 7.0], [3.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 6.0], [4.0, 6.0, 5.0], [6.0, 2.0, 5.0], [5.0, 5.0, 8.0]], [[3.0, 8.0, 5.0], [3.0, 
3.0, 2.0], [3.0, 6.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 3.0, 3.0], [5.0, 7.0, 2.0], [6.0, 6.0, 7.0], [3.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CPU(self):
      arg=Data(numpy.array([[[5.0, 8.0, 6.0], [6.0, 7.0, 8.0], [8.0, 6.0, 5.0], [8.0, 6.0, 5.0]], [[6.0, 5.0, 5.0], [7.0, 2.0, 
7.0], [3.0, 2.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 2.0, 3.0], [1.0, 7.0, 2.0], [2.0, 3.0, 8.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[8.0, 2.0], [1.0, 4.0], [2.0, 4.0]])
      arg[:,2,:2]=val
      ref=Data(numpy.array([[[5.0, 8.0, 6.0], [6.0, 7.0, 8.0], [8.0, 2.0, 5.0], [8.0, 6.0, 5.0]], [[6.0, 5.0, 5.0], [7.0, 2.0, 
7.0], [1.0, 4.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 2.0, 3.0], [1.0, 7.0, 2.0], [2.0, 4.0, 8.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CPU(self):
      arg=Data(numpy.array([[[3.0, 1.0, 2.0], [4.0, 8.0, 1.0], [2.0, 6.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 8.0], [3.0, 5.0, 
6.0], [5.0, 5.0, 1.0], [1.0, 1.0, 1.0]], [[7.0, 4.0, 6.0], [1.0, 7.0, 1.0], [3.0, 2.0, 5.0], [7.0, 8.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[6.0, 2.0], [5.0, 6.0], [1.0, 3.0]]),self.functionspace)
      arg[:,2,:2]=val
      ref=Data(numpy.array([[[3.0, 1.0, 2.0], [4.0, 8.0, 1.0], [6.0, 2.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 8.0], [3.0, 5.0, 
6.0], [5.0, 6.0, 1.0], [1.0, 1.0, 1.0]], [[7.0, 4.0, 6.0], [1.0, 7.0, 1.0], [1.0, 3.0, 5.0], [7.0, 8.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CPU(self):
      arg=Data(numpy.array([[[7.0, 2.0, 6.0], [2.0, 2.0, 4.0], [7.0, 4.0, 6.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 3.0], [3.0, 5.0, 
4.0], [4.0, 7.0, 8.0], [5.0, 8.0, 5.0]], [[8.0, 4.0, 1.0], [2.0, 7.0, 4.0], [3.0, 5.0, 4.0], [5.0, 5.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 1.0], [1.0, 6.0], [4.0, 7.0]])+(1.-msk_val)*numpy.array([[1.0, 7.0], [8.0, 7.0], [6.0, 
5.0]])
      arg[:,2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 6.0], [2.0, 2.0, 4.0], [4.0, 1.0, 6.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 3.0], [3.0, 
5.0, 4.0], [1.0, 6.0, 8.0], [5.0, 8.0, 5.0]], [[8.0, 4.0, 1.0], [2.0, 7.0, 4.0], [4.0, 7.0, 4.0], [5.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 6.0], [2.0, 2.0, 4.0], [1.0, 7.0, 6.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 3.0], [3.0, 
5.0, 4.0], [8.0, 7.0, 8.0], [5.0, 8.0, 5.0]], [[8.0, 4.0, 1.0], [2.0, 7.0, 4.0], [6.0, 5.0, 4.0], [5.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LUP(self):
      arg=Data(numpy.array([[[6.0, 1.0, 4.0], [7.0, 6.0, 6.0], [1.0, 2.0, 8.0], [1.0, 7.0, 7.0]], [[5.0, 5.0, 4.0], [1.0, 1.0, 
3.0], [3.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[2.0, 7.0, 5.0], [6.0, 2.0, 6.0], [6.0, 4.0, 8.0], [4.0, 3.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[5.0, 2.0, 5.0]])
      arg[2:,:3,1]=val
      ref=Data(numpy.array([[[6.0, 1.0, 4.0], [7.0, 6.0, 6.0], [1.0, 2.0, 8.0], [1.0, 7.0, 7.0]], [[5.0, 5.0, 4.0], [1.0, 1.0, 
3.0], [3.0, 4.0, 3.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 5.0], [6.0, 2.0, 6.0], [6.0, 5.0, 8.0], [4.0, 3.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LUP(self):
      arg=Data(numpy.array([[[8.0, 4.0, 7.0], [1.0, 2.0, 6.0], [2.0, 6.0, 1.0], [6.0, 5.0, 7.0]], [[2.0, 3.0, 1.0], [3.0, 6.0, 
3.0], [6.0, 4.0, 4.0], [7.0, 7.0, 6.0]], [[7.0, 4.0, 7.0], [5.0, 4.0, 5.0], [3.0, 8.0, 3.0], [4.0, 8.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 7.0, 2.0]]),self.functionspace)
      arg[2:,:3,1]=val
      ref=Data(numpy.array([[[8.0, 4.0, 7.0], [1.0, 2.0, 6.0], [2.0, 6.0, 1.0], [6.0, 5.0, 7.0]], [[2.0, 3.0, 1.0], [3.0, 6.0, 
3.0], [6.0, 4.0, 4.0], [7.0, 7.0, 6.0]], [[7.0, 4.0, 7.0], [5.0, 7.0, 5.0], [3.0, 2.0, 3.0], [4.0, 8.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LUP(self):
      arg=Data(numpy.array([[[2.0, 2.0, 1.0], [1.0, 1.0, 7.0], [8.0, 1.0, 7.0], [8.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 3.0, 
7.0], [2.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[3.0, 1.0, 5.0], [2.0, 4.0, 5.0], [1.0, 2.0, 3.0], [2.0, 8.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 2.0, 3.0]])+(1.-msk_val)*numpy.array([[2.0, 5.0, 1.0]])
      arg[2:,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 1.0], [1.0, 1.0, 7.0], [8.0, 1.0, 7.0], [8.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 
3.0, 7.0], [2.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[3.0, 5.0, 5.0], [2.0, 2.0, 5.0], [1.0, 3.0, 3.0], [2.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 1.0], [1.0, 1.0, 7.0], [8.0, 1.0, 7.0], [8.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 
3.0, 7.0], [2.0, 1.0, 5.0], [7.0, 8.0, 7.0]], [[3.0, 2.0, 5.0], [2.0, 5.0, 5.0], [1.0, 1.0, 3.0], [2.0, 8.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LLP(self):
      arg=Data(numpy.array([[[8.0, 8.0, 6.0], [8.0, 3.0, 5.0], [2.0, 8.0, 5.0], [6.0, 4.0, 1.0]], [[8.0, 1.0, 3.0], [6.0, 7.0, 
8.0], [4.0, 4.0, 5.0], [5.0, 3.0, 5.0]], [[4.0, 2.0, 2.0], [1.0, 1.0, 2.0], [4.0, 1.0, 7.0], [7.0, 6.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[1.0, 4.0]])
      arg[2:,2:,1]=val
      ref=Data(numpy.array([[[8.0, 8.0, 6.0], [8.0, 3.0, 5.0], [2.0, 8.0, 5.0], [6.0, 4.0, 1.0]], [[8.0, 1.0, 3.0], [6.0, 7.0, 
8.0], [4.0, 4.0, 5.0], [5.0, 3.0, 5.0]], [[4.0, 2.0, 2.0], [1.0, 1.0, 2.0], [4.0, 1.0, 7.0], [7.0, 4.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LLP(self):
      arg=Data(numpy.array([[[7.0, 5.0, 2.0], [7.0, 6.0, 3.0], [5.0, 6.0, 4.0], [8.0, 6.0, 5.0]], [[7.0, 8.0, 6.0], [3.0, 5.0, 
6.0], [1.0, 4.0, 1.0], [7.0, 3.0, 5.0]], [[8.0, 4.0, 5.0], [6.0, 2.0, 1.0], [2.0, 6.0, 4.0], [1.0, 4.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[6.0, 4.0]]),self.functionspace)
      arg[2:,2:,1]=val
      ref=Data(numpy.array([[[7.0, 5.0, 2.0], [7.0, 6.0, 3.0], [5.0, 6.0, 4.0], [8.0, 6.0, 5.0]], [[7.0, 8.0, 6.0], [3.0, 5.0, 
6.0], [1.0, 4.0, 1.0], [7.0, 3.0, 5.0]], [[8.0, 4.0, 5.0], [6.0, 2.0, 1.0], [2.0, 6.0, 4.0], [1.0, 4.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LLP(self):
      arg=Data(numpy.array([[[3.0, 3.0, 7.0], [1.0, 1.0, 8.0], [3.0, 2.0, 4.0], [4.0, 1.0, 2.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 
7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 6.0], [3.0, 8.0, 4.0], [6.0, 8.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 1.0]])+(1.-msk_val)*numpy.array([[1.0, 7.0]])
      arg[2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 7.0], [1.0, 1.0, 8.0], [3.0, 2.0, 4.0], [4.0, 1.0, 2.0]], [[7.0, 2.0, 6.0], [6.0, 
1.0, 7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 6.0], [3.0, 4.0, 4.0], [6.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 7.0], [1.0, 1.0, 8.0], [3.0, 2.0, 4.0], [4.0, 1.0, 2.0]], [[7.0, 2.0, 6.0], [6.0, 
1.0, 7.0], [2.0, 1.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 6.0], [3.0, 1.0, 4.0], [6.0, 7.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PCC(self):
      arg=Data(numpy.array([[[8.0, 8.0, 1.0], [7.0, 2.0, 6.0], [8.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[1.0, 2.0, 4.0], [4.0, 7.0, 
7.0], [3.0, 7.0, 6.0], [2.0, 2.0, 6.0]], [[7.0, 3.0, 1.0], [2.0, 2.0, 8.0], [5.0, 2.0, 3.0], [1.0, 5.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[7.0, 7.0, 5.0], [3.0, 4.0, 8.0], [4.0, 6.0, 4.0], [7.0, 6.0, 4.0]])
      arg[1,:,:]=val
      ref=Data(numpy.array([[[8.0, 8.0, 1.0], [7.0, 2.0, 6.0], [8.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[7.0, 7.0, 5.0], [3.0, 4.0, 
8.0], [4.0, 6.0, 4.0], [7.0, 6.0, 4.0]], [[7.0, 3.0, 1.0], [2.0, 2.0, 8.0], [5.0, 2.0, 3.0], [1.0, 5.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PCC(self):
      arg=Data(numpy.array([[[6.0, 1.0, 6.0], [4.0, 2.0, 3.0], [6.0, 4.0, 4.0], [7.0, 5.0, 5.0]], [[8.0, 2.0, 3.0], [6.0, 4.0, 
2.0], [2.0, 6.0, 4.0], [5.0, 8.0, 2.0]], [[3.0, 3.0, 2.0], [2.0, 4.0, 3.0], [4.0, 8.0, 3.0], [3.0, 5.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 8.0, 2.0], [2.0, 8.0, 7.0], [4.0, 2.0, 7.0], [6.0, 6.0, 8.0]]),self.functionspace)
      arg[1,:,:]=val
      ref=Data(numpy.array([[[6.0, 1.0, 6.0], [4.0, 2.0, 3.0], [6.0, 4.0, 4.0], [7.0, 5.0, 5.0]], [[3.0, 8.0, 2.0], [2.0, 8.0, 
7.0], [4.0, 2.0, 7.0], [6.0, 6.0, 8.0]], [[3.0, 3.0, 2.0], [2.0, 4.0, 3.0], [4.0, 8.0, 3.0], [3.0, 5.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PCC(self):
      arg=Data(numpy.array([[[4.0, 4.0, 6.0], [4.0, 5.0, 3.0], [1.0, 6.0, 6.0], [2.0, 1.0, 5.0]], [[6.0, 8.0, 6.0], [5.0, 6.0, 
6.0], [7.0, 2.0, 2.0], [2.0, 1.0, 8.0]], [[3.0, 8.0, 6.0], [6.0, 2.0, 8.0], [8.0, 4.0, 3.0], [3.0, 6.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 5.0, 2.0], [4.0, 3.0, 3.0], [1.0, 8.0, 3.0], [3.0, 8.0, 
2.0]])+(1.-msk_val)*numpy.array([[4.0, 8.0, 7.0], [4.0, 2.0, 1.0], [6.0, 2.0, 6.0], [3.0, 2.0, 3.0]])
      arg[1,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 6.0], [4.0, 5.0, 3.0], [1.0, 6.0, 6.0], [2.0, 1.0, 5.0]], [[3.0, 5.0, 2.0], [4.0, 
3.0, 3.0], [1.0, 8.0, 3.0], [3.0, 8.0, 2.0]], [[3.0, 8.0, 6.0], [6.0, 2.0, 8.0], [8.0, 4.0, 3.0], [3.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 6.0], [4.0, 5.0, 3.0], [1.0, 6.0, 6.0], [2.0, 1.0, 5.0]], [[4.0, 8.0, 7.0], [4.0, 
2.0, 1.0], [6.0, 2.0, 6.0], [3.0, 2.0, 3.0]], [[3.0, 8.0, 6.0], [6.0, 2.0, 8.0], [8.0, 4.0, 3.0], [3.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RRU(self):
      arg=Data(numpy.array([[[7.0, 7.0, 7.0], [6.0, 6.0, 6.0], [5.0, 6.0, 6.0], [1.0, 5.0, 5.0]], [[4.0, 7.0, 1.0], [8.0, 8.0, 
2.0], [2.0, 2.0, 6.0], [2.0, 1.0, 1.0]], [[7.0, 7.0, 6.0], [1.0, 2.0, 7.0], [2.0, 6.0, 5.0], [3.0, 4.0, 
8.0]]]),self.functionspace)
      val=numpy.array([[[2.0, 5.0], [6.0, 2.0]]])
      arg[1:2,1:3,:2]=val
      ref=Data(numpy.array([[[7.0, 7.0, 7.0], [6.0, 6.0, 6.0], [5.0, 6.0, 6.0], [1.0, 5.0, 5.0]], [[4.0, 7.0, 1.0], [2.0, 5.0, 
2.0], [6.0, 2.0, 6.0], [2.0, 1.0, 1.0]], [[7.0, 7.0, 6.0], [1.0, 2.0, 7.0], [2.0, 6.0, 5.0], [3.0, 4.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RRU(self):
      arg=Data(numpy.array([[[5.0, 4.0, 4.0], [6.0, 4.0, 6.0], [1.0, 2.0, 6.0], [2.0, 2.0, 4.0]], [[1.0, 7.0, 3.0], [7.0, 2.0, 
6.0], [5.0, 8.0, 2.0], [5.0, 2.0, 5.0]], [[3.0, 1.0, 2.0], [3.0, 3.0, 4.0], [4.0, 7.0, 4.0], [5.0, 6.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 7.0], [5.0, 7.0]]]),self.functionspace)
      arg[1:2,1:3,:2]=val
      ref=Data(numpy.array([[[5.0, 4.0, 4.0], [6.0, 4.0, 6.0], [1.0, 2.0, 6.0], [2.0, 2.0, 4.0]], [[1.0, 7.0, 3.0], [6.0, 7.0, 
6.0], [5.0, 7.0, 2.0], [5.0, 2.0, 5.0]], [[3.0, 1.0, 2.0], [3.0, 3.0, 4.0], [4.0, 7.0, 4.0], [5.0, 6.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RRU(self):
      arg=Data(numpy.array([[[7.0, 8.0, 3.0], [4.0, 7.0, 4.0], [8.0, 6.0, 3.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 1.0], [3.0, 5.0, 
7.0], [7.0, 2.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], [1.0, 5.0, 1.0], [7.0, 5.0, 5.0], [8.0, 1.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 3.0], [3.0, 1.0]]])+(1.-msk_val)*numpy.array([[[6.0, 5.0], [6.0, 4.0]]])
      arg[1:2,1:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 3.0], [4.0, 7.0, 4.0], [8.0, 6.0, 3.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 1.0], [7.0, 
3.0, 7.0], [3.0, 1.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], [1.0, 5.0, 1.0], [7.0, 5.0, 5.0], [8.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 3.0], [4.0, 7.0, 4.0], [8.0, 6.0, 3.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 
5.0, 7.0], [6.0, 4.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], [1.0, 5.0, 1.0], [7.0, 5.0, 5.0], [8.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LLN(self):
      arg=Data(numpy.array([[[8.0, 6.0, 3.0], [2.0, 8.0, 1.0], [3.0, 8.0, 7.0], [7.0, 4.0, 1.0]], [[8.0, 1.0, 7.0], [2.0, 7.0, 
7.0], [5.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[3.0, 8.0, 5.0], [4.0, 6.0, 5.0], [4.0, 6.0, 6.0], [6.0, 7.0, 
6.0]]]),self.functionspace)
      val=numpy.array([[[6.0, 8.0, 8.0], [4.0, 8.0, 6.0]]])
      arg[2:,2:]=val
      ref=Data(numpy.array([[[8.0, 6.0, 3.0], [2.0, 8.0, 1.0], [3.0, 8.0, 7.0], [7.0, 4.0, 1.0]], [[8.0, 1.0, 7.0], [2.0, 7.0, 
7.0], [5.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[3.0, 8.0, 5.0], [4.0, 6.0, 5.0], [6.0, 8.0, 8.0], [4.0, 8.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LLN(self):
      arg=Data(numpy.array([[[2.0, 1.0, 3.0], [2.0, 6.0, 2.0], [4.0, 2.0, 4.0], [4.0, 3.0, 6.0]], [[6.0, 6.0, 4.0], [4.0, 1.0, 
8.0], [5.0, 2.0, 8.0], [6.0, 1.0, 1.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 8.0], [1.0, 8.0, 1.0], [3.0, 8.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 7.0, 6.0], [8.0, 4.0, 1.0]]]),self.functionspace)
      arg[2:,2:]=val
      ref=Data(numpy.array([[[2.0, 1.0, 3.0], [2.0, 6.0, 2.0], [4.0, 2.0, 4.0], [4.0, 3.0, 6.0]], [[6.0, 6.0, 4.0], [4.0, 1.0, 
8.0], [5.0, 2.0, 8.0], [6.0, 1.0, 1.0]], [[3.0, 1.0, 5.0], [2.0, 2.0, 8.0], [5.0, 7.0, 6.0], [8.0, 4.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LLN(self):
      arg=Data(numpy.array([[[7.0, 5.0, 8.0], [8.0, 6.0, 7.0], [5.0, 8.0, 4.0], [1.0, 5.0, 4.0]], [[7.0, 2.0, 6.0], [7.0, 8.0, 
4.0], [1.0, 7.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 6.0, 5.0], [2.0, 7.0, 6.0], [6.0, 7.0, 1.0], [5.0, 5.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 2.0, 3.0], [4.0, 6.0, 4.0]]])+(1.-msk_val)*numpy.array([[[7.0, 6.0, 5.0], [4.0, 5.0, 
5.0]]])
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 8.0], [8.0, 6.0, 7.0], [5.0, 8.0, 4.0], [1.0, 5.0, 4.0]], [[7.0, 2.0, 6.0], [7.0, 
8.0, 4.0], [1.0, 7.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 6.0, 5.0], [2.0, 7.0, 6.0], [6.0, 2.0, 3.0], [4.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 8.0], [8.0, 6.0, 7.0], [5.0, 8.0, 4.0], [1.0, 5.0, 4.0]], [[7.0, 2.0, 6.0], [7.0, 
8.0, 4.0], [1.0, 7.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 6.0, 5.0], [2.0, 7.0, 6.0], [7.0, 6.0, 5.0], [4.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UCP(self):
      arg=Data(numpy.array([[[1.0, 7.0, 2.0], [5.0, 2.0, 1.0], [1.0, 4.0, 8.0], [4.0, 5.0, 1.0]], [[8.0, 5.0, 5.0], [3.0, 1.0, 
3.0], [2.0, 5.0, 1.0], [4.0, 7.0, 6.0]], [[4.0, 3.0, 6.0], [5.0, 6.0, 8.0], [4.0, 7.0, 7.0], [6.0, 6.0, 
8.0]]]),self.functionspace)
      val=numpy.array([[4.0, 3.0, 1.0, 1.0], [8.0, 7.0, 2.0, 2.0]])
      arg[:2,:,1]=val
      ref=Data(numpy.array([[[1.0, 4.0, 2.0], [5.0, 3.0, 1.0], [1.0, 1.0, 8.0], [4.0, 1.0, 1.0]], [[8.0, 8.0, 5.0], [3.0, 7.0, 
3.0], [2.0, 2.0, 1.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 6.0], [5.0, 6.0, 8.0], [4.0, 7.0, 7.0], [6.0, 6.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UCP(self):
      arg=Data(numpy.array([[[4.0, 8.0, 5.0], [5.0, 1.0, 8.0], [8.0, 8.0, 6.0], [7.0, 7.0, 6.0]], [[5.0, 1.0, 8.0], [6.0, 3.0, 
8.0], [6.0, 4.0, 6.0], [5.0, 1.0, 5.0]], [[5.0, 4.0, 5.0], [6.0, 6.0, 8.0], [3.0, 1.0, 6.0], [7.0, 3.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 7.0, 4.0, 2.0], [6.0, 6.0, 8.0, 2.0]]),self.functionspace)
      arg[:2,:,1]=val
      ref=Data(numpy.array([[[4.0, 2.0, 5.0], [5.0, 7.0, 8.0], [8.0, 4.0, 6.0], [7.0, 2.0, 6.0]], [[5.0, 6.0, 8.0], [6.0, 6.0, 
8.0], [6.0, 8.0, 6.0], [5.0, 2.0, 5.0]], [[5.0, 4.0, 5.0], [6.0, 6.0, 8.0], [3.0, 1.0, 6.0], [7.0, 3.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UCP(self):
      arg=Data(numpy.array([[[4.0, 1.0, 3.0], [7.0, 8.0, 8.0], [2.0, 3.0, 4.0], [4.0, 2.0, 2.0]], [[3.0, 4.0, 6.0], [1.0, 4.0, 
3.0], [3.0, 8.0, 3.0], [3.0, 5.0, 2.0]], [[3.0, 5.0, 1.0], [4.0, 3.0, 2.0], [4.0, 5.0, 6.0], [3.0, 8.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 8.0, 5.0, 4.0], [8.0, 5.0, 5.0, 2.0]])+(1.-msk_val)*numpy.array([[6.0, 8.0, 8.0, 1.0], 
[3.0, 6.0, 4.0, 3.0]])
      arg[:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 3.0], [7.0, 8.0, 8.0], [2.0, 5.0, 4.0], [4.0, 4.0, 2.0]], [[3.0, 8.0, 6.0], [1.0, 
5.0, 3.0], [3.0, 5.0, 3.0], [3.0, 2.0, 2.0]], [[3.0, 5.0, 1.0], [4.0, 3.0, 2.0], [4.0, 5.0, 6.0], [3.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 3.0], [7.0, 8.0, 8.0], [2.0, 8.0, 4.0], [4.0, 1.0, 2.0]], [[3.0, 3.0, 6.0], [1.0, 
6.0, 3.0], [3.0, 4.0, 3.0], [3.0, 3.0, 2.0]], [[3.0, 5.0, 1.0], [4.0, 3.0, 2.0], [4.0, 5.0, 6.0], [3.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PPN(self):
      arg=Data(numpy.array([[[8.0, 5.0, 2.0], [3.0, 5.0, 5.0], [2.0, 8.0, 7.0], [6.0, 2.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 7.0, 
5.0], [4.0, 1.0, 6.0], [6.0, 6.0, 3.0]], [[4.0, 8.0, 5.0], [2.0, 8.0, 8.0], [7.0, 7.0, 4.0], [6.0, 5.0, 
2.0]]]),self.functionspace)
      val=numpy.array([2.0, 7.0, 3.0])
      arg[1,2]=val
      ref=Data(numpy.array([[[8.0, 5.0, 2.0], [3.0, 5.0, 5.0], [2.0, 8.0, 7.0], [6.0, 2.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 7.0, 
5.0], [2.0, 7.0, 3.0], [6.0, 6.0, 3.0]], [[4.0, 8.0, 5.0], [2.0, 8.0, 8.0], [7.0, 7.0, 4.0], [6.0, 5.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PPN(self):
      arg=Data(numpy.array([[[5.0, 1.0, 8.0], [2.0, 8.0, 8.0], [6.0, 7.0, 6.0], [1.0, 8.0, 8.0]], [[4.0, 4.0, 3.0], [2.0, 8.0, 
6.0], [2.0, 8.0, 7.0], [2.0, 1.0, 8.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 1.0], [2.0, 2.0, 1.0], [5.0, 7.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([5.0, 5.0, 5.0]),self.functionspace)
      arg[1,2]=val
      ref=Data(numpy.array([[[5.0, 1.0, 8.0], [2.0, 8.0, 8.0], [6.0, 7.0, 6.0], [1.0, 8.0, 8.0]], [[4.0, 4.0, 3.0], [2.0, 8.0, 
6.0], [5.0, 5.0, 5.0], [2.0, 1.0, 8.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 1.0], [2.0, 2.0, 1.0], [5.0, 7.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PPN(self):
      arg=Data(numpy.array([[[2.0, 3.0, 6.0], [3.0, 6.0, 3.0], [4.0, 4.0, 5.0], [2.0, 3.0, 8.0]], [[2.0, 4.0, 7.0], [1.0, 3.0, 
5.0], [2.0, 3.0, 7.0], [4.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], [5.0, 5.0, 8.0], [7.0, 5.0, 4.0], [8.0, 4.0, 
6.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([6.0, 8.0, 4.0])+(1.-msk_val)*numpy.array([1.0, 8.0, 1.0])
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 6.0], [3.0, 6.0, 3.0], [4.0, 4.0, 5.0], [2.0, 3.0, 8.0]], [[2.0, 4.0, 7.0], [1.0, 
3.0, 5.0], [6.0, 8.0, 4.0], [4.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], [5.0, 5.0, 8.0], [7.0, 5.0, 4.0], [8.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 6.0], [3.0, 6.0, 3.0], [4.0, 4.0, 5.0], [2.0, 3.0, 8.0]], [[2.0, 4.0, 7.0], [1.0, 
3.0, 5.0], [1.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], [5.0, 5.0, 8.0], [7.0, 5.0, 4.0], [8.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LPN(self):
      arg=Data(numpy.array([[[5.0, 7.0, 2.0], [3.0, 2.0, 2.0], [8.0, 2.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 6.0, 6.0], [3.0, 8.0, 
5.0], [4.0, 3.0, 3.0], [1.0, 4.0, 2.0]], [[5.0, 5.0, 8.0], [7.0, 5.0, 5.0], [7.0, 6.0, 2.0], [7.0, 3.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[1.0, 1.0, 6.0]])
      arg[2:,2]=val
      ref=Data(numpy.array([[[5.0, 7.0, 2.0], [3.0, 2.0, 2.0], [8.0, 2.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 6.0, 6.0], [3.0, 8.0, 
5.0], [4.0, 3.0, 3.0], [1.0, 4.0, 2.0]], [[5.0, 5.0, 8.0], [7.0, 5.0, 5.0], [1.0, 1.0, 6.0], [7.0, 3.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LPN(self):
      arg=Data(numpy.array([[[5.0, 2.0, 7.0], [6.0, 5.0, 1.0], [3.0, 4.0, 8.0], [7.0, 8.0, 4.0]], [[8.0, 4.0, 4.0], [6.0, 1.0, 
5.0], [7.0, 4.0, 1.0], [7.0, 1.0, 1.0]], [[6.0, 5.0, 3.0], [1.0, 2.0, 4.0], [7.0, 6.0, 4.0], [7.0, 6.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 7.0, 3.0]]),self.functionspace)
      arg[2:,2]=val
      ref=Data(numpy.array([[[5.0, 2.0, 7.0], [6.0, 5.0, 1.0], [3.0, 4.0, 8.0], [7.0, 8.0, 4.0]], [[8.0, 4.0, 4.0], [6.0, 1.0, 
5.0], [7.0, 4.0, 1.0], [7.0, 1.0, 1.0]], [[6.0, 5.0, 3.0], [1.0, 2.0, 4.0], [2.0, 7.0, 3.0], [7.0, 6.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LPN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 4.0], [7.0, 1.0, 2.0], [7.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[6.0, 8.0, 1.0], [7.0, 3.0, 
2.0], [7.0, 3.0, 7.0], [3.0, 1.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 5.0, 4.0], [6.0, 6.0, 1.0], [2.0, 2.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 8.0, 6.0]])+(1.-msk_val)*numpy.array([[4.0, 4.0, 5.0]])
      arg[2:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 4.0], [7.0, 1.0, 2.0], [7.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[6.0, 8.0, 1.0], [7.0, 
3.0, 2.0], [7.0, 3.0, 7.0], [3.0, 1.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 5.0, 4.0], [1.0, 8.0, 6.0], [2.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 4.0], [7.0, 1.0, 2.0], [7.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[6.0, 8.0, 1.0], [7.0, 
3.0, 2.0], [7.0, 3.0, 7.0], [3.0, 1.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 5.0, 4.0], [4.0, 4.0, 5.0], [2.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UPN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 4.0], [6.0, 2.0, 8.0], [5.0, 2.0, 1.0], [3.0, 8.0, 5.0]], [[2.0, 1.0, 1.0], [1.0, 6.0, 
4.0], [8.0, 1.0, 4.0], [7.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [4.0, 1.0, 6.0], [5.0, 6.0, 2.0], [6.0, 6.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[3.0, 6.0, 1.0], [5.0, 1.0, 6.0]])
      arg[:2,2]=val
      ref=Data(numpy.array([[[3.0, 7.0, 4.0], [6.0, 2.0, 8.0], [3.0, 6.0, 1.0], [3.0, 8.0, 5.0]], [[2.0, 1.0, 1.0], [1.0, 6.0, 
4.0], [5.0, 1.0, 6.0], [7.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [4.0, 1.0, 6.0], [5.0, 6.0, 2.0], [6.0, 6.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UPN(self):
      arg=Data(numpy.array([[[1.0, 2.0, 1.0], [1.0, 3.0, 3.0], [4.0, 6.0, 5.0], [2.0, 7.0, 6.0]], [[4.0, 6.0, 8.0], [1.0, 3.0, 
2.0], [4.0, 2.0, 1.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [8.0, 5.0, 7.0], [4.0, 5.0, 5.0], [2.0, 3.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 7.0, 5.0], [2.0, 5.0, 1.0]]),self.functionspace)
      arg[:2,2]=val
      ref=Data(numpy.array([[[1.0, 2.0, 1.0], [1.0, 3.0, 3.0], [8.0, 7.0, 5.0], [2.0, 7.0, 6.0]], [[4.0, 6.0, 8.0], [1.0, 3.0, 
2.0], [2.0, 5.0, 1.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [8.0, 5.0, 7.0], [4.0, 5.0, 5.0], [2.0, 3.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UPN(self):
      arg=Data(numpy.array([[[3.0, 4.0, 6.0], [6.0, 8.0, 5.0], [2.0, 5.0, 5.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 4.0], [6.0, 2.0, 
2.0], [5.0, 6.0, 3.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 2.0, 2.0], [8.0, 5.0, 5.0], [4.0, 8.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 5.0, 2.0], [3.0, 7.0, 6.0]])+(1.-msk_val)*numpy.array([[6.0, 3.0, 7.0], [8.0, 6.0, 3.0]])
      arg[:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 6.0], [6.0, 8.0, 5.0], [6.0, 5.0, 2.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 4.0], [6.0, 
2.0, 2.0], [3.0, 7.0, 6.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 2.0, 2.0], [8.0, 5.0, 5.0], [4.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 6.0], [6.0, 8.0, 5.0], [6.0, 3.0, 7.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 4.0], [6.0, 
2.0, 2.0], [8.0, 6.0, 3.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 2.0, 2.0], [8.0, 5.0, 5.0], [4.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UPR(self):
      arg=Data(numpy.array([[[3.0, 4.0, 8.0], [2.0, 2.0, 1.0], [6.0, 3.0, 2.0], [5.0, 7.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 2.0, 
8.0], [4.0, 5.0, 3.0], [8.0, 7.0, 8.0]], [[2.0, 8.0, 7.0], [8.0, 6.0, 1.0], [2.0, 1.0, 8.0], [2.0, 2.0, 
3.0]]]),self.functionspace)
      val=numpy.array([[5.0], [6.0]])
      arg[:2,2,1:2]=val
      ref=Data(numpy.array([[[3.0, 4.0, 8.0], [2.0, 2.0, 1.0], [6.0, 5.0, 2.0], [5.0, 7.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 2.0, 
8.0], [4.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[2.0, 8.0, 7.0], [8.0, 6.0, 1.0], [2.0, 1.0, 8.0], [2.0, 2.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UPR(self):
      arg=Data(numpy.array([[[3.0, 7.0, 3.0], [5.0, 2.0, 2.0], [4.0, 2.0, 5.0], [7.0, 5.0, 2.0]], [[7.0, 7.0, 1.0], [2.0, 1.0, 
1.0], [7.0, 1.0, 6.0], [3.0, 6.0, 8.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 3.0], [6.0, 6.0, 8.0], [3.0, 6.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0], [4.0]]),self.functionspace)
      arg[:2,2,1:2]=val
      ref=Data(numpy.array([[[3.0, 7.0, 3.0], [5.0, 2.0, 2.0], [4.0, 3.0, 5.0], [7.0, 5.0, 2.0]], [[7.0, 7.0, 1.0], [2.0, 1.0, 
1.0], [7.0, 4.0, 6.0], [3.0, 6.0, 8.0]], [[2.0, 8.0, 5.0], [8.0, 4.0, 3.0], [6.0, 6.0, 8.0], [3.0, 6.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UPR(self):
      arg=Data(numpy.array([[[6.0, 5.0, 5.0], [8.0, 8.0, 4.0], [4.0, 3.0, 8.0], [1.0, 2.0, 5.0]], [[3.0, 6.0, 8.0], [6.0, 4.0, 
1.0], [8.0, 7.0, 8.0], [1.0, 7.0, 4.0]], [[5.0, 7.0, 7.0], [7.0, 1.0, 4.0], [5.0, 1.0, 2.0], [2.0, 4.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0], [7.0]])+(1.-msk_val)*numpy.array([[6.0], [1.0]])
      arg[:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 5.0], [8.0, 8.0, 4.0], [4.0, 2.0, 8.0], [1.0, 2.0, 5.0]], [[3.0, 6.0, 8.0], [6.0, 
4.0, 1.0], [8.0, 7.0, 8.0], [1.0, 7.0, 4.0]], [[5.0, 7.0, 7.0], [7.0, 1.0, 4.0], [5.0, 1.0, 2.0], [2.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 5.0], [8.0, 8.0, 4.0], [4.0, 6.0, 8.0], [1.0, 2.0, 5.0]], [[3.0, 6.0, 8.0], [6.0, 
4.0, 1.0], [8.0, 1.0, 8.0], [1.0, 7.0, 4.0]], [[5.0, 7.0, 7.0], [7.0, 1.0, 4.0], [5.0, 1.0, 2.0], [2.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PRC(self):
      arg=Data(numpy.array([[[2.0, 2.0, 6.0], [7.0, 2.0, 8.0], [2.0, 4.0, 7.0], [1.0, 6.0, 3.0]], [[5.0, 8.0, 6.0], [8.0, 2.0, 
4.0], [5.0, 2.0, 3.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 5.0], [4.0, 1.0, 4.0], [4.0, 8.0, 5.0], [5.0, 4.0, 
8.0]]]),self.functionspace)
      val=numpy.array([[4.0, 6.0, 8.0], [6.0, 1.0, 7.0]])
      arg[1,1:3,:]=val
      ref=Data(numpy.array([[[2.0, 2.0, 6.0], [7.0, 2.0, 8.0], [2.0, 4.0, 7.0], [1.0, 6.0, 3.0]], [[5.0, 8.0, 6.0], [4.0, 6.0, 
8.0], [6.0, 1.0, 7.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 5.0], [4.0, 1.0, 4.0], [4.0, 8.0, 5.0], [5.0, 4.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PRC(self):
      arg=Data(numpy.array([[[8.0, 5.0, 7.0], [3.0, 2.0, 8.0], [3.0, 6.0, 4.0], [8.0, 4.0, 8.0]], [[6.0, 3.0, 4.0], [6.0, 4.0, 
4.0], [2.0, 3.0, 6.0], [3.0, 4.0, 1.0]], [[7.0, 1.0, 1.0], [8.0, 7.0, 1.0], [5.0, 6.0, 1.0], [7.0, 6.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 1.0, 3.0], [1.0, 1.0, 1.0]]),self.functionspace)
      arg[1,1:3,:]=val
      ref=Data(numpy.array([[[8.0, 5.0, 7.0], [3.0, 2.0, 8.0], [3.0, 6.0, 4.0], [8.0, 4.0, 8.0]], [[6.0, 3.0, 4.0], [1.0, 1.0, 
3.0], [1.0, 1.0, 1.0], [3.0, 4.0, 1.0]], [[7.0, 1.0, 1.0], [8.0, 7.0, 1.0], [5.0, 6.0, 1.0], [7.0, 6.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PRC(self):
      arg=Data(numpy.array([[[7.0, 8.0, 2.0], [8.0, 8.0, 1.0], [2.0, 1.0, 8.0], [5.0, 1.0, 8.0]], [[4.0, 6.0, 3.0], [2.0, 4.0, 
2.0], [5.0, 5.0, 2.0], [7.0, 5.0, 4.0]], [[7.0, 4.0, 1.0], [1.0, 8.0, 8.0], [7.0, 3.0, 7.0], [1.0, 8.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 7.0, 1.0], [4.0, 3.0, 1.0]])+(1.-msk_val)*numpy.array([[8.0, 5.0, 8.0], [5.0, 5.0, 3.0]])
      arg[1,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 2.0], [8.0, 8.0, 1.0], [2.0, 1.0, 8.0], [5.0, 1.0, 8.0]], [[4.0, 6.0, 3.0], [8.0, 
7.0, 1.0], [4.0, 3.0, 1.0], [7.0, 5.0, 4.0]], [[7.0, 4.0, 1.0], [1.0, 8.0, 8.0], [7.0, 3.0, 7.0], [1.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 2.0], [8.0, 8.0, 1.0], [2.0, 1.0, 8.0], [5.0, 1.0, 8.0]], [[4.0, 6.0, 3.0], [8.0, 
5.0, 8.0], [5.0, 5.0, 3.0], [7.0, 5.0, 4.0]], [[7.0, 4.0, 1.0], [1.0, 8.0, 8.0], [7.0, 3.0, 7.0], [1.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RLN(self):
      arg=Data(numpy.array([[[7.0, 6.0, 7.0], [4.0, 3.0, 1.0], [1.0, 8.0, 7.0], [7.0, 4.0, 4.0]], [[1.0, 8.0, 6.0], [7.0, 6.0, 
1.0], [8.0, 6.0, 3.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 6.0], [8.0, 1.0, 5.0], [6.0, 3.0, 5.0], [1.0, 2.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[3.0, 3.0, 7.0], [7.0, 5.0, 5.0]]])
      arg[1:2,2:]=val
      ref=Data(numpy.array([[[7.0, 6.0, 7.0], [4.0, 3.0, 1.0], [1.0, 8.0, 7.0], [7.0, 4.0, 4.0]], [[1.0, 8.0, 6.0], [7.0, 6.0, 
1.0], [3.0, 3.0, 7.0], [7.0, 5.0, 5.0]], [[3.0, 5.0, 6.0], [8.0, 1.0, 5.0], [6.0, 3.0, 5.0], [1.0, 2.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RLN(self):
      arg=Data(numpy.array([[[2.0, 7.0, 8.0], [1.0, 5.0, 1.0], [7.0, 1.0, 7.0], [4.0, 4.0, 1.0]], [[7.0, 4.0, 2.0], [8.0, 1.0, 
3.0], [5.0, 1.0, 7.0], [5.0, 7.0, 2.0]], [[3.0, 2.0, 4.0], [4.0, 3.0, 3.0], [7.0, 8.0, 8.0], [2.0, 5.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 1.0, 1.0], [7.0, 7.0, 4.0]]]),self.functionspace)
      arg[1:2,2:]=val
      ref=Data(numpy.array([[[2.0, 7.0, 8.0], [1.0, 5.0, 1.0], [7.0, 1.0, 7.0], [4.0, 4.0, 1.0]], [[7.0, 4.0, 2.0], [8.0, 1.0, 
3.0], [1.0, 1.0, 1.0], [7.0, 7.0, 4.0]], [[3.0, 2.0, 4.0], [4.0, 3.0, 3.0], [7.0, 8.0, 8.0], [2.0, 5.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RLN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 1.0], [1.0, 3.0, 7.0], [4.0, 7.0, 4.0], [7.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [3.0, 4.0, 
2.0], [8.0, 3.0, 5.0], [3.0, 3.0, 7.0]], [[8.0, 1.0, 3.0], [7.0, 8.0, 5.0], [4.0, 6.0, 3.0], [5.0, 7.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 7.0, 2.0], [2.0, 4.0, 1.0]]])+(1.-msk_val)*numpy.array([[[7.0, 5.0, 4.0], [4.0, 4.0, 
8.0]]])
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 1.0], [1.0, 3.0, 7.0], [4.0, 7.0, 4.0], [7.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [3.0, 
4.0, 2.0], [6.0, 7.0, 2.0], [2.0, 4.0, 1.0]], [[8.0, 1.0, 3.0], [7.0, 8.0, 5.0], [4.0, 6.0, 3.0], [5.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 1.0], [1.0, 3.0, 7.0], [4.0, 7.0, 4.0], [7.0, 3.0, 5.0]], [[6.0, 8.0, 5.0], [3.0, 
4.0, 2.0], [7.0, 5.0, 4.0], [4.0, 4.0, 8.0]], [[8.0, 1.0, 3.0], [7.0, 8.0, 5.0], [4.0, 6.0, 3.0], [5.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UCL(self):
      arg=Data(numpy.array([[[5.0, 7.0, 6.0], [3.0, 2.0, 8.0], [3.0, 8.0, 7.0], [7.0, 1.0, 2.0]], [[6.0, 2.0, 1.0], [7.0, 5.0, 
7.0], [8.0, 4.0, 6.0], [8.0, 7.0, 1.0]], [[1.0, 5.0, 1.0], [3.0, 7.0, 4.0], [5.0, 8.0, 8.0], [8.0, 7.0, 
8.0]]]),self.functionspace)
      val=numpy.array([[[8.0], [5.0], [3.0], [6.0]], [[2.0], [7.0], [7.0], [1.0]]])
      arg[:2,:,2:]=val
      ref=Data(numpy.array([[[5.0, 7.0, 8.0], [3.0, 2.0, 5.0], [3.0, 8.0, 3.0], [7.0, 1.0, 6.0]], [[6.0, 2.0, 2.0], [7.0, 5.0, 
7.0], [8.0, 4.0, 7.0], [8.0, 7.0, 1.0]], [[1.0, 5.0, 1.0], [3.0, 7.0, 4.0], [5.0, 8.0, 8.0], [8.0, 7.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UCL(self):
      arg=Data(numpy.array([[[8.0, 6.0, 4.0], [4.0, 3.0, 4.0], [4.0, 7.0, 3.0], [6.0, 6.0, 4.0]], [[6.0, 6.0, 6.0], [4.0, 8.0, 
5.0], [2.0, 3.0, 1.0], [4.0, 6.0, 7.0]], [[3.0, 8.0, 7.0], [3.0, 8.0, 8.0], [1.0, 5.0, 4.0], [3.0, 5.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0], [1.0], [6.0], [5.0]], [[6.0], [2.0], [4.0], [2.0]]]),self.functionspace)
      arg[:2,:,2:]=val
      ref=Data(numpy.array([[[8.0, 6.0, 1.0], [4.0, 3.0, 1.0], [4.0, 7.0, 6.0], [6.0, 6.0, 5.0]], [[6.0, 6.0, 6.0], [4.0, 8.0, 
2.0], [2.0, 3.0, 4.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 8.0, 8.0], [1.0, 5.0, 4.0], [3.0, 5.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UCL(self):
      arg=Data(numpy.array([[[5.0, 2.0, 5.0], [4.0, 5.0, 7.0], [6.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[5.0, 8.0, 5.0], [6.0, 2.0, 
1.0], [3.0, 6.0, 2.0], [8.0, 5.0, 2.0]], [[4.0, 8.0, 5.0], [5.0, 8.0, 7.0], [1.0, 1.0, 7.0], [2.0, 8.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0], [3.0], [7.0], [5.0]], [[1.0], [2.0], [8.0], [1.0]]])+(1.-msk_val)*numpy.array([[[7.0], 
[4.0], [5.0], [8.0]], [[5.0], [3.0], [2.0], [3.0]]])
      arg[:2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 4.0], [4.0, 5.0, 3.0], [6.0, 3.0, 7.0], [8.0, 2.0, 5.0]], [[5.0, 8.0, 1.0], [6.0, 
2.0, 2.0], [3.0, 6.0, 8.0], [8.0, 5.0, 1.0]], [[4.0, 8.0, 5.0], [5.0, 8.0, 7.0], [1.0, 1.0, 7.0], [2.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 7.0], [4.0, 5.0, 4.0], [6.0, 3.0, 5.0], [8.0, 2.0, 8.0]], [[5.0, 8.0, 5.0], [6.0, 
2.0, 3.0], [3.0, 6.0, 2.0], [8.0, 5.0, 3.0]], [[4.0, 8.0, 5.0], [5.0, 8.0, 7.0], [1.0, 1.0, 7.0], [2.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CLP(self):
      arg=Data(numpy.array([[[8.0, 7.0, 3.0], [2.0, 5.0, 3.0], [5.0, 2.0, 5.0], [3.0, 1.0, 6.0]], [[6.0, 1.0, 6.0], [1.0, 2.0, 
3.0], [7.0, 8.0, 1.0], [5.0, 6.0, 7.0]], [[4.0, 2.0, 7.0], [3.0, 4.0, 5.0], [4.0, 2.0, 6.0], [5.0, 6.0, 
3.0]]]),self.functionspace)
      val=numpy.array([[5.0, 4.0], [5.0, 8.0], [7.0, 3.0]])
      arg[:,2:,1]=val
      ref=Data(numpy.array([[[8.0, 7.0, 3.0], [2.0, 5.0, 3.0], [5.0, 5.0, 5.0], [3.0, 4.0, 6.0]], [[6.0, 1.0, 6.0], [1.0, 2.0, 
3.0], [7.0, 5.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 2.0, 7.0], [3.0, 4.0, 5.0], [4.0, 7.0, 6.0], [5.0, 3.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CLP(self):
      arg=Data(numpy.array([[[4.0, 2.0, 7.0], [7.0, 5.0, 5.0], [7.0, 4.0, 6.0], [8.0, 7.0, 8.0]], [[2.0, 5.0, 5.0], [8.0, 8.0, 
7.0], [3.0, 5.0, 8.0], [2.0, 1.0, 8.0]], [[7.0, 8.0, 1.0], [7.0, 4.0, 6.0], [2.0, 3.0, 8.0], [2.0, 1.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 1.0], [7.0, 6.0], [1.0, 8.0]]),self.functionspace)
      arg[:,2:,1]=val
      ref=Data(numpy.array([[[4.0, 2.0, 7.0], [7.0, 5.0, 5.0], [7.0, 1.0, 6.0], [8.0, 1.0, 8.0]], [[2.0, 5.0, 5.0], [8.0, 8.0, 
7.0], [3.0, 7.0, 8.0], [2.0, 6.0, 8.0]], [[7.0, 8.0, 1.0], [7.0, 4.0, 6.0], [2.0, 1.0, 8.0], [2.0, 8.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CLP(self):
      arg=Data(numpy.array([[[7.0, 1.0, 5.0], [2.0, 4.0, 7.0], [1.0, 1.0, 2.0], [2.0, 3.0, 5.0]], [[4.0, 1.0, 7.0], [3.0, 7.0, 
6.0], [7.0, 8.0, 6.0], [2.0, 8.0, 5.0]], [[7.0, 6.0, 6.0], [5.0, 8.0, 1.0], [5.0, 4.0, 3.0], [2.0, 3.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 6.0], [6.0, 7.0], [5.0, 5.0]])+(1.-msk_val)*numpy.array([[3.0, 3.0], [1.0, 8.0], [8.0, 
1.0]])
      arg[:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 5.0], [2.0, 4.0, 7.0], [1.0, 3.0, 2.0], [2.0, 6.0, 5.0]], [[4.0, 1.0, 7.0], [3.0, 
7.0, 6.0], [7.0, 6.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 6.0, 6.0], [5.0, 8.0, 1.0], [5.0, 5.0, 3.0], [2.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 5.0], [2.0, 4.0, 7.0], [1.0, 3.0, 2.0], [2.0, 3.0, 5.0]], [[4.0, 1.0, 7.0], [3.0, 
7.0, 6.0], [7.0, 1.0, 6.0], [2.0, 8.0, 5.0]], [[7.0, 6.0, 6.0], [5.0, 8.0, 1.0], [5.0, 8.0, 3.0], [2.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PRL(self):
      arg=Data(numpy.array([[[8.0, 1.0, 5.0], [7.0, 6.0, 7.0], [2.0, 6.0, 8.0], [4.0, 2.0, 2.0]], [[4.0, 8.0, 6.0], [7.0, 3.0, 
4.0], [7.0, 8.0, 6.0], [1.0, 3.0, 1.0]], [[1.0, 6.0, 7.0], [7.0, 8.0, 3.0], [4.0, 1.0, 7.0], [4.0, 3.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[3.0], [4.0]])
      arg[1,1:3,2:]=val
      ref=Data(numpy.array([[[8.0, 1.0, 5.0], [7.0, 6.0, 7.0], [2.0, 6.0, 8.0], [4.0, 2.0, 2.0]], [[4.0, 8.0, 6.0], [7.0, 3.0, 
3.0], [7.0, 8.0, 4.0], [1.0, 3.0, 1.0]], [[1.0, 6.0, 7.0], [7.0, 8.0, 3.0], [4.0, 1.0, 7.0], [4.0, 3.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PRL(self):
      arg=Data(numpy.array([[[8.0, 7.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 7.0], [2.0, 2.0, 6.0]], [[2.0, 7.0, 2.0], [7.0, 1.0, 
1.0], [5.0, 7.0, 2.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 5.0], [1.0, 1.0, 2.0], [4.0, 3.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[3.0], [4.0]]),self.functionspace)
      arg[1,1:3,2:]=val
      ref=Data(numpy.array([[[8.0, 7.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 7.0], [2.0, 2.0, 6.0]], [[2.0, 7.0, 2.0], [7.0, 1.0, 
3.0], [5.0, 7.0, 4.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 3.0], [1.0, 7.0, 5.0], [1.0, 1.0, 2.0], [4.0, 3.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PRL(self):
      arg=Data(numpy.array([[[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [8.0, 3.0, 7.0], [2.0, 8.0, 5.0]], [[6.0, 7.0, 6.0], [6.0, 1.0, 
4.0], [4.0, 6.0, 3.0], [8.0, 4.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 1.0, 5.0], [2.0, 4.0, 1.0], [1.0, 4.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0], [8.0]])+(1.-msk_val)*numpy.array([[6.0], [7.0]])
      arg[1,1:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [8.0, 3.0, 7.0], [2.0, 8.0, 5.0]], [[6.0, 7.0, 6.0], [6.0, 
1.0, 1.0], [4.0, 6.0, 8.0], [8.0, 4.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 1.0, 5.0], [2.0, 4.0, 1.0], [1.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [8.0, 3.0, 7.0], [2.0, 8.0, 5.0]], [[6.0, 7.0, 6.0], [6.0, 
1.0, 6.0], [4.0, 6.0, 7.0], [8.0, 4.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 1.0, 5.0], [2.0, 4.0, 1.0], [1.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RLL(self):
      arg=Data(numpy.array([[[8.0, 5.0, 4.0], [5.0, 5.0, 2.0], [5.0, 2.0, 3.0], [5.0, 5.0, 3.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 
8.0], [7.0, 5.0, 7.0], [3.0, 6.0, 7.0]], [[8.0, 5.0, 6.0], [5.0, 3.0, 5.0], [3.0, 2.0, 4.0], [8.0, 7.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[8.0], [4.0]]])
      arg[1:2,2:,2:]=val
      ref=Data(numpy.array([[[8.0, 5.0, 4.0], [5.0, 5.0, 2.0], [5.0, 2.0, 3.0], [5.0, 5.0, 3.0]], [[3.0, 6.0, 7.0], [1.0, 7.0, 
8.0], [7.0, 5.0, 8.0], [3.0, 6.0, 4.0]], [[8.0, 5.0, 6.0], [5.0, 3.0, 5.0], [3.0, 2.0, 4.0], [8.0, 7.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RLL(self):
      arg=Data(numpy.array([[[8.0, 5.0, 6.0], [3.0, 5.0, 4.0], [2.0, 5.0, 2.0], [6.0, 1.0, 4.0]], [[7.0, 6.0, 2.0], [7.0, 6.0, 
8.0], [8.0, 8.0, 7.0], [5.0, 1.0, 4.0]], [[6.0, 6.0, 2.0], [2.0, 6.0, 6.0], [6.0, 7.0, 5.0], [8.0, 1.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0], [1.0]]]),self.functionspace)
      arg[1:2,2:,2:]=val
      ref=Data(numpy.array([[[8.0, 5.0, 6.0], [3.0, 5.0, 4.0], [2.0, 5.0, 2.0], [6.0, 1.0, 4.0]], [[7.0, 6.0, 2.0], [7.0, 6.0, 
8.0], [8.0, 8.0, 6.0], [5.0, 1.0, 1.0]], [[6.0, 6.0, 2.0], [2.0, 6.0, 6.0], [6.0, 7.0, 5.0], [8.0, 1.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RLL(self):
      arg=Data(numpy.array([[[1.0, 2.0, 7.0], [4.0, 6.0, 7.0], [5.0, 6.0, 5.0], [1.0, 3.0, 6.0]], [[7.0, 1.0, 1.0], [8.0, 2.0, 
4.0], [3.0, 8.0, 7.0], [1.0, 3.0, 2.0]], [[7.0, 1.0, 2.0], [8.0, 6.0, 5.0], [2.0, 1.0, 1.0], [3.0, 6.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0], [5.0]]])+(1.-msk_val)*numpy.array([[[1.0], [7.0]]])
      arg[1:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 7.0], [4.0, 6.0, 7.0], [5.0, 6.0, 5.0], [1.0, 3.0, 6.0]], [[7.0, 1.0, 1.0], [8.0, 
2.0, 4.0], [3.0, 8.0, 1.0], [1.0, 3.0, 5.0]], [[7.0, 1.0, 2.0], [8.0, 6.0, 5.0], [2.0, 1.0, 1.0], [3.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 7.0], [4.0, 6.0, 7.0], [5.0, 6.0, 5.0], [1.0, 3.0, 6.0]], [[7.0, 1.0, 1.0], [8.0, 
2.0, 4.0], [3.0, 8.0, 1.0], [1.0, 3.0, 7.0]], [[7.0, 1.0, 2.0], [8.0, 6.0, 5.0], [2.0, 1.0, 1.0], [3.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PPL(self):
      arg=Data(numpy.array([[[4.0, 8.0, 3.0], [8.0, 6.0, 3.0], [4.0, 6.0, 4.0], [2.0, 6.0, 3.0]], [[7.0, 6.0, 2.0], [2.0, 5.0, 
8.0], [6.0, 4.0, 8.0], [5.0, 5.0, 7.0]], [[3.0, 8.0, 1.0], [8.0, 4.0, 5.0], [8.0, 6.0, 5.0], [2.0, 6.0, 
1.0]]]),self.functionspace)
      val=numpy.array([4.0])
      arg[1,2,2:]=val
      ref=Data(numpy.array([[[4.0, 8.0, 3.0], [8.0, 6.0, 3.0], [4.0, 6.0, 4.0], [2.0, 6.0, 3.0]], [[7.0, 6.0, 2.0], [2.0, 5.0, 
8.0], [6.0, 4.0, 4.0], [5.0, 5.0, 7.0]], [[3.0, 8.0, 1.0], [8.0, 4.0, 5.0], [8.0, 6.0, 5.0], [2.0, 6.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PPL(self):
      arg=Data(numpy.array([[[7.0, 5.0, 7.0], [3.0, 4.0, 4.0], [6.0, 6.0, 4.0], [1.0, 5.0, 8.0]], [[7.0, 2.0, 7.0], [7.0, 4.0, 
4.0], [1.0, 1.0, 8.0], [7.0, 1.0, 4.0]], [[6.0, 5.0, 5.0], [1.0, 1.0, 5.0], [2.0, 6.0, 3.0], [8.0, 6.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([1.0]),self.functionspace)
      arg[1,2,2:]=val
      ref=Data(numpy.array([[[7.0, 5.0, 7.0], [3.0, 4.0, 4.0], [6.0, 6.0, 4.0], [1.0, 5.0, 8.0]], [[7.0, 2.0, 7.0], [7.0, 4.0, 
4.0], [1.0, 1.0, 1.0], [7.0, 1.0, 4.0]], [[6.0, 5.0, 5.0], [1.0, 1.0, 5.0], [2.0, 6.0, 3.0], [8.0, 6.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PPL(self):
      arg=Data(numpy.array([[[4.0, 2.0, 1.0], [1.0, 6.0, 7.0], [1.0, 2.0, 7.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 7.0], [4.0, 1.0, 
3.0], [7.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[7.0, 4.0, 1.0], [5.0, 5.0, 8.0], [2.0, 6.0, 3.0], [4.0, 4.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([3.0])+(1.-msk_val)*numpy.array([4.0])
      arg[1,2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 1.0], [1.0, 6.0, 7.0], [1.0, 2.0, 7.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 7.0], [4.0, 
1.0, 3.0], [7.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[7.0, 4.0, 1.0], [5.0, 5.0, 8.0], [2.0, 6.0, 3.0], [4.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 1.0], [1.0, 6.0, 7.0], [1.0, 2.0, 7.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 7.0], [4.0, 
1.0, 3.0], [7.0, 7.0, 4.0], [1.0, 3.0, 8.0]], [[7.0, 4.0, 1.0], [5.0, 5.0, 8.0], [2.0, 6.0, 3.0], [4.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RPN(self):
      arg=Data(numpy.array([[[5.0, 7.0, 7.0], [4.0, 2.0, 6.0], [8.0, 5.0, 7.0], [6.0, 6.0, 4.0]], [[3.0, 5.0, 1.0], [1.0, 8.0, 
6.0], [2.0, 2.0, 4.0], [1.0, 1.0, 2.0]], [[1.0, 5.0, 3.0], [3.0, 1.0, 6.0], [8.0, 3.0, 7.0], [7.0, 1.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[5.0, 5.0, 6.0]])
      arg[1:2,2]=val
      ref=Data(numpy.array([[[5.0, 7.0, 7.0], [4.0, 2.0, 6.0], [8.0, 5.0, 7.0], [6.0, 6.0, 4.0]], [[3.0, 5.0, 1.0], [1.0, 8.0, 
6.0], [5.0, 5.0, 6.0], [1.0, 1.0, 2.0]], [[1.0, 5.0, 3.0], [3.0, 1.0, 6.0], [8.0, 3.0, 7.0], [7.0, 1.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RPN(self):
      arg=Data(numpy.array([[[2.0, 4.0, 4.0], [1.0, 4.0, 7.0], [3.0, 6.0, 2.0], [6.0, 4.0, 7.0]], [[6.0, 4.0, 5.0], [8.0, 7.0, 
3.0], [6.0, 6.0, 3.0], [5.0, 5.0, 3.0]], [[5.0, 5.0, 4.0], [5.0, 6.0, 1.0], [2.0, 5.0, 3.0], [7.0, 8.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 2.0, 1.0]]),self.functionspace)
      arg[1:2,2]=val
      ref=Data(numpy.array([[[2.0, 4.0, 4.0], [1.0, 4.0, 7.0], [3.0, 6.0, 2.0], [6.0, 4.0, 7.0]], [[6.0, 4.0, 5.0], [8.0, 7.0, 
3.0], [8.0, 2.0, 1.0], [5.0, 5.0, 3.0]], [[5.0, 5.0, 4.0], [5.0, 6.0, 1.0], [2.0, 5.0, 3.0], [7.0, 8.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RPN(self):
      arg=Data(numpy.array([[[1.0, 4.0, 5.0], [2.0, 2.0, 2.0], [6.0, 2.0, 1.0], [1.0, 2.0, 1.0]], [[5.0, 2.0, 3.0], [1.0, 5.0, 
4.0], [5.0, 5.0, 7.0], [7.0, 7.0, 8.0]], [[7.0, 3.0, 7.0], [5.0, 2.0, 5.0], [7.0, 1.0, 1.0], [7.0, 8.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 7.0, 2.0]])+(1.-msk_val)*numpy.array([[5.0, 1.0, 5.0]])
      arg[1:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 5.0], [2.0, 2.0, 2.0], [6.0, 2.0, 1.0], [1.0, 2.0, 1.0]], [[5.0, 2.0, 3.0], [1.0, 
5.0, 4.0], [7.0, 7.0, 2.0], [7.0, 7.0, 8.0]], [[7.0, 3.0, 7.0], [5.0, 2.0, 5.0], [7.0, 1.0, 1.0], [7.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 5.0], [2.0, 2.0, 2.0], [6.0, 2.0, 1.0], [1.0, 2.0, 1.0]], [[5.0, 2.0, 3.0], [1.0, 
5.0, 4.0], [5.0, 1.0, 5.0], [7.0, 7.0, 8.0]], [[7.0, 3.0, 7.0], [5.0, 2.0, 5.0], [7.0, 1.0, 1.0], [7.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CUR(self):
      arg=Data(numpy.array([[[3.0, 5.0, 2.0], [4.0, 4.0, 7.0], [3.0, 3.0, 4.0], [7.0, 3.0, 4.0]], [[1.0, 8.0, 7.0], [6.0, 1.0, 
6.0], [2.0, 8.0, 3.0], [1.0, 4.0, 5.0]], [[8.0, 3.0, 7.0], [4.0, 6.0, 6.0], [3.0, 8.0, 3.0], [3.0, 2.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[5.0], [2.0], [4.0]], [[5.0], [3.0], [6.0]], [[3.0], [8.0], [2.0]]])
      arg[:,:3,1:2]=val
      ref=Data(numpy.array([[[3.0, 5.0, 2.0], [4.0, 2.0, 7.0], [3.0, 4.0, 4.0], [7.0, 3.0, 4.0]], [[1.0, 5.0, 7.0], [6.0, 3.0, 
6.0], [2.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[8.0, 3.0, 7.0], [4.0, 8.0, 6.0], [3.0, 2.0, 3.0], [3.0, 2.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CUR(self):
      arg=Data(numpy.array([[[4.0, 2.0, 2.0], [8.0, 1.0, 4.0], [8.0, 3.0, 6.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 2.0], [5.0, 8.0, 
3.0], [4.0, 5.0, 2.0], [1.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [8.0, 7.0, 1.0], [2.0, 6.0, 5.0], [4.0, 7.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0], [8.0], [7.0]], [[3.0], [1.0], [3.0]], [[6.0], [7.0], [5.0]]]),self.functionspace)
      arg[:,:3,1:2]=val
      ref=Data(numpy.array([[[4.0, 6.0, 2.0], [8.0, 8.0, 4.0], [8.0, 7.0, 6.0], [3.0, 8.0, 2.0]], [[3.0, 3.0, 2.0], [5.0, 1.0, 
3.0], [4.0, 3.0, 2.0], [1.0, 2.0, 5.0]], [[7.0, 6.0, 2.0], [8.0, 7.0, 1.0], [2.0, 5.0, 5.0], [4.0, 7.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CUR(self):
      arg=Data(numpy.array([[[1.0, 5.0, 4.0], [8.0, 2.0, 3.0], [1.0, 3.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 8.0, 2.0], [4.0, 8.0, 
8.0], [3.0, 6.0, 7.0], [2.0, 5.0, 2.0]], [[5.0, 2.0, 4.0], [6.0, 8.0, 2.0], [3.0, 2.0, 5.0], [5.0, 1.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0], [7.0], [7.0]], [[1.0], [3.0], [2.0]], [[2.0], [7.0], 
[8.0]]])+(1.-msk_val)*numpy.array([[[5.0], [5.0], [2.0]], [[3.0], [1.0], [7.0]], [[3.0], [3.0], [4.0]]])
      arg[:,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 4.0], [8.0, 7.0, 3.0], [1.0, 7.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 1.0, 2.0], [4.0, 
3.0, 8.0], [3.0, 2.0, 7.0], [2.0, 5.0, 2.0]], [[5.0, 2.0, 4.0], [6.0, 7.0, 2.0], [3.0, 8.0, 5.0], [5.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 4.0], [8.0, 5.0, 3.0], [1.0, 2.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 3.0, 2.0], [4.0, 
1.0, 8.0], [3.0, 7.0, 7.0], [2.0, 5.0, 2.0]], [[5.0, 3.0, 4.0], [6.0, 3.0, 2.0], [3.0, 4.0, 5.0], [5.0, 1.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LRN(self):
      arg=Data(numpy.array([[[8.0, 6.0, 1.0], [2.0, 4.0, 4.0], [3.0, 5.0, 3.0], [5.0, 8.0, 6.0]], [[1.0, 8.0, 4.0], [7.0, 3.0, 
1.0], [5.0, 8.0, 6.0], [2.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [1.0, 8.0, 6.0], [6.0, 4.0, 1.0], [7.0, 4.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[7.0, 1.0, 1.0], [8.0, 8.0, 1.0]]])
      arg[2:,1:3]=val
      ref=Data(numpy.array([[[8.0, 6.0, 1.0], [2.0, 4.0, 4.0], [3.0, 5.0, 3.0], [5.0, 8.0, 6.0]], [[1.0, 8.0, 4.0], [7.0, 3.0, 
1.0], [5.0, 8.0, 6.0], [2.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [7.0, 1.0, 1.0], [8.0, 8.0, 1.0], [7.0, 4.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LRN(self):
      arg=Data(numpy.array([[[1.0, 3.0, 8.0], [6.0, 4.0, 5.0], [2.0, 4.0, 8.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 4.0], [5.0, 3.0, 
8.0], [8.0, 5.0, 1.0], [6.0, 6.0, 3.0]], [[4.0, 4.0, 8.0], [5.0, 6.0, 4.0], [1.0, 1.0, 2.0], [2.0, 3.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 6.0, 7.0], [2.0, 4.0, 3.0]]]),self.functionspace)
      arg[2:,1:3]=val
      ref=Data(numpy.array([[[1.0, 3.0, 8.0], [6.0, 4.0, 5.0], [2.0, 4.0, 8.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 4.0], [5.0, 3.0, 
8.0], [8.0, 5.0, 1.0], [6.0, 6.0, 3.0]], [[4.0, 4.0, 8.0], [2.0, 6.0, 7.0], [2.0, 4.0, 3.0], [2.0, 3.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LRN(self):
      arg=Data(numpy.array([[[5.0, 1.0, 3.0], [5.0, 3.0, 1.0], [5.0, 5.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 8.0, 
6.0], [5.0, 2.0, 8.0], [5.0, 3.0, 6.0]], [[4.0, 1.0, 8.0], [2.0, 2.0, 7.0], [4.0, 1.0, 7.0], [8.0, 4.0, 
6.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 7.0, 5.0], [7.0, 2.0, 1.0]]])+(1.-msk_val)*numpy.array([[[4.0, 1.0, 6.0], [5.0, 3.0, 
6.0]]])
      arg[2:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 3.0], [5.0, 3.0, 1.0], [5.0, 5.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 
8.0, 6.0], [5.0, 2.0, 8.0], [5.0, 3.0, 6.0]], [[4.0, 1.0, 8.0], [3.0, 7.0, 5.0], [7.0, 2.0, 1.0], [8.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 3.0], [5.0, 3.0, 1.0], [5.0, 5.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 1.0], [4.0, 
8.0, 6.0], [5.0, 2.0, 8.0], [5.0, 3.0, 6.0]], [[4.0, 1.0, 8.0], [4.0, 1.0, 6.0], [5.0, 3.0, 6.0], [8.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PCN(self):
      arg=Data(numpy.array([[[6.0, 3.0, 8.0], [3.0, 7.0, 1.0], [5.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[1.0, 4.0, 1.0], [7.0, 1.0, 
2.0], [3.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 6.0], [3.0, 2.0, 3.0], [3.0, 4.0, 4.0], [5.0, 7.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[8.0, 8.0, 2.0], [5.0, 6.0, 1.0], [7.0, 4.0, 7.0], [8.0, 7.0, 2.0]])
      arg[1,:]=val
      ref=Data(numpy.array([[[6.0, 3.0, 8.0], [3.0, 7.0, 1.0], [5.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[8.0, 8.0, 2.0], [5.0, 6.0, 
1.0], [7.0, 4.0, 7.0], [8.0, 7.0, 2.0]], [[3.0, 8.0, 6.0], [3.0, 2.0, 3.0], [3.0, 4.0, 4.0], [5.0, 7.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PCN(self):
      arg=Data(numpy.array([[[8.0, 7.0, 1.0], [4.0, 3.0, 1.0], [6.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[7.0, 6.0, 3.0], [6.0, 3.0, 
5.0], [6.0, 4.0, 6.0], [8.0, 1.0, 6.0]], [[7.0, 5.0, 7.0], [1.0, 4.0, 6.0], [5.0, 8.0, 8.0], [7.0, 7.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 4.0, 7.0], [8.0, 2.0, 3.0], [5.0, 8.0, 8.0], [4.0, 6.0, 6.0]]),self.functionspace)
      arg[1,:]=val
      ref=Data(numpy.array([[[8.0, 7.0, 1.0], [4.0, 3.0, 1.0], [6.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[2.0, 4.0, 7.0], [8.0, 2.0, 
3.0], [5.0, 8.0, 8.0], [4.0, 6.0, 6.0]], [[7.0, 5.0, 7.0], [1.0, 4.0, 6.0], [5.0, 8.0, 8.0], [7.0, 7.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PCN(self):
      arg=Data(numpy.array([[[1.0, 5.0, 4.0], [7.0, 4.0, 6.0], [4.0, 6.0, 2.0], [5.0, 3.0, 4.0]], [[2.0, 1.0, 1.0], [2.0, 7.0, 
7.0], [2.0, 7.0, 5.0], [4.0, 7.0, 5.0]], [[8.0, 1.0, 1.0], [6.0, 1.0, 5.0], [7.0, 1.0, 2.0], [7.0, 5.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 4.0, 6.0], [4.0, 5.0, 7.0], [3.0, 6.0, 4.0], [8.0, 6.0, 
7.0]])+(1.-msk_val)*numpy.array([[6.0, 7.0, 3.0], [6.0, 3.0, 8.0], [2.0, 3.0, 8.0], [4.0, 1.0, 5.0]])
      arg[1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 4.0], [7.0, 4.0, 6.0], [4.0, 6.0, 2.0], [5.0, 3.0, 4.0]], [[6.0, 4.0, 6.0], [4.0, 
5.0, 7.0], [3.0, 6.0, 4.0], [8.0, 6.0, 7.0]], [[8.0, 1.0, 1.0], [6.0, 1.0, 5.0], [7.0, 1.0, 2.0], [7.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 4.0], [7.0, 4.0, 6.0], [4.0, 6.0, 2.0], [5.0, 3.0, 4.0]], [[6.0, 7.0, 3.0], [6.0, 
3.0, 8.0], [2.0, 3.0, 8.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [6.0, 1.0, 5.0], [7.0, 1.0, 2.0], [7.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PRR(self):
      arg=Data(numpy.array([[[4.0, 5.0, 2.0], [6.0, 4.0, 3.0], [5.0, 8.0, 8.0], [2.0, 4.0, 1.0]], [[2.0, 4.0, 5.0], [3.0, 1.0, 
7.0], [5.0, 5.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 4.0, 8.0], [7.0, 3.0, 5.0], [2.0, 7.0, 2.0], [8.0, 2.0, 
6.0]]]),self.functionspace)
      val=numpy.array([[6.0], [6.0]])
      arg[1,1:3,1:2]=val
      ref=Data(numpy.array([[[4.0, 5.0, 2.0], [6.0, 4.0, 3.0], [5.0, 8.0, 8.0], [2.0, 4.0, 1.0]], [[2.0, 4.0, 5.0], [3.0, 6.0, 
7.0], [5.0, 6.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 4.0, 8.0], [7.0, 3.0, 5.0], [2.0, 7.0, 2.0], [8.0, 2.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PRR(self):
      arg=Data(numpy.array([[[3.0, 8.0, 1.0], [8.0, 5.0, 6.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0]], [[4.0, 4.0, 6.0], [7.0, 4.0, 
3.0], [2.0, 1.0, 8.0], [3.0, 3.0, 3.0]], [[4.0, 7.0, 7.0], [3.0, 4.0, 6.0], [4.0, 3.0, 4.0], [7.0, 6.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[8.0], [2.0]]),self.functionspace)
      arg[1,1:3,1:2]=val
      ref=Data(numpy.array([[[3.0, 8.0, 1.0], [8.0, 5.0, 6.0], [3.0, 6.0, 4.0], [8.0, 2.0, 2.0]], [[4.0, 4.0, 6.0], [7.0, 8.0, 
3.0], [2.0, 2.0, 8.0], [3.0, 3.0, 3.0]], [[4.0, 7.0, 7.0], [3.0, 4.0, 6.0], [4.0, 3.0, 4.0], [7.0, 6.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PRR(self):
      arg=Data(numpy.array([[[8.0, 1.0, 1.0], [4.0, 2.0, 3.0], [6.0, 7.0, 7.0], [4.0, 2.0, 4.0]], [[1.0, 3.0, 4.0], [2.0, 5.0, 
4.0], [8.0, 2.0, 5.0], [5.0, 4.0, 3.0]], [[3.0, 6.0, 8.0], [4.0, 2.0, 7.0], [5.0, 5.0, 2.0], [7.0, 4.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0], [5.0]])+(1.-msk_val)*numpy.array([[8.0], [8.0]])
      arg[1,1:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 1.0, 1.0], [4.0, 2.0, 3.0], [6.0, 7.0, 7.0], [4.0, 2.0, 4.0]], [[1.0, 3.0, 4.0], [2.0, 
5.0, 4.0], [8.0, 5.0, 5.0], [5.0, 4.0, 3.0]], [[3.0, 6.0, 8.0], [4.0, 2.0, 7.0], [5.0, 5.0, 2.0], [7.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 1.0], [4.0, 2.0, 3.0], [6.0, 7.0, 7.0], [4.0, 2.0, 4.0]], [[1.0, 3.0, 4.0], [2.0, 
8.0, 4.0], [8.0, 8.0, 5.0], [5.0, 4.0, 3.0]], [[3.0, 6.0, 8.0], [4.0, 2.0, 7.0], [5.0, 5.0, 2.0], [7.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PRN(self):
      arg=Data(numpy.array([[[3.0, 8.0, 1.0], [8.0, 2.0, 4.0], [2.0, 3.0, 5.0], [8.0, 5.0, 3.0]], [[1.0, 8.0, 4.0], [4.0, 6.0, 
7.0], [2.0, 5.0, 7.0], [2.0, 1.0, 8.0]], [[6.0, 6.0, 8.0], [5.0, 4.0, 7.0], [6.0, 3.0, 5.0], [3.0, 3.0, 
3.0]]]),self.functionspace)
      val=numpy.array([[3.0, 8.0, 4.0], [3.0, 6.0, 3.0]])
      arg[1,1:3]=val
      ref=Data(numpy.array([[[3.0, 8.0, 1.0], [8.0, 2.0, 4.0], [2.0, 3.0, 5.0], [8.0, 5.0, 3.0]], [[1.0, 8.0, 4.0], [3.0, 8.0, 
4.0], [3.0, 6.0, 3.0], [2.0, 1.0, 8.0]], [[6.0, 6.0, 8.0], [5.0, 4.0, 7.0], [6.0, 3.0, 5.0], [3.0, 3.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PRN(self):
      arg=Data(numpy.array([[[6.0, 3.0, 6.0], [1.0, 1.0, 1.0], [5.0, 3.0, 3.0], [4.0, 6.0, 6.0]], [[1.0, 1.0, 6.0], [5.0, 4.0, 
7.0], [8.0, 6.0, 3.0], [4.0, 5.0, 6.0]], [[3.0, 1.0, 5.0], [8.0, 6.0, 1.0], [1.0, 7.0, 3.0], [4.0, 3.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[6.0, 2.0, 5.0], [4.0, 2.0, 1.0]]),self.functionspace)
      arg[1,1:3]=val
      ref=Data(numpy.array([[[6.0, 3.0, 6.0], [1.0, 1.0, 1.0], [5.0, 3.0, 3.0], [4.0, 6.0, 6.0]], [[1.0, 1.0, 6.0], [6.0, 2.0, 
5.0], [4.0, 2.0, 1.0], [4.0, 5.0, 6.0]], [[3.0, 1.0, 5.0], [8.0, 6.0, 1.0], [1.0, 7.0, 3.0], [4.0, 3.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PRN(self):
      arg=Data(numpy.array([[[6.0, 3.0, 3.0], [5.0, 7.0, 7.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[6.0, 7.0, 3.0], [2.0, 2.0, 
3.0], [4.0, 3.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 6.0], [1.0, 3.0, 2.0], [7.0, 5.0, 4.0], [1.0, 5.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 7.0, 6.0], [1.0, 5.0, 1.0]])+(1.-msk_val)*numpy.array([[4.0, 7.0, 1.0], [4.0, 2.0, 4.0]])
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 3.0], [5.0, 7.0, 7.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[6.0, 7.0, 3.0], [2.0, 
7.0, 6.0], [1.0, 5.0, 1.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 6.0], [1.0, 3.0, 2.0], [7.0, 5.0, 4.0], [1.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 3.0], [5.0, 7.0, 7.0], [5.0, 1.0, 1.0], [5.0, 6.0, 1.0]], [[6.0, 7.0, 3.0], [4.0, 
7.0, 1.0], [4.0, 2.0, 4.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 6.0], [1.0, 3.0, 2.0], [7.0, 5.0, 4.0], [1.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RCN(self):
      arg=Data(numpy.array([[[1.0, 8.0, 6.0], [5.0, 5.0, 7.0], [2.0, 4.0, 8.0], [2.0, 3.0, 2.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 
8.0], [5.0, 5.0, 2.0], [8.0, 8.0, 8.0]], [[6.0, 8.0, 1.0], [1.0, 4.0, 8.0], [5.0, 6.0, 1.0], [6.0, 2.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[3.0, 3.0, 8.0], [2.0, 8.0, 8.0], [2.0, 3.0, 1.0], [2.0, 6.0, 1.0]]])
      arg[1:2,:]=val
      ref=Data(numpy.array([[[1.0, 8.0, 6.0], [5.0, 5.0, 7.0], [2.0, 4.0, 8.0], [2.0, 3.0, 2.0]], [[3.0, 3.0, 8.0], [2.0, 8.0, 
8.0], [2.0, 3.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 8.0, 1.0], [1.0, 4.0, 8.0], [5.0, 6.0, 1.0], [6.0, 2.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RCN(self):
      arg=Data(numpy.array([[[1.0, 7.0, 7.0], [2.0, 1.0, 2.0], [1.0, 2.0, 6.0], [2.0, 8.0, 6.0]], [[5.0, 6.0, 1.0], [3.0, 4.0, 
8.0], [6.0, 7.0, 4.0], [6.0, 4.0, 4.0]], [[4.0, 1.0, 6.0], [1.0, 6.0, 7.0], [8.0, 4.0, 1.0], [8.0, 4.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 1.0, 7.0], [6.0, 8.0, 6.0], [6.0, 6.0, 7.0], [5.0, 5.0, 8.0]]]),self.functionspace)
      arg[1:2,:]=val
      ref=Data(numpy.array([[[1.0, 7.0, 7.0], [2.0, 1.0, 2.0], [1.0, 2.0, 6.0], [2.0, 8.0, 6.0]], [[5.0, 1.0, 7.0], [6.0, 8.0, 
6.0], [6.0, 6.0, 7.0], [5.0, 5.0, 8.0]], [[4.0, 1.0, 6.0], [1.0, 6.0, 7.0], [8.0, 4.0, 1.0], [8.0, 4.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RCN(self):
      arg=Data(numpy.array([[[5.0, 3.0, 7.0], [5.0, 7.0, 1.0], [3.0, 7.0, 8.0], [2.0, 1.0, 8.0]], [[3.0, 8.0, 8.0], [3.0, 8.0, 
4.0], [4.0, 4.0, 2.0], [3.0, 7.0, 2.0]], [[4.0, 3.0, 3.0], [4.0, 4.0, 7.0], [4.0, 5.0, 8.0], [1.0, 4.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 1.0, 2.0], [3.0, 6.0, 6.0], [8.0, 5.0, 3.0], [2.0, 5.0, 
3.0]]])+(1.-msk_val)*numpy.array([[[3.0, 3.0, 3.0], [8.0, 7.0, 4.0], [1.0, 3.0, 6.0], [1.0, 8.0, 4.0]]])
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 7.0], [5.0, 7.0, 1.0], [3.0, 7.0, 8.0], [2.0, 1.0, 8.0]], [[6.0, 1.0, 2.0], [3.0, 
6.0, 6.0], [8.0, 5.0, 3.0], [2.0, 5.0, 3.0]], [[4.0, 3.0, 3.0], [4.0, 4.0, 7.0], [4.0, 5.0, 8.0], [1.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 7.0], [5.0, 7.0, 1.0], [3.0, 7.0, 8.0], [2.0, 1.0, 8.0]], [[3.0, 3.0, 3.0], [8.0, 
7.0, 4.0], [1.0, 3.0, 6.0], [1.0, 8.0, 4.0]], [[4.0, 3.0, 3.0], [4.0, 4.0, 7.0], [4.0, 5.0, 8.0], [1.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UUN(self):
      arg=Data(numpy.array([[[2.0, 8.0, 2.0], [8.0, 2.0, 4.0], [7.0, 6.0, 6.0], [8.0, 1.0, 3.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 
6.0], [4.0, 5.0, 7.0], [4.0, 2.0, 5.0]], [[4.0, 1.0, 6.0], [2.0, 1.0, 4.0], [3.0, 2.0, 4.0], [3.0, 5.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[8.0, 1.0, 4.0], [4.0, 6.0, 7.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 8.0], [6.0, 3.0, 8.0], [1.0, 2.0, 
1.0]]])
      arg[:2,:3]=val
      ref=Data(numpy.array([[[8.0, 1.0, 4.0], [4.0, 6.0, 7.0], [2.0, 4.0, 2.0], [8.0, 1.0, 3.0]], [[8.0, 8.0, 8.0], [6.0, 3.0, 
8.0], [1.0, 2.0, 1.0], [4.0, 2.0, 5.0]], [[4.0, 1.0, 6.0], [2.0, 1.0, 4.0], [3.0, 2.0, 4.0], [3.0, 5.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UUN(self):
      arg=Data(numpy.array([[[1.0, 8.0, 6.0], [5.0, 2.0, 1.0], [5.0, 3.0, 5.0], [4.0, 1.0, 1.0]], [[3.0, 1.0, 4.0], [1.0, 8.0, 
4.0], [4.0, 8.0, 3.0], [1.0, 2.0, 4.0]], [[4.0, 1.0, 1.0], [6.0, 3.0, 2.0], [7.0, 2.0, 5.0], [1.0, 1.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 6.0, 6.0], [6.0, 6.0, 7.0], [3.0, 5.0, 2.0]], [[5.0, 5.0, 8.0], [1.0, 8.0, 7.0], [5.0, 8.0, 
4.0]]]),self.functionspace)
      arg[:2,:3]=val
      ref=Data(numpy.array([[[1.0, 6.0, 6.0], [6.0, 6.0, 7.0], [3.0, 5.0, 2.0], [4.0, 1.0, 1.0]], [[5.0, 5.0, 8.0], [1.0, 8.0, 
7.0], [5.0, 8.0, 4.0], [1.0, 2.0, 4.0]], [[4.0, 1.0, 1.0], [6.0, 3.0, 2.0], [7.0, 2.0, 5.0], [1.0, 1.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UUN(self):
      arg=Data(numpy.array([[[1.0, 2.0, 1.0], [5.0, 1.0, 4.0], [2.0, 3.0, 5.0], [5.0, 8.0, 4.0]], [[3.0, 4.0, 8.0], [6.0, 8.0, 
2.0], [7.0, 2.0, 8.0], [4.0, 1.0, 5.0]], [[7.0, 7.0, 5.0], [4.0, 8.0, 4.0], [5.0, 6.0, 8.0], [5.0, 8.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 3.0, 2.0], [7.0, 1.0, 7.0], [5.0, 7.0, 7.0]], [[4.0, 5.0, 6.0], [8.0, 8.0, 2.0], [6.0, 
6.0, 3.0]]])+(1.-msk_val)*numpy.array([[[7.0, 2.0, 6.0], [8.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[6.0, 2.0, 6.0], [6.0, 4.0, 3.0], 
[7.0, 1.0, 6.0]]])
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 2.0], [7.0, 1.0, 7.0], [5.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[4.0, 5.0, 6.0], [8.0, 
8.0, 2.0], [6.0, 6.0, 3.0], [4.0, 1.0, 5.0]], [[7.0, 7.0, 5.0], [4.0, 8.0, 4.0], [5.0, 6.0, 8.0], [5.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 6.0], [8.0, 1.0, 5.0], [8.0, 5.0, 8.0], [5.0, 8.0, 4.0]], [[6.0, 2.0, 6.0], [6.0, 
4.0, 3.0], [7.0, 1.0, 6.0], [4.0, 1.0, 5.0]], [[7.0, 7.0, 5.0], [4.0, 8.0, 4.0], [5.0, 6.0, 8.0], [5.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_URN(self):
      arg=Data(numpy.array([[[2.0, 2.0, 7.0], [3.0, 7.0, 4.0], [4.0, 4.0, 7.0], [3.0, 4.0, 5.0]], [[7.0, 8.0, 4.0], [1.0, 6.0, 
4.0], [8.0, 8.0, 2.0], [5.0, 3.0, 5.0]], [[7.0, 7.0, 1.0], [2.0, 7.0, 5.0], [3.0, 3.0, 7.0], [7.0, 7.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[7.0, 2.0, 3.0], [7.0, 5.0, 4.0]], [[2.0, 8.0, 2.0], [5.0, 3.0, 3.0]]])
      arg[:2,1:3]=val
      ref=Data(numpy.array([[[2.0, 2.0, 7.0], [7.0, 2.0, 3.0], [7.0, 5.0, 4.0], [3.0, 4.0, 5.0]], [[7.0, 8.0, 4.0], [2.0, 8.0, 
2.0], [5.0, 3.0, 3.0], [5.0, 3.0, 5.0]], [[7.0, 7.0, 1.0], [2.0, 7.0, 5.0], [3.0, 3.0, 7.0], [7.0, 7.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_URN(self):
      arg=Data(numpy.array([[[8.0, 6.0, 5.0], [3.0, 5.0, 1.0], [4.0, 4.0, 2.0], [1.0, 4.0, 5.0]], [[8.0, 5.0, 2.0], [8.0, 2.0, 
1.0], [4.0, 8.0, 3.0], [8.0, 2.0, 7.0]], [[4.0, 2.0, 5.0], [4.0, 3.0, 1.0], [2.0, 8.0, 7.0], [5.0, 7.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 6.0, 7.0], [8.0, 8.0, 8.0]], [[6.0, 8.0, 3.0], [6.0, 6.0, 2.0]]]),self.functionspace)
      arg[:2,1:3]=val
      ref=Data(numpy.array([[[8.0, 6.0, 5.0], [6.0, 6.0, 7.0], [8.0, 8.0, 8.0], [1.0, 4.0, 5.0]], [[8.0, 5.0, 2.0], [6.0, 8.0, 
3.0], [6.0, 6.0, 2.0], [8.0, 2.0, 7.0]], [[4.0, 2.0, 5.0], [4.0, 3.0, 1.0], [2.0, 8.0, 7.0], [5.0, 7.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_URN(self):
      arg=Data(numpy.array([[[3.0, 4.0, 4.0], [8.0, 6.0, 6.0], [2.0, 3.0, 4.0], [7.0, 2.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 6.0, 
6.0], [4.0, 4.0, 4.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 6.0, 1.0], [4.0, 2.0, 2.0], [3.0, 2.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 1.0, 6.0], [2.0, 8.0, 6.0]], [[4.0, 1.0, 3.0], [1.0, 1.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[8.0, 2.0, 2.0], [4.0, 4.0, 7.0]], [[6.0, 5.0, 2.0], [2.0, 4.0, 1.0]]])
      arg[:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 4.0], [3.0, 1.0, 6.0], [2.0, 8.0, 6.0], [7.0, 2.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 
1.0, 3.0], [1.0, 1.0, 6.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 6.0, 1.0], [4.0, 2.0, 2.0], [3.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 4.0], [8.0, 2.0, 2.0], [4.0, 4.0, 7.0], [7.0, 2.0, 8.0]], [[7.0, 2.0, 2.0], [6.0, 
5.0, 2.0], [2.0, 4.0, 1.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 6.0, 1.0], [4.0, 2.0, 2.0], [3.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RCL(self):
      arg=Data(numpy.array([[[2.0, 3.0, 1.0], [1.0, 4.0, 4.0], [4.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[5.0, 3.0, 3.0], [2.0, 7.0, 
8.0], [8.0, 7.0, 8.0], [8.0, 8.0, 1.0]], [[4.0, 2.0, 6.0], [3.0, 8.0, 6.0], [4.0, 3.0, 3.0], [2.0, 8.0, 
6.0]]]),self.functionspace)
      val=numpy.array([[[7.0], [5.0], [2.0], [2.0]]])
      arg[1:2,:,2:]=val
      ref=Data(numpy.array([[[2.0, 3.0, 1.0], [1.0, 4.0, 4.0], [4.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[5.0, 3.0, 7.0], [2.0, 7.0, 
5.0], [8.0, 7.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 2.0, 6.0], [3.0, 8.0, 6.0], [4.0, 3.0, 3.0], [2.0, 8.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RCL(self):
      arg=Data(numpy.array([[[1.0, 7.0, 3.0], [1.0, 2.0, 3.0], [8.0, 2.0, 4.0], [7.0, 5.0, 6.0]], [[2.0, 1.0, 4.0], [8.0, 6.0, 
3.0], [3.0, 4.0, 1.0], [3.0, 3.0, 5.0]], [[7.0, 7.0, 7.0], [8.0, 1.0, 8.0], [2.0, 4.0, 7.0], [6.0, 4.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [6.0], [5.0], [7.0]]]),self.functionspace)
      arg[1:2,:,2:]=val
      ref=Data(numpy.array([[[1.0, 7.0, 3.0], [1.0, 2.0, 3.0], [8.0, 2.0, 4.0], [7.0, 5.0, 6.0]], [[2.0, 1.0, 4.0], [8.0, 6.0, 
6.0], [3.0, 4.0, 5.0], [3.0, 3.0, 7.0]], [[7.0, 7.0, 7.0], [8.0, 1.0, 8.0], [2.0, 4.0, 7.0], [6.0, 4.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RCL(self):
      arg=Data(numpy.array([[[6.0, 4.0, 8.0], [6.0, 6.0, 7.0], [3.0, 4.0, 8.0], [8.0, 3.0, 7.0]], [[8.0, 7.0, 8.0], [3.0, 2.0, 
1.0], [5.0, 8.0, 8.0], [8.0, 8.0, 6.0]], [[4.0, 6.0, 3.0], [2.0, 3.0, 1.0], [4.0, 3.0, 3.0], [8.0, 5.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0], [5.0], [7.0], [2.0]]])+(1.-msk_val)*numpy.array([[[7.0], [2.0], [4.0], [5.0]]])
      arg[1:2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 8.0], [6.0, 6.0, 7.0], [3.0, 4.0, 8.0], [8.0, 3.0, 7.0]], [[8.0, 7.0, 3.0], [3.0, 
2.0, 5.0], [5.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [2.0, 3.0, 1.0], [4.0, 3.0, 3.0], [8.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 8.0], [6.0, 6.0, 7.0], [3.0, 4.0, 8.0], [8.0, 3.0, 7.0]], [[8.0, 7.0, 7.0], [3.0, 
2.0, 2.0], [5.0, 8.0, 4.0], [8.0, 8.0, 5.0]], [[4.0, 6.0, 3.0], [2.0, 3.0, 1.0], [4.0, 3.0, 3.0], [8.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CCC(self):
      arg=Data(numpy.array([[[2.0, 4.0, 8.0], [4.0, 7.0, 8.0], [5.0, 1.0, 4.0], [5.0, 6.0, 3.0]], [[2.0, 4.0, 2.0], [5.0, 6.0, 
2.0], [3.0, 5.0, 6.0], [7.0, 5.0, 2.0]], [[8.0, 1.0, 5.0], [6.0, 8.0, 2.0], [3.0, 5.0, 8.0], [7.0, 2.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[[3.0, 5.0, 3.0], [8.0, 8.0, 5.0], [8.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[7.0, 2.0, 4.0], [1.0, 2.0, 1.0], 
[2.0, 6.0, 8.0], [8.0, 8.0, 6.0]], [[5.0, 3.0, 6.0], [6.0, 7.0, 8.0], [7.0, 1.0, 6.0], [6.0, 2.0, 6.0]]])
      arg[:,:,:]=val
      ref=Data(numpy.array([[[3.0, 5.0, 3.0], [8.0, 8.0, 5.0], [8.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[7.0, 2.0, 4.0], [1.0, 2.0, 
1.0], [2.0, 6.0, 8.0], [8.0, 8.0, 6.0]], [[5.0, 3.0, 6.0], [6.0, 7.0, 8.0], [7.0, 1.0, 6.0], [6.0, 2.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CCC(self):
      arg=Data(numpy.array([[[3.0, 5.0, 4.0], [7.0, 2.0, 7.0], [6.0, 6.0, 8.0], [8.0, 1.0, 2.0]], [[8.0, 6.0, 7.0], [7.0, 2.0, 
6.0], [4.0, 3.0, 5.0], [4.0, 6.0, 4.0]], [[4.0, 5.0, 6.0], [5.0, 4.0, 3.0], [6.0, 2.0, 5.0], [2.0, 3.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 1.0, 4.0], [4.0, 8.0, 2.0], [8.0, 3.0, 4.0], [5.0, 4.0, 5.0]], [[6.0, 7.0, 7.0], [4.0, 6.0, 
6.0], [3.0, 8.0, 3.0], [4.0, 3.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 3.0, 3.0], [5.0, 8.0, 1.0], [3.0, 3.0, 
4.0]]]),self.functionspace)
      arg[:,:,:]=val
      ref=Data(numpy.array([[[4.0, 1.0, 4.0], [4.0, 8.0, 2.0], [8.0, 3.0, 4.0], [5.0, 4.0, 5.0]], [[6.0, 7.0, 7.0], [4.0, 6.0, 
6.0], [3.0, 8.0, 3.0], [4.0, 3.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 3.0, 3.0], [5.0, 8.0, 1.0], [3.0, 3.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CCC(self):
      arg=Data(numpy.array([[[3.0, 5.0, 4.0], [5.0, 7.0, 5.0], [3.0, 1.0, 1.0], [8.0, 4.0, 1.0]], [[7.0, 2.0, 8.0], [2.0, 7.0, 
4.0], [2.0, 2.0, 3.0], [8.0, 2.0, 4.0]], [[5.0, 1.0, 8.0], [4.0, 4.0, 3.0], [7.0, 5.0, 1.0], [4.0, 3.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 3.0, 1.0], [6.0, 2.0, 5.0], [6.0, 7.0, 6.0], [8.0, 1.0, 2.0]], [[4.0, 7.0, 1.0], [8.0, 
4.0, 3.0], [1.0, 5.0, 8.0], [6.0, 7.0, 6.0]], [[2.0, 2.0, 3.0], [2.0, 8.0, 2.0], [4.0, 1.0, 6.0], [7.0, 8.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 1.0], [8.0, 7.0, 4.0], [4.0, 4.0, 4.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 1.0], [7.0, 
8.0, 3.0], [4.0, 3.0, 5.0], [5.0, 1.0, 6.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 2.0], [3.0, 3.0, 4.0], [4.0, 4.0, 6.0]]])
      arg[:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 3.0, 1.0], [6.0, 2.0, 5.0], [6.0, 7.0, 6.0], [8.0, 1.0, 2.0]], [[4.0, 7.0, 1.0], [8.0, 
4.0, 3.0], [1.0, 5.0, 8.0], [6.0, 7.0, 6.0]], [[2.0, 2.0, 3.0], [2.0, 8.0, 2.0], [4.0, 1.0, 6.0], [7.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 1.0], [8.0, 7.0, 4.0], [4.0, 4.0, 4.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 1.0], [7.0, 
8.0, 3.0], [4.0, 3.0, 5.0], [5.0, 1.0, 6.0]], [[2.0, 2.0, 4.0], [2.0, 3.0, 2.0], [3.0, 3.0, 4.0], [4.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PLU(self):
      arg=Data(numpy.array([[[2.0, 4.0, 8.0], [4.0, 7.0, 4.0], [2.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[3.0, 8.0, 4.0], [5.0, 3.0, 
4.0], [6.0, 8.0, 1.0], [1.0, 2.0, 2.0]], [[5.0, 5.0, 2.0], [4.0, 6.0, 1.0], [4.0, 7.0, 4.0], [6.0, 3.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[6.0, 7.0], [8.0, 5.0]])
      arg[1,2:,:2]=val
      ref=Data(numpy.array([[[2.0, 4.0, 8.0], [4.0, 7.0, 4.0], [2.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[3.0, 8.0, 4.0], [5.0, 3.0, 
4.0], [6.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[5.0, 5.0, 2.0], [4.0, 6.0, 1.0], [4.0, 7.0, 4.0], [6.0, 3.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PLU(self):
      arg=Data(numpy.array([[[5.0, 8.0, 8.0], [5.0, 5.0, 8.0], [6.0, 3.0, 1.0], [1.0, 7.0, 2.0]], [[4.0, 6.0, 6.0], [4.0, 5.0, 
1.0], [4.0, 7.0, 7.0], [5.0, 5.0, 6.0]], [[7.0, 7.0, 3.0], [7.0, 4.0, 6.0], [4.0, 4.0, 8.0], [1.0, 7.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[6.0, 1.0], [7.0, 6.0]]),self.functionspace)
      arg[1,2:,:2]=val
      ref=Data(numpy.array([[[5.0, 8.0, 8.0], [5.0, 5.0, 8.0], [6.0, 3.0, 1.0], [1.0, 7.0, 2.0]], [[4.0, 6.0, 6.0], [4.0, 5.0, 
1.0], [6.0, 1.0, 7.0], [7.0, 6.0, 6.0]], [[7.0, 7.0, 3.0], [7.0, 4.0, 6.0], [4.0, 4.0, 8.0], [1.0, 7.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PLU(self):
      arg=Data(numpy.array([[[3.0, 6.0, 8.0], [7.0, 5.0, 4.0], [7.0, 7.0, 5.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 1.0, 
5.0], [3.0, 8.0, 4.0], [3.0, 4.0, 1.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 3.0], [7.0, 8.0, 7.0], [6.0, 5.0, 
8.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 6.0], [1.0, 5.0]])+(1.-msk_val)*numpy.array([[3.0, 6.0], [2.0, 1.0]])
      arg[1,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 8.0], [7.0, 5.0, 4.0], [7.0, 7.0, 5.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 
1.0, 5.0], [4.0, 6.0, 4.0], [1.0, 5.0, 1.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 3.0], [7.0, 8.0, 7.0], [6.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 6.0, 8.0], [7.0, 5.0, 4.0], [7.0, 7.0, 5.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 
1.0, 5.0], [3.0, 6.0, 4.0], [2.0, 1.0, 1.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 3.0], [7.0, 8.0, 7.0], [6.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RRN(self):
      arg=Data(numpy.array([[[2.0, 4.0, 3.0], [5.0, 6.0, 7.0], [3.0, 1.0, 6.0], [1.0, 8.0, 7.0]], [[2.0, 2.0, 7.0], [3.0, 5.0, 
2.0], [2.0, 8.0, 4.0], [2.0, 3.0, 2.0]], [[2.0, 4.0, 2.0], [4.0, 8.0, 5.0], [5.0, 4.0, 4.0], [3.0, 4.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[2.0, 2.0, 5.0], [8.0, 6.0, 8.0]]])
      arg[1:2,1:3]=val
      ref=Data(numpy.array([[[2.0, 4.0, 3.0], [5.0, 6.0, 7.0], [3.0, 1.0, 6.0], [1.0, 8.0, 7.0]], [[2.0, 2.0, 7.0], [2.0, 2.0, 
5.0], [8.0, 6.0, 8.0], [2.0, 3.0, 2.0]], [[2.0, 4.0, 2.0], [4.0, 8.0, 5.0], [5.0, 4.0, 4.0], [3.0, 4.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RRN(self):
      arg=Data(numpy.array([[[8.0, 3.0, 8.0], [8.0, 2.0, 4.0], [8.0, 2.0, 5.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 7.0], [3.0, 5.0, 
5.0], [6.0, 7.0, 3.0], [4.0, 3.0, 6.0]], [[4.0, 6.0, 7.0], [5.0, 3.0, 4.0], [7.0, 1.0, 4.0], [6.0, 1.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 4.0, 7.0], [7.0, 7.0, 8.0]]]),self.functionspace)
      arg[1:2,1:3]=val
      ref=Data(numpy.array([[[8.0, 3.0, 8.0], [8.0, 2.0, 4.0], [8.0, 2.0, 5.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 7.0], [2.0, 4.0, 
7.0], [7.0, 7.0, 8.0], [4.0, 3.0, 6.0]], [[4.0, 6.0, 7.0], [5.0, 3.0, 4.0], [7.0, 1.0, 4.0], [6.0, 1.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RRN(self):
      arg=Data(numpy.array([[[4.0, 1.0, 8.0], [8.0, 7.0, 6.0], [1.0, 6.0, 5.0], [7.0, 2.0, 3.0]], [[1.0, 2.0, 7.0], [5.0, 2.0, 
4.0], [8.0, 3.0, 3.0], [4.0, 4.0, 7.0]], [[1.0, 4.0, 5.0], [3.0, 2.0, 3.0], [3.0, 1.0, 3.0], [7.0, 2.0, 
7.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 6.0, 3.0], [7.0, 5.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 8.0, 4.0], [4.0, 3.0, 
6.0]]])
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 1.0, 8.0], [8.0, 7.0, 6.0], [1.0, 6.0, 5.0], [7.0, 2.0, 3.0]], [[1.0, 2.0, 7.0], [7.0, 
6.0, 3.0], [7.0, 5.0, 3.0], [4.0, 4.0, 7.0]], [[1.0, 4.0, 5.0], [3.0, 2.0, 3.0], [3.0, 1.0, 3.0], [7.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 1.0, 8.0], [8.0, 7.0, 6.0], [1.0, 6.0, 5.0], [7.0, 2.0, 3.0]], [[1.0, 2.0, 7.0], [1.0, 
8.0, 4.0], [4.0, 3.0, 6.0], [4.0, 4.0, 7.0]], [[1.0, 4.0, 5.0], [3.0, 2.0, 3.0], [3.0, 1.0, 3.0], [7.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CPR(self):
      arg=Data(numpy.array([[[5.0, 4.0, 2.0], [7.0, 3.0, 3.0], [4.0, 6.0, 5.0], [8.0, 4.0, 6.0]], [[7.0, 8.0, 7.0], [5.0, 5.0, 
2.0], [2.0, 7.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 1.0, 2.0], [1.0, 3.0, 7.0], [4.0, 1.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[2.0], [4.0], [7.0]])
      arg[:,2,1:2]=val
      ref=Data(numpy.array([[[5.0, 4.0, 2.0], [7.0, 3.0, 3.0], [4.0, 2.0, 5.0], [8.0, 4.0, 6.0]], [[7.0, 8.0, 7.0], [5.0, 5.0, 
2.0], [2.0, 4.0, 5.0], [5.0, 2.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 1.0, 2.0], [1.0, 7.0, 7.0], [4.0, 1.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CPR(self):
      arg=Data(numpy.array([[[1.0, 8.0, 5.0], [1.0, 5.0, 3.0], [5.0, 7.0, 1.0], [5.0, 5.0, 1.0]], [[4.0, 8.0, 2.0], [2.0, 7.0, 
1.0], [3.0, 8.0, 3.0], [2.0, 6.0, 8.0]], [[6.0, 6.0, 6.0], [1.0, 3.0, 4.0], [3.0, 2.0, 7.0], [5.0, 4.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[6.0], [7.0], [2.0]]),self.functionspace)
      arg[:,2,1:2]=val
      ref=Data(numpy.array([[[1.0, 8.0, 5.0], [1.0, 5.0, 3.0], [5.0, 6.0, 1.0], [5.0, 5.0, 1.0]], [[4.0, 8.0, 2.0], [2.0, 7.0, 
1.0], [3.0, 7.0, 3.0], [2.0, 6.0, 8.0]], [[6.0, 6.0, 6.0], [1.0, 3.0, 4.0], [3.0, 2.0, 7.0], [5.0, 4.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CPR(self):
      arg=Data(numpy.array([[[1.0, 5.0, 2.0], [2.0, 3.0, 3.0], [3.0, 6.0, 4.0], [8.0, 4.0, 4.0]], [[6.0, 4.0, 7.0], [1.0, 2.0, 
8.0], [7.0, 5.0, 3.0], [6.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 6.0], [1.0, 6.0, 1.0], [8.0, 4.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0], [3.0], [8.0]])+(1.-msk_val)*numpy.array([[4.0], [4.0], [2.0]])
      arg[:,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 2.0], [2.0, 3.0, 3.0], [3.0, 7.0, 4.0], [8.0, 4.0, 4.0]], [[6.0, 4.0, 7.0], [1.0, 
2.0, 8.0], [7.0, 3.0, 3.0], [6.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 6.0], [1.0, 8.0, 1.0], [8.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 2.0], [2.0, 3.0, 3.0], [3.0, 4.0, 4.0], [8.0, 4.0, 4.0]], [[6.0, 4.0, 7.0], [1.0, 
2.0, 8.0], [7.0, 4.0, 3.0], [6.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [6.0, 7.0, 6.0], [1.0, 2.0, 1.0], [8.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CPN(self):
      arg=Data(numpy.array([[[5.0, 4.0, 8.0], [8.0, 5.0, 3.0], [4.0, 6.0, 5.0], [6.0, 8.0, 7.0]], [[3.0, 6.0, 2.0], [3.0, 2.0, 
4.0], [8.0, 8.0, 7.0], [4.0, 3.0, 3.0]], [[7.0, 5.0, 1.0], [7.0, 3.0, 7.0], [5.0, 6.0, 7.0], [1.0, 8.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[2.0, 3.0, 1.0], [2.0, 5.0, 4.0], [3.0, 3.0, 5.0]])
      arg[:,2]=val
      ref=Data(numpy.array([[[5.0, 4.0, 8.0], [8.0, 5.0, 3.0], [2.0, 3.0, 1.0], [6.0, 8.0, 7.0]], [[3.0, 6.0, 2.0], [3.0, 2.0, 
4.0], [2.0, 5.0, 4.0], [4.0, 3.0, 3.0]], [[7.0, 5.0, 1.0], [7.0, 3.0, 7.0], [3.0, 3.0, 5.0], [1.0, 8.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CPN(self):
      arg=Data(numpy.array([[[7.0, 2.0, 7.0], [1.0, 3.0, 3.0], [2.0, 1.0, 4.0], [1.0, 8.0, 3.0]], [[7.0, 8.0, 7.0], [4.0, 7.0, 
5.0], [4.0, 7.0, 2.0], [1.0, 4.0, 6.0]], [[2.0, 2.0, 4.0], [5.0, 6.0, 5.0], [4.0, 7.0, 7.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 7.0, 7.0], [8.0, 8.0, 1.0], [8.0, 8.0, 6.0]]),self.functionspace)
      arg[:,2]=val
      ref=Data(numpy.array([[[7.0, 2.0, 7.0], [1.0, 3.0, 3.0], [1.0, 7.0, 7.0], [1.0, 8.0, 3.0]], [[7.0, 8.0, 7.0], [4.0, 7.0, 
5.0], [8.0, 8.0, 1.0], [1.0, 4.0, 6.0]], [[2.0, 2.0, 4.0], [5.0, 6.0, 5.0], [8.0, 8.0, 6.0], [8.0, 4.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CPN(self):
      arg=Data(numpy.array([[[6.0, 1.0, 1.0], [6.0, 4.0, 4.0], [1.0, 2.0, 8.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [5.0, 3.0, 
1.0], [7.0, 5.0, 5.0], [5.0, 6.0, 7.0]], [[5.0, 7.0, 4.0], [5.0, 5.0, 6.0], [4.0, 1.0, 2.0], [2.0, 1.0, 
7.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 8.0, 3.0], [7.0, 1.0, 6.0], [5.0, 1.0, 8.0]])+(1.-msk_val)*numpy.array([[6.0, 7.0, 5.0], 
[6.0, 8.0, 7.0], [3.0, 6.0, 2.0]])
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 1.0], [6.0, 4.0, 4.0], [6.0, 8.0, 3.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [5.0, 
3.0, 1.0], [7.0, 1.0, 6.0], [5.0, 6.0, 7.0]], [[5.0, 7.0, 4.0], [5.0, 5.0, 6.0], [5.0, 1.0, 8.0], [2.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 1.0], [6.0, 4.0, 4.0], [6.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [5.0, 
3.0, 1.0], [6.0, 8.0, 7.0], [5.0, 6.0, 7.0]], [[5.0, 7.0, 4.0], [5.0, 5.0, 6.0], [3.0, 6.0, 2.0], [2.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CRN(self):
      arg=Data(numpy.array([[[4.0, 4.0, 1.0], [2.0, 7.0, 8.0], [7.0, 6.0, 5.0], [3.0, 2.0, 6.0]], [[7.0, 7.0, 3.0], [2.0, 7.0, 
5.0], [6.0, 8.0, 5.0], [2.0, 3.0, 1.0]], [[4.0, 8.0, 7.0], [1.0, 1.0, 2.0], [7.0, 2.0, 3.0], [4.0, 3.0, 
3.0]]]),self.functionspace)
      val=numpy.array([[[6.0, 2.0, 5.0], [8.0, 8.0, 7.0]], [[5.0, 8.0, 3.0], [5.0, 8.0, 1.0]], [[2.0, 8.0, 4.0], [3.0, 4.0, 
3.0]]])
      arg[:,1:3]=val
      ref=Data(numpy.array([[[4.0, 4.0, 1.0], [6.0, 2.0, 5.0], [8.0, 8.0, 7.0], [3.0, 2.0, 6.0]], [[7.0, 7.0, 3.0], [5.0, 8.0, 
3.0], [5.0, 8.0, 1.0], [2.0, 3.0, 1.0]], [[4.0, 8.0, 7.0], [2.0, 8.0, 4.0], [3.0, 4.0, 3.0], [4.0, 3.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CRN(self):
      arg=Data(numpy.array([[[3.0, 8.0, 7.0], [2.0, 6.0, 6.0], [5.0, 1.0, 5.0], [2.0, 7.0, 5.0]], [[1.0, 6.0, 4.0], [5.0, 8.0, 
7.0], [5.0, 8.0, 6.0], [7.0, 6.0, 5.0]], [[5.0, 5.0, 4.0], [4.0, 7.0, 2.0], [8.0, 8.0, 1.0], [1.0, 7.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 3.0, 2.0], [5.0, 5.0, 6.0]], [[6.0, 2.0, 6.0], [3.0, 1.0, 2.0]], [[3.0, 4.0, 6.0], [7.0, 
5.0, 3.0]]]),self.functionspace)
      arg[:,1:3]=val
      ref=Data(numpy.array([[[3.0, 8.0, 7.0], [2.0, 3.0, 2.0], [5.0, 5.0, 6.0], [2.0, 7.0, 5.0]], [[1.0, 6.0, 4.0], [6.0, 2.0, 
6.0], [3.0, 1.0, 2.0], [7.0, 6.0, 5.0]], [[5.0, 5.0, 4.0], [3.0, 4.0, 6.0], [7.0, 5.0, 3.0], [1.0, 7.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CRN(self):
      arg=Data(numpy.array([[[2.0, 3.0, 1.0], [4.0, 4.0, 1.0], [4.0, 3.0, 4.0], [3.0, 4.0, 8.0]], [[8.0, 1.0, 5.0], [8.0, 5.0, 
6.0], [7.0, 8.0, 1.0], [4.0, 7.0, 3.0]], [[5.0, 4.0, 8.0], [5.0, 2.0, 1.0], [5.0, 1.0, 5.0], [4.0, 7.0, 
6.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 1.0, 8.0], [2.0, 8.0, 1.0]], [[2.0, 3.0, 3.0], [2.0, 8.0, 3.0]], [[8.0, 8.0, 1.0], [8.0, 
2.0, 7.0]]])+(1.-msk_val)*numpy.array([[[3.0, 6.0, 2.0], [4.0, 4.0, 4.0]], [[7.0, 2.0, 7.0], [7.0, 4.0, 8.0]], [[1.0, 5.0, 
1.0], [1.0, 3.0, 4.0]]])
      arg[:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 1.0], [4.0, 1.0, 8.0], [2.0, 8.0, 1.0], [3.0, 4.0, 8.0]], [[8.0, 1.0, 5.0], [2.0, 
3.0, 3.0], [2.0, 8.0, 3.0], [4.0, 7.0, 3.0]], [[5.0, 4.0, 8.0], [8.0, 8.0, 1.0], [8.0, 2.0, 7.0], [4.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 1.0], [3.0, 6.0, 2.0], [4.0, 4.0, 4.0], [3.0, 4.0, 8.0]], [[8.0, 1.0, 5.0], [7.0, 
2.0, 7.0], [7.0, 4.0, 8.0], [4.0, 7.0, 3.0]], [[5.0, 4.0, 8.0], [1.0, 5.0, 1.0], [1.0, 3.0, 4.0], [4.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CRC(self):
      arg=Data(numpy.array([[[1.0, 2.0, 3.0], [3.0, 7.0, 7.0], [7.0, 5.0, 4.0], [2.0, 5.0, 1.0]], [[2.0, 1.0, 1.0], [5.0, 8.0, 
8.0], [7.0, 3.0, 2.0], [6.0, 3.0, 6.0]], [[7.0, 8.0, 6.0], [4.0, 5.0, 2.0], [5.0, 4.0, 8.0], [7.0, 1.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[1.0, 1.0, 2.0], [1.0, 7.0, 7.0]], [[3.0, 1.0, 4.0], [2.0, 1.0, 7.0]], [[6.0, 6.0, 7.0], [5.0, 1.0, 
8.0]]])
      arg[:,1:3,:]=val
      ref=Data(numpy.array([[[1.0, 2.0, 3.0], [1.0, 1.0, 2.0], [1.0, 7.0, 7.0], [2.0, 5.0, 1.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 
4.0], [2.0, 1.0, 7.0], [6.0, 3.0, 6.0]], [[7.0, 8.0, 6.0], [6.0, 6.0, 7.0], [5.0, 1.0, 8.0], [7.0, 1.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CRC(self):
      arg=Data(numpy.array([[[7.0, 1.0, 8.0], [2.0, 4.0, 8.0], [8.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[5.0, 8.0, 4.0], [7.0, 2.0, 
5.0], [5.0, 6.0, 5.0], [2.0, 3.0, 2.0]], [[2.0, 6.0, 8.0], [3.0, 6.0, 3.0], [7.0, 7.0, 3.0], [3.0, 4.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 4.0, 8.0], [3.0, 2.0, 8.0]], [[6.0, 8.0, 4.0], [5.0, 2.0, 8.0]], [[1.0, 2.0, 8.0], [1.0, 
8.0, 7.0]]]),self.functionspace)
      arg[:,1:3,:]=val
      ref=Data(numpy.array([[[7.0, 1.0, 8.0], [7.0, 4.0, 8.0], [3.0, 2.0, 8.0], [6.0, 4.0, 5.0]], [[5.0, 8.0, 4.0], [6.0, 8.0, 
4.0], [5.0, 2.0, 8.0], [2.0, 3.0, 2.0]], [[2.0, 6.0, 8.0], [1.0, 2.0, 8.0], [1.0, 8.0, 7.0], [3.0, 4.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CRC(self):
      arg=Data(numpy.array([[[7.0, 3.0, 6.0], [1.0, 1.0, 6.0], [8.0, 2.0, 4.0], [5.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [4.0, 1.0, 
6.0], [6.0, 7.0, 1.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 5.0], [7.0, 8.0, 2.0], [1.0, 7.0, 2.0], [3.0, 3.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 6.0, 3.0], [8.0, 6.0, 4.0]], [[7.0, 3.0, 8.0], [1.0, 3.0, 6.0]], [[1.0, 6.0, 3.0], [6.0, 
5.0, 6.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 4.0], [5.0, 4.0, 3.0]], [[5.0, 6.0, 8.0], [4.0, 2.0, 3.0]], [[7.0, 5.0, 
7.0], [5.0, 8.0, 1.0]]])
      arg[:,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 6.0], [3.0, 6.0, 3.0], [8.0, 6.0, 4.0], [5.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [7.0, 
3.0, 8.0], [1.0, 3.0, 6.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 5.0], [1.0, 6.0, 3.0], [6.0, 5.0, 6.0], [3.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 6.0], [7.0, 3.0, 4.0], [5.0, 4.0, 3.0], [5.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [5.0, 
6.0, 8.0], [4.0, 2.0, 3.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 5.0], [7.0, 5.0, 7.0], [5.0, 8.0, 1.0], [3.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PPR(self):
      arg=Data(numpy.array([[[6.0, 7.0, 3.0], [1.0, 2.0, 7.0], [8.0, 2.0, 6.0], [2.0, 5.0, 8.0]], [[2.0, 6.0, 3.0], [6.0, 8.0, 
2.0], [1.0, 8.0, 5.0], [5.0, 7.0, 3.0]], [[1.0, 4.0, 3.0], [1.0, 2.0, 6.0], [4.0, 2.0, 3.0], [3.0, 5.0, 
1.0]]]),self.functionspace)
      val=numpy.array([6.0])
      arg[1,2,1:2]=val
      ref=Data(numpy.array([[[6.0, 7.0, 3.0], [1.0, 2.0, 7.0], [8.0, 2.0, 6.0], [2.0, 5.0, 8.0]], [[2.0, 6.0, 3.0], [6.0, 8.0, 
2.0], [1.0, 6.0, 5.0], [5.0, 7.0, 3.0]], [[1.0, 4.0, 3.0], [1.0, 2.0, 6.0], [4.0, 2.0, 3.0], [3.0, 5.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PPR(self):
      arg=Data(numpy.array([[[1.0, 3.0, 2.0], [3.0, 2.0, 5.0], [6.0, 3.0, 8.0], [3.0, 7.0, 6.0]], [[7.0, 1.0, 2.0], [3.0, 3.0, 
2.0], [7.0, 8.0, 5.0], [1.0, 8.0, 3.0]], [[1.0, 4.0, 6.0], [6.0, 5.0, 2.0], [4.0, 1.0, 4.0], [7.0, 4.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([4.0]),self.functionspace)
      arg[1,2,1:2]=val
      ref=Data(numpy.array([[[1.0, 3.0, 2.0], [3.0, 2.0, 5.0], [6.0, 3.0, 8.0], [3.0, 7.0, 6.0]], [[7.0, 1.0, 2.0], [3.0, 3.0, 
2.0], [7.0, 4.0, 5.0], [1.0, 8.0, 3.0]], [[1.0, 4.0, 6.0], [6.0, 5.0, 2.0], [4.0, 1.0, 4.0], [7.0, 4.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PPR(self):
      arg=Data(numpy.array([[[8.0, 2.0, 2.0], [6.0, 8.0, 1.0], [4.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [1.0, 1.0, 
4.0], [3.0, 2.0, 6.0], [1.0, 2.0, 1.0]], [[7.0, 8.0, 7.0], [6.0, 2.0, 3.0], [1.0, 6.0, 6.0], [1.0, 5.0, 
5.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([4.0])+(1.-msk_val)*numpy.array([5.0])
      arg[1,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 2.0], [6.0, 8.0, 1.0], [4.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [1.0, 
1.0, 4.0], [3.0, 4.0, 6.0], [1.0, 2.0, 1.0]], [[7.0, 8.0, 7.0], [6.0, 2.0, 3.0], [1.0, 6.0, 6.0], [1.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 2.0], [6.0, 8.0, 1.0], [4.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [1.0, 
1.0, 4.0], [3.0, 5.0, 6.0], [1.0, 2.0, 1.0]], [[7.0, 8.0, 7.0], [6.0, 2.0, 3.0], [1.0, 6.0, 6.0], [1.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PUU(self):
      arg=Data(numpy.array([[[2.0, 2.0, 8.0], [2.0, 1.0, 5.0], [5.0, 5.0, 2.0], [1.0, 4.0, 6.0]], [[2.0, 3.0, 7.0], [5.0, 5.0, 
6.0], [8.0, 5.0, 8.0], [5.0, 7.0, 7.0]], [[3.0, 2.0, 5.0], [8.0, 6.0, 5.0], [8.0, 6.0, 8.0], [1.0, 6.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[2.0, 2.0], [6.0, 7.0], [1.0, 4.0]])
      arg[1,:3,:2]=val
      ref=Data(numpy.array([[[2.0, 2.0, 8.0], [2.0, 1.0, 5.0], [5.0, 5.0, 2.0], [1.0, 4.0, 6.0]], [[2.0, 2.0, 7.0], [6.0, 7.0, 
6.0], [1.0, 4.0, 8.0], [5.0, 7.0, 7.0]], [[3.0, 2.0, 5.0], [8.0, 6.0, 5.0], [8.0, 6.0, 8.0], [1.0, 6.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PUU(self):
      arg=Data(numpy.array([[[8.0, 2.0, 6.0], [6.0, 6.0, 7.0], [4.0, 7.0, 7.0], [3.0, 2.0, 2.0]], [[4.0, 1.0, 7.0], [6.0, 5.0, 
8.0], [5.0, 1.0, 3.0], [4.0, 7.0, 6.0]], [[2.0, 3.0, 3.0], [4.0, 8.0, 8.0], [4.0, 8.0, 4.0], [1.0, 6.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 4.0], [6.0, 1.0], [5.0, 1.0]]),self.functionspace)
      arg[1,:3,:2]=val
      ref=Data(numpy.array([[[8.0, 2.0, 6.0], [6.0, 6.0, 7.0], [4.0, 7.0, 7.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 7.0], [6.0, 1.0, 
8.0], [5.0, 1.0, 3.0], [4.0, 7.0, 6.0]], [[2.0, 3.0, 3.0], [4.0, 8.0, 8.0], [4.0, 8.0, 4.0], [1.0, 6.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PUU(self):
      arg=Data(numpy.array([[[5.0, 6.0, 5.0], [1.0, 7.0, 8.0], [4.0, 7.0, 8.0], [6.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [6.0, 3.0, 
6.0], [8.0, 6.0, 2.0], [8.0, 5.0, 5.0]], [[8.0, 6.0, 2.0], [2.0, 8.0, 2.0], [1.0, 7.0, 6.0], [6.0, 4.0, 
7.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 5.0], [8.0, 5.0], [3.0, 7.0]])+(1.-msk_val)*numpy.array([[8.0, 8.0], [4.0, 7.0], [7.0, 
5.0]])
      arg[1,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 5.0], [1.0, 7.0, 8.0], [4.0, 7.0, 8.0], [6.0, 6.0, 5.0]], [[7.0, 5.0, 3.0], [8.0, 
5.0, 6.0], [3.0, 7.0, 2.0], [8.0, 5.0, 5.0]], [[8.0, 6.0, 2.0], [2.0, 8.0, 2.0], [1.0, 7.0, 6.0], [6.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 5.0], [1.0, 7.0, 8.0], [4.0, 7.0, 8.0], [6.0, 6.0, 5.0]], [[8.0, 8.0, 3.0], [4.0, 
7.0, 6.0], [7.0, 5.0, 2.0], [8.0, 5.0, 5.0]], [[8.0, 6.0, 2.0], [2.0, 8.0, 2.0], [1.0, 7.0, 6.0], [6.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UCN(self):
      arg=Data(numpy.array([[[4.0, 1.0, 8.0], [3.0, 7.0, 5.0], [8.0, 5.0, 1.0], [4.0, 1.0, 4.0]], [[6.0, 8.0, 1.0], [4.0, 5.0, 
7.0], [3.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[5.0, 6.0, 3.0], [7.0, 5.0, 2.0], [2.0, 7.0, 1.0], [5.0, 7.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[8.0, 8.0, 6.0], [4.0, 8.0, 8.0], [3.0, 2.0, 2.0], [1.0, 4.0, 3.0]], [[6.0, 3.0, 8.0], [8.0, 6.0, 2.0], 
[6.0, 7.0, 1.0], [4.0, 1.0, 2.0]]])
      arg[:2,:]=val
      ref=Data(numpy.array([[[8.0, 8.0, 6.0], [4.0, 8.0, 8.0], [3.0, 2.0, 2.0], [1.0, 4.0, 3.0]], [[6.0, 3.0, 8.0], [8.0, 6.0, 
2.0], [6.0, 7.0, 1.0], [4.0, 1.0, 2.0]], [[5.0, 6.0, 3.0], [7.0, 5.0, 2.0], [2.0, 7.0, 1.0], [5.0, 7.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UCN(self):
      arg=Data(numpy.array([[[3.0, 3.0, 3.0], [2.0, 6.0, 6.0], [6.0, 4.0, 3.0], [6.0, 2.0, 3.0]], [[8.0, 3.0, 2.0], [3.0, 2.0, 
5.0], [8.0, 2.0, 5.0], [5.0, 6.0, 5.0]], [[1.0, 8.0, 1.0], [6.0, 1.0, 8.0], [1.0, 7.0, 3.0], [2.0, 4.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 2.0, 2.0], [7.0, 4.0, 2.0], [6.0, 4.0, 3.0], [2.0, 7.0, 5.0]], [[1.0, 1.0, 7.0], [6.0, 7.0, 
1.0], [3.0, 1.0, 2.0], [2.0, 1.0, 5.0]]]),self.functionspace)
      arg[:2,:]=val
      ref=Data(numpy.array([[[8.0, 2.0, 2.0], [7.0, 4.0, 2.0], [6.0, 4.0, 3.0], [2.0, 7.0, 5.0]], [[1.0, 1.0, 7.0], [6.0, 7.0, 
1.0], [3.0, 1.0, 2.0], [2.0, 1.0, 5.0]], [[1.0, 8.0, 1.0], [6.0, 1.0, 8.0], [1.0, 7.0, 3.0], [2.0, 4.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UCN(self):
      arg=Data(numpy.array([[[4.0, 4.0, 5.0], [8.0, 2.0, 2.0], [4.0, 2.0, 5.0], [4.0, 3.0, 4.0]], [[2.0, 8.0, 6.0], [7.0, 6.0, 
7.0], [7.0, 1.0, 6.0], [4.0, 4.0, 1.0]], [[7.0, 5.0, 3.0], [8.0, 6.0, 1.0], [8.0, 4.0, 1.0], [5.0, 5.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 4.0, 2.0], [8.0, 4.0, 8.0], [2.0, 5.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 7.0, 8.0], [8.0, 
8.0, 3.0], [6.0, 2.0, 4.0], [3.0, 5.0, 2.0]]])+(1.-msk_val)*numpy.array([[[7.0, 5.0, 2.0], [7.0, 4.0, 5.0], [1.0, 3.0, 6.0], 
[8.0, 1.0, 4.0]], [[3.0, 5.0, 6.0], [1.0, 2.0, 5.0], [7.0, 6.0, 2.0], [3.0, 4.0, 4.0]]])
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 2.0], [8.0, 4.0, 8.0], [2.0, 5.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 7.0, 8.0], [8.0, 
8.0, 3.0], [6.0, 2.0, 4.0], [3.0, 5.0, 2.0]], [[7.0, 5.0, 3.0], [8.0, 6.0, 1.0], [8.0, 4.0, 1.0], [5.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 2.0], [7.0, 4.0, 5.0], [1.0, 3.0, 6.0], [8.0, 1.0, 4.0]], [[3.0, 5.0, 6.0], [1.0, 
2.0, 5.0], [7.0, 6.0, 2.0], [3.0, 4.0, 4.0]], [[7.0, 5.0, 3.0], [8.0, 6.0, 1.0], [8.0, 4.0, 1.0], [5.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RUN(self):
      arg=Data(numpy.array([[[8.0, 1.0, 4.0], [1.0, 6.0, 7.0], [7.0, 6.0, 7.0], [6.0, 3.0, 8.0]], [[4.0, 1.0, 7.0], [3.0, 1.0, 
7.0], [2.0, 2.0, 7.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 5.0], [1.0, 1.0, 2.0], [7.0, 8.0, 5.0], [2.0, 5.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[[5.0, 3.0, 2.0], [2.0, 3.0, 3.0], [6.0, 4.0, 3.0]]])
      arg[1:2,:3]=val
      ref=Data(numpy.array([[[8.0, 1.0, 4.0], [1.0, 6.0, 7.0], [7.0, 6.0, 7.0], [6.0, 3.0, 8.0]], [[5.0, 3.0, 2.0], [2.0, 3.0, 
3.0], [6.0, 4.0, 3.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 5.0], [1.0, 1.0, 2.0], [7.0, 8.0, 5.0], [2.0, 5.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RUN(self):
      arg=Data(numpy.array([[[6.0, 3.0, 1.0], [2.0, 7.0, 3.0], [7.0, 5.0, 5.0], [3.0, 5.0, 7.0]], [[1.0, 1.0, 1.0], [6.0, 6.0, 
8.0], [6.0, 5.0, 6.0], [5.0, 3.0, 5.0]], [[1.0, 5.0, 8.0], [6.0, 7.0, 2.0], [4.0, 4.0, 6.0], [6.0, 5.0, 
8.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 2.0, 6.0], [5.0, 2.0, 8.0], [6.0, 7.0, 3.0]]]),self.functionspace)
      arg[1:2,:3]=val
      ref=Data(numpy.array([[[6.0, 3.0, 1.0], [2.0, 7.0, 3.0], [7.0, 5.0, 5.0], [3.0, 5.0, 7.0]], [[7.0, 2.0, 6.0], [5.0, 2.0, 
8.0], [6.0, 7.0, 3.0], [5.0, 3.0, 5.0]], [[1.0, 5.0, 8.0], [6.0, 7.0, 2.0], [4.0, 4.0, 6.0], [6.0, 5.0, 
8.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RUN(self):
      arg=Data(numpy.array([[[5.0, 4.0, 7.0], [4.0, 7.0, 7.0], [2.0, 4.0, 7.0], [5.0, 4.0, 5.0]], [[8.0, 6.0, 8.0], [1.0, 6.0, 
2.0], [5.0, 7.0, 1.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [6.0, 8.0, 3.0], [5.0, 4.0, 8.0], [1.0, 5.0, 
7.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 8.0, 5.0], [1.0, 7.0, 1.0], [2.0, 3.0, 2.0]]])+(1.-msk_val)*numpy.array([[[2.0, 3.0, 
7.0], [4.0, 3.0, 1.0], [8.0, 8.0, 3.0]]])
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 7.0], [4.0, 7.0, 7.0], [2.0, 4.0, 7.0], [5.0, 4.0, 5.0]], [[3.0, 8.0, 5.0], [1.0, 
7.0, 1.0], [2.0, 3.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [6.0, 8.0, 3.0], [5.0, 4.0, 8.0], [1.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 7.0], [4.0, 7.0, 7.0], [2.0, 4.0, 7.0], [5.0, 4.0, 5.0]], [[2.0, 3.0, 7.0], [4.0, 
3.0, 1.0], [8.0, 8.0, 3.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [6.0, 8.0, 3.0], [5.0, 4.0, 8.0], [1.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LUC(self):
      arg=Data(numpy.array([[[8.0, 1.0, 3.0], [4.0, 5.0, 5.0], [5.0, 3.0, 1.0], [2.0, 3.0, 5.0]], [[8.0, 3.0, 3.0], [5.0, 4.0, 
2.0], [1.0, 6.0, 8.0], [2.0, 2.0, 2.0]], [[6.0, 3.0, 4.0], [8.0, 4.0, 3.0], [6.0, 6.0, 5.0], [8.0, 6.0, 
7.0]]]),self.functionspace)
      val=numpy.array([[[7.0, 4.0, 8.0], [4.0, 5.0, 8.0], [4.0, 6.0, 7.0]]])
      arg[2:,:3,:]=val
      ref=Data(numpy.array([[[8.0, 1.0, 3.0], [4.0, 5.0, 5.0], [5.0, 3.0, 1.0], [2.0, 3.0, 5.0]], [[8.0, 3.0, 3.0], [5.0, 4.0, 
2.0], [1.0, 6.0, 8.0], [2.0, 2.0, 2.0]], [[7.0, 4.0, 8.0], [4.0, 5.0, 8.0], [4.0, 6.0, 7.0], [8.0, 6.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LUC(self):
      arg=Data(numpy.array([[[6.0, 5.0, 6.0], [2.0, 6.0, 3.0], [3.0, 4.0, 6.0], [6.0, 7.0, 8.0]], [[8.0, 8.0, 4.0], [4.0, 1.0, 
4.0], [7.0, 6.0, 7.0], [7.0, 4.0, 8.0]], [[6.0, 8.0, 7.0], [7.0, 6.0, 4.0], [4.0, 8.0, 6.0], [7.0, 4.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 2.0, 4.0], [6.0, 6.0, 5.0], [4.0, 1.0, 8.0]]]),self.functionspace)
      arg[2:,:3,:]=val
      ref=Data(numpy.array([[[6.0, 5.0, 6.0], [2.0, 6.0, 3.0], [3.0, 4.0, 6.0], [6.0, 7.0, 8.0]], [[8.0, 8.0, 4.0], [4.0, 1.0, 
4.0], [7.0, 6.0, 7.0], [7.0, 4.0, 8.0]], [[1.0, 2.0, 4.0], [6.0, 6.0, 5.0], [4.0, 1.0, 8.0], [7.0, 4.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LUC(self):
      arg=Data(numpy.array([[[4.0, 2.0, 3.0], [7.0, 3.0, 7.0], [1.0, 1.0, 3.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 4.0], [1.0, 8.0, 
1.0], [2.0, 8.0, 2.0], [2.0, 6.0, 5.0]], [[5.0, 5.0, 1.0], [4.0, 8.0, 8.0], [2.0, 1.0, 6.0], [2.0, 2.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 5.0, 4.0], [1.0, 1.0, 7.0], [7.0, 6.0, 6.0]]])+(1.-msk_val)*numpy.array([[[7.0, 6.0, 
1.0], [4.0, 2.0, 8.0], [4.0, 1.0, 4.0]]])
      arg[2:,:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 3.0], [7.0, 3.0, 7.0], [1.0, 1.0, 3.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 4.0], [1.0, 
8.0, 1.0], [2.0, 8.0, 2.0], [2.0, 6.0, 5.0]], [[2.0, 5.0, 4.0], [1.0, 1.0, 7.0], [7.0, 6.0, 6.0], [2.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 3.0], [7.0, 3.0, 7.0], [1.0, 1.0, 3.0], [7.0, 4.0, 1.0]], [[7.0, 1.0, 4.0], [1.0, 
8.0, 1.0], [2.0, 8.0, 2.0], [2.0, 6.0, 5.0]], [[7.0, 6.0, 1.0], [4.0, 2.0, 8.0], [4.0, 1.0, 4.0], [2.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_UUC(self):
      arg=Data(numpy.array([[[1.0, 8.0, 5.0], [8.0, 5.0, 3.0], [2.0, 5.0, 2.0], [3.0, 8.0, 6.0]], [[4.0, 1.0, 1.0], [1.0, 1.0, 
4.0], [4.0, 7.0, 5.0], [6.0, 2.0, 1.0]], [[4.0, 7.0, 2.0], [8.0, 2.0, 7.0], [4.0, 1.0, 2.0], [8.0, 6.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[[2.0, 2.0, 8.0], [3.0, 6.0, 7.0], [1.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [7.0, 3.0, 8.0], [7.0, 8.0, 
4.0]]])
      arg[:2,:3,:]=val
      ref=Data(numpy.array([[[2.0, 2.0, 8.0], [3.0, 6.0, 7.0], [1.0, 2.0, 5.0], [3.0, 8.0, 6.0]], [[6.0, 2.0, 7.0], [7.0, 3.0, 
8.0], [7.0, 8.0, 4.0], [6.0, 2.0, 1.0]], [[4.0, 7.0, 2.0], [8.0, 2.0, 7.0], [4.0, 1.0, 2.0], [8.0, 6.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_UUC(self):
      arg=Data(numpy.array([[[7.0, 6.0, 2.0], [3.0, 3.0, 7.0], [1.0, 8.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 1.0, 2.0], [2.0, 7.0, 
4.0], [2.0, 8.0, 8.0], [3.0, 3.0, 8.0]], [[7.0, 3.0, 5.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0], [2.0, 7.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 3.0, 5.0], [1.0, 7.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 5.0, 1.0], [2.0, 1.0, 6.0], [5.0, 5.0, 
6.0]]]),self.functionspace)
      arg[:2,:3,:]=val
      ref=Data(numpy.array([[[8.0, 3.0, 5.0], [1.0, 7.0, 8.0], [6.0, 5.0, 5.0], [8.0, 2.0, 4.0]], [[3.0, 5.0, 1.0], [2.0, 1.0, 
6.0], [5.0, 5.0, 6.0], [3.0, 3.0, 8.0]], [[7.0, 3.0, 5.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0], [2.0, 7.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_UUC(self):
      arg=Data(numpy.array([[[2.0, 6.0, 3.0], [2.0, 4.0, 4.0], [5.0, 3.0, 2.0], [3.0, 4.0, 1.0]], [[6.0, 6.0, 5.0], [7.0, 8.0, 
6.0], [8.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 5.0, 6.0], [6.0, 1.0, 6.0], [8.0, 5.0, 7.0], [7.0, 8.0, 
1.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 4.0, 3.0], [3.0, 5.0, 1.0], [7.0, 1.0, 1.0]], [[7.0, 2.0, 6.0], [8.0, 5.0, 4.0], [2.0, 
8.0, 5.0]]])+(1.-msk_val)*numpy.array([[[6.0, 5.0, 4.0], [8.0, 5.0, 8.0], [3.0, 3.0, 1.0]], [[1.0, 2.0, 1.0], [1.0, 4.0, 8.0], 
[2.0, 4.0, 5.0]]])
      arg[:2,:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 3.0], [3.0, 5.0, 1.0], [7.0, 1.0, 1.0], [3.0, 4.0, 1.0]], [[7.0, 2.0, 6.0], [8.0, 
5.0, 4.0], [2.0, 8.0, 5.0], [2.0, 4.0, 1.0]], [[3.0, 5.0, 6.0], [6.0, 1.0, 6.0], [8.0, 5.0, 7.0], [7.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 4.0], [8.0, 5.0, 8.0], [3.0, 3.0, 1.0], [3.0, 4.0, 1.0]], [[1.0, 2.0, 1.0], [1.0, 
4.0, 8.0], [2.0, 4.0, 5.0], [2.0, 4.0, 1.0]], [[3.0, 5.0, 6.0], [6.0, 1.0, 6.0], [8.0, 5.0, 7.0], [7.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RLU(self):
      arg=Data(numpy.array([[[5.0, 5.0, 3.0], [7.0, 2.0, 2.0], [4.0, 8.0, 8.0], [7.0, 1.0, 6.0]], [[8.0, 4.0, 1.0], [3.0, 8.0, 
3.0], [6.0, 8.0, 7.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 4.0], [1.0, 7.0, 5.0], [2.0, 8.0, 1.0], [6.0, 8.0, 
4.0]]]),self.functionspace)
      val=numpy.array([[[7.0, 1.0], [8.0, 7.0]]])
      arg[1:2,2:,:2]=val
      ref=Data(numpy.array([[[5.0, 5.0, 3.0], [7.0, 2.0, 2.0], [4.0, 8.0, 8.0], [7.0, 1.0, 6.0]], [[8.0, 4.0, 1.0], [3.0, 8.0, 
3.0], [7.0, 1.0, 7.0], [8.0, 7.0, 2.0]], [[7.0, 6.0, 4.0], [1.0, 7.0, 5.0], [2.0, 8.0, 1.0], [6.0, 8.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RLU(self):
      arg=Data(numpy.array([[[3.0, 6.0, 6.0], [7.0, 5.0, 6.0], [4.0, 1.0, 7.0], [7.0, 2.0, 6.0]], [[3.0, 3.0, 4.0], [5.0, 3.0, 
4.0], [5.0, 7.0, 5.0], [3.0, 7.0, 2.0]], [[3.0, 1.0, 2.0], [8.0, 4.0, 4.0], [1.0, 1.0, 3.0], [5.0, 5.0, 
1.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 4.0], [6.0, 4.0]]]),self.functionspace)
      arg[1:2,2:,:2]=val
      ref=Data(numpy.array([[[3.0, 6.0, 6.0], [7.0, 5.0, 6.0], [4.0, 1.0, 7.0], [7.0, 2.0, 6.0]], [[3.0, 3.0, 4.0], [5.0, 3.0, 
4.0], [7.0, 4.0, 5.0], [6.0, 4.0, 2.0]], [[3.0, 1.0, 2.0], [8.0, 4.0, 4.0], [1.0, 1.0, 3.0], [5.0, 5.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RLU(self):
      arg=Data(numpy.array([[[5.0, 7.0, 4.0], [4.0, 6.0, 7.0], [2.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[3.0, 1.0, 8.0], [4.0, 8.0, 
6.0], [2.0, 7.0, 5.0], [2.0, 6.0, 7.0]], [[3.0, 1.0, 5.0], [5.0, 8.0, 8.0], [6.0, 4.0, 2.0], [7.0, 8.0, 
6.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 2.0], [3.0, 1.0]]])+(1.-msk_val)*numpy.array([[[5.0, 7.0], [7.0, 1.0]]])
      arg[1:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 4.0], [4.0, 6.0, 7.0], [2.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[3.0, 1.0, 8.0], [4.0, 
8.0, 6.0], [4.0, 2.0, 5.0], [3.0, 1.0, 7.0]], [[3.0, 1.0, 5.0], [5.0, 8.0, 8.0], [6.0, 4.0, 2.0], [7.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 4.0], [4.0, 6.0, 7.0], [2.0, 7.0, 3.0], [1.0, 8.0, 5.0]], [[3.0, 1.0, 8.0], [4.0, 
8.0, 6.0], [5.0, 7.0, 5.0], [7.0, 1.0, 7.0]], [[3.0, 1.0, 5.0], [5.0, 8.0, 8.0], [6.0, 4.0, 2.0], [7.0, 8.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_ULN(self):
      arg=Data(numpy.array([[[3.0, 1.0, 3.0], [6.0, 3.0, 8.0], [8.0, 6.0, 8.0], [6.0, 6.0, 8.0]], [[4.0, 5.0, 8.0], [5.0, 8.0, 
7.0], [2.0, 6.0, 4.0], [3.0, 7.0, 5.0]], [[5.0, 1.0, 2.0], [7.0, 4.0, 7.0], [4.0, 5.0, 4.0], [4.0, 5.0, 
1.0]]]),self.functionspace)
      val=numpy.array([[[6.0, 8.0, 4.0], [4.0, 6.0, 8.0]], [[2.0, 2.0, 7.0], [2.0, 3.0, 8.0]]])
      arg[:2,2:]=val
      ref=Data(numpy.array([[[3.0, 1.0, 3.0], [6.0, 3.0, 8.0], [6.0, 8.0, 4.0], [4.0, 6.0, 8.0]], [[4.0, 5.0, 8.0], [5.0, 8.0, 
7.0], [2.0, 2.0, 7.0], [2.0, 3.0, 8.0]], [[5.0, 1.0, 2.0], [7.0, 4.0, 7.0], [4.0, 5.0, 4.0], [4.0, 5.0, 
1.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_ULN(self):
      arg=Data(numpy.array([[[4.0, 8.0, 2.0], [6.0, 7.0, 8.0], [7.0, 8.0, 6.0], [8.0, 6.0, 5.0]], [[7.0, 5.0, 5.0], [5.0, 7.0, 
5.0], [2.0, 6.0, 4.0], [8.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [8.0, 5.0, 6.0], [6.0, 4.0, 3.0], [8.0, 4.0, 
5.0]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 7.0, 8.0], [7.0, 4.0, 8.0]], [[7.0, 5.0, 4.0], [2.0, 8.0, 7.0]]]),self.functionspace)
      arg[:2,2:]=val
      ref=Data(numpy.array([[[4.0, 8.0, 2.0], [6.0, 7.0, 8.0], [7.0, 7.0, 8.0], [7.0, 4.0, 8.0]], [[7.0, 5.0, 5.0], [5.0, 7.0, 
5.0], [7.0, 5.0, 4.0], [2.0, 8.0, 7.0]], [[6.0, 7.0, 3.0], [8.0, 5.0, 6.0], [6.0, 4.0, 3.0], [8.0, 4.0, 
5.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_ULN(self):
      arg=Data(numpy.array([[[7.0, 3.0, 3.0], [8.0, 2.0, 2.0], [1.0, 3.0, 7.0], [1.0, 7.0, 3.0]], [[4.0, 2.0, 8.0], [4.0, 6.0, 
5.0], [7.0, 8.0, 7.0], [2.0, 8.0, 4.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 4.0], [8.0, 1.0, 6.0], [8.0, 3.0, 
2.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 1.0, 8.0], [2.0, 7.0, 7.0]], [[3.0, 8.0, 8.0], [1.0, 3.0, 
8.0]]])+(1.-msk_val)*numpy.array([[[6.0, 8.0, 7.0], [7.0, 5.0, 8.0]], [[2.0, 4.0, 7.0], [6.0, 8.0, 5.0]]])
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 3.0], [8.0, 2.0, 2.0], [2.0, 1.0, 8.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 8.0], [4.0, 
6.0, 5.0], [3.0, 8.0, 8.0], [1.0, 3.0, 8.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 4.0], [8.0, 1.0, 6.0], [8.0, 3.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 3.0], [8.0, 2.0, 2.0], [6.0, 8.0, 7.0], [7.0, 5.0, 8.0]], [[4.0, 2.0, 8.0], [4.0, 
6.0, 5.0], [2.0, 4.0, 7.0], [6.0, 8.0, 5.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 4.0], [8.0, 1.0, 6.0], [8.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_RUP(self):
      arg=Data(numpy.array([[[4.0, 4.0, 7.0], [6.0, 4.0, 6.0], [4.0, 6.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 4.0], [5.0, 7.0, 
4.0], [7.0, 5.0, 7.0], [6.0, 6.0, 4.0]], [[2.0, 4.0, 7.0], [8.0, 8.0, 2.0], [3.0, 4.0, 3.0], [7.0, 3.0, 
6.0]]]),self.functionspace)
      val=numpy.array([[3.0, 6.0, 2.0]])
      arg[1:2,:3,1]=val
      ref=Data(numpy.array([[[4.0, 4.0, 7.0], [6.0, 4.0, 6.0], [4.0, 6.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 4.0], [5.0, 6.0, 
4.0], [7.0, 2.0, 7.0], [6.0, 6.0, 4.0]], [[2.0, 4.0, 7.0], [8.0, 8.0, 2.0], [3.0, 4.0, 3.0], [7.0, 3.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_RUP(self):
      arg=Data(numpy.array([[[4.0, 6.0, 4.0], [1.0, 6.0, 7.0], [6.0, 5.0, 5.0], [3.0, 7.0, 7.0]], [[8.0, 6.0, 3.0], [5.0, 4.0, 
1.0], [2.0, 5.0, 7.0], [6.0, 8.0, 6.0]], [[4.0, 1.0, 8.0], [6.0, 2.0, 8.0], [7.0, 5.0, 2.0], [3.0, 7.0, 
3.0]]]),self.functionspace)
      val=Data(numpy.array([[5.0, 2.0, 3.0]]),self.functionspace)
      arg[1:2,:3,1]=val
      ref=Data(numpy.array([[[4.0, 6.0, 4.0], [1.0, 6.0, 7.0], [6.0, 5.0, 5.0], [3.0, 7.0, 7.0]], [[8.0, 5.0, 3.0], [5.0, 2.0, 
1.0], [2.0, 3.0, 7.0], [6.0, 8.0, 6.0]], [[4.0, 1.0, 8.0], [6.0, 2.0, 8.0], [7.0, 5.0, 2.0], [3.0, 7.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_RUP(self):
      arg=Data(numpy.array([[[1.0, 1.0, 3.0], [8.0, 4.0, 3.0], [7.0, 5.0, 6.0], [2.0, 7.0, 4.0]], [[1.0, 7.0, 1.0], [3.0, 8.0, 
3.0], [6.0, 5.0, 8.0], [7.0, 6.0, 6.0]], [[1.0, 1.0, 6.0], [2.0, 5.0, 6.0], [2.0, 6.0, 8.0], [7.0, 1.0, 
7.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 1.0, 2.0]])+(1.-msk_val)*numpy.array([[2.0, 4.0, 5.0]])
      arg[1:2,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 3.0], [8.0, 4.0, 3.0], [7.0, 5.0, 6.0], [2.0, 7.0, 4.0]], [[1.0, 4.0, 1.0], [3.0, 
1.0, 3.0], [6.0, 2.0, 8.0], [7.0, 6.0, 6.0]], [[1.0, 1.0, 6.0], [2.0, 5.0, 6.0], [2.0, 6.0, 8.0], [7.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 3.0], [8.0, 4.0, 3.0], [7.0, 5.0, 6.0], [2.0, 7.0, 4.0]], [[1.0, 2.0, 1.0], [3.0, 
4.0, 3.0], [6.0, 5.0, 8.0], [7.0, 6.0, 6.0]], [[1.0, 1.0, 6.0], [2.0, 5.0, 6.0], [2.0, 6.0, 8.0], [7.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LUN(self):
      arg=Data(numpy.array([[[2.0, 6.0, 8.0], [3.0, 4.0, 1.0], [1.0, 3.0, 3.0], [6.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [1.0, 4.0, 
7.0], [8.0, 5.0, 5.0], [3.0, 6.0, 5.0]], [[7.0, 2.0, 6.0], [5.0, 1.0, 4.0], [2.0, 5.0, 6.0], [7.0, 6.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[[8.0, 5.0, 1.0], [8.0, 5.0, 4.0], [1.0, 3.0, 5.0]]])
      arg[2:,:3]=val
      ref=Data(numpy.array([[[2.0, 6.0, 8.0], [3.0, 4.0, 1.0], [1.0, 3.0, 3.0], [6.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [1.0, 4.0, 
7.0], [8.0, 5.0, 5.0], [3.0, 6.0, 5.0]], [[8.0, 5.0, 1.0], [8.0, 5.0, 4.0], [1.0, 3.0, 5.0], [7.0, 6.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LUN(self):
      arg=Data(numpy.array([[[8.0, 4.0, 2.0], [2.0, 5.0, 6.0], [6.0, 4.0, 1.0], [8.0, 1.0, 1.0]], [[6.0, 7.0, 2.0], [6.0, 5.0, 
1.0], [6.0, 8.0, 6.0], [7.0, 5.0, 6.0]], [[5.0, 8.0, 7.0], [5.0, 1.0, 6.0], [3.0, 6.0, 2.0], [1.0, 8.0, 
2.0]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 3.0, 1.0], [8.0, 7.0, 8.0], [7.0, 2.0, 8.0]]]),self.functionspace)
      arg[2:,:3]=val
      ref=Data(numpy.array([[[8.0, 4.0, 2.0], [2.0, 5.0, 6.0], [6.0, 4.0, 1.0], [8.0, 1.0, 1.0]], [[6.0, 7.0, 2.0], [6.0, 5.0, 
1.0], [6.0, 8.0, 6.0], [7.0, 5.0, 6.0]], [[5.0, 3.0, 1.0], [8.0, 7.0, 8.0], [7.0, 2.0, 8.0], [1.0, 8.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LUN(self):
      arg=Data(numpy.array([[[2.0, 1.0, 5.0], [4.0, 1.0, 1.0], [4.0, 7.0, 6.0], [4.0, 3.0, 4.0]], [[3.0, 6.0, 1.0], [3.0, 3.0, 
1.0], [8.0, 1.0, 5.0], [4.0, 4.0, 5.0]], [[6.0, 8.0, 4.0], [5.0, 4.0, 6.0], [5.0, 4.0, 8.0], [5.0, 4.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 5.0, 4.0], [3.0, 7.0, 5.0], [7.0, 7.0, 5.0]]])+(1.-msk_val)*numpy.array([[[1.0, 8.0, 
1.0], [8.0, 3.0, 5.0], [2.0, 8.0, 4.0]]])
      arg[2:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 5.0], [4.0, 1.0, 1.0], [4.0, 7.0, 6.0], [4.0, 3.0, 4.0]], [[3.0, 6.0, 1.0], [3.0, 
3.0, 1.0], [8.0, 1.0, 5.0], [4.0, 4.0, 5.0]], [[6.0, 5.0, 4.0], [3.0, 7.0, 5.0], [7.0, 7.0, 5.0], [5.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 5.0], [4.0, 1.0, 1.0], [4.0, 7.0, 6.0], [4.0, 3.0, 4.0]], [[3.0, 6.0, 1.0], [3.0, 
3.0, 1.0], [8.0, 1.0, 5.0], [4.0, 4.0, 5.0]], [[1.0, 8.0, 1.0], [8.0, 3.0, 5.0], [2.0, 8.0, 4.0], [5.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_CLN(self):
      arg=Data(numpy.array([[[2.0, 3.0, 8.0], [8.0, 7.0, 7.0], [1.0, 8.0, 2.0], [6.0, 2.0, 6.0]], [[4.0, 6.0, 3.0], [3.0, 1.0, 
5.0], [2.0, 4.0, 7.0], [5.0, 3.0, 7.0]], [[2.0, 4.0, 2.0], [6.0, 2.0, 7.0], [7.0, 4.0, 4.0], [6.0, 3.0, 
2.0]]]),self.functionspace)
      val=numpy.array([[[2.0, 3.0, 5.0], [1.0, 7.0, 5.0]], [[7.0, 7.0, 1.0], [1.0, 8.0, 8.0]], [[6.0, 4.0, 2.0], [6.0, 7.0, 
4.0]]])
      arg[:,2:]=val
      ref=Data(numpy.array([[[2.0, 3.0, 8.0], [8.0, 7.0, 7.0], [2.0, 3.0, 5.0], [1.0, 7.0, 5.0]], [[4.0, 6.0, 3.0], [3.0, 1.0, 
5.0], [7.0, 7.0, 1.0], [1.0, 8.0, 8.0]], [[2.0, 4.0, 2.0], [6.0, 2.0, 7.0], [6.0, 4.0, 2.0], [6.0, 7.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_CLN(self):
      arg=Data(numpy.array([[[3.0, 7.0, 2.0], [4.0, 1.0, 8.0], [5.0, 1.0, 1.0], [5.0, 8.0, 5.0]], [[3.0, 8.0, 4.0], [1.0, 6.0, 
1.0], [7.0, 5.0, 7.0], [2.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [6.0, 4.0, 6.0], [8.0, 7.0, 1.0], [5.0, 8.0, 
6.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 1.0, 6.0], [8.0, 5.0, 6.0]], [[2.0, 7.0, 1.0], [3.0, 7.0, 7.0]], [[3.0, 1.0, 7.0], [5.0, 
4.0, 2.0]]]),self.functionspace)
      arg[:,2:]=val
      ref=Data(numpy.array([[[3.0, 7.0, 2.0], [4.0, 1.0, 8.0], [6.0, 1.0, 6.0], [8.0, 5.0, 6.0]], [[3.0, 8.0, 4.0], [1.0, 6.0, 
1.0], [2.0, 7.0, 1.0], [3.0, 7.0, 7.0]], [[7.0, 3.0, 1.0], [6.0, 4.0, 6.0], [3.0, 1.0, 7.0], [5.0, 4.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_CLN(self):
      arg=Data(numpy.array([[[6.0, 3.0, 6.0], [6.0, 3.0, 6.0], [6.0, 6.0, 4.0], [6.0, 7.0, 3.0]], [[8.0, 2.0, 6.0], [8.0, 4.0, 
4.0], [4.0, 5.0, 8.0], [4.0, 3.0, 2.0]], [[7.0, 1.0, 8.0], [5.0, 8.0, 7.0], [2.0, 1.0, 7.0], [6.0, 4.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 2.0, 2.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[5.0, 1.0, 8.0], [6.0, 
1.0, 7.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 8.0], [5.0, 2.0, 5.0]], [[5.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 7.0, 
1.0], [2.0, 7.0, 8.0]]])
      arg[:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 6.0], [6.0, 3.0, 6.0], [4.0, 2.0, 2.0], [2.0, 3.0, 5.0]], [[8.0, 2.0, 6.0], [8.0, 
4.0, 4.0], [4.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[7.0, 1.0, 8.0], [5.0, 8.0, 7.0], [5.0, 1.0, 8.0], [6.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 6.0], [6.0, 3.0, 6.0], [4.0, 6.0, 8.0], [5.0, 2.0, 5.0]], [[8.0, 2.0, 6.0], [8.0, 
4.0, 4.0], [5.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[7.0, 1.0, 8.0], [5.0, 8.0, 7.0], [4.0, 7.0, 1.0], [2.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_LCN(self):
      arg=Data(numpy.array([[[8.0, 3.0, 3.0], [3.0, 5.0, 7.0], [8.0, 3.0, 6.0], [6.0, 8.0, 7.0]], [[3.0, 3.0, 7.0], [7.0, 5.0, 
3.0], [7.0, 3.0, 8.0], [8.0, 1.0, 8.0]], [[8.0, 6.0, 7.0], [2.0, 8.0, 7.0], [4.0, 3.0, 8.0], [1.0, 2.0, 
5.0]]]),self.functionspace)
      val=numpy.array([[[2.0, 8.0, 3.0], [3.0, 2.0, 5.0], [6.0, 8.0, 4.0], [7.0, 5.0, 2.0]]])
      arg[2:,:]=val
      ref=Data(numpy.array([[[8.0, 3.0, 3.0], [3.0, 5.0, 7.0], [8.0, 3.0, 6.0], [6.0, 8.0, 7.0]], [[3.0, 3.0, 7.0], [7.0, 5.0, 
3.0], [7.0, 3.0, 8.0], [8.0, 1.0, 8.0]], [[2.0, 8.0, 3.0], [3.0, 2.0, 5.0], [6.0, 8.0, 4.0], [7.0, 5.0, 
2.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_LCN(self):
      arg=Data(numpy.array([[[5.0, 2.0, 7.0], [2.0, 7.0, 3.0], [5.0, 4.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 1.0, 6.0], [8.0, 7.0, 
2.0], [6.0, 5.0, 6.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 2.0], [1.0, 4.0, 4.0], [2.0, 3.0, 4.0], [8.0, 8.0, 
7.0]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 3.0, 7.0], [4.0, 8.0, 7.0], [4.0, 4.0, 1.0], [8.0, 2.0, 7.0]]]),self.functionspace)
      arg[2:,:]=val
      ref=Data(numpy.array([[[5.0, 2.0, 7.0], [2.0, 7.0, 3.0], [5.0, 4.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 1.0, 6.0], [8.0, 7.0, 
2.0], [6.0, 5.0, 6.0], [2.0, 1.0, 6.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 7.0], [4.0, 4.0, 1.0], [8.0, 2.0, 
7.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_LCN(self):
      arg=Data(numpy.array([[[2.0, 1.0, 3.0], [6.0, 1.0, 5.0], [4.0, 2.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 4.0, 4.0], [3.0, 1.0, 
7.0], [5.0, 4.0, 5.0], [6.0, 2.0, 4.0]], [[2.0, 8.0, 1.0], [7.0, 8.0, 4.0], [8.0, 6.0, 4.0], [6.0, 6.0, 
3.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 7.0, 2.0], [4.0, 3.0, 8.0], [7.0, 2.0, 8.0], [6.0, 2.0, 
8.0]]])+(1.-msk_val)*numpy.array([[[3.0, 7.0, 5.0], [2.0, 7.0, 7.0], [1.0, 4.0, 8.0], [4.0, 2.0, 4.0]]])
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 3.0], [6.0, 1.0, 5.0], [4.0, 2.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 4.0, 4.0], [3.0, 
1.0, 7.0], [5.0, 4.0, 5.0], [6.0, 2.0, 4.0]], [[7.0, 7.0, 2.0], [4.0, 3.0, 8.0], [7.0, 2.0, 8.0], [6.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 3.0], [6.0, 1.0, 5.0], [4.0, 2.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 4.0, 4.0], [3.0, 
1.0, 7.0], [5.0, 4.0, 5.0], [6.0, 2.0, 4.0]], [[3.0, 7.0, 5.0], [2.0, 7.0, 7.0], [1.0, 4.0, 8.0], [4.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_array_PUN(self):
      arg=Data(numpy.array([[[3.0, 5.0, 1.0], [6.0, 6.0, 3.0], [1.0, 3.0, 8.0], [1.0, 5.0, 5.0]], [[5.0, 6.0, 6.0], [8.0, 8.0, 
6.0], [2.0, 4.0, 8.0], [1.0, 1.0, 5.0]], [[5.0, 5.0, 7.0], [1.0, 2.0, 8.0], [5.0, 5.0, 5.0], [1.0, 7.0, 
3.0]]]),self.functionspace)
      val=numpy.array([[6.0, 8.0, 7.0], [8.0, 1.0, 2.0], [6.0, 6.0, 6.0]])
      arg[1,:3]=val
      ref=Data(numpy.array([[[3.0, 5.0, 1.0], [6.0, 6.0, 3.0], [1.0, 3.0, 8.0], [1.0, 5.0, 5.0]], [[6.0, 8.0, 7.0], [8.0, 1.0, 
2.0], [6.0, 6.0, 6.0], [1.0, 1.0, 5.0]], [[5.0, 5.0, 7.0], [1.0, 2.0, 8.0], [5.0, 5.0, 5.0], [1.0, 7.0, 
3.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_constData_PUN(self):
      arg=Data(numpy.array([[[2.0, 8.0, 5.0], [1.0, 5.0, 2.0], [5.0, 7.0, 2.0], [4.0, 3.0, 4.0]], [[1.0, 7.0, 3.0], [5.0, 5.0, 
1.0], [1.0, 2.0, 7.0], [8.0, 1.0, 3.0]], [[4.0, 5.0, 6.0], [4.0, 1.0, 3.0], [7.0, 6.0, 5.0], [3.0, 7.0, 
4.0]]]),self.functionspace)
      val=Data(numpy.array([[5.0, 1.0, 7.0], [2.0, 4.0, 1.0], [7.0, 2.0, 8.0]]),self.functionspace)
      arg[1,:3]=val
      ref=Data(numpy.array([[[2.0, 8.0, 5.0], [1.0, 5.0, 2.0], [5.0, 7.0, 2.0], [4.0, 3.0, 4.0]], [[5.0, 1.0, 7.0], [2.0, 4.0, 
1.0], [7.0, 2.0, 8.0], [8.0, 1.0, 3.0]], [[4.0, 5.0, 6.0], [4.0, 1.0, 3.0], [7.0, 6.0, 5.0], [3.0, 7.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank3_expandedData_PUN(self):
      arg=Data(numpy.array([[[8.0, 2.0, 3.0], [3.0, 5.0, 1.0], [1.0, 4.0, 6.0], [7.0, 3.0, 7.0]], [[7.0, 3.0, 3.0], [1.0, 5.0, 
4.0], [1.0, 6.0, 3.0], [8.0, 7.0, 6.0]], [[6.0, 8.0, 8.0], [7.0, 8.0, 5.0], [4.0, 1.0, 1.0], [1.0, 2.0, 
4.0]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 1.0, 2.0], [6.0, 5.0, 2.0], [8.0, 2.0, 4.0]])+(1.-msk_val)*numpy.array([[2.0, 5.0, 3.0], 
[6.0, 2.0, 3.0], [2.0, 3.0, 3.0]])
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 3.0], [3.0, 5.0, 1.0], [1.0, 4.0, 6.0], [7.0, 3.0, 7.0]], [[8.0, 1.0, 2.0], [6.0, 
5.0, 2.0], [8.0, 2.0, 4.0], [8.0, 7.0, 6.0]], [[6.0, 8.0, 8.0], [7.0, 8.0, 5.0], [4.0, 1.0, 1.0], [1.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 3.0], [3.0, 5.0, 1.0], [1.0, 4.0, 6.0], [7.0, 3.0, 7.0]], [[2.0, 5.0, 3.0], [6.0, 
2.0, 3.0], [2.0, 3.0, 3.0], [8.0, 7.0, 6.0]], [[6.0, 8.0, 8.0], [7.0, 8.0, 5.0], [4.0, 1.0, 1.0], [1.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RURL(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 2.0], [8.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[4.0, 1.0, 4.0], [6.0, 3.0, 5.0], [8.0, 2.0, 
4.0]], [[7.0, 1.0, 6.0], [5.0, 1.0, 5.0], [7.0, 6.0, 5.0]]], [[[4.0, 3.0, 5.0], [5.0, 4.0, 3.0], [2.0, 8.0, 3.0]], [[4.0, 2.0, 
5.0], [5.0, 8.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 3.0, 2.0], [1.0, 5.0, 6.0], [2.0, 6.0, 7.0]]], [[[3.0, 3.0, 2.0], [5.0, 8.0, 
4.0], [6.0, 2.0, 2.0]], [[8.0, 1.0, 8.0], [4.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 
7.0]]], [[[1.0, 6.0, 3.0], [4.0, 3.0, 3.0], [4.0, 7.0, 6.0]], [[7.0, 1.0, 6.0], [7.0, 1.0, 2.0], [6.0, 4.0, 8.0]], [[6.0, 3.0, 
1.0], [3.0, 8.0, 2.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0]], [[2.0]]], [[[4.0]], [[2.0]]]])
      arg[1:3,:2,1:2,2:]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 2.0], [8.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[4.0, 1.0, 4.0], [6.0, 3.0, 5.0], [8.0, 2.0, 
4.0]], [[7.0, 1.0, 6.0], [5.0, 1.0, 5.0], [7.0, 6.0, 5.0]]], [[[4.0, 3.0, 5.0], [5.0, 4.0, 3.0], [2.0, 8.0, 3.0]], [[4.0, 2.0, 
5.0], [5.0, 8.0, 2.0], [2.0, 1.0, 1.0]], [[6.0, 3.0, 2.0], [1.0, 5.0, 6.0], [2.0, 6.0, 7.0]]], [[[3.0, 3.0, 2.0], [5.0, 8.0, 
4.0], [6.0, 2.0, 2.0]], [[8.0, 1.0, 8.0], [4.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 
7.0]]], [[[1.0, 6.0, 3.0], [4.0, 3.0, 3.0], [4.0, 7.0, 6.0]], [[7.0, 1.0, 6.0], [7.0, 1.0, 2.0], [6.0, 4.0, 8.0]], [[6.0, 3.0, 
1.0], [3.0, 8.0, 2.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RURL(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 3.0], [2.0, 6.0, 5.0], [6.0, 6.0, 8.0]], [[8.0, 2.0, 1.0], [3.0, 1.0, 7.0], [6.0, 6.0, 
2.0]], [[6.0, 7.0, 7.0], [5.0, 6.0, 2.0], [8.0, 2.0, 4.0]]], [[[3.0, 7.0, 7.0], [3.0, 2.0, 7.0], [1.0, 5.0, 1.0]], [[6.0, 1.0, 
1.0], [5.0, 2.0, 4.0], [4.0, 6.0, 6.0]], [[2.0, 5.0, 7.0], [8.0, 2.0, 8.0], [8.0, 6.0, 2.0]]], [[[8.0, 5.0, 7.0], [7.0, 2.0, 
6.0], [8.0, 1.0, 3.0]], [[5.0, 5.0, 2.0], [7.0, 2.0, 6.0], [1.0, 3.0, 4.0]], [[5.0, 2.0, 2.0], [7.0, 3.0, 4.0], [6.0, 1.0, 
7.0]]], [[[8.0, 5.0, 3.0], [2.0, 2.0, 4.0], [6.0, 1.0, 2.0]], [[6.0, 3.0, 3.0], [5.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[5.0, 3.0, 
2.0], [6.0, 5.0, 7.0], [7.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0]], [[3.0]]], [[[8.0]], [[2.0]]]]),self.functionspace)
      arg[1:3,:2,1:2,2:]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 3.0], [2.0, 6.0, 5.0], [6.0, 6.0, 8.0]], [[8.0, 2.0, 1.0], [3.0, 1.0, 7.0], [6.0, 6.0, 
2.0]], [[6.0, 7.0, 7.0], [5.0, 6.0, 2.0], [8.0, 2.0, 4.0]]], [[[3.0, 7.0, 7.0], [3.0, 2.0, 2.0], [1.0, 5.0, 1.0]], [[6.0, 1.0, 
1.0], [5.0, 2.0, 3.0], [4.0, 6.0, 6.0]], [[2.0, 5.0, 7.0], [8.0, 2.0, 8.0], [8.0, 6.0, 2.0]]], [[[8.0, 5.0, 7.0], [7.0, 2.0, 
8.0], [8.0, 1.0, 3.0]], [[5.0, 5.0, 2.0], [7.0, 2.0, 2.0], [1.0, 3.0, 4.0]], [[5.0, 2.0, 2.0], [7.0, 3.0, 4.0], [6.0, 1.0, 
7.0]]], [[[8.0, 5.0, 3.0], [2.0, 2.0, 4.0], [6.0, 1.0, 2.0]], [[6.0, 3.0, 3.0], [5.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[5.0, 3.0, 
2.0], [6.0, 5.0, 7.0], [7.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RURL(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 7.0], [7.0, 3.0, 7.0], [2.0, 5.0, 7.0]], [[1.0, 3.0, 3.0], [7.0, 1.0, 5.0], [7.0, 5.0, 
3.0]], [[4.0, 3.0, 7.0], [2.0, 5.0, 1.0], [7.0, 4.0, 6.0]]], [[[2.0, 2.0, 5.0], [4.0, 2.0, 2.0], [8.0, 8.0, 7.0]], [[4.0, 6.0, 
3.0], [7.0, 3.0, 8.0], [6.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [1.0, 6.0, 4.0], [7.0, 7.0, 6.0]]], [[[6.0, 5.0, 5.0], [7.0, 5.0, 
4.0], [3.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [7.0, 4.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 5.0, 6.0], [7.0, 2.0, 8.0], [4.0, 2.0, 
3.0]]], [[[3.0, 6.0, 6.0], [4.0, 5.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 4.0], [4.0, 1.0, 4.0], [3.0, 5.0, 1.0]], [[1.0, 2.0, 
5.0], [8.0, 8.0, 4.0], [6.0, 5.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0]], [[4.0]]], [[[5.0]], [[5.0]]]])+(1.-msk_val)*numpy.array([[[[6.0]], [[1.0]]], [[[7.0]], 
[[8.0]]]])
      arg[1:3,:2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 7.0], [7.0, 3.0, 7.0], [2.0, 5.0, 7.0]], [[1.0, 3.0, 3.0], [7.0, 1.0, 5.0], [7.0, 
5.0, 3.0]], [[4.0, 3.0, 7.0], [2.0, 5.0, 1.0], [7.0, 4.0, 6.0]]], [[[2.0, 2.0, 5.0], [4.0, 2.0, 5.0], [8.0, 8.0, 7.0]], [[4.0, 
6.0, 3.0], [7.0, 3.0, 4.0], [6.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [1.0, 6.0, 4.0], [7.0, 7.0, 6.0]]], [[[6.0, 5.0, 5.0], [7.0, 
5.0, 5.0], [3.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [7.0, 4.0, 5.0], [2.0, 8.0, 2.0]], [[5.0, 5.0, 6.0], [7.0, 2.0, 8.0], [4.0, 2.0, 
3.0]]], [[[3.0, 6.0, 6.0], [4.0, 5.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 4.0], [4.0, 1.0, 4.0], [3.0, 5.0, 1.0]], [[1.0, 2.0, 
5.0], [8.0, 8.0, 4.0], [6.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 7.0], [7.0, 3.0, 7.0], [2.0, 5.0, 7.0]], 
[[1.0, 3.0, 3.0], [7.0, 1.0, 5.0], [7.0, 5.0, 3.0]], [[4.0, 3.0, 7.0], [2.0, 5.0, 1.0], [7.0, 4.0, 6.0]]], [[[2.0, 2.0, 5.0], 
[4.0, 2.0, 6.0], [8.0, 8.0, 7.0]], [[4.0, 6.0, 3.0], [7.0, 3.0, 1.0], [6.0, 6.0, 5.0]], [[6.0, 8.0, 8.0], [1.0, 6.0, 4.0], 
[7.0, 7.0, 6.0]]], [[[6.0, 5.0, 5.0], [7.0, 5.0, 7.0], [3.0, 6.0, 8.0]], [[6.0, 7.0, 3.0], [7.0, 4.0, 8.0], [2.0, 8.0, 2.0]], 
[[5.0, 5.0, 6.0], [7.0, 2.0, 8.0], [4.0, 2.0, 3.0]]], [[[3.0, 6.0, 6.0], [4.0, 5.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 4.0], 
[4.0, 1.0, 4.0], [3.0, 5.0, 1.0]], [[1.0, 2.0, 5.0], [8.0, 8.0, 4.0], [6.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPUP(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 6.0], [7.0, 6.0, 7.0], [7.0, 8.0, 8.0]], [[4.0, 3.0, 7.0], [2.0, 4.0, 2.0], [8.0, 4.0, 
4.0]], [[1.0, 7.0, 5.0], [2.0, 5.0, 2.0], [5.0, 7.0, 2.0]]], [[[1.0, 2.0, 7.0], [8.0, 7.0, 4.0], [4.0, 8.0, 5.0]], [[2.0, 2.0, 
4.0], [6.0, 6.0, 2.0], [5.0, 8.0, 5.0]], [[7.0, 6.0, 7.0], [6.0, 3.0, 5.0], [4.0, 5.0, 2.0]]], [[[6.0, 8.0, 5.0], [8.0, 7.0, 
8.0], [1.0, 7.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 4.0, 3.0], [2.0, 8.0, 4.0]], [[8.0, 8.0, 1.0], [1.0, 2.0, 6.0], [5.0, 6.0, 
6.0]]], [[[6.0, 1.0, 5.0], [4.0, 5.0, 8.0], [2.0, 8.0, 6.0]], [[7.0, 2.0, 4.0], [5.0, 2.0, 2.0], [5.0, 1.0, 5.0]], [[6.0, 4.0, 
6.0], [1.0, 1.0, 7.0], [6.0, 8.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[3.0, 6.0], [8.0, 7.0]])
      arg[1:3,1,:2,1]=val
      ref=Data(numpy.array([[[[1.0, 2.0, 6.0], [7.0, 6.0, 7.0], [7.0, 8.0, 8.0]], [[4.0, 3.0, 7.0], [2.0, 4.0, 2.0], [8.0, 4.0, 
4.0]], [[1.0, 7.0, 5.0], [2.0, 5.0, 2.0], [5.0, 7.0, 2.0]]], [[[1.0, 2.0, 7.0], [8.0, 7.0, 4.0], [4.0, 8.0, 5.0]], [[2.0, 3.0, 
4.0], [6.0, 6.0, 2.0], [5.0, 8.0, 5.0]], [[7.0, 6.0, 7.0], [6.0, 3.0, 5.0], [4.0, 5.0, 2.0]]], [[[6.0, 8.0, 5.0], [8.0, 7.0, 
8.0], [1.0, 7.0, 7.0]], [[4.0, 8.0, 4.0], [4.0, 7.0, 3.0], [2.0, 8.0, 4.0]], [[8.0, 8.0, 1.0], [1.0, 2.0, 6.0], [5.0, 6.0, 
6.0]]], [[[6.0, 1.0, 5.0], [4.0, 5.0, 8.0], [2.0, 8.0, 6.0]], [[7.0, 2.0, 4.0], [5.0, 2.0, 2.0], [5.0, 1.0, 5.0]], [[6.0, 4.0, 
6.0], [1.0, 1.0, 7.0], [6.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPUP(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 1.0], [1.0, 4.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 1.0, 4.0], [1.0, 7.0, 5.0], [1.0, 2.0, 
2.0]], [[5.0, 1.0, 7.0], [6.0, 4.0, 4.0], [1.0, 8.0, 1.0]]], [[[3.0, 2.0, 1.0], [2.0, 4.0, 8.0], [8.0, 6.0, 2.0]], [[1.0, 7.0, 
4.0], [7.0, 3.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 6.0, 1.0], [5.0, 2.0, 8.0], [4.0, 3.0, 3.0]]], [[[4.0, 2.0, 1.0], [8.0, 2.0, 
1.0], [3.0, 2.0, 4.0]], [[1.0, 3.0, 4.0], [2.0, 7.0, 3.0], [6.0, 7.0, 5.0]], [[3.0, 6.0, 2.0], [7.0, 4.0, 3.0], [3.0, 3.0, 
2.0]]], [[[8.0, 6.0, 1.0], [4.0, 6.0, 5.0], [7.0, 7.0, 5.0]], [[3.0, 2.0, 5.0], [5.0, 5.0, 6.0], [7.0, 2.0, 1.0]], [[3.0, 7.0, 
4.0], [6.0, 4.0, 5.0], [4.0, 4.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 6.0], [2.0, 3.0]]),self.functionspace)
      arg[1:3,1,:2,1]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 1.0], [1.0, 4.0, 7.0], [2.0, 8.0, 4.0]], [[2.0, 1.0, 4.0], [1.0, 7.0, 5.0], [1.0, 2.0, 
2.0]], [[5.0, 1.0, 7.0], [6.0, 4.0, 4.0], [1.0, 8.0, 1.0]]], [[[3.0, 2.0, 1.0], [2.0, 4.0, 8.0], [8.0, 6.0, 2.0]], [[1.0, 1.0, 
4.0], [7.0, 6.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 6.0, 1.0], [5.0, 2.0, 8.0], [4.0, 3.0, 3.0]]], [[[4.0, 2.0, 1.0], [8.0, 2.0, 
1.0], [3.0, 2.0, 4.0]], [[1.0, 2.0, 4.0], [2.0, 3.0, 3.0], [6.0, 7.0, 5.0]], [[3.0, 6.0, 2.0], [7.0, 4.0, 3.0], [3.0, 3.0, 
2.0]]], [[[8.0, 6.0, 1.0], [4.0, 6.0, 5.0], [7.0, 7.0, 5.0]], [[3.0, 2.0, 5.0], [5.0, 5.0, 6.0], [7.0, 2.0, 1.0]], [[3.0, 7.0, 
4.0], [6.0, 4.0, 5.0], [4.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPUP(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 1.0], [5.0, 6.0, 1.0], [7.0, 5.0, 5.0]], [[7.0, 1.0, 4.0], [5.0, 1.0, 2.0], [4.0, 8.0, 
7.0]], [[7.0, 2.0, 8.0], [2.0, 6.0, 2.0], [5.0, 4.0, 1.0]]], [[[3.0, 1.0, 2.0], [6.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 4.0, 
4.0], [3.0, 7.0, 7.0], [2.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [2.0, 4.0, 2.0], [4.0, 5.0, 5.0]]], [[[2.0, 6.0, 8.0], [5.0, 6.0, 
2.0], [4.0, 7.0, 6.0]], [[1.0, 3.0, 6.0], [2.0, 5.0, 6.0], [1.0, 8.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 6.0, 1.0], [2.0, 2.0, 
7.0]]], [[[7.0, 8.0, 4.0], [8.0, 8.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 4.0, 3.0], [6.0, 5.0, 5.0], [2.0, 8.0, 4.0]], [[5.0, 2.0, 
4.0], [6.0, 7.0, 5.0], [2.0, 7.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 8.0], [3.0, 5.0]])+(1.-msk_val)*numpy.array([[8.0, 3.0], [4.0, 1.0]])
      arg[1:3,1,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 1.0], [5.0, 6.0, 1.0], [7.0, 5.0, 5.0]], [[7.0, 1.0, 4.0], [5.0, 1.0, 2.0], [4.0, 
8.0, 7.0]], [[7.0, 2.0, 8.0], [2.0, 6.0, 2.0], [5.0, 4.0, 1.0]]], [[[3.0, 1.0, 2.0], [6.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 
4.0, 4.0], [3.0, 8.0, 7.0], [2.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [2.0, 4.0, 2.0], [4.0, 5.0, 5.0]]], [[[2.0, 6.0, 8.0], [5.0, 
6.0, 2.0], [4.0, 7.0, 6.0]], [[1.0, 3.0, 6.0], [2.0, 5.0, 6.0], [1.0, 8.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 6.0, 1.0], [2.0, 2.0, 
7.0]]], [[[7.0, 8.0, 4.0], [8.0, 8.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 4.0, 3.0], [6.0, 5.0, 5.0], [2.0, 8.0, 4.0]], [[5.0, 2.0, 
4.0], [6.0, 7.0, 5.0], [2.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 1.0], [5.0, 6.0, 1.0], [7.0, 5.0, 5.0]], 
[[7.0, 1.0, 4.0], [5.0, 1.0, 2.0], [4.0, 8.0, 7.0]], [[7.0, 2.0, 8.0], [2.0, 6.0, 2.0], [5.0, 4.0, 1.0]]], [[[3.0, 1.0, 2.0], 
[6.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[3.0, 8.0, 4.0], [3.0, 3.0, 7.0], [2.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [2.0, 4.0, 2.0], 
[4.0, 5.0, 5.0]]], [[[2.0, 6.0, 8.0], [5.0, 6.0, 2.0], [4.0, 7.0, 6.0]], [[1.0, 4.0, 6.0], [2.0, 1.0, 6.0], [1.0, 8.0, 2.0]], 
[[7.0, 8.0, 3.0], [8.0, 6.0, 1.0], [2.0, 2.0, 7.0]]], [[[7.0, 8.0, 4.0], [8.0, 8.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 4.0, 3.0], 
[6.0, 5.0, 5.0], [2.0, 8.0, 4.0]], [[5.0, 2.0, 4.0], [6.0, 7.0, 5.0], [2.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRPP(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 1.0], [1.0, 6.0, 5.0], [8.0, 1.0, 4.0]], [[1.0, 4.0, 3.0], [4.0, 5.0, 7.0], [4.0, 6.0, 
2.0]], [[4.0, 2.0, 7.0], [5.0, 5.0, 6.0], [4.0, 3.0, 3.0]]], [[[5.0, 8.0, 3.0], [1.0, 8.0, 6.0], [3.0, 5.0, 8.0]], [[5.0, 5.0, 
1.0], [6.0, 3.0, 8.0], [8.0, 3.0, 7.0]], [[5.0, 1.0, 4.0], [4.0, 1.0, 8.0], [8.0, 8.0, 6.0]]], [[[7.0, 6.0, 4.0], [3.0, 5.0, 
3.0], [1.0, 5.0, 2.0]], [[1.0, 1.0, 1.0], [5.0, 8.0, 2.0], [1.0, 7.0, 3.0]], [[8.0, 4.0, 8.0], [5.0, 5.0, 8.0], [8.0, 5.0, 
7.0]]], [[[1.0, 2.0, 4.0], [8.0, 3.0, 6.0], [1.0, 7.0, 3.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 6.0], [3.0, 5.0, 8.0]], [[5.0, 4.0, 
7.0], [7.0, 7.0, 4.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([3.0])
      arg[2,1:2,1,1]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 1.0], [1.0, 6.0, 5.0], [8.0, 1.0, 4.0]], [[1.0, 4.0, 3.0], [4.0, 5.0, 7.0], [4.0, 6.0, 
2.0]], [[4.0, 2.0, 7.0], [5.0, 5.0, 6.0], [4.0, 3.0, 3.0]]], [[[5.0, 8.0, 3.0], [1.0, 8.0, 6.0], [3.0, 5.0, 8.0]], [[5.0, 5.0, 
1.0], [6.0, 3.0, 8.0], [8.0, 3.0, 7.0]], [[5.0, 1.0, 4.0], [4.0, 1.0, 8.0], [8.0, 8.0, 6.0]]], [[[7.0, 6.0, 4.0], [3.0, 5.0, 
3.0], [1.0, 5.0, 2.0]], [[1.0, 1.0, 1.0], [5.0, 3.0, 2.0], [1.0, 7.0, 3.0]], [[8.0, 4.0, 8.0], [5.0, 5.0, 8.0], [8.0, 5.0, 
7.0]]], [[[1.0, 2.0, 4.0], [8.0, 3.0, 6.0], [1.0, 7.0, 3.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 6.0], [3.0, 5.0, 8.0]], [[5.0, 4.0, 
7.0], [7.0, 7.0, 4.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRPP(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [5.0, 2.0, 7.0], [2.0, 4.0, 1.0]], [[6.0, 5.0, 8.0], [8.0, 2.0, 6.0], [5.0, 2.0, 
3.0]], [[1.0, 3.0, 1.0], [2.0, 5.0, 8.0], [7.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], [2.0, 1.0, 4.0], [5.0, 2.0, 5.0]], [[1.0, 6.0, 
6.0], [4.0, 6.0, 7.0], [5.0, 3.0, 3.0]], [[3.0, 2.0, 5.0], [7.0, 8.0, 6.0], [3.0, 2.0, 6.0]]], [[[3.0, 2.0, 2.0], [6.0, 1.0, 
7.0], [4.0, 4.0, 8.0]], [[4.0, 8.0, 6.0], [4.0, 5.0, 4.0], [6.0, 4.0, 5.0]], [[4.0, 4.0, 1.0], [4.0, 8.0, 8.0], [4.0, 3.0, 
8.0]]], [[[6.0, 5.0, 1.0], [7.0, 2.0, 6.0], [5.0, 2.0, 8.0]], [[5.0, 7.0, 1.0], [4.0, 8.0, 5.0], [7.0, 8.0, 4.0]], [[3.0, 2.0, 
4.0], [7.0, 2.0, 5.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([3.0]),self.functionspace)
      arg[2,1:2,1,1]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 5.0], [5.0, 2.0, 7.0], [2.0, 4.0, 1.0]], [[6.0, 5.0, 8.0], [8.0, 2.0, 6.0], [5.0, 2.0, 
3.0]], [[1.0, 3.0, 1.0], [2.0, 5.0, 8.0], [7.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], [2.0, 1.0, 4.0], [5.0, 2.0, 5.0]], [[1.0, 6.0, 
6.0], [4.0, 6.0, 7.0], [5.0, 3.0, 3.0]], [[3.0, 2.0, 5.0], [7.0, 8.0, 6.0], [3.0, 2.0, 6.0]]], [[[3.0, 2.0, 2.0], [6.0, 1.0, 
7.0], [4.0, 4.0, 8.0]], [[4.0, 8.0, 6.0], [4.0, 3.0, 4.0], [6.0, 4.0, 5.0]], [[4.0, 4.0, 1.0], [4.0, 8.0, 8.0], [4.0, 3.0, 
8.0]]], [[[6.0, 5.0, 1.0], [7.0, 2.0, 6.0], [5.0, 2.0, 8.0]], [[5.0, 7.0, 1.0], [4.0, 8.0, 5.0], [7.0, 8.0, 4.0]], [[3.0, 2.0, 
4.0], [7.0, 2.0, 5.0], [8.0, 5.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRPP(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 2.0], [3.0, 8.0, 4.0], [2.0, 1.0, 6.0]], [[7.0, 7.0, 5.0], [8.0, 2.0, 6.0], [5.0, 6.0, 
2.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 2.0], [5.0, 7.0, 5.0]]], [[[1.0, 8.0, 1.0], [5.0, 1.0, 4.0], [3.0, 8.0, 5.0]], [[8.0, 4.0, 
3.0], [7.0, 5.0, 6.0], [5.0, 2.0, 8.0]], [[8.0, 3.0, 5.0], [8.0, 5.0, 7.0], [3.0, 1.0, 8.0]]], [[[4.0, 7.0, 8.0], [2.0, 1.0, 
5.0], [5.0, 4.0, 4.0]], [[3.0, 7.0, 6.0], [8.0, 5.0, 3.0], [7.0, 3.0, 4.0]], [[3.0, 1.0, 3.0], [1.0, 8.0, 8.0], [1.0, 4.0, 
7.0]]], [[[6.0, 1.0, 1.0], [2.0, 3.0, 6.0], [2.0, 5.0, 4.0]], [[3.0, 3.0, 6.0], [6.0, 7.0, 2.0], [8.0, 8.0, 7.0]], [[4.0, 7.0, 
3.0], [7.0, 1.0, 1.0], [5.0, 1.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([4.0])+(1.-msk_val)*numpy.array([7.0])
      arg[2,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 2.0], [3.0, 8.0, 4.0], [2.0, 1.0, 6.0]], [[7.0, 7.0, 5.0], [8.0, 2.0, 6.0], [5.0, 
6.0, 2.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 2.0], [5.0, 7.0, 5.0]]], [[[1.0, 8.0, 1.0], [5.0, 1.0, 4.0], [3.0, 8.0, 5.0]], [[8.0, 
4.0, 3.0], [7.0, 5.0, 6.0], [5.0, 2.0, 8.0]], [[8.0, 3.0, 5.0], [8.0, 5.0, 7.0], [3.0, 1.0, 8.0]]], [[[4.0, 7.0, 8.0], [2.0, 
1.0, 5.0], [5.0, 4.0, 4.0]], [[3.0, 7.0, 6.0], [8.0, 4.0, 3.0], [7.0, 3.0, 4.0]], [[3.0, 1.0, 3.0], [1.0, 8.0, 8.0], [1.0, 4.0, 
7.0]]], [[[6.0, 1.0, 1.0], [2.0, 3.0, 6.0], [2.0, 5.0, 4.0]], [[3.0, 3.0, 6.0], [6.0, 7.0, 2.0], [8.0, 8.0, 7.0]], [[4.0, 7.0, 
3.0], [7.0, 1.0, 1.0], [5.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 2.0], [3.0, 8.0, 4.0], [2.0, 1.0, 6.0]], 
[[7.0, 7.0, 5.0], [8.0, 2.0, 6.0], [5.0, 6.0, 2.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 2.0], [5.0, 7.0, 5.0]]], [[[1.0, 8.0, 1.0], 
[5.0, 1.0, 4.0], [3.0, 8.0, 5.0]], [[8.0, 4.0, 3.0], [7.0, 5.0, 6.0], [5.0, 2.0, 8.0]], [[8.0, 3.0, 5.0], [8.0, 5.0, 7.0], 
[3.0, 1.0, 8.0]]], [[[4.0, 7.0, 8.0], [2.0, 1.0, 5.0], [5.0, 4.0, 4.0]], [[3.0, 7.0, 6.0], [8.0, 7.0, 3.0], [7.0, 3.0, 4.0]], 
[[3.0, 1.0, 3.0], [1.0, 8.0, 8.0], [1.0, 4.0, 7.0]]], [[[6.0, 1.0, 1.0], [2.0, 3.0, 6.0], [2.0, 5.0, 4.0]], [[3.0, 3.0, 6.0], 
[6.0, 7.0, 2.0], [8.0, 8.0, 7.0]], [[4.0, 7.0, 3.0], [7.0, 1.0, 1.0], [5.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLPC(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 1.0], [7.0, 6.0, 5.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 2.0], [2.0, 6.0, 5.0], [6.0, 5.0, 
2.0]], [[7.0, 5.0, 4.0], [5.0, 5.0, 3.0], [2.0, 5.0, 6.0]]], [[[6.0, 6.0, 6.0], [7.0, 8.0, 6.0], [8.0, 3.0, 6.0]], [[3.0, 5.0, 
6.0], [7.0, 8.0, 3.0], [3.0, 3.0, 1.0]], [[1.0, 2.0, 8.0], [8.0, 1.0, 5.0], [6.0, 5.0, 7.0]]], [[[3.0, 8.0, 4.0], [4.0, 8.0, 
8.0], [6.0, 3.0, 5.0]], [[8.0, 8.0, 8.0], [4.0, 8.0, 1.0], [7.0, 3.0, 3.0]], [[6.0, 8.0, 1.0], [8.0, 1.0, 2.0], [5.0, 6.0, 
2.0]]], [[[1.0, 5.0, 1.0], [8.0, 5.0, 7.0], [1.0, 4.0, 3.0]], [[7.0, 4.0, 2.0], [7.0, 6.0, 1.0], [5.0, 3.0, 8.0]], [[2.0, 8.0, 
5.0], [6.0, 8.0, 7.0], [4.0, 3.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 3.0, 8.0]], [[4.0, 2.0, 1.0]]])
      arg[2:,2:,1,:]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 1.0], [7.0, 6.0, 5.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 2.0], [2.0, 6.0, 5.0], [6.0, 5.0, 
2.0]], [[7.0, 5.0, 4.0], [5.0, 5.0, 3.0], [2.0, 5.0, 6.0]]], [[[6.0, 6.0, 6.0], [7.0, 8.0, 6.0], [8.0, 3.0, 6.0]], [[3.0, 5.0, 
6.0], [7.0, 8.0, 3.0], [3.0, 3.0, 1.0]], [[1.0, 2.0, 8.0], [8.0, 1.0, 5.0], [6.0, 5.0, 7.0]]], [[[3.0, 8.0, 4.0], [4.0, 8.0, 
8.0], [6.0, 3.0, 5.0]], [[8.0, 8.0, 8.0], [4.0, 8.0, 1.0], [7.0, 3.0, 3.0]], [[6.0, 8.0, 1.0], [1.0, 3.0, 8.0], [5.0, 6.0, 
2.0]]], [[[1.0, 5.0, 1.0], [8.0, 5.0, 7.0], [1.0, 4.0, 3.0]], [[7.0, 4.0, 2.0], [7.0, 6.0, 1.0], [5.0, 3.0, 8.0]], [[2.0, 8.0, 
5.0], [4.0, 2.0, 1.0], [4.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLPC(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 6.0], [6.0, 4.0, 3.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 3.0], [3.0, 4.0, 2.0], [6.0, 4.0, 
2.0]], [[2.0, 5.0, 3.0], [6.0, 8.0, 1.0], [3.0, 2.0, 7.0]]], [[[7.0, 8.0, 5.0], [7.0, 2.0, 8.0], [8.0, 8.0, 7.0]], [[1.0, 3.0, 
6.0], [8.0, 5.0, 7.0], [6.0, 6.0, 5.0]], [[6.0, 5.0, 5.0], [4.0, 8.0, 4.0], [8.0, 1.0, 5.0]]], [[[7.0, 8.0, 1.0], [2.0, 7.0, 
7.0], [4.0, 6.0, 7.0]], [[2.0, 4.0, 3.0], [8.0, 5.0, 5.0], [3.0, 2.0, 6.0]], [[5.0, 7.0, 4.0], [6.0, 7.0, 2.0], [6.0, 5.0, 
5.0]]], [[[5.0, 2.0, 5.0], [7.0, 7.0, 1.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 4.0], [7.0, 3.0, 4.0], [8.0, 6.0, 3.0]], [[1.0, 5.0, 
5.0], [7.0, 4.0, 1.0], [1.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 8.0, 1.0]], [[2.0, 4.0, 4.0]]]),self.functionspace)
      arg[2:,2:,1,:]=val
      ref=Data(numpy.array([[[[4.0, 2.0, 6.0], [6.0, 4.0, 3.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 3.0], [3.0, 4.0, 2.0], [6.0, 4.0, 
2.0]], [[2.0, 5.0, 3.0], [6.0, 8.0, 1.0], [3.0, 2.0, 7.0]]], [[[7.0, 8.0, 5.0], [7.0, 2.0, 8.0], [8.0, 8.0, 7.0]], [[1.0, 3.0, 
6.0], [8.0, 5.0, 7.0], [6.0, 6.0, 5.0]], [[6.0, 5.0, 5.0], [4.0, 8.0, 4.0], [8.0, 1.0, 5.0]]], [[[7.0, 8.0, 1.0], [2.0, 7.0, 
7.0], [4.0, 6.0, 7.0]], [[2.0, 4.0, 3.0], [8.0, 5.0, 5.0], [3.0, 2.0, 6.0]], [[5.0, 7.0, 4.0], [2.0, 8.0, 1.0], [6.0, 5.0, 
5.0]]], [[[5.0, 2.0, 5.0], [7.0, 7.0, 1.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 4.0], [7.0, 3.0, 4.0], [8.0, 6.0, 3.0]], [[1.0, 5.0, 
5.0], [2.0, 4.0, 4.0], [1.0, 8.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLPC(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 3.0], [2.0, 5.0, 2.0], [8.0, 7.0, 5.0]], [[7.0, 8.0, 2.0], [1.0, 3.0, 4.0], [7.0, 4.0, 
3.0]], [[5.0, 4.0, 3.0], [8.0, 3.0, 2.0], [5.0, 5.0, 4.0]]], [[[8.0, 2.0, 8.0], [7.0, 2.0, 4.0], [5.0, 7.0, 7.0]], [[1.0, 6.0, 
5.0], [1.0, 8.0, 6.0], [8.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 5.0], [6.0, 5.0, 4.0]]], [[[3.0, 1.0, 5.0], [8.0, 5.0, 
4.0], [3.0, 2.0, 5.0]], [[3.0, 5.0, 7.0], [6.0, 4.0, 8.0], [8.0, 4.0, 4.0]], [[5.0, 4.0, 8.0], [4.0, 5.0, 5.0], [6.0, 7.0, 
1.0]]], [[[6.0, 1.0, 8.0], [6.0, 1.0, 6.0], [2.0, 2.0, 4.0]], [[6.0, 7.0, 6.0], [4.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[1.0, 8.0, 
5.0], [2.0, 5.0, 2.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 8.0, 5.0]], [[5.0, 2.0, 3.0]]])+(1.-msk_val)*numpy.array([[[5.0, 7.0, 1.0]], [[4.0, 3.0, 
8.0]]])
      arg[2:,2:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 3.0], [2.0, 5.0, 2.0], [8.0, 7.0, 5.0]], [[7.0, 8.0, 2.0], [1.0, 3.0, 4.0], [7.0, 
4.0, 3.0]], [[5.0, 4.0, 3.0], [8.0, 3.0, 2.0], [5.0, 5.0, 4.0]]], [[[8.0, 2.0, 8.0], [7.0, 2.0, 4.0], [5.0, 7.0, 7.0]], [[1.0, 
6.0, 5.0], [1.0, 8.0, 6.0], [8.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 5.0], [6.0, 5.0, 4.0]]], [[[3.0, 1.0, 5.0], [8.0, 
5.0, 4.0], [3.0, 2.0, 5.0]], [[3.0, 5.0, 7.0], [6.0, 4.0, 8.0], [8.0, 4.0, 4.0]], [[5.0, 4.0, 8.0], [6.0, 8.0, 5.0], [6.0, 7.0, 
1.0]]], [[[6.0, 1.0, 8.0], [6.0, 1.0, 6.0], [2.0, 2.0, 4.0]], [[6.0, 7.0, 6.0], [4.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[1.0, 8.0, 
5.0], [5.0, 2.0, 3.0], [6.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 3.0], [2.0, 5.0, 2.0], [8.0, 7.0, 5.0]], 
[[7.0, 8.0, 2.0], [1.0, 3.0, 4.0], [7.0, 4.0, 3.0]], [[5.0, 4.0, 3.0], [8.0, 3.0, 2.0], [5.0, 5.0, 4.0]]], [[[8.0, 2.0, 8.0], 
[7.0, 2.0, 4.0], [5.0, 7.0, 7.0]], [[1.0, 6.0, 5.0], [1.0, 8.0, 6.0], [8.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 5.0], 
[6.0, 5.0, 4.0]]], [[[3.0, 1.0, 5.0], [8.0, 5.0, 4.0], [3.0, 2.0, 5.0]], [[3.0, 5.0, 7.0], [6.0, 4.0, 8.0], [8.0, 4.0, 4.0]], 
[[5.0, 4.0, 8.0], [5.0, 7.0, 1.0], [6.0, 7.0, 1.0]]], [[[6.0, 1.0, 8.0], [6.0, 1.0, 6.0], [2.0, 2.0, 4.0]], [[6.0, 7.0, 6.0], 
[4.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[1.0, 8.0, 5.0], [4.0, 3.0, 8.0], [6.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LUPU(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 3.0], [5.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[8.0, 5.0, 5.0], [4.0, 7.0, 6.0], [5.0, 8.0, 
6.0]], [[3.0, 2.0, 5.0], [2.0, 1.0, 3.0], [6.0, 4.0, 8.0]]], [[[1.0, 8.0, 8.0], [5.0, 8.0, 6.0], [7.0, 4.0, 1.0]], [[6.0, 2.0, 
5.0], [5.0, 2.0, 1.0], [6.0, 8.0, 7.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 6.0], [3.0, 8.0, 8.0]]], [[[2.0, 3.0, 5.0], [5.0, 5.0, 
5.0], [3.0, 3.0, 1.0]], [[4.0, 7.0, 4.0], [3.0, 2.0, 8.0], [8.0, 8.0, 8.0]], [[2.0, 2.0, 3.0], [5.0, 7.0, 3.0], [1.0, 2.0, 
4.0]]], [[[4.0, 4.0, 2.0], [8.0, 3.0, 8.0], [6.0, 7.0, 4.0]], [[7.0, 3.0, 6.0], [5.0, 2.0, 5.0], [7.0, 4.0, 6.0]], [[3.0, 7.0, 
7.0], [8.0, 1.0, 1.0], [4.0, 1.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 2.0], [2.0, 4.0]], [[6.0, 5.0], [6.0, 5.0]]])
      arg[2:,:2,1,:2]=val
      ref=Data(numpy.array([[[[3.0, 3.0, 3.0], [5.0, 5.0, 4.0], [3.0, 8.0, 1.0]], [[8.0, 5.0, 5.0], [4.0, 7.0, 6.0], [5.0, 8.0, 
6.0]], [[3.0, 2.0, 5.0], [2.0, 1.0, 3.0], [6.0, 4.0, 8.0]]], [[[1.0, 8.0, 8.0], [5.0, 8.0, 6.0], [7.0, 4.0, 1.0]], [[6.0, 2.0, 
5.0], [5.0, 2.0, 1.0], [6.0, 8.0, 7.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 6.0], [3.0, 8.0, 8.0]]], [[[2.0, 3.0, 5.0], [4.0, 2.0, 
5.0], [3.0, 3.0, 1.0]], [[4.0, 7.0, 4.0], [2.0, 4.0, 8.0], [8.0, 8.0, 8.0]], [[2.0, 2.0, 3.0], [5.0, 7.0, 3.0], [1.0, 2.0, 
4.0]]], [[[4.0, 4.0, 2.0], [6.0, 5.0, 8.0], [6.0, 7.0, 4.0]], [[7.0, 3.0, 6.0], [6.0, 5.0, 5.0], [7.0, 4.0, 6.0]], [[3.0, 7.0, 
7.0], [8.0, 1.0, 1.0], [4.0, 1.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LUPU(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 7.0], [8.0, 3.0, 4.0], [5.0, 4.0, 8.0]], [[4.0, 6.0, 8.0], [2.0, 7.0, 2.0], [3.0, 4.0, 
8.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 3.0], [8.0, 8.0, 8.0]]], [[[4.0, 7.0, 8.0], [8.0, 8.0, 2.0], [8.0, 5.0, 2.0]], [[8.0, 3.0, 
8.0], [5.0, 3.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 1.0, 5.0], [3.0, 7.0, 2.0], [2.0, 3.0, 1.0]]], [[[4.0, 4.0, 6.0], [6.0, 6.0, 
5.0], [2.0, 6.0, 7.0]], [[7.0, 5.0, 2.0], [1.0, 2.0, 6.0], [6.0, 6.0, 2.0]], [[5.0, 4.0, 6.0], [7.0, 8.0, 7.0], [4.0, 1.0, 
2.0]]], [[[1.0, 3.0, 4.0], [4.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[5.0, 4.0, 7.0], [1.0, 4.0, 5.0], [6.0, 3.0, 7.0]], [[3.0, 5.0, 
4.0], [4.0, 2.0, 5.0], [2.0, 3.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 3.0], [1.0, 3.0]], [[3.0, 1.0], [7.0, 1.0]]]),self.functionspace)
      arg[2:,:2,1,:2]=val
      ref=Data(numpy.array([[[[4.0, 2.0, 7.0], [8.0, 3.0, 4.0], [5.0, 4.0, 8.0]], [[4.0, 6.0, 8.0], [2.0, 7.0, 2.0], [3.0, 4.0, 
8.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 3.0], [8.0, 8.0, 8.0]]], [[[4.0, 7.0, 8.0], [8.0, 8.0, 2.0], [8.0, 5.0, 2.0]], [[8.0, 3.0, 
8.0], [5.0, 3.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 1.0, 5.0], [3.0, 7.0, 2.0], [2.0, 3.0, 1.0]]], [[[4.0, 4.0, 6.0], [3.0, 3.0, 
5.0], [2.0, 6.0, 7.0]], [[7.0, 5.0, 2.0], [1.0, 3.0, 6.0], [6.0, 6.0, 2.0]], [[5.0, 4.0, 6.0], [7.0, 8.0, 7.0], [4.0, 1.0, 
2.0]]], [[[1.0, 3.0, 4.0], [3.0, 1.0, 6.0], [4.0, 7.0, 4.0]], [[5.0, 4.0, 7.0], [7.0, 1.0, 5.0], [6.0, 3.0, 7.0]], [[3.0, 5.0, 
4.0], [4.0, 2.0, 5.0], [2.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LUPU(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 3.0], [1.0, 6.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 5.0], [3.0, 3.0, 2.0], [6.0, 1.0, 
3.0]], [[3.0, 2.0, 8.0], [7.0, 3.0, 5.0], [1.0, 3.0, 6.0]]], [[[6.0, 3.0, 2.0], [6.0, 1.0, 6.0], [7.0, 8.0, 2.0]], [[4.0, 1.0, 
3.0], [2.0, 6.0, 2.0], [2.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 3.0, 2.0], [8.0, 3.0, 6.0]]], [[[6.0, 3.0, 8.0], [3.0, 5.0, 
6.0], [2.0, 6.0, 5.0]], [[4.0, 5.0, 4.0], [1.0, 6.0, 5.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 7.0], [1.0, 8.0, 
3.0]]], [[[1.0, 5.0, 6.0], [5.0, 3.0, 6.0], [4.0, 3.0, 4.0]], [[4.0, 2.0, 8.0], [2.0, 6.0, 3.0], [1.0, 2.0, 6.0]], [[3.0, 3.0, 
4.0], [2.0, 6.0, 4.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 5.0], [7.0, 6.0]], [[2.0, 1.0], [6.0, 5.0]]])+(1.-msk_val)*numpy.array([[[6.0, 5.0], 
[8.0, 3.0]], [[5.0, 6.0], [5.0, 6.0]]])
      arg[2:,:2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 3.0], [1.0, 6.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 5.0], [3.0, 3.0, 2.0], [6.0, 
1.0, 3.0]], [[3.0, 2.0, 8.0], [7.0, 3.0, 5.0], [1.0, 3.0, 6.0]]], [[[6.0, 3.0, 2.0], [6.0, 1.0, 6.0], [7.0, 8.0, 2.0]], [[4.0, 
1.0, 3.0], [2.0, 6.0, 2.0], [2.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 3.0, 2.0], [8.0, 3.0, 6.0]]], [[[6.0, 3.0, 8.0], [7.0, 
5.0, 6.0], [2.0, 6.0, 5.0]], [[4.0, 5.0, 4.0], [7.0, 6.0, 5.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 7.0], [1.0, 8.0, 
3.0]]], [[[1.0, 5.0, 6.0], [2.0, 1.0, 6.0], [4.0, 3.0, 4.0]], [[4.0, 2.0, 8.0], [6.0, 5.0, 3.0], [1.0, 2.0, 6.0]], [[3.0, 3.0, 
4.0], [2.0, 6.0, 4.0], [5.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 3.0], [1.0, 6.0, 2.0], [7.0, 8.0, 2.0]], 
[[7.0, 2.0, 5.0], [3.0, 3.0, 2.0], [6.0, 1.0, 3.0]], [[3.0, 2.0, 8.0], [7.0, 3.0, 5.0], [1.0, 3.0, 6.0]]], [[[6.0, 3.0, 2.0], 
[6.0, 1.0, 6.0], [7.0, 8.0, 2.0]], [[4.0, 1.0, 3.0], [2.0, 6.0, 2.0], [2.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 3.0, 2.0], 
[8.0, 3.0, 6.0]]], [[[6.0, 3.0, 8.0], [6.0, 5.0, 6.0], [2.0, 6.0, 5.0]], [[4.0, 5.0, 4.0], [8.0, 3.0, 5.0], [2.0, 3.0, 3.0]], 
[[7.0, 7.0, 6.0], [3.0, 6.0, 7.0], [1.0, 8.0, 3.0]]], [[[1.0, 5.0, 6.0], [5.0, 6.0, 6.0], [4.0, 3.0, 4.0]], [[4.0, 2.0, 8.0], 
[5.0, 6.0, 3.0], [1.0, 2.0, 6.0]], [[3.0, 3.0, 4.0], [2.0, 6.0, 4.0], [5.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRRP(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 1.0], [2.0, 3.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 7.0, 7.0], [6.0, 6.0, 6.0], [6.0, 2.0, 
4.0]], [[2.0, 2.0, 3.0], [8.0, 2.0, 7.0], [3.0, 2.0, 7.0]]], [[[1.0, 5.0, 7.0], [1.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[6.0, 8.0, 
1.0], [4.0, 6.0, 7.0], [5.0, 7.0, 7.0]], [[4.0, 8.0, 1.0], [6.0, 2.0, 2.0], [1.0, 6.0, 4.0]]], [[[2.0, 3.0, 2.0], [3.0, 4.0, 
5.0], [6.0, 4.0, 8.0]], [[7.0, 3.0, 7.0], [7.0, 8.0, 4.0], [5.0, 1.0, 1.0]], [[5.0, 8.0, 1.0], [3.0, 2.0, 8.0], [5.0, 3.0, 
6.0]]], [[[6.0, 7.0, 8.0], [4.0, 2.0, 3.0], [1.0, 7.0, 1.0]], [[8.0, 7.0, 1.0], [6.0, 4.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 6.0, 
3.0], [1.0, 5.0, 8.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[3.0]], [[3.0]]])
      arg[1:3,1:2,1:2,1]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 1.0], [2.0, 3.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 7.0, 7.0], [6.0, 6.0, 6.0], [6.0, 2.0, 
4.0]], [[2.0, 2.0, 3.0], [8.0, 2.0, 7.0], [3.0, 2.0, 7.0]]], [[[1.0, 5.0, 7.0], [1.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[6.0, 8.0, 
1.0], [4.0, 3.0, 7.0], [5.0, 7.0, 7.0]], [[4.0, 8.0, 1.0], [6.0, 2.0, 2.0], [1.0, 6.0, 4.0]]], [[[2.0, 3.0, 2.0], [3.0, 4.0, 
5.0], [6.0, 4.0, 8.0]], [[7.0, 3.0, 7.0], [7.0, 3.0, 4.0], [5.0, 1.0, 1.0]], [[5.0, 8.0, 1.0], [3.0, 2.0, 8.0], [5.0, 3.0, 
6.0]]], [[[6.0, 7.0, 8.0], [4.0, 2.0, 3.0], [1.0, 7.0, 1.0]], [[8.0, 7.0, 1.0], [6.0, 4.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 6.0, 
3.0], [1.0, 5.0, 8.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRRP(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 3.0], [1.0, 3.0, 7.0], [2.0, 7.0, 4.0]], [[5.0, 2.0, 7.0], [1.0, 7.0, 8.0], [1.0, 8.0, 
4.0]], [[4.0, 8.0, 8.0], [3.0, 5.0, 4.0], [1.0, 7.0, 6.0]]], [[[7.0, 7.0, 8.0], [6.0, 7.0, 8.0], [8.0, 1.0, 8.0]], [[1.0, 5.0, 
6.0], [2.0, 7.0, 6.0], [6.0, 2.0, 7.0]], [[6.0, 4.0, 5.0], [3.0, 5.0, 1.0], [3.0, 5.0, 2.0]]], [[[1.0, 8.0, 4.0], [1.0, 2.0, 
7.0], [7.0, 8.0, 5.0]], [[5.0, 4.0, 5.0], [6.0, 6.0, 8.0], [2.0, 5.0, 1.0]], [[5.0, 4.0, 7.0], [8.0, 6.0, 2.0], [1.0, 5.0, 
6.0]]], [[[3.0, 4.0, 5.0], [1.0, 1.0, 5.0], [6.0, 4.0, 3.0]], [[7.0, 6.0, 5.0], [5.0, 5.0, 6.0], [1.0, 8.0, 1.0]], [[4.0, 5.0, 
2.0], [5.0, 3.0, 4.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0]], [[8.0]]]),self.functionspace)
      arg[1:3,1:2,1:2,1]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 3.0], [1.0, 3.0, 7.0], [2.0, 7.0, 4.0]], [[5.0, 2.0, 7.0], [1.0, 7.0, 8.0], [1.0, 8.0, 
4.0]], [[4.0, 8.0, 8.0], [3.0, 5.0, 4.0], [1.0, 7.0, 6.0]]], [[[7.0, 7.0, 8.0], [6.0, 7.0, 8.0], [8.0, 1.0, 8.0]], [[1.0, 5.0, 
6.0], [2.0, 8.0, 6.0], [6.0, 2.0, 7.0]], [[6.0, 4.0, 5.0], [3.0, 5.0, 1.0], [3.0, 5.0, 2.0]]], [[[1.0, 8.0, 4.0], [1.0, 2.0, 
7.0], [7.0, 8.0, 5.0]], [[5.0, 4.0, 5.0], [6.0, 8.0, 8.0], [2.0, 5.0, 1.0]], [[5.0, 4.0, 7.0], [8.0, 6.0, 2.0], [1.0, 5.0, 
6.0]]], [[[3.0, 4.0, 5.0], [1.0, 1.0, 5.0], [6.0, 4.0, 3.0]], [[7.0, 6.0, 5.0], [5.0, 5.0, 6.0], [1.0, 8.0, 1.0]], [[4.0, 5.0, 
2.0], [5.0, 3.0, 4.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRRP(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 7.0], [7.0, 6.0, 3.0], [3.0, 1.0, 5.0]], [[1.0, 2.0, 6.0], [6.0, 3.0, 6.0], [4.0, 8.0, 
7.0]], [[2.0, 4.0, 4.0], [3.0, 2.0, 1.0], [3.0, 6.0, 7.0]]], [[[3.0, 7.0, 7.0], [7.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[5.0, 5.0, 
2.0], [2.0, 6.0, 1.0], [3.0, 2.0, 3.0]], [[6.0, 5.0, 5.0], [1.0, 8.0, 7.0], [5.0, 2.0, 5.0]]], [[[7.0, 5.0, 3.0], [5.0, 5.0, 
2.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 5.0], [2.0, 7.0, 4.0], [4.0, 1.0, 1.0]], [[5.0, 4.0, 7.0], [5.0, 8.0, 5.0], [8.0, 5.0, 
3.0]]], [[[3.0, 5.0, 7.0], [8.0, 1.0, 7.0], [1.0, 4.0, 8.0]], [[6.0, 1.0, 6.0], [8.0, 5.0, 1.0], [8.0, 6.0, 7.0]], [[8.0, 3.0, 
5.0], [3.0, 7.0, 5.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0]], [[2.0]]])+(1.-msk_val)*numpy.array([[[5.0]], [[1.0]]])
      arg[1:3,1:2,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 7.0], [7.0, 6.0, 3.0], [3.0, 1.0, 5.0]], [[1.0, 2.0, 6.0], [6.0, 3.0, 6.0], [4.0, 
8.0, 7.0]], [[2.0, 4.0, 4.0], [3.0, 2.0, 1.0], [3.0, 6.0, 7.0]]], [[[3.0, 7.0, 7.0], [7.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[5.0, 
5.0, 2.0], [2.0, 3.0, 1.0], [3.0, 2.0, 3.0]], [[6.0, 5.0, 5.0], [1.0, 8.0, 7.0], [5.0, 2.0, 5.0]]], [[[7.0, 5.0, 3.0], [5.0, 
5.0, 2.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 5.0], [2.0, 2.0, 4.0], [4.0, 1.0, 1.0]], [[5.0, 4.0, 7.0], [5.0, 8.0, 5.0], [8.0, 5.0, 
3.0]]], [[[3.0, 5.0, 7.0], [8.0, 1.0, 7.0], [1.0, 4.0, 8.0]], [[6.0, 1.0, 6.0], [8.0, 5.0, 1.0], [8.0, 6.0, 7.0]], [[8.0, 3.0, 
5.0], [3.0, 7.0, 5.0], [1.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 7.0], [7.0, 6.0, 3.0], [3.0, 1.0, 5.0]], 
[[1.0, 2.0, 6.0], [6.0, 3.0, 6.0], [4.0, 8.0, 7.0]], [[2.0, 4.0, 4.0], [3.0, 2.0, 1.0], [3.0, 6.0, 7.0]]], [[[3.0, 7.0, 7.0], 
[7.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[5.0, 5.0, 2.0], [2.0, 5.0, 1.0], [3.0, 2.0, 3.0]], [[6.0, 5.0, 5.0], [1.0, 8.0, 7.0], 
[5.0, 2.0, 5.0]]], [[[7.0, 5.0, 3.0], [5.0, 5.0, 2.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 5.0], [2.0, 1.0, 4.0], [4.0, 1.0, 1.0]], 
[[5.0, 4.0, 7.0], [5.0, 8.0, 5.0], [8.0, 5.0, 3.0]]], [[[3.0, 5.0, 7.0], [8.0, 1.0, 7.0], [1.0, 4.0, 8.0]], [[6.0, 1.0, 6.0], 
[8.0, 5.0, 1.0], [8.0, 6.0, 7.0]], [[8.0, 3.0, 5.0], [3.0, 7.0, 5.0], [1.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCPP(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 1.0], [4.0, 8.0, 2.0], [8.0, 8.0, 5.0]], [[4.0, 5.0, 7.0], [3.0, 8.0, 1.0], [7.0, 1.0, 
4.0]], [[6.0, 8.0, 5.0], [3.0, 6.0, 8.0], [5.0, 3.0, 6.0]]], [[[3.0, 4.0, 7.0], [8.0, 5.0, 1.0], [5.0, 1.0, 7.0]], [[1.0, 8.0, 
6.0], [4.0, 4.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 2.0, 7.0], [5.0, 8.0, 8.0], [3.0, 2.0, 4.0]]], [[[4.0, 2.0, 5.0], [3.0, 1.0, 
8.0], [3.0, 3.0, 6.0]], [[8.0, 5.0, 2.0], [2.0, 2.0, 6.0], [5.0, 3.0, 6.0]], [[7.0, 1.0, 8.0], [5.0, 8.0, 3.0], [7.0, 5.0, 
3.0]]], [[[5.0, 8.0, 4.0], [3.0, 3.0, 6.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 8.0], [6.0, 3.0, 5.0], [1.0, 4.0, 3.0]], [[2.0, 1.0, 
8.0], [4.0, 1.0, 1.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[3.0, 3.0, 5.0], [1.0, 5.0, 1.0]])
      arg[1:3,:,1,1]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 1.0], [4.0, 8.0, 2.0], [8.0, 8.0, 5.0]], [[4.0, 5.0, 7.0], [3.0, 8.0, 1.0], [7.0, 1.0, 
4.0]], [[6.0, 8.0, 5.0], [3.0, 6.0, 8.0], [5.0, 3.0, 6.0]]], [[[3.0, 4.0, 7.0], [8.0, 3.0, 1.0], [5.0, 1.0, 7.0]], [[1.0, 8.0, 
6.0], [4.0, 3.0, 7.0], [6.0, 2.0, 1.0]], [[2.0, 2.0, 7.0], [5.0, 5.0, 8.0], [3.0, 2.0, 4.0]]], [[[4.0, 2.0, 5.0], [3.0, 1.0, 
8.0], [3.0, 3.0, 6.0]], [[8.0, 5.0, 2.0], [2.0, 5.0, 6.0], [5.0, 3.0, 6.0]], [[7.0, 1.0, 8.0], [5.0, 1.0, 3.0], [7.0, 5.0, 
3.0]]], [[[5.0, 8.0, 4.0], [3.0, 3.0, 6.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 8.0], [6.0, 3.0, 5.0], [1.0, 4.0, 3.0]], [[2.0, 1.0, 
8.0], [4.0, 1.0, 1.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCPP(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 1.0], [7.0, 5.0, 8.0], [7.0, 2.0, 8.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 2.0], [1.0, 3.0, 
7.0]], [[1.0, 7.0, 1.0], [6.0, 2.0, 7.0], [1.0, 4.0, 1.0]]], [[[5.0, 7.0, 2.0], [1.0, 3.0, 7.0], [6.0, 8.0, 5.0]], [[7.0, 3.0, 
7.0], [6.0, 1.0, 8.0], [1.0, 1.0, 2.0]], [[4.0, 6.0, 1.0], [7.0, 2.0, 6.0], [8.0, 3.0, 6.0]]], [[[7.0, 1.0, 3.0], [6.0, 4.0, 
5.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 2.0], [4.0, 6.0, 3.0], [6.0, 1.0, 7.0]], [[1.0, 7.0, 1.0], [6.0, 1.0, 6.0], [5.0, 6.0, 
4.0]]], [[[5.0, 2.0, 8.0], [5.0, 4.0, 7.0], [7.0, 3.0, 6.0]], [[7.0, 2.0, 7.0], [5.0, 7.0, 4.0], [3.0, 6.0, 8.0]], [[4.0, 6.0, 
8.0], [3.0, 6.0, 5.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 4.0, 1.0], [8.0, 1.0, 1.0]]),self.functionspace)
      arg[1:3,:,1,1]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 1.0], [7.0, 5.0, 8.0], [7.0, 2.0, 8.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 2.0], [1.0, 3.0, 
7.0]], [[1.0, 7.0, 1.0], [6.0, 2.0, 7.0], [1.0, 4.0, 1.0]]], [[[5.0, 7.0, 2.0], [1.0, 3.0, 7.0], [6.0, 8.0, 5.0]], [[7.0, 3.0, 
7.0], [6.0, 4.0, 8.0], [1.0, 1.0, 2.0]], [[4.0, 6.0, 1.0], [7.0, 1.0, 6.0], [8.0, 3.0, 6.0]]], [[[7.0, 1.0, 3.0], [6.0, 8.0, 
5.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 2.0], [4.0, 1.0, 3.0], [6.0, 1.0, 7.0]], [[1.0, 7.0, 1.0], [6.0, 1.0, 6.0], [5.0, 6.0, 
4.0]]], [[[5.0, 2.0, 8.0], [5.0, 4.0, 7.0], [7.0, 3.0, 6.0]], [[7.0, 2.0, 7.0], [5.0, 7.0, 4.0], [3.0, 6.0, 8.0]], [[4.0, 6.0, 
8.0], [3.0, 6.0, 5.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCPP(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 2.0], [1.0, 3.0, 6.0], [2.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], [2.0, 1.0, 4.0], [5.0, 7.0, 
7.0]], [[6.0, 8.0, 4.0], [6.0, 8.0, 4.0], [6.0, 1.0, 1.0]]], [[[2.0, 6.0, 7.0], [3.0, 1.0, 4.0], [6.0, 6.0, 3.0]], [[6.0, 8.0, 
6.0], [3.0, 7.0, 5.0], [1.0, 8.0, 8.0]], [[3.0, 7.0, 8.0], [3.0, 4.0, 5.0], [6.0, 8.0, 2.0]]], [[[4.0, 3.0, 6.0], [5.0, 6.0, 
6.0], [5.0, 3.0, 4.0]], [[5.0, 6.0, 3.0], [3.0, 7.0, 6.0], [2.0, 2.0, 7.0]], [[3.0, 7.0, 7.0], [4.0, 3.0, 1.0], [8.0, 7.0, 
7.0]]], [[[4.0, 6.0, 1.0], [3.0, 6.0, 3.0], [1.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], [7.0, 8.0, 5.0], [8.0, 3.0, 3.0]], [[5.0, 3.0, 
5.0], [4.0, 6.0, 3.0], [5.0, 2.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 7.0, 8.0], [8.0, 6.0, 7.0]])+(1.-msk_val)*numpy.array([[6.0, 2.0, 5.0], [8.0, 8.0, 7.0]])
      arg[1:3,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 2.0], [1.0, 3.0, 6.0], [2.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], [2.0, 1.0, 4.0], [5.0, 
7.0, 7.0]], [[6.0, 8.0, 4.0], [6.0, 8.0, 4.0], [6.0, 1.0, 1.0]]], [[[2.0, 6.0, 7.0], [3.0, 2.0, 4.0], [6.0, 6.0, 3.0]], [[6.0, 
8.0, 6.0], [3.0, 7.0, 5.0], [1.0, 8.0, 8.0]], [[3.0, 7.0, 8.0], [3.0, 8.0, 5.0], [6.0, 8.0, 2.0]]], [[[4.0, 3.0, 6.0], [5.0, 
8.0, 6.0], [5.0, 3.0, 4.0]], [[5.0, 6.0, 3.0], [3.0, 6.0, 6.0], [2.0, 2.0, 7.0]], [[3.0, 7.0, 7.0], [4.0, 7.0, 1.0], [8.0, 7.0, 
7.0]]], [[[4.0, 6.0, 1.0], [3.0, 6.0, 3.0], [1.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], [7.0, 8.0, 5.0], [8.0, 3.0, 3.0]], [[5.0, 3.0, 
5.0], [4.0, 6.0, 3.0], [5.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 2.0], [1.0, 3.0, 6.0], [2.0, 7.0, 8.0]], 
[[7.0, 6.0, 1.0], [2.0, 1.0, 4.0], [5.0, 7.0, 7.0]], [[6.0, 8.0, 4.0], [6.0, 8.0, 4.0], [6.0, 1.0, 1.0]]], [[[2.0, 6.0, 7.0], 
[3.0, 6.0, 4.0], [6.0, 6.0, 3.0]], [[6.0, 8.0, 6.0], [3.0, 2.0, 5.0], [1.0, 8.0, 8.0]], [[3.0, 7.0, 8.0], [3.0, 5.0, 5.0], 
[6.0, 8.0, 2.0]]], [[[4.0, 3.0, 6.0], [5.0, 8.0, 6.0], [5.0, 3.0, 4.0]], [[5.0, 6.0, 3.0], [3.0, 8.0, 6.0], [2.0, 2.0, 7.0]], 
[[3.0, 7.0, 7.0], [4.0, 7.0, 1.0], [8.0, 7.0, 7.0]]], [[[4.0, 6.0, 1.0], [3.0, 6.0, 3.0], [1.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], 
[7.0, 8.0, 5.0], [8.0, 3.0, 3.0]], [[5.0, 3.0, 5.0], [4.0, 6.0, 3.0], [5.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUCC(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 1.0], [6.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[4.0, 4.0, 6.0], [8.0, 5.0, 7.0], [8.0, 2.0, 
7.0]], [[7.0, 3.0, 5.0], [3.0, 8.0, 6.0], [1.0, 2.0, 4.0]]], [[[6.0, 3.0, 5.0], [7.0, 1.0, 1.0], [3.0, 1.0, 3.0]], [[6.0, 3.0, 
6.0], [4.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[8.0, 8.0, 2.0], [5.0, 7.0, 4.0], [2.0, 7.0, 7.0]]], [[[2.0, 1.0, 7.0], [3.0, 6.0, 
5.0], [2.0, 8.0, 8.0]], [[5.0, 3.0, 8.0], [6.0, 4.0, 1.0], [4.0, 3.0, 7.0]], [[2.0, 2.0, 3.0], [5.0, 7.0, 6.0], [4.0, 4.0, 
2.0]]], [[[1.0, 8.0, 3.0], [3.0, 1.0, 5.0], [5.0, 6.0, 3.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 8.0], [6.0, 7.0, 6.0]], [[2.0, 7.0, 
6.0], [2.0, 1.0, 4.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 8.0, 3.0], [4.0, 5.0, 3.0], [3.0, 2.0, 5.0]], [[2.0, 6.0, 8.0], [2.0, 6.0, 8.0], [7.0, 8.0, 
7.0]]], [[[3.0, 8.0, 4.0], [8.0, 3.0, 2.0], [7.0, 2.0, 4.0]], [[4.0, 2.0, 2.0], [6.0, 7.0, 2.0], [2.0, 4.0, 2.0]]]])
      arg[1:3,:2,:,:]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 1.0], [6.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[4.0, 4.0, 6.0], [8.0, 5.0, 7.0], [8.0, 2.0, 
7.0]], [[7.0, 3.0, 5.0], [3.0, 8.0, 6.0], [1.0, 2.0, 4.0]]], [[[2.0, 8.0, 3.0], [4.0, 5.0, 3.0], [3.0, 2.0, 5.0]], [[2.0, 6.0, 
8.0], [2.0, 6.0, 8.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 2.0], [5.0, 7.0, 4.0], [2.0, 7.0, 7.0]]], [[[3.0, 8.0, 4.0], [8.0, 3.0, 
2.0], [7.0, 2.0, 4.0]], [[4.0, 2.0, 2.0], [6.0, 7.0, 2.0], [2.0, 4.0, 2.0]], [[2.0, 2.0, 3.0], [5.0, 7.0, 6.0], [4.0, 4.0, 
2.0]]], [[[1.0, 8.0, 3.0], [3.0, 1.0, 5.0], [5.0, 6.0, 3.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 8.0], [6.0, 7.0, 6.0]], [[2.0, 7.0, 
6.0], [2.0, 1.0, 4.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUCC(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 7.0], [5.0, 8.0, 5.0], [5.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [4.0, 1.0, 5.0], [5.0, 2.0, 
1.0]], [[7.0, 8.0, 3.0], [3.0, 8.0, 3.0], [8.0, 8.0, 8.0]]], [[[3.0, 7.0, 5.0], [2.0, 2.0, 2.0], [7.0, 4.0, 8.0]], [[8.0, 1.0, 
5.0], [2.0, 3.0, 4.0], [2.0, 3.0, 4.0]], [[3.0, 3.0, 2.0], [7.0, 8.0, 3.0], [2.0, 3.0, 6.0]]], [[[3.0, 1.0, 5.0], [1.0, 8.0, 
2.0], [8.0, 8.0, 8.0]], [[3.0, 5.0, 1.0], [2.0, 7.0, 8.0], [6.0, 5.0, 7.0]], [[2.0, 6.0, 3.0], [3.0, 6.0, 5.0], [7.0, 5.0, 
2.0]]], [[[3.0, 1.0, 2.0], [6.0, 8.0, 3.0], [8.0, 1.0, 7.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 2.0, 
8.0], [7.0, 8.0, 5.0], [2.0, 1.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 2.0, 3.0], [6.0, 5.0, 2.0], [2.0, 5.0, 7.0]], [[4.0, 6.0, 2.0], [4.0, 7.0, 3.0], [4.0, 5.0, 
4.0]]], [[[7.0, 3.0, 5.0], [4.0, 3.0, 7.0], [2.0, 5.0, 8.0]], [[2.0, 5.0, 7.0], [2.0, 2.0, 2.0], [4.0, 2.0, 
7.0]]]]),self.functionspace)
      arg[1:3,:2,:,:]=val
      ref=Data(numpy.array([[[[6.0, 3.0, 7.0], [5.0, 8.0, 5.0], [5.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [4.0, 1.0, 5.0], [5.0, 2.0, 
1.0]], [[7.0, 8.0, 3.0], [3.0, 8.0, 3.0], [8.0, 8.0, 8.0]]], [[[3.0, 2.0, 3.0], [6.0, 5.0, 2.0], [2.0, 5.0, 7.0]], [[4.0, 6.0, 
2.0], [4.0, 7.0, 3.0], [4.0, 5.0, 4.0]], [[3.0, 3.0, 2.0], [7.0, 8.0, 3.0], [2.0, 3.0, 6.0]]], [[[7.0, 3.0, 5.0], [4.0, 3.0, 
7.0], [2.0, 5.0, 8.0]], [[2.0, 5.0, 7.0], [2.0, 2.0, 2.0], [4.0, 2.0, 7.0]], [[2.0, 6.0, 3.0], [3.0, 6.0, 5.0], [7.0, 5.0, 
2.0]]], [[[3.0, 1.0, 2.0], [6.0, 8.0, 3.0], [8.0, 1.0, 7.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 2.0, 
8.0], [7.0, 8.0, 5.0], [2.0, 1.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUCC(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 1.0], [3.0, 6.0, 5.0], [5.0, 4.0, 2.0]], [[2.0, 5.0, 7.0], [5.0, 6.0, 3.0], [3.0, 5.0, 
2.0]], [[6.0, 6.0, 3.0], [8.0, 6.0, 7.0], [7.0, 3.0, 3.0]]], [[[6.0, 5.0, 8.0], [7.0, 8.0, 8.0], [5.0, 1.0, 3.0]], [[4.0, 6.0, 
5.0], [6.0, 2.0, 3.0], [6.0, 8.0, 8.0]], [[5.0, 8.0, 6.0], [3.0, 6.0, 3.0], [1.0, 8.0, 2.0]]], [[[6.0, 6.0, 2.0], [1.0, 2.0, 
5.0], [4.0, 8.0, 4.0]], [[6.0, 2.0, 2.0], [2.0, 1.0, 2.0], [4.0, 7.0, 1.0]], [[5.0, 7.0, 6.0], [7.0, 1.0, 7.0], [7.0, 7.0, 
5.0]]], [[[7.0, 1.0, 5.0], [6.0, 6.0, 5.0], [5.0, 5.0, 7.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[6.0, 8.0, 
3.0], [2.0, 1.0, 2.0], [7.0, 4.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 4.0, 1.0], [3.0, 7.0, 3.0], [8.0, 3.0, 7.0]], [[2.0, 8.0, 4.0], [8.0, 7.0, 6.0], [1.0, 
4.0, 2.0]]], [[[6.0, 7.0, 8.0], [6.0, 3.0, 5.0], [7.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 6.0, 8.0], [8.0, 4.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 6.0, 1.0], [3.0, 5.0, 8.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [6.0, 3.0, 7.0], 
[4.0, 1.0, 4.0]]], [[[2.0, 6.0, 7.0], [3.0, 3.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 5.0, 1.0], [3.0, 7.0, 1.0], [5.0, 7.0, 6.0]]]])
      arg[1:3,:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 1.0], [3.0, 6.0, 5.0], [5.0, 4.0, 2.0]], [[2.0, 5.0, 7.0], [5.0, 6.0, 3.0], [3.0, 
5.0, 2.0]], [[6.0, 6.0, 3.0], [8.0, 6.0, 7.0], [7.0, 3.0, 3.0]]], [[[7.0, 4.0, 1.0], [3.0, 7.0, 3.0], [8.0, 3.0, 7.0]], [[2.0, 
8.0, 4.0], [8.0, 7.0, 6.0], [1.0, 4.0, 2.0]], [[5.0, 8.0, 6.0], [3.0, 6.0, 3.0], [1.0, 8.0, 2.0]]], [[[6.0, 7.0, 8.0], [6.0, 
3.0, 5.0], [7.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 6.0, 8.0], [8.0, 4.0, 4.0]], [[5.0, 7.0, 6.0], [7.0, 1.0, 7.0], [7.0, 7.0, 
5.0]]], [[[7.0, 1.0, 5.0], [6.0, 6.0, 5.0], [5.0, 5.0, 7.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[6.0, 8.0, 
3.0], [2.0, 1.0, 2.0], [7.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 1.0], [3.0, 6.0, 5.0], [5.0, 4.0, 2.0]], 
[[2.0, 5.0, 7.0], [5.0, 6.0, 3.0], [3.0, 5.0, 2.0]], [[6.0, 6.0, 3.0], [8.0, 6.0, 7.0], [7.0, 3.0, 3.0]]], [[[2.0, 6.0, 1.0], 
[3.0, 5.0, 8.0], [1.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [6.0, 3.0, 7.0], [4.0, 1.0, 4.0]], [[5.0, 8.0, 6.0], [3.0, 6.0, 3.0], 
[1.0, 8.0, 2.0]]], [[[2.0, 6.0, 7.0], [3.0, 3.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 5.0, 1.0], [3.0, 7.0, 1.0], [5.0, 7.0, 6.0]], 
[[5.0, 7.0, 6.0], [7.0, 1.0, 7.0], [7.0, 7.0, 5.0]]], [[[7.0, 1.0, 5.0], [6.0, 6.0, 5.0], [5.0, 5.0, 7.0]], [[2.0, 8.0, 1.0], 
[5.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[6.0, 8.0, 3.0], [2.0, 1.0, 2.0], [7.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRCL(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 4.0], [3.0, 1.0, 3.0], [1.0, 1.0, 3.0]], [[8.0, 7.0, 2.0], [1.0, 3.0, 8.0], [5.0, 5.0, 
5.0]], [[7.0, 7.0, 5.0], [5.0, 3.0, 4.0], [1.0, 5.0, 3.0]]], [[[4.0, 2.0, 1.0], [1.0, 8.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 5.0, 
5.0], [3.0, 3.0, 3.0], [3.0, 6.0, 8.0]], [[3.0, 2.0, 6.0], [2.0, 7.0, 6.0], [4.0, 4.0, 2.0]]], [[[4.0, 3.0, 8.0], [8.0, 4.0, 
2.0], [1.0, 5.0, 3.0]], [[6.0, 7.0, 3.0], [1.0, 1.0, 4.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], [3.0, 2.0, 5.0], [7.0, 6.0, 
7.0]]], [[[7.0, 1.0, 8.0], [6.0, 1.0, 4.0], [6.0, 2.0, 8.0]], [[1.0, 2.0, 8.0], [2.0, 8.0, 1.0], [6.0, 6.0, 8.0]], [[2.0, 1.0, 
8.0], [7.0, 7.0, 1.0], [6.0, 2.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0], [7.0], [4.0]]], [[[4.0], [5.0], [7.0]]]])
      arg[1:3,1:2,:,2:]=val
      ref=Data(numpy.array([[[[4.0, 7.0, 4.0], [3.0, 1.0, 3.0], [1.0, 1.0, 3.0]], [[8.0, 7.0, 2.0], [1.0, 3.0, 8.0], [5.0, 5.0, 
5.0]], [[7.0, 7.0, 5.0], [5.0, 3.0, 4.0], [1.0, 5.0, 3.0]]], [[[4.0, 2.0, 1.0], [1.0, 8.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 5.0, 
6.0], [3.0, 3.0, 7.0], [3.0, 6.0, 4.0]], [[3.0, 2.0, 6.0], [2.0, 7.0, 6.0], [4.0, 4.0, 2.0]]], [[[4.0, 3.0, 8.0], [8.0, 4.0, 
2.0], [1.0, 5.0, 3.0]], [[6.0, 7.0, 4.0], [1.0, 1.0, 5.0], [6.0, 2.0, 7.0]], [[2.0, 8.0, 5.0], [3.0, 2.0, 5.0], [7.0, 6.0, 
7.0]]], [[[7.0, 1.0, 8.0], [6.0, 1.0, 4.0], [6.0, 2.0, 8.0]], [[1.0, 2.0, 8.0], [2.0, 8.0, 1.0], [6.0, 6.0, 8.0]], [[2.0, 1.0, 
8.0], [7.0, 7.0, 1.0], [6.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRCL(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 6.0], [7.0, 6.0, 2.0], [2.0, 7.0, 4.0]], [[2.0, 8.0, 2.0], [8.0, 4.0, 7.0], [5.0, 1.0, 
2.0]], [[5.0, 5.0, 4.0], [4.0, 1.0, 6.0], [1.0, 4.0, 3.0]]], [[[4.0, 7.0, 7.0], [7.0, 8.0, 8.0], [5.0, 3.0, 5.0]], [[4.0, 6.0, 
4.0], [4.0, 5.0, 8.0], [1.0, 1.0, 5.0]], [[6.0, 3.0, 2.0], [4.0, 7.0, 8.0], [4.0, 3.0, 2.0]]], [[[7.0, 8.0, 1.0], [1.0, 1.0, 
5.0], [8.0, 6.0, 7.0]], [[2.0, 4.0, 2.0], [1.0, 8.0, 3.0], [3.0, 4.0, 4.0]], [[4.0, 2.0, 1.0], [8.0, 1.0, 6.0], [4.0, 3.0, 
2.0]]], [[[6.0, 3.0, 8.0], [6.0, 2.0, 4.0], [2.0, 4.0, 6.0]], [[5.0, 7.0, 1.0], [3.0, 3.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 3.0, 
8.0], [2.0, 5.0, 4.0], [6.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0], [4.0], [8.0]]], [[[5.0], [2.0], [4.0]]]]),self.functionspace)
      arg[1:3,1:2,:,2:]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 6.0], [7.0, 6.0, 2.0], [2.0, 7.0, 4.0]], [[2.0, 8.0, 2.0], [8.0, 4.0, 7.0], [5.0, 1.0, 
2.0]], [[5.0, 5.0, 4.0], [4.0, 1.0, 6.0], [1.0, 4.0, 3.0]]], [[[4.0, 7.0, 7.0], [7.0, 8.0, 8.0], [5.0, 3.0, 5.0]], [[4.0, 6.0, 
5.0], [4.0, 5.0, 4.0], [1.0, 1.0, 8.0]], [[6.0, 3.0, 2.0], [4.0, 7.0, 8.0], [4.0, 3.0, 2.0]]], [[[7.0, 8.0, 1.0], [1.0, 1.0, 
5.0], [8.0, 6.0, 7.0]], [[2.0, 4.0, 5.0], [1.0, 8.0, 2.0], [3.0, 4.0, 4.0]], [[4.0, 2.0, 1.0], [8.0, 1.0, 6.0], [4.0, 3.0, 
2.0]]], [[[6.0, 3.0, 8.0], [6.0, 2.0, 4.0], [2.0, 4.0, 6.0]], [[5.0, 7.0, 1.0], [3.0, 3.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 3.0, 
8.0], [2.0, 5.0, 4.0], [6.0, 8.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRCL(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 8.0], [3.0, 3.0, 7.0], [8.0, 1.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 2.0, 6.0], [6.0, 2.0, 
4.0]], [[2.0, 8.0, 1.0], [3.0, 8.0, 7.0], [1.0, 6.0, 3.0]]], [[[1.0, 1.0, 1.0], [7.0, 8.0, 6.0], [5.0, 2.0, 4.0]], [[8.0, 3.0, 
1.0], [6.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[3.0, 1.0, 5.0], [1.0, 5.0, 6.0], [7.0, 2.0, 1.0]]], [[[1.0, 2.0, 2.0], [3.0, 3.0, 
2.0], [8.0, 1.0, 5.0]], [[6.0, 3.0, 3.0], [8.0, 4.0, 6.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 6.0], [2.0, 6.0, 
2.0]]], [[[6.0, 1.0, 4.0], [2.0, 8.0, 7.0], [2.0, 5.0, 7.0]], [[6.0, 6.0, 1.0], [2.0, 3.0, 4.0], [3.0, 1.0, 7.0]], [[1.0, 3.0, 
5.0], [2.0, 2.0, 4.0], [5.0, 2.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0], [7.0], [4.0]]], [[[2.0], [6.0], [4.0]]]])+(1.-msk_val)*numpy.array([[[[7.0], [6.0], 
[5.0]]], [[[5.0], [2.0], [2.0]]]])
      arg[1:3,1:2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 8.0], [3.0, 3.0, 7.0], [8.0, 1.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 2.0, 6.0], [6.0, 
2.0, 4.0]], [[2.0, 8.0, 1.0], [3.0, 8.0, 7.0], [1.0, 6.0, 3.0]]], [[[1.0, 1.0, 1.0], [7.0, 8.0, 6.0], [5.0, 2.0, 4.0]], [[8.0, 
3.0, 3.0], [6.0, 7.0, 7.0], [2.0, 3.0, 4.0]], [[3.0, 1.0, 5.0], [1.0, 5.0, 6.0], [7.0, 2.0, 1.0]]], [[[1.0, 2.0, 2.0], [3.0, 
3.0, 2.0], [8.0, 1.0, 5.0]], [[6.0, 3.0, 2.0], [8.0, 4.0, 6.0], [4.0, 4.0, 4.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 6.0], [2.0, 6.0, 
2.0]]], [[[6.0, 1.0, 4.0], [2.0, 8.0, 7.0], [2.0, 5.0, 7.0]], [[6.0, 6.0, 1.0], [2.0, 3.0, 4.0], [3.0, 1.0, 7.0]], [[1.0, 3.0, 
5.0], [2.0, 2.0, 4.0], [5.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 8.0], [3.0, 3.0, 7.0], [8.0, 1.0, 1.0]], 
[[1.0, 5.0, 6.0], [1.0, 2.0, 6.0], [6.0, 2.0, 4.0]], [[2.0, 8.0, 1.0], [3.0, 8.0, 7.0], [1.0, 6.0, 3.0]]], [[[1.0, 1.0, 1.0], 
[7.0, 8.0, 6.0], [5.0, 2.0, 4.0]], [[8.0, 3.0, 7.0], [6.0, 7.0, 6.0], [2.0, 3.0, 5.0]], [[3.0, 1.0, 5.0], [1.0, 5.0, 6.0], 
[7.0, 2.0, 1.0]]], [[[1.0, 2.0, 2.0], [3.0, 3.0, 2.0], [8.0, 1.0, 5.0]], [[6.0, 3.0, 5.0], [8.0, 4.0, 2.0], [4.0, 4.0, 2.0]], 
[[8.0, 7.0, 8.0], [3.0, 7.0, 6.0], [2.0, 6.0, 2.0]]], [[[6.0, 1.0, 4.0], [2.0, 8.0, 7.0], [2.0, 5.0, 7.0]], [[6.0, 6.0, 1.0], 
[2.0, 3.0, 4.0], [3.0, 1.0, 7.0]], [[1.0, 3.0, 5.0], [2.0, 2.0, 4.0], [5.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URUL(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 1.0], [1.0, 5.0, 1.0], [3.0, 4.0, 3.0]], [[1.0, 6.0, 1.0], [1.0, 8.0, 6.0], [6.0, 3.0, 
5.0]], [[5.0, 8.0, 7.0], [1.0, 4.0, 8.0], [3.0, 7.0, 5.0]]], [[[1.0, 8.0, 6.0], [3.0, 1.0, 7.0], [1.0, 5.0, 1.0]], [[1.0, 1.0, 
7.0], [2.0, 5.0, 2.0], [5.0, 8.0, 2.0]], [[1.0, 2.0, 5.0], [5.0, 4.0, 4.0], [2.0, 2.0, 7.0]]], [[[3.0, 5.0, 4.0], [8.0, 3.0, 
4.0], [8.0, 6.0, 1.0]], [[5.0, 3.0, 5.0], [1.0, 4.0, 2.0], [8.0, 2.0, 4.0]], [[2.0, 8.0, 5.0], [6.0, 7.0, 3.0], [2.0, 8.0, 
5.0]]], [[[6.0, 7.0, 8.0], [8.0, 2.0, 8.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 1.0], [8.0, 5.0, 4.0], [2.0, 7.0, 5.0]], [[3.0, 3.0, 
6.0], [5.0, 6.0, 5.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0], [1.0]]], [[[3.0], [5.0]]], [[[5.0], [6.0]]]])
      arg[:3,1:2,:2,2:]=val
      ref=Data(numpy.array([[[[1.0, 2.0, 1.0], [1.0, 5.0, 1.0], [3.0, 4.0, 3.0]], [[1.0, 6.0, 6.0], [1.0, 8.0, 1.0], [6.0, 3.0, 
5.0]], [[5.0, 8.0, 7.0], [1.0, 4.0, 8.0], [3.0, 7.0, 5.0]]], [[[1.0, 8.0, 6.0], [3.0, 1.0, 7.0], [1.0, 5.0, 1.0]], [[1.0, 1.0, 
3.0], [2.0, 5.0, 5.0], [5.0, 8.0, 2.0]], [[1.0, 2.0, 5.0], [5.0, 4.0, 4.0], [2.0, 2.0, 7.0]]], [[[3.0, 5.0, 4.0], [8.0, 3.0, 
4.0], [8.0, 6.0, 1.0]], [[5.0, 3.0, 5.0], [1.0, 4.0, 6.0], [8.0, 2.0, 4.0]], [[2.0, 8.0, 5.0], [6.0, 7.0, 3.0], [2.0, 8.0, 
5.0]]], [[[6.0, 7.0, 8.0], [8.0, 2.0, 8.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 1.0], [8.0, 5.0, 4.0], [2.0, 7.0, 5.0]], [[3.0, 3.0, 
6.0], [5.0, 6.0, 5.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URUL(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [3.0, 3.0, 2.0], [8.0, 1.0, 8.0]], [[6.0, 5.0, 7.0], [4.0, 8.0, 2.0], [7.0, 7.0, 
3.0]], [[1.0, 7.0, 6.0], [2.0, 2.0, 4.0], [5.0, 7.0, 6.0]]], [[[3.0, 7.0, 7.0], [4.0, 1.0, 8.0], [5.0, 6.0, 1.0]], [[6.0, 5.0, 
3.0], [7.0, 7.0, 7.0], [6.0, 5.0, 5.0]], [[2.0, 5.0, 7.0], [1.0, 2.0, 1.0], [3.0, 3.0, 7.0]]], [[[1.0, 4.0, 3.0], [6.0, 5.0, 
7.0], [8.0, 1.0, 5.0]], [[6.0, 5.0, 1.0], [2.0, 7.0, 3.0], [5.0, 2.0, 4.0]], [[5.0, 3.0, 8.0], [5.0, 4.0, 7.0], [1.0, 6.0, 
7.0]]], [[[3.0, 4.0, 4.0], [8.0, 5.0, 8.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 4.0], [8.0, 5.0, 8.0], [6.0, 7.0, 1.0]], [[7.0, 2.0, 
2.0], [6.0, 8.0, 3.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0], [3.0]]], [[[3.0], [5.0]]], [[[2.0], [6.0]]]]),self.functionspace)
      arg[:3,1:2,:2,2:]=val
      ref=Data(numpy.array([[[[5.0, 2.0, 8.0], [3.0, 3.0, 2.0], [8.0, 1.0, 8.0]], [[6.0, 5.0, 7.0], [4.0, 8.0, 3.0], [7.0, 7.0, 
3.0]], [[1.0, 7.0, 6.0], [2.0, 2.0, 4.0], [5.0, 7.0, 6.0]]], [[[3.0, 7.0, 7.0], [4.0, 1.0, 8.0], [5.0, 6.0, 1.0]], [[6.0, 5.0, 
3.0], [7.0, 7.0, 5.0], [6.0, 5.0, 5.0]], [[2.0, 5.0, 7.0], [1.0, 2.0, 1.0], [3.0, 3.0, 7.0]]], [[[1.0, 4.0, 3.0], [6.0, 5.0, 
7.0], [8.0, 1.0, 5.0]], [[6.0, 5.0, 2.0], [2.0, 7.0, 6.0], [5.0, 2.0, 4.0]], [[5.0, 3.0, 8.0], [5.0, 4.0, 7.0], [1.0, 6.0, 
7.0]]], [[[3.0, 4.0, 4.0], [8.0, 5.0, 8.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 4.0], [8.0, 5.0, 8.0], [6.0, 7.0, 1.0]], [[7.0, 2.0, 
2.0], [6.0, 8.0, 3.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URUL(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 4.0], [1.0, 4.0, 5.0], [1.0, 3.0, 1.0]], [[1.0, 6.0, 8.0], [8.0, 4.0, 1.0], [7.0, 3.0, 
4.0]], [[7.0, 6.0, 3.0], [2.0, 2.0, 1.0], [1.0, 7.0, 4.0]]], [[[1.0, 5.0, 5.0], [6.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[5.0, 6.0, 
2.0], [3.0, 3.0, 3.0], [2.0, 3.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 5.0, 1.0], [1.0, 3.0, 6.0]]], [[[6.0, 8.0, 2.0], [5.0, 8.0, 
5.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 5.0], [1.0, 5.0, 8.0], [6.0, 7.0, 4.0]], [[8.0, 4.0, 1.0], [6.0, 1.0, 7.0], [4.0, 1.0, 
1.0]]], [[[7.0, 4.0, 1.0], [3.0, 8.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 5.0, 1.0], [4.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 7.0, 
7.0], [5.0, 7.0, 6.0], [3.0, 5.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0], [6.0]]], [[[1.0], [8.0]]], [[[8.0], [1.0]]]])+(1.-msk_val)*numpy.array([[[[6.0], 
[3.0]]], [[[4.0], [6.0]]], [[[4.0], [7.0]]]])
      arg[:3,1:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 4.0], [1.0, 4.0, 5.0], [1.0, 3.0, 1.0]], [[1.0, 6.0, 7.0], [8.0, 4.0, 6.0], [7.0, 
3.0, 4.0]], [[7.0, 6.0, 3.0], [2.0, 2.0, 1.0], [1.0, 7.0, 4.0]]], [[[1.0, 5.0, 5.0], [6.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[5.0, 
6.0, 1.0], [3.0, 3.0, 8.0], [2.0, 3.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 5.0, 1.0], [1.0, 3.0, 6.0]]], [[[6.0, 8.0, 2.0], [5.0, 
8.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 5.0, 1.0], [6.0, 7.0, 4.0]], [[8.0, 4.0, 1.0], [6.0, 1.0, 7.0], [4.0, 1.0, 
1.0]]], [[[7.0, 4.0, 1.0], [3.0, 8.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 5.0, 1.0], [4.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 7.0, 
7.0], [5.0, 7.0, 6.0], [3.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 4.0], [1.0, 4.0, 5.0], [1.0, 3.0, 1.0]], 
[[1.0, 6.0, 6.0], [8.0, 4.0, 3.0], [7.0, 3.0, 4.0]], [[7.0, 6.0, 3.0], [2.0, 2.0, 1.0], [1.0, 7.0, 4.0]]], [[[1.0, 5.0, 5.0], 
[6.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], [3.0, 3.0, 6.0], [2.0, 3.0, 2.0]], [[3.0, 7.0, 6.0], [7.0, 5.0, 1.0], 
[1.0, 3.0, 6.0]]], [[[6.0, 8.0, 2.0], [5.0, 8.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 2.0, 4.0], [1.0, 5.0, 7.0], [6.0, 7.0, 4.0]], 
[[8.0, 4.0, 1.0], [6.0, 1.0, 7.0], [4.0, 1.0, 1.0]]], [[[7.0, 4.0, 1.0], [3.0, 8.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 5.0, 1.0], 
[4.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 7.0, 7.0], [5.0, 7.0, 6.0], [3.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPRP(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 8.0], [2.0, 8.0, 4.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 5.0], [8.0, 7.0, 8.0], [3.0, 7.0, 
3.0]], [[2.0, 2.0, 2.0], [4.0, 4.0, 4.0], [3.0, 7.0, 8.0]]], [[[6.0, 8.0, 3.0], [8.0, 1.0, 6.0], [1.0, 2.0, 4.0]], [[7.0, 8.0, 
7.0], [7.0, 7.0, 5.0], [7.0, 7.0, 2.0]], [[3.0, 3.0, 2.0], [5.0, 6.0, 1.0], [4.0, 5.0, 4.0]]], [[[3.0, 1.0, 1.0], [2.0, 6.0, 
8.0], [2.0, 4.0, 5.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 8.0], [2.0, 6.0, 7.0]], [[1.0, 8.0, 7.0], [6.0, 3.0, 3.0], [2.0, 2.0, 
3.0]]], [[[5.0, 4.0, 2.0], [3.0, 8.0, 3.0], [1.0, 1.0, 3.0]], [[1.0, 8.0, 3.0], [4.0, 7.0, 2.0], [6.0, 4.0, 4.0]], [[3.0, 4.0, 
6.0], [8.0, 4.0, 5.0], [6.0, 6.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[6.0], [1.0]])
      arg[1:3,1,1:2,1]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 8.0], [2.0, 8.0, 4.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 5.0], [8.0, 7.0, 8.0], [3.0, 7.0, 
3.0]], [[2.0, 2.0, 2.0], [4.0, 4.0, 4.0], [3.0, 7.0, 8.0]]], [[[6.0, 8.0, 3.0], [8.0, 1.0, 6.0], [1.0, 2.0, 4.0]], [[7.0, 8.0, 
7.0], [7.0, 6.0, 5.0], [7.0, 7.0, 2.0]], [[3.0, 3.0, 2.0], [5.0, 6.0, 1.0], [4.0, 5.0, 4.0]]], [[[3.0, 1.0, 1.0], [2.0, 6.0, 
8.0], [2.0, 4.0, 5.0]], [[7.0, 5.0, 2.0], [5.0, 1.0, 8.0], [2.0, 6.0, 7.0]], [[1.0, 8.0, 7.0], [6.0, 3.0, 3.0], [2.0, 2.0, 
3.0]]], [[[5.0, 4.0, 2.0], [3.0, 8.0, 3.0], [1.0, 1.0, 3.0]], [[1.0, 8.0, 3.0], [4.0, 7.0, 2.0], [6.0, 4.0, 4.0]], [[3.0, 4.0, 
6.0], [8.0, 4.0, 5.0], [6.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPRP(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 3.0], [3.0, 6.0, 4.0], [4.0, 5.0, 5.0]], [[7.0, 8.0, 5.0], [3.0, 5.0, 2.0], [2.0, 4.0, 
6.0]], [[2.0, 3.0, 7.0], [6.0, 4.0, 7.0], [5.0, 3.0, 1.0]]], [[[6.0, 7.0, 6.0], [4.0, 4.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 7.0, 
3.0], [5.0, 4.0, 1.0], [4.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [6.0, 2.0, 7.0], [7.0, 6.0, 6.0]]], [[[5.0, 8.0, 2.0], [2.0, 7.0, 
8.0], [1.0, 8.0, 5.0]], [[7.0, 6.0, 8.0], [3.0, 6.0, 6.0], [8.0, 3.0, 5.0]], [[3.0, 1.0, 8.0], [1.0, 2.0, 2.0], [2.0, 4.0, 
3.0]]], [[[3.0, 3.0, 2.0], [2.0, 1.0, 7.0], [7.0, 3.0, 7.0]], [[4.0, 1.0, 5.0], [5.0, 7.0, 3.0], [4.0, 5.0, 4.0]], [[6.0, 4.0, 
6.0], [5.0, 2.0, 4.0], [4.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0], [1.0]]),self.functionspace)
      arg[1:3,1,1:2,1]=val
      ref=Data(numpy.array([[[[6.0, 4.0, 3.0], [3.0, 6.0, 4.0], [4.0, 5.0, 5.0]], [[7.0, 8.0, 5.0], [3.0, 5.0, 2.0], [2.0, 4.0, 
6.0]], [[2.0, 3.0, 7.0], [6.0, 4.0, 7.0], [5.0, 3.0, 1.0]]], [[[6.0, 7.0, 6.0], [4.0, 4.0, 4.0], [2.0, 6.0, 5.0]], [[1.0, 7.0, 
3.0], [5.0, 2.0, 1.0], [4.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [6.0, 2.0, 7.0], [7.0, 6.0, 6.0]]], [[[5.0, 8.0, 2.0], [2.0, 7.0, 
8.0], [1.0, 8.0, 5.0]], [[7.0, 6.0, 8.0], [3.0, 1.0, 6.0], [8.0, 3.0, 5.0]], [[3.0, 1.0, 8.0], [1.0, 2.0, 2.0], [2.0, 4.0, 
3.0]]], [[[3.0, 3.0, 2.0], [2.0, 1.0, 7.0], [7.0, 3.0, 7.0]], [[4.0, 1.0, 5.0], [5.0, 7.0, 3.0], [4.0, 5.0, 4.0]], [[6.0, 4.0, 
6.0], [5.0, 2.0, 4.0], [4.0, 6.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPRP(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 2.0], [6.0, 2.0, 5.0], [7.0, 2.0, 7.0]], [[5.0, 1.0, 6.0], [5.0, 2.0, 7.0], [4.0, 3.0, 
5.0]], [[1.0, 7.0, 4.0], [5.0, 8.0, 3.0], [5.0, 6.0, 3.0]]], [[[5.0, 1.0, 3.0], [6.0, 4.0, 3.0], [1.0, 3.0, 3.0]], [[7.0, 6.0, 
1.0], [2.0, 2.0, 1.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 6.0], [6.0, 2.0, 8.0], [2.0, 8.0, 1.0]]], [[[2.0, 7.0, 8.0], [5.0, 4.0, 
3.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 2.0], [7.0, 7.0, 3.0], [2.0, 8.0, 3.0]], [[4.0, 1.0, 5.0], [5.0, 1.0, 8.0], [8.0, 7.0, 
7.0]]], [[[3.0, 7.0, 6.0], [6.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[5.0, 3.0, 6.0], [1.0, 4.0, 8.0], [6.0, 3.0, 4.0]], [[8.0, 8.0, 
5.0], [1.0, 8.0, 8.0], [1.0, 6.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0], [6.0]])+(1.-msk_val)*numpy.array([[2.0], [1.0]])
      arg[1:3,1,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 2.0], [6.0, 2.0, 5.0], [7.0, 2.0, 7.0]], [[5.0, 1.0, 6.0], [5.0, 2.0, 7.0], [4.0, 
3.0, 5.0]], [[1.0, 7.0, 4.0], [5.0, 8.0, 3.0], [5.0, 6.0, 3.0]]], [[[5.0, 1.0, 3.0], [6.0, 4.0, 3.0], [1.0, 3.0, 3.0]], [[7.0, 
6.0, 1.0], [2.0, 2.0, 1.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 6.0], [6.0, 2.0, 8.0], [2.0, 8.0, 1.0]]], [[[2.0, 7.0, 8.0], [5.0, 
4.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 2.0], [7.0, 6.0, 3.0], [2.0, 8.0, 3.0]], [[4.0, 1.0, 5.0], [5.0, 1.0, 8.0], [8.0, 7.0, 
7.0]]], [[[3.0, 7.0, 6.0], [6.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[5.0, 3.0, 6.0], [1.0, 4.0, 8.0], [6.0, 3.0, 4.0]], [[8.0, 8.0, 
5.0], [1.0, 8.0, 8.0], [1.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 2.0], [6.0, 2.0, 5.0], [7.0, 2.0, 7.0]], 
[[5.0, 1.0, 6.0], [5.0, 2.0, 7.0], [4.0, 3.0, 5.0]], [[1.0, 7.0, 4.0], [5.0, 8.0, 3.0], [5.0, 6.0, 3.0]]], [[[5.0, 1.0, 3.0], 
[6.0, 4.0, 3.0], [1.0, 3.0, 3.0]], [[7.0, 6.0, 1.0], [2.0, 2.0, 1.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 6.0], [6.0, 2.0, 8.0], 
[2.0, 8.0, 1.0]]], [[[2.0, 7.0, 8.0], [5.0, 4.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 2.0], [7.0, 1.0, 3.0], [2.0, 8.0, 3.0]], 
[[4.0, 1.0, 5.0], [5.0, 1.0, 8.0], [8.0, 7.0, 7.0]]], [[[3.0, 7.0, 6.0], [6.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[5.0, 3.0, 6.0], 
[1.0, 4.0, 8.0], [6.0, 3.0, 4.0]], [[8.0, 8.0, 5.0], [1.0, 8.0, 8.0], [1.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UULN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 3.0], [6.0, 4.0, 3.0], [5.0, 6.0, 7.0]], [[1.0, 8.0, 4.0], [8.0, 6.0, 3.0], [7.0, 6.0, 
5.0]], [[2.0, 1.0, 5.0], [2.0, 4.0, 4.0], [5.0, 8.0, 8.0]]], [[[3.0, 6.0, 2.0], [4.0, 7.0, 3.0], [6.0, 7.0, 5.0]], [[5.0, 5.0, 
6.0], [4.0, 7.0, 7.0], [3.0, 6.0, 1.0]], [[8.0, 6.0, 5.0], [6.0, 4.0, 5.0], [5.0, 5.0, 5.0]]], [[[1.0, 3.0, 3.0], [6.0, 6.0, 
3.0], [1.0, 3.0, 5.0]], [[6.0, 1.0, 3.0], [7.0, 3.0, 7.0], [8.0, 4.0, 1.0]], [[3.0, 3.0, 3.0], [3.0, 3.0, 7.0], [3.0, 4.0, 
7.0]]], [[[3.0, 6.0, 8.0], [8.0, 1.0, 8.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 7.0], [5.0, 3.0, 6.0], [2.0, 2.0, 1.0]], [[3.0, 8.0, 
4.0], [4.0, 5.0, 6.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 2.0, 4.0]], [[1.0, 5.0, 4.0]]], [[[6.0, 7.0, 6.0]], [[2.0, 5.0, 8.0]]], [[[2.0, 6.0, 5.0]], 
[[6.0, 4.0, 4.0]]]])
      arg[:3,:2,2:]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 3.0], [6.0, 4.0, 3.0], [3.0, 2.0, 4.0]], [[1.0, 8.0, 4.0], [8.0, 6.0, 3.0], [1.0, 5.0, 
4.0]], [[2.0, 1.0, 5.0], [2.0, 4.0, 4.0], [5.0, 8.0, 8.0]]], [[[3.0, 6.0, 2.0], [4.0, 7.0, 3.0], [6.0, 7.0, 6.0]], [[5.0, 5.0, 
6.0], [4.0, 7.0, 7.0], [2.0, 5.0, 8.0]], [[8.0, 6.0, 5.0], [6.0, 4.0, 5.0], [5.0, 5.0, 5.0]]], [[[1.0, 3.0, 3.0], [6.0, 6.0, 
3.0], [2.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [7.0, 3.0, 7.0], [6.0, 4.0, 4.0]], [[3.0, 3.0, 3.0], [3.0, 3.0, 7.0], [3.0, 4.0, 
7.0]]], [[[3.0, 6.0, 8.0], [8.0, 1.0, 8.0], [6.0, 8.0, 4.0]], [[1.0, 6.0, 7.0], [5.0, 3.0, 6.0], [2.0, 2.0, 1.0]], [[3.0, 8.0, 
4.0], [4.0, 5.0, 6.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UULN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 7.0], [7.0, 8.0, 3.0], [2.0, 6.0, 2.0]], [[1.0, 8.0, 2.0], [8.0, 3.0, 6.0], [1.0, 2.0, 
1.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 8.0], [3.0, 8.0, 3.0]]], [[[4.0, 7.0, 2.0], [3.0, 2.0, 1.0], [4.0, 5.0, 8.0]], [[3.0, 1.0, 
3.0], [3.0, 5.0, 5.0], [5.0, 6.0, 5.0]], [[6.0, 1.0, 7.0], [7.0, 1.0, 5.0], [3.0, 4.0, 3.0]]], [[[3.0, 1.0, 8.0], [2.0, 5.0, 
8.0], [7.0, 5.0, 3.0]], [[8.0, 2.0, 3.0], [5.0, 6.0, 6.0], [5.0, 5.0, 7.0]], [[3.0, 1.0, 2.0], [1.0, 5.0, 5.0], [7.0, 2.0, 
3.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 4.0], [8.0, 7.0, 6.0]], [[7.0, 8.0, 8.0], [3.0, 4.0, 6.0], [5.0, 6.0, 6.0]], [[3.0, 5.0, 
5.0], [6.0, 6.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 2.0, 5.0]], [[1.0, 3.0, 2.0]]], [[[1.0, 4.0, 4.0]], [[4.0, 3.0, 2.0]]], [[[4.0, 4.0, 8.0]], 
[[5.0, 6.0, 1.0]]]]),self.functionspace)
      arg[:3,:2,2:]=val
      ref=Data(numpy.array([[[[7.0, 6.0, 7.0], [7.0, 8.0, 3.0], [6.0, 2.0, 5.0]], [[1.0, 8.0, 2.0], [8.0, 3.0, 6.0], [1.0, 3.0, 
2.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 8.0], [3.0, 8.0, 3.0]]], [[[4.0, 7.0, 2.0], [3.0, 2.0, 1.0], [1.0, 4.0, 4.0]], [[3.0, 1.0, 
3.0], [3.0, 5.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 1.0, 7.0], [7.0, 1.0, 5.0], [3.0, 4.0, 3.0]]], [[[3.0, 1.0, 8.0], [2.0, 5.0, 
8.0], [4.0, 4.0, 8.0]], [[8.0, 2.0, 3.0], [5.0, 6.0, 6.0], [5.0, 6.0, 1.0]], [[3.0, 1.0, 2.0], [1.0, 5.0, 5.0], [7.0, 2.0, 
3.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 4.0], [8.0, 7.0, 6.0]], [[7.0, 8.0, 8.0], [3.0, 4.0, 6.0], [5.0, 6.0, 6.0]], [[3.0, 5.0, 
5.0], [6.0, 6.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UULN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 4.0], [2.0, 2.0, 7.0], [7.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], [3.0, 7.0, 8.0], [6.0, 5.0, 
6.0]], [[4.0, 3.0, 3.0], [7.0, 2.0, 8.0], [8.0, 8.0, 2.0]]], [[[1.0, 5.0, 8.0], [2.0, 6.0, 8.0], [2.0, 1.0, 1.0]], [[8.0, 8.0, 
8.0], [4.0, 8.0, 5.0], [1.0, 4.0, 3.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 6.0], [1.0, 7.0, 1.0]]], [[[7.0, 8.0, 1.0], [1.0, 6.0, 
6.0], [1.0, 6.0, 1.0]], [[3.0, 3.0, 8.0], [3.0, 2.0, 7.0], [6.0, 2.0, 6.0]], [[3.0, 8.0, 3.0], [6.0, 4.0, 1.0], [7.0, 3.0, 
1.0]]], [[[3.0, 2.0, 7.0], [1.0, 1.0, 8.0], [1.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [5.0, 4.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 
1.0], [2.0, 7.0, 1.0], [2.0, 1.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 8.0, 5.0]], [[5.0, 3.0, 3.0]]], [[[3.0, 6.0, 5.0]], [[6.0, 7.0, 2.0]]], [[[4.0, 7.0, 
1.0]], [[7.0, 1.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 6.0, 2.0]], [[4.0, 8.0, 2.0]]], [[[3.0, 1.0, 2.0]], [[8.0, 7.0, 
4.0]]], [[[2.0, 8.0, 2.0]], [[8.0, 5.0, 4.0]]]])
      arg[:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 4.0], [2.0, 2.0, 7.0], [2.0, 8.0, 5.0]], [[2.0, 8.0, 7.0], [3.0, 7.0, 8.0], [5.0, 
3.0, 3.0]], [[4.0, 3.0, 3.0], [7.0, 2.0, 8.0], [8.0, 8.0, 2.0]]], [[[1.0, 5.0, 8.0], [2.0, 6.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 
8.0, 8.0], [4.0, 8.0, 5.0], [6.0, 7.0, 2.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 6.0], [1.0, 7.0, 1.0]]], [[[7.0, 8.0, 1.0], [1.0, 
6.0, 6.0], [4.0, 7.0, 1.0]], [[3.0, 3.0, 8.0], [3.0, 2.0, 7.0], [7.0, 1.0, 2.0]], [[3.0, 8.0, 3.0], [6.0, 4.0, 1.0], [7.0, 3.0, 
1.0]]], [[[3.0, 2.0, 7.0], [1.0, 1.0, 8.0], [1.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], [5.0, 4.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 
1.0], [2.0, 7.0, 1.0], [2.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [2.0, 2.0, 7.0], [5.0, 6.0, 2.0]], 
[[2.0, 8.0, 7.0], [3.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[4.0, 3.0, 3.0], [7.0, 2.0, 8.0], [8.0, 8.0, 2.0]]], [[[1.0, 5.0, 8.0], 
[2.0, 6.0, 8.0], [3.0, 1.0, 2.0]], [[8.0, 8.0, 8.0], [4.0, 8.0, 5.0], [8.0, 7.0, 4.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 6.0], 
[1.0, 7.0, 1.0]]], [[[7.0, 8.0, 1.0], [1.0, 6.0, 6.0], [2.0, 8.0, 2.0]], [[3.0, 3.0, 8.0], [3.0, 2.0, 7.0], [8.0, 5.0, 4.0]], 
[[3.0, 8.0, 3.0], [6.0, 4.0, 1.0], [7.0, 3.0, 1.0]]], [[[3.0, 2.0, 7.0], [1.0, 1.0, 8.0], [1.0, 2.0, 7.0]], [[8.0, 7.0, 6.0], 
[5.0, 4.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 1.0], [2.0, 7.0, 1.0], [2.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCRN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 3.0], [6.0, 8.0, 5.0], [3.0, 8.0, 5.0]], [[1.0, 1.0, 5.0], [8.0, 4.0, 5.0], [2.0, 2.0, 
4.0]], [[2.0, 3.0, 5.0], [5.0, 4.0, 6.0], [7.0, 1.0, 7.0]]], [[[5.0, 8.0, 7.0], [5.0, 1.0, 2.0], [7.0, 7.0, 4.0]], [[6.0, 2.0, 
1.0], [3.0, 2.0, 1.0], [7.0, 5.0, 3.0]], [[8.0, 8.0, 3.0], [8.0, 5.0, 5.0], [8.0, 8.0, 1.0]]], [[[6.0, 7.0, 4.0], [5.0, 4.0, 
5.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 6.0], [6.0, 3.0, 7.0], [6.0, 7.0, 4.0]], [[1.0, 8.0, 2.0], [7.0, 8.0, 4.0], [6.0, 7.0, 
7.0]]], [[[2.0, 5.0, 8.0], [7.0, 6.0, 6.0], [5.0, 1.0, 8.0]], [[7.0, 5.0, 8.0], [8.0, 6.0, 7.0], [5.0, 3.0, 7.0]], [[7.0, 6.0, 
6.0], [2.0, 5.0, 3.0], [7.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 7.0, 5.0]], [[2.0, 4.0, 5.0]], [[4.0, 5.0, 6.0]]], [[[3.0, 3.0, 7.0]], [[8.0, 4.0, 6.0]], [[7.0, 
4.0, 2.0]]], [[[5.0, 1.0, 3.0]], [[2.0, 3.0, 7.0]], [[7.0, 1.0, 5.0]]]])
      arg[:3,:,1:2]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 3.0], [8.0, 7.0, 5.0], [3.0, 8.0, 5.0]], [[1.0, 1.0, 5.0], [2.0, 4.0, 5.0], [2.0, 2.0, 
4.0]], [[2.0, 3.0, 5.0], [4.0, 5.0, 6.0], [7.0, 1.0, 7.0]]], [[[5.0, 8.0, 7.0], [3.0, 3.0, 7.0], [7.0, 7.0, 4.0]], [[6.0, 2.0, 
1.0], [8.0, 4.0, 6.0], [7.0, 5.0, 3.0]], [[8.0, 8.0, 3.0], [7.0, 4.0, 2.0], [8.0, 8.0, 1.0]]], [[[6.0, 7.0, 4.0], [5.0, 1.0, 
3.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 6.0], [2.0, 3.0, 7.0], [6.0, 7.0, 4.0]], [[1.0, 8.0, 2.0], [7.0, 1.0, 5.0], [6.0, 7.0, 
7.0]]], [[[2.0, 5.0, 8.0], [7.0, 6.0, 6.0], [5.0, 1.0, 8.0]], [[7.0, 5.0, 8.0], [8.0, 6.0, 7.0], [5.0, 3.0, 7.0]], [[7.0, 6.0, 
6.0], [2.0, 5.0, 3.0], [7.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCRN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [1.0, 6.0, 1.0], [6.0, 5.0, 1.0]], [[8.0, 8.0, 6.0], [7.0, 7.0, 3.0], [7.0, 8.0, 
1.0]], [[7.0, 1.0, 8.0], [5.0, 1.0, 2.0], [6.0, 5.0, 1.0]]], [[[6.0, 7.0, 2.0], [4.0, 7.0, 8.0], [1.0, 3.0, 5.0]], [[2.0, 4.0, 
3.0], [5.0, 5.0, 7.0], [7.0, 3.0, 7.0]], [[8.0, 3.0, 1.0], [3.0, 2.0, 1.0], [8.0, 3.0, 5.0]]], [[[1.0, 1.0, 1.0], [7.0, 8.0, 
2.0], [3.0, 7.0, 3.0]], [[8.0, 6.0, 7.0], [1.0, 3.0, 2.0], [4.0, 6.0, 4.0]], [[1.0, 7.0, 1.0], [1.0, 8.0, 4.0], [2.0, 7.0, 
2.0]]], [[[8.0, 3.0, 6.0], [3.0, 8.0, 3.0], [7.0, 8.0, 4.0]], [[2.0, 5.0, 8.0], [2.0, 2.0, 7.0], [5.0, 8.0, 1.0]], [[6.0, 5.0, 
7.0], [5.0, 3.0, 6.0], [4.0, 5.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 1.0, 7.0]], [[8.0, 3.0, 8.0]], [[1.0, 1.0, 7.0]]], [[[6.0, 7.0, 4.0]], [[7.0, 8.0, 4.0]], 
[[5.0, 1.0, 3.0]]], [[[7.0, 7.0, 8.0]], [[8.0, 8.0, 2.0]], [[4.0, 2.0, 8.0]]]]),self.functionspace)
      arg[:3,:,1:2]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 5.0], [3.0, 1.0, 7.0], [6.0, 5.0, 1.0]], [[8.0, 8.0, 6.0], [8.0, 3.0, 8.0], [7.0, 8.0, 
1.0]], [[7.0, 1.0, 8.0], [1.0, 1.0, 7.0], [6.0, 5.0, 1.0]]], [[[6.0, 7.0, 2.0], [6.0, 7.0, 4.0], [1.0, 3.0, 5.0]], [[2.0, 4.0, 
3.0], [7.0, 8.0, 4.0], [7.0, 3.0, 7.0]], [[8.0, 3.0, 1.0], [5.0, 1.0, 3.0], [8.0, 3.0, 5.0]]], [[[1.0, 1.0, 1.0], [7.0, 7.0, 
8.0], [3.0, 7.0, 3.0]], [[8.0, 6.0, 7.0], [8.0, 8.0, 2.0], [4.0, 6.0, 4.0]], [[1.0, 7.0, 1.0], [4.0, 2.0, 8.0], [2.0, 7.0, 
2.0]]], [[[8.0, 3.0, 6.0], [3.0, 8.0, 3.0], [7.0, 8.0, 4.0]], [[2.0, 5.0, 8.0], [2.0, 2.0, 7.0], [5.0, 8.0, 1.0]], [[6.0, 5.0, 
7.0], [5.0, 3.0, 6.0], [4.0, 5.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCRN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 8.0], [6.0, 6.0, 7.0], [6.0, 7.0, 6.0]], [[4.0, 7.0, 6.0], [7.0, 7.0, 7.0], [1.0, 2.0, 
3.0]], [[4.0, 6.0, 7.0], [6.0, 1.0, 7.0], [5.0, 8.0, 7.0]]], [[[6.0, 3.0, 1.0], [5.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[1.0, 1.0, 
5.0], [2.0, 6.0, 1.0], [8.0, 4.0, 8.0]], [[3.0, 2.0, 4.0], [5.0, 1.0, 1.0], [1.0, 5.0, 1.0]]], [[[1.0, 3.0, 7.0], [8.0, 6.0, 
4.0], [7.0, 8.0, 2.0]], [[8.0, 1.0, 2.0], [5.0, 3.0, 4.0], [3.0, 3.0, 3.0]], [[2.0, 2.0, 8.0], [4.0, 8.0, 4.0], [1.0, 7.0, 
2.0]]], [[[2.0, 4.0, 8.0], [1.0, 2.0, 2.0], [4.0, 8.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[5.0, 8.0, 
5.0], [4.0, 3.0, 2.0], [5.0, 6.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 4.0, 6.0]], [[2.0, 5.0, 5.0]], [[7.0, 5.0, 3.0]]], [[[2.0, 1.0, 3.0]], [[2.0, 3.0, 
2.0]], [[5.0, 8.0, 8.0]]], [[[5.0, 4.0, 4.0]], [[2.0, 2.0, 2.0]], [[8.0, 7.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 4.0, 
2.0]], [[2.0, 5.0, 5.0]], [[5.0, 6.0, 2.0]]], [[[6.0, 7.0, 2.0]], [[8.0, 7.0, 1.0]], [[2.0, 1.0, 1.0]]], [[[8.0, 2.0, 2.0]], 
[[1.0, 7.0, 2.0]], [[1.0, 4.0, 7.0]]]])
      arg[:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 8.0], [7.0, 4.0, 6.0], [6.0, 7.0, 6.0]], [[4.0, 7.0, 6.0], [2.0, 5.0, 5.0], [1.0, 
2.0, 3.0]], [[4.0, 6.0, 7.0], [7.0, 5.0, 3.0], [5.0, 8.0, 7.0]]], [[[6.0, 3.0, 1.0], [2.0, 1.0, 3.0], [5.0, 4.0, 4.0]], [[1.0, 
1.0, 5.0], [2.0, 3.0, 2.0], [8.0, 4.0, 8.0]], [[3.0, 2.0, 4.0], [5.0, 8.0, 8.0], [1.0, 5.0, 1.0]]], [[[1.0, 3.0, 7.0], [5.0, 
4.0, 4.0], [7.0, 8.0, 2.0]], [[8.0, 1.0, 2.0], [2.0, 2.0, 2.0], [3.0, 3.0, 3.0]], [[2.0, 2.0, 8.0], [8.0, 7.0, 5.0], [1.0, 7.0, 
2.0]]], [[[2.0, 4.0, 8.0], [1.0, 2.0, 2.0], [4.0, 8.0, 8.0]], [[5.0, 4.0, 5.0], [2.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[5.0, 8.0, 
5.0], [4.0, 3.0, 2.0], [5.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 8.0], [4.0, 4.0, 2.0], [6.0, 7.0, 6.0]], 
[[4.0, 7.0, 6.0], [2.0, 5.0, 5.0], [1.0, 2.0, 3.0]], [[4.0, 6.0, 7.0], [5.0, 6.0, 2.0], [5.0, 8.0, 7.0]]], [[[6.0, 3.0, 1.0], 
[6.0, 7.0, 2.0], [5.0, 4.0, 4.0]], [[1.0, 1.0, 5.0], [8.0, 7.0, 1.0], [8.0, 4.0, 8.0]], [[3.0, 2.0, 4.0], [2.0, 1.0, 1.0], 
[1.0, 5.0, 1.0]]], [[[1.0, 3.0, 7.0], [8.0, 2.0, 2.0], [7.0, 8.0, 2.0]], [[8.0, 1.0, 2.0], [1.0, 7.0, 2.0], [3.0, 3.0, 3.0]], 
[[2.0, 2.0, 8.0], [1.0, 4.0, 7.0], [1.0, 7.0, 2.0]]], [[[2.0, 4.0, 8.0], [1.0, 2.0, 2.0], [4.0, 8.0, 8.0]], [[5.0, 4.0, 5.0], 
[2.0, 6.0, 1.0], [7.0, 4.0, 8.0]], [[5.0, 8.0, 5.0], [4.0, 3.0, 2.0], [5.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUCU(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 2.0], [1.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[1.0, 8.0, 8.0], [3.0, 1.0, 8.0], [6.0, 2.0, 
3.0]], [[8.0, 1.0, 1.0], [4.0, 8.0, 4.0], [2.0, 4.0, 4.0]]], [[[8.0, 7.0, 6.0], [8.0, 7.0, 3.0], [6.0, 8.0, 6.0]], [[2.0, 8.0, 
3.0], [3.0, 1.0, 1.0], [3.0, 2.0, 8.0]], [[1.0, 8.0, 5.0], [2.0, 6.0, 1.0], [4.0, 8.0, 3.0]]], [[[5.0, 6.0, 5.0], [7.0, 1.0, 
1.0], [7.0, 8.0, 5.0]], [[8.0, 1.0, 5.0], [3.0, 4.0, 2.0], [8.0, 8.0, 3.0]], [[3.0, 6.0, 4.0], [7.0, 7.0, 7.0], [5.0, 5.0, 
8.0]]], [[[2.0, 5.0, 4.0], [7.0, 4.0, 3.0], [2.0, 6.0, 6.0]], [[5.0, 2.0, 4.0], [1.0, 4.0, 8.0], [8.0, 8.0, 2.0]], [[5.0, 6.0, 
8.0], [7.0, 7.0, 1.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 4.0], [6.0, 4.0], [4.0, 3.0]], [[2.0, 3.0], [1.0, 5.0], [3.0, 8.0]]], [[[7.0, 5.0], [1.0, 7.0], 
[6.0, 3.0]], [[6.0, 6.0], [1.0, 2.0], [2.0, 8.0]]]])
      arg[1:3,:2,:,:2]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 2.0], [1.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[1.0, 8.0, 8.0], [3.0, 1.0, 8.0], [6.0, 2.0, 
3.0]], [[8.0, 1.0, 1.0], [4.0, 8.0, 4.0], [2.0, 4.0, 4.0]]], [[[5.0, 4.0, 6.0], [6.0, 4.0, 3.0], [4.0, 3.0, 6.0]], [[2.0, 3.0, 
3.0], [1.0, 5.0, 1.0], [3.0, 8.0, 8.0]], [[1.0, 8.0, 5.0], [2.0, 6.0, 1.0], [4.0, 8.0, 3.0]]], [[[7.0, 5.0, 5.0], [1.0, 7.0, 
1.0], [6.0, 3.0, 5.0]], [[6.0, 6.0, 5.0], [1.0, 2.0, 2.0], [2.0, 8.0, 3.0]], [[3.0, 6.0, 4.0], [7.0, 7.0, 7.0], [5.0, 5.0, 
8.0]]], [[[2.0, 5.0, 4.0], [7.0, 4.0, 3.0], [2.0, 6.0, 6.0]], [[5.0, 2.0, 4.0], [1.0, 4.0, 8.0], [8.0, 8.0, 2.0]], [[5.0, 6.0, 
8.0], [7.0, 7.0, 1.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUCU(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 4.0], [7.0, 7.0, 4.0], [7.0, 2.0, 5.0]], [[8.0, 1.0, 5.0], [1.0, 7.0, 1.0], [1.0, 6.0, 
7.0]], [[5.0, 4.0, 2.0], [7.0, 7.0, 3.0], [4.0, 8.0, 3.0]]], [[[1.0, 3.0, 1.0], [5.0, 4.0, 7.0], [2.0, 2.0, 7.0]], [[2.0, 4.0, 
4.0], [5.0, 1.0, 8.0], [2.0, 4.0, 5.0]], [[6.0, 5.0, 8.0], [8.0, 8.0, 4.0], [2.0, 2.0, 6.0]]], [[[4.0, 1.0, 2.0], [5.0, 3.0, 
2.0], [4.0, 6.0, 1.0]], [[3.0, 1.0, 4.0], [6.0, 4.0, 8.0], [8.0, 6.0, 6.0]], [[2.0, 4.0, 5.0], [6.0, 1.0, 7.0], [4.0, 8.0, 
1.0]]], [[[6.0, 4.0, 6.0], [3.0, 1.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 1.0, 3.0], [1.0, 4.0, 1.0], [8.0, 2.0, 3.0]], [[2.0, 2.0, 
5.0], [5.0, 4.0, 3.0], [1.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 7.0], [3.0, 3.0], [3.0, 6.0]], [[8.0, 4.0], [3.0, 8.0], [4.0, 5.0]]], [[[3.0, 5.0], [1.0, 
1.0], [6.0, 4.0]], [[5.0, 8.0], [3.0, 8.0], [7.0, 7.0]]]]),self.functionspace)
      arg[1:3,:2,:,:2]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 4.0], [7.0, 7.0, 4.0], [7.0, 2.0, 5.0]], [[8.0, 1.0, 5.0], [1.0, 7.0, 1.0], [1.0, 6.0, 
7.0]], [[5.0, 4.0, 2.0], [7.0, 7.0, 3.0], [4.0, 8.0, 3.0]]], [[[1.0, 7.0, 1.0], [3.0, 3.0, 7.0], [3.0, 6.0, 7.0]], [[8.0, 4.0, 
4.0], [3.0, 8.0, 8.0], [4.0, 5.0, 5.0]], [[6.0, 5.0, 8.0], [8.0, 8.0, 4.0], [2.0, 2.0, 6.0]]], [[[3.0, 5.0, 2.0], [1.0, 1.0, 
2.0], [6.0, 4.0, 1.0]], [[5.0, 8.0, 4.0], [3.0, 8.0, 8.0], [7.0, 7.0, 6.0]], [[2.0, 4.0, 5.0], [6.0, 1.0, 7.0], [4.0, 8.0, 
1.0]]], [[[6.0, 4.0, 6.0], [3.0, 1.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 1.0, 3.0], [1.0, 4.0, 1.0], [8.0, 2.0, 3.0]], [[2.0, 2.0, 
5.0], [5.0, 4.0, 3.0], [1.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUCU(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 7.0], [3.0, 1.0, 5.0], [3.0, 3.0, 2.0]], [[7.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 6.0, 
1.0]], [[4.0, 6.0, 1.0], [3.0, 3.0, 8.0], [7.0, 3.0, 1.0]]], [[[4.0, 4.0, 8.0], [5.0, 1.0, 4.0], [1.0, 6.0, 3.0]], [[2.0, 5.0, 
1.0], [6.0, 2.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 2.0, 5.0], [6.0, 6.0, 6.0], [5.0, 3.0, 8.0]]], [[[2.0, 3.0, 4.0], [1.0, 3.0, 
2.0], [2.0, 7.0, 8.0]], [[8.0, 5.0, 2.0], [2.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 4.0, 5.0], [2.0, 2.0, 
5.0]]], [[[4.0, 5.0, 6.0], [3.0, 3.0, 5.0], [3.0, 3.0, 5.0]], [[5.0, 3.0, 4.0], [6.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[5.0, 5.0, 
3.0], [8.0, 4.0, 5.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 5.0], [3.0, 3.0], [6.0, 6.0]], [[3.0, 8.0], [7.0, 2.0], [5.0, 3.0]]], [[[8.0, 3.0], 
[5.0, 8.0], [5.0, 1.0]], [[8.0, 8.0], [4.0, 4.0], [1.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 7.0], [4.0, 1.0], [8.0, 
1.0]], [[4.0, 3.0], [6.0, 7.0], [7.0, 5.0]]], [[[2.0, 7.0], [2.0, 2.0], [6.0, 7.0]], [[6.0, 6.0], [7.0, 2.0], [2.0, 4.0]]]])
      arg[1:3,:2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 7.0], [3.0, 1.0, 5.0], [3.0, 3.0, 2.0]], [[7.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 
6.0, 1.0]], [[4.0, 6.0, 1.0], [3.0, 3.0, 8.0], [7.0, 3.0, 1.0]]], [[[8.0, 5.0, 8.0], [3.0, 3.0, 4.0], [6.0, 6.0, 3.0]], [[3.0, 
8.0, 1.0], [7.0, 2.0, 5.0], [5.0, 3.0, 4.0]], [[7.0, 2.0, 5.0], [6.0, 6.0, 6.0], [5.0, 3.0, 8.0]]], [[[8.0, 3.0, 4.0], [5.0, 
8.0, 2.0], [5.0, 1.0, 8.0]], [[8.0, 8.0, 2.0], [4.0, 4.0, 1.0], [1.0, 4.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 4.0, 5.0], [2.0, 2.0, 
5.0]]], [[[4.0, 5.0, 6.0], [3.0, 3.0, 5.0], [3.0, 3.0, 5.0]], [[5.0, 3.0, 4.0], [6.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[5.0, 5.0, 
3.0], [8.0, 4.0, 5.0], [4.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 7.0], [3.0, 1.0, 5.0], [3.0, 3.0, 2.0]], 
[[7.0, 3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 6.0, 1.0]], [[4.0, 6.0, 1.0], [3.0, 3.0, 8.0], [7.0, 3.0, 1.0]]], [[[7.0, 7.0, 8.0], 
[4.0, 1.0, 4.0], [8.0, 1.0, 3.0]], [[4.0, 3.0, 1.0], [6.0, 7.0, 5.0], [7.0, 5.0, 4.0]], [[7.0, 2.0, 5.0], [6.0, 6.0, 6.0], 
[5.0, 3.0, 8.0]]], [[[2.0, 7.0, 4.0], [2.0, 2.0, 2.0], [6.0, 7.0, 8.0]], [[6.0, 6.0, 2.0], [7.0, 2.0, 1.0], [2.0, 4.0, 7.0]], 
[[8.0, 6.0, 7.0], [2.0, 4.0, 5.0], [2.0, 2.0, 5.0]]], [[[4.0, 5.0, 6.0], [3.0, 3.0, 5.0], [3.0, 3.0, 5.0]], [[5.0, 3.0, 4.0], 
[6.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [8.0, 4.0, 5.0], [4.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLUR(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 3.0], [7.0, 4.0, 4.0], [5.0, 5.0, 6.0]], [[2.0, 7.0, 8.0], [4.0, 1.0, 2.0], [1.0, 4.0, 
5.0]], [[8.0, 4.0, 1.0], [6.0, 5.0, 1.0], [4.0, 4.0, 5.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 8.0], [8.0, 5.0, 3.0]], [[4.0, 8.0, 
4.0], [7.0, 4.0, 6.0], [4.0, 1.0, 4.0]], [[2.0, 8.0, 2.0], [6.0, 1.0, 2.0], [5.0, 7.0, 3.0]]], [[[5.0, 8.0, 4.0], [6.0, 8.0, 
4.0], [6.0, 6.0, 5.0]], [[6.0, 7.0, 2.0], [8.0, 1.0, 1.0], [4.0, 5.0, 8.0]], [[4.0, 8.0, 6.0], [8.0, 4.0, 7.0], [4.0, 3.0, 
6.0]]], [[[2.0, 5.0, 6.0], [1.0, 6.0, 2.0], [7.0, 3.0, 3.0]], [[1.0, 7.0, 7.0], [8.0, 1.0, 3.0], [1.0, 8.0, 8.0]], [[4.0, 7.0, 
7.0], [8.0, 7.0, 1.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[8.0], [3.0]]])
      arg[2,2:,:2,1:2]=val
      ref=Data(numpy.array([[[[7.0, 6.0, 3.0], [7.0, 4.0, 4.0], [5.0, 5.0, 6.0]], [[2.0, 7.0, 8.0], [4.0, 1.0, 2.0], [1.0, 4.0, 
5.0]], [[8.0, 4.0, 1.0], [6.0, 5.0, 1.0], [4.0, 4.0, 5.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 8.0], [8.0, 5.0, 3.0]], [[4.0, 8.0, 
4.0], [7.0, 4.0, 6.0], [4.0, 1.0, 4.0]], [[2.0, 8.0, 2.0], [6.0, 1.0, 2.0], [5.0, 7.0, 3.0]]], [[[5.0, 8.0, 4.0], [6.0, 8.0, 
4.0], [6.0, 6.0, 5.0]], [[6.0, 7.0, 2.0], [8.0, 1.0, 1.0], [4.0, 5.0, 8.0]], [[4.0, 8.0, 6.0], [8.0, 3.0, 7.0], [4.0, 3.0, 
6.0]]], [[[2.0, 5.0, 6.0], [1.0, 6.0, 2.0], [7.0, 3.0, 3.0]], [[1.0, 7.0, 7.0], [8.0, 1.0, 3.0], [1.0, 8.0, 8.0]], [[4.0, 7.0, 
7.0], [8.0, 7.0, 1.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLUR(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 4.0], [7.0, 3.0, 8.0], [2.0, 5.0, 4.0]], [[3.0, 7.0, 6.0], [4.0, 5.0, 1.0], [2.0, 2.0, 
4.0]], [[2.0, 3.0, 7.0], [1.0, 6.0, 4.0], [1.0, 5.0, 8.0]]], [[[8.0, 4.0, 5.0], [6.0, 5.0, 1.0], [8.0, 8.0, 2.0]], [[5.0, 2.0, 
1.0], [2.0, 7.0, 8.0], [1.0, 2.0, 5.0]], [[4.0, 3.0, 2.0], [8.0, 6.0, 5.0], [4.0, 7.0, 2.0]]], [[[7.0, 7.0, 1.0], [7.0, 2.0, 
8.0], [2.0, 5.0, 6.0]], [[1.0, 1.0, 1.0], [1.0, 4.0, 6.0], [3.0, 6.0, 7.0]], [[8.0, 1.0, 3.0], [8.0, 1.0, 4.0], [1.0, 3.0, 
1.0]]], [[[8.0, 3.0, 6.0], [5.0, 1.0, 4.0], [4.0, 3.0, 3.0]], [[4.0, 2.0, 2.0], [3.0, 8.0, 7.0], [1.0, 4.0, 8.0]], [[4.0, 2.0, 
8.0], [6.0, 2.0, 7.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0], [1.0]]]),self.functionspace)
      arg[2,2:,:2,1:2]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 4.0], [7.0, 3.0, 8.0], [2.0, 5.0, 4.0]], [[3.0, 7.0, 6.0], [4.0, 5.0, 1.0], [2.0, 2.0, 
4.0]], [[2.0, 3.0, 7.0], [1.0, 6.0, 4.0], [1.0, 5.0, 8.0]]], [[[8.0, 4.0, 5.0], [6.0, 5.0, 1.0], [8.0, 8.0, 2.0]], [[5.0, 2.0, 
1.0], [2.0, 7.0, 8.0], [1.0, 2.0, 5.0]], [[4.0, 3.0, 2.0], [8.0, 6.0, 5.0], [4.0, 7.0, 2.0]]], [[[7.0, 7.0, 1.0], [7.0, 2.0, 
8.0], [2.0, 5.0, 6.0]], [[1.0, 1.0, 1.0], [1.0, 4.0, 6.0], [3.0, 6.0, 7.0]], [[8.0, 4.0, 3.0], [8.0, 1.0, 4.0], [1.0, 3.0, 
1.0]]], [[[8.0, 3.0, 6.0], [5.0, 1.0, 4.0], [4.0, 3.0, 3.0]], [[4.0, 2.0, 2.0], [3.0, 8.0, 7.0], [1.0, 4.0, 8.0]], [[4.0, 2.0, 
8.0], [6.0, 2.0, 7.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLUR(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 6.0], [8.0, 6.0, 1.0], [4.0, 3.0, 2.0]], [[5.0, 4.0, 8.0], [5.0, 4.0, 4.0], [6.0, 6.0, 
7.0]], [[5.0, 8.0, 4.0], [8.0, 1.0, 6.0], [6.0, 4.0, 5.0]]], [[[7.0, 2.0, 4.0], [4.0, 2.0, 1.0], [7.0, 1.0, 4.0]], [[4.0, 8.0, 
8.0], [2.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 2.0], [1.0, 6.0, 3.0]]], [[[5.0, 7.0, 8.0], [4.0, 1.0, 
7.0], [7.0, 1.0, 4.0]], [[2.0, 3.0, 8.0], [5.0, 4.0, 5.0], [7.0, 2.0, 8.0]], [[6.0, 2.0, 7.0], [4.0, 4.0, 7.0], [3.0, 6.0, 
8.0]]], [[[7.0, 5.0, 5.0], [6.0, 5.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [5.0, 8.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 4.0, 
4.0], [8.0, 3.0, 2.0], [7.0, 4.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0], [5.0]]])+(1.-msk_val)*numpy.array([[[3.0], [3.0]]])
      arg[2,2:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [8.0, 6.0, 1.0], [4.0, 3.0, 2.0]], [[5.0, 4.0, 8.0], [5.0, 4.0, 4.0], [6.0, 
6.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 1.0, 6.0], [6.0, 4.0, 5.0]]], [[[7.0, 2.0, 4.0], [4.0, 2.0, 1.0], [7.0, 1.0, 4.0]], [[4.0, 
8.0, 8.0], [2.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 2.0], [1.0, 6.0, 3.0]]], [[[5.0, 7.0, 8.0], [4.0, 
1.0, 7.0], [7.0, 1.0, 4.0]], [[2.0, 3.0, 8.0], [5.0, 4.0, 5.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 7.0], [4.0, 5.0, 7.0], [3.0, 6.0, 
8.0]]], [[[7.0, 5.0, 5.0], [6.0, 5.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [5.0, 8.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 4.0, 
4.0], [8.0, 3.0, 2.0], [7.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 6.0], [8.0, 6.0, 1.0], [4.0, 3.0, 2.0]], 
[[5.0, 4.0, 8.0], [5.0, 4.0, 4.0], [6.0, 6.0, 7.0]], [[5.0, 8.0, 4.0], [8.0, 1.0, 6.0], [6.0, 4.0, 5.0]]], [[[7.0, 2.0, 4.0], 
[4.0, 2.0, 1.0], [7.0, 1.0, 4.0]], [[4.0, 8.0, 8.0], [2.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 2.0], 
[1.0, 6.0, 3.0]]], [[[5.0, 7.0, 8.0], [4.0, 1.0, 7.0], [7.0, 1.0, 4.0]], [[2.0, 3.0, 8.0], [5.0, 4.0, 5.0], [7.0, 2.0, 8.0]], 
[[6.0, 3.0, 7.0], [4.0, 3.0, 7.0], [3.0, 6.0, 8.0]]], [[[7.0, 5.0, 5.0], [6.0, 5.0, 1.0], [7.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], 
[5.0, 8.0, 6.0], [3.0, 4.0, 6.0]], [[6.0, 4.0, 4.0], [8.0, 3.0, 2.0], [7.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CULU(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 1.0], [7.0, 6.0, 1.0], [2.0, 6.0, 3.0]], [[3.0, 4.0, 6.0], [7.0, 2.0, 4.0], [1.0, 4.0, 
8.0]], [[5.0, 3.0, 1.0], [1.0, 2.0, 3.0], [4.0, 7.0, 8.0]]], [[[2.0, 2.0, 7.0], [5.0, 7.0, 7.0], [1.0, 5.0, 2.0]], [[4.0, 2.0, 
1.0], [4.0, 1.0, 8.0], [1.0, 1.0, 7.0]], [[4.0, 8.0, 3.0], [5.0, 1.0, 5.0], [8.0, 2.0, 6.0]]], [[[2.0, 6.0, 2.0], [6.0, 6.0, 
4.0], [5.0, 8.0, 1.0]], [[3.0, 5.0, 6.0], [3.0, 5.0, 7.0], [5.0, 1.0, 2.0]], [[8.0, 4.0, 8.0], [6.0, 7.0, 2.0], [2.0, 1.0, 
4.0]]], [[[1.0, 1.0, 6.0], [8.0, 4.0, 2.0], [1.0, 5.0, 5.0]], [[5.0, 7.0, 8.0], [1.0, 6.0, 2.0], [6.0, 7.0, 2.0]], [[3.0, 4.0, 
5.0], [1.0, 6.0, 4.0], [1.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 6.0]], [[1.0, 6.0]]], [[[3.0, 2.0]], [[4.0, 5.0]]], [[[6.0, 4.0]], [[5.0, 3.0]]], [[[1.0, 2.0]], 
[[6.0, 2.0]]]])
      arg[:,:2,2:,:2]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 1.0], [7.0, 6.0, 1.0], [2.0, 6.0, 3.0]], [[3.0, 4.0, 6.0], [7.0, 2.0, 4.0], [1.0, 6.0, 
8.0]], [[5.0, 3.0, 1.0], [1.0, 2.0, 3.0], [4.0, 7.0, 8.0]]], [[[2.0, 2.0, 7.0], [5.0, 7.0, 7.0], [3.0, 2.0, 2.0]], [[4.0, 2.0, 
1.0], [4.0, 1.0, 8.0], [4.0, 5.0, 7.0]], [[4.0, 8.0, 3.0], [5.0, 1.0, 5.0], [8.0, 2.0, 6.0]]], [[[2.0, 6.0, 2.0], [6.0, 6.0, 
4.0], [6.0, 4.0, 1.0]], [[3.0, 5.0, 6.0], [3.0, 5.0, 7.0], [5.0, 3.0, 2.0]], [[8.0, 4.0, 8.0], [6.0, 7.0, 2.0], [2.0, 1.0, 
4.0]]], [[[1.0, 1.0, 6.0], [8.0, 4.0, 2.0], [1.0, 2.0, 5.0]], [[5.0, 7.0, 8.0], [1.0, 6.0, 2.0], [6.0, 2.0, 2.0]], [[3.0, 4.0, 
5.0], [1.0, 6.0, 4.0], [1.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CULU(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 3.0], [7.0, 7.0, 2.0], [7.0, 7.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 3.0, 7.0], [2.0, 8.0, 
6.0]], [[5.0, 6.0, 5.0], [7.0, 4.0, 7.0], [1.0, 3.0, 5.0]]], [[[4.0, 7.0, 3.0], [3.0, 4.0, 7.0], [6.0, 1.0, 3.0]], [[7.0, 3.0, 
1.0], [6.0, 6.0, 8.0], [3.0, 2.0, 5.0]], [[2.0, 7.0, 2.0], [3.0, 5.0, 5.0], [1.0, 8.0, 5.0]]], [[[1.0, 2.0, 1.0], [6.0, 4.0, 
2.0], [1.0, 7.0, 4.0]], [[2.0, 7.0, 1.0], [2.0, 7.0, 6.0], [7.0, 5.0, 5.0]], [[2.0, 3.0, 4.0], [7.0, 8.0, 7.0], [1.0, 2.0, 
7.0]]], [[[5.0, 8.0, 3.0], [5.0, 4.0, 4.0], [4.0, 8.0, 2.0]], [[6.0, 7.0, 5.0], [1.0, 3.0, 3.0], [1.0, 6.0, 2.0]], [[3.0, 5.0, 
5.0], [4.0, 8.0, 4.0], [6.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 8.0]], [[4.0, 5.0]]], [[[7.0, 4.0]], [[8.0, 1.0]]], [[[5.0, 3.0]], [[6.0, 2.0]]], [[[5.0, 
4.0]], [[3.0, 7.0]]]]),self.functionspace)
      arg[:,:2,2:,:2]=val
      ref=Data(numpy.array([[[[5.0, 7.0, 3.0], [7.0, 7.0, 2.0], [8.0, 8.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 3.0, 7.0], [4.0, 5.0, 
6.0]], [[5.0, 6.0, 5.0], [7.0, 4.0, 7.0], [1.0, 3.0, 5.0]]], [[[4.0, 7.0, 3.0], [3.0, 4.0, 7.0], [7.0, 4.0, 3.0]], [[7.0, 3.0, 
1.0], [6.0, 6.0, 8.0], [8.0, 1.0, 5.0]], [[2.0, 7.0, 2.0], [3.0, 5.0, 5.0], [1.0, 8.0, 5.0]]], [[[1.0, 2.0, 1.0], [6.0, 4.0, 
2.0], [5.0, 3.0, 4.0]], [[2.0, 7.0, 1.0], [2.0, 7.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 3.0, 4.0], [7.0, 8.0, 7.0], [1.0, 2.0, 
7.0]]], [[[5.0, 8.0, 3.0], [5.0, 4.0, 4.0], [5.0, 4.0, 2.0]], [[6.0, 7.0, 5.0], [1.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[3.0, 5.0, 
5.0], [4.0, 8.0, 4.0], [6.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CULU(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 6.0], [6.0, 4.0, 2.0], [8.0, 2.0, 7.0]], [[6.0, 3.0, 6.0], [4.0, 6.0, 4.0], [7.0, 1.0, 
5.0]], [[7.0, 8.0, 4.0], [8.0, 6.0, 6.0], [6.0, 3.0, 5.0]]], [[[2.0, 2.0, 5.0], [8.0, 2.0, 5.0], [8.0, 3.0, 6.0]], [[3.0, 1.0, 
7.0], [2.0, 4.0, 6.0], [8.0, 5.0, 2.0]], [[7.0, 3.0, 8.0], [8.0, 7.0, 8.0], [2.0, 2.0, 6.0]]], [[[5.0, 5.0, 4.0], [2.0, 6.0, 
2.0], [7.0, 4.0, 4.0]], [[8.0, 5.0, 1.0], [6.0, 1.0, 6.0], [7.0, 5.0, 7.0]], [[6.0, 7.0, 2.0], [8.0, 8.0, 6.0], [2.0, 8.0, 
3.0]]], [[[6.0, 3.0, 6.0], [8.0, 5.0, 2.0], [8.0, 7.0, 8.0]], [[3.0, 8.0, 5.0], [5.0, 7.0, 6.0], [1.0, 5.0, 3.0]], [[8.0, 2.0, 
7.0], [8.0, 2.0, 8.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 5.0]], [[6.0, 4.0]]], [[[1.0, 6.0]], [[1.0, 4.0]]], [[[2.0, 2.0]], [[4.0, 3.0]]], 
[[[5.0, 7.0]], [[2.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 7.0]], [[8.0, 5.0]]], [[[6.0, 7.0]], [[2.0, 4.0]]], [[[5.0, 
4.0]], [[5.0, 2.0]]], [[[8.0, 3.0]], [[8.0, 3.0]]]])
      arg[:,:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 6.0], [6.0, 4.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 3.0, 6.0], [4.0, 6.0, 4.0], [6.0, 
4.0, 5.0]], [[7.0, 8.0, 4.0], [8.0, 6.0, 6.0], [6.0, 3.0, 5.0]]], [[[2.0, 2.0, 5.0], [8.0, 2.0, 5.0], [1.0, 6.0, 6.0]], [[3.0, 
1.0, 7.0], [2.0, 4.0, 6.0], [1.0, 4.0, 2.0]], [[7.0, 3.0, 8.0], [8.0, 7.0, 8.0], [2.0, 2.0, 6.0]]], [[[5.0, 5.0, 4.0], [2.0, 
6.0, 2.0], [2.0, 2.0, 4.0]], [[8.0, 5.0, 1.0], [6.0, 1.0, 6.0], [4.0, 3.0, 7.0]], [[6.0, 7.0, 2.0], [8.0, 8.0, 6.0], [2.0, 8.0, 
3.0]]], [[[6.0, 3.0, 6.0], [8.0, 5.0, 2.0], [5.0, 7.0, 8.0]], [[3.0, 8.0, 5.0], [5.0, 7.0, 6.0], [2.0, 1.0, 3.0]], [[8.0, 2.0, 
7.0], [8.0, 2.0, 8.0], [8.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 6.0], [6.0, 4.0, 2.0], [7.0, 7.0, 7.0]], 
[[6.0, 3.0, 6.0], [4.0, 6.0, 4.0], [8.0, 5.0, 5.0]], [[7.0, 8.0, 4.0], [8.0, 6.0, 6.0], [6.0, 3.0, 5.0]]], [[[2.0, 2.0, 5.0], 
[8.0, 2.0, 5.0], [6.0, 7.0, 6.0]], [[3.0, 1.0, 7.0], [2.0, 4.0, 6.0], [2.0, 4.0, 2.0]], [[7.0, 3.0, 8.0], [8.0, 7.0, 8.0], 
[2.0, 2.0, 6.0]]], [[[5.0, 5.0, 4.0], [2.0, 6.0, 2.0], [5.0, 4.0, 4.0]], [[8.0, 5.0, 1.0], [6.0, 1.0, 6.0], [5.0, 2.0, 7.0]], 
[[6.0, 7.0, 2.0], [8.0, 8.0, 6.0], [2.0, 8.0, 3.0]]], [[[6.0, 3.0, 6.0], [8.0, 5.0, 2.0], [8.0, 3.0, 8.0]], [[3.0, 8.0, 5.0], 
[5.0, 7.0, 6.0], [8.0, 3.0, 3.0]], [[8.0, 2.0, 7.0], [8.0, 2.0, 8.0], [8.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPCL(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [7.0, 5.0, 6.0], [6.0, 4.0, 7.0]], [[2.0, 6.0, 8.0], [4.0, 4.0, 5.0], [5.0, 6.0, 
5.0]], [[8.0, 7.0, 8.0], [6.0, 4.0, 5.0], [4.0, 4.0, 7.0]]], [[[1.0, 6.0, 3.0], [1.0, 3.0, 6.0], [5.0, 1.0, 6.0]], [[4.0, 1.0, 
8.0], [4.0, 5.0, 4.0], [8.0, 3.0, 1.0]], [[3.0, 5.0, 3.0], [1.0, 8.0, 6.0], [8.0, 2.0, 8.0]]], [[[2.0, 5.0, 8.0], [4.0, 6.0, 
7.0], [8.0, 8.0, 3.0]], [[7.0, 4.0, 7.0], [4.0, 3.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 7.0, 7.0], [3.0, 8.0, 6.0], [3.0, 2.0, 
4.0]]], [[[4.0, 1.0, 2.0], [2.0, 5.0, 6.0], [3.0, 8.0, 2.0]], [[6.0, 7.0, 1.0], [6.0, 1.0, 7.0], [1.0, 4.0, 2.0]], [[1.0, 8.0, 
5.0], [3.0, 8.0, 2.0], [4.0, 5.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[8.0], [4.0], [4.0]])
      arg[2,1,:,2:]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 6.0], [7.0, 5.0, 6.0], [6.0, 4.0, 7.0]], [[2.0, 6.0, 8.0], [4.0, 4.0, 5.0], [5.0, 6.0, 
5.0]], [[8.0, 7.0, 8.0], [6.0, 4.0, 5.0], [4.0, 4.0, 7.0]]], [[[1.0, 6.0, 3.0], [1.0, 3.0, 6.0], [5.0, 1.0, 6.0]], [[4.0, 1.0, 
8.0], [4.0, 5.0, 4.0], [8.0, 3.0, 1.0]], [[3.0, 5.0, 3.0], [1.0, 8.0, 6.0], [8.0, 2.0, 8.0]]], [[[2.0, 5.0, 8.0], [4.0, 6.0, 
7.0], [8.0, 8.0, 3.0]], [[7.0, 4.0, 8.0], [4.0, 3.0, 4.0], [5.0, 5.0, 4.0]], [[5.0, 7.0, 7.0], [3.0, 8.0, 6.0], [3.0, 2.0, 
4.0]]], [[[4.0, 1.0, 2.0], [2.0, 5.0, 6.0], [3.0, 8.0, 2.0]], [[6.0, 7.0, 1.0], [6.0, 1.0, 7.0], [1.0, 4.0, 2.0]], [[1.0, 8.0, 
5.0], [3.0, 8.0, 2.0], [4.0, 5.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPCL(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 3.0], [8.0, 2.0, 5.0], [7.0, 3.0, 7.0]], [[4.0, 3.0, 4.0], [3.0, 5.0, 5.0], [3.0, 8.0, 
8.0]], [[3.0, 5.0, 5.0], [3.0, 7.0, 2.0], [3.0, 5.0, 8.0]]], [[[7.0, 6.0, 7.0], [5.0, 7.0, 8.0], [4.0, 6.0, 4.0]], [[2.0, 4.0, 
2.0], [3.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[4.0, 6.0, 1.0], [5.0, 1.0, 6.0], [8.0, 6.0, 6.0]]], [[[3.0, 2.0, 3.0], [2.0, 7.0, 
1.0], [8.0, 8.0, 4.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 8.0], [8.0, 6.0, 6.0]], [[7.0, 1.0, 4.0], [7.0, 2.0, 4.0], [4.0, 6.0, 
5.0]]], [[[4.0, 1.0, 4.0], [6.0, 7.0, 8.0], [2.0, 2.0, 5.0]], [[2.0, 5.0, 3.0], [8.0, 8.0, 7.0], [5.0, 8.0, 4.0]], [[8.0, 8.0, 
7.0], [2.0, 3.0, 3.0], [8.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[5.0], [8.0], [8.0]]),self.functionspace)
      arg[2,1,:,2:]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 3.0], [8.0, 2.0, 5.0], [7.0, 3.0, 7.0]], [[4.0, 3.0, 4.0], [3.0, 5.0, 5.0], [3.0, 8.0, 
8.0]], [[3.0, 5.0, 5.0], [3.0, 7.0, 2.0], [3.0, 5.0, 8.0]]], [[[7.0, 6.0, 7.0], [5.0, 7.0, 8.0], [4.0, 6.0, 4.0]], [[2.0, 4.0, 
2.0], [3.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[4.0, 6.0, 1.0], [5.0, 1.0, 6.0], [8.0, 6.0, 6.0]]], [[[3.0, 2.0, 3.0], [2.0, 7.0, 
1.0], [8.0, 8.0, 4.0]], [[4.0, 2.0, 5.0], [6.0, 1.0, 8.0], [8.0, 6.0, 8.0]], [[7.0, 1.0, 4.0], [7.0, 2.0, 4.0], [4.0, 6.0, 
5.0]]], [[[4.0, 1.0, 4.0], [6.0, 7.0, 8.0], [2.0, 2.0, 5.0]], [[2.0, 5.0, 3.0], [8.0, 8.0, 7.0], [5.0, 8.0, 4.0]], [[8.0, 8.0, 
7.0], [2.0, 3.0, 3.0], [8.0, 4.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPCL(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 2.0], [2.0, 5.0, 1.0], [5.0, 7.0, 4.0]], [[5.0, 7.0, 2.0], [6.0, 5.0, 7.0], [5.0, 2.0, 
5.0]], [[2.0, 2.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 3.0]]], [[[2.0, 1.0, 7.0], [4.0, 6.0, 6.0], [8.0, 7.0, 7.0]], [[4.0, 2.0, 
3.0], [3.0, 7.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 5.0], [2.0, 7.0, 7.0]]], [[[8.0, 7.0, 2.0], [8.0, 7.0, 
1.0], [7.0, 2.0, 5.0]], [[3.0, 2.0, 3.0], [4.0, 7.0, 6.0], [3.0, 3.0, 8.0]], [[3.0, 3.0, 3.0], [8.0, 3.0, 8.0], [8.0, 8.0, 
3.0]]], [[[8.0, 8.0, 1.0], [2.0, 7.0, 1.0], [4.0, 4.0, 7.0]], [[2.0, 5.0, 4.0], [4.0, 3.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 7.0, 
5.0], [1.0, 5.0, 8.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0], [6.0], [8.0]])+(1.-msk_val)*numpy.array([[1.0], [8.0], [4.0]])
      arg[2,1,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 2.0], [2.0, 5.0, 1.0], [5.0, 7.0, 4.0]], [[5.0, 7.0, 2.0], [6.0, 5.0, 7.0], [5.0, 
2.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 3.0]]], [[[2.0, 1.0, 7.0], [4.0, 6.0, 6.0], [8.0, 7.0, 7.0]], [[4.0, 
2.0, 3.0], [3.0, 7.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 5.0], [2.0, 7.0, 7.0]]], [[[8.0, 7.0, 2.0], [8.0, 
7.0, 1.0], [7.0, 2.0, 5.0]], [[3.0, 2.0, 4.0], [4.0, 7.0, 6.0], [3.0, 3.0, 8.0]], [[3.0, 3.0, 3.0], [8.0, 3.0, 8.0], [8.0, 8.0, 
3.0]]], [[[8.0, 8.0, 1.0], [2.0, 7.0, 1.0], [4.0, 4.0, 7.0]], [[2.0, 5.0, 4.0], [4.0, 3.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 7.0, 
5.0], [1.0, 5.0, 8.0], [3.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 2.0], [2.0, 5.0, 1.0], [5.0, 7.0, 4.0]], 
[[5.0, 7.0, 2.0], [6.0, 5.0, 7.0], [5.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 2.0, 4.0], [3.0, 1.0, 3.0]]], [[[2.0, 1.0, 7.0], 
[4.0, 6.0, 6.0], [8.0, 7.0, 7.0]], [[4.0, 2.0, 3.0], [3.0, 7.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 5.0], 
[2.0, 7.0, 7.0]]], [[[8.0, 7.0, 2.0], [8.0, 7.0, 1.0], [7.0, 2.0, 5.0]], [[3.0, 2.0, 1.0], [4.0, 7.0, 8.0], [3.0, 3.0, 4.0]], 
[[3.0, 3.0, 3.0], [8.0, 3.0, 8.0], [8.0, 8.0, 3.0]]], [[[8.0, 8.0, 1.0], [2.0, 7.0, 1.0], [4.0, 4.0, 7.0]], [[2.0, 5.0, 4.0], 
[4.0, 3.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 7.0, 5.0], [1.0, 5.0, 8.0], [3.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCUN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 1.0], [1.0, 6.0, 2.0], [5.0, 5.0, 5.0]], [[7.0, 5.0, 3.0], [5.0, 3.0, 1.0], [8.0, 3.0, 
8.0]], [[4.0, 6.0, 5.0], [3.0, 7.0, 5.0], [7.0, 6.0, 7.0]]], [[[5.0, 6.0, 7.0], [2.0, 5.0, 8.0], [4.0, 7.0, 3.0]], [[2.0, 8.0, 
4.0], [6.0, 4.0, 1.0], [2.0, 8.0, 2.0]], [[1.0, 1.0, 8.0], [6.0, 3.0, 3.0], [7.0, 2.0, 7.0]]], [[[6.0, 3.0, 8.0], [2.0, 3.0, 
3.0], [5.0, 8.0, 1.0]], [[3.0, 1.0, 6.0], [2.0, 2.0, 7.0], [7.0, 5.0, 1.0]], [[3.0, 4.0, 1.0], [5.0, 7.0, 6.0], [7.0, 8.0, 
1.0]]], [[[5.0, 1.0, 7.0], [5.0, 8.0, 4.0], [2.0, 6.0, 3.0]], [[5.0, 2.0, 8.0], [1.0, 4.0, 3.0], [7.0, 6.0, 4.0]], [[7.0, 8.0, 
5.0], [2.0, 6.0, 3.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[7.0, 2.0, 2.0], [8.0, 8.0, 8.0]], [[8.0, 3.0, 4.0], [6.0, 8.0, 
1.0]]], [[[5.0, 8.0, 8.0], [8.0, 2.0, 3.0]], [[8.0, 2.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 2.0, 7.0], [8.0, 3.0, 3.0]]]])
      arg[1:3,:,:2]=val
      ref=Data(numpy.array([[[[2.0, 7.0, 1.0], [1.0, 6.0, 2.0], [5.0, 5.0, 5.0]], [[7.0, 5.0, 3.0], [5.0, 3.0, 1.0], [8.0, 3.0, 
8.0]], [[4.0, 6.0, 5.0], [3.0, 7.0, 5.0], [7.0, 6.0, 7.0]]], [[[1.0, 6.0, 3.0], [1.0, 1.0, 5.0], [4.0, 7.0, 3.0]], [[7.0, 2.0, 
2.0], [8.0, 8.0, 8.0], [2.0, 8.0, 2.0]], [[8.0, 3.0, 4.0], [6.0, 8.0, 1.0], [7.0, 2.0, 7.0]]], [[[5.0, 8.0, 8.0], [8.0, 2.0, 
3.0], [5.0, 8.0, 1.0]], [[8.0, 2.0, 8.0], [3.0, 7.0, 3.0], [7.0, 5.0, 1.0]], [[4.0, 2.0, 7.0], [8.0, 3.0, 3.0], [7.0, 8.0, 
1.0]]], [[[5.0, 1.0, 7.0], [5.0, 8.0, 4.0], [2.0, 6.0, 3.0]], [[5.0, 2.0, 8.0], [1.0, 4.0, 3.0], [7.0, 6.0, 4.0]], [[7.0, 8.0, 
5.0], [2.0, 6.0, 3.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCUN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 2.0], [8.0, 2.0, 2.0], [1.0, 1.0, 4.0]], [[5.0, 2.0, 4.0], [3.0, 3.0, 5.0], [5.0, 1.0, 
2.0]], [[4.0, 7.0, 1.0], [5.0, 7.0, 8.0], [2.0, 4.0, 5.0]]], [[[7.0, 5.0, 3.0], [3.0, 1.0, 3.0], [5.0, 3.0, 6.0]], [[3.0, 7.0, 
3.0], [3.0, 8.0, 3.0], [8.0, 2.0, 5.0]], [[8.0, 8.0, 3.0], [3.0, 8.0, 6.0], [3.0, 1.0, 4.0]]], [[[6.0, 2.0, 2.0], [7.0, 8.0, 
3.0], [4.0, 8.0, 3.0]], [[7.0, 7.0, 5.0], [1.0, 5.0, 6.0], [1.0, 3.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 6.0, 8.0], [4.0, 7.0, 
4.0]]], [[[5.0, 1.0, 3.0], [2.0, 1.0, 8.0], [4.0, 5.0, 2.0]], [[3.0, 1.0, 5.0], [7.0, 1.0, 8.0], [6.0, 6.0, 6.0]], [[5.0, 2.0, 
2.0], [3.0, 1.0, 3.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 5.0, 7.0], [1.0, 6.0, 5.0]], [[4.0, 3.0, 8.0], [4.0, 7.0, 7.0]], [[2.0, 3.0, 5.0], [8.0, 
4.0, 8.0]]], [[[8.0, 4.0, 7.0], [5.0, 8.0, 7.0]], [[1.0, 3.0, 3.0], [1.0, 7.0, 3.0]], [[8.0, 2.0, 4.0], [8.0, 7.0, 
8.0]]]]),self.functionspace)
      arg[1:3,:,:2]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 2.0], [8.0, 2.0, 2.0], [1.0, 1.0, 4.0]], [[5.0, 2.0, 4.0], [3.0, 3.0, 5.0], [5.0, 1.0, 
2.0]], [[4.0, 7.0, 1.0], [5.0, 7.0, 8.0], [2.0, 4.0, 5.0]]], [[[2.0, 5.0, 7.0], [1.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[4.0, 3.0, 
8.0], [4.0, 7.0, 7.0], [8.0, 2.0, 5.0]], [[2.0, 3.0, 5.0], [8.0, 4.0, 8.0], [3.0, 1.0, 4.0]]], [[[8.0, 4.0, 7.0], [5.0, 8.0, 
7.0], [4.0, 8.0, 3.0]], [[1.0, 3.0, 3.0], [1.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[8.0, 2.0, 4.0], [8.0, 7.0, 8.0], [4.0, 7.0, 
4.0]]], [[[5.0, 1.0, 3.0], [2.0, 1.0, 8.0], [4.0, 5.0, 2.0]], [[3.0, 1.0, 5.0], [7.0, 1.0, 8.0], [6.0, 6.0, 6.0]], [[5.0, 2.0, 
2.0], [3.0, 1.0, 3.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCUN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 1.0], [2.0, 6.0, 6.0], [3.0, 3.0, 6.0]], [[2.0, 2.0, 8.0], [7.0, 4.0, 6.0], [6.0, 2.0, 
3.0]], [[7.0, 3.0, 3.0], [6.0, 3.0, 4.0], [6.0, 3.0, 4.0]]], [[[2.0, 3.0, 1.0], [3.0, 1.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 1.0, 
5.0], [2.0, 5.0, 4.0], [4.0, 5.0, 2.0]], [[8.0, 3.0, 4.0], [2.0, 8.0, 1.0], [3.0, 1.0, 8.0]]], [[[4.0, 7.0, 1.0], [6.0, 5.0, 
2.0], [1.0, 3.0, 8.0]], [[6.0, 2.0, 6.0], [5.0, 8.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 5.0, 6.0], [6.0, 3.0, 5.0], [7.0, 7.0, 
8.0]]], [[[2.0, 3.0, 8.0], [4.0, 8.0, 4.0], [1.0, 1.0, 1.0]], [[1.0, 7.0, 2.0], [4.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 4.0, 
8.0], [2.0, 7.0, 7.0], [5.0, 7.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 4.0, 6.0], [5.0, 3.0, 1.0]], [[6.0, 2.0, 7.0], [3.0, 1.0, 4.0]], [[7.0, 5.0, 5.0], [6.0, 
1.0, 8.0]]], [[[8.0, 4.0, 4.0], [7.0, 1.0, 7.0]], [[7.0, 5.0, 2.0], [8.0, 7.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 2.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 8.0, 3.0], [8.0, 3.0, 8.0]], [[1.0, 6.0, 2.0], [8.0, 7.0, 2.0]], [[6.0, 7.0, 7.0], 
[1.0, 5.0, 8.0]]], [[[8.0, 4.0, 2.0], [3.0, 8.0, 4.0]], [[3.0, 2.0, 3.0], [7.0, 4.0, 8.0]], [[3.0, 7.0, 1.0], [2.0, 8.0, 
8.0]]]])
      arg[1:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 1.0], [2.0, 6.0, 6.0], [3.0, 3.0, 6.0]], [[2.0, 2.0, 8.0], [7.0, 4.0, 6.0], [6.0, 
2.0, 3.0]], [[7.0, 3.0, 3.0], [6.0, 3.0, 4.0], [6.0, 3.0, 4.0]]], [[[6.0, 4.0, 6.0], [5.0, 3.0, 1.0], [5.0, 5.0, 4.0]], [[6.0, 
2.0, 7.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]], [[7.0, 5.0, 5.0], [6.0, 1.0, 8.0], [3.0, 1.0, 8.0]]], [[[8.0, 4.0, 4.0], [7.0, 
1.0, 7.0], [1.0, 3.0, 8.0]], [[7.0, 5.0, 2.0], [8.0, 7.0, 5.0], [5.0, 1.0, 3.0]], [[6.0, 3.0, 7.0], [4.0, 2.0, 7.0], [7.0, 7.0, 
8.0]]], [[[2.0, 3.0, 8.0], [4.0, 8.0, 4.0], [1.0, 1.0, 1.0]], [[1.0, 7.0, 2.0], [4.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 4.0, 
8.0], [2.0, 7.0, 7.0], [5.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 1.0], [2.0, 6.0, 6.0], [3.0, 3.0, 6.0]], 
[[2.0, 2.0, 8.0], [7.0, 4.0, 6.0], [6.0, 2.0, 3.0]], [[7.0, 3.0, 3.0], [6.0, 3.0, 4.0], [6.0, 3.0, 4.0]]], [[[5.0, 8.0, 3.0], 
[8.0, 3.0, 8.0], [5.0, 5.0, 4.0]], [[1.0, 6.0, 2.0], [8.0, 7.0, 2.0], [4.0, 5.0, 2.0]], [[6.0, 7.0, 7.0], [1.0, 5.0, 8.0], 
[3.0, 1.0, 8.0]]], [[[8.0, 4.0, 2.0], [3.0, 8.0, 4.0], [1.0, 3.0, 8.0]], [[3.0, 2.0, 3.0], [7.0, 4.0, 8.0], [5.0, 1.0, 3.0]], 
[[3.0, 7.0, 1.0], [2.0, 8.0, 8.0], [7.0, 7.0, 8.0]]], [[[2.0, 3.0, 8.0], [4.0, 8.0, 4.0], [1.0, 1.0, 1.0]], [[1.0, 7.0, 2.0], 
[4.0, 6.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 7.0, 7.0], [5.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPLR(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 4.0], [8.0, 5.0, 7.0], [7.0, 6.0, 5.0]], [[3.0, 1.0, 5.0], [6.0, 7.0, 8.0], [2.0, 1.0, 
8.0]], [[4.0, 3.0, 4.0], [7.0, 1.0, 7.0], [5.0, 8.0, 6.0]]], [[[7.0, 7.0, 4.0], [5.0, 8.0, 5.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 
3.0], [6.0, 4.0, 1.0], [8.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 2.0], [7.0, 2.0, 6.0]]], [[[5.0, 6.0, 4.0], [6.0, 7.0, 
6.0], [8.0, 8.0, 6.0]], [[1.0, 2.0, 8.0], [8.0, 2.0, 6.0], [8.0, 5.0, 8.0]], [[7.0, 2.0, 7.0], [5.0, 2.0, 5.0], [8.0, 5.0, 
7.0]]], [[[2.0, 3.0, 5.0], [7.0, 4.0, 7.0], [4.0, 5.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 8.0, 4.0], [8.0, 5.0, 5.0]], [[6.0, 1.0, 
1.0], [6.0, 6.0, 3.0], [6.0, 4.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[1.0]], [[7.0]]])
      arg[1:3,1,2:,1:2]=val
      ref=Data(numpy.array([[[[7.0, 2.0, 4.0], [8.0, 5.0, 7.0], [7.0, 6.0, 5.0]], [[3.0, 1.0, 5.0], [6.0, 7.0, 8.0], [2.0, 1.0, 
8.0]], [[4.0, 3.0, 4.0], [7.0, 1.0, 7.0], [5.0, 8.0, 6.0]]], [[[7.0, 7.0, 4.0], [5.0, 8.0, 5.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 
3.0], [6.0, 4.0, 1.0], [8.0, 1.0, 4.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 2.0], [7.0, 2.0, 6.0]]], [[[5.0, 6.0, 4.0], [6.0, 7.0, 
6.0], [8.0, 8.0, 6.0]], [[1.0, 2.0, 8.0], [8.0, 2.0, 6.0], [8.0, 7.0, 8.0]], [[7.0, 2.0, 7.0], [5.0, 2.0, 5.0], [8.0, 5.0, 
7.0]]], [[[2.0, 3.0, 5.0], [7.0, 4.0, 7.0], [4.0, 5.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 8.0, 4.0], [8.0, 5.0, 5.0]], [[6.0, 1.0, 
1.0], [6.0, 6.0, 3.0], [6.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPLR(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 5.0], [8.0, 5.0, 6.0], [6.0, 3.0, 2.0]], [[4.0, 1.0, 6.0], [7.0, 6.0, 7.0], [2.0, 7.0, 
2.0]], [[8.0, 3.0, 8.0], [8.0, 3.0, 8.0], [3.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [4.0, 8.0, 6.0], [5.0, 5.0, 6.0]], [[4.0, 2.0, 
2.0], [2.0, 5.0, 5.0], [1.0, 1.0, 1.0]], [[3.0, 3.0, 3.0], [3.0, 8.0, 2.0], [2.0, 4.0, 4.0]]], [[[3.0, 3.0, 6.0], [2.0, 6.0, 
4.0], [6.0, 2.0, 4.0]], [[5.0, 8.0, 3.0], [4.0, 7.0, 5.0], [8.0, 6.0, 2.0]], [[4.0, 4.0, 2.0], [2.0, 3.0, 4.0], [3.0, 6.0, 
8.0]]], [[[8.0, 8.0, 4.0], [7.0, 2.0, 4.0], [3.0, 1.0, 8.0]], [[5.0, 2.0, 3.0], [4.0, 4.0, 8.0], [2.0, 4.0, 4.0]], [[1.0, 8.0, 
8.0], [5.0, 6.0, 4.0], [4.0, 6.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0]], [[8.0]]]),self.functionspace)
      arg[1:3,1,2:,1:2]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 5.0], [8.0, 5.0, 6.0], [6.0, 3.0, 2.0]], [[4.0, 1.0, 6.0], [7.0, 6.0, 7.0], [2.0, 7.0, 
2.0]], [[8.0, 3.0, 8.0], [8.0, 3.0, 8.0], [3.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [4.0, 8.0, 6.0], [5.0, 5.0, 6.0]], [[4.0, 2.0, 
2.0], [2.0, 5.0, 5.0], [1.0, 3.0, 1.0]], [[3.0, 3.0, 3.0], [3.0, 8.0, 2.0], [2.0, 4.0, 4.0]]], [[[3.0, 3.0, 6.0], [2.0, 6.0, 
4.0], [6.0, 2.0, 4.0]], [[5.0, 8.0, 3.0], [4.0, 7.0, 5.0], [8.0, 8.0, 2.0]], [[4.0, 4.0, 2.0], [2.0, 3.0, 4.0], [3.0, 6.0, 
8.0]]], [[[8.0, 8.0, 4.0], [7.0, 2.0, 4.0], [3.0, 1.0, 8.0]], [[5.0, 2.0, 3.0], [4.0, 4.0, 8.0], [2.0, 4.0, 4.0]], [[1.0, 8.0, 
8.0], [5.0, 6.0, 4.0], [4.0, 6.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPLR(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [4.0, 8.0, 4.0], [2.0, 6.0, 3.0]], [[3.0, 2.0, 2.0], [1.0, 7.0, 4.0], [8.0, 7.0, 
7.0]], [[3.0, 2.0, 4.0], [2.0, 8.0, 6.0], [3.0, 8.0, 2.0]]], [[[6.0, 8.0, 3.0], [5.0, 7.0, 1.0], [6.0, 6.0, 3.0]], [[3.0, 5.0, 
2.0], [5.0, 1.0, 5.0], [4.0, 5.0, 5.0]], [[4.0, 2.0, 2.0], [1.0, 8.0, 1.0], [5.0, 3.0, 3.0]]], [[[2.0, 7.0, 3.0], [7.0, 8.0, 
1.0], [7.0, 4.0, 6.0]], [[8.0, 1.0, 4.0], [5.0, 7.0, 4.0], [7.0, 3.0, 8.0]], [[7.0, 7.0, 7.0], [6.0, 8.0, 4.0], [2.0, 1.0, 
6.0]]], [[[7.0, 7.0, 8.0], [3.0, 7.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], [4.0, 3.0, 8.0], [4.0, 6.0, 2.0]], [[4.0, 2.0, 
6.0], [7.0, 4.0, 8.0], [2.0, 6.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0]], [[6.0]]])+(1.-msk_val)*numpy.array([[[4.0]], [[1.0]]])
      arg[1:3,1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 7.0], [4.0, 8.0, 4.0], [2.0, 6.0, 3.0]], [[3.0, 2.0, 2.0], [1.0, 7.0, 4.0], [8.0, 
7.0, 7.0]], [[3.0, 2.0, 4.0], [2.0, 8.0, 6.0], [3.0, 8.0, 2.0]]], [[[6.0, 8.0, 3.0], [5.0, 7.0, 1.0], [6.0, 6.0, 3.0]], [[3.0, 
5.0, 2.0], [5.0, 1.0, 5.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 2.0], [1.0, 8.0, 1.0], [5.0, 3.0, 3.0]]], [[[2.0, 7.0, 3.0], [7.0, 
8.0, 1.0], [7.0, 4.0, 6.0]], [[8.0, 1.0, 4.0], [5.0, 7.0, 4.0], [7.0, 6.0, 8.0]], [[7.0, 7.0, 7.0], [6.0, 8.0, 4.0], [2.0, 1.0, 
6.0]]], [[[7.0, 7.0, 8.0], [3.0, 7.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], [4.0, 3.0, 8.0], [4.0, 6.0, 2.0]], [[4.0, 2.0, 
6.0], [7.0, 4.0, 8.0], [2.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 7.0], [4.0, 8.0, 4.0], [2.0, 6.0, 3.0]], 
[[3.0, 2.0, 2.0], [1.0, 7.0, 4.0], [8.0, 7.0, 7.0]], [[3.0, 2.0, 4.0], [2.0, 8.0, 6.0], [3.0, 8.0, 2.0]]], [[[6.0, 8.0, 3.0], 
[5.0, 7.0, 1.0], [6.0, 6.0, 3.0]], [[3.0, 5.0, 2.0], [5.0, 1.0, 5.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 2.0], [1.0, 8.0, 1.0], 
[5.0, 3.0, 3.0]]], [[[2.0, 7.0, 3.0], [7.0, 8.0, 1.0], [7.0, 4.0, 6.0]], [[8.0, 1.0, 4.0], [5.0, 7.0, 4.0], [7.0, 1.0, 8.0]], 
[[7.0, 7.0, 7.0], [6.0, 8.0, 4.0], [2.0, 1.0, 6.0]]], [[[7.0, 7.0, 8.0], [3.0, 7.0, 2.0], [8.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], 
[4.0, 3.0, 8.0], [4.0, 6.0, 2.0]], [[4.0, 2.0, 6.0], [7.0, 4.0, 8.0], [2.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLUP(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 1.0], [7.0, 6.0, 7.0], [8.0, 7.0, 8.0]], [[1.0, 3.0, 3.0], [5.0, 6.0, 6.0], [2.0, 4.0, 
3.0]], [[2.0, 8.0, 6.0], [3.0, 2.0, 6.0], [1.0, 4.0, 6.0]]], [[[5.0, 5.0, 1.0], [5.0, 6.0, 2.0], [1.0, 6.0, 2.0]], [[7.0, 2.0, 
4.0], [4.0, 4.0, 2.0], [5.0, 5.0, 5.0]], [[7.0, 1.0, 2.0], [8.0, 3.0, 7.0], [8.0, 3.0, 2.0]]], [[[5.0, 3.0, 4.0], [6.0, 1.0, 
5.0], [5.0, 5.0, 1.0]], [[2.0, 1.0, 6.0], [5.0, 7.0, 6.0], [8.0, 1.0, 7.0]], [[8.0, 8.0, 5.0], [8.0, 5.0, 2.0], [8.0, 2.0, 
8.0]]], [[[7.0, 3.0, 1.0], [6.0, 8.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 5.0, 7.0], [3.0, 1.0, 5.0], [1.0, 1.0, 2.0]], [[8.0, 8.0, 
4.0], [4.0, 6.0, 4.0], [7.0, 8.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[2.0, 4.0]], [[7.0, 6.0]]])
      arg[2:,2:,:2,1]=val
      ref=Data(numpy.array([[[[1.0, 2.0, 1.0], [7.0, 6.0, 7.0], [8.0, 7.0, 8.0]], [[1.0, 3.0, 3.0], [5.0, 6.0, 6.0], [2.0, 4.0, 
3.0]], [[2.0, 8.0, 6.0], [3.0, 2.0, 6.0], [1.0, 4.0, 6.0]]], [[[5.0, 5.0, 1.0], [5.0, 6.0, 2.0], [1.0, 6.0, 2.0]], [[7.0, 2.0, 
4.0], [4.0, 4.0, 2.0], [5.0, 5.0, 5.0]], [[7.0, 1.0, 2.0], [8.0, 3.0, 7.0], [8.0, 3.0, 2.0]]], [[[5.0, 3.0, 4.0], [6.0, 1.0, 
5.0], [5.0, 5.0, 1.0]], [[2.0, 1.0, 6.0], [5.0, 7.0, 6.0], [8.0, 1.0, 7.0]], [[8.0, 2.0, 5.0], [8.0, 4.0, 2.0], [8.0, 2.0, 
8.0]]], [[[7.0, 3.0, 1.0], [6.0, 8.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 5.0, 7.0], [3.0, 1.0, 5.0], [1.0, 1.0, 2.0]], [[8.0, 7.0, 
4.0], [4.0, 6.0, 4.0], [7.0, 8.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLUP(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 3.0], [1.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 4.0], [4.0, 6.0, 
1.0]], [[7.0, 7.0, 1.0], [7.0, 8.0, 3.0], [5.0, 3.0, 5.0]]], [[[1.0, 2.0, 3.0], [3.0, 3.0, 7.0], [8.0, 7.0, 6.0]], [[5.0, 3.0, 
7.0], [8.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[3.0, 2.0, 4.0], [8.0, 7.0, 3.0], [5.0, 1.0, 2.0]]], [[[4.0, 1.0, 2.0], [8.0, 7.0, 
1.0], [8.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [2.0, 1.0, 2.0], [2.0, 7.0, 7.0]], [[3.0, 2.0, 2.0], [1.0, 5.0, 4.0], [7.0, 2.0, 
4.0]]], [[[6.0, 1.0, 1.0], [6.0, 7.0, 6.0], [3.0, 3.0, 4.0]], [[4.0, 3.0, 7.0], [2.0, 2.0, 5.0], [2.0, 8.0, 3.0]], [[4.0, 5.0, 
2.0], [1.0, 6.0, 8.0], [1.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0]], [[8.0, 4.0]]]),self.functionspace)
      arg[2:,2:,:2,1]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 3.0], [1.0, 6.0, 3.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 4.0], [4.0, 6.0, 
1.0]], [[7.0, 7.0, 1.0], [7.0, 8.0, 3.0], [5.0, 3.0, 5.0]]], [[[1.0, 2.0, 3.0], [3.0, 3.0, 7.0], [8.0, 7.0, 6.0]], [[5.0, 3.0, 
7.0], [8.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[3.0, 2.0, 4.0], [8.0, 7.0, 3.0], [5.0, 1.0, 2.0]]], [[[4.0, 1.0, 2.0], [8.0, 7.0, 
1.0], [8.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [2.0, 1.0, 2.0], [2.0, 7.0, 7.0]], [[3.0, 4.0, 2.0], [1.0, 7.0, 4.0], [7.0, 2.0, 
4.0]]], [[[6.0, 1.0, 1.0], [6.0, 7.0, 6.0], [3.0, 3.0, 4.0]], [[4.0, 3.0, 7.0], [2.0, 2.0, 5.0], [2.0, 8.0, 3.0]], [[4.0, 8.0, 
2.0], [1.0, 4.0, 8.0], [1.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLUP(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 6.0], [5.0, 8.0, 7.0], [4.0, 1.0, 7.0]], [[1.0, 2.0, 2.0], [4.0, 3.0, 8.0], [3.0, 7.0, 
7.0]], [[2.0, 6.0, 4.0], [4.0, 8.0, 4.0], [2.0, 5.0, 4.0]]], [[[1.0, 6.0, 1.0], [1.0, 6.0, 4.0], [1.0, 3.0, 3.0]], [[7.0, 7.0, 
5.0], [8.0, 4.0, 5.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 4.0], [5.0, 6.0, 2.0], [7.0, 6.0, 5.0]]], [[[3.0, 1.0, 5.0], [1.0, 1.0, 
6.0], [7.0, 6.0, 8.0]], [[3.0, 6.0, 3.0], [5.0, 1.0, 8.0], [6.0, 4.0, 3.0]], [[8.0, 7.0, 8.0], [7.0, 5.0, 4.0], [4.0, 4.0, 
7.0]]], [[[4.0, 7.0, 4.0], [5.0, 2.0, 4.0], [5.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [8.0, 1.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 6.0, 
8.0], [4.0, 3.0, 4.0], [1.0, 2.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 1.0]], [[2.0, 8.0]]])+(1.-msk_val)*numpy.array([[[6.0, 5.0]], [[6.0, 7.0]]])
      arg[2:,2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [5.0, 8.0, 7.0], [4.0, 1.0, 7.0]], [[1.0, 2.0, 2.0], [4.0, 3.0, 8.0], [3.0, 
7.0, 7.0]], [[2.0, 6.0, 4.0], [4.0, 8.0, 4.0], [2.0, 5.0, 4.0]]], [[[1.0, 6.0, 1.0], [1.0, 6.0, 4.0], [1.0, 3.0, 3.0]], [[7.0, 
7.0, 5.0], [8.0, 4.0, 5.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 4.0], [5.0, 6.0, 2.0], [7.0, 6.0, 5.0]]], [[[3.0, 1.0, 5.0], [1.0, 
1.0, 6.0], [7.0, 6.0, 8.0]], [[3.0, 6.0, 3.0], [5.0, 1.0, 8.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 8.0], [7.0, 1.0, 4.0], [4.0, 4.0, 
7.0]]], [[[4.0, 7.0, 4.0], [5.0, 2.0, 4.0], [5.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [8.0, 1.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 
8.0], [4.0, 8.0, 4.0], [1.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 6.0], [5.0, 8.0, 7.0], [4.0, 1.0, 7.0]], 
[[1.0, 2.0, 2.0], [4.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[2.0, 6.0, 4.0], [4.0, 8.0, 4.0], [2.0, 5.0, 4.0]]], [[[1.0, 6.0, 1.0], 
[1.0, 6.0, 4.0], [1.0, 3.0, 3.0]], [[7.0, 7.0, 5.0], [8.0, 4.0, 5.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 4.0], [5.0, 6.0, 2.0], 
[7.0, 6.0, 5.0]]], [[[3.0, 1.0, 5.0], [1.0, 1.0, 6.0], [7.0, 6.0, 8.0]], [[3.0, 6.0, 3.0], [5.0, 1.0, 8.0], [6.0, 4.0, 3.0]], 
[[8.0, 6.0, 8.0], [7.0, 5.0, 4.0], [4.0, 4.0, 7.0]]], [[[4.0, 7.0, 4.0], [5.0, 2.0, 4.0], [5.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], 
[8.0, 1.0, 1.0], [4.0, 7.0, 1.0]], [[3.0, 6.0, 8.0], [4.0, 7.0, 4.0], [1.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRLU(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 8.0], [4.0, 1.0, 8.0], [1.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 5.0, 2.0], [1.0, 2.0, 
1.0]], [[6.0, 6.0, 5.0], [7.0, 7.0, 1.0], [3.0, 2.0, 5.0]]], [[[2.0, 7.0, 7.0], [2.0, 6.0, 5.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 
2.0], [1.0, 3.0, 1.0], [8.0, 8.0, 4.0]], [[3.0, 3.0, 3.0], [5.0, 7.0, 5.0], [1.0, 1.0, 2.0]]], [[[4.0, 3.0, 4.0], [7.0, 4.0, 
5.0], [5.0, 4.0, 5.0]], [[2.0, 3.0, 1.0], [6.0, 5.0, 5.0], [5.0, 7.0, 6.0]], [[2.0, 7.0, 8.0], [3.0, 6.0, 6.0], [1.0, 4.0, 
6.0]]], [[[1.0, 4.0, 2.0], [4.0, 4.0, 8.0], [8.0, 2.0, 3.0]], [[7.0, 6.0, 7.0], [2.0, 8.0, 5.0], [2.0, 6.0, 3.0]], [[8.0, 2.0, 
3.0], [3.0, 3.0, 5.0], [6.0, 4.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 3.0]]], [[[2.0, 2.0]]]])
      arg[2:,1:2,2:,:2]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 8.0], [4.0, 1.0, 8.0], [1.0, 5.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 5.0, 2.0], [1.0, 2.0, 
1.0]], [[6.0, 6.0, 5.0], [7.0, 7.0, 1.0], [3.0, 2.0, 5.0]]], [[[2.0, 7.0, 7.0], [2.0, 6.0, 5.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 
2.0], [1.0, 3.0, 1.0], [8.0, 8.0, 4.0]], [[3.0, 3.0, 3.0], [5.0, 7.0, 5.0], [1.0, 1.0, 2.0]]], [[[4.0, 3.0, 4.0], [7.0, 4.0, 
5.0], [5.0, 4.0, 5.0]], [[2.0, 3.0, 1.0], [6.0, 5.0, 5.0], [8.0, 3.0, 6.0]], [[2.0, 7.0, 8.0], [3.0, 6.0, 6.0], [1.0, 4.0, 
6.0]]], [[[1.0, 4.0, 2.0], [4.0, 4.0, 8.0], [8.0, 2.0, 3.0]], [[7.0, 6.0, 7.0], [2.0, 8.0, 5.0], [2.0, 2.0, 3.0]], [[8.0, 2.0, 
3.0], [3.0, 3.0, 5.0], [6.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRLU(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 2.0], [6.0, 5.0, 5.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 2.0], [6.0, 5.0, 3.0], [8.0, 8.0, 
6.0]], [[2.0, 1.0, 2.0], [3.0, 2.0, 6.0], [7.0, 5.0, 1.0]]], [[[5.0, 2.0, 6.0], [2.0, 2.0, 8.0], [7.0, 6.0, 6.0]], [[4.0, 7.0, 
7.0], [2.0, 1.0, 2.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 1.0], [7.0, 7.0, 1.0], [6.0, 2.0, 2.0]]], [[[3.0, 8.0, 1.0], [5.0, 6.0, 
4.0], [3.0, 3.0, 7.0]], [[5.0, 4.0, 7.0], [4.0, 3.0, 6.0], [8.0, 2.0, 1.0]], [[4.0, 6.0, 8.0], [7.0, 2.0, 3.0], [5.0, 7.0, 
6.0]]], [[[2.0, 5.0, 8.0], [5.0, 4.0, 8.0], [5.0, 6.0, 2.0]], [[5.0, 5.0, 6.0], [8.0, 6.0, 6.0], [7.0, 8.0, 5.0]], [[6.0, 6.0, 
1.0], [8.0, 6.0, 7.0], [3.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 6.0]]], [[[2.0, 3.0]]]]),self.functionspace)
      arg[2:,1:2,2:,:2]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 2.0], [6.0, 5.0, 5.0], [7.0, 2.0, 6.0]], [[5.0, 7.0, 2.0], [6.0, 5.0, 3.0], [8.0, 8.0, 
6.0]], [[2.0, 1.0, 2.0], [3.0, 2.0, 6.0], [7.0, 5.0, 1.0]]], [[[5.0, 2.0, 6.0], [2.0, 2.0, 8.0], [7.0, 6.0, 6.0]], [[4.0, 7.0, 
7.0], [2.0, 1.0, 2.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 1.0], [7.0, 7.0, 1.0], [6.0, 2.0, 2.0]]], [[[3.0, 8.0, 1.0], [5.0, 6.0, 
4.0], [3.0, 3.0, 7.0]], [[5.0, 4.0, 7.0], [4.0, 3.0, 6.0], [2.0, 6.0, 1.0]], [[4.0, 6.0, 8.0], [7.0, 2.0, 3.0], [5.0, 7.0, 
6.0]]], [[[2.0, 5.0, 8.0], [5.0, 4.0, 8.0], [5.0, 6.0, 2.0]], [[5.0, 5.0, 6.0], [8.0, 6.0, 6.0], [2.0, 3.0, 5.0]], [[6.0, 6.0, 
1.0], [8.0, 6.0, 7.0], [3.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRLU(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 4.0], [7.0, 4.0, 4.0], [6.0, 6.0, 6.0]], [[2.0, 8.0, 8.0], [5.0, 1.0, 1.0], [5.0, 2.0, 
1.0]], [[5.0, 3.0, 3.0], [4.0, 7.0, 8.0], [3.0, 6.0, 8.0]]], [[[6.0, 2.0, 7.0], [2.0, 6.0, 1.0], [3.0, 7.0, 3.0]], [[6.0, 5.0, 
6.0], [4.0, 2.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 3.0, 1.0], [3.0, 2.0, 3.0], [8.0, 3.0, 3.0]]], [[[8.0, 5.0, 6.0], [3.0, 5.0, 
4.0], [5.0, 2.0, 1.0]], [[5.0, 7.0, 2.0], [8.0, 2.0, 7.0], [8.0, 5.0, 2.0]], [[2.0, 7.0, 3.0], [5.0, 4.0, 7.0], [3.0, 8.0, 
1.0]]], [[[5.0, 1.0, 6.0], [6.0, 2.0, 6.0], [6.0, 5.0, 3.0]], [[3.0, 6.0, 6.0], [4.0, 4.0, 6.0], [1.0, 4.0, 3.0]], [[5.0, 3.0, 
6.0], [6.0, 1.0, 1.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 2.0]]], [[[2.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0]]], [[[6.0, 1.0]]]])
      arg[2:,1:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 4.0], [7.0, 4.0, 4.0], [6.0, 6.0, 6.0]], [[2.0, 8.0, 8.0], [5.0, 1.0, 1.0], [5.0, 
2.0, 1.0]], [[5.0, 3.0, 3.0], [4.0, 7.0, 8.0], [3.0, 6.0, 8.0]]], [[[6.0, 2.0, 7.0], [2.0, 6.0, 1.0], [3.0, 7.0, 3.0]], [[6.0, 
5.0, 6.0], [4.0, 2.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 3.0, 1.0], [3.0, 2.0, 3.0], [8.0, 3.0, 3.0]]], [[[8.0, 5.0, 6.0], [3.0, 
5.0, 4.0], [5.0, 2.0, 1.0]], [[5.0, 7.0, 2.0], [8.0, 2.0, 7.0], [2.0, 2.0, 2.0]], [[2.0, 7.0, 3.0], [5.0, 4.0, 7.0], [3.0, 8.0, 
1.0]]], [[[5.0, 1.0, 6.0], [6.0, 2.0, 6.0], [6.0, 5.0, 3.0]], [[3.0, 6.0, 6.0], [4.0, 4.0, 6.0], [2.0, 4.0, 3.0]], [[5.0, 3.0, 
6.0], [6.0, 1.0, 1.0], [3.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 4.0], [7.0, 4.0, 4.0], [6.0, 6.0, 6.0]], 
[[2.0, 8.0, 8.0], [5.0, 1.0, 1.0], [5.0, 2.0, 1.0]], [[5.0, 3.0, 3.0], [4.0, 7.0, 8.0], [3.0, 6.0, 8.0]]], [[[6.0, 2.0, 7.0], 
[2.0, 6.0, 1.0], [3.0, 7.0, 3.0]], [[6.0, 5.0, 6.0], [4.0, 2.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 3.0, 1.0], [3.0, 2.0, 3.0], 
[8.0, 3.0, 3.0]]], [[[8.0, 5.0, 6.0], [3.0, 5.0, 4.0], [5.0, 2.0, 1.0]], [[5.0, 7.0, 2.0], [8.0, 2.0, 7.0], [2.0, 5.0, 2.0]], 
[[2.0, 7.0, 3.0], [5.0, 4.0, 7.0], [3.0, 8.0, 1.0]]], [[[5.0, 1.0, 6.0], [6.0, 2.0, 6.0], [6.0, 5.0, 3.0]], [[3.0, 6.0, 6.0], 
[4.0, 4.0, 6.0], [6.0, 1.0, 3.0]], [[5.0, 3.0, 6.0], [6.0, 1.0, 1.0], [3.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPPR(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 8.0], [5.0, 2.0, 6.0], [2.0, 5.0, 3.0]], [[7.0, 1.0, 8.0], [1.0, 4.0, 4.0], [1.0, 5.0, 
8.0]], [[1.0, 6.0, 4.0], [1.0, 6.0, 6.0], [6.0, 8.0, 4.0]]], [[[3.0, 1.0, 2.0], [4.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[5.0, 1.0, 
6.0], [3.0, 6.0, 1.0], [7.0, 3.0, 5.0]], [[7.0, 5.0, 3.0], [8.0, 7.0, 3.0], [5.0, 5.0, 3.0]]], [[[4.0, 6.0, 4.0], [3.0, 5.0, 
1.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 4.0], [6.0, 5.0, 4.0], [8.0, 1.0, 1.0]], [[7.0, 8.0, 5.0], [4.0, 1.0, 7.0], [5.0, 1.0, 
3.0]]], [[[5.0, 5.0, 1.0], [2.0, 4.0, 6.0], [2.0, 1.0, 4.0]], [[4.0, 3.0, 2.0], [5.0, 8.0, 6.0], [6.0, 7.0, 5.0]], [[2.0, 8.0, 
1.0], [6.0, 4.0, 6.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[7.0], [1.0]])
      arg[2:,1,1,1:2]=val
      ref=Data(numpy.array([[[[8.0, 6.0, 8.0], [5.0, 2.0, 6.0], [2.0, 5.0, 3.0]], [[7.0, 1.0, 8.0], [1.0, 4.0, 4.0], [1.0, 5.0, 
8.0]], [[1.0, 6.0, 4.0], [1.0, 6.0, 6.0], [6.0, 8.0, 4.0]]], [[[3.0, 1.0, 2.0], [4.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[5.0, 1.0, 
6.0], [3.0, 6.0, 1.0], [7.0, 3.0, 5.0]], [[7.0, 5.0, 3.0], [8.0, 7.0, 3.0], [5.0, 5.0, 3.0]]], [[[4.0, 6.0, 4.0], [3.0, 5.0, 
1.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 4.0], [6.0, 7.0, 4.0], [8.0, 1.0, 1.0]], [[7.0, 8.0, 5.0], [4.0, 1.0, 7.0], [5.0, 1.0, 
3.0]]], [[[5.0, 5.0, 1.0], [2.0, 4.0, 6.0], [2.0, 1.0, 4.0]], [[4.0, 3.0, 2.0], [5.0, 1.0, 6.0], [6.0, 7.0, 5.0]], [[2.0, 8.0, 
1.0], [6.0, 4.0, 6.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPPR(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 2.0], [5.0, 1.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 3.0, 5.0], [4.0, 7.0, 7.0], [5.0, 3.0, 
2.0]], [[7.0, 3.0, 6.0], [5.0, 8.0, 6.0], [6.0, 6.0, 4.0]]], [[[1.0, 6.0, 1.0], [1.0, 7.0, 5.0], [6.0, 4.0, 1.0]], [[2.0, 1.0, 
4.0], [8.0, 3.0, 7.0], [2.0, 2.0, 6.0]], [[4.0, 7.0, 6.0], [3.0, 1.0, 7.0], [5.0, 5.0, 2.0]]], [[[7.0, 4.0, 2.0], [5.0, 3.0, 
7.0], [3.0, 6.0, 4.0]], [[1.0, 5.0, 2.0], [4.0, 2.0, 5.0], [3.0, 3.0, 7.0]], [[1.0, 8.0, 8.0], [2.0, 7.0, 4.0], [5.0, 1.0, 
2.0]]], [[[6.0, 1.0, 5.0], [7.0, 2.0, 1.0], [1.0, 8.0, 4.0]], [[8.0, 6.0, 1.0], [5.0, 2.0, 3.0], [3.0, 1.0, 2.0]], [[5.0, 4.0, 
4.0], [8.0, 1.0, 2.0], [5.0, 2.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0], [5.0]]),self.functionspace)
      arg[2:,1,1,1:2]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 2.0], [5.0, 1.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 3.0, 5.0], [4.0, 7.0, 7.0], [5.0, 3.0, 
2.0]], [[7.0, 3.0, 6.0], [5.0, 8.0, 6.0], [6.0, 6.0, 4.0]]], [[[1.0, 6.0, 1.0], [1.0, 7.0, 5.0], [6.0, 4.0, 1.0]], [[2.0, 1.0, 
4.0], [8.0, 3.0, 7.0], [2.0, 2.0, 6.0]], [[4.0, 7.0, 6.0], [3.0, 1.0, 7.0], [5.0, 5.0, 2.0]]], [[[7.0, 4.0, 2.0], [5.0, 3.0, 
7.0], [3.0, 6.0, 4.0]], [[1.0, 5.0, 2.0], [4.0, 3.0, 5.0], [3.0, 3.0, 7.0]], [[1.0, 8.0, 8.0], [2.0, 7.0, 4.0], [5.0, 1.0, 
2.0]]], [[[6.0, 1.0, 5.0], [7.0, 2.0, 1.0], [1.0, 8.0, 4.0]], [[8.0, 6.0, 1.0], [5.0, 5.0, 3.0], [3.0, 1.0, 2.0]], [[5.0, 4.0, 
4.0], [8.0, 1.0, 2.0], [5.0, 2.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPPR(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 2.0], [3.0, 3.0, 6.0], [2.0, 8.0, 2.0]], [[1.0, 4.0, 3.0], [7.0, 5.0, 4.0], [1.0, 3.0, 
8.0]], [[6.0, 2.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 6.0]]], [[[2.0, 7.0, 4.0], [3.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[1.0, 5.0, 
6.0], [6.0, 5.0, 5.0], [5.0, 5.0, 8.0]], [[5.0, 1.0, 8.0], [6.0, 1.0, 8.0], [6.0, 6.0, 3.0]]], [[[8.0, 5.0, 3.0], [7.0, 1.0, 
2.0], [7.0, 7.0, 6.0]], [[5.0, 2.0, 2.0], [4.0, 5.0, 4.0], [3.0, 2.0, 2.0]], [[7.0, 3.0, 6.0], [4.0, 4.0, 7.0], [4.0, 8.0, 
4.0]]], [[[4.0, 5.0, 8.0], [7.0, 8.0, 3.0], [6.0, 7.0, 7.0]], [[4.0, 3.0, 3.0], [5.0, 3.0, 3.0], [8.0, 3.0, 4.0]], [[4.0, 6.0, 
3.0], [8.0, 7.0, 2.0], [1.0, 7.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0], [1.0]])+(1.-msk_val)*numpy.array([[5.0], [5.0]])
      arg[2:,1,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 2.0], [3.0, 3.0, 6.0], [2.0, 8.0, 2.0]], [[1.0, 4.0, 3.0], [7.0, 5.0, 4.0], [1.0, 
3.0, 8.0]], [[6.0, 2.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 6.0]]], [[[2.0, 7.0, 4.0], [3.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[1.0, 
5.0, 6.0], [6.0, 5.0, 5.0], [5.0, 5.0, 8.0]], [[5.0, 1.0, 8.0], [6.0, 1.0, 8.0], [6.0, 6.0, 3.0]]], [[[8.0, 5.0, 3.0], [7.0, 
1.0, 2.0], [7.0, 7.0, 6.0]], [[5.0, 2.0, 2.0], [4.0, 6.0, 4.0], [3.0, 2.0, 2.0]], [[7.0, 3.0, 6.0], [4.0, 4.0, 7.0], [4.0, 8.0, 
4.0]]], [[[4.0, 5.0, 8.0], [7.0, 8.0, 3.0], [6.0, 7.0, 7.0]], [[4.0, 3.0, 3.0], [5.0, 1.0, 3.0], [8.0, 3.0, 4.0]], [[4.0, 6.0, 
3.0], [8.0, 7.0, 2.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 2.0], [3.0, 3.0, 6.0], [2.0, 8.0, 2.0]], 
[[1.0, 4.0, 3.0], [7.0, 5.0, 4.0], [1.0, 3.0, 8.0]], [[6.0, 2.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 6.0]]], [[[2.0, 7.0, 4.0], 
[3.0, 2.0, 6.0], [2.0, 2.0, 1.0]], [[1.0, 5.0, 6.0], [6.0, 5.0, 5.0], [5.0, 5.0, 8.0]], [[5.0, 1.0, 8.0], [6.0, 1.0, 8.0], 
[6.0, 6.0, 3.0]]], [[[8.0, 5.0, 3.0], [7.0, 1.0, 2.0], [7.0, 7.0, 6.0]], [[5.0, 2.0, 2.0], [4.0, 5.0, 4.0], [3.0, 2.0, 2.0]], 
[[7.0, 3.0, 6.0], [4.0, 4.0, 7.0], [4.0, 8.0, 4.0]]], [[[4.0, 5.0, 8.0], [7.0, 8.0, 3.0], [6.0, 7.0, 7.0]], [[4.0, 3.0, 3.0], 
[5.0, 5.0, 3.0], [8.0, 3.0, 4.0]], [[4.0, 6.0, 3.0], [8.0, 7.0, 2.0], [1.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLLL(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 4.0], [6.0, 6.0, 6.0], [1.0, 7.0, 1.0]], [[7.0, 3.0, 4.0], [5.0, 8.0, 6.0], [8.0, 7.0, 
6.0]], [[4.0, 7.0, 3.0], [8.0, 5.0, 6.0], [3.0, 5.0, 5.0]]], [[[8.0, 7.0, 4.0], [2.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[1.0, 4.0, 
1.0], [8.0, 4.0, 3.0], [6.0, 8.0, 6.0]], [[6.0, 6.0, 3.0], [1.0, 6.0, 6.0], [3.0, 4.0, 4.0]]], [[[7.0, 1.0, 5.0], [6.0, 3.0, 
6.0], [3.0, 8.0, 2.0]], [[6.0, 2.0, 8.0], [8.0, 6.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 6.0, 1.0], [8.0, 6.0, 2.0], [2.0, 1.0, 
1.0]]], [[[1.0, 1.0, 5.0], [6.0, 2.0, 5.0], [2.0, 2.0, 3.0]], [[4.0, 8.0, 8.0], [6.0, 3.0, 7.0], [7.0, 4.0, 5.0]], [[4.0, 7.0, 
5.0], [1.0, 7.0, 2.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0]]], [[[5.0]]], [[[7.0]]], [[[3.0]]]])
      arg[:,2:,2:,2:]=val
      ref=Data(numpy.array([[[[4.0, 5.0, 4.0], [6.0, 6.0, 6.0], [1.0, 7.0, 1.0]], [[7.0, 3.0, 4.0], [5.0, 8.0, 6.0], [8.0, 7.0, 
6.0]], [[4.0, 7.0, 3.0], [8.0, 5.0, 6.0], [3.0, 5.0, 2.0]]], [[[8.0, 7.0, 4.0], [2.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[1.0, 4.0, 
1.0], [8.0, 4.0, 3.0], [6.0, 8.0, 6.0]], [[6.0, 6.0, 3.0], [1.0, 6.0, 6.0], [3.0, 4.0, 5.0]]], [[[7.0, 1.0, 5.0], [6.0, 3.0, 
6.0], [3.0, 8.0, 2.0]], [[6.0, 2.0, 8.0], [8.0, 6.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 6.0, 1.0], [8.0, 6.0, 2.0], [2.0, 1.0, 
7.0]]], [[[1.0, 1.0, 5.0], [6.0, 2.0, 5.0], [2.0, 2.0, 3.0]], [[4.0, 8.0, 8.0], [6.0, 3.0, 7.0], [7.0, 4.0, 5.0]], [[4.0, 7.0, 
5.0], [1.0, 7.0, 2.0], [2.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLLL(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 6.0], [3.0, 5.0, 5.0], [8.0, 1.0, 6.0]], [[8.0, 3.0, 2.0], [1.0, 7.0, 1.0], [4.0, 3.0, 
6.0]], [[5.0, 7.0, 5.0], [6.0, 3.0, 8.0], [8.0, 2.0, 5.0]]], [[[8.0, 4.0, 5.0], [7.0, 8.0, 8.0], [3.0, 8.0, 1.0]], [[7.0, 5.0, 
4.0], [7.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 7.0], [5.0, 3.0, 5.0], [3.0, 5.0, 6.0]]], [[[7.0, 4.0, 2.0], [1.0, 2.0, 
4.0], [3.0, 2.0, 7.0]], [[1.0, 4.0, 7.0], [4.0, 3.0, 6.0], [6.0, 3.0, 1.0]], [[8.0, 4.0, 2.0], [3.0, 3.0, 1.0], [3.0, 6.0, 
4.0]]], [[[2.0, 5.0, 8.0], [7.0, 3.0, 4.0], [1.0, 2.0, 8.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 4.0], [7.0, 2.0, 3.0]], [[5.0, 4.0, 
5.0], [2.0, 3.0, 7.0], [5.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0]]], [[[7.0]]], [[[3.0]]], [[[2.0]]]]),self.functionspace)
      arg[:,2:,2:,2:]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 6.0], [3.0, 5.0, 5.0], [8.0, 1.0, 6.0]], [[8.0, 3.0, 2.0], [1.0, 7.0, 1.0], [4.0, 3.0, 
6.0]], [[5.0, 7.0, 5.0], [6.0, 3.0, 8.0], [8.0, 2.0, 2.0]]], [[[8.0, 4.0, 5.0], [7.0, 8.0, 8.0], [3.0, 8.0, 1.0]], [[7.0, 5.0, 
4.0], [7.0, 3.0, 2.0], [5.0, 5.0, 2.0]], [[4.0, 8.0, 7.0], [5.0, 3.0, 5.0], [3.0, 5.0, 7.0]]], [[[7.0, 4.0, 2.0], [1.0, 2.0, 
4.0], [3.0, 2.0, 7.0]], [[1.0, 4.0, 7.0], [4.0, 3.0, 6.0], [6.0, 3.0, 1.0]], [[8.0, 4.0, 2.0], [3.0, 3.0, 1.0], [3.0, 6.0, 
3.0]]], [[[2.0, 5.0, 8.0], [7.0, 3.0, 4.0], [1.0, 2.0, 8.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 4.0], [7.0, 2.0, 3.0]], [[5.0, 4.0, 
5.0], [2.0, 3.0, 7.0], [5.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLLL(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 8.0], [5.0, 1.0, 2.0], [1.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [8.0, 3.0, 3.0], [4.0, 5.0, 
7.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 2.0], [1.0, 7.0, 7.0]]], [[[6.0, 2.0, 8.0], [2.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[2.0, 7.0, 
8.0], [1.0, 6.0, 5.0], [4.0, 3.0, 2.0]], [[8.0, 2.0, 5.0], [7.0, 4.0, 6.0], [8.0, 1.0, 1.0]]], [[[8.0, 8.0, 3.0], [5.0, 7.0, 
6.0], [6.0, 6.0, 5.0]], [[8.0, 8.0, 4.0], [5.0, 6.0, 3.0], [6.0, 3.0, 6.0]], [[2.0, 2.0, 2.0], [4.0, 5.0, 8.0], [1.0, 1.0, 
4.0]]], [[[4.0, 5.0, 1.0], [8.0, 8.0, 6.0], [1.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [4.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[4.0, 4.0, 
3.0], [4.0, 6.0, 7.0], [6.0, 1.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0]]], [[[4.0]]], [[[5.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[4.0]]], [[[8.0]]], 
[[[3.0]]], [[[8.0]]]])
      arg[:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 8.0], [5.0, 1.0, 2.0], [1.0, 5.0, 6.0]], [[3.0, 4.0, 4.0], [8.0, 3.0, 3.0], [4.0, 
5.0, 7.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 2.0], [1.0, 7.0, 1.0]]], [[[6.0, 2.0, 8.0], [2.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[2.0, 
7.0, 8.0], [1.0, 6.0, 5.0], [4.0, 3.0, 2.0]], [[8.0, 2.0, 5.0], [7.0, 4.0, 6.0], [8.0, 1.0, 4.0]]], [[[8.0, 8.0, 3.0], [5.0, 
7.0, 6.0], [6.0, 6.0, 5.0]], [[8.0, 8.0, 4.0], [5.0, 6.0, 3.0], [6.0, 3.0, 6.0]], [[2.0, 2.0, 2.0], [4.0, 5.0, 8.0], [1.0, 1.0, 
5.0]]], [[[4.0, 5.0, 1.0], [8.0, 8.0, 6.0], [1.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], [4.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[4.0, 4.0, 
3.0], [4.0, 6.0, 7.0], [6.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [5.0, 1.0, 2.0], [1.0, 5.0, 6.0]], 
[[3.0, 4.0, 4.0], [8.0, 3.0, 3.0], [4.0, 5.0, 7.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 2.0], [1.0, 7.0, 4.0]]], [[[6.0, 2.0, 8.0], 
[2.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[2.0, 7.0, 8.0], [1.0, 6.0, 5.0], [4.0, 3.0, 2.0]], [[8.0, 2.0, 5.0], [7.0, 4.0, 6.0], 
[8.0, 1.0, 8.0]]], [[[8.0, 8.0, 3.0], [5.0, 7.0, 6.0], [6.0, 6.0, 5.0]], [[8.0, 8.0, 4.0], [5.0, 6.0, 3.0], [6.0, 3.0, 6.0]], 
[[2.0, 2.0, 2.0], [4.0, 5.0, 8.0], [1.0, 1.0, 3.0]]], [[[4.0, 5.0, 1.0], [8.0, 8.0, 6.0], [1.0, 2.0, 3.0]], [[2.0, 1.0, 4.0], 
[4.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[4.0, 4.0, 3.0], [4.0, 6.0, 7.0], [6.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCLN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 2.0], [5.0, 3.0, 5.0], [3.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 3.0, 3.0], [6.0, 3.0, 
3.0]], [[5.0, 8.0, 8.0], [5.0, 8.0, 6.0], [6.0, 3.0, 1.0]]], [[[2.0, 2.0, 3.0], [6.0, 5.0, 7.0], [5.0, 3.0, 4.0]], [[1.0, 8.0, 
7.0], [6.0, 5.0, 1.0], [8.0, 3.0, 6.0]], [[8.0, 2.0, 2.0], [7.0, 5.0, 8.0], [4.0, 1.0, 8.0]]], [[[3.0, 1.0, 6.0], [5.0, 5.0, 
6.0], [5.0, 6.0, 1.0]], [[6.0, 6.0, 4.0], [5.0, 1.0, 3.0], [7.0, 4.0, 1.0]], [[8.0, 3.0, 2.0], [4.0, 1.0, 8.0], [3.0, 6.0, 
4.0]]], [[[6.0, 8.0, 3.0], [4.0, 6.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 3.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 5.0]], [[7.0, 2.0, 
3.0], [3.0, 7.0, 1.0], [6.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 5.0, 2.0]], [[4.0, 1.0, 2.0]], [[8.0, 4.0, 8.0]]], [[[1.0, 8.0, 7.0]], [[6.0, 7.0, 1.0]], [[3.0, 
8.0, 8.0]]], [[[3.0, 3.0, 3.0]], [[8.0, 7.0, 5.0]], [[4.0, 8.0, 6.0]]]])
      arg[:3,:,2:]=val
      ref=Data(numpy.array([[[[3.0, 8.0, 2.0], [5.0, 3.0, 5.0], [8.0, 5.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 3.0, 3.0], [4.0, 1.0, 
2.0]], [[5.0, 8.0, 8.0], [5.0, 8.0, 6.0], [8.0, 4.0, 8.0]]], [[[2.0, 2.0, 3.0], [6.0, 5.0, 7.0], [1.0, 8.0, 7.0]], [[1.0, 8.0, 
7.0], [6.0, 5.0, 1.0], [6.0, 7.0, 1.0]], [[8.0, 2.0, 2.0], [7.0, 5.0, 8.0], [3.0, 8.0, 8.0]]], [[[3.0, 1.0, 6.0], [5.0, 5.0, 
6.0], [3.0, 3.0, 3.0]], [[6.0, 6.0, 4.0], [5.0, 1.0, 3.0], [8.0, 7.0, 5.0]], [[8.0, 3.0, 2.0], [4.0, 1.0, 8.0], [4.0, 8.0, 
6.0]]], [[[6.0, 8.0, 3.0], [4.0, 6.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 3.0, 2.0], [2.0, 4.0, 8.0], [7.0, 6.0, 5.0]], [[7.0, 2.0, 
3.0], [3.0, 7.0, 1.0], [6.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCLN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 2.0], [3.0, 2.0, 8.0], [7.0, 8.0, 6.0]], [[6.0, 5.0, 5.0], [4.0, 2.0, 1.0], [1.0, 5.0, 
4.0]], [[5.0, 5.0, 8.0], [1.0, 5.0, 5.0], [4.0, 4.0, 8.0]]], [[[8.0, 4.0, 2.0], [5.0, 6.0, 8.0], [6.0, 6.0, 4.0]], [[7.0, 5.0, 
5.0], [6.0, 3.0, 7.0], [2.0, 2.0, 1.0]], [[5.0, 2.0, 8.0], [3.0, 2.0, 8.0], [7.0, 4.0, 7.0]]], [[[6.0, 7.0, 2.0], [1.0, 4.0, 
5.0], [3.0, 6.0, 3.0]], [[1.0, 1.0, 1.0], [4.0, 2.0, 3.0], [4.0, 4.0, 4.0]], [[2.0, 7.0, 6.0], [4.0, 7.0, 5.0], [1.0, 4.0, 
7.0]]], [[[1.0, 5.0, 8.0], [4.0, 6.0, 8.0], [8.0, 8.0, 1.0]], [[2.0, 8.0, 4.0], [8.0, 2.0, 3.0], [3.0, 6.0, 2.0]], [[5.0, 2.0, 
1.0], [5.0, 5.0, 3.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 2.0, 4.0]], [[2.0, 4.0, 8.0]], [[7.0, 8.0, 4.0]]], [[[5.0, 8.0, 8.0]], [[8.0, 1.0, 6.0]], 
[[4.0, 5.0, 6.0]]], [[[1.0, 3.0, 5.0]], [[7.0, 4.0, 1.0]], [[8.0, 2.0, 6.0]]]]),self.functionspace)
      arg[:3,:,2:]=val
      ref=Data(numpy.array([[[[7.0, 2.0, 2.0], [3.0, 2.0, 8.0], [5.0, 2.0, 4.0]], [[6.0, 5.0, 5.0], [4.0, 2.0, 1.0], [2.0, 4.0, 
8.0]], [[5.0, 5.0, 8.0], [1.0, 5.0, 5.0], [7.0, 8.0, 4.0]]], [[[8.0, 4.0, 2.0], [5.0, 6.0, 8.0], [5.0, 8.0, 8.0]], [[7.0, 5.0, 
5.0], [6.0, 3.0, 7.0], [8.0, 1.0, 6.0]], [[5.0, 2.0, 8.0], [3.0, 2.0, 8.0], [4.0, 5.0, 6.0]]], [[[6.0, 7.0, 2.0], [1.0, 4.0, 
5.0], [1.0, 3.0, 5.0]], [[1.0, 1.0, 1.0], [4.0, 2.0, 3.0], [7.0, 4.0, 1.0]], [[2.0, 7.0, 6.0], [4.0, 7.0, 5.0], [8.0, 2.0, 
6.0]]], [[[1.0, 5.0, 8.0], [4.0, 6.0, 8.0], [8.0, 8.0, 1.0]], [[2.0, 8.0, 4.0], [8.0, 2.0, 3.0], [3.0, 6.0, 2.0]], [[5.0, 2.0, 
1.0], [5.0, 5.0, 3.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCLN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 4.0], [6.0, 4.0, 7.0], [3.0, 4.0, 1.0]], [[8.0, 1.0, 2.0], [2.0, 3.0, 7.0], [7.0, 1.0, 
8.0]], [[7.0, 3.0, 7.0], [3.0, 2.0, 7.0], [6.0, 7.0, 5.0]]], [[[3.0, 2.0, 5.0], [4.0, 7.0, 7.0], [6.0, 1.0, 7.0]], [[3.0, 5.0, 
2.0], [3.0, 3.0, 4.0], [7.0, 7.0, 7.0]], [[6.0, 7.0, 4.0], [3.0, 4.0, 7.0], [2.0, 4.0, 1.0]]], [[[6.0, 7.0, 3.0], [2.0, 1.0, 
3.0], [6.0, 4.0, 6.0]], [[2.0, 7.0, 6.0], [7.0, 8.0, 2.0], [7.0, 7.0, 2.0]], [[8.0, 5.0, 1.0], [8.0, 3.0, 5.0], [7.0, 2.0, 
4.0]]], [[[4.0, 1.0, 3.0], [2.0, 5.0, 7.0], [6.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], [1.0, 5.0, 5.0], [6.0, 3.0, 6.0]], [[3.0, 8.0, 
6.0], [8.0, 8.0, 2.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 1.0, 1.0]], [[4.0, 5.0, 7.0]], [[7.0, 2.0, 3.0]]], [[[3.0, 8.0, 5.0]], [[3.0, 2.0, 
6.0]], [[3.0, 8.0, 8.0]]], [[[1.0, 1.0, 5.0]], [[1.0, 4.0, 2.0]], [[4.0, 3.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 1.0, 
1.0]], [[2.0, 5.0, 6.0]], [[8.0, 4.0, 4.0]]], [[[8.0, 4.0, 1.0]], [[1.0, 3.0, 6.0]], [[7.0, 5.0, 2.0]]], [[[2.0, 8.0, 8.0]], 
[[2.0, 3.0, 6.0]], [[6.0, 7.0, 1.0]]]])
      arg[:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 4.0], [6.0, 4.0, 7.0], [7.0, 1.0, 1.0]], [[8.0, 1.0, 2.0], [2.0, 3.0, 7.0], [4.0, 
5.0, 7.0]], [[7.0, 3.0, 7.0], [3.0, 2.0, 7.0], [7.0, 2.0, 3.0]]], [[[3.0, 2.0, 5.0], [4.0, 7.0, 7.0], [3.0, 8.0, 5.0]], [[3.0, 
5.0, 2.0], [3.0, 3.0, 4.0], [3.0, 2.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 4.0, 7.0], [3.0, 8.0, 8.0]]], [[[6.0, 7.0, 3.0], [2.0, 
1.0, 3.0], [1.0, 1.0, 5.0]], [[2.0, 7.0, 6.0], [7.0, 8.0, 2.0], [1.0, 4.0, 2.0]], [[8.0, 5.0, 1.0], [8.0, 3.0, 5.0], [4.0, 3.0, 
8.0]]], [[[4.0, 1.0, 3.0], [2.0, 5.0, 7.0], [6.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], [1.0, 5.0, 5.0], [6.0, 3.0, 6.0]], [[3.0, 8.0, 
6.0], [8.0, 8.0, 2.0], [5.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 4.0], [6.0, 4.0, 7.0], [1.0, 1.0, 1.0]], 
[[8.0, 1.0, 2.0], [2.0, 3.0, 7.0], [2.0, 5.0, 6.0]], [[7.0, 3.0, 7.0], [3.0, 2.0, 7.0], [8.0, 4.0, 4.0]]], [[[3.0, 2.0, 5.0], 
[4.0, 7.0, 7.0], [8.0, 4.0, 1.0]], [[3.0, 5.0, 2.0], [3.0, 3.0, 4.0], [1.0, 3.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 4.0, 7.0], 
[7.0, 5.0, 2.0]]], [[[6.0, 7.0, 3.0], [2.0, 1.0, 3.0], [2.0, 8.0, 8.0]], [[2.0, 7.0, 6.0], [7.0, 8.0, 2.0], [2.0, 3.0, 6.0]], 
[[8.0, 5.0, 1.0], [8.0, 3.0, 5.0], [6.0, 7.0, 1.0]]], [[[4.0, 1.0, 3.0], [2.0, 5.0, 7.0], [6.0, 1.0, 6.0]], [[4.0, 8.0, 2.0], 
[1.0, 5.0, 5.0], [6.0, 3.0, 6.0]], [[3.0, 8.0, 6.0], [8.0, 8.0, 2.0], [5.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URPN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 6.0], [4.0, 4.0, 4.0], [3.0, 7.0, 1.0]], [[5.0, 3.0, 7.0], [8.0, 7.0, 8.0], [2.0, 2.0, 
4.0]], [[8.0, 3.0, 6.0], [5.0, 5.0, 6.0], [6.0, 5.0, 5.0]]], [[[8.0, 2.0, 6.0], [8.0, 2.0, 8.0], [2.0, 8.0, 6.0]], [[4.0, 7.0, 
1.0], [6.0, 4.0, 6.0], [2.0, 7.0, 4.0]], [[3.0, 1.0, 7.0], [8.0, 5.0, 3.0], [5.0, 5.0, 2.0]]], [[[2.0, 3.0, 3.0], [5.0, 8.0, 
2.0], [8.0, 3.0, 4.0]], [[5.0, 6.0, 8.0], [3.0, 7.0, 4.0], [8.0, 5.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 3.0, 2.0], [7.0, 2.0, 
3.0]]], [[[5.0, 8.0, 2.0], [7.0, 2.0, 8.0], [6.0, 2.0, 6.0]], [[2.0, 8.0, 7.0], [4.0, 1.0, 8.0], [2.0, 8.0, 8.0]], [[4.0, 8.0, 
4.0], [3.0, 8.0, 3.0], [2.0, 8.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 5.0, 7.0]], [[7.0, 7.0, 6.0]], [[4.0, 3.0, 3.0]]])
      arg[:3,1:2,1]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 6.0], [4.0, 4.0, 4.0], [3.0, 7.0, 1.0]], [[5.0, 3.0, 7.0], [3.0, 5.0, 7.0], [2.0, 2.0, 
4.0]], [[8.0, 3.0, 6.0], [5.0, 5.0, 6.0], [6.0, 5.0, 5.0]]], [[[8.0, 2.0, 6.0], [8.0, 2.0, 8.0], [2.0, 8.0, 6.0]], [[4.0, 7.0, 
1.0], [7.0, 7.0, 6.0], [2.0, 7.0, 4.0]], [[3.0, 1.0, 7.0], [8.0, 5.0, 3.0], [5.0, 5.0, 2.0]]], [[[2.0, 3.0, 3.0], [5.0, 8.0, 
2.0], [8.0, 3.0, 4.0]], [[5.0, 6.0, 8.0], [4.0, 3.0, 3.0], [8.0, 5.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 3.0, 2.0], [7.0, 2.0, 
3.0]]], [[[5.0, 8.0, 2.0], [7.0, 2.0, 8.0], [6.0, 2.0, 6.0]], [[2.0, 8.0, 7.0], [4.0, 1.0, 8.0], [2.0, 8.0, 8.0]], [[4.0, 8.0, 
4.0], [3.0, 8.0, 3.0], [2.0, 8.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URPN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 1.0], [2.0, 1.0, 6.0], [4.0, 3.0, 2.0]], [[7.0, 5.0, 7.0], [5.0, 4.0, 5.0], [6.0, 1.0, 
5.0]], [[8.0, 4.0, 4.0], [4.0, 4.0, 5.0], [8.0, 3.0, 7.0]]], [[[7.0, 3.0, 5.0], [1.0, 6.0, 6.0], [7.0, 5.0, 3.0]], [[7.0, 5.0, 
7.0], [1.0, 1.0, 7.0], [1.0, 7.0, 8.0]], [[8.0, 2.0, 2.0], [2.0, 4.0, 5.0], [4.0, 5.0, 2.0]]], [[[4.0, 4.0, 8.0], [8.0, 3.0, 
6.0], [1.0, 7.0, 6.0]], [[1.0, 1.0, 2.0], [2.0, 7.0, 2.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 6.0], [8.0, 5.0, 7.0], [7.0, 1.0, 
3.0]]], [[[1.0, 2.0, 5.0], [8.0, 4.0, 7.0], [5.0, 2.0, 5.0]], [[5.0, 2.0, 7.0], [1.0, 7.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 8.0, 
1.0], [6.0, 3.0, 7.0], [6.0, 4.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 5.0, 3.0]], [[6.0, 2.0, 5.0]], [[6.0, 8.0, 3.0]]]),self.functionspace)
      arg[:3,1:2,1]=val
      ref=Data(numpy.array([[[[3.0, 3.0, 1.0], [2.0, 1.0, 6.0], [4.0, 3.0, 2.0]], [[7.0, 5.0, 7.0], [4.0, 5.0, 3.0], [6.0, 1.0, 
5.0]], [[8.0, 4.0, 4.0], [4.0, 4.0, 5.0], [8.0, 3.0, 7.0]]], [[[7.0, 3.0, 5.0], [1.0, 6.0, 6.0], [7.0, 5.0, 3.0]], [[7.0, 5.0, 
7.0], [6.0, 2.0, 5.0], [1.0, 7.0, 8.0]], [[8.0, 2.0, 2.0], [2.0, 4.0, 5.0], [4.0, 5.0, 2.0]]], [[[4.0, 4.0, 8.0], [8.0, 3.0, 
6.0], [1.0, 7.0, 6.0]], [[1.0, 1.0, 2.0], [6.0, 8.0, 3.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 6.0], [8.0, 5.0, 7.0], [7.0, 1.0, 
3.0]]], [[[1.0, 2.0, 5.0], [8.0, 4.0, 7.0], [5.0, 2.0, 5.0]], [[5.0, 2.0, 7.0], [1.0, 7.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 8.0, 
1.0], [6.0, 3.0, 7.0], [6.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URPN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 3.0], [3.0, 6.0, 7.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 8.0], [1.0, 5.0, 6.0], [2.0, 3.0, 
3.0]], [[4.0, 5.0, 3.0], [8.0, 1.0, 5.0], [7.0, 7.0, 5.0]]], [[[7.0, 7.0, 7.0], [1.0, 7.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 3.0, 
8.0], [7.0, 2.0, 1.0], [1.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [3.0, 8.0, 2.0], [5.0, 2.0, 7.0]]], [[[1.0, 4.0, 5.0], [4.0, 6.0, 
4.0], [3.0, 4.0, 4.0]], [[2.0, 2.0, 4.0], [6.0, 7.0, 3.0], [7.0, 4.0, 6.0]], [[8.0, 4.0, 5.0], [4.0, 8.0, 3.0], [2.0, 8.0, 
8.0]]], [[[4.0, 2.0, 5.0], [7.0, 1.0, 4.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 6.0], [1.0, 3.0, 7.0], [5.0, 7.0, 7.0]], [[1.0, 3.0, 
4.0], [5.0, 2.0, 3.0], [8.0, 6.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 6.0, 2.0]], [[2.0, 6.0, 7.0]], [[7.0, 3.0, 3.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0, 
6.0]], [[1.0, 5.0, 3.0]], [[6.0, 8.0, 6.0]]])
      arg[:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 3.0], [3.0, 6.0, 7.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 8.0], [3.0, 6.0, 2.0], [2.0, 
3.0, 3.0]], [[4.0, 5.0, 3.0], [8.0, 1.0, 5.0], [7.0, 7.0, 5.0]]], [[[7.0, 7.0, 7.0], [1.0, 7.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 
3.0, 8.0], [2.0, 6.0, 7.0], [1.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [3.0, 8.0, 2.0], [5.0, 2.0, 7.0]]], [[[1.0, 4.0, 5.0], [4.0, 
6.0, 4.0], [3.0, 4.0, 4.0]], [[2.0, 2.0, 4.0], [7.0, 3.0, 3.0], [7.0, 4.0, 6.0]], [[8.0, 4.0, 5.0], [4.0, 8.0, 3.0], [2.0, 8.0, 
8.0]]], [[[4.0, 2.0, 5.0], [7.0, 1.0, 4.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 6.0], [1.0, 3.0, 7.0], [5.0, 7.0, 7.0]], [[1.0, 3.0, 
4.0], [5.0, 2.0, 3.0], [8.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 3.0], [3.0, 6.0, 7.0], [3.0, 3.0, 3.0]], 
[[1.0, 5.0, 8.0], [4.0, 7.0, 6.0], [2.0, 3.0, 3.0]], [[4.0, 5.0, 3.0], [8.0, 1.0, 5.0], [7.0, 7.0, 5.0]]], [[[7.0, 7.0, 7.0], 
[1.0, 7.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 3.0, 8.0], [1.0, 5.0, 3.0], [1.0, 3.0, 5.0]], [[6.0, 4.0, 1.0], [3.0, 8.0, 2.0], 
[5.0, 2.0, 7.0]]], [[[1.0, 4.0, 5.0], [4.0, 6.0, 4.0], [3.0, 4.0, 4.0]], [[2.0, 2.0, 4.0], [6.0, 8.0, 6.0], [7.0, 4.0, 6.0]], 
[[8.0, 4.0, 5.0], [4.0, 8.0, 3.0], [2.0, 8.0, 8.0]]], [[[4.0, 2.0, 5.0], [7.0, 1.0, 4.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 6.0], 
[1.0, 3.0, 7.0], [5.0, 7.0, 7.0]], [[1.0, 3.0, 4.0], [5.0, 2.0, 3.0], [8.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLPN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [8.0, 5.0, 4.0], [6.0, 5.0, 2.0]], [[3.0, 8.0, 8.0], [4.0, 2.0, 7.0], [4.0, 2.0, 
7.0]], [[5.0, 7.0, 2.0], [1.0, 4.0, 5.0], [6.0, 6.0, 3.0]]], [[[5.0, 3.0, 5.0], [7.0, 6.0, 7.0], [2.0, 5.0, 1.0]], [[6.0, 7.0, 
8.0], [3.0, 3.0, 5.0], [7.0, 2.0, 1.0]], [[1.0, 2.0, 3.0], [8.0, 4.0, 4.0], [1.0, 5.0, 8.0]]], [[[3.0, 8.0, 2.0], [1.0, 4.0, 
5.0], [7.0, 5.0, 2.0]], [[3.0, 5.0, 6.0], [7.0, 5.0, 8.0], [3.0, 5.0, 6.0]], [[3.0, 2.0, 3.0], [8.0, 3.0, 3.0], [4.0, 6.0, 
7.0]]], [[[3.0, 6.0, 7.0], [2.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[5.0, 1.0, 1.0], [6.0, 5.0, 6.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 
1.0], [5.0, 7.0, 7.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[6.0, 2.0, 6.0]])
      arg[2,2:,1]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 6.0], [8.0, 5.0, 4.0], [6.0, 5.0, 2.0]], [[3.0, 8.0, 8.0], [4.0, 2.0, 7.0], [4.0, 2.0, 
7.0]], [[5.0, 7.0, 2.0], [1.0, 4.0, 5.0], [6.0, 6.0, 3.0]]], [[[5.0, 3.0, 5.0], [7.0, 6.0, 7.0], [2.0, 5.0, 1.0]], [[6.0, 7.0, 
8.0], [3.0, 3.0, 5.0], [7.0, 2.0, 1.0]], [[1.0, 2.0, 3.0], [8.0, 4.0, 4.0], [1.0, 5.0, 8.0]]], [[[3.0, 8.0, 2.0], [1.0, 4.0, 
5.0], [7.0, 5.0, 2.0]], [[3.0, 5.0, 6.0], [7.0, 5.0, 8.0], [3.0, 5.0, 6.0]], [[3.0, 2.0, 3.0], [6.0, 2.0, 6.0], [4.0, 6.0, 
7.0]]], [[[3.0, 6.0, 7.0], [2.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[5.0, 1.0, 1.0], [6.0, 5.0, 6.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 
1.0], [5.0, 7.0, 7.0], [7.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLPN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 7.0], [1.0, 5.0, 4.0], [3.0, 3.0, 4.0]], [[2.0, 1.0, 4.0], [1.0, 2.0, 2.0], [1.0, 7.0, 
4.0]], [[6.0, 6.0, 8.0], [8.0, 5.0, 3.0], [4.0, 7.0, 4.0]]], [[[6.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 5.0, 7.0]], [[8.0, 5.0, 
3.0], [4.0, 2.0, 2.0], [6.0, 4.0, 3.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 1.0], [2.0, 1.0, 8.0]]], [[[7.0, 8.0, 4.0], [2.0, 1.0, 
3.0], [8.0, 8.0, 7.0]], [[3.0, 5.0, 8.0], [2.0, 5.0, 1.0], [1.0, 4.0, 6.0]], [[1.0, 6.0, 3.0], [6.0, 6.0, 3.0], [6.0, 2.0, 
1.0]]], [[[8.0, 7.0, 4.0], [5.0, 2.0, 6.0], [6.0, 8.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 5.0, 4.0], [5.0, 5.0, 8.0]], [[2.0, 8.0, 
1.0], [1.0, 5.0, 4.0], [7.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 6.0, 3.0]]),self.functionspace)
      arg[2,2:,1]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 7.0], [1.0, 5.0, 4.0], [3.0, 3.0, 4.0]], [[2.0, 1.0, 4.0], [1.0, 2.0, 2.0], [1.0, 7.0, 
4.0]], [[6.0, 6.0, 8.0], [8.0, 5.0, 3.0], [4.0, 7.0, 4.0]]], [[[6.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 5.0, 7.0]], [[8.0, 5.0, 
3.0], [4.0, 2.0, 2.0], [6.0, 4.0, 3.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 1.0], [2.0, 1.0, 8.0]]], [[[7.0, 8.0, 4.0], [2.0, 1.0, 
3.0], [8.0, 8.0, 7.0]], [[3.0, 5.0, 8.0], [2.0, 5.0, 1.0], [1.0, 4.0, 6.0]], [[1.0, 6.0, 3.0], [7.0, 6.0, 3.0], [6.0, 2.0, 
1.0]]], [[[8.0, 7.0, 4.0], [5.0, 2.0, 6.0], [6.0, 8.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 5.0, 4.0], [5.0, 5.0, 8.0]], [[2.0, 8.0, 
1.0], [1.0, 5.0, 4.0], [7.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLPN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 6.0], [5.0, 8.0, 2.0], [3.0, 4.0, 5.0]], [[7.0, 6.0, 8.0], [7.0, 4.0, 1.0], [5.0, 4.0, 
5.0]], [[3.0, 7.0, 5.0], [6.0, 7.0, 2.0], [6.0, 3.0, 3.0]]], [[[8.0, 6.0, 7.0], [1.0, 2.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 7.0, 
1.0], [7.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], [7.0, 6.0, 2.0], [1.0, 5.0, 3.0]]], [[[8.0, 2.0, 7.0], [8.0, 4.0, 
7.0], [2.0, 8.0, 4.0]], [[4.0, 6.0, 4.0], [6.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[1.0, 3.0, 8.0], [3.0, 4.0, 4.0], [3.0, 1.0, 
5.0]]], [[[6.0, 7.0, 4.0], [6.0, 6.0, 7.0], [5.0, 6.0, 4.0]], [[7.0, 7.0, 2.0], [7.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[6.0, 4.0, 
7.0], [2.0, 1.0, 6.0], [3.0, 8.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 8.0, 4.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0, 7.0]])
      arg[2,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 6.0], [5.0, 8.0, 2.0], [3.0, 4.0, 5.0]], [[7.0, 6.0, 8.0], [7.0, 4.0, 1.0], [5.0, 
4.0, 5.0]], [[3.0, 7.0, 5.0], [6.0, 7.0, 2.0], [6.0, 3.0, 3.0]]], [[[8.0, 6.0, 7.0], [1.0, 2.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 
7.0, 1.0], [7.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], [7.0, 6.0, 2.0], [1.0, 5.0, 3.0]]], [[[8.0, 2.0, 7.0], [8.0, 
4.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 6.0, 4.0], [6.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[1.0, 3.0, 8.0], [5.0, 8.0, 4.0], [3.0, 1.0, 
5.0]]], [[[6.0, 7.0, 4.0], [6.0, 6.0, 7.0], [5.0, 6.0, 4.0]], [[7.0, 7.0, 2.0], [7.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[6.0, 4.0, 
7.0], [2.0, 1.0, 6.0], [3.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 6.0], [5.0, 8.0, 2.0], [3.0, 4.0, 5.0]], 
[[7.0, 6.0, 8.0], [7.0, 4.0, 1.0], [5.0, 4.0, 5.0]], [[3.0, 7.0, 5.0], [6.0, 7.0, 2.0], [6.0, 3.0, 3.0]]], [[[8.0, 6.0, 7.0], 
[1.0, 2.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 7.0, 1.0], [7.0, 1.0, 3.0], [6.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], [7.0, 6.0, 2.0], 
[1.0, 5.0, 3.0]]], [[[8.0, 2.0, 7.0], [8.0, 4.0, 7.0], [2.0, 8.0, 4.0]], [[4.0, 6.0, 4.0], [6.0, 5.0, 2.0], [5.0, 2.0, 4.0]], 
[[1.0, 3.0, 8.0], [3.0, 4.0, 7.0], [3.0, 1.0, 5.0]]], [[[6.0, 7.0, 4.0], [6.0, 6.0, 7.0], [5.0, 6.0, 4.0]], [[7.0, 7.0, 2.0], 
[7.0, 8.0, 7.0], [3.0, 3.0, 7.0]], [[6.0, 4.0, 7.0], [2.0, 1.0, 6.0], [3.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLCN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 1.0], [7.0, 5.0, 5.0], [5.0, 5.0, 8.0]], [[6.0, 3.0, 1.0], [3.0, 4.0, 6.0], [4.0, 6.0, 
1.0]], [[5.0, 3.0, 7.0], [7.0, 7.0, 4.0], [1.0, 7.0, 4.0]]], [[[1.0, 6.0, 6.0], [8.0, 8.0, 2.0], [1.0, 8.0, 8.0]], [[5.0, 2.0, 
8.0], [7.0, 5.0, 5.0], [6.0, 7.0, 1.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 4.0], [6.0, 7.0, 4.0]]], [[[5.0, 6.0, 2.0], [6.0, 3.0, 
3.0], [1.0, 8.0, 5.0]], [[3.0, 6.0, 5.0], [4.0, 2.0, 8.0], [7.0, 2.0, 6.0]], [[6.0, 8.0, 2.0], [5.0, 8.0, 2.0], [2.0, 1.0, 
2.0]]], [[[3.0, 5.0, 1.0], [5.0, 4.0, 1.0], [8.0, 4.0, 4.0]], [[5.0, 7.0, 7.0], [1.0, 4.0, 7.0], [3.0, 3.0, 1.0]], [[2.0, 6.0, 
3.0], [7.0, 4.0, 6.0], [1.0, 1.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 3.0, 1.0], [8.0, 3.0, 8.0], [3.0, 5.0, 1.0]]], [[[3.0, 5.0, 6.0], [7.0, 5.0, 8.0], [2.0, 1.0, 
2.0]]], [[[2.0, 4.0, 1.0], [1.0, 7.0, 5.0], [8.0, 3.0, 3.0]]], [[[3.0, 8.0, 1.0], [3.0, 6.0, 2.0], [4.0, 3.0, 8.0]]]])
      arg[:,2:,:]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 1.0], [7.0, 5.0, 5.0], [5.0, 5.0, 8.0]], [[6.0, 3.0, 1.0], [3.0, 4.0, 6.0], [4.0, 6.0, 
1.0]], [[5.0, 3.0, 1.0], [8.0, 3.0, 8.0], [3.0, 5.0, 1.0]]], [[[1.0, 6.0, 6.0], [8.0, 8.0, 2.0], [1.0, 8.0, 8.0]], [[5.0, 2.0, 
8.0], [7.0, 5.0, 5.0], [6.0, 7.0, 1.0]], [[3.0, 5.0, 6.0], [7.0, 5.0, 8.0], [2.0, 1.0, 2.0]]], [[[5.0, 6.0, 2.0], [6.0, 3.0, 
3.0], [1.0, 8.0, 5.0]], [[3.0, 6.0, 5.0], [4.0, 2.0, 8.0], [7.0, 2.0, 6.0]], [[2.0, 4.0, 1.0], [1.0, 7.0, 5.0], [8.0, 3.0, 
3.0]]], [[[3.0, 5.0, 1.0], [5.0, 4.0, 1.0], [8.0, 4.0, 4.0]], [[5.0, 7.0, 7.0], [1.0, 4.0, 7.0], [3.0, 3.0, 1.0]], [[3.0, 8.0, 
1.0], [3.0, 6.0, 2.0], [4.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLCN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [5.0, 6.0, 1.0], [4.0, 8.0, 3.0]], [[7.0, 1.0, 1.0], [7.0, 7.0, 5.0], [1.0, 8.0, 
6.0]], [[6.0, 4.0, 2.0], [3.0, 3.0, 6.0], [4.0, 2.0, 4.0]]], [[[2.0, 5.0, 4.0], [7.0, 1.0, 6.0], [2.0, 2.0, 2.0]], [[3.0, 8.0, 
2.0], [4.0, 4.0, 1.0], [7.0, 1.0, 7.0]], [[5.0, 3.0, 3.0], [1.0, 7.0, 1.0], [1.0, 5.0, 6.0]]], [[[2.0, 1.0, 7.0], [6.0, 5.0, 
7.0], [6.0, 7.0, 4.0]], [[6.0, 7.0, 2.0], [4.0, 3.0, 8.0], [3.0, 2.0, 8.0]], [[5.0, 3.0, 5.0], [2.0, 3.0, 1.0], [5.0, 4.0, 
5.0]]], [[[8.0, 2.0, 8.0], [2.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[8.0, 6.0, 3.0], [4.0, 5.0, 8.0], [3.0, 3.0, 2.0]], [[6.0, 2.0, 
7.0], [1.0, 6.0, 3.0], [8.0, 5.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 6.0, 5.0], [4.0, 6.0, 5.0], [1.0, 3.0, 4.0]]], [[[3.0, 7.0, 2.0], [7.0, 8.0, 6.0], [6.0, 
3.0, 8.0]]], [[[1.0, 1.0, 5.0], [4.0, 1.0, 4.0], [5.0, 8.0, 7.0]]], [[[5.0, 3.0, 7.0], [3.0, 8.0, 7.0], [2.0, 6.0, 
3.0]]]]),self.functionspace)
      arg[:,2:,:]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 5.0], [5.0, 6.0, 1.0], [4.0, 8.0, 3.0]], [[7.0, 1.0, 1.0], [7.0, 7.0, 5.0], [1.0, 8.0, 
6.0]], [[6.0, 6.0, 5.0], [4.0, 6.0, 5.0], [1.0, 3.0, 4.0]]], [[[2.0, 5.0, 4.0], [7.0, 1.0, 6.0], [2.0, 2.0, 2.0]], [[3.0, 8.0, 
2.0], [4.0, 4.0, 1.0], [7.0, 1.0, 7.0]], [[3.0, 7.0, 2.0], [7.0, 8.0, 6.0], [6.0, 3.0, 8.0]]], [[[2.0, 1.0, 7.0], [6.0, 5.0, 
7.0], [6.0, 7.0, 4.0]], [[6.0, 7.0, 2.0], [4.0, 3.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 1.0, 5.0], [4.0, 1.0, 4.0], [5.0, 8.0, 
7.0]]], [[[8.0, 2.0, 8.0], [2.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[8.0, 6.0, 3.0], [4.0, 5.0, 8.0], [3.0, 3.0, 2.0]], [[5.0, 3.0, 
7.0], [3.0, 8.0, 7.0], [2.0, 6.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 8.0], [2.0, 2.0, 4.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 5.0], [5.0, 6.0, 
6.0]], [[5.0, 8.0, 4.0], [1.0, 6.0, 4.0], [7.0, 8.0, 8.0]]], [[[1.0, 4.0, 4.0], [8.0, 5.0, 4.0], [3.0, 7.0, 5.0]], [[4.0, 3.0, 
7.0], [1.0, 7.0, 1.0], [2.0, 8.0, 5.0]], [[2.0, 8.0, 7.0], [1.0, 2.0, 1.0], [8.0, 5.0, 2.0]]], [[[1.0, 4.0, 5.0], [5.0, 7.0, 
3.0], [1.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 4.0], [7.0, 5.0, 1.0]], [[7.0, 4.0, 5.0], [2.0, 8.0, 6.0], [6.0, 1.0, 
6.0]]], [[[2.0, 1.0, 7.0], [8.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 3.0], [4.0, 7.0, 2.0], [4.0, 8.0, 3.0]], [[4.0, 6.0, 
4.0], [4.0, 6.0, 7.0], [4.0, 1.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 6.0, 7.0], [2.0, 1.0, 1.0], [6.0, 5.0, 5.0]]], [[[7.0, 2.0, 7.0], [8.0, 1.0, 3.0], [5.0, 
4.0, 7.0]]], [[[3.0, 6.0, 3.0], [5.0, 4.0, 6.0], [4.0, 2.0, 8.0]]], [[[6.0, 4.0, 8.0], [1.0, 4.0, 6.0], [6.0, 8.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 6.0], [3.0, 5.0, 4.0], [1.0, 8.0, 1.0]]], [[[4.0, 7.0, 2.0], [1.0, 2.0, 3.0], 
[8.0, 6.0, 3.0]]], [[[4.0, 2.0, 6.0], [3.0, 4.0, 5.0], [7.0, 6.0, 3.0]]], [[[5.0, 4.0, 1.0], [6.0, 5.0, 4.0], [5.0, 2.0, 
6.0]]]])
      arg[:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 8.0], [2.0, 2.0, 4.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 5.0], [5.0, 
6.0, 6.0]], [[3.0, 6.0, 7.0], [2.0, 1.0, 1.0], [6.0, 5.0, 5.0]]], [[[1.0, 4.0, 4.0], [8.0, 5.0, 4.0], [3.0, 7.0, 5.0]], [[4.0, 
3.0, 7.0], [1.0, 7.0, 1.0], [2.0, 8.0, 5.0]], [[7.0, 2.0, 7.0], [8.0, 1.0, 3.0], [5.0, 4.0, 7.0]]], [[[1.0, 4.0, 5.0], [5.0, 
7.0, 3.0], [1.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 4.0], [7.0, 5.0, 1.0]], [[3.0, 6.0, 3.0], [5.0, 4.0, 6.0], [4.0, 2.0, 
8.0]]], [[[2.0, 1.0, 7.0], [8.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 3.0], [4.0, 7.0, 2.0], [4.0, 8.0, 3.0]], [[6.0, 4.0, 
8.0], [1.0, 4.0, 6.0], [6.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 8.0], [2.0, 2.0, 4.0], [5.0, 5.0, 8.0]], 
[[8.0, 6.0, 8.0], [6.0, 4.0, 5.0], [5.0, 6.0, 6.0]], [[5.0, 4.0, 6.0], [3.0, 5.0, 4.0], [1.0, 8.0, 1.0]]], [[[1.0, 4.0, 4.0], 
[8.0, 5.0, 4.0], [3.0, 7.0, 5.0]], [[4.0, 3.0, 7.0], [1.0, 7.0, 1.0], [2.0, 8.0, 5.0]], [[4.0, 7.0, 2.0], [1.0, 2.0, 3.0], 
[8.0, 6.0, 3.0]]], [[[1.0, 4.0, 5.0], [5.0, 7.0, 3.0], [1.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 4.0], [7.0, 5.0, 1.0]], 
[[4.0, 2.0, 6.0], [3.0, 4.0, 5.0], [7.0, 6.0, 3.0]]], [[[2.0, 1.0, 7.0], [8.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 3.0], 
[4.0, 7.0, 2.0], [4.0, 8.0, 3.0]], [[5.0, 4.0, 1.0], [6.0, 5.0, 4.0], [5.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRUR(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 4.0], [8.0, 2.0, 3.0], [3.0, 7.0, 2.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 8.0], [7.0, 4.0, 
4.0]], [[3.0, 4.0, 7.0], [5.0, 2.0, 1.0], [4.0, 4.0, 4.0]]], [[[2.0, 1.0, 6.0], [6.0, 5.0, 2.0], [4.0, 2.0, 5.0]], [[8.0, 5.0, 
2.0], [4.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[4.0, 1.0, 4.0], [3.0, 7.0, 1.0], [8.0, 2.0, 4.0]]], [[[1.0, 4.0, 5.0], [3.0, 8.0, 
7.0], [7.0, 2.0, 2.0]], [[2.0, 7.0, 4.0], [2.0, 8.0, 6.0], [2.0, 7.0, 6.0]], [[4.0, 4.0, 2.0], [8.0, 1.0, 2.0], [7.0, 7.0, 
6.0]]], [[[1.0, 1.0, 3.0], [6.0, 8.0, 1.0], [3.0, 4.0, 2.0]], [[8.0, 5.0, 2.0], [2.0, 5.0, 1.0], [8.0, 8.0, 7.0]], [[4.0, 1.0, 
1.0], [3.0, 2.0, 2.0], [1.0, 8.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0], [6.0]]], [[[2.0], [6.0]]]])
      arg[1:3,1:2,:2,1:2]=val
      ref=Data(numpy.array([[[[1.0, 3.0, 4.0], [8.0, 2.0, 3.0], [3.0, 7.0, 2.0]], [[4.0, 8.0, 5.0], [4.0, 5.0, 8.0], [7.0, 4.0, 
4.0]], [[3.0, 4.0, 7.0], [5.0, 2.0, 1.0], [4.0, 4.0, 4.0]]], [[[2.0, 1.0, 6.0], [6.0, 5.0, 2.0], [4.0, 2.0, 5.0]], [[8.0, 5.0, 
2.0], [4.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[4.0, 1.0, 4.0], [3.0, 7.0, 1.0], [8.0, 2.0, 4.0]]], [[[1.0, 4.0, 5.0], [3.0, 8.0, 
7.0], [7.0, 2.0, 2.0]], [[2.0, 2.0, 4.0], [2.0, 6.0, 6.0], [2.0, 7.0, 6.0]], [[4.0, 4.0, 2.0], [8.0, 1.0, 2.0], [7.0, 7.0, 
6.0]]], [[[1.0, 1.0, 3.0], [6.0, 8.0, 1.0], [3.0, 4.0, 2.0]], [[8.0, 5.0, 2.0], [2.0, 5.0, 1.0], [8.0, 8.0, 7.0]], [[4.0, 1.0, 
1.0], [3.0, 2.0, 2.0], [1.0, 8.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRUR(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 1.0], [1.0, 4.0, 3.0], [1.0, 3.0, 6.0]], [[1.0, 3.0, 6.0], [7.0, 5.0, 4.0], [7.0, 7.0, 
5.0]], [[8.0, 1.0, 7.0], [2.0, 7.0, 4.0], [3.0, 6.0, 2.0]]], [[[3.0, 3.0, 3.0], [5.0, 6.0, 4.0], [8.0, 5.0, 2.0]], [[1.0, 3.0, 
2.0], [3.0, 7.0, 1.0], [4.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [8.0, 2.0, 7.0], [4.0, 5.0, 3.0]]], [[[8.0, 5.0, 6.0], [7.0, 5.0, 
3.0], [4.0, 8.0, 7.0]], [[7.0, 8.0, 7.0], [6.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 1.0], [3.0, 5.0, 
7.0]]], [[[1.0, 7.0, 3.0], [8.0, 1.0, 8.0], [8.0, 7.0, 2.0]], [[1.0, 7.0, 1.0], [8.0, 5.0, 7.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 
4.0], [7.0, 1.0, 8.0], [6.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0], [3.0]]], [[[4.0], [7.0]]]]),self.functionspace)
      arg[1:3,1:2,:2,1:2]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 1.0], [1.0, 4.0, 3.0], [1.0, 3.0, 6.0]], [[1.0, 3.0, 6.0], [7.0, 5.0, 4.0], [7.0, 7.0, 
5.0]], [[8.0, 1.0, 7.0], [2.0, 7.0, 4.0], [3.0, 6.0, 2.0]]], [[[3.0, 3.0, 3.0], [5.0, 6.0, 4.0], [8.0, 5.0, 2.0]], [[1.0, 3.0, 
2.0], [3.0, 3.0, 1.0], [4.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [8.0, 2.0, 7.0], [4.0, 5.0, 3.0]]], [[[8.0, 5.0, 6.0], [7.0, 5.0, 
3.0], [4.0, 8.0, 7.0]], [[7.0, 4.0, 7.0], [6.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 1.0], [3.0, 5.0, 
7.0]]], [[[1.0, 7.0, 3.0], [8.0, 1.0, 8.0], [8.0, 7.0, 2.0]], [[1.0, 7.0, 1.0], [8.0, 5.0, 7.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 
4.0], [7.0, 1.0, 8.0], [6.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRUR(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 1.0], [6.0, 5.0, 4.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 6.0], [4.0, 2.0, 8.0], [5.0, 1.0, 
6.0]], [[5.0, 7.0, 4.0], [2.0, 7.0, 1.0], [7.0, 5.0, 4.0]]], [[[8.0, 5.0, 6.0], [7.0, 6.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 5.0, 
4.0], [7.0, 8.0, 7.0], [1.0, 5.0, 2.0]], [[4.0, 5.0, 4.0], [3.0, 7.0, 6.0], [6.0, 5.0, 7.0]]], [[[4.0, 4.0, 3.0], [2.0, 6.0, 
7.0], [8.0, 7.0, 7.0]], [[2.0, 1.0, 2.0], [7.0, 6.0, 7.0], [5.0, 3.0, 5.0]], [[2.0, 4.0, 2.0], [7.0, 8.0, 4.0], [5.0, 8.0, 
2.0]]], [[[8.0, 4.0, 2.0], [5.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[8.0, 7.0, 2.0], [7.0, 8.0, 3.0], [8.0, 7.0, 2.0]], [[7.0, 6.0, 
4.0], [3.0, 2.0, 8.0], [3.0, 2.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0], [7.0]]], [[[5.0], [5.0]]]])+(1.-msk_val)*numpy.array([[[[2.0], [6.0]]], [[[3.0], 
[1.0]]]])
      arg[1:3,1:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 1.0], [6.0, 5.0, 4.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 6.0], [4.0, 2.0, 8.0], [5.0, 
1.0, 6.0]], [[5.0, 7.0, 4.0], [2.0, 7.0, 1.0], [7.0, 5.0, 4.0]]], [[[8.0, 5.0, 6.0], [7.0, 6.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 
7.0, 4.0], [7.0, 7.0, 7.0], [1.0, 5.0, 2.0]], [[4.0, 5.0, 4.0], [3.0, 7.0, 6.0], [6.0, 5.0, 7.0]]], [[[4.0, 4.0, 3.0], [2.0, 
6.0, 7.0], [8.0, 7.0, 7.0]], [[2.0, 5.0, 2.0], [7.0, 5.0, 7.0], [5.0, 3.0, 5.0]], [[2.0, 4.0, 2.0], [7.0, 8.0, 4.0], [5.0, 8.0, 
2.0]]], [[[8.0, 4.0, 2.0], [5.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[8.0, 7.0, 2.0], [7.0, 8.0, 3.0], [8.0, 7.0, 2.0]], [[7.0, 6.0, 
4.0], [3.0, 2.0, 8.0], [3.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 1.0], [6.0, 5.0, 4.0], [4.0, 7.0, 6.0]], 
[[2.0, 1.0, 6.0], [4.0, 2.0, 8.0], [5.0, 1.0, 6.0]], [[5.0, 7.0, 4.0], [2.0, 7.0, 1.0], [7.0, 5.0, 4.0]]], [[[8.0, 5.0, 6.0], 
[7.0, 6.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 2.0, 4.0], [7.0, 6.0, 7.0], [1.0, 5.0, 2.0]], [[4.0, 5.0, 4.0], [3.0, 7.0, 6.0], 
[6.0, 5.0, 7.0]]], [[[4.0, 4.0, 3.0], [2.0, 6.0, 7.0], [8.0, 7.0, 7.0]], [[2.0, 3.0, 2.0], [7.0, 1.0, 7.0], [5.0, 3.0, 5.0]], 
[[2.0, 4.0, 2.0], [7.0, 8.0, 4.0], [5.0, 8.0, 2.0]]], [[[8.0, 4.0, 2.0], [5.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[8.0, 7.0, 2.0], 
[7.0, 8.0, 3.0], [8.0, 7.0, 2.0]], [[7.0, 6.0, 4.0], [3.0, 2.0, 8.0], [3.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUCR(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 5.0], [3.0, 7.0, 1.0], [6.0, 1.0, 5.0]], [[2.0, 3.0, 6.0], [7.0, 4.0, 3.0], [4.0, 7.0, 
6.0]], [[1.0, 8.0, 2.0], [6.0, 8.0, 1.0], [5.0, 6.0, 1.0]]], [[[5.0, 5.0, 3.0], [8.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[2.0, 8.0, 
6.0], [7.0, 7.0, 5.0], [4.0, 4.0, 8.0]], [[8.0, 7.0, 2.0], [8.0, 6.0, 8.0], [3.0, 5.0, 4.0]]], [[[3.0, 2.0, 6.0], [5.0, 4.0, 
3.0], [1.0, 1.0, 4.0]], [[6.0, 1.0, 8.0], [6.0, 2.0, 1.0], [5.0, 6.0, 1.0]], [[6.0, 1.0, 2.0], [3.0, 2.0, 7.0], [3.0, 3.0, 
7.0]]], [[[1.0, 5.0, 6.0], [1.0, 2.0, 2.0], [7.0, 8.0, 6.0]], [[6.0, 8.0, 3.0], [4.0, 3.0, 7.0], [5.0, 2.0, 1.0]], [[7.0, 4.0, 
5.0], [6.0, 3.0, 1.0], [5.0, 3.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0], [8.0], [3.0]], [[5.0], [4.0], [7.0]]], [[[3.0], [8.0], [4.0]], [[1.0], [4.0], [2.0]]]])
      arg[1:3,:2,:,1:2]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 5.0], [3.0, 7.0, 1.0], [6.0, 1.0, 5.0]], [[2.0, 3.0, 6.0], [7.0, 4.0, 3.0], [4.0, 7.0, 
6.0]], [[1.0, 8.0, 2.0], [6.0, 8.0, 1.0], [5.0, 6.0, 1.0]]], [[[5.0, 6.0, 3.0], [8.0, 8.0, 8.0], [8.0, 3.0, 5.0]], [[2.0, 5.0, 
6.0], [7.0, 4.0, 5.0], [4.0, 7.0, 8.0]], [[8.0, 7.0, 2.0], [8.0, 6.0, 8.0], [3.0, 5.0, 4.0]]], [[[3.0, 3.0, 6.0], [5.0, 8.0, 
3.0], [1.0, 4.0, 4.0]], [[6.0, 1.0, 8.0], [6.0, 4.0, 1.0], [5.0, 2.0, 1.0]], [[6.0, 1.0, 2.0], [3.0, 2.0, 7.0], [3.0, 3.0, 
7.0]]], [[[1.0, 5.0, 6.0], [1.0, 2.0, 2.0], [7.0, 8.0, 6.0]], [[6.0, 8.0, 3.0], [4.0, 3.0, 7.0], [5.0, 2.0, 1.0]], [[7.0, 4.0, 
5.0], [6.0, 3.0, 1.0], [5.0, 3.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUCR(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 8.0], [2.0, 5.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [4.0, 5.0, 7.0], [2.0, 1.0, 
6.0]], [[1.0, 2.0, 2.0], [4.0, 4.0, 1.0], [5.0, 2.0, 5.0]]], [[[2.0, 6.0, 5.0], [1.0, 8.0, 3.0], [7.0, 1.0, 8.0]], [[8.0, 3.0, 
2.0], [2.0, 7.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 2.0], [1.0, 2.0, 4.0], [3.0, 8.0, 8.0]]], [[[4.0, 5.0, 1.0], [5.0, 7.0, 
2.0], [3.0, 7.0, 5.0]], [[3.0, 4.0, 6.0], [7.0, 2.0, 5.0], [7.0, 6.0, 1.0]], [[2.0, 2.0, 8.0], [3.0, 7.0, 3.0], [5.0, 7.0, 
4.0]]], [[[2.0, 4.0, 5.0], [1.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[3.0, 5.0, 8.0], [7.0, 6.0, 2.0], [6.0, 6.0, 2.0]], [[1.0, 3.0, 
1.0], [7.0, 5.0, 4.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0], [5.0], [6.0]], [[3.0], [3.0], [7.0]]], [[[6.0], [8.0], [8.0]], [[8.0], [5.0], 
[6.0]]]]),self.functionspace)
      arg[1:3,:2,:,1:2]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 8.0], [2.0, 5.0, 1.0], [1.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [4.0, 5.0, 7.0], [2.0, 1.0, 
6.0]], [[1.0, 2.0, 2.0], [4.0, 4.0, 1.0], [5.0, 2.0, 5.0]]], [[[2.0, 4.0, 5.0], [1.0, 5.0, 3.0], [7.0, 6.0, 8.0]], [[8.0, 3.0, 
2.0], [2.0, 3.0, 8.0], [6.0, 7.0, 7.0]], [[6.0, 6.0, 2.0], [1.0, 2.0, 4.0], [3.0, 8.0, 8.0]]], [[[4.0, 6.0, 1.0], [5.0, 8.0, 
2.0], [3.0, 8.0, 5.0]], [[3.0, 8.0, 6.0], [7.0, 5.0, 5.0], [7.0, 6.0, 1.0]], [[2.0, 2.0, 8.0], [3.0, 7.0, 3.0], [5.0, 7.0, 
4.0]]], [[[2.0, 4.0, 5.0], [1.0, 8.0, 6.0], [8.0, 3.0, 2.0]], [[3.0, 5.0, 8.0], [7.0, 6.0, 2.0], [6.0, 6.0, 2.0]], [[1.0, 3.0, 
1.0], [7.0, 5.0, 4.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUCR(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 7.0], [5.0, 7.0, 6.0], [4.0, 4.0, 8.0]], [[5.0, 6.0, 3.0], [1.0, 3.0, 2.0], [6.0, 4.0, 
5.0]], [[3.0, 4.0, 4.0], [6.0, 8.0, 5.0], [2.0, 3.0, 5.0]]], [[[6.0, 3.0, 4.0], [5.0, 1.0, 2.0], [3.0, 5.0, 8.0]], [[7.0, 3.0, 
3.0], [6.0, 8.0, 5.0], [7.0, 8.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 6.0, 6.0], [8.0, 6.0, 6.0]]], [[[8.0, 3.0, 6.0], [6.0, 8.0, 
4.0], [1.0, 6.0, 4.0]], [[7.0, 6.0, 1.0], [2.0, 7.0, 6.0], [4.0, 5.0, 3.0]], [[2.0, 3.0, 2.0], [2.0, 2.0, 6.0], [4.0, 2.0, 
2.0]]], [[[8.0, 3.0, 3.0], [7.0, 2.0, 2.0], [1.0, 1.0, 6.0]], [[6.0, 1.0, 6.0], [8.0, 8.0, 4.0], [3.0, 2.0, 1.0]], [[6.0, 4.0, 
8.0], [6.0, 2.0, 7.0], [4.0, 5.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0], [7.0], [2.0]], [[3.0], [7.0], [3.0]]], [[[5.0], [8.0], [7.0]], [[5.0], [4.0], 
[8.0]]]])+(1.-msk_val)*numpy.array([[[[5.0], [6.0], [6.0]], [[7.0], [6.0], [2.0]]], [[[1.0], [5.0], [6.0]], [[7.0], [4.0], 
[7.0]]]])
      arg[1:3,:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 7.0], [5.0, 7.0, 6.0], [4.0, 4.0, 8.0]], [[5.0, 6.0, 3.0], [1.0, 3.0, 2.0], [6.0, 
4.0, 5.0]], [[3.0, 4.0, 4.0], [6.0, 8.0, 5.0], [2.0, 3.0, 5.0]]], [[[6.0, 4.0, 4.0], [5.0, 7.0, 2.0], [3.0, 2.0, 8.0]], [[7.0, 
3.0, 3.0], [6.0, 7.0, 5.0], [7.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 6.0, 6.0], [8.0, 6.0, 6.0]]], [[[8.0, 5.0, 6.0], [6.0, 
8.0, 4.0], [1.0, 7.0, 4.0]], [[7.0, 5.0, 1.0], [2.0, 4.0, 6.0], [4.0, 8.0, 3.0]], [[2.0, 3.0, 2.0], [2.0, 2.0, 6.0], [4.0, 2.0, 
2.0]]], [[[8.0, 3.0, 3.0], [7.0, 2.0, 2.0], [1.0, 1.0, 6.0]], [[6.0, 1.0, 6.0], [8.0, 8.0, 4.0], [3.0, 2.0, 1.0]], [[6.0, 4.0, 
8.0], [6.0, 2.0, 7.0], [4.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 7.0], [5.0, 7.0, 6.0], [4.0, 4.0, 8.0]], 
[[5.0, 6.0, 3.0], [1.0, 3.0, 2.0], [6.0, 4.0, 5.0]], [[3.0, 4.0, 4.0], [6.0, 8.0, 5.0], [2.0, 3.0, 5.0]]], [[[6.0, 5.0, 4.0], 
[5.0, 6.0, 2.0], [3.0, 6.0, 8.0]], [[7.0, 7.0, 3.0], [6.0, 6.0, 5.0], [7.0, 2.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 6.0, 6.0], 
[8.0, 6.0, 6.0]]], [[[8.0, 1.0, 6.0], [6.0, 5.0, 4.0], [1.0, 6.0, 4.0]], [[7.0, 7.0, 1.0], [2.0, 4.0, 6.0], [4.0, 7.0, 3.0]], 
[[2.0, 3.0, 2.0], [2.0, 2.0, 6.0], [4.0, 2.0, 2.0]]], [[[8.0, 3.0, 3.0], [7.0, 2.0, 2.0], [1.0, 1.0, 6.0]], [[6.0, 1.0, 6.0], 
[8.0, 8.0, 4.0], [3.0, 2.0, 1.0]], [[6.0, 4.0, 8.0], [6.0, 2.0, 7.0], [4.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCPP(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 4.0], [7.0, 7.0, 3.0], [6.0, 2.0, 2.0]], [[4.0, 1.0, 8.0], [3.0, 4.0, 5.0], [8.0, 4.0, 
5.0]], [[4.0, 1.0, 1.0], [4.0, 8.0, 2.0], [6.0, 3.0, 2.0]]], [[[1.0, 8.0, 2.0], [8.0, 8.0, 2.0], [5.0, 7.0, 1.0]], [[2.0, 6.0, 
7.0], [1.0, 1.0, 4.0], [8.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [6.0, 7.0, 5.0], [7.0, 4.0, 4.0]]], [[[2.0, 6.0, 1.0], [7.0, 6.0, 
4.0], [4.0, 8.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 3.0, 3.0], [8.0, 6.0, 5.0]], [[3.0, 8.0, 4.0], [8.0, 1.0, 1.0], [8.0, 5.0, 
1.0]]], [[[4.0, 8.0, 7.0], [6.0, 5.0, 7.0], [1.0, 1.0, 6.0]], [[7.0, 3.0, 6.0], [4.0, 6.0, 7.0], [5.0, 4.0, 1.0]], [[6.0, 8.0, 
5.0], [2.0, 8.0, 2.0], [7.0, 3.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[3.0, 4.0, 5.0], [7.0, 5.0, 4.0], [1.0, 4.0, 8.0]])
      arg[:3,:,1,1]=val
      ref=Data(numpy.array([[[[1.0, 6.0, 4.0], [7.0, 3.0, 3.0], [6.0, 2.0, 2.0]], [[4.0, 1.0, 8.0], [3.0, 4.0, 5.0], [8.0, 4.0, 
5.0]], [[4.0, 1.0, 1.0], [4.0, 5.0, 2.0], [6.0, 3.0, 2.0]]], [[[1.0, 8.0, 2.0], [8.0, 7.0, 2.0], [5.0, 7.0, 1.0]], [[2.0, 6.0, 
7.0], [1.0, 5.0, 4.0], [8.0, 3.0, 6.0]], [[4.0, 8.0, 7.0], [6.0, 4.0, 5.0], [7.0, 4.0, 4.0]]], [[[2.0, 6.0, 1.0], [7.0, 1.0, 
4.0], [4.0, 8.0, 2.0]], [[5.0, 5.0, 1.0], [6.0, 4.0, 3.0], [8.0, 6.0, 5.0]], [[3.0, 8.0, 4.0], [8.0, 8.0, 1.0], [8.0, 5.0, 
1.0]]], [[[4.0, 8.0, 7.0], [6.0, 5.0, 7.0], [1.0, 1.0, 6.0]], [[7.0, 3.0, 6.0], [4.0, 6.0, 7.0], [5.0, 4.0, 1.0]], [[6.0, 8.0, 
5.0], [2.0, 8.0, 2.0], [7.0, 3.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCPP(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 6.0], [3.0, 4.0, 6.0], [4.0, 7.0, 6.0]], [[8.0, 3.0, 8.0], [6.0, 3.0, 3.0], [8.0, 6.0, 
4.0]], [[7.0, 7.0, 8.0], [3.0, 3.0, 4.0], [5.0, 4.0, 1.0]]], [[[2.0, 8.0, 4.0], [8.0, 6.0, 3.0], [2.0, 5.0, 5.0]], [[5.0, 4.0, 
6.0], [7.0, 1.0, 3.0], [2.0, 7.0, 3.0]], [[1.0, 1.0, 3.0], [7.0, 4.0, 8.0], [7.0, 1.0, 1.0]]], [[[2.0, 7.0, 1.0], [3.0, 8.0, 
4.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 5.0], [8.0, 2.0, 5.0], [2.0, 4.0, 8.0]], [[6.0, 3.0, 1.0], [8.0, 3.0, 6.0], [8.0, 2.0, 
7.0]]], [[[6.0, 5.0, 6.0], [1.0, 1.0, 7.0], [1.0, 4.0, 4.0]], [[5.0, 8.0, 7.0], [7.0, 5.0, 5.0], [8.0, 6.0, 3.0]], [[4.0, 8.0, 
8.0], [2.0, 3.0, 8.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 2.0, 7.0], [1.0, 3.0, 3.0], [4.0, 7.0, 6.0]]),self.functionspace)
      arg[:3,:,1,1]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 6.0], [3.0, 8.0, 6.0], [4.0, 7.0, 6.0]], [[8.0, 3.0, 8.0], [6.0, 2.0, 3.0], [8.0, 6.0, 
4.0]], [[7.0, 7.0, 8.0], [3.0, 7.0, 4.0], [5.0, 4.0, 1.0]]], [[[2.0, 8.0, 4.0], [8.0, 1.0, 3.0], [2.0, 5.0, 5.0]], [[5.0, 4.0, 
6.0], [7.0, 3.0, 3.0], [2.0, 7.0, 3.0]], [[1.0, 1.0, 3.0], [7.0, 3.0, 8.0], [7.0, 1.0, 1.0]]], [[[2.0, 7.0, 1.0], [3.0, 4.0, 
4.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 5.0], [8.0, 7.0, 5.0], [2.0, 4.0, 8.0]], [[6.0, 3.0, 1.0], [8.0, 6.0, 6.0], [8.0, 2.0, 
7.0]]], [[[6.0, 5.0, 6.0], [1.0, 1.0, 7.0], [1.0, 4.0, 4.0]], [[5.0, 8.0, 7.0], [7.0, 5.0, 5.0], [8.0, 6.0, 3.0]], [[4.0, 8.0, 
8.0], [2.0, 3.0, 8.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCPP(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 5.0], [4.0, 1.0, 2.0], [2.0, 5.0, 3.0]], [[2.0, 5.0, 8.0], [5.0, 8.0, 3.0], [7.0, 7.0, 
7.0]], [[6.0, 1.0, 8.0], [1.0, 3.0, 2.0], [2.0, 4.0, 6.0]]], [[[6.0, 4.0, 4.0], [7.0, 1.0, 1.0], [8.0, 5.0, 3.0]], [[5.0, 8.0, 
6.0], [8.0, 3.0, 2.0], [4.0, 5.0, 4.0]], [[5.0, 6.0, 2.0], [5.0, 6.0, 3.0], [7.0, 2.0, 4.0]]], [[[5.0, 8.0, 5.0], [6.0, 5.0, 
1.0], [5.0, 4.0, 6.0]], [[7.0, 5.0, 4.0], [8.0, 2.0, 7.0], [8.0, 3.0, 6.0]], [[4.0, 7.0, 4.0], [4.0, 1.0, 1.0], [8.0, 4.0, 
3.0]]], [[[2.0, 1.0, 5.0], [3.0, 4.0, 7.0], [5.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], [1.0, 7.0, 5.0], [3.0, 8.0, 8.0]], [[4.0, 6.0, 
6.0], [5.0, 6.0, 4.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 7.0, 3.0], [5.0, 3.0, 5.0], [2.0, 7.0, 6.0]])+(1.-msk_val)*numpy.array([[4.0, 5.0, 3.0], 
[6.0, 2.0, 4.0], [8.0, 4.0, 3.0]])
      arg[:3,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 5.0], [4.0, 7.0, 2.0], [2.0, 5.0, 3.0]], [[2.0, 5.0, 8.0], [5.0, 7.0, 3.0], [7.0, 
7.0, 7.0]], [[6.0, 1.0, 8.0], [1.0, 3.0, 2.0], [2.0, 4.0, 6.0]]], [[[6.0, 4.0, 4.0], [7.0, 5.0, 1.0], [8.0, 5.0, 3.0]], [[5.0, 
8.0, 6.0], [8.0, 3.0, 2.0], [4.0, 5.0, 4.0]], [[5.0, 6.0, 2.0], [5.0, 5.0, 3.0], [7.0, 2.0, 4.0]]], [[[5.0, 8.0, 5.0], [6.0, 
2.0, 1.0], [5.0, 4.0, 6.0]], [[7.0, 5.0, 4.0], [8.0, 7.0, 7.0], [8.0, 3.0, 6.0]], [[4.0, 7.0, 4.0], [4.0, 6.0, 1.0], [8.0, 4.0, 
3.0]]], [[[2.0, 1.0, 5.0], [3.0, 4.0, 7.0], [5.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], [1.0, 7.0, 5.0], [3.0, 8.0, 8.0]], [[4.0, 6.0, 
6.0], [5.0, 6.0, 4.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 5.0], [4.0, 4.0, 2.0], [2.0, 5.0, 3.0]], 
[[2.0, 5.0, 8.0], [5.0, 5.0, 3.0], [7.0, 7.0, 7.0]], [[6.0, 1.0, 8.0], [1.0, 3.0, 2.0], [2.0, 4.0, 6.0]]], [[[6.0, 4.0, 4.0], 
[7.0, 6.0, 1.0], [8.0, 5.0, 3.0]], [[5.0, 8.0, 6.0], [8.0, 2.0, 2.0], [4.0, 5.0, 4.0]], [[5.0, 6.0, 2.0], [5.0, 4.0, 3.0], 
[7.0, 2.0, 4.0]]], [[[5.0, 8.0, 5.0], [6.0, 8.0, 1.0], [5.0, 4.0, 6.0]], [[7.0, 5.0, 4.0], [8.0, 4.0, 7.0], [8.0, 3.0, 6.0]], 
[[4.0, 7.0, 4.0], [4.0, 3.0, 1.0], [8.0, 4.0, 3.0]]], [[[2.0, 1.0, 5.0], [3.0, 4.0, 7.0], [5.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], 
[1.0, 7.0, 5.0], [3.0, 8.0, 8.0]], [[4.0, 6.0, 6.0], [5.0, 6.0, 4.0], [2.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCRC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 3.0], [6.0, 1.0, 8.0], [3.0, 1.0, 5.0]], [[7.0, 6.0, 7.0], [8.0, 3.0, 8.0], [1.0, 1.0, 
2.0]], [[8.0, 5.0, 4.0], [8.0, 3.0, 7.0], [2.0, 5.0, 2.0]]], [[[7.0, 6.0, 1.0], [6.0, 8.0, 1.0], [6.0, 4.0, 4.0]], [[6.0, 5.0, 
1.0], [4.0, 2.0, 7.0], [6.0, 2.0, 7.0]], [[2.0, 1.0, 1.0], [3.0, 5.0, 2.0], [6.0, 1.0, 5.0]]], [[[4.0, 8.0, 8.0], [2.0, 6.0, 
5.0], [6.0, 1.0, 8.0]], [[1.0, 4.0, 2.0], [3.0, 8.0, 2.0], [2.0, 2.0, 6.0]], [[2.0, 1.0, 4.0], [6.0, 3.0, 1.0], [4.0, 5.0, 
5.0]]], [[[8.0, 3.0, 3.0], [2.0, 1.0, 1.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 3.0], [8.0, 7.0, 2.0], [1.0, 4.0, 3.0]], [[1.0, 7.0, 
3.0], [2.0, 2.0, 2.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 2.0, 7.0]], [[1.0, 6.0, 2.0]], [[2.0, 6.0, 7.0]]], [[[1.0, 8.0, 3.0]], [[5.0, 8.0, 7.0]], [[6.0, 
1.0, 8.0]]], [[[6.0, 7.0, 8.0]], [[7.0, 2.0, 4.0]], [[4.0, 7.0, 8.0]]], [[[4.0, 3.0, 7.0]], [[4.0, 2.0, 1.0]], [[6.0, 4.0, 
8.0]]]])
      arg[:,:,1:2,:]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 3.0], [6.0, 2.0, 7.0], [3.0, 1.0, 5.0]], [[7.0, 6.0, 7.0], [1.0, 6.0, 2.0], [1.0, 1.0, 
2.0]], [[8.0, 5.0, 4.0], [2.0, 6.0, 7.0], [2.0, 5.0, 2.0]]], [[[7.0, 6.0, 1.0], [1.0, 8.0, 3.0], [6.0, 4.0, 4.0]], [[6.0, 5.0, 
1.0], [5.0, 8.0, 7.0], [6.0, 2.0, 7.0]], [[2.0, 1.0, 1.0], [6.0, 1.0, 8.0], [6.0, 1.0, 5.0]]], [[[4.0, 8.0, 8.0], [6.0, 7.0, 
8.0], [6.0, 1.0, 8.0]], [[1.0, 4.0, 2.0], [7.0, 2.0, 4.0], [2.0, 2.0, 6.0]], [[2.0, 1.0, 4.0], [4.0, 7.0, 8.0], [4.0, 5.0, 
5.0]]], [[[8.0, 3.0, 3.0], [4.0, 3.0, 7.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 3.0], [4.0, 2.0, 1.0], [1.0, 4.0, 3.0]], [[1.0, 7.0, 
3.0], [6.0, 4.0, 8.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCRC(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 7.0], [7.0, 7.0, 4.0], [5.0, 3.0, 4.0]], [[2.0, 3.0, 8.0], [5.0, 3.0, 7.0], [6.0, 6.0, 
7.0]], [[6.0, 2.0, 1.0], [3.0, 7.0, 6.0], [2.0, 2.0, 1.0]]], [[[7.0, 1.0, 5.0], [2.0, 5.0, 7.0], [5.0, 7.0, 7.0]], [[2.0, 6.0, 
5.0], [7.0, 5.0, 6.0], [4.0, 4.0, 7.0]], [[8.0, 8.0, 1.0], [3.0, 3.0, 2.0], [6.0, 8.0, 5.0]]], [[[2.0, 5.0, 6.0], [8.0, 1.0, 
7.0], [3.0, 3.0, 6.0]], [[2.0, 8.0, 1.0], [6.0, 4.0, 1.0], [8.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [1.0, 4.0, 2.0], [6.0, 7.0, 
4.0]]], [[[6.0, 4.0, 2.0], [3.0, 3.0, 4.0], [7.0, 4.0, 1.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 3.0], [5.0, 3.0, 6.0]], [[4.0, 8.0, 
4.0], [5.0, 7.0, 2.0], [5.0, 4.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 6.0, 3.0]], [[2.0, 6.0, 3.0]], [[6.0, 2.0, 2.0]]], [[[6.0, 1.0, 7.0]], [[4.0, 7.0, 1.0]], 
[[2.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0]], [[2.0, 2.0, 7.0]], [[1.0, 7.0, 5.0]]], [[[8.0, 1.0, 7.0]], [[8.0, 4.0, 7.0]], [[4.0, 
2.0, 5.0]]]]),self.functionspace)
      arg[:,:,1:2,:]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 7.0], [4.0, 6.0, 3.0], [5.0, 3.0, 4.0]], [[2.0, 3.0, 8.0], [2.0, 6.0, 3.0], [6.0, 6.0, 
7.0]], [[6.0, 2.0, 1.0], [6.0, 2.0, 2.0], [2.0, 2.0, 1.0]]], [[[7.0, 1.0, 5.0], [6.0, 1.0, 7.0], [5.0, 7.0, 7.0]], [[2.0, 6.0, 
5.0], [4.0, 7.0, 1.0], [4.0, 4.0, 7.0]], [[8.0, 8.0, 1.0], [2.0, 6.0, 1.0], [6.0, 8.0, 5.0]]], [[[2.0, 5.0, 6.0], [5.0, 7.0, 
7.0], [3.0, 3.0, 6.0]], [[2.0, 8.0, 1.0], [2.0, 2.0, 7.0], [8.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [1.0, 7.0, 5.0], [6.0, 7.0, 
4.0]]], [[[6.0, 4.0, 2.0], [8.0, 1.0, 7.0], [7.0, 4.0, 1.0]], [[6.0, 1.0, 2.0], [8.0, 4.0, 7.0], [5.0, 3.0, 6.0]], [[4.0, 8.0, 
4.0], [4.0, 2.0, 5.0], [5.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCRC(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 2.0], [6.0, 2.0, 2.0], [4.0, 5.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 1.0, 7.0], [6.0, 5.0, 
4.0]], [[4.0, 1.0, 5.0], [2.0, 4.0, 7.0], [7.0, 4.0, 5.0]]], [[[3.0, 8.0, 7.0], [7.0, 1.0, 2.0], [6.0, 8.0, 1.0]], [[3.0, 7.0, 
2.0], [1.0, 6.0, 3.0], [5.0, 5.0, 4.0]], [[6.0, 5.0, 1.0], [5.0, 2.0, 1.0], [7.0, 5.0, 4.0]]], [[[8.0, 2.0, 3.0], [4.0, 5.0, 
6.0], [2.0, 8.0, 1.0]], [[5.0, 4.0, 2.0], [3.0, 5.0, 8.0], [1.0, 3.0, 7.0]], [[4.0, 2.0, 3.0], [2.0, 3.0, 3.0], [5.0, 1.0, 
1.0]]], [[[8.0, 5.0, 4.0], [6.0, 7.0, 1.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 2.0], [4.0, 1.0, 6.0], [2.0, 6.0, 6.0]], [[2.0, 3.0, 
3.0], [3.0, 5.0, 1.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 2.0, 2.0]], [[3.0, 5.0, 5.0]], [[7.0, 7.0, 2.0]]], [[[4.0, 8.0, 6.0]], [[4.0, 7.0, 
8.0]], [[2.0, 5.0, 4.0]]], [[[8.0, 3.0, 6.0]], [[1.0, 6.0, 1.0]], [[1.0, 8.0, 3.0]]], [[[6.0, 4.0, 6.0]], [[5.0, 3.0, 4.0]], 
[[5.0, 2.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 2.0]], [[2.0, 1.0, 3.0]], [[5.0, 7.0, 6.0]]], [[[6.0, 2.0, 2.0]], 
[[6.0, 1.0, 1.0]], [[3.0, 3.0, 5.0]]], [[[3.0, 2.0, 4.0]], [[6.0, 2.0, 6.0]], [[5.0, 1.0, 5.0]]], [[[8.0, 2.0, 2.0]], [[5.0, 
1.0, 5.0]], [[4.0, 6.0, 4.0]]]])
      arg[:,:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 2.0], [5.0, 2.0, 2.0], [4.0, 5.0, 5.0]], [[2.0, 4.0, 8.0], [3.0, 5.0, 5.0], [6.0, 
5.0, 4.0]], [[4.0, 1.0, 5.0], [7.0, 7.0, 2.0], [7.0, 4.0, 5.0]]], [[[3.0, 8.0, 7.0], [4.0, 8.0, 6.0], [6.0, 8.0, 1.0]], [[3.0, 
7.0, 2.0], [4.0, 7.0, 8.0], [5.0, 5.0, 4.0]], [[6.0, 5.0, 1.0], [2.0, 5.0, 4.0], [7.0, 5.0, 4.0]]], [[[8.0, 2.0, 3.0], [8.0, 
3.0, 6.0], [2.0, 8.0, 1.0]], [[5.0, 4.0, 2.0], [1.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[4.0, 2.0, 3.0], [1.0, 8.0, 3.0], [5.0, 1.0, 
1.0]]], [[[8.0, 5.0, 4.0], [6.0, 4.0, 6.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 2.0], [5.0, 3.0, 4.0], [2.0, 6.0, 6.0]], [[2.0, 3.0, 
3.0], [5.0, 2.0, 3.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 2.0], [2.0, 5.0, 2.0], [4.0, 5.0, 5.0]], 
[[2.0, 4.0, 8.0], [2.0, 1.0, 3.0], [6.0, 5.0, 4.0]], [[4.0, 1.0, 5.0], [5.0, 7.0, 6.0], [7.0, 4.0, 5.0]]], [[[3.0, 8.0, 7.0], 
[6.0, 2.0, 2.0], [6.0, 8.0, 1.0]], [[3.0, 7.0, 2.0], [6.0, 1.0, 1.0], [5.0, 5.0, 4.0]], [[6.0, 5.0, 1.0], [3.0, 3.0, 5.0], 
[7.0, 5.0, 4.0]]], [[[8.0, 2.0, 3.0], [3.0, 2.0, 4.0], [2.0, 8.0, 1.0]], [[5.0, 4.0, 2.0], [6.0, 2.0, 6.0], [1.0, 3.0, 7.0]], 
[[4.0, 2.0, 3.0], [5.0, 1.0, 5.0], [5.0, 1.0, 1.0]]], [[[8.0, 5.0, 4.0], [8.0, 2.0, 2.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 2.0], 
[5.0, 1.0, 5.0], [2.0, 6.0, 6.0]], [[2.0, 3.0, 3.0], [4.0, 6.0, 4.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLPN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 7.0], [4.0, 1.0, 2.0], [3.0, 6.0, 4.0]], [[6.0, 2.0, 5.0], [4.0, 2.0, 8.0], [1.0, 4.0, 
1.0]], [[3.0, 4.0, 7.0], [8.0, 6.0, 3.0], [1.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0], [8.0, 4.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 3.0, 
6.0], [3.0, 7.0, 5.0], [4.0, 5.0, 3.0]], [[8.0, 5.0, 3.0], [8.0, 8.0, 1.0], [1.0, 5.0, 6.0]]], [[[2.0, 2.0, 1.0], [5.0, 6.0, 
6.0], [7.0, 4.0, 6.0]], [[2.0, 8.0, 8.0], [6.0, 1.0, 8.0], [7.0, 1.0, 7.0]], [[1.0, 7.0, 5.0], [6.0, 4.0, 5.0], [1.0, 7.0, 
4.0]]], [[[1.0, 5.0, 8.0], [3.0, 8.0, 5.0], [8.0, 3.0, 8.0]], [[1.0, 4.0, 3.0], [3.0, 6.0, 6.0], [5.0, 1.0, 5.0]], [[5.0, 6.0, 
8.0], [5.0, 8.0, 3.0], [4.0, 4.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[2.0, 2.0, 2.0]], [[4.0, 1.0, 5.0]]])
      arg[2:,2:,1]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 7.0], [4.0, 1.0, 2.0], [3.0, 6.0, 4.0]], [[6.0, 2.0, 5.0], [4.0, 2.0, 8.0], [1.0, 4.0, 
1.0]], [[3.0, 4.0, 7.0], [8.0, 6.0, 3.0], [1.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0], [8.0, 4.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 3.0, 
6.0], [3.0, 7.0, 5.0], [4.0, 5.0, 3.0]], [[8.0, 5.0, 3.0], [8.0, 8.0, 1.0], [1.0, 5.0, 6.0]]], [[[2.0, 2.0, 1.0], [5.0, 6.0, 
6.0], [7.0, 4.0, 6.0]], [[2.0, 8.0, 8.0], [6.0, 1.0, 8.0], [7.0, 1.0, 7.0]], [[1.0, 7.0, 5.0], [2.0, 2.0, 2.0], [1.0, 7.0, 
4.0]]], [[[1.0, 5.0, 8.0], [3.0, 8.0, 5.0], [8.0, 3.0, 8.0]], [[1.0, 4.0, 3.0], [3.0, 6.0, 6.0], [5.0, 1.0, 5.0]], [[5.0, 6.0, 
8.0], [4.0, 1.0, 5.0], [4.0, 4.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLPN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 4.0], [5.0, 5.0, 3.0], [4.0, 7.0, 6.0]], [[2.0, 4.0, 5.0], [8.0, 7.0, 7.0], [1.0, 4.0, 
8.0]], [[8.0, 7.0, 3.0], [5.0, 6.0, 8.0], [3.0, 7.0, 2.0]]], [[[5.0, 4.0, 6.0], [8.0, 2.0, 8.0], [5.0, 8.0, 1.0]], [[8.0, 3.0, 
6.0], [4.0, 5.0, 2.0], [5.0, 5.0, 1.0]], [[6.0, 3.0, 5.0], [3.0, 5.0, 2.0], [1.0, 6.0, 2.0]]], [[[3.0, 4.0, 8.0], [3.0, 6.0, 
8.0], [8.0, 3.0, 8.0]], [[5.0, 5.0, 2.0], [2.0, 1.0, 2.0], [5.0, 5.0, 1.0]], [[8.0, 7.0, 5.0], [2.0, 1.0, 6.0], [7.0, 3.0, 
2.0]]], [[[7.0, 5.0, 6.0], [7.0, 4.0, 3.0], [6.0, 5.0, 1.0]], [[5.0, 4.0, 2.0], [4.0, 4.0, 5.0], [1.0, 4.0, 3.0]], [[2.0, 7.0, 
6.0], [3.0, 3.0, 8.0], [5.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 8.0, 6.0]], [[2.0, 4.0, 8.0]]]),self.functionspace)
      arg[2:,2:,1]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 4.0], [5.0, 5.0, 3.0], [4.0, 7.0, 6.0]], [[2.0, 4.0, 5.0], [8.0, 7.0, 7.0], [1.0, 4.0, 
8.0]], [[8.0, 7.0, 3.0], [5.0, 6.0, 8.0], [3.0, 7.0, 2.0]]], [[[5.0, 4.0, 6.0], [8.0, 2.0, 8.0], [5.0, 8.0, 1.0]], [[8.0, 3.0, 
6.0], [4.0, 5.0, 2.0], [5.0, 5.0, 1.0]], [[6.0, 3.0, 5.0], [3.0, 5.0, 2.0], [1.0, 6.0, 2.0]]], [[[3.0, 4.0, 8.0], [3.0, 6.0, 
8.0], [8.0, 3.0, 8.0]], [[5.0, 5.0, 2.0], [2.0, 1.0, 2.0], [5.0, 5.0, 1.0]], [[8.0, 7.0, 5.0], [3.0, 8.0, 6.0], [7.0, 3.0, 
2.0]]], [[[7.0, 5.0, 6.0], [7.0, 4.0, 3.0], [6.0, 5.0, 1.0]], [[5.0, 4.0, 2.0], [4.0, 4.0, 5.0], [1.0, 4.0, 3.0]], [[2.0, 7.0, 
6.0], [2.0, 4.0, 8.0], [5.0, 4.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLPN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 1.0], [2.0, 4.0, 7.0], [3.0, 1.0, 3.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 4.0], [2.0, 6.0, 
1.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 7.0], [1.0, 3.0, 1.0]]], [[[8.0, 8.0, 8.0], [6.0, 7.0, 1.0], [1.0, 5.0, 8.0]], [[2.0, 6.0, 
7.0], [2.0, 7.0, 7.0], [1.0, 4.0, 6.0]], [[1.0, 7.0, 3.0], [2.0, 1.0, 2.0], [5.0, 7.0, 7.0]]], [[[2.0, 1.0, 2.0], [3.0, 1.0, 
5.0], [2.0, 5.0, 3.0]], [[7.0, 5.0, 1.0], [8.0, 5.0, 2.0], [4.0, 1.0, 1.0]], [[2.0, 1.0, 8.0], [8.0, 6.0, 2.0], [8.0, 7.0, 
7.0]]], [[[8.0, 4.0, 7.0], [6.0, 4.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 2.0], [5.0, 2.0, 7.0], [4.0, 6.0, 8.0]], [[6.0, 6.0, 
3.0], [3.0, 3.0, 1.0], [6.0, 6.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 1.0, 5.0]], [[4.0, 1.0, 6.0]]])+(1.-msk_val)*numpy.array([[[4.0, 1.0, 7.0]], [[1.0, 4.0, 
2.0]]])
      arg[2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 1.0], [2.0, 4.0, 7.0], [3.0, 1.0, 3.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 4.0], [2.0, 
6.0, 1.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 7.0], [1.0, 3.0, 1.0]]], [[[8.0, 8.0, 8.0], [6.0, 7.0, 1.0], [1.0, 5.0, 8.0]], [[2.0, 
6.0, 7.0], [2.0, 7.0, 7.0], [1.0, 4.0, 6.0]], [[1.0, 7.0, 3.0], [2.0, 1.0, 2.0], [5.0, 7.0, 7.0]]], [[[2.0, 1.0, 2.0], [3.0, 
1.0, 5.0], [2.0, 5.0, 3.0]], [[7.0, 5.0, 1.0], [8.0, 5.0, 2.0], [4.0, 1.0, 1.0]], [[2.0, 1.0, 8.0], [7.0, 1.0, 5.0], [8.0, 7.0, 
7.0]]], [[[8.0, 4.0, 7.0], [6.0, 4.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 2.0], [5.0, 2.0, 7.0], [4.0, 6.0, 8.0]], [[6.0, 6.0, 
3.0], [4.0, 1.0, 6.0], [6.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [2.0, 4.0, 7.0], [3.0, 1.0, 3.0]], 
[[4.0, 4.0, 6.0], [1.0, 8.0, 4.0], [2.0, 6.0, 1.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 7.0], [1.0, 3.0, 1.0]]], [[[8.0, 8.0, 8.0], 
[6.0, 7.0, 1.0], [1.0, 5.0, 8.0]], [[2.0, 6.0, 7.0], [2.0, 7.0, 7.0], [1.0, 4.0, 6.0]], [[1.0, 7.0, 3.0], [2.0, 1.0, 2.0], 
[5.0, 7.0, 7.0]]], [[[2.0, 1.0, 2.0], [3.0, 1.0, 5.0], [2.0, 5.0, 3.0]], [[7.0, 5.0, 1.0], [8.0, 5.0, 2.0], [4.0, 1.0, 1.0]], 
[[2.0, 1.0, 8.0], [4.0, 1.0, 7.0], [8.0, 7.0, 7.0]]], [[[8.0, 4.0, 7.0], [6.0, 4.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 2.0], 
[5.0, 2.0, 7.0], [4.0, 6.0, 8.0]], [[6.0, 6.0, 3.0], [1.0, 4.0, 2.0], [6.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRUC(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 2.0], [8.0, 1.0, 4.0], [7.0, 3.0, 3.0]], [[1.0, 2.0, 6.0], [7.0, 4.0, 2.0], [3.0, 5.0, 
7.0]], [[2.0, 3.0, 3.0], [3.0, 1.0, 1.0], [7.0, 2.0, 2.0]]], [[[5.0, 7.0, 7.0], [6.0, 3.0, 5.0], [2.0, 4.0, 2.0]], [[4.0, 5.0, 
4.0], [5.0, 2.0, 3.0], [8.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [2.0, 8.0, 3.0], [7.0, 3.0, 1.0]]], [[[1.0, 4.0, 1.0], [6.0, 6.0, 
7.0], [6.0, 8.0, 6.0]], [[6.0, 6.0, 4.0], [2.0, 4.0, 6.0], [3.0, 4.0, 1.0]], [[4.0, 7.0, 2.0], [4.0, 6.0, 5.0], [4.0, 3.0, 
1.0]]], [[[3.0, 4.0, 1.0], [7.0, 3.0, 3.0], [2.0, 7.0, 8.0]], [[6.0, 1.0, 2.0], [6.0, 5.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 4.0, 
8.0], [5.0, 6.0, 5.0], [3.0, 1.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 3.0, 7.0], [8.0, 2.0, 8.0]]], [[[8.0, 4.0, 2.0], [6.0, 2.0, 4.0]]]])
      arg[2:,1:2,:2,:]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 2.0], [8.0, 1.0, 4.0], [7.0, 3.0, 3.0]], [[1.0, 2.0, 6.0], [7.0, 4.0, 2.0], [3.0, 5.0, 
7.0]], [[2.0, 3.0, 3.0], [3.0, 1.0, 1.0], [7.0, 2.0, 2.0]]], [[[5.0, 7.0, 7.0], [6.0, 3.0, 5.0], [2.0, 4.0, 2.0]], [[4.0, 5.0, 
4.0], [5.0, 2.0, 3.0], [8.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [2.0, 8.0, 3.0], [7.0, 3.0, 1.0]]], [[[1.0, 4.0, 1.0], [6.0, 6.0, 
7.0], [6.0, 8.0, 6.0]], [[7.0, 3.0, 7.0], [8.0, 2.0, 8.0], [3.0, 4.0, 1.0]], [[4.0, 7.0, 2.0], [4.0, 6.0, 5.0], [4.0, 3.0, 
1.0]]], [[[3.0, 4.0, 1.0], [7.0, 3.0, 3.0], [2.0, 7.0, 8.0]], [[8.0, 4.0, 2.0], [6.0, 2.0, 4.0], [7.0, 5.0, 4.0]], [[3.0, 4.0, 
8.0], [5.0, 6.0, 5.0], [3.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRUC(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [3.0, 1.0, 5.0], [8.0, 2.0, 2.0]], [[2.0, 2.0, 7.0], [2.0, 1.0, 2.0], [6.0, 3.0, 
4.0]], [[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [2.0, 6.0, 8.0]]], [[[4.0, 7.0, 6.0], [7.0, 6.0, 2.0], [6.0, 6.0, 3.0]], [[7.0, 7.0, 
4.0], [8.0, 5.0, 7.0], [4.0, 7.0, 3.0]], [[6.0, 5.0, 3.0], [2.0, 2.0, 2.0], [6.0, 3.0, 1.0]]], [[[7.0, 1.0, 8.0], [1.0, 5.0, 
6.0], [2.0, 5.0, 8.0]], [[6.0, 5.0, 7.0], [4.0, 3.0, 5.0], [3.0, 4.0, 2.0]], [[1.0, 8.0, 2.0], [2.0, 4.0, 3.0], [5.0, 7.0, 
4.0]]], [[[4.0, 4.0, 3.0], [8.0, 1.0, 3.0], [8.0, 7.0, 7.0]], [[7.0, 1.0, 5.0], [3.0, 8.0, 2.0], [6.0, 2.0, 4.0]], [[7.0, 7.0, 
4.0], [4.0, 8.0, 7.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 8.0, 5.0], [2.0, 8.0, 4.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 2.0]]]]),self.functionspace)
      arg[2:,1:2,:2,:]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 5.0], [3.0, 1.0, 5.0], [8.0, 2.0, 2.0]], [[2.0, 2.0, 7.0], [2.0, 1.0, 2.0], [6.0, 3.0, 
4.0]], [[5.0, 5.0, 6.0], [8.0, 4.0, 4.0], [2.0, 6.0, 8.0]]], [[[4.0, 7.0, 6.0], [7.0, 6.0, 2.0], [6.0, 6.0, 3.0]], [[7.0, 7.0, 
4.0], [8.0, 5.0, 7.0], [4.0, 7.0, 3.0]], [[6.0, 5.0, 3.0], [2.0, 2.0, 2.0], [6.0, 3.0, 1.0]]], [[[7.0, 1.0, 8.0], [1.0, 5.0, 
6.0], [2.0, 5.0, 8.0]], [[1.0, 8.0, 5.0], [2.0, 8.0, 4.0], [3.0, 4.0, 2.0]], [[1.0, 8.0, 2.0], [2.0, 4.0, 3.0], [5.0, 7.0, 
4.0]]], [[[4.0, 4.0, 3.0], [8.0, 1.0, 3.0], [8.0, 7.0, 7.0]], [[2.0, 5.0, 5.0], [5.0, 2.0, 2.0], [6.0, 2.0, 4.0]], [[7.0, 7.0, 
4.0], [4.0, 8.0, 7.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRUC(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 8.0], [4.0, 6.0, 2.0], [6.0, 4.0, 2.0]], [[5.0, 1.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 
5.0]], [[1.0, 7.0, 6.0], [8.0, 6.0, 6.0], [8.0, 1.0, 5.0]]], [[[7.0, 1.0, 5.0], [7.0, 2.0, 4.0], [2.0, 2.0, 4.0]], [[1.0, 4.0, 
7.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]], [[4.0, 8.0, 5.0], [4.0, 6.0, 4.0], [3.0, 6.0, 5.0]]], [[[7.0, 7.0, 3.0], [5.0, 5.0, 
2.0], [3.0, 1.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 8.0, 8.0], [3.0, 7.0, 3.0]], [[5.0, 7.0, 8.0], [4.0, 8.0, 2.0], [7.0, 4.0, 
3.0]]], [[[8.0, 2.0, 6.0], [6.0, 7.0, 7.0], [6.0, 7.0, 5.0]], [[6.0, 3.0, 3.0], [7.0, 6.0, 7.0], [1.0, 8.0, 4.0]], [[5.0, 1.0, 
4.0], [7.0, 5.0, 8.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 8.0, 1.0], [7.0, 3.0, 6.0]]], [[[6.0, 6.0, 4.0], [3.0, 6.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 4.0, 4.0], [1.0, 6.0, 2.0]]], [[[4.0, 4.0, 8.0], [7.0, 4.0, 3.0]]]])
      arg[2:,1:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 8.0], [4.0, 6.0, 2.0], [6.0, 4.0, 2.0]], [[5.0, 1.0, 6.0], [2.0, 6.0, 8.0], [3.0, 
1.0, 5.0]], [[1.0, 7.0, 6.0], [8.0, 6.0, 6.0], [8.0, 1.0, 5.0]]], [[[7.0, 1.0, 5.0], [7.0, 2.0, 4.0], [2.0, 2.0, 4.0]], [[1.0, 
4.0, 7.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]], [[4.0, 8.0, 5.0], [4.0, 6.0, 4.0], [3.0, 6.0, 5.0]]], [[[7.0, 7.0, 3.0], [5.0, 
5.0, 2.0], [3.0, 1.0, 2.0]], [[5.0, 8.0, 1.0], [7.0, 3.0, 6.0], [3.0, 7.0, 3.0]], [[5.0, 7.0, 8.0], [4.0, 8.0, 2.0], [7.0, 4.0, 
3.0]]], [[[8.0, 2.0, 6.0], [6.0, 7.0, 7.0], [6.0, 7.0, 5.0]], [[6.0, 6.0, 4.0], [3.0, 6.0, 5.0], [1.0, 8.0, 4.0]], [[5.0, 1.0, 
4.0], [7.0, 5.0, 8.0], [4.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 8.0], [4.0, 6.0, 2.0], [6.0, 4.0, 2.0]], 
[[5.0, 1.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[1.0, 7.0, 6.0], [8.0, 6.0, 6.0], [8.0, 1.0, 5.0]]], [[[7.0, 1.0, 5.0], 
[7.0, 2.0, 4.0], [2.0, 2.0, 4.0]], [[1.0, 4.0, 7.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]], [[4.0, 8.0, 5.0], [4.0, 6.0, 4.0], 
[3.0, 6.0, 5.0]]], [[[7.0, 7.0, 3.0], [5.0, 5.0, 2.0], [3.0, 1.0, 2.0]], [[3.0, 4.0, 4.0], [1.0, 6.0, 2.0], [3.0, 7.0, 3.0]], 
[[5.0, 7.0, 8.0], [4.0, 8.0, 2.0], [7.0, 4.0, 3.0]]], [[[8.0, 2.0, 6.0], [6.0, 7.0, 7.0], [6.0, 7.0, 5.0]], [[4.0, 4.0, 8.0], 
[7.0, 4.0, 3.0], [1.0, 8.0, 4.0]], [[5.0, 1.0, 4.0], [7.0, 5.0, 8.0], [4.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCLP(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 5.0], [3.0, 2.0, 1.0], [6.0, 5.0, 6.0]], [[5.0, 5.0, 8.0], [1.0, 5.0, 1.0], [8.0, 1.0, 
3.0]], [[4.0, 1.0, 8.0], [5.0, 7.0, 4.0], [3.0, 2.0, 6.0]]], [[[2.0, 6.0, 1.0], [2.0, 7.0, 4.0], [4.0, 3.0, 2.0]], [[6.0, 2.0, 
7.0], [4.0, 2.0, 7.0], [5.0, 1.0, 4.0]], [[7.0, 7.0, 7.0], [6.0, 7.0, 7.0], [8.0, 5.0, 8.0]]], [[[1.0, 6.0, 1.0], [7.0, 3.0, 
4.0], [8.0, 4.0, 2.0]], [[1.0, 7.0, 4.0], [1.0, 8.0, 6.0], [1.0, 3.0, 3.0]], [[1.0, 1.0, 1.0], [7.0, 6.0, 7.0], [8.0, 5.0, 
2.0]]], [[[6.0, 4.0, 7.0], [8.0, 3.0, 2.0], [3.0, 7.0, 2.0]], [[3.0, 7.0, 4.0], [4.0, 4.0, 5.0], [7.0, 1.0, 5.0]], [[3.0, 4.0, 
1.0], [7.0, 2.0, 6.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[5.0], [5.0], [3.0]], [[1.0], [1.0], [6.0]], [[1.0], [4.0], [6.0]], [[5.0], [4.0], [1.0]]])
      arg[:,:,2:,1]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 5.0], [3.0, 2.0, 1.0], [6.0, 5.0, 6.0]], [[5.0, 5.0, 8.0], [1.0, 5.0, 1.0], [8.0, 5.0, 
3.0]], [[4.0, 1.0, 8.0], [5.0, 7.0, 4.0], [3.0, 3.0, 6.0]]], [[[2.0, 6.0, 1.0], [2.0, 7.0, 4.0], [4.0, 1.0, 2.0]], [[6.0, 2.0, 
7.0], [4.0, 2.0, 7.0], [5.0, 1.0, 4.0]], [[7.0, 7.0, 7.0], [6.0, 7.0, 7.0], [8.0, 6.0, 8.0]]], [[[1.0, 6.0, 1.0], [7.0, 3.0, 
4.0], [8.0, 1.0, 2.0]], [[1.0, 7.0, 4.0], [1.0, 8.0, 6.0], [1.0, 4.0, 3.0]], [[1.0, 1.0, 1.0], [7.0, 6.0, 7.0], [8.0, 6.0, 
2.0]]], [[[6.0, 4.0, 7.0], [8.0, 3.0, 2.0], [3.0, 5.0, 2.0]], [[3.0, 7.0, 4.0], [4.0, 4.0, 5.0], [7.0, 4.0, 5.0]], [[3.0, 4.0, 
1.0], [7.0, 2.0, 6.0], [7.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCLP(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 5.0], [7.0, 1.0, 5.0], [1.0, 3.0, 7.0]], [[2.0, 6.0, 4.0], [5.0, 1.0, 8.0], [8.0, 3.0, 
2.0]], [[8.0, 6.0, 8.0], [8.0, 8.0, 5.0], [4.0, 5.0, 8.0]]], [[[7.0, 7.0, 3.0], [1.0, 3.0, 1.0], [7.0, 3.0, 6.0]], [[3.0, 4.0, 
7.0], [4.0, 6.0, 8.0], [7.0, 7.0, 1.0]], [[1.0, 8.0, 8.0], [7.0, 4.0, 1.0], [5.0, 1.0, 8.0]]], [[[5.0, 4.0, 5.0], [4.0, 8.0, 
1.0], [1.0, 4.0, 7.0]], [[6.0, 5.0, 3.0], [6.0, 6.0, 5.0], [2.0, 6.0, 8.0]], [[4.0, 5.0, 5.0], [3.0, 7.0, 1.0], [7.0, 3.0, 
7.0]]], [[[1.0, 4.0, 3.0], [2.0, 6.0, 1.0], [6.0, 7.0, 1.0]], [[1.0, 5.0, 4.0], [7.0, 1.0, 3.0], [4.0, 1.0, 3.0]], [[3.0, 1.0, 
7.0], [1.0, 7.0, 5.0], [6.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0], [1.0], [2.0]], [[1.0], [5.0], [6.0]], [[2.0], [3.0], [4.0]], [[1.0], [4.0], 
[6.0]]]),self.functionspace)
      arg[:,:,2:,1]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 5.0], [7.0, 1.0, 5.0], [1.0, 5.0, 7.0]], [[2.0, 6.0, 4.0], [5.0, 1.0, 8.0], [8.0, 1.0, 
2.0]], [[8.0, 6.0, 8.0], [8.0, 8.0, 5.0], [4.0, 2.0, 8.0]]], [[[7.0, 7.0, 3.0], [1.0, 3.0, 1.0], [7.0, 1.0, 6.0]], [[3.0, 4.0, 
7.0], [4.0, 6.0, 8.0], [7.0, 5.0, 1.0]], [[1.0, 8.0, 8.0], [7.0, 4.0, 1.0], [5.0, 6.0, 8.0]]], [[[5.0, 4.0, 5.0], [4.0, 8.0, 
1.0], [1.0, 2.0, 7.0]], [[6.0, 5.0, 3.0], [6.0, 6.0, 5.0], [2.0, 3.0, 8.0]], [[4.0, 5.0, 5.0], [3.0, 7.0, 1.0], [7.0, 4.0, 
7.0]]], [[[1.0, 4.0, 3.0], [2.0, 6.0, 1.0], [6.0, 1.0, 1.0]], [[1.0, 5.0, 4.0], [7.0, 1.0, 3.0], [4.0, 4.0, 3.0]], [[3.0, 1.0, 
7.0], [1.0, 7.0, 5.0], [6.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCLP(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 3.0], [5.0, 2.0, 7.0], [7.0, 5.0, 7.0]], [[7.0, 7.0, 3.0], [2.0, 6.0, 7.0], [4.0, 8.0, 
4.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 6.0], [1.0, 4.0, 5.0]]], [[[2.0, 3.0, 1.0], [7.0, 7.0, 2.0], [8.0, 2.0, 8.0]], [[4.0, 4.0, 
6.0], [2.0, 6.0, 4.0], [4.0, 6.0, 7.0]], [[8.0, 3.0, 1.0], [6.0, 8.0, 1.0], [1.0, 5.0, 4.0]]], [[[5.0, 8.0, 7.0], [2.0, 4.0, 
8.0], [7.0, 7.0, 7.0]], [[4.0, 5.0, 2.0], [5.0, 7.0, 4.0], [3.0, 1.0, 1.0]], [[2.0, 4.0, 8.0], [8.0, 8.0, 2.0], [6.0, 2.0, 
7.0]]], [[[4.0, 6.0, 4.0], [6.0, 2.0, 8.0], [6.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [8.0, 5.0, 4.0], [3.0, 5.0, 6.0]], [[3.0, 5.0, 
3.0], [4.0, 6.0, 3.0], [7.0, 6.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0], [6.0], [8.0]], [[3.0], [8.0], [2.0]], [[1.0], [5.0], [5.0]], [[3.0], [1.0], 
[3.0]]])+(1.-msk_val)*numpy.array([[[7.0], [4.0], [5.0]], [[1.0], [4.0], [6.0]], [[5.0], [3.0], [5.0]], [[1.0], [2.0], [5.0]]])
      arg[:,:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 3.0], [5.0, 2.0, 7.0], [7.0, 7.0, 7.0]], [[7.0, 7.0, 3.0], [2.0, 6.0, 7.0], [4.0, 
6.0, 4.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 6.0], [1.0, 8.0, 5.0]]], [[[2.0, 3.0, 1.0], [7.0, 7.0, 2.0], [8.0, 3.0, 8.0]], [[4.0, 
4.0, 6.0], [2.0, 6.0, 4.0], [4.0, 8.0, 7.0]], [[8.0, 3.0, 1.0], [6.0, 8.0, 1.0], [1.0, 2.0, 4.0]]], [[[5.0, 8.0, 7.0], [2.0, 
4.0, 8.0], [7.0, 1.0, 7.0]], [[4.0, 5.0, 2.0], [5.0, 7.0, 4.0], [3.0, 5.0, 1.0]], [[2.0, 4.0, 8.0], [8.0, 8.0, 2.0], [6.0, 5.0, 
7.0]]], [[[4.0, 6.0, 4.0], [6.0, 2.0, 8.0], [6.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [8.0, 5.0, 4.0], [3.0, 1.0, 6.0]], [[3.0, 5.0, 
3.0], [4.0, 6.0, 3.0], [7.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [5.0, 2.0, 7.0], [7.0, 7.0, 7.0]], 
[[7.0, 7.0, 3.0], [2.0, 6.0, 7.0], [4.0, 4.0, 4.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 6.0], [1.0, 5.0, 5.0]]], [[[2.0, 3.0, 1.0], 
[7.0, 7.0, 2.0], [8.0, 1.0, 8.0]], [[4.0, 4.0, 6.0], [2.0, 6.0, 4.0], [4.0, 4.0, 7.0]], [[8.0, 3.0, 1.0], [6.0, 8.0, 1.0], 
[1.0, 6.0, 4.0]]], [[[5.0, 8.0, 7.0], [2.0, 4.0, 8.0], [7.0, 5.0, 7.0]], [[4.0, 5.0, 2.0], [5.0, 7.0, 4.0], [3.0, 3.0, 1.0]], 
[[2.0, 4.0, 8.0], [8.0, 8.0, 2.0], [6.0, 5.0, 7.0]]], [[[4.0, 6.0, 4.0], [6.0, 2.0, 8.0], [6.0, 1.0, 7.0]], [[8.0, 3.0, 3.0], 
[8.0, 5.0, 4.0], [3.0, 2.0, 6.0]], [[3.0, 5.0, 3.0], [4.0, 6.0, 3.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UUPC(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 7.0], [2.0, 1.0, 6.0], [8.0, 5.0, 2.0]], [[8.0, 1.0, 6.0], [8.0, 1.0, 6.0], [7.0, 8.0, 
7.0]], [[4.0, 6.0, 7.0], [4.0, 2.0, 5.0], [5.0, 7.0, 2.0]]], [[[5.0, 3.0, 5.0], [2.0, 4.0, 4.0], [4.0, 8.0, 3.0]], [[3.0, 1.0, 
2.0], [7.0, 2.0, 7.0], [3.0, 5.0, 8.0]], [[4.0, 4.0, 4.0], [2.0, 5.0, 3.0], [5.0, 5.0, 2.0]]], [[[6.0, 4.0, 2.0], [2.0, 6.0, 
8.0], [6.0, 5.0, 2.0]], [[3.0, 4.0, 6.0], [6.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[5.0, 2.0, 4.0], [7.0, 3.0, 1.0], [7.0, 2.0, 
2.0]]], [[[5.0, 7.0, 1.0], [8.0, 4.0, 8.0], [7.0, 5.0, 3.0]], [[5.0, 1.0, 4.0], [2.0, 5.0, 5.0], [2.0, 8.0, 1.0]], [[4.0, 2.0, 
1.0], [1.0, 1.0, 8.0], [5.0, 8.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 5.0, 1.0], [2.0, 4.0, 1.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 5.0]], [[6.0, 2.0, 4.0], [2.0, 5.0, 
3.0]]])
      arg[:3,:2,1,:]=val
      ref=Data(numpy.array([[[[1.0, 7.0, 7.0], [1.0, 5.0, 1.0], [8.0, 5.0, 2.0]], [[8.0, 1.0, 6.0], [2.0, 4.0, 1.0], [7.0, 8.0, 
7.0]], [[4.0, 6.0, 7.0], [4.0, 2.0, 5.0], [5.0, 7.0, 2.0]]], [[[5.0, 3.0, 5.0], [7.0, 2.0, 3.0], [4.0, 8.0, 3.0]], [[3.0, 1.0, 
2.0], [8.0, 4.0, 5.0], [3.0, 5.0, 8.0]], [[4.0, 4.0, 4.0], [2.0, 5.0, 3.0], [5.0, 5.0, 2.0]]], [[[6.0, 4.0, 2.0], [6.0, 2.0, 
4.0], [6.0, 5.0, 2.0]], [[3.0, 4.0, 6.0], [2.0, 5.0, 3.0], [4.0, 2.0, 4.0]], [[5.0, 2.0, 4.0], [7.0, 3.0, 1.0], [7.0, 2.0, 
2.0]]], [[[5.0, 7.0, 1.0], [8.0, 4.0, 8.0], [7.0, 5.0, 3.0]], [[5.0, 1.0, 4.0], [2.0, 5.0, 5.0], [2.0, 8.0, 1.0]], [[4.0, 2.0, 
1.0], [1.0, 1.0, 8.0], [5.0, 8.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UUPC(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [7.0, 4.0, 8.0], [7.0, 3.0, 1.0]], [[2.0, 4.0, 5.0], [5.0, 3.0, 1.0], [8.0, 1.0, 
1.0]], [[7.0, 5.0, 1.0], [3.0, 3.0, 1.0], [1.0, 2.0, 5.0]]], [[[7.0, 2.0, 6.0], [7.0, 5.0, 5.0], [3.0, 4.0, 3.0]], [[4.0, 7.0, 
1.0], [1.0, 7.0, 7.0], [4.0, 4.0, 2.0]], [[4.0, 2.0, 4.0], [3.0, 8.0, 4.0], [2.0, 6.0, 3.0]]], [[[7.0, 3.0, 1.0], [8.0, 4.0, 
1.0], [4.0, 8.0, 6.0]], [[8.0, 1.0, 8.0], [5.0, 8.0, 6.0], [5.0, 6.0, 8.0]], [[5.0, 6.0, 4.0], [2.0, 7.0, 6.0], [7.0, 3.0, 
6.0]]], [[[1.0, 6.0, 2.0], [7.0, 8.0, 1.0], [1.0, 6.0, 7.0]], [[1.0, 4.0, 7.0], [4.0, 2.0, 3.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 
3.0], [8.0, 2.0, 8.0], [3.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 1.0], [1.0, 8.0, 4.0]], [[8.0, 8.0, 6.0], [3.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [7.0, 
4.0, 2.0]]]),self.functionspace)
      arg[:3,:2,1,:]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 6.0], [4.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[2.0, 4.0, 5.0], [1.0, 8.0, 4.0], [8.0, 1.0, 
1.0]], [[7.0, 5.0, 1.0], [3.0, 3.0, 1.0], [1.0, 2.0, 5.0]]], [[[7.0, 2.0, 6.0], [8.0, 8.0, 6.0], [3.0, 4.0, 3.0]], [[4.0, 7.0, 
1.0], [3.0, 8.0, 4.0], [4.0, 4.0, 2.0]], [[4.0, 2.0, 4.0], [3.0, 8.0, 4.0], [2.0, 6.0, 3.0]]], [[[7.0, 3.0, 1.0], [4.0, 7.0, 
5.0], [4.0, 8.0, 6.0]], [[8.0, 1.0, 8.0], [7.0, 4.0, 2.0], [5.0, 6.0, 8.0]], [[5.0, 6.0, 4.0], [2.0, 7.0, 6.0], [7.0, 3.0, 
6.0]]], [[[1.0, 6.0, 2.0], [7.0, 8.0, 1.0], [1.0, 6.0, 7.0]], [[1.0, 4.0, 7.0], [4.0, 2.0, 3.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 
3.0], [8.0, 2.0, 8.0], [3.0, 1.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UUPC(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 3.0], [6.0, 3.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 4.0, 5.0], [1.0, 4.0, 6.0], [4.0, 8.0, 
6.0]], [[6.0, 1.0, 1.0], [3.0, 5.0, 2.0], [8.0, 2.0, 7.0]]], [[[8.0, 8.0, 6.0], [4.0, 6.0, 1.0], [1.0, 3.0, 1.0]], [[4.0, 2.0, 
2.0], [4.0, 8.0, 4.0], [5.0, 5.0, 8.0]], [[7.0, 2.0, 5.0], [3.0, 2.0, 8.0], [8.0, 1.0, 7.0]]], [[[2.0, 4.0, 3.0], [7.0, 4.0, 
3.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 2.0], [4.0, 6.0, 1.0], [5.0, 2.0, 6.0]], [[2.0, 7.0, 8.0], [8.0, 1.0, 2.0], [8.0, 2.0, 
6.0]]], [[[8.0, 4.0, 5.0], [7.0, 8.0, 8.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 2.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 1.0, 
1.0], [4.0, 6.0, 7.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 4.0, 7.0], [4.0, 5.0, 2.0]], [[3.0, 8.0, 6.0], [8.0, 6.0, 3.0]], [[7.0, 7.0, 6.0], [5.0, 
3.0, 6.0]]])+(1.-msk_val)*numpy.array([[[3.0, 5.0, 3.0], [4.0, 4.0, 3.0]], [[8.0, 5.0, 7.0], [3.0, 4.0, 4.0]], [[6.0, 8.0, 
6.0], [3.0, 1.0, 3.0]]])
      arg[:3,:2,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 3.0], [7.0, 4.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 4.0, 5.0], [4.0, 5.0, 2.0], [4.0, 
8.0, 6.0]], [[6.0, 1.0, 1.0], [3.0, 5.0, 2.0], [8.0, 2.0, 7.0]]], [[[8.0, 8.0, 6.0], [3.0, 8.0, 6.0], [1.0, 3.0, 1.0]], [[4.0, 
2.0, 2.0], [8.0, 6.0, 3.0], [5.0, 5.0, 8.0]], [[7.0, 2.0, 5.0], [3.0, 2.0, 8.0], [8.0, 1.0, 7.0]]], [[[2.0, 4.0, 3.0], [7.0, 
7.0, 6.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 2.0], [5.0, 3.0, 6.0], [5.0, 2.0, 6.0]], [[2.0, 7.0, 8.0], [8.0, 1.0, 2.0], [8.0, 2.0, 
6.0]]], [[[8.0, 4.0, 5.0], [7.0, 8.0, 8.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], [5.0, 2.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 1.0, 
1.0], [4.0, 6.0, 7.0], [2.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 3.0], [3.0, 5.0, 3.0], [7.0, 2.0, 5.0]], 
[[2.0, 4.0, 5.0], [4.0, 4.0, 3.0], [4.0, 8.0, 6.0]], [[6.0, 1.0, 1.0], [3.0, 5.0, 2.0], [8.0, 2.0, 7.0]]], [[[8.0, 8.0, 6.0], 
[8.0, 5.0, 7.0], [1.0, 3.0, 1.0]], [[4.0, 2.0, 2.0], [3.0, 4.0, 4.0], [5.0, 5.0, 8.0]], [[7.0, 2.0, 5.0], [3.0, 2.0, 8.0], 
[8.0, 1.0, 7.0]]], [[[2.0, 4.0, 3.0], [6.0, 8.0, 6.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 2.0], [3.0, 1.0, 3.0], [5.0, 2.0, 6.0]], 
[[2.0, 7.0, 8.0], [8.0, 1.0, 2.0], [8.0, 2.0, 6.0]]], [[[8.0, 4.0, 5.0], [7.0, 8.0, 8.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 2.0], 
[5.0, 2.0, 6.0], [6.0, 7.0, 3.0]], [[2.0, 1.0, 1.0], [4.0, 6.0, 7.0], [2.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCUL(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 4.0], [7.0, 3.0, 7.0], [4.0, 8.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 2.0, 5.0], [7.0, 3.0, 
3.0]], [[2.0, 5.0, 8.0], [4.0, 8.0, 6.0], [2.0, 2.0, 6.0]]], [[[1.0, 3.0, 2.0], [8.0, 2.0, 3.0], [1.0, 8.0, 5.0]], [[5.0, 4.0, 
8.0], [7.0, 3.0, 7.0], [8.0, 2.0, 3.0]], [[2.0, 4.0, 1.0], [6.0, 2.0, 6.0], [4.0, 8.0, 6.0]]], [[[7.0, 4.0, 3.0], [6.0, 3.0, 
5.0], [6.0, 4.0, 2.0]], [[1.0, 5.0, 4.0], [5.0, 5.0, 1.0], [2.0, 5.0, 8.0]], [[2.0, 5.0, 4.0], [2.0, 2.0, 7.0], [3.0, 3.0, 
7.0]]], [[[3.0, 7.0, 1.0], [2.0, 7.0, 7.0], [7.0, 5.0, 1.0]], [[1.0, 8.0, 6.0], [1.0, 5.0, 6.0], [1.0, 6.0, 2.0]], [[3.0, 2.0, 
5.0], [6.0, 2.0, 5.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[7.0], [6.0]], [[8.0], [7.0]], [[1.0], [7.0]]])
      arg[2,:,:2,2:]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 4.0], [7.0, 3.0, 7.0], [4.0, 8.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 2.0, 5.0], [7.0, 3.0, 
3.0]], [[2.0, 5.0, 8.0], [4.0, 8.0, 6.0], [2.0, 2.0, 6.0]]], [[[1.0, 3.0, 2.0], [8.0, 2.0, 3.0], [1.0, 8.0, 5.0]], [[5.0, 4.0, 
8.0], [7.0, 3.0, 7.0], [8.0, 2.0, 3.0]], [[2.0, 4.0, 1.0], [6.0, 2.0, 6.0], [4.0, 8.0, 6.0]]], [[[7.0, 4.0, 7.0], [6.0, 3.0, 
6.0], [6.0, 4.0, 2.0]], [[1.0, 5.0, 8.0], [5.0, 5.0, 7.0], [2.0, 5.0, 8.0]], [[2.0, 5.0, 1.0], [2.0, 2.0, 7.0], [3.0, 3.0, 
7.0]]], [[[3.0, 7.0, 1.0], [2.0, 7.0, 7.0], [7.0, 5.0, 1.0]], [[1.0, 8.0, 6.0], [1.0, 5.0, 6.0], [1.0, 6.0, 2.0]], [[3.0, 2.0, 
5.0], [6.0, 2.0, 5.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCUL(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 2.0], [8.0, 2.0, 8.0], [3.0, 7.0, 5.0]], [[1.0, 1.0, 3.0], [5.0, 2.0, 1.0], [7.0, 2.0, 
2.0]], [[2.0, 1.0, 2.0], [5.0, 1.0, 5.0], [6.0, 3.0, 3.0]]], [[[7.0, 5.0, 4.0], [4.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[3.0, 7.0, 
3.0], [8.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[4.0, 3.0, 6.0], [8.0, 6.0, 1.0], [5.0, 2.0, 4.0]]], [[[3.0, 6.0, 3.0], [6.0, 4.0, 
8.0], [6.0, 6.0, 4.0]], [[4.0, 2.0, 3.0], [7.0, 4.0, 1.0], [4.0, 3.0, 3.0]], [[6.0, 5.0, 5.0], [5.0, 1.0, 6.0], [3.0, 3.0, 
2.0]]], [[[6.0, 4.0, 3.0], [5.0, 8.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 3.0, 4.0], [3.0, 2.0, 8.0], [7.0, 7.0, 5.0]], [[6.0, 4.0, 
2.0], [8.0, 3.0, 7.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0], [1.0]], [[8.0], [4.0]], [[8.0], [4.0]]]),self.functionspace)
      arg[2,:,:2,2:]=val
      ref=Data(numpy.array([[[[5.0, 6.0, 2.0], [8.0, 2.0, 8.0], [3.0, 7.0, 5.0]], [[1.0, 1.0, 3.0], [5.0, 2.0, 1.0], [7.0, 2.0, 
2.0]], [[2.0, 1.0, 2.0], [5.0, 1.0, 5.0], [6.0, 3.0, 3.0]]], [[[7.0, 5.0, 4.0], [4.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[3.0, 7.0, 
3.0], [8.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[4.0, 3.0, 6.0], [8.0, 6.0, 1.0], [5.0, 2.0, 4.0]]], [[[3.0, 6.0, 8.0], [6.0, 4.0, 
1.0], [6.0, 6.0, 4.0]], [[4.0, 2.0, 8.0], [7.0, 4.0, 4.0], [4.0, 3.0, 3.0]], [[6.0, 5.0, 8.0], [5.0, 1.0, 4.0], [3.0, 3.0, 
2.0]]], [[[6.0, 4.0, 3.0], [5.0, 8.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 3.0, 4.0], [3.0, 2.0, 8.0], [7.0, 7.0, 5.0]], [[6.0, 4.0, 
2.0], [8.0, 3.0, 7.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCUL(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 3.0], [8.0, 3.0, 3.0], [6.0, 6.0, 1.0]], [[8.0, 8.0, 5.0], [6.0, 6.0, 7.0], [2.0, 3.0, 
1.0]], [[3.0, 6.0, 1.0], [5.0, 7.0, 6.0], [8.0, 6.0, 3.0]]], [[[3.0, 6.0, 1.0], [1.0, 7.0, 2.0], [6.0, 2.0, 3.0]], [[7.0, 7.0, 
2.0], [2.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 5.0, 1.0], [3.0, 7.0, 4.0], [1.0, 2.0, 3.0]]], [[[6.0, 8.0, 4.0], [8.0, 8.0, 
1.0], [2.0, 1.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 1.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 1.0], [7.0, 7.0, 7.0], [5.0, 2.0, 
2.0]]], [[[6.0, 4.0, 6.0], [1.0, 6.0, 7.0], [8.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], [1.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 
5.0], [3.0, 8.0, 7.0], [6.0, 5.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0], [2.0]], [[2.0], [2.0]], [[3.0], [7.0]]])+(1.-msk_val)*numpy.array([[[4.0], [7.0]], 
[[4.0], [6.0]], [[1.0], [2.0]]])
      arg[2,:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 3.0, 3.0], [6.0, 6.0, 1.0]], [[8.0, 8.0, 5.0], [6.0, 6.0, 7.0], [2.0, 
3.0, 1.0]], [[3.0, 6.0, 1.0], [5.0, 7.0, 6.0], [8.0, 6.0, 3.0]]], [[[3.0, 6.0, 1.0], [1.0, 7.0, 2.0], [6.0, 2.0, 3.0]], [[7.0, 
7.0, 2.0], [2.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 5.0, 1.0], [3.0, 7.0, 4.0], [1.0, 2.0, 3.0]]], [[[6.0, 8.0, 4.0], [8.0, 
8.0, 2.0], [2.0, 1.0, 7.0]], [[7.0, 7.0, 2.0], [5.0, 5.0, 2.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 3.0], [7.0, 7.0, 7.0], [5.0, 2.0, 
2.0]]], [[[6.0, 4.0, 6.0], [1.0, 6.0, 7.0], [8.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], [1.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 
5.0], [3.0, 8.0, 7.0], [6.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 3.0, 3.0], [6.0, 6.0, 1.0]], 
[[8.0, 8.0, 5.0], [6.0, 6.0, 7.0], [2.0, 3.0, 1.0]], [[3.0, 6.0, 1.0], [5.0, 7.0, 6.0], [8.0, 6.0, 3.0]]], [[[3.0, 6.0, 1.0], 
[1.0, 7.0, 2.0], [6.0, 2.0, 3.0]], [[7.0, 7.0, 2.0], [2.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 5.0, 1.0], [3.0, 7.0, 4.0], 
[1.0, 2.0, 3.0]]], [[[6.0, 8.0, 4.0], [8.0, 8.0, 7.0], [2.0, 1.0, 7.0]], [[7.0, 7.0, 4.0], [5.0, 5.0, 6.0], [5.0, 6.0, 3.0]], 
[[2.0, 8.0, 1.0], [7.0, 7.0, 2.0], [5.0, 2.0, 2.0]]], [[[6.0, 4.0, 6.0], [1.0, 6.0, 7.0], [8.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], 
[1.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 5.0], [3.0, 8.0, 7.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLRN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [2.0, 4.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 7.0, 6.0], [8.0, 7.0, 4.0], [4.0, 4.0, 
4.0]], [[6.0, 6.0, 3.0], [8.0, 7.0, 1.0], [6.0, 6.0, 5.0]]], [[[2.0, 4.0, 2.0], [3.0, 4.0, 1.0], [4.0, 1.0, 8.0]], [[5.0, 3.0, 
3.0], [6.0, 8.0, 7.0], [1.0, 3.0, 1.0]], [[1.0, 8.0, 7.0], [4.0, 8.0, 4.0], [1.0, 5.0, 6.0]]], [[[6.0, 5.0, 2.0], [6.0, 6.0, 
6.0], [6.0, 2.0, 2.0]], [[6.0, 8.0, 6.0], [3.0, 7.0, 1.0], [4.0, 8.0, 4.0]], [[3.0, 6.0, 3.0], [6.0, 5.0, 8.0], [3.0, 3.0, 
8.0]]], [[[3.0, 8.0, 6.0], [4.0, 3.0, 3.0], [7.0, 2.0, 2.0]], [[2.0, 6.0, 8.0], [8.0, 5.0, 5.0], [1.0, 2.0, 7.0]], [[3.0, 2.0, 
6.0], [1.0, 6.0, 6.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 6.0, 8.0]]], [[[7.0, 1.0, 5.0]]]])
      arg[2:,2:,1:2]=val
      ref=Data(numpy.array([[[[4.0, 1.0, 3.0], [2.0, 4.0, 8.0], [6.0, 6.0, 3.0]], [[2.0, 7.0, 6.0], [8.0, 7.0, 4.0], [4.0, 4.0, 
4.0]], [[6.0, 6.0, 3.0], [8.0, 7.0, 1.0], [6.0, 6.0, 5.0]]], [[[2.0, 4.0, 2.0], [3.0, 4.0, 1.0], [4.0, 1.0, 8.0]], [[5.0, 3.0, 
3.0], [6.0, 8.0, 7.0], [1.0, 3.0, 1.0]], [[1.0, 8.0, 7.0], [4.0, 8.0, 4.0], [1.0, 5.0, 6.0]]], [[[6.0, 5.0, 2.0], [6.0, 6.0, 
6.0], [6.0, 2.0, 2.0]], [[6.0, 8.0, 6.0], [3.0, 7.0, 1.0], [4.0, 8.0, 4.0]], [[3.0, 6.0, 3.0], [6.0, 6.0, 8.0], [3.0, 3.0, 
8.0]]], [[[3.0, 8.0, 6.0], [4.0, 3.0, 3.0], [7.0, 2.0, 2.0]], [[2.0, 6.0, 8.0], [8.0, 5.0, 5.0], [1.0, 2.0, 7.0]], [[3.0, 2.0, 
6.0], [7.0, 1.0, 5.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLRN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 1.0], [4.0, 2.0, 7.0], [5.0, 3.0, 8.0]], [[4.0, 6.0, 4.0], [4.0, 6.0, 7.0], [8.0, 4.0, 
2.0]], [[4.0, 7.0, 3.0], [4.0, 2.0, 8.0], [3.0, 3.0, 4.0]]], [[[1.0, 1.0, 8.0], [1.0, 6.0, 5.0], [6.0, 2.0, 4.0]], [[4.0, 7.0, 
1.0], [2.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[6.0, 3.0, 2.0], [3.0, 3.0, 2.0], [5.0, 7.0, 5.0]]], [[[5.0, 5.0, 4.0], [7.0, 1.0, 
1.0], [4.0, 6.0, 7.0]], [[8.0, 1.0, 3.0], [7.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[4.0, 3.0, 5.0], [4.0, 3.0, 2.0], [8.0, 8.0, 
3.0]]], [[[3.0, 1.0, 3.0], [2.0, 7.0, 3.0], [3.0, 3.0, 2.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 7.0], [5.0, 7.0, 6.0]], [[4.0, 6.0, 
7.0], [4.0, 3.0, 2.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 4.0, 2.0]]], [[[4.0, 8.0, 3.0]]]]),self.functionspace)
      arg[2:,2:,1:2]=val
      ref=Data(numpy.array([[[[4.0, 6.0, 1.0], [4.0, 2.0, 7.0], [5.0, 3.0, 8.0]], [[4.0, 6.0, 4.0], [4.0, 6.0, 7.0], [8.0, 4.0, 
2.0]], [[4.0, 7.0, 3.0], [4.0, 2.0, 8.0], [3.0, 3.0, 4.0]]], [[[1.0, 1.0, 8.0], [1.0, 6.0, 5.0], [6.0, 2.0, 4.0]], [[4.0, 7.0, 
1.0], [2.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[6.0, 3.0, 2.0], [3.0, 3.0, 2.0], [5.0, 7.0, 5.0]]], [[[5.0, 5.0, 4.0], [7.0, 1.0, 
1.0], [4.0, 6.0, 7.0]], [[8.0, 1.0, 3.0], [7.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[4.0, 3.0, 5.0], [8.0, 4.0, 2.0], [8.0, 8.0, 
3.0]]], [[[3.0, 1.0, 3.0], [2.0, 7.0, 3.0], [3.0, 3.0, 2.0]], [[5.0, 4.0, 2.0], [3.0, 7.0, 7.0], [5.0, 7.0, 6.0]], [[4.0, 6.0, 
7.0], [4.0, 8.0, 3.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLRN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 8.0], [7.0, 7.0, 4.0], [1.0, 4.0, 5.0]], [[2.0, 1.0, 7.0], [8.0, 2.0, 6.0], [1.0, 2.0, 
5.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 4.0], [7.0, 6.0, 6.0]]], [[[4.0, 5.0, 7.0], [1.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[6.0, 2.0, 
2.0], [8.0, 4.0, 2.0], [3.0, 2.0, 5.0]], [[2.0, 4.0, 1.0], [4.0, 3.0, 4.0], [5.0, 3.0, 7.0]]], [[[7.0, 2.0, 3.0], [2.0, 3.0, 
2.0], [2.0, 8.0, 3.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 5.0], [7.0, 8.0, 6.0]], [[8.0, 2.0, 5.0], [1.0, 2.0, 7.0], [7.0, 5.0, 
2.0]]], [[[7.0, 1.0, 5.0], [4.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[4.0, 5.0, 2.0], [1.0, 1.0, 7.0], [6.0, 1.0, 4.0]], [[8.0, 8.0, 
3.0], [7.0, 2.0, 1.0], [5.0, 6.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 1.0, 6.0]]], [[[2.0, 1.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 2.0]]], 
[[[6.0, 3.0, 4.0]]]])
      arg[2:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 8.0], [7.0, 7.0, 4.0], [1.0, 4.0, 5.0]], [[2.0, 1.0, 7.0], [8.0, 2.0, 6.0], [1.0, 
2.0, 5.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 4.0], [7.0, 6.0, 6.0]]], [[[4.0, 5.0, 7.0], [1.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[6.0, 
2.0, 2.0], [8.0, 4.0, 2.0], [3.0, 2.0, 5.0]], [[2.0, 4.0, 1.0], [4.0, 3.0, 4.0], [5.0, 3.0, 7.0]]], [[[7.0, 2.0, 3.0], [2.0, 
3.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 5.0], [7.0, 8.0, 6.0]], [[8.0, 2.0, 5.0], [3.0, 1.0, 6.0], [7.0, 5.0, 
2.0]]], [[[7.0, 1.0, 5.0], [4.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[4.0, 5.0, 2.0], [1.0, 1.0, 7.0], [6.0, 1.0, 4.0]], [[8.0, 8.0, 
3.0], [2.0, 1.0, 2.0], [5.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 8.0], [7.0, 7.0, 4.0], [1.0, 4.0, 5.0]], 
[[2.0, 1.0, 7.0], [8.0, 2.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 4.0], [7.0, 6.0, 6.0]]], [[[4.0, 5.0, 7.0], 
[1.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[6.0, 2.0, 2.0], [8.0, 4.0, 2.0], [3.0, 2.0, 5.0]], [[2.0, 4.0, 1.0], [4.0, 3.0, 4.0], 
[5.0, 3.0, 7.0]]], [[[7.0, 2.0, 3.0], [2.0, 3.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 5.0], [7.0, 8.0, 6.0]], 
[[8.0, 2.0, 5.0], [6.0, 5.0, 2.0], [7.0, 5.0, 2.0]]], [[[7.0, 1.0, 5.0], [4.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[4.0, 5.0, 2.0], 
[1.0, 1.0, 7.0], [6.0, 1.0, 4.0]], [[8.0, 8.0, 3.0], [6.0, 3.0, 4.0], [5.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLPL(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 1.0], [1.0, 3.0, 8.0], [6.0, 7.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 8.0, 4.0], [2.0, 5.0, 
5.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 4.0], [2.0, 8.0, 3.0]]], [[[5.0, 6.0, 8.0], [5.0, 4.0, 7.0], [2.0, 6.0, 5.0]], [[6.0, 7.0, 
3.0], [4.0, 6.0, 1.0], [6.0, 6.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 8.0, 3.0], [6.0, 3.0, 6.0]]], [[[5.0, 2.0, 1.0], [6.0, 2.0, 
2.0], [3.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], [8.0, 7.0, 3.0], [4.0, 2.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 8.0], [8.0, 7.0, 
7.0]]], [[[8.0, 3.0, 2.0], [7.0, 2.0, 6.0], [2.0, 3.0, 3.0]], [[2.0, 5.0, 5.0], [4.0, 4.0, 8.0], [2.0, 2.0, 3.0]], [[4.0, 6.0, 
7.0], [4.0, 8.0, 7.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[1.0]], [[2.0]]])
      arg[1:3,2:,1,2:]=val
      ref=Data(numpy.array([[[[3.0, 1.0, 1.0], [1.0, 3.0, 8.0], [6.0, 7.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 8.0, 4.0], [2.0, 5.0, 
5.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 4.0], [2.0, 8.0, 3.0]]], [[[5.0, 6.0, 8.0], [5.0, 4.0, 7.0], [2.0, 6.0, 5.0]], [[6.0, 7.0, 
3.0], [4.0, 6.0, 1.0], [6.0, 6.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 8.0, 1.0], [6.0, 3.0, 6.0]]], [[[5.0, 2.0, 1.0], [6.0, 2.0, 
2.0], [3.0, 3.0, 5.0]], [[7.0, 8.0, 2.0], [8.0, 7.0, 3.0], [4.0, 2.0, 7.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 2.0], [8.0, 7.0, 
7.0]]], [[[8.0, 3.0, 2.0], [7.0, 2.0, 6.0], [2.0, 3.0, 3.0]], [[2.0, 5.0, 5.0], [4.0, 4.0, 8.0], [2.0, 2.0, 3.0]], [[4.0, 6.0, 
7.0], [4.0, 8.0, 7.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLPL(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 6.0], [1.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 3.0], [4.0, 3.0, 6.0], [8.0, 1.0, 
2.0]], [[8.0, 8.0, 5.0], [8.0, 8.0, 2.0], [6.0, 2.0, 4.0]]], [[[2.0, 3.0, 8.0], [4.0, 1.0, 3.0], [1.0, 8.0, 4.0]], [[5.0, 1.0, 
6.0], [5.0, 2.0, 4.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 6.0], [7.0, 1.0, 6.0], [3.0, 2.0, 3.0]]], [[[3.0, 7.0, 1.0], [1.0, 5.0, 
3.0], [6.0, 5.0, 5.0]], [[5.0, 7.0, 3.0], [6.0, 1.0, 7.0], [5.0, 8.0, 3.0]], [[2.0, 5.0, 4.0], [4.0, 7.0, 1.0], [5.0, 4.0, 
1.0]]], [[[8.0, 8.0, 2.0], [7.0, 1.0, 1.0], [3.0, 5.0, 2.0]], [[4.0, 4.0, 8.0], [6.0, 3.0, 3.0], [3.0, 3.0, 6.0]], [[3.0, 5.0, 
4.0], [6.0, 3.0, 4.0], [6.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0]], [[4.0]]]),self.functionspace)
      arg[1:3,2:,1,2:]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 6.0], [1.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 3.0], [4.0, 3.0, 6.0], [8.0, 1.0, 
2.0]], [[8.0, 8.0, 5.0], [8.0, 8.0, 2.0], [6.0, 2.0, 4.0]]], [[[2.0, 3.0, 8.0], [4.0, 1.0, 3.0], [1.0, 8.0, 4.0]], [[5.0, 1.0, 
6.0], [5.0, 2.0, 4.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 6.0], [7.0, 1.0, 5.0], [3.0, 2.0, 3.0]]], [[[3.0, 7.0, 1.0], [1.0, 5.0, 
3.0], [6.0, 5.0, 5.0]], [[5.0, 7.0, 3.0], [6.0, 1.0, 7.0], [5.0, 8.0, 3.0]], [[2.0, 5.0, 4.0], [4.0, 7.0, 4.0], [5.0, 4.0, 
1.0]]], [[[8.0, 8.0, 2.0], [7.0, 1.0, 1.0], [3.0, 5.0, 2.0]], [[4.0, 4.0, 8.0], [6.0, 3.0, 3.0], [3.0, 3.0, 6.0]], [[3.0, 5.0, 
4.0], [6.0, 3.0, 4.0], [6.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLPL(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 1.0], [6.0, 3.0, 4.0], [3.0, 4.0, 1.0]], [[6.0, 8.0, 8.0], [6.0, 8.0, 1.0], [7.0, 8.0, 
4.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 2.0], [3.0, 3.0, 7.0]]], [[[6.0, 5.0, 5.0], [7.0, 4.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 5.0, 
3.0], [2.0, 7.0, 2.0], [1.0, 5.0, 1.0]], [[3.0, 4.0, 2.0], [4.0, 1.0, 8.0], [7.0, 4.0, 2.0]]], [[[8.0, 7.0, 5.0], [5.0, 2.0, 
3.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 5.0], [8.0, 3.0, 3.0], [4.0, 2.0, 4.0]], [[4.0, 5.0, 8.0], [4.0, 2.0, 4.0], [8.0, 2.0, 
3.0]]], [[[4.0, 8.0, 6.0], [3.0, 3.0, 8.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [1.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 2.0, 
5.0], [8.0, 5.0, 4.0], [8.0, 4.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0]], [[2.0]]])+(1.-msk_val)*numpy.array([[[5.0]], [[7.0]]])
      arg[1:3,2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 1.0], [6.0, 3.0, 4.0], [3.0, 4.0, 1.0]], [[6.0, 8.0, 8.0], [6.0, 8.0, 1.0], [7.0, 
8.0, 4.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 2.0], [3.0, 3.0, 7.0]]], [[[6.0, 5.0, 5.0], [7.0, 4.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 
5.0, 3.0], [2.0, 7.0, 2.0], [1.0, 5.0, 1.0]], [[3.0, 4.0, 2.0], [4.0, 1.0, 5.0], [7.0, 4.0, 2.0]]], [[[8.0, 7.0, 5.0], [5.0, 
2.0, 3.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 5.0], [8.0, 3.0, 3.0], [4.0, 2.0, 4.0]], [[4.0, 5.0, 8.0], [4.0, 2.0, 2.0], [8.0, 2.0, 
3.0]]], [[[4.0, 8.0, 6.0], [3.0, 3.0, 8.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], [1.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 2.0, 
5.0], [8.0, 5.0, 4.0], [8.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 1.0], [6.0, 3.0, 4.0], [3.0, 4.0, 1.0]], 
[[6.0, 8.0, 8.0], [6.0, 8.0, 1.0], [7.0, 8.0, 4.0]], [[5.0, 3.0, 6.0], [5.0, 1.0, 2.0], [3.0, 3.0, 7.0]]], [[[6.0, 5.0, 5.0], 
[7.0, 4.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 5.0, 3.0], [2.0, 7.0, 2.0], [1.0, 5.0, 1.0]], [[3.0, 4.0, 2.0], [4.0, 1.0, 5.0], 
[7.0, 4.0, 2.0]]], [[[8.0, 7.0, 5.0], [5.0, 2.0, 3.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 5.0], [8.0, 3.0, 3.0], [4.0, 2.0, 4.0]], 
[[4.0, 5.0, 8.0], [4.0, 2.0, 7.0], [8.0, 2.0, 3.0]]], [[[4.0, 8.0, 6.0], [3.0, 3.0, 8.0], [3.0, 1.0, 5.0]], [[6.0, 3.0, 8.0], 
[1.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 2.0, 5.0], [8.0, 5.0, 4.0], [8.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LCUL(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 5.0], [8.0, 6.0, 1.0], [4.0, 2.0, 5.0]], [[7.0, 4.0, 8.0], [6.0, 4.0, 6.0], [5.0, 3.0, 
5.0]], [[2.0, 4.0, 6.0], [5.0, 1.0, 3.0], [1.0, 5.0, 4.0]]], [[[8.0, 4.0, 3.0], [7.0, 3.0, 4.0], [2.0, 5.0, 4.0]], [[7.0, 2.0, 
5.0], [6.0, 2.0, 2.0], [8.0, 6.0, 8.0]], [[7.0, 4.0, 2.0], [7.0, 6.0, 1.0], [2.0, 4.0, 1.0]]], [[[3.0, 5.0, 5.0], [8.0, 3.0, 
3.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 6.0], [2.0, 4.0, 3.0], [5.0, 7.0, 7.0]], [[3.0, 1.0, 1.0], [4.0, 2.0, 4.0], [7.0, 5.0, 
5.0]]], [[[5.0, 4.0, 7.0], [5.0, 8.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 2.0, 2.0], [7.0, 4.0, 1.0], [3.0, 6.0, 8.0]], [[1.0, 6.0, 
6.0], [3.0, 6.0, 3.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0], [4.0]], [[6.0], [5.0]], [[4.0], [3.0]]], [[[6.0], [5.0]], [[4.0], [8.0]], [[8.0], [2.0]]]])
      arg[2:,:,:2,2:]=val
      ref=Data(numpy.array([[[[5.0, 1.0, 5.0], [8.0, 6.0, 1.0], [4.0, 2.0, 5.0]], [[7.0, 4.0, 8.0], [6.0, 4.0, 6.0], [5.0, 3.0, 
5.0]], [[2.0, 4.0, 6.0], [5.0, 1.0, 3.0], [1.0, 5.0, 4.0]]], [[[8.0, 4.0, 3.0], [7.0, 3.0, 4.0], [2.0, 5.0, 4.0]], [[7.0, 2.0, 
5.0], [6.0, 2.0, 2.0], [8.0, 6.0, 8.0]], [[7.0, 4.0, 2.0], [7.0, 6.0, 1.0], [2.0, 4.0, 1.0]]], [[[3.0, 5.0, 5.0], [8.0, 3.0, 
4.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 6.0], [2.0, 4.0, 5.0], [5.0, 7.0, 7.0]], [[3.0, 1.0, 4.0], [4.0, 2.0, 3.0], [7.0, 5.0, 
5.0]]], [[[5.0, 4.0, 6.0], [5.0, 8.0, 5.0], [7.0, 8.0, 5.0]], [[8.0, 2.0, 4.0], [7.0, 4.0, 8.0], [3.0, 6.0, 8.0]], [[1.0, 6.0, 
8.0], [3.0, 6.0, 2.0], [3.0, 4.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LCUL(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 1.0], [8.0, 1.0, 3.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 8.0], [4.0, 8.0, 5.0], [3.0, 8.0, 
5.0]], [[1.0, 7.0, 8.0], [3.0, 2.0, 6.0], [4.0, 1.0, 8.0]]], [[[8.0, 7.0, 8.0], [5.0, 2.0, 8.0], [2.0, 8.0, 4.0]], [[4.0, 5.0, 
1.0], [7.0, 8.0, 4.0], [5.0, 8.0, 1.0]], [[6.0, 1.0, 2.0], [2.0, 4.0, 5.0], [6.0, 4.0, 6.0]]], [[[1.0, 6.0, 8.0], [6.0, 3.0, 
5.0], [5.0, 4.0, 2.0]], [[5.0, 3.0, 4.0], [6.0, 4.0, 4.0], [8.0, 6.0, 1.0]], [[8.0, 7.0, 5.0], [8.0, 8.0, 5.0], [3.0, 5.0, 
8.0]]], [[[1.0, 6.0, 5.0], [5.0, 6.0, 1.0], [7.0, 6.0, 3.0]], [[1.0, 1.0, 3.0], [4.0, 1.0, 8.0], [2.0, 1.0, 8.0]], [[7.0, 1.0, 
7.0], [4.0, 6.0, 6.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0], [6.0]], [[7.0], [7.0]], [[7.0], [3.0]]], [[[3.0], [8.0]], [[1.0], [5.0]], [[4.0], 
[4.0]]]]),self.functionspace)
      arg[2:,:,:2,2:]=val
      ref=Data(numpy.array([[[[6.0, 6.0, 1.0], [8.0, 1.0, 3.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 8.0], [4.0, 8.0, 5.0], [3.0, 8.0, 
5.0]], [[1.0, 7.0, 8.0], [3.0, 2.0, 6.0], [4.0, 1.0, 8.0]]], [[[8.0, 7.0, 8.0], [5.0, 2.0, 8.0], [2.0, 8.0, 4.0]], [[4.0, 5.0, 
1.0], [7.0, 8.0, 4.0], [5.0, 8.0, 1.0]], [[6.0, 1.0, 2.0], [2.0, 4.0, 5.0], [6.0, 4.0, 6.0]]], [[[1.0, 6.0, 6.0], [6.0, 3.0, 
6.0], [5.0, 4.0, 2.0]], [[5.0, 3.0, 7.0], [6.0, 4.0, 7.0], [8.0, 6.0, 1.0]], [[8.0, 7.0, 7.0], [8.0, 8.0, 3.0], [3.0, 5.0, 
8.0]]], [[[1.0, 6.0, 3.0], [5.0, 6.0, 8.0], [7.0, 6.0, 3.0]], [[1.0, 1.0, 1.0], [4.0, 1.0, 5.0], [2.0, 1.0, 8.0]], [[7.0, 1.0, 
4.0], [4.0, 6.0, 4.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LCUL(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 1.0], [7.0, 1.0, 5.0], [5.0, 2.0, 7.0]], [[8.0, 4.0, 1.0], [2.0, 8.0, 2.0], [7.0, 6.0, 
4.0]], [[5.0, 2.0, 2.0], [2.0, 8.0, 4.0], [2.0, 1.0, 5.0]]], [[[4.0, 3.0, 4.0], [4.0, 7.0, 5.0], [2.0, 6.0, 4.0]], [[7.0, 2.0, 
3.0], [1.0, 1.0, 6.0], [1.0, 6.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 4.0, 1.0], [3.0, 8.0, 5.0]]], [[[1.0, 7.0, 1.0], [2.0, 3.0, 
1.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 5.0], [2.0, 1.0, 7.0], [4.0, 3.0, 5.0]], [[6.0, 7.0, 6.0], [8.0, 4.0, 7.0], [4.0, 2.0, 
5.0]]], [[[6.0, 2.0, 6.0], [3.0, 5.0, 8.0], [6.0, 1.0, 4.0]], [[3.0, 2.0, 6.0], [3.0, 4.0, 3.0], [5.0, 5.0, 8.0]], [[7.0, 4.0, 
4.0], [4.0, 4.0, 3.0], [4.0, 7.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0], [5.0]], [[2.0], [1.0]], [[6.0], [6.0]]], [[[5.0], [5.0]], [[5.0], [8.0]], [[3.0], 
[8.0]]]])+(1.-msk_val)*numpy.array([[[[2.0], [4.0]], [[8.0], [4.0]], [[6.0], [3.0]]], [[[4.0], [3.0]], [[8.0], [5.0]], [[5.0], 
[8.0]]]])
      arg[2:,:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 1.0], [7.0, 1.0, 5.0], [5.0, 2.0, 7.0]], [[8.0, 4.0, 1.0], [2.0, 8.0, 2.0], [7.0, 
6.0, 4.0]], [[5.0, 2.0, 2.0], [2.0, 8.0, 4.0], [2.0, 1.0, 5.0]]], [[[4.0, 3.0, 4.0], [4.0, 7.0, 5.0], [2.0, 6.0, 4.0]], [[7.0, 
2.0, 3.0], [1.0, 1.0, 6.0], [1.0, 6.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 4.0, 1.0], [3.0, 8.0, 5.0]]], [[[1.0, 7.0, 6.0], [2.0, 
3.0, 5.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 2.0], [2.0, 1.0, 1.0], [4.0, 3.0, 5.0]], [[6.0, 7.0, 6.0], [8.0, 4.0, 6.0], [4.0, 2.0, 
5.0]]], [[[6.0, 2.0, 5.0], [3.0, 5.0, 5.0], [6.0, 1.0, 4.0]], [[3.0, 2.0, 5.0], [3.0, 4.0, 8.0], [5.0, 5.0, 8.0]], [[7.0, 4.0, 
3.0], [4.0, 4.0, 8.0], [4.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 1.0], [7.0, 1.0, 5.0], [5.0, 2.0, 7.0]], 
[[8.0, 4.0, 1.0], [2.0, 8.0, 2.0], [7.0, 6.0, 4.0]], [[5.0, 2.0, 2.0], [2.0, 8.0, 4.0], [2.0, 1.0, 5.0]]], [[[4.0, 3.0, 4.0], 
[4.0, 7.0, 5.0], [2.0, 6.0, 4.0]], [[7.0, 2.0, 3.0], [1.0, 1.0, 6.0], [1.0, 6.0, 1.0]], [[8.0, 3.0, 6.0], [8.0, 4.0, 1.0], 
[3.0, 8.0, 5.0]]], [[[1.0, 7.0, 2.0], [2.0, 3.0, 4.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 8.0], [2.0, 1.0, 4.0], [4.0, 3.0, 5.0]], 
[[6.0, 7.0, 6.0], [8.0, 4.0, 3.0], [4.0, 2.0, 5.0]]], [[[6.0, 2.0, 4.0], [3.0, 5.0, 3.0], [6.0, 1.0, 4.0]], [[3.0, 2.0, 8.0], 
[3.0, 4.0, 5.0], [5.0, 5.0, 8.0]], [[7.0, 4.0, 5.0], [4.0, 4.0, 8.0], [4.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPUU(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 2.0], [8.0, 5.0, 4.0], [3.0, 6.0, 2.0]], [[5.0, 4.0, 1.0], [3.0, 7.0, 8.0], [7.0, 6.0, 
4.0]], [[6.0, 7.0, 4.0], [6.0, 1.0, 8.0], [5.0, 2.0, 2.0]]], [[[4.0, 5.0, 1.0], [4.0, 8.0, 2.0], [6.0, 6.0, 8.0]], [[4.0, 8.0, 
3.0], [8.0, 8.0, 3.0], [3.0, 3.0, 4.0]], [[2.0, 6.0, 5.0], [5.0, 8.0, 8.0], [4.0, 4.0, 5.0]]], [[[1.0, 3.0, 1.0], [4.0, 8.0, 
1.0], [1.0, 5.0, 8.0]], [[5.0, 6.0, 3.0], [8.0, 4.0, 6.0], [2.0, 2.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 5.0, 3.0], [5.0, 7.0, 
4.0]]], [[[3.0, 2.0, 2.0], [7.0, 7.0, 6.0], [7.0, 7.0, 1.0]], [[6.0, 2.0, 8.0], [7.0, 1.0, 4.0], [1.0, 2.0, 4.0]], [[1.0, 2.0, 
3.0], [4.0, 7.0, 2.0], [6.0, 6.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[1.0, 3.0], [3.0, 1.0]])
      arg[2,1,:2,:2]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 2.0], [8.0, 5.0, 4.0], [3.0, 6.0, 2.0]], [[5.0, 4.0, 1.0], [3.0, 7.0, 8.0], [7.0, 6.0, 
4.0]], [[6.0, 7.0, 4.0], [6.0, 1.0, 8.0], [5.0, 2.0, 2.0]]], [[[4.0, 5.0, 1.0], [4.0, 8.0, 2.0], [6.0, 6.0, 8.0]], [[4.0, 8.0, 
3.0], [8.0, 8.0, 3.0], [3.0, 3.0, 4.0]], [[2.0, 6.0, 5.0], [5.0, 8.0, 8.0], [4.0, 4.0, 5.0]]], [[[1.0, 3.0, 1.0], [4.0, 8.0, 
1.0], [1.0, 5.0, 8.0]], [[1.0, 3.0, 3.0], [3.0, 1.0, 6.0], [2.0, 2.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 5.0, 3.0], [5.0, 7.0, 
4.0]]], [[[3.0, 2.0, 2.0], [7.0, 7.0, 6.0], [7.0, 7.0, 1.0]], [[6.0, 2.0, 8.0], [7.0, 1.0, 4.0], [1.0, 2.0, 4.0]], [[1.0, 2.0, 
3.0], [4.0, 7.0, 2.0], [6.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPUU(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 1.0], [6.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [3.0, 7.0, 1.0], [8.0, 6.0, 
7.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 4.0], [4.0, 4.0, 2.0]]], [[[5.0, 8.0, 2.0], [5.0, 4.0, 7.0], [6.0, 2.0, 8.0]], [[7.0, 6.0, 
8.0], [5.0, 3.0, 6.0], [8.0, 3.0, 8.0]], [[3.0, 7.0, 7.0], [1.0, 7.0, 7.0], [8.0, 7.0, 4.0]]], [[[3.0, 7.0, 6.0], [1.0, 8.0, 
1.0], [3.0, 5.0, 1.0]], [[2.0, 6.0, 7.0], [2.0, 4.0, 2.0], [1.0, 7.0, 4.0]], [[6.0, 4.0, 3.0], [5.0, 8.0, 6.0], [1.0, 3.0, 
6.0]]], [[[1.0, 7.0, 2.0], [1.0, 1.0, 8.0], [4.0, 7.0, 4.0]], [[4.0, 5.0, 7.0], [8.0, 3.0, 3.0], [4.0, 2.0, 2.0]], [[2.0, 6.0, 
8.0], [6.0, 3.0, 7.0], [5.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 3.0], [8.0, 2.0]]),self.functionspace)
      arg[2,1,:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 1.0, 1.0], [6.0, 7.0, 3.0], [4.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [3.0, 7.0, 1.0], [8.0, 6.0, 
7.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 4.0], [4.0, 4.0, 2.0]]], [[[5.0, 8.0, 2.0], [5.0, 4.0, 7.0], [6.0, 2.0, 8.0]], [[7.0, 6.0, 
8.0], [5.0, 3.0, 6.0], [8.0, 3.0, 8.0]], [[3.0, 7.0, 7.0], [1.0, 7.0, 7.0], [8.0, 7.0, 4.0]]], [[[3.0, 7.0, 6.0], [1.0, 8.0, 
1.0], [3.0, 5.0, 1.0]], [[8.0, 3.0, 7.0], [8.0, 2.0, 2.0], [1.0, 7.0, 4.0]], [[6.0, 4.0, 3.0], [5.0, 8.0, 6.0], [1.0, 3.0, 
6.0]]], [[[1.0, 7.0, 2.0], [1.0, 1.0, 8.0], [4.0, 7.0, 4.0]], [[4.0, 5.0, 7.0], [8.0, 3.0, 3.0], [4.0, 2.0, 2.0]], [[2.0, 6.0, 
8.0], [6.0, 3.0, 7.0], [5.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPUU(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 8.0], [2.0, 1.0, 8.0], [5.0, 5.0, 2.0]], [[6.0, 2.0, 6.0], [4.0, 6.0, 5.0], [7.0, 2.0, 
4.0]], [[4.0, 1.0, 6.0], [8.0, 5.0, 1.0], [7.0, 7.0, 4.0]]], [[[4.0, 8.0, 6.0], [8.0, 7.0, 2.0], [4.0, 3.0, 3.0]], [[1.0, 1.0, 
5.0], [6.0, 2.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 3.0], [7.0, 2.0, 5.0], [4.0, 1.0, 4.0]]], [[[4.0, 8.0, 3.0], [6.0, 5.0, 
8.0], [8.0, 8.0, 5.0]], [[4.0, 8.0, 4.0], [8.0, 8.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 6.0, 1.0], [2.0, 8.0, 8.0], [4.0, 4.0, 
8.0]]], [[[5.0, 4.0, 7.0], [7.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[6.0, 1.0, 7.0], [1.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[3.0, 8.0, 
8.0], [2.0, 1.0, 4.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 3.0], [5.0, 1.0]])+(1.-msk_val)*numpy.array([[3.0, 3.0], [4.0, 7.0]])
      arg[2,1,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 8.0], [2.0, 1.0, 8.0], [5.0, 5.0, 2.0]], [[6.0, 2.0, 6.0], [4.0, 6.0, 5.0], [7.0, 
2.0, 4.0]], [[4.0, 1.0, 6.0], [8.0, 5.0, 1.0], [7.0, 7.0, 4.0]]], [[[4.0, 8.0, 6.0], [8.0, 7.0, 2.0], [4.0, 3.0, 3.0]], [[1.0, 
1.0, 5.0], [6.0, 2.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 3.0], [7.0, 2.0, 5.0], [4.0, 1.0, 4.0]]], [[[4.0, 8.0, 3.0], [6.0, 
5.0, 8.0], [8.0, 8.0, 5.0]], [[7.0, 3.0, 4.0], [5.0, 1.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 6.0, 1.0], [2.0, 8.0, 8.0], [4.0, 4.0, 
8.0]]], [[[5.0, 4.0, 7.0], [7.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[6.0, 1.0, 7.0], [1.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[3.0, 8.0, 
8.0], [2.0, 1.0, 4.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 8.0], [2.0, 1.0, 8.0], [5.0, 5.0, 2.0]], 
[[6.0, 2.0, 6.0], [4.0, 6.0, 5.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 6.0], [8.0, 5.0, 1.0], [7.0, 7.0, 4.0]]], [[[4.0, 8.0, 6.0], 
[8.0, 7.0, 2.0], [4.0, 3.0, 3.0]], [[1.0, 1.0, 5.0], [6.0, 2.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 3.0], [7.0, 2.0, 5.0], 
[4.0, 1.0, 4.0]]], [[[4.0, 8.0, 3.0], [6.0, 5.0, 8.0], [8.0, 8.0, 5.0]], [[3.0, 3.0, 4.0], [4.0, 7.0, 4.0], [8.0, 4.0, 6.0]], 
[[2.0, 6.0, 1.0], [2.0, 8.0, 8.0], [4.0, 4.0, 8.0]]], [[[5.0, 4.0, 7.0], [7.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[6.0, 1.0, 7.0], 
[1.0, 1.0, 7.0], [1.0, 6.0, 7.0]], [[3.0, 8.0, 8.0], [2.0, 1.0, 4.0], [4.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPLP(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 3.0], [1.0, 3.0, 4.0], [7.0, 7.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 2.0, 6.0], [4.0, 7.0, 
2.0]], [[2.0, 4.0, 2.0], [6.0, 8.0, 5.0], [1.0, 1.0, 5.0]]], [[[8.0, 2.0, 6.0], [2.0, 8.0, 6.0], [2.0, 2.0, 1.0]], [[8.0, 1.0, 
8.0], [6.0, 6.0, 7.0], [5.0, 1.0, 7.0]], [[7.0, 6.0, 7.0], [5.0, 7.0, 7.0], [6.0, 3.0, 2.0]]], [[[5.0, 1.0, 1.0], [3.0, 5.0, 
2.0], [4.0, 2.0, 3.0]], [[4.0, 6.0, 2.0], [2.0, 8.0, 4.0], [7.0, 7.0, 8.0]], [[4.0, 3.0, 1.0], [6.0, 6.0, 2.0], [3.0, 5.0, 
1.0]]], [[[8.0, 5.0, 7.0], [6.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[6.0, 2.0, 3.0], [1.0, 6.0, 5.0], [7.0, 7.0, 3.0]], [[3.0, 5.0, 
8.0], [7.0, 6.0, 7.0], [8.0, 6.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[1.0], [7.0]])
      arg[2:,1,2:,1]=val
      ref=Data(numpy.array([[[[5.0, 1.0, 3.0], [1.0, 3.0, 4.0], [7.0, 7.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 2.0, 6.0], [4.0, 7.0, 
2.0]], [[2.0, 4.0, 2.0], [6.0, 8.0, 5.0], [1.0, 1.0, 5.0]]], [[[8.0, 2.0, 6.0], [2.0, 8.0, 6.0], [2.0, 2.0, 1.0]], [[8.0, 1.0, 
8.0], [6.0, 6.0, 7.0], [5.0, 1.0, 7.0]], [[7.0, 6.0, 7.0], [5.0, 7.0, 7.0], [6.0, 3.0, 2.0]]], [[[5.0, 1.0, 1.0], [3.0, 5.0, 
2.0], [4.0, 2.0, 3.0]], [[4.0, 6.0, 2.0], [2.0, 8.0, 4.0], [7.0, 1.0, 8.0]], [[4.0, 3.0, 1.0], [6.0, 6.0, 2.0], [3.0, 5.0, 
1.0]]], [[[8.0, 5.0, 7.0], [6.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[6.0, 2.0, 3.0], [1.0, 6.0, 5.0], [7.0, 7.0, 3.0]], [[3.0, 5.0, 
8.0], [7.0, 6.0, 7.0], [8.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPLP(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 4.0], [2.0, 7.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 8.0, 6.0], [6.0, 2.0, 2.0], [8.0, 1.0, 
2.0]], [[5.0, 5.0, 1.0], [7.0, 8.0, 8.0], [5.0, 8.0, 5.0]]], [[[4.0, 4.0, 8.0], [7.0, 6.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 1.0, 
8.0], [8.0, 5.0, 7.0], [4.0, 4.0, 4.0]], [[7.0, 8.0, 8.0], [5.0, 3.0, 5.0], [1.0, 2.0, 7.0]]], [[[8.0, 8.0, 1.0], [2.0, 6.0, 
7.0], [3.0, 8.0, 6.0]], [[2.0, 3.0, 6.0], [1.0, 1.0, 6.0], [2.0, 7.0, 7.0]], [[8.0, 5.0, 5.0], [2.0, 6.0, 4.0], [4.0, 3.0, 
6.0]]], [[[1.0, 1.0, 3.0], [7.0, 6.0, 3.0], [6.0, 6.0, 3.0]], [[3.0, 7.0, 3.0], [8.0, 4.0, 1.0], [8.0, 1.0, 5.0]], [[1.0, 8.0, 
2.0], [1.0, 4.0, 8.0], [1.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0], [3.0]]),self.functionspace)
      arg[2:,1,2:,1]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 4.0], [2.0, 7.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 8.0, 6.0], [6.0, 2.0, 2.0], [8.0, 1.0, 
2.0]], [[5.0, 5.0, 1.0], [7.0, 8.0, 8.0], [5.0, 8.0, 5.0]]], [[[4.0, 4.0, 8.0], [7.0, 6.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 1.0, 
8.0], [8.0, 5.0, 7.0], [4.0, 4.0, 4.0]], [[7.0, 8.0, 8.0], [5.0, 3.0, 5.0], [1.0, 2.0, 7.0]]], [[[8.0, 8.0, 1.0], [2.0, 6.0, 
7.0], [3.0, 8.0, 6.0]], [[2.0, 3.0, 6.0], [1.0, 1.0, 6.0], [2.0, 1.0, 7.0]], [[8.0, 5.0, 5.0], [2.0, 6.0, 4.0], [4.0, 3.0, 
6.0]]], [[[1.0, 1.0, 3.0], [7.0, 6.0, 3.0], [6.0, 6.0, 3.0]], [[3.0, 7.0, 3.0], [8.0, 4.0, 1.0], [8.0, 3.0, 5.0]], [[1.0, 8.0, 
2.0], [1.0, 4.0, 8.0], [1.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPLP(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 4.0], [3.0, 4.0, 1.0], [4.0, 1.0, 4.0]], [[3.0, 3.0, 5.0], [8.0, 1.0, 8.0], [1.0, 8.0, 
6.0]], [[1.0, 2.0, 4.0], [4.0, 2.0, 6.0], [5.0, 3.0, 7.0]]], [[[3.0, 3.0, 4.0], [4.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[6.0, 6.0, 
8.0], [1.0, 7.0, 8.0], [6.0, 5.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 4.0], [1.0, 6.0, 5.0]]], [[[8.0, 2.0, 6.0], [5.0, 1.0, 
5.0], [7.0, 1.0, 2.0]], [[4.0, 3.0, 2.0], [2.0, 1.0, 1.0], [2.0, 2.0, 1.0]], [[5.0, 4.0, 7.0], [1.0, 4.0, 5.0], [5.0, 1.0, 
3.0]]], [[[8.0, 6.0, 7.0], [5.0, 8.0, 3.0], [2.0, 7.0, 1.0]], [[4.0, 7.0, 5.0], [8.0, 7.0, 5.0], [5.0, 7.0, 3.0]], [[6.0, 6.0, 
5.0], [4.0, 6.0, 4.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0], [8.0]])+(1.-msk_val)*numpy.array([[2.0], [8.0]])
      arg[2:,1,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 4.0], [3.0, 4.0, 1.0], [4.0, 1.0, 4.0]], [[3.0, 3.0, 5.0], [8.0, 1.0, 8.0], [1.0, 
8.0, 6.0]], [[1.0, 2.0, 4.0], [4.0, 2.0, 6.0], [5.0, 3.0, 7.0]]], [[[3.0, 3.0, 4.0], [4.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[6.0, 
6.0, 8.0], [1.0, 7.0, 8.0], [6.0, 5.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 4.0], [1.0, 6.0, 5.0]]], [[[8.0, 2.0, 6.0], [5.0, 
1.0, 5.0], [7.0, 1.0, 2.0]], [[4.0, 3.0, 2.0], [2.0, 1.0, 1.0], [2.0, 7.0, 1.0]], [[5.0, 4.0, 7.0], [1.0, 4.0, 5.0], [5.0, 1.0, 
3.0]]], [[[8.0, 6.0, 7.0], [5.0, 8.0, 3.0], [2.0, 7.0, 1.0]], [[4.0, 7.0, 5.0], [8.0, 7.0, 5.0], [5.0, 8.0, 3.0]], [[6.0, 6.0, 
5.0], [4.0, 6.0, 4.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 4.0], [3.0, 4.0, 1.0], [4.0, 1.0, 4.0]], 
[[3.0, 3.0, 5.0], [8.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[1.0, 2.0, 4.0], [4.0, 2.0, 6.0], [5.0, 3.0, 7.0]]], [[[3.0, 3.0, 4.0], 
[4.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[6.0, 6.0, 8.0], [1.0, 7.0, 8.0], [6.0, 5.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 4.0], 
[1.0, 6.0, 5.0]]], [[[8.0, 2.0, 6.0], [5.0, 1.0, 5.0], [7.0, 1.0, 2.0]], [[4.0, 3.0, 2.0], [2.0, 1.0, 1.0], [2.0, 2.0, 1.0]], 
[[5.0, 4.0, 7.0], [1.0, 4.0, 5.0], [5.0, 1.0, 3.0]]], [[[8.0, 6.0, 7.0], [5.0, 8.0, 3.0], [2.0, 7.0, 1.0]], [[4.0, 7.0, 5.0], 
[8.0, 7.0, 5.0], [5.0, 8.0, 3.0]], [[6.0, 6.0, 5.0], [4.0, 6.0, 4.0], [1.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PUPP(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 6.0], [2.0, 3.0, 3.0], [7.0, 6.0, 3.0]], [[4.0, 7.0, 5.0], [2.0, 7.0, 3.0], [2.0, 7.0, 
3.0]], [[6.0, 5.0, 8.0], [1.0, 4.0, 2.0], [8.0, 5.0, 7.0]]], [[[7.0, 8.0, 1.0], [3.0, 5.0, 3.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 
3.0], [3.0, 5.0, 2.0], [4.0, 7.0, 3.0]], [[6.0, 6.0, 8.0], [1.0, 8.0, 7.0], [2.0, 6.0, 3.0]]], [[[2.0, 7.0, 6.0], [7.0, 3.0, 
4.0], [4.0, 7.0, 2.0]], [[8.0, 1.0, 2.0], [6.0, 1.0, 3.0], [5.0, 6.0, 3.0]], [[3.0, 1.0, 6.0], [6.0, 6.0, 3.0], [5.0, 7.0, 
1.0]]], [[[6.0, 8.0, 2.0], [7.0, 3.0, 3.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 6.0], [3.0, 2.0, 5.0], [4.0, 1.0, 3.0]], [[7.0, 1.0, 
1.0], [5.0, 2.0, 2.0], [5.0, 5.0, 3.0]]]]),self.functionspace)
      val=numpy.array([4.0, 5.0])
      arg[2,:2,1,1]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 6.0], [2.0, 3.0, 3.0], [7.0, 6.0, 3.0]], [[4.0, 7.0, 5.0], [2.0, 7.0, 3.0], [2.0, 7.0, 
3.0]], [[6.0, 5.0, 8.0], [1.0, 4.0, 2.0], [8.0, 5.0, 7.0]]], [[[7.0, 8.0, 1.0], [3.0, 5.0, 3.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 
3.0], [3.0, 5.0, 2.0], [4.0, 7.0, 3.0]], [[6.0, 6.0, 8.0], [1.0, 8.0, 7.0], [2.0, 6.0, 3.0]]], [[[2.0, 7.0, 6.0], [7.0, 4.0, 
4.0], [4.0, 7.0, 2.0]], [[8.0, 1.0, 2.0], [6.0, 5.0, 3.0], [5.0, 6.0, 3.0]], [[3.0, 1.0, 6.0], [6.0, 6.0, 3.0], [5.0, 7.0, 
1.0]]], [[[6.0, 8.0, 2.0], [7.0, 3.0, 3.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 6.0], [3.0, 2.0, 5.0], [4.0, 1.0, 3.0]], [[7.0, 1.0, 
1.0], [5.0, 2.0, 2.0], [5.0, 5.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PUPP(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 8.0], [5.0, 6.0, 3.0], [5.0, 8.0, 8.0]], [[4.0, 3.0, 4.0], [7.0, 6.0, 2.0], [8.0, 7.0, 
8.0]], [[6.0, 5.0, 4.0], [3.0, 2.0, 8.0], [5.0, 3.0, 2.0]]], [[[4.0, 4.0, 7.0], [7.0, 5.0, 2.0], [6.0, 7.0, 4.0]], [[4.0, 6.0, 
4.0], [4.0, 4.0, 3.0], [6.0, 4.0, 3.0]], [[1.0, 8.0, 8.0], [1.0, 5.0, 4.0], [5.0, 7.0, 2.0]]], [[[4.0, 6.0, 8.0], [8.0, 3.0, 
1.0], [2.0, 6.0, 7.0]], [[6.0, 3.0, 8.0], [5.0, 3.0, 5.0], [1.0, 4.0, 7.0]], [[2.0, 1.0, 3.0], [6.0, 6.0, 3.0], [5.0, 6.0, 
8.0]]], [[[4.0, 5.0, 7.0], [3.0, 2.0, 1.0], [2.0, 5.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 3.0, 3.0], [7.0, 3.0, 6.0]], [[4.0, 4.0, 
7.0], [5.0, 2.0, 3.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([1.0, 7.0]),self.functionspace)
      arg[2,:2,1,1]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 8.0], [5.0, 6.0, 3.0], [5.0, 8.0, 8.0]], [[4.0, 3.0, 4.0], [7.0, 6.0, 2.0], [8.0, 7.0, 
8.0]], [[6.0, 5.0, 4.0], [3.0, 2.0, 8.0], [5.0, 3.0, 2.0]]], [[[4.0, 4.0, 7.0], [7.0, 5.0, 2.0], [6.0, 7.0, 4.0]], [[4.0, 6.0, 
4.0], [4.0, 4.0, 3.0], [6.0, 4.0, 3.0]], [[1.0, 8.0, 8.0], [1.0, 5.0, 4.0], [5.0, 7.0, 2.0]]], [[[4.0, 6.0, 8.0], [8.0, 1.0, 
1.0], [2.0, 6.0, 7.0]], [[6.0, 3.0, 8.0], [5.0, 7.0, 5.0], [1.0, 4.0, 7.0]], [[2.0, 1.0, 3.0], [6.0, 6.0, 3.0], [5.0, 6.0, 
8.0]]], [[[4.0, 5.0, 7.0], [3.0, 2.0, 1.0], [2.0, 5.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 3.0, 3.0], [7.0, 3.0, 6.0]], [[4.0, 4.0, 
7.0], [5.0, 2.0, 3.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PUPP(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 6.0], [5.0, 5.0, 7.0], [1.0, 4.0, 7.0]], [[2.0, 1.0, 6.0], [7.0, 8.0, 3.0], [4.0, 6.0, 
6.0]], [[6.0, 8.0, 1.0], [4.0, 7.0, 5.0], [5.0, 2.0, 7.0]]], [[[3.0, 7.0, 6.0], [1.0, 4.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 4.0, 
2.0], [1.0, 5.0, 3.0], [6.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], [7.0, 6.0, 8.0], [3.0, 6.0, 7.0]]], [[[7.0, 3.0, 4.0], [8.0, 5.0, 
3.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [4.0, 1.0, 1.0], [6.0, 3.0, 5.0]], [[1.0, 3.0, 7.0], [6.0, 4.0, 1.0], [2.0, 7.0, 
2.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 4.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 7.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 8.0, 
1.0], [7.0, 4.0, 5.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0, 4.0])+(1.-msk_val)*numpy.array([4.0, 3.0])
      arg[2,:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 6.0], [5.0, 5.0, 7.0], [1.0, 4.0, 7.0]], [[2.0, 1.0, 6.0], [7.0, 8.0, 3.0], [4.0, 
6.0, 6.0]], [[6.0, 8.0, 1.0], [4.0, 7.0, 5.0], [5.0, 2.0, 7.0]]], [[[3.0, 7.0, 6.0], [1.0, 4.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 
4.0, 2.0], [1.0, 5.0, 3.0], [6.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], [7.0, 6.0, 8.0], [3.0, 6.0, 7.0]]], [[[7.0, 3.0, 4.0], [8.0, 
5.0, 3.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [4.0, 4.0, 1.0], [6.0, 3.0, 5.0]], [[1.0, 3.0, 7.0], [6.0, 4.0, 1.0], [2.0, 7.0, 
2.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 4.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 7.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 8.0, 
1.0], [7.0, 4.0, 5.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 6.0], [5.0, 5.0, 7.0], [1.0, 4.0, 7.0]], 
[[2.0, 1.0, 6.0], [7.0, 8.0, 3.0], [4.0, 6.0, 6.0]], [[6.0, 8.0, 1.0], [4.0, 7.0, 5.0], [5.0, 2.0, 7.0]]], [[[3.0, 7.0, 6.0], 
[1.0, 4.0, 6.0], [1.0, 7.0, 6.0]], [[8.0, 4.0, 2.0], [1.0, 5.0, 3.0], [6.0, 2.0, 5.0]], [[6.0, 5.0, 8.0], [7.0, 6.0, 8.0], 
[3.0, 6.0, 7.0]]], [[[7.0, 3.0, 4.0], [8.0, 4.0, 3.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [4.0, 3.0, 1.0], [6.0, 3.0, 5.0]], 
[[1.0, 3.0, 7.0], [6.0, 4.0, 1.0], [2.0, 7.0, 2.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 4.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], 
[3.0, 7.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 8.0, 1.0], [7.0, 4.0, 5.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_ULCN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 3.0], [2.0, 2.0, 6.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 8.0], [7.0, 2.0, 6.0], [6.0, 2.0, 
1.0]], [[7.0, 7.0, 7.0], [5.0, 8.0, 1.0], [5.0, 4.0, 2.0]]], [[[2.0, 1.0, 5.0], [6.0, 3.0, 5.0], [5.0, 6.0, 5.0]], [[6.0, 4.0, 
7.0], [4.0, 7.0, 5.0], [5.0, 8.0, 7.0]], [[4.0, 5.0, 6.0], [6.0, 6.0, 5.0], [4.0, 1.0, 1.0]]], [[[2.0, 5.0, 2.0], [3.0, 8.0, 
5.0], [2.0, 3.0, 7.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 2.0], [1.0, 7.0, 3.0]], [[6.0, 2.0, 4.0], [4.0, 2.0, 2.0], [3.0, 5.0, 
3.0]]], [[[1.0, 3.0, 4.0], [6.0, 7.0, 4.0], [5.0, 6.0, 1.0]], [[4.0, 4.0, 2.0], [5.0, 7.0, 3.0], [3.0, 1.0, 1.0]], [[4.0, 5.0, 
6.0], [3.0, 4.0, 8.0], [2.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 2.0, 3.0], [1.0, 6.0, 4.0], [6.0, 3.0, 1.0]]], [[[4.0, 5.0, 8.0], [2.0, 4.0, 2.0], [4.0, 8.0, 
6.0]]], [[[2.0, 7.0, 4.0], [6.0, 6.0, 1.0], [7.0, 8.0, 8.0]]]])
      arg[:3,2:,:]=val
      ref=Data(numpy.array([[[[2.0, 8.0, 3.0], [2.0, 2.0, 6.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 8.0], [7.0, 2.0, 6.0], [6.0, 2.0, 
1.0]], [[8.0, 2.0, 3.0], [1.0, 6.0, 4.0], [6.0, 3.0, 1.0]]], [[[2.0, 1.0, 5.0], [6.0, 3.0, 5.0], [5.0, 6.0, 5.0]], [[6.0, 4.0, 
7.0], [4.0, 7.0, 5.0], [5.0, 8.0, 7.0]], [[4.0, 5.0, 8.0], [2.0, 4.0, 2.0], [4.0, 8.0, 6.0]]], [[[2.0, 5.0, 2.0], [3.0, 8.0, 
5.0], [2.0, 3.0, 7.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 2.0], [1.0, 7.0, 3.0]], [[2.0, 7.0, 4.0], [6.0, 6.0, 1.0], [7.0, 8.0, 
8.0]]], [[[1.0, 3.0, 4.0], [6.0, 7.0, 4.0], [5.0, 6.0, 1.0]], [[4.0, 4.0, 2.0], [5.0, 7.0, 3.0], [3.0, 1.0, 1.0]], [[4.0, 5.0, 
6.0], [3.0, 4.0, 8.0], [2.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_ULCN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 5.0], [1.0, 8.0, 6.0], [8.0, 6.0, 7.0]], [[1.0, 1.0, 7.0], [5.0, 1.0, 5.0], [3.0, 7.0, 
8.0]], [[6.0, 1.0, 7.0], [4.0, 7.0, 6.0], [7.0, 6.0, 2.0]]], [[[5.0, 5.0, 6.0], [3.0, 5.0, 1.0], [2.0, 2.0, 3.0]], [[6.0, 1.0, 
8.0], [4.0, 7.0, 5.0], [3.0, 3.0, 6.0]], [[7.0, 5.0, 2.0], [4.0, 5.0, 8.0], [7.0, 4.0, 7.0]]], [[[5.0, 8.0, 4.0], [8.0, 8.0, 
2.0], [6.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [2.0, 1.0, 5.0], [6.0, 8.0, 8.0]], [[8.0, 3.0, 8.0], [1.0, 8.0, 3.0], [6.0, 3.0, 
2.0]]], [[[8.0, 7.0, 8.0], [8.0, 3.0, 7.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 8.0, 6.0], [4.0, 7.0, 5.0]], [[3.0, 1.0, 
1.0], [2.0, 4.0, 1.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 5.0, 3.0], [1.0, 5.0, 8.0], [2.0, 4.0, 3.0]]], [[[1.0, 5.0, 4.0], [1.0, 6.0, 7.0], [7.0, 
1.0, 7.0]]], [[[1.0, 1.0, 2.0], [6.0, 5.0, 4.0], [2.0, 4.0, 1.0]]]]),self.functionspace)
      arg[:3,2:,:]=val
      ref=Data(numpy.array([[[[3.0, 8.0, 5.0], [1.0, 8.0, 6.0], [8.0, 6.0, 7.0]], [[1.0, 1.0, 7.0], [5.0, 1.0, 5.0], [3.0, 7.0, 
8.0]], [[1.0, 5.0, 3.0], [1.0, 5.0, 8.0], [2.0, 4.0, 3.0]]], [[[5.0, 5.0, 6.0], [3.0, 5.0, 1.0], [2.0, 2.0, 3.0]], [[6.0, 1.0, 
8.0], [4.0, 7.0, 5.0], [3.0, 3.0, 6.0]], [[1.0, 5.0, 4.0], [1.0, 6.0, 7.0], [7.0, 1.0, 7.0]]], [[[5.0, 8.0, 4.0], [8.0, 8.0, 
2.0], [6.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [2.0, 1.0, 5.0], [6.0, 8.0, 8.0]], [[1.0, 1.0, 2.0], [6.0, 5.0, 4.0], [2.0, 4.0, 
1.0]]], [[[8.0, 7.0, 8.0], [8.0, 3.0, 7.0], [7.0, 4.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 8.0, 6.0], [4.0, 7.0, 5.0]], [[3.0, 1.0, 
1.0], [2.0, 4.0, 1.0], [3.0, 2.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_ULCN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 8.0], [7.0, 3.0, 6.0]], [[3.0, 3.0, 4.0], [2.0, 1.0, 8.0], [7.0, 8.0, 
4.0]], [[5.0, 3.0, 5.0], [7.0, 1.0, 4.0], [7.0, 7.0, 4.0]]], [[[1.0, 1.0, 2.0], [4.0, 8.0, 6.0], [6.0, 4.0, 8.0]], [[4.0, 4.0, 
2.0], [5.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[8.0, 6.0, 4.0], [5.0, 1.0, 5.0], [5.0, 8.0, 8.0]]], [[[6.0, 7.0, 5.0], [4.0, 1.0, 
5.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 7.0], [8.0, 2.0, 4.0], [4.0, 2.0, 2.0]], [[8.0, 1.0, 7.0], [7.0, 3.0, 5.0], [1.0, 5.0, 
8.0]]], [[[6.0, 2.0, 2.0], [6.0, 7.0, 4.0], [6.0, 2.0, 8.0]], [[6.0, 2.0, 7.0], [6.0, 3.0, 2.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 
6.0], [5.0, 3.0, 3.0], [2.0, 7.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 7.0, 8.0], [8.0, 7.0, 5.0], [1.0, 4.0, 2.0]]], [[[5.0, 5.0, 7.0], [6.0, 4.0, 6.0], [3.0, 
7.0, 6.0]]], [[[7.0, 7.0, 8.0], [1.0, 7.0, 5.0], [6.0, 5.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 8.0, 8.0], [4.0, 5.0, 
1.0], [4.0, 6.0, 6.0]]], [[[1.0, 5.0, 8.0], [5.0, 7.0, 4.0], [7.0, 6.0, 2.0]]], [[[8.0, 1.0, 7.0], [8.0, 4.0, 5.0], [8.0, 7.0, 
1.0]]]])
      arg[:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 8.0], [7.0, 3.0, 6.0]], [[3.0, 3.0, 4.0], [2.0, 1.0, 8.0], [7.0, 
8.0, 4.0]], [[2.0, 7.0, 8.0], [8.0, 7.0, 5.0], [1.0, 4.0, 2.0]]], [[[1.0, 1.0, 2.0], [4.0, 8.0, 6.0], [6.0, 4.0, 8.0]], [[4.0, 
4.0, 2.0], [5.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[5.0, 5.0, 7.0], [6.0, 4.0, 6.0], [3.0, 7.0, 6.0]]], [[[6.0, 7.0, 5.0], [4.0, 
1.0, 5.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 7.0], [8.0, 2.0, 4.0], [4.0, 2.0, 2.0]], [[7.0, 7.0, 8.0], [1.0, 7.0, 5.0], [6.0, 5.0, 
1.0]]], [[[6.0, 2.0, 2.0], [6.0, 7.0, 4.0], [6.0, 2.0, 8.0]], [[6.0, 2.0, 7.0], [6.0, 3.0, 2.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 
6.0], [5.0, 3.0, 3.0], [2.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 8.0], [7.0, 3.0, 6.0]], 
[[3.0, 3.0, 4.0], [2.0, 1.0, 8.0], [7.0, 8.0, 4.0]], [[2.0, 8.0, 8.0], [4.0, 5.0, 1.0], [4.0, 6.0, 6.0]]], [[[1.0, 1.0, 2.0], 
[4.0, 8.0, 6.0], [6.0, 4.0, 8.0]], [[4.0, 4.0, 2.0], [5.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 8.0], [5.0, 7.0, 4.0], 
[7.0, 6.0, 2.0]]], [[[6.0, 7.0, 5.0], [4.0, 1.0, 5.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 7.0], [8.0, 2.0, 4.0], [4.0, 2.0, 2.0]], 
[[8.0, 1.0, 7.0], [8.0, 4.0, 5.0], [8.0, 7.0, 1.0]]], [[[6.0, 2.0, 2.0], [6.0, 7.0, 4.0], [6.0, 2.0, 8.0]], [[6.0, 2.0, 7.0], 
[6.0, 3.0, 2.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 6.0], [5.0, 3.0, 3.0], [2.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCPL(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 6.0]], [[7.0, 3.0, 2.0], [6.0, 4.0, 4.0], [3.0, 4.0, 
3.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 1.0], [8.0, 4.0, 6.0]]], [[[4.0, 2.0, 1.0], [5.0, 4.0, 2.0], [5.0, 8.0, 3.0]], [[8.0, 5.0, 
3.0], [4.0, 2.0, 3.0], [5.0, 7.0, 7.0]], [[8.0, 5.0, 8.0], [7.0, 8.0, 7.0], [7.0, 3.0, 6.0]]], [[[5.0, 8.0, 8.0], [6.0, 2.0, 
3.0], [7.0, 8.0, 5.0]], [[7.0, 2.0, 3.0], [1.0, 6.0, 8.0], [4.0, 1.0, 5.0]], [[5.0, 7.0, 7.0], [6.0, 2.0, 2.0], [7.0, 1.0, 
3.0]]], [[[5.0, 1.0, 3.0], [7.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 6.0], [7.0, 1.0, 6.0], [2.0, 3.0, 7.0]], [[4.0, 1.0, 
5.0], [8.0, 6.0, 4.0], [7.0, 7.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[8.0], [6.0], [3.0]], [[4.0], [8.0], [2.0]]])
      arg[1:3,:,1,2:]=val
      ref=Data(numpy.array([[[[4.0, 5.0, 7.0], [1.0, 1.0, 5.0], [3.0, 8.0, 6.0]], [[7.0, 3.0, 2.0], [6.0, 4.0, 4.0], [3.0, 4.0, 
3.0]], [[4.0, 5.0, 8.0], [8.0, 2.0, 1.0], [8.0, 4.0, 6.0]]], [[[4.0, 2.0, 1.0], [5.0, 4.0, 8.0], [5.0, 8.0, 3.0]], [[8.0, 5.0, 
3.0], [4.0, 2.0, 6.0], [5.0, 7.0, 7.0]], [[8.0, 5.0, 8.0], [7.0, 8.0, 3.0], [7.0, 3.0, 6.0]]], [[[5.0, 8.0, 8.0], [6.0, 2.0, 
4.0], [7.0, 8.0, 5.0]], [[7.0, 2.0, 3.0], [1.0, 6.0, 8.0], [4.0, 1.0, 5.0]], [[5.0, 7.0, 7.0], [6.0, 2.0, 2.0], [7.0, 1.0, 
3.0]]], [[[5.0, 1.0, 3.0], [7.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 6.0], [7.0, 1.0, 6.0], [2.0, 3.0, 7.0]], [[4.0, 1.0, 
5.0], [8.0, 6.0, 4.0], [7.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCPL(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 1.0], [5.0, 7.0, 8.0], [6.0, 8.0, 7.0]], [[2.0, 7.0, 6.0], [8.0, 8.0, 5.0], [2.0, 4.0, 
2.0]], [[7.0, 1.0, 6.0], [1.0, 3.0, 5.0], [2.0, 1.0, 7.0]]], [[[7.0, 4.0, 5.0], [7.0, 7.0, 3.0], [4.0, 4.0, 3.0]], [[3.0, 6.0, 
2.0], [5.0, 7.0, 3.0], [7.0, 8.0, 3.0]], [[3.0, 3.0, 7.0], [1.0, 5.0, 7.0], [8.0, 6.0, 1.0]]], [[[7.0, 5.0, 6.0], [1.0, 8.0, 
1.0], [1.0, 8.0, 3.0]], [[3.0, 7.0, 3.0], [6.0, 8.0, 7.0], [4.0, 6.0, 6.0]], [[7.0, 3.0, 8.0], [3.0, 8.0, 4.0], [4.0, 2.0, 
6.0]]], [[[3.0, 3.0, 5.0], [8.0, 7.0, 4.0], [8.0, 1.0, 2.0]], [[6.0, 6.0, 1.0], [3.0, 8.0, 4.0], [8.0, 5.0, 6.0]], [[4.0, 2.0, 
2.0], [2.0, 4.0, 8.0], [4.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[1.0], [8.0], [2.0]], [[1.0], [8.0], [4.0]]]),self.functionspace)
      arg[1:3,:,1,2:]=val
      ref=Data(numpy.array([[[[1.0, 2.0, 1.0], [5.0, 7.0, 8.0], [6.0, 8.0, 7.0]], [[2.0, 7.0, 6.0], [8.0, 8.0, 5.0], [2.0, 4.0, 
2.0]], [[7.0, 1.0, 6.0], [1.0, 3.0, 5.0], [2.0, 1.0, 7.0]]], [[[7.0, 4.0, 5.0], [7.0, 7.0, 1.0], [4.0, 4.0, 3.0]], [[3.0, 6.0, 
2.0], [5.0, 7.0, 8.0], [7.0, 8.0, 3.0]], [[3.0, 3.0, 7.0], [1.0, 5.0, 2.0], [8.0, 6.0, 1.0]]], [[[7.0, 5.0, 6.0], [1.0, 8.0, 
1.0], [1.0, 8.0, 3.0]], [[3.0, 7.0, 3.0], [6.0, 8.0, 8.0], [4.0, 6.0, 6.0]], [[7.0, 3.0, 8.0], [3.0, 8.0, 4.0], [4.0, 2.0, 
6.0]]], [[[3.0, 3.0, 5.0], [8.0, 7.0, 4.0], [8.0, 1.0, 2.0]], [[6.0, 6.0, 1.0], [3.0, 8.0, 4.0], [8.0, 5.0, 6.0]], [[4.0, 2.0, 
2.0], [2.0, 4.0, 8.0], [4.0, 3.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCPL(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 1.0], [6.0, 8.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 6.0, 7.0], [1.0, 6.0, 
7.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [6.0, 7.0, 3.0]]], [[[7.0, 2.0, 8.0], [7.0, 4.0, 6.0], [3.0, 2.0, 8.0]], [[1.0, 3.0, 
3.0], [5.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[1.0, 1.0, 2.0], [8.0, 7.0, 2.0], [2.0, 7.0, 5.0]]], [[[7.0, 5.0, 4.0], [3.0, 6.0, 
8.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 4.0], [8.0, 2.0, 5.0]], [[2.0, 7.0, 1.0], [3.0, 2.0, 5.0], [8.0, 4.0, 
3.0]]], [[[2.0, 5.0, 6.0], [5.0, 4.0, 4.0], [3.0, 7.0, 2.0]], [[5.0, 1.0, 5.0], [4.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[7.0, 2.0, 
6.0], [7.0, 1.0, 5.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0], [6.0], [1.0]], [[1.0], [1.0], [7.0]]])+(1.-msk_val)*numpy.array([[[7.0], [6.0], [7.0]], 
[[1.0], [2.0], [3.0]]])
      arg[1:3,:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 1.0], [6.0, 8.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 6.0, 7.0], [1.0, 
6.0, 7.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [6.0, 7.0, 3.0]]], [[[7.0, 2.0, 8.0], [7.0, 4.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 
3.0, 3.0], [5.0, 6.0, 6.0], [8.0, 7.0, 8.0]], [[1.0, 1.0, 2.0], [8.0, 7.0, 1.0], [2.0, 7.0, 5.0]]], [[[7.0, 5.0, 4.0], [3.0, 
6.0, 1.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 1.0], [8.0, 2.0, 5.0]], [[2.0, 7.0, 1.0], [3.0, 2.0, 7.0], [8.0, 4.0, 
3.0]]], [[[2.0, 5.0, 6.0], [5.0, 4.0, 4.0], [3.0, 7.0, 2.0]], [[5.0, 1.0, 5.0], [4.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[7.0, 2.0, 
6.0], [7.0, 1.0, 5.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 1.0], [6.0, 8.0, 7.0], [7.0, 7.0, 2.0]], 
[[3.0, 8.0, 7.0], [3.0, 6.0, 7.0], [1.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [6.0, 7.0, 3.0]]], [[[7.0, 2.0, 8.0], 
[7.0, 4.0, 7.0], [3.0, 2.0, 8.0]], [[1.0, 3.0, 3.0], [5.0, 6.0, 6.0], [8.0, 7.0, 8.0]], [[1.0, 1.0, 2.0], [8.0, 7.0, 7.0], 
[2.0, 7.0, 5.0]]], [[[7.0, 5.0, 4.0], [3.0, 6.0, 1.0], [7.0, 3.0, 3.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 2.0], [8.0, 2.0, 5.0]], 
[[2.0, 7.0, 1.0], [3.0, 2.0, 3.0], [8.0, 4.0, 3.0]]], [[[2.0, 5.0, 6.0], [5.0, 4.0, 4.0], [3.0, 7.0, 2.0]], [[5.0, 1.0, 5.0], 
[4.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[7.0, 2.0, 6.0], [7.0, 1.0, 5.0], [1.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLCL(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 1.0], [6.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[5.0, 2.0, 1.0], [2.0, 8.0, 7.0], [5.0, 7.0, 
3.0]], [[3.0, 5.0, 1.0], [8.0, 2.0, 7.0], [4.0, 1.0, 5.0]]], [[[1.0, 7.0, 1.0], [8.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[6.0, 1.0, 
5.0], [8.0, 3.0, 3.0], [2.0, 6.0, 6.0]], [[1.0, 4.0, 8.0], [6.0, 7.0, 1.0], [1.0, 4.0, 2.0]]], [[[1.0, 3.0, 5.0], [7.0, 5.0, 
6.0], [8.0, 8.0, 8.0]], [[7.0, 5.0, 7.0], [8.0, 1.0, 5.0], [6.0, 1.0, 6.0]], [[6.0, 4.0, 1.0], [2.0, 5.0, 2.0], [8.0, 6.0, 
3.0]]], [[[1.0, 5.0, 1.0], [6.0, 7.0, 7.0], [8.0, 1.0, 4.0]], [[4.0, 8.0, 1.0], [7.0, 6.0, 6.0], [5.0, 8.0, 1.0]], [[3.0, 3.0, 
3.0], [1.0, 1.0, 8.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0], [7.0], [7.0]]], [[[2.0], [7.0], [6.0]]]])
      arg[1:3,2:,:,2:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 1.0], [6.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[5.0, 2.0, 1.0], [2.0, 8.0, 7.0], [5.0, 7.0, 
3.0]], [[3.0, 5.0, 1.0], [8.0, 2.0, 7.0], [4.0, 1.0, 5.0]]], [[[1.0, 7.0, 1.0], [8.0, 4.0, 4.0], [8.0, 8.0, 7.0]], [[6.0, 1.0, 
5.0], [8.0, 3.0, 3.0], [2.0, 6.0, 6.0]], [[1.0, 4.0, 4.0], [6.0, 7.0, 7.0], [1.0, 4.0, 7.0]]], [[[1.0, 3.0, 5.0], [7.0, 5.0, 
6.0], [8.0, 8.0, 8.0]], [[7.0, 5.0, 7.0], [8.0, 1.0, 5.0], [6.0, 1.0, 6.0]], [[6.0, 4.0, 2.0], [2.0, 5.0, 7.0], [8.0, 6.0, 
6.0]]], [[[1.0, 5.0, 1.0], [6.0, 7.0, 7.0], [8.0, 1.0, 4.0]], [[4.0, 8.0, 1.0], [7.0, 6.0, 6.0], [5.0, 8.0, 1.0]], [[3.0, 3.0, 
3.0], [1.0, 1.0, 8.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLCL(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 7.0], [4.0, 3.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [7.0, 5.0, 5.0], [7.0, 8.0, 
3.0]], [[4.0, 1.0, 8.0], [4.0, 2.0, 1.0], [3.0, 6.0, 2.0]]], [[[6.0, 7.0, 4.0], [4.0, 8.0, 1.0], [2.0, 3.0, 6.0]], [[3.0, 4.0, 
1.0], [1.0, 4.0, 7.0], [1.0, 7.0, 6.0]], [[8.0, 4.0, 4.0], [5.0, 6.0, 5.0], [8.0, 5.0, 1.0]]], [[[5.0, 5.0, 4.0], [7.0, 5.0, 
2.0], [6.0, 5.0, 7.0]], [[7.0, 5.0, 8.0], [4.0, 3.0, 5.0], [6.0, 1.0, 1.0]], [[1.0, 4.0, 8.0], [3.0, 4.0, 2.0], [3.0, 1.0, 
7.0]]], [[[4.0, 1.0, 1.0], [1.0, 6.0, 4.0], [5.0, 7.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 8.0, 5.0], [4.0, 4.0, 5.0]], [[2.0, 1.0, 
2.0], [7.0, 4.0, 5.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0], [7.0], [3.0]]], [[[7.0], [8.0], [7.0]]]]),self.functionspace)
      arg[1:3,2:,:,2:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 7.0], [4.0, 3.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [7.0, 5.0, 5.0], [7.0, 8.0, 
3.0]], [[4.0, 1.0, 8.0], [4.0, 2.0, 1.0], [3.0, 6.0, 2.0]]], [[[6.0, 7.0, 4.0], [4.0, 8.0, 1.0], [2.0, 3.0, 6.0]], [[3.0, 4.0, 
1.0], [1.0, 4.0, 7.0], [1.0, 7.0, 6.0]], [[8.0, 4.0, 2.0], [5.0, 6.0, 7.0], [8.0, 5.0, 3.0]]], [[[5.0, 5.0, 4.0], [7.0, 5.0, 
2.0], [6.0, 5.0, 7.0]], [[7.0, 5.0, 8.0], [4.0, 3.0, 5.0], [6.0, 1.0, 1.0]], [[1.0, 4.0, 7.0], [3.0, 4.0, 8.0], [3.0, 1.0, 
7.0]]], [[[4.0, 1.0, 1.0], [1.0, 6.0, 4.0], [5.0, 7.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 8.0, 5.0], [4.0, 4.0, 5.0]], [[2.0, 1.0, 
2.0], [7.0, 4.0, 5.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLCL(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 8.0], [2.0, 2.0, 7.0], [1.0, 3.0, 8.0]], [[3.0, 5.0, 1.0], [2.0, 6.0, 2.0], [3.0, 1.0, 
6.0]], [[7.0, 8.0, 6.0], [6.0, 2.0, 6.0], [7.0, 8.0, 4.0]]], [[[7.0, 5.0, 5.0], [8.0, 3.0, 7.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 
8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 8.0]], [[4.0, 6.0, 7.0], [6.0, 3.0, 8.0], [7.0, 3.0, 4.0]]], [[[5.0, 1.0, 8.0], [7.0, 7.0, 
2.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 3.0], [8.0, 1.0, 4.0], [3.0, 7.0, 5.0]], [[6.0, 3.0, 5.0], [6.0, 2.0, 6.0], [1.0, 1.0, 
2.0]]], [[[6.0, 1.0, 4.0], [7.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[5.0, 3.0, 6.0], [3.0, 2.0, 2.0], [3.0, 2.0, 8.0]], [[3.0, 1.0, 
8.0], [4.0, 6.0, 7.0], [6.0, 4.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0], [3.0], [5.0]]], [[[8.0], [5.0], [2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0], [7.0], 
[3.0]]], [[[6.0], [2.0], [4.0]]]])
      arg[1:3,2:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 8.0], [2.0, 2.0, 7.0], [1.0, 3.0, 8.0]], [[3.0, 5.0, 1.0], [2.0, 6.0, 2.0], [3.0, 
1.0, 6.0]], [[7.0, 8.0, 6.0], [6.0, 2.0, 6.0], [7.0, 8.0, 4.0]]], [[[7.0, 5.0, 5.0], [8.0, 3.0, 7.0], [8.0, 4.0, 8.0]], [[5.0, 
5.0, 8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 8.0]], [[4.0, 6.0, 8.0], [6.0, 3.0, 3.0], [7.0, 3.0, 5.0]]], [[[5.0, 1.0, 8.0], [7.0, 
7.0, 2.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 3.0], [8.0, 1.0, 4.0], [3.0, 7.0, 5.0]], [[6.0, 3.0, 8.0], [6.0, 2.0, 5.0], [1.0, 1.0, 
2.0]]], [[[6.0, 1.0, 4.0], [7.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[5.0, 3.0, 6.0], [3.0, 2.0, 2.0], [3.0, 2.0, 8.0]], [[3.0, 1.0, 
8.0], [4.0, 6.0, 7.0], [6.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 8.0], [2.0, 2.0, 7.0], [1.0, 3.0, 8.0]], 
[[3.0, 5.0, 1.0], [2.0, 6.0, 2.0], [3.0, 1.0, 6.0]], [[7.0, 8.0, 6.0], [6.0, 2.0, 6.0], [7.0, 8.0, 4.0]]], [[[7.0, 5.0, 5.0], 
[8.0, 3.0, 7.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 7.0, 7.0], [2.0, 8.0, 8.0]], [[4.0, 6.0, 2.0], [6.0, 3.0, 7.0], 
[7.0, 3.0, 3.0]]], [[[5.0, 1.0, 8.0], [7.0, 7.0, 2.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 3.0], [8.0, 1.0, 4.0], [3.0, 7.0, 5.0]], 
[[6.0, 3.0, 6.0], [6.0, 2.0, 2.0], [1.0, 1.0, 4.0]]], [[[6.0, 1.0, 4.0], [7.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[5.0, 3.0, 6.0], 
[3.0, 2.0, 2.0], [3.0, 2.0, 8.0]], [[3.0, 1.0, 8.0], [4.0, 6.0, 7.0], [6.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_ULRN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 6.0], [2.0, 3.0, 4.0], [4.0, 8.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 3.0, 3.0], [2.0, 7.0, 
4.0]], [[8.0, 7.0, 3.0], [1.0, 6.0, 7.0], [3.0, 6.0, 7.0]]], [[[1.0, 8.0, 3.0], [6.0, 8.0, 1.0], [2.0, 5.0, 7.0]], [[2.0, 2.0, 
6.0], [7.0, 7.0, 6.0], [2.0, 5.0, 6.0]], [[2.0, 8.0, 6.0], [6.0, 2.0, 4.0], [8.0, 7.0, 1.0]]], [[[3.0, 7.0, 7.0], [4.0, 1.0, 
7.0], [6.0, 7.0, 3.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 3.0], [4.0, 8.0, 4.0]], [[4.0, 3.0, 5.0], [7.0, 4.0, 7.0], [8.0, 7.0, 
3.0]]], [[[2.0, 5.0, 5.0], [5.0, 3.0, 8.0], [1.0, 8.0, 3.0]], [[5.0, 6.0, 4.0], [1.0, 6.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 3.0, 
8.0], [3.0, 4.0, 3.0], [6.0, 5.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 5.0, 1.0]]], [[[3.0, 3.0, 6.0]]], [[[7.0, 7.0, 7.0]]]])
      arg[:3,2:,1:2]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 6.0], [2.0, 3.0, 4.0], [4.0, 8.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 3.0, 3.0], [2.0, 7.0, 
4.0]], [[8.0, 7.0, 3.0], [1.0, 5.0, 1.0], [3.0, 6.0, 7.0]]], [[[1.0, 8.0, 3.0], [6.0, 8.0, 1.0], [2.0, 5.0, 7.0]], [[2.0, 2.0, 
6.0], [7.0, 7.0, 6.0], [2.0, 5.0, 6.0]], [[2.0, 8.0, 6.0], [3.0, 3.0, 6.0], [8.0, 7.0, 1.0]]], [[[3.0, 7.0, 7.0], [4.0, 1.0, 
7.0], [6.0, 7.0, 3.0]], [[3.0, 6.0, 1.0], [1.0, 3.0, 3.0], [4.0, 8.0, 4.0]], [[4.0, 3.0, 5.0], [7.0, 7.0, 7.0], [8.0, 7.0, 
3.0]]], [[[2.0, 5.0, 5.0], [5.0, 3.0, 8.0], [1.0, 8.0, 3.0]], [[5.0, 6.0, 4.0], [1.0, 6.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 3.0, 
8.0], [3.0, 4.0, 3.0], [6.0, 5.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_ULRN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 2.0], [6.0, 1.0, 7.0], [1.0, 7.0, 5.0]], [[5.0, 5.0, 2.0], [6.0, 8.0, 5.0], [6.0, 8.0, 
5.0]], [[1.0, 4.0, 2.0], [8.0, 5.0, 7.0], [8.0, 8.0, 2.0]]], [[[2.0, 4.0, 2.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]], [[4.0, 1.0, 
6.0], [2.0, 2.0, 4.0], [1.0, 7.0, 6.0]], [[7.0, 2.0, 4.0], [2.0, 3.0, 8.0], [4.0, 1.0, 6.0]]], [[[2.0, 7.0, 1.0], [4.0, 3.0, 
1.0], [4.0, 6.0, 7.0]], [[3.0, 3.0, 8.0], [3.0, 7.0, 8.0], [4.0, 4.0, 3.0]], [[4.0, 5.0, 6.0], [6.0, 7.0, 1.0], [8.0, 2.0, 
2.0]]], [[[5.0, 6.0, 2.0], [2.0, 2.0, 2.0], [3.0, 3.0, 4.0]], [[3.0, 2.0, 3.0], [1.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[3.0, 5.0, 
5.0], [8.0, 5.0, 1.0], [3.0, 8.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 4.0, 1.0]]], [[[7.0, 4.0, 1.0]]], [[[3.0, 7.0, 7.0]]]]),self.functionspace)
      arg[:3,2:,1:2]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 2.0], [6.0, 1.0, 7.0], [1.0, 7.0, 5.0]], [[5.0, 5.0, 2.0], [6.0, 8.0, 5.0], [6.0, 8.0, 
5.0]], [[1.0, 4.0, 2.0], [1.0, 4.0, 1.0], [8.0, 8.0, 2.0]]], [[[2.0, 4.0, 2.0], [8.0, 7.0, 8.0], [6.0, 7.0, 1.0]], [[4.0, 1.0, 
6.0], [2.0, 2.0, 4.0], [1.0, 7.0, 6.0]], [[7.0, 2.0, 4.0], [7.0, 4.0, 1.0], [4.0, 1.0, 6.0]]], [[[2.0, 7.0, 1.0], [4.0, 3.0, 
1.0], [4.0, 6.0, 7.0]], [[3.0, 3.0, 8.0], [3.0, 7.0, 8.0], [4.0, 4.0, 3.0]], [[4.0, 5.0, 6.0], [3.0, 7.0, 7.0], [8.0, 2.0, 
2.0]]], [[[5.0, 6.0, 2.0], [2.0, 2.0, 2.0], [3.0, 3.0, 4.0]], [[3.0, 2.0, 3.0], [1.0, 1.0, 8.0], [1.0, 5.0, 2.0]], [[3.0, 5.0, 
5.0], [8.0, 5.0, 1.0], [3.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_ULRN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 3.0], [3.0, 3.0, 8.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 8.0, 8.0], [4.0, 2.0, 
6.0]], [[3.0, 5.0, 6.0], [7.0, 5.0, 6.0], [8.0, 6.0, 2.0]]], [[[3.0, 7.0, 5.0], [4.0, 6.0, 7.0], [3.0, 2.0, 6.0]], [[1.0, 8.0, 
4.0], [3.0, 8.0, 3.0], [4.0, 3.0, 4.0]], [[2.0, 3.0, 5.0], [6.0, 8.0, 5.0], [8.0, 5.0, 3.0]]], [[[5.0, 6.0, 6.0], [2.0, 7.0, 
7.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 8.0], [4.0, 6.0, 6.0], [3.0, 4.0, 7.0]], [[7.0, 3.0, 5.0], [5.0, 8.0, 4.0], [2.0, 8.0, 
2.0]]], [[[7.0, 6.0, 4.0], [5.0, 3.0, 8.0], [5.0, 2.0, 1.0]], [[8.0, 2.0, 3.0], [1.0, 7.0, 5.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 
4.0], [1.0, 2.0, 1.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 7.0, 5.0]]], [[[3.0, 7.0, 5.0]]], [[[5.0, 1.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 4.0, 6.0]]], [[[1.0, 6.0, 6.0]]], [[[6.0, 7.0, 6.0]]]])
      arg[:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 3.0], [3.0, 3.0, 8.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 8.0, 8.0], [4.0, 
2.0, 6.0]], [[3.0, 5.0, 6.0], [3.0, 7.0, 5.0], [8.0, 6.0, 2.0]]], [[[3.0, 7.0, 5.0], [4.0, 6.0, 7.0], [3.0, 2.0, 6.0]], [[1.0, 
8.0, 4.0], [3.0, 8.0, 3.0], [4.0, 3.0, 4.0]], [[2.0, 3.0, 5.0], [3.0, 7.0, 5.0], [8.0, 5.0, 3.0]]], [[[5.0, 6.0, 6.0], [2.0, 
7.0, 7.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 8.0], [4.0, 6.0, 6.0], [3.0, 4.0, 7.0]], [[7.0, 3.0, 5.0], [5.0, 1.0, 4.0], [2.0, 8.0, 
2.0]]], [[[7.0, 6.0, 4.0], [5.0, 3.0, 8.0], [5.0, 2.0, 1.0]], [[8.0, 2.0, 3.0], [1.0, 7.0, 5.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 
4.0], [1.0, 2.0, 1.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 3.0], [3.0, 3.0, 8.0], [6.0, 6.0, 2.0]], 
[[6.0, 1.0, 6.0], [4.0, 8.0, 8.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 6.0], [8.0, 4.0, 6.0], [8.0, 6.0, 2.0]]], [[[3.0, 7.0, 5.0], 
[4.0, 6.0, 7.0], [3.0, 2.0, 6.0]], [[1.0, 8.0, 4.0], [3.0, 8.0, 3.0], [4.0, 3.0, 4.0]], [[2.0, 3.0, 5.0], [1.0, 6.0, 6.0], 
[8.0, 5.0, 3.0]]], [[[5.0, 6.0, 6.0], [2.0, 7.0, 7.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 8.0], [4.0, 6.0, 6.0], [3.0, 4.0, 7.0]], 
[[7.0, 3.0, 5.0], [6.0, 7.0, 6.0], [2.0, 8.0, 2.0]]], [[[7.0, 6.0, 4.0], [5.0, 3.0, 8.0], [5.0, 2.0, 1.0]], [[8.0, 2.0, 3.0], 
[1.0, 7.0, 5.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 4.0], [1.0, 2.0, 1.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CUUC(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 2.0], [2.0, 7.0, 6.0], [1.0, 6.0, 3.0]], [[7.0, 5.0, 8.0], [1.0, 2.0, 5.0], [4.0, 7.0, 
3.0]], [[2.0, 1.0, 4.0], [5.0, 7.0, 7.0], [2.0, 4.0, 4.0]]], [[[7.0, 3.0, 6.0], [4.0, 4.0, 3.0], [6.0, 1.0, 8.0]], [[5.0, 8.0, 
7.0], [5.0, 3.0, 3.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 4.0], [4.0, 3.0, 8.0], [2.0, 7.0, 2.0]]], [[[4.0, 1.0, 7.0], [1.0, 6.0, 
8.0], [7.0, 5.0, 5.0]], [[6.0, 5.0, 4.0], [5.0, 7.0, 3.0], [7.0, 7.0, 2.0]], [[6.0, 2.0, 6.0], [2.0, 2.0, 3.0], [5.0, 2.0, 
8.0]]], [[[5.0, 2.0, 8.0], [7.0, 4.0, 5.0], [2.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], [1.0, 6.0, 7.0], [7.0, 5.0, 6.0]], [[8.0, 3.0, 
1.0], [2.0, 1.0, 8.0], [2.0, 2.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 8.0, 2.0], [6.0, 2.0, 8.0]], [[5.0, 7.0, 7.0], [8.0, 3.0, 2.0]]], [[[1.0, 6.0, 7.0], [1.0, 3.0, 
6.0]], [[8.0, 1.0, 1.0], [3.0, 4.0, 6.0]]], [[[2.0, 6.0, 4.0], [4.0, 5.0, 4.0]], [[8.0, 8.0, 8.0], [7.0, 8.0, 6.0]]], [[[1.0, 
4.0, 8.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 6.0], [3.0, 7.0, 1.0]]]])
      arg[:,:2,:2,:]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 2.0], [6.0, 2.0, 8.0], [1.0, 6.0, 3.0]], [[5.0, 7.0, 7.0], [8.0, 3.0, 2.0], [4.0, 7.0, 
3.0]], [[2.0, 1.0, 4.0], [5.0, 7.0, 7.0], [2.0, 4.0, 4.0]]], [[[1.0, 6.0, 7.0], [1.0, 3.0, 6.0], [6.0, 1.0, 8.0]], [[8.0, 1.0, 
1.0], [3.0, 4.0, 6.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 4.0], [4.0, 3.0, 8.0], [2.0, 7.0, 2.0]]], [[[2.0, 6.0, 4.0], [4.0, 5.0, 
4.0], [7.0, 5.0, 5.0]], [[8.0, 8.0, 8.0], [7.0, 8.0, 6.0], [7.0, 7.0, 2.0]], [[6.0, 2.0, 6.0], [2.0, 2.0, 3.0], [5.0, 2.0, 
8.0]]], [[[1.0, 4.0, 8.0], [1.0, 6.0, 5.0], [2.0, 6.0, 5.0]], [[7.0, 2.0, 6.0], [3.0, 7.0, 1.0], [7.0, 5.0, 6.0]], [[8.0, 3.0, 
1.0], [2.0, 1.0, 8.0], [2.0, 2.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CUUC(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 6.0], [2.0, 5.0, 1.0], [6.0, 4.0, 3.0]], [[5.0, 1.0, 7.0], [4.0, 4.0, 6.0], [7.0, 2.0, 
7.0]], [[2.0, 7.0, 4.0], [4.0, 4.0, 6.0], [7.0, 5.0, 8.0]]], [[[4.0, 1.0, 6.0], [7.0, 7.0, 4.0], [7.0, 3.0, 6.0]], [[3.0, 3.0, 
7.0], [4.0, 6.0, 3.0], [5.0, 5.0, 6.0]], [[7.0, 8.0, 2.0], [3.0, 7.0, 5.0], [5.0, 1.0, 3.0]]], [[[6.0, 7.0, 8.0], [1.0, 8.0, 
4.0], [8.0, 6.0, 8.0]], [[2.0, 8.0, 8.0], [2.0, 5.0, 5.0], [7.0, 1.0, 6.0]], [[3.0, 6.0, 2.0], [6.0, 5.0, 2.0], [5.0, 1.0, 
3.0]]], [[[3.0, 4.0, 5.0], [8.0, 5.0, 4.0], [7.0, 7.0, 8.0]], [[7.0, 7.0, 2.0], [1.0, 8.0, 3.0], [2.0, 7.0, 3.0]], [[8.0, 4.0, 
1.0], [3.0, 4.0, 4.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 4.0, 4.0], [6.0, 2.0, 5.0]], [[2.0, 5.0, 1.0], [6.0, 6.0, 1.0]]], [[[7.0, 4.0, 7.0], [3.0, 
7.0, 4.0]], [[4.0, 5.0, 1.0], [2.0, 5.0, 5.0]]], [[[7.0, 3.0, 4.0], [3.0, 2.0, 6.0]], [[2.0, 3.0, 4.0], [5.0, 2.0, 2.0]]], 
[[[6.0, 7.0, 3.0], [5.0, 4.0, 8.0]], [[5.0, 4.0, 6.0], [7.0, 3.0, 4.0]]]]),self.functionspace)
      arg[:,:2,:2,:]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 4.0], [6.0, 2.0, 5.0], [6.0, 4.0, 3.0]], [[2.0, 5.0, 1.0], [6.0, 6.0, 1.0], [7.0, 2.0, 
7.0]], [[2.0, 7.0, 4.0], [4.0, 4.0, 6.0], [7.0, 5.0, 8.0]]], [[[7.0, 4.0, 7.0], [3.0, 7.0, 4.0], [7.0, 3.0, 6.0]], [[4.0, 5.0, 
1.0], [2.0, 5.0, 5.0], [5.0, 5.0, 6.0]], [[7.0, 8.0, 2.0], [3.0, 7.0, 5.0], [5.0, 1.0, 3.0]]], [[[7.0, 3.0, 4.0], [3.0, 2.0, 
6.0], [8.0, 6.0, 8.0]], [[2.0, 3.0, 4.0], [5.0, 2.0, 2.0], [7.0, 1.0, 6.0]], [[3.0, 6.0, 2.0], [6.0, 5.0, 2.0], [5.0, 1.0, 
3.0]]], [[[6.0, 7.0, 3.0], [5.0, 4.0, 8.0], [7.0, 7.0, 8.0]], [[5.0, 4.0, 6.0], [7.0, 3.0, 4.0], [2.0, 7.0, 3.0]], [[8.0, 4.0, 
1.0], [3.0, 4.0, 4.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CUUC(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 1.0], [7.0, 6.0, 4.0], [5.0, 3.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 7.0, 8.0], [3.0, 5.0, 
6.0]], [[5.0, 3.0, 7.0], [8.0, 4.0, 3.0], [3.0, 3.0, 3.0]]], [[[8.0, 3.0, 5.0], [5.0, 7.0, 4.0], [1.0, 8.0, 1.0]], [[7.0, 8.0, 
7.0], [2.0, 3.0, 3.0], [1.0, 8.0, 5.0]], [[6.0, 4.0, 8.0], [8.0, 5.0, 5.0], [2.0, 4.0, 5.0]]], [[[2.0, 7.0, 7.0], [6.0, 8.0, 
4.0], [6.0, 2.0, 5.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 8.0], [3.0, 6.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 2.0, 6.0], [2.0, 4.0, 
6.0]]], [[[3.0, 4.0, 5.0], [8.0, 1.0, 2.0], [3.0, 6.0, 1.0]], [[6.0, 6.0, 1.0], [8.0, 1.0, 7.0], [1.0, 5.0, 6.0]], [[6.0, 6.0, 
8.0], [3.0, 3.0, 5.0], [6.0, 1.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 2.0, 4.0], [7.0, 2.0, 3.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 8.0]]], [[[3.0, 7.0, 5.0], 
[8.0, 2.0, 3.0]], [[8.0, 8.0, 8.0], [6.0, 3.0, 7.0]]], [[[4.0, 6.0, 4.0], [4.0, 5.0, 7.0]], [[5.0, 8.0, 3.0], [2.0, 1.0, 
8.0]]], [[[8.0, 5.0, 4.0], [1.0, 3.0, 5.0]], [[3.0, 8.0, 6.0], [4.0, 3.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 8.0, 6.0], 
[2.0, 3.0, 7.0]], [[8.0, 1.0, 7.0], [8.0, 1.0, 6.0]]], [[[8.0, 1.0, 4.0], [1.0, 2.0, 7.0]], [[6.0, 5.0, 8.0], [4.0, 5.0, 
4.0]]], [[[6.0, 6.0, 8.0], [4.0, 4.0, 3.0]], [[6.0, 7.0, 7.0], [2.0, 3.0, 1.0]]], [[[6.0, 2.0, 7.0], [1.0, 7.0, 3.0]], [[5.0, 
3.0, 8.0], [2.0, 8.0, 3.0]]]])
      arg[:,:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 4.0], [7.0, 2.0, 3.0], [5.0, 3.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 1.0, 8.0], [3.0, 
5.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 4.0, 3.0], [3.0, 3.0, 3.0]]], [[[3.0, 7.0, 5.0], [8.0, 2.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 
8.0, 8.0], [6.0, 3.0, 7.0], [1.0, 8.0, 5.0]], [[6.0, 4.0, 8.0], [8.0, 5.0, 5.0], [2.0, 4.0, 5.0]]], [[[4.0, 6.0, 4.0], [4.0, 
5.0, 7.0], [6.0, 2.0, 5.0]], [[5.0, 8.0, 3.0], [2.0, 1.0, 8.0], [3.0, 6.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 2.0, 6.0], [2.0, 4.0, 
6.0]]], [[[8.0, 5.0, 4.0], [1.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[3.0, 8.0, 6.0], [4.0, 3.0, 5.0], [1.0, 5.0, 6.0]], [[6.0, 6.0, 
8.0], [3.0, 3.0, 5.0], [6.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 3.0, 7.0], [5.0, 3.0, 2.0]], 
[[8.0, 1.0, 7.0], [8.0, 1.0, 6.0], [3.0, 5.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 4.0, 3.0], [3.0, 3.0, 3.0]]], [[[8.0, 1.0, 4.0], 
[1.0, 2.0, 7.0], [1.0, 8.0, 1.0]], [[6.0, 5.0, 8.0], [4.0, 5.0, 4.0], [1.0, 8.0, 5.0]], [[6.0, 4.0, 8.0], [8.0, 5.0, 5.0], 
[2.0, 4.0, 5.0]]], [[[6.0, 6.0, 8.0], [4.0, 4.0, 3.0], [6.0, 2.0, 5.0]], [[6.0, 7.0, 7.0], [2.0, 3.0, 1.0], [3.0, 6.0, 1.0]], 
[[4.0, 3.0, 1.0], [6.0, 2.0, 6.0], [2.0, 4.0, 6.0]]], [[[6.0, 2.0, 7.0], [1.0, 7.0, 3.0], [3.0, 6.0, 1.0]], [[5.0, 3.0, 8.0], 
[2.0, 8.0, 3.0], [1.0, 5.0, 6.0]], [[6.0, 6.0, 8.0], [3.0, 3.0, 5.0], [6.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCUL(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 1.0], [7.0, 1.0, 5.0], [4.0, 2.0, 3.0]], [[6.0, 1.0, 3.0], [3.0, 2.0, 4.0], [7.0, 1.0, 
2.0]], [[5.0, 1.0, 1.0], [7.0, 1.0, 5.0], [5.0, 2.0, 3.0]]], [[[6.0, 3.0, 5.0], [7.0, 5.0, 7.0], [7.0, 5.0, 3.0]], [[6.0, 2.0, 
3.0], [1.0, 6.0, 4.0], [7.0, 1.0, 4.0]], [[8.0, 8.0, 5.0], [8.0, 3.0, 5.0], [2.0, 2.0, 2.0]]], [[[3.0, 5.0, 3.0], [5.0, 1.0, 
4.0], [5.0, 4.0, 8.0]], [[6.0, 7.0, 4.0], [8.0, 3.0, 3.0], [7.0, 5.0, 1.0]], [[1.0, 4.0, 3.0], [6.0, 3.0, 6.0], [7.0, 1.0, 
8.0]]], [[[6.0, 7.0, 3.0], [7.0, 6.0, 1.0], [6.0, 6.0, 7.0]], [[2.0, 6.0, 4.0], [1.0, 1.0, 6.0], [2.0, 7.0, 4.0]], [[8.0, 8.0, 
8.0], [2.0, 8.0, 8.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0], [1.0]], [[8.0], [8.0]], [[7.0], [8.0]]], [[[4.0], [3.0]], [[5.0], [6.0]], [[8.0], [2.0]]], 
[[[8.0], [3.0]], [[7.0], [2.0]], [[7.0], [4.0]]], [[[8.0], [7.0]], [[3.0], [7.0]], [[8.0], [1.0]]]])
      arg[:,:,:2,2:]=val
      ref=Data(numpy.array([[[[6.0, 4.0, 4.0], [7.0, 1.0, 1.0], [4.0, 2.0, 3.0]], [[6.0, 1.0, 8.0], [3.0, 2.0, 8.0], [7.0, 1.0, 
2.0]], [[5.0, 1.0, 7.0], [7.0, 1.0, 8.0], [5.0, 2.0, 3.0]]], [[[6.0, 3.0, 4.0], [7.0, 5.0, 3.0], [7.0, 5.0, 3.0]], [[6.0, 2.0, 
5.0], [1.0, 6.0, 6.0], [7.0, 1.0, 4.0]], [[8.0, 8.0, 8.0], [8.0, 3.0, 2.0], [2.0, 2.0, 2.0]]], [[[3.0, 5.0, 8.0], [5.0, 1.0, 
3.0], [5.0, 4.0, 8.0]], [[6.0, 7.0, 7.0], [8.0, 3.0, 2.0], [7.0, 5.0, 1.0]], [[1.0, 4.0, 7.0], [6.0, 3.0, 4.0], [7.0, 1.0, 
8.0]]], [[[6.0, 7.0, 8.0], [7.0, 6.0, 7.0], [6.0, 6.0, 7.0]], [[2.0, 6.0, 3.0], [1.0, 1.0, 7.0], [2.0, 7.0, 4.0]], [[8.0, 8.0, 
8.0], [2.0, 8.0, 1.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCUL(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 6.0], [7.0, 8.0, 3.0], [4.0, 5.0, 1.0]], [[2.0, 5.0, 3.0], [3.0, 7.0, 1.0], [4.0, 4.0, 
6.0]], [[2.0, 8.0, 5.0], [5.0, 8.0, 3.0], [8.0, 2.0, 1.0]]], [[[4.0, 6.0, 6.0], [6.0, 1.0, 1.0], [7.0, 7.0, 2.0]], [[2.0, 1.0, 
4.0], [3.0, 2.0, 3.0], [2.0, 7.0, 2.0]], [[2.0, 5.0, 2.0], [7.0, 8.0, 2.0], [1.0, 6.0, 7.0]]], [[[2.0, 8.0, 5.0], [1.0, 4.0, 
4.0], [3.0, 6.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 3.0, 2.0], [2.0, 4.0, 7.0]], [[7.0, 6.0, 2.0], [5.0, 2.0, 8.0], [2.0, 3.0, 
6.0]]], [[[2.0, 5.0, 5.0], [5.0, 6.0, 3.0], [1.0, 8.0, 4.0]], [[4.0, 7.0, 1.0], [5.0, 6.0, 7.0], [4.0, 6.0, 7.0]], [[7.0, 7.0, 
6.0], [3.0, 7.0, 2.0], [8.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0], [5.0]], [[2.0], [8.0]], [[4.0], [4.0]]], [[[6.0], [8.0]], [[4.0], [5.0]], [[6.0], [1.0]]], 
[[[2.0], [3.0]], [[7.0], [1.0]], [[1.0], [6.0]]], [[[7.0], [7.0]], [[6.0], [7.0]], [[7.0], [3.0]]]]),self.functionspace)
      arg[:,:,:2,2:]=val
      ref=Data(numpy.array([[[[4.0, 1.0, 3.0], [7.0, 8.0, 5.0], [4.0, 5.0, 1.0]], [[2.0, 5.0, 2.0], [3.0, 7.0, 8.0], [4.0, 4.0, 
6.0]], [[2.0, 8.0, 4.0], [5.0, 8.0, 4.0], [8.0, 2.0, 1.0]]], [[[4.0, 6.0, 6.0], [6.0, 1.0, 8.0], [7.0, 7.0, 2.0]], [[2.0, 1.0, 
4.0], [3.0, 2.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 5.0, 6.0], [7.0, 8.0, 1.0], [1.0, 6.0, 7.0]]], [[[2.0, 8.0, 2.0], [1.0, 4.0, 
3.0], [3.0, 6.0, 4.0]], [[5.0, 6.0, 7.0], [2.0, 3.0, 1.0], [2.0, 4.0, 7.0]], [[7.0, 6.0, 1.0], [5.0, 2.0, 6.0], [2.0, 3.0, 
6.0]]], [[[2.0, 5.0, 7.0], [5.0, 6.0, 7.0], [1.0, 8.0, 4.0]], [[4.0, 7.0, 6.0], [5.0, 6.0, 7.0], [4.0, 6.0, 7.0]], [[7.0, 7.0, 
7.0], [3.0, 7.0, 3.0], [8.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCUL(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [1.0, 7.0, 2.0], [2.0, 1.0, 1.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 4.0], [4.0, 2.0, 
1.0]], [[1.0, 1.0, 8.0], [5.0, 7.0, 6.0], [1.0, 4.0, 7.0]]], [[[3.0, 3.0, 8.0], [8.0, 5.0, 2.0], [6.0, 7.0, 2.0]], [[4.0, 5.0, 
4.0], [8.0, 1.0, 3.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 2.0], [8.0, 1.0, 8.0], [6.0, 1.0, 2.0]]], [[[7.0, 1.0, 2.0], [4.0, 2.0, 
2.0], [6.0, 4.0, 6.0]], [[6.0, 3.0, 8.0], [3.0, 1.0, 3.0], [6.0, 6.0, 5.0]], [[1.0, 7.0, 2.0], [1.0, 5.0, 3.0], [2.0, 4.0, 
1.0]]], [[[1.0, 7.0, 1.0], [5.0, 4.0, 8.0], [7.0, 5.0, 3.0]], [[8.0, 5.0, 5.0], [1.0, 3.0, 8.0], [1.0, 1.0, 2.0]], [[3.0, 6.0, 
6.0], [6.0, 5.0, 4.0], [2.0, 6.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0], [8.0]], [[2.0], [7.0]], [[7.0], [8.0]]], [[[3.0], [5.0]], [[1.0], [7.0]], [[1.0], 
[2.0]]], [[[6.0], [7.0]], [[5.0], [3.0]], [[8.0], [7.0]]], [[[2.0], [8.0]], [[8.0], [3.0]], [[4.0], 
[7.0]]]])+(1.-msk_val)*numpy.array([[[[1.0], [5.0]], [[8.0], [3.0]], [[5.0], [8.0]]], [[[1.0], [5.0]], [[1.0], [5.0]], [[8.0], 
[2.0]]], [[[2.0], [2.0]], [[7.0], [7.0]], [[2.0], [8.0]]], [[[2.0], [4.0]], [[2.0], [5.0]], [[1.0], [1.0]]]])
      arg[:,:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 3.0], [1.0, 7.0, 8.0], [2.0, 1.0, 1.0]], [[2.0, 2.0, 2.0], [7.0, 6.0, 7.0], [4.0, 
2.0, 1.0]], [[1.0, 1.0, 7.0], [5.0, 7.0, 8.0], [1.0, 4.0, 7.0]]], [[[3.0, 3.0, 3.0], [8.0, 5.0, 5.0], [6.0, 7.0, 2.0]], [[4.0, 
5.0, 1.0], [8.0, 1.0, 7.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], [8.0, 1.0, 2.0], [6.0, 1.0, 2.0]]], [[[7.0, 1.0, 6.0], [4.0, 
2.0, 7.0], [6.0, 4.0, 6.0]], [[6.0, 3.0, 5.0], [3.0, 1.0, 3.0], [6.0, 6.0, 5.0]], [[1.0, 7.0, 8.0], [1.0, 5.0, 7.0], [2.0, 4.0, 
1.0]]], [[[1.0, 7.0, 2.0], [5.0, 4.0, 8.0], [7.0, 5.0, 3.0]], [[8.0, 5.0, 8.0], [1.0, 3.0, 3.0], [1.0, 1.0, 2.0]], [[3.0, 6.0, 
4.0], [6.0, 5.0, 7.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 1.0], [1.0, 7.0, 5.0], [2.0, 1.0, 1.0]], 
[[2.0, 2.0, 8.0], [7.0, 6.0, 3.0], [4.0, 2.0, 1.0]], [[1.0, 1.0, 5.0], [5.0, 7.0, 8.0], [1.0, 4.0, 7.0]]], [[[3.0, 3.0, 1.0], 
[8.0, 5.0, 5.0], [6.0, 7.0, 2.0]], [[4.0, 5.0, 1.0], [8.0, 1.0, 5.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 8.0], [8.0, 1.0, 2.0], 
[6.0, 1.0, 2.0]]], [[[7.0, 1.0, 2.0], [4.0, 2.0, 2.0], [6.0, 4.0, 6.0]], [[6.0, 3.0, 7.0], [3.0, 1.0, 7.0], [6.0, 6.0, 5.0]], 
[[1.0, 7.0, 2.0], [1.0, 5.0, 8.0], [2.0, 4.0, 1.0]]], [[[1.0, 7.0, 2.0], [5.0, 4.0, 4.0], [7.0, 5.0, 3.0]], [[8.0, 5.0, 2.0], 
[1.0, 3.0, 5.0], [1.0, 1.0, 2.0]], [[3.0, 6.0, 1.0], [6.0, 5.0, 1.0], [2.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUUU(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 6.0], [1.0, 1.0, 7.0], [2.0, 7.0, 2.0]], [[3.0, 2.0, 5.0], [6.0, 2.0, 8.0], [2.0, 6.0, 
6.0]], [[7.0, 6.0, 2.0], [2.0, 8.0, 3.0], [8.0, 2.0, 7.0]]], [[[4.0, 7.0, 6.0], [8.0, 6.0, 6.0], [2.0, 2.0, 8.0]], [[4.0, 6.0, 
8.0], [5.0, 2.0, 6.0], [3.0, 5.0, 4.0]], [[8.0, 5.0, 2.0], [5.0, 5.0, 1.0], [2.0, 3.0, 3.0]]], [[[6.0, 5.0, 1.0], [2.0, 5.0, 
5.0], [8.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [2.0, 5.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 2.0, 2.0], [6.0, 3.0, 8.0], [6.0, 4.0, 
6.0]]], [[[4.0, 8.0, 8.0], [4.0, 8.0, 6.0], [4.0, 6.0, 8.0]], [[6.0, 7.0, 2.0], [7.0, 1.0, 1.0], [3.0, 2.0, 4.0]], [[2.0, 8.0, 
3.0], [8.0, 1.0, 8.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 2.0], [4.0, 1.0]], [[3.0, 6.0], [8.0, 8.0]]], [[[8.0, 7.0], [1.0, 1.0]], [[3.0, 3.0], [7.0, 
1.0]]]])
      arg[1:3,:2,:2,:2]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 6.0], [1.0, 1.0, 7.0], [2.0, 7.0, 2.0]], [[3.0, 2.0, 5.0], [6.0, 2.0, 8.0], [2.0, 6.0, 
6.0]], [[7.0, 6.0, 2.0], [2.0, 8.0, 3.0], [8.0, 2.0, 7.0]]], [[[4.0, 2.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 8.0]], [[3.0, 6.0, 
8.0], [8.0, 8.0, 6.0], [3.0, 5.0, 4.0]], [[8.0, 5.0, 2.0], [5.0, 5.0, 1.0], [2.0, 3.0, 3.0]]], [[[8.0, 7.0, 1.0], [1.0, 1.0, 
5.0], [8.0, 4.0, 8.0]], [[3.0, 3.0, 3.0], [7.0, 1.0, 8.0], [5.0, 2.0, 8.0]], [[2.0, 2.0, 2.0], [6.0, 3.0, 8.0], [6.0, 4.0, 
6.0]]], [[[4.0, 8.0, 8.0], [4.0, 8.0, 6.0], [4.0, 6.0, 8.0]], [[6.0, 7.0, 2.0], [7.0, 1.0, 1.0], [3.0, 2.0, 4.0]], [[2.0, 8.0, 
3.0], [8.0, 1.0, 8.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUUU(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 6.0], [4.0, 5.0, 5.0], [8.0, 3.0, 7.0]], [[5.0, 7.0, 1.0], [6.0, 2.0, 5.0], [7.0, 1.0, 
5.0]], [[1.0, 2.0, 1.0], [6.0, 3.0, 6.0], [5.0, 2.0, 8.0]]], [[[3.0, 4.0, 2.0], [5.0, 6.0, 8.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 
2.0], [1.0, 1.0, 8.0], [7.0, 2.0, 3.0]], [[8.0, 7.0, 3.0], [6.0, 8.0, 3.0], [4.0, 3.0, 5.0]]], [[[2.0, 2.0, 6.0], [8.0, 4.0, 
1.0], [5.0, 8.0, 1.0]], [[2.0, 6.0, 4.0], [5.0, 4.0, 8.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 6.0], [1.0, 2.0, 7.0], [3.0, 2.0, 
5.0]]], [[[1.0, 4.0, 5.0], [4.0, 8.0, 5.0], [1.0, 3.0, 5.0]], [[7.0, 4.0, 2.0], [2.0, 5.0, 2.0], [5.0, 1.0, 1.0]], [[1.0, 5.0, 
2.0], [1.0, 4.0, 2.0], [4.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 8.0], [1.0, 3.0]], [[5.0, 1.0], [2.0, 1.0]]], [[[1.0, 2.0], [7.0, 5.0]], [[8.0, 1.0], [7.0, 
2.0]]]]),self.functionspace)
      arg[1:3,:2,:2,:2]=val
      ref=Data(numpy.array([[[[1.0, 7.0, 6.0], [4.0, 5.0, 5.0], [8.0, 3.0, 7.0]], [[5.0, 7.0, 1.0], [6.0, 2.0, 5.0], [7.0, 1.0, 
5.0]], [[1.0, 2.0, 1.0], [6.0, 3.0, 6.0], [5.0, 2.0, 8.0]]], [[[2.0, 8.0, 2.0], [1.0, 3.0, 8.0], [5.0, 3.0, 1.0]], [[5.0, 1.0, 
2.0], [2.0, 1.0, 8.0], [7.0, 2.0, 3.0]], [[8.0, 7.0, 3.0], [6.0, 8.0, 3.0], [4.0, 3.0, 5.0]]], [[[1.0, 2.0, 6.0], [7.0, 5.0, 
1.0], [5.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [7.0, 2.0, 8.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 6.0], [1.0, 2.0, 7.0], [3.0, 2.0, 
5.0]]], [[[1.0, 4.0, 5.0], [4.0, 8.0, 5.0], [1.0, 3.0, 5.0]], [[7.0, 4.0, 2.0], [2.0, 5.0, 2.0], [5.0, 1.0, 1.0]], [[1.0, 5.0, 
2.0], [1.0, 4.0, 2.0], [4.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUUU(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 5.0], [5.0, 2.0, 5.0], [3.0, 6.0, 8.0]], [[8.0, 8.0, 5.0], [8.0, 2.0, 8.0], [7.0, 7.0, 
8.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 8.0], [6.0, 8.0, 5.0]]], [[[6.0, 7.0, 4.0], [5.0, 2.0, 4.0], [7.0, 1.0, 4.0]], [[7.0, 7.0, 
8.0], [8.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 6.0], [8.0, 3.0, 7.0]]], [[[5.0, 6.0, 5.0], [8.0, 7.0, 
7.0], [1.0, 8.0, 8.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 3.0], [8.0, 7.0, 4.0]], [[7.0, 1.0, 3.0], [8.0, 5.0, 1.0], [8.0, 4.0, 
1.0]]], [[[8.0, 4.0, 2.0], [6.0, 4.0, 5.0], [6.0, 2.0, 1.0]], [[1.0, 3.0, 1.0], [1.0, 8.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 1.0, 
2.0], [2.0, 1.0, 1.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 3.0], [5.0, 4.0]], [[4.0, 6.0], [5.0, 7.0]]], [[[6.0, 6.0], [6.0, 2.0]], [[3.0, 3.0], 
[1.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 3.0], [7.0, 8.0]], [[5.0, 7.0], [3.0, 6.0]]], [[[3.0, 5.0], [5.0, 4.0]], 
[[2.0, 7.0], [3.0, 7.0]]]])
      arg[1:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 5.0], [5.0, 2.0, 5.0], [3.0, 6.0, 8.0]], [[8.0, 8.0, 5.0], [8.0, 2.0, 8.0], [7.0, 
7.0, 8.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 8.0], [6.0, 8.0, 5.0]]], [[[3.0, 3.0, 4.0], [5.0, 4.0, 4.0], [7.0, 1.0, 4.0]], [[4.0, 
6.0, 8.0], [5.0, 7.0, 5.0], [6.0, 1.0, 5.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 6.0], [8.0, 3.0, 7.0]]], [[[6.0, 6.0, 5.0], [6.0, 
2.0, 7.0], [1.0, 8.0, 8.0]], [[3.0, 3.0, 4.0], [1.0, 3.0, 3.0], [8.0, 7.0, 4.0]], [[7.0, 1.0, 3.0], [8.0, 5.0, 1.0], [8.0, 4.0, 
1.0]]], [[[8.0, 4.0, 2.0], [6.0, 4.0, 5.0], [6.0, 2.0, 1.0]], [[1.0, 3.0, 1.0], [1.0, 8.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 1.0, 
2.0], [2.0, 1.0, 1.0], [3.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 5.0], [5.0, 2.0, 5.0], [3.0, 6.0, 8.0]], 
[[8.0, 8.0, 5.0], [8.0, 2.0, 8.0], [7.0, 7.0, 8.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 8.0], [6.0, 8.0, 5.0]]], [[[6.0, 3.0, 4.0], 
[7.0, 8.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 7.0, 8.0], [3.0, 6.0, 5.0], [6.0, 1.0, 5.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 6.0], 
[8.0, 3.0, 7.0]]], [[[3.0, 5.0, 5.0], [5.0, 4.0, 7.0], [1.0, 8.0, 8.0]], [[2.0, 7.0, 4.0], [3.0, 7.0, 3.0], [8.0, 7.0, 4.0]], 
[[7.0, 1.0, 3.0], [8.0, 5.0, 1.0], [8.0, 4.0, 1.0]]], [[[8.0, 4.0, 2.0], [6.0, 4.0, 5.0], [6.0, 2.0, 1.0]], [[1.0, 3.0, 1.0], 
[1.0, 8.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [2.0, 1.0, 1.0], [3.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLPU(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 2.0], [3.0, 2.0, 8.0], [5.0, 4.0, 1.0]], [[2.0, 2.0, 1.0], [2.0, 7.0, 1.0], [6.0, 4.0, 
1.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 8.0], [8.0, 4.0, 3.0]]], [[[1.0, 4.0, 6.0], [1.0, 1.0, 2.0], [8.0, 7.0, 5.0]], [[2.0, 3.0, 
8.0], [7.0, 4.0, 5.0], [6.0, 7.0, 5.0]], [[2.0, 4.0, 1.0], [2.0, 1.0, 2.0], [5.0, 1.0, 2.0]]], [[[1.0, 8.0, 1.0], [5.0, 3.0, 
1.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 4.0], [3.0, 8.0, 4.0], [2.0, 1.0, 8.0]], [[2.0, 3.0, 4.0], [4.0, 8.0, 6.0], [6.0, 1.0, 
3.0]]], [[[7.0, 3.0, 5.0], [8.0, 5.0, 7.0], [6.0, 2.0, 5.0]], [[7.0, 1.0, 6.0], [5.0, 4.0, 2.0], [1.0, 2.0, 6.0]], [[5.0, 7.0, 
4.0], [7.0, 6.0, 8.0], [8.0, 6.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[8.0, 6.0]])
      arg[2,2:,1,:2]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 2.0], [3.0, 2.0, 8.0], [5.0, 4.0, 1.0]], [[2.0, 2.0, 1.0], [2.0, 7.0, 1.0], [6.0, 4.0, 
1.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 8.0], [8.0, 4.0, 3.0]]], [[[1.0, 4.0, 6.0], [1.0, 1.0, 2.0], [8.0, 7.0, 5.0]], [[2.0, 3.0, 
8.0], [7.0, 4.0, 5.0], [6.0, 7.0, 5.0]], [[2.0, 4.0, 1.0], [2.0, 1.0, 2.0], [5.0, 1.0, 2.0]]], [[[1.0, 8.0, 1.0], [5.0, 3.0, 
1.0], [4.0, 4.0, 1.0]], [[2.0, 7.0, 4.0], [3.0, 8.0, 4.0], [2.0, 1.0, 8.0]], [[2.0, 3.0, 4.0], [8.0, 6.0, 6.0], [6.0, 1.0, 
3.0]]], [[[7.0, 3.0, 5.0], [8.0, 5.0, 7.0], [6.0, 2.0, 5.0]], [[7.0, 1.0, 6.0], [5.0, 4.0, 2.0], [1.0, 2.0, 6.0]], [[5.0, 7.0, 
4.0], [7.0, 6.0, 8.0], [8.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLPU(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 1.0], [1.0, 2.0, 7.0], [3.0, 5.0, 8.0]], [[6.0, 5.0, 5.0], [2.0, 6.0, 8.0], [7.0, 7.0, 
2.0]], [[3.0, 6.0, 3.0], [3.0, 1.0, 2.0], [6.0, 4.0, 6.0]]], [[[2.0, 4.0, 5.0], [7.0, 8.0, 2.0], [8.0, 7.0, 7.0]], [[1.0, 6.0, 
6.0], [2.0, 5.0, 5.0], [3.0, 8.0, 7.0]], [[1.0, 7.0, 5.0], [3.0, 4.0, 7.0], [3.0, 3.0, 1.0]]], [[[8.0, 1.0, 2.0], [4.0, 1.0, 
5.0], [4.0, 6.0, 4.0]], [[8.0, 8.0, 3.0], [7.0, 4.0, 6.0], [5.0, 8.0, 3.0]], [[8.0, 3.0, 2.0], [8.0, 1.0, 4.0], [1.0, 7.0, 
1.0]]], [[[3.0, 1.0, 8.0], [7.0, 6.0, 2.0], [6.0, 2.0, 4.0]], [[5.0, 7.0, 5.0], [1.0, 5.0, 4.0], [4.0, 2.0, 4.0]], [[3.0, 2.0, 
6.0], [3.0, 4.0, 3.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 5.0]]),self.functionspace)
      arg[2,2:,1,:2]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 1.0], [1.0, 2.0, 7.0], [3.0, 5.0, 8.0]], [[6.0, 5.0, 5.0], [2.0, 6.0, 8.0], [7.0, 7.0, 
2.0]], [[3.0, 6.0, 3.0], [3.0, 1.0, 2.0], [6.0, 4.0, 6.0]]], [[[2.0, 4.0, 5.0], [7.0, 8.0, 2.0], [8.0, 7.0, 7.0]], [[1.0, 6.0, 
6.0], [2.0, 5.0, 5.0], [3.0, 8.0, 7.0]], [[1.0, 7.0, 5.0], [3.0, 4.0, 7.0], [3.0, 3.0, 1.0]]], [[[8.0, 1.0, 2.0], [4.0, 1.0, 
5.0], [4.0, 6.0, 4.0]], [[8.0, 8.0, 3.0], [7.0, 4.0, 6.0], [5.0, 8.0, 3.0]], [[8.0, 3.0, 2.0], [4.0, 5.0, 4.0], [1.0, 7.0, 
1.0]]], [[[3.0, 1.0, 8.0], [7.0, 6.0, 2.0], [6.0, 2.0, 4.0]], [[5.0, 7.0, 5.0], [1.0, 5.0, 4.0], [4.0, 2.0, 4.0]], [[3.0, 2.0, 
6.0], [3.0, 4.0, 3.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLPU(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 6.0], [4.0, 8.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 5.0, 2.0], [1.0, 4.0, 4.0], [6.0, 1.0, 
2.0]], [[6.0, 8.0, 2.0], [6.0, 4.0, 1.0], [8.0, 3.0, 1.0]]], [[[1.0, 3.0, 8.0], [3.0, 7.0, 6.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 
2.0], [5.0, 8.0, 6.0], [4.0, 6.0, 6.0]], [[6.0, 1.0, 4.0], [5.0, 1.0, 3.0], [3.0, 8.0, 7.0]]], [[[7.0, 6.0, 3.0], [5.0, 2.0, 
3.0], [4.0, 8.0, 5.0]], [[6.0, 6.0, 4.0], [2.0, 7.0, 5.0], [6.0, 5.0, 2.0]], [[2.0, 3.0, 4.0], [8.0, 7.0, 8.0], [3.0, 6.0, 
4.0]]], [[[6.0, 8.0, 1.0], [4.0, 6.0, 4.0], [8.0, 7.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 5.0, 4.0], [8.0, 3.0, 5.0]], [[1.0, 3.0, 
7.0], [8.0, 5.0, 5.0], [1.0, 8.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 4.0]])+(1.-msk_val)*numpy.array([[6.0, 8.0]])
      arg[2,2:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 6.0], [4.0, 8.0, 1.0], [3.0, 7.0, 2.0]], [[8.0, 5.0, 2.0], [1.0, 4.0, 4.0], [6.0, 
1.0, 2.0]], [[6.0, 8.0, 2.0], [6.0, 4.0, 1.0], [8.0, 3.0, 1.0]]], [[[1.0, 3.0, 8.0], [3.0, 7.0, 6.0], [1.0, 5.0, 5.0]], [[2.0, 
2.0, 2.0], [5.0, 8.0, 6.0], [4.0, 6.0, 6.0]], [[6.0, 1.0, 4.0], [5.0, 1.0, 3.0], [3.0, 8.0, 7.0]]], [[[7.0, 6.0, 3.0], [5.0, 
2.0, 3.0], [4.0, 8.0, 5.0]], [[6.0, 6.0, 4.0], [2.0, 7.0, 5.0], [6.0, 5.0, 2.0]], [[2.0, 3.0, 4.0], [2.0, 4.0, 8.0], [3.0, 6.0, 
4.0]]], [[[6.0, 8.0, 1.0], [4.0, 6.0, 4.0], [8.0, 7.0, 2.0]], [[3.0, 1.0, 1.0], [7.0, 5.0, 4.0], [8.0, 3.0, 5.0]], [[1.0, 3.0, 
7.0], [8.0, 5.0, 5.0], [1.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 6.0], [4.0, 8.0, 1.0], [3.0, 7.0, 2.0]], 
[[8.0, 5.0, 2.0], [1.0, 4.0, 4.0], [6.0, 1.0, 2.0]], [[6.0, 8.0, 2.0], [6.0, 4.0, 1.0], [8.0, 3.0, 1.0]]], [[[1.0, 3.0, 8.0], 
[3.0, 7.0, 6.0], [1.0, 5.0, 5.0]], [[2.0, 2.0, 2.0], [5.0, 8.0, 6.0], [4.0, 6.0, 6.0]], [[6.0, 1.0, 4.0], [5.0, 1.0, 3.0], 
[3.0, 8.0, 7.0]]], [[[7.0, 6.0, 3.0], [5.0, 2.0, 3.0], [4.0, 8.0, 5.0]], [[6.0, 6.0, 4.0], [2.0, 7.0, 5.0], [6.0, 5.0, 2.0]], 
[[2.0, 3.0, 4.0], [6.0, 8.0, 8.0], [3.0, 6.0, 4.0]]], [[[6.0, 8.0, 1.0], [4.0, 6.0, 4.0], [8.0, 7.0, 2.0]], [[3.0, 1.0, 1.0], 
[7.0, 5.0, 4.0], [8.0, 3.0, 5.0]], [[1.0, 3.0, 7.0], [8.0, 5.0, 5.0], [1.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRUC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 8.0], [7.0, 1.0, 6.0], [7.0, 7.0, 6.0]], [[5.0, 8.0, 1.0], [4.0, 4.0, 3.0], [2.0, 1.0, 
2.0]], [[8.0, 2.0, 2.0], [5.0, 1.0, 3.0], [6.0, 3.0, 8.0]]], [[[6.0, 6.0, 1.0], [2.0, 6.0, 7.0], [8.0, 3.0, 1.0]], [[1.0, 5.0, 
6.0], [6.0, 3.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 5.0, 7.0], [1.0, 5.0, 5.0], [1.0, 1.0, 1.0]]], [[[3.0, 1.0, 5.0], [7.0, 8.0, 
2.0], [3.0, 2.0, 6.0]], [[2.0, 1.0, 8.0], [5.0, 3.0, 8.0], [4.0, 1.0, 6.0]], [[3.0, 6.0, 5.0], [6.0, 4.0, 5.0], [6.0, 2.0, 
5.0]]], [[[2.0, 5.0, 6.0], [2.0, 6.0, 5.0], [7.0, 3.0, 8.0]], [[6.0, 1.0, 6.0], [7.0, 5.0, 1.0], [7.0, 6.0, 8.0]], [[5.0, 4.0, 
1.0], [1.0, 4.0, 4.0], [7.0, 2.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 1.0, 3.0], [3.0, 3.0, 4.0]]])
      arg[2,1:2,:2,:]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 8.0], [7.0, 1.0, 6.0], [7.0, 7.0, 6.0]], [[5.0, 8.0, 1.0], [4.0, 4.0, 3.0], [2.0, 1.0, 
2.0]], [[8.0, 2.0, 2.0], [5.0, 1.0, 3.0], [6.0, 3.0, 8.0]]], [[[6.0, 6.0, 1.0], [2.0, 6.0, 7.0], [8.0, 3.0, 1.0]], [[1.0, 5.0, 
6.0], [6.0, 3.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 5.0, 7.0], [1.0, 5.0, 5.0], [1.0, 1.0, 1.0]]], [[[3.0, 1.0, 5.0], [7.0, 8.0, 
2.0], [3.0, 2.0, 6.0]], [[3.0, 1.0, 3.0], [3.0, 3.0, 4.0], [4.0, 1.0, 6.0]], [[3.0, 6.0, 5.0], [6.0, 4.0, 5.0], [6.0, 2.0, 
5.0]]], [[[2.0, 5.0, 6.0], [2.0, 6.0, 5.0], [7.0, 3.0, 8.0]], [[6.0, 1.0, 6.0], [7.0, 5.0, 1.0], [7.0, 6.0, 8.0]], [[5.0, 4.0, 
1.0], [1.0, 4.0, 4.0], [7.0, 2.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRUC(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 1.0], [7.0, 1.0, 7.0], [5.0, 7.0, 6.0]], [[8.0, 6.0, 7.0], [3.0, 1.0, 6.0], [6.0, 3.0, 
5.0]], [[5.0, 3.0, 5.0], [8.0, 6.0, 8.0], [3.0, 7.0, 4.0]]], [[[5.0, 8.0, 5.0], [7.0, 4.0, 8.0], [4.0, 5.0, 4.0]], [[3.0, 7.0, 
5.0], [1.0, 8.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 4.0, 1.0], [6.0, 7.0, 7.0], [5.0, 3.0, 2.0]]], [[[4.0, 2.0, 3.0], [5.0, 4.0, 
4.0], [8.0, 2.0, 8.0]], [[5.0, 3.0, 2.0], [7.0, 6.0, 5.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 4.0], [6.0, 8.0, 2.0], [5.0, 8.0, 
6.0]]], [[[4.0, 3.0, 5.0], [5.0, 4.0, 2.0], [8.0, 4.0, 8.0]], [[2.0, 8.0, 6.0], [5.0, 2.0, 4.0], [8.0, 3.0, 8.0]], [[5.0, 7.0, 
8.0], [7.0, 6.0, 7.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 5.0, 6.0], [7.0, 2.0, 3.0]]]),self.functionspace)
      arg[2,1:2,:2,:]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 1.0], [7.0, 1.0, 7.0], [5.0, 7.0, 6.0]], [[8.0, 6.0, 7.0], [3.0, 1.0, 6.0], [6.0, 3.0, 
5.0]], [[5.0, 3.0, 5.0], [8.0, 6.0, 8.0], [3.0, 7.0, 4.0]]], [[[5.0, 8.0, 5.0], [7.0, 4.0, 8.0], [4.0, 5.0, 4.0]], [[3.0, 7.0, 
5.0], [1.0, 8.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 4.0, 1.0], [6.0, 7.0, 7.0], [5.0, 3.0, 2.0]]], [[[4.0, 2.0, 3.0], [5.0, 4.0, 
4.0], [8.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [7.0, 2.0, 3.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 4.0], [6.0, 8.0, 2.0], [5.0, 8.0, 
6.0]]], [[[4.0, 3.0, 5.0], [5.0, 4.0, 2.0], [8.0, 4.0, 8.0]], [[2.0, 8.0, 6.0], [5.0, 2.0, 4.0], [8.0, 3.0, 8.0]], [[5.0, 7.0, 
8.0], [7.0, 6.0, 7.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRUC(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 8.0], [5.0, 5.0, 1.0], [3.0, 8.0, 6.0]], [[6.0, 4.0, 6.0], [5.0, 8.0, 7.0], [2.0, 3.0, 
4.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 3.0], [7.0, 8.0, 7.0]]], [[[6.0, 8.0, 1.0], [6.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[8.0, 3.0, 
8.0], [4.0, 7.0, 4.0], [3.0, 2.0, 4.0]], [[8.0, 6.0, 8.0], [2.0, 2.0, 7.0], [3.0, 8.0, 8.0]]], [[[1.0, 2.0, 5.0], [8.0, 1.0, 
1.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 1.0], [1.0, 6.0, 6.0], [8.0, 7.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 5.0, 1.0], [7.0, 7.0, 
3.0]]], [[[7.0, 3.0, 8.0], [6.0, 3.0, 1.0], [6.0, 4.0, 8.0]], [[6.0, 4.0, 1.0], [4.0, 7.0, 4.0], [5.0, 7.0, 8.0]], [[6.0, 4.0, 
4.0], [3.0, 8.0, 2.0], [7.0, 7.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 7.0, 8.0], [2.0, 1.0, 3.0]]])+(1.-msk_val)*numpy.array([[[2.0, 6.0, 4.0], [4.0, 4.0, 
3.0]]])
      arg[2,1:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 8.0], [5.0, 5.0, 1.0], [3.0, 8.0, 6.0]], [[6.0, 4.0, 6.0], [5.0, 8.0, 7.0], [2.0, 
3.0, 4.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 3.0], [7.0, 8.0, 7.0]]], [[[6.0, 8.0, 1.0], [6.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[8.0, 
3.0, 8.0], [4.0, 7.0, 4.0], [3.0, 2.0, 4.0]], [[8.0, 6.0, 8.0], [2.0, 2.0, 7.0], [3.0, 8.0, 8.0]]], [[[1.0, 2.0, 5.0], [8.0, 
1.0, 1.0], [4.0, 2.0, 1.0]], [[2.0, 7.0, 8.0], [2.0, 1.0, 3.0], [8.0, 7.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 5.0, 1.0], [7.0, 7.0, 
3.0]]], [[[7.0, 3.0, 8.0], [6.0, 3.0, 1.0], [6.0, 4.0, 8.0]], [[6.0, 4.0, 1.0], [4.0, 7.0, 4.0], [5.0, 7.0, 8.0]], [[6.0, 4.0, 
4.0], [3.0, 8.0, 2.0], [7.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 8.0], [5.0, 5.0, 1.0], [3.0, 8.0, 6.0]], 
[[6.0, 4.0, 6.0], [5.0, 8.0, 7.0], [2.0, 3.0, 4.0]], [[1.0, 3.0, 6.0], [8.0, 3.0, 3.0], [7.0, 8.0, 7.0]]], [[[6.0, 8.0, 1.0], 
[6.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[8.0, 3.0, 8.0], [4.0, 7.0, 4.0], [3.0, 2.0, 4.0]], [[8.0, 6.0, 8.0], [2.0, 2.0, 7.0], 
[3.0, 8.0, 8.0]]], [[[1.0, 2.0, 5.0], [8.0, 1.0, 1.0], [4.0, 2.0, 1.0]], [[2.0, 6.0, 4.0], [4.0, 4.0, 3.0], [8.0, 7.0, 6.0]], 
[[7.0, 3.0, 2.0], [8.0, 5.0, 1.0], [7.0, 7.0, 3.0]]], [[[7.0, 3.0, 8.0], [6.0, 3.0, 1.0], [6.0, 4.0, 8.0]], [[6.0, 4.0, 1.0], 
[4.0, 7.0, 4.0], [5.0, 7.0, 8.0]], [[6.0, 4.0, 4.0], [3.0, 8.0, 2.0], [7.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCRC(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 8.0], [1.0, 3.0, 3.0], [3.0, 1.0, 3.0]], [[5.0, 2.0, 7.0], [2.0, 5.0, 8.0], [6.0, 2.0, 
3.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [6.0, 6.0, 7.0]]], [[[5.0, 3.0, 2.0], [3.0, 8.0, 5.0], [2.0, 5.0, 3.0]], [[2.0, 6.0, 
2.0], [3.0, 6.0, 5.0], [7.0, 3.0, 7.0]], [[5.0, 7.0, 4.0], [1.0, 3.0, 6.0], [7.0, 7.0, 7.0]]], [[[1.0, 6.0, 2.0], [3.0, 5.0, 
3.0], [1.0, 7.0, 3.0]], [[2.0, 3.0, 8.0], [7.0, 4.0, 8.0], [1.0, 5.0, 7.0]], [[1.0, 7.0, 4.0], [1.0, 2.0, 4.0], [5.0, 2.0, 
5.0]]], [[[3.0, 1.0, 6.0], [3.0, 1.0, 1.0], [6.0, 1.0, 3.0]], [[4.0, 3.0, 2.0], [5.0, 7.0, 5.0], [2.0, 4.0, 3.0]], [[7.0, 8.0, 
2.0], [2.0, 2.0, 4.0], [8.0, 3.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 1.0, 8.0]], [[1.0, 3.0, 5.0]], [[4.0, 1.0, 2.0]]], [[[7.0, 4.0, 6.0]], [[7.0, 7.0, 3.0]], [[5.0, 
7.0, 2.0]]]])
      arg[1:3,:,1:2,:]=val
      ref=Data(numpy.array([[[[7.0, 5.0, 8.0], [1.0, 3.0, 3.0], [3.0, 1.0, 3.0]], [[5.0, 2.0, 7.0], [2.0, 5.0, 8.0], [6.0, 2.0, 
3.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [6.0, 6.0, 7.0]]], [[[5.0, 3.0, 2.0], [5.0, 1.0, 8.0], [2.0, 5.0, 3.0]], [[2.0, 6.0, 
2.0], [1.0, 3.0, 5.0], [7.0, 3.0, 7.0]], [[5.0, 7.0, 4.0], [4.0, 1.0, 2.0], [7.0, 7.0, 7.0]]], [[[1.0, 6.0, 2.0], [7.0, 4.0, 
6.0], [1.0, 7.0, 3.0]], [[2.0, 3.0, 8.0], [7.0, 7.0, 3.0], [1.0, 5.0, 7.0]], [[1.0, 7.0, 4.0], [5.0, 7.0, 2.0], [5.0, 2.0, 
5.0]]], [[[3.0, 1.0, 6.0], [3.0, 1.0, 1.0], [6.0, 1.0, 3.0]], [[4.0, 3.0, 2.0], [5.0, 7.0, 5.0], [2.0, 4.0, 3.0]], [[7.0, 8.0, 
2.0], [2.0, 2.0, 4.0], [8.0, 3.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCRC(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 1.0], [6.0, 6.0, 6.0], [4.0, 3.0, 5.0]], [[4.0, 1.0, 2.0], [1.0, 6.0, 4.0], [4.0, 2.0, 
8.0]], [[5.0, 6.0, 3.0], [4.0, 6.0, 8.0], [6.0, 3.0, 7.0]]], [[[1.0, 4.0, 6.0], [2.0, 7.0, 5.0], [6.0, 6.0, 4.0]], [[4.0, 8.0, 
6.0], [7.0, 7.0, 6.0], [2.0, 2.0, 3.0]], [[2.0, 4.0, 6.0], [1.0, 6.0, 7.0], [8.0, 3.0, 8.0]]], [[[1.0, 3.0, 3.0], [8.0, 4.0, 
7.0], [7.0, 2.0, 4.0]], [[2.0, 7.0, 8.0], [8.0, 5.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], [2.0, 1.0, 2.0], [4.0, 1.0, 
6.0]]], [[[2.0, 5.0, 3.0], [4.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[3.0, 5.0, 1.0], [1.0, 4.0, 3.0], [4.0, 1.0, 8.0]], [[1.0, 3.0, 
3.0], [4.0, 7.0, 2.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 1.0, 1.0]], [[7.0, 1.0, 4.0]], [[3.0, 5.0, 8.0]]], [[[2.0, 6.0, 7.0]], [[2.0, 1.0, 4.0]], 
[[2.0, 2.0, 6.0]]]]),self.functionspace)
      arg[1:3,:,1:2,:]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 1.0], [6.0, 6.0, 6.0], [4.0, 3.0, 5.0]], [[4.0, 1.0, 2.0], [1.0, 6.0, 4.0], [4.0, 2.0, 
8.0]], [[5.0, 6.0, 3.0], [4.0, 6.0, 8.0], [6.0, 3.0, 7.0]]], [[[1.0, 4.0, 6.0], [8.0, 1.0, 1.0], [6.0, 6.0, 4.0]], [[4.0, 8.0, 
6.0], [7.0, 1.0, 4.0], [2.0, 2.0, 3.0]], [[2.0, 4.0, 6.0], [3.0, 5.0, 8.0], [8.0, 3.0, 8.0]]], [[[1.0, 3.0, 3.0], [2.0, 6.0, 
7.0], [7.0, 2.0, 4.0]], [[2.0, 7.0, 8.0], [2.0, 1.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], [2.0, 2.0, 6.0], [4.0, 1.0, 
6.0]]], [[[2.0, 5.0, 3.0], [4.0, 1.0, 8.0], [5.0, 2.0, 7.0]], [[3.0, 5.0, 1.0], [1.0, 4.0, 3.0], [4.0, 1.0, 8.0]], [[1.0, 3.0, 
3.0], [4.0, 7.0, 2.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCRC(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 5.0], [2.0, 1.0, 5.0], [1.0, 1.0, 5.0]], [[5.0, 2.0, 1.0], [4.0, 1.0, 7.0], [2.0, 2.0, 
4.0]], [[5.0, 2.0, 3.0], [3.0, 1.0, 2.0], [5.0, 5.0, 8.0]]], [[[8.0, 2.0, 8.0], [5.0, 8.0, 3.0], [3.0, 1.0, 4.0]], [[6.0, 4.0, 
1.0], [4.0, 5.0, 6.0], [6.0, 7.0, 1.0]], [[8.0, 4.0, 5.0], [4.0, 7.0, 5.0], [7.0, 4.0, 7.0]]], [[[5.0, 6.0, 1.0], [2.0, 2.0, 
4.0], [4.0, 7.0, 6.0]], [[1.0, 5.0, 6.0], [8.0, 4.0, 6.0], [1.0, 4.0, 6.0]], [[4.0, 4.0, 5.0], [6.0, 8.0, 1.0], [3.0, 2.0, 
4.0]]], [[[4.0, 5.0, 4.0], [3.0, 4.0, 6.0], [6.0, 1.0, 2.0]], [[1.0, 7.0, 3.0], [5.0, 7.0, 2.0], [6.0, 2.0, 2.0]], [[8.0, 4.0, 
6.0], [4.0, 2.0, 8.0], [4.0, 5.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 7.0, 3.0]], [[2.0, 7.0, 5.0]], [[1.0, 1.0, 5.0]]], [[[7.0, 8.0, 3.0]], [[7.0, 6.0, 
7.0]], [[5.0, 4.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 6.0, 4.0]], [[6.0, 7.0, 2.0]], [[1.0, 3.0, 6.0]]], [[[3.0, 4.0, 
3.0]], [[6.0, 8.0, 7.0]], [[1.0, 1.0, 2.0]]]])
      arg[1:3,:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 5.0], [2.0, 1.0, 5.0], [1.0, 1.0, 5.0]], [[5.0, 2.0, 1.0], [4.0, 1.0, 7.0], [2.0, 
2.0, 4.0]], [[5.0, 2.0, 3.0], [3.0, 1.0, 2.0], [5.0, 5.0, 8.0]]], [[[8.0, 2.0, 8.0], [8.0, 7.0, 3.0], [3.0, 1.0, 4.0]], [[6.0, 
4.0, 1.0], [2.0, 7.0, 5.0], [6.0, 7.0, 1.0]], [[8.0, 4.0, 5.0], [1.0, 1.0, 5.0], [7.0, 4.0, 7.0]]], [[[5.0, 6.0, 1.0], [7.0, 
8.0, 3.0], [4.0, 7.0, 6.0]], [[1.0, 5.0, 6.0], [7.0, 6.0, 7.0], [1.0, 4.0, 6.0]], [[4.0, 4.0, 5.0], [5.0, 4.0, 4.0], [3.0, 2.0, 
4.0]]], [[[4.0, 5.0, 4.0], [3.0, 4.0, 6.0], [6.0, 1.0, 2.0]], [[1.0, 7.0, 3.0], [5.0, 7.0, 2.0], [6.0, 2.0, 2.0]], [[8.0, 4.0, 
6.0], [4.0, 2.0, 8.0], [4.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 5.0], [2.0, 1.0, 5.0], [1.0, 1.0, 5.0]], 
[[5.0, 2.0, 1.0], [4.0, 1.0, 7.0], [2.0, 2.0, 4.0]], [[5.0, 2.0, 3.0], [3.0, 1.0, 2.0], [5.0, 5.0, 8.0]]], [[[8.0, 2.0, 8.0], 
[5.0, 6.0, 4.0], [3.0, 1.0, 4.0]], [[6.0, 4.0, 1.0], [6.0, 7.0, 2.0], [6.0, 7.0, 1.0]], [[8.0, 4.0, 5.0], [1.0, 3.0, 6.0], 
[7.0, 4.0, 7.0]]], [[[5.0, 6.0, 1.0], [3.0, 4.0, 3.0], [4.0, 7.0, 6.0]], [[1.0, 5.0, 6.0], [6.0, 8.0, 7.0], [1.0, 4.0, 6.0]], 
[[4.0, 4.0, 5.0], [1.0, 1.0, 2.0], [3.0, 2.0, 4.0]]], [[[4.0, 5.0, 4.0], [3.0, 4.0, 6.0], [6.0, 1.0, 2.0]], [[1.0, 7.0, 3.0], 
[5.0, 7.0, 2.0], [6.0, 2.0, 2.0]], [[8.0, 4.0, 6.0], [4.0, 2.0, 8.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RURN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 6.0], [5.0, 5.0, 1.0], [5.0, 7.0, 3.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 5.0], [4.0, 3.0, 
1.0]], [[2.0, 6.0, 6.0], [1.0, 6.0, 2.0], [1.0, 7.0, 2.0]]], [[[4.0, 3.0, 1.0], [1.0, 7.0, 8.0], [7.0, 8.0, 6.0]], [[3.0, 5.0, 
2.0], [1.0, 5.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 2.0, 3.0], [4.0, 5.0, 2.0]]], [[[3.0, 7.0, 6.0], [1.0, 6.0, 
3.0], [5.0, 5.0, 4.0]], [[8.0, 2.0, 8.0], [3.0, 6.0, 3.0], [5.0, 2.0, 2.0]], [[4.0, 3.0, 2.0], [2.0, 1.0, 2.0], [2.0, 5.0, 
7.0]]], [[[5.0, 6.0, 8.0], [3.0, 3.0, 3.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [1.0, 2.0, 8.0], [7.0, 8.0, 2.0]], [[2.0, 4.0, 
7.0], [5.0, 5.0, 1.0], [8.0, 3.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 8.0, 1.0]], [[5.0, 4.0, 5.0]]], [[[5.0, 5.0, 1.0]], [[2.0, 4.0, 5.0]]]])
      arg[1:3,:2,1:2]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 6.0], [5.0, 5.0, 1.0], [5.0, 7.0, 3.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 5.0], [4.0, 3.0, 
1.0]], [[2.0, 6.0, 6.0], [1.0, 6.0, 2.0], [1.0, 7.0, 2.0]]], [[[4.0, 3.0, 1.0], [1.0, 8.0, 1.0], [7.0, 8.0, 6.0]], [[3.0, 5.0, 
2.0], [5.0, 4.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 2.0, 3.0], [4.0, 5.0, 2.0]]], [[[3.0, 7.0, 6.0], [5.0, 5.0, 
1.0], [5.0, 5.0, 4.0]], [[8.0, 2.0, 8.0], [2.0, 4.0, 5.0], [5.0, 2.0, 2.0]], [[4.0, 3.0, 2.0], [2.0, 1.0, 2.0], [2.0, 5.0, 
7.0]]], [[[5.0, 6.0, 8.0], [3.0, 3.0, 3.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [1.0, 2.0, 8.0], [7.0, 8.0, 2.0]], [[2.0, 4.0, 
7.0], [5.0, 5.0, 1.0], [8.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RURN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 1.0], [4.0, 2.0, 6.0], [2.0, 7.0, 6.0]], [[4.0, 8.0, 4.0], [7.0, 2.0, 2.0], [8.0, 8.0, 
5.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 3.0], [2.0, 6.0, 1.0]]], [[[4.0, 3.0, 4.0], [5.0, 3.0, 6.0], [7.0, 2.0, 4.0]], [[2.0, 2.0, 
6.0], [6.0, 1.0, 3.0], [1.0, 7.0, 2.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 3.0], [7.0, 7.0, 6.0]]], [[[7.0, 6.0, 3.0], [8.0, 5.0, 
4.0], [4.0, 2.0, 1.0]], [[4.0, 7.0, 1.0], [8.0, 4.0, 5.0], [4.0, 6.0, 2.0]], [[8.0, 3.0, 2.0], [7.0, 1.0, 3.0], [6.0, 2.0, 
5.0]]], [[[4.0, 3.0, 3.0], [7.0, 4.0, 1.0], [7.0, 3.0, 7.0]], [[3.0, 7.0, 7.0], [4.0, 1.0, 7.0], [5.0, 7.0, 6.0]], [[7.0, 6.0, 
4.0], [3.0, 8.0, 6.0], [1.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 1.0, 8.0]], [[4.0, 5.0, 7.0]]], [[[2.0, 8.0, 5.0]], [[6.0, 4.0, 4.0]]]]),self.functionspace)
      arg[1:3,:2,1:2]=val
      ref=Data(numpy.array([[[[4.0, 7.0, 1.0], [4.0, 2.0, 6.0], [2.0, 7.0, 6.0]], [[4.0, 8.0, 4.0], [7.0, 2.0, 2.0], [8.0, 8.0, 
5.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 3.0], [2.0, 6.0, 1.0]]], [[[4.0, 3.0, 4.0], [2.0, 1.0, 8.0], [7.0, 2.0, 4.0]], [[2.0, 2.0, 
6.0], [4.0, 5.0, 7.0], [1.0, 7.0, 2.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 3.0], [7.0, 7.0, 6.0]]], [[[7.0, 6.0, 3.0], [2.0, 8.0, 
5.0], [4.0, 2.0, 1.0]], [[4.0, 7.0, 1.0], [6.0, 4.0, 4.0], [4.0, 6.0, 2.0]], [[8.0, 3.0, 2.0], [7.0, 1.0, 3.0], [6.0, 2.0, 
5.0]]], [[[4.0, 3.0, 3.0], [7.0, 4.0, 1.0], [7.0, 3.0, 7.0]], [[3.0, 7.0, 7.0], [4.0, 1.0, 7.0], [5.0, 7.0, 6.0]], [[7.0, 6.0, 
4.0], [3.0, 8.0, 6.0], [1.0, 1.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RURN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [5.0, 5.0, 2.0], [4.0, 6.0, 8.0]], [[4.0, 6.0, 3.0], [2.0, 7.0, 3.0], [1.0, 2.0, 
1.0]], [[5.0, 6.0, 2.0], [7.0, 3.0, 4.0], [3.0, 2.0, 1.0]]], [[[4.0, 8.0, 6.0], [8.0, 3.0, 1.0], [2.0, 1.0, 7.0]], [[7.0, 7.0, 
3.0], [5.0, 3.0, 1.0], [1.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [8.0, 8.0, 3.0], [3.0, 2.0, 5.0]]], [[[8.0, 5.0, 5.0], [6.0, 4.0, 
5.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 8.0], [1.0, 2.0, 1.0], [5.0, 8.0, 6.0]], [[5.0, 5.0, 8.0], [2.0, 3.0, 7.0], [2.0, 3.0, 
2.0]]], [[[6.0, 5.0, 6.0], [2.0, 3.0, 7.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], [4.0, 7.0, 5.0], [4.0, 2.0, 2.0]], [[2.0, 5.0, 
8.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 2.0, 5.0]], [[2.0, 1.0, 2.0]]], [[[6.0, 1.0, 4.0]], [[5.0, 5.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 2.0, 1.0]], [[3.0, 1.0, 5.0]]], [[[6.0, 1.0, 7.0]], [[5.0, 5.0, 8.0]]]])
      arg[1:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [5.0, 5.0, 2.0], [4.0, 6.0, 8.0]], [[4.0, 6.0, 3.0], [2.0, 7.0, 3.0], [1.0, 
2.0, 1.0]], [[5.0, 6.0, 2.0], [7.0, 3.0, 4.0], [3.0, 2.0, 1.0]]], [[[4.0, 8.0, 6.0], [8.0, 2.0, 5.0], [2.0, 1.0, 7.0]], [[7.0, 
7.0, 3.0], [2.0, 1.0, 2.0], [1.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [8.0, 8.0, 3.0], [3.0, 2.0, 5.0]]], [[[8.0, 5.0, 5.0], [6.0, 
1.0, 4.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 8.0], [5.0, 5.0, 8.0], [5.0, 8.0, 6.0]], [[5.0, 5.0, 8.0], [2.0, 3.0, 7.0], [2.0, 3.0, 
2.0]]], [[[6.0, 5.0, 6.0], [2.0, 3.0, 7.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], [4.0, 7.0, 5.0], [4.0, 2.0, 2.0]], [[2.0, 5.0, 
8.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 8.0], [5.0, 5.0, 2.0], [4.0, 6.0, 8.0]], 
[[4.0, 6.0, 3.0], [2.0, 7.0, 3.0], [1.0, 2.0, 1.0]], [[5.0, 6.0, 2.0], [7.0, 3.0, 4.0], [3.0, 2.0, 1.0]]], [[[4.0, 8.0, 6.0], 
[1.0, 2.0, 1.0], [2.0, 1.0, 7.0]], [[7.0, 7.0, 3.0], [3.0, 1.0, 5.0], [1.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [8.0, 8.0, 3.0], 
[3.0, 2.0, 5.0]]], [[[8.0, 5.0, 5.0], [6.0, 1.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 8.0], [5.0, 5.0, 8.0], [5.0, 8.0, 6.0]], 
[[5.0, 5.0, 8.0], [2.0, 3.0, 7.0], [2.0, 3.0, 2.0]]], [[[6.0, 5.0, 6.0], [2.0, 3.0, 7.0], [4.0, 2.0, 4.0]], [[7.0, 8.0, 5.0], 
[4.0, 7.0, 5.0], [4.0, 2.0, 2.0]], [[2.0, 5.0, 8.0], [2.0, 7.0, 3.0], [4.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CUPC(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 2.0], [2.0, 4.0, 5.0], [3.0, 7.0, 6.0]], [[2.0, 6.0, 5.0], [1.0, 6.0, 6.0], [7.0, 4.0, 
4.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 2.0], [8.0, 3.0, 4.0]]], [[[4.0, 1.0, 8.0], [3.0, 5.0, 6.0], [7.0, 8.0, 7.0]], [[3.0, 4.0, 
4.0], [1.0, 6.0, 1.0], [2.0, 5.0, 4.0]], [[5.0, 3.0, 7.0], [2.0, 2.0, 3.0], [1.0, 2.0, 6.0]]], [[[1.0, 2.0, 1.0], [3.0, 6.0, 
7.0], [1.0, 5.0, 6.0]], [[5.0, 8.0, 5.0], [8.0, 1.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 2.0, 8.0], [2.0, 2.0, 4.0], [1.0, 2.0, 
5.0]]], [[[4.0, 6.0, 1.0], [8.0, 3.0, 5.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 1.0], [6.0, 2.0, 3.0], [7.0, 1.0, 7.0]], [[4.0, 1.0, 
8.0], [7.0, 3.0, 6.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 7.0, 1.0], [6.0, 5.0, 3.0]], [[3.0, 5.0, 8.0], [6.0, 2.0, 8.0]], [[7.0, 5.0, 3.0], [3.0, 6.0, 
4.0]], [[8.0, 8.0, 7.0], [7.0, 3.0, 7.0]]])
      arg[:,:2,1,:]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 2.0], [5.0, 7.0, 1.0], [3.0, 7.0, 6.0]], [[2.0, 6.0, 5.0], [6.0, 5.0, 3.0], [7.0, 4.0, 
4.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 2.0], [8.0, 3.0, 4.0]]], [[[4.0, 1.0, 8.0], [3.0, 5.0, 8.0], [7.0, 8.0, 7.0]], [[3.0, 4.0, 
4.0], [6.0, 2.0, 8.0], [2.0, 5.0, 4.0]], [[5.0, 3.0, 7.0], [2.0, 2.0, 3.0], [1.0, 2.0, 6.0]]], [[[1.0, 2.0, 1.0], [7.0, 5.0, 
3.0], [1.0, 5.0, 6.0]], [[5.0, 8.0, 5.0], [3.0, 6.0, 4.0], [8.0, 2.0, 1.0]], [[7.0, 2.0, 8.0], [2.0, 2.0, 4.0], [1.0, 2.0, 
5.0]]], [[[4.0, 6.0, 1.0], [8.0, 8.0, 7.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 1.0], [7.0, 3.0, 7.0], [7.0, 1.0, 7.0]], [[4.0, 1.0, 
8.0], [7.0, 3.0, 6.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CUPC(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 6.0], [7.0, 2.0, 2.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 2.0], [3.0, 8.0, 3.0], [4.0, 6.0, 
8.0]], [[5.0, 2.0, 8.0], [4.0, 5.0, 6.0], [1.0, 3.0, 4.0]]], [[[1.0, 6.0, 3.0], [1.0, 8.0, 2.0], [6.0, 5.0, 8.0]], [[3.0, 6.0, 
4.0], [7.0, 5.0, 6.0], [6.0, 5.0, 7.0]], [[5.0, 4.0, 1.0], [2.0, 8.0, 4.0], [7.0, 5.0, 3.0]]], [[[2.0, 5.0, 8.0], [8.0, 3.0, 
5.0], [3.0, 2.0, 5.0]], [[8.0, 1.0, 7.0], [6.0, 3.0, 2.0], [2.0, 4.0, 5.0]], [[5.0, 6.0, 1.0], [8.0, 7.0, 7.0], [2.0, 7.0, 
1.0]]], [[[4.0, 7.0, 5.0], [3.0, 4.0, 2.0], [2.0, 5.0, 5.0]], [[4.0, 8.0, 6.0], [7.0, 4.0, 6.0], [4.0, 5.0, 8.0]], [[8.0, 1.0, 
6.0], [5.0, 2.0, 4.0], [6.0, 8.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 5.0, 1.0], [4.0, 3.0, 6.0]], [[3.0, 4.0, 1.0], [8.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 
5.0, 8.0]], [[8.0, 4.0, 8.0], [3.0, 2.0, 1.0]]]),self.functionspace)
      arg[:,:2,1,:]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 6.0], [1.0, 5.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 2.0], [4.0, 3.0, 6.0], [4.0, 6.0, 
8.0]], [[5.0, 2.0, 8.0], [4.0, 5.0, 6.0], [1.0, 3.0, 4.0]]], [[[1.0, 6.0, 3.0], [3.0, 4.0, 1.0], [6.0, 5.0, 8.0]], [[3.0, 6.0, 
4.0], [8.0, 4.0, 2.0], [6.0, 5.0, 7.0]], [[5.0, 4.0, 1.0], [2.0, 8.0, 4.0], [7.0, 5.0, 3.0]]], [[[2.0, 5.0, 8.0], [2.0, 6.0, 
2.0], [3.0, 2.0, 5.0]], [[8.0, 1.0, 7.0], [3.0, 5.0, 8.0], [2.0, 4.0, 5.0]], [[5.0, 6.0, 1.0], [8.0, 7.0, 7.0], [2.0, 7.0, 
1.0]]], [[[4.0, 7.0, 5.0], [8.0, 4.0, 8.0], [2.0, 5.0, 5.0]], [[4.0, 8.0, 6.0], [3.0, 2.0, 1.0], [4.0, 5.0, 8.0]], [[8.0, 1.0, 
6.0], [5.0, 2.0, 4.0], [6.0, 8.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CUPC(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 5.0], [2.0, 2.0, 6.0], [8.0, 7.0, 8.0]], [[7.0, 6.0, 8.0], [3.0, 5.0, 2.0], [6.0, 7.0, 
3.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[8.0, 2.0, 2.0], [6.0, 1.0, 2.0], [3.0, 4.0, 1.0]], [[7.0, 6.0, 
5.0], [8.0, 2.0, 8.0], [4.0, 3.0, 2.0]], [[6.0, 3.0, 2.0], [6.0, 4.0, 4.0], [1.0, 5.0, 1.0]]], [[[6.0, 7.0, 1.0], [7.0, 4.0, 
1.0], [6.0, 4.0, 8.0]], [[1.0, 8.0, 4.0], [5.0, 8.0, 5.0], [4.0, 1.0, 2.0]], [[7.0, 3.0, 2.0], [1.0, 5.0, 7.0], [5.0, 6.0, 
1.0]]], [[[5.0, 8.0, 6.0], [4.0, 4.0, 7.0], [1.0, 5.0, 6.0]], [[8.0, 5.0, 7.0], [4.0, 3.0, 7.0], [5.0, 2.0, 6.0]], [[2.0, 8.0, 
5.0], [4.0, 7.0, 6.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 6.0, 3.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 5.0], [3.0, 2.0, 3.0]], [[3.0, 4.0, 4.0], [7.0, 
4.0, 4.0]], [[3.0, 5.0, 8.0], [8.0, 5.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 2.0, 2.0], [4.0, 3.0, 2.0]], [[4.0, 5.0, 
3.0], [1.0, 1.0, 8.0]], [[2.0, 2.0, 5.0], [8.0, 2.0, 2.0]], [[3.0, 1.0, 5.0], [1.0, 4.0, 3.0]]])
      arg[:,:2,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 5.0], [7.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[7.0, 6.0, 8.0], [3.0, 3.0, 7.0], [6.0, 
7.0, 3.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[8.0, 2.0, 2.0], [5.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[7.0, 
6.0, 5.0], [3.0, 2.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 3.0, 2.0], [6.0, 4.0, 4.0], [1.0, 5.0, 1.0]]], [[[6.0, 7.0, 1.0], [3.0, 
4.0, 4.0], [6.0, 4.0, 8.0]], [[1.0, 8.0, 4.0], [7.0, 4.0, 4.0], [4.0, 1.0, 2.0]], [[7.0, 3.0, 2.0], [1.0, 5.0, 7.0], [5.0, 6.0, 
1.0]]], [[[5.0, 8.0, 6.0], [3.0, 5.0, 8.0], [1.0, 5.0, 6.0]], [[8.0, 5.0, 7.0], [8.0, 5.0, 3.0], [5.0, 2.0, 6.0]], [[2.0, 8.0, 
5.0], [4.0, 7.0, 6.0], [2.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [1.0, 2.0, 2.0], [8.0, 7.0, 8.0]], 
[[7.0, 6.0, 8.0], [4.0, 3.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 4.0], [5.0, 1.0, 5.0]]], [[[8.0, 2.0, 2.0], 
[4.0, 5.0, 3.0], [3.0, 4.0, 1.0]], [[7.0, 6.0, 5.0], [1.0, 1.0, 8.0], [4.0, 3.0, 2.0]], [[6.0, 3.0, 2.0], [6.0, 4.0, 4.0], 
[1.0, 5.0, 1.0]]], [[[6.0, 7.0, 1.0], [2.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[1.0, 8.0, 4.0], [8.0, 2.0, 2.0], [4.0, 1.0, 2.0]], 
[[7.0, 3.0, 2.0], [1.0, 5.0, 7.0], [5.0, 6.0, 1.0]]], [[[5.0, 8.0, 6.0], [3.0, 1.0, 5.0], [1.0, 5.0, 6.0]], [[8.0, 5.0, 7.0], 
[1.0, 4.0, 3.0], [5.0, 2.0, 6.0]], [[2.0, 8.0, 5.0], [4.0, 7.0, 6.0], [2.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRLR(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 8.0], [5.0, 3.0, 6.0], [1.0, 4.0, 8.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 1.0], [1.0, 2.0, 
5.0]], [[4.0, 3.0, 8.0], [5.0, 7.0, 5.0], [6.0, 7.0, 5.0]]], [[[3.0, 5.0, 7.0], [5.0, 2.0, 3.0], [2.0, 8.0, 8.0]], [[2.0, 5.0, 
8.0], [2.0, 2.0, 7.0], [4.0, 8.0, 6.0]], [[1.0, 1.0, 1.0], [5.0, 6.0, 3.0], [4.0, 4.0, 5.0]]], [[[2.0, 5.0, 5.0], [3.0, 2.0, 
4.0], [6.0, 5.0, 8.0]], [[5.0, 7.0, 2.0], [4.0, 8.0, 8.0], [2.0, 7.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 5.0, 7.0], [8.0, 1.0, 
5.0]]], [[[4.0, 1.0, 4.0], [1.0, 4.0, 8.0], [6.0, 7.0, 5.0]], [[3.0, 5.0, 1.0], [2.0, 4.0, 1.0], [2.0, 1.0, 6.0]], [[1.0, 6.0, 
1.0], [8.0, 4.0, 2.0], [7.0, 8.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0]]], [[[7.0]]]])
      arg[1:3,1:2,2:,1:2]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 8.0], [5.0, 3.0, 6.0], [1.0, 4.0, 8.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 1.0], [1.0, 2.0, 
5.0]], [[4.0, 3.0, 8.0], [5.0, 7.0, 5.0], [6.0, 7.0, 5.0]]], [[[3.0, 5.0, 7.0], [5.0, 2.0, 3.0], [2.0, 8.0, 8.0]], [[2.0, 5.0, 
8.0], [2.0, 2.0, 7.0], [4.0, 2.0, 6.0]], [[1.0, 1.0, 1.0], [5.0, 6.0, 3.0], [4.0, 4.0, 5.0]]], [[[2.0, 5.0, 5.0], [3.0, 2.0, 
4.0], [6.0, 5.0, 8.0]], [[5.0, 7.0, 2.0], [4.0, 8.0, 8.0], [2.0, 7.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 5.0, 7.0], [8.0, 1.0, 
5.0]]], [[[4.0, 1.0, 4.0], [1.0, 4.0, 8.0], [6.0, 7.0, 5.0]], [[3.0, 5.0, 1.0], [2.0, 4.0, 1.0], [2.0, 1.0, 6.0]], [[1.0, 6.0, 
1.0], [8.0, 4.0, 2.0], [7.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRLR(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 7.0], [4.0, 4.0, 4.0], [5.0, 1.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 6.0, 8.0], [1.0, 3.0, 
6.0]], [[7.0, 3.0, 1.0], [3.0, 4.0, 1.0], [1.0, 4.0, 7.0]]], [[[2.0, 6.0, 7.0], [7.0, 7.0, 8.0], [5.0, 8.0, 8.0]], [[1.0, 3.0, 
3.0], [4.0, 4.0, 5.0], [5.0, 1.0, 1.0]], [[1.0, 6.0, 8.0], [6.0, 1.0, 3.0], [6.0, 4.0, 1.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 
7.0], [4.0, 5.0, 2.0]], [[6.0, 4.0, 2.0], [1.0, 7.0, 6.0], [1.0, 4.0, 3.0]], [[3.0, 2.0, 8.0], [5.0, 1.0, 5.0], [7.0, 6.0, 
1.0]]], [[[2.0, 6.0, 4.0], [6.0, 2.0, 7.0], [6.0, 5.0, 4.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 3.0], [8.0, 6.0, 8.0]], [[3.0, 3.0, 
3.0], [1.0, 2.0, 1.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0]]], [[[8.0]]]]),self.functionspace)
      arg[1:3,1:2,2:,1:2]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 7.0], [4.0, 4.0, 4.0], [5.0, 1.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 6.0, 8.0], [1.0, 3.0, 
6.0]], [[7.0, 3.0, 1.0], [3.0, 4.0, 1.0], [1.0, 4.0, 7.0]]], [[[2.0, 6.0, 7.0], [7.0, 7.0, 8.0], [5.0, 8.0, 8.0]], [[1.0, 3.0, 
3.0], [4.0, 4.0, 5.0], [5.0, 6.0, 1.0]], [[1.0, 6.0, 8.0], [6.0, 1.0, 3.0], [6.0, 4.0, 1.0]]], [[[1.0, 3.0, 3.0], [8.0, 7.0, 
7.0], [4.0, 5.0, 2.0]], [[6.0, 4.0, 2.0], [1.0, 7.0, 6.0], [1.0, 8.0, 3.0]], [[3.0, 2.0, 8.0], [5.0, 1.0, 5.0], [7.0, 6.0, 
1.0]]], [[[2.0, 6.0, 4.0], [6.0, 2.0, 7.0], [6.0, 5.0, 4.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 3.0], [8.0, 6.0, 8.0]], [[3.0, 3.0, 
3.0], [1.0, 2.0, 1.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRLR(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 7.0], [7.0, 2.0, 5.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 4.0], [3.0, 5.0, 8.0], [6.0, 7.0, 
7.0]], [[7.0, 4.0, 1.0], [3.0, 3.0, 5.0], [7.0, 1.0, 4.0]]], [[[4.0, 8.0, 5.0], [5.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[1.0, 1.0, 
7.0], [6.0, 7.0, 2.0], [5.0, 3.0, 1.0]], [[1.0, 6.0, 7.0], [8.0, 5.0, 5.0], [7.0, 6.0, 4.0]]], [[[5.0, 6.0, 1.0], [5.0, 8.0, 
3.0], [5.0, 7.0, 5.0]], [[3.0, 8.0, 7.0], [1.0, 1.0, 2.0], [2.0, 3.0, 5.0]], [[7.0, 7.0, 7.0], [6.0, 1.0, 6.0], [5.0, 3.0, 
6.0]]], [[[2.0, 2.0, 2.0], [8.0, 1.0, 5.0], [7.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], [3.0, 7.0, 5.0], [7.0, 1.0, 5.0]], [[6.0, 6.0, 
7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0]]], [[[6.0]]]])+(1.-msk_val)*numpy.array([[[[8.0]]], [[[1.0]]]])
      arg[1:3,1:2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 7.0], [7.0, 2.0, 5.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 4.0], [3.0, 5.0, 8.0], [6.0, 
7.0, 7.0]], [[7.0, 4.0, 1.0], [3.0, 3.0, 5.0], [7.0, 1.0, 4.0]]], [[[4.0, 8.0, 5.0], [5.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[1.0, 
1.0, 7.0], [6.0, 7.0, 2.0], [5.0, 4.0, 1.0]], [[1.0, 6.0, 7.0], [8.0, 5.0, 5.0], [7.0, 6.0, 4.0]]], [[[5.0, 6.0, 1.0], [5.0, 
8.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 8.0, 7.0], [1.0, 1.0, 2.0], [2.0, 6.0, 5.0]], [[7.0, 7.0, 7.0], [6.0, 1.0, 6.0], [5.0, 3.0, 
6.0]]], [[[2.0, 2.0, 2.0], [8.0, 1.0, 5.0], [7.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], [3.0, 7.0, 5.0], [7.0, 1.0, 5.0]], [[6.0, 6.0, 
7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [7.0, 2.0, 5.0], [8.0, 1.0, 3.0]], 
[[1.0, 1.0, 4.0], [3.0, 5.0, 8.0], [6.0, 7.0, 7.0]], [[7.0, 4.0, 1.0], [3.0, 3.0, 5.0], [7.0, 1.0, 4.0]]], [[[4.0, 8.0, 5.0], 
[5.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[1.0, 1.0, 7.0], [6.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[1.0, 6.0, 7.0], [8.0, 5.0, 5.0], 
[7.0, 6.0, 4.0]]], [[[5.0, 6.0, 1.0], [5.0, 8.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 8.0, 7.0], [1.0, 1.0, 2.0], [2.0, 1.0, 5.0]], 
[[7.0, 7.0, 7.0], [6.0, 1.0, 6.0], [5.0, 3.0, 6.0]]], [[[2.0, 2.0, 2.0], [8.0, 1.0, 5.0], [7.0, 6.0, 6.0]], [[5.0, 5.0, 7.0], 
[3.0, 7.0, 5.0], [7.0, 1.0, 5.0]], [[6.0, 6.0, 7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRRL(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 6.0], [7.0, 6.0, 4.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 5.0], [5.0, 3.0, 7.0], [4.0, 1.0, 
3.0]], [[2.0, 1.0, 6.0], [1.0, 7.0, 7.0], [4.0, 7.0, 5.0]]], [[[8.0, 5.0, 5.0], [7.0, 7.0, 1.0], [2.0, 5.0, 5.0]], [[4.0, 8.0, 
1.0], [2.0, 6.0, 6.0], [7.0, 7.0, 2.0]], [[8.0, 4.0, 6.0], [3.0, 8.0, 7.0], [6.0, 7.0, 5.0]]], [[[7.0, 2.0, 7.0], [7.0, 4.0, 
6.0], [1.0, 4.0, 8.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 6.0], [2.0, 5.0, 1.0]], [[7.0, 3.0, 7.0], [3.0, 4.0, 8.0], [6.0, 7.0, 
8.0]]], [[[6.0, 2.0, 8.0], [2.0, 1.0, 2.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 7.0], [2.0, 7.0, 3.0], [2.0, 1.0, 5.0]], [[6.0, 4.0, 
6.0], [1.0, 1.0, 2.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0]]], [[[3.0]]]])
      arg[2:,1:2,1:2,2:]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 6.0], [7.0, 6.0, 4.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 5.0], [5.0, 3.0, 7.0], [4.0, 1.0, 
3.0]], [[2.0, 1.0, 6.0], [1.0, 7.0, 7.0], [4.0, 7.0, 5.0]]], [[[8.0, 5.0, 5.0], [7.0, 7.0, 1.0], [2.0, 5.0, 5.0]], [[4.0, 8.0, 
1.0], [2.0, 6.0, 6.0], [7.0, 7.0, 2.0]], [[8.0, 4.0, 6.0], [3.0, 8.0, 7.0], [6.0, 7.0, 5.0]]], [[[7.0, 2.0, 7.0], [7.0, 4.0, 
6.0], [1.0, 4.0, 8.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 8.0], [2.0, 5.0, 1.0]], [[7.0, 3.0, 7.0], [3.0, 4.0, 8.0], [6.0, 7.0, 
8.0]]], [[[6.0, 2.0, 8.0], [2.0, 1.0, 2.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 7.0], [2.0, 7.0, 3.0], [2.0, 1.0, 5.0]], [[6.0, 4.0, 
6.0], [1.0, 1.0, 2.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRRL(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 6.0], [2.0, 2.0, 4.0], [5.0, 2.0, 3.0]], [[2.0, 8.0, 7.0], [6.0, 6.0, 3.0], [5.0, 3.0, 
6.0]], [[7.0, 7.0, 3.0], [2.0, 8.0, 7.0], [8.0, 1.0, 3.0]]], [[[1.0, 2.0, 4.0], [4.0, 7.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 2.0, 
7.0], [7.0, 7.0, 3.0], [4.0, 7.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 3.0, 7.0], [2.0, 3.0, 8.0]]], [[[8.0, 2.0, 4.0], [1.0, 2.0, 
2.0], [4.0, 7.0, 4.0]], [[1.0, 3.0, 7.0], [8.0, 8.0, 2.0], [8.0, 5.0, 2.0]], [[2.0, 5.0, 4.0], [2.0, 1.0, 3.0], [8.0, 6.0, 
4.0]]], [[[3.0, 7.0, 2.0], [8.0, 5.0, 5.0], [4.0, 1.0, 5.0]], [[2.0, 6.0, 8.0], [3.0, 1.0, 1.0], [5.0, 5.0, 6.0]], [[2.0, 3.0, 
8.0], [8.0, 1.0, 4.0], [8.0, 2.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0]]], [[[2.0]]]]),self.functionspace)
      arg[2:,1:2,1:2,2:]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 6.0], [2.0, 2.0, 4.0], [5.0, 2.0, 3.0]], [[2.0, 8.0, 7.0], [6.0, 6.0, 3.0], [5.0, 3.0, 
6.0]], [[7.0, 7.0, 3.0], [2.0, 8.0, 7.0], [8.0, 1.0, 3.0]]], [[[1.0, 2.0, 4.0], [4.0, 7.0, 7.0], [1.0, 8.0, 2.0]], [[3.0, 2.0, 
7.0], [7.0, 7.0, 3.0], [4.0, 7.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 3.0, 7.0], [2.0, 3.0, 8.0]]], [[[8.0, 2.0, 4.0], [1.0, 2.0, 
2.0], [4.0, 7.0, 4.0]], [[1.0, 3.0, 7.0], [8.0, 8.0, 1.0], [8.0, 5.0, 2.0]], [[2.0, 5.0, 4.0], [2.0, 1.0, 3.0], [8.0, 6.0, 
4.0]]], [[[3.0, 7.0, 2.0], [8.0, 5.0, 5.0], [4.0, 1.0, 5.0]], [[2.0, 6.0, 8.0], [3.0, 1.0, 2.0], [5.0, 5.0, 6.0]], [[2.0, 3.0, 
8.0], [8.0, 1.0, 4.0], [8.0, 2.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRRL(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 3.0], [7.0, 7.0, 1.0], [8.0, 7.0, 1.0]], [[4.0, 6.0, 7.0], [4.0, 1.0, 8.0], [8.0, 6.0, 
7.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[4.0, 8.0, 4.0], [3.0, 7.0, 6.0], [8.0, 7.0, 4.0]], [[3.0, 3.0, 
2.0], [1.0, 8.0, 1.0], [8.0, 1.0, 2.0]], [[7.0, 8.0, 5.0], [5.0, 6.0, 5.0], [6.0, 2.0, 1.0]]], [[[8.0, 8.0, 5.0], [4.0, 8.0, 
8.0], [8.0, 2.0, 7.0]], [[7.0, 3.0, 8.0], [7.0, 8.0, 6.0], [8.0, 3.0, 5.0]], [[5.0, 7.0, 3.0], [2.0, 4.0, 7.0], [7.0, 5.0, 
7.0]]], [[[8.0, 8.0, 5.0], [5.0, 8.0, 3.0], [3.0, 5.0, 7.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 6.0], [5.0, 1.0, 7.0]], [[5.0, 7.0, 
5.0], [7.0, 8.0, 2.0], [1.0, 6.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0]]], [[[3.0]]]])+(1.-msk_val)*numpy.array([[[[6.0]]], [[[1.0]]]])
      arg[2:,1:2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 3.0], [7.0, 7.0, 1.0], [8.0, 7.0, 1.0]], [[4.0, 6.0, 7.0], [4.0, 1.0, 8.0], [8.0, 
6.0, 7.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[4.0, 8.0, 4.0], [3.0, 7.0, 6.0], [8.0, 7.0, 4.0]], [[3.0, 
3.0, 2.0], [1.0, 8.0, 1.0], [8.0, 1.0, 2.0]], [[7.0, 8.0, 5.0], [5.0, 6.0, 5.0], [6.0, 2.0, 1.0]]], [[[8.0, 8.0, 5.0], [4.0, 
8.0, 8.0], [8.0, 2.0, 7.0]], [[7.0, 3.0, 8.0], [7.0, 8.0, 7.0], [8.0, 3.0, 5.0]], [[5.0, 7.0, 3.0], [2.0, 4.0, 7.0], [7.0, 5.0, 
7.0]]], [[[8.0, 8.0, 5.0], [5.0, 8.0, 3.0], [3.0, 5.0, 7.0]], [[7.0, 3.0, 2.0], [6.0, 3.0, 3.0], [5.0, 1.0, 7.0]], [[5.0, 7.0, 
5.0], [7.0, 8.0, 2.0], [1.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 3.0], [7.0, 7.0, 1.0], [8.0, 7.0, 1.0]], 
[[4.0, 6.0, 7.0], [4.0, 1.0, 8.0], [8.0, 6.0, 7.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[4.0, 8.0, 4.0], 
[3.0, 7.0, 6.0], [8.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], [1.0, 8.0, 1.0], [8.0, 1.0, 2.0]], [[7.0, 8.0, 5.0], [5.0, 6.0, 5.0], 
[6.0, 2.0, 1.0]]], [[[8.0, 8.0, 5.0], [4.0, 8.0, 8.0], [8.0, 2.0, 7.0]], [[7.0, 3.0, 8.0], [7.0, 8.0, 6.0], [8.0, 3.0, 5.0]], 
[[5.0, 7.0, 3.0], [2.0, 4.0, 7.0], [7.0, 5.0, 7.0]]], [[[8.0, 8.0, 5.0], [5.0, 8.0, 3.0], [3.0, 5.0, 7.0]], [[7.0, 3.0, 2.0], 
[6.0, 3.0, 1.0], [5.0, 1.0, 7.0]], [[5.0, 7.0, 5.0], [7.0, 8.0, 2.0], [1.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCRC(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 8.0], [5.0, 3.0, 3.0], [3.0, 5.0, 1.0]], [[2.0, 7.0, 4.0], [8.0, 4.0, 8.0], [5.0, 5.0, 
1.0]], [[3.0, 4.0, 7.0], [8.0, 8.0, 5.0], [8.0, 7.0, 3.0]]], [[[2.0, 2.0, 6.0], [6.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 3.0, 
3.0], [7.0, 4.0, 4.0], [4.0, 8.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 6.0], [6.0, 8.0, 7.0]]], [[[4.0, 1.0, 4.0], [5.0, 1.0, 
5.0], [2.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [7.0, 5.0, 8.0], [5.0, 6.0, 2.0]], [[5.0, 2.0, 2.0], [4.0, 6.0, 1.0], [1.0, 5.0, 
3.0]]], [[[8.0, 1.0, 3.0], [4.0, 3.0, 1.0], [8.0, 4.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 2.0, 1.0], [1.0, 1.0, 6.0]], [[1.0, 4.0, 
2.0], [5.0, 2.0, 5.0], [8.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 4.0, 3.0]], [[2.0, 3.0, 3.0]], [[7.0, 8.0, 4.0]]])
      arg[2,:,1:2,:]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 8.0], [5.0, 3.0, 3.0], [3.0, 5.0, 1.0]], [[2.0, 7.0, 4.0], [8.0, 4.0, 8.0], [5.0, 5.0, 
1.0]], [[3.0, 4.0, 7.0], [8.0, 8.0, 5.0], [8.0, 7.0, 3.0]]], [[[2.0, 2.0, 6.0], [6.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[2.0, 3.0, 
3.0], [7.0, 4.0, 4.0], [4.0, 8.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 8.0, 6.0], [6.0, 8.0, 7.0]]], [[[4.0, 1.0, 4.0], [3.0, 4.0, 
3.0], [2.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [2.0, 3.0, 3.0], [5.0, 6.0, 2.0]], [[5.0, 2.0, 2.0], [7.0, 8.0, 4.0], [1.0, 5.0, 
3.0]]], [[[8.0, 1.0, 3.0], [4.0, 3.0, 1.0], [8.0, 4.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 2.0, 1.0], [1.0, 1.0, 6.0]], [[1.0, 4.0, 
2.0], [5.0, 2.0, 5.0], [8.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCRC(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 2.0], [4.0, 3.0, 2.0], [1.0, 2.0, 7.0]], [[7.0, 7.0, 7.0], [2.0, 1.0, 6.0], [2.0, 1.0, 
7.0]], [[1.0, 8.0, 6.0], [1.0, 8.0, 1.0], [5.0, 1.0, 8.0]]], [[[1.0, 7.0, 7.0], [3.0, 6.0, 6.0], [6.0, 4.0, 8.0]], [[6.0, 2.0, 
4.0], [7.0, 2.0, 6.0], [3.0, 2.0, 7.0]], [[5.0, 7.0, 3.0], [1.0, 4.0, 4.0], [4.0, 1.0, 8.0]]], [[[2.0, 5.0, 2.0], [7.0, 2.0, 
8.0], [8.0, 8.0, 7.0]], [[8.0, 8.0, 1.0], [5.0, 5.0, 3.0], [6.0, 5.0, 3.0]], [[4.0, 8.0, 4.0], [6.0, 5.0, 3.0], [5.0, 1.0, 
3.0]]], [[[8.0, 5.0, 2.0], [7.0, 2.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 1.0], [8.0, 7.0, 1.0]], [[8.0, 6.0, 
7.0], [4.0, 3.0, 6.0], [6.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 7.0, 4.0]], [[7.0, 6.0, 2.0]], [[1.0, 7.0, 1.0]]]),self.functionspace)
      arg[2,:,1:2,:]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 2.0], [4.0, 3.0, 2.0], [1.0, 2.0, 7.0]], [[7.0, 7.0, 7.0], [2.0, 1.0, 6.0], [2.0, 1.0, 
7.0]], [[1.0, 8.0, 6.0], [1.0, 8.0, 1.0], [5.0, 1.0, 8.0]]], [[[1.0, 7.0, 7.0], [3.0, 6.0, 6.0], [6.0, 4.0, 8.0]], [[6.0, 2.0, 
4.0], [7.0, 2.0, 6.0], [3.0, 2.0, 7.0]], [[5.0, 7.0, 3.0], [1.0, 4.0, 4.0], [4.0, 1.0, 8.0]]], [[[2.0, 5.0, 2.0], [5.0, 7.0, 
4.0], [8.0, 8.0, 7.0]], [[8.0, 8.0, 1.0], [7.0, 6.0, 2.0], [6.0, 5.0, 3.0]], [[4.0, 8.0, 4.0], [1.0, 7.0, 1.0], [5.0, 1.0, 
3.0]]], [[[8.0, 5.0, 2.0], [7.0, 2.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 1.0], [8.0, 7.0, 1.0]], [[8.0, 6.0, 
7.0], [4.0, 3.0, 6.0], [6.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCRC(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 7.0], [5.0, 5.0, 2.0], [4.0, 3.0, 4.0]], [[4.0, 4.0, 3.0], [5.0, 2.0, 5.0], [1.0, 3.0, 
2.0]], [[2.0, 5.0, 2.0], [4.0, 2.0, 3.0], [4.0, 2.0, 2.0]]], [[[7.0, 6.0, 8.0], [1.0, 1.0, 5.0], [5.0, 2.0, 2.0]], [[1.0, 3.0, 
1.0], [7.0, 2.0, 6.0], [3.0, 8.0, 1.0]], [[2.0, 4.0, 7.0], [2.0, 8.0, 6.0], [4.0, 7.0, 8.0]]], [[[3.0, 2.0, 2.0], [6.0, 7.0, 
8.0], [7.0, 5.0, 3.0]], [[8.0, 7.0, 6.0], [3.0, 1.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [1.0, 1.0, 5.0], [5.0, 1.0, 
3.0]]], [[[2.0, 2.0, 4.0], [7.0, 8.0, 6.0], [5.0, 4.0, 4.0]], [[1.0, 6.0, 1.0], [6.0, 4.0, 2.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 
7.0], [4.0, 8.0, 8.0], [4.0, 4.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 4.0, 3.0]], [[1.0, 2.0, 1.0]], [[8.0, 7.0, 7.0]]])+(1.-msk_val)*numpy.array([[[7.0, 6.0, 
2.0]], [[7.0, 1.0, 3.0]], [[7.0, 3.0, 3.0]]])
      arg[2,:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 7.0], [5.0, 5.0, 2.0], [4.0, 3.0, 4.0]], [[4.0, 4.0, 3.0], [5.0, 2.0, 5.0], [1.0, 
3.0, 2.0]], [[2.0, 5.0, 2.0], [4.0, 2.0, 3.0], [4.0, 2.0, 2.0]]], [[[7.0, 6.0, 8.0], [1.0, 1.0, 5.0], [5.0, 2.0, 2.0]], [[1.0, 
3.0, 1.0], [7.0, 2.0, 6.0], [3.0, 8.0, 1.0]], [[2.0, 4.0, 7.0], [2.0, 8.0, 6.0], [4.0, 7.0, 8.0]]], [[[3.0, 2.0, 2.0], [3.0, 
4.0, 3.0], [7.0, 5.0, 3.0]], [[8.0, 7.0, 6.0], [1.0, 2.0, 1.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [8.0, 7.0, 7.0], [5.0, 1.0, 
3.0]]], [[[2.0, 2.0, 4.0], [7.0, 8.0, 6.0], [5.0, 4.0, 4.0]], [[1.0, 6.0, 1.0], [6.0, 4.0, 2.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 
7.0], [4.0, 8.0, 8.0], [4.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 7.0], [5.0, 5.0, 2.0], [4.0, 3.0, 4.0]], 
[[4.0, 4.0, 3.0], [5.0, 2.0, 5.0], [1.0, 3.0, 2.0]], [[2.0, 5.0, 2.0], [4.0, 2.0, 3.0], [4.0, 2.0, 2.0]]], [[[7.0, 6.0, 8.0], 
[1.0, 1.0, 5.0], [5.0, 2.0, 2.0]], [[1.0, 3.0, 1.0], [7.0, 2.0, 6.0], [3.0, 8.0, 1.0]], [[2.0, 4.0, 7.0], [2.0, 8.0, 6.0], 
[4.0, 7.0, 8.0]]], [[[3.0, 2.0, 2.0], [7.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[8.0, 7.0, 6.0], [7.0, 1.0, 3.0], [7.0, 7.0, 1.0]], 
[[4.0, 8.0, 4.0], [7.0, 3.0, 3.0], [5.0, 1.0, 3.0]]], [[[2.0, 2.0, 4.0], [7.0, 8.0, 6.0], [5.0, 4.0, 4.0]], [[1.0, 6.0, 1.0], 
[6.0, 4.0, 2.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 7.0], [4.0, 8.0, 8.0], [4.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [6.0, 5.0, 6.0], [1.0, 7.0, 4.0]], [[3.0, 6.0, 6.0], [4.0, 7.0, 6.0], [1.0, 4.0, 
1.0]], [[8.0, 4.0, 5.0], [7.0, 1.0, 3.0], [7.0, 5.0, 1.0]]], [[[5.0, 3.0, 4.0], [6.0, 5.0, 2.0], [3.0, 7.0, 1.0]], [[5.0, 1.0, 
3.0], [4.0, 2.0, 7.0], [3.0, 4.0, 2.0]], [[3.0, 6.0, 8.0], [8.0, 4.0, 1.0], [3.0, 1.0, 4.0]]], [[[6.0, 4.0, 1.0], [2.0, 2.0, 
3.0], [5.0, 2.0, 1.0]], [[4.0, 1.0, 8.0], [1.0, 4.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 1.0], [5.0, 4.0, 7.0], [4.0, 2.0, 
6.0]]], [[[4.0, 2.0, 1.0], [8.0, 8.0, 8.0], [5.0, 8.0, 5.0]], [[5.0, 2.0, 4.0], [7.0, 6.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 3.0, 
7.0], [1.0, 3.0, 8.0], [7.0, 8.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[1.0, 8.0, 8.0], [4.0, 3.0, 5.0]]], [[[7.0, 7.0, 7.0], [7.0, 4.0, 
2.0]], [[1.0, 1.0, 1.0], [7.0, 3.0, 8.0]]]])
      arg[1:3,:2,:2]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 7.0], [6.0, 5.0, 6.0], [1.0, 7.0, 4.0]], [[3.0, 6.0, 6.0], [4.0, 7.0, 6.0], [1.0, 4.0, 
1.0]], [[8.0, 4.0, 5.0], [7.0, 1.0, 3.0], [7.0, 5.0, 1.0]]], [[[1.0, 7.0, 6.0], [5.0, 7.0, 4.0], [3.0, 7.0, 1.0]], [[1.0, 8.0, 
8.0], [4.0, 3.0, 5.0], [3.0, 4.0, 2.0]], [[3.0, 6.0, 8.0], [8.0, 4.0, 1.0], [3.0, 1.0, 4.0]]], [[[7.0, 7.0, 7.0], [7.0, 4.0, 
2.0], [5.0, 2.0, 1.0]], [[1.0, 1.0, 1.0], [7.0, 3.0, 8.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 1.0], [5.0, 4.0, 7.0], [4.0, 2.0, 
6.0]]], [[[4.0, 2.0, 1.0], [8.0, 8.0, 8.0], [5.0, 8.0, 5.0]], [[5.0, 2.0, 4.0], [7.0, 6.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 3.0, 
7.0], [1.0, 3.0, 8.0], [7.0, 8.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUUN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 2.0], [4.0, 8.0, 7.0], [7.0, 1.0, 7.0]], [[8.0, 1.0, 7.0], [6.0, 4.0, 7.0], [7.0, 2.0, 
8.0]], [[6.0, 6.0, 7.0], [5.0, 5.0, 5.0], [8.0, 1.0, 8.0]]], [[[5.0, 7.0, 7.0], [7.0, 6.0, 2.0], [6.0, 8.0, 4.0]], [[2.0, 6.0, 
7.0], [7.0, 8.0, 8.0], [7.0, 4.0, 1.0]], [[6.0, 7.0, 6.0], [3.0, 3.0, 2.0], [1.0, 1.0, 4.0]]], [[[4.0, 8.0, 4.0], [5.0, 8.0, 
1.0], [5.0, 7.0, 6.0]], [[1.0, 4.0, 3.0], [8.0, 1.0, 4.0], [6.0, 6.0, 7.0]], [[7.0, 2.0, 7.0], [4.0, 5.0, 1.0], [2.0, 8.0, 
7.0]]], [[[5.0, 4.0, 5.0], [7.0, 1.0, 1.0], [8.0, 5.0, 4.0]], [[2.0, 8.0, 1.0], [7.0, 1.0, 8.0], [7.0, 6.0, 4.0]], [[4.0, 2.0, 
6.0], [2.0, 1.0, 1.0], [2.0, 5.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 3.0, 6.0], [3.0, 3.0, 5.0]], [[2.0, 2.0, 2.0], [7.0, 7.0, 6.0]]], [[[2.0, 6.0, 3.0], [7.0, 
1.0, 2.0]], [[5.0, 4.0, 4.0], [7.0, 6.0, 5.0]]]]),self.functionspace)
      arg[1:3,:2,:2]=val
      ref=Data(numpy.array([[[[3.0, 8.0, 2.0], [4.0, 8.0, 7.0], [7.0, 1.0, 7.0]], [[8.0, 1.0, 7.0], [6.0, 4.0, 7.0], [7.0, 2.0, 
8.0]], [[6.0, 6.0, 7.0], [5.0, 5.0, 5.0], [8.0, 1.0, 8.0]]], [[[3.0, 3.0, 6.0], [3.0, 3.0, 5.0], [6.0, 8.0, 4.0]], [[2.0, 2.0, 
2.0], [7.0, 7.0, 6.0], [7.0, 4.0, 1.0]], [[6.0, 7.0, 6.0], [3.0, 3.0, 2.0], [1.0, 1.0, 4.0]]], [[[2.0, 6.0, 3.0], [7.0, 1.0, 
2.0], [5.0, 7.0, 6.0]], [[5.0, 4.0, 4.0], [7.0, 6.0, 5.0], [6.0, 6.0, 7.0]], [[7.0, 2.0, 7.0], [4.0, 5.0, 1.0], [2.0, 8.0, 
7.0]]], [[[5.0, 4.0, 5.0], [7.0, 1.0, 1.0], [8.0, 5.0, 4.0]], [[2.0, 8.0, 1.0], [7.0, 1.0, 8.0], [7.0, 6.0, 4.0]], [[4.0, 2.0, 
6.0], [2.0, 1.0, 1.0], [2.0, 5.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUUN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 6.0], [3.0, 2.0, 6.0], [2.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [5.0, 7.0, 5.0], [5.0, 1.0, 
7.0]], [[1.0, 8.0, 7.0], [7.0, 2.0, 7.0], [7.0, 4.0, 6.0]]], [[[3.0, 2.0, 8.0], [3.0, 8.0, 5.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 
5.0], [7.0, 8.0, 5.0], [4.0, 4.0, 6.0]], [[8.0, 3.0, 8.0], [7.0, 3.0, 2.0], [8.0, 2.0, 7.0]]], [[[3.0, 5.0, 4.0], [3.0, 1.0, 
6.0], [4.0, 3.0, 7.0]], [[7.0, 5.0, 3.0], [8.0, 4.0, 7.0], [7.0, 5.0, 6.0]], [[3.0, 2.0, 3.0], [4.0, 6.0, 6.0], [1.0, 1.0, 
2.0]]], [[[2.0, 2.0, 1.0], [4.0, 3.0, 6.0], [6.0, 8.0, 6.0]], [[5.0, 6.0, 8.0], [7.0, 4.0, 2.0], [6.0, 5.0, 2.0]], [[5.0, 3.0, 
2.0], [3.0, 3.0, 7.0], [2.0, 1.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 8.0, 4.0], [3.0, 6.0, 1.0]]], [[[3.0, 6.0, 2.0], 
[2.0, 8.0, 8.0]], [[1.0, 8.0, 5.0], [6.0, 2.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 5.0, 6.0], [4.0, 1.0, 3.0]], [[8.0, 
6.0, 4.0], [3.0, 3.0, 5.0]]], [[[5.0, 2.0, 8.0], [1.0, 4.0, 3.0]], [[3.0, 1.0, 4.0], [3.0, 8.0, 7.0]]]])
      arg[1:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 6.0], [3.0, 2.0, 6.0], [2.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [5.0, 7.0, 5.0], [5.0, 
1.0, 7.0]], [[1.0, 8.0, 7.0], [7.0, 2.0, 7.0], [7.0, 4.0, 6.0]]], [[[2.0, 4.0, 7.0], [5.0, 2.0, 7.0], [1.0, 1.0, 7.0]], [[6.0, 
8.0, 4.0], [3.0, 6.0, 1.0], [4.0, 4.0, 6.0]], [[8.0, 3.0, 8.0], [7.0, 3.0, 2.0], [8.0, 2.0, 7.0]]], [[[3.0, 6.0, 2.0], [2.0, 
8.0, 8.0], [4.0, 3.0, 7.0]], [[1.0, 8.0, 5.0], [6.0, 2.0, 3.0], [7.0, 5.0, 6.0]], [[3.0, 2.0, 3.0], [4.0, 6.0, 6.0], [1.0, 1.0, 
2.0]]], [[[2.0, 2.0, 1.0], [4.0, 3.0, 6.0], [6.0, 8.0, 6.0]], [[5.0, 6.0, 8.0], [7.0, 4.0, 2.0], [6.0, 5.0, 2.0]], [[5.0, 3.0, 
2.0], [3.0, 3.0, 7.0], [2.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 6.0], [3.0, 2.0, 6.0], [2.0, 3.0, 8.0]], 
[[3.0, 8.0, 2.0], [5.0, 7.0, 5.0], [5.0, 1.0, 7.0]], [[1.0, 8.0, 7.0], [7.0, 2.0, 7.0], [7.0, 4.0, 6.0]]], [[[7.0, 5.0, 6.0], 
[4.0, 1.0, 3.0], [1.0, 1.0, 7.0]], [[8.0, 6.0, 4.0], [3.0, 3.0, 5.0], [4.0, 4.0, 6.0]], [[8.0, 3.0, 8.0], [7.0, 3.0, 2.0], 
[8.0, 2.0, 7.0]]], [[[5.0, 2.0, 8.0], [1.0, 4.0, 3.0], [4.0, 3.0, 7.0]], [[3.0, 1.0, 4.0], [3.0, 8.0, 7.0], [7.0, 5.0, 6.0]], 
[[3.0, 2.0, 3.0], [4.0, 6.0, 6.0], [1.0, 1.0, 2.0]]], [[[2.0, 2.0, 1.0], [4.0, 3.0, 6.0], [6.0, 8.0, 6.0]], [[5.0, 6.0, 8.0], 
[7.0, 4.0, 2.0], [6.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [3.0, 3.0, 7.0], [2.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URRP(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 8.0], [6.0, 8.0, 4.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 8.0], [2.0, 2.0, 3.0], [8.0, 6.0, 
3.0]], [[5.0, 8.0, 3.0], [1.0, 8.0, 8.0], [6.0, 7.0, 5.0]]], [[[7.0, 3.0, 8.0], [5.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 1.0, 
8.0], [8.0, 8.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 1.0, 4.0], [4.0, 6.0, 7.0], [5.0, 8.0, 5.0]]], [[[3.0, 6.0, 1.0], [2.0, 7.0, 
8.0], [4.0, 3.0, 4.0]], [[7.0, 5.0, 1.0], [4.0, 1.0, 6.0], [4.0, 1.0, 4.0]], [[2.0, 5.0, 1.0], [5.0, 3.0, 1.0], [8.0, 7.0, 
2.0]]], [[[4.0, 3.0, 7.0], [3.0, 2.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 3.0, 5.0], [4.0, 1.0, 4.0], [6.0, 7.0, 3.0]], [[7.0, 5.0, 
4.0], [4.0, 7.0, 3.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[3.0]], [[3.0]], [[7.0]]])
      arg[:3,1:2,1:2,1]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 8.0], [6.0, 8.0, 4.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 8.0], [2.0, 3.0, 3.0], [8.0, 6.0, 
3.0]], [[5.0, 8.0, 3.0], [1.0, 8.0, 8.0], [6.0, 7.0, 5.0]]], [[[7.0, 3.0, 8.0], [5.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[1.0, 1.0, 
8.0], [8.0, 3.0, 8.0], [7.0, 8.0, 6.0]], [[5.0, 1.0, 4.0], [4.0, 6.0, 7.0], [5.0, 8.0, 5.0]]], [[[3.0, 6.0, 1.0], [2.0, 7.0, 
8.0], [4.0, 3.0, 4.0]], [[7.0, 5.0, 1.0], [4.0, 7.0, 6.0], [4.0, 1.0, 4.0]], [[2.0, 5.0, 1.0], [5.0, 3.0, 1.0], [8.0, 7.0, 
2.0]]], [[[4.0, 3.0, 7.0], [3.0, 2.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 3.0, 5.0], [4.0, 1.0, 4.0], [6.0, 7.0, 3.0]], [[7.0, 5.0, 
4.0], [4.0, 7.0, 3.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URRP(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 2.0], [2.0, 6.0, 6.0], [2.0, 3.0, 3.0]], [[3.0, 2.0, 8.0], [6.0, 7.0, 6.0], [4.0, 1.0, 
1.0]], [[5.0, 8.0, 5.0], [4.0, 5.0, 8.0], [5.0, 1.0, 6.0]]], [[[4.0, 2.0, 7.0], [6.0, 5.0, 6.0], [4.0, 5.0, 4.0]], [[6.0, 1.0, 
2.0], [7.0, 8.0, 6.0], [6.0, 4.0, 8.0]], [[3.0, 4.0, 2.0], [8.0, 5.0, 7.0], [2.0, 7.0, 6.0]]], [[[1.0, 7.0, 6.0], [4.0, 7.0, 
8.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 7.0], [4.0, 3.0, 3.0], [1.0, 7.0, 5.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 7.0], [1.0, 6.0, 
4.0]]], [[[8.0, 2.0, 2.0], [2.0, 2.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 2.0, 5.0], [2.0, 6.0, 3.0], [6.0, 2.0, 6.0]], [[2.0, 7.0, 
8.0], [6.0, 4.0, 4.0], [2.0, 4.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0]], [[8.0]], [[2.0]]]),self.functionspace)
      arg[:3,1:2,1:2,1]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 2.0], [2.0, 6.0, 6.0], [2.0, 3.0, 3.0]], [[3.0, 2.0, 8.0], [6.0, 6.0, 6.0], [4.0, 1.0, 
1.0]], [[5.0, 8.0, 5.0], [4.0, 5.0, 8.0], [5.0, 1.0, 6.0]]], [[[4.0, 2.0, 7.0], [6.0, 5.0, 6.0], [4.0, 5.0, 4.0]], [[6.0, 1.0, 
2.0], [7.0, 8.0, 6.0], [6.0, 4.0, 8.0]], [[3.0, 4.0, 2.0], [8.0, 5.0, 7.0], [2.0, 7.0, 6.0]]], [[[1.0, 7.0, 6.0], [4.0, 7.0, 
8.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 7.0], [4.0, 2.0, 3.0], [1.0, 7.0, 5.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 7.0], [1.0, 6.0, 
4.0]]], [[[8.0, 2.0, 2.0], [2.0, 2.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 2.0, 5.0], [2.0, 6.0, 3.0], [6.0, 2.0, 6.0]], [[2.0, 7.0, 
8.0], [6.0, 4.0, 4.0], [2.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URRP(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 3.0], [3.0, 5.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [1.0, 6.0, 7.0], [7.0, 2.0, 
7.0]], [[3.0, 8.0, 3.0], [2.0, 3.0, 8.0], [3.0, 8.0, 5.0]]], [[[4.0, 1.0, 3.0], [5.0, 2.0, 1.0], [5.0, 2.0, 3.0]], [[3.0, 2.0, 
4.0], [2.0, 8.0, 1.0], [3.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 2.0], [2.0, 4.0, 8.0]]], [[[2.0, 7.0, 7.0], [8.0, 5.0, 
2.0], [2.0, 4.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 8.0, 8.0], [5.0, 7.0, 1.0]], [[1.0, 4.0, 2.0], [7.0, 4.0, 6.0], [2.0, 4.0, 
8.0]]], [[[4.0, 5.0, 5.0], [3.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 8.0, 8.0], [7.0, 3.0, 6.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 
8.0], [3.0, 2.0, 1.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0]], [[2.0]], [[5.0]]])+(1.-msk_val)*numpy.array([[[2.0]], [[8.0]], [[5.0]]])
      arg[:3,1:2,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [3.0, 5.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [1.0, 4.0, 7.0], [7.0, 
2.0, 7.0]], [[3.0, 8.0, 3.0], [2.0, 3.0, 8.0], [3.0, 8.0, 5.0]]], [[[4.0, 1.0, 3.0], [5.0, 2.0, 1.0], [5.0, 2.0, 3.0]], [[3.0, 
2.0, 4.0], [2.0, 2.0, 1.0], [3.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 2.0], [2.0, 4.0, 8.0]]], [[[2.0, 7.0, 7.0], [8.0, 
5.0, 2.0], [2.0, 4.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 5.0, 8.0], [5.0, 7.0, 1.0]], [[1.0, 4.0, 2.0], [7.0, 4.0, 6.0], [2.0, 4.0, 
8.0]]], [[[4.0, 5.0, 5.0], [3.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 8.0, 8.0], [7.0, 3.0, 6.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 
8.0], [3.0, 2.0, 1.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [3.0, 5.0, 1.0], [6.0, 4.0, 2.0]], 
[[4.0, 4.0, 5.0], [1.0, 2.0, 7.0], [7.0, 2.0, 7.0]], [[3.0, 8.0, 3.0], [2.0, 3.0, 8.0], [3.0, 8.0, 5.0]]], [[[4.0, 1.0, 3.0], 
[5.0, 2.0, 1.0], [5.0, 2.0, 3.0]], [[3.0, 2.0, 4.0], [2.0, 8.0, 1.0], [3.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 2.0], 
[2.0, 4.0, 8.0]]], [[[2.0, 7.0, 7.0], [8.0, 5.0, 2.0], [2.0, 4.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 5.0, 8.0], [5.0, 7.0, 1.0]], 
[[1.0, 4.0, 2.0], [7.0, 4.0, 6.0], [2.0, 4.0, 8.0]]], [[[4.0, 5.0, 5.0], [3.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 8.0, 8.0], 
[7.0, 3.0, 6.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 8.0], [3.0, 2.0, 1.0], [3.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLUL(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [1.0, 2.0, 6.0], [5.0, 5.0, 6.0]], [[3.0, 8.0, 3.0], [6.0, 6.0, 1.0], [3.0, 1.0, 
3.0]], [[3.0, 1.0, 5.0], [5.0, 7.0, 1.0], [2.0, 6.0, 8.0]]], [[[7.0, 1.0, 2.0], [6.0, 7.0, 6.0], [6.0, 3.0, 8.0]], [[6.0, 7.0, 
8.0], [7.0, 2.0, 5.0], [8.0, 7.0, 4.0]], [[7.0, 1.0, 7.0], [4.0, 4.0, 3.0], [7.0, 5.0, 8.0]]], [[[1.0, 1.0, 6.0], [7.0, 4.0, 
8.0], [7.0, 8.0, 3.0]], [[7.0, 5.0, 4.0], [5.0, 3.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 8.0, 4.0], [5.0, 3.0, 1.0], [1.0, 1.0, 
2.0]]], [[[6.0, 7.0, 7.0], [4.0, 8.0, 1.0], [7.0, 7.0, 4.0]], [[8.0, 2.0, 7.0], [6.0, 2.0, 2.0], [4.0, 8.0, 8.0]], [[2.0, 5.0, 
4.0], [5.0, 4.0, 4.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0], [1.0]]], [[[5.0], [7.0]]]])
      arg[2:,2:,:2,2:]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 6.0], [1.0, 2.0, 6.0], [5.0, 5.0, 6.0]], [[3.0, 8.0, 3.0], [6.0, 6.0, 1.0], [3.0, 1.0, 
3.0]], [[3.0, 1.0, 5.0], [5.0, 7.0, 1.0], [2.0, 6.0, 8.0]]], [[[7.0, 1.0, 2.0], [6.0, 7.0, 6.0], [6.0, 3.0, 8.0]], [[6.0, 7.0, 
8.0], [7.0, 2.0, 5.0], [8.0, 7.0, 4.0]], [[7.0, 1.0, 7.0], [4.0, 4.0, 3.0], [7.0, 5.0, 8.0]]], [[[1.0, 1.0, 6.0], [7.0, 4.0, 
8.0], [7.0, 8.0, 3.0]], [[7.0, 5.0, 4.0], [5.0, 3.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 8.0, 4.0], [5.0, 3.0, 1.0], [1.0, 1.0, 
2.0]]], [[[6.0, 7.0, 7.0], [4.0, 8.0, 1.0], [7.0, 7.0, 4.0]], [[8.0, 2.0, 7.0], [6.0, 2.0, 2.0], [4.0, 8.0, 8.0]], [[2.0, 5.0, 
5.0], [5.0, 4.0, 7.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLUL(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 7.0], [6.0, 7.0, 7.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 8.0], [3.0, 6.0, 4.0], [3.0, 4.0, 
7.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 7.0], [2.0, 7.0, 3.0]]], [[[3.0, 1.0, 8.0], [6.0, 6.0, 8.0], [3.0, 2.0, 6.0]], [[6.0, 5.0, 
8.0], [2.0, 4.0, 6.0], [8.0, 1.0, 6.0]], [[4.0, 4.0, 7.0], [4.0, 4.0, 1.0], [2.0, 2.0, 1.0]]], [[[8.0, 1.0, 7.0], [1.0, 1.0, 
6.0], [3.0, 6.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 6.0], [7.0, 4.0, 5.0]], [[8.0, 3.0, 3.0], [3.0, 5.0, 7.0], [6.0, 2.0, 
1.0]]], [[[8.0, 8.0, 1.0], [2.0, 2.0, 4.0], [6.0, 3.0, 4.0]], [[8.0, 1.0, 2.0], [5.0, 5.0, 8.0], [3.0, 8.0, 3.0]], [[1.0, 7.0, 
2.0], [2.0, 6.0, 6.0], [1.0, 5.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0], [8.0]]], [[[3.0], [5.0]]]]),self.functionspace)
      arg[2:,2:,:2,2:]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 7.0], [6.0, 7.0, 7.0], [5.0, 2.0, 5.0]], [[6.0, 4.0, 8.0], [3.0, 6.0, 4.0], [3.0, 4.0, 
7.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 7.0], [2.0, 7.0, 3.0]]], [[[3.0, 1.0, 8.0], [6.0, 6.0, 8.0], [3.0, 2.0, 6.0]], [[6.0, 5.0, 
8.0], [2.0, 4.0, 6.0], [8.0, 1.0, 6.0]], [[4.0, 4.0, 7.0], [4.0, 4.0, 1.0], [2.0, 2.0, 1.0]]], [[[8.0, 1.0, 7.0], [1.0, 1.0, 
6.0], [3.0, 6.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 6.0], [7.0, 4.0, 5.0]], [[8.0, 3.0, 7.0], [3.0, 5.0, 8.0], [6.0, 2.0, 
1.0]]], [[[8.0, 8.0, 1.0], [2.0, 2.0, 4.0], [6.0, 3.0, 4.0]], [[8.0, 1.0, 2.0], [5.0, 5.0, 8.0], [3.0, 8.0, 3.0]], [[1.0, 7.0, 
3.0], [2.0, 6.0, 5.0], [1.0, 5.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLUL(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 3.0], [6.0, 7.0, 8.0], [7.0, 3.0, 2.0]], [[5.0, 6.0, 4.0], [4.0, 6.0, 4.0], [4.0, 3.0, 
7.0]], [[1.0, 5.0, 2.0], [5.0, 3.0, 4.0], [5.0, 6.0, 7.0]]], [[[5.0, 2.0, 4.0], [7.0, 4.0, 1.0], [7.0, 8.0, 8.0]], [[8.0, 4.0, 
6.0], [3.0, 3.0, 5.0], [7.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], [1.0, 3.0, 6.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 6.0], [5.0, 7.0, 
4.0], [8.0, 3.0, 5.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 7.0]], [[3.0, 8.0, 2.0], [6.0, 2.0, 6.0], [2.0, 1.0, 
8.0]]], [[[3.0, 7.0, 3.0], [5.0, 3.0, 6.0], [1.0, 7.0, 8.0]], [[1.0, 4.0, 7.0], [4.0, 8.0, 7.0], [4.0, 1.0, 3.0]], [[4.0, 6.0, 
1.0], [1.0, 2.0, 4.0], [6.0, 4.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0], [8.0]]], [[[1.0], [2.0]]]])+(1.-msk_val)*numpy.array([[[[3.0], [7.0]]], [[[5.0], 
[6.0]]]])
      arg[2:,2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 3.0], [6.0, 7.0, 8.0], [7.0, 3.0, 2.0]], [[5.0, 6.0, 4.0], [4.0, 6.0, 4.0], [4.0, 
3.0, 7.0]], [[1.0, 5.0, 2.0], [5.0, 3.0, 4.0], [5.0, 6.0, 7.0]]], [[[5.0, 2.0, 4.0], [7.0, 4.0, 1.0], [7.0, 8.0, 8.0]], [[8.0, 
4.0, 6.0], [3.0, 3.0, 5.0], [7.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], [1.0, 3.0, 6.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 6.0], [5.0, 
7.0, 4.0], [8.0, 3.0, 5.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 7.0]], [[3.0, 8.0, 8.0], [6.0, 2.0, 8.0], [2.0, 1.0, 
8.0]]], [[[3.0, 7.0, 3.0], [5.0, 3.0, 6.0], [1.0, 7.0, 8.0]], [[1.0, 4.0, 7.0], [4.0, 8.0, 7.0], [4.0, 1.0, 3.0]], [[4.0, 6.0, 
1.0], [1.0, 2.0, 2.0], [6.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 3.0], [6.0, 7.0, 8.0], [7.0, 3.0, 2.0]], 
[[5.0, 6.0, 4.0], [4.0, 6.0, 4.0], [4.0, 3.0, 7.0]], [[1.0, 5.0, 2.0], [5.0, 3.0, 4.0], [5.0, 6.0, 7.0]]], [[[5.0, 2.0, 4.0], 
[7.0, 4.0, 1.0], [7.0, 8.0, 8.0]], [[8.0, 4.0, 6.0], [3.0, 3.0, 5.0], [7.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], [1.0, 3.0, 6.0], 
[4.0, 7.0, 1.0]]], [[[7.0, 2.0, 6.0], [5.0, 7.0, 4.0], [8.0, 3.0, 5.0]], [[5.0, 3.0, 7.0], [1.0, 1.0, 2.0], [1.0, 1.0, 7.0]], 
[[3.0, 8.0, 3.0], [6.0, 2.0, 7.0], [2.0, 1.0, 8.0]]], [[[3.0, 7.0, 3.0], [5.0, 3.0, 6.0], [1.0, 7.0, 8.0]], [[1.0, 4.0, 7.0], 
[4.0, 8.0, 7.0], [4.0, 1.0, 3.0]], [[4.0, 6.0, 5.0], [1.0, 2.0, 6.0], [6.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CRCN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 1.0], [5.0, 6.0, 1.0], [6.0, 8.0, 7.0]], [[3.0, 2.0, 7.0], [3.0, 7.0, 7.0], [1.0, 7.0, 
2.0]], [[4.0, 7.0, 5.0], [1.0, 4.0, 5.0], [5.0, 6.0, 4.0]]], [[[5.0, 6.0, 6.0], [1.0, 3.0, 1.0], [6.0, 6.0, 5.0]], [[7.0, 5.0, 
1.0], [1.0, 8.0, 8.0], [2.0, 3.0, 1.0]], [[1.0, 6.0, 5.0], [8.0, 3.0, 4.0], [8.0, 1.0, 3.0]]], [[[6.0, 3.0, 7.0], [6.0, 6.0, 
7.0], [2.0, 3.0, 3.0]], [[1.0, 6.0, 8.0], [1.0, 3.0, 7.0], [2.0, 2.0, 4.0]], [[8.0, 7.0, 4.0], [2.0, 2.0, 2.0], [7.0, 7.0, 
1.0]]], [[[4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [2.0, 7.0, 2.0]], [[4.0, 7.0, 1.0], [3.0, 7.0, 2.0], [5.0, 7.0, 1.0]], [[5.0, 3.0, 
1.0], [8.0, 3.0, 7.0], [8.0, 1.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 6.0, 7.0], [7.0, 4.0, 5.0], [2.0, 4.0, 6.0]]], [[[7.0, 7.0, 7.0], [2.0, 1.0, 6.0], [4.0, 8.0, 
3.0]]], [[[4.0, 2.0, 2.0], [3.0, 5.0, 7.0], [2.0, 8.0, 7.0]]], [[[4.0, 4.0, 7.0], [2.0, 3.0, 4.0], [7.0, 5.0, 1.0]]]])
      arg[:,1:2,:]=val
      ref=Data(numpy.array([[[[1.0, 5.0, 1.0], [5.0, 6.0, 1.0], [6.0, 8.0, 7.0]], [[4.0, 6.0, 7.0], [7.0, 4.0, 5.0], [2.0, 4.0, 
6.0]], [[4.0, 7.0, 5.0], [1.0, 4.0, 5.0], [5.0, 6.0, 4.0]]], [[[5.0, 6.0, 6.0], [1.0, 3.0, 1.0], [6.0, 6.0, 5.0]], [[7.0, 7.0, 
7.0], [2.0, 1.0, 6.0], [4.0, 8.0, 3.0]], [[1.0, 6.0, 5.0], [8.0, 3.0, 4.0], [8.0, 1.0, 3.0]]], [[[6.0, 3.0, 7.0], [6.0, 6.0, 
7.0], [2.0, 3.0, 3.0]], [[4.0, 2.0, 2.0], [3.0, 5.0, 7.0], [2.0, 8.0, 7.0]], [[8.0, 7.0, 4.0], [2.0, 2.0, 2.0], [7.0, 7.0, 
1.0]]], [[[4.0, 6.0, 1.0], [3.0, 1.0, 7.0], [2.0, 7.0, 2.0]], [[4.0, 4.0, 7.0], [2.0, 3.0, 4.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 
1.0], [8.0, 3.0, 7.0], [8.0, 1.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CRCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 2.0], [3.0, 8.0, 1.0], [8.0, 7.0, 2.0]], [[8.0, 7.0, 8.0], [8.0, 5.0, 5.0], [7.0, 5.0, 
4.0]], [[6.0, 7.0, 2.0], [6.0, 3.0, 1.0], [5.0, 2.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 4.0, 3.0], [8.0, 7.0, 1.0]], [[8.0, 7.0, 
2.0], [3.0, 3.0, 4.0], [8.0, 6.0, 7.0]], [[4.0, 2.0, 3.0], [2.0, 5.0, 6.0], [1.0, 3.0, 6.0]]], [[[8.0, 7.0, 7.0], [6.0, 4.0, 
5.0], [4.0, 6.0, 3.0]], [[3.0, 1.0, 6.0], [3.0, 2.0, 1.0], [7.0, 6.0, 8.0]], [[3.0, 2.0, 3.0], [5.0, 6.0, 1.0], [8.0, 7.0, 
5.0]]], [[[4.0, 3.0, 2.0], [7.0, 7.0, 2.0], [8.0, 7.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 1.0, 1.0], [6.0, 4.0, 7.0]], [[6.0, 7.0, 
4.0], [2.0, 1.0, 7.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 6.0, 8.0], [7.0, 5.0, 1.0], [4.0, 3.0, 4.0]]], [[[2.0, 3.0, 6.0], [2.0, 1.0, 1.0], [4.0, 
8.0, 4.0]]], [[[5.0, 1.0, 6.0], [7.0, 4.0, 8.0], [1.0, 6.0, 8.0]]], [[[7.0, 7.0, 1.0], [7.0, 3.0, 1.0], [2.0, 6.0, 
3.0]]]]),self.functionspace)
      arg[:,1:2,:]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 2.0], [3.0, 8.0, 1.0], [8.0, 7.0, 2.0]], [[3.0, 6.0, 8.0], [7.0, 5.0, 1.0], [4.0, 3.0, 
4.0]], [[6.0, 7.0, 2.0], [6.0, 3.0, 1.0], [5.0, 2.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 4.0, 3.0], [8.0, 7.0, 1.0]], [[2.0, 3.0, 
6.0], [2.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[4.0, 2.0, 3.0], [2.0, 5.0, 6.0], [1.0, 3.0, 6.0]]], [[[8.0, 7.0, 7.0], [6.0, 4.0, 
5.0], [4.0, 6.0, 3.0]], [[5.0, 1.0, 6.0], [7.0, 4.0, 8.0], [1.0, 6.0, 8.0]], [[3.0, 2.0, 3.0], [5.0, 6.0, 1.0], [8.0, 7.0, 
5.0]]], [[[4.0, 3.0, 2.0], [7.0, 7.0, 2.0], [8.0, 7.0, 2.0]], [[7.0, 7.0, 1.0], [7.0, 3.0, 1.0], [2.0, 6.0, 3.0]], [[6.0, 7.0, 
4.0], [2.0, 1.0, 7.0], [2.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CRCN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 8.0], [5.0, 6.0, 4.0], [2.0, 1.0, 4.0]], [[8.0, 7.0, 2.0], [7.0, 5.0, 2.0], [5.0, 7.0, 
5.0]], [[3.0, 2.0, 7.0], [2.0, 5.0, 3.0], [6.0, 5.0, 6.0]]], [[[1.0, 4.0, 4.0], [7.0, 4.0, 6.0], [1.0, 6.0, 5.0]], [[1.0, 2.0, 
3.0], [8.0, 3.0, 4.0], [1.0, 6.0, 5.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 7.0], [7.0, 1.0, 6.0]]], [[[8.0, 5.0, 8.0], [7.0, 1.0, 
1.0], [7.0, 7.0, 3.0]], [[3.0, 2.0, 3.0], [3.0, 2.0, 2.0], [5.0, 3.0, 3.0]], [[8.0, 6.0, 4.0], [6.0, 4.0, 3.0], [4.0, 1.0, 
5.0]]], [[[1.0, 3.0, 6.0], [3.0, 8.0, 5.0], [3.0, 2.0, 1.0]], [[4.0, 8.0, 8.0], [6.0, 6.0, 5.0], [7.0, 2.0, 2.0]], [[4.0, 4.0, 
7.0], [8.0, 1.0, 6.0], [7.0, 8.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 3.0, 4.0], [1.0, 7.0, 1.0], [7.0, 3.0, 2.0]]], [[[2.0, 6.0, 5.0], [3.0, 4.0, 7.0], [3.0, 
6.0, 6.0]]], [[[6.0, 6.0, 5.0], [2.0, 3.0, 7.0], [4.0, 6.0, 7.0]]], [[[2.0, 8.0, 4.0], [2.0, 1.0, 1.0], [3.0, 8.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 3.0, 7.0], [2.0, 2.0, 8.0], [2.0, 5.0, 4.0]]], [[[4.0, 3.0, 2.0], [6.0, 7.0, 2.0], 
[7.0, 6.0, 8.0]]], [[[4.0, 4.0, 8.0], [2.0, 2.0, 2.0], [6.0, 6.0, 5.0]]], [[[5.0, 2.0, 2.0], [5.0, 4.0, 2.0], [4.0, 8.0, 
6.0]]]])
      arg[:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [5.0, 6.0, 4.0], [2.0, 1.0, 4.0]], [[6.0, 3.0, 4.0], [1.0, 7.0, 1.0], [7.0, 
3.0, 2.0]], [[3.0, 2.0, 7.0], [2.0, 5.0, 3.0], [6.0, 5.0, 6.0]]], [[[1.0, 4.0, 4.0], [7.0, 4.0, 6.0], [1.0, 6.0, 5.0]], [[2.0, 
6.0, 5.0], [3.0, 4.0, 7.0], [3.0, 6.0, 6.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 7.0], [7.0, 1.0, 6.0]]], [[[8.0, 5.0, 8.0], [7.0, 
1.0, 1.0], [7.0, 7.0, 3.0]], [[6.0, 6.0, 5.0], [2.0, 3.0, 7.0], [4.0, 6.0, 7.0]], [[8.0, 6.0, 4.0], [6.0, 4.0, 3.0], [4.0, 1.0, 
5.0]]], [[[1.0, 3.0, 6.0], [3.0, 8.0, 5.0], [3.0, 2.0, 1.0]], [[2.0, 8.0, 4.0], [2.0, 1.0, 1.0], [3.0, 8.0, 7.0]], [[4.0, 4.0, 
7.0], [8.0, 1.0, 6.0], [7.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 8.0], [5.0, 6.0, 4.0], [2.0, 1.0, 4.0]], 
[[6.0, 3.0, 7.0], [2.0, 2.0, 8.0], [2.0, 5.0, 4.0]], [[3.0, 2.0, 7.0], [2.0, 5.0, 3.0], [6.0, 5.0, 6.0]]], [[[1.0, 4.0, 4.0], 
[7.0, 4.0, 6.0], [1.0, 6.0, 5.0]], [[4.0, 3.0, 2.0], [6.0, 7.0, 2.0], [7.0, 6.0, 8.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 7.0], 
[7.0, 1.0, 6.0]]], [[[8.0, 5.0, 8.0], [7.0, 1.0, 1.0], [7.0, 7.0, 3.0]], [[4.0, 4.0, 8.0], [2.0, 2.0, 2.0], [6.0, 6.0, 5.0]], 
[[8.0, 6.0, 4.0], [6.0, 4.0, 3.0], [4.0, 1.0, 5.0]]], [[[1.0, 3.0, 6.0], [3.0, 8.0, 5.0], [3.0, 2.0, 1.0]], [[5.0, 2.0, 2.0], 
[5.0, 4.0, 2.0], [4.0, 8.0, 6.0]], [[4.0, 4.0, 7.0], [8.0, 1.0, 6.0], [7.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CPCN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 1.0], [1.0, 2.0, 2.0], [1.0, 8.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 4.0, 3.0], [2.0, 6.0, 
2.0]], [[6.0, 7.0, 1.0], [7.0, 7.0, 7.0], [3.0, 2.0, 7.0]]], [[[4.0, 4.0, 7.0], [4.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[1.0, 1.0, 
2.0], [1.0, 6.0, 8.0], [5.0, 3.0, 6.0]], [[3.0, 2.0, 5.0], [4.0, 8.0, 4.0], [2.0, 6.0, 3.0]]], [[[2.0, 2.0, 6.0], [2.0, 1.0, 
4.0], [5.0, 1.0, 4.0]], [[8.0, 4.0, 2.0], [6.0, 4.0, 6.0], [3.0, 5.0, 1.0]], [[6.0, 7.0, 6.0], [1.0, 7.0, 7.0], [2.0, 7.0, 
5.0]]], [[[1.0, 3.0, 3.0], [2.0, 6.0, 4.0], [3.0, 8.0, 5.0]], [[6.0, 5.0, 8.0], [1.0, 6.0, 1.0], [4.0, 4.0, 3.0]], [[8.0, 3.0, 
2.0], [1.0, 6.0, 6.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 1.0, 6.0], [2.0, 3.0, 2.0], [5.0, 6.0, 7.0]], [[1.0, 6.0, 5.0], [2.0, 6.0, 1.0], [3.0, 8.0, 
3.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 5.0], [3.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], [5.0, 4.0, 2.0], [3.0, 6.0, 3.0]]])
      arg[:,1,:]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 1.0], [1.0, 2.0, 2.0], [1.0, 8.0, 4.0]], [[1.0, 1.0, 6.0], [2.0, 3.0, 2.0], [5.0, 6.0, 
7.0]], [[6.0, 7.0, 1.0], [7.0, 7.0, 7.0], [3.0, 2.0, 7.0]]], [[[4.0, 4.0, 7.0], [4.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[1.0, 6.0, 
5.0], [2.0, 6.0, 1.0], [3.0, 8.0, 3.0]], [[3.0, 2.0, 5.0], [4.0, 8.0, 4.0], [2.0, 6.0, 3.0]]], [[[2.0, 2.0, 6.0], [2.0, 1.0, 
4.0], [5.0, 1.0, 4.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 5.0], [3.0, 3.0, 3.0]], [[6.0, 7.0, 6.0], [1.0, 7.0, 7.0], [2.0, 7.0, 
5.0]]], [[[1.0, 3.0, 3.0], [2.0, 6.0, 4.0], [3.0, 8.0, 5.0]], [[5.0, 3.0, 1.0], [5.0, 4.0, 2.0], [3.0, 6.0, 3.0]], [[8.0, 3.0, 
2.0], [1.0, 6.0, 6.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CPCN(self):
      arg=Data(numpy.array([[[[4.0, 1.0, 1.0], [3.0, 5.0, 5.0], [1.0, 3.0, 2.0]], [[3.0, 5.0, 2.0], [8.0, 6.0, 8.0], [5.0, 1.0, 
5.0]], [[6.0, 1.0, 2.0], [4.0, 1.0, 5.0], [6.0, 2.0, 1.0]]], [[[7.0, 8.0, 7.0], [8.0, 2.0, 1.0], [7.0, 1.0, 3.0]], [[6.0, 3.0, 
7.0], [2.0, 6.0, 2.0], [6.0, 8.0, 4.0]], [[7.0, 7.0, 2.0], [7.0, 5.0, 4.0], [2.0, 4.0, 4.0]]], [[[4.0, 4.0, 2.0], [6.0, 2.0, 
4.0], [6.0, 5.0, 4.0]], [[2.0, 8.0, 4.0], [1.0, 6.0, 2.0], [4.0, 7.0, 4.0]], [[8.0, 8.0, 1.0], [4.0, 6.0, 8.0], [1.0, 5.0, 
4.0]]], [[[7.0, 3.0, 2.0], [5.0, 8.0, 5.0], [7.0, 7.0, 6.0]], [[7.0, 7.0, 1.0], [8.0, 1.0, 4.0], [8.0, 6.0, 6.0]], [[2.0, 4.0, 
4.0], [5.0, 4.0, 8.0], [2.0, 5.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 2.0, 3.0], [3.0, 4.0, 5.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 7.0], [4.0, 1.0, 3.0], [4.0, 5.0, 
6.0]], [[2.0, 1.0, 1.0], [8.0, 1.0, 1.0], [7.0, 8.0, 4.0]], [[1.0, 7.0, 5.0], [8.0, 2.0, 7.0], [5.0, 5.0, 
1.0]]]),self.functionspace)
      arg[:,1,:]=val
      ref=Data(numpy.array([[[[4.0, 1.0, 1.0], [3.0, 5.0, 5.0], [1.0, 3.0, 2.0]], [[5.0, 2.0, 3.0], [3.0, 4.0, 5.0], [5.0, 6.0, 
5.0]], [[6.0, 1.0, 2.0], [4.0, 1.0, 5.0], [6.0, 2.0, 1.0]]], [[[7.0, 8.0, 7.0], [8.0, 2.0, 1.0], [7.0, 1.0, 3.0]], [[3.0, 4.0, 
7.0], [4.0, 1.0, 3.0], [4.0, 5.0, 6.0]], [[7.0, 7.0, 2.0], [7.0, 5.0, 4.0], [2.0, 4.0, 4.0]]], [[[4.0, 4.0, 2.0], [6.0, 2.0, 
4.0], [6.0, 5.0, 4.0]], [[2.0, 1.0, 1.0], [8.0, 1.0, 1.0], [7.0, 8.0, 4.0]], [[8.0, 8.0, 1.0], [4.0, 6.0, 8.0], [1.0, 5.0, 
4.0]]], [[[7.0, 3.0, 2.0], [5.0, 8.0, 5.0], [7.0, 7.0, 6.0]], [[1.0, 7.0, 5.0], [8.0, 2.0, 7.0], [5.0, 5.0, 1.0]], [[2.0, 4.0, 
4.0], [5.0, 4.0, 8.0], [2.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CPCN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 7.0], [5.0, 7.0, 2.0], [4.0, 2.0, 5.0]], [[3.0, 5.0, 1.0], [4.0, 1.0, 7.0], [6.0, 5.0, 
1.0]], [[7.0, 7.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 4.0]]], [[[2.0, 5.0, 7.0], [7.0, 7.0, 5.0], [4.0, 4.0, 5.0]], [[2.0, 1.0, 
6.0], [3.0, 8.0, 4.0], [4.0, 1.0, 4.0]], [[6.0, 7.0, 6.0], [2.0, 3.0, 4.0], [6.0, 3.0, 8.0]]], [[[1.0, 5.0, 1.0], [6.0, 3.0, 
2.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 1.0], [6.0, 8.0, 8.0], [8.0, 2.0, 8.0]], [[3.0, 4.0, 7.0], [5.0, 5.0, 3.0], [8.0, 7.0, 
7.0]]], [[[5.0, 8.0, 2.0], [1.0, 3.0, 3.0], [7.0, 1.0, 1.0]], [[2.0, 7.0, 3.0], [4.0, 1.0, 5.0], [5.0, 4.0, 3.0]], [[5.0, 1.0, 
4.0], [6.0, 6.0, 4.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 3.0, 6.0], [4.0, 6.0, 1.0], [7.0, 4.0, 7.0]], [[5.0, 3.0, 7.0], [8.0, 5.0, 2.0], [7.0, 
2.0, 2.0]], [[1.0, 8.0, 6.0], [2.0, 4.0, 8.0], [2.0, 8.0, 3.0]], [[5.0, 4.0, 5.0], [7.0, 6.0, 8.0], [8.0, 4.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[2.0, 6.0, 6.0], [5.0, 3.0, 4.0], [6.0, 3.0, 4.0]], [[4.0, 2.0, 4.0], [2.0, 1.0, 6.0], [7.0, 
1.0, 4.0]], [[3.0, 1.0, 5.0], [5.0, 6.0, 6.0], [7.0, 4.0, 2.0]], [[4.0, 1.0, 2.0], [4.0, 1.0, 3.0], [8.0, 1.0, 2.0]]])
      arg[:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 7.0], [5.0, 7.0, 2.0], [4.0, 2.0, 5.0]], [[7.0, 3.0, 6.0], [4.0, 6.0, 1.0], [7.0, 
4.0, 7.0]], [[7.0, 7.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 4.0]]], [[[2.0, 5.0, 7.0], [7.0, 7.0, 5.0], [4.0, 4.0, 5.0]], [[5.0, 
3.0, 7.0], [8.0, 5.0, 2.0], [7.0, 2.0, 2.0]], [[6.0, 7.0, 6.0], [2.0, 3.0, 4.0], [6.0, 3.0, 8.0]]], [[[1.0, 5.0, 1.0], [6.0, 
3.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 6.0], [2.0, 4.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 4.0, 7.0], [5.0, 5.0, 3.0], [8.0, 7.0, 
7.0]]], [[[5.0, 8.0, 2.0], [1.0, 3.0, 3.0], [7.0, 1.0, 1.0]], [[5.0, 4.0, 5.0], [7.0, 6.0, 8.0], [8.0, 4.0, 1.0]], [[5.0, 1.0, 
4.0], [6.0, 6.0, 4.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 7.0], [5.0, 7.0, 2.0], [4.0, 2.0, 5.0]], 
[[2.0, 6.0, 6.0], [5.0, 3.0, 4.0], [6.0, 3.0, 4.0]], [[7.0, 7.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 4.0]]], [[[2.0, 5.0, 7.0], 
[7.0, 7.0, 5.0], [4.0, 4.0, 5.0]], [[4.0, 2.0, 4.0], [2.0, 1.0, 6.0], [7.0, 1.0, 4.0]], [[6.0, 7.0, 6.0], [2.0, 3.0, 4.0], 
[6.0, 3.0, 8.0]]], [[[1.0, 5.0, 1.0], [6.0, 3.0, 2.0], [5.0, 5.0, 4.0]], [[3.0, 1.0, 5.0], [5.0, 6.0, 6.0], [7.0, 4.0, 2.0]], 
[[3.0, 4.0, 7.0], [5.0, 5.0, 3.0], [8.0, 7.0, 7.0]]], [[[5.0, 8.0, 2.0], [1.0, 3.0, 3.0], [7.0, 1.0, 1.0]], [[4.0, 1.0, 2.0], 
[4.0, 1.0, 3.0], [8.0, 1.0, 2.0]], [[5.0, 1.0, 4.0], [6.0, 6.0, 4.0], [6.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRUU(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 5.0], [2.0, 4.0, 8.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [3.0, 7.0, 2.0], [1.0, 5.0, 
6.0]], [[7.0, 5.0, 7.0], [8.0, 6.0, 8.0], [8.0, 8.0, 4.0]]], [[[7.0, 4.0, 7.0], [4.0, 4.0, 6.0], [5.0, 3.0, 5.0]], [[7.0, 4.0, 
7.0], [2.0, 1.0, 2.0], [6.0, 2.0, 2.0]], [[5.0, 6.0, 1.0], [2.0, 6.0, 8.0], [8.0, 4.0, 8.0]]], [[[1.0, 5.0, 7.0], [6.0, 4.0, 
4.0], [2.0, 2.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 8.0, 6.0], [6.0, 4.0, 4.0]], [[6.0, 5.0, 8.0], [4.0, 8.0, 8.0], [4.0, 8.0, 
1.0]]], [[[8.0, 8.0, 8.0], [3.0, 8.0, 7.0], [4.0, 8.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 8.0, 1.0], [1.0, 6.0, 4.0]], [[6.0, 3.0, 
6.0], [5.0, 7.0, 3.0], [6.0, 7.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 2.0], [3.0, 8.0]]])
      arg[2,1:2,:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 5.0], [2.0, 4.0, 8.0], [7.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [3.0, 7.0, 2.0], [1.0, 5.0, 
6.0]], [[7.0, 5.0, 7.0], [8.0, 6.0, 8.0], [8.0, 8.0, 4.0]]], [[[7.0, 4.0, 7.0], [4.0, 4.0, 6.0], [5.0, 3.0, 5.0]], [[7.0, 4.0, 
7.0], [2.0, 1.0, 2.0], [6.0, 2.0, 2.0]], [[5.0, 6.0, 1.0], [2.0, 6.0, 8.0], [8.0, 4.0, 8.0]]], [[[1.0, 5.0, 7.0], [6.0, 4.0, 
4.0], [2.0, 2.0, 1.0]], [[7.0, 2.0, 3.0], [3.0, 8.0, 6.0], [6.0, 4.0, 4.0]], [[6.0, 5.0, 8.0], [4.0, 8.0, 8.0], [4.0, 8.0, 
1.0]]], [[[8.0, 8.0, 8.0], [3.0, 8.0, 7.0], [4.0, 8.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 8.0, 1.0], [1.0, 6.0, 4.0]], [[6.0, 3.0, 
6.0], [5.0, 7.0, 3.0], [6.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRUU(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 3.0], [6.0, 4.0, 5.0], [6.0, 2.0, 2.0]], [[3.0, 1.0, 3.0], [7.0, 3.0, 6.0], [1.0, 8.0, 
3.0]], [[6.0, 4.0, 3.0], [3.0, 3.0, 3.0], [3.0, 8.0, 4.0]]], [[[2.0, 6.0, 6.0], [4.0, 3.0, 4.0], [6.0, 6.0, 6.0]], [[1.0, 1.0, 
7.0], [1.0, 5.0, 5.0], [2.0, 8.0, 5.0]], [[4.0, 5.0, 6.0], [1.0, 6.0, 4.0], [7.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0], [7.0, 8.0, 
3.0], [8.0, 6.0, 5.0]], [[2.0, 2.0, 2.0], [5.0, 8.0, 3.0], [2.0, 5.0, 8.0]], [[5.0, 5.0, 8.0], [7.0, 6.0, 2.0], [1.0, 4.0, 
5.0]]], [[[3.0, 8.0, 7.0], [6.0, 8.0, 2.0], [1.0, 1.0, 8.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 8.0], [3.0, 1.0, 7.0]], [[6.0, 4.0, 
4.0], [6.0, 1.0, 8.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 6.0], [3.0, 5.0]]]),self.functionspace)
      arg[2,1:2,:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 3.0], [6.0, 4.0, 5.0], [6.0, 2.0, 2.0]], [[3.0, 1.0, 3.0], [7.0, 3.0, 6.0], [1.0, 8.0, 
3.0]], [[6.0, 4.0, 3.0], [3.0, 3.0, 3.0], [3.0, 8.0, 4.0]]], [[[2.0, 6.0, 6.0], [4.0, 3.0, 4.0], [6.0, 6.0, 6.0]], [[1.0, 1.0, 
7.0], [1.0, 5.0, 5.0], [2.0, 8.0, 5.0]], [[4.0, 5.0, 6.0], [1.0, 6.0, 4.0], [7.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0], [7.0, 8.0, 
3.0], [8.0, 6.0, 5.0]], [[3.0, 6.0, 2.0], [3.0, 5.0, 3.0], [2.0, 5.0, 8.0]], [[5.0, 5.0, 8.0], [7.0, 6.0, 2.0], [1.0, 4.0, 
5.0]]], [[[3.0, 8.0, 7.0], [6.0, 8.0, 2.0], [1.0, 1.0, 8.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 8.0], [3.0, 1.0, 7.0]], [[6.0, 4.0, 
4.0], [6.0, 1.0, 8.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRUU(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [6.0, 6.0, 8.0], [4.0, 4.0, 8.0]], [[4.0, 7.0, 2.0], [8.0, 5.0, 1.0], [3.0, 6.0, 
4.0]], [[6.0, 8.0, 6.0], [1.0, 3.0, 5.0], [2.0, 1.0, 3.0]]], [[[1.0, 7.0, 6.0], [7.0, 5.0, 8.0], [7.0, 7.0, 8.0]], [[2.0, 7.0, 
1.0], [4.0, 5.0, 4.0], [4.0, 8.0, 5.0]], [[7.0, 5.0, 4.0], [1.0, 3.0, 6.0], [8.0, 6.0, 4.0]]], [[[7.0, 8.0, 1.0], [5.0, 6.0, 
4.0], [8.0, 6.0, 2.0]], [[1.0, 8.0, 6.0], [3.0, 4.0, 7.0], [3.0, 4.0, 4.0]], [[7.0, 3.0, 2.0], [1.0, 6.0, 2.0], [6.0, 7.0, 
2.0]]], [[[1.0, 7.0, 4.0], [4.0, 1.0, 7.0], [7.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], [6.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[7.0, 6.0, 
8.0], [7.0, 6.0, 8.0], [4.0, 5.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 1.0], [6.0, 2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0], [4.0, 6.0]]])
      arg[2,1:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [6.0, 6.0, 8.0], [4.0, 4.0, 8.0]], [[4.0, 7.0, 2.0], [8.0, 5.0, 1.0], [3.0, 
6.0, 4.0]], [[6.0, 8.0, 6.0], [1.0, 3.0, 5.0], [2.0, 1.0, 3.0]]], [[[1.0, 7.0, 6.0], [7.0, 5.0, 8.0], [7.0, 7.0, 8.0]], [[2.0, 
7.0, 1.0], [4.0, 5.0, 4.0], [4.0, 8.0, 5.0]], [[7.0, 5.0, 4.0], [1.0, 3.0, 6.0], [8.0, 6.0, 4.0]]], [[[7.0, 8.0, 1.0], [5.0, 
6.0, 4.0], [8.0, 6.0, 2.0]], [[7.0, 1.0, 6.0], [6.0, 2.0, 7.0], [3.0, 4.0, 4.0]], [[7.0, 3.0, 2.0], [1.0, 6.0, 2.0], [6.0, 7.0, 
2.0]]], [[[1.0, 7.0, 4.0], [4.0, 1.0, 7.0], [7.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], [6.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[7.0, 6.0, 
8.0], [7.0, 6.0, 8.0], [4.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 7.0], [6.0, 6.0, 8.0], [4.0, 4.0, 8.0]], 
[[4.0, 7.0, 2.0], [8.0, 5.0, 1.0], [3.0, 6.0, 4.0]], [[6.0, 8.0, 6.0], [1.0, 3.0, 5.0], [2.0, 1.0, 3.0]]], [[[1.0, 7.0, 6.0], 
[7.0, 5.0, 8.0], [7.0, 7.0, 8.0]], [[2.0, 7.0, 1.0], [4.0, 5.0, 4.0], [4.0, 8.0, 5.0]], [[7.0, 5.0, 4.0], [1.0, 3.0, 6.0], 
[8.0, 6.0, 4.0]]], [[[7.0, 8.0, 1.0], [5.0, 6.0, 4.0], [8.0, 6.0, 2.0]], [[4.0, 6.0, 6.0], [4.0, 6.0, 7.0], [3.0, 4.0, 4.0]], 
[[7.0, 3.0, 2.0], [1.0, 6.0, 2.0], [6.0, 7.0, 2.0]]], [[[1.0, 7.0, 4.0], [4.0, 1.0, 7.0], [7.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], 
[6.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[7.0, 6.0, 8.0], [7.0, 6.0, 8.0], [4.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLRC(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 4.0], [2.0, 1.0, 2.0], [5.0, 8.0, 5.0]], [[7.0, 5.0, 2.0], [4.0, 7.0, 6.0], [6.0, 5.0, 
2.0]], [[3.0, 8.0, 8.0], [2.0, 5.0, 5.0], [2.0, 5.0, 5.0]]], [[[8.0, 4.0, 7.0], [2.0, 5.0, 2.0], [5.0, 4.0, 5.0]], [[2.0, 1.0, 
3.0], [7.0, 4.0, 1.0], [8.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], [1.0, 8.0, 7.0], [3.0, 3.0, 2.0]]], [[[2.0, 6.0, 1.0], [5.0, 2.0, 
2.0], [5.0, 1.0, 1.0]], [[6.0, 1.0, 5.0], [4.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[5.0, 4.0, 8.0], [1.0, 5.0, 5.0], [6.0, 6.0, 
4.0]]], [[[2.0, 8.0, 3.0], [2.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 6.0, 
1.0], [2.0, 2.0, 5.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 6.0, 8.0]]], [[[4.0, 7.0, 3.0]]], [[[2.0, 2.0, 6.0]]], [[[1.0, 2.0, 1.0]]]])
      arg[:,2:,1:2,:]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 4.0], [2.0, 1.0, 2.0], [5.0, 8.0, 5.0]], [[7.0, 5.0, 2.0], [4.0, 7.0, 6.0], [6.0, 5.0, 
2.0]], [[3.0, 8.0, 8.0], [3.0, 6.0, 8.0], [2.0, 5.0, 5.0]]], [[[8.0, 4.0, 7.0], [2.0, 5.0, 2.0], [5.0, 4.0, 5.0]], [[2.0, 1.0, 
3.0], [7.0, 4.0, 1.0], [8.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], [4.0, 7.0, 3.0], [3.0, 3.0, 2.0]]], [[[2.0, 6.0, 1.0], [5.0, 2.0, 
2.0], [5.0, 1.0, 1.0]], [[6.0, 1.0, 5.0], [4.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[5.0, 4.0, 8.0], [2.0, 2.0, 6.0], [6.0, 6.0, 
4.0]]], [[[2.0, 8.0, 3.0], [2.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 6.0, 
1.0], [1.0, 2.0, 1.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLRC(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [8.0, 8.0, 1.0], [4.0, 1.0, 2.0]], [[3.0, 2.0, 3.0], [6.0, 1.0, 7.0], [2.0, 8.0, 
8.0]], [[8.0, 8.0, 3.0], [1.0, 2.0, 3.0], [2.0, 5.0, 5.0]]], [[[3.0, 4.0, 2.0], [7.0, 7.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 1.0, 
6.0], [7.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 2.0], [8.0, 6.0, 7.0], [3.0, 4.0, 3.0]]], [[[1.0, 1.0, 8.0], [7.0, 6.0, 
5.0], [1.0, 4.0, 8.0]], [[3.0, 1.0, 4.0], [7.0, 3.0, 2.0], [2.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], [8.0, 4.0, 6.0], [3.0, 2.0, 
4.0]]], [[[5.0, 5.0, 6.0], [1.0, 2.0, 2.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 2.0], [8.0, 6.0, 2.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 
5.0], [4.0, 7.0, 8.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 1.0, 7.0]]], [[[2.0, 1.0, 5.0]]], [[[7.0, 5.0, 8.0]]], [[[4.0, 3.0, 
3.0]]]]),self.functionspace)
      arg[:,2:,1:2,:]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 5.0], [8.0, 8.0, 1.0], [4.0, 1.0, 2.0]], [[3.0, 2.0, 3.0], [6.0, 1.0, 7.0], [2.0, 8.0, 
8.0]], [[8.0, 8.0, 3.0], [3.0, 1.0, 7.0], [2.0, 5.0, 5.0]]], [[[3.0, 4.0, 2.0], [7.0, 7.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 1.0, 
6.0], [7.0, 3.0, 7.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 2.0], [2.0, 1.0, 5.0], [3.0, 4.0, 3.0]]], [[[1.0, 1.0, 8.0], [7.0, 6.0, 
5.0], [1.0, 4.0, 8.0]], [[3.0, 1.0, 4.0], [7.0, 3.0, 2.0], [2.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], [7.0, 5.0, 8.0], [3.0, 2.0, 
4.0]]], [[[5.0, 5.0, 6.0], [1.0, 2.0, 2.0], [7.0, 5.0, 8.0]], [[8.0, 8.0, 2.0], [8.0, 6.0, 2.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 
5.0], [4.0, 3.0, 3.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLRC(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 8.0], [3.0, 1.0, 5.0], [2.0, 4.0, 1.0]], [[5.0, 1.0, 1.0], [2.0, 4.0, 6.0], [3.0, 5.0, 
7.0]], [[1.0, 1.0, 3.0], [2.0, 4.0, 8.0], [4.0, 5.0, 8.0]]], [[[1.0, 4.0, 4.0], [7.0, 6.0, 2.0], [1.0, 5.0, 5.0]], [[5.0, 8.0, 
7.0], [3.0, 2.0, 6.0], [8.0, 1.0, 8.0]], [[4.0, 3.0, 8.0], [7.0, 5.0, 7.0], [3.0, 7.0, 1.0]]], [[[6.0, 3.0, 8.0], [3.0, 1.0, 
6.0], [1.0, 8.0, 2.0]], [[6.0, 2.0, 1.0], [8.0, 7.0, 4.0], [8.0, 1.0, 4.0]], [[4.0, 2.0, 2.0], [8.0, 7.0, 8.0], [5.0, 8.0, 
7.0]]], [[[7.0, 7.0, 6.0], [1.0, 7.0, 4.0], [8.0, 7.0, 1.0]], [[1.0, 3.0, 8.0], [2.0, 8.0, 7.0], [5.0, 3.0, 3.0]], [[7.0, 3.0, 
4.0], [3.0, 7.0, 2.0], [1.0, 6.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 8.0, 2.0]]], [[[4.0, 5.0, 6.0]]], [[[7.0, 3.0, 6.0]]], [[[8.0, 8.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 4.0, 1.0]]], [[[7.0, 7.0, 8.0]]], [[[3.0, 1.0, 7.0]]], [[[3.0, 1.0, 2.0]]]])
      arg[:,2:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 8.0], [3.0, 1.0, 5.0], [2.0, 4.0, 1.0]], [[5.0, 1.0, 1.0], [2.0, 4.0, 6.0], [3.0, 
5.0, 7.0]], [[1.0, 1.0, 3.0], [1.0, 8.0, 2.0], [4.0, 5.0, 8.0]]], [[[1.0, 4.0, 4.0], [7.0, 6.0, 2.0], [1.0, 5.0, 5.0]], [[5.0, 
8.0, 7.0], [3.0, 2.0, 6.0], [8.0, 1.0, 8.0]], [[4.0, 3.0, 8.0], [4.0, 5.0, 6.0], [3.0, 7.0, 1.0]]], [[[6.0, 3.0, 8.0], [3.0, 
1.0, 6.0], [1.0, 8.0, 2.0]], [[6.0, 2.0, 1.0], [8.0, 7.0, 4.0], [8.0, 1.0, 4.0]], [[4.0, 2.0, 2.0], [7.0, 3.0, 6.0], [5.0, 8.0, 
7.0]]], [[[7.0, 7.0, 6.0], [1.0, 7.0, 4.0], [8.0, 7.0, 1.0]], [[1.0, 3.0, 8.0], [2.0, 8.0, 7.0], [5.0, 3.0, 3.0]], [[7.0, 3.0, 
4.0], [8.0, 8.0, 6.0], [1.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [3.0, 1.0, 5.0], [2.0, 4.0, 1.0]], 
[[5.0, 1.0, 1.0], [2.0, 4.0, 6.0], [3.0, 5.0, 7.0]], [[1.0, 1.0, 3.0], [4.0, 4.0, 1.0], [4.0, 5.0, 8.0]]], [[[1.0, 4.0, 4.0], 
[7.0, 6.0, 2.0], [1.0, 5.0, 5.0]], [[5.0, 8.0, 7.0], [3.0, 2.0, 6.0], [8.0, 1.0, 8.0]], [[4.0, 3.0, 8.0], [7.0, 7.0, 8.0], 
[3.0, 7.0, 1.0]]], [[[6.0, 3.0, 8.0], [3.0, 1.0, 6.0], [1.0, 8.0, 2.0]], [[6.0, 2.0, 1.0], [8.0, 7.0, 4.0], [8.0, 1.0, 4.0]], 
[[4.0, 2.0, 2.0], [3.0, 1.0, 7.0], [5.0, 8.0, 7.0]]], [[[7.0, 7.0, 6.0], [1.0, 7.0, 4.0], [8.0, 7.0, 1.0]], [[1.0, 3.0, 8.0], 
[2.0, 8.0, 7.0], [5.0, 3.0, 3.0]], [[7.0, 3.0, 4.0], [3.0, 1.0, 2.0], [1.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UUCN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 8.0], [4.0, 2.0, 5.0], [5.0, 6.0, 7.0]], [[2.0, 1.0, 2.0], [2.0, 2.0, 5.0], [1.0, 5.0, 
8.0]], [[5.0, 1.0, 4.0], [5.0, 4.0, 3.0], [8.0, 8.0, 6.0]]], [[[8.0, 1.0, 2.0], [7.0, 2.0, 3.0], [6.0, 6.0, 7.0]], [[6.0, 8.0, 
8.0], [6.0, 4.0, 3.0], [4.0, 1.0, 1.0]], [[3.0, 4.0, 6.0], [2.0, 6.0, 3.0], [4.0, 5.0, 6.0]]], [[[7.0, 6.0, 1.0], [5.0, 1.0, 
8.0], [2.0, 2.0, 8.0]], [[3.0, 8.0, 6.0], [8.0, 8.0, 5.0], [5.0, 7.0, 8.0]], [[5.0, 4.0, 2.0], [7.0, 7.0, 5.0], [3.0, 7.0, 
8.0]]], [[[2.0, 2.0, 2.0], [6.0, 1.0, 8.0], [4.0, 2.0, 5.0]], [[7.0, 5.0, 1.0], [3.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 8.0, 
4.0], [6.0, 1.0, 1.0], [5.0, 7.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 8.0, 5.0], [4.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 4.0], [4.0, 4.0, 
7.0]]], [[[2.0, 3.0, 5.0], [5.0, 4.0, 1.0], [8.0, 8.0, 6.0]], [[8.0, 2.0, 8.0], [5.0, 2.0, 4.0], [7.0, 3.0, 6.0]]], [[[3.0, 
4.0, 5.0], [4.0, 1.0, 6.0], [1.0, 7.0, 1.0]], [[5.0, 4.0, 7.0], [6.0, 2.0, 6.0], [2.0, 1.0, 8.0]]]])
      arg[:3,:2,:]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 5.0], [4.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 4.0], [4.0, 4.0, 
7.0]], [[5.0, 1.0, 4.0], [5.0, 4.0, 3.0], [8.0, 8.0, 6.0]]], [[[2.0, 3.0, 5.0], [5.0, 4.0, 1.0], [8.0, 8.0, 6.0]], [[8.0, 2.0, 
8.0], [5.0, 2.0, 4.0], [7.0, 3.0, 6.0]], [[3.0, 4.0, 6.0], [2.0, 6.0, 3.0], [4.0, 5.0, 6.0]]], [[[3.0, 4.0, 5.0], [4.0, 1.0, 
6.0], [1.0, 7.0, 1.0]], [[5.0, 4.0, 7.0], [6.0, 2.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 4.0, 2.0], [7.0, 7.0, 5.0], [3.0, 7.0, 
8.0]]], [[[2.0, 2.0, 2.0], [6.0, 1.0, 8.0], [4.0, 2.0, 5.0]], [[7.0, 5.0, 1.0], [3.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 8.0, 
4.0], [6.0, 1.0, 1.0], [5.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UUCN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 8.0], [1.0, 8.0, 4.0], [5.0, 5.0, 1.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 8.0], [7.0, 7.0, 
3.0]], [[4.0, 2.0, 8.0], [4.0, 4.0, 8.0], [8.0, 4.0, 5.0]]], [[[7.0, 7.0, 1.0], [4.0, 6.0, 5.0], [6.0, 6.0, 5.0]], [[4.0, 8.0, 
5.0], [6.0, 3.0, 4.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 7.0], [6.0, 7.0, 7.0], [1.0, 1.0, 2.0]]], [[[8.0, 6.0, 6.0], [3.0, 6.0, 
6.0], [5.0, 8.0, 6.0]], [[7.0, 2.0, 3.0], [8.0, 2.0, 4.0], [8.0, 7.0, 4.0]], [[4.0, 1.0, 8.0], [3.0, 3.0, 4.0], [4.0, 1.0, 
1.0]]], [[[1.0, 8.0, 3.0], [2.0, 7.0, 8.0], [7.0, 3.0, 2.0]], [[5.0, 5.0, 1.0], [2.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[5.0, 3.0, 
8.0], [6.0, 8.0, 7.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 7.0, 8.0], [3.0, 2.0, 4.0], [5.0, 1.0, 6.0]], [[7.0, 6.0, 7.0], [8.0, 3.0, 4.0], [1.0, 7.0, 
7.0]]], [[[7.0, 3.0, 5.0], [7.0, 3.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 6.0, 3.0], [1.0, 8.0, 1.0], [6.0, 8.0, 6.0]]], [[[2.0, 
2.0, 7.0], [8.0, 4.0, 8.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 7.0], [8.0, 3.0, 1.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      arg[:3,:2,:]=val
      ref=Data(numpy.array([[[[5.0, 7.0, 8.0], [3.0, 2.0, 4.0], [5.0, 1.0, 6.0]], [[7.0, 6.0, 7.0], [8.0, 3.0, 4.0], [1.0, 7.0, 
7.0]], [[4.0, 2.0, 8.0], [4.0, 4.0, 8.0], [8.0, 4.0, 5.0]]], [[[7.0, 3.0, 5.0], [7.0, 3.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 6.0, 
3.0], [1.0, 8.0, 1.0], [6.0, 8.0, 6.0]], [[7.0, 7.0, 7.0], [6.0, 7.0, 7.0], [1.0, 1.0, 2.0]]], [[[2.0, 2.0, 7.0], [8.0, 4.0, 
8.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 7.0], [8.0, 3.0, 1.0], [8.0, 5.0, 8.0]], [[4.0, 1.0, 8.0], [3.0, 3.0, 4.0], [4.0, 1.0, 
1.0]]], [[[1.0, 8.0, 3.0], [2.0, 7.0, 8.0], [7.0, 3.0, 2.0]], [[5.0, 5.0, 1.0], [2.0, 4.0, 4.0], [2.0, 7.0, 5.0]], [[5.0, 3.0, 
8.0], [6.0, 8.0, 7.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UUCN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 8.0], [2.0, 3.0, 5.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 4.0], [8.0, 7.0, 2.0], [1.0, 4.0, 
6.0]], [[8.0, 2.0, 8.0], [5.0, 4.0, 3.0], [5.0, 4.0, 1.0]]], [[[3.0, 8.0, 7.0], [5.0, 7.0, 1.0], [7.0, 5.0, 5.0]], [[3.0, 1.0, 
1.0], [8.0, 4.0, 1.0], [2.0, 8.0, 2.0]], [[1.0, 2.0, 8.0], [3.0, 3.0, 7.0], [4.0, 3.0, 5.0]]], [[[1.0, 1.0, 1.0], [4.0, 1.0, 
5.0], [1.0, 8.0, 3.0]], [[2.0, 2.0, 4.0], [8.0, 3.0, 8.0], [8.0, 6.0, 7.0]], [[7.0, 8.0, 3.0], [2.0, 7.0, 2.0], [3.0, 5.0, 
1.0]]], [[[5.0, 3.0, 5.0], [5.0, 1.0, 2.0], [1.0, 3.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 8.0, 5.0], [8.0, 5.0, 5.0]], [[8.0, 4.0, 
8.0], [8.0, 2.0, 1.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 5.0, 4.0], [6.0, 2.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 7.0, 8.0], [2.0, 8.0, 2.0], [2.0, 
2.0, 5.0]]], [[[8.0, 7.0, 8.0], [2.0, 3.0, 5.0], [6.0, 6.0, 2.0]], [[5.0, 6.0, 5.0], [6.0, 1.0, 2.0], [7.0, 8.0, 5.0]]], 
[[[1.0, 4.0, 4.0], [6.0, 5.0, 5.0], [4.0, 4.0, 1.0]], [[5.0, 4.0, 1.0], [8.0, 7.0, 3.0], [8.0, 1.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 8.0, 8.0], [5.0, 1.0, 3.0], [8.0, 6.0, 4.0]], [[3.0, 1.0, 7.0], [5.0, 1.0, 7.0], 
[6.0, 5.0, 6.0]]], [[[5.0, 6.0, 8.0], [2.0, 6.0, 8.0], [1.0, 7.0, 3.0]], [[7.0, 3.0, 5.0], [8.0, 6.0, 6.0], [4.0, 3.0, 3.0]]], 
[[[7.0, 5.0, 5.0], [2.0, 6.0, 6.0], [3.0, 8.0, 7.0]], [[2.0, 7.0, 1.0], [3.0, 6.0, 2.0], [1.0, 8.0, 1.0]]]])
      arg[:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 4.0], [6.0, 2.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 7.0, 8.0], [2.0, 8.0, 2.0], [2.0, 
2.0, 5.0]], [[8.0, 2.0, 8.0], [5.0, 4.0, 3.0], [5.0, 4.0, 1.0]]], [[[8.0, 7.0, 8.0], [2.0, 3.0, 5.0], [6.0, 6.0, 2.0]], [[5.0, 
6.0, 5.0], [6.0, 1.0, 2.0], [7.0, 8.0, 5.0]], [[1.0, 2.0, 8.0], [3.0, 3.0, 7.0], [4.0, 3.0, 5.0]]], [[[1.0, 4.0, 4.0], [6.0, 
5.0, 5.0], [4.0, 4.0, 1.0]], [[5.0, 4.0, 1.0], [8.0, 7.0, 3.0], [8.0, 1.0, 7.0]], [[7.0, 8.0, 3.0], [2.0, 7.0, 2.0], [3.0, 5.0, 
1.0]]], [[[5.0, 3.0, 5.0], [5.0, 1.0, 2.0], [1.0, 3.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 8.0, 5.0], [8.0, 5.0, 5.0]], [[8.0, 4.0, 
8.0], [8.0, 2.0, 1.0], [8.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 8.0], [5.0, 1.0, 3.0], [8.0, 6.0, 4.0]], 
[[3.0, 1.0, 7.0], [5.0, 1.0, 7.0], [6.0, 5.0, 6.0]], [[8.0, 2.0, 8.0], [5.0, 4.0, 3.0], [5.0, 4.0, 1.0]]], [[[5.0, 6.0, 8.0], 
[2.0, 6.0, 8.0], [1.0, 7.0, 3.0]], [[7.0, 3.0, 5.0], [8.0, 6.0, 6.0], [4.0, 3.0, 3.0]], [[1.0, 2.0, 8.0], [3.0, 3.0, 7.0], 
[4.0, 3.0, 5.0]]], [[[7.0, 5.0, 5.0], [2.0, 6.0, 6.0], [3.0, 8.0, 7.0]], [[2.0, 7.0, 1.0], [3.0, 6.0, 2.0], [1.0, 8.0, 1.0]], 
[[7.0, 8.0, 3.0], [2.0, 7.0, 2.0], [3.0, 5.0, 1.0]]], [[[5.0, 3.0, 5.0], [5.0, 1.0, 2.0], [1.0, 3.0, 1.0]], [[8.0, 5.0, 1.0], 
[7.0, 8.0, 5.0], [8.0, 5.0, 5.0]], [[8.0, 4.0, 8.0], [8.0, 2.0, 1.0], [8.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 1.0], [3.0, 1.0, 4.0], [8.0, 1.0, 8.0]], [[3.0, 7.0, 1.0], [1.0, 2.0, 2.0], [7.0, 8.0, 
5.0]], [[7.0, 4.0, 7.0], [6.0, 4.0, 1.0], [4.0, 5.0, 7.0]]], [[[7.0, 6.0, 5.0], [1.0, 7.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 6.0, 
6.0], [5.0, 3.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], [7.0, 7.0, 2.0], [4.0, 7.0, 6.0]]], [[[5.0, 1.0, 1.0], [7.0, 1.0, 
4.0], [5.0, 3.0, 5.0]], [[3.0, 8.0, 7.0], [4.0, 2.0, 7.0], [4.0, 4.0, 2.0]], [[1.0, 2.0, 5.0], [5.0, 4.0, 1.0], [4.0, 5.0, 
3.0]]], [[[8.0, 1.0, 6.0], [7.0, 4.0, 8.0], [3.0, 8.0, 6.0]], [[2.0, 1.0, 8.0], [1.0, 7.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 4.0, 
1.0], [5.0, 8.0, 6.0], [8.0, 2.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 6.0, 5.0]]])
      arg[2,2:,2:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 1.0], [3.0, 1.0, 4.0], [8.0, 1.0, 8.0]], [[3.0, 7.0, 1.0], [1.0, 2.0, 2.0], [7.0, 8.0, 
5.0]], [[7.0, 4.0, 7.0], [6.0, 4.0, 1.0], [4.0, 5.0, 7.0]]], [[[7.0, 6.0, 5.0], [1.0, 7.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 6.0, 
6.0], [5.0, 3.0, 6.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 7.0], [7.0, 7.0, 2.0], [4.0, 7.0, 6.0]]], [[[5.0, 1.0, 1.0], [7.0, 1.0, 
4.0], [5.0, 3.0, 5.0]], [[3.0, 8.0, 7.0], [4.0, 2.0, 7.0], [4.0, 4.0, 2.0]], [[1.0, 2.0, 5.0], [5.0, 4.0, 1.0], [4.0, 6.0, 
5.0]]], [[[8.0, 1.0, 6.0], [7.0, 4.0, 8.0], [3.0, 8.0, 6.0]], [[2.0, 1.0, 8.0], [1.0, 7.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 4.0, 
1.0], [5.0, 8.0, 6.0], [8.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLLN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 3.0], [7.0, 1.0, 6.0], [6.0, 6.0, 2.0]], [[2.0, 8.0, 1.0], [6.0, 2.0, 5.0], [3.0, 1.0, 
3.0]], [[5.0, 4.0, 5.0], [8.0, 8.0, 7.0], [4.0, 7.0, 2.0]]], [[[4.0, 4.0, 5.0], [5.0, 4.0, 2.0], [1.0, 7.0, 8.0]], [[4.0, 5.0, 
1.0], [5.0, 7.0, 8.0], [3.0, 3.0, 7.0]], [[5.0, 5.0, 6.0], [4.0, 3.0, 3.0], [6.0, 7.0, 6.0]]], [[[8.0, 6.0, 2.0], [2.0, 6.0, 
5.0], [5.0, 8.0, 6.0]], [[8.0, 7.0, 1.0], [8.0, 3.0, 1.0], [1.0, 2.0, 2.0]], [[5.0, 3.0, 7.0], [8.0, 3.0, 8.0], [3.0, 3.0, 
7.0]]], [[[6.0, 4.0, 4.0], [4.0, 6.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 3.0, 6.0], [8.0, 4.0, 7.0], [5.0, 7.0, 5.0]], [[4.0, 4.0, 
8.0], [4.0, 3.0, 1.0], [1.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 4.0, 5.0]]]),self.functionspace)
      arg[2,2:,2:]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 3.0], [7.0, 1.0, 6.0], [6.0, 6.0, 2.0]], [[2.0, 8.0, 1.0], [6.0, 2.0, 5.0], [3.0, 1.0, 
3.0]], [[5.0, 4.0, 5.0], [8.0, 8.0, 7.0], [4.0, 7.0, 2.0]]], [[[4.0, 4.0, 5.0], [5.0, 4.0, 2.0], [1.0, 7.0, 8.0]], [[4.0, 5.0, 
1.0], [5.0, 7.0, 8.0], [3.0, 3.0, 7.0]], [[5.0, 5.0, 6.0], [4.0, 3.0, 3.0], [6.0, 7.0, 6.0]]], [[[8.0, 6.0, 2.0], [2.0, 6.0, 
5.0], [5.0, 8.0, 6.0]], [[8.0, 7.0, 1.0], [8.0, 3.0, 1.0], [1.0, 2.0, 2.0]], [[5.0, 3.0, 7.0], [8.0, 3.0, 8.0], [5.0, 4.0, 
5.0]]], [[[6.0, 4.0, 4.0], [4.0, 6.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 3.0, 6.0], [8.0, 4.0, 7.0], [5.0, 7.0, 5.0]], [[4.0, 4.0, 
8.0], [4.0, 3.0, 1.0], [1.0, 4.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLLN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 4.0], [1.0, 8.0, 6.0], [6.0, 5.0, 3.0]], [[1.0, 7.0, 4.0], [1.0, 5.0, 7.0], [5.0, 1.0, 
3.0]], [[4.0, 5.0, 6.0], [5.0, 2.0, 7.0], [6.0, 3.0, 3.0]]], [[[7.0, 4.0, 1.0], [1.0, 4.0, 2.0], [3.0, 3.0, 1.0]], [[3.0, 8.0, 
3.0], [1.0, 6.0, 7.0], [3.0, 5.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 7.0, 2.0], [5.0, 4.0, 4.0]]], [[[7.0, 3.0, 7.0], [2.0, 2.0, 
6.0], [1.0, 2.0, 4.0]], [[5.0, 7.0, 1.0], [5.0, 3.0, 7.0], [6.0, 8.0, 3.0]], [[3.0, 4.0, 6.0], [5.0, 4.0, 6.0], [2.0, 3.0, 
8.0]]], [[[2.0, 1.0, 5.0], [3.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 6.0], [3.0, 5.0, 4.0], [6.0, 2.0, 7.0]], [[1.0, 4.0, 
4.0], [8.0, 1.0, 3.0], [2.0, 1.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 7.0, 3.0]]])+(1.-msk_val)*numpy.array([[[5.0, 2.0, 5.0]]])
      arg[2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 4.0], [1.0, 8.0, 6.0], [6.0, 5.0, 3.0]], [[1.0, 7.0, 4.0], [1.0, 5.0, 7.0], [5.0, 
1.0, 3.0]], [[4.0, 5.0, 6.0], [5.0, 2.0, 7.0], [6.0, 3.0, 3.0]]], [[[7.0, 4.0, 1.0], [1.0, 4.0, 2.0], [3.0, 3.0, 1.0]], [[3.0, 
8.0, 3.0], [1.0, 6.0, 7.0], [3.0, 5.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 7.0, 2.0], [5.0, 4.0, 4.0]]], [[[7.0, 3.0, 7.0], [2.0, 
2.0, 6.0], [1.0, 2.0, 4.0]], [[5.0, 7.0, 1.0], [5.0, 3.0, 7.0], [6.0, 8.0, 3.0]], [[3.0, 4.0, 6.0], [5.0, 4.0, 6.0], [4.0, 7.0, 
3.0]]], [[[2.0, 1.0, 5.0], [3.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 6.0], [3.0, 5.0, 4.0], [6.0, 2.0, 7.0]], [[1.0, 4.0, 
4.0], [8.0, 1.0, 3.0], [2.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [1.0, 8.0, 6.0], [6.0, 5.0, 3.0]], 
[[1.0, 7.0, 4.0], [1.0, 5.0, 7.0], [5.0, 1.0, 3.0]], [[4.0, 5.0, 6.0], [5.0, 2.0, 7.0], [6.0, 3.0, 3.0]]], [[[7.0, 4.0, 1.0], 
[1.0, 4.0, 2.0], [3.0, 3.0, 1.0]], [[3.0, 8.0, 3.0], [1.0, 6.0, 7.0], [3.0, 5.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 7.0, 2.0], 
[5.0, 4.0, 4.0]]], [[[7.0, 3.0, 7.0], [2.0, 2.0, 6.0], [1.0, 2.0, 4.0]], [[5.0, 7.0, 1.0], [5.0, 3.0, 7.0], [6.0, 8.0, 3.0]], 
[[3.0, 4.0, 6.0], [5.0, 4.0, 6.0], [5.0, 2.0, 5.0]]], [[[2.0, 1.0, 5.0], [3.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[3.0, 5.0, 6.0], 
[3.0, 5.0, 4.0], [6.0, 2.0, 7.0]], [[1.0, 4.0, 4.0], [8.0, 1.0, 3.0], [2.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UUUN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 4.0], [7.0, 2.0, 7.0], [7.0, 1.0, 5.0]], [[8.0, 3.0, 2.0], [8.0, 3.0, 2.0], [6.0, 1.0, 
6.0]], [[5.0, 4.0, 7.0], [5.0, 5.0, 3.0], [4.0, 3.0, 6.0]]], [[[6.0, 5.0, 7.0], [8.0, 5.0, 6.0], [2.0, 1.0, 2.0]], [[7.0, 8.0, 
2.0], [3.0, 5.0, 7.0], [2.0, 1.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 5.0, 2.0], [8.0, 6.0, 6.0]]], [[[8.0, 7.0, 4.0], [8.0, 6.0, 
2.0], [1.0, 2.0, 7.0]], [[2.0, 8.0, 7.0], [1.0, 5.0, 2.0], [7.0, 1.0, 8.0]], [[8.0, 2.0, 5.0], [1.0, 6.0, 3.0], [3.0, 2.0, 
7.0]]], [[[1.0, 8.0, 2.0], [6.0, 8.0, 3.0], [2.0, 2.0, 7.0]], [[7.0, 2.0, 6.0], [5.0, 8.0, 7.0], [3.0, 5.0, 5.0]], [[3.0, 7.0, 
3.0], [5.0, 3.0, 1.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 6.0, 2.0], [3.0, 6.0, 1.0]], [[5.0, 6.0, 5.0], [6.0, 6.0, 5.0]]], [[[2.0, 5.0, 6.0], [7.0, 7.0, 
1.0]], [[7.0, 1.0, 1.0], [1.0, 7.0, 6.0]]], [[[3.0, 8.0, 4.0], [2.0, 3.0, 3.0]], [[3.0, 1.0, 3.0], [2.0, 4.0, 4.0]]]])
      arg[:3,:2,:2]=val
      ref=Data(numpy.array([[[[7.0, 6.0, 2.0], [3.0, 6.0, 1.0], [7.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [6.0, 6.0, 5.0], [6.0, 1.0, 
6.0]], [[5.0, 4.0, 7.0], [5.0, 5.0, 3.0], [4.0, 3.0, 6.0]]], [[[2.0, 5.0, 6.0], [7.0, 7.0, 1.0], [2.0, 1.0, 2.0]], [[7.0, 1.0, 
1.0], [1.0, 7.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 5.0, 2.0], [8.0, 6.0, 6.0]]], [[[3.0, 8.0, 4.0], [2.0, 3.0, 
3.0], [1.0, 2.0, 7.0]], [[3.0, 1.0, 3.0], [2.0, 4.0, 4.0], [7.0, 1.0, 8.0]], [[8.0, 2.0, 5.0], [1.0, 6.0, 3.0], [3.0, 2.0, 
7.0]]], [[[1.0, 8.0, 2.0], [6.0, 8.0, 3.0], [2.0, 2.0, 7.0]], [[7.0, 2.0, 6.0], [5.0, 8.0, 7.0], [3.0, 5.0, 5.0]], [[3.0, 7.0, 
3.0], [5.0, 3.0, 1.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UUUN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 3.0], [3.0, 3.0, 8.0], [8.0, 4.0, 3.0]], [[8.0, 7.0, 7.0], [1.0, 8.0, 4.0], [8.0, 7.0, 
1.0]], [[8.0, 5.0, 5.0], [5.0, 1.0, 2.0], [6.0, 8.0, 6.0]]], [[[5.0, 2.0, 2.0], [7.0, 3.0, 2.0], [2.0, 1.0, 2.0]], [[4.0, 5.0, 
7.0], [4.0, 4.0, 5.0], [2.0, 8.0, 2.0]], [[1.0, 6.0, 3.0], [5.0, 3.0, 6.0], [5.0, 5.0, 4.0]]], [[[4.0, 2.0, 5.0], [6.0, 2.0, 
4.0], [5.0, 8.0, 4.0]], [[4.0, 5.0, 2.0], [5.0, 8.0, 1.0], [1.0, 7.0, 3.0]], [[2.0, 7.0, 7.0], [8.0, 3.0, 1.0], [1.0, 2.0, 
4.0]]], [[[2.0, 7.0, 1.0], [3.0, 4.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 5.0], [8.0, 5.0, 6.0], [6.0, 7.0, 4.0]], [[2.0, 2.0, 
8.0], [1.0, 3.0, 4.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 4.0, 6.0], [1.0, 2.0, 2.0]], [[2.0, 2.0, 2.0], [5.0, 2.0, 8.0]]], [[[2.0, 5.0, 2.0], [3.0, 
7.0, 3.0]], [[8.0, 3.0, 2.0], [3.0, 8.0, 6.0]]], [[[5.0, 7.0, 4.0], [6.0, 6.0, 1.0]], [[6.0, 4.0, 7.0], [2.0, 7.0, 
2.0]]]]),self.functionspace)
      arg[:3,:2,:2]=val
      ref=Data(numpy.array([[[[7.0, 4.0, 6.0], [1.0, 2.0, 2.0], [8.0, 4.0, 3.0]], [[2.0, 2.0, 2.0], [5.0, 2.0, 8.0], [8.0, 7.0, 
1.0]], [[8.0, 5.0, 5.0], [5.0, 1.0, 2.0], [6.0, 8.0, 6.0]]], [[[2.0, 5.0, 2.0], [3.0, 7.0, 3.0], [2.0, 1.0, 2.0]], [[8.0, 3.0, 
2.0], [3.0, 8.0, 6.0], [2.0, 8.0, 2.0]], [[1.0, 6.0, 3.0], [5.0, 3.0, 6.0], [5.0, 5.0, 4.0]]], [[[5.0, 7.0, 4.0], [6.0, 6.0, 
1.0], [5.0, 8.0, 4.0]], [[6.0, 4.0, 7.0], [2.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[2.0, 7.0, 7.0], [8.0, 3.0, 1.0], [1.0, 2.0, 
4.0]]], [[[2.0, 7.0, 1.0], [3.0, 4.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 5.0], [8.0, 5.0, 6.0], [6.0, 7.0, 4.0]], [[2.0, 2.0, 
8.0], [1.0, 3.0, 4.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UUUN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 3.0], [6.0, 5.0, 1.0], [3.0, 8.0, 2.0]], [[2.0, 6.0, 4.0], [3.0, 2.0, 3.0], [7.0, 8.0, 
1.0]], [[6.0, 8.0, 5.0], [7.0, 3.0, 4.0], [6.0, 3.0, 6.0]]], [[[5.0, 7.0, 7.0], [5.0, 6.0, 6.0], [7.0, 5.0, 1.0]], [[2.0, 3.0, 
8.0], [3.0, 5.0, 3.0], [2.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 8.0, 7.0], [3.0, 6.0, 4.0]]], [[[3.0, 7.0, 4.0], [2.0, 1.0, 
8.0], [2.0, 3.0, 5.0]], [[7.0, 5.0, 6.0], [1.0, 8.0, 3.0], [4.0, 4.0, 7.0]], [[5.0, 5.0, 7.0], [8.0, 7.0, 7.0], [7.0, 5.0, 
8.0]]], [[[3.0, 6.0, 6.0], [1.0, 2.0, 3.0], [7.0, 7.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 4.0, 5.0], [7.0, 1.0, 1.0]], [[2.0, 7.0, 
4.0], [5.0, 4.0, 4.0], [2.0, 1.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 5.0, 1.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 7.0], [3.0, 7.0, 1.0]]], [[[2.0, 2.0, 2.0], 
[1.0, 7.0, 4.0]], [[3.0, 7.0, 7.0], [8.0, 8.0, 3.0]]], [[[3.0, 7.0, 3.0], [2.0, 4.0, 6.0]], [[6.0, 4.0, 4.0], [1.0, 2.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 3.0, 5.0], [2.0, 4.0, 5.0]], [[4.0, 5.0, 7.0], [4.0, 3.0, 5.0]]], [[[6.0, 8.0, 8.0], 
[6.0, 4.0, 5.0]], [[2.0, 2.0, 2.0], [4.0, 7.0, 6.0]]], [[[7.0, 6.0, 5.0], [2.0, 7.0, 1.0]], [[6.0, 2.0, 7.0], [1.0, 6.0, 
1.0]]]])
      arg[:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 1.0], [4.0, 8.0, 3.0], [3.0, 8.0, 2.0]], [[4.0, 3.0, 7.0], [3.0, 7.0, 1.0], [7.0, 
8.0, 1.0]], [[6.0, 8.0, 5.0], [7.0, 3.0, 4.0], [6.0, 3.0, 6.0]]], [[[2.0, 2.0, 2.0], [1.0, 7.0, 4.0], [7.0, 5.0, 1.0]], [[3.0, 
7.0, 7.0], [8.0, 8.0, 3.0], [2.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 8.0, 7.0], [3.0, 6.0, 4.0]]], [[[3.0, 7.0, 3.0], [2.0, 
4.0, 6.0], [2.0, 3.0, 5.0]], [[6.0, 4.0, 4.0], [1.0, 2.0, 5.0], [4.0, 4.0, 7.0]], [[5.0, 5.0, 7.0], [8.0, 7.0, 7.0], [7.0, 5.0, 
8.0]]], [[[3.0, 6.0, 6.0], [1.0, 2.0, 3.0], [7.0, 7.0, 5.0]], [[1.0, 5.0, 5.0], [2.0, 4.0, 5.0], [7.0, 1.0, 1.0]], [[2.0, 7.0, 
4.0], [5.0, 4.0, 4.0], [2.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [2.0, 4.0, 5.0], [3.0, 8.0, 2.0]], 
[[4.0, 5.0, 7.0], [4.0, 3.0, 5.0], [7.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [7.0, 3.0, 4.0], [6.0, 3.0, 6.0]]], [[[6.0, 8.0, 8.0], 
[6.0, 4.0, 5.0], [7.0, 5.0, 1.0]], [[2.0, 2.0, 2.0], [4.0, 7.0, 6.0], [2.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 8.0, 7.0], 
[3.0, 6.0, 4.0]]], [[[7.0, 6.0, 5.0], [2.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 6.0, 1.0], [4.0, 4.0, 7.0]], 
[[5.0, 5.0, 7.0], [8.0, 7.0, 7.0], [7.0, 5.0, 8.0]]], [[[3.0, 6.0, 6.0], [1.0, 2.0, 3.0], [7.0, 7.0, 5.0]], [[1.0, 5.0, 5.0], 
[2.0, 4.0, 5.0], [7.0, 1.0, 1.0]], [[2.0, 7.0, 4.0], [5.0, 4.0, 4.0], [2.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCRP(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 3.0], [6.0, 8.0, 7.0], [1.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [1.0, 2.0, 6.0], [2.0, 7.0, 
2.0]], [[1.0, 5.0, 3.0], [6.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[6.0, 4.0, 6.0], [6.0, 3.0, 5.0], [7.0, 1.0, 3.0]], [[1.0, 7.0, 
1.0], [8.0, 2.0, 2.0], [5.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [5.0, 1.0, 1.0], [5.0, 6.0, 2.0]]], [[[7.0, 4.0, 1.0], [3.0, 5.0, 
7.0], [4.0, 4.0, 1.0]], [[1.0, 4.0, 1.0], [2.0, 5.0, 5.0], [6.0, 8.0, 3.0]], [[7.0, 1.0, 5.0], [4.0, 5.0, 8.0], [3.0, 1.0, 
7.0]]], [[[1.0, 4.0, 5.0], [1.0, 6.0, 8.0], [3.0, 1.0, 6.0]], [[2.0, 5.0, 2.0], [8.0, 1.0, 3.0], [3.0, 2.0, 6.0]], [[8.0, 1.0, 
8.0], [5.0, 3.0, 5.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[4.0], [6.0], [6.0]], [[8.0], [8.0], [1.0]]])
      arg[1:3,:,1:2,1]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 3.0], [6.0, 8.0, 7.0], [1.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [1.0, 2.0, 6.0], [2.0, 7.0, 
2.0]], [[1.0, 5.0, 3.0], [6.0, 5.0, 8.0], [7.0, 5.0, 7.0]]], [[[6.0, 4.0, 6.0], [6.0, 4.0, 5.0], [7.0, 1.0, 3.0]], [[1.0, 7.0, 
1.0], [8.0, 6.0, 2.0], [5.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [5.0, 6.0, 1.0], [5.0, 6.0, 2.0]]], [[[7.0, 4.0, 1.0], [3.0, 8.0, 
7.0], [4.0, 4.0, 1.0]], [[1.0, 4.0, 1.0], [2.0, 8.0, 5.0], [6.0, 8.0, 3.0]], [[7.0, 1.0, 5.0], [4.0, 1.0, 8.0], [3.0, 1.0, 
7.0]]], [[[1.0, 4.0, 5.0], [1.0, 6.0, 8.0], [3.0, 1.0, 6.0]], [[2.0, 5.0, 2.0], [8.0, 1.0, 3.0], [3.0, 2.0, 6.0]], [[8.0, 1.0, 
8.0], [5.0, 3.0, 5.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCRP(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 6.0], [7.0, 2.0, 7.0], [8.0, 7.0, 5.0]], [[6.0, 8.0, 1.0], [3.0, 7.0, 1.0], [7.0, 8.0, 
7.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 7.0], [5.0, 4.0, 5.0]]], [[[1.0, 3.0, 8.0], [7.0, 5.0, 1.0], [3.0, 5.0, 4.0]], [[4.0, 4.0, 
8.0], [4.0, 7.0, 2.0], [1.0, 1.0, 7.0]], [[1.0, 4.0, 1.0], [4.0, 8.0, 6.0], [6.0, 6.0, 2.0]]], [[[6.0, 8.0, 5.0], [6.0, 8.0, 
4.0], [4.0, 7.0, 3.0]], [[2.0, 2.0, 2.0], [2.0, 3.0, 8.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 8.0], [3.0, 8.0, 4.0], [6.0, 5.0, 
4.0]]], [[[5.0, 7.0, 7.0], [6.0, 3.0, 8.0], [5.0, 8.0, 6.0]], [[8.0, 3.0, 5.0], [3.0, 3.0, 2.0], [5.0, 8.0, 5.0]], [[1.0, 7.0, 
7.0], [4.0, 3.0, 8.0], [1.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0], [6.0], [4.0]], [[6.0], [6.0], [4.0]]]),self.functionspace)
      arg[1:3,:,1:2,1]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 6.0], [7.0, 2.0, 7.0], [8.0, 7.0, 5.0]], [[6.0, 8.0, 1.0], [3.0, 7.0, 1.0], [7.0, 8.0, 
7.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 7.0], [5.0, 4.0, 5.0]]], [[[1.0, 3.0, 8.0], [7.0, 2.0, 1.0], [3.0, 5.0, 4.0]], [[4.0, 4.0, 
8.0], [4.0, 6.0, 2.0], [1.0, 1.0, 7.0]], [[1.0, 4.0, 1.0], [4.0, 4.0, 6.0], [6.0, 6.0, 2.0]]], [[[6.0, 8.0, 5.0], [6.0, 6.0, 
4.0], [4.0, 7.0, 3.0]], [[2.0, 2.0, 2.0], [2.0, 6.0, 8.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 8.0], [3.0, 4.0, 4.0], [6.0, 5.0, 
4.0]]], [[[5.0, 7.0, 7.0], [6.0, 3.0, 8.0], [5.0, 8.0, 6.0]], [[8.0, 3.0, 5.0], [3.0, 3.0, 2.0], [5.0, 8.0, 5.0]], [[1.0, 7.0, 
7.0], [4.0, 3.0, 8.0], [1.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCRP(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 3.0], [1.0, 5.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 5.0, 5.0], [7.0, 3.0, 4.0], [8.0, 5.0, 
2.0]], [[2.0, 2.0, 7.0], [7.0, 4.0, 8.0], [7.0, 1.0, 5.0]]], [[[7.0, 1.0, 6.0], [7.0, 5.0, 7.0], [8.0, 3.0, 3.0]], [[1.0, 2.0, 
3.0], [8.0, 3.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 7.0, 3.0], [3.0, 8.0, 5.0]]], [[[2.0, 8.0, 5.0], [4.0, 5.0, 
5.0], [8.0, 7.0, 4.0]], [[2.0, 6.0, 2.0], [4.0, 5.0, 7.0], [2.0, 6.0, 3.0]], [[5.0, 1.0, 3.0], [4.0, 2.0, 1.0], [1.0, 8.0, 
6.0]]], [[[3.0, 2.0, 3.0], [7.0, 8.0, 5.0], [2.0, 1.0, 6.0]], [[4.0, 7.0, 4.0], [6.0, 1.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 1.0, 
1.0], [4.0, 6.0, 1.0], [7.0, 3.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0], [3.0], [8.0]], [[5.0], [1.0], [7.0]]])+(1.-msk_val)*numpy.array([[[6.0], [6.0], [1.0]], 
[[3.0], [5.0], [4.0]]])
      arg[1:3,:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [1.0, 5.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 5.0, 5.0], [7.0, 3.0, 4.0], [8.0, 
5.0, 2.0]], [[2.0, 2.0, 7.0], [7.0, 4.0, 8.0], [7.0, 1.0, 5.0]]], [[[7.0, 1.0, 6.0], [7.0, 2.0, 7.0], [8.0, 3.0, 3.0]], [[1.0, 
2.0, 3.0], [8.0, 3.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 8.0, 3.0], [3.0, 8.0, 5.0]]], [[[2.0, 8.0, 5.0], [4.0, 
5.0, 5.0], [8.0, 7.0, 4.0]], [[2.0, 6.0, 2.0], [4.0, 1.0, 7.0], [2.0, 6.0, 3.0]], [[5.0, 1.0, 3.0], [4.0, 7.0, 1.0], [1.0, 8.0, 
6.0]]], [[[3.0, 2.0, 3.0], [7.0, 8.0, 5.0], [2.0, 1.0, 6.0]], [[4.0, 7.0, 4.0], [6.0, 1.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 1.0, 
1.0], [4.0, 6.0, 1.0], [7.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 3.0], [1.0, 5.0, 5.0], [4.0, 3.0, 6.0]], 
[[6.0, 5.0, 5.0], [7.0, 3.0, 4.0], [8.0, 5.0, 2.0]], [[2.0, 2.0, 7.0], [7.0, 4.0, 8.0], [7.0, 1.0, 5.0]]], [[[7.0, 1.0, 6.0], 
[7.0, 6.0, 7.0], [8.0, 3.0, 3.0]], [[1.0, 2.0, 3.0], [8.0, 6.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 1.0, 3.0], 
[3.0, 8.0, 5.0]]], [[[2.0, 8.0, 5.0], [4.0, 3.0, 5.0], [8.0, 7.0, 4.0]], [[2.0, 6.0, 2.0], [4.0, 5.0, 7.0], [2.0, 6.0, 3.0]], 
[[5.0, 1.0, 3.0], [4.0, 4.0, 1.0], [1.0, 8.0, 6.0]]], [[[3.0, 2.0, 3.0], [7.0, 8.0, 5.0], [2.0, 1.0, 6.0]], [[4.0, 7.0, 4.0], 
[6.0, 1.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 1.0, 1.0], [4.0, 6.0, 1.0], [7.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPUN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 7.0], [8.0, 2.0, 3.0], [1.0, 6.0, 6.0]], [[8.0, 5.0, 6.0], [3.0, 1.0, 5.0], [7.0, 6.0, 
6.0]], [[2.0, 7.0, 7.0], [1.0, 8.0, 1.0], [4.0, 1.0, 7.0]]], [[[6.0, 7.0, 8.0], [3.0, 6.0, 6.0], [5.0, 2.0, 1.0]], [[6.0, 1.0, 
7.0], [7.0, 1.0, 5.0], [1.0, 1.0, 5.0]], [[3.0, 2.0, 6.0], [4.0, 5.0, 7.0], [6.0, 3.0, 5.0]]], [[[4.0, 6.0, 8.0], [3.0, 6.0, 
5.0], [2.0, 2.0, 4.0]], [[2.0, 7.0, 8.0], [7.0, 5.0, 2.0], [5.0, 3.0, 8.0]], [[2.0, 3.0, 6.0], [1.0, 1.0, 8.0], [4.0, 4.0, 
4.0]]], [[[1.0, 7.0, 7.0], [7.0, 4.0, 1.0], [4.0, 1.0, 2.0]], [[2.0, 8.0, 4.0], [7.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[6.0, 4.0, 
5.0], [5.0, 8.0, 3.0], [7.0, 6.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 7.0, 6.0], [1.0, 1.0, 4.0]], [[2.0, 1.0, 1.0], [2.0, 8.0, 2.0]]])
      arg[1:3,1,:2]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 7.0], [8.0, 2.0, 3.0], [1.0, 6.0, 6.0]], [[8.0, 5.0, 6.0], [3.0, 1.0, 5.0], [7.0, 6.0, 
6.0]], [[2.0, 7.0, 7.0], [1.0, 8.0, 1.0], [4.0, 1.0, 7.0]]], [[[6.0, 7.0, 8.0], [3.0, 6.0, 6.0], [5.0, 2.0, 1.0]], [[1.0, 7.0, 
6.0], [1.0, 1.0, 4.0], [1.0, 1.0, 5.0]], [[3.0, 2.0, 6.0], [4.0, 5.0, 7.0], [6.0, 3.0, 5.0]]], [[[4.0, 6.0, 8.0], [3.0, 6.0, 
5.0], [2.0, 2.0, 4.0]], [[2.0, 1.0, 1.0], [2.0, 8.0, 2.0], [5.0, 3.0, 8.0]], [[2.0, 3.0, 6.0], [1.0, 1.0, 8.0], [4.0, 4.0, 
4.0]]], [[[1.0, 7.0, 7.0], [7.0, 4.0, 1.0], [4.0, 1.0, 2.0]], [[2.0, 8.0, 4.0], [7.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[6.0, 4.0, 
5.0], [5.0, 8.0, 3.0], [7.0, 6.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPUN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 7.0], [5.0, 3.0, 6.0], [1.0, 6.0, 1.0]], [[5.0, 4.0, 1.0], [7.0, 6.0, 7.0], [7.0, 4.0, 
8.0]], [[6.0, 6.0, 8.0], [2.0, 3.0, 3.0], [8.0, 5.0, 8.0]]], [[[7.0, 7.0, 1.0], [5.0, 6.0, 8.0], [2.0, 7.0, 7.0]], [[8.0, 3.0, 
7.0], [7.0, 7.0, 6.0], [7.0, 7.0, 6.0]], [[6.0, 2.0, 4.0], [8.0, 1.0, 4.0], [6.0, 4.0, 7.0]]], [[[2.0, 3.0, 1.0], [3.0, 6.0, 
1.0], [2.0, 2.0, 8.0]], [[7.0, 1.0, 5.0], [3.0, 4.0, 1.0], [8.0, 8.0, 6.0]], [[6.0, 3.0, 2.0], [3.0, 6.0, 4.0], [1.0, 1.0, 
5.0]]], [[[1.0, 1.0, 3.0], [6.0, 7.0, 2.0], [6.0, 6.0, 1.0]], [[5.0, 6.0, 5.0], [2.0, 7.0, 8.0], [4.0, 7.0, 4.0]], [[7.0, 4.0, 
6.0], [1.0, 7.0, 4.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 8.0, 4.0], [3.0, 4.0, 4.0]], [[2.0, 4.0, 7.0], [2.0, 4.0, 8.0]]]),self.functionspace)
      arg[1:3,1,:2]=val
      ref=Data(numpy.array([[[[7.0, 3.0, 7.0], [5.0, 3.0, 6.0], [1.0, 6.0, 1.0]], [[5.0, 4.0, 1.0], [7.0, 6.0, 7.0], [7.0, 4.0, 
8.0]], [[6.0, 6.0, 8.0], [2.0, 3.0, 3.0], [8.0, 5.0, 8.0]]], [[[7.0, 7.0, 1.0], [5.0, 6.0, 8.0], [2.0, 7.0, 7.0]], [[5.0, 8.0, 
4.0], [3.0, 4.0, 4.0], [7.0, 7.0, 6.0]], [[6.0, 2.0, 4.0], [8.0, 1.0, 4.0], [6.0, 4.0, 7.0]]], [[[2.0, 3.0, 1.0], [3.0, 6.0, 
1.0], [2.0, 2.0, 8.0]], [[2.0, 4.0, 7.0], [2.0, 4.0, 8.0], [8.0, 8.0, 6.0]], [[6.0, 3.0, 2.0], [3.0, 6.0, 4.0], [1.0, 1.0, 
5.0]]], [[[1.0, 1.0, 3.0], [6.0, 7.0, 2.0], [6.0, 6.0, 1.0]], [[5.0, 6.0, 5.0], [2.0, 7.0, 8.0], [4.0, 7.0, 4.0]], [[7.0, 4.0, 
6.0], [1.0, 7.0, 4.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPUN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 5.0], [6.0, 5.0, 8.0], [7.0, 8.0, 2.0]], [[1.0, 4.0, 4.0], [1.0, 4.0, 7.0], [1.0, 7.0, 
7.0]], [[4.0, 8.0, 2.0], [2.0, 2.0, 2.0], [1.0, 8.0, 7.0]]], [[[1.0, 4.0, 3.0], [4.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[7.0, 8.0, 
8.0], [7.0, 6.0, 5.0], [4.0, 7.0, 4.0]], [[6.0, 8.0, 7.0], [8.0, 2.0, 1.0], [3.0, 3.0, 7.0]]], [[[7.0, 4.0, 5.0], [8.0, 8.0, 
6.0], [5.0, 4.0, 4.0]], [[1.0, 7.0, 3.0], [3.0, 4.0, 3.0], [6.0, 2.0, 6.0]], [[5.0, 3.0, 8.0], [4.0, 6.0, 4.0], [3.0, 6.0, 
5.0]]], [[[1.0, 4.0, 1.0], [4.0, 5.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 8.0, 6.0], [2.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[3.0, 7.0, 
2.0], [1.0, 1.0, 1.0], [2.0, 1.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 7.0, 7.0], [8.0, 3.0, 6.0]], [[8.0, 1.0, 8.0], [1.0, 2.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[8.0, 2.0, 8.0], [1.0, 2.0, 6.0]], [[1.0, 7.0, 5.0], [8.0, 3.0, 8.0]]])
      arg[1:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 5.0], [6.0, 5.0, 8.0], [7.0, 8.0, 2.0]], [[1.0, 4.0, 4.0], [1.0, 4.0, 7.0], [1.0, 
7.0, 7.0]], [[4.0, 8.0, 2.0], [2.0, 2.0, 2.0], [1.0, 8.0, 7.0]]], [[[1.0, 4.0, 3.0], [4.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[5.0, 
7.0, 7.0], [8.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 8.0, 7.0], [8.0, 2.0, 1.0], [3.0, 3.0, 7.0]]], [[[7.0, 4.0, 5.0], [8.0, 
8.0, 6.0], [5.0, 4.0, 4.0]], [[8.0, 1.0, 8.0], [1.0, 2.0, 6.0], [6.0, 2.0, 6.0]], [[5.0, 3.0, 8.0], [4.0, 6.0, 4.0], [3.0, 6.0, 
5.0]]], [[[1.0, 4.0, 1.0], [4.0, 5.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 8.0, 6.0], [2.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[3.0, 7.0, 
2.0], [1.0, 1.0, 1.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [6.0, 5.0, 8.0], [7.0, 8.0, 2.0]], 
[[1.0, 4.0, 4.0], [1.0, 4.0, 7.0], [1.0, 7.0, 7.0]], [[4.0, 8.0, 2.0], [2.0, 2.0, 2.0], [1.0, 8.0, 7.0]]], [[[1.0, 4.0, 3.0], 
[4.0, 3.0, 1.0], [8.0, 2.0, 1.0]], [[8.0, 2.0, 8.0], [1.0, 2.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 8.0, 7.0], [8.0, 2.0, 1.0], 
[3.0, 3.0, 7.0]]], [[[7.0, 4.0, 5.0], [8.0, 8.0, 6.0], [5.0, 4.0, 4.0]], [[1.0, 7.0, 5.0], [8.0, 3.0, 8.0], [6.0, 2.0, 6.0]], 
[[5.0, 3.0, 8.0], [4.0, 6.0, 4.0], [3.0, 6.0, 5.0]]], [[[1.0, 4.0, 1.0], [4.0, 5.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 8.0, 6.0], 
[2.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[3.0, 7.0, 2.0], [1.0, 1.0, 1.0], [2.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLLL(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [1.0, 1.0, 7.0], [3.0, 5.0, 4.0]], [[4.0, 8.0, 5.0], [6.0, 3.0, 5.0], [8.0, 7.0, 
3.0]], [[2.0, 5.0, 7.0], [6.0, 8.0, 3.0], [7.0, 5.0, 4.0]]], [[[7.0, 1.0, 4.0], [1.0, 2.0, 8.0], [7.0, 7.0, 1.0]], [[5.0, 1.0, 
7.0], [1.0, 5.0, 1.0], [2.0, 5.0, 7.0]], [[1.0, 6.0, 6.0], [7.0, 7.0, 2.0], [5.0, 7.0, 3.0]]], [[[4.0, 6.0, 8.0], [8.0, 2.0, 
5.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 2.0], [3.0, 8.0, 8.0], [8.0, 4.0, 4.0]], [[6.0, 2.0, 6.0], [6.0, 1.0, 2.0], [7.0, 7.0, 
5.0]]], [[[7.0, 7.0, 8.0], [6.0, 4.0, 6.0], [8.0, 7.0, 2.0]], [[8.0, 4.0, 3.0], [4.0, 5.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 7.0, 
1.0], [4.0, 7.0, 5.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0]]], [[[4.0]]]])
      arg[1:3,2:,2:,2:]=val
      ref=Data(numpy.array([[[[7.0, 7.0, 5.0], [1.0, 1.0, 7.0], [3.0, 5.0, 4.0]], [[4.0, 8.0, 5.0], [6.0, 3.0, 5.0], [8.0, 7.0, 
3.0]], [[2.0, 5.0, 7.0], [6.0, 8.0, 3.0], [7.0, 5.0, 4.0]]], [[[7.0, 1.0, 4.0], [1.0, 2.0, 8.0], [7.0, 7.0, 1.0]], [[5.0, 1.0, 
7.0], [1.0, 5.0, 1.0], [2.0, 5.0, 7.0]], [[1.0, 6.0, 6.0], [7.0, 7.0, 2.0], [5.0, 7.0, 3.0]]], [[[4.0, 6.0, 8.0], [8.0, 2.0, 
5.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 2.0], [3.0, 8.0, 8.0], [8.0, 4.0, 4.0]], [[6.0, 2.0, 6.0], [6.0, 1.0, 2.0], [7.0, 7.0, 
4.0]]], [[[7.0, 7.0, 8.0], [6.0, 4.0, 6.0], [8.0, 7.0, 2.0]], [[8.0, 4.0, 3.0], [4.0, 5.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 7.0, 
1.0], [4.0, 7.0, 5.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLLL(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [6.0, 3.0, 5.0], [5.0, 5.0, 1.0]], [[6.0, 8.0, 2.0], [4.0, 6.0, 6.0], [2.0, 3.0, 
1.0]], [[7.0, 5.0, 3.0], [7.0, 5.0, 5.0], [4.0, 7.0, 8.0]]], [[[8.0, 2.0, 4.0], [2.0, 2.0, 2.0], [3.0, 8.0, 6.0]], [[3.0, 8.0, 
7.0], [4.0, 1.0, 6.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 7.0], [6.0, 6.0, 4.0], [7.0, 1.0, 8.0]]], [[[3.0, 8.0, 2.0], [8.0, 6.0, 
8.0], [5.0, 8.0, 8.0]], [[7.0, 8.0, 2.0], [5.0, 1.0, 5.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 8.0, 5.0], [6.0, 5.0, 
4.0]]], [[[1.0, 2.0, 5.0], [1.0, 1.0, 7.0], [5.0, 3.0, 7.0]], [[6.0, 2.0, 3.0], [2.0, 4.0, 5.0], [5.0, 2.0, 4.0]], [[6.0, 6.0, 
1.0], [6.0, 2.0, 2.0], [1.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0]]], [[[5.0]]]]),self.functionspace)
      arg[1:3,2:,2:,2:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 3.0], [6.0, 3.0, 5.0], [5.0, 5.0, 1.0]], [[6.0, 8.0, 2.0], [4.0, 6.0, 6.0], [2.0, 3.0, 
1.0]], [[7.0, 5.0, 3.0], [7.0, 5.0, 5.0], [4.0, 7.0, 8.0]]], [[[8.0, 2.0, 4.0], [2.0, 2.0, 2.0], [3.0, 8.0, 6.0]], [[3.0, 8.0, 
7.0], [4.0, 1.0, 6.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 7.0], [6.0, 6.0, 4.0], [7.0, 1.0, 1.0]]], [[[3.0, 8.0, 2.0], [8.0, 6.0, 
8.0], [5.0, 8.0, 8.0]], [[7.0, 8.0, 2.0], [5.0, 1.0, 5.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 6.0], [5.0, 8.0, 5.0], [6.0, 5.0, 
5.0]]], [[[1.0, 2.0, 5.0], [1.0, 1.0, 7.0], [5.0, 3.0, 7.0]], [[6.0, 2.0, 3.0], [2.0, 4.0, 5.0], [5.0, 2.0, 4.0]], [[6.0, 6.0, 
1.0], [6.0, 2.0, 2.0], [1.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLLL(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 5.0], [4.0, 7.0, 1.0], [6.0, 4.0, 5.0]], [[1.0, 2.0, 3.0], [1.0, 5.0, 6.0], [3.0, 7.0, 
5.0]], [[6.0, 3.0, 8.0], [8.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[8.0, 4.0, 6.0], [1.0, 3.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 2.0, 
8.0], [5.0, 3.0, 3.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 6.0], [8.0, 4.0, 5.0], [4.0, 7.0, 4.0]]], [[[7.0, 1.0, 6.0], [6.0, 5.0, 
3.0], [3.0, 6.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 5.0, 1.0], [1.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [4.0, 6.0, 3.0], [7.0, 3.0, 
6.0]]], [[[2.0, 6.0, 6.0], [1.0, 1.0, 3.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 6.0], [1.0, 2.0, 6.0], [8.0, 5.0, 6.0]], [[4.0, 2.0, 
3.0], [6.0, 6.0, 1.0], [3.0, 1.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[8.0]]], [[[5.0]]]])
      arg[1:3,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 5.0], [4.0, 7.0, 1.0], [6.0, 4.0, 5.0]], [[1.0, 2.0, 3.0], [1.0, 5.0, 6.0], [3.0, 
7.0, 5.0]], [[6.0, 3.0, 8.0], [8.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[8.0, 4.0, 6.0], [1.0, 3.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 
2.0, 8.0], [5.0, 3.0, 3.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 6.0], [8.0, 4.0, 5.0], [4.0, 7.0, 7.0]]], [[[7.0, 1.0, 6.0], [6.0, 
5.0, 3.0], [3.0, 6.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 5.0, 1.0], [1.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [4.0, 6.0, 3.0], [7.0, 3.0, 
2.0]]], [[[2.0, 6.0, 6.0], [1.0, 1.0, 3.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 6.0], [1.0, 2.0, 6.0], [8.0, 5.0, 6.0]], [[4.0, 2.0, 
3.0], [6.0, 6.0, 1.0], [3.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 5.0], [4.0, 7.0, 1.0], [6.0, 4.0, 5.0]], 
[[1.0, 2.0, 3.0], [1.0, 5.0, 6.0], [3.0, 7.0, 5.0]], [[6.0, 3.0, 8.0], [8.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[8.0, 4.0, 6.0], 
[1.0, 3.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 2.0, 8.0], [5.0, 3.0, 3.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 6.0], [8.0, 4.0, 5.0], 
[4.0, 7.0, 8.0]]], [[[7.0, 1.0, 6.0], [6.0, 5.0, 3.0], [3.0, 6.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 5.0, 1.0], [1.0, 3.0, 7.0]], 
[[5.0, 8.0, 2.0], [4.0, 6.0, 3.0], [7.0, 3.0, 5.0]]], [[[2.0, 6.0, 6.0], [1.0, 1.0, 3.0], [1.0, 1.0, 7.0]], [[4.0, 7.0, 6.0], 
[1.0, 2.0, 6.0], [8.0, 5.0, 6.0]], [[4.0, 2.0, 3.0], [6.0, 6.0, 1.0], [3.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLPN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 8.0], [5.0, 8.0, 5.0], [4.0, 2.0, 3.0]], [[5.0, 4.0, 1.0], [2.0, 2.0, 6.0], [1.0, 8.0, 
5.0]], [[1.0, 2.0, 3.0], [5.0, 8.0, 5.0], [3.0, 5.0, 8.0]]], [[[2.0, 8.0, 8.0], [1.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[2.0, 3.0, 
1.0], [6.0, 8.0, 8.0], [8.0, 7.0, 6.0]], [[8.0, 7.0, 1.0], [4.0, 7.0, 7.0], [8.0, 1.0, 1.0]]], [[[3.0, 4.0, 4.0], [8.0, 2.0, 
6.0], [5.0, 8.0, 6.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [3.0, 2.0, 2.0], [3.0, 5.0, 
1.0]]], [[[7.0, 2.0, 3.0], [2.0, 4.0, 3.0], [4.0, 7.0, 4.0]], [[5.0, 8.0, 5.0], [5.0, 5.0, 8.0], [7.0, 6.0, 2.0]], [[6.0, 4.0, 
8.0], [4.0, 4.0, 8.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[8.0, 3.0, 6.0]], [[4.0, 1.0, 3.0]]])
      arg[1:3,2:,1]=val
      ref=Data(numpy.array([[[[7.0, 4.0, 8.0], [5.0, 8.0, 5.0], [4.0, 2.0, 3.0]], [[5.0, 4.0, 1.0], [2.0, 2.0, 6.0], [1.0, 8.0, 
5.0]], [[1.0, 2.0, 3.0], [5.0, 8.0, 5.0], [3.0, 5.0, 8.0]]], [[[2.0, 8.0, 8.0], [1.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[2.0, 3.0, 
1.0], [6.0, 8.0, 8.0], [8.0, 7.0, 6.0]], [[8.0, 7.0, 1.0], [8.0, 3.0, 6.0], [8.0, 1.0, 1.0]]], [[[3.0, 4.0, 4.0], [8.0, 2.0, 
6.0], [5.0, 8.0, 6.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [4.0, 1.0, 3.0], [3.0, 5.0, 
1.0]]], [[[7.0, 2.0, 3.0], [2.0, 4.0, 3.0], [4.0, 7.0, 4.0]], [[5.0, 8.0, 5.0], [5.0, 5.0, 8.0], [7.0, 6.0, 2.0]], [[6.0, 4.0, 
8.0], [4.0, 4.0, 8.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLPN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 4.0], [1.0, 7.0, 5.0], [4.0, 7.0, 1.0]], [[3.0, 1.0, 2.0], [5.0, 3.0, 5.0], [1.0, 3.0, 
3.0]], [[1.0, 4.0, 2.0], [8.0, 8.0, 1.0], [1.0, 6.0, 6.0]]], [[[7.0, 7.0, 1.0], [8.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[1.0, 7.0, 
2.0], [4.0, 7.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 7.0, 2.0], [4.0, 2.0, 3.0], [2.0, 6.0, 6.0]]], [[[1.0, 2.0, 5.0], [6.0, 1.0, 
2.0], [8.0, 6.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 8.0, 4.0], [6.0, 2.0, 7.0]], [[8.0, 5.0, 2.0], [6.0, 6.0, 2.0], [6.0, 8.0, 
7.0]]], [[[2.0, 1.0, 6.0], [6.0, 1.0, 1.0], [1.0, 3.0, 4.0]], [[1.0, 8.0, 7.0], [6.0, 1.0, 5.0], [7.0, 5.0, 7.0]], [[1.0, 5.0, 
5.0], [4.0, 1.0, 2.0], [6.0, 5.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 2.0, 4.0]], [[2.0, 8.0, 7.0]]]),self.functionspace)
      arg[1:3,2:,1]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 4.0], [1.0, 7.0, 5.0], [4.0, 7.0, 1.0]], [[3.0, 1.0, 2.0], [5.0, 3.0, 5.0], [1.0, 3.0, 
3.0]], [[1.0, 4.0, 2.0], [8.0, 8.0, 1.0], [1.0, 6.0, 6.0]]], [[[7.0, 7.0, 1.0], [8.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[1.0, 7.0, 
2.0], [4.0, 7.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 7.0, 2.0], [3.0, 2.0, 4.0], [2.0, 6.0, 6.0]]], [[[1.0, 2.0, 5.0], [6.0, 1.0, 
2.0], [8.0, 6.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 8.0, 4.0], [6.0, 2.0, 7.0]], [[8.0, 5.0, 2.0], [2.0, 8.0, 7.0], [6.0, 8.0, 
7.0]]], [[[2.0, 1.0, 6.0], [6.0, 1.0, 1.0], [1.0, 3.0, 4.0]], [[1.0, 8.0, 7.0], [6.0, 1.0, 5.0], [7.0, 5.0, 7.0]], [[1.0, 5.0, 
5.0], [4.0, 1.0, 2.0], [6.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLPN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 2.0], [8.0, 3.0, 4.0], [8.0, 3.0, 6.0]], [[8.0, 4.0, 2.0], [4.0, 5.0, 1.0], [8.0, 1.0, 
1.0]], [[7.0, 7.0, 8.0], [2.0, 7.0, 1.0], [4.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], [6.0, 6.0, 4.0], [3.0, 7.0, 3.0]], [[2.0, 6.0, 
8.0], [5.0, 6.0, 8.0], [1.0, 6.0, 3.0]], [[4.0, 6.0, 8.0], [8.0, 1.0, 2.0], [2.0, 7.0, 3.0]]], [[[7.0, 6.0, 2.0], [8.0, 7.0, 
3.0], [1.0, 7.0, 1.0]], [[3.0, 7.0, 1.0], [2.0, 6.0, 4.0], [1.0, 2.0, 3.0]], [[8.0, 2.0, 5.0], [7.0, 7.0, 3.0], [6.0, 8.0, 
3.0]]], [[[4.0, 2.0, 2.0], [4.0, 3.0, 8.0], [2.0, 8.0, 7.0]], [[2.0, 6.0, 1.0], [3.0, 2.0, 2.0], [4.0, 4.0, 5.0]], [[7.0, 6.0, 
4.0], [6.0, 3.0, 8.0], [4.0, 7.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 8.0, 1.0]], [[3.0, 4.0, 1.0]]])+(1.-msk_val)*numpy.array([[[6.0, 8.0, 1.0]], [[6.0, 6.0, 
2.0]]])
      arg[1:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 2.0], [8.0, 3.0, 4.0], [8.0, 3.0, 6.0]], [[8.0, 4.0, 2.0], [4.0, 5.0, 1.0], [8.0, 
1.0, 1.0]], [[7.0, 7.0, 8.0], [2.0, 7.0, 1.0], [4.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], [6.0, 6.0, 4.0], [3.0, 7.0, 3.0]], [[2.0, 
6.0, 8.0], [5.0, 6.0, 8.0], [1.0, 6.0, 3.0]], [[4.0, 6.0, 8.0], [5.0, 8.0, 1.0], [2.0, 7.0, 3.0]]], [[[7.0, 6.0, 2.0], [8.0, 
7.0, 3.0], [1.0, 7.0, 1.0]], [[3.0, 7.0, 1.0], [2.0, 6.0, 4.0], [1.0, 2.0, 3.0]], [[8.0, 2.0, 5.0], [3.0, 4.0, 1.0], [6.0, 8.0, 
3.0]]], [[[4.0, 2.0, 2.0], [4.0, 3.0, 8.0], [2.0, 8.0, 7.0]], [[2.0, 6.0, 1.0], [3.0, 2.0, 2.0], [4.0, 4.0, 5.0]], [[7.0, 6.0, 
4.0], [6.0, 3.0, 8.0], [4.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 2.0], [8.0, 3.0, 4.0], [8.0, 3.0, 6.0]], 
[[8.0, 4.0, 2.0], [4.0, 5.0, 1.0], [8.0, 1.0, 1.0]], [[7.0, 7.0, 8.0], [2.0, 7.0, 1.0], [4.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], 
[6.0, 6.0, 4.0], [3.0, 7.0, 3.0]], [[2.0, 6.0, 8.0], [5.0, 6.0, 8.0], [1.0, 6.0, 3.0]], [[4.0, 6.0, 8.0], [6.0, 8.0, 1.0], 
[2.0, 7.0, 3.0]]], [[[7.0, 6.0, 2.0], [8.0, 7.0, 3.0], [1.0, 7.0, 1.0]], [[3.0, 7.0, 1.0], [2.0, 6.0, 4.0], [1.0, 2.0, 3.0]], 
[[8.0, 2.0, 5.0], [6.0, 6.0, 2.0], [6.0, 8.0, 3.0]]], [[[4.0, 2.0, 2.0], [4.0, 3.0, 8.0], [2.0, 8.0, 7.0]], [[2.0, 6.0, 1.0], 
[3.0, 2.0, 2.0], [4.0, 4.0, 5.0]], [[7.0, 6.0, 4.0], [6.0, 3.0, 8.0], [4.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LULU(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 4.0], [1.0, 8.0, 7.0], [7.0, 3.0, 3.0]], [[8.0, 2.0, 1.0], [2.0, 8.0, 4.0], [3.0, 1.0, 
8.0]], [[3.0, 3.0, 7.0], [2.0, 8.0, 7.0], [7.0, 3.0, 6.0]]], [[[4.0, 4.0, 4.0], [2.0, 6.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 8.0, 
3.0], [6.0, 7.0, 6.0], [4.0, 2.0, 1.0]], [[6.0, 8.0, 1.0], [5.0, 8.0, 1.0], [3.0, 3.0, 5.0]]], [[[6.0, 2.0, 7.0], [7.0, 7.0, 
6.0], [7.0, 6.0, 1.0]], [[6.0, 1.0, 4.0], [3.0, 5.0, 8.0], [1.0, 2.0, 3.0]], [[1.0, 6.0, 5.0], [2.0, 7.0, 2.0], [4.0, 4.0, 
8.0]]], [[[5.0, 7.0, 7.0], [5.0, 7.0, 3.0], [8.0, 6.0, 4.0]], [[1.0, 4.0, 7.0], [6.0, 2.0, 6.0], [4.0, 2.0, 2.0]], [[6.0, 1.0, 
2.0], [8.0, 4.0, 1.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 7.0]], [[3.0, 3.0]]], [[[3.0, 2.0]], [[2.0, 7.0]]]])
      arg[2:,:2,2:,:2]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 4.0], [1.0, 8.0, 7.0], [7.0, 3.0, 3.0]], [[8.0, 2.0, 1.0], [2.0, 8.0, 4.0], [3.0, 1.0, 
8.0]], [[3.0, 3.0, 7.0], [2.0, 8.0, 7.0], [7.0, 3.0, 6.0]]], [[[4.0, 4.0, 4.0], [2.0, 6.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 8.0, 
3.0], [6.0, 7.0, 6.0], [4.0, 2.0, 1.0]], [[6.0, 8.0, 1.0], [5.0, 8.0, 1.0], [3.0, 3.0, 5.0]]], [[[6.0, 2.0, 7.0], [7.0, 7.0, 
6.0], [7.0, 7.0, 1.0]], [[6.0, 1.0, 4.0], [3.0, 5.0, 8.0], [3.0, 3.0, 3.0]], [[1.0, 6.0, 5.0], [2.0, 7.0, 2.0], [4.0, 4.0, 
8.0]]], [[[5.0, 7.0, 7.0], [5.0, 7.0, 3.0], [3.0, 2.0, 4.0]], [[1.0, 4.0, 7.0], [6.0, 2.0, 6.0], [2.0, 7.0, 2.0]], [[6.0, 1.0, 
2.0], [8.0, 4.0, 1.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LULU(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 7.0], [6.0, 3.0, 1.0], [1.0, 5.0, 4.0]], [[2.0, 8.0, 8.0], [6.0, 8.0, 8.0], [5.0, 7.0, 
2.0]], [[5.0, 8.0, 5.0], [5.0, 7.0, 7.0], [3.0, 7.0, 2.0]]], [[[3.0, 2.0, 5.0], [3.0, 2.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 5.0, 
6.0], [6.0, 1.0, 1.0], [8.0, 6.0, 8.0]], [[8.0, 3.0, 4.0], [1.0, 5.0, 2.0], [1.0, 1.0, 4.0]]], [[[7.0, 5.0, 4.0], [1.0, 5.0, 
7.0], [3.0, 4.0, 4.0]], [[7.0, 2.0, 5.0], [1.0, 4.0, 7.0], [1.0, 6.0, 5.0]], [[1.0, 5.0, 5.0], [7.0, 7.0, 2.0], [1.0, 4.0, 
6.0]]], [[[6.0, 2.0, 3.0], [7.0, 2.0, 5.0], [4.0, 2.0, 7.0]], [[1.0, 1.0, 1.0], [8.0, 6.0, 3.0], [6.0, 1.0, 7.0]], [[2.0, 6.0, 
2.0], [1.0, 2.0, 1.0], [2.0, 5.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 5.0]], [[8.0, 8.0]]], [[[3.0, 3.0]], [[4.0, 5.0]]]]),self.functionspace)
      arg[2:,:2,2:,:2]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 7.0], [6.0, 3.0, 1.0], [1.0, 5.0, 4.0]], [[2.0, 8.0, 8.0], [6.0, 8.0, 8.0], [5.0, 7.0, 
2.0]], [[5.0, 8.0, 5.0], [5.0, 7.0, 7.0], [3.0, 7.0, 2.0]]], [[[3.0, 2.0, 5.0], [3.0, 2.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 5.0, 
6.0], [6.0, 1.0, 1.0], [8.0, 6.0, 8.0]], [[8.0, 3.0, 4.0], [1.0, 5.0, 2.0], [1.0, 1.0, 4.0]]], [[[7.0, 5.0, 4.0], [1.0, 5.0, 
7.0], [8.0, 5.0, 4.0]], [[7.0, 2.0, 5.0], [1.0, 4.0, 7.0], [8.0, 8.0, 5.0]], [[1.0, 5.0, 5.0], [7.0, 7.0, 2.0], [1.0, 4.0, 
6.0]]], [[[6.0, 2.0, 3.0], [7.0, 2.0, 5.0], [3.0, 3.0, 7.0]], [[1.0, 1.0, 1.0], [8.0, 6.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 6.0, 
2.0], [1.0, 2.0, 1.0], [2.0, 5.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LULU(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 7.0], [8.0, 8.0, 8.0], [7.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 7.0, 5.0], [5.0, 1.0, 
7.0]], [[3.0, 3.0, 4.0], [1.0, 4.0, 4.0], [5.0, 2.0, 2.0]]], [[[1.0, 6.0, 1.0], [7.0, 7.0, 2.0], [6.0, 3.0, 3.0]], [[8.0, 7.0, 
6.0], [6.0, 7.0, 7.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 8.0], [3.0, 2.0, 2.0]]], [[[3.0, 3.0, 8.0], [1.0, 8.0, 
5.0], [8.0, 8.0, 7.0]], [[3.0, 1.0, 2.0], [5.0, 1.0, 7.0], [6.0, 1.0, 4.0]], [[6.0, 3.0, 5.0], [8.0, 1.0, 2.0], [3.0, 8.0, 
4.0]]], [[[4.0, 5.0, 2.0], [2.0, 5.0, 3.0], [6.0, 6.0, 6.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 5.0], [6.0, 4.0, 2.0]], [[2.0, 2.0, 
3.0], [8.0, 6.0, 1.0], [5.0, 3.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 8.0]], [[3.0, 8.0]]], [[[8.0, 7.0]], [[3.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 
1.0]], [[5.0, 8.0]]], [[[3.0, 5.0]], [[2.0, 6.0]]]])
      arg[2:,:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 7.0], [8.0, 8.0, 8.0], [7.0, 3.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 7.0, 5.0], [5.0, 
1.0, 7.0]], [[3.0, 3.0, 4.0], [1.0, 4.0, 4.0], [5.0, 2.0, 2.0]]], [[[1.0, 6.0, 1.0], [7.0, 7.0, 2.0], [6.0, 3.0, 3.0]], [[8.0, 
7.0, 6.0], [6.0, 7.0, 7.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 8.0], [3.0, 2.0, 2.0]]], [[[3.0, 3.0, 8.0], [1.0, 
8.0, 5.0], [2.0, 8.0, 7.0]], [[3.0, 1.0, 2.0], [5.0, 1.0, 7.0], [3.0, 8.0, 4.0]], [[6.0, 3.0, 5.0], [8.0, 1.0, 2.0], [3.0, 8.0, 
4.0]]], [[[4.0, 5.0, 2.0], [2.0, 5.0, 3.0], [8.0, 7.0, 6.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 5.0], [3.0, 1.0, 2.0]], [[2.0, 2.0, 
3.0], [8.0, 6.0, 1.0], [5.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 7.0], [8.0, 8.0, 8.0], [7.0, 3.0, 2.0]], 
[[6.0, 2.0, 8.0], [6.0, 7.0, 5.0], [5.0, 1.0, 7.0]], [[3.0, 3.0, 4.0], [1.0, 4.0, 4.0], [5.0, 2.0, 2.0]]], [[[1.0, 6.0, 1.0], 
[7.0, 7.0, 2.0], [6.0, 3.0, 3.0]], [[8.0, 7.0, 6.0], [6.0, 7.0, 7.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 3.0], [8.0, 4.0, 8.0], 
[3.0, 2.0, 2.0]]], [[[3.0, 3.0, 8.0], [1.0, 8.0, 5.0], [5.0, 1.0, 7.0]], [[3.0, 1.0, 2.0], [5.0, 1.0, 7.0], [5.0, 8.0, 4.0]], 
[[6.0, 3.0, 5.0], [8.0, 1.0, 2.0], [3.0, 8.0, 4.0]]], [[[4.0, 5.0, 2.0], [2.0, 5.0, 3.0], [3.0, 5.0, 6.0]], [[2.0, 5.0, 2.0], 
[5.0, 6.0, 5.0], [2.0, 6.0, 2.0]], [[2.0, 2.0, 3.0], [8.0, 6.0, 1.0], [5.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UUUC(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 3.0], [7.0, 5.0, 7.0], [3.0, 4.0, 3.0]], [[4.0, 7.0, 5.0], [8.0, 4.0, 1.0], [1.0, 2.0, 
7.0]], [[6.0, 7.0, 5.0], [8.0, 4.0, 3.0], [6.0, 4.0, 4.0]]], [[[6.0, 8.0, 8.0], [6.0, 7.0, 2.0], [8.0, 3.0, 6.0]], [[1.0, 2.0, 
5.0], [4.0, 2.0, 5.0], [6.0, 7.0, 7.0]], [[7.0, 3.0, 4.0], [2.0, 6.0, 4.0], [7.0, 4.0, 5.0]]], [[[4.0, 8.0, 3.0], [1.0, 5.0, 
1.0], [5.0, 4.0, 3.0]], [[5.0, 1.0, 2.0], [1.0, 4.0, 8.0], [8.0, 3.0, 4.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 5.0], [5.0, 2.0, 
7.0]]], [[[3.0, 3.0, 1.0], [7.0, 1.0, 8.0], [2.0, 3.0, 8.0]], [[6.0, 4.0, 4.0], [4.0, 3.0, 7.0], [2.0, 3.0, 5.0]], [[6.0, 1.0, 
8.0], [7.0, 2.0, 7.0], [7.0, 2.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 8.0, 7.0], [2.0, 1.0, 8.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 3.0]]], [[[6.0, 6.0, 7.0], [4.0, 3.0, 
1.0]], [[2.0, 1.0, 4.0], [5.0, 4.0, 1.0]]], [[[8.0, 1.0, 7.0], [6.0, 3.0, 4.0]], [[8.0, 3.0, 4.0], [8.0, 4.0, 5.0]]]])
      arg[:3,:2,:2,:]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 7.0], [2.0, 1.0, 8.0], [3.0, 4.0, 3.0]], [[6.0, 5.0, 3.0], [4.0, 3.0, 3.0], [1.0, 2.0, 
7.0]], [[6.0, 7.0, 5.0], [8.0, 4.0, 3.0], [6.0, 4.0, 4.0]]], [[[6.0, 6.0, 7.0], [4.0, 3.0, 1.0], [8.0, 3.0, 6.0]], [[2.0, 1.0, 
4.0], [5.0, 4.0, 1.0], [6.0, 7.0, 7.0]], [[7.0, 3.0, 4.0], [2.0, 6.0, 4.0], [7.0, 4.0, 5.0]]], [[[8.0, 1.0, 7.0], [6.0, 3.0, 
4.0], [5.0, 4.0, 3.0]], [[8.0, 3.0, 4.0], [8.0, 4.0, 5.0], [8.0, 3.0, 4.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 5.0], [5.0, 2.0, 
7.0]]], [[[3.0, 3.0, 1.0], [7.0, 1.0, 8.0], [2.0, 3.0, 8.0]], [[6.0, 4.0, 4.0], [4.0, 3.0, 7.0], [2.0, 3.0, 5.0]], [[6.0, 1.0, 
8.0], [7.0, 2.0, 7.0], [7.0, 2.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UUUC(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [5.0, 3.0, 4.0], [1.0, 3.0, 5.0]], [[6.0, 6.0, 3.0], [4.0, 5.0, 7.0], [2.0, 2.0, 
4.0]], [[3.0, 6.0, 4.0], [5.0, 5.0, 8.0], [7.0, 2.0, 3.0]]], [[[4.0, 2.0, 7.0], [2.0, 5.0, 6.0], [4.0, 4.0, 3.0]], [[7.0, 5.0, 
3.0], [4.0, 7.0, 5.0], [1.0, 4.0, 4.0]], [[5.0, 8.0, 2.0], [7.0, 1.0, 1.0], [1.0, 3.0, 4.0]]], [[[7.0, 2.0, 6.0], [2.0, 2.0, 
7.0], [3.0, 2.0, 5.0]], [[6.0, 1.0, 4.0], [6.0, 2.0, 8.0], [4.0, 1.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 6.0, 6.0], [5.0, 6.0, 
7.0]]], [[[1.0, 5.0, 6.0], [5.0, 7.0, 8.0], [5.0, 1.0, 4.0]], [[4.0, 8.0, 2.0], [1.0, 2.0, 3.0], [7.0, 4.0, 5.0]], [[3.0, 5.0, 
6.0], [4.0, 6.0, 6.0], [2.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0, 2.0], [5.0, 4.0, 8.0]], [[7.0, 6.0, 5.0], [4.0, 7.0, 5.0]]], [[[5.0, 2.0, 1.0], [3.0, 
2.0, 3.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 5.0]]], [[[7.0, 4.0, 7.0], [7.0, 7.0, 3.0]], [[4.0, 1.0, 4.0], [7.0, 8.0, 
7.0]]]]),self.functionspace)
      arg[:3,:2,:2,:]=val
      ref=Data(numpy.array([[[[5.0, 1.0, 2.0], [5.0, 4.0, 8.0], [1.0, 3.0, 5.0]], [[7.0, 6.0, 5.0], [4.0, 7.0, 5.0], [2.0, 2.0, 
4.0]], [[3.0, 6.0, 4.0], [5.0, 5.0, 8.0], [7.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [3.0, 2.0, 3.0], [4.0, 4.0, 3.0]], [[6.0, 4.0, 
1.0], [5.0, 7.0, 5.0], [1.0, 4.0, 4.0]], [[5.0, 8.0, 2.0], [7.0, 1.0, 1.0], [1.0, 3.0, 4.0]]], [[[7.0, 4.0, 7.0], [7.0, 7.0, 
3.0], [3.0, 2.0, 5.0]], [[4.0, 1.0, 4.0], [7.0, 8.0, 7.0], [4.0, 1.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 6.0, 6.0], [5.0, 6.0, 
7.0]]], [[[1.0, 5.0, 6.0], [5.0, 7.0, 8.0], [5.0, 1.0, 4.0]], [[4.0, 8.0, 2.0], [1.0, 2.0, 3.0], [7.0, 4.0, 5.0]], [[3.0, 5.0, 
6.0], [4.0, 6.0, 6.0], [2.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UUUC(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 2.0], [2.0, 3.0, 4.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 6.0], [3.0, 8.0, 1.0], [7.0, 5.0, 
1.0]], [[8.0, 8.0, 2.0], [8.0, 1.0, 3.0], [1.0, 8.0, 8.0]]], [[[8.0, 1.0, 2.0], [5.0, 4.0, 6.0], [4.0, 3.0, 3.0]], [[6.0, 1.0, 
4.0], [5.0, 6.0, 4.0], [4.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [7.0, 7.0, 8.0], [8.0, 4.0, 6.0]]], [[[5.0, 3.0, 4.0], [6.0, 5.0, 
2.0], [7.0, 7.0, 4.0]], [[6.0, 1.0, 1.0], [5.0, 2.0, 7.0], [2.0, 4.0, 4.0]], [[8.0, 5.0, 8.0], [1.0, 3.0, 5.0], [4.0, 3.0, 
6.0]]], [[[8.0, 1.0, 6.0], [6.0, 8.0, 6.0], [2.0, 6.0, 5.0]], [[6.0, 6.0, 5.0], [4.0, 2.0, 3.0], [2.0, 7.0, 7.0]], [[6.0, 5.0, 
3.0], [6.0, 3.0, 3.0], [8.0, 7.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 1.0, 6.0], [4.0, 4.0, 4.0]], [[3.0, 8.0, 8.0], [8.0, 6.0, 7.0]]], [[[1.0, 4.0, 3.0], 
[7.0, 7.0, 8.0]], [[2.0, 1.0, 1.0], [3.0, 4.0, 4.0]]], [[[1.0, 5.0, 3.0], [2.0, 6.0, 1.0]], [[4.0, 8.0, 5.0], [5.0, 8.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 8.0], [4.0, 5.0, 8.0]], [[3.0, 5.0, 6.0], [3.0, 7.0, 7.0]]], [[[3.0, 4.0, 4.0], 
[4.0, 3.0, 3.0]], [[8.0, 4.0, 5.0], [7.0, 5.0, 6.0]]], [[[7.0, 4.0, 1.0], [5.0, 3.0, 2.0]], [[3.0, 4.0, 5.0], [6.0, 1.0, 
7.0]]]])
      arg[:3,:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [4.0, 4.0, 4.0], [3.0, 1.0, 3.0]], [[3.0, 8.0, 8.0], [8.0, 6.0, 7.0], [7.0, 
5.0, 1.0]], [[8.0, 8.0, 2.0], [8.0, 1.0, 3.0], [1.0, 8.0, 8.0]]], [[[1.0, 4.0, 3.0], [7.0, 7.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 
1.0, 1.0], [3.0, 4.0, 4.0], [4.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [7.0, 7.0, 8.0], [8.0, 4.0, 6.0]]], [[[1.0, 5.0, 3.0], [2.0, 
6.0, 1.0], [7.0, 7.0, 4.0]], [[4.0, 8.0, 5.0], [5.0, 8.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 5.0, 8.0], [1.0, 3.0, 5.0], [4.0, 3.0, 
6.0]]], [[[8.0, 1.0, 6.0], [6.0, 8.0, 6.0], [2.0, 6.0, 5.0]], [[6.0, 6.0, 5.0], [4.0, 2.0, 3.0], [2.0, 7.0, 7.0]], [[6.0, 5.0, 
3.0], [6.0, 3.0, 3.0], [8.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 8.0], [4.0, 5.0, 8.0], [3.0, 1.0, 3.0]], 
[[3.0, 5.0, 6.0], [3.0, 7.0, 7.0], [7.0, 5.0, 1.0]], [[8.0, 8.0, 2.0], [8.0, 1.0, 3.0], [1.0, 8.0, 8.0]]], [[[3.0, 4.0, 4.0], 
[4.0, 3.0, 3.0], [4.0, 3.0, 3.0]], [[8.0, 4.0, 5.0], [7.0, 5.0, 6.0], [4.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [7.0, 7.0, 8.0], 
[8.0, 4.0, 6.0]]], [[[7.0, 4.0, 1.0], [5.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[3.0, 4.0, 5.0], [6.0, 1.0, 7.0], [2.0, 4.0, 4.0]], 
[[8.0, 5.0, 8.0], [1.0, 3.0, 5.0], [4.0, 3.0, 6.0]]], [[[8.0, 1.0, 6.0], [6.0, 8.0, 6.0], [2.0, 6.0, 5.0]], [[6.0, 6.0, 5.0], 
[4.0, 2.0, 3.0], [2.0, 7.0, 7.0]], [[6.0, 5.0, 3.0], [6.0, 3.0, 3.0], [8.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLCC(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 6.0], [7.0, 5.0, 8.0], [4.0, 1.0, 6.0]], [[3.0, 6.0, 6.0], [4.0, 5.0, 6.0], [8.0, 4.0, 
2.0]], [[7.0, 1.0, 2.0], [6.0, 1.0, 4.0], [6.0, 1.0, 1.0]]], [[[4.0, 7.0, 2.0], [4.0, 7.0, 5.0], [8.0, 2.0, 7.0]], [[6.0, 1.0, 
7.0], [2.0, 6.0, 5.0], [5.0, 3.0, 1.0]], [[5.0, 3.0, 1.0], [1.0, 2.0, 6.0], [6.0, 1.0, 3.0]]], [[[4.0, 7.0, 8.0], [5.0, 2.0, 
7.0], [3.0, 2.0, 3.0]], [[6.0, 2.0, 2.0], [4.0, 4.0, 2.0], [5.0, 2.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 8.0, 1.0], [1.0, 8.0, 
7.0]]], [[[5.0, 6.0, 5.0], [5.0, 7.0, 3.0], [7.0, 4.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 6.0, 4.0], [5.0, 6.0, 8.0]], [[4.0, 8.0, 
3.0], [8.0, 7.0, 4.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 6.0, 7.0], [1.0, 7.0, 3.0], [8.0, 7.0, 7.0]]], [[[3.0, 5.0, 1.0], [3.0, 4.0, 2.0], [1.0, 2.0, 
6.0]]], [[[1.0, 4.0, 2.0], [5.0, 3.0, 3.0], [1.0, 7.0, 7.0]]], [[[2.0, 3.0, 8.0], [8.0, 4.0, 4.0], [3.0, 5.0, 3.0]]]])
      arg[:,2:,:,:]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 6.0], [7.0, 5.0, 8.0], [4.0, 1.0, 6.0]], [[3.0, 6.0, 6.0], [4.0, 5.0, 6.0], [8.0, 4.0, 
2.0]], [[2.0, 6.0, 7.0], [1.0, 7.0, 3.0], [8.0, 7.0, 7.0]]], [[[4.0, 7.0, 2.0], [4.0, 7.0, 5.0], [8.0, 2.0, 7.0]], [[6.0, 1.0, 
7.0], [2.0, 6.0, 5.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 1.0], [3.0, 4.0, 2.0], [1.0, 2.0, 6.0]]], [[[4.0, 7.0, 8.0], [5.0, 2.0, 
7.0], [3.0, 2.0, 3.0]], [[6.0, 2.0, 2.0], [4.0, 4.0, 2.0], [5.0, 2.0, 4.0]], [[1.0, 4.0, 2.0], [5.0, 3.0, 3.0], [1.0, 7.0, 
7.0]]], [[[5.0, 6.0, 5.0], [5.0, 7.0, 3.0], [7.0, 4.0, 1.0]], [[6.0, 8.0, 2.0], [8.0, 6.0, 4.0], [5.0, 6.0, 8.0]], [[2.0, 3.0, 
8.0], [8.0, 4.0, 4.0], [3.0, 5.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLCC(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 2.0], [2.0, 4.0, 2.0], [7.0, 4.0, 4.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 2.0], [8.0, 6.0, 
8.0]], [[7.0, 1.0, 4.0], [4.0, 5.0, 3.0], [5.0, 7.0, 5.0]]], [[[1.0, 2.0, 6.0], [2.0, 3.0, 4.0], [5.0, 5.0, 5.0]], [[1.0, 4.0, 
5.0], [4.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 4.0, 3.0], [1.0, 2.0, 4.0], [3.0, 5.0, 3.0]]], [[[7.0, 1.0, 2.0], [8.0, 3.0, 
4.0], [4.0, 2.0, 1.0]], [[1.0, 7.0, 8.0], [6.0, 8.0, 5.0], [2.0, 6.0, 6.0]], [[7.0, 6.0, 8.0], [6.0, 4.0, 2.0], [5.0, 4.0, 
8.0]]], [[[4.0, 4.0, 8.0], [5.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[6.0, 8.0, 7.0], [5.0, 7.0, 2.0], [4.0, 7.0, 7.0]], [[5.0, 4.0, 
7.0], [8.0, 8.0, 2.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 7.0, 6.0], [3.0, 5.0, 6.0], [2.0, 1.0, 2.0]]], [[[7.0, 2.0, 8.0], [8.0, 3.0, 6.0], [1.0, 
4.0, 5.0]]], [[[8.0, 5.0, 6.0], [3.0, 4.0, 2.0], [4.0, 6.0, 7.0]]], [[[8.0, 8.0, 1.0], [7.0, 5.0, 2.0], [6.0, 6.0, 
3.0]]]]),self.functionspace)
      arg[:,2:,:,:]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 2.0], [2.0, 4.0, 2.0], [7.0, 4.0, 4.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 2.0], [8.0, 6.0, 
8.0]], [[7.0, 7.0, 6.0], [3.0, 5.0, 6.0], [2.0, 1.0, 2.0]]], [[[1.0, 2.0, 6.0], [2.0, 3.0, 4.0], [5.0, 5.0, 5.0]], [[1.0, 4.0, 
5.0], [4.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[7.0, 2.0, 8.0], [8.0, 3.0, 6.0], [1.0, 4.0, 5.0]]], [[[7.0, 1.0, 2.0], [8.0, 3.0, 
4.0], [4.0, 2.0, 1.0]], [[1.0, 7.0, 8.0], [6.0, 8.0, 5.0], [2.0, 6.0, 6.0]], [[8.0, 5.0, 6.0], [3.0, 4.0, 2.0], [4.0, 6.0, 
7.0]]], [[[4.0, 4.0, 8.0], [5.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[6.0, 8.0, 7.0], [5.0, 7.0, 2.0], [4.0, 7.0, 7.0]], [[8.0, 8.0, 
1.0], [7.0, 5.0, 2.0], [6.0, 6.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLCC(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 2.0], [3.0, 8.0, 6.0], [6.0, 2.0, 4.0]], [[2.0, 4.0, 5.0], [6.0, 8.0, 8.0], [2.0, 8.0, 
5.0]], [[3.0, 6.0, 2.0], [4.0, 1.0, 4.0], [8.0, 6.0, 3.0]]], [[[8.0, 6.0, 4.0], [3.0, 4.0, 6.0], [8.0, 6.0, 1.0]], [[6.0, 2.0, 
1.0], [1.0, 8.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 3.0, 8.0], [2.0, 4.0, 4.0], [7.0, 7.0, 6.0]]], [[[8.0, 5.0, 2.0], [1.0, 2.0, 
6.0], [7.0, 4.0, 8.0]], [[5.0, 1.0, 2.0], [4.0, 8.0, 1.0], [7.0, 5.0, 6.0]], [[6.0, 1.0, 1.0], [3.0, 6.0, 3.0], [1.0, 7.0, 
6.0]]], [[[7.0, 6.0, 3.0], [5.0, 7.0, 2.0], [6.0, 8.0, 2.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 3.0], [3.0, 4.0, 3.0]], [[1.0, 8.0, 
4.0], [6.0, 2.0, 4.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 3.0, 5.0], [2.0, 2.0, 1.0], [8.0, 5.0, 2.0]]], [[[7.0, 3.0, 2.0], [8.0, 5.0, 7.0], [7.0, 
3.0, 2.0]]], [[[6.0, 4.0, 1.0], [7.0, 2.0, 4.0], [3.0, 3.0, 7.0]]], [[[5.0, 8.0, 1.0], [4.0, 2.0, 7.0], [7.0, 6.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 2.0, 3.0], [6.0, 1.0, 6.0], [6.0, 7.0, 1.0]]], [[[5.0, 8.0, 4.0], [1.0, 3.0, 2.0], 
[3.0, 5.0, 1.0]]], [[[5.0, 5.0, 6.0], [6.0, 1.0, 6.0], [3.0, 6.0, 1.0]]], [[[7.0, 3.0, 2.0], [5.0, 1.0, 5.0], [4.0, 6.0, 
6.0]]]])
      arg[:,2:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 2.0], [3.0, 8.0, 6.0], [6.0, 2.0, 4.0]], [[2.0, 4.0, 5.0], [6.0, 8.0, 8.0], [2.0, 
8.0, 5.0]], [[5.0, 3.0, 5.0], [2.0, 2.0, 1.0], [8.0, 5.0, 2.0]]], [[[8.0, 6.0, 4.0], [3.0, 4.0, 6.0], [8.0, 6.0, 1.0]], [[6.0, 
2.0, 1.0], [1.0, 8.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 3.0, 2.0], [8.0, 5.0, 7.0], [7.0, 3.0, 2.0]]], [[[8.0, 5.0, 2.0], [1.0, 
2.0, 6.0], [7.0, 4.0, 8.0]], [[5.0, 1.0, 2.0], [4.0, 8.0, 1.0], [7.0, 5.0, 6.0]], [[6.0, 4.0, 1.0], [7.0, 2.0, 4.0], [3.0, 3.0, 
7.0]]], [[[7.0, 6.0, 3.0], [5.0, 7.0, 2.0], [6.0, 8.0, 2.0]], [[3.0, 3.0, 2.0], [3.0, 7.0, 3.0], [3.0, 4.0, 3.0]], [[5.0, 8.0, 
1.0], [4.0, 2.0, 7.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 2.0], [3.0, 8.0, 6.0], [6.0, 2.0, 4.0]], 
[[2.0, 4.0, 5.0], [6.0, 8.0, 8.0], [2.0, 8.0, 5.0]], [[2.0, 2.0, 3.0], [6.0, 1.0, 6.0], [6.0, 7.0, 1.0]]], [[[8.0, 6.0, 4.0], 
[3.0, 4.0, 6.0], [8.0, 6.0, 1.0]], [[6.0, 2.0, 1.0], [1.0, 8.0, 7.0], [8.0, 7.0, 1.0]], [[5.0, 8.0, 4.0], [1.0, 3.0, 2.0], 
[3.0, 5.0, 1.0]]], [[[8.0, 5.0, 2.0], [1.0, 2.0, 6.0], [7.0, 4.0, 8.0]], [[5.0, 1.0, 2.0], [4.0, 8.0, 1.0], [7.0, 5.0, 6.0]], 
[[5.0, 5.0, 6.0], [6.0, 1.0, 6.0], [3.0, 6.0, 1.0]]], [[[7.0, 6.0, 3.0], [5.0, 7.0, 2.0], [6.0, 8.0, 2.0]], [[3.0, 3.0, 2.0], 
[3.0, 7.0, 3.0], [3.0, 4.0, 3.0]], [[7.0, 3.0, 2.0], [5.0, 1.0, 5.0], [4.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPCC(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 5.0], [5.0, 2.0, 6.0], [6.0, 4.0, 4.0]], [[1.0, 4.0, 2.0], [6.0, 1.0, 1.0], [4.0, 2.0, 
1.0]], [[6.0, 7.0, 2.0], [8.0, 1.0, 3.0], [5.0, 3.0, 6.0]]], [[[8.0, 3.0, 7.0], [1.0, 5.0, 7.0], [1.0, 6.0, 3.0]], [[3.0, 1.0, 
1.0], [2.0, 7.0, 5.0], [1.0, 7.0, 6.0]], [[8.0, 1.0, 8.0], [8.0, 6.0, 6.0], [3.0, 7.0, 8.0]]], [[[7.0, 4.0, 3.0], [6.0, 7.0, 
5.0], [3.0, 5.0, 3.0]], [[6.0, 3.0, 8.0], [8.0, 5.0, 6.0], [1.0, 3.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 6.0], [5.0, 2.0, 
2.0]]], [[[7.0, 6.0, 1.0], [5.0, 3.0, 8.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [4.0, 8.0, 6.0], [8.0, 2.0, 8.0]], [[3.0, 2.0, 
6.0], [7.0, 8.0, 3.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[3.0, 8.0, 5.0], [6.0, 2.0, 6.0], [3.0, 2.0, 5.0]])
      arg[2,1,:,:]=val
      ref=Data(numpy.array([[[[1.0, 2.0, 5.0], [5.0, 2.0, 6.0], [6.0, 4.0, 4.0]], [[1.0, 4.0, 2.0], [6.0, 1.0, 1.0], [4.0, 2.0, 
1.0]], [[6.0, 7.0, 2.0], [8.0, 1.0, 3.0], [5.0, 3.0, 6.0]]], [[[8.0, 3.0, 7.0], [1.0, 5.0, 7.0], [1.0, 6.0, 3.0]], [[3.0, 1.0, 
1.0], [2.0, 7.0, 5.0], [1.0, 7.0, 6.0]], [[8.0, 1.0, 8.0], [8.0, 6.0, 6.0], [3.0, 7.0, 8.0]]], [[[7.0, 4.0, 3.0], [6.0, 7.0, 
5.0], [3.0, 5.0, 3.0]], [[3.0, 8.0, 5.0], [6.0, 2.0, 6.0], [3.0, 2.0, 5.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 6.0], [5.0, 2.0, 
2.0]]], [[[7.0, 6.0, 1.0], [5.0, 3.0, 8.0], [7.0, 8.0, 7.0]], [[8.0, 8.0, 6.0], [4.0, 8.0, 6.0], [8.0, 2.0, 8.0]], [[3.0, 2.0, 
6.0], [7.0, 8.0, 3.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPCC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 5.0], [2.0, 1.0, 1.0], [2.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [8.0, 1.0, 1.0], [2.0, 7.0, 
5.0]], [[2.0, 6.0, 2.0], [3.0, 2.0, 3.0], [7.0, 3.0, 6.0]]], [[[8.0, 7.0, 6.0], [7.0, 7.0, 2.0], [7.0, 3.0, 4.0]], [[6.0, 7.0, 
2.0], [7.0, 4.0, 5.0], [2.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [3.0, 2.0, 3.0], [2.0, 4.0, 5.0]]], [[[5.0, 5.0, 2.0], [2.0, 4.0, 
2.0], [2.0, 5.0, 3.0]], [[1.0, 2.0, 1.0], [6.0, 7.0, 2.0], [5.0, 8.0, 2.0]], [[6.0, 1.0, 3.0], [8.0, 5.0, 4.0], [4.0, 7.0, 
5.0]]], [[[7.0, 3.0, 3.0], [8.0, 7.0, 8.0], [8.0, 7.0, 7.0]], [[2.0, 8.0, 5.0], [5.0, 8.0, 2.0], [5.0, 6.0, 1.0]], [[7.0, 2.0, 
1.0], [7.0, 7.0, 6.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[8.0, 1.0, 1.0], [2.0, 2.0, 8.0], [2.0, 2.0, 4.0]]),self.functionspace)
      arg[2,1,:,:]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 5.0], [2.0, 1.0, 1.0], [2.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [8.0, 1.0, 1.0], [2.0, 7.0, 
5.0]], [[2.0, 6.0, 2.0], [3.0, 2.0, 3.0], [7.0, 3.0, 6.0]]], [[[8.0, 7.0, 6.0], [7.0, 7.0, 2.0], [7.0, 3.0, 4.0]], [[6.0, 7.0, 
2.0], [7.0, 4.0, 5.0], [2.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [3.0, 2.0, 3.0], [2.0, 4.0, 5.0]]], [[[5.0, 5.0, 2.0], [2.0, 4.0, 
2.0], [2.0, 5.0, 3.0]], [[8.0, 1.0, 1.0], [2.0, 2.0, 8.0], [2.0, 2.0, 4.0]], [[6.0, 1.0, 3.0], [8.0, 5.0, 4.0], [4.0, 7.0, 
5.0]]], [[[7.0, 3.0, 3.0], [8.0, 7.0, 8.0], [8.0, 7.0, 7.0]], [[2.0, 8.0, 5.0], [5.0, 8.0, 2.0], [5.0, 6.0, 1.0]], [[7.0, 2.0, 
1.0], [7.0, 7.0, 6.0], [4.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPCC(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 7.0], [3.0, 2.0, 1.0], [2.0, 7.0, 7.0]], [[1.0, 2.0, 8.0], [1.0, 1.0, 4.0], [4.0, 7.0, 
7.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 2.0], [7.0, 2.0, 5.0]]], [[[5.0, 5.0, 8.0], [5.0, 6.0, 3.0], [3.0, 1.0, 6.0]], [[1.0, 8.0, 
6.0], [7.0, 6.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 4.0], [2.0, 8.0, 8.0]]], [[[4.0, 1.0, 8.0], [7.0, 5.0, 
4.0], [5.0, 8.0, 4.0]], [[6.0, 4.0, 3.0], [6.0, 6.0, 1.0], [5.0, 7.0, 8.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 1.0], [2.0, 3.0, 
8.0]]], [[[8.0, 1.0, 4.0], [4.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[5.0, 8.0, 6.0], [3.0, 8.0, 6.0], [2.0, 5.0, 3.0]], [[3.0, 1.0, 
3.0], [8.0, 7.0, 3.0], [5.0, 4.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 7.0, 3.0], [4.0, 2.0, 7.0], [3.0, 1.0, 3.0]])+(1.-msk_val)*numpy.array([[5.0, 5.0, 7.0], 
[3.0, 4.0, 5.0], [6.0, 7.0, 8.0]])
      arg[2,1,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 7.0], [3.0, 2.0, 1.0], [2.0, 7.0, 7.0]], [[1.0, 2.0, 8.0], [1.0, 1.0, 4.0], [4.0, 
7.0, 7.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 2.0], [7.0, 2.0, 5.0]]], [[[5.0, 5.0, 8.0], [5.0, 6.0, 3.0], [3.0, 1.0, 6.0]], [[1.0, 
8.0, 6.0], [7.0, 6.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 4.0], [2.0, 8.0, 8.0]]], [[[4.0, 1.0, 8.0], [7.0, 
5.0, 4.0], [5.0, 8.0, 4.0]], [[5.0, 7.0, 3.0], [4.0, 2.0, 7.0], [3.0, 1.0, 3.0]], [[6.0, 5.0, 2.0], [2.0, 2.0, 1.0], [2.0, 3.0, 
8.0]]], [[[8.0, 1.0, 4.0], [4.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[5.0, 8.0, 6.0], [3.0, 8.0, 6.0], [2.0, 5.0, 3.0]], [[3.0, 1.0, 
3.0], [8.0, 7.0, 3.0], [5.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [3.0, 2.0, 1.0], [2.0, 7.0, 7.0]], 
[[1.0, 2.0, 8.0], [1.0, 1.0, 4.0], [4.0, 7.0, 7.0]], [[5.0, 5.0, 5.0], [7.0, 2.0, 2.0], [7.0, 2.0, 5.0]]], [[[5.0, 5.0, 8.0], 
[5.0, 6.0, 3.0], [3.0, 1.0, 6.0]], [[1.0, 8.0, 6.0], [7.0, 6.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 4.0], 
[2.0, 8.0, 8.0]]], [[[4.0, 1.0, 8.0], [7.0, 5.0, 4.0], [5.0, 8.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 4.0, 5.0], [6.0, 7.0, 8.0]], 
[[6.0, 5.0, 2.0], [2.0, 2.0, 1.0], [2.0, 3.0, 8.0]]], [[[8.0, 1.0, 4.0], [4.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[5.0, 8.0, 6.0], 
[3.0, 8.0, 6.0], [2.0, 5.0, 3.0]], [[3.0, 1.0, 3.0], [8.0, 7.0, 3.0], [5.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPCN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 2.0], [7.0, 8.0, 7.0], [6.0, 1.0, 4.0]], [[1.0, 3.0, 8.0], [8.0, 8.0, 6.0], [2.0, 1.0, 
2.0]], [[7.0, 3.0, 3.0], [5.0, 7.0, 4.0], [1.0, 4.0, 1.0]]], [[[6.0, 2.0, 5.0], [4.0, 6.0, 1.0], [7.0, 6.0, 2.0]], [[2.0, 2.0, 
3.0], [8.0, 5.0, 5.0], [6.0, 6.0, 8.0]], [[7.0, 7.0, 6.0], [6.0, 1.0, 2.0], [7.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [6.0, 5.0, 
2.0], [3.0, 2.0, 3.0]], [[7.0, 4.0, 3.0], [6.0, 5.0, 7.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 4.0], [2.0, 4.0, 2.0], [1.0, 5.0, 
6.0]]], [[[4.0, 4.0, 2.0], [2.0, 3.0, 3.0], [4.0, 1.0, 2.0]], [[6.0, 7.0, 5.0], [8.0, 4.0, 8.0], [6.0, 2.0, 7.0]], [[6.0, 1.0, 
8.0], [6.0, 2.0, 4.0], [3.0, 1.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 5.0, 1.0], [2.0, 3.0, 2.0], [7.0, 3.0, 1.0]], [[6.0, 4.0, 6.0], [8.0, 4.0, 3.0], [5.0, 5.0, 
7.0]]])
      arg[2:,1,:]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 2.0], [7.0, 8.0, 7.0], [6.0, 1.0, 4.0]], [[1.0, 3.0, 8.0], [8.0, 8.0, 6.0], [2.0, 1.0, 
2.0]], [[7.0, 3.0, 3.0], [5.0, 7.0, 4.0], [1.0, 4.0, 1.0]]], [[[6.0, 2.0, 5.0], [4.0, 6.0, 1.0], [7.0, 6.0, 2.0]], [[2.0, 2.0, 
3.0], [8.0, 5.0, 5.0], [6.0, 6.0, 8.0]], [[7.0, 7.0, 6.0], [6.0, 1.0, 2.0], [7.0, 1.0, 7.0]]], [[[2.0, 6.0, 5.0], [6.0, 5.0, 
2.0], [3.0, 2.0, 3.0]], [[5.0, 5.0, 1.0], [2.0, 3.0, 2.0], [7.0, 3.0, 1.0]], [[4.0, 4.0, 4.0], [2.0, 4.0, 2.0], [1.0, 5.0, 
6.0]]], [[[4.0, 4.0, 2.0], [2.0, 3.0, 3.0], [4.0, 1.0, 2.0]], [[6.0, 4.0, 6.0], [8.0, 4.0, 3.0], [5.0, 5.0, 7.0]], [[6.0, 1.0, 
8.0], [6.0, 2.0, 4.0], [3.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPCN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 8.0], [7.0, 3.0, 2.0], [7.0, 2.0, 4.0]], [[8.0, 1.0, 1.0], [4.0, 2.0, 2.0], [3.0, 5.0, 
5.0]], [[3.0, 2.0, 1.0], [2.0, 6.0, 8.0], [3.0, 5.0, 2.0]]], [[[6.0, 4.0, 1.0], [8.0, 3.0, 4.0], [4.0, 7.0, 7.0]], [[3.0, 3.0, 
1.0], [1.0, 1.0, 2.0], [8.0, 3.0, 4.0]], [[6.0, 5.0, 4.0], [5.0, 5.0, 3.0], [1.0, 1.0, 1.0]]], [[[4.0, 7.0, 8.0], [8.0, 8.0, 
6.0], [8.0, 8.0, 5.0]], [[5.0, 8.0, 5.0], [7.0, 7.0, 3.0], [1.0, 4.0, 3.0]], [[4.0, 1.0, 1.0], [7.0, 4.0, 1.0], [7.0, 7.0, 
5.0]]], [[[5.0, 1.0, 5.0], [2.0, 5.0, 1.0], [6.0, 6.0, 8.0]], [[8.0, 3.0, 1.0], [7.0, 5.0, 6.0], [1.0, 1.0, 4.0]], [[7.0, 4.0, 
2.0], [3.0, 7.0, 6.0], [2.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 7.0, 3.0], [4.0, 3.0, 8.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 5.0], [1.0, 4.0, 5.0], [1.0, 2.0, 
2.0]]]),self.functionspace)
      arg[2:,1,:]=val
      ref=Data(numpy.array([[[[2.0, 1.0, 8.0], [7.0, 3.0, 2.0], [7.0, 2.0, 4.0]], [[8.0, 1.0, 1.0], [4.0, 2.0, 2.0], [3.0, 5.0, 
5.0]], [[3.0, 2.0, 1.0], [2.0, 6.0, 8.0], [3.0, 5.0, 2.0]]], [[[6.0, 4.0, 1.0], [8.0, 3.0, 4.0], [4.0, 7.0, 7.0]], [[3.0, 3.0, 
1.0], [1.0, 1.0, 2.0], [8.0, 3.0, 4.0]], [[6.0, 5.0, 4.0], [5.0, 5.0, 3.0], [1.0, 1.0, 1.0]]], [[[4.0, 7.0, 8.0], [8.0, 8.0, 
6.0], [8.0, 8.0, 5.0]], [[8.0, 7.0, 3.0], [4.0, 3.0, 8.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 1.0], [7.0, 4.0, 1.0], [7.0, 7.0, 
5.0]]], [[[5.0, 1.0, 5.0], [2.0, 5.0, 1.0], [6.0, 6.0, 8.0]], [[5.0, 3.0, 5.0], [1.0, 4.0, 5.0], [1.0, 2.0, 2.0]], [[7.0, 4.0, 
2.0], [3.0, 7.0, 6.0], [2.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPCN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 3.0], [4.0, 4.0, 5.0], [7.0, 5.0, 1.0]], [[4.0, 3.0, 5.0], [7.0, 2.0, 4.0], [2.0, 1.0, 
2.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 3.0], [8.0, 2.0, 1.0]]], [[[2.0, 5.0, 7.0], [7.0, 8.0, 5.0], [5.0, 1.0, 5.0]], [[5.0, 5.0, 
8.0], [7.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 2.0, 5.0], [2.0, 1.0, 5.0]]], [[[7.0, 6.0, 5.0], [2.0, 1.0, 
7.0], [4.0, 6.0, 5.0]], [[7.0, 6.0, 2.0], [4.0, 7.0, 4.0], [2.0, 2.0, 2.0]], [[6.0, 2.0, 2.0], [5.0, 8.0, 8.0], [3.0, 7.0, 
8.0]]], [[[1.0, 6.0, 4.0], [5.0, 6.0, 3.0], [8.0, 8.0, 6.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 4.0], [3.0, 2.0, 8.0]], [[3.0, 1.0, 
6.0], [7.0, 6.0, 1.0], [4.0, 5.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 1.0, 5.0], [2.0, 4.0, 4.0], [7.0, 6.0, 3.0]], [[2.0, 6.0, 2.0], [6.0, 1.0, 2.0], [1.0, 
1.0, 4.0]]])+(1.-msk_val)*numpy.array([[[5.0, 7.0, 4.0], [4.0, 6.0, 1.0], [2.0, 5.0, 3.0]], [[4.0, 7.0, 1.0], [8.0, 5.0, 2.0], 
[1.0, 5.0, 3.0]]])
      arg[2:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 3.0], [4.0, 4.0, 5.0], [7.0, 5.0, 1.0]], [[4.0, 3.0, 5.0], [7.0, 2.0, 4.0], [2.0, 
1.0, 2.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 3.0], [8.0, 2.0, 1.0]]], [[[2.0, 5.0, 7.0], [7.0, 8.0, 5.0], [5.0, 1.0, 5.0]], [[5.0, 
5.0, 8.0], [7.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 2.0, 5.0], [2.0, 1.0, 5.0]]], [[[7.0, 6.0, 5.0], [2.0, 
1.0, 7.0], [4.0, 6.0, 5.0]], [[6.0, 1.0, 5.0], [2.0, 4.0, 4.0], [7.0, 6.0, 3.0]], [[6.0, 2.0, 2.0], [5.0, 8.0, 8.0], [3.0, 7.0, 
8.0]]], [[[1.0, 6.0, 4.0], [5.0, 6.0, 3.0], [8.0, 8.0, 6.0]], [[2.0, 6.0, 2.0], [6.0, 1.0, 2.0], [1.0, 1.0, 4.0]], [[3.0, 1.0, 
6.0], [7.0, 6.0, 1.0], [4.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 3.0], [4.0, 4.0, 5.0], [7.0, 5.0, 1.0]], 
[[4.0, 3.0, 5.0], [7.0, 2.0, 4.0], [2.0, 1.0, 2.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 3.0], [8.0, 2.0, 1.0]]], [[[2.0, 5.0, 7.0], 
[7.0, 8.0, 5.0], [5.0, 1.0, 5.0]], [[5.0, 5.0, 8.0], [7.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 2.0, 5.0], 
[2.0, 1.0, 5.0]]], [[[7.0, 6.0, 5.0], [2.0, 1.0, 7.0], [4.0, 6.0, 5.0]], [[5.0, 7.0, 4.0], [4.0, 6.0, 1.0], [2.0, 5.0, 3.0]], 
[[6.0, 2.0, 2.0], [5.0, 8.0, 8.0], [3.0, 7.0, 8.0]]], [[[1.0, 6.0, 4.0], [5.0, 6.0, 3.0], [8.0, 8.0, 6.0]], [[4.0, 7.0, 1.0], 
[8.0, 5.0, 2.0], [1.0, 5.0, 3.0]], [[3.0, 1.0, 6.0], [7.0, 6.0, 1.0], [4.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCCN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 8.0], [8.0, 5.0, 5.0], [1.0, 7.0, 4.0]], [[7.0, 3.0, 5.0], [5.0, 1.0, 3.0], [6.0, 7.0, 
8.0]], [[1.0, 6.0, 2.0], [3.0, 7.0, 1.0], [4.0, 4.0, 4.0]]], [[[7.0, 4.0, 1.0], [8.0, 1.0, 7.0], [3.0, 2.0, 8.0]], [[1.0, 5.0, 
2.0], [7.0, 5.0, 3.0], [4.0, 6.0, 7.0]], [[1.0, 4.0, 2.0], [2.0, 3.0, 3.0], [7.0, 7.0, 3.0]]], [[[3.0, 8.0, 6.0], [3.0, 2.0, 
6.0], [1.0, 6.0, 7.0]], [[8.0, 4.0, 4.0], [3.0, 6.0, 6.0], [3.0, 4.0, 7.0]], [[5.0, 3.0, 2.0], [3.0, 3.0, 7.0], [3.0, 1.0, 
2.0]]], [[[5.0, 8.0, 6.0], [3.0, 5.0, 4.0], [7.0, 3.0, 7.0]], [[3.0, 6.0, 8.0], [2.0, 7.0, 4.0], [7.0, 1.0, 1.0]], [[3.0, 8.0, 
5.0], [4.0, 7.0, 3.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 1.0, 1.0], [2.0, 5.0, 8.0], [4.0, 2.0, 5.0]], [[3.0, 3.0, 8.0], [6.0, 8.0, 7.0], [1.0, 5.0, 
4.0]], [[2.0, 5.0, 5.0], [7.0, 1.0, 4.0], [6.0, 1.0, 5.0]]], [[[5.0, 8.0, 6.0], [3.0, 5.0, 8.0], [4.0, 4.0, 1.0]], [[3.0, 6.0, 
1.0], [5.0, 6.0, 7.0], [5.0, 4.0, 6.0]], [[7.0, 7.0, 3.0], [2.0, 5.0, 6.0], [3.0, 2.0, 5.0]]], [[[2.0, 6.0, 3.0], [2.0, 6.0, 
1.0], [3.0, 2.0, 1.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 1.0], [8.0, 1.0, 7.0]], [[5.0, 7.0, 8.0], [8.0, 2.0, 1.0], [7.0, 1.0, 
1.0]]]])
      arg[:3,:,:]=val
      ref=Data(numpy.array([[[[4.0, 1.0, 1.0], [2.0, 5.0, 8.0], [4.0, 2.0, 5.0]], [[3.0, 3.0, 8.0], [6.0, 8.0, 7.0], [1.0, 5.0, 
4.0]], [[2.0, 5.0, 5.0], [7.0, 1.0, 4.0], [6.0, 1.0, 5.0]]], [[[5.0, 8.0, 6.0], [3.0, 5.0, 8.0], [4.0, 4.0, 1.0]], [[3.0, 6.0, 
1.0], [5.0, 6.0, 7.0], [5.0, 4.0, 6.0]], [[7.0, 7.0, 3.0], [2.0, 5.0, 6.0], [3.0, 2.0, 5.0]]], [[[2.0, 6.0, 3.0], [2.0, 6.0, 
1.0], [3.0, 2.0, 1.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 1.0], [8.0, 1.0, 7.0]], [[5.0, 7.0, 8.0], [8.0, 2.0, 1.0], [7.0, 1.0, 
1.0]]], [[[5.0, 8.0, 6.0], [3.0, 5.0, 4.0], [7.0, 3.0, 7.0]], [[3.0, 6.0, 8.0], [2.0, 7.0, 4.0], [7.0, 1.0, 1.0]], [[3.0, 8.0, 
5.0], [4.0, 7.0, 3.0], [1.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCCN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 5.0], [4.0, 2.0, 7.0], [6.0, 7.0, 1.0]], [[5.0, 3.0, 1.0], [4.0, 8.0, 3.0], [8.0, 3.0, 
2.0]], [[8.0, 3.0, 6.0], [3.0, 7.0, 1.0], [1.0, 4.0, 7.0]]], [[[8.0, 2.0, 1.0], [4.0, 6.0, 4.0], [2.0, 7.0, 4.0]], [[3.0, 3.0, 
3.0], [6.0, 3.0, 2.0], [2.0, 7.0, 1.0]], [[7.0, 6.0, 4.0], [4.0, 2.0, 7.0], [1.0, 5.0, 6.0]]], [[[2.0, 2.0, 7.0], [3.0, 7.0, 
5.0], [5.0, 7.0, 1.0]], [[1.0, 1.0, 7.0], [4.0, 8.0, 8.0], [6.0, 3.0, 4.0]], [[8.0, 5.0, 7.0], [7.0, 3.0, 2.0], [4.0, 5.0, 
3.0]]], [[[7.0, 7.0, 1.0], [1.0, 4.0, 4.0], [7.0, 1.0, 5.0]], [[2.0, 5.0, 2.0], [4.0, 8.0, 4.0], [4.0, 8.0, 3.0]], [[2.0, 6.0, 
6.0], [4.0, 6.0, 3.0], [4.0, 4.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 6.0, 2.0], [3.0, 3.0, 6.0], [1.0, 7.0, 5.0]], [[4.0, 8.0, 8.0], [7.0, 1.0, 8.0], [5.0, 4.0, 
5.0]], [[1.0, 8.0, 5.0], [5.0, 1.0, 3.0], [2.0, 7.0, 5.0]]], [[[5.0, 6.0, 6.0], [2.0, 5.0, 7.0], [7.0, 7.0, 3.0]], [[6.0, 8.0, 
1.0], [3.0, 2.0, 5.0], [5.0, 8.0, 5.0]], [[2.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 5.0, 6.0]]], [[[4.0, 1.0, 4.0], [4.0, 6.0, 
6.0], [7.0, 2.0, 5.0]], [[7.0, 6.0, 7.0], [2.0, 1.0, 3.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 4.0], [6.0, 1.0, 8.0], [2.0, 5.0, 
8.0]]]]),self.functionspace)
      arg[:3,:,:]=val
      ref=Data(numpy.array([[[[1.0, 6.0, 2.0], [3.0, 3.0, 6.0], [1.0, 7.0, 5.0]], [[4.0, 8.0, 8.0], [7.0, 1.0, 8.0], [5.0, 4.0, 
5.0]], [[1.0, 8.0, 5.0], [5.0, 1.0, 3.0], [2.0, 7.0, 5.0]]], [[[5.0, 6.0, 6.0], [2.0, 5.0, 7.0], [7.0, 7.0, 3.0]], [[6.0, 8.0, 
1.0], [3.0, 2.0, 5.0], [5.0, 8.0, 5.0]], [[2.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 5.0, 6.0]]], [[[4.0, 1.0, 4.0], [4.0, 6.0, 
6.0], [7.0, 2.0, 5.0]], [[7.0, 6.0, 7.0], [2.0, 1.0, 3.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 4.0], [6.0, 1.0, 8.0], [2.0, 5.0, 
8.0]]], [[[7.0, 7.0, 1.0], [1.0, 4.0, 4.0], [7.0, 1.0, 5.0]], [[2.0, 5.0, 2.0], [4.0, 8.0, 4.0], [4.0, 8.0, 3.0]], [[2.0, 6.0, 
6.0], [4.0, 6.0, 3.0], [4.0, 4.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCCN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 4.0], [6.0, 4.0, 6.0], [7.0, 4.0, 1.0]], [[1.0, 4.0, 3.0], [7.0, 6.0, 3.0], [3.0, 4.0, 
6.0]], [[4.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 7.0, 5.0]]], [[[4.0, 2.0, 7.0], [8.0, 6.0, 2.0], [7.0, 7.0, 6.0]], [[8.0, 3.0, 
1.0], [3.0, 3.0, 4.0], [3.0, 7.0, 3.0]], [[3.0, 2.0, 6.0], [2.0, 4.0, 8.0], [4.0, 8.0, 6.0]]], [[[7.0, 5.0, 4.0], [8.0, 7.0, 
2.0], [5.0, 5.0, 7.0]], [[7.0, 1.0, 3.0], [2.0, 2.0, 4.0], [6.0, 3.0, 8.0]], [[1.0, 7.0, 3.0], [1.0, 4.0, 6.0], [6.0, 6.0, 
1.0]]], [[[6.0, 4.0, 7.0], [1.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 4.0], [2.0, 7.0, 6.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 
4.0], [5.0, 1.0, 4.0], [2.0, 5.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 7.0, 6.0], [1.0, 5.0, 5.0], [8.0, 7.0, 5.0]], [[2.0, 8.0, 2.0], [5.0, 4.0, 7.0], [7.0, 
4.0, 8.0]], [[3.0, 3.0, 5.0], [5.0, 8.0, 5.0], [2.0, 8.0, 8.0]]], [[[1.0, 1.0, 3.0], [1.0, 3.0, 5.0], [2.0, 3.0, 5.0]], [[8.0, 
6.0, 1.0], [8.0, 2.0, 2.0], [3.0, 1.0, 6.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 8.0], [7.0, 5.0, 4.0]]], [[[1.0, 7.0, 6.0], [6.0, 
2.0, 6.0], [6.0, 7.0, 5.0]], [[7.0, 6.0, 3.0], [8.0, 3.0, 2.0], [5.0, 5.0, 4.0]], [[7.0, 2.0, 4.0], [1.0, 5.0, 4.0], [1.0, 8.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 2.0], [5.0, 6.0, 4.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [6.0, 8.0, 5.0], 
[7.0, 6.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 4.0, 6.0], [2.0, 5.0, 3.0]]], [[[7.0, 2.0, 3.0], [7.0, 7.0, 8.0], [6.0, 8.0, 6.0]], 
[[6.0, 5.0, 4.0], [1.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[3.0, 3.0, 8.0], [1.0, 6.0, 6.0], [1.0, 1.0, 1.0]]], [[[6.0, 5.0, 4.0], 
[8.0, 3.0, 3.0], [7.0, 5.0, 2.0]], [[4.0, 8.0, 1.0], [2.0, 5.0, 2.0], [2.0, 2.0, 7.0]], [[6.0, 2.0, 2.0], [3.0, 2.0, 5.0], 
[5.0, 3.0, 8.0]]]])
      arg[:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 6.0], [1.0, 5.0, 5.0], [8.0, 7.0, 5.0]], [[2.0, 8.0, 2.0], [5.0, 4.0, 7.0], [7.0, 
4.0, 8.0]], [[3.0, 3.0, 5.0], [5.0, 8.0, 5.0], [2.0, 8.0, 8.0]]], [[[1.0, 1.0, 3.0], [1.0, 3.0, 5.0], [2.0, 3.0, 5.0]], [[8.0, 
6.0, 1.0], [8.0, 2.0, 2.0], [3.0, 1.0, 6.0]], [[5.0, 7.0, 3.0], [7.0, 3.0, 8.0], [7.0, 5.0, 4.0]]], [[[1.0, 7.0, 6.0], [6.0, 
2.0, 6.0], [6.0, 7.0, 5.0]], [[7.0, 6.0, 3.0], [8.0, 3.0, 2.0], [5.0, 5.0, 4.0]], [[7.0, 2.0, 4.0], [1.0, 5.0, 4.0], [1.0, 8.0, 
2.0]]], [[[6.0, 4.0, 7.0], [1.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 4.0], [2.0, 7.0, 6.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 
4.0], [5.0, 1.0, 4.0], [2.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 2.0], [5.0, 6.0, 4.0], [5.0, 5.0, 4.0]], 
[[5.0, 6.0, 6.0], [6.0, 8.0, 5.0], [7.0, 6.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 4.0, 6.0], [2.0, 5.0, 3.0]]], [[[7.0, 2.0, 3.0], 
[7.0, 7.0, 8.0], [6.0, 8.0, 6.0]], [[6.0, 5.0, 4.0], [1.0, 8.0, 4.0], [4.0, 8.0, 4.0]], [[3.0, 3.0, 8.0], [1.0, 6.0, 6.0], 
[1.0, 1.0, 1.0]]], [[[6.0, 5.0, 4.0], [8.0, 3.0, 3.0], [7.0, 5.0, 2.0]], [[4.0, 8.0, 1.0], [2.0, 5.0, 2.0], [2.0, 2.0, 7.0]], 
[[6.0, 2.0, 2.0], [3.0, 2.0, 5.0], [5.0, 3.0, 8.0]]], [[[6.0, 4.0, 7.0], [1.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[5.0, 7.0, 4.0], 
[2.0, 7.0, 6.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 4.0], [5.0, 1.0, 4.0], [2.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRPN(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 2.0], [5.0, 8.0, 8.0], [8.0, 1.0, 1.0]], [[8.0, 5.0, 1.0], [3.0, 4.0, 7.0], [5.0, 4.0, 
7.0]], [[4.0, 7.0, 6.0], [8.0, 7.0, 3.0], [2.0, 2.0, 5.0]]], [[[4.0, 2.0, 4.0], [1.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 7.0, 
2.0], [3.0, 6.0, 8.0], [1.0, 4.0, 3.0]], [[1.0, 4.0, 3.0], [4.0, 2.0, 7.0], [2.0, 2.0, 8.0]]], [[[1.0, 7.0, 1.0], [4.0, 1.0, 
3.0], [1.0, 8.0, 8.0]], [[1.0, 6.0, 3.0], [4.0, 1.0, 2.0], [7.0, 3.0, 3.0]], [[2.0, 3.0, 5.0], [6.0, 4.0, 6.0], [6.0, 3.0, 
1.0]]], [[[3.0, 2.0, 7.0], [8.0, 5.0, 1.0], [1.0, 4.0, 3.0]], [[5.0, 3.0, 1.0], [8.0, 4.0, 6.0], [8.0, 7.0, 7.0]], [[7.0, 4.0, 
6.0], [1.0, 8.0, 7.0], [1.0, 4.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 2.0, 3.0]], [[6.0, 2.0, 5.0]]])
      arg[2:,1:2,1]=val
      ref=Data(numpy.array([[[[7.0, 2.0, 2.0], [5.0, 8.0, 8.0], [8.0, 1.0, 1.0]], [[8.0, 5.0, 1.0], [3.0, 4.0, 7.0], [5.0, 4.0, 
7.0]], [[4.0, 7.0, 6.0], [8.0, 7.0, 3.0], [2.0, 2.0, 5.0]]], [[[4.0, 2.0, 4.0], [1.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 7.0, 
2.0], [3.0, 6.0, 8.0], [1.0, 4.0, 3.0]], [[1.0, 4.0, 3.0], [4.0, 2.0, 7.0], [2.0, 2.0, 8.0]]], [[[1.0, 7.0, 1.0], [4.0, 1.0, 
3.0], [1.0, 8.0, 8.0]], [[1.0, 6.0, 3.0], [4.0, 2.0, 3.0], [7.0, 3.0, 3.0]], [[2.0, 3.0, 5.0], [6.0, 4.0, 6.0], [6.0, 3.0, 
1.0]]], [[[3.0, 2.0, 7.0], [8.0, 5.0, 1.0], [1.0, 4.0, 3.0]], [[5.0, 3.0, 1.0], [6.0, 2.0, 5.0], [8.0, 7.0, 7.0]], [[7.0, 4.0, 
6.0], [1.0, 8.0, 7.0], [1.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRPN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 2.0], [2.0, 4.0, 7.0], [4.0, 2.0, 5.0]], [[7.0, 5.0, 8.0], [1.0, 4.0, 5.0], [2.0, 4.0, 
6.0]], [[4.0, 4.0, 7.0], [5.0, 4.0, 4.0], [7.0, 2.0, 8.0]]], [[[4.0, 3.0, 4.0], [7.0, 1.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 8.0, 
6.0], [3.0, 1.0, 6.0], [6.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [7.0, 6.0, 7.0], [2.0, 7.0, 1.0]]], [[[8.0, 7.0, 5.0], [1.0, 6.0, 
1.0], [1.0, 7.0, 8.0]], [[1.0, 8.0, 2.0], [5.0, 8.0, 3.0], [4.0, 8.0, 5.0]], [[3.0, 3.0, 7.0], [3.0, 2.0, 1.0], [6.0, 1.0, 
7.0]]], [[[5.0, 6.0, 1.0], [6.0, 5.0, 2.0], [7.0, 3.0, 8.0]], [[3.0, 5.0, 7.0], [8.0, 2.0, 7.0], [3.0, 8.0, 5.0]], [[6.0, 4.0, 
5.0], [8.0, 2.0, 6.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 3.0, 1.0]], [[2.0, 6.0, 6.0]]]),self.functionspace)
      arg[2:,1:2,1]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 2.0], [2.0, 4.0, 7.0], [4.0, 2.0, 5.0]], [[7.0, 5.0, 8.0], [1.0, 4.0, 5.0], [2.0, 4.0, 
6.0]], [[4.0, 4.0, 7.0], [5.0, 4.0, 4.0], [7.0, 2.0, 8.0]]], [[[4.0, 3.0, 4.0], [7.0, 1.0, 5.0], [2.0, 1.0, 4.0]], [[7.0, 8.0, 
6.0], [3.0, 1.0, 6.0], [6.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [7.0, 6.0, 7.0], [2.0, 7.0, 1.0]]], [[[8.0, 7.0, 5.0], [1.0, 6.0, 
1.0], [1.0, 7.0, 8.0]], [[1.0, 8.0, 2.0], [7.0, 3.0, 1.0], [4.0, 8.0, 5.0]], [[3.0, 3.0, 7.0], [3.0, 2.0, 1.0], [6.0, 1.0, 
7.0]]], [[[5.0, 6.0, 1.0], [6.0, 5.0, 2.0], [7.0, 3.0, 8.0]], [[3.0, 5.0, 7.0], [2.0, 6.0, 6.0], [3.0, 8.0, 5.0]], [[6.0, 4.0, 
5.0], [8.0, 2.0, 6.0], [2.0, 5.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRPN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 4.0], [6.0, 3.0, 5.0], [3.0, 1.0, 8.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 3.0], [8.0, 4.0, 
5.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 1.0], [3.0, 1.0, 4.0]]], [[[5.0, 3.0, 4.0], [5.0, 2.0, 7.0], [6.0, 2.0, 5.0]], [[5.0, 8.0, 
2.0], [7.0, 6.0, 4.0], [4.0, 5.0, 1.0]], [[1.0, 6.0, 4.0], [5.0, 8.0, 7.0], [5.0, 7.0, 5.0]]], [[[6.0, 8.0, 1.0], [5.0, 6.0, 
4.0], [5.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [2.0, 7.0, 7.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 1.0], [5.0, 6.0, 5.0], [8.0, 5.0, 
1.0]]], [[[4.0, 3.0, 7.0], [6.0, 4.0, 6.0], [1.0, 1.0, 8.0]], [[4.0, 7.0, 5.0], [7.0, 2.0, 6.0], [6.0, 3.0, 4.0]], [[8.0, 7.0, 
2.0], [3.0, 8.0, 3.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 4.0, 1.0]], [[2.0, 4.0, 7.0]]])+(1.-msk_val)*numpy.array([[[2.0, 4.0, 1.0]], [[7.0, 4.0, 
7.0]]])
      arg[2:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [6.0, 3.0, 5.0], [3.0, 1.0, 8.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 3.0], [8.0, 
4.0, 5.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 1.0], [3.0, 1.0, 4.0]]], [[[5.0, 3.0, 4.0], [5.0, 2.0, 7.0], [6.0, 2.0, 5.0]], [[5.0, 
8.0, 2.0], [7.0, 6.0, 4.0], [4.0, 5.0, 1.0]], [[1.0, 6.0, 4.0], [5.0, 8.0, 7.0], [5.0, 7.0, 5.0]]], [[[6.0, 8.0, 1.0], [5.0, 
6.0, 4.0], [5.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 1.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 1.0], [5.0, 6.0, 5.0], [8.0, 5.0, 
1.0]]], [[[4.0, 3.0, 7.0], [6.0, 4.0, 6.0], [1.0, 1.0, 8.0]], [[4.0, 7.0, 5.0], [2.0, 4.0, 7.0], [6.0, 3.0, 4.0]], [[8.0, 7.0, 
2.0], [3.0, 8.0, 3.0], [3.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 4.0], [6.0, 3.0, 5.0], [3.0, 1.0, 8.0]], 
[[5.0, 8.0, 4.0], [4.0, 2.0, 3.0], [8.0, 4.0, 5.0]], [[7.0, 3.0, 7.0], [4.0, 5.0, 1.0], [3.0, 1.0, 4.0]]], [[[5.0, 3.0, 4.0], 
[5.0, 2.0, 7.0], [6.0, 2.0, 5.0]], [[5.0, 8.0, 2.0], [7.0, 6.0, 4.0], [4.0, 5.0, 1.0]], [[1.0, 6.0, 4.0], [5.0, 8.0, 7.0], 
[5.0, 7.0, 5.0]]], [[[6.0, 8.0, 1.0], [5.0, 6.0, 4.0], [5.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [2.0, 4.0, 1.0], [5.0, 5.0, 4.0]], 
[[2.0, 1.0, 1.0], [5.0, 6.0, 5.0], [8.0, 5.0, 1.0]]], [[[4.0, 3.0, 7.0], [6.0, 4.0, 6.0], [1.0, 1.0, 8.0]], [[4.0, 7.0, 5.0], 
[7.0, 4.0, 7.0], [6.0, 3.0, 4.0]], [[8.0, 7.0, 2.0], [3.0, 8.0, 3.0], [3.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CPRN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 1.0], [8.0, 4.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 5.0, 1.0], [8.0, 8.0, 8.0], [8.0, 3.0, 
2.0]], [[7.0, 4.0, 1.0], [2.0, 4.0, 6.0], [2.0, 4.0, 5.0]]], [[[5.0, 7.0, 7.0], [8.0, 5.0, 2.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 
4.0], [6.0, 2.0, 3.0], [8.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [7.0, 6.0, 6.0], [7.0, 3.0, 8.0]]], [[[4.0, 4.0, 2.0], [2.0, 5.0, 
2.0], [8.0, 6.0, 4.0]], [[2.0, 5.0, 7.0], [6.0, 6.0, 3.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 4.0, 4.0], [8.0, 1.0, 
2.0]]], [[[2.0, 3.0, 6.0], [6.0, 2.0, 4.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 2.0], [5.0, 6.0, 5.0], [2.0, 2.0, 5.0]], [[7.0, 2.0, 
5.0], [8.0, 6.0, 7.0], [8.0, 1.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 6.0, 3.0]], [[3.0, 4.0, 8.0]], [[5.0, 1.0, 7.0]], [[4.0, 7.0, 1.0]]])
      arg[:,1,1:2]=val
      ref=Data(numpy.array([[[[5.0, 6.0, 1.0], [8.0, 4.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 5.0, 1.0], [5.0, 6.0, 3.0], [8.0, 3.0, 
2.0]], [[7.0, 4.0, 1.0], [2.0, 4.0, 6.0], [2.0, 4.0, 5.0]]], [[[5.0, 7.0, 7.0], [8.0, 5.0, 2.0], [7.0, 2.0, 6.0]], [[7.0, 7.0, 
4.0], [3.0, 4.0, 8.0], [8.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [7.0, 6.0, 6.0], [7.0, 3.0, 8.0]]], [[[4.0, 4.0, 2.0], [2.0, 5.0, 
2.0], [8.0, 6.0, 4.0]], [[2.0, 5.0, 7.0], [5.0, 1.0, 7.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 4.0, 4.0], [8.0, 1.0, 
2.0]]], [[[2.0, 3.0, 6.0], [6.0, 2.0, 4.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 2.0], [4.0, 7.0, 1.0], [2.0, 2.0, 5.0]], [[7.0, 2.0, 
5.0], [8.0, 6.0, 7.0], [8.0, 1.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CPRN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 7.0], [3.0, 2.0, 3.0], [5.0, 8.0, 8.0]], [[1.0, 5.0, 1.0], [1.0, 5.0, 1.0], [8.0, 3.0, 
5.0]], [[3.0, 7.0, 1.0], [6.0, 4.0, 8.0], [6.0, 3.0, 8.0]]], [[[8.0, 3.0, 3.0], [2.0, 6.0, 5.0], [2.0, 1.0, 1.0]], [[6.0, 2.0, 
7.0], [4.0, 1.0, 8.0], [1.0, 5.0, 1.0]], [[8.0, 2.0, 2.0], [5.0, 1.0, 7.0], [2.0, 6.0, 6.0]]], [[[8.0, 5.0, 1.0], [7.0, 3.0, 
1.0], [5.0, 5.0, 1.0]], [[1.0, 4.0, 7.0], [2.0, 2.0, 5.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 6.0], [5.0, 7.0, 7.0], [2.0, 2.0, 
3.0]]], [[[6.0, 6.0, 7.0], [5.0, 5.0, 3.0], [2.0, 4.0, 6.0]], [[7.0, 2.0, 7.0], [7.0, 8.0, 5.0], [5.0, 4.0, 7.0]], [[2.0, 2.0, 
8.0], [1.0, 8.0, 3.0], [2.0, 6.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 7.0, 1.0]], [[1.0, 1.0, 1.0]], [[1.0, 4.0, 4.0]], [[4.0, 3.0, 8.0]]]),self.functionspace)
      arg[:,1,1:2]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 7.0], [3.0, 2.0, 3.0], [5.0, 8.0, 8.0]], [[1.0, 5.0, 1.0], [2.0, 7.0, 1.0], [8.0, 3.0, 
5.0]], [[3.0, 7.0, 1.0], [6.0, 4.0, 8.0], [6.0, 3.0, 8.0]]], [[[8.0, 3.0, 3.0], [2.0, 6.0, 5.0], [2.0, 1.0, 1.0]], [[6.0, 2.0, 
7.0], [1.0, 1.0, 1.0], [1.0, 5.0, 1.0]], [[8.0, 2.0, 2.0], [5.0, 1.0, 7.0], [2.0, 6.0, 6.0]]], [[[8.0, 5.0, 1.0], [7.0, 3.0, 
1.0], [5.0, 5.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 4.0, 4.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 6.0], [5.0, 7.0, 7.0], [2.0, 2.0, 
3.0]]], [[[6.0, 6.0, 7.0], [5.0, 5.0, 3.0], [2.0, 4.0, 6.0]], [[7.0, 2.0, 7.0], [4.0, 3.0, 8.0], [5.0, 4.0, 7.0]], [[2.0, 2.0, 
8.0], [1.0, 8.0, 3.0], [2.0, 6.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CPRN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[1.0, 2.0, 3.0], [1.0, 7.0, 8.0], [2.0, 8.0, 
6.0]], [[2.0, 7.0, 8.0], [4.0, 7.0, 6.0], [8.0, 4.0, 2.0]]], [[[3.0, 4.0, 4.0], [1.0, 3.0, 4.0], [2.0, 6.0, 6.0]], [[4.0, 5.0, 
8.0], [6.0, 3.0, 4.0], [4.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [5.0, 2.0, 6.0], [6.0, 3.0, 5.0]]], [[[4.0, 3.0, 5.0], [7.0, 8.0, 
6.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [6.0, 6.0, 7.0], [4.0, 8.0, 5.0]], [[2.0, 1.0, 3.0], [7.0, 3.0, 6.0], [6.0, 2.0, 
8.0]]], [[[6.0, 4.0, 6.0], [6.0, 5.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 7.0, 5.0], [8.0, 1.0, 2.0], [3.0, 3.0, 1.0]], [[6.0, 8.0, 
6.0], [4.0, 3.0, 8.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 3.0, 1.0]], [[2.0, 1.0, 6.0]], [[6.0, 6.0, 5.0]], [[3.0, 3.0, 
7.0]]])+(1.-msk_val)*numpy.array([[[1.0, 3.0, 1.0]], [[6.0, 1.0, 1.0]], [[5.0, 3.0, 1.0]], [[5.0, 7.0, 8.0]]])
      arg[:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[1.0, 2.0, 3.0], [5.0, 3.0, 1.0], [2.0, 
8.0, 6.0]], [[2.0, 7.0, 8.0], [4.0, 7.0, 6.0], [8.0, 4.0, 2.0]]], [[[3.0, 4.0, 4.0], [1.0, 3.0, 4.0], [2.0, 6.0, 6.0]], [[4.0, 
5.0, 8.0], [2.0, 1.0, 6.0], [4.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [5.0, 2.0, 6.0], [6.0, 3.0, 5.0]]], [[[4.0, 3.0, 5.0], [7.0, 
8.0, 6.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [6.0, 6.0, 5.0], [4.0, 8.0, 5.0]], [[2.0, 1.0, 3.0], [7.0, 3.0, 6.0], [6.0, 2.0, 
8.0]]], [[[6.0, 4.0, 6.0], [6.0, 5.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 7.0, 5.0], [3.0, 3.0, 7.0], [3.0, 3.0, 1.0]], [[6.0, 8.0, 
6.0], [4.0, 3.0, 8.0], [3.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 6.0], [2.0, 6.0, 8.0], [3.0, 1.0, 5.0]], 
[[1.0, 2.0, 3.0], [1.0, 3.0, 1.0], [2.0, 8.0, 6.0]], [[2.0, 7.0, 8.0], [4.0, 7.0, 6.0], [8.0, 4.0, 2.0]]], [[[3.0, 4.0, 4.0], 
[1.0, 3.0, 4.0], [2.0, 6.0, 6.0]], [[4.0, 5.0, 8.0], [6.0, 1.0, 1.0], [4.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [5.0, 2.0, 6.0], 
[6.0, 3.0, 5.0]]], [[[4.0, 3.0, 5.0], [7.0, 8.0, 6.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 2.0], [5.0, 3.0, 1.0], [4.0, 8.0, 5.0]], 
[[2.0, 1.0, 3.0], [7.0, 3.0, 6.0], [6.0, 2.0, 8.0]]], [[[6.0, 4.0, 6.0], [6.0, 5.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 7.0, 5.0], 
[5.0, 7.0, 8.0], [3.0, 3.0, 1.0]], [[6.0, 8.0, 6.0], [4.0, 3.0, 8.0], [3.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPLL(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 1.0], [8.0, 3.0, 8.0], [5.0, 4.0, 4.0]], [[3.0, 3.0, 2.0], [2.0, 7.0, 6.0], [7.0, 3.0, 
3.0]], [[7.0, 4.0, 1.0], [7.0, 8.0, 2.0], [1.0, 5.0, 2.0]]], [[[8.0, 3.0, 2.0], [5.0, 5.0, 3.0], [5.0, 3.0, 6.0]], [[8.0, 5.0, 
7.0], [1.0, 8.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 6.0], [7.0, 1.0, 4.0], [5.0, 4.0, 2.0]]], [[[3.0, 3.0, 3.0], [8.0, 1.0, 
5.0], [2.0, 7.0, 7.0]], [[8.0, 6.0, 8.0], [1.0, 4.0, 1.0], [8.0, 2.0, 8.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 2.0], [6.0, 2.0, 
4.0]]], [[[8.0, 6.0, 1.0], [2.0, 7.0, 1.0], [6.0, 7.0, 6.0]], [[5.0, 8.0, 8.0], [3.0, 6.0, 7.0], [4.0, 6.0, 3.0]], [[5.0, 8.0, 
4.0], [2.0, 6.0, 1.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[2.0]], [[8.0]]])
      arg[2:,1,2:,2:]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 1.0], [8.0, 3.0, 8.0], [5.0, 4.0, 4.0]], [[3.0, 3.0, 2.0], [2.0, 7.0, 6.0], [7.0, 3.0, 
3.0]], [[7.0, 4.0, 1.0], [7.0, 8.0, 2.0], [1.0, 5.0, 2.0]]], [[[8.0, 3.0, 2.0], [5.0, 5.0, 3.0], [5.0, 3.0, 6.0]], [[8.0, 5.0, 
7.0], [1.0, 8.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 6.0], [7.0, 1.0, 4.0], [5.0, 4.0, 2.0]]], [[[3.0, 3.0, 3.0], [8.0, 1.0, 
5.0], [2.0, 7.0, 7.0]], [[8.0, 6.0, 8.0], [1.0, 4.0, 1.0], [8.0, 2.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 2.0], [6.0, 2.0, 
4.0]]], [[[8.0, 6.0, 1.0], [2.0, 7.0, 1.0], [6.0, 7.0, 6.0]], [[5.0, 8.0, 8.0], [3.0, 6.0, 7.0], [4.0, 6.0, 8.0]], [[5.0, 8.0, 
4.0], [2.0, 6.0, 1.0], [6.0, 8.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPLL(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 2.0], [8.0, 4.0, 6.0], [1.0, 6.0, 5.0]], [[5.0, 2.0, 8.0], [8.0, 1.0, 6.0], [5.0, 4.0, 
8.0]], [[8.0, 1.0, 8.0], [4.0, 7.0, 1.0], [1.0, 8.0, 5.0]]], [[[1.0, 1.0, 4.0], [3.0, 4.0, 4.0], [3.0, 7.0, 5.0]], [[4.0, 3.0, 
7.0], [4.0, 6.0, 7.0], [4.0, 5.0, 1.0]], [[2.0, 7.0, 4.0], [3.0, 5.0, 2.0], [7.0, 5.0, 1.0]]], [[[4.0, 2.0, 7.0], [1.0, 8.0, 
3.0], [2.0, 1.0, 5.0]], [[2.0, 4.0, 8.0], [5.0, 2.0, 8.0], [1.0, 2.0, 4.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 8.0], [3.0, 5.0, 
5.0]]], [[[8.0, 1.0, 5.0], [6.0, 5.0, 4.0], [1.0, 8.0, 1.0]], [[2.0, 5.0, 6.0], [8.0, 5.0, 8.0], [5.0, 7.0, 4.0]], [[8.0, 4.0, 
5.0], [1.0, 5.0, 5.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0]], [[8.0]]]),self.functionspace)
      arg[2:,1,2:,2:]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 2.0], [8.0, 4.0, 6.0], [1.0, 6.0, 5.0]], [[5.0, 2.0, 8.0], [8.0, 1.0, 6.0], [5.0, 4.0, 
8.0]], [[8.0, 1.0, 8.0], [4.0, 7.0, 1.0], [1.0, 8.0, 5.0]]], [[[1.0, 1.0, 4.0], [3.0, 4.0, 4.0], [3.0, 7.0, 5.0]], [[4.0, 3.0, 
7.0], [4.0, 6.0, 7.0], [4.0, 5.0, 1.0]], [[2.0, 7.0, 4.0], [3.0, 5.0, 2.0], [7.0, 5.0, 1.0]]], [[[4.0, 2.0, 7.0], [1.0, 8.0, 
3.0], [2.0, 1.0, 5.0]], [[2.0, 4.0, 8.0], [5.0, 2.0, 8.0], [1.0, 2.0, 6.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 8.0], [3.0, 5.0, 
5.0]]], [[[8.0, 1.0, 5.0], [6.0, 5.0, 4.0], [1.0, 8.0, 1.0]], [[2.0, 5.0, 6.0], [8.0, 5.0, 8.0], [5.0, 7.0, 8.0]], [[8.0, 4.0, 
5.0], [1.0, 5.0, 5.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPLL(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 5.0], [5.0, 3.0, 5.0], [2.0, 2.0, 2.0]], [[2.0, 8.0, 8.0], [6.0, 8.0, 6.0], [2.0, 7.0, 
6.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 5.0], [8.0, 5.0, 2.0]]], [[[3.0, 1.0, 8.0], [7.0, 8.0, 2.0], [5.0, 4.0, 8.0]], [[4.0, 8.0, 
7.0], [4.0, 3.0, 6.0], [1.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [2.0, 5.0, 3.0], [8.0, 4.0, 1.0]]], [[[5.0, 6.0, 5.0], [1.0, 3.0, 
5.0], [2.0, 3.0, 7.0]], [[1.0, 5.0, 2.0], [2.0, 4.0, 4.0], [1.0, 1.0, 1.0]], [[5.0, 5.0, 1.0], [6.0, 1.0, 6.0], [1.0, 6.0, 
8.0]]], [[[5.0, 2.0, 3.0], [3.0, 4.0, 8.0], [8.0, 4.0, 1.0]], [[5.0, 4.0, 5.0], [1.0, 8.0, 8.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 
2.0], [4.0, 4.0, 1.0], [4.0, 7.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0]], [[7.0]]])+(1.-msk_val)*numpy.array([[[8.0]], [[2.0]]])
      arg[2:,1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 5.0], [5.0, 3.0, 5.0], [2.0, 2.0, 2.0]], [[2.0, 8.0, 8.0], [6.0, 8.0, 6.0], [2.0, 
7.0, 6.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 5.0], [8.0, 5.0, 2.0]]], [[[3.0, 1.0, 8.0], [7.0, 8.0, 2.0], [5.0, 4.0, 8.0]], [[4.0, 
8.0, 7.0], [4.0, 3.0, 6.0], [1.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [2.0, 5.0, 3.0], [8.0, 4.0, 1.0]]], [[[5.0, 6.0, 5.0], [1.0, 
3.0, 5.0], [2.0, 3.0, 7.0]], [[1.0, 5.0, 2.0], [2.0, 4.0, 4.0], [1.0, 1.0, 7.0]], [[5.0, 5.0, 1.0], [6.0, 1.0, 6.0], [1.0, 6.0, 
8.0]]], [[[5.0, 2.0, 3.0], [3.0, 4.0, 8.0], [8.0, 4.0, 1.0]], [[5.0, 4.0, 5.0], [1.0, 8.0, 8.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 
2.0], [4.0, 4.0, 1.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 5.0], [5.0, 3.0, 5.0], [2.0, 2.0, 2.0]], 
[[2.0, 8.0, 8.0], [6.0, 8.0, 6.0], [2.0, 7.0, 6.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 5.0], [8.0, 5.0, 2.0]]], [[[3.0, 1.0, 8.0], 
[7.0, 8.0, 2.0], [5.0, 4.0, 8.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 6.0], [1.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [2.0, 5.0, 3.0], 
[8.0, 4.0, 1.0]]], [[[5.0, 6.0, 5.0], [1.0, 3.0, 5.0], [2.0, 3.0, 7.0]], [[1.0, 5.0, 2.0], [2.0, 4.0, 4.0], [1.0, 1.0, 8.0]], 
[[5.0, 5.0, 1.0], [6.0, 1.0, 6.0], [1.0, 6.0, 8.0]]], [[[5.0, 2.0, 3.0], [3.0, 4.0, 8.0], [8.0, 4.0, 1.0]], [[5.0, 4.0, 5.0], 
[1.0, 8.0, 8.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 2.0], [4.0, 4.0, 1.0], [4.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CRRN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 7.0], [8.0, 2.0, 2.0], [1.0, 7.0, 4.0]], [[4.0, 2.0, 7.0], [1.0, 5.0, 5.0], [3.0, 8.0, 
6.0]], [[6.0, 7.0, 3.0], [8.0, 6.0, 7.0], [4.0, 4.0, 4.0]]], [[[1.0, 1.0, 6.0], [6.0, 5.0, 2.0], [5.0, 3.0, 1.0]], [[2.0, 4.0, 
7.0], [8.0, 5.0, 4.0], [3.0, 1.0, 1.0]], [[4.0, 6.0, 2.0], [1.0, 5.0, 5.0], [3.0, 8.0, 4.0]]], [[[7.0, 8.0, 7.0], [6.0, 7.0, 
2.0], [3.0, 3.0, 6.0]], [[1.0, 7.0, 4.0], [1.0, 6.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 4.0, 4.0], [1.0, 7.0, 2.0], [7.0, 5.0, 
3.0]]], [[[4.0, 8.0, 8.0], [6.0, 5.0, 5.0], [7.0, 3.0, 2.0]], [[5.0, 8.0, 4.0], [2.0, 1.0, 7.0], [7.0, 6.0, 3.0]], [[1.0, 5.0, 
6.0], [7.0, 2.0, 2.0], [4.0, 6.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 4.0, 7.0]]], [[[7.0, 1.0, 1.0]]], [[[3.0, 5.0, 7.0]]], [[[1.0, 8.0, 2.0]]]])
      arg[:,1:2,1:2]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 7.0], [8.0, 2.0, 2.0], [1.0, 7.0, 4.0]], [[4.0, 2.0, 7.0], [5.0, 4.0, 7.0], [3.0, 8.0, 
6.0]], [[6.0, 7.0, 3.0], [8.0, 6.0, 7.0], [4.0, 4.0, 4.0]]], [[[1.0, 1.0, 6.0], [6.0, 5.0, 2.0], [5.0, 3.0, 1.0]], [[2.0, 4.0, 
7.0], [7.0, 1.0, 1.0], [3.0, 1.0, 1.0]], [[4.0, 6.0, 2.0], [1.0, 5.0, 5.0], [3.0, 8.0, 4.0]]], [[[7.0, 8.0, 7.0], [6.0, 7.0, 
2.0], [3.0, 3.0, 6.0]], [[1.0, 7.0, 4.0], [3.0, 5.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 4.0, 4.0], [1.0, 7.0, 2.0], [7.0, 5.0, 
3.0]]], [[[4.0, 8.0, 8.0], [6.0, 5.0, 5.0], [7.0, 3.0, 2.0]], [[5.0, 8.0, 4.0], [1.0, 8.0, 2.0], [7.0, 6.0, 3.0]], [[1.0, 5.0, 
6.0], [7.0, 2.0, 2.0], [4.0, 6.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CRRN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [8.0, 3.0, 1.0], [5.0, 7.0, 1.0]], [[7.0, 2.0, 8.0], [5.0, 4.0, 6.0], [5.0, 5.0, 
6.0]], [[8.0, 8.0, 5.0], [2.0, 2.0, 7.0], [1.0, 6.0, 5.0]]], [[[2.0, 5.0, 3.0], [4.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[4.0, 5.0, 
7.0], [3.0, 8.0, 8.0], [5.0, 4.0, 1.0]], [[5.0, 1.0, 4.0], [8.0, 2.0, 1.0], [5.0, 3.0, 2.0]]], [[[6.0, 6.0, 3.0], [3.0, 8.0, 
4.0], [1.0, 2.0, 8.0]], [[6.0, 7.0, 3.0], [8.0, 3.0, 5.0], [7.0, 7.0, 4.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 3.0], [5.0, 2.0, 
5.0]]], [[[8.0, 2.0, 3.0], [4.0, 3.0, 7.0], [1.0, 8.0, 7.0]], [[5.0, 7.0, 1.0], [7.0, 7.0, 6.0], [1.0, 8.0, 5.0]], [[5.0, 1.0, 
4.0], [6.0, 6.0, 4.0], [7.0, 8.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 5.0, 1.0]]], [[[1.0, 4.0, 8.0]]], [[[8.0, 8.0, 2.0]]], [[[8.0, 1.0, 
2.0]]]]),self.functionspace)
      arg[:,1:2,1:2]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 6.0], [8.0, 3.0, 1.0], [5.0, 7.0, 1.0]], [[7.0, 2.0, 8.0], [2.0, 5.0, 1.0], [5.0, 5.0, 
6.0]], [[8.0, 8.0, 5.0], [2.0, 2.0, 7.0], [1.0, 6.0, 5.0]]], [[[2.0, 5.0, 3.0], [4.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[4.0, 5.0, 
7.0], [1.0, 4.0, 8.0], [5.0, 4.0, 1.0]], [[5.0, 1.0, 4.0], [8.0, 2.0, 1.0], [5.0, 3.0, 2.0]]], [[[6.0, 6.0, 3.0], [3.0, 8.0, 
4.0], [1.0, 2.0, 8.0]], [[6.0, 7.0, 3.0], [8.0, 8.0, 2.0], [7.0, 7.0, 4.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 3.0], [5.0, 2.0, 
5.0]]], [[[8.0, 2.0, 3.0], [4.0, 3.0, 7.0], [1.0, 8.0, 7.0]], [[5.0, 7.0, 1.0], [8.0, 1.0, 2.0], [1.0, 8.0, 5.0]], [[5.0, 1.0, 
4.0], [6.0, 6.0, 4.0], [7.0, 8.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CRRN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 7.0], [4.0, 1.0, 3.0], [7.0, 5.0, 8.0]], [[5.0, 7.0, 6.0], [8.0, 1.0, 5.0], [8.0, 2.0, 
2.0]], [[1.0, 2.0, 4.0], [1.0, 8.0, 8.0], [7.0, 2.0, 8.0]]], [[[6.0, 7.0, 3.0], [8.0, 8.0, 7.0], [8.0, 1.0, 8.0]], [[1.0, 4.0, 
7.0], [8.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 8.0, 2.0], [4.0, 5.0, 4.0]]], [[[3.0, 1.0, 6.0], [8.0, 4.0, 
2.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 8.0], [8.0, 8.0, 7.0], [6.0, 3.0, 3.0]], [[2.0, 3.0, 8.0], [4.0, 2.0, 3.0], [5.0, 2.0, 
8.0]]], [[[1.0, 1.0, 1.0], [3.0, 5.0, 5.0], [1.0, 8.0, 7.0]], [[6.0, 4.0, 4.0], [7.0, 2.0, 1.0], [5.0, 8.0, 2.0]], [[4.0, 3.0, 
5.0], [3.0, 2.0, 8.0], [5.0, 6.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 3.0, 8.0]]], [[[6.0, 4.0, 2.0]]], [[[7.0, 5.0, 4.0]]], [[[2.0, 7.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 6.0, 5.0]]], [[[2.0, 4.0, 6.0]]], [[[6.0, 8.0, 7.0]]], [[[5.0, 6.0, 4.0]]]])
      arg[:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 7.0], [4.0, 1.0, 3.0], [7.0, 5.0, 8.0]], [[5.0, 7.0, 6.0], [1.0, 3.0, 8.0], [8.0, 
2.0, 2.0]], [[1.0, 2.0, 4.0], [1.0, 8.0, 8.0], [7.0, 2.0, 8.0]]], [[[6.0, 7.0, 3.0], [8.0, 8.0, 7.0], [8.0, 1.0, 8.0]], [[1.0, 
4.0, 7.0], [6.0, 4.0, 2.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 8.0, 2.0], [4.0, 5.0, 4.0]]], [[[3.0, 1.0, 6.0], [8.0, 
4.0, 2.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 8.0], [7.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[2.0, 3.0, 8.0], [4.0, 2.0, 3.0], [5.0, 2.0, 
8.0]]], [[[1.0, 1.0, 1.0], [3.0, 5.0, 5.0], [1.0, 8.0, 7.0]], [[6.0, 4.0, 4.0], [2.0, 7.0, 7.0], [5.0, 8.0, 2.0]], [[4.0, 3.0, 
5.0], [3.0, 2.0, 8.0], [5.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 7.0], [4.0, 1.0, 3.0], [7.0, 5.0, 8.0]], 
[[5.0, 7.0, 6.0], [4.0, 6.0, 5.0], [8.0, 2.0, 2.0]], [[1.0, 2.0, 4.0], [1.0, 8.0, 8.0], [7.0, 2.0, 8.0]]], [[[6.0, 7.0, 3.0], 
[8.0, 8.0, 7.0], [8.0, 1.0, 8.0]], [[1.0, 4.0, 7.0], [2.0, 4.0, 6.0], [7.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [1.0, 8.0, 2.0], 
[4.0, 5.0, 4.0]]], [[[3.0, 1.0, 6.0], [8.0, 4.0, 2.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 8.0], [6.0, 8.0, 7.0], [6.0, 3.0, 3.0]], 
[[2.0, 3.0, 8.0], [4.0, 2.0, 3.0], [5.0, 2.0, 8.0]]], [[[1.0, 1.0, 1.0], [3.0, 5.0, 5.0], [1.0, 8.0, 7.0]], [[6.0, 4.0, 4.0], 
[5.0, 6.0, 4.0], [5.0, 8.0, 2.0]], [[4.0, 3.0, 5.0], [3.0, 2.0, 8.0], [5.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCLN(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 2.0], [2.0, 1.0, 1.0], [7.0, 4.0, 6.0]], [[8.0, 1.0, 5.0], [2.0, 8.0, 7.0], [2.0, 6.0, 
7.0]], [[7.0, 1.0, 1.0], [3.0, 5.0, 1.0], [1.0, 1.0, 3.0]]], [[[7.0, 1.0, 7.0], [4.0, 4.0, 3.0], [6.0, 4.0, 4.0]], [[6.0, 5.0, 
8.0], [7.0, 2.0, 7.0], [3.0, 7.0, 5.0]], [[7.0, 6.0, 4.0], [5.0, 2.0, 5.0], [1.0, 1.0, 6.0]]], [[[4.0, 2.0, 7.0], [3.0, 2.0, 
2.0], [3.0, 3.0, 2.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 6.0], [5.0, 6.0, 7.0]], [[6.0, 6.0, 1.0], [3.0, 8.0, 3.0], [1.0, 5.0, 
3.0]]], [[[4.0, 7.0, 8.0], [1.0, 5.0, 7.0], [7.0, 6.0, 2.0]], [[4.0, 8.0, 2.0], [4.0, 5.0, 3.0], [2.0, 6.0, 3.0]], [[1.0, 6.0, 
7.0], [1.0, 3.0, 4.0], [2.0, 2.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 5.0, 1.0]], [[2.0, 2.0, 3.0]], [[4.0, 6.0, 5.0]]], [[[6.0, 7.0, 8.0]], [[3.0, 1.0, 6.0]], [[2.0, 
7.0, 6.0]]]])
      arg[1:3,:,2:]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 2.0], [2.0, 1.0, 1.0], [7.0, 4.0, 6.0]], [[8.0, 1.0, 5.0], [2.0, 8.0, 7.0], [2.0, 6.0, 
7.0]], [[7.0, 1.0, 1.0], [3.0, 5.0, 1.0], [1.0, 1.0, 3.0]]], [[[7.0, 1.0, 7.0], [4.0, 4.0, 3.0], [7.0, 5.0, 1.0]], [[6.0, 5.0, 
8.0], [7.0, 2.0, 7.0], [2.0, 2.0, 3.0]], [[7.0, 6.0, 4.0], [5.0, 2.0, 5.0], [4.0, 6.0, 5.0]]], [[[4.0, 2.0, 7.0], [3.0, 2.0, 
2.0], [6.0, 7.0, 8.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 6.0], [3.0, 1.0, 6.0]], [[6.0, 6.0, 1.0], [3.0, 8.0, 3.0], [2.0, 7.0, 
6.0]]], [[[4.0, 7.0, 8.0], [1.0, 5.0, 7.0], [7.0, 6.0, 2.0]], [[4.0, 8.0, 2.0], [4.0, 5.0, 3.0], [2.0, 6.0, 3.0]], [[1.0, 6.0, 
7.0], [1.0, 3.0, 4.0], [2.0, 2.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCLN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 8.0], [1.0, 7.0, 8.0], [1.0, 3.0, 7.0]], [[8.0, 7.0, 8.0], [1.0, 4.0, 6.0], [4.0, 8.0, 
6.0]], [[1.0, 6.0, 8.0], [1.0, 8.0, 8.0], [1.0, 8.0, 1.0]]], [[[2.0, 8.0, 8.0], [1.0, 7.0, 7.0], [2.0, 1.0, 2.0]], [[2.0, 6.0, 
3.0], [4.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[2.0, 2.0, 3.0], [2.0, 6.0, 5.0], [6.0, 2.0, 6.0]]], [[[6.0, 3.0, 6.0], [1.0, 5.0, 
6.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 4.0], [5.0, 8.0, 2.0], [2.0, 3.0, 7.0]], [[3.0, 1.0, 2.0], [1.0, 1.0, 6.0], [7.0, 7.0, 
7.0]]], [[[1.0, 1.0, 8.0], [1.0, 4.0, 1.0], [7.0, 7.0, 6.0]], [[8.0, 5.0, 4.0], [5.0, 5.0, 5.0], [6.0, 7.0, 1.0]], [[4.0, 7.0, 
1.0], [6.0, 3.0, 3.0], [8.0, 6.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 2.0, 1.0]], [[8.0, 3.0, 8.0]], [[4.0, 2.0, 8.0]]], [[[8.0, 1.0, 4.0]], [[1.0, 2.0, 7.0]], 
[[4.0, 3.0, 5.0]]]]),self.functionspace)
      arg[1:3,:,2:]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 8.0], [1.0, 7.0, 8.0], [1.0, 3.0, 7.0]], [[8.0, 7.0, 8.0], [1.0, 4.0, 6.0], [4.0, 8.0, 
6.0]], [[1.0, 6.0, 8.0], [1.0, 8.0, 8.0], [1.0, 8.0, 1.0]]], [[[2.0, 8.0, 8.0], [1.0, 7.0, 7.0], [4.0, 2.0, 1.0]], [[2.0, 6.0, 
3.0], [4.0, 4.0, 8.0], [8.0, 3.0, 8.0]], [[2.0, 2.0, 3.0], [2.0, 6.0, 5.0], [4.0, 2.0, 8.0]]], [[[6.0, 3.0, 6.0], [1.0, 5.0, 
6.0], [8.0, 1.0, 4.0]], [[3.0, 5.0, 4.0], [5.0, 8.0, 2.0], [1.0, 2.0, 7.0]], [[3.0, 1.0, 2.0], [1.0, 1.0, 6.0], [4.0, 3.0, 
5.0]]], [[[1.0, 1.0, 8.0], [1.0, 4.0, 1.0], [7.0, 7.0, 6.0]], [[8.0, 5.0, 4.0], [5.0, 5.0, 5.0], [6.0, 7.0, 1.0]], [[4.0, 7.0, 
1.0], [6.0, 3.0, 3.0], [8.0, 6.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCLN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 4.0], [5.0, 1.0, 1.0], [1.0, 6.0, 4.0]], [[5.0, 8.0, 4.0], [7.0, 8.0, 4.0], [5.0, 7.0, 
7.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 1.0], [6.0, 7.0, 4.0]]], [[[4.0, 8.0, 8.0], [7.0, 7.0, 8.0], [2.0, 2.0, 4.0]], [[4.0, 1.0, 
4.0], [3.0, 8.0, 1.0], [8.0, 6.0, 2.0]], [[7.0, 7.0, 8.0], [4.0, 2.0, 8.0], [4.0, 7.0, 2.0]]], [[[5.0, 6.0, 7.0], [3.0, 7.0, 
8.0], [1.0, 8.0, 4.0]], [[3.0, 3.0, 2.0], [6.0, 8.0, 4.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 6.0], [2.0, 7.0, 
8.0]]], [[[3.0, 6.0, 3.0], [3.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[8.0, 4.0, 3.0], [1.0, 7.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 5.0, 
7.0], [2.0, 8.0, 4.0], [7.0, 5.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 7.0, 3.0]], [[1.0, 1.0, 4.0]], [[4.0, 8.0, 5.0]]], [[[5.0, 7.0, 7.0]], [[3.0, 5.0, 
5.0]], [[5.0, 3.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 4.0, 3.0]], [[3.0, 4.0, 3.0]], [[2.0, 2.0, 7.0]]], [[[8.0, 3.0, 
1.0]], [[1.0, 1.0, 5.0]], [[1.0, 8.0, 8.0]]]])
      arg[1:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 1.0, 1.0], [1.0, 6.0, 4.0]], [[5.0, 8.0, 4.0], [7.0, 8.0, 4.0], [5.0, 
7.0, 7.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 1.0], [6.0, 7.0, 4.0]]], [[[4.0, 8.0, 8.0], [7.0, 7.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 
1.0, 4.0], [3.0, 8.0, 1.0], [1.0, 1.0, 4.0]], [[7.0, 7.0, 8.0], [4.0, 2.0, 8.0], [4.0, 8.0, 5.0]]], [[[5.0, 6.0, 7.0], [3.0, 
7.0, 8.0], [5.0, 7.0, 7.0]], [[3.0, 3.0, 2.0], [6.0, 8.0, 4.0], [3.0, 5.0, 5.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 6.0], [5.0, 3.0, 
7.0]]], [[[3.0, 6.0, 3.0], [3.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[8.0, 4.0, 3.0], [1.0, 7.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 5.0, 
7.0], [2.0, 8.0, 4.0], [7.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 1.0, 1.0], [1.0, 6.0, 4.0]], 
[[5.0, 8.0, 4.0], [7.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 1.0], [6.0, 7.0, 4.0]]], [[[4.0, 8.0, 8.0], 
[7.0, 7.0, 8.0], [6.0, 4.0, 3.0]], [[4.0, 1.0, 4.0], [3.0, 8.0, 1.0], [3.0, 4.0, 3.0]], [[7.0, 7.0, 8.0], [4.0, 2.0, 8.0], 
[2.0, 2.0, 7.0]]], [[[5.0, 6.0, 7.0], [3.0, 7.0, 8.0], [8.0, 3.0, 1.0]], [[3.0, 3.0, 2.0], [6.0, 8.0, 4.0], [1.0, 1.0, 5.0]], 
[[5.0, 1.0, 2.0], [2.0, 1.0, 6.0], [1.0, 8.0, 8.0]]], [[[3.0, 6.0, 3.0], [3.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[8.0, 4.0, 3.0], 
[1.0, 7.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 5.0, 7.0], [2.0, 8.0, 4.0], [7.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUCN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 7.0], [1.0, 7.0, 6.0], [5.0, 8.0, 8.0]], [[3.0, 5.0, 2.0], [1.0, 3.0, 4.0], [2.0, 2.0, 
7.0]], [[3.0, 6.0, 3.0], [1.0, 2.0, 7.0], [1.0, 6.0, 1.0]]], [[[8.0, 3.0, 2.0], [6.0, 4.0, 3.0], [7.0, 1.0, 8.0]], [[4.0, 8.0, 
6.0], [1.0, 5.0, 4.0], [7.0, 6.0, 2.0]], [[7.0, 7.0, 4.0], [4.0, 3.0, 4.0], [7.0, 1.0, 5.0]]], [[[6.0, 4.0, 4.0], [8.0, 1.0, 
1.0], [6.0, 4.0, 4.0]], [[6.0, 7.0, 5.0], [7.0, 8.0, 5.0], [6.0, 7.0, 7.0]], [[5.0, 1.0, 5.0], [3.0, 2.0, 1.0], [3.0, 3.0, 
8.0]]], [[[8.0, 4.0, 1.0], [1.0, 5.0, 7.0], [2.0, 7.0, 4.0]], [[3.0, 1.0, 1.0], [3.0, 1.0, 1.0], [3.0, 1.0, 1.0]], [[1.0, 4.0, 
2.0], [1.0, 8.0, 6.0], [4.0, 7.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 4.0, 4.0], [7.0, 1.0, 5.0], [1.0, 7.0, 2.0]], [[7.0, 1.0, 1.0], [8.0, 2.0, 7.0], [6.0, 3.0, 
2.0]]], [[[2.0, 2.0, 3.0], [6.0, 3.0, 8.0], [6.0, 8.0, 2.0]], [[7.0, 6.0, 3.0], [2.0, 5.0, 5.0], [3.0, 1.0, 5.0]]]])
      arg[1:3,:2,:]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 7.0], [1.0, 7.0, 6.0], [5.0, 8.0, 8.0]], [[3.0, 5.0, 2.0], [1.0, 3.0, 4.0], [2.0, 2.0, 
7.0]], [[3.0, 6.0, 3.0], [1.0, 2.0, 7.0], [1.0, 6.0, 1.0]]], [[[4.0, 4.0, 4.0], [7.0, 1.0, 5.0], [1.0, 7.0, 2.0]], [[7.0, 1.0, 
1.0], [8.0, 2.0, 7.0], [6.0, 3.0, 2.0]], [[7.0, 7.0, 4.0], [4.0, 3.0, 4.0], [7.0, 1.0, 5.0]]], [[[2.0, 2.0, 3.0], [6.0, 3.0, 
8.0], [6.0, 8.0, 2.0]], [[7.0, 6.0, 3.0], [2.0, 5.0, 5.0], [3.0, 1.0, 5.0]], [[5.0, 1.0, 5.0], [3.0, 2.0, 1.0], [3.0, 3.0, 
8.0]]], [[[8.0, 4.0, 1.0], [1.0, 5.0, 7.0], [2.0, 7.0, 4.0]], [[3.0, 1.0, 1.0], [3.0, 1.0, 1.0], [3.0, 1.0, 1.0]], [[1.0, 4.0, 
2.0], [1.0, 8.0, 6.0], [4.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUCN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 8.0], [4.0, 7.0, 6.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 1.0], [3.0, 8.0, 1.0], [6.0, 1.0, 
4.0]], [[1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [7.0, 8.0, 8.0]]], [[[5.0, 4.0, 6.0], [8.0, 6.0, 4.0], [6.0, 6.0, 8.0]], [[1.0, 6.0, 
7.0], [6.0, 5.0, 5.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 6.0], [2.0, 3.0, 5.0], [6.0, 5.0, 1.0]]], [[[5.0, 5.0, 4.0], [8.0, 8.0, 
5.0], [8.0, 3.0, 1.0]], [[5.0, 7.0, 1.0], [1.0, 1.0, 7.0], [7.0, 1.0, 7.0]], [[2.0, 2.0, 1.0], [6.0, 7.0, 7.0], [1.0, 4.0, 
1.0]]], [[[4.0, 6.0, 7.0], [8.0, 1.0, 5.0], [1.0, 3.0, 7.0]], [[6.0, 3.0, 5.0], [5.0, 7.0, 3.0], [2.0, 8.0, 4.0]], [[3.0, 3.0, 
6.0], [6.0, 5.0, 1.0], [8.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 8.0, 2.0], [6.0, 3.0, 5.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 4.0], [6.0, 1.0, 1.0], [7.0, 2.0, 
2.0]]], [[[5.0, 7.0, 6.0], [1.0, 4.0, 2.0], [7.0, 7.0, 2.0]], [[4.0, 3.0, 8.0], [8.0, 1.0, 1.0], [2.0, 5.0, 
3.0]]]]),self.functionspace)
      arg[1:3,:2,:]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 8.0], [4.0, 7.0, 6.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 1.0], [3.0, 8.0, 1.0], [6.0, 1.0, 
4.0]], [[1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [7.0, 8.0, 8.0]]], [[[3.0, 8.0, 2.0], [6.0, 3.0, 5.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 
4.0], [6.0, 1.0, 1.0], [7.0, 2.0, 2.0]], [[5.0, 7.0, 6.0], [2.0, 3.0, 5.0], [6.0, 5.0, 1.0]]], [[[5.0, 7.0, 6.0], [1.0, 4.0, 
2.0], [7.0, 7.0, 2.0]], [[4.0, 3.0, 8.0], [8.0, 1.0, 1.0], [2.0, 5.0, 3.0]], [[2.0, 2.0, 1.0], [6.0, 7.0, 7.0], [1.0, 4.0, 
1.0]]], [[[4.0, 6.0, 7.0], [8.0, 1.0, 5.0], [1.0, 3.0, 7.0]], [[6.0, 3.0, 5.0], [5.0, 7.0, 3.0], [2.0, 8.0, 4.0]], [[3.0, 3.0, 
6.0], [6.0, 5.0, 1.0], [8.0, 8.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUCN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 3.0], [8.0, 5.0, 5.0], [2.0, 1.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 1.0, 1.0], [4.0, 2.0, 
4.0]], [[7.0, 1.0, 5.0], [1.0, 8.0, 3.0], [1.0, 5.0, 2.0]]], [[[8.0, 7.0, 3.0], [2.0, 4.0, 8.0], [2.0, 6.0, 8.0]], [[7.0, 6.0, 
5.0], [6.0, 1.0, 8.0], [6.0, 3.0, 8.0]], [[3.0, 7.0, 4.0], [6.0, 2.0, 5.0], [6.0, 8.0, 6.0]]], [[[2.0, 1.0, 6.0], [1.0, 2.0, 
3.0], [3.0, 3.0, 7.0]], [[7.0, 4.0, 1.0], [4.0, 3.0, 7.0], [2.0, 6.0, 6.0]], [[4.0, 8.0, 7.0], [6.0, 7.0, 6.0], [8.0, 2.0, 
5.0]]], [[[7.0, 8.0, 4.0], [8.0, 3.0, 8.0], [2.0, 8.0, 4.0]], [[6.0, 6.0, 6.0], [8.0, 7.0, 6.0], [6.0, 7.0, 4.0]], [[8.0, 2.0, 
8.0], [2.0, 1.0, 4.0], [4.0, 5.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 2.0, 8.0], [3.0, 8.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 8.0, 5.0], [6.0, 
1.0, 5.0]]], [[[5.0, 7.0, 3.0], [8.0, 4.0, 7.0], [4.0, 6.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 3.0, 4.0], [6.0, 7.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 5.0, 7.0], [3.0, 3.0, 8.0], [3.0, 4.0, 2.0]], [[2.0, 6.0, 4.0], [8.0, 1.0, 7.0], 
[6.0, 6.0, 2.0]]], [[[3.0, 6.0, 5.0], [1.0, 7.0, 8.0], [7.0, 4.0, 8.0]], [[4.0, 1.0, 4.0], [3.0, 7.0, 7.0], [7.0, 1.0, 3.0]]]])
      arg[1:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 3.0], [8.0, 5.0, 5.0], [2.0, 1.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 1.0, 1.0], [4.0, 
2.0, 4.0]], [[7.0, 1.0, 5.0], [1.0, 8.0, 3.0], [1.0, 5.0, 2.0]]], [[[6.0, 2.0, 7.0], [4.0, 2.0, 8.0], [3.0, 8.0, 4.0]], [[1.0, 
6.0, 4.0], [4.0, 8.0, 5.0], [6.0, 1.0, 5.0]], [[3.0, 7.0, 4.0], [6.0, 2.0, 5.0], [6.0, 8.0, 6.0]]], [[[5.0, 7.0, 3.0], [8.0, 
4.0, 7.0], [4.0, 6.0, 7.0]], [[6.0, 6.0, 8.0], [4.0, 3.0, 4.0], [6.0, 7.0, 7.0]], [[4.0, 8.0, 7.0], [6.0, 7.0, 6.0], [8.0, 2.0, 
5.0]]], [[[7.0, 8.0, 4.0], [8.0, 3.0, 8.0], [2.0, 8.0, 4.0]], [[6.0, 6.0, 6.0], [8.0, 7.0, 6.0], [6.0, 7.0, 4.0]], [[8.0, 2.0, 
8.0], [2.0, 1.0, 4.0], [4.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 3.0], [8.0, 5.0, 5.0], [2.0, 1.0, 6.0]], 
[[3.0, 1.0, 2.0], [6.0, 1.0, 1.0], [4.0, 2.0, 4.0]], [[7.0, 1.0, 5.0], [1.0, 8.0, 3.0], [1.0, 5.0, 2.0]]], [[[8.0, 5.0, 7.0], 
[3.0, 3.0, 8.0], [3.0, 4.0, 2.0]], [[2.0, 6.0, 4.0], [8.0, 1.0, 7.0], [6.0, 6.0, 2.0]], [[3.0, 7.0, 4.0], [6.0, 2.0, 5.0], 
[6.0, 8.0, 6.0]]], [[[3.0, 6.0, 5.0], [1.0, 7.0, 8.0], [7.0, 4.0, 8.0]], [[4.0, 1.0, 4.0], [3.0, 7.0, 7.0], [7.0, 1.0, 3.0]], 
[[4.0, 8.0, 7.0], [6.0, 7.0, 6.0], [8.0, 2.0, 5.0]]], [[[7.0, 8.0, 4.0], [8.0, 3.0, 8.0], [2.0, 8.0, 4.0]], [[6.0, 6.0, 6.0], 
[8.0, 7.0, 6.0], [6.0, 7.0, 4.0]], [[8.0, 2.0, 8.0], [2.0, 1.0, 4.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCLL(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 2.0], [1.0, 2.0, 8.0], [1.0, 2.0, 5.0]], [[1.0, 5.0, 8.0], [4.0, 3.0, 4.0], [8.0, 7.0, 
6.0]], [[6.0, 3.0, 8.0], [5.0, 7.0, 8.0], [6.0, 2.0, 7.0]]], [[[5.0, 3.0, 2.0], [2.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[1.0, 1.0, 
2.0], [5.0, 7.0, 6.0], [4.0, 6.0, 6.0]], [[3.0, 5.0, 3.0], [3.0, 5.0, 3.0], [2.0, 7.0, 5.0]]], [[[4.0, 5.0, 6.0], [8.0, 5.0, 
1.0], [1.0, 7.0, 3.0]], [[6.0, 4.0, 5.0], [8.0, 1.0, 2.0], [4.0, 2.0, 2.0]], [[4.0, 3.0, 1.0], [8.0, 2.0, 7.0], [7.0, 6.0, 
6.0]]], [[[6.0, 3.0, 1.0], [4.0, 4.0, 7.0], [7.0, 5.0, 4.0]], [[1.0, 3.0, 2.0], [1.0, 1.0, 8.0], [1.0, 8.0, 4.0]], [[8.0, 2.0, 
1.0], [5.0, 1.0, 8.0], [3.0, 2.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0]], [[7.0]], [[5.0]]], [[[2.0]], [[7.0]], [[6.0]]], [[[2.0]], [[7.0]], [[4.0]]], [[[2.0]], 
[[3.0]], [[4.0]]]])
      arg[:,:,2:,2:]=val
      ref=Data(numpy.array([[[[7.0, 6.0, 2.0], [1.0, 2.0, 8.0], [1.0, 2.0, 2.0]], [[1.0, 5.0, 8.0], [4.0, 3.0, 4.0], [8.0, 7.0, 
7.0]], [[6.0, 3.0, 8.0], [5.0, 7.0, 8.0], [6.0, 2.0, 5.0]]], [[[5.0, 3.0, 2.0], [2.0, 3.0, 6.0], [2.0, 3.0, 2.0]], [[1.0, 1.0, 
2.0], [5.0, 7.0, 6.0], [4.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [3.0, 5.0, 3.0], [2.0, 7.0, 6.0]]], [[[4.0, 5.0, 6.0], [8.0, 5.0, 
1.0], [1.0, 7.0, 2.0]], [[6.0, 4.0, 5.0], [8.0, 1.0, 2.0], [4.0, 2.0, 7.0]], [[4.0, 3.0, 1.0], [8.0, 2.0, 7.0], [7.0, 6.0, 
4.0]]], [[[6.0, 3.0, 1.0], [4.0, 4.0, 7.0], [7.0, 5.0, 2.0]], [[1.0, 3.0, 2.0], [1.0, 1.0, 8.0], [1.0, 8.0, 3.0]], [[8.0, 2.0, 
1.0], [5.0, 1.0, 8.0], [3.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCLL(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [8.0, 4.0, 7.0], [4.0, 4.0, 3.0]], [[4.0, 2.0, 1.0], [4.0, 3.0, 2.0], [1.0, 6.0, 
2.0]], [[2.0, 8.0, 4.0], [2.0, 3.0, 3.0], [4.0, 8.0, 1.0]]], [[[7.0, 5.0, 2.0], [4.0, 8.0, 3.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 
7.0], [8.0, 5.0, 5.0], [4.0, 2.0, 5.0]], [[7.0, 5.0, 5.0], [6.0, 8.0, 4.0], [1.0, 7.0, 4.0]]], [[[5.0, 2.0, 8.0], [4.0, 4.0, 
4.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 4.0], [6.0, 7.0, 4.0], [2.0, 7.0, 1.0]], [[2.0, 2.0, 6.0], [7.0, 7.0, 1.0], [6.0, 5.0, 
7.0]]], [[[5.0, 3.0, 8.0], [6.0, 3.0, 3.0], [3.0, 4.0, 6.0]], [[7.0, 5.0, 6.0], [5.0, 6.0, 2.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 
4.0], [7.0, 4.0, 1.0], [5.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0]], [[3.0]], [[7.0]]], [[[1.0]], [[2.0]], [[1.0]]], [[[8.0]], [[1.0]], [[8.0]]], [[[5.0]], 
[[7.0]], [[3.0]]]]),self.functionspace)
      arg[:,:,2:,2:]=val
      ref=Data(numpy.array([[[[5.0, 2.0, 8.0], [8.0, 4.0, 7.0], [4.0, 4.0, 3.0]], [[4.0, 2.0, 1.0], [4.0, 3.0, 2.0], [1.0, 6.0, 
3.0]], [[2.0, 8.0, 4.0], [2.0, 3.0, 3.0], [4.0, 8.0, 7.0]]], [[[7.0, 5.0, 2.0], [4.0, 8.0, 3.0], [5.0, 2.0, 1.0]], [[1.0, 3.0, 
7.0], [8.0, 5.0, 5.0], [4.0, 2.0, 2.0]], [[7.0, 5.0, 5.0], [6.0, 8.0, 4.0], [1.0, 7.0, 1.0]]], [[[5.0, 2.0, 8.0], [4.0, 4.0, 
4.0], [1.0, 5.0, 8.0]], [[5.0, 7.0, 4.0], [6.0, 7.0, 4.0], [2.0, 7.0, 1.0]], [[2.0, 2.0, 6.0], [7.0, 7.0, 1.0], [6.0, 5.0, 
8.0]]], [[[5.0, 3.0, 8.0], [6.0, 3.0, 3.0], [3.0, 4.0, 5.0]], [[7.0, 5.0, 6.0], [5.0, 6.0, 2.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 
4.0], [7.0, 4.0, 1.0], [5.0, 2.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCLL(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 1.0], [2.0, 7.0, 1.0], [8.0, 3.0, 4.0]], [[1.0, 7.0, 3.0], [7.0, 7.0, 7.0], [1.0, 8.0, 
7.0]], [[5.0, 3.0, 1.0], [4.0, 2.0, 7.0], [5.0, 5.0, 4.0]]], [[[3.0, 5.0, 5.0], [8.0, 2.0, 2.0], [6.0, 5.0, 6.0]], [[8.0, 1.0, 
8.0], [5.0, 3.0, 2.0], [6.0, 4.0, 3.0]], [[5.0, 1.0, 5.0], [1.0, 1.0, 5.0], [3.0, 7.0, 2.0]]], [[[6.0, 3.0, 3.0], [7.0, 1.0, 
6.0], [4.0, 6.0, 7.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 4.0], [3.0, 1.0, 3.0]], [[7.0, 1.0, 4.0], [3.0, 5.0, 8.0], [6.0, 1.0, 
4.0]]], [[[5.0, 7.0, 5.0], [6.0, 8.0, 2.0], [1.0, 5.0, 3.0]], [[3.0, 5.0, 8.0], [3.0, 6.0, 4.0], [5.0, 7.0, 2.0]], [[1.0, 5.0, 
1.0], [4.0, 5.0, 1.0], [8.0, 4.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0]], [[3.0]], [[2.0]]], [[[4.0]], [[1.0]], [[8.0]]], [[[4.0]], [[5.0]], [[3.0]]], [[[4.0]], 
[[1.0]], [[2.0]]]])+(1.-msk_val)*numpy.array([[[[1.0]], [[3.0]], [[7.0]]], [[[3.0]], [[4.0]], [[8.0]]], [[[2.0]], [[7.0]], 
[[4.0]]], [[[4.0]], [[5.0]], [[8.0]]]])
      arg[:,:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 1.0], [2.0, 7.0, 1.0], [8.0, 3.0, 8.0]], [[1.0, 7.0, 3.0], [7.0, 7.0, 7.0], [1.0, 
8.0, 3.0]], [[5.0, 3.0, 1.0], [4.0, 2.0, 7.0], [5.0, 5.0, 2.0]]], [[[3.0, 5.0, 5.0], [8.0, 2.0, 2.0], [6.0, 5.0, 4.0]], [[8.0, 
1.0, 8.0], [5.0, 3.0, 2.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 5.0], [1.0, 1.0, 5.0], [3.0, 7.0, 8.0]]], [[[6.0, 3.0, 3.0], [7.0, 
1.0, 6.0], [4.0, 6.0, 4.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 4.0], [3.0, 1.0, 5.0]], [[7.0, 1.0, 4.0], [3.0, 5.0, 8.0], [6.0, 1.0, 
3.0]]], [[[5.0, 7.0, 5.0], [6.0, 8.0, 2.0], [1.0, 5.0, 4.0]], [[3.0, 5.0, 8.0], [3.0, 6.0, 4.0], [5.0, 7.0, 1.0]], [[1.0, 5.0, 
1.0], [4.0, 5.0, 1.0], [8.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 1.0], [2.0, 7.0, 1.0], [8.0, 3.0, 1.0]], 
[[1.0, 7.0, 3.0], [7.0, 7.0, 7.0], [1.0, 8.0, 3.0]], [[5.0, 3.0, 1.0], [4.0, 2.0, 7.0], [5.0, 5.0, 7.0]]], [[[3.0, 5.0, 5.0], 
[8.0, 2.0, 2.0], [6.0, 5.0, 3.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 2.0], [6.0, 4.0, 4.0]], [[5.0, 1.0, 5.0], [1.0, 1.0, 5.0], 
[3.0, 7.0, 8.0]]], [[[6.0, 3.0, 3.0], [7.0, 1.0, 6.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 4.0], [3.0, 1.0, 7.0]], 
[[7.0, 1.0, 4.0], [3.0, 5.0, 8.0], [6.0, 1.0, 4.0]]], [[[5.0, 7.0, 5.0], [6.0, 8.0, 2.0], [1.0, 5.0, 4.0]], [[3.0, 5.0, 8.0], 
[3.0, 6.0, 4.0], [5.0, 7.0, 5.0]], [[1.0, 5.0, 1.0], [4.0, 5.0, 1.0], [8.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPPP(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [5.0, 6.0, 7.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 3.0], [5.0, 2.0, 6.0], [6.0, 4.0, 
8.0]], [[2.0, 5.0, 4.0], [8.0, 8.0, 7.0], [8.0, 3.0, 2.0]]], [[[8.0, 8.0, 4.0], [4.0, 4.0, 7.0], [6.0, 7.0, 2.0]], [[7.0, 2.0, 
7.0], [4.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[8.0, 6.0, 8.0], [4.0, 4.0, 6.0], [1.0, 8.0, 4.0]]], [[[8.0, 3.0, 3.0], [4.0, 5.0, 
3.0], [4.0, 8.0, 3.0]], [[6.0, 3.0, 3.0], [6.0, 2.0, 1.0], [8.0, 5.0, 4.0]], [[7.0, 8.0, 1.0], [3.0, 8.0, 3.0], [3.0, 4.0, 
5.0]]], [[[6.0, 3.0, 5.0], [8.0, 2.0, 8.0], [8.0, 1.0, 4.0]], [[7.0, 8.0, 6.0], [5.0, 2.0, 1.0], [8.0, 1.0, 2.0]], [[5.0, 3.0, 
7.0], [6.0, 4.0, 6.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      val=numpy.array([3.0, 4.0])
      arg[2:,1,1,1]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 7.0], [5.0, 6.0, 7.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 3.0], [5.0, 2.0, 6.0], [6.0, 4.0, 
8.0]], [[2.0, 5.0, 4.0], [8.0, 8.0, 7.0], [8.0, 3.0, 2.0]]], [[[8.0, 8.0, 4.0], [4.0, 4.0, 7.0], [6.0, 7.0, 2.0]], [[7.0, 2.0, 
7.0], [4.0, 4.0, 5.0], [5.0, 4.0, 2.0]], [[8.0, 6.0, 8.0], [4.0, 4.0, 6.0], [1.0, 8.0, 4.0]]], [[[8.0, 3.0, 3.0], [4.0, 5.0, 
3.0], [4.0, 8.0, 3.0]], [[6.0, 3.0, 3.0], [6.0, 3.0, 1.0], [8.0, 5.0, 4.0]], [[7.0, 8.0, 1.0], [3.0, 8.0, 3.0], [3.0, 4.0, 
5.0]]], [[[6.0, 3.0, 5.0], [8.0, 2.0, 8.0], [8.0, 1.0, 4.0]], [[7.0, 8.0, 6.0], [5.0, 4.0, 1.0], [8.0, 1.0, 2.0]], [[5.0, 3.0, 
7.0], [6.0, 4.0, 6.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPPP(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 5.0], [4.0, 1.0, 7.0], [4.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [7.0, 5.0, 4.0], [4.0, 2.0, 
6.0]], [[3.0, 7.0, 4.0], [6.0, 3.0, 2.0], [5.0, 3.0, 3.0]]], [[[4.0, 2.0, 4.0], [1.0, 2.0, 1.0], [6.0, 8.0, 4.0]], [[3.0, 1.0, 
8.0], [4.0, 7.0, 1.0], [3.0, 8.0, 4.0]], [[3.0, 2.0, 3.0], [4.0, 5.0, 3.0], [7.0, 6.0, 7.0]]], [[[2.0, 1.0, 7.0], [6.0, 6.0, 
8.0], [4.0, 1.0, 6.0]], [[5.0, 3.0, 2.0], [4.0, 1.0, 2.0], [6.0, 4.0, 4.0]], [[1.0, 4.0, 4.0], [2.0, 3.0, 8.0], [1.0, 1.0, 
8.0]]], [[[8.0, 7.0, 7.0], [3.0, 7.0, 1.0], [2.0, 7.0, 7.0]], [[7.0, 5.0, 6.0], [3.0, 8.0, 6.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 
5.0], [3.0, 7.0, 7.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([2.0, 2.0]),self.functionspace)
      arg[2:,1,1,1]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 5.0], [4.0, 1.0, 7.0], [4.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [7.0, 5.0, 4.0], [4.0, 2.0, 
6.0]], [[3.0, 7.0, 4.0], [6.0, 3.0, 2.0], [5.0, 3.0, 3.0]]], [[[4.0, 2.0, 4.0], [1.0, 2.0, 1.0], [6.0, 8.0, 4.0]], [[3.0, 1.0, 
8.0], [4.0, 7.0, 1.0], [3.0, 8.0, 4.0]], [[3.0, 2.0, 3.0], [4.0, 5.0, 3.0], [7.0, 6.0, 7.0]]], [[[2.0, 1.0, 7.0], [6.0, 6.0, 
8.0], [4.0, 1.0, 6.0]], [[5.0, 3.0, 2.0], [4.0, 2.0, 2.0], [6.0, 4.0, 4.0]], [[1.0, 4.0, 4.0], [2.0, 3.0, 8.0], [1.0, 1.0, 
8.0]]], [[[8.0, 7.0, 7.0], [3.0, 7.0, 1.0], [2.0, 7.0, 7.0]], [[7.0, 5.0, 6.0], [3.0, 2.0, 6.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 
5.0], [3.0, 7.0, 7.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPPP(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 7.0], [6.0, 7.0, 6.0], [1.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], [1.0, 5.0, 2.0], [5.0, 8.0, 
5.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 2.0], [6.0, 4.0, 2.0]]], [[[4.0, 8.0, 8.0], [3.0, 5.0, 1.0], [5.0, 5.0, 7.0]], [[2.0, 5.0, 
6.0], [7.0, 1.0, 3.0], [3.0, 8.0, 2.0]], [[7.0, 4.0, 1.0], [5.0, 1.0, 8.0], [7.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [6.0, 3.0, 
8.0], [6.0, 8.0, 3.0]], [[7.0, 5.0, 1.0], [7.0, 7.0, 2.0], [1.0, 5.0, 5.0]], [[3.0, 7.0, 7.0], [1.0, 6.0, 5.0], [5.0, 4.0, 
5.0]]], [[[7.0, 5.0, 7.0], [3.0, 3.0, 3.0], [4.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [8.0, 6.0, 8.0], [7.0, 2.0, 2.0]], [[8.0, 8.0, 
1.0], [5.0, 3.0, 7.0], [3.0, 8.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0, 5.0])+(1.-msk_val)*numpy.array([7.0, 6.0])
      arg[2:,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 7.0], [6.0, 7.0, 6.0], [1.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], [1.0, 5.0, 2.0], [5.0, 
8.0, 5.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 2.0], [6.0, 4.0, 2.0]]], [[[4.0, 8.0, 8.0], [3.0, 5.0, 1.0], [5.0, 5.0, 7.0]], [[2.0, 
5.0, 6.0], [7.0, 1.0, 3.0], [3.0, 8.0, 2.0]], [[7.0, 4.0, 1.0], [5.0, 1.0, 8.0], [7.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [6.0, 
3.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 5.0, 1.0], [7.0, 5.0, 2.0], [1.0, 5.0, 5.0]], [[3.0, 7.0, 7.0], [1.0, 6.0, 5.0], [5.0, 4.0, 
5.0]]], [[[7.0, 5.0, 7.0], [3.0, 3.0, 3.0], [4.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [8.0, 5.0, 8.0], [7.0, 2.0, 2.0]], [[8.0, 8.0, 
1.0], [5.0, 3.0, 7.0], [3.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [6.0, 7.0, 6.0], [1.0, 8.0, 4.0]], 
[[5.0, 6.0, 8.0], [1.0, 5.0, 2.0], [5.0, 8.0, 5.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 2.0], [6.0, 4.0, 2.0]]], [[[4.0, 8.0, 8.0], 
[3.0, 5.0, 1.0], [5.0, 5.0, 7.0]], [[2.0, 5.0, 6.0], [7.0, 1.0, 3.0], [3.0, 8.0, 2.0]], [[7.0, 4.0, 1.0], [5.0, 1.0, 8.0], 
[7.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [6.0, 3.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 5.0, 1.0], [7.0, 7.0, 2.0], [1.0, 5.0, 5.0]], 
[[3.0, 7.0, 7.0], [1.0, 6.0, 5.0], [5.0, 4.0, 5.0]]], [[[7.0, 5.0, 7.0], [3.0, 3.0, 3.0], [4.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], 
[8.0, 6.0, 8.0], [7.0, 2.0, 2.0]], [[8.0, 8.0, 1.0], [5.0, 3.0, 7.0], [3.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PUUU(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 1.0], [8.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 8.0], [3.0, 6.0, 1.0], [2.0, 7.0, 
2.0]], [[1.0, 2.0, 2.0], [7.0, 1.0, 8.0], [2.0, 8.0, 3.0]]], [[[7.0, 7.0, 2.0], [5.0, 5.0, 5.0], [3.0, 6.0, 5.0]], [[4.0, 3.0, 
3.0], [3.0, 4.0, 5.0], [3.0, 6.0, 1.0]], [[3.0, 2.0, 6.0], [5.0, 3.0, 6.0], [7.0, 8.0, 1.0]]], [[[2.0, 3.0, 5.0], [4.0, 8.0, 
4.0], [4.0, 3.0, 1.0]], [[2.0, 1.0, 4.0], [5.0, 8.0, 1.0], [3.0, 2.0, 2.0]], [[2.0, 6.0, 6.0], [7.0, 8.0, 4.0], [2.0, 8.0, 
5.0]]], [[[2.0, 7.0, 6.0], [5.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[1.0, 5.0, 1.0], [7.0, 5.0, 7.0], [3.0, 6.0, 2.0]], [[8.0, 7.0, 
8.0], [7.0, 2.0, 6.0], [1.0, 6.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[2.0, 2.0], [2.0, 6.0]], [[3.0, 3.0], [7.0, 6.0]]])
      arg[2,:2,:2,:2]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 1.0], [8.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 8.0], [3.0, 6.0, 1.0], [2.0, 7.0, 
2.0]], [[1.0, 2.0, 2.0], [7.0, 1.0, 8.0], [2.0, 8.0, 3.0]]], [[[7.0, 7.0, 2.0], [5.0, 5.0, 5.0], [3.0, 6.0, 5.0]], [[4.0, 3.0, 
3.0], [3.0, 4.0, 5.0], [3.0, 6.0, 1.0]], [[3.0, 2.0, 6.0], [5.0, 3.0, 6.0], [7.0, 8.0, 1.0]]], [[[2.0, 2.0, 5.0], [2.0, 6.0, 
4.0], [4.0, 3.0, 1.0]], [[3.0, 3.0, 4.0], [7.0, 6.0, 1.0], [3.0, 2.0, 2.0]], [[2.0, 6.0, 6.0], [7.0, 8.0, 4.0], [2.0, 8.0, 
5.0]]], [[[2.0, 7.0, 6.0], [5.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[1.0, 5.0, 1.0], [7.0, 5.0, 7.0], [3.0, 6.0, 2.0]], [[8.0, 7.0, 
8.0], [7.0, 2.0, 6.0], [1.0, 6.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PUUU(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 1.0], [4.0, 6.0, 3.0], [3.0, 3.0, 4.0]], [[3.0, 4.0, 3.0], [3.0, 2.0, 4.0], [2.0, 8.0, 
7.0]], [[6.0, 2.0, 5.0], [6.0, 6.0, 3.0], [6.0, 7.0, 8.0]]], [[[7.0, 8.0, 2.0], [5.0, 7.0, 7.0], [3.0, 5.0, 6.0]], [[1.0, 1.0, 
1.0], [5.0, 8.0, 7.0], [4.0, 5.0, 4.0]], [[1.0, 2.0, 5.0], [6.0, 4.0, 1.0], [2.0, 6.0, 7.0]]], [[[3.0, 2.0, 8.0], [6.0, 3.0, 
5.0], [2.0, 2.0, 6.0]], [[5.0, 1.0, 3.0], [6.0, 6.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 4.0, 4.0], [8.0, 6.0, 1.0], [7.0, 8.0, 
3.0]]], [[[7.0, 7.0, 4.0], [1.0, 1.0, 5.0], [4.0, 3.0, 3.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 5.0], [8.0, 8.0, 7.0]], [[5.0, 6.0, 
6.0], [6.0, 3.0, 2.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 7.0], [5.0, 6.0]], [[1.0, 8.0], [1.0, 4.0]]]),self.functionspace)
      arg[2,:2,:2,:2]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 1.0], [4.0, 6.0, 3.0], [3.0, 3.0, 4.0]], [[3.0, 4.0, 3.0], [3.0, 2.0, 4.0], [2.0, 8.0, 
7.0]], [[6.0, 2.0, 5.0], [6.0, 6.0, 3.0], [6.0, 7.0, 8.0]]], [[[7.0, 8.0, 2.0], [5.0, 7.0, 7.0], [3.0, 5.0, 6.0]], [[1.0, 1.0, 
1.0], [5.0, 8.0, 7.0], [4.0, 5.0, 4.0]], [[1.0, 2.0, 5.0], [6.0, 4.0, 1.0], [2.0, 6.0, 7.0]]], [[[6.0, 7.0, 8.0], [5.0, 6.0, 
5.0], [2.0, 2.0, 6.0]], [[1.0, 8.0, 3.0], [1.0, 4.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 4.0, 4.0], [8.0, 6.0, 1.0], [7.0, 8.0, 
3.0]]], [[[7.0, 7.0, 4.0], [1.0, 1.0, 5.0], [4.0, 3.0, 3.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 5.0], [8.0, 8.0, 7.0]], [[5.0, 6.0, 
6.0], [6.0, 3.0, 2.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PUUU(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 7.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]], [[3.0, 8.0, 5.0], [3.0, 6.0, 1.0], [5.0, 7.0, 
3.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 8.0], [1.0, 3.0, 6.0]]], [[[8.0, 8.0, 4.0], [4.0, 8.0, 8.0], [2.0, 8.0, 6.0]], [[3.0, 2.0, 
6.0], [7.0, 8.0, 1.0], [6.0, 2.0, 7.0]], [[3.0, 4.0, 2.0], [3.0, 3.0, 2.0], [2.0, 6.0, 5.0]]], [[[6.0, 4.0, 6.0], [6.0, 4.0, 
1.0], [1.0, 6.0, 4.0]], [[6.0, 2.0, 7.0], [7.0, 7.0, 1.0], [2.0, 3.0, 1.0]], [[3.0, 6.0, 3.0], [1.0, 7.0, 4.0], [3.0, 3.0, 
8.0]]], [[[3.0, 4.0, 3.0], [7.0, 3.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 5.0, 3.0], [4.0, 5.0, 4.0], [2.0, 7.0, 8.0]], [[6.0, 6.0, 
7.0], [1.0, 3.0, 1.0], [1.0, 1.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 7.0], [2.0, 8.0]], [[3.0, 6.0], [8.0, 5.0]]])+(1.-msk_val)*numpy.array([[[2.0, 1.0], 
[3.0, 4.0]], [[7.0, 1.0], [6.0, 2.0]]])
      arg[2,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 7.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]], [[3.0, 8.0, 5.0], [3.0, 6.0, 1.0], [5.0, 
7.0, 3.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 8.0], [1.0, 3.0, 6.0]]], [[[8.0, 8.0, 4.0], [4.0, 8.0, 8.0], [2.0, 8.0, 6.0]], [[3.0, 
2.0, 6.0], [7.0, 8.0, 1.0], [6.0, 2.0, 7.0]], [[3.0, 4.0, 2.0], [3.0, 3.0, 2.0], [2.0, 6.0, 5.0]]], [[[5.0, 7.0, 6.0], [2.0, 
8.0, 1.0], [1.0, 6.0, 4.0]], [[3.0, 6.0, 7.0], [8.0, 5.0, 1.0], [2.0, 3.0, 1.0]], [[3.0, 6.0, 3.0], [1.0, 7.0, 4.0], [3.0, 3.0, 
8.0]]], [[[3.0, 4.0, 3.0], [7.0, 3.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 5.0, 3.0], [4.0, 5.0, 4.0], [2.0, 7.0, 8.0]], [[6.0, 6.0, 
7.0], [1.0, 3.0, 1.0], [1.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 7.0], [6.0, 4.0, 2.0], [6.0, 8.0, 7.0]], 
[[3.0, 8.0, 5.0], [3.0, 6.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 8.0], [1.0, 3.0, 6.0]]], [[[8.0, 8.0, 4.0], 
[4.0, 8.0, 8.0], [2.0, 8.0, 6.0]], [[3.0, 2.0, 6.0], [7.0, 8.0, 1.0], [6.0, 2.0, 7.0]], [[3.0, 4.0, 2.0], [3.0, 3.0, 2.0], 
[2.0, 6.0, 5.0]]], [[[2.0, 1.0, 6.0], [3.0, 4.0, 1.0], [1.0, 6.0, 4.0]], [[7.0, 1.0, 7.0], [6.0, 2.0, 1.0], [2.0, 3.0, 1.0]], 
[[3.0, 6.0, 3.0], [1.0, 7.0, 4.0], [3.0, 3.0, 8.0]]], [[[3.0, 4.0, 3.0], [7.0, 3.0, 6.0], [4.0, 4.0, 6.0]], [[7.0, 5.0, 3.0], 
[4.0, 5.0, 4.0], [2.0, 7.0, 8.0]], [[6.0, 6.0, 7.0], [1.0, 3.0, 1.0], [1.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 7.0], [3.0, 5.0, 8.0], [3.0, 6.0, 8.0]], [[5.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 3.0, 
1.0]], [[4.0, 7.0, 3.0], [6.0, 7.0, 3.0], [1.0, 1.0, 6.0]]], [[[4.0, 1.0, 1.0], [5.0, 7.0, 7.0], [4.0, 1.0, 8.0]], [[2.0, 7.0, 
8.0], [2.0, 5.0, 4.0], [8.0, 3.0, 4.0]], [[7.0, 7.0, 7.0], [5.0, 7.0, 7.0], [3.0, 5.0, 8.0]]], [[[1.0, 4.0, 8.0], [7.0, 2.0, 
2.0], [3.0, 4.0, 7.0]], [[3.0, 2.0, 3.0], [1.0, 2.0, 1.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 8.0], [8.0, 5.0, 7.0], [5.0, 3.0, 
1.0]]], [[[3.0, 3.0, 8.0], [2.0, 6.0, 2.0], [3.0, 1.0, 7.0]], [[3.0, 4.0, 3.0], [1.0, 2.0, 1.0], [5.0, 4.0, 2.0]], [[1.0, 3.0, 
5.0], [3.0, 3.0, 5.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 8.0, 4.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 3.0, 3.0]], [[1.0, 3.0, 3.0], [4.0, 3.0, 
5.0]]])
      arg[2,:,:2]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 7.0], [3.0, 5.0, 8.0], [3.0, 6.0, 8.0]], [[5.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 3.0, 
1.0]], [[4.0, 7.0, 3.0], [6.0, 7.0, 3.0], [1.0, 1.0, 6.0]]], [[[4.0, 1.0, 1.0], [5.0, 7.0, 7.0], [4.0, 1.0, 8.0]], [[2.0, 7.0, 
8.0], [2.0, 5.0, 4.0], [8.0, 3.0, 4.0]], [[7.0, 7.0, 7.0], [5.0, 7.0, 7.0], [3.0, 5.0, 8.0]]], [[[4.0, 8.0, 4.0], [8.0, 2.0, 
5.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 8.0], [6.0, 3.0, 3.0], [6.0, 4.0, 1.0]], [[1.0, 3.0, 3.0], [4.0, 3.0, 5.0], [5.0, 3.0, 
1.0]]], [[[3.0, 3.0, 8.0], [2.0, 6.0, 2.0], [3.0, 1.0, 7.0]], [[3.0, 4.0, 3.0], [1.0, 2.0, 1.0], [5.0, 4.0, 2.0]], [[1.0, 3.0, 
5.0], [3.0, 3.0, 5.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCUN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 2.0], [7.0, 1.0, 7.0], [3.0, 4.0, 5.0]], [[8.0, 6.0, 8.0], [5.0, 3.0, 2.0], [4.0, 3.0, 
2.0]], [[2.0, 3.0, 1.0], [6.0, 2.0, 1.0], [1.0, 2.0, 3.0]]], [[[2.0, 3.0, 5.0], [8.0, 8.0, 7.0], [8.0, 2.0, 3.0]], [[3.0, 3.0, 
3.0], [8.0, 4.0, 8.0], [8.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 8.0], [2.0, 2.0, 1.0]]], [[[2.0, 4.0, 8.0], [1.0, 3.0, 
1.0], [5.0, 1.0, 1.0]], [[7.0, 2.0, 1.0], [4.0, 2.0, 6.0], [2.0, 1.0, 7.0]], [[4.0, 3.0, 5.0], [2.0, 2.0, 6.0], [7.0, 3.0, 
7.0]]], [[[8.0, 4.0, 4.0], [1.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[8.0, 7.0, 5.0], [1.0, 7.0, 7.0], [3.0, 2.0, 7.0]], [[7.0, 8.0, 
3.0], [6.0, 7.0, 1.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 8.0, 8.0], [5.0, 8.0, 2.0]], [[7.0, 3.0, 6.0], [8.0, 3.0, 6.0]], [[7.0, 3.0, 5.0], [1.0, 
3.0, 4.0]]]),self.functionspace)
      arg[2,:,:2]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 2.0], [7.0, 1.0, 7.0], [3.0, 4.0, 5.0]], [[8.0, 6.0, 8.0], [5.0, 3.0, 2.0], [4.0, 3.0, 
2.0]], [[2.0, 3.0, 1.0], [6.0, 2.0, 1.0], [1.0, 2.0, 3.0]]], [[[2.0, 3.0, 5.0], [8.0, 8.0, 7.0], [8.0, 2.0, 3.0]], [[3.0, 3.0, 
3.0], [8.0, 4.0, 8.0], [8.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [6.0, 8.0, 8.0], [2.0, 2.0, 1.0]]], [[[5.0, 8.0, 8.0], [5.0, 8.0, 
2.0], [5.0, 1.0, 1.0]], [[7.0, 3.0, 6.0], [8.0, 3.0, 6.0], [2.0, 1.0, 7.0]], [[7.0, 3.0, 5.0], [1.0, 3.0, 4.0], [7.0, 3.0, 
7.0]]], [[[8.0, 4.0, 4.0], [1.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[8.0, 7.0, 5.0], [1.0, 7.0, 7.0], [3.0, 2.0, 7.0]], [[7.0, 8.0, 
3.0], [6.0, 7.0, 1.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCUN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 5.0], [1.0, 2.0, 3.0], [6.0, 1.0, 6.0]], [[5.0, 6.0, 2.0], [1.0, 4.0, 8.0], [8.0, 1.0, 
5.0]], [[1.0, 6.0, 1.0], [2.0, 5.0, 2.0], [7.0, 1.0, 5.0]]], [[[2.0, 6.0, 3.0], [2.0, 3.0, 5.0], [3.0, 8.0, 4.0]], [[7.0, 5.0, 
8.0], [8.0, 1.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 2.0], [2.0, 1.0, 6.0]]], [[[3.0, 6.0, 3.0], [8.0, 4.0, 
4.0], [7.0, 2.0, 4.0]], [[6.0, 6.0, 7.0], [7.0, 1.0, 2.0], [5.0, 3.0, 3.0]], [[1.0, 8.0, 1.0], [5.0, 3.0, 6.0], [6.0, 6.0, 
4.0]]], [[[4.0, 3.0, 4.0], [2.0, 8.0, 6.0], [1.0, 2.0, 7.0]], [[8.0, 8.0, 2.0], [8.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 8.0, 
4.0], [1.0, 7.0, 2.0], [6.0, 5.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 7.0, 4.0], [8.0, 8.0, 6.0]], [[4.0, 6.0, 7.0], [6.0, 6.0, 3.0]], [[2.0, 4.0, 6.0], [5.0, 
6.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 7.0, 6.0], [4.0, 3.0, 1.0]], [[2.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[6.0, 5.0, 
1.0], [1.0, 7.0, 2.0]]])
      arg[2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 5.0], [1.0, 2.0, 3.0], [6.0, 1.0, 6.0]], [[5.0, 6.0, 2.0], [1.0, 4.0, 8.0], [8.0, 
1.0, 5.0]], [[1.0, 6.0, 1.0], [2.0, 5.0, 2.0], [7.0, 1.0, 5.0]]], [[[2.0, 6.0, 3.0], [2.0, 3.0, 5.0], [3.0, 8.0, 4.0]], [[7.0, 
5.0, 8.0], [8.0, 1.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 2.0], [2.0, 1.0, 6.0]]], [[[7.0, 7.0, 4.0], [8.0, 
8.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 6.0, 7.0], [6.0, 6.0, 3.0], [5.0, 3.0, 3.0]], [[2.0, 4.0, 6.0], [5.0, 6.0, 3.0], [6.0, 6.0, 
4.0]]], [[[4.0, 3.0, 4.0], [2.0, 8.0, 6.0], [1.0, 2.0, 7.0]], [[8.0, 8.0, 2.0], [8.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 8.0, 
4.0], [1.0, 7.0, 2.0], [6.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 5.0], [1.0, 2.0, 3.0], [6.0, 1.0, 6.0]], 
[[5.0, 6.0, 2.0], [1.0, 4.0, 8.0], [8.0, 1.0, 5.0]], [[1.0, 6.0, 1.0], [2.0, 5.0, 2.0], [7.0, 1.0, 5.0]]], [[[2.0, 6.0, 3.0], 
[2.0, 3.0, 5.0], [3.0, 8.0, 4.0]], [[7.0, 5.0, 8.0], [8.0, 1.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 7.0, 1.0], [8.0, 7.0, 2.0], 
[2.0, 1.0, 6.0]]], [[[1.0, 7.0, 6.0], [4.0, 3.0, 1.0], [7.0, 2.0, 4.0]], [[2.0, 1.0, 2.0], [5.0, 5.0, 2.0], [5.0, 3.0, 3.0]], 
[[6.0, 5.0, 1.0], [1.0, 7.0, 2.0], [6.0, 6.0, 4.0]]], [[[4.0, 3.0, 4.0], [2.0, 8.0, 6.0], [1.0, 2.0, 7.0]], [[8.0, 8.0, 2.0], 
[8.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 8.0, 4.0], [1.0, 7.0, 2.0], [6.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCCC(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 4.0], [1.0, 3.0, 2.0], [5.0, 5.0, 4.0]], [[4.0, 1.0, 5.0], [2.0, 7.0, 6.0], [3.0, 5.0, 
8.0]], [[2.0, 7.0, 8.0], [6.0, 6.0, 6.0], [1.0, 3.0, 5.0]]], [[[4.0, 2.0, 4.0], [2.0, 1.0, 3.0], [3.0, 4.0, 5.0]], [[4.0, 3.0, 
1.0], [3.0, 5.0, 1.0], [8.0, 2.0, 4.0]], [[1.0, 7.0, 3.0], [1.0, 8.0, 2.0], [4.0, 1.0, 8.0]]], [[[8.0, 8.0, 1.0], [6.0, 6.0, 
7.0], [5.0, 1.0, 6.0]], [[4.0, 7.0, 6.0], [1.0, 7.0, 2.0], [6.0, 5.0, 5.0]], [[3.0, 3.0, 8.0], [5.0, 3.0, 2.0], [8.0, 5.0, 
4.0]]], [[[2.0, 2.0, 1.0], [4.0, 5.0, 6.0], [4.0, 5.0, 8.0]], [[6.0, 1.0, 1.0], [6.0, 5.0, 8.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 
5.0], [6.0, 1.0, 6.0], [1.0, 8.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 3.0, 7.0], [3.0, 5.0, 6.0], [8.0, 4.0, 8.0]], [[3.0, 8.0, 3.0], [7.0, 3.0, 3.0], [7.0, 6.0, 
5.0]], [[8.0, 8.0, 2.0], [4.0, 4.0, 7.0], [7.0, 4.0, 7.0]]], [[[3.0, 5.0, 2.0], [3.0, 3.0, 5.0], [2.0, 4.0, 5.0]], [[4.0, 3.0, 
6.0], [1.0, 6.0, 5.0], [4.0, 7.0, 8.0]], [[7.0, 7.0, 7.0], [8.0, 4.0, 4.0], [1.0, 7.0, 6.0]]], [[[3.0, 6.0, 6.0], [6.0, 2.0, 
8.0], [2.0, 6.0, 1.0]], [[4.0, 5.0, 3.0], [6.0, 3.0, 4.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 5.0], [7.0, 5.0, 6.0], [5.0, 8.0, 
7.0]]]])
      arg[:3,:,:,:]=val
      ref=Data(numpy.array([[[[1.0, 3.0, 7.0], [3.0, 5.0, 6.0], [8.0, 4.0, 8.0]], [[3.0, 8.0, 3.0], [7.0, 3.0, 3.0], [7.0, 6.0, 
5.0]], [[8.0, 8.0, 2.0], [4.0, 4.0, 7.0], [7.0, 4.0, 7.0]]], [[[3.0, 5.0, 2.0], [3.0, 3.0, 5.0], [2.0, 4.0, 5.0]], [[4.0, 3.0, 
6.0], [1.0, 6.0, 5.0], [4.0, 7.0, 8.0]], [[7.0, 7.0, 7.0], [8.0, 4.0, 4.0], [1.0, 7.0, 6.0]]], [[[3.0, 6.0, 6.0], [6.0, 2.0, 
8.0], [2.0, 6.0, 1.0]], [[4.0, 5.0, 3.0], [6.0, 3.0, 4.0], [4.0, 7.0, 5.0]], [[8.0, 6.0, 5.0], [7.0, 5.0, 6.0], [5.0, 8.0, 
7.0]]], [[[2.0, 2.0, 1.0], [4.0, 5.0, 6.0], [4.0, 5.0, 8.0]], [[6.0, 1.0, 1.0], [6.0, 5.0, 8.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 
5.0], [6.0, 1.0, 6.0], [1.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCCC(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 5.0], [8.0, 7.0, 7.0], [7.0, 3.0, 8.0]], [[5.0, 7.0, 6.0], [2.0, 7.0, 6.0], [7.0, 1.0, 
6.0]], [[3.0, 3.0, 4.0], [5.0, 4.0, 5.0], [7.0, 7.0, 3.0]]], [[[2.0, 7.0, 3.0], [6.0, 8.0, 2.0], [5.0, 7.0, 7.0]], [[3.0, 6.0, 
3.0], [4.0, 7.0, 8.0], [6.0, 1.0, 3.0]], [[4.0, 4.0, 5.0], [7.0, 7.0, 1.0], [6.0, 3.0, 3.0]]], [[[3.0, 5.0, 4.0], [2.0, 4.0, 
4.0], [6.0, 1.0, 8.0]], [[5.0, 2.0, 5.0], [3.0, 5.0, 5.0], [7.0, 4.0, 7.0]], [[4.0, 6.0, 6.0], [7.0, 3.0, 4.0], [6.0, 8.0, 
1.0]]], [[[5.0, 7.0, 2.0], [8.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 8.0], [5.0, 6.0, 3.0], [8.0, 4.0, 2.0]], [[7.0, 3.0, 
3.0], [6.0, 5.0, 6.0], [4.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 1.0, 6.0], [6.0, 3.0, 4.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 5.0], [7.0, 1.0, 4.0], [4.0, 2.0, 
3.0]], [[4.0, 3.0, 1.0], [4.0, 3.0, 6.0], [2.0, 7.0, 8.0]]], [[[7.0, 8.0, 7.0], [3.0, 5.0, 5.0], [1.0, 6.0, 4.0]], [[7.0, 3.0, 
2.0], [7.0, 6.0, 1.0], [7.0, 3.0, 8.0]], [[6.0, 2.0, 8.0], [8.0, 7.0, 1.0], [8.0, 7.0, 8.0]]], [[[2.0, 6.0, 2.0], [8.0, 8.0, 
8.0], [2.0, 2.0, 2.0]], [[6.0, 8.0, 3.0], [2.0, 3.0, 4.0], [8.0, 6.0, 7.0]], [[6.0, 8.0, 1.0], [8.0, 5.0, 6.0], [6.0, 3.0, 
8.0]]]]),self.functionspace)
      arg[:3,:,:,:]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 6.0], [6.0, 3.0, 4.0], [4.0, 8.0, 2.0]], [[6.0, 3.0, 5.0], [7.0, 1.0, 4.0], [4.0, 2.0, 
3.0]], [[4.0, 3.0, 1.0], [4.0, 3.0, 6.0], [2.0, 7.0, 8.0]]], [[[7.0, 8.0, 7.0], [3.0, 5.0, 5.0], [1.0, 6.0, 4.0]], [[7.0, 3.0, 
2.0], [7.0, 6.0, 1.0], [7.0, 3.0, 8.0]], [[6.0, 2.0, 8.0], [8.0, 7.0, 1.0], [8.0, 7.0, 8.0]]], [[[2.0, 6.0, 2.0], [8.0, 8.0, 
8.0], [2.0, 2.0, 2.0]], [[6.0, 8.0, 3.0], [2.0, 3.0, 4.0], [8.0, 6.0, 7.0]], [[6.0, 8.0, 1.0], [8.0, 5.0, 6.0], [6.0, 3.0, 
8.0]]], [[[5.0, 7.0, 2.0], [8.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 8.0], [5.0, 6.0, 3.0], [8.0, 4.0, 2.0]], [[7.0, 3.0, 
3.0], [6.0, 5.0, 6.0], [4.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCCC(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 6.0], [3.0, 4.0, 8.0], [2.0, 4.0, 7.0]], [[7.0, 7.0, 1.0], [2.0, 6.0, 1.0], [1.0, 2.0, 
1.0]], [[3.0, 5.0, 1.0], [5.0, 8.0, 6.0], [3.0, 4.0, 5.0]]], [[[5.0, 1.0, 1.0], [1.0, 8.0, 2.0], [8.0, 4.0, 4.0]], [[4.0, 1.0, 
6.0], [2.0, 7.0, 4.0], [8.0, 6.0, 3.0]], [[8.0, 7.0, 5.0], [4.0, 7.0, 1.0], [2.0, 7.0, 7.0]]], [[[6.0, 7.0, 1.0], [4.0, 8.0, 
2.0], [2.0, 2.0, 8.0]], [[6.0, 1.0, 5.0], [2.0, 2.0, 6.0], [6.0, 5.0, 1.0]], [[3.0, 6.0, 5.0], [8.0, 3.0, 1.0], [7.0, 8.0, 
1.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[7.0, 6.0, 6.0], [3.0, 5.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 1.0, 
1.0], [5.0, 1.0, 6.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 8.0, 8.0], [6.0, 8.0, 7.0], [2.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [7.0, 6.0, 2.0], [8.0, 
7.0, 7.0]], [[7.0, 2.0, 4.0], [8.0, 7.0, 3.0], [3.0, 6.0, 4.0]]], [[[2.0, 5.0, 8.0], [1.0, 5.0, 8.0], [3.0, 1.0, 6.0]], [[5.0, 
1.0, 5.0], [5.0, 8.0, 5.0], [6.0, 2.0, 4.0]], [[6.0, 7.0, 8.0], [4.0, 3.0, 8.0], [6.0, 7.0, 8.0]]], [[[6.0, 8.0, 1.0], [1.0, 
8.0, 7.0], [1.0, 1.0, 3.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 5.0], [6.0, 3.0, 8.0]], [[3.0, 6.0, 3.0], [7.0, 3.0, 4.0], [5.0, 3.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 8.0, 1.0], [5.0, 3.0, 1.0], [7.0, 1.0, 2.0]], [[7.0, 4.0, 3.0], [2.0, 5.0, 1.0], 
[5.0, 7.0, 4.0]], [[6.0, 5.0, 3.0], [7.0, 5.0, 2.0], [6.0, 5.0, 8.0]]], [[[7.0, 6.0, 6.0], [6.0, 3.0, 5.0], [5.0, 7.0, 6.0]], 
[[7.0, 7.0, 5.0], [7.0, 4.0, 1.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 8.0], [7.0, 1.0, 7.0], [3.0, 4.0, 8.0]]], [[[7.0, 5.0, 3.0], 
[4.0, 2.0, 8.0], [8.0, 8.0, 8.0]], [[8.0, 3.0, 5.0], [8.0, 6.0, 6.0], [7.0, 7.0, 8.0]], [[3.0, 3.0, 4.0], [2.0, 8.0, 3.0], 
[4.0, 3.0, 5.0]]]])
      arg[:3,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 8.0], [6.0, 8.0, 7.0], [2.0, 3.0, 4.0]], [[7.0, 7.0, 8.0], [7.0, 6.0, 2.0], [8.0, 
7.0, 7.0]], [[7.0, 2.0, 4.0], [8.0, 7.0, 3.0], [3.0, 6.0, 4.0]]], [[[2.0, 5.0, 8.0], [1.0, 5.0, 8.0], [3.0, 1.0, 6.0]], [[5.0, 
1.0, 5.0], [5.0, 8.0, 5.0], [6.0, 2.0, 4.0]], [[6.0, 7.0, 8.0], [4.0, 3.0, 8.0], [6.0, 7.0, 8.0]]], [[[6.0, 8.0, 1.0], [1.0, 
8.0, 7.0], [1.0, 1.0, 3.0]], [[3.0, 2.0, 3.0], [2.0, 4.0, 5.0], [6.0, 3.0, 8.0]], [[3.0, 6.0, 3.0], [7.0, 3.0, 4.0], [5.0, 3.0, 
1.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[7.0, 6.0, 6.0], [3.0, 5.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 1.0, 
1.0], [5.0, 1.0, 6.0], [3.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [5.0, 3.0, 1.0], [7.0, 1.0, 2.0]], 
[[7.0, 4.0, 3.0], [2.0, 5.0, 1.0], [5.0, 7.0, 4.0]], [[6.0, 5.0, 3.0], [7.0, 5.0, 2.0], [6.0, 5.0, 8.0]]], [[[7.0, 6.0, 6.0], 
[6.0, 3.0, 5.0], [5.0, 7.0, 6.0]], [[7.0, 7.0, 5.0], [7.0, 4.0, 1.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 8.0], [7.0, 1.0, 7.0], 
[3.0, 4.0, 8.0]]], [[[7.0, 5.0, 3.0], [4.0, 2.0, 8.0], [8.0, 8.0, 8.0]], [[8.0, 3.0, 5.0], [8.0, 6.0, 6.0], [7.0, 7.0, 8.0]], 
[[3.0, 3.0, 4.0], [2.0, 8.0, 3.0], [4.0, 3.0, 5.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[7.0, 6.0, 6.0], 
[3.0, 5.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 1.0, 1.0], [5.0, 1.0, 6.0], [3.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCCC(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 8.0], [2.0, 2.0, 3.0], [1.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 8.0, 5.0], [5.0, 1.0, 
6.0]], [[2.0, 1.0, 5.0], [7.0, 3.0, 5.0], [6.0, 7.0, 7.0]]], [[[2.0, 8.0, 4.0], [4.0, 4.0, 8.0], [6.0, 5.0, 2.0]], [[3.0, 3.0, 
1.0], [8.0, 1.0, 8.0], [7.0, 6.0, 1.0]], [[2.0, 5.0, 7.0], [5.0, 2.0, 3.0], [7.0, 2.0, 6.0]]], [[[7.0, 8.0, 7.0], [7.0, 8.0, 
5.0], [7.0, 5.0, 3.0]], [[4.0, 1.0, 7.0], [2.0, 4.0, 7.0], [1.0, 1.0, 7.0]], [[5.0, 5.0, 5.0], [2.0, 6.0, 1.0], [4.0, 5.0, 
6.0]]], [[[1.0, 7.0, 7.0], [5.0, 8.0, 4.0], [2.0, 8.0, 7.0]], [[2.0, 4.0, 7.0], [5.0, 8.0, 3.0], [6.0, 4.0, 7.0]], [[4.0, 2.0, 
2.0], [3.0, 5.0, 3.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 8.0, 5.0], [2.0, 4.0, 8.0], [4.0, 8.0, 2.0]], [[4.0, 2.0, 2.0], [2.0, 1.0, 6.0], [3.0, 7.0, 
2.0]], [[1.0, 2.0, 8.0], [6.0, 3.0, 3.0], [5.0, 4.0, 8.0]]])
      arg[2,:,:,:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 8.0], [2.0, 2.0, 3.0], [1.0, 5.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 8.0, 5.0], [5.0, 1.0, 
6.0]], [[2.0, 1.0, 5.0], [7.0, 3.0, 5.0], [6.0, 7.0, 7.0]]], [[[2.0, 8.0, 4.0], [4.0, 4.0, 8.0], [6.0, 5.0, 2.0]], [[3.0, 3.0, 
1.0], [8.0, 1.0, 8.0], [7.0, 6.0, 1.0]], [[2.0, 5.0, 7.0], [5.0, 2.0, 3.0], [7.0, 2.0, 6.0]]], [[[7.0, 8.0, 5.0], [2.0, 4.0, 
8.0], [4.0, 8.0, 2.0]], [[4.0, 2.0, 2.0], [2.0, 1.0, 6.0], [3.0, 7.0, 2.0]], [[1.0, 2.0, 8.0], [6.0, 3.0, 3.0], [5.0, 4.0, 
8.0]]], [[[1.0, 7.0, 7.0], [5.0, 8.0, 4.0], [2.0, 8.0, 7.0]], [[2.0, 4.0, 7.0], [5.0, 8.0, 3.0], [6.0, 4.0, 7.0]], [[4.0, 2.0, 
2.0], [3.0, 5.0, 3.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCCC(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 4.0], [5.0, 6.0, 4.0], [7.0, 6.0, 1.0]], [[6.0, 5.0, 4.0], [7.0, 3.0, 1.0], [5.0, 4.0, 
6.0]], [[8.0, 3.0, 8.0], [2.0, 7.0, 4.0], [7.0, 4.0, 7.0]]], [[[6.0, 6.0, 7.0], [7.0, 5.0, 7.0], [6.0, 3.0, 7.0]], [[6.0, 1.0, 
1.0], [5.0, 6.0, 7.0], [2.0, 8.0, 6.0]], [[4.0, 4.0, 6.0], [8.0, 2.0, 7.0], [1.0, 6.0, 1.0]]], [[[3.0, 8.0, 7.0], [5.0, 5.0, 
8.0], [3.0, 6.0, 7.0]], [[5.0, 7.0, 3.0], [5.0, 8.0, 2.0], [7.0, 8.0, 6.0]], [[7.0, 3.0, 4.0], [1.0, 3.0, 1.0], [2.0, 2.0, 
8.0]]], [[[7.0, 8.0, 3.0], [7.0, 8.0, 2.0], [4.0, 8.0, 1.0]], [[6.0, 5.0, 1.0], [2.0, 7.0, 8.0], [3.0, 2.0, 4.0]], [[3.0, 7.0, 
2.0], [3.0, 7.0, 5.0], [7.0, 5.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 2.0, 8.0], [7.0, 7.0, 6.0], [1.0, 3.0, 4.0]], [[7.0, 1.0, 3.0], [4.0, 5.0, 5.0], [7.0, 4.0, 
5.0]], [[8.0, 2.0, 3.0], [6.0, 2.0, 5.0], [4.0, 4.0, 5.0]]]),self.functionspace)
      arg[2,:,:,:]=val
      ref=Data(numpy.array([[[[8.0, 7.0, 4.0], [5.0, 6.0, 4.0], [7.0, 6.0, 1.0]], [[6.0, 5.0, 4.0], [7.0, 3.0, 1.0], [5.0, 4.0, 
6.0]], [[8.0, 3.0, 8.0], [2.0, 7.0, 4.0], [7.0, 4.0, 7.0]]], [[[6.0, 6.0, 7.0], [7.0, 5.0, 7.0], [6.0, 3.0, 7.0]], [[6.0, 1.0, 
1.0], [5.0, 6.0, 7.0], [2.0, 8.0, 6.0]], [[4.0, 4.0, 6.0], [8.0, 2.0, 7.0], [1.0, 6.0, 1.0]]], [[[7.0, 2.0, 8.0], [7.0, 7.0, 
6.0], [1.0, 3.0, 4.0]], [[7.0, 1.0, 3.0], [4.0, 5.0, 5.0], [7.0, 4.0, 5.0]], [[8.0, 2.0, 3.0], [6.0, 2.0, 5.0], [4.0, 4.0, 
5.0]]], [[[7.0, 8.0, 3.0], [7.0, 8.0, 2.0], [4.0, 8.0, 1.0]], [[6.0, 5.0, 1.0], [2.0, 7.0, 8.0], [3.0, 2.0, 4.0]], [[3.0, 7.0, 
2.0], [3.0, 7.0, 5.0], [7.0, 5.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCCC(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 5.0], [1.0, 5.0, 7.0], [5.0, 4.0, 7.0]], [[1.0, 4.0, 1.0], [7.0, 6.0, 6.0], [4.0, 2.0, 
2.0]], [[8.0, 1.0, 6.0], [6.0, 3.0, 4.0], [2.0, 4.0, 6.0]]], [[[7.0, 8.0, 5.0], [1.0, 8.0, 3.0], [3.0, 1.0, 6.0]], [[4.0, 6.0, 
4.0], [6.0, 6.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 5.0, 2.0], [3.0, 4.0, 4.0], [2.0, 8.0, 3.0]]], [[[7.0, 2.0, 3.0], [6.0, 3.0, 
5.0], [8.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [3.0, 2.0, 5.0], [4.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [2.0, 4.0, 5.0], [5.0, 7.0, 
1.0]]], [[[3.0, 4.0, 7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 8.0]], [[7.0, 3.0, 1.0], [1.0, 4.0, 3.0], [6.0, 1.0, 1.0]], [[8.0, 8.0, 
8.0], [4.0, 6.0, 6.0], [2.0, 4.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 6.0, 8.0], [7.0, 6.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 7.0, 2.0], [8.0, 7.0, 2.0], [4.0, 
6.0, 2.0]], [[5.0, 2.0, 1.0], [5.0, 3.0, 5.0], [2.0, 2.0, 8.0]]])+(1.-msk_val)*numpy.array([[[2.0, 3.0, 4.0], [1.0, 2.0, 2.0], 
[6.0, 5.0, 3.0]], [[3.0, 3.0, 2.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]], [[4.0, 1.0, 2.0], [5.0, 8.0, 7.0], [4.0, 4.0, 1.0]]])
      arg[2,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 5.0], [1.0, 5.0, 7.0], [5.0, 4.0, 7.0]], [[1.0, 4.0, 1.0], [7.0, 6.0, 6.0], [4.0, 
2.0, 2.0]], [[8.0, 1.0, 6.0], [6.0, 3.0, 4.0], [2.0, 4.0, 6.0]]], [[[7.0, 8.0, 5.0], [1.0, 8.0, 3.0], [3.0, 1.0, 6.0]], [[4.0, 
6.0, 4.0], [6.0, 6.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 5.0, 2.0], [3.0, 4.0, 4.0], [2.0, 8.0, 3.0]]], [[[6.0, 6.0, 8.0], [7.0, 
6.0, 2.0], [4.0, 7.0, 3.0]], [[1.0, 7.0, 2.0], [8.0, 7.0, 2.0], [4.0, 6.0, 2.0]], [[5.0, 2.0, 1.0], [5.0, 3.0, 5.0], [2.0, 2.0, 
8.0]]], [[[3.0, 4.0, 7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 8.0]], [[7.0, 3.0, 1.0], [1.0, 4.0, 3.0], [6.0, 1.0, 1.0]], [[8.0, 8.0, 
8.0], [4.0, 6.0, 6.0], [2.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 5.0], [1.0, 5.0, 7.0], [5.0, 4.0, 7.0]], 
[[1.0, 4.0, 1.0], [7.0, 6.0, 6.0], [4.0, 2.0, 2.0]], [[8.0, 1.0, 6.0], [6.0, 3.0, 4.0], [2.0, 4.0, 6.0]]], [[[7.0, 8.0, 5.0], 
[1.0, 8.0, 3.0], [3.0, 1.0, 6.0]], [[4.0, 6.0, 4.0], [6.0, 6.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 5.0, 2.0], [3.0, 4.0, 4.0], 
[2.0, 8.0, 3.0]]], [[[2.0, 3.0, 4.0], [1.0, 2.0, 2.0], [6.0, 5.0, 3.0]], [[3.0, 3.0, 2.0], [5.0, 7.0, 3.0], [2.0, 4.0, 4.0]], 
[[4.0, 1.0, 2.0], [5.0, 8.0, 7.0], [4.0, 4.0, 1.0]]], [[[3.0, 4.0, 7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 8.0]], [[7.0, 3.0, 1.0], 
[1.0, 4.0, 3.0], [6.0, 1.0, 1.0]], [[8.0, 8.0, 8.0], [4.0, 6.0, 6.0], [2.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPLN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 4.0], [2.0, 5.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 5.0, 7.0], [8.0, 5.0, 8.0], [1.0, 1.0, 
4.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 2.0], [2.0, 7.0, 1.0]]], [[[1.0, 5.0, 8.0], [4.0, 6.0, 4.0], [5.0, 5.0, 7.0]], [[1.0, 8.0, 
7.0], [4.0, 4.0, 1.0], [7.0, 7.0, 4.0]], [[3.0, 3.0, 8.0], [2.0, 2.0, 1.0], [1.0, 6.0, 8.0]]], [[[7.0, 7.0, 6.0], [8.0, 8.0, 
3.0], [6.0, 8.0, 7.0]], [[5.0, 2.0, 3.0], [8.0, 8.0, 4.0], [1.0, 7.0, 3.0]], [[8.0, 7.0, 8.0], [5.0, 2.0, 3.0], [1.0, 2.0, 
4.0]]], [[[2.0, 7.0, 8.0], [1.0, 5.0, 7.0], [3.0, 3.0, 6.0]], [[6.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 7.0, 4.0]], [[3.0, 8.0, 
7.0], [7.0, 1.0, 3.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[8.0, 7.0, 2.0]])
      arg[2,1,2:]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 4.0], [2.0, 5.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 5.0, 7.0], [8.0, 5.0, 8.0], [1.0, 1.0, 
4.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 2.0], [2.0, 7.0, 1.0]]], [[[1.0, 5.0, 8.0], [4.0, 6.0, 4.0], [5.0, 5.0, 7.0]], [[1.0, 8.0, 
7.0], [4.0, 4.0, 1.0], [7.0, 7.0, 4.0]], [[3.0, 3.0, 8.0], [2.0, 2.0, 1.0], [1.0, 6.0, 8.0]]], [[[7.0, 7.0, 6.0], [8.0, 8.0, 
3.0], [6.0, 8.0, 7.0]], [[5.0, 2.0, 3.0], [8.0, 8.0, 4.0], [8.0, 7.0, 2.0]], [[8.0, 7.0, 8.0], [5.0, 2.0, 3.0], [1.0, 2.0, 
4.0]]], [[[2.0, 7.0, 8.0], [1.0, 5.0, 7.0], [3.0, 3.0, 6.0]], [[6.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 7.0, 4.0]], [[3.0, 8.0, 
7.0], [7.0, 1.0, 3.0], [4.0, 8.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPLN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 3.0], [6.0, 4.0, 4.0], [4.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 2.0], [3.0, 5.0, 
1.0]], [[7.0, 8.0, 6.0], [5.0, 4.0, 4.0], [3.0, 5.0, 1.0]]], [[[6.0, 4.0, 8.0], [2.0, 2.0, 4.0], [6.0, 2.0, 4.0]], [[4.0, 1.0, 
6.0], [3.0, 7.0, 1.0], [7.0, 3.0, 8.0]], [[1.0, 8.0, 7.0], [6.0, 6.0, 5.0], [8.0, 2.0, 8.0]]], [[[3.0, 1.0, 6.0], [4.0, 7.0, 
4.0], [8.0, 2.0, 3.0]], [[4.0, 8.0, 2.0], [6.0, 5.0, 2.0], [7.0, 3.0, 6.0]], [[1.0, 7.0, 2.0], [4.0, 2.0, 3.0], [5.0, 7.0, 
8.0]]], [[[5.0, 5.0, 6.0], [8.0, 2.0, 7.0], [6.0, 3.0, 1.0]], [[3.0, 4.0, 2.0], [4.0, 3.0, 4.0], [4.0, 5.0, 5.0]], [[5.0, 7.0, 
6.0], [7.0, 4.0, 8.0], [7.0, 6.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 8.0, 7.0]]),self.functionspace)
      arg[2,1,2:]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 3.0], [6.0, 4.0, 4.0], [4.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 2.0], [3.0, 5.0, 
1.0]], [[7.0, 8.0, 6.0], [5.0, 4.0, 4.0], [3.0, 5.0, 1.0]]], [[[6.0, 4.0, 8.0], [2.0, 2.0, 4.0], [6.0, 2.0, 4.0]], [[4.0, 1.0, 
6.0], [3.0, 7.0, 1.0], [7.0, 3.0, 8.0]], [[1.0, 8.0, 7.0], [6.0, 6.0, 5.0], [8.0, 2.0, 8.0]]], [[[3.0, 1.0, 6.0], [4.0, 7.0, 
4.0], [8.0, 2.0, 3.0]], [[4.0, 8.0, 2.0], [6.0, 5.0, 2.0], [2.0, 8.0, 7.0]], [[1.0, 7.0, 2.0], [4.0, 2.0, 3.0], [5.0, 7.0, 
8.0]]], [[[5.0, 5.0, 6.0], [8.0, 2.0, 7.0], [6.0, 3.0, 1.0]], [[3.0, 4.0, 2.0], [4.0, 3.0, 4.0], [4.0, 5.0, 5.0]], [[5.0, 7.0, 
6.0], [7.0, 4.0, 8.0], [7.0, 6.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPLN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 8.0], [5.0, 1.0, 3.0], [7.0, 4.0, 7.0]], [[4.0, 1.0, 1.0], [3.0, 1.0, 3.0], [1.0, 8.0, 
2.0]], [[7.0, 3.0, 4.0], [5.0, 1.0, 3.0], [5.0, 3.0, 2.0]]], [[[2.0, 7.0, 3.0], [2.0, 7.0, 8.0], [3.0, 5.0, 1.0]], [[1.0, 5.0, 
2.0], [2.0, 3.0, 7.0], [3.0, 2.0, 6.0]], [[8.0, 3.0, 8.0], [4.0, 2.0, 5.0], [7.0, 2.0, 3.0]]], [[[6.0, 5.0, 1.0], [1.0, 3.0, 
4.0], [1.0, 8.0, 6.0]], [[4.0, 6.0, 3.0], [6.0, 2.0, 4.0], [7.0, 6.0, 4.0]], [[6.0, 8.0, 5.0], [2.0, 3.0, 8.0], [8.0, 6.0, 
1.0]]], [[[7.0, 5.0, 6.0], [2.0, 2.0, 6.0], [5.0, 6.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 1.0, 8.0], [1.0, 4.0, 8.0]], [[1.0, 8.0, 
4.0], [7.0, 3.0, 8.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 4.0, 1.0]])+(1.-msk_val)*numpy.array([[4.0, 7.0, 5.0]])
      arg[2,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 8.0], [5.0, 1.0, 3.0], [7.0, 4.0, 7.0]], [[4.0, 1.0, 1.0], [3.0, 1.0, 3.0], [1.0, 
8.0, 2.0]], [[7.0, 3.0, 4.0], [5.0, 1.0, 3.0], [5.0, 3.0, 2.0]]], [[[2.0, 7.0, 3.0], [2.0, 7.0, 8.0], [3.0, 5.0, 1.0]], [[1.0, 
5.0, 2.0], [2.0, 3.0, 7.0], [3.0, 2.0, 6.0]], [[8.0, 3.0, 8.0], [4.0, 2.0, 5.0], [7.0, 2.0, 3.0]]], [[[6.0, 5.0, 1.0], [1.0, 
3.0, 4.0], [1.0, 8.0, 6.0]], [[4.0, 6.0, 3.0], [6.0, 2.0, 4.0], [2.0, 4.0, 1.0]], [[6.0, 8.0, 5.0], [2.0, 3.0, 8.0], [8.0, 6.0, 
1.0]]], [[[7.0, 5.0, 6.0], [2.0, 2.0, 6.0], [5.0, 6.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 1.0, 8.0], [1.0, 4.0, 8.0]], [[1.0, 8.0, 
4.0], [7.0, 3.0, 8.0], [5.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 8.0], [5.0, 1.0, 3.0], [7.0, 4.0, 7.0]], 
[[4.0, 1.0, 1.0], [3.0, 1.0, 3.0], [1.0, 8.0, 2.0]], [[7.0, 3.0, 4.0], [5.0, 1.0, 3.0], [5.0, 3.0, 2.0]]], [[[2.0, 7.0, 3.0], 
[2.0, 7.0, 8.0], [3.0, 5.0, 1.0]], [[1.0, 5.0, 2.0], [2.0, 3.0, 7.0], [3.0, 2.0, 6.0]], [[8.0, 3.0, 8.0], [4.0, 2.0, 5.0], 
[7.0, 2.0, 3.0]]], [[[6.0, 5.0, 1.0], [1.0, 3.0, 4.0], [1.0, 8.0, 6.0]], [[4.0, 6.0, 3.0], [6.0, 2.0, 4.0], [4.0, 7.0, 5.0]], 
[[6.0, 8.0, 5.0], [2.0, 3.0, 8.0], [8.0, 6.0, 1.0]]], [[[7.0, 5.0, 6.0], [2.0, 2.0, 6.0], [5.0, 6.0, 3.0]], [[2.0, 3.0, 2.0], 
[1.0, 1.0, 8.0], [1.0, 4.0, 8.0]], [[1.0, 8.0, 4.0], [7.0, 3.0, 8.0], [5.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLRN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 4.0], [2.0, 1.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 2.0, 6.0], [1.0, 6.0, 3.0], [7.0, 3.0, 
4.0]], [[4.0, 6.0, 4.0], [5.0, 1.0, 3.0], [4.0, 3.0, 6.0]]], [[[4.0, 5.0, 7.0], [4.0, 7.0, 3.0], [7.0, 1.0, 8.0]], [[8.0, 5.0, 
5.0], [4.0, 6.0, 3.0], [2.0, 5.0, 4.0]], [[1.0, 1.0, 4.0], [1.0, 4.0, 5.0], [3.0, 7.0, 3.0]]], [[[3.0, 5.0, 5.0], [8.0, 1.0, 
2.0], [6.0, 4.0, 5.0]], [[6.0, 3.0, 3.0], [2.0, 8.0, 3.0], [8.0, 5.0, 3.0]], [[7.0, 2.0, 6.0], [4.0, 7.0, 6.0], [4.0, 7.0, 
2.0]]], [[[2.0, 7.0, 4.0], [6.0, 4.0, 1.0], [5.0, 1.0, 2.0]], [[4.0, 2.0, 3.0], [2.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[6.0, 4.0, 
1.0], [2.0, 3.0, 4.0], [1.0, 7.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 3.0, 6.0]]], [[[4.0, 5.0, 7.0]]]])
      arg[1:3,2:,1:2]=val
      ref=Data(numpy.array([[[[7.0, 4.0, 4.0], [2.0, 1.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 2.0, 6.0], [1.0, 6.0, 3.0], [7.0, 3.0, 
4.0]], [[4.0, 6.0, 4.0], [5.0, 1.0, 3.0], [4.0, 3.0, 6.0]]], [[[4.0, 5.0, 7.0], [4.0, 7.0, 3.0], [7.0, 1.0, 8.0]], [[8.0, 5.0, 
5.0], [4.0, 6.0, 3.0], [2.0, 5.0, 4.0]], [[1.0, 1.0, 4.0], [6.0, 3.0, 6.0], [3.0, 7.0, 3.0]]], [[[3.0, 5.0, 5.0], [8.0, 1.0, 
2.0], [6.0, 4.0, 5.0]], [[6.0, 3.0, 3.0], [2.0, 8.0, 3.0], [8.0, 5.0, 3.0]], [[7.0, 2.0, 6.0], [4.0, 5.0, 7.0], [4.0, 7.0, 
2.0]]], [[[2.0, 7.0, 4.0], [6.0, 4.0, 1.0], [5.0, 1.0, 2.0]], [[4.0, 2.0, 3.0], [2.0, 8.0, 3.0], [3.0, 4.0, 8.0]], [[6.0, 4.0, 
1.0], [2.0, 3.0, 4.0], [1.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLRN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 7.0], [4.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 7.0, 7.0], [8.0, 1.0, 
8.0]], [[2.0, 2.0, 4.0], [8.0, 1.0, 6.0], [2.0, 3.0, 6.0]]], [[[7.0, 1.0, 4.0], [7.0, 5.0, 7.0], [6.0, 7.0, 4.0]], [[4.0, 2.0, 
3.0], [3.0, 2.0, 1.0], [8.0, 2.0, 3.0]], [[3.0, 1.0, 8.0], [3.0, 2.0, 1.0], [7.0, 5.0, 4.0]]], [[[4.0, 5.0, 5.0], [6.0, 7.0, 
8.0], [4.0, 5.0, 1.0]], [[8.0, 8.0, 2.0], [5.0, 1.0, 3.0], [5.0, 7.0, 5.0]], [[7.0, 4.0, 6.0], [2.0, 7.0, 7.0], [5.0, 5.0, 
2.0]]], [[[8.0, 6.0, 7.0], [7.0, 3.0, 1.0], [2.0, 2.0, 6.0]], [[5.0, 1.0, 2.0], [1.0, 8.0, 2.0], [3.0, 1.0, 6.0]], [[2.0, 8.0, 
5.0], [3.0, 4.0, 2.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 2.0, 4.0]]], [[[3.0, 4.0, 8.0]]]]),self.functionspace)
      arg[1:3,2:,1:2]=val
      ref=Data(numpy.array([[[[1.0, 2.0, 7.0], [4.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 7.0, 7.0], [8.0, 1.0, 
8.0]], [[2.0, 2.0, 4.0], [8.0, 1.0, 6.0], [2.0, 3.0, 6.0]]], [[[7.0, 1.0, 4.0], [7.0, 5.0, 7.0], [6.0, 7.0, 4.0]], [[4.0, 2.0, 
3.0], [3.0, 2.0, 1.0], [8.0, 2.0, 3.0]], [[3.0, 1.0, 8.0], [7.0, 2.0, 4.0], [7.0, 5.0, 4.0]]], [[[4.0, 5.0, 5.0], [6.0, 7.0, 
8.0], [4.0, 5.0, 1.0]], [[8.0, 8.0, 2.0], [5.0, 1.0, 3.0], [5.0, 7.0, 5.0]], [[7.0, 4.0, 6.0], [3.0, 4.0, 8.0], [5.0, 5.0, 
2.0]]], [[[8.0, 6.0, 7.0], [7.0, 3.0, 1.0], [2.0, 2.0, 6.0]], [[5.0, 1.0, 2.0], [1.0, 8.0, 2.0], [3.0, 1.0, 6.0]], [[2.0, 8.0, 
5.0], [3.0, 4.0, 2.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLRN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 1.0], [3.0, 3.0, 4.0], [2.0, 2.0, 4.0]], [[6.0, 8.0, 2.0], [6.0, 3.0, 3.0], [8.0, 8.0, 
3.0]], [[7.0, 7.0, 6.0], [2.0, 6.0, 4.0], [3.0, 7.0, 8.0]]], [[[5.0, 4.0, 8.0], [7.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[4.0, 6.0, 
3.0], [5.0, 1.0, 2.0], [7.0, 8.0, 4.0]], [[8.0, 8.0, 3.0], [2.0, 4.0, 4.0], [2.0, 4.0, 2.0]]], [[[8.0, 8.0, 8.0], [7.0, 4.0, 
2.0], [7.0, 4.0, 4.0]], [[3.0, 6.0, 2.0], [1.0, 8.0, 3.0], [2.0, 5.0, 3.0]], [[4.0, 6.0, 1.0], [4.0, 8.0, 2.0], [7.0, 3.0, 
5.0]]], [[[4.0, 7.0, 6.0], [1.0, 3.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 3.0, 1.0], [6.0, 8.0, 2.0], [2.0, 1.0, 4.0]], [[3.0, 3.0, 
1.0], [6.0, 7.0, 5.0], [3.0, 7.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 5.0, 2.0]]], [[[5.0, 4.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 1.0, 5.0]]], 
[[[6.0, 4.0, 4.0]]]])
      arg[1:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 1.0], [3.0, 3.0, 4.0], [2.0, 2.0, 4.0]], [[6.0, 8.0, 2.0], [6.0, 3.0, 3.0], [8.0, 
8.0, 3.0]], [[7.0, 7.0, 6.0], [2.0, 6.0, 4.0], [3.0, 7.0, 8.0]]], [[[5.0, 4.0, 8.0], [7.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[4.0, 
6.0, 3.0], [5.0, 1.0, 2.0], [7.0, 8.0, 4.0]], [[8.0, 8.0, 3.0], [3.0, 5.0, 2.0], [2.0, 4.0, 2.0]]], [[[8.0, 8.0, 8.0], [7.0, 
4.0, 2.0], [7.0, 4.0, 4.0]], [[3.0, 6.0, 2.0], [1.0, 8.0, 3.0], [2.0, 5.0, 3.0]], [[4.0, 6.0, 1.0], [5.0, 4.0, 1.0], [7.0, 3.0, 
5.0]]], [[[4.0, 7.0, 6.0], [1.0, 3.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 3.0, 1.0], [6.0, 8.0, 2.0], [2.0, 1.0, 4.0]], [[3.0, 3.0, 
1.0], [6.0, 7.0, 5.0], [3.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 1.0], [3.0, 3.0, 4.0], [2.0, 2.0, 4.0]], 
[[6.0, 8.0, 2.0], [6.0, 3.0, 3.0], [8.0, 8.0, 3.0]], [[7.0, 7.0, 6.0], [2.0, 6.0, 4.0], [3.0, 7.0, 8.0]]], [[[5.0, 4.0, 8.0], 
[7.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[4.0, 6.0, 3.0], [5.0, 1.0, 2.0], [7.0, 8.0, 4.0]], [[8.0, 8.0, 3.0], [3.0, 1.0, 5.0], 
[2.0, 4.0, 2.0]]], [[[8.0, 8.0, 8.0], [7.0, 4.0, 2.0], [7.0, 4.0, 4.0]], [[3.0, 6.0, 2.0], [1.0, 8.0, 3.0], [2.0, 5.0, 3.0]], 
[[4.0, 6.0, 1.0], [6.0, 4.0, 4.0], [7.0, 3.0, 5.0]]], [[[4.0, 7.0, 6.0], [1.0, 3.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 3.0, 1.0], 
[6.0, 8.0, 2.0], [2.0, 1.0, 4.0]], [[3.0, 3.0, 1.0], [6.0, 7.0, 5.0], [3.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LCCN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 2.0], [1.0, 7.0, 6.0], [3.0, 7.0, 4.0]], [[5.0, 1.0, 1.0], [1.0, 4.0, 2.0], [7.0, 3.0, 
1.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 1.0], [2.0, 7.0, 4.0]]], [[[5.0, 5.0, 3.0], [6.0, 8.0, 4.0], [6.0, 1.0, 8.0]], [[3.0, 4.0, 
6.0], [7.0, 4.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 4.0, 2.0], [4.0, 2.0, 1.0], [2.0, 1.0, 7.0]]], [[[1.0, 7.0, 1.0], [2.0, 7.0, 
4.0], [7.0, 4.0, 7.0]], [[2.0, 1.0, 3.0], [4.0, 5.0, 6.0], [2.0, 6.0, 2.0]], [[8.0, 8.0, 4.0], [7.0, 8.0, 8.0], [1.0, 3.0, 
1.0]]], [[[6.0, 3.0, 6.0], [4.0, 3.0, 3.0], [3.0, 8.0, 3.0]], [[1.0, 1.0, 2.0], [7.0, 4.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 3.0, 
6.0], [7.0, 6.0, 7.0], [3.0, 1.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 6.0, 3.0], [3.0, 7.0, 3.0], [6.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [1.0, 5.0, 8.0], [3.0, 5.0, 
8.0]], [[6.0, 6.0, 8.0], [7.0, 2.0, 6.0], [4.0, 4.0, 1.0]]], [[[2.0, 6.0, 2.0], [5.0, 3.0, 1.0], [8.0, 8.0, 5.0]], [[2.0, 7.0, 
3.0], [3.0, 1.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 4.0, 1.0], [2.0, 5.0, 5.0], [4.0, 8.0, 7.0]]]])
      arg[2:,:,:]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 2.0], [1.0, 7.0, 6.0], [3.0, 7.0, 4.0]], [[5.0, 1.0, 1.0], [1.0, 4.0, 2.0], [7.0, 3.0, 
1.0]], [[3.0, 3.0, 3.0], [8.0, 7.0, 1.0], [2.0, 7.0, 4.0]]], [[[5.0, 5.0, 3.0], [6.0, 8.0, 4.0], [6.0, 1.0, 8.0]], [[3.0, 4.0, 
6.0], [7.0, 4.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 4.0, 2.0], [4.0, 2.0, 1.0], [2.0, 1.0, 7.0]]], [[[3.0, 6.0, 3.0], [3.0, 7.0, 
3.0], [6.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [1.0, 5.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 6.0, 8.0], [7.0, 2.0, 6.0], [4.0, 4.0, 
1.0]]], [[[2.0, 6.0, 2.0], [5.0, 3.0, 1.0], [8.0, 8.0, 5.0]], [[2.0, 7.0, 3.0], [3.0, 1.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 4.0, 
1.0], [2.0, 5.0, 5.0], [4.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LCCN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 2.0], [4.0, 3.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [7.0, 5.0, 
8.0]], [[3.0, 6.0, 5.0], [8.0, 1.0, 4.0], [8.0, 8.0, 8.0]]], [[[3.0, 6.0, 1.0], [6.0, 3.0, 1.0], [8.0, 4.0, 6.0]], [[2.0, 6.0, 
5.0], [6.0, 5.0, 2.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 2.0], [5.0, 6.0, 5.0], [6.0, 6.0, 7.0]]], [[[5.0, 8.0, 1.0], [7.0, 5.0, 
2.0], [2.0, 2.0, 5.0]], [[1.0, 4.0, 3.0], [8.0, 3.0, 6.0], [6.0, 5.0, 1.0]], [[3.0, 4.0, 8.0], [8.0, 8.0, 7.0], [1.0, 8.0, 
4.0]]], [[[5.0, 2.0, 1.0], [4.0, 7.0, 1.0], [6.0, 3.0, 5.0]], [[8.0, 3.0, 5.0], [7.0, 2.0, 2.0], [3.0, 5.0, 1.0]], [[4.0, 3.0, 
6.0], [2.0, 6.0, 4.0], [3.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 2.0, 7.0], [6.0, 2.0, 7.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 4.0], [8.0, 2.0, 8.0], [1.0, 1.0, 
3.0]], [[4.0, 3.0, 8.0], [8.0, 7.0, 5.0], [5.0, 7.0, 4.0]]], [[[2.0, 2.0, 5.0], [2.0, 1.0, 5.0], [7.0, 7.0, 7.0]], [[5.0, 7.0, 
3.0], [8.0, 3.0, 8.0], [8.0, 8.0, 7.0]], [[2.0, 1.0, 8.0], [3.0, 5.0, 6.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      arg[2:,:,:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 2.0], [4.0, 3.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [7.0, 5.0, 
8.0]], [[3.0, 6.0, 5.0], [8.0, 1.0, 4.0], [8.0, 8.0, 8.0]]], [[[3.0, 6.0, 1.0], [6.0, 3.0, 1.0], [8.0, 4.0, 6.0]], [[2.0, 6.0, 
5.0], [6.0, 5.0, 2.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 2.0], [5.0, 6.0, 5.0], [6.0, 6.0, 7.0]]], [[[8.0, 2.0, 7.0], [6.0, 2.0, 
7.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 4.0], [8.0, 2.0, 8.0], [1.0, 1.0, 3.0]], [[4.0, 3.0, 8.0], [8.0, 7.0, 5.0], [5.0, 7.0, 
4.0]]], [[[2.0, 2.0, 5.0], [2.0, 1.0, 5.0], [7.0, 7.0, 7.0]], [[5.0, 7.0, 3.0], [8.0, 3.0, 8.0], [8.0, 8.0, 7.0]], [[2.0, 1.0, 
8.0], [3.0, 5.0, 6.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LCCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 4.0], [3.0, 6.0, 6.0], [1.0, 7.0, 8.0]], [[8.0, 3.0, 4.0], [7.0, 3.0, 3.0], [2.0, 4.0, 
4.0]], [[3.0, 3.0, 3.0], [7.0, 7.0, 1.0], [1.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], [5.0, 2.0, 8.0], [8.0, 4.0, 6.0]], [[6.0, 6.0, 
4.0], [2.0, 5.0, 5.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 3.0], [2.0, 1.0, 6.0], [1.0, 2.0, 3.0]]], [[[5.0, 2.0, 6.0], [8.0, 3.0, 
8.0], [7.0, 8.0, 8.0]], [[5.0, 7.0, 3.0], [1.0, 3.0, 5.0], [2.0, 1.0, 5.0]], [[1.0, 7.0, 1.0], [7.0, 5.0, 2.0], [7.0, 1.0, 
6.0]]], [[[4.0, 8.0, 1.0], [8.0, 8.0, 1.0], [1.0, 7.0, 4.0]], [[4.0, 2.0, 2.0], [5.0, 6.0, 5.0], [2.0, 4.0, 2.0]], [[7.0, 6.0, 
5.0], [4.0, 1.0, 7.0], [5.0, 8.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 4.0, 3.0], [8.0, 7.0, 4.0], [1.0, 5.0, 6.0]], [[8.0, 1.0, 5.0], [1.0, 6.0, 2.0], [1.0, 
3.0, 3.0]], [[5.0, 6.0, 3.0], [2.0, 5.0, 1.0], [4.0, 2.0, 5.0]]], [[[3.0, 1.0, 8.0], [6.0, 5.0, 6.0], [1.0, 7.0, 1.0]], [[1.0, 
3.0, 2.0], [4.0, 1.0, 1.0], [2.0, 4.0, 7.0]], [[1.0, 8.0, 2.0], [4.0, 2.0, 4.0], [4.0, 8.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 5.0, 6.0], [5.0, 8.0, 3.0], [6.0, 3.0, 8.0]], [[7.0, 6.0, 5.0], [2.0, 7.0, 5.0], 
[2.0, 6.0, 6.0]], [[2.0, 5.0, 3.0], [2.0, 3.0, 2.0], [8.0, 7.0, 3.0]]], [[[4.0, 1.0, 5.0], [4.0, 2.0, 4.0], [5.0, 4.0, 5.0]], 
[[5.0, 4.0, 2.0], [2.0, 1.0, 5.0], [1.0, 7.0, 4.0]], [[8.0, 6.0, 3.0], [4.0, 3.0, 1.0], [3.0, 3.0, 2.0]]]])
      arg[2:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [3.0, 6.0, 6.0], [1.0, 7.0, 8.0]], [[8.0, 3.0, 4.0], [7.0, 3.0, 3.0], [2.0, 
4.0, 4.0]], [[3.0, 3.0, 3.0], [7.0, 7.0, 1.0], [1.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], [5.0, 2.0, 8.0], [8.0, 4.0, 6.0]], [[6.0, 
6.0, 4.0], [2.0, 5.0, 5.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 3.0], [2.0, 1.0, 6.0], [1.0, 2.0, 3.0]]], [[[1.0, 4.0, 3.0], [8.0, 
7.0, 4.0], [1.0, 5.0, 6.0]], [[8.0, 1.0, 5.0], [1.0, 6.0, 2.0], [1.0, 3.0, 3.0]], [[5.0, 6.0, 3.0], [2.0, 5.0, 1.0], [4.0, 2.0, 
5.0]]], [[[3.0, 1.0, 8.0], [6.0, 5.0, 6.0], [1.0, 7.0, 1.0]], [[1.0, 3.0, 2.0], [4.0, 1.0, 1.0], [2.0, 4.0, 7.0]], [[1.0, 8.0, 
2.0], [4.0, 2.0, 4.0], [4.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 4.0], [3.0, 6.0, 6.0], [1.0, 7.0, 8.0]], 
[[8.0, 3.0, 4.0], [7.0, 3.0, 3.0], [2.0, 4.0, 4.0]], [[3.0, 3.0, 3.0], [7.0, 7.0, 1.0], [1.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], 
[5.0, 2.0, 8.0], [8.0, 4.0, 6.0]], [[6.0, 6.0, 4.0], [2.0, 5.0, 5.0], [3.0, 7.0, 6.0]], [[3.0, 6.0, 3.0], [2.0, 1.0, 6.0], 
[1.0, 2.0, 3.0]]], [[[7.0, 5.0, 6.0], [5.0, 8.0, 3.0], [6.0, 3.0, 8.0]], [[7.0, 6.0, 5.0], [2.0, 7.0, 5.0], [2.0, 6.0, 6.0]], 
[[2.0, 5.0, 3.0], [2.0, 3.0, 2.0], [8.0, 7.0, 3.0]]], [[[4.0, 1.0, 5.0], [4.0, 2.0, 4.0], [5.0, 4.0, 5.0]], [[5.0, 4.0, 2.0], 
[2.0, 1.0, 5.0], [1.0, 7.0, 4.0]], [[8.0, 6.0, 3.0], [4.0, 3.0, 1.0], [3.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RURU(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 6.0], [5.0, 3.0, 1.0], [8.0, 3.0, 4.0]], [[8.0, 7.0, 3.0], [6.0, 5.0, 8.0], [4.0, 3.0, 
7.0]], [[1.0, 2.0, 6.0], [3.0, 8.0, 4.0], [1.0, 3.0, 2.0]]], [[[5.0, 7.0, 1.0], [3.0, 3.0, 7.0], [3.0, 3.0, 6.0]], [[8.0, 5.0, 
8.0], [1.0, 6.0, 4.0], [7.0, 2.0, 4.0]], [[4.0, 8.0, 1.0], [5.0, 8.0, 3.0], [2.0, 8.0, 7.0]]], [[[2.0, 1.0, 3.0], [2.0, 2.0, 
4.0], [1.0, 5.0, 6.0]], [[6.0, 1.0, 3.0], [5.0, 4.0, 6.0], [5.0, 5.0, 3.0]], [[5.0, 4.0, 7.0], [3.0, 3.0, 7.0], [7.0, 3.0, 
6.0]]], [[[5.0, 4.0, 7.0], [7.0, 1.0, 2.0], [7.0, 6.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 8.0, 3.0], [3.0, 4.0, 7.0]], [[3.0, 8.0, 
2.0], [2.0, 2.0, 8.0], [3.0, 7.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 5.0]], [[3.0, 2.0]]], [[[8.0, 3.0]], [[1.0, 4.0]]]])
      arg[1:3,:2,1:2,:2]=val
      ref=Data(numpy.array([[[[7.0, 2.0, 6.0], [5.0, 3.0, 1.0], [8.0, 3.0, 4.0]], [[8.0, 7.0, 3.0], [6.0, 5.0, 8.0], [4.0, 3.0, 
7.0]], [[1.0, 2.0, 6.0], [3.0, 8.0, 4.0], [1.0, 3.0, 2.0]]], [[[5.0, 7.0, 1.0], [7.0, 5.0, 7.0], [3.0, 3.0, 6.0]], [[8.0, 5.0, 
8.0], [3.0, 2.0, 4.0], [7.0, 2.0, 4.0]], [[4.0, 8.0, 1.0], [5.0, 8.0, 3.0], [2.0, 8.0, 7.0]]], [[[2.0, 1.0, 3.0], [8.0, 3.0, 
4.0], [1.0, 5.0, 6.0]], [[6.0, 1.0, 3.0], [1.0, 4.0, 6.0], [5.0, 5.0, 3.0]], [[5.0, 4.0, 7.0], [3.0, 3.0, 7.0], [7.0, 3.0, 
6.0]]], [[[5.0, 4.0, 7.0], [7.0, 1.0, 2.0], [7.0, 6.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 8.0, 3.0], [3.0, 4.0, 7.0]], [[3.0, 8.0, 
2.0], [2.0, 2.0, 8.0], [3.0, 7.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RURU(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 6.0], [4.0, 2.0, 7.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 8.0], [5.0, 3.0, 4.0], [5.0, 1.0, 
8.0]], [[5.0, 5.0, 5.0], [1.0, 3.0, 8.0], [5.0, 7.0, 7.0]]], [[[6.0, 3.0, 4.0], [4.0, 6.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 6.0, 
5.0], [3.0, 1.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 1.0, 8.0], [4.0, 6.0, 6.0], [7.0, 6.0, 4.0]]], [[[3.0, 8.0, 3.0], [3.0, 2.0, 
8.0], [2.0, 1.0, 1.0]], [[7.0, 7.0, 1.0], [1.0, 5.0, 4.0], [3.0, 1.0, 7.0]], [[1.0, 5.0, 7.0], [3.0, 8.0, 2.0], [8.0, 7.0, 
7.0]]], [[[8.0, 2.0, 5.0], [1.0, 6.0, 1.0], [4.0, 7.0, 5.0]], [[5.0, 7.0, 7.0], [4.0, 3.0, 3.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 
8.0], [3.0, 5.0, 7.0], [7.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 3.0]], [[3.0, 8.0]]], [[[3.0, 1.0]], [[6.0, 4.0]]]]),self.functionspace)
      arg[1:3,:2,1:2,:2]=val
      ref=Data(numpy.array([[[[4.0, 6.0, 6.0], [4.0, 2.0, 7.0], [2.0, 4.0, 7.0]], [[6.0, 4.0, 8.0], [5.0, 3.0, 4.0], [5.0, 1.0, 
8.0]], [[5.0, 5.0, 5.0], [1.0, 3.0, 8.0], [5.0, 7.0, 7.0]]], [[[6.0, 3.0, 4.0], [5.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 6.0, 
5.0], [3.0, 8.0, 4.0], [6.0, 1.0, 6.0]], [[1.0, 1.0, 8.0], [4.0, 6.0, 6.0], [7.0, 6.0, 4.0]]], [[[3.0, 8.0, 3.0], [3.0, 1.0, 
8.0], [2.0, 1.0, 1.0]], [[7.0, 7.0, 1.0], [6.0, 4.0, 4.0], [3.0, 1.0, 7.0]], [[1.0, 5.0, 7.0], [3.0, 8.0, 2.0], [8.0, 7.0, 
7.0]]], [[[8.0, 2.0, 5.0], [1.0, 6.0, 1.0], [4.0, 7.0, 5.0]], [[5.0, 7.0, 7.0], [4.0, 3.0, 3.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 
8.0], [3.0, 5.0, 7.0], [7.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RURU(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 7.0], [7.0, 1.0, 2.0], [1.0, 7.0, 8.0]], [[3.0, 3.0, 2.0], [4.0, 7.0, 8.0], [8.0, 3.0, 
6.0]], [[3.0, 3.0, 3.0], [3.0, 4.0, 6.0], [8.0, 8.0, 2.0]]], [[[4.0, 8.0, 4.0], [6.0, 2.0, 4.0], [1.0, 5.0, 8.0]], [[1.0, 7.0, 
3.0], [2.0, 2.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 4.0, 8.0], [3.0, 1.0, 1.0], [3.0, 3.0, 5.0]]], [[[3.0, 7.0, 6.0], [1.0, 7.0, 
2.0], [2.0, 4.0, 2.0]], [[4.0, 5.0, 7.0], [5.0, 1.0, 6.0], [8.0, 2.0, 3.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 6.0], [5.0, 6.0, 
8.0]]], [[[3.0, 4.0, 2.0], [3.0, 4.0, 6.0], [2.0, 5.0, 8.0]], [[7.0, 4.0, 2.0], [1.0, 1.0, 6.0], [8.0, 3.0, 6.0]], [[7.0, 4.0, 
1.0], [7.0, 7.0, 7.0], [5.0, 3.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 5.0]], [[1.0, 2.0]]], [[[4.0, 2.0]], [[7.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 
8.0]], [[6.0, 2.0]]], [[[6.0, 1.0]], [[7.0, 2.0]]]])
      arg[1:3,:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [7.0, 1.0, 2.0], [1.0, 7.0, 8.0]], [[3.0, 3.0, 2.0], [4.0, 7.0, 8.0], [8.0, 
3.0, 6.0]], [[3.0, 3.0, 3.0], [3.0, 4.0, 6.0], [8.0, 8.0, 2.0]]], [[[4.0, 8.0, 4.0], [1.0, 5.0, 4.0], [1.0, 5.0, 8.0]], [[1.0, 
7.0, 3.0], [1.0, 2.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 4.0, 8.0], [3.0, 1.0, 1.0], [3.0, 3.0, 5.0]]], [[[3.0, 7.0, 6.0], [4.0, 
2.0, 2.0], [2.0, 4.0, 2.0]], [[4.0, 5.0, 7.0], [7.0, 8.0, 6.0], [8.0, 2.0, 3.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 6.0], [5.0, 6.0, 
8.0]]], [[[3.0, 4.0, 2.0], [3.0, 4.0, 6.0], [2.0, 5.0, 8.0]], [[7.0, 4.0, 2.0], [1.0, 1.0, 6.0], [8.0, 3.0, 6.0]], [[7.0, 4.0, 
1.0], [7.0, 7.0, 7.0], [5.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 7.0], [7.0, 1.0, 2.0], [1.0, 7.0, 8.0]], 
[[3.0, 3.0, 2.0], [4.0, 7.0, 8.0], [8.0, 3.0, 6.0]], [[3.0, 3.0, 3.0], [3.0, 4.0, 6.0], [8.0, 8.0, 2.0]]], [[[4.0, 8.0, 4.0], 
[5.0, 8.0, 4.0], [1.0, 5.0, 8.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 4.0, 8.0], [3.0, 1.0, 1.0], 
[3.0, 3.0, 5.0]]], [[[3.0, 7.0, 6.0], [6.0, 1.0, 2.0], [2.0, 4.0, 2.0]], [[4.0, 5.0, 7.0], [7.0, 2.0, 6.0], [8.0, 2.0, 3.0]], 
[[8.0, 4.0, 3.0], [1.0, 6.0, 6.0], [5.0, 6.0, 8.0]]], [[[3.0, 4.0, 2.0], [3.0, 4.0, 6.0], [2.0, 5.0, 8.0]], [[7.0, 4.0, 2.0], 
[1.0, 1.0, 6.0], [8.0, 3.0, 6.0]], [[7.0, 4.0, 1.0], [7.0, 7.0, 7.0], [5.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPCP(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 1.0], [3.0, 8.0, 7.0], [6.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [5.0, 4.0, 5.0], [3.0, 5.0, 
6.0]], [[7.0, 3.0, 8.0], [1.0, 6.0, 1.0], [5.0, 8.0, 8.0]]], [[[7.0, 1.0, 1.0], [8.0, 1.0, 3.0], [7.0, 8.0, 4.0]], [[7.0, 8.0, 
8.0], [7.0, 4.0, 1.0], [6.0, 3.0, 8.0]], [[1.0, 5.0, 4.0], [2.0, 6.0, 5.0], [3.0, 1.0, 2.0]]], [[[1.0, 6.0, 8.0], [7.0, 8.0, 
6.0], [1.0, 8.0, 6.0]], [[1.0, 6.0, 1.0], [3.0, 5.0, 5.0], [5.0, 5.0, 7.0]], [[4.0, 6.0, 3.0], [7.0, 7.0, 5.0], [2.0, 8.0, 
6.0]]], [[[7.0, 6.0, 8.0], [1.0, 2.0, 2.0], [8.0, 4.0, 8.0]], [[4.0, 7.0, 1.0], [2.0, 5.0, 5.0], [6.0, 6.0, 4.0]], [[3.0, 6.0, 
7.0], [2.0, 3.0, 1.0], [6.0, 6.0, 5.0]]]]),self.functionspace)
      val=numpy.array([1.0, 1.0, 5.0])
      arg[2,1,:,1]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 1.0], [3.0, 8.0, 7.0], [6.0, 6.0, 1.0]], [[5.0, 8.0, 8.0], [5.0, 4.0, 5.0], [3.0, 5.0, 
6.0]], [[7.0, 3.0, 8.0], [1.0, 6.0, 1.0], [5.0, 8.0, 8.0]]], [[[7.0, 1.0, 1.0], [8.0, 1.0, 3.0], [7.0, 8.0, 4.0]], [[7.0, 8.0, 
8.0], [7.0, 4.0, 1.0], [6.0, 3.0, 8.0]], [[1.0, 5.0, 4.0], [2.0, 6.0, 5.0], [3.0, 1.0, 2.0]]], [[[1.0, 6.0, 8.0], [7.0, 8.0, 
6.0], [1.0, 8.0, 6.0]], [[1.0, 1.0, 1.0], [3.0, 1.0, 5.0], [5.0, 5.0, 7.0]], [[4.0, 6.0, 3.0], [7.0, 7.0, 5.0], [2.0, 8.0, 
6.0]]], [[[7.0, 6.0, 8.0], [1.0, 2.0, 2.0], [8.0, 4.0, 8.0]], [[4.0, 7.0, 1.0], [2.0, 5.0, 5.0], [6.0, 6.0, 4.0]], [[3.0, 6.0, 
7.0], [2.0, 3.0, 1.0], [6.0, 6.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPCP(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 4.0], [1.0, 6.0, 3.0], [3.0, 7.0, 3.0]], [[7.0, 6.0, 3.0], [4.0, 3.0, 3.0], [8.0, 6.0, 
5.0]], [[7.0, 1.0, 1.0], [7.0, 5.0, 4.0], [3.0, 1.0, 4.0]]], [[[5.0, 8.0, 2.0], [6.0, 5.0, 8.0], [6.0, 8.0, 2.0]], [[5.0, 3.0, 
7.0], [8.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[2.0, 7.0, 7.0], [4.0, 2.0, 5.0], [4.0, 8.0, 6.0]]], [[[3.0, 2.0, 4.0], [1.0, 1.0, 
3.0], [1.0, 3.0, 2.0]], [[4.0, 5.0, 2.0], [5.0, 3.0, 5.0], [7.0, 6.0, 8.0]], [[7.0, 2.0, 1.0], [4.0, 3.0, 8.0], [6.0, 6.0, 
5.0]]], [[[3.0, 1.0, 8.0], [2.0, 4.0, 7.0], [2.0, 5.0, 8.0]], [[8.0, 2.0, 1.0], [1.0, 5.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 6.0, 
6.0], [2.0, 4.0, 4.0], [7.0, 8.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([8.0, 3.0, 1.0]),self.functionspace)
      arg[2,1,:,1]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 4.0], [1.0, 6.0, 3.0], [3.0, 7.0, 3.0]], [[7.0, 6.0, 3.0], [4.0, 3.0, 3.0], [8.0, 6.0, 
5.0]], [[7.0, 1.0, 1.0], [7.0, 5.0, 4.0], [3.0, 1.0, 4.0]]], [[[5.0, 8.0, 2.0], [6.0, 5.0, 8.0], [6.0, 8.0, 2.0]], [[5.0, 3.0, 
7.0], [8.0, 8.0, 8.0], [2.0, 1.0, 7.0]], [[2.0, 7.0, 7.0], [4.0, 2.0, 5.0], [4.0, 8.0, 6.0]]], [[[3.0, 2.0, 4.0], [1.0, 1.0, 
3.0], [1.0, 3.0, 2.0]], [[4.0, 8.0, 2.0], [5.0, 3.0, 5.0], [7.0, 1.0, 8.0]], [[7.0, 2.0, 1.0], [4.0, 3.0, 8.0], [6.0, 6.0, 
5.0]]], [[[3.0, 1.0, 8.0], [2.0, 4.0, 7.0], [2.0, 5.0, 8.0]], [[8.0, 2.0, 1.0], [1.0, 5.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 6.0, 
6.0], [2.0, 4.0, 4.0], [7.0, 8.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPCP(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 5.0], [6.0, 8.0, 2.0], [8.0, 4.0, 2.0]], [[4.0, 8.0, 6.0], [8.0, 1.0, 1.0], [5.0, 3.0, 
3.0]], [[3.0, 2.0, 1.0], [4.0, 7.0, 1.0], [3.0, 3.0, 7.0]]], [[[6.0, 6.0, 7.0], [4.0, 2.0, 6.0], [8.0, 6.0, 7.0]], [[5.0, 7.0, 
2.0], [5.0, 3.0, 3.0], [5.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 6.0, 7.0], [2.0, 6.0, 7.0]]], [[[5.0, 4.0, 2.0], [8.0, 4.0, 
1.0], [4.0, 8.0, 2.0]], [[5.0, 5.0, 8.0], [1.0, 6.0, 6.0], [3.0, 3.0, 7.0]], [[3.0, 7.0, 3.0], [8.0, 1.0, 3.0], [3.0, 2.0, 
8.0]]], [[[4.0, 5.0, 5.0], [5.0, 1.0, 2.0], [2.0, 2.0, 1.0]], [[8.0, 2.0, 1.0], [4.0, 2.0, 6.0], [4.0, 2.0, 5.0]], [[8.0, 4.0, 
8.0], [3.0, 6.0, 5.0], [4.0, 8.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([8.0, 5.0, 1.0])+(1.-msk_val)*numpy.array([5.0, 8.0, 1.0])
      arg[2,1,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 5.0], [6.0, 8.0, 2.0], [8.0, 4.0, 2.0]], [[4.0, 8.0, 6.0], [8.0, 1.0, 1.0], [5.0, 
3.0, 3.0]], [[3.0, 2.0, 1.0], [4.0, 7.0, 1.0], [3.0, 3.0, 7.0]]], [[[6.0, 6.0, 7.0], [4.0, 2.0, 6.0], [8.0, 6.0, 7.0]], [[5.0, 
7.0, 2.0], [5.0, 3.0, 3.0], [5.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 6.0, 7.0], [2.0, 6.0, 7.0]]], [[[5.0, 4.0, 2.0], [8.0, 
4.0, 1.0], [4.0, 8.0, 2.0]], [[5.0, 8.0, 8.0], [1.0, 5.0, 6.0], [3.0, 1.0, 7.0]], [[3.0, 7.0, 3.0], [8.0, 1.0, 3.0], [3.0, 2.0, 
8.0]]], [[[4.0, 5.0, 5.0], [5.0, 1.0, 2.0], [2.0, 2.0, 1.0]], [[8.0, 2.0, 1.0], [4.0, 2.0, 6.0], [4.0, 2.0, 5.0]], [[8.0, 4.0, 
8.0], [3.0, 6.0, 5.0], [4.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [6.0, 8.0, 2.0], [8.0, 4.0, 2.0]], 
[[4.0, 8.0, 6.0], [8.0, 1.0, 1.0], [5.0, 3.0, 3.0]], [[3.0, 2.0, 1.0], [4.0, 7.0, 1.0], [3.0, 3.0, 7.0]]], [[[6.0, 6.0, 7.0], 
[4.0, 2.0, 6.0], [8.0, 6.0, 7.0]], [[5.0, 7.0, 2.0], [5.0, 3.0, 3.0], [5.0, 3.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 6.0, 7.0], 
[2.0, 6.0, 7.0]]], [[[5.0, 4.0, 2.0], [8.0, 4.0, 1.0], [4.0, 8.0, 2.0]], [[5.0, 5.0, 8.0], [1.0, 8.0, 6.0], [3.0, 1.0, 7.0]], 
[[3.0, 7.0, 3.0], [8.0, 1.0, 3.0], [3.0, 2.0, 8.0]]], [[[4.0, 5.0, 5.0], [5.0, 1.0, 2.0], [2.0, 2.0, 1.0]], [[8.0, 2.0, 1.0], 
[4.0, 2.0, 6.0], [4.0, 2.0, 5.0]], [[8.0, 4.0, 8.0], [3.0, 6.0, 5.0], [4.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCPN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 6.0], [1.0, 3.0, 1.0], [4.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [8.0, 4.0, 3.0], [7.0, 1.0, 
5.0]], [[5.0, 4.0, 8.0], [4.0, 3.0, 2.0], [5.0, 3.0, 1.0]]], [[[7.0, 3.0, 4.0], [7.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[1.0, 6.0, 
1.0], [1.0, 6.0, 1.0], [2.0, 8.0, 3.0]], [[7.0, 5.0, 2.0], [3.0, 8.0, 2.0], [2.0, 1.0, 7.0]]], [[[1.0, 8.0, 7.0], [3.0, 8.0, 
8.0], [8.0, 6.0, 7.0]], [[4.0, 4.0, 7.0], [2.0, 3.0, 1.0], [3.0, 6.0, 3.0]], [[1.0, 1.0, 3.0], [3.0, 3.0, 4.0], [8.0, 8.0, 
5.0]]], [[[2.0, 6.0, 1.0], [3.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[3.0, 5.0, 6.0], [6.0, 8.0, 6.0], [3.0, 2.0, 7.0]], [[5.0, 2.0, 
7.0], [6.0, 7.0, 8.0], [4.0, 4.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[4.0, 3.0, 6.0], [7.0, 7.0, 1.0], [3.0, 8.0, 4.0]])
      arg[2,:,1]=val
      ref=Data(numpy.array([[[[4.0, 6.0, 6.0], [1.0, 3.0, 1.0], [4.0, 1.0, 6.0]], [[2.0, 2.0, 8.0], [8.0, 4.0, 3.0], [7.0, 1.0, 
5.0]], [[5.0, 4.0, 8.0], [4.0, 3.0, 2.0], [5.0, 3.0, 1.0]]], [[[7.0, 3.0, 4.0], [7.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[1.0, 6.0, 
1.0], [1.0, 6.0, 1.0], [2.0, 8.0, 3.0]], [[7.0, 5.0, 2.0], [3.0, 8.0, 2.0], [2.0, 1.0, 7.0]]], [[[1.0, 8.0, 7.0], [4.0, 3.0, 
6.0], [8.0, 6.0, 7.0]], [[4.0, 4.0, 7.0], [7.0, 7.0, 1.0], [3.0, 6.0, 3.0]], [[1.0, 1.0, 3.0], [3.0, 8.0, 4.0], [8.0, 8.0, 
5.0]]], [[[2.0, 6.0, 1.0], [3.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[3.0, 5.0, 6.0], [6.0, 8.0, 6.0], [3.0, 2.0, 7.0]], [[5.0, 2.0, 
7.0], [6.0, 7.0, 8.0], [4.0, 4.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCPN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 6.0], [1.0, 4.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 5.0, 8.0], [2.0, 2.0, 2.0], [5.0, 7.0, 
1.0]], [[7.0, 1.0, 8.0], [4.0, 7.0, 2.0], [6.0, 7.0, 7.0]]], [[[8.0, 3.0, 8.0], [4.0, 2.0, 7.0], [6.0, 8.0, 8.0]], [[2.0, 8.0, 
6.0], [1.0, 6.0, 5.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 6.0], [8.0, 1.0, 6.0], [4.0, 3.0, 5.0]]], [[[4.0, 8.0, 8.0], [5.0, 5.0, 
6.0], [3.0, 7.0, 1.0]], [[7.0, 8.0, 3.0], [2.0, 2.0, 1.0], [1.0, 4.0, 6.0]], [[6.0, 8.0, 3.0], [8.0, 5.0, 5.0], [4.0, 2.0, 
7.0]]], [[[6.0, 1.0, 5.0], [8.0, 4.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[1.0, 8.0, 
8.0], [7.0, 7.0, 4.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 1.0, 6.0], [4.0, 3.0, 1.0], [5.0, 3.0, 6.0]]),self.functionspace)
      arg[2,:,1]=val
      ref=Data(numpy.array([[[[3.0, 3.0, 6.0], [1.0, 4.0, 4.0], [8.0, 4.0, 6.0]], [[1.0, 5.0, 8.0], [2.0, 2.0, 2.0], [5.0, 7.0, 
1.0]], [[7.0, 1.0, 8.0], [4.0, 7.0, 2.0], [6.0, 7.0, 7.0]]], [[[8.0, 3.0, 8.0], [4.0, 2.0, 7.0], [6.0, 8.0, 8.0]], [[2.0, 8.0, 
6.0], [1.0, 6.0, 5.0], [8.0, 5.0, 1.0]], [[6.0, 3.0, 6.0], [8.0, 1.0, 6.0], [4.0, 3.0, 5.0]]], [[[4.0, 8.0, 8.0], [2.0, 1.0, 
6.0], [3.0, 7.0, 1.0]], [[7.0, 8.0, 3.0], [4.0, 3.0, 1.0], [1.0, 4.0, 6.0]], [[6.0, 8.0, 3.0], [5.0, 3.0, 6.0], [4.0, 2.0, 
7.0]]], [[[6.0, 1.0, 5.0], [8.0, 4.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[1.0, 8.0, 
8.0], [7.0, 7.0, 4.0], [3.0, 2.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCPN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 8.0], [8.0, 5.0, 6.0], [7.0, 6.0, 4.0]], [[8.0, 3.0, 8.0], [4.0, 5.0, 4.0], [2.0, 5.0, 
4.0]], [[6.0, 3.0, 8.0], [2.0, 2.0, 7.0], [1.0, 7.0, 8.0]]], [[[8.0, 4.0, 6.0], [7.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[8.0, 2.0, 
2.0], [4.0, 1.0, 2.0], [8.0, 7.0, 8.0]], [[4.0, 4.0, 7.0], [3.0, 4.0, 6.0], [8.0, 8.0, 2.0]]], [[[1.0, 8.0, 4.0], [4.0, 7.0, 
7.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 5.0], [7.0, 4.0, 3.0], [6.0, 7.0, 1.0]], [[8.0, 8.0, 3.0], [6.0, 2.0, 3.0], [7.0, 6.0, 
2.0]]], [[[2.0, 4.0, 2.0], [1.0, 1.0, 5.0], [4.0, 8.0, 5.0]], [[6.0, 5.0, 5.0], [6.0, 6.0, 6.0], [1.0, 1.0, 1.0]], [[6.0, 2.0, 
3.0], [7.0, 8.0, 1.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 7.0, 6.0], [8.0, 7.0, 3.0], [1.0, 4.0, 2.0]])+(1.-msk_val)*numpy.array([[4.0, 5.0, 1.0], 
[8.0, 1.0, 5.0], [7.0, 2.0, 1.0]])
      arg[2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [8.0, 5.0, 6.0], [7.0, 6.0, 4.0]], [[8.0, 3.0, 8.0], [4.0, 5.0, 4.0], [2.0, 
5.0, 4.0]], [[6.0, 3.0, 8.0], [2.0, 2.0, 7.0], [1.0, 7.0, 8.0]]], [[[8.0, 4.0, 6.0], [7.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[8.0, 
2.0, 2.0], [4.0, 1.0, 2.0], [8.0, 7.0, 8.0]], [[4.0, 4.0, 7.0], [3.0, 4.0, 6.0], [8.0, 8.0, 2.0]]], [[[1.0, 8.0, 4.0], [2.0, 
7.0, 6.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 5.0], [8.0, 7.0, 3.0], [6.0, 7.0, 1.0]], [[8.0, 8.0, 3.0], [1.0, 4.0, 2.0], [7.0, 6.0, 
2.0]]], [[[2.0, 4.0, 2.0], [1.0, 1.0, 5.0], [4.0, 8.0, 5.0]], [[6.0, 5.0, 5.0], [6.0, 6.0, 6.0], [1.0, 1.0, 1.0]], [[6.0, 2.0, 
3.0], [7.0, 8.0, 1.0], [4.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 8.0], [8.0, 5.0, 6.0], [7.0, 6.0, 4.0]], 
[[8.0, 3.0, 8.0], [4.0, 5.0, 4.0], [2.0, 5.0, 4.0]], [[6.0, 3.0, 8.0], [2.0, 2.0, 7.0], [1.0, 7.0, 8.0]]], [[[8.0, 4.0, 6.0], 
[7.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[8.0, 2.0, 2.0], [4.0, 1.0, 2.0], [8.0, 7.0, 8.0]], [[4.0, 4.0, 7.0], [3.0, 4.0, 6.0], 
[8.0, 8.0, 2.0]]], [[[1.0, 8.0, 4.0], [4.0, 5.0, 1.0], [6.0, 8.0, 8.0]], [[7.0, 5.0, 5.0], [8.0, 1.0, 5.0], [6.0, 7.0, 1.0]], 
[[8.0, 8.0, 3.0], [7.0, 2.0, 1.0], [7.0, 6.0, 2.0]]], [[[2.0, 4.0, 2.0], [1.0, 1.0, 5.0], [4.0, 8.0, 5.0]], [[6.0, 5.0, 5.0], 
[6.0, 6.0, 6.0], [1.0, 1.0, 1.0]], [[6.0, 2.0, 3.0], [7.0, 8.0, 1.0], [4.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRRN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [1.0, 5.0, 6.0], [2.0, 6.0, 1.0]], [[7.0, 3.0, 1.0], [4.0, 7.0, 1.0], [8.0, 8.0, 
5.0]], [[6.0, 4.0, 4.0], [8.0, 2.0, 7.0], [3.0, 2.0, 7.0]]], [[[8.0, 4.0, 8.0], [1.0, 8.0, 1.0], [4.0, 7.0, 2.0]], [[8.0, 3.0, 
7.0], [8.0, 2.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 7.0], [6.0, 3.0, 3.0], [4.0, 7.0, 5.0]]], [[[6.0, 2.0, 4.0], [5.0, 2.0, 
8.0], [3.0, 6.0, 8.0]], [[4.0, 6.0, 6.0], [7.0, 1.0, 4.0], [2.0, 1.0, 5.0]], [[7.0, 2.0, 2.0], [2.0, 2.0, 7.0], [6.0, 6.0, 
4.0]]], [[[4.0, 4.0, 5.0], [3.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 7.0, 4.0], [2.0, 4.0, 1.0], [4.0, 6.0, 3.0]], [[2.0, 4.0, 
7.0], [2.0, 7.0, 5.0], [1.0, 5.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 4.0, 6.0]]])
      arg[2,1:2,1:2]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 6.0], [1.0, 5.0, 6.0], [2.0, 6.0, 1.0]], [[7.0, 3.0, 1.0], [4.0, 7.0, 1.0], [8.0, 8.0, 
5.0]], [[6.0, 4.0, 4.0], [8.0, 2.0, 7.0], [3.0, 2.0, 7.0]]], [[[8.0, 4.0, 8.0], [1.0, 8.0, 1.0], [4.0, 7.0, 2.0]], [[8.0, 3.0, 
7.0], [8.0, 2.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 7.0], [6.0, 3.0, 3.0], [4.0, 7.0, 5.0]]], [[[6.0, 2.0, 4.0], [5.0, 2.0, 
8.0], [3.0, 6.0, 8.0]], [[4.0, 6.0, 6.0], [3.0, 4.0, 6.0], [2.0, 1.0, 5.0]], [[7.0, 2.0, 2.0], [2.0, 2.0, 7.0], [6.0, 6.0, 
4.0]]], [[[4.0, 4.0, 5.0], [3.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[3.0, 7.0, 4.0], [2.0, 4.0, 1.0], [4.0, 6.0, 3.0]], [[2.0, 4.0, 
7.0], [2.0, 7.0, 5.0], [1.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRRN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 1.0], [8.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[5.0, 1.0, 6.0], [1.0, 8.0, 1.0], [3.0, 4.0, 
8.0]], [[8.0, 8.0, 5.0], [2.0, 3.0, 5.0], [1.0, 3.0, 3.0]]], [[[7.0, 4.0, 3.0], [7.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[3.0, 5.0, 
8.0], [5.0, 8.0, 8.0], [6.0, 7.0, 1.0]], [[5.0, 2.0, 1.0], [8.0, 3.0, 4.0], [1.0, 3.0, 8.0]]], [[[5.0, 8.0, 3.0], [3.0, 8.0, 
1.0], [1.0, 5.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 5.0, 6.0], [5.0, 7.0, 2.0]], [[2.0, 7.0, 7.0], [1.0, 6.0, 6.0], [4.0, 6.0, 
8.0]]], [[[3.0, 5.0, 7.0], [2.0, 7.0, 3.0], [6.0, 5.0, 8.0]], [[8.0, 6.0, 3.0], [4.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[7.0, 2.0, 
5.0], [8.0, 3.0, 2.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 1.0, 1.0]]]),self.functionspace)
      arg[2,1:2,1:2]=val
      ref=Data(numpy.array([[[[3.0, 5.0, 1.0], [8.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[5.0, 1.0, 6.0], [1.0, 8.0, 1.0], [3.0, 4.0, 
8.0]], [[8.0, 8.0, 5.0], [2.0, 3.0, 5.0], [1.0, 3.0, 3.0]]], [[[7.0, 4.0, 3.0], [7.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[3.0, 5.0, 
8.0], [5.0, 8.0, 8.0], [6.0, 7.0, 1.0]], [[5.0, 2.0, 1.0], [8.0, 3.0, 4.0], [1.0, 3.0, 8.0]]], [[[5.0, 8.0, 3.0], [3.0, 8.0, 
1.0], [1.0, 5.0, 6.0]], [[3.0, 5.0, 8.0], [7.0, 1.0, 1.0], [5.0, 7.0, 2.0]], [[2.0, 7.0, 7.0], [1.0, 6.0, 6.0], [4.0, 6.0, 
8.0]]], [[[3.0, 5.0, 7.0], [2.0, 7.0, 3.0], [6.0, 5.0, 8.0]], [[8.0, 6.0, 3.0], [4.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[7.0, 2.0, 
5.0], [8.0, 3.0, 2.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRRN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [1.0, 8.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 3.0, 3.0], [6.0, 4.0, 4.0], [3.0, 3.0, 
1.0]], [[2.0, 3.0, 7.0], [1.0, 7.0, 5.0], [3.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], [8.0, 2.0, 6.0], [4.0, 3.0, 3.0]], [[4.0, 6.0, 
2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 5.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 6.0], [7.0, 4.0, 7.0]]], [[[5.0, 1.0, 5.0], [4.0, 2.0, 
3.0], [6.0, 1.0, 3.0]], [[5.0, 5.0, 7.0], [3.0, 1.0, 1.0], [2.0, 1.0, 4.0]], [[1.0, 8.0, 2.0], [2.0, 6.0, 6.0], [4.0, 6.0, 
8.0]]], [[[5.0, 4.0, 1.0], [2.0, 1.0, 4.0], [6.0, 5.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 6.0, 6.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 
1.0], [2.0, 7.0, 7.0], [4.0, 2.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 4.0, 4.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0, 6.0]]])
      arg[2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [1.0, 8.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 3.0, 3.0], [6.0, 4.0, 4.0], [3.0, 
3.0, 1.0]], [[2.0, 3.0, 7.0], [1.0, 7.0, 5.0], [3.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], [8.0, 2.0, 6.0], [4.0, 3.0, 3.0]], [[4.0, 
6.0, 2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 5.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 6.0], [7.0, 4.0, 7.0]]], [[[5.0, 1.0, 5.0], [4.0, 
2.0, 3.0], [6.0, 1.0, 3.0]], [[5.0, 5.0, 7.0], [4.0, 4.0, 4.0], [2.0, 1.0, 4.0]], [[1.0, 8.0, 2.0], [2.0, 6.0, 6.0], [4.0, 6.0, 
8.0]]], [[[5.0, 4.0, 1.0], [2.0, 1.0, 4.0], [6.0, 5.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 6.0, 6.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 
1.0], [2.0, 7.0, 7.0], [4.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 3.0], [1.0, 8.0, 8.0], [5.0, 4.0, 7.0]], 
[[6.0, 3.0, 3.0], [6.0, 4.0, 4.0], [3.0, 3.0, 1.0]], [[2.0, 3.0, 7.0], [1.0, 7.0, 5.0], [3.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], 
[8.0, 2.0, 6.0], [4.0, 3.0, 3.0]], [[4.0, 6.0, 2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 5.0]], [[2.0, 8.0, 2.0], [1.0, 1.0, 6.0], 
[7.0, 4.0, 7.0]]], [[[5.0, 1.0, 5.0], [4.0, 2.0, 3.0], [6.0, 1.0, 3.0]], [[5.0, 5.0, 7.0], [5.0, 4.0, 6.0], [2.0, 1.0, 4.0]], 
[[1.0, 8.0, 2.0], [2.0, 6.0, 6.0], [4.0, 6.0, 8.0]]], [[[5.0, 4.0, 1.0], [2.0, 1.0, 4.0], [6.0, 5.0, 4.0]], [[7.0, 2.0, 8.0], 
[6.0, 6.0, 6.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 1.0], [2.0, 7.0, 7.0], [4.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLRN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 6.0], [8.0, 8.0, 4.0], [1.0, 5.0, 7.0]], [[4.0, 4.0, 4.0], [7.0, 5.0, 7.0], [8.0, 1.0, 
1.0]], [[5.0, 4.0, 2.0], [1.0, 2.0, 7.0], [8.0, 4.0, 8.0]]], [[[8.0, 3.0, 5.0], [5.0, 2.0, 2.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 
7.0], [2.0, 7.0, 4.0], [6.0, 1.0, 7.0]], [[2.0, 1.0, 1.0], [2.0, 7.0, 1.0], [5.0, 5.0, 3.0]]], [[[6.0, 2.0, 8.0], [2.0, 2.0, 
7.0], [8.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [4.0, 5.0, 6.0], [2.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [2.0, 1.0, 4.0], [2.0, 1.0, 
5.0]]], [[[1.0, 2.0, 2.0], [7.0, 3.0, 3.0], [5.0, 5.0, 7.0]], [[8.0, 7.0, 1.0], [8.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 1.0, 
5.0], [5.0, 2.0, 4.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[6.0, 8.0, 1.0]]])
      arg[2,2:,1:2]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 6.0], [8.0, 8.0, 4.0], [1.0, 5.0, 7.0]], [[4.0, 4.0, 4.0], [7.0, 5.0, 7.0], [8.0, 1.0, 
1.0]], [[5.0, 4.0, 2.0], [1.0, 2.0, 7.0], [8.0, 4.0, 8.0]]], [[[8.0, 3.0, 5.0], [5.0, 2.0, 2.0], [3.0, 1.0, 7.0]], [[6.0, 2.0, 
7.0], [2.0, 7.0, 4.0], [6.0, 1.0, 7.0]], [[2.0, 1.0, 1.0], [2.0, 7.0, 1.0], [5.0, 5.0, 3.0]]], [[[6.0, 2.0, 8.0], [2.0, 2.0, 
7.0], [8.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [4.0, 5.0, 6.0], [2.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [6.0, 8.0, 1.0], [2.0, 1.0, 
5.0]]], [[[1.0, 2.0, 2.0], [7.0, 3.0, 3.0], [5.0, 5.0, 7.0]], [[8.0, 7.0, 1.0], [8.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 1.0, 
5.0], [5.0, 2.0, 4.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLRN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 1.0], [6.0, 8.0, 3.0], [3.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], [8.0, 3.0, 2.0], [1.0, 5.0, 
7.0]], [[3.0, 2.0, 4.0], [4.0, 4.0, 3.0], [1.0, 2.0, 5.0]]], [[[6.0, 1.0, 3.0], [3.0, 1.0, 5.0], [5.0, 5.0, 2.0]], [[5.0, 8.0, 
2.0], [8.0, 5.0, 1.0], [2.0, 1.0, 6.0]], [[4.0, 4.0, 3.0], [4.0, 2.0, 5.0], [4.0, 6.0, 8.0]]], [[[4.0, 5.0, 2.0], [5.0, 7.0, 
7.0], [6.0, 8.0, 3.0]], [[6.0, 8.0, 7.0], [7.0, 4.0, 4.0], [3.0, 4.0, 1.0]], [[8.0, 8.0, 6.0], [8.0, 8.0, 2.0], [6.0, 1.0, 
6.0]]], [[[7.0, 5.0, 4.0], [7.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[8.0, 3.0, 2.0], [8.0, 2.0, 1.0], [4.0, 4.0, 5.0]], [[3.0, 4.0, 
6.0], [2.0, 6.0, 8.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 8.0, 8.0]]]),self.functionspace)
      arg[2,2:,1:2]=val
      ref=Data(numpy.array([[[[6.0, 6.0, 1.0], [6.0, 8.0, 3.0], [3.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], [8.0, 3.0, 2.0], [1.0, 5.0, 
7.0]], [[3.0, 2.0, 4.0], [4.0, 4.0, 3.0], [1.0, 2.0, 5.0]]], [[[6.0, 1.0, 3.0], [3.0, 1.0, 5.0], [5.0, 5.0, 2.0]], [[5.0, 8.0, 
2.0], [8.0, 5.0, 1.0], [2.0, 1.0, 6.0]], [[4.0, 4.0, 3.0], [4.0, 2.0, 5.0], [4.0, 6.0, 8.0]]], [[[4.0, 5.0, 2.0], [5.0, 7.0, 
7.0], [6.0, 8.0, 3.0]], [[6.0, 8.0, 7.0], [7.0, 4.0, 4.0], [3.0, 4.0, 1.0]], [[8.0, 8.0, 6.0], [6.0, 8.0, 8.0], [6.0, 1.0, 
6.0]]], [[[7.0, 5.0, 4.0], [7.0, 4.0, 6.0], [1.0, 3.0, 8.0]], [[8.0, 3.0, 2.0], [8.0, 2.0, 1.0], [4.0, 4.0, 5.0]], [[3.0, 4.0, 
6.0], [2.0, 6.0, 8.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLRN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 8.0], [3.0, 8.0, 4.0], [7.0, 4.0, 4.0]], [[8.0, 8.0, 5.0], [4.0, 3.0, 7.0], [3.0, 5.0, 
7.0]], [[6.0, 2.0, 6.0], [4.0, 8.0, 4.0], [1.0, 3.0, 1.0]]], [[[2.0, 6.0, 4.0], [6.0, 3.0, 7.0], [5.0, 6.0, 3.0]], [[3.0, 2.0, 
5.0], [1.0, 5.0, 7.0], [1.0, 5.0, 2.0]], [[2.0, 6.0, 1.0], [5.0, 1.0, 5.0], [3.0, 3.0, 8.0]]], [[[3.0, 7.0, 4.0], [8.0, 8.0, 
8.0], [8.0, 7.0, 3.0]], [[2.0, 3.0, 5.0], [4.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 3.0], [6.0, 1.0, 
1.0]]], [[[5.0, 1.0, 4.0], [6.0, 8.0, 4.0], [5.0, 2.0, 6.0]], [[8.0, 4.0, 7.0], [1.0, 2.0, 5.0], [1.0, 5.0, 6.0]], [[1.0, 2.0, 
7.0], [6.0, 3.0, 7.0], [2.0, 1.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 1.0, 7.0]]])+(1.-msk_val)*numpy.array([[[2.0, 4.0, 5.0]]])
      arg[2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 8.0], [3.0, 8.0, 4.0], [7.0, 4.0, 4.0]], [[8.0, 8.0, 5.0], [4.0, 3.0, 7.0], [3.0, 
5.0, 7.0]], [[6.0, 2.0, 6.0], [4.0, 8.0, 4.0], [1.0, 3.0, 1.0]]], [[[2.0, 6.0, 4.0], [6.0, 3.0, 7.0], [5.0, 6.0, 3.0]], [[3.0, 
2.0, 5.0], [1.0, 5.0, 7.0], [1.0, 5.0, 2.0]], [[2.0, 6.0, 1.0], [5.0, 1.0, 5.0], [3.0, 3.0, 8.0]]], [[[3.0, 7.0, 4.0], [8.0, 
8.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 3.0, 5.0], [4.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 2.0], [5.0, 1.0, 7.0], [6.0, 1.0, 
1.0]]], [[[5.0, 1.0, 4.0], [6.0, 8.0, 4.0], [5.0, 2.0, 6.0]], [[8.0, 4.0, 7.0], [1.0, 2.0, 5.0], [1.0, 5.0, 6.0]], [[1.0, 2.0, 
7.0], [6.0, 3.0, 7.0], [2.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 8.0], [3.0, 8.0, 4.0], [7.0, 4.0, 4.0]], 
[[8.0, 8.0, 5.0], [4.0, 3.0, 7.0], [3.0, 5.0, 7.0]], [[6.0, 2.0, 6.0], [4.0, 8.0, 4.0], [1.0, 3.0, 1.0]]], [[[2.0, 6.0, 4.0], 
[6.0, 3.0, 7.0], [5.0, 6.0, 3.0]], [[3.0, 2.0, 5.0], [1.0, 5.0, 7.0], [1.0, 5.0, 2.0]], [[2.0, 6.0, 1.0], [5.0, 1.0, 5.0], 
[3.0, 3.0, 8.0]]], [[[3.0, 7.0, 4.0], [8.0, 8.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 3.0, 5.0], [4.0, 6.0, 3.0], [1.0, 1.0, 5.0]], 
[[5.0, 8.0, 2.0], [2.0, 4.0, 5.0], [6.0, 1.0, 1.0]]], [[[5.0, 1.0, 4.0], [6.0, 8.0, 4.0], [5.0, 2.0, 6.0]], [[8.0, 4.0, 7.0], 
[1.0, 2.0, 5.0], [1.0, 5.0, 6.0]], [[1.0, 2.0, 7.0], [6.0, 3.0, 7.0], [2.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UPUN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [7.0, 8.0, 1.0], [3.0, 2.0, 1.0]], [[8.0, 6.0, 2.0], [6.0, 2.0, 2.0], [3.0, 8.0, 
7.0]], [[4.0, 5.0, 4.0], [3.0, 7.0, 2.0], [3.0, 8.0, 4.0]]], [[[2.0, 4.0, 8.0], [1.0, 6.0, 7.0], [5.0, 2.0, 1.0]], [[1.0, 5.0, 
3.0], [2.0, 4.0, 3.0], [4.0, 3.0, 3.0]], [[4.0, 5.0, 4.0], [6.0, 3.0, 2.0], [5.0, 7.0, 4.0]]], [[[2.0, 3.0, 5.0], [5.0, 2.0, 
4.0], [6.0, 4.0, 7.0]], [[7.0, 4.0, 1.0], [5.0, 3.0, 7.0], [1.0, 4.0, 6.0]], [[2.0, 8.0, 7.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
6.0]]], [[[8.0, 7.0, 7.0], [2.0, 1.0, 3.0], [1.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [8.0, 6.0, 6.0], [4.0, 3.0, 4.0]], [[7.0, 5.0, 
4.0], [3.0, 4.0, 6.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 3.0, 2.0], [7.0, 1.0, 1.0]], [[6.0, 1.0, 4.0], [5.0, 1.0, 2.0]], [[1.0, 3.0, 8.0], [3.0, 1.0, 
1.0]]])
      arg[:3,1,:2]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 5.0], [7.0, 8.0, 1.0], [3.0, 2.0, 1.0]], [[1.0, 3.0, 2.0], [7.0, 1.0, 1.0], [3.0, 8.0, 
7.0]], [[4.0, 5.0, 4.0], [3.0, 7.0, 2.0], [3.0, 8.0, 4.0]]], [[[2.0, 4.0, 8.0], [1.0, 6.0, 7.0], [5.0, 2.0, 1.0]], [[6.0, 1.0, 
4.0], [5.0, 1.0, 2.0], [4.0, 3.0, 3.0]], [[4.0, 5.0, 4.0], [6.0, 3.0, 2.0], [5.0, 7.0, 4.0]]], [[[2.0, 3.0, 5.0], [5.0, 2.0, 
4.0], [6.0, 4.0, 7.0]], [[1.0, 3.0, 8.0], [3.0, 1.0, 1.0], [1.0, 4.0, 6.0]], [[2.0, 8.0, 7.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
6.0]]], [[[8.0, 7.0, 7.0], [2.0, 1.0, 3.0], [1.0, 7.0, 3.0]], [[6.0, 6.0, 4.0], [8.0, 6.0, 6.0], [4.0, 3.0, 4.0]], [[7.0, 5.0, 
4.0], [3.0, 4.0, 6.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UPUN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 6.0], [2.0, 1.0, 6.0], [4.0, 8.0, 8.0]], [[8.0, 4.0, 1.0], [6.0, 3.0, 2.0], [8.0, 8.0, 
7.0]], [[4.0, 3.0, 1.0], [1.0, 3.0, 7.0], [8.0, 7.0, 5.0]]], [[[4.0, 5.0, 3.0], [6.0, 2.0, 5.0], [4.0, 1.0, 6.0]], [[3.0, 4.0, 
6.0], [2.0, 6.0, 5.0], [6.0, 3.0, 8.0]], [[5.0, 7.0, 2.0], [2.0, 2.0, 4.0], [6.0, 1.0, 5.0]]], [[[1.0, 2.0, 2.0], [4.0, 6.0, 
8.0], [8.0, 6.0, 2.0]], [[6.0, 2.0, 2.0], [5.0, 2.0, 8.0], [3.0, 2.0, 5.0]], [[5.0, 3.0, 5.0], [5.0, 3.0, 7.0], [8.0, 8.0, 
6.0]]], [[[3.0, 8.0, 8.0], [7.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[3.0, 4.0, 7.0], [1.0, 6.0, 5.0], [7.0, 7.0, 5.0]], [[2.0, 8.0, 
1.0], [7.0, 4.0, 8.0], [6.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 7.0, 3.0], [2.0, 7.0, 3.0]], [[6.0, 7.0, 2.0], [1.0, 6.0, 8.0]], [[8.0, 7.0, 4.0], [1.0, 
4.0, 4.0]]]),self.functionspace)
      arg[:3,1,:2]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 6.0], [2.0, 1.0, 6.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 3.0], [2.0, 7.0, 3.0], [8.0, 8.0, 
7.0]], [[4.0, 3.0, 1.0], [1.0, 3.0, 7.0], [8.0, 7.0, 5.0]]], [[[4.0, 5.0, 3.0], [6.0, 2.0, 5.0], [4.0, 1.0, 6.0]], [[6.0, 7.0, 
2.0], [1.0, 6.0, 8.0], [6.0, 3.0, 8.0]], [[5.0, 7.0, 2.0], [2.0, 2.0, 4.0], [6.0, 1.0, 5.0]]], [[[1.0, 2.0, 2.0], [4.0, 6.0, 
8.0], [8.0, 6.0, 2.0]], [[8.0, 7.0, 4.0], [1.0, 4.0, 4.0], [3.0, 2.0, 5.0]], [[5.0, 3.0, 5.0], [5.0, 3.0, 7.0], [8.0, 8.0, 
6.0]]], [[[3.0, 8.0, 8.0], [7.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[3.0, 4.0, 7.0], [1.0, 6.0, 5.0], [7.0, 7.0, 5.0]], [[2.0, 8.0, 
1.0], [7.0, 4.0, 8.0], [6.0, 4.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UPUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 5.0], [7.0, 1.0, 8.0], [7.0, 1.0, 8.0]], [[4.0, 4.0, 4.0], [5.0, 7.0, 1.0], [7.0, 7.0, 
5.0]], [[2.0, 5.0, 4.0], [4.0, 8.0, 4.0], [8.0, 1.0, 2.0]]], [[[2.0, 6.0, 5.0], [7.0, 6.0, 8.0], [5.0, 4.0, 1.0]], [[7.0, 4.0, 
6.0], [3.0, 1.0, 8.0], [6.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [4.0, 5.0, 8.0], [7.0, 3.0, 4.0]]], [[[8.0, 1.0, 5.0], [5.0, 7.0, 
4.0], [2.0, 5.0, 4.0]], [[5.0, 3.0, 7.0], [7.0, 2.0, 6.0], [8.0, 2.0, 8.0]], [[5.0, 6.0, 2.0], [5.0, 3.0, 4.0], [8.0, 8.0, 
7.0]]], [[[2.0, 6.0, 5.0], [6.0, 1.0, 6.0], [4.0, 6.0, 2.0]], [[7.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 6.0, 2.0]], [[1.0, 6.0, 
1.0], [6.0, 1.0, 8.0], [8.0, 4.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 8.0, 8.0], [8.0, 3.0, 7.0]], [[2.0, 7.0, 2.0], [5.0, 5.0, 3.0]], [[2.0, 8.0, 8.0], [4.0, 
4.0, 5.0]]])+(1.-msk_val)*numpy.array([[[2.0, 3.0, 7.0], [7.0, 3.0, 5.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 6.0]], [[2.0, 4.0, 
7.0], [6.0, 1.0, 3.0]]])
      arg[:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 5.0], [7.0, 1.0, 8.0], [7.0, 1.0, 8.0]], [[2.0, 8.0, 8.0], [8.0, 3.0, 7.0], [7.0, 
7.0, 5.0]], [[2.0, 5.0, 4.0], [4.0, 8.0, 4.0], [8.0, 1.0, 2.0]]], [[[2.0, 6.0, 5.0], [7.0, 6.0, 8.0], [5.0, 4.0, 1.0]], [[2.0, 
7.0, 2.0], [5.0, 5.0, 3.0], [6.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [4.0, 5.0, 8.0], [7.0, 3.0, 4.0]]], [[[8.0, 1.0, 5.0], [5.0, 
7.0, 4.0], [2.0, 5.0, 4.0]], [[2.0, 8.0, 8.0], [4.0, 4.0, 5.0], [8.0, 2.0, 8.0]], [[5.0, 6.0, 2.0], [5.0, 3.0, 4.0], [8.0, 8.0, 
7.0]]], [[[2.0, 6.0, 5.0], [6.0, 1.0, 6.0], [4.0, 6.0, 2.0]], [[7.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 6.0, 2.0]], [[1.0, 6.0, 
1.0], [6.0, 1.0, 8.0], [8.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 5.0], [7.0, 1.0, 8.0], [7.0, 1.0, 8.0]], 
[[2.0, 3.0, 7.0], [7.0, 3.0, 5.0], [7.0, 7.0, 5.0]], [[2.0, 5.0, 4.0], [4.0, 8.0, 4.0], [8.0, 1.0, 2.0]]], [[[2.0, 6.0, 5.0], 
[7.0, 6.0, 8.0], [5.0, 4.0, 1.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 6.0], [6.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [4.0, 5.0, 8.0], 
[7.0, 3.0, 4.0]]], [[[8.0, 1.0, 5.0], [5.0, 7.0, 4.0], [2.0, 5.0, 4.0]], [[2.0, 4.0, 7.0], [6.0, 1.0, 3.0], [8.0, 2.0, 8.0]], 
[[5.0, 6.0, 2.0], [5.0, 3.0, 4.0], [8.0, 8.0, 7.0]]], [[[2.0, 6.0, 5.0], [6.0, 1.0, 6.0], [4.0, 6.0, 2.0]], [[7.0, 8.0, 7.0], 
[5.0, 4.0, 5.0], [6.0, 6.0, 2.0]], [[1.0, 6.0, 1.0], [6.0, 1.0, 8.0], [8.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCRN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 5.0], [4.0, 7.0, 6.0], [3.0, 2.0, 5.0]], [[5.0, 3.0, 6.0], [1.0, 8.0, 3.0], [8.0, 5.0, 
4.0]], [[8.0, 5.0, 2.0], [5.0, 4.0, 5.0], [4.0, 1.0, 2.0]]], [[[6.0, 1.0, 6.0], [7.0, 6.0, 6.0], [3.0, 8.0, 6.0]], [[2.0, 7.0, 
2.0], [4.0, 2.0, 8.0], [8.0, 2.0, 1.0]], [[5.0, 3.0, 6.0], [4.0, 4.0, 1.0], [4.0, 8.0, 4.0]]], [[[6.0, 4.0, 5.0], [8.0, 1.0, 
2.0], [8.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [3.0, 5.0, 2.0], [5.0, 2.0, 2.0]], [[3.0, 4.0, 1.0], [7.0, 1.0, 7.0], [6.0, 4.0, 
1.0]]], [[[4.0, 5.0, 6.0], [5.0, 4.0, 7.0], [6.0, 3.0, 7.0]], [[3.0, 4.0, 5.0], [5.0, 7.0, 7.0], [1.0, 3.0, 8.0]], [[1.0, 2.0, 
3.0], [5.0, 3.0, 1.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 1.0, 1.0]], [[3.0, 3.0, 6.0]], [[8.0, 1.0, 8.0]]], [[[4.0, 1.0, 6.0]], [[1.0, 5.0, 5.0]], [[3.0, 
5.0, 1.0]]], [[[7.0, 3.0, 4.0]], [[1.0, 6.0, 3.0]], [[1.0, 2.0, 3.0]]], [[[5.0, 4.0, 7.0]], [[3.0, 7.0, 3.0]], [[8.0, 1.0, 
7.0]]]])
      arg[:,:,1:2]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 5.0], [2.0, 1.0, 1.0], [3.0, 2.0, 5.0]], [[5.0, 3.0, 6.0], [3.0, 3.0, 6.0], [8.0, 5.0, 
4.0]], [[8.0, 5.0, 2.0], [8.0, 1.0, 8.0], [4.0, 1.0, 2.0]]], [[[6.0, 1.0, 6.0], [4.0, 1.0, 6.0], [3.0, 8.0, 6.0]], [[2.0, 7.0, 
2.0], [1.0, 5.0, 5.0], [8.0, 2.0, 1.0]], [[5.0, 3.0, 6.0], [3.0, 5.0, 1.0], [4.0, 8.0, 4.0]]], [[[6.0, 4.0, 5.0], [7.0, 3.0, 
4.0], [8.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [1.0, 6.0, 3.0], [5.0, 2.0, 2.0]], [[3.0, 4.0, 1.0], [1.0, 2.0, 3.0], [6.0, 4.0, 
1.0]]], [[[4.0, 5.0, 6.0], [5.0, 4.0, 7.0], [6.0, 3.0, 7.0]], [[3.0, 4.0, 5.0], [3.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[1.0, 2.0, 
3.0], [8.0, 1.0, 7.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCRN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 1.0], [4.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 3.0], [7.0, 6.0, 7.0], [5.0, 2.0, 
5.0]], [[8.0, 1.0, 8.0], [1.0, 5.0, 8.0], [1.0, 7.0, 7.0]]], [[[6.0, 1.0, 3.0], [7.0, 8.0, 2.0], [3.0, 3.0, 5.0]], [[3.0, 3.0, 
2.0], [3.0, 5.0, 5.0], [1.0, 4.0, 7.0]], [[7.0, 3.0, 2.0], [7.0, 6.0, 8.0], [8.0, 1.0, 1.0]]], [[[7.0, 4.0, 7.0], [4.0, 3.0, 
2.0], [1.0, 4.0, 5.0]], [[8.0, 3.0, 2.0], [4.0, 3.0, 3.0], [3.0, 6.0, 5.0]], [[7.0, 4.0, 3.0], [7.0, 7.0, 3.0], [3.0, 3.0, 
4.0]]], [[[2.0, 5.0, 7.0], [3.0, 2.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 6.0, 5.0], [4.0, 8.0, 1.0], [8.0, 7.0, 7.0]], [[4.0, 8.0, 
1.0], [7.0, 7.0, 3.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 3.0, 5.0]], [[7.0, 2.0, 5.0]], [[6.0, 2.0, 2.0]]], [[[6.0, 8.0, 1.0]], [[1.0, 8.0, 2.0]], 
[[5.0, 4.0, 2.0]]], [[[3.0, 7.0, 4.0]], [[7.0, 5.0, 6.0]], [[4.0, 1.0, 5.0]]], [[[2.0, 5.0, 7.0]], [[4.0, 6.0, 4.0]], [[1.0, 
5.0, 4.0]]]]),self.functionspace)
      arg[:,:,1:2]=val
      ref=Data(numpy.array([[[[2.0, 8.0, 1.0], [4.0, 3.0, 5.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 3.0], [7.0, 2.0, 5.0], [5.0, 2.0, 
5.0]], [[8.0, 1.0, 8.0], [6.0, 2.0, 2.0], [1.0, 7.0, 7.0]]], [[[6.0, 1.0, 3.0], [6.0, 8.0, 1.0], [3.0, 3.0, 5.0]], [[3.0, 3.0, 
2.0], [1.0, 8.0, 2.0], [1.0, 4.0, 7.0]], [[7.0, 3.0, 2.0], [5.0, 4.0, 2.0], [8.0, 1.0, 1.0]]], [[[7.0, 4.0, 7.0], [3.0, 7.0, 
4.0], [1.0, 4.0, 5.0]], [[8.0, 3.0, 2.0], [7.0, 5.0, 6.0], [3.0, 6.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 5.0], [3.0, 3.0, 
4.0]]], [[[2.0, 5.0, 7.0], [2.0, 5.0, 7.0], [2.0, 8.0, 7.0]], [[7.0, 6.0, 5.0], [4.0, 6.0, 4.0], [8.0, 7.0, 7.0]], [[4.0, 8.0, 
1.0], [1.0, 5.0, 4.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCRN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 7.0], [1.0, 6.0, 5.0], [1.0, 4.0, 8.0]], [[6.0, 5.0, 4.0], [7.0, 4.0, 5.0], [5.0, 5.0, 
4.0]], [[5.0, 1.0, 7.0], [4.0, 4.0, 7.0], [3.0, 5.0, 5.0]]], [[[3.0, 1.0, 4.0], [4.0, 2.0, 4.0], [4.0, 3.0, 6.0]], [[7.0, 4.0, 
4.0], [3.0, 2.0, 7.0], [3.0, 2.0, 2.0]], [[1.0, 4.0, 5.0], [6.0, 5.0, 6.0], [7.0, 7.0, 5.0]]], [[[8.0, 7.0, 4.0], [2.0, 4.0, 
7.0], [2.0, 6.0, 7.0]], [[8.0, 6.0, 2.0], [3.0, 4.0, 4.0], [4.0, 6.0, 2.0]], [[4.0, 4.0, 8.0], [4.0, 4.0, 3.0], [1.0, 4.0, 
8.0]]], [[[3.0, 7.0, 6.0], [6.0, 2.0, 3.0], [6.0, 4.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 1.0, 8.0], [5.0, 6.0, 6.0]], [[6.0, 1.0, 
3.0], [4.0, 1.0, 4.0], [4.0, 4.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 2.0, 4.0]], [[2.0, 4.0, 4.0]], [[7.0, 7.0, 4.0]]], [[[8.0, 4.0, 3.0]], [[4.0, 2.0, 
6.0]], [[4.0, 6.0, 8.0]]], [[[5.0, 1.0, 4.0]], [[4.0, 5.0, 7.0]], [[5.0, 2.0, 7.0]]], [[[6.0, 8.0, 6.0]], [[3.0, 4.0, 1.0]], 
[[2.0, 2.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 5.0, 5.0]], [[1.0, 2.0, 6.0]], [[6.0, 3.0, 6.0]]], [[[7.0, 3.0, 1.0]], 
[[6.0, 2.0, 7.0]], [[3.0, 3.0, 6.0]]], [[[4.0, 8.0, 3.0]], [[5.0, 4.0, 8.0]], [[6.0, 5.0, 8.0]]], [[[7.0, 7.0, 4.0]], [[4.0, 
1.0, 3.0]], [[1.0, 5.0, 2.0]]]])
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 7.0], [4.0, 2.0, 4.0], [1.0, 4.0, 8.0]], [[6.0, 5.0, 4.0], [2.0, 4.0, 4.0], [5.0, 
5.0, 4.0]], [[5.0, 1.0, 7.0], [7.0, 7.0, 4.0], [3.0, 5.0, 5.0]]], [[[3.0, 1.0, 4.0], [8.0, 4.0, 3.0], [4.0, 3.0, 6.0]], [[7.0, 
4.0, 4.0], [4.0, 2.0, 6.0], [3.0, 2.0, 2.0]], [[1.0, 4.0, 5.0], [4.0, 6.0, 8.0], [7.0, 7.0, 5.0]]], [[[8.0, 7.0, 4.0], [5.0, 
1.0, 4.0], [2.0, 6.0, 7.0]], [[8.0, 6.0, 2.0], [4.0, 5.0, 7.0], [4.0, 6.0, 2.0]], [[4.0, 4.0, 8.0], [5.0, 2.0, 7.0], [1.0, 4.0, 
8.0]]], [[[3.0, 7.0, 6.0], [6.0, 8.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 4.0, 5.0], [3.0, 4.0, 1.0], [5.0, 6.0, 6.0]], [[6.0, 1.0, 
3.0], [2.0, 2.0, 7.0], [4.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 7.0], [3.0, 5.0, 5.0], [1.0, 4.0, 8.0]], 
[[6.0, 5.0, 4.0], [1.0, 2.0, 6.0], [5.0, 5.0, 4.0]], [[5.0, 1.0, 7.0], [6.0, 3.0, 6.0], [3.0, 5.0, 5.0]]], [[[3.0, 1.0, 4.0], 
[7.0, 3.0, 1.0], [4.0, 3.0, 6.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 7.0], [3.0, 2.0, 2.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 6.0], 
[7.0, 7.0, 5.0]]], [[[8.0, 7.0, 4.0], [4.0, 8.0, 3.0], [2.0, 6.0, 7.0]], [[8.0, 6.0, 2.0], [5.0, 4.0, 8.0], [4.0, 6.0, 2.0]], 
[[4.0, 4.0, 8.0], [6.0, 5.0, 8.0], [1.0, 4.0, 8.0]]], [[[3.0, 7.0, 6.0], [7.0, 7.0, 4.0], [6.0, 4.0, 6.0]], [[5.0, 4.0, 5.0], 
[4.0, 1.0, 3.0], [5.0, 6.0, 6.0]], [[6.0, 1.0, 3.0], [1.0, 5.0, 2.0], [4.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UULU(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [5.0, 4.0, 1.0], [5.0, 7.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 4.0, 1.0], [2.0, 1.0, 
4.0]], [[5.0, 7.0, 3.0], [3.0, 3.0, 8.0], [2.0, 4.0, 6.0]]], [[[6.0, 6.0, 7.0], [1.0, 1.0, 3.0], [6.0, 4.0, 1.0]], [[2.0, 7.0, 
1.0], [8.0, 3.0, 6.0], [1.0, 4.0, 8.0]], [[2.0, 5.0, 7.0], [3.0, 1.0, 5.0], [3.0, 6.0, 3.0]]], [[[8.0, 3.0, 4.0], [4.0, 1.0, 
1.0], [6.0, 4.0, 5.0]], [[8.0, 6.0, 7.0], [2.0, 7.0, 5.0], [2.0, 6.0, 2.0]], [[2.0, 5.0, 5.0], [7.0, 7.0, 4.0], [4.0, 4.0, 
1.0]]], [[[4.0, 2.0, 2.0], [8.0, 8.0, 6.0], [8.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [3.0, 8.0, 3.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 
7.0], [5.0, 8.0, 8.0], [8.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 4.0]], [[7.0, 6.0]]], [[[6.0, 1.0]], [[1.0, 7.0]]], [[[3.0, 4.0]], [[2.0, 3.0]]]])
      arg[:3,:2,2:,:2]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 7.0], [5.0, 4.0, 1.0], [8.0, 4.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 4.0, 1.0], [7.0, 6.0, 
4.0]], [[5.0, 7.0, 3.0], [3.0, 3.0, 8.0], [2.0, 4.0, 6.0]]], [[[6.0, 6.0, 7.0], [1.0, 1.0, 3.0], [6.0, 1.0, 1.0]], [[2.0, 7.0, 
1.0], [8.0, 3.0, 6.0], [1.0, 7.0, 8.0]], [[2.0, 5.0, 7.0], [3.0, 1.0, 5.0], [3.0, 6.0, 3.0]]], [[[8.0, 3.0, 4.0], [4.0, 1.0, 
1.0], [3.0, 4.0, 5.0]], [[8.0, 6.0, 7.0], [2.0, 7.0, 5.0], [2.0, 3.0, 2.0]], [[2.0, 5.0, 5.0], [7.0, 7.0, 4.0], [4.0, 4.0, 
1.0]]], [[[4.0, 2.0, 2.0], [8.0, 8.0, 6.0], [8.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [3.0, 8.0, 3.0], [8.0, 4.0, 8.0]], [[5.0, 1.0, 
7.0], [5.0, 8.0, 8.0], [8.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UULU(self):
      arg=Data(numpy.array([[[[7.0, 2.0, 1.0], [1.0, 7.0, 5.0], [2.0, 8.0, 2.0]], [[4.0, 6.0, 8.0], [4.0, 7.0, 4.0], [4.0, 6.0, 
4.0]], [[5.0, 1.0, 2.0], [7.0, 6.0, 5.0], [3.0, 1.0, 4.0]]], [[[6.0, 8.0, 1.0], [2.0, 6.0, 2.0], [8.0, 7.0, 1.0]], [[4.0, 8.0, 
1.0], [5.0, 4.0, 2.0], [1.0, 4.0, 6.0]], [[8.0, 7.0, 1.0], [5.0, 8.0, 4.0], [5.0, 6.0, 7.0]]], [[[5.0, 7.0, 2.0], [3.0, 7.0, 
7.0], [5.0, 3.0, 6.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 3.0], [6.0, 1.0, 8.0]], [[7.0, 1.0, 6.0], [4.0, 6.0, 3.0], [1.0, 7.0, 
5.0]]], [[[4.0, 1.0, 8.0], [2.0, 1.0, 4.0], [7.0, 2.0, 8.0]], [[1.0, 8.0, 1.0], [4.0, 1.0, 6.0], [6.0, 8.0, 7.0]], [[5.0, 7.0, 
3.0], [7.0, 7.0, 8.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 3.0]], [[8.0, 3.0]]], [[[1.0, 5.0]], [[8.0, 8.0]]], [[[1.0, 8.0]], [[1.0, 
6.0]]]]),self.functionspace)
      arg[:3,:2,2:,:2]=val
      ref=Data(numpy.array([[[[7.0, 2.0, 1.0], [1.0, 7.0, 5.0], [2.0, 3.0, 2.0]], [[4.0, 6.0, 8.0], [4.0, 7.0, 4.0], [8.0, 3.0, 
4.0]], [[5.0, 1.0, 2.0], [7.0, 6.0, 5.0], [3.0, 1.0, 4.0]]], [[[6.0, 8.0, 1.0], [2.0, 6.0, 2.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 
1.0], [5.0, 4.0, 2.0], [8.0, 8.0, 6.0]], [[8.0, 7.0, 1.0], [5.0, 8.0, 4.0], [5.0, 6.0, 7.0]]], [[[5.0, 7.0, 2.0], [3.0, 7.0, 
7.0], [1.0, 8.0, 6.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 3.0], [1.0, 6.0, 8.0]], [[7.0, 1.0, 6.0], [4.0, 6.0, 3.0], [1.0, 7.0, 
5.0]]], [[[4.0, 1.0, 8.0], [2.0, 1.0, 4.0], [7.0, 2.0, 8.0]], [[1.0, 8.0, 1.0], [4.0, 1.0, 6.0], [6.0, 8.0, 7.0]], [[5.0, 7.0, 
3.0], [7.0, 7.0, 8.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UULU(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 7.0], [7.0, 8.0, 1.0], [4.0, 8.0, 1.0]], [[8.0, 6.0, 5.0], [8.0, 1.0, 3.0], [3.0, 1.0, 
5.0]], [[1.0, 5.0, 8.0], [5.0, 7.0, 8.0], [3.0, 3.0, 4.0]]], [[[4.0, 4.0, 7.0], [8.0, 2.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 4.0, 
7.0], [5.0, 7.0, 4.0], [2.0, 3.0, 6.0]], [[2.0, 1.0, 2.0], [7.0, 4.0, 8.0], [3.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], [1.0, 8.0, 
2.0], [5.0, 3.0, 2.0]], [[6.0, 3.0, 4.0], [5.0, 8.0, 3.0], [1.0, 8.0, 8.0]], [[4.0, 1.0, 5.0], [4.0, 5.0, 5.0], [3.0, 7.0, 
6.0]]], [[[4.0, 3.0, 5.0], [4.0, 1.0, 6.0], [7.0, 3.0, 1.0]], [[7.0, 1.0, 4.0], [2.0, 2.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 1.0, 
7.0], [5.0, 8.0, 4.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 7.0]], [[7.0, 7.0]]], [[[5.0, 6.0]], [[3.0, 2.0]]], [[[6.0, 8.0]], [[4.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 5.0]], [[2.0, 6.0]]], [[[8.0, 6.0]], [[3.0, 5.0]]], [[[3.0, 1.0]], [[7.0, 4.0]]]])
      arg[:3,:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [7.0, 8.0, 1.0], [2.0, 7.0, 1.0]], [[8.0, 6.0, 5.0], [8.0, 1.0, 3.0], [7.0, 
7.0, 5.0]], [[1.0, 5.0, 8.0], [5.0, 7.0, 8.0], [3.0, 3.0, 4.0]]], [[[4.0, 4.0, 7.0], [8.0, 2.0, 6.0], [5.0, 6.0, 7.0]], [[5.0, 
4.0, 7.0], [5.0, 7.0, 4.0], [3.0, 2.0, 6.0]], [[2.0, 1.0, 2.0], [7.0, 4.0, 8.0], [3.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], [1.0, 
8.0, 2.0], [6.0, 8.0, 2.0]], [[6.0, 3.0, 4.0], [5.0, 8.0, 3.0], [4.0, 5.0, 8.0]], [[4.0, 1.0, 5.0], [4.0, 5.0, 5.0], [3.0, 7.0, 
6.0]]], [[[4.0, 3.0, 5.0], [4.0, 1.0, 6.0], [7.0, 3.0, 1.0]], [[7.0, 1.0, 4.0], [2.0, 2.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 1.0, 
7.0], [5.0, 8.0, 4.0], [7.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [7.0, 8.0, 1.0], [3.0, 5.0, 1.0]], 
[[8.0, 6.0, 5.0], [8.0, 1.0, 3.0], [2.0, 6.0, 5.0]], [[1.0, 5.0, 8.0], [5.0, 7.0, 8.0], [3.0, 3.0, 4.0]]], [[[4.0, 4.0, 7.0], 
[8.0, 2.0, 6.0], [8.0, 6.0, 7.0]], [[5.0, 4.0, 7.0], [5.0, 7.0, 4.0], [3.0, 5.0, 6.0]], [[2.0, 1.0, 2.0], [7.0, 4.0, 8.0], 
[3.0, 4.0, 5.0]]], [[[8.0, 1.0, 1.0], [1.0, 8.0, 2.0], [3.0, 1.0, 2.0]], [[6.0, 3.0, 4.0], [5.0, 8.0, 3.0], [7.0, 4.0, 8.0]], 
[[4.0, 1.0, 5.0], [4.0, 5.0, 5.0], [3.0, 7.0, 6.0]]], [[[4.0, 3.0, 5.0], [4.0, 1.0, 6.0], [7.0, 3.0, 1.0]], [[7.0, 1.0, 4.0], 
[2.0, 2.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 1.0, 7.0], [5.0, 8.0, 4.0], [7.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLCN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 5.0], [8.0, 7.0, 8.0], [6.0, 3.0, 2.0]], [[2.0, 1.0, 8.0], [3.0, 6.0, 2.0], [8.0, 2.0, 
4.0]], [[2.0, 4.0, 4.0], [8.0, 8.0, 8.0], [2.0, 8.0, 6.0]]], [[[8.0, 7.0, 4.0], [5.0, 7.0, 2.0], [7.0, 2.0, 6.0]], [[3.0, 7.0, 
8.0], [1.0, 8.0, 3.0], [8.0, 4.0, 3.0]], [[1.0, 6.0, 2.0], [1.0, 3.0, 8.0], [8.0, 5.0, 2.0]]], [[[4.0, 6.0, 7.0], [2.0, 4.0, 
8.0], [4.0, 3.0, 1.0]], [[5.0, 1.0, 5.0], [5.0, 3.0, 3.0], [8.0, 1.0, 3.0]], [[1.0, 3.0, 7.0], [5.0, 4.0, 1.0], [6.0, 6.0, 
5.0]]], [[[7.0, 4.0, 8.0], [8.0, 7.0, 4.0], [4.0, 7.0, 8.0]], [[8.0, 5.0, 7.0], [4.0, 5.0, 5.0], [5.0, 3.0, 7.0]], [[2.0, 3.0, 
5.0], [2.0, 2.0, 3.0], [5.0, 1.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 6.0, 2.0], [1.0, 4.0, 5.0], [4.0, 2.0, 5.0]]], [[[7.0, 4.0, 2.0], [8.0, 3.0, 1.0], [8.0, 2.0, 
4.0]]]])
      arg[2:,2:,:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 5.0], [8.0, 7.0, 8.0], [6.0, 3.0, 2.0]], [[2.0, 1.0, 8.0], [3.0, 6.0, 2.0], [8.0, 2.0, 
4.0]], [[2.0, 4.0, 4.0], [8.0, 8.0, 8.0], [2.0, 8.0, 6.0]]], [[[8.0, 7.0, 4.0], [5.0, 7.0, 2.0], [7.0, 2.0, 6.0]], [[3.0, 7.0, 
8.0], [1.0, 8.0, 3.0], [8.0, 4.0, 3.0]], [[1.0, 6.0, 2.0], [1.0, 3.0, 8.0], [8.0, 5.0, 2.0]]], [[[4.0, 6.0, 7.0], [2.0, 4.0, 
8.0], [4.0, 3.0, 1.0]], [[5.0, 1.0, 5.0], [5.0, 3.0, 3.0], [8.0, 1.0, 3.0]], [[6.0, 6.0, 2.0], [1.0, 4.0, 5.0], [4.0, 2.0, 
5.0]]], [[[7.0, 4.0, 8.0], [8.0, 7.0, 4.0], [4.0, 7.0, 8.0]], [[8.0, 5.0, 7.0], [4.0, 5.0, 5.0], [5.0, 3.0, 7.0]], [[7.0, 4.0, 
2.0], [8.0, 3.0, 1.0], [8.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLCN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 1.0], [2.0, 3.0, 4.0], [1.0, 6.0, 4.0]], [[5.0, 2.0, 7.0], [4.0, 7.0, 5.0], [6.0, 8.0, 
4.0]], [[1.0, 2.0, 3.0], [2.0, 5.0, 1.0], [8.0, 5.0, 6.0]]], [[[4.0, 7.0, 1.0], [3.0, 7.0, 4.0], [3.0, 4.0, 8.0]], [[5.0, 1.0, 
1.0], [1.0, 4.0, 6.0], [4.0, 3.0, 7.0]], [[2.0, 4.0, 7.0], [1.0, 2.0, 7.0], [3.0, 4.0, 8.0]]], [[[4.0, 3.0, 5.0], [6.0, 4.0, 
7.0], [4.0, 8.0, 8.0]], [[7.0, 1.0, 6.0], [8.0, 6.0, 3.0], [6.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [8.0, 7.0, 8.0], [3.0, 6.0, 
1.0]]], [[[7.0, 5.0, 2.0], [6.0, 8.0, 4.0], [4.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], [6.0, 8.0, 5.0], [5.0, 7.0, 6.0]], [[4.0, 6.0, 
6.0], [3.0, 7.0, 5.0], [4.0, 7.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 8.0, 5.0], [8.0, 6.0, 3.0], [2.0, 8.0, 3.0]]], [[[7.0, 8.0, 1.0], [5.0, 7.0, 3.0], [7.0, 
6.0, 5.0]]]]),self.functionspace)
      arg[2:,2:,:]=val
      ref=Data(numpy.array([[[[1.0, 6.0, 1.0], [2.0, 3.0, 4.0], [1.0, 6.0, 4.0]], [[5.0, 2.0, 7.0], [4.0, 7.0, 5.0], [6.0, 8.0, 
4.0]], [[1.0, 2.0, 3.0], [2.0, 5.0, 1.0], [8.0, 5.0, 6.0]]], [[[4.0, 7.0, 1.0], [3.0, 7.0, 4.0], [3.0, 4.0, 8.0]], [[5.0, 1.0, 
1.0], [1.0, 4.0, 6.0], [4.0, 3.0, 7.0]], [[2.0, 4.0, 7.0], [1.0, 2.0, 7.0], [3.0, 4.0, 8.0]]], [[[4.0, 3.0, 5.0], [6.0, 4.0, 
7.0], [4.0, 8.0, 8.0]], [[7.0, 1.0, 6.0], [8.0, 6.0, 3.0], [6.0, 8.0, 5.0]], [[1.0, 8.0, 5.0], [8.0, 6.0, 3.0], [2.0, 8.0, 
3.0]]], [[[7.0, 5.0, 2.0], [6.0, 8.0, 4.0], [4.0, 7.0, 1.0]], [[1.0, 4.0, 4.0], [6.0, 8.0, 5.0], [5.0, 7.0, 6.0]], [[7.0, 8.0, 
1.0], [5.0, 7.0, 3.0], [7.0, 6.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 1.0], [1.0, 5.0, 5.0], [8.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [6.0, 5.0, 1.0], [2.0, 4.0, 
1.0]], [[8.0, 7.0, 5.0], [8.0, 3.0, 1.0], [8.0, 4.0, 1.0]]], [[[2.0, 4.0, 2.0], [2.0, 2.0, 2.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 
3.0], [5.0, 2.0, 3.0], [8.0, 2.0, 1.0]], [[1.0, 7.0, 3.0], [2.0, 7.0, 4.0], [2.0, 3.0, 2.0]]], [[[7.0, 5.0, 4.0], [4.0, 2.0, 
7.0], [2.0, 2.0, 8.0]], [[2.0, 2.0, 3.0], [2.0, 7.0, 4.0], [8.0, 6.0, 7.0]], [[8.0, 8.0, 8.0], [8.0, 6.0, 1.0], [8.0, 3.0, 
7.0]]], [[[4.0, 4.0, 4.0], [6.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[6.0, 2.0, 6.0], [4.0, 3.0, 1.0], [8.0, 2.0, 6.0]], [[2.0, 1.0, 
2.0], [7.0, 7.0, 7.0], [6.0, 3.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 4.0, 8.0], [6.0, 2.0, 8.0], [4.0, 8.0, 3.0]]], [[[8.0, 1.0, 8.0], [8.0, 1.0, 6.0], [7.0, 
1.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 7.0, 5.0], [7.0, 6.0, 1.0], [8.0, 6.0, 7.0]]], [[[3.0, 2.0, 1.0], [2.0, 5.0, 
2.0], [4.0, 2.0, 6.0]]]])
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [1.0, 5.0, 5.0], [8.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [6.0, 5.0, 1.0], [2.0, 
4.0, 1.0]], [[8.0, 7.0, 5.0], [8.0, 3.0, 1.0], [8.0, 4.0, 1.0]]], [[[2.0, 4.0, 2.0], [2.0, 2.0, 2.0], [1.0, 4.0, 8.0]], [[3.0, 
5.0, 3.0], [5.0, 2.0, 3.0], [8.0, 2.0, 1.0]], [[1.0, 7.0, 3.0], [2.0, 7.0, 4.0], [2.0, 3.0, 2.0]]], [[[7.0, 5.0, 4.0], [4.0, 
2.0, 7.0], [2.0, 2.0, 8.0]], [[2.0, 2.0, 3.0], [2.0, 7.0, 4.0], [8.0, 6.0, 7.0]], [[4.0, 4.0, 8.0], [6.0, 2.0, 8.0], [4.0, 8.0, 
3.0]]], [[[4.0, 4.0, 4.0], [6.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[6.0, 2.0, 6.0], [4.0, 3.0, 1.0], [8.0, 2.0, 6.0]], [[8.0, 1.0, 
8.0], [8.0, 1.0, 6.0], [7.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 1.0], [1.0, 5.0, 5.0], [8.0, 1.0, 8.0]], 
[[5.0, 1.0, 6.0], [6.0, 5.0, 1.0], [2.0, 4.0, 1.0]], [[8.0, 7.0, 5.0], [8.0, 3.0, 1.0], [8.0, 4.0, 1.0]]], [[[2.0, 4.0, 2.0], 
[2.0, 2.0, 2.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [5.0, 2.0, 3.0], [8.0, 2.0, 1.0]], [[1.0, 7.0, 3.0], [2.0, 7.0, 4.0], 
[2.0, 3.0, 2.0]]], [[[7.0, 5.0, 4.0], [4.0, 2.0, 7.0], [2.0, 2.0, 8.0]], [[2.0, 2.0, 3.0], [2.0, 7.0, 4.0], [8.0, 6.0, 7.0]], 
[[7.0, 7.0, 5.0], [7.0, 6.0, 1.0], [8.0, 6.0, 7.0]]], [[[4.0, 4.0, 4.0], [6.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[6.0, 2.0, 6.0], 
[4.0, 3.0, 1.0], [8.0, 2.0, 6.0]], [[3.0, 2.0, 1.0], [2.0, 5.0, 2.0], [4.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPCN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 6.0], [3.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[4.0, 6.0, 7.0], [5.0, 3.0, 3.0], [7.0, 1.0, 
7.0]], [[8.0, 2.0, 2.0], [2.0, 7.0, 2.0], [3.0, 4.0, 8.0]]], [[[2.0, 5.0, 5.0], [3.0, 4.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 6.0, 
1.0], [6.0, 4.0, 8.0], [2.0, 4.0, 8.0]], [[3.0, 8.0, 3.0], [2.0, 1.0, 1.0], [6.0, 6.0, 8.0]]], [[[7.0, 5.0, 7.0], [8.0, 7.0, 
4.0], [7.0, 8.0, 7.0]], [[1.0, 1.0, 6.0], [8.0, 8.0, 6.0], [2.0, 3.0, 3.0]], [[1.0, 5.0, 5.0], [6.0, 6.0, 7.0], [7.0, 2.0, 
6.0]]], [[[6.0, 7.0, 6.0], [7.0, 4.0, 2.0], [2.0, 4.0, 6.0]], [[2.0, 3.0, 2.0], [3.0, 7.0, 1.0], [1.0, 8.0, 4.0]], [[1.0, 1.0, 
2.0], [5.0, 7.0, 4.0], [5.0, 5.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[6.0, 3.0, 5.0], [1.0, 5.0, 2.0], [3.0, 6.0, 4.0]])
      arg[2,1,:]=val
      ref=Data(numpy.array([[[[2.0, 1.0, 6.0], [3.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[4.0, 6.0, 7.0], [5.0, 3.0, 3.0], [7.0, 1.0, 
7.0]], [[8.0, 2.0, 2.0], [2.0, 7.0, 2.0], [3.0, 4.0, 8.0]]], [[[2.0, 5.0, 5.0], [3.0, 4.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 6.0, 
1.0], [6.0, 4.0, 8.0], [2.0, 4.0, 8.0]], [[3.0, 8.0, 3.0], [2.0, 1.0, 1.0], [6.0, 6.0, 8.0]]], [[[7.0, 5.0, 7.0], [8.0, 7.0, 
4.0], [7.0, 8.0, 7.0]], [[6.0, 3.0, 5.0], [1.0, 5.0, 2.0], [3.0, 6.0, 4.0]], [[1.0, 5.0, 5.0], [6.0, 6.0, 7.0], [7.0, 2.0, 
6.0]]], [[[6.0, 7.0, 6.0], [7.0, 4.0, 2.0], [2.0, 4.0, 6.0]], [[2.0, 3.0, 2.0], [3.0, 7.0, 1.0], [1.0, 8.0, 4.0]], [[1.0, 1.0, 
2.0], [5.0, 7.0, 4.0], [5.0, 5.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPCN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 8.0], [2.0, 8.0, 4.0], [1.0, 8.0, 5.0]], [[3.0, 5.0, 8.0], [7.0, 3.0, 4.0], [7.0, 6.0, 
5.0]], [[7.0, 6.0, 3.0], [8.0, 6.0, 6.0], [3.0, 4.0, 6.0]]], [[[4.0, 3.0, 6.0], [8.0, 7.0, 3.0], [1.0, 7.0, 6.0]], [[4.0, 7.0, 
5.0], [2.0, 4.0, 5.0], [6.0, 7.0, 2.0]], [[4.0, 5.0, 3.0], [2.0, 8.0, 4.0], [6.0, 8.0, 6.0]]], [[[8.0, 2.0, 1.0], [1.0, 2.0, 
1.0], [4.0, 1.0, 7.0]], [[1.0, 5.0, 7.0], [3.0, 2.0, 4.0], [4.0, 2.0, 5.0]], [[7.0, 7.0, 7.0], [1.0, 6.0, 3.0], [4.0, 7.0, 
4.0]]], [[[6.0, 8.0, 5.0], [6.0, 5.0, 2.0], [5.0, 8.0, 2.0]], [[8.0, 8.0, 2.0], [8.0, 7.0, 5.0], [1.0, 3.0, 7.0]], [[5.0, 3.0, 
2.0], [3.0, 8.0, 1.0], [5.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[7.0, 6.0, 6.0], [3.0, 3.0, 4.0], [7.0, 1.0, 3.0]]),self.functionspace)
      arg[2,1,:]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 8.0], [2.0, 8.0, 4.0], [1.0, 8.0, 5.0]], [[3.0, 5.0, 8.0], [7.0, 3.0, 4.0], [7.0, 6.0, 
5.0]], [[7.0, 6.0, 3.0], [8.0, 6.0, 6.0], [3.0, 4.0, 6.0]]], [[[4.0, 3.0, 6.0], [8.0, 7.0, 3.0], [1.0, 7.0, 6.0]], [[4.0, 7.0, 
5.0], [2.0, 4.0, 5.0], [6.0, 7.0, 2.0]], [[4.0, 5.0, 3.0], [2.0, 8.0, 4.0], [6.0, 8.0, 6.0]]], [[[8.0, 2.0, 1.0], [1.0, 2.0, 
1.0], [4.0, 1.0, 7.0]], [[7.0, 6.0, 6.0], [3.0, 3.0, 4.0], [7.0, 1.0, 3.0]], [[7.0, 7.0, 7.0], [1.0, 6.0, 3.0], [4.0, 7.0, 
4.0]]], [[[6.0, 8.0, 5.0], [6.0, 5.0, 2.0], [5.0, 8.0, 2.0]], [[8.0, 8.0, 2.0], [8.0, 7.0, 5.0], [1.0, 3.0, 7.0]], [[5.0, 3.0, 
2.0], [3.0, 8.0, 1.0], [5.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPCN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 1.0], [3.0, 3.0, 6.0], [8.0, 5.0, 3.0]], [[8.0, 8.0, 1.0], [1.0, 5.0, 4.0], [1.0, 7.0, 
8.0]], [[5.0, 7.0, 3.0], [4.0, 4.0, 8.0], [7.0, 1.0, 4.0]]], [[[7.0, 7.0, 5.0], [3.0, 7.0, 4.0], [7.0, 2.0, 4.0]], [[7.0, 8.0, 
4.0], [7.0, 6.0, 3.0], [7.0, 4.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 7.0, 6.0], [8.0, 6.0, 4.0]]], [[[5.0, 4.0, 2.0], [1.0, 4.0, 
3.0], [3.0, 5.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 5.0, 5.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 2.0], [2.0, 7.0, 3.0], [8.0, 8.0, 
3.0]]], [[[2.0, 7.0, 4.0], [3.0, 5.0, 4.0], [3.0, 8.0, 7.0]], [[6.0, 1.0, 4.0], [3.0, 2.0, 7.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 
1.0], [7.0, 7.0, 2.0], [8.0, 1.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 6.0, 1.0], [7.0, 1.0, 4.0], [6.0, 4.0, 8.0]])+(1.-msk_val)*numpy.array([[7.0, 2.0, 4.0], 
[8.0, 6.0, 1.0], [4.0, 5.0, 4.0]])
      arg[2,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 1.0], [3.0, 3.0, 6.0], [8.0, 5.0, 3.0]], [[8.0, 8.0, 1.0], [1.0, 5.0, 4.0], [1.0, 
7.0, 8.0]], [[5.0, 7.0, 3.0], [4.0, 4.0, 8.0], [7.0, 1.0, 4.0]]], [[[7.0, 7.0, 5.0], [3.0, 7.0, 4.0], [7.0, 2.0, 4.0]], [[7.0, 
8.0, 4.0], [7.0, 6.0, 3.0], [7.0, 4.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 7.0, 6.0], [8.0, 6.0, 4.0]]], [[[5.0, 4.0, 2.0], [1.0, 
4.0, 3.0], [3.0, 5.0, 5.0]], [[1.0, 6.0, 1.0], [7.0, 1.0, 4.0], [6.0, 4.0, 8.0]], [[4.0, 7.0, 2.0], [2.0, 7.0, 3.0], [8.0, 8.0, 
3.0]]], [[[2.0, 7.0, 4.0], [3.0, 5.0, 4.0], [3.0, 8.0, 7.0]], [[6.0, 1.0, 4.0], [3.0, 2.0, 7.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 
1.0], [7.0, 7.0, 2.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 1.0], [3.0, 3.0, 6.0], [8.0, 5.0, 3.0]], 
[[8.0, 8.0, 1.0], [1.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[5.0, 7.0, 3.0], [4.0, 4.0, 8.0], [7.0, 1.0, 4.0]]], [[[7.0, 7.0, 5.0], 
[3.0, 7.0, 4.0], [7.0, 2.0, 4.0]], [[7.0, 8.0, 4.0], [7.0, 6.0, 3.0], [7.0, 4.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 7.0, 6.0], 
[8.0, 6.0, 4.0]]], [[[5.0, 4.0, 2.0], [1.0, 4.0, 3.0], [3.0, 5.0, 5.0]], [[7.0, 2.0, 4.0], [8.0, 6.0, 1.0], [4.0, 5.0, 4.0]], 
[[4.0, 7.0, 2.0], [2.0, 7.0, 3.0], [8.0, 8.0, 3.0]]], [[[2.0, 7.0, 4.0], [3.0, 5.0, 4.0], [3.0, 8.0, 7.0]], [[6.0, 1.0, 4.0], 
[3.0, 2.0, 7.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 1.0], [7.0, 7.0, 2.0], [8.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRUN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 1.0], [1.0, 1.0, 8.0], [8.0, 4.0, 8.0]], [[7.0, 5.0, 7.0], [4.0, 1.0, 4.0], [2.0, 4.0, 
5.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 1.0], [6.0, 6.0, 3.0]]], [[[1.0, 3.0, 5.0], [4.0, 7.0, 6.0], [5.0, 1.0, 4.0]], [[3.0, 5.0, 
1.0], [3.0, 5.0, 2.0], [2.0, 6.0, 6.0]], [[5.0, 1.0, 6.0], [1.0, 8.0, 4.0], [6.0, 4.0, 4.0]]], [[[7.0, 4.0, 4.0], [6.0, 7.0, 
7.0], [2.0, 8.0, 5.0]], [[8.0, 5.0, 3.0], [7.0, 2.0, 5.0], [7.0, 5.0, 3.0]], [[7.0, 3.0, 3.0], [5.0, 2.0, 6.0], [8.0, 7.0, 
6.0]]], [[[6.0, 2.0, 5.0], [8.0, 8.0, 2.0], [5.0, 1.0, 2.0]], [[4.0, 8.0, 8.0], [5.0, 6.0, 8.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 
4.0], [4.0, 4.0, 3.0], [8.0, 8.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 2.0, 3.0], [8.0, 8.0, 2.0]]], [[[6.0, 4.0, 3.0], [1.0, 6.0, 4.0]]]])
      arg[1:3,1:2,:2]=val
      ref=Data(numpy.array([[[[7.0, 5.0, 1.0], [1.0, 1.0, 8.0], [8.0, 4.0, 8.0]], [[7.0, 5.0, 7.0], [4.0, 1.0, 4.0], [2.0, 4.0, 
5.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 1.0], [6.0, 6.0, 3.0]]], [[[1.0, 3.0, 5.0], [4.0, 7.0, 6.0], [5.0, 1.0, 4.0]], [[7.0, 2.0, 
3.0], [8.0, 8.0, 2.0], [2.0, 6.0, 6.0]], [[5.0, 1.0, 6.0], [1.0, 8.0, 4.0], [6.0, 4.0, 4.0]]], [[[7.0, 4.0, 4.0], [6.0, 7.0, 
7.0], [2.0, 8.0, 5.0]], [[6.0, 4.0, 3.0], [1.0, 6.0, 4.0], [7.0, 5.0, 3.0]], [[7.0, 3.0, 3.0], [5.0, 2.0, 6.0], [8.0, 7.0, 
6.0]]], [[[6.0, 2.0, 5.0], [8.0, 8.0, 2.0], [5.0, 1.0, 2.0]], [[4.0, 8.0, 8.0], [5.0, 6.0, 8.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 
4.0], [4.0, 4.0, 3.0], [8.0, 8.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRUN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 5.0], [2.0, 5.0, 3.0], [3.0, 2.0, 3.0]], [[3.0, 3.0, 2.0], [1.0, 4.0, 6.0], [7.0, 5.0, 
4.0]], [[6.0, 2.0, 2.0], [5.0, 5.0, 3.0], [8.0, 6.0, 4.0]]], [[[8.0, 6.0, 1.0], [8.0, 7.0, 3.0], [3.0, 3.0, 7.0]], [[3.0, 5.0, 
6.0], [6.0, 8.0, 5.0], [8.0, 6.0, 5.0]], [[2.0, 3.0, 7.0], [5.0, 5.0, 2.0], [2.0, 7.0, 3.0]]], [[[7.0, 7.0, 2.0], [5.0, 3.0, 
6.0], [6.0, 6.0, 7.0]], [[7.0, 5.0, 7.0], [7.0, 3.0, 2.0], [8.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 1.0, 6.0], [1.0, 4.0, 
4.0]]], [[[7.0, 6.0, 2.0], [6.0, 4.0, 8.0], [3.0, 3.0, 6.0]], [[5.0, 3.0, 7.0], [5.0, 2.0, 8.0], [3.0, 1.0, 7.0]], [[4.0, 4.0, 
1.0], [5.0, 5.0, 4.0], [5.0, 8.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 1.0, 5.0], [3.0, 6.0, 8.0]]], [[[5.0, 3.0, 4.0], [7.0, 2.0, 1.0]]]]),self.functionspace)
      arg[1:3,1:2,:2]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 5.0], [2.0, 5.0, 3.0], [3.0, 2.0, 3.0]], [[3.0, 3.0, 2.0], [1.0, 4.0, 6.0], [7.0, 5.0, 
4.0]], [[6.0, 2.0, 2.0], [5.0, 5.0, 3.0], [8.0, 6.0, 4.0]]], [[[8.0, 6.0, 1.0], [8.0, 7.0, 3.0], [3.0, 3.0, 7.0]], [[7.0, 1.0, 
5.0], [3.0, 6.0, 8.0], [8.0, 6.0, 5.0]], [[2.0, 3.0, 7.0], [5.0, 5.0, 2.0], [2.0, 7.0, 3.0]]], [[[7.0, 7.0, 2.0], [5.0, 3.0, 
6.0], [6.0, 6.0, 7.0]], [[5.0, 3.0, 4.0], [7.0, 2.0, 1.0], [8.0, 5.0, 4.0]], [[3.0, 8.0, 5.0], [8.0, 1.0, 6.0], [1.0, 4.0, 
4.0]]], [[[7.0, 6.0, 2.0], [6.0, 4.0, 8.0], [3.0, 3.0, 6.0]], [[5.0, 3.0, 7.0], [5.0, 2.0, 8.0], [3.0, 1.0, 7.0]], [[4.0, 4.0, 
1.0], [5.0, 5.0, 4.0], [5.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRUN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 8.0], [2.0, 4.0, 3.0], [3.0, 3.0, 3.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 4.0], [3.0, 5.0, 
4.0]], [[7.0, 5.0, 6.0], [6.0, 7.0, 3.0], [3.0, 2.0, 6.0]]], [[[2.0, 4.0, 7.0], [5.0, 7.0, 3.0], [7.0, 6.0, 5.0]], [[3.0, 1.0, 
8.0], [1.0, 4.0, 3.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], [2.0, 5.0, 7.0], [5.0, 5.0, 4.0]]], [[[5.0, 2.0, 2.0], [2.0, 8.0, 
2.0], [5.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [1.0, 6.0, 1.0], [3.0, 7.0, 3.0]], [[3.0, 5.0, 3.0], [1.0, 3.0, 1.0], [2.0, 4.0, 
4.0]]], [[[3.0, 2.0, 7.0], [3.0, 2.0, 7.0], [8.0, 5.0, 6.0]], [[5.0, 8.0, 2.0], [4.0, 8.0, 4.0], [7.0, 3.0, 1.0]], [[2.0, 6.0, 
6.0], [6.0, 7.0, 7.0], [2.0, 6.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 5.0, 5.0], [2.0, 3.0, 7.0]]], [[[1.0, 1.0, 6.0], [3.0, 1.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 8.0, 6.0], [2.0, 1.0, 6.0]]], [[[4.0, 3.0, 2.0], [8.0, 8.0, 6.0]]]])
      arg[1:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 8.0], [2.0, 4.0, 3.0], [3.0, 3.0, 3.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 4.0], [3.0, 
5.0, 4.0]], [[7.0, 5.0, 6.0], [6.0, 7.0, 3.0], [3.0, 2.0, 6.0]]], [[[2.0, 4.0, 7.0], [5.0, 7.0, 3.0], [7.0, 6.0, 5.0]], [[4.0, 
5.0, 5.0], [2.0, 3.0, 7.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], [2.0, 5.0, 7.0], [5.0, 5.0, 4.0]]], [[[5.0, 2.0, 2.0], [2.0, 
8.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 1.0, 6.0], [3.0, 1.0, 1.0], [3.0, 7.0, 3.0]], [[3.0, 5.0, 3.0], [1.0, 3.0, 1.0], [2.0, 4.0, 
4.0]]], [[[3.0, 2.0, 7.0], [3.0, 2.0, 7.0], [8.0, 5.0, 6.0]], [[5.0, 8.0, 2.0], [4.0, 8.0, 4.0], [7.0, 3.0, 1.0]], [[2.0, 6.0, 
6.0], [6.0, 7.0, 7.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 8.0], [2.0, 4.0, 3.0], [3.0, 3.0, 3.0]], 
[[8.0, 4.0, 4.0], [5.0, 5.0, 4.0], [3.0, 5.0, 4.0]], [[7.0, 5.0, 6.0], [6.0, 7.0, 3.0], [3.0, 2.0, 6.0]]], [[[2.0, 4.0, 7.0], 
[5.0, 7.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 8.0, 6.0], [2.0, 1.0, 6.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 1.0], [2.0, 5.0, 7.0], 
[5.0, 5.0, 4.0]]], [[[5.0, 2.0, 2.0], [2.0, 8.0, 2.0], [5.0, 7.0, 3.0]], [[4.0, 3.0, 2.0], [8.0, 8.0, 6.0], [3.0, 7.0, 3.0]], 
[[3.0, 5.0, 3.0], [1.0, 3.0, 1.0], [2.0, 4.0, 4.0]]], [[[3.0, 2.0, 7.0], [3.0, 2.0, 7.0], [8.0, 5.0, 6.0]], [[5.0, 8.0, 2.0], 
[4.0, 8.0, 4.0], [7.0, 3.0, 1.0]], [[2.0, 6.0, 6.0], [6.0, 7.0, 7.0], [2.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPRN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 6.0], [2.0, 5.0, 8.0], [5.0, 2.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 6.0, 3.0], [2.0, 7.0, 
6.0]], [[2.0, 1.0, 2.0], [3.0, 2.0, 8.0], [2.0, 7.0, 6.0]]], [[[3.0, 8.0, 1.0], [1.0, 4.0, 5.0], [3.0, 8.0, 6.0]], [[5.0, 1.0, 
1.0], [2.0, 5.0, 6.0], [2.0, 8.0, 2.0]], [[5.0, 6.0, 4.0], [2.0, 7.0, 4.0], [5.0, 2.0, 2.0]]], [[[5.0, 8.0, 5.0], [3.0, 1.0, 
6.0], [8.0, 1.0, 6.0]], [[3.0, 1.0, 1.0], [1.0, 2.0, 2.0], [7.0, 3.0, 2.0]], [[1.0, 1.0, 1.0], [3.0, 8.0, 8.0], [7.0, 3.0, 
2.0]]], [[[1.0, 5.0, 7.0], [1.0, 1.0, 5.0], [2.0, 4.0, 5.0]], [[2.0, 4.0, 7.0], [6.0, 4.0, 2.0], [6.0, 4.0, 5.0]], [[7.0, 2.0, 
2.0], [7.0, 3.0, 1.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[3.0, 6.0, 5.0]])
      arg[2,1,1:2]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 6.0], [2.0, 5.0, 8.0], [5.0, 2.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 6.0, 3.0], [2.0, 7.0, 
6.0]], [[2.0, 1.0, 2.0], [3.0, 2.0, 8.0], [2.0, 7.0, 6.0]]], [[[3.0, 8.0, 1.0], [1.0, 4.0, 5.0], [3.0, 8.0, 6.0]], [[5.0, 1.0, 
1.0], [2.0, 5.0, 6.0], [2.0, 8.0, 2.0]], [[5.0, 6.0, 4.0], [2.0, 7.0, 4.0], [5.0, 2.0, 2.0]]], [[[5.0, 8.0, 5.0], [3.0, 1.0, 
6.0], [8.0, 1.0, 6.0]], [[3.0, 1.0, 1.0], [3.0, 6.0, 5.0], [7.0, 3.0, 2.0]], [[1.0, 1.0, 1.0], [3.0, 8.0, 8.0], [7.0, 3.0, 
2.0]]], [[[1.0, 5.0, 7.0], [1.0, 1.0, 5.0], [2.0, 4.0, 5.0]], [[2.0, 4.0, 7.0], [6.0, 4.0, 2.0], [6.0, 4.0, 5.0]], [[7.0, 2.0, 
2.0], [7.0, 3.0, 1.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPRN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 4.0], [6.0, 8.0, 4.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 1.0], [7.0, 2.0, 4.0], [1.0, 5.0, 
2.0]], [[8.0, 2.0, 7.0], [2.0, 3.0, 6.0], [7.0, 6.0, 8.0]]], [[[1.0, 2.0, 2.0], [4.0, 5.0, 1.0], [3.0, 5.0, 2.0]], [[8.0, 2.0, 
7.0], [1.0, 3.0, 1.0], [2.0, 2.0, 3.0]], [[8.0, 8.0, 4.0], [6.0, 2.0, 4.0], [2.0, 8.0, 4.0]]], [[[3.0, 6.0, 8.0], [7.0, 5.0, 
6.0], [6.0, 6.0, 7.0]], [[8.0, 7.0, 5.0], [7.0, 4.0, 4.0], [6.0, 1.0, 7.0]], [[5.0, 7.0, 7.0], [5.0, 1.0, 7.0], [7.0, 6.0, 
4.0]]], [[[7.0, 6.0, 8.0], [5.0, 7.0, 6.0], [6.0, 7.0, 6.0]], [[3.0, 2.0, 6.0], [3.0, 1.0, 1.0], [2.0, 2.0, 5.0]], [[3.0, 5.0, 
4.0], [6.0, 6.0, 2.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[1.0, 7.0, 8.0]]),self.functionspace)
      arg[2,1,1:2]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 4.0], [6.0, 8.0, 4.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 1.0], [7.0, 2.0, 4.0], [1.0, 5.0, 
2.0]], [[8.0, 2.0, 7.0], [2.0, 3.0, 6.0], [7.0, 6.0, 8.0]]], [[[1.0, 2.0, 2.0], [4.0, 5.0, 1.0], [3.0, 5.0, 2.0]], [[8.0, 2.0, 
7.0], [1.0, 3.0, 1.0], [2.0, 2.0, 3.0]], [[8.0, 8.0, 4.0], [6.0, 2.0, 4.0], [2.0, 8.0, 4.0]]], [[[3.0, 6.0, 8.0], [7.0, 5.0, 
6.0], [6.0, 6.0, 7.0]], [[8.0, 7.0, 5.0], [1.0, 7.0, 8.0], [6.0, 1.0, 7.0]], [[5.0, 7.0, 7.0], [5.0, 1.0, 7.0], [7.0, 6.0, 
4.0]]], [[[7.0, 6.0, 8.0], [5.0, 7.0, 6.0], [6.0, 7.0, 6.0]], [[3.0, 2.0, 6.0], [3.0, 1.0, 1.0], [2.0, 2.0, 5.0]], [[3.0, 5.0, 
4.0], [6.0, 6.0, 2.0], [7.0, 8.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPRN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 6.0], [8.0, 8.0, 3.0], [6.0, 7.0, 6.0]], [[6.0, 1.0, 8.0], [8.0, 3.0, 2.0], [8.0, 2.0, 
4.0]], [[4.0, 5.0, 2.0], [7.0, 6.0, 6.0], [2.0, 7.0, 4.0]]], [[[7.0, 7.0, 5.0], [1.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[4.0, 4.0, 
3.0], [3.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [2.0, 1.0, 7.0], [1.0, 3.0, 1.0]]], [[[8.0, 8.0, 1.0], [4.0, 4.0, 
5.0], [5.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [1.0, 6.0, 3.0], [2.0, 8.0, 4.0]], [[2.0, 5.0, 6.0], [2.0, 3.0, 7.0], [5.0, 4.0, 
8.0]]], [[[1.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 6.0], [8.0, 3.0, 6.0]], [[6.0, 7.0, 
3.0], [2.0, 8.0, 2.0], [8.0, 3.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 5.0, 5.0]])+(1.-msk_val)*numpy.array([[1.0, 6.0, 7.0]])
      arg[2,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 6.0], [8.0, 8.0, 3.0], [6.0, 7.0, 6.0]], [[6.0, 1.0, 8.0], [8.0, 3.0, 2.0], [8.0, 
2.0, 4.0]], [[4.0, 5.0, 2.0], [7.0, 6.0, 6.0], [2.0, 7.0, 4.0]]], [[[7.0, 7.0, 5.0], [1.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[4.0, 
4.0, 3.0], [3.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [2.0, 1.0, 7.0], [1.0, 3.0, 1.0]]], [[[8.0, 8.0, 1.0], [4.0, 
4.0, 5.0], [5.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [3.0, 5.0, 5.0], [2.0, 8.0, 4.0]], [[2.0, 5.0, 6.0], [2.0, 3.0, 7.0], [5.0, 4.0, 
8.0]]], [[[1.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 8.0], [4.0, 8.0, 6.0], [8.0, 3.0, 6.0]], [[6.0, 7.0, 
3.0], [2.0, 8.0, 2.0], [8.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 6.0], [8.0, 8.0, 3.0], [6.0, 7.0, 6.0]], 
[[6.0, 1.0, 8.0], [8.0, 3.0, 2.0], [8.0, 2.0, 4.0]], [[4.0, 5.0, 2.0], [7.0, 6.0, 6.0], [2.0, 7.0, 4.0]]], [[[7.0, 7.0, 5.0], 
[1.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[4.0, 4.0, 3.0], [3.0, 5.0, 4.0], [5.0, 4.0, 7.0]], [[3.0, 7.0, 5.0], [2.0, 1.0, 7.0], 
[1.0, 3.0, 1.0]]], [[[8.0, 8.0, 1.0], [4.0, 4.0, 5.0], [5.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [1.0, 6.0, 7.0], [2.0, 8.0, 4.0]], 
[[2.0, 5.0, 6.0], [2.0, 3.0, 7.0], [5.0, 4.0, 8.0]]], [[[1.0, 8.0, 8.0], [6.0, 4.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 8.0], 
[4.0, 8.0, 6.0], [8.0, 3.0, 6.0]], [[6.0, 7.0, 3.0], [2.0, 8.0, 2.0], [8.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCLN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 8.0], [1.0, 2.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 3.0, 1.0], [8.0, 3.0, 6.0], [1.0, 8.0, 
6.0]], [[8.0, 5.0, 6.0], [4.0, 6.0, 5.0], [1.0, 5.0, 2.0]]], [[[1.0, 8.0, 7.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]], [[3.0, 1.0, 
2.0], [8.0, 5.0, 4.0], [6.0, 1.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 3.0, 1.0], [4.0, 1.0, 1.0]]], [[[4.0, 6.0, 7.0], [5.0, 7.0, 
1.0], [1.0, 5.0, 6.0]], [[4.0, 7.0, 8.0], [6.0, 1.0, 4.0], [4.0, 6.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 5.0, 3.0], [4.0, 5.0, 
1.0]]], [[[5.0, 5.0, 3.0], [8.0, 1.0, 3.0], [5.0, 4.0, 3.0]], [[6.0, 2.0, 2.0], [1.0, 6.0, 8.0], [3.0, 2.0, 3.0]], [[3.0, 6.0, 
5.0], [5.0, 4.0, 6.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 6.0, 2.0]], [[5.0, 7.0, 7.0]], [[3.0, 4.0, 6.0]]])
      arg[2,:,2:]=val
      ref=Data(numpy.array([[[[3.0, 1.0, 8.0], [1.0, 2.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 3.0, 1.0], [8.0, 3.0, 6.0], [1.0, 8.0, 
6.0]], [[8.0, 5.0, 6.0], [4.0, 6.0, 5.0], [1.0, 5.0, 2.0]]], [[[1.0, 8.0, 7.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]], [[3.0, 1.0, 
2.0], [8.0, 5.0, 4.0], [6.0, 1.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 3.0, 1.0], [4.0, 1.0, 1.0]]], [[[4.0, 6.0, 7.0], [5.0, 7.0, 
1.0], [5.0, 6.0, 2.0]], [[4.0, 7.0, 8.0], [6.0, 1.0, 4.0], [5.0, 7.0, 7.0]], [[8.0, 5.0, 8.0], [5.0, 5.0, 3.0], [3.0, 4.0, 
6.0]]], [[[5.0, 5.0, 3.0], [8.0, 1.0, 3.0], [5.0, 4.0, 3.0]], [[6.0, 2.0, 2.0], [1.0, 6.0, 8.0], [3.0, 2.0, 3.0]], [[3.0, 6.0, 
5.0], [5.0, 4.0, 6.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCLN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 6.0], [7.0, 1.0, 5.0], [6.0, 5.0, 1.0]], [[1.0, 7.0, 6.0], [8.0, 1.0, 1.0], [8.0, 4.0, 
1.0]], [[6.0, 8.0, 1.0], [1.0, 8.0, 1.0], [7.0, 3.0, 3.0]]], [[[4.0, 2.0, 7.0], [3.0, 7.0, 2.0], [1.0, 2.0, 4.0]], [[7.0, 4.0, 
5.0], [7.0, 2.0, 4.0], [4.0, 6.0, 3.0]], [[8.0, 7.0, 4.0], [6.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[5.0, 7.0, 3.0], [1.0, 1.0, 
6.0], [6.0, 3.0, 1.0]], [[7.0, 7.0, 7.0], [8.0, 3.0, 3.0], [1.0, 2.0, 4.0]], [[2.0, 4.0, 4.0], [5.0, 3.0, 5.0], [5.0, 8.0, 
1.0]]], [[[6.0, 8.0, 1.0], [6.0, 2.0, 1.0], [8.0, 2.0, 5.0]], [[5.0, 3.0, 3.0], [1.0, 7.0, 8.0], [8.0, 5.0, 4.0]], [[5.0, 3.0, 
8.0], [5.0, 2.0, 8.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 2.0, 6.0]], [[3.0, 7.0, 3.0]], [[6.0, 4.0, 8.0]]]),self.functionspace)
      arg[2,:,2:]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 6.0], [7.0, 1.0, 5.0], [6.0, 5.0, 1.0]], [[1.0, 7.0, 6.0], [8.0, 1.0, 1.0], [8.0, 4.0, 
1.0]], [[6.0, 8.0, 1.0], [1.0, 8.0, 1.0], [7.0, 3.0, 3.0]]], [[[4.0, 2.0, 7.0], [3.0, 7.0, 2.0], [1.0, 2.0, 4.0]], [[7.0, 4.0, 
5.0], [7.0, 2.0, 4.0], [4.0, 6.0, 3.0]], [[8.0, 7.0, 4.0], [6.0, 7.0, 8.0], [6.0, 2.0, 3.0]]], [[[5.0, 7.0, 3.0], [1.0, 1.0, 
6.0], [4.0, 2.0, 6.0]], [[7.0, 7.0, 7.0], [8.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[2.0, 4.0, 4.0], [5.0, 3.0, 5.0], [6.0, 4.0, 
8.0]]], [[[6.0, 8.0, 1.0], [6.0, 2.0, 1.0], [8.0, 2.0, 5.0]], [[5.0, 3.0, 3.0], [1.0, 7.0, 8.0], [8.0, 5.0, 4.0]], [[5.0, 3.0, 
8.0], [5.0, 2.0, 8.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCLN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 4.0], [7.0, 3.0, 5.0], [8.0, 5.0, 1.0]], [[3.0, 7.0, 5.0], [8.0, 1.0, 3.0], [6.0, 7.0, 
2.0]], [[4.0, 6.0, 8.0], [7.0, 4.0, 7.0], [3.0, 8.0, 5.0]]], [[[7.0, 4.0, 5.0], [8.0, 3.0, 5.0], [6.0, 5.0, 8.0]], [[6.0, 4.0, 
4.0], [8.0, 5.0, 1.0], [6.0, 1.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 4.0, 6.0], [2.0, 6.0, 8.0]]], [[[3.0, 2.0, 1.0], [1.0, 4.0, 
2.0], [6.0, 7.0, 7.0]], [[3.0, 5.0, 5.0], [7.0, 5.0, 3.0], [3.0, 2.0, 2.0]], [[3.0, 7.0, 3.0], [4.0, 8.0, 2.0], [5.0, 3.0, 
7.0]]], [[[8.0, 2.0, 4.0], [7.0, 3.0, 5.0], [2.0, 1.0, 4.0]], [[6.0, 6.0, 2.0], [2.0, 1.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 1.0, 
8.0], [4.0, 2.0, 8.0], [1.0, 2.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 2.0, 7.0]], [[7.0, 8.0, 1.0]], [[4.0, 1.0, 2.0]]])+(1.-msk_val)*numpy.array([[[6.0, 6.0, 
4.0]], [[2.0, 8.0, 3.0]], [[2.0, 5.0, 8.0]]])
      arg[2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 4.0], [7.0, 3.0, 5.0], [8.0, 5.0, 1.0]], [[3.0, 7.0, 5.0], [8.0, 1.0, 3.0], [6.0, 
7.0, 2.0]], [[4.0, 6.0, 8.0], [7.0, 4.0, 7.0], [3.0, 8.0, 5.0]]], [[[7.0, 4.0, 5.0], [8.0, 3.0, 5.0], [6.0, 5.0, 8.0]], [[6.0, 
4.0, 4.0], [8.0, 5.0, 1.0], [6.0, 1.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 4.0, 6.0], [2.0, 6.0, 8.0]]], [[[3.0, 2.0, 1.0], [1.0, 
4.0, 2.0], [4.0, 2.0, 7.0]], [[3.0, 5.0, 5.0], [7.0, 5.0, 3.0], [7.0, 8.0, 1.0]], [[3.0, 7.0, 3.0], [4.0, 8.0, 2.0], [4.0, 1.0, 
2.0]]], [[[8.0, 2.0, 4.0], [7.0, 3.0, 5.0], [2.0, 1.0, 4.0]], [[6.0, 6.0, 2.0], [2.0, 1.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 1.0, 
8.0], [4.0, 2.0, 8.0], [1.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 4.0], [7.0, 3.0, 5.0], [8.0, 5.0, 1.0]], 
[[3.0, 7.0, 5.0], [8.0, 1.0, 3.0], [6.0, 7.0, 2.0]], [[4.0, 6.0, 8.0], [7.0, 4.0, 7.0], [3.0, 8.0, 5.0]]], [[[7.0, 4.0, 5.0], 
[8.0, 3.0, 5.0], [6.0, 5.0, 8.0]], [[6.0, 4.0, 4.0], [8.0, 5.0, 1.0], [6.0, 1.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 4.0, 6.0], 
[2.0, 6.0, 8.0]]], [[[3.0, 2.0, 1.0], [1.0, 4.0, 2.0], [6.0, 6.0, 4.0]], [[3.0, 5.0, 5.0], [7.0, 5.0, 3.0], [2.0, 8.0, 3.0]], 
[[3.0, 7.0, 3.0], [4.0, 8.0, 2.0], [2.0, 5.0, 8.0]]], [[[8.0, 2.0, 4.0], [7.0, 3.0, 5.0], [2.0, 1.0, 4.0]], [[6.0, 6.0, 2.0], 
[2.0, 1.0, 5.0], [4.0, 3.0, 6.0]], [[6.0, 1.0, 8.0], [4.0, 2.0, 8.0], [1.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCUN(self):
      arg=Data(numpy.array([[[[3.0, 8.0, 5.0], [8.0, 5.0, 2.0], [3.0, 8.0, 2.0]], [[8.0, 1.0, 3.0], [8.0, 5.0, 4.0], [4.0, 6.0, 
5.0]], [[1.0, 2.0, 2.0], [1.0, 1.0, 8.0], [7.0, 8.0, 1.0]]], [[[4.0, 8.0, 1.0], [8.0, 1.0, 2.0], [4.0, 4.0, 7.0]], [[7.0, 4.0, 
1.0], [3.0, 5.0, 8.0], [7.0, 8.0, 1.0]], [[4.0, 7.0, 7.0], [3.0, 8.0, 2.0], [8.0, 5.0, 8.0]]], [[[1.0, 1.0, 7.0], [2.0, 3.0, 
6.0], [3.0, 7.0, 8.0]], [[5.0, 3.0, 4.0], [1.0, 2.0, 4.0], [1.0, 5.0, 5.0]], [[7.0, 7.0, 3.0], [5.0, 1.0, 4.0], [5.0, 1.0, 
2.0]]], [[[2.0, 7.0, 2.0], [2.0, 8.0, 4.0], [7.0, 3.0, 8.0]], [[4.0, 2.0, 7.0], [3.0, 4.0, 6.0], [1.0, 2.0, 1.0]], [[8.0, 8.0, 
3.0], [5.0, 2.0, 8.0], [4.0, 3.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 4.0, 7.0], [2.0, 6.0, 1.0]], [[3.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[3.0, 1.0, 8.0], [6.0, 6.0, 
2.0]]], [[[2.0, 6.0, 4.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 6.0], [7.0, 6.0, 8.0]], [[1.0, 7.0, 7.0], [3.0, 5.0, 1.0]]], [[[3.0, 
2.0, 3.0], [1.0, 1.0, 7.0]], [[2.0, 3.0, 1.0], [3.0, 2.0, 1.0]], [[2.0, 1.0, 5.0], [7.0, 3.0, 8.0]]]])
      arg[:3,:,:2]=val
      ref=Data(numpy.array([[[[1.0, 4.0, 7.0], [2.0, 6.0, 1.0], [3.0, 8.0, 2.0]], [[3.0, 5.0, 7.0], [4.0, 5.0, 1.0], [4.0, 6.0, 
5.0]], [[3.0, 1.0, 8.0], [6.0, 6.0, 2.0], [7.0, 8.0, 1.0]]], [[[2.0, 6.0, 4.0], [8.0, 1.0, 6.0], [4.0, 4.0, 7.0]], [[8.0, 8.0, 
6.0], [7.0, 6.0, 8.0], [7.0, 8.0, 1.0]], [[1.0, 7.0, 7.0], [3.0, 5.0, 1.0], [8.0, 5.0, 8.0]]], [[[3.0, 2.0, 3.0], [1.0, 1.0, 
7.0], [3.0, 7.0, 8.0]], [[2.0, 3.0, 1.0], [3.0, 2.0, 1.0], [1.0, 5.0, 5.0]], [[2.0, 1.0, 5.0], [7.0, 3.0, 8.0], [5.0, 1.0, 
2.0]]], [[[2.0, 7.0, 2.0], [2.0, 8.0, 4.0], [7.0, 3.0, 8.0]], [[4.0, 2.0, 7.0], [3.0, 4.0, 6.0], [1.0, 2.0, 1.0]], [[8.0, 8.0, 
3.0], [5.0, 2.0, 8.0], [4.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCUN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 8.0], [6.0, 2.0, 5.0], [2.0, 3.0, 5.0]], [[4.0, 7.0, 8.0], [2.0, 7.0, 2.0], [6.0, 4.0, 
6.0]], [[2.0, 3.0, 6.0], [8.0, 6.0, 2.0], [7.0, 5.0, 3.0]]], [[[8.0, 1.0, 4.0], [3.0, 4.0, 4.0], [8.0, 2.0, 6.0]], [[8.0, 1.0, 
3.0], [7.0, 5.0, 4.0], [6.0, 2.0, 7.0]], [[7.0, 6.0, 2.0], [2.0, 3.0, 7.0], [1.0, 2.0, 4.0]]], [[[8.0, 1.0, 3.0], [7.0, 5.0, 
5.0], [7.0, 6.0, 7.0]], [[1.0, 7.0, 3.0], [6.0, 1.0, 4.0], [4.0, 4.0, 8.0]], [[6.0, 8.0, 5.0], [6.0, 1.0, 7.0], [5.0, 8.0, 
6.0]]], [[[7.0, 2.0, 7.0], [7.0, 8.0, 4.0], [4.0, 3.0, 6.0]], [[3.0, 4.0, 2.0], [7.0, 3.0, 5.0], [7.0, 2.0, 2.0]], [[8.0, 7.0, 
4.0], [8.0, 8.0, 5.0], [5.0, 4.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 3.0, 7.0], [7.0, 7.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[7.0, 2.0, 8.0], [4.0, 
1.0, 4.0]]], [[[2.0, 7.0, 4.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 2.0], [2.0, 8.0, 7.0]], [[6.0, 2.0, 1.0], [6.0, 2.0, 8.0]]], 
[[[8.0, 8.0, 5.0], [4.0, 8.0, 2.0]], [[1.0, 7.0, 6.0], [1.0, 7.0, 1.0]], [[6.0, 4.0, 1.0], [6.0, 6.0, 
5.0]]]]),self.functionspace)
      arg[:3,:,:2]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 7.0], [7.0, 7.0, 3.0], [2.0, 3.0, 5.0]], [[7.0, 7.0, 1.0], [5.0, 2.0, 6.0], [6.0, 4.0, 
6.0]], [[7.0, 2.0, 8.0], [4.0, 1.0, 4.0], [7.0, 5.0, 3.0]]], [[[2.0, 7.0, 4.0], [3.0, 2.0, 7.0], [8.0, 2.0, 6.0]], [[1.0, 3.0, 
2.0], [2.0, 8.0, 7.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 1.0], [6.0, 2.0, 8.0], [1.0, 2.0, 4.0]]], [[[8.0, 8.0, 5.0], [4.0, 8.0, 
2.0], [7.0, 6.0, 7.0]], [[1.0, 7.0, 6.0], [1.0, 7.0, 1.0], [4.0, 4.0, 8.0]], [[6.0, 4.0, 1.0], [6.0, 6.0, 5.0], [5.0, 8.0, 
6.0]]], [[[7.0, 2.0, 7.0], [7.0, 8.0, 4.0], [4.0, 3.0, 6.0]], [[3.0, 4.0, 2.0], [7.0, 3.0, 5.0], [7.0, 2.0, 2.0]], [[8.0, 7.0, 
4.0], [8.0, 8.0, 5.0], [5.0, 4.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCUN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 1.0], [2.0, 4.0, 3.0], [5.0, 3.0, 6.0]], [[2.0, 3.0, 3.0], [1.0, 6.0, 6.0], [3.0, 7.0, 
7.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 1.0], [8.0, 1.0, 5.0]]], [[[5.0, 3.0, 3.0], [5.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[3.0, 5.0, 
5.0], [7.0, 5.0, 7.0], [5.0, 2.0, 7.0]], [[2.0, 6.0, 6.0], [4.0, 6.0, 2.0], [3.0, 7.0, 5.0]]], [[[1.0, 3.0, 3.0], [1.0, 4.0, 
7.0], [4.0, 5.0, 8.0]], [[4.0, 1.0, 2.0], [6.0, 7.0, 4.0], [6.0, 5.0, 1.0]], [[3.0, 2.0, 7.0], [5.0, 5.0, 1.0], [6.0, 4.0, 
4.0]]], [[[4.0, 1.0, 5.0], [4.0, 1.0, 2.0], [4.0, 7.0, 4.0]], [[1.0, 2.0, 7.0], [3.0, 7.0, 3.0], [3.0, 1.0, 2.0]], [[7.0, 8.0, 
2.0], [8.0, 2.0, 4.0], [8.0, 2.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 4.0, 5.0]], [[5.0, 7.0, 7.0], [4.0, 5.0, 7.0]], [[7.0, 3.0, 2.0], [6.0, 
8.0, 8.0]]], [[[1.0, 8.0, 6.0], [2.0, 7.0, 8.0]], [[4.0, 8.0, 5.0], [1.0, 2.0, 6.0]], [[2.0, 7.0, 1.0], [1.0, 6.0, 2.0]]], 
[[[3.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[4.0, 7.0, 3.0], [4.0, 1.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 5.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 6.0, 1.0]], [[7.0, 8.0, 5.0], [8.0, 5.0, 6.0]], [[2.0, 1.0, 2.0], 
[1.0, 4.0, 6.0]]], [[[4.0, 2.0, 2.0], [7.0, 7.0, 4.0]], [[2.0, 3.0, 7.0], [8.0, 1.0, 6.0]], [[2.0, 6.0, 3.0], [6.0, 7.0, 
1.0]]], [[[5.0, 4.0, 2.0], [2.0, 4.0, 3.0]], [[1.0, 8.0, 6.0], [2.0, 7.0, 4.0]], [[6.0, 1.0, 5.0], [1.0, 8.0, 2.0]]]])
      arg[:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 4.0, 5.0], [5.0, 3.0, 6.0]], [[5.0, 7.0, 7.0], [4.0, 5.0, 7.0], [3.0, 
7.0, 7.0]], [[7.0, 3.0, 2.0], [6.0, 8.0, 8.0], [8.0, 1.0, 5.0]]], [[[1.0, 8.0, 6.0], [2.0, 7.0, 8.0], [5.0, 8.0, 4.0]], [[4.0, 
8.0, 5.0], [1.0, 2.0, 6.0], [5.0, 2.0, 7.0]], [[2.0, 7.0, 1.0], [1.0, 6.0, 2.0], [3.0, 7.0, 5.0]]], [[[3.0, 7.0, 3.0], [4.0, 
4.0, 8.0], [4.0, 5.0, 8.0]], [[4.0, 7.0, 3.0], [4.0, 1.0, 1.0], [6.0, 5.0, 1.0]], [[6.0, 8.0, 8.0], [5.0, 5.0, 5.0], [6.0, 4.0, 
4.0]]], [[[4.0, 1.0, 5.0], [4.0, 1.0, 2.0], [4.0, 7.0, 4.0]], [[1.0, 2.0, 7.0], [3.0, 7.0, 3.0], [3.0, 1.0, 2.0]], [[7.0, 8.0, 
2.0], [8.0, 2.0, 4.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 6.0, 1.0], [5.0, 3.0, 6.0]], 
[[7.0, 8.0, 5.0], [8.0, 5.0, 6.0], [3.0, 7.0, 7.0]], [[2.0, 1.0, 2.0], [1.0, 4.0, 6.0], [8.0, 1.0, 5.0]]], [[[4.0, 2.0, 2.0], 
[7.0, 7.0, 4.0], [5.0, 8.0, 4.0]], [[2.0, 3.0, 7.0], [8.0, 1.0, 6.0], [5.0, 2.0, 7.0]], [[2.0, 6.0, 3.0], [6.0, 7.0, 1.0], 
[3.0, 7.0, 5.0]]], [[[5.0, 4.0, 2.0], [2.0, 4.0, 3.0], [4.0, 5.0, 8.0]], [[1.0, 8.0, 6.0], [2.0, 7.0, 4.0], [6.0, 5.0, 1.0]], 
[[6.0, 1.0, 5.0], [1.0, 8.0, 2.0], [6.0, 4.0, 4.0]]], [[[4.0, 1.0, 5.0], [4.0, 1.0, 2.0], [4.0, 7.0, 4.0]], [[1.0, 2.0, 7.0], 
[3.0, 7.0, 3.0], [3.0, 1.0, 2.0]], [[7.0, 8.0, 2.0], [8.0, 2.0, 4.0], [8.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCLN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [7.0, 8.0, 1.0], [4.0, 2.0, 3.0]], [[5.0, 3.0, 6.0], [7.0, 8.0, 6.0], [3.0, 1.0, 
3.0]], [[1.0, 4.0, 6.0], [2.0, 8.0, 1.0], [3.0, 5.0, 8.0]]], [[[6.0, 8.0, 5.0], [4.0, 3.0, 6.0], [2.0, 1.0, 2.0]], [[8.0, 5.0, 
6.0], [3.0, 5.0, 5.0], [8.0, 2.0, 2.0]], [[4.0, 1.0, 4.0], [6.0, 3.0, 1.0], [5.0, 3.0, 1.0]]], [[[5.0, 2.0, 5.0], [7.0, 6.0, 
3.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 7.0], [2.0, 4.0, 7.0], [7.0, 5.0, 5.0]], [[5.0, 3.0, 2.0], [6.0, 2.0, 8.0], [1.0, 2.0, 
7.0]]], [[[8.0, 5.0, 7.0], [1.0, 8.0, 8.0], [4.0, 4.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 4.0, 5.0], [1.0, 4.0, 3.0]], [[5.0, 1.0, 
5.0], [7.0, 8.0, 1.0], [6.0, 4.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 3.0, 4.0]], [[1.0, 5.0, 1.0]], [[8.0, 7.0, 8.0]]], [[[7.0, 1.0, 5.0]], [[8.0, 5.0, 5.0]], [[8.0, 
2.0, 3.0]]], [[[1.0, 8.0, 8.0]], [[8.0, 6.0, 8.0]], [[3.0, 6.0, 2.0]]], [[[7.0, 6.0, 3.0]], [[2.0, 1.0, 6.0]], [[6.0, 2.0, 
8.0]]]])
      arg[:,:,2:]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 7.0], [7.0, 8.0, 1.0], [8.0, 3.0, 4.0]], [[5.0, 3.0, 6.0], [7.0, 8.0, 6.0], [1.0, 5.0, 
1.0]], [[1.0, 4.0, 6.0], [2.0, 8.0, 1.0], [8.0, 7.0, 8.0]]], [[[6.0, 8.0, 5.0], [4.0, 3.0, 6.0], [7.0, 1.0, 5.0]], [[8.0, 5.0, 
6.0], [3.0, 5.0, 5.0], [8.0, 5.0, 5.0]], [[4.0, 1.0, 4.0], [6.0, 3.0, 1.0], [8.0, 2.0, 3.0]]], [[[5.0, 2.0, 5.0], [7.0, 6.0, 
3.0], [1.0, 8.0, 8.0]], [[8.0, 5.0, 7.0], [2.0, 4.0, 7.0], [8.0, 6.0, 8.0]], [[5.0, 3.0, 2.0], [6.0, 2.0, 8.0], [3.0, 6.0, 
2.0]]], [[[8.0, 5.0, 7.0], [1.0, 8.0, 8.0], [7.0, 6.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 4.0, 5.0], [2.0, 1.0, 6.0]], [[5.0, 1.0, 
5.0], [7.0, 8.0, 1.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCLN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 7.0], [6.0, 7.0, 2.0], [1.0, 3.0, 5.0]], [[7.0, 5.0, 3.0], [3.0, 4.0, 6.0], [3.0, 1.0, 
8.0]], [[7.0, 8.0, 1.0], [6.0, 8.0, 8.0], [7.0, 2.0, 3.0]]], [[[1.0, 4.0, 5.0], [6.0, 1.0, 4.0], [6.0, 2.0, 6.0]], [[7.0, 1.0, 
7.0], [1.0, 1.0, 6.0], [6.0, 5.0, 2.0]], [[8.0, 8.0, 3.0], [3.0, 7.0, 7.0], [6.0, 4.0, 8.0]]], [[[1.0, 1.0, 1.0], [8.0, 5.0, 
8.0], [4.0, 4.0, 3.0]], [[7.0, 4.0, 1.0], [2.0, 1.0, 5.0], [8.0, 8.0, 5.0]], [[2.0, 6.0, 5.0], [3.0, 1.0, 7.0], [2.0, 7.0, 
4.0]]], [[[5.0, 7.0, 4.0], [4.0, 7.0, 1.0], [2.0, 1.0, 5.0]], [[1.0, 7.0, 3.0], [8.0, 7.0, 4.0], [1.0, 7.0, 2.0]], [[5.0, 6.0, 
4.0], [1.0, 6.0, 3.0], [2.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 7.0, 4.0]], [[3.0, 3.0, 4.0]], [[8.0, 7.0, 8.0]]], [[[6.0, 1.0, 8.0]], [[4.0, 6.0, 8.0]], 
[[4.0, 3.0, 8.0]]], [[[8.0, 7.0, 5.0]], [[5.0, 4.0, 7.0]], [[5.0, 8.0, 1.0]]], [[[3.0, 6.0, 8.0]], [[1.0, 8.0, 6.0]], [[8.0, 
5.0, 7.0]]]]),self.functionspace)
      arg[:,:,2:]=val
      ref=Data(numpy.array([[[[4.0, 7.0, 7.0], [6.0, 7.0, 2.0], [4.0, 7.0, 4.0]], [[7.0, 5.0, 3.0], [3.0, 4.0, 6.0], [3.0, 3.0, 
4.0]], [[7.0, 8.0, 1.0], [6.0, 8.0, 8.0], [8.0, 7.0, 8.0]]], [[[1.0, 4.0, 5.0], [6.0, 1.0, 4.0], [6.0, 1.0, 8.0]], [[7.0, 1.0, 
7.0], [1.0, 1.0, 6.0], [4.0, 6.0, 8.0]], [[8.0, 8.0, 3.0], [3.0, 7.0, 7.0], [4.0, 3.0, 8.0]]], [[[1.0, 1.0, 1.0], [8.0, 5.0, 
8.0], [8.0, 7.0, 5.0]], [[7.0, 4.0, 1.0], [2.0, 1.0, 5.0], [5.0, 4.0, 7.0]], [[2.0, 6.0, 5.0], [3.0, 1.0, 7.0], [5.0, 8.0, 
1.0]]], [[[5.0, 7.0, 4.0], [4.0, 7.0, 1.0], [3.0, 6.0, 8.0]], [[1.0, 7.0, 3.0], [8.0, 7.0, 4.0], [1.0, 8.0, 6.0]], [[5.0, 6.0, 
4.0], [1.0, 6.0, 3.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCLN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 3.0], [5.0, 5.0, 8.0], [4.0, 8.0, 6.0]], [[8.0, 2.0, 4.0], [5.0, 2.0, 8.0], [7.0, 8.0, 
2.0]], [[8.0, 5.0, 8.0], [1.0, 7.0, 1.0], [7.0, 8.0, 7.0]]], [[[7.0, 1.0, 4.0], [4.0, 8.0, 3.0], [2.0, 4.0, 6.0]], [[8.0, 3.0, 
4.0], [5.0, 6.0, 5.0], [4.0, 8.0, 4.0]], [[7.0, 4.0, 5.0], [7.0, 1.0, 7.0], [2.0, 3.0, 2.0]]], [[[7.0, 1.0, 2.0], [4.0, 8.0, 
3.0], [6.0, 2.0, 8.0]], [[1.0, 1.0, 6.0], [8.0, 3.0, 5.0], [8.0, 5.0, 8.0]], [[3.0, 8.0, 5.0], [6.0, 5.0, 8.0], [2.0, 6.0, 
7.0]]], [[[7.0, 1.0, 2.0], [2.0, 5.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 8.0], [4.0, 5.0, 1.0]], [[4.0, 8.0, 
4.0], [6.0, 3.0, 6.0], [3.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 3.0, 4.0]], [[4.0, 7.0, 7.0]], [[2.0, 4.0, 5.0]]], [[[3.0, 1.0, 5.0]], [[5.0, 7.0, 
5.0]], [[2.0, 2.0, 1.0]]], [[[5.0, 3.0, 7.0]], [[4.0, 2.0, 6.0]], [[5.0, 7.0, 6.0]]], [[[6.0, 1.0, 1.0]], [[5.0, 7.0, 7.0]], 
[[6.0, 5.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 1.0, 4.0]], [[7.0, 7.0, 3.0]], [[4.0, 1.0, 6.0]]], [[[1.0, 2.0, 2.0]], 
[[2.0, 3.0, 6.0]], [[3.0, 1.0, 7.0]]], [[[5.0, 6.0, 5.0]], [[1.0, 3.0, 2.0]], [[5.0, 5.0, 4.0]]], [[[1.0, 7.0, 1.0]], [[1.0, 
7.0, 3.0]], [[8.0, 2.0, 6.0]]]])
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 3.0], [5.0, 5.0, 8.0], [4.0, 3.0, 4.0]], [[8.0, 2.0, 4.0], [5.0, 2.0, 8.0], [4.0, 
7.0, 7.0]], [[8.0, 5.0, 8.0], [1.0, 7.0, 1.0], [2.0, 4.0, 5.0]]], [[[7.0, 1.0, 4.0], [4.0, 8.0, 3.0], [3.0, 1.0, 5.0]], [[8.0, 
3.0, 4.0], [5.0, 6.0, 5.0], [5.0, 7.0, 5.0]], [[7.0, 4.0, 5.0], [7.0, 1.0, 7.0], [2.0, 2.0, 1.0]]], [[[7.0, 1.0, 2.0], [4.0, 
8.0, 3.0], [5.0, 3.0, 7.0]], [[1.0, 1.0, 6.0], [8.0, 3.0, 5.0], [4.0, 2.0, 6.0]], [[3.0, 8.0, 5.0], [6.0, 5.0, 8.0], [5.0, 7.0, 
6.0]]], [[[7.0, 1.0, 2.0], [2.0, 5.0, 8.0], [6.0, 1.0, 1.0]], [[3.0, 4.0, 8.0], [4.0, 1.0, 8.0], [5.0, 7.0, 7.0]], [[4.0, 8.0, 
4.0], [6.0, 3.0, 6.0], [6.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 3.0], [5.0, 5.0, 8.0], [6.0, 1.0, 4.0]], 
[[8.0, 2.0, 4.0], [5.0, 2.0, 8.0], [7.0, 7.0, 3.0]], [[8.0, 5.0, 8.0], [1.0, 7.0, 1.0], [4.0, 1.0, 6.0]]], [[[7.0, 1.0, 4.0], 
[4.0, 8.0, 3.0], [1.0, 2.0, 2.0]], [[8.0, 3.0, 4.0], [5.0, 6.0, 5.0], [2.0, 3.0, 6.0]], [[7.0, 4.0, 5.0], [7.0, 1.0, 7.0], 
[3.0, 1.0, 7.0]]], [[[7.0, 1.0, 2.0], [4.0, 8.0, 3.0], [5.0, 6.0, 5.0]], [[1.0, 1.0, 6.0], [8.0, 3.0, 5.0], [1.0, 3.0, 2.0]], 
[[3.0, 8.0, 5.0], [6.0, 5.0, 8.0], [5.0, 5.0, 4.0]]], [[[7.0, 1.0, 2.0], [2.0, 5.0, 8.0], [1.0, 7.0, 1.0]], [[3.0, 4.0, 8.0], 
[4.0, 1.0, 8.0], [1.0, 7.0, 3.0]], [[4.0, 8.0, 4.0], [6.0, 3.0, 6.0], [8.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LUCN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 1.0], [3.0, 8.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 1.0, 2.0], [8.0, 1.0, 3.0], [8.0, 1.0, 
4.0]], [[2.0, 7.0, 1.0], [6.0, 2.0, 2.0], [1.0, 5.0, 1.0]]], [[[6.0, 1.0, 4.0], [6.0, 2.0, 5.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 
3.0], [8.0, 7.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 4.0, 4.0], [7.0, 1.0, 1.0], [5.0, 4.0, 8.0]]], [[[7.0, 6.0, 2.0], [3.0, 4.0, 
2.0], [5.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], [6.0, 5.0, 3.0], [1.0, 6.0, 1.0]], [[1.0, 5.0, 4.0], [2.0, 2.0, 8.0], [1.0, 6.0, 
7.0]]], [[[2.0, 7.0, 3.0], [7.0, 7.0, 4.0], [1.0, 5.0, 7.0]], [[4.0, 6.0, 7.0], [5.0, 6.0, 7.0], [5.0, 5.0, 2.0]], [[6.0, 1.0, 
1.0], [6.0, 6.0, 2.0], [2.0, 5.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 7.0, 6.0], [8.0, 7.0, 6.0], [3.0, 8.0, 4.0]], [[3.0, 7.0, 7.0], [7.0, 3.0, 6.0], [6.0, 6.0, 
2.0]]], [[[5.0, 3.0, 1.0], [2.0, 1.0, 2.0], [8.0, 6.0, 5.0]], [[1.0, 1.0, 3.0], [1.0, 7.0, 6.0], [2.0, 1.0, 5.0]]]])
      arg[2:,:2,:]=val
      ref=Data(numpy.array([[[[7.0, 7.0, 1.0], [3.0, 8.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 1.0, 2.0], [8.0, 1.0, 3.0], [8.0, 1.0, 
4.0]], [[2.0, 7.0, 1.0], [6.0, 2.0, 2.0], [1.0, 5.0, 1.0]]], [[[6.0, 1.0, 4.0], [6.0, 2.0, 5.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 
3.0], [8.0, 7.0, 1.0], [8.0, 6.0, 6.0]], [[5.0, 4.0, 4.0], [7.0, 1.0, 1.0], [5.0, 4.0, 8.0]]], [[[7.0, 7.0, 6.0], [8.0, 7.0, 
6.0], [3.0, 8.0, 4.0]], [[3.0, 7.0, 7.0], [7.0, 3.0, 6.0], [6.0, 6.0, 2.0]], [[1.0, 5.0, 4.0], [2.0, 2.0, 8.0], [1.0, 6.0, 
7.0]]], [[[5.0, 3.0, 1.0], [2.0, 1.0, 2.0], [8.0, 6.0, 5.0]], [[1.0, 1.0, 3.0], [1.0, 7.0, 6.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 
1.0], [6.0, 6.0, 2.0], [2.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LUCN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [5.0, 4.0, 6.0], [3.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 6.0], [3.0, 6.0, 
3.0]], [[6.0, 3.0, 6.0], [3.0, 2.0, 3.0], [7.0, 3.0, 6.0]]], [[[3.0, 7.0, 3.0], [8.0, 3.0, 5.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 
5.0], [4.0, 8.0, 6.0], [6.0, 1.0, 4.0]], [[4.0, 4.0, 8.0], [4.0, 3.0, 4.0], [7.0, 1.0, 3.0]]], [[[8.0, 8.0, 5.0], [4.0, 4.0, 
8.0], [6.0, 2.0, 3.0]], [[7.0, 4.0, 8.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[4.0, 5.0, 7.0], [6.0, 7.0, 7.0], [6.0, 8.0, 
8.0]]], [[[2.0, 5.0, 8.0], [2.0, 5.0, 8.0], [4.0, 8.0, 7.0]], [[8.0, 7.0, 1.0], [8.0, 8.0, 7.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 2.0, 4.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 1.0, 7.0], [3.0, 2.0, 8.0], [3.0, 4.0, 5.0]], [[8.0, 7.0, 7.0], [2.0, 8.0, 4.0], [6.0, 1.0, 
7.0]]], [[[3.0, 6.0, 2.0], [5.0, 2.0, 2.0], [3.0, 3.0, 1.0]], [[8.0, 7.0, 4.0], [5.0, 6.0, 7.0], [2.0, 6.0, 
3.0]]]]),self.functionspace)
      arg[2:,:2,:]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 7.0], [5.0, 4.0, 6.0], [3.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 6.0], [3.0, 6.0, 
3.0]], [[6.0, 3.0, 6.0], [3.0, 2.0, 3.0], [7.0, 3.0, 6.0]]], [[[3.0, 7.0, 3.0], [8.0, 3.0, 5.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 
5.0], [4.0, 8.0, 6.0], [6.0, 1.0, 4.0]], [[4.0, 4.0, 8.0], [4.0, 3.0, 4.0], [7.0, 1.0, 3.0]]], [[[8.0, 1.0, 7.0], [3.0, 2.0, 
8.0], [3.0, 4.0, 5.0]], [[8.0, 7.0, 7.0], [2.0, 8.0, 4.0], [6.0, 1.0, 7.0]], [[4.0, 5.0, 7.0], [6.0, 7.0, 7.0], [6.0, 8.0, 
8.0]]], [[[3.0, 6.0, 2.0], [5.0, 2.0, 2.0], [3.0, 3.0, 1.0]], [[8.0, 7.0, 4.0], [5.0, 6.0, 7.0], [2.0, 6.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 2.0, 4.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LUCN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 6.0], [1.0, 5.0, 4.0], [7.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [2.0, 4.0, 1.0], [7.0, 7.0, 
1.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 5.0], [8.0, 5.0, 5.0]]], [[[5.0, 4.0, 5.0], [8.0, 2.0, 7.0], [2.0, 7.0, 5.0]], [[2.0, 8.0, 
7.0], [2.0, 4.0, 5.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 1.0], [1.0, 6.0, 5.0], [6.0, 4.0, 3.0]]], [[[1.0, 6.0, 1.0], [3.0, 2.0, 
1.0], [2.0, 2.0, 8.0]], [[5.0, 8.0, 7.0], [1.0, 2.0, 2.0], [2.0, 6.0, 7.0]], [[6.0, 5.0, 7.0], [8.0, 2.0, 1.0], [1.0, 1.0, 
3.0]]], [[[4.0, 2.0, 6.0], [5.0, 1.0, 6.0], [3.0, 2.0, 6.0]], [[5.0, 8.0, 2.0], [5.0, 5.0, 6.0], [2.0, 6.0, 3.0]], [[5.0, 1.0, 
6.0], [1.0, 4.0, 4.0], [2.0, 5.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 7.0, 6.0], [7.0, 5.0, 6.0]], [[2.0, 7.0, 3.0], [8.0, 2.0, 3.0], [4.0, 
1.0, 2.0]]], [[[4.0, 3.0, 7.0], [8.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[1.0, 5.0, 3.0], [1.0, 1.0, 6.0], [6.0, 4.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 8.0, 8.0], [7.0, 1.0, 8.0], [4.0, 7.0, 6.0]], [[4.0, 8.0, 3.0], [6.0, 7.0, 1.0], 
[8.0, 6.0, 4.0]]], [[[3.0, 8.0, 8.0], [3.0, 2.0, 4.0], [4.0, 4.0, 3.0]], [[1.0, 3.0, 8.0], [4.0, 8.0, 1.0], [5.0, 7.0, 7.0]]]])
      arg[2:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 6.0], [1.0, 5.0, 4.0], [7.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [2.0, 4.0, 1.0], [7.0, 
7.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 5.0], [8.0, 5.0, 5.0]]], [[[5.0, 4.0, 5.0], [8.0, 2.0, 7.0], [2.0, 7.0, 5.0]], [[2.0, 
8.0, 7.0], [2.0, 4.0, 5.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 1.0], [1.0, 6.0, 5.0], [6.0, 4.0, 3.0]]], [[[3.0, 5.0, 2.0], [2.0, 
7.0, 6.0], [7.0, 5.0, 6.0]], [[2.0, 7.0, 3.0], [8.0, 2.0, 3.0], [4.0, 1.0, 2.0]], [[6.0, 5.0, 7.0], [8.0, 2.0, 1.0], [1.0, 1.0, 
3.0]]], [[[4.0, 3.0, 7.0], [8.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[1.0, 5.0, 3.0], [1.0, 1.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 1.0, 
6.0], [1.0, 4.0, 4.0], [2.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [1.0, 5.0, 4.0], [7.0, 7.0, 6.0]], 
[[2.0, 6.0, 8.0], [2.0, 4.0, 1.0], [7.0, 7.0, 1.0]], [[2.0, 7.0, 7.0], [1.0, 5.0, 5.0], [8.0, 5.0, 5.0]]], [[[5.0, 4.0, 5.0], 
[8.0, 2.0, 7.0], [2.0, 7.0, 5.0]], [[2.0, 8.0, 7.0], [2.0, 4.0, 5.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 1.0], [1.0, 6.0, 5.0], 
[6.0, 4.0, 3.0]]], [[[5.0, 8.0, 8.0], [7.0, 1.0, 8.0], [4.0, 7.0, 6.0]], [[4.0, 8.0, 3.0], [6.0, 7.0, 1.0], [8.0, 6.0, 4.0]], 
[[6.0, 5.0, 7.0], [8.0, 2.0, 1.0], [1.0, 1.0, 3.0]]], [[[3.0, 8.0, 8.0], [3.0, 2.0, 4.0], [4.0, 4.0, 3.0]], [[1.0, 3.0, 8.0], 
[4.0, 8.0, 1.0], [5.0, 7.0, 7.0]], [[5.0, 1.0, 6.0], [1.0, 4.0, 4.0], [2.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LURN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 4.0], [1.0, 5.0, 8.0], [8.0, 1.0, 3.0]], [[1.0, 2.0, 1.0], [6.0, 8.0, 4.0], [2.0, 5.0, 
5.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 7.0], [6.0, 6.0, 2.0]]], [[[7.0, 1.0, 7.0], [5.0, 4.0, 2.0], [5.0, 4.0, 5.0]], [[1.0, 2.0, 
3.0], [1.0, 3.0, 5.0], [5.0, 6.0, 2.0]], [[3.0, 8.0, 2.0], [4.0, 3.0, 2.0], [3.0, 2.0, 2.0]]], [[[4.0, 6.0, 6.0], [6.0, 2.0, 
6.0], [4.0, 7.0, 6.0]], [[1.0, 6.0, 8.0], [8.0, 1.0, 5.0], [7.0, 4.0, 6.0]], [[3.0, 5.0, 2.0], [2.0, 2.0, 7.0], [3.0, 2.0, 
5.0]]], [[[4.0, 2.0, 3.0], [4.0, 7.0, 4.0], [8.0, 2.0, 3.0]], [[3.0, 2.0, 5.0], [1.0, 7.0, 3.0], [5.0, 5.0, 3.0]], [[8.0, 4.0, 
6.0], [7.0, 5.0, 4.0], [3.0, 6.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 1.0, 1.0]], [[2.0, 1.0, 5.0]]], [[[2.0, 5.0, 4.0]], [[1.0, 4.0, 6.0]]]])
      arg[2:,:2,1:2]=val
      ref=Data(numpy.array([[[[6.0, 6.0, 4.0], [1.0, 5.0, 8.0], [8.0, 1.0, 3.0]], [[1.0, 2.0, 1.0], [6.0, 8.0, 4.0], [2.0, 5.0, 
5.0]], [[2.0, 2.0, 2.0], [8.0, 8.0, 7.0], [6.0, 6.0, 2.0]]], [[[7.0, 1.0, 7.0], [5.0, 4.0, 2.0], [5.0, 4.0, 5.0]], [[1.0, 2.0, 
3.0], [1.0, 3.0, 5.0], [5.0, 6.0, 2.0]], [[3.0, 8.0, 2.0], [4.0, 3.0, 2.0], [3.0, 2.0, 2.0]]], [[[4.0, 6.0, 6.0], [8.0, 1.0, 
1.0], [4.0, 7.0, 6.0]], [[1.0, 6.0, 8.0], [2.0, 1.0, 5.0], [7.0, 4.0, 6.0]], [[3.0, 5.0, 2.0], [2.0, 2.0, 7.0], [3.0, 2.0, 
5.0]]], [[[4.0, 2.0, 3.0], [2.0, 5.0, 4.0], [8.0, 2.0, 3.0]], [[3.0, 2.0, 5.0], [1.0, 4.0, 6.0], [5.0, 5.0, 3.0]], [[8.0, 4.0, 
6.0], [7.0, 5.0, 4.0], [3.0, 6.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LURN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 7.0], [7.0, 6.0, 4.0], [2.0, 1.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 8.0, 3.0], [2.0, 7.0, 
7.0]], [[8.0, 6.0, 4.0], [6.0, 2.0, 8.0], [1.0, 2.0, 5.0]]], [[[8.0, 8.0, 6.0], [4.0, 1.0, 8.0], [4.0, 1.0, 1.0]], [[8.0, 4.0, 
7.0], [1.0, 5.0, 8.0], [8.0, 4.0, 7.0]], [[4.0, 3.0, 5.0], [6.0, 8.0, 3.0], [1.0, 4.0, 2.0]]], [[[1.0, 4.0, 5.0], [5.0, 1.0, 
1.0], [5.0, 1.0, 2.0]], [[1.0, 5.0, 8.0], [3.0, 1.0, 3.0], [5.0, 4.0, 1.0]], [[4.0, 2.0, 7.0], [1.0, 7.0, 8.0], [6.0, 8.0, 
2.0]]], [[[4.0, 8.0, 4.0], [4.0, 8.0, 8.0], [3.0, 4.0, 1.0]], [[6.0, 1.0, 8.0], [1.0, 8.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 1.0, 
7.0], [8.0, 2.0, 7.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 7.0, 5.0]], [[1.0, 6.0, 8.0]]], [[[6.0, 8.0, 5.0]], [[4.0, 2.0, 6.0]]]]),self.functionspace)
      arg[2:,:2,1:2]=val
      ref=Data(numpy.array([[[[1.0, 3.0, 7.0], [7.0, 6.0, 4.0], [2.0, 1.0, 1.0]], [[3.0, 5.0, 3.0], [3.0, 8.0, 3.0], [2.0, 7.0, 
7.0]], [[8.0, 6.0, 4.0], [6.0, 2.0, 8.0], [1.0, 2.0, 5.0]]], [[[8.0, 8.0, 6.0], [4.0, 1.0, 8.0], [4.0, 1.0, 1.0]], [[8.0, 4.0, 
7.0], [1.0, 5.0, 8.0], [8.0, 4.0, 7.0]], [[4.0, 3.0, 5.0], [6.0, 8.0, 3.0], [1.0, 4.0, 2.0]]], [[[1.0, 4.0, 5.0], [7.0, 7.0, 
5.0], [5.0, 1.0, 2.0]], [[1.0, 5.0, 8.0], [1.0, 6.0, 8.0], [5.0, 4.0, 1.0]], [[4.0, 2.0, 7.0], [1.0, 7.0, 8.0], [6.0, 8.0, 
2.0]]], [[[4.0, 8.0, 4.0], [6.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[6.0, 1.0, 8.0], [4.0, 2.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 1.0, 
7.0], [8.0, 2.0, 7.0], [3.0, 6.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LURN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]], [[1.0, 4.0, 1.0], [4.0, 3.0, 3.0], [2.0, 1.0, 
2.0]], [[2.0, 3.0, 4.0], [4.0, 3.0, 4.0], [7.0, 6.0, 1.0]]], [[[2.0, 8.0, 3.0], [7.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 
2.0], [6.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 3.0, 5.0], [4.0, 5.0, 8.0], [8.0, 2.0, 2.0]]], [[[2.0, 8.0, 3.0], [8.0, 8.0, 
5.0], [4.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [5.0, 8.0, 6.0], [2.0, 6.0, 3.0]], [[4.0, 5.0, 1.0], [2.0, 8.0, 7.0], [8.0, 7.0, 
6.0]]], [[[3.0, 3.0, 5.0], [1.0, 7.0, 5.0], [5.0, 5.0, 8.0]], [[8.0, 3.0, 3.0], [8.0, 2.0, 1.0], [6.0, 2.0, 2.0]], [[3.0, 8.0, 
5.0], [5.0, 5.0, 4.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 7.0, 8.0]], [[3.0, 2.0, 4.0]]], [[[7.0, 6.0, 1.0]], [[4.0, 1.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 7.0, 8.0]], [[7.0, 7.0, 6.0]]], [[[1.0, 8.0, 6.0]], [[3.0, 4.0, 8.0]]]])
      arg[2:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]], [[1.0, 4.0, 1.0], [4.0, 3.0, 3.0], [2.0, 
1.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 3.0, 4.0], [7.0, 6.0, 1.0]]], [[[2.0, 8.0, 3.0], [7.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[5.0, 
1.0, 2.0], [6.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 3.0, 5.0], [4.0, 5.0, 8.0], [8.0, 2.0, 2.0]]], [[[2.0, 8.0, 3.0], [2.0, 
7.0, 8.0], [4.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [3.0, 2.0, 4.0], [2.0, 6.0, 3.0]], [[4.0, 5.0, 1.0], [2.0, 8.0, 7.0], [8.0, 7.0, 
6.0]]], [[[3.0, 3.0, 5.0], [7.0, 6.0, 1.0], [5.0, 5.0, 8.0]], [[8.0, 3.0, 3.0], [4.0, 1.0, 3.0], [6.0, 2.0, 2.0]], [[3.0, 8.0, 
5.0], [5.0, 5.0, 4.0], [7.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 2.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]], 
[[1.0, 4.0, 1.0], [4.0, 3.0, 3.0], [2.0, 1.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 3.0, 4.0], [7.0, 6.0, 1.0]]], [[[2.0, 8.0, 3.0], 
[7.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 3.0, 5.0], [4.0, 5.0, 8.0], 
[8.0, 2.0, 2.0]]], [[[2.0, 8.0, 3.0], [1.0, 7.0, 8.0], [4.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [7.0, 7.0, 6.0], [2.0, 6.0, 3.0]], 
[[4.0, 5.0, 1.0], [2.0, 8.0, 7.0], [8.0, 7.0, 6.0]]], [[[3.0, 3.0, 5.0], [1.0, 8.0, 6.0], [5.0, 5.0, 8.0]], [[8.0, 3.0, 3.0], 
[3.0, 4.0, 8.0], [6.0, 2.0, 2.0]], [[3.0, 8.0, 5.0], [5.0, 5.0, 4.0], [7.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_ULLN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 7.0], [4.0, 1.0, 4.0], [6.0, 3.0, 4.0]], [[6.0, 6.0, 5.0], [3.0, 6.0, 3.0], [3.0, 3.0, 
4.0]], [[7.0, 4.0, 1.0], [2.0, 6.0, 5.0], [6.0, 6.0, 8.0]]], [[[6.0, 7.0, 4.0], [2.0, 7.0, 8.0], [6.0, 2.0, 4.0]], [[5.0, 5.0, 
3.0], [5.0, 2.0, 3.0], [3.0, 1.0, 4.0]], [[4.0, 6.0, 2.0], [8.0, 2.0, 7.0], [2.0, 7.0, 2.0]]], [[[2.0, 1.0, 4.0], [4.0, 7.0, 
1.0], [3.0, 2.0, 6.0]], [[6.0, 2.0, 7.0], [8.0, 2.0, 7.0], [7.0, 8.0, 6.0]], [[4.0, 8.0, 8.0], [4.0, 2.0, 7.0], [7.0, 2.0, 
2.0]]], [[[3.0, 7.0, 4.0], [6.0, 1.0, 3.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 8.0], [3.0, 6.0, 8.0], [4.0, 6.0, 1.0]], [[3.0, 6.0, 
3.0], [1.0, 8.0, 4.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 7.0, 2.0]]], [[[2.0, 3.0, 7.0]]], [[[3.0, 1.0, 1.0]]]])
      arg[:3,2:,2:]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 7.0], [4.0, 1.0, 4.0], [6.0, 3.0, 4.0]], [[6.0, 6.0, 5.0], [3.0, 6.0, 3.0], [3.0, 3.0, 
4.0]], [[7.0, 4.0, 1.0], [2.0, 6.0, 5.0], [8.0, 7.0, 2.0]]], [[[6.0, 7.0, 4.0], [2.0, 7.0, 8.0], [6.0, 2.0, 4.0]], [[5.0, 5.0, 
3.0], [5.0, 2.0, 3.0], [3.0, 1.0, 4.0]], [[4.0, 6.0, 2.0], [8.0, 2.0, 7.0], [2.0, 3.0, 7.0]]], [[[2.0, 1.0, 4.0], [4.0, 7.0, 
1.0], [3.0, 2.0, 6.0]], [[6.0, 2.0, 7.0], [8.0, 2.0, 7.0], [7.0, 8.0, 6.0]], [[4.0, 8.0, 8.0], [4.0, 2.0, 7.0], [3.0, 1.0, 
1.0]]], [[[3.0, 7.0, 4.0], [6.0, 1.0, 3.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 8.0], [3.0, 6.0, 8.0], [4.0, 6.0, 1.0]], [[3.0, 6.0, 
3.0], [1.0, 8.0, 4.0], [2.0, 3.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_ULLN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 8.0], [7.0, 1.0, 3.0], [3.0, 1.0, 4.0]], [[1.0, 5.0, 5.0], [6.0, 8.0, 4.0], [1.0, 3.0, 
4.0]], [[4.0, 7.0, 8.0], [8.0, 1.0, 3.0], [8.0, 1.0, 6.0]]], [[[2.0, 3.0, 6.0], [8.0, 7.0, 8.0], [5.0, 5.0, 4.0]], [[2.0, 7.0, 
8.0], [3.0, 6.0, 3.0], [5.0, 8.0, 3.0]], [[6.0, 8.0, 3.0], [6.0, 8.0, 6.0], [2.0, 1.0, 1.0]]], [[[2.0, 3.0, 4.0], [6.0, 8.0, 
3.0], [2.0, 6.0, 7.0]], [[4.0, 7.0, 4.0], [4.0, 7.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 7.0, 8.0], [5.0, 2.0, 
2.0]]], [[[3.0, 8.0, 3.0], [4.0, 5.0, 1.0], [8.0, 6.0, 7.0]], [[5.0, 7.0, 7.0], [7.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 
8.0], [5.0, 2.0, 2.0], [3.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0, 2.0]]], [[[4.0, 4.0, 6.0]]], [[[5.0, 6.0, 3.0]]]]),self.functionspace)
      arg[:3,2:,2:]=val
      ref=Data(numpy.array([[[[4.0, 6.0, 8.0], [7.0, 1.0, 3.0], [3.0, 1.0, 4.0]], [[1.0, 5.0, 5.0], [6.0, 8.0, 4.0], [1.0, 3.0, 
4.0]], [[4.0, 7.0, 8.0], [8.0, 1.0, 3.0], [5.0, 1.0, 2.0]]], [[[2.0, 3.0, 6.0], [8.0, 7.0, 8.0], [5.0, 5.0, 4.0]], [[2.0, 7.0, 
8.0], [3.0, 6.0, 3.0], [5.0, 8.0, 3.0]], [[6.0, 8.0, 3.0], [6.0, 8.0, 6.0], [4.0, 4.0, 6.0]]], [[[2.0, 3.0, 4.0], [6.0, 8.0, 
3.0], [2.0, 6.0, 7.0]], [[4.0, 7.0, 4.0], [4.0, 7.0, 4.0], [7.0, 8.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 7.0, 8.0], [5.0, 6.0, 
3.0]]], [[[3.0, 8.0, 3.0], [4.0, 5.0, 1.0], [8.0, 6.0, 7.0]], [[5.0, 7.0, 7.0], [7.0, 7.0, 8.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 
8.0], [5.0, 2.0, 2.0], [3.0, 3.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_ULLN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 6.0], [5.0, 2.0, 5.0], [7.0, 3.0, 6.0]], [[4.0, 4.0, 1.0], [7.0, 2.0, 3.0], [5.0, 1.0, 
8.0]], [[8.0, 8.0, 8.0], [8.0, 6.0, 5.0], [7.0, 7.0, 8.0]]], [[[8.0, 3.0, 6.0], [8.0, 8.0, 5.0], [2.0, 8.0, 6.0]], [[3.0, 8.0, 
1.0], [7.0, 5.0, 6.0], [5.0, 2.0, 5.0]], [[4.0, 7.0, 8.0], [7.0, 1.0, 7.0], [7.0, 7.0, 1.0]]], [[[7.0, 5.0, 6.0], [5.0, 5.0, 
8.0], [3.0, 3.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 6.0, 3.0], [2.0, 8.0, 3.0], [4.0, 3.0, 
4.0]]], [[[5.0, 2.0, 5.0], [2.0, 2.0, 1.0], [3.0, 6.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 4.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 6.0, 
2.0], [5.0, 5.0, 5.0], [7.0, 6.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 7.0, 5.0]]], [[[1.0, 3.0, 7.0]]], [[[4.0, 4.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 1.0, 2.0]]], [[[1.0, 7.0, 1.0]]], [[[3.0, 4.0, 8.0]]]])
      arg[:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 6.0], [5.0, 2.0, 5.0], [7.0, 3.0, 6.0]], [[4.0, 4.0, 1.0], [7.0, 2.0, 3.0], [5.0, 
1.0, 8.0]], [[8.0, 8.0, 8.0], [8.0, 6.0, 5.0], [1.0, 7.0, 5.0]]], [[[8.0, 3.0, 6.0], [8.0, 8.0, 5.0], [2.0, 8.0, 6.0]], [[3.0, 
8.0, 1.0], [7.0, 5.0, 6.0], [5.0, 2.0, 5.0]], [[4.0, 7.0, 8.0], [7.0, 1.0, 7.0], [1.0, 3.0, 7.0]]], [[[7.0, 5.0, 6.0], [5.0, 
5.0, 8.0], [3.0, 3.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 6.0, 3.0], [2.0, 8.0, 3.0], [4.0, 4.0, 
5.0]]], [[[5.0, 2.0, 5.0], [2.0, 2.0, 1.0], [3.0, 6.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 4.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 6.0, 
2.0], [5.0, 5.0, 5.0], [7.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 6.0], [5.0, 2.0, 5.0], [7.0, 3.0, 6.0]], 
[[4.0, 4.0, 1.0], [7.0, 2.0, 3.0], [5.0, 1.0, 8.0]], [[8.0, 8.0, 8.0], [8.0, 6.0, 5.0], [4.0, 1.0, 2.0]]], [[[8.0, 3.0, 6.0], 
[8.0, 8.0, 5.0], [2.0, 8.0, 6.0]], [[3.0, 8.0, 1.0], [7.0, 5.0, 6.0], [5.0, 2.0, 5.0]], [[4.0, 7.0, 8.0], [7.0, 1.0, 7.0], 
[1.0, 7.0, 1.0]]], [[[7.0, 5.0, 6.0], [5.0, 5.0, 8.0], [3.0, 3.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 2.0, 1.0], [5.0, 3.0, 2.0]], 
[[6.0, 6.0, 3.0], [2.0, 8.0, 3.0], [3.0, 4.0, 8.0]]], [[[5.0, 2.0, 5.0], [2.0, 2.0, 1.0], [3.0, 6.0, 7.0]], [[2.0, 3.0, 3.0], 
[2.0, 4.0, 5.0], [5.0, 4.0, 7.0]], [[3.0, 6.0, 2.0], [5.0, 5.0, 5.0], [7.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CUCN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 1.0], [5.0, 5.0, 1.0], [7.0, 5.0, 7.0]], [[4.0, 1.0, 7.0], [6.0, 6.0, 6.0], [4.0, 4.0, 
6.0]], [[8.0, 5.0, 7.0], [4.0, 4.0, 2.0], [5.0, 8.0, 8.0]]], [[[6.0, 3.0, 8.0], [8.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[7.0, 2.0, 
5.0], [5.0, 3.0, 6.0], [6.0, 3.0, 2.0]], [[8.0, 8.0, 3.0], [6.0, 4.0, 7.0], [3.0, 6.0, 3.0]]], [[[4.0, 6.0, 8.0], [7.0, 5.0, 
6.0], [5.0, 6.0, 3.0]], [[2.0, 3.0, 1.0], [6.0, 4.0, 3.0], [4.0, 4.0, 8.0]], [[1.0, 4.0, 1.0], [2.0, 8.0, 7.0], [1.0, 4.0, 
4.0]]], [[[4.0, 6.0, 8.0], [2.0, 2.0, 5.0], [5.0, 1.0, 7.0]], [[5.0, 3.0, 8.0], [7.0, 6.0, 5.0], [6.0, 3.0, 5.0]], [[1.0, 7.0, 
6.0], [6.0, 1.0, 2.0], [4.0, 2.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 2.0, 5.0], [7.0, 6.0, 6.0], [3.0, 3.0, 3.0]], [[5.0, 4.0, 2.0], [4.0, 7.0, 5.0], [8.0, 3.0, 
5.0]]], [[[7.0, 6.0, 5.0], [2.0, 4.0, 2.0], [6.0, 1.0, 2.0]], [[7.0, 5.0, 1.0], [7.0, 5.0, 5.0], [8.0, 6.0, 1.0]]], [[[2.0, 
7.0, 3.0], [6.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[1.0, 1.0, 2.0], [5.0, 7.0, 5.0], [5.0, 6.0, 4.0]]], [[[6.0, 6.0, 8.0], [2.0, 
7.0, 7.0], [7.0, 5.0, 8.0]], [[8.0, 5.0, 2.0], [6.0, 2.0, 4.0], [3.0, 7.0, 1.0]]]])
      arg[:,:2,:]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 5.0], [7.0, 6.0, 6.0], [3.0, 3.0, 3.0]], [[5.0, 4.0, 2.0], [4.0, 7.0, 5.0], [8.0, 3.0, 
5.0]], [[8.0, 5.0, 7.0], [4.0, 4.0, 2.0], [5.0, 8.0, 8.0]]], [[[7.0, 6.0, 5.0], [2.0, 4.0, 2.0], [6.0, 1.0, 2.0]], [[7.0, 5.0, 
1.0], [7.0, 5.0, 5.0], [8.0, 6.0, 1.0]], [[8.0, 8.0, 3.0], [6.0, 4.0, 7.0], [3.0, 6.0, 3.0]]], [[[2.0, 7.0, 3.0], [6.0, 2.0, 
5.0], [8.0, 8.0, 1.0]], [[1.0, 1.0, 2.0], [5.0, 7.0, 5.0], [5.0, 6.0, 4.0]], [[1.0, 4.0, 1.0], [2.0, 8.0, 7.0], [1.0, 4.0, 
4.0]]], [[[6.0, 6.0, 8.0], [2.0, 7.0, 7.0], [7.0, 5.0, 8.0]], [[8.0, 5.0, 2.0], [6.0, 2.0, 4.0], [3.0, 7.0, 1.0]], [[1.0, 7.0, 
6.0], [6.0, 1.0, 2.0], [4.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CUCN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 4.0], [8.0, 1.0, 3.0], [1.0, 1.0, 2.0]], [[1.0, 4.0, 7.0], [1.0, 1.0, 8.0], [8.0, 3.0, 
4.0]], [[1.0, 5.0, 5.0], [8.0, 1.0, 6.0], [5.0, 8.0, 3.0]]], [[[4.0, 6.0, 5.0], [6.0, 8.0, 3.0], [7.0, 6.0, 5.0]], [[2.0, 5.0, 
8.0], [1.0, 3.0, 5.0], [6.0, 3.0, 1.0]], [[4.0, 5.0, 3.0], [3.0, 7.0, 2.0], [8.0, 5.0, 5.0]]], [[[3.0, 5.0, 7.0], [4.0, 6.0, 
7.0], [6.0, 4.0, 2.0]], [[2.0, 7.0, 4.0], [3.0, 8.0, 7.0], [3.0, 3.0, 5.0]], [[4.0, 4.0, 8.0], [8.0, 5.0, 7.0], [6.0, 6.0, 
4.0]]], [[[1.0, 2.0, 5.0], [3.0, 5.0, 6.0], [6.0, 3.0, 4.0]], [[3.0, 4.0, 8.0], [5.0, 3.0, 3.0], [2.0, 4.0, 8.0]], [[1.0, 2.0, 
6.0], [8.0, 7.0, 2.0], [2.0, 6.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 5.0, 5.0], [3.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[8.0, 5.0, 4.0], [4.0, 6.0, 5.0], [1.0, 3.0, 
7.0]]], [[[1.0, 6.0, 1.0], [8.0, 3.0, 7.0], [6.0, 6.0, 7.0]], [[1.0, 2.0, 7.0], [3.0, 6.0, 4.0], [1.0, 3.0, 6.0]]], [[[3.0, 
5.0, 8.0], [1.0, 1.0, 5.0], [8.0, 7.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 1.0, 6.0], [8.0, 3.0, 6.0]]], [[[8.0, 8.0, 7.0], [2.0, 
4.0, 5.0], [6.0, 2.0, 7.0]], [[6.0, 7.0, 1.0], [8.0, 7.0, 1.0], [5.0, 5.0, 8.0]]]]),self.functionspace)
      arg[:,:2,:]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 5.0], [3.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[8.0, 5.0, 4.0], [4.0, 6.0, 5.0], [1.0, 3.0, 
7.0]], [[1.0, 5.0, 5.0], [8.0, 1.0, 6.0], [5.0, 8.0, 3.0]]], [[[1.0, 6.0, 1.0], [8.0, 3.0, 7.0], [6.0, 6.0, 7.0]], [[1.0, 2.0, 
7.0], [3.0, 6.0, 4.0], [1.0, 3.0, 6.0]], [[4.0, 5.0, 3.0], [3.0, 7.0, 2.0], [8.0, 5.0, 5.0]]], [[[3.0, 5.0, 8.0], [1.0, 1.0, 
5.0], [8.0, 7.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 1.0, 6.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 8.0], [8.0, 5.0, 7.0], [6.0, 6.0, 
4.0]]], [[[8.0, 8.0, 7.0], [2.0, 4.0, 5.0], [6.0, 2.0, 7.0]], [[6.0, 7.0, 1.0], [8.0, 7.0, 1.0], [5.0, 5.0, 8.0]], [[1.0, 2.0, 
6.0], [8.0, 7.0, 2.0], [2.0, 6.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CUCN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 8.0], [3.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[6.0, 2.0, 7.0], [2.0, 2.0, 4.0], [7.0, 4.0, 
6.0]], [[8.0, 4.0, 7.0], [4.0, 3.0, 8.0], [3.0, 1.0, 4.0]]], [[[7.0, 4.0, 8.0], [6.0, 3.0, 8.0], [7.0, 5.0, 1.0]], [[6.0, 4.0, 
8.0], [8.0, 7.0, 1.0], [3.0, 2.0, 4.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 6.0], [8.0, 8.0, 5.0]]], [[[4.0, 2.0, 3.0], [4.0, 6.0, 
4.0], [8.0, 6.0, 6.0]], [[4.0, 8.0, 6.0], [7.0, 1.0, 5.0], [6.0, 2.0, 8.0]], [[2.0, 2.0, 7.0], [3.0, 5.0, 4.0], [1.0, 5.0, 
4.0]]], [[[6.0, 2.0, 4.0], [8.0, 1.0, 4.0], [1.0, 1.0, 6.0]], [[6.0, 7.0, 5.0], [6.0, 5.0, 2.0], [4.0, 4.0, 4.0]], [[5.0, 8.0, 
2.0], [8.0, 1.0, 1.0], [4.0, 4.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 2.0, 7.0], [7.0, 1.0, 7.0], [1.0, 1.0, 1.0]], [[3.0, 3.0, 7.0], [8.0, 5.0, 2.0], [7.0, 
8.0, 1.0]]], [[[6.0, 6.0, 6.0], [6.0, 2.0, 1.0], [1.0, 7.0, 2.0]], [[6.0, 2.0, 3.0], [4.0, 7.0, 3.0], [6.0, 7.0, 8.0]]], 
[[[6.0, 7.0, 1.0], [1.0, 1.0, 6.0], [6.0, 3.0, 4.0]], [[3.0, 6.0, 5.0], [3.0, 2.0, 1.0], [3.0, 4.0, 6.0]]], [[[4.0, 1.0, 2.0], 
[2.0, 4.0, 8.0], [4.0, 3.0, 2.0]], [[5.0, 3.0, 2.0], [4.0, 7.0, 6.0], [4.0, 7.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 
5.0, 5.0], [8.0, 7.0, 1.0], [1.0, 3.0, 7.0]], [[1.0, 3.0, 6.0], [1.0, 8.0, 2.0], [1.0, 4.0, 3.0]]], [[[6.0, 1.0, 3.0], [2.0, 
3.0, 5.0], [1.0, 8.0, 6.0]], [[8.0, 8.0, 8.0], [1.0, 7.0, 3.0], [8.0, 3.0, 8.0]]], [[[2.0, 5.0, 2.0], [4.0, 8.0, 5.0], [3.0, 
5.0, 8.0]], [[1.0, 6.0, 8.0], [1.0, 2.0, 1.0], [7.0, 8.0, 8.0]]], [[[8.0, 1.0, 2.0], [2.0, 2.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 
4.0, 4.0], [6.0, 1.0, 5.0], [4.0, 3.0, 1.0]]]])
      arg[:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [7.0, 1.0, 7.0], [1.0, 1.0, 1.0]], [[3.0, 3.0, 7.0], [8.0, 5.0, 2.0], [7.0, 
8.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 3.0, 8.0], [3.0, 1.0, 4.0]]], [[[6.0, 6.0, 6.0], [6.0, 2.0, 1.0], [1.0, 7.0, 2.0]], [[6.0, 
2.0, 3.0], [4.0, 7.0, 3.0], [6.0, 7.0, 8.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 6.0], [8.0, 8.0, 5.0]]], [[[6.0, 7.0, 1.0], [1.0, 
1.0, 6.0], [6.0, 3.0, 4.0]], [[3.0, 6.0, 5.0], [3.0, 2.0, 1.0], [3.0, 4.0, 6.0]], [[2.0, 2.0, 7.0], [3.0, 5.0, 4.0], [1.0, 5.0, 
4.0]]], [[[4.0, 1.0, 2.0], [2.0, 4.0, 8.0], [4.0, 3.0, 2.0]], [[5.0, 3.0, 2.0], [4.0, 7.0, 6.0], [4.0, 7.0, 3.0]], [[5.0, 8.0, 
2.0], [8.0, 1.0, 1.0], [4.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [8.0, 7.0, 1.0], [1.0, 3.0, 7.0]], 
[[1.0, 3.0, 6.0], [1.0, 8.0, 2.0], [1.0, 4.0, 3.0]], [[8.0, 4.0, 7.0], [4.0, 3.0, 8.0], [3.0, 1.0, 4.0]]], [[[6.0, 1.0, 3.0], 
[2.0, 3.0, 5.0], [1.0, 8.0, 6.0]], [[8.0, 8.0, 8.0], [1.0, 7.0, 3.0], [8.0, 3.0, 8.0]], [[1.0, 6.0, 3.0], [6.0, 3.0, 6.0], 
[8.0, 8.0, 5.0]]], [[[2.0, 5.0, 2.0], [4.0, 8.0, 5.0], [3.0, 5.0, 8.0]], [[1.0, 6.0, 8.0], [1.0, 2.0, 1.0], [7.0, 8.0, 8.0]], 
[[2.0, 2.0, 7.0], [3.0, 5.0, 4.0], [1.0, 5.0, 4.0]]], [[[8.0, 1.0, 2.0], [2.0, 2.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 4.0, 4.0], 
[6.0, 1.0, 5.0], [4.0, 3.0, 1.0]], [[5.0, 8.0, 2.0], [8.0, 1.0, 1.0], [4.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCRR(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 6.0], [5.0, 2.0, 6.0], [2.0, 1.0, 3.0]], [[1.0, 4.0, 3.0], [2.0, 3.0, 8.0], [5.0, 4.0, 
3.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 6.0], [8.0, 5.0, 5.0]]], [[[2.0, 3.0, 6.0], [7.0, 2.0, 1.0], [3.0, 6.0, 2.0]], [[3.0, 3.0, 
5.0], [8.0, 5.0, 8.0], [6.0, 8.0, 5.0]], [[4.0, 8.0, 5.0], [2.0, 5.0, 3.0], [4.0, 4.0, 8.0]]], [[[7.0, 8.0, 2.0], [2.0, 5.0, 
1.0], [8.0, 5.0, 1.0]], [[8.0, 6.0, 2.0], [2.0, 4.0, 5.0], [4.0, 7.0, 7.0]], [[5.0, 3.0, 5.0], [5.0, 3.0, 4.0], [6.0, 1.0, 
7.0]]], [[[4.0, 2.0, 3.0], [7.0, 8.0, 7.0], [3.0, 8.0, 7.0]], [[2.0, 8.0, 2.0], [7.0, 1.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 5.0, 
2.0], [1.0, 5.0, 5.0], [1.0, 1.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0]], [[2.0]], [[4.0]]], [[[2.0]], [[3.0]], [[2.0]]]])
      arg[1:3,:,1:2,1:2]=val
      ref=Data(numpy.array([[[[8.0, 6.0, 6.0], [5.0, 2.0, 6.0], [2.0, 1.0, 3.0]], [[1.0, 4.0, 3.0], [2.0, 3.0, 8.0], [5.0, 4.0, 
3.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 6.0], [8.0, 5.0, 5.0]]], [[[2.0, 3.0, 6.0], [7.0, 7.0, 1.0], [3.0, 6.0, 2.0]], [[3.0, 3.0, 
5.0], [8.0, 2.0, 8.0], [6.0, 8.0, 5.0]], [[4.0, 8.0, 5.0], [2.0, 4.0, 3.0], [4.0, 4.0, 8.0]]], [[[7.0, 8.0, 2.0], [2.0, 2.0, 
1.0], [8.0, 5.0, 1.0]], [[8.0, 6.0, 2.0], [2.0, 3.0, 5.0], [4.0, 7.0, 7.0]], [[5.0, 3.0, 5.0], [5.0, 2.0, 4.0], [6.0, 1.0, 
7.0]]], [[[4.0, 2.0, 3.0], [7.0, 8.0, 7.0], [3.0, 8.0, 7.0]], [[2.0, 8.0, 2.0], [7.0, 1.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 5.0, 
2.0], [1.0, 5.0, 5.0], [1.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCRR(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [8.0, 3.0, 1.0], [7.0, 3.0, 5.0]], [[5.0, 3.0, 2.0], [4.0, 8.0, 7.0], [8.0, 6.0, 
6.0]], [[3.0, 5.0, 5.0], [2.0, 2.0, 2.0], [8.0, 8.0, 8.0]]], [[[3.0, 4.0, 3.0], [7.0, 5.0, 2.0], [2.0, 3.0, 7.0]], [[6.0, 7.0, 
3.0], [6.0, 6.0, 7.0], [6.0, 8.0, 4.0]], [[3.0, 8.0, 8.0], [4.0, 2.0, 5.0], [6.0, 5.0, 7.0]]], [[[8.0, 8.0, 6.0], [3.0, 1.0, 
3.0], [5.0, 3.0, 2.0]], [[7.0, 7.0, 2.0], [7.0, 1.0, 8.0], [4.0, 6.0, 7.0]], [[1.0, 8.0, 8.0], [7.0, 5.0, 5.0], [3.0, 6.0, 
1.0]]], [[[8.0, 5.0, 3.0], [6.0, 3.0, 8.0], [7.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], [7.0, 1.0, 2.0], [8.0, 7.0, 6.0]], [[3.0, 1.0, 
5.0], [2.0, 8.0, 3.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0]], [[5.0]], [[7.0]]], [[[5.0]], [[2.0]], [[3.0]]]]),self.functionspace)
      arg[1:3,:,1:2,1:2]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 5.0], [8.0, 3.0, 1.0], [7.0, 3.0, 5.0]], [[5.0, 3.0, 2.0], [4.0, 8.0, 7.0], [8.0, 6.0, 
6.0]], [[3.0, 5.0, 5.0], [2.0, 2.0, 2.0], [8.0, 8.0, 8.0]]], [[[3.0, 4.0, 3.0], [7.0, 8.0, 2.0], [2.0, 3.0, 7.0]], [[6.0, 7.0, 
3.0], [6.0, 5.0, 7.0], [6.0, 8.0, 4.0]], [[3.0, 8.0, 8.0], [4.0, 7.0, 5.0], [6.0, 5.0, 7.0]]], [[[8.0, 8.0, 6.0], [3.0, 5.0, 
3.0], [5.0, 3.0, 2.0]], [[7.0, 7.0, 2.0], [7.0, 2.0, 8.0], [4.0, 6.0, 7.0]], [[1.0, 8.0, 8.0], [7.0, 3.0, 5.0], [3.0, 6.0, 
1.0]]], [[[8.0, 5.0, 3.0], [6.0, 3.0, 8.0], [7.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], [7.0, 1.0, 2.0], [8.0, 7.0, 6.0]], [[3.0, 1.0, 
5.0], [2.0, 8.0, 3.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCRR(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 1.0], [5.0, 6.0, 7.0], [3.0, 2.0, 1.0]], [[7.0, 2.0, 4.0], [4.0, 4.0, 1.0], [1.0, 7.0, 
1.0]], [[6.0, 1.0, 4.0], [6.0, 7.0, 3.0], [1.0, 8.0, 8.0]]], [[[3.0, 8.0, 8.0], [2.0, 3.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 2.0, 
1.0], [6.0, 7.0, 7.0], [8.0, 3.0, 6.0]], [[7.0, 5.0, 4.0], [7.0, 3.0, 5.0], [7.0, 8.0, 2.0]]], [[[3.0, 2.0, 6.0], [4.0, 8.0, 
1.0], [1.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [8.0, 5.0, 6.0], [2.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [6.0, 5.0, 5.0], [3.0, 5.0, 
8.0]]], [[[5.0, 8.0, 5.0], [6.0, 5.0, 6.0], [1.0, 3.0, 6.0]], [[8.0, 3.0, 3.0], [1.0, 1.0, 8.0], [2.0, 4.0, 3.0]], [[7.0, 3.0, 
6.0], [6.0, 1.0, 8.0], [7.0, 6.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0]], [[4.0]], [[1.0]]], [[[4.0]], [[7.0]], [[3.0]]]])+(1.-msk_val)*numpy.array([[[[8.0]], 
[[6.0]], [[8.0]]], [[[7.0]], [[1.0]], [[2.0]]]])
      arg[1:3,:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 1.0], [5.0, 6.0, 7.0], [3.0, 2.0, 1.0]], [[7.0, 2.0, 4.0], [4.0, 4.0, 1.0], [1.0, 
7.0, 1.0]], [[6.0, 1.0, 4.0], [6.0, 7.0, 3.0], [1.0, 8.0, 8.0]]], [[[3.0, 8.0, 8.0], [2.0, 6.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 
2.0, 1.0], [6.0, 4.0, 7.0], [8.0, 3.0, 6.0]], [[7.0, 5.0, 4.0], [7.0, 1.0, 5.0], [7.0, 8.0, 2.0]]], [[[3.0, 2.0, 6.0], [4.0, 
4.0, 1.0], [1.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [8.0, 7.0, 6.0], [2.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [6.0, 3.0, 5.0], [3.0, 5.0, 
8.0]]], [[[5.0, 8.0, 5.0], [6.0, 5.0, 6.0], [1.0, 3.0, 6.0]], [[8.0, 3.0, 3.0], [1.0, 1.0, 8.0], [2.0, 4.0, 3.0]], [[7.0, 3.0, 
6.0], [6.0, 1.0, 8.0], [7.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [5.0, 6.0, 7.0], [3.0, 2.0, 1.0]], 
[[7.0, 2.0, 4.0], [4.0, 4.0, 1.0], [1.0, 7.0, 1.0]], [[6.0, 1.0, 4.0], [6.0, 7.0, 3.0], [1.0, 8.0, 8.0]]], [[[3.0, 8.0, 8.0], 
[2.0, 8.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 2.0, 1.0], [6.0, 6.0, 7.0], [8.0, 3.0, 6.0]], [[7.0, 5.0, 4.0], [7.0, 8.0, 5.0], 
[7.0, 8.0, 2.0]]], [[[3.0, 2.0, 6.0], [4.0, 7.0, 1.0], [1.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [8.0, 1.0, 6.0], [2.0, 3.0, 4.0]], 
[[6.0, 1.0, 8.0], [6.0, 2.0, 5.0], [3.0, 5.0, 8.0]]], [[[5.0, 8.0, 5.0], [6.0, 5.0, 6.0], [1.0, 3.0, 6.0]], [[8.0, 3.0, 3.0], 
[1.0, 1.0, 8.0], [2.0, 4.0, 3.0]], [[7.0, 3.0, 6.0], [6.0, 1.0, 8.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLLN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 8.0], [3.0, 2.0, 6.0], [6.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 3.0], [2.0, 7.0, 
2.0]], [[2.0, 8.0, 1.0], [5.0, 3.0, 1.0], [5.0, 6.0, 8.0]]], [[[1.0, 7.0, 6.0], [5.0, 6.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 1.0, 
4.0], [5.0, 1.0, 2.0], [7.0, 3.0, 4.0]], [[5.0, 7.0, 1.0], [8.0, 1.0, 1.0], [8.0, 7.0, 3.0]]], [[[5.0, 3.0, 1.0], [6.0, 3.0, 
8.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 3.0], [5.0, 6.0, 3.0], [6.0, 8.0, 1.0]], [[5.0, 8.0, 2.0], [3.0, 2.0, 3.0], [8.0, 3.0, 
3.0]]], [[[7.0, 6.0, 3.0], [5.0, 8.0, 2.0], [8.0, 8.0, 7.0]], [[3.0, 7.0, 2.0], [8.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[4.0, 4.0, 
8.0], [3.0, 8.0, 6.0], [1.0, 3.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 7.0, 4.0]]], [[[3.0, 7.0, 7.0]]]])
      arg[1:3,2:,2:]=val
      ref=Data(numpy.array([[[[6.0, 3.0, 8.0], [3.0, 2.0, 6.0], [6.0, 7.0, 2.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 3.0], [2.0, 7.0, 
2.0]], [[2.0, 8.0, 1.0], [5.0, 3.0, 1.0], [5.0, 6.0, 8.0]]], [[[1.0, 7.0, 6.0], [5.0, 6.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 1.0, 
4.0], [5.0, 1.0, 2.0], [7.0, 3.0, 4.0]], [[5.0, 7.0, 1.0], [8.0, 1.0, 1.0], [4.0, 7.0, 4.0]]], [[[5.0, 3.0, 1.0], [6.0, 3.0, 
8.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 3.0], [5.0, 6.0, 3.0], [6.0, 8.0, 1.0]], [[5.0, 8.0, 2.0], [3.0, 2.0, 3.0], [3.0, 7.0, 
7.0]]], [[[7.0, 6.0, 3.0], [5.0, 8.0, 2.0], [8.0, 8.0, 7.0]], [[3.0, 7.0, 2.0], [8.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[4.0, 4.0, 
8.0], [3.0, 8.0, 6.0], [1.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLLN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 8.0], [6.0, 4.0, 6.0], [4.0, 5.0, 7.0]], [[4.0, 1.0, 2.0], [1.0, 3.0, 7.0], [6.0, 1.0, 
4.0]], [[8.0, 5.0, 3.0], [5.0, 4.0, 1.0], [2.0, 8.0, 6.0]]], [[[7.0, 7.0, 2.0], [2.0, 1.0, 6.0], [7.0, 5.0, 3.0]], [[1.0, 3.0, 
5.0], [6.0, 2.0, 8.0], [1.0, 5.0, 4.0]], [[4.0, 5.0, 6.0], [5.0, 4.0, 1.0], [7.0, 3.0, 3.0]]], [[[4.0, 7.0, 4.0], [2.0, 8.0, 
4.0], [5.0, 8.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 3.0], [2.0, 8.0, 6.0]], [[4.0, 1.0, 4.0], [3.0, 8.0, 4.0], [1.0, 5.0, 
8.0]]], [[[7.0, 1.0, 1.0], [4.0, 1.0, 6.0], [5.0, 4.0, 8.0]], [[5.0, 4.0, 6.0], [4.0, 8.0, 7.0], [5.0, 8.0, 8.0]], [[4.0, 8.0, 
7.0], [5.0, 4.0, 1.0], [8.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 2.0, 8.0]]], [[[3.0, 5.0, 4.0]]]]),self.functionspace)
      arg[1:3,2:,2:]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 8.0], [6.0, 4.0, 6.0], [4.0, 5.0, 7.0]], [[4.0, 1.0, 2.0], [1.0, 3.0, 7.0], [6.0, 1.0, 
4.0]], [[8.0, 5.0, 3.0], [5.0, 4.0, 1.0], [2.0, 8.0, 6.0]]], [[[7.0, 7.0, 2.0], [2.0, 1.0, 6.0], [7.0, 5.0, 3.0]], [[1.0, 3.0, 
5.0], [6.0, 2.0, 8.0], [1.0, 5.0, 4.0]], [[4.0, 5.0, 6.0], [5.0, 4.0, 1.0], [4.0, 2.0, 8.0]]], [[[4.0, 7.0, 4.0], [2.0, 8.0, 
4.0], [5.0, 8.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 3.0], [2.0, 8.0, 6.0]], [[4.0, 1.0, 4.0], [3.0, 8.0, 4.0], [3.0, 5.0, 
4.0]]], [[[7.0, 1.0, 1.0], [4.0, 1.0, 6.0], [5.0, 4.0, 8.0]], [[5.0, 4.0, 6.0], [4.0, 8.0, 7.0], [5.0, 8.0, 8.0]], [[4.0, 8.0, 
7.0], [5.0, 4.0, 1.0], [8.0, 8.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLLN(self):
      arg=Data(numpy.array([[[[4.0, 5.0, 2.0], [8.0, 8.0, 4.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 4.0], [7.0, 1.0, 1.0], [1.0, 7.0, 
1.0]], [[8.0, 5.0, 8.0], [3.0, 1.0, 3.0], [6.0, 1.0, 4.0]]], [[[2.0, 8.0, 5.0], [1.0, 6.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 1.0, 
5.0], [1.0, 4.0, 8.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [6.0, 8.0, 8.0], [5.0, 8.0, 4.0]]], [[[6.0, 1.0, 3.0], [8.0, 1.0, 
2.0], [8.0, 8.0, 4.0]], [[2.0, 4.0, 2.0], [6.0, 6.0, 1.0], [1.0, 7.0, 8.0]], [[7.0, 1.0, 5.0], [3.0, 4.0, 3.0], [6.0, 5.0, 
3.0]]], [[[5.0, 7.0, 3.0], [5.0, 7.0, 1.0], [7.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], [3.0, 3.0, 3.0], [7.0, 7.0, 4.0]], [[1.0, 3.0, 
3.0], [3.0, 2.0, 2.0], [6.0, 7.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 6.0, 1.0]]], [[[2.0, 1.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 1.0, 4.0]]], 
[[[8.0, 7.0, 4.0]]]])
      arg[1:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 2.0], [8.0, 8.0, 4.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 4.0], [7.0, 1.0, 1.0], [1.0, 
7.0, 1.0]], [[8.0, 5.0, 8.0], [3.0, 1.0, 3.0], [6.0, 1.0, 4.0]]], [[[2.0, 8.0, 5.0], [1.0, 6.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 
1.0, 5.0], [1.0, 4.0, 8.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [6.0, 8.0, 8.0], [2.0, 6.0, 1.0]]], [[[6.0, 1.0, 3.0], [8.0, 
1.0, 2.0], [8.0, 8.0, 4.0]], [[2.0, 4.0, 2.0], [6.0, 6.0, 1.0], [1.0, 7.0, 8.0]], [[7.0, 1.0, 5.0], [3.0, 4.0, 3.0], [2.0, 1.0, 
8.0]]], [[[5.0, 7.0, 3.0], [5.0, 7.0, 1.0], [7.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], [3.0, 3.0, 3.0], [7.0, 7.0, 4.0]], [[1.0, 3.0, 
3.0], [3.0, 2.0, 2.0], [6.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 2.0], [8.0, 8.0, 4.0], [1.0, 5.0, 3.0]], 
[[4.0, 3.0, 4.0], [7.0, 1.0, 1.0], [1.0, 7.0, 1.0]], [[8.0, 5.0, 8.0], [3.0, 1.0, 3.0], [6.0, 1.0, 4.0]]], [[[2.0, 8.0, 5.0], 
[1.0, 6.0, 3.0], [8.0, 4.0, 4.0]], [[5.0, 1.0, 5.0], [1.0, 4.0, 8.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [6.0, 8.0, 8.0], 
[3.0, 1.0, 4.0]]], [[[6.0, 1.0, 3.0], [8.0, 1.0, 2.0], [8.0, 8.0, 4.0]], [[2.0, 4.0, 2.0], [6.0, 6.0, 1.0], [1.0, 7.0, 8.0]], 
[[7.0, 1.0, 5.0], [3.0, 4.0, 3.0], [8.0, 7.0, 4.0]]], [[[5.0, 7.0, 3.0], [5.0, 7.0, 1.0], [7.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], 
[3.0, 3.0, 3.0], [7.0, 7.0, 4.0]], [[1.0, 3.0, 3.0], [3.0, 2.0, 2.0], [6.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CURN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 1.0], [2.0, 2.0, 5.0], [7.0, 4.0, 6.0]], [[1.0, 2.0, 4.0], [6.0, 4.0, 6.0], [4.0, 3.0, 
1.0]], [[6.0, 5.0, 8.0], [1.0, 4.0, 2.0], [3.0, 8.0, 7.0]]], [[[1.0, 7.0, 7.0], [8.0, 2.0, 5.0], [1.0, 5.0, 4.0]], [[1.0, 5.0, 
1.0], [7.0, 7.0, 4.0], [3.0, 5.0, 8.0]], [[3.0, 8.0, 5.0], [2.0, 1.0, 8.0], [6.0, 1.0, 6.0]]], [[[2.0, 1.0, 8.0], [1.0, 2.0, 
8.0], [3.0, 3.0, 3.0]], [[2.0, 8.0, 8.0], [7.0, 2.0, 6.0], [8.0, 5.0, 6.0]], [[3.0, 1.0, 2.0], [7.0, 2.0, 6.0], [7.0, 1.0, 
2.0]]], [[[7.0, 2.0, 2.0], [5.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[6.0, 7.0, 4.0], [4.0, 2.0, 8.0], [8.0, 3.0, 2.0]], [[1.0, 3.0, 
2.0], [5.0, 1.0, 1.0], [8.0, 2.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 3.0, 8.0]], [[3.0, 2.0, 3.0]]], [[[6.0, 7.0, 3.0]], [[3.0, 4.0, 3.0]]], [[[8.0, 2.0, 4.0]], 
[[5.0, 2.0, 5.0]]], [[[3.0, 5.0, 8.0]], [[1.0, 3.0, 5.0]]]])
      arg[:,:2,1:2]=val
      ref=Data(numpy.array([[[[7.0, 5.0, 1.0], [8.0, 3.0, 8.0], [7.0, 4.0, 6.0]], [[1.0, 2.0, 4.0], [3.0, 2.0, 3.0], [4.0, 3.0, 
1.0]], [[6.0, 5.0, 8.0], [1.0, 4.0, 2.0], [3.0, 8.0, 7.0]]], [[[1.0, 7.0, 7.0], [6.0, 7.0, 3.0], [1.0, 5.0, 4.0]], [[1.0, 5.0, 
1.0], [3.0, 4.0, 3.0], [3.0, 5.0, 8.0]], [[3.0, 8.0, 5.0], [2.0, 1.0, 8.0], [6.0, 1.0, 6.0]]], [[[2.0, 1.0, 8.0], [8.0, 2.0, 
4.0], [3.0, 3.0, 3.0]], [[2.0, 8.0, 8.0], [5.0, 2.0, 5.0], [8.0, 5.0, 6.0]], [[3.0, 1.0, 2.0], [7.0, 2.0, 6.0], [7.0, 1.0, 
2.0]]], [[[7.0, 2.0, 2.0], [3.0, 5.0, 8.0], [2.0, 5.0, 3.0]], [[6.0, 7.0, 4.0], [1.0, 3.0, 5.0], [8.0, 3.0, 2.0]], [[1.0, 3.0, 
2.0], [5.0, 1.0, 1.0], [8.0, 2.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CURN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 4.0], [3.0, 4.0, 4.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 5.0], [8.0, 3.0, 5.0], [1.0, 1.0, 
4.0]], [[6.0, 5.0, 7.0], [6.0, 4.0, 7.0], [5.0, 1.0, 6.0]]], [[[8.0, 3.0, 8.0], [1.0, 8.0, 2.0], [2.0, 7.0, 3.0]], [[2.0, 4.0, 
1.0], [5.0, 1.0, 7.0], [3.0, 6.0, 5.0]], [[6.0, 3.0, 3.0], [1.0, 2.0, 3.0], [3.0, 2.0, 3.0]]], [[[7.0, 4.0, 6.0], [2.0, 8.0, 
5.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 4.0], [5.0, 7.0, 3.0], [3.0, 6.0, 4.0]], [[3.0, 3.0, 1.0], [8.0, 3.0, 7.0], [6.0, 8.0, 
7.0]]], [[[5.0, 3.0, 6.0], [2.0, 4.0, 8.0], [2.0, 8.0, 8.0]], [[8.0, 4.0, 6.0], [6.0, 2.0, 7.0], [5.0, 7.0, 8.0]], [[1.0, 2.0, 
4.0], [4.0, 2.0, 1.0], [7.0, 8.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 4.0, 7.0]], [[3.0, 4.0, 4.0]]], [[[3.0, 1.0, 2.0]], [[7.0, 6.0, 7.0]]], [[[6.0, 1.0, 1.0]], 
[[7.0, 8.0, 2.0]]], [[[3.0, 6.0, 8.0]], [[4.0, 3.0, 8.0]]]]),self.functionspace)
      arg[:,:2,1:2]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 4.0], [8.0, 4.0, 7.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 5.0], [3.0, 4.0, 4.0], [1.0, 1.0, 
4.0]], [[6.0, 5.0, 7.0], [6.0, 4.0, 7.0], [5.0, 1.0, 6.0]]], [[[8.0, 3.0, 8.0], [3.0, 1.0, 2.0], [2.0, 7.0, 3.0]], [[2.0, 4.0, 
1.0], [7.0, 6.0, 7.0], [3.0, 6.0, 5.0]], [[6.0, 3.0, 3.0], [1.0, 2.0, 3.0], [3.0, 2.0, 3.0]]], [[[7.0, 4.0, 6.0], [6.0, 1.0, 
1.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 4.0], [7.0, 8.0, 2.0], [3.0, 6.0, 4.0]], [[3.0, 3.0, 1.0], [8.0, 3.0, 7.0], [6.0, 8.0, 
7.0]]], [[[5.0, 3.0, 6.0], [3.0, 6.0, 8.0], [2.0, 8.0, 8.0]], [[8.0, 4.0, 6.0], [4.0, 3.0, 8.0], [5.0, 7.0, 8.0]], [[1.0, 2.0, 
4.0], [4.0, 2.0, 1.0], [7.0, 8.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CURN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 1.0], [5.0, 5.0, 3.0], [1.0, 7.0, 3.0]], [[6.0, 3.0, 7.0], [3.0, 1.0, 4.0], [6.0, 2.0, 
5.0]], [[7.0, 3.0, 6.0], [2.0, 2.0, 5.0], [1.0, 4.0, 8.0]]], [[[2.0, 7.0, 7.0], [3.0, 7.0, 4.0], [7.0, 5.0, 3.0]], [[1.0, 8.0, 
3.0], [5.0, 3.0, 8.0], [3.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [3.0, 4.0, 8.0], [8.0, 5.0, 7.0]]], [[[6.0, 8.0, 8.0], [8.0, 7.0, 
5.0], [7.0, 7.0, 3.0]], [[8.0, 8.0, 7.0], [1.0, 2.0, 2.0], [5.0, 2.0, 1.0]], [[4.0, 1.0, 6.0], [3.0, 7.0, 4.0], [7.0, 5.0, 
6.0]]], [[[3.0, 1.0, 1.0], [4.0, 7.0, 4.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 6.0], [4.0, 5.0, 7.0], [2.0, 5.0, 2.0]], [[4.0, 2.0, 
5.0], [2.0, 2.0, 6.0], [4.0, 3.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 5.0, 5.0]], [[4.0, 5.0, 8.0]]], [[[6.0, 7.0, 8.0]], [[1.0, 5.0, 5.0]]], [[[4.0, 3.0, 
4.0]], [[7.0, 3.0, 2.0]]], [[[2.0, 5.0, 1.0]], [[5.0, 6.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 5.0, 7.0]], [[7.0, 7.0, 
5.0]]], [[[7.0, 3.0, 3.0]], [[3.0, 6.0, 5.0]]], [[[6.0, 8.0, 3.0]], [[6.0, 1.0, 2.0]]], [[[3.0, 3.0, 2.0]], [[8.0, 1.0, 6.0]]]])
      arg[:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 1.0], [4.0, 5.0, 5.0], [1.0, 7.0, 3.0]], [[6.0, 3.0, 7.0], [4.0, 5.0, 8.0], [6.0, 
2.0, 5.0]], [[7.0, 3.0, 6.0], [2.0, 2.0, 5.0], [1.0, 4.0, 8.0]]], [[[2.0, 7.0, 7.0], [6.0, 7.0, 8.0], [7.0, 5.0, 3.0]], [[1.0, 
8.0, 3.0], [1.0, 5.0, 5.0], [3.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [3.0, 4.0, 8.0], [8.0, 5.0, 7.0]]], [[[6.0, 8.0, 8.0], [4.0, 
3.0, 4.0], [7.0, 7.0, 3.0]], [[8.0, 8.0, 7.0], [7.0, 3.0, 2.0], [5.0, 2.0, 1.0]], [[4.0, 1.0, 6.0], [3.0, 7.0, 4.0], [7.0, 5.0, 
6.0]]], [[[3.0, 1.0, 1.0], [2.0, 5.0, 1.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 6.0], [5.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[4.0, 2.0, 
5.0], [2.0, 2.0, 6.0], [4.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 1.0], [5.0, 5.0, 7.0], [1.0, 7.0, 3.0]], 
[[6.0, 3.0, 7.0], [7.0, 7.0, 5.0], [6.0, 2.0, 5.0]], [[7.0, 3.0, 6.0], [2.0, 2.0, 5.0], [1.0, 4.0, 8.0]]], [[[2.0, 7.0, 7.0], 
[7.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[1.0, 8.0, 3.0], [3.0, 6.0, 5.0], [3.0, 1.0, 1.0]], [[2.0, 1.0, 6.0], [3.0, 4.0, 8.0], 
[8.0, 5.0, 7.0]]], [[[6.0, 8.0, 8.0], [6.0, 8.0, 3.0], [7.0, 7.0, 3.0]], [[8.0, 8.0, 7.0], [6.0, 1.0, 2.0], [5.0, 2.0, 1.0]], 
[[4.0, 1.0, 6.0], [3.0, 7.0, 4.0], [7.0, 5.0, 6.0]]], [[[3.0, 1.0, 1.0], [3.0, 3.0, 2.0], [7.0, 2.0, 7.0]], [[6.0, 6.0, 6.0], 
[8.0, 1.0, 6.0], [2.0, 5.0, 2.0]], [[4.0, 2.0, 5.0], [2.0, 2.0, 6.0], [4.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLLN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 8.0], [4.0, 8.0, 6.0], [8.0, 4.0, 8.0]], [[2.0, 5.0, 4.0], [8.0, 6.0, 5.0], [4.0, 1.0, 
2.0]], [[3.0, 8.0, 2.0], [8.0, 6.0, 4.0], [5.0, 8.0, 7.0]]], [[[8.0, 6.0, 7.0], [1.0, 7.0, 5.0], [6.0, 6.0, 8.0]], [[3.0, 2.0, 
3.0], [1.0, 7.0, 6.0], [4.0, 2.0, 1.0]], [[5.0, 4.0, 8.0], [5.0, 7.0, 6.0], [7.0, 4.0, 3.0]]], [[[8.0, 6.0, 3.0], [6.0, 4.0, 
8.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 3.0], [6.0, 1.0, 6.0], [7.0, 8.0, 6.0]], [[1.0, 3.0, 1.0], [2.0, 2.0, 6.0], [8.0, 8.0, 
8.0]]], [[[1.0, 7.0, 4.0], [1.0, 5.0, 2.0], [5.0, 5.0, 5.0]], [[8.0, 8.0, 7.0], [2.0, 6.0, 6.0], [4.0, 2.0, 1.0]], [[2.0, 3.0, 
4.0], [3.0, 4.0, 4.0], [2.0, 3.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 8.0, 6.0]]], [[[8.0, 4.0, 5.0]]]])
      arg[2:,2:,2:]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 8.0], [4.0, 8.0, 6.0], [8.0, 4.0, 8.0]], [[2.0, 5.0, 4.0], [8.0, 6.0, 5.0], [4.0, 1.0, 
2.0]], [[3.0, 8.0, 2.0], [8.0, 6.0, 4.0], [5.0, 8.0, 7.0]]], [[[8.0, 6.0, 7.0], [1.0, 7.0, 5.0], [6.0, 6.0, 8.0]], [[3.0, 2.0, 
3.0], [1.0, 7.0, 6.0], [4.0, 2.0, 1.0]], [[5.0, 4.0, 8.0], [5.0, 7.0, 6.0], [7.0, 4.0, 3.0]]], [[[8.0, 6.0, 3.0], [6.0, 4.0, 
8.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 3.0], [6.0, 1.0, 6.0], [7.0, 8.0, 6.0]], [[1.0, 3.0, 1.0], [2.0, 2.0, 6.0], [2.0, 8.0, 
6.0]]], [[[1.0, 7.0, 4.0], [1.0, 5.0, 2.0], [5.0, 5.0, 5.0]], [[8.0, 8.0, 7.0], [2.0, 6.0, 6.0], [4.0, 2.0, 1.0]], [[2.0, 3.0, 
4.0], [3.0, 4.0, 4.0], [8.0, 4.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLLN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 1.0], [7.0, 8.0, 8.0], [5.0, 7.0, 8.0]], [[1.0, 2.0, 5.0], [5.0, 3.0, 3.0], [7.0, 2.0, 
8.0]], [[5.0, 8.0, 6.0], [3.0, 2.0, 3.0], [6.0, 5.0, 3.0]]], [[[8.0, 4.0, 2.0], [5.0, 4.0, 1.0], [6.0, 2.0, 8.0]], [[2.0, 8.0, 
4.0], [3.0, 5.0, 2.0], [5.0, 8.0, 1.0]], [[7.0, 5.0, 8.0], [2.0, 7.0, 1.0], [6.0, 8.0, 3.0]]], [[[6.0, 5.0, 2.0], [5.0, 2.0, 
5.0], [2.0, 6.0, 1.0]], [[7.0, 4.0, 5.0], [8.0, 8.0, 6.0], [5.0, 5.0, 3.0]], [[7.0, 6.0, 2.0], [5.0, 1.0, 7.0], [7.0, 2.0, 
7.0]]], [[[2.0, 3.0, 1.0], [6.0, 8.0, 4.0], [5.0, 3.0, 5.0]], [[7.0, 4.0, 8.0], [3.0, 8.0, 8.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 
6.0], [4.0, 7.0, 2.0], [5.0, 4.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 2.0, 6.0]]], [[[5.0, 1.0, 1.0]]]]),self.functionspace)
      arg[2:,2:,2:]=val
      ref=Data(numpy.array([[[[2.0, 7.0, 1.0], [7.0, 8.0, 8.0], [5.0, 7.0, 8.0]], [[1.0, 2.0, 5.0], [5.0, 3.0, 3.0], [7.0, 2.0, 
8.0]], [[5.0, 8.0, 6.0], [3.0, 2.0, 3.0], [6.0, 5.0, 3.0]]], [[[8.0, 4.0, 2.0], [5.0, 4.0, 1.0], [6.0, 2.0, 8.0]], [[2.0, 8.0, 
4.0], [3.0, 5.0, 2.0], [5.0, 8.0, 1.0]], [[7.0, 5.0, 8.0], [2.0, 7.0, 1.0], [6.0, 8.0, 3.0]]], [[[6.0, 5.0, 2.0], [5.0, 2.0, 
5.0], [2.0, 6.0, 1.0]], [[7.0, 4.0, 5.0], [8.0, 8.0, 6.0], [5.0, 5.0, 3.0]], [[7.0, 6.0, 2.0], [5.0, 1.0, 7.0], [3.0, 2.0, 
6.0]]], [[[2.0, 3.0, 1.0], [6.0, 8.0, 4.0], [5.0, 3.0, 5.0]], [[7.0, 4.0, 8.0], [3.0, 8.0, 8.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 
6.0], [4.0, 7.0, 2.0], [5.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLLN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 2.0], [6.0, 3.0, 7.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 1.0], [4.0, 5.0, 5.0], [8.0, 6.0, 
3.0]], [[5.0, 8.0, 8.0], [2.0, 4.0, 5.0], [8.0, 5.0, 1.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 7.0], [5.0, 5.0, 6.0]], [[4.0, 2.0, 
1.0], [4.0, 6.0, 5.0], [1.0, 3.0, 1.0]], [[8.0, 8.0, 4.0], [3.0, 5.0, 4.0], [7.0, 8.0, 4.0]]], [[[6.0, 7.0, 2.0], [6.0, 2.0, 
4.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 4.0], [6.0, 7.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [5.0, 2.0, 6.0], [8.0, 7.0, 
3.0]]], [[[4.0, 1.0, 7.0], [7.0, 7.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 4.0], [1.0, 8.0, 5.0], [5.0, 8.0, 2.0]], [[8.0, 5.0, 
3.0], [3.0, 6.0, 3.0], [6.0, 5.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 5.0, 7.0]]], [[[7.0, 5.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 3.0, 3.0]]], 
[[[2.0, 6.0, 1.0]]]])
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 2.0], [6.0, 3.0, 7.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 1.0], [4.0, 5.0, 5.0], [8.0, 
6.0, 3.0]], [[5.0, 8.0, 8.0], [2.0, 4.0, 5.0], [8.0, 5.0, 1.0]]], [[[8.0, 8.0, 6.0], [5.0, 8.0, 7.0], [5.0, 5.0, 6.0]], [[4.0, 
2.0, 1.0], [4.0, 6.0, 5.0], [1.0, 3.0, 1.0]], [[8.0, 8.0, 4.0], [3.0, 5.0, 4.0], [7.0, 8.0, 4.0]]], [[[6.0, 7.0, 2.0], [6.0, 
2.0, 4.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 4.0], [6.0, 7.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [5.0, 2.0, 6.0], [6.0, 5.0, 
7.0]]], [[[4.0, 1.0, 7.0], [7.0, 7.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 4.0], [1.0, 8.0, 5.0], [5.0, 8.0, 2.0]], [[8.0, 5.0, 
3.0], [3.0, 6.0, 3.0], [7.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 2.0], [6.0, 3.0, 7.0], [7.0, 6.0, 8.0]], 
[[6.0, 5.0, 1.0], [4.0, 5.0, 5.0], [8.0, 6.0, 3.0]], [[5.0, 8.0, 8.0], [2.0, 4.0, 5.0], [8.0, 5.0, 1.0]]], [[[8.0, 8.0, 6.0], 
[5.0, 8.0, 7.0], [5.0, 5.0, 6.0]], [[4.0, 2.0, 1.0], [4.0, 6.0, 5.0], [1.0, 3.0, 1.0]], [[8.0, 8.0, 4.0], [3.0, 5.0, 4.0], 
[7.0, 8.0, 4.0]]], [[[6.0, 7.0, 2.0], [6.0, 2.0, 4.0], [8.0, 7.0, 3.0]], [[6.0, 3.0, 4.0], [6.0, 7.0, 4.0], [5.0, 2.0, 8.0]], 
[[3.0, 2.0, 4.0], [5.0, 2.0, 6.0], [4.0, 3.0, 3.0]]], [[[4.0, 1.0, 7.0], [7.0, 7.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 4.0, 4.0], 
[1.0, 8.0, 5.0], [5.0, 8.0, 2.0]], [[8.0, 5.0, 3.0], [3.0, 6.0, 3.0], [2.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URRN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 8.0], [7.0, 6.0, 4.0], [1.0, 4.0, 5.0]], [[6.0, 1.0, 1.0], [6.0, 8.0, 6.0], [4.0, 6.0, 
4.0]], [[6.0, 6.0, 3.0], [5.0, 2.0, 8.0], [7.0, 3.0, 7.0]]], [[[2.0, 2.0, 3.0], [8.0, 1.0, 8.0], [7.0, 6.0, 5.0]], [[7.0, 8.0, 
1.0], [4.0, 6.0, 5.0], [8.0, 3.0, 3.0]], [[2.0, 5.0, 7.0], [7.0, 4.0, 5.0], [1.0, 8.0, 8.0]]], [[[6.0, 8.0, 1.0], [8.0, 8.0, 
8.0], [3.0, 2.0, 8.0]], [[8.0, 5.0, 2.0], [5.0, 5.0, 5.0], [5.0, 7.0, 8.0]], [[7.0, 5.0, 1.0], [8.0, 7.0, 2.0], [2.0, 7.0, 
5.0]]], [[[8.0, 2.0, 2.0], [8.0, 6.0, 7.0], [4.0, 6.0, 2.0]], [[6.0, 1.0, 1.0], [5.0, 1.0, 5.0], [7.0, 8.0, 3.0]], [[7.0, 5.0, 
2.0], [7.0, 3.0, 6.0], [3.0, 4.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 8.0, 2.0]]], [[[1.0, 6.0, 1.0]]], [[[4.0, 2.0, 3.0]]]])
      arg[:3,1:2,1:2]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 8.0], [7.0, 6.0, 4.0], [1.0, 4.0, 5.0]], [[6.0, 1.0, 1.0], [6.0, 8.0, 2.0], [4.0, 6.0, 
4.0]], [[6.0, 6.0, 3.0], [5.0, 2.0, 8.0], [7.0, 3.0, 7.0]]], [[[2.0, 2.0, 3.0], [8.0, 1.0, 8.0], [7.0, 6.0, 5.0]], [[7.0, 8.0, 
1.0], [1.0, 6.0, 1.0], [8.0, 3.0, 3.0]], [[2.0, 5.0, 7.0], [7.0, 4.0, 5.0], [1.0, 8.0, 8.0]]], [[[6.0, 8.0, 1.0], [8.0, 8.0, 
8.0], [3.0, 2.0, 8.0]], [[8.0, 5.0, 2.0], [4.0, 2.0, 3.0], [5.0, 7.0, 8.0]], [[7.0, 5.0, 1.0], [8.0, 7.0, 2.0], [2.0, 7.0, 
5.0]]], [[[8.0, 2.0, 2.0], [8.0, 6.0, 7.0], [4.0, 6.0, 2.0]], [[6.0, 1.0, 1.0], [5.0, 1.0, 5.0], [7.0, 8.0, 3.0]], [[7.0, 5.0, 
2.0], [7.0, 3.0, 6.0], [3.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URRN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 3.0], [5.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[2.0, 6.0, 3.0], [7.0, 4.0, 2.0], [8.0, 7.0, 
4.0]], [[6.0, 8.0, 4.0], [5.0, 7.0, 5.0], [1.0, 2.0, 6.0]]], [[[2.0, 2.0, 6.0], [2.0, 1.0, 4.0], [3.0, 8.0, 1.0]], [[3.0, 2.0, 
4.0], [6.0, 1.0, 1.0], [8.0, 2.0, 8.0]], [[4.0, 6.0, 8.0], [3.0, 1.0, 8.0], [7.0, 1.0, 3.0]]], [[[1.0, 3.0, 8.0], [5.0, 1.0, 
3.0], [6.0, 1.0, 3.0]], [[8.0, 5.0, 2.0], [8.0, 3.0, 3.0], [5.0, 7.0, 6.0]], [[8.0, 7.0, 5.0], [1.0, 7.0, 4.0], [1.0, 1.0, 
3.0]]], [[[6.0, 3.0, 6.0], [1.0, 4.0, 2.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 2.0], [6.0, 7.0, 8.0], [2.0, 5.0, 6.0]], [[3.0, 6.0, 
2.0], [3.0, 4.0, 7.0], [2.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[8.0, 4.0, 1.0]]], [[[8.0, 4.0, 8.0]]], [[[8.0, 8.0, 5.0]]]]),self.functionspace)
      arg[:3,1:2,1:2]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 3.0], [5.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[2.0, 6.0, 3.0], [8.0, 4.0, 1.0], [8.0, 7.0, 
4.0]], [[6.0, 8.0, 4.0], [5.0, 7.0, 5.0], [1.0, 2.0, 6.0]]], [[[2.0, 2.0, 6.0], [2.0, 1.0, 4.0], [3.0, 8.0, 1.0]], [[3.0, 2.0, 
4.0], [8.0, 4.0, 8.0], [8.0, 2.0, 8.0]], [[4.0, 6.0, 8.0], [3.0, 1.0, 8.0], [7.0, 1.0, 3.0]]], [[[1.0, 3.0, 8.0], [5.0, 1.0, 
3.0], [6.0, 1.0, 3.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 5.0], [5.0, 7.0, 6.0]], [[8.0, 7.0, 5.0], [1.0, 7.0, 4.0], [1.0, 1.0, 
3.0]]], [[[6.0, 3.0, 6.0], [1.0, 4.0, 2.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 2.0], [6.0, 7.0, 8.0], [2.0, 5.0, 6.0]], [[3.0, 6.0, 
2.0], [3.0, 4.0, 7.0], [2.0, 8.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URRN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 5.0], [3.0, 8.0, 8.0], [1.0, 3.0, 1.0]], [[3.0, 8.0, 7.0], [1.0, 3.0, 6.0], [7.0, 5.0, 
6.0]], [[6.0, 8.0, 3.0], [4.0, 7.0, 1.0], [7.0, 7.0, 6.0]]], [[[8.0, 5.0, 4.0], [6.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[4.0, 5.0, 
6.0], [3.0, 7.0, 5.0], [6.0, 5.0, 4.0]], [[6.0, 7.0, 5.0], [7.0, 6.0, 4.0], [1.0, 7.0, 2.0]]], [[[6.0, 8.0, 8.0], [3.0, 3.0, 
3.0], [1.0, 7.0, 6.0]], [[8.0, 5.0, 6.0], [5.0, 2.0, 8.0], [7.0, 5.0, 5.0]], [[7.0, 4.0, 1.0], [2.0, 2.0, 8.0], [3.0, 6.0, 
6.0]]], [[[6.0, 1.0, 8.0], [4.0, 8.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], [3.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 
4.0], [7.0, 1.0, 7.0], [5.0, 6.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 1.0, 8.0]]], [[[1.0, 1.0, 6.0]]], [[[4.0, 6.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 5.0, 7.0]]], [[[7.0, 7.0, 6.0]]], [[[5.0, 8.0, 3.0]]]])
      arg[:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 5.0], [3.0, 8.0, 8.0], [1.0, 3.0, 1.0]], [[3.0, 8.0, 7.0], [3.0, 1.0, 8.0], [7.0, 
5.0, 6.0]], [[6.0, 8.0, 3.0], [4.0, 7.0, 1.0], [7.0, 7.0, 6.0]]], [[[8.0, 5.0, 4.0], [6.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[4.0, 
5.0, 6.0], [1.0, 1.0, 6.0], [6.0, 5.0, 4.0]], [[6.0, 7.0, 5.0], [7.0, 6.0, 4.0], [1.0, 7.0, 2.0]]], [[[6.0, 8.0, 8.0], [3.0, 
3.0, 3.0], [1.0, 7.0, 6.0]], [[8.0, 5.0, 6.0], [4.0, 6.0, 2.0], [7.0, 5.0, 5.0]], [[7.0, 4.0, 1.0], [2.0, 2.0, 8.0], [3.0, 6.0, 
6.0]]], [[[6.0, 1.0, 8.0], [4.0, 8.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], [3.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 
4.0], [7.0, 1.0, 7.0], [5.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [3.0, 8.0, 8.0], [1.0, 3.0, 1.0]], 
[[3.0, 8.0, 7.0], [3.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[6.0, 8.0, 3.0], [4.0, 7.0, 1.0], [7.0, 7.0, 6.0]]], [[[8.0, 5.0, 4.0], 
[6.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[4.0, 5.0, 6.0], [7.0, 7.0, 6.0], [6.0, 5.0, 4.0]], [[6.0, 7.0, 5.0], [7.0, 6.0, 4.0], 
[1.0, 7.0, 2.0]]], [[[6.0, 8.0, 8.0], [3.0, 3.0, 3.0], [1.0, 7.0, 6.0]], [[8.0, 5.0, 6.0], [5.0, 8.0, 3.0], [7.0, 5.0, 5.0]], 
[[7.0, 4.0, 1.0], [2.0, 2.0, 8.0], [3.0, 6.0, 6.0]]], [[[6.0, 1.0, 8.0], [4.0, 8.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], 
[3.0, 5.0, 3.0], [2.0, 4.0, 5.0]], [[1.0, 1.0, 4.0], [7.0, 1.0, 7.0], [5.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCCC(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 5.0], [4.0, 8.0, 8.0], [2.0, 3.0, 5.0]], [[1.0, 6.0, 6.0], [3.0, 1.0, 7.0], [6.0, 8.0, 
1.0]], [[3.0, 2.0, 1.0], [5.0, 1.0, 5.0], [6.0, 3.0, 2.0]]], [[[8.0, 8.0, 3.0], [7.0, 6.0, 5.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 
1.0], [2.0, 7.0, 3.0], [6.0, 2.0, 5.0]], [[2.0, 5.0, 1.0], [7.0, 3.0, 7.0], [1.0, 6.0, 6.0]]], [[[2.0, 6.0, 2.0], [8.0, 7.0, 
4.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 7.0, 8.0], [5.0, 3.0, 7.0]], [[4.0, 1.0, 1.0], [2.0, 2.0, 4.0], [5.0, 2.0, 
4.0]]], [[[3.0, 5.0, 7.0], [5.0, 6.0, 1.0], [7.0, 6.0, 4.0]], [[2.0, 5.0, 5.0], [8.0, 6.0, 2.0], [3.0, 2.0, 8.0]], [[2.0, 6.0, 
5.0], [8.0, 6.0, 3.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 4.0, 4.0], [2.0, 8.0, 6.0], [5.0, 4.0, 5.0]], [[6.0, 5.0, 2.0], [1.0, 4.0, 3.0], [8.0, 4.0, 
5.0]], [[2.0, 5.0, 1.0], [4.0, 3.0, 3.0], [7.0, 3.0, 8.0]]], [[[6.0, 5.0, 6.0], [3.0, 1.0, 3.0], [1.0, 6.0, 1.0]], [[7.0, 6.0, 
2.0], [1.0, 1.0, 6.0], [4.0, 5.0, 1.0]], [[8.0, 6.0, 7.0], [2.0, 7.0, 1.0], [1.0, 5.0, 7.0]]], [[[1.0, 4.0, 7.0], [5.0, 8.0, 
3.0], [7.0, 8.0, 1.0]], [[3.0, 3.0, 6.0], [8.0, 7.0, 5.0], [8.0, 6.0, 5.0]], [[3.0, 1.0, 6.0], [8.0, 6.0, 3.0], [6.0, 4.0, 
3.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 4.0], [4.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [4.0, 2.0, 6.0], [2.0, 7.0, 7.0]], [[8.0, 8.0, 
1.0], [5.0, 5.0, 1.0], [6.0, 2.0, 7.0]]]])
      arg[:,:,:,:]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 4.0], [2.0, 8.0, 6.0], [5.0, 4.0, 5.0]], [[6.0, 5.0, 2.0], [1.0, 4.0, 3.0], [8.0, 4.0, 
5.0]], [[2.0, 5.0, 1.0], [4.0, 3.0, 3.0], [7.0, 3.0, 8.0]]], [[[6.0, 5.0, 6.0], [3.0, 1.0, 3.0], [1.0, 6.0, 1.0]], [[7.0, 6.0, 
2.0], [1.0, 1.0, 6.0], [4.0, 5.0, 1.0]], [[8.0, 6.0, 7.0], [2.0, 7.0, 1.0], [1.0, 5.0, 7.0]]], [[[1.0, 4.0, 7.0], [5.0, 8.0, 
3.0], [7.0, 8.0, 1.0]], [[3.0, 3.0, 6.0], [8.0, 7.0, 5.0], [8.0, 6.0, 5.0]], [[3.0, 1.0, 6.0], [8.0, 6.0, 3.0], [6.0, 4.0, 
3.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 4.0], [4.0, 3.0, 7.0]], [[7.0, 2.0, 2.0], [4.0, 2.0, 6.0], [2.0, 7.0, 7.0]], [[8.0, 8.0, 
1.0], [5.0, 5.0, 1.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCCC(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 8.0], [7.0, 6.0, 6.0], [2.0, 1.0, 1.0]], [[8.0, 4.0, 3.0], [5.0, 1.0, 7.0], [1.0, 1.0, 
4.0]], [[6.0, 3.0, 1.0], [4.0, 5.0, 5.0], [2.0, 4.0, 5.0]]], [[[4.0, 4.0, 3.0], [1.0, 7.0, 5.0], [6.0, 1.0, 2.0]], [[6.0, 3.0, 
1.0], [4.0, 3.0, 2.0], [7.0, 3.0, 4.0]], [[7.0, 8.0, 3.0], [1.0, 4.0, 4.0], [8.0, 1.0, 7.0]]], [[[2.0, 7.0, 2.0], [8.0, 8.0, 
3.0], [6.0, 8.0, 6.0]], [[8.0, 8.0, 5.0], [8.0, 6.0, 5.0], [4.0, 5.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 2.0, 7.0], [4.0, 5.0, 
5.0]]], [[[5.0, 2.0, 1.0], [6.0, 4.0, 5.0], [3.0, 8.0, 3.0]], [[4.0, 8.0, 7.0], [7.0, 6.0, 4.0], [6.0, 4.0, 1.0]], [[8.0, 3.0, 
5.0], [1.0, 1.0, 1.0], [1.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 7.0, 5.0], [4.0, 7.0, 5.0], [7.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], [3.0, 3.0, 5.0], [2.0, 2.0, 
6.0]], [[4.0, 3.0, 4.0], [4.0, 2.0, 3.0], [8.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [8.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[8.0, 1.0, 
6.0], [3.0, 7.0, 8.0], [6.0, 6.0, 3.0]], [[3.0, 2.0, 1.0], [2.0, 7.0, 6.0], [2.0, 5.0, 4.0]]], [[[3.0, 6.0, 5.0], [3.0, 7.0, 
6.0], [6.0, 3.0, 5.0]], [[3.0, 3.0, 8.0], [8.0, 8.0, 8.0], [4.0, 3.0, 2.0]], [[4.0, 5.0, 3.0], [4.0, 5.0, 4.0], [1.0, 5.0, 
8.0]]], [[[8.0, 4.0, 5.0], [2.0, 1.0, 4.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 4.0], [5.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 
6.0], [7.0, 6.0, 3.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      arg[:,:,:,:]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 5.0], [4.0, 7.0, 5.0], [7.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], [3.0, 3.0, 5.0], [2.0, 2.0, 
6.0]], [[4.0, 3.0, 4.0], [4.0, 2.0, 3.0], [8.0, 8.0, 5.0]]], [[[5.0, 6.0, 8.0], [8.0, 6.0, 8.0], [8.0, 7.0, 3.0]], [[8.0, 1.0, 
6.0], [3.0, 7.0, 8.0], [6.0, 6.0, 3.0]], [[3.0, 2.0, 1.0], [2.0, 7.0, 6.0], [2.0, 5.0, 4.0]]], [[[3.0, 6.0, 5.0], [3.0, 7.0, 
6.0], [6.0, 3.0, 5.0]], [[3.0, 3.0, 8.0], [8.0, 8.0, 8.0], [4.0, 3.0, 2.0]], [[4.0, 5.0, 3.0], [4.0, 5.0, 4.0], [1.0, 5.0, 
8.0]]], [[[8.0, 4.0, 5.0], [2.0, 1.0, 4.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 4.0], [5.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 
6.0], [7.0, 6.0, 3.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCCC(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 4.0], [1.0, 5.0, 1.0], [6.0, 7.0, 4.0]], [[5.0, 5.0, 7.0], [7.0, 3.0, 5.0], [5.0, 6.0, 
7.0]], [[5.0, 8.0, 6.0], [8.0, 2.0, 7.0], [5.0, 6.0, 8.0]]], [[[5.0, 7.0, 8.0], [4.0, 7.0, 3.0], [2.0, 2.0, 6.0]], [[8.0, 1.0, 
7.0], [7.0, 5.0, 2.0], [3.0, 5.0, 5.0]], [[7.0, 6.0, 2.0], [2.0, 3.0, 2.0], [1.0, 3.0, 1.0]]], [[[2.0, 8.0, 6.0], [8.0, 1.0, 
6.0], [1.0, 2.0, 5.0]], [[4.0, 2.0, 2.0], [3.0, 8.0, 3.0], [2.0, 4.0, 4.0]], [[4.0, 6.0, 7.0], [4.0, 4.0, 2.0], [4.0, 4.0, 
6.0]]], [[[4.0, 6.0, 5.0], [1.0, 6.0, 7.0], [8.0, 1.0, 6.0]], [[8.0, 1.0, 3.0], [7.0, 8.0, 5.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 
1.0], [6.0, 3.0, 1.0], [4.0, 5.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 6.0, 6.0], [5.0, 8.0, 2.0], [2.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], [6.0, 6.0, 3.0], [6.0, 
7.0, 5.0]], [[5.0, 4.0, 8.0], [1.0, 4.0, 1.0], [6.0, 8.0, 8.0]]], [[[1.0, 8.0, 1.0], [1.0, 3.0, 8.0], [4.0, 2.0, 3.0]], [[3.0, 
5.0, 5.0], [3.0, 6.0, 2.0], [1.0, 7.0, 7.0]], [[5.0, 6.0, 1.0], [3.0, 3.0, 3.0], [3.0, 6.0, 4.0]]], [[[7.0, 7.0, 8.0], [1.0, 
6.0, 1.0], [1.0, 4.0, 4.0]], [[6.0, 7.0, 3.0], [2.0, 3.0, 7.0], [1.0, 3.0, 3.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 4.0], [3.0, 5.0, 
6.0]]], [[[7.0, 6.0, 5.0], [1.0, 4.0, 3.0], [2.0, 5.0, 5.0]], [[7.0, 3.0, 3.0], [3.0, 8.0, 7.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 
4.0], [2.0, 6.0, 1.0], [1.0, 8.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 3.0, 3.0], [7.0, 1.0, 6.0], [8.0, 8.0, 3.0]], 
[[4.0, 5.0, 3.0], [4.0, 7.0, 3.0], [3.0, 7.0, 6.0]], [[4.0, 4.0, 4.0], [1.0, 8.0, 7.0], [1.0, 1.0, 1.0]]], [[[5.0, 4.0, 8.0], 
[2.0, 2.0, 8.0], [2.0, 3.0, 3.0]], [[5.0, 5.0, 4.0], [7.0, 5.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 3.0, 7.0], [6.0, 4.0, 4.0], 
[1.0, 6.0, 1.0]]], [[[7.0, 3.0, 6.0], [7.0, 8.0, 4.0], [2.0, 5.0, 8.0]], [[1.0, 2.0, 3.0], [6.0, 7.0, 3.0], [5.0, 6.0, 2.0]], 
[[2.0, 8.0, 2.0], [5.0, 7.0, 5.0], [7.0, 5.0, 6.0]]], [[[7.0, 1.0, 1.0], [4.0, 3.0, 3.0], [4.0, 4.0, 1.0]], [[5.0, 4.0, 6.0], 
[4.0, 6.0, 3.0], [7.0, 7.0, 5.0]], [[1.0, 3.0, 1.0], [6.0, 7.0, 2.0], [2.0, 1.0, 3.0]]]])
      arg[:,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 6.0], [5.0, 8.0, 2.0], [2.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], [6.0, 6.0, 3.0], [6.0, 
7.0, 5.0]], [[5.0, 4.0, 8.0], [1.0, 4.0, 1.0], [6.0, 8.0, 8.0]]], [[[1.0, 8.0, 1.0], [1.0, 3.0, 8.0], [4.0, 2.0, 3.0]], [[3.0, 
5.0, 5.0], [3.0, 6.0, 2.0], [1.0, 7.0, 7.0]], [[5.0, 6.0, 1.0], [3.0, 3.0, 3.0], [3.0, 6.0, 4.0]]], [[[7.0, 7.0, 8.0], [1.0, 
6.0, 1.0], [1.0, 4.0, 4.0]], [[6.0, 7.0, 3.0], [2.0, 3.0, 7.0], [1.0, 3.0, 3.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 4.0], [3.0, 5.0, 
6.0]]], [[[7.0, 6.0, 5.0], [1.0, 4.0, 3.0], [2.0, 5.0, 5.0]], [[7.0, 3.0, 3.0], [3.0, 8.0, 7.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 
4.0], [2.0, 6.0, 1.0], [1.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 3.0], [7.0, 1.0, 6.0], [8.0, 8.0, 3.0]], 
[[4.0, 5.0, 3.0], [4.0, 7.0, 3.0], [3.0, 7.0, 6.0]], [[4.0, 4.0, 4.0], [1.0, 8.0, 7.0], [1.0, 1.0, 1.0]]], [[[5.0, 4.0, 8.0], 
[2.0, 2.0, 8.0], [2.0, 3.0, 3.0]], [[5.0, 5.0, 4.0], [7.0, 5.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 3.0, 7.0], [6.0, 4.0, 4.0], 
[1.0, 6.0, 1.0]]], [[[7.0, 3.0, 6.0], [7.0, 8.0, 4.0], [2.0, 5.0, 8.0]], [[1.0, 2.0, 3.0], [6.0, 7.0, 3.0], [5.0, 6.0, 2.0]], 
[[2.0, 8.0, 2.0], [5.0, 7.0, 5.0], [7.0, 5.0, 6.0]]], [[[7.0, 1.0, 1.0], [4.0, 3.0, 3.0], [4.0, 4.0, 1.0]], [[5.0, 4.0, 6.0], 
[4.0, 6.0, 3.0], [7.0, 7.0, 5.0]], [[1.0, 3.0, 1.0], [6.0, 7.0, 2.0], [2.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CULN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 1.0], [8.0, 3.0, 1.0], [7.0, 2.0, 3.0]], [[8.0, 1.0, 5.0], [4.0, 2.0, 3.0], [4.0, 2.0, 
2.0]], [[5.0, 4.0, 6.0], [6.0, 4.0, 7.0], [8.0, 5.0, 1.0]]], [[[7.0, 1.0, 8.0], [6.0, 8.0, 3.0], [4.0, 1.0, 4.0]], [[1.0, 8.0, 
7.0], [4.0, 7.0, 6.0], [5.0, 4.0, 7.0]], [[5.0, 4.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], [4.0, 8.0, 
6.0], [1.0, 4.0, 8.0]], [[2.0, 8.0, 4.0], [3.0, 2.0, 6.0], [6.0, 7.0, 8.0]], [[1.0, 3.0, 8.0], [5.0, 7.0, 7.0], [5.0, 1.0, 
1.0]]], [[[1.0, 4.0, 5.0], [6.0, 6.0, 2.0], [6.0, 4.0, 6.0]], [[4.0, 1.0, 1.0], [1.0, 1.0, 3.0], [2.0, 7.0, 3.0]], [[7.0, 1.0, 
2.0], [8.0, 1.0, 8.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 6.0, 2.0]], [[8.0, 5.0, 8.0]]], [[[5.0, 2.0, 3.0]], [[3.0, 5.0, 6.0]]], [[[1.0, 4.0, 2.0]], 
[[6.0, 4.0, 4.0]]], [[[7.0, 7.0, 3.0]], [[4.0, 3.0, 6.0]]]])
      arg[:,:2,2:]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 1.0], [8.0, 3.0, 1.0], [8.0, 6.0, 2.0]], [[8.0, 1.0, 5.0], [4.0, 2.0, 3.0], [8.0, 5.0, 
8.0]], [[5.0, 4.0, 6.0], [6.0, 4.0, 7.0], [8.0, 5.0, 1.0]]], [[[7.0, 1.0, 8.0], [6.0, 8.0, 3.0], [5.0, 2.0, 3.0]], [[1.0, 8.0, 
7.0], [4.0, 7.0, 6.0], [3.0, 5.0, 6.0]], [[5.0, 4.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 1.0]]], [[[4.0, 6.0, 7.0], [4.0, 8.0, 
6.0], [1.0, 4.0, 2.0]], [[2.0, 8.0, 4.0], [3.0, 2.0, 6.0], [6.0, 4.0, 4.0]], [[1.0, 3.0, 8.0], [5.0, 7.0, 7.0], [5.0, 1.0, 
1.0]]], [[[1.0, 4.0, 5.0], [6.0, 6.0, 2.0], [7.0, 7.0, 3.0]], [[4.0, 1.0, 1.0], [1.0, 1.0, 3.0], [4.0, 3.0, 6.0]], [[7.0, 1.0, 
2.0], [8.0, 1.0, 8.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CULN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 8.0], [2.0, 8.0, 8.0], [6.0, 6.0, 7.0]], [[7.0, 5.0, 7.0], [1.0, 7.0, 8.0], [8.0, 6.0, 
6.0]], [[3.0, 3.0, 8.0], [7.0, 1.0, 5.0], [8.0, 4.0, 8.0]]], [[[8.0, 3.0, 4.0], [4.0, 5.0, 6.0], [4.0, 6.0, 8.0]], [[5.0, 6.0, 
1.0], [7.0, 5.0, 8.0], [3.0, 7.0, 8.0]], [[3.0, 2.0, 3.0], [4.0, 8.0, 3.0], [6.0, 1.0, 4.0]]], [[[8.0, 5.0, 1.0], [7.0, 4.0, 
8.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 6.0], [2.0, 1.0, 7.0], [1.0, 8.0, 7.0]], [[1.0, 7.0, 2.0], [3.0, 3.0, 5.0], [5.0, 7.0, 
6.0]]], [[[6.0, 7.0, 4.0], [5.0, 2.0, 4.0], [4.0, 4.0, 5.0]], [[4.0, 1.0, 7.0], [3.0, 3.0, 6.0], [3.0, 2.0, 8.0]], [[6.0, 8.0, 
2.0], [3.0, 7.0, 1.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 7.0, 2.0]], [[1.0, 5.0, 2.0]]], [[[6.0, 2.0, 5.0]], [[8.0, 8.0, 4.0]]], [[[3.0, 5.0, 3.0]], 
[[1.0, 5.0, 6.0]]], [[[3.0, 4.0, 7.0]], [[3.0, 5.0, 3.0]]]]),self.functionspace)
      arg[:,:2,2:]=val
      ref=Data(numpy.array([[[[6.0, 2.0, 8.0], [2.0, 8.0, 8.0], [6.0, 7.0, 2.0]], [[7.0, 5.0, 7.0], [1.0, 7.0, 8.0], [1.0, 5.0, 
2.0]], [[3.0, 3.0, 8.0], [7.0, 1.0, 5.0], [8.0, 4.0, 8.0]]], [[[8.0, 3.0, 4.0], [4.0, 5.0, 6.0], [6.0, 2.0, 5.0]], [[5.0, 6.0, 
1.0], [7.0, 5.0, 8.0], [8.0, 8.0, 4.0]], [[3.0, 2.0, 3.0], [4.0, 8.0, 3.0], [6.0, 1.0, 4.0]]], [[[8.0, 5.0, 1.0], [7.0, 4.0, 
8.0], [3.0, 5.0, 3.0]], [[7.0, 2.0, 6.0], [2.0, 1.0, 7.0], [1.0, 5.0, 6.0]], [[1.0, 7.0, 2.0], [3.0, 3.0, 5.0], [5.0, 7.0, 
6.0]]], [[[6.0, 7.0, 4.0], [5.0, 2.0, 4.0], [3.0, 4.0, 7.0]], [[4.0, 1.0, 7.0], [3.0, 3.0, 6.0], [3.0, 5.0, 3.0]], [[6.0, 8.0, 
2.0], [3.0, 7.0, 1.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CULN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 8.0], [1.0, 2.0, 3.0], [6.0, 4.0, 3.0]], [[8.0, 5.0, 7.0], [3.0, 7.0, 4.0], [6.0, 2.0, 
5.0]], [[4.0, 5.0, 4.0], [7.0, 4.0, 1.0], [8.0, 8.0, 3.0]]], [[[8.0, 6.0, 5.0], [8.0, 2.0, 5.0], [3.0, 8.0, 3.0]], [[3.0, 1.0, 
6.0], [6.0, 6.0, 2.0], [1.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], [3.0, 6.0, 5.0], [2.0, 8.0, 7.0]]], [[[6.0, 1.0, 1.0], [2.0, 8.0, 
2.0], [4.0, 3.0, 1.0]], [[8.0, 1.0, 2.0], [1.0, 6.0, 4.0], [4.0, 8.0, 4.0]], [[1.0, 5.0, 1.0], [6.0, 5.0, 8.0], [4.0, 5.0, 
2.0]]], [[[3.0, 3.0, 4.0], [2.0, 1.0, 5.0], [6.0, 7.0, 7.0]], [[5.0, 2.0, 8.0], [5.0, 4.0, 7.0], [4.0, 3.0, 7.0]], [[4.0, 3.0, 
3.0], [3.0, 4.0, 5.0], [4.0, 5.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 1.0, 1.0]], [[2.0, 2.0, 7.0]]], [[[8.0, 6.0, 5.0]], [[5.0, 8.0, 6.0]]], [[[5.0, 2.0, 
3.0]], [[4.0, 7.0, 3.0]]], [[[7.0, 5.0, 5.0]], [[7.0, 4.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 8.0, 5.0]], [[6.0, 3.0, 
6.0]]], [[[5.0, 8.0, 1.0]], [[2.0, 7.0, 1.0]]], [[[7.0, 8.0, 4.0]], [[1.0, 4.0, 7.0]]], [[[1.0, 7.0, 4.0]], [[4.0, 5.0, 5.0]]]])
      arg[:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [1.0, 2.0, 3.0], [2.0, 1.0, 1.0]], [[8.0, 5.0, 7.0], [3.0, 7.0, 4.0], [2.0, 
2.0, 7.0]], [[4.0, 5.0, 4.0], [7.0, 4.0, 1.0], [8.0, 8.0, 3.0]]], [[[8.0, 6.0, 5.0], [8.0, 2.0, 5.0], [8.0, 6.0, 5.0]], [[3.0, 
1.0, 6.0], [6.0, 6.0, 2.0], [5.0, 8.0, 6.0]], [[8.0, 6.0, 7.0], [3.0, 6.0, 5.0], [2.0, 8.0, 7.0]]], [[[6.0, 1.0, 1.0], [2.0, 
8.0, 2.0], [5.0, 2.0, 3.0]], [[8.0, 1.0, 2.0], [1.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[1.0, 5.0, 1.0], [6.0, 5.0, 8.0], [4.0, 5.0, 
2.0]]], [[[3.0, 3.0, 4.0], [2.0, 1.0, 5.0], [7.0, 5.0, 5.0]], [[5.0, 2.0, 8.0], [5.0, 4.0, 7.0], [7.0, 4.0, 7.0]], [[4.0, 3.0, 
3.0], [3.0, 4.0, 5.0], [4.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 8.0], [1.0, 2.0, 3.0], [1.0, 8.0, 5.0]], 
[[8.0, 5.0, 7.0], [3.0, 7.0, 4.0], [6.0, 3.0, 6.0]], [[4.0, 5.0, 4.0], [7.0, 4.0, 1.0], [8.0, 8.0, 3.0]]], [[[8.0, 6.0, 5.0], 
[8.0, 2.0, 5.0], [5.0, 8.0, 1.0]], [[3.0, 1.0, 6.0], [6.0, 6.0, 2.0], [2.0, 7.0, 1.0]], [[8.0, 6.0, 7.0], [3.0, 6.0, 5.0], 
[2.0, 8.0, 7.0]]], [[[6.0, 1.0, 1.0], [2.0, 8.0, 2.0], [7.0, 8.0, 4.0]], [[8.0, 1.0, 2.0], [1.0, 6.0, 4.0], [1.0, 4.0, 7.0]], 
[[1.0, 5.0, 1.0], [6.0, 5.0, 8.0], [4.0, 5.0, 2.0]]], [[[3.0, 3.0, 4.0], [2.0, 1.0, 5.0], [1.0, 7.0, 4.0]], [[5.0, 2.0, 8.0], 
[5.0, 4.0, 7.0], [4.0, 5.0, 5.0]], [[4.0, 3.0, 3.0], [3.0, 4.0, 5.0], [4.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLUN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 3.0], [5.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 2.0, 4.0], [6.0, 5.0, 7.0], [6.0, 5.0, 
2.0]], [[5.0, 2.0, 1.0], [3.0, 7.0, 1.0], [3.0, 3.0, 8.0]]], [[[6.0, 6.0, 6.0], [2.0, 5.0, 4.0], [8.0, 8.0, 5.0]], [[5.0, 3.0, 
6.0], [7.0, 8.0, 7.0], [2.0, 5.0, 7.0]], [[1.0, 5.0, 4.0], [3.0, 2.0, 5.0], [5.0, 2.0, 1.0]]], [[[3.0, 6.0, 2.0], [4.0, 5.0, 
4.0], [5.0, 2.0, 6.0]], [[8.0, 8.0, 7.0], [4.0, 5.0, 4.0], [8.0, 3.0, 1.0]], [[8.0, 8.0, 4.0], [5.0, 8.0, 3.0], [7.0, 4.0, 
1.0]]], [[[6.0, 3.0, 7.0], [5.0, 8.0, 7.0], [1.0, 4.0, 1.0]], [[7.0, 2.0, 2.0], [3.0, 7.0, 2.0], [5.0, 7.0, 4.0]], [[3.0, 5.0, 
6.0], [6.0, 1.0, 5.0], [4.0, 3.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 7.0, 4.0], [5.0, 4.0, 5.0]]])
      arg[2,2:,:2]=val
      ref=Data(numpy.array([[[[1.0, 7.0, 3.0], [5.0, 3.0, 1.0], [8.0, 1.0, 1.0]], [[4.0, 2.0, 4.0], [6.0, 5.0, 7.0], [6.0, 5.0, 
2.0]], [[5.0, 2.0, 1.0], [3.0, 7.0, 1.0], [3.0, 3.0, 8.0]]], [[[6.0, 6.0, 6.0], [2.0, 5.0, 4.0], [8.0, 8.0, 5.0]], [[5.0, 3.0, 
6.0], [7.0, 8.0, 7.0], [2.0, 5.0, 7.0]], [[1.0, 5.0, 4.0], [3.0, 2.0, 5.0], [5.0, 2.0, 1.0]]], [[[3.0, 6.0, 2.0], [4.0, 5.0, 
4.0], [5.0, 2.0, 6.0]], [[8.0, 8.0, 7.0], [4.0, 5.0, 4.0], [8.0, 3.0, 1.0]], [[1.0, 7.0, 4.0], [5.0, 4.0, 5.0], [7.0, 4.0, 
1.0]]], [[[6.0, 3.0, 7.0], [5.0, 8.0, 7.0], [1.0, 4.0, 1.0]], [[7.0, 2.0, 2.0], [3.0, 7.0, 2.0], [5.0, 7.0, 4.0]], [[3.0, 5.0, 
6.0], [6.0, 1.0, 5.0], [4.0, 3.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLUN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 8.0], [8.0, 8.0, 6.0], [6.0, 8.0, 2.0]], [[2.0, 5.0, 3.0], [5.0, 3.0, 5.0], [6.0, 5.0, 
1.0]], [[7.0, 6.0, 5.0], [5.0, 4.0, 5.0], [7.0, 3.0, 2.0]]], [[[4.0, 5.0, 8.0], [6.0, 5.0, 8.0], [4.0, 3.0, 7.0]], [[6.0, 2.0, 
5.0], [4.0, 7.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 8.0, 1.0], [3.0, 1.0, 5.0], [5.0, 4.0, 6.0]]], [[[8.0, 4.0, 7.0], [4.0, 2.0, 
8.0], [8.0, 8.0, 3.0]], [[1.0, 6.0, 2.0], [6.0, 7.0, 7.0], [6.0, 1.0, 3.0]], [[3.0, 8.0, 5.0], [5.0, 7.0, 5.0], [1.0, 8.0, 
5.0]]], [[[6.0, 8.0, 4.0], [2.0, 2.0, 1.0], [3.0, 3.0, 3.0]], [[2.0, 2.0, 8.0], [7.0, 3.0, 4.0], [6.0, 4.0, 8.0]], [[6.0, 4.0, 
7.0], [3.0, 3.0, 8.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 1.0, 8.0], [6.0, 7.0, 6.0]]]),self.functionspace)
      arg[2,2:,:2]=val
      ref=Data(numpy.array([[[[1.0, 3.0, 8.0], [8.0, 8.0, 6.0], [6.0, 8.0, 2.0]], [[2.0, 5.0, 3.0], [5.0, 3.0, 5.0], [6.0, 5.0, 
1.0]], [[7.0, 6.0, 5.0], [5.0, 4.0, 5.0], [7.0, 3.0, 2.0]]], [[[4.0, 5.0, 8.0], [6.0, 5.0, 8.0], [4.0, 3.0, 7.0]], [[6.0, 2.0, 
5.0], [4.0, 7.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 8.0, 1.0], [3.0, 1.0, 5.0], [5.0, 4.0, 6.0]]], [[[8.0, 4.0, 7.0], [4.0, 2.0, 
8.0], [8.0, 8.0, 3.0]], [[1.0, 6.0, 2.0], [6.0, 7.0, 7.0], [6.0, 1.0, 3.0]], [[7.0, 1.0, 8.0], [6.0, 7.0, 6.0], [1.0, 8.0, 
5.0]]], [[[6.0, 8.0, 4.0], [2.0, 2.0, 1.0], [3.0, 3.0, 3.0]], [[2.0, 2.0, 8.0], [7.0, 3.0, 4.0], [6.0, 4.0, 8.0]], [[6.0, 4.0, 
7.0], [3.0, 3.0, 8.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLUN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 1.0], [2.0, 5.0, 2.0], [4.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [2.0, 2.0, 1.0], [2.0, 5.0, 
5.0]], [[6.0, 4.0, 2.0], [2.0, 1.0, 7.0], [4.0, 2.0, 7.0]]], [[[2.0, 7.0, 2.0], [6.0, 4.0, 4.0], [3.0, 8.0, 2.0]], [[7.0, 3.0, 
5.0], [7.0, 1.0, 3.0], [1.0, 1.0, 4.0]], [[4.0, 4.0, 4.0], [1.0, 6.0, 7.0], [1.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 8.0, 
6.0], [7.0, 3.0, 1.0]], [[3.0, 7.0, 6.0], [3.0, 7.0, 8.0], [8.0, 1.0, 4.0]], [[2.0, 8.0, 4.0], [7.0, 1.0, 5.0], [2.0, 2.0, 
4.0]]], [[[1.0, 3.0, 4.0], [5.0, 5.0, 2.0], [1.0, 5.0, 8.0]], [[2.0, 1.0, 2.0], [1.0, 1.0, 1.0], [4.0, 4.0, 1.0]], [[1.0, 5.0, 
8.0], [3.0, 1.0, 7.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 5.0, 7.0], [4.0, 1.0, 1.0]]])+(1.-msk_val)*numpy.array([[[8.0, 8.0, 4.0], [7.0, 3.0, 
6.0]]])
      arg[2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 1.0], [2.0, 5.0, 2.0], [4.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [2.0, 2.0, 1.0], [2.0, 
5.0, 5.0]], [[6.0, 4.0, 2.0], [2.0, 1.0, 7.0], [4.0, 2.0, 7.0]]], [[[2.0, 7.0, 2.0], [6.0, 4.0, 4.0], [3.0, 8.0, 2.0]], [[7.0, 
3.0, 5.0], [7.0, 1.0, 3.0], [1.0, 1.0, 4.0]], [[4.0, 4.0, 4.0], [1.0, 6.0, 7.0], [1.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 
8.0, 6.0], [7.0, 3.0, 1.0]], [[3.0, 7.0, 6.0], [3.0, 7.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 5.0, 7.0], [4.0, 1.0, 1.0], [2.0, 2.0, 
4.0]]], [[[1.0, 3.0, 4.0], [5.0, 5.0, 2.0], [1.0, 5.0, 8.0]], [[2.0, 1.0, 2.0], [1.0, 1.0, 1.0], [4.0, 4.0, 1.0]], [[1.0, 5.0, 
8.0], [3.0, 1.0, 7.0], [8.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 1.0], [2.0, 5.0, 2.0], [4.0, 5.0, 8.0]], 
[[4.0, 2.0, 2.0], [2.0, 2.0, 1.0], [2.0, 5.0, 5.0]], [[6.0, 4.0, 2.0], [2.0, 1.0, 7.0], [4.0, 2.0, 7.0]]], [[[2.0, 7.0, 2.0], 
[6.0, 4.0, 4.0], [3.0, 8.0, 2.0]], [[7.0, 3.0, 5.0], [7.0, 1.0, 3.0], [1.0, 1.0, 4.0]], [[4.0, 4.0, 4.0], [1.0, 6.0, 7.0], 
[1.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 8.0, 6.0], [7.0, 3.0, 1.0]], [[3.0, 7.0, 6.0], [3.0, 7.0, 8.0], [8.0, 1.0, 4.0]], 
[[8.0, 8.0, 4.0], [7.0, 3.0, 6.0], [2.0, 2.0, 4.0]]], [[[1.0, 3.0, 4.0], [5.0, 5.0, 2.0], [1.0, 5.0, 8.0]], [[2.0, 1.0, 2.0], 
[1.0, 1.0, 1.0], [4.0, 4.0, 1.0]], [[1.0, 5.0, 8.0], [3.0, 1.0, 7.0], [8.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CPLN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 1.0], [3.0, 4.0, 5.0], [5.0, 8.0, 2.0]], [[5.0, 5.0, 5.0], [2.0, 3.0, 2.0], [5.0, 7.0, 
4.0]], [[3.0, 1.0, 6.0], [6.0, 8.0, 7.0], [5.0, 6.0, 2.0]]], [[[8.0, 3.0, 7.0], [2.0, 8.0, 6.0], [5.0, 6.0, 2.0]], [[5.0, 1.0, 
2.0], [3.0, 4.0, 4.0], [8.0, 6.0, 3.0]], [[5.0, 4.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 4.0]]], [[[2.0, 7.0, 2.0], [8.0, 1.0, 
8.0], [6.0, 4.0, 3.0]], [[2.0, 4.0, 2.0], [8.0, 2.0, 4.0], [7.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [4.0, 4.0, 6.0], [7.0, 3.0, 
6.0]]], [[[3.0, 6.0, 7.0], [6.0, 5.0, 6.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 7.0], [5.0, 1.0, 3.0]], [[8.0, 6.0, 
8.0], [3.0, 4.0, 3.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 7.0, 4.0]], [[7.0, 6.0, 6.0]], [[6.0, 5.0, 8.0]], [[7.0, 4.0, 5.0]]])
      arg[:,1,2:]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 1.0], [3.0, 4.0, 5.0], [5.0, 8.0, 2.0]], [[5.0, 5.0, 5.0], [2.0, 3.0, 2.0], [1.0, 7.0, 
4.0]], [[3.0, 1.0, 6.0], [6.0, 8.0, 7.0], [5.0, 6.0, 2.0]]], [[[8.0, 3.0, 7.0], [2.0, 8.0, 6.0], [5.0, 6.0, 2.0]], [[5.0, 1.0, 
2.0], [3.0, 4.0, 4.0], [7.0, 6.0, 6.0]], [[5.0, 4.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 4.0]]], [[[2.0, 7.0, 2.0], [8.0, 1.0, 
8.0], [6.0, 4.0, 3.0]], [[2.0, 4.0, 2.0], [8.0, 2.0, 4.0], [6.0, 5.0, 8.0]], [[8.0, 6.0, 7.0], [4.0, 4.0, 6.0], [7.0, 3.0, 
6.0]]], [[[3.0, 6.0, 7.0], [6.0, 5.0, 6.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [6.0, 2.0, 7.0], [7.0, 4.0, 5.0]], [[8.0, 6.0, 
8.0], [3.0, 4.0, 3.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CPLN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 3.0], [5.0, 6.0, 2.0], [1.0, 8.0, 1.0]], [[5.0, 7.0, 4.0], [5.0, 7.0, 8.0], [6.0, 7.0, 
5.0]], [[4.0, 5.0, 3.0], [4.0, 3.0, 6.0], [5.0, 2.0, 8.0]]], [[[6.0, 8.0, 5.0], [1.0, 7.0, 5.0], [4.0, 3.0, 8.0]], [[7.0, 1.0, 
5.0], [4.0, 4.0, 4.0], [8.0, 2.0, 2.0]], [[5.0, 1.0, 8.0], [2.0, 6.0, 7.0], [7.0, 6.0, 7.0]]], [[[3.0, 6.0, 6.0], [6.0, 4.0, 
5.0], [4.0, 4.0, 1.0]], [[3.0, 3.0, 1.0], [1.0, 7.0, 2.0], [5.0, 8.0, 4.0]], [[5.0, 4.0, 4.0], [6.0, 8.0, 3.0], [8.0, 7.0, 
8.0]]], [[[3.0, 6.0, 4.0], [4.0, 5.0, 5.0], [4.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [6.0, 3.0, 4.0], [5.0, 7.0, 2.0]], [[3.0, 6.0, 
3.0], [1.0, 7.0, 2.0], [6.0, 5.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 6.0, 4.0]], [[5.0, 1.0, 6.0]], [[6.0, 2.0, 7.0]], [[4.0, 2.0, 8.0]]]),self.functionspace)
      arg[:,1,2:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 3.0], [5.0, 6.0, 2.0], [1.0, 8.0, 1.0]], [[5.0, 7.0, 4.0], [5.0, 7.0, 8.0], [7.0, 6.0, 
4.0]], [[4.0, 5.0, 3.0], [4.0, 3.0, 6.0], [5.0, 2.0, 8.0]]], [[[6.0, 8.0, 5.0], [1.0, 7.0, 5.0], [4.0, 3.0, 8.0]], [[7.0, 1.0, 
5.0], [4.0, 4.0, 4.0], [5.0, 1.0, 6.0]], [[5.0, 1.0, 8.0], [2.0, 6.0, 7.0], [7.0, 6.0, 7.0]]], [[[3.0, 6.0, 6.0], [6.0, 4.0, 
5.0], [4.0, 4.0, 1.0]], [[3.0, 3.0, 1.0], [1.0, 7.0, 2.0], [6.0, 2.0, 7.0]], [[5.0, 4.0, 4.0], [6.0, 8.0, 3.0], [8.0, 7.0, 
8.0]]], [[[3.0, 6.0, 4.0], [4.0, 5.0, 5.0], [4.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [6.0, 3.0, 4.0], [4.0, 2.0, 8.0]], [[3.0, 6.0, 
3.0], [1.0, 7.0, 2.0], [6.0, 5.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CPLN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 5.0], [8.0, 5.0, 2.0], [1.0, 8.0, 2.0]], [[2.0, 7.0, 6.0], [7.0, 1.0, 4.0], [2.0, 2.0, 
1.0]], [[4.0, 3.0, 5.0], [7.0, 6.0, 6.0], [5.0, 7.0, 1.0]]], [[[8.0, 8.0, 2.0], [6.0, 4.0, 3.0], [3.0, 2.0, 8.0]], [[7.0, 2.0, 
1.0], [6.0, 6.0, 4.0], [5.0, 2.0, 5.0]], [[2.0, 4.0, 4.0], [7.0, 2.0, 2.0], [3.0, 5.0, 8.0]]], [[[2.0, 2.0, 3.0], [4.0, 4.0, 
3.0], [3.0, 3.0, 6.0]], [[6.0, 5.0, 6.0], [4.0, 5.0, 7.0], [5.0, 7.0, 6.0]], [[4.0, 2.0, 8.0], [5.0, 2.0, 6.0], [3.0, 7.0, 
3.0]]], [[[2.0, 6.0, 3.0], [3.0, 8.0, 8.0], [7.0, 3.0, 6.0]], [[8.0, 8.0, 3.0], [5.0, 4.0, 1.0], [5.0, 2.0, 4.0]], [[8.0, 3.0, 
1.0], [6.0, 5.0, 6.0], [1.0, 8.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 4.0, 8.0]], [[1.0, 5.0, 6.0]], [[3.0, 4.0, 8.0]], [[4.0, 6.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[2.0, 1.0, 2.0]], [[1.0, 8.0, 3.0]], [[5.0, 3.0, 8.0]], [[5.0, 7.0, 6.0]]])
      arg[:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 5.0], [8.0, 5.0, 2.0], [1.0, 8.0, 2.0]], [[2.0, 7.0, 6.0], [7.0, 1.0, 4.0], [6.0, 
4.0, 8.0]], [[4.0, 3.0, 5.0], [7.0, 6.0, 6.0], [5.0, 7.0, 1.0]]], [[[8.0, 8.0, 2.0], [6.0, 4.0, 3.0], [3.0, 2.0, 8.0]], [[7.0, 
2.0, 1.0], [6.0, 6.0, 4.0], [1.0, 5.0, 6.0]], [[2.0, 4.0, 4.0], [7.0, 2.0, 2.0], [3.0, 5.0, 8.0]]], [[[2.0, 2.0, 3.0], [4.0, 
4.0, 3.0], [3.0, 3.0, 6.0]], [[6.0, 5.0, 6.0], [4.0, 5.0, 7.0], [3.0, 4.0, 8.0]], [[4.0, 2.0, 8.0], [5.0, 2.0, 6.0], [3.0, 7.0, 
3.0]]], [[[2.0, 6.0, 3.0], [3.0, 8.0, 8.0], [7.0, 3.0, 6.0]], [[8.0, 8.0, 3.0], [5.0, 4.0, 1.0], [4.0, 6.0, 4.0]], [[8.0, 3.0, 
1.0], [6.0, 5.0, 6.0], [1.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [8.0, 5.0, 2.0], [1.0, 8.0, 2.0]], 
[[2.0, 7.0, 6.0], [7.0, 1.0, 4.0], [2.0, 1.0, 2.0]], [[4.0, 3.0, 5.0], [7.0, 6.0, 6.0], [5.0, 7.0, 1.0]]], [[[8.0, 8.0, 2.0], 
[6.0, 4.0, 3.0], [3.0, 2.0, 8.0]], [[7.0, 2.0, 1.0], [6.0, 6.0, 4.0], [1.0, 8.0, 3.0]], [[2.0, 4.0, 4.0], [7.0, 2.0, 2.0], 
[3.0, 5.0, 8.0]]], [[[2.0, 2.0, 3.0], [4.0, 4.0, 3.0], [3.0, 3.0, 6.0]], [[6.0, 5.0, 6.0], [4.0, 5.0, 7.0], [5.0, 3.0, 8.0]], 
[[4.0, 2.0, 8.0], [5.0, 2.0, 6.0], [3.0, 7.0, 3.0]]], [[[2.0, 6.0, 3.0], [3.0, 8.0, 8.0], [7.0, 3.0, 6.0]], [[8.0, 8.0, 3.0], 
[5.0, 4.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 3.0, 1.0], [6.0, 5.0, 6.0], [1.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRCN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 6.0], [6.0, 3.0, 3.0], [5.0, 3.0, 8.0]], [[6.0, 5.0, 6.0], [8.0, 7.0, 8.0], [8.0, 3.0, 
8.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 4.0], [2.0, 1.0, 4.0]]], [[[5.0, 7.0, 8.0], [8.0, 4.0, 6.0], [2.0, 2.0, 5.0]], [[7.0, 6.0, 
2.0], [5.0, 3.0, 5.0], [6.0, 6.0, 2.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 2.0], [2.0, 6.0, 8.0]]], [[[2.0, 2.0, 8.0], [1.0, 8.0, 
8.0], [3.0, 6.0, 8.0]], [[8.0, 2.0, 5.0], [7.0, 1.0, 8.0], [4.0, 5.0, 8.0]], [[2.0, 5.0, 7.0], [2.0, 3.0, 5.0], [8.0, 4.0, 
5.0]]], [[[8.0, 3.0, 2.0], [5.0, 7.0, 4.0], [6.0, 2.0, 4.0]], [[1.0, 3.0, 6.0], [4.0, 1.0, 4.0], [5.0, 5.0, 7.0]], [[6.0, 1.0, 
8.0], [3.0, 4.0, 3.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 2.0, 2.0], [3.0, 8.0, 8.0], [8.0, 8.0, 2.0]]], [[[5.0, 1.0, 1.0], [3.0, 6.0, 7.0], [7.0, 6.0, 
8.0]]]])
      arg[1:3,1:2,:]=val
      ref=Data(numpy.array([[[[2.0, 1.0, 6.0], [6.0, 3.0, 3.0], [5.0, 3.0, 8.0]], [[6.0, 5.0, 6.0], [8.0, 7.0, 8.0], [8.0, 3.0, 
8.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 4.0], [2.0, 1.0, 4.0]]], [[[5.0, 7.0, 8.0], [8.0, 4.0, 6.0], [2.0, 2.0, 5.0]], [[3.0, 2.0, 
2.0], [3.0, 8.0, 8.0], [8.0, 8.0, 2.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 2.0], [2.0, 6.0, 8.0]]], [[[2.0, 2.0, 8.0], [1.0, 8.0, 
8.0], [3.0, 6.0, 8.0]], [[5.0, 1.0, 1.0], [3.0, 6.0, 7.0], [7.0, 6.0, 8.0]], [[2.0, 5.0, 7.0], [2.0, 3.0, 5.0], [8.0, 4.0, 
5.0]]], [[[8.0, 3.0, 2.0], [5.0, 7.0, 4.0], [6.0, 2.0, 4.0]], [[1.0, 3.0, 6.0], [4.0, 1.0, 4.0], [5.0, 5.0, 7.0]], [[6.0, 1.0, 
8.0], [3.0, 4.0, 3.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRCN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 4.0], [5.0, 1.0, 4.0], [7.0, 3.0, 2.0]], [[7.0, 4.0, 2.0], [8.0, 2.0, 3.0], [8.0, 7.0, 
3.0]], [[7.0, 6.0, 2.0], [7.0, 5.0, 6.0], [2.0, 7.0, 5.0]]], [[[3.0, 7.0, 2.0], [5.0, 7.0, 8.0], [6.0, 5.0, 5.0]], [[5.0, 1.0, 
4.0], [5.0, 8.0, 8.0], [7.0, 8.0, 3.0]], [[2.0, 5.0, 8.0], [1.0, 8.0, 4.0], [6.0, 8.0, 2.0]]], [[[1.0, 5.0, 7.0], [8.0, 3.0, 
2.0], [3.0, 4.0, 6.0]], [[5.0, 6.0, 3.0], [7.0, 5.0, 2.0], [4.0, 8.0, 3.0]], [[6.0, 8.0, 7.0], [8.0, 5.0, 8.0], [6.0, 2.0, 
8.0]]], [[[6.0, 2.0, 8.0], [6.0, 2.0, 1.0], [7.0, 5.0, 4.0]], [[1.0, 5.0, 1.0], [4.0, 5.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 1.0, 
8.0], [2.0, 3.0, 5.0], [3.0, 4.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 5.0, 4.0], [2.0, 7.0, 7.0], [2.0, 3.0, 7.0]]], [[[4.0, 4.0, 8.0], [1.0, 5.0, 7.0], [8.0, 
7.0, 7.0]]]]),self.functionspace)
      arg[1:3,1:2,:]=val
      ref=Data(numpy.array([[[[7.0, 6.0, 4.0], [5.0, 1.0, 4.0], [7.0, 3.0, 2.0]], [[7.0, 4.0, 2.0], [8.0, 2.0, 3.0], [8.0, 7.0, 
3.0]], [[7.0, 6.0, 2.0], [7.0, 5.0, 6.0], [2.0, 7.0, 5.0]]], [[[3.0, 7.0, 2.0], [5.0, 7.0, 8.0], [6.0, 5.0, 5.0]], [[2.0, 5.0, 
4.0], [2.0, 7.0, 7.0], [2.0, 3.0, 7.0]], [[2.0, 5.0, 8.0], [1.0, 8.0, 4.0], [6.0, 8.0, 2.0]]], [[[1.0, 5.0, 7.0], [8.0, 3.0, 
2.0], [3.0, 4.0, 6.0]], [[4.0, 4.0, 8.0], [1.0, 5.0, 7.0], [8.0, 7.0, 7.0]], [[6.0, 8.0, 7.0], [8.0, 5.0, 8.0], [6.0, 2.0, 
8.0]]], [[[6.0, 2.0, 8.0], [6.0, 2.0, 1.0], [7.0, 5.0, 4.0]], [[1.0, 5.0, 1.0], [4.0, 5.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 1.0, 
8.0], [2.0, 3.0, 5.0], [3.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRCN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 3.0], [7.0, 2.0, 7.0], [6.0, 7.0, 4.0]], [[8.0, 1.0, 8.0], [7.0, 7.0, 2.0], [5.0, 5.0, 
1.0]], [[2.0, 1.0, 7.0], [5.0, 5.0, 5.0], [1.0, 6.0, 1.0]]], [[[2.0, 4.0, 6.0], [8.0, 4.0, 7.0], [6.0, 5.0, 6.0]], [[1.0, 7.0, 
8.0], [5.0, 5.0, 6.0], [8.0, 1.0, 2.0]], [[2.0, 1.0, 4.0], [4.0, 8.0, 7.0], [7.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [8.0, 5.0, 
3.0], [8.0, 1.0, 7.0]], [[7.0, 1.0, 8.0], [7.0, 4.0, 5.0], [5.0, 1.0, 7.0]], [[1.0, 6.0, 8.0], [6.0, 7.0, 5.0], [8.0, 5.0, 
2.0]]], [[[8.0, 4.0, 2.0], [8.0, 4.0, 8.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 8.0], [5.0, 5.0, 2.0], [2.0, 4.0, 3.0]], [[6.0, 3.0, 
5.0], [7.0, 8.0, 5.0], [5.0, 8.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 4.0, 2.0], [3.0, 6.0, 5.0], [1.0, 5.0, 4.0]]], [[[8.0, 4.0, 8.0], [1.0, 6.0, 8.0], [5.0, 
3.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 6.0, 2.0], [7.0, 6.0, 8.0], [2.0, 1.0, 8.0]]], [[[3.0, 3.0, 6.0], [4.0, 4.0, 
8.0], [8.0, 4.0, 7.0]]]])
      arg[1:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 3.0], [7.0, 2.0, 7.0], [6.0, 7.0, 4.0]], [[8.0, 1.0, 8.0], [7.0, 7.0, 2.0], [5.0, 
5.0, 1.0]], [[2.0, 1.0, 7.0], [5.0, 5.0, 5.0], [1.0, 6.0, 1.0]]], [[[2.0, 4.0, 6.0], [8.0, 4.0, 7.0], [6.0, 5.0, 6.0]], [[5.0, 
4.0, 2.0], [3.0, 6.0, 5.0], [1.0, 5.0, 4.0]], [[2.0, 1.0, 4.0], [4.0, 8.0, 7.0], [7.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [8.0, 
5.0, 3.0], [8.0, 1.0, 7.0]], [[8.0, 4.0, 8.0], [1.0, 6.0, 8.0], [5.0, 3.0, 6.0]], [[1.0, 6.0, 8.0], [6.0, 7.0, 5.0], [8.0, 5.0, 
2.0]]], [[[8.0, 4.0, 2.0], [8.0, 4.0, 8.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 8.0], [5.0, 5.0, 2.0], [2.0, 4.0, 3.0]], [[6.0, 3.0, 
5.0], [7.0, 8.0, 5.0], [5.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 3.0], [7.0, 2.0, 7.0], [6.0, 7.0, 4.0]], 
[[8.0, 1.0, 8.0], [7.0, 7.0, 2.0], [5.0, 5.0, 1.0]], [[2.0, 1.0, 7.0], [5.0, 5.0, 5.0], [1.0, 6.0, 1.0]]], [[[2.0, 4.0, 6.0], 
[8.0, 4.0, 7.0], [6.0, 5.0, 6.0]], [[3.0, 6.0, 2.0], [7.0, 6.0, 8.0], [2.0, 1.0, 8.0]], [[2.0, 1.0, 4.0], [4.0, 8.0, 7.0], 
[7.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [8.0, 5.0, 3.0], [8.0, 1.0, 7.0]], [[3.0, 3.0, 6.0], [4.0, 4.0, 8.0], [8.0, 4.0, 7.0]], 
[[1.0, 6.0, 8.0], [6.0, 7.0, 5.0], [8.0, 5.0, 2.0]]], [[[8.0, 4.0, 2.0], [8.0, 4.0, 8.0], [1.0, 3.0, 7.0]], [[5.0, 4.0, 8.0], 
[5.0, 5.0, 2.0], [2.0, 4.0, 3.0]], [[6.0, 3.0, 5.0], [7.0, 8.0, 5.0], [5.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 4.0], [7.0, 5.0, 7.0], [7.0, 2.0, 4.0]], [[6.0, 6.0, 3.0], [2.0, 5.0, 7.0], [6.0, 7.0, 
7.0]], [[6.0, 6.0, 6.0], [8.0, 4.0, 4.0], [6.0, 8.0, 8.0]]], [[[3.0, 3.0, 1.0], [1.0, 7.0, 4.0], [1.0, 4.0, 7.0]], [[4.0, 5.0, 
5.0], [1.0, 4.0, 2.0], [5.0, 3.0, 8.0]], [[8.0, 2.0, 5.0], [8.0, 2.0, 2.0], [3.0, 6.0, 8.0]]], [[[8.0, 5.0, 2.0], [7.0, 2.0, 
6.0], [8.0, 7.0, 7.0]], [[6.0, 3.0, 3.0], [2.0, 2.0, 5.0], [1.0, 5.0, 4.0]], [[6.0, 1.0, 1.0], [8.0, 4.0, 3.0], [7.0, 1.0, 
8.0]]], [[[4.0, 3.0, 8.0], [4.0, 1.0, 2.0], [7.0, 5.0, 4.0]], [[4.0, 8.0, 6.0], [2.0, 2.0, 6.0], [5.0, 6.0, 7.0]], [[2.0, 8.0, 
6.0], [8.0, 4.0, 1.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[5.0, 5.0, 8.0]])
      arg[2,1:2,1]=val
      ref=Data(numpy.array([[[[3.0, 5.0, 4.0], [7.0, 5.0, 7.0], [7.0, 2.0, 4.0]], [[6.0, 6.0, 3.0], [2.0, 5.0, 7.0], [6.0, 7.0, 
7.0]], [[6.0, 6.0, 6.0], [8.0, 4.0, 4.0], [6.0, 8.0, 8.0]]], [[[3.0, 3.0, 1.0], [1.0, 7.0, 4.0], [1.0, 4.0, 7.0]], [[4.0, 5.0, 
5.0], [1.0, 4.0, 2.0], [5.0, 3.0, 8.0]], [[8.0, 2.0, 5.0], [8.0, 2.0, 2.0], [3.0, 6.0, 8.0]]], [[[8.0, 5.0, 2.0], [7.0, 2.0, 
6.0], [8.0, 7.0, 7.0]], [[6.0, 3.0, 3.0], [5.0, 5.0, 8.0], [1.0, 5.0, 4.0]], [[6.0, 1.0, 1.0], [8.0, 4.0, 3.0], [7.0, 1.0, 
8.0]]], [[[4.0, 3.0, 8.0], [4.0, 1.0, 2.0], [7.0, 5.0, 4.0]], [[4.0, 8.0, 6.0], [2.0, 2.0, 6.0], [5.0, 6.0, 7.0]], [[2.0, 8.0, 
6.0], [8.0, 4.0, 1.0], [1.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRPN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 1.0], [7.0, 1.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 5.0, 3.0], [7.0, 6.0, 7.0], [7.0, 7.0, 
7.0]], [[6.0, 7.0, 5.0], [7.0, 5.0, 2.0], [4.0, 3.0, 3.0]]], [[[7.0, 1.0, 7.0], [4.0, 3.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 7.0, 
4.0], [6.0, 1.0, 1.0], [8.0, 8.0, 1.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 8.0], [7.0, 7.0, 4.0]]], [[[3.0, 4.0, 5.0], [8.0, 3.0, 
4.0], [2.0, 3.0, 1.0]], [[1.0, 8.0, 3.0], [6.0, 7.0, 4.0], [4.0, 8.0, 6.0]], [[8.0, 5.0, 6.0], [1.0, 6.0, 8.0], [3.0, 4.0, 
4.0]]], [[[3.0, 6.0, 3.0], [3.0, 1.0, 3.0], [8.0, 8.0, 2.0]], [[8.0, 5.0, 6.0], [3.0, 1.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 1.0, 
5.0], [5.0, 5.0, 4.0], [5.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[3.0, 4.0, 2.0]]),self.functionspace)
      arg[2,1:2,1]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 1.0], [7.0, 1.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 5.0, 3.0], [7.0, 6.0, 7.0], [7.0, 7.0, 
7.0]], [[6.0, 7.0, 5.0], [7.0, 5.0, 2.0], [4.0, 3.0, 3.0]]], [[[7.0, 1.0, 7.0], [4.0, 3.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 7.0, 
4.0], [6.0, 1.0, 1.0], [8.0, 8.0, 1.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 8.0], [7.0, 7.0, 4.0]]], [[[3.0, 4.0, 5.0], [8.0, 3.0, 
4.0], [2.0, 3.0, 1.0]], [[1.0, 8.0, 3.0], [3.0, 4.0, 2.0], [4.0, 8.0, 6.0]], [[8.0, 5.0, 6.0], [1.0, 6.0, 8.0], [3.0, 4.0, 
4.0]]], [[[3.0, 6.0, 3.0], [3.0, 1.0, 3.0], [8.0, 8.0, 2.0]], [[8.0, 5.0, 6.0], [3.0, 1.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 1.0, 
5.0], [5.0, 5.0, 4.0], [5.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRPN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [5.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [6.0, 7.0, 4.0], [5.0, 3.0, 
3.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 6.0], [3.0, 6.0, 1.0]]], [[[3.0, 6.0, 6.0], [1.0, 8.0, 7.0], [1.0, 6.0, 3.0]], [[2.0, 6.0, 
3.0], [5.0, 5.0, 7.0], [4.0, 5.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 6.0, 7.0], [4.0, 1.0, 4.0]]], [[[4.0, 8.0, 8.0], [4.0, 1.0, 
7.0], [8.0, 2.0, 7.0]], [[4.0, 1.0, 3.0], [4.0, 8.0, 6.0], [1.0, 1.0, 4.0]], [[8.0, 3.0, 3.0], [1.0, 8.0, 5.0], [3.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [3.0, 5.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 2.0, 8.0], [2.0, 8.0, 6.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 
3.0], [3.0, 5.0, 4.0], [5.0, 7.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 8.0, 4.0]])+(1.-msk_val)*numpy.array([[6.0, 3.0, 5.0]])
      arg[2,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [5.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [6.0, 7.0, 4.0], [5.0, 
3.0, 3.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 6.0], [3.0, 6.0, 1.0]]], [[[3.0, 6.0, 6.0], [1.0, 8.0, 7.0], [1.0, 6.0, 3.0]], [[2.0, 
6.0, 3.0], [5.0, 5.0, 7.0], [4.0, 5.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 6.0, 7.0], [4.0, 1.0, 4.0]]], [[[4.0, 8.0, 8.0], [4.0, 
1.0, 7.0], [8.0, 2.0, 7.0]], [[4.0, 1.0, 3.0], [3.0, 8.0, 4.0], [1.0, 1.0, 4.0]], [[8.0, 3.0, 3.0], [1.0, 8.0, 5.0], [3.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [3.0, 5.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 2.0, 8.0], [2.0, 8.0, 6.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 
3.0], [3.0, 5.0, 4.0], [5.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [5.0, 6.0, 5.0]], 
[[6.0, 2.0, 4.0], [6.0, 7.0, 4.0], [5.0, 3.0, 3.0]], [[8.0, 3.0, 6.0], [6.0, 8.0, 6.0], [3.0, 6.0, 1.0]]], [[[3.0, 6.0, 6.0], 
[1.0, 8.0, 7.0], [1.0, 6.0, 3.0]], [[2.0, 6.0, 3.0], [5.0, 5.0, 7.0], [4.0, 5.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 6.0, 7.0], 
[4.0, 1.0, 4.0]]], [[[4.0, 8.0, 8.0], [4.0, 1.0, 7.0], [8.0, 2.0, 7.0]], [[4.0, 1.0, 3.0], [6.0, 3.0, 5.0], [1.0, 1.0, 4.0]], 
[[8.0, 3.0, 3.0], [1.0, 8.0, 5.0], [3.0, 8.0, 8.0]]], [[[2.0, 7.0, 3.0], [3.0, 5.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 2.0, 8.0], 
[2.0, 8.0, 6.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 3.0], [3.0, 5.0, 4.0], [5.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPUN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 7.0], [6.0, 2.0, 1.0], [8.0, 4.0, 3.0]], [[4.0, 7.0, 4.0], [6.0, 6.0, 8.0], [3.0, 4.0, 
8.0]], [[3.0, 2.0, 4.0], [8.0, 5.0, 8.0], [5.0, 4.0, 3.0]]], [[[7.0, 7.0, 4.0], [4.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[5.0, 3.0, 
3.0], [5.0, 1.0, 1.0], [1.0, 5.0, 5.0]], [[5.0, 8.0, 8.0], [5.0, 5.0, 7.0], [1.0, 1.0, 8.0]]], [[[5.0, 3.0, 2.0], [2.0, 6.0, 
2.0], [1.0, 5.0, 4.0]], [[2.0, 7.0, 1.0], [7.0, 5.0, 8.0], [1.0, 8.0, 5.0]], [[8.0, 2.0, 7.0], [2.0, 6.0, 6.0], [8.0, 4.0, 
3.0]]], [[[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [5.0, 3.0, 2.0]], [[8.0, 8.0, 5.0], [5.0, 3.0, 2.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 
2.0], [3.0, 6.0, 4.0], [5.0, 1.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[4.0, 2.0, 5.0], [3.0, 7.0, 1.0]])
      arg[2,1,:2]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 7.0], [6.0, 2.0, 1.0], [8.0, 4.0, 3.0]], [[4.0, 7.0, 4.0], [6.0, 6.0, 8.0], [3.0, 4.0, 
8.0]], [[3.0, 2.0, 4.0], [8.0, 5.0, 8.0], [5.0, 4.0, 3.0]]], [[[7.0, 7.0, 4.0], [4.0, 3.0, 4.0], [8.0, 4.0, 6.0]], [[5.0, 3.0, 
3.0], [5.0, 1.0, 1.0], [1.0, 5.0, 5.0]], [[5.0, 8.0, 8.0], [5.0, 5.0, 7.0], [1.0, 1.0, 8.0]]], [[[5.0, 3.0, 2.0], [2.0, 6.0, 
2.0], [1.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [3.0, 7.0, 1.0], [1.0, 8.0, 5.0]], [[8.0, 2.0, 7.0], [2.0, 6.0, 6.0], [8.0, 4.0, 
3.0]]], [[[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [5.0, 3.0, 2.0]], [[8.0, 8.0, 5.0], [5.0, 3.0, 2.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 
2.0], [3.0, 6.0, 4.0], [5.0, 1.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPUN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 8.0], [4.0, 2.0, 5.0], [4.0, 2.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 1.0, 8.0], [5.0, 7.0, 
2.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 4.0], [5.0, 5.0, 1.0]]], [[[4.0, 2.0, 7.0], [3.0, 7.0, 5.0], [7.0, 7.0, 7.0]], [[4.0, 8.0, 
1.0], [8.0, 4.0, 1.0], [4.0, 6.0, 4.0]], [[2.0, 7.0, 8.0], [4.0, 2.0, 4.0], [6.0, 5.0, 1.0]]], [[[3.0, 3.0, 5.0], [4.0, 6.0, 
6.0], [3.0, 3.0, 1.0]], [[4.0, 8.0, 8.0], [8.0, 1.0, 8.0], [3.0, 8.0, 4.0]], [[7.0, 8.0, 2.0], [6.0, 1.0, 1.0], [6.0, 6.0, 
8.0]]], [[[3.0, 4.0, 2.0], [1.0, 4.0, 3.0], [2.0, 4.0, 8.0]], [[4.0, 1.0, 1.0], [8.0, 2.0, 1.0], [6.0, 2.0, 7.0]], [[6.0, 4.0, 
7.0], [7.0, 7.0, 3.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[4.0, 3.0, 2.0], [3.0, 7.0, 7.0]]),self.functionspace)
      arg[2,1,:2]=val
      ref=Data(numpy.array([[[[8.0, 6.0, 8.0], [4.0, 2.0, 5.0], [4.0, 2.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 1.0, 8.0], [5.0, 7.0, 
2.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 4.0], [5.0, 5.0, 1.0]]], [[[4.0, 2.0, 7.0], [3.0, 7.0, 5.0], [7.0, 7.0, 7.0]], [[4.0, 8.0, 
1.0], [8.0, 4.0, 1.0], [4.0, 6.0, 4.0]], [[2.0, 7.0, 8.0], [4.0, 2.0, 4.0], [6.0, 5.0, 1.0]]], [[[3.0, 3.0, 5.0], [4.0, 6.0, 
6.0], [3.0, 3.0, 1.0]], [[4.0, 3.0, 2.0], [3.0, 7.0, 7.0], [3.0, 8.0, 4.0]], [[7.0, 8.0, 2.0], [6.0, 1.0, 1.0], [6.0, 6.0, 
8.0]]], [[[3.0, 4.0, 2.0], [1.0, 4.0, 3.0], [2.0, 4.0, 8.0]], [[4.0, 1.0, 1.0], [8.0, 2.0, 1.0], [6.0, 2.0, 7.0]], [[6.0, 4.0, 
7.0], [7.0, 7.0, 3.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPUN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 4.0], [4.0, 3.0, 8.0], [1.0, 8.0, 4.0]], [[2.0, 8.0, 5.0], [3.0, 8.0, 7.0], [6.0, 1.0, 
6.0]], [[4.0, 1.0, 4.0], [2.0, 2.0, 8.0], [7.0, 4.0, 7.0]]], [[[3.0, 5.0, 2.0], [8.0, 3.0, 2.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 
3.0], [6.0, 2.0, 5.0], [2.0, 5.0, 8.0]], [[3.0, 1.0, 7.0], [7.0, 4.0, 5.0], [4.0, 3.0, 8.0]]], [[[7.0, 7.0, 4.0], [7.0, 2.0, 
6.0], [4.0, 7.0, 1.0]], [[2.0, 8.0, 4.0], [7.0, 2.0, 1.0], [1.0, 3.0, 5.0]], [[3.0, 4.0, 4.0], [7.0, 3.0, 3.0], [6.0, 4.0, 
3.0]]], [[[3.0, 7.0, 7.0], [2.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 5.0, 3.0], [3.0, 3.0, 6.0], [3.0, 3.0, 5.0]], [[1.0, 1.0, 
4.0], [6.0, 8.0, 7.0], [2.0, 7.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 5.0, 6.0], [5.0, 2.0, 6.0]])+(1.-msk_val)*numpy.array([[3.0, 3.0, 7.0], [3.0, 1.0, 2.0]])
      arg[2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 4.0], [4.0, 3.0, 8.0], [1.0, 8.0, 4.0]], [[2.0, 8.0, 5.0], [3.0, 8.0, 7.0], [6.0, 
1.0, 6.0]], [[4.0, 1.0, 4.0], [2.0, 2.0, 8.0], [7.0, 4.0, 7.0]]], [[[3.0, 5.0, 2.0], [8.0, 3.0, 2.0], [2.0, 4.0, 2.0]], [[8.0, 
8.0, 3.0], [6.0, 2.0, 5.0], [2.0, 5.0, 8.0]], [[3.0, 1.0, 7.0], [7.0, 4.0, 5.0], [4.0, 3.0, 8.0]]], [[[7.0, 7.0, 4.0], [7.0, 
2.0, 6.0], [4.0, 7.0, 1.0]], [[7.0, 5.0, 6.0], [5.0, 2.0, 6.0], [1.0, 3.0, 5.0]], [[3.0, 4.0, 4.0], [7.0, 3.0, 3.0], [6.0, 4.0, 
3.0]]], [[[3.0, 7.0, 7.0], [2.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 5.0, 3.0], [3.0, 3.0, 6.0], [3.0, 3.0, 5.0]], [[1.0, 1.0, 
4.0], [6.0, 8.0, 7.0], [2.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 4.0], [4.0, 3.0, 8.0], [1.0, 8.0, 4.0]], 
[[2.0, 8.0, 5.0], [3.0, 8.0, 7.0], [6.0, 1.0, 6.0]], [[4.0, 1.0, 4.0], [2.0, 2.0, 8.0], [7.0, 4.0, 7.0]]], [[[3.0, 5.0, 2.0], 
[8.0, 3.0, 2.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 3.0], [6.0, 2.0, 5.0], [2.0, 5.0, 8.0]], [[3.0, 1.0, 7.0], [7.0, 4.0, 5.0], 
[4.0, 3.0, 8.0]]], [[[7.0, 7.0, 4.0], [7.0, 2.0, 6.0], [4.0, 7.0, 1.0]], [[3.0, 3.0, 7.0], [3.0, 1.0, 2.0], [1.0, 3.0, 5.0]], 
[[3.0, 4.0, 4.0], [7.0, 3.0, 3.0], [6.0, 4.0, 3.0]]], [[[3.0, 7.0, 7.0], [2.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 5.0, 3.0], 
[3.0, 3.0, 6.0], [3.0, 3.0, 5.0]], [[1.0, 1.0, 4.0], [6.0, 8.0, 7.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLPN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 8.0], [5.0, 1.0, 2.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 2.0], [3.0, 3.0, 3.0], [3.0, 2.0, 
4.0]], [[8.0, 3.0, 7.0], [8.0, 6.0, 4.0], [7.0, 6.0, 1.0]]], [[[8.0, 2.0, 2.0], [6.0, 8.0, 8.0], [4.0, 7.0, 6.0]], [[1.0, 4.0, 
6.0], [7.0, 1.0, 5.0], [8.0, 6.0, 7.0]], [[2.0, 2.0, 2.0], [8.0, 4.0, 2.0], [1.0, 2.0, 1.0]]], [[[5.0, 8.0, 6.0], [7.0, 8.0, 
3.0], [5.0, 5.0, 1.0]], [[3.0, 1.0, 8.0], [3.0, 1.0, 6.0], [1.0, 2.0, 2.0]], [[3.0, 1.0, 1.0], [6.0, 7.0, 1.0], [5.0, 4.0, 
7.0]]], [[[3.0, 5.0, 6.0], [4.0, 5.0, 1.0], [6.0, 3.0, 1.0]], [[2.0, 2.0, 7.0], [3.0, 7.0, 3.0], [2.0, 1.0, 3.0]], [[4.0, 5.0, 
5.0], [1.0, 1.0, 7.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[8.0, 3.0, 6.0]], [[3.0, 6.0, 2.0]], [[4.0, 6.0, 5.0]], [[8.0, 2.0, 3.0]]])
      arg[:,2:,1]=val
      ref=Data(numpy.array([[[[2.0, 1.0, 8.0], [5.0, 1.0, 2.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 2.0], [3.0, 3.0, 3.0], [3.0, 2.0, 
4.0]], [[8.0, 3.0, 7.0], [8.0, 3.0, 6.0], [7.0, 6.0, 1.0]]], [[[8.0, 2.0, 2.0], [6.0, 8.0, 8.0], [4.0, 7.0, 6.0]], [[1.0, 4.0, 
6.0], [7.0, 1.0, 5.0], [8.0, 6.0, 7.0]], [[2.0, 2.0, 2.0], [3.0, 6.0, 2.0], [1.0, 2.0, 1.0]]], [[[5.0, 8.0, 6.0], [7.0, 8.0, 
3.0], [5.0, 5.0, 1.0]], [[3.0, 1.0, 8.0], [3.0, 1.0, 6.0], [1.0, 2.0, 2.0]], [[3.0, 1.0, 1.0], [4.0, 6.0, 5.0], [5.0, 4.0, 
7.0]]], [[[3.0, 5.0, 6.0], [4.0, 5.0, 1.0], [6.0, 3.0, 1.0]], [[2.0, 2.0, 7.0], [3.0, 7.0, 3.0], [2.0, 1.0, 3.0]], [[4.0, 5.0, 
5.0], [8.0, 2.0, 3.0], [6.0, 1.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLPN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 7.0], [4.0, 5.0, 1.0], [4.0, 3.0, 7.0]], [[4.0, 4.0, 1.0], [5.0, 3.0, 7.0], [3.0, 4.0, 
2.0]], [[8.0, 1.0, 3.0], [4.0, 5.0, 1.0], [8.0, 1.0, 3.0]]], [[[8.0, 3.0, 2.0], [4.0, 8.0, 5.0], [8.0, 6.0, 1.0]], [[2.0, 7.0, 
3.0], [4.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[5.0, 3.0, 6.0], [2.0, 7.0, 4.0], [2.0, 7.0, 7.0]]], [[[7.0, 6.0, 8.0], [8.0, 6.0, 
2.0], [3.0, 3.0, 6.0]], [[4.0, 5.0, 8.0], [7.0, 4.0, 5.0], [3.0, 4.0, 1.0]], [[1.0, 3.0, 5.0], [7.0, 1.0, 5.0], [5.0, 3.0, 
2.0]]], [[[4.0, 7.0, 4.0], [8.0, 3.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 2.0, 8.0], [5.0, 2.0, 8.0]], [[3.0, 5.0, 
2.0], [8.0, 6.0, 4.0], [4.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 4.0, 7.0]], [[5.0, 5.0, 7.0]], [[7.0, 2.0, 2.0]], [[1.0, 5.0, 2.0]]]),self.functionspace)
      arg[:,2:,1]=val
      ref=Data(numpy.array([[[[7.0, 7.0, 7.0], [4.0, 5.0, 1.0], [4.0, 3.0, 7.0]], [[4.0, 4.0, 1.0], [5.0, 3.0, 7.0], [3.0, 4.0, 
2.0]], [[8.0, 1.0, 3.0], [3.0, 4.0, 7.0], [8.0, 1.0, 3.0]]], [[[8.0, 3.0, 2.0], [4.0, 8.0, 5.0], [8.0, 6.0, 1.0]], [[2.0, 7.0, 
3.0], [4.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[5.0, 3.0, 6.0], [5.0, 5.0, 7.0], [2.0, 7.0, 7.0]]], [[[7.0, 6.0, 8.0], [8.0, 6.0, 
2.0], [3.0, 3.0, 6.0]], [[4.0, 5.0, 8.0], [7.0, 4.0, 5.0], [3.0, 4.0, 1.0]], [[1.0, 3.0, 5.0], [7.0, 2.0, 2.0], [5.0, 3.0, 
2.0]]], [[[4.0, 7.0, 4.0], [8.0, 3.0, 8.0], [3.0, 8.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 2.0, 8.0], [5.0, 2.0, 8.0]], [[3.0, 5.0, 
2.0], [1.0, 5.0, 2.0], [4.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLPN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 6.0], [5.0, 5.0, 2.0], [8.0, 3.0, 5.0]], [[8.0, 6.0, 7.0], [4.0, 6.0, 8.0], [3.0, 7.0, 
5.0]], [[6.0, 8.0, 3.0], [2.0, 7.0, 3.0], [8.0, 3.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 4.0, 8.0], [2.0, 4.0, 4.0]], [[4.0, 4.0, 
6.0], [1.0, 6.0, 5.0], [1.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], [1.0, 3.0, 8.0], [5.0, 8.0, 5.0]]], [[[1.0, 5.0, 1.0], [6.0, 2.0, 
3.0], [8.0, 2.0, 1.0]], [[1.0, 8.0, 4.0], [3.0, 2.0, 7.0], [6.0, 8.0, 1.0]], [[2.0, 8.0, 8.0], [3.0, 5.0, 5.0], [7.0, 3.0, 
4.0]]], [[[8.0, 5.0, 3.0], [1.0, 3.0, 2.0], [2.0, 2.0, 6.0]], [[3.0, 8.0, 3.0], [2.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[5.0, 8.0, 
3.0], [4.0, 8.0, 5.0], [1.0, 8.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 7.0, 3.0]], [[5.0, 3.0, 3.0]], [[4.0, 4.0, 4.0]], [[6.0, 6.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[8.0, 8.0, 2.0]], [[3.0, 3.0, 8.0]], [[4.0, 3.0, 2.0]], [[2.0, 5.0, 2.0]]])
      arg[:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 6.0], [5.0, 5.0, 2.0], [8.0, 3.0, 5.0]], [[8.0, 6.0, 7.0], [4.0, 6.0, 8.0], [3.0, 
7.0, 5.0]], [[6.0, 8.0, 3.0], [1.0, 7.0, 3.0], [8.0, 3.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 4.0, 8.0], [2.0, 4.0, 4.0]], [[4.0, 
4.0, 6.0], [1.0, 6.0, 5.0], [1.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], [5.0, 3.0, 3.0], [5.0, 8.0, 5.0]]], [[[1.0, 5.0, 1.0], [6.0, 
2.0, 3.0], [8.0, 2.0, 1.0]], [[1.0, 8.0, 4.0], [3.0, 2.0, 7.0], [6.0, 8.0, 1.0]], [[2.0, 8.0, 8.0], [4.0, 4.0, 4.0], [7.0, 3.0, 
4.0]]], [[[8.0, 5.0, 3.0], [1.0, 3.0, 2.0], [2.0, 2.0, 6.0]], [[3.0, 8.0, 3.0], [2.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[5.0, 8.0, 
3.0], [6.0, 6.0, 6.0], [1.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 6.0], [5.0, 5.0, 2.0], [8.0, 3.0, 5.0]], 
[[8.0, 6.0, 7.0], [4.0, 6.0, 8.0], [3.0, 7.0, 5.0]], [[6.0, 8.0, 3.0], [8.0, 8.0, 2.0], [8.0, 3.0, 3.0]]], [[[4.0, 1.0, 7.0], 
[6.0, 4.0, 8.0], [2.0, 4.0, 4.0]], [[4.0, 4.0, 6.0], [1.0, 6.0, 5.0], [1.0, 3.0, 4.0]], [[5.0, 4.0, 2.0], [3.0, 3.0, 8.0], 
[5.0, 8.0, 5.0]]], [[[1.0, 5.0, 1.0], [6.0, 2.0, 3.0], [8.0, 2.0, 1.0]], [[1.0, 8.0, 4.0], [3.0, 2.0, 7.0], [6.0, 8.0, 1.0]], 
[[2.0, 8.0, 8.0], [4.0, 3.0, 2.0], [7.0, 3.0, 4.0]]], [[[8.0, 5.0, 3.0], [1.0, 3.0, 2.0], [2.0, 2.0, 6.0]], [[3.0, 8.0, 3.0], 
[2.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[5.0, 8.0, 3.0], [2.0, 5.0, 2.0], [1.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UURN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 2.0], [8.0, 7.0, 1.0], [2.0, 1.0, 6.0]], [[1.0, 7.0, 3.0], [4.0, 3.0, 6.0], [7.0, 3.0, 
3.0]], [[6.0, 7.0, 2.0], [8.0, 6.0, 3.0], [2.0, 8.0, 2.0]]], [[[3.0, 7.0, 2.0], [6.0, 1.0, 8.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 
6.0], [4.0, 7.0, 6.0], [2.0, 5.0, 1.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 3.0], [4.0, 5.0, 4.0]]], [[[3.0, 7.0, 1.0], [4.0, 2.0, 
2.0], [3.0, 8.0, 6.0]], [[6.0, 5.0, 1.0], [6.0, 2.0, 1.0], [5.0, 1.0, 4.0]], [[4.0, 4.0, 5.0], [7.0, 5.0, 5.0], [3.0, 8.0, 
1.0]]], [[[2.0, 4.0, 6.0], [8.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[2.0, 7.0, 1.0], [2.0, 8.0, 7.0], [5.0, 3.0, 1.0]], [[6.0, 1.0, 
2.0], [7.0, 2.0, 7.0], [3.0, 8.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 3.0, 8.0]], [[7.0, 1.0, 8.0]]], [[[2.0, 2.0, 4.0]], [[4.0, 2.0, 4.0]]], [[[8.0, 4.0, 4.0]], 
[[8.0, 3.0, 7.0]]]])
      arg[:3,:2,1:2]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 2.0], [2.0, 3.0, 8.0], [2.0, 1.0, 6.0]], [[1.0, 7.0, 3.0], [7.0, 1.0, 8.0], [7.0, 3.0, 
3.0]], [[6.0, 7.0, 2.0], [8.0, 6.0, 3.0], [2.0, 8.0, 2.0]]], [[[3.0, 7.0, 2.0], [2.0, 2.0, 4.0], [7.0, 6.0, 8.0]], [[3.0, 4.0, 
6.0], [4.0, 2.0, 4.0], [2.0, 5.0, 1.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 3.0], [4.0, 5.0, 4.0]]], [[[3.0, 7.0, 1.0], [8.0, 4.0, 
4.0], [3.0, 8.0, 6.0]], [[6.0, 5.0, 1.0], [8.0, 3.0, 7.0], [5.0, 1.0, 4.0]], [[4.0, 4.0, 5.0], [7.0, 5.0, 5.0], [3.0, 8.0, 
1.0]]], [[[2.0, 4.0, 6.0], [8.0, 8.0, 2.0], [6.0, 5.0, 1.0]], [[2.0, 7.0, 1.0], [2.0, 8.0, 7.0], [5.0, 3.0, 1.0]], [[6.0, 1.0, 
2.0], [7.0, 2.0, 7.0], [3.0, 8.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UURN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 2.0], [3.0, 3.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 3.0, 1.0], [3.0, 1.0, 3.0], [8.0, 4.0, 
3.0]], [[4.0, 6.0, 1.0], [6.0, 2.0, 3.0], [8.0, 7.0, 8.0]]], [[[8.0, 8.0, 4.0], [5.0, 4.0, 5.0], [3.0, 5.0, 8.0]], [[2.0, 1.0, 
1.0], [1.0, 7.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 2.0, 2.0], [4.0, 6.0, 6.0]]], [[[5.0, 6.0, 4.0], [3.0, 4.0, 
4.0], [8.0, 7.0, 3.0]], [[4.0, 5.0, 4.0], [8.0, 1.0, 2.0], [2.0, 2.0, 7.0]], [[1.0, 8.0, 2.0], [5.0, 6.0, 1.0], [4.0, 3.0, 
1.0]]], [[[1.0, 6.0, 4.0], [7.0, 4.0, 8.0], [5.0, 1.0, 7.0]], [[1.0, 1.0, 5.0], [8.0, 4.0, 2.0], [8.0, 2.0, 4.0]], [[8.0, 8.0, 
3.0], [8.0, 1.0, 3.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 6.0, 2.0]], [[8.0, 7.0, 3.0]]], [[[4.0, 4.0, 2.0]], [[1.0, 5.0, 2.0]]], [[[4.0, 2.0, 1.0]], 
[[6.0, 7.0, 6.0]]]]),self.functionspace)
      arg[:3,:2,1:2]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 2.0], [5.0, 6.0, 2.0], [3.0, 2.0, 8.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 3.0], [8.0, 4.0, 
3.0]], [[4.0, 6.0, 1.0], [6.0, 2.0, 3.0], [8.0, 7.0, 8.0]]], [[[8.0, 8.0, 4.0], [4.0, 4.0, 2.0], [3.0, 5.0, 8.0]], [[2.0, 1.0, 
1.0], [1.0, 5.0, 2.0], [8.0, 1.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 2.0, 2.0], [4.0, 6.0, 6.0]]], [[[5.0, 6.0, 4.0], [4.0, 2.0, 
1.0], [8.0, 7.0, 3.0]], [[4.0, 5.0, 4.0], [6.0, 7.0, 6.0], [2.0, 2.0, 7.0]], [[1.0, 8.0, 2.0], [5.0, 6.0, 1.0], [4.0, 3.0, 
1.0]]], [[[1.0, 6.0, 4.0], [7.0, 4.0, 8.0], [5.0, 1.0, 7.0]], [[1.0, 1.0, 5.0], [8.0, 4.0, 2.0], [8.0, 2.0, 4.0]], [[8.0, 8.0, 
3.0], [8.0, 1.0, 3.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UURN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [4.0, 5.0, 1.0], [3.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [7.0, 6.0, 6.0], [6.0, 3.0, 
4.0]], [[1.0, 7.0, 4.0], [1.0, 1.0, 1.0], [3.0, 2.0, 2.0]]], [[[1.0, 4.0, 1.0], [6.0, 4.0, 2.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 
8.0], [4.0, 5.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 7.0], [4.0, 8.0, 7.0]]], [[[1.0, 6.0, 7.0], [5.0, 2.0, 
1.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 4.0], [1.0, 5.0, 4.0], [6.0, 3.0, 8.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 4.0], [5.0, 2.0, 
5.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 8.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 
7.0], [5.0, 1.0, 3.0], [8.0, 3.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 3.0, 1.0]], [[7.0, 6.0, 2.0]]], [[[3.0, 2.0, 7.0]], [[4.0, 3.0, 6.0]]], [[[8.0, 1.0, 
4.0]], [[6.0, 3.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 4.0, 6.0]], [[2.0, 8.0, 2.0]]], [[[3.0, 8.0, 1.0]], [[5.0, 2.0, 
7.0]]], [[[7.0, 1.0, 5.0]], [[8.0, 1.0, 6.0]]]])
      arg[:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [7.0, 3.0, 1.0], [3.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [7.0, 6.0, 2.0], [6.0, 
3.0, 4.0]], [[1.0, 7.0, 4.0], [1.0, 1.0, 1.0], [3.0, 2.0, 2.0]]], [[[1.0, 4.0, 1.0], [3.0, 2.0, 7.0], [3.0, 8.0, 3.0]], [[7.0, 
7.0, 8.0], [4.0, 3.0, 6.0], [1.0, 6.0, 1.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 7.0], [4.0, 8.0, 7.0]]], [[[1.0, 6.0, 7.0], [8.0, 
1.0, 4.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 4.0], [6.0, 3.0, 7.0], [6.0, 3.0, 8.0]], [[5.0, 8.0, 4.0], [4.0, 2.0, 4.0], [5.0, 2.0, 
5.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[8.0, 2.0, 6.0], [7.0, 7.0, 8.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 
7.0], [5.0, 1.0, 3.0], [8.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 8.0], [3.0, 4.0, 6.0], [3.0, 1.0, 2.0]], 
[[6.0, 3.0, 5.0], [2.0, 8.0, 2.0], [6.0, 3.0, 4.0]], [[1.0, 7.0, 4.0], [1.0, 1.0, 1.0], [3.0, 2.0, 2.0]]], [[[1.0, 4.0, 1.0], 
[3.0, 8.0, 1.0], [3.0, 8.0, 3.0]], [[7.0, 7.0, 8.0], [5.0, 2.0, 7.0], [1.0, 6.0, 1.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 7.0], 
[4.0, 8.0, 7.0]]], [[[1.0, 6.0, 7.0], [7.0, 1.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 6.0, 4.0], [8.0, 1.0, 6.0], [6.0, 3.0, 8.0]], 
[[5.0, 8.0, 4.0], [4.0, 2.0, 4.0], [5.0, 2.0, 5.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[8.0, 2.0, 6.0], 
[7.0, 7.0, 8.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 7.0], [5.0, 1.0, 3.0], [8.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRCN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 2.0], [4.0, 2.0, 6.0], [5.0, 8.0, 1.0]], [[6.0, 7.0, 4.0], [7.0, 8.0, 5.0], [3.0, 5.0, 
5.0]], [[6.0, 8.0, 5.0], [5.0, 8.0, 4.0], [8.0, 7.0, 3.0]]], [[[3.0, 3.0, 6.0], [4.0, 3.0, 8.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 
1.0], [2.0, 2.0, 1.0], [3.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [1.0, 1.0, 1.0], [2.0, 3.0, 4.0]]], [[[5.0, 4.0, 7.0], [1.0, 7.0, 
3.0], [1.0, 3.0, 7.0]], [[6.0, 2.0, 3.0], [4.0, 6.0, 5.0], [3.0, 8.0, 2.0]], [[1.0, 2.0, 6.0], [2.0, 1.0, 6.0], [7.0, 4.0, 
2.0]]], [[[5.0, 5.0, 6.0], [8.0, 8.0, 6.0], [4.0, 6.0, 4.0]], [[4.0, 1.0, 4.0], [1.0, 5.0, 4.0], [2.0, 7.0, 7.0]], [[7.0, 2.0, 
4.0], [5.0, 7.0, 1.0], [7.0, 8.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 3.0, 4.0], [4.0, 6.0, 8.0], [4.0, 1.0, 6.0]]], [[[1.0, 2.0, 7.0], [1.0, 1.0, 2.0], [2.0, 2.0, 
1.0]]]])
      arg[2:,1:2,:]=val
      ref=Data(numpy.array([[[[8.0, 7.0, 2.0], [4.0, 2.0, 6.0], [5.0, 8.0, 1.0]], [[6.0, 7.0, 4.0], [7.0, 8.0, 5.0], [3.0, 5.0, 
5.0]], [[6.0, 8.0, 5.0], [5.0, 8.0, 4.0], [8.0, 7.0, 3.0]]], [[[3.0, 3.0, 6.0], [4.0, 3.0, 8.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 
1.0], [2.0, 2.0, 1.0], [3.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], [1.0, 1.0, 1.0], [2.0, 3.0, 4.0]]], [[[5.0, 4.0, 7.0], [1.0, 7.0, 
3.0], [1.0, 3.0, 7.0]], [[4.0, 3.0, 4.0], [4.0, 6.0, 8.0], [4.0, 1.0, 6.0]], [[1.0, 2.0, 6.0], [2.0, 1.0, 6.0], [7.0, 4.0, 
2.0]]], [[[5.0, 5.0, 6.0], [8.0, 8.0, 6.0], [4.0, 6.0, 4.0]], [[1.0, 2.0, 7.0], [1.0, 1.0, 2.0], [2.0, 2.0, 1.0]], [[7.0, 2.0, 
4.0], [5.0, 7.0, 1.0], [7.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRCN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 1.0], [2.0, 8.0, 1.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 2.0], [1.0, 1.0, 7.0], [8.0, 1.0, 
1.0]], [[2.0, 5.0, 2.0], [6.0, 4.0, 3.0], [8.0, 8.0, 6.0]]], [[[8.0, 5.0, 3.0], [2.0, 1.0, 6.0], [4.0, 7.0, 1.0]], [[2.0, 4.0, 
2.0], [7.0, 7.0, 6.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 7.0], [6.0, 5.0, 4.0], [7.0, 8.0, 3.0]]], [[[7.0, 2.0, 3.0], [3.0, 4.0, 
1.0], [7.0, 3.0, 5.0]], [[1.0, 7.0, 4.0], [5.0, 1.0, 5.0], [1.0, 8.0, 8.0]], [[1.0, 1.0, 1.0], [7.0, 5.0, 4.0], [8.0, 5.0, 
8.0]]], [[[6.0, 7.0, 6.0], [6.0, 3.0, 2.0], [4.0, 2.0, 8.0]], [[6.0, 7.0, 2.0], [6.0, 4.0, 6.0], [8.0, 3.0, 8.0]], [[6.0, 1.0, 
5.0], [7.0, 1.0, 4.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 6.0, 4.0], [6.0, 4.0, 3.0], [4.0, 6.0, 5.0]]], [[[2.0, 7.0, 2.0], [2.0, 1.0, 7.0], [7.0, 
5.0, 2.0]]]]),self.functionspace)
      arg[2:,1:2,:]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 1.0], [2.0, 8.0, 1.0], [2.0, 6.0, 5.0]], [[6.0, 4.0, 2.0], [1.0, 1.0, 7.0], [8.0, 1.0, 
1.0]], [[2.0, 5.0, 2.0], [6.0, 4.0, 3.0], [8.0, 8.0, 6.0]]], [[[8.0, 5.0, 3.0], [2.0, 1.0, 6.0], [4.0, 7.0, 1.0]], [[2.0, 4.0, 
2.0], [7.0, 7.0, 6.0], [2.0, 4.0, 8.0]], [[5.0, 1.0, 7.0], [6.0, 5.0, 4.0], [7.0, 8.0, 3.0]]], [[[7.0, 2.0, 3.0], [3.0, 4.0, 
1.0], [7.0, 3.0, 5.0]], [[4.0, 6.0, 4.0], [6.0, 4.0, 3.0], [4.0, 6.0, 5.0]], [[1.0, 1.0, 1.0], [7.0, 5.0, 4.0], [8.0, 5.0, 
8.0]]], [[[6.0, 7.0, 6.0], [6.0, 3.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 7.0, 2.0], [2.0, 1.0, 7.0], [7.0, 5.0, 2.0]], [[6.0, 1.0, 
5.0], [7.0, 1.0, 4.0], [6.0, 2.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRCN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 5.0], [7.0, 8.0, 1.0], [8.0, 5.0, 1.0]], [[6.0, 7.0, 8.0], [8.0, 5.0, 6.0], [7.0, 8.0, 
8.0]], [[6.0, 6.0, 1.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]]], [[[5.0, 6.0, 4.0], [5.0, 4.0, 8.0], [2.0, 4.0, 3.0]], [[6.0, 7.0, 
7.0], [1.0, 7.0, 3.0], [7.0, 8.0, 1.0]], [[7.0, 5.0, 5.0], [8.0, 5.0, 7.0], [6.0, 5.0, 6.0]]], [[[5.0, 2.0, 8.0], [4.0, 6.0, 
4.0], [7.0, 1.0, 8.0]], [[4.0, 3.0, 7.0], [7.0, 2.0, 2.0], [2.0, 7.0, 5.0]], [[6.0, 2.0, 2.0], [2.0, 7.0, 8.0], [7.0, 6.0, 
6.0]]], [[[3.0, 6.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[3.0, 8.0, 5.0], [5.0, 1.0, 1.0], [1.0, 3.0, 6.0]], [[5.0, 8.0, 
2.0], [2.0, 5.0, 5.0], [1.0, 4.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 1.0, 7.0], [6.0, 1.0, 8.0], [7.0, 2.0, 4.0]]], [[[4.0, 2.0, 8.0], [1.0, 4.0, 3.0], [1.0, 
1.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 4.0, 8.0], [7.0, 8.0, 8.0], [5.0, 8.0, 7.0]]], [[[5.0, 7.0, 4.0], [5.0, 4.0, 
4.0], [5.0, 2.0, 2.0]]]])
      arg[2:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 5.0], [7.0, 8.0, 1.0], [8.0, 5.0, 1.0]], [[6.0, 7.0, 8.0], [8.0, 5.0, 6.0], [7.0, 
8.0, 8.0]], [[6.0, 6.0, 1.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]]], [[[5.0, 6.0, 4.0], [5.0, 4.0, 8.0], [2.0, 4.0, 3.0]], [[6.0, 
7.0, 7.0], [1.0, 7.0, 3.0], [7.0, 8.0, 1.0]], [[7.0, 5.0, 5.0], [8.0, 5.0, 7.0], [6.0, 5.0, 6.0]]], [[[5.0, 2.0, 8.0], [4.0, 
6.0, 4.0], [7.0, 1.0, 8.0]], [[1.0, 1.0, 7.0], [6.0, 1.0, 8.0], [7.0, 2.0, 4.0]], [[6.0, 2.0, 2.0], [2.0, 7.0, 8.0], [7.0, 6.0, 
6.0]]], [[[3.0, 6.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[4.0, 2.0, 8.0], [1.0, 4.0, 3.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 
2.0], [2.0, 5.0, 5.0], [1.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 5.0], [7.0, 8.0, 1.0], [8.0, 5.0, 1.0]], 
[[6.0, 7.0, 8.0], [8.0, 5.0, 6.0], [7.0, 8.0, 8.0]], [[6.0, 6.0, 1.0], [3.0, 1.0, 4.0], [4.0, 5.0, 2.0]]], [[[5.0, 6.0, 4.0], 
[5.0, 4.0, 8.0], [2.0, 4.0, 3.0]], [[6.0, 7.0, 7.0], [1.0, 7.0, 3.0], [7.0, 8.0, 1.0]], [[7.0, 5.0, 5.0], [8.0, 5.0, 7.0], 
[6.0, 5.0, 6.0]]], [[[5.0, 2.0, 8.0], [4.0, 6.0, 4.0], [7.0, 1.0, 8.0]], [[2.0, 4.0, 8.0], [7.0, 8.0, 8.0], [5.0, 8.0, 7.0]], 
[[6.0, 2.0, 2.0], [2.0, 7.0, 8.0], [7.0, 6.0, 6.0]]], [[[3.0, 6.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], 
[5.0, 4.0, 4.0], [5.0, 2.0, 2.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 5.0], [1.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLCN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 6.0], [2.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[2.0, 3.0, 4.0], [8.0, 8.0, 8.0], [4.0, 6.0, 
8.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 4.0], [4.0, 4.0, 3.0]]], [[[4.0, 5.0, 2.0], [8.0, 8.0, 4.0], [3.0, 5.0, 8.0]], [[3.0, 5.0, 
1.0], [8.0, 5.0, 5.0], [4.0, 6.0, 3.0]], [[7.0, 7.0, 2.0], [1.0, 4.0, 6.0], [6.0, 4.0, 4.0]]], [[[4.0, 4.0, 1.0], [7.0, 4.0, 
7.0], [1.0, 3.0, 1.0]], [[2.0, 4.0, 1.0], [5.0, 3.0, 8.0], [7.0, 7.0, 7.0]], [[3.0, 6.0, 4.0], [6.0, 2.0, 2.0], [8.0, 6.0, 
3.0]]], [[[7.0, 1.0, 4.0], [2.0, 6.0, 8.0], [1.0, 1.0, 6.0]], [[5.0, 4.0, 1.0], [5.0, 7.0, 2.0], [7.0, 1.0, 3.0]], [[4.0, 8.0, 
5.0], [7.0, 3.0, 6.0], [1.0, 2.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 8.0, 6.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0]]], [[[7.0, 4.0, 5.0], [5.0, 1.0, 7.0], [2.0, 6.0, 
6.0]]]])
      arg[1:3,2:,:]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 6.0], [2.0, 7.0, 7.0], [2.0, 7.0, 8.0]], [[2.0, 3.0, 4.0], [8.0, 8.0, 8.0], [4.0, 6.0, 
8.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 4.0], [4.0, 4.0, 3.0]]], [[[4.0, 5.0, 2.0], [8.0, 8.0, 4.0], [3.0, 5.0, 8.0]], [[3.0, 5.0, 
1.0], [8.0, 5.0, 5.0], [4.0, 6.0, 3.0]], [[4.0, 8.0, 6.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0]]], [[[4.0, 4.0, 1.0], [7.0, 4.0, 
7.0], [1.0, 3.0, 1.0]], [[2.0, 4.0, 1.0], [5.0, 3.0, 8.0], [7.0, 7.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 1.0, 7.0], [2.0, 6.0, 
6.0]]], [[[7.0, 1.0, 4.0], [2.0, 6.0, 8.0], [1.0, 1.0, 6.0]], [[5.0, 4.0, 1.0], [5.0, 7.0, 2.0], [7.0, 1.0, 3.0]], [[4.0, 8.0, 
5.0], [7.0, 3.0, 6.0], [1.0, 2.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLCN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 4.0], [2.0, 8.0, 7.0], [1.0, 1.0, 8.0]], [[5.0, 1.0, 5.0], [3.0, 1.0, 8.0], [1.0, 1.0, 
7.0]], [[2.0, 2.0, 6.0], [5.0, 4.0, 6.0], [7.0, 2.0, 5.0]]], [[[8.0, 3.0, 8.0], [1.0, 1.0, 6.0], [4.0, 4.0, 3.0]], [[7.0, 4.0, 
7.0], [5.0, 8.0, 3.0], [3.0, 3.0, 5.0]], [[6.0, 1.0, 4.0], [6.0, 8.0, 6.0], [7.0, 3.0, 4.0]]], [[[2.0, 2.0, 3.0], [6.0, 1.0, 
1.0], [6.0, 1.0, 8.0]], [[7.0, 1.0, 5.0], [7.0, 8.0, 6.0], [7.0, 6.0, 8.0]], [[4.0, 3.0, 6.0], [3.0, 6.0, 3.0], [5.0, 6.0, 
1.0]]], [[[1.0, 5.0, 3.0], [7.0, 4.0, 2.0], [7.0, 7.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 4.0, 6.0], [5.0, 4.0, 6.0]], [[5.0, 3.0, 
6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 2.0, 2.0], [5.0, 2.0, 4.0], [8.0, 1.0, 7.0]]], [[[5.0, 7.0, 3.0], [4.0, 5.0, 2.0], [4.0, 
1.0, 1.0]]]]),self.functionspace)
      arg[1:3,2:,:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 4.0], [2.0, 8.0, 7.0], [1.0, 1.0, 8.0]], [[5.0, 1.0, 5.0], [3.0, 1.0, 8.0], [1.0, 1.0, 
7.0]], [[2.0, 2.0, 6.0], [5.0, 4.0, 6.0], [7.0, 2.0, 5.0]]], [[[8.0, 3.0, 8.0], [1.0, 1.0, 6.0], [4.0, 4.0, 3.0]], [[7.0, 4.0, 
7.0], [5.0, 8.0, 3.0], [3.0, 3.0, 5.0]], [[7.0, 2.0, 2.0], [5.0, 2.0, 4.0], [8.0, 1.0, 7.0]]], [[[2.0, 2.0, 3.0], [6.0, 1.0, 
1.0], [6.0, 1.0, 8.0]], [[7.0, 1.0, 5.0], [7.0, 8.0, 6.0], [7.0, 6.0, 8.0]], [[5.0, 7.0, 3.0], [4.0, 5.0, 2.0], [4.0, 1.0, 
1.0]]], [[[1.0, 5.0, 3.0], [7.0, 4.0, 2.0], [7.0, 7.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 4.0, 6.0], [5.0, 4.0, 6.0]], [[5.0, 3.0, 
6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLCN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 6.0], [2.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 2.0], [2.0, 3.0, 
4.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 2.0], [3.0, 6.0, 3.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 8.0], [4.0, 7.0, 6.0]], [[2.0, 7.0, 
8.0], [2.0, 8.0, 6.0], [6.0, 1.0, 5.0]], [[5.0, 3.0, 3.0], [7.0, 3.0, 4.0], [6.0, 6.0, 4.0]]], [[[3.0, 2.0, 5.0], [6.0, 7.0, 
2.0], [2.0, 8.0, 6.0]], [[6.0, 1.0, 2.0], [7.0, 6.0, 7.0], [5.0, 8.0, 1.0]], [[1.0, 1.0, 4.0], [1.0, 5.0, 8.0], [3.0, 7.0, 
7.0]]], [[[3.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 4.0, 2.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[6.0, 4.0, 
4.0], [8.0, 4.0, 8.0], [7.0, 4.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 5.0, 1.0], [7.0, 1.0, 5.0], [3.0, 1.0, 4.0]]], [[[4.0, 3.0, 4.0], [3.0, 2.0, 7.0], [7.0, 
7.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 2.0, 2.0], [5.0, 7.0, 5.0], [5.0, 4.0, 5.0]]], [[[8.0, 8.0, 6.0], [2.0, 4.0, 
5.0], [5.0, 2.0, 8.0]]]])
      arg[1:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 6.0], [2.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 8.0, 1.0], [5.0, 6.0, 2.0], [2.0, 
3.0, 4.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 2.0], [3.0, 6.0, 3.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 8.0], [4.0, 7.0, 6.0]], [[2.0, 
7.0, 8.0], [2.0, 8.0, 6.0], [6.0, 1.0, 5.0]], [[6.0, 5.0, 1.0], [7.0, 1.0, 5.0], [3.0, 1.0, 4.0]]], [[[3.0, 2.0, 5.0], [6.0, 
7.0, 2.0], [2.0, 8.0, 6.0]], [[6.0, 1.0, 2.0], [7.0, 6.0, 7.0], [5.0, 8.0, 1.0]], [[4.0, 3.0, 4.0], [3.0, 2.0, 7.0], [7.0, 7.0, 
2.0]]], [[[3.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 4.0, 2.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[6.0, 4.0, 
4.0], [8.0, 4.0, 8.0], [7.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 6.0], [2.0, 3.0, 8.0], [4.0, 5.0, 6.0]], 
[[2.0, 8.0, 1.0], [5.0, 6.0, 2.0], [2.0, 3.0, 4.0]], [[2.0, 6.0, 8.0], [7.0, 7.0, 2.0], [3.0, 6.0, 3.0]]], [[[3.0, 5.0, 2.0], 
[2.0, 2.0, 8.0], [4.0, 7.0, 6.0]], [[2.0, 7.0, 8.0], [2.0, 8.0, 6.0], [6.0, 1.0, 5.0]], [[5.0, 2.0, 2.0], [5.0, 7.0, 5.0], 
[5.0, 4.0, 5.0]]], [[[3.0, 2.0, 5.0], [6.0, 7.0, 2.0], [2.0, 8.0, 6.0]], [[6.0, 1.0, 2.0], [7.0, 6.0, 7.0], [5.0, 8.0, 1.0]], 
[[8.0, 8.0, 6.0], [2.0, 4.0, 5.0], [5.0, 2.0, 8.0]]], [[[3.0, 7.0, 6.0], [4.0, 6.0, 5.0], [4.0, 4.0, 2.0]], [[8.0, 6.0, 7.0], 
[2.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[6.0, 4.0, 4.0], [8.0, 4.0, 8.0], [7.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LCUN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 6.0], [2.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[8.0, 3.0, 1.0], [4.0, 5.0, 6.0], [2.0, 6.0, 
3.0]], [[6.0, 2.0, 6.0], [6.0, 6.0, 1.0], [1.0, 8.0, 1.0]]], [[[3.0, 3.0, 7.0], [4.0, 8.0, 4.0], [4.0, 7.0, 8.0]], [[7.0, 7.0, 
7.0], [8.0, 3.0, 5.0], [3.0, 1.0, 8.0]], [[1.0, 5.0, 6.0], [8.0, 6.0, 2.0], [6.0, 6.0, 2.0]]], [[[4.0, 2.0, 2.0], [3.0, 8.0, 
6.0], [6.0, 7.0, 3.0]], [[1.0, 3.0, 3.0], [8.0, 3.0, 6.0], [2.0, 4.0, 3.0]], [[6.0, 3.0, 5.0], [3.0, 4.0, 2.0], [4.0, 6.0, 
2.0]]], [[[5.0, 2.0, 5.0], [3.0, 1.0, 3.0], [3.0, 2.0, 8.0]], [[2.0, 4.0, 3.0], [6.0, 5.0, 7.0], [3.0, 3.0, 4.0]], [[6.0, 6.0, 
8.0], [5.0, 8.0, 7.0], [2.0, 8.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 1.0, 7.0], [2.0, 1.0, 4.0]], [[2.0, 5.0, 8.0], [2.0, 4.0, 4.0]], [[2.0, 1.0, 4.0], [3.0, 1.0, 
2.0]]], [[[8.0, 8.0, 3.0], [1.0, 8.0, 3.0]], [[8.0, 3.0, 6.0], [7.0, 3.0, 6.0]], [[8.0, 7.0, 7.0], [1.0, 3.0, 5.0]]]])
      arg[2:,:,:2]=val
      ref=Data(numpy.array([[[[3.0, 1.0, 6.0], [2.0, 5.0, 8.0], [8.0, 6.0, 7.0]], [[8.0, 3.0, 1.0], [4.0, 5.0, 6.0], [2.0, 6.0, 
3.0]], [[6.0, 2.0, 6.0], [6.0, 6.0, 1.0], [1.0, 8.0, 1.0]]], [[[3.0, 3.0, 7.0], [4.0, 8.0, 4.0], [4.0, 7.0, 8.0]], [[7.0, 7.0, 
7.0], [8.0, 3.0, 5.0], [3.0, 1.0, 8.0]], [[1.0, 5.0, 6.0], [8.0, 6.0, 2.0], [6.0, 6.0, 2.0]]], [[[2.0, 1.0, 7.0], [2.0, 1.0, 
4.0], [6.0, 7.0, 3.0]], [[2.0, 5.0, 8.0], [2.0, 4.0, 4.0], [2.0, 4.0, 3.0]], [[2.0, 1.0, 4.0], [3.0, 1.0, 2.0], [4.0, 6.0, 
2.0]]], [[[8.0, 8.0, 3.0], [1.0, 8.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [7.0, 3.0, 6.0], [3.0, 3.0, 4.0]], [[8.0, 7.0, 
7.0], [1.0, 3.0, 5.0], [2.0, 8.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LCUN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 5.0], [5.0, 7.0, 7.0], [5.0, 6.0, 7.0]], [[2.0, 8.0, 8.0], [3.0, 8.0, 3.0], [7.0, 2.0, 
5.0]], [[8.0, 2.0, 4.0], [4.0, 8.0, 4.0], [7.0, 1.0, 6.0]]], [[[2.0, 7.0, 8.0], [2.0, 2.0, 8.0], [7.0, 6.0, 4.0]], [[2.0, 3.0, 
8.0], [4.0, 3.0, 4.0], [6.0, 7.0, 3.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 1.0], [8.0, 5.0, 5.0]]], [[[8.0, 2.0, 8.0], [6.0, 7.0, 
2.0], [3.0, 2.0, 2.0]], [[8.0, 5.0, 2.0], [4.0, 5.0, 5.0], [5.0, 2.0, 8.0]], [[6.0, 1.0, 4.0], [7.0, 5.0, 5.0], [3.0, 2.0, 
5.0]]], [[[1.0, 3.0, 2.0], [2.0, 2.0, 5.0], [8.0, 3.0, 5.0]], [[3.0, 7.0, 1.0], [8.0, 3.0, 3.0], [2.0, 1.0, 5.0]], [[3.0, 2.0, 
3.0], [6.0, 5.0, 5.0], [5.0, 7.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 6.0, 1.0], [2.0, 5.0, 7.0]], [[2.0, 4.0, 4.0], [5.0, 7.0, 5.0]], [[3.0, 4.0, 6.0], [2.0, 
6.0, 8.0]]], [[[1.0, 8.0, 6.0], [8.0, 2.0, 7.0]], [[4.0, 2.0, 7.0], [4.0, 5.0, 3.0]], [[2.0, 7.0, 1.0], [5.0, 3.0, 
4.0]]]]),self.functionspace)
      arg[2:,:,:2]=val
      ref=Data(numpy.array([[[[8.0, 2.0, 5.0], [5.0, 7.0, 7.0], [5.0, 6.0, 7.0]], [[2.0, 8.0, 8.0], [3.0, 8.0, 3.0], [7.0, 2.0, 
5.0]], [[8.0, 2.0, 4.0], [4.0, 8.0, 4.0], [7.0, 1.0, 6.0]]], [[[2.0, 7.0, 8.0], [2.0, 2.0, 8.0], [7.0, 6.0, 4.0]], [[2.0, 3.0, 
8.0], [4.0, 3.0, 4.0], [6.0, 7.0, 3.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 1.0], [8.0, 5.0, 5.0]]], [[[3.0, 6.0, 1.0], [2.0, 5.0, 
7.0], [3.0, 2.0, 2.0]], [[2.0, 4.0, 4.0], [5.0, 7.0, 5.0], [5.0, 2.0, 8.0]], [[3.0, 4.0, 6.0], [2.0, 6.0, 8.0], [3.0, 2.0, 
5.0]]], [[[1.0, 8.0, 6.0], [8.0, 2.0, 7.0], [8.0, 3.0, 5.0]], [[4.0, 2.0, 7.0], [4.0, 5.0, 3.0], [2.0, 1.0, 5.0]], [[2.0, 7.0, 
1.0], [5.0, 3.0, 4.0], [5.0, 7.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LCUN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 2.0], [8.0, 5.0, 6.0], [7.0, 5.0, 8.0]], [[4.0, 6.0, 2.0], [2.0, 6.0, 7.0], [6.0, 3.0, 
7.0]], [[2.0, 2.0, 1.0], [6.0, 8.0, 2.0], [2.0, 6.0, 5.0]]], [[[8.0, 8.0, 6.0], [4.0, 3.0, 5.0], [6.0, 5.0, 7.0]], [[5.0, 8.0, 
1.0], [5.0, 1.0, 5.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 4.0], [5.0, 4.0, 1.0], [8.0, 6.0, 3.0]]], [[[4.0, 2.0, 8.0], [7.0, 6.0, 
5.0], [5.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], [6.0, 7.0, 1.0], [1.0, 5.0, 8.0]], [[6.0, 1.0, 1.0], [6.0, 6.0, 8.0], [5.0, 4.0, 
5.0]]], [[[1.0, 2.0, 2.0], [5.0, 7.0, 1.0], [4.0, 5.0, 5.0]], [[4.0, 1.0, 8.0], [7.0, 8.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 
7.0], [7.0, 3.0, 1.0], [4.0, 2.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 2.0, 2.0], [2.0, 8.0, 8.0]], [[7.0, 3.0, 4.0], [2.0, 1.0, 4.0]], [[6.0, 6.0, 3.0], [5.0, 
3.0, 1.0]]], [[[8.0, 6.0, 2.0], [2.0, 3.0, 5.0]], [[1.0, 4.0, 8.0], [1.0, 4.0, 2.0]], [[6.0, 1.0, 8.0], [4.0, 1.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 5.0], [1.0, 6.0, 6.0]], [[6.0, 3.0, 1.0], [8.0, 4.0, 7.0]], [[2.0, 1.0, 7.0], 
[7.0, 1.0, 2.0]]], [[[7.0, 6.0, 7.0], [8.0, 2.0, 5.0]], [[5.0, 2.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 
3.0]]]])
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 2.0], [8.0, 5.0, 6.0], [7.0, 5.0, 8.0]], [[4.0, 6.0, 2.0], [2.0, 6.0, 7.0], [6.0, 
3.0, 7.0]], [[2.0, 2.0, 1.0], [6.0, 8.0, 2.0], [2.0, 6.0, 5.0]]], [[[8.0, 8.0, 6.0], [4.0, 3.0, 5.0], [6.0, 5.0, 7.0]], [[5.0, 
8.0, 1.0], [5.0, 1.0, 5.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 4.0], [5.0, 4.0, 1.0], [8.0, 6.0, 3.0]]], [[[8.0, 2.0, 2.0], [2.0, 
8.0, 8.0], [5.0, 1.0, 3.0]], [[7.0, 3.0, 4.0], [2.0, 1.0, 4.0], [1.0, 5.0, 8.0]], [[6.0, 6.0, 3.0], [5.0, 3.0, 1.0], [5.0, 4.0, 
5.0]]], [[[8.0, 6.0, 2.0], [2.0, 3.0, 5.0], [4.0, 5.0, 5.0]], [[1.0, 4.0, 8.0], [1.0, 4.0, 2.0], [7.0, 6.0, 8.0]], [[6.0, 1.0, 
8.0], [4.0, 1.0, 2.0], [4.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 2.0], [8.0, 5.0, 6.0], [7.0, 5.0, 8.0]], 
[[4.0, 6.0, 2.0], [2.0, 6.0, 7.0], [6.0, 3.0, 7.0]], [[2.0, 2.0, 1.0], [6.0, 8.0, 2.0], [2.0, 6.0, 5.0]]], [[[8.0, 8.0, 6.0], 
[4.0, 3.0, 5.0], [6.0, 5.0, 7.0]], [[5.0, 8.0, 1.0], [5.0, 1.0, 5.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 4.0], [5.0, 4.0, 1.0], 
[8.0, 6.0, 3.0]]], [[[5.0, 4.0, 5.0], [1.0, 6.0, 6.0], [5.0, 1.0, 3.0]], [[6.0, 3.0, 1.0], [8.0, 4.0, 7.0], [1.0, 5.0, 8.0]], 
[[2.0, 1.0, 7.0], [7.0, 1.0, 2.0], [5.0, 4.0, 5.0]]], [[[7.0, 6.0, 7.0], [8.0, 2.0, 5.0], [4.0, 5.0, 5.0]], [[5.0, 2.0, 4.0], 
[6.0, 5.0, 1.0], [7.0, 6.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 5.0, 3.0], [4.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPUN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 4.0], [2.0, 2.0, 8.0], [5.0, 2.0, 6.0]], [[1.0, 5.0, 8.0], [7.0, 8.0, 7.0], [3.0, 1.0, 
4.0]], [[4.0, 3.0, 4.0], [7.0, 6.0, 5.0], [5.0, 5.0, 7.0]]], [[[4.0, 5.0, 1.0], [6.0, 1.0, 1.0], [1.0, 2.0, 3.0]], [[4.0, 1.0, 
4.0], [8.0, 5.0, 2.0], [2.0, 3.0, 6.0]], [[5.0, 5.0, 4.0], [8.0, 6.0, 8.0], [2.0, 4.0, 2.0]]], [[[3.0, 2.0, 4.0], [6.0, 2.0, 
5.0], [5.0, 4.0, 1.0]], [[5.0, 1.0, 7.0], [2.0, 5.0, 1.0], [3.0, 3.0, 3.0]], [[2.0, 3.0, 1.0], [2.0, 7.0, 4.0], [7.0, 8.0, 
3.0]]], [[[1.0, 4.0, 4.0], [5.0, 7.0, 7.0], [7.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], [7.0, 1.0, 7.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 
6.0], [7.0, 7.0, 8.0], [1.0, 1.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 8.0, 2.0], [2.0, 3.0, 4.0]], [[1.0, 7.0, 5.0], [1.0, 7.0, 8.0]]])
      arg[2:,1,:2]=val
      ref=Data(numpy.array([[[[5.0, 2.0, 4.0], [2.0, 2.0, 8.0], [5.0, 2.0, 6.0]], [[1.0, 5.0, 8.0], [7.0, 8.0, 7.0], [3.0, 1.0, 
4.0]], [[4.0, 3.0, 4.0], [7.0, 6.0, 5.0], [5.0, 5.0, 7.0]]], [[[4.0, 5.0, 1.0], [6.0, 1.0, 1.0], [1.0, 2.0, 3.0]], [[4.0, 1.0, 
4.0], [8.0, 5.0, 2.0], [2.0, 3.0, 6.0]], [[5.0, 5.0, 4.0], [8.0, 6.0, 8.0], [2.0, 4.0, 2.0]]], [[[3.0, 2.0, 4.0], [6.0, 2.0, 
5.0], [5.0, 4.0, 1.0]], [[5.0, 8.0, 2.0], [2.0, 3.0, 4.0], [3.0, 3.0, 3.0]], [[2.0, 3.0, 1.0], [2.0, 7.0, 4.0], [7.0, 8.0, 
3.0]]], [[[1.0, 4.0, 4.0], [5.0, 7.0, 7.0], [7.0, 6.0, 2.0]], [[1.0, 7.0, 5.0], [1.0, 7.0, 8.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 
6.0], [7.0, 7.0, 8.0], [1.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 8.0], [6.0, 5.0, 3.0], [5.0, 2.0, 8.0]], [[7.0, 6.0, 7.0], [6.0, 2.0, 1.0], [8.0, 1.0, 
1.0]], [[4.0, 4.0, 2.0], [4.0, 3.0, 7.0], [5.0, 8.0, 8.0]]], [[[4.0, 2.0, 1.0], [4.0, 6.0, 8.0], [8.0, 6.0, 8.0]], [[4.0, 7.0, 
1.0], [8.0, 3.0, 3.0], [1.0, 2.0, 5.0]], [[7.0, 1.0, 7.0], [1.0, 6.0, 4.0], [6.0, 1.0, 6.0]]], [[[5.0, 5.0, 8.0], [1.0, 6.0, 
7.0], [6.0, 3.0, 8.0]], [[3.0, 7.0, 1.0], [3.0, 4.0, 7.0], [4.0, 4.0, 6.0]], [[4.0, 1.0, 7.0], [6.0, 4.0, 2.0], [6.0, 1.0, 
7.0]]], [[[1.0, 8.0, 6.0], [3.0, 1.0, 6.0], [1.0, 3.0, 7.0]], [[3.0, 7.0, 4.0], [4.0, 5.0, 6.0], [3.0, 4.0, 2.0]], [[3.0, 6.0, 
6.0], [7.0, 5.0, 6.0], [1.0, 1.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 4.0, 2.0], [4.0, 3.0, 1.0]], [[6.0, 6.0, 5.0], [1.0, 5.0, 3.0]]]),self.functionspace)
      arg[2:,1,:2]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 8.0], [6.0, 5.0, 3.0], [5.0, 2.0, 8.0]], [[7.0, 6.0, 7.0], [6.0, 2.0, 1.0], [8.0, 1.0, 
1.0]], [[4.0, 4.0, 2.0], [4.0, 3.0, 7.0], [5.0, 8.0, 8.0]]], [[[4.0, 2.0, 1.0], [4.0, 6.0, 8.0], [8.0, 6.0, 8.0]], [[4.0, 7.0, 
1.0], [8.0, 3.0, 3.0], [1.0, 2.0, 5.0]], [[7.0, 1.0, 7.0], [1.0, 6.0, 4.0], [6.0, 1.0, 6.0]]], [[[5.0, 5.0, 8.0], [1.0, 6.0, 
7.0], [6.0, 3.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 3.0, 1.0], [4.0, 4.0, 6.0]], [[4.0, 1.0, 7.0], [6.0, 4.0, 2.0], [6.0, 1.0, 
7.0]]], [[[1.0, 8.0, 6.0], [3.0, 1.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 5.0], [1.0, 5.0, 3.0], [3.0, 4.0, 2.0]], [[3.0, 6.0, 
6.0], [7.0, 5.0, 6.0], [1.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPUN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 4.0], [5.0, 8.0, 8.0], [3.0, 4.0, 7.0]], [[7.0, 7.0, 8.0], [4.0, 3.0, 3.0], [2.0, 3.0, 
5.0]], [[8.0, 8.0, 6.0], [8.0, 6.0, 2.0], [8.0, 3.0, 3.0]]], [[[5.0, 3.0, 5.0], [1.0, 2.0, 1.0], [6.0, 4.0, 3.0]], [[3.0, 7.0, 
3.0], [1.0, 1.0, 3.0], [2.0, 8.0, 1.0]], [[6.0, 2.0, 4.0], [1.0, 5.0, 3.0], [2.0, 7.0, 5.0]]], [[[8.0, 4.0, 3.0], [7.0, 1.0, 
5.0], [6.0, 7.0, 8.0]], [[5.0, 2.0, 6.0], [6.0, 2.0, 5.0], [1.0, 8.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 2.0, 4.0], [3.0, 5.0, 
6.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 3.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 
4.0], [7.0, 2.0, 1.0], [1.0, 5.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 5.0, 4.0], [2.0, 7.0, 3.0]], [[7.0, 2.0, 1.0], [8.0, 8.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[6.0, 3.0, 1.0], [8.0, 2.0, 4.0]]])
      arg[2:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 4.0], [5.0, 8.0, 8.0], [3.0, 4.0, 7.0]], [[7.0, 7.0, 8.0], [4.0, 3.0, 3.0], [2.0, 
3.0, 5.0]], [[8.0, 8.0, 6.0], [8.0, 6.0, 2.0], [8.0, 3.0, 3.0]]], [[[5.0, 3.0, 5.0], [1.0, 2.0, 1.0], [6.0, 4.0, 3.0]], [[3.0, 
7.0, 3.0], [1.0, 1.0, 3.0], [2.0, 8.0, 1.0]], [[6.0, 2.0, 4.0], [1.0, 5.0, 3.0], [2.0, 7.0, 5.0]]], [[[8.0, 4.0, 3.0], [7.0, 
1.0, 5.0], [6.0, 7.0, 8.0]], [[7.0, 5.0, 4.0], [2.0, 7.0, 3.0], [1.0, 8.0, 4.0]], [[1.0, 3.0, 8.0], [2.0, 2.0, 4.0], [3.0, 5.0, 
6.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[7.0, 2.0, 1.0], [8.0, 8.0, 4.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 
4.0], [7.0, 2.0, 1.0], [1.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 4.0], [5.0, 8.0, 8.0], [3.0, 4.0, 7.0]], 
[[7.0, 7.0, 8.0], [4.0, 3.0, 3.0], [2.0, 3.0, 5.0]], [[8.0, 8.0, 6.0], [8.0, 6.0, 2.0], [8.0, 3.0, 3.0]]], [[[5.0, 3.0, 5.0], 
[1.0, 2.0, 1.0], [6.0, 4.0, 3.0]], [[3.0, 7.0, 3.0], [1.0, 1.0, 3.0], [2.0, 8.0, 1.0]], [[6.0, 2.0, 4.0], [1.0, 5.0, 3.0], 
[2.0, 7.0, 5.0]]], [[[8.0, 4.0, 3.0], [7.0, 1.0, 5.0], [6.0, 7.0, 8.0]], [[7.0, 3.0, 6.0], [6.0, 1.0, 2.0], [1.0, 8.0, 4.0]], 
[[1.0, 3.0, 8.0], [2.0, 2.0, 4.0], [3.0, 5.0, 6.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[6.0, 3.0, 1.0], 
[8.0, 2.0, 4.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 4.0], [7.0, 2.0, 1.0], [1.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCCN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 2.0], [1.0, 3.0, 4.0], [7.0, 7.0, 3.0]], [[3.0, 5.0, 4.0], [7.0, 8.0, 6.0], [4.0, 6.0, 
7.0]], [[2.0, 3.0, 2.0], [6.0, 5.0, 1.0], [8.0, 4.0, 4.0]]], [[[8.0, 4.0, 2.0], [7.0, 1.0, 3.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 
2.0], [6.0, 2.0, 1.0], [8.0, 1.0, 2.0]], [[2.0, 6.0, 4.0], [1.0, 8.0, 8.0], [1.0, 8.0, 4.0]]], [[[6.0, 8.0, 1.0], [2.0, 7.0, 
4.0], [7.0, 2.0, 3.0]], [[1.0, 5.0, 1.0], [2.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[2.0, 6.0, 6.0], [1.0, 4.0, 1.0], [4.0, 7.0, 
1.0]]], [[[5.0, 7.0, 8.0], [6.0, 1.0, 3.0], [6.0, 4.0, 7.0]], [[5.0, 1.0, 4.0], [8.0, 3.0, 3.0], [1.0, 4.0, 8.0]], [[7.0, 3.0, 
1.0], [4.0, 3.0, 6.0], [6.0, 1.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 7.0, 8.0], [2.0, 8.0, 3.0], [2.0, 5.0, 4.0]], [[6.0, 1.0, 6.0], [7.0, 1.0, 5.0], [2.0, 4.0, 
1.0]], [[4.0, 6.0, 2.0], [2.0, 3.0, 6.0], [7.0, 7.0, 7.0]]])
      arg[2,:,:]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 2.0], [1.0, 3.0, 4.0], [7.0, 7.0, 3.0]], [[3.0, 5.0, 4.0], [7.0, 8.0, 6.0], [4.0, 6.0, 
7.0]], [[2.0, 3.0, 2.0], [6.0, 5.0, 1.0], [8.0, 4.0, 4.0]]], [[[8.0, 4.0, 2.0], [7.0, 1.0, 3.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 
2.0], [6.0, 2.0, 1.0], [8.0, 1.0, 2.0]], [[2.0, 6.0, 4.0], [1.0, 8.0, 8.0], [1.0, 8.0, 4.0]]], [[[4.0, 7.0, 8.0], [2.0, 8.0, 
3.0], [2.0, 5.0, 4.0]], [[6.0, 1.0, 6.0], [7.0, 1.0, 5.0], [2.0, 4.0, 1.0]], [[4.0, 6.0, 2.0], [2.0, 3.0, 6.0], [7.0, 7.0, 
7.0]]], [[[5.0, 7.0, 8.0], [6.0, 1.0, 3.0], [6.0, 4.0, 7.0]], [[5.0, 1.0, 4.0], [8.0, 3.0, 3.0], [1.0, 4.0, 8.0]], [[7.0, 3.0, 
1.0], [4.0, 3.0, 6.0], [6.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCCN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 4.0], [2.0, 6.0, 4.0], [6.0, 3.0, 6.0]], [[1.0, 8.0, 5.0], [3.0, 6.0, 6.0], [1.0, 2.0, 
6.0]], [[3.0, 1.0, 6.0], [3.0, 5.0, 2.0], [5.0, 8.0, 8.0]]], [[[6.0, 8.0, 5.0], [3.0, 1.0, 8.0], [6.0, 5.0, 8.0]], [[4.0, 7.0, 
2.0], [8.0, 2.0, 4.0], [1.0, 3.0, 5.0]], [[4.0, 1.0, 5.0], [6.0, 7.0, 2.0], [5.0, 7.0, 5.0]]], [[[5.0, 6.0, 2.0], [7.0, 2.0, 
1.0], [1.0, 5.0, 3.0]], [[4.0, 2.0, 5.0], [6.0, 4.0, 3.0], [5.0, 8.0, 6.0]], [[4.0, 8.0, 7.0], [3.0, 6.0, 7.0], [6.0, 1.0, 
4.0]]], [[[2.0, 8.0, 5.0], [2.0, 4.0, 8.0], [7.0, 8.0, 4.0]], [[3.0, 2.0, 3.0], [7.0, 6.0, 6.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 
1.0], [7.0, 5.0, 1.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 4.0], [8.0, 1.0, 3.0], [7.0, 1.0, 2.0]], [[3.0, 8.0, 2.0], [7.0, 3.0, 5.0], [5.0, 4.0, 
7.0]], [[2.0, 7.0, 3.0], [2.0, 1.0, 7.0], [6.0, 5.0, 4.0]]]),self.functionspace)
      arg[2,:,:]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 4.0], [2.0, 6.0, 4.0], [6.0, 3.0, 6.0]], [[1.0, 8.0, 5.0], [3.0, 6.0, 6.0], [1.0, 2.0, 
6.0]], [[3.0, 1.0, 6.0], [3.0, 5.0, 2.0], [5.0, 8.0, 8.0]]], [[[6.0, 8.0, 5.0], [3.0, 1.0, 8.0], [6.0, 5.0, 8.0]], [[4.0, 7.0, 
2.0], [8.0, 2.0, 4.0], [1.0, 3.0, 5.0]], [[4.0, 1.0, 5.0], [6.0, 7.0, 2.0], [5.0, 7.0, 5.0]]], [[[4.0, 7.0, 4.0], [8.0, 1.0, 
3.0], [7.0, 1.0, 2.0]], [[3.0, 8.0, 2.0], [7.0, 3.0, 5.0], [5.0, 4.0, 7.0]], [[2.0, 7.0, 3.0], [2.0, 1.0, 7.0], [6.0, 5.0, 
4.0]]], [[[2.0, 8.0, 5.0], [2.0, 4.0, 8.0], [7.0, 8.0, 4.0]], [[3.0, 2.0, 3.0], [7.0, 6.0, 6.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 
1.0], [7.0, 5.0, 1.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCCN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 2.0], [4.0, 5.0, 3.0], [2.0, 6.0, 4.0]], [[1.0, 7.0, 8.0], [2.0, 5.0, 3.0], [7.0, 6.0, 
2.0]], [[4.0, 2.0, 6.0], [1.0, 2.0, 2.0], [4.0, 8.0, 6.0]]], [[[3.0, 4.0, 1.0], [8.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[2.0, 6.0, 
6.0], [5.0, 8.0, 3.0], [3.0, 5.0, 4.0]], [[4.0, 1.0, 2.0], [7.0, 5.0, 7.0], [2.0, 4.0, 3.0]]], [[[6.0, 3.0, 5.0], [7.0, 1.0, 
6.0], [6.0, 4.0, 5.0]], [[3.0, 3.0, 5.0], [7.0, 2.0, 8.0], [3.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], [4.0, 5.0, 8.0], [7.0, 1.0, 
8.0]]], [[[3.0, 4.0, 8.0], [3.0, 2.0, 5.0], [7.0, 6.0, 1.0]], [[1.0, 2.0, 5.0], [1.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[6.0, 2.0, 
5.0], [6.0, 6.0, 3.0], [2.0, 1.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 2.0, 5.0], [4.0, 1.0, 6.0], [7.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [2.0, 3.0, 6.0], [4.0, 
3.0, 6.0]], [[4.0, 7.0, 5.0], [6.0, 8.0, 2.0], [8.0, 1.0, 5.0]]])+(1.-msk_val)*numpy.array([[[1.0, 7.0, 7.0], [1.0, 7.0, 3.0], 
[3.0, 5.0, 7.0]], [[1.0, 3.0, 4.0], [1.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[1.0, 4.0, 6.0], [3.0, 2.0, 6.0], [1.0, 6.0, 7.0]]])
      arg[2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 2.0], [4.0, 5.0, 3.0], [2.0, 6.0, 4.0]], [[1.0, 7.0, 8.0], [2.0, 5.0, 3.0], [7.0, 
6.0, 2.0]], [[4.0, 2.0, 6.0], [1.0, 2.0, 2.0], [4.0, 8.0, 6.0]]], [[[3.0, 4.0, 1.0], [8.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[2.0, 
6.0, 6.0], [5.0, 8.0, 3.0], [3.0, 5.0, 4.0]], [[4.0, 1.0, 2.0], [7.0, 5.0, 7.0], [2.0, 4.0, 3.0]]], [[[4.0, 2.0, 5.0], [4.0, 
1.0, 6.0], [7.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [2.0, 3.0, 6.0], [4.0, 3.0, 6.0]], [[4.0, 7.0, 5.0], [6.0, 8.0, 2.0], [8.0, 1.0, 
5.0]]], [[[3.0, 4.0, 8.0], [3.0, 2.0, 5.0], [7.0, 6.0, 1.0]], [[1.0, 2.0, 5.0], [1.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[6.0, 2.0, 
5.0], [6.0, 6.0, 3.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 2.0], [4.0, 5.0, 3.0], [2.0, 6.0, 4.0]], 
[[1.0, 7.0, 8.0], [2.0, 5.0, 3.0], [7.0, 6.0, 2.0]], [[4.0, 2.0, 6.0], [1.0, 2.0, 2.0], [4.0, 8.0, 6.0]]], [[[3.0, 4.0, 1.0], 
[8.0, 2.0, 4.0], [3.0, 1.0, 3.0]], [[2.0, 6.0, 6.0], [5.0, 8.0, 3.0], [3.0, 5.0, 4.0]], [[4.0, 1.0, 2.0], [7.0, 5.0, 7.0], 
[2.0, 4.0, 3.0]]], [[[1.0, 7.0, 7.0], [1.0, 7.0, 3.0], [3.0, 5.0, 7.0]], [[1.0, 3.0, 4.0], [1.0, 8.0, 4.0], [3.0, 6.0, 7.0]], 
[[1.0, 4.0, 6.0], [3.0, 2.0, 6.0], [1.0, 6.0, 7.0]]], [[[3.0, 4.0, 8.0], [3.0, 2.0, 5.0], [7.0, 6.0, 1.0]], [[1.0, 2.0, 5.0], 
[1.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[6.0, 2.0, 5.0], [6.0, 6.0, 3.0], [2.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CRUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 7.0], [2.0, 4.0, 5.0], [1.0, 1.0, 4.0]], [[2.0, 3.0, 2.0], [3.0, 1.0, 6.0], [4.0, 5.0, 
5.0]], [[8.0, 7.0, 3.0], [4.0, 5.0, 7.0], [6.0, 7.0, 7.0]]], [[[2.0, 5.0, 1.0], [2.0, 7.0, 8.0], [7.0, 8.0, 8.0]], [[3.0, 7.0, 
1.0], [4.0, 5.0, 3.0], [1.0, 4.0, 2.0]], [[5.0, 4.0, 8.0], [5.0, 1.0, 1.0], [2.0, 1.0, 6.0]]], [[[7.0, 4.0, 4.0], [8.0, 6.0, 
3.0], [8.0, 5.0, 8.0]], [[4.0, 6.0, 2.0], [5.0, 7.0, 1.0], [4.0, 3.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 1.0, 6.0], [5.0, 8.0, 
8.0]]], [[[4.0, 5.0, 3.0], [3.0, 7.0, 5.0], [8.0, 2.0, 5.0]], [[4.0, 2.0, 4.0], [2.0, 8.0, 4.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 
1.0], [3.0, 6.0, 2.0], [8.0, 4.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 4.0, 6.0], [4.0, 1.0, 4.0]]], [[[7.0, 2.0, 2.0], [8.0, 3.0, 2.0]]], [[[3.0, 8.0, 3.0], [6.0, 
7.0, 2.0]]], [[[2.0, 2.0, 5.0], [3.0, 7.0, 8.0]]]])
      arg[:,1:2,:2]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 7.0], [2.0, 4.0, 5.0], [1.0, 1.0, 4.0]], [[6.0, 4.0, 6.0], [4.0, 1.0, 4.0], [4.0, 5.0, 
5.0]], [[8.0, 7.0, 3.0], [4.0, 5.0, 7.0], [6.0, 7.0, 7.0]]], [[[2.0, 5.0, 1.0], [2.0, 7.0, 8.0], [7.0, 8.0, 8.0]], [[7.0, 2.0, 
2.0], [8.0, 3.0, 2.0], [1.0, 4.0, 2.0]], [[5.0, 4.0, 8.0], [5.0, 1.0, 1.0], [2.0, 1.0, 6.0]]], [[[7.0, 4.0, 4.0], [8.0, 6.0, 
3.0], [8.0, 5.0, 8.0]], [[3.0, 8.0, 3.0], [6.0, 7.0, 2.0], [4.0, 3.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 1.0, 6.0], [5.0, 8.0, 
8.0]]], [[[4.0, 5.0, 3.0], [3.0, 7.0, 5.0], [8.0, 2.0, 5.0]], [[2.0, 2.0, 5.0], [3.0, 7.0, 8.0], [8.0, 1.0, 5.0]], [[3.0, 2.0, 
1.0], [3.0, 6.0, 2.0], [8.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CRUN(self):
      arg=Data(numpy.array([[[[3.0, 6.0, 2.0], [5.0, 8.0, 8.0], [3.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 3.0, 6.0], [2.0, 1.0, 
6.0]], [[2.0, 4.0, 1.0], [4.0, 2.0, 1.0], [5.0, 8.0, 8.0]]], [[[3.0, 8.0, 6.0], [3.0, 7.0, 2.0], [5.0, 3.0, 4.0]], [[3.0, 5.0, 
4.0], [4.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[8.0, 7.0, 5.0], [8.0, 1.0, 6.0], [5.0, 3.0, 3.0]]], [[[4.0, 8.0, 3.0], [5.0, 8.0, 
2.0], [3.0, 8.0, 7.0]], [[2.0, 3.0, 4.0], [4.0, 3.0, 6.0], [7.0, 6.0, 7.0]], [[8.0, 1.0, 4.0], [1.0, 7.0, 6.0], [4.0, 7.0, 
4.0]]], [[[4.0, 8.0, 3.0], [3.0, 6.0, 4.0], [7.0, 1.0, 8.0]], [[4.0, 5.0, 4.0], [1.0, 5.0, 1.0], [1.0, 4.0, 7.0]], [[1.0, 3.0, 
4.0], [7.0, 6.0, 6.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 8.0, 2.0], [3.0, 5.0, 8.0]]], [[[6.0, 3.0, 2.0], [3.0, 7.0, 6.0]]], [[[2.0, 8.0, 3.0], 
[6.0, 1.0, 7.0]]], [[[8.0, 4.0, 3.0], [8.0, 5.0, 4.0]]]]),self.functionspace)
      arg[:,1:2,:2]=val
      ref=Data(numpy.array([[[[3.0, 6.0, 2.0], [5.0, 8.0, 8.0], [3.0, 6.0, 7.0]], [[3.0, 8.0, 2.0], [3.0, 5.0, 8.0], [2.0, 1.0, 
6.0]], [[2.0, 4.0, 1.0], [4.0, 2.0, 1.0], [5.0, 8.0, 8.0]]], [[[3.0, 8.0, 6.0], [3.0, 7.0, 2.0], [5.0, 3.0, 4.0]], [[6.0, 3.0, 
2.0], [3.0, 7.0, 6.0], [5.0, 7.0, 3.0]], [[8.0, 7.0, 5.0], [8.0, 1.0, 6.0], [5.0, 3.0, 3.0]]], [[[4.0, 8.0, 3.0], [5.0, 8.0, 
2.0], [3.0, 8.0, 7.0]], [[2.0, 8.0, 3.0], [6.0, 1.0, 7.0], [7.0, 6.0, 7.0]], [[8.0, 1.0, 4.0], [1.0, 7.0, 6.0], [4.0, 7.0, 
4.0]]], [[[4.0, 8.0, 3.0], [3.0, 6.0, 4.0], [7.0, 1.0, 8.0]], [[8.0, 4.0, 3.0], [8.0, 5.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 3.0, 
4.0], [7.0, 6.0, 6.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CRUN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 8.0], [7.0, 4.0, 2.0], [4.0, 1.0, 7.0]], [[4.0, 5.0, 7.0], [2.0, 3.0, 4.0], [8.0, 3.0, 
5.0]], [[3.0, 8.0, 1.0], [5.0, 7.0, 6.0], [5.0, 1.0, 6.0]]], [[[5.0, 3.0, 8.0], [4.0, 6.0, 4.0], [8.0, 4.0, 3.0]], [[1.0, 6.0, 
5.0], [5.0, 6.0, 6.0], [6.0, 8.0, 6.0]], [[7.0, 3.0, 2.0], [7.0, 1.0, 2.0], [7.0, 1.0, 4.0]]], [[[1.0, 3.0, 5.0], [8.0, 5.0, 
8.0], [2.0, 6.0, 3.0]], [[5.0, 5.0, 6.0], [5.0, 8.0, 1.0], [3.0, 6.0, 1.0]], [[8.0, 2.0, 2.0], [7.0, 2.0, 6.0], [7.0, 4.0, 
4.0]]], [[[7.0, 1.0, 5.0], [7.0, 5.0, 1.0], [1.0, 3.0, 1.0]], [[3.0, 4.0, 3.0], [1.0, 8.0, 7.0], [7.0, 8.0, 1.0]], [[2.0, 5.0, 
5.0], [2.0, 1.0, 8.0], [6.0, 2.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 1.0, 3.0], [5.0, 8.0, 8.0]]], [[[3.0, 3.0, 4.0], [1.0, 7.0, 3.0]]], [[[5.0, 3.0, 5.0], 
[3.0, 5.0, 1.0]]], [[[2.0, 1.0, 7.0], [7.0, 8.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 6.0, 8.0], [5.0, 3.0, 4.0]]], 
[[[1.0, 6.0, 6.0], [6.0, 6.0, 8.0]]], [[[2.0, 7.0, 7.0], [7.0, 5.0, 8.0]]], [[[3.0, 3.0, 4.0], [8.0, 4.0, 4.0]]]])
      arg[:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 8.0], [7.0, 4.0, 2.0], [4.0, 1.0, 7.0]], [[8.0, 1.0, 3.0], [5.0, 8.0, 8.0], [8.0, 
3.0, 5.0]], [[3.0, 8.0, 1.0], [5.0, 7.0, 6.0], [5.0, 1.0, 6.0]]], [[[5.0, 3.0, 8.0], [4.0, 6.0, 4.0], [8.0, 4.0, 3.0]], [[3.0, 
3.0, 4.0], [1.0, 7.0, 3.0], [6.0, 8.0, 6.0]], [[7.0, 3.0, 2.0], [7.0, 1.0, 2.0], [7.0, 1.0, 4.0]]], [[[1.0, 3.0, 5.0], [8.0, 
5.0, 8.0], [2.0, 6.0, 3.0]], [[5.0, 3.0, 5.0], [3.0, 5.0, 1.0], [3.0, 6.0, 1.0]], [[8.0, 2.0, 2.0], [7.0, 2.0, 6.0], [7.0, 4.0, 
4.0]]], [[[7.0, 1.0, 5.0], [7.0, 5.0, 1.0], [1.0, 3.0, 1.0]], [[2.0, 1.0, 7.0], [7.0, 8.0, 6.0], [7.0, 8.0, 1.0]], [[2.0, 5.0, 
5.0], [2.0, 1.0, 8.0], [6.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 8.0], [7.0, 4.0, 2.0], [4.0, 1.0, 7.0]], 
[[5.0, 6.0, 8.0], [5.0, 3.0, 4.0], [8.0, 3.0, 5.0]], [[3.0, 8.0, 1.0], [5.0, 7.0, 6.0], [5.0, 1.0, 6.0]]], [[[5.0, 3.0, 8.0], 
[4.0, 6.0, 4.0], [8.0, 4.0, 3.0]], [[1.0, 6.0, 6.0], [6.0, 6.0, 8.0], [6.0, 8.0, 6.0]], [[7.0, 3.0, 2.0], [7.0, 1.0, 2.0], 
[7.0, 1.0, 4.0]]], [[[1.0, 3.0, 5.0], [8.0, 5.0, 8.0], [2.0, 6.0, 3.0]], [[2.0, 7.0, 7.0], [7.0, 5.0, 8.0], [3.0, 6.0, 1.0]], 
[[8.0, 2.0, 2.0], [7.0, 2.0, 6.0], [7.0, 4.0, 4.0]]], [[[7.0, 1.0, 5.0], [7.0, 5.0, 1.0], [1.0, 3.0, 1.0]], [[3.0, 3.0, 4.0], 
[8.0, 4.0, 4.0], [7.0, 8.0, 1.0]], [[2.0, 5.0, 5.0], [2.0, 1.0, 8.0], [6.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPPN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 2.0], [6.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[5.0, 5.0, 7.0], [4.0, 3.0, 8.0], [4.0, 5.0, 
5.0]], [[2.0, 7.0, 5.0], [7.0, 4.0, 7.0], [4.0, 7.0, 4.0]]], [[[2.0, 1.0, 3.0], [5.0, 7.0, 5.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 
8.0], [3.0, 8.0, 1.0], [7.0, 8.0, 4.0]], [[2.0, 6.0, 5.0], [5.0, 7.0, 3.0], [8.0, 7.0, 3.0]]], [[[7.0, 3.0, 8.0], [7.0, 3.0, 
7.0], [5.0, 1.0, 7.0]], [[8.0, 5.0, 5.0], [7.0, 4.0, 6.0], [4.0, 7.0, 5.0]], [[1.0, 8.0, 4.0], [3.0, 4.0, 7.0], [4.0, 6.0, 
4.0]]], [[[4.0, 2.0, 2.0], [7.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[2.0, 3.0, 2.0], [5.0, 1.0, 8.0], [3.0, 3.0, 1.0]], [[6.0, 6.0, 
5.0], [3.0, 4.0, 5.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[5.0, 5.0, 7.0], [8.0, 7.0, 7.0]])
      arg[2:,1,1]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 2.0], [6.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[5.0, 5.0, 7.0], [4.0, 3.0, 8.0], [4.0, 5.0, 
5.0]], [[2.0, 7.0, 5.0], [7.0, 4.0, 7.0], [4.0, 7.0, 4.0]]], [[[2.0, 1.0, 3.0], [5.0, 7.0, 5.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 
8.0], [3.0, 8.0, 1.0], [7.0, 8.0, 4.0]], [[2.0, 6.0, 5.0], [5.0, 7.0, 3.0], [8.0, 7.0, 3.0]]], [[[7.0, 3.0, 8.0], [7.0, 3.0, 
7.0], [5.0, 1.0, 7.0]], [[8.0, 5.0, 5.0], [5.0, 5.0, 7.0], [4.0, 7.0, 5.0]], [[1.0, 8.0, 4.0], [3.0, 4.0, 7.0], [4.0, 6.0, 
4.0]]], [[[4.0, 2.0, 2.0], [7.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[2.0, 3.0, 2.0], [8.0, 7.0, 7.0], [3.0, 3.0, 1.0]], [[6.0, 6.0, 
5.0], [3.0, 4.0, 5.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 7.0], [6.0, 8.0, 8.0], [5.0, 7.0, 2.0]], [[2.0, 7.0, 1.0], [4.0, 7.0, 5.0], [5.0, 4.0, 
2.0]], [[3.0, 8.0, 3.0], [4.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[6.0, 8.0, 3.0], [7.0, 2.0, 1.0], [7.0, 2.0, 4.0]], [[2.0, 2.0, 
7.0], [4.0, 2.0, 4.0], [5.0, 8.0, 5.0]], [[1.0, 1.0, 6.0], [6.0, 2.0, 3.0], [1.0, 5.0, 3.0]]], [[[3.0, 2.0, 3.0], [6.0, 1.0, 
4.0], [4.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [1.0, 7.0, 8.0], [3.0, 4.0, 8.0]], [[6.0, 6.0, 2.0], [5.0, 7.0, 7.0], [1.0, 2.0, 
2.0]]], [[[8.0, 3.0, 5.0], [1.0, 7.0, 4.0], [3.0, 5.0, 7.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 6.0], [6.0, 1.0, 8.0]], [[3.0, 5.0, 
4.0], [7.0, 5.0, 6.0], [5.0, 1.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[5.0, 8.0, 4.0], [5.0, 4.0, 3.0]]),self.functionspace)
      arg[2:,1,1]=val
      ref=Data(numpy.array([[[[3.0, 5.0, 7.0], [6.0, 8.0, 8.0], [5.0, 7.0, 2.0]], [[2.0, 7.0, 1.0], [4.0, 7.0, 5.0], [5.0, 4.0, 
2.0]], [[3.0, 8.0, 3.0], [4.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[6.0, 8.0, 3.0], [7.0, 2.0, 1.0], [7.0, 2.0, 4.0]], [[2.0, 2.0, 
7.0], [4.0, 2.0, 4.0], [5.0, 8.0, 5.0]], [[1.0, 1.0, 6.0], [6.0, 2.0, 3.0], [1.0, 5.0, 3.0]]], [[[3.0, 2.0, 3.0], [6.0, 1.0, 
4.0], [4.0, 4.0, 5.0]], [[1.0, 1.0, 1.0], [5.0, 8.0, 4.0], [3.0, 4.0, 8.0]], [[6.0, 6.0, 2.0], [5.0, 7.0, 7.0], [1.0, 2.0, 
2.0]]], [[[8.0, 3.0, 5.0], [1.0, 7.0, 4.0], [3.0, 5.0, 7.0]], [[4.0, 8.0, 8.0], [5.0, 4.0, 3.0], [6.0, 1.0, 8.0]], [[3.0, 5.0, 
4.0], [7.0, 5.0, 6.0], [5.0, 1.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPPN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 1.0], [3.0, 3.0, 5.0], [1.0, 5.0, 8.0]], [[8.0, 4.0, 5.0], [1.0, 1.0, 4.0], [8.0, 1.0, 
1.0]], [[8.0, 3.0, 4.0], [4.0, 1.0, 7.0], [7.0, 4.0, 4.0]]], [[[1.0, 2.0, 1.0], [3.0, 5.0, 3.0], [3.0, 5.0, 4.0]], [[5.0, 7.0, 
1.0], [1.0, 8.0, 2.0], [3.0, 5.0, 8.0]], [[6.0, 7.0, 1.0], [8.0, 4.0, 1.0], [6.0, 4.0, 3.0]]], [[[5.0, 8.0, 5.0], [6.0, 5.0, 
2.0], [8.0, 1.0, 5.0]], [[7.0, 4.0, 4.0], [5.0, 7.0, 6.0], [2.0, 3.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 4.0], [1.0, 4.0, 
8.0]]], [[[1.0, 2.0, 6.0], [6.0, 4.0, 6.0], [8.0, 8.0, 1.0]], [[3.0, 5.0, 1.0], [1.0, 7.0, 4.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 
6.0], [5.0, 5.0, 7.0], [3.0, 4.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 4.0, 4.0], [5.0, 7.0, 6.0]])+(1.-msk_val)*numpy.array([[2.0, 2.0, 1.0], [2.0, 8.0, 7.0]])
      arg[2:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 1.0], [3.0, 3.0, 5.0], [1.0, 5.0, 8.0]], [[8.0, 4.0, 5.0], [1.0, 1.0, 4.0], [8.0, 
1.0, 1.0]], [[8.0, 3.0, 4.0], [4.0, 1.0, 7.0], [7.0, 4.0, 4.0]]], [[[1.0, 2.0, 1.0], [3.0, 5.0, 3.0], [3.0, 5.0, 4.0]], [[5.0, 
7.0, 1.0], [1.0, 8.0, 2.0], [3.0, 5.0, 8.0]], [[6.0, 7.0, 1.0], [8.0, 4.0, 1.0], [6.0, 4.0, 3.0]]], [[[5.0, 8.0, 5.0], [6.0, 
5.0, 2.0], [8.0, 1.0, 5.0]], [[7.0, 4.0, 4.0], [7.0, 4.0, 4.0], [2.0, 3.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 4.0], [1.0, 4.0, 
8.0]]], [[[1.0, 2.0, 6.0], [6.0, 4.0, 6.0], [8.0, 8.0, 1.0]], [[3.0, 5.0, 1.0], [5.0, 7.0, 6.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 
6.0], [5.0, 5.0, 7.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 1.0], [3.0, 3.0, 5.0], [1.0, 5.0, 8.0]], 
[[8.0, 4.0, 5.0], [1.0, 1.0, 4.0], [8.0, 1.0, 1.0]], [[8.0, 3.0, 4.0], [4.0, 1.0, 7.0], [7.0, 4.0, 4.0]]], [[[1.0, 2.0, 1.0], 
[3.0, 5.0, 3.0], [3.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [1.0, 8.0, 2.0], [3.0, 5.0, 8.0]], [[6.0, 7.0, 1.0], [8.0, 4.0, 1.0], 
[6.0, 4.0, 3.0]]], [[[5.0, 8.0, 5.0], [6.0, 5.0, 2.0], [8.0, 1.0, 5.0]], [[7.0, 4.0, 4.0], [2.0, 2.0, 1.0], [2.0, 3.0, 8.0]], 
[[2.0, 3.0, 3.0], [5.0, 5.0, 4.0], [1.0, 4.0, 8.0]]], [[[1.0, 2.0, 6.0], [6.0, 4.0, 6.0], [8.0, 8.0, 1.0]], [[3.0, 5.0, 1.0], 
[2.0, 8.0, 7.0], [7.0, 8.0, 3.0]], [[5.0, 1.0, 6.0], [5.0, 5.0, 7.0], [3.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRUN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 8.0], [3.0, 5.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 7.0, 6.0], [7.0, 3.0, 7.0], [5.0, 1.0, 
4.0]], [[8.0, 8.0, 1.0], [3.0, 2.0, 3.0], [1.0, 5.0, 3.0]]], [[[3.0, 5.0, 7.0], [6.0, 4.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 3.0, 
6.0], [6.0, 2.0, 8.0], [1.0, 8.0, 2.0]], [[6.0, 2.0, 3.0], [3.0, 2.0, 8.0], [1.0, 7.0, 8.0]]], [[[8.0, 5.0, 4.0], [1.0, 6.0, 
3.0], [4.0, 5.0, 5.0]], [[4.0, 5.0, 7.0], [2.0, 5.0, 3.0], [8.0, 4.0, 6.0]], [[7.0, 2.0, 6.0], [4.0, 7.0, 2.0], [5.0, 2.0, 
2.0]]], [[[6.0, 2.0, 3.0], [7.0, 1.0, 2.0], [6.0, 7.0, 3.0]], [[8.0, 1.0, 2.0], [3.0, 2.0, 5.0], [5.0, 1.0, 1.0]], [[1.0, 8.0, 
6.0], [3.0, 4.0, 3.0], [2.0, 3.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 4.0, 2.0], [1.0, 1.0, 4.0]]])
      arg[2,1:2,:2]=val
      ref=Data(numpy.array([[[[4.0, 2.0, 8.0], [3.0, 5.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 7.0, 6.0], [7.0, 3.0, 7.0], [5.0, 1.0, 
4.0]], [[8.0, 8.0, 1.0], [3.0, 2.0, 3.0], [1.0, 5.0, 3.0]]], [[[3.0, 5.0, 7.0], [6.0, 4.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 3.0, 
6.0], [6.0, 2.0, 8.0], [1.0, 8.0, 2.0]], [[6.0, 2.0, 3.0], [3.0, 2.0, 8.0], [1.0, 7.0, 8.0]]], [[[8.0, 5.0, 4.0], [1.0, 6.0, 
3.0], [4.0, 5.0, 5.0]], [[4.0, 4.0, 2.0], [1.0, 1.0, 4.0], [8.0, 4.0, 6.0]], [[7.0, 2.0, 6.0], [4.0, 7.0, 2.0], [5.0, 2.0, 
2.0]]], [[[6.0, 2.0, 3.0], [7.0, 1.0, 2.0], [6.0, 7.0, 3.0]], [[8.0, 1.0, 2.0], [3.0, 2.0, 5.0], [5.0, 1.0, 1.0]], [[1.0, 8.0, 
6.0], [3.0, 4.0, 3.0], [2.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRUN(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 8.0], [3.0, 7.0, 2.0], [2.0, 8.0, 4.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 1.0], [1.0, 2.0, 
8.0]], [[1.0, 2.0, 8.0], [6.0, 6.0, 3.0], [7.0, 2.0, 7.0]]], [[[5.0, 5.0, 8.0], [3.0, 6.0, 1.0], [4.0, 1.0, 5.0]], [[3.0, 8.0, 
6.0], [4.0, 8.0, 8.0], [4.0, 3.0, 6.0]], [[5.0, 2.0, 3.0], [3.0, 8.0, 7.0], [2.0, 4.0, 7.0]]], [[[3.0, 4.0, 5.0], [5.0, 3.0, 
5.0], [8.0, 1.0, 6.0]], [[6.0, 3.0, 3.0], [8.0, 5.0, 7.0], [8.0, 5.0, 6.0]], [[1.0, 4.0, 7.0], [4.0, 6.0, 1.0], [2.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [8.0, 7.0, 3.0], [8.0, 1.0, 2.0]], [[6.0, 7.0, 2.0], [6.0, 6.0, 1.0], [6.0, 6.0, 3.0]], [[7.0, 7.0, 
7.0], [4.0, 6.0, 1.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 1.0, 8.0], [3.0, 6.0, 6.0]]]),self.functionspace)
      arg[2,1:2,:2]=val
      ref=Data(numpy.array([[[[1.0, 3.0, 8.0], [3.0, 7.0, 2.0], [2.0, 8.0, 4.0]], [[5.0, 2.0, 7.0], [4.0, 2.0, 1.0], [1.0, 2.0, 
8.0]], [[1.0, 2.0, 8.0], [6.0, 6.0, 3.0], [7.0, 2.0, 7.0]]], [[[5.0, 5.0, 8.0], [3.0, 6.0, 1.0], [4.0, 1.0, 5.0]], [[3.0, 8.0, 
6.0], [4.0, 8.0, 8.0], [4.0, 3.0, 6.0]], [[5.0, 2.0, 3.0], [3.0, 8.0, 7.0], [2.0, 4.0, 7.0]]], [[[3.0, 4.0, 5.0], [5.0, 3.0, 
5.0], [8.0, 1.0, 6.0]], [[7.0, 1.0, 8.0], [3.0, 6.0, 6.0], [8.0, 5.0, 6.0]], [[1.0, 4.0, 7.0], [4.0, 6.0, 1.0], [2.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [8.0, 7.0, 3.0], [8.0, 1.0, 2.0]], [[6.0, 7.0, 2.0], [6.0, 6.0, 1.0], [6.0, 6.0, 3.0]], [[7.0, 7.0, 
7.0], [4.0, 6.0, 1.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRUN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 7.0], [5.0, 1.0, 2.0], [8.0, 2.0, 3.0]], [[5.0, 7.0, 1.0], [3.0, 7.0, 7.0], [3.0, 1.0, 
2.0]], [[7.0, 1.0, 6.0], [5.0, 7.0, 8.0], [3.0, 2.0, 7.0]]], [[[6.0, 4.0, 6.0], [3.0, 7.0, 5.0], [7.0, 1.0, 3.0]], [[8.0, 1.0, 
2.0], [1.0, 7.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 7.0, 6.0], [8.0, 4.0, 7.0], [8.0, 2.0, 2.0]]], [[[4.0, 2.0, 7.0], [5.0, 8.0, 
4.0], [7.0, 7.0, 7.0]], [[7.0, 3.0, 4.0], [5.0, 8.0, 4.0], [2.0, 1.0, 2.0]], [[3.0, 4.0, 1.0], [3.0, 7.0, 1.0], [4.0, 1.0, 
3.0]]], [[[7.0, 4.0, 6.0], [3.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[6.0, 4.0, 2.0], [7.0, 2.0, 5.0], [1.0, 4.0, 2.0]], [[3.0, 7.0, 
4.0], [8.0, 6.0, 6.0], [3.0, 7.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 6.0, 7.0], [4.0, 4.0, 7.0]]])+(1.-msk_val)*numpy.array([[[1.0, 8.0, 8.0], [5.0, 2.0, 
8.0]]])
      arg[2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 7.0], [5.0, 1.0, 2.0], [8.0, 2.0, 3.0]], [[5.0, 7.0, 1.0], [3.0, 7.0, 7.0], [3.0, 
1.0, 2.0]], [[7.0, 1.0, 6.0], [5.0, 7.0, 8.0], [3.0, 2.0, 7.0]]], [[[6.0, 4.0, 6.0], [3.0, 7.0, 5.0], [7.0, 1.0, 3.0]], [[8.0, 
1.0, 2.0], [1.0, 7.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 7.0, 6.0], [8.0, 4.0, 7.0], [8.0, 2.0, 2.0]]], [[[4.0, 2.0, 7.0], [5.0, 
8.0, 4.0], [7.0, 7.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 4.0, 7.0], [2.0, 1.0, 2.0]], [[3.0, 4.0, 1.0], [3.0, 7.0, 1.0], [4.0, 1.0, 
3.0]]], [[[7.0, 4.0, 6.0], [3.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[6.0, 4.0, 2.0], [7.0, 2.0, 5.0], [1.0, 4.0, 2.0]], [[3.0, 7.0, 
4.0], [8.0, 6.0, 6.0], [3.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [5.0, 1.0, 2.0], [8.0, 2.0, 3.0]], 
[[5.0, 7.0, 1.0], [3.0, 7.0, 7.0], [3.0, 1.0, 2.0]], [[7.0, 1.0, 6.0], [5.0, 7.0, 8.0], [3.0, 2.0, 7.0]]], [[[6.0, 4.0, 6.0], 
[3.0, 7.0, 5.0], [7.0, 1.0, 3.0]], [[8.0, 1.0, 2.0], [1.0, 7.0, 3.0], [7.0, 6.0, 5.0]], [[8.0, 7.0, 6.0], [8.0, 4.0, 7.0], 
[8.0, 2.0, 2.0]]], [[[4.0, 2.0, 7.0], [5.0, 8.0, 4.0], [7.0, 7.0, 7.0]], [[1.0, 8.0, 8.0], [5.0, 2.0, 8.0], [2.0, 1.0, 2.0]], 
[[3.0, 4.0, 1.0], [3.0, 7.0, 1.0], [4.0, 1.0, 3.0]]], [[[7.0, 4.0, 6.0], [3.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[6.0, 4.0, 2.0], 
[7.0, 2.0, 5.0], [1.0, 4.0, 2.0]], [[3.0, 7.0, 4.0], [8.0, 6.0, 6.0], [3.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UCPN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 3.0], [7.0, 3.0, 4.0], [3.0, 6.0, 1.0]], [[4.0, 4.0, 1.0], [6.0, 2.0, 5.0], [3.0, 6.0, 
3.0]], [[7.0, 7.0, 8.0], [4.0, 6.0, 1.0], [4.0, 5.0, 1.0]]], [[[4.0, 3.0, 4.0], [6.0, 7.0, 1.0], [6.0, 5.0, 4.0]], [[8.0, 5.0, 
4.0], [6.0, 4.0, 8.0], [3.0, 5.0, 8.0]], [[5.0, 2.0, 4.0], [5.0, 1.0, 2.0], [7.0, 8.0, 2.0]]], [[[6.0, 1.0, 2.0], [5.0, 8.0, 
5.0], [5.0, 3.0, 2.0]], [[6.0, 1.0, 5.0], [7.0, 1.0, 7.0], [7.0, 1.0, 4.0]], [[8.0, 1.0, 2.0], [7.0, 7.0, 7.0], [8.0, 6.0, 
7.0]]], [[[2.0, 4.0, 6.0], [4.0, 4.0, 1.0], [3.0, 2.0, 5.0]], [[1.0, 4.0, 8.0], [5.0, 3.0, 5.0], [5.0, 6.0, 2.0]], [[2.0, 6.0, 
1.0], [1.0, 7.0, 2.0], [3.0, 6.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 2.0, 5.0], [2.0, 8.0, 4.0], [8.0, 1.0, 7.0]], [[3.0, 6.0, 8.0], [1.0, 1.0, 2.0], [5.0, 6.0, 
7.0]], [[8.0, 4.0, 6.0], [2.0, 8.0, 7.0], [7.0, 6.0, 2.0]]])
      arg[:3,:,1]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 3.0], [7.0, 2.0, 5.0], [3.0, 6.0, 1.0]], [[4.0, 4.0, 1.0], [2.0, 8.0, 4.0], [3.0, 6.0, 
3.0]], [[7.0, 7.0, 8.0], [8.0, 1.0, 7.0], [4.0, 5.0, 1.0]]], [[[4.0, 3.0, 4.0], [3.0, 6.0, 8.0], [6.0, 5.0, 4.0]], [[8.0, 5.0, 
4.0], [1.0, 1.0, 2.0], [3.0, 5.0, 8.0]], [[5.0, 2.0, 4.0], [5.0, 6.0, 7.0], [7.0, 8.0, 2.0]]], [[[6.0, 1.0, 2.0], [8.0, 4.0, 
6.0], [5.0, 3.0, 2.0]], [[6.0, 1.0, 5.0], [2.0, 8.0, 7.0], [7.0, 1.0, 4.0]], [[8.0, 1.0, 2.0], [7.0, 6.0, 2.0], [8.0, 6.0, 
7.0]]], [[[2.0, 4.0, 6.0], [4.0, 4.0, 1.0], [3.0, 2.0, 5.0]], [[1.0, 4.0, 8.0], [5.0, 3.0, 5.0], [5.0, 6.0, 2.0]], [[2.0, 6.0, 
1.0], [1.0, 7.0, 2.0], [3.0, 6.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UCPN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 2.0], [2.0, 4.0, 6.0], [5.0, 8.0, 3.0]], [[4.0, 7.0, 8.0], [7.0, 1.0, 8.0], [5.0, 4.0, 
4.0]], [[8.0, 2.0, 6.0], [7.0, 3.0, 3.0], [3.0, 5.0, 6.0]]], [[[2.0, 6.0, 6.0], [7.0, 1.0, 1.0], [4.0, 1.0, 5.0]], [[4.0, 1.0, 
7.0], [6.0, 6.0, 6.0], [8.0, 5.0, 7.0]], [[3.0, 4.0, 5.0], [8.0, 8.0, 8.0], [4.0, 8.0, 5.0]]], [[[1.0, 3.0, 2.0], [6.0, 5.0, 
8.0], [2.0, 3.0, 3.0]], [[2.0, 3.0, 2.0], [6.0, 7.0, 2.0], [1.0, 8.0, 1.0]], [[7.0, 6.0, 3.0], [7.0, 6.0, 6.0], [6.0, 7.0, 
2.0]]], [[[4.0, 8.0, 4.0], [8.0, 8.0, 8.0], [4.0, 3.0, 2.0]], [[4.0, 2.0, 3.0], [6.0, 3.0, 6.0], [2.0, 2.0, 7.0]], [[1.0, 4.0, 
4.0], [2.0, 3.0, 3.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 8.0, 1.0], [8.0, 7.0, 7.0], [8.0, 5.0, 7.0]], [[4.0, 4.0, 5.0], [3.0, 7.0, 4.0], [3.0, 2.0, 
8.0]], [[7.0, 1.0, 2.0], [7.0, 1.0, 2.0], [7.0, 8.0, 3.0]]]),self.functionspace)
      arg[:3,:,1]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 2.0], [3.0, 8.0, 1.0], [5.0, 8.0, 3.0]], [[4.0, 7.0, 8.0], [8.0, 7.0, 7.0], [5.0, 4.0, 
4.0]], [[8.0, 2.0, 6.0], [8.0, 5.0, 7.0], [3.0, 5.0, 6.0]]], [[[2.0, 6.0, 6.0], [4.0, 4.0, 5.0], [4.0, 1.0, 5.0]], [[4.0, 1.0, 
7.0], [3.0, 7.0, 4.0], [8.0, 5.0, 7.0]], [[3.0, 4.0, 5.0], [3.0, 2.0, 8.0], [4.0, 8.0, 5.0]]], [[[1.0, 3.0, 2.0], [7.0, 1.0, 
2.0], [2.0, 3.0, 3.0]], [[2.0, 3.0, 2.0], [7.0, 1.0, 2.0], [1.0, 8.0, 1.0]], [[7.0, 6.0, 3.0], [7.0, 8.0, 3.0], [6.0, 7.0, 
2.0]]], [[[4.0, 8.0, 4.0], [8.0, 8.0, 8.0], [4.0, 3.0, 2.0]], [[4.0, 2.0, 3.0], [6.0, 3.0, 6.0], [2.0, 2.0, 7.0]], [[1.0, 4.0, 
4.0], [2.0, 3.0, 3.0], [3.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UCPN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 6.0], [6.0, 2.0, 4.0], [3.0, 7.0, 8.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 3.0], [8.0, 7.0, 
8.0]], [[6.0, 5.0, 8.0], [6.0, 4.0, 6.0], [7.0, 7.0, 3.0]]], [[[5.0, 1.0, 6.0], [6.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[1.0, 4.0, 
2.0], [8.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[1.0, 8.0, 8.0], [8.0, 1.0, 5.0], [1.0, 4.0, 6.0]]], [[[4.0, 8.0, 7.0], [1.0, 5.0, 
3.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 6.0], [4.0, 8.0, 7.0], [3.0, 5.0, 4.0]], [[2.0, 3.0, 5.0], [5.0, 7.0, 4.0], [6.0, 8.0, 
7.0]]], [[[6.0, 1.0, 4.0], [6.0, 4.0, 3.0], [5.0, 4.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 7.0, 3.0], [6.0, 5.0, 1.0]], [[5.0, 7.0, 
1.0], [3.0, 2.0, 1.0], [3.0, 3.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 5.0, 5.0], [5.0, 4.0, 4.0], [7.0, 7.0, 8.0]], [[7.0, 7.0, 6.0], [5.0, 8.0, 7.0], [2.0, 
2.0, 3.0]], [[8.0, 7.0, 7.0], [6.0, 1.0, 3.0], [8.0, 5.0, 1.0]]])+(1.-msk_val)*numpy.array([[[1.0, 8.0, 4.0], [3.0, 8.0, 3.0], 
[3.0, 1.0, 7.0]], [[2.0, 5.0, 1.0], [3.0, 2.0, 6.0], [6.0, 4.0, 5.0]], [[6.0, 5.0, 6.0], [6.0, 6.0, 4.0], [7.0, 3.0, 7.0]]])
      arg[:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 6.0], [1.0, 5.0, 5.0], [3.0, 7.0, 8.0]], [[2.0, 2.0, 4.0], [5.0, 4.0, 4.0], [8.0, 
7.0, 8.0]], [[6.0, 5.0, 8.0], [7.0, 7.0, 8.0], [7.0, 7.0, 3.0]]], [[[5.0, 1.0, 6.0], [7.0, 7.0, 6.0], [8.0, 2.0, 1.0]], [[1.0, 
4.0, 2.0], [5.0, 8.0, 7.0], [2.0, 6.0, 3.0]], [[1.0, 8.0, 8.0], [2.0, 2.0, 3.0], [1.0, 4.0, 6.0]]], [[[4.0, 8.0, 7.0], [8.0, 
7.0, 7.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 6.0], [6.0, 1.0, 3.0], [3.0, 5.0, 4.0]], [[2.0, 3.0, 5.0], [8.0, 5.0, 1.0], [6.0, 8.0, 
7.0]]], [[[6.0, 1.0, 4.0], [6.0, 4.0, 3.0], [5.0, 4.0, 8.0]], [[8.0, 2.0, 1.0], [6.0, 7.0, 3.0], [6.0, 5.0, 1.0]], [[5.0, 7.0, 
1.0], [3.0, 2.0, 1.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [1.0, 8.0, 4.0], [3.0, 7.0, 8.0]], 
[[2.0, 2.0, 4.0], [3.0, 8.0, 3.0], [8.0, 7.0, 8.0]], [[6.0, 5.0, 8.0], [3.0, 1.0, 7.0], [7.0, 7.0, 3.0]]], [[[5.0, 1.0, 6.0], 
[2.0, 5.0, 1.0], [8.0, 2.0, 1.0]], [[1.0, 4.0, 2.0], [3.0, 2.0, 6.0], [2.0, 6.0, 3.0]], [[1.0, 8.0, 8.0], [6.0, 4.0, 5.0], 
[1.0, 4.0, 6.0]]], [[[4.0, 8.0, 7.0], [6.0, 5.0, 6.0], [4.0, 4.0, 7.0]], [[4.0, 2.0, 6.0], [6.0, 6.0, 4.0], [3.0, 5.0, 4.0]], 
[[2.0, 3.0, 5.0], [7.0, 3.0, 7.0], [6.0, 8.0, 7.0]]], [[[6.0, 1.0, 4.0], [6.0, 4.0, 3.0], [5.0, 4.0, 8.0]], [[8.0, 2.0, 1.0], 
[6.0, 7.0, 3.0], [6.0, 5.0, 1.0]], [[5.0, 7.0, 1.0], [3.0, 2.0, 1.0], [3.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLLN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 1.0], [3.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [8.0, 7.0, 2.0], [2.0, 5.0, 
1.0]], [[3.0, 6.0, 5.0], [5.0, 7.0, 1.0], [7.0, 7.0, 2.0]]], [[[6.0, 7.0, 5.0], [5.0, 1.0, 1.0], [3.0, 3.0, 8.0]], [[4.0, 2.0, 
8.0], [5.0, 2.0, 6.0], [6.0, 4.0, 2.0]], [[8.0, 4.0, 4.0], [6.0, 3.0, 7.0], [6.0, 4.0, 6.0]]], [[[5.0, 6.0, 2.0], [7.0, 3.0, 
6.0], [6.0, 2.0, 8.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 8.0, 2.0], [1.0, 2.0, 6.0], [8.0, 5.0, 
4.0]]], [[[6.0, 5.0, 2.0], [7.0, 3.0, 4.0], [7.0, 6.0, 3.0]], [[4.0, 2.0, 1.0], [2.0, 7.0, 5.0], [8.0, 4.0, 4.0]], [[1.0, 8.0, 
2.0], [5.0, 2.0, 1.0], [8.0, 5.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 5.0, 2.0]]], [[[1.0, 3.0, 7.0]]], [[[1.0, 6.0, 4.0]]], [[[5.0, 3.0, 7.0]]]])
      arg[:,2:,2:]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 1.0], [3.0, 1.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [8.0, 7.0, 2.0], [2.0, 5.0, 
1.0]], [[3.0, 6.0, 5.0], [5.0, 7.0, 1.0], [1.0, 5.0, 2.0]]], [[[6.0, 7.0, 5.0], [5.0, 1.0, 1.0], [3.0, 3.0, 8.0]], [[4.0, 2.0, 
8.0], [5.0, 2.0, 6.0], [6.0, 4.0, 2.0]], [[8.0, 4.0, 4.0], [6.0, 3.0, 7.0], [1.0, 3.0, 7.0]]], [[[5.0, 6.0, 2.0], [7.0, 3.0, 
6.0], [6.0, 2.0, 8.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 7.0], [5.0, 8.0, 2.0]], [[7.0, 8.0, 2.0], [1.0, 2.0, 6.0], [1.0, 6.0, 
4.0]]], [[[6.0, 5.0, 2.0], [7.0, 3.0, 4.0], [7.0, 6.0, 3.0]], [[4.0, 2.0, 1.0], [2.0, 7.0, 5.0], [8.0, 4.0, 4.0]], [[1.0, 8.0, 
2.0], [5.0, 2.0, 1.0], [5.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLLN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 3.0], [5.0, 3.0, 2.0], [5.0, 6.0, 1.0]], [[1.0, 6.0, 1.0], [1.0, 7.0, 4.0], [8.0, 7.0, 
1.0]], [[5.0, 1.0, 2.0], [8.0, 2.0, 1.0], [6.0, 7.0, 4.0]]], [[[2.0, 8.0, 7.0], [3.0, 5.0, 5.0], [3.0, 4.0, 6.0]], [[2.0, 6.0, 
6.0], [2.0, 7.0, 2.0], [3.0, 1.0, 2.0]], [[8.0, 4.0, 3.0], [4.0, 3.0, 1.0], [5.0, 6.0, 5.0]]], [[[3.0, 4.0, 3.0], [8.0, 7.0, 
8.0], [3.0, 2.0, 5.0]], [[6.0, 3.0, 4.0], [5.0, 6.0, 7.0], [3.0, 7.0, 4.0]], [[8.0, 1.0, 6.0], [7.0, 6.0, 4.0], [7.0, 8.0, 
1.0]]], [[[4.0, 4.0, 4.0], [8.0, 3.0, 7.0], [6.0, 2.0, 7.0]], [[7.0, 1.0, 1.0], [5.0, 3.0, 7.0], [6.0, 7.0, 8.0]], [[6.0, 7.0, 
4.0], [1.0, 1.0, 1.0], [5.0, 8.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 5.0, 5.0]]], [[[7.0, 7.0, 1.0]]], [[[7.0, 1.0, 3.0]]], [[[8.0, 1.0, 
1.0]]]]),self.functionspace)
      arg[:,2:,2:]=val
      ref=Data(numpy.array([[[[2.0, 8.0, 3.0], [5.0, 3.0, 2.0], [5.0, 6.0, 1.0]], [[1.0, 6.0, 1.0], [1.0, 7.0, 4.0], [8.0, 7.0, 
1.0]], [[5.0, 1.0, 2.0], [8.0, 2.0, 1.0], [3.0, 5.0, 5.0]]], [[[2.0, 8.0, 7.0], [3.0, 5.0, 5.0], [3.0, 4.0, 6.0]], [[2.0, 6.0, 
6.0], [2.0, 7.0, 2.0], [3.0, 1.0, 2.0]], [[8.0, 4.0, 3.0], [4.0, 3.0, 1.0], [7.0, 7.0, 1.0]]], [[[3.0, 4.0, 3.0], [8.0, 7.0, 
8.0], [3.0, 2.0, 5.0]], [[6.0, 3.0, 4.0], [5.0, 6.0, 7.0], [3.0, 7.0, 4.0]], [[8.0, 1.0, 6.0], [7.0, 6.0, 4.0], [7.0, 1.0, 
3.0]]], [[[4.0, 4.0, 4.0], [8.0, 3.0, 7.0], [6.0, 2.0, 7.0]], [[7.0, 1.0, 1.0], [5.0, 3.0, 7.0], [6.0, 7.0, 8.0]], [[6.0, 7.0, 
4.0], [1.0, 1.0, 1.0], [8.0, 1.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [1.0, 5.0, 7.0], [2.0, 3.0, 1.0]], [[4.0, 8.0, 7.0], [6.0, 1.0, 7.0], [7.0, 2.0, 
2.0]], [[7.0, 4.0, 6.0], [2.0, 8.0, 5.0], [1.0, 1.0, 3.0]]], [[[4.0, 5.0, 8.0], [1.0, 5.0, 4.0], [5.0, 6.0, 2.0]], [[5.0, 6.0, 
8.0], [5.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[3.0, 1.0, 2.0], [8.0, 4.0, 6.0], [6.0, 5.0, 2.0]]], [[[1.0, 1.0, 1.0], [5.0, 3.0, 
2.0], [7.0, 8.0, 8.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 4.0], [6.0, 7.0, 3.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 6.0], [8.0, 7.0, 
3.0]]], [[[5.0, 6.0, 7.0], [3.0, 4.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 1.0, 1.0], [7.0, 1.0, 4.0], [4.0, 2.0, 3.0]], [[6.0, 2.0, 
8.0], [1.0, 3.0, 1.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 2.0, 8.0]]], [[[4.0, 5.0, 4.0]]], [[[7.0, 4.0, 6.0]]], [[[2.0, 1.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 4.0, 3.0]]], [[[3.0, 7.0, 1.0]]], [[[3.0, 4.0, 7.0]]], [[[2.0, 7.0, 5.0]]]])
      arg[:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [1.0, 5.0, 7.0], [2.0, 3.0, 1.0]], [[4.0, 8.0, 7.0], [6.0, 1.0, 7.0], [7.0, 
2.0, 2.0]], [[7.0, 4.0, 6.0], [2.0, 8.0, 5.0], [8.0, 2.0, 8.0]]], [[[4.0, 5.0, 8.0], [1.0, 5.0, 4.0], [5.0, 6.0, 2.0]], [[5.0, 
6.0, 8.0], [5.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[3.0, 1.0, 2.0], [8.0, 4.0, 6.0], [4.0, 5.0, 4.0]]], [[[1.0, 1.0, 1.0], [5.0, 
3.0, 2.0], [7.0, 8.0, 8.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 4.0], [6.0, 7.0, 3.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 6.0], [7.0, 4.0, 
6.0]]], [[[5.0, 6.0, 7.0], [3.0, 4.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 1.0, 1.0], [7.0, 1.0, 4.0], [4.0, 2.0, 3.0]], [[6.0, 2.0, 
8.0], [1.0, 3.0, 1.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 4.0], [1.0, 5.0, 7.0], [2.0, 3.0, 1.0]], 
[[4.0, 8.0, 7.0], [6.0, 1.0, 7.0], [7.0, 2.0, 2.0]], [[7.0, 4.0, 6.0], [2.0, 8.0, 5.0], [6.0, 4.0, 3.0]]], [[[4.0, 5.0, 8.0], 
[1.0, 5.0, 4.0], [5.0, 6.0, 2.0]], [[5.0, 6.0, 8.0], [5.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[3.0, 1.0, 2.0], [8.0, 4.0, 6.0], 
[3.0, 7.0, 1.0]]], [[[1.0, 1.0, 1.0], [5.0, 3.0, 2.0], [7.0, 8.0, 8.0]], [[3.0, 4.0, 3.0], [4.0, 4.0, 4.0], [6.0, 7.0, 3.0]], 
[[5.0, 8.0, 3.0], [7.0, 6.0, 6.0], [3.0, 4.0, 7.0]]], [[[5.0, 6.0, 7.0], [3.0, 4.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 1.0, 1.0], 
[7.0, 1.0, 4.0], [4.0, 2.0, 3.0]], [[6.0, 2.0, 8.0], [1.0, 3.0, 1.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRLN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 2.0], [3.0, 1.0, 1.0], [7.0, 5.0, 7.0]], [[7.0, 5.0, 4.0], [4.0, 7.0, 5.0], [5.0, 1.0, 
6.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 6.0], [2.0, 2.0, 1.0]]], [[[8.0, 8.0, 4.0], [1.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[6.0, 2.0, 
8.0], [6.0, 7.0, 1.0], [4.0, 2.0, 3.0]], [[4.0, 2.0, 1.0], [1.0, 8.0, 7.0], [7.0, 5.0, 7.0]]], [[[7.0, 4.0, 6.0], [7.0, 6.0, 
2.0], [7.0, 8.0, 3.0]], [[6.0, 1.0, 2.0], [6.0, 6.0, 8.0], [2.0, 6.0, 3.0]], [[6.0, 2.0, 5.0], [6.0, 5.0, 5.0], [5.0, 2.0, 
4.0]]], [[[8.0, 6.0, 6.0], [6.0, 2.0, 3.0], [2.0, 3.0, 2.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 1.0], [8.0, 3.0, 6.0]], [[1.0, 7.0, 
1.0], [5.0, 2.0, 4.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 8.0, 8.0]]])
      arg[2,1:2,2:]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 2.0], [3.0, 1.0, 1.0], [7.0, 5.0, 7.0]], [[7.0, 5.0, 4.0], [4.0, 7.0, 5.0], [5.0, 1.0, 
6.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 6.0], [2.0, 2.0, 1.0]]], [[[8.0, 8.0, 4.0], [1.0, 2.0, 2.0], [3.0, 3.0, 7.0]], [[6.0, 2.0, 
8.0], [6.0, 7.0, 1.0], [4.0, 2.0, 3.0]], [[4.0, 2.0, 1.0], [1.0, 8.0, 7.0], [7.0, 5.0, 7.0]]], [[[7.0, 4.0, 6.0], [7.0, 6.0, 
2.0], [7.0, 8.0, 3.0]], [[6.0, 1.0, 2.0], [6.0, 6.0, 8.0], [4.0, 8.0, 8.0]], [[6.0, 2.0, 5.0], [6.0, 5.0, 5.0], [5.0, 2.0, 
4.0]]], [[[8.0, 6.0, 6.0], [6.0, 2.0, 3.0], [2.0, 3.0, 2.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 1.0], [8.0, 3.0, 6.0]], [[1.0, 7.0, 
1.0], [5.0, 2.0, 4.0], [5.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRLN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 7.0], [1.0, 2.0, 2.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 1.0], [4.0, 7.0, 7.0], [1.0, 3.0, 
7.0]], [[1.0, 8.0, 3.0], [2.0, 3.0, 4.0], [7.0, 4.0, 1.0]]], [[[8.0, 3.0, 6.0], [6.0, 6.0, 3.0], [6.0, 2.0, 8.0]], [[6.0, 3.0, 
4.0], [6.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[6.0, 6.0, 8.0], [5.0, 6.0, 2.0], [5.0, 2.0, 3.0]]], [[[2.0, 3.0, 7.0], [6.0, 6.0, 
3.0], [8.0, 3.0, 1.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 5.0], [7.0, 8.0, 6.0]], [[4.0, 3.0, 8.0], [5.0, 2.0, 5.0], [4.0, 4.0, 
8.0]]], [[[3.0, 5.0, 2.0], [4.0, 4.0, 7.0], [8.0, 2.0, 5.0]], [[1.0, 3.0, 3.0], [2.0, 6.0, 6.0], [7.0, 2.0, 8.0]], [[1.0, 2.0, 
4.0], [5.0, 3.0, 1.0], [2.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 7.0, 2.0]]]),self.functionspace)
      arg[2,1:2,2:]=val
      ref=Data(numpy.array([[[[7.0, 3.0, 7.0], [1.0, 2.0, 2.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 1.0], [4.0, 7.0, 7.0], [1.0, 3.0, 
7.0]], [[1.0, 8.0, 3.0], [2.0, 3.0, 4.0], [7.0, 4.0, 1.0]]], [[[8.0, 3.0, 6.0], [6.0, 6.0, 3.0], [6.0, 2.0, 8.0]], [[6.0, 3.0, 
4.0], [6.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[6.0, 6.0, 8.0], [5.0, 6.0, 2.0], [5.0, 2.0, 3.0]]], [[[2.0, 3.0, 7.0], [6.0, 6.0, 
3.0], [8.0, 3.0, 1.0]], [[4.0, 1.0, 5.0], [4.0, 8.0, 5.0], [7.0, 7.0, 2.0]], [[4.0, 3.0, 8.0], [5.0, 2.0, 5.0], [4.0, 4.0, 
8.0]]], [[[3.0, 5.0, 2.0], [4.0, 4.0, 7.0], [8.0, 2.0, 5.0]], [[1.0, 3.0, 3.0], [2.0, 6.0, 6.0], [7.0, 2.0, 8.0]], [[1.0, 2.0, 
4.0], [5.0, 3.0, 1.0], [2.0, 8.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRLN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [6.0, 5.0, 5.0], [5.0, 6.0, 5.0]], [[2.0, 3.0, 2.0], [4.0, 1.0, 4.0], [2.0, 7.0, 
2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 4.0], [8.0, 8.0, 4.0]]], [[[7.0, 5.0, 5.0], [3.0, 5.0, 8.0], [1.0, 5.0, 8.0]], [[1.0, 6.0, 
7.0], [3.0, 8.0, 1.0], [2.0, 5.0, 7.0]], [[8.0, 4.0, 2.0], [8.0, 3.0, 4.0], [2.0, 3.0, 8.0]]], [[[6.0, 1.0, 8.0], [2.0, 1.0, 
7.0], [1.0, 6.0, 5.0]], [[7.0, 8.0, 4.0], [6.0, 5.0, 6.0], [8.0, 4.0, 2.0]], [[3.0, 4.0, 3.0], [6.0, 3.0, 2.0], [1.0, 6.0, 
7.0]]], [[[6.0, 2.0, 7.0], [2.0, 6.0, 4.0], [5.0, 2.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 6.0, 8.0], [4.0, 1.0, 2.0]], [[5.0, 3.0, 
2.0], [2.0, 6.0, 7.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 3.0, 6.0]]])+(1.-msk_val)*numpy.array([[[1.0, 4.0, 3.0]]])
      arg[2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [6.0, 5.0, 5.0], [5.0, 6.0, 5.0]], [[2.0, 3.0, 2.0], [4.0, 1.0, 4.0], [2.0, 
7.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 4.0], [8.0, 8.0, 4.0]]], [[[7.0, 5.0, 5.0], [3.0, 5.0, 8.0], [1.0, 5.0, 8.0]], [[1.0, 
6.0, 7.0], [3.0, 8.0, 1.0], [2.0, 5.0, 7.0]], [[8.0, 4.0, 2.0], [8.0, 3.0, 4.0], [2.0, 3.0, 8.0]]], [[[6.0, 1.0, 8.0], [2.0, 
1.0, 7.0], [1.0, 6.0, 5.0]], [[7.0, 8.0, 4.0], [6.0, 5.0, 6.0], [7.0, 3.0, 6.0]], [[3.0, 4.0, 3.0], [6.0, 3.0, 2.0], [1.0, 6.0, 
7.0]]], [[[6.0, 2.0, 7.0], [2.0, 6.0, 4.0], [5.0, 2.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 6.0, 8.0], [4.0, 1.0, 2.0]], [[5.0, 3.0, 
2.0], [2.0, 6.0, 7.0], [7.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 1.0], [6.0, 5.0, 5.0], [5.0, 6.0, 5.0]], 
[[2.0, 3.0, 2.0], [4.0, 1.0, 4.0], [2.0, 7.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 4.0], [8.0, 8.0, 4.0]]], [[[7.0, 5.0, 5.0], 
[3.0, 5.0, 8.0], [1.0, 5.0, 8.0]], [[1.0, 6.0, 7.0], [3.0, 8.0, 1.0], [2.0, 5.0, 7.0]], [[8.0, 4.0, 2.0], [8.0, 3.0, 4.0], 
[2.0, 3.0, 8.0]]], [[[6.0, 1.0, 8.0], [2.0, 1.0, 7.0], [1.0, 6.0, 5.0]], [[7.0, 8.0, 4.0], [6.0, 5.0, 6.0], [1.0, 4.0, 3.0]], 
[[3.0, 4.0, 3.0], [6.0, 3.0, 2.0], [1.0, 6.0, 7.0]]], [[[6.0, 2.0, 7.0], [2.0, 6.0, 4.0], [5.0, 2.0, 6.0]], [[2.0, 1.0, 5.0], 
[2.0, 6.0, 8.0], [4.0, 1.0, 2.0]], [[5.0, 3.0, 2.0], [2.0, 6.0, 7.0], [7.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CPPN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 6.0], [2.0, 8.0, 2.0], [4.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [8.0, 6.0, 6.0], [8.0, 8.0, 
3.0]], [[4.0, 5.0, 4.0], [4.0, 2.0, 3.0], [6.0, 5.0, 8.0]]], [[[5.0, 3.0, 6.0], [3.0, 6.0, 8.0], [6.0, 7.0, 8.0]], [[6.0, 1.0, 
5.0], [7.0, 7.0, 4.0], [2.0, 8.0, 8.0]], [[3.0, 3.0, 5.0], [6.0, 8.0, 4.0], [7.0, 8.0, 5.0]]], [[[4.0, 2.0, 7.0], [4.0, 5.0, 
5.0], [6.0, 6.0, 6.0]], [[7.0, 4.0, 8.0], [7.0, 6.0, 2.0], [5.0, 1.0, 3.0]], [[7.0, 4.0, 2.0], [1.0, 5.0, 7.0], [2.0, 5.0, 
7.0]]], [[[6.0, 6.0, 4.0], [2.0, 5.0, 7.0], [4.0, 5.0, 7.0]], [[7.0, 3.0, 6.0], [4.0, 3.0, 6.0], [5.0, 8.0, 8.0]], [[7.0, 6.0, 
3.0], [7.0, 8.0, 4.0], [6.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[6.0, 5.0, 6.0], [1.0, 1.0, 1.0], [6.0, 7.0, 4.0], [7.0, 5.0, 5.0]])
      arg[:,1,1]=val
      ref=Data(numpy.array([[[[1.0, 6.0, 6.0], [2.0, 8.0, 2.0], [4.0, 7.0, 4.0]], [[4.0, 4.0, 5.0], [6.0, 5.0, 6.0], [8.0, 8.0, 
3.0]], [[4.0, 5.0, 4.0], [4.0, 2.0, 3.0], [6.0, 5.0, 8.0]]], [[[5.0, 3.0, 6.0], [3.0, 6.0, 8.0], [6.0, 7.0, 8.0]], [[6.0, 1.0, 
5.0], [1.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[3.0, 3.0, 5.0], [6.0, 8.0, 4.0], [7.0, 8.0, 5.0]]], [[[4.0, 2.0, 7.0], [4.0, 5.0, 
5.0], [6.0, 6.0, 6.0]], [[7.0, 4.0, 8.0], [6.0, 7.0, 4.0], [5.0, 1.0, 3.0]], [[7.0, 4.0, 2.0], [1.0, 5.0, 7.0], [2.0, 5.0, 
7.0]]], [[[6.0, 6.0, 4.0], [2.0, 5.0, 7.0], [4.0, 5.0, 7.0]], [[7.0, 3.0, 6.0], [7.0, 5.0, 5.0], [5.0, 8.0, 8.0]], [[7.0, 6.0, 
3.0], [7.0, 8.0, 4.0], [6.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CPPN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 6.0], [2.0, 5.0, 1.0], [6.0, 7.0, 4.0]], [[5.0, 2.0, 6.0], [6.0, 4.0, 7.0], [5.0, 4.0, 
2.0]], [[6.0, 3.0, 2.0], [4.0, 8.0, 2.0], [8.0, 6.0, 6.0]]], [[[6.0, 6.0, 7.0], [6.0, 7.0, 3.0], [8.0, 8.0, 7.0]], [[4.0, 2.0, 
6.0], [4.0, 1.0, 6.0], [5.0, 6.0, 3.0]], [[2.0, 6.0, 2.0], [3.0, 7.0, 5.0], [7.0, 6.0, 8.0]]], [[[2.0, 7.0, 7.0], [1.0, 8.0, 
4.0], [5.0, 6.0, 5.0]], [[1.0, 4.0, 6.0], [4.0, 3.0, 5.0], [5.0, 6.0, 3.0]], [[5.0, 3.0, 6.0], [2.0, 8.0, 8.0], [2.0, 7.0, 
6.0]]], [[[3.0, 8.0, 2.0], [4.0, 2.0, 1.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 7.0], [8.0, 4.0, 8.0], [4.0, 7.0, 8.0]], [[8.0, 7.0, 
1.0], [8.0, 4.0, 5.0], [5.0, 5.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 7.0, 4.0], [3.0, 8.0, 1.0], [7.0, 6.0, 6.0], [5.0, 3.0, 3.0]]),self.functionspace)
      arg[:,1,1]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 6.0], [2.0, 5.0, 1.0], [6.0, 7.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 7.0, 4.0], [5.0, 4.0, 
2.0]], [[6.0, 3.0, 2.0], [4.0, 8.0, 2.0], [8.0, 6.0, 6.0]]], [[[6.0, 6.0, 7.0], [6.0, 7.0, 3.0], [8.0, 8.0, 7.0]], [[4.0, 2.0, 
6.0], [3.0, 8.0, 1.0], [5.0, 6.0, 3.0]], [[2.0, 6.0, 2.0], [3.0, 7.0, 5.0], [7.0, 6.0, 8.0]]], [[[2.0, 7.0, 7.0], [1.0, 8.0, 
4.0], [5.0, 6.0, 5.0]], [[1.0, 4.0, 6.0], [7.0, 6.0, 6.0], [5.0, 6.0, 3.0]], [[5.0, 3.0, 6.0], [2.0, 8.0, 8.0], [2.0, 7.0, 
6.0]]], [[[3.0, 8.0, 2.0], [4.0, 2.0, 1.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 7.0], [5.0, 3.0, 3.0], [4.0, 7.0, 8.0]], [[8.0, 7.0, 
1.0], [8.0, 4.0, 5.0], [5.0, 5.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CPPN(self):
      arg=Data(numpy.array([[[[4.0, 6.0, 4.0], [2.0, 8.0, 4.0], [5.0, 7.0, 4.0]], [[5.0, 8.0, 1.0], [6.0, 6.0, 6.0], [6.0, 4.0, 
5.0]], [[2.0, 1.0, 1.0], [6.0, 4.0, 6.0], [6.0, 6.0, 8.0]]], [[[5.0, 3.0, 3.0], [1.0, 5.0, 5.0], [4.0, 6.0, 6.0]], [[5.0, 3.0, 
8.0], [2.0, 3.0, 4.0], [6.0, 8.0, 7.0]], [[8.0, 2.0, 4.0], [4.0, 1.0, 3.0], [3.0, 1.0, 6.0]]], [[[7.0, 3.0, 8.0], [8.0, 3.0, 
2.0], [2.0, 1.0, 6.0]], [[1.0, 7.0, 5.0], [8.0, 7.0, 5.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 6.0], [3.0, 3.0, 4.0], [7.0, 5.0, 
8.0]]], [[[8.0, 3.0, 4.0], [4.0, 3.0, 4.0], [1.0, 3.0, 5.0]], [[2.0, 1.0, 3.0], [3.0, 2.0, 7.0], [6.0, 5.0, 4.0]], [[5.0, 1.0, 
3.0], [3.0, 7.0, 4.0], [8.0, 7.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 3.0, 1.0], [2.0, 4.0, 7.0], [3.0, 6.0, 8.0], [3.0, 6.0, 
6.0]])+(1.-msk_val)*numpy.array([[6.0, 4.0, 5.0], [2.0, 1.0, 1.0], [1.0, 3.0, 6.0], [7.0, 2.0, 2.0]])
      arg[:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 4.0], [2.0, 8.0, 4.0], [5.0, 7.0, 4.0]], [[5.0, 8.0, 1.0], [2.0, 3.0, 1.0], [6.0, 
4.0, 5.0]], [[2.0, 1.0, 1.0], [6.0, 4.0, 6.0], [6.0, 6.0, 8.0]]], [[[5.0, 3.0, 3.0], [1.0, 5.0, 5.0], [4.0, 6.0, 6.0]], [[5.0, 
3.0, 8.0], [2.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[8.0, 2.0, 4.0], [4.0, 1.0, 3.0], [3.0, 1.0, 6.0]]], [[[7.0, 3.0, 8.0], [8.0, 
3.0, 2.0], [2.0, 1.0, 6.0]], [[1.0, 7.0, 5.0], [3.0, 6.0, 8.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 6.0], [3.0, 3.0, 4.0], [7.0, 5.0, 
8.0]]], [[[8.0, 3.0, 4.0], [4.0, 3.0, 4.0], [1.0, 3.0, 5.0]], [[2.0, 1.0, 3.0], [3.0, 6.0, 6.0], [6.0, 5.0, 4.0]], [[5.0, 1.0, 
3.0], [3.0, 7.0, 4.0], [8.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 4.0], [2.0, 8.0, 4.0], [5.0, 7.0, 4.0]], 
[[5.0, 8.0, 1.0], [6.0, 4.0, 5.0], [6.0, 4.0, 5.0]], [[2.0, 1.0, 1.0], [6.0, 4.0, 6.0], [6.0, 6.0, 8.0]]], [[[5.0, 3.0, 3.0], 
[1.0, 5.0, 5.0], [4.0, 6.0, 6.0]], [[5.0, 3.0, 8.0], [2.0, 1.0, 1.0], [6.0, 8.0, 7.0]], [[8.0, 2.0, 4.0], [4.0, 1.0, 3.0], 
[3.0, 1.0, 6.0]]], [[[7.0, 3.0, 8.0], [8.0, 3.0, 2.0], [2.0, 1.0, 6.0]], [[1.0, 7.0, 5.0], [1.0, 3.0, 6.0], [5.0, 8.0, 2.0]], 
[[2.0, 4.0, 6.0], [3.0, 3.0, 4.0], [7.0, 5.0, 8.0]]], [[[8.0, 3.0, 4.0], [4.0, 3.0, 4.0], [1.0, 3.0, 5.0]], [[2.0, 1.0, 3.0], 
[7.0, 2.0, 2.0], [6.0, 5.0, 4.0]], [[5.0, 1.0, 3.0], [3.0, 7.0, 4.0], [8.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PLCN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 7.0], [3.0, 3.0, 7.0], [3.0, 6.0, 2.0]], [[5.0, 8.0, 4.0], [5.0, 5.0, 7.0], [4.0, 2.0, 
4.0]], [[8.0, 2.0, 1.0], [1.0, 1.0, 6.0], [6.0, 5.0, 2.0]]], [[[1.0, 3.0, 8.0], [5.0, 1.0, 1.0], [6.0, 4.0, 3.0]], [[6.0, 7.0, 
5.0], [6.0, 4.0, 8.0], [8.0, 4.0, 6.0]], [[1.0, 2.0, 8.0], [4.0, 6.0, 6.0], [5.0, 4.0, 4.0]]], [[[7.0, 6.0, 4.0], [2.0, 5.0, 
8.0], [1.0, 1.0, 8.0]], [[5.0, 4.0, 3.0], [4.0, 8.0, 4.0], [8.0, 4.0, 5.0]], [[5.0, 2.0, 1.0], [8.0, 5.0, 2.0], [4.0, 3.0, 
7.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 3.0], [1.0, 4.0, 8.0]], [[1.0, 1.0, 6.0], [3.0, 8.0, 8.0], [6.0, 8.0, 6.0]], [[1.0, 7.0, 
5.0], [6.0, 5.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 2.0, 6.0], [7.0, 6.0, 8.0], [7.0, 4.0, 2.0]]])
      arg[2,2:,:]=val
      ref=Data(numpy.array([[[[5.0, 2.0, 7.0], [3.0, 3.0, 7.0], [3.0, 6.0, 2.0]], [[5.0, 8.0, 4.0], [5.0, 5.0, 7.0], [4.0, 2.0, 
4.0]], [[8.0, 2.0, 1.0], [1.0, 1.0, 6.0], [6.0, 5.0, 2.0]]], [[[1.0, 3.0, 8.0], [5.0, 1.0, 1.0], [6.0, 4.0, 3.0]], [[6.0, 7.0, 
5.0], [6.0, 4.0, 8.0], [8.0, 4.0, 6.0]], [[1.0, 2.0, 8.0], [4.0, 6.0, 6.0], [5.0, 4.0, 4.0]]], [[[7.0, 6.0, 4.0], [2.0, 5.0, 
8.0], [1.0, 1.0, 8.0]], [[5.0, 4.0, 3.0], [4.0, 8.0, 4.0], [8.0, 4.0, 5.0]], [[5.0, 2.0, 6.0], [7.0, 6.0, 8.0], [7.0, 4.0, 
2.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 3.0], [1.0, 4.0, 8.0]], [[1.0, 1.0, 6.0], [3.0, 8.0, 8.0], [6.0, 8.0, 6.0]], [[1.0, 7.0, 
5.0], [6.0, 5.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PLCN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 4.0], [4.0, 6.0, 4.0], [1.0, 1.0, 1.0]], [[6.0, 3.0, 4.0], [7.0, 4.0, 4.0], [6.0, 4.0, 
7.0]], [[6.0, 7.0, 5.0], [7.0, 5.0, 3.0], [4.0, 1.0, 4.0]]], [[[1.0, 1.0, 6.0], [8.0, 3.0, 1.0], [6.0, 5.0, 6.0]], [[4.0, 2.0, 
3.0], [4.0, 8.0, 5.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 3.0], [6.0, 7.0, 3.0], [5.0, 5.0, 8.0]]], [[[6.0, 4.0, 3.0], [6.0, 3.0, 
4.0], [2.0, 1.0, 6.0]], [[1.0, 1.0, 6.0], [1.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 4.0], [3.0, 3.0, 8.0], [5.0, 3.0, 
1.0]]], [[[6.0, 2.0, 1.0], [8.0, 2.0, 6.0], [1.0, 4.0, 2.0]], [[4.0, 8.0, 3.0], [1.0, 4.0, 2.0], [1.0, 2.0, 2.0]], [[5.0, 1.0, 
7.0], [5.0, 3.0, 7.0], [2.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 7.0, 1.0], [4.0, 2.0, 6.0], [4.0, 8.0, 6.0]]]),self.functionspace)
      arg[2,2:,:]=val
      ref=Data(numpy.array([[[[3.0, 5.0, 4.0], [4.0, 6.0, 4.0], [1.0, 1.0, 1.0]], [[6.0, 3.0, 4.0], [7.0, 4.0, 4.0], [6.0, 4.0, 
7.0]], [[6.0, 7.0, 5.0], [7.0, 5.0, 3.0], [4.0, 1.0, 4.0]]], [[[1.0, 1.0, 6.0], [8.0, 3.0, 1.0], [6.0, 5.0, 6.0]], [[4.0, 2.0, 
3.0], [4.0, 8.0, 5.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 3.0], [6.0, 7.0, 3.0], [5.0, 5.0, 8.0]]], [[[6.0, 4.0, 3.0], [6.0, 3.0, 
4.0], [2.0, 1.0, 6.0]], [[1.0, 1.0, 6.0], [1.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[2.0, 7.0, 1.0], [4.0, 2.0, 6.0], [4.0, 8.0, 
6.0]]], [[[6.0, 2.0, 1.0], [8.0, 2.0, 6.0], [1.0, 4.0, 2.0]], [[4.0, 8.0, 3.0], [1.0, 4.0, 2.0], [1.0, 2.0, 2.0]], [[5.0, 1.0, 
7.0], [5.0, 3.0, 7.0], [2.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PLCN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 1.0], [5.0, 5.0, 5.0], [7.0, 7.0, 8.0]], [[4.0, 7.0, 6.0], [5.0, 2.0, 4.0], [5.0, 4.0, 
3.0]], [[2.0, 6.0, 3.0], [3.0, 3.0, 7.0], [3.0, 6.0, 3.0]]], [[[4.0, 6.0, 7.0], [7.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[2.0, 8.0, 
2.0], [3.0, 1.0, 2.0], [7.0, 6.0, 8.0]], [[1.0, 8.0, 5.0], [3.0, 3.0, 6.0], [3.0, 7.0, 2.0]]], [[[3.0, 5.0, 2.0], [5.0, 6.0, 
2.0], [6.0, 4.0, 5.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 2.0], [8.0, 4.0, 2.0]], [[2.0, 3.0, 8.0], [1.0, 8.0, 7.0], [4.0, 8.0, 
8.0]]], [[[5.0, 8.0, 6.0], [1.0, 3.0, 7.0], [7.0, 7.0, 8.0]], [[8.0, 3.0, 1.0], [6.0, 2.0, 2.0], [4.0, 4.0, 7.0]], [[3.0, 3.0, 
7.0], [7.0, 4.0, 5.0], [6.0, 6.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 1.0, 3.0], [7.0, 5.0, 4.0], [2.0, 3.0, 8.0]]])+(1.-msk_val)*numpy.array([[[1.0, 5.0, 
4.0], [5.0, 1.0, 6.0], [5.0, 2.0, 6.0]]])
      arg[2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 1.0], [5.0, 5.0, 5.0], [7.0, 7.0, 8.0]], [[4.0, 7.0, 6.0], [5.0, 2.0, 4.0], [5.0, 
4.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 3.0, 7.0], [3.0, 6.0, 3.0]]], [[[4.0, 6.0, 7.0], [7.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[2.0, 
8.0, 2.0], [3.0, 1.0, 2.0], [7.0, 6.0, 8.0]], [[1.0, 8.0, 5.0], [3.0, 3.0, 6.0], [3.0, 7.0, 2.0]]], [[[3.0, 5.0, 2.0], [5.0, 
6.0, 2.0], [6.0, 4.0, 5.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 2.0], [8.0, 4.0, 2.0]], [[7.0, 1.0, 3.0], [7.0, 5.0, 4.0], [2.0, 3.0, 
8.0]]], [[[5.0, 8.0, 6.0], [1.0, 3.0, 7.0], [7.0, 7.0, 8.0]], [[8.0, 3.0, 1.0], [6.0, 2.0, 2.0], [4.0, 4.0, 7.0]], [[3.0, 3.0, 
7.0], [7.0, 4.0, 5.0], [6.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 1.0], [5.0, 5.0, 5.0], [7.0, 7.0, 8.0]], 
[[4.0, 7.0, 6.0], [5.0, 2.0, 4.0], [5.0, 4.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 3.0, 7.0], [3.0, 6.0, 3.0]]], [[[4.0, 6.0, 7.0], 
[7.0, 7.0, 1.0], [2.0, 3.0, 5.0]], [[2.0, 8.0, 2.0], [3.0, 1.0, 2.0], [7.0, 6.0, 8.0]], [[1.0, 8.0, 5.0], [3.0, 3.0, 6.0], 
[3.0, 7.0, 2.0]]], [[[3.0, 5.0, 2.0], [5.0, 6.0, 2.0], [6.0, 4.0, 5.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 2.0], [8.0, 4.0, 2.0]], 
[[1.0, 5.0, 4.0], [5.0, 1.0, 6.0], [5.0, 2.0, 6.0]]], [[[5.0, 8.0, 6.0], [1.0, 3.0, 7.0], [7.0, 7.0, 8.0]], [[8.0, 3.0, 1.0], 
[6.0, 2.0, 2.0], [4.0, 4.0, 7.0]], [[3.0, 3.0, 7.0], [7.0, 4.0, 5.0], [6.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PURN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 1.0], [6.0, 3.0, 4.0], [7.0, 5.0, 2.0]], [[5.0, 2.0, 3.0], [3.0, 7.0, 2.0], [8.0, 1.0, 
4.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 7.0], [7.0, 4.0, 6.0]]], [[[5.0, 4.0, 6.0], [3.0, 6.0, 5.0], [4.0, 4.0, 6.0]], [[8.0, 4.0, 
5.0], [7.0, 3.0, 6.0], [5.0, 8.0, 7.0]], [[3.0, 1.0, 3.0], [3.0, 1.0, 4.0], [2.0, 1.0, 6.0]]], [[[4.0, 1.0, 2.0], [2.0, 3.0, 
4.0], [3.0, 7.0, 8.0]], [[5.0, 1.0, 6.0], [3.0, 8.0, 6.0], [8.0, 2.0, 6.0]], [[6.0, 5.0, 7.0], [5.0, 7.0, 1.0], [5.0, 2.0, 
3.0]]], [[[6.0, 3.0, 5.0], [8.0, 6.0, 6.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 7.0], [2.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[5.0, 5.0, 
7.0], [5.0, 7.0, 3.0], [1.0, 1.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[8.0, 5.0, 7.0]], [[5.0, 4.0, 8.0]]])
      arg[2,:2,1:2]=val
      ref=Data(numpy.array([[[[4.0, 7.0, 1.0], [6.0, 3.0, 4.0], [7.0, 5.0, 2.0]], [[5.0, 2.0, 3.0], [3.0, 7.0, 2.0], [8.0, 1.0, 
4.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 7.0], [7.0, 4.0, 6.0]]], [[[5.0, 4.0, 6.0], [3.0, 6.0, 5.0], [4.0, 4.0, 6.0]], [[8.0, 4.0, 
5.0], [7.0, 3.0, 6.0], [5.0, 8.0, 7.0]], [[3.0, 1.0, 3.0], [3.0, 1.0, 4.0], [2.0, 1.0, 6.0]]], [[[4.0, 1.0, 2.0], [8.0, 5.0, 
7.0], [3.0, 7.0, 8.0]], [[5.0, 1.0, 6.0], [5.0, 4.0, 8.0], [8.0, 2.0, 6.0]], [[6.0, 5.0, 7.0], [5.0, 7.0, 1.0], [5.0, 2.0, 
3.0]]], [[[6.0, 3.0, 5.0], [8.0, 6.0, 6.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 7.0], [2.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[5.0, 5.0, 
7.0], [5.0, 7.0, 3.0], [1.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PURN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [7.0, 8.0, 8.0], [2.0, 2.0, 3.0]], [[2.0, 8.0, 6.0], [1.0, 8.0, 3.0], [5.0, 6.0, 
5.0]], [[3.0, 7.0, 1.0], [2.0, 2.0, 5.0], [8.0, 2.0, 6.0]]], [[[2.0, 7.0, 6.0], [4.0, 5.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 4.0, 
6.0], [8.0, 2.0, 4.0], [1.0, 3.0, 5.0]], [[3.0, 3.0, 7.0], [7.0, 2.0, 5.0], [6.0, 7.0, 6.0]]], [[[4.0, 7.0, 2.0], [4.0, 2.0, 
2.0], [2.0, 1.0, 5.0]], [[2.0, 8.0, 7.0], [7.0, 8.0, 1.0], [2.0, 2.0, 2.0]], [[5.0, 7.0, 1.0], [8.0, 5.0, 3.0], [1.0, 3.0, 
4.0]]], [[[5.0, 8.0, 5.0], [5.0, 1.0, 5.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 8.0], [1.0, 8.0, 4.0], [1.0, 1.0, 3.0]], [[5.0, 7.0, 
8.0], [8.0, 3.0, 3.0], [3.0, 3.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 4.0, 7.0]], [[7.0, 1.0, 3.0]]]),self.functionspace)
      arg[2,:2,1:2]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 7.0], [7.0, 8.0, 8.0], [2.0, 2.0, 3.0]], [[2.0, 8.0, 6.0], [1.0, 8.0, 3.0], [5.0, 6.0, 
5.0]], [[3.0, 7.0, 1.0], [2.0, 2.0, 5.0], [8.0, 2.0, 6.0]]], [[[2.0, 7.0, 6.0], [4.0, 5.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 4.0, 
6.0], [8.0, 2.0, 4.0], [1.0, 3.0, 5.0]], [[3.0, 3.0, 7.0], [7.0, 2.0, 5.0], [6.0, 7.0, 6.0]]], [[[4.0, 7.0, 2.0], [6.0, 4.0, 
7.0], [2.0, 1.0, 5.0]], [[2.0, 8.0, 7.0], [7.0, 1.0, 3.0], [2.0, 2.0, 2.0]], [[5.0, 7.0, 1.0], [8.0, 5.0, 3.0], [1.0, 3.0, 
4.0]]], [[[5.0, 8.0, 5.0], [5.0, 1.0, 5.0], [5.0, 3.0, 1.0]], [[5.0, 4.0, 8.0], [1.0, 8.0, 4.0], [1.0, 1.0, 3.0]], [[5.0, 7.0, 
8.0], [8.0, 3.0, 3.0], [3.0, 3.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PURN(self):
      arg=Data(numpy.array([[[[2.0, 1.0, 6.0], [2.0, 6.0, 8.0], [4.0, 2.0, 2.0]], [[5.0, 6.0, 5.0], [4.0, 2.0, 7.0], [7.0, 6.0, 
4.0]], [[4.0, 3.0, 7.0], [3.0, 3.0, 3.0], [8.0, 5.0, 4.0]]], [[[8.0, 8.0, 4.0], [8.0, 8.0, 4.0], [5.0, 5.0, 8.0]], [[8.0, 8.0, 
5.0], [8.0, 2.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 5.0, 5.0], [7.0, 3.0, 8.0], [6.0, 8.0, 2.0]]], [[[1.0, 5.0, 5.0], [4.0, 5.0, 
1.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 5.0], [7.0, 7.0, 1.0], [7.0, 6.0, 4.0]], [[4.0, 1.0, 3.0], [7.0, 3.0, 6.0], [2.0, 5.0, 
2.0]]], [[[7.0, 3.0, 4.0], [1.0, 1.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 3.0, 7.0], [2.0, 8.0, 8.0], [5.0, 7.0, 3.0]], [[6.0, 1.0, 
8.0], [3.0, 1.0, 7.0], [5.0, 6.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 8.0, 3.0]], [[6.0, 5.0, 8.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 8.0]], [[3.0, 8.0, 
5.0]]])
      arg[2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 6.0], [2.0, 6.0, 8.0], [4.0, 2.0, 2.0]], [[5.0, 6.0, 5.0], [4.0, 2.0, 7.0], [7.0, 
6.0, 4.0]], [[4.0, 3.0, 7.0], [3.0, 3.0, 3.0], [8.0, 5.0, 4.0]]], [[[8.0, 8.0, 4.0], [8.0, 8.0, 4.0], [5.0, 5.0, 8.0]], [[8.0, 
8.0, 5.0], [8.0, 2.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 5.0, 5.0], [7.0, 3.0, 8.0], [6.0, 8.0, 2.0]]], [[[1.0, 5.0, 5.0], [2.0, 
8.0, 3.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 8.0], [7.0, 6.0, 4.0]], [[4.0, 1.0, 3.0], [7.0, 3.0, 6.0], [2.0, 5.0, 
2.0]]], [[[7.0, 3.0, 4.0], [1.0, 1.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 3.0, 7.0], [2.0, 8.0, 8.0], [5.0, 7.0, 3.0]], [[6.0, 1.0, 
8.0], [3.0, 1.0, 7.0], [5.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [2.0, 6.0, 8.0], [4.0, 2.0, 2.0]], 
[[5.0, 6.0, 5.0], [4.0, 2.0, 7.0], [7.0, 6.0, 4.0]], [[4.0, 3.0, 7.0], [3.0, 3.0, 3.0], [8.0, 5.0, 4.0]]], [[[8.0, 8.0, 4.0], 
[8.0, 8.0, 4.0], [5.0, 5.0, 8.0]], [[8.0, 8.0, 5.0], [8.0, 2.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 5.0, 5.0], [7.0, 3.0, 8.0], 
[6.0, 8.0, 2.0]]], [[[1.0, 5.0, 5.0], [2.0, 7.0, 8.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 8.0, 5.0], [7.0, 6.0, 4.0]], 
[[4.0, 1.0, 3.0], [7.0, 3.0, 6.0], [2.0, 5.0, 2.0]]], [[[7.0, 3.0, 4.0], [1.0, 1.0, 1.0], [3.0, 7.0, 7.0]], [[8.0, 3.0, 7.0], 
[2.0, 8.0, 8.0], [5.0, 7.0, 3.0]], [[6.0, 1.0, 8.0], [3.0, 1.0, 7.0], [5.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UPRN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 6.0], [5.0, 4.0, 8.0], [7.0, 6.0, 4.0]], [[1.0, 8.0, 8.0], [5.0, 2.0, 2.0], [5.0, 3.0, 
5.0]], [[4.0, 4.0, 3.0], [7.0, 7.0, 2.0], [7.0, 2.0, 7.0]]], [[[2.0, 5.0, 1.0], [4.0, 7.0, 5.0], [6.0, 4.0, 4.0]], [[4.0, 6.0, 
3.0], [4.0, 5.0, 4.0], [8.0, 1.0, 4.0]], [[1.0, 5.0, 3.0], [2.0, 4.0, 4.0], [3.0, 3.0, 1.0]]], [[[5.0, 7.0, 2.0], [1.0, 5.0, 
2.0], [4.0, 4.0, 7.0]], [[7.0, 3.0, 8.0], [2.0, 2.0, 1.0], [3.0, 3.0, 6.0]], [[1.0, 3.0, 5.0], [5.0, 4.0, 7.0], [1.0, 6.0, 
8.0]]], [[[7.0, 7.0, 5.0], [3.0, 8.0, 7.0], [1.0, 5.0, 1.0]], [[1.0, 5.0, 6.0], [2.0, 4.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 2.0, 
8.0], [1.0, 5.0, 8.0], [5.0, 7.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 7.0, 6.0]], [[5.0, 6.0, 8.0]], [[2.0, 7.0, 3.0]]])
      arg[:3,1,1:2]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 6.0], [5.0, 4.0, 8.0], [7.0, 6.0, 4.0]], [[1.0, 8.0, 8.0], [7.0, 7.0, 6.0], [5.0, 3.0, 
5.0]], [[4.0, 4.0, 3.0], [7.0, 7.0, 2.0], [7.0, 2.0, 7.0]]], [[[2.0, 5.0, 1.0], [4.0, 7.0, 5.0], [6.0, 4.0, 4.0]], [[4.0, 6.0, 
3.0], [5.0, 6.0, 8.0], [8.0, 1.0, 4.0]], [[1.0, 5.0, 3.0], [2.0, 4.0, 4.0], [3.0, 3.0, 1.0]]], [[[5.0, 7.0, 2.0], [1.0, 5.0, 
2.0], [4.0, 4.0, 7.0]], [[7.0, 3.0, 8.0], [2.0, 7.0, 3.0], [3.0, 3.0, 6.0]], [[1.0, 3.0, 5.0], [5.0, 4.0, 7.0], [1.0, 6.0, 
8.0]]], [[[7.0, 7.0, 5.0], [3.0, 8.0, 7.0], [1.0, 5.0, 1.0]], [[1.0, 5.0, 6.0], [2.0, 4.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 2.0, 
8.0], [1.0, 5.0, 8.0], [5.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UPRN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 8.0], [4.0, 2.0, 2.0], [8.0, 7.0, 7.0]], [[8.0, 3.0, 1.0], [6.0, 3.0, 7.0], [6.0, 8.0, 
7.0]], [[2.0, 4.0, 2.0], [2.0, 6.0, 8.0], [2.0, 3.0, 8.0]]], [[[5.0, 6.0, 2.0], [7.0, 5.0, 1.0], [3.0, 3.0, 5.0]], [[6.0, 6.0, 
3.0], [1.0, 3.0, 8.0], [5.0, 4.0, 8.0]], [[7.0, 4.0, 6.0], [7.0, 8.0, 5.0], [5.0, 4.0, 5.0]]], [[[8.0, 1.0, 7.0], [2.0, 4.0, 
5.0], [4.0, 3.0, 6.0]], [[4.0, 8.0, 8.0], [2.0, 6.0, 1.0], [7.0, 8.0, 2.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 8.0], [5.0, 7.0, 
7.0]]], [[[3.0, 6.0, 1.0], [8.0, 1.0, 5.0], [3.0, 4.0, 5.0]], [[6.0, 8.0, 7.0], [6.0, 8.0, 6.0], [1.0, 6.0, 5.0]], [[1.0, 7.0, 
3.0], [8.0, 4.0, 6.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 4.0, 1.0]], [[3.0, 8.0, 2.0]], [[6.0, 5.0, 4.0]]]),self.functionspace)
      arg[:3,1,1:2]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 8.0], [4.0, 2.0, 2.0], [8.0, 7.0, 7.0]], [[8.0, 3.0, 1.0], [5.0, 4.0, 1.0], [6.0, 8.0, 
7.0]], [[2.0, 4.0, 2.0], [2.0, 6.0, 8.0], [2.0, 3.0, 8.0]]], [[[5.0, 6.0, 2.0], [7.0, 5.0, 1.0], [3.0, 3.0, 5.0]], [[6.0, 6.0, 
3.0], [3.0, 8.0, 2.0], [5.0, 4.0, 8.0]], [[7.0, 4.0, 6.0], [7.0, 8.0, 5.0], [5.0, 4.0, 5.0]]], [[[8.0, 1.0, 7.0], [2.0, 4.0, 
5.0], [4.0, 3.0, 6.0]], [[4.0, 8.0, 8.0], [6.0, 5.0, 4.0], [7.0, 8.0, 2.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 8.0], [5.0, 7.0, 
7.0]]], [[[3.0, 6.0, 1.0], [8.0, 1.0, 5.0], [3.0, 4.0, 5.0]], [[6.0, 8.0, 7.0], [6.0, 8.0, 6.0], [1.0, 6.0, 5.0]], [[1.0, 7.0, 
3.0], [8.0, 4.0, 6.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UPRN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 7.0], [1.0, 5.0, 5.0], [2.0, 1.0, 6.0]], [[1.0, 8.0, 4.0], [6.0, 2.0, 7.0], [1.0, 3.0, 
4.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 8.0], [6.0, 7.0, 6.0]]], [[[6.0, 2.0, 4.0], [5.0, 2.0, 5.0], [6.0, 3.0, 8.0]], [[4.0, 2.0, 
2.0], [2.0, 5.0, 7.0], [2.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 4.0, 8.0], [5.0, 2.0, 4.0]]], [[[3.0, 5.0, 7.0], [6.0, 6.0, 
2.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 4.0], [7.0, 1.0, 4.0], [2.0, 5.0, 4.0]], [[8.0, 4.0, 2.0], [1.0, 6.0, 6.0], [5.0, 8.0, 
3.0]]], [[[2.0, 5.0, 8.0], [6.0, 5.0, 7.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [7.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 
8.0], [5.0, 3.0, 6.0], [6.0, 3.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 7.0, 2.0]], [[8.0, 5.0, 7.0]], [[5.0, 4.0, 4.0]]])+(1.-msk_val)*numpy.array([[[8.0, 8.0, 
2.0]], [[2.0, 4.0, 2.0]], [[4.0, 4.0, 3.0]]])
      arg[:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 7.0], [1.0, 5.0, 5.0], [2.0, 1.0, 6.0]], [[1.0, 8.0, 4.0], [5.0, 7.0, 2.0], [1.0, 
3.0, 4.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 8.0], [6.0, 7.0, 6.0]]], [[[6.0, 2.0, 4.0], [5.0, 2.0, 5.0], [6.0, 3.0, 8.0]], [[4.0, 
2.0, 2.0], [8.0, 5.0, 7.0], [2.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 4.0, 8.0], [5.0, 2.0, 4.0]]], [[[3.0, 5.0, 7.0], [6.0, 
6.0, 2.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 4.0], [5.0, 4.0, 4.0], [2.0, 5.0, 4.0]], [[8.0, 4.0, 2.0], [1.0, 6.0, 6.0], [5.0, 8.0, 
3.0]]], [[[2.0, 5.0, 8.0], [6.0, 5.0, 7.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], [7.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 
8.0], [5.0, 3.0, 6.0], [6.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 7.0], [1.0, 5.0, 5.0], [2.0, 1.0, 6.0]], 
[[1.0, 8.0, 4.0], [8.0, 8.0, 2.0], [1.0, 3.0, 4.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 8.0], [6.0, 7.0, 6.0]]], [[[6.0, 2.0, 4.0], 
[5.0, 2.0, 5.0], [6.0, 3.0, 8.0]], [[4.0, 2.0, 2.0], [2.0, 4.0, 2.0], [2.0, 1.0, 2.0]], [[7.0, 8.0, 7.0], [3.0, 4.0, 8.0], 
[5.0, 2.0, 4.0]]], [[[3.0, 5.0, 7.0], [6.0, 6.0, 2.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 4.0], [4.0, 4.0, 3.0], [2.0, 5.0, 4.0]], 
[[8.0, 4.0, 2.0], [1.0, 6.0, 6.0], [5.0, 8.0, 3.0]]], [[[2.0, 5.0, 8.0], [6.0, 5.0, 7.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 1.0], 
[7.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 8.0], [5.0, 3.0, 6.0], [6.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRLN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 5.0], [4.0, 7.0, 5.0], [1.0, 3.0, 3.0]], [[2.0, 1.0, 3.0], [1.0, 1.0, 8.0], [8.0, 1.0, 
7.0]], [[6.0, 7.0, 3.0], [4.0, 5.0, 4.0], [7.0, 6.0, 3.0]]], [[[2.0, 2.0, 1.0], [1.0, 6.0, 5.0], [7.0, 4.0, 3.0]], [[4.0, 1.0, 
8.0], [5.0, 6.0, 3.0], [3.0, 4.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 7.0, 3.0], [7.0, 2.0, 6.0]]], [[[7.0, 2.0, 5.0], [2.0, 3.0, 
2.0], [5.0, 6.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 1.0, 6.0], [2.0, 3.0, 4.0]], [[6.0, 6.0, 2.0], [4.0, 6.0, 4.0], [1.0, 8.0, 
5.0]]], [[[5.0, 3.0, 3.0], [2.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 6.0, 1.0], [5.0, 1.0, 7.0], [4.0, 4.0, 2.0]], [[6.0, 7.0, 
8.0], [1.0, 5.0, 7.0], [6.0, 4.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 8.0, 5.0]]], [[[4.0, 6.0, 6.0]]]])
      arg[2:,1:2,2:]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 5.0], [4.0, 7.0, 5.0], [1.0, 3.0, 3.0]], [[2.0, 1.0, 3.0], [1.0, 1.0, 8.0], [8.0, 1.0, 
7.0]], [[6.0, 7.0, 3.0], [4.0, 5.0, 4.0], [7.0, 6.0, 3.0]]], [[[2.0, 2.0, 1.0], [1.0, 6.0, 5.0], [7.0, 4.0, 3.0]], [[4.0, 1.0, 
8.0], [5.0, 6.0, 3.0], [3.0, 4.0, 2.0]], [[7.0, 1.0, 1.0], [7.0, 7.0, 3.0], [7.0, 2.0, 6.0]]], [[[7.0, 2.0, 5.0], [2.0, 3.0, 
2.0], [5.0, 6.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 1.0, 6.0], [4.0, 8.0, 5.0]], [[6.0, 6.0, 2.0], [4.0, 6.0, 4.0], [1.0, 8.0, 
5.0]]], [[[5.0, 3.0, 3.0], [2.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 6.0, 1.0], [5.0, 1.0, 7.0], [4.0, 6.0, 6.0]], [[6.0, 7.0, 
8.0], [1.0, 5.0, 7.0], [6.0, 4.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRLN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 6.0], [6.0, 6.0, 2.0], [8.0, 8.0, 3.0]], [[8.0, 3.0, 1.0], [8.0, 2.0, 8.0], [7.0, 6.0, 
4.0]], [[8.0, 3.0, 3.0], [8.0, 1.0, 6.0], [3.0, 2.0, 8.0]]], [[[7.0, 8.0, 4.0], [4.0, 6.0, 4.0], [7.0, 7.0, 8.0]], [[1.0, 4.0, 
8.0], [7.0, 7.0, 7.0], [7.0, 5.0, 8.0]], [[1.0, 4.0, 5.0], [8.0, 5.0, 1.0], [1.0, 8.0, 2.0]]], [[[2.0, 5.0, 6.0], [7.0, 2.0, 
5.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 4.0], [4.0, 3.0, 3.0], [6.0, 1.0, 2.0]], [[6.0, 5.0, 2.0], [4.0, 6.0, 6.0], [8.0, 8.0, 
7.0]]], [[[2.0, 6.0, 1.0], [7.0, 8.0, 8.0], [7.0, 2.0, 3.0]], [[4.0, 4.0, 2.0], [2.0, 4.0, 5.0], [3.0, 3.0, 7.0]], [[7.0, 8.0, 
4.0], [2.0, 8.0, 7.0], [1.0, 3.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 4.0, 6.0]]], [[[7.0, 4.0, 6.0]]]]),self.functionspace)
      arg[2:,1:2,2:]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 6.0], [6.0, 6.0, 2.0], [8.0, 8.0, 3.0]], [[8.0, 3.0, 1.0], [8.0, 2.0, 8.0], [7.0, 6.0, 
4.0]], [[8.0, 3.0, 3.0], [8.0, 1.0, 6.0], [3.0, 2.0, 8.0]]], [[[7.0, 8.0, 4.0], [4.0, 6.0, 4.0], [7.0, 7.0, 8.0]], [[1.0, 4.0, 
8.0], [7.0, 7.0, 7.0], [7.0, 5.0, 8.0]], [[1.0, 4.0, 5.0], [8.0, 5.0, 1.0], [1.0, 8.0, 2.0]]], [[[2.0, 5.0, 6.0], [7.0, 2.0, 
5.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 4.0], [4.0, 3.0, 3.0], [1.0, 4.0, 6.0]], [[6.0, 5.0, 2.0], [4.0, 6.0, 6.0], [8.0, 8.0, 
7.0]]], [[[2.0, 6.0, 1.0], [7.0, 8.0, 8.0], [7.0, 2.0, 3.0]], [[4.0, 4.0, 2.0], [2.0, 4.0, 5.0], [7.0, 4.0, 6.0]], [[7.0, 8.0, 
4.0], [2.0, 8.0, 7.0], [1.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRLN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 1.0], [3.0, 3.0, 3.0], [1.0, 8.0, 7.0]], [[3.0, 5.0, 5.0], [2.0, 8.0, 3.0], [8.0, 4.0, 
2.0]], [[5.0, 1.0, 6.0], [5.0, 7.0, 6.0], [8.0, 1.0, 6.0]]], [[[8.0, 8.0, 3.0], [7.0, 3.0, 8.0], [1.0, 5.0, 5.0]], [[8.0, 6.0, 
4.0], [3.0, 4.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 2.0, 4.0], [5.0, 3.0, 3.0], [6.0, 5.0, 7.0]]], [[[8.0, 1.0, 2.0], [2.0, 3.0, 
4.0], [6.0, 4.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 4.0, 8.0], [3.0, 3.0, 8.0]], [[6.0, 8.0, 3.0], [3.0, 8.0, 6.0], [2.0, 8.0, 
8.0]]], [[[7.0, 8.0, 6.0], [2.0, 2.0, 8.0], [4.0, 4.0, 6.0]], [[2.0, 5.0, 4.0], [8.0, 1.0, 6.0], [2.0, 5.0, 2.0]], [[1.0, 1.0, 
6.0], [2.0, 7.0, 2.0], [4.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 8.0, 2.0]]], [[[1.0, 2.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 8.0, 5.0]]], 
[[[1.0, 4.0, 7.0]]]])
      arg[2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [3.0, 3.0, 3.0], [1.0, 8.0, 7.0]], [[3.0, 5.0, 5.0], [2.0, 8.0, 3.0], [8.0, 
4.0, 2.0]], [[5.0, 1.0, 6.0], [5.0, 7.0, 6.0], [8.0, 1.0, 6.0]]], [[[8.0, 8.0, 3.0], [7.0, 3.0, 8.0], [1.0, 5.0, 5.0]], [[8.0, 
6.0, 4.0], [3.0, 4.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 2.0, 4.0], [5.0, 3.0, 3.0], [6.0, 5.0, 7.0]]], [[[8.0, 1.0, 2.0], [2.0, 
3.0, 4.0], [6.0, 4.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 4.0, 8.0], [2.0, 8.0, 2.0]], [[6.0, 8.0, 3.0], [3.0, 8.0, 6.0], [2.0, 8.0, 
8.0]]], [[[7.0, 8.0, 6.0], [2.0, 2.0, 8.0], [4.0, 4.0, 6.0]], [[2.0, 5.0, 4.0], [8.0, 1.0, 6.0], [1.0, 2.0, 1.0]], [[1.0, 1.0, 
6.0], [2.0, 7.0, 2.0], [4.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 1.0], [3.0, 3.0, 3.0], [1.0, 8.0, 7.0]], 
[[3.0, 5.0, 5.0], [2.0, 8.0, 3.0], [8.0, 4.0, 2.0]], [[5.0, 1.0, 6.0], [5.0, 7.0, 6.0], [8.0, 1.0, 6.0]]], [[[8.0, 8.0, 3.0], 
[7.0, 3.0, 8.0], [1.0, 5.0, 5.0]], [[8.0, 6.0, 4.0], [3.0, 4.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 2.0, 4.0], [5.0, 3.0, 3.0], 
[6.0, 5.0, 7.0]]], [[[8.0, 1.0, 2.0], [2.0, 3.0, 4.0], [6.0, 4.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 4.0, 8.0], [3.0, 8.0, 5.0]], 
[[6.0, 8.0, 3.0], [3.0, 8.0, 6.0], [2.0, 8.0, 8.0]]], [[[7.0, 8.0, 6.0], [2.0, 2.0, 8.0], [4.0, 4.0, 6.0]], [[2.0, 5.0, 4.0], 
[8.0, 1.0, 6.0], [1.0, 4.0, 7.0]], [[1.0, 1.0, 6.0], [2.0, 7.0, 2.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLUN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 8.0], [4.0, 3.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 5.0], [1.0, 1.0, 
8.0]], [[8.0, 6.0, 3.0], [6.0, 5.0, 2.0], [5.0, 3.0, 8.0]]], [[[1.0, 7.0, 6.0], [2.0, 8.0, 2.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 
1.0], [3.0, 2.0, 5.0], [1.0, 4.0, 5.0]], [[4.0, 3.0, 6.0], [6.0, 5.0, 6.0], [2.0, 6.0, 3.0]]], [[[6.0, 4.0, 2.0], [2.0, 1.0, 
5.0], [5.0, 6.0, 5.0]], [[6.0, 8.0, 3.0], [5.0, 5.0, 8.0], [5.0, 3.0, 7.0]], [[3.0, 5.0, 5.0], [8.0, 2.0, 1.0], [5.0, 4.0, 
6.0]]], [[[4.0, 1.0, 8.0], [5.0, 6.0, 1.0], [6.0, 8.0, 7.0]], [[5.0, 4.0, 3.0], [1.0, 6.0, 4.0], [4.0, 5.0, 3.0]], [[3.0, 5.0, 
4.0], [4.0, 2.0, 2.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 2.0, 6.0], [5.0, 8.0, 5.0]]], [[[7.0, 7.0, 2.0], [8.0, 7.0, 7.0]]]])
      arg[2:,2:,:2]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 8.0], [4.0, 3.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 5.0], [1.0, 1.0, 
8.0]], [[8.0, 6.0, 3.0], [6.0, 5.0, 2.0], [5.0, 3.0, 8.0]]], [[[1.0, 7.0, 6.0], [2.0, 8.0, 2.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 
1.0], [3.0, 2.0, 5.0], [1.0, 4.0, 5.0]], [[4.0, 3.0, 6.0], [6.0, 5.0, 6.0], [2.0, 6.0, 3.0]]], [[[6.0, 4.0, 2.0], [2.0, 1.0, 
5.0], [5.0, 6.0, 5.0]], [[6.0, 8.0, 3.0], [5.0, 5.0, 8.0], [5.0, 3.0, 7.0]], [[4.0, 2.0, 6.0], [5.0, 8.0, 5.0], [5.0, 4.0, 
6.0]]], [[[4.0, 1.0, 8.0], [5.0, 6.0, 1.0], [6.0, 8.0, 7.0]], [[5.0, 4.0, 3.0], [1.0, 6.0, 4.0], [4.0, 5.0, 3.0]], [[7.0, 7.0, 
2.0], [8.0, 7.0, 7.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLUN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 1.0], [8.0, 8.0, 2.0], [1.0, 7.0, 2.0]], [[7.0, 7.0, 6.0], [5.0, 8.0, 8.0], [5.0, 2.0, 
6.0]], [[4.0, 5.0, 6.0], [1.0, 2.0, 8.0], [1.0, 6.0, 1.0]]], [[[4.0, 7.0, 3.0], [5.0, 7.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 
4.0], [2.0, 5.0, 3.0], [3.0, 5.0, 2.0]], [[7.0, 3.0, 3.0], [6.0, 7.0, 5.0], [6.0, 3.0, 2.0]]], [[[1.0, 3.0, 1.0], [4.0, 4.0, 
3.0], [5.0, 5.0, 3.0]], [[2.0, 3.0, 1.0], [5.0, 2.0, 2.0], [7.0, 6.0, 3.0]], [[5.0, 1.0, 6.0], [3.0, 2.0, 8.0], [4.0, 1.0, 
8.0]]], [[[8.0, 2.0, 8.0], [6.0, 5.0, 3.0], [5.0, 2.0, 1.0]], [[1.0, 5.0, 6.0], [6.0, 4.0, 4.0], [6.0, 4.0, 3.0]], [[4.0, 7.0, 
4.0], [4.0, 8.0, 3.0], [3.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 5.0, 1.0], [4.0, 1.0, 8.0]]], [[[3.0, 3.0, 2.0], [3.0, 6.0, 6.0]]]]),self.functionspace)
      arg[2:,2:,:2]=val
      ref=Data(numpy.array([[[[8.0, 7.0, 1.0], [8.0, 8.0, 2.0], [1.0, 7.0, 2.0]], [[7.0, 7.0, 6.0], [5.0, 8.0, 8.0], [5.0, 2.0, 
6.0]], [[4.0, 5.0, 6.0], [1.0, 2.0, 8.0], [1.0, 6.0, 1.0]]], [[[4.0, 7.0, 3.0], [5.0, 7.0, 7.0], [8.0, 6.0, 6.0]], [[5.0, 3.0, 
4.0], [2.0, 5.0, 3.0], [3.0, 5.0, 2.0]], [[7.0, 3.0, 3.0], [6.0, 7.0, 5.0], [6.0, 3.0, 2.0]]], [[[1.0, 3.0, 1.0], [4.0, 4.0, 
3.0], [5.0, 5.0, 3.0]], [[2.0, 3.0, 1.0], [5.0, 2.0, 2.0], [7.0, 6.0, 3.0]], [[5.0, 5.0, 1.0], [4.0, 1.0, 8.0], [4.0, 1.0, 
8.0]]], [[[8.0, 2.0, 8.0], [6.0, 5.0, 3.0], [5.0, 2.0, 1.0]], [[1.0, 5.0, 6.0], [6.0, 4.0, 4.0], [6.0, 4.0, 3.0]], [[3.0, 3.0, 
2.0], [3.0, 6.0, 6.0], [3.0, 5.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLUN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 3.0], [3.0, 5.0, 8.0], [1.0, 8.0, 3.0]], [[7.0, 5.0, 8.0], [8.0, 4.0, 6.0], [5.0, 8.0, 
1.0]], [[7.0, 2.0, 6.0], [8.0, 7.0, 4.0], [5.0, 7.0, 7.0]]], [[[7.0, 2.0, 6.0], [8.0, 2.0, 7.0], [6.0, 4.0, 4.0]], [[4.0, 4.0, 
3.0], [5.0, 5.0, 4.0], [2.0, 1.0, 5.0]], [[6.0, 2.0, 1.0], [1.0, 7.0, 2.0], [7.0, 3.0, 5.0]]], [[[2.0, 7.0, 8.0], [4.0, 1.0, 
2.0], [4.0, 6.0, 8.0]], [[6.0, 8.0, 7.0], [3.0, 8.0, 6.0], [7.0, 3.0, 4.0]], [[4.0, 2.0, 7.0], [5.0, 4.0, 4.0], [4.0, 6.0, 
1.0]]], [[[6.0, 1.0, 1.0], [4.0, 8.0, 4.0], [8.0, 7.0, 5.0]], [[4.0, 5.0, 6.0], [1.0, 1.0, 4.0], [1.0, 4.0, 1.0]], [[6.0, 6.0, 
3.0], [3.0, 8.0, 6.0], [5.0, 5.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 8.0, 4.0], [8.0, 7.0, 8.0]]], [[[2.0, 5.0, 5.0], [4.0, 5.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 6.0, 6.0], [4.0, 6.0, 7.0]]], [[[5.0, 6.0, 8.0], [7.0, 6.0, 7.0]]]])
      arg[2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 3.0], [3.0, 5.0, 8.0], [1.0, 8.0, 3.0]], [[7.0, 5.0, 8.0], [8.0, 4.0, 6.0], [5.0, 
8.0, 1.0]], [[7.0, 2.0, 6.0], [8.0, 7.0, 4.0], [5.0, 7.0, 7.0]]], [[[7.0, 2.0, 6.0], [8.0, 2.0, 7.0], [6.0, 4.0, 4.0]], [[4.0, 
4.0, 3.0], [5.0, 5.0, 4.0], [2.0, 1.0, 5.0]], [[6.0, 2.0, 1.0], [1.0, 7.0, 2.0], [7.0, 3.0, 5.0]]], [[[2.0, 7.0, 8.0], [4.0, 
1.0, 2.0], [4.0, 6.0, 8.0]], [[6.0, 8.0, 7.0], [3.0, 8.0, 6.0], [7.0, 3.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 7.0, 8.0], [4.0, 6.0, 
1.0]]], [[[6.0, 1.0, 1.0], [4.0, 8.0, 4.0], [8.0, 7.0, 5.0]], [[4.0, 5.0, 6.0], [1.0, 1.0, 4.0], [1.0, 4.0, 1.0]], [[2.0, 5.0, 
5.0], [4.0, 5.0, 8.0], [5.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 3.0], [3.0, 5.0, 8.0], [1.0, 8.0, 3.0]], 
[[7.0, 5.0, 8.0], [8.0, 4.0, 6.0], [5.0, 8.0, 1.0]], [[7.0, 2.0, 6.0], [8.0, 7.0, 4.0], [5.0, 7.0, 7.0]]], [[[7.0, 2.0, 6.0], 
[8.0, 2.0, 7.0], [6.0, 4.0, 4.0]], [[4.0, 4.0, 3.0], [5.0, 5.0, 4.0], [2.0, 1.0, 5.0]], [[6.0, 2.0, 1.0], [1.0, 7.0, 2.0], 
[7.0, 3.0, 5.0]]], [[[2.0, 7.0, 8.0], [4.0, 1.0, 2.0], [4.0, 6.0, 8.0]], [[6.0, 8.0, 7.0], [3.0, 8.0, 6.0], [7.0, 3.0, 4.0]], 
[[2.0, 6.0, 6.0], [4.0, 6.0, 7.0], [4.0, 6.0, 1.0]]], [[[6.0, 1.0, 1.0], [4.0, 8.0, 4.0], [8.0, 7.0, 5.0]], [[4.0, 5.0, 6.0], 
[1.0, 1.0, 4.0], [1.0, 4.0, 1.0]], [[5.0, 6.0, 8.0], [7.0, 6.0, 7.0], [5.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PULN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 8.0], [3.0, 2.0, 1.0], [5.0, 8.0, 5.0]], [[6.0, 1.0, 5.0], [1.0, 6.0, 3.0], [3.0, 3.0, 
6.0]], [[7.0, 8.0, 7.0], [1.0, 3.0, 8.0], [3.0, 1.0, 4.0]]], [[[7.0, 2.0, 5.0], [5.0, 3.0, 3.0], [5.0, 1.0, 5.0]], [[4.0, 8.0, 
7.0], [6.0, 8.0, 8.0], [4.0, 2.0, 8.0]], [[5.0, 1.0, 8.0], [5.0, 2.0, 3.0], [7.0, 5.0, 8.0]]], [[[2.0, 8.0, 2.0], [5.0, 3.0, 
1.0], [6.0, 6.0, 7.0]], [[6.0, 5.0, 5.0], [4.0, 1.0, 4.0], [7.0, 5.0, 6.0]], [[4.0, 3.0, 3.0], [7.0, 6.0, 4.0], [3.0, 6.0, 
5.0]]], [[[8.0, 7.0, 2.0], [3.0, 8.0, 7.0], [2.0, 1.0, 6.0]], [[3.0, 6.0, 5.0], [7.0, 4.0, 7.0], [2.0, 5.0, 7.0]], [[4.0, 6.0, 
6.0], [4.0, 6.0, 6.0], [8.0, 7.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[6.0, 7.0, 5.0]], [[3.0, 2.0, 8.0]]])
      arg[2,:2,2:]=val
      ref=Data(numpy.array([[[[5.0, 7.0, 8.0], [3.0, 2.0, 1.0], [5.0, 8.0, 5.0]], [[6.0, 1.0, 5.0], [1.0, 6.0, 3.0], [3.0, 3.0, 
6.0]], [[7.0, 8.0, 7.0], [1.0, 3.0, 8.0], [3.0, 1.0, 4.0]]], [[[7.0, 2.0, 5.0], [5.0, 3.0, 3.0], [5.0, 1.0, 5.0]], [[4.0, 8.0, 
7.0], [6.0, 8.0, 8.0], [4.0, 2.0, 8.0]], [[5.0, 1.0, 8.0], [5.0, 2.0, 3.0], [7.0, 5.0, 8.0]]], [[[2.0, 8.0, 2.0], [5.0, 3.0, 
1.0], [6.0, 7.0, 5.0]], [[6.0, 5.0, 5.0], [4.0, 1.0, 4.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], [7.0, 6.0, 4.0], [3.0, 6.0, 
5.0]]], [[[8.0, 7.0, 2.0], [3.0, 8.0, 7.0], [2.0, 1.0, 6.0]], [[3.0, 6.0, 5.0], [7.0, 4.0, 7.0], [2.0, 5.0, 7.0]], [[4.0, 6.0, 
6.0], [4.0, 6.0, 6.0], [8.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PULN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 7.0], [6.0, 8.0, 4.0], [3.0, 2.0, 7.0]], [[1.0, 8.0, 1.0], [7.0, 5.0, 6.0], [8.0, 2.0, 
3.0]], [[2.0, 6.0, 5.0], [6.0, 2.0, 3.0], [5.0, 2.0, 5.0]]], [[[7.0, 3.0, 1.0], [5.0, 4.0, 1.0], [1.0, 7.0, 5.0]], [[4.0, 2.0, 
2.0], [7.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[2.0, 2.0, 7.0], [2.0, 6.0, 5.0], [3.0, 3.0, 3.0]]], [[[6.0, 1.0, 8.0], [3.0, 4.0, 
2.0], [1.0, 3.0, 6.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 2.0], [1.0, 4.0, 7.0]], [[5.0, 6.0, 2.0], [6.0, 3.0, 6.0], [1.0, 7.0, 
3.0]]], [[[3.0, 8.0, 5.0], [1.0, 2.0, 3.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 1.0], [2.0, 2.0, 4.0], [8.0, 8.0, 6.0]], [[5.0, 1.0, 
5.0], [7.0, 7.0, 1.0], [6.0, 2.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 7.0, 7.0]], [[1.0, 8.0, 8.0]]]),self.functionspace)
      arg[2,:2,2:]=val
      ref=Data(numpy.array([[[[1.0, 5.0, 7.0], [6.0, 8.0, 4.0], [3.0, 2.0, 7.0]], [[1.0, 8.0, 1.0], [7.0, 5.0, 6.0], [8.0, 2.0, 
3.0]], [[2.0, 6.0, 5.0], [6.0, 2.0, 3.0], [5.0, 2.0, 5.0]]], [[[7.0, 3.0, 1.0], [5.0, 4.0, 1.0], [1.0, 7.0, 5.0]], [[4.0, 2.0, 
2.0], [7.0, 7.0, 5.0], [2.0, 2.0, 3.0]], [[2.0, 2.0, 7.0], [2.0, 6.0, 5.0], [3.0, 3.0, 3.0]]], [[[6.0, 1.0, 8.0], [3.0, 4.0, 
2.0], [7.0, 7.0, 7.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 2.0], [1.0, 8.0, 8.0]], [[5.0, 6.0, 2.0], [6.0, 3.0, 6.0], [1.0, 7.0, 
3.0]]], [[[3.0, 8.0, 5.0], [1.0, 2.0, 3.0], [3.0, 2.0, 5.0]], [[7.0, 8.0, 1.0], [2.0, 2.0, 4.0], [8.0, 8.0, 6.0]], [[5.0, 1.0, 
5.0], [7.0, 7.0, 1.0], [6.0, 2.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PULN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 5.0], [1.0, 4.0, 1.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 3.0], [6.0, 6.0, 2.0], [6.0, 1.0, 
3.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 4.0], [3.0, 1.0, 6.0]]], [[[8.0, 7.0, 3.0], [1.0, 2.0, 8.0], [6.0, 5.0, 1.0]], [[4.0, 2.0, 
3.0], [6.0, 1.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 7.0], [2.0, 3.0, 2.0], [4.0, 4.0, 8.0]]], [[[6.0, 2.0, 5.0], [6.0, 4.0, 
6.0], [6.0, 7.0, 7.0]], [[4.0, 8.0, 1.0], [1.0, 8.0, 3.0], [2.0, 7.0, 7.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 2.0], [6.0, 7.0, 
3.0]]], [[[7.0, 8.0, 8.0], [4.0, 1.0, 5.0], [7.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 1.0], [4.0, 2.0, 5.0]], [[3.0, 3.0, 
4.0], [1.0, 1.0, 8.0], [2.0, 8.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 3.0, 8.0]], [[6.0, 1.0, 2.0]]])+(1.-msk_val)*numpy.array([[[7.0, 1.0, 2.0]], [[8.0, 7.0, 
5.0]]])
      arg[2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [1.0, 4.0, 1.0], [2.0, 1.0, 3.0]], [[3.0, 1.0, 3.0], [6.0, 6.0, 2.0], [6.0, 
1.0, 3.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 4.0], [3.0, 1.0, 6.0]]], [[[8.0, 7.0, 3.0], [1.0, 2.0, 8.0], [6.0, 5.0, 1.0]], [[4.0, 
2.0, 3.0], [6.0, 1.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 7.0], [2.0, 3.0, 2.0], [4.0, 4.0, 8.0]]], [[[6.0, 2.0, 5.0], [6.0, 
4.0, 6.0], [1.0, 3.0, 8.0]], [[4.0, 8.0, 1.0], [1.0, 8.0, 3.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 2.0], [6.0, 7.0, 
3.0]]], [[[7.0, 8.0, 8.0], [4.0, 1.0, 5.0], [7.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 1.0], [4.0, 2.0, 5.0]], [[3.0, 3.0, 
4.0], [1.0, 1.0, 8.0], [2.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 5.0], [1.0, 4.0, 1.0], [2.0, 1.0, 3.0]], 
[[3.0, 1.0, 3.0], [6.0, 6.0, 2.0], [6.0, 1.0, 3.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 4.0], [3.0, 1.0, 6.0]]], [[[8.0, 7.0, 3.0], 
[1.0, 2.0, 8.0], [6.0, 5.0, 1.0]], [[4.0, 2.0, 3.0], [6.0, 1.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 7.0], [2.0, 3.0, 2.0], 
[4.0, 4.0, 8.0]]], [[[6.0, 2.0, 5.0], [6.0, 4.0, 6.0], [7.0, 1.0, 2.0]], [[4.0, 8.0, 1.0], [1.0, 8.0, 3.0], [8.0, 7.0, 5.0]], 
[[5.0, 1.0, 2.0], [2.0, 1.0, 2.0], [6.0, 7.0, 3.0]]], [[[7.0, 8.0, 8.0], [4.0, 1.0, 5.0], [7.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], 
[6.0, 1.0, 1.0], [4.0, 2.0, 5.0]], [[3.0, 3.0, 4.0], [1.0, 1.0, 8.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PCRN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 5.0], [1.0, 5.0, 1.0], [7.0, 1.0, 3.0]], [[7.0, 4.0, 5.0], [3.0, 5.0, 5.0], [5.0, 6.0, 
5.0]], [[1.0, 3.0, 4.0], [5.0, 6.0, 1.0], [4.0, 3.0, 3.0]]], [[[1.0, 2.0, 1.0], [2.0, 8.0, 2.0], [2.0, 2.0, 2.0]], [[6.0, 2.0, 
2.0], [2.0, 2.0, 6.0], [3.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [6.0, 5.0, 8.0], [1.0, 7.0, 5.0]]], [[[2.0, 2.0, 3.0], [5.0, 5.0, 
4.0], [1.0, 4.0, 3.0]], [[6.0, 3.0, 7.0], [7.0, 1.0, 7.0], [2.0, 8.0, 3.0]], [[3.0, 8.0, 1.0], [6.0, 2.0, 8.0], [1.0, 5.0, 
6.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 7.0], [6.0, 6.0, 8.0]], [[4.0, 7.0, 1.0], [6.0, 2.0, 6.0], [5.0, 2.0, 1.0]], [[6.0, 4.0, 
1.0], [1.0, 1.0, 4.0], [1.0, 2.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[6.0, 7.0, 5.0]], [[7.0, 5.0, 2.0]], [[7.0, 2.0, 2.0]]])
      arg[2,:,1:2]=val
      ref=Data(numpy.array([[[[1.0, 5.0, 5.0], [1.0, 5.0, 1.0], [7.0, 1.0, 3.0]], [[7.0, 4.0, 5.0], [3.0, 5.0, 5.0], [5.0, 6.0, 
5.0]], [[1.0, 3.0, 4.0], [5.0, 6.0, 1.0], [4.0, 3.0, 3.0]]], [[[1.0, 2.0, 1.0], [2.0, 8.0, 2.0], [2.0, 2.0, 2.0]], [[6.0, 2.0, 
2.0], [2.0, 2.0, 6.0], [3.0, 6.0, 5.0]], [[7.0, 3.0, 2.0], [6.0, 5.0, 8.0], [1.0, 7.0, 5.0]]], [[[2.0, 2.0, 3.0], [6.0, 7.0, 
5.0], [1.0, 4.0, 3.0]], [[6.0, 3.0, 7.0], [7.0, 5.0, 2.0], [2.0, 8.0, 3.0]], [[3.0, 8.0, 1.0], [7.0, 2.0, 2.0], [1.0, 5.0, 
6.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 7.0], [6.0, 6.0, 8.0]], [[4.0, 7.0, 1.0], [6.0, 2.0, 6.0], [5.0, 2.0, 1.0]], [[6.0, 4.0, 
1.0], [1.0, 1.0, 4.0], [1.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PCRN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 8.0], [3.0, 5.0, 3.0], [5.0, 5.0, 5.0]], [[1.0, 1.0, 8.0], [3.0, 7.0, 7.0], [6.0, 5.0, 
6.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 1.0], [5.0, 3.0, 3.0]]], [[[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 7.0, 1.0]], [[8.0, 2.0, 
7.0], [3.0, 8.0, 8.0], [1.0, 7.0, 8.0]], [[8.0, 4.0, 7.0], [8.0, 1.0, 6.0], [1.0, 3.0, 4.0]]], [[[8.0, 3.0, 6.0], [1.0, 6.0, 
1.0], [5.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [1.0, 8.0, 1.0], [8.0, 3.0, 3.0]], [[7.0, 5.0, 4.0], [7.0, 4.0, 6.0], [4.0, 4.0, 
3.0]]], [[[2.0, 4.0, 3.0], [7.0, 4.0, 5.0], [8.0, 3.0, 4.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 7.0], [8.0, 4.0, 5.0]], [[7.0, 2.0, 
8.0], [1.0, 3.0, 2.0], [1.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[8.0, 2.0, 3.0]], [[7.0, 4.0, 5.0]], [[1.0, 2.0, 5.0]]]),self.functionspace)
      arg[2,:,1:2]=val
      ref=Data(numpy.array([[[[8.0, 1.0, 8.0], [3.0, 5.0, 3.0], [5.0, 5.0, 5.0]], [[1.0, 1.0, 8.0], [3.0, 7.0, 7.0], [6.0, 5.0, 
6.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 1.0], [5.0, 3.0, 3.0]]], [[[1.0, 2.0, 1.0], [5.0, 3.0, 3.0], [5.0, 7.0, 1.0]], [[8.0, 2.0, 
7.0], [3.0, 8.0, 8.0], [1.0, 7.0, 8.0]], [[8.0, 4.0, 7.0], [8.0, 1.0, 6.0], [1.0, 3.0, 4.0]]], [[[8.0, 3.0, 6.0], [8.0, 2.0, 
3.0], [5.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [7.0, 4.0, 5.0], [8.0, 3.0, 3.0]], [[7.0, 5.0, 4.0], [1.0, 2.0, 5.0], [4.0, 4.0, 
3.0]]], [[[2.0, 4.0, 3.0], [7.0, 4.0, 5.0], [8.0, 3.0, 4.0]], [[3.0, 3.0, 3.0], [4.0, 8.0, 7.0], [8.0, 4.0, 5.0]], [[7.0, 2.0, 
8.0], [1.0, 3.0, 2.0], [1.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PCRN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 7.0], [4.0, 7.0, 1.0], [6.0, 3.0, 4.0]], [[7.0, 2.0, 1.0], [6.0, 8.0, 3.0], [6.0, 7.0, 
4.0]], [[4.0, 6.0, 7.0], [4.0, 4.0, 6.0], [5.0, 6.0, 6.0]]], [[[8.0, 3.0, 7.0], [7.0, 5.0, 8.0], [7.0, 5.0, 7.0]], [[4.0, 1.0, 
5.0], [2.0, 4.0, 1.0], [6.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [5.0, 4.0, 8.0], [3.0, 3.0, 6.0]]], [[[7.0, 6.0, 4.0], [6.0, 3.0, 
3.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 6.0], [2.0, 1.0, 7.0], [3.0, 3.0, 5.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 7.0], [8.0, 6.0, 
6.0]]], [[[2.0, 8.0, 2.0], [5.0, 7.0, 4.0], [7.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], [4.0, 2.0, 2.0], [6.0, 5.0, 4.0]], [[3.0, 4.0, 
7.0], [1.0, 4.0, 8.0], [4.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 3.0, 1.0]], [[4.0, 3.0, 1.0]], [[3.0, 8.0, 5.0]]])+(1.-msk_val)*numpy.array([[[2.0, 1.0, 
6.0]], [[5.0, 8.0, 6.0]], [[4.0, 1.0, 4.0]]])
      arg[2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 7.0], [4.0, 7.0, 1.0], [6.0, 3.0, 4.0]], [[7.0, 2.0, 1.0], [6.0, 8.0, 3.0], [6.0, 
7.0, 4.0]], [[4.0, 6.0, 7.0], [4.0, 4.0, 6.0], [5.0, 6.0, 6.0]]], [[[8.0, 3.0, 7.0], [7.0, 5.0, 8.0], [7.0, 5.0, 7.0]], [[4.0, 
1.0, 5.0], [2.0, 4.0, 1.0], [6.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [5.0, 4.0, 8.0], [3.0, 3.0, 6.0]]], [[[7.0, 6.0, 4.0], [1.0, 
3.0, 1.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 6.0], [4.0, 3.0, 1.0], [3.0, 3.0, 5.0]], [[5.0, 5.0, 8.0], [3.0, 8.0, 5.0], [8.0, 6.0, 
6.0]]], [[[2.0, 8.0, 2.0], [5.0, 7.0, 4.0], [7.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], [4.0, 2.0, 2.0], [6.0, 5.0, 4.0]], [[3.0, 4.0, 
7.0], [1.0, 4.0, 8.0], [4.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 7.0], [4.0, 7.0, 1.0], [6.0, 3.0, 4.0]], 
[[7.0, 2.0, 1.0], [6.0, 8.0, 3.0], [6.0, 7.0, 4.0]], [[4.0, 6.0, 7.0], [4.0, 4.0, 6.0], [5.0, 6.0, 6.0]]], [[[8.0, 3.0, 7.0], 
[7.0, 5.0, 8.0], [7.0, 5.0, 7.0]], [[4.0, 1.0, 5.0], [2.0, 4.0, 1.0], [6.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [5.0, 4.0, 8.0], 
[3.0, 3.0, 6.0]]], [[[7.0, 6.0, 4.0], [2.0, 1.0, 6.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 6.0], [5.0, 8.0, 6.0], [3.0, 3.0, 5.0]], 
[[5.0, 5.0, 8.0], [4.0, 1.0, 4.0], [8.0, 6.0, 6.0]]], [[[2.0, 8.0, 2.0], [5.0, 7.0, 4.0], [7.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], 
[4.0, 2.0, 2.0], [6.0, 5.0, 4.0]], [[3.0, 4.0, 7.0], [1.0, 4.0, 8.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCUN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 8.0], [3.0, 1.0, 8.0], [4.0, 8.0, 3.0]], [[8.0, 6.0, 7.0], [5.0, 6.0, 3.0], [7.0, 5.0, 
4.0]], [[2.0, 6.0, 8.0], [2.0, 6.0, 5.0], [3.0, 3.0, 7.0]]], [[[1.0, 4.0, 2.0], [7.0, 6.0, 3.0], [6.0, 5.0, 4.0]], [[3.0, 8.0, 
6.0], [5.0, 6.0, 4.0], [3.0, 3.0, 7.0]], [[4.0, 6.0, 3.0], [6.0, 3.0, 6.0], [5.0, 8.0, 7.0]]], [[[2.0, 1.0, 7.0], [6.0, 8.0, 
6.0], [4.0, 4.0, 5.0]], [[1.0, 6.0, 3.0], [3.0, 6.0, 2.0], [1.0, 2.0, 4.0]], [[7.0, 1.0, 5.0], [2.0, 3.0, 3.0], [7.0, 2.0, 
7.0]]], [[[8.0, 6.0, 4.0], [6.0, 4.0, 4.0], [3.0, 3.0, 5.0]], [[5.0, 2.0, 4.0], [6.0, 6.0, 2.0], [4.0, 6.0, 1.0]], [[3.0, 4.0, 
6.0], [6.0, 2.0, 6.0], [6.0, 7.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 2.0, 5.0], [7.0, 6.0, 8.0]], [[4.0, 6.0, 2.0], [7.0, 4.0, 7.0]], [[5.0, 7.0, 2.0], [5.0, 7.0, 
7.0]]], [[[8.0, 1.0, 8.0], [4.0, 4.0, 2.0]], [[6.0, 2.0, 3.0], [7.0, 5.0, 6.0]], [[7.0, 2.0, 1.0], [8.0, 8.0, 2.0]]], [[[3.0, 
5.0, 6.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 8.0], [7.0, 2.0, 4.0]], [[4.0, 8.0, 2.0], [3.0, 4.0, 8.0]]], [[[1.0, 5.0, 7.0], [1.0, 
4.0, 5.0]], [[4.0, 2.0, 4.0], [2.0, 4.0, 1.0]], [[2.0, 8.0, 4.0], [3.0, 5.0, 4.0]]]])
      arg[:,:,:2]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 5.0], [7.0, 6.0, 8.0], [4.0, 8.0, 3.0]], [[4.0, 6.0, 2.0], [7.0, 4.0, 7.0], [7.0, 5.0, 
4.0]], [[5.0, 7.0, 2.0], [5.0, 7.0, 7.0], [3.0, 3.0, 7.0]]], [[[8.0, 1.0, 8.0], [4.0, 4.0, 2.0], [6.0, 5.0, 4.0]], [[6.0, 2.0, 
3.0], [7.0, 5.0, 6.0], [3.0, 3.0, 7.0]], [[7.0, 2.0, 1.0], [8.0, 8.0, 2.0], [5.0, 8.0, 7.0]]], [[[3.0, 5.0, 6.0], [5.0, 3.0, 
8.0], [4.0, 4.0, 5.0]], [[5.0, 5.0, 8.0], [7.0, 2.0, 4.0], [1.0, 2.0, 4.0]], [[4.0, 8.0, 2.0], [3.0, 4.0, 8.0], [7.0, 2.0, 
7.0]]], [[[1.0, 5.0, 7.0], [1.0, 4.0, 5.0], [3.0, 3.0, 5.0]], [[4.0, 2.0, 4.0], [2.0, 4.0, 1.0], [4.0, 6.0, 1.0]], [[2.0, 8.0, 
4.0], [3.0, 5.0, 4.0], [6.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCUN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 3.0], [7.0, 8.0, 2.0], [6.0, 1.0, 2.0]], [[3.0, 5.0, 4.0], [3.0, 3.0, 1.0], [2.0, 8.0, 
4.0]], [[4.0, 7.0, 7.0], [6.0, 7.0, 8.0], [3.0, 4.0, 2.0]]], [[[5.0, 1.0, 5.0], [6.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[3.0, 3.0, 
3.0], [2.0, 5.0, 7.0], [5.0, 8.0, 4.0]], [[4.0, 8.0, 6.0], [5.0, 1.0, 7.0], [4.0, 6.0, 7.0]]], [[[4.0, 1.0, 7.0], [5.0, 1.0, 
4.0], [7.0, 6.0, 6.0]], [[8.0, 6.0, 1.0], [5.0, 3.0, 7.0], [8.0, 2.0, 2.0]], [[7.0, 3.0, 8.0], [1.0, 5.0, 6.0], [3.0, 3.0, 
1.0]]], [[[2.0, 1.0, 4.0], [5.0, 1.0, 7.0], [3.0, 6.0, 7.0]], [[1.0, 3.0, 6.0], [3.0, 1.0, 4.0], [2.0, 8.0, 6.0]], [[8.0, 4.0, 
3.0], [8.0, 7.0, 6.0], [1.0, 2.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 8.0, 4.0], [1.0, 2.0, 8.0]], [[4.0, 8.0, 2.0], [1.0, 6.0, 8.0]], [[8.0, 5.0, 4.0], [8.0, 
8.0, 1.0]]], [[[7.0, 6.0, 8.0], [1.0, 4.0, 1.0]], [[3.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 4.0]]], 
[[[6.0, 5.0, 3.0], [6.0, 7.0, 8.0]], [[1.0, 4.0, 6.0], [1.0, 7.0, 3.0]], [[8.0, 4.0, 1.0], [3.0, 7.0, 8.0]]], [[[5.0, 2.0, 
8.0], [5.0, 5.0, 7.0]], [[4.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[1.0, 3.0, 7.0], [7.0, 3.0, 3.0]]]]),self.functionspace)
      arg[:,:,:2]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 4.0], [1.0, 2.0, 8.0], [6.0, 1.0, 2.0]], [[4.0, 8.0, 2.0], [1.0, 6.0, 8.0], [2.0, 8.0, 
4.0]], [[8.0, 5.0, 4.0], [8.0, 8.0, 1.0], [3.0, 4.0, 2.0]]], [[[7.0, 6.0, 8.0], [1.0, 4.0, 1.0], [3.0, 8.0, 6.0]], [[3.0, 3.0, 
2.0], [7.0, 7.0, 4.0], [5.0, 8.0, 4.0]], [[5.0, 7.0, 7.0], [2.0, 4.0, 4.0], [4.0, 6.0, 7.0]]], [[[6.0, 5.0, 3.0], [6.0, 7.0, 
8.0], [7.0, 6.0, 6.0]], [[1.0, 4.0, 6.0], [1.0, 7.0, 3.0], [8.0, 2.0, 2.0]], [[8.0, 4.0, 1.0], [3.0, 7.0, 8.0], [3.0, 3.0, 
1.0]]], [[[5.0, 2.0, 8.0], [5.0, 5.0, 7.0], [3.0, 6.0, 7.0]], [[4.0, 2.0, 3.0], [8.0, 1.0, 5.0], [2.0, 8.0, 6.0]], [[1.0, 3.0, 
7.0], [7.0, 3.0, 3.0], [1.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCUN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 6.0], [8.0, 7.0, 3.0], [4.0, 5.0, 5.0]], [[5.0, 4.0, 3.0], [2.0, 3.0, 5.0], [3.0, 5.0, 
3.0]], [[2.0, 2.0, 1.0], [5.0, 7.0, 3.0], [2.0, 2.0, 3.0]]], [[[8.0, 6.0, 2.0], [2.0, 4.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 7.0, 
8.0], [4.0, 1.0, 6.0], [8.0, 1.0, 7.0]], [[4.0, 7.0, 2.0], [7.0, 6.0, 3.0], [8.0, 3.0, 4.0]]], [[[7.0, 4.0, 1.0], [2.0, 3.0, 
2.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 2.0], [6.0, 3.0, 2.0], [4.0, 4.0, 7.0]], [[7.0, 2.0, 3.0], [5.0, 4.0, 6.0], [3.0, 6.0, 
1.0]]], [[[5.0, 7.0, 6.0], [6.0, 1.0, 1.0], [4.0, 2.0, 5.0]], [[3.0, 8.0, 7.0], [6.0, 3.0, 7.0], [7.0, 1.0, 4.0]], [[3.0, 2.0, 
8.0], [7.0, 7.0, 2.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 4.0, 8.0], [5.0, 4.0, 4.0]], [[3.0, 5.0, 3.0], [6.0, 2.0, 7.0]], [[6.0, 2.0, 1.0], [5.0, 
3.0, 6.0]]], [[[8.0, 6.0, 7.0], [7.0, 2.0, 1.0]], [[2.0, 2.0, 3.0], [7.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [3.0, 1.0, 7.0]]], 
[[[6.0, 8.0, 6.0], [5.0, 7.0, 5.0]], [[8.0, 5.0, 3.0], [6.0, 6.0, 7.0]], [[5.0, 2.0, 6.0], [2.0, 7.0, 7.0]]], [[[8.0, 5.0, 
8.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 1.0], [2.0, 7.0, 8.0]], [[8.0, 3.0, 2.0], [6.0, 1.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 8.0, 5.0], [4.0, 6.0, 8.0]], [[4.0, 3.0, 1.0], [1.0, 5.0, 1.0]], [[8.0, 1.0, 6.0], 
[6.0, 8.0, 3.0]]], [[[2.0, 7.0, 7.0], [3.0, 3.0, 3.0]], [[5.0, 7.0, 5.0], [5.0, 1.0, 5.0]], [[4.0, 5.0, 3.0], [1.0, 7.0, 
2.0]]], [[[5.0, 5.0, 1.0], [8.0, 1.0, 8.0]], [[7.0, 5.0, 3.0], [4.0, 3.0, 5.0]], [[4.0, 1.0, 2.0], [4.0, 2.0, 6.0]]], [[[5.0, 
7.0, 5.0], [5.0, 2.0, 2.0]], [[5.0, 4.0, 7.0], [4.0, 7.0, 6.0]], [[4.0, 3.0, 7.0], [3.0, 1.0, 7.0]]]])
      arg[:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 8.0], [5.0, 4.0, 4.0], [4.0, 5.0, 5.0]], [[3.0, 5.0, 3.0], [6.0, 2.0, 7.0], [3.0, 
5.0, 3.0]], [[6.0, 2.0, 1.0], [5.0, 3.0, 6.0], [2.0, 2.0, 3.0]]], [[[8.0, 6.0, 7.0], [7.0, 2.0, 1.0], [3.0, 7.0, 3.0]], [[2.0, 
2.0, 3.0], [7.0, 7.0, 3.0], [8.0, 1.0, 7.0]], [[2.0, 3.0, 6.0], [3.0, 1.0, 7.0], [8.0, 3.0, 4.0]]], [[[6.0, 8.0, 6.0], [5.0, 
7.0, 5.0], [5.0, 4.0, 2.0]], [[8.0, 5.0, 3.0], [6.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[5.0, 2.0, 6.0], [2.0, 7.0, 7.0], [3.0, 6.0, 
1.0]]], [[[8.0, 5.0, 8.0], [8.0, 2.0, 2.0], [4.0, 2.0, 5.0]], [[7.0, 1.0, 1.0], [2.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[8.0, 3.0, 
2.0], [6.0, 1.0, 5.0], [7.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 5.0], [4.0, 6.0, 8.0], [4.0, 5.0, 5.0]], 
[[4.0, 3.0, 1.0], [1.0, 5.0, 1.0], [3.0, 5.0, 3.0]], [[8.0, 1.0, 6.0], [6.0, 8.0, 3.0], [2.0, 2.0, 3.0]]], [[[2.0, 7.0, 7.0], 
[3.0, 3.0, 3.0], [3.0, 7.0, 3.0]], [[5.0, 7.0, 5.0], [5.0, 1.0, 5.0], [8.0, 1.0, 7.0]], [[4.0, 5.0, 3.0], [1.0, 7.0, 2.0], 
[8.0, 3.0, 4.0]]], [[[5.0, 5.0, 1.0], [8.0, 1.0, 8.0], [5.0, 4.0, 2.0]], [[7.0, 5.0, 3.0], [4.0, 3.0, 5.0], [4.0, 4.0, 7.0]], 
[[4.0, 1.0, 2.0], [4.0, 2.0, 6.0], [3.0, 6.0, 1.0]]], [[[5.0, 7.0, 5.0], [5.0, 2.0, 2.0], [4.0, 2.0, 5.0]], [[5.0, 4.0, 7.0], 
[4.0, 7.0, 6.0], [7.0, 1.0, 4.0]], [[4.0, 3.0, 7.0], [3.0, 1.0, 7.0], [7.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 6.0], [3.0, 8.0, 1.0], [5.0, 1.0, 2.0]], [[6.0, 4.0, 7.0], [2.0, 8.0, 8.0], [4.0, 2.0, 
3.0]], [[4.0, 6.0, 2.0], [4.0, 3.0, 1.0], [4.0, 6.0, 7.0]]], [[[7.0, 8.0, 7.0], [4.0, 7.0, 4.0], [5.0, 3.0, 3.0]], [[5.0, 3.0, 
3.0], [7.0, 2.0, 6.0], [6.0, 1.0, 7.0]], [[8.0, 1.0, 8.0], [2.0, 8.0, 3.0], [5.0, 5.0, 4.0]]], [[[4.0, 6.0, 8.0], [7.0, 6.0, 
1.0], [4.0, 7.0, 4.0]], [[8.0, 7.0, 6.0], [6.0, 7.0, 2.0], [6.0, 7.0, 5.0]], [[7.0, 2.0, 8.0], [5.0, 2.0, 2.0], [7.0, 6.0, 
4.0]]], [[[1.0, 3.0, 6.0], [4.0, 8.0, 4.0], [2.0, 4.0, 5.0]], [[3.0, 1.0, 7.0], [7.0, 6.0, 5.0], [7.0, 1.0, 1.0]], [[7.0, 3.0, 
3.0], [6.0, 8.0, 4.0], [1.0, 4.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 6.0, 4.0]]], [[[5.0, 4.0, 3.0]]], [[[8.0, 7.0, 1.0]]]])
      arg[:3,1:2,2:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 6.0], [3.0, 8.0, 1.0], [5.0, 1.0, 2.0]], [[6.0, 4.0, 7.0], [2.0, 8.0, 8.0], [8.0, 6.0, 
4.0]], [[4.0, 6.0, 2.0], [4.0, 3.0, 1.0], [4.0, 6.0, 7.0]]], [[[7.0, 8.0, 7.0], [4.0, 7.0, 4.0], [5.0, 3.0, 3.0]], [[5.0, 3.0, 
3.0], [7.0, 2.0, 6.0], [5.0, 4.0, 3.0]], [[8.0, 1.0, 8.0], [2.0, 8.0, 3.0], [5.0, 5.0, 4.0]]], [[[4.0, 6.0, 8.0], [7.0, 6.0, 
1.0], [4.0, 7.0, 4.0]], [[8.0, 7.0, 6.0], [6.0, 7.0, 2.0], [8.0, 7.0, 1.0]], [[7.0, 2.0, 8.0], [5.0, 2.0, 2.0], [7.0, 6.0, 
4.0]]], [[[1.0, 3.0, 6.0], [4.0, 8.0, 4.0], [2.0, 4.0, 5.0]], [[3.0, 1.0, 7.0], [7.0, 6.0, 5.0], [7.0, 1.0, 1.0]], [[7.0, 3.0, 
3.0], [6.0, 8.0, 4.0], [1.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URLN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 5.0], [8.0, 7.0, 2.0], [3.0, 8.0, 6.0]], [[6.0, 4.0, 1.0], [6.0, 1.0, 1.0], [1.0, 4.0, 
8.0]], [[1.0, 5.0, 3.0], [2.0, 5.0, 5.0], [7.0, 6.0, 3.0]]], [[[4.0, 5.0, 6.0], [7.0, 2.0, 5.0], [3.0, 4.0, 1.0]], [[3.0, 8.0, 
7.0], [6.0, 7.0, 7.0], [1.0, 6.0, 6.0]], [[4.0, 2.0, 5.0], [8.0, 2.0, 8.0], [8.0, 1.0, 8.0]]], [[[7.0, 3.0, 3.0], [3.0, 8.0, 
6.0], [8.0, 2.0, 7.0]], [[2.0, 5.0, 5.0], [5.0, 3.0, 7.0], [3.0, 6.0, 5.0]], [[2.0, 5.0, 7.0], [5.0, 4.0, 7.0], [2.0, 7.0, 
7.0]]], [[[3.0, 3.0, 3.0], [6.0, 4.0, 1.0], [5.0, 3.0, 3.0]], [[6.0, 3.0, 1.0], [5.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[3.0, 5.0, 
5.0], [2.0, 4.0, 2.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 2.0, 5.0]]], [[[7.0, 3.0, 5.0]]], [[[2.0, 7.0, 8.0]]]]),self.functionspace)
      arg[:3,1:2,2:]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 5.0], [8.0, 7.0, 2.0], [3.0, 8.0, 6.0]], [[6.0, 4.0, 1.0], [6.0, 1.0, 1.0], [6.0, 2.0, 
5.0]], [[1.0, 5.0, 3.0], [2.0, 5.0, 5.0], [7.0, 6.0, 3.0]]], [[[4.0, 5.0, 6.0], [7.0, 2.0, 5.0], [3.0, 4.0, 1.0]], [[3.0, 8.0, 
7.0], [6.0, 7.0, 7.0], [7.0, 3.0, 5.0]], [[4.0, 2.0, 5.0], [8.0, 2.0, 8.0], [8.0, 1.0, 8.0]]], [[[7.0, 3.0, 3.0], [3.0, 8.0, 
6.0], [8.0, 2.0, 7.0]], [[2.0, 5.0, 5.0], [5.0, 3.0, 7.0], [2.0, 7.0, 8.0]], [[2.0, 5.0, 7.0], [5.0, 4.0, 7.0], [2.0, 7.0, 
7.0]]], [[[3.0, 3.0, 3.0], [6.0, 4.0, 1.0], [5.0, 3.0, 3.0]], [[6.0, 3.0, 1.0], [5.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[3.0, 5.0, 
5.0], [2.0, 4.0, 2.0], [8.0, 1.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URLN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 1.0], [5.0, 1.0, 5.0], [7.0, 3.0, 5.0]], [[8.0, 2.0, 4.0], [3.0, 7.0, 1.0], [4.0, 8.0, 
1.0]], [[4.0, 1.0, 2.0], [8.0, 1.0, 5.0], [6.0, 2.0, 6.0]]], [[[2.0, 1.0, 8.0], [1.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[2.0, 6.0, 
4.0], [3.0, 7.0, 6.0], [7.0, 5.0, 3.0]], [[1.0, 2.0, 4.0], [4.0, 1.0, 7.0], [4.0, 5.0, 7.0]]], [[[6.0, 5.0, 6.0], [6.0, 3.0, 
7.0], [3.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [3.0, 5.0, 3.0], [5.0, 6.0, 7.0]], [[2.0, 2.0, 3.0], [2.0, 4.0, 7.0], [2.0, 4.0, 
2.0]]], [[[2.0, 4.0, 3.0], [7.0, 4.0, 3.0], [1.0, 8.0, 7.0]], [[5.0, 5.0, 3.0], [3.0, 2.0, 2.0], [2.0, 1.0, 6.0]], [[6.0, 5.0, 
5.0], [2.0, 4.0, 2.0], [7.0, 1.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 3.0, 3.0]]], [[[8.0, 7.0, 6.0]]], [[[3.0, 4.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 4.0, 1.0]]], [[[3.0, 1.0, 4.0]]], [[[4.0, 1.0, 4.0]]]])
      arg[:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 1.0], [5.0, 1.0, 5.0], [7.0, 3.0, 5.0]], [[8.0, 2.0, 4.0], [3.0, 7.0, 1.0], [5.0, 
3.0, 3.0]], [[4.0, 1.0, 2.0], [8.0, 1.0, 5.0], [6.0, 2.0, 6.0]]], [[[2.0, 1.0, 8.0], [1.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[2.0, 
6.0, 4.0], [3.0, 7.0, 6.0], [8.0, 7.0, 6.0]], [[1.0, 2.0, 4.0], [4.0, 1.0, 7.0], [4.0, 5.0, 7.0]]], [[[6.0, 5.0, 6.0], [6.0, 
3.0, 7.0], [3.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [3.0, 5.0, 3.0], [3.0, 4.0, 5.0]], [[2.0, 2.0, 3.0], [2.0, 4.0, 7.0], [2.0, 4.0, 
2.0]]], [[[2.0, 4.0, 3.0], [7.0, 4.0, 3.0], [1.0, 8.0, 7.0]], [[5.0, 5.0, 3.0], [3.0, 2.0, 2.0], [2.0, 1.0, 6.0]], [[6.0, 5.0, 
5.0], [2.0, 4.0, 2.0], [7.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 1.0], [5.0, 1.0, 5.0], [7.0, 3.0, 5.0]], 
[[8.0, 2.0, 4.0], [3.0, 7.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 1.0, 2.0], [8.0, 1.0, 5.0], [6.0, 2.0, 6.0]]], [[[2.0, 1.0, 8.0], 
[1.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[2.0, 6.0, 4.0], [3.0, 7.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 2.0, 4.0], [4.0, 1.0, 7.0], 
[4.0, 5.0, 7.0]]], [[[6.0, 5.0, 6.0], [6.0, 3.0, 7.0], [3.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [3.0, 5.0, 3.0], [4.0, 1.0, 4.0]], 
[[2.0, 2.0, 3.0], [2.0, 4.0, 7.0], [2.0, 4.0, 2.0]]], [[[2.0, 4.0, 3.0], [7.0, 4.0, 3.0], [1.0, 8.0, 7.0]], [[5.0, 5.0, 3.0], 
[3.0, 2.0, 2.0], [2.0, 1.0, 6.0]], [[6.0, 5.0, 5.0], [2.0, 4.0, 2.0], [7.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CUUN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 1.0], [8.0, 4.0, 3.0], [6.0, 4.0, 8.0]], [[2.0, 1.0, 7.0], [7.0, 7.0, 7.0], [8.0, 1.0, 
5.0]], [[5.0, 6.0, 8.0], [2.0, 1.0, 7.0], [4.0, 2.0, 7.0]]], [[[3.0, 3.0, 5.0], [5.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[3.0, 1.0, 
8.0], [6.0, 6.0, 4.0], [4.0, 3.0, 8.0]], [[8.0, 1.0, 3.0], [4.0, 1.0, 8.0], [6.0, 5.0, 6.0]]], [[[4.0, 4.0, 5.0], [5.0, 2.0, 
7.0], [2.0, 3.0, 5.0]], [[4.0, 4.0, 2.0], [3.0, 6.0, 2.0], [3.0, 6.0, 5.0]], [[7.0, 6.0, 5.0], [6.0, 3.0, 7.0], [5.0, 5.0, 
3.0]]], [[[7.0, 5.0, 4.0], [4.0, 5.0, 1.0], [6.0, 5.0, 2.0]], [[5.0, 5.0, 2.0], [5.0, 1.0, 5.0], [5.0, 6.0, 2.0]], [[5.0, 6.0, 
1.0], [7.0, 1.0, 3.0], [6.0, 6.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 3.0, 6.0], [1.0, 6.0, 7.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 3.0]]], [[[6.0, 1.0, 2.0], [8.0, 8.0, 
7.0]], [[8.0, 1.0, 6.0], [3.0, 5.0, 4.0]]], [[[6.0, 3.0, 4.0], [8.0, 2.0, 2.0]], [[3.0, 7.0, 7.0], [5.0, 3.0, 7.0]]], [[[6.0, 
2.0, 3.0], [4.0, 8.0, 3.0]], [[7.0, 3.0, 3.0], [4.0, 4.0, 6.0]]]])
      arg[:,:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 6.0], [1.0, 6.0, 7.0], [6.0, 4.0, 8.0]], [[5.0, 2.0, 7.0], [6.0, 6.0, 3.0], [8.0, 1.0, 
5.0]], [[5.0, 6.0, 8.0], [2.0, 1.0, 7.0], [4.0, 2.0, 7.0]]], [[[6.0, 1.0, 2.0], [8.0, 8.0, 7.0], [7.0, 8.0, 5.0]], [[8.0, 1.0, 
6.0], [3.0, 5.0, 4.0], [4.0, 3.0, 8.0]], [[8.0, 1.0, 3.0], [4.0, 1.0, 8.0], [6.0, 5.0, 6.0]]], [[[6.0, 3.0, 4.0], [8.0, 2.0, 
2.0], [2.0, 3.0, 5.0]], [[3.0, 7.0, 7.0], [5.0, 3.0, 7.0], [3.0, 6.0, 5.0]], [[7.0, 6.0, 5.0], [6.0, 3.0, 7.0], [5.0, 5.0, 
3.0]]], [[[6.0, 2.0, 3.0], [4.0, 8.0, 3.0], [6.0, 5.0, 2.0]], [[7.0, 3.0, 3.0], [4.0, 4.0, 6.0], [5.0, 6.0, 2.0]], [[5.0, 6.0, 
1.0], [7.0, 1.0, 3.0], [6.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CUUN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 4.0], [5.0, 6.0, 1.0], [7.0, 3.0, 6.0]], [[8.0, 6.0, 5.0], [6.0, 2.0, 6.0], [3.0, 4.0, 
5.0]], [[6.0, 1.0, 6.0], [6.0, 2.0, 8.0], [2.0, 1.0, 4.0]]], [[[7.0, 5.0, 7.0], [2.0, 7.0, 3.0], [4.0, 4.0, 7.0]], [[1.0, 2.0, 
8.0], [5.0, 8.0, 3.0], [6.0, 7.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 8.0], [8.0, 8.0, 3.0]]], [[[5.0, 2.0, 8.0], [8.0, 8.0, 
3.0], [2.0, 3.0, 7.0]], [[6.0, 5.0, 1.0], [4.0, 2.0, 4.0], [1.0, 6.0, 3.0]], [[7.0, 6.0, 1.0], [1.0, 1.0, 8.0], [4.0, 7.0, 
2.0]]], [[[3.0, 2.0, 8.0], [4.0, 4.0, 4.0], [5.0, 1.0, 4.0]], [[1.0, 5.0, 5.0], [8.0, 2.0, 6.0], [5.0, 1.0, 1.0]], [[7.0, 4.0, 
5.0], [4.0, 7.0, 1.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 3.0, 7.0], [4.0, 7.0, 1.0]], [[7.0, 1.0, 7.0], [6.0, 5.0, 3.0]]], [[[2.0, 2.0, 2.0], [8.0, 
6.0, 7.0]], [[5.0, 7.0, 6.0], [3.0, 7.0, 2.0]]], [[[2.0, 6.0, 1.0], [6.0, 1.0, 6.0]], [[3.0, 5.0, 8.0], [8.0, 1.0, 4.0]]], 
[[[2.0, 2.0, 1.0], [1.0, 3.0, 2.0]], [[4.0, 6.0, 3.0], [8.0, 2.0, 1.0]]]]),self.functionspace)
      arg[:,:2,:2]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 7.0], [4.0, 7.0, 1.0], [7.0, 3.0, 6.0]], [[7.0, 1.0, 7.0], [6.0, 5.0, 3.0], [3.0, 4.0, 
5.0]], [[6.0, 1.0, 6.0], [6.0, 2.0, 8.0], [2.0, 1.0, 4.0]]], [[[2.0, 2.0, 2.0], [8.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[5.0, 7.0, 
6.0], [3.0, 7.0, 2.0], [6.0, 7.0, 3.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 8.0], [8.0, 8.0, 3.0]]], [[[2.0, 6.0, 1.0], [6.0, 1.0, 
6.0], [2.0, 3.0, 7.0]], [[3.0, 5.0, 8.0], [8.0, 1.0, 4.0], [1.0, 6.0, 3.0]], [[7.0, 6.0, 1.0], [1.0, 1.0, 8.0], [4.0, 7.0, 
2.0]]], [[[2.0, 2.0, 1.0], [1.0, 3.0, 2.0], [5.0, 1.0, 4.0]], [[4.0, 6.0, 3.0], [8.0, 2.0, 1.0], [5.0, 1.0, 1.0]], [[7.0, 4.0, 
5.0], [4.0, 7.0, 1.0], [2.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CUUN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 3.0], [3.0, 5.0, 8.0], [5.0, 5.0, 2.0]], [[3.0, 5.0, 5.0], [4.0, 8.0, 8.0], [8.0, 7.0, 
3.0]], [[4.0, 2.0, 7.0], [2.0, 3.0, 1.0], [6.0, 7.0, 3.0]]], [[[3.0, 5.0, 2.0], [5.0, 1.0, 4.0], [7.0, 7.0, 6.0]], [[4.0, 3.0, 
8.0], [5.0, 6.0, 3.0], [3.0, 3.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 1.0, 2.0], [6.0, 1.0, 8.0]]], [[[8.0, 3.0, 2.0], [3.0, 4.0, 
4.0], [5.0, 3.0, 7.0]], [[5.0, 5.0, 8.0], [1.0, 3.0, 8.0], [7.0, 3.0, 7.0]], [[3.0, 2.0, 5.0], [2.0, 7.0, 8.0], [3.0, 1.0, 
3.0]]], [[[1.0, 7.0, 2.0], [1.0, 8.0, 4.0], [3.0, 1.0, 7.0]], [[2.0, 2.0, 8.0], [7.0, 4.0, 2.0], [5.0, 8.0, 4.0]], [[6.0, 1.0, 
4.0], [7.0, 4.0, 7.0], [2.0, 1.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 3.0, 1.0], [4.0, 8.0, 7.0]], [[8.0, 4.0, 3.0], [7.0, 3.0, 1.0]]], [[[4.0, 6.0, 6.0], 
[7.0, 7.0, 3.0]], [[3.0, 1.0, 8.0], [8.0, 8.0, 2.0]]], [[[8.0, 4.0, 8.0], [1.0, 7.0, 7.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 
4.0]]], [[[4.0, 6.0, 3.0], [1.0, 4.0, 7.0]], [[2.0, 8.0, 8.0], [2.0, 6.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 1.0, 7.0], 
[7.0, 1.0, 4.0]], [[1.0, 5.0, 1.0], [8.0, 1.0, 4.0]]], [[[7.0, 6.0, 3.0], [5.0, 7.0, 6.0]], [[6.0, 8.0, 4.0], [6.0, 1.0, 
3.0]]], [[[6.0, 6.0, 8.0], [3.0, 6.0, 1.0]], [[5.0, 6.0, 5.0], [2.0, 7.0, 6.0]]], [[[6.0, 5.0, 4.0], [8.0, 2.0, 7.0]], [[5.0, 
4.0, 5.0], [8.0, 8.0, 8.0]]]])
      arg[:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 1.0], [4.0, 8.0, 7.0], [5.0, 5.0, 2.0]], [[8.0, 4.0, 3.0], [7.0, 3.0, 1.0], [8.0, 
7.0, 3.0]], [[4.0, 2.0, 7.0], [2.0, 3.0, 1.0], [6.0, 7.0, 3.0]]], [[[4.0, 6.0, 6.0], [7.0, 7.0, 3.0], [7.0, 7.0, 6.0]], [[3.0, 
1.0, 8.0], [8.0, 8.0, 2.0], [3.0, 3.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 1.0, 2.0], [6.0, 1.0, 8.0]]], [[[8.0, 4.0, 8.0], [1.0, 
7.0, 7.0], [5.0, 3.0, 7.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 4.0], [7.0, 3.0, 7.0]], [[3.0, 2.0, 5.0], [2.0, 7.0, 8.0], [3.0, 1.0, 
3.0]]], [[[4.0, 6.0, 3.0], [1.0, 4.0, 7.0], [3.0, 1.0, 7.0]], [[2.0, 8.0, 8.0], [2.0, 6.0, 1.0], [5.0, 8.0, 4.0]], [[6.0, 1.0, 
4.0], [7.0, 4.0, 7.0], [2.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 7.0], [7.0, 1.0, 4.0], [5.0, 5.0, 2.0]], 
[[1.0, 5.0, 1.0], [8.0, 1.0, 4.0], [8.0, 7.0, 3.0]], [[4.0, 2.0, 7.0], [2.0, 3.0, 1.0], [6.0, 7.0, 3.0]]], [[[7.0, 6.0, 3.0], 
[5.0, 7.0, 6.0], [7.0, 7.0, 6.0]], [[6.0, 8.0, 4.0], [6.0, 1.0, 3.0], [3.0, 3.0, 1.0]], [[8.0, 4.0, 7.0], [4.0, 1.0, 2.0], 
[6.0, 1.0, 8.0]]], [[[6.0, 6.0, 8.0], [3.0, 6.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 6.0, 5.0], [2.0, 7.0, 6.0], [7.0, 3.0, 7.0]], 
[[3.0, 2.0, 5.0], [2.0, 7.0, 8.0], [3.0, 1.0, 3.0]]], [[[6.0, 5.0, 4.0], [8.0, 2.0, 7.0], [3.0, 1.0, 7.0]], [[5.0, 4.0, 5.0], 
[8.0, 8.0, 8.0], [5.0, 8.0, 4.0]], [[6.0, 1.0, 4.0], [7.0, 4.0, 7.0], [2.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UUPN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 2.0], [2.0, 8.0, 4.0], [6.0, 3.0, 7.0]], [[2.0, 5.0, 3.0], [4.0, 7.0, 1.0], [3.0, 2.0, 
8.0]], [[4.0, 4.0, 2.0], [2.0, 8.0, 3.0], [1.0, 4.0, 5.0]]], [[[1.0, 6.0, 1.0], [8.0, 7.0, 5.0], [2.0, 4.0, 3.0]], [[7.0, 6.0, 
8.0], [6.0, 4.0, 5.0], [5.0, 5.0, 6.0]], [[6.0, 3.0, 3.0], [3.0, 7.0, 4.0], [7.0, 2.0, 3.0]]], [[[2.0, 1.0, 5.0], [1.0, 6.0, 
7.0], [8.0, 1.0, 7.0]], [[5.0, 4.0, 5.0], [3.0, 8.0, 8.0], [6.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [8.0, 7.0, 3.0], [1.0, 7.0, 
4.0]]], [[[1.0, 3.0, 6.0], [4.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 3.0, 3.0], [8.0, 1.0, 2.0]], [[4.0, 7.0, 
8.0], [8.0, 5.0, 8.0], [1.0, 2.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 2.0, 1.0], [4.0, 3.0, 5.0]], [[6.0, 1.0, 8.0], [3.0, 1.0, 7.0]], [[2.0, 6.0, 3.0], [4.0, 1.0, 
6.0]]])
      arg[:3,:2,1]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 2.0], [3.0, 2.0, 1.0], [6.0, 3.0, 7.0]], [[2.0, 5.0, 3.0], [4.0, 3.0, 5.0], [3.0, 2.0, 
8.0]], [[4.0, 4.0, 2.0], [2.0, 8.0, 3.0], [1.0, 4.0, 5.0]]], [[[1.0, 6.0, 1.0], [6.0, 1.0, 8.0], [2.0, 4.0, 3.0]], [[7.0, 6.0, 
8.0], [3.0, 1.0, 7.0], [5.0, 5.0, 6.0]], [[6.0, 3.0, 3.0], [3.0, 7.0, 4.0], [7.0, 2.0, 3.0]]], [[[2.0, 1.0, 5.0], [2.0, 6.0, 
3.0], [8.0, 1.0, 7.0]], [[5.0, 4.0, 5.0], [4.0, 1.0, 6.0], [6.0, 5.0, 4.0]], [[4.0, 7.0, 8.0], [8.0, 7.0, 3.0], [1.0, 7.0, 
4.0]]], [[[1.0, 3.0, 6.0], [4.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 3.0, 3.0], [8.0, 1.0, 2.0]], [[4.0, 7.0, 
8.0], [8.0, 5.0, 8.0], [1.0, 2.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UUPN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 5.0], [6.0, 4.0, 2.0], [4.0, 6.0, 8.0]], [[3.0, 8.0, 4.0], [1.0, 4.0, 5.0], [4.0, 8.0, 
5.0]], [[3.0, 5.0, 8.0], [7.0, 4.0, 1.0], [6.0, 2.0, 7.0]]], [[[7.0, 8.0, 3.0], [1.0, 2.0, 1.0], [1.0, 7.0, 8.0]], [[3.0, 7.0, 
6.0], [5.0, 4.0, 3.0], [4.0, 4.0, 4.0]], [[7.0, 5.0, 8.0], [3.0, 1.0, 8.0], [5.0, 7.0, 3.0]]], [[[7.0, 1.0, 5.0], [3.0, 8.0, 
6.0], [3.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [6.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[2.0, 6.0, 3.0], [8.0, 4.0, 5.0], [1.0, 6.0, 
6.0]]], [[[5.0, 7.0, 8.0], [7.0, 3.0, 1.0], [7.0, 5.0, 1.0]], [[3.0, 6.0, 8.0], [7.0, 4.0, 5.0], [1.0, 4.0, 2.0]], [[2.0, 4.0, 
6.0], [4.0, 1.0, 5.0], [6.0, 2.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 1.0, 2.0], [1.0, 7.0, 3.0]], [[3.0, 1.0, 8.0], [2.0, 1.0, 6.0]], [[3.0, 7.0, 4.0], [1.0, 
7.0, 3.0]]]),self.functionspace)
      arg[:3,:2,1]=val
      ref=Data(numpy.array([[[[3.0, 3.0, 5.0], [5.0, 1.0, 2.0], [4.0, 6.0, 8.0]], [[3.0, 8.0, 4.0], [1.0, 7.0, 3.0], [4.0, 8.0, 
5.0]], [[3.0, 5.0, 8.0], [7.0, 4.0, 1.0], [6.0, 2.0, 7.0]]], [[[7.0, 8.0, 3.0], [3.0, 1.0, 8.0], [1.0, 7.0, 8.0]], [[3.0, 7.0, 
6.0], [2.0, 1.0, 6.0], [4.0, 4.0, 4.0]], [[7.0, 5.0, 8.0], [3.0, 1.0, 8.0], [5.0, 7.0, 3.0]]], [[[7.0, 1.0, 5.0], [3.0, 7.0, 
4.0], [3.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [1.0, 7.0, 3.0], [4.0, 4.0, 3.0]], [[2.0, 6.0, 3.0], [8.0, 4.0, 5.0], [1.0, 6.0, 
6.0]]], [[[5.0, 7.0, 8.0], [7.0, 3.0, 1.0], [7.0, 5.0, 1.0]], [[3.0, 6.0, 8.0], [7.0, 4.0, 5.0], [1.0, 4.0, 2.0]], [[2.0, 4.0, 
6.0], [4.0, 1.0, 5.0], [6.0, 2.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UUPN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 3.0], [2.0, 4.0, 5.0], [2.0, 5.0, 6.0]], [[3.0, 1.0, 6.0], [7.0, 2.0, 5.0], [8.0, 4.0, 
8.0]], [[2.0, 4.0, 6.0], [2.0, 5.0, 4.0], [5.0, 4.0, 6.0]]], [[[1.0, 3.0, 4.0], [2.0, 2.0, 5.0], [8.0, 7.0, 1.0]], [[7.0, 7.0, 
1.0], [2.0, 2.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 3.0], [3.0, 6.0, 3.0], [5.0, 5.0, 8.0]]], [[[1.0, 7.0, 1.0], [7.0, 1.0, 
5.0], [8.0, 5.0, 7.0]], [[6.0, 5.0, 4.0], [2.0, 5.0, 7.0], [4.0, 1.0, 2.0]], [[2.0, 2.0, 5.0], [5.0, 5.0, 6.0], [5.0, 3.0, 
1.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 3.0], [8.0, 3.0, 4.0]], [[8.0, 8.0, 3.0], [7.0, 3.0, 6.0], [2.0, 7.0, 2.0]], [[8.0, 2.0, 
8.0], [3.0, 8.0, 6.0], [6.0, 5.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 8.0, 4.0], [7.0, 2.0, 2.0]], [[2.0, 8.0, 6.0], [1.0, 8.0, 6.0]], [[4.0, 7.0, 2.0], [7.0, 
5.0, 6.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 8.0], [4.0, 7.0, 2.0]], [[3.0, 4.0, 6.0], [7.0, 6.0, 8.0]], [[1.0, 8.0, 
8.0], [2.0, 2.0, 7.0]]])
      arg[:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 3.0], [3.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[3.0, 1.0, 6.0], [7.0, 2.0, 2.0], [8.0, 
4.0, 8.0]], [[2.0, 4.0, 6.0], [2.0, 5.0, 4.0], [5.0, 4.0, 6.0]]], [[[1.0, 3.0, 4.0], [2.0, 8.0, 6.0], [8.0, 7.0, 1.0]], [[7.0, 
7.0, 1.0], [1.0, 8.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 3.0], [3.0, 6.0, 3.0], [5.0, 5.0, 8.0]]], [[[1.0, 7.0, 1.0], [4.0, 
7.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 5.0, 4.0], [7.0, 5.0, 6.0], [4.0, 1.0, 2.0]], [[2.0, 2.0, 5.0], [5.0, 5.0, 6.0], [5.0, 3.0, 
1.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 3.0], [8.0, 3.0, 4.0]], [[8.0, 8.0, 3.0], [7.0, 3.0, 6.0], [2.0, 7.0, 2.0]], [[8.0, 2.0, 
8.0], [3.0, 8.0, 6.0], [6.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 3.0], [2.0, 7.0, 8.0], [2.0, 5.0, 6.0]], 
[[3.0, 1.0, 6.0], [4.0, 7.0, 2.0], [8.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [2.0, 5.0, 4.0], [5.0, 4.0, 6.0]]], [[[1.0, 3.0, 4.0], 
[3.0, 4.0, 6.0], [8.0, 7.0, 1.0]], [[7.0, 7.0, 1.0], [7.0, 6.0, 8.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 3.0], [3.0, 6.0, 3.0], 
[5.0, 5.0, 8.0]]], [[[1.0, 7.0, 1.0], [1.0, 8.0, 8.0], [8.0, 5.0, 7.0]], [[6.0, 5.0, 4.0], [2.0, 2.0, 7.0], [4.0, 1.0, 2.0]], 
[[2.0, 2.0, 5.0], [5.0, 5.0, 6.0], [5.0, 3.0, 1.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 3.0], [8.0, 3.0, 4.0]], [[8.0, 8.0, 3.0], 
[7.0, 3.0, 6.0], [2.0, 7.0, 2.0]], [[8.0, 2.0, 8.0], [3.0, 8.0, 6.0], [6.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPPN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 5.0], [4.0, 3.0, 4.0], [6.0, 4.0, 5.0]], [[6.0, 6.0, 5.0], [4.0, 2.0, 1.0], [5.0, 2.0, 
5.0]], [[5.0, 3.0, 3.0], [8.0, 5.0, 8.0], [2.0, 4.0, 3.0]]], [[[4.0, 5.0, 1.0], [4.0, 8.0, 6.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 
8.0], [7.0, 1.0, 2.0], [1.0, 8.0, 3.0]], [[2.0, 4.0, 4.0], [7.0, 5.0, 4.0], [6.0, 1.0, 5.0]]], [[[7.0, 1.0, 1.0], [3.0, 1.0, 
7.0], [6.0, 4.0, 7.0]], [[2.0, 6.0, 6.0], [3.0, 1.0, 4.0], [5.0, 4.0, 5.0]], [[2.0, 8.0, 6.0], [1.0, 2.0, 1.0], [4.0, 7.0, 
6.0]]], [[[6.0, 7.0, 6.0], [8.0, 3.0, 8.0], [8.0, 8.0, 3.0]], [[6.0, 8.0, 5.0], [1.0, 1.0, 3.0], [6.0, 7.0, 5.0]], [[8.0, 2.0, 
5.0], [5.0, 7.0, 5.0], [4.0, 1.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[2.0, 2.0, 6.0], [6.0, 3.0, 6.0]])
      arg[1:3,1,1]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 5.0], [4.0, 3.0, 4.0], [6.0, 4.0, 5.0]], [[6.0, 6.0, 5.0], [4.0, 2.0, 1.0], [5.0, 2.0, 
5.0]], [[5.0, 3.0, 3.0], [8.0, 5.0, 8.0], [2.0, 4.0, 3.0]]], [[[4.0, 5.0, 1.0], [4.0, 8.0, 6.0], [3.0, 3.0, 8.0]], [[3.0, 8.0, 
8.0], [2.0, 2.0, 6.0], [1.0, 8.0, 3.0]], [[2.0, 4.0, 4.0], [7.0, 5.0, 4.0], [6.0, 1.0, 5.0]]], [[[7.0, 1.0, 1.0], [3.0, 1.0, 
7.0], [6.0, 4.0, 7.0]], [[2.0, 6.0, 6.0], [6.0, 3.0, 6.0], [5.0, 4.0, 5.0]], [[2.0, 8.0, 6.0], [1.0, 2.0, 1.0], [4.0, 7.0, 
6.0]]], [[[6.0, 7.0, 6.0], [8.0, 3.0, 8.0], [8.0, 8.0, 3.0]], [[6.0, 8.0, 5.0], [1.0, 1.0, 3.0], [6.0, 7.0, 5.0]], [[8.0, 2.0, 
5.0], [5.0, 7.0, 5.0], [4.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPPN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 4.0], [4.0, 6.0, 3.0], [8.0, 1.0, 8.0]], [[3.0, 4.0, 4.0], [7.0, 4.0, 7.0], [1.0, 2.0, 
4.0]], [[6.0, 7.0, 1.0], [1.0, 2.0, 3.0], [4.0, 8.0, 6.0]]], [[[6.0, 1.0, 4.0], [5.0, 2.0, 5.0], [8.0, 4.0, 3.0]], [[7.0, 6.0, 
7.0], [5.0, 2.0, 3.0], [6.0, 1.0, 6.0]], [[3.0, 1.0, 8.0], [1.0, 2.0, 6.0], [2.0, 6.0, 1.0]]], [[[5.0, 5.0, 7.0], [8.0, 2.0, 
2.0], [1.0, 3.0, 4.0]], [[6.0, 1.0, 4.0], [1.0, 7.0, 6.0], [2.0, 4.0, 4.0]], [[6.0, 3.0, 7.0], [8.0, 7.0, 7.0], [8.0, 7.0, 
6.0]]], [[[7.0, 4.0, 8.0], [1.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[5.0, 3.0, 8.0], [3.0, 6.0, 2.0], [6.0, 7.0, 2.0]], [[3.0, 1.0, 
7.0], [7.0, 3.0, 8.0], [8.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[2.0, 6.0, 7.0], [8.0, 8.0, 2.0]]),self.functionspace)
      arg[1:3,1,1]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 4.0], [4.0, 6.0, 3.0], [8.0, 1.0, 8.0]], [[3.0, 4.0, 4.0], [7.0, 4.0, 7.0], [1.0, 2.0, 
4.0]], [[6.0, 7.0, 1.0], [1.0, 2.0, 3.0], [4.0, 8.0, 6.0]]], [[[6.0, 1.0, 4.0], [5.0, 2.0, 5.0], [8.0, 4.0, 3.0]], [[7.0, 6.0, 
7.0], [2.0, 6.0, 7.0], [6.0, 1.0, 6.0]], [[3.0, 1.0, 8.0], [1.0, 2.0, 6.0], [2.0, 6.0, 1.0]]], [[[5.0, 5.0, 7.0], [8.0, 2.0, 
2.0], [1.0, 3.0, 4.0]], [[6.0, 1.0, 4.0], [8.0, 8.0, 2.0], [2.0, 4.0, 4.0]], [[6.0, 3.0, 7.0], [8.0, 7.0, 7.0], [8.0, 7.0, 
6.0]]], [[[7.0, 4.0, 8.0], [1.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[5.0, 3.0, 8.0], [3.0, 6.0, 2.0], [6.0, 7.0, 2.0]], [[3.0, 1.0, 
7.0], [7.0, 3.0, 8.0], [8.0, 5.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPPN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 7.0], [8.0, 2.0, 1.0], [5.0, 6.0, 3.0]], [[2.0, 5.0, 4.0], [3.0, 5.0, 8.0], [4.0, 7.0, 
6.0]], [[5.0, 3.0, 3.0], [5.0, 5.0, 8.0], [6.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], [4.0, 3.0, 2.0], [3.0, 8.0, 8.0]], [[4.0, 3.0, 
4.0], [8.0, 2.0, 8.0], [5.0, 6.0, 3.0]], [[6.0, 5.0, 2.0], [2.0, 6.0, 5.0], [5.0, 1.0, 5.0]]], [[[3.0, 5.0, 4.0], [6.0, 4.0, 
6.0], [7.0, 4.0, 6.0]], [[2.0, 7.0, 7.0], [3.0, 3.0, 4.0], [1.0, 6.0, 3.0]], [[3.0, 4.0, 8.0], [5.0, 1.0, 4.0], [7.0, 4.0, 
2.0]]], [[[3.0, 7.0, 4.0], [1.0, 6.0, 5.0], [8.0, 5.0, 3.0]], [[1.0, 5.0, 4.0], [6.0, 3.0, 4.0], [3.0, 1.0, 5.0]], [[8.0, 8.0, 
3.0], [4.0, 3.0, 6.0], [6.0, 8.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 8.0, 2.0], [5.0, 6.0, 7.0]])+(1.-msk_val)*numpy.array([[7.0, 4.0, 5.0], [6.0, 5.0, 3.0]])
      arg[1:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [8.0, 2.0, 1.0], [5.0, 6.0, 3.0]], [[2.0, 5.0, 4.0], [3.0, 5.0, 8.0], [4.0, 
7.0, 6.0]], [[5.0, 3.0, 3.0], [5.0, 5.0, 8.0], [6.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], [4.0, 3.0, 2.0], [3.0, 8.0, 8.0]], [[4.0, 
3.0, 4.0], [6.0, 8.0, 2.0], [5.0, 6.0, 3.0]], [[6.0, 5.0, 2.0], [2.0, 6.0, 5.0], [5.0, 1.0, 5.0]]], [[[3.0, 5.0, 4.0], [6.0, 
4.0, 6.0], [7.0, 4.0, 6.0]], [[2.0, 7.0, 7.0], [5.0, 6.0, 7.0], [1.0, 6.0, 3.0]], [[3.0, 4.0, 8.0], [5.0, 1.0, 4.0], [7.0, 4.0, 
2.0]]], [[[3.0, 7.0, 4.0], [1.0, 6.0, 5.0], [8.0, 5.0, 3.0]], [[1.0, 5.0, 4.0], [6.0, 3.0, 4.0], [3.0, 1.0, 5.0]], [[8.0, 8.0, 
3.0], [4.0, 3.0, 6.0], [6.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 7.0], [8.0, 2.0, 1.0], [5.0, 6.0, 3.0]], 
[[2.0, 5.0, 4.0], [3.0, 5.0, 8.0], [4.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [5.0, 5.0, 8.0], [6.0, 1.0, 1.0]]], [[[7.0, 3.0, 4.0], 
[4.0, 3.0, 2.0], [3.0, 8.0, 8.0]], [[4.0, 3.0, 4.0], [7.0, 4.0, 5.0], [5.0, 6.0, 3.0]], [[6.0, 5.0, 2.0], [2.0, 6.0, 5.0], 
[5.0, 1.0, 5.0]]], [[[3.0, 5.0, 4.0], [6.0, 4.0, 6.0], [7.0, 4.0, 6.0]], [[2.0, 7.0, 7.0], [6.0, 5.0, 3.0], [1.0, 6.0, 3.0]], 
[[3.0, 4.0, 8.0], [5.0, 1.0, 4.0], [7.0, 4.0, 2.0]]], [[[3.0, 7.0, 4.0], [1.0, 6.0, 5.0], [8.0, 5.0, 3.0]], [[1.0, 5.0, 4.0], 
[6.0, 3.0, 4.0], [3.0, 1.0, 5.0]], [[8.0, 8.0, 3.0], [4.0, 3.0, 6.0], [6.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LULN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 2.0], [6.0, 7.0, 1.0], [7.0, 6.0, 7.0]], [[8.0, 2.0, 1.0], [3.0, 1.0, 2.0], [1.0, 7.0, 
1.0]], [[4.0, 2.0, 6.0], [7.0, 4.0, 3.0], [2.0, 8.0, 3.0]]], [[[6.0, 1.0, 1.0], [7.0, 1.0, 6.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 
3.0], [4.0, 7.0, 4.0], [3.0, 5.0, 4.0]], [[5.0, 3.0, 3.0], [2.0, 4.0, 6.0], [4.0, 6.0, 2.0]]], [[[5.0, 5.0, 5.0], [7.0, 5.0, 
8.0], [8.0, 7.0, 6.0]], [[2.0, 3.0, 1.0], [1.0, 7.0, 2.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 4.0], [1.0, 1.0, 8.0], [5.0, 5.0, 
6.0]]], [[[6.0, 7.0, 8.0], [5.0, 7.0, 6.0], [2.0, 7.0, 1.0]], [[3.0, 3.0, 7.0], [6.0, 3.0, 8.0], [6.0, 6.0, 2.0]], [[1.0, 2.0, 
2.0], [4.0, 6.0, 5.0], [8.0, 4.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 4.0, 3.0]], [[6.0, 8.0, 5.0]]], [[[4.0, 8.0, 1.0]], [[3.0, 8.0, 1.0]]]])
      arg[2:,:2,2:]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 2.0], [6.0, 7.0, 1.0], [7.0, 6.0, 7.0]], [[8.0, 2.0, 1.0], [3.0, 1.0, 2.0], [1.0, 7.0, 
1.0]], [[4.0, 2.0, 6.0], [7.0, 4.0, 3.0], [2.0, 8.0, 3.0]]], [[[6.0, 1.0, 1.0], [7.0, 1.0, 6.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 
3.0], [4.0, 7.0, 4.0], [3.0, 5.0, 4.0]], [[5.0, 3.0, 3.0], [2.0, 4.0, 6.0], [4.0, 6.0, 2.0]]], [[[5.0, 5.0, 5.0], [7.0, 5.0, 
8.0], [8.0, 4.0, 3.0]], [[2.0, 3.0, 1.0], [1.0, 7.0, 2.0], [6.0, 8.0, 5.0]], [[5.0, 5.0, 4.0], [1.0, 1.0, 8.0], [5.0, 5.0, 
6.0]]], [[[6.0, 7.0, 8.0], [5.0, 7.0, 6.0], [4.0, 8.0, 1.0]], [[3.0, 3.0, 7.0], [6.0, 3.0, 8.0], [3.0, 8.0, 1.0]], [[1.0, 2.0, 
2.0], [4.0, 6.0, 5.0], [8.0, 4.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LULN(self):
      arg=Data(numpy.array([[[[1.0, 5.0, 6.0], [6.0, 1.0, 8.0], [1.0, 8.0, 2.0]], [[2.0, 6.0, 7.0], [8.0, 8.0, 8.0], [8.0, 3.0, 
2.0]], [[8.0, 3.0, 8.0], [1.0, 2.0, 2.0], [8.0, 3.0, 3.0]]], [[[5.0, 1.0, 6.0], [6.0, 7.0, 4.0], [5.0, 2.0, 3.0]], [[4.0, 4.0, 
2.0], [6.0, 4.0, 4.0], [4.0, 1.0, 4.0]], [[2.0, 3.0, 1.0], [6.0, 8.0, 1.0], [6.0, 1.0, 2.0]]], [[[2.0, 8.0, 5.0], [1.0, 8.0, 
6.0], [1.0, 3.0, 4.0]], [[3.0, 1.0, 5.0], [8.0, 3.0, 7.0], [4.0, 3.0, 4.0]], [[7.0, 5.0, 7.0], [8.0, 4.0, 3.0], [8.0, 3.0, 
3.0]]], [[[1.0, 2.0, 3.0], [5.0, 5.0, 4.0], [4.0, 8.0, 7.0]], [[2.0, 3.0, 2.0], [4.0, 6.0, 5.0], [8.0, 3.0, 5.0]], [[8.0, 3.0, 
6.0], [3.0, 6.0, 4.0], [7.0, 4.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 8.0, 1.0]], [[4.0, 5.0, 7.0]]], [[[6.0, 2.0, 3.0]], [[8.0, 5.0, 8.0]]]]),self.functionspace)
      arg[2:,:2,2:]=val
      ref=Data(numpy.array([[[[1.0, 5.0, 6.0], [6.0, 1.0, 8.0], [1.0, 8.0, 2.0]], [[2.0, 6.0, 7.0], [8.0, 8.0, 8.0], [8.0, 3.0, 
2.0]], [[8.0, 3.0, 8.0], [1.0, 2.0, 2.0], [8.0, 3.0, 3.0]]], [[[5.0, 1.0, 6.0], [6.0, 7.0, 4.0], [5.0, 2.0, 3.0]], [[4.0, 4.0, 
2.0], [6.0, 4.0, 4.0], [4.0, 1.0, 4.0]], [[2.0, 3.0, 1.0], [6.0, 8.0, 1.0], [6.0, 1.0, 2.0]]], [[[2.0, 8.0, 5.0], [1.0, 8.0, 
6.0], [5.0, 8.0, 1.0]], [[3.0, 1.0, 5.0], [8.0, 3.0, 7.0], [4.0, 5.0, 7.0]], [[7.0, 5.0, 7.0], [8.0, 4.0, 3.0], [8.0, 3.0, 
3.0]]], [[[1.0, 2.0, 3.0], [5.0, 5.0, 4.0], [6.0, 2.0, 3.0]], [[2.0, 3.0, 2.0], [4.0, 6.0, 5.0], [8.0, 5.0, 8.0]], [[8.0, 3.0, 
6.0], [3.0, 6.0, 4.0], [7.0, 4.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LULN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 2.0], [6.0, 4.0, 8.0], [8.0, 8.0, 4.0]], [[4.0, 1.0, 4.0], [8.0, 4.0, 7.0], [4.0, 4.0, 
7.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], [5.0, 5.0, 6.0]]], [[[7.0, 3.0, 8.0], [3.0, 8.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 5.0, 
8.0], [6.0, 7.0, 8.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 2.0], [4.0, 6.0, 2.0], [5.0, 6.0, 7.0]]], [[[1.0, 6.0, 8.0], [6.0, 2.0, 
7.0], [1.0, 5.0, 2.0]], [[1.0, 4.0, 6.0], [5.0, 7.0, 3.0], [5.0, 6.0, 6.0]], [[2.0, 5.0, 6.0], [6.0, 6.0, 4.0], [3.0, 2.0, 
3.0]]], [[[8.0, 5.0, 8.0], [3.0, 3.0, 2.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 1.0], [5.0, 4.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 2.0, 
8.0], [5.0, 6.0, 6.0], [7.0, 1.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 6.0, 8.0]], [[2.0, 8.0, 5.0]]], [[[6.0, 8.0, 1.0]], [[6.0, 5.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 5.0]], [[7.0, 6.0, 5.0]]], [[[8.0, 3.0, 3.0]], [[5.0, 5.0, 2.0]]]])
      arg[2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 2.0], [6.0, 4.0, 8.0], [8.0, 8.0, 4.0]], [[4.0, 1.0, 4.0], [8.0, 4.0, 7.0], [4.0, 
4.0, 7.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], [5.0, 5.0, 6.0]]], [[[7.0, 3.0, 8.0], [3.0, 8.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 
5.0, 8.0], [6.0, 7.0, 8.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 2.0], [4.0, 6.0, 2.0], [5.0, 6.0, 7.0]]], [[[1.0, 6.0, 8.0], [6.0, 
2.0, 7.0], [5.0, 6.0, 8.0]], [[1.0, 4.0, 6.0], [5.0, 7.0, 3.0], [2.0, 8.0, 5.0]], [[2.0, 5.0, 6.0], [6.0, 6.0, 4.0], [3.0, 2.0, 
3.0]]], [[[8.0, 5.0, 8.0], [3.0, 3.0, 2.0], [6.0, 8.0, 1.0]], [[1.0, 7.0, 1.0], [5.0, 4.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 2.0, 
8.0], [5.0, 6.0, 6.0], [7.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 2.0], [6.0, 4.0, 8.0], [8.0, 8.0, 4.0]], 
[[4.0, 1.0, 4.0], [8.0, 4.0, 7.0], [4.0, 4.0, 7.0]], [[2.0, 8.0, 8.0], [3.0, 1.0, 3.0], [5.0, 5.0, 6.0]]], [[[7.0, 3.0, 8.0], 
[3.0, 8.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 5.0, 8.0], [6.0, 7.0, 8.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 2.0], [4.0, 6.0, 2.0], 
[5.0, 6.0, 7.0]]], [[[1.0, 6.0, 8.0], [6.0, 2.0, 7.0], [5.0, 4.0, 5.0]], [[1.0, 4.0, 6.0], [5.0, 7.0, 3.0], [7.0, 6.0, 5.0]], 
[[2.0, 5.0, 6.0], [6.0, 6.0, 4.0], [3.0, 2.0, 3.0]]], [[[8.0, 5.0, 8.0], [3.0, 3.0, 2.0], [8.0, 3.0, 3.0]], [[1.0, 7.0, 1.0], 
[5.0, 4.0, 8.0], [5.0, 5.0, 2.0]], [[3.0, 2.0, 8.0], [5.0, 6.0, 6.0], [7.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LLLL(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 6.0], [3.0, 4.0, 8.0], [7.0, 6.0, 2.0]], [[5.0, 1.0, 3.0], [2.0, 1.0, 7.0], [8.0, 6.0, 
6.0]], [[6.0, 3.0, 1.0], [7.0, 7.0, 4.0], [1.0, 5.0, 3.0]]], [[[8.0, 8.0, 5.0], [3.0, 7.0, 8.0], [7.0, 3.0, 1.0]], [[6.0, 3.0, 
7.0], [7.0, 8.0, 2.0], [3.0, 3.0, 3.0]], [[3.0, 1.0, 3.0], [1.0, 4.0, 2.0], [7.0, 2.0, 1.0]]], [[[5.0, 8.0, 1.0], [3.0, 3.0, 
8.0], [8.0, 4.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 3.0, 2.0], [2.0, 4.0, 7.0]], [[6.0, 6.0, 1.0], [8.0, 3.0, 3.0], [3.0, 6.0, 
2.0]]], [[[3.0, 5.0, 7.0], [2.0, 5.0, 4.0], [4.0, 3.0, 8.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 7.0], [2.0, 3.0, 5.0]], [[8.0, 6.0, 
2.0], [1.0, 3.0, 1.0], [5.0, 2.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0]]], [[[5.0]]]])
      arg[2:,2:,2:,2:]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 6.0], [3.0, 4.0, 8.0], [7.0, 6.0, 2.0]], [[5.0, 1.0, 3.0], [2.0, 1.0, 7.0], [8.0, 6.0, 
6.0]], [[6.0, 3.0, 1.0], [7.0, 7.0, 4.0], [1.0, 5.0, 3.0]]], [[[8.0, 8.0, 5.0], [3.0, 7.0, 8.0], [7.0, 3.0, 1.0]], [[6.0, 3.0, 
7.0], [7.0, 8.0, 2.0], [3.0, 3.0, 3.0]], [[3.0, 1.0, 3.0], [1.0, 4.0, 2.0], [7.0, 2.0, 1.0]]], [[[5.0, 8.0, 1.0], [3.0, 3.0, 
8.0], [8.0, 4.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 3.0, 2.0], [2.0, 4.0, 7.0]], [[6.0, 6.0, 1.0], [8.0, 3.0, 3.0], [3.0, 6.0, 
1.0]]], [[[3.0, 5.0, 7.0], [2.0, 5.0, 4.0], [4.0, 3.0, 8.0]], [[6.0, 8.0, 8.0], [5.0, 8.0, 7.0], [2.0, 3.0, 5.0]], [[8.0, 6.0, 
2.0], [1.0, 3.0, 1.0], [5.0, 2.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LLLL(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 8.0], [7.0, 5.0, 1.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 6.0], [3.0, 6.0, 4.0], [2.0, 2.0, 
5.0]], [[6.0, 1.0, 2.0], [2.0, 7.0, 4.0], [5.0, 2.0, 4.0]]], [[[3.0, 7.0, 5.0], [1.0, 8.0, 1.0], [7.0, 1.0, 1.0]], [[4.0, 7.0, 
4.0], [7.0, 7.0, 5.0], [6.0, 8.0, 6.0]], [[8.0, 4.0, 8.0], [1.0, 5.0, 6.0], [2.0, 3.0, 4.0]]], [[[4.0, 6.0, 4.0], [2.0, 1.0, 
2.0], [4.0, 6.0, 5.0]], [[7.0, 1.0, 3.0], [7.0, 4.0, 6.0], [2.0, 3.0, 1.0]], [[3.0, 4.0, 7.0], [5.0, 2.0, 2.0], [5.0, 1.0, 
2.0]]], [[[8.0, 5.0, 1.0], [7.0, 6.0, 1.0], [3.0, 1.0, 5.0]], [[3.0, 1.0, 6.0], [8.0, 7.0, 1.0], [7.0, 7.0, 2.0]], [[7.0, 4.0, 
1.0], [5.0, 1.0, 6.0], [6.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0]]], [[[1.0]]]]),self.functionspace)
      arg[2:,2:,2:,2:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 8.0], [7.0, 5.0, 1.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 6.0], [3.0, 6.0, 4.0], [2.0, 2.0, 
5.0]], [[6.0, 1.0, 2.0], [2.0, 7.0, 4.0], [5.0, 2.0, 4.0]]], [[[3.0, 7.0, 5.0], [1.0, 8.0, 1.0], [7.0, 1.0, 1.0]], [[4.0, 7.0, 
4.0], [7.0, 7.0, 5.0], [6.0, 8.0, 6.0]], [[8.0, 4.0, 8.0], [1.0, 5.0, 6.0], [2.0, 3.0, 4.0]]], [[[4.0, 6.0, 4.0], [2.0, 1.0, 
2.0], [4.0, 6.0, 5.0]], [[7.0, 1.0, 3.0], [7.0, 4.0, 6.0], [2.0, 3.0, 1.0]], [[3.0, 4.0, 7.0], [5.0, 2.0, 2.0], [5.0, 1.0, 
6.0]]], [[[8.0, 5.0, 1.0], [7.0, 6.0, 1.0], [3.0, 1.0, 5.0]], [[3.0, 1.0, 6.0], [8.0, 7.0, 1.0], [7.0, 7.0, 2.0]], [[7.0, 4.0, 
1.0], [5.0, 1.0, 6.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LLLL(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 1.0], [8.0, 5.0, 6.0], [2.0, 5.0, 5.0]], [[3.0, 7.0, 3.0], [2.0, 1.0, 8.0], [2.0, 8.0, 
1.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 8.0], [2.0, 5.0, 3.0]]], [[[1.0, 8.0, 1.0], [5.0, 2.0, 1.0], [1.0, 4.0, 2.0]], [[1.0, 8.0, 
5.0], [6.0, 6.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 5.0], [4.0, 5.0, 4.0], [1.0, 1.0, 8.0]]], [[[7.0, 6.0, 3.0], [5.0, 2.0, 
2.0], [4.0, 1.0, 6.0]], [[1.0, 5.0, 7.0], [4.0, 7.0, 7.0], [4.0, 3.0, 5.0]], [[3.0, 2.0, 6.0], [8.0, 4.0, 1.0], [2.0, 8.0, 
4.0]]], [[[5.0, 2.0, 3.0], [4.0, 4.0, 5.0], [2.0, 7.0, 7.0]], [[4.0, 3.0, 4.0], [5.0, 7.0, 3.0], [7.0, 8.0, 8.0]], [[7.0, 8.0, 
6.0], [6.0, 5.0, 8.0], [6.0, 3.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0]]], [[[1.0]]]])+(1.-msk_val)*numpy.array([[[[2.0]]], [[[3.0]]]])
      arg[2:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 1.0], [8.0, 5.0, 6.0], [2.0, 5.0, 5.0]], [[3.0, 7.0, 3.0], [2.0, 1.0, 8.0], [2.0, 
8.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 8.0], [2.0, 5.0, 3.0]]], [[[1.0, 8.0, 1.0], [5.0, 2.0, 1.0], [1.0, 4.0, 2.0]], [[1.0, 
8.0, 5.0], [6.0, 6.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 5.0], [4.0, 5.0, 4.0], [1.0, 1.0, 8.0]]], [[[7.0, 6.0, 3.0], [5.0, 
2.0, 2.0], [4.0, 1.0, 6.0]], [[1.0, 5.0, 7.0], [4.0, 7.0, 7.0], [4.0, 3.0, 5.0]], [[3.0, 2.0, 6.0], [8.0, 4.0, 1.0], [2.0, 8.0, 
2.0]]], [[[5.0, 2.0, 3.0], [4.0, 4.0, 5.0], [2.0, 7.0, 7.0]], [[4.0, 3.0, 4.0], [5.0, 7.0, 3.0], [7.0, 8.0, 8.0]], [[7.0, 8.0, 
6.0], [6.0, 5.0, 8.0], [6.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [8.0, 5.0, 6.0], [2.0, 5.0, 5.0]], 
[[3.0, 7.0, 3.0], [2.0, 1.0, 8.0], [2.0, 8.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 8.0], [2.0, 5.0, 3.0]]], [[[1.0, 8.0, 1.0], 
[5.0, 2.0, 1.0], [1.0, 4.0, 2.0]], [[1.0, 8.0, 5.0], [6.0, 6.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 6.0, 5.0], [4.0, 5.0, 4.0], 
[1.0, 1.0, 8.0]]], [[[7.0, 6.0, 3.0], [5.0, 2.0, 2.0], [4.0, 1.0, 6.0]], [[1.0, 5.0, 7.0], [4.0, 7.0, 7.0], [4.0, 3.0, 5.0]], 
[[3.0, 2.0, 6.0], [8.0, 4.0, 1.0], [2.0, 8.0, 2.0]]], [[[5.0, 2.0, 3.0], [4.0, 4.0, 5.0], [2.0, 7.0, 7.0]], [[4.0, 3.0, 4.0], 
[5.0, 7.0, 3.0], [7.0, 8.0, 8.0]], [[7.0, 8.0, 6.0], [6.0, 5.0, 8.0], [6.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URCN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 4.0], [2.0, 4.0, 1.0], [8.0, 6.0, 1.0]], [[5.0, 2.0, 8.0], [2.0, 1.0, 3.0], [4.0, 4.0, 
5.0]], [[7.0, 5.0, 2.0], [5.0, 7.0, 1.0], [3.0, 2.0, 1.0]]], [[[5.0, 1.0, 7.0], [5.0, 2.0, 5.0], [5.0, 8.0, 5.0]], [[7.0, 8.0, 
7.0], [5.0, 3.0, 8.0], [5.0, 6.0, 1.0]], [[2.0, 8.0, 7.0], [4.0, 7.0, 2.0], [7.0, 6.0, 5.0]]], [[[6.0, 4.0, 3.0], [3.0, 6.0, 
7.0], [2.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [5.0, 4.0, 8.0], [1.0, 3.0, 4.0]], [[2.0, 1.0, 8.0], [4.0, 7.0, 7.0], [7.0, 8.0, 
1.0]]], [[[3.0, 4.0, 5.0], [7.0, 5.0, 4.0], [8.0, 8.0, 7.0]], [[6.0, 8.0, 4.0], [5.0, 2.0, 2.0], [5.0, 6.0, 1.0]], [[2.0, 4.0, 
2.0], [7.0, 2.0, 1.0], [2.0, 8.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 4.0, 1.0], [4.0, 7.0, 7.0], [6.0, 5.0, 8.0]]], [[[8.0, 3.0, 2.0], [3.0, 4.0, 8.0], [2.0, 3.0, 
2.0]]], [[[2.0, 2.0, 7.0], [8.0, 6.0, 5.0], [3.0, 3.0, 6.0]]]])
      arg[:3,1:2,:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 4.0], [2.0, 4.0, 1.0], [8.0, 6.0, 1.0]], [[1.0, 4.0, 1.0], [4.0, 7.0, 7.0], [6.0, 5.0, 
8.0]], [[7.0, 5.0, 2.0], [5.0, 7.0, 1.0], [3.0, 2.0, 1.0]]], [[[5.0, 1.0, 7.0], [5.0, 2.0, 5.0], [5.0, 8.0, 5.0]], [[8.0, 3.0, 
2.0], [3.0, 4.0, 8.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 7.0], [4.0, 7.0, 2.0], [7.0, 6.0, 5.0]]], [[[6.0, 4.0, 3.0], [3.0, 6.0, 
7.0], [2.0, 2.0, 6.0]], [[2.0, 2.0, 7.0], [8.0, 6.0, 5.0], [3.0, 3.0, 6.0]], [[2.0, 1.0, 8.0], [4.0, 7.0, 7.0], [7.0, 8.0, 
1.0]]], [[[3.0, 4.0, 5.0], [7.0, 5.0, 4.0], [8.0, 8.0, 7.0]], [[6.0, 8.0, 4.0], [5.0, 2.0, 2.0], [5.0, 6.0, 1.0]], [[2.0, 4.0, 
2.0], [7.0, 2.0, 1.0], [2.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URCN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 4.0], [8.0, 7.0, 5.0], [4.0, 6.0, 3.0]], [[8.0, 3.0, 1.0], [6.0, 6.0, 4.0], [5.0, 8.0, 
1.0]], [[1.0, 2.0, 7.0], [7.0, 8.0, 6.0], [1.0, 4.0, 2.0]]], [[[6.0, 8.0, 3.0], [3.0, 8.0, 6.0], [4.0, 1.0, 2.0]], [[4.0, 2.0, 
4.0], [2.0, 8.0, 1.0], [3.0, 1.0, 2.0]], [[5.0, 8.0, 4.0], [4.0, 1.0, 2.0], [5.0, 2.0, 3.0]]], [[[7.0, 6.0, 8.0], [2.0, 3.0, 
2.0], [4.0, 7.0, 2.0]], [[3.0, 3.0, 3.0], [8.0, 3.0, 4.0], [2.0, 5.0, 7.0]], [[5.0, 5.0, 4.0], [3.0, 7.0, 5.0], [8.0, 5.0, 
5.0]]], [[[6.0, 4.0, 3.0], [3.0, 8.0, 4.0], [7.0, 8.0, 5.0]], [[2.0, 7.0, 6.0], [3.0, 1.0, 7.0], [2.0, 4.0, 8.0]], [[8.0, 6.0, 
4.0], [5.0, 2.0, 2.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 5.0, 3.0], [3.0, 4.0, 4.0], [6.0, 5.0, 6.0]]], [[[5.0, 6.0, 1.0], [4.0, 2.0, 1.0], [3.0, 
3.0, 5.0]]], [[[5.0, 6.0, 2.0], [6.0, 5.0, 8.0], [6.0, 3.0, 8.0]]]]),self.functionspace)
      arg[:3,1:2,:]=val
      ref=Data(numpy.array([[[[7.0, 3.0, 4.0], [8.0, 7.0, 5.0], [4.0, 6.0, 3.0]], [[4.0, 5.0, 3.0], [3.0, 4.0, 4.0], [6.0, 5.0, 
6.0]], [[1.0, 2.0, 7.0], [7.0, 8.0, 6.0], [1.0, 4.0, 2.0]]], [[[6.0, 8.0, 3.0], [3.0, 8.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 6.0, 
1.0], [4.0, 2.0, 1.0], [3.0, 3.0, 5.0]], [[5.0, 8.0, 4.0], [4.0, 1.0, 2.0], [5.0, 2.0, 3.0]]], [[[7.0, 6.0, 8.0], [2.0, 3.0, 
2.0], [4.0, 7.0, 2.0]], [[5.0, 6.0, 2.0], [6.0, 5.0, 8.0], [6.0, 3.0, 8.0]], [[5.0, 5.0, 4.0], [3.0, 7.0, 5.0], [8.0, 5.0, 
5.0]]], [[[6.0, 4.0, 3.0], [3.0, 8.0, 4.0], [7.0, 8.0, 5.0]], [[2.0, 7.0, 6.0], [3.0, 1.0, 7.0], [2.0, 4.0, 8.0]], [[8.0, 6.0, 
4.0], [5.0, 2.0, 2.0], [3.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URCN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 1.0], [5.0, 3.0, 7.0], [4.0, 2.0, 6.0]], [[7.0, 6.0, 4.0], [8.0, 5.0, 6.0], [6.0, 7.0, 
6.0]], [[7.0, 1.0, 3.0], [4.0, 1.0, 6.0], [1.0, 2.0, 1.0]]], [[[5.0, 6.0, 1.0], [4.0, 7.0, 4.0], [2.0, 2.0, 5.0]], [[2.0, 7.0, 
5.0], [4.0, 5.0, 7.0], [5.0, 1.0, 3.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 6.0], [6.0, 6.0, 2.0]]], [[[4.0, 5.0, 4.0], [8.0, 1.0, 
3.0], [7.0, 2.0, 5.0]], [[1.0, 8.0, 5.0], [1.0, 6.0, 3.0], [3.0, 5.0, 1.0]], [[4.0, 5.0, 4.0], [2.0, 5.0, 5.0], [6.0, 2.0, 
1.0]]], [[[4.0, 4.0, 3.0], [1.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 2.0], [4.0, 4.0, 7.0], [8.0, 7.0, 4.0]], [[8.0, 3.0, 
6.0], [3.0, 1.0, 2.0], [4.0, 8.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 6.0, 5.0], [5.0, 6.0, 3.0], [4.0, 2.0, 5.0]]], [[[1.0, 1.0, 8.0], [7.0, 5.0, 1.0], [6.0, 
4.0, 7.0]]], [[[4.0, 7.0, 3.0], [4.0, 5.0, 3.0], [7.0, 4.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 1.0, 1.0], [5.0, 3.0, 
1.0], [7.0, 7.0, 5.0]]], [[[8.0, 7.0, 8.0], [8.0, 4.0, 2.0], [4.0, 5.0, 6.0]]], [[[3.0, 3.0, 5.0], [1.0, 3.0, 6.0], [6.0, 7.0, 
3.0]]]])
      arg[:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 1.0], [5.0, 3.0, 7.0], [4.0, 2.0, 6.0]], [[8.0, 6.0, 5.0], [5.0, 6.0, 3.0], [4.0, 
2.0, 5.0]], [[7.0, 1.0, 3.0], [4.0, 1.0, 6.0], [1.0, 2.0, 1.0]]], [[[5.0, 6.0, 1.0], [4.0, 7.0, 4.0], [2.0, 2.0, 5.0]], [[1.0, 
1.0, 8.0], [7.0, 5.0, 1.0], [6.0, 4.0, 7.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 6.0], [6.0, 6.0, 2.0]]], [[[4.0, 5.0, 4.0], [8.0, 
1.0, 3.0], [7.0, 2.0, 5.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 3.0], [7.0, 4.0, 7.0]], [[4.0, 5.0, 4.0], [2.0, 5.0, 5.0], [6.0, 2.0, 
1.0]]], [[[4.0, 4.0, 3.0], [1.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 2.0], [4.0, 4.0, 7.0], [8.0, 7.0, 4.0]], [[8.0, 3.0, 
6.0], [3.0, 1.0, 2.0], [4.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 1.0], [5.0, 3.0, 7.0], [4.0, 2.0, 6.0]], 
[[4.0, 1.0, 1.0], [5.0, 3.0, 1.0], [7.0, 7.0, 5.0]], [[7.0, 1.0, 3.0], [4.0, 1.0, 6.0], [1.0, 2.0, 1.0]]], [[[5.0, 6.0, 1.0], 
[4.0, 7.0, 4.0], [2.0, 2.0, 5.0]], [[8.0, 7.0, 8.0], [8.0, 4.0, 2.0], [4.0, 5.0, 6.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 6.0], 
[6.0, 6.0, 2.0]]], [[[4.0, 5.0, 4.0], [8.0, 1.0, 3.0], [7.0, 2.0, 5.0]], [[3.0, 3.0, 5.0], [1.0, 3.0, 6.0], [6.0, 7.0, 3.0]], 
[[4.0, 5.0, 4.0], [2.0, 5.0, 5.0], [6.0, 2.0, 1.0]]], [[[4.0, 4.0, 3.0], [1.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 2.0], 
[4.0, 4.0, 7.0], [8.0, 7.0, 4.0]], [[8.0, 3.0, 6.0], [3.0, 1.0, 2.0], [4.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCCN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 7.0], [4.0, 3.0, 8.0], [1.0, 3.0, 4.0]], [[5.0, 8.0, 5.0], [7.0, 2.0, 4.0], [1.0, 3.0, 
8.0]], [[3.0, 6.0, 3.0], [6.0, 8.0, 5.0], [2.0, 2.0, 5.0]]], [[[6.0, 4.0, 1.0], [5.0, 5.0, 4.0], [8.0, 7.0, 6.0]], [[8.0, 1.0, 
7.0], [6.0, 1.0, 1.0], [2.0, 6.0, 3.0]], [[7.0, 6.0, 4.0], [3.0, 2.0, 8.0], [1.0, 8.0, 6.0]]], [[[7.0, 1.0, 7.0], [3.0, 1.0, 
8.0], [1.0, 2.0, 2.0]], [[8.0, 3.0, 6.0], [8.0, 6.0, 3.0], [7.0, 8.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 5.0, 2.0], [5.0, 3.0, 
7.0]]], [[[7.0, 1.0, 5.0], [5.0, 5.0, 6.0], [5.0, 6.0, 2.0]], [[2.0, 7.0, 2.0], [3.0, 2.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 5.0, 
4.0], [6.0, 6.0, 7.0], [1.0, 4.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 1.0, 6.0], [3.0, 2.0, 7.0], [5.0, 2.0, 4.0]], [[3.0, 1.0, 3.0], [2.0, 6.0, 5.0], [5.0, 1.0, 
5.0]], [[4.0, 2.0, 8.0], [5.0, 7.0, 1.0], [7.0, 4.0, 2.0]]], [[[4.0, 6.0, 3.0], [4.0, 3.0, 6.0], [8.0, 2.0, 8.0]], [[4.0, 6.0, 
8.0], [7.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 1.0, 2.0], [1.0, 8.0, 1.0], [2.0, 2.0, 5.0]]], [[[6.0, 6.0, 8.0], [1.0, 3.0, 
7.0], [4.0, 5.0, 1.0]], [[8.0, 7.0, 6.0], [6.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 5.0], [8.0, 5.0, 2.0], [8.0, 6.0, 
3.0]]], [[[5.0, 7.0, 2.0], [5.0, 2.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 4.0, 3.0], [1.0, 8.0, 5.0], [7.0, 3.0, 8.0]], [[3.0, 3.0, 
4.0], [4.0, 1.0, 1.0], [6.0, 2.0, 8.0]]]])
      arg[:,:,:]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 6.0], [3.0, 2.0, 7.0], [5.0, 2.0, 4.0]], [[3.0, 1.0, 3.0], [2.0, 6.0, 5.0], [5.0, 1.0, 
5.0]], [[4.0, 2.0, 8.0], [5.0, 7.0, 1.0], [7.0, 4.0, 2.0]]], [[[4.0, 6.0, 3.0], [4.0, 3.0, 6.0], [8.0, 2.0, 8.0]], [[4.0, 6.0, 
8.0], [7.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 1.0, 2.0], [1.0, 8.0, 1.0], [2.0, 2.0, 5.0]]], [[[6.0, 6.0, 8.0], [1.0, 3.0, 
7.0], [4.0, 5.0, 1.0]], [[8.0, 7.0, 6.0], [6.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 5.0], [8.0, 5.0, 2.0], [8.0, 6.0, 
3.0]]], [[[5.0, 7.0, 2.0], [5.0, 2.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 4.0, 3.0], [1.0, 8.0, 5.0], [7.0, 3.0, 8.0]], [[3.0, 3.0, 
4.0], [4.0, 1.0, 1.0], [6.0, 2.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCCN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 3.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]], [[6.0, 4.0, 5.0], [8.0, 4.0, 3.0], [7.0, 2.0, 
3.0]], [[2.0, 8.0, 3.0], [4.0, 7.0, 7.0], [4.0, 1.0, 6.0]]], [[[5.0, 2.0, 2.0], [8.0, 4.0, 8.0], [6.0, 2.0, 5.0]], [[3.0, 7.0, 
2.0], [8.0, 4.0, 7.0], [3.0, 3.0, 5.0]], [[1.0, 7.0, 8.0], [1.0, 4.0, 1.0], [1.0, 2.0, 8.0]]], [[[3.0, 6.0, 3.0], [4.0, 2.0, 
7.0], [7.0, 4.0, 5.0]], [[3.0, 7.0, 3.0], [5.0, 8.0, 7.0], [8.0, 7.0, 1.0]], [[6.0, 7.0, 5.0], [6.0, 6.0, 7.0], [3.0, 3.0, 
7.0]]], [[[2.0, 2.0, 4.0], [4.0, 3.0, 3.0], [1.0, 2.0, 7.0]], [[3.0, 7.0, 4.0], [6.0, 4.0, 6.0], [2.0, 3.0, 2.0]], [[4.0, 1.0, 
1.0], [5.0, 4.0, 4.0], [2.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 4.0, 2.0], [5.0, 6.0, 6.0], [2.0, 3.0, 8.0]], [[6.0, 6.0, 1.0], [4.0, 8.0, 4.0], [3.0, 6.0, 
8.0]], [[1.0, 6.0, 8.0], [6.0, 6.0, 4.0], [1.0, 7.0, 6.0]]], [[[8.0, 4.0, 4.0], [3.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[1.0, 5.0, 
6.0], [7.0, 2.0, 8.0], [3.0, 6.0, 4.0]], [[8.0, 2.0, 3.0], [2.0, 2.0, 6.0], [1.0, 1.0, 3.0]]], [[[3.0, 6.0, 4.0], [6.0, 2.0, 
8.0], [1.0, 1.0, 6.0]], [[6.0, 5.0, 3.0], [4.0, 4.0, 3.0], [2.0, 3.0, 1.0]], [[1.0, 2.0, 5.0], [5.0, 1.0, 6.0], [2.0, 4.0, 
2.0]]], [[[7.0, 3.0, 5.0], [4.0, 6.0, 8.0], [8.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 7.0], [2.0, 1.0, 2.0]], [[1.0, 5.0, 
5.0], [7.0, 6.0, 6.0], [2.0, 2.0, 6.0]]]]),self.functionspace)
      arg[:,:,:]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 2.0], [5.0, 6.0, 6.0], [2.0, 3.0, 8.0]], [[6.0, 6.0, 1.0], [4.0, 8.0, 4.0], [3.0, 6.0, 
8.0]], [[1.0, 6.0, 8.0], [6.0, 6.0, 4.0], [1.0, 7.0, 6.0]]], [[[8.0, 4.0, 4.0], [3.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[1.0, 5.0, 
6.0], [7.0, 2.0, 8.0], [3.0, 6.0, 4.0]], [[8.0, 2.0, 3.0], [2.0, 2.0, 6.0], [1.0, 1.0, 3.0]]], [[[3.0, 6.0, 4.0], [6.0, 2.0, 
8.0], [1.0, 1.0, 6.0]], [[6.0, 5.0, 3.0], [4.0, 4.0, 3.0], [2.0, 3.0, 1.0]], [[1.0, 2.0, 5.0], [5.0, 1.0, 6.0], [2.0, 4.0, 
2.0]]], [[[7.0, 3.0, 5.0], [4.0, 6.0, 8.0], [8.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [2.0, 1.0, 7.0], [2.0, 1.0, 2.0]], [[1.0, 5.0, 
5.0], [7.0, 6.0, 6.0], [2.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCCN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 3.0], [7.0, 1.0, 2.0], [3.0, 8.0, 8.0]], [[2.0, 3.0, 4.0], [3.0, 5.0, 5.0], [3.0, 1.0, 
8.0]], [[3.0, 3.0, 2.0], [2.0, 6.0, 8.0], [6.0, 1.0, 3.0]]], [[[8.0, 7.0, 4.0], [2.0, 4.0, 7.0], [8.0, 4.0, 2.0]], [[6.0, 8.0, 
1.0], [2.0, 4.0, 7.0], [7.0, 7.0, 5.0]], [[1.0, 4.0, 5.0], [4.0, 4.0, 2.0], [1.0, 1.0, 5.0]]], [[[7.0, 6.0, 4.0], [5.0, 6.0, 
4.0], [7.0, 7.0, 1.0]], [[2.0, 3.0, 2.0], [6.0, 3.0, 5.0], [1.0, 2.0, 2.0]], [[1.0, 2.0, 4.0], [7.0, 6.0, 7.0], [3.0, 7.0, 
2.0]]], [[[2.0, 7.0, 5.0], [4.0, 1.0, 3.0], [1.0, 6.0, 2.0]], [[6.0, 5.0, 7.0], [1.0, 4.0, 3.0], [5.0, 8.0, 5.0]], [[3.0, 3.0, 
5.0], [5.0, 1.0, 2.0], [2.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 6.0, 1.0], [5.0, 3.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 8.0], [4.0, 3.0, 1.0], [2.0, 
4.0, 6.0]], [[6.0, 1.0, 8.0], [5.0, 1.0, 1.0], [7.0, 5.0, 7.0]]], [[[8.0, 5.0, 4.0], [3.0, 3.0, 2.0], [4.0, 5.0, 1.0]], [[3.0, 
6.0, 6.0], [8.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 5.0], [6.0, 3.0, 4.0], [7.0, 7.0, 4.0]]], [[[1.0, 4.0, 4.0], [1.0, 
1.0, 5.0], [7.0, 8.0, 4.0]], [[2.0, 7.0, 5.0], [4.0, 7.0, 7.0], [6.0, 3.0, 5.0]], [[6.0, 3.0, 1.0], [8.0, 7.0, 6.0], [3.0, 1.0, 
7.0]]], [[[2.0, 1.0, 5.0], [4.0, 5.0, 5.0], [6.0, 1.0, 5.0]], [[5.0, 2.0, 8.0], [1.0, 7.0, 2.0], [4.0, 4.0, 2.0]], [[5.0, 6.0, 
2.0], [3.0, 4.0, 2.0], [4.0, 7.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 6.0, 8.0], [6.0, 6.0, 4.0], [1.0, 4.0, 1.0]], 
[[7.0, 8.0, 6.0], [5.0, 7.0, 3.0], [3.0, 2.0, 1.0]], [[8.0, 5.0, 3.0], [8.0, 4.0, 4.0], [4.0, 7.0, 1.0]]], [[[3.0, 1.0, 2.0], 
[3.0, 2.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 6.0, 8.0], [8.0, 3.0, 1.0], [5.0, 2.0, 2.0]], [[6.0, 5.0, 7.0], [1.0, 3.0, 1.0], 
[4.0, 2.0, 4.0]]], [[[8.0, 3.0, 6.0], [8.0, 1.0, 6.0], [6.0, 5.0, 7.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 1.0], [3.0, 3.0, 4.0]], 
[[7.0, 2.0, 2.0], [3.0, 4.0, 2.0], [4.0, 1.0, 5.0]]], [[[1.0, 7.0, 7.0], [6.0, 1.0, 7.0], [1.0, 8.0, 8.0]], [[1.0, 4.0, 8.0], 
[6.0, 6.0, 1.0], [8.0, 6.0, 1.0]], [[6.0, 3.0, 3.0], [1.0, 8.0, 5.0], [4.0, 1.0, 5.0]]]])
      arg[:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 1.0], [5.0, 3.0, 5.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 8.0], [4.0, 3.0, 1.0], [2.0, 
4.0, 6.0]], [[6.0, 1.0, 8.0], [5.0, 1.0, 1.0], [7.0, 5.0, 7.0]]], [[[8.0, 5.0, 4.0], [3.0, 3.0, 2.0], [4.0, 5.0, 1.0]], [[3.0, 
6.0, 6.0], [8.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 5.0], [6.0, 3.0, 4.0], [7.0, 7.0, 4.0]]], [[[1.0, 4.0, 4.0], [1.0, 
1.0, 5.0], [7.0, 8.0, 4.0]], [[2.0, 7.0, 5.0], [4.0, 7.0, 7.0], [6.0, 3.0, 5.0]], [[6.0, 3.0, 1.0], [8.0, 7.0, 6.0], [3.0, 1.0, 
7.0]]], [[[2.0, 1.0, 5.0], [4.0, 5.0, 5.0], [6.0, 1.0, 5.0]], [[5.0, 2.0, 8.0], [1.0, 7.0, 2.0], [4.0, 4.0, 2.0]], [[5.0, 6.0, 
2.0], [3.0, 4.0, 2.0], [4.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 8.0], [6.0, 6.0, 4.0], [1.0, 4.0, 1.0]], 
[[7.0, 8.0, 6.0], [5.0, 7.0, 3.0], [3.0, 2.0, 1.0]], [[8.0, 5.0, 3.0], [8.0, 4.0, 4.0], [4.0, 7.0, 1.0]]], [[[3.0, 1.0, 2.0], 
[3.0, 2.0, 5.0], [2.0, 7.0, 2.0]], [[2.0, 6.0, 8.0], [8.0, 3.0, 1.0], [5.0, 2.0, 2.0]], [[6.0, 5.0, 7.0], [1.0, 3.0, 1.0], 
[4.0, 2.0, 4.0]]], [[[8.0, 3.0, 6.0], [8.0, 1.0, 6.0], [6.0, 5.0, 7.0]], [[1.0, 5.0, 4.0], [6.0, 8.0, 1.0], [3.0, 3.0, 4.0]], 
[[7.0, 2.0, 2.0], [3.0, 4.0, 2.0], [4.0, 1.0, 5.0]]], [[[1.0, 7.0, 7.0], [6.0, 1.0, 7.0], [1.0, 8.0, 8.0]], [[1.0, 4.0, 8.0], 
[6.0, 6.0, 1.0], [8.0, 6.0, 1.0]], [[6.0, 3.0, 3.0], [1.0, 8.0, 5.0], [4.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CRLN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 2.0], [3.0, 4.0, 8.0], [6.0, 4.0, 8.0]], [[6.0, 5.0, 5.0], [6.0, 6.0, 1.0], [8.0, 7.0, 
6.0]], [[3.0, 8.0, 6.0], [2.0, 6.0, 2.0], [2.0, 7.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 7.0, 2.0], [5.0, 3.0, 3.0]], [[8.0, 5.0, 
3.0], [3.0, 4.0, 7.0], [8.0, 8.0, 7.0]], [[5.0, 3.0, 8.0], [8.0, 7.0, 5.0], [2.0, 1.0, 1.0]]], [[[6.0, 3.0, 3.0], [4.0, 7.0, 
6.0], [2.0, 6.0, 5.0]], [[8.0, 3.0, 4.0], [5.0, 2.0, 7.0], [4.0, 1.0, 8.0]], [[4.0, 7.0, 6.0], [3.0, 1.0, 3.0], [8.0, 7.0, 
3.0]]], [[[5.0, 4.0, 6.0], [1.0, 2.0, 3.0], [4.0, 7.0, 8.0]], [[1.0, 7.0, 8.0], [2.0, 7.0, 7.0], [6.0, 8.0, 2.0]], [[5.0, 6.0, 
5.0], [7.0, 3.0, 3.0], [6.0, 8.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 3.0, 1.0]]], [[[6.0, 1.0, 4.0]]], [[[6.0, 3.0, 5.0]]], [[[5.0, 6.0, 4.0]]]])
      arg[:,1:2,2:]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 2.0], [3.0, 4.0, 8.0], [6.0, 4.0, 8.0]], [[6.0, 5.0, 5.0], [6.0, 6.0, 1.0], [4.0, 3.0, 
1.0]], [[3.0, 8.0, 6.0], [2.0, 6.0, 2.0], [2.0, 7.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 7.0, 2.0], [5.0, 3.0, 3.0]], [[8.0, 5.0, 
3.0], [3.0, 4.0, 7.0], [6.0, 1.0, 4.0]], [[5.0, 3.0, 8.0], [8.0, 7.0, 5.0], [2.0, 1.0, 1.0]]], [[[6.0, 3.0, 3.0], [4.0, 7.0, 
6.0], [2.0, 6.0, 5.0]], [[8.0, 3.0, 4.0], [5.0, 2.0, 7.0], [6.0, 3.0, 5.0]], [[4.0, 7.0, 6.0], [3.0, 1.0, 3.0], [8.0, 7.0, 
3.0]]], [[[5.0, 4.0, 6.0], [1.0, 2.0, 3.0], [4.0, 7.0, 8.0]], [[1.0, 7.0, 8.0], [2.0, 7.0, 7.0], [5.0, 6.0, 4.0]], [[5.0, 6.0, 
5.0], [7.0, 3.0, 3.0], [6.0, 8.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CRLN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 7.0], [4.0, 3.0, 8.0], [3.0, 5.0, 1.0]], [[8.0, 4.0, 7.0], [1.0, 1.0, 6.0], [2.0, 2.0, 
2.0]], [[1.0, 6.0, 1.0], [2.0, 8.0, 5.0], [5.0, 7.0, 2.0]]], [[[5.0, 4.0, 7.0], [1.0, 3.0, 8.0], [8.0, 8.0, 2.0]], [[1.0, 7.0, 
7.0], [2.0, 2.0, 7.0], [7.0, 7.0, 8.0]], [[7.0, 7.0, 2.0], [6.0, 4.0, 3.0], [8.0, 3.0, 5.0]]], [[[2.0, 7.0, 1.0], [2.0, 6.0, 
2.0], [1.0, 8.0, 3.0]], [[5.0, 3.0, 8.0], [6.0, 5.0, 4.0], [6.0, 5.0, 3.0]], [[6.0, 7.0, 6.0], [8.0, 6.0, 5.0], [6.0, 2.0, 
7.0]]], [[[1.0, 4.0, 5.0], [8.0, 7.0, 1.0], [8.0, 8.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 6.0], [4.0, 7.0, 5.0]], [[7.0, 5.0, 
8.0], [4.0, 5.0, 7.0], [2.0, 5.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 8.0, 7.0]]], [[[3.0, 6.0, 2.0]]], [[[4.0, 1.0, 2.0]]], [[[4.0, 7.0, 
1.0]]]]),self.functionspace)
      arg[:,1:2,2:]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 7.0], [4.0, 3.0, 8.0], [3.0, 5.0, 1.0]], [[8.0, 4.0, 7.0], [1.0, 1.0, 6.0], [4.0, 8.0, 
7.0]], [[1.0, 6.0, 1.0], [2.0, 8.0, 5.0], [5.0, 7.0, 2.0]]], [[[5.0, 4.0, 7.0], [1.0, 3.0, 8.0], [8.0, 8.0, 2.0]], [[1.0, 7.0, 
7.0], [2.0, 2.0, 7.0], [3.0, 6.0, 2.0]], [[7.0, 7.0, 2.0], [6.0, 4.0, 3.0], [8.0, 3.0, 5.0]]], [[[2.0, 7.0, 1.0], [2.0, 6.0, 
2.0], [1.0, 8.0, 3.0]], [[5.0, 3.0, 8.0], [6.0, 5.0, 4.0], [4.0, 1.0, 2.0]], [[6.0, 7.0, 6.0], [8.0, 6.0, 5.0], [6.0, 2.0, 
7.0]]], [[[1.0, 4.0, 5.0], [8.0, 7.0, 1.0], [8.0, 8.0, 2.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 6.0], [4.0, 7.0, 1.0]], [[7.0, 5.0, 
8.0], [4.0, 5.0, 7.0], [2.0, 5.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CRLN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 1.0], [2.0, 4.0, 3.0], [2.0, 2.0, 7.0]], [[6.0, 8.0, 8.0], [2.0, 8.0, 4.0], [6.0, 8.0, 
8.0]], [[5.0, 1.0, 2.0], [7.0, 5.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 8.0, 1.0], [2.0, 5.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 
3.0], [5.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 7.0], [4.0, 7.0, 7.0]]], [[[4.0, 8.0, 6.0], [2.0, 6.0, 
2.0], [5.0, 3.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 5.0, 4.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 4.0], [7.0, 4.0, 2.0], [7.0, 4.0, 
8.0]]], [[[3.0, 4.0, 8.0], [6.0, 7.0, 5.0], [7.0, 2.0, 2.0]], [[4.0, 8.0, 7.0], [4.0, 8.0, 6.0], [7.0, 6.0, 7.0]], [[7.0, 4.0, 
2.0], [6.0, 7.0, 3.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 6.0, 5.0]]], [[[5.0, 7.0, 8.0]]], [[[2.0, 6.0, 2.0]]], [[[2.0, 4.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 1.0, 6.0]]], [[[8.0, 3.0, 8.0]]], [[[1.0, 8.0, 2.0]]], [[[8.0, 7.0, 1.0]]]])
      arg[:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [2.0, 4.0, 3.0], [2.0, 2.0, 7.0]], [[6.0, 8.0, 8.0], [2.0, 8.0, 4.0], [4.0, 
6.0, 5.0]], [[5.0, 1.0, 2.0], [7.0, 5.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 8.0, 1.0], [2.0, 5.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 
1.0, 3.0], [5.0, 3.0, 5.0], [5.0, 7.0, 8.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 7.0], [4.0, 7.0, 7.0]]], [[[4.0, 8.0, 6.0], [2.0, 
6.0, 2.0], [5.0, 3.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 5.0, 4.0], [2.0, 6.0, 2.0]], [[3.0, 6.0, 4.0], [7.0, 4.0, 2.0], [7.0, 4.0, 
8.0]]], [[[3.0, 4.0, 8.0], [6.0, 7.0, 5.0], [7.0, 2.0, 2.0]], [[4.0, 8.0, 7.0], [4.0, 8.0, 6.0], [2.0, 4.0, 8.0]], [[7.0, 4.0, 
2.0], [6.0, 7.0, 3.0], [5.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 1.0], [2.0, 4.0, 3.0], [2.0, 2.0, 7.0]], 
[[6.0, 8.0, 8.0], [2.0, 8.0, 4.0], [8.0, 1.0, 6.0]], [[5.0, 1.0, 2.0], [7.0, 5.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 8.0, 1.0], 
[2.0, 5.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 1.0, 3.0], [5.0, 3.0, 5.0], [8.0, 3.0, 8.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 7.0], 
[4.0, 7.0, 7.0]]], [[[4.0, 8.0, 6.0], [2.0, 6.0, 2.0], [5.0, 3.0, 8.0]], [[6.0, 1.0, 1.0], [2.0, 5.0, 4.0], [1.0, 8.0, 2.0]], 
[[3.0, 6.0, 4.0], [7.0, 4.0, 2.0], [7.0, 4.0, 8.0]]], [[[3.0, 4.0, 8.0], [6.0, 7.0, 5.0], [7.0, 2.0, 2.0]], [[4.0, 8.0, 7.0], 
[4.0, 8.0, 6.0], [8.0, 7.0, 1.0]], [[7.0, 4.0, 2.0], [6.0, 7.0, 3.0], [5.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRRR(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 5.0], [1.0, 3.0, 8.0], [5.0, 7.0, 2.0]], [[7.0, 3.0, 3.0], [8.0, 1.0, 1.0], [6.0, 5.0, 
6.0]], [[3.0, 8.0, 1.0], [4.0, 7.0, 1.0], [2.0, 6.0, 5.0]]], [[[4.0, 3.0, 5.0], [8.0, 8.0, 6.0], [6.0, 7.0, 8.0]], [[8.0, 2.0, 
2.0], [1.0, 1.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 5.0, 2.0], [2.0, 5.0, 4.0], [4.0, 8.0, 2.0]]], [[[5.0, 7.0, 4.0], [4.0, 5.0, 
3.0], [2.0, 1.0, 3.0]], [[7.0, 8.0, 2.0], [6.0, 1.0, 3.0], [2.0, 1.0, 2.0]], [[1.0, 6.0, 8.0], [7.0, 7.0, 3.0], [4.0, 5.0, 
3.0]]], [[[5.0, 8.0, 7.0], [6.0, 7.0, 4.0], [3.0, 7.0, 2.0]], [[6.0, 5.0, 5.0], [4.0, 3.0, 1.0], [7.0, 6.0, 6.0]], [[2.0, 7.0, 
8.0], [6.0, 7.0, 4.0], [7.0, 2.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0]]], [[[8.0]]]])
      arg[1:3,1:2,1:2,1:2]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 5.0], [1.0, 3.0, 8.0], [5.0, 7.0, 2.0]], [[7.0, 3.0, 3.0], [8.0, 1.0, 1.0], [6.0, 5.0, 
6.0]], [[3.0, 8.0, 1.0], [4.0, 7.0, 1.0], [2.0, 6.0, 5.0]]], [[[4.0, 3.0, 5.0], [8.0, 8.0, 6.0], [6.0, 7.0, 8.0]], [[8.0, 2.0, 
2.0], [1.0, 7.0, 7.0], [3.0, 5.0, 3.0]], [[2.0, 5.0, 2.0], [2.0, 5.0, 4.0], [4.0, 8.0, 2.0]]], [[[5.0, 7.0, 4.0], [4.0, 5.0, 
3.0], [2.0, 1.0, 3.0]], [[7.0, 8.0, 2.0], [6.0, 8.0, 3.0], [2.0, 1.0, 2.0]], [[1.0, 6.0, 8.0], [7.0, 7.0, 3.0], [4.0, 5.0, 
3.0]]], [[[5.0, 8.0, 7.0], [6.0, 7.0, 4.0], [3.0, 7.0, 2.0]], [[6.0, 5.0, 5.0], [4.0, 3.0, 1.0], [7.0, 6.0, 6.0]], [[2.0, 7.0, 
8.0], [6.0, 7.0, 4.0], [7.0, 2.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRRR(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 7.0], [2.0, 1.0, 6.0], [8.0, 4.0, 3.0]], [[2.0, 1.0, 7.0], [7.0, 6.0, 4.0], [1.0, 2.0, 
4.0]], [[2.0, 1.0, 3.0], [7.0, 5.0, 7.0], [2.0, 5.0, 2.0]]], [[[3.0, 7.0, 6.0], [7.0, 3.0, 8.0], [1.0, 1.0, 8.0]], [[2.0, 1.0, 
7.0], [3.0, 1.0, 6.0], [1.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [7.0, 1.0, 6.0], [4.0, 1.0, 6.0]]], [[[8.0, 4.0, 6.0], [6.0, 8.0, 
2.0], [7.0, 5.0, 7.0]], [[4.0, 5.0, 6.0], [6.0, 4.0, 8.0], [7.0, 3.0, 1.0]], [[3.0, 6.0, 8.0], [3.0, 3.0, 4.0], [1.0, 6.0, 
7.0]]], [[[8.0, 1.0, 1.0], [5.0, 4.0, 5.0], [8.0, 7.0, 5.0]], [[7.0, 2.0, 2.0], [1.0, 2.0, 4.0], [4.0, 2.0, 5.0]], [[1.0, 6.0, 
4.0], [6.0, 1.0, 2.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0]]], [[[5.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2,1:2]=val
      ref=Data(numpy.array([[[[3.0, 3.0, 7.0], [2.0, 1.0, 6.0], [8.0, 4.0, 3.0]], [[2.0, 1.0, 7.0], [7.0, 6.0, 4.0], [1.0, 2.0, 
4.0]], [[2.0, 1.0, 3.0], [7.0, 5.0, 7.0], [2.0, 5.0, 2.0]]], [[[3.0, 7.0, 6.0], [7.0, 3.0, 8.0], [1.0, 1.0, 8.0]], [[2.0, 1.0, 
7.0], [3.0, 1.0, 6.0], [1.0, 2.0, 6.0]], [[8.0, 2.0, 3.0], [7.0, 1.0, 6.0], [4.0, 1.0, 6.0]]], [[[8.0, 4.0, 6.0], [6.0, 8.0, 
2.0], [7.0, 5.0, 7.0]], [[4.0, 5.0, 6.0], [6.0, 5.0, 8.0], [7.0, 3.0, 1.0]], [[3.0, 6.0, 8.0], [3.0, 3.0, 4.0], [1.0, 6.0, 
7.0]]], [[[8.0, 1.0, 1.0], [5.0, 4.0, 5.0], [8.0, 7.0, 5.0]], [[7.0, 2.0, 2.0], [1.0, 2.0, 4.0], [4.0, 2.0, 5.0]], [[1.0, 6.0, 
4.0], [6.0, 1.0, 2.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRRR(self):
      arg=Data(numpy.array([[[[1.0, 3.0, 3.0], [6.0, 6.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 4.0, 1.0], [8.0, 8.0, 8.0], [1.0, 3.0, 
3.0]], [[5.0, 8.0, 5.0], [7.0, 8.0, 7.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 8.0], [3.0, 1.0, 5.0], [8.0, 8.0, 2.0]], [[2.0, 3.0, 
6.0], [4.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 8.0], [2.0, 8.0, 4.0], [4.0, 2.0, 6.0]]], [[[7.0, 3.0, 6.0], [4.0, 2.0, 
1.0], [2.0, 5.0, 3.0]], [[5.0, 2.0, 2.0], [6.0, 2.0, 3.0], [3.0, 2.0, 5.0]], [[3.0, 8.0, 2.0], [6.0, 4.0, 6.0], [3.0, 1.0, 
4.0]]], [[[8.0, 8.0, 3.0], [4.0, 6.0, 7.0], [1.0, 2.0, 5.0]], [[5.0, 4.0, 2.0], [7.0, 4.0, 4.0], [4.0, 3.0, 5.0]], [[5.0, 1.0, 
2.0], [1.0, 6.0, 4.0], [8.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0]]], [[[4.0]]]])+(1.-msk_val)*numpy.array([[[[1.0]]], [[[1.0]]]])
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 3.0], [6.0, 6.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 4.0, 1.0], [8.0, 8.0, 8.0], [1.0, 
3.0, 3.0]], [[5.0, 8.0, 5.0], [7.0, 8.0, 7.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 8.0], [3.0, 1.0, 5.0], [8.0, 8.0, 2.0]], [[2.0, 
3.0, 6.0], [4.0, 7.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 8.0], [2.0, 8.0, 4.0], [4.0, 2.0, 6.0]]], [[[7.0, 3.0, 6.0], [4.0, 
2.0, 1.0], [2.0, 5.0, 3.0]], [[5.0, 2.0, 2.0], [6.0, 4.0, 3.0], [3.0, 2.0, 5.0]], [[3.0, 8.0, 2.0], [6.0, 4.0, 6.0], [3.0, 1.0, 
4.0]]], [[[8.0, 8.0, 3.0], [4.0, 6.0, 7.0], [1.0, 2.0, 5.0]], [[5.0, 4.0, 2.0], [7.0, 4.0, 4.0], [4.0, 3.0, 5.0]], [[5.0, 1.0, 
2.0], [1.0, 6.0, 4.0], [8.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 3.0], [6.0, 6.0, 3.0], [1.0, 8.0, 7.0]], 
[[7.0, 4.0, 1.0], [8.0, 8.0, 8.0], [1.0, 3.0, 3.0]], [[5.0, 8.0, 5.0], [7.0, 8.0, 7.0], [1.0, 1.0, 6.0]]], [[[8.0, 4.0, 8.0], 
[3.0, 1.0, 5.0], [8.0, 8.0, 2.0]], [[2.0, 3.0, 6.0], [4.0, 1.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 8.0], [2.0, 8.0, 4.0], 
[4.0, 2.0, 6.0]]], [[[7.0, 3.0, 6.0], [4.0, 2.0, 1.0], [2.0, 5.0, 3.0]], [[5.0, 2.0, 2.0], [6.0, 1.0, 3.0], [3.0, 2.0, 5.0]], 
[[3.0, 8.0, 2.0], [6.0, 4.0, 6.0], [3.0, 1.0, 4.0]]], [[[8.0, 8.0, 3.0], [4.0, 6.0, 7.0], [1.0, 2.0, 5.0]], [[5.0, 4.0, 2.0], 
[7.0, 4.0, 4.0], [4.0, 3.0, 5.0]], [[5.0, 1.0, 2.0], [1.0, 6.0, 4.0], [8.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRRN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 8.0], [4.0, 8.0, 2.0], [2.0, 7.0, 7.0]], [[2.0, 1.0, 4.0], [6.0, 3.0, 8.0], [8.0, 4.0, 
2.0]], [[5.0, 8.0, 7.0], [5.0, 5.0, 5.0], [6.0, 7.0, 3.0]]], [[[4.0, 6.0, 5.0], [1.0, 8.0, 1.0], [7.0, 5.0, 3.0]], [[4.0, 4.0, 
7.0], [5.0, 7.0, 4.0], [8.0, 8.0, 7.0]], [[4.0, 8.0, 7.0], [1.0, 7.0, 4.0], [3.0, 5.0, 3.0]]], [[[3.0, 3.0, 5.0], [1.0, 8.0, 
7.0], [1.0, 8.0, 3.0]], [[8.0, 7.0, 7.0], [1.0, 3.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 5.0, 3.0], [8.0, 3.0, 1.0], [8.0, 4.0, 
2.0]]], [[[5.0, 6.0, 5.0], [5.0, 6.0, 3.0], [8.0, 4.0, 2.0]], [[2.0, 3.0, 3.0], [2.0, 4.0, 4.0], [6.0, 3.0, 8.0]], [[3.0, 7.0, 
1.0], [3.0, 7.0, 3.0], [5.0, 5.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[[1.0, 1.0, 4.0]]], [[[7.0, 7.0, 7.0]]]])
      arg[1:3,1:2,1:2]=val
      ref=Data(numpy.array([[[[7.0, 8.0, 8.0], [4.0, 8.0, 2.0], [2.0, 7.0, 7.0]], [[2.0, 1.0, 4.0], [6.0, 3.0, 8.0], [8.0, 4.0, 
2.0]], [[5.0, 8.0, 7.0], [5.0, 5.0, 5.0], [6.0, 7.0, 3.0]]], [[[4.0, 6.0, 5.0], [1.0, 8.0, 1.0], [7.0, 5.0, 3.0]], [[4.0, 4.0, 
7.0], [1.0, 1.0, 4.0], [8.0, 8.0, 7.0]], [[4.0, 8.0, 7.0], [1.0, 7.0, 4.0], [3.0, 5.0, 3.0]]], [[[3.0, 3.0, 5.0], [1.0, 8.0, 
7.0], [1.0, 8.0, 3.0]], [[8.0, 7.0, 7.0], [7.0, 7.0, 7.0], [4.0, 3.0, 3.0]], [[2.0, 5.0, 3.0], [8.0, 3.0, 1.0], [8.0, 4.0, 
2.0]]], [[[5.0, 6.0, 5.0], [5.0, 6.0, 3.0], [8.0, 4.0, 2.0]], [[2.0, 3.0, 3.0], [2.0, 4.0, 4.0], [6.0, 3.0, 8.0]], [[3.0, 7.0, 
1.0], [3.0, 7.0, 3.0], [5.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRRN(self):
      arg=Data(numpy.array([[[[8.0, 7.0, 8.0], [7.0, 1.0, 1.0], [4.0, 6.0, 2.0]], [[8.0, 7.0, 6.0], [2.0, 7.0, 2.0], [7.0, 3.0, 
5.0]], [[2.0, 6.0, 7.0], [5.0, 6.0, 3.0], [7.0, 6.0, 3.0]]], [[[6.0, 2.0, 4.0], [8.0, 1.0, 4.0], [6.0, 3.0, 4.0]], [[6.0, 2.0, 
3.0], [2.0, 4.0, 7.0], [5.0, 4.0, 4.0]], [[2.0, 6.0, 5.0], [8.0, 1.0, 5.0], [5.0, 7.0, 7.0]]], [[[3.0, 6.0, 7.0], [2.0, 4.0, 
4.0], [6.0, 2.0, 8.0]], [[4.0, 1.0, 1.0], [4.0, 8.0, 1.0], [3.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 4.0], [3.0, 4.0, 
6.0]]], [[[4.0, 1.0, 2.0], [8.0, 6.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 1.0, 1.0], [5.0, 3.0, 7.0], [7.0, 2.0, 6.0]], [[5.0, 5.0, 
3.0], [2.0, 3.0, 8.0], [7.0, 4.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 8.0, 7.0]]], [[[7.0, 1.0, 6.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2]=val
      ref=Data(numpy.array([[[[8.0, 7.0, 8.0], [7.0, 1.0, 1.0], [4.0, 6.0, 2.0]], [[8.0, 7.0, 6.0], [2.0, 7.0, 2.0], [7.0, 3.0, 
5.0]], [[2.0, 6.0, 7.0], [5.0, 6.0, 3.0], [7.0, 6.0, 3.0]]], [[[6.0, 2.0, 4.0], [8.0, 1.0, 4.0], [6.0, 3.0, 4.0]], [[6.0, 2.0, 
3.0], [6.0, 8.0, 7.0], [5.0, 4.0, 4.0]], [[2.0, 6.0, 5.0], [8.0, 1.0, 5.0], [5.0, 7.0, 7.0]]], [[[3.0, 6.0, 7.0], [2.0, 4.0, 
4.0], [6.0, 2.0, 8.0]], [[4.0, 1.0, 1.0], [7.0, 1.0, 6.0], [3.0, 4.0, 3.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 4.0], [3.0, 4.0, 
6.0]]], [[[4.0, 1.0, 2.0], [8.0, 6.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 1.0, 1.0], [5.0, 3.0, 7.0], [7.0, 2.0, 6.0]], [[5.0, 5.0, 
3.0], [2.0, 3.0, 8.0], [7.0, 4.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRRN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 1.0], [5.0, 3.0, 6.0], [3.0, 1.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 5.0, 6.0], [2.0, 6.0, 
8.0]], [[7.0, 2.0, 8.0], [1.0, 5.0, 3.0], [1.0, 7.0, 1.0]]], [[[2.0, 5.0, 1.0], [5.0, 5.0, 2.0], [4.0, 3.0, 8.0]], [[1.0, 6.0, 
3.0], [7.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 3.0], [4.0, 5.0, 5.0]]], [[[8.0, 7.0, 4.0], [7.0, 1.0, 
4.0], [3.0, 5.0, 6.0]], [[4.0, 3.0, 5.0], [5.0, 6.0, 7.0], [4.0, 5.0, 1.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 8.0], [6.0, 6.0, 
7.0]]], [[[5.0, 6.0, 3.0], [3.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], [7.0, 1.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 
8.0], [3.0, 5.0, 5.0], [4.0, 8.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 2.0, 3.0]]], [[[2.0, 2.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 7.0, 8.0]]], 
[[[8.0, 3.0, 2.0]]]])
      arg[1:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 1.0], [5.0, 3.0, 6.0], [3.0, 1.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 5.0, 6.0], [2.0, 
6.0, 8.0]], [[7.0, 2.0, 8.0], [1.0, 5.0, 3.0], [1.0, 7.0, 1.0]]], [[[2.0, 5.0, 1.0], [5.0, 5.0, 2.0], [4.0, 3.0, 8.0]], [[1.0, 
6.0, 3.0], [8.0, 2.0, 3.0], [8.0, 8.0, 5.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 3.0], [4.0, 5.0, 5.0]]], [[[8.0, 7.0, 4.0], [7.0, 
1.0, 4.0], [3.0, 5.0, 6.0]], [[4.0, 3.0, 5.0], [2.0, 2.0, 2.0], [4.0, 5.0, 1.0]], [[8.0, 6.0, 8.0], [1.0, 7.0, 8.0], [6.0, 6.0, 
7.0]]], [[[5.0, 6.0, 3.0], [3.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], [7.0, 1.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 
8.0], [3.0, 5.0, 5.0], [4.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 1.0], [5.0, 3.0, 6.0], [3.0, 1.0, 8.0]], 
[[2.0, 6.0, 8.0], [1.0, 5.0, 6.0], [2.0, 6.0, 8.0]], [[7.0, 2.0, 8.0], [1.0, 5.0, 3.0], [1.0, 7.0, 1.0]]], [[[2.0, 5.0, 1.0], 
[5.0, 5.0, 2.0], [4.0, 3.0, 8.0]], [[1.0, 6.0, 3.0], [2.0, 7.0, 8.0], [8.0, 8.0, 5.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 3.0], 
[4.0, 5.0, 5.0]]], [[[8.0, 7.0, 4.0], [7.0, 1.0, 4.0], [3.0, 5.0, 6.0]], [[4.0, 3.0, 5.0], [8.0, 3.0, 2.0], [4.0, 5.0, 1.0]], 
[[8.0, 6.0, 8.0], [1.0, 7.0, 8.0], [6.0, 6.0, 7.0]]], [[[5.0, 6.0, 3.0], [3.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], 
[7.0, 1.0, 5.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 8.0], [3.0, 5.0, 5.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PUPN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 7.0], [5.0, 6.0, 3.0], [8.0, 1.0, 6.0]], [[7.0, 4.0, 6.0], [7.0, 8.0, 8.0], [2.0, 5.0, 
4.0]], [[3.0, 1.0, 2.0], [6.0, 8.0, 1.0], [6.0, 6.0, 1.0]]], [[[2.0, 8.0, 1.0], [7.0, 7.0, 1.0], [4.0, 8.0, 3.0]], [[8.0, 5.0, 
1.0], [3.0, 5.0, 2.0], [3.0, 8.0, 3.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 3.0], [6.0, 5.0, 3.0]]], [[[6.0, 6.0, 3.0], [1.0, 4.0, 
5.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 4.0], [3.0, 8.0, 7.0], [2.0, 8.0, 6.0]], [[1.0, 2.0, 3.0], [1.0, 6.0, 1.0], [3.0, 3.0, 
7.0]]], [[[8.0, 6.0, 7.0], [6.0, 7.0, 3.0], [8.0, 8.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 2.0, 8.0], [7.0, 2.0, 8.0]], [[8.0, 1.0, 
4.0], [7.0, 4.0, 6.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[1.0, 4.0, 5.0], [7.0, 4.0, 8.0]])
      arg[2,:2,1]=val
      ref=Data(numpy.array([[[[2.0, 8.0, 7.0], [5.0, 6.0, 3.0], [8.0, 1.0, 6.0]], [[7.0, 4.0, 6.0], [7.0, 8.0, 8.0], [2.0, 5.0, 
4.0]], [[3.0, 1.0, 2.0], [6.0, 8.0, 1.0], [6.0, 6.0, 1.0]]], [[[2.0, 8.0, 1.0], [7.0, 7.0, 1.0], [4.0, 8.0, 3.0]], [[8.0, 5.0, 
1.0], [3.0, 5.0, 2.0], [3.0, 8.0, 3.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 3.0], [6.0, 5.0, 3.0]]], [[[6.0, 6.0, 3.0], [1.0, 4.0, 
5.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 4.0], [7.0, 4.0, 8.0], [2.0, 8.0, 6.0]], [[1.0, 2.0, 3.0], [1.0, 6.0, 1.0], [3.0, 3.0, 
7.0]]], [[[8.0, 6.0, 7.0], [6.0, 7.0, 3.0], [8.0, 8.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 2.0, 8.0], [7.0, 2.0, 8.0]], [[8.0, 1.0, 
4.0], [7.0, 4.0, 6.0], [3.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PUPN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 3.0], [3.0, 5.0, 2.0], [7.0, 1.0, 8.0]], [[2.0, 1.0, 3.0], [4.0, 7.0, 1.0], [3.0, 6.0, 
2.0]], [[7.0, 6.0, 8.0], [1.0, 8.0, 1.0], [1.0, 3.0, 6.0]]], [[[2.0, 8.0, 7.0], [4.0, 5.0, 1.0], [1.0, 3.0, 2.0]], [[3.0, 3.0, 
2.0], [8.0, 1.0, 8.0], [2.0, 3.0, 5.0]], [[1.0, 5.0, 5.0], [6.0, 1.0, 3.0], [5.0, 1.0, 6.0]]], [[[6.0, 4.0, 7.0], [5.0, 3.0, 
5.0], [2.0, 3.0, 5.0]], [[8.0, 5.0, 4.0], [5.0, 6.0, 5.0], [5.0, 6.0, 8.0]], [[5.0, 1.0, 4.0], [2.0, 8.0, 6.0], [4.0, 7.0, 
1.0]]], [[[1.0, 7.0, 8.0], [4.0, 4.0, 3.0], [7.0, 2.0, 2.0]], [[6.0, 6.0, 1.0], [4.0, 4.0, 7.0], [2.0, 2.0, 7.0]], [[8.0, 1.0, 
1.0], [8.0, 4.0, 2.0], [5.0, 2.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[6.0, 8.0, 2.0], [8.0, 3.0, 6.0]]),self.functionspace)
      arg[2,:2,1]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 3.0], [3.0, 5.0, 2.0], [7.0, 1.0, 8.0]], [[2.0, 1.0, 3.0], [4.0, 7.0, 1.0], [3.0, 6.0, 
2.0]], [[7.0, 6.0, 8.0], [1.0, 8.0, 1.0], [1.0, 3.0, 6.0]]], [[[2.0, 8.0, 7.0], [4.0, 5.0, 1.0], [1.0, 3.0, 2.0]], [[3.0, 3.0, 
2.0], [8.0, 1.0, 8.0], [2.0, 3.0, 5.0]], [[1.0, 5.0, 5.0], [6.0, 1.0, 3.0], [5.0, 1.0, 6.0]]], [[[6.0, 4.0, 7.0], [6.0, 8.0, 
2.0], [2.0, 3.0, 5.0]], [[8.0, 5.0, 4.0], [8.0, 3.0, 6.0], [5.0, 6.0, 8.0]], [[5.0, 1.0, 4.0], [2.0, 8.0, 6.0], [4.0, 7.0, 
1.0]]], [[[1.0, 7.0, 8.0], [4.0, 4.0, 3.0], [7.0, 2.0, 2.0]], [[6.0, 6.0, 1.0], [4.0, 4.0, 7.0], [2.0, 2.0, 7.0]], [[8.0, 1.0, 
1.0], [8.0, 4.0, 2.0], [5.0, 2.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PUPN(self):
      arg=Data(numpy.array([[[[1.0, 2.0, 5.0], [1.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[8.0, 3.0, 6.0], [1.0, 8.0, 7.0], [6.0, 1.0, 
5.0]], [[6.0, 3.0, 6.0], [2.0, 7.0, 2.0], [2.0, 4.0, 3.0]]], [[[5.0, 7.0, 6.0], [8.0, 2.0, 8.0], [5.0, 6.0, 3.0]], [[1.0, 5.0, 
1.0], [3.0, 5.0, 7.0], [6.0, 6.0, 1.0]], [[2.0, 7.0, 4.0], [4.0, 2.0, 2.0], [3.0, 3.0, 8.0]]], [[[7.0, 4.0, 2.0], [6.0, 6.0, 
4.0], [5.0, 7.0, 4.0]], [[2.0, 2.0, 6.0], [6.0, 2.0, 6.0], [3.0, 6.0, 1.0]], [[4.0, 5.0, 1.0], [3.0, 5.0, 6.0], [6.0, 6.0, 
1.0]]], [[[4.0, 2.0, 6.0], [7.0, 2.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], [3.0, 8.0, 8.0], [5.0, 7.0, 1.0]], [[6.0, 3.0, 
6.0], [1.0, 8.0, 2.0], [2.0, 6.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 6.0, 8.0], [5.0, 3.0, 8.0]])+(1.-msk_val)*numpy.array([[6.0, 4.0, 3.0], [6.0, 6.0, 7.0]])
      arg[2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [1.0, 2.0, 4.0], [2.0, 1.0, 1.0]], [[8.0, 3.0, 6.0], [1.0, 8.0, 7.0], [6.0, 
1.0, 5.0]], [[6.0, 3.0, 6.0], [2.0, 7.0, 2.0], [2.0, 4.0, 3.0]]], [[[5.0, 7.0, 6.0], [8.0, 2.0, 8.0], [5.0, 6.0, 3.0]], [[1.0, 
5.0, 1.0], [3.0, 5.0, 7.0], [6.0, 6.0, 1.0]], [[2.0, 7.0, 4.0], [4.0, 2.0, 2.0], [3.0, 3.0, 8.0]]], [[[7.0, 4.0, 2.0], [5.0, 
6.0, 8.0], [5.0, 7.0, 4.0]], [[2.0, 2.0, 6.0], [5.0, 3.0, 8.0], [3.0, 6.0, 1.0]], [[4.0, 5.0, 1.0], [3.0, 5.0, 6.0], [6.0, 6.0, 
1.0]]], [[[4.0, 2.0, 6.0], [7.0, 2.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], [3.0, 8.0, 8.0], [5.0, 7.0, 1.0]], [[6.0, 3.0, 
6.0], [1.0, 8.0, 2.0], [2.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 5.0], [1.0, 2.0, 4.0], [2.0, 1.0, 1.0]], 
[[8.0, 3.0, 6.0], [1.0, 8.0, 7.0], [6.0, 1.0, 5.0]], [[6.0, 3.0, 6.0], [2.0, 7.0, 2.0], [2.0, 4.0, 3.0]]], [[[5.0, 7.0, 6.0], 
[8.0, 2.0, 8.0], [5.0, 6.0, 3.0]], [[1.0, 5.0, 1.0], [3.0, 5.0, 7.0], [6.0, 6.0, 1.0]], [[2.0, 7.0, 4.0], [4.0, 2.0, 2.0], 
[3.0, 3.0, 8.0]]], [[[7.0, 4.0, 2.0], [6.0, 4.0, 3.0], [5.0, 7.0, 4.0]], [[2.0, 2.0, 6.0], [6.0, 6.0, 7.0], [3.0, 6.0, 1.0]], 
[[4.0, 5.0, 1.0], [3.0, 5.0, 6.0], [6.0, 6.0, 1.0]]], [[[4.0, 2.0, 6.0], [7.0, 2.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 6.0, 1.0], 
[3.0, 8.0, 8.0], [5.0, 7.0, 1.0]], [[6.0, 3.0, 6.0], [1.0, 8.0, 2.0], [2.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPRN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 1.0], [8.0, 2.0, 8.0], [2.0, 6.0, 4.0]], [[3.0, 2.0, 4.0], [3.0, 1.0, 4.0], [2.0, 2.0, 
7.0]], [[6.0, 6.0, 6.0], [7.0, 1.0, 5.0], [6.0, 6.0, 6.0]]], [[[2.0, 5.0, 7.0], [5.0, 5.0, 3.0], [2.0, 2.0, 7.0]], [[5.0, 6.0, 
8.0], [1.0, 1.0, 4.0], [8.0, 4.0, 7.0]], [[6.0, 1.0, 4.0], [8.0, 3.0, 6.0], [3.0, 6.0, 1.0]]], [[[1.0, 1.0, 7.0], [7.0, 2.0, 
2.0], [6.0, 6.0, 3.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 8.0], [1.0, 7.0, 2.0]], [[3.0, 7.0, 8.0], [1.0, 7.0, 5.0], [8.0, 3.0, 
5.0]]], [[[2.0, 4.0, 2.0], [1.0, 6.0, 7.0], [4.0, 2.0, 7.0]], [[7.0, 6.0, 6.0], [2.0, 8.0, 8.0], [8.0, 5.0, 2.0]], [[7.0, 5.0, 
8.0], [6.0, 2.0, 1.0], [5.0, 6.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 2.0, 1.0]], [[3.0, 3.0, 8.0]]])
      arg[1:3,1,1:2]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 1.0], [8.0, 2.0, 8.0], [2.0, 6.0, 4.0]], [[3.0, 2.0, 4.0], [3.0, 1.0, 4.0], [2.0, 2.0, 
7.0]], [[6.0, 6.0, 6.0], [7.0, 1.0, 5.0], [6.0, 6.0, 6.0]]], [[[2.0, 5.0, 7.0], [5.0, 5.0, 3.0], [2.0, 2.0, 7.0]], [[5.0, 6.0, 
8.0], [4.0, 2.0, 1.0], [8.0, 4.0, 7.0]], [[6.0, 1.0, 4.0], [8.0, 3.0, 6.0], [3.0, 6.0, 1.0]]], [[[1.0, 1.0, 7.0], [7.0, 2.0, 
2.0], [6.0, 6.0, 3.0]], [[4.0, 5.0, 7.0], [3.0, 3.0, 8.0], [1.0, 7.0, 2.0]], [[3.0, 7.0, 8.0], [1.0, 7.0, 5.0], [8.0, 3.0, 
5.0]]], [[[2.0, 4.0, 2.0], [1.0, 6.0, 7.0], [4.0, 2.0, 7.0]], [[7.0, 6.0, 6.0], [2.0, 8.0, 8.0], [8.0, 5.0, 2.0]], [[7.0, 5.0, 
8.0], [6.0, 2.0, 1.0], [5.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPRN(self):
      arg=Data(numpy.array([[[[4.0, 7.0, 5.0], [1.0, 3.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 1.0, 2.0], [6.0, 5.0, 3.0], [7.0, 2.0, 
1.0]], [[3.0, 3.0, 3.0], [2.0, 7.0, 2.0], [3.0, 1.0, 4.0]]], [[[2.0, 6.0, 2.0], [6.0, 8.0, 1.0], [8.0, 5.0, 5.0]], [[8.0, 5.0, 
3.0], [8.0, 3.0, 3.0], [6.0, 5.0, 4.0]], [[3.0, 8.0, 7.0], [6.0, 2.0, 7.0], [7.0, 3.0, 2.0]]], [[[2.0, 4.0, 8.0], [7.0, 1.0, 
2.0], [4.0, 4.0, 2.0]], [[6.0, 8.0, 4.0], [1.0, 3.0, 5.0], [2.0, 3.0, 6.0]], [[4.0, 2.0, 8.0], [8.0, 4.0, 1.0], [4.0, 8.0, 
1.0]]], [[[5.0, 7.0, 7.0], [4.0, 1.0, 1.0], [3.0, 8.0, 2.0]], [[3.0, 4.0, 4.0], [6.0, 3.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 1.0, 
1.0], [4.0, 1.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 5.0, 6.0]], [[7.0, 6.0, 1.0]]]),self.functionspace)
      arg[1:3,1,1:2]=val
      ref=Data(numpy.array([[[[4.0, 7.0, 5.0], [1.0, 3.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 1.0, 2.0], [6.0, 5.0, 3.0], [7.0, 2.0, 
1.0]], [[3.0, 3.0, 3.0], [2.0, 7.0, 2.0], [3.0, 1.0, 4.0]]], [[[2.0, 6.0, 2.0], [6.0, 8.0, 1.0], [8.0, 5.0, 5.0]], [[8.0, 5.0, 
3.0], [5.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[3.0, 8.0, 7.0], [6.0, 2.0, 7.0], [7.0, 3.0, 2.0]]], [[[2.0, 4.0, 8.0], [7.0, 1.0, 
2.0], [4.0, 4.0, 2.0]], [[6.0, 8.0, 4.0], [7.0, 6.0, 1.0], [2.0, 3.0, 6.0]], [[4.0, 2.0, 8.0], [8.0, 4.0, 1.0], [4.0, 8.0, 
1.0]]], [[[5.0, 7.0, 7.0], [4.0, 1.0, 1.0], [3.0, 8.0, 2.0]], [[3.0, 4.0, 4.0], [6.0, 3.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 1.0, 
1.0], [4.0, 1.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPRN(self):
      arg=Data(numpy.array([[[[5.0, 6.0, 5.0], [3.0, 3.0, 5.0], [8.0, 5.0, 5.0]], [[4.0, 3.0, 5.0], [6.0, 1.0, 1.0], [6.0, 5.0, 
8.0]], [[4.0, 6.0, 1.0], [4.0, 7.0, 6.0], [8.0, 8.0, 5.0]]], [[[2.0, 3.0, 4.0], [8.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[6.0, 7.0, 
6.0], [3.0, 8.0, 7.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 5.0], [6.0, 7.0, 5.0], [8.0, 6.0, 1.0]]], [[[5.0, 2.0, 5.0], [7.0, 5.0, 
5.0], [1.0, 6.0, 1.0]], [[7.0, 1.0, 3.0], [4.0, 4.0, 3.0], [8.0, 7.0, 2.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 3.0], [5.0, 8.0, 
8.0]]], [[[3.0, 2.0, 3.0], [4.0, 4.0, 4.0], [2.0, 1.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 1.0, 6.0], [5.0, 5.0, 5.0]], [[6.0, 5.0, 
8.0], [1.0, 3.0, 7.0], [2.0, 5.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 1.0, 4.0]], [[4.0, 6.0, 8.0]]])+(1.-msk_val)*numpy.array([[[3.0, 5.0, 8.0]], [[1.0, 5.0, 
7.0]]])
      arg[1:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 5.0], [3.0, 3.0, 5.0], [8.0, 5.0, 5.0]], [[4.0, 3.0, 5.0], [6.0, 1.0, 1.0], [6.0, 
5.0, 8.0]], [[4.0, 6.0, 1.0], [4.0, 7.0, 6.0], [8.0, 8.0, 5.0]]], [[[2.0, 3.0, 4.0], [8.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[6.0, 
7.0, 6.0], [4.0, 1.0, 4.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 5.0], [6.0, 7.0, 5.0], [8.0, 6.0, 1.0]]], [[[5.0, 2.0, 5.0], [7.0, 
5.0, 5.0], [1.0, 6.0, 1.0]], [[7.0, 1.0, 3.0], [4.0, 6.0, 8.0], [8.0, 7.0, 2.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 3.0], [5.0, 8.0, 
8.0]]], [[[3.0, 2.0, 3.0], [4.0, 4.0, 4.0], [2.0, 1.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 1.0, 6.0], [5.0, 5.0, 5.0]], [[6.0, 5.0, 
8.0], [1.0, 3.0, 7.0], [2.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 5.0], [3.0, 3.0, 5.0], [8.0, 5.0, 5.0]], 
[[4.0, 3.0, 5.0], [6.0, 1.0, 1.0], [6.0, 5.0, 8.0]], [[4.0, 6.0, 1.0], [4.0, 7.0, 6.0], [8.0, 8.0, 5.0]]], [[[2.0, 3.0, 4.0], 
[8.0, 7.0, 1.0], [4.0, 3.0, 8.0]], [[6.0, 7.0, 6.0], [3.0, 5.0, 8.0], [6.0, 3.0, 3.0]], [[8.0, 1.0, 5.0], [6.0, 7.0, 5.0], 
[8.0, 6.0, 1.0]]], [[[5.0, 2.0, 5.0], [7.0, 5.0, 5.0], [1.0, 6.0, 1.0]], [[7.0, 1.0, 3.0], [1.0, 5.0, 7.0], [8.0, 7.0, 2.0]], 
[[6.0, 3.0, 5.0], [1.0, 7.0, 3.0], [5.0, 8.0, 8.0]]], [[[3.0, 2.0, 3.0], [4.0, 4.0, 4.0], [2.0, 1.0, 4.0]], [[1.0, 6.0, 5.0], 
[1.0, 1.0, 6.0], [5.0, 5.0, 5.0]], [[6.0, 5.0, 8.0], [1.0, 3.0, 7.0], [2.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCRN(self):
      arg=Data(numpy.array([[[[6.0, 7.0, 4.0], [6.0, 2.0, 1.0], [3.0, 6.0, 4.0]], [[4.0, 1.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 
8.0]], [[6.0, 5.0, 1.0], [4.0, 7.0, 4.0], [6.0, 6.0, 3.0]]], [[[3.0, 1.0, 8.0], [3.0, 6.0, 6.0], [7.0, 3.0, 8.0]], [[8.0, 3.0, 
5.0], [8.0, 2.0, 3.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 4.0], [4.0, 5.0, 6.0], [4.0, 5.0, 3.0]]], [[[4.0, 8.0, 3.0], [8.0, 5.0, 
4.0], [6.0, 2.0, 8.0]], [[8.0, 8.0, 3.0], [8.0, 1.0, 1.0], [7.0, 1.0, 7.0]], [[5.0, 6.0, 3.0], [7.0, 4.0, 3.0], [8.0, 3.0, 
1.0]]], [[[7.0, 4.0, 7.0], [7.0, 5.0, 6.0], [5.0, 5.0, 6.0]], [[3.0, 5.0, 2.0], [1.0, 6.0, 2.0], [5.0, 6.0, 4.0]], [[8.0, 6.0, 
2.0], [1.0, 5.0, 2.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 2.0, 4.0]], [[3.0, 1.0, 8.0]], [[6.0, 3.0, 2.0]]], [[[5.0, 6.0, 6.0]], [[6.0, 4.0, 4.0]], [[5.0, 
6.0, 1.0]]]])
      arg[1:3,:,1:2]=val
      ref=Data(numpy.array([[[[6.0, 7.0, 4.0], [6.0, 2.0, 1.0], [3.0, 6.0, 4.0]], [[4.0, 1.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 
8.0]], [[6.0, 5.0, 1.0], [4.0, 7.0, 4.0], [6.0, 6.0, 3.0]]], [[[3.0, 1.0, 8.0], [6.0, 2.0, 4.0], [7.0, 3.0, 8.0]], [[8.0, 3.0, 
5.0], [3.0, 1.0, 8.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 4.0], [6.0, 3.0, 2.0], [4.0, 5.0, 3.0]]], [[[4.0, 8.0, 3.0], [5.0, 6.0, 
6.0], [6.0, 2.0, 8.0]], [[8.0, 8.0, 3.0], [6.0, 4.0, 4.0], [7.0, 1.0, 7.0]], [[5.0, 6.0, 3.0], [5.0, 6.0, 1.0], [8.0, 3.0, 
1.0]]], [[[7.0, 4.0, 7.0], [7.0, 5.0, 6.0], [5.0, 5.0, 6.0]], [[3.0, 5.0, 2.0], [1.0, 6.0, 2.0], [5.0, 6.0, 4.0]], [[8.0, 6.0, 
2.0], [1.0, 5.0, 2.0], [7.0, 5.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCRN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 5.0], [5.0, 1.0, 3.0], [8.0, 8.0, 6.0]], [[1.0, 4.0, 6.0], [5.0, 7.0, 6.0], [6.0, 4.0, 
1.0]], [[3.0, 1.0, 8.0], [1.0, 3.0, 2.0], [2.0, 6.0, 8.0]]], [[[2.0, 6.0, 4.0], [3.0, 8.0, 6.0], [2.0, 4.0, 8.0]], [[4.0, 4.0, 
6.0], [1.0, 3.0, 8.0], [6.0, 4.0, 5.0]], [[2.0, 3.0, 8.0], [8.0, 5.0, 8.0], [7.0, 5.0, 4.0]]], [[[4.0, 5.0, 4.0], [2.0, 3.0, 
2.0], [8.0, 4.0, 8.0]], [[7.0, 1.0, 2.0], [6.0, 7.0, 1.0], [6.0, 3.0, 2.0]], [[2.0, 3.0, 3.0], [8.0, 6.0, 2.0], [6.0, 4.0, 
6.0]]], [[[1.0, 8.0, 1.0], [3.0, 3.0, 6.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 1.0], [5.0, 6.0, 8.0], [1.0, 3.0, 5.0]], [[7.0, 7.0, 
8.0], [7.0, 3.0, 4.0], [5.0, 5.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 1.0, 5.0]], [[3.0, 5.0, 8.0]], [[6.0, 5.0, 5.0]]], [[[4.0, 2.0, 8.0]], [[2.0, 1.0, 2.0]], 
[[8.0, 7.0, 6.0]]]]),self.functionspace)
      arg[1:3,:,1:2]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 5.0], [5.0, 1.0, 3.0], [8.0, 8.0, 6.0]], [[1.0, 4.0, 6.0], [5.0, 7.0, 6.0], [6.0, 4.0, 
1.0]], [[3.0, 1.0, 8.0], [1.0, 3.0, 2.0], [2.0, 6.0, 8.0]]], [[[2.0, 6.0, 4.0], [6.0, 1.0, 5.0], [2.0, 4.0, 8.0]], [[4.0, 4.0, 
6.0], [3.0, 5.0, 8.0], [6.0, 4.0, 5.0]], [[2.0, 3.0, 8.0], [6.0, 5.0, 5.0], [7.0, 5.0, 4.0]]], [[[4.0, 5.0, 4.0], [4.0, 2.0, 
8.0], [8.0, 4.0, 8.0]], [[7.0, 1.0, 2.0], [2.0, 1.0, 2.0], [6.0, 3.0, 2.0]], [[2.0, 3.0, 3.0], [8.0, 7.0, 6.0], [6.0, 4.0, 
6.0]]], [[[1.0, 8.0, 1.0], [3.0, 3.0, 6.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 1.0], [5.0, 6.0, 8.0], [1.0, 3.0, 5.0]], [[7.0, 7.0, 
8.0], [7.0, 3.0, 4.0], [5.0, 5.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCRN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 3.0], [8.0, 8.0, 6.0], [3.0, 2.0, 1.0]], [[5.0, 4.0, 5.0], [1.0, 3.0, 2.0], [3.0, 6.0, 
6.0]], [[8.0, 8.0, 8.0], [8.0, 4.0, 2.0], [8.0, 6.0, 2.0]]], [[[1.0, 5.0, 4.0], [3.0, 2.0, 7.0], [6.0, 3.0, 8.0]], [[7.0, 7.0, 
4.0], [5.0, 6.0, 1.0], [1.0, 5.0, 8.0]], [[2.0, 2.0, 6.0], [8.0, 1.0, 2.0], [7.0, 4.0, 1.0]]], [[[2.0, 5.0, 4.0], [1.0, 8.0, 
1.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 7.0, 6.0], [7.0, 6.0, 8.0]], [[5.0, 6.0, 1.0], [5.0, 4.0, 5.0], [8.0, 3.0, 
2.0]]], [[[4.0, 3.0, 2.0], [6.0, 3.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 3.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 
4.0], [3.0, 2.0, 4.0], [1.0, 6.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 6.0, 2.0]], [[6.0, 5.0, 2.0]], [[3.0, 3.0, 3.0]]], [[[6.0, 6.0, 6.0]], [[2.0, 8.0, 
5.0]], [[2.0, 6.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 8.0, 2.0]], [[7.0, 3.0, 4.0]], [[6.0, 4.0, 1.0]]], [[[4.0, 8.0, 
7.0]], [[3.0, 8.0, 7.0]], [[5.0, 5.0, 3.0]]]])
      arg[1:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 3.0], [8.0, 8.0, 6.0], [3.0, 2.0, 1.0]], [[5.0, 4.0, 5.0], [1.0, 3.0, 2.0], [3.0, 
6.0, 6.0]], [[8.0, 8.0, 8.0], [8.0, 4.0, 2.0], [8.0, 6.0, 2.0]]], [[[1.0, 5.0, 4.0], [7.0, 6.0, 2.0], [6.0, 3.0, 8.0]], [[7.0, 
7.0, 4.0], [6.0, 5.0, 2.0], [1.0, 5.0, 8.0]], [[2.0, 2.0, 6.0], [3.0, 3.0, 3.0], [7.0, 4.0, 1.0]]], [[[2.0, 5.0, 4.0], [6.0, 
6.0, 6.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [2.0, 8.0, 5.0], [7.0, 6.0, 8.0]], [[5.0, 6.0, 1.0], [2.0, 6.0, 1.0], [8.0, 3.0, 
2.0]]], [[[4.0, 3.0, 2.0], [6.0, 3.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 3.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 
4.0], [3.0, 2.0, 4.0], [1.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 3.0], [8.0, 8.0, 6.0], [3.0, 2.0, 1.0]], 
[[5.0, 4.0, 5.0], [1.0, 3.0, 2.0], [3.0, 6.0, 6.0]], [[8.0, 8.0, 8.0], [8.0, 4.0, 2.0], [8.0, 6.0, 2.0]]], [[[1.0, 5.0, 4.0], 
[6.0, 8.0, 2.0], [6.0, 3.0, 8.0]], [[7.0, 7.0, 4.0], [7.0, 3.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 2.0, 6.0], [6.0, 4.0, 1.0], 
[7.0, 4.0, 1.0]]], [[[2.0, 5.0, 4.0], [4.0, 8.0, 7.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [3.0, 8.0, 7.0], [7.0, 6.0, 8.0]], 
[[5.0, 6.0, 1.0], [5.0, 5.0, 3.0], [8.0, 3.0, 2.0]]], [[[4.0, 3.0, 2.0], [6.0, 3.0, 3.0], [4.0, 2.0, 7.0]], [[6.0, 4.0, 1.0], 
[5.0, 7.0, 3.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 4.0], [3.0, 2.0, 4.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LUPN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 7.0], [6.0, 8.0, 1.0], [7.0, 2.0, 3.0]], [[6.0, 1.0, 4.0], [3.0, 5.0, 7.0], [2.0, 5.0, 
2.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 7.0], [3.0, 6.0, 4.0]]], [[[7.0, 4.0, 8.0], [5.0, 5.0, 8.0], [7.0, 5.0, 5.0]], [[8.0, 7.0, 
2.0], [4.0, 1.0, 3.0], [5.0, 4.0, 7.0]], [[8.0, 4.0, 4.0], [5.0, 7.0, 5.0], [2.0, 6.0, 6.0]]], [[[3.0, 3.0, 4.0], [6.0, 1.0, 
3.0], [1.0, 7.0, 3.0]], [[1.0, 7.0, 1.0], [1.0, 2.0, 2.0], [5.0, 7.0, 2.0]], [[8.0, 2.0, 2.0], [8.0, 2.0, 4.0], [8.0, 4.0, 
1.0]]], [[[1.0, 8.0, 6.0], [5.0, 8.0, 8.0], [5.0, 8.0, 8.0]], [[5.0, 2.0, 8.0], [4.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[8.0, 6.0, 
3.0], [7.0, 4.0, 8.0], [3.0, 7.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[8.0, 2.0, 6.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 4.0], [3.0, 1.0, 8.0]]])
      arg[2:,:2,1]=val
      ref=Data(numpy.array([[[[6.0, 4.0, 7.0], [6.0, 8.0, 1.0], [7.0, 2.0, 3.0]], [[6.0, 1.0, 4.0], [3.0, 5.0, 7.0], [2.0, 5.0, 
2.0]], [[4.0, 5.0, 7.0], [5.0, 7.0, 7.0], [3.0, 6.0, 4.0]]], [[[7.0, 4.0, 8.0], [5.0, 5.0, 8.0], [7.0, 5.0, 5.0]], [[8.0, 7.0, 
2.0], [4.0, 1.0, 3.0], [5.0, 4.0, 7.0]], [[8.0, 4.0, 4.0], [5.0, 7.0, 5.0], [2.0, 6.0, 6.0]]], [[[3.0, 3.0, 4.0], [8.0, 2.0, 
6.0], [1.0, 7.0, 3.0]], [[1.0, 7.0, 1.0], [6.0, 2.0, 7.0], [5.0, 7.0, 2.0]], [[8.0, 2.0, 2.0], [8.0, 2.0, 4.0], [8.0, 4.0, 
1.0]]], [[[1.0, 8.0, 6.0], [3.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[5.0, 2.0, 8.0], [3.0, 1.0, 8.0], [3.0, 3.0, 7.0]], [[8.0, 6.0, 
3.0], [7.0, 4.0, 8.0], [3.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LUPN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 3.0], [7.0, 3.0, 6.0], [4.0, 6.0, 1.0]], [[7.0, 3.0, 5.0], [2.0, 7.0, 3.0], [3.0, 2.0, 
8.0]], [[6.0, 8.0, 6.0], [5.0, 3.0, 4.0], [5.0, 5.0, 1.0]]], [[[2.0, 1.0, 7.0], [8.0, 5.0, 7.0], [6.0, 7.0, 5.0]], [[2.0, 4.0, 
4.0], [8.0, 8.0, 5.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 1.0], [5.0, 1.0, 5.0]]], [[[6.0, 4.0, 8.0], [4.0, 4.0, 
7.0], [4.0, 5.0, 4.0]], [[5.0, 8.0, 7.0], [3.0, 8.0, 7.0], [2.0, 8.0, 7.0]], [[6.0, 8.0, 5.0], [5.0, 1.0, 8.0], [4.0, 6.0, 
4.0]]], [[[4.0, 5.0, 2.0], [4.0, 2.0, 1.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 4.0], [5.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 
2.0], [7.0, 5.0, 4.0], [1.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 1.0, 4.0], [4.0, 3.0, 3.0]], [[1.0, 8.0, 4.0], [7.0, 4.0, 5.0]]]),self.functionspace)
      arg[2:,:2,1]=val
      ref=Data(numpy.array([[[[8.0, 6.0, 3.0], [7.0, 3.0, 6.0], [4.0, 6.0, 1.0]], [[7.0, 3.0, 5.0], [2.0, 7.0, 3.0], [3.0, 2.0, 
8.0]], [[6.0, 8.0, 6.0], [5.0, 3.0, 4.0], [5.0, 5.0, 1.0]]], [[[2.0, 1.0, 7.0], [8.0, 5.0, 7.0], [6.0, 7.0, 5.0]], [[2.0, 4.0, 
4.0], [8.0, 8.0, 5.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 1.0], [5.0, 1.0, 5.0]]], [[[6.0, 4.0, 8.0], [4.0, 1.0, 
4.0], [4.0, 5.0, 4.0]], [[5.0, 8.0, 7.0], [4.0, 3.0, 3.0], [2.0, 8.0, 7.0]], [[6.0, 8.0, 5.0], [5.0, 1.0, 8.0], [4.0, 6.0, 
4.0]]], [[[4.0, 5.0, 2.0], [1.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 4.0], [7.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 
2.0], [7.0, 5.0, 4.0], [1.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LUPN(self):
      arg=Data(numpy.array([[[[8.0, 1.0, 3.0], [2.0, 5.0, 2.0], [3.0, 6.0, 5.0]], [[5.0, 5.0, 3.0], [5.0, 6.0, 1.0], [3.0, 5.0, 
2.0]], [[8.0, 3.0, 7.0], [2.0, 6.0, 5.0], [3.0, 7.0, 4.0]]], [[[4.0, 3.0, 8.0], [2.0, 1.0, 5.0], [6.0, 5.0, 4.0]], [[5.0, 7.0, 
8.0], [1.0, 7.0, 3.0], [1.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 1.0, 5.0], [3.0, 8.0, 3.0]]], [[[4.0, 4.0, 3.0], [3.0, 8.0, 
7.0], [6.0, 1.0, 5.0]], [[1.0, 4.0, 7.0], [6.0, 5.0, 4.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 3.0], [4.0, 3.0, 4.0], [7.0, 6.0, 
4.0]]], [[[4.0, 5.0, 4.0], [4.0, 1.0, 5.0], [8.0, 3.0, 2.0]], [[7.0, 3.0, 2.0], [8.0, 1.0, 3.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 
2.0], [3.0, 8.0, 8.0], [5.0, 4.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 1.0, 2.0], [5.0, 6.0, 8.0]], [[2.0, 1.0, 4.0], [3.0, 6.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[1.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[5.0, 1.0, 3.0], [1.0, 4.0, 3.0]]])
      arg[2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 3.0], [2.0, 5.0, 2.0], [3.0, 6.0, 5.0]], [[5.0, 5.0, 3.0], [5.0, 6.0, 1.0], [3.0, 
5.0, 2.0]], [[8.0, 3.0, 7.0], [2.0, 6.0, 5.0], [3.0, 7.0, 4.0]]], [[[4.0, 3.0, 8.0], [2.0, 1.0, 5.0], [6.0, 5.0, 4.0]], [[5.0, 
7.0, 8.0], [1.0, 7.0, 3.0], [1.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 1.0, 5.0], [3.0, 8.0, 3.0]]], [[[4.0, 4.0, 3.0], [6.0, 
1.0, 2.0], [6.0, 1.0, 5.0]], [[1.0, 4.0, 7.0], [5.0, 6.0, 8.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 3.0], [4.0, 3.0, 4.0], [7.0, 6.0, 
4.0]]], [[[4.0, 5.0, 4.0], [2.0, 1.0, 4.0], [8.0, 3.0, 2.0]], [[7.0, 3.0, 2.0], [3.0, 6.0, 1.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 
2.0], [3.0, 8.0, 8.0], [5.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 3.0], [2.0, 5.0, 2.0], [3.0, 6.0, 5.0]], 
[[5.0, 5.0, 3.0], [5.0, 6.0, 1.0], [3.0, 5.0, 2.0]], [[8.0, 3.0, 7.0], [2.0, 6.0, 5.0], [3.0, 7.0, 4.0]]], [[[4.0, 3.0, 8.0], 
[2.0, 1.0, 5.0], [6.0, 5.0, 4.0]], [[5.0, 7.0, 8.0], [1.0, 7.0, 3.0], [1.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [4.0, 1.0, 5.0], 
[3.0, 8.0, 3.0]]], [[[4.0, 4.0, 3.0], [1.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 4.0, 7.0], [2.0, 4.0, 2.0], [6.0, 4.0, 3.0]], 
[[8.0, 1.0, 3.0], [4.0, 3.0, 4.0], [7.0, 6.0, 4.0]]], [[[4.0, 5.0, 4.0], [5.0, 1.0, 3.0], [8.0, 3.0, 2.0]], [[7.0, 3.0, 2.0], 
[1.0, 4.0, 3.0], [2.0, 2.0, 2.0]], [[1.0, 2.0, 2.0], [3.0, 8.0, 8.0], [5.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CRPN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 6.0], [5.0, 4.0, 3.0], [1.0, 4.0, 3.0]], [[7.0, 4.0, 1.0], [7.0, 3.0, 8.0], [7.0, 6.0, 
3.0]], [[6.0, 5.0, 1.0], [6.0, 7.0, 8.0], [3.0, 3.0, 8.0]]], [[[1.0, 3.0, 2.0], [5.0, 3.0, 8.0], [4.0, 6.0, 6.0]], [[2.0, 1.0, 
2.0], [6.0, 5.0, 3.0], [4.0, 4.0, 5.0]], [[8.0, 8.0, 8.0], [5.0, 5.0, 2.0], [4.0, 1.0, 6.0]]], [[[1.0, 3.0, 6.0], [7.0, 4.0, 
3.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 5.0], [2.0, 2.0, 2.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 5.0], [7.0, 8.0, 2.0], [6.0, 7.0, 
2.0]]], [[[4.0, 5.0, 6.0], [7.0, 4.0, 3.0], [8.0, 7.0, 3.0]], [[7.0, 4.0, 7.0], [3.0, 4.0, 8.0], [7.0, 4.0, 2.0]], [[2.0, 7.0, 
8.0], [1.0, 3.0, 1.0], [4.0, 6.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 1.0, 6.0]], [[3.0, 5.0, 8.0]], [[8.0, 3.0, 1.0]], [[2.0, 3.0, 7.0]]])
      arg[:,1:2,1]=val
      ref=Data(numpy.array([[[[7.0, 3.0, 6.0], [5.0, 4.0, 3.0], [1.0, 4.0, 3.0]], [[7.0, 4.0, 1.0], [1.0, 1.0, 6.0], [7.0, 6.0, 
3.0]], [[6.0, 5.0, 1.0], [6.0, 7.0, 8.0], [3.0, 3.0, 8.0]]], [[[1.0, 3.0, 2.0], [5.0, 3.0, 8.0], [4.0, 6.0, 6.0]], [[2.0, 1.0, 
2.0], [3.0, 5.0, 8.0], [4.0, 4.0, 5.0]], [[8.0, 8.0, 8.0], [5.0, 5.0, 2.0], [4.0, 1.0, 6.0]]], [[[1.0, 3.0, 6.0], [7.0, 4.0, 
3.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 5.0], [8.0, 3.0, 1.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 5.0], [7.0, 8.0, 2.0], [6.0, 7.0, 
2.0]]], [[[4.0, 5.0, 6.0], [7.0, 4.0, 3.0], [8.0, 7.0, 3.0]], [[7.0, 4.0, 7.0], [2.0, 3.0, 7.0], [7.0, 4.0, 2.0]], [[2.0, 7.0, 
8.0], [1.0, 3.0, 1.0], [4.0, 6.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CRPN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 2.0], [8.0, 2.0, 2.0], [4.0, 4.0, 1.0]], [[3.0, 7.0, 8.0], [3.0, 6.0, 5.0], [5.0, 1.0, 
3.0]], [[3.0, 4.0, 4.0], [5.0, 1.0, 2.0], [2.0, 5.0, 6.0]]], [[[3.0, 6.0, 7.0], [7.0, 5.0, 2.0], [8.0, 1.0, 5.0]], [[2.0, 4.0, 
3.0], [2.0, 1.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 3.0, 2.0], [1.0, 5.0, 3.0], [1.0, 4.0, 5.0]]], [[[4.0, 8.0, 2.0], [6.0, 2.0, 
5.0], [8.0, 7.0, 1.0]], [[3.0, 6.0, 8.0], [3.0, 7.0, 2.0], [2.0, 4.0, 8.0]], [[4.0, 2.0, 3.0], [4.0, 6.0, 5.0], [3.0, 6.0, 
6.0]]], [[[4.0, 7.0, 3.0], [8.0, 5.0, 8.0], [5.0, 2.0, 4.0]], [[1.0, 5.0, 5.0], [6.0, 1.0, 2.0], [1.0, 8.0, 1.0]], [[4.0, 8.0, 
5.0], [7.0, 4.0, 5.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 6.0, 1.0]], [[1.0, 3.0, 4.0]], [[2.0, 7.0, 3.0]], [[2.0, 6.0, 4.0]]]),self.functionspace)
      arg[:,1:2,1]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 2.0], [8.0, 2.0, 2.0], [4.0, 4.0, 1.0]], [[3.0, 7.0, 8.0], [2.0, 6.0, 1.0], [5.0, 1.0, 
3.0]], [[3.0, 4.0, 4.0], [5.0, 1.0, 2.0], [2.0, 5.0, 6.0]]], [[[3.0, 6.0, 7.0], [7.0, 5.0, 2.0], [8.0, 1.0, 5.0]], [[2.0, 4.0, 
3.0], [1.0, 3.0, 4.0], [7.0, 6.0, 2.0]], [[6.0, 3.0, 2.0], [1.0, 5.0, 3.0], [1.0, 4.0, 5.0]]], [[[4.0, 8.0, 2.0], [6.0, 2.0, 
5.0], [8.0, 7.0, 1.0]], [[3.0, 6.0, 8.0], [2.0, 7.0, 3.0], [2.0, 4.0, 8.0]], [[4.0, 2.0, 3.0], [4.0, 6.0, 5.0], [3.0, 6.0, 
6.0]]], [[[4.0, 7.0, 3.0], [8.0, 5.0, 8.0], [5.0, 2.0, 4.0]], [[1.0, 5.0, 5.0], [2.0, 6.0, 4.0], [1.0, 8.0, 1.0]], [[4.0, 8.0, 
5.0], [7.0, 4.0, 5.0], [8.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CRPN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 5.0], [7.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 6.0, 8.0], [8.0, 2.0, 
4.0]], [[6.0, 7.0, 1.0], [1.0, 8.0, 3.0], [1.0, 8.0, 4.0]]], [[[3.0, 8.0, 2.0], [5.0, 3.0, 4.0], [7.0, 1.0, 2.0]], [[5.0, 3.0, 
4.0], [8.0, 4.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 1.0, 1.0], [3.0, 3.0, 6.0], [3.0, 4.0, 8.0]]], [[[6.0, 3.0, 3.0], [5.0, 3.0, 
5.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 2.0], [7.0, 6.0, 5.0], [7.0, 7.0, 8.0]], [[3.0, 7.0, 7.0], [3.0, 7.0, 4.0], [6.0, 1.0, 
3.0]]], [[[6.0, 2.0, 4.0], [7.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[8.0, 6.0, 3.0], [7.0, 7.0, 6.0], [1.0, 1.0, 8.0]], [[6.0, 3.0, 
2.0], [2.0, 8.0, 8.0], [5.0, 4.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 2.0, 1.0]], [[6.0, 5.0, 8.0]], [[1.0, 8.0, 8.0]], [[8.0, 3.0, 
3.0]]])+(1.-msk_val)*numpy.array([[[3.0, 4.0, 7.0]], [[6.0, 8.0, 6.0]], [[6.0, 1.0, 4.0]], [[4.0, 5.0, 5.0]]])
      arg[:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 5.0], [7.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 2.0, 2.0], [1.0, 2.0, 1.0], [8.0, 
2.0, 4.0]], [[6.0, 7.0, 1.0], [1.0, 8.0, 3.0], [1.0, 8.0, 4.0]]], [[[3.0, 8.0, 2.0], [5.0, 3.0, 4.0], [7.0, 1.0, 2.0]], [[5.0, 
3.0, 4.0], [6.0, 5.0, 8.0], [6.0, 2.0, 7.0]], [[5.0, 1.0, 1.0], [3.0, 3.0, 6.0], [3.0, 4.0, 8.0]]], [[[6.0, 3.0, 3.0], [5.0, 
3.0, 5.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 2.0], [1.0, 8.0, 8.0], [7.0, 7.0, 8.0]], [[3.0, 7.0, 7.0], [3.0, 7.0, 4.0], [6.0, 1.0, 
3.0]]], [[[6.0, 2.0, 4.0], [7.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[8.0, 6.0, 3.0], [8.0, 3.0, 3.0], [1.0, 1.0, 8.0]], [[6.0, 3.0, 
2.0], [2.0, 8.0, 8.0], [5.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [7.0, 7.0, 5.0], [2.0, 7.0, 1.0]], 
[[4.0, 2.0, 2.0], [3.0, 4.0, 7.0], [8.0, 2.0, 4.0]], [[6.0, 7.0, 1.0], [1.0, 8.0, 3.0], [1.0, 8.0, 4.0]]], [[[3.0, 8.0, 2.0], 
[5.0, 3.0, 4.0], [7.0, 1.0, 2.0]], [[5.0, 3.0, 4.0], [6.0, 8.0, 6.0], [6.0, 2.0, 7.0]], [[5.0, 1.0, 1.0], [3.0, 3.0, 6.0], 
[3.0, 4.0, 8.0]]], [[[6.0, 3.0, 3.0], [5.0, 3.0, 5.0], [5.0, 8.0, 7.0]], [[4.0, 1.0, 2.0], [6.0, 1.0, 4.0], [7.0, 7.0, 8.0]], 
[[3.0, 7.0, 7.0], [3.0, 7.0, 4.0], [6.0, 1.0, 3.0]]], [[[6.0, 2.0, 4.0], [7.0, 8.0, 8.0], [4.0, 5.0, 2.0]], [[8.0, 6.0, 3.0], 
[4.0, 5.0, 5.0], [1.0, 1.0, 8.0]], [[6.0, 3.0, 2.0], [2.0, 8.0, 8.0], [5.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [6.0, 5.0, 4.0], [6.0, 1.0, 7.0]], [[5.0, 5.0, 7.0], [7.0, 3.0, 3.0], [1.0, 6.0, 
8.0]], [[3.0, 3.0, 3.0], [2.0, 1.0, 4.0], [6.0, 8.0, 4.0]]], [[[4.0, 7.0, 1.0], [4.0, 1.0, 6.0], [5.0, 3.0, 5.0]], [[6.0, 3.0, 
1.0], [8.0, 2.0, 1.0], [5.0, 4.0, 1.0]], [[4.0, 3.0, 7.0], [4.0, 3.0, 4.0], [5.0, 6.0, 6.0]]], [[[1.0, 2.0, 7.0], [8.0, 4.0, 
2.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 4.0], [6.0, 8.0, 2.0], [5.0, 7.0, 6.0]], [[3.0, 4.0, 8.0], [7.0, 4.0, 7.0], [2.0, 7.0, 
8.0]]], [[[7.0, 3.0, 3.0], [5.0, 6.0, 7.0], [1.0, 2.0, 3.0]], [[1.0, 3.0, 4.0], [3.0, 1.0, 7.0], [1.0, 4.0, 1.0]], [[4.0, 1.0, 
1.0], [8.0, 5.0, 5.0], [2.0, 4.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 2.0, 6.0], [5.0, 4.0, 8.0]]], [[[4.0, 7.0, 1.0], [6.0, 8.0, 5.0]]], [[[7.0, 8.0, 4.0], [5.0, 
8.0, 8.0]]], [[[2.0, 6.0, 8.0], [6.0, 3.0, 2.0]]]])
      arg[:,2:,:2]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 3.0], [6.0, 5.0, 4.0], [6.0, 1.0, 7.0]], [[5.0, 5.0, 7.0], [7.0, 3.0, 3.0], [1.0, 6.0, 
8.0]], [[5.0, 2.0, 6.0], [5.0, 4.0, 8.0], [6.0, 8.0, 4.0]]], [[[4.0, 7.0, 1.0], [4.0, 1.0, 6.0], [5.0, 3.0, 5.0]], [[6.0, 3.0, 
1.0], [8.0, 2.0, 1.0], [5.0, 4.0, 1.0]], [[4.0, 7.0, 1.0], [6.0, 8.0, 5.0], [5.0, 6.0, 6.0]]], [[[1.0, 2.0, 7.0], [8.0, 4.0, 
2.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 4.0], [6.0, 8.0, 2.0], [5.0, 7.0, 6.0]], [[7.0, 8.0, 4.0], [5.0, 8.0, 8.0], [2.0, 7.0, 
8.0]]], [[[7.0, 3.0, 3.0], [5.0, 6.0, 7.0], [1.0, 2.0, 3.0]], [[1.0, 3.0, 4.0], [3.0, 1.0, 7.0], [1.0, 4.0, 1.0]], [[2.0, 6.0, 
8.0], [6.0, 3.0, 2.0], [2.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLUN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 2.0], [6.0, 5.0, 7.0], [4.0, 8.0, 6.0]], [[1.0, 6.0, 1.0], [7.0, 1.0, 3.0], [8.0, 1.0, 
1.0]], [[6.0, 7.0, 8.0], [5.0, 2.0, 3.0], [1.0, 5.0, 8.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 7.0], [2.0, 7.0, 3.0]], [[1.0, 6.0, 
5.0], [3.0, 8.0, 4.0], [4.0, 3.0, 7.0]], [[7.0, 1.0, 5.0], [7.0, 1.0, 7.0], [6.0, 8.0, 5.0]]], [[[7.0, 6.0, 2.0], [3.0, 2.0, 
4.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 3.0], [5.0, 3.0, 3.0], [7.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], [5.0, 8.0, 8.0], [8.0, 6.0, 
4.0]]], [[[1.0, 1.0, 7.0], [4.0, 3.0, 4.0], [5.0, 6.0, 4.0]], [[2.0, 5.0, 4.0], [4.0, 1.0, 1.0], [1.0, 6.0, 5.0]], [[6.0, 6.0, 
6.0], [3.0, 7.0, 1.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0, 6.0], [2.0, 6.0, 4.0]]], [[[8.0, 1.0, 7.0], [2.0, 7.0, 1.0]]], [[[8.0, 5.0, 7.0], 
[4.0, 1.0, 8.0]]], [[[5.0, 6.0, 3.0], [4.0, 1.0, 2.0]]]]),self.functionspace)
      arg[:,2:,:2]=val
      ref=Data(numpy.array([[[[6.0, 4.0, 2.0], [6.0, 5.0, 7.0], [4.0, 8.0, 6.0]], [[1.0, 6.0, 1.0], [7.0, 1.0, 3.0], [8.0, 1.0, 
1.0]], [[5.0, 1.0, 6.0], [2.0, 6.0, 4.0], [1.0, 5.0, 8.0]]], [[[2.0, 5.0, 5.0], [5.0, 2.0, 7.0], [2.0, 7.0, 3.0]], [[1.0, 6.0, 
5.0], [3.0, 8.0, 4.0], [4.0, 3.0, 7.0]], [[8.0, 1.0, 7.0], [2.0, 7.0, 1.0], [6.0, 8.0, 5.0]]], [[[7.0, 6.0, 2.0], [3.0, 2.0, 
4.0], [5.0, 8.0, 7.0]], [[3.0, 8.0, 3.0], [5.0, 3.0, 3.0], [7.0, 4.0, 8.0]], [[8.0, 5.0, 7.0], [4.0, 1.0, 8.0], [8.0, 6.0, 
4.0]]], [[[1.0, 1.0, 7.0], [4.0, 3.0, 4.0], [5.0, 6.0, 4.0]], [[2.0, 5.0, 4.0], [4.0, 1.0, 1.0], [1.0, 6.0, 5.0]], [[5.0, 6.0, 
3.0], [4.0, 1.0, 2.0], [5.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLUN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 2.0], [5.0, 3.0, 1.0], [8.0, 5.0, 4.0]], [[3.0, 1.0, 6.0], [3.0, 8.0, 4.0], [6.0, 4.0, 
3.0]], [[4.0, 7.0, 1.0], [1.0, 7.0, 7.0], [2.0, 5.0, 1.0]]], [[[6.0, 6.0, 1.0], [3.0, 5.0, 7.0], [7.0, 2.0, 4.0]], [[5.0, 7.0, 
3.0], [7.0, 2.0, 3.0], [1.0, 4.0, 1.0]], [[2.0, 2.0, 6.0], [7.0, 2.0, 6.0], [3.0, 1.0, 1.0]]], [[[1.0, 8.0, 7.0], [2.0, 6.0, 
1.0], [5.0, 2.0, 7.0]], [[1.0, 3.0, 3.0], [5.0, 5.0, 8.0], [4.0, 6.0, 4.0]], [[4.0, 6.0, 6.0], [8.0, 2.0, 3.0], [2.0, 5.0, 
2.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 6.0], [8.0, 4.0, 6.0]], [[1.0, 1.0, 6.0], [2.0, 7.0, 8.0], [6.0, 4.0, 3.0]], [[8.0, 3.0, 
4.0], [7.0, 7.0, 5.0], [1.0, 4.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 6.0, 4.0], [3.0, 2.0, 7.0]]], [[[3.0, 3.0, 2.0], [1.0, 7.0, 2.0]]], [[[5.0, 2.0, 2.0], 
[6.0, 7.0, 8.0]]], [[[6.0, 6.0, 6.0], [1.0, 5.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 7.0, 4.0], [2.0, 3.0, 2.0]]], 
[[[7.0, 4.0, 8.0], [4.0, 5.0, 5.0]]], [[[8.0, 8.0, 2.0], [3.0, 5.0, 6.0]]], [[[5.0, 2.0, 8.0], [1.0, 6.0, 8.0]]]])
      arg[:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 2.0], [5.0, 3.0, 1.0], [8.0, 5.0, 4.0]], [[3.0, 1.0, 6.0], [3.0, 8.0, 4.0], [6.0, 
4.0, 3.0]], [[2.0, 6.0, 4.0], [3.0, 2.0, 7.0], [2.0, 5.0, 1.0]]], [[[6.0, 6.0, 1.0], [3.0, 5.0, 7.0], [7.0, 2.0, 4.0]], [[5.0, 
7.0, 3.0], [7.0, 2.0, 3.0], [1.0, 4.0, 1.0]], [[3.0, 3.0, 2.0], [1.0, 7.0, 2.0], [3.0, 1.0, 1.0]]], [[[1.0, 8.0, 7.0], [2.0, 
6.0, 1.0], [5.0, 2.0, 7.0]], [[1.0, 3.0, 3.0], [5.0, 5.0, 8.0], [4.0, 6.0, 4.0]], [[5.0, 2.0, 2.0], [6.0, 7.0, 8.0], [2.0, 5.0, 
2.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 6.0], [8.0, 4.0, 6.0]], [[1.0, 1.0, 6.0], [2.0, 7.0, 8.0], [6.0, 4.0, 3.0]], [[6.0, 6.0, 
6.0], [1.0, 5.0, 8.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 2.0], [5.0, 3.0, 1.0], [8.0, 5.0, 4.0]], 
[[3.0, 1.0, 6.0], [3.0, 8.0, 4.0], [6.0, 4.0, 3.0]], [[3.0, 7.0, 4.0], [2.0, 3.0, 2.0], [2.0, 5.0, 1.0]]], [[[6.0, 6.0, 1.0], 
[3.0, 5.0, 7.0], [7.0, 2.0, 4.0]], [[5.0, 7.0, 3.0], [7.0, 2.0, 3.0], [1.0, 4.0, 1.0]], [[7.0, 4.0, 8.0], [4.0, 5.0, 5.0], 
[3.0, 1.0, 1.0]]], [[[1.0, 8.0, 7.0], [2.0, 6.0, 1.0], [5.0, 2.0, 7.0]], [[1.0, 3.0, 3.0], [5.0, 5.0, 8.0], [4.0, 6.0, 4.0]], 
[[8.0, 8.0, 2.0], [3.0, 5.0, 6.0], [2.0, 5.0, 2.0]]], [[[5.0, 8.0, 8.0], [6.0, 8.0, 6.0], [8.0, 4.0, 6.0]], [[1.0, 1.0, 6.0], 
[2.0, 7.0, 8.0], [6.0, 4.0, 3.0]], [[5.0, 2.0, 8.0], [1.0, 6.0, 8.0], [1.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PRCN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 7.0], [6.0, 3.0, 7.0], [2.0, 5.0, 2.0]], [[2.0, 6.0, 6.0], [2.0, 5.0, 7.0], [8.0, 7.0, 
5.0]], [[8.0, 1.0, 6.0], [4.0, 1.0, 2.0], [8.0, 1.0, 1.0]]], [[[4.0, 4.0, 7.0], [4.0, 3.0, 5.0], [1.0, 3.0, 2.0]], [[8.0, 1.0, 
1.0], [1.0, 8.0, 6.0], [3.0, 3.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 7.0, 5.0], [3.0, 6.0, 2.0]]], [[[7.0, 5.0, 2.0], [5.0, 8.0, 
8.0], [1.0, 4.0, 2.0]], [[6.0, 5.0, 5.0], [4.0, 6.0, 1.0], [4.0, 3.0, 7.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 6.0], [6.0, 2.0, 
6.0]]], [[[1.0, 5.0, 6.0], [6.0, 4.0, 6.0], [8.0, 6.0, 8.0]], [[8.0, 6.0, 2.0], [3.0, 3.0, 8.0], [8.0, 2.0, 7.0]], [[2.0, 6.0, 
7.0], [3.0, 7.0, 3.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      val=numpy.array([[[2.0, 8.0, 4.0], [4.0, 6.0, 2.0], [1.0, 4.0, 6.0]]])
      arg[2,1:2,:]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 7.0], [6.0, 3.0, 7.0], [2.0, 5.0, 2.0]], [[2.0, 6.0, 6.0], [2.0, 5.0, 7.0], [8.0, 7.0, 
5.0]], [[8.0, 1.0, 6.0], [4.0, 1.0, 2.0], [8.0, 1.0, 1.0]]], [[[4.0, 4.0, 7.0], [4.0, 3.0, 5.0], [1.0, 3.0, 2.0]], [[8.0, 1.0, 
1.0], [1.0, 8.0, 6.0], [3.0, 3.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 7.0, 5.0], [3.0, 6.0, 2.0]]], [[[7.0, 5.0, 2.0], [5.0, 8.0, 
8.0], [1.0, 4.0, 2.0]], [[2.0, 8.0, 4.0], [4.0, 6.0, 2.0], [1.0, 4.0, 6.0]], [[2.0, 4.0, 3.0], [5.0, 6.0, 6.0], [6.0, 2.0, 
6.0]]], [[[1.0, 5.0, 6.0], [6.0, 4.0, 6.0], [8.0, 6.0, 8.0]], [[8.0, 6.0, 2.0], [3.0, 3.0, 8.0], [8.0, 2.0, 7.0]], [[2.0, 6.0, 
7.0], [3.0, 7.0, 3.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PRCN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 7.0], [1.0, 8.0, 6.0], [2.0, 4.0, 2.0]], [[6.0, 2.0, 1.0], [2.0, 7.0, 6.0], [5.0, 6.0, 
2.0]], [[4.0, 4.0, 4.0], [4.0, 6.0, 8.0], [4.0, 8.0, 6.0]]], [[[6.0, 6.0, 7.0], [4.0, 8.0, 2.0], [2.0, 6.0, 3.0]], [[7.0, 3.0, 
1.0], [7.0, 1.0, 2.0], [3.0, 7.0, 1.0]], [[3.0, 1.0, 7.0], [6.0, 7.0, 5.0], [2.0, 5.0, 3.0]]], [[[5.0, 7.0, 4.0], [8.0, 5.0, 
2.0], [2.0, 7.0, 4.0]], [[2.0, 4.0, 3.0], [2.0, 2.0, 6.0], [6.0, 1.0, 8.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 8.0], [8.0, 2.0, 
2.0]]], [[[5.0, 4.0, 6.0], [2.0, 1.0, 4.0], [7.0, 2.0, 2.0]], [[7.0, 5.0, 5.0], [4.0, 2.0, 8.0], [7.0, 7.0, 7.0]], [[8.0, 3.0, 
7.0], [5.0, 4.0, 1.0], [6.0, 8.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 4.0, 3.0], [2.0, 8.0, 6.0], [6.0, 1.0, 1.0]]]),self.functionspace)
      arg[2,1:2,:]=val
      ref=Data(numpy.array([[[[3.0, 1.0, 7.0], [1.0, 8.0, 6.0], [2.0, 4.0, 2.0]], [[6.0, 2.0, 1.0], [2.0, 7.0, 6.0], [5.0, 6.0, 
2.0]], [[4.0, 4.0, 4.0], [4.0, 6.0, 8.0], [4.0, 8.0, 6.0]]], [[[6.0, 6.0, 7.0], [4.0, 8.0, 2.0], [2.0, 6.0, 3.0]], [[7.0, 3.0, 
1.0], [7.0, 1.0, 2.0], [3.0, 7.0, 1.0]], [[3.0, 1.0, 7.0], [6.0, 7.0, 5.0], [2.0, 5.0, 3.0]]], [[[5.0, 7.0, 4.0], [8.0, 5.0, 
2.0], [2.0, 7.0, 4.0]], [[5.0, 4.0, 3.0], [2.0, 8.0, 6.0], [6.0, 1.0, 1.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 8.0], [8.0, 2.0, 
2.0]]], [[[5.0, 4.0, 6.0], [2.0, 1.0, 4.0], [7.0, 2.0, 2.0]], [[7.0, 5.0, 5.0], [4.0, 2.0, 8.0], [7.0, 7.0, 7.0]], [[8.0, 3.0, 
7.0], [5.0, 4.0, 1.0], [6.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PRCN(self):
      arg=Data(numpy.array([[[[1.0, 7.0, 2.0], [5.0, 5.0, 3.0], [3.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 1.0, 6.0], [3.0, 6.0, 
5.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 4.0], [5.0, 1.0, 1.0]]], [[[5.0, 8.0, 7.0], [2.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[6.0, 8.0, 
4.0], [4.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[3.0, 5.0, 1.0], [3.0, 6.0, 3.0], [2.0, 3.0, 3.0]]], [[[1.0, 1.0, 4.0], [4.0, 8.0, 
3.0], [7.0, 4.0, 4.0]], [[1.0, 5.0, 5.0], [6.0, 2.0, 1.0], [2.0, 1.0, 1.0]], [[3.0, 2.0, 6.0], [5.0, 6.0, 6.0], [4.0, 3.0, 
1.0]]], [[[4.0, 7.0, 6.0], [7.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[1.0, 2.0, 7.0], [3.0, 4.0, 4.0], [7.0, 4.0, 1.0]], [[5.0, 6.0, 
3.0], [1.0, 6.0, 1.0], [1.0, 6.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 1.0, 5.0], [1.0, 5.0, 6.0], [2.0, 3.0, 2.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 
3.0], [4.0, 8.0, 1.0], [6.0, 3.0, 3.0]]])
      arg[2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 2.0], [5.0, 5.0, 3.0], [3.0, 1.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 1.0, 6.0], [3.0, 
6.0, 5.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 4.0], [5.0, 1.0, 1.0]]], [[[5.0, 8.0, 7.0], [2.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[6.0, 
8.0, 4.0], [4.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[3.0, 5.0, 1.0], [3.0, 6.0, 3.0], [2.0, 3.0, 3.0]]], [[[1.0, 1.0, 4.0], [4.0, 
8.0, 3.0], [7.0, 4.0, 4.0]], [[7.0, 1.0, 5.0], [1.0, 5.0, 6.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 6.0], [5.0, 6.0, 6.0], [4.0, 3.0, 
1.0]]], [[[4.0, 7.0, 6.0], [7.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[1.0, 2.0, 7.0], [3.0, 4.0, 4.0], [7.0, 4.0, 1.0]], [[5.0, 6.0, 
3.0], [1.0, 6.0, 1.0], [1.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 2.0], [5.0, 5.0, 3.0], [3.0, 1.0, 8.0]], 
[[6.0, 4.0, 2.0], [4.0, 1.0, 6.0], [3.0, 6.0, 5.0]], [[6.0, 4.0, 1.0], [5.0, 7.0, 4.0], [5.0, 1.0, 1.0]]], [[[5.0, 8.0, 7.0], 
[2.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[6.0, 8.0, 4.0], [4.0, 6.0, 7.0], [1.0, 5.0, 4.0]], [[3.0, 5.0, 1.0], [3.0, 6.0, 3.0], 
[2.0, 3.0, 3.0]]], [[[1.0, 1.0, 4.0], [4.0, 8.0, 3.0], [7.0, 4.0, 4.0]], [[2.0, 7.0, 3.0], [4.0, 8.0, 1.0], [6.0, 3.0, 3.0]], 
[[3.0, 2.0, 6.0], [5.0, 6.0, 6.0], [4.0, 3.0, 1.0]]], [[[4.0, 7.0, 6.0], [7.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[1.0, 2.0, 7.0], 
[3.0, 4.0, 4.0], [7.0, 4.0, 1.0]], [[5.0, 6.0, 3.0], [1.0, 6.0, 1.0], [1.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPLN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 7.0], [6.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], [1.0, 1.0, 5.0], [1.0, 7.0, 
3.0]], [[5.0, 7.0, 1.0], [6.0, 8.0, 1.0], [6.0, 3.0, 5.0]]], [[[1.0, 3.0, 8.0], [4.0, 3.0, 1.0], [8.0, 6.0, 4.0]], [[2.0, 8.0, 
5.0], [5.0, 1.0, 2.0], [1.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], [1.0, 8.0, 2.0], [1.0, 7.0, 6.0]]], [[[4.0, 2.0, 2.0], [5.0, 5.0, 
1.0], [8.0, 8.0, 7.0]], [[8.0, 5.0, 5.0], [6.0, 7.0, 5.0], [3.0, 3.0, 7.0]], [[4.0, 2.0, 8.0], [6.0, 6.0, 2.0], [3.0, 5.0, 
6.0]]], [[[6.0, 5.0, 5.0], [1.0, 2.0, 5.0], [5.0, 4.0, 6.0]], [[2.0, 3.0, 3.0], [5.0, 1.0, 6.0], [4.0, 2.0, 5.0]], [[2.0, 6.0, 
2.0], [4.0, 3.0, 5.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 6.0, 6.0]], [[4.0, 4.0, 2.0]]])
      arg[2:,1,2:]=val
      ref=Data(numpy.array([[[[8.0, 8.0, 7.0], [6.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[6.0, 6.0, 2.0], [1.0, 1.0, 5.0], [1.0, 7.0, 
3.0]], [[5.0, 7.0, 1.0], [6.0, 8.0, 1.0], [6.0, 3.0, 5.0]]], [[[1.0, 3.0, 8.0], [4.0, 3.0, 1.0], [8.0, 6.0, 4.0]], [[2.0, 8.0, 
5.0], [5.0, 1.0, 2.0], [1.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], [1.0, 8.0, 2.0], [1.0, 7.0, 6.0]]], [[[4.0, 2.0, 2.0], [5.0, 5.0, 
1.0], [8.0, 8.0, 7.0]], [[8.0, 5.0, 5.0], [6.0, 7.0, 5.0], [7.0, 6.0, 6.0]], [[4.0, 2.0, 8.0], [6.0, 6.0, 2.0], [3.0, 5.0, 
6.0]]], [[[6.0, 5.0, 5.0], [1.0, 2.0, 5.0], [5.0, 4.0, 6.0]], [[2.0, 3.0, 3.0], [5.0, 1.0, 6.0], [4.0, 4.0, 2.0]], [[2.0, 6.0, 
2.0], [4.0, 3.0, 5.0], [3.0, 7.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPLN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 4.0], [8.0, 3.0, 5.0], [4.0, 8.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 4.0, 6.0], [1.0, 3.0, 
8.0]], [[5.0, 1.0, 3.0], [3.0, 7.0, 1.0], [4.0, 4.0, 2.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 
1.0], [5.0, 5.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], [8.0, 6.0, 2.0], [6.0, 1.0, 5.0]]], [[[4.0, 8.0, 5.0], [4.0, 7.0, 
6.0], [6.0, 3.0, 8.0]], [[2.0, 7.0, 4.0], [8.0, 6.0, 2.0], [2.0, 2.0, 4.0]], [[7.0, 8.0, 2.0], [3.0, 4.0, 3.0], [2.0, 7.0, 
2.0]]], [[[3.0, 4.0, 6.0], [8.0, 3.0, 2.0], [4.0, 3.0, 2.0]], [[5.0, 6.0, 6.0], [6.0, 7.0, 4.0], [5.0, 8.0, 6.0]], [[1.0, 3.0, 
5.0], [7.0, 7.0, 6.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 1.0]], [[7.0, 4.0, 8.0]]]),self.functionspace)
      arg[2:,1,2:]=val
      ref=Data(numpy.array([[[[4.0, 3.0, 4.0], [8.0, 3.0, 5.0], [4.0, 8.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 4.0, 6.0], [1.0, 3.0, 
8.0]], [[5.0, 1.0, 3.0], [3.0, 7.0, 1.0], [4.0, 4.0, 2.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 3.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 
1.0], [5.0, 5.0, 2.0], [6.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], [8.0, 6.0, 2.0], [6.0, 1.0, 5.0]]], [[[4.0, 8.0, 5.0], [4.0, 7.0, 
6.0], [6.0, 3.0, 8.0]], [[2.0, 7.0, 4.0], [8.0, 6.0, 2.0], [4.0, 7.0, 1.0]], [[7.0, 8.0, 2.0], [3.0, 4.0, 3.0], [2.0, 7.0, 
2.0]]], [[[3.0, 4.0, 6.0], [8.0, 3.0, 2.0], [4.0, 3.0, 2.0]], [[5.0, 6.0, 6.0], [6.0, 7.0, 4.0], [7.0, 4.0, 8.0]], [[1.0, 3.0, 
5.0], [7.0, 7.0, 6.0], [3.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPLN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 2.0], [4.0, 2.0, 6.0], [7.0, 8.0, 5.0]], [[5.0, 2.0, 2.0], [7.0, 5.0, 5.0], [7.0, 2.0, 
5.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 3.0], [1.0, 6.0, 4.0]]], [[[6.0, 6.0, 6.0], [5.0, 7.0, 6.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 
1.0], [1.0, 8.0, 1.0], [5.0, 6.0, 7.0]], [[8.0, 1.0, 5.0], [6.0, 2.0, 8.0], [8.0, 2.0, 7.0]]], [[[7.0, 4.0, 6.0], [6.0, 2.0, 
7.0], [2.0, 4.0, 8.0]], [[5.0, 5.0, 2.0], [4.0, 4.0, 6.0], [6.0, 2.0, 4.0]], [[2.0, 7.0, 2.0], [7.0, 3.0, 5.0], [6.0, 8.0, 
8.0]]], [[[7.0, 7.0, 1.0], [1.0, 1.0, 3.0], [6.0, 1.0, 4.0]], [[2.0, 6.0, 6.0], [6.0, 3.0, 8.0], [8.0, 8.0, 1.0]], [[5.0, 2.0, 
4.0], [2.0, 2.0, 8.0], [2.0, 8.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 6.0, 5.0]], [[6.0, 6.0, 2.0]]])+(1.-msk_val)*numpy.array([[[5.0, 1.0, 8.0]], [[5.0, 5.0, 
1.0]]])
      arg[2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [4.0, 2.0, 6.0], [7.0, 8.0, 5.0]], [[5.0, 2.0, 2.0], [7.0, 5.0, 5.0], [7.0, 
2.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 3.0], [1.0, 6.0, 4.0]]], [[[6.0, 6.0, 6.0], [5.0, 7.0, 6.0], [5.0, 8.0, 7.0]], [[4.0, 
6.0, 1.0], [1.0, 8.0, 1.0], [5.0, 6.0, 7.0]], [[8.0, 1.0, 5.0], [6.0, 2.0, 8.0], [8.0, 2.0, 7.0]]], [[[7.0, 4.0, 6.0], [6.0, 
2.0, 7.0], [2.0, 4.0, 8.0]], [[5.0, 5.0, 2.0], [4.0, 4.0, 6.0], [1.0, 6.0, 5.0]], [[2.0, 7.0, 2.0], [7.0, 3.0, 5.0], [6.0, 8.0, 
8.0]]], [[[7.0, 7.0, 1.0], [1.0, 1.0, 3.0], [6.0, 1.0, 4.0]], [[2.0, 6.0, 6.0], [6.0, 3.0, 8.0], [6.0, 6.0, 2.0]], [[5.0, 2.0, 
4.0], [2.0, 2.0, 8.0], [2.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 2.0], [4.0, 2.0, 6.0], [7.0, 8.0, 5.0]], 
[[5.0, 2.0, 2.0], [7.0, 5.0, 5.0], [7.0, 2.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 3.0], [1.0, 6.0, 4.0]]], [[[6.0, 6.0, 6.0], 
[5.0, 7.0, 6.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 1.0], [1.0, 8.0, 1.0], [5.0, 6.0, 7.0]], [[8.0, 1.0, 5.0], [6.0, 2.0, 8.0], 
[8.0, 2.0, 7.0]]], [[[7.0, 4.0, 6.0], [6.0, 2.0, 7.0], [2.0, 4.0, 8.0]], [[5.0, 5.0, 2.0], [4.0, 4.0, 6.0], [5.0, 1.0, 8.0]], 
[[2.0, 7.0, 2.0], [7.0, 3.0, 5.0], [6.0, 8.0, 8.0]]], [[[7.0, 7.0, 1.0], [1.0, 1.0, 3.0], [6.0, 1.0, 4.0]], [[2.0, 6.0, 6.0], 
[6.0, 3.0, 8.0], [5.0, 5.0, 1.0]], [[5.0, 2.0, 4.0], [2.0, 2.0, 8.0], [2.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_ULPN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 8.0], [5.0, 8.0, 1.0], [5.0, 8.0, 4.0]], [[3.0, 6.0, 3.0], [8.0, 7.0, 2.0], [8.0, 2.0, 
1.0]], [[6.0, 8.0, 5.0], [1.0, 5.0, 7.0], [7.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0], [5.0, 5.0, 7.0], [7.0, 7.0, 7.0]], [[6.0, 3.0, 
6.0], [6.0, 2.0, 4.0], [4.0, 1.0, 4.0]], [[1.0, 4.0, 4.0], [1.0, 1.0, 4.0], [7.0, 5.0, 4.0]]], [[[2.0, 8.0, 3.0], [1.0, 4.0, 
5.0], [6.0, 7.0, 5.0]], [[7.0, 8.0, 4.0], [8.0, 6.0, 8.0], [3.0, 5.0, 3.0]], [[5.0, 6.0, 7.0], [8.0, 2.0, 3.0], [8.0, 6.0, 
5.0]]], [[[2.0, 7.0, 3.0], [4.0, 4.0, 5.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 3.0], [8.0, 7.0, 6.0], [3.0, 3.0, 4.0]], [[5.0, 7.0, 
7.0], [2.0, 8.0, 6.0], [4.0, 1.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 8.0, 5.0]], [[4.0, 3.0, 7.0]], [[1.0, 3.0, 2.0]]])
      arg[:3,2:,1]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 8.0], [5.0, 8.0, 1.0], [5.0, 8.0, 4.0]], [[3.0, 6.0, 3.0], [8.0, 7.0, 2.0], [8.0, 2.0, 
1.0]], [[6.0, 8.0, 5.0], [3.0, 8.0, 5.0], [7.0, 6.0, 1.0]]], [[[5.0, 7.0, 7.0], [5.0, 5.0, 7.0], [7.0, 7.0, 7.0]], [[6.0, 3.0, 
6.0], [6.0, 2.0, 4.0], [4.0, 1.0, 4.0]], [[1.0, 4.0, 4.0], [4.0, 3.0, 7.0], [7.0, 5.0, 4.0]]], [[[2.0, 8.0, 3.0], [1.0, 4.0, 
5.0], [6.0, 7.0, 5.0]], [[7.0, 8.0, 4.0], [8.0, 6.0, 8.0], [3.0, 5.0, 3.0]], [[5.0, 6.0, 7.0], [1.0, 3.0, 2.0], [8.0, 6.0, 
5.0]]], [[[2.0, 7.0, 3.0], [4.0, 4.0, 5.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 3.0], [8.0, 7.0, 6.0], [3.0, 3.0, 4.0]], [[5.0, 7.0, 
7.0], [2.0, 8.0, 6.0], [4.0, 1.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_ULPN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 5.0], [5.0, 8.0, 4.0], [3.0, 5.0, 2.0]], [[8.0, 1.0, 8.0], [6.0, 7.0, 8.0], [4.0, 6.0, 
1.0]], [[2.0, 1.0, 5.0], [7.0, 5.0, 3.0], [2.0, 8.0, 8.0]]], [[[6.0, 3.0, 8.0], [3.0, 6.0, 5.0], [7.0, 1.0, 4.0]], [[4.0, 3.0, 
1.0], [4.0, 6.0, 7.0], [7.0, 3.0, 2.0]], [[5.0, 7.0, 4.0], [6.0, 3.0, 6.0], [1.0, 1.0, 4.0]]], [[[6.0, 2.0, 3.0], [8.0, 1.0, 
2.0], [8.0, 4.0, 7.0]], [[8.0, 6.0, 8.0], [7.0, 8.0, 1.0], [2.0, 8.0, 2.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 1.0], [5.0, 2.0, 
5.0]]], [[[7.0, 4.0, 2.0], [8.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[3.0, 6.0, 8.0], [2.0, 5.0, 1.0], [3.0, 7.0, 7.0]], [[1.0, 1.0, 
2.0], [2.0, 6.0, 2.0], [6.0, 2.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 8.0, 3.0]], [[5.0, 8.0, 3.0]], [[3.0, 7.0, 2.0]]]),self.functionspace)
      arg[:3,2:,1]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 5.0], [5.0, 8.0, 4.0], [3.0, 5.0, 2.0]], [[8.0, 1.0, 8.0], [6.0, 7.0, 8.0], [4.0, 6.0, 
1.0]], [[2.0, 1.0, 5.0], [6.0, 8.0, 3.0], [2.0, 8.0, 8.0]]], [[[6.0, 3.0, 8.0], [3.0, 6.0, 5.0], [7.0, 1.0, 4.0]], [[4.0, 3.0, 
1.0], [4.0, 6.0, 7.0], [7.0, 3.0, 2.0]], [[5.0, 7.0, 4.0], [5.0, 8.0, 3.0], [1.0, 1.0, 4.0]]], [[[6.0, 2.0, 3.0], [8.0, 1.0, 
2.0], [8.0, 4.0, 7.0]], [[8.0, 6.0, 8.0], [7.0, 8.0, 1.0], [2.0, 8.0, 2.0]], [[8.0, 8.0, 3.0], [3.0, 7.0, 2.0], [5.0, 2.0, 
5.0]]], [[[7.0, 4.0, 2.0], [8.0, 5.0, 7.0], [8.0, 6.0, 1.0]], [[3.0, 6.0, 8.0], [2.0, 5.0, 1.0], [3.0, 7.0, 7.0]], [[1.0, 1.0, 
2.0], [2.0, 6.0, 2.0], [6.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_ULPN(self):
      arg=Data(numpy.array([[[[5.0, 2.0, 8.0], [8.0, 5.0, 8.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 5.0, 8.0], [3.0, 6.0, 
2.0]], [[7.0, 8.0, 6.0], [3.0, 1.0, 2.0], [1.0, 7.0, 7.0]]], [[[6.0, 5.0, 4.0], [3.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[8.0, 3.0, 
6.0], [2.0, 7.0, 1.0], [5.0, 6.0, 8.0]], [[7.0, 1.0, 6.0], [8.0, 5.0, 7.0], [8.0, 6.0, 5.0]]], [[[6.0, 2.0, 6.0], [7.0, 1.0, 
2.0], [2.0, 3.0, 2.0]], [[6.0, 5.0, 7.0], [1.0, 5.0, 2.0], [1.0, 8.0, 3.0]], [[1.0, 6.0, 4.0], [4.0, 1.0, 8.0], [7.0, 3.0, 
8.0]]], [[[3.0, 3.0, 1.0], [8.0, 3.0, 1.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 8.0], [2.0, 1.0, 4.0], [2.0, 5.0, 2.0]], [[8.0, 6.0, 
7.0], [4.0, 8.0, 8.0], [8.0, 8.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 2.0, 4.0]], [[6.0, 8.0, 6.0]], [[7.0, 8.0, 1.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0, 
7.0]], [[1.0, 8.0, 8.0]], [[7.0, 2.0, 1.0]]])
      arg[:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [8.0, 5.0, 8.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 5.0, 8.0], [3.0, 
6.0, 2.0]], [[7.0, 8.0, 6.0], [8.0, 2.0, 4.0], [1.0, 7.0, 7.0]]], [[[6.0, 5.0, 4.0], [3.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[8.0, 
3.0, 6.0], [2.0, 7.0, 1.0], [5.0, 6.0, 8.0]], [[7.0, 1.0, 6.0], [6.0, 8.0, 6.0], [8.0, 6.0, 5.0]]], [[[6.0, 2.0, 6.0], [7.0, 
1.0, 2.0], [2.0, 3.0, 2.0]], [[6.0, 5.0, 7.0], [1.0, 5.0, 2.0], [1.0, 8.0, 3.0]], [[1.0, 6.0, 4.0], [7.0, 8.0, 1.0], [7.0, 3.0, 
8.0]]], [[[3.0, 3.0, 1.0], [8.0, 3.0, 1.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 8.0], [2.0, 1.0, 4.0], [2.0, 5.0, 2.0]], [[8.0, 6.0, 
7.0], [4.0, 8.0, 8.0], [8.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 8.0], [8.0, 5.0, 8.0], [4.0, 2.0, 6.0]], 
[[3.0, 5.0, 8.0], [5.0, 5.0, 8.0], [3.0, 6.0, 2.0]], [[7.0, 8.0, 6.0], [5.0, 4.0, 7.0], [1.0, 7.0, 7.0]]], [[[6.0, 5.0, 4.0], 
[3.0, 5.0, 8.0], [4.0, 2.0, 8.0]], [[8.0, 3.0, 6.0], [2.0, 7.0, 1.0], [5.0, 6.0, 8.0]], [[7.0, 1.0, 6.0], [1.0, 8.0, 8.0], 
[8.0, 6.0, 5.0]]], [[[6.0, 2.0, 6.0], [7.0, 1.0, 2.0], [2.0, 3.0, 2.0]], [[6.0, 5.0, 7.0], [1.0, 5.0, 2.0], [1.0, 8.0, 3.0]], 
[[1.0, 6.0, 4.0], [7.0, 2.0, 1.0], [7.0, 3.0, 8.0]]], [[[3.0, 3.0, 1.0], [8.0, 3.0, 1.0], [5.0, 8.0, 3.0]], [[4.0, 1.0, 8.0], 
[2.0, 1.0, 4.0], [2.0, 5.0, 2.0]], [[8.0, 6.0, 7.0], [4.0, 8.0, 8.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRRN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 8.0], [6.0, 4.0, 5.0], [3.0, 8.0, 5.0]], [[7.0, 7.0, 5.0], [1.0, 6.0, 3.0], [6.0, 2.0, 
8.0]], [[7.0, 7.0, 1.0], [1.0, 5.0, 1.0], [6.0, 8.0, 5.0]]], [[[1.0, 8.0, 6.0], [8.0, 1.0, 8.0], [6.0, 2.0, 4.0]], [[7.0, 4.0, 
3.0], [1.0, 8.0, 8.0], [5.0, 2.0, 4.0]], [[3.0, 8.0, 3.0], [3.0, 3.0, 5.0], [1.0, 6.0, 7.0]]], [[[2.0, 3.0, 8.0], [4.0, 7.0, 
3.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 2.0], [3.0, 2.0, 2.0], [8.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [7.0, 1.0, 1.0], [6.0, 3.0, 
6.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 2.0], [7.0, 2.0, 8.0]], [[2.0, 6.0, 1.0], [6.0, 5.0, 1.0], [6.0, 5.0, 1.0]], [[6.0, 7.0, 
2.0], [7.0, 6.0, 5.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 2.0, 2.0]]], [[[1.0, 8.0, 6.0]]]])
      arg[2:,1:2,1:2]=val
      ref=Data(numpy.array([[[[3.0, 7.0, 8.0], [6.0, 4.0, 5.0], [3.0, 8.0, 5.0]], [[7.0, 7.0, 5.0], [1.0, 6.0, 3.0], [6.0, 2.0, 
8.0]], [[7.0, 7.0, 1.0], [1.0, 5.0, 1.0], [6.0, 8.0, 5.0]]], [[[1.0, 8.0, 6.0], [8.0, 1.0, 8.0], [6.0, 2.0, 4.0]], [[7.0, 4.0, 
3.0], [1.0, 8.0, 8.0], [5.0, 2.0, 4.0]], [[3.0, 8.0, 3.0], [3.0, 3.0, 5.0], [1.0, 6.0, 7.0]]], [[[2.0, 3.0, 8.0], [4.0, 7.0, 
3.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 2.0], [3.0, 2.0, 2.0], [8.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [7.0, 1.0, 1.0], [6.0, 3.0, 
6.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 2.0], [7.0, 2.0, 8.0]], [[2.0, 6.0, 1.0], [1.0, 8.0, 6.0], [6.0, 5.0, 1.0]], [[6.0, 7.0, 
2.0], [7.0, 6.0, 5.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRRN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 2.0], [3.0, 5.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 1.0, 3.0], [8.0, 5.0, 1.0], [1.0, 7.0, 
7.0]], [[6.0, 7.0, 4.0], [6.0, 5.0, 6.0], [3.0, 7.0, 4.0]]], [[[8.0, 2.0, 5.0], [4.0, 8.0, 2.0], [3.0, 2.0, 2.0]], [[5.0, 4.0, 
7.0], [5.0, 4.0, 3.0], [1.0, 1.0, 5.0]], [[2.0, 7.0, 6.0], [4.0, 1.0, 8.0], [5.0, 7.0, 2.0]]], [[[6.0, 2.0, 4.0], [4.0, 5.0, 
6.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], [7.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[6.0, 6.0, 3.0], [1.0, 4.0, 8.0], [6.0, 8.0, 
3.0]]], [[[2.0, 6.0, 8.0], [6.0, 8.0, 8.0], [8.0, 6.0, 2.0]], [[2.0, 6.0, 4.0], [5.0, 2.0, 1.0], [6.0, 6.0, 4.0]], [[8.0, 1.0, 
1.0], [3.0, 4.0, 5.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 5.0, 7.0]]], [[[5.0, 3.0, 5.0]]]]),self.functionspace)
      arg[2:,1:2,1:2]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 2.0], [3.0, 5.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 1.0, 3.0], [8.0, 5.0, 1.0], [1.0, 7.0, 
7.0]], [[6.0, 7.0, 4.0], [6.0, 5.0, 6.0], [3.0, 7.0, 4.0]]], [[[8.0, 2.0, 5.0], [4.0, 8.0, 2.0], [3.0, 2.0, 2.0]], [[5.0, 4.0, 
7.0], [5.0, 4.0, 3.0], [1.0, 1.0, 5.0]], [[2.0, 7.0, 6.0], [4.0, 1.0, 8.0], [5.0, 7.0, 2.0]]], [[[6.0, 2.0, 4.0], [4.0, 5.0, 
6.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 5.0], [6.0, 5.0, 7.0], [4.0, 4.0, 7.0]], [[6.0, 6.0, 3.0], [1.0, 4.0, 8.0], [6.0, 8.0, 
3.0]]], [[[2.0, 6.0, 8.0], [6.0, 8.0, 8.0], [8.0, 6.0, 2.0]], [[2.0, 6.0, 4.0], [5.0, 3.0, 5.0], [6.0, 6.0, 4.0]], [[8.0, 1.0, 
1.0], [3.0, 4.0, 5.0], [2.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRRN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 3.0], [1.0, 3.0, 8.0]], [[3.0, 5.0, 3.0], [1.0, 6.0, 2.0], [4.0, 3.0, 
6.0]], [[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [4.0, 7.0, 6.0]]], [[[8.0, 2.0, 7.0], [3.0, 7.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 6.0, 
6.0], [4.0, 6.0, 6.0], [5.0, 1.0, 5.0]], [[2.0, 5.0, 5.0], [4.0, 7.0, 6.0], [1.0, 7.0, 8.0]]], [[[3.0, 3.0, 7.0], [7.0, 8.0, 
8.0], [6.0, 2.0, 8.0]], [[1.0, 7.0, 1.0], [3.0, 6.0, 4.0], [4.0, 5.0, 2.0]], [[3.0, 4.0, 3.0], [3.0, 3.0, 5.0], [7.0, 5.0, 
6.0]]], [[[3.0, 3.0, 5.0], [8.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[8.0, 4.0, 1.0], [3.0, 3.0, 5.0], [8.0, 1.0, 2.0]], [[6.0, 2.0, 
3.0], [6.0, 7.0, 1.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 7.0, 1.0]]], [[[4.0, 6.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 1.0, 5.0]]], 
[[[4.0, 8.0, 6.0]]]])
      arg[2:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 3.0], [1.0, 3.0, 8.0]], [[3.0, 5.0, 3.0], [1.0, 6.0, 2.0], [4.0, 
3.0, 6.0]], [[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [4.0, 7.0, 6.0]]], [[[8.0, 2.0, 7.0], [3.0, 7.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 
6.0, 6.0], [4.0, 6.0, 6.0], [5.0, 1.0, 5.0]], [[2.0, 5.0, 5.0], [4.0, 7.0, 6.0], [1.0, 7.0, 8.0]]], [[[3.0, 3.0, 7.0], [7.0, 
8.0, 8.0], [6.0, 2.0, 8.0]], [[1.0, 7.0, 1.0], [5.0, 7.0, 1.0], [4.0, 5.0, 2.0]], [[3.0, 4.0, 3.0], [3.0, 3.0, 5.0], [7.0, 5.0, 
6.0]]], [[[3.0, 3.0, 5.0], [8.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[8.0, 4.0, 1.0], [4.0, 6.0, 7.0], [8.0, 1.0, 2.0]], [[6.0, 2.0, 
3.0], [6.0, 7.0, 1.0], [6.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 8.0, 3.0], [1.0, 3.0, 8.0]], 
[[3.0, 5.0, 3.0], [1.0, 6.0, 2.0], [4.0, 3.0, 6.0]], [[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [4.0, 7.0, 6.0]]], [[[8.0, 2.0, 7.0], 
[3.0, 7.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 6.0, 6.0], [4.0, 6.0, 6.0], [5.0, 1.0, 5.0]], [[2.0, 5.0, 5.0], [4.0, 7.0, 6.0], 
[1.0, 7.0, 8.0]]], [[[3.0, 3.0, 7.0], [7.0, 8.0, 8.0], [6.0, 2.0, 8.0]], [[1.0, 7.0, 1.0], [6.0, 1.0, 5.0], [4.0, 5.0, 2.0]], 
[[3.0, 4.0, 3.0], [3.0, 3.0, 5.0], [7.0, 5.0, 6.0]]], [[[3.0, 3.0, 5.0], [8.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[8.0, 4.0, 1.0], 
[4.0, 8.0, 6.0], [8.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [6.0, 7.0, 1.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CUPN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 1.0], [4.0, 4.0, 2.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 3.0], [6.0, 3.0, 5.0], [1.0, 8.0, 
8.0]], [[3.0, 5.0, 8.0], [3.0, 1.0, 4.0], [7.0, 8.0, 4.0]]], [[[1.0, 4.0, 5.0], [6.0, 3.0, 3.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 
7.0], [3.0, 8.0, 6.0], [6.0, 3.0, 5.0]], [[7.0, 3.0, 6.0], [8.0, 3.0, 4.0], [2.0, 6.0, 5.0]]], [[[4.0, 1.0, 3.0], [7.0, 3.0, 
3.0], [6.0, 6.0, 8.0]], [[3.0, 6.0, 5.0], [5.0, 3.0, 6.0], [8.0, 3.0, 5.0]], [[4.0, 6.0, 5.0], [8.0, 5.0, 8.0], [2.0, 4.0, 
4.0]]], [[[5.0, 2.0, 6.0], [2.0, 1.0, 1.0], [1.0, 2.0, 4.0]], [[5.0, 3.0, 5.0], [7.0, 8.0, 8.0], [3.0, 2.0, 3.0]], [[3.0, 1.0, 
6.0], [2.0, 7.0, 2.0], [3.0, 3.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 8.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 7.0], [6.0, 4.0, 3.0]], [[4.0, 8.0, 2.0], [2.0, 3.0, 
5.0]], [[1.0, 5.0, 5.0], [3.0, 1.0, 1.0]]])
      arg[:,:2,1]=val
      ref=Data(numpy.array([[[[1.0, 6.0, 1.0], [4.0, 8.0, 4.0], [2.0, 3.0, 2.0]], [[3.0, 2.0, 3.0], [5.0, 5.0, 4.0], [1.0, 8.0, 
8.0]], [[3.0, 5.0, 8.0], [3.0, 1.0, 4.0], [7.0, 8.0, 4.0]]], [[[1.0, 4.0, 5.0], [2.0, 1.0, 7.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 
7.0], [6.0, 4.0, 3.0], [6.0, 3.0, 5.0]], [[7.0, 3.0, 6.0], [8.0, 3.0, 4.0], [2.0, 6.0, 5.0]]], [[[4.0, 1.0, 3.0], [4.0, 8.0, 
2.0], [6.0, 6.0, 8.0]], [[3.0, 6.0, 5.0], [2.0, 3.0, 5.0], [8.0, 3.0, 5.0]], [[4.0, 6.0, 5.0], [8.0, 5.0, 8.0], [2.0, 4.0, 
4.0]]], [[[5.0, 2.0, 6.0], [1.0, 5.0, 5.0], [1.0, 2.0, 4.0]], [[5.0, 3.0, 5.0], [3.0, 1.0, 1.0], [3.0, 2.0, 3.0]], [[3.0, 1.0, 
6.0], [2.0, 7.0, 2.0], [3.0, 3.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CUPN(self):
      arg=Data(numpy.array([[[[7.0, 3.0, 5.0], [5.0, 8.0, 2.0], [3.0, 5.0, 3.0]], [[3.0, 4.0, 1.0], [6.0, 6.0, 2.0], [4.0, 2.0, 
6.0]], [[6.0, 7.0, 3.0], [8.0, 6.0, 5.0], [6.0, 2.0, 7.0]]], [[[4.0, 4.0, 6.0], [1.0, 8.0, 2.0], [4.0, 2.0, 5.0]], [[5.0, 7.0, 
7.0], [6.0, 3.0, 8.0], [3.0, 6.0, 6.0]], [[4.0, 3.0, 1.0], [4.0, 4.0, 2.0], [2.0, 8.0, 3.0]]], [[[2.0, 6.0, 4.0], [3.0, 8.0, 
8.0], [1.0, 3.0, 3.0]], [[5.0, 6.0, 7.0], [6.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[4.0, 6.0, 4.0], [6.0, 7.0, 4.0], [1.0, 5.0, 
3.0]]], [[[2.0, 1.0, 5.0], [8.0, 8.0, 5.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 7.0], [7.0, 4.0, 4.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 
3.0], [6.0, 6.0, 8.0], [8.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 7.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 1.0, 5.0], [2.0, 6.0, 1.0]], [[8.0, 2.0, 7.0], [3.0, 
5.0, 1.0]], [[2.0, 2.0, 2.0], [7.0, 4.0, 4.0]]]),self.functionspace)
      arg[:,:2,1]=val
      ref=Data(numpy.array([[[[7.0, 3.0, 5.0], [6.0, 7.0, 2.0], [3.0, 5.0, 3.0]], [[3.0, 4.0, 1.0], [7.0, 4.0, 8.0], [4.0, 2.0, 
6.0]], [[6.0, 7.0, 3.0], [8.0, 6.0, 5.0], [6.0, 2.0, 7.0]]], [[[4.0, 4.0, 6.0], [4.0, 1.0, 5.0], [4.0, 2.0, 5.0]], [[5.0, 7.0, 
7.0], [2.0, 6.0, 1.0], [3.0, 6.0, 6.0]], [[4.0, 3.0, 1.0], [4.0, 4.0, 2.0], [2.0, 8.0, 3.0]]], [[[2.0, 6.0, 4.0], [8.0, 2.0, 
7.0], [1.0, 3.0, 3.0]], [[5.0, 6.0, 7.0], [3.0, 5.0, 1.0], [4.0, 1.0, 2.0]], [[4.0, 6.0, 4.0], [6.0, 7.0, 4.0], [1.0, 5.0, 
3.0]]], [[[2.0, 1.0, 5.0], [2.0, 2.0, 2.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 7.0], [7.0, 4.0, 4.0], [7.0, 2.0, 4.0]], [[4.0, 1.0, 
3.0], [6.0, 6.0, 8.0], [8.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CUPN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 4.0], [5.0, 8.0, 5.0], [2.0, 4.0, 4.0]], [[8.0, 1.0, 2.0], [5.0, 8.0, 6.0], [5.0, 1.0, 
8.0]], [[1.0, 7.0, 4.0], [6.0, 7.0, 5.0], [6.0, 8.0, 1.0]]], [[[7.0, 1.0, 2.0], [4.0, 7.0, 1.0], [6.0, 2.0, 7.0]], [[4.0, 1.0, 
3.0], [7.0, 5.0, 7.0], [1.0, 7.0, 3.0]], [[5.0, 4.0, 3.0], [5.0, 7.0, 8.0], [5.0, 4.0, 6.0]]], [[[7.0, 1.0, 6.0], [3.0, 6.0, 
4.0], [6.0, 3.0, 8.0]], [[1.0, 4.0, 8.0], [4.0, 8.0, 6.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 6.0], [2.0, 7.0, 4.0], [2.0, 4.0, 
3.0]]], [[[8.0, 6.0, 5.0], [3.0, 8.0, 8.0], [5.0, 6.0, 3.0]], [[4.0, 2.0, 4.0], [6.0, 7.0, 5.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 
4.0], [1.0, 1.0, 1.0], [4.0, 4.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 8.0, 7.0], [8.0, 5.0, 5.0]], [[2.0, 8.0, 4.0], [6.0, 6.0, 1.0]], [[3.0, 7.0, 4.0], [2.0, 
1.0, 6.0]], [[1.0, 7.0, 8.0], [1.0, 3.0, 2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0, 8.0], [4.0, 7.0, 1.0]], [[7.0, 4.0, 
7.0], [7.0, 7.0, 1.0]], [[7.0, 5.0, 3.0], [8.0, 6.0, 7.0]], [[6.0, 7.0, 6.0], [7.0, 1.0, 8.0]]])
      arg[:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 4.0], [4.0, 8.0, 7.0], [2.0, 4.0, 4.0]], [[8.0, 1.0, 2.0], [8.0, 5.0, 5.0], [5.0, 
1.0, 8.0]], [[1.0, 7.0, 4.0], [6.0, 7.0, 5.0], [6.0, 8.0, 1.0]]], [[[7.0, 1.0, 2.0], [2.0, 8.0, 4.0], [6.0, 2.0, 7.0]], [[4.0, 
1.0, 3.0], [6.0, 6.0, 1.0], [1.0, 7.0, 3.0]], [[5.0, 4.0, 3.0], [5.0, 7.0, 8.0], [5.0, 4.0, 6.0]]], [[[7.0, 1.0, 6.0], [3.0, 
7.0, 4.0], [6.0, 3.0, 8.0]], [[1.0, 4.0, 8.0], [2.0, 1.0, 6.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 6.0], [2.0, 7.0, 4.0], [2.0, 4.0, 
3.0]]], [[[8.0, 6.0, 5.0], [1.0, 7.0, 8.0], [5.0, 6.0, 3.0]], [[4.0, 2.0, 4.0], [1.0, 3.0, 2.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 
4.0], [1.0, 1.0, 1.0], [4.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 4.0], [4.0, 7.0, 8.0], [2.0, 4.0, 4.0]], 
[[8.0, 1.0, 2.0], [4.0, 7.0, 1.0], [5.0, 1.0, 8.0]], [[1.0, 7.0, 4.0], [6.0, 7.0, 5.0], [6.0, 8.0, 1.0]]], [[[7.0, 1.0, 2.0], 
[7.0, 4.0, 7.0], [6.0, 2.0, 7.0]], [[4.0, 1.0, 3.0], [7.0, 7.0, 1.0], [1.0, 7.0, 3.0]], [[5.0, 4.0, 3.0], [5.0, 7.0, 8.0], 
[5.0, 4.0, 6.0]]], [[[7.0, 1.0, 6.0], [7.0, 5.0, 3.0], [6.0, 3.0, 8.0]], [[1.0, 4.0, 8.0], [8.0, 6.0, 7.0], [6.0, 4.0, 5.0]], 
[[4.0, 5.0, 6.0], [2.0, 7.0, 4.0], [2.0, 4.0, 3.0]]], [[[8.0, 6.0, 5.0], [6.0, 7.0, 6.0], [5.0, 6.0, 3.0]], [[4.0, 2.0, 4.0], 
[7.0, 1.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 1.0, 4.0], [1.0, 1.0, 1.0], [4.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCPN(self):
      arg=Data(numpy.array([[[[6.0, 8.0, 6.0], [7.0, 7.0, 1.0], [2.0, 2.0, 8.0]], [[1.0, 2.0, 7.0], [2.0, 6.0, 7.0], [6.0, 5.0, 
2.0]], [[2.0, 7.0, 5.0], [7.0, 1.0, 7.0], [6.0, 1.0, 2.0]]], [[[4.0, 5.0, 8.0], [5.0, 8.0, 7.0], [6.0, 1.0, 4.0]], [[6.0, 6.0, 
5.0], [8.0, 4.0, 8.0], [5.0, 7.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 5.0, 2.0], [1.0, 1.0, 7.0]]], [[[5.0, 6.0, 3.0], [2.0, 1.0, 
2.0], [4.0, 7.0, 3.0]], [[3.0, 1.0, 1.0], [5.0, 7.0, 3.0], [6.0, 4.0, 7.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 4.0], [2.0, 4.0, 
6.0]]], [[[4.0, 6.0, 7.0], [4.0, 3.0, 4.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [7.0, 1.0, 8.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 
3.0], [8.0, 5.0, 3.0], [8.0, 2.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 4.0, 8.0], [4.0, 8.0, 5.0], [5.0, 8.0, 2.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 2.0], [7.0, 4.0, 
6.0]]])
      arg[1:3,:,1]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 6.0], [7.0, 7.0, 1.0], [2.0, 2.0, 8.0]], [[1.0, 2.0, 7.0], [2.0, 6.0, 7.0], [6.0, 5.0, 
2.0]], [[2.0, 7.0, 5.0], [7.0, 1.0, 7.0], [6.0, 1.0, 2.0]]], [[[4.0, 5.0, 8.0], [5.0, 4.0, 8.0], [6.0, 1.0, 4.0]], [[6.0, 6.0, 
5.0], [4.0, 8.0, 5.0], [5.0, 7.0, 1.0]], [[3.0, 5.0, 1.0], [5.0, 8.0, 2.0], [1.0, 1.0, 7.0]]], [[[5.0, 6.0, 3.0], [7.0, 8.0, 
5.0], [4.0, 7.0, 3.0]], [[3.0, 1.0, 1.0], [6.0, 5.0, 2.0], [6.0, 4.0, 7.0]], [[6.0, 5.0, 5.0], [7.0, 4.0, 6.0], [2.0, 4.0, 
6.0]]], [[[4.0, 6.0, 7.0], [4.0, 3.0, 4.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 2.0], [7.0, 1.0, 8.0], [5.0, 1.0, 5.0]], [[1.0, 8.0, 
3.0], [8.0, 5.0, 3.0], [8.0, 2.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCPN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 2.0], [4.0, 8.0, 5.0], [8.0, 5.0, 1.0]], [[5.0, 1.0, 7.0], [8.0, 7.0, 3.0], [4.0, 5.0, 
7.0]], [[6.0, 8.0, 8.0], [4.0, 1.0, 5.0], [5.0, 5.0, 4.0]]], [[[3.0, 5.0, 4.0], [3.0, 8.0, 5.0], [6.0, 7.0, 5.0]], [[7.0, 4.0, 
1.0], [8.0, 8.0, 6.0], [1.0, 1.0, 4.0]], [[6.0, 2.0, 5.0], [1.0, 6.0, 5.0], [3.0, 3.0, 6.0]]], [[[7.0, 3.0, 2.0], [1.0, 8.0, 
1.0], [6.0, 7.0, 8.0]], [[3.0, 2.0, 3.0], [3.0, 4.0, 3.0], [5.0, 6.0, 4.0]], [[3.0, 4.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 
8.0]]], [[[2.0, 6.0, 3.0], [5.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[7.0, 2.0, 2.0], [1.0, 3.0, 3.0], [4.0, 6.0, 2.0]], [[7.0, 7.0, 
8.0], [8.0, 3.0, 2.0], [2.0, 8.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 8.0, 1.0], [7.0, 6.0, 3.0], [8.0, 5.0, 4.0]], [[1.0, 2.0, 2.0], [4.0, 1.0, 5.0], [6.0, 5.0, 
4.0]]]),self.functionspace)
      arg[1:3,:,1]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 2.0], [4.0, 8.0, 5.0], [8.0, 5.0, 1.0]], [[5.0, 1.0, 7.0], [8.0, 7.0, 3.0], [4.0, 5.0, 
7.0]], [[6.0, 8.0, 8.0], [4.0, 1.0, 5.0], [5.0, 5.0, 4.0]]], [[[3.0, 5.0, 4.0], [3.0, 8.0, 1.0], [6.0, 7.0, 5.0]], [[7.0, 4.0, 
1.0], [7.0, 6.0, 3.0], [1.0, 1.0, 4.0]], [[6.0, 2.0, 5.0], [8.0, 5.0, 4.0], [3.0, 3.0, 6.0]]], [[[7.0, 3.0, 2.0], [1.0, 2.0, 
2.0], [6.0, 7.0, 8.0]], [[3.0, 2.0, 3.0], [4.0, 1.0, 5.0], [5.0, 6.0, 4.0]], [[3.0, 4.0, 8.0], [6.0, 5.0, 4.0], [2.0, 2.0, 
8.0]]], [[[2.0, 6.0, 3.0], [5.0, 5.0, 3.0], [2.0, 8.0, 6.0]], [[7.0, 2.0, 2.0], [1.0, 3.0, 3.0], [4.0, 6.0, 2.0]], [[7.0, 7.0, 
8.0], [8.0, 3.0, 2.0], [2.0, 8.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCPN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 4.0], [4.0, 7.0, 3.0], [2.0, 5.0, 2.0]], [[8.0, 1.0, 8.0], [6.0, 1.0, 2.0], [4.0, 6.0, 
7.0]], [[5.0, 3.0, 5.0], [5.0, 6.0, 8.0], [6.0, 2.0, 3.0]]], [[[1.0, 2.0, 7.0], [2.0, 4.0, 2.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 
8.0], [5.0, 5.0, 5.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 1.0], [2.0, 6.0, 3.0]]], [[[1.0, 6.0, 1.0], [2.0, 2.0, 
6.0], [7.0, 2.0, 4.0]], [[8.0, 2.0, 5.0], [8.0, 5.0, 2.0], [3.0, 8.0, 2.0]], [[4.0, 7.0, 6.0], [2.0, 2.0, 8.0], [6.0, 4.0, 
1.0]]], [[[2.0, 7.0, 6.0], [3.0, 2.0, 2.0], [7.0, 2.0, 2.0]], [[8.0, 6.0, 4.0], [5.0, 2.0, 7.0], [1.0, 3.0, 6.0]], [[2.0, 8.0, 
5.0], [2.0, 7.0, 5.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 3.0, 1.0], [5.0, 2.0, 1.0], [1.0, 4.0, 7.0]], [[5.0, 2.0, 7.0], [4.0, 6.0, 5.0], [3.0, 
5.0, 3.0]]])+(1.-msk_val)*numpy.array([[[2.0, 3.0, 2.0], [1.0, 1.0, 8.0], [8.0, 4.0, 5.0]], [[5.0, 3.0, 5.0], [2.0, 8.0, 7.0], 
[7.0, 8.0, 2.0]]])
      arg[1:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 4.0], [4.0, 7.0, 3.0], [2.0, 5.0, 2.0]], [[8.0, 1.0, 8.0], [6.0, 1.0, 2.0], [4.0, 
6.0, 7.0]], [[5.0, 3.0, 5.0], [5.0, 6.0, 8.0], [6.0, 2.0, 3.0]]], [[[1.0, 2.0, 7.0], [8.0, 3.0, 1.0], [2.0, 7.0, 2.0]], [[3.0, 
7.0, 8.0], [5.0, 2.0, 1.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 4.0], [1.0, 4.0, 7.0], [2.0, 6.0, 3.0]]], [[[1.0, 6.0, 1.0], [5.0, 
2.0, 7.0], [7.0, 2.0, 4.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 5.0], [3.0, 8.0, 2.0]], [[4.0, 7.0, 6.0], [3.0, 5.0, 3.0], [6.0, 4.0, 
1.0]]], [[[2.0, 7.0, 6.0], [3.0, 2.0, 2.0], [7.0, 2.0, 2.0]], [[8.0, 6.0, 4.0], [5.0, 2.0, 7.0], [1.0, 3.0, 6.0]], [[2.0, 8.0, 
5.0], [2.0, 7.0, 5.0], [3.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 4.0], [4.0, 7.0, 3.0], [2.0, 5.0, 2.0]], 
[[8.0, 1.0, 8.0], [6.0, 1.0, 2.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 5.0], [5.0, 6.0, 8.0], [6.0, 2.0, 3.0]]], [[[1.0, 2.0, 7.0], 
[2.0, 3.0, 2.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 8.0], [1.0, 1.0, 8.0], [1.0, 4.0, 6.0]], [[4.0, 8.0, 4.0], [8.0, 4.0, 5.0], 
[2.0, 6.0, 3.0]]], [[[1.0, 6.0, 1.0], [5.0, 3.0, 5.0], [7.0, 2.0, 4.0]], [[8.0, 2.0, 5.0], [2.0, 8.0, 7.0], [3.0, 8.0, 2.0]], 
[[4.0, 7.0, 6.0], [7.0, 8.0, 2.0], [6.0, 4.0, 1.0]]], [[[2.0, 7.0, 6.0], [3.0, 2.0, 2.0], [7.0, 2.0, 2.0]], [[8.0, 6.0, 4.0], 
[5.0, 2.0, 7.0], [1.0, 3.0, 6.0]], [[2.0, 8.0, 5.0], [2.0, 7.0, 5.0], [3.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UPCN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 7.0], [3.0, 1.0, 3.0], [6.0, 8.0, 4.0]], [[2.0, 6.0, 3.0], [1.0, 3.0, 7.0], [2.0, 7.0, 
2.0]], [[6.0, 2.0, 5.0], [6.0, 6.0, 7.0], [4.0, 8.0, 5.0]]], [[[5.0, 6.0, 1.0], [3.0, 4.0, 8.0], [3.0, 8.0, 3.0]], [[1.0, 8.0, 
6.0], [3.0, 1.0, 8.0], [4.0, 6.0, 2.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 8.0], [8.0, 1.0, 7.0]]], [[[4.0, 8.0, 1.0], [8.0, 3.0, 
8.0], [7.0, 5.0, 5.0]], [[1.0, 6.0, 2.0], [6.0, 6.0, 8.0], [1.0, 3.0, 6.0]], [[6.0, 6.0, 7.0], [4.0, 4.0, 7.0], [7.0, 5.0, 
3.0]]], [[[8.0, 5.0, 7.0], [2.0, 7.0, 5.0], [8.0, 1.0, 1.0]], [[7.0, 6.0, 6.0], [1.0, 5.0, 7.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 
2.0], [3.0, 6.0, 2.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[6.0, 7.0, 5.0], [8.0, 7.0, 3.0], [3.0, 2.0, 6.0]], [[2.0, 1.0, 3.0], [3.0, 8.0, 5.0], [5.0, 5.0, 
7.0]], [[5.0, 3.0, 6.0], [5.0, 4.0, 4.0], [8.0, 2.0, 4.0]]])
      arg[:3,1,:]=val
      ref=Data(numpy.array([[[[2.0, 4.0, 7.0], [3.0, 1.0, 3.0], [6.0, 8.0, 4.0]], [[6.0, 7.0, 5.0], [8.0, 7.0, 3.0], [3.0, 2.0, 
6.0]], [[6.0, 2.0, 5.0], [6.0, 6.0, 7.0], [4.0, 8.0, 5.0]]], [[[5.0, 6.0, 1.0], [3.0, 4.0, 8.0], [3.0, 8.0, 3.0]], [[2.0, 1.0, 
3.0], [3.0, 8.0, 5.0], [5.0, 5.0, 7.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 8.0], [8.0, 1.0, 7.0]]], [[[4.0, 8.0, 1.0], [8.0, 3.0, 
8.0], [7.0, 5.0, 5.0]], [[5.0, 3.0, 6.0], [5.0, 4.0, 4.0], [8.0, 2.0, 4.0]], [[6.0, 6.0, 7.0], [4.0, 4.0, 7.0], [7.0, 5.0, 
3.0]]], [[[8.0, 5.0, 7.0], [2.0, 7.0, 5.0], [8.0, 1.0, 1.0]], [[7.0, 6.0, 6.0], [1.0, 5.0, 7.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 
2.0], [3.0, 6.0, 2.0], [6.0, 2.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UPCN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 5.0], [6.0, 8.0, 5.0], [4.0, 8.0, 5.0]], [[2.0, 6.0, 8.0], [6.0, 5.0, 7.0], [6.0, 1.0, 
4.0]], [[7.0, 7.0, 8.0], [1.0, 8.0, 6.0], [6.0, 3.0, 2.0]]], [[[4.0, 8.0, 1.0], [5.0, 2.0, 1.0], [8.0, 6.0, 8.0]], [[7.0, 3.0, 
4.0], [7.0, 8.0, 5.0], [3.0, 2.0, 4.0]], [[1.0, 1.0, 5.0], [7.0, 5.0, 3.0], [2.0, 1.0, 5.0]]], [[[6.0, 7.0, 5.0], [6.0, 5.0, 
4.0], [1.0, 6.0, 5.0]], [[4.0, 3.0, 6.0], [5.0, 8.0, 4.0], [4.0, 7.0, 8.0]], [[2.0, 4.0, 7.0], [6.0, 1.0, 1.0], [5.0, 5.0, 
1.0]]], [[[5.0, 8.0, 7.0], [2.0, 6.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 3.0, 6.0], [2.0, 2.0, 1.0], [8.0, 5.0, 6.0]], [[2.0, 7.0, 
3.0], [2.0, 1.0, 7.0], [4.0, 4.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[7.0, 4.0, 6.0], [3.0, 7.0, 8.0], [5.0, 4.0, 1.0]], [[2.0, 6.0, 4.0], [2.0, 1.0, 7.0], [3.0, 3.0, 
6.0]], [[7.0, 2.0, 5.0], [6.0, 7.0, 3.0], [3.0, 7.0, 3.0]]]),self.functionspace)
      arg[:3,1,:]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 5.0], [6.0, 8.0, 5.0], [4.0, 8.0, 5.0]], [[7.0, 4.0, 6.0], [3.0, 7.0, 8.0], [5.0, 4.0, 
1.0]], [[7.0, 7.0, 8.0], [1.0, 8.0, 6.0], [6.0, 3.0, 2.0]]], [[[4.0, 8.0, 1.0], [5.0, 2.0, 1.0], [8.0, 6.0, 8.0]], [[2.0, 6.0, 
4.0], [2.0, 1.0, 7.0], [3.0, 3.0, 6.0]], [[1.0, 1.0, 5.0], [7.0, 5.0, 3.0], [2.0, 1.0, 5.0]]], [[[6.0, 7.0, 5.0], [6.0, 5.0, 
4.0], [1.0, 6.0, 5.0]], [[7.0, 2.0, 5.0], [6.0, 7.0, 3.0], [3.0, 7.0, 3.0]], [[2.0, 4.0, 7.0], [6.0, 1.0, 1.0], [5.0, 5.0, 
1.0]]], [[[5.0, 8.0, 7.0], [2.0, 6.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 3.0, 6.0], [2.0, 2.0, 1.0], [8.0, 5.0, 6.0]], [[2.0, 7.0, 
3.0], [2.0, 1.0, 7.0], [4.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UPCN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 4.0], [6.0, 1.0, 6.0], [4.0, 7.0, 4.0]], [[5.0, 7.0, 3.0], [3.0, 4.0, 3.0], [7.0, 5.0, 
3.0]], [[5.0, 1.0, 3.0], [8.0, 8.0, 3.0], [4.0, 1.0, 5.0]]], [[[1.0, 6.0, 3.0], [1.0, 6.0, 1.0], [7.0, 5.0, 6.0]], [[6.0, 1.0, 
6.0], [4.0, 3.0, 3.0], [2.0, 4.0, 8.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 1.0], [7.0, 7.0, 4.0]]], [[[1.0, 7.0, 3.0], [3.0, 8.0, 
1.0], [6.0, 5.0, 1.0]], [[2.0, 2.0, 6.0], [3.0, 4.0, 6.0], [2.0, 8.0, 5.0]], [[6.0, 8.0, 4.0], [3.0, 1.0, 8.0], [3.0, 1.0, 
3.0]]], [[[2.0, 2.0, 4.0], [7.0, 2.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 4.0, 1.0], [4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[1.0, 5.0, 
1.0], [1.0, 8.0, 6.0], [6.0, 1.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 5.0, 2.0], [5.0, 1.0, 7.0], [6.0, 4.0, 5.0]], [[8.0, 8.0, 7.0], [4.0, 7.0, 8.0], [5.0, 
4.0, 2.0]], [[6.0, 7.0, 1.0], [1.0, 7.0, 2.0], [4.0, 1.0, 1.0]]])+(1.-msk_val)*numpy.array([[[1.0, 5.0, 5.0], [7.0, 7.0, 7.0], 
[3.0, 3.0, 2.0]], [[2.0, 3.0, 5.0], [6.0, 7.0, 8.0], [6.0, 6.0, 7.0]], [[4.0, 1.0, 8.0], [4.0, 6.0, 4.0], [2.0, 7.0, 1.0]]])
      arg[:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 4.0], [6.0, 1.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [5.0, 1.0, 7.0], [6.0, 
4.0, 5.0]], [[5.0, 1.0, 3.0], [8.0, 8.0, 3.0], [4.0, 1.0, 5.0]]], [[[1.0, 6.0, 3.0], [1.0, 6.0, 1.0], [7.0, 5.0, 6.0]], [[8.0, 
8.0, 7.0], [4.0, 7.0, 8.0], [5.0, 4.0, 2.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 1.0], [7.0, 7.0, 4.0]]], [[[1.0, 7.0, 3.0], [3.0, 
8.0, 1.0], [6.0, 5.0, 1.0]], [[6.0, 7.0, 1.0], [1.0, 7.0, 2.0], [4.0, 1.0, 1.0]], [[6.0, 8.0, 4.0], [3.0, 1.0, 8.0], [3.0, 1.0, 
3.0]]], [[[2.0, 2.0, 4.0], [7.0, 2.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 4.0, 1.0], [4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[1.0, 5.0, 
1.0], [1.0, 8.0, 6.0], [6.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 4.0], [6.0, 1.0, 6.0], [4.0, 7.0, 4.0]], 
[[1.0, 5.0, 5.0], [7.0, 7.0, 7.0], [3.0, 3.0, 2.0]], [[5.0, 1.0, 3.0], [8.0, 8.0, 3.0], [4.0, 1.0, 5.0]]], [[[1.0, 6.0, 3.0], 
[1.0, 6.0, 1.0], [7.0, 5.0, 6.0]], [[2.0, 3.0, 5.0], [6.0, 7.0, 8.0], [6.0, 6.0, 7.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 1.0], 
[7.0, 7.0, 4.0]]], [[[1.0, 7.0, 3.0], [3.0, 8.0, 1.0], [6.0, 5.0, 1.0]], [[4.0, 1.0, 8.0], [4.0, 6.0, 4.0], [2.0, 7.0, 1.0]], 
[[6.0, 8.0, 4.0], [3.0, 1.0, 8.0], [3.0, 1.0, 3.0]]], [[[2.0, 2.0, 4.0], [7.0, 2.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 4.0, 1.0], 
[4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[1.0, 5.0, 1.0], [1.0, 8.0, 6.0], [6.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPCN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 1.0], [7.0, 5.0, 5.0], [4.0, 3.0, 1.0]], [[1.0, 6.0, 3.0], [6.0, 7.0, 3.0], [2.0, 2.0, 
7.0]], [[7.0, 7.0, 5.0], [5.0, 6.0, 5.0], [6.0, 5.0, 6.0]]], [[[6.0, 6.0, 8.0], [6.0, 6.0, 8.0], [8.0, 1.0, 3.0]], [[5.0, 1.0, 
8.0], [5.0, 7.0, 3.0], [6.0, 2.0, 4.0]], [[2.0, 5.0, 2.0], [4.0, 1.0, 2.0], [1.0, 1.0, 6.0]]], [[[1.0, 7.0, 7.0], [6.0, 5.0, 
3.0], [6.0, 2.0, 8.0]], [[5.0, 4.0, 2.0], [2.0, 8.0, 8.0], [2.0, 8.0, 4.0]], [[3.0, 6.0, 1.0], [5.0, 2.0, 1.0], [3.0, 1.0, 
6.0]]], [[[7.0, 7.0, 7.0], [6.0, 6.0, 6.0], [3.0, 4.0, 8.0]], [[1.0, 7.0, 6.0], [7.0, 5.0, 8.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 
3.0], [3.0, 1.0, 3.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      val=numpy.array([[[1.0, 7.0, 4.0], [6.0, 1.0, 8.0], [3.0, 7.0, 7.0]], [[5.0, 4.0, 2.0], [2.0, 4.0, 6.0], [7.0, 3.0, 
6.0]]])
      arg[1:3,1,:]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 1.0], [7.0, 5.0, 5.0], [4.0, 3.0, 1.0]], [[1.0, 6.0, 3.0], [6.0, 7.0, 3.0], [2.0, 2.0, 
7.0]], [[7.0, 7.0, 5.0], [5.0, 6.0, 5.0], [6.0, 5.0, 6.0]]], [[[6.0, 6.0, 8.0], [6.0, 6.0, 8.0], [8.0, 1.0, 3.0]], [[1.0, 7.0, 
4.0], [6.0, 1.0, 8.0], [3.0, 7.0, 7.0]], [[2.0, 5.0, 2.0], [4.0, 1.0, 2.0], [1.0, 1.0, 6.0]]], [[[1.0, 7.0, 7.0], [6.0, 5.0, 
3.0], [6.0, 2.0, 8.0]], [[5.0, 4.0, 2.0], [2.0, 4.0, 6.0], [7.0, 3.0, 6.0]], [[3.0, 6.0, 1.0], [5.0, 2.0, 1.0], [3.0, 1.0, 
6.0]]], [[[7.0, 7.0, 7.0], [6.0, 6.0, 6.0], [3.0, 4.0, 8.0]], [[1.0, 7.0, 6.0], [7.0, 5.0, 8.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 
3.0], [3.0, 1.0, 3.0], [5.0, 4.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPCN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 2.0], [5.0, 2.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 6.0, 6.0], [7.0, 5.0, 1.0], [1.0, 1.0, 
8.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 5.0], [1.0, 8.0, 4.0]]], [[[6.0, 1.0, 8.0], [6.0, 2.0, 3.0], [3.0, 8.0, 2.0]], [[1.0, 4.0, 
6.0], [5.0, 1.0, 3.0], [3.0, 2.0, 1.0]], [[7.0, 6.0, 2.0], [4.0, 5.0, 1.0], [3.0, 3.0, 8.0]]], [[[6.0, 6.0, 3.0], [7.0, 5.0, 
5.0], [7.0, 7.0, 7.0]], [[8.0, 4.0, 2.0], [2.0, 4.0, 3.0], [3.0, 3.0, 5.0]], [[7.0, 2.0, 6.0], [6.0, 2.0, 3.0], [6.0, 3.0, 
1.0]]], [[[3.0, 3.0, 4.0], [5.0, 1.0, 1.0], [7.0, 8.0, 1.0]], [[5.0, 4.0, 5.0], [7.0, 1.0, 4.0], [3.0, 7.0, 2.0]], [[6.0, 7.0, 
3.0], [7.0, 4.0, 2.0], [3.0, 3.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[2.0, 8.0, 1.0], [4.0, 6.0, 8.0], [3.0, 3.0, 5.0]], [[2.0, 8.0, 4.0], [2.0, 5.0, 6.0], [4.0, 7.0, 
1.0]]]),self.functionspace)
      arg[1:3,1,:]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 2.0], [5.0, 2.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 6.0, 6.0], [7.0, 5.0, 1.0], [1.0, 1.0, 
8.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 5.0], [1.0, 8.0, 4.0]]], [[[6.0, 1.0, 8.0], [6.0, 2.0, 3.0], [3.0, 8.0, 2.0]], [[2.0, 8.0, 
1.0], [4.0, 6.0, 8.0], [3.0, 3.0, 5.0]], [[7.0, 6.0, 2.0], [4.0, 5.0, 1.0], [3.0, 3.0, 8.0]]], [[[6.0, 6.0, 3.0], [7.0, 5.0, 
5.0], [7.0, 7.0, 7.0]], [[2.0, 8.0, 4.0], [2.0, 5.0, 6.0], [4.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 2.0, 3.0], [6.0, 3.0, 
1.0]]], [[[3.0, 3.0, 4.0], [5.0, 1.0, 1.0], [7.0, 8.0, 1.0]], [[5.0, 4.0, 5.0], [7.0, 1.0, 4.0], [3.0, 7.0, 2.0]], [[6.0, 7.0, 
3.0], [7.0, 4.0, 2.0], [3.0, 3.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPCN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 7.0], [6.0, 3.0, 3.0], [5.0, 3.0, 1.0]], [[8.0, 8.0, 7.0], [1.0, 4.0, 4.0], [3.0, 2.0, 
4.0]], [[2.0, 3.0, 2.0], [7.0, 1.0, 3.0], [1.0, 6.0, 2.0]]], [[[1.0, 4.0, 5.0], [4.0, 8.0, 5.0], [3.0, 6.0, 7.0]], [[8.0, 7.0, 
6.0], [2.0, 8.0, 2.0], [1.0, 8.0, 8.0]], [[3.0, 3.0, 6.0], [4.0, 4.0, 4.0], [8.0, 7.0, 4.0]]], [[[3.0, 7.0, 5.0], [4.0, 4.0, 
7.0], [1.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [8.0, 1.0, 4.0], [3.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [3.0, 2.0, 2.0], [4.0, 2.0, 
4.0]]], [[[2.0, 8.0, 5.0], [8.0, 5.0, 5.0], [5.0, 2.0, 3.0]], [[8.0, 3.0, 2.0], [4.0, 1.0, 2.0], [7.0, 8.0, 3.0]], [[1.0, 3.0, 
1.0], [3.0, 3.0, 5.0], [2.0, 3.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 6.0, 4.0], [7.0, 8.0, 3.0], [6.0, 4.0, 7.0]], [[1.0, 2.0, 1.0], [5.0, 5.0, 1.0], [1.0, 
7.0, 2.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 4.0], [4.0, 3.0, 7.0], [1.0, 3.0, 3.0]], [[2.0, 7.0, 1.0], [5.0, 2.0, 4.0], 
[7.0, 2.0, 4.0]]])
      arg[1:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [6.0, 3.0, 3.0], [5.0, 3.0, 1.0]], [[8.0, 8.0, 7.0], [1.0, 4.0, 4.0], [3.0, 
2.0, 4.0]], [[2.0, 3.0, 2.0], [7.0, 1.0, 3.0], [1.0, 6.0, 2.0]]], [[[1.0, 4.0, 5.0], [4.0, 8.0, 5.0], [3.0, 6.0, 7.0]], [[6.0, 
6.0, 4.0], [7.0, 8.0, 3.0], [6.0, 4.0, 7.0]], [[3.0, 3.0, 6.0], [4.0, 4.0, 4.0], [8.0, 7.0, 4.0]]], [[[3.0, 7.0, 5.0], [4.0, 
4.0, 7.0], [1.0, 6.0, 5.0]], [[1.0, 2.0, 1.0], [5.0, 5.0, 1.0], [1.0, 7.0, 2.0]], [[5.0, 7.0, 6.0], [3.0, 2.0, 2.0], [4.0, 2.0, 
4.0]]], [[[2.0, 8.0, 5.0], [8.0, 5.0, 5.0], [5.0, 2.0, 3.0]], [[8.0, 3.0, 2.0], [4.0, 1.0, 2.0], [7.0, 8.0, 3.0]], [[1.0, 3.0, 
1.0], [3.0, 3.0, 5.0], [2.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 7.0], [6.0, 3.0, 3.0], [5.0, 3.0, 1.0]], 
[[8.0, 8.0, 7.0], [1.0, 4.0, 4.0], [3.0, 2.0, 4.0]], [[2.0, 3.0, 2.0], [7.0, 1.0, 3.0], [1.0, 6.0, 2.0]]], [[[1.0, 4.0, 5.0], 
[4.0, 8.0, 5.0], [3.0, 6.0, 7.0]], [[2.0, 7.0, 4.0], [4.0, 3.0, 7.0], [1.0, 3.0, 3.0]], [[3.0, 3.0, 6.0], [4.0, 4.0, 4.0], 
[8.0, 7.0, 4.0]]], [[[3.0, 7.0, 5.0], [4.0, 4.0, 7.0], [1.0, 6.0, 5.0]], [[2.0, 7.0, 1.0], [5.0, 2.0, 4.0], [7.0, 2.0, 4.0]], 
[[5.0, 7.0, 6.0], [3.0, 2.0, 2.0], [4.0, 2.0, 4.0]]], [[[2.0, 8.0, 5.0], [8.0, 5.0, 5.0], [5.0, 2.0, 3.0]], [[8.0, 3.0, 2.0], 
[4.0, 1.0, 2.0], [7.0, 8.0, 3.0]], [[1.0, 3.0, 1.0], [3.0, 3.0, 5.0], [2.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CLRN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 1.0], [1.0, 4.0, 6.0], [6.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], [5.0, 8.0, 8.0], [6.0, 5.0, 
4.0]], [[1.0, 7.0, 3.0], [4.0, 4.0, 8.0], [4.0, 4.0, 5.0]]], [[[1.0, 4.0, 8.0], [8.0, 5.0, 8.0], [3.0, 5.0, 7.0]], [[8.0, 7.0, 
4.0], [8.0, 3.0, 4.0], [3.0, 3.0, 3.0]], [[7.0, 7.0, 4.0], [8.0, 4.0, 8.0], [4.0, 8.0, 1.0]]], [[[7.0, 2.0, 8.0], [6.0, 5.0, 
4.0], [5.0, 7.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 3.0, 5.0], [1.0, 4.0, 6.0]], [[5.0, 6.0, 7.0], [6.0, 5.0, 7.0], [8.0, 8.0, 
8.0]]], [[[6.0, 6.0, 1.0], [5.0, 1.0, 3.0], [8.0, 7.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 7.0, 3.0], [8.0, 1.0, 4.0]], [[8.0, 4.0, 
7.0], [1.0, 4.0, 6.0], [3.0, 3.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[8.0, 5.0, 4.0]]], [[[3.0, 5.0, 2.0]]], [[[1.0, 1.0, 7.0]]], [[[6.0, 2.0, 2.0]]]])
      arg[:,2:,1:2]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 1.0], [1.0, 4.0, 6.0], [6.0, 4.0, 6.0]], [[3.0, 8.0, 7.0], [5.0, 8.0, 8.0], [6.0, 5.0, 
4.0]], [[1.0, 7.0, 3.0], [8.0, 5.0, 4.0], [4.0, 4.0, 5.0]]], [[[1.0, 4.0, 8.0], [8.0, 5.0, 8.0], [3.0, 5.0, 7.0]], [[8.0, 7.0, 
4.0], [8.0, 3.0, 4.0], [3.0, 3.0, 3.0]], [[7.0, 7.0, 4.0], [3.0, 5.0, 2.0], [4.0, 8.0, 1.0]]], [[[7.0, 2.0, 8.0], [6.0, 5.0, 
4.0], [5.0, 7.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 3.0, 5.0], [1.0, 4.0, 6.0]], [[5.0, 6.0, 7.0], [1.0, 1.0, 7.0], [8.0, 8.0, 
8.0]]], [[[6.0, 6.0, 1.0], [5.0, 1.0, 3.0], [8.0, 7.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 7.0, 3.0], [8.0, 1.0, 4.0]], [[8.0, 4.0, 
7.0], [6.0, 2.0, 2.0], [3.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CLRN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 2.0], [5.0, 7.0, 7.0], [4.0, 8.0, 8.0]], [[2.0, 4.0, 5.0], [6.0, 3.0, 4.0], [7.0, 1.0, 
2.0]], [[6.0, 4.0, 5.0], [4.0, 8.0, 5.0], [6.0, 4.0, 1.0]]], [[[2.0, 8.0, 4.0], [5.0, 6.0, 6.0], [6.0, 6.0, 4.0]], [[2.0, 1.0, 
8.0], [2.0, 6.0, 6.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 4.0], [5.0, 8.0, 6.0], [8.0, 5.0, 4.0]]], [[[2.0, 7.0, 5.0], [8.0, 3.0, 
4.0], [3.0, 8.0, 3.0]], [[7.0, 4.0, 6.0], [3.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 6.0, 8.0], [8.0, 2.0, 7.0], [2.0, 8.0, 
4.0]]], [[[7.0, 1.0, 8.0], [1.0, 8.0, 5.0], [7.0, 1.0, 6.0]], [[3.0, 1.0, 2.0], [3.0, 2.0, 4.0], [8.0, 8.0, 5.0]], [[3.0, 6.0, 
7.0], [8.0, 7.0, 8.0], [7.0, 6.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 3.0, 7.0]]], [[[6.0, 2.0, 5.0]]], [[[4.0, 3.0, 7.0]]], [[[4.0, 3.0, 
5.0]]]]),self.functionspace)
      arg[:,2:,1:2]=val
      ref=Data(numpy.array([[[[8.0, 6.0, 2.0], [5.0, 7.0, 7.0], [4.0, 8.0, 8.0]], [[2.0, 4.0, 5.0], [6.0, 3.0, 4.0], [7.0, 1.0, 
2.0]], [[6.0, 4.0, 5.0], [5.0, 3.0, 7.0], [6.0, 4.0, 1.0]]], [[[2.0, 8.0, 4.0], [5.0, 6.0, 6.0], [6.0, 6.0, 4.0]], [[2.0, 1.0, 
8.0], [2.0, 6.0, 6.0], [8.0, 7.0, 8.0]], [[5.0, 8.0, 4.0], [6.0, 2.0, 5.0], [8.0, 5.0, 4.0]]], [[[2.0, 7.0, 5.0], [8.0, 3.0, 
4.0], [3.0, 8.0, 3.0]], [[7.0, 4.0, 6.0], [3.0, 7.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 6.0, 8.0], [4.0, 3.0, 7.0], [2.0, 8.0, 
4.0]]], [[[7.0, 1.0, 8.0], [1.0, 8.0, 5.0], [7.0, 1.0, 6.0]], [[3.0, 1.0, 2.0], [3.0, 2.0, 4.0], [8.0, 8.0, 5.0]], [[3.0, 6.0, 
7.0], [4.0, 3.0, 5.0], [7.0, 6.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CLRN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 3.0]], [[7.0, 4.0, 4.0], [7.0, 4.0, 5.0], [8.0, 8.0, 
3.0]], [[6.0, 4.0, 4.0], [7.0, 7.0, 7.0], [4.0, 7.0, 6.0]]], [[[5.0, 8.0, 4.0], [5.0, 4.0, 3.0], [4.0, 3.0, 3.0]], [[2.0, 4.0, 
6.0], [1.0, 3.0, 3.0], [5.0, 2.0, 3.0]], [[5.0, 1.0, 7.0], [6.0, 2.0, 3.0], [2.0, 7.0, 4.0]]], [[[7.0, 2.0, 5.0], [2.0, 5.0, 
3.0], [7.0, 4.0, 6.0]], [[3.0, 2.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 2.0]], [[1.0, 5.0, 5.0], [3.0, 3.0, 8.0], [8.0, 2.0, 
8.0]]], [[[3.0, 5.0, 6.0], [2.0, 3.0, 3.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 3.0], [3.0, 3.0, 5.0], [7.0, 7.0, 6.0]], [[8.0, 7.0, 
7.0], [3.0, 5.0, 7.0], [8.0, 4.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 8.0, 5.0]]], [[[8.0, 1.0, 4.0]]], [[[2.0, 2.0, 1.0]]], [[[8.0, 3.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 3.0, 7.0]]], [[[6.0, 6.0, 8.0]]], [[[6.0, 6.0, 8.0]]], [[[8.0, 4.0, 3.0]]]])
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 3.0]], [[7.0, 4.0, 4.0], [7.0, 4.0, 5.0], [8.0, 
8.0, 3.0]], [[6.0, 4.0, 4.0], [1.0, 8.0, 5.0], [4.0, 7.0, 6.0]]], [[[5.0, 8.0, 4.0], [5.0, 4.0, 3.0], [4.0, 3.0, 3.0]], [[2.0, 
4.0, 6.0], [1.0, 3.0, 3.0], [5.0, 2.0, 3.0]], [[5.0, 1.0, 7.0], [8.0, 1.0, 4.0], [2.0, 7.0, 4.0]]], [[[7.0, 2.0, 5.0], [2.0, 
5.0, 3.0], [7.0, 4.0, 6.0]], [[3.0, 2.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 2.0]], [[1.0, 5.0, 5.0], [2.0, 2.0, 1.0], [8.0, 2.0, 
8.0]]], [[[3.0, 5.0, 6.0], [2.0, 3.0, 3.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 3.0], [3.0, 3.0, 5.0], [7.0, 7.0, 6.0]], [[8.0, 7.0, 
7.0], [8.0, 3.0, 4.0], [8.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 3.0]], 
[[7.0, 4.0, 4.0], [7.0, 4.0, 5.0], [8.0, 8.0, 3.0]], [[6.0, 4.0, 4.0], [5.0, 3.0, 7.0], [4.0, 7.0, 6.0]]], [[[5.0, 8.0, 4.0], 
[5.0, 4.0, 3.0], [4.0, 3.0, 3.0]], [[2.0, 4.0, 6.0], [1.0, 3.0, 3.0], [5.0, 2.0, 3.0]], [[5.0, 1.0, 7.0], [6.0, 6.0, 8.0], 
[2.0, 7.0, 4.0]]], [[[7.0, 2.0, 5.0], [2.0, 5.0, 3.0], [7.0, 4.0, 6.0]], [[3.0, 2.0, 3.0], [4.0, 8.0, 2.0], [1.0, 2.0, 2.0]], 
[[1.0, 5.0, 5.0], [6.0, 6.0, 8.0], [8.0, 2.0, 8.0]]], [[[3.0, 5.0, 6.0], [2.0, 3.0, 3.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 3.0], 
[3.0, 3.0, 5.0], [7.0, 7.0, 6.0]], [[8.0, 7.0, 7.0], [8.0, 4.0, 3.0], [8.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LCPN(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 3.0], [5.0, 8.0, 8.0], [1.0, 3.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 4.0], [8.0, 1.0, 
6.0]], [[4.0, 3.0, 5.0], [2.0, 8.0, 8.0], [3.0, 7.0, 6.0]]], [[[3.0, 8.0, 8.0], [7.0, 6.0, 2.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 
5.0], [8.0, 4.0, 3.0], [7.0, 4.0, 3.0]], [[2.0, 8.0, 8.0], [7.0, 5.0, 2.0], [2.0, 5.0, 4.0]]], [[[6.0, 1.0, 8.0], [6.0, 5.0, 
7.0], [4.0, 3.0, 3.0]], [[8.0, 5.0, 8.0], [1.0, 3.0, 7.0], [5.0, 1.0, 3.0]], [[6.0, 1.0, 3.0], [7.0, 2.0, 8.0], [8.0, 3.0, 
6.0]]], [[[6.0, 1.0, 1.0], [4.0, 6.0, 7.0], [4.0, 8.0, 7.0]], [[7.0, 6.0, 5.0], [7.0, 5.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 3.0, 
8.0], [7.0, 6.0, 3.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[8.0, 6.0, 4.0], [7.0, 4.0, 5.0], [4.0, 4.0, 4.0]], [[2.0, 6.0, 3.0], [7.0, 7.0, 4.0], [6.0, 7.0, 
4.0]]])
      arg[2:,:,1]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 3.0], [5.0, 8.0, 8.0], [1.0, 3.0, 2.0]], [[2.0, 6.0, 2.0], [3.0, 6.0, 4.0], [8.0, 1.0, 
6.0]], [[4.0, 3.0, 5.0], [2.0, 8.0, 8.0], [3.0, 7.0, 6.0]]], [[[3.0, 8.0, 8.0], [7.0, 6.0, 2.0], [1.0, 2.0, 3.0]], [[4.0, 2.0, 
5.0], [8.0, 4.0, 3.0], [7.0, 4.0, 3.0]], [[2.0, 8.0, 8.0], [7.0, 5.0, 2.0], [2.0, 5.0, 4.0]]], [[[6.0, 1.0, 8.0], [8.0, 6.0, 
4.0], [4.0, 3.0, 3.0]], [[8.0, 5.0, 8.0], [7.0, 4.0, 5.0], [5.0, 1.0, 3.0]], [[6.0, 1.0, 3.0], [4.0, 4.0, 4.0], [8.0, 3.0, 
6.0]]], [[[6.0, 1.0, 1.0], [2.0, 6.0, 3.0], [4.0, 8.0, 7.0]], [[7.0, 6.0, 5.0], [7.0, 7.0, 4.0], [6.0, 4.0, 5.0]], [[8.0, 3.0, 
8.0], [6.0, 7.0, 4.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LCPN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 8.0], [1.0, 8.0, 7.0], [4.0, 2.0, 6.0]], [[2.0, 2.0, 1.0], [5.0, 5.0, 2.0], [2.0, 7.0, 
8.0]], [[5.0, 6.0, 6.0], [7.0, 7.0, 7.0], [4.0, 2.0, 1.0]]], [[[8.0, 7.0, 8.0], [3.0, 3.0, 6.0], [4.0, 6.0, 8.0]], [[3.0, 3.0, 
4.0], [6.0, 1.0, 4.0], [3.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [2.0, 6.0, 8.0], [7.0, 3.0, 1.0]]], [[[7.0, 6.0, 1.0], [4.0, 1.0, 
3.0], [6.0, 4.0, 2.0]], [[3.0, 2.0, 1.0], [8.0, 6.0, 5.0], [5.0, 1.0, 3.0]], [[2.0, 7.0, 5.0], [4.0, 6.0, 3.0], [8.0, 2.0, 
4.0]]], [[[2.0, 7.0, 7.0], [2.0, 4.0, 4.0], [2.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [3.0, 8.0, 3.0], [4.0, 5.0, 4.0]], [[2.0, 1.0, 
7.0], [7.0, 1.0, 8.0], [7.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 4.0, 7.0], [6.0, 7.0, 2.0], [7.0, 6.0, 8.0]], [[2.0, 7.0, 3.0], [6.0, 8.0, 7.0], [8.0, 8.0, 
6.0]]]),self.functionspace)
      arg[2:,:,1]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 8.0], [1.0, 8.0, 7.0], [4.0, 2.0, 6.0]], [[2.0, 2.0, 1.0], [5.0, 5.0, 2.0], [2.0, 7.0, 
8.0]], [[5.0, 6.0, 6.0], [7.0, 7.0, 7.0], [4.0, 2.0, 1.0]]], [[[8.0, 7.0, 8.0], [3.0, 3.0, 6.0], [4.0, 6.0, 8.0]], [[3.0, 3.0, 
4.0], [6.0, 1.0, 4.0], [3.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [2.0, 6.0, 8.0], [7.0, 3.0, 1.0]]], [[[7.0, 6.0, 1.0], [4.0, 4.0, 
7.0], [6.0, 4.0, 2.0]], [[3.0, 2.0, 1.0], [6.0, 7.0, 2.0], [5.0, 1.0, 3.0]], [[2.0, 7.0, 5.0], [7.0, 6.0, 8.0], [8.0, 2.0, 
4.0]]], [[[2.0, 7.0, 7.0], [2.0, 7.0, 3.0], [2.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 8.0, 7.0], [4.0, 5.0, 4.0]], [[2.0, 1.0, 
7.0], [8.0, 8.0, 6.0], [7.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LCPN(self):
      arg=Data(numpy.array([[[[4.0, 3.0, 1.0], [1.0, 5.0, 8.0], [4.0, 3.0, 2.0]], [[7.0, 8.0, 4.0], [5.0, 8.0, 7.0], [8.0, 6.0, 
1.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 4.0], [3.0, 6.0, 6.0]]], [[[2.0, 7.0, 5.0], [7.0, 5.0, 2.0], [8.0, 4.0, 3.0]], [[6.0, 7.0, 
2.0], [2.0, 7.0, 6.0], [3.0, 8.0, 4.0]], [[3.0, 8.0, 6.0], [7.0, 2.0, 4.0], [1.0, 3.0, 8.0]]], [[[5.0, 6.0, 6.0], [8.0, 6.0, 
5.0], [7.0, 6.0, 2.0]], [[1.0, 3.0, 5.0], [3.0, 1.0, 7.0], [3.0, 7.0, 8.0]], [[3.0, 6.0, 2.0], [5.0, 6.0, 7.0], [3.0, 3.0, 
7.0]]], [[[6.0, 8.0, 5.0], [3.0, 1.0, 4.0], [1.0, 2.0, 4.0]], [[3.0, 2.0, 8.0], [8.0, 4.0, 5.0], [5.0, 3.0, 4.0]], [[4.0, 4.0, 
3.0], [4.0, 4.0, 4.0], [6.0, 6.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 7.0, 5.0], [8.0, 2.0, 8.0], [2.0, 3.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 2.0, 7.0], [8.0, 
1.0, 3.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 5.0], [4.0, 2.0, 7.0], [6.0, 3.0, 7.0]], [[7.0, 1.0, 5.0], [6.0, 3.0, 8.0], 
[7.0, 7.0, 6.0]]])
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 1.0], [1.0, 5.0, 8.0], [4.0, 3.0, 2.0]], [[7.0, 8.0, 4.0], [5.0, 8.0, 7.0], [8.0, 
6.0, 1.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 4.0], [3.0, 6.0, 6.0]]], [[[2.0, 7.0, 5.0], [7.0, 5.0, 2.0], [8.0, 4.0, 3.0]], [[6.0, 
7.0, 2.0], [2.0, 7.0, 6.0], [3.0, 8.0, 4.0]], [[3.0, 8.0, 6.0], [7.0, 2.0, 4.0], [1.0, 3.0, 8.0]]], [[[5.0, 6.0, 6.0], [8.0, 
7.0, 5.0], [7.0, 6.0, 2.0]], [[1.0, 3.0, 5.0], [8.0, 2.0, 8.0], [3.0, 7.0, 8.0]], [[3.0, 6.0, 2.0], [2.0, 3.0, 6.0], [3.0, 3.0, 
7.0]]], [[[6.0, 8.0, 5.0], [1.0, 7.0, 7.0], [1.0, 2.0, 4.0]], [[3.0, 2.0, 8.0], [8.0, 2.0, 7.0], [5.0, 3.0, 4.0]], [[4.0, 4.0, 
3.0], [8.0, 1.0, 3.0], [6.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 1.0], [1.0, 5.0, 8.0], [4.0, 3.0, 2.0]], 
[[7.0, 8.0, 4.0], [5.0, 8.0, 7.0], [8.0, 6.0, 1.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 4.0], [3.0, 6.0, 6.0]]], [[[2.0, 7.0, 5.0], 
[7.0, 5.0, 2.0], [8.0, 4.0, 3.0]], [[6.0, 7.0, 2.0], [2.0, 7.0, 6.0], [3.0, 8.0, 4.0]], [[3.0, 8.0, 6.0], [7.0, 2.0, 4.0], 
[1.0, 3.0, 8.0]]], [[[5.0, 6.0, 6.0], [7.0, 3.0, 5.0], [7.0, 6.0, 2.0]], [[1.0, 3.0, 5.0], [4.0, 2.0, 7.0], [3.0, 7.0, 8.0]], 
[[3.0, 6.0, 2.0], [6.0, 3.0, 7.0], [3.0, 3.0, 7.0]]], [[[6.0, 8.0, 5.0], [7.0, 1.0, 5.0], [1.0, 2.0, 4.0]], [[3.0, 2.0, 8.0], 
[6.0, 3.0, 8.0], [5.0, 3.0, 4.0]], [[4.0, 4.0, 3.0], [7.0, 7.0, 6.0], [6.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_ULUN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 3.0], [6.0, 3.0, 1.0], [4.0, 3.0, 7.0]], [[6.0, 7.0, 7.0], [2.0, 2.0, 2.0], [7.0, 1.0, 
8.0]], [[2.0, 6.0, 2.0], [6.0, 7.0, 8.0], [2.0, 4.0, 3.0]]], [[[1.0, 2.0, 2.0], [8.0, 6.0, 4.0], [2.0, 6.0, 2.0]], [[6.0, 8.0, 
2.0], [5.0, 3.0, 4.0], [2.0, 4.0, 6.0]], [[1.0, 3.0, 4.0], [5.0, 1.0, 4.0], [6.0, 8.0, 5.0]]], [[[5.0, 8.0, 5.0], [3.0, 7.0, 
5.0], [1.0, 4.0, 3.0]], [[6.0, 7.0, 7.0], [8.0, 3.0, 1.0], [5.0, 5.0, 8.0]], [[3.0, 8.0, 2.0], [5.0, 6.0, 3.0], [4.0, 4.0, 
2.0]]], [[[5.0, 7.0, 5.0], [6.0, 7.0, 8.0], [3.0, 5.0, 1.0]], [[1.0, 4.0, 4.0], [7.0, 4.0, 7.0], [2.0, 4.0, 8.0]], [[7.0, 1.0, 
5.0], [2.0, 7.0, 6.0], [6.0, 2.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 5.0, 4.0], [3.0, 7.0, 5.0]]], [[[2.0, 2.0, 7.0], [8.0, 4.0, 7.0]]], [[[6.0, 4.0, 2.0], [6.0, 
5.0, 1.0]]]])
      arg[:3,2:,:2]=val
      ref=Data(numpy.array([[[[8.0, 3.0, 3.0], [6.0, 3.0, 1.0], [4.0, 3.0, 7.0]], [[6.0, 7.0, 7.0], [2.0, 2.0, 2.0], [7.0, 1.0, 
8.0]], [[5.0, 5.0, 4.0], [3.0, 7.0, 5.0], [2.0, 4.0, 3.0]]], [[[1.0, 2.0, 2.0], [8.0, 6.0, 4.0], [2.0, 6.0, 2.0]], [[6.0, 8.0, 
2.0], [5.0, 3.0, 4.0], [2.0, 4.0, 6.0]], [[2.0, 2.0, 7.0], [8.0, 4.0, 7.0], [6.0, 8.0, 5.0]]], [[[5.0, 8.0, 5.0], [3.0, 7.0, 
5.0], [1.0, 4.0, 3.0]], [[6.0, 7.0, 7.0], [8.0, 3.0, 1.0], [5.0, 5.0, 8.0]], [[6.0, 4.0, 2.0], [6.0, 5.0, 1.0], [4.0, 4.0, 
2.0]]], [[[5.0, 7.0, 5.0], [6.0, 7.0, 8.0], [3.0, 5.0, 1.0]], [[1.0, 4.0, 4.0], [7.0, 4.0, 7.0], [2.0, 4.0, 8.0]], [[7.0, 1.0, 
5.0], [2.0, 7.0, 6.0], [6.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_ULUN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [8.0, 7.0, 1.0], [2.0, 2.0, 3.0]], [[3.0, 2.0, 2.0], [7.0, 5.0, 1.0], [1.0, 4.0, 
2.0]], [[7.0, 3.0, 1.0], [7.0, 1.0, 5.0], [2.0, 3.0, 8.0]]], [[[2.0, 2.0, 2.0], [2.0, 5.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 1.0, 
5.0], [5.0, 4.0, 5.0], [2.0, 1.0, 5.0]], [[2.0, 8.0, 2.0], [8.0, 4.0, 4.0], [1.0, 4.0, 3.0]]], [[[2.0, 5.0, 3.0], [5.0, 1.0, 
2.0], [7.0, 4.0, 6.0]], [[3.0, 7.0, 3.0], [2.0, 4.0, 3.0], [3.0, 4.0, 2.0]], [[5.0, 7.0, 5.0], [6.0, 2.0, 2.0], [6.0, 8.0, 
2.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[4.0, 6.0, 8.0], [5.0, 7.0, 1.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 
8.0], [6.0, 1.0, 4.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0, 3.0], [8.0, 1.0, 3.0]]], [[[4.0, 2.0, 7.0], [2.0, 4.0, 8.0]]], [[[2.0, 3.0, 7.0], 
[1.0, 2.0, 2.0]]]]),self.functionspace)
      arg[:3,2:,:2]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 6.0], [8.0, 7.0, 1.0], [2.0, 2.0, 3.0]], [[3.0, 2.0, 2.0], [7.0, 5.0, 1.0], [1.0, 4.0, 
2.0]], [[5.0, 1.0, 3.0], [8.0, 1.0, 3.0], [2.0, 3.0, 8.0]]], [[[2.0, 2.0, 2.0], [2.0, 5.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 1.0, 
5.0], [5.0, 4.0, 5.0], [2.0, 1.0, 5.0]], [[4.0, 2.0, 7.0], [2.0, 4.0, 8.0], [1.0, 4.0, 3.0]]], [[[2.0, 5.0, 3.0], [5.0, 1.0, 
2.0], [7.0, 4.0, 6.0]], [[3.0, 7.0, 3.0], [2.0, 4.0, 3.0], [3.0, 4.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 2.0, 2.0], [6.0, 8.0, 
2.0]]], [[[8.0, 1.0, 2.0], [3.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[4.0, 6.0, 8.0], [5.0, 7.0, 1.0], [5.0, 8.0, 3.0]], [[5.0, 5.0, 
8.0], [6.0, 1.0, 4.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_ULUN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 7.0], [3.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[2.0, 6.0, 1.0], [3.0, 5.0, 3.0], [2.0, 5.0, 
5.0]], [[4.0, 5.0, 8.0], [1.0, 8.0, 8.0], [4.0, 3.0, 8.0]]], [[[1.0, 8.0, 5.0], [2.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[8.0, 5.0, 
6.0], [7.0, 2.0, 5.0], [3.0, 6.0, 7.0]], [[1.0, 5.0, 7.0], [1.0, 4.0, 6.0], [2.0, 3.0, 7.0]]], [[[6.0, 1.0, 6.0], [5.0, 5.0, 
5.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 5.0], [2.0, 2.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 2.0], [7.0, 6.0, 3.0], [4.0, 6.0, 
2.0]]], [[[1.0, 6.0, 8.0], [5.0, 7.0, 4.0], [5.0, 8.0, 2.0]], [[2.0, 6.0, 8.0], [6.0, 6.0, 5.0], [3.0, 6.0, 5.0]], [[5.0, 6.0, 
4.0], [6.0, 2.0, 7.0], [6.0, 7.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 1.0, 2.0], [6.0, 7.0, 5.0]]], [[[2.0, 5.0, 3.0], [7.0, 2.0, 8.0]]], [[[8.0, 5.0, 7.0], 
[7.0, 3.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 6.0, 4.0], [7.0, 7.0, 2.0]]], [[[1.0, 7.0, 6.0], [5.0, 1.0, 2.0]]], 
[[[2.0, 4.0, 2.0], [5.0, 3.0, 3.0]]]])
      arg[:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 7.0], [3.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[2.0, 6.0, 1.0], [3.0, 5.0, 3.0], [2.0, 
5.0, 5.0]], [[2.0, 1.0, 2.0], [6.0, 7.0, 5.0], [4.0, 3.0, 8.0]]], [[[1.0, 8.0, 5.0], [2.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[8.0, 
5.0, 6.0], [7.0, 2.0, 5.0], [3.0, 6.0, 7.0]], [[2.0, 5.0, 3.0], [7.0, 2.0, 8.0], [2.0, 3.0, 7.0]]], [[[6.0, 1.0, 6.0], [5.0, 
5.0, 5.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 5.0], [2.0, 2.0, 6.0], [6.0, 2.0, 5.0]], [[8.0, 5.0, 7.0], [7.0, 3.0, 4.0], [4.0, 6.0, 
2.0]]], [[[1.0, 6.0, 8.0], [5.0, 7.0, 4.0], [5.0, 8.0, 2.0]], [[2.0, 6.0, 8.0], [6.0, 6.0, 5.0], [3.0, 6.0, 5.0]], [[5.0, 6.0, 
4.0], [6.0, 2.0, 7.0], [6.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 7.0], [3.0, 4.0, 4.0], [1.0, 4.0, 5.0]], 
[[2.0, 6.0, 1.0], [3.0, 5.0, 3.0], [2.0, 5.0, 5.0]], [[8.0, 6.0, 4.0], [7.0, 7.0, 2.0], [4.0, 3.0, 8.0]]], [[[1.0, 8.0, 5.0], 
[2.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[8.0, 5.0, 6.0], [7.0, 2.0, 5.0], [3.0, 6.0, 7.0]], [[1.0, 7.0, 6.0], [5.0, 1.0, 2.0], 
[2.0, 3.0, 7.0]]], [[[6.0, 1.0, 6.0], [5.0, 5.0, 5.0], [8.0, 5.0, 7.0]], [[5.0, 7.0, 5.0], [2.0, 2.0, 6.0], [6.0, 2.0, 5.0]], 
[[2.0, 4.0, 2.0], [5.0, 3.0, 3.0], [4.0, 6.0, 2.0]]], [[[1.0, 6.0, 8.0], [5.0, 7.0, 4.0], [5.0, 8.0, 2.0]], [[2.0, 6.0, 8.0], 
[6.0, 6.0, 5.0], [3.0, 6.0, 5.0]], [[5.0, 6.0, 4.0], [6.0, 2.0, 7.0], [6.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_URUN(self):
      arg=Data(numpy.array([[[[4.0, 8.0, 3.0], [8.0, 1.0, 5.0], [2.0, 8.0, 6.0]], [[5.0, 3.0, 2.0], [4.0, 1.0, 3.0], [4.0, 4.0, 
5.0]], [[5.0, 1.0, 4.0], [8.0, 8.0, 6.0], [7.0, 2.0, 8.0]]], [[[4.0, 2.0, 3.0], [6.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[3.0, 1.0, 
7.0], [2.0, 8.0, 7.0], [2.0, 6.0, 6.0]], [[2.0, 6.0, 1.0], [3.0, 4.0, 1.0], [1.0, 8.0, 3.0]]], [[[1.0, 1.0, 5.0], [3.0, 3.0, 
8.0], [3.0, 1.0, 7.0]], [[6.0, 8.0, 6.0], [7.0, 2.0, 8.0], [6.0, 3.0, 4.0]], [[6.0, 5.0, 1.0], [1.0, 1.0, 4.0], [1.0, 8.0, 
8.0]]], [[[6.0, 4.0, 2.0], [2.0, 5.0, 5.0], [4.0, 2.0, 1.0]], [[7.0, 4.0, 4.0], [7.0, 6.0, 4.0], [2.0, 6.0, 7.0]], [[4.0, 4.0, 
5.0], [2.0, 5.0, 6.0], [5.0, 5.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 8.0, 2.0], [8.0, 6.0, 2.0]]], [[[7.0, 8.0, 5.0], [3.0, 1.0, 7.0]]], [[[1.0, 1.0, 1.0], [6.0, 
4.0, 8.0]]]])
      arg[:3,1:2,:2]=val
      ref=Data(numpy.array([[[[4.0, 8.0, 3.0], [8.0, 1.0, 5.0], [2.0, 8.0, 6.0]], [[2.0, 8.0, 2.0], [8.0, 6.0, 2.0], [4.0, 4.0, 
5.0]], [[5.0, 1.0, 4.0], [8.0, 8.0, 6.0], [7.0, 2.0, 8.0]]], [[[4.0, 2.0, 3.0], [6.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[7.0, 8.0, 
5.0], [3.0, 1.0, 7.0], [2.0, 6.0, 6.0]], [[2.0, 6.0, 1.0], [3.0, 4.0, 1.0], [1.0, 8.0, 3.0]]], [[[1.0, 1.0, 5.0], [3.0, 3.0, 
8.0], [3.0, 1.0, 7.0]], [[1.0, 1.0, 1.0], [6.0, 4.0, 8.0], [6.0, 3.0, 4.0]], [[6.0, 5.0, 1.0], [1.0, 1.0, 4.0], [1.0, 8.0, 
8.0]]], [[[6.0, 4.0, 2.0], [2.0, 5.0, 5.0], [4.0, 2.0, 1.0]], [[7.0, 4.0, 4.0], [7.0, 6.0, 4.0], [2.0, 6.0, 7.0]], [[4.0, 4.0, 
5.0], [2.0, 5.0, 6.0], [5.0, 5.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_URUN(self):
      arg=Data(numpy.array([[[[4.0, 4.0, 1.0], [6.0, 1.0, 7.0], [4.0, 6.0, 3.0]], [[8.0, 5.0, 6.0], [7.0, 7.0, 5.0], [7.0, 4.0, 
3.0]], [[7.0, 2.0, 4.0], [3.0, 4.0, 6.0], [2.0, 1.0, 5.0]]], [[[7.0, 6.0, 1.0], [5.0, 5.0, 7.0], [6.0, 6.0, 2.0]], [[6.0, 1.0, 
1.0], [6.0, 2.0, 6.0], [4.0, 1.0, 2.0]], [[8.0, 5.0, 2.0], [8.0, 4.0, 3.0], [6.0, 5.0, 7.0]]], [[[5.0, 1.0, 3.0], [5.0, 7.0, 
1.0], [7.0, 4.0, 2.0]], [[6.0, 6.0, 2.0], [5.0, 5.0, 1.0], [4.0, 5.0, 4.0]], [[2.0, 5.0, 2.0], [7.0, 6.0, 1.0], [3.0, 6.0, 
1.0]]], [[[8.0, 8.0, 8.0], [4.0, 2.0, 1.0], [3.0, 1.0, 1.0]], [[4.0, 1.0, 6.0], [1.0, 4.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 3.0, 
4.0], [8.0, 7.0, 5.0], [7.0, 4.0, 8.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 3.0, 4.0], [4.0, 4.0, 8.0]]], [[[6.0, 8.0, 6.0], [2.0, 6.0, 7.0]]], [[[2.0, 4.0, 7.0], 
[5.0, 3.0, 4.0]]]]),self.functionspace)
      arg[:3,1:2,:2]=val
      ref=Data(numpy.array([[[[4.0, 4.0, 1.0], [6.0, 1.0, 7.0], [4.0, 6.0, 3.0]], [[2.0, 3.0, 4.0], [4.0, 4.0, 8.0], [7.0, 4.0, 
3.0]], [[7.0, 2.0, 4.0], [3.0, 4.0, 6.0], [2.0, 1.0, 5.0]]], [[[7.0, 6.0, 1.0], [5.0, 5.0, 7.0], [6.0, 6.0, 2.0]], [[6.0, 8.0, 
6.0], [2.0, 6.0, 7.0], [4.0, 1.0, 2.0]], [[8.0, 5.0, 2.0], [8.0, 4.0, 3.0], [6.0, 5.0, 7.0]]], [[[5.0, 1.0, 3.0], [5.0, 7.0, 
1.0], [7.0, 4.0, 2.0]], [[2.0, 4.0, 7.0], [5.0, 3.0, 4.0], [4.0, 5.0, 4.0]], [[2.0, 5.0, 2.0], [7.0, 6.0, 1.0], [3.0, 6.0, 
1.0]]], [[[8.0, 8.0, 8.0], [4.0, 2.0, 1.0], [3.0, 1.0, 1.0]], [[4.0, 1.0, 6.0], [1.0, 4.0, 4.0], [1.0, 2.0, 3.0]], [[4.0, 3.0, 
4.0], [8.0, 7.0, 5.0], [7.0, 4.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_URUN(self):
      arg=Data(numpy.array([[[[7.0, 6.0, 3.0], [7.0, 1.0, 4.0], [5.0, 1.0, 3.0]], [[1.0, 1.0, 5.0], [1.0, 1.0, 1.0], [2.0, 6.0, 
1.0]], [[1.0, 4.0, 1.0], [4.0, 3.0, 4.0], [6.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [7.0, 3.0, 7.0], [4.0, 3.0, 8.0]], [[3.0, 6.0, 
3.0], [3.0, 3.0, 7.0], [5.0, 4.0, 3.0]], [[6.0, 5.0, 8.0], [5.0, 6.0, 5.0], [6.0, 5.0, 7.0]]], [[[6.0, 6.0, 6.0], [8.0, 4.0, 
8.0], [6.0, 1.0, 1.0]], [[5.0, 3.0, 4.0], [8.0, 2.0, 5.0], [2.0, 8.0, 1.0]], [[7.0, 7.0, 1.0], [8.0, 5.0, 6.0], [5.0, 5.0, 
8.0]]], [[[1.0, 5.0, 8.0], [5.0, 8.0, 1.0], [8.0, 1.0, 3.0]], [[2.0, 1.0, 4.0], [1.0, 4.0, 8.0], [8.0, 7.0, 1.0]], [[2.0, 7.0, 
1.0], [7.0, 7.0, 1.0], [5.0, 7.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 8.0, 5.0], [8.0, 4.0, 5.0]]], [[[6.0, 7.0, 7.0], [2.0, 6.0, 8.0]]], [[[2.0, 7.0, 7.0], 
[5.0, 8.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 2.0, 6.0], [1.0, 8.0, 6.0]]], [[[7.0, 1.0, 2.0], [1.0, 1.0, 4.0]]], 
[[[7.0, 4.0, 4.0], [7.0, 4.0, 5.0]]]])
      arg[:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 3.0], [7.0, 1.0, 4.0], [5.0, 1.0, 3.0]], [[5.0, 8.0, 5.0], [8.0, 4.0, 5.0], [2.0, 
6.0, 1.0]], [[1.0, 4.0, 1.0], [4.0, 3.0, 4.0], [6.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [7.0, 3.0, 7.0], [4.0, 3.0, 8.0]], [[6.0, 
7.0, 7.0], [2.0, 6.0, 8.0], [5.0, 4.0, 3.0]], [[6.0, 5.0, 8.0], [5.0, 6.0, 5.0], [6.0, 5.0, 7.0]]], [[[6.0, 6.0, 6.0], [8.0, 
4.0, 8.0], [6.0, 1.0, 1.0]], [[2.0, 7.0, 7.0], [5.0, 8.0, 1.0], [2.0, 8.0, 1.0]], [[7.0, 7.0, 1.0], [8.0, 5.0, 6.0], [5.0, 5.0, 
8.0]]], [[[1.0, 5.0, 8.0], [5.0, 8.0, 1.0], [8.0, 1.0, 3.0]], [[2.0, 1.0, 4.0], [1.0, 4.0, 8.0], [8.0, 7.0, 1.0]], [[2.0, 7.0, 
1.0], [7.0, 7.0, 1.0], [5.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 3.0], [7.0, 1.0, 4.0], [5.0, 1.0, 3.0]], 
[[4.0, 2.0, 6.0], [1.0, 8.0, 6.0], [2.0, 6.0, 1.0]], [[1.0, 4.0, 1.0], [4.0, 3.0, 4.0], [6.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], 
[7.0, 3.0, 7.0], [4.0, 3.0, 8.0]], [[7.0, 1.0, 2.0], [1.0, 1.0, 4.0], [5.0, 4.0, 3.0]], [[6.0, 5.0, 8.0], [5.0, 6.0, 5.0], 
[6.0, 5.0, 7.0]]], [[[6.0, 6.0, 6.0], [8.0, 4.0, 8.0], [6.0, 1.0, 1.0]], [[7.0, 4.0, 4.0], [7.0, 4.0, 5.0], [2.0, 8.0, 1.0]], 
[[7.0, 7.0, 1.0], [8.0, 5.0, 6.0], [5.0, 5.0, 8.0]]], [[[1.0, 5.0, 8.0], [5.0, 8.0, 1.0], [8.0, 1.0, 3.0]], [[2.0, 1.0, 4.0], 
[1.0, 4.0, 8.0], [8.0, 7.0, 1.0]], [[2.0, 7.0, 1.0], [7.0, 7.0, 1.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPPN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 6.0], [2.0, 1.0, 6.0], [7.0, 1.0, 8.0]], [[7.0, 3.0, 5.0], [2.0, 8.0, 4.0], [3.0, 8.0, 
4.0]], [[5.0, 1.0, 8.0], [2.0, 3.0, 8.0], [2.0, 2.0, 8.0]]], [[[3.0, 5.0, 1.0], [7.0, 5.0, 2.0], [7.0, 2.0, 3.0]], [[4.0, 6.0, 
4.0], [3.0, 8.0, 4.0], [8.0, 3.0, 4.0]], [[3.0, 8.0, 8.0], [7.0, 5.0, 6.0], [5.0, 8.0, 6.0]]], [[[3.0, 6.0, 4.0], [6.0, 3.0, 
5.0], [3.0, 7.0, 1.0]], [[5.0, 8.0, 6.0], [2.0, 4.0, 8.0], [3.0, 3.0, 4.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 2.0], [2.0, 3.0, 
2.0]]], [[[5.0, 5.0, 7.0], [8.0, 6.0, 8.0], [2.0, 7.0, 3.0]], [[1.0, 3.0, 2.0], [7.0, 6.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 8.0, 
7.0], [7.0, 8.0, 8.0], [3.0, 2.0, 4.0]]]]),self.functionspace)
      val=numpy.array([4.0, 6.0, 4.0])
      arg[2,1,1]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 6.0], [2.0, 1.0, 6.0], [7.0, 1.0, 8.0]], [[7.0, 3.0, 5.0], [2.0, 8.0, 4.0], [3.0, 8.0, 
4.0]], [[5.0, 1.0, 8.0], [2.0, 3.0, 8.0], [2.0, 2.0, 8.0]]], [[[3.0, 5.0, 1.0], [7.0, 5.0, 2.0], [7.0, 2.0, 3.0]], [[4.0, 6.0, 
4.0], [3.0, 8.0, 4.0], [8.0, 3.0, 4.0]], [[3.0, 8.0, 8.0], [7.0, 5.0, 6.0], [5.0, 8.0, 6.0]]], [[[3.0, 6.0, 4.0], [6.0, 3.0, 
5.0], [3.0, 7.0, 1.0]], [[5.0, 8.0, 6.0], [4.0, 6.0, 4.0], [3.0, 3.0, 4.0]], [[4.0, 8.0, 8.0], [1.0, 5.0, 2.0], [2.0, 3.0, 
2.0]]], [[[5.0, 5.0, 7.0], [8.0, 6.0, 8.0], [2.0, 7.0, 3.0]], [[1.0, 3.0, 2.0], [7.0, 6.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 8.0, 
7.0], [7.0, 8.0, 8.0], [3.0, 2.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPPN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 6.0], [1.0, 1.0, 6.0], [1.0, 1.0, 3.0]], [[2.0, 5.0, 1.0], [2.0, 1.0, 3.0], [6.0, 7.0, 
1.0]], [[7.0, 8.0, 3.0], [1.0, 8.0, 5.0], [5.0, 7.0, 7.0]]], [[[3.0, 7.0, 7.0], [8.0, 7.0, 3.0], [5.0, 6.0, 6.0]], [[2.0, 4.0, 
2.0], [7.0, 7.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 8.0, 5.0], [2.0, 4.0, 8.0], [7.0, 2.0, 2.0]]], [[[2.0, 5.0, 6.0], [5.0, 6.0, 
7.0], [1.0, 7.0, 6.0]], [[6.0, 4.0, 4.0], [3.0, 3.0, 3.0], [2.0, 4.0, 8.0]], [[8.0, 8.0, 5.0], [2.0, 5.0, 2.0], [7.0, 2.0, 
5.0]]], [[[4.0, 7.0, 2.0], [5.0, 3.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 1.0, 7.0], [3.0, 1.0, 3.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 
2.0], [6.0, 2.0, 6.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([8.0, 2.0, 7.0]),self.functionspace)
      arg[2,1,1]=val
      ref=Data(numpy.array([[[[3.0, 5.0, 6.0], [1.0, 1.0, 6.0], [1.0, 1.0, 3.0]], [[2.0, 5.0, 1.0], [2.0, 1.0, 3.0], [6.0, 7.0, 
1.0]], [[7.0, 8.0, 3.0], [1.0, 8.0, 5.0], [5.0, 7.0, 7.0]]], [[[3.0, 7.0, 7.0], [8.0, 7.0, 3.0], [5.0, 6.0, 6.0]], [[2.0, 4.0, 
2.0], [7.0, 7.0, 6.0], [2.0, 7.0, 5.0]], [[7.0, 8.0, 5.0], [2.0, 4.0, 8.0], [7.0, 2.0, 2.0]]], [[[2.0, 5.0, 6.0], [5.0, 6.0, 
7.0], [1.0, 7.0, 6.0]], [[6.0, 4.0, 4.0], [8.0, 2.0, 7.0], [2.0, 4.0, 8.0]], [[8.0, 8.0, 5.0], [2.0, 5.0, 2.0], [7.0, 2.0, 
5.0]]], [[[4.0, 7.0, 2.0], [5.0, 3.0, 8.0], [7.0, 5.0, 8.0]], [[1.0, 1.0, 7.0], [3.0, 1.0, 3.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 
2.0], [6.0, 2.0, 6.0], [3.0, 4.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPPN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 1.0], [7.0, 8.0, 5.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 5.0], [2.0, 7.0, 3.0], [5.0, 5.0, 
7.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 5.0], [8.0, 2.0, 3.0]]], [[[7.0, 3.0, 8.0], [6.0, 2.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 4.0, 
8.0], [2.0, 3.0, 2.0], [5.0, 7.0, 5.0]], [[4.0, 5.0, 7.0], [3.0, 4.0, 1.0], [1.0, 3.0, 4.0]]], [[[8.0, 3.0, 6.0], [4.0, 4.0, 
6.0], [8.0, 5.0, 5.0]], [[8.0, 8.0, 3.0], [1.0, 4.0, 4.0], [4.0, 7.0, 7.0]], [[2.0, 2.0, 8.0], [5.0, 7.0, 1.0], [3.0, 8.0, 
7.0]]], [[[7.0, 3.0, 1.0], [7.0, 3.0, 5.0], [1.0, 4.0, 4.0]], [[8.0, 6.0, 4.0], [5.0, 4.0, 6.0], [4.0, 2.0, 8.0]], [[5.0, 5.0, 
1.0], [1.0, 8.0, 3.0], [2.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([1.0, 2.0, 6.0])+(1.-msk_val)*numpy.array([1.0, 4.0, 6.0])
      arg[2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 1.0], [7.0, 8.0, 5.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 5.0], [2.0, 7.0, 3.0], [5.0, 
5.0, 7.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 5.0], [8.0, 2.0, 3.0]]], [[[7.0, 3.0, 8.0], [6.0, 2.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 
4.0, 8.0], [2.0, 3.0, 2.0], [5.0, 7.0, 5.0]], [[4.0, 5.0, 7.0], [3.0, 4.0, 1.0], [1.0, 3.0, 4.0]]], [[[8.0, 3.0, 6.0], [4.0, 
4.0, 6.0], [8.0, 5.0, 5.0]], [[8.0, 8.0, 3.0], [1.0, 2.0, 6.0], [4.0, 7.0, 7.0]], [[2.0, 2.0, 8.0], [5.0, 7.0, 1.0], [3.0, 8.0, 
7.0]]], [[[7.0, 3.0, 1.0], [7.0, 3.0, 5.0], [1.0, 4.0, 4.0]], [[8.0, 6.0, 4.0], [5.0, 4.0, 6.0], [4.0, 2.0, 8.0]], [[5.0, 5.0, 
1.0], [1.0, 8.0, 3.0], [2.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 1.0], [7.0, 8.0, 5.0], [2.0, 1.0, 6.0]], 
[[2.0, 4.0, 5.0], [2.0, 7.0, 3.0], [5.0, 5.0, 7.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 5.0], [8.0, 2.0, 3.0]]], [[[7.0, 3.0, 8.0], 
[6.0, 2.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 4.0, 8.0], [2.0, 3.0, 2.0], [5.0, 7.0, 5.0]], [[4.0, 5.0, 7.0], [3.0, 4.0, 1.0], 
[1.0, 3.0, 4.0]]], [[[8.0, 3.0, 6.0], [4.0, 4.0, 6.0], [8.0, 5.0, 5.0]], [[8.0, 8.0, 3.0], [1.0, 4.0, 6.0], [4.0, 7.0, 7.0]], 
[[2.0, 2.0, 8.0], [5.0, 7.0, 1.0], [3.0, 8.0, 7.0]]], [[[7.0, 3.0, 1.0], [7.0, 3.0, 5.0], [1.0, 4.0, 4.0]], [[8.0, 6.0, 4.0], 
[5.0, 4.0, 6.0], [4.0, 2.0, 8.0]], [[5.0, 5.0, 1.0], [1.0, 8.0, 3.0], [2.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PPPP(self):
      arg=Data(numpy.array([[[[1.0, 1.0, 4.0], [7.0, 7.0, 4.0], [1.0, 7.0, 7.0]], [[5.0, 5.0, 3.0], [4.0, 4.0, 7.0], [1.0, 7.0, 
1.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 4.0], [8.0, 5.0, 5.0]]], [[[5.0, 2.0, 1.0], [5.0, 1.0, 3.0], [5.0, 6.0, 7.0]], [[3.0, 7.0, 
7.0], [5.0, 3.0, 5.0], [2.0, 6.0, 3.0]], [[5.0, 8.0, 5.0], [2.0, 6.0, 7.0], [5.0, 8.0, 8.0]]], [[[8.0, 1.0, 7.0], [3.0, 7.0, 
2.0], [1.0, 2.0, 3.0]], [[8.0, 8.0, 5.0], [5.0, 4.0, 3.0], [4.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [5.0, 2.0, 6.0], [2.0, 5.0, 
2.0]]], [[[2.0, 7.0, 7.0], [6.0, 8.0, 1.0], [2.0, 1.0, 8.0]], [[7.0, 4.0, 6.0], [4.0, 8.0, 3.0], [5.0, 8.0, 8.0]], [[3.0, 5.0, 
7.0], [2.0, 3.0, 6.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      val=numpy.array(1.0)
      arg[2,1,1,1]=val
      ref=Data(numpy.array([[[[1.0, 1.0, 4.0], [7.0, 7.0, 4.0], [1.0, 7.0, 7.0]], [[5.0, 5.0, 3.0], [4.0, 4.0, 7.0], [1.0, 7.0, 
1.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 4.0], [8.0, 5.0, 5.0]]], [[[5.0, 2.0, 1.0], [5.0, 1.0, 3.0], [5.0, 6.0, 7.0]], [[3.0, 7.0, 
7.0], [5.0, 3.0, 5.0], [2.0, 6.0, 3.0]], [[5.0, 8.0, 5.0], [2.0, 6.0, 7.0], [5.0, 8.0, 8.0]]], [[[8.0, 1.0, 7.0], [3.0, 7.0, 
2.0], [1.0, 2.0, 3.0]], [[8.0, 8.0, 5.0], [5.0, 1.0, 3.0], [4.0, 5.0, 7.0]], [[4.0, 1.0, 8.0], [5.0, 2.0, 6.0], [2.0, 5.0, 
2.0]]], [[[2.0, 7.0, 7.0], [6.0, 8.0, 1.0], [2.0, 1.0, 8.0]], [[7.0, 4.0, 6.0], [4.0, 8.0, 3.0], [5.0, 8.0, 8.0]], [[3.0, 5.0, 
7.0], [2.0, 3.0, 6.0], [6.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PPPP(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 5.0], [2.0, 8.0, 4.0], [6.0, 5.0, 3.0]], [[4.0, 4.0, 4.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
4.0]], [[5.0, 5.0, 5.0], [3.0, 8.0, 3.0], [4.0, 3.0, 4.0]]], [[[6.0, 4.0, 4.0], [8.0, 3.0, 3.0], [8.0, 7.0, 7.0]], [[3.0, 3.0, 
8.0], [6.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[3.0, 8.0, 7.0], [5.0, 3.0, 6.0], [3.0, 6.0, 7.0]]], [[[1.0, 4.0, 5.0], [5.0, 2.0, 
6.0], [2.0, 6.0, 2.0]], [[7.0, 3.0, 1.0], [3.0, 8.0, 3.0], [4.0, 6.0, 6.0]], [[6.0, 3.0, 5.0], [3.0, 1.0, 5.0], [4.0, 1.0, 
8.0]]], [[[2.0, 1.0, 4.0], [8.0, 6.0, 3.0], [2.0, 4.0, 2.0]], [[1.0, 8.0, 2.0], [3.0, 5.0, 4.0], [5.0, 8.0, 2.0]], [[7.0, 1.0, 
2.0], [6.0, 3.0, 1.0], [7.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(1.0,self.functionspace)
      arg[2,1,1,1]=val
      ref=Data(numpy.array([[[[5.0, 7.0, 5.0], [2.0, 8.0, 4.0], [6.0, 5.0, 3.0]], [[4.0, 4.0, 4.0], [6.0, 8.0, 1.0], [6.0, 3.0, 
4.0]], [[5.0, 5.0, 5.0], [3.0, 8.0, 3.0], [4.0, 3.0, 4.0]]], [[[6.0, 4.0, 4.0], [8.0, 3.0, 3.0], [8.0, 7.0, 7.0]], [[3.0, 3.0, 
8.0], [6.0, 3.0, 4.0], [8.0, 4.0, 3.0]], [[3.0, 8.0, 7.0], [5.0, 3.0, 6.0], [3.0, 6.0, 7.0]]], [[[1.0, 4.0, 5.0], [5.0, 2.0, 
6.0], [2.0, 6.0, 2.0]], [[7.0, 3.0, 1.0], [3.0, 1.0, 3.0], [4.0, 6.0, 6.0]], [[6.0, 3.0, 5.0], [3.0, 1.0, 5.0], [4.0, 1.0, 
8.0]]], [[[2.0, 1.0, 4.0], [8.0, 6.0, 3.0], [2.0, 4.0, 2.0]], [[1.0, 8.0, 2.0], [3.0, 5.0, 4.0], [5.0, 8.0, 2.0]], [[7.0, 1.0, 
2.0], [6.0, 3.0, 1.0], [7.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PPPP(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 7.0], [4.0, 3.0, 4.0], [5.0, 5.0, 7.0]], [[7.0, 7.0, 5.0], [2.0, 5.0, 7.0], [1.0, 4.0, 
8.0]], [[1.0, 8.0, 4.0], [5.0, 2.0, 8.0], [6.0, 1.0, 2.0]]], [[[5.0, 3.0, 7.0], [7.0, 6.0, 7.0], [1.0, 8.0, 5.0]], [[6.0, 7.0, 
6.0], [7.0, 1.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 4.0, 8.0], [2.0, 5.0, 4.0], [1.0, 5.0, 2.0]]], [[[4.0, 3.0, 2.0], [6.0, 6.0, 
7.0], [2.0, 8.0, 2.0]], [[4.0, 2.0, 5.0], [1.0, 7.0, 5.0], [3.0, 5.0, 3.0]], [[4.0, 2.0, 7.0], [8.0, 6.0, 2.0], [1.0, 5.0, 
7.0]]], [[[4.0, 2.0, 4.0], [1.0, 3.0, 7.0], [2.0, 3.0, 7.0]], [[6.0, 4.0, 3.0], [3.0, 3.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 4.0, 
7.0], [3.0, 8.0, 3.0], [4.0, 6.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*(1.0)+(1.-msk_val)*(1.0)
      arg[2,1,1,1]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 7.0], [4.0, 3.0, 4.0], [5.0, 5.0, 7.0]], [[7.0, 7.0, 5.0], [2.0, 5.0, 7.0], [1.0, 4.0, 
8.0]], [[1.0, 8.0, 4.0], [5.0, 2.0, 8.0], [6.0, 1.0, 2.0]]], [[[5.0, 3.0, 7.0], [7.0, 6.0, 7.0], [1.0, 8.0, 5.0]], [[6.0, 7.0, 
6.0], [7.0, 1.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 4.0, 8.0], [2.0, 5.0, 4.0], [1.0, 5.0, 2.0]]], [[[4.0, 3.0, 2.0], [6.0, 6.0, 
7.0], [2.0, 8.0, 2.0]], [[4.0, 2.0, 5.0], [1.0, 1.0, 5.0], [3.0, 5.0, 3.0]], [[4.0, 2.0, 7.0], [8.0, 6.0, 2.0], [1.0, 5.0, 
7.0]]], [[[4.0, 2.0, 4.0], [1.0, 3.0, 7.0], [2.0, 3.0, 7.0]], [[6.0, 4.0, 3.0], [3.0, 3.0, 7.0], [5.0, 5.0, 6.0]], [[5.0, 4.0, 
7.0], [3.0, 8.0, 3.0], [4.0, 6.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PUUN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 5.0], [2.0, 3.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 1.0, 2.0], [4.0, 7.0, 1.0], [3.0, 7.0, 
7.0]], [[7.0, 8.0, 6.0], [7.0, 4.0, 5.0], [3.0, 6.0, 8.0]]], [[[6.0, 6.0, 1.0], [1.0, 2.0, 5.0], [4.0, 2.0, 7.0]], [[8.0, 7.0, 
7.0], [5.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 4.0], [3.0, 2.0, 8.0]]], [[[2.0, 3.0, 3.0], [2.0, 2.0, 
1.0], [4.0, 8.0, 5.0]], [[3.0, 8.0, 4.0], [1.0, 2.0, 2.0], [4.0, 2.0, 4.0]], [[6.0, 1.0, 8.0], [1.0, 3.0, 2.0], [3.0, 8.0, 
1.0]]], [[[7.0, 8.0, 6.0], [4.0, 3.0, 5.0], [4.0, 6.0, 1.0]], [[4.0, 2.0, 5.0], [3.0, 5.0, 7.0], [7.0, 1.0, 5.0]], [[5.0, 7.0, 
6.0], [7.0, 3.0, 4.0], [4.0, 6.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[3.0, 1.0, 1.0], [2.0, 7.0, 8.0]], [[5.0, 3.0, 2.0], [8.0, 4.0, 2.0]]])
      arg[2,:2,:2]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 5.0], [2.0, 3.0, 3.0], [6.0, 4.0, 5.0]], [[8.0, 1.0, 2.0], [4.0, 7.0, 1.0], [3.0, 7.0, 
7.0]], [[7.0, 8.0, 6.0], [7.0, 4.0, 5.0], [3.0, 6.0, 8.0]]], [[[6.0, 6.0, 1.0], [1.0, 2.0, 5.0], [4.0, 2.0, 7.0]], [[8.0, 7.0, 
7.0], [5.0, 2.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 4.0], [3.0, 2.0, 8.0]]], [[[3.0, 1.0, 1.0], [2.0, 7.0, 
8.0], [4.0, 8.0, 5.0]], [[5.0, 3.0, 2.0], [8.0, 4.0, 2.0], [4.0, 2.0, 4.0]], [[6.0, 1.0, 8.0], [1.0, 3.0, 2.0], [3.0, 8.0, 
1.0]]], [[[7.0, 8.0, 6.0], [4.0, 3.0, 5.0], [4.0, 6.0, 1.0]], [[4.0, 2.0, 5.0], [3.0, 5.0, 7.0], [7.0, 1.0, 5.0]], [[5.0, 7.0, 
6.0], [7.0, 3.0, 4.0], [4.0, 6.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PUUN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 1.0], [1.0, 1.0, 4.0], [7.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [3.0, 8.0, 7.0], [3.0, 5.0, 
3.0]], [[3.0, 8.0, 4.0], [7.0, 5.0, 7.0], [6.0, 7.0, 3.0]]], [[[6.0, 7.0, 1.0], [7.0, 8.0, 3.0], [1.0, 5.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[6.0, 4.0, 6.0], [3.0, 2.0, 7.0], [7.0, 8.0, 2.0]]], [[[3.0, 4.0, 8.0], [1.0, 4.0, 
2.0], [5.0, 4.0, 8.0]], [[4.0, 2.0, 4.0], [8.0, 7.0, 4.0], [2.0, 5.0, 7.0]], [[5.0, 6.0, 7.0], [7.0, 3.0, 2.0], [3.0, 5.0, 
5.0]]], [[[1.0, 2.0, 7.0], [3.0, 7.0, 3.0], [2.0, 2.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 6.0, 7.0], [1.0, 3.0, 5.0]], [[1.0, 5.0, 
6.0], [6.0, 3.0, 5.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 6.0, 1.0], [2.0, 4.0, 3.0]], [[4.0, 2.0, 1.0], [7.0, 5.0, 5.0]]]),self.functionspace)
      arg[2,:2,:2]=val
      ref=Data(numpy.array([[[[4.0, 2.0, 1.0], [1.0, 1.0, 4.0], [7.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [3.0, 8.0, 7.0], [3.0, 5.0, 
3.0]], [[3.0, 8.0, 4.0], [7.0, 5.0, 7.0], [6.0, 7.0, 3.0]]], [[[6.0, 7.0, 1.0], [7.0, 8.0, 3.0], [1.0, 5.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[6.0, 4.0, 6.0], [3.0, 2.0, 7.0], [7.0, 8.0, 2.0]]], [[[1.0, 6.0, 1.0], [2.0, 4.0, 
3.0], [5.0, 4.0, 8.0]], [[4.0, 2.0, 1.0], [7.0, 5.0, 5.0], [2.0, 5.0, 7.0]], [[5.0, 6.0, 7.0], [7.0, 3.0, 2.0], [3.0, 5.0, 
5.0]]], [[[1.0, 2.0, 7.0], [3.0, 7.0, 3.0], [2.0, 2.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 6.0, 7.0], [1.0, 3.0, 5.0]], [[1.0, 5.0, 
6.0], [6.0, 3.0, 5.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PUUN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 1.0], [5.0, 6.0, 4.0], [2.0, 4.0, 1.0]], [[1.0, 5.0, 5.0], [6.0, 1.0, 3.0], [3.0, 1.0, 
2.0]], [[4.0, 1.0, 5.0], [8.0, 4.0, 4.0], [7.0, 4.0, 5.0]]], [[[8.0, 2.0, 1.0], [6.0, 8.0, 8.0], [6.0, 2.0, 7.0]], [[4.0, 6.0, 
1.0], [8.0, 1.0, 3.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 1.0], [4.0, 8.0, 6.0], [1.0, 2.0, 7.0]]], [[[8.0, 1.0, 4.0], [4.0, 2.0, 
5.0], [4.0, 3.0, 2.0]], [[2.0, 1.0, 2.0], [3.0, 3.0, 3.0], [2.0, 6.0, 2.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 8.0], [3.0, 5.0, 
7.0]]], [[[1.0, 8.0, 5.0], [2.0, 3.0, 6.0], [7.0, 3.0, 1.0]], [[1.0, 5.0, 7.0], [5.0, 2.0, 4.0], [5.0, 7.0, 1.0]], [[7.0, 2.0, 
2.0], [8.0, 3.0, 7.0], [1.0, 7.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 8.0, 4.0], [3.0, 1.0, 7.0]], [[2.0, 4.0, 7.0], [8.0, 7.0, 
7.0]]])+(1.-msk_val)*numpy.array([[[1.0, 3.0, 6.0], [5.0, 4.0, 4.0]], [[3.0, 2.0, 2.0], [6.0, 3.0, 2.0]]])
      arg[2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 1.0], [5.0, 6.0, 4.0], [2.0, 4.0, 1.0]], [[1.0, 5.0, 5.0], [6.0, 1.0, 3.0], [3.0, 
1.0, 2.0]], [[4.0, 1.0, 5.0], [8.0, 4.0, 4.0], [7.0, 4.0, 5.0]]], [[[8.0, 2.0, 1.0], [6.0, 8.0, 8.0], [6.0, 2.0, 7.0]], [[4.0, 
6.0, 1.0], [8.0, 1.0, 3.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 1.0], [4.0, 8.0, 6.0], [1.0, 2.0, 7.0]]], [[[8.0, 8.0, 4.0], [3.0, 
1.0, 7.0], [4.0, 3.0, 2.0]], [[2.0, 4.0, 7.0], [8.0, 7.0, 7.0], [2.0, 6.0, 2.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 8.0], [3.0, 5.0, 
7.0]]], [[[1.0, 8.0, 5.0], [2.0, 3.0, 6.0], [7.0, 3.0, 1.0]], [[1.0, 5.0, 7.0], [5.0, 2.0, 4.0], [5.0, 7.0, 1.0]], [[7.0, 2.0, 
2.0], [8.0, 3.0, 7.0], [1.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [5.0, 6.0, 4.0], [2.0, 4.0, 1.0]], 
[[1.0, 5.0, 5.0], [6.0, 1.0, 3.0], [3.0, 1.0, 2.0]], [[4.0, 1.0, 5.0], [8.0, 4.0, 4.0], [7.0, 4.0, 5.0]]], [[[8.0, 2.0, 1.0], 
[6.0, 8.0, 8.0], [6.0, 2.0, 7.0]], [[4.0, 6.0, 1.0], [8.0, 1.0, 3.0], [5.0, 2.0, 1.0]], [[6.0, 3.0, 1.0], [4.0, 8.0, 6.0], 
[1.0, 2.0, 7.0]]], [[[1.0, 3.0, 6.0], [5.0, 4.0, 4.0], [4.0, 3.0, 2.0]], [[3.0, 2.0, 2.0], [6.0, 3.0, 2.0], [2.0, 6.0, 2.0]], 
[[3.0, 5.0, 5.0], [6.0, 3.0, 8.0], [3.0, 5.0, 7.0]]], [[[1.0, 8.0, 5.0], [2.0, 3.0, 6.0], [7.0, 3.0, 1.0]], [[1.0, 5.0, 7.0], 
[5.0, 2.0, 4.0], [5.0, 7.0, 1.0]], [[7.0, 2.0, 2.0], [8.0, 3.0, 7.0], [1.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RPLN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 7.0], [4.0, 7.0, 7.0], [7.0, 1.0, 2.0]], [[7.0, 5.0, 1.0], [7.0, 2.0, 3.0], [4.0, 2.0, 
2.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 7.0], [4.0, 7.0, 3.0]]], [[[8.0, 4.0, 2.0], [3.0, 5.0, 4.0], [3.0, 2.0, 6.0]], [[4.0, 3.0, 
6.0], [2.0, 5.0, 7.0], [1.0, 7.0, 4.0]], [[4.0, 7.0, 8.0], [6.0, 4.0, 8.0], [5.0, 6.0, 6.0]]], [[[3.0, 5.0, 4.0], [6.0, 3.0, 
5.0], [8.0, 4.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 3.0, 4.0], [7.0, 1.0, 8.0]], [[3.0, 1.0, 8.0], [5.0, 6.0, 2.0], [2.0, 8.0, 
4.0]]], [[[8.0, 6.0, 6.0], [3.0, 7.0, 7.0], [3.0, 4.0, 6.0]], [[7.0, 1.0, 6.0], [1.0, 7.0, 4.0], [1.0, 2.0, 6.0]], [[4.0, 6.0, 
1.0], [2.0, 7.0, 5.0], [8.0, 4.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[6.0, 4.0, 5.0]], [[4.0, 1.0, 8.0]]])
      arg[1:3,1,2:]=val
      ref=Data(numpy.array([[[[8.0, 2.0, 7.0], [4.0, 7.0, 7.0], [7.0, 1.0, 2.0]], [[7.0, 5.0, 1.0], [7.0, 2.0, 3.0], [4.0, 2.0, 
2.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 7.0], [4.0, 7.0, 3.0]]], [[[8.0, 4.0, 2.0], [3.0, 5.0, 4.0], [3.0, 2.0, 6.0]], [[4.0, 3.0, 
6.0], [2.0, 5.0, 7.0], [6.0, 4.0, 5.0]], [[4.0, 7.0, 8.0], [6.0, 4.0, 8.0], [5.0, 6.0, 6.0]]], [[[3.0, 5.0, 4.0], [6.0, 3.0, 
5.0], [8.0, 4.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 3.0, 4.0], [4.0, 1.0, 8.0]], [[3.0, 1.0, 8.0], [5.0, 6.0, 2.0], [2.0, 8.0, 
4.0]]], [[[8.0, 6.0, 6.0], [3.0, 7.0, 7.0], [3.0, 4.0, 6.0]], [[7.0, 1.0, 6.0], [1.0, 7.0, 4.0], [1.0, 2.0, 6.0]], [[4.0, 6.0, 
1.0], [2.0, 7.0, 5.0], [8.0, 4.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RPLN(self):
      arg=Data(numpy.array([[[[5.0, 1.0, 4.0], [3.0, 8.0, 2.0], [4.0, 8.0, 5.0]], [[2.0, 4.0, 6.0], [1.0, 2.0, 3.0], [7.0, 7.0, 
4.0]], [[3.0, 2.0, 6.0], [7.0, 7.0, 3.0], [5.0, 3.0, 6.0]]], [[[7.0, 1.0, 5.0], [6.0, 3.0, 1.0], [1.0, 3.0, 5.0]], [[3.0, 4.0, 
2.0], [4.0, 5.0, 5.0], [6.0, 1.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 6.0, 4.0], [6.0, 4.0, 1.0]]], [[[4.0, 6.0, 3.0], [4.0, 2.0, 
2.0], [7.0, 1.0, 7.0]], [[4.0, 7.0, 1.0], [2.0, 4.0, 4.0], [7.0, 1.0, 6.0]], [[5.0, 3.0, 6.0], [2.0, 6.0, 4.0], [7.0, 6.0, 
1.0]]], [[[8.0, 5.0, 1.0], [8.0, 6.0, 2.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 1.0], [2.0, 6.0, 1.0], [5.0, 7.0, 1.0]], [[6.0, 8.0, 
8.0], [3.0, 1.0, 8.0], [4.0, 7.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 2.0, 5.0]], [[7.0, 8.0, 4.0]]]),self.functionspace)
      arg[1:3,1,2:]=val
      ref=Data(numpy.array([[[[5.0, 1.0, 4.0], [3.0, 8.0, 2.0], [4.0, 8.0, 5.0]], [[2.0, 4.0, 6.0], [1.0, 2.0, 3.0], [7.0, 7.0, 
4.0]], [[3.0, 2.0, 6.0], [7.0, 7.0, 3.0], [5.0, 3.0, 6.0]]], [[[7.0, 1.0, 5.0], [6.0, 3.0, 1.0], [1.0, 3.0, 5.0]], [[3.0, 4.0, 
2.0], [4.0, 5.0, 5.0], [5.0, 2.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 6.0, 4.0], [6.0, 4.0, 1.0]]], [[[4.0, 6.0, 3.0], [4.0, 2.0, 
2.0], [7.0, 1.0, 7.0]], [[4.0, 7.0, 1.0], [2.0, 4.0, 4.0], [7.0, 8.0, 4.0]], [[5.0, 3.0, 6.0], [2.0, 6.0, 4.0], [7.0, 6.0, 
1.0]]], [[[8.0, 5.0, 1.0], [8.0, 6.0, 2.0], [4.0, 6.0, 7.0]], [[5.0, 3.0, 1.0], [2.0, 6.0, 1.0], [5.0, 7.0, 1.0]], [[6.0, 8.0, 
8.0], [3.0, 1.0, 8.0], [4.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RPLN(self):
      arg=Data(numpy.array([[[[8.0, 8.0, 1.0], [1.0, 5.0, 6.0], [1.0, 8.0, 4.0]], [[8.0, 2.0, 3.0], [6.0, 4.0, 1.0], [2.0, 3.0, 
6.0]], [[4.0, 1.0, 7.0], [3.0, 4.0, 4.0], [2.0, 4.0, 7.0]]], [[[6.0, 8.0, 7.0], [2.0, 7.0, 2.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 
1.0], [6.0, 6.0, 3.0], [6.0, 1.0, 2.0]], [[5.0, 2.0, 3.0], [4.0, 6.0, 3.0], [4.0, 6.0, 3.0]]], [[[4.0, 8.0, 8.0], [2.0, 8.0, 
8.0], [8.0, 4.0, 3.0]], [[3.0, 8.0, 7.0], [6.0, 8.0, 5.0], [2.0, 7.0, 3.0]], [[7.0, 8.0, 1.0], [1.0, 4.0, 2.0], [2.0, 2.0, 
5.0]]], [[[4.0, 3.0, 3.0], [3.0, 1.0, 3.0], [7.0, 5.0, 2.0]], [[2.0, 6.0, 2.0], [5.0, 8.0, 2.0], [3.0, 4.0, 5.0]], [[5.0, 7.0, 
3.0], [8.0, 2.0, 1.0], [8.0, 5.0, 4.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 1.0, 8.0]], [[6.0, 5.0, 1.0]]])+(1.-msk_val)*numpy.array([[[3.0, 2.0, 1.0]], [[3.0, 4.0, 
7.0]]])
      arg[1:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 1.0], [1.0, 5.0, 6.0], [1.0, 8.0, 4.0]], [[8.0, 2.0, 3.0], [6.0, 4.0, 1.0], [2.0, 
3.0, 6.0]], [[4.0, 1.0, 7.0], [3.0, 4.0, 4.0], [2.0, 4.0, 7.0]]], [[[6.0, 8.0, 7.0], [2.0, 7.0, 2.0], [2.0, 8.0, 8.0]], [[1.0, 
3.0, 1.0], [6.0, 6.0, 3.0], [1.0, 1.0, 8.0]], [[5.0, 2.0, 3.0], [4.0, 6.0, 3.0], [4.0, 6.0, 3.0]]], [[[4.0, 8.0, 8.0], [2.0, 
8.0, 8.0], [8.0, 4.0, 3.0]], [[3.0, 8.0, 7.0], [6.0, 8.0, 5.0], [6.0, 5.0, 1.0]], [[7.0, 8.0, 1.0], [1.0, 4.0, 2.0], [2.0, 2.0, 
5.0]]], [[[4.0, 3.0, 3.0], [3.0, 1.0, 3.0], [7.0, 5.0, 2.0]], [[2.0, 6.0, 2.0], [5.0, 8.0, 2.0], [3.0, 4.0, 5.0]], [[5.0, 7.0, 
3.0], [8.0, 2.0, 1.0], [8.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [1.0, 5.0, 6.0], [1.0, 8.0, 4.0]], 
[[8.0, 2.0, 3.0], [6.0, 4.0, 1.0], [2.0, 3.0, 6.0]], [[4.0, 1.0, 7.0], [3.0, 4.0, 4.0], [2.0, 4.0, 7.0]]], [[[6.0, 8.0, 7.0], 
[2.0, 7.0, 2.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 1.0], [6.0, 6.0, 3.0], [3.0, 2.0, 1.0]], [[5.0, 2.0, 3.0], [4.0, 6.0, 3.0], 
[4.0, 6.0, 3.0]]], [[[4.0, 8.0, 8.0], [2.0, 8.0, 8.0], [8.0, 4.0, 3.0]], [[3.0, 8.0, 7.0], [6.0, 8.0, 5.0], [3.0, 4.0, 7.0]], 
[[7.0, 8.0, 1.0], [1.0, 4.0, 2.0], [2.0, 2.0, 5.0]]], [[[4.0, 3.0, 3.0], [3.0, 1.0, 3.0], [7.0, 5.0, 2.0]], [[2.0, 6.0, 2.0], 
[5.0, 8.0, 2.0], [3.0, 4.0, 5.0]], [[5.0, 7.0, 3.0], [8.0, 2.0, 1.0], [8.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CPUN(self):
      arg=Data(numpy.array([[[[8.0, 5.0, 1.0], [1.0, 1.0, 6.0], [3.0, 8.0, 1.0]], [[4.0, 3.0, 4.0], [7.0, 5.0, 6.0], [4.0, 1.0, 
2.0]], [[7.0, 7.0, 4.0], [2.0, 5.0, 6.0], [6.0, 6.0, 2.0]]], [[[4.0, 3.0, 1.0], [1.0, 4.0, 2.0], [3.0, 6.0, 3.0]], [[8.0, 2.0, 
7.0], [6.0, 1.0, 7.0], [4.0, 4.0, 7.0]], [[1.0, 1.0, 6.0], [6.0, 8.0, 4.0], [6.0, 2.0, 4.0]]], [[[5.0, 1.0, 8.0], [5.0, 3.0, 
3.0], [4.0, 5.0, 3.0]], [[4.0, 7.0, 4.0], [1.0, 7.0, 6.0], [2.0, 7.0, 4.0]], [[2.0, 3.0, 6.0], [6.0, 3.0, 2.0], [3.0, 7.0, 
2.0]]], [[[4.0, 7.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 8.0]], [[7.0, 8.0, 6.0], [6.0, 1.0, 4.0], [3.0, 7.0, 8.0]], [[2.0, 1.0, 
1.0], [8.0, 8.0, 4.0], [7.0, 5.0, 5.0]]]]),self.functionspace)
      val=numpy.array([[[8.0, 6.0, 1.0], [6.0, 1.0, 4.0]], [[2.0, 1.0, 4.0], [1.0, 3.0, 1.0]], [[1.0, 2.0, 2.0], [3.0, 7.0, 
6.0]], [[4.0, 2.0, 2.0], [6.0, 4.0, 5.0]]])
      arg[:,1,:2]=val
      ref=Data(numpy.array([[[[8.0, 5.0, 1.0], [1.0, 1.0, 6.0], [3.0, 8.0, 1.0]], [[8.0, 6.0, 1.0], [6.0, 1.0, 4.0], [4.0, 1.0, 
2.0]], [[7.0, 7.0, 4.0], [2.0, 5.0, 6.0], [6.0, 6.0, 2.0]]], [[[4.0, 3.0, 1.0], [1.0, 4.0, 2.0], [3.0, 6.0, 3.0]], [[2.0, 1.0, 
4.0], [1.0, 3.0, 1.0], [4.0, 4.0, 7.0]], [[1.0, 1.0, 6.0], [6.0, 8.0, 4.0], [6.0, 2.0, 4.0]]], [[[5.0, 1.0, 8.0], [5.0, 3.0, 
3.0], [4.0, 5.0, 3.0]], [[1.0, 2.0, 2.0], [3.0, 7.0, 6.0], [2.0, 7.0, 4.0]], [[2.0, 3.0, 6.0], [6.0, 3.0, 2.0], [3.0, 7.0, 
2.0]]], [[[4.0, 7.0, 2.0], [2.0, 8.0, 7.0], [6.0, 6.0, 8.0]], [[4.0, 2.0, 2.0], [6.0, 4.0, 5.0], [3.0, 7.0, 8.0]], [[2.0, 1.0, 
1.0], [8.0, 8.0, 4.0], [7.0, 5.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CPUN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 7.0], [4.0, 7.0, 2.0], [8.0, 4.0, 5.0]], [[8.0, 1.0, 6.0], [5.0, 2.0, 5.0], [4.0, 8.0, 
1.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 8.0], [7.0, 4.0, 2.0]]], [[[4.0, 1.0, 8.0], [5.0, 5.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 
2.0], [2.0, 3.0, 3.0], [3.0, 7.0, 2.0]], [[8.0, 1.0, 1.0], [1.0, 4.0, 4.0], [5.0, 8.0, 1.0]]], [[[4.0, 5.0, 7.0], [4.0, 2.0, 
1.0], [6.0, 1.0, 3.0]], [[8.0, 8.0, 1.0], [6.0, 7.0, 8.0], [2.0, 4.0, 6.0]], [[5.0, 4.0, 6.0], [3.0, 1.0, 1.0], [3.0, 7.0, 
2.0]]], [[[7.0, 4.0, 6.0], [3.0, 4.0, 3.0], [2.0, 1.0, 7.0]], [[6.0, 6.0, 2.0], [7.0, 6.0, 5.0], [3.0, 8.0, 2.0]], [[4.0, 1.0, 
3.0], [1.0, 1.0, 4.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[5.0, 8.0, 5.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 3.0], [5.0, 7.0, 3.0]], [[7.0, 3.0, 2.0], [1.0, 
6.0, 4.0]], [[5.0, 5.0, 3.0], [2.0, 1.0, 2.0]]]),self.functionspace)
      arg[:,1,:2]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 7.0], [4.0, 7.0, 2.0], [8.0, 4.0, 5.0]], [[5.0, 8.0, 5.0], [1.0, 3.0, 5.0], [4.0, 8.0, 
1.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 8.0], [7.0, 4.0, 2.0]]], [[[4.0, 1.0, 8.0], [5.0, 5.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 7.0, 
3.0], [5.0, 7.0, 3.0], [3.0, 7.0, 2.0]], [[8.0, 1.0, 1.0], [1.0, 4.0, 4.0], [5.0, 8.0, 1.0]]], [[[4.0, 5.0, 7.0], [4.0, 2.0, 
1.0], [6.0, 1.0, 3.0]], [[7.0, 3.0, 2.0], [1.0, 6.0, 4.0], [2.0, 4.0, 6.0]], [[5.0, 4.0, 6.0], [3.0, 1.0, 1.0], [3.0, 7.0, 
2.0]]], [[[7.0, 4.0, 6.0], [3.0, 4.0, 3.0], [2.0, 1.0, 7.0]], [[5.0, 5.0, 3.0], [2.0, 1.0, 2.0], [3.0, 8.0, 2.0]], [[4.0, 1.0, 
3.0], [1.0, 1.0, 4.0], [1.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CPUN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 8.0], [7.0, 5.0, 7.0], [5.0, 2.0, 3.0]], [[3.0, 6.0, 4.0], [5.0, 7.0, 4.0], [4.0, 7.0, 
3.0]], [[2.0, 6.0, 6.0], [2.0, 7.0, 4.0], [1.0, 5.0, 6.0]]], [[[8.0, 4.0, 3.0], [3.0, 5.0, 5.0], [4.0, 1.0, 1.0]], [[6.0, 3.0, 
5.0], [2.0, 7.0, 7.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 5.0], [4.0, 8.0, 3.0], [5.0, 6.0, 1.0]]], [[[3.0, 8.0, 2.0], [5.0, 4.0, 
4.0], [8.0, 2.0, 6.0]], [[8.0, 2.0, 8.0], [6.0, 6.0, 8.0], [6.0, 7.0, 8.0]], [[4.0, 7.0, 2.0], [8.0, 6.0, 8.0], [6.0, 2.0, 
7.0]]], [[[7.0, 2.0, 6.0], [4.0, 5.0, 7.0], [1.0, 5.0, 6.0]], [[3.0, 4.0, 2.0], [1.0, 4.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 3.0, 
3.0], [8.0, 5.0, 8.0], [2.0, 1.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 5.0, 6.0], [8.0, 8.0, 8.0]], [[2.0, 6.0, 1.0], [7.0, 3.0, 4.0]], [[3.0, 1.0, 8.0], [7.0, 
3.0, 3.0]], [[2.0, 8.0, 1.0], [1.0, 8.0, 7.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0, 7.0], [8.0, 6.0, 5.0]], [[8.0, 5.0, 
7.0], [5.0, 6.0, 8.0]], [[7.0, 2.0, 8.0], [2.0, 5.0, 3.0]], [[2.0, 4.0, 7.0], [4.0, 5.0, 2.0]]])
      arg[:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 8.0], [7.0, 5.0, 7.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 6.0], [8.0, 8.0, 8.0], [4.0, 
7.0, 3.0]], [[2.0, 6.0, 6.0], [2.0, 7.0, 4.0], [1.0, 5.0, 6.0]]], [[[8.0, 4.0, 3.0], [3.0, 5.0, 5.0], [4.0, 1.0, 1.0]], [[2.0, 
6.0, 1.0], [7.0, 3.0, 4.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 5.0], [4.0, 8.0, 3.0], [5.0, 6.0, 1.0]]], [[[3.0, 8.0, 2.0], [5.0, 
4.0, 4.0], [8.0, 2.0, 6.0]], [[3.0, 1.0, 8.0], [7.0, 3.0, 3.0], [6.0, 7.0, 8.0]], [[4.0, 7.0, 2.0], [8.0, 6.0, 8.0], [6.0, 2.0, 
7.0]]], [[[7.0, 2.0, 6.0], [4.0, 5.0, 7.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 1.0], [1.0, 8.0, 7.0], [8.0, 6.0, 6.0]], [[3.0, 3.0, 
3.0], [8.0, 5.0, 8.0], [2.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 8.0], [7.0, 5.0, 7.0], [5.0, 2.0, 3.0]], 
[[5.0, 4.0, 7.0], [8.0, 6.0, 5.0], [4.0, 7.0, 3.0]], [[2.0, 6.0, 6.0], [2.0, 7.0, 4.0], [1.0, 5.0, 6.0]]], [[[8.0, 4.0, 3.0], 
[3.0, 5.0, 5.0], [4.0, 1.0, 1.0]], [[8.0, 5.0, 7.0], [5.0, 6.0, 8.0], [6.0, 6.0, 7.0]], [[2.0, 4.0, 5.0], [4.0, 8.0, 3.0], 
[5.0, 6.0, 1.0]]], [[[3.0, 8.0, 2.0], [5.0, 4.0, 4.0], [8.0, 2.0, 6.0]], [[7.0, 2.0, 8.0], [2.0, 5.0, 3.0], [6.0, 7.0, 8.0]], 
[[4.0, 7.0, 2.0], [8.0, 6.0, 8.0], [6.0, 2.0, 7.0]]], [[[7.0, 2.0, 6.0], [4.0, 5.0, 7.0], [1.0, 5.0, 6.0]], [[2.0, 4.0, 7.0], 
[4.0, 5.0, 2.0], [8.0, 6.0, 6.0]], [[3.0, 3.0, 3.0], [8.0, 5.0, 8.0], [2.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UUUU(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 5.0], [8.0, 5.0, 3.0], [3.0, 8.0, 2.0]], [[1.0, 5.0, 1.0], [3.0, 8.0, 8.0], [7.0, 5.0, 
5.0]], [[7.0, 4.0, 3.0], [6.0, 2.0, 1.0], [7.0, 1.0, 2.0]]], [[[7.0, 5.0, 8.0], [4.0, 6.0, 2.0], [5.0, 5.0, 5.0]], [[7.0, 4.0, 
4.0], [5.0, 2.0, 1.0], [8.0, 2.0, 2.0]], [[5.0, 1.0, 4.0], [1.0, 5.0, 8.0], [4.0, 5.0, 4.0]]], [[[2.0, 8.0, 1.0], [8.0, 4.0, 
8.0], [3.0, 8.0, 3.0]], [[8.0, 4.0, 6.0], [8.0, 8.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 8.0], [3.0, 6.0, 
8.0]]], [[[6.0, 5.0, 8.0], [3.0, 4.0, 8.0], [1.0, 4.0, 1.0]], [[5.0, 3.0, 2.0], [5.0, 2.0, 1.0], [7.0, 6.0, 8.0]], [[1.0, 5.0, 
1.0], [8.0, 2.0, 4.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[2.0, 7.0], [4.0, 6.0]], [[8.0, 2.0], [1.0, 3.0]]], [[[4.0, 7.0], [5.0, 2.0]], [[6.0, 4.0], [5.0, 
3.0]]], [[[1.0, 7.0], [5.0, 3.0]], [[7.0, 1.0], [4.0, 8.0]]]])
      arg[:3,:2,:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 7.0, 5.0], [4.0, 6.0, 3.0], [3.0, 8.0, 2.0]], [[8.0, 2.0, 1.0], [1.0, 3.0, 8.0], [7.0, 5.0, 
5.0]], [[7.0, 4.0, 3.0], [6.0, 2.0, 1.0], [7.0, 1.0, 2.0]]], [[[4.0, 7.0, 8.0], [5.0, 2.0, 2.0], [5.0, 5.0, 5.0]], [[6.0, 4.0, 
4.0], [5.0, 3.0, 1.0], [8.0, 2.0, 2.0]], [[5.0, 1.0, 4.0], [1.0, 5.0, 8.0], [4.0, 5.0, 4.0]]], [[[1.0, 7.0, 1.0], [5.0, 3.0, 
8.0], [3.0, 8.0, 3.0]], [[7.0, 1.0, 6.0], [4.0, 8.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 8.0], [3.0, 6.0, 
8.0]]], [[[6.0, 5.0, 8.0], [3.0, 4.0, 8.0], [1.0, 4.0, 1.0]], [[5.0, 3.0, 2.0], [5.0, 2.0, 1.0], [7.0, 6.0, 8.0]], [[1.0, 5.0, 
1.0], [8.0, 2.0, 4.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UUUU(self):
      arg=Data(numpy.array([[[[1.0, 4.0, 8.0], [4.0, 1.0, 5.0], [7.0, 2.0, 1.0]], [[3.0, 8.0, 2.0], [2.0, 3.0, 2.0], [2.0, 3.0, 
1.0]], [[8.0, 8.0, 6.0], [4.0, 1.0, 7.0], [6.0, 7.0, 3.0]]], [[[5.0, 3.0, 7.0], [7.0, 4.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 1.0, 
5.0], [7.0, 6.0, 2.0], [4.0, 1.0, 2.0]], [[2.0, 2.0, 2.0], [3.0, 8.0, 5.0], [7.0, 6.0, 8.0]]], [[[4.0, 4.0, 1.0], [7.0, 1.0, 
1.0], [4.0, 6.0, 3.0]], [[6.0, 6.0, 3.0], [6.0, 2.0, 3.0], [5.0, 2.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 5.0, 6.0], [4.0, 2.0, 
7.0]]], [[[3.0, 3.0, 3.0], [2.0, 2.0, 5.0], [2.0, 2.0, 7.0]], [[5.0, 2.0, 2.0], [3.0, 1.0, 6.0], [4.0, 7.0, 8.0]], [[3.0, 1.0, 
2.0], [7.0, 3.0, 1.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 6.0], [6.0, 8.0]], [[8.0, 6.0], [3.0, 1.0]]], [[[2.0, 7.0], [1.0, 2.0]], [[6.0, 6.0], [6.0, 
2.0]]], [[[3.0, 7.0], [2.0, 1.0]], [[4.0, 8.0], [8.0, 8.0]]]]),self.functionspace)
      arg[:3,:2,:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 8.0], [6.0, 8.0, 5.0], [7.0, 2.0, 1.0]], [[8.0, 6.0, 2.0], [3.0, 1.0, 2.0], [2.0, 3.0, 
1.0]], [[8.0, 8.0, 6.0], [4.0, 1.0, 7.0], [6.0, 7.0, 3.0]]], [[[2.0, 7.0, 7.0], [1.0, 2.0, 3.0], [8.0, 5.0, 4.0]], [[6.0, 6.0, 
5.0], [6.0, 2.0, 2.0], [4.0, 1.0, 2.0]], [[2.0, 2.0, 2.0], [3.0, 8.0, 5.0], [7.0, 6.0, 8.0]]], [[[3.0, 7.0, 1.0], [2.0, 1.0, 
1.0], [4.0, 6.0, 3.0]], [[4.0, 8.0, 3.0], [8.0, 8.0, 3.0], [5.0, 2.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 5.0, 6.0], [4.0, 2.0, 
7.0]]], [[[3.0, 3.0, 3.0], [2.0, 2.0, 5.0], [2.0, 2.0, 7.0]], [[5.0, 2.0, 2.0], [3.0, 1.0, 6.0], [4.0, 7.0, 8.0]], [[3.0, 1.0, 
2.0], [7.0, 3.0, 1.0], [6.0, 1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UUUU(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 6.0], [2.0, 3.0, 7.0], [8.0, 6.0, 6.0]], [[1.0, 3.0, 1.0], [7.0, 4.0, 4.0], [7.0, 6.0, 
2.0]], [[7.0, 2.0, 3.0], [2.0, 3.0, 6.0], [1.0, 1.0, 5.0]]], [[[6.0, 1.0, 6.0], [8.0, 6.0, 5.0], [6.0, 4.0, 6.0]], [[6.0, 5.0, 
3.0], [1.0, 4.0, 7.0], [7.0, 2.0, 3.0]], [[2.0, 6.0, 5.0], [6.0, 4.0, 4.0], [8.0, 6.0, 7.0]]], [[[3.0, 3.0, 6.0], [7.0, 7.0, 
6.0], [2.0, 6.0, 5.0]], [[2.0, 6.0, 4.0], [6.0, 2.0, 4.0], [4.0, 8.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 2.0, 1.0], [5.0, 7.0, 
5.0]]], [[[1.0, 7.0, 6.0], [1.0, 8.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 3.0, 7.0], [5.0, 6.0, 2.0], [4.0, 8.0, 8.0]], [[8.0, 1.0, 
2.0], [3.0, 8.0, 3.0], [8.0, 5.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 1.0], [5.0, 3.0]], [[8.0, 6.0], [8.0, 4.0]]], [[[4.0, 1.0], [6.0, 7.0]], [[1.0, 1.0], 
[4.0, 8.0]]], [[[1.0, 7.0], [3.0, 5.0]], [[6.0, 2.0], [2.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 3.0], [4.0, 6.0]], 
[[2.0, 8.0], [4.0, 3.0]]], [[[4.0, 3.0], [6.0, 6.0]], [[3.0, 4.0], [3.0, 3.0]]], [[[6.0, 7.0], [3.0, 6.0]], [[1.0, 4.0], [1.0, 
8.0]]]])
      arg[:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [5.0, 3.0, 7.0], [8.0, 6.0, 6.0]], [[8.0, 6.0, 1.0], [8.0, 4.0, 4.0], [7.0, 
6.0, 2.0]], [[7.0, 2.0, 3.0], [2.0, 3.0, 6.0], [1.0, 1.0, 5.0]]], [[[4.0, 1.0, 6.0], [6.0, 7.0, 5.0], [6.0, 4.0, 6.0]], [[1.0, 
1.0, 3.0], [4.0, 8.0, 7.0], [7.0, 2.0, 3.0]], [[2.0, 6.0, 5.0], [6.0, 4.0, 4.0], [8.0, 6.0, 7.0]]], [[[1.0, 7.0, 6.0], [3.0, 
5.0, 6.0], [2.0, 6.0, 5.0]], [[6.0, 2.0, 4.0], [2.0, 8.0, 4.0], [4.0, 8.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 2.0, 1.0], [5.0, 7.0, 
5.0]]], [[[1.0, 7.0, 6.0], [1.0, 8.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 3.0, 7.0], [5.0, 6.0, 2.0], [4.0, 8.0, 8.0]], [[8.0, 1.0, 
2.0], [3.0, 8.0, 3.0], [8.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 6.0], [4.0, 6.0, 7.0], [8.0, 6.0, 6.0]], 
[[2.0, 8.0, 1.0], [4.0, 3.0, 4.0], [7.0, 6.0, 2.0]], [[7.0, 2.0, 3.0], [2.0, 3.0, 6.0], [1.0, 1.0, 5.0]]], [[[4.0, 3.0, 6.0], 
[6.0, 6.0, 5.0], [6.0, 4.0, 6.0]], [[3.0, 4.0, 3.0], [3.0, 3.0, 7.0], [7.0, 2.0, 3.0]], [[2.0, 6.0, 5.0], [6.0, 4.0, 4.0], 
[8.0, 6.0, 7.0]]], [[[6.0, 7.0, 6.0], [3.0, 6.0, 6.0], [2.0, 6.0, 5.0]], [[1.0, 4.0, 4.0], [1.0, 8.0, 4.0], [4.0, 8.0, 8.0]], 
[[6.0, 4.0, 3.0], [7.0, 2.0, 1.0], [5.0, 7.0, 5.0]]], [[[1.0, 7.0, 6.0], [1.0, 8.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 3.0, 7.0], 
[5.0, 6.0, 2.0], [4.0, 8.0, 8.0]], [[8.0, 1.0, 2.0], [3.0, 8.0, 3.0], [8.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRLN(self):
      arg=Data(numpy.array([[[[5.0, 4.0, 6.0], [7.0, 7.0, 4.0], [8.0, 5.0, 1.0]], [[6.0, 4.0, 4.0], [1.0, 6.0, 4.0], [6.0, 3.0, 
6.0]], [[3.0, 6.0, 8.0], [7.0, 6.0, 6.0], [6.0, 7.0, 3.0]]], [[[4.0, 8.0, 7.0], [4.0, 6.0, 5.0], [8.0, 5.0, 3.0]], [[4.0, 1.0, 
1.0], [8.0, 1.0, 7.0], [1.0, 5.0, 4.0]], [[6.0, 7.0, 8.0], [5.0, 7.0, 8.0], [6.0, 2.0, 1.0]]], [[[7.0, 6.0, 5.0], [2.0, 3.0, 
1.0], [2.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [8.0, 4.0, 8.0], [7.0, 7.0, 2.0]], [[8.0, 8.0, 4.0], [8.0, 6.0, 5.0], [7.0, 6.0, 
3.0]]], [[[2.0, 8.0, 2.0], [6.0, 5.0, 4.0], [8.0, 1.0, 4.0]], [[7.0, 6.0, 8.0], [1.0, 4.0, 1.0], [1.0, 6.0, 7.0]], [[2.0, 5.0, 
5.0], [5.0, 8.0, 8.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[7.0, 5.0, 5.0]]], [[[6.0, 2.0, 6.0]]]])
      arg[1:3,1:2,2:]=val
      ref=Data(numpy.array([[[[5.0, 4.0, 6.0], [7.0, 7.0, 4.0], [8.0, 5.0, 1.0]], [[6.0, 4.0, 4.0], [1.0, 6.0, 4.0], [6.0, 3.0, 
6.0]], [[3.0, 6.0, 8.0], [7.0, 6.0, 6.0], [6.0, 7.0, 3.0]]], [[[4.0, 8.0, 7.0], [4.0, 6.0, 5.0], [8.0, 5.0, 3.0]], [[4.0, 1.0, 
1.0], [8.0, 1.0, 7.0], [7.0, 5.0, 5.0]], [[6.0, 7.0, 8.0], [5.0, 7.0, 8.0], [6.0, 2.0, 1.0]]], [[[7.0, 6.0, 5.0], [2.0, 3.0, 
1.0], [2.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [8.0, 4.0, 8.0], [6.0, 2.0, 6.0]], [[8.0, 8.0, 4.0], [8.0, 6.0, 5.0], [7.0, 6.0, 
3.0]]], [[[2.0, 8.0, 2.0], [6.0, 5.0, 4.0], [8.0, 1.0, 4.0]], [[7.0, 6.0, 8.0], [1.0, 4.0, 1.0], [1.0, 6.0, 7.0]], [[2.0, 5.0, 
5.0], [5.0, 8.0, 8.0], [4.0, 7.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRLN(self):
      arg=Data(numpy.array([[[[3.0, 5.0, 2.0], [3.0, 6.0, 2.0], [5.0, 4.0, 2.0]], [[7.0, 5.0, 1.0], [8.0, 7.0, 6.0], [3.0, 7.0, 
6.0]], [[7.0, 5.0, 8.0], [8.0, 8.0, 1.0], [2.0, 3.0, 8.0]]], [[[8.0, 7.0, 4.0], [5.0, 6.0, 4.0], [1.0, 4.0, 2.0]], [[5.0, 2.0, 
8.0], [1.0, 8.0, 2.0], [8.0, 3.0, 6.0]], [[4.0, 1.0, 3.0], [3.0, 5.0, 5.0], [8.0, 6.0, 2.0]]], [[[1.0, 7.0, 2.0], [6.0, 2.0, 
7.0], [1.0, 2.0, 6.0]], [[8.0, 1.0, 5.0], [1.0, 1.0, 7.0], [2.0, 5.0, 1.0]], [[4.0, 6.0, 7.0], [2.0, 8.0, 3.0], [3.0, 2.0, 
5.0]]], [[[1.0, 6.0, 1.0], [8.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [3.0, 3.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 5.0, 
5.0], [5.0, 6.0, 8.0], [7.0, 7.0, 4.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[4.0, 7.0, 6.0]]], [[[3.0, 1.0, 2.0]]]]),self.functionspace)
      arg[1:3,1:2,2:]=val
      ref=Data(numpy.array([[[[3.0, 5.0, 2.0], [3.0, 6.0, 2.0], [5.0, 4.0, 2.0]], [[7.0, 5.0, 1.0], [8.0, 7.0, 6.0], [3.0, 7.0, 
6.0]], [[7.0, 5.0, 8.0], [8.0, 8.0, 1.0], [2.0, 3.0, 8.0]]], [[[8.0, 7.0, 4.0], [5.0, 6.0, 4.0], [1.0, 4.0, 2.0]], [[5.0, 2.0, 
8.0], [1.0, 8.0, 2.0], [4.0, 7.0, 6.0]], [[4.0, 1.0, 3.0], [3.0, 5.0, 5.0], [8.0, 6.0, 2.0]]], [[[1.0, 7.0, 2.0], [6.0, 2.0, 
7.0], [1.0, 2.0, 6.0]], [[8.0, 1.0, 5.0], [1.0, 1.0, 7.0], [3.0, 1.0, 2.0]], [[4.0, 6.0, 7.0], [2.0, 8.0, 3.0], [3.0, 2.0, 
5.0]]], [[[1.0, 6.0, 1.0], [8.0, 2.0, 5.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [3.0, 3.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 5.0, 
5.0], [5.0, 6.0, 8.0], [7.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRLN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 6.0], [3.0, 8.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 8.0, 7.0], [4.0, 8.0, 4.0], [1.0, 7.0, 
2.0]], [[3.0, 2.0, 6.0], [3.0, 5.0, 5.0], [1.0, 7.0, 2.0]]], [[[6.0, 5.0, 3.0], [4.0, 3.0, 7.0], [5.0, 7.0, 6.0]], [[1.0, 2.0, 
3.0], [2.0, 7.0, 2.0], [2.0, 7.0, 2.0]], [[1.0, 2.0, 8.0], [7.0, 2.0, 3.0], [7.0, 6.0, 5.0]]], [[[1.0, 8.0, 7.0], [5.0, 6.0, 
4.0], [4.0, 7.0, 8.0]], [[1.0, 3.0, 7.0], [5.0, 6.0, 6.0], [7.0, 7.0, 5.0]], [[6.0, 1.0, 2.0], [1.0, 5.0, 4.0], [3.0, 6.0, 
7.0]]], [[[6.0, 3.0, 4.0], [6.0, 5.0, 5.0], [2.0, 5.0, 7.0]], [[5.0, 1.0, 6.0], [7.0, 3.0, 8.0], [2.0, 3.0, 7.0]], [[5.0, 5.0, 
5.0], [4.0, 6.0, 8.0], [1.0, 1.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 2.0, 2.0]]], [[[4.0, 8.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 1.0, 1.0]]], 
[[[2.0, 6.0, 7.0]]]])
      arg[1:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [3.0, 8.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 8.0, 7.0], [4.0, 8.0, 4.0], [1.0, 
7.0, 2.0]], [[3.0, 2.0, 6.0], [3.0, 5.0, 5.0], [1.0, 7.0, 2.0]]], [[[6.0, 5.0, 3.0], [4.0, 3.0, 7.0], [5.0, 7.0, 6.0]], [[1.0, 
2.0, 3.0], [2.0, 7.0, 2.0], [4.0, 2.0, 2.0]], [[1.0, 2.0, 8.0], [7.0, 2.0, 3.0], [7.0, 6.0, 5.0]]], [[[1.0, 8.0, 7.0], [5.0, 
6.0, 4.0], [4.0, 7.0, 8.0]], [[1.0, 3.0, 7.0], [5.0, 6.0, 6.0], [4.0, 8.0, 2.0]], [[6.0, 1.0, 2.0], [1.0, 5.0, 4.0], [3.0, 6.0, 
7.0]]], [[[6.0, 3.0, 4.0], [6.0, 5.0, 5.0], [2.0, 5.0, 7.0]], [[5.0, 1.0, 6.0], [7.0, 3.0, 8.0], [2.0, 3.0, 7.0]], [[5.0, 5.0, 
5.0], [4.0, 6.0, 8.0], [1.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 6.0], [3.0, 8.0, 3.0], [2.0, 5.0, 5.0]], 
[[6.0, 8.0, 7.0], [4.0, 8.0, 4.0], [1.0, 7.0, 2.0]], [[3.0, 2.0, 6.0], [3.0, 5.0, 5.0], [1.0, 7.0, 2.0]]], [[[6.0, 5.0, 3.0], 
[4.0, 3.0, 7.0], [5.0, 7.0, 6.0]], [[1.0, 2.0, 3.0], [2.0, 7.0, 2.0], [5.0, 1.0, 1.0]], [[1.0, 2.0, 8.0], [7.0, 2.0, 3.0], 
[7.0, 6.0, 5.0]]], [[[1.0, 8.0, 7.0], [5.0, 6.0, 4.0], [4.0, 7.0, 8.0]], [[1.0, 3.0, 7.0], [5.0, 6.0, 6.0], [2.0, 6.0, 7.0]], 
[[6.0, 1.0, 2.0], [1.0, 5.0, 4.0], [3.0, 6.0, 7.0]]], [[[6.0, 3.0, 4.0], [6.0, 5.0, 5.0], [2.0, 5.0, 7.0]], [[5.0, 1.0, 6.0], 
[7.0, 3.0, 8.0], [2.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [4.0, 6.0, 8.0], [1.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LRUN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 5.0], [2.0, 3.0, 8.0], [1.0, 6.0, 7.0]], [[4.0, 6.0, 3.0], [2.0, 1.0, 4.0], [2.0, 5.0, 
4.0]], [[4.0, 2.0, 5.0], [4.0, 7.0, 2.0], [4.0, 3.0, 5.0]]], [[[3.0, 2.0, 1.0], [2.0, 4.0, 7.0], [7.0, 5.0, 8.0]], [[2.0, 7.0, 
6.0], [3.0, 3.0, 7.0], [4.0, 8.0, 4.0]], [[2.0, 2.0, 1.0], [7.0, 6.0, 8.0], [3.0, 1.0, 4.0]]], [[[5.0, 6.0, 8.0], [5.0, 2.0, 
8.0], [1.0, 5.0, 3.0]], [[4.0, 2.0, 3.0], [5.0, 4.0, 3.0], [5.0, 5.0, 3.0]], [[1.0, 4.0, 7.0], [7.0, 2.0, 2.0], [3.0, 5.0, 
5.0]]], [[[2.0, 1.0, 6.0], [1.0, 6.0, 8.0], [8.0, 1.0, 2.0]], [[1.0, 7.0, 2.0], [6.0, 7.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 8.0, 
6.0], [7.0, 3.0, 7.0], [4.0, 5.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[5.0, 6.0, 1.0], [8.0, 1.0, 4.0]]], [[[7.0, 7.0, 7.0], [1.0, 7.0, 4.0]]]])
      arg[2:,1:2,:2]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 5.0], [2.0, 3.0, 8.0], [1.0, 6.0, 7.0]], [[4.0, 6.0, 3.0], [2.0, 1.0, 4.0], [2.0, 5.0, 
4.0]], [[4.0, 2.0, 5.0], [4.0, 7.0, 2.0], [4.0, 3.0, 5.0]]], [[[3.0, 2.0, 1.0], [2.0, 4.0, 7.0], [7.0, 5.0, 8.0]], [[2.0, 7.0, 
6.0], [3.0, 3.0, 7.0], [4.0, 8.0, 4.0]], [[2.0, 2.0, 1.0], [7.0, 6.0, 8.0], [3.0, 1.0, 4.0]]], [[[5.0, 6.0, 8.0], [5.0, 2.0, 
8.0], [1.0, 5.0, 3.0]], [[5.0, 6.0, 1.0], [8.0, 1.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 4.0, 7.0], [7.0, 2.0, 2.0], [3.0, 5.0, 
5.0]]], [[[2.0, 1.0, 6.0], [1.0, 6.0, 8.0], [8.0, 1.0, 2.0]], [[7.0, 7.0, 7.0], [1.0, 7.0, 4.0], [5.0, 1.0, 2.0]], [[6.0, 8.0, 
6.0], [7.0, 3.0, 7.0], [4.0, 5.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LRUN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 5.0], [1.0, 7.0, 2.0], [1.0, 4.0, 2.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 2.0], [6.0, 1.0, 
7.0]], [[2.0, 8.0, 6.0], [1.0, 6.0, 5.0], [2.0, 8.0, 1.0]]], [[[6.0, 7.0, 5.0], [6.0, 8.0, 2.0], [4.0, 4.0, 6.0]], [[5.0, 4.0, 
8.0], [3.0, 2.0, 7.0], [8.0, 4.0, 1.0]], [[6.0, 5.0, 8.0], [1.0, 5.0, 4.0], [5.0, 1.0, 8.0]]], [[[1.0, 4.0, 7.0], [1.0, 1.0, 
7.0], [2.0, 6.0, 6.0]], [[4.0, 1.0, 6.0], [4.0, 2.0, 8.0], [1.0, 1.0, 8.0]], [[2.0, 1.0, 7.0], [7.0, 6.0, 6.0], [6.0, 3.0, 
1.0]]], [[[1.0, 5.0, 8.0], [8.0, 3.0, 4.0], [5.0, 6.0, 4.0]], [[8.0, 3.0, 1.0], [8.0, 5.0, 8.0], [6.0, 6.0, 7.0]], [[8.0, 8.0, 
3.0], [3.0, 2.0, 8.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[2.0, 8.0, 4.0], [3.0, 3.0, 2.0]]], [[[5.0, 4.0, 5.0], [3.0, 3.0, 1.0]]]]),self.functionspace)
      arg[2:,1:2,:2]=val
      ref=Data(numpy.array([[[[2.0, 8.0, 5.0], [1.0, 7.0, 2.0], [1.0, 4.0, 2.0]], [[4.0, 8.0, 7.0], [4.0, 3.0, 2.0], [6.0, 1.0, 
7.0]], [[2.0, 8.0, 6.0], [1.0, 6.0, 5.0], [2.0, 8.0, 1.0]]], [[[6.0, 7.0, 5.0], [6.0, 8.0, 2.0], [4.0, 4.0, 6.0]], [[5.0, 4.0, 
8.0], [3.0, 2.0, 7.0], [8.0, 4.0, 1.0]], [[6.0, 5.0, 8.0], [1.0, 5.0, 4.0], [5.0, 1.0, 8.0]]], [[[1.0, 4.0, 7.0], [1.0, 1.0, 
7.0], [2.0, 6.0, 6.0]], [[2.0, 8.0, 4.0], [3.0, 3.0, 2.0], [1.0, 1.0, 8.0]], [[2.0, 1.0, 7.0], [7.0, 6.0, 6.0], [6.0, 3.0, 
1.0]]], [[[1.0, 5.0, 8.0], [8.0, 3.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 4.0, 5.0], [3.0, 3.0, 1.0], [6.0, 6.0, 7.0]], [[8.0, 8.0, 
3.0], [3.0, 2.0, 8.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LRUN(self):
      arg=Data(numpy.array([[[[3.0, 3.0, 2.0], [3.0, 6.0, 6.0], [4.0, 7.0, 7.0]], [[7.0, 6.0, 6.0], [7.0, 5.0, 4.0], [6.0, 3.0, 
6.0]], [[3.0, 5.0, 4.0], [3.0, 2.0, 1.0], [6.0, 7.0, 6.0]]], [[[4.0, 8.0, 2.0], [2.0, 2.0, 2.0], [1.0, 7.0, 2.0]], [[3.0, 6.0, 
2.0], [7.0, 2.0, 6.0], [4.0, 5.0, 6.0]], [[1.0, 8.0, 5.0], [2.0, 3.0, 6.0], [8.0, 4.0, 6.0]]], [[[1.0, 2.0, 4.0], [6.0, 8.0, 
1.0], [4.0, 6.0, 4.0]], [[1.0, 6.0, 1.0], [6.0, 1.0, 4.0], [1.0, 5.0, 3.0]], [[3.0, 1.0, 6.0], [8.0, 6.0, 7.0], [4.0, 3.0, 
4.0]]], [[[2.0, 7.0, 5.0], [3.0, 7.0, 4.0], [4.0, 4.0, 2.0]], [[6.0, 5.0, 5.0], [6.0, 7.0, 1.0], [2.0, 3.0, 1.0]], [[8.0, 5.0, 
6.0], [6.0, 5.0, 6.0], [5.0, 3.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 7.0, 1.0], [4.0, 6.0, 8.0]]], [[[7.0, 4.0, 5.0], [7.0, 6.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 6.0, 3.0], [6.0, 4.0, 6.0]]], [[[2.0, 3.0, 3.0], [4.0, 8.0, 8.0]]]])
      arg[2:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 2.0], [3.0, 6.0, 6.0], [4.0, 7.0, 7.0]], [[7.0, 6.0, 6.0], [7.0, 5.0, 4.0], [6.0, 
3.0, 6.0]], [[3.0, 5.0, 4.0], [3.0, 2.0, 1.0], [6.0, 7.0, 6.0]]], [[[4.0, 8.0, 2.0], [2.0, 2.0, 2.0], [1.0, 7.0, 2.0]], [[3.0, 
6.0, 2.0], [7.0, 2.0, 6.0], [4.0, 5.0, 6.0]], [[1.0, 8.0, 5.0], [2.0, 3.0, 6.0], [8.0, 4.0, 6.0]]], [[[1.0, 2.0, 4.0], [6.0, 
8.0, 1.0], [4.0, 6.0, 4.0]], [[4.0, 7.0, 1.0], [4.0, 6.0, 8.0], [1.0, 5.0, 3.0]], [[3.0, 1.0, 6.0], [8.0, 6.0, 7.0], [4.0, 3.0, 
4.0]]], [[[2.0, 7.0, 5.0], [3.0, 7.0, 4.0], [4.0, 4.0, 2.0]], [[7.0, 4.0, 5.0], [7.0, 6.0, 3.0], [2.0, 3.0, 1.0]], [[8.0, 5.0, 
6.0], [6.0, 5.0, 6.0], [5.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 2.0], [3.0, 6.0, 6.0], [4.0, 7.0, 7.0]], 
[[7.0, 6.0, 6.0], [7.0, 5.0, 4.0], [6.0, 3.0, 6.0]], [[3.0, 5.0, 4.0], [3.0, 2.0, 1.0], [6.0, 7.0, 6.0]]], [[[4.0, 8.0, 2.0], 
[2.0, 2.0, 2.0], [1.0, 7.0, 2.0]], [[3.0, 6.0, 2.0], [7.0, 2.0, 6.0], [4.0, 5.0, 6.0]], [[1.0, 8.0, 5.0], [2.0, 3.0, 6.0], 
[8.0, 4.0, 6.0]]], [[[1.0, 2.0, 4.0], [6.0, 8.0, 1.0], [4.0, 6.0, 4.0]], [[6.0, 6.0, 3.0], [6.0, 4.0, 6.0], [1.0, 5.0, 3.0]], 
[[3.0, 1.0, 6.0], [8.0, 6.0, 7.0], [4.0, 3.0, 4.0]]], [[[2.0, 7.0, 5.0], [3.0, 7.0, 4.0], [4.0, 4.0, 2.0]], [[2.0, 3.0, 3.0], 
[4.0, 8.0, 8.0], [2.0, 3.0, 1.0]], [[8.0, 5.0, 6.0], [6.0, 5.0, 6.0], [5.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UPPN(self):
      arg=Data(numpy.array([[[[7.0, 4.0, 3.0], [2.0, 3.0, 1.0], [2.0, 6.0, 2.0]], [[3.0, 1.0, 5.0], [7.0, 8.0, 8.0], [6.0, 7.0, 
6.0]], [[8.0, 6.0, 2.0], [5.0, 7.0, 3.0], [2.0, 8.0, 4.0]]], [[[5.0, 6.0, 1.0], [1.0, 2.0, 7.0], [2.0, 6.0, 6.0]], [[3.0, 2.0, 
2.0], [5.0, 7.0, 7.0], [3.0, 2.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 5.0, 4.0], [4.0, 8.0, 3.0]]], [[[6.0, 4.0, 8.0], [5.0, 1.0, 
5.0], [1.0, 6.0, 2.0]], [[7.0, 7.0, 8.0], [8.0, 3.0, 1.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 4.0], [7.0, 5.0, 8.0], [5.0, 3.0, 
7.0]]], [[[8.0, 2.0, 4.0], [4.0, 8.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 6.0, 1.0], [6.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[5.0, 6.0, 
5.0], [6.0, 4.0, 7.0], [6.0, 3.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[3.0, 1.0, 1.0], [1.0, 8.0, 6.0], [6.0, 7.0, 2.0]])
      arg[:3,1,1]=val
      ref=Data(numpy.array([[[[7.0, 4.0, 3.0], [2.0, 3.0, 1.0], [2.0, 6.0, 2.0]], [[3.0, 1.0, 5.0], [3.0, 1.0, 1.0], [6.0, 7.0, 
6.0]], [[8.0, 6.0, 2.0], [5.0, 7.0, 3.0], [2.0, 8.0, 4.0]]], [[[5.0, 6.0, 1.0], [1.0, 2.0, 7.0], [2.0, 6.0, 6.0]], [[3.0, 2.0, 
2.0], [1.0, 8.0, 6.0], [3.0, 2.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 5.0, 4.0], [4.0, 8.0, 3.0]]], [[[6.0, 4.0, 8.0], [5.0, 1.0, 
5.0], [1.0, 6.0, 2.0]], [[7.0, 7.0, 8.0], [6.0, 7.0, 2.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 4.0], [7.0, 5.0, 8.0], [5.0, 3.0, 
7.0]]], [[[8.0, 2.0, 4.0], [4.0, 8.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 6.0, 1.0], [6.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[5.0, 6.0, 
5.0], [6.0, 4.0, 7.0], [6.0, 3.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UPPN(self):
      arg=Data(numpy.array([[[[6.0, 4.0, 3.0], [4.0, 2.0, 3.0], [6.0, 4.0, 3.0]], [[5.0, 5.0, 6.0], [8.0, 5.0, 7.0], [2.0, 4.0, 
1.0]], [[2.0, 7.0, 7.0], [3.0, 7.0, 4.0], [2.0, 2.0, 6.0]]], [[[6.0, 8.0, 1.0], [1.0, 8.0, 3.0], [5.0, 2.0, 1.0]], [[4.0, 7.0, 
1.0], [5.0, 1.0, 7.0], [4.0, 6.0, 2.0]], [[3.0, 6.0, 6.0], [2.0, 3.0, 2.0], [3.0, 1.0, 6.0]]], [[[2.0, 7.0, 4.0], [7.0, 4.0, 
1.0], [5.0, 4.0, 6.0]], [[3.0, 8.0, 6.0], [6.0, 6.0, 6.0], [4.0, 8.0, 4.0]], [[5.0, 6.0, 5.0], [6.0, 5.0, 4.0], [1.0, 6.0, 
6.0]]], [[[1.0, 8.0, 2.0], [4.0, 8.0, 7.0], [1.0, 5.0, 7.0]], [[5.0, 3.0, 2.0], [2.0, 8.0, 1.0], [2.0, 2.0, 7.0]], [[1.0, 6.0, 
2.0], [4.0, 5.0, 3.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[5.0, 8.0, 1.0], [2.0, 1.0, 3.0], [7.0, 8.0, 8.0]]),self.functionspace)
      arg[:3,1,1]=val
      ref=Data(numpy.array([[[[6.0, 4.0, 3.0], [4.0, 2.0, 3.0], [6.0, 4.0, 3.0]], [[5.0, 5.0, 6.0], [5.0, 8.0, 1.0], [2.0, 4.0, 
1.0]], [[2.0, 7.0, 7.0], [3.0, 7.0, 4.0], [2.0, 2.0, 6.0]]], [[[6.0, 8.0, 1.0], [1.0, 8.0, 3.0], [5.0, 2.0, 1.0]], [[4.0, 7.0, 
1.0], [2.0, 1.0, 3.0], [4.0, 6.0, 2.0]], [[3.0, 6.0, 6.0], [2.0, 3.0, 2.0], [3.0, 1.0, 6.0]]], [[[2.0, 7.0, 4.0], [7.0, 4.0, 
1.0], [5.0, 4.0, 6.0]], [[3.0, 8.0, 6.0], [7.0, 8.0, 8.0], [4.0, 8.0, 4.0]], [[5.0, 6.0, 5.0], [6.0, 5.0, 4.0], [1.0, 6.0, 
6.0]]], [[[1.0, 8.0, 2.0], [4.0, 8.0, 7.0], [1.0, 5.0, 7.0]], [[5.0, 3.0, 2.0], [2.0, 8.0, 1.0], [2.0, 2.0, 7.0]], [[1.0, 6.0, 
2.0], [4.0, 5.0, 3.0], [7.0, 5.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UPPN(self):
      arg=Data(numpy.array([[[[2.0, 8.0, 8.0], [6.0, 3.0, 3.0], [1.0, 3.0, 8.0]], [[8.0, 4.0, 8.0], [5.0, 3.0, 4.0], [2.0, 2.0, 
7.0]], [[4.0, 2.0, 5.0], [7.0, 2.0, 5.0], [1.0, 7.0, 2.0]]], [[[2.0, 5.0, 8.0], [4.0, 2.0, 2.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 
5.0], [5.0, 5.0, 3.0], [3.0, 3.0, 2.0]], [[6.0, 3.0, 2.0], [7.0, 1.0, 4.0], [3.0, 7.0, 5.0]]], [[[3.0, 6.0, 5.0], [3.0, 6.0, 
2.0], [2.0, 8.0, 4.0]], [[6.0, 3.0, 7.0], [4.0, 3.0, 4.0], [7.0, 7.0, 4.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 4.0], [1.0, 2.0, 
4.0]]], [[[6.0, 4.0, 2.0], [8.0, 6.0, 2.0], [8.0, 4.0, 5.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 5.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 
6.0], [4.0, 1.0, 2.0], [1.0, 3.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 5.0, 7.0], [2.0, 1.0, 2.0], [4.0, 2.0, 4.0]])+(1.-msk_val)*numpy.array([[3.0, 7.0, 3.0], 
[2.0, 2.0, 4.0], [5.0, 1.0, 6.0]])
      arg[:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 3.0, 3.0], [1.0, 3.0, 8.0]], [[8.0, 4.0, 8.0], [2.0, 5.0, 7.0], [2.0, 
2.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 2.0, 5.0], [1.0, 7.0, 2.0]]], [[[2.0, 5.0, 8.0], [4.0, 2.0, 2.0], [1.0, 5.0, 4.0]], [[8.0, 
1.0, 5.0], [2.0, 1.0, 2.0], [3.0, 3.0, 2.0]], [[6.0, 3.0, 2.0], [7.0, 1.0, 4.0], [3.0, 7.0, 5.0]]], [[[3.0, 6.0, 5.0], [3.0, 
6.0, 2.0], [2.0, 8.0, 4.0]], [[6.0, 3.0, 7.0], [4.0, 2.0, 4.0], [7.0, 7.0, 4.0]], [[5.0, 2.0, 1.0], [2.0, 1.0, 4.0], [1.0, 2.0, 
4.0]]], [[[6.0, 4.0, 2.0], [8.0, 6.0, 2.0], [8.0, 4.0, 5.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 5.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 
6.0], [4.0, 1.0, 2.0], [1.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 3.0, 3.0], [1.0, 3.0, 8.0]], 
[[8.0, 4.0, 8.0], [3.0, 7.0, 3.0], [2.0, 2.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 2.0, 5.0], [1.0, 7.0, 2.0]]], [[[2.0, 5.0, 8.0], 
[4.0, 2.0, 2.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 5.0], [2.0, 2.0, 4.0], [3.0, 3.0, 2.0]], [[6.0, 3.0, 2.0], [7.0, 1.0, 4.0], 
[3.0, 7.0, 5.0]]], [[[3.0, 6.0, 5.0], [3.0, 6.0, 2.0], [2.0, 8.0, 4.0]], [[6.0, 3.0, 7.0], [5.0, 1.0, 6.0], [7.0, 7.0, 4.0]], 
[[5.0, 2.0, 1.0], [2.0, 1.0, 4.0], [1.0, 2.0, 4.0]]], [[[6.0, 4.0, 2.0], [8.0, 6.0, 2.0], [8.0, 4.0, 5.0]], [[7.0, 1.0, 2.0], 
[6.0, 4.0, 5.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 6.0], [4.0, 1.0, 2.0], [1.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RULN(self):
      arg=Data(numpy.array([[[[8.0, 6.0, 2.0], [3.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 1.0], [7.0, 3.0, 
7.0]], [[6.0, 7.0, 1.0], [1.0, 1.0, 5.0], [4.0, 3.0, 4.0]]], [[[4.0, 3.0, 4.0], [5.0, 7.0, 2.0], [7.0, 8.0, 6.0]], [[7.0, 2.0, 
2.0], [8.0, 6.0, 5.0], [2.0, 7.0, 5.0]], [[4.0, 6.0, 4.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]]], [[[4.0, 3.0, 8.0], [8.0, 7.0, 
5.0], [3.0, 7.0, 1.0]], [[6.0, 7.0, 3.0], [7.0, 7.0, 2.0], [5.0, 5.0, 1.0]], [[2.0, 5.0, 3.0], [6.0, 2.0, 7.0], [7.0, 8.0, 
2.0]]], [[[3.0, 3.0, 5.0], [2.0, 7.0, 6.0], [4.0, 4.0, 1.0]], [[8.0, 2.0, 8.0], [1.0, 8.0, 5.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 
3.0], [5.0, 7.0, 7.0], [4.0, 3.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 5.0, 3.0]], [[6.0, 2.0, 5.0]]], [[[7.0, 4.0, 3.0]], [[3.0, 3.0, 5.0]]]])
      arg[1:3,:2,2:]=val
      ref=Data(numpy.array([[[[8.0, 6.0, 2.0], [3.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 3.0, 3.0], [2.0, 3.0, 1.0], [7.0, 3.0, 
7.0]], [[6.0, 7.0, 1.0], [1.0, 1.0, 5.0], [4.0, 3.0, 4.0]]], [[[4.0, 3.0, 4.0], [5.0, 7.0, 2.0], [4.0, 5.0, 3.0]], [[7.0, 2.0, 
2.0], [8.0, 6.0, 5.0], [6.0, 2.0, 5.0]], [[4.0, 6.0, 4.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]]], [[[4.0, 3.0, 8.0], [8.0, 7.0, 
5.0], [7.0, 4.0, 3.0]], [[6.0, 7.0, 3.0], [7.0, 7.0, 2.0], [3.0, 3.0, 5.0]], [[2.0, 5.0, 3.0], [6.0, 2.0, 7.0], [7.0, 8.0, 
2.0]]], [[[3.0, 3.0, 5.0], [2.0, 7.0, 6.0], [4.0, 4.0, 1.0]], [[8.0, 2.0, 8.0], [1.0, 8.0, 5.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 
3.0], [5.0, 7.0, 7.0], [4.0, 3.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RULN(self):
      arg=Data(numpy.array([[[[7.0, 1.0, 7.0], [3.0, 6.0, 3.0], [3.0, 4.0, 8.0]], [[4.0, 8.0, 5.0], [6.0, 2.0, 3.0], [4.0, 2.0, 
6.0]], [[8.0, 8.0, 5.0], [8.0, 2.0, 4.0], [2.0, 5.0, 6.0]]], [[[3.0, 6.0, 5.0], [1.0, 3.0, 8.0], [5.0, 1.0, 8.0]], [[2.0, 8.0, 
8.0], [3.0, 1.0, 1.0], [4.0, 6.0, 8.0]], [[3.0, 6.0, 8.0], [8.0, 2.0, 8.0], [5.0, 3.0, 8.0]]], [[[2.0, 5.0, 8.0], [1.0, 3.0, 
4.0], [5.0, 4.0, 8.0]], [[8.0, 4.0, 7.0], [5.0, 4.0, 8.0], [1.0, 1.0, 2.0]], [[4.0, 1.0, 7.0], [6.0, 1.0, 2.0], [8.0, 8.0, 
1.0]]], [[[4.0, 8.0, 5.0], [6.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[3.0, 3.0, 7.0], [7.0, 4.0, 4.0], [2.0, 7.0, 6.0]], [[6.0, 5.0, 
4.0], [4.0, 3.0, 1.0], [6.0, 5.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[3.0, 5.0, 6.0]], [[5.0, 6.0, 4.0]]], [[[3.0, 7.0, 2.0]], [[6.0, 7.0, 3.0]]]]),self.functionspace)
      arg[1:3,:2,2:]=val
      ref=Data(numpy.array([[[[7.0, 1.0, 7.0], [3.0, 6.0, 3.0], [3.0, 4.0, 8.0]], [[4.0, 8.0, 5.0], [6.0, 2.0, 3.0], [4.0, 2.0, 
6.0]], [[8.0, 8.0, 5.0], [8.0, 2.0, 4.0], [2.0, 5.0, 6.0]]], [[[3.0, 6.0, 5.0], [1.0, 3.0, 8.0], [3.0, 5.0, 6.0]], [[2.0, 8.0, 
8.0], [3.0, 1.0, 1.0], [5.0, 6.0, 4.0]], [[3.0, 6.0, 8.0], [8.0, 2.0, 8.0], [5.0, 3.0, 8.0]]], [[[2.0, 5.0, 8.0], [1.0, 3.0, 
4.0], [3.0, 7.0, 2.0]], [[8.0, 4.0, 7.0], [5.0, 4.0, 8.0], [6.0, 7.0, 3.0]], [[4.0, 1.0, 7.0], [6.0, 1.0, 2.0], [8.0, 8.0, 
1.0]]], [[[4.0, 8.0, 5.0], [6.0, 8.0, 1.0], [4.0, 6.0, 2.0]], [[3.0, 3.0, 7.0], [7.0, 4.0, 4.0], [2.0, 7.0, 6.0]], [[6.0, 5.0, 
4.0], [4.0, 3.0, 1.0], [6.0, 5.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RULN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 3.0], [2.0, 8.0, 7.0], [8.0, 1.0, 7.0]], [[7.0, 3.0, 4.0], [7.0, 5.0, 2.0], [7.0, 2.0, 
3.0]], [[7.0, 5.0, 8.0], [4.0, 7.0, 4.0], [3.0, 4.0, 4.0]]], [[[6.0, 7.0, 1.0], [1.0, 3.0, 2.0], [6.0, 6.0, 4.0]], [[7.0, 1.0, 
5.0], [1.0, 2.0, 7.0], [5.0, 1.0, 7.0]], [[3.0, 8.0, 4.0], [5.0, 8.0, 7.0], [6.0, 2.0, 6.0]]], [[[8.0, 1.0, 6.0], [5.0, 4.0, 
8.0], [6.0, 6.0, 2.0]], [[4.0, 8.0, 3.0], [2.0, 6.0, 6.0], [2.0, 1.0, 5.0]], [[1.0, 1.0, 3.0], [3.0, 6.0, 8.0], [5.0, 2.0, 
7.0]]], [[[6.0, 6.0, 2.0], [1.0, 5.0, 6.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 4.0], [7.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[5.0, 6.0, 
4.0], [4.0, 6.0, 7.0], [8.0, 7.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 5.0, 4.0]], [[6.0, 7.0, 5.0]]], [[[2.0, 4.0, 4.0]], [[2.0, 5.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 4.0]], [[6.0, 5.0, 3.0]]], [[[3.0, 2.0, 4.0]], [[4.0, 2.0, 1.0]]]])
      arg[1:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 3.0], [2.0, 8.0, 7.0], [8.0, 1.0, 7.0]], [[7.0, 3.0, 4.0], [7.0, 5.0, 2.0], [7.0, 
2.0, 3.0]], [[7.0, 5.0, 8.0], [4.0, 7.0, 4.0], [3.0, 4.0, 4.0]]], [[[6.0, 7.0, 1.0], [1.0, 3.0, 2.0], [3.0, 5.0, 4.0]], [[7.0, 
1.0, 5.0], [1.0, 2.0, 7.0], [6.0, 7.0, 5.0]], [[3.0, 8.0, 4.0], [5.0, 8.0, 7.0], [6.0, 2.0, 6.0]]], [[[8.0, 1.0, 6.0], [5.0, 
4.0, 8.0], [2.0, 4.0, 4.0]], [[4.0, 8.0, 3.0], [2.0, 6.0, 6.0], [2.0, 5.0, 4.0]], [[1.0, 1.0, 3.0], [3.0, 6.0, 8.0], [5.0, 2.0, 
7.0]]], [[[6.0, 6.0, 2.0], [1.0, 5.0, 6.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 4.0], [7.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[5.0, 6.0, 
4.0], [4.0, 6.0, 7.0], [8.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 3.0], [2.0, 8.0, 7.0], [8.0, 1.0, 7.0]], 
[[7.0, 3.0, 4.0], [7.0, 5.0, 2.0], [7.0, 2.0, 3.0]], [[7.0, 5.0, 8.0], [4.0, 7.0, 4.0], [3.0, 4.0, 4.0]]], [[[6.0, 7.0, 1.0], 
[1.0, 3.0, 2.0], [6.0, 5.0, 4.0]], [[7.0, 1.0, 5.0], [1.0, 2.0, 7.0], [6.0, 5.0, 3.0]], [[3.0, 8.0, 4.0], [5.0, 8.0, 7.0], 
[6.0, 2.0, 6.0]]], [[[8.0, 1.0, 6.0], [5.0, 4.0, 8.0], [3.0, 2.0, 4.0]], [[4.0, 8.0, 3.0], [2.0, 6.0, 6.0], [4.0, 2.0, 1.0]], 
[[1.0, 1.0, 3.0], [3.0, 6.0, 8.0], [5.0, 2.0, 7.0]]], [[[6.0, 6.0, 2.0], [1.0, 5.0, 6.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 4.0], 
[7.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[5.0, 6.0, 4.0], [4.0, 6.0, 7.0], [8.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RCCN(self):
      arg=Data(numpy.array([[[[2.0, 7.0, 6.0], [7.0, 4.0, 1.0], [4.0, 2.0, 7.0]], [[4.0, 4.0, 5.0], [3.0, 7.0, 6.0], [5.0, 3.0, 
5.0]], [[6.0, 2.0, 1.0], [2.0, 7.0, 3.0], [7.0, 2.0, 4.0]]], [[[4.0, 4.0, 3.0], [8.0, 4.0, 5.0], [5.0, 5.0, 3.0]], [[5.0, 8.0, 
3.0], [4.0, 5.0, 4.0], [3.0, 2.0, 4.0]], [[2.0, 8.0, 1.0], [2.0, 3.0, 5.0], [8.0, 2.0, 1.0]]], [[[8.0, 5.0, 4.0], [3.0, 4.0, 
8.0], [4.0, 1.0, 6.0]], [[5.0, 2.0, 3.0], [4.0, 8.0, 8.0], [8.0, 7.0, 5.0]], [[3.0, 1.0, 7.0], [8.0, 1.0, 1.0], [1.0, 7.0, 
4.0]]], [[[3.0, 3.0, 7.0], [2.0, 4.0, 4.0], [5.0, 7.0, 4.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 1.0], [1.0, 3.0, 7.0]], [[7.0, 3.0, 
4.0], [1.0, 2.0, 7.0], [6.0, 7.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[3.0, 3.0, 1.0], [7.0, 3.0, 5.0], [8.0, 7.0, 6.0]], [[4.0, 4.0, 7.0], [4.0, 3.0, 6.0], [1.0, 3.0, 
2.0]], [[4.0, 5.0, 6.0], [5.0, 7.0, 8.0], [5.0, 3.0, 2.0]]], [[[8.0, 7.0, 8.0], [7.0, 1.0, 7.0], [3.0, 6.0, 6.0]], [[1.0, 2.0, 
8.0], [3.0, 1.0, 2.0], [6.0, 7.0, 6.0]], [[3.0, 7.0, 1.0], [4.0, 4.0, 6.0], [2.0, 3.0, 2.0]]]])
      arg[1:3,:,:]=val
      ref=Data(numpy.array([[[[2.0, 7.0, 6.0], [7.0, 4.0, 1.0], [4.0, 2.0, 7.0]], [[4.0, 4.0, 5.0], [3.0, 7.0, 6.0], [5.0, 3.0, 
5.0]], [[6.0, 2.0, 1.0], [2.0, 7.0, 3.0], [7.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], [7.0, 3.0, 5.0], [8.0, 7.0, 6.0]], [[4.0, 4.0, 
7.0], [4.0, 3.0, 6.0], [1.0, 3.0, 2.0]], [[4.0, 5.0, 6.0], [5.0, 7.0, 8.0], [5.0, 3.0, 2.0]]], [[[8.0, 7.0, 8.0], [7.0, 1.0, 
7.0], [3.0, 6.0, 6.0]], [[1.0, 2.0, 8.0], [3.0, 1.0, 2.0], [6.0, 7.0, 6.0]], [[3.0, 7.0, 1.0], [4.0, 4.0, 6.0], [2.0, 3.0, 
2.0]]], [[[3.0, 3.0, 7.0], [2.0, 4.0, 4.0], [5.0, 7.0, 4.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 1.0], [1.0, 3.0, 7.0]], [[7.0, 3.0, 
4.0], [1.0, 2.0, 7.0], [6.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RCCN(self):
      arg=Data(numpy.array([[[[2.0, 3.0, 1.0], [7.0, 5.0, 4.0], [4.0, 3.0, 1.0]], [[2.0, 5.0, 2.0], [1.0, 5.0, 8.0], [3.0, 1.0, 
1.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 3.0], [2.0, 6.0, 6.0]]], [[[2.0, 5.0, 6.0], [6.0, 3.0, 4.0], [3.0, 4.0, 7.0]], [[7.0, 5.0, 
6.0], [4.0, 5.0, 8.0], [8.0, 6.0, 1.0]], [[1.0, 2.0, 8.0], [7.0, 5.0, 8.0], [3.0, 8.0, 5.0]]], [[[4.0, 7.0, 1.0], [1.0, 8.0, 
7.0], [6.0, 7.0, 2.0]], [[8.0, 2.0, 4.0], [1.0, 3.0, 2.0], [2.0, 2.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 5.0, 7.0], [8.0, 7.0, 
8.0]]], [[[8.0, 7.0, 5.0], [8.0, 4.0, 6.0], [2.0, 3.0, 8.0]], [[3.0, 7.0, 1.0], [5.0, 2.0, 1.0], [4.0, 7.0, 7.0]], [[6.0, 4.0, 
1.0], [2.0, 1.0, 3.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[1.0, 6.0, 4.0], [7.0, 2.0, 6.0], [8.0, 1.0, 2.0]], [[8.0, 2.0, 6.0], [7.0, 8.0, 3.0], [2.0, 8.0, 
8.0]], [[3.0, 2.0, 7.0], [8.0, 7.0, 8.0], [1.0, 4.0, 6.0]]], [[[5.0, 6.0, 2.0], [5.0, 7.0, 4.0], [5.0, 4.0, 6.0]], [[1.0, 7.0, 
2.0], [2.0, 4.0, 4.0], [1.0, 7.0, 4.0]], [[1.0, 4.0, 5.0], [2.0, 8.0, 8.0], [5.0, 5.0, 1.0]]]]),self.functionspace)
      arg[1:3,:,:]=val
      ref=Data(numpy.array([[[[2.0, 3.0, 1.0], [7.0, 5.0, 4.0], [4.0, 3.0, 1.0]], [[2.0, 5.0, 2.0], [1.0, 5.0, 8.0], [3.0, 1.0, 
1.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 3.0], [2.0, 6.0, 6.0]]], [[[1.0, 6.0, 4.0], [7.0, 2.0, 6.0], [8.0, 1.0, 2.0]], [[8.0, 2.0, 
6.0], [7.0, 8.0, 3.0], [2.0, 8.0, 8.0]], [[3.0, 2.0, 7.0], [8.0, 7.0, 8.0], [1.0, 4.0, 6.0]]], [[[5.0, 6.0, 2.0], [5.0, 7.0, 
4.0], [5.0, 4.0, 6.0]], [[1.0, 7.0, 2.0], [2.0, 4.0, 4.0], [1.0, 7.0, 4.0]], [[1.0, 4.0, 5.0], [2.0, 8.0, 8.0], [5.0, 5.0, 
1.0]]], [[[8.0, 7.0, 5.0], [8.0, 4.0, 6.0], [2.0, 3.0, 8.0]], [[3.0, 7.0, 1.0], [5.0, 2.0, 1.0], [4.0, 7.0, 7.0]], [[6.0, 4.0, 
1.0], [2.0, 1.0, 3.0], [8.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RCCN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 5.0], [1.0, 7.0, 8.0], [3.0, 8.0, 2.0]], [[8.0, 4.0, 8.0], [5.0, 8.0, 3.0], [3.0, 8.0, 
6.0]], [[4.0, 6.0, 4.0], [2.0, 8.0, 4.0], [7.0, 6.0, 4.0]]], [[[2.0, 5.0, 6.0], [6.0, 3.0, 4.0], [4.0, 4.0, 8.0]], [[3.0, 5.0, 
3.0], [3.0, 1.0, 3.0], [3.0, 5.0, 7.0]], [[6.0, 3.0, 6.0], [3.0, 7.0, 8.0], [3.0, 2.0, 3.0]]], [[[8.0, 7.0, 6.0], [5.0, 2.0, 
4.0], [3.0, 7.0, 5.0]], [[1.0, 4.0, 2.0], [5.0, 3.0, 7.0], [3.0, 1.0, 4.0]], [[8.0, 2.0, 4.0], [7.0, 6.0, 1.0], [1.0, 4.0, 
8.0]]], [[[8.0, 2.0, 5.0], [8.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[5.0, 4.0, 3.0], [2.0, 4.0, 8.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 
3.0], [4.0, 3.0, 4.0], [6.0, 7.0, 6.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 6.0, 4.0], [3.0, 7.0, 1.0], [2.0, 1.0, 5.0]], [[5.0, 5.0, 5.0], [2.0, 3.0, 2.0], [4.0, 
2.0, 2.0]], [[8.0, 2.0, 5.0], [3.0, 4.0, 4.0], [3.0, 3.0, 4.0]]], [[[3.0, 2.0, 4.0], [7.0, 6.0, 3.0], [3.0, 5.0, 8.0]], [[8.0, 
6.0, 1.0], [2.0, 8.0, 6.0], [4.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], [2.0, 1.0, 4.0], [1.0, 4.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 6.0, 7.0], [6.0, 1.0, 4.0], [4.0, 4.0, 1.0]], [[3.0, 8.0, 7.0], [5.0, 1.0, 7.0], 
[3.0, 1.0, 7.0]], [[5.0, 5.0, 7.0], [5.0, 4.0, 5.0], [2.0, 7.0, 1.0]]], [[[6.0, 3.0, 4.0], [2.0, 3.0, 7.0], [1.0, 6.0, 7.0]], 
[[5.0, 7.0, 3.0], [5.0, 4.0, 7.0], [5.0, 1.0, 7.0]], [[5.0, 1.0, 6.0], [2.0, 8.0, 8.0], [3.0, 4.0, 1.0]]]])
      arg[1:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 5.0], [1.0, 7.0, 8.0], [3.0, 8.0, 2.0]], [[8.0, 4.0, 8.0], [5.0, 8.0, 3.0], [3.0, 
8.0, 6.0]], [[4.0, 6.0, 4.0], [2.0, 8.0, 4.0], [7.0, 6.0, 4.0]]], [[[8.0, 6.0, 4.0], [3.0, 7.0, 1.0], [2.0, 1.0, 5.0]], [[5.0, 
5.0, 5.0], [2.0, 3.0, 2.0], [4.0, 2.0, 2.0]], [[8.0, 2.0, 5.0], [3.0, 4.0, 4.0], [3.0, 3.0, 4.0]]], [[[3.0, 2.0, 4.0], [7.0, 
6.0, 3.0], [3.0, 5.0, 8.0]], [[8.0, 6.0, 1.0], [2.0, 8.0, 6.0], [4.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], [2.0, 1.0, 4.0], [1.0, 4.0, 
7.0]]], [[[8.0, 2.0, 5.0], [8.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[5.0, 4.0, 3.0], [2.0, 4.0, 8.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 
3.0], [4.0, 3.0, 4.0], [6.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 5.0], [1.0, 7.0, 8.0], [3.0, 8.0, 2.0]], 
[[8.0, 4.0, 8.0], [5.0, 8.0, 3.0], [3.0, 8.0, 6.0]], [[4.0, 6.0, 4.0], [2.0, 8.0, 4.0], [7.0, 6.0, 4.0]]], [[[4.0, 6.0, 7.0], 
[6.0, 1.0, 4.0], [4.0, 4.0, 1.0]], [[3.0, 8.0, 7.0], [5.0, 1.0, 7.0], [3.0, 1.0, 7.0]], [[5.0, 5.0, 7.0], [5.0, 4.0, 5.0], 
[2.0, 7.0, 1.0]]], [[[6.0, 3.0, 4.0], [2.0, 3.0, 7.0], [1.0, 6.0, 7.0]], [[5.0, 7.0, 3.0], [5.0, 4.0, 7.0], [5.0, 1.0, 7.0]], 
[[5.0, 1.0, 6.0], [2.0, 8.0, 8.0], [3.0, 4.0, 1.0]]], [[[8.0, 2.0, 5.0], [8.0, 3.0, 3.0], [8.0, 3.0, 8.0]], [[5.0, 4.0, 3.0], 
[2.0, 4.0, 8.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 3.0], [4.0, 3.0, 4.0], [6.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_UPLN(self):
      arg=Data(numpy.array([[[[1.0, 8.0, 3.0], [4.0, 3.0, 4.0], [5.0, 1.0, 1.0]], [[8.0, 5.0, 3.0], [5.0, 2.0, 3.0], [3.0, 6.0, 
8.0]], [[5.0, 8.0, 5.0], [7.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 2.0, 5.0], [4.0, 8.0, 1.0], [1.0, 8.0, 6.0]], [[8.0, 4.0, 
7.0], [7.0, 4.0, 2.0], [3.0, 5.0, 7.0]], [[2.0, 6.0, 7.0], [1.0, 5.0, 2.0], [1.0, 8.0, 8.0]]], [[[6.0, 6.0, 3.0], [6.0, 2.0, 
1.0], [5.0, 8.0, 2.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 1.0], [1.0, 1.0, 8.0]], [[2.0, 7.0, 8.0], [1.0, 5.0, 3.0], [6.0, 6.0, 
8.0]]], [[[8.0, 1.0, 1.0], [6.0, 1.0, 1.0], [4.0, 1.0, 4.0]], [[3.0, 2.0, 6.0], [1.0, 5.0, 8.0], [4.0, 1.0, 2.0]], [[3.0, 3.0, 
2.0], [3.0, 3.0, 6.0], [6.0, 5.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 1.0, 7.0]], [[2.0, 3.0, 2.0]], [[3.0, 3.0, 6.0]]])
      arg[:3,1,2:]=val
      ref=Data(numpy.array([[[[1.0, 8.0, 3.0], [4.0, 3.0, 4.0], [5.0, 1.0, 1.0]], [[8.0, 5.0, 3.0], [5.0, 2.0, 3.0], [7.0, 1.0, 
7.0]], [[5.0, 8.0, 5.0], [7.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 2.0, 5.0], [4.0, 8.0, 1.0], [1.0, 8.0, 6.0]], [[8.0, 4.0, 
7.0], [7.0, 4.0, 2.0], [2.0, 3.0, 2.0]], [[2.0, 6.0, 7.0], [1.0, 5.0, 2.0], [1.0, 8.0, 8.0]]], [[[6.0, 6.0, 3.0], [6.0, 2.0, 
1.0], [5.0, 8.0, 2.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 1.0], [3.0, 3.0, 6.0]], [[2.0, 7.0, 8.0], [1.0, 5.0, 3.0], [6.0, 6.0, 
8.0]]], [[[8.0, 1.0, 1.0], [6.0, 1.0, 1.0], [4.0, 1.0, 4.0]], [[3.0, 2.0, 6.0], [1.0, 5.0, 8.0], [4.0, 1.0, 2.0]], [[3.0, 3.0, 
2.0], [3.0, 3.0, 6.0], [6.0, 5.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_UPLN(self):
      arg=Data(numpy.array([[[[6.0, 6.0, 2.0], [7.0, 8.0, 7.0], [7.0, 2.0, 3.0]], [[1.0, 1.0, 5.0], [8.0, 6.0, 1.0], [7.0, 2.0, 
3.0]], [[5.0, 1.0, 3.0], [5.0, 5.0, 3.0], [4.0, 6.0, 4.0]]], [[[6.0, 5.0, 3.0], [4.0, 6.0, 2.0], [1.0, 8.0, 6.0]], [[2.0, 4.0, 
8.0], [6.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[2.0, 6.0, 7.0], [8.0, 3.0, 5.0], [1.0, 1.0, 4.0]]], [[[5.0, 1.0, 5.0], [1.0, 8.0, 
1.0], [8.0, 3.0, 8.0]], [[5.0, 3.0, 4.0], [1.0, 3.0, 3.0], [7.0, 4.0, 1.0]], [[7.0, 8.0, 4.0], [6.0, 1.0, 2.0], [5.0, 4.0, 
2.0]]], [[[2.0, 1.0, 4.0], [1.0, 1.0, 7.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 1.0], [5.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[1.0, 6.0, 
7.0], [3.0, 6.0, 3.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 4.0, 8.0]], [[2.0, 7.0, 4.0]], [[6.0, 4.0, 2.0]]]),self.functionspace)
      arg[:3,1,2:]=val
      ref=Data(numpy.array([[[[6.0, 6.0, 2.0], [7.0, 8.0, 7.0], [7.0, 2.0, 3.0]], [[1.0, 1.0, 5.0], [8.0, 6.0, 1.0], [6.0, 4.0, 
8.0]], [[5.0, 1.0, 3.0], [5.0, 5.0, 3.0], [4.0, 6.0, 4.0]]], [[[6.0, 5.0, 3.0], [4.0, 6.0, 2.0], [1.0, 8.0, 6.0]], [[2.0, 4.0, 
8.0], [6.0, 2.0, 6.0], [2.0, 7.0, 4.0]], [[2.0, 6.0, 7.0], [8.0, 3.0, 5.0], [1.0, 1.0, 4.0]]], [[[5.0, 1.0, 5.0], [1.0, 8.0, 
1.0], [8.0, 3.0, 8.0]], [[5.0, 3.0, 4.0], [1.0, 3.0, 3.0], [6.0, 4.0, 2.0]], [[7.0, 8.0, 4.0], [6.0, 1.0, 2.0], [5.0, 4.0, 
2.0]]], [[[2.0, 1.0, 4.0], [1.0, 1.0, 7.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 1.0], [5.0, 1.0, 5.0], [8.0, 5.0, 8.0]], [[1.0, 6.0, 
7.0], [3.0, 6.0, 3.0], [8.0, 3.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_UPLN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 5.0], [4.0, 4.0, 4.0], [6.0, 8.0, 6.0]], [[6.0, 2.0, 6.0], [7.0, 8.0, 7.0], [5.0, 2.0, 
4.0]], [[8.0, 1.0, 5.0], [6.0, 3.0, 6.0], [3.0, 3.0, 3.0]]], [[[1.0, 4.0, 6.0], [1.0, 3.0, 8.0], [1.0, 6.0, 2.0]], [[3.0, 4.0, 
2.0], [6.0, 8.0, 3.0], [4.0, 7.0, 3.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 4.0], [1.0, 1.0, 6.0]]], [[[1.0, 4.0, 7.0], [7.0, 3.0, 
8.0], [5.0, 4.0, 6.0]], [[8.0, 1.0, 8.0], [8.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[2.0, 4.0, 7.0], [3.0, 1.0, 1.0], [8.0, 4.0, 
3.0]]], [[[7.0, 7.0, 4.0], [3.0, 2.0, 7.0], [1.0, 1.0, 4.0]], [[6.0, 4.0, 4.0], [7.0, 3.0, 1.0], [7.0, 4.0, 5.0]], [[6.0, 4.0, 
7.0], [8.0, 8.0, 8.0], [6.0, 8.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 4.0, 2.0]], [[6.0, 1.0, 7.0]], [[8.0, 4.0, 2.0]]])+(1.-msk_val)*numpy.array([[[3.0, 4.0, 
2.0]], [[3.0, 5.0, 8.0]], [[5.0, 1.0, 4.0]]])
      arg[:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [4.0, 4.0, 4.0], [6.0, 8.0, 6.0]], [[6.0, 2.0, 6.0], [7.0, 8.0, 7.0], [8.0, 
4.0, 2.0]], [[8.0, 1.0, 5.0], [6.0, 3.0, 6.0], [3.0, 3.0, 3.0]]], [[[1.0, 4.0, 6.0], [1.0, 3.0, 8.0], [1.0, 6.0, 2.0]], [[3.0, 
4.0, 2.0], [6.0, 8.0, 3.0], [6.0, 1.0, 7.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 4.0], [1.0, 1.0, 6.0]]], [[[1.0, 4.0, 7.0], [7.0, 
3.0, 8.0], [5.0, 4.0, 6.0]], [[8.0, 1.0, 8.0], [8.0, 3.0, 5.0], [8.0, 4.0, 2.0]], [[2.0, 4.0, 7.0], [3.0, 1.0, 1.0], [8.0, 4.0, 
3.0]]], [[[7.0, 7.0, 4.0], [3.0, 2.0, 7.0], [1.0, 1.0, 4.0]], [[6.0, 4.0, 4.0], [7.0, 3.0, 1.0], [7.0, 4.0, 5.0]], [[6.0, 4.0, 
7.0], [8.0, 8.0, 8.0], [6.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 5.0], [4.0, 4.0, 4.0], [6.0, 8.0, 6.0]], 
[[6.0, 2.0, 6.0], [7.0, 8.0, 7.0], [3.0, 4.0, 2.0]], [[8.0, 1.0, 5.0], [6.0, 3.0, 6.0], [3.0, 3.0, 3.0]]], [[[1.0, 4.0, 6.0], 
[1.0, 3.0, 8.0], [1.0, 6.0, 2.0]], [[3.0, 4.0, 2.0], [6.0, 8.0, 3.0], [3.0, 5.0, 8.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 4.0], 
[1.0, 1.0, 6.0]]], [[[1.0, 4.0, 7.0], [7.0, 3.0, 8.0], [5.0, 4.0, 6.0]], [[8.0, 1.0, 8.0], [8.0, 3.0, 5.0], [5.0, 1.0, 4.0]], 
[[2.0, 4.0, 7.0], [3.0, 1.0, 1.0], [8.0, 4.0, 3.0]]], [[[7.0, 7.0, 4.0], [3.0, 2.0, 7.0], [1.0, 1.0, 4.0]], [[6.0, 4.0, 4.0], 
[7.0, 3.0, 1.0], [7.0, 4.0, 5.0]], [[6.0, 4.0, 7.0], [8.0, 8.0, 8.0], [6.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_PUCN(self):
      arg=Data(numpy.array([[[[3.0, 1.0, 6.0], [1.0, 3.0, 1.0], [3.0, 4.0, 3.0]], [[6.0, 5.0, 2.0], [6.0, 7.0, 1.0], [7.0, 6.0, 
6.0]], [[2.0, 6.0, 2.0], [2.0, 7.0, 3.0], [7.0, 8.0, 4.0]]], [[[1.0, 7.0, 4.0], [4.0, 3.0, 1.0], [6.0, 8.0, 6.0]], [[8.0, 2.0, 
8.0], [6.0, 7.0, 1.0], [4.0, 6.0, 5.0]], [[2.0, 8.0, 7.0], [4.0, 8.0, 6.0], [2.0, 5.0, 7.0]]], [[[3.0, 5.0, 5.0], [5.0, 7.0, 
4.0], [5.0, 6.0, 4.0]], [[3.0, 1.0, 7.0], [5.0, 5.0, 3.0], [8.0, 6.0, 3.0]], [[2.0, 2.0, 3.0], [2.0, 5.0, 8.0], [7.0, 8.0, 
3.0]]], [[[2.0, 3.0, 5.0], [1.0, 7.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 8.0, 5.0], [8.0, 4.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 8.0, 
3.0], [8.0, 3.0, 6.0], [1.0, 3.0, 3.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 1.0, 5.0], [4.0, 7.0, 2.0], [5.0, 6.0, 7.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 4.0], [8.0, 6.0, 
8.0]]])
      arg[2,:2,:]=val
      ref=Data(numpy.array([[[[3.0, 1.0, 6.0], [1.0, 3.0, 1.0], [3.0, 4.0, 3.0]], [[6.0, 5.0, 2.0], [6.0, 7.0, 1.0], [7.0, 6.0, 
6.0]], [[2.0, 6.0, 2.0], [2.0, 7.0, 3.0], [7.0, 8.0, 4.0]]], [[[1.0, 7.0, 4.0], [4.0, 3.0, 1.0], [6.0, 8.0, 6.0]], [[8.0, 2.0, 
8.0], [6.0, 7.0, 1.0], [4.0, 6.0, 5.0]], [[2.0, 8.0, 7.0], [4.0, 8.0, 6.0], [2.0, 5.0, 7.0]]], [[[5.0, 1.0, 5.0], [4.0, 7.0, 
2.0], [5.0, 6.0, 7.0]], [[2.0, 2.0, 8.0], [6.0, 8.0, 4.0], [8.0, 6.0, 8.0]], [[2.0, 2.0, 3.0], [2.0, 5.0, 8.0], [7.0, 8.0, 
3.0]]], [[[2.0, 3.0, 5.0], [1.0, 7.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 8.0, 5.0], [8.0, 4.0, 2.0], [6.0, 7.0, 3.0]], [[6.0, 8.0, 
3.0], [8.0, 3.0, 6.0], [1.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_PUCN(self):
      arg=Data(numpy.array([[[[8.0, 2.0, 6.0], [8.0, 7.0, 5.0], [6.0, 8.0, 8.0]], [[2.0, 6.0, 1.0], [3.0, 7.0, 8.0], [6.0, 8.0, 
8.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 5.0], [8.0, 6.0, 6.0]]], [[[6.0, 2.0, 7.0], [5.0, 3.0, 1.0], [3.0, 3.0, 1.0]], [[8.0, 5.0, 
3.0], [6.0, 2.0, 5.0], [7.0, 3.0, 5.0]], [[5.0, 7.0, 1.0], [4.0, 3.0, 3.0], [6.0, 1.0, 5.0]]], [[[6.0, 3.0, 1.0], [6.0, 2.0, 
3.0], [1.0, 6.0, 2.0]], [[4.0, 3.0, 4.0], [8.0, 2.0, 8.0], [7.0, 4.0, 5.0]], [[3.0, 5.0, 6.0], [3.0, 1.0, 8.0], [4.0, 8.0, 
2.0]]], [[[4.0, 8.0, 2.0], [6.0, 5.0, 5.0], [1.0, 8.0, 2.0]], [[4.0, 3.0, 1.0], [2.0, 2.0, 6.0], [6.0, 4.0, 5.0]], [[1.0, 5.0, 
6.0], [7.0, 8.0, 5.0], [8.0, 3.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[6.0, 1.0, 7.0], [7.0, 2.0, 7.0], [1.0, 6.0, 6.0]], [[2.0, 8.0, 2.0], [1.0, 2.0, 2.0], [5.0, 8.0, 
2.0]]]),self.functionspace)
      arg[2,:2,:]=val
      ref=Data(numpy.array([[[[8.0, 2.0, 6.0], [8.0, 7.0, 5.0], [6.0, 8.0, 8.0]], [[2.0, 6.0, 1.0], [3.0, 7.0, 8.0], [6.0, 8.0, 
8.0]], [[5.0, 2.0, 1.0], [4.0, 8.0, 5.0], [8.0, 6.0, 6.0]]], [[[6.0, 2.0, 7.0], [5.0, 3.0, 1.0], [3.0, 3.0, 1.0]], [[8.0, 5.0, 
3.0], [6.0, 2.0, 5.0], [7.0, 3.0, 5.0]], [[5.0, 7.0, 1.0], [4.0, 3.0, 3.0], [6.0, 1.0, 5.0]]], [[[6.0, 1.0, 7.0], [7.0, 2.0, 
7.0], [1.0, 6.0, 6.0]], [[2.0, 8.0, 2.0], [1.0, 2.0, 2.0], [5.0, 8.0, 2.0]], [[3.0, 5.0, 6.0], [3.0, 1.0, 8.0], [4.0, 8.0, 
2.0]]], [[[4.0, 8.0, 2.0], [6.0, 5.0, 5.0], [1.0, 8.0, 2.0]], [[4.0, 3.0, 1.0], [2.0, 2.0, 6.0], [6.0, 4.0, 5.0]], [[1.0, 5.0, 
6.0], [7.0, 8.0, 5.0], [8.0, 3.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_PUCN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 6.0], [2.0, 2.0, 3.0], [4.0, 4.0, 5.0]], [[1.0, 7.0, 4.0], [5.0, 6.0, 7.0], [8.0, 6.0, 
3.0]], [[6.0, 4.0, 8.0], [3.0, 5.0, 6.0], [6.0, 3.0, 6.0]]], [[[8.0, 3.0, 2.0], [5.0, 7.0, 6.0], [6.0, 2.0, 2.0]], [[4.0, 4.0, 
1.0], [8.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 4.0, 1.0], [2.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], [2.0, 8.0, 
3.0], [2.0, 4.0, 5.0]], [[4.0, 7.0, 5.0], [4.0, 4.0, 7.0], [7.0, 7.0, 2.0]], [[1.0, 5.0, 8.0], [6.0, 1.0, 3.0], [8.0, 7.0, 
4.0]]], [[[4.0, 3.0, 3.0], [6.0, 6.0, 4.0], [1.0, 8.0, 2.0]], [[6.0, 6.0, 8.0], [7.0, 3.0, 8.0], [5.0, 5.0, 1.0]], [[3.0, 7.0, 
8.0], [7.0, 2.0, 1.0], [1.0, 2.0, 1.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 7.0, 6.0], [3.0, 8.0, 4.0], [6.0, 8.0, 2.0]], [[2.0, 3.0, 1.0], [1.0, 2.0, 6.0], [2.0, 
7.0, 5.0]]])+(1.-msk_val)*numpy.array([[[3.0, 6.0, 8.0], [8.0, 4.0, 6.0], [5.0, 8.0, 6.0]], [[3.0, 7.0, 1.0], [1.0, 4.0, 5.0], 
[3.0, 1.0, 7.0]]])
      arg[2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 6.0], [2.0, 2.0, 3.0], [4.0, 4.0, 5.0]], [[1.0, 7.0, 4.0], [5.0, 6.0, 7.0], [8.0, 
6.0, 3.0]], [[6.0, 4.0, 8.0], [3.0, 5.0, 6.0], [6.0, 3.0, 6.0]]], [[[8.0, 3.0, 2.0], [5.0, 7.0, 6.0], [6.0, 2.0, 2.0]], [[4.0, 
4.0, 1.0], [8.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 4.0, 1.0], [2.0, 2.0, 4.0]]], [[[6.0, 7.0, 6.0], [3.0, 
8.0, 4.0], [6.0, 8.0, 2.0]], [[2.0, 3.0, 1.0], [1.0, 2.0, 6.0], [2.0, 7.0, 5.0]], [[1.0, 5.0, 8.0], [6.0, 1.0, 3.0], [8.0, 7.0, 
4.0]]], [[[4.0, 3.0, 3.0], [6.0, 6.0, 4.0], [1.0, 8.0, 2.0]], [[6.0, 6.0, 8.0], [7.0, 3.0, 8.0], [5.0, 5.0, 1.0]], [[3.0, 7.0, 
8.0], [7.0, 2.0, 1.0], [1.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 6.0], [2.0, 2.0, 3.0], [4.0, 4.0, 5.0]], 
[[1.0, 7.0, 4.0], [5.0, 6.0, 7.0], [8.0, 6.0, 3.0]], [[6.0, 4.0, 8.0], [3.0, 5.0, 6.0], [6.0, 3.0, 6.0]]], [[[8.0, 3.0, 2.0], 
[5.0, 7.0, 6.0], [6.0, 2.0, 2.0]], [[4.0, 4.0, 1.0], [8.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 4.0, 1.0], 
[2.0, 2.0, 4.0]]], [[[3.0, 6.0, 8.0], [8.0, 4.0, 6.0], [5.0, 8.0, 6.0]], [[3.0, 7.0, 1.0], [1.0, 4.0, 5.0], [3.0, 1.0, 7.0]], 
[[1.0, 5.0, 8.0], [6.0, 1.0, 3.0], [8.0, 7.0, 4.0]]], [[[4.0, 3.0, 3.0], [6.0, 6.0, 4.0], [1.0, 8.0, 2.0]], [[6.0, 6.0, 8.0], 
[7.0, 3.0, 8.0], [5.0, 5.0, 1.0]], [[3.0, 7.0, 8.0], [7.0, 2.0, 1.0], [1.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LCLN(self):
      arg=Data(numpy.array([[[[5.0, 7.0, 5.0], [2.0, 2.0, 3.0], [8.0, 6.0, 2.0]], [[7.0, 6.0, 7.0], [4.0, 4.0, 6.0], [8.0, 4.0, 
6.0]], [[6.0, 2.0, 8.0], [5.0, 1.0, 7.0], [2.0, 6.0, 8.0]]], [[[7.0, 6.0, 8.0], [6.0, 4.0, 2.0], [2.0, 1.0, 5.0]], [[6.0, 4.0, 
6.0], [8.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[7.0, 7.0, 3.0], [5.0, 3.0, 5.0], [2.0, 8.0, 6.0]]], [[[5.0, 5.0, 6.0], [7.0, 1.0, 
7.0], [8.0, 4.0, 7.0]], [[6.0, 2.0, 5.0], [8.0, 1.0, 2.0], [3.0, 2.0, 3.0]], [[6.0, 7.0, 3.0], [6.0, 7.0, 2.0], [3.0, 7.0, 
4.0]]], [[[5.0, 3.0, 6.0], [2.0, 6.0, 7.0], [1.0, 6.0, 6.0]], [[1.0, 3.0, 8.0], [5.0, 1.0, 5.0], [7.0, 2.0, 3.0]], [[8.0, 3.0, 
5.0], [1.0, 5.0, 6.0], [3.0, 8.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 6.0, 3.0]], [[2.0, 2.0, 7.0]], [[5.0, 4.0, 6.0]]], [[[6.0, 3.0, 2.0]], [[3.0, 5.0, 6.0]], [[8.0, 
3.0, 3.0]]]])
      arg[2:,:,2:]=val
      ref=Data(numpy.array([[[[5.0, 7.0, 5.0], [2.0, 2.0, 3.0], [8.0, 6.0, 2.0]], [[7.0, 6.0, 7.0], [4.0, 4.0, 6.0], [8.0, 4.0, 
6.0]], [[6.0, 2.0, 8.0], [5.0, 1.0, 7.0], [2.0, 6.0, 8.0]]], [[[7.0, 6.0, 8.0], [6.0, 4.0, 2.0], [2.0, 1.0, 5.0]], [[6.0, 4.0, 
6.0], [8.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[7.0, 7.0, 3.0], [5.0, 3.0, 5.0], [2.0, 8.0, 6.0]]], [[[5.0, 5.0, 6.0], [7.0, 1.0, 
7.0], [6.0, 6.0, 3.0]], [[6.0, 2.0, 5.0], [8.0, 1.0, 2.0], [2.0, 2.0, 7.0]], [[6.0, 7.0, 3.0], [6.0, 7.0, 2.0], [5.0, 4.0, 
6.0]]], [[[5.0, 3.0, 6.0], [2.0, 6.0, 7.0], [6.0, 3.0, 2.0]], [[1.0, 3.0, 8.0], [5.0, 1.0, 5.0], [3.0, 5.0, 6.0]], [[8.0, 3.0, 
5.0], [1.0, 5.0, 6.0], [8.0, 3.0, 3.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LCLN(self):
      arg=Data(numpy.array([[[[4.0, 2.0, 2.0], [8.0, 6.0, 5.0], [4.0, 1.0, 3.0]], [[6.0, 5.0, 8.0], [1.0, 6.0, 3.0], [2.0, 6.0, 
2.0]], [[1.0, 8.0, 2.0], [1.0, 4.0, 1.0], [1.0, 7.0, 4.0]]], [[[6.0, 6.0, 7.0], [6.0, 6.0, 2.0], [3.0, 1.0, 2.0]], [[2.0, 5.0, 
6.0], [3.0, 6.0, 2.0], [4.0, 1.0, 8.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 8.0], [4.0, 6.0, 8.0]]], [[[6.0, 6.0, 4.0], [2.0, 1.0, 
7.0], [1.0, 2.0, 3.0]], [[5.0, 6.0, 5.0], [7.0, 7.0, 1.0], [5.0, 4.0, 5.0]], [[2.0, 5.0, 2.0], [8.0, 6.0, 3.0], [4.0, 1.0, 
6.0]]], [[[6.0, 5.0, 2.0], [8.0, 5.0, 3.0], [6.0, 8.0, 8.0]], [[7.0, 8.0, 6.0], [7.0, 8.0, 8.0], [6.0, 7.0, 7.0]], [[3.0, 7.0, 
2.0], [1.0, 8.0, 2.0], [8.0, 8.0, 3.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 1.0, 8.0]], [[4.0, 7.0, 6.0]], [[5.0, 3.0, 1.0]]], [[[5.0, 3.0, 8.0]], [[7.0, 2.0, 7.0]], 
[[3.0, 1.0, 6.0]]]]),self.functionspace)
      arg[2:,:,2:]=val
      ref=Data(numpy.array([[[[4.0, 2.0, 2.0], [8.0, 6.0, 5.0], [4.0, 1.0, 3.0]], [[6.0, 5.0, 8.0], [1.0, 6.0, 3.0], [2.0, 6.0, 
2.0]], [[1.0, 8.0, 2.0], [1.0, 4.0, 1.0], [1.0, 7.0, 4.0]]], [[[6.0, 6.0, 7.0], [6.0, 6.0, 2.0], [3.0, 1.0, 2.0]], [[2.0, 5.0, 
6.0], [3.0, 6.0, 2.0], [4.0, 1.0, 8.0]], [[4.0, 5.0, 2.0], [3.0, 8.0, 8.0], [4.0, 6.0, 8.0]]], [[[6.0, 6.0, 4.0], [2.0, 1.0, 
7.0], [6.0, 1.0, 8.0]], [[5.0, 6.0, 5.0], [7.0, 7.0, 1.0], [4.0, 7.0, 6.0]], [[2.0, 5.0, 2.0], [8.0, 6.0, 3.0], [5.0, 3.0, 
1.0]]], [[[6.0, 5.0, 2.0], [8.0, 5.0, 3.0], [5.0, 3.0, 8.0]], [[7.0, 8.0, 6.0], [7.0, 8.0, 8.0], [7.0, 2.0, 7.0]], [[3.0, 7.0, 
2.0], [1.0, 8.0, 2.0], [3.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LCLN(self):
      arg=Data(numpy.array([[[[8.0, 3.0, 5.0], [8.0, 7.0, 3.0], [8.0, 4.0, 4.0]], [[1.0, 2.0, 1.0], [2.0, 3.0, 1.0], [1.0, 6.0, 
4.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 2.0], [3.0, 5.0, 2.0]]], [[[4.0, 4.0, 3.0], [6.0, 8.0, 1.0], [3.0, 2.0, 1.0]], [[5.0, 8.0, 
7.0], [6.0, 2.0, 1.0], [4.0, 1.0, 6.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 7.0], [4.0, 5.0, 8.0]]], [[[8.0, 1.0, 2.0], [7.0, 8.0, 
3.0], [2.0, 3.0, 5.0]], [[5.0, 3.0, 8.0], [5.0, 7.0, 8.0], [3.0, 6.0, 8.0]], [[3.0, 7.0, 6.0], [2.0, 4.0, 6.0], [7.0, 3.0, 
8.0]]], [[[5.0, 2.0, 2.0], [6.0, 4.0, 6.0], [5.0, 1.0, 1.0]], [[6.0, 8.0, 4.0], [1.0, 8.0, 8.0], [8.0, 7.0, 2.0]], [[6.0, 3.0, 
6.0], [3.0, 1.0, 7.0], [7.0, 1.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 6.0, 6.0]], [[8.0, 3.0, 4.0]], [[8.0, 4.0, 8.0]]], [[[6.0, 3.0, 3.0]], [[7.0, 5.0, 
6.0]], [[8.0, 2.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 5.0, 5.0]], [[4.0, 8.0, 2.0]], [[4.0, 5.0, 8.0]]], [[[2.0, 6.0, 
7.0]], [[1.0, 7.0, 6.0]], [[1.0, 4.0, 4.0]]]])
      arg[2:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 5.0], [8.0, 7.0, 3.0], [8.0, 4.0, 4.0]], [[1.0, 2.0, 1.0], [2.0, 3.0, 1.0], [1.0, 
6.0, 4.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 2.0], [3.0, 5.0, 2.0]]], [[[4.0, 4.0, 3.0], [6.0, 8.0, 1.0], [3.0, 2.0, 1.0]], [[5.0, 
8.0, 7.0], [6.0, 2.0, 1.0], [4.0, 1.0, 6.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 7.0], [4.0, 5.0, 8.0]]], [[[8.0, 1.0, 2.0], [7.0, 
8.0, 3.0], [5.0, 6.0, 6.0]], [[5.0, 3.0, 8.0], [5.0, 7.0, 8.0], [8.0, 3.0, 4.0]], [[3.0, 7.0, 6.0], [2.0, 4.0, 6.0], [8.0, 4.0, 
8.0]]], [[[5.0, 2.0, 2.0], [6.0, 4.0, 6.0], [6.0, 3.0, 3.0]], [[6.0, 8.0, 4.0], [1.0, 8.0, 8.0], [7.0, 5.0, 6.0]], [[6.0, 3.0, 
6.0], [3.0, 1.0, 7.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [8.0, 7.0, 3.0], [8.0, 4.0, 4.0]], 
[[1.0, 2.0, 1.0], [2.0, 3.0, 1.0], [1.0, 6.0, 4.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 2.0], [3.0, 5.0, 2.0]]], [[[4.0, 4.0, 3.0], 
[6.0, 8.0, 1.0], [3.0, 2.0, 1.0]], [[5.0, 8.0, 7.0], [6.0, 2.0, 1.0], [4.0, 1.0, 6.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 7.0], 
[4.0, 5.0, 8.0]]], [[[8.0, 1.0, 2.0], [7.0, 8.0, 3.0], [3.0, 5.0, 5.0]], [[5.0, 3.0, 8.0], [5.0, 7.0, 8.0], [4.0, 8.0, 2.0]], 
[[3.0, 7.0, 6.0], [2.0, 4.0, 6.0], [4.0, 5.0, 8.0]]], [[[5.0, 2.0, 2.0], [6.0, 4.0, 6.0], [2.0, 6.0, 7.0]], [[6.0, 8.0, 4.0], 
[1.0, 8.0, 8.0], [1.0, 7.0, 6.0]], [[6.0, 3.0, 6.0], [3.0, 1.0, 7.0], [1.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LCRN(self):
      arg=Data(numpy.array([[[[5.0, 3.0, 7.0], [7.0, 4.0, 3.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 2.0], [4.0, 5.0, 4.0], [3.0, 8.0, 
3.0]], [[1.0, 1.0, 5.0], [5.0, 3.0, 6.0], [8.0, 3.0, 7.0]]], [[[5.0, 4.0, 1.0], [4.0, 7.0, 5.0], [4.0, 5.0, 8.0]], [[5.0, 3.0, 
2.0], [8.0, 1.0, 1.0], [3.0, 6.0, 5.0]], [[7.0, 6.0, 5.0], [2.0, 8.0, 1.0], [5.0, 6.0, 5.0]]], [[[7.0, 7.0, 5.0], [3.0, 8.0, 
4.0], [2.0, 3.0, 6.0]], [[6.0, 4.0, 5.0], [8.0, 3.0, 6.0], [1.0, 8.0, 2.0]], [[2.0, 1.0, 7.0], [4.0, 5.0, 4.0], [1.0, 4.0, 
8.0]]], [[[8.0, 5.0, 1.0], [1.0, 5.0, 6.0], [6.0, 1.0, 1.0]], [[3.0, 1.0, 7.0], [7.0, 1.0, 5.0], [2.0, 4.0, 4.0]], [[5.0, 4.0, 
7.0], [2.0, 2.0, 1.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      val=numpy.array([[[[4.0, 2.0, 5.0]], [[7.0, 8.0, 4.0]], [[7.0, 1.0, 4.0]]], [[[5.0, 8.0, 2.0]], [[5.0, 7.0, 8.0]], [[1.0, 
6.0, 3.0]]]])
      arg[2:,:,1:2]=val
      ref=Data(numpy.array([[[[5.0, 3.0, 7.0], [7.0, 4.0, 3.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 2.0], [4.0, 5.0, 4.0], [3.0, 8.0, 
3.0]], [[1.0, 1.0, 5.0], [5.0, 3.0, 6.0], [8.0, 3.0, 7.0]]], [[[5.0, 4.0, 1.0], [4.0, 7.0, 5.0], [4.0, 5.0, 8.0]], [[5.0, 3.0, 
2.0], [8.0, 1.0, 1.0], [3.0, 6.0, 5.0]], [[7.0, 6.0, 5.0], [2.0, 8.0, 1.0], [5.0, 6.0, 5.0]]], [[[7.0, 7.0, 5.0], [4.0, 2.0, 
5.0], [2.0, 3.0, 6.0]], [[6.0, 4.0, 5.0], [7.0, 8.0, 4.0], [1.0, 8.0, 2.0]], [[2.0, 1.0, 7.0], [7.0, 1.0, 4.0], [1.0, 4.0, 
8.0]]], [[[8.0, 5.0, 1.0], [5.0, 8.0, 2.0], [6.0, 1.0, 1.0]], [[3.0, 1.0, 7.0], [5.0, 7.0, 8.0], [2.0, 4.0, 4.0]], [[5.0, 4.0, 
7.0], [1.0, 6.0, 3.0], [1.0, 7.0, 8.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LCRN(self):
      arg=Data(numpy.array([[[[2.0, 2.0, 1.0], [2.0, 5.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 3.0, 8.0], [8.0, 8.0, 4.0], [4.0, 2.0, 
4.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 1.0], [4.0, 6.0, 5.0]]], [[[8.0, 8.0, 3.0], [5.0, 1.0, 5.0], [5.0, 5.0, 4.0]], [[2.0, 7.0, 
5.0], [8.0, 6.0, 3.0], [6.0, 5.0, 4.0]], [[5.0, 4.0, 2.0], [2.0, 7.0, 4.0], [3.0, 7.0, 7.0]]], [[[4.0, 5.0, 1.0], [2.0, 8.0, 
5.0], [7.0, 6.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 6.0, 5.0], [2.0, 3.0, 8.0]], [[6.0, 4.0, 8.0], [2.0, 6.0, 3.0], [2.0, 6.0, 
2.0]]], [[[6.0, 8.0, 1.0], [1.0, 3.0, 5.0], [2.0, 7.0, 7.0]], [[1.0, 8.0, 4.0], [7.0, 4.0, 4.0], [8.0, 4.0, 7.0]], [[2.0, 1.0, 
6.0], [8.0, 2.0, 3.0], [5.0, 8.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[5.0, 1.0, 2.0]], [[6.0, 6.0, 7.0]], [[2.0, 4.0, 3.0]]], [[[4.0, 2.0, 1.0]], [[2.0, 1.0, 8.0]], 
[[8.0, 2.0, 6.0]]]]),self.functionspace)
      arg[2:,:,1:2]=val
      ref=Data(numpy.array([[[[2.0, 2.0, 1.0], [2.0, 5.0, 4.0], [6.0, 5.0, 6.0]], [[3.0, 3.0, 8.0], [8.0, 8.0, 4.0], [4.0, 2.0, 
4.0]], [[1.0, 6.0, 1.0], [8.0, 3.0, 1.0], [4.0, 6.0, 5.0]]], [[[8.0, 8.0, 3.0], [5.0, 1.0, 5.0], [5.0, 5.0, 4.0]], [[2.0, 7.0, 
5.0], [8.0, 6.0, 3.0], [6.0, 5.0, 4.0]], [[5.0, 4.0, 2.0], [2.0, 7.0, 4.0], [3.0, 7.0, 7.0]]], [[[4.0, 5.0, 1.0], [5.0, 1.0, 
2.0], [7.0, 6.0, 4.0]], [[2.0, 5.0, 3.0], [6.0, 6.0, 7.0], [2.0, 3.0, 8.0]], [[6.0, 4.0, 8.0], [2.0, 4.0, 3.0], [2.0, 6.0, 
2.0]]], [[[6.0, 8.0, 1.0], [4.0, 2.0, 1.0], [2.0, 7.0, 7.0]], [[1.0, 8.0, 4.0], [2.0, 1.0, 8.0], [8.0, 4.0, 7.0]], [[2.0, 1.0, 
6.0], [8.0, 2.0, 6.0], [5.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LCRN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 6.0], [2.0, 4.0, 7.0], [4.0, 8.0, 4.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 5.0], [6.0, 2.0, 
3.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 6.0], [6.0, 3.0, 8.0]]], [[[2.0, 6.0, 1.0], [1.0, 6.0, 8.0], [6.0, 8.0, 2.0]], [[8.0, 2.0, 
4.0], [5.0, 7.0, 3.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 4.0], [5.0, 8.0, 1.0], [7.0, 8.0, 5.0]]], [[[6.0, 2.0, 5.0], [6.0, 5.0, 
3.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 2.0], [7.0, 8.0, 8.0], [5.0, 1.0, 8.0]], [[5.0, 5.0, 5.0], [7.0, 6.0, 1.0], [8.0, 1.0, 
1.0]]], [[[3.0, 8.0, 1.0], [4.0, 2.0, 1.0], [3.0, 2.0, 1.0]], [[3.0, 1.0, 8.0], [6.0, 4.0, 5.0], [3.0, 5.0, 2.0]], [[8.0, 6.0, 
2.0], [1.0, 4.0, 4.0], [7.0, 7.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 1.0, 7.0]], [[5.0, 2.0, 4.0]], [[6.0, 8.0, 8.0]]], [[[8.0, 3.0, 2.0]], [[6.0, 8.0, 
3.0]], [[1.0, 2.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 1.0]], [[1.0, 6.0, 7.0]], [[1.0, 5.0, 1.0]]], [[[6.0, 2.0, 
3.0]], [[6.0, 7.0, 7.0]], [[1.0, 7.0, 6.0]]]])
      arg[2:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [2.0, 4.0, 7.0], [4.0, 8.0, 4.0]], [[6.0, 4.0, 3.0], [4.0, 1.0, 5.0], [6.0, 
2.0, 3.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 6.0], [6.0, 3.0, 8.0]]], [[[2.0, 6.0, 1.0], [1.0, 6.0, 8.0], [6.0, 8.0, 2.0]], [[8.0, 
2.0, 4.0], [5.0, 7.0, 3.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 4.0], [5.0, 8.0, 1.0], [7.0, 8.0, 5.0]]], [[[6.0, 2.0, 5.0], [4.0, 
1.0, 7.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 2.0], [5.0, 2.0, 4.0], [5.0, 1.0, 8.0]], [[5.0, 5.0, 5.0], [6.0, 8.0, 8.0], [8.0, 1.0, 
1.0]]], [[[3.0, 8.0, 1.0], [8.0, 3.0, 2.0], [3.0, 2.0, 1.0]], [[3.0, 1.0, 8.0], [6.0, 8.0, 3.0], [3.0, 5.0, 2.0]], [[8.0, 6.0, 
2.0], [1.0, 2.0, 3.0], [7.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 6.0], [2.0, 4.0, 7.0], [4.0, 8.0, 4.0]], 
[[6.0, 4.0, 3.0], [4.0, 1.0, 5.0], [6.0, 2.0, 3.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 6.0], [6.0, 3.0, 8.0]]], [[[2.0, 6.0, 1.0], 
[1.0, 6.0, 8.0], [6.0, 8.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 7.0, 3.0], [6.0, 8.0, 5.0]], [[7.0, 5.0, 4.0], [5.0, 8.0, 1.0], 
[7.0, 8.0, 5.0]]], [[[6.0, 2.0, 5.0], [2.0, 5.0, 1.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 2.0], [1.0, 6.0, 7.0], [5.0, 1.0, 8.0]], 
[[5.0, 5.0, 5.0], [1.0, 5.0, 1.0], [8.0, 1.0, 1.0]]], [[[3.0, 8.0, 1.0], [6.0, 2.0, 3.0], [3.0, 2.0, 1.0]], [[3.0, 1.0, 8.0], 
[6.0, 7.0, 7.0], [3.0, 5.0, 2.0]], [[8.0, 6.0, 2.0], [1.0, 7.0, 6.0], [7.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RLUN(self):
      arg=Data(numpy.array([[[[3.0, 4.0, 3.0], [3.0, 7.0, 7.0], [2.0, 7.0, 6.0]], [[8.0, 7.0, 1.0], [7.0, 3.0, 1.0], [7.0, 6.0, 
3.0]], [[8.0, 1.0, 4.0], [3.0, 3.0, 7.0], [1.0, 1.0, 4.0]]], [[[6.0, 1.0, 5.0], [7.0, 3.0, 1.0], [4.0, 4.0, 6.0]], [[4.0, 5.0, 
8.0], [6.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[7.0, 4.0, 7.0], [2.0, 1.0, 7.0], [5.0, 2.0, 1.0]]], [[[1.0, 5.0, 3.0], [2.0, 5.0, 
1.0], [3.0, 6.0, 3.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 7.0], [8.0, 4.0, 3.0]], [[3.0, 1.0, 2.0], [7.0, 8.0, 2.0], [1.0, 7.0, 
2.0]]], [[[1.0, 2.0, 7.0], [1.0, 3.0, 3.0], [4.0, 3.0, 5.0]], [[7.0, 5.0, 8.0], [3.0, 7.0, 8.0], [7.0, 6.0, 2.0]], [[3.0, 4.0, 
5.0], [2.0, 6.0, 5.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 8.0, 3.0], [7.0, 8.0, 1.0]]], [[[2.0, 8.0, 3.0], [2.0, 2.0, 1.0]]]])
      arg[1:3,2:,:2]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 3.0], [3.0, 7.0, 7.0], [2.0, 7.0, 6.0]], [[8.0, 7.0, 1.0], [7.0, 3.0, 1.0], [7.0, 6.0, 
3.0]], [[8.0, 1.0, 4.0], [3.0, 3.0, 7.0], [1.0, 1.0, 4.0]]], [[[6.0, 1.0, 5.0], [7.0, 3.0, 1.0], [4.0, 4.0, 6.0]], [[4.0, 5.0, 
8.0], [6.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[6.0, 8.0, 3.0], [7.0, 8.0, 1.0], [5.0, 2.0, 1.0]]], [[[1.0, 5.0, 3.0], [2.0, 5.0, 
1.0], [3.0, 6.0, 3.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 7.0], [8.0, 4.0, 3.0]], [[2.0, 8.0, 3.0], [2.0, 2.0, 1.0], [1.0, 7.0, 
2.0]]], [[[1.0, 2.0, 7.0], [1.0, 3.0, 3.0], [4.0, 3.0, 5.0]], [[7.0, 5.0, 8.0], [3.0, 7.0, 8.0], [7.0, 6.0, 2.0]], [[3.0, 4.0, 
5.0], [2.0, 6.0, 5.0], [3.0, 8.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RLUN(self):
      arg=Data(numpy.array([[[[8.0, 4.0, 1.0], [8.0, 4.0, 4.0], [5.0, 8.0, 7.0]], [[8.0, 3.0, 5.0], [2.0, 4.0, 1.0], [6.0, 8.0, 
8.0]], [[6.0, 8.0, 6.0], [4.0, 4.0, 1.0], [3.0, 3.0, 4.0]]], [[[6.0, 1.0, 3.0], [2.0, 2.0, 3.0], [7.0, 3.0, 1.0]], [[3.0, 7.0, 
7.0], [1.0, 2.0, 1.0], [7.0, 3.0, 7.0]], [[5.0, 2.0, 6.0], [1.0, 2.0, 3.0], [5.0, 7.0, 2.0]]], [[[8.0, 7.0, 8.0], [6.0, 8.0, 
1.0], [4.0, 6.0, 6.0]], [[7.0, 4.0, 8.0], [3.0, 7.0, 6.0], [5.0, 8.0, 3.0]], [[3.0, 7.0, 6.0], [3.0, 7.0, 3.0], [1.0, 6.0, 
3.0]]], [[[5.0, 7.0, 5.0], [4.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[7.0, 6.0, 1.0], [6.0, 8.0, 8.0], [4.0, 5.0, 3.0]], [[8.0, 4.0, 
6.0], [5.0, 1.0, 6.0], [6.0, 1.0, 7.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[6.0, 2.0, 1.0], [8.0, 7.0, 2.0]]], [[[7.0, 3.0, 2.0], [6.0, 7.0, 8.0]]]]),self.functionspace)
      arg[1:3,2:,:2]=val
      ref=Data(numpy.array([[[[8.0, 4.0, 1.0], [8.0, 4.0, 4.0], [5.0, 8.0, 7.0]], [[8.0, 3.0, 5.0], [2.0, 4.0, 1.0], [6.0, 8.0, 
8.0]], [[6.0, 8.0, 6.0], [4.0, 4.0, 1.0], [3.0, 3.0, 4.0]]], [[[6.0, 1.0, 3.0], [2.0, 2.0, 3.0], [7.0, 3.0, 1.0]], [[3.0, 7.0, 
7.0], [1.0, 2.0, 1.0], [7.0, 3.0, 7.0]], [[6.0, 2.0, 1.0], [8.0, 7.0, 2.0], [5.0, 7.0, 2.0]]], [[[8.0, 7.0, 8.0], [6.0, 8.0, 
1.0], [4.0, 6.0, 6.0]], [[7.0, 4.0, 8.0], [3.0, 7.0, 6.0], [5.0, 8.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 7.0, 8.0], [1.0, 6.0, 
3.0]]], [[[5.0, 7.0, 5.0], [4.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[7.0, 6.0, 1.0], [6.0, 8.0, 8.0], [4.0, 5.0, 3.0]], [[8.0, 4.0, 
6.0], [5.0, 1.0, 6.0], [6.0, 1.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RLUN(self):
      arg=Data(numpy.array([[[[6.0, 2.0, 6.0], [7.0, 1.0, 1.0], [8.0, 5.0, 8.0]], [[7.0, 7.0, 2.0], [4.0, 1.0, 1.0], [4.0, 8.0, 
4.0]], [[7.0, 6.0, 3.0], [2.0, 2.0, 2.0], [2.0, 2.0, 2.0]]], [[[6.0, 4.0, 1.0], [6.0, 7.0, 6.0], [3.0, 2.0, 3.0]], [[8.0, 1.0, 
3.0], [1.0, 3.0, 3.0], [6.0, 2.0, 3.0]], [[6.0, 5.0, 6.0], [4.0, 2.0, 2.0], [8.0, 8.0, 1.0]]], [[[3.0, 6.0, 3.0], [1.0, 7.0, 
4.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 3.0], [8.0, 5.0, 1.0], [7.0, 5.0, 6.0]], [[1.0, 3.0, 8.0], [2.0, 2.0, 7.0], [4.0, 8.0, 
1.0]]], [[[4.0, 6.0, 8.0], [7.0, 5.0, 6.0], [8.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], [6.0, 8.0, 5.0], [4.0, 8.0, 3.0]], [[4.0, 4.0, 
8.0], [2.0, 8.0, 2.0], [1.0, 7.0, 5.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 3.0, 7.0], [7.0, 7.0, 8.0]]], [[[6.0, 4.0, 7.0], [5.0, 1.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 1.0], [6.0, 2.0, 8.0]]], [[[7.0, 3.0, 5.0], [2.0, 2.0, 7.0]]]])
      arg[1:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 6.0], [7.0, 1.0, 1.0], [8.0, 5.0, 8.0]], [[7.0, 7.0, 2.0], [4.0, 1.0, 1.0], [4.0, 
8.0, 4.0]], [[7.0, 6.0, 3.0], [2.0, 2.0, 2.0], [2.0, 2.0, 2.0]]], [[[6.0, 4.0, 1.0], [6.0, 7.0, 6.0], [3.0, 2.0, 3.0]], [[8.0, 
1.0, 3.0], [1.0, 3.0, 3.0], [6.0, 2.0, 3.0]], [[7.0, 3.0, 7.0], [7.0, 7.0, 8.0], [8.0, 8.0, 1.0]]], [[[3.0, 6.0, 3.0], [1.0, 
7.0, 4.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 3.0], [8.0, 5.0, 1.0], [7.0, 5.0, 6.0]], [[6.0, 4.0, 7.0], [5.0, 1.0, 2.0], [4.0, 8.0, 
1.0]]], [[[4.0, 6.0, 8.0], [7.0, 5.0, 6.0], [8.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], [6.0, 8.0, 5.0], [4.0, 8.0, 3.0]], [[4.0, 4.0, 
8.0], [2.0, 8.0, 2.0], [1.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 6.0], [7.0, 1.0, 1.0], [8.0, 5.0, 8.0]], 
[[7.0, 7.0, 2.0], [4.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[7.0, 6.0, 3.0], [2.0, 2.0, 2.0], [2.0, 2.0, 2.0]]], [[[6.0, 4.0, 1.0], 
[6.0, 7.0, 6.0], [3.0, 2.0, 3.0]], [[8.0, 1.0, 3.0], [1.0, 3.0, 3.0], [6.0, 2.0, 3.0]], [[2.0, 5.0, 1.0], [6.0, 2.0, 8.0], 
[8.0, 8.0, 1.0]]], [[[3.0, 6.0, 3.0], [1.0, 7.0, 4.0], [8.0, 6.0, 8.0]], [[6.0, 4.0, 3.0], [8.0, 5.0, 1.0], [7.0, 5.0, 6.0]], 
[[7.0, 3.0, 5.0], [2.0, 2.0, 7.0], [4.0, 8.0, 1.0]]], [[[4.0, 6.0, 8.0], [7.0, 5.0, 6.0], [8.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], 
[6.0, 8.0, 5.0], [4.0, 8.0, 3.0]], [[4.0, 4.0, 8.0], [2.0, 8.0, 2.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RRPN(self):
      arg=Data(numpy.array([[[[2.0, 6.0, 4.0], [4.0, 6.0, 6.0], [6.0, 8.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 6.0, 4.0], [7.0, 6.0, 
1.0]], [[7.0, 3.0, 6.0], [2.0, 3.0, 4.0], [1.0, 8.0, 5.0]]], [[[6.0, 3.0, 8.0], [6.0, 2.0, 1.0], [4.0, 2.0, 7.0]], [[3.0, 3.0, 
5.0], [5.0, 7.0, 5.0], [8.0, 8.0, 3.0]], [[8.0, 7.0, 1.0], [7.0, 7.0, 7.0], [3.0, 7.0, 6.0]]], [[[7.0, 5.0, 4.0], [2.0, 3.0, 
6.0], [5.0, 1.0, 1.0]], [[3.0, 6.0, 6.0], [7.0, 2.0, 6.0], [8.0, 6.0, 2.0]], [[7.0, 4.0, 3.0], [2.0, 1.0, 7.0], [6.0, 8.0, 
7.0]]], [[[2.0, 3.0, 3.0], [6.0, 4.0, 3.0], [1.0, 6.0, 4.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 4.0], [6.0, 1.0, 6.0]], [[5.0, 1.0, 
6.0], [3.0, 5.0, 3.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      val=numpy.array([[[5.0, 8.0, 1.0]], [[6.0, 8.0, 2.0]]])
      arg[1:3,1:2,1]=val
      ref=Data(numpy.array([[[[2.0, 6.0, 4.0], [4.0, 6.0, 6.0], [6.0, 8.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 6.0, 4.0], [7.0, 6.0, 
1.0]], [[7.0, 3.0, 6.0], [2.0, 3.0, 4.0], [1.0, 8.0, 5.0]]], [[[6.0, 3.0, 8.0], [6.0, 2.0, 1.0], [4.0, 2.0, 7.0]], [[3.0, 3.0, 
5.0], [5.0, 8.0, 1.0], [8.0, 8.0, 3.0]], [[8.0, 7.0, 1.0], [7.0, 7.0, 7.0], [3.0, 7.0, 6.0]]], [[[7.0, 5.0, 4.0], [2.0, 3.0, 
6.0], [5.0, 1.0, 1.0]], [[3.0, 6.0, 6.0], [6.0, 8.0, 2.0], [8.0, 6.0, 2.0]], [[7.0, 4.0, 3.0], [2.0, 1.0, 7.0], [6.0, 8.0, 
7.0]]], [[[2.0, 3.0, 3.0], [6.0, 4.0, 3.0], [1.0, 6.0, 4.0]], [[5.0, 8.0, 7.0], [4.0, 5.0, 4.0], [6.0, 1.0, 6.0]], [[5.0, 1.0, 
6.0], [3.0, 5.0, 3.0], [7.0, 7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RRPN(self):
      arg=Data(numpy.array([[[[5.0, 8.0, 3.0], [3.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[3.0, 4.0, 3.0], [1.0, 2.0, 7.0], [7.0, 6.0, 
3.0]], [[8.0, 6.0, 4.0], [6.0, 4.0, 8.0], [8.0, 2.0, 3.0]]], [[[1.0, 2.0, 4.0], [7.0, 3.0, 8.0], [5.0, 2.0, 8.0]], [[1.0, 6.0, 
6.0], [6.0, 3.0, 7.0], [6.0, 4.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 2.0, 3.0], [1.0, 8.0, 7.0]]], [[[6.0, 8.0, 7.0], [3.0, 7.0, 
5.0], [1.0, 8.0, 8.0]], [[1.0, 6.0, 6.0], [1.0, 3.0, 5.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 5.0], [7.0, 2.0, 7.0], [7.0, 8.0, 
1.0]]], [[[5.0, 4.0, 5.0], [8.0, 3.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 7.0, 6.0], [1.0, 6.0, 2.0], [1.0, 3.0, 7.0]], [[8.0, 4.0, 
2.0], [1.0, 4.0, 1.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 5.0]], [[1.0, 1.0, 4.0]]]),self.functionspace)
      arg[1:3,1:2,1]=val
      ref=Data(numpy.array([[[[5.0, 8.0, 3.0], [3.0, 3.0, 8.0], [2.0, 6.0, 4.0]], [[3.0, 4.0, 3.0], [1.0, 2.0, 7.0], [7.0, 6.0, 
3.0]], [[8.0, 6.0, 4.0], [6.0, 4.0, 8.0], [8.0, 2.0, 3.0]]], [[[1.0, 2.0, 4.0], [7.0, 3.0, 8.0], [5.0, 2.0, 8.0]], [[1.0, 6.0, 
6.0], [4.0, 7.0, 5.0], [6.0, 4.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 2.0, 3.0], [1.0, 8.0, 7.0]]], [[[6.0, 8.0, 7.0], [3.0, 7.0, 
5.0], [1.0, 8.0, 8.0]], [[1.0, 6.0, 6.0], [1.0, 1.0, 4.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 5.0], [7.0, 2.0, 7.0], [7.0, 8.0, 
1.0]]], [[[5.0, 4.0, 5.0], [8.0, 3.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 7.0, 6.0], [1.0, 6.0, 2.0], [1.0, 3.0, 7.0]], [[8.0, 4.0, 
2.0], [1.0, 4.0, 1.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RRPN(self):
      arg=Data(numpy.array([[[[2.0, 4.0, 5.0], [4.0, 4.0, 4.0], [4.0, 6.0, 4.0]], [[8.0, 7.0, 2.0], [2.0, 5.0, 1.0], [7.0, 7.0, 
7.0]], [[5.0, 4.0, 5.0], [2.0, 7.0, 8.0], [4.0, 3.0, 3.0]]], [[[6.0, 5.0, 2.0], [1.0, 7.0, 4.0], [3.0, 7.0, 2.0]], [[1.0, 7.0, 
2.0], [7.0, 4.0, 4.0], [1.0, 6.0, 3.0]], [[5.0, 1.0, 5.0], [1.0, 2.0, 3.0], [8.0, 3.0, 4.0]]], [[[8.0, 7.0, 3.0], [6.0, 2.0, 
6.0], [6.0, 1.0, 3.0]], [[1.0, 1.0, 2.0], [1.0, 3.0, 5.0], [6.0, 1.0, 8.0]], [[2.0, 8.0, 7.0], [3.0, 7.0, 7.0], [1.0, 8.0, 
2.0]]], [[[5.0, 8.0, 8.0], [1.0, 5.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 4.0, 7.0], [1.0, 8.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 
6.0], [6.0, 4.0, 7.0], [7.0, 8.0, 2.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 3.0, 5.0]], [[3.0, 6.0, 7.0]]])+(1.-msk_val)*numpy.array([[[2.0, 5.0, 8.0]], [[4.0, 3.0, 
8.0]]])
      arg[1:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 5.0], [4.0, 4.0, 4.0], [4.0, 6.0, 4.0]], [[8.0, 7.0, 2.0], [2.0, 5.0, 1.0], [7.0, 
7.0, 7.0]], [[5.0, 4.0, 5.0], [2.0, 7.0, 8.0], [4.0, 3.0, 3.0]]], [[[6.0, 5.0, 2.0], [1.0, 7.0, 4.0], [3.0, 7.0, 2.0]], [[1.0, 
7.0, 2.0], [7.0, 3.0, 5.0], [1.0, 6.0, 3.0]], [[5.0, 1.0, 5.0], [1.0, 2.0, 3.0], [8.0, 3.0, 4.0]]], [[[8.0, 7.0, 3.0], [6.0, 
2.0, 6.0], [6.0, 1.0, 3.0]], [[1.0, 1.0, 2.0], [3.0, 6.0, 7.0], [6.0, 1.0, 8.0]], [[2.0, 8.0, 7.0], [3.0, 7.0, 7.0], [1.0, 8.0, 
2.0]]], [[[5.0, 8.0, 8.0], [1.0, 5.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 4.0, 7.0], [1.0, 8.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 
6.0], [6.0, 4.0, 7.0], [7.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 5.0], [4.0, 4.0, 4.0], [4.0, 6.0, 4.0]], 
[[8.0, 7.0, 2.0], [2.0, 5.0, 1.0], [7.0, 7.0, 7.0]], [[5.0, 4.0, 5.0], [2.0, 7.0, 8.0], [4.0, 3.0, 3.0]]], [[[6.0, 5.0, 2.0], 
[1.0, 7.0, 4.0], [3.0, 7.0, 2.0]], [[1.0, 7.0, 2.0], [2.0, 5.0, 8.0], [1.0, 6.0, 3.0]], [[5.0, 1.0, 5.0], [1.0, 2.0, 3.0], 
[8.0, 3.0, 4.0]]], [[[8.0, 7.0, 3.0], [6.0, 2.0, 6.0], [6.0, 1.0, 3.0]], [[1.0, 1.0, 2.0], [4.0, 3.0, 8.0], [6.0, 1.0, 8.0]], 
[[2.0, 8.0, 7.0], [3.0, 7.0, 7.0], [1.0, 8.0, 2.0]]], [[[5.0, 8.0, 8.0], [1.0, 5.0, 8.0], [2.0, 1.0, 7.0]], [[6.0, 4.0, 7.0], 
[1.0, 8.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 6.0], [6.0, 4.0, 7.0], [7.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LUUN(self):
      arg=Data(numpy.array([[[[6.0, 1.0, 4.0], [5.0, 2.0, 5.0], [7.0, 7.0, 3.0]], [[8.0, 3.0, 4.0], [5.0, 1.0, 3.0], [3.0, 4.0, 
1.0]], [[3.0, 6.0, 7.0], [5.0, 3.0, 1.0], [7.0, 4.0, 4.0]]], [[[5.0, 4.0, 5.0], [7.0, 3.0, 1.0], [2.0, 3.0, 6.0]], [[5.0, 6.0, 
4.0], [4.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[5.0, 3.0, 8.0], [5.0, 6.0, 5.0], [6.0, 7.0, 4.0]]], [[[5.0, 5.0, 7.0], [2.0, 2.0, 
4.0], [3.0, 6.0, 5.0]], [[1.0, 7.0, 3.0], [6.0, 8.0, 3.0], [8.0, 1.0, 1.0]], [[8.0, 1.0, 8.0], [4.0, 3.0, 6.0], [2.0, 2.0, 
1.0]]], [[[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [7.0, 1.0, 7.0]], [[8.0, 4.0, 3.0], [3.0, 5.0, 4.0], [2.0, 6.0, 7.0]], [[6.0, 8.0, 
3.0], [5.0, 5.0, 8.0], [7.0, 1.0, 6.0]]]]),self.functionspace)
      val=numpy.array([[[[6.0, 7.0, 7.0], [8.0, 5.0, 3.0]], [[5.0, 1.0, 4.0], [6.0, 5.0, 8.0]]], [[[2.0, 5.0, 3.0], [5.0, 6.0, 
5.0]], [[8.0, 2.0, 7.0], [5.0, 2.0, 7.0]]]])
      arg[2:,:2,:2]=val
      ref=Data(numpy.array([[[[6.0, 1.0, 4.0], [5.0, 2.0, 5.0], [7.0, 7.0, 3.0]], [[8.0, 3.0, 4.0], [5.0, 1.0, 3.0], [3.0, 4.0, 
1.0]], [[3.0, 6.0, 7.0], [5.0, 3.0, 1.0], [7.0, 4.0, 4.0]]], [[[5.0, 4.0, 5.0], [7.0, 3.0, 1.0], [2.0, 3.0, 6.0]], [[5.0, 6.0, 
4.0], [4.0, 7.0, 3.0], [4.0, 4.0, 8.0]], [[5.0, 3.0, 8.0], [5.0, 6.0, 5.0], [6.0, 7.0, 4.0]]], [[[6.0, 7.0, 7.0], [8.0, 5.0, 
3.0], [3.0, 6.0, 5.0]], [[5.0, 1.0, 4.0], [6.0, 5.0, 8.0], [8.0, 1.0, 1.0]], [[8.0, 1.0, 8.0], [4.0, 3.0, 6.0], [2.0, 2.0, 
1.0]]], [[[2.0, 5.0, 3.0], [5.0, 6.0, 5.0], [7.0, 1.0, 7.0]], [[8.0, 2.0, 7.0], [5.0, 2.0, 7.0], [2.0, 6.0, 7.0]], [[6.0, 8.0, 
3.0], [5.0, 5.0, 8.0], [7.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LUUN(self):
      arg=Data(numpy.array([[[[6.0, 3.0, 3.0], [8.0, 3.0, 4.0], [5.0, 5.0, 2.0]], [[1.0, 6.0, 8.0], [7.0, 8.0, 2.0], [7.0, 4.0, 
3.0]], [[6.0, 6.0, 7.0], [8.0, 2.0, 7.0], [7.0, 6.0, 8.0]]], [[[4.0, 6.0, 4.0], [5.0, 5.0, 3.0], [7.0, 3.0, 3.0]], [[6.0, 4.0, 
6.0], [3.0, 8.0, 2.0], [2.0, 4.0, 1.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 8.0], [3.0, 6.0, 7.0]]], [[[2.0, 3.0, 7.0], [2.0, 4.0, 
8.0], [1.0, 4.0, 5.0]], [[3.0, 3.0, 5.0], [8.0, 1.0, 1.0], [5.0, 6.0, 8.0]], [[4.0, 1.0, 5.0], [3.0, 8.0, 5.0], [3.0, 5.0, 
3.0]]], [[[2.0, 4.0, 6.0], [2.0, 2.0, 5.0], [7.0, 1.0, 3.0]], [[7.0, 1.0, 2.0], [1.0, 2.0, 2.0], [2.0, 7.0, 3.0]], [[5.0, 6.0, 
7.0], [6.0, 3.0, 3.0], [3.0, 1.0, 5.0]]]]),self.functionspace)
      val=Data(numpy.array([[[[7.0, 8.0, 2.0], [5.0, 1.0, 6.0]], [[8.0, 4.0, 6.0], [4.0, 2.0, 8.0]]], [[[3.0, 8.0, 8.0], [5.0, 
7.0, 4.0]], [[3.0, 5.0, 7.0], [2.0, 5.0, 2.0]]]]),self.functionspace)
      arg[2:,:2,:2]=val
      ref=Data(numpy.array([[[[6.0, 3.0, 3.0], [8.0, 3.0, 4.0], [5.0, 5.0, 2.0]], [[1.0, 6.0, 8.0], [7.0, 8.0, 2.0], [7.0, 4.0, 
3.0]], [[6.0, 6.0, 7.0], [8.0, 2.0, 7.0], [7.0, 6.0, 8.0]]], [[[4.0, 6.0, 4.0], [5.0, 5.0, 3.0], [7.0, 3.0, 3.0]], [[6.0, 4.0, 
6.0], [3.0, 8.0, 2.0], [2.0, 4.0, 1.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 8.0], [3.0, 6.0, 7.0]]], [[[7.0, 8.0, 2.0], [5.0, 1.0, 
6.0], [1.0, 4.0, 5.0]], [[8.0, 4.0, 6.0], [4.0, 2.0, 8.0], [5.0, 6.0, 8.0]], [[4.0, 1.0, 5.0], [3.0, 8.0, 5.0], [3.0, 5.0, 
3.0]]], [[[3.0, 8.0, 8.0], [5.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[3.0, 5.0, 7.0], [2.0, 5.0, 2.0], [2.0, 7.0, 3.0]], [[5.0, 6.0, 
7.0], [6.0, 3.0, 3.0], [3.0, 1.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LUUN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 4.0], [8.0, 4.0, 7.0], [3.0, 8.0, 7.0]], [[8.0, 2.0, 4.0], [7.0, 8.0, 7.0], [8.0, 2.0, 
3.0]], [[5.0, 1.0, 3.0], [5.0, 7.0, 6.0], [2.0, 2.0, 7.0]]], [[[6.0, 2.0, 4.0], [3.0, 7.0, 1.0], [4.0, 6.0, 3.0]], [[6.0, 7.0, 
7.0], [7.0, 7.0, 7.0], [3.0, 2.0, 5.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 6.0], [2.0, 6.0, 7.0]]], [[[1.0, 1.0, 3.0], [6.0, 6.0, 
7.0], [1.0, 8.0, 3.0]], [[8.0, 1.0, 7.0], [4.0, 7.0, 5.0], [1.0, 7.0, 3.0]], [[4.0, 1.0, 4.0], [2.0, 5.0, 7.0], [3.0, 4.0, 
3.0]]], [[[7.0, 6.0, 8.0], [4.0, 3.0, 8.0], [3.0, 1.0, 7.0]], [[7.0, 4.0, 5.0], [8.0, 5.0, 5.0], [7.0, 7.0, 8.0]], [[8.0, 2.0, 
4.0], [2.0, 2.0, 5.0], [1.0, 2.0, 8.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 5.0, 4.0], [1.0, 1.0, 5.0]], [[3.0, 5.0, 6.0], [2.0, 3.0, 1.0]]], [[[3.0, 8.0, 5.0], 
[6.0, 4.0, 1.0]], [[4.0, 7.0, 5.0], [7.0, 6.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 3.0, 5.0], [7.0, 1.0, 4.0]], [[8.0, 
6.0, 5.0], [2.0, 2.0, 1.0]]], [[[7.0, 6.0, 8.0], [7.0, 2.0, 4.0]], [[7.0, 2.0, 7.0], [4.0, 2.0, 4.0]]]])
      arg[2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 4.0], [8.0, 4.0, 7.0], [3.0, 8.0, 7.0]], [[8.0, 2.0, 4.0], [7.0, 8.0, 7.0], [8.0, 
2.0, 3.0]], [[5.0, 1.0, 3.0], [5.0, 7.0, 6.0], [2.0, 2.0, 7.0]]], [[[6.0, 2.0, 4.0], [3.0, 7.0, 1.0], [4.0, 6.0, 3.0]], [[6.0, 
7.0, 7.0], [7.0, 7.0, 7.0], [3.0, 2.0, 5.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 6.0], [2.0, 6.0, 7.0]]], [[[8.0, 5.0, 4.0], [1.0, 
1.0, 5.0], [1.0, 8.0, 3.0]], [[3.0, 5.0, 6.0], [2.0, 3.0, 1.0], [1.0, 7.0, 3.0]], [[4.0, 1.0, 4.0], [2.0, 5.0, 7.0], [3.0, 4.0, 
3.0]]], [[[3.0, 8.0, 5.0], [6.0, 4.0, 1.0], [3.0, 1.0, 7.0]], [[4.0, 7.0, 5.0], [7.0, 6.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 2.0, 
4.0], [2.0, 2.0, 5.0], [1.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 4.0], [8.0, 4.0, 7.0], [3.0, 8.0, 7.0]], 
[[8.0, 2.0, 4.0], [7.0, 8.0, 7.0], [8.0, 2.0, 3.0]], [[5.0, 1.0, 3.0], [5.0, 7.0, 6.0], [2.0, 2.0, 7.0]]], [[[6.0, 2.0, 4.0], 
[3.0, 7.0, 1.0], [4.0, 6.0, 3.0]], [[6.0, 7.0, 7.0], [7.0, 7.0, 7.0], [3.0, 2.0, 5.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 6.0], 
[2.0, 6.0, 7.0]]], [[[6.0, 3.0, 5.0], [7.0, 1.0, 4.0], [1.0, 8.0, 3.0]], [[8.0, 6.0, 5.0], [2.0, 2.0, 1.0], [1.0, 7.0, 3.0]], 
[[4.0, 1.0, 4.0], [2.0, 5.0, 7.0], [3.0, 4.0, 3.0]]], [[[7.0, 6.0, 8.0], [7.0, 2.0, 4.0], [3.0, 1.0, 7.0]], [[7.0, 2.0, 7.0], 
[4.0, 2.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 2.0, 4.0], [2.0, 2.0, 5.0], [1.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_LPRN(self):
      arg=Data(numpy.array([[[[1.0, 6.0, 2.0], [1.0, 7.0, 6.0], [7.0, 2.0, 2.0]], [[7.0, 3.0, 7.0], [4.0, 8.0, 4.0], [2.0, 3.0, 
4.0]], [[8.0, 5.0, 4.0], [6.0, 8.0, 5.0], [7.0, 1.0, 6.0]]], [[[5.0, 7.0, 2.0], [6.0, 7.0, 4.0], [3.0, 5.0, 6.0]], [[3.0, 1.0, 
8.0], [6.0, 8.0, 1.0], [8.0, 3.0, 7.0]], [[3.0, 2.0, 6.0], [7.0, 8.0, 3.0], [5.0, 1.0, 3.0]]], [[[2.0, 4.0, 3.0], [1.0, 8.0, 
2.0], [4.0, 5.0, 6.0]], [[5.0, 4.0, 1.0], [4.0, 4.0, 1.0], [6.0, 7.0, 7.0]], [[7.0, 1.0, 4.0], [3.0, 3.0, 6.0], [7.0, 7.0, 
6.0]]], [[[7.0, 7.0, 7.0], [6.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 6.0], [7.0, 3.0, 8.0]], [[3.0, 5.0, 
3.0], [1.0, 6.0, 7.0], [2.0, 8.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 3.0, 4.0]], [[7.0, 2.0, 8.0]]])
      arg[2:,1,1:2]=val
      ref=Data(numpy.array([[[[1.0, 6.0, 2.0], [1.0, 7.0, 6.0], [7.0, 2.0, 2.0]], [[7.0, 3.0, 7.0], [4.0, 8.0, 4.0], [2.0, 3.0, 
4.0]], [[8.0, 5.0, 4.0], [6.0, 8.0, 5.0], [7.0, 1.0, 6.0]]], [[[5.0, 7.0, 2.0], [6.0, 7.0, 4.0], [3.0, 5.0, 6.0]], [[3.0, 1.0, 
8.0], [6.0, 8.0, 1.0], [8.0, 3.0, 7.0]], [[3.0, 2.0, 6.0], [7.0, 8.0, 3.0], [5.0, 1.0, 3.0]]], [[[2.0, 4.0, 3.0], [1.0, 8.0, 
2.0], [4.0, 5.0, 6.0]], [[5.0, 4.0, 1.0], [7.0, 3.0, 4.0], [6.0, 7.0, 7.0]], [[7.0, 1.0, 4.0], [3.0, 3.0, 6.0], [7.0, 7.0, 
6.0]]], [[[7.0, 7.0, 7.0], [6.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[3.0, 8.0, 8.0], [7.0, 2.0, 8.0], [7.0, 3.0, 8.0]], [[3.0, 5.0, 
3.0], [1.0, 6.0, 7.0], [2.0, 8.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_LPRN(self):
      arg=Data(numpy.array([[[[2.0, 5.0, 8.0], [2.0, 1.0, 3.0], [1.0, 3.0, 6.0]], [[5.0, 6.0, 3.0], [4.0, 8.0, 7.0], [1.0, 3.0, 
1.0]], [[3.0, 4.0, 6.0], [4.0, 3.0, 2.0], [8.0, 6.0, 8.0]]], [[[1.0, 7.0, 7.0], [3.0, 3.0, 5.0], [3.0, 4.0, 1.0]], [[1.0, 7.0, 
4.0], [7.0, 5.0, 5.0], [6.0, 6.0, 7.0]], [[3.0, 8.0, 1.0], [1.0, 7.0, 1.0], [2.0, 6.0, 8.0]]], [[[6.0, 6.0, 3.0], [4.0, 3.0, 
8.0], [2.0, 2.0, 7.0]], [[5.0, 2.0, 6.0], [4.0, 5.0, 3.0], [4.0, 8.0, 3.0]], [[2.0, 1.0, 6.0], [5.0, 5.0, 2.0], [2.0, 7.0, 
1.0]]], [[[3.0, 6.0, 2.0], [5.0, 4.0, 2.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 2.0], [4.0, 3.0, 3.0], [2.0, 3.0, 3.0]], [[7.0, 5.0, 
3.0], [7.0, 5.0, 7.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      val=Data(numpy.array([[[1.0, 2.0, 4.0]], [[5.0, 7.0, 2.0]]]),self.functionspace)
      arg[2:,1,1:2]=val
      ref=Data(numpy.array([[[[2.0, 5.0, 8.0], [2.0, 1.0, 3.0], [1.0, 3.0, 6.0]], [[5.0, 6.0, 3.0], [4.0, 8.0, 7.0], [1.0, 3.0, 
1.0]], [[3.0, 4.0, 6.0], [4.0, 3.0, 2.0], [8.0, 6.0, 8.0]]], [[[1.0, 7.0, 7.0], [3.0, 3.0, 5.0], [3.0, 4.0, 1.0]], [[1.0, 7.0, 
4.0], [7.0, 5.0, 5.0], [6.0, 6.0, 7.0]], [[3.0, 8.0, 1.0], [1.0, 7.0, 1.0], [2.0, 6.0, 8.0]]], [[[6.0, 6.0, 3.0], [4.0, 3.0, 
8.0], [2.0, 2.0, 7.0]], [[5.0, 2.0, 6.0], [1.0, 2.0, 4.0], [4.0, 8.0, 3.0]], [[2.0, 1.0, 6.0], [5.0, 5.0, 2.0], [2.0, 7.0, 
1.0]]], [[[3.0, 6.0, 2.0], [5.0, 4.0, 2.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 2.0], [5.0, 7.0, 2.0], [2.0, 3.0, 3.0]], [[7.0, 5.0, 
3.0], [7.0, 5.0, 7.0], [7.0, 3.0, 2.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_LPRN(self):
      arg=Data(numpy.array([[[[3.0, 7.0, 4.0], [2.0, 1.0, 1.0], [2.0, 6.0, 8.0]], [[2.0, 4.0, 6.0], [2.0, 2.0, 3.0], [6.0, 2.0, 
5.0]], [[1.0, 4.0, 1.0], [3.0, 5.0, 6.0], [7.0, 7.0, 3.0]]], [[[2.0, 1.0, 2.0], [1.0, 4.0, 7.0], [7.0, 4.0, 2.0]], [[4.0, 3.0, 
7.0], [1.0, 8.0, 4.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 3.0], [4.0, 5.0, 3.0], [2.0, 2.0, 2.0]]], [[[7.0, 4.0, 6.0], [3.0, 6.0, 
8.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [7.0, 8.0, 4.0], [2.0, 2.0, 6.0]], [[7.0, 6.0, 4.0], [1.0, 3.0, 2.0], [6.0, 2.0, 
2.0]]], [[[3.0, 4.0, 8.0], [6.0, 8.0, 4.0], [8.0, 7.0, 4.0]], [[6.0, 7.0, 5.0], [8.0, 3.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 
6.0], [8.0, 5.0, 5.0], [8.0, 5.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 8.0, 4.0]], [[5.0, 8.0, 2.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 6.0]], [[2.0, 1.0, 
2.0]]])
      arg[2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 4.0], [2.0, 1.0, 1.0], [2.0, 6.0, 8.0]], [[2.0, 4.0, 6.0], [2.0, 2.0, 3.0], [6.0, 
2.0, 5.0]], [[1.0, 4.0, 1.0], [3.0, 5.0, 6.0], [7.0, 7.0, 3.0]]], [[[2.0, 1.0, 2.0], [1.0, 4.0, 7.0], [7.0, 4.0, 2.0]], [[4.0, 
3.0, 7.0], [1.0, 8.0, 4.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 3.0], [4.0, 5.0, 3.0], [2.0, 2.0, 2.0]]], [[[7.0, 4.0, 6.0], [3.0, 
6.0, 8.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [5.0, 8.0, 4.0], [2.0, 2.0, 6.0]], [[7.0, 6.0, 4.0], [1.0, 3.0, 2.0], [6.0, 2.0, 
2.0]]], [[[3.0, 4.0, 8.0], [6.0, 8.0, 4.0], [8.0, 7.0, 4.0]], [[6.0, 7.0, 5.0], [5.0, 8.0, 2.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 
6.0], [8.0, 5.0, 5.0], [8.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 4.0], [2.0, 1.0, 1.0], [2.0, 6.0, 8.0]], 
[[2.0, 4.0, 6.0], [2.0, 2.0, 3.0], [6.0, 2.0, 5.0]], [[1.0, 4.0, 1.0], [3.0, 5.0, 6.0], [7.0, 7.0, 3.0]]], [[[2.0, 1.0, 2.0], 
[1.0, 4.0, 7.0], [7.0, 4.0, 2.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 4.0], [6.0, 4.0, 7.0]], [[6.0, 6.0, 3.0], [4.0, 5.0, 3.0], 
[2.0, 2.0, 2.0]]], [[[7.0, 4.0, 6.0], [3.0, 6.0, 8.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 2.0], [8.0, 1.0, 6.0], [2.0, 2.0, 6.0]], 
[[7.0, 6.0, 4.0], [1.0, 3.0, 2.0], [6.0, 2.0, 2.0]]], [[[3.0, 4.0, 8.0], [6.0, 8.0, 4.0], [8.0, 7.0, 4.0]], [[6.0, 7.0, 5.0], 
[2.0, 1.0, 2.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 6.0], [8.0, 5.0, 5.0], [8.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_CCPN(self):
      arg=Data(numpy.array([[[[5.0, 5.0, 1.0], [7.0, 8.0, 5.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 2.0], [5.0, 8.0, 7.0], [1.0, 1.0, 
2.0]], [[8.0, 8.0, 2.0], [3.0, 2.0, 6.0], [6.0, 6.0, 7.0]]], [[[3.0, 5.0, 7.0], [7.0, 7.0, 3.0], [8.0, 5.0, 2.0]], [[2.0, 2.0, 
7.0], [3.0, 1.0, 2.0], [5.0, 7.0, 7.0]], [[6.0, 8.0, 6.0], [8.0, 2.0, 4.0], [7.0, 3.0, 3.0]]], [[[3.0, 6.0, 2.0], [8.0, 3.0, 
7.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 1.0], [5.0, 7.0, 2.0], [3.0, 7.0, 4.0]], [[2.0, 3.0, 3.0], [3.0, 1.0, 2.0], [5.0, 5.0, 
5.0]]], [[[5.0, 4.0, 1.0], [6.0, 2.0, 5.0], [1.0, 3.0, 4.0]], [[4.0, 3.0, 2.0], [8.0, 5.0, 8.0], [8.0, 7.0, 2.0]], [[4.0, 4.0, 
6.0], [2.0, 3.0, 4.0], [8.0, 7.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[4.0, 2.0, 3.0], [4.0, 2.0, 3.0], [5.0, 8.0, 4.0]], [[5.0, 7.0, 3.0], [7.0, 8.0, 8.0], [4.0, 4.0, 
3.0]], [[8.0, 2.0, 6.0], [4.0, 1.0, 7.0], [6.0, 3.0, 8.0]], [[8.0, 4.0, 6.0], [3.0, 2.0, 2.0], [4.0, 3.0, 7.0]]])
      arg[:,:,1]=val
      ref=Data(numpy.array([[[[5.0, 5.0, 1.0], [4.0, 2.0, 3.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 2.0], [4.0, 2.0, 3.0], [1.0, 1.0, 
2.0]], [[8.0, 8.0, 2.0], [5.0, 8.0, 4.0], [6.0, 6.0, 7.0]]], [[[3.0, 5.0, 7.0], [5.0, 7.0, 3.0], [8.0, 5.0, 2.0]], [[2.0, 2.0, 
7.0], [7.0, 8.0, 8.0], [5.0, 7.0, 7.0]], [[6.0, 8.0, 6.0], [4.0, 4.0, 3.0], [7.0, 3.0, 3.0]]], [[[3.0, 6.0, 2.0], [8.0, 2.0, 
6.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 1.0], [4.0, 1.0, 7.0], [3.0, 7.0, 4.0]], [[2.0, 3.0, 3.0], [6.0, 3.0, 8.0], [5.0, 5.0, 
5.0]]], [[[5.0, 4.0, 1.0], [8.0, 4.0, 6.0], [1.0, 3.0, 4.0]], [[4.0, 3.0, 2.0], [3.0, 2.0, 2.0], [8.0, 7.0, 2.0]], [[4.0, 4.0, 
6.0], [4.0, 3.0, 7.0], [8.0, 7.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_CCPN(self):
      arg=Data(numpy.array([[[[6.0, 5.0, 1.0], [3.0, 4.0, 5.0], [8.0, 7.0, 1.0]], [[1.0, 7.0, 5.0], [7.0, 2.0, 6.0], [1.0, 5.0, 
3.0]], [[1.0, 5.0, 2.0], [6.0, 8.0, 5.0], [4.0, 5.0, 7.0]]], [[[5.0, 6.0, 4.0], [5.0, 8.0, 6.0], [1.0, 8.0, 4.0]], [[1.0, 2.0, 
4.0], [7.0, 2.0, 5.0], [7.0, 2.0, 2.0]], [[4.0, 5.0, 7.0], [5.0, 1.0, 5.0], [2.0, 4.0, 2.0]]], [[[7.0, 4.0, 1.0], [2.0, 4.0, 
5.0], [7.0, 7.0, 1.0]], [[6.0, 4.0, 2.0], [1.0, 1.0, 7.0], [1.0, 7.0, 5.0]], [[4.0, 1.0, 3.0], [8.0, 3.0, 7.0], [4.0, 3.0, 
4.0]]], [[[4.0, 2.0, 1.0], [6.0, 4.0, 6.0], [3.0, 1.0, 8.0]], [[7.0, 8.0, 1.0], [2.0, 5.0, 6.0], [7.0, 8.0, 8.0]], [[3.0, 8.0, 
2.0], [7.0, 6.0, 7.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      val=Data(numpy.array([[[4.0, 7.0, 7.0], [8.0, 6.0, 6.0], [7.0, 5.0, 1.0]], [[8.0, 7.0, 1.0], [7.0, 8.0, 4.0], [1.0, 8.0, 
2.0]], [[7.0, 8.0, 6.0], [3.0, 4.0, 3.0], [6.0, 7.0, 1.0]], [[1.0, 2.0, 7.0], [7.0, 8.0, 5.0], [8.0, 6.0, 
4.0]]]),self.functionspace)
      arg[:,:,1]=val
      ref=Data(numpy.array([[[[6.0, 5.0, 1.0], [4.0, 7.0, 7.0], [8.0, 7.0, 1.0]], [[1.0, 7.0, 5.0], [8.0, 6.0, 6.0], [1.0, 5.0, 
3.0]], [[1.0, 5.0, 2.0], [7.0, 5.0, 1.0], [4.0, 5.0, 7.0]]], [[[5.0, 6.0, 4.0], [8.0, 7.0, 1.0], [1.0, 8.0, 4.0]], [[1.0, 2.0, 
4.0], [7.0, 8.0, 4.0], [7.0, 2.0, 2.0]], [[4.0, 5.0, 7.0], [1.0, 8.0, 2.0], [2.0, 4.0, 2.0]]], [[[7.0, 4.0, 1.0], [7.0, 8.0, 
6.0], [7.0, 7.0, 1.0]], [[6.0, 4.0, 2.0], [3.0, 4.0, 3.0], [1.0, 7.0, 5.0]], [[4.0, 1.0, 3.0], [6.0, 7.0, 1.0], [4.0, 3.0, 
4.0]]], [[[4.0, 2.0, 1.0], [1.0, 2.0, 7.0], [3.0, 1.0, 8.0]], [[7.0, 8.0, 1.0], [7.0, 8.0, 5.0], [7.0, 8.0, 8.0]], [[3.0, 8.0, 
2.0], [8.0, 6.0, 4.0], [7.0, 2.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_CCPN(self):
      arg=Data(numpy.array([[[[7.0, 8.0, 5.0], [1.0, 1.0, 6.0], [3.0, 4.0, 3.0]], [[5.0, 2.0, 4.0], [5.0, 5.0, 3.0], [6.0, 5.0, 
2.0]], [[7.0, 7.0, 6.0], [1.0, 1.0, 1.0], [5.0, 7.0, 7.0]]], [[[2.0, 3.0, 6.0], [2.0, 5.0, 4.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 
2.0], [6.0, 8.0, 4.0], [8.0, 7.0, 6.0]], [[7.0, 8.0, 4.0], [6.0, 7.0, 3.0], [7.0, 1.0, 8.0]]], [[[7.0, 3.0, 8.0], [6.0, 5.0, 
4.0], [6.0, 8.0, 7.0]], [[6.0, 5.0, 7.0], [3.0, 1.0, 2.0], [3.0, 7.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 2.0, 4.0], [5.0, 1.0, 
1.0]]], [[[1.0, 6.0, 4.0], [6.0, 3.0, 3.0], [2.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], [2.0, 2.0, 1.0], [4.0, 1.0, 8.0]], [[5.0, 1.0, 
6.0], [3.0, 2.0, 2.0], [4.0, 3.0, 3.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 8.0, 6.0], [5.0, 4.0, 6.0], [1.0, 1.0, 2.0]], [[8.0, 1.0, 2.0], [1.0, 3.0, 4.0], [8.0, 
5.0, 2.0]], [[1.0, 1.0, 4.0], [5.0, 2.0, 7.0], [4.0, 7.0, 8.0]], [[5.0, 6.0, 1.0], [7.0, 1.0, 4.0], [4.0, 7.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[8.0, 4.0, 1.0], [1.0, 2.0, 1.0], [8.0, 2.0, 2.0]], [[6.0, 7.0, 7.0], [7.0, 1.0, 1.0], [2.0, 
6.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 5.0, 1.0], [3.0, 1.0, 4.0]], [[7.0, 5.0, 7.0], [6.0, 7.0, 1.0], [6.0, 2.0, 8.0]]])
      arg[:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 5.0], [6.0, 8.0, 6.0], [3.0, 4.0, 3.0]], [[5.0, 2.0, 4.0], [5.0, 4.0, 6.0], [6.0, 
5.0, 2.0]], [[7.0, 7.0, 6.0], [1.0, 1.0, 2.0], [5.0, 7.0, 7.0]]], [[[2.0, 3.0, 6.0], [8.0, 1.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 
5.0, 2.0], [1.0, 3.0, 4.0], [8.0, 7.0, 6.0]], [[7.0, 8.0, 4.0], [8.0, 5.0, 2.0], [7.0, 1.0, 8.0]]], [[[7.0, 3.0, 8.0], [1.0, 
1.0, 4.0], [6.0, 8.0, 7.0]], [[6.0, 5.0, 7.0], [5.0, 2.0, 7.0], [3.0, 7.0, 5.0]], [[3.0, 7.0, 6.0], [4.0, 7.0, 8.0], [5.0, 1.0, 
1.0]]], [[[1.0, 6.0, 4.0], [5.0, 6.0, 1.0], [2.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], [7.0, 1.0, 4.0], [4.0, 1.0, 8.0]], [[5.0, 1.0, 
6.0], [4.0, 7.0, 4.0], [4.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 5.0], [8.0, 4.0, 1.0], [3.0, 4.0, 3.0]], 
[[5.0, 2.0, 4.0], [1.0, 2.0, 1.0], [6.0, 5.0, 2.0]], [[7.0, 7.0, 6.0], [8.0, 2.0, 2.0], [5.0, 7.0, 7.0]]], [[[2.0, 3.0, 6.0], 
[6.0, 7.0, 7.0], [1.0, 2.0, 7.0]], [[4.0, 5.0, 2.0], [7.0, 1.0, 1.0], [8.0, 7.0, 6.0]], [[7.0, 8.0, 4.0], [2.0, 6.0, 8.0], 
[7.0, 1.0, 8.0]]], [[[7.0, 3.0, 8.0], [1.0, 3.0, 8.0], [6.0, 8.0, 7.0]], [[6.0, 5.0, 7.0], [7.0, 5.0, 1.0], [3.0, 7.0, 5.0]], 
[[3.0, 7.0, 6.0], [3.0, 1.0, 4.0], [5.0, 1.0, 1.0]]], [[[1.0, 6.0, 4.0], [7.0, 5.0, 7.0], [2.0, 4.0, 6.0]], [[5.0, 2.0, 7.0], 
[6.0, 7.0, 1.0], [4.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [6.0, 2.0, 8.0], [4.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_array_RUPN(self):
      arg=Data(numpy.array([[[[3.0, 2.0, 6.0], [2.0, 6.0, 6.0], [4.0, 6.0, 7.0]], [[1.0, 6.0, 4.0], [3.0, 1.0, 1.0], [3.0, 4.0, 
4.0]], [[8.0, 3.0, 7.0], [7.0, 8.0, 4.0], [4.0, 1.0, 4.0]]], [[[3.0, 3.0, 1.0], [3.0, 5.0, 8.0], [2.0, 4.0, 4.0]], [[4.0, 5.0, 
4.0], [6.0, 5.0, 5.0], [2.0, 5.0, 8.0]], [[5.0, 3.0, 5.0], [5.0, 1.0, 7.0], [5.0, 2.0, 6.0]]], [[[7.0, 5.0, 6.0], [7.0, 4.0, 
8.0], [7.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [3.0, 5.0, 4.0], [3.0, 7.0, 4.0]], [[2.0, 7.0, 5.0], [4.0, 1.0, 8.0], [7.0, 6.0, 
6.0]]], [[[8.0, 6.0, 5.0], [1.0, 5.0, 6.0], [8.0, 3.0, 4.0]], [[8.0, 6.0, 6.0], [1.0, 8.0, 3.0], [4.0, 5.0, 1.0]], [[3.0, 6.0, 
1.0], [8.0, 2.0, 2.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      val=numpy.array([[[7.0, 8.0, 8.0], [5.0, 5.0, 5.0]], [[6.0, 5.0, 3.0], [6.0, 6.0, 5.0]]])
      arg[1:3,:2,1]=val
      ref=Data(numpy.array([[[[3.0, 2.0, 6.0], [2.0, 6.0, 6.0], [4.0, 6.0, 7.0]], [[1.0, 6.0, 4.0], [3.0, 1.0, 1.0], [3.0, 4.0, 
4.0]], [[8.0, 3.0, 7.0], [7.0, 8.0, 4.0], [4.0, 1.0, 4.0]]], [[[3.0, 3.0, 1.0], [7.0, 8.0, 8.0], [2.0, 4.0, 4.0]], [[4.0, 5.0, 
4.0], [5.0, 5.0, 5.0], [2.0, 5.0, 8.0]], [[5.0, 3.0, 5.0], [5.0, 1.0, 7.0], [5.0, 2.0, 6.0]]], [[[7.0, 5.0, 6.0], [6.0, 5.0, 
3.0], [7.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [6.0, 6.0, 5.0], [3.0, 7.0, 4.0]], [[2.0, 7.0, 5.0], [4.0, 1.0, 8.0], [7.0, 6.0, 
6.0]]], [[[8.0, 6.0, 5.0], [1.0, 5.0, 6.0], [8.0, 3.0, 4.0]], [[8.0, 6.0, 6.0], [1.0, 8.0, 3.0], [4.0, 5.0, 1.0]], [[3.0, 6.0, 
1.0], [8.0, 2.0, 2.0], [2.0, 3.0, 4.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_constData_RUPN(self):
      arg=Data(numpy.array([[[[7.0, 5.0, 5.0], [1.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[2.0, 7.0, 7.0], [1.0, 1.0, 7.0], [8.0, 4.0, 
1.0]], [[1.0, 1.0, 8.0], [1.0, 4.0, 1.0], [4.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [5.0, 7.0, 7.0], [1.0, 6.0, 4.0]], [[1.0, 5.0, 
8.0], [8.0, 6.0, 7.0], [6.0, 4.0, 1.0]], [[3.0, 8.0, 2.0], [7.0, 6.0, 5.0], [6.0, 8.0, 7.0]]], [[[5.0, 4.0, 2.0], [1.0, 3.0, 
8.0], [2.0, 4.0, 3.0]], [[7.0, 8.0, 7.0], [4.0, 7.0, 6.0], [8.0, 6.0, 5.0]], [[3.0, 1.0, 2.0], [5.0, 3.0, 3.0], [2.0, 8.0, 
1.0]]], [[[8.0, 2.0, 8.0], [1.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[7.0, 3.0, 6.0], [8.0, 1.0, 8.0], [6.0, 6.0, 7.0]], [[4.0, 7.0, 
2.0], [1.0, 3.0, 8.0], [1.0, 7.0, 1.0]]]]),self.functionspace)
      val=Data(numpy.array([[[3.0, 5.0, 6.0], [1.0, 1.0, 5.0]], [[2.0, 2.0, 6.0], [2.0, 2.0, 3.0]]]),self.functionspace)
      arg[1:3,:2,1]=val
      ref=Data(numpy.array([[[[7.0, 5.0, 5.0], [1.0, 4.0, 5.0], [8.0, 1.0, 8.0]], [[2.0, 7.0, 7.0], [1.0, 1.0, 7.0], [8.0, 4.0, 
1.0]], [[1.0, 1.0, 8.0], [1.0, 4.0, 1.0], [4.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [3.0, 5.0, 6.0], [1.0, 6.0, 4.0]], [[1.0, 5.0, 
8.0], [1.0, 1.0, 5.0], [6.0, 4.0, 1.0]], [[3.0, 8.0, 2.0], [7.0, 6.0, 5.0], [6.0, 8.0, 7.0]]], [[[5.0, 4.0, 2.0], [2.0, 2.0, 
6.0], [2.0, 4.0, 3.0]], [[7.0, 8.0, 7.0], [2.0, 2.0, 3.0], [8.0, 6.0, 5.0]], [[3.0, 1.0, 2.0], [5.0, 3.0, 3.0], [2.0, 8.0, 
1.0]]], [[[8.0, 2.0, 8.0], [1.0, 8.0, 3.0], [5.0, 2.0, 7.0]], [[7.0, 3.0, 6.0], [8.0, 1.0, 8.0], [6.0, 6.0, 7.0]], [[4.0, 7.0, 
2.0], [1.0, 3.0, 8.0], [1.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_constData_rank4_expandedData_RUPN(self):
      arg=Data(numpy.array([[[[7.0, 7.0, 5.0], [5.0, 4.0, 5.0], [6.0, 4.0, 7.0]], [[3.0, 8.0, 2.0], [6.0, 3.0, 1.0], [6.0, 1.0, 
7.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 5.0], [5.0, 5.0, 7.0]]], [[[6.0, 8.0, 6.0], [7.0, 7.0, 6.0], [3.0, 6.0, 7.0]], [[7.0, 3.0, 
2.0], [2.0, 8.0, 7.0], [2.0, 1.0, 5.0]], [[7.0, 1.0, 3.0], [1.0, 7.0, 2.0], [6.0, 7.0, 2.0]]], [[[5.0, 6.0, 5.0], [5.0, 4.0, 
6.0], [6.0, 7.0, 1.0]], [[5.0, 6.0, 4.0], [6.0, 6.0, 2.0], [3.0, 1.0, 3.0]], [[7.0, 7.0, 5.0], [7.0, 1.0, 8.0], [4.0, 2.0, 
8.0]]], [[[3.0, 1.0, 7.0], [8.0, 1.0, 3.0], [5.0, 8.0, 7.0]], [[5.0, 7.0, 3.0], [6.0, 3.0, 7.0], [3.0, 4.0, 4.0]], [[3.0, 3.0, 
3.0], [6.0, 4.0, 4.0], [2.0, 4.0, 7.0]]]]),self.functionspace)
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 7.0, 1.0], [5.0, 4.0, 2.0]], [[7.0, 8.0, 4.0], [8.0, 8.0, 
5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 8.0, 2.0], [4.0, 6.0, 8.0]], [[2.0, 8.0, 8.0], [2.0, 3.0, 8.0]]])
      arg[1:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 5.0], [5.0, 4.0, 5.0], [6.0, 4.0, 7.0]], [[3.0, 8.0, 2.0], [6.0, 3.0, 1.0], [6.0, 
1.0, 7.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 5.0], [5.0, 5.0, 7.0]]], [[[6.0, 8.0, 6.0], [7.0, 7.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 
3.0, 2.0], [5.0, 4.0, 2.0], [2.0, 1.0, 5.0]], [[7.0, 1.0, 3.0], [1.0, 7.0, 2.0], [6.0, 7.0, 2.0]]], [[[5.0, 6.0, 5.0], [7.0, 
8.0, 4.0], [6.0, 7.0, 1.0]], [[5.0, 6.0, 4.0], [8.0, 8.0, 5.0], [3.0, 1.0, 3.0]], [[7.0, 7.0, 5.0], [7.0, 1.0, 8.0], [4.0, 2.0, 
8.0]]], [[[3.0, 1.0, 7.0], [8.0, 1.0, 3.0], [5.0, 8.0, 7.0]], [[5.0, 7.0, 3.0], [6.0, 3.0, 7.0], [3.0, 4.0, 4.0]], [[3.0, 3.0, 
3.0], [6.0, 4.0, 4.0], [2.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 5.0], [5.0, 4.0, 5.0], [6.0, 4.0, 7.0]], 
[[3.0, 8.0, 2.0], [6.0, 3.0, 1.0], [6.0, 1.0, 7.0]], [[4.0, 3.0, 6.0], [8.0, 7.0, 5.0], [5.0, 5.0, 7.0]]], [[[6.0, 8.0, 6.0], 
[8.0, 8.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 3.0, 2.0], [4.0, 6.0, 8.0], [2.0, 1.0, 5.0]], [[7.0, 1.0, 3.0], [1.0, 7.0, 2.0], 
[6.0, 7.0, 2.0]]], [[[5.0, 6.0, 5.0], [2.0, 8.0, 8.0], [6.0, 7.0, 1.0]], [[5.0, 6.0, 4.0], [2.0, 3.0, 8.0], [3.0, 1.0, 3.0]], 
[[7.0, 7.0, 5.0], [7.0, 1.0, 8.0], [4.0, 2.0, 8.0]]], [[[3.0, 1.0, 7.0], [8.0, 1.0, 3.0], [5.0, 8.0, 7.0]], [[5.0, 7.0, 3.0], 
[6.0, 3.0, 7.0], [3.0, 4.0, 4.0]], [[3.0, 3.0, 3.0], [6.0, 4.0, 4.0], [2.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_array_U(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([6.0, 5.0, 8.0])+(1.-msk_arg)*numpy.array([2.0, 8.0, 7.0])
      val=numpy.array([8.0, 8.0])
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([8.0, 8.0, 8.0])+(1.-msk_ref)*numpy.array([8.0, 8.0, 7.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_constData_U(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([8.0, 8.0, 7.0])+(1.-msk_arg)*numpy.array([4.0, 3.0, 6.0])
      val=Data(numpy.array([2.0, 4.0]),self.functionspace)
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.0, 4.0, 7.0])+(1.-msk_ref)*numpy.array([2.0, 4.0, 6.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_expandedData_U(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([6.0, 2.0, 8.0])+(1.-msk_arg)*numpy.array([2.0, 1.0, 1.0])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0, 5.0])+(1.-msk_val)*numpy.array([8.0, 1.0])
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([5.0, 5.0, 8.0])+(1.-msk_ref)*numpy.array([8.0, 1.0, 1.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_array_L(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([1.0, 5.0, 3.0])+(1.-msk_arg)*numpy.array([7.0, 5.0, 8.0])
      val=numpy.array([7.0])
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.0, 5.0, 7.0])+(1.-msk_ref)*numpy.array([7.0, 5.0, 7.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_constData_L(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([4.0, 7.0, 6.0])+(1.-msk_arg)*numpy.array([4.0, 2.0, 2.0])
      val=Data(numpy.array([1.0]),self.functionspace)
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.0, 7.0, 1.0])+(1.-msk_ref)*numpy.array([4.0, 2.0, 1.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_expandedData_L(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([7.0, 7.0, 5.0])+(1.-msk_arg)*numpy.array([1.0, 2.0, 4.0])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([6.0])+(1.-msk_val)*numpy.array([5.0])
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.0, 7.0, 6.0])+(1.-msk_ref)*numpy.array([1.0, 2.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_array_R(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([8.0, 3.0, 2.0])+(1.-msk_arg)*numpy.array([1.0, 1.0, 5.0])
      val=numpy.array([2.0])
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([8.0, 2.0, 2.0])+(1.-msk_ref)*numpy.array([1.0, 2.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_constData_R(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([4.0, 5.0, 3.0])+(1.-msk_arg)*numpy.array([6.0, 2.0, 5.0])
      val=Data(numpy.array([2.0]),self.functionspace)
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.0, 2.0, 3.0])+(1.-msk_ref)*numpy.array([6.0, 2.0, 5.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_expandedData_R(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([7.0, 4.0, 4.0])+(1.-msk_arg)*numpy.array([3.0, 8.0, 8.0])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([4.0])+(1.-msk_val)*numpy.array([8.0])
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.0, 4.0, 4.0])+(1.-msk_ref)*numpy.array([3.0, 8.0, 8.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_array_P(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([2.0, 6.0, 4.0])+(1.-msk_arg)*numpy.array([5.0, 8.0, 1.0])
      val=numpy.array(1.0)
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.0, 1.0, 4.0])+(1.-msk_ref)*numpy.array([5.0, 1.0, 1.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_constData_P(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([6.0, 3.0, 7.0])+(1.-msk_arg)*numpy.array([3.0, 1.0, 3.0])
      val=Data(1.0,self.functionspace)
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([6.0, 1.0, 7.0])+(1.-msk_ref)*numpy.array([3.0, 1.0, 3.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_expandedData_P(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([5.0, 3.0, 8.0])+(1.-msk_arg)*numpy.array([2.0, 4.0, 8.0])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*(1.0)+(1.-msk_val)*(1.0)
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([5.0, 1.0, 8.0])+(1.-msk_ref)*numpy.array([2.0, 1.0, 8.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_array_C(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([3.0, 1.0, 5.0])+(1.-msk_arg)*numpy.array([3.0, 8.0, 2.0])
      val=numpy.array([4.0, 6.0, 2.0])
      arg[:]=val
      ref=Data(numpy.array([4.0, 6.0, 2.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_constData_C(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([3.0, 2.0, 7.0])+(1.-msk_arg)*numpy.array([1.0, 7.0, 2.0])
      val=Data(numpy.array([7.0, 6.0, 1.0]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([7.0, 6.0, 1.0]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank1_expandedData_C(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([8.0, 7.0, 4.0])+(1.-msk_arg)*numpy.array([1.0, 1.0, 1.0])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([3.0, 2.0, 4.0])+(1.-msk_val)*numpy.array([8.0, 5.0, 8.0])
      arg[:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([3.0, 2.0, 4.0])+(1.-msk_ref)*numpy.array([8.0, 5.0, 8.0])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_UN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 7.0, 7.0, 1.0], [8.0, 2.0, 7.0, 6.0], [2.0, 2.0, 6.0, 
2.0]])+(1.-msk_arg)*numpy.array([[5.0, 5.0, 6.0, 1.0], [5.0, 5.0, 4.0, 4.0], [4.0, 6.0, 1.0, 7.0]])
      val=numpy.array([[5.0, 1.0, 1.0, 5.0], [7.0, 2.0, 4.0, 7.0]])
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 1.0, 1.0, 5.0], [7.0, 2.0, 4.0, 7.0], [2.0, 2.0, 6.0, 
2.0]])+(1.-msk_ref)*numpy.array([[5.0, 1.0, 1.0, 5.0], [7.0, 2.0, 4.0, 7.0], [4.0, 6.0, 1.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_UN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 5.0, 2.0, 2.0], [3.0, 3.0, 6.0, 6.0], [7.0, 7.0, 7.0, 
3.0]])+(1.-msk_arg)*numpy.array([[4.0, 3.0, 4.0, 5.0], [1.0, 1.0, 7.0, 6.0], [4.0, 3.0, 8.0, 6.0]])
      val=Data(numpy.array([[2.0, 2.0, 4.0, 8.0], [3.0, 2.0, 8.0, 8.0]]),self.functionspace)
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 2.0, 4.0, 8.0], [3.0, 2.0, 8.0, 8.0], [7.0, 7.0, 7.0, 
3.0]])+(1.-msk_ref)*numpy.array([[2.0, 2.0, 4.0, 8.0], [3.0, 2.0, 8.0, 8.0], [4.0, 3.0, 8.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_UN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[4.0, 6.0, 8.0, 8.0], [7.0, 7.0, 4.0, 1.0], [7.0, 2.0, 1.0, 
2.0]])+(1.-msk_arg)*numpy.array([[7.0, 3.0, 5.0, 4.0], [1.0, 5.0, 6.0, 6.0], [4.0, 5.0, 6.0, 1.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 2.0, 2.0, 3.0], [7.0, 6.0, 6.0, 6.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0, 1.0, 5.0], 
[8.0, 2.0, 7.0, 5.0]])
      arg[:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 2.0, 2.0, 3.0], [7.0, 6.0, 6.0, 6.0], [7.0, 2.0, 1.0, 
2.0]])+(1.-msk_ref)*numpy.array([[3.0, 4.0, 1.0, 5.0], [8.0, 2.0, 7.0, 5.0], [4.0, 5.0, 6.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_UC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 3.0, 1.0, 4.0], [5.0, 2.0, 6.0, 5.0], [3.0, 8.0, 7.0, 
5.0]])+(1.-msk_arg)*numpy.array([[4.0, 5.0, 5.0, 7.0], [1.0, 6.0, 6.0, 1.0], [3.0, 3.0, 3.0, 7.0]])
      val=numpy.array([[4.0, 7.0, 2.0, 7.0], [6.0, 2.0, 2.0, 8.0]])
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 7.0, 2.0, 7.0], [6.0, 2.0, 2.0, 8.0], [3.0, 8.0, 7.0, 
5.0]])+(1.-msk_ref)*numpy.array([[4.0, 7.0, 2.0, 7.0], [6.0, 2.0, 2.0, 8.0], [3.0, 3.0, 3.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_UC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 6.0, 2.0, 1.0], [8.0, 8.0, 7.0, 3.0], [1.0, 3.0, 6.0, 
4.0]])+(1.-msk_arg)*numpy.array([[6.0, 8.0, 5.0, 2.0], [7.0, 3.0, 8.0, 6.0], [7.0, 1.0, 4.0, 4.0]])
      val=Data(numpy.array([[3.0, 2.0, 5.0, 6.0], [6.0, 6.0, 3.0, 2.0]]),self.functionspace)
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 2.0, 5.0, 6.0], [6.0, 6.0, 3.0, 2.0], [1.0, 3.0, 6.0, 
4.0]])+(1.-msk_ref)*numpy.array([[3.0, 2.0, 5.0, 6.0], [6.0, 6.0, 3.0, 2.0], [7.0, 1.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_UC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 2.0, 1.0, 1.0], [5.0, 4.0, 2.0, 7.0], [2.0, 2.0, 8.0, 
6.0]])+(1.-msk_arg)*numpy.array([[1.0, 7.0, 1.0, 1.0], [3.0, 1.0, 4.0, 5.0], [6.0, 1.0, 8.0, 1.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 4.0, 4.0, 8.0], [8.0, 2.0, 5.0, 2.0]])+(1.-msk_val)*numpy.array([[1.0, 6.0, 8.0, 1.0], 
[5.0, 6.0, 4.0, 4.0]])
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 4.0, 4.0, 8.0], [8.0, 2.0, 5.0, 2.0], [2.0, 2.0, 8.0, 
6.0]])+(1.-msk_ref)*numpy.array([[1.0, 6.0, 8.0, 1.0], [5.0, 6.0, 4.0, 4.0], [6.0, 1.0, 8.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_CC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 6.0, 4.0, 2.0], [6.0, 1.0, 3.0, 6.0], [2.0, 3.0, 1.0, 
4.0]])+(1.-msk_arg)*numpy.array([[3.0, 4.0, 6.0, 7.0], [3.0, 6.0, 7.0, 7.0], [4.0, 4.0, 7.0, 8.0]])
      val=numpy.array([[6.0, 8.0, 3.0, 8.0], [8.0, 1.0, 5.0, 6.0], [5.0, 8.0, 2.0, 2.0]])
      arg[:,:]=val
      ref=Data(numpy.array([[6.0, 8.0, 3.0, 8.0], [8.0, 1.0, 5.0, 6.0], [5.0, 8.0, 2.0, 2.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_CC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 2.0, 8.0, 6.0], [6.0, 1.0, 6.0, 7.0], [6.0, 1.0, 2.0, 
1.0]])+(1.-msk_arg)*numpy.array([[1.0, 4.0, 6.0, 6.0], [1.0, 4.0, 4.0, 1.0], [2.0, 3.0, 7.0, 3.0]])
      val=Data(numpy.array([[3.0, 6.0, 4.0, 5.0], [2.0, 5.0, 5.0, 6.0], [7.0, 8.0, 6.0, 2.0]]),self.functionspace)
      arg[:,:]=val
      ref=Data(numpy.array([[3.0, 6.0, 4.0, 5.0], [2.0, 5.0, 5.0, 6.0], [7.0, 8.0, 6.0, 2.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_CC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 6.0, 2.0, 7.0], [7.0, 1.0, 6.0, 3.0], [3.0, 3.0, 2.0, 
2.0]])+(1.-msk_arg)*numpy.array([[6.0, 8.0, 1.0, 6.0], [1.0, 1.0, 3.0, 3.0], [7.0, 4.0, 1.0, 4.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 2.0, 1.0, 7.0], [6.0, 1.0, 2.0, 2.0], [3.0, 4.0, 4.0, 
1.0]])+(1.-msk_val)*numpy.array([[6.0, 5.0, 3.0, 4.0], [1.0, 1.0, 2.0, 8.0], [8.0, 1.0, 7.0, 1.0]])
      arg[:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 2.0, 1.0, 7.0], [6.0, 1.0, 2.0, 2.0], [3.0, 4.0, 4.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 5.0, 3.0, 4.0], [1.0, 1.0, 2.0, 8.0], [8.0, 1.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_UL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 4.0, 7.0, 6.0], [7.0, 2.0, 6.0, 5.0], [5.0, 2.0, 2.0, 
8.0]])+(1.-msk_arg)*numpy.array([[2.0, 6.0, 5.0, 6.0], [1.0, 7.0, 7.0, 5.0], [6.0, 4.0, 8.0, 5.0]])
      val=numpy.array([[8.0, 7.0], [5.0, 1.0]])
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 4.0, 8.0, 7.0], [7.0, 2.0, 5.0, 1.0], [5.0, 2.0, 2.0, 
8.0]])+(1.-msk_ref)*numpy.array([[2.0, 6.0, 8.0, 7.0], [1.0, 7.0, 5.0, 1.0], [6.0, 4.0, 8.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_UL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 2.0, 1.0, 1.0], [7.0, 1.0, 6.0, 3.0], [6.0, 6.0, 6.0, 
8.0]])+(1.-msk_arg)*numpy.array([[7.0, 1.0, 1.0, 4.0], [7.0, 1.0, 5.0, 2.0], [2.0, 5.0, 8.0, 3.0]])
      val=Data(numpy.array([[8.0, 6.0], [1.0, 6.0]]),self.functionspace)
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 2.0, 8.0, 6.0], [7.0, 1.0, 1.0, 6.0], [6.0, 6.0, 6.0, 
8.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 8.0, 6.0], [7.0, 1.0, 1.0, 6.0], [2.0, 5.0, 8.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_UL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 2.0, 4.0, 1.0], [2.0, 4.0, 6.0, 8.0], [5.0, 1.0, 6.0, 
8.0]])+(1.-msk_arg)*numpy.array([[6.0, 2.0, 1.0, 1.0], [8.0, 8.0, 4.0, 2.0], [6.0, 2.0, 8.0, 7.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 5.0], [4.0, 7.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0], [3.0, 1.0]])
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 2.0, 4.0, 5.0], [2.0, 4.0, 4.0, 7.0], [5.0, 1.0, 6.0, 
8.0]])+(1.-msk_ref)*numpy.array([[6.0, 2.0, 3.0, 4.0], [8.0, 8.0, 3.0, 1.0], [6.0, 2.0, 8.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_PR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 3.0, 8.0, 1.0], [7.0, 7.0, 4.0, 5.0], [8.0, 7.0, 2.0, 
1.0]])+(1.-msk_arg)*numpy.array([[8.0, 1.0, 6.0, 5.0], [7.0, 3.0, 8.0, 1.0], [8.0, 6.0, 6.0, 4.0]])
      val=numpy.array([7.0, 1.0])
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 3.0, 8.0, 1.0], [7.0, 7.0, 1.0, 5.0], [8.0, 7.0, 2.0, 
1.0]])+(1.-msk_ref)*numpy.array([[8.0, 1.0, 6.0, 5.0], [7.0, 7.0, 1.0, 1.0], [8.0, 6.0, 6.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_PR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 2.0, 6.0, 6.0], [6.0, 1.0, 8.0, 8.0], [8.0, 6.0, 8.0, 
7.0]])+(1.-msk_arg)*numpy.array([[4.0, 6.0, 5.0, 6.0], [5.0, 5.0, 2.0, 4.0], [7.0, 1.0, 2.0, 6.0]])
      val=Data(numpy.array([4.0, 5.0]),self.functionspace)
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 2.0, 6.0, 6.0], [6.0, 4.0, 5.0, 8.0], [8.0, 6.0, 8.0, 
7.0]])+(1.-msk_ref)*numpy.array([[4.0, 6.0, 5.0, 6.0], [5.0, 4.0, 5.0, 4.0], [7.0, 1.0, 2.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_PR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 1.0, 8.0, 4.0], [7.0, 2.0, 8.0, 3.0], [4.0, 2.0, 4.0, 
3.0]])+(1.-msk_arg)*numpy.array([[8.0, 1.0, 4.0, 7.0], [4.0, 8.0, 1.0, 6.0], [5.0, 3.0, 1.0, 1.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([2.0, 2.0])+(1.-msk_val)*numpy.array([4.0, 7.0])
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 1.0, 8.0, 4.0], [7.0, 2.0, 2.0, 3.0], [4.0, 2.0, 4.0, 
3.0]])+(1.-msk_ref)*numpy.array([[8.0, 1.0, 4.0, 7.0], [4.0, 4.0, 7.0, 6.0], [5.0, 3.0, 1.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_PL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 8.0, 8.0, 4.0], [5.0, 1.0, 2.0, 8.0], [2.0, 3.0, 3.0, 
7.0]])+(1.-msk_arg)*numpy.array([[8.0, 4.0, 5.0, 6.0], [7.0, 8.0, 8.0, 5.0], [2.0, 6.0, 3.0, 1.0]])
      val=numpy.array([4.0, 2.0])
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 8.0, 8.0, 4.0], [5.0, 1.0, 4.0, 2.0], [2.0, 3.0, 3.0, 
7.0]])+(1.-msk_ref)*numpy.array([[8.0, 4.0, 5.0, 6.0], [7.0, 8.0, 4.0, 2.0], [2.0, 6.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_PL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 3.0, 1.0, 6.0], [7.0, 8.0, 3.0, 8.0], [5.0, 3.0, 3.0, 
5.0]])+(1.-msk_arg)*numpy.array([[6.0, 3.0, 1.0, 4.0], [4.0, 5.0, 7.0, 4.0], [5.0, 1.0, 5.0, 1.0]])
      val=Data(numpy.array([4.0, 1.0]),self.functionspace)
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 3.0, 1.0, 6.0], [7.0, 8.0, 4.0, 1.0], [5.0, 3.0, 3.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 3.0, 1.0, 4.0], [4.0, 5.0, 4.0, 1.0], [5.0, 1.0, 5.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_PL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 6.0, 5.0, 3.0], [3.0, 1.0, 8.0, 7.0], [3.0, 6.0, 1.0, 
1.0]])+(1.-msk_arg)*numpy.array([[6.0, 4.0, 1.0, 1.0], [1.0, 1.0, 1.0, 8.0], [1.0, 2.0, 8.0, 1.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([7.0, 1.0])+(1.-msk_val)*numpy.array([8.0, 3.0])
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 6.0, 5.0, 3.0], [3.0, 1.0, 7.0, 1.0], [3.0, 6.0, 1.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 1.0, 1.0], [1.0, 1.0, 8.0, 3.0], [1.0, 2.0, 8.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_RU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 8.0, 1.0, 3.0], [2.0, 1.0, 6.0, 4.0], [4.0, 2.0, 8.0, 
6.0]])+(1.-msk_arg)*numpy.array([[3.0, 8.0, 2.0, 7.0], [3.0, 6.0, 7.0, 5.0], [3.0, 3.0, 1.0, 6.0]])
      val=numpy.array([[8.0, 5.0, 4.0]])
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 8.0, 1.0, 3.0], [8.0, 5.0, 4.0, 4.0], [4.0, 2.0, 8.0, 
6.0]])+(1.-msk_ref)*numpy.array([[3.0, 8.0, 2.0, 7.0], [8.0, 5.0, 4.0, 5.0], [3.0, 3.0, 1.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_RU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 8.0, 2.0, 8.0], [8.0, 1.0, 3.0, 8.0], [7.0, 7.0, 7.0, 
7.0]])+(1.-msk_arg)*numpy.array([[4.0, 5.0, 1.0, 5.0], [7.0, 6.0, 1.0, 6.0], [2.0, 3.0, 2.0, 7.0]])
      val=Data(numpy.array([[8.0, 8.0, 6.0]]),self.functionspace)
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 8.0, 2.0, 8.0], [8.0, 8.0, 6.0, 8.0], [7.0, 7.0, 7.0, 
7.0]])+(1.-msk_ref)*numpy.array([[4.0, 5.0, 1.0, 5.0], [8.0, 8.0, 6.0, 6.0], [2.0, 3.0, 2.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_RU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 7.0, 3.0, 6.0], [5.0, 8.0, 4.0, 6.0], [7.0, 7.0, 7.0, 
6.0]])+(1.-msk_arg)*numpy.array([[6.0, 5.0, 4.0, 1.0], [8.0, 1.0, 6.0, 8.0], [5.0, 2.0, 1.0, 3.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 6.0, 6.0]])+(1.-msk_val)*numpy.array([[3.0, 5.0, 5.0]])
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 7.0, 3.0, 6.0], [2.0, 6.0, 6.0, 6.0], [7.0, 7.0, 7.0, 
6.0]])+(1.-msk_ref)*numpy.array([[6.0, 5.0, 4.0, 1.0], [3.0, 5.0, 5.0, 8.0], [5.0, 2.0, 1.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_CP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 1.0, 4.0, 4.0], [8.0, 6.0, 1.0, 8.0], [8.0, 4.0, 8.0, 
3.0]])+(1.-msk_arg)*numpy.array([[5.0, 4.0, 4.0, 8.0], [6.0, 1.0, 6.0, 4.0], [8.0, 4.0, 2.0, 7.0]])
      val=numpy.array([7.0, 4.0, 3.0])
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 1.0, 7.0, 4.0], [8.0, 6.0, 4.0, 8.0], [8.0, 4.0, 3.0, 
3.0]])+(1.-msk_ref)*numpy.array([[5.0, 4.0, 7.0, 8.0], [6.0, 1.0, 4.0, 4.0], [8.0, 4.0, 3.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_CP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 6.0, 1.0, 7.0], [2.0, 3.0, 4.0, 1.0], [8.0, 5.0, 1.0, 
6.0]])+(1.-msk_arg)*numpy.array([[4.0, 5.0, 4.0, 4.0], [5.0, 2.0, 3.0, 6.0], [6.0, 8.0, 8.0, 6.0]])
      val=Data(numpy.array([5.0, 3.0, 6.0]),self.functionspace)
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 6.0, 5.0, 7.0], [2.0, 3.0, 3.0, 1.0], [8.0, 5.0, 6.0, 
6.0]])+(1.-msk_ref)*numpy.array([[4.0, 5.0, 5.0, 4.0], [5.0, 2.0, 3.0, 6.0], [6.0, 8.0, 6.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_CP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[4.0, 3.0, 8.0, 4.0], [3.0, 2.0, 1.0, 5.0], [6.0, 6.0, 7.0, 
3.0]])+(1.-msk_arg)*numpy.array([[1.0, 4.0, 2.0, 8.0], [3.0, 8.0, 5.0, 1.0], [5.0, 6.0, 8.0, 1.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([4.0, 4.0, 8.0])+(1.-msk_val)*numpy.array([8.0, 2.0, 3.0])
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 3.0, 4.0, 4.0], [3.0, 2.0, 4.0, 5.0], [6.0, 6.0, 8.0, 
3.0]])+(1.-msk_ref)*numpy.array([[1.0, 4.0, 8.0, 8.0], [3.0, 8.0, 2.0, 1.0], [5.0, 6.0, 3.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_PU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[6.0, 1.0, 1.0, 6.0], [6.0, 4.0, 6.0, 1.0], [1.0, 6.0, 4.0, 
6.0]])+(1.-msk_arg)*numpy.array([[4.0, 1.0, 8.0, 8.0], [5.0, 5.0, 2.0, 1.0], [8.0, 8.0, 7.0, 4.0]])
      val=numpy.array([3.0, 6.0, 6.0])
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 1.0, 1.0, 6.0], [3.0, 6.0, 6.0, 1.0], [1.0, 6.0, 4.0, 
6.0]])+(1.-msk_ref)*numpy.array([[4.0, 1.0, 8.0, 8.0], [3.0, 6.0, 6.0, 1.0], [8.0, 8.0, 7.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_PU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 1.0, 2.0, 4.0], [8.0, 8.0, 5.0, 6.0], [2.0, 3.0, 5.0, 
4.0]])+(1.-msk_arg)*numpy.array([[7.0, 3.0, 6.0, 4.0], [4.0, 2.0, 3.0, 3.0], [3.0, 2.0, 6.0, 7.0]])
      val=Data(numpy.array([5.0, 8.0, 1.0]),self.functionspace)
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 1.0, 2.0, 4.0], [5.0, 8.0, 1.0, 6.0], [2.0, 3.0, 5.0, 
4.0]])+(1.-msk_ref)*numpy.array([[7.0, 3.0, 6.0, 4.0], [5.0, 8.0, 1.0, 3.0], [3.0, 2.0, 6.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_PU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[4.0, 6.0, 3.0, 5.0], [7.0, 6.0, 5.0, 8.0], [8.0, 7.0, 3.0, 
7.0]])+(1.-msk_arg)*numpy.array([[3.0, 2.0, 3.0, 7.0], [5.0, 4.0, 3.0, 7.0], [8.0, 6.0, 1.0, 7.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0, 1.0, 3.0])+(1.-msk_val)*numpy.array([3.0, 2.0, 4.0])
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 6.0, 3.0, 5.0], [5.0, 1.0, 3.0, 8.0], [8.0, 7.0, 3.0, 
7.0]])+(1.-msk_ref)*numpy.array([[3.0, 2.0, 3.0, 7.0], [3.0, 2.0, 4.0, 7.0], [8.0, 6.0, 1.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_RN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 2.0, 5.0, 5.0], [3.0, 8.0, 5.0, 4.0], [4.0, 3.0, 4.0, 
5.0]])+(1.-msk_arg)*numpy.array([[3.0, 8.0, 1.0, 8.0], [2.0, 1.0, 4.0, 2.0], [5.0, 5.0, 6.0, 8.0]])
      val=numpy.array([[6.0, 1.0, 5.0, 8.0]])
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 2.0, 5.0, 5.0], [6.0, 1.0, 5.0, 8.0], [4.0, 3.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[3.0, 8.0, 1.0, 8.0], [6.0, 1.0, 5.0, 8.0], [5.0, 5.0, 6.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_RN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 5.0, 3.0, 6.0], [3.0, 4.0, 3.0, 3.0], [7.0, 7.0, 4.0, 
4.0]])+(1.-msk_arg)*numpy.array([[3.0, 8.0, 6.0, 7.0], [4.0, 2.0, 4.0, 4.0], [6.0, 5.0, 3.0, 4.0]])
      val=Data(numpy.array([[5.0, 4.0, 1.0, 1.0]]),self.functionspace)
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 5.0, 3.0, 6.0], [5.0, 4.0, 1.0, 1.0], [7.0, 7.0, 4.0, 
4.0]])+(1.-msk_ref)*numpy.array([[3.0, 8.0, 6.0, 7.0], [5.0, 4.0, 1.0, 1.0], [6.0, 5.0, 3.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_RN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 8.0, 8.0, 1.0], [8.0, 3.0, 5.0, 8.0], [2.0, 8.0, 4.0, 
5.0]])+(1.-msk_arg)*numpy.array([[6.0, 5.0, 3.0, 1.0], [7.0, 2.0, 1.0, 3.0], [4.0, 8.0, 6.0, 8.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 4.0, 3.0, 5.0]])+(1.-msk_val)*numpy.array([[1.0, 2.0, 2.0, 6.0]])
      arg[1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 8.0, 8.0, 1.0], [2.0, 4.0, 3.0, 5.0], [2.0, 8.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[6.0, 5.0, 3.0, 1.0], [1.0, 2.0, 2.0, 6.0], [4.0, 8.0, 6.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_RC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[6.0, 2.0, 6.0, 6.0], [3.0, 3.0, 1.0, 1.0], [7.0, 3.0, 7.0, 
5.0]])+(1.-msk_arg)*numpy.array([[7.0, 4.0, 1.0, 2.0], [8.0, 3.0, 1.0, 3.0], [8.0, 1.0, 1.0, 5.0]])
      val=numpy.array([[5.0, 7.0, 2.0, 5.0]])
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 2.0, 6.0, 6.0], [5.0, 7.0, 2.0, 5.0], [7.0, 3.0, 7.0, 
5.0]])+(1.-msk_ref)*numpy.array([[7.0, 4.0, 1.0, 2.0], [5.0, 7.0, 2.0, 5.0], [8.0, 1.0, 1.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_RC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 1.0, 8.0, 4.0], [4.0, 2.0, 6.0, 7.0], [7.0, 5.0, 4.0, 
5.0]])+(1.-msk_arg)*numpy.array([[5.0, 5.0, 8.0, 6.0], [1.0, 4.0, 7.0, 7.0], [4.0, 7.0, 7.0, 1.0]])
      val=Data(numpy.array([[3.0, 6.0, 2.0, 1.0]]),self.functionspace)
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 1.0, 8.0, 4.0], [3.0, 6.0, 2.0, 1.0], [7.0, 5.0, 4.0, 
5.0]])+(1.-msk_ref)*numpy.array([[5.0, 5.0, 8.0, 6.0], [3.0, 6.0, 2.0, 1.0], [4.0, 7.0, 7.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_RC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 7.0, 8.0, 8.0], [5.0, 5.0, 6.0, 1.0], [5.0, 6.0, 7.0, 
6.0]])+(1.-msk_arg)*numpy.array([[5.0, 3.0, 4.0, 3.0], [8.0, 3.0, 6.0, 3.0], [6.0, 8.0, 8.0, 7.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 2.0, 2.0, 1.0]])+(1.-msk_val)*numpy.array([[5.0, 2.0, 6.0, 8.0]])
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 7.0, 8.0, 8.0], [1.0, 2.0, 2.0, 1.0], [5.0, 6.0, 7.0, 
6.0]])+(1.-msk_ref)*numpy.array([[5.0, 3.0, 4.0, 3.0], [5.0, 2.0, 6.0, 8.0], [6.0, 8.0, 8.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_CN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 2.0, 5.0, 6.0], [5.0, 6.0, 5.0, 2.0], [3.0, 3.0, 6.0, 
1.0]])+(1.-msk_arg)*numpy.array([[3.0, 3.0, 1.0, 4.0], [7.0, 8.0, 2.0, 2.0], [2.0, 1.0, 8.0, 6.0]])
      val=numpy.array([[7.0, 6.0, 6.0, 8.0], [3.0, 6.0, 8.0, 1.0], [3.0, 5.0, 2.0, 3.0]])
      arg[:]=val
      ref=Data(numpy.array([[7.0, 6.0, 6.0, 8.0], [3.0, 6.0, 8.0, 1.0], [3.0, 5.0, 2.0, 3.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_CN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 4.0, 2.0, 6.0], [8.0, 6.0, 7.0, 2.0], [2.0, 4.0, 6.0, 
3.0]])+(1.-msk_arg)*numpy.array([[3.0, 5.0, 8.0, 8.0], [5.0, 8.0, 4.0, 6.0], [5.0, 7.0, 3.0, 2.0]])
      val=Data(numpy.array([[6.0, 7.0, 6.0, 8.0], [4.0, 7.0, 4.0, 2.0], [6.0, 2.0, 8.0, 5.0]]),self.functionspace)
      arg[:]=val
      ref=Data(numpy.array([[6.0, 7.0, 6.0, 8.0], [4.0, 7.0, 4.0, 2.0], [6.0, 2.0, 8.0, 5.0]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_CN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 4.0, 6.0, 6.0], [1.0, 4.0, 1.0, 1.0], [6.0, 2.0, 4.0, 
5.0]])+(1.-msk_arg)*numpy.array([[1.0, 4.0, 3.0, 2.0], [4.0, 6.0, 2.0, 5.0], [2.0, 3.0, 6.0, 6.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 1.0, 3.0, 8.0], [7.0, 7.0, 4.0, 5.0], [3.0, 6.0, 3.0, 
2.0]])+(1.-msk_val)*numpy.array([[8.0, 4.0, 2.0, 6.0], [2.0, 1.0, 5.0, 1.0], [3.0, 8.0, 8.0, 3.0]])
      arg[:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 1.0, 3.0, 8.0], [7.0, 7.0, 4.0, 5.0], [3.0, 6.0, 3.0, 
2.0]])+(1.-msk_ref)*numpy.array([[8.0, 4.0, 2.0, 6.0], [2.0, 1.0, 5.0, 1.0], [3.0, 8.0, 8.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_RL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[4.0, 4.0, 8.0, 2.0], [1.0, 4.0, 4.0, 5.0], [2.0, 6.0, 7.0, 
1.0]])+(1.-msk_arg)*numpy.array([[1.0, 6.0, 6.0, 4.0], [3.0, 1.0, 3.0, 7.0], [6.0, 7.0, 4.0, 4.0]])
      val=numpy.array([[7.0, 4.0]])
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 4.0, 8.0, 2.0], [1.0, 4.0, 7.0, 4.0], [2.0, 6.0, 7.0, 
1.0]])+(1.-msk_ref)*numpy.array([[1.0, 6.0, 6.0, 4.0], [3.0, 1.0, 7.0, 4.0], [6.0, 7.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_RL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 1.0, 4.0, 8.0], [3.0, 7.0, 2.0, 4.0], [8.0, 7.0, 7.0, 
8.0]])+(1.-msk_arg)*numpy.array([[3.0, 5.0, 8.0, 8.0], [6.0, 3.0, 1.0, 6.0], [5.0, 3.0, 1.0, 4.0]])
      val=Data(numpy.array([[5.0, 6.0]]),self.functionspace)
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 1.0, 4.0, 8.0], [3.0, 7.0, 5.0, 6.0], [8.0, 7.0, 7.0, 
8.0]])+(1.-msk_ref)*numpy.array([[3.0, 5.0, 8.0, 8.0], [6.0, 3.0, 5.0, 6.0], [5.0, 3.0, 1.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_RL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 4.0, 5.0, 1.0], [6.0, 7.0, 2.0, 8.0], [2.0, 1.0, 4.0, 
7.0]])+(1.-msk_arg)*numpy.array([[5.0, 8.0, 3.0, 8.0], [6.0, 6.0, 5.0, 2.0], [4.0, 2.0, 4.0, 3.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 3.0]])+(1.-msk_val)*numpy.array([[1.0, 3.0]])
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 4.0, 5.0, 1.0], [6.0, 7.0, 3.0, 3.0], [2.0, 1.0, 4.0, 
7.0]])+(1.-msk_ref)*numpy.array([[5.0, 8.0, 3.0, 8.0], [6.0, 6.0, 1.0, 3.0], [4.0, 2.0, 4.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_LC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 1.0, 5.0, 6.0], [2.0, 5.0, 7.0, 1.0], [7.0, 7.0, 3.0, 
4.0]])+(1.-msk_arg)*numpy.array([[8.0, 3.0, 2.0, 2.0], [2.0, 1.0, 1.0, 2.0], [8.0, 8.0, 6.0, 7.0]])
      val=numpy.array([[2.0, 8.0, 7.0, 4.0]])
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 1.0, 5.0, 6.0], [2.0, 5.0, 7.0, 1.0], [2.0, 8.0, 7.0, 
4.0]])+(1.-msk_ref)*numpy.array([[8.0, 3.0, 2.0, 2.0], [2.0, 1.0, 1.0, 2.0], [2.0, 8.0, 7.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_LC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 4.0, 6.0, 8.0], [7.0, 1.0, 6.0, 1.0], [8.0, 5.0, 1.0, 
5.0]])+(1.-msk_arg)*numpy.array([[8.0, 2.0, 4.0, 7.0], [6.0, 2.0, 8.0, 5.0], [6.0, 5.0, 1.0, 7.0]])
      val=Data(numpy.array([[4.0, 4.0, 6.0, 2.0]]),self.functionspace)
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 4.0, 6.0, 8.0], [7.0, 1.0, 6.0, 1.0], [4.0, 4.0, 6.0, 
2.0]])+(1.-msk_ref)*numpy.array([[8.0, 2.0, 4.0, 7.0], [6.0, 2.0, 8.0, 5.0], [4.0, 4.0, 6.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_LC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 2.0, 5.0, 8.0], [3.0, 5.0, 5.0, 6.0], [4.0, 5.0, 8.0, 
3.0]])+(1.-msk_arg)*numpy.array([[7.0, 1.0, 1.0, 2.0], [4.0, 2.0, 5.0, 5.0], [4.0, 5.0, 5.0, 2.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 1.0, 8.0, 2.0]])+(1.-msk_val)*numpy.array([[6.0, 4.0, 4.0, 4.0]])
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 2.0, 5.0, 8.0], [3.0, 5.0, 5.0, 6.0], [3.0, 1.0, 8.0, 
2.0]])+(1.-msk_ref)*numpy.array([[7.0, 1.0, 1.0, 2.0], [4.0, 2.0, 5.0, 5.0], [6.0, 4.0, 4.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_PN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 6.0, 6.0, 5.0], [2.0, 3.0, 4.0, 3.0], [1.0, 6.0, 4.0, 
4.0]])+(1.-msk_arg)*numpy.array([[6.0, 4.0, 4.0, 8.0], [3.0, 2.0, 4.0, 1.0], [8.0, 2.0, 3.0, 3.0]])
      val=numpy.array([2.0, 3.0, 2.0, 6.0])
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 6.0, 6.0, 5.0], [2.0, 3.0, 2.0, 6.0], [1.0, 6.0, 4.0, 
4.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0, 4.0, 8.0], [2.0, 3.0, 2.0, 6.0], [8.0, 2.0, 3.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_PN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 3.0, 7.0, 6.0], [8.0, 2.0, 6.0, 3.0], [8.0, 4.0, 8.0, 
5.0]])+(1.-msk_arg)*numpy.array([[4.0, 3.0, 5.0, 3.0], [2.0, 8.0, 5.0, 4.0], [4.0, 6.0, 2.0, 7.0]])
      val=Data(numpy.array([8.0, 4.0, 8.0, 7.0]),self.functionspace)
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 3.0, 7.0, 6.0], [8.0, 4.0, 8.0, 7.0], [8.0, 4.0, 8.0, 
5.0]])+(1.-msk_ref)*numpy.array([[4.0, 3.0, 5.0, 3.0], [8.0, 4.0, 8.0, 7.0], [4.0, 6.0, 2.0, 7.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_PN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 7.0, 6.0, 4.0], [8.0, 8.0, 1.0, 1.0], [6.0, 7.0, 5.0, 
1.0]])+(1.-msk_arg)*numpy.array([[6.0, 3.0, 1.0, 1.0], [8.0, 2.0, 4.0, 5.0], [4.0, 3.0, 4.0, 2.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([7.0, 8.0, 8.0, 4.0])+(1.-msk_val)*numpy.array([6.0, 8.0, 5.0, 2.0])
      arg[1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 7.0, 6.0, 4.0], [7.0, 8.0, 8.0, 4.0], [6.0, 7.0, 5.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 3.0, 1.0, 1.0], [6.0, 8.0, 5.0, 2.0], [4.0, 3.0, 4.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_LL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 7.0, 5.0, 1.0], [1.0, 7.0, 8.0, 2.0], [6.0, 7.0, 5.0, 
2.0]])+(1.-msk_arg)*numpy.array([[4.0, 3.0, 7.0, 1.0], [3.0, 6.0, 2.0, 2.0], [4.0, 3.0, 8.0, 3.0]])
      val=numpy.array([[5.0, 4.0]])
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 7.0, 5.0, 1.0], [1.0, 7.0, 8.0, 2.0], [6.0, 7.0, 5.0, 
4.0]])+(1.-msk_ref)*numpy.array([[4.0, 3.0, 7.0, 1.0], [3.0, 6.0, 2.0, 2.0], [4.0, 3.0, 5.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_LL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 8.0, 1.0, 2.0], [6.0, 3.0, 2.0, 6.0], [5.0, 8.0, 5.0, 
7.0]])+(1.-msk_arg)*numpy.array([[6.0, 5.0, 7.0, 4.0], [3.0, 2.0, 8.0, 4.0], [4.0, 3.0, 7.0, 1.0]])
      val=Data(numpy.array([[2.0, 1.0]]),self.functionspace)
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0, 8.0, 1.0, 2.0], [6.0, 3.0, 2.0, 6.0], [5.0, 8.0, 2.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 5.0, 7.0, 4.0], [3.0, 2.0, 8.0, 4.0], [4.0, 3.0, 2.0, 1.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_LL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[4.0, 1.0, 1.0, 3.0], [7.0, 5.0, 3.0, 4.0], [4.0, 5.0, 5.0, 
8.0]])+(1.-msk_arg)*numpy.array([[8.0, 7.0, 8.0, 3.0], [4.0, 5.0, 6.0, 7.0], [1.0, 7.0, 6.0, 3.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 7.0]])+(1.-msk_val)*numpy.array([[1.0, 8.0]])
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.0, 1.0, 1.0, 3.0], [7.0, 5.0, 3.0, 4.0], [4.0, 5.0, 8.0, 
7.0]])+(1.-msk_ref)*numpy.array([[8.0, 7.0, 8.0, 3.0], [4.0, 5.0, 6.0, 7.0], [1.0, 7.0, 1.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_RR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[6.0, 6.0, 1.0, 7.0], [6.0, 7.0, 3.0, 1.0], [3.0, 7.0, 8.0, 
8.0]])+(1.-msk_arg)*numpy.array([[4.0, 7.0, 5.0, 7.0], [2.0, 5.0, 8.0, 3.0], [7.0, 7.0, 1.0, 4.0]])
      val=numpy.array([[7.0, 8.0]])
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 6.0, 1.0, 7.0], [6.0, 7.0, 8.0, 1.0], [3.0, 7.0, 8.0, 
8.0]])+(1.-msk_ref)*numpy.array([[4.0, 7.0, 5.0, 7.0], [2.0, 7.0, 8.0, 3.0], [7.0, 7.0, 1.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_RR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[6.0, 8.0, 8.0, 8.0], [6.0, 2.0, 7.0, 4.0], [1.0, 5.0, 4.0, 
7.0]])+(1.-msk_arg)*numpy.array([[8.0, 3.0, 6.0, 3.0], [2.0, 7.0, 2.0, 7.0], [1.0, 1.0, 3.0, 6.0]])
      val=Data(numpy.array([[8.0, 7.0]]),self.functionspace)
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0, 8.0, 8.0, 8.0], [6.0, 8.0, 7.0, 4.0], [1.0, 5.0, 4.0, 
7.0]])+(1.-msk_ref)*numpy.array([[8.0, 3.0, 6.0, 3.0], [2.0, 8.0, 7.0, 7.0], [1.0, 1.0, 3.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_RR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 4.0, 5.0, 8.0], [4.0, 2.0, 2.0, 6.0], [1.0, 1.0, 1.0, 
4.0]])+(1.-msk_arg)*numpy.array([[1.0, 8.0, 7.0, 3.0], [7.0, 3.0, 2.0, 6.0], [5.0, 2.0, 1.0, 4.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 3.0]])+(1.-msk_val)*numpy.array([[6.0, 7.0]])
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 4.0, 5.0, 8.0], [4.0, 5.0, 3.0, 6.0], [1.0, 1.0, 1.0, 
4.0]])+(1.-msk_ref)*numpy.array([[1.0, 8.0, 7.0, 3.0], [7.0, 6.0, 7.0, 6.0], [5.0, 2.0, 1.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_LN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[2.0, 2.0, 2.0, 7.0], [5.0, 1.0, 3.0, 4.0], [7.0, 5.0, 8.0, 
4.0]])+(1.-msk_arg)*numpy.array([[6.0, 8.0, 8.0, 2.0], [5.0, 2.0, 5.0, 7.0], [3.0, 1.0, 6.0, 3.0]])
      val=numpy.array([[1.0, 5.0, 7.0, 3.0]])
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 2.0, 2.0, 7.0], [5.0, 1.0, 3.0, 4.0], [1.0, 5.0, 7.0, 
3.0]])+(1.-msk_ref)*numpy.array([[6.0, 8.0, 8.0, 2.0], [5.0, 2.0, 5.0, 7.0], [1.0, 5.0, 7.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_LN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 7.0, 4.0, 2.0], [4.0, 1.0, 2.0, 4.0], [8.0, 3.0, 6.0, 
5.0]])+(1.-msk_arg)*numpy.array([[1.0, 2.0, 5.0, 3.0], [1.0, 4.0, 6.0, 8.0], [4.0, 1.0, 6.0, 1.0]])
      val=Data(numpy.array([[5.0, 6.0, 6.0, 6.0]]),self.functionspace)
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 7.0, 4.0, 2.0], [4.0, 1.0, 2.0, 4.0], [5.0, 6.0, 6.0, 
6.0]])+(1.-msk_ref)*numpy.array([[1.0, 2.0, 5.0, 3.0], [1.0, 4.0, 6.0, 8.0], [5.0, 6.0, 6.0, 6.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_LN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[5.0, 8.0, 3.0, 5.0], [2.0, 5.0, 8.0, 5.0], [7.0, 1.0, 4.0, 
2.0]])+(1.-msk_arg)*numpy.array([[6.0, 6.0, 1.0, 1.0], [7.0, 2.0, 6.0, 5.0], [3.0, 7.0, 2.0, 4.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 5.0, 3.0, 1.0]])+(1.-msk_val)*numpy.array([[7.0, 8.0, 6.0, 3.0]])
      arg[2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 8.0, 3.0, 5.0], [2.0, 5.0, 8.0, 5.0], [6.0, 5.0, 3.0, 
1.0]])+(1.-msk_ref)*numpy.array([[6.0, 6.0, 1.0, 1.0], [7.0, 2.0, 6.0, 5.0], [7.0, 8.0, 6.0, 3.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_UU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 1.0, 1.0, 6.0], [4.0, 7.0, 8.0, 4.0], [1.0, 8.0, 2.0, 
5.0]])+(1.-msk_arg)*numpy.array([[5.0, 5.0, 5.0, 3.0], [6.0, 4.0, 7.0, 6.0], [4.0, 2.0, 1.0, 8.0]])
      val=numpy.array([[8.0, 1.0, 4.0], [2.0, 3.0, 8.0]])
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 1.0, 4.0, 6.0], [2.0, 3.0, 8.0, 4.0], [1.0, 8.0, 2.0, 
5.0]])+(1.-msk_ref)*numpy.array([[8.0, 1.0, 4.0, 3.0], [2.0, 3.0, 8.0, 6.0], [4.0, 2.0, 1.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_UU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 8.0, 1.0, 4.0], [1.0, 6.0, 6.0, 2.0], [5.0, 4.0, 2.0, 
7.0]])+(1.-msk_arg)*numpy.array([[4.0, 1.0, 7.0, 3.0], [6.0, 6.0, 7.0, 5.0], [6.0, 2.0, 6.0, 5.0]])
      val=Data(numpy.array([[2.0, 7.0, 1.0], [2.0, 2.0, 4.0]]),self.functionspace)
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0, 7.0, 1.0, 4.0], [2.0, 2.0, 4.0, 2.0], [5.0, 4.0, 2.0, 
7.0]])+(1.-msk_ref)*numpy.array([[2.0, 7.0, 1.0, 3.0], [2.0, 2.0, 4.0, 5.0], [6.0, 2.0, 6.0, 5.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_UU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[3.0, 2.0, 6.0, 3.0], [3.0, 4.0, 6.0, 7.0], [5.0, 1.0, 3.0, 
4.0]])+(1.-msk_arg)*numpy.array([[8.0, 8.0, 7.0, 1.0], [4.0, 4.0, 1.0, 8.0], [4.0, 6.0, 2.0, 2.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 6.0, 7.0], [5.0, 5.0, 5.0]])+(1.-msk_val)*numpy.array([[5.0, 4.0, 6.0], [3.0, 6.0, 3.0]])
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.0, 6.0, 7.0, 3.0], [5.0, 5.0, 5.0, 7.0], [5.0, 1.0, 3.0, 
4.0]])+(1.-msk_ref)*numpy.array([[5.0, 4.0, 6.0, 1.0], [3.0, 6.0, 3.0, 8.0], [4.0, 6.0, 2.0, 2.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_array_PP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[1.0, 4.0, 1.0, 3.0], [4.0, 6.0, 1.0, 4.0], [2.0, 8.0, 2.0, 
1.0]])+(1.-msk_arg)*numpy.array([[3.0, 1.0, 1.0, 5.0], [8.0, 1.0, 2.0, 1.0], [4.0, 2.0, 5.0, 8.0]])
      val=numpy.array(1.0)
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0, 4.0, 1.0, 3.0], [4.0, 6.0, 1.0, 4.0], [2.0, 8.0, 2.0, 
1.0]])+(1.-msk_ref)*numpy.array([[3.0, 1.0, 1.0, 5.0], [8.0, 1.0, 1.0, 1.0], [4.0, 2.0, 5.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_constData_PP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[7.0, 7.0, 8.0, 4.0], [7.0, 3.0, 1.0, 7.0], [6.0, 7.0, 1.0, 
2.0]])+(1.-msk_arg)*numpy.array([[8.0, 6.0, 1.0, 1.0], [6.0, 7.0, 3.0, 6.0], [4.0, 4.0, 8.0, 8.0]])
      val=Data(1.0,self.functionspace)
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.0, 7.0, 8.0, 4.0], [7.0, 3.0, 1.0, 7.0], [6.0, 7.0, 1.0, 
2.0]])+(1.-msk_ref)*numpy.array([[8.0, 6.0, 1.0, 1.0], [6.0, 7.0, 1.0, 6.0], [4.0, 4.0, 8.0, 8.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank2_expandedData_PP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[8.0, 7.0, 8.0, 3.0], [5.0, 1.0, 8.0, 7.0], [3.0, 7.0, 8.0, 
1.0]])+(1.-msk_arg)*numpy.array([[5.0, 1.0, 4.0, 1.0], [7.0, 1.0, 6.0, 6.0], [1.0, 4.0, 6.0, 4.0]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*(1.0)+(1.-msk_val)*(1.0)
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.0, 7.0, 8.0, 3.0], [5.0, 1.0, 1.0, 7.0], [3.0, 7.0, 8.0, 
1.0]])+(1.-msk_ref)*numpy.array([[5.0, 1.0, 4.0, 1.0], [7.0, 1.0, 1.0, 6.0], [1.0, 4.0, 6.0, 4.0]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 2.0, 5.0], [5.0, 3.0, 8.0], [1.0, 4.0, 3.0], [6.0, 4.0, 1.0]], [[8.0, 1.0, 5.0], [4.0, 
3.0, 4.0], [1.0, 2.0, 5.0], [5.0, 7.0, 4.0]], [[4.0, 6.0, 6.0], [5.0, 6.0, 6.0], [5.0, 7.0, 4.0], [4.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 7.0, 3.0], [2.0, 2.0, 2.0], [5.0, 2.0, 7.0], [1.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [5.0, 
6.0, 5.0], [5.0, 2.0, 1.0], [8.0, 3.0, 1.0]], [[6.0, 8.0, 1.0], [1.0, 6.0, 3.0], [1.0, 6.0, 5.0], [8.0, 5.0, 6.0]]])
      val=numpy.array([[[7.0, 6.0, 6.0], [6.0, 8.0, 3.0]]])
      arg[2:,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 5.0], [5.0, 3.0, 8.0], [1.0, 4.0, 3.0], [6.0, 4.0, 1.0]], [[8.0, 1.0, 5.0], [4.0, 
3.0, 4.0], [1.0, 2.0, 5.0], [5.0, 7.0, 4.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 6.0], [6.0, 8.0, 3.0], [4.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 3.0], [2.0, 2.0, 2.0], [5.0, 2.0, 7.0], [1.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [5.0, 
6.0, 5.0], [5.0, 2.0, 1.0], [8.0, 3.0, 1.0]], [[6.0, 8.0, 1.0], [7.0, 6.0, 6.0], [6.0, 8.0, 3.0], [8.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 5.0], [1.0, 3.0, 1.0], [8.0, 6.0, 5.0], [8.0, 3.0, 3.0]], [[1.0, 4.0, 7.0], [3.0, 
6.0, 7.0], [3.0, 6.0, 6.0], [5.0, 3.0, 2.0]], [[1.0, 4.0, 7.0], [3.0, 5.0, 5.0], [2.0, 7.0, 1.0], [5.0, 1.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 4.0, 8.0], [4.0, 3.0, 3.0], [7.0, 3.0, 1.0], [4.0, 2.0, 7.0]], [[7.0, 7.0, 7.0], [8.0, 
3.0, 4.0], [4.0, 6.0, 7.0], [5.0, 7.0, 8.0]], [[5.0, 4.0, 8.0], [5.0, 1.0, 8.0], [7.0, 2.0, 2.0], [4.0, 6.0, 6.0]]])
      val=Data(numpy.array([[[3.0, 4.0, 4.0], [7.0, 4.0, 5.0]]]),self.functionspace)
      arg[2:,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 5.0], [1.0, 3.0, 1.0], [8.0, 6.0, 5.0], [8.0, 3.0, 3.0]], [[1.0, 4.0, 7.0], [3.0, 
6.0, 7.0], [3.0, 6.0, 6.0], [5.0, 3.0, 2.0]], [[1.0, 4.0, 7.0], [3.0, 4.0, 4.0], [7.0, 4.0, 5.0], [5.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 8.0], [4.0, 3.0, 3.0], [7.0, 3.0, 1.0], [4.0, 2.0, 7.0]], [[7.0, 7.0, 7.0], [8.0, 
3.0, 4.0], [4.0, 6.0, 7.0], [5.0, 7.0, 8.0]], [[5.0, 4.0, 8.0], [3.0, 4.0, 4.0], [7.0, 4.0, 5.0], [4.0, 6.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 2.0, 8.0], [3.0, 6.0, 2.0], [5.0, 2.0, 6.0], [6.0, 1.0, 2.0]], [[6.0, 8.0, 2.0], [6.0, 
2.0, 6.0], [7.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[5.0, 5.0, 2.0], [7.0, 1.0, 4.0], [8.0, 3.0, 2.0], [1.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 1.0, 4.0], [1.0, 6.0, 8.0], [4.0, 3.0, 8.0], [7.0, 8.0, 7.0]], [[6.0, 3.0, 6.0], [2.0, 
2.0, 7.0], [7.0, 8.0, 3.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 5.0], [2.0, 4.0, 2.0], [4.0, 7.0, 3.0], [5.0, 1.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 1.0, 1.0], [2.0, 5.0, 1.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 1.0], [1.0, 4.0, 
1.0]]])
      arg[2:,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 8.0], [3.0, 6.0, 2.0], [5.0, 2.0, 6.0], [6.0, 1.0, 2.0]], [[6.0, 8.0, 2.0], [6.0, 
2.0, 6.0], [7.0, 7.0, 2.0], [6.0, 8.0, 7.0]], [[5.0, 5.0, 2.0], [7.0, 1.0, 1.0], [2.0, 5.0, 1.0], [1.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 4.0], [1.0, 6.0, 8.0], [4.0, 3.0, 8.0], [7.0, 8.0, 7.0]], [[6.0, 3.0, 6.0], [2.0, 
2.0, 7.0], [7.0, 8.0, 3.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 5.0], [2.0, 7.0, 1.0], [1.0, 4.0, 1.0], [5.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 3.0, 5.0], [7.0, 7.0, 4.0], [6.0, 5.0, 1.0], [5.0, 4.0, 7.0]], [[5.0, 3.0, 4.0], [2.0, 
2.0, 7.0], [8.0, 3.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 3.0, 2.0], [4.0, 5.0, 5.0], [1.0, 7.0, 1.0], [5.0, 1.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 8.0], [8.0, 3.0, 7.0], [3.0, 2.0, 1.0], [3.0, 4.0, 4.0]], [[6.0, 1.0, 8.0], [6.0, 
6.0, 1.0], [5.0, 7.0, 5.0], [7.0, 1.0, 7.0]], [[7.0, 1.0, 1.0], [4.0, 4.0, 2.0], [1.0, 4.0, 3.0], [8.0, 2.0, 3.0]]])
      val=numpy.array([[2.0], [4.0], [7.0]])
      arg[:,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 5.0], [7.0, 7.0, 4.0], [6.0, 2.0, 1.0], [5.0, 4.0, 7.0]], [[5.0, 3.0, 4.0], [2.0, 
2.0, 7.0], [8.0, 4.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 3.0, 2.0], [4.0, 5.0, 5.0], [1.0, 7.0, 1.0], [5.0, 1.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 8.0], [8.0, 3.0, 7.0], [3.0, 2.0, 1.0], [3.0, 4.0, 4.0]], [[6.0, 1.0, 8.0], [6.0, 
6.0, 1.0], [5.0, 4.0, 5.0], [7.0, 1.0, 7.0]], [[7.0, 1.0, 1.0], [4.0, 4.0, 2.0], [1.0, 7.0, 3.0], [8.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 1.0, 3.0], [3.0, 8.0, 3.0], [3.0, 8.0, 4.0], [3.0, 1.0, 6.0]], [[7.0, 5.0, 5.0], [8.0, 
2.0, 1.0], [3.0, 1.0, 5.0], [6.0, 6.0, 8.0]], [[3.0, 6.0, 4.0], [8.0, 5.0, 8.0], [8.0, 1.0, 1.0], [6.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 5.0, 6.0], [7.0, 8.0, 7.0], [8.0, 7.0, 3.0], [4.0, 1.0, 2.0]], [[6.0, 1.0, 2.0], [8.0, 
8.0, 8.0], [2.0, 4.0, 6.0], [8.0, 4.0, 7.0]], [[8.0, 1.0, 7.0], [6.0, 5.0, 2.0], [4.0, 6.0, 5.0], [2.0, 1.0, 4.0]]])
      val=Data(numpy.array([[4.0], [3.0], [1.0]]),self.functionspace)
      arg[:,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 3.0], [3.0, 8.0, 3.0], [3.0, 4.0, 4.0], [3.0, 1.0, 6.0]], [[7.0, 5.0, 5.0], [8.0, 
2.0, 1.0], [3.0, 3.0, 5.0], [6.0, 6.0, 8.0]], [[3.0, 6.0, 4.0], [8.0, 5.0, 8.0], [8.0, 1.0, 1.0], [6.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 6.0], [7.0, 8.0, 7.0], [8.0, 4.0, 3.0], [4.0, 1.0, 2.0]], [[6.0, 1.0, 2.0], [8.0, 
8.0, 8.0], [2.0, 3.0, 6.0], [8.0, 4.0, 7.0]], [[8.0, 1.0, 7.0], [6.0, 5.0, 2.0], [4.0, 1.0, 5.0], [2.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 1.0, 2.0], [3.0, 4.0, 1.0], [5.0, 6.0, 3.0], [7.0, 7.0, 1.0]], [[7.0, 2.0, 4.0], [2.0, 
7.0, 6.0], [6.0, 7.0, 6.0], [5.0, 6.0, 1.0]], [[5.0, 5.0, 2.0], [7.0, 7.0, 5.0], [4.0, 7.0, 3.0], [5.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 1.0, 3.0], [3.0, 2.0, 4.0], [7.0, 5.0, 2.0], [5.0, 7.0, 6.0]], [[4.0, 7.0, 4.0], [5.0, 
8.0, 1.0], [6.0, 8.0, 7.0], [7.0, 5.0, 2.0]], [[6.0, 4.0, 4.0], [4.0, 4.0, 5.0], [8.0, 6.0, 4.0], [4.0, 4.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0], [3.0], [3.0]])+(1.-msk_val)*numpy.array([[2.0], [3.0], [5.0]])
      arg[:,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 2.0], [3.0, 4.0, 1.0], [5.0, 1.0, 3.0], [7.0, 7.0, 1.0]], [[7.0, 2.0, 4.0], [2.0, 
7.0, 6.0], [6.0, 3.0, 6.0], [5.0, 6.0, 1.0]], [[5.0, 5.0, 2.0], [7.0, 7.0, 5.0], [4.0, 3.0, 3.0], [5.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 3.0], [3.0, 2.0, 4.0], [7.0, 2.0, 2.0], [5.0, 7.0, 6.0]], [[4.0, 7.0, 4.0], [5.0, 
8.0, 1.0], [6.0, 3.0, 7.0], [7.0, 5.0, 2.0]], [[6.0, 4.0, 4.0], [4.0, 4.0, 5.0], [8.0, 5.0, 4.0], [4.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 5.0, 1.0], [6.0, 7.0, 8.0], [1.0, 1.0, 2.0], [8.0, 6.0, 6.0]], [[5.0, 5.0, 5.0], [4.0, 
6.0, 2.0], [4.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[5.0, 1.0, 3.0], [4.0, 1.0, 8.0], [6.0, 5.0, 6.0], [1.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 4.0], [4.0, 7.0, 6.0], [5.0, 8.0, 2.0], [5.0, 4.0, 6.0]], [[6.0, 5.0, 8.0], [4.0, 
5.0, 6.0], [3.0, 4.0, 7.0], [5.0, 6.0, 7.0]], [[4.0, 3.0, 8.0], [3.0, 4.0, 6.0], [2.0, 6.0, 5.0], [3.0, 5.0, 5.0]]])
      val=numpy.array([7.0, 6.0, 7.0])
      arg[1,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 1.0], [6.0, 7.0, 8.0], [1.0, 1.0, 2.0], [8.0, 6.0, 6.0]], [[5.0, 7.0, 5.0], [4.0, 
6.0, 2.0], [4.0, 7.0, 6.0], [4.0, 1.0, 1.0]], [[5.0, 1.0, 3.0], [4.0, 1.0, 8.0], [6.0, 5.0, 6.0], [1.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 4.0], [4.0, 7.0, 6.0], [5.0, 8.0, 2.0], [5.0, 4.0, 6.0]], [[6.0, 7.0, 8.0], [4.0, 
6.0, 6.0], [3.0, 7.0, 7.0], [5.0, 6.0, 7.0]], [[4.0, 3.0, 8.0], [3.0, 4.0, 6.0], [2.0, 6.0, 5.0], [3.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 7.0, 8.0], [3.0, 6.0, 1.0], [5.0, 7.0, 6.0], [6.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [7.0, 
7.0, 8.0], [6.0, 5.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 1.0, 3.0], [7.0, 3.0, 3.0], [6.0, 6.0, 6.0], [8.0, 6.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 5.0, 3.0], [7.0, 7.0, 4.0], [7.0, 6.0, 5.0], [5.0, 8.0, 7.0]], [[5.0, 5.0, 3.0], [1.0, 
8.0, 2.0], [3.0, 4.0, 1.0], [5.0, 2.0, 6.0]], [[6.0, 8.0, 4.0], [2.0, 4.0, 5.0], [6.0, 6.0, 7.0], [4.0, 4.0, 2.0]]])
      val=Data(numpy.array([8.0, 8.0, 1.0]),self.functionspace)
      arg[1,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 8.0], [3.0, 6.0, 1.0], [5.0, 7.0, 6.0], [6.0, 3.0, 1.0]], [[6.0, 8.0, 3.0], [7.0, 
8.0, 8.0], [6.0, 1.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 1.0, 3.0], [7.0, 3.0, 3.0], [6.0, 6.0, 6.0], [8.0, 6.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 3.0], [7.0, 7.0, 4.0], [7.0, 6.0, 5.0], [5.0, 8.0, 7.0]], [[5.0, 8.0, 3.0], [1.0, 
8.0, 2.0], [3.0, 1.0, 1.0], [5.0, 2.0, 6.0]], [[6.0, 8.0, 4.0], [2.0, 4.0, 5.0], [6.0, 6.0, 7.0], [4.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 1.0, 2.0], [7.0, 3.0, 2.0], [3.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[7.0, 6.0, 7.0], [1.0, 
3.0, 5.0], [5.0, 7.0, 7.0], [2.0, 8.0, 4.0]], [[7.0, 7.0, 3.0], [8.0, 8.0, 3.0], [2.0, 2.0, 8.0], [2.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 8.0, 1.0], [7.0, 8.0, 1.0], [6.0, 1.0, 4.0], [5.0, 2.0, 2.0]], [[5.0, 7.0, 7.0], [6.0, 
8.0, 5.0], [5.0, 6.0, 1.0], [1.0, 4.0, 1.0]], [[6.0, 5.0, 4.0], [3.0, 6.0, 1.0], [7.0, 7.0, 4.0], [5.0, 4.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([1.0, 5.0, 6.0])+(1.-msk_val)*numpy.array([3.0, 7.0, 1.0])
      arg[1,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 2.0], [7.0, 3.0, 2.0], [3.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[7.0, 1.0, 7.0], [1.0, 
5.0, 5.0], [5.0, 6.0, 7.0], [2.0, 8.0, 4.0]], [[7.0, 7.0, 3.0], [8.0, 8.0, 3.0], [2.0, 2.0, 8.0], [2.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 1.0], [7.0, 8.0, 1.0], [6.0, 1.0, 4.0], [5.0, 2.0, 2.0]], [[5.0, 3.0, 7.0], [6.0, 
7.0, 5.0], [5.0, 1.0, 1.0], [1.0, 4.0, 1.0]], [[6.0, 5.0, 4.0], [3.0, 6.0, 1.0], [7.0, 7.0, 4.0], [5.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 8.0], [3.0, 3.0, 8.0], [1.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[1.0, 3.0, 6.0], [1.0, 
8.0, 3.0], [4.0, 7.0, 4.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 5.0], [2.0, 5.0, 8.0], [8.0, 2.0, 5.0], [7.0, 8.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 1.0, 1.0], [5.0, 4.0, 3.0], [7.0, 4.0, 6.0], [1.0, 8.0, 6.0]], [[5.0, 3.0, 4.0], [2.0, 
3.0, 4.0], [3.0, 4.0, 7.0], [8.0, 7.0, 2.0]], [[8.0, 2.0, 5.0], [2.0, 3.0, 5.0], [3.0, 5.0, 4.0], [3.0, 6.0, 8.0]]])
      val=numpy.array([[[1.0, 3.0], [6.0, 5.0]]])
      arg[1:2,1:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 8.0], [3.0, 3.0, 8.0], [1.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[1.0, 3.0, 6.0], [1.0, 
3.0, 3.0], [6.0, 5.0, 4.0], [7.0, 6.0, 5.0]], [[2.0, 1.0, 5.0], [2.0, 5.0, 8.0], [8.0, 2.0, 5.0], [7.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 1.0], [5.0, 4.0, 3.0], [7.0, 4.0, 6.0], [1.0, 8.0, 6.0]], [[5.0, 3.0, 4.0], [1.0, 
3.0, 4.0], [6.0, 5.0, 7.0], [8.0, 7.0, 2.0]], [[8.0, 2.0, 5.0], [2.0, 3.0, 5.0], [3.0, 5.0, 4.0], [3.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 7.0, 4.0], [8.0, 4.0, 6.0], [1.0, 3.0, 2.0], [5.0, 8.0, 7.0]], [[5.0, 2.0, 8.0], [7.0, 
2.0, 6.0], [2.0, 4.0, 3.0], [8.0, 7.0, 2.0]], [[5.0, 3.0, 3.0], [3.0, 3.0, 1.0], [2.0, 7.0, 4.0], [4.0, 7.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 2.0, 2.0], [6.0, 4.0, 1.0], [6.0, 3.0, 2.0], [2.0, 4.0, 1.0]], [[2.0, 5.0, 7.0], [6.0, 
7.0, 2.0], [4.0, 3.0, 4.0], [1.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 1.0, 2.0], [1.0, 5.0, 6.0], [3.0, 7.0, 5.0]]])
      val=Data(numpy.array([[[6.0, 5.0], [2.0, 2.0]]]),self.functionspace)
      arg[1:2,1:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 4.0], [8.0, 4.0, 6.0], [1.0, 3.0, 2.0], [5.0, 8.0, 7.0]], [[5.0, 2.0, 8.0], [6.0, 
5.0, 6.0], [2.0, 2.0, 3.0], [8.0, 7.0, 2.0]], [[5.0, 3.0, 3.0], [3.0, 3.0, 1.0], [2.0, 7.0, 4.0], [4.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 2.0], [6.0, 4.0, 1.0], [6.0, 3.0, 2.0], [2.0, 4.0, 1.0]], [[2.0, 5.0, 7.0], [6.0, 
5.0, 2.0], [2.0, 2.0, 4.0], [1.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 1.0, 2.0], [1.0, 5.0, 6.0], [3.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 3.0, 2.0], [6.0, 5.0, 5.0], [7.0, 4.0, 2.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 2.0], [1.0, 
5.0, 5.0], [1.0, 3.0, 8.0], [6.0, 7.0, 7.0]], [[8.0, 8.0, 3.0], [8.0, 5.0, 6.0], [8.0, 4.0, 3.0], [2.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 6.0], [2.0, 1.0, 2.0], [7.0, 2.0, 8.0], [1.0, 6.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 
7.0, 3.0], [8.0, 5.0, 7.0], [1.0, 5.0, 8.0]], [[4.0, 4.0, 3.0], [2.0, 3.0, 7.0], [7.0, 2.0, 3.0], [5.0, 2.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 3.0], [7.0, 8.0]]])+(1.-msk_val)*numpy.array([[[6.0, 7.0], [1.0, 7.0]]])
      arg[1:2,1:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 2.0], [6.0, 5.0, 5.0], [7.0, 4.0, 2.0], [7.0, 3.0, 4.0]], [[7.0, 7.0, 2.0], [3.0, 
3.0, 5.0], [7.0, 8.0, 8.0], [6.0, 7.0, 7.0]], [[8.0, 8.0, 3.0], [8.0, 5.0, 6.0], [8.0, 4.0, 3.0], [2.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 6.0], [2.0, 1.0, 2.0], [7.0, 2.0, 8.0], [1.0, 6.0, 2.0]], [[1.0, 1.0, 5.0], [6.0, 
7.0, 3.0], [1.0, 7.0, 7.0], [1.0, 5.0, 8.0]], [[4.0, 4.0, 3.0], [2.0, 3.0, 7.0], [7.0, 2.0, 3.0], [5.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 6.0, 8.0], [7.0, 5.0, 4.0], [8.0, 4.0, 2.0], [7.0, 4.0, 1.0]], [[4.0, 7.0, 4.0], [5.0, 
8.0, 1.0], [3.0, 3.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 1.0, 2.0], [2.0, 4.0, 3.0], [3.0, 7.0, 3.0], [5.0, 5.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 7.0, 2.0], [2.0, 1.0, 8.0], [4.0, 6.0, 1.0], [4.0, 7.0, 7.0]], [[4.0, 5.0, 5.0], [8.0, 
3.0, 3.0], [8.0, 2.0, 8.0], [5.0, 2.0, 8.0]], [[4.0, 7.0, 7.0], [1.0, 2.0, 7.0], [5.0, 7.0, 4.0], [3.0, 3.0, 7.0]]])
      val=numpy.array([[7.0, 2.0, 1.0], [8.0, 2.0, 3.0], [7.0, 3.0, 3.0]])
      arg[:,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 8.0], [7.0, 5.0, 4.0], [7.0, 2.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 7.0, 4.0], [5.0, 
8.0, 1.0], [8.0, 2.0, 3.0], [5.0, 1.0, 2.0]], [[6.0, 1.0, 2.0], [2.0, 4.0, 3.0], [7.0, 3.0, 3.0], [5.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 2.0], [2.0, 1.0, 8.0], [7.0, 2.0, 1.0], [4.0, 7.0, 7.0]], [[4.0, 5.0, 5.0], [8.0, 
3.0, 3.0], [8.0, 2.0, 3.0], [5.0, 2.0, 8.0]], [[4.0, 7.0, 7.0], [1.0, 2.0, 7.0], [7.0, 3.0, 3.0], [3.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 5.0, 4.0], [1.0, 4.0, 5.0], [8.0, 7.0, 2.0], [8.0, 6.0, 2.0]], [[2.0, 8.0, 2.0], [4.0, 
8.0, 5.0], [4.0, 4.0, 7.0], [1.0, 3.0, 5.0]], [[4.0, 4.0, 6.0], [2.0, 7.0, 3.0], [7.0, 7.0, 1.0], [3.0, 8.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 8.0, 8.0], [5.0, 2.0, 5.0], [7.0, 8.0, 5.0], [1.0, 1.0, 7.0]], [[2.0, 7.0, 5.0], [2.0, 
5.0, 1.0], [2.0, 4.0, 2.0], [7.0, 5.0, 4.0]], [[6.0, 4.0, 8.0], [4.0, 3.0, 8.0], [5.0, 4.0, 5.0], [2.0, 4.0, 4.0]]])
      val=Data(numpy.array([[5.0, 6.0, 5.0], [3.0, 3.0, 8.0], [6.0, 5.0, 8.0]]),self.functionspace)
      arg[:,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 5.0, 4.0], [1.0, 4.0, 5.0], [5.0, 6.0, 5.0], [8.0, 6.0, 2.0]], [[2.0, 8.0, 2.0], [4.0, 
8.0, 5.0], [3.0, 3.0, 8.0], [1.0, 3.0, 5.0]], [[4.0, 4.0, 6.0], [2.0, 7.0, 3.0], [6.0, 5.0, 8.0], [3.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 8.0, 8.0], [5.0, 2.0, 5.0], [5.0, 6.0, 5.0], [1.0, 1.0, 7.0]], [[2.0, 7.0, 5.0], [2.0, 
5.0, 1.0], [3.0, 3.0, 8.0], [7.0, 5.0, 4.0]], [[6.0, 4.0, 8.0], [4.0, 3.0, 8.0], [6.0, 5.0, 8.0], [2.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 8.0, 1.0], [3.0, 8.0, 5.0], [7.0, 1.0, 5.0], [7.0, 5.0, 1.0]], [[8.0, 2.0, 6.0], [8.0, 
8.0, 6.0], [5.0, 1.0, 2.0], [8.0, 1.0, 3.0]], [[5.0, 6.0, 1.0], [2.0, 4.0, 8.0], [7.0, 3.0, 6.0], [2.0, 8.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 2.0], [3.0, 2.0, 2.0], [3.0, 3.0, 2.0], [3.0, 7.0, 3.0]], [[1.0, 2.0, 8.0], [4.0, 
6.0, 1.0], [3.0, 7.0, 1.0], [1.0, 2.0, 6.0]], [[6.0, 5.0, 1.0], [3.0, 2.0, 6.0], [3.0, 7.0, 7.0], [2.0, 8.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 8.0, 5.0], [8.0, 7.0, 2.0], [6.0, 3.0, 7.0]])+(1.-msk_val)*numpy.array([[6.0, 8.0, 1.0], 
[2.0, 2.0, 2.0], [8.0, 8.0, 5.0]])
      arg[:,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 1.0], [3.0, 8.0, 5.0], [5.0, 8.0, 5.0], [7.0, 5.0, 1.0]], [[8.0, 2.0, 6.0], [8.0, 
8.0, 6.0], [8.0, 7.0, 2.0], [8.0, 1.0, 3.0]], [[5.0, 6.0, 1.0], [2.0, 4.0, 8.0], [6.0, 3.0, 7.0], [2.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 2.0], [3.0, 2.0, 2.0], [6.0, 8.0, 1.0], [3.0, 7.0, 3.0]], [[1.0, 2.0, 8.0], [4.0, 
6.0, 1.0], [2.0, 2.0, 2.0], [1.0, 2.0, 6.0]], [[6.0, 5.0, 1.0], [3.0, 2.0, 6.0], [8.0, 8.0, 5.0], [2.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 4.0], [2.0, 3.0, 4.0], [2.0, 2.0, 2.0], [1.0, 5.0, 3.0]], [[1.0, 6.0, 2.0], [8.0, 
5.0, 4.0], [8.0, 3.0, 5.0], [4.0, 4.0, 4.0]], [[6.0, 8.0, 8.0], [2.0, 8.0, 5.0], [4.0, 5.0, 3.0], [6.0, 1.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 6.0, 4.0], [2.0, 2.0, 6.0], [6.0, 2.0, 1.0], [5.0, 8.0, 6.0]], [[2.0, 1.0, 6.0], [7.0, 
4.0, 7.0], [6.0, 5.0, 2.0], [3.0, 1.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 3.0, 8.0], [2.0, 4.0, 8.0], [7.0, 3.0, 6.0]]])
      val=numpy.array([[[6.0, 4.0, 8.0], [8.0, 4.0, 8.0], [5.0, 5.0, 3.0]], [[1.0, 6.0, 5.0], [4.0, 8.0, 5.0], [3.0, 3.0, 
6.0]]])
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 8.0], [8.0, 4.0, 8.0], [5.0, 5.0, 3.0], [1.0, 5.0, 3.0]], [[1.0, 6.0, 5.0], [4.0, 
8.0, 5.0], [3.0, 3.0, 6.0], [4.0, 4.0, 4.0]], [[6.0, 8.0, 8.0], [2.0, 8.0, 5.0], [4.0, 5.0, 3.0], [6.0, 1.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 8.0], [8.0, 4.0, 8.0], [5.0, 5.0, 3.0], [5.0, 8.0, 6.0]], [[1.0, 6.0, 5.0], [4.0, 
8.0, 5.0], [3.0, 3.0, 6.0], [3.0, 1.0, 5.0]], [[8.0, 2.0, 6.0], [6.0, 3.0, 8.0], [2.0, 4.0, 8.0], [7.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 5.0, 8.0], [2.0, 6.0, 7.0], [7.0, 2.0, 3.0], [7.0, 3.0, 3.0]], [[5.0, 8.0, 4.0], [2.0, 
5.0, 2.0], [1.0, 7.0, 6.0], [3.0, 6.0, 5.0]], [[1.0, 8.0, 7.0], [4.0, 4.0, 7.0], [5.0, 4.0, 8.0], [3.0, 7.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 8.0, 7.0], [4.0, 4.0, 6.0], [3.0, 2.0, 6.0], [8.0, 1.0, 4.0]], [[1.0, 3.0, 8.0], [8.0, 
5.0, 8.0], [8.0, 6.0, 7.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 4.0], [5.0, 6.0, 8.0], [2.0, 2.0, 7.0], [6.0, 5.0, 1.0]]])
      val=Data(numpy.array([[[2.0, 5.0, 4.0], [3.0, 3.0, 4.0], [6.0, 1.0, 5.0]], [[7.0, 5.0, 6.0], [5.0, 7.0, 8.0], [2.0, 4.0, 
4.0]]]),self.functionspace)
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 4.0], [3.0, 3.0, 4.0], [6.0, 1.0, 5.0], [7.0, 3.0, 3.0]], [[7.0, 5.0, 6.0], [5.0, 
7.0, 8.0], [2.0, 4.0, 4.0], [3.0, 6.0, 5.0]], [[1.0, 8.0, 7.0], [4.0, 4.0, 7.0], [5.0, 4.0, 8.0], [3.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 4.0], [3.0, 3.0, 4.0], [6.0, 1.0, 5.0], [8.0, 1.0, 4.0]], [[7.0, 5.0, 6.0], [5.0, 
7.0, 8.0], [2.0, 4.0, 4.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 4.0], [5.0, 6.0, 8.0], [2.0, 2.0, 7.0], [6.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 5.0, 8.0], [7.0, 4.0, 5.0], [6.0, 8.0, 2.0], [4.0, 6.0, 3.0]], [[5.0, 7.0, 6.0], [4.0, 
5.0, 4.0], [4.0, 8.0, 7.0], [6.0, 7.0, 6.0]], [[3.0, 4.0, 8.0], [2.0, 1.0, 1.0], [4.0, 7.0, 7.0], [1.0, 4.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 6.0, 4.0], [1.0, 1.0, 5.0], [7.0, 8.0, 3.0], [7.0, 4.0, 8.0]], [[7.0, 6.0, 6.0], [3.0, 
6.0, 5.0], [4.0, 5.0, 6.0], [2.0, 2.0, 7.0]], [[7.0, 4.0, 5.0], [3.0, 7.0, 2.0], [8.0, 1.0, 5.0], [6.0, 8.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 6.0, 3.0], [6.0, 2.0, 2.0], [3.0, 2.0, 8.0]], [[2.0, 5.0, 2.0], [8.0, 2.0, 5.0], [6.0, 
5.0, 2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 5.0, 1.0], [5.0, 4.0, 8.0], [1.0, 4.0, 4.0]], [[3.0, 5.0, 2.0], [2.0, 3.0, 1.0], 
[6.0, 6.0, 2.0]]])
      arg[:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 3.0], [6.0, 2.0, 2.0], [3.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[2.0, 5.0, 2.0], [8.0, 
2.0, 5.0], [6.0, 5.0, 2.0], [6.0, 7.0, 6.0]], [[3.0, 4.0, 8.0], [2.0, 1.0, 1.0], [4.0, 7.0, 7.0], [1.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 1.0], [5.0, 4.0, 8.0], [1.0, 4.0, 4.0], [7.0, 4.0, 8.0]], [[3.0, 5.0, 2.0], [2.0, 
3.0, 1.0], [6.0, 6.0, 2.0], [2.0, 2.0, 7.0]], [[7.0, 4.0, 5.0], [3.0, 7.0, 2.0], [8.0, 1.0, 5.0], [6.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 3.0, 7.0], [6.0, 7.0, 5.0], [4.0, 5.0, 3.0], [8.0, 1.0, 7.0]], [[8.0, 6.0, 5.0], [6.0, 
4.0, 3.0], [1.0, 7.0, 4.0], [7.0, 7.0, 2.0]], [[2.0, 6.0, 8.0], [2.0, 5.0, 8.0], [8.0, 6.0, 5.0], [3.0, 8.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 5.0], [7.0, 8.0, 4.0], [3.0, 7.0, 3.0], [4.0, 8.0, 6.0]], [[2.0, 8.0, 3.0], [1.0, 
6.0, 6.0], [5.0, 2.0, 8.0], [2.0, 7.0, 7.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 2.0], [2.0, 3.0, 3.0], [8.0, 1.0, 5.0]]])
      val=numpy.array([[[8.0], [2.0], [6.0]], [[7.0], [5.0], [8.0]], [[5.0], [4.0], [6.0]]])
      arg[:,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 8.0], [6.0, 7.0, 2.0], [4.0, 5.0, 6.0], [8.0, 1.0, 7.0]], [[8.0, 6.0, 7.0], [6.0, 
4.0, 5.0], [1.0, 7.0, 8.0], [7.0, 7.0, 2.0]], [[2.0, 6.0, 5.0], [2.0, 5.0, 4.0], [8.0, 6.0, 6.0], [3.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 8.0], [7.0, 8.0, 2.0], [3.0, 7.0, 6.0], [4.0, 8.0, 6.0]], [[2.0, 8.0, 7.0], [1.0, 
6.0, 5.0], [5.0, 2.0, 8.0], [2.0, 7.0, 7.0]], [[7.0, 7.0, 5.0], [3.0, 6.0, 4.0], [2.0, 3.0, 6.0], [8.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 3.0, 8.0], [1.0, 4.0, 3.0], [8.0, 8.0, 2.0], [5.0, 5.0, 7.0]], [[5.0, 3.0, 7.0], [1.0, 
8.0, 5.0], [1.0, 6.0, 6.0], [4.0, 7.0, 6.0]], [[5.0, 1.0, 7.0], [4.0, 1.0, 5.0], [2.0, 8.0, 8.0], [3.0, 6.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 4.0], [2.0, 5.0, 5.0], [2.0, 8.0, 1.0], [8.0, 4.0, 4.0]], [[7.0, 7.0, 7.0], [8.0, 
1.0, 7.0], [8.0, 2.0, 2.0], [5.0, 6.0, 6.0]], [[7.0, 3.0, 4.0], [6.0, 3.0, 3.0], [4.0, 5.0, 8.0], [6.0, 2.0, 7.0]]])
      val=Data(numpy.array([[[8.0], [5.0], [5.0]], [[8.0], [4.0], [5.0]], [[6.0], [5.0], [1.0]]]),self.functionspace)
      arg[:,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 8.0], [1.0, 4.0, 5.0], [8.0, 8.0, 5.0], [5.0, 5.0, 7.0]], [[5.0, 3.0, 8.0], [1.0, 
8.0, 4.0], [1.0, 6.0, 5.0], [4.0, 7.0, 6.0]], [[5.0, 1.0, 6.0], [4.0, 1.0, 5.0], [2.0, 8.0, 1.0], [3.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 8.0], [2.0, 5.0, 5.0], [2.0, 8.0, 5.0], [8.0, 4.0, 4.0]], [[7.0, 7.0, 8.0], [8.0, 
1.0, 4.0], [8.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[7.0, 3.0, 6.0], [6.0, 3.0, 5.0], [4.0, 5.0, 1.0], [6.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 2.0, 8.0], [7.0, 3.0, 2.0], [5.0, 4.0, 8.0], [8.0, 8.0, 8.0]], [[4.0, 5.0, 1.0], [4.0, 
4.0, 2.0], [8.0, 8.0, 2.0], [5.0, 1.0, 3.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 4.0], [8.0, 8.0, 2.0], [8.0, 2.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 5.0, 7.0], [1.0, 6.0, 8.0], [3.0, 6.0, 3.0], [5.0, 3.0, 3.0]], [[8.0, 3.0, 6.0], [7.0, 
2.0, 6.0], [4.0, 6.0, 3.0], [6.0, 1.0, 7.0]], [[8.0, 7.0, 6.0], [6.0, 6.0, 1.0], [2.0, 5.0, 2.0], [8.0, 7.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0], [5.0], [7.0]], [[1.0], [7.0], [3.0]], [[1.0], [6.0], 
[3.0]]])+(1.-msk_val)*numpy.array([[[1.0], [1.0], [7.0]], [[1.0], [2.0], [8.0]], [[5.0], [1.0], [6.0]]])
      arg[:,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 5.0], [7.0, 3.0, 5.0], [5.0, 4.0, 7.0], [8.0, 8.0, 8.0]], [[4.0, 5.0, 1.0], [4.0, 
4.0, 7.0], [8.0, 8.0, 3.0], [5.0, 1.0, 3.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 6.0], [8.0, 8.0, 3.0], [8.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 1.0], [1.0, 6.0, 1.0], [3.0, 6.0, 7.0], [5.0, 3.0, 3.0]], [[8.0, 3.0, 1.0], [7.0, 
2.0, 2.0], [4.0, 6.0, 8.0], [6.0, 1.0, 7.0]], [[8.0, 7.0, 5.0], [6.0, 6.0, 1.0], [2.0, 5.0, 6.0], [8.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 3.0, 6.0], [5.0, 1.0, 4.0], [3.0, 2.0, 1.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 2.0], [7.0, 
3.0, 7.0], [7.0, 3.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 5.0, 5.0], [6.0, 3.0, 4.0], [4.0, 5.0, 2.0], [6.0, 7.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 5.0, 5.0], [7.0, 2.0, 5.0], [2.0, 7.0, 2.0], [2.0, 5.0, 3.0]], [[3.0, 7.0, 1.0], [4.0, 
4.0, 1.0], [2.0, 2.0, 2.0], [6.0, 8.0, 6.0]], [[6.0, 7.0, 8.0], [5.0, 4.0, 5.0], [6.0, 3.0, 6.0], [2.0, 4.0, 1.0]]])
      val=numpy.array([[1.0, 6.0, 5.0, 4.0]])
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 6.0], [5.0, 1.0, 4.0], [3.0, 2.0, 1.0], [4.0, 7.0, 1.0]], [[6.0, 8.0, 2.0], [7.0, 
3.0, 7.0], [7.0, 3.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 1.0, 5.0], [6.0, 6.0, 4.0], [4.0, 5.0, 2.0], [6.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 5.0], [7.0, 2.0, 5.0], [2.0, 7.0, 2.0], [2.0, 5.0, 3.0]], [[3.0, 7.0, 1.0], [4.0, 
4.0, 1.0], [2.0, 2.0, 2.0], [6.0, 8.0, 6.0]], [[6.0, 1.0, 8.0], [5.0, 6.0, 5.0], [6.0, 5.0, 6.0], [2.0, 4.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 3.0, 1.0], [1.0, 1.0, 7.0], [7.0, 2.0, 1.0], [8.0, 5.0, 5.0]], [[8.0, 8.0, 4.0], [2.0, 
2.0, 6.0], [4.0, 8.0, 7.0], [7.0, 5.0, 5.0]], [[1.0, 2.0, 4.0], [8.0, 6.0, 6.0], [8.0, 3.0, 3.0], [3.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 5.0], [8.0, 3.0, 1.0], [2.0, 8.0, 2.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 4.0], [2.0, 
7.0, 8.0], [7.0, 2.0, 7.0], [1.0, 2.0, 5.0]], [[8.0, 4.0, 3.0], [3.0, 8.0, 1.0], [5.0, 1.0, 5.0], [4.0, 4.0, 8.0]]])
      val=Data(numpy.array([[8.0, 8.0, 4.0, 8.0]]),self.functionspace)
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 3.0, 1.0], [1.0, 1.0, 7.0], [7.0, 2.0, 1.0], [8.0, 5.0, 5.0]], [[8.0, 8.0, 4.0], [2.0, 
2.0, 6.0], [4.0, 8.0, 7.0], [7.0, 5.0, 5.0]], [[1.0, 8.0, 4.0], [8.0, 8.0, 6.0], [8.0, 4.0, 3.0], [3.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 5.0], [8.0, 3.0, 1.0], [2.0, 8.0, 2.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 4.0], [2.0, 
7.0, 8.0], [7.0, 2.0, 7.0], [1.0, 2.0, 5.0]], [[8.0, 8.0, 3.0], [3.0, 8.0, 1.0], [5.0, 4.0, 5.0], [4.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 4.0], [8.0, 7.0, 8.0], [1.0, 7.0, 5.0], [2.0, 1.0, 5.0]], [[5.0, 3.0, 8.0], [3.0, 
8.0, 7.0], [5.0, 2.0, 4.0], [2.0, 6.0, 4.0]], [[2.0, 2.0, 6.0], [6.0, 3.0, 3.0], [5.0, 6.0, 8.0], [5.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 7.0, 2.0], [5.0, 4.0, 4.0], [6.0, 7.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 7.0], [7.0, 
2.0, 2.0], [1.0, 7.0, 8.0], [2.0, 4.0, 6.0]], [[1.0, 2.0, 4.0], [5.0, 5.0, 4.0], [2.0, 1.0, 3.0], [3.0, 6.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 7.0, 4.0, 3.0]])+(1.-msk_val)*numpy.array([[8.0, 6.0, 2.0, 1.0]])
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 4.0], [8.0, 7.0, 8.0], [1.0, 7.0, 5.0], [2.0, 1.0, 5.0]], [[5.0, 3.0, 8.0], [3.0, 
8.0, 7.0], [5.0, 2.0, 4.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 6.0], [6.0, 7.0, 3.0], [5.0, 4.0, 8.0], [5.0, 3.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 2.0], [5.0, 4.0, 4.0], [6.0, 7.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 7.0], [7.0, 
2.0, 2.0], [1.0, 7.0, 8.0], [2.0, 4.0, 6.0]], [[1.0, 8.0, 4.0], [5.0, 6.0, 4.0], [2.0, 2.0, 3.0], [3.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 2.0], [2.0, 4.0, 3.0], [3.0, 1.0, 3.0], [5.0, 1.0, 6.0]], [[8.0, 5.0, 8.0], [3.0, 
7.0, 7.0], [7.0, 8.0, 6.0], [7.0, 4.0, 3.0]], [[1.0, 2.0, 1.0], [5.0, 7.0, 6.0], [6.0, 5.0, 1.0], [5.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 1.0, 8.0], [7.0, 5.0, 1.0], [3.0, 1.0, 5.0], [7.0, 5.0, 4.0]], [[5.0, 1.0, 7.0], [6.0, 
2.0, 4.0], [4.0, 6.0, 4.0], [5.0, 7.0, 4.0]], [[2.0, 7.0, 7.0], [8.0, 4.0, 6.0], [7.0, 7.0, 3.0], [1.0, 2.0, 6.0]]])
      val=numpy.array([[7.0, 3.0, 1.0], [4.0, 1.0, 2.0]])
      arg[1,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 2.0], [2.0, 4.0, 3.0], [3.0, 1.0, 3.0], [5.0, 1.0, 6.0]], [[8.0, 5.0, 8.0], [3.0, 
7.0, 7.0], [7.0, 3.0, 1.0], [4.0, 1.0, 2.0]], [[1.0, 2.0, 1.0], [5.0, 7.0, 6.0], [6.0, 5.0, 1.0], [5.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 8.0], [7.0, 5.0, 1.0], [3.0, 1.0, 5.0], [7.0, 5.0, 4.0]], [[5.0, 1.0, 7.0], [6.0, 
2.0, 4.0], [7.0, 3.0, 1.0], [4.0, 1.0, 2.0]], [[2.0, 7.0, 7.0], [8.0, 4.0, 6.0], [7.0, 7.0, 3.0], [1.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 5.0, 3.0], [8.0, 8.0, 2.0], [6.0, 1.0, 8.0], [4.0, 5.0, 3.0]], [[2.0, 7.0, 4.0], [1.0, 
6.0, 3.0], [5.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[6.0, 2.0, 5.0], [5.0, 2.0, 1.0], [5.0, 4.0, 4.0], [7.0, 8.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 5.0, 8.0], [4.0, 3.0, 3.0], [8.0, 6.0, 1.0], [1.0, 8.0, 1.0]], [[2.0, 6.0, 2.0], [4.0, 
2.0, 8.0], [8.0, 4.0, 2.0], [1.0, 6.0, 8.0]], [[1.0, 6.0, 7.0], [5.0, 1.0, 4.0], [1.0, 5.0, 2.0], [2.0, 8.0, 1.0]]])
      val=Data(numpy.array([[7.0, 1.0, 5.0], [7.0, 2.0, 8.0]]),self.functionspace)
      arg[1,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 3.0], [8.0, 8.0, 2.0], [6.0, 1.0, 8.0], [4.0, 5.0, 3.0]], [[2.0, 7.0, 4.0], [1.0, 
6.0, 3.0], [7.0, 1.0, 5.0], [7.0, 2.0, 8.0]], [[6.0, 2.0, 5.0], [5.0, 2.0, 1.0], [5.0, 4.0, 4.0], [7.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 5.0, 8.0], [4.0, 3.0, 3.0], [8.0, 6.0, 1.0], [1.0, 8.0, 1.0]], [[2.0, 6.0, 2.0], [4.0, 
2.0, 8.0], [7.0, 1.0, 5.0], [7.0, 2.0, 8.0]], [[1.0, 6.0, 7.0], [5.0, 1.0, 4.0], [1.0, 5.0, 2.0], [2.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 3.0, 5.0], [2.0, 1.0, 3.0], [7.0, 5.0, 1.0], [4.0, 8.0, 4.0]], [[6.0, 2.0, 4.0], [3.0, 
6.0, 7.0], [8.0, 4.0, 8.0], [2.0, 8.0, 6.0]], [[6.0, 1.0, 6.0], [2.0, 2.0, 1.0], [5.0, 3.0, 7.0], [4.0, 5.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 6.0, 4.0], [7.0, 4.0, 8.0], [4.0, 1.0, 6.0], [5.0, 7.0, 7.0]], [[3.0, 4.0, 8.0], [2.0, 
8.0, 7.0], [8.0, 3.0, 7.0], [8.0, 6.0, 3.0]], [[7.0, 2.0, 5.0], [1.0, 6.0, 6.0], [6.0, 1.0, 7.0], [4.0, 6.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 6.0, 7.0], [1.0, 5.0, 5.0]])+(1.-msk_val)*numpy.array([[5.0, 4.0, 7.0], [1.0, 5.0, 1.0]])
      arg[1,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 5.0], [2.0, 1.0, 3.0], [7.0, 5.0, 1.0], [4.0, 8.0, 4.0]], [[6.0, 2.0, 4.0], [3.0, 
6.0, 7.0], [3.0, 6.0, 7.0], [1.0, 5.0, 5.0]], [[6.0, 1.0, 6.0], [2.0, 2.0, 1.0], [5.0, 3.0, 7.0], [4.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 4.0], [7.0, 4.0, 8.0], [4.0, 1.0, 6.0], [5.0, 7.0, 7.0]], [[3.0, 4.0, 8.0], [2.0, 
8.0, 7.0], [5.0, 4.0, 7.0], [1.0, 5.0, 1.0]], [[7.0, 2.0, 5.0], [1.0, 6.0, 6.0], [6.0, 1.0, 7.0], [4.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 2.0, 8.0], [2.0, 6.0, 5.0], [2.0, 6.0, 6.0], [1.0, 8.0, 5.0]], [[1.0, 8.0, 8.0], [6.0, 
6.0, 4.0], [8.0, 4.0, 2.0], [6.0, 6.0, 6.0]], [[6.0, 2.0, 8.0], [3.0, 2.0, 8.0], [2.0, 6.0, 1.0], [6.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 4.0, 3.0], [7.0, 1.0, 3.0], [7.0, 6.0, 5.0], [6.0, 4.0, 8.0]], [[1.0, 7.0, 2.0], [5.0, 
3.0, 3.0], [8.0, 4.0, 3.0], [5.0, 1.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 2.0, 3.0], [1.0, 3.0, 5.0], [7.0, 8.0, 3.0]]])
      val=numpy.array([[[8.0], [6.0], [1.0], [8.0]], [[3.0], [8.0], [8.0], [6.0]]])
      arg[:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 8.0], [2.0, 6.0, 5.0], [2.0, 1.0, 6.0], [1.0, 8.0, 5.0]], [[1.0, 3.0, 8.0], [6.0, 
8.0, 4.0], [8.0, 8.0, 2.0], [6.0, 6.0, 6.0]], [[6.0, 2.0, 8.0], [3.0, 2.0, 8.0], [2.0, 6.0, 1.0], [6.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 3.0], [7.0, 6.0, 3.0], [7.0, 1.0, 5.0], [6.0, 8.0, 8.0]], [[1.0, 3.0, 2.0], [5.0, 
8.0, 3.0], [8.0, 8.0, 3.0], [5.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [6.0, 2.0, 3.0], [1.0, 3.0, 5.0], [7.0, 8.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 6.0, 3.0], [5.0, 7.0, 8.0], [5.0, 6.0, 4.0], [4.0, 8.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 
7.0, 2.0], [7.0, 2.0, 5.0], [2.0, 1.0, 1.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 4.0], [8.0, 8.0, 5.0], [1.0, 3.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 4.0, 1.0], [4.0, 1.0, 2.0], [2.0, 8.0, 3.0], [7.0, 6.0, 2.0]], [[2.0, 1.0, 6.0], [2.0, 
1.0, 8.0], [8.0, 1.0, 3.0], [4.0, 5.0, 2.0]], [[5.0, 7.0, 6.0], [7.0, 4.0, 6.0], [1.0, 6.0, 7.0], [6.0, 6.0, 4.0]]])
      val=Data(numpy.array([[[4.0], [1.0], [4.0], [2.0]], [[7.0], [5.0], [4.0], [1.0]]]),self.functionspace)
      arg[:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 3.0], [5.0, 1.0, 8.0], [5.0, 4.0, 4.0], [4.0, 2.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 
5.0, 2.0], [7.0, 4.0, 5.0], [2.0, 1.0, 1.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 4.0], [8.0, 8.0, 5.0], [1.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 1.0], [4.0, 1.0, 2.0], [2.0, 4.0, 3.0], [7.0, 2.0, 2.0]], [[2.0, 7.0, 6.0], [2.0, 
5.0, 8.0], [8.0, 4.0, 3.0], [4.0, 1.0, 2.0]], [[5.0, 7.0, 6.0], [7.0, 4.0, 6.0], [1.0, 6.0, 7.0], [6.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 4.0], [1.0, 8.0, 5.0], [7.0, 2.0, 4.0], [6.0, 1.0, 8.0]], [[7.0, 7.0, 5.0], [5.0, 
4.0, 6.0], [4.0, 5.0, 7.0], [7.0, 4.0, 2.0]], [[1.0, 7.0, 8.0], [6.0, 8.0, 8.0], [8.0, 6.0, 1.0], [1.0, 1.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 4.0], [5.0, 8.0, 5.0], [4.0, 1.0, 7.0], [4.0, 1.0, 3.0]], [[4.0, 8.0, 6.0], [2.0, 
6.0, 4.0], [8.0, 4.0, 4.0], [4.0, 5.0, 1.0]], [[7.0, 6.0, 5.0], [8.0, 4.0, 2.0], [5.0, 3.0, 3.0], [2.0, 1.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0], [6.0], [6.0], [8.0]], [[4.0], [1.0], [5.0], [1.0]]])+(1.-msk_val)*numpy.array([[[4.0], 
[3.0], [5.0], [6.0]], [[2.0], [2.0], [4.0], [6.0]]])
      arg[:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 4.0], [1.0, 6.0, 5.0], [7.0, 6.0, 4.0], [6.0, 8.0, 8.0]], [[7.0, 4.0, 5.0], [5.0, 
1.0, 6.0], [4.0, 5.0, 7.0], [7.0, 1.0, 2.0]], [[1.0, 7.0, 8.0], [6.0, 8.0, 8.0], [8.0, 6.0, 1.0], [1.0, 1.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 4.0], [5.0, 3.0, 5.0], [4.0, 5.0, 7.0], [4.0, 6.0, 3.0]], [[4.0, 2.0, 6.0], [2.0, 
2.0, 4.0], [8.0, 4.0, 4.0], [4.0, 6.0, 1.0]], [[7.0, 6.0, 5.0], [8.0, 4.0, 2.0], [5.0, 3.0, 3.0], [2.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_URN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 2.0], [3.0, 8.0, 7.0], [5.0, 8.0, 8.0], [5.0, 1.0, 6.0]], [[3.0, 7.0, 4.0], [3.0, 
8.0, 7.0], [1.0, 1.0, 4.0], [6.0, 6.0, 5.0]], [[5.0, 7.0, 1.0], [3.0, 2.0, 5.0], [2.0, 7.0, 5.0], [3.0, 3.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 7.0, 3.0], [1.0, 6.0, 1.0], [3.0, 8.0, 8.0], [6.0, 7.0, 1.0]], [[6.0, 4.0, 2.0], [8.0, 
1.0, 2.0], [7.0, 2.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 6.0, 2.0], [2.0, 6.0, 8.0], [1.0, 3.0, 2.0], [5.0, 8.0, 4.0]]])
      val=numpy.array([[[1.0, 4.0, 4.0], [6.0, 1.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 2.0, 2.0]]])
      arg[:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 2.0], [1.0, 4.0, 4.0], [6.0, 1.0, 8.0], [5.0, 1.0, 6.0]], [[3.0, 7.0, 4.0], [6.0, 
2.0, 1.0], [5.0, 2.0, 2.0], [6.0, 6.0, 5.0]], [[5.0, 7.0, 1.0], [3.0, 2.0, 5.0], [2.0, 7.0, 5.0], [3.0, 3.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 7.0, 3.0], [1.0, 4.0, 4.0], [6.0, 1.0, 8.0], [6.0, 7.0, 1.0]], [[6.0, 4.0, 2.0], [6.0, 
2.0, 1.0], [5.0, 2.0, 2.0], [2.0, 1.0, 3.0]], [[1.0, 6.0, 2.0], [2.0, 6.0, 8.0], [1.0, 3.0, 2.0], [5.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_URN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 1.0, 7.0], [2.0, 8.0, 1.0], [5.0, 4.0, 8.0], [5.0, 2.0, 2.0]], [[2.0, 5.0, 6.0], [7.0, 
1.0, 7.0], [6.0, 2.0, 2.0], [5.0, 3.0, 5.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 2.0, 1.0], [2.0, 2.0, 8.0], [6.0, 6.0, 1.0], [4.0, 2.0, 5.0]], [[8.0, 4.0, 2.0], [7.0, 
5.0, 6.0], [2.0, 1.0, 3.0], [6.0, 6.0, 7.0]], [[5.0, 2.0, 2.0], [8.0, 7.0, 3.0], [7.0, 3.0, 3.0], [5.0, 3.0, 8.0]]])
      val=Data(numpy.array([[[7.0, 6.0, 8.0], [8.0, 6.0, 1.0]], [[6.0, 2.0, 6.0], [3.0, 3.0, 1.0]]]),self.functionspace)
      arg[:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 7.0], [7.0, 6.0, 8.0], [8.0, 6.0, 1.0], [5.0, 2.0, 2.0]], [[2.0, 5.0, 6.0], [6.0, 
2.0, 6.0], [3.0, 3.0, 1.0], [5.0, 3.0, 5.0]], [[8.0, 1.0, 6.0], [1.0, 8.0, 8.0], [6.0, 2.0, 1.0], [2.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 1.0], [7.0, 6.0, 8.0], [8.0, 6.0, 1.0], [4.0, 2.0, 5.0]], [[8.0, 4.0, 2.0], [6.0, 
2.0, 6.0], [3.0, 3.0, 1.0], [6.0, 6.0, 7.0]], [[5.0, 2.0, 2.0], [8.0, 7.0, 3.0], [7.0, 3.0, 3.0], [5.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_URN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 4.0, 8.0], [4.0, 2.0, 5.0], [4.0, 5.0, 2.0], [2.0, 5.0, 2.0]], [[3.0, 1.0, 3.0], [8.0, 
1.0, 2.0], [6.0, 1.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [6.0, 2.0, 6.0], [3.0, 8.0, 2.0], [8.0, 5.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 3.0], [4.0, 1.0, 1.0], [4.0, 8.0, 5.0], [8.0, 8.0, 3.0]], [[7.0, 8.0, 2.0], [1.0, 
2.0, 8.0], [2.0, 2.0, 4.0], [6.0, 8.0, 4.0]], [[8.0, 1.0, 3.0], [4.0, 3.0, 3.0], [6.0, 7.0, 3.0], [4.0, 2.0, 2.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 3.0, 4.0], [6.0, 6.0, 2.0]], [[1.0, 5.0, 5.0], [2.0, 8.0, 
5.0]]])+(1.-msk_val)*numpy.array([[[1.0, 2.0, 6.0], [7.0, 1.0, 4.0]], [[2.0, 3.0, 1.0], [3.0, 6.0, 1.0]]])
      arg[:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 4.0, 8.0], [2.0, 3.0, 4.0], [6.0, 6.0, 2.0], [2.0, 5.0, 2.0]], [[3.0, 1.0, 3.0], [1.0, 
5.0, 5.0], [2.0, 8.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [6.0, 2.0, 6.0], [3.0, 8.0, 2.0], [8.0, 5.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 3.0], [1.0, 2.0, 6.0], [7.0, 1.0, 4.0], [8.0, 8.0, 3.0]], [[7.0, 8.0, 2.0], [2.0, 
3.0, 1.0], [3.0, 6.0, 1.0], [6.0, 8.0, 4.0]], [[8.0, 1.0, 3.0], [4.0, 3.0, 3.0], [6.0, 7.0, 3.0], [4.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 5.0, 4.0], [6.0, 5.0, 5.0], [4.0, 6.0, 4.0], [2.0, 7.0, 1.0]], [[5.0, 8.0, 7.0], [3.0, 
8.0, 8.0], [8.0, 5.0, 1.0], [7.0, 5.0, 5.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 1.0], [5.0, 6.0, 8.0], [2.0, 1.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 1.0, 1.0], [7.0, 7.0, 7.0], [4.0, 6.0, 2.0], [4.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [5.0, 
2.0, 6.0], [4.0, 1.0, 1.0], [1.0, 6.0, 4.0]], [[7.0, 6.0, 2.0], [4.0, 5.0, 2.0], [3.0, 7.0, 3.0], [7.0, 2.0, 8.0]]])
      val=numpy.array([[3.0]])
      arg[1:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 4.0], [6.0, 5.0, 5.0], [4.0, 6.0, 4.0], [2.0, 7.0, 1.0]], [[5.0, 8.0, 7.0], [3.0, 
8.0, 8.0], [8.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 1.0], [5.0, 6.0, 8.0], [2.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 1.0, 1.0], [7.0, 7.0, 7.0], [4.0, 6.0, 2.0], [4.0, 5.0, 7.0]], [[4.0, 4.0, 2.0], [5.0, 
2.0, 6.0], [4.0, 3.0, 1.0], [1.0, 6.0, 4.0]], [[7.0, 6.0, 2.0], [4.0, 5.0, 2.0], [3.0, 7.0, 3.0], [7.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 5.0, 3.0], [1.0, 6.0, 2.0], [6.0, 3.0, 5.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 7.0], [6.0, 
5.0, 8.0], [6.0, 5.0, 3.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 4.0], [7.0, 2.0, 3.0], [5.0, 2.0, 4.0], [1.0, 8.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 7.0, 6.0], [1.0, 3.0, 8.0], [6.0, 7.0, 3.0], [1.0, 4.0, 2.0]], [[8.0, 1.0, 8.0], [2.0, 
4.0, 2.0], [8.0, 6.0, 4.0], [6.0, 3.0, 4.0]], [[3.0, 4.0, 2.0], [6.0, 4.0, 3.0], [1.0, 8.0, 5.0], [3.0, 3.0, 3.0]]])
      val=Data(numpy.array([[4.0]]),self.functionspace)
      arg[1:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 3.0], [1.0, 6.0, 2.0], [6.0, 3.0, 5.0], [8.0, 8.0, 8.0]], [[4.0, 8.0, 7.0], [6.0, 
5.0, 8.0], [6.0, 4.0, 3.0], [7.0, 2.0, 1.0]], [[5.0, 4.0, 4.0], [7.0, 2.0, 3.0], [5.0, 2.0, 4.0], [1.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 6.0], [1.0, 3.0, 8.0], [6.0, 7.0, 3.0], [1.0, 4.0, 2.0]], [[8.0, 1.0, 8.0], [2.0, 
4.0, 2.0], [8.0, 4.0, 4.0], [6.0, 3.0, 4.0]], [[3.0, 4.0, 2.0], [6.0, 4.0, 3.0], [1.0, 8.0, 5.0], [3.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 1.0], [1.0, 2.0, 4.0], [7.0, 6.0, 1.0], [5.0, 6.0, 2.0]], [[1.0, 7.0, 5.0], [1.0, 
3.0, 8.0], [5.0, 6.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 7.0, 7.0], [4.0, 4.0, 5.0], [6.0, 2.0, 2.0], [2.0, 8.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 7.0, 3.0], [8.0, 4.0, 3.0], [8.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [4.0, 
6.0, 1.0], [5.0, 1.0, 4.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 3.0], [3.0, 3.0, 3.0], [7.0, 2.0, 2.0], [2.0, 3.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0]])+(1.-msk_val)*numpy.array([[5.0]])
      arg[1:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 1.0], [1.0, 2.0, 4.0], [7.0, 6.0, 1.0], [5.0, 6.0, 2.0]], [[1.0, 7.0, 5.0], [1.0, 
3.0, 8.0], [5.0, 7.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 7.0, 7.0], [4.0, 4.0, 5.0], [6.0, 2.0, 2.0], [2.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 7.0, 3.0], [8.0, 4.0, 3.0], [8.0, 7.0, 7.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [4.0, 
6.0, 1.0], [5.0, 5.0, 4.0], [2.0, 4.0, 4.0]], [[3.0, 2.0, 3.0], [3.0, 3.0, 3.0], [7.0, 2.0, 2.0], [2.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 6.0, 2.0], [3.0, 8.0, 6.0], [1.0, 2.0, 2.0], [8.0, 7.0, 5.0]], [[8.0, 6.0, 4.0], [4.0, 
4.0, 4.0], [1.0, 1.0, 8.0], [1.0, 6.0, 8.0]], [[4.0, 3.0, 6.0], [5.0, 5.0, 3.0], [8.0, 1.0, 7.0], [3.0, 8.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 2.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 5.0], [5.0, 7.0, 8.0]], [[7.0, 3.0, 4.0], [7.0, 
7.0, 3.0], [4.0, 6.0, 1.0], [8.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [5.0, 2.0, 3.0], [8.0, 2.0, 7.0], [5.0, 1.0, 8.0]]])
      val=numpy.array([[1.0, 3.0], [6.0, 7.0], [6.0, 6.0]])
      arg[1,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 2.0], [3.0, 8.0, 6.0], [1.0, 2.0, 2.0], [8.0, 7.0, 5.0]], [[1.0, 3.0, 4.0], [6.0, 
7.0, 4.0], [6.0, 6.0, 8.0], [1.0, 6.0, 8.0]], [[4.0, 3.0, 6.0], [5.0, 5.0, 3.0], [8.0, 1.0, 7.0], [3.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 2.0, 1.0], [7.0, 1.0, 5.0], [6.0, 2.0, 5.0], [5.0, 7.0, 8.0]], [[1.0, 3.0, 4.0], [6.0, 
7.0, 3.0], [6.0, 6.0, 1.0], [8.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [5.0, 2.0, 3.0], [8.0, 2.0, 7.0], [5.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 2.0, 7.0], [1.0, 5.0, 2.0], [8.0, 3.0, 4.0], [7.0, 7.0, 7.0]], [[6.0, 6.0, 5.0], [5.0, 
8.0, 6.0], [2.0, 1.0, 6.0], [2.0, 2.0, 1.0]], [[2.0, 4.0, 8.0], [4.0, 7.0, 5.0], [5.0, 7.0, 2.0], [3.0, 2.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 1.0, 1.0], [1.0, 4.0, 2.0]], [[4.0, 7.0, 1.0], [1.0, 
2.0, 4.0], [8.0, 8.0, 4.0], [4.0, 4.0, 1.0]], [[5.0, 7.0, 3.0], [4.0, 3.0, 1.0], [1.0, 3.0, 3.0], [7.0, 4.0, 6.0]]])
      val=Data(numpy.array([[1.0, 2.0], [8.0, 6.0], [7.0, 4.0]]),self.functionspace)
      arg[1,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 7.0], [1.0, 5.0, 2.0], [8.0, 3.0, 4.0], [7.0, 7.0, 7.0]], [[1.0, 2.0, 5.0], [8.0, 
6.0, 6.0], [7.0, 4.0, 6.0], [2.0, 2.0, 1.0]], [[2.0, 4.0, 8.0], [4.0, 7.0, 5.0], [5.0, 7.0, 2.0], [3.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 5.0], [3.0, 7.0, 2.0], [6.0, 1.0, 1.0], [1.0, 4.0, 2.0]], [[1.0, 2.0, 1.0], [8.0, 
6.0, 4.0], [7.0, 4.0, 4.0], [4.0, 4.0, 1.0]], [[5.0, 7.0, 3.0], [4.0, 3.0, 1.0], [1.0, 3.0, 3.0], [7.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 1.0], [6.0, 7.0, 2.0], [1.0, 3.0, 5.0], [8.0, 2.0, 2.0]], [[3.0, 1.0, 3.0], [4.0, 
6.0, 6.0], [3.0, 3.0, 5.0], [5.0, 2.0, 8.0]], [[6.0, 1.0, 7.0], [1.0, 2.0, 1.0], [1.0, 1.0, 6.0], [8.0, 6.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 4.0], [4.0, 8.0, 3.0], [5.0, 3.0, 6.0], [5.0, 8.0, 3.0]], [[1.0, 4.0, 6.0], [7.0, 
3.0, 6.0], [5.0, 5.0, 4.0], [5.0, 4.0, 8.0]], [[8.0, 1.0, 8.0], [2.0, 2.0, 4.0], [7.0, 2.0, 4.0], [4.0, 2.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 1.0], [5.0, 6.0], [2.0, 2.0]])+(1.-msk_val)*numpy.array([[6.0, 7.0], [2.0, 4.0], [6.0, 
3.0]])
      arg[1,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 1.0], [6.0, 7.0, 2.0], [1.0, 3.0, 5.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 3.0], [5.0, 
6.0, 6.0], [2.0, 2.0, 5.0], [5.0, 2.0, 8.0]], [[6.0, 1.0, 7.0], [1.0, 2.0, 1.0], [1.0, 1.0, 6.0], [8.0, 6.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 4.0], [4.0, 8.0, 3.0], [5.0, 3.0, 6.0], [5.0, 8.0, 3.0]], [[6.0, 7.0, 6.0], [2.0, 
4.0, 6.0], [6.0, 3.0, 4.0], [5.0, 4.0, 8.0]], [[8.0, 1.0, 8.0], [2.0, 2.0, 4.0], [7.0, 2.0, 4.0], [4.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 5.0, 2.0], [2.0, 8.0, 3.0], [3.0, 5.0, 8.0], [3.0, 3.0, 2.0]], [[5.0, 4.0, 5.0], [8.0, 
4.0, 8.0], [6.0, 6.0, 8.0], [6.0, 1.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 5.0, 6.0], [6.0, 8.0, 1.0], [5.0, 4.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 6.0], [3.0, 3.0, 3.0], [5.0, 4.0, 6.0], [7.0, 6.0, 4.0]], [[6.0, 2.0, 6.0], [4.0, 
5.0, 1.0], [7.0, 5.0, 6.0], [3.0, 2.0, 1.0]], [[4.0, 1.0, 6.0], [2.0, 5.0, 4.0], [5.0, 3.0, 8.0], [6.0, 8.0, 6.0]]])
      val=numpy.array([[[5.0, 1.0, 7.0], [3.0, 7.0, 6.0], [1.0, 1.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 5.0, 5.0], [5.0, 4.0, 
2.0]], [[3.0, 3.0, 1.0], [1.0, 5.0, 2.0], [8.0, 7.0, 6.0]]])
      arg[:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 7.0], [3.0, 7.0, 6.0], [1.0, 1.0, 6.0], [3.0, 3.0, 2.0]], [[7.0, 3.0, 2.0], [8.0, 
5.0, 5.0], [5.0, 4.0, 2.0], [6.0, 1.0, 5.0]], [[3.0, 3.0, 1.0], [1.0, 5.0, 2.0], [8.0, 7.0, 6.0], [5.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 7.0], [3.0, 7.0, 6.0], [1.0, 1.0, 6.0], [7.0, 6.0, 4.0]], [[7.0, 3.0, 2.0], [8.0, 
5.0, 5.0], [5.0, 4.0, 2.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 1.0], [1.0, 5.0, 2.0], [8.0, 7.0, 6.0], [6.0, 8.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 7.0, 1.0], [2.0, 6.0, 3.0], [3.0, 7.0, 8.0], [1.0, 1.0, 4.0]], [[2.0, 6.0, 7.0], [5.0, 
5.0, 8.0], [7.0, 7.0, 3.0], [1.0, 2.0, 2.0]], [[1.0, 5.0, 5.0], [1.0, 5.0, 3.0], [1.0, 1.0, 6.0], [3.0, 3.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 4.0], [7.0, 1.0, 2.0], [4.0, 3.0, 8.0], [8.0, 4.0, 7.0]], [[8.0, 3.0, 4.0], [5.0, 
6.0, 7.0], [3.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[5.0, 1.0, 4.0], [4.0, 2.0, 6.0], [3.0, 2.0, 3.0], [7.0, 3.0, 3.0]]])
      val=Data(numpy.array([[[7.0, 2.0, 7.0], [5.0, 6.0, 3.0], [1.0, 7.0, 8.0]], [[8.0, 4.0, 1.0], [4.0, 7.0, 3.0], [1.0, 6.0, 
2.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 5.0], [7.0, 2.0, 6.0]]]),self.functionspace)
      arg[:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 7.0], [5.0, 6.0, 3.0], [1.0, 7.0, 8.0], [1.0, 1.0, 4.0]], [[8.0, 4.0, 1.0], [4.0, 
7.0, 3.0], [1.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 5.0], [7.0, 2.0, 6.0], [3.0, 3.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 7.0], [5.0, 6.0, 3.0], [1.0, 7.0, 8.0], [8.0, 4.0, 7.0]], [[8.0, 4.0, 1.0], [4.0, 
7.0, 3.0], [1.0, 6.0, 2.0], [5.0, 2.0, 4.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 5.0], [7.0, 2.0, 6.0], [7.0, 3.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 4.0, 4.0], [3.0, 8.0, 3.0], [8.0, 8.0, 2.0], [7.0, 8.0, 1.0]], [[8.0, 3.0, 2.0], [1.0, 
2.0, 5.0], [2.0, 3.0, 2.0], [3.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [5.0, 6.0, 1.0], [7.0, 8.0, 3.0], [7.0, 1.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 6.0, 1.0], [8.0, 4.0, 1.0], [4.0, 3.0, 2.0], [1.0, 8.0, 6.0]], [[7.0, 6.0, 4.0], [8.0, 
1.0, 2.0], [6.0, 2.0, 7.0], [1.0, 3.0, 1.0]], [[5.0, 1.0, 5.0], [5.0, 8.0, 1.0], [1.0, 2.0, 8.0], [7.0, 1.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 3.0, 8.0], [7.0, 5.0, 7.0], [5.0, 6.0, 2.0]], [[6.0, 8.0, 6.0], [2.0, 7.0, 1.0], [2.0, 
4.0, 7.0]], [[6.0, 2.0, 5.0], [5.0, 6.0, 4.0], [2.0, 6.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 7.0, 7.0], [3.0, 5.0, 1.0], 
[5.0, 7.0, 2.0]], [[8.0, 7.0, 8.0], [3.0, 3.0, 7.0], [7.0, 4.0, 6.0]], [[4.0, 8.0, 5.0], [5.0, 1.0, 7.0], [5.0, 3.0, 2.0]]])
      arg[:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 3.0, 8.0], [7.0, 5.0, 7.0], [5.0, 6.0, 2.0], [7.0, 8.0, 1.0]], [[6.0, 8.0, 6.0], [2.0, 
7.0, 1.0], [2.0, 4.0, 7.0], [3.0, 6.0, 3.0]], [[6.0, 2.0, 5.0], [5.0, 6.0, 4.0], [2.0, 6.0, 5.0], [7.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 7.0], [3.0, 5.0, 1.0], [5.0, 7.0, 2.0], [1.0, 8.0, 6.0]], [[8.0, 7.0, 8.0], [3.0, 
3.0, 7.0], [7.0, 4.0, 6.0], [1.0, 3.0, 1.0]], [[4.0, 8.0, 5.0], [5.0, 1.0, 7.0], [5.0, 3.0, 2.0], [7.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 7.0, 5.0], [7.0, 6.0, 2.0], [2.0, 8.0, 2.0], [2.0, 5.0, 4.0]], [[3.0, 5.0, 6.0], [2.0, 
7.0, 1.0], [7.0, 4.0, 1.0], [1.0, 4.0, 3.0]], [[6.0, 6.0, 5.0], [8.0, 8.0, 7.0], [4.0, 8.0, 2.0], [2.0, 5.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 6.0, 4.0], [6.0, 4.0, 2.0], [6.0, 6.0, 7.0], [6.0, 7.0, 3.0]], [[8.0, 6.0, 3.0], [1.0, 
3.0, 8.0], [4.0, 4.0, 3.0], [7.0, 2.0, 4.0]], [[7.0, 5.0, 4.0], [3.0, 2.0, 4.0], [3.0, 1.0, 8.0], [6.0, 6.0, 3.0]]])
      val=numpy.array([[8.0, 5.0, 4.0], [2.0, 3.0, 3.0], [1.0, 5.0, 8.0]])
      arg[:,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 5.0], [7.0, 5.0, 2.0], [2.0, 4.0, 2.0], [2.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [2.0, 
3.0, 1.0], [7.0, 3.0, 1.0], [1.0, 4.0, 3.0]], [[6.0, 1.0, 5.0], [8.0, 5.0, 7.0], [4.0, 8.0, 2.0], [2.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 8.0, 4.0], [6.0, 5.0, 2.0], [6.0, 4.0, 7.0], [6.0, 7.0, 3.0]], [[8.0, 2.0, 3.0], [1.0, 
3.0, 8.0], [4.0, 3.0, 3.0], [7.0, 2.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 5.0, 4.0], [3.0, 8.0, 8.0], [6.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 7.0, 6.0], [6.0, 6.0, 7.0], [1.0, 6.0, 4.0], [5.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], [6.0, 
5.0, 3.0], [6.0, 3.0, 4.0], [2.0, 4.0, 5.0]], [[6.0, 7.0, 5.0], [2.0, 2.0, 4.0], [7.0, 7.0, 1.0], [3.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 6.0, 6.0], [8.0, 8.0, 5.0], [2.0, 7.0, 7.0], [1.0, 6.0, 4.0]], [[8.0, 3.0, 7.0], [1.0, 
4.0, 2.0], [3.0, 5.0, 5.0], [1.0, 8.0, 4.0]], [[5.0, 8.0, 6.0], [1.0, 7.0, 4.0], [2.0, 5.0, 7.0], [4.0, 7.0, 4.0]]])
      val=Data(numpy.array([[4.0, 2.0, 8.0], [7.0, 6.0, 4.0], [5.0, 6.0, 5.0]]),self.functionspace)
      arg[:,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 6.0], [6.0, 2.0, 7.0], [1.0, 8.0, 4.0], [5.0, 6.0, 4.0]], [[2.0, 7.0, 1.0], [6.0, 
6.0, 3.0], [6.0, 4.0, 4.0], [2.0, 4.0, 5.0]], [[6.0, 5.0, 5.0], [2.0, 6.0, 4.0], [7.0, 5.0, 1.0], [3.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 6.0], [8.0, 2.0, 5.0], [2.0, 8.0, 7.0], [1.0, 6.0, 4.0]], [[8.0, 7.0, 7.0], [1.0, 
6.0, 2.0], [3.0, 4.0, 5.0], [1.0, 8.0, 4.0]], [[5.0, 5.0, 6.0], [1.0, 6.0, 4.0], [2.0, 5.0, 7.0], [4.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 4.0], [5.0, 3.0, 1.0], [3.0, 6.0, 6.0], [8.0, 8.0, 7.0]], [[7.0, 8.0, 1.0], [7.0, 
3.0, 3.0], [3.0, 2.0, 3.0], [2.0, 1.0, 6.0]], [[2.0, 7.0, 2.0], [1.0, 4.0, 2.0], [7.0, 4.0, 8.0], [2.0, 6.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 1.0], [4.0, 4.0, 8.0], [3.0, 5.0, 3.0], [6.0, 8.0, 4.0]], [[2.0, 6.0, 3.0], [4.0, 
5.0, 8.0], [1.0, 3.0, 7.0], [3.0, 8.0, 4.0]], [[7.0, 7.0, 5.0], [7.0, 2.0, 5.0], [4.0, 7.0, 8.0], [7.0, 7.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 4.0, 7.0], [4.0, 3.0, 2.0], [4.0, 3.0, 1.0]])+(1.-msk_val)*numpy.array([[6.0, 3.0, 7.0], 
[1.0, 4.0, 7.0], [6.0, 8.0, 5.0]])
      arg[:,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 4.0, 4.0], [5.0, 4.0, 1.0], [3.0, 7.0, 6.0], [8.0, 8.0, 7.0]], [[7.0, 4.0, 1.0], [7.0, 
3.0, 3.0], [3.0, 2.0, 3.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 2.0], [1.0, 3.0, 2.0], [7.0, 1.0, 8.0], [2.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 1.0], [4.0, 3.0, 8.0], [3.0, 7.0, 3.0], [6.0, 8.0, 4.0]], [[2.0, 1.0, 3.0], [4.0, 
4.0, 8.0], [1.0, 7.0, 7.0], [3.0, 8.0, 4.0]], [[7.0, 6.0, 5.0], [7.0, 8.0, 5.0], [4.0, 5.0, 8.0], [7.0, 7.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 1.0, 3.0], [6.0, 4.0, 3.0], [3.0, 4.0, 1.0], [3.0, 3.0, 4.0]], [[8.0, 8.0, 2.0], [5.0, 
1.0, 8.0], [4.0, 5.0, 8.0], [6.0, 4.0, 2.0]], [[8.0, 7.0, 8.0], [4.0, 8.0, 6.0], [1.0, 2.0, 4.0], [6.0, 2.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 5.0], [6.0, 3.0, 4.0], [8.0, 8.0, 2.0], [5.0, 8.0, 8.0]], [[6.0, 6.0, 3.0], [2.0, 
6.0, 7.0], [6.0, 2.0, 4.0], [1.0, 4.0, 7.0]], [[1.0, 7.0, 6.0], [8.0, 3.0, 4.0], [4.0, 8.0, 7.0], [4.0, 4.0, 7.0]]])
      val=numpy.array([[[8.0], [7.0], [4.0], [8.0]], [[1.0], [6.0], [7.0], [8.0]], [[4.0], [1.0], [4.0], [8.0]]])
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 8.0], [6.0, 4.0, 7.0], [3.0, 4.0, 4.0], [3.0, 3.0, 8.0]], [[8.0, 8.0, 1.0], [5.0, 
1.0, 6.0], [4.0, 5.0, 7.0], [6.0, 4.0, 8.0]], [[8.0, 7.0, 4.0], [4.0, 8.0, 1.0], [1.0, 2.0, 4.0], [6.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 8.0], [6.0, 3.0, 7.0], [8.0, 8.0, 4.0], [5.0, 8.0, 8.0]], [[6.0, 6.0, 1.0], [2.0, 
6.0, 6.0], [6.0, 2.0, 7.0], [1.0, 4.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 3.0, 1.0], [4.0, 8.0, 4.0], [4.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 2.0], [3.0, 2.0, 8.0], [2.0, 3.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 7.0, 5.0], [5.0, 
7.0, 6.0], [6.0, 3.0, 3.0], [8.0, 6.0, 1.0]], [[1.0, 4.0, 4.0], [8.0, 2.0, 1.0], [5.0, 6.0, 1.0], [3.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 2.0, 5.0], [6.0, 6.0, 5.0], [1.0, 4.0, 1.0], [1.0, 5.0, 5.0]], [[7.0, 8.0, 2.0], [4.0, 
2.0, 4.0], [5.0, 1.0, 2.0], [3.0, 8.0, 4.0]], [[2.0, 4.0, 7.0], [4.0, 4.0, 8.0], [3.0, 6.0, 2.0], [6.0, 4.0, 7.0]]])
      val=Data(numpy.array([[[4.0], [3.0], [4.0], [4.0]], [[6.0], [1.0], [4.0], [6.0]], [[7.0], [4.0], [6.0], 
[4.0]]]),self.functionspace)
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 4.0], [3.0, 2.0, 3.0], [2.0, 3.0, 4.0], [5.0, 3.0, 4.0]], [[8.0, 7.0, 6.0], [5.0, 
7.0, 1.0], [6.0, 3.0, 4.0], [8.0, 6.0, 6.0]], [[1.0, 4.0, 7.0], [8.0, 2.0, 4.0], [5.0, 6.0, 6.0], [3.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 4.0], [6.0, 6.0, 3.0], [1.0, 4.0, 4.0], [1.0, 5.0, 4.0]], [[7.0, 8.0, 6.0], [4.0, 
2.0, 1.0], [5.0, 1.0, 4.0], [3.0, 8.0, 6.0]], [[2.0, 4.0, 7.0], [4.0, 4.0, 4.0], [3.0, 6.0, 6.0], [6.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 8.0, 4.0], [7.0, 5.0, 7.0], [3.0, 4.0, 4.0], [8.0, 3.0, 1.0]], [[8.0, 3.0, 3.0], [4.0, 
6.0, 2.0], [4.0, 6.0, 2.0], [8.0, 8.0, 5.0]], [[7.0, 5.0, 8.0], [8.0, 8.0, 4.0], [6.0, 5.0, 3.0], [6.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 4.0, 1.0], [5.0, 8.0, 7.0], [5.0, 2.0, 8.0], [7.0, 7.0, 8.0]], [[2.0, 3.0, 5.0], [5.0, 
4.0, 6.0], [3.0, 2.0, 4.0], [2.0, 7.0, 3.0]], [[1.0, 4.0, 6.0], [8.0, 6.0, 1.0], [6.0, 5.0, 1.0], [6.0, 8.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0], [4.0], [5.0], [8.0]], [[5.0], [4.0], [4.0], [4.0]], [[5.0], [7.0], [3.0], 
[6.0]]])+(1.-msk_val)*numpy.array([[[3.0], [4.0], [8.0], [5.0]], [[3.0], [4.0], [2.0], [6.0]], [[7.0], [8.0], [3.0], [4.0]]])
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 8.0], [7.0, 5.0, 4.0], [3.0, 4.0, 5.0], [8.0, 3.0, 8.0]], [[8.0, 3.0, 5.0], [4.0, 
6.0, 4.0], [4.0, 6.0, 4.0], [8.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [8.0, 8.0, 7.0], [6.0, 5.0, 3.0], [6.0, 2.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 3.0], [5.0, 8.0, 4.0], [5.0, 2.0, 8.0], [7.0, 7.0, 5.0]], [[2.0, 3.0, 3.0], [5.0, 
4.0, 4.0], [3.0, 2.0, 2.0], [2.0, 7.0, 6.0]], [[1.0, 4.0, 7.0], [8.0, 6.0, 8.0], [6.0, 5.0, 3.0], [6.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 2.0, 3.0], [2.0, 3.0, 3.0], [8.0, 8.0, 7.0], [3.0, 2.0, 8.0]], [[8.0, 2.0, 1.0], [3.0, 
7.0, 5.0], [5.0, 2.0, 6.0], [4.0, 2.0, 7.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 5.0], [8.0, 6.0, 3.0], [3.0, 2.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 1.0], [3.0, 5.0, 2.0], [8.0, 5.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 4.0, 2.0], [6.0, 
2.0, 5.0], [1.0, 7.0, 7.0], [5.0, 3.0, 6.0]], [[7.0, 2.0, 3.0], [3.0, 1.0, 4.0], [3.0, 5.0, 4.0], [8.0, 5.0, 7.0]]])
      val=numpy.array([[7.0, 8.0, 1.0, 3.0]])
      arg[1:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 3.0], [2.0, 3.0, 3.0], [8.0, 8.0, 7.0], [3.0, 2.0, 8.0]], [[8.0, 7.0, 1.0], [3.0, 
8.0, 5.0], [5.0, 1.0, 6.0], [4.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [4.0, 8.0, 5.0], [8.0, 6.0, 3.0], [3.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 1.0], [3.0, 5.0, 2.0], [8.0, 5.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 7.0, 2.0], [6.0, 
8.0, 5.0], [1.0, 1.0, 7.0], [5.0, 3.0, 6.0]], [[7.0, 2.0, 3.0], [3.0, 1.0, 4.0], [3.0, 5.0, 4.0], [8.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 6.0, 8.0], [4.0, 5.0, 4.0], [6.0, 8.0, 8.0], [5.0, 8.0, 7.0]], [[6.0, 4.0, 7.0], [1.0, 
8.0, 7.0], [4.0, 3.0, 7.0], [8.0, 5.0, 7.0]], [[1.0, 1.0, 8.0], [6.0, 5.0, 1.0], [4.0, 6.0, 5.0], [6.0, 4.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 6.0, 1.0], [3.0, 8.0, 3.0], [3.0, 3.0, 4.0], [8.0, 1.0, 1.0]], [[7.0, 2.0, 6.0], [6.0, 
7.0, 3.0], [2.0, 1.0, 3.0], [6.0, 7.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 7.0, 8.0], [6.0, 2.0, 4.0], [6.0, 1.0, 7.0]]])
      val=Data(numpy.array([[6.0, 6.0, 6.0, 6.0]]),self.functionspace)
      arg[1:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 8.0], [4.0, 5.0, 4.0], [6.0, 8.0, 8.0], [5.0, 8.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 
6.0, 7.0], [4.0, 6.0, 7.0], [8.0, 6.0, 7.0]], [[1.0, 1.0, 8.0], [6.0, 5.0, 1.0], [4.0, 6.0, 5.0], [6.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 1.0], [3.0, 8.0, 3.0], [3.0, 3.0, 4.0], [8.0, 1.0, 1.0]], [[7.0, 6.0, 6.0], [6.0, 
6.0, 3.0], [2.0, 6.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 7.0, 8.0], [6.0, 2.0, 4.0], [6.0, 1.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 4.0, 3.0], [5.0, 5.0, 5.0], [8.0, 8.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 7.0, 5.0], [6.0, 
6.0, 6.0], [6.0, 6.0, 1.0], [2.0, 8.0, 1.0]], [[4.0, 2.0, 3.0], [5.0, 2.0, 6.0], [3.0, 3.0, 7.0], [2.0, 6.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 2.0, 3.0], [7.0, 7.0, 3.0], [5.0, 8.0, 5.0], [3.0, 1.0, 7.0]], [[2.0, 5.0, 3.0], [6.0, 
1.0, 2.0], [3.0, 6.0, 6.0], [4.0, 1.0, 1.0]], [[6.0, 8.0, 4.0], [7.0, 8.0, 8.0], [8.0, 3.0, 1.0], [2.0, 1.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 4.0, 3.0, 3.0]])+(1.-msk_val)*numpy.array([[6.0, 1.0, 4.0, 6.0]])
      arg[1:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 3.0], [5.0, 5.0, 5.0], [8.0, 8.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 5.0, 5.0], [6.0, 
4.0, 6.0], [6.0, 3.0, 1.0], [2.0, 3.0, 1.0]], [[4.0, 2.0, 3.0], [5.0, 2.0, 6.0], [3.0, 3.0, 7.0], [2.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 3.0], [7.0, 7.0, 3.0], [5.0, 8.0, 5.0], [3.0, 1.0, 7.0]], [[2.0, 6.0, 3.0], [6.0, 
1.0, 2.0], [3.0, 4.0, 6.0], [4.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [7.0, 8.0, 8.0], [8.0, 3.0, 1.0], [2.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 5.0], [8.0, 2.0, 5.0], [4.0, 5.0, 5.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [4.0, 
4.0, 8.0], [4.0, 7.0, 6.0], [7.0, 4.0, 5.0]], [[5.0, 6.0, 7.0], [3.0, 8.0, 4.0], [1.0, 8.0, 4.0], [5.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 2.0, 2.0], [1.0, 6.0, 1.0], [7.0, 4.0, 2.0], [4.0, 5.0, 5.0]], [[4.0, 1.0, 2.0], [2.0, 
8.0, 6.0], [1.0, 4.0, 4.0], [4.0, 3.0, 2.0]], [[8.0, 2.0, 6.0], [6.0, 6.0, 7.0], [1.0, 1.0, 7.0], [1.0, 2.0, 7.0]]])
      val=numpy.array([2.0])
      arg[1,2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 5.0], [8.0, 2.0, 5.0], [4.0, 5.0, 5.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [4.0, 
4.0, 8.0], [4.0, 7.0, 2.0], [7.0, 4.0, 5.0]], [[5.0, 6.0, 7.0], [3.0, 8.0, 4.0], [1.0, 8.0, 4.0], [5.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 2.0], [1.0, 6.0, 1.0], [7.0, 4.0, 2.0], [4.0, 5.0, 5.0]], [[4.0, 1.0, 2.0], [2.0, 
8.0, 6.0], [1.0, 4.0, 2.0], [4.0, 3.0, 2.0]], [[8.0, 2.0, 6.0], [6.0, 6.0, 7.0], [1.0, 1.0, 7.0], [1.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 1.0], [5.0, 7.0, 8.0], [2.0, 3.0, 3.0], [1.0, 7.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 
1.0, 3.0], [7.0, 2.0, 8.0], [2.0, 8.0, 5.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 7.0], [6.0, 8.0, 3.0], [4.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 2.0, 5.0], [4.0, 5.0, 5.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[2.0, 5.0, 3.0], [2.0, 
1.0, 4.0], [7.0, 6.0, 2.0], [1.0, 3.0, 5.0]], [[2.0, 6.0, 3.0], [6.0, 6.0, 1.0], [1.0, 2.0, 1.0], [7.0, 3.0, 2.0]]])
      val=Data(numpy.array([5.0]),self.functionspace)
      arg[1,2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 1.0], [5.0, 7.0, 8.0], [2.0, 3.0, 3.0], [1.0, 7.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 
1.0, 3.0], [7.0, 2.0, 5.0], [2.0, 8.0, 5.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 7.0], [6.0, 8.0, 3.0], [4.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 5.0], [4.0, 5.0, 5.0], [7.0, 2.0, 8.0], [4.0, 7.0, 5.0]], [[2.0, 5.0, 3.0], [2.0, 
1.0, 4.0], [7.0, 6.0, 5.0], [1.0, 3.0, 5.0]], [[2.0, 6.0, 3.0], [6.0, 6.0, 1.0], [1.0, 2.0, 1.0], [7.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 5.0, 7.0], [4.0, 8.0, 1.0], [5.0, 4.0, 5.0], [6.0, 7.0, 7.0]], [[7.0, 7.0, 1.0], [3.0, 
8.0, 7.0], [7.0, 2.0, 3.0], [6.0, 8.0, 1.0]], [[7.0, 5.0, 7.0], [3.0, 1.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 8.0, 8.0], [7.0, 4.0, 2.0], [5.0, 5.0, 8.0], [4.0, 2.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 
8.0, 2.0], [1.0, 7.0, 3.0], [8.0, 3.0, 3.0]], [[7.0, 1.0, 4.0], [2.0, 2.0, 5.0], [4.0, 2.0, 5.0], [2.0, 6.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([4.0])+(1.-msk_val)*numpy.array([4.0])
      arg[1,2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 7.0], [4.0, 8.0, 1.0], [5.0, 4.0, 5.0], [6.0, 7.0, 7.0]], [[7.0, 7.0, 1.0], [3.0, 
8.0, 7.0], [7.0, 2.0, 4.0], [6.0, 8.0, 1.0]], [[7.0, 5.0, 7.0], [3.0, 1.0, 5.0], [2.0, 8.0, 1.0], [5.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 8.0], [7.0, 4.0, 2.0], [5.0, 5.0, 8.0], [4.0, 2.0, 2.0]], [[6.0, 3.0, 1.0], [3.0, 
8.0, 2.0], [1.0, 7.0, 4.0], [8.0, 3.0, 3.0]], [[7.0, 1.0, 4.0], [2.0, 2.0, 5.0], [4.0, 2.0, 5.0], [2.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 5.0, 8.0], [8.0, 3.0, 5.0], [6.0, 5.0, 6.0], [8.0, 7.0, 4.0]], [[8.0, 8.0, 3.0], [6.0, 
7.0, 8.0], [7.0, 3.0, 4.0], [5.0, 8.0, 2.0]], [[1.0, 8.0, 1.0], [7.0, 2.0, 8.0], [4.0, 5.0, 2.0], [6.0, 4.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 4.0, 4.0], [8.0, 8.0, 4.0], [8.0, 7.0, 8.0], [6.0, 2.0, 7.0]], [[6.0, 6.0, 1.0], [5.0, 
6.0, 7.0], [8.0, 8.0, 1.0], [8.0, 6.0, 7.0]], [[7.0, 7.0, 5.0], [5.0, 4.0, 8.0], [6.0, 8.0, 1.0], [7.0, 2.0, 1.0]]])
      val=numpy.array([[5.0, 7.0]])
      arg[2:,1:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 5.0, 8.0], [8.0, 3.0, 5.0], [6.0, 5.0, 6.0], [8.0, 7.0, 4.0]], [[8.0, 8.0, 3.0], [6.0, 
7.0, 8.0], [7.0, 3.0, 4.0], [5.0, 8.0, 2.0]], [[1.0, 8.0, 1.0], [7.0, 5.0, 8.0], [4.0, 7.0, 2.0], [6.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 4.0], [8.0, 8.0, 4.0], [8.0, 7.0, 8.0], [6.0, 2.0, 7.0]], [[6.0, 6.0, 1.0], [5.0, 
6.0, 7.0], [8.0, 8.0, 1.0], [8.0, 6.0, 7.0]], [[7.0, 7.0, 5.0], [5.0, 5.0, 8.0], [6.0, 7.0, 1.0], [7.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 4.0], [7.0, 4.0, 5.0], [2.0, 5.0, 4.0], [4.0, 8.0, 3.0]], [[5.0, 7.0, 2.0], [4.0, 
4.0, 5.0], [5.0, 6.0, 7.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 4.0], [6.0, 8.0, 3.0], [3.0, 7.0, 1.0], [1.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 6.0, 8.0], [1.0, 5.0, 1.0], [8.0, 7.0, 2.0], [5.0, 6.0, 3.0]], [[2.0, 7.0, 2.0], [2.0, 
6.0, 5.0], [5.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[4.0, 3.0, 6.0], [8.0, 2.0, 4.0], [4.0, 6.0, 1.0], [1.0, 6.0, 5.0]]])
      val=Data(numpy.array([[8.0, 3.0]]),self.functionspace)
      arg[2:,1:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 4.0], [7.0, 4.0, 5.0], [2.0, 5.0, 4.0], [4.0, 8.0, 3.0]], [[5.0, 7.0, 2.0], [4.0, 
4.0, 5.0], [5.0, 6.0, 7.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 4.0], [6.0, 8.0, 3.0], [3.0, 3.0, 1.0], [1.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 8.0], [1.0, 5.0, 1.0], [8.0, 7.0, 2.0], [5.0, 6.0, 3.0]], [[2.0, 7.0, 2.0], [2.0, 
6.0, 5.0], [5.0, 3.0, 7.0], [1.0, 1.0, 5.0]], [[4.0, 3.0, 6.0], [8.0, 8.0, 4.0], [4.0, 3.0, 1.0], [1.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 1.0, 1.0], [3.0, 6.0, 4.0], [5.0, 6.0, 5.0], [7.0, 5.0, 7.0]], [[8.0, 5.0, 5.0], [5.0, 
5.0, 2.0], [5.0, 8.0, 7.0], [6.0, 8.0, 5.0]], [[5.0, 4.0, 3.0], [8.0, 8.0, 6.0], [8.0, 6.0, 5.0], [7.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 4.0], [4.0, 1.0, 8.0], [8.0, 2.0, 1.0], [6.0, 5.0, 1.0]], [[3.0, 2.0, 2.0], [3.0, 
2.0, 7.0], [8.0, 4.0, 4.0], [1.0, 5.0, 1.0]], [[5.0, 7.0, 7.0], [5.0, 3.0, 4.0], [7.0, 7.0, 7.0], [1.0, 1.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 5.0]])+(1.-msk_val)*numpy.array([[2.0, 5.0]])
      arg[2:,1:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 1.0], [3.0, 6.0, 4.0], [5.0, 6.0, 5.0], [7.0, 5.0, 7.0]], [[8.0, 5.0, 5.0], [5.0, 
5.0, 2.0], [5.0, 8.0, 7.0], [6.0, 8.0, 5.0]], [[5.0, 4.0, 3.0], [8.0, 2.0, 6.0], [8.0, 5.0, 5.0], [7.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 4.0], [4.0, 1.0, 8.0], [8.0, 2.0, 1.0], [6.0, 5.0, 1.0]], [[3.0, 2.0, 2.0], [3.0, 
2.0, 7.0], [8.0, 4.0, 4.0], [1.0, 5.0, 1.0]], [[5.0, 7.0, 7.0], [5.0, 2.0, 4.0], [7.0, 5.0, 7.0], [1.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 4.0], [1.0, 4.0, 1.0], [5.0, 4.0, 6.0], [6.0, 8.0, 8.0]], [[8.0, 8.0, 4.0], [3.0, 
1.0, 3.0], [3.0, 3.0, 4.0], [6.0, 6.0, 5.0]], [[6.0, 2.0, 7.0], [5.0, 4.0, 1.0], [4.0, 2.0, 4.0], [2.0, 4.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 6.0, 4.0], [3.0, 2.0, 4.0], [8.0, 8.0, 6.0], [7.0, 1.0, 1.0]], [[5.0, 1.0, 2.0], [6.0, 
4.0, 8.0], [4.0, 3.0, 3.0], [8.0, 1.0, 4.0]], [[7.0, 5.0, 8.0], [7.0, 7.0, 1.0], [1.0, 1.0, 7.0], [4.0, 4.0, 5.0]]])
      val=numpy.array([6.0])
      arg[1,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 4.0], [1.0, 4.0, 1.0], [5.0, 4.0, 6.0], [6.0, 8.0, 8.0]], [[8.0, 8.0, 4.0], [3.0, 
1.0, 3.0], [3.0, 6.0, 4.0], [6.0, 6.0, 5.0]], [[6.0, 2.0, 7.0], [5.0, 4.0, 1.0], [4.0, 2.0, 4.0], [2.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 4.0], [3.0, 2.0, 4.0], [8.0, 8.0, 6.0], [7.0, 1.0, 1.0]], [[5.0, 1.0, 2.0], [6.0, 
4.0, 8.0], [4.0, 6.0, 3.0], [8.0, 1.0, 4.0]], [[7.0, 5.0, 8.0], [7.0, 7.0, 1.0], [1.0, 1.0, 7.0], [4.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 3.0, 4.0], [4.0, 8.0, 2.0], [6.0, 7.0, 6.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 1.0], [6.0, 
3.0, 7.0], [6.0, 5.0, 6.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 5.0], [2.0, 8.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 3.0, 6.0], [2.0, 4.0, 6.0], [6.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[2.0, 6.0, 2.0], [4.0, 
7.0, 4.0], [8.0, 7.0, 7.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 2.0], [6.0, 8.0, 6.0], [4.0, 3.0, 4.0], [2.0, 2.0, 2.0]]])
      val=Data(numpy.array([7.0]),self.functionspace)
      arg[1,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 4.0], [4.0, 8.0, 2.0], [6.0, 7.0, 6.0], [8.0, 3.0, 1.0]], [[2.0, 2.0, 1.0], [6.0, 
3.0, 7.0], [6.0, 7.0, 6.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 5.0], [2.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 6.0], [2.0, 4.0, 6.0], [6.0, 4.0, 7.0], [6.0, 8.0, 7.0]], [[2.0, 6.0, 2.0], [4.0, 
7.0, 4.0], [8.0, 7.0, 7.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 2.0], [6.0, 8.0, 6.0], [4.0, 3.0, 4.0], [2.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 6.0, 4.0], [2.0, 7.0, 3.0], [3.0, 8.0, 2.0], [3.0, 2.0, 7.0]], [[1.0, 5.0, 6.0], [6.0, 
6.0, 2.0], [4.0, 4.0, 1.0], [6.0, 2.0, 3.0]], [[6.0, 8.0, 7.0], [1.0, 3.0, 1.0], [6.0, 3.0, 8.0], [4.0, 8.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 2.0, 1.0], [8.0, 3.0, 3.0], [2.0, 6.0, 3.0], [6.0, 3.0, 7.0]], [[4.0, 6.0, 2.0], [8.0, 
6.0, 6.0], [2.0, 8.0, 3.0], [4.0, 6.0, 1.0]], [[7.0, 6.0, 3.0], [8.0, 7.0, 7.0], [7.0, 5.0, 6.0], [3.0, 2.0, 6.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([2.0])+(1.-msk_val)*numpy.array([1.0])
      arg[1,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 6.0, 4.0], [2.0, 7.0, 3.0], [3.0, 8.0, 2.0], [3.0, 2.0, 7.0]], [[1.0, 5.0, 6.0], [6.0, 
6.0, 2.0], [4.0, 2.0, 1.0], [6.0, 2.0, 3.0]], [[6.0, 8.0, 7.0], [1.0, 3.0, 1.0], [6.0, 3.0, 8.0], [4.0, 8.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 1.0], [8.0, 3.0, 3.0], [2.0, 6.0, 3.0], [6.0, 3.0, 7.0]], [[4.0, 6.0, 2.0], [8.0, 
6.0, 6.0], [2.0, 1.0, 3.0], [4.0, 6.0, 1.0]], [[7.0, 6.0, 3.0], [8.0, 7.0, 7.0], [7.0, 5.0, 6.0], [3.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 4.0, 6.0], [3.0, 8.0, 6.0], [1.0, 1.0, 6.0], [3.0, 1.0, 5.0]], [[4.0, 2.0, 8.0], [1.0, 
8.0, 2.0], [1.0, 4.0, 4.0], [2.0, 6.0, 1.0]], [[6.0, 3.0, 7.0], [4.0, 6.0, 6.0], [1.0, 5.0, 4.0], [6.0, 4.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 8.0, 8.0], [3.0, 2.0, 1.0], [8.0, 5.0, 4.0], [3.0, 3.0, 2.0]], [[7.0, 7.0, 5.0], [5.0, 
4.0, 3.0], [3.0, 5.0, 6.0], [3.0, 7.0, 6.0]], [[8.0, 8.0, 8.0], [8.0, 8.0, 8.0], [1.0, 8.0, 1.0], [3.0, 1.0, 6.0]]])
      val=numpy.array([[[5.0], [7.0], [8.0]], [[2.0], [3.0], [5.0]]])
      arg[:2,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 5.0], [3.0, 8.0, 7.0], [1.0, 1.0, 8.0], [3.0, 1.0, 5.0]], [[4.0, 2.0, 2.0], [1.0, 
8.0, 3.0], [1.0, 4.0, 5.0], [2.0, 6.0, 1.0]], [[6.0, 3.0, 7.0], [4.0, 6.0, 6.0], [1.0, 5.0, 4.0], [6.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 5.0], [3.0, 2.0, 7.0], [8.0, 5.0, 8.0], [3.0, 3.0, 2.0]], [[7.0, 7.0, 2.0], [5.0, 
4.0, 3.0], [3.0, 5.0, 5.0], [3.0, 7.0, 6.0]], [[8.0, 8.0, 8.0], [8.0, 8.0, 8.0], [1.0, 8.0, 1.0], [3.0, 1.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 7.0, 4.0], [3.0, 1.0, 7.0], [8.0, 6.0, 8.0], [8.0, 4.0, 3.0]], [[8.0, 2.0, 3.0], [8.0, 
4.0, 2.0], [6.0, 2.0, 3.0], [1.0, 1.0, 8.0]], [[5.0, 3.0, 3.0], [2.0, 3.0, 7.0], [1.0, 7.0, 2.0], [1.0, 4.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 6.0, 5.0], [6.0, 4.0, 5.0], [3.0, 6.0, 4.0], [4.0, 7.0, 6.0]], [[6.0, 6.0, 2.0], [1.0, 
5.0, 4.0], [5.0, 8.0, 6.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 8.0], [5.0, 2.0, 7.0], [3.0, 3.0, 7.0], [1.0, 4.0, 6.0]]])
      val=Data(numpy.array([[[3.0], [3.0], [1.0]], [[6.0], [1.0], [3.0]]]),self.functionspace)
      arg[:2,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 3.0], [3.0, 1.0, 3.0], [8.0, 6.0, 1.0], [8.0, 4.0, 3.0]], [[8.0, 2.0, 6.0], [8.0, 
4.0, 1.0], [6.0, 2.0, 3.0], [1.0, 1.0, 8.0]], [[5.0, 3.0, 3.0], [2.0, 3.0, 7.0], [1.0, 7.0, 2.0], [1.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 3.0], [6.0, 4.0, 3.0], [3.0, 6.0, 1.0], [4.0, 7.0, 6.0]], [[6.0, 6.0, 6.0], [1.0, 
5.0, 1.0], [5.0, 8.0, 3.0], [4.0, 6.0, 2.0]], [[5.0, 8.0, 8.0], [5.0, 2.0, 7.0], [3.0, 3.0, 7.0], [1.0, 4.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 5.0, 5.0], [1.0, 6.0, 1.0], [2.0, 1.0, 5.0], [3.0, 1.0, 4.0]], [[3.0, 7.0, 6.0], [4.0, 
3.0, 5.0], [4.0, 5.0, 2.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 6.0], [6.0, 2.0, 8.0], [1.0, 5.0, 1.0], [1.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 5.0], [3.0, 4.0, 8.0], [4.0, 2.0, 3.0], [6.0, 5.0, 1.0]], [[7.0, 1.0, 8.0], [4.0, 
5.0, 2.0], [5.0, 7.0, 1.0], [8.0, 8.0, 5.0]], [[3.0, 7.0, 7.0], [8.0, 1.0, 3.0], [8.0, 1.0, 4.0], [4.0, 5.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0], [7.0], [2.0]], [[3.0], [6.0], [7.0]]])+(1.-msk_val)*numpy.array([[[5.0], [4.0], [4.0]], 
[[8.0], [4.0], [1.0]]])
      arg[:2,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 4.0], [1.0, 6.0, 7.0], [2.0, 1.0, 2.0], [3.0, 1.0, 4.0]], [[3.0, 7.0, 3.0], [4.0, 
3.0, 6.0], [4.0, 5.0, 7.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 6.0], [6.0, 2.0, 8.0], [1.0, 5.0, 1.0], [1.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 5.0], [3.0, 4.0, 4.0], [4.0, 2.0, 4.0], [6.0, 5.0, 1.0]], [[7.0, 1.0, 8.0], [4.0, 
5.0, 4.0], [5.0, 7.0, 1.0], [8.0, 8.0, 5.0]], [[3.0, 7.0, 7.0], [8.0, 1.0, 3.0], [8.0, 1.0, 4.0], [4.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 3.0, 1.0], [6.0, 6.0, 5.0], [2.0, 2.0, 5.0], [1.0, 5.0, 7.0]], [[7.0, 4.0, 5.0], [7.0, 
1.0, 5.0], [3.0, 8.0, 6.0], [4.0, 2.0, 4.0]], [[4.0, 8.0, 3.0], [7.0, 4.0, 3.0], [2.0, 7.0, 5.0], [2.0, 6.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 2.0, 3.0], [4.0, 2.0, 1.0], [8.0, 3.0, 2.0], [1.0, 3.0, 2.0]], [[4.0, 8.0, 7.0], [3.0, 
8.0, 5.0], [2.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[1.0, 2.0, 7.0], [8.0, 3.0, 6.0], [4.0, 7.0, 1.0], [7.0, 1.0, 5.0]]])
      val=numpy.array([[[3.0, 2.0, 1.0], [7.0, 4.0, 6.0]]])
      arg[1:2,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 1.0], [6.0, 6.0, 5.0], [2.0, 2.0, 5.0], [1.0, 5.0, 7.0]], [[7.0, 4.0, 5.0], [3.0, 
2.0, 1.0], [7.0, 4.0, 6.0], [4.0, 2.0, 4.0]], [[4.0, 8.0, 3.0], [7.0, 4.0, 3.0], [2.0, 7.0, 5.0], [2.0, 6.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 3.0], [4.0, 2.0, 1.0], [8.0, 3.0, 2.0], [1.0, 3.0, 2.0]], [[4.0, 8.0, 7.0], [3.0, 
2.0, 1.0], [7.0, 4.0, 6.0], [1.0, 5.0, 1.0]], [[1.0, 2.0, 7.0], [8.0, 3.0, 6.0], [4.0, 7.0, 1.0], [7.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 6.0, 4.0], [1.0, 3.0, 8.0], [5.0, 5.0, 8.0], [5.0, 5.0, 1.0]], [[2.0, 4.0, 1.0], [5.0, 
2.0, 3.0], [8.0, 8.0, 2.0], [2.0, 2.0, 3.0]], [[4.0, 7.0, 1.0], [2.0, 2.0, 7.0], [3.0, 6.0, 8.0], [8.0, 6.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 3.0, 5.0], [8.0, 7.0, 7.0], [6.0, 5.0, 2.0], [4.0, 2.0, 5.0]], [[2.0, 3.0, 5.0], [5.0, 
5.0, 1.0], [4.0, 6.0, 4.0], [3.0, 6.0, 4.0]], [[7.0, 3.0, 7.0], [8.0, 4.0, 5.0], [5.0, 1.0, 7.0], [3.0, 6.0, 8.0]]])
      val=Data(numpy.array([[[1.0, 4.0, 2.0], [5.0, 1.0, 3.0]]]),self.functionspace)
      arg[1:2,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 4.0], [1.0, 3.0, 8.0], [5.0, 5.0, 8.0], [5.0, 5.0, 1.0]], [[2.0, 4.0, 1.0], [1.0, 
4.0, 2.0], [5.0, 1.0, 3.0], [2.0, 2.0, 3.0]], [[4.0, 7.0, 1.0], [2.0, 2.0, 7.0], [3.0, 6.0, 8.0], [8.0, 6.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 5.0], [8.0, 7.0, 7.0], [6.0, 5.0, 2.0], [4.0, 2.0, 5.0]], [[2.0, 3.0, 5.0], [1.0, 
4.0, 2.0], [5.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 3.0, 7.0], [8.0, 4.0, 5.0], [5.0, 1.0, 7.0], [3.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 3.0, 7.0], [2.0, 8.0, 5.0], [1.0, 6.0, 7.0], [2.0, 6.0, 3.0]], [[2.0, 4.0, 1.0], [5.0, 
1.0, 5.0], [2.0, 6.0, 5.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 8.0], [5.0, 1.0, 5.0], [7.0, 1.0, 7.0], [7.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 8.0], [7.0, 3.0, 2.0], [7.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[1.0, 7.0, 3.0], [3.0, 
8.0, 8.0], [3.0, 2.0, 6.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 4.0], [6.0, 4.0, 4.0], [6.0, 2.0, 1.0], [5.0, 7.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 5.0, 7.0], [7.0, 4.0, 7.0]]])+(1.-msk_val)*numpy.array([[[4.0, 3.0, 4.0], [2.0, 2.0, 
8.0]]])
      arg[1:2,1:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 7.0], [2.0, 8.0, 5.0], [1.0, 6.0, 7.0], [2.0, 6.0, 3.0]], [[2.0, 4.0, 1.0], [6.0, 
5.0, 7.0], [7.0, 4.0, 7.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 8.0], [5.0, 1.0, 5.0], [7.0, 1.0, 7.0], [7.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 8.0], [7.0, 3.0, 2.0], [7.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[1.0, 7.0, 3.0], [4.0, 
3.0, 4.0], [2.0, 2.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 4.0], [6.0, 4.0, 4.0], [6.0, 2.0, 1.0], [5.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 6.0, 2.0], [1.0, 8.0, 7.0], [2.0, 7.0, 1.0], [8.0, 3.0, 3.0]], [[4.0, 1.0, 7.0], [8.0, 
5.0, 8.0], [7.0, 5.0, 7.0], [4.0, 6.0, 7.0]], [[7.0, 4.0, 4.0], [2.0, 7.0, 4.0], [7.0, 6.0, 6.0], [3.0, 3.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 2.0], [3.0, 5.0, 3.0], [7.0, 4.0, 2.0], [6.0, 6.0, 2.0]], [[4.0, 4.0, 4.0], [4.0, 
1.0, 6.0], [3.0, 6.0, 1.0], [8.0, 5.0, 3.0]], [[8.0, 3.0, 2.0], [4.0, 8.0, 1.0], [4.0, 3.0, 4.0], [5.0, 7.0, 5.0]]])
      val=numpy.array([[6.0], [6.0], [4.0]])
      arg[1,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 2.0], [1.0, 8.0, 7.0], [2.0, 7.0, 1.0], [8.0, 3.0, 3.0]], [[4.0, 1.0, 6.0], [8.0, 
5.0, 6.0], [7.0, 5.0, 4.0], [4.0, 6.0, 7.0]], [[7.0, 4.0, 4.0], [2.0, 7.0, 4.0], [7.0, 6.0, 6.0], [3.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 2.0], [3.0, 5.0, 3.0], [7.0, 4.0, 2.0], [6.0, 6.0, 2.0]], [[4.0, 4.0, 6.0], [4.0, 
1.0, 6.0], [3.0, 6.0, 4.0], [8.0, 5.0, 3.0]], [[8.0, 3.0, 2.0], [4.0, 8.0, 1.0], [4.0, 3.0, 4.0], [5.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 4.0, 3.0], [5.0, 5.0, 4.0], [7.0, 3.0, 2.0], [1.0, 5.0, 2.0]], [[1.0, 5.0, 8.0], [6.0, 
6.0, 3.0], [4.0, 7.0, 7.0], [7.0, 1.0, 4.0]], [[4.0, 8.0, 8.0], [2.0, 3.0, 1.0], [7.0, 4.0, 2.0], [2.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 6.0, 6.0], [2.0, 5.0, 2.0], [6.0, 5.0, 5.0], [4.0, 2.0, 1.0]], [[4.0, 5.0, 5.0], [4.0, 
4.0, 5.0], [5.0, 2.0, 5.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 6.0, 1.0], [6.0, 8.0, 3.0], [3.0, 7.0, 6.0]]])
      val=Data(numpy.array([[5.0], [2.0], [4.0]]),self.functionspace)
      arg[1,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 3.0], [5.0, 5.0, 4.0], [7.0, 3.0, 2.0], [1.0, 5.0, 2.0]], [[1.0, 5.0, 5.0], [6.0, 
6.0, 2.0], [4.0, 7.0, 4.0], [7.0, 1.0, 4.0]], [[4.0, 8.0, 8.0], [2.0, 3.0, 1.0], [7.0, 4.0, 2.0], [2.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 6.0], [2.0, 5.0, 2.0], [6.0, 5.0, 5.0], [4.0, 2.0, 1.0]], [[4.0, 5.0, 5.0], [4.0, 
4.0, 2.0], [5.0, 2.0, 4.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 3.0], [8.0, 6.0, 1.0], [6.0, 8.0, 3.0], [3.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 2.0, 4.0], [6.0, 4.0, 1.0], [6.0, 2.0, 8.0], [6.0, 6.0, 3.0]], [[7.0, 1.0, 6.0], [8.0, 
8.0, 3.0], [1.0, 5.0, 7.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 7.0], [6.0, 1.0, 6.0], [4.0, 3.0, 2.0], [1.0, 2.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 8.0, 6.0], [3.0, 7.0, 1.0], [2.0, 2.0, 1.0], [6.0, 2.0, 8.0]], [[7.0, 1.0, 3.0], [2.0, 
1.0, 2.0], [7.0, 5.0, 8.0], [5.0, 2.0, 6.0]], [[1.0, 1.0, 7.0], [1.0, 6.0, 1.0], [4.0, 2.0, 5.0], [6.0, 1.0, 6.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0], [5.0], [8.0]])+(1.-msk_val)*numpy.array([[5.0], [2.0], [7.0]])
      arg[1,:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 2.0, 4.0], [6.0, 4.0, 1.0], [6.0, 2.0, 8.0], [6.0, 6.0, 3.0]], [[7.0, 1.0, 3.0], [8.0, 
8.0, 5.0], [1.0, 5.0, 8.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 7.0], [6.0, 1.0, 6.0], [4.0, 3.0, 2.0], [1.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 6.0], [3.0, 7.0, 1.0], [2.0, 2.0, 1.0], [6.0, 2.0, 8.0]], [[7.0, 1.0, 5.0], [2.0, 
1.0, 2.0], [7.0, 5.0, 7.0], [5.0, 2.0, 6.0]], [[1.0, 1.0, 7.0], [1.0, 6.0, 1.0], [4.0, 2.0, 5.0], [6.0, 1.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 4.0], [6.0, 6.0, 7.0], [7.0, 1.0, 3.0], [7.0, 3.0, 6.0]], [[7.0, 3.0, 7.0], [7.0, 
1.0, 5.0], [4.0, 5.0, 4.0], [7.0, 8.0, 4.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 5.0], [1.0, 4.0, 4.0], [4.0, 4.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 7.0, 6.0], [5.0, 8.0, 4.0], [4.0, 4.0, 8.0], [8.0, 8.0, 3.0]], [[3.0, 1.0, 4.0], [4.0, 
8.0, 8.0], [7.0, 4.0, 1.0], [2.0, 5.0, 8.0]], [[8.0, 3.0, 5.0], [3.0, 6.0, 6.0], [4.0, 8.0, 2.0], [2.0, 1.0, 8.0]]])
      val=numpy.array([8.0, 5.0, 1.0])
      arg[1,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 4.0], [6.0, 6.0, 7.0], [7.0, 1.0, 3.0], [7.0, 3.0, 6.0]], [[7.0, 3.0, 7.0], [7.0, 
1.0, 5.0], [8.0, 5.0, 1.0], [7.0, 8.0, 4.0]], [[2.0, 2.0, 6.0], [3.0, 7.0, 5.0], [1.0, 4.0, 4.0], [4.0, 4.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 6.0], [5.0, 8.0, 4.0], [4.0, 4.0, 8.0], [8.0, 8.0, 3.0]], [[3.0, 1.0, 4.0], [4.0, 
8.0, 8.0], [8.0, 5.0, 1.0], [2.0, 5.0, 8.0]], [[8.0, 3.0, 5.0], [3.0, 6.0, 6.0], [4.0, 8.0, 2.0], [2.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 2.0, 4.0], [5.0, 4.0, 8.0], [1.0, 3.0, 3.0], [1.0, 7.0, 2.0]], [[4.0, 6.0, 2.0], [3.0, 
7.0, 4.0], [5.0, 4.0, 7.0], [8.0, 1.0, 2.0]], [[3.0, 4.0, 3.0], [8.0, 4.0, 8.0], [1.0, 5.0, 5.0], [1.0, 2.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 4.0, 3.0], [8.0, 8.0, 4.0], [2.0, 6.0, 3.0], [2.0, 7.0, 8.0]], [[3.0, 3.0, 6.0], [5.0, 
4.0, 1.0], [8.0, 2.0, 4.0], [7.0, 5.0, 7.0]], [[5.0, 1.0, 6.0], [3.0, 4.0, 5.0], [7.0, 6.0, 4.0], [3.0, 4.0, 3.0]]])
      val=Data(numpy.array([6.0, 8.0, 7.0]),self.functionspace)
      arg[1,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 4.0], [5.0, 4.0, 8.0], [1.0, 3.0, 3.0], [1.0, 7.0, 2.0]], [[4.0, 6.0, 2.0], [3.0, 
7.0, 4.0], [6.0, 8.0, 7.0], [8.0, 1.0, 2.0]], [[3.0, 4.0, 3.0], [8.0, 4.0, 8.0], [1.0, 5.0, 5.0], [1.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 4.0, 3.0], [8.0, 8.0, 4.0], [2.0, 6.0, 3.0], [2.0, 7.0, 8.0]], [[3.0, 3.0, 6.0], [5.0, 
4.0, 1.0], [6.0, 8.0, 7.0], [7.0, 5.0, 7.0]], [[5.0, 1.0, 6.0], [3.0, 4.0, 5.0], [7.0, 6.0, 4.0], [3.0, 4.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 2.0, 7.0], [8.0, 4.0, 2.0], [5.0, 1.0, 5.0], [6.0, 1.0, 2.0]], [[4.0, 7.0, 1.0], [5.0, 
1.0, 3.0], [3.0, 8.0, 1.0], [4.0, 6.0, 1.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 3.0], [4.0, 2.0, 1.0], [8.0, 6.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 6.0], [2.0, 3.0, 2.0], [6.0, 1.0, 8.0], [5.0, 6.0, 5.0]], [[8.0, 3.0, 6.0], [7.0, 
3.0, 7.0], [4.0, 5.0, 8.0], [1.0, 5.0, 8.0]], [[4.0, 4.0, 5.0], [2.0, 4.0, 5.0], [5.0, 3.0, 7.0], [4.0, 8.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([8.0, 5.0, 2.0])+(1.-msk_val)*numpy.array([6.0, 8.0, 5.0])
      arg[1,2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 7.0], [8.0, 4.0, 2.0], [5.0, 1.0, 5.0], [6.0, 1.0, 2.0]], [[4.0, 7.0, 1.0], [5.0, 
1.0, 3.0], [8.0, 5.0, 2.0], [4.0, 6.0, 1.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 3.0], [4.0, 2.0, 1.0], [8.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 6.0], [2.0, 3.0, 2.0], [6.0, 1.0, 8.0], [5.0, 6.0, 5.0]], [[8.0, 3.0, 6.0], [7.0, 
3.0, 7.0], [6.0, 8.0, 5.0], [1.0, 5.0, 8.0]], [[4.0, 4.0, 5.0], [2.0, 4.0, 5.0], [5.0, 3.0, 7.0], [4.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 6.0, 7.0], [3.0, 7.0, 3.0], [1.0, 5.0, 1.0], [8.0, 6.0, 2.0]], [[8.0, 6.0, 2.0], [8.0, 
7.0, 3.0], [5.0, 7.0, 1.0], [1.0, 8.0, 4.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 8.0], [6.0, 4.0, 6.0], [6.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 7.0, 3.0], [3.0, 6.0, 1.0], [2.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[3.0, 6.0, 4.0], [2.0, 
6.0, 3.0], [3.0, 3.0, 8.0], [3.0, 6.0, 5.0]], [[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [3.0, 6.0, 6.0], [2.0, 2.0, 6.0]]])
      val=numpy.array([[1.0, 2.0, 4.0], [2.0, 1.0, 5.0]])
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 7.0], [3.0, 7.0, 3.0], [1.0, 5.0, 1.0], [8.0, 6.0, 2.0]], [[8.0, 6.0, 2.0], [8.0, 
7.0, 3.0], [1.0, 2.0, 4.0], [2.0, 1.0, 5.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 8.0], [6.0, 4.0, 6.0], [6.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 3.0], [3.0, 6.0, 1.0], [2.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[3.0, 6.0, 4.0], [2.0, 
6.0, 3.0], [1.0, 2.0, 4.0], [2.0, 1.0, 5.0]], [[7.0, 1.0, 7.0], [1.0, 4.0, 2.0], [3.0, 6.0, 6.0], [2.0, 2.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 1.0, 7.0], [7.0, 8.0, 3.0], [7.0, 5.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 1.0, 3.0], [5.0, 
3.0, 7.0], [5.0, 6.0, 4.0], [4.0, 5.0, 6.0]], [[1.0, 5.0, 2.0], [3.0, 2.0, 5.0], [5.0, 7.0, 7.0], [3.0, 3.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 6.0, 2.0], [6.0, 1.0, 3.0], [7.0, 3.0, 8.0], [8.0, 6.0, 8.0]], [[6.0, 1.0, 2.0], [3.0, 
6.0, 7.0], [4.0, 8.0, 5.0], [3.0, 4.0, 5.0]], [[2.0, 3.0, 2.0], [7.0, 8.0, 7.0], [7.0, 2.0, 2.0], [8.0, 8.0, 7.0]]])
      val=Data(numpy.array([[3.0, 3.0, 2.0], [7.0, 4.0, 7.0]]),self.functionspace)
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 7.0], [7.0, 8.0, 3.0], [7.0, 5.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 1.0, 3.0], [5.0, 
3.0, 7.0], [3.0, 3.0, 2.0], [7.0, 4.0, 7.0]], [[1.0, 5.0, 2.0], [3.0, 2.0, 5.0], [5.0, 7.0, 7.0], [3.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 6.0, 2.0], [6.0, 1.0, 3.0], [7.0, 3.0, 8.0], [8.0, 6.0, 8.0]], [[6.0, 1.0, 2.0], [3.0, 
6.0, 7.0], [3.0, 3.0, 2.0], [7.0, 4.0, 7.0]], [[2.0, 3.0, 2.0], [7.0, 8.0, 7.0], [7.0, 2.0, 2.0], [8.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 1.0, 4.0], [8.0, 5.0, 7.0], [8.0, 6.0, 5.0], [8.0, 2.0, 4.0]], [[6.0, 7.0, 7.0], [7.0, 
2.0, 2.0], [7.0, 6.0, 2.0], [4.0, 3.0, 2.0]], [[2.0, 1.0, 6.0], [5.0, 4.0, 8.0], [4.0, 5.0, 4.0], [4.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 5.0, 2.0], [3.0, 6.0, 1.0], [1.0, 5.0, 3.0], [4.0, 6.0, 3.0]], [[5.0, 2.0, 7.0], [5.0, 
7.0, 6.0], [4.0, 3.0, 3.0], [3.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], [1.0, 7.0, 6.0], [3.0, 4.0, 3.0], [1.0, 2.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 4.0, 2.0], [6.0, 8.0, 1.0]])+(1.-msk_val)*numpy.array([[1.0, 1.0, 5.0], [2.0, 7.0, 7.0]])
      arg[1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 4.0], [8.0, 5.0, 7.0], [8.0, 6.0, 5.0], [8.0, 2.0, 4.0]], [[6.0, 7.0, 7.0], [7.0, 
2.0, 2.0], [6.0, 4.0, 2.0], [6.0, 8.0, 1.0]], [[2.0, 1.0, 6.0], [5.0, 4.0, 8.0], [4.0, 5.0, 4.0], [4.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 2.0], [3.0, 6.0, 1.0], [1.0, 5.0, 3.0], [4.0, 6.0, 3.0]], [[5.0, 2.0, 7.0], [5.0, 
7.0, 6.0], [1.0, 1.0, 5.0], [2.0, 7.0, 7.0]], [[3.0, 1.0, 6.0], [1.0, 7.0, 6.0], [3.0, 4.0, 3.0], [1.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 8.0, 8.0], [6.0, 2.0, 6.0], [2.0, 7.0, 7.0], [2.0, 6.0, 1.0]], [[4.0, 5.0, 8.0], [8.0, 
3.0, 6.0], [3.0, 5.0, 2.0], [5.0, 3.0, 3.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 2.0], [6.0, 6.0, 6.0], [2.0, 6.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 1.0], [5.0, 8.0, 8.0], [5.0, 2.0, 3.0], [4.0, 8.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 
5.0, 1.0], [3.0, 6.0, 3.0], [2.0, 3.0, 3.0]], [[5.0, 3.0, 1.0], [3.0, 5.0, 3.0], [6.0, 8.0, 4.0], [3.0, 8.0, 2.0]]])
      val=numpy.array([[[2.0], [1.0]], [[7.0], [4.0]], [[2.0], [5.0]]])
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 8.0], [6.0, 2.0, 6.0], [2.0, 2.0, 7.0], [2.0, 1.0, 1.0]], [[4.0, 5.0, 8.0], [8.0, 
3.0, 6.0], [3.0, 7.0, 2.0], [5.0, 4.0, 3.0]], [[2.0, 1.0, 7.0], [2.0, 7.0, 2.0], [6.0, 2.0, 6.0], [2.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 1.0], [5.0, 8.0, 8.0], [5.0, 2.0, 3.0], [4.0, 1.0, 2.0]], [[3.0, 8.0, 7.0], [3.0, 
5.0, 1.0], [3.0, 7.0, 3.0], [2.0, 4.0, 3.0]], [[5.0, 3.0, 1.0], [3.0, 5.0, 3.0], [6.0, 2.0, 4.0], [3.0, 5.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 7.0, 4.0], [2.0, 6.0, 4.0], [8.0, 6.0, 2.0], [6.0, 1.0, 2.0]], [[1.0, 8.0, 5.0], [6.0, 
8.0, 4.0], [3.0, 3.0, 7.0], [6.0, 2.0, 3.0]], [[2.0, 3.0, 7.0], [8.0, 3.0, 3.0], [2.0, 7.0, 1.0], [4.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 5.0], [1.0, 8.0, 4.0], [6.0, 3.0, 2.0], [3.0, 1.0, 4.0]], [[7.0, 6.0, 5.0], [6.0, 
5.0, 6.0], [3.0, 8.0, 5.0], [4.0, 7.0, 3.0]], [[5.0, 5.0, 1.0], [6.0, 6.0, 3.0], [2.0, 8.0, 4.0], [7.0, 4.0, 6.0]]])
      val=Data(numpy.array([[[6.0], [1.0]], [[5.0], [5.0]], [[3.0], [5.0]]]),self.functionspace)
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 7.0, 4.0], [2.0, 6.0, 4.0], [8.0, 6.0, 2.0], [6.0, 1.0, 2.0]], [[1.0, 8.0, 5.0], [6.0, 
8.0, 4.0], [3.0, 5.0, 7.0], [6.0, 5.0, 3.0]], [[2.0, 3.0, 7.0], [8.0, 3.0, 3.0], [2.0, 3.0, 1.0], [4.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 5.0], [1.0, 8.0, 4.0], [6.0, 6.0, 2.0], [3.0, 1.0, 4.0]], [[7.0, 6.0, 5.0], [6.0, 
5.0, 6.0], [3.0, 5.0, 5.0], [4.0, 5.0, 3.0]], [[5.0, 5.0, 1.0], [6.0, 6.0, 3.0], [2.0, 3.0, 4.0], [7.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 3.0, 4.0], [4.0, 8.0, 5.0], [4.0, 3.0, 8.0], [4.0, 6.0, 2.0]], [[3.0, 8.0, 1.0], [4.0, 
6.0, 2.0], [1.0, 2.0, 1.0], [1.0, 6.0, 8.0]], [[2.0, 1.0, 2.0], [4.0, 8.0, 3.0], [7.0, 7.0, 1.0], [8.0, 7.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 5.0, 2.0], [5.0, 8.0, 6.0], [8.0, 8.0, 3.0], [3.0, 8.0, 3.0]], [[3.0, 7.0, 3.0], [4.0, 
4.0, 8.0], [4.0, 7.0, 6.0], [7.0, 3.0, 6.0]], [[1.0, 8.0, 6.0], [1.0, 7.0, 2.0], [7.0, 1.0, 3.0], [3.0, 7.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0], [8.0]], [[7.0], [4.0]], [[4.0], [6.0]]])+(1.-msk_val)*numpy.array([[[1.0], [5.0]], 
[[6.0], [6.0]], [[7.0], [5.0]]])
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 4.0], [4.0, 8.0, 5.0], [4.0, 7.0, 8.0], [4.0, 8.0, 2.0]], [[3.0, 8.0, 1.0], [4.0, 
6.0, 2.0], [1.0, 7.0, 1.0], [1.0, 4.0, 8.0]], [[2.0, 1.0, 2.0], [4.0, 8.0, 3.0], [7.0, 4.0, 1.0], [8.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 2.0], [5.0, 8.0, 6.0], [8.0, 1.0, 3.0], [3.0, 5.0, 3.0]], [[3.0, 7.0, 3.0], [4.0, 
4.0, 8.0], [4.0, 6.0, 6.0], [7.0, 6.0, 6.0]], [[1.0, 8.0, 6.0], [1.0, 7.0, 2.0], [7.0, 7.0, 3.0], [3.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_ULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 7.0, 1.0], [2.0, 3.0, 5.0], [8.0, 4.0, 1.0], [2.0, 4.0, 4.0]], [[3.0, 5.0, 8.0], [8.0, 
4.0, 2.0], [8.0, 1.0, 5.0], [6.0, 2.0, 6.0]], [[6.0, 4.0, 2.0], [7.0, 3.0, 4.0], [4.0, 8.0, 4.0], [7.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 4.0, 1.0], [4.0, 7.0, 3.0], [2.0, 2.0, 5.0], [5.0, 5.0, 5.0]], [[8.0, 2.0, 3.0], [3.0, 
6.0, 7.0], [3.0, 3.0, 1.0], [2.0, 6.0, 8.0]], [[1.0, 8.0, 1.0], [6.0, 8.0, 7.0], [6.0, 6.0, 4.0], [4.0, 4.0, 5.0]]])
      val=numpy.array([[[5.0, 5.0, 5.0], [2.0, 2.0, 5.0]], [[2.0, 4.0, 6.0], [8.0, 3.0, 7.0]]])
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 1.0], [2.0, 3.0, 5.0], [5.0, 5.0, 5.0], [2.0, 2.0, 5.0]], [[3.0, 5.0, 8.0], [8.0, 
4.0, 2.0], [2.0, 4.0, 6.0], [8.0, 3.0, 7.0]], [[6.0, 4.0, 2.0], [7.0, 3.0, 4.0], [4.0, 8.0, 4.0], [7.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 1.0], [4.0, 7.0, 3.0], [5.0, 5.0, 5.0], [2.0, 2.0, 5.0]], [[8.0, 2.0, 3.0], [3.0, 
6.0, 7.0], [2.0, 4.0, 6.0], [8.0, 3.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 8.0, 7.0], [6.0, 6.0, 4.0], [4.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_ULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 2.0, 4.0], [3.0, 2.0, 7.0], [4.0, 3.0, 6.0], [8.0, 1.0, 3.0]], [[8.0, 5.0, 5.0], [5.0, 
4.0, 2.0], [7.0, 2.0, 4.0], [4.0, 2.0, 4.0]], [[4.0, 7.0, 2.0], [1.0, 1.0, 3.0], [5.0, 1.0, 1.0], [5.0, 5.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 1.0, 6.0], [1.0, 4.0, 2.0], [6.0, 6.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 4.0], [3.0, 
3.0, 1.0], [6.0, 2.0, 2.0], [5.0, 7.0, 8.0]], [[4.0, 6.0, 8.0], [3.0, 8.0, 4.0], [3.0, 7.0, 2.0], [1.0, 1.0, 1.0]]])
      val=Data(numpy.array([[[4.0, 6.0, 5.0], [8.0, 8.0, 6.0]], [[6.0, 2.0, 4.0], [5.0, 3.0, 7.0]]]),self.functionspace)
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 4.0], [3.0, 2.0, 7.0], [4.0, 6.0, 5.0], [8.0, 8.0, 6.0]], [[8.0, 5.0, 5.0], [5.0, 
4.0, 2.0], [6.0, 2.0, 4.0], [5.0, 3.0, 7.0]], [[4.0, 7.0, 2.0], [1.0, 1.0, 3.0], [5.0, 1.0, 1.0], [5.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 6.0], [1.0, 4.0, 2.0], [4.0, 6.0, 5.0], [8.0, 8.0, 6.0]], [[2.0, 1.0, 4.0], [3.0, 
3.0, 1.0], [6.0, 2.0, 4.0], [5.0, 3.0, 7.0]], [[4.0, 6.0, 8.0], [3.0, 8.0, 4.0], [3.0, 7.0, 2.0], [1.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_ULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 7.0], [1.0, 6.0, 6.0], [4.0, 3.0, 7.0], [7.0, 1.0, 2.0]], [[2.0, 7.0, 2.0], [1.0, 
2.0, 4.0], [2.0, 2.0, 2.0], [6.0, 7.0, 4.0]], [[2.0, 7.0, 7.0], [4.0, 2.0, 4.0], [6.0, 8.0, 2.0], [7.0, 6.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 2.0, 4.0], [3.0, 4.0, 5.0], [2.0, 1.0, 5.0], [8.0, 4.0, 7.0]], [[6.0, 4.0, 7.0], [5.0, 
2.0, 5.0], [5.0, 2.0, 6.0], [2.0, 5.0, 4.0]], [[3.0, 2.0, 1.0], [5.0, 4.0, 1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 1.0, 2.0], [4.0, 6.0, 4.0]], [[7.0, 7.0, 2.0], [6.0, 2.0, 
7.0]]])+(1.-msk_val)*numpy.array([[[3.0, 7.0, 2.0], [4.0, 3.0, 7.0]], [[3.0, 5.0, 8.0], [3.0, 4.0, 8.0]]])
      arg[:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 7.0], [1.0, 6.0, 6.0], [5.0, 1.0, 2.0], [4.0, 6.0, 4.0]], [[2.0, 7.0, 2.0], [1.0, 
2.0, 4.0], [7.0, 7.0, 2.0], [6.0, 2.0, 7.0]], [[2.0, 7.0, 7.0], [4.0, 2.0, 4.0], [6.0, 8.0, 2.0], [7.0, 6.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 4.0], [3.0, 4.0, 5.0], [3.0, 7.0, 2.0], [4.0, 3.0, 7.0]], [[6.0, 4.0, 7.0], [5.0, 
2.0, 5.0], [3.0, 5.0, 8.0], [3.0, 4.0, 8.0]], [[3.0, 2.0, 1.0], [5.0, 4.0, 1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 6.0, 1.0], [3.0, 4.0, 3.0], [4.0, 4.0, 6.0], [6.0, 2.0, 2.0]], [[3.0, 5.0, 7.0], [4.0, 
4.0, 8.0], [5.0, 6.0, 5.0], [5.0, 2.0, 2.0]], [[6.0, 1.0, 8.0], [4.0, 1.0, 2.0], [2.0, 8.0, 7.0], [8.0, 1.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 7.0, 8.0], [2.0, 8.0, 3.0], [2.0, 3.0, 2.0], [7.0, 8.0, 2.0]], [[2.0, 5.0, 3.0], [4.0, 
3.0, 1.0], [4.0, 3.0, 3.0], [4.0, 5.0, 3.0]], [[3.0, 2.0, 4.0], [5.0, 8.0, 8.0], [7.0, 7.0, 1.0], [5.0, 7.0, 7.0]]])
      val=numpy.array([[6.0, 2.0, 6.0]])
      arg[1:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 1.0], [3.0, 4.0, 3.0], [4.0, 4.0, 6.0], [6.0, 2.0, 2.0]], [[3.0, 5.0, 7.0], [4.0, 
4.0, 8.0], [6.0, 2.0, 6.0], [5.0, 2.0, 2.0]], [[6.0, 1.0, 8.0], [4.0, 1.0, 2.0], [2.0, 8.0, 7.0], [8.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 7.0, 8.0], [2.0, 8.0, 3.0], [2.0, 3.0, 2.0], [7.0, 8.0, 2.0]], [[2.0, 5.0, 3.0], [4.0, 
3.0, 1.0], [6.0, 2.0, 6.0], [4.0, 5.0, 3.0]], [[3.0, 2.0, 4.0], [5.0, 8.0, 8.0], [7.0, 7.0, 1.0], [5.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 5.0, 2.0], [7.0, 2.0, 4.0], [6.0, 6.0, 2.0], [7.0, 6.0, 2.0]], [[5.0, 8.0, 5.0], [4.0, 
2.0, 5.0], [1.0, 3.0, 8.0], [3.0, 7.0, 2.0]], [[2.0, 8.0, 7.0], [6.0, 8.0, 3.0], [4.0, 7.0, 5.0], [5.0, 4.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 5.0, 2.0], [4.0, 3.0, 7.0], [6.0, 7.0, 1.0], [4.0, 4.0, 2.0]], [[3.0, 6.0, 8.0], [7.0, 
6.0, 4.0], [4.0, 2.0, 1.0], [2.0, 5.0, 8.0]], [[4.0, 6.0, 6.0], [4.0, 6.0, 8.0], [3.0, 5.0, 2.0], [5.0, 2.0, 8.0]]])
      val=Data(numpy.array([[6.0, 8.0, 4.0]]),self.functionspace)
      arg[1:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 2.0], [7.0, 2.0, 4.0], [6.0, 6.0, 2.0], [7.0, 6.0, 2.0]], [[5.0, 8.0, 5.0], [4.0, 
2.0, 5.0], [6.0, 8.0, 4.0], [3.0, 7.0, 2.0]], [[2.0, 8.0, 7.0], [6.0, 8.0, 3.0], [4.0, 7.0, 5.0], [5.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 2.0], [4.0, 3.0, 7.0], [6.0, 7.0, 1.0], [4.0, 4.0, 2.0]], [[3.0, 6.0, 8.0], [7.0, 
6.0, 4.0], [6.0, 8.0, 4.0], [2.0, 5.0, 8.0]], [[4.0, 6.0, 6.0], [4.0, 6.0, 8.0], [3.0, 5.0, 2.0], [5.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 8.0, 2.0], [6.0, 1.0, 2.0], [4.0, 1.0, 4.0], [6.0, 6.0, 3.0]], [[2.0, 6.0, 4.0], [2.0, 
8.0, 1.0], [2.0, 8.0, 4.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 7.0], [4.0, 5.0, 6.0], [6.0, 3.0, 8.0], [6.0, 8.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 5.0, 1.0], [3.0, 8.0, 6.0], [8.0, 2.0, 7.0], [1.0, 6.0, 3.0]], [[8.0, 8.0, 5.0], [2.0, 
7.0, 8.0], [8.0, 8.0, 1.0], [3.0, 8.0, 6.0]], [[2.0, 3.0, 4.0], [7.0, 1.0, 4.0], [4.0, 7.0, 7.0], [8.0, 3.0, 2.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 3.0, 6.0]])+(1.-msk_val)*numpy.array([[7.0, 4.0, 5.0]])
      arg[1:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 2.0], [6.0, 1.0, 2.0], [4.0, 1.0, 4.0], [6.0, 6.0, 3.0]], [[2.0, 6.0, 4.0], [2.0, 
8.0, 1.0], [5.0, 3.0, 6.0], [2.0, 7.0, 2.0]], [[7.0, 4.0, 7.0], [4.0, 5.0, 6.0], [6.0, 3.0, 8.0], [6.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 1.0], [3.0, 8.0, 6.0], [8.0, 2.0, 7.0], [1.0, 6.0, 3.0]], [[8.0, 8.0, 5.0], [2.0, 
7.0, 8.0], [7.0, 4.0, 5.0], [3.0, 8.0, 6.0]], [[2.0, 3.0, 4.0], [7.0, 1.0, 4.0], [4.0, 7.0, 7.0], [8.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 4.0, 8.0], [6.0, 4.0, 8.0], [8.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], [7.0, 
7.0, 8.0], [3.0, 2.0, 6.0], [1.0, 7.0, 7.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 7.0], [1.0, 5.0, 3.0], [6.0, 7.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 8.0, 3.0], [2.0, 5.0, 1.0], [8.0, 4.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 4.0, 7.0], [6.0, 
6.0, 7.0], [7.0, 5.0, 2.0], [5.0, 7.0, 3.0]], [[1.0, 4.0, 5.0], [6.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 2.0, 8.0]]])
      val=numpy.array([[4.0], [8.0]])
      arg[1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 8.0], [6.0, 4.0, 8.0], [8.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[7.0, 7.0, 5.0], [7.0, 
7.0, 8.0], [3.0, 4.0, 6.0], [1.0, 8.0, 7.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 7.0], [1.0, 5.0, 3.0], [6.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 8.0, 3.0], [2.0, 5.0, 1.0], [8.0, 4.0, 8.0], [3.0, 3.0, 4.0]], [[3.0, 4.0, 7.0], [6.0, 
6.0, 7.0], [7.0, 4.0, 2.0], [5.0, 8.0, 3.0]], [[1.0, 4.0, 5.0], [6.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 2.0, 5.0], [7.0, 2.0, 8.0], [5.0, 8.0, 6.0], [2.0, 2.0, 4.0]], [[4.0, 4.0, 3.0], [1.0, 
7.0, 6.0], [5.0, 7.0, 7.0], [8.0, 7.0, 2.0]], [[2.0, 3.0, 2.0], [6.0, 8.0, 6.0], [3.0, 2.0, 4.0], [1.0, 4.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 2.0], [2.0, 8.0, 6.0], [8.0, 5.0, 5.0], [4.0, 3.0, 4.0]], [[3.0, 5.0, 1.0], [1.0, 
4.0, 6.0], [2.0, 8.0, 2.0], [8.0, 3.0, 4.0]], [[6.0, 1.0, 6.0], [2.0, 1.0, 1.0], [8.0, 5.0, 6.0], [5.0, 6.0, 1.0]]])
      val=Data(numpy.array([[2.0], [7.0]]),self.functionspace)
      arg[1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 5.0], [7.0, 2.0, 8.0], [5.0, 8.0, 6.0], [2.0, 2.0, 4.0]], [[4.0, 4.0, 3.0], [1.0, 
7.0, 6.0], [5.0, 2.0, 7.0], [8.0, 7.0, 2.0]], [[2.0, 3.0, 2.0], [6.0, 8.0, 6.0], [3.0, 2.0, 4.0], [1.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 2.0], [2.0, 8.0, 6.0], [8.0, 5.0, 5.0], [4.0, 3.0, 4.0]], [[3.0, 5.0, 1.0], [1.0, 
4.0, 6.0], [2.0, 2.0, 2.0], [8.0, 7.0, 4.0]], [[6.0, 1.0, 6.0], [2.0, 1.0, 1.0], [8.0, 5.0, 6.0], [5.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 7.0, 4.0], [1.0, 2.0, 3.0], [7.0, 5.0, 5.0], [2.0, 7.0, 5.0]], [[3.0, 3.0, 2.0], [5.0, 
4.0, 3.0], [5.0, 8.0, 1.0], [4.0, 2.0, 5.0]], [[8.0, 7.0, 5.0], [8.0, 8.0, 1.0], [4.0, 3.0, 8.0], [5.0, 7.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 2.0, 3.0], [8.0, 7.0, 4.0], [1.0, 1.0, 4.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 4.0], [3.0, 
1.0, 3.0], [1.0, 7.0, 8.0], [7.0, 3.0, 8.0]], [[8.0, 4.0, 5.0], [1.0, 6.0, 3.0], [5.0, 5.0, 6.0], [7.0, 7.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0], [3.0]])+(1.-msk_val)*numpy.array([[3.0], [7.0]])
      arg[1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 7.0, 4.0], [1.0, 2.0, 3.0], [7.0, 5.0, 5.0], [2.0, 7.0, 5.0]], [[3.0, 3.0, 2.0], [5.0, 
4.0, 3.0], [5.0, 6.0, 1.0], [4.0, 3.0, 5.0]], [[8.0, 7.0, 5.0], [8.0, 8.0, 1.0], [4.0, 3.0, 8.0], [5.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 2.0, 3.0], [8.0, 7.0, 4.0], [1.0, 1.0, 4.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 4.0], [3.0, 
1.0, 3.0], [1.0, 3.0, 8.0], [7.0, 7.0, 8.0]], [[8.0, 4.0, 5.0], [1.0, 6.0, 3.0], [5.0, 5.0, 6.0], [7.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 7.0], [2.0, 5.0, 8.0], [3.0, 5.0, 8.0], [6.0, 1.0, 5.0]], [[4.0, 3.0, 5.0], [5.0, 
6.0, 2.0], [2.0, 8.0, 8.0], [7.0, 2.0, 5.0]], [[3.0, 1.0, 4.0], [1.0, 4.0, 4.0], [4.0, 6.0, 1.0], [8.0, 7.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 5.0, 4.0], [6.0, 5.0, 1.0], [7.0, 2.0, 2.0], [2.0, 3.0, 3.0]], [[5.0, 7.0, 5.0], [5.0, 
2.0, 5.0], [4.0, 1.0, 4.0], [3.0, 2.0, 6.0]], [[8.0, 2.0, 8.0], [4.0, 8.0, 3.0], [7.0, 6.0, 2.0], [7.0, 4.0, 7.0]]])
      val=numpy.array([[[3.0], [8.0], [4.0]]])
      arg[2:,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 7.0], [2.0, 5.0, 8.0], [3.0, 5.0, 8.0], [6.0, 1.0, 5.0]], [[4.0, 3.0, 5.0], [5.0, 
6.0, 2.0], [2.0, 8.0, 8.0], [7.0, 2.0, 5.0]], [[3.0, 3.0, 4.0], [1.0, 8.0, 4.0], [4.0, 4.0, 1.0], [8.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 4.0], [6.0, 5.0, 1.0], [7.0, 2.0, 2.0], [2.0, 3.0, 3.0]], [[5.0, 7.0, 5.0], [5.0, 
2.0, 5.0], [4.0, 1.0, 4.0], [3.0, 2.0, 6.0]], [[8.0, 3.0, 8.0], [4.0, 8.0, 3.0], [7.0, 4.0, 2.0], [7.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 3.0, 8.0], [6.0, 4.0, 8.0], [3.0, 8.0, 3.0], [3.0, 6.0, 6.0]], [[6.0, 6.0, 5.0], [4.0, 
5.0, 7.0], [6.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[7.0, 7.0, 4.0], [1.0, 5.0, 1.0], [1.0, 1.0, 7.0], [7.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 1.0, 8.0], [1.0, 5.0, 8.0], [2.0, 7.0, 6.0], [5.0, 6.0, 7.0]], [[5.0, 1.0, 3.0], [5.0, 
5.0, 1.0], [8.0, 7.0, 6.0], [3.0, 2.0, 5.0]], [[3.0, 6.0, 3.0], [5.0, 7.0, 3.0], [8.0, 2.0, 1.0], [2.0, 4.0, 3.0]]])
      val=Data(numpy.array([[[3.0], [1.0], [2.0]]]),self.functionspace)
      arg[2:,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 8.0], [6.0, 4.0, 8.0], [3.0, 8.0, 3.0], [3.0, 6.0, 6.0]], [[6.0, 6.0, 5.0], [4.0, 
5.0, 7.0], [6.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[7.0, 3.0, 4.0], [1.0, 1.0, 1.0], [1.0, 2.0, 7.0], [7.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 8.0], [1.0, 5.0, 8.0], [2.0, 7.0, 6.0], [5.0, 6.0, 7.0]], [[5.0, 1.0, 3.0], [5.0, 
5.0, 1.0], [8.0, 7.0, 6.0], [3.0, 2.0, 5.0]], [[3.0, 3.0, 3.0], [5.0, 1.0, 3.0], [8.0, 2.0, 1.0], [2.0, 4.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 2.0, 5.0], [4.0, 5.0, 2.0], [6.0, 5.0, 5.0], [2.0, 1.0, 1.0]], [[4.0, 2.0, 1.0], [3.0, 
3.0, 7.0], [1.0, 5.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 6.0, 3.0], [4.0, 6.0, 1.0], [8.0, 3.0, 5.0], [3.0, 4.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 1.0, 4.0], [3.0, 2.0, 1.0], [1.0, 5.0, 3.0], [3.0, 2.0, 1.0]], [[2.0, 6.0, 4.0], [3.0, 
6.0, 8.0], [3.0, 6.0, 1.0], [3.0, 2.0, 2.0]], [[5.0, 3.0, 4.0], [7.0, 2.0, 3.0], [2.0, 5.0, 6.0], [2.0, 8.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0], [5.0], [6.0]]])+(1.-msk_val)*numpy.array([[[6.0], [2.0], [8.0]]])
      arg[2:,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 5.0], [4.0, 5.0, 2.0], [6.0, 5.0, 5.0], [2.0, 1.0, 1.0]], [[4.0, 2.0, 1.0], [3.0, 
3.0, 7.0], [1.0, 5.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 3.0], [4.0, 5.0, 1.0], [8.0, 6.0, 5.0], [3.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 4.0], [3.0, 2.0, 1.0], [1.0, 5.0, 3.0], [3.0, 2.0, 1.0]], [[2.0, 6.0, 4.0], [3.0, 
6.0, 8.0], [3.0, 6.0, 1.0], [3.0, 2.0, 2.0]], [[5.0, 6.0, 4.0], [7.0, 2.0, 3.0], [2.0, 8.0, 6.0], [2.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 2.0, 2.0], [7.0, 2.0, 8.0], [8.0, 5.0, 8.0], [6.0, 6.0, 7.0]], [[8.0, 4.0, 7.0], [4.0, 
6.0, 6.0], [8.0, 4.0, 4.0], [4.0, 5.0, 5.0]], [[8.0, 5.0, 7.0], [2.0, 2.0, 7.0], [3.0, 4.0, 4.0], [2.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 6.0, 8.0], [2.0, 2.0, 8.0], [5.0, 2.0, 4.0], [2.0, 1.0, 4.0]], [[1.0, 8.0, 1.0], [1.0, 
1.0, 3.0], [6.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[1.0, 1.0, 3.0], [6.0, 5.0, 7.0], [8.0, 5.0, 4.0], [3.0, 3.0, 5.0]]])
      val=numpy.array([[2.0, 5.0], [4.0, 8.0], [8.0, 8.0], [6.0, 7.0]])
      arg[1,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 2.0], [7.0, 2.0, 8.0], [8.0, 5.0, 8.0], [6.0, 6.0, 7.0]], [[2.0, 5.0, 7.0], [4.0, 
8.0, 6.0], [8.0, 8.0, 4.0], [6.0, 7.0, 5.0]], [[8.0, 5.0, 7.0], [2.0, 2.0, 7.0], [3.0, 4.0, 4.0], [2.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 6.0, 8.0], [2.0, 2.0, 8.0], [5.0, 2.0, 4.0], [2.0, 1.0, 4.0]], [[2.0, 5.0, 1.0], [4.0, 
8.0, 3.0], [8.0, 8.0, 7.0], [6.0, 7.0, 7.0]], [[1.0, 1.0, 3.0], [6.0, 5.0, 7.0], [8.0, 5.0, 4.0], [3.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 2.0], [7.0, 2.0, 8.0], [3.0, 2.0, 2.0], [8.0, 3.0, 4.0]], [[7.0, 1.0, 8.0], [5.0, 
8.0, 3.0], [6.0, 4.0, 6.0], [1.0, 4.0, 6.0]], [[2.0, 3.0, 8.0], [2.0, 7.0, 5.0], [5.0, 5.0, 2.0], [4.0, 2.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 2.0], [3.0, 8.0, 8.0], [8.0, 7.0, 5.0], [2.0, 2.0, 7.0]], [[7.0, 4.0, 7.0], [2.0, 
6.0, 3.0], [1.0, 6.0, 8.0], [6.0, 5.0, 6.0]], [[3.0, 3.0, 1.0], [3.0, 6.0, 1.0], [2.0, 5.0, 8.0], [3.0, 4.0, 7.0]]])
      val=Data(numpy.array([[5.0, 5.0], [2.0, 7.0], [5.0, 2.0], [8.0, 2.0]]),self.functionspace)
      arg[1,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 2.0], [7.0, 2.0, 8.0], [3.0, 2.0, 2.0], [8.0, 3.0, 4.0]], [[5.0, 5.0, 8.0], [2.0, 
7.0, 3.0], [5.0, 2.0, 6.0], [8.0, 2.0, 6.0]], [[2.0, 3.0, 8.0], [2.0, 7.0, 5.0], [5.0, 5.0, 2.0], [4.0, 2.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 2.0], [3.0, 8.0, 8.0], [8.0, 7.0, 5.0], [2.0, 2.0, 7.0]], [[5.0, 5.0, 7.0], [2.0, 
7.0, 3.0], [5.0, 2.0, 8.0], [8.0, 2.0, 6.0]], [[3.0, 3.0, 1.0], [3.0, 6.0, 1.0], [2.0, 5.0, 8.0], [3.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 8.0, 7.0], [7.0, 1.0, 3.0], [2.0, 5.0, 8.0], [4.0, 8.0, 7.0]], [[1.0, 4.0, 4.0], [7.0, 
7.0, 3.0], [4.0, 2.0, 1.0], [8.0, 2.0, 4.0]], [[2.0, 6.0, 7.0], [8.0, 4.0, 4.0], [5.0, 2.0, 1.0], [5.0, 3.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 7.0, 4.0], [2.0, 3.0, 3.0], [2.0, 5.0, 3.0], [8.0, 1.0, 1.0]], [[7.0, 1.0, 3.0], [8.0, 
2.0, 4.0], [3.0, 4.0, 6.0], [7.0, 4.0, 5.0]], [[2.0, 8.0, 2.0], [4.0, 1.0, 8.0], [6.0, 2.0, 4.0], [8.0, 2.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 6.0], [4.0, 3.0], [1.0, 8.0], [7.0, 5.0]])+(1.-msk_val)*numpy.array([[4.0, 2.0], [1.0, 
7.0], [8.0, 8.0], [3.0, 7.0]])
      arg[1,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 7.0], [7.0, 1.0, 3.0], [2.0, 5.0, 8.0], [4.0, 8.0, 7.0]], [[4.0, 6.0, 4.0], [4.0, 
3.0, 3.0], [1.0, 8.0, 1.0], [7.0, 5.0, 4.0]], [[2.0, 6.0, 7.0], [8.0, 4.0, 4.0], [5.0, 2.0, 1.0], [5.0, 3.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 7.0, 4.0], [2.0, 3.0, 3.0], [2.0, 5.0, 3.0], [8.0, 1.0, 1.0]], [[4.0, 2.0, 3.0], [1.0, 
7.0, 4.0], [8.0, 8.0, 6.0], [3.0, 7.0, 5.0]], [[2.0, 8.0, 2.0], [4.0, 1.0, 8.0], [6.0, 2.0, 4.0], [8.0, 2.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 7.0], [7.0, 8.0, 6.0], [5.0, 4.0, 8.0], [7.0, 3.0, 8.0]], [[4.0, 6.0, 2.0], [3.0, 
3.0, 1.0], [8.0, 1.0, 4.0], [3.0, 3.0, 5.0]], [[5.0, 1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 2.0, 6.0], [4.0, 2.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 2.0], [1.0, 1.0, 5.0], [8.0, 4.0, 3.0], [8.0, 6.0, 2.0]], [[7.0, 2.0, 5.0], [8.0, 
1.0, 6.0], [1.0, 7.0, 5.0], [1.0, 2.0, 4.0]], [[5.0, 5.0, 3.0], [3.0, 2.0, 6.0], [2.0, 3.0, 7.0], [2.0, 5.0, 4.0]]])
      val=numpy.array([[4.0], [5.0]])
      arg[1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 7.0], [7.0, 8.0, 6.0], [5.0, 4.0, 8.0], [7.0, 3.0, 8.0]], [[4.0, 6.0, 2.0], [3.0, 
3.0, 1.0], [8.0, 1.0, 4.0], [3.0, 3.0, 5.0]], [[5.0, 1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 2.0, 6.0], [4.0, 2.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 2.0], [1.0, 1.0, 5.0], [8.0, 4.0, 3.0], [8.0, 6.0, 2.0]], [[7.0, 2.0, 5.0], [8.0, 
1.0, 6.0], [1.0, 7.0, 4.0], [1.0, 2.0, 5.0]], [[5.0, 5.0, 3.0], [3.0, 2.0, 6.0], [2.0, 3.0, 7.0], [2.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 4.0, 4.0], [4.0, 1.0, 5.0], [6.0, 3.0, 2.0], [8.0, 3.0, 2.0]], [[3.0, 5.0, 4.0], [3.0, 
5.0, 8.0], [2.0, 1.0, 1.0], [6.0, 5.0, 8.0]], [[4.0, 5.0, 5.0], [7.0, 3.0, 5.0], [3.0, 3.0, 4.0], [2.0, 8.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 6.0], [2.0, 1.0, 3.0], [6.0, 1.0, 8.0], [3.0, 2.0, 7.0]], [[2.0, 3.0, 1.0], [2.0, 
4.0, 8.0], [8.0, 8.0, 8.0], [1.0, 5.0, 2.0]], [[8.0, 8.0, 5.0], [3.0, 1.0, 2.0], [8.0, 1.0, 8.0], [1.0, 5.0, 8.0]]])
      val=Data(numpy.array([[3.0], [1.0]]),self.functionspace)
      arg[1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 4.0], [4.0, 1.0, 5.0], [6.0, 3.0, 2.0], [8.0, 3.0, 2.0]], [[3.0, 5.0, 4.0], [3.0, 
5.0, 8.0], [2.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[4.0, 5.0, 5.0], [7.0, 3.0, 5.0], [3.0, 3.0, 4.0], [2.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 6.0], [2.0, 1.0, 3.0], [6.0, 1.0, 8.0], [3.0, 2.0, 7.0]], [[2.0, 3.0, 1.0], [2.0, 
4.0, 8.0], [8.0, 8.0, 3.0], [1.0, 5.0, 1.0]], [[8.0, 8.0, 5.0], [3.0, 1.0, 2.0], [8.0, 1.0, 8.0], [1.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 6.0], [2.0, 7.0, 8.0], [2.0, 7.0, 8.0], [2.0, 5.0, 4.0]], [[7.0, 5.0, 1.0], [4.0, 
2.0, 6.0], [8.0, 3.0, 5.0], [1.0, 3.0, 3.0]], [[1.0, 2.0, 4.0], [6.0, 8.0, 1.0], [1.0, 3.0, 1.0], [6.0, 8.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 3.0, 2.0], [3.0, 5.0, 4.0], [4.0, 5.0, 4.0], [7.0, 4.0, 7.0]], [[4.0, 5.0, 3.0], [3.0, 
4.0, 5.0], [1.0, 8.0, 7.0], [4.0, 1.0, 8.0]], [[4.0, 8.0, 8.0], [2.0, 2.0, 4.0], [7.0, 3.0, 7.0], [2.0, 7.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0], [1.0]])+(1.-msk_val)*numpy.array([[6.0], [2.0]])
      arg[1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 6.0], [2.0, 7.0, 8.0], [2.0, 7.0, 8.0], [2.0, 5.0, 4.0]], [[7.0, 5.0, 1.0], [4.0, 
2.0, 6.0], [8.0, 3.0, 5.0], [1.0, 3.0, 1.0]], [[1.0, 2.0, 4.0], [6.0, 8.0, 1.0], [1.0, 3.0, 1.0], [6.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 3.0, 2.0], [3.0, 5.0, 4.0], [4.0, 5.0, 4.0], [7.0, 4.0, 7.0]], [[4.0, 5.0, 3.0], [3.0, 
4.0, 5.0], [1.0, 8.0, 6.0], [4.0, 1.0, 2.0]], [[4.0, 8.0, 8.0], [2.0, 2.0, 4.0], [7.0, 3.0, 7.0], [2.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 7.0, 2.0], [8.0, 7.0, 4.0], [5.0, 4.0, 4.0], [8.0, 7.0, 8.0]], [[8.0, 5.0, 1.0], [8.0, 
1.0, 3.0], [1.0, 2.0, 3.0], [4.0, 5.0, 8.0]], [[5.0, 3.0, 2.0], [1.0, 5.0, 5.0], [3.0, 3.0, 2.0], [3.0, 6.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 7.0, 6.0], [3.0, 5.0, 6.0], [2.0, 5.0, 4.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 1.0], [3.0, 
2.0, 7.0], [3.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[2.0, 4.0, 3.0], [1.0, 6.0, 2.0], [8.0, 5.0, 8.0], [1.0, 7.0, 8.0]]])
      val=numpy.array([[7.0, 3.0]])
      arg[2:,2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 7.0, 2.0], [8.0, 7.0, 4.0], [5.0, 4.0, 4.0], [8.0, 7.0, 8.0]], [[8.0, 5.0, 1.0], [8.0, 
1.0, 3.0], [1.0, 2.0, 3.0], [4.0, 5.0, 8.0]], [[5.0, 3.0, 2.0], [1.0, 5.0, 5.0], [7.0, 3.0, 2.0], [3.0, 6.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 7.0, 6.0], [3.0, 5.0, 6.0], [2.0, 5.0, 4.0], [7.0, 2.0, 4.0]], [[3.0, 2.0, 1.0], [3.0, 
2.0, 7.0], [3.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[2.0, 4.0, 3.0], [1.0, 6.0, 2.0], [7.0, 3.0, 8.0], [1.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 4.0], [5.0, 3.0, 1.0], [5.0, 2.0, 4.0], [6.0, 1.0, 6.0]], [[6.0, 5.0, 3.0], [8.0, 
1.0, 1.0], [3.0, 6.0, 7.0], [8.0, 6.0, 6.0]], [[8.0, 3.0, 1.0], [5.0, 6.0, 5.0], [5.0, 1.0, 8.0], [1.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 6.0, 1.0], [8.0, 2.0, 5.0], [3.0, 8.0, 8.0], [1.0, 7.0, 8.0]], [[4.0, 8.0, 4.0], [7.0, 
6.0, 6.0], [2.0, 8.0, 8.0], [8.0, 7.0, 1.0]], [[4.0, 3.0, 6.0], [6.0, 6.0, 7.0], [4.0, 5.0, 3.0], [1.0, 6.0, 8.0]]])
      val=Data(numpy.array([[6.0, 8.0]]),self.functionspace)
      arg[2:,2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 4.0], [5.0, 3.0, 1.0], [5.0, 2.0, 4.0], [6.0, 1.0, 6.0]], [[6.0, 5.0, 3.0], [8.0, 
1.0, 1.0], [3.0, 6.0, 7.0], [8.0, 6.0, 6.0]], [[8.0, 3.0, 1.0], [5.0, 6.0, 5.0], [6.0, 8.0, 8.0], [1.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 1.0], [8.0, 2.0, 5.0], [3.0, 8.0, 8.0], [1.0, 7.0, 8.0]], [[4.0, 8.0, 4.0], [7.0, 
6.0, 6.0], [2.0, 8.0, 8.0], [8.0, 7.0, 1.0]], [[4.0, 3.0, 6.0], [6.0, 6.0, 7.0], [6.0, 8.0, 3.0], [1.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 8.0, 4.0], [2.0, 6.0, 8.0], [8.0, 5.0, 7.0], [2.0, 2.0, 3.0]], [[5.0, 8.0, 4.0], [4.0, 
3.0, 3.0], [1.0, 5.0, 2.0], [4.0, 7.0, 7.0]], [[4.0, 8.0, 5.0], [6.0, 4.0, 7.0], [8.0, 6.0, 2.0], [1.0, 7.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 5.0, 8.0], [8.0, 2.0, 6.0], [7.0, 6.0, 8.0], [5.0, 8.0, 2.0]], [[2.0, 3.0, 6.0], [6.0, 
1.0, 5.0], [4.0, 1.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 2.0, 2.0], [2.0, 5.0, 1.0], [4.0, 5.0, 7.0], [3.0, 6.0, 2.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 8.0]])+(1.-msk_val)*numpy.array([[5.0, 6.0]])
      arg[2:,2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 4.0], [2.0, 6.0, 8.0], [8.0, 5.0, 7.0], [2.0, 2.0, 3.0]], [[5.0, 8.0, 4.0], [4.0, 
3.0, 3.0], [1.0, 5.0, 2.0], [4.0, 7.0, 7.0]], [[4.0, 8.0, 5.0], [6.0, 4.0, 7.0], [1.0, 8.0, 2.0], [1.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 8.0], [8.0, 2.0, 6.0], [7.0, 6.0, 8.0], [5.0, 8.0, 2.0]], [[2.0, 3.0, 6.0], [6.0, 
1.0, 5.0], [4.0, 1.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 2.0, 2.0], [2.0, 5.0, 1.0], [5.0, 6.0, 7.0], [3.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 8.0, 4.0], [2.0, 8.0, 3.0], [8.0, 1.0, 5.0], [6.0, 1.0, 4.0]], [[5.0, 3.0, 1.0], [3.0, 
4.0, 5.0], [8.0, 2.0, 4.0], [7.0, 7.0, 8.0]], [[6.0, 2.0, 6.0], [5.0, 1.0, 6.0], [2.0, 2.0, 8.0], [7.0, 7.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 2.0, 2.0], [2.0, 6.0, 8.0], [2.0, 3.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 1.0, 2.0], [5.0, 
2.0, 3.0], [3.0, 8.0, 4.0], [8.0, 6.0, 7.0]], [[1.0, 8.0, 6.0], [4.0, 4.0, 5.0], [8.0, 8.0, 4.0], [7.0, 3.0, 7.0]]])
      val=numpy.array([[[8.0, 8.0, 3.0], [7.0, 6.0, 1.0]], [[1.0, 6.0, 3.0], [5.0, 8.0, 3.0]], [[5.0, 1.0, 6.0], [3.0, 7.0, 
5.0]]])
      arg[:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 4.0], [2.0, 8.0, 3.0], [8.0, 8.0, 3.0], [7.0, 6.0, 1.0]], [[5.0, 3.0, 1.0], [3.0, 
4.0, 5.0], [1.0, 6.0, 3.0], [5.0, 8.0, 3.0]], [[6.0, 2.0, 6.0], [5.0, 1.0, 6.0], [5.0, 1.0, 6.0], [3.0, 7.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 2.0], [2.0, 6.0, 8.0], [8.0, 8.0, 3.0], [7.0, 6.0, 1.0]], [[7.0, 1.0, 2.0], [5.0, 
2.0, 3.0], [1.0, 6.0, 3.0], [5.0, 8.0, 3.0]], [[1.0, 8.0, 6.0], [4.0, 4.0, 5.0], [5.0, 1.0, 6.0], [3.0, 7.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 8.0, 4.0], [2.0, 3.0, 8.0], [4.0, 1.0, 6.0], [1.0, 3.0, 2.0]], [[4.0, 6.0, 5.0], [7.0, 
6.0, 4.0], [8.0, 2.0, 2.0], [2.0, 1.0, 7.0]], [[6.0, 3.0, 1.0], [7.0, 2.0, 5.0], [7.0, 1.0, 8.0], [3.0, 3.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 5.0, 4.0], [4.0, 8.0, 2.0], [2.0, 8.0, 5.0], [8.0, 8.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 
2.0, 7.0], [3.0, 2.0, 1.0], [1.0, 7.0, 2.0]], [[5.0, 2.0, 2.0], [1.0, 6.0, 7.0], [8.0, 5.0, 3.0], [6.0, 7.0, 2.0]]])
      val=Data(numpy.array([[[1.0, 4.0, 2.0], [3.0, 8.0, 1.0]], [[6.0, 7.0, 7.0], [5.0, 5.0, 1.0]], [[8.0, 2.0, 5.0], [3.0, 
4.0, 8.0]]]),self.functionspace)
      arg[:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 4.0], [2.0, 3.0, 8.0], [1.0, 4.0, 2.0], [3.0, 8.0, 1.0]], [[4.0, 6.0, 5.0], [7.0, 
6.0, 4.0], [6.0, 7.0, 7.0], [5.0, 5.0, 1.0]], [[6.0, 3.0, 1.0], [7.0, 2.0, 5.0], [8.0, 2.0, 5.0], [3.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 4.0], [4.0, 8.0, 2.0], [1.0, 4.0, 2.0], [3.0, 8.0, 1.0]], [[6.0, 6.0, 4.0], [6.0, 
2.0, 7.0], [6.0, 7.0, 7.0], [5.0, 5.0, 1.0]], [[5.0, 2.0, 2.0], [1.0, 6.0, 7.0], [8.0, 2.0, 5.0], [3.0, 4.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 8.0, 3.0], [7.0, 5.0, 4.0], [7.0, 1.0, 7.0], [6.0, 7.0, 3.0]], [[5.0, 1.0, 8.0], [4.0, 
1.0, 5.0], [3.0, 8.0, 7.0], [8.0, 8.0, 8.0]], [[5.0, 7.0, 8.0], [2.0, 7.0, 2.0], [6.0, 4.0, 7.0], [4.0, 7.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 4.0, 5.0], [2.0, 8.0, 7.0], [8.0, 2.0, 4.0], [8.0, 7.0, 8.0]], [[6.0, 5.0, 3.0], [4.0, 
1.0, 8.0], [8.0, 4.0, 7.0], [4.0, 4.0, 4.0]], [[1.0, 1.0, 6.0], [2.0, 8.0, 2.0], [7.0, 7.0, 5.0], [1.0, 5.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 3.0, 8.0], [5.0, 8.0, 4.0]], [[4.0, 4.0, 1.0], [8.0, 1.0, 8.0]], [[8.0, 1.0, 2.0], [3.0, 
6.0, 4.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 6.0], [1.0, 3.0, 1.0]], [[5.0, 3.0, 2.0], [3.0, 7.0, 7.0]], [[2.0, 5.0, 
1.0], [3.0, 2.0, 7.0]]])
      arg[:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 3.0], [7.0, 5.0, 4.0], [1.0, 3.0, 8.0], [5.0, 8.0, 4.0]], [[5.0, 1.0, 8.0], [4.0, 
1.0, 5.0], [4.0, 4.0, 1.0], [8.0, 1.0, 8.0]], [[5.0, 7.0, 8.0], [2.0, 7.0, 2.0], [8.0, 1.0, 2.0], [3.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 5.0], [2.0, 8.0, 7.0], [8.0, 1.0, 6.0], [1.0, 3.0, 1.0]], [[6.0, 5.0, 3.0], [4.0, 
1.0, 8.0], [5.0, 3.0, 2.0], [3.0, 7.0, 7.0]], [[1.0, 1.0, 6.0], [2.0, 8.0, 2.0], [2.0, 5.0, 1.0], [3.0, 2.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 8.0], [8.0, 1.0, 4.0], [8.0, 2.0, 4.0], [5.0, 2.0, 5.0]], [[1.0, 3.0, 6.0], [6.0, 
4.0, 5.0], [3.0, 7.0, 3.0], [6.0, 5.0, 4.0]], [[4.0, 8.0, 5.0], [5.0, 1.0, 3.0], [7.0, 8.0, 1.0], [4.0, 7.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 3.0], [1.0, 6.0, 7.0], [5.0, 8.0, 2.0], [2.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], [3.0, 
3.0, 3.0], [8.0, 2.0, 1.0], [1.0, 6.0, 6.0]], [[7.0, 4.0, 3.0], [8.0, 6.0, 1.0], [6.0, 1.0, 2.0], [7.0, 5.0, 6.0]]])
      val=numpy.array([[[3.0, 5.0], [3.0, 1.0], [5.0, 2.0]], [[3.0, 5.0], [2.0, 4.0], [6.0, 2.0]]])
      arg[:2,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 8.0], [3.0, 1.0, 4.0], [5.0, 2.0, 4.0], [5.0, 2.0, 5.0]], [[3.0, 5.0, 6.0], [2.0, 
4.0, 5.0], [6.0, 2.0, 3.0], [6.0, 5.0, 4.0]], [[4.0, 8.0, 5.0], [5.0, 1.0, 3.0], [7.0, 8.0, 1.0], [4.0, 7.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 3.0], [3.0, 1.0, 7.0], [5.0, 2.0, 2.0], [2.0, 8.0, 2.0]], [[3.0, 5.0, 8.0], [2.0, 
4.0, 3.0], [6.0, 2.0, 1.0], [1.0, 6.0, 6.0]], [[7.0, 4.0, 3.0], [8.0, 6.0, 1.0], [6.0, 1.0, 2.0], [7.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 5.0, 4.0], [2.0, 6.0, 1.0], [1.0, 3.0, 8.0], [1.0, 2.0, 2.0]], [[7.0, 3.0, 4.0], [5.0, 
2.0, 4.0], [8.0, 5.0, 4.0], [2.0, 8.0, 1.0]], [[2.0, 3.0, 5.0], [6.0, 6.0, 2.0], [6.0, 3.0, 3.0], [2.0, 8.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 7.0, 8.0], [5.0, 7.0, 1.0], [6.0, 2.0, 2.0], [1.0, 5.0, 7.0]], [[6.0, 4.0, 1.0], [3.0, 
4.0, 6.0], [2.0, 4.0, 8.0], [6.0, 7.0, 5.0]], [[4.0, 4.0, 8.0], [8.0, 3.0, 4.0], [4.0, 1.0, 5.0], [3.0, 1.0, 8.0]]])
      val=Data(numpy.array([[[7.0, 6.0], [5.0, 1.0], [7.0, 3.0]], [[2.0, 7.0], [4.0, 8.0], [1.0, 2.0]]]),self.functionspace)
      arg[:2,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 4.0], [5.0, 1.0, 1.0], [7.0, 3.0, 8.0], [1.0, 2.0, 2.0]], [[2.0, 7.0, 4.0], [4.0, 
8.0, 4.0], [1.0, 2.0, 4.0], [2.0, 8.0, 1.0]], [[2.0, 3.0, 5.0], [6.0, 6.0, 2.0], [6.0, 3.0, 3.0], [2.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 8.0], [5.0, 1.0, 1.0], [7.0, 3.0, 2.0], [1.0, 5.0, 7.0]], [[2.0, 7.0, 1.0], [4.0, 
8.0, 6.0], [1.0, 2.0, 8.0], [6.0, 7.0, 5.0]], [[4.0, 4.0, 8.0], [8.0, 3.0, 4.0], [4.0, 1.0, 5.0], [3.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 7.0, 2.0], [1.0, 2.0, 8.0], [1.0, 1.0, 8.0], [6.0, 3.0, 7.0]], [[2.0, 2.0, 7.0], [8.0, 
2.0, 8.0], [8.0, 1.0, 1.0], [6.0, 7.0, 7.0]], [[1.0, 6.0, 2.0], [8.0, 4.0, 3.0], [8.0, 2.0, 8.0], [3.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 6.0], [2.0, 4.0, 8.0], [4.0, 8.0, 1.0], [4.0, 3.0, 2.0]], [[8.0, 4.0, 3.0], [5.0, 
8.0, 6.0], [1.0, 6.0, 4.0], [4.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [4.0, 3.0, 7.0], [5.0, 8.0, 3.0], [7.0, 1.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 3.0], [6.0, 8.0], [4.0, 7.0]], [[7.0, 1.0], [5.0, 5.0], [8.0, 
8.0]]])+(1.-msk_val)*numpy.array([[[5.0, 2.0], [2.0, 5.0], [8.0, 6.0]], [[3.0, 7.0], [1.0, 5.0], [5.0, 4.0]]])
      arg[:2,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 2.0], [6.0, 8.0, 8.0], [4.0, 7.0, 8.0], [6.0, 3.0, 7.0]], [[7.0, 1.0, 7.0], [5.0, 
5.0, 8.0], [8.0, 8.0, 1.0], [6.0, 7.0, 7.0]], [[1.0, 6.0, 2.0], [8.0, 4.0, 3.0], [8.0, 2.0, 8.0], [3.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 6.0], [2.0, 5.0, 8.0], [8.0, 6.0, 1.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 3.0], [1.0, 
5.0, 6.0], [5.0, 4.0, 4.0], [4.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [4.0, 3.0, 7.0], [5.0, 8.0, 3.0], [7.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 6.0], [6.0, 4.0, 2.0], [7.0, 4.0, 1.0], [6.0, 2.0, 8.0]], [[5.0, 7.0, 4.0], [1.0, 
4.0, 3.0], [2.0, 6.0, 4.0], [4.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [1.0, 8.0, 8.0], [4.0, 5.0, 1.0], [5.0, 3.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 4.0, 7.0], [5.0, 1.0, 3.0], [6.0, 2.0, 8.0], [8.0, 8.0, 3.0]], [[6.0, 4.0, 6.0], [2.0, 
7.0, 1.0], [3.0, 8.0, 4.0], [8.0, 8.0, 3.0]], [[7.0, 6.0, 3.0], [2.0, 1.0, 8.0], [4.0, 2.0, 1.0], [3.0, 2.0, 2.0]]])
      val=numpy.array([[3.0, 2.0, 3.0], [8.0, 4.0, 6.0]])
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 6.0], [6.0, 4.0, 2.0], [7.0, 4.0, 1.0], [6.0, 2.0, 8.0]], [[5.0, 7.0, 4.0], [3.0, 
2.0, 3.0], [8.0, 4.0, 6.0], [4.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [1.0, 8.0, 8.0], [4.0, 5.0, 1.0], [5.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 4.0, 7.0], [5.0, 1.0, 3.0], [6.0, 2.0, 8.0], [8.0, 8.0, 3.0]], [[6.0, 4.0, 6.0], [3.0, 
2.0, 3.0], [8.0, 4.0, 6.0], [8.0, 8.0, 3.0]], [[7.0, 6.0, 3.0], [2.0, 1.0, 8.0], [4.0, 2.0, 1.0], [3.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 1.0, 1.0], [4.0, 2.0, 6.0], [4.0, 2.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 8.0, 4.0], [5.0, 
1.0, 1.0], [1.0, 2.0, 4.0], [2.0, 3.0, 1.0]], [[7.0, 4.0, 8.0], [7.0, 7.0, 4.0], [5.0, 5.0, 5.0], [8.0, 7.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 5.0, 5.0], [3.0, 5.0, 6.0], [8.0, 3.0, 5.0], [3.0, 8.0, 6.0]], [[5.0, 5.0, 4.0], [7.0, 
7.0, 2.0], [8.0, 4.0, 1.0], [6.0, 4.0, 7.0]], [[6.0, 8.0, 7.0], [2.0, 8.0, 7.0], [7.0, 8.0, 7.0], [2.0, 6.0, 8.0]]])
      val=Data(numpy.array([[4.0, 3.0, 7.0], [3.0, 4.0, 6.0]]),self.functionspace)
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 1.0], [4.0, 2.0, 6.0], [4.0, 2.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 8.0, 4.0], [4.0, 
3.0, 7.0], [3.0, 4.0, 6.0], [2.0, 3.0, 1.0]], [[7.0, 4.0, 8.0], [7.0, 7.0, 4.0], [5.0, 5.0, 5.0], [8.0, 7.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 5.0], [3.0, 5.0, 6.0], [8.0, 3.0, 5.0], [3.0, 8.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 
3.0, 7.0], [3.0, 4.0, 6.0], [6.0, 4.0, 7.0]], [[6.0, 8.0, 7.0], [2.0, 8.0, 7.0], [7.0, 8.0, 7.0], [2.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 5.0, 4.0], [3.0, 3.0, 4.0], [1.0, 3.0, 1.0], [4.0, 6.0, 3.0]], [[7.0, 4.0, 7.0], [8.0, 
2.0, 4.0], [6.0, 1.0, 5.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 4.0], [6.0, 8.0, 7.0], [1.0, 1.0, 7.0], [4.0, 5.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 6.0, 1.0], [6.0, 1.0, 1.0], [4.0, 8.0, 2.0], [8.0, 2.0, 3.0]], [[5.0, 4.0, 7.0], [3.0, 
8.0, 7.0], [5.0, 5.0, 6.0], [1.0, 2.0, 1.0]], [[2.0, 1.0, 1.0], [2.0, 8.0, 2.0], [4.0, 7.0, 2.0], [5.0, 1.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 3.0, 1.0], [2.0, 7.0, 4.0]])+(1.-msk_val)*numpy.array([[5.0, 5.0, 4.0], [6.0, 4.0, 2.0]])
      arg[1,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 4.0], [3.0, 3.0, 4.0], [1.0, 3.0, 1.0], [4.0, 6.0, 3.0]], [[7.0, 4.0, 7.0], [5.0, 
3.0, 1.0], [2.0, 7.0, 4.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 4.0], [6.0, 8.0, 7.0], [1.0, 1.0, 7.0], [4.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 6.0, 1.0], [6.0, 1.0, 1.0], [4.0, 8.0, 2.0], [8.0, 2.0, 3.0]], [[5.0, 4.0, 7.0], [5.0, 
5.0, 4.0], [6.0, 4.0, 2.0], [1.0, 2.0, 1.0]], [[2.0, 1.0, 1.0], [2.0, 8.0, 2.0], [4.0, 7.0, 2.0], [5.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 5.0, 5.0], [7.0, 8.0, 2.0], [4.0, 3.0, 2.0], [2.0, 5.0, 5.0]], [[1.0, 1.0, 5.0], [3.0, 
7.0, 8.0], [7.0, 2.0, 4.0], [2.0, 1.0, 3.0]], [[8.0, 4.0, 2.0], [6.0, 4.0, 5.0], [4.0, 3.0, 5.0], [2.0, 7.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 8.0], [6.0, 8.0, 1.0], [1.0, 3.0, 1.0], [8.0, 1.0, 6.0]], [[5.0, 4.0, 3.0], [5.0, 
4.0, 6.0], [2.0, 7.0, 1.0], [8.0, 2.0, 8.0]], [[7.0, 3.0, 3.0], [6.0, 8.0, 6.0], [1.0, 3.0, 6.0], [6.0, 1.0, 5.0]]])
      val=numpy.array([[[1.0, 6.0, 5.0], [7.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 5.0, 8.0]]])
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 5.0, 5.0], [7.0, 8.0, 2.0], [4.0, 3.0, 2.0], [2.0, 5.0, 5.0]], [[1.0, 1.0, 5.0], [3.0, 
7.0, 8.0], [7.0, 2.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 5.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 8.0], [6.0, 8.0, 1.0], [1.0, 3.0, 1.0], [8.0, 1.0, 6.0]], [[5.0, 4.0, 3.0], [5.0, 
4.0, 6.0], [2.0, 7.0, 1.0], [8.0, 2.0, 8.0]], [[1.0, 6.0, 5.0], [7.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 4.0], [7.0, 8.0, 5.0], [8.0, 1.0, 6.0], [3.0, 4.0, 5.0]], [[8.0, 3.0, 4.0], [7.0, 
1.0, 7.0], [8.0, 3.0, 6.0], [3.0, 2.0, 8.0]], [[7.0, 8.0, 6.0], [8.0, 5.0, 4.0], [5.0, 4.0, 1.0], [3.0, 2.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 3.0, 2.0], [4.0, 6.0, 3.0], [5.0, 3.0, 4.0], [7.0, 6.0, 8.0]], [[8.0, 7.0, 8.0], [1.0, 
8.0, 5.0], [6.0, 4.0, 7.0], [2.0, 4.0, 5.0]], [[2.0, 7.0, 5.0], [1.0, 4.0, 4.0], [3.0, 4.0, 6.0], [7.0, 4.0, 8.0]]])
      val=Data(numpy.array([[[7.0, 1.0, 1.0], [6.0, 1.0, 6.0], [3.0, 1.0, 6.0], [2.0, 3.0, 7.0]]]),self.functionspace)
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 4.0], [7.0, 8.0, 5.0], [8.0, 1.0, 6.0], [3.0, 4.0, 5.0]], [[8.0, 3.0, 4.0], [7.0, 
1.0, 7.0], [8.0, 3.0, 6.0], [3.0, 2.0, 8.0]], [[7.0, 1.0, 1.0], [6.0, 1.0, 6.0], [3.0, 1.0, 6.0], [2.0, 3.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 2.0], [4.0, 6.0, 3.0], [5.0, 3.0, 4.0], [7.0, 6.0, 8.0]], [[8.0, 7.0, 8.0], [1.0, 
8.0, 5.0], [6.0, 4.0, 7.0], [2.0, 4.0, 5.0]], [[7.0, 1.0, 1.0], [6.0, 1.0, 6.0], [3.0, 1.0, 6.0], [2.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 8.0, 6.0], [1.0, 1.0, 6.0], [2.0, 7.0, 7.0], [4.0, 3.0, 4.0]], [[6.0, 8.0, 3.0], [8.0, 
6.0, 6.0], [5.0, 3.0, 7.0], [1.0, 2.0, 3.0]], [[2.0, 5.0, 4.0], [8.0, 4.0, 8.0], [3.0, 6.0, 2.0], [2.0, 4.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 5.0, 8.0], [7.0, 8.0, 7.0], [4.0, 6.0, 7.0], [5.0, 4.0, 2.0]], [[7.0, 3.0, 8.0], [5.0, 
4.0, 1.0], [3.0, 7.0, 8.0], [7.0, 4.0, 3.0]], [[2.0, 2.0, 3.0], [6.0, 3.0, 2.0], [8.0, 1.0, 7.0], [8.0, 7.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 1.0, 2.0], [2.0, 3.0, 1.0], [4.0, 4.0, 7.0], [6.0, 8.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[1.0, 1.0, 6.0], [5.0, 1.0, 8.0], [3.0, 7.0, 2.0], [7.0, 3.0, 8.0]]])
      arg[2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 8.0, 6.0], [1.0, 1.0, 6.0], [2.0, 7.0, 7.0], [4.0, 3.0, 4.0]], [[6.0, 8.0, 3.0], [8.0, 
6.0, 6.0], [5.0, 3.0, 7.0], [1.0, 2.0, 3.0]], [[8.0, 1.0, 2.0], [2.0, 3.0, 1.0], [4.0, 4.0, 7.0], [6.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 8.0], [7.0, 8.0, 7.0], [4.0, 6.0, 7.0], [5.0, 4.0, 2.0]], [[7.0, 3.0, 8.0], [5.0, 
4.0, 1.0], [3.0, 7.0, 8.0], [7.0, 4.0, 3.0]], [[1.0, 1.0, 6.0], [5.0, 1.0, 8.0], [3.0, 7.0, 2.0], [7.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 2.0, 5.0], [4.0, 7.0, 7.0], [1.0, 3.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 5.0, 7.0], [2.0, 
2.0, 7.0], [8.0, 6.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 4.0, 1.0], [5.0, 1.0, 7.0], [3.0, 7.0, 6.0], [4.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 7.0, 5.0], [6.0, 7.0, 1.0], [2.0, 4.0, 5.0], [6.0, 7.0, 6.0]], [[5.0, 1.0, 6.0], [2.0, 
1.0, 2.0], [6.0, 2.0, 8.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 5.0], [5.0, 2.0, 8.0], [7.0, 3.0, 2.0], [6.0, 5.0, 6.0]]])
      val=numpy.array([[[2.0, 6.0], [7.0, 8.0], [2.0, 3.0]], [[8.0, 5.0], [5.0, 5.0], [5.0, 8.0]], [[3.0, 1.0], [7.0, 7.0], 
[7.0, 3.0]]])
      arg[:,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 6.0, 5.0], [7.0, 8.0, 7.0], [2.0, 3.0, 4.0], [5.0, 5.0, 3.0]], [[8.0, 5.0, 7.0], [5.0, 
5.0, 7.0], [5.0, 8.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 1.0, 1.0], [7.0, 7.0, 7.0], [7.0, 3.0, 6.0], [4.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 5.0], [7.0, 8.0, 1.0], [2.0, 3.0, 5.0], [6.0, 7.0, 6.0]], [[8.0, 5.0, 6.0], [5.0, 
5.0, 2.0], [5.0, 8.0, 8.0], [4.0, 1.0, 1.0]], [[3.0, 1.0, 5.0], [7.0, 7.0, 8.0], [7.0, 3.0, 2.0], [6.0, 5.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 3.0, 6.0], [3.0, 4.0, 3.0], [3.0, 3.0, 1.0], [6.0, 3.0, 5.0]], [[8.0, 2.0, 2.0], [1.0, 
8.0, 1.0], [2.0, 4.0, 8.0], [5.0, 6.0, 6.0]], [[2.0, 7.0, 1.0], [5.0, 5.0, 2.0], [2.0, 5.0, 1.0], [2.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 4.0, 2.0], [8.0, 8.0, 3.0], [7.0, 8.0, 6.0], [5.0, 6.0, 6.0]], [[8.0, 1.0, 1.0], [2.0, 
7.0, 7.0], [1.0, 4.0, 2.0], [5.0, 1.0, 1.0]], [[4.0, 3.0, 4.0], [2.0, 5.0, 4.0], [1.0, 8.0, 4.0], [4.0, 5.0, 4.0]]])
      val=Data(numpy.array([[[4.0, 5.0], [5.0, 2.0], [6.0, 6.0]], [[2.0, 7.0], [2.0, 4.0], [6.0, 4.0]], [[4.0, 7.0], [2.0, 
7.0], [5.0, 7.0]]]),self.functionspace)
      arg[:,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 5.0, 6.0], [5.0, 2.0, 3.0], [6.0, 6.0, 1.0], [6.0, 3.0, 5.0]], [[2.0, 7.0, 2.0], [2.0, 
4.0, 1.0], [6.0, 4.0, 8.0], [5.0, 6.0, 6.0]], [[4.0, 7.0, 1.0], [2.0, 7.0, 2.0], [5.0, 7.0, 1.0], [2.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 2.0], [5.0, 2.0, 3.0], [6.0, 6.0, 6.0], [5.0, 6.0, 6.0]], [[2.0, 7.0, 1.0], [2.0, 
4.0, 7.0], [6.0, 4.0, 2.0], [5.0, 1.0, 1.0]], [[4.0, 7.0, 4.0], [2.0, 7.0, 4.0], [5.0, 7.0, 4.0], [4.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 3.0, 5.0], [6.0, 3.0, 5.0], [8.0, 5.0, 8.0], [2.0, 5.0, 3.0]], [[1.0, 2.0, 3.0], [5.0, 
3.0, 3.0], [1.0, 2.0, 1.0], [8.0, 5.0, 8.0]], [[1.0, 8.0, 2.0], [5.0, 6.0, 1.0], [3.0, 2.0, 7.0], [2.0, 4.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 5.0, 1.0], [6.0, 7.0, 7.0], [5.0, 3.0, 4.0], [2.0, 6.0, 7.0]], [[2.0, 6.0, 1.0], [5.0, 
8.0, 8.0], [4.0, 1.0, 1.0], [2.0, 7.0, 5.0]], [[7.0, 2.0, 5.0], [8.0, 5.0, 5.0], [6.0, 7.0, 8.0], [8.0, 3.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 5.0], [8.0, 3.0], [1.0, 8.0]], [[4.0, 8.0], [5.0, 1.0], [3.0, 3.0]], [[8.0, 1.0], [8.0, 
8.0], [4.0, 7.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0], [7.0, 5.0], [2.0, 6.0]], [[8.0, 6.0], [8.0, 5.0], [2.0, 6.0]], 
[[5.0, 6.0], [1.0, 5.0], [1.0, 5.0]]])
      arg[:,:3,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 5.0], [8.0, 3.0, 5.0], [1.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[4.0, 8.0, 3.0], [5.0, 
1.0, 3.0], [3.0, 3.0, 1.0], [8.0, 5.0, 8.0]], [[8.0, 1.0, 2.0], [8.0, 8.0, 1.0], [4.0, 7.0, 7.0], [2.0, 4.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 1.0], [7.0, 5.0, 7.0], [2.0, 6.0, 4.0], [2.0, 6.0, 7.0]], [[8.0, 6.0, 1.0], [8.0, 
5.0, 8.0], [2.0, 6.0, 1.0], [2.0, 7.0, 5.0]], [[5.0, 6.0, 5.0], [1.0, 5.0, 5.0], [1.0, 5.0, 8.0], [8.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 7.0, 6.0], [7.0, 6.0, 1.0], [7.0, 1.0, 2.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 8.0], [2.0, 
2.0, 3.0], [8.0, 6.0, 6.0], [3.0, 8.0, 1.0]], [[7.0, 1.0, 2.0], [1.0, 7.0, 1.0], [2.0, 8.0, 7.0], [7.0, 4.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 6.0, 5.0], [8.0, 4.0, 7.0], [5.0, 3.0, 2.0], [7.0, 3.0, 5.0]], [[1.0, 4.0, 7.0], [7.0, 
4.0, 8.0], [6.0, 2.0, 5.0], [3.0, 3.0, 7.0]], [[4.0, 4.0, 6.0], [8.0, 1.0, 8.0], [3.0, 7.0, 4.0], [7.0, 4.0, 2.0]]])
      val=numpy.array([[[1.0], [3.0], [3.0]], [[2.0], [3.0], [1.0]]])
      arg[:2,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 1.0, 6.0], [7.0, 3.0, 1.0], [7.0, 3.0, 2.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 8.0], [2.0, 
3.0, 3.0], [8.0, 1.0, 6.0], [3.0, 8.0, 1.0]], [[7.0, 1.0, 2.0], [1.0, 7.0, 1.0], [2.0, 8.0, 7.0], [7.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 5.0], [8.0, 3.0, 7.0], [5.0, 3.0, 2.0], [7.0, 3.0, 5.0]], [[1.0, 2.0, 7.0], [7.0, 
3.0, 8.0], [6.0, 1.0, 5.0], [3.0, 3.0, 7.0]], [[4.0, 4.0, 6.0], [8.0, 1.0, 8.0], [3.0, 7.0, 4.0], [7.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 3.0, 6.0], [3.0, 5.0, 3.0], [3.0, 2.0, 3.0], [4.0, 1.0, 8.0]], [[7.0, 1.0, 1.0], [3.0, 
7.0, 3.0], [4.0, 8.0, 2.0], [8.0, 8.0, 2.0]], [[2.0, 5.0, 8.0], [8.0, 4.0, 6.0], [1.0, 8.0, 3.0], [6.0, 1.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 3.0], [5.0, 5.0, 2.0], [4.0, 1.0, 4.0], [1.0, 8.0, 7.0]], [[1.0, 1.0, 7.0], [7.0, 
7.0, 4.0], [3.0, 4.0, 2.0], [7.0, 8.0, 3.0]], [[4.0, 4.0, 5.0], [4.0, 2.0, 7.0], [6.0, 4.0, 7.0], [4.0, 7.0, 6.0]]])
      val=Data(numpy.array([[[2.0], [6.0], [2.0]], [[7.0], [8.0], [1.0]]]),self.functionspace)
      arg[:2,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 6.0], [3.0, 6.0, 3.0], [3.0, 2.0, 3.0], [4.0, 1.0, 8.0]], [[7.0, 7.0, 1.0], [3.0, 
8.0, 3.0], [4.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[2.0, 5.0, 8.0], [8.0, 4.0, 6.0], [1.0, 8.0, 3.0], [6.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 3.0], [5.0, 6.0, 2.0], [4.0, 2.0, 4.0], [1.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [7.0, 
8.0, 4.0], [3.0, 1.0, 2.0], [7.0, 8.0, 3.0]], [[4.0, 4.0, 5.0], [4.0, 2.0, 7.0], [6.0, 4.0, 7.0], [4.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 5.0, 5.0], [8.0, 1.0, 7.0], [1.0, 4.0, 2.0], [5.0, 3.0, 3.0]], [[7.0, 5.0, 3.0], [3.0, 
8.0, 2.0], [8.0, 1.0, 4.0], [8.0, 8.0, 1.0]], [[4.0, 7.0, 7.0], [5.0, 5.0, 4.0], [8.0, 8.0, 6.0], [6.0, 8.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 5.0], [8.0, 7.0, 8.0], [4.0, 4.0, 3.0], [6.0, 6.0, 7.0]], [[1.0, 8.0, 4.0], [8.0, 
8.0, 3.0], [6.0, 6.0, 6.0], [2.0, 4.0, 5.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 6.0], [6.0, 3.0, 2.0], [3.0, 2.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0], [7.0], [1.0]], [[1.0], [4.0], [3.0]]])+(1.-msk_val)*numpy.array([[[8.0], [3.0], [3.0]], 
[[8.0], [1.0], [1.0]]])
      arg[:2,:3,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 3.0, 5.0], [8.0, 7.0, 7.0], [1.0, 1.0, 2.0], [5.0, 3.0, 3.0]], [[7.0, 1.0, 3.0], [3.0, 
4.0, 2.0], [8.0, 3.0, 4.0], [8.0, 8.0, 1.0]], [[4.0, 7.0, 7.0], [5.0, 5.0, 4.0], [8.0, 8.0, 6.0], [6.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 8.0, 5.0], [8.0, 3.0, 8.0], [4.0, 3.0, 3.0], [6.0, 6.0, 7.0]], [[1.0, 8.0, 4.0], [8.0, 
1.0, 3.0], [6.0, 1.0, 6.0], [2.0, 4.0, 5.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 6.0], [6.0, 3.0, 2.0], [3.0, 2.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 5.0, 6.0], [2.0, 6.0, 3.0], [5.0, 6.0, 8.0], [5.0, 4.0, 6.0]], [[4.0, 5.0, 1.0], [2.0, 
8.0, 1.0], [3.0, 4.0, 6.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 4.0], [4.0, 4.0, 3.0], [5.0, 3.0, 7.0], [8.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 7.0, 7.0], [8.0, 4.0, 6.0], [8.0, 5.0, 7.0], [1.0, 8.0, 6.0]], [[8.0, 4.0, 1.0], [7.0, 
7.0, 3.0], [3.0, 8.0, 7.0], [1.0, 6.0, 4.0]], [[1.0, 6.0, 2.0], [5.0, 3.0, 4.0], [2.0, 4.0, 1.0], [6.0, 7.0, 2.0]]])
      val=numpy.array([[[3.0, 6.0, 8.0], [5.0, 8.0, 1.0]]])
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 6.0], [2.0, 6.0, 3.0], [5.0, 6.0, 8.0], [5.0, 4.0, 6.0]], [[4.0, 5.0, 1.0], [2.0, 
8.0, 1.0], [3.0, 4.0, 6.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 4.0], [4.0, 4.0, 3.0], [3.0, 6.0, 8.0], [5.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 7.0], [8.0, 4.0, 6.0], [8.0, 5.0, 7.0], [1.0, 8.0, 6.0]], [[8.0, 4.0, 1.0], [7.0, 
7.0, 3.0], [3.0, 8.0, 7.0], [1.0, 6.0, 4.0]], [[1.0, 6.0, 2.0], [5.0, 3.0, 4.0], [3.0, 6.0, 8.0], [5.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 4.0, 8.0], [1.0, 4.0, 4.0], [6.0, 7.0, 3.0], [7.0, 3.0, 6.0]], [[4.0, 1.0, 8.0], [5.0, 
7.0, 4.0], [6.0, 1.0, 4.0], [4.0, 3.0, 6.0]], [[2.0, 5.0, 4.0], [7.0, 5.0, 2.0], [5.0, 1.0, 8.0], [1.0, 8.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 4.0, 5.0], [6.0, 6.0, 4.0], [1.0, 5.0, 3.0], [6.0, 2.0, 3.0]], [[8.0, 4.0, 7.0], [5.0, 
2.0, 2.0], [4.0, 4.0, 4.0], [7.0, 8.0, 6.0]], [[5.0, 8.0, 7.0], [5.0, 3.0, 6.0], [2.0, 1.0, 8.0], [4.0, 7.0, 1.0]]])
      val=Data(numpy.array([[[2.0, 3.0, 1.0], [6.0, 2.0, 8.0]]]),self.functionspace)
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 4.0, 8.0], [1.0, 4.0, 4.0], [6.0, 7.0, 3.0], [7.0, 3.0, 6.0]], [[4.0, 1.0, 8.0], [5.0, 
7.0, 4.0], [6.0, 1.0, 4.0], [4.0, 3.0, 6.0]], [[2.0, 5.0, 4.0], [7.0, 5.0, 2.0], [2.0, 3.0, 1.0], [6.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 4.0, 5.0], [6.0, 6.0, 4.0], [1.0, 5.0, 3.0], [6.0, 2.0, 3.0]], [[8.0, 4.0, 7.0], [5.0, 
2.0, 2.0], [4.0, 4.0, 4.0], [7.0, 8.0, 6.0]], [[5.0, 8.0, 7.0], [5.0, 3.0, 6.0], [2.0, 3.0, 1.0], [6.0, 2.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 5.0], [1.0, 6.0, 5.0], [1.0, 1.0, 8.0], [2.0, 6.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 
6.0, 8.0], [7.0, 2.0, 3.0], [4.0, 1.0, 5.0]], [[1.0, 8.0, 4.0], [2.0, 8.0, 5.0], [1.0, 7.0, 6.0], [8.0, 8.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 7.0, 1.0], [3.0, 8.0, 3.0], [4.0, 7.0, 2.0], [4.0, 6.0, 2.0]], [[5.0, 7.0, 5.0], [2.0, 
3.0, 2.0], [5.0, 1.0, 2.0], [3.0, 4.0, 8.0]], [[5.0, 8.0, 1.0], [7.0, 8.0, 1.0], [4.0, 3.0, 8.0], [8.0, 2.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 3.0, 7.0], [7.0, 7.0, 1.0]]])+(1.-msk_val)*numpy.array([[[7.0, 4.0, 3.0], [7.0, 7.0, 
6.0]]])
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 5.0], [1.0, 6.0, 5.0], [1.0, 1.0, 8.0], [2.0, 6.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 
6.0, 8.0], [7.0, 2.0, 3.0], [4.0, 1.0, 5.0]], [[1.0, 8.0, 4.0], [2.0, 8.0, 5.0], [4.0, 3.0, 7.0], [7.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 7.0, 1.0], [3.0, 8.0, 3.0], [4.0, 7.0, 2.0], [4.0, 6.0, 2.0]], [[5.0, 7.0, 5.0], [2.0, 
3.0, 2.0], [5.0, 1.0, 2.0], [3.0, 4.0, 8.0]], [[5.0, 8.0, 1.0], [7.0, 8.0, 1.0], [7.0, 4.0, 3.0], [7.0, 7.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 3.0, 6.0], [3.0, 5.0, 2.0], [3.0, 6.0, 3.0], [4.0, 5.0, 4.0]], [[8.0, 8.0, 1.0], [6.0, 
7.0, 7.0], [8.0, 6.0, 5.0], [2.0, 6.0, 4.0]], [[5.0, 3.0, 4.0], [1.0, 5.0, 8.0], [4.0, 7.0, 7.0], [7.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 8.0, 5.0], [6.0, 5.0, 6.0], [5.0, 6.0, 6.0], [4.0, 6.0, 6.0]], [[3.0, 4.0, 3.0], [1.0, 
3.0, 5.0], [1.0, 8.0, 5.0], [7.0, 1.0, 6.0]], [[8.0, 6.0, 3.0], [8.0, 2.0, 5.0], [5.0, 7.0, 5.0], [2.0, 5.0, 8.0]]])
      val=numpy.array([[[1.0], [8.0], [3.0], [4.0]], [[3.0], [4.0], [8.0], [7.0]], [[5.0], [5.0], [7.0], [7.0]]])
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 6.0], [3.0, 8.0, 2.0], [3.0, 3.0, 3.0], [4.0, 4.0, 4.0]], [[8.0, 3.0, 1.0], [6.0, 
4.0, 7.0], [8.0, 8.0, 5.0], [2.0, 7.0, 4.0]], [[5.0, 5.0, 4.0], [1.0, 5.0, 8.0], [4.0, 7.0, 7.0], [7.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 5.0], [6.0, 8.0, 6.0], [5.0, 3.0, 6.0], [4.0, 4.0, 6.0]], [[3.0, 3.0, 3.0], [1.0, 
4.0, 5.0], [1.0, 8.0, 5.0], [7.0, 7.0, 6.0]], [[8.0, 5.0, 3.0], [8.0, 5.0, 5.0], [5.0, 7.0, 5.0], [2.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 2.0, 3.0], [2.0, 1.0, 8.0], [6.0, 6.0, 6.0], [6.0, 7.0, 2.0]], [[4.0, 2.0, 3.0], [5.0, 
7.0, 7.0], [8.0, 1.0, 1.0], [8.0, 3.0, 8.0]], [[2.0, 7.0, 7.0], [6.0, 8.0, 6.0], [8.0, 8.0, 6.0], [6.0, 5.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 1.0, 5.0], [4.0, 6.0, 1.0], [3.0, 8.0, 8.0], [6.0, 1.0, 5.0]], [[1.0, 4.0, 2.0], [5.0, 
2.0, 6.0], [2.0, 4.0, 6.0], [2.0, 7.0, 3.0]], [[5.0, 3.0, 6.0], [3.0, 5.0, 5.0], [8.0, 7.0, 4.0], [2.0, 4.0, 2.0]]])
      val=Data(numpy.array([[[6.0], [7.0], [3.0], [8.0]], [[1.0], [8.0], [4.0], [3.0]], [[7.0], [2.0], [2.0], 
[1.0]]]),self.functionspace)
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 3.0], [2.0, 7.0, 8.0], [6.0, 3.0, 6.0], [6.0, 8.0, 2.0]], [[4.0, 1.0, 3.0], [5.0, 
8.0, 7.0], [8.0, 4.0, 1.0], [8.0, 3.0, 8.0]], [[2.0, 7.0, 7.0], [6.0, 2.0, 6.0], [8.0, 2.0, 6.0], [6.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 6.0, 5.0], [4.0, 7.0, 1.0], [3.0, 3.0, 8.0], [6.0, 8.0, 5.0]], [[1.0, 1.0, 2.0], [5.0, 
8.0, 6.0], [2.0, 4.0, 6.0], [2.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [3.0, 2.0, 5.0], [8.0, 2.0, 4.0], [2.0, 1.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 2.0, 5.0], [4.0, 1.0, 5.0], [2.0, 7.0, 2.0], [6.0, 5.0, 5.0]], [[5.0, 8.0, 1.0], [3.0, 
5.0, 5.0], [1.0, 1.0, 1.0], [3.0, 7.0, 2.0]], [[7.0, 3.0, 6.0], [6.0, 4.0, 8.0], [2.0, 2.0, 4.0], [1.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 1.0, 4.0], [7.0, 2.0, 6.0], [1.0, 3.0, 3.0], [3.0, 5.0, 1.0]], [[1.0, 1.0, 1.0], [3.0, 
7.0, 4.0], [5.0, 2.0, 3.0], [7.0, 8.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 4.0, 3.0], [4.0, 5.0, 4.0], [6.0, 4.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0], [4.0], [2.0], [8.0]], [[7.0], [5.0], [1.0], [6.0]], [[7.0], [6.0], [3.0], 
[5.0]]])+(1.-msk_val)*numpy.array([[[7.0], [8.0], [2.0], [8.0]], [[6.0], [1.0], [2.0], [4.0]], [[8.0], [4.0], [1.0], [1.0]]])
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 2.0, 5.0], [4.0, 4.0, 5.0], [2.0, 2.0, 2.0], [6.0, 8.0, 5.0]], [[5.0, 7.0, 1.0], [3.0, 
5.0, 5.0], [1.0, 1.0, 1.0], [3.0, 6.0, 2.0]], [[7.0, 7.0, 6.0], [6.0, 6.0, 8.0], [2.0, 3.0, 4.0], [1.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 7.0, 4.0], [7.0, 8.0, 6.0], [1.0, 2.0, 3.0], [3.0, 8.0, 1.0]], [[1.0, 6.0, 1.0], [3.0, 
1.0, 4.0], [5.0, 2.0, 3.0], [7.0, 4.0, 6.0]], [[5.0, 8.0, 4.0], [4.0, 4.0, 3.0], [4.0, 1.0, 4.0], [6.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 3.0, 2.0], [1.0, 6.0, 7.0], [8.0, 7.0, 4.0], [3.0, 2.0, 6.0]], [[8.0, 2.0, 7.0], [7.0, 
3.0, 7.0], [1.0, 8.0, 5.0], [4.0, 3.0, 5.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 3.0], [2.0, 8.0, 8.0], [5.0, 7.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 2.0, 5.0], [5.0, 4.0, 7.0], [2.0, 4.0, 8.0], [4.0, 4.0, 1.0]], [[4.0, 2.0, 4.0], [2.0, 
3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 6.0, 1.0]], [[3.0, 6.0, 4.0], [4.0, 7.0, 5.0], [7.0, 2.0, 6.0], [3.0, 1.0, 1.0]]])
      val=numpy.array([[[2.0, 7.0, 8.0], [3.0, 3.0, 6.0]]])
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 2.0], [1.0, 6.0, 7.0], [8.0, 7.0, 4.0], [3.0, 2.0, 6.0]], [[8.0, 2.0, 7.0], [2.0, 
7.0, 8.0], [3.0, 3.0, 6.0], [4.0, 3.0, 5.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 3.0], [2.0, 8.0, 8.0], [5.0, 7.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 5.0], [5.0, 4.0, 7.0], [2.0, 4.0, 8.0], [4.0, 4.0, 1.0]], [[4.0, 2.0, 4.0], [2.0, 
7.0, 8.0], [3.0, 3.0, 6.0], [6.0, 6.0, 1.0]], [[3.0, 6.0, 4.0], [4.0, 7.0, 5.0], [7.0, 2.0, 6.0], [3.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 2.0, 1.0], [2.0, 8.0, 6.0], [6.0, 3.0, 1.0], [2.0, 3.0, 8.0]], [[1.0, 7.0, 3.0], [4.0, 
3.0, 6.0], [1.0, 7.0, 2.0], [2.0, 7.0, 6.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [3.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 7.0], [4.0, 3.0, 1.0], [3.0, 4.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 2.0, 4.0], [8.0, 
3.0, 1.0], [2.0, 3.0, 6.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 8.0], [3.0, 3.0, 4.0], [6.0, 2.0, 1.0], [1.0, 7.0, 2.0]]])
      val=Data(numpy.array([[[7.0, 4.0, 2.0], [2.0, 5.0, 7.0]]]),self.functionspace)
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 1.0], [2.0, 8.0, 6.0], [6.0, 3.0, 1.0], [2.0, 3.0, 8.0]], [[1.0, 7.0, 3.0], [7.0, 
4.0, 2.0], [2.0, 5.0, 7.0], [2.0, 7.0, 6.0]], [[6.0, 5.0, 2.0], [1.0, 8.0, 2.0], [8.0, 3.0, 8.0], [3.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 7.0], [4.0, 3.0, 1.0], [3.0, 4.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 2.0, 4.0], [7.0, 
4.0, 2.0], [2.0, 5.0, 7.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 8.0], [3.0, 3.0, 4.0], [6.0, 2.0, 1.0], [1.0, 7.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 1.0], [5.0, 7.0, 7.0], [4.0, 2.0, 4.0], [7.0, 3.0, 2.0]], [[8.0, 4.0, 4.0], [4.0, 
6.0, 7.0], [3.0, 5.0, 8.0], [5.0, 3.0, 8.0]], [[5.0, 6.0, 4.0], [7.0, 6.0, 8.0], [7.0, 8.0, 4.0], [4.0, 1.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 4.0, 3.0], [7.0, 1.0, 7.0], [1.0, 7.0, 3.0], [3.0, 8.0, 1.0]], [[4.0, 2.0, 2.0], [6.0, 
2.0, 1.0], [3.0, 6.0, 7.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 1.0], [7.0, 2.0, 6.0], [5.0, 5.0, 7.0], [8.0, 6.0, 2.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 6.0, 3.0], [3.0, 6.0, 7.0]]])+(1.-msk_val)*numpy.array([[[5.0, 6.0, 1.0], [4.0, 3.0, 
5.0]]])
      arg[1:2,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 1.0], [5.0, 7.0, 7.0], [4.0, 2.0, 4.0], [7.0, 3.0, 2.0]], [[8.0, 4.0, 4.0], [6.0, 
6.0, 3.0], [3.0, 6.0, 7.0], [5.0, 3.0, 8.0]], [[5.0, 6.0, 4.0], [7.0, 6.0, 8.0], [7.0, 8.0, 4.0], [4.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 4.0, 3.0], [7.0, 1.0, 7.0], [1.0, 7.0, 3.0], [3.0, 8.0, 1.0]], [[4.0, 2.0, 2.0], [5.0, 
6.0, 1.0], [4.0, 3.0, 5.0], [8.0, 1.0, 3.0]], [[3.0, 6.0, 1.0], [7.0, 2.0, 6.0], [5.0, 5.0, 7.0], [8.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 8.0], [7.0, 3.0, 4.0], [7.0, 3.0, 1.0], [4.0, 2.0, 4.0]], [[2.0, 6.0, 4.0], [6.0, 
6.0, 3.0], [2.0, 3.0, 1.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 8.0], [4.0, 7.0, 4.0], [3.0, 7.0, 6.0], [6.0, 6.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 1.0, 5.0], [2.0, 6.0, 3.0], [4.0, 6.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 5.0, 8.0], [3.0, 
8.0, 3.0], [2.0, 3.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 5.0, 5.0], [7.0, 3.0, 1.0], [5.0, 2.0, 4.0], [1.0, 3.0, 5.0]]])
      val=numpy.array([[7.0], [2.0]])
      arg[:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 8.0], [7.0, 3.0, 4.0], [7.0, 7.0, 1.0], [4.0, 2.0, 4.0]], [[2.0, 6.0, 4.0], [6.0, 
6.0, 3.0], [2.0, 2.0, 1.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 8.0], [4.0, 7.0, 4.0], [3.0, 7.0, 6.0], [6.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 1.0, 5.0], [2.0, 6.0, 3.0], [4.0, 7.0, 3.0], [7.0, 8.0, 7.0]], [[2.0, 5.0, 8.0], [3.0, 
8.0, 3.0], [2.0, 2.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 5.0, 5.0], [7.0, 3.0, 1.0], [5.0, 2.0, 4.0], [1.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 7.0, 2.0], [2.0, 4.0, 5.0], [4.0, 6.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 7.0, 7.0], [6.0, 
8.0, 2.0], [5.0, 7.0, 2.0], [3.0, 5.0, 5.0]], [[6.0, 6.0, 7.0], [8.0, 3.0, 4.0], [6.0, 4.0, 6.0], [8.0, 4.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 1.0, 1.0], [5.0, 7.0, 6.0], [2.0, 1.0, 4.0], [1.0, 4.0, 7.0]], [[7.0, 7.0, 1.0], [6.0, 
3.0, 2.0], [1.0, 1.0, 7.0], [7.0, 3.0, 6.0]], [[5.0, 3.0, 6.0], [8.0, 2.0, 5.0], [3.0, 1.0, 1.0], [8.0, 6.0, 8.0]]])
      val=Data(numpy.array([[8.0], [7.0]]),self.functionspace)
      arg[:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 2.0], [2.0, 4.0, 5.0], [4.0, 8.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 7.0, 7.0], [6.0, 
8.0, 2.0], [5.0, 7.0, 2.0], [3.0, 5.0, 5.0]], [[6.0, 6.0, 7.0], [8.0, 3.0, 4.0], [6.0, 4.0, 6.0], [8.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 1.0, 1.0], [5.0, 7.0, 6.0], [2.0, 8.0, 4.0], [1.0, 4.0, 7.0]], [[7.0, 7.0, 1.0], [6.0, 
3.0, 2.0], [1.0, 7.0, 7.0], [7.0, 3.0, 6.0]], [[5.0, 3.0, 6.0], [8.0, 2.0, 5.0], [3.0, 1.0, 1.0], [8.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 6.0, 1.0], [2.0, 5.0, 6.0], [4.0, 2.0, 2.0], [1.0, 3.0, 4.0]], [[6.0, 2.0, 4.0], [1.0, 
6.0, 5.0], [4.0, 4.0, 3.0], [4.0, 7.0, 2.0]], [[6.0, 1.0, 7.0], [7.0, 6.0, 2.0], [3.0, 5.0, 8.0], [3.0, 6.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 8.0], [1.0, 4.0, 2.0], [1.0, 3.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 6.0, 5.0], [5.0, 
1.0, 6.0], [1.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 1.0, 3.0], [2.0, 1.0, 1.0], [6.0, 2.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0], [4.0]])+(1.-msk_val)*numpy.array([[6.0], [5.0]])
      arg[:2,2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 6.0, 1.0], [2.0, 5.0, 6.0], [4.0, 2.0, 2.0], [1.0, 3.0, 4.0]], [[6.0, 2.0, 4.0], [1.0, 
6.0, 5.0], [4.0, 4.0, 3.0], [4.0, 7.0, 2.0]], [[6.0, 1.0, 7.0], [7.0, 6.0, 2.0], [3.0, 5.0, 8.0], [3.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 8.0], [1.0, 4.0, 2.0], [1.0, 6.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 6.0, 5.0], [5.0, 
1.0, 6.0], [1.0, 5.0, 6.0], [1.0, 4.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 1.0, 3.0], [2.0, 1.0, 1.0], [6.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 6.0, 6.0], [8.0, 7.0, 1.0], [1.0, 4.0, 5.0], [8.0, 5.0, 2.0]], [[3.0, 1.0, 6.0], [2.0, 
2.0, 4.0], [1.0, 8.0, 1.0], [8.0, 4.0, 3.0]], [[2.0, 6.0, 6.0], [6.0, 2.0, 5.0], [8.0, 1.0, 3.0], [8.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 5.0, 5.0], [1.0, 7.0, 8.0], [4.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[4.0, 4.0, 8.0], [7.0, 
7.0, 8.0], [6.0, 6.0, 6.0], [7.0, 1.0, 6.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 6.0], [7.0, 5.0, 2.0], [1.0, 8.0, 1.0]]])
      val=numpy.array([[[5.0, 6.0, 2.0], [4.0, 4.0, 6.0], [4.0, 6.0, 8.0], [8.0, 6.0, 7.0]]])
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 6.0], [8.0, 7.0, 1.0], [1.0, 4.0, 5.0], [8.0, 5.0, 2.0]], [[5.0, 6.0, 2.0], [4.0, 
4.0, 6.0], [4.0, 6.0, 8.0], [8.0, 6.0, 7.0]], [[2.0, 6.0, 6.0], [6.0, 2.0, 5.0], [8.0, 1.0, 3.0], [8.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 5.0, 5.0], [1.0, 7.0, 8.0], [4.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[5.0, 6.0, 2.0], [4.0, 
4.0, 6.0], [4.0, 6.0, 8.0], [8.0, 6.0, 7.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 6.0], [7.0, 5.0, 2.0], [1.0, 8.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 6.0, 8.0], [7.0, 4.0, 2.0], [7.0, 3.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 6.0, 7.0], [1.0, 
5.0, 3.0], [8.0, 6.0, 5.0], [7.0, 3.0, 4.0]], [[7.0, 5.0, 8.0], [6.0, 5.0, 5.0], [8.0, 5.0, 6.0], [5.0, 1.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 8.0, 1.0], [7.0, 6.0, 8.0], [2.0, 3.0, 3.0], [5.0, 3.0, 7.0]], [[3.0, 1.0, 4.0], [2.0, 
3.0, 7.0], [6.0, 2.0, 1.0], [7.0, 6.0, 4.0]], [[1.0, 6.0, 6.0], [6.0, 8.0, 7.0], [5.0, 5.0, 7.0], [5.0, 8.0, 7.0]]])
      val=Data(numpy.array([[[8.0, 1.0, 4.0], [5.0, 1.0, 7.0], [1.0, 8.0, 4.0], [2.0, 4.0, 8.0]]]),self.functionspace)
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 8.0], [7.0, 4.0, 2.0], [7.0, 3.0, 4.0], [3.0, 8.0, 6.0]], [[8.0, 1.0, 4.0], [5.0, 
1.0, 7.0], [1.0, 8.0, 4.0], [2.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [6.0, 5.0, 5.0], [8.0, 5.0, 6.0], [5.0, 1.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 8.0, 1.0], [7.0, 6.0, 8.0], [2.0, 3.0, 3.0], [5.0, 3.0, 7.0]], [[8.0, 1.0, 4.0], [5.0, 
1.0, 7.0], [1.0, 8.0, 4.0], [2.0, 4.0, 8.0]], [[1.0, 6.0, 6.0], [6.0, 8.0, 7.0], [5.0, 5.0, 7.0], [5.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 4.0, 5.0], [8.0, 8.0, 7.0], [7.0, 3.0, 2.0], [3.0, 1.0, 3.0]], [[2.0, 3.0, 2.0], [5.0, 
4.0, 3.0], [5.0, 7.0, 4.0], [8.0, 4.0, 5.0]], [[2.0, 1.0, 6.0], [4.0, 6.0, 8.0], [3.0, 7.0, 2.0], [7.0, 8.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 3.0, 2.0], [5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [5.0, 5.0, 4.0]], [[7.0, 5.0, 3.0], [6.0, 
5.0, 4.0], [2.0, 3.0, 3.0], [8.0, 5.0, 3.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 5.0], [3.0, 4.0, 2.0], [6.0, 6.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 8.0, 2.0], [7.0, 1.0, 2.0], [7.0, 1.0, 1.0], [3.0, 2.0, 
3.0]]])+(1.-msk_val)*numpy.array([[[4.0, 1.0, 8.0], [8.0, 6.0, 7.0], [8.0, 3.0, 7.0], [8.0, 5.0, 2.0]]])
      arg[1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 5.0], [8.0, 8.0, 7.0], [7.0, 3.0, 2.0], [3.0, 1.0, 3.0]], [[3.0, 8.0, 2.0], [7.0, 
1.0, 2.0], [7.0, 1.0, 1.0], [3.0, 2.0, 3.0]], [[2.0, 1.0, 6.0], [4.0, 6.0, 8.0], [3.0, 7.0, 2.0], [7.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 3.0, 2.0], [5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [5.0, 5.0, 4.0]], [[4.0, 1.0, 8.0], [8.0, 
6.0, 7.0], [8.0, 3.0, 7.0], [8.0, 5.0, 2.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 5.0], [3.0, 4.0, 2.0], [6.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 2.0], [4.0, 5.0, 8.0], [7.0, 3.0, 3.0], [7.0, 2.0, 8.0]], [[6.0, 7.0, 2.0], [3.0, 
7.0, 7.0], [1.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[2.0, 1.0, 7.0], [4.0, 8.0, 6.0], [8.0, 4.0, 7.0], [1.0, 5.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 4.0, 8.0], [8.0, 5.0, 1.0], [7.0, 4.0, 5.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 8.0], [4.0, 
8.0, 8.0], [2.0, 1.0, 7.0], [1.0, 3.0, 3.0]], [[6.0, 7.0, 5.0], [3.0, 7.0, 5.0], [8.0, 2.0, 6.0], [4.0, 6.0, 4.0]]])
      val=numpy.array([[[5.0, 4.0, 6.0], [6.0, 8.0, 3.0], [6.0, 8.0, 1.0], [3.0, 4.0, 3.0]], [[2.0, 8.0, 4.0], [8.0, 5.0, 7.0], 
[4.0, 6.0, 1.0], [1.0, 1.0, 7.0]]])
      arg[:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 4.0, 6.0], [6.0, 8.0, 3.0], [6.0, 8.0, 1.0], [3.0, 4.0, 3.0]], [[2.0, 8.0, 4.0], [8.0, 
5.0, 7.0], [4.0, 6.0, 1.0], [1.0, 1.0, 7.0]], [[2.0, 1.0, 7.0], [4.0, 8.0, 6.0], [8.0, 4.0, 7.0], [1.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 6.0], [6.0, 8.0, 3.0], [6.0, 8.0, 1.0], [3.0, 4.0, 3.0]], [[2.0, 8.0, 4.0], [8.0, 
5.0, 7.0], [4.0, 6.0, 1.0], [1.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [3.0, 7.0, 5.0], [8.0, 2.0, 6.0], [4.0, 6.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 2.0, 3.0], [4.0, 8.0, 8.0], [2.0, 3.0, 3.0], [1.0, 1.0, 8.0]], [[2.0, 4.0, 8.0], [5.0, 
2.0, 1.0], [8.0, 5.0, 5.0], [5.0, 1.0, 6.0]], [[7.0, 2.0, 4.0], [6.0, 7.0, 3.0], [7.0, 4.0, 4.0], [4.0, 4.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 8.0, 1.0], [1.0, 4.0, 3.0], [4.0, 7.0, 3.0], [7.0, 3.0, 6.0]], [[2.0, 8.0, 1.0], [3.0, 
3.0, 3.0], [7.0, 5.0, 3.0], [8.0, 7.0, 8.0]], [[5.0, 4.0, 2.0], [1.0, 7.0, 6.0], [8.0, 5.0, 3.0], [7.0, 4.0, 5.0]]])
      val=Data(numpy.array([[[8.0, 2.0, 6.0], [7.0, 2.0, 6.0], [2.0, 8.0, 2.0], [4.0, 2.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 6.0, 
6.0], [8.0, 8.0, 4.0], [7.0, 2.0, 6.0]]]),self.functionspace)
      arg[:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 6.0], [7.0, 2.0, 6.0], [2.0, 8.0, 2.0], [4.0, 2.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 
6.0, 6.0], [8.0, 8.0, 4.0], [7.0, 2.0, 6.0]], [[7.0, 2.0, 4.0], [6.0, 7.0, 3.0], [7.0, 4.0, 4.0], [4.0, 4.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 6.0], [7.0, 2.0, 6.0], [2.0, 8.0, 2.0], [4.0, 2.0, 4.0]], [[1.0, 5.0, 3.0], [4.0, 
6.0, 6.0], [8.0, 8.0, 4.0], [7.0, 2.0, 6.0]], [[5.0, 4.0, 2.0], [1.0, 7.0, 6.0], [8.0, 5.0, 3.0], [7.0, 4.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 7.0, 2.0], [1.0, 7.0, 8.0], [8.0, 7.0, 1.0], [3.0, 2.0, 6.0]], [[4.0, 8.0, 7.0], [4.0, 
4.0, 7.0], [1.0, 6.0, 2.0], [8.0, 5.0, 2.0]], [[7.0, 3.0, 2.0], [7.0, 2.0, 5.0], [3.0, 2.0, 4.0], [7.0, 1.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 7.0], [2.0, 5.0, 3.0], [2.0, 7.0, 6.0], [6.0, 3.0, 2.0]], [[4.0, 4.0, 1.0], [3.0, 
6.0, 7.0], [4.0, 5.0, 7.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 5.0], [3.0, 7.0, 8.0], [5.0, 2.0, 5.0], [5.0, 8.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 
5.0, 3.0], [2.0, 2.0, 3.0], [8.0, 7.0, 8.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 2.0], [5.0, 1.0, 4.0], [4.0, 5.0, 5.0], 
[6.0, 7.0, 5.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 5.0], [8.0, 8.0, 7.0], [4.0, 5.0, 7.0]]])
      arg[:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 3.0], [7.0, 4.0, 3.0], [4.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 
5.0, 3.0], [2.0, 2.0, 3.0], [8.0, 7.0, 8.0]], [[7.0, 3.0, 2.0], [7.0, 2.0, 5.0], [3.0, 2.0, 4.0], [7.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 2.0], [5.0, 1.0, 4.0], [4.0, 5.0, 5.0], [6.0, 7.0, 5.0]], [[6.0, 4.0, 4.0], [2.0, 
5.0, 5.0], [8.0, 8.0, 7.0], [4.0, 5.0, 7.0]], [[1.0, 6.0, 5.0], [3.0, 7.0, 8.0], [5.0, 2.0, 5.0], [5.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_ULL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 5.0, 2.0], [8.0, 1.0, 8.0], [1.0, 7.0, 1.0], [7.0, 4.0, 1.0]], [[2.0, 6.0, 2.0], [2.0, 
7.0, 1.0], [5.0, 2.0, 5.0], [3.0, 4.0, 6.0]], [[6.0, 7.0, 6.0], [3.0, 6.0, 2.0], [8.0, 5.0, 8.0], [7.0, 7.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 4.0, 3.0], [4.0, 1.0, 7.0], [8.0, 8.0, 5.0], [5.0, 6.0, 3.0]], [[2.0, 6.0, 3.0], [3.0, 
6.0, 1.0], [8.0, 6.0, 2.0], [2.0, 4.0, 6.0]], [[2.0, 8.0, 3.0], [2.0, 2.0, 5.0], [5.0, 8.0, 8.0], [3.0, 6.0, 7.0]]])
      val=numpy.array([[[3.0], [2.0]], [[2.0], [6.0]]])
      arg[:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 2.0], [8.0, 1.0, 8.0], [1.0, 7.0, 3.0], [7.0, 4.0, 2.0]], [[2.0, 6.0, 2.0], [2.0, 
7.0, 1.0], [5.0, 2.0, 2.0], [3.0, 4.0, 6.0]], [[6.0, 7.0, 6.0], [3.0, 6.0, 2.0], [8.0, 5.0, 8.0], [7.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0, 3.0], [4.0, 1.0, 7.0], [8.0, 8.0, 3.0], [5.0, 6.0, 2.0]], [[2.0, 6.0, 3.0], [3.0, 
6.0, 1.0], [8.0, 6.0, 2.0], [2.0, 4.0, 6.0]], [[2.0, 8.0, 3.0], [2.0, 2.0, 5.0], [5.0, 8.0, 8.0], [3.0, 6.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_ULL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 3.0], [1.0, 3.0, 1.0], [5.0, 2.0, 2.0], [6.0, 7.0, 7.0]], [[8.0, 7.0, 7.0], [5.0, 
6.0, 1.0], [2.0, 6.0, 1.0], [5.0, 2.0, 5.0]], [[3.0, 4.0, 6.0], [6.0, 1.0, 1.0], [7.0, 4.0, 4.0], [2.0, 4.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 4.0], [2.0, 6.0, 3.0], [2.0, 5.0, 5.0], [5.0, 1.0, 8.0]], [[5.0, 2.0, 3.0], [8.0, 
5.0, 3.0], [3.0, 8.0, 6.0], [1.0, 2.0, 7.0]], [[5.0, 7.0, 3.0], [1.0, 2.0, 2.0], [7.0, 5.0, 4.0], [3.0, 3.0, 5.0]]])
      val=Data(numpy.array([[[2.0], [3.0]], [[2.0], [7.0]]]),self.functionspace)
      arg[:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 3.0], [1.0, 3.0, 1.0], [5.0, 2.0, 2.0], [6.0, 7.0, 3.0]], [[8.0, 7.0, 7.0], [5.0, 
6.0, 1.0], [2.0, 6.0, 2.0], [5.0, 2.0, 7.0]], [[3.0, 4.0, 6.0], [6.0, 1.0, 1.0], [7.0, 4.0, 4.0], [2.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 4.0], [2.0, 6.0, 3.0], [2.0, 5.0, 2.0], [5.0, 1.0, 3.0]], [[5.0, 2.0, 3.0], [8.0, 
5.0, 3.0], [3.0, 8.0, 2.0], [1.0, 2.0, 7.0]], [[5.0, 7.0, 3.0], [1.0, 2.0, 2.0], [7.0, 5.0, 4.0], [3.0, 3.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_ULL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 3.0, 6.0], [3.0, 4.0, 5.0], [8.0, 4.0, 8.0], [5.0, 8.0, 6.0]], [[5.0, 6.0, 1.0], [6.0, 
3.0, 7.0], [6.0, 5.0, 4.0], [4.0, 8.0, 6.0]], [[4.0, 7.0, 5.0], [5.0, 3.0, 8.0], [7.0, 3.0, 2.0], [6.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 8.0], [1.0, 8.0, 6.0], [8.0, 8.0, 5.0], [8.0, 3.0, 6.0]], [[3.0, 5.0, 4.0], [1.0, 
1.0, 3.0], [2.0, 7.0, 4.0], [4.0, 2.0, 6.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 4.0], [1.0, 2.0, 5.0], [3.0, 5.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0], [2.0]], [[5.0], [3.0]]])+(1.-msk_val)*numpy.array([[[7.0], [5.0]], [[8.0], [6.0]]])
      arg[:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 6.0], [3.0, 4.0, 5.0], [8.0, 4.0, 7.0], [5.0, 8.0, 2.0]], [[5.0, 6.0, 1.0], [6.0, 
3.0, 7.0], [6.0, 5.0, 5.0], [4.0, 8.0, 3.0]], [[4.0, 7.0, 5.0], [5.0, 3.0, 8.0], [7.0, 3.0, 2.0], [6.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 8.0], [1.0, 8.0, 6.0], [8.0, 8.0, 7.0], [8.0, 3.0, 5.0]], [[3.0, 5.0, 4.0], [1.0, 
1.0, 3.0], [2.0, 7.0, 8.0], [4.0, 2.0, 6.0]], [[1.0, 4.0, 5.0], [3.0, 3.0, 4.0], [1.0, 2.0, 5.0], [3.0, 5.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 6.0, 2.0], [6.0, 2.0, 4.0], [1.0, 6.0, 1.0], [1.0, 3.0, 6.0]], [[6.0, 1.0, 5.0], [6.0, 
4.0, 4.0], [6.0, 1.0, 4.0], [1.0, 3.0, 2.0]], [[8.0, 8.0, 6.0], [6.0, 2.0, 2.0], [7.0, 4.0, 4.0], [2.0, 4.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 6.0, 4.0], [7.0, 8.0, 1.0], [5.0, 7.0, 7.0], [4.0, 5.0, 7.0]], [[6.0, 2.0, 8.0], [1.0, 
2.0, 7.0], [6.0, 3.0, 1.0], [6.0, 5.0, 2.0]], [[1.0, 3.0, 6.0], [7.0, 3.0, 3.0], [3.0, 5.0, 7.0], [6.0, 3.0, 8.0]]])
      val=numpy.array([[[3.0], [5.0]]])
      arg[1:2,1:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 2.0], [6.0, 2.0, 4.0], [1.0, 6.0, 1.0], [1.0, 3.0, 6.0]], [[6.0, 1.0, 5.0], [6.0, 
4.0, 3.0], [6.0, 1.0, 5.0], [1.0, 3.0, 2.0]], [[8.0, 8.0, 6.0], [6.0, 2.0, 2.0], [7.0, 4.0, 4.0], [2.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 4.0], [7.0, 8.0, 1.0], [5.0, 7.0, 7.0], [4.0, 5.0, 7.0]], [[6.0, 2.0, 8.0], [1.0, 
2.0, 3.0], [6.0, 3.0, 5.0], [6.0, 5.0, 2.0]], [[1.0, 3.0, 6.0], [7.0, 3.0, 3.0], [3.0, 5.0, 7.0], [6.0, 3.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 5.0], [6.0, 6.0, 3.0], [2.0, 1.0, 8.0], [3.0, 7.0, 3.0]], [[5.0, 1.0, 3.0], [3.0, 
5.0, 8.0], [3.0, 8.0, 6.0], [6.0, 6.0, 4.0]], [[2.0, 7.0, 4.0], [6.0, 6.0, 5.0], [6.0, 2.0, 8.0], [1.0, 5.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 3.0, 6.0], [3.0, 3.0, 4.0], [3.0, 7.0, 4.0], [4.0, 1.0, 6.0]], [[5.0, 8.0, 4.0], [8.0, 
8.0, 8.0], [3.0, 5.0, 7.0], [2.0, 5.0, 8.0]], [[5.0, 8.0, 8.0], [6.0, 8.0, 3.0], [1.0, 8.0, 3.0], [1.0, 7.0, 4.0]]])
      val=Data(numpy.array([[[8.0], [3.0]]]),self.functionspace)
      arg[1:2,1:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 5.0], [6.0, 6.0, 3.0], [2.0, 1.0, 8.0], [3.0, 7.0, 3.0]], [[5.0, 1.0, 3.0], [3.0, 
5.0, 8.0], [3.0, 8.0, 3.0], [6.0, 6.0, 4.0]], [[2.0, 7.0, 4.0], [6.0, 6.0, 5.0], [6.0, 2.0, 8.0], [1.0, 5.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 3.0, 6.0], [3.0, 3.0, 4.0], [3.0, 7.0, 4.0], [4.0, 1.0, 6.0]], [[5.0, 8.0, 4.0], [8.0, 
8.0, 8.0], [3.0, 5.0, 3.0], [2.0, 5.0, 8.0]], [[5.0, 8.0, 8.0], [6.0, 8.0, 3.0], [1.0, 8.0, 3.0], [1.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 1.0, 1.0], [1.0, 3.0, 2.0], [6.0, 4.0, 5.0], [6.0, 2.0, 5.0]], [[7.0, 3.0, 2.0], [2.0, 
7.0, 2.0], [7.0, 2.0, 7.0], [7.0, 1.0, 1.0]], [[8.0, 1.0, 3.0], [3.0, 4.0, 1.0], [6.0, 5.0, 5.0], [2.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 6.0, 3.0], [2.0, 4.0, 1.0], [4.0, 1.0, 7.0], [4.0, 2.0, 8.0]], [[3.0, 1.0, 1.0], [3.0, 
2.0, 2.0], [3.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[2.0, 7.0, 7.0], [5.0, 1.0, 6.0], [2.0, 3.0, 5.0], [1.0, 2.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0], [8.0]]])+(1.-msk_val)*numpy.array([[[6.0], [2.0]]])
      arg[1:2,1:3,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 1.0, 1.0], [1.0, 3.0, 2.0], [6.0, 4.0, 5.0], [6.0, 2.0, 5.0]], [[7.0, 3.0, 2.0], [2.0, 
7.0, 6.0], [7.0, 2.0, 8.0], [7.0, 1.0, 1.0]], [[8.0, 1.0, 3.0], [3.0, 4.0, 1.0], [6.0, 5.0, 5.0], [2.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 6.0, 3.0], [2.0, 4.0, 1.0], [4.0, 1.0, 7.0], [4.0, 2.0, 8.0]], [[3.0, 1.0, 1.0], [3.0, 
2.0, 6.0], [3.0, 4.0, 2.0], [6.0, 1.0, 6.0]], [[2.0, 7.0, 7.0], [5.0, 1.0, 6.0], [2.0, 3.0, 5.0], [1.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 8.0, 5.0], [6.0, 2.0, 8.0], [4.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[1.0, 2.0, 7.0], [2.0, 
3.0, 8.0], [8.0, 1.0, 2.0], [3.0, 5.0, 5.0]], [[8.0, 3.0, 4.0], [2.0, 8.0, 2.0], [1.0, 6.0, 2.0], [6.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 5.0, 1.0], [6.0, 8.0, 7.0], [4.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[5.0, 1.0, 1.0], [3.0, 
4.0, 1.0], [7.0, 5.0, 5.0], [2.0, 7.0, 7.0]], [[4.0, 1.0, 1.0], [8.0, 3.0, 2.0], [6.0, 1.0, 6.0], [3.0, 8.0, 4.0]]])
      val=numpy.array([[[7.0, 7.0, 5.0], [6.0, 8.0, 2.0], [3.0, 6.0, 5.0]]])
      arg[2:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 5.0], [6.0, 2.0, 8.0], [4.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[1.0, 2.0, 7.0], [2.0, 
3.0, 8.0], [8.0, 1.0, 2.0], [3.0, 5.0, 5.0]], [[7.0, 7.0, 5.0], [6.0, 8.0, 2.0], [3.0, 6.0, 5.0], [6.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 5.0, 1.0], [6.0, 8.0, 7.0], [4.0, 5.0, 2.0], [8.0, 2.0, 2.0]], [[5.0, 1.0, 1.0], [3.0, 
4.0, 1.0], [7.0, 5.0, 5.0], [2.0, 7.0, 7.0]], [[7.0, 7.0, 5.0], [6.0, 8.0, 2.0], [3.0, 6.0, 5.0], [3.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 1.0, 2.0], [2.0, 7.0, 8.0], [2.0, 6.0, 7.0], [8.0, 1.0, 8.0]], [[4.0, 3.0, 6.0], [3.0, 
8.0, 1.0], [7.0, 1.0, 2.0], [6.0, 3.0, 6.0]], [[1.0, 2.0, 8.0], [7.0, 7.0, 8.0], [8.0, 5.0, 4.0], [7.0, 8.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 5.0, 3.0], [6.0, 5.0, 7.0], [1.0, 5.0, 1.0], [6.0, 2.0, 7.0]], [[5.0, 6.0, 1.0], [1.0, 
5.0, 7.0], [6.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[8.0, 2.0, 3.0], [5.0, 5.0, 6.0], [2.0, 2.0, 4.0], [8.0, 2.0, 2.0]]])
      val=Data(numpy.array([[[4.0, 3.0, 6.0], [1.0, 4.0, 2.0], [1.0, 1.0, 5.0]]]),self.functionspace)
      arg[2:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 1.0, 2.0], [2.0, 7.0, 8.0], [2.0, 6.0, 7.0], [8.0, 1.0, 8.0]], [[4.0, 3.0, 6.0], [3.0, 
8.0, 1.0], [7.0, 1.0, 2.0], [6.0, 3.0, 6.0]], [[4.0, 3.0, 6.0], [1.0, 4.0, 2.0], [1.0, 1.0, 5.0], [7.0, 8.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 3.0], [6.0, 5.0, 7.0], [1.0, 5.0, 1.0], [6.0, 2.0, 7.0]], [[5.0, 6.0, 1.0], [1.0, 
5.0, 7.0], [6.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[4.0, 3.0, 6.0], [1.0, 4.0, 2.0], [1.0, 1.0, 5.0], [8.0, 2.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 4.0, 6.0], [8.0, 8.0, 1.0], [8.0, 4.0, 4.0], [2.0, 5.0, 1.0]], [[1.0, 1.0, 5.0], [4.0, 
2.0, 8.0], [1.0, 6.0, 1.0], [3.0, 1.0, 1.0]], [[2.0, 5.0, 3.0], [2.0, 4.0, 1.0], [4.0, 6.0, 3.0], [2.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 2.0], [1.0, 4.0, 2.0], [5.0, 6.0, 5.0], [2.0, 2.0, 5.0]], [[7.0, 8.0, 4.0], [7.0, 
5.0, 2.0], [7.0, 3.0, 8.0], [3.0, 8.0, 1.0]], [[1.0, 2.0, 7.0], [2.0, 4.0, 1.0], [8.0, 7.0, 6.0], [6.0, 8.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 1.0, 8.0], [8.0, 1.0, 7.0], [4.0, 6.0, 5.0]]])+(1.-msk_val)*numpy.array([[[7.0, 8.0, 
7.0], [7.0, 5.0, 8.0], [2.0, 5.0, 8.0]]])
      arg[2:,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 6.0], [8.0, 8.0, 1.0], [8.0, 4.0, 4.0], [2.0, 5.0, 1.0]], [[1.0, 1.0, 5.0], [4.0, 
2.0, 8.0], [1.0, 6.0, 1.0], [3.0, 1.0, 1.0]], [[2.0, 1.0, 8.0], [8.0, 1.0, 7.0], [4.0, 6.0, 5.0], [2.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 2.0], [1.0, 4.0, 2.0], [5.0, 6.0, 5.0], [2.0, 2.0, 5.0]], [[7.0, 8.0, 4.0], [7.0, 
5.0, 2.0], [7.0, 3.0, 8.0], [3.0, 8.0, 1.0]], [[7.0, 8.0, 7.0], [7.0, 5.0, 8.0], [2.0, 5.0, 8.0], [6.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 7.0, 6.0], [5.0, 7.0, 4.0], [3.0, 5.0, 2.0], [7.0, 7.0, 8.0]], [[1.0, 4.0, 8.0], [4.0, 
2.0, 7.0], [8.0, 2.0, 5.0], [8.0, 1.0, 1.0]], [[4.0, 5.0, 8.0], [5.0, 3.0, 7.0], [6.0, 8.0, 2.0], [4.0, 7.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 2.0, 6.0], [3.0, 1.0, 5.0], [7.0, 7.0, 5.0], [1.0, 5.0, 3.0]], [[8.0, 1.0, 6.0], [8.0, 
5.0, 5.0], [6.0, 5.0, 8.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 3.0], [1.0, 3.0, 7.0], [7.0, 7.0, 2.0], [2.0, 1.0, 5.0]]])
      val=numpy.array([[5.0, 6.0, 8.0], [1.0, 6.0, 3.0], [4.0, 6.0, 6.0], [1.0, 2.0, 4.0]])
      arg[1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 7.0, 6.0], [5.0, 7.0, 4.0], [3.0, 5.0, 2.0], [7.0, 7.0, 8.0]], [[5.0, 6.0, 8.0], [1.0, 
6.0, 3.0], [4.0, 6.0, 6.0], [1.0, 2.0, 4.0]], [[4.0, 5.0, 8.0], [5.0, 3.0, 7.0], [6.0, 8.0, 2.0], [4.0, 7.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 2.0, 6.0], [3.0, 1.0, 5.0], [7.0, 7.0, 5.0], [1.0, 5.0, 3.0]], [[5.0, 6.0, 8.0], [1.0, 
6.0, 3.0], [4.0, 6.0, 6.0], [1.0, 2.0, 4.0]], [[5.0, 7.0, 3.0], [1.0, 3.0, 7.0], [7.0, 7.0, 2.0], [2.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 1.0, 4.0], [1.0, 8.0, 5.0], [4.0, 7.0, 1.0], [6.0, 5.0, 8.0]], [[6.0, 4.0, 4.0], [8.0, 
2.0, 5.0], [7.0, 8.0, 4.0], [3.0, 3.0, 5.0]], [[6.0, 2.0, 6.0], [6.0, 8.0, 2.0], [7.0, 1.0, 7.0], [7.0, 8.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 2.0, 4.0], [3.0, 6.0, 4.0], [6.0, 3.0, 7.0], [2.0, 4.0, 2.0]], [[2.0, 3.0, 4.0], [8.0, 
7.0, 7.0], [4.0, 3.0, 4.0], [3.0, 5.0, 5.0]], [[2.0, 3.0, 1.0], [6.0, 2.0, 5.0], [7.0, 5.0, 2.0], [8.0, 1.0, 4.0]]])
      val=Data(numpy.array([[1.0, 1.0, 7.0], [3.0, 4.0, 2.0], [8.0, 4.0, 2.0], [6.0, 3.0, 8.0]]),self.functionspace)
      arg[1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 1.0, 4.0], [1.0, 8.0, 5.0], [4.0, 7.0, 1.0], [6.0, 5.0, 8.0]], [[1.0, 1.0, 7.0], [3.0, 
4.0, 2.0], [8.0, 4.0, 2.0], [6.0, 3.0, 8.0]], [[6.0, 2.0, 6.0], [6.0, 8.0, 2.0], [7.0, 1.0, 7.0], [7.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 2.0, 4.0], [3.0, 6.0, 4.0], [6.0, 3.0, 7.0], [2.0, 4.0, 2.0]], [[1.0, 1.0, 7.0], [3.0, 
4.0, 2.0], [8.0, 4.0, 2.0], [6.0, 3.0, 8.0]], [[2.0, 3.0, 1.0], [6.0, 2.0, 5.0], [7.0, 5.0, 2.0], [8.0, 1.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 8.0, 2.0], [7.0, 5.0, 6.0], [1.0, 3.0, 3.0], [2.0, 1.0, 2.0]], [[8.0, 3.0, 1.0], [3.0, 
2.0, 2.0], [8.0, 2.0, 1.0], [2.0, 3.0, 4.0]], [[5.0, 8.0, 8.0], [2.0, 6.0, 3.0], [8.0, 2.0, 6.0], [7.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 3.0, 4.0], [3.0, 2.0, 6.0], [1.0, 3.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 2.0, 7.0], [8.0, 
7.0, 4.0], [8.0, 2.0, 8.0], [8.0, 4.0, 6.0]], [[4.0, 5.0, 7.0], [4.0, 7.0, 7.0], [3.0, 5.0, 5.0], [3.0, 7.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 1.0, 5.0], [5.0, 6.0, 4.0], [4.0, 6.0, 6.0], [2.0, 7.0, 
7.0]])+(1.-msk_val)*numpy.array([[3.0, 3.0, 7.0], [4.0, 7.0, 5.0], [6.0, 6.0, 1.0], [6.0, 8.0, 2.0]])
      arg[1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 2.0], [7.0, 5.0, 6.0], [1.0, 3.0, 3.0], [2.0, 1.0, 2.0]], [[2.0, 1.0, 5.0], [5.0, 
6.0, 4.0], [4.0, 6.0, 6.0], [2.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [2.0, 6.0, 3.0], [8.0, 2.0, 6.0], [7.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 4.0], [3.0, 2.0, 6.0], [1.0, 3.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 3.0, 7.0], [4.0, 
7.0, 5.0], [6.0, 6.0, 1.0], [6.0, 8.0, 2.0]], [[4.0, 5.0, 7.0], [4.0, 7.0, 7.0], [3.0, 5.0, 5.0], [3.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 7.0], [1.0, 5.0, 2.0], [1.0, 4.0, 4.0], [2.0, 8.0, 3.0]], [[8.0, 5.0, 7.0], [6.0, 
2.0, 7.0], [7.0, 4.0, 8.0], [8.0, 6.0, 4.0]], [[5.0, 8.0, 3.0], [7.0, 8.0, 1.0], [1.0, 3.0, 8.0], [7.0, 5.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 3.0, 3.0], [3.0, 6.0, 5.0], [8.0, 5.0, 4.0], [7.0, 5.0, 6.0]], [[6.0, 2.0, 1.0], [5.0, 
1.0, 3.0], [3.0, 8.0, 7.0], [7.0, 4.0, 5.0]], [[4.0, 6.0, 8.0], [1.0, 8.0, 7.0], [7.0, 5.0, 6.0], [1.0, 6.0, 5.0]]])
      val=numpy.array([[[5.0, 5.0], [1.0, 6.0]]])
      arg[1:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 7.0], [1.0, 5.0, 2.0], [1.0, 4.0, 4.0], [2.0, 8.0, 3.0]], [[8.0, 5.0, 7.0], [6.0, 
2.0, 7.0], [5.0, 5.0, 8.0], [1.0, 6.0, 4.0]], [[5.0, 8.0, 3.0], [7.0, 8.0, 1.0], [1.0, 3.0, 8.0], [7.0, 5.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 3.0], [3.0, 6.0, 5.0], [8.0, 5.0, 4.0], [7.0, 5.0, 6.0]], [[6.0, 2.0, 1.0], [5.0, 
1.0, 3.0], [5.0, 5.0, 7.0], [1.0, 6.0, 5.0]], [[4.0, 6.0, 8.0], [1.0, 8.0, 7.0], [7.0, 5.0, 6.0], [1.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 6.0, 1.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0], [8.0, 3.0, 3.0]], [[2.0, 3.0, 2.0], [3.0, 
8.0, 4.0], [4.0, 7.0, 5.0], [3.0, 6.0, 6.0]], [[1.0, 1.0, 7.0], [3.0, 2.0, 8.0], [6.0, 1.0, 1.0], [8.0, 5.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 7.0], [7.0, 2.0, 2.0], [6.0, 7.0, 6.0], [4.0, 6.0, 2.0]], [[8.0, 8.0, 6.0], [4.0, 
8.0, 8.0], [4.0, 7.0, 7.0], [5.0, 5.0, 2.0]], [[2.0, 1.0, 4.0], [6.0, 3.0, 7.0], [4.0, 5.0, 6.0], [2.0, 3.0, 1.0]]])
      val=Data(numpy.array([[[1.0, 3.0], [3.0, 2.0]]]),self.functionspace)
      arg[1:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 1.0], [7.0, 2.0, 8.0], [6.0, 3.0, 6.0], [8.0, 3.0, 3.0]], [[2.0, 3.0, 2.0], [3.0, 
8.0, 4.0], [1.0, 3.0, 5.0], [3.0, 2.0, 6.0]], [[1.0, 1.0, 7.0], [3.0, 2.0, 8.0], [6.0, 1.0, 1.0], [8.0, 5.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 7.0], [7.0, 2.0, 2.0], [6.0, 7.0, 6.0], [4.0, 6.0, 2.0]], [[8.0, 8.0, 6.0], [4.0, 
8.0, 8.0], [1.0, 3.0, 7.0], [3.0, 2.0, 2.0]], [[2.0, 1.0, 4.0], [6.0, 3.0, 7.0], [4.0, 5.0, 6.0], [2.0, 3.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 8.0, 4.0], [2.0, 6.0, 1.0], [1.0, 2.0, 1.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 8.0], [8.0, 
2.0, 4.0], [1.0, 7.0, 4.0], [8.0, 4.0, 4.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 2.0], [7.0, 2.0, 2.0], [2.0, 6.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 2.0, 5.0], [7.0, 2.0, 6.0], [4.0, 5.0, 1.0], [2.0, 5.0, 4.0]], [[3.0, 8.0, 4.0], [1.0, 
5.0, 7.0], [3.0, 3.0, 1.0], [4.0, 5.0, 8.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 6.0], [2.0, 3.0, 5.0], [3.0, 7.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 4.0], [6.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 7.0], [5.0, 4.0]]])
      arg[1:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 4.0], [2.0, 6.0, 1.0], [1.0, 2.0, 1.0], [1.0, 7.0, 3.0]], [[3.0, 4.0, 8.0], [8.0, 
2.0, 4.0], [7.0, 4.0, 4.0], [6.0, 3.0, 4.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 2.0], [7.0, 2.0, 2.0], [2.0, 6.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 5.0], [7.0, 2.0, 6.0], [4.0, 5.0, 1.0], [2.0, 5.0, 4.0]], [[3.0, 8.0, 4.0], [1.0, 
5.0, 7.0], [1.0, 7.0, 1.0], [5.0, 4.0, 8.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 6.0], [2.0, 3.0, 5.0], [3.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 8.0, 2.0], [1.0, 8.0, 6.0], [4.0, 4.0, 5.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 2.0], [1.0, 
1.0, 5.0], [2.0, 6.0, 1.0], [8.0, 5.0, 3.0]], [[1.0, 6.0, 3.0], [7.0, 5.0, 5.0], [2.0, 1.0, 7.0], [7.0, 8.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 2.0, 7.0], [5.0, 3.0, 3.0], [1.0, 4.0, 8.0], [1.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [5.0, 
3.0, 1.0], [3.0, 3.0, 1.0], [1.0, 4.0, 8.0]], [[5.0, 6.0, 1.0], [2.0, 2.0, 1.0], [2.0, 2.0, 8.0], [5.0, 1.0, 8.0]]])
      val=numpy.array([[[8.0], [2.0]]])
      arg[1:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 8.0, 2.0], [1.0, 8.0, 6.0], [4.0, 4.0, 5.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 2.0], [1.0, 
1.0, 5.0], [2.0, 6.0, 8.0], [8.0, 5.0, 2.0]], [[1.0, 6.0, 3.0], [7.0, 5.0, 5.0], [2.0, 1.0, 7.0], [7.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 7.0], [5.0, 3.0, 3.0], [1.0, 4.0, 8.0], [1.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [5.0, 
3.0, 1.0], [3.0, 3.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 1.0], [2.0, 2.0, 1.0], [2.0, 2.0, 8.0], [5.0, 1.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 3.0, 3.0], [1.0, 8.0, 3.0], [3.0, 6.0, 3.0], [6.0, 4.0, 5.0]], [[1.0, 6.0, 6.0], [6.0, 
3.0, 5.0], [8.0, 5.0, 6.0], [3.0, 8.0, 1.0]], [[5.0, 8.0, 1.0], [2.0, 1.0, 3.0], [4.0, 3.0, 4.0], [7.0, 7.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 3.0], [1.0, 3.0, 7.0], [4.0, 1.0, 2.0], [5.0, 2.0, 8.0]], [[4.0, 7.0, 1.0], [4.0, 
3.0, 1.0], [7.0, 4.0, 8.0], [2.0, 5.0, 4.0]], [[4.0, 6.0, 5.0], [5.0, 6.0, 1.0], [2.0, 2.0, 4.0], [7.0, 2.0, 1.0]]])
      val=Data(numpy.array([[[7.0], [6.0]]]),self.functionspace)
      arg[1:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 3.0], [1.0, 8.0, 3.0], [3.0, 6.0, 3.0], [6.0, 4.0, 5.0]], [[1.0, 6.0, 6.0], [6.0, 
3.0, 5.0], [8.0, 5.0, 7.0], [3.0, 8.0, 6.0]], [[5.0, 8.0, 1.0], [2.0, 1.0, 3.0], [4.0, 3.0, 4.0], [7.0, 7.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 8.0, 3.0], [1.0, 3.0, 7.0], [4.0, 1.0, 2.0], [5.0, 2.0, 8.0]], [[4.0, 7.0, 1.0], [4.0, 
3.0, 1.0], [7.0, 4.0, 7.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 5.0], [5.0, 6.0, 1.0], [2.0, 2.0, 4.0], [7.0, 2.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 5.0, 3.0], [7.0, 6.0, 5.0], [5.0, 4.0, 1.0], [2.0, 3.0, 1.0]], [[4.0, 4.0, 5.0], [5.0, 
4.0, 3.0], [6.0, 1.0, 3.0], [7.0, 2.0, 2.0]], [[7.0, 2.0, 7.0], [6.0, 5.0, 3.0], [2.0, 3.0, 2.0], [6.0, 5.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 1.0, 1.0], [2.0, 3.0, 7.0], [8.0, 3.0, 6.0], [7.0, 7.0, 5.0]], [[3.0, 2.0, 4.0], [3.0, 
6.0, 8.0], [4.0, 2.0, 7.0], [8.0, 6.0, 2.0]], [[6.0, 2.0, 2.0], [5.0, 3.0, 2.0], [5.0, 7.0, 5.0], [3.0, 4.0, 7.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0], [6.0]]])+(1.-msk_val)*numpy.array([[[4.0], [7.0]]])
      arg[1:2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 5.0, 3.0], [7.0, 6.0, 5.0], [5.0, 4.0, 1.0], [2.0, 3.0, 1.0]], [[4.0, 4.0, 5.0], [5.0, 
4.0, 3.0], [6.0, 1.0, 1.0], [7.0, 2.0, 6.0]], [[7.0, 2.0, 7.0], [6.0, 5.0, 3.0], [2.0, 3.0, 2.0], [6.0, 5.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 1.0, 1.0], [2.0, 3.0, 7.0], [8.0, 3.0, 6.0], [7.0, 7.0, 5.0]], [[3.0, 2.0, 4.0], [3.0, 
6.0, 8.0], [4.0, 2.0, 4.0], [8.0, 6.0, 7.0]], [[6.0, 2.0, 2.0], [5.0, 3.0, 2.0], [5.0, 7.0, 5.0], [3.0, 4.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 6.0], [7.0, 3.0, 2.0], [7.0, 4.0, 7.0], [7.0, 3.0, 8.0]], [[8.0, 3.0, 4.0], [5.0, 
8.0, 1.0], [4.0, 7.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 3.0, 3.0], [8.0, 8.0, 3.0], [1.0, 3.0, 2.0], [1.0, 8.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 1.0, 2.0], [3.0, 5.0, 8.0], [5.0, 8.0, 2.0], [5.0, 4.0, 1.0]], [[8.0, 8.0, 2.0], [3.0, 
1.0, 3.0], [4.0, 1.0, 2.0], [2.0, 7.0, 6.0]], [[7.0, 8.0, 1.0], [4.0, 3.0, 6.0], [2.0, 2.0, 1.0], [2.0, 3.0, 2.0]]])
      val=numpy.array([[[1.0], [6.0]]])
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 6.0], [7.0, 3.0, 2.0], [7.0, 4.0, 7.0], [7.0, 3.0, 8.0]], [[8.0, 3.0, 4.0], [5.0, 
8.0, 1.0], [4.0, 7.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 3.0, 3.0], [8.0, 8.0, 3.0], [1.0, 3.0, 1.0], [1.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 1.0, 2.0], [3.0, 5.0, 8.0], [5.0, 8.0, 2.0], [5.0, 4.0, 1.0]], [[8.0, 8.0, 2.0], [3.0, 
1.0, 3.0], [4.0, 1.0, 2.0], [2.0, 7.0, 6.0]], [[7.0, 8.0, 1.0], [4.0, 3.0, 6.0], [2.0, 2.0, 1.0], [2.0, 3.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 3.0, 3.0], [6.0, 7.0, 3.0], [8.0, 6.0, 6.0], [6.0, 2.0, 8.0]], [[1.0, 2.0, 1.0], [4.0, 
8.0, 6.0], [1.0, 2.0, 4.0], [7.0, 6.0, 1.0]], [[4.0, 1.0, 5.0], [5.0, 1.0, 7.0], [8.0, 6.0, 4.0], [7.0, 3.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 2.0, 8.0], [6.0, 3.0, 3.0], [3.0, 5.0, 2.0], [3.0, 1.0, 1.0]], [[7.0, 2.0, 7.0], [6.0, 
4.0, 5.0], [2.0, 5.0, 4.0], [8.0, 7.0, 8.0]], [[6.0, 1.0, 8.0], [3.0, 5.0, 1.0], [2.0, 1.0, 2.0], [8.0, 5.0, 3.0]]])
      val=Data(numpy.array([[[1.0], [5.0]]]),self.functionspace)
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 3.0], [6.0, 7.0, 3.0], [8.0, 6.0, 6.0], [6.0, 2.0, 8.0]], [[1.0, 2.0, 1.0], [4.0, 
8.0, 6.0], [1.0, 2.0, 4.0], [7.0, 6.0, 1.0]], [[4.0, 1.0, 5.0], [5.0, 1.0, 7.0], [8.0, 6.0, 1.0], [7.0, 3.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 8.0], [6.0, 3.0, 3.0], [3.0, 5.0, 2.0], [3.0, 1.0, 1.0]], [[7.0, 2.0, 7.0], [6.0, 
4.0, 5.0], [2.0, 5.0, 4.0], [8.0, 7.0, 8.0]], [[6.0, 1.0, 8.0], [3.0, 5.0, 1.0], [2.0, 1.0, 1.0], [8.0, 5.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 1.0, 8.0], [3.0, 6.0, 6.0], [3.0, 2.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 2.0, 8.0], [5.0, 
2.0, 8.0], [8.0, 6.0, 5.0], [7.0, 3.0, 2.0]], [[3.0, 2.0, 4.0], [8.0, 6.0, 5.0], [2.0, 6.0, 1.0], [8.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 3.0, 2.0], [7.0, 5.0, 4.0], [3.0, 2.0, 4.0], [1.0, 7.0, 2.0]], [[5.0, 1.0, 1.0], [5.0, 
1.0, 8.0], [2.0, 5.0, 2.0], [2.0, 5.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 6.0, 4.0], [3.0, 2.0, 2.0], [7.0, 8.0, 6.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0], [7.0]]])+(1.-msk_val)*numpy.array([[[3.0], [8.0]]])
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 8.0], [3.0, 6.0, 6.0], [3.0, 2.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 2.0, 8.0], [5.0, 
2.0, 8.0], [8.0, 6.0, 5.0], [7.0, 3.0, 2.0]], [[3.0, 2.0, 4.0], [8.0, 6.0, 5.0], [2.0, 6.0, 8.0], [8.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 2.0], [7.0, 5.0, 4.0], [3.0, 2.0, 4.0], [1.0, 7.0, 2.0]], [[5.0, 1.0, 1.0], [5.0, 
1.0, 8.0], [2.0, 5.0, 2.0], [2.0, 5.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 6.0, 4.0], [3.0, 2.0, 3.0], [7.0, 8.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 2.0, 2.0], [2.0, 7.0, 1.0], [7.0, 4.0, 4.0], [5.0, 4.0, 8.0]], [[2.0, 1.0, 1.0], [5.0, 
5.0, 6.0], [7.0, 7.0, 4.0], [6.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [1.0, 2.0, 5.0], [8.0, 3.0, 2.0], [2.0, 2.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 5.0], [7.0, 1.0, 4.0], [1.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 5.0], [2.0, 
8.0, 2.0], [1.0, 5.0, 1.0], [6.0, 4.0, 5.0]], [[3.0, 6.0, 5.0], [1.0, 4.0, 4.0], [6.0, 2.0, 8.0], [1.0, 4.0, 2.0]]])
      val=numpy.array([7.0, 6.0, 1.0])
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 2.0, 2.0], [2.0, 7.0, 1.0], [7.0, 4.0, 4.0], [5.0, 4.0, 8.0]], [[2.0, 1.0, 1.0], [5.0, 
5.0, 6.0], [7.0, 6.0, 1.0], [6.0, 5.0, 5.0]], [[7.0, 3.0, 7.0], [1.0, 2.0, 5.0], [8.0, 3.0, 2.0], [2.0, 2.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 5.0], [7.0, 1.0, 4.0], [1.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 5.0], [2.0, 
8.0, 2.0], [7.0, 6.0, 1.0], [6.0, 4.0, 5.0]], [[3.0, 6.0, 5.0], [1.0, 4.0, 4.0], [6.0, 2.0, 8.0], [1.0, 4.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 5.0, 8.0], [4.0, 3.0, 1.0], [6.0, 8.0, 7.0], [7.0, 8.0, 6.0]], [[4.0, 7.0, 3.0], [4.0, 
7.0, 4.0], [1.0, 2.0, 3.0], [6.0, 5.0, 3.0]], [[2.0, 7.0, 7.0], [4.0, 2.0, 3.0], [8.0, 1.0, 2.0], [4.0, 1.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 4.0, 5.0], [2.0, 6.0, 7.0], [6.0, 6.0, 2.0], [5.0, 2.0, 8.0]], [[4.0, 4.0, 6.0], [5.0, 
8.0, 7.0], [7.0, 8.0, 4.0], [5.0, 5.0, 6.0]], [[5.0, 7.0, 7.0], [4.0, 4.0, 5.0], [2.0, 2.0, 5.0], [2.0, 6.0, 8.0]]])
      val=Data(numpy.array([7.0, 2.0, 1.0]),self.functionspace)
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 5.0, 8.0], [4.0, 3.0, 1.0], [6.0, 8.0, 7.0], [7.0, 8.0, 6.0]], [[4.0, 7.0, 3.0], [4.0, 
7.0, 4.0], [7.0, 2.0, 1.0], [6.0, 5.0, 3.0]], [[2.0, 7.0, 7.0], [4.0, 2.0, 3.0], [8.0, 1.0, 2.0], [4.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 4.0, 5.0], [2.0, 6.0, 7.0], [6.0, 6.0, 2.0], [5.0, 2.0, 8.0]], [[4.0, 4.0, 6.0], [5.0, 
8.0, 7.0], [7.0, 2.0, 1.0], [5.0, 5.0, 6.0]], [[5.0, 7.0, 7.0], [4.0, 4.0, 5.0], [2.0, 2.0, 5.0], [2.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 3.0, 5.0], [3.0, 8.0, 1.0], [4.0, 5.0, 7.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 7.0], [2.0, 
8.0, 1.0], [2.0, 8.0, 2.0], [8.0, 3.0, 2.0]], [[4.0, 5.0, 1.0], [3.0, 4.0, 1.0], [4.0, 3.0, 1.0], [3.0, 1.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 2.0, 3.0], [5.0, 6.0, 6.0], [5.0, 8.0, 1.0], [7.0, 1.0, 4.0]], [[2.0, 8.0, 8.0], [6.0, 
3.0, 5.0], [1.0, 4.0, 6.0], [5.0, 5.0, 4.0]], [[1.0, 6.0, 3.0], [2.0, 6.0, 5.0], [7.0, 1.0, 5.0], [2.0, 6.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([8.0, 7.0, 6.0])+(1.-msk_val)*numpy.array([7.0, 6.0, 7.0])
      arg[1,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 5.0], [3.0, 8.0, 1.0], [4.0, 5.0, 7.0], [8.0, 4.0, 5.0]], [[3.0, 5.0, 7.0], [2.0, 
8.0, 1.0], [8.0, 7.0, 6.0], [8.0, 3.0, 2.0]], [[4.0, 5.0, 1.0], [3.0, 4.0, 1.0], [4.0, 3.0, 1.0], [3.0, 1.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 3.0], [5.0, 6.0, 6.0], [5.0, 8.0, 1.0], [7.0, 1.0, 4.0]], [[2.0, 8.0, 8.0], [6.0, 
3.0, 5.0], [7.0, 6.0, 7.0], [5.0, 5.0, 4.0]], [[1.0, 6.0, 3.0], [2.0, 6.0, 5.0], [7.0, 1.0, 5.0], [2.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 5.0, 5.0], [8.0, 3.0, 2.0], [6.0, 2.0, 2.0], [3.0, 4.0, 8.0]], [[1.0, 6.0, 7.0], [1.0, 
7.0, 2.0], [2.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 6.0, 3.0], [3.0, 5.0, 3.0], [1.0, 3.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 4.0, 5.0], [4.0, 6.0, 2.0], [1.0, 2.0, 4.0], [1.0, 5.0, 7.0]], [[8.0, 5.0, 2.0], [4.0, 
4.0, 1.0], [8.0, 1.0, 5.0], [2.0, 2.0, 7.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 1.0], [2.0, 5.0, 4.0], [6.0, 6.0, 7.0]]])
      val=numpy.array([[[8.0, 8.0, 4.0], [6.0, 1.0, 5.0]]])
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 5.0, 5.0], [8.0, 3.0, 2.0], [6.0, 2.0, 2.0], [3.0, 4.0, 8.0]], [[1.0, 6.0, 7.0], [1.0, 
7.0, 2.0], [2.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 6.0, 3.0], [8.0, 8.0, 4.0], [6.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 5.0], [4.0, 6.0, 2.0], [1.0, 2.0, 4.0], [1.0, 5.0, 7.0]], [[8.0, 5.0, 2.0], [4.0, 
4.0, 1.0], [8.0, 1.0, 5.0], [2.0, 2.0, 7.0]], [[1.0, 5.0, 4.0], [3.0, 3.0, 1.0], [8.0, 8.0, 4.0], [6.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 7.0], [4.0, 7.0, 8.0], [1.0, 5.0, 4.0], [4.0, 3.0, 6.0]], [[2.0, 4.0, 5.0], [7.0, 
6.0, 1.0], [7.0, 1.0, 1.0], [6.0, 5.0, 1.0]], [[6.0, 8.0, 1.0], [7.0, 8.0, 8.0], [6.0, 7.0, 7.0], [7.0, 3.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 5.0, 3.0], [5.0, 7.0, 4.0], [4.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 1.0, 3.0], [5.0, 
8.0, 7.0], [7.0, 6.0, 8.0], [3.0, 2.0, 4.0]], [[3.0, 5.0, 1.0], [6.0, 1.0, 5.0], [4.0, 5.0, 5.0], [7.0, 3.0, 4.0]]])
      val=Data(numpy.array([[[4.0, 4.0, 4.0], [1.0, 5.0, 7.0]]]),self.functionspace)
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 7.0], [4.0, 7.0, 8.0], [1.0, 5.0, 4.0], [4.0, 3.0, 6.0]], [[2.0, 4.0, 5.0], [7.0, 
6.0, 1.0], [7.0, 1.0, 1.0], [6.0, 5.0, 1.0]], [[6.0, 8.0, 1.0], [7.0, 8.0, 8.0], [4.0, 4.0, 4.0], [1.0, 5.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 3.0], [5.0, 7.0, 4.0], [4.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 1.0, 3.0], [5.0, 
8.0, 7.0], [7.0, 6.0, 8.0], [3.0, 2.0, 4.0]], [[3.0, 5.0, 1.0], [6.0, 1.0, 5.0], [4.0, 4.0, 4.0], [1.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 3.0, 7.0], [1.0, 5.0, 5.0], [4.0, 6.0, 6.0], [2.0, 5.0, 2.0]], [[2.0, 4.0, 8.0], [2.0, 
2.0, 6.0], [4.0, 2.0, 5.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [5.0, 2.0, 8.0], [4.0, 1.0, 2.0], [6.0, 6.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 5.0, 4.0], [6.0, 1.0, 1.0], [6.0, 2.0, 4.0], [6.0, 2.0, 1.0]], [[5.0, 8.0, 1.0], [8.0, 
1.0, 6.0], [7.0, 3.0, 1.0], [1.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [5.0, 4.0, 2.0], [6.0, 4.0, 6.0], [8.0, 6.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 2.0, 1.0], [3.0, 7.0, 1.0]]])+(1.-msk_val)*numpy.array([[[2.0, 1.0, 5.0], [8.0, 8.0, 
2.0]]])
      arg[2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 3.0, 7.0], [1.0, 5.0, 5.0], [4.0, 6.0, 6.0], [2.0, 5.0, 2.0]], [[2.0, 4.0, 8.0], [2.0, 
2.0, 6.0], [4.0, 2.0, 5.0], [7.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [5.0, 2.0, 8.0], [4.0, 2.0, 1.0], [3.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 4.0], [6.0, 1.0, 1.0], [6.0, 2.0, 4.0], [6.0, 2.0, 1.0]], [[5.0, 8.0, 1.0], [8.0, 
1.0, 6.0], [7.0, 3.0, 1.0], [1.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [5.0, 4.0, 2.0], [2.0, 1.0, 5.0], [8.0, 8.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 8.0, 7.0], [4.0, 6.0, 5.0], [2.0, 1.0, 6.0], [7.0, 2.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 
7.0, 3.0], [6.0, 1.0, 3.0], [4.0, 2.0, 2.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 4.0], [5.0, 4.0, 2.0], [5.0, 7.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 5.0, 1.0], [7.0, 7.0, 6.0], [8.0, 7.0, 6.0], [5.0, 3.0, 5.0]], [[2.0, 2.0, 8.0], [8.0, 
2.0, 1.0], [6.0, 5.0, 7.0], [8.0, 1.0, 6.0]], [[6.0, 7.0, 7.0], [1.0, 6.0, 7.0], [8.0, 7.0, 3.0], [4.0, 3.0, 2.0]]])
      val=numpy.array([[[6.0, 3.0, 7.0], [4.0, 6.0, 5.0], [1.0, 7.0, 3.0]]])
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 8.0, 7.0], [4.0, 6.0, 5.0], [2.0, 1.0, 6.0], [7.0, 2.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 
6.0, 5.0], [1.0, 7.0, 3.0], [4.0, 2.0, 2.0]], [[7.0, 2.0, 3.0], [8.0, 4.0, 4.0], [5.0, 4.0, 2.0], [5.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 5.0, 1.0], [7.0, 7.0, 6.0], [8.0, 7.0, 6.0], [5.0, 3.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 
6.0, 5.0], [1.0, 7.0, 3.0], [8.0, 1.0, 6.0]], [[6.0, 7.0, 7.0], [1.0, 6.0, 7.0], [8.0, 7.0, 3.0], [4.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 1.0, 7.0], [2.0, 3.0, 4.0], [2.0, 4.0, 4.0], [4.0, 6.0, 3.0]], [[5.0, 2.0, 8.0], [1.0, 
2.0, 3.0], [7.0, 7.0, 8.0], [7.0, 6.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 1.0, 5.0], [2.0, 5.0, 7.0], [6.0, 2.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 2.0, 8.0], [6.0, 7.0, 1.0], [5.0, 6.0, 2.0], [1.0, 8.0, 5.0]], [[3.0, 3.0, 7.0], [1.0, 
3.0, 5.0], [1.0, 8.0, 7.0], [2.0, 2.0, 3.0]], [[5.0, 8.0, 4.0], [6.0, 3.0, 3.0], [6.0, 2.0, 7.0], [1.0, 7.0, 7.0]]])
      val=Data(numpy.array([[[5.0, 3.0, 3.0], [3.0, 3.0, 8.0], [1.0, 3.0, 8.0]]]),self.functionspace)
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 1.0, 7.0], [2.0, 3.0, 4.0], [2.0, 4.0, 4.0], [4.0, 6.0, 3.0]], [[5.0, 3.0, 3.0], [3.0, 
3.0, 8.0], [1.0, 3.0, 8.0], [7.0, 6.0, 7.0]], [[6.0, 3.0, 7.0], [8.0, 1.0, 5.0], [2.0, 5.0, 7.0], [6.0, 2.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 8.0], [6.0, 7.0, 1.0], [5.0, 6.0, 2.0], [1.0, 8.0, 5.0]], [[5.0, 3.0, 3.0], [3.0, 
3.0, 8.0], [1.0, 3.0, 8.0], [2.0, 2.0, 3.0]], [[5.0, 8.0, 4.0], [6.0, 3.0, 3.0], [6.0, 2.0, 7.0], [1.0, 7.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 2.0, 5.0], [7.0, 8.0, 6.0], [1.0, 7.0, 7.0], [7.0, 5.0, 2.0]], [[5.0, 8.0, 4.0], [6.0, 
8.0, 2.0], [7.0, 5.0, 6.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 1.0], [2.0, 5.0, 6.0], [4.0, 8.0, 1.0], [7.0, 2.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 8.0], [8.0, 1.0, 7.0], [4.0, 8.0, 2.0], [7.0, 4.0, 1.0]], [[5.0, 2.0, 3.0], [3.0, 
8.0, 8.0], [6.0, 5.0, 1.0], [4.0, 3.0, 7.0]], [[7.0, 6.0, 7.0], [6.0, 7.0, 7.0], [8.0, 3.0, 7.0], [4.0, 7.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 1.0, 2.0], [1.0, 3.0, 6.0], [5.0, 2.0, 5.0]]])+(1.-msk_val)*numpy.array([[[2.0, 7.0, 
8.0], [2.0, 2.0, 7.0], [1.0, 4.0, 4.0]]])
      arg[1:2,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 2.0, 5.0], [7.0, 8.0, 6.0], [1.0, 7.0, 7.0], [7.0, 5.0, 2.0]], [[6.0, 1.0, 2.0], [1.0, 
3.0, 6.0], [5.0, 2.0, 5.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 1.0], [2.0, 5.0, 6.0], [4.0, 8.0, 1.0], [7.0, 2.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 8.0], [8.0, 1.0, 7.0], [4.0, 8.0, 2.0], [7.0, 4.0, 1.0]], [[2.0, 7.0, 8.0], [2.0, 
2.0, 7.0], [1.0, 4.0, 4.0], [4.0, 3.0, 7.0]], [[7.0, 6.0, 7.0], [6.0, 7.0, 7.0], [8.0, 3.0, 7.0], [4.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 2.0], [6.0, 7.0, 4.0], [1.0, 8.0, 3.0], [8.0, 4.0, 5.0]], [[7.0, 3.0, 7.0], [3.0, 
4.0, 4.0], [4.0, 8.0, 3.0], [2.0, 4.0, 3.0]], [[2.0, 7.0, 3.0], [4.0, 6.0, 3.0], [7.0, 2.0, 4.0], [7.0, 7.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 3.0], [3.0, 5.0, 4.0], [4.0, 1.0, 3.0], [3.0, 8.0, 2.0]], [[5.0, 1.0, 3.0], [4.0, 
5.0, 8.0], [5.0, 7.0, 8.0], [2.0, 5.0, 1.0]], [[1.0, 4.0, 6.0], [8.0, 1.0, 2.0], [4.0, 8.0, 3.0], [4.0, 4.0, 4.0]]])
      val=numpy.array([[[5.0, 8.0, 2.0], [8.0, 2.0, 4.0]], [[3.0, 4.0, 7.0], [5.0, 4.0, 5.0]], [[5.0, 7.0, 4.0], [4.0, 2.0, 
4.0]]])
      arg[:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 2.0], [5.0, 8.0, 2.0], [8.0, 2.0, 4.0], [8.0, 4.0, 5.0]], [[7.0, 3.0, 7.0], [3.0, 
4.0, 7.0], [5.0, 4.0, 5.0], [2.0, 4.0, 3.0]], [[2.0, 7.0, 3.0], [5.0, 7.0, 4.0], [4.0, 2.0, 4.0], [7.0, 7.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 3.0], [5.0, 8.0, 2.0], [8.0, 2.0, 4.0], [3.0, 8.0, 2.0]], [[5.0, 1.0, 3.0], [3.0, 
4.0, 7.0], [5.0, 4.0, 5.0], [2.0, 5.0, 1.0]], [[1.0, 4.0, 6.0], [5.0, 7.0, 4.0], [4.0, 2.0, 4.0], [4.0, 4.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 7.0, 3.0], [2.0, 8.0, 5.0], [1.0, 8.0, 3.0], [3.0, 8.0, 2.0]], [[5.0, 1.0, 4.0], [1.0, 
4.0, 5.0], [4.0, 2.0, 4.0], [7.0, 1.0, 6.0]], [[6.0, 7.0, 3.0], [1.0, 4.0, 6.0], [7.0, 3.0, 3.0], [4.0, 4.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 5.0, 1.0], [1.0, 1.0, 6.0], [1.0, 5.0, 2.0], [2.0, 2.0, 3.0]], [[7.0, 2.0, 1.0], [3.0, 
1.0, 3.0], [3.0, 8.0, 6.0], [8.0, 7.0, 1.0]], [[6.0, 2.0, 4.0], [2.0, 5.0, 3.0], [7.0, 8.0, 3.0], [3.0, 8.0, 6.0]]])
      val=Data(numpy.array([[[7.0, 3.0, 1.0], [2.0, 1.0, 1.0]], [[8.0, 8.0, 1.0], [3.0, 6.0, 5.0]], [[5.0, 4.0, 8.0], [2.0, 
4.0, 3.0]]]),self.functionspace)
      arg[:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 7.0, 3.0], [7.0, 3.0, 1.0], [2.0, 1.0, 1.0], [3.0, 8.0, 2.0]], [[5.0, 1.0, 4.0], [8.0, 
8.0, 1.0], [3.0, 6.0, 5.0], [7.0, 1.0, 6.0]], [[6.0, 7.0, 3.0], [5.0, 4.0, 8.0], [2.0, 4.0, 3.0], [4.0, 4.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 5.0, 1.0], [7.0, 3.0, 1.0], [2.0, 1.0, 1.0], [2.0, 2.0, 3.0]], [[7.0, 2.0, 1.0], [8.0, 
8.0, 1.0], [3.0, 6.0, 5.0], [8.0, 7.0, 1.0]], [[6.0, 2.0, 4.0], [5.0, 4.0, 8.0], [2.0, 4.0, 3.0], [3.0, 8.0, 6.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 6.0, 7.0], [8.0, 2.0, 1.0], [8.0, 5.0, 7.0], [8.0, 8.0, 3.0]], [[3.0, 1.0, 7.0], [6.0, 
6.0, 6.0], [1.0, 8.0, 6.0], [7.0, 6.0, 1.0]], [[7.0, 6.0, 8.0], [3.0, 3.0, 6.0], [5.0, 1.0, 6.0], [7.0, 5.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 4.0, 2.0], [3.0, 3.0, 6.0], [5.0, 6.0, 5.0], [2.0, 1.0, 5.0]], [[3.0, 4.0, 4.0], [1.0, 
8.0, 3.0], [3.0, 7.0, 5.0], [6.0, 7.0, 6.0]], [[6.0, 5.0, 1.0], [3.0, 1.0, 6.0], [5.0, 7.0, 6.0], [5.0, 7.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 7.0, 5.0], [5.0, 8.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 1.0, 5.0]], [[1.0, 3.0, 6.0], [3.0, 
7.0, 3.0]]])+(1.-msk_val)*numpy.array([[[2.0, 5.0, 2.0], [3.0, 1.0, 8.0]], [[4.0, 1.0, 4.0], [7.0, 8.0, 8.0]], [[2.0, 5.0, 
4.0], [3.0, 5.0, 5.0]]])
      arg[:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 7.0], [7.0, 7.0, 5.0], [5.0, 8.0, 1.0], [8.0, 8.0, 3.0]], [[3.0, 1.0, 7.0], [7.0, 
7.0, 7.0], [1.0, 1.0, 5.0], [7.0, 6.0, 1.0]], [[7.0, 6.0, 8.0], [1.0, 3.0, 6.0], [3.0, 7.0, 3.0], [7.0, 5.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 4.0, 2.0], [2.0, 5.0, 2.0], [3.0, 1.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 4.0, 4.0], [4.0, 
1.0, 4.0], [7.0, 8.0, 8.0], [6.0, 7.0, 6.0]], [[6.0, 5.0, 1.0], [2.0, 5.0, 4.0], [3.0, 5.0, 5.0], [5.0, 7.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 6.0, 3.0], [8.0, 3.0, 1.0], [4.0, 6.0, 3.0], [1.0, 1.0, 2.0]], [[7.0, 7.0, 1.0], [4.0, 
7.0, 1.0], [2.0, 5.0, 1.0], [4.0, 2.0, 1.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 7.0], [6.0, 7.0, 3.0], [1.0, 1.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 3.0, 1.0], [6.0, 6.0, 6.0], [4.0, 4.0, 1.0], [8.0, 1.0, 7.0]], [[6.0, 3.0, 7.0], [5.0, 
1.0, 1.0], [5.0, 4.0, 2.0], [4.0, 1.0, 2.0]], [[8.0, 6.0, 4.0], [3.0, 5.0, 2.0], [2.0, 4.0, 8.0], [5.0, 7.0, 8.0]]])
      val=numpy.array([[[2.0, 2.0, 1.0], [4.0, 2.0, 3.0], [8.0, 6.0, 3.0], [3.0, 8.0, 8.0]], [[8.0, 5.0, 5.0], [6.0, 6.0, 7.0], 
[3.0, 1.0, 2.0], [2.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 4.0], [3.0, 8.0, 8.0], [8.0, 6.0, 4.0]]])
      arg[:,:]=val
      ref=Data(numpy.array([[[2.0, 2.0, 1.0], [4.0, 2.0, 3.0], [8.0, 6.0, 3.0], [3.0, 8.0, 8.0]], [[8.0, 5.0, 5.0], [6.0, 6.0, 
7.0], [3.0, 1.0, 2.0], [2.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [2.0, 2.0, 4.0], [3.0, 8.0, 8.0], [8.0, 6.0, 
4.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 7.0, 8.0], [2.0, 1.0, 1.0], [3.0, 4.0, 6.0], [4.0, 1.0, 4.0]], [[6.0, 5.0, 3.0], [1.0, 
5.0, 2.0], [3.0, 1.0, 4.0], [2.0, 4.0, 4.0]], [[4.0, 1.0, 7.0], [5.0, 2.0, 3.0], [6.0, 7.0, 4.0], [5.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 1.0, 2.0], [5.0, 2.0, 7.0], [1.0, 1.0, 2.0], [7.0, 6.0, 3.0]], [[1.0, 6.0, 6.0], [6.0, 
1.0, 1.0], [8.0, 3.0, 6.0], [2.0, 2.0, 1.0]], [[5.0, 6.0, 1.0], [6.0, 2.0, 8.0], [1.0, 2.0, 4.0], [3.0, 8.0, 6.0]]])
      val=Data(numpy.array([[[8.0, 7.0, 1.0], [1.0, 8.0, 8.0], [5.0, 7.0, 3.0], [2.0, 3.0, 6.0]], [[3.0, 8.0, 2.0], [8.0, 6.0, 
2.0], [1.0, 4.0, 3.0], [8.0, 3.0, 2.0]], [[7.0, 1.0, 8.0], [2.0, 5.0, 5.0], [7.0, 2.0, 5.0], [4.0, 8.0, 
6.0]]]),self.functionspace)
      arg[:,:]=val
      ref=Data(numpy.array([[[8.0, 7.0, 1.0], [1.0, 8.0, 8.0], [5.0, 7.0, 3.0], [2.0, 3.0, 6.0]], [[3.0, 8.0, 2.0], [8.0, 6.0, 
2.0], [1.0, 4.0, 3.0], [8.0, 3.0, 2.0]], [[7.0, 1.0, 8.0], [2.0, 5.0, 5.0], [7.0, 2.0, 5.0], [4.0, 8.0, 
6.0]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 4.0, 8.0], [4.0, 8.0, 7.0], [8.0, 2.0, 1.0], [7.0, 6.0, 5.0]], [[1.0, 8.0, 6.0], [2.0, 
2.0, 8.0], [1.0, 6.0, 6.0], [8.0, 7.0, 5.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 4.0], [8.0, 4.0, 1.0], [3.0, 1.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 4.0, 8.0], [2.0, 8.0, 2.0], [6.0, 4.0, 5.0], [8.0, 3.0, 1.0]], [[4.0, 4.0, 7.0], [2.0, 
4.0, 3.0], [4.0, 1.0, 8.0], [3.0, 1.0, 4.0]], [[8.0, 1.0, 4.0], [4.0, 8.0, 6.0], [4.0, 7.0, 2.0], [6.0, 5.0, 6.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 2.0, 6.0], [6.0, 1.0, 1.0], [1.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[4.0, 8.0, 4.0], [1.0, 
4.0, 1.0], [5.0, 8.0, 8.0], [3.0, 6.0, 4.0]], [[1.0, 8.0, 5.0], [8.0, 1.0, 5.0], [3.0, 8.0, 4.0], [8.0, 4.0, 
2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 7.0], [3.0, 6.0, 6.0], [8.0, 2.0, 3.0], [4.0, 3.0, 8.0]], [[6.0, 8.0, 2.0], [4.0, 
7.0, 6.0], [7.0, 1.0, 4.0], [1.0, 7.0, 6.0]], [[4.0, 1.0, 3.0], [6.0, 5.0, 8.0], [1.0, 4.0, 4.0], [6.0, 3.0, 7.0]]])
      arg[:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 2.0, 6.0], [6.0, 1.0, 1.0], [1.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[4.0, 8.0, 4.0], [1.0, 
4.0, 1.0], [5.0, 8.0, 8.0], [3.0, 6.0, 4.0]], [[1.0, 8.0, 5.0], [8.0, 1.0, 5.0], [3.0, 8.0, 4.0], [8.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 7.0], [3.0, 6.0, 6.0], [8.0, 2.0, 3.0], [4.0, 3.0, 8.0]], [[6.0, 8.0, 2.0], [4.0, 
7.0, 6.0], [7.0, 1.0, 4.0], [1.0, 7.0, 6.0]], [[4.0, 1.0, 3.0], [6.0, 5.0, 8.0], [1.0, 4.0, 4.0], [6.0, 3.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_PUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 6.0, 5.0], [8.0, 8.0, 6.0], [2.0, 4.0, 7.0], [4.0, 4.0, 3.0]], [[1.0, 4.0, 8.0], [8.0, 
5.0, 6.0], [5.0, 5.0, 2.0], [6.0, 4.0, 8.0]], [[8.0, 2.0, 4.0], [8.0, 4.0, 1.0], [1.0, 8.0, 4.0], [6.0, 8.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 6.0, 5.0], [3.0, 7.0, 8.0], [5.0, 6.0, 4.0], [8.0, 7.0, 8.0]], [[4.0, 8.0, 5.0], [5.0, 
2.0, 7.0], [7.0, 2.0, 4.0], [7.0, 6.0, 8.0]], [[8.0, 7.0, 5.0], [5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [2.0, 7.0, 3.0]]])
      val=numpy.array([[5.0, 3.0, 6.0], [7.0, 1.0, 3.0], [6.0, 4.0, 8.0]])
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 6.0, 5.0], [8.0, 8.0, 6.0], [2.0, 4.0, 7.0], [4.0, 4.0, 3.0]], [[5.0, 3.0, 6.0], [7.0, 
1.0, 3.0], [6.0, 4.0, 8.0], [6.0, 4.0, 8.0]], [[8.0, 2.0, 4.0], [8.0, 4.0, 1.0], [1.0, 8.0, 4.0], [6.0, 8.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 6.0, 5.0], [3.0, 7.0, 8.0], [5.0, 6.0, 4.0], [8.0, 7.0, 8.0]], [[5.0, 3.0, 6.0], [7.0, 
1.0, 3.0], [6.0, 4.0, 8.0], [7.0, 6.0, 8.0]], [[8.0, 7.0, 5.0], [5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [2.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_PUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 2.0], [5.0, 8.0, 2.0], [6.0, 3.0, 2.0], [8.0, 2.0, 3.0]], [[5.0, 1.0, 3.0], [2.0, 
2.0, 4.0], [3.0, 4.0, 5.0], [7.0, 8.0, 1.0]], [[5.0, 1.0, 5.0], [3.0, 5.0, 4.0], [1.0, 4.0, 8.0], [2.0, 8.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 4.0, 4.0], [2.0, 8.0, 2.0], [6.0, 2.0, 1.0], [6.0, 3.0, 8.0]], [[6.0, 5.0, 2.0], [6.0, 
5.0, 7.0], [8.0, 1.0, 6.0], [2.0, 7.0, 6.0]], [[6.0, 8.0, 2.0], [4.0, 7.0, 3.0], [3.0, 1.0, 5.0], [8.0, 3.0, 2.0]]])
      val=Data(numpy.array([[8.0, 2.0, 6.0], [4.0, 7.0, 4.0], [3.0, 7.0, 1.0]]),self.functionspace)
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 2.0], [5.0, 8.0, 2.0], [6.0, 3.0, 2.0], [8.0, 2.0, 3.0]], [[8.0, 2.0, 6.0], [4.0, 
7.0, 4.0], [3.0, 7.0, 1.0], [7.0, 8.0, 1.0]], [[5.0, 1.0, 5.0], [3.0, 5.0, 4.0], [1.0, 4.0, 8.0], [2.0, 8.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 4.0, 4.0], [2.0, 8.0, 2.0], [6.0, 2.0, 1.0], [6.0, 3.0, 8.0]], [[8.0, 2.0, 6.0], [4.0, 
7.0, 4.0], [3.0, 7.0, 1.0], [2.0, 7.0, 6.0]], [[6.0, 8.0, 2.0], [4.0, 7.0, 3.0], [3.0, 1.0, 5.0], [8.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_PUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 7.0, 5.0], [2.0, 4.0, 6.0], [5.0, 7.0, 3.0], [3.0, 4.0, 3.0]], [[1.0, 6.0, 8.0], [2.0, 
5.0, 5.0], [2.0, 7.0, 2.0], [5.0, 8.0, 5.0]], [[3.0, 2.0, 4.0], [8.0, 7.0, 1.0], [8.0, 2.0, 1.0], [7.0, 2.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 5.0, 2.0], [3.0, 8.0, 8.0], [5.0, 7.0, 7.0], [3.0, 5.0, 1.0]], [[5.0, 2.0, 4.0], [1.0, 
5.0, 5.0], [3.0, 1.0, 6.0], [1.0, 3.0, 6.0]], [[6.0, 2.0, 5.0], [7.0, 8.0, 6.0], [2.0, 5.0, 2.0], [3.0, 1.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 7.0, 7.0], [1.0, 6.0, 8.0], [6.0, 3.0, 3.0]])+(1.-msk_val)*numpy.array([[8.0, 2.0, 2.0], 
[1.0, 8.0, 6.0], [3.0, 1.0, 5.0]])
      arg[1,:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 7.0, 5.0], [2.0, 4.0, 6.0], [5.0, 7.0, 3.0], [3.0, 4.0, 3.0]], [[6.0, 7.0, 7.0], [1.0, 
6.0, 8.0], [6.0, 3.0, 3.0], [5.0, 8.0, 5.0]], [[3.0, 2.0, 4.0], [8.0, 7.0, 1.0], [8.0, 2.0, 1.0], [7.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 5.0, 2.0], [3.0, 8.0, 8.0], [5.0, 7.0, 7.0], [3.0, 5.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 
8.0, 6.0], [3.0, 1.0, 5.0], [1.0, 3.0, 6.0]], [[6.0, 2.0, 5.0], [7.0, 8.0, 6.0], [2.0, 5.0, 2.0], [3.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 6.0, 4.0], [6.0, 6.0, 7.0], [1.0, 8.0, 2.0], [4.0, 8.0, 7.0]], [[2.0, 3.0, 7.0], [2.0, 
6.0, 2.0], [8.0, 1.0, 5.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 6.0], [7.0, 1.0, 2.0], [6.0, 4.0, 8.0], [8.0, 8.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 2.0, 3.0], [6.0, 8.0, 8.0], [7.0, 3.0, 8.0], [6.0, 4.0, 6.0]], [[4.0, 8.0, 4.0], [8.0, 
6.0, 6.0], [1.0, 5.0, 5.0], [5.0, 4.0, 4.0]], [[1.0, 4.0, 6.0], [2.0, 1.0, 6.0], [4.0, 2.0, 3.0], [3.0, 5.0, 3.0]]])
      val=numpy.array([[7.0, 6.0, 5.0], [7.0, 1.0, 1.0]])
      arg[:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 6.0, 4.0], [6.0, 6.0, 7.0], [7.0, 6.0, 5.0], [4.0, 8.0, 7.0]], [[2.0, 3.0, 7.0], [2.0, 
6.0, 2.0], [7.0, 1.0, 1.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 6.0], [7.0, 1.0, 2.0], [6.0, 4.0, 8.0], [8.0, 8.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, 3.0], [6.0, 8.0, 8.0], [7.0, 6.0, 5.0], [6.0, 4.0, 6.0]], [[4.0, 8.0, 4.0], [8.0, 
6.0, 6.0], [7.0, 1.0, 1.0], [5.0, 4.0, 4.0]], [[1.0, 4.0, 6.0], [2.0, 1.0, 6.0], [4.0, 2.0, 3.0], [3.0, 5.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 2.0, 2.0], [4.0, 4.0, 2.0], [8.0, 1.0, 8.0], [2.0, 6.0, 6.0]], [[2.0, 1.0, 5.0], [4.0, 
6.0, 3.0], [3.0, 5.0, 8.0], [5.0, 3.0, 2.0]], [[6.0, 7.0, 4.0], [8.0, 6.0, 6.0], [4.0, 7.0, 6.0], [1.0, 7.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 7.0], [5.0, 8.0, 1.0], [4.0, 2.0, 2.0], [5.0, 4.0, 2.0]], [[8.0, 3.0, 6.0], [6.0, 
5.0, 4.0], [4.0, 5.0, 1.0], [4.0, 8.0, 4.0]], [[1.0, 3.0, 7.0], [7.0, 6.0, 1.0], [6.0, 7.0, 7.0], [2.0, 7.0, 3.0]]])
      val=Data(numpy.array([[2.0, 8.0, 6.0], [7.0, 2.0, 6.0]]),self.functionspace)
      arg[:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 2.0, 2.0], [4.0, 4.0, 2.0], [2.0, 8.0, 6.0], [2.0, 6.0, 6.0]], [[2.0, 1.0, 5.0], [4.0, 
6.0, 3.0], [7.0, 2.0, 6.0], [5.0, 3.0, 2.0]], [[6.0, 7.0, 4.0], [8.0, 6.0, 6.0], [4.0, 7.0, 6.0], [1.0, 7.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 7.0], [5.0, 8.0, 1.0], [2.0, 8.0, 6.0], [5.0, 4.0, 2.0]], [[8.0, 3.0, 6.0], [6.0, 
5.0, 4.0], [7.0, 2.0, 6.0], [4.0, 8.0, 4.0]], [[1.0, 3.0, 7.0], [7.0, 6.0, 1.0], [6.0, 7.0, 7.0], [2.0, 7.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 2.0, 7.0], [3.0, 4.0, 6.0], [5.0, 6.0, 3.0], [8.0, 7.0, 7.0]], [[1.0, 3.0, 7.0], [4.0, 
8.0, 2.0], [5.0, 6.0, 7.0], [1.0, 6.0, 8.0]], [[5.0, 3.0, 1.0], [4.0, 5.0, 8.0], [2.0, 4.0, 8.0], [8.0, 7.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 2.0, 6.0], [5.0, 6.0, 3.0], [8.0, 2.0, 8.0], [6.0, 1.0, 2.0]], [[3.0, 7.0, 7.0], [3.0, 
7.0, 4.0], [7.0, 8.0, 6.0], [2.0, 5.0, 3.0]], [[2.0, 2.0, 3.0], [8.0, 1.0, 1.0], [2.0, 4.0, 5.0], [8.0, 5.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 4.0, 5.0], [1.0, 4.0, 1.0]])+(1.-msk_val)*numpy.array([[1.0, 8.0, 6.0], [1.0, 6.0, 6.0]])
      arg[:2,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 2.0, 7.0], [3.0, 4.0, 6.0], [6.0, 4.0, 5.0], [8.0, 7.0, 7.0]], [[1.0, 3.0, 7.0], [4.0, 
8.0, 2.0], [1.0, 4.0, 1.0], [1.0, 6.0, 8.0]], [[5.0, 3.0, 1.0], [4.0, 5.0, 8.0], [2.0, 4.0, 8.0], [8.0, 7.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 2.0, 6.0], [5.0, 6.0, 3.0], [1.0, 8.0, 6.0], [6.0, 1.0, 2.0]], [[3.0, 7.0, 7.0], [3.0, 
7.0, 4.0], [1.0, 6.0, 6.0], [2.0, 5.0, 3.0]], [[2.0, 2.0, 3.0], [8.0, 1.0, 1.0], [2.0, 4.0, 5.0], [8.0, 5.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 8.0, 2.0], [3.0, 8.0, 3.0], [2.0, 1.0, 7.0], [2.0, 6.0, 6.0]], [[6.0, 3.0, 2.0], [3.0, 
7.0, 5.0], [2.0, 8.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 2.0, 8.0], [8.0, 8.0, 1.0], [6.0, 8.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 1.0, 6.0], [5.0, 8.0, 8.0], [8.0, 2.0, 7.0], [1.0, 5.0, 3.0]], [[8.0, 3.0, 3.0], [8.0, 
1.0, 1.0], [2.0, 2.0, 4.0], [5.0, 1.0, 8.0]], [[2.0, 2.0, 1.0], [5.0, 3.0, 5.0], [1.0, 1.0, 4.0], [2.0, 1.0, 3.0]]])
      val=numpy.array([[[4.0, 8.0, 7.0], [5.0, 4.0, 1.0]]])
      arg[2:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 2.0], [3.0, 8.0, 3.0], [2.0, 1.0, 7.0], [2.0, 6.0, 6.0]], [[6.0, 3.0, 2.0], [3.0, 
7.0, 5.0], [2.0, 8.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 1.0, 2.0], [4.0, 8.0, 7.0], [5.0, 4.0, 1.0], [6.0, 8.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 1.0, 6.0], [5.0, 8.0, 8.0], [8.0, 2.0, 7.0], [1.0, 5.0, 3.0]], [[8.0, 3.0, 3.0], [8.0, 
1.0, 1.0], [2.0, 2.0, 4.0], [5.0, 1.0, 8.0]], [[2.0, 2.0, 1.0], [4.0, 8.0, 7.0], [5.0, 4.0, 1.0], [2.0, 1.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[6.0, 6.0, 6.0], [6.0, 4.0, 4.0], [3.0, 4.0, 5.0], [3.0, 3.0, 3.0]], [[4.0, 6.0, 4.0], [6.0, 
1.0, 1.0], [7.0, 7.0, 3.0], [8.0, 6.0, 3.0]], [[4.0, 8.0, 1.0], [1.0, 2.0, 3.0], [2.0, 4.0, 6.0], [2.0, 6.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 3.0, 4.0], [2.0, 5.0, 3.0], [1.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[4.0, 2.0, 1.0], [5.0, 
1.0, 2.0], [8.0, 5.0, 6.0], [4.0, 5.0, 6.0]], [[6.0, 5.0, 8.0], [3.0, 3.0, 3.0], [1.0, 8.0, 1.0], [2.0, 8.0, 4.0]]])
      val=Data(numpy.array([[[6.0, 8.0, 1.0], [8.0, 2.0, 1.0]]]),self.functionspace)
      arg[2:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.0, 6.0, 6.0], [6.0, 4.0, 4.0], [3.0, 4.0, 5.0], [3.0, 3.0, 3.0]], [[4.0, 6.0, 4.0], [6.0, 
1.0, 1.0], [7.0, 7.0, 3.0], [8.0, 6.0, 3.0]], [[4.0, 8.0, 1.0], [6.0, 8.0, 1.0], [8.0, 2.0, 1.0], [2.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 3.0, 4.0], [2.0, 5.0, 3.0], [1.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[4.0, 2.0, 1.0], [5.0, 
1.0, 2.0], [8.0, 5.0, 6.0], [4.0, 5.0, 6.0]], [[6.0, 5.0, 8.0], [6.0, 8.0, 1.0], [8.0, 2.0, 1.0], [2.0, 8.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 8.0, 6.0], [4.0, 5.0, 4.0], [7.0, 6.0, 1.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 2.0], [5.0, 
4.0, 8.0], [5.0, 7.0, 7.0], [8.0, 1.0, 1.0]], [[7.0, 6.0, 4.0], [7.0, 1.0, 2.0], [6.0, 3.0, 8.0], [3.0, 3.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 2.0, 8.0], [8.0, 7.0, 3.0], [2.0, 8.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 8.0, 7.0], [7.0, 
1.0, 3.0], [1.0, 5.0, 8.0], [2.0, 6.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 3.0, 4.0], [4.0, 4.0, 3.0], [5.0, 3.0, 2.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 8.0, 4.0], [1.0, 4.0, 1.0]]])+(1.-msk_val)*numpy.array([[[6.0, 3.0, 6.0], [3.0, 8.0, 
8.0]]])
      arg[2:,1:3]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 8.0, 6.0], [4.0, 5.0, 4.0], [7.0, 6.0, 1.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 2.0], [5.0, 
4.0, 8.0], [5.0, 7.0, 7.0], [8.0, 1.0, 1.0]], [[7.0, 6.0, 4.0], [1.0, 8.0, 4.0], [1.0, 4.0, 1.0], [3.0, 3.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 2.0, 8.0], [8.0, 7.0, 3.0], [2.0, 8.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 8.0, 7.0], [7.0, 
1.0, 3.0], [1.0, 5.0, 8.0], [2.0, 6.0, 2.0]], [[2.0, 3.0, 4.0], [6.0, 3.0, 6.0], [3.0, 8.0, 8.0], [5.0, 3.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 6.0, 3.0], [5.0, 5.0, 7.0], [5.0, 6.0, 1.0], [4.0, 8.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 
4.0, 2.0], [4.0, 6.0, 3.0], [1.0, 5.0, 5.0]], [[5.0, 1.0, 4.0], [5.0, 4.0, 6.0], [6.0, 2.0, 6.0], [2.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 4.0, 6.0], [4.0, 6.0, 2.0], [6.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[7.0, 8.0, 4.0], [7.0, 
8.0, 4.0], [5.0, 1.0, 1.0], [8.0, 4.0, 6.0]], [[5.0, 4.0, 2.0], [2.0, 6.0, 8.0], [1.0, 1.0, 7.0], [6.0, 6.0, 1.0]]])
      val=numpy.array([[[4.0, 7.0, 8.0], [5.0, 8.0, 3.0]]])
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 3.0], [5.0, 5.0, 7.0], [5.0, 6.0, 1.0], [4.0, 8.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 
4.0, 2.0], [4.0, 7.0, 8.0], [5.0, 8.0, 3.0]], [[5.0, 1.0, 4.0], [5.0, 4.0, 6.0], [6.0, 2.0, 6.0], [2.0, 6.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 4.0, 6.0], [4.0, 6.0, 2.0], [6.0, 7.0, 8.0], [7.0, 1.0, 4.0]], [[7.0, 8.0, 4.0], [7.0, 
8.0, 4.0], [4.0, 7.0, 8.0], [5.0, 8.0, 3.0]], [[5.0, 4.0, 2.0], [2.0, 6.0, 8.0], [1.0, 1.0, 7.0], [6.0, 6.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 5.0, 3.0], [2.0, 1.0, 7.0], [7.0, 7.0, 2.0], [8.0, 6.0, 8.0]], [[3.0, 1.0, 6.0], [2.0, 
4.0, 1.0], [2.0, 6.0, 1.0], [3.0, 5.0, 4.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 6.0], [8.0, 8.0, 8.0], [1.0, 1.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 5.0, 8.0], [1.0, 1.0, 1.0], [7.0, 7.0, 6.0], [4.0, 5.0, 6.0]], [[5.0, 8.0, 4.0], [8.0, 
8.0, 6.0], [3.0, 2.0, 1.0], [4.0, 6.0, 4.0]], [[8.0, 5.0, 2.0], [1.0, 6.0, 1.0], [7.0, 3.0, 4.0], [2.0, 6.0, 2.0]]])
      val=Data(numpy.array([[[8.0, 6.0, 1.0], [1.0, 6.0, 4.0]]]),self.functionspace)
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 5.0, 3.0], [2.0, 1.0, 7.0], [7.0, 7.0, 2.0], [8.0, 6.0, 8.0]], [[3.0, 1.0, 6.0], [2.0, 
4.0, 1.0], [8.0, 6.0, 1.0], [1.0, 6.0, 4.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 6.0], [8.0, 8.0, 8.0], [1.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 5.0, 8.0], [1.0, 1.0, 1.0], [7.0, 7.0, 6.0], [4.0, 5.0, 6.0]], [[5.0, 8.0, 4.0], [8.0, 
8.0, 6.0], [8.0, 6.0, 1.0], [1.0, 6.0, 4.0]], [[8.0, 5.0, 2.0], [1.0, 6.0, 1.0], [7.0, 3.0, 4.0], [2.0, 6.0, 2.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 6.0, 2.0], [1.0, 1.0, 3.0], [4.0, 6.0, 8.0], [2.0, 7.0, 1.0]], [[3.0, 7.0, 3.0], [5.0, 
3.0, 2.0], [1.0, 1.0, 5.0], [4.0, 4.0, 4.0]], [[7.0, 7.0, 3.0], [3.0, 3.0, 2.0], [3.0, 1.0, 5.0], [4.0, 3.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 7.0], [1.0, 1.0, 2.0], [1.0, 2.0, 4.0], [8.0, 5.0, 7.0]], [[4.0, 5.0, 2.0], [2.0, 
8.0, 6.0], [6.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[1.0, 5.0, 1.0], [1.0, 5.0, 2.0], [1.0, 7.0, 6.0], [7.0, 6.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 8.0, 2.0], [3.0, 7.0, 6.0]]])+(1.-msk_val)*numpy.array([[[3.0, 6.0, 6.0], [3.0, 4.0, 
4.0]]])
      arg[1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 6.0, 2.0], [1.0, 1.0, 3.0], [4.0, 6.0, 8.0], [2.0, 7.0, 1.0]], [[3.0, 7.0, 3.0], [5.0, 
3.0, 2.0], [7.0, 8.0, 2.0], [3.0, 7.0, 6.0]], [[7.0, 7.0, 3.0], [3.0, 3.0, 2.0], [3.0, 1.0, 5.0], [4.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 7.0], [1.0, 1.0, 2.0], [1.0, 2.0, 4.0], [8.0, 5.0, 7.0]], [[4.0, 5.0, 2.0], [2.0, 
8.0, 6.0], [3.0, 6.0, 6.0], [3.0, 4.0, 4.0]], [[1.0, 5.0, 1.0], [1.0, 5.0, 2.0], [1.0, 7.0, 6.0], [7.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[4.0, 4.0, 3.0], [7.0, 1.0, 1.0], [8.0, 4.0, 4.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 1.0], [2.0, 
6.0, 6.0], [3.0, 8.0, 6.0], [7.0, 3.0, 7.0]], [[5.0, 7.0, 6.0], [6.0, 7.0, 5.0], [7.0, 5.0, 3.0], [3.0, 5.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 3.0, 6.0], [8.0, 2.0, 3.0], [4.0, 6.0, 2.0], [6.0, 2.0, 1.0]], [[5.0, 6.0, 7.0], [3.0, 
6.0, 2.0], [8.0, 6.0, 4.0], [5.0, 3.0, 4.0]], [[5.0, 3.0, 6.0], [2.0, 2.0, 5.0], [3.0, 2.0, 5.0], [5.0, 5.0, 3.0]]])
      val=numpy.array([[[8.0, 8.0], [7.0, 7.0], [1.0, 4.0], [3.0, 4.0]]])
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.0, 4.0, 3.0], [7.0, 1.0, 1.0], [8.0, 4.0, 4.0], [1.0, 3.0, 7.0]], [[5.0, 7.0, 1.0], [2.0, 
6.0, 6.0], [3.0, 8.0, 6.0], [7.0, 3.0, 7.0]], [[8.0, 8.0, 6.0], [7.0, 7.0, 5.0], [1.0, 4.0, 3.0], [3.0, 4.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 3.0, 6.0], [8.0, 2.0, 3.0], [4.0, 6.0, 2.0], [6.0, 2.0, 1.0]], [[5.0, 6.0, 7.0], [3.0, 
6.0, 2.0], [8.0, 6.0, 4.0], [5.0, 3.0, 4.0]], [[8.0, 8.0, 6.0], [7.0, 7.0, 5.0], [1.0, 4.0, 5.0], [3.0, 4.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 4.0, 1.0], [3.0, 4.0, 1.0], [4.0, 3.0, 8.0], [5.0, 8.0, 2.0]], [[7.0, 4.0, 4.0], [8.0, 
7.0, 3.0], [2.0, 7.0, 7.0], [5.0, 5.0, 3.0]], [[4.0, 6.0, 4.0], [8.0, 1.0, 7.0], [1.0, 2.0, 2.0], [5.0, 6.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 2.0, 6.0], [1.0, 7.0, 3.0], [5.0, 5.0, 1.0], [4.0, 2.0, 5.0]], [[2.0, 3.0, 7.0], [8.0, 
5.0, 4.0], [8.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[4.0, 7.0, 8.0], [2.0, 4.0, 3.0], [5.0, 4.0, 8.0], [4.0, 8.0, 1.0]]])
      val=Data(numpy.array([[[2.0, 3.0], [8.0, 8.0], [1.0, 6.0], [7.0, 1.0]]]),self.functionspace)
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 4.0, 1.0], [3.0, 4.0, 1.0], [4.0, 3.0, 8.0], [5.0, 8.0, 2.0]], [[7.0, 4.0, 4.0], [8.0, 
7.0, 3.0], [2.0, 7.0, 7.0], [5.0, 5.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 8.0, 7.0], [1.0, 6.0, 2.0], [7.0, 1.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 2.0, 6.0], [1.0, 7.0, 3.0], [5.0, 5.0, 1.0], [4.0, 2.0, 5.0]], [[2.0, 3.0, 7.0], [8.0, 
5.0, 4.0], [8.0, 2.0, 3.0], [4.0, 3.0, 4.0]], [[2.0, 3.0, 8.0], [8.0, 8.0, 3.0], [1.0, 6.0, 8.0], [7.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 5.0, 4.0], [6.0, 8.0, 2.0], [6.0, 2.0, 5.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 1.0], [3.0, 
2.0, 8.0], [1.0, 3.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 3.0, 7.0], [1.0, 1.0, 3.0], [1.0, 2.0, 5.0], [2.0, 5.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 1.0, 7.0], [8.0, 3.0, 3.0], [5.0, 3.0, 8.0], [2.0, 8.0, 3.0]], [[7.0, 8.0, 3.0], [7.0, 
4.0, 8.0], [3.0, 2.0, 4.0], [8.0, 7.0, 2.0]], [[7.0, 3.0, 1.0], [8.0, 1.0, 2.0], [8.0, 5.0, 4.0], [6.0, 2.0, 8.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 1.0], [7.0, 3.0], [1.0, 2.0], [8.0, 4.0]]])+(1.-msk_val)*numpy.array([[[3.0, 7.0], [3.0, 
4.0], [4.0, 3.0], [4.0, 6.0]]])
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 5.0, 4.0], [6.0, 8.0, 2.0], [6.0, 2.0, 5.0], [4.0, 5.0, 1.0]], [[5.0, 1.0, 1.0], [3.0, 
2.0, 8.0], [1.0, 3.0, 5.0], [8.0, 1.0, 8.0]], [[8.0, 1.0, 7.0], [7.0, 3.0, 3.0], [1.0, 2.0, 5.0], [8.0, 4.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 7.0], [8.0, 3.0, 3.0], [5.0, 3.0, 8.0], [2.0, 8.0, 3.0]], [[7.0, 8.0, 3.0], [7.0, 
4.0, 8.0], [3.0, 2.0, 4.0], [8.0, 7.0, 2.0]], [[3.0, 7.0, 1.0], [3.0, 4.0, 2.0], [4.0, 3.0, 4.0], [4.0, 6.0, 8.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 1.0, 5.0], [4.0, 7.0, 7.0], [4.0, 3.0, 7.0], [2.0, 1.0, 3.0]], [[3.0, 6.0, 1.0], [5.0, 
4.0, 4.0], [5.0, 4.0, 3.0], [6.0, 2.0, 8.0]], [[1.0, 5.0, 1.0], [4.0, 3.0, 1.0], [8.0, 7.0, 6.0], [3.0, 3.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 6.0, 6.0], [7.0, 3.0, 8.0], [4.0, 1.0, 4.0], [1.0, 1.0, 7.0]], [[7.0, 4.0, 5.0], [7.0, 
7.0, 7.0], [7.0, 2.0, 1.0], [1.0, 5.0, 5.0]], [[6.0, 2.0, 6.0], [1.0, 6.0, 3.0], [7.0, 6.0, 5.0], [7.0, 5.0, 7.0]]])
      val=numpy.array([[[2.0, 3.0, 2.0], [1.0, 8.0, 1.0], [1.0, 4.0, 8.0]]])
      arg[1:2,:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 1.0, 5.0], [4.0, 7.0, 7.0], [4.0, 3.0, 7.0], [2.0, 1.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 
8.0, 1.0], [1.0, 4.0, 8.0], [6.0, 2.0, 8.0]], [[1.0, 5.0, 1.0], [4.0, 3.0, 1.0], [8.0, 7.0, 6.0], [3.0, 3.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 6.0, 6.0], [7.0, 3.0, 8.0], [4.0, 1.0, 4.0], [1.0, 1.0, 7.0]], [[2.0, 3.0, 2.0], [1.0, 
8.0, 1.0], [1.0, 4.0, 8.0], [1.0, 5.0, 5.0]], [[6.0, 2.0, 6.0], [1.0, 6.0, 3.0], [7.0, 6.0, 5.0], [7.0, 5.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 2.0, 5.0], [5.0, 3.0, 8.0], [5.0, 6.0, 5.0], [5.0, 7.0, 2.0]], [[5.0, 2.0, 3.0], [8.0, 
3.0, 1.0], [5.0, 5.0, 8.0], [6.0, 7.0, 1.0]], [[6.0, 5.0, 6.0], [8.0, 8.0, 5.0], [8.0, 3.0, 3.0], [8.0, 5.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[4.0, 7.0, 4.0], [1.0, 2.0, 2.0], [4.0, 1.0, 2.0], [2.0, 2.0, 1.0]], [[8.0, 4.0, 2.0], [8.0, 
7.0, 5.0], [8.0, 7.0, 1.0], [1.0, 5.0, 3.0]], [[5.0, 1.0, 6.0], [8.0, 8.0, 5.0], [7.0, 2.0, 8.0], [5.0, 6.0, 3.0]]])
      val=Data(numpy.array([[[2.0, 6.0, 1.0], [6.0, 7.0, 4.0], [2.0, 8.0, 3.0]]]),self.functionspace)
      arg[1:2,:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 2.0, 5.0], [5.0, 3.0, 8.0], [5.0, 6.0, 5.0], [5.0, 7.0, 2.0]], [[2.0, 6.0, 1.0], [6.0, 
7.0, 4.0], [2.0, 8.0, 3.0], [6.0, 7.0, 1.0]], [[6.0, 5.0, 6.0], [8.0, 8.0, 5.0], [8.0, 3.0, 3.0], [8.0, 5.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 7.0, 4.0], [1.0, 2.0, 2.0], [4.0, 1.0, 2.0], [2.0, 2.0, 1.0]], [[2.0, 6.0, 1.0], [6.0, 
7.0, 4.0], [2.0, 8.0, 3.0], [1.0, 5.0, 3.0]], [[5.0, 1.0, 6.0], [8.0, 8.0, 5.0], [7.0, 2.0, 8.0], [5.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 4.0, 6.0], [4.0, 4.0, 8.0], [8.0, 8.0, 3.0], [8.0, 4.0, 2.0]], [[2.0, 5.0, 4.0], [3.0, 
6.0, 8.0], [7.0, 2.0, 8.0], [2.0, 8.0, 6.0]], [[2.0, 8.0, 3.0], [5.0, 1.0, 4.0], [7.0, 7.0, 1.0], [2.0, 4.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 1.0, 3.0], [1.0, 2.0, 6.0], [6.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 7.0, 3.0], [2.0, 
3.0, 7.0], [8.0, 5.0, 3.0], [3.0, 8.0, 4.0]], [[3.0, 2.0, 5.0], [7.0, 6.0, 4.0], [6.0, 5.0, 3.0], [8.0, 7.0, 4.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 7.0, 6.0], [3.0, 2.0, 5.0], [7.0, 8.0, 2.0]]])+(1.-msk_val)*numpy.array([[[6.0, 2.0, 
3.0], [3.0, 4.0, 7.0], [4.0, 6.0, 5.0]]])
      arg[1:2,:3,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 4.0, 6.0], [4.0, 4.0, 8.0], [8.0, 8.0, 3.0], [8.0, 4.0, 2.0]], [[8.0, 7.0, 6.0], [3.0, 
2.0, 5.0], [7.0, 8.0, 2.0], [2.0, 8.0, 6.0]], [[2.0, 8.0, 3.0], [5.0, 1.0, 4.0], [7.0, 7.0, 1.0], [2.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 3.0], [1.0, 2.0, 6.0], [6.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[6.0, 2.0, 3.0], [3.0, 
4.0, 7.0], [4.0, 6.0, 5.0], [3.0, 8.0, 4.0]], [[3.0, 2.0, 5.0], [7.0, 6.0, 4.0], [6.0, 5.0, 3.0], [8.0, 7.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_RUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 7.0, 4.0], [2.0, 4.0, 4.0], [3.0, 2.0, 8.0], [3.0, 7.0, 8.0]], [[7.0, 6.0, 4.0], [8.0, 
1.0, 3.0], [7.0, 6.0, 6.0], [7.0, 4.0, 7.0]], [[1.0, 2.0, 8.0], [2.0, 5.0, 6.0], [2.0, 5.0, 2.0], [4.0, 4.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 4.0, 1.0], [6.0, 5.0, 7.0], [7.0, 5.0, 4.0], [1.0, 6.0, 4.0]], [[6.0, 8.0, 3.0], [8.0, 
5.0, 3.0], [3.0, 7.0, 4.0], [8.0, 5.0, 7.0]], [[3.0, 3.0, 2.0], [4.0, 8.0, 3.0], [3.0, 2.0, 5.0], [7.0, 6.0, 7.0]]])
      val=numpy.array([[4.0, 3.0, 1.0]])
      arg[1:2,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 7.0, 4.0], [2.0, 4.0, 4.0], [3.0, 2.0, 8.0], [3.0, 7.0, 8.0]], [[7.0, 4.0, 4.0], [8.0, 
3.0, 3.0], [7.0, 1.0, 6.0], [7.0, 4.0, 7.0]], [[1.0, 2.0, 8.0], [2.0, 5.0, 6.0], [2.0, 5.0, 2.0], [4.0, 4.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 4.0, 1.0], [6.0, 5.0, 7.0], [7.0, 5.0, 4.0], [1.0, 6.0, 4.0]], [[6.0, 4.0, 3.0], [8.0, 
3.0, 3.0], [3.0, 1.0, 4.0], [8.0, 5.0, 7.0]], [[3.0, 3.0, 2.0], [4.0, 8.0, 3.0], [3.0, 2.0, 5.0], [7.0, 6.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_RUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 3.0, 6.0], [7.0, 3.0, 3.0], [7.0, 3.0, 6.0], [4.0, 8.0, 6.0]], [[3.0, 7.0, 6.0], [7.0, 
6.0, 7.0], [1.0, 2.0, 5.0], [6.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [4.0, 4.0, 7.0], [3.0, 2.0, 4.0], [2.0, 1.0, 
5.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 3.0, 7.0], [5.0, 2.0, 2.0], [7.0, 1.0, 8.0], [8.0, 3.0, 1.0]], [[2.0, 7.0, 4.0], [2.0, 
8.0, 4.0], [2.0, 5.0, 5.0], [6.0, 3.0, 6.0]], [[4.0, 5.0, 3.0], [5.0, 4.0, 3.0], [6.0, 3.0, 7.0], [1.0, 5.0, 1.0]]])
      val=Data(numpy.array([[8.0, 8.0, 7.0]]),self.functionspace)
      arg[1:2,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0, 3.0, 6.0], [7.0, 3.0, 3.0], [7.0, 3.0, 6.0], [4.0, 8.0, 6.0]], [[3.0, 8.0, 6.0], [7.0, 
8.0, 7.0], [1.0, 7.0, 5.0], [6.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [4.0, 4.0, 7.0], [3.0, 2.0, 4.0], [2.0, 1.0, 
5.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 3.0, 7.0], [5.0, 2.0, 2.0], [7.0, 1.0, 8.0], [8.0, 3.0, 1.0]], [[2.0, 8.0, 4.0], [2.0, 
8.0, 4.0], [2.0, 7.0, 5.0], [6.0, 3.0, 6.0]], [[4.0, 5.0, 3.0], [5.0, 4.0, 3.0], [6.0, 3.0, 7.0], [1.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_RUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 1.0, 2.0], [2.0, 2.0, 1.0], [2.0, 4.0, 7.0], [6.0, 5.0, 3.0]], [[6.0, 3.0, 2.0], [6.0, 
8.0, 2.0], [3.0, 6.0, 8.0], [4.0, 8.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 6.0, 7.0], [8.0, 8.0, 6.0], [6.0, 8.0, 
6.0]]])+(1.-msk_arg)*numpy.array([[[6.0, 1.0, 1.0], [5.0, 5.0, 1.0], [5.0, 3.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 7.0, 2.0], [6.0, 
3.0, 3.0], [5.0, 5.0, 8.0], [2.0, 2.0, 3.0]], [[5.0, 2.0, 5.0], [1.0, 6.0, 6.0], [3.0, 7.0, 3.0], [7.0, 1.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 4.0, 3.0]])+(1.-msk_val)*numpy.array([[6.0, 3.0, 4.0]])
      arg[1:2,:3,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 1.0, 2.0], [2.0, 2.0, 1.0], [2.0, 4.0, 7.0], [6.0, 5.0, 3.0]], [[6.0, 7.0, 2.0], [6.0, 
4.0, 2.0], [3.0, 3.0, 8.0], [4.0, 8.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 6.0, 7.0], [8.0, 8.0, 6.0], [6.0, 8.0, 
6.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 1.0], [5.0, 5.0, 1.0], [5.0, 3.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 6.0, 2.0], [6.0, 
3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 2.0, 3.0]], [[5.0, 2.0, 5.0], [1.0, 6.0, 6.0], [3.0, 7.0, 3.0], [7.0, 1.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_LPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 4.0, 1.0], [4.0, 7.0, 8.0], [7.0, 8.0, 3.0], [7.0, 5.0, 3.0]], [[5.0, 6.0, 4.0], [8.0, 
3.0, 8.0], [4.0, 1.0, 4.0], [3.0, 5.0, 5.0]], [[2.0, 6.0, 1.0], [7.0, 1.0, 2.0], [2.0, 6.0, 4.0], [7.0, 6.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[3.0, 3.0, 6.0], [8.0, 2.0, 6.0], [8.0, 5.0, 7.0], [5.0, 8.0, 2.0]], [[1.0, 2.0, 3.0], [4.0, 
2.0, 5.0], [6.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 1.0, 5.0], [5.0, 1.0, 3.0], [8.0, 6.0, 5.0]]])
      val=numpy.array([[8.0, 2.0, 8.0]])
      arg[2:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 4.0, 1.0], [4.0, 7.0, 8.0], [7.0, 8.0, 3.0], [7.0, 5.0, 3.0]], [[5.0, 6.0, 4.0], [8.0, 
3.0, 8.0], [4.0, 1.0, 4.0], [3.0, 5.0, 5.0]], [[2.0, 6.0, 1.0], [7.0, 1.0, 2.0], [8.0, 2.0, 8.0], [7.0, 6.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 3.0, 6.0], [8.0, 2.0, 6.0], [8.0, 5.0, 7.0], [5.0, 8.0, 2.0]], [[1.0, 2.0, 3.0], [4.0, 
2.0, 5.0], [6.0, 7.0, 7.0], [5.0, 8.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 1.0, 5.0], [8.0, 2.0, 8.0], [8.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_LPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[3.0, 5.0, 7.0], [4.0, 4.0, 3.0], [8.0, 7.0, 2.0], [8.0, 6.0, 7.0]], [[7.0, 8.0, 1.0], [6.0, 
5.0, 4.0], [7.0, 7.0, 5.0], [8.0, 4.0, 2.0]], [[1.0, 3.0, 6.0], [5.0, 7.0, 3.0], [4.0, 5.0, 3.0], [5.0, 6.0, 
7.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 3.0, 7.0], [5.0, 2.0, 8.0], [3.0, 2.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 5.0, 1.0], [5.0, 
4.0, 2.0], [5.0, 3.0, 8.0], [5.0, 1.0, 5.0]], [[3.0, 2.0, 1.0], [1.0, 1.0, 4.0], [5.0, 8.0, 7.0], [5.0, 8.0, 7.0]]])
      val=Data(numpy.array([[1.0, 4.0, 1.0]]),self.functionspace)
      arg[2:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0, 5.0, 7.0], [4.0, 4.0, 3.0], [8.0, 7.0, 2.0], [8.0, 6.0, 7.0]], [[7.0, 8.0, 1.0], [6.0, 
5.0, 4.0], [7.0, 7.0, 5.0], [8.0, 4.0, 2.0]], [[1.0, 3.0, 6.0], [5.0, 7.0, 3.0], [1.0, 4.0, 1.0], [5.0, 6.0, 
7.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 3.0, 7.0], [5.0, 2.0, 8.0], [3.0, 2.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 5.0, 1.0], [5.0, 
4.0, 2.0], [5.0, 3.0, 8.0], [5.0, 1.0, 5.0]], [[3.0, 2.0, 1.0], [1.0, 1.0, 4.0], [1.0, 4.0, 1.0], [5.0, 8.0, 7.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_LPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 3.0, 2.0], [6.0, 6.0, 2.0], [7.0, 4.0, 3.0], [6.0, 6.0, 6.0]], [[6.0, 6.0, 3.0], [6.0, 
8.0, 3.0], [7.0, 6.0, 5.0], [1.0, 8.0, 1.0]], [[5.0, 8.0, 5.0], [7.0, 2.0, 6.0], [8.0, 5.0, 5.0], [4.0, 8.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 7.0, 8.0], [4.0, 4.0, 1.0], [4.0, 6.0, 2.0], [4.0, 8.0, 6.0]], [[2.0, 3.0, 1.0], [6.0, 
7.0, 8.0], [6.0, 7.0, 8.0], [1.0, 6.0, 8.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 5.0], [2.0, 1.0, 4.0], [8.0, 5.0, 1.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 6.0, 2.0]])+(1.-msk_val)*numpy.array([[5.0, 8.0, 3.0]])
      arg[2:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 3.0, 2.0], [6.0, 6.0, 2.0], [7.0, 4.0, 3.0], [6.0, 6.0, 6.0]], [[6.0, 6.0, 3.0], [6.0, 
8.0, 3.0], [7.0, 6.0, 5.0], [1.0, 8.0, 1.0]], [[5.0, 8.0, 5.0], [7.0, 2.0, 6.0], [1.0, 6.0, 2.0], [4.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 7.0, 8.0], [4.0, 4.0, 1.0], [4.0, 6.0, 2.0], [4.0, 8.0, 6.0]], [[2.0, 3.0, 1.0], [6.0, 
7.0, 8.0], [6.0, 7.0, 8.0], [1.0, 6.0, 8.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 5.0], [5.0, 8.0, 3.0], [8.0, 5.0, 1.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_UCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[1.0, 6.0, 6.0], [1.0, 7.0, 3.0], [3.0, 2.0, 7.0], [6.0, 4.0, 6.0]], [[3.0, 7.0, 3.0], [7.0, 
6.0, 6.0], [6.0, 2.0, 4.0], [5.0, 3.0, 3.0]], [[6.0, 6.0, 3.0], [7.0, 6.0, 1.0], [2.0, 3.0, 7.0], [1.0, 4.0, 
4.0]]])+(1.-msk_arg)*numpy.array([[[2.0, 1.0, 1.0], [8.0, 2.0, 1.0], [8.0, 8.0, 4.0], [5.0, 4.0, 8.0]], [[8.0, 7.0, 4.0], [7.0, 
8.0, 1.0], [7.0, 5.0, 8.0], [3.0, 3.0, 6.0]], [[1.0, 4.0, 1.0], [1.0, 6.0, 4.0], [1.0, 4.0, 2.0], [5.0, 2.0, 5.0]]])
      val=numpy.array([[[8.0, 6.0, 5.0], [3.0, 4.0, 6.0], [1.0, 3.0, 5.0], [6.0, 7.0, 8.0]], [[5.0, 3.0, 8.0], [7.0, 4.0, 3.0], 
[6.0, 5.0, 4.0], [3.0, 3.0, 6.0]]])
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 5.0], [3.0, 4.0, 6.0], [1.0, 3.0, 5.0], [6.0, 7.0, 8.0]], [[5.0, 3.0, 8.0], [7.0, 
4.0, 3.0], [6.0, 5.0, 4.0], [3.0, 3.0, 6.0]], [[6.0, 6.0, 3.0], [7.0, 6.0, 1.0], [2.0, 3.0, 7.0], [1.0, 4.0, 
4.0]]])+(1.-msk_ref)*numpy.array([[[8.0, 6.0, 5.0], [3.0, 4.0, 6.0], [1.0, 3.0, 5.0], [6.0, 7.0, 8.0]], [[5.0, 3.0, 8.0], [7.0, 
4.0, 3.0], [6.0, 5.0, 4.0], [3.0, 3.0, 6.0]], [[1.0, 4.0, 1.0], [1.0, 6.0, 4.0], [1.0, 4.0, 2.0], [5.0, 2.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_UCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[5.0, 1.0, 8.0], [7.0, 7.0, 4.0], [5.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[8.0, 4.0, 1.0], [6.0, 
8.0, 6.0], [3.0, 4.0, 6.0], [1.0, 7.0, 7.0]], [[7.0, 6.0, 8.0], [1.0, 5.0, 8.0], [5.0, 4.0, 3.0], [7.0, 6.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 6.0, 7.0], [6.0, 4.0, 5.0], [6.0, 1.0, 1.0], [2.0, 7.0, 6.0]], [[8.0, 3.0, 2.0], [1.0, 
3.0, 7.0], [6.0, 7.0, 7.0], [1.0, 5.0, 6.0]], [[4.0, 7.0, 3.0], [7.0, 1.0, 8.0], [3.0, 7.0, 3.0], [4.0, 1.0, 5.0]]])
      val=Data(numpy.array([[[5.0, 8.0, 5.0], [5.0, 8.0, 3.0], [3.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 3.0, 
4.0], [5.0, 8.0, 2.0], [4.0, 2.0, 1.0]]]),self.functionspace)
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.0, 8.0, 5.0], [5.0, 8.0, 3.0], [3.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 
3.0, 4.0], [5.0, 8.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 6.0, 8.0], [1.0, 5.0, 8.0], [5.0, 4.0, 3.0], [7.0, 6.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 8.0, 5.0], [5.0, 8.0, 3.0], [3.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 
3.0, 4.0], [5.0, 8.0, 2.0], [4.0, 2.0, 1.0]], [[4.0, 7.0, 3.0], [7.0, 1.0, 8.0], [3.0, 7.0, 3.0], [4.0, 1.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_UCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 8.0, 3.0], [4.0, 6.0, 2.0], [3.0, 8.0, 6.0], [1.0, 7.0, 8.0]], [[3.0, 4.0, 5.0], [2.0, 
7.0, 8.0], [6.0, 3.0, 5.0], [3.0, 2.0, 3.0]], [[6.0, 5.0, 5.0], [7.0, 6.0, 6.0], [3.0, 8.0, 3.0], [3.0, 1.0, 
1.0]]])+(1.-msk_arg)*numpy.array([[[8.0, 8.0, 6.0], [1.0, 2.0, 4.0], [4.0, 8.0, 3.0], [7.0, 7.0, 4.0]], [[1.0, 3.0, 3.0], [6.0, 
3.0, 3.0], [4.0, 1.0, 3.0], [7.0, 1.0, 4.0]], [[8.0, 3.0, 3.0], [6.0, 4.0, 1.0], [2.0, 4.0, 2.0], [2.0, 6.0, 5.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 8.0, 4.0], [3.0, 7.0, 4.0], [4.0, 5.0, 3.0], [7.0, 8.0, 1.0]], [[6.0, 5.0, 6.0], [4.0, 
6.0, 3.0], [5.0, 8.0, 5.0], [5.0, 6.0, 7.0]]])+(1.-msk_val)*numpy.array([[[6.0, 1.0, 4.0], [8.0, 8.0, 3.0], [3.0, 4.0, 3.0], 
[3.0, 7.0, 3.0]], [[1.0, 5.0, 6.0], [5.0, 5.0, 7.0], [6.0, 4.0, 8.0], [5.0, 3.0, 4.0]]])
      arg[:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 8.0, 4.0], [3.0, 7.0, 4.0], [4.0, 5.0, 3.0], [7.0, 8.0, 1.0]], [[6.0, 5.0, 6.0], [4.0, 
6.0, 3.0], [5.0, 8.0, 5.0], [5.0, 6.0, 7.0]], [[6.0, 5.0, 5.0], [7.0, 6.0, 6.0], [3.0, 8.0, 3.0], [3.0, 1.0, 
1.0]]])+(1.-msk_ref)*numpy.array([[[6.0, 1.0, 4.0], [8.0, 8.0, 3.0], [3.0, 4.0, 3.0], [3.0, 7.0, 3.0]], [[1.0, 5.0, 6.0], [5.0, 
5.0, 7.0], [6.0, 4.0, 8.0], [5.0, 3.0, 4.0]], [[8.0, 3.0, 3.0], [6.0, 4.0, 1.0], [2.0, 4.0, 2.0], [2.0, 6.0, 5.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_array_CPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[2.0, 5.0, 2.0], [8.0, 8.0, 7.0], [8.0, 1.0, 8.0], [8.0, 7.0, 6.0]], [[2.0, 8.0, 6.0], [3.0, 
5.0, 4.0], [6.0, 3.0, 5.0], [1.0, 7.0, 6.0]], [[3.0, 5.0, 4.0], [8.0, 8.0, 7.0], [4.0, 8.0, 7.0], [3.0, 8.0, 
8.0]]])+(1.-msk_arg)*numpy.array([[[7.0, 5.0, 7.0], [8.0, 2.0, 5.0], [3.0, 6.0, 1.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 6.0], [5.0, 
1.0, 3.0], [1.0, 1.0, 6.0], [2.0, 1.0, 7.0]], [[5.0, 8.0, 6.0], [3.0, 4.0, 5.0], [4.0, 2.0, 8.0], [3.0, 6.0, 3.0]]])
      val=numpy.array([[8.0, 7.0, 7.0], [1.0, 2.0, 3.0], [5.0, 4.0, 2.0]])
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0, 5.0, 2.0], [8.0, 8.0, 7.0], [8.0, 7.0, 7.0], [8.0, 7.0, 6.0]], [[2.0, 8.0, 6.0], [3.0, 
5.0, 4.0], [1.0, 2.0, 3.0], [1.0, 7.0, 6.0]], [[3.0, 5.0, 4.0], [8.0, 8.0, 7.0], [5.0, 4.0, 2.0], [3.0, 8.0, 
8.0]]])+(1.-msk_ref)*numpy.array([[[7.0, 5.0, 7.0], [8.0, 2.0, 5.0], [8.0, 7.0, 7.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 6.0], [5.0, 
1.0, 3.0], [1.0, 2.0, 3.0], [2.0, 1.0, 7.0]], [[5.0, 8.0, 6.0], [3.0, 4.0, 5.0], [5.0, 4.0, 2.0], [3.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_constData_CPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[8.0, 3.0, 4.0], [3.0, 6.0, 1.0], [6.0, 4.0, 2.0], [8.0, 6.0, 5.0]], [[8.0, 1.0, 5.0], [4.0, 
8.0, 8.0], [4.0, 8.0, 7.0], [8.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [3.0, 7.0, 1.0], [8.0, 6.0, 5.0], [5.0, 2.0, 
3.0]]])+(1.-msk_arg)*numpy.array([[[1.0, 5.0, 5.0], [2.0, 2.0, 7.0], [7.0, 1.0, 6.0], [1.0, 6.0, 7.0]], [[7.0, 6.0, 1.0], [8.0, 
6.0, 7.0], [8.0, 1.0, 2.0], [4.0, 6.0, 3.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 3.0], [1.0, 4.0, 5.0], [5.0, 3.0, 4.0]]])
      val=Data(numpy.array([[5.0, 3.0, 1.0], [4.0, 7.0, 4.0], [8.0, 5.0, 8.0]]),self.functionspace)
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0, 3.0, 4.0], [3.0, 6.0, 1.0], [5.0, 3.0, 1.0], [8.0, 6.0, 5.0]], [[8.0, 1.0, 5.0], [4.0, 
8.0, 8.0], [4.0, 7.0, 4.0], [8.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [3.0, 7.0, 1.0], [8.0, 5.0, 8.0], [5.0, 2.0, 
3.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 5.0, 5.0], [2.0, 2.0, 7.0], [5.0, 3.0, 1.0], [1.0, 6.0, 7.0]], [[7.0, 6.0, 1.0], [8.0, 
6.0, 7.0], [4.0, 7.0, 4.0], [4.0, 6.0, 3.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 3.0], [8.0, 5.0, 8.0], [5.0, 3.0, 4.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank3_expandedData_CPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[7.0, 8.0, 2.0], [8.0, 5.0, 8.0], [2.0, 3.0, 4.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [4.0, 
4.0, 4.0], [3.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[5.0, 2.0, 4.0], [7.0, 3.0, 4.0], [7.0, 1.0, 6.0], [4.0, 7.0, 
2.0]]])+(1.-msk_arg)*numpy.array([[[5.0, 7.0, 2.0], [8.0, 1.0, 6.0], [3.0, 5.0, 5.0], [2.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [5.0, 
8.0, 3.0], [6.0, 4.0, 6.0], [2.0, 4.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 1.0, 8.0], [3.0, 3.0, 4.0], [8.0, 6.0, 3.0]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 2.0, 5.0], [6.0, 7.0, 2.0], [7.0, 3.0, 3.0]])+(1.-msk_val)*numpy.array([[6.0, 3.0, 5.0], 
[6.0, 2.0, 2.0], [1.0, 4.0, 8.0]])
      arg[:,2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0, 8.0, 2.0], [8.0, 5.0, 8.0], [4.0, 2.0, 5.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [4.0, 
4.0, 4.0], [6.0, 7.0, 2.0], [1.0, 6.0, 3.0]], [[5.0, 2.0, 4.0], [7.0, 3.0, 4.0], [7.0, 3.0, 3.0], [4.0, 7.0, 
2.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0, 2.0], [8.0, 1.0, 6.0], [6.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[8.0, 5.0, 2.0], [5.0, 
8.0, 3.0], [6.0, 2.0, 2.0], [2.0, 4.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 1.0, 8.0], [1.0, 4.0, 8.0], [8.0, 6.0, 3.0]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 3.0], [1.0, 3.0, 5.0], [2.0, 4.0, 5.0]], [[1.0, 4.0, 8.0], [6.0, 2.0, 6.0], [3.0, 
5.0, 7.0]], [[1.0, 7.0, 6.0], [8.0, 4.0, 2.0], [4.0, 4.0, 4.0]]], [[[5.0, 7.0, 1.0], [4.0, 5.0, 8.0], [4.0, 2.0, 2.0]], [[1.0, 
2.0, 8.0], [7.0, 5.0, 7.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 2.0], [4.0, 4.0, 6.0], [8.0, 1.0, 1.0]]], [[[6.0, 1.0, 4.0], [3.0, 
7.0, 6.0], [7.0, 8.0, 1.0]], [[1.0, 8.0, 5.0], [6.0, 6.0, 7.0], [2.0, 8.0, 1.0]], [[3.0, 4.0, 6.0], [8.0, 5.0, 3.0], [8.0, 3.0, 
6.0]]], [[[8.0, 4.0, 5.0], [3.0, 3.0, 1.0], [6.0, 5.0, 3.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 5.0], [4.0, 2.0, 2.0]], [[3.0, 3.0, 
4.0], [7.0, 3.0, 3.0], [6.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 4.0]], 
[[3.0, 8.0, 6.0], [2.0, 1.0, 8.0], [4.0, 8.0, 2.0]], [[1.0, 2.0, 7.0], [6.0, 5.0, 1.0], [8.0, 7.0, 3.0]]], [[[7.0, 1.0, 1.0], 
[4.0, 5.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 3.0, 1.0], [5.0, 8.0, 5.0], [3.0, 2.0, 5.0]], [[8.0, 8.0, 8.0], [6.0, 4.0, 4.0], 
[5.0, 8.0, 1.0]]], [[[5.0, 3.0, 5.0], [2.0, 4.0, 8.0], [5.0, 6.0, 7.0]], [[1.0, 2.0, 4.0], [1.0, 2.0, 7.0], [3.0, 7.0, 6.0]], 
[[3.0, 7.0, 8.0], [6.0, 5.0, 7.0], [8.0, 8.0, 2.0]]], [[[4.0, 3.0, 8.0], [7.0, 1.0, 7.0], [1.0, 2.0, 5.0]], [[4.0, 8.0, 4.0], 
[1.0, 7.0, 1.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 1.0], [8.0, 5.0, 4.0], [3.0, 2.0, 7.0]]]])
      val=numpy.array([[[8.0, 5.0]], [[8.0, 8.0]], [[5.0, 6.0]]])
      arg[2,:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 3.0], [1.0, 3.0, 5.0], [2.0, 4.0, 5.0]], [[1.0, 4.0, 8.0], [6.0, 2.0, 6.0], [3.0, 
5.0, 7.0]], [[1.0, 7.0, 6.0], [8.0, 4.0, 2.0], [4.0, 4.0, 4.0]]], [[[5.0, 7.0, 1.0], [4.0, 5.0, 8.0], [4.0, 2.0, 2.0]], [[1.0, 
2.0, 8.0], [7.0, 5.0, 7.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 2.0], [4.0, 4.0, 6.0], [8.0, 1.0, 1.0]]], [[[6.0, 1.0, 4.0], [8.0, 
5.0, 6.0], [7.0, 8.0, 1.0]], [[1.0, 8.0, 5.0], [8.0, 8.0, 7.0], [2.0, 8.0, 1.0]], [[3.0, 4.0, 6.0], [5.0, 6.0, 3.0], [8.0, 3.0, 
6.0]]], [[[8.0, 4.0, 5.0], [3.0, 3.0, 1.0], [6.0, 5.0, 3.0]], [[7.0, 7.0, 3.0], [5.0, 5.0, 5.0], [4.0, 2.0, 2.0]], [[3.0, 3.0, 
4.0], [7.0, 3.0, 3.0], [6.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 5.0], [5.0, 1.0, 8.0], [3.0, 3.0, 4.0]], 
[[3.0, 8.0, 6.0], [2.0, 1.0, 8.0], [4.0, 8.0, 2.0]], [[1.0, 2.0, 7.0], [6.0, 5.0, 1.0], [8.0, 7.0, 3.0]]], [[[7.0, 1.0, 1.0], 
[4.0, 5.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 3.0, 1.0], [5.0, 8.0, 5.0], [3.0, 2.0, 5.0]], [[8.0, 8.0, 8.0], [6.0, 4.0, 4.0], 
[5.0, 8.0, 1.0]]], [[[5.0, 3.0, 5.0], [8.0, 5.0, 8.0], [5.0, 6.0, 7.0]], [[1.0, 2.0, 4.0], [8.0, 8.0, 7.0], [3.0, 7.0, 6.0]], 
[[3.0, 7.0, 8.0], [5.0, 6.0, 7.0], [8.0, 8.0, 2.0]]], [[[4.0, 3.0, 8.0], [7.0, 1.0, 7.0], [1.0, 2.0, 5.0]], [[4.0, 8.0, 4.0], 
[1.0, 7.0, 1.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 1.0], [8.0, 5.0, 4.0], [3.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 1.0], [6.0, 1.0, 7.0], [6.0, 6.0, 4.0]], [[4.0, 1.0, 5.0], [3.0, 6.0, 4.0], [4.0, 
3.0, 6.0]], [[2.0, 1.0, 5.0], [4.0, 7.0, 6.0], [1.0, 3.0, 1.0]]], [[[7.0, 6.0, 3.0], [5.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[6.0, 
1.0, 5.0], [6.0, 4.0, 5.0], [4.0, 5.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 8.0, 7.0], [1.0, 2.0, 1.0]]], [[[5.0, 7.0, 3.0], [7.0, 
1.0, 2.0], [6.0, 4.0, 1.0]], [[2.0, 7.0, 5.0], [3.0, 7.0, 3.0], [3.0, 3.0, 4.0]], [[5.0, 8.0, 3.0], [7.0, 1.0, 2.0], [5.0, 6.0, 
7.0]]], [[[2.0, 6.0, 8.0], [5.0, 3.0, 4.0], [8.0, 1.0, 2.0]], [[1.0, 2.0, 5.0], [3.0, 4.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 4.0, 
3.0], [7.0, 8.0, 2.0], [6.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 7.0], [7.0, 4.0, 4.0], [7.0, 7.0, 1.0]], 
[[1.0, 8.0, 7.0], [2.0, 8.0, 8.0], [4.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [3.0, 5.0, 8.0], [4.0, 1.0, 1.0]]], [[[3.0, 3.0, 4.0], 
[8.0, 5.0, 2.0], [1.0, 1.0, 7.0]], [[7.0, 1.0, 8.0], [5.0, 4.0, 1.0], [2.0, 3.0, 2.0]], [[7.0, 6.0, 8.0], [3.0, 7.0, 5.0], 
[8.0, 7.0, 1.0]]], [[[6.0, 6.0, 1.0], [2.0, 2.0, 3.0], [2.0, 5.0, 5.0]], [[8.0, 1.0, 3.0], [2.0, 3.0, 4.0], [3.0, 8.0, 7.0]], 
[[3.0, 8.0, 8.0], [3.0, 5.0, 6.0], [4.0, 8.0, 2.0]]], [[[4.0, 2.0, 8.0], [7.0, 1.0, 1.0], [6.0, 2.0, 6.0]], [[5.0, 6.0, 7.0], 
[1.0, 4.0, 3.0], [1.0, 3.0, 6.0]], [[8.0, 8.0, 5.0], [4.0, 2.0, 4.0], [5.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[1.0, 8.0]], [[2.0, 8.0]], [[7.0, 5.0]]]),self.functionspace)
      arg[2,:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 1.0], [6.0, 1.0, 7.0], [6.0, 6.0, 4.0]], [[4.0, 1.0, 5.0], [3.0, 6.0, 4.0], [4.0, 
3.0, 6.0]], [[2.0, 1.0, 5.0], [4.0, 7.0, 6.0], [1.0, 3.0, 1.0]]], [[[7.0, 6.0, 3.0], [5.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[6.0, 
1.0, 5.0], [6.0, 4.0, 5.0], [4.0, 5.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 8.0, 7.0], [1.0, 2.0, 1.0]]], [[[5.0, 7.0, 3.0], [1.0, 
8.0, 2.0], [6.0, 4.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 8.0, 3.0], [3.0, 3.0, 4.0]], [[5.0, 8.0, 3.0], [7.0, 5.0, 2.0], [5.0, 6.0, 
7.0]]], [[[2.0, 6.0, 8.0], [5.0, 3.0, 4.0], [8.0, 1.0, 2.0]], [[1.0, 2.0, 5.0], [3.0, 4.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 4.0, 
3.0], [7.0, 8.0, 2.0], [6.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 7.0], [7.0, 4.0, 4.0], [7.0, 7.0, 1.0]], 
[[1.0, 8.0, 7.0], [2.0, 8.0, 8.0], [4.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [3.0, 5.0, 8.0], [4.0, 1.0, 1.0]]], [[[3.0, 3.0, 4.0], 
[8.0, 5.0, 2.0], [1.0, 1.0, 7.0]], [[7.0, 1.0, 8.0], [5.0, 4.0, 1.0], [2.0, 3.0, 2.0]], [[7.0, 6.0, 8.0], [3.0, 7.0, 5.0], 
[8.0, 7.0, 1.0]]], [[[6.0, 6.0, 1.0], [1.0, 8.0, 3.0], [2.0, 5.0, 5.0]], [[8.0, 1.0, 3.0], [2.0, 8.0, 4.0], [3.0, 8.0, 7.0]], 
[[3.0, 8.0, 8.0], [7.0, 5.0, 6.0], [4.0, 8.0, 2.0]]], [[[4.0, 2.0, 8.0], [7.0, 1.0, 1.0], [6.0, 2.0, 6.0]], [[5.0, 6.0, 7.0], 
[1.0, 4.0, 3.0], [1.0, 3.0, 6.0]], [[8.0, 8.0, 5.0], [4.0, 2.0, 4.0], [5.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 7.0], [4.0, 2.0, 1.0], [4.0, 7.0, 2.0]], [[1.0, 1.0, 1.0], [4.0, 8.0, 6.0], [3.0, 
3.0, 4.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 7.0], [1.0, 5.0, 8.0]]], [[[4.0, 1.0, 1.0], [7.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[2.0, 
6.0, 8.0], [5.0, 1.0, 7.0], [5.0, 8.0, 7.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 8.0], [3.0, 3.0, 6.0]]], [[[5.0, 2.0, 4.0], [4.0, 
7.0, 3.0], [8.0, 8.0, 4.0]], [[4.0, 1.0, 3.0], [4.0, 4.0, 2.0], [1.0, 2.0, 6.0]], [[1.0, 7.0, 5.0], [5.0, 6.0, 5.0], [8.0, 3.0, 
2.0]]], [[[8.0, 3.0, 3.0], [7.0, 8.0, 3.0], [4.0, 8.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 
7.0], [6.0, 3.0, 7.0], [7.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 8.0], [4.0, 6.0, 7.0], [2.0, 6.0, 6.0]], 
[[3.0, 6.0, 4.0], [1.0, 7.0, 7.0], [7.0, 3.0, 3.0]], [[8.0, 8.0, 1.0], [3.0, 2.0, 8.0], [2.0, 7.0, 6.0]]], [[[1.0, 5.0, 4.0], 
[7.0, 3.0, 4.0], [7.0, 6.0, 1.0]], [[6.0, 2.0, 6.0], [8.0, 4.0, 6.0], [1.0, 5.0, 6.0]], [[2.0, 2.0, 5.0], [6.0, 6.0, 1.0], 
[2.0, 5.0, 1.0]]], [[[4.0, 4.0, 8.0], [4.0, 6.0, 8.0], [1.0, 4.0, 8.0]], [[6.0, 8.0, 8.0], [4.0, 5.0, 8.0], [6.0, 6.0, 4.0]], 
[[5.0, 2.0, 5.0], [8.0, 7.0, 1.0], [6.0, 8.0, 8.0]]], [[[6.0, 6.0, 5.0], [4.0, 3.0, 2.0], [4.0, 7.0, 1.0]], [[1.0, 7.0, 7.0], 
[1.0, 1.0, 6.0], [3.0, 6.0, 2.0]], [[8.0, 8.0, 7.0], [5.0, 8.0, 2.0], [3.0, 4.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 8.0]], [[4.0, 1.0]], [[3.0, 2.0]]])+(1.-msk_val)*numpy.array([[[3.0, 1.0]], [[8.0, 2.0]], 
[[4.0, 5.0]]])
      arg[2,:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [4.0, 2.0, 1.0], [4.0, 7.0, 2.0]], [[1.0, 1.0, 1.0], [4.0, 8.0, 6.0], [3.0, 
3.0, 4.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 7.0], [1.0, 5.0, 8.0]]], [[[4.0, 1.0, 1.0], [7.0, 4.0, 2.0], [6.0, 4.0, 1.0]], [[2.0, 
6.0, 8.0], [5.0, 1.0, 7.0], [5.0, 8.0, 7.0]], [[2.0, 8.0, 5.0], [7.0, 1.0, 8.0], [3.0, 3.0, 6.0]]], [[[5.0, 2.0, 4.0], [2.0, 
8.0, 3.0], [8.0, 8.0, 4.0]], [[4.0, 1.0, 3.0], [4.0, 1.0, 2.0], [1.0, 2.0, 6.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 5.0], [8.0, 3.0, 
2.0]]], [[[8.0, 3.0, 3.0], [7.0, 8.0, 3.0], [4.0, 8.0, 1.0]], [[1.0, 4.0, 7.0], [1.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 
7.0], [6.0, 3.0, 7.0], [7.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 8.0], [4.0, 6.0, 7.0], [2.0, 6.0, 6.0]], 
[[3.0, 6.0, 4.0], [1.0, 7.0, 7.0], [7.0, 3.0, 3.0]], [[8.0, 8.0, 1.0], [3.0, 2.0, 8.0], [2.0, 7.0, 6.0]]], [[[1.0, 5.0, 4.0], 
[7.0, 3.0, 4.0], [7.0, 6.0, 1.0]], [[6.0, 2.0, 6.0], [8.0, 4.0, 6.0], [1.0, 5.0, 6.0]], [[2.0, 2.0, 5.0], [6.0, 6.0, 1.0], 
[2.0, 5.0, 1.0]]], [[[4.0, 4.0, 8.0], [3.0, 1.0, 8.0], [1.0, 4.0, 8.0]], [[6.0, 8.0, 8.0], [8.0, 2.0, 8.0], [6.0, 6.0, 4.0]], 
[[5.0, 2.0, 5.0], [4.0, 5.0, 1.0], [6.0, 8.0, 8.0]]], [[[6.0, 6.0, 5.0], [4.0, 3.0, 2.0], [4.0, 7.0, 1.0]], [[1.0, 7.0, 7.0], 
[1.0, 1.0, 6.0], [3.0, 6.0, 2.0]], [[8.0, 8.0, 7.0], [5.0, 8.0, 2.0], [3.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 7.0], [5.0, 2.0, 8.0], [2.0, 4.0, 3.0]], [[6.0, 3.0, 4.0], [7.0, 7.0, 6.0], [6.0, 
3.0, 7.0]], [[8.0, 5.0, 3.0], [1.0, 5.0, 6.0], [1.0, 5.0, 3.0]]], [[[7.0, 4.0, 7.0], [5.0, 6.0, 4.0], [4.0, 8.0, 5.0]], [[3.0, 
6.0, 4.0], [1.0, 6.0, 2.0], [1.0, 7.0, 1.0]], [[2.0, 3.0, 3.0], [6.0, 8.0, 8.0], [7.0, 2.0, 1.0]]], [[[1.0, 4.0, 7.0], [8.0, 
2.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 5.0, 7.0], [1.0, 2.0, 3.0], [4.0, 1.0, 5.0]], [[2.0, 2.0, 2.0], [8.0, 7.0, 3.0], [5.0, 4.0, 
2.0]]], [[[4.0, 2.0, 5.0], [3.0, 4.0, 4.0], [2.0, 6.0, 7.0]], [[1.0, 1.0, 3.0], [5.0, 3.0, 7.0], [8.0, 4.0, 5.0]], [[7.0, 7.0, 
4.0], [3.0, 6.0, 4.0], [4.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 5.0], [5.0, 3.0, 2.0], [1.0, 2.0, 2.0]], 
[[6.0, 5.0, 3.0], [2.0, 2.0, 3.0], [8.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [1.0, 1.0, 3.0], [1.0, 1.0, 5.0]]], [[[3.0, 3.0, 3.0], 
[6.0, 7.0, 7.0], [6.0, 6.0, 1.0]], [[2.0, 8.0, 4.0], [8.0, 6.0, 8.0], [2.0, 8.0, 1.0]], [[2.0, 2.0, 6.0], [2.0, 5.0, 1.0], 
[5.0, 5.0, 3.0]]], [[[2.0, 2.0, 2.0], [4.0, 5.0, 2.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 4.0], [8.0, 3.0, 3.0]], 
[[3.0, 8.0, 6.0], [2.0, 3.0, 7.0], [4.0, 4.0, 7.0]]], [[[5.0, 5.0, 4.0], [3.0, 8.0, 2.0], [2.0, 4.0, 1.0]], [[8.0, 1.0, 8.0], 
[1.0, 3.0, 8.0], [8.0, 1.0, 6.0]], [[1.0, 2.0, 1.0], [5.0, 1.0, 2.0], [2.0, 5.0, 2.0]]]])
      val=numpy.array([[[[3.0], [4.0]]], [[[7.0], [5.0]]], [[[4.0], [3.0]]]])
      arg[:3,1:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 7.0], [5.0, 2.0, 8.0], [2.0, 4.0, 3.0]], [[6.0, 3.0, 3.0], [7.0, 7.0, 4.0], [6.0, 
3.0, 7.0]], [[8.0, 5.0, 3.0], [1.0, 5.0, 6.0], [1.0, 5.0, 3.0]]], [[[7.0, 4.0, 7.0], [5.0, 6.0, 4.0], [4.0, 8.0, 5.0]], [[3.0, 
6.0, 7.0], [1.0, 6.0, 5.0], [1.0, 7.0, 1.0]], [[2.0, 3.0, 3.0], [6.0, 8.0, 8.0], [7.0, 2.0, 1.0]]], [[[1.0, 4.0, 7.0], [8.0, 
2.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 5.0, 4.0], [1.0, 2.0, 3.0], [4.0, 1.0, 5.0]], [[2.0, 2.0, 2.0], [8.0, 7.0, 3.0], [5.0, 4.0, 
2.0]]], [[[4.0, 2.0, 5.0], [3.0, 4.0, 4.0], [2.0, 6.0, 7.0]], [[1.0, 1.0, 3.0], [5.0, 3.0, 7.0], [8.0, 4.0, 5.0]], [[7.0, 7.0, 
4.0], [3.0, 6.0, 4.0], [4.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [5.0, 3.0, 2.0], [1.0, 2.0, 2.0]], 
[[6.0, 5.0, 3.0], [2.0, 2.0, 4.0], [8.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [1.0, 1.0, 3.0], [1.0, 1.0, 5.0]]], [[[3.0, 3.0, 3.0], 
[6.0, 7.0, 7.0], [6.0, 6.0, 1.0]], [[2.0, 8.0, 7.0], [8.0, 6.0, 5.0], [2.0, 8.0, 1.0]], [[2.0, 2.0, 6.0], [2.0, 5.0, 1.0], 
[5.0, 5.0, 3.0]]], [[[2.0, 2.0, 2.0], [4.0, 5.0, 2.0], [2.0, 4.0, 2.0]], [[8.0, 8.0, 4.0], [8.0, 8.0, 3.0], [8.0, 3.0, 3.0]], 
[[3.0, 8.0, 6.0], [2.0, 3.0, 7.0], [4.0, 4.0, 7.0]]], [[[5.0, 5.0, 4.0], [3.0, 8.0, 2.0], [2.0, 4.0, 1.0]], [[8.0, 1.0, 8.0], 
[1.0, 3.0, 8.0], [8.0, 1.0, 6.0]], [[1.0, 2.0, 1.0], [5.0, 1.0, 2.0], [2.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 8.0], [8.0, 6.0, 4.0], [6.0, 5.0, 7.0]], [[1.0, 6.0, 7.0], [1.0, 7.0, 6.0], [3.0, 
7.0, 2.0]], [[1.0, 1.0, 3.0], [5.0, 4.0, 6.0], [1.0, 5.0, 7.0]]], [[[6.0, 1.0, 3.0], [5.0, 2.0, 4.0], [6.0, 1.0, 8.0]], [[5.0, 
1.0, 1.0], [1.0, 6.0, 6.0], [7.0, 2.0, 8.0]], [[3.0, 7.0, 2.0], [1.0, 1.0, 3.0], [6.0, 7.0, 3.0]]], [[[1.0, 6.0, 7.0], [8.0, 
1.0, 1.0], [6.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [3.0, 7.0, 6.0], [2.0, 8.0, 5.0]], [[5.0, 2.0, 8.0], [3.0, 8.0, 6.0], [2.0, 3.0, 
4.0]]], [[[7.0, 3.0, 1.0], [5.0, 3.0, 4.0], [8.0, 6.0, 4.0]], [[1.0, 3.0, 2.0], [1.0, 5.0, 7.0], [5.0, 8.0, 5.0]], [[8.0, 8.0, 
7.0], [2.0, 4.0, 7.0], [7.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 2.0], [5.0, 6.0, 3.0], [5.0, 1.0, 7.0]], 
[[1.0, 3.0, 4.0], [8.0, 5.0, 2.0], [1.0, 8.0, 7.0]], [[6.0, 5.0, 5.0], [4.0, 8.0, 2.0], [4.0, 1.0, 6.0]]], [[[1.0, 2.0, 4.0], 
[3.0, 2.0, 1.0], [5.0, 7.0, 2.0]], [[1.0, 5.0, 3.0], [1.0, 4.0, 8.0], [2.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 5.0], 
[5.0, 8.0, 4.0]]], [[[3.0, 3.0, 2.0], [1.0, 7.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 7.0, 3.0], [2.0, 7.0, 6.0], [7.0, 7.0, 1.0]], 
[[5.0, 1.0, 1.0], [5.0, 3.0, 2.0], [3.0, 7.0, 4.0]]], [[[8.0, 7.0, 2.0], [1.0, 7.0, 4.0], [7.0, 8.0, 4.0]], [[7.0, 1.0, 5.0], 
[7.0, 6.0, 8.0], [8.0, 5.0, 7.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 2.0], [5.0, 3.0, 5.0]]]])
      val=Data(numpy.array([[[[8.0], [7.0]]], [[[6.0], [2.0]]], [[[7.0], [3.0]]]]),self.functionspace)
      arg[:3,1:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 8.0], [8.0, 6.0, 4.0], [6.0, 5.0, 7.0]], [[1.0, 6.0, 8.0], [1.0, 7.0, 7.0], [3.0, 
7.0, 2.0]], [[1.0, 1.0, 3.0], [5.0, 4.0, 6.0], [1.0, 5.0, 7.0]]], [[[6.0, 1.0, 3.0], [5.0, 2.0, 4.0], [6.0, 1.0, 8.0]], [[5.0, 
1.0, 6.0], [1.0, 6.0, 2.0], [7.0, 2.0, 8.0]], [[3.0, 7.0, 2.0], [1.0, 1.0, 3.0], [6.0, 7.0, 3.0]]], [[[1.0, 6.0, 7.0], [8.0, 
1.0, 1.0], [6.0, 2.0, 6.0]], [[7.0, 1.0, 7.0], [3.0, 7.0, 3.0], [2.0, 8.0, 5.0]], [[5.0, 2.0, 8.0], [3.0, 8.0, 6.0], [2.0, 3.0, 
4.0]]], [[[7.0, 3.0, 1.0], [5.0, 3.0, 4.0], [8.0, 6.0, 4.0]], [[1.0, 3.0, 2.0], [1.0, 5.0, 7.0], [5.0, 8.0, 5.0]], [[8.0, 8.0, 
7.0], [2.0, 4.0, 7.0], [7.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 2.0], [5.0, 6.0, 3.0], [5.0, 1.0, 7.0]], 
[[1.0, 3.0, 8.0], [8.0, 5.0, 7.0], [1.0, 8.0, 7.0]], [[6.0, 5.0, 5.0], [4.0, 8.0, 2.0], [4.0, 1.0, 6.0]]], [[[1.0, 2.0, 4.0], 
[3.0, 2.0, 1.0], [5.0, 7.0, 2.0]], [[1.0, 5.0, 6.0], [1.0, 4.0, 2.0], [2.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 5.0], 
[5.0, 8.0, 4.0]]], [[[3.0, 3.0, 2.0], [1.0, 7.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 7.0, 7.0], [2.0, 7.0, 3.0], [7.0, 7.0, 1.0]], 
[[5.0, 1.0, 1.0], [5.0, 3.0, 2.0], [3.0, 7.0, 4.0]]], [[[8.0, 7.0, 2.0], [1.0, 7.0, 4.0], [7.0, 8.0, 4.0]], [[7.0, 1.0, 5.0], 
[7.0, 6.0, 8.0], [8.0, 5.0, 7.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 2.0], [5.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 7.0], [4.0, 5.0, 1.0], [5.0, 5.0, 2.0]], [[6.0, 8.0, 7.0], [3.0, 6.0, 5.0], [3.0, 
8.0, 8.0]], [[2.0, 4.0, 7.0], [6.0, 7.0, 1.0], [1.0, 6.0, 6.0]]], [[[2.0, 1.0, 8.0], [5.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[3.0, 
4.0, 6.0], [6.0, 5.0, 5.0], [6.0, 2.0, 3.0]], [[4.0, 3.0, 4.0], [8.0, 5.0, 1.0], [5.0, 1.0, 5.0]]], [[[3.0, 7.0, 7.0], [8.0, 
4.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 4.0, 1.0], [3.0, 6.0, 4.0], [7.0, 5.0, 7.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 5.0], [7.0, 7.0, 
3.0]]], [[[3.0, 8.0, 5.0], [4.0, 4.0, 6.0], [5.0, 5.0, 8.0]], [[3.0, 8.0, 1.0], [8.0, 2.0, 2.0], [5.0, 4.0, 5.0]], [[2.0, 2.0, 
2.0], [4.0, 8.0, 6.0], [6.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 3.0], [5.0, 4.0, 5.0], [8.0, 8.0, 8.0]], 
[[6.0, 6.0, 1.0], [5.0, 6.0, 2.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 5.0], [3.0, 2.0, 4.0], [3.0, 5.0, 2.0]]], [[[6.0, 8.0, 4.0], 
[5.0, 5.0, 5.0], [6.0, 4.0, 4.0]], [[4.0, 2.0, 3.0], [6.0, 7.0, 7.0], [4.0, 3.0, 8.0]], [[6.0, 6.0, 8.0], [7.0, 5.0, 5.0], 
[7.0, 7.0, 6.0]]], [[[3.0, 1.0, 5.0], [5.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 2.0, 4.0], [6.0, 1.0, 1.0], [5.0, 1.0, 3.0]], 
[[6.0, 4.0, 2.0], [4.0, 6.0, 6.0], [3.0, 8.0, 7.0]]], [[[3.0, 2.0, 3.0], [1.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[3.0, 2.0, 1.0], 
[5.0, 8.0, 6.0], [2.0, 3.0, 5.0]], [[4.0, 2.0, 8.0], [8.0, 8.0, 3.0], [6.0, 1.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0], [1.0]]], [[[4.0], [7.0]]], [[[7.0], [1.0]]]])+(1.-msk_val)*numpy.array([[[[6.0], 
[3.0]]], [[[6.0], [8.0]]], [[[6.0], [4.0]]]])
      arg[:3,1:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 7.0], [4.0, 5.0, 1.0], [5.0, 5.0, 2.0]], [[6.0, 8.0, 8.0], [3.0, 6.0, 1.0], [3.0, 
8.0, 8.0]], [[2.0, 4.0, 7.0], [6.0, 7.0, 1.0], [1.0, 6.0, 6.0]]], [[[2.0, 1.0, 8.0], [5.0, 4.0, 2.0], [1.0, 5.0, 6.0]], [[3.0, 
4.0, 4.0], [6.0, 5.0, 7.0], [6.0, 2.0, 3.0]], [[4.0, 3.0, 4.0], [8.0, 5.0, 1.0], [5.0, 1.0, 5.0]]], [[[3.0, 7.0, 7.0], [8.0, 
4.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 4.0, 7.0], [3.0, 6.0, 1.0], [7.0, 5.0, 7.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 5.0], [7.0, 7.0, 
3.0]]], [[[3.0, 8.0, 5.0], [4.0, 4.0, 6.0], [5.0, 5.0, 8.0]], [[3.0, 8.0, 1.0], [8.0, 2.0, 2.0], [5.0, 4.0, 5.0]], [[2.0, 2.0, 
2.0], [4.0, 8.0, 6.0], [6.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 3.0], [5.0, 4.0, 5.0], [8.0, 8.0, 8.0]], 
[[6.0, 6.0, 6.0], [5.0, 6.0, 3.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 5.0], [3.0, 2.0, 4.0], [3.0, 5.0, 2.0]]], [[[6.0, 8.0, 4.0], 
[5.0, 5.0, 5.0], [6.0, 4.0, 4.0]], [[4.0, 2.0, 6.0], [6.0, 7.0, 8.0], [4.0, 3.0, 8.0]], [[6.0, 6.0, 8.0], [7.0, 5.0, 5.0], 
[7.0, 7.0, 6.0]]], [[[3.0, 1.0, 5.0], [5.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 2.0, 6.0], [6.0, 1.0, 4.0], [5.0, 1.0, 3.0]], 
[[6.0, 4.0, 2.0], [4.0, 6.0, 6.0], [3.0, 8.0, 7.0]]], [[[3.0, 2.0, 3.0], [1.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[3.0, 2.0, 1.0], 
[5.0, 8.0, 6.0], [2.0, 3.0, 5.0]], [[4.0, 2.0, 8.0], [8.0, 8.0, 3.0], [6.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 8.0], [3.0, 1.0, 8.0], [5.0, 5.0, 1.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 5.0], [7.0, 
4.0, 4.0]], [[3.0, 7.0, 7.0], [3.0, 2.0, 2.0], [2.0, 3.0, 2.0]]], [[[2.0, 3.0, 4.0], [4.0, 6.0, 8.0], [6.0, 3.0, 3.0]], [[8.0, 
2.0, 2.0], [8.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[8.0, 7.0, 6.0], [5.0, 4.0, 7.0], [7.0, 3.0, 6.0]]], [[[2.0, 1.0, 2.0], [4.0, 
1.0, 6.0], [5.0, 1.0, 1.0]], [[3.0, 4.0, 7.0], [2.0, 5.0, 2.0], [6.0, 7.0, 1.0]], [[4.0, 6.0, 8.0], [2.0, 8.0, 4.0], [8.0, 4.0, 
4.0]]], [[[5.0, 8.0, 4.0], [5.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[2.0, 8.0, 8.0], [2.0, 1.0, 8.0], [1.0, 6.0, 2.0]], [[3.0, 7.0, 
3.0], [4.0, 2.0, 2.0], [7.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 3.0], [5.0, 6.0, 3.0], [6.0, 1.0, 1.0]], 
[[2.0, 3.0, 5.0], [1.0, 8.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 4.0, 4.0], [1.0, 6.0, 4.0]]], [[[8.0, 7.0, 6.0], 
[3.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 6.0], [6.0, 5.0, 7.0], [7.0, 6.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 3.0, 8.0], 
[5.0, 1.0, 6.0]]], [[[6.0, 5.0, 3.0], [4.0, 2.0, 1.0], [2.0, 7.0, 2.0]], [[1.0, 2.0, 1.0], [7.0, 8.0, 4.0], [1.0, 2.0, 2.0]], 
[[7.0, 6.0, 3.0], [4.0, 7.0, 6.0], [3.0, 2.0, 2.0]]], [[[8.0, 1.0, 4.0], [7.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 1.0], 
[6.0, 5.0, 1.0], [7.0, 5.0, 2.0]], [[6.0, 3.0, 2.0], [5.0, 7.0, 4.0], [5.0, 7.0, 4.0]]]])
      val=numpy.array([8.0])
      arg[2,1,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 8.0], [3.0, 1.0, 8.0], [5.0, 5.0, 1.0]], [[3.0, 8.0, 1.0], [2.0, 7.0, 5.0], [7.0, 
4.0, 4.0]], [[3.0, 7.0, 7.0], [3.0, 2.0, 2.0], [2.0, 3.0, 2.0]]], [[[2.0, 3.0, 4.0], [4.0, 6.0, 8.0], [6.0, 3.0, 3.0]], [[8.0, 
2.0, 2.0], [8.0, 4.0, 3.0], [4.0, 3.0, 4.0]], [[8.0, 7.0, 6.0], [5.0, 4.0, 7.0], [7.0, 3.0, 6.0]]], [[[2.0, 1.0, 2.0], [4.0, 
1.0, 6.0], [5.0, 1.0, 1.0]], [[3.0, 4.0, 7.0], [2.0, 8.0, 2.0], [6.0, 7.0, 1.0]], [[4.0, 6.0, 8.0], [2.0, 8.0, 4.0], [8.0, 4.0, 
4.0]]], [[[5.0, 8.0, 4.0], [5.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[2.0, 8.0, 8.0], [2.0, 1.0, 8.0], [1.0, 6.0, 2.0]], [[3.0, 7.0, 
3.0], [4.0, 2.0, 2.0], [7.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 3.0], [5.0, 6.0, 3.0], [6.0, 1.0, 1.0]], 
[[2.0, 3.0, 5.0], [1.0, 8.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 4.0, 4.0], [1.0, 6.0, 4.0]]], [[[8.0, 7.0, 6.0], 
[3.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 6.0], [6.0, 5.0, 7.0], [7.0, 6.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 3.0, 8.0], 
[5.0, 1.0, 6.0]]], [[[6.0, 5.0, 3.0], [4.0, 2.0, 1.0], [2.0, 7.0, 2.0]], [[1.0, 2.0, 1.0], [7.0, 8.0, 4.0], [1.0, 2.0, 2.0]], 
[[7.0, 6.0, 3.0], [4.0, 7.0, 6.0], [3.0, 2.0, 2.0]]], [[[8.0, 1.0, 4.0], [7.0, 5.0, 3.0], [3.0, 2.0, 6.0]], [[5.0, 2.0, 1.0], 
[6.0, 5.0, 1.0], [7.0, 5.0, 2.0]], [[6.0, 3.0, 2.0], [5.0, 7.0, 4.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 6.0], [7.0, 3.0, 2.0], [4.0, 6.0, 6.0]], [[2.0, 7.0, 8.0], [4.0, 4.0, 2.0], [5.0, 
3.0, 7.0]], [[2.0, 2.0, 4.0], [2.0, 1.0, 5.0], [8.0, 2.0, 7.0]]], [[[4.0, 4.0, 8.0], [3.0, 7.0, 8.0], [3.0, 3.0, 6.0]], [[5.0, 
7.0, 4.0], [3.0, 6.0, 6.0], [6.0, 6.0, 1.0]], [[1.0, 1.0, 5.0], [8.0, 1.0, 3.0], [1.0, 4.0, 5.0]]], [[[5.0, 7.0, 1.0], [5.0, 
4.0, 6.0], [1.0, 5.0, 1.0]], [[1.0, 2.0, 2.0], [4.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[3.0, 2.0, 5.0], [6.0, 5.0, 5.0], [1.0, 1.0, 
6.0]]], [[[8.0, 6.0, 2.0], [4.0, 4.0, 3.0], [5.0, 3.0, 6.0]], [[7.0, 8.0, 6.0], [3.0, 1.0, 2.0], [1.0, 6.0, 4.0]], [[1.0, 3.0, 
6.0], [4.0, 1.0, 1.0], [5.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 1.0], [6.0, 3.0, 4.0]], 
[[2.0, 2.0, 2.0], [4.0, 3.0, 5.0], [1.0, 3.0, 7.0]], [[1.0, 3.0, 6.0], [6.0, 6.0, 3.0], [8.0, 8.0, 3.0]]], [[[8.0, 4.0, 7.0], 
[3.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 7.0, 8.0], [5.0, 8.0, 3.0], [1.0, 1.0, 6.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 2.0], 
[4.0, 2.0, 2.0]]], [[[4.0, 6.0, 6.0], [8.0, 1.0, 8.0], [1.0, 5.0, 4.0]], [[1.0, 1.0, 4.0], [4.0, 6.0, 8.0], [3.0, 7.0, 1.0]], 
[[4.0, 8.0, 3.0], [8.0, 7.0, 6.0], [2.0, 4.0, 2.0]]], [[[5.0, 8.0, 5.0], [3.0, 5.0, 3.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 4.0], 
[4.0, 7.0, 5.0], [7.0, 7.0, 3.0]], [[2.0, 1.0, 1.0], [8.0, 3.0, 3.0], [1.0, 8.0, 5.0]]]])
      val=Data(numpy.array([2.0]),self.functionspace)
      arg[2,1,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [7.0, 3.0, 2.0], [4.0, 6.0, 6.0]], [[2.0, 7.0, 8.0], [4.0, 4.0, 2.0], [5.0, 
3.0, 7.0]], [[2.0, 2.0, 4.0], [2.0, 1.0, 5.0], [8.0, 2.0, 7.0]]], [[[4.0, 4.0, 8.0], [3.0, 7.0, 8.0], [3.0, 3.0, 6.0]], [[5.0, 
7.0, 4.0], [3.0, 6.0, 6.0], [6.0, 6.0, 1.0]], [[1.0, 1.0, 5.0], [8.0, 1.0, 3.0], [1.0, 4.0, 5.0]]], [[[5.0, 7.0, 1.0], [5.0, 
4.0, 6.0], [1.0, 5.0, 1.0]], [[1.0, 2.0, 2.0], [4.0, 2.0, 8.0], [4.0, 5.0, 6.0]], [[3.0, 2.0, 5.0], [6.0, 5.0, 5.0], [1.0, 1.0, 
6.0]]], [[[8.0, 6.0, 2.0], [4.0, 4.0, 3.0], [5.0, 3.0, 6.0]], [[7.0, 8.0, 6.0], [3.0, 1.0, 2.0], [1.0, 6.0, 4.0]], [[1.0, 3.0, 
6.0], [4.0, 1.0, 1.0], [5.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 5.0], [5.0, 7.0, 1.0], [6.0, 3.0, 4.0]], 
[[2.0, 2.0, 2.0], [4.0, 3.0, 5.0], [1.0, 3.0, 7.0]], [[1.0, 3.0, 6.0], [6.0, 6.0, 3.0], [8.0, 8.0, 3.0]]], [[[8.0, 4.0, 7.0], 
[3.0, 6.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 7.0, 8.0], [5.0, 8.0, 3.0], [1.0, 1.0, 6.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 2.0], 
[4.0, 2.0, 2.0]]], [[[4.0, 6.0, 6.0], [8.0, 1.0, 8.0], [1.0, 5.0, 4.0]], [[1.0, 1.0, 4.0], [4.0, 2.0, 8.0], [3.0, 7.0, 1.0]], 
[[4.0, 8.0, 3.0], [8.0, 7.0, 6.0], [2.0, 4.0, 2.0]]], [[[5.0, 8.0, 5.0], [3.0, 5.0, 3.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 4.0], 
[4.0, 7.0, 5.0], [7.0, 7.0, 3.0]], [[2.0, 1.0, 1.0], [8.0, 3.0, 3.0], [1.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 5.0], [7.0, 1.0, 5.0], [1.0, 3.0, 2.0]], [[6.0, 3.0, 5.0], [6.0, 6.0, 6.0], [3.0, 
3.0, 7.0]], [[3.0, 8.0, 3.0], [6.0, 8.0, 2.0], [6.0, 8.0, 8.0]]], [[[3.0, 2.0, 6.0], [8.0, 3.0, 2.0], [3.0, 1.0, 5.0]], [[3.0, 
3.0, 2.0], [8.0, 7.0, 6.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 4.0], [8.0, 7.0, 5.0], [7.0, 5.0, 5.0]]], [[[2.0, 1.0, 2.0], [2.0, 
1.0, 5.0], [2.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [7.0, 1.0, 1.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 2.0], [1.0, 8.0, 3.0], [4.0, 1.0, 
8.0]]], [[[2.0, 6.0, 6.0], [8.0, 1.0, 1.0], [2.0, 8.0, 7.0]], [[2.0, 5.0, 4.0], [2.0, 3.0, 5.0], [2.0, 3.0, 5.0]], [[2.0, 6.0, 
1.0], [2.0, 2.0, 5.0], [3.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 3.0], [4.0, 3.0, 1.0], [3.0, 1.0, 5.0]], 
[[4.0, 3.0, 3.0], [7.0, 4.0, 3.0], [5.0, 7.0, 7.0]], [[3.0, 7.0, 8.0], [8.0, 6.0, 2.0], [7.0, 3.0, 3.0]]], [[[5.0, 5.0, 4.0], 
[7.0, 5.0, 2.0], [2.0, 8.0, 2.0]], [[8.0, 3.0, 4.0], [4.0, 6.0, 5.0], [6.0, 1.0, 2.0]], [[7.0, 2.0, 6.0], [2.0, 4.0, 5.0], 
[8.0, 4.0, 4.0]]], [[[1.0, 6.0, 5.0], [4.0, 2.0, 8.0], [1.0, 8.0, 3.0]], [[6.0, 4.0, 7.0], [4.0, 1.0, 6.0], [3.0, 8.0, 6.0]], 
[[3.0, 4.0, 7.0], [2.0, 7.0, 5.0], [2.0, 3.0, 5.0]]], [[[2.0, 8.0, 6.0], [1.0, 6.0, 6.0], [3.0, 7.0, 8.0]], [[6.0, 2.0, 8.0], 
[6.0, 6.0, 8.0], [3.0, 6.0, 7.0]], [[5.0, 1.0, 8.0], [8.0, 2.0, 4.0], [3.0, 6.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0])+(1.-msk_val)*numpy.array([6.0])
      arg[2,1,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 5.0], [7.0, 1.0, 5.0], [1.0, 3.0, 2.0]], [[6.0, 3.0, 5.0], [6.0, 6.0, 6.0], [3.0, 
3.0, 7.0]], [[3.0, 8.0, 3.0], [6.0, 8.0, 2.0], [6.0, 8.0, 8.0]]], [[[3.0, 2.0, 6.0], [8.0, 3.0, 2.0], [3.0, 1.0, 5.0]], [[3.0, 
3.0, 2.0], [8.0, 7.0, 6.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 4.0], [8.0, 7.0, 5.0], [7.0, 5.0, 5.0]]], [[[2.0, 1.0, 2.0], [2.0, 
1.0, 5.0], [2.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 1.0], [7.0, 1.0, 2.0]], [[8.0, 2.0, 2.0], [1.0, 8.0, 3.0], [4.0, 1.0, 
8.0]]], [[[2.0, 6.0, 6.0], [8.0, 1.0, 1.0], [2.0, 8.0, 7.0]], [[2.0, 5.0, 4.0], [2.0, 3.0, 5.0], [2.0, 3.0, 5.0]], [[2.0, 6.0, 
1.0], [2.0, 2.0, 5.0], [3.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [4.0, 3.0, 1.0], [3.0, 1.0, 5.0]], 
[[4.0, 3.0, 3.0], [7.0, 4.0, 3.0], [5.0, 7.0, 7.0]], [[3.0, 7.0, 8.0], [8.0, 6.0, 2.0], [7.0, 3.0, 3.0]]], [[[5.0, 5.0, 4.0], 
[7.0, 5.0, 2.0], [2.0, 8.0, 2.0]], [[8.0, 3.0, 4.0], [4.0, 6.0, 5.0], [6.0, 1.0, 2.0]], [[7.0, 2.0, 6.0], [2.0, 4.0, 5.0], 
[8.0, 4.0, 4.0]]], [[[1.0, 6.0, 5.0], [4.0, 2.0, 8.0], [1.0, 8.0, 3.0]], [[6.0, 4.0, 7.0], [4.0, 6.0, 6.0], [3.0, 8.0, 6.0]], 
[[3.0, 4.0, 7.0], [2.0, 7.0, 5.0], [2.0, 3.0, 5.0]]], [[[2.0, 8.0, 6.0], [1.0, 6.0, 6.0], [3.0, 7.0, 8.0]], [[6.0, 2.0, 8.0], 
[6.0, 6.0, 8.0], [3.0, 6.0, 7.0]], [[5.0, 1.0, 8.0], [8.0, 2.0, 4.0], [3.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 3.0], [1.0, 7.0, 6.0], [7.0, 6.0, 6.0]], [[6.0, 1.0, 4.0], [4.0, 5.0, 6.0], [2.0, 
5.0, 5.0]], [[2.0, 5.0, 5.0], [7.0, 6.0, 4.0], [4.0, 4.0, 6.0]]], [[[1.0, 4.0, 2.0], [6.0, 2.0, 7.0], [7.0, 8.0, 4.0]], [[4.0, 
2.0, 6.0], [1.0, 5.0, 6.0], [5.0, 4.0, 8.0]], [[2.0, 5.0, 1.0], [7.0, 3.0, 1.0], [8.0, 4.0, 6.0]]], [[[5.0, 3.0, 3.0], [3.0, 
4.0, 7.0], [6.0, 7.0, 2.0]], [[1.0, 4.0, 2.0], [4.0, 7.0, 1.0], [8.0, 3.0, 3.0]], [[7.0, 2.0, 6.0], [8.0, 8.0, 3.0], [5.0, 2.0, 
2.0]]], [[[3.0, 4.0, 8.0], [2.0, 7.0, 5.0], [4.0, 1.0, 6.0]], [[8.0, 8.0, 2.0], [2.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[4.0, 8.0, 
8.0], [5.0, 2.0, 5.0], [1.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 5.0], [8.0, 5.0, 2.0], [1.0, 8.0, 6.0]], 
[[1.0, 2.0, 1.0], [2.0, 8.0, 1.0], [4.0, 4.0, 2.0]], [[5.0, 4.0, 7.0], [4.0, 7.0, 2.0], [8.0, 3.0, 8.0]]], [[[5.0, 3.0, 6.0], 
[7.0, 4.0, 4.0], [6.0, 2.0, 5.0]], [[5.0, 7.0, 4.0], [7.0, 7.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 5.0, 6.0], [4.0, 5.0, 2.0], 
[5.0, 2.0, 7.0]]], [[[8.0, 1.0, 2.0], [8.0, 7.0, 4.0], [4.0, 2.0, 6.0]], [[7.0, 8.0, 2.0], [3.0, 4.0, 1.0], [2.0, 7.0, 2.0]], 
[[5.0, 3.0, 4.0], [2.0, 5.0, 5.0], [8.0, 3.0, 1.0]]], [[[1.0, 3.0, 3.0], [3.0, 5.0, 2.0], [4.0, 8.0, 4.0]], [[7.0, 7.0, 2.0], 
[5.0, 5.0, 8.0], [3.0, 2.0, 1.0]], [[2.0, 4.0, 3.0], [2.0, 3.0, 3.0], [8.0, 3.0, 1.0]]]])
      val=numpy.array([[[7.0, 3.0, 4.0], [5.0, 3.0, 6.0]], [[8.0, 6.0, 4.0], [1.0, 1.0, 8.0]]])
      arg[1:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 3.0], [1.0, 7.0, 6.0], [7.0, 6.0, 6.0]], [[6.0, 1.0, 4.0], [4.0, 5.0, 6.0], [2.0, 
5.0, 5.0]], [[2.0, 5.0, 5.0], [7.0, 6.0, 4.0], [4.0, 4.0, 6.0]]], [[[1.0, 4.0, 2.0], [6.0, 2.0, 7.0], [7.0, 8.0, 4.0]], [[7.0, 
3.0, 4.0], [5.0, 3.0, 6.0], [5.0, 4.0, 8.0]], [[2.0, 5.0, 1.0], [7.0, 3.0, 1.0], [8.0, 4.0, 6.0]]], [[[5.0, 3.0, 3.0], [3.0, 
4.0, 7.0], [6.0, 7.0, 2.0]], [[8.0, 6.0, 4.0], [1.0, 1.0, 8.0], [8.0, 3.0, 3.0]], [[7.0, 2.0, 6.0], [8.0, 8.0, 3.0], [5.0, 2.0, 
2.0]]], [[[3.0, 4.0, 8.0], [2.0, 7.0, 5.0], [4.0, 1.0, 6.0]], [[8.0, 8.0, 2.0], [2.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[4.0, 8.0, 
8.0], [5.0, 2.0, 5.0], [1.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 5.0], [8.0, 5.0, 2.0], [1.0, 8.0, 6.0]], 
[[1.0, 2.0, 1.0], [2.0, 8.0, 1.0], [4.0, 4.0, 2.0]], [[5.0, 4.0, 7.0], [4.0, 7.0, 2.0], [8.0, 3.0, 8.0]]], [[[5.0, 3.0, 6.0], 
[7.0, 4.0, 4.0], [6.0, 2.0, 5.0]], [[7.0, 3.0, 4.0], [5.0, 3.0, 6.0], [7.0, 4.0, 1.0]], [[8.0, 5.0, 6.0], [4.0, 5.0, 2.0], 
[5.0, 2.0, 7.0]]], [[[8.0, 1.0, 2.0], [8.0, 7.0, 4.0], [4.0, 2.0, 6.0]], [[8.0, 6.0, 4.0], [1.0, 1.0, 8.0], [2.0, 7.0, 2.0]], 
[[5.0, 3.0, 4.0], [2.0, 5.0, 5.0], [8.0, 3.0, 1.0]]], [[[1.0, 3.0, 3.0], [3.0, 5.0, 2.0], [4.0, 8.0, 4.0]], [[7.0, 7.0, 2.0], 
[5.0, 5.0, 8.0], [3.0, 2.0, 1.0]], [[2.0, 4.0, 3.0], [2.0, 3.0, 3.0], [8.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 6.0], [7.0, 8.0, 6.0], [3.0, 1.0, 7.0]], [[1.0, 7.0, 7.0], [8.0, 7.0, 8.0], [6.0, 
4.0, 4.0]], [[6.0, 1.0, 4.0], [8.0, 3.0, 3.0], [6.0, 1.0, 3.0]]], [[[5.0, 2.0, 8.0], [5.0, 6.0, 2.0], [7.0, 2.0, 5.0]], [[6.0, 
4.0, 6.0], [1.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 8.0], [6.0, 1.0, 1.0], [2.0, 4.0, 3.0]]], [[[3.0, 1.0, 3.0], [7.0, 
3.0, 8.0], [4.0, 6.0, 4.0]], [[1.0, 4.0, 2.0], [7.0, 8.0, 2.0], [6.0, 8.0, 1.0]], [[8.0, 8.0, 8.0], [7.0, 5.0, 7.0], [7.0, 6.0, 
1.0]]], [[[1.0, 2.0, 2.0], [1.0, 8.0, 7.0], [3.0, 1.0, 6.0]], [[7.0, 2.0, 8.0], [6.0, 3.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 5.0, 
5.0], [1.0, 7.0, 8.0], [2.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 7.0, 3.0], [8.0, 3.0, 8.0]], 
[[3.0, 8.0, 7.0], [8.0, 4.0, 1.0], [5.0, 2.0, 7.0]], [[5.0, 1.0, 1.0], [6.0, 4.0, 4.0], [7.0, 6.0, 7.0]]], [[[1.0, 2.0, 6.0], 
[6.0, 7.0, 8.0], [5.0, 2.0, 6.0]], [[7.0, 1.0, 4.0], [1.0, 5.0, 7.0], [1.0, 5.0, 8.0]], [[7.0, 2.0, 7.0], [6.0, 7.0, 3.0], 
[4.0, 4.0, 5.0]]], [[[1.0, 7.0, 2.0], [1.0, 2.0, 5.0], [1.0, 2.0, 1.0]], [[7.0, 4.0, 6.0], [3.0, 4.0, 6.0], [2.0, 7.0, 8.0]], 
[[7.0, 8.0, 5.0], [5.0, 3.0, 8.0], [1.0, 6.0, 5.0]]], [[[5.0, 1.0, 5.0], [5.0, 7.0, 2.0], [6.0, 6.0, 2.0]], [[1.0, 5.0, 1.0], 
[4.0, 6.0, 4.0], [2.0, 4.0, 1.0]], [[1.0, 6.0, 6.0], [8.0, 5.0, 8.0], [6.0, 7.0, 5.0]]]])
      val=Data(numpy.array([[[6.0, 7.0, 8.0], [6.0, 1.0, 5.0]], [[2.0, 5.0, 7.0], [3.0, 2.0, 5.0]]]),self.functionspace)
      arg[1:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 6.0], [7.0, 8.0, 6.0], [3.0, 1.0, 7.0]], [[1.0, 7.0, 7.0], [8.0, 7.0, 8.0], [6.0, 
4.0, 4.0]], [[6.0, 1.0, 4.0], [8.0, 3.0, 3.0], [6.0, 1.0, 3.0]]], [[[5.0, 2.0, 8.0], [5.0, 6.0, 2.0], [7.0, 2.0, 5.0]], [[6.0, 
7.0, 8.0], [6.0, 1.0, 5.0], [3.0, 7.0, 2.0]], [[8.0, 2.0, 8.0], [6.0, 1.0, 1.0], [2.0, 4.0, 3.0]]], [[[3.0, 1.0, 3.0], [7.0, 
3.0, 8.0], [4.0, 6.0, 4.0]], [[2.0, 5.0, 7.0], [3.0, 2.0, 5.0], [6.0, 8.0, 1.0]], [[8.0, 8.0, 8.0], [7.0, 5.0, 7.0], [7.0, 6.0, 
1.0]]], [[[1.0, 2.0, 2.0], [1.0, 8.0, 7.0], [3.0, 1.0, 6.0]], [[7.0, 2.0, 8.0], [6.0, 3.0, 7.0], [2.0, 8.0, 8.0]], [[5.0, 5.0, 
5.0], [1.0, 7.0, 8.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 7.0, 3.0], [8.0, 3.0, 8.0]], 
[[3.0, 8.0, 7.0], [8.0, 4.0, 1.0], [5.0, 2.0, 7.0]], [[5.0, 1.0, 1.0], [6.0, 4.0, 4.0], [7.0, 6.0, 7.0]]], [[[1.0, 2.0, 6.0], 
[6.0, 7.0, 8.0], [5.0, 2.0, 6.0]], [[6.0, 7.0, 8.0], [6.0, 1.0, 5.0], [1.0, 5.0, 8.0]], [[7.0, 2.0, 7.0], [6.0, 7.0, 3.0], 
[4.0, 4.0, 5.0]]], [[[1.0, 7.0, 2.0], [1.0, 2.0, 5.0], [1.0, 2.0, 1.0]], [[2.0, 5.0, 7.0], [3.0, 2.0, 5.0], [2.0, 7.0, 8.0]], 
[[7.0, 8.0, 5.0], [5.0, 3.0, 8.0], [1.0, 6.0, 5.0]]], [[[5.0, 1.0, 5.0], [5.0, 7.0, 2.0], [6.0, 6.0, 2.0]], [[1.0, 5.0, 1.0], 
[4.0, 6.0, 4.0], [2.0, 4.0, 1.0]], [[1.0, 6.0, 6.0], [8.0, 5.0, 8.0], [6.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 5.0], [3.0, 7.0, 1.0], [2.0, 5.0, 8.0]], [[1.0, 3.0, 7.0], [3.0, 1.0, 8.0], [2.0, 
2.0, 7.0]], [[2.0, 3.0, 8.0], [3.0, 7.0, 1.0], [4.0, 6.0, 5.0]]], [[[1.0, 3.0, 6.0], [5.0, 5.0, 1.0], [6.0, 3.0, 8.0]], [[4.0, 
5.0, 7.0], [5.0, 3.0, 5.0], [6.0, 2.0, 5.0]], [[3.0, 3.0, 4.0], [6.0, 7.0, 1.0], [3.0, 5.0, 2.0]]], [[[1.0, 3.0, 4.0], [6.0, 
7.0, 3.0], [2.0, 8.0, 7.0]], [[5.0, 7.0, 4.0], [7.0, 2.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 1.0, 6.0], [3.0, 5.0, 
7.0]]], [[[1.0, 4.0, 4.0], [4.0, 7.0, 5.0], [4.0, 3.0, 5.0]], [[1.0, 4.0, 7.0], [1.0, 1.0, 4.0], [1.0, 8.0, 1.0]], [[6.0, 5.0, 
1.0], [5.0, 3.0, 4.0], [5.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 1.0], [7.0, 2.0, 7.0], [5.0, 5.0, 4.0]], 
[[6.0, 8.0, 6.0], [3.0, 6.0, 6.0], [7.0, 4.0, 2.0]], [[6.0, 2.0, 8.0], [4.0, 4.0, 1.0], [6.0, 1.0, 8.0]]], [[[2.0, 4.0, 4.0], 
[5.0, 8.0, 1.0], [3.0, 2.0, 7.0]], [[3.0, 5.0, 4.0], [5.0, 3.0, 7.0], [4.0, 7.0, 3.0]], [[6.0, 3.0, 1.0], [6.0, 1.0, 6.0], 
[3.0, 8.0, 3.0]]], [[[5.0, 5.0, 5.0], [2.0, 7.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 5.0, 6.0], [1.0, 2.0, 8.0], [3.0, 8.0, 1.0]], 
[[6.0, 3.0, 2.0], [3.0, 1.0, 5.0], [6.0, 1.0, 7.0]]], [[[7.0, 4.0, 7.0], [5.0, 4.0, 3.0], [7.0, 2.0, 5.0]], [[3.0, 3.0, 6.0], 
[8.0, 4.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 3.0, 2.0], [7.0, 7.0, 5.0], [6.0, 2.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 5.0, 1.0], [6.0, 3.0, 1.0]], [[2.0, 3.0, 7.0], [6.0, 5.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[5.0, 2.0, 6.0], [1.0, 8.0, 3.0]], [[3.0, 6.0, 3.0], [8.0, 8.0, 3.0]]])
      arg[1:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 5.0], [3.0, 7.0, 1.0], [2.0, 5.0, 8.0]], [[1.0, 3.0, 7.0], [3.0, 1.0, 8.0], [2.0, 
2.0, 7.0]], [[2.0, 3.0, 8.0], [3.0, 7.0, 1.0], [4.0, 6.0, 5.0]]], [[[1.0, 3.0, 6.0], [5.0, 5.0, 1.0], [6.0, 3.0, 8.0]], [[2.0, 
5.0, 1.0], [6.0, 3.0, 1.0], [6.0, 2.0, 5.0]], [[3.0, 3.0, 4.0], [6.0, 7.0, 1.0], [3.0, 5.0, 2.0]]], [[[1.0, 3.0, 4.0], [6.0, 
7.0, 3.0], [2.0, 8.0, 7.0]], [[2.0, 3.0, 7.0], [6.0, 5.0, 6.0], [8.0, 6.0, 6.0]], [[3.0, 4.0, 4.0], [3.0, 1.0, 6.0], [3.0, 5.0, 
7.0]]], [[[1.0, 4.0, 4.0], [4.0, 7.0, 5.0], [4.0, 3.0, 5.0]], [[1.0, 4.0, 7.0], [1.0, 1.0, 4.0], [1.0, 8.0, 1.0]], [[6.0, 5.0, 
1.0], [5.0, 3.0, 4.0], [5.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 1.0], [7.0, 2.0, 7.0], [5.0, 5.0, 4.0]], 
[[6.0, 8.0, 6.0], [3.0, 6.0, 6.0], [7.0, 4.0, 2.0]], [[6.0, 2.0, 8.0], [4.0, 4.0, 1.0], [6.0, 1.0, 8.0]]], [[[2.0, 4.0, 4.0], 
[5.0, 8.0, 1.0], [3.0, 2.0, 7.0]], [[5.0, 2.0, 6.0], [1.0, 8.0, 3.0], [4.0, 7.0, 3.0]], [[6.0, 3.0, 1.0], [6.0, 1.0, 6.0], 
[3.0, 8.0, 3.0]]], [[[5.0, 5.0, 5.0], [2.0, 7.0, 2.0], [3.0, 3.0, 6.0]], [[3.0, 6.0, 3.0], [8.0, 8.0, 3.0], [3.0, 8.0, 1.0]], 
[[6.0, 3.0, 2.0], [3.0, 1.0, 5.0], [6.0, 1.0, 7.0]]], [[[7.0, 4.0, 7.0], [5.0, 4.0, 3.0], [7.0, 2.0, 5.0]], [[3.0, 3.0, 6.0], 
[8.0, 4.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 3.0, 2.0], [7.0, 7.0, 5.0], [6.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 5.0], [6.0, 2.0, 8.0], [2.0, 3.0, 6.0]], [[8.0, 7.0, 5.0], [1.0, 3.0, 8.0], [5.0, 
3.0, 1.0]], [[7.0, 4.0, 7.0], [8.0, 4.0, 3.0], [7.0, 1.0, 3.0]]], [[[7.0, 8.0, 3.0], [6.0, 1.0, 6.0], [7.0, 7.0, 8.0]], [[5.0, 
7.0, 1.0], [3.0, 2.0, 3.0], [5.0, 3.0, 7.0]], [[8.0, 1.0, 5.0], [8.0, 7.0, 8.0], [3.0, 6.0, 6.0]]], [[[3.0, 6.0, 2.0], [6.0, 
5.0, 6.0], [8.0, 8.0, 2.0]], [[8.0, 4.0, 2.0], [4.0, 3.0, 4.0], [8.0, 2.0, 1.0]], [[3.0, 6.0, 2.0], [6.0, 8.0, 7.0], [8.0, 6.0, 
2.0]]], [[[5.0, 1.0, 3.0], [2.0, 8.0, 1.0], [8.0, 8.0, 8.0]], [[2.0, 6.0, 4.0], [6.0, 4.0, 7.0], [5.0, 5.0, 8.0]], [[8.0, 6.0, 
3.0], [3.0, 8.0, 8.0], [7.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 7.0], [8.0, 3.0, 1.0], [4.0, 3.0, 3.0]], 
[[1.0, 5.0, 5.0], [5.0, 1.0, 5.0], [7.0, 7.0, 6.0]], [[4.0, 1.0, 4.0], [5.0, 8.0, 4.0], [4.0, 5.0, 5.0]]], [[[2.0, 1.0, 7.0], 
[3.0, 6.0, 8.0], [1.0, 7.0, 2.0]], [[4.0, 6.0, 7.0], [8.0, 7.0, 1.0], [5.0, 2.0, 3.0]], [[1.0, 7.0, 7.0], [4.0, 7.0, 6.0], 
[3.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 7.0, 6.0], [3.0, 5.0, 8.0]], [[8.0, 3.0, 1.0], [8.0, 3.0, 2.0], [6.0, 4.0, 8.0]], 
[[6.0, 8.0, 2.0], [4.0, 3.0, 3.0], [4.0, 1.0, 7.0]]], [[[2.0, 5.0, 6.0], [3.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 5.0], 
[7.0, 3.0, 4.0], [2.0, 3.0, 1.0]], [[1.0, 8.0, 2.0], [4.0, 6.0, 5.0], [7.0, 7.0, 3.0]]]])
      val=numpy.array([[[4.0], [4.0], [1.0]], [[2.0], [6.0], [8.0]], [[4.0], [8.0], [1.0]], [[5.0], [4.0], [7.0]]])
      arg[:,1,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 5.0], [6.0, 2.0, 8.0], [2.0, 3.0, 6.0]], [[8.0, 7.0, 4.0], [1.0, 3.0, 4.0], [5.0, 
3.0, 1.0]], [[7.0, 4.0, 7.0], [8.0, 4.0, 3.0], [7.0, 1.0, 3.0]]], [[[7.0, 8.0, 3.0], [6.0, 1.0, 6.0], [7.0, 7.0, 8.0]], [[5.0, 
7.0, 2.0], [3.0, 2.0, 6.0], [5.0, 3.0, 8.0]], [[8.0, 1.0, 5.0], [8.0, 7.0, 8.0], [3.0, 6.0, 6.0]]], [[[3.0, 6.0, 2.0], [6.0, 
5.0, 6.0], [8.0, 8.0, 2.0]], [[8.0, 4.0, 4.0], [4.0, 3.0, 8.0], [8.0, 2.0, 1.0]], [[3.0, 6.0, 2.0], [6.0, 8.0, 7.0], [8.0, 6.0, 
2.0]]], [[[5.0, 1.0, 3.0], [2.0, 8.0, 1.0], [8.0, 8.0, 8.0]], [[2.0, 6.0, 5.0], [6.0, 4.0, 4.0], [5.0, 5.0, 7.0]], [[8.0, 6.0, 
3.0], [3.0, 8.0, 8.0], [7.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 7.0], [8.0, 3.0, 1.0], [4.0, 3.0, 3.0]], 
[[1.0, 5.0, 4.0], [5.0, 1.0, 4.0], [7.0, 7.0, 1.0]], [[4.0, 1.0, 4.0], [5.0, 8.0, 4.0], [4.0, 5.0, 5.0]]], [[[2.0, 1.0, 7.0], 
[3.0, 6.0, 8.0], [1.0, 7.0, 2.0]], [[4.0, 6.0, 2.0], [8.0, 7.0, 6.0], [5.0, 2.0, 8.0]], [[1.0, 7.0, 7.0], [4.0, 7.0, 6.0], 
[3.0, 2.0, 1.0]]], [[[2.0, 2.0, 5.0], [8.0, 7.0, 6.0], [3.0, 5.0, 8.0]], [[8.0, 3.0, 4.0], [8.0, 3.0, 8.0], [6.0, 4.0, 1.0]], 
[[6.0, 8.0, 2.0], [4.0, 3.0, 3.0], [4.0, 1.0, 7.0]]], [[[2.0, 5.0, 6.0], [3.0, 1.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 5.0], 
[7.0, 3.0, 4.0], [2.0, 3.0, 7.0]], [[1.0, 8.0, 2.0], [4.0, 6.0, 5.0], [7.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 1.0], [2.0, 3.0, 3.0], [3.0, 1.0, 1.0]], [[6.0, 2.0, 4.0], [7.0, 4.0, 6.0], [4.0, 
2.0, 6.0]], [[4.0, 7.0, 8.0], [3.0, 1.0, 8.0], [6.0, 3.0, 6.0]]], [[[7.0, 4.0, 5.0], [4.0, 3.0, 3.0], [7.0, 1.0, 6.0]], [[4.0, 
6.0, 4.0], [5.0, 3.0, 1.0], [4.0, 8.0, 2.0]], [[4.0, 2.0, 3.0], [1.0, 4.0, 3.0], [1.0, 1.0, 1.0]]], [[[3.0, 7.0, 2.0], [5.0, 
1.0, 4.0], [3.0, 8.0, 3.0]], [[1.0, 1.0, 4.0], [7.0, 7.0, 8.0], [6.0, 7.0, 5.0]], [[8.0, 4.0, 4.0], [2.0, 5.0, 6.0], [7.0, 4.0, 
6.0]]], [[[8.0, 1.0, 8.0], [8.0, 1.0, 4.0], [7.0, 2.0, 6.0]], [[6.0, 5.0, 6.0], [5.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[8.0, 3.0, 
8.0], [2.0, 5.0, 1.0], [7.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 1.0], [3.0, 3.0, 6.0], [2.0, 3.0, 5.0]], 
[[1.0, 1.0, 3.0], [4.0, 8.0, 1.0], [8.0, 7.0, 4.0]], [[3.0, 2.0, 5.0], [4.0, 2.0, 2.0], [1.0, 2.0, 6.0]]], [[[1.0, 2.0, 4.0], 
[7.0, 7.0, 2.0], [5.0, 3.0, 6.0]], [[6.0, 4.0, 3.0], [3.0, 7.0, 8.0], [4.0, 3.0, 1.0]], [[2.0, 8.0, 6.0], [8.0, 1.0, 8.0], 
[4.0, 1.0, 7.0]]], [[[3.0, 7.0, 1.0], [8.0, 5.0, 1.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 5.0], [2.0, 1.0, 7.0], [2.0, 1.0, 4.0]], 
[[1.0, 4.0, 5.0], [2.0, 4.0, 6.0], [3.0, 1.0, 4.0]]], [[[2.0, 8.0, 4.0], [5.0, 1.0, 5.0], [5.0, 7.0, 7.0]], [[7.0, 5.0, 5.0], 
[7.0, 8.0, 3.0], [1.0, 3.0, 3.0]], [[4.0, 7.0, 7.0], [7.0, 3.0, 3.0], [4.0, 6.0, 5.0]]]])
      val=Data(numpy.array([[[8.0], [2.0], [7.0]], [[6.0], [3.0], [2.0]], [[6.0], [7.0], [3.0]], [[6.0], [5.0], 
[2.0]]]),self.functionspace)
      arg[:,1,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [2.0, 3.0, 3.0], [3.0, 1.0, 1.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 2.0], [4.0, 
2.0, 7.0]], [[4.0, 7.0, 8.0], [3.0, 1.0, 8.0], [6.0, 3.0, 6.0]]], [[[7.0, 4.0, 5.0], [4.0, 3.0, 3.0], [7.0, 1.0, 6.0]], [[4.0, 
6.0, 6.0], [5.0, 3.0, 3.0], [4.0, 8.0, 2.0]], [[4.0, 2.0, 3.0], [1.0, 4.0, 3.0], [1.0, 1.0, 1.0]]], [[[3.0, 7.0, 2.0], [5.0, 
1.0, 4.0], [3.0, 8.0, 3.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 7.0], [6.0, 7.0, 3.0]], [[8.0, 4.0, 4.0], [2.0, 5.0, 6.0], [7.0, 4.0, 
6.0]]], [[[8.0, 1.0, 8.0], [8.0, 1.0, 4.0], [7.0, 2.0, 6.0]], [[6.0, 5.0, 6.0], [5.0, 4.0, 5.0], [2.0, 1.0, 2.0]], [[8.0, 3.0, 
8.0], [2.0, 5.0, 1.0], [7.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 1.0], [3.0, 3.0, 6.0], [2.0, 3.0, 5.0]], 
[[1.0, 1.0, 8.0], [4.0, 8.0, 2.0], [8.0, 7.0, 7.0]], [[3.0, 2.0, 5.0], [4.0, 2.0, 2.0], [1.0, 2.0, 6.0]]], [[[1.0, 2.0, 4.0], 
[7.0, 7.0, 2.0], [5.0, 3.0, 6.0]], [[6.0, 4.0, 6.0], [3.0, 7.0, 3.0], [4.0, 3.0, 2.0]], [[2.0, 8.0, 6.0], [8.0, 1.0, 8.0], 
[4.0, 1.0, 7.0]]], [[[3.0, 7.0, 1.0], [8.0, 5.0, 1.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 6.0], [2.0, 1.0, 7.0], [2.0, 1.0, 3.0]], 
[[1.0, 4.0, 5.0], [2.0, 4.0, 6.0], [3.0, 1.0, 4.0]]], [[[2.0, 8.0, 4.0], [5.0, 1.0, 5.0], [5.0, 7.0, 7.0]], [[7.0, 5.0, 6.0], 
[7.0, 8.0, 5.0], [1.0, 3.0, 2.0]], [[4.0, 7.0, 7.0], [7.0, 3.0, 3.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 7.0], [3.0, 5.0, 2.0], [7.0, 1.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 7.0, 6.0], [2.0, 
1.0, 6.0]], [[4.0, 4.0, 8.0], [4.0, 2.0, 1.0], [2.0, 3.0, 1.0]]], [[[5.0, 8.0, 1.0], [6.0, 3.0, 8.0], [1.0, 7.0, 1.0]], [[3.0, 
1.0, 1.0], [7.0, 3.0, 6.0], [6.0, 2.0, 6.0]], [[2.0, 8.0, 1.0], [4.0, 7.0, 5.0], [1.0, 1.0, 8.0]]], [[[2.0, 6.0, 2.0], [8.0, 
3.0, 3.0], [6.0, 4.0, 4.0]], [[1.0, 5.0, 3.0], [2.0, 3.0, 8.0], [4.0, 3.0, 3.0]], [[1.0, 7.0, 8.0], [4.0, 3.0, 5.0], [5.0, 7.0, 
8.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 7.0, 5.0], [8.0, 6.0, 1.0], [6.0, 8.0, 3.0]], [[4.0, 2.0, 
7.0], [2.0, 1.0, 5.0], [8.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 7.0], [2.0, 1.0, 7.0], [6.0, 6.0, 8.0]], 
[[6.0, 2.0, 3.0], [2.0, 1.0, 4.0], [7.0, 3.0, 1.0]], [[2.0, 2.0, 4.0], [3.0, 2.0, 2.0], [7.0, 8.0, 5.0]]], [[[2.0, 3.0, 7.0], 
[6.0, 5.0, 6.0], [5.0, 2.0, 6.0]], [[3.0, 5.0, 2.0], [2.0, 5.0, 1.0], [7.0, 3.0, 8.0]], [[5.0, 1.0, 7.0], [7.0, 3.0, 2.0], 
[1.0, 6.0, 1.0]]], [[[4.0, 4.0, 4.0], [2.0, 4.0, 4.0], [4.0, 3.0, 3.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 4.0], [4.0, 1.0, 3.0]], 
[[4.0, 2.0, 4.0], [4.0, 6.0, 7.0], [8.0, 1.0, 2.0]]], [[[3.0, 8.0, 4.0], [7.0, 1.0, 4.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 3.0], 
[2.0, 7.0, 2.0], [6.0, 3.0, 7.0]], [[4.0, 5.0, 3.0], [5.0, 5.0, 8.0], [6.0, 4.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0], [1.0], [3.0]], [[6.0], [2.0], [8.0]], [[8.0], [5.0], [3.0]], [[7.0], [1.0], 
[2.0]]])+(1.-msk_val)*numpy.array([[[7.0], [1.0], [1.0]], [[4.0], [5.0], [3.0]], [[7.0], [5.0], [4.0]], [[8.0], [1.0], [7.0]]])
      arg[:,1,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 7.0], [3.0, 5.0, 2.0], [7.0, 1.0, 5.0]], [[2.0, 4.0, 2.0], [4.0, 7.0, 1.0], [2.0, 
1.0, 3.0]], [[4.0, 4.0, 8.0], [4.0, 2.0, 1.0], [2.0, 3.0, 1.0]]], [[[5.0, 8.0, 1.0], [6.0, 3.0, 8.0], [1.0, 7.0, 1.0]], [[3.0, 
1.0, 6.0], [7.0, 3.0, 2.0], [6.0, 2.0, 8.0]], [[2.0, 8.0, 1.0], [4.0, 7.0, 5.0], [1.0, 1.0, 8.0]]], [[[2.0, 6.0, 2.0], [8.0, 
3.0, 3.0], [6.0, 4.0, 4.0]], [[1.0, 5.0, 8.0], [2.0, 3.0, 5.0], [4.0, 3.0, 3.0]], [[1.0, 7.0, 8.0], [4.0, 3.0, 5.0], [5.0, 7.0, 
8.0]]], [[[4.0, 7.0, 7.0], [5.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[6.0, 7.0, 7.0], [8.0, 6.0, 1.0], [6.0, 8.0, 2.0]], [[4.0, 2.0, 
7.0], [2.0, 1.0, 5.0], [8.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 7.0], [2.0, 1.0, 7.0], [6.0, 6.0, 8.0]], 
[[6.0, 2.0, 7.0], [2.0, 1.0, 1.0], [7.0, 3.0, 1.0]], [[2.0, 2.0, 4.0], [3.0, 2.0, 2.0], [7.0, 8.0, 5.0]]], [[[2.0, 3.0, 7.0], 
[6.0, 5.0, 6.0], [5.0, 2.0, 6.0]], [[3.0, 5.0, 4.0], [2.0, 5.0, 5.0], [7.0, 3.0, 3.0]], [[5.0, 1.0, 7.0], [7.0, 3.0, 2.0], 
[1.0, 6.0, 1.0]]], [[[4.0, 4.0, 4.0], [2.0, 4.0, 4.0], [4.0, 3.0, 3.0]], [[1.0, 2.0, 7.0], [8.0, 4.0, 5.0], [4.0, 1.0, 4.0]], 
[[4.0, 2.0, 4.0], [4.0, 6.0, 7.0], [8.0, 1.0, 2.0]]], [[[3.0, 8.0, 4.0], [7.0, 1.0, 4.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 8.0], 
[2.0, 7.0, 1.0], [6.0, 3.0, 7.0]], [[4.0, 5.0, 3.0], [5.0, 5.0, 8.0], [6.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 5.0, 5.0], [2.0, 8.0, 8.0]], [[5.0, 2.0, 7.0], [8.0, 7.0, 2.0], [1.0, 
2.0, 7.0]], [[1.0, 1.0, 1.0], [8.0, 4.0, 8.0], [6.0, 6.0, 8.0]]], [[[4.0, 6.0, 7.0], [7.0, 5.0, 6.0], [1.0, 1.0, 6.0]], [[7.0, 
1.0, 1.0], [5.0, 2.0, 8.0], [8.0, 5.0, 2.0]], [[8.0, 3.0, 4.0], [7.0, 2.0, 6.0], [6.0, 3.0, 5.0]]], [[[8.0, 3.0, 8.0], [5.0, 
6.0, 3.0], [6.0, 8.0, 1.0]], [[2.0, 4.0, 2.0], [7.0, 4.0, 8.0], [4.0, 6.0, 8.0]], [[2.0, 5.0, 1.0], [1.0, 5.0, 7.0], [7.0, 6.0, 
2.0]]], [[[5.0, 8.0, 7.0], [4.0, 4.0, 1.0], [6.0, 6.0, 5.0]], [[4.0, 7.0, 8.0], [1.0, 7.0, 5.0], [7.0, 7.0, 1.0]], [[3.0, 2.0, 
8.0], [2.0, 1.0, 8.0], [6.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 4.0], [2.0, 8.0, 7.0], [3.0, 7.0, 3.0]], 
[[7.0, 6.0, 8.0], [2.0, 3.0, 4.0], [3.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [8.0, 7.0, 3.0], [6.0, 7.0, 1.0]]], [[[8.0, 1.0, 5.0], 
[2.0, 5.0, 3.0], [3.0, 4.0, 8.0]], [[4.0, 8.0, 5.0], [4.0, 4.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 8.0, 1.0], [6.0, 2.0, 6.0], 
[3.0, 3.0, 8.0]]], [[[6.0, 7.0, 5.0], [8.0, 3.0, 2.0], [6.0, 8.0, 5.0]], [[8.0, 1.0, 6.0], [8.0, 7.0, 2.0], [4.0, 3.0, 8.0]], 
[[3.0, 7.0, 5.0], [1.0, 1.0, 5.0], [3.0, 8.0, 2.0]]], [[[3.0, 4.0, 2.0], [6.0, 7.0, 8.0], [5.0, 7.0, 6.0]], [[7.0, 5.0, 3.0], 
[4.0, 3.0, 4.0], [4.0, 3.0, 8.0]], [[4.0, 6.0, 8.0], [8.0, 2.0, 2.0], [3.0, 7.0, 2.0]]]])
      val=numpy.array([[[2.0]], [[2.0]]])
      arg[2:,2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 5.0, 5.0], [2.0, 8.0, 8.0]], [[5.0, 2.0, 7.0], [8.0, 7.0, 2.0], [1.0, 
2.0, 7.0]], [[1.0, 1.0, 1.0], [8.0, 4.0, 8.0], [6.0, 6.0, 8.0]]], [[[4.0, 6.0, 7.0], [7.0, 5.0, 6.0], [1.0, 1.0, 6.0]], [[7.0, 
1.0, 1.0], [5.0, 2.0, 8.0], [8.0, 5.0, 2.0]], [[8.0, 3.0, 4.0], [7.0, 2.0, 6.0], [6.0, 3.0, 5.0]]], [[[8.0, 3.0, 8.0], [5.0, 
6.0, 3.0], [6.0, 8.0, 1.0]], [[2.0, 4.0, 2.0], [7.0, 4.0, 8.0], [4.0, 6.0, 8.0]], [[2.0, 5.0, 1.0], [1.0, 2.0, 7.0], [7.0, 6.0, 
2.0]]], [[[5.0, 8.0, 7.0], [4.0, 4.0, 1.0], [6.0, 6.0, 5.0]], [[4.0, 7.0, 8.0], [1.0, 7.0, 5.0], [7.0, 7.0, 1.0]], [[3.0, 2.0, 
8.0], [2.0, 2.0, 8.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 4.0], [2.0, 8.0, 7.0], [3.0, 7.0, 3.0]], 
[[7.0, 6.0, 8.0], [2.0, 3.0, 4.0], [3.0, 4.0, 6.0]], [[1.0, 6.0, 8.0], [8.0, 7.0, 3.0], [6.0, 7.0, 1.0]]], [[[8.0, 1.0, 5.0], 
[2.0, 5.0, 3.0], [3.0, 4.0, 8.0]], [[4.0, 8.0, 5.0], [4.0, 4.0, 2.0], [8.0, 2.0, 2.0]], [[7.0, 8.0, 1.0], [6.0, 2.0, 6.0], 
[3.0, 3.0, 8.0]]], [[[6.0, 7.0, 5.0], [8.0, 3.0, 2.0], [6.0, 8.0, 5.0]], [[8.0, 1.0, 6.0], [8.0, 7.0, 2.0], [4.0, 3.0, 8.0]], 
[[3.0, 7.0, 5.0], [1.0, 2.0, 5.0], [3.0, 8.0, 2.0]]], [[[3.0, 4.0, 2.0], [6.0, 7.0, 8.0], [5.0, 7.0, 6.0]], [[7.0, 5.0, 3.0], 
[4.0, 3.0, 4.0], [4.0, 3.0, 8.0]], [[4.0, 6.0, 8.0], [8.0, 2.0, 2.0], [3.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 2.0], [5.0, 8.0, 7.0], [8.0, 7.0, 8.0]], [[2.0, 2.0, 3.0], [8.0, 8.0, 3.0], [5.0, 
7.0, 2.0]], [[7.0, 6.0, 2.0], [3.0, 3.0, 2.0], [7.0, 7.0, 4.0]]], [[[4.0, 5.0, 6.0], [1.0, 1.0, 5.0], [4.0, 6.0, 1.0]], [[7.0, 
2.0, 4.0], [7.0, 8.0, 2.0], [6.0, 2.0, 1.0]], [[2.0, 4.0, 3.0], [2.0, 8.0, 1.0], [3.0, 1.0, 5.0]]], [[[8.0, 1.0, 8.0], [4.0, 
2.0, 4.0], [1.0, 8.0, 7.0]], [[3.0, 2.0, 8.0], [8.0, 4.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 4.0, 3.0], [5.0, 3.0, 4.0], [8.0, 4.0, 
3.0]]], [[[8.0, 1.0, 4.0], [1.0, 1.0, 8.0], [5.0, 2.0, 4.0]], [[4.0, 3.0, 3.0], [4.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[2.0, 8.0, 
5.0], [7.0, 2.0, 7.0], [6.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 2.0], [7.0, 2.0, 6.0], [7.0, 1.0, 5.0]], 
[[5.0, 5.0, 4.0], [2.0, 6.0, 6.0], [5.0, 7.0, 7.0]], [[2.0, 8.0, 5.0], [8.0, 6.0, 6.0], [7.0, 8.0, 5.0]]], [[[2.0, 2.0, 3.0], 
[7.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 1.0], [1.0, 1.0, 7.0]], [[2.0, 4.0, 4.0], [8.0, 8.0, 3.0], 
[2.0, 3.0, 2.0]]], [[[4.0, 8.0, 4.0], [2.0, 4.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 8.0], [8.0, 5.0, 7.0], [7.0, 7.0, 1.0]], 
[[2.0, 4.0, 3.0], [8.0, 5.0, 4.0], [3.0, 8.0, 3.0]]], [[[7.0, 5.0, 5.0], [8.0, 3.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 8.0], 
[8.0, 2.0, 5.0], [2.0, 1.0, 3.0]], [[5.0, 8.0, 7.0], [2.0, 6.0, 6.0], [7.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[1.0]], [[7.0]]]),self.functionspace)
      arg[2:,2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 2.0], [5.0, 8.0, 7.0], [8.0, 7.0, 8.0]], [[2.0, 2.0, 3.0], [8.0, 8.0, 3.0], [5.0, 
7.0, 2.0]], [[7.0, 6.0, 2.0], [3.0, 3.0, 2.0], [7.0, 7.0, 4.0]]], [[[4.0, 5.0, 6.0], [1.0, 1.0, 5.0], [4.0, 6.0, 1.0]], [[7.0, 
2.0, 4.0], [7.0, 8.0, 2.0], [6.0, 2.0, 1.0]], [[2.0, 4.0, 3.0], [2.0, 8.0, 1.0], [3.0, 1.0, 5.0]]], [[[8.0, 1.0, 8.0], [4.0, 
2.0, 4.0], [1.0, 8.0, 7.0]], [[3.0, 2.0, 8.0], [8.0, 4.0, 2.0], [8.0, 8.0, 5.0]], [[6.0, 4.0, 3.0], [5.0, 1.0, 4.0], [8.0, 4.0, 
3.0]]], [[[8.0, 1.0, 4.0], [1.0, 1.0, 8.0], [5.0, 2.0, 4.0]], [[4.0, 3.0, 3.0], [4.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[2.0, 8.0, 
5.0], [7.0, 7.0, 7.0], [6.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 2.0], [7.0, 2.0, 6.0], [7.0, 1.0, 5.0]], 
[[5.0, 5.0, 4.0], [2.0, 6.0, 6.0], [5.0, 7.0, 7.0]], [[2.0, 8.0, 5.0], [8.0, 6.0, 6.0], [7.0, 8.0, 5.0]]], [[[2.0, 2.0, 3.0], 
[7.0, 2.0, 5.0], [8.0, 8.0, 1.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 1.0], [1.0, 1.0, 7.0]], [[2.0, 4.0, 4.0], [8.0, 8.0, 3.0], 
[2.0, 3.0, 2.0]]], [[[4.0, 8.0, 4.0], [2.0, 4.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 8.0], [8.0, 5.0, 7.0], [7.0, 7.0, 1.0]], 
[[2.0, 4.0, 3.0], [8.0, 1.0, 4.0], [3.0, 8.0, 3.0]]], [[[7.0, 5.0, 5.0], [8.0, 3.0, 7.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 8.0], 
[8.0, 2.0, 5.0], [2.0, 1.0, 3.0]], [[5.0, 8.0, 7.0], [2.0, 7.0, 6.0], [7.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLPR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 7.0], [7.0, 4.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 6.0, 4.0], [5.0, 6.0, 4.0], [4.0, 
6.0, 8.0]], [[1.0, 5.0, 7.0], [8.0, 4.0, 1.0], [6.0, 5.0, 5.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 8.0], [7.0, 2.0, 1.0]], [[6.0, 
8.0, 4.0], [8.0, 8.0, 4.0], [2.0, 8.0, 3.0]], [[2.0, 6.0, 7.0], [3.0, 5.0, 3.0], [4.0, 1.0, 8.0]]], [[[7.0, 7.0, 3.0], [8.0, 
4.0, 4.0], [1.0, 5.0, 7.0]], [[8.0, 4.0, 1.0], [4.0, 4.0, 8.0], [5.0, 1.0, 1.0]], [[8.0, 2.0, 7.0], [3.0, 4.0, 7.0], [4.0, 8.0, 
1.0]]], [[[2.0, 8.0, 2.0], [6.0, 5.0, 5.0], [7.0, 3.0, 8.0]], [[2.0, 4.0, 7.0], [4.0, 4.0, 6.0], [7.0, 3.0, 8.0]], [[6.0, 8.0, 
3.0], [5.0, 2.0, 1.0], [1.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 4.0], [5.0, 7.0, 8.0], [6.0, 3.0, 3.0]], 
[[3.0, 2.0, 2.0], [7.0, 2.0, 2.0], [1.0, 2.0, 3.0]], [[2.0, 2.0, 5.0], [8.0, 7.0, 5.0], [4.0, 4.0, 2.0]]], [[[7.0, 3.0, 4.0], 
[1.0, 4.0, 8.0], [5.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [8.0, 2.0, 8.0], [6.0, 4.0, 2.0]], [[3.0, 5.0, 5.0], [5.0, 8.0, 6.0], 
[1.0, 3.0, 1.0]]], [[[4.0, 2.0, 3.0], [1.0, 5.0, 1.0], [5.0, 1.0, 7.0]], [[1.0, 2.0, 5.0], [4.0, 2.0, 3.0], [1.0, 8.0, 2.0]], 
[[3.0, 5.0, 4.0], [1.0, 1.0, 7.0], [4.0, 6.0, 7.0]]], [[[6.0, 2.0, 5.0], [8.0, 3.0, 2.0], [7.0, 6.0, 4.0]], [[6.0, 8.0, 1.0], 
[5.0, 7.0, 4.0], [4.0, 6.0, 1.0]], [[4.0, 2.0, 2.0], [4.0, 1.0, 6.0], [5.0, 4.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0]], [[7.0]]])+(1.-msk_val)*numpy.array([[[8.0]], [[2.0]]])
      arg[2:,2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [7.0, 4.0, 4.0], [5.0, 6.0, 8.0]], [[8.0, 6.0, 4.0], [5.0, 6.0, 4.0], [4.0, 
6.0, 8.0]], [[1.0, 5.0, 7.0], [8.0, 4.0, 1.0], [6.0, 5.0, 5.0]]], [[[8.0, 5.0, 4.0], [1.0, 4.0, 8.0], [7.0, 2.0, 1.0]], [[6.0, 
8.0, 4.0], [8.0, 8.0, 4.0], [2.0, 8.0, 3.0]], [[2.0, 6.0, 7.0], [3.0, 5.0, 3.0], [4.0, 1.0, 8.0]]], [[[7.0, 7.0, 3.0], [8.0, 
4.0, 4.0], [1.0, 5.0, 7.0]], [[8.0, 4.0, 1.0], [4.0, 4.0, 8.0], [5.0, 1.0, 1.0]], [[8.0, 2.0, 7.0], [3.0, 4.0, 7.0], [4.0, 8.0, 
1.0]]], [[[2.0, 8.0, 2.0], [6.0, 5.0, 5.0], [7.0, 3.0, 8.0]], [[2.0, 4.0, 7.0], [4.0, 4.0, 6.0], [7.0, 3.0, 8.0]], [[6.0, 8.0, 
3.0], [5.0, 7.0, 1.0], [1.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 4.0], [5.0, 7.0, 8.0], [6.0, 3.0, 3.0]], 
[[3.0, 2.0, 2.0], [7.0, 2.0, 2.0], [1.0, 2.0, 3.0]], [[2.0, 2.0, 5.0], [8.0, 7.0, 5.0], [4.0, 4.0, 2.0]]], [[[7.0, 3.0, 4.0], 
[1.0, 4.0, 8.0], [5.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [8.0, 2.0, 8.0], [6.0, 4.0, 2.0]], [[3.0, 5.0, 5.0], [5.0, 8.0, 6.0], 
[1.0, 3.0, 1.0]]], [[[4.0, 2.0, 3.0], [1.0, 5.0, 1.0], [5.0, 1.0, 7.0]], [[1.0, 2.0, 5.0], [4.0, 2.0, 3.0], [1.0, 8.0, 2.0]], 
[[3.0, 5.0, 4.0], [1.0, 8.0, 7.0], [4.0, 6.0, 7.0]]], [[[6.0, 2.0, 5.0], [8.0, 3.0, 2.0], [7.0, 6.0, 4.0]], [[6.0, 8.0, 1.0], 
[5.0, 7.0, 4.0], [4.0, 6.0, 1.0]], [[4.0, 2.0, 2.0], [4.0, 2.0, 6.0], [5.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 4.0], [5.0, 5.0, 4.0], [7.0, 6.0, 2.0]], [[6.0, 4.0, 1.0], [2.0, 8.0, 4.0], [1.0, 
6.0, 6.0]], [[3.0, 3.0, 5.0], [3.0, 5.0, 1.0], [8.0, 8.0, 8.0]]], [[[6.0, 7.0, 5.0], [8.0, 2.0, 6.0], [4.0, 7.0, 3.0]], [[8.0, 
3.0, 8.0], [8.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[5.0, 7.0, 5.0], [4.0, 5.0, 1.0], [2.0, 1.0, 6.0]]], [[[6.0, 8.0, 1.0], [5.0, 
1.0, 2.0], [2.0, 7.0, 7.0]], [[7.0, 2.0, 5.0], [1.0, 8.0, 8.0], [8.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [6.0, 7.0, 6.0], [7.0, 1.0, 
2.0]]], [[[5.0, 4.0, 4.0], [4.0, 8.0, 4.0], [1.0, 8.0, 7.0]], [[8.0, 3.0, 7.0], [3.0, 7.0, 6.0], [7.0, 6.0, 8.0]], [[4.0, 6.0, 
4.0], [2.0, 5.0, 8.0], [2.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 7.0], [6.0, 2.0, 5.0], [6.0, 8.0, 2.0]], 
[[6.0, 6.0, 3.0], [4.0, 7.0, 6.0], [8.0, 5.0, 6.0]], [[8.0, 6.0, 7.0], [2.0, 3.0, 7.0], [8.0, 6.0, 1.0]]], [[[2.0, 6.0, 6.0], 
[7.0, 6.0, 2.0], [2.0, 3.0, 3.0]], [[6.0, 2.0, 3.0], [7.0, 3.0, 5.0], [8.0, 5.0, 4.0]], [[6.0, 2.0, 8.0], [7.0, 8.0, 8.0], 
[7.0, 2.0, 8.0]]], [[[7.0, 1.0, 3.0], [7.0, 4.0, 7.0], [8.0, 2.0, 1.0]], [[6.0, 8.0, 3.0], [4.0, 4.0, 3.0], [8.0, 8.0, 4.0]], 
[[2.0, 3.0, 2.0], [8.0, 8.0, 7.0], [3.0, 8.0, 5.0]]], [[[3.0, 5.0, 8.0], [5.0, 4.0, 6.0], [2.0, 8.0, 8.0]], [[3.0, 3.0, 2.0], 
[1.0, 2.0, 8.0], [3.0, 7.0, 8.0]], [[8.0, 3.0, 3.0], [5.0, 7.0, 1.0], [2.0, 7.0, 5.0]]]])
      val=numpy.array([[[2.0]]])
      arg[2,1:2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 4.0], [5.0, 5.0, 4.0], [7.0, 6.0, 2.0]], [[6.0, 4.0, 1.0], [2.0, 8.0, 4.0], [1.0, 
6.0, 6.0]], [[3.0, 3.0, 5.0], [3.0, 5.0, 1.0], [8.0, 8.0, 8.0]]], [[[6.0, 7.0, 5.0], [8.0, 2.0, 6.0], [4.0, 7.0, 3.0]], [[8.0, 
3.0, 8.0], [8.0, 5.0, 6.0], [6.0, 5.0, 6.0]], [[5.0, 7.0, 5.0], [4.0, 5.0, 1.0], [2.0, 1.0, 6.0]]], [[[6.0, 8.0, 1.0], [5.0, 
1.0, 2.0], [2.0, 7.0, 7.0]], [[7.0, 2.0, 5.0], [1.0, 8.0, 2.0], [8.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [6.0, 7.0, 6.0], [7.0, 1.0, 
2.0]]], [[[5.0, 4.0, 4.0], [4.0, 8.0, 4.0], [1.0, 8.0, 7.0]], [[8.0, 3.0, 7.0], [3.0, 7.0, 6.0], [7.0, 6.0, 8.0]], [[4.0, 6.0, 
4.0], [2.0, 5.0, 8.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 7.0], [6.0, 2.0, 5.0], [6.0, 8.0, 2.0]], 
[[6.0, 6.0, 3.0], [4.0, 7.0, 6.0], [8.0, 5.0, 6.0]], [[8.0, 6.0, 7.0], [2.0, 3.0, 7.0], [8.0, 6.0, 1.0]]], [[[2.0, 6.0, 6.0], 
[7.0, 6.0, 2.0], [2.0, 3.0, 3.0]], [[6.0, 2.0, 3.0], [7.0, 3.0, 5.0], [8.0, 5.0, 4.0]], [[6.0, 2.0, 8.0], [7.0, 8.0, 8.0], 
[7.0, 2.0, 8.0]]], [[[7.0, 1.0, 3.0], [7.0, 4.0, 7.0], [8.0, 2.0, 1.0]], [[6.0, 8.0, 3.0], [4.0, 4.0, 2.0], [8.0, 8.0, 4.0]], 
[[2.0, 3.0, 2.0], [8.0, 8.0, 7.0], [3.0, 8.0, 5.0]]], [[[3.0, 5.0, 8.0], [5.0, 4.0, 6.0], [2.0, 8.0, 8.0]], [[3.0, 3.0, 2.0], 
[1.0, 2.0, 8.0], [3.0, 7.0, 8.0]], [[8.0, 3.0, 3.0], [5.0, 7.0, 1.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 8.0], [6.0, 2.0, 3.0], [6.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [3.0, 4.0, 7.0], [5.0, 
8.0, 6.0]], [[8.0, 7.0, 8.0], [1.0, 8.0, 2.0], [8.0, 1.0, 8.0]]], [[[5.0, 1.0, 1.0], [6.0, 2.0, 2.0], [7.0, 4.0, 6.0]], [[4.0, 
4.0, 4.0], [4.0, 4.0, 8.0], [7.0, 7.0, 4.0]], [[6.0, 3.0, 2.0], [4.0, 2.0, 4.0], [8.0, 3.0, 8.0]]], [[[5.0, 7.0, 4.0], [8.0, 
3.0, 2.0], [8.0, 2.0, 7.0]], [[4.0, 7.0, 2.0], [2.0, 2.0, 8.0], [2.0, 8.0, 6.0]], [[8.0, 2.0, 8.0], [6.0, 2.0, 2.0], [5.0, 5.0, 
2.0]]], [[[8.0, 6.0, 6.0], [3.0, 4.0, 3.0], [3.0, 3.0, 8.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 4.0], [3.0, 5.0, 3.0]], [[8.0, 1.0, 
8.0], [3.0, 2.0, 8.0], [6.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 7.0, 3.0], [1.0, 2.0, 8.0], [4.0, 5.0, 1.0]], 
[[1.0, 1.0, 8.0], [1.0, 8.0, 8.0], [7.0, 6.0, 5.0]], [[6.0, 3.0, 6.0], [8.0, 1.0, 8.0], [7.0, 5.0, 3.0]]], [[[3.0, 3.0, 4.0], 
[8.0, 7.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 8.0, 2.0], [7.0, 2.0, 7.0], [1.0, 1.0, 5.0]], [[2.0, 4.0, 6.0], [6.0, 1.0, 6.0], 
[3.0, 7.0, 7.0]]], [[[1.0, 7.0, 8.0], [1.0, 6.0, 8.0], [4.0, 3.0, 8.0]], [[3.0, 6.0, 1.0], [2.0, 3.0, 6.0], [8.0, 7.0, 4.0]], 
[[5.0, 6.0, 2.0], [5.0, 2.0, 2.0], [4.0, 6.0, 2.0]]], [[[8.0, 3.0, 5.0], [2.0, 2.0, 4.0], [7.0, 5.0, 2.0]], [[7.0, 2.0, 7.0], 
[3.0, 5.0, 1.0], [6.0, 5.0, 3.0]], [[2.0, 2.0, 8.0], [4.0, 6.0, 3.0], [1.0, 8.0, 5.0]]]])
      val=Data(numpy.array([[[7.0]]]),self.functionspace)
      arg[2,1:2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 8.0], [6.0, 2.0, 3.0], [6.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [3.0, 4.0, 7.0], [5.0, 
8.0, 6.0]], [[8.0, 7.0, 8.0], [1.0, 8.0, 2.0], [8.0, 1.0, 8.0]]], [[[5.0, 1.0, 1.0], [6.0, 2.0, 2.0], [7.0, 4.0, 6.0]], [[4.0, 
4.0, 4.0], [4.0, 4.0, 8.0], [7.0, 7.0, 4.0]], [[6.0, 3.0, 2.0], [4.0, 2.0, 4.0], [8.0, 3.0, 8.0]]], [[[5.0, 7.0, 4.0], [8.0, 
3.0, 2.0], [8.0, 2.0, 7.0]], [[4.0, 7.0, 2.0], [2.0, 2.0, 7.0], [2.0, 8.0, 6.0]], [[8.0, 2.0, 8.0], [6.0, 2.0, 2.0], [5.0, 5.0, 
2.0]]], [[[8.0, 6.0, 6.0], [3.0, 4.0, 3.0], [3.0, 3.0, 8.0]], [[4.0, 6.0, 2.0], [4.0, 1.0, 4.0], [3.0, 5.0, 3.0]], [[8.0, 1.0, 
8.0], [3.0, 2.0, 8.0], [6.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 3.0], [1.0, 2.0, 8.0], [4.0, 5.0, 1.0]], 
[[1.0, 1.0, 8.0], [1.0, 8.0, 8.0], [7.0, 6.0, 5.0]], [[6.0, 3.0, 6.0], [8.0, 1.0, 8.0], [7.0, 5.0, 3.0]]], [[[3.0, 3.0, 4.0], 
[8.0, 7.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 8.0, 2.0], [7.0, 2.0, 7.0], [1.0, 1.0, 5.0]], [[2.0, 4.0, 6.0], [6.0, 1.0, 6.0], 
[3.0, 7.0, 7.0]]], [[[1.0, 7.0, 8.0], [1.0, 6.0, 8.0], [4.0, 3.0, 8.0]], [[3.0, 6.0, 1.0], [2.0, 3.0, 7.0], [8.0, 7.0, 4.0]], 
[[5.0, 6.0, 2.0], [5.0, 2.0, 2.0], [4.0, 6.0, 2.0]]], [[[8.0, 3.0, 5.0], [2.0, 2.0, 4.0], [7.0, 5.0, 2.0]], [[7.0, 2.0, 7.0], 
[3.0, 5.0, 1.0], [6.0, 5.0, 3.0]], [[2.0, 2.0, 8.0], [4.0, 6.0, 3.0], [1.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 7.0, 8.0], [4.0, 8.0, 5.0]], [[5.0, 6.0, 7.0], [4.0, 8.0, 7.0], [1.0, 
3.0, 8.0]], [[1.0, 5.0, 6.0], [3.0, 6.0, 1.0], [8.0, 5.0, 7.0]]], [[[3.0, 3.0, 1.0], [6.0, 1.0, 8.0], [3.0, 4.0, 7.0]], [[6.0, 
8.0, 2.0], [3.0, 1.0, 2.0], [6.0, 8.0, 7.0]], [[5.0, 4.0, 4.0], [5.0, 8.0, 3.0], [3.0, 8.0, 1.0]]], [[[5.0, 2.0, 7.0], [1.0, 
2.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 6.0, 8.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 6.0], [2.0, 7.0, 8.0], [8.0, 1.0, 
2.0]]], [[[1.0, 3.0, 1.0], [1.0, 4.0, 1.0], [1.0, 6.0, 1.0]], [[8.0, 5.0, 7.0], [3.0, 8.0, 8.0], [3.0, 4.0, 4.0]], [[6.0, 7.0, 
4.0], [4.0, 7.0, 1.0], [2.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 5.0], [4.0, 2.0, 2.0], [2.0, 4.0, 7.0]], 
[[6.0, 8.0, 8.0], [1.0, 6.0, 1.0], [6.0, 1.0, 6.0]], [[8.0, 1.0, 7.0], [7.0, 6.0, 5.0], [7.0, 1.0, 3.0]]], [[[6.0, 5.0, 1.0], 
[7.0, 7.0, 2.0], [4.0, 2.0, 4.0]], [[5.0, 7.0, 4.0], [7.0, 3.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 3.0, 4.0], [3.0, 3.0, 7.0], 
[4.0, 3.0, 1.0]]], [[[8.0, 1.0, 5.0], [3.0, 6.0, 6.0], [8.0, 6.0, 5.0]], [[8.0, 6.0, 1.0], [6.0, 1.0, 8.0], [1.0, 8.0, 4.0]], 
[[1.0, 6.0, 5.0], [5.0, 7.0, 4.0], [2.0, 8.0, 8.0]]], [[[8.0, 6.0, 5.0], [8.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[3.0, 5.0, 6.0], 
[6.0, 1.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 5.0, 2.0], [8.0, 7.0, 1.0], [4.0, 5.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0]]])+(1.-msk_val)*numpy.array([[[2.0]]])
      arg[2,1:2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 7.0, 8.0], [4.0, 8.0, 5.0]], [[5.0, 6.0, 7.0], [4.0, 8.0, 7.0], [1.0, 
3.0, 8.0]], [[1.0, 5.0, 6.0], [3.0, 6.0, 1.0], [8.0, 5.0, 7.0]]], [[[3.0, 3.0, 1.0], [6.0, 1.0, 8.0], [3.0, 4.0, 7.0]], [[6.0, 
8.0, 2.0], [3.0, 1.0, 2.0], [6.0, 8.0, 7.0]], [[5.0, 4.0, 4.0], [5.0, 8.0, 3.0], [3.0, 8.0, 1.0]]], [[[5.0, 2.0, 7.0], [1.0, 
2.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 6.0, 4.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 6.0], [2.0, 7.0, 8.0], [8.0, 1.0, 
2.0]]], [[[1.0, 3.0, 1.0], [1.0, 4.0, 1.0], [1.0, 6.0, 1.0]], [[8.0, 5.0, 7.0], [3.0, 8.0, 8.0], [3.0, 4.0, 4.0]], [[6.0, 7.0, 
4.0], [4.0, 7.0, 1.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 5.0], [4.0, 2.0, 2.0], [2.0, 4.0, 7.0]], 
[[6.0, 8.0, 8.0], [1.0, 6.0, 1.0], [6.0, 1.0, 6.0]], [[8.0, 1.0, 7.0], [7.0, 6.0, 5.0], [7.0, 1.0, 3.0]]], [[[6.0, 5.0, 1.0], 
[7.0, 7.0, 2.0], [4.0, 2.0, 4.0]], [[5.0, 7.0, 4.0], [7.0, 3.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 3.0, 4.0], [3.0, 3.0, 7.0], 
[4.0, 3.0, 1.0]]], [[[8.0, 1.0, 5.0], [3.0, 6.0, 6.0], [8.0, 6.0, 5.0]], [[8.0, 6.0, 1.0], [6.0, 1.0, 2.0], [1.0, 8.0, 4.0]], 
[[1.0, 6.0, 5.0], [5.0, 7.0, 4.0], [2.0, 8.0, 8.0]]], [[[8.0, 6.0, 5.0], [8.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[3.0, 5.0, 6.0], 
[6.0, 1.0, 5.0], [5.0, 6.0, 6.0]], [[1.0, 5.0, 2.0], [8.0, 7.0, 1.0], [4.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 7.0, 7.0], [1.0, 3.0, 1.0]], [[5.0, 7.0, 8.0], [6.0, 8.0, 7.0], [5.0, 
4.0, 6.0]], [[2.0, 6.0, 8.0], [2.0, 5.0, 7.0], [7.0, 8.0, 7.0]]], [[[8.0, 8.0, 8.0], [3.0, 6.0, 3.0], [7.0, 3.0, 7.0]], [[1.0, 
3.0, 6.0], [2.0, 6.0, 5.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 2.0], [3.0, 6.0, 1.0], [3.0, 4.0, 3.0]]], [[[6.0, 7.0, 8.0], [2.0, 
5.0, 1.0], [5.0, 1.0, 7.0]], [[2.0, 6.0, 5.0], [8.0, 3.0, 3.0], [2.0, 7.0, 1.0]], [[5.0, 1.0, 4.0], [6.0, 1.0, 2.0], [4.0, 7.0, 
4.0]]], [[[1.0, 8.0, 6.0], [6.0, 8.0, 2.0], [3.0, 2.0, 2.0]], [[5.0, 7.0, 5.0], [6.0, 1.0, 2.0], [7.0, 3.0, 1.0]], [[4.0, 5.0, 
1.0], [8.0, 7.0, 2.0], [1.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 6.0], [1.0, 2.0, 3.0], [1.0, 4.0, 6.0]], 
[[5.0, 2.0, 5.0], [2.0, 2.0, 8.0], [4.0, 7.0, 4.0]], [[2.0, 2.0, 7.0], [5.0, 6.0, 2.0], [3.0, 2.0, 3.0]]], [[[7.0, 4.0, 2.0], 
[4.0, 5.0, 5.0], [1.0, 1.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 5.0, 7.0], [3.0, 1.0, 4.0]], [[5.0, 3.0, 8.0], [4.0, 7.0, 6.0], 
[4.0, 6.0, 4.0]]], [[[7.0, 5.0, 5.0], [7.0, 8.0, 6.0], [3.0, 4.0, 6.0]], [[3.0, 7.0, 4.0], [2.0, 4.0, 8.0], [5.0, 4.0, 5.0]], 
[[1.0, 5.0, 3.0], [5.0, 8.0, 5.0], [3.0, 3.0, 2.0]]], [[[6.0, 2.0, 3.0], [2.0, 7.0, 2.0], [5.0, 1.0, 8.0]], [[5.0, 3.0, 5.0], 
[7.0, 1.0, 1.0], [5.0, 1.0, 5.0]], [[6.0, 4.0, 4.0], [8.0, 3.0, 5.0], [3.0, 8.0, 7.0]]]])
      val=numpy.array([[[[7.0]]], [[[8.0]]]])
      arg[2:,2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 6.0], [6.0, 7.0, 7.0], [1.0, 3.0, 1.0]], [[5.0, 7.0, 8.0], [6.0, 8.0, 7.0], [5.0, 
4.0, 6.0]], [[2.0, 6.0, 8.0], [2.0, 5.0, 7.0], [7.0, 8.0, 7.0]]], [[[8.0, 8.0, 8.0], [3.0, 6.0, 3.0], [7.0, 3.0, 7.0]], [[1.0, 
3.0, 6.0], [2.0, 6.0, 5.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 2.0], [3.0, 6.0, 1.0], [3.0, 4.0, 3.0]]], [[[6.0, 7.0, 8.0], [2.0, 
5.0, 1.0], [5.0, 1.0, 7.0]], [[2.0, 6.0, 5.0], [8.0, 3.0, 3.0], [2.0, 7.0, 1.0]], [[5.0, 1.0, 4.0], [6.0, 1.0, 7.0], [4.0, 7.0, 
4.0]]], [[[1.0, 8.0, 6.0], [6.0, 8.0, 2.0], [3.0, 2.0, 2.0]], [[5.0, 7.0, 5.0], [6.0, 1.0, 2.0], [7.0, 3.0, 1.0]], [[4.0, 5.0, 
1.0], [8.0, 7.0, 8.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 6.0], [1.0, 2.0, 3.0], [1.0, 4.0, 6.0]], 
[[5.0, 2.0, 5.0], [2.0, 2.0, 8.0], [4.0, 7.0, 4.0]], [[2.0, 2.0, 7.0], [5.0, 6.0, 2.0], [3.0, 2.0, 3.0]]], [[[7.0, 4.0, 2.0], 
[4.0, 5.0, 5.0], [1.0, 1.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 5.0, 7.0], [3.0, 1.0, 4.0]], [[5.0, 3.0, 8.0], [4.0, 7.0, 6.0], 
[4.0, 6.0, 4.0]]], [[[7.0, 5.0, 5.0], [7.0, 8.0, 6.0], [3.0, 4.0, 6.0]], [[3.0, 7.0, 4.0], [2.0, 4.0, 8.0], [5.0, 4.0, 5.0]], 
[[1.0, 5.0, 3.0], [5.0, 8.0, 7.0], [3.0, 3.0, 2.0]]], [[[6.0, 2.0, 3.0], [2.0, 7.0, 2.0], [5.0, 1.0, 8.0]], [[5.0, 3.0, 5.0], 
[7.0, 1.0, 1.0], [5.0, 1.0, 5.0]], [[6.0, 4.0, 4.0], [8.0, 3.0, 8.0], [3.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 7.0], [3.0, 2.0, 4.0], [8.0, 5.0, 2.0]], [[7.0, 6.0, 7.0], [3.0, 4.0, 2.0], [8.0, 
3.0, 2.0]], [[7.0, 7.0, 8.0], [1.0, 1.0, 5.0], [3.0, 1.0, 5.0]]], [[[3.0, 2.0, 4.0], [6.0, 6.0, 2.0], [5.0, 3.0, 2.0]], [[6.0, 
6.0, 6.0], [2.0, 3.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 4.0, 2.0], [8.0, 3.0, 4.0], [6.0, 3.0, 7.0]]], [[[3.0, 3.0, 5.0], [4.0, 
6.0, 2.0], [3.0, 5.0, 3.0]], [[5.0, 7.0, 8.0], [3.0, 2.0, 8.0], [3.0, 2.0, 6.0]], [[8.0, 4.0, 6.0], [3.0, 4.0, 7.0], [3.0, 8.0, 
3.0]]], [[[5.0, 2.0, 4.0], [2.0, 4.0, 5.0], [7.0, 3.0, 1.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 6.0], [6.0, 4.0, 8.0]], [[6.0, 3.0, 
2.0], [2.0, 2.0, 6.0], [7.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 1.0], [1.0, 1.0, 7.0], [6.0, 3.0, 3.0]], 
[[6.0, 2.0, 6.0], [6.0, 2.0, 2.0], [8.0, 5.0, 8.0]], [[4.0, 3.0, 1.0], [8.0, 6.0, 3.0], [5.0, 4.0, 7.0]]], [[[5.0, 5.0, 3.0], 
[7.0, 7.0, 7.0], [4.0, 5.0, 6.0]], [[2.0, 1.0, 1.0], [2.0, 2.0, 7.0], [5.0, 3.0, 7.0]], [[2.0, 4.0, 4.0], [7.0, 6.0, 6.0], 
[7.0, 2.0, 8.0]]], [[[1.0, 7.0, 8.0], [8.0, 6.0, 3.0], [2.0, 5.0, 1.0]], [[7.0, 5.0, 6.0], [4.0, 3.0, 3.0], [7.0, 3.0, 3.0]], 
[[1.0, 2.0, 1.0], [8.0, 2.0, 4.0], [4.0, 7.0, 2.0]]], [[[5.0, 8.0, 8.0], [7.0, 6.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 1.0, 5.0], 
[8.0, 5.0, 3.0], [3.0, 8.0, 6.0]], [[1.0, 5.0, 5.0], [2.0, 2.0, 8.0], [1.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[[5.0]]], [[[4.0]]]]),self.functionspace)
      arg[2:,2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 7.0], [3.0, 2.0, 4.0], [8.0, 5.0, 2.0]], [[7.0, 6.0, 7.0], [3.0, 4.0, 2.0], [8.0, 
3.0, 2.0]], [[7.0, 7.0, 8.0], [1.0, 1.0, 5.0], [3.0, 1.0, 5.0]]], [[[3.0, 2.0, 4.0], [6.0, 6.0, 2.0], [5.0, 3.0, 2.0]], [[6.0, 
6.0, 6.0], [2.0, 3.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 4.0, 2.0], [8.0, 3.0, 4.0], [6.0, 3.0, 7.0]]], [[[3.0, 3.0, 5.0], [4.0, 
6.0, 2.0], [3.0, 5.0, 3.0]], [[5.0, 7.0, 8.0], [3.0, 2.0, 8.0], [3.0, 2.0, 6.0]], [[8.0, 4.0, 6.0], [3.0, 4.0, 5.0], [3.0, 8.0, 
3.0]]], [[[5.0, 2.0, 4.0], [2.0, 4.0, 5.0], [7.0, 3.0, 1.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 6.0], [6.0, 4.0, 8.0]], [[6.0, 3.0, 
2.0], [2.0, 2.0, 4.0], [7.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 1.0], [1.0, 1.0, 7.0], [6.0, 3.0, 3.0]], 
[[6.0, 2.0, 6.0], [6.0, 2.0, 2.0], [8.0, 5.0, 8.0]], [[4.0, 3.0, 1.0], [8.0, 6.0, 3.0], [5.0, 4.0, 7.0]]], [[[5.0, 5.0, 3.0], 
[7.0, 7.0, 7.0], [4.0, 5.0, 6.0]], [[2.0, 1.0, 1.0], [2.0, 2.0, 7.0], [5.0, 3.0, 7.0]], [[2.0, 4.0, 4.0], [7.0, 6.0, 6.0], 
[7.0, 2.0, 8.0]]], [[[1.0, 7.0, 8.0], [8.0, 6.0, 3.0], [2.0, 5.0, 1.0]], [[7.0, 5.0, 6.0], [4.0, 3.0, 3.0], [7.0, 3.0, 3.0]], 
[[1.0, 2.0, 1.0], [8.0, 2.0, 5.0], [4.0, 7.0, 2.0]]], [[[5.0, 8.0, 8.0], [7.0, 6.0, 1.0], [3.0, 5.0, 2.0]], [[2.0, 1.0, 5.0], 
[8.0, 5.0, 3.0], [3.0, 8.0, 6.0]], [[1.0, 5.0, 5.0], [2.0, 2.0, 4.0], [1.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLRL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 8.0], [6.0, 5.0, 3.0], [2.0, 4.0, 8.0]], [[1.0, 2.0, 3.0], [7.0, 7.0, 1.0], [5.0, 
5.0, 8.0]], [[6.0, 6.0, 4.0], [7.0, 3.0, 2.0], [8.0, 8.0, 3.0]]], [[[1.0, 4.0, 7.0], [4.0, 1.0, 8.0], [5.0, 3.0, 1.0]], [[3.0, 
7.0, 1.0], [3.0, 2.0, 2.0], [6.0, 2.0, 8.0]], [[1.0, 3.0, 8.0], [8.0, 3.0, 8.0], [8.0, 3.0, 7.0]]], [[[4.0, 8.0, 7.0], [3.0, 
2.0, 6.0], [5.0, 4.0, 4.0]], [[3.0, 5.0, 3.0], [7.0, 6.0, 3.0], [3.0, 8.0, 1.0]], [[7.0, 1.0, 3.0], [3.0, 8.0, 2.0], [8.0, 6.0, 
6.0]]], [[[3.0, 6.0, 2.0], [7.0, 3.0, 2.0], [1.0, 7.0, 8.0]], [[7.0, 2.0, 1.0], [4.0, 2.0, 6.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 
5.0], [3.0, 2.0, 4.0], [4.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 7.0], [4.0, 2.0, 2.0], [4.0, 1.0, 5.0]], 
[[7.0, 5.0, 6.0], [8.0, 6.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 3.0, 7.0], [5.0, 8.0, 7.0], [1.0, 7.0, 5.0]]], [[[4.0, 3.0, 7.0], 
[2.0, 6.0, 6.0], [2.0, 1.0, 8.0]], [[6.0, 6.0, 8.0], [6.0, 7.0, 7.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 1.0, 6.0], 
[2.0, 7.0, 4.0]]], [[[8.0, 3.0, 5.0], [6.0, 5.0, 3.0], [8.0, 1.0, 7.0]], [[3.0, 7.0, 8.0], [6.0, 3.0, 4.0], [2.0, 5.0, 2.0]], 
[[3.0, 5.0, 1.0], [4.0, 2.0, 6.0], [8.0, 3.0, 1.0]]], [[[7.0, 4.0, 1.0], [3.0, 2.0, 6.0], [6.0, 5.0, 4.0]], [[4.0, 4.0, 1.0], 
[2.0, 4.0, 6.0], [7.0, 6.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 8.0, 2.0], [5.0, 6.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0]]], [[[4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0]]], [[[7.0]]]])
      arg[2:,2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 8.0], [6.0, 5.0, 3.0], [2.0, 4.0, 8.0]], [[1.0, 2.0, 3.0], [7.0, 7.0, 1.0], [5.0, 
5.0, 8.0]], [[6.0, 6.0, 4.0], [7.0, 3.0, 2.0], [8.0, 8.0, 3.0]]], [[[1.0, 4.0, 7.0], [4.0, 1.0, 8.0], [5.0, 3.0, 1.0]], [[3.0, 
7.0, 1.0], [3.0, 2.0, 2.0], [6.0, 2.0, 8.0]], [[1.0, 3.0, 8.0], [8.0, 3.0, 8.0], [8.0, 3.0, 7.0]]], [[[4.0, 8.0, 7.0], [3.0, 
2.0, 6.0], [5.0, 4.0, 4.0]], [[3.0, 5.0, 3.0], [7.0, 6.0, 3.0], [3.0, 8.0, 1.0]], [[7.0, 1.0, 3.0], [3.0, 8.0, 8.0], [8.0, 6.0, 
6.0]]], [[[3.0, 6.0, 2.0], [7.0, 3.0, 2.0], [1.0, 7.0, 8.0]], [[7.0, 2.0, 1.0], [4.0, 2.0, 6.0], [3.0, 8.0, 3.0]], [[5.0, 7.0, 
5.0], [3.0, 2.0, 4.0], [4.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [4.0, 2.0, 2.0], [4.0, 1.0, 5.0]], 
[[7.0, 5.0, 6.0], [8.0, 6.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 3.0, 7.0], [5.0, 8.0, 7.0], [1.0, 7.0, 5.0]]], [[[4.0, 3.0, 7.0], 
[2.0, 6.0, 6.0], [2.0, 1.0, 8.0]], [[6.0, 6.0, 8.0], [6.0, 7.0, 7.0], [4.0, 6.0, 2.0]], [[2.0, 3.0, 7.0], [1.0, 1.0, 6.0], 
[2.0, 7.0, 4.0]]], [[[8.0, 3.0, 5.0], [6.0, 5.0, 3.0], [8.0, 1.0, 7.0]], [[3.0, 7.0, 8.0], [6.0, 3.0, 4.0], [2.0, 5.0, 2.0]], 
[[3.0, 5.0, 1.0], [4.0, 2.0, 5.0], [8.0, 3.0, 1.0]]], [[[7.0, 4.0, 1.0], [3.0, 2.0, 6.0], [6.0, 5.0, 4.0]], [[4.0, 4.0, 1.0], 
[2.0, 4.0, 6.0], [7.0, 6.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 8.0, 7.0], [5.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 1.0], [7.0, 6.0, 3.0], [4.0, 2.0, 8.0]], [[1.0, 7.0, 4.0], [1.0, 2.0, 4.0], [8.0, 
2.0, 4.0]], [[1.0, 2.0, 3.0], [4.0, 3.0, 4.0], [4.0, 8.0, 5.0]]], [[[6.0, 1.0, 2.0], [2.0, 1.0, 6.0], [4.0, 6.0, 7.0]], [[4.0, 
3.0, 5.0], [4.0, 2.0, 7.0], [3.0, 5.0, 4.0]], [[8.0, 2.0, 1.0], [8.0, 4.0, 6.0], [6.0, 3.0, 7.0]]], [[[7.0, 7.0, 8.0], [7.0, 
6.0, 6.0], [8.0, 6.0, 8.0]], [[7.0, 3.0, 2.0], [7.0, 4.0, 7.0], [7.0, 7.0, 4.0]], [[3.0, 5.0, 7.0], [8.0, 4.0, 6.0], [5.0, 6.0, 
6.0]]], [[[1.0, 6.0, 8.0], [4.0, 8.0, 8.0], [1.0, 1.0, 8.0]], [[2.0, 6.0, 8.0], [6.0, 6.0, 5.0], [2.0, 3.0, 5.0]], [[7.0, 1.0, 
3.0], [5.0, 7.0, 7.0], [4.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 4.0], [6.0, 4.0, 8.0], [2.0, 4.0, 2.0]], 
[[7.0, 6.0, 4.0], [6.0, 6.0, 8.0], [8.0, 7.0, 4.0]], [[4.0, 1.0, 3.0], [8.0, 8.0, 5.0], [1.0, 2.0, 8.0]]], [[[3.0, 5.0, 5.0], 
[5.0, 6.0, 3.0], [8.0, 7.0, 8.0]], [[2.0, 6.0, 4.0], [5.0, 8.0, 7.0], [5.0, 4.0, 3.0]], [[4.0, 1.0, 8.0], [6.0, 4.0, 1.0], 
[1.0, 5.0, 8.0]]], [[[8.0, 5.0, 7.0], [3.0, 3.0, 5.0], [5.0, 6.0, 4.0]], [[2.0, 1.0, 4.0], [2.0, 1.0, 5.0], [8.0, 5.0, 5.0]], 
[[4.0, 5.0, 4.0], [1.0, 3.0, 7.0], [8.0, 5.0, 7.0]]], [[[5.0, 8.0, 3.0], [7.0, 2.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 7.0, 4.0], 
[4.0, 7.0, 3.0], [1.0, 1.0, 6.0]], [[3.0, 6.0, 4.0], [5.0, 8.0, 2.0], [3.0, 3.0, 1.0]]]])
      val=numpy.array([[[[3.0, 1.0]], [[5.0, 7.0]], [[2.0, 4.0]]], [[[6.0, 7.0]], [[3.0, 2.0]], [[2.0, 1.0]]], [[[1.0, 8.0]], 
[[4.0, 7.0]], [[2.0, 5.0]]], [[[2.0, 5.0]], [[2.0, 1.0]], [[1.0, 8.0]]]])
      arg[:,:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 1.0], [3.0, 1.0, 3.0], [4.0, 2.0, 8.0]], [[1.0, 7.0, 4.0], [5.0, 7.0, 4.0], [8.0, 
2.0, 4.0]], [[1.0, 2.0, 3.0], [2.0, 4.0, 4.0], [4.0, 8.0, 5.0]]], [[[6.0, 1.0, 2.0], [6.0, 7.0, 6.0], [4.0, 6.0, 7.0]], [[4.0, 
3.0, 5.0], [3.0, 2.0, 7.0], [3.0, 5.0, 4.0]], [[8.0, 2.0, 1.0], [2.0, 1.0, 6.0], [6.0, 3.0, 7.0]]], [[[7.0, 7.0, 8.0], [1.0, 
8.0, 6.0], [8.0, 6.0, 8.0]], [[7.0, 3.0, 2.0], [4.0, 7.0, 7.0], [7.0, 7.0, 4.0]], [[3.0, 5.0, 7.0], [2.0, 5.0, 6.0], [5.0, 6.0, 
6.0]]], [[[1.0, 6.0, 8.0], [2.0, 5.0, 8.0], [1.0, 1.0, 8.0]], [[2.0, 6.0, 8.0], [2.0, 1.0, 5.0], [2.0, 3.0, 5.0]], [[7.0, 1.0, 
3.0], [1.0, 8.0, 7.0], [4.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 4.0], [3.0, 1.0, 8.0], [2.0, 4.0, 2.0]], 
[[7.0, 6.0, 4.0], [5.0, 7.0, 8.0], [8.0, 7.0, 4.0]], [[4.0, 1.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 8.0]]], [[[3.0, 5.0, 5.0], 
[6.0, 7.0, 3.0], [8.0, 7.0, 8.0]], [[2.0, 6.0, 4.0], [3.0, 2.0, 7.0], [5.0, 4.0, 3.0]], [[4.0, 1.0, 8.0], [2.0, 1.0, 1.0], 
[1.0, 5.0, 8.0]]], [[[8.0, 5.0, 7.0], [1.0, 8.0, 5.0], [5.0, 6.0, 4.0]], [[2.0, 1.0, 4.0], [4.0, 7.0, 5.0], [8.0, 5.0, 5.0]], 
[[4.0, 5.0, 4.0], [2.0, 5.0, 7.0], [8.0, 5.0, 7.0]]], [[[5.0, 8.0, 3.0], [2.0, 5.0, 7.0], [1.0, 1.0, 3.0]], [[6.0, 7.0, 4.0], 
[2.0, 1.0, 3.0], [1.0, 1.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 8.0, 2.0], [3.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 8.0], [4.0, 5.0, 3.0], [5.0, 6.0, 7.0]], [[6.0, 4.0, 5.0], [8.0, 2.0, 2.0], [6.0, 
3.0, 3.0]], [[5.0, 2.0, 6.0], [1.0, 2.0, 3.0], [7.0, 7.0, 8.0]]], [[[4.0, 3.0, 5.0], [4.0, 7.0, 1.0], [7.0, 4.0, 6.0]], [[3.0, 
4.0, 7.0], [6.0, 6.0, 2.0], [6.0, 8.0, 8.0]], [[1.0, 4.0, 7.0], [3.0, 2.0, 6.0], [6.0, 3.0, 1.0]]], [[[4.0, 8.0, 1.0], [5.0, 
3.0, 4.0], [2.0, 7.0, 7.0]], [[7.0, 6.0, 1.0], [4.0, 4.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 4.0, 8.0], [7.0, 5.0, 4.0], [7.0, 1.0, 
5.0]]], [[[1.0, 4.0, 3.0], [7.0, 8.0, 2.0], [5.0, 1.0, 5.0]], [[4.0, 6.0, 7.0], [5.0, 3.0, 1.0], [8.0, 5.0, 8.0]], [[1.0, 4.0, 
8.0], [7.0, 4.0, 8.0], [8.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 8.0], [2.0, 5.0, 3.0], [3.0, 8.0, 1.0]], 
[[7.0, 4.0, 4.0], [4.0, 2.0, 3.0], [7.0, 2.0, 1.0]], [[4.0, 7.0, 5.0], [2.0, 3.0, 2.0], [4.0, 7.0, 4.0]]], [[[4.0, 1.0, 1.0], 
[4.0, 5.0, 5.0], [3.0, 4.0, 7.0]], [[8.0, 4.0, 6.0], [4.0, 5.0, 4.0], [1.0, 5.0, 1.0]], [[3.0, 1.0, 6.0], [5.0, 2.0, 7.0], 
[7.0, 6.0, 7.0]]], [[[5.0, 3.0, 8.0], [6.0, 7.0, 8.0], [6.0, 2.0, 2.0]], [[8.0, 6.0, 3.0], [1.0, 4.0, 1.0], [8.0, 8.0, 3.0]], 
[[1.0, 5.0, 2.0], [7.0, 4.0, 7.0], [6.0, 8.0, 7.0]]], [[[2.0, 6.0, 5.0], [1.0, 5.0, 5.0], [2.0, 7.0, 5.0]], [[5.0, 5.0, 7.0], 
[4.0, 6.0, 7.0], [5.0, 6.0, 3.0]], [[7.0, 8.0, 2.0], [7.0, 8.0, 3.0], [1.0, 2.0, 1.0]]]])
      val=Data(numpy.array([[[[6.0, 6.0]], [[5.0, 6.0]], [[4.0, 5.0]]], [[[2.0, 5.0]], [[7.0, 5.0]], [[7.0, 3.0]]], [[[7.0, 
2.0]], [[5.0, 3.0]], [[1.0, 8.0]]], [[[4.0, 3.0]], [[6.0, 7.0]], [[1.0, 6.0]]]]),self.functionspace)
      arg[:,:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 8.0], [6.0, 6.0, 3.0], [5.0, 6.0, 7.0]], [[6.0, 4.0, 5.0], [5.0, 6.0, 2.0], [6.0, 
3.0, 3.0]], [[5.0, 2.0, 6.0], [4.0, 5.0, 3.0], [7.0, 7.0, 8.0]]], [[[4.0, 3.0, 5.0], [2.0, 5.0, 1.0], [7.0, 4.0, 6.0]], [[3.0, 
4.0, 7.0], [7.0, 5.0, 2.0], [6.0, 8.0, 8.0]], [[1.0, 4.0, 7.0], [7.0, 3.0, 6.0], [6.0, 3.0, 1.0]]], [[[4.0, 8.0, 1.0], [7.0, 
2.0, 4.0], [2.0, 7.0, 7.0]], [[7.0, 6.0, 1.0], [5.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 4.0, 8.0], [1.0, 8.0, 4.0], [7.0, 1.0, 
5.0]]], [[[1.0, 4.0, 3.0], [4.0, 3.0, 2.0], [5.0, 1.0, 5.0]], [[4.0, 6.0, 7.0], [6.0, 7.0, 1.0], [8.0, 5.0, 8.0]], [[1.0, 4.0, 
8.0], [1.0, 6.0, 8.0], [8.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 8.0], [6.0, 6.0, 3.0], [3.0, 8.0, 1.0]], 
[[7.0, 4.0, 4.0], [5.0, 6.0, 3.0], [7.0, 2.0, 1.0]], [[4.0, 7.0, 5.0], [4.0, 5.0, 2.0], [4.0, 7.0, 4.0]]], [[[4.0, 1.0, 1.0], 
[2.0, 5.0, 5.0], [3.0, 4.0, 7.0]], [[8.0, 4.0, 6.0], [7.0, 5.0, 4.0], [1.0, 5.0, 1.0]], [[3.0, 1.0, 6.0], [7.0, 3.0, 7.0], 
[7.0, 6.0, 7.0]]], [[[5.0, 3.0, 8.0], [7.0, 2.0, 8.0], [6.0, 2.0, 2.0]], [[8.0, 6.0, 3.0], [5.0, 3.0, 1.0], [8.0, 8.0, 3.0]], 
[[1.0, 5.0, 2.0], [1.0, 8.0, 7.0], [6.0, 8.0, 7.0]]], [[[2.0, 6.0, 5.0], [4.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[5.0, 5.0, 7.0], 
[6.0, 7.0, 7.0], [5.0, 6.0, 3.0]], [[7.0, 8.0, 2.0], [1.0, 6.0, 3.0], [1.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 3.0], [6.0, 1.0, 4.0], [2.0, 2.0, 1.0]], [[6.0, 8.0, 7.0], [8.0, 3.0, 4.0], [1.0, 
5.0, 4.0]], [[5.0, 1.0, 6.0], [1.0, 6.0, 4.0], [1.0, 8.0, 2.0]]], [[[2.0, 1.0, 6.0], [4.0, 3.0, 7.0], [1.0, 6.0, 3.0]], [[3.0, 
5.0, 2.0], [4.0, 6.0, 3.0], [6.0, 2.0, 7.0]], [[4.0, 1.0, 4.0], [4.0, 6.0, 4.0], [3.0, 6.0, 4.0]]], [[[4.0, 7.0, 8.0], [6.0, 
5.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 4.0, 4.0], [6.0, 5.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], [1.0, 2.0, 8.0], [8.0, 4.0, 
8.0]]], [[[8.0, 3.0, 2.0], [4.0, 4.0, 5.0], [6.0, 1.0, 5.0]], [[3.0, 4.0, 4.0], [2.0, 6.0, 5.0], [1.0, 7.0, 3.0]], [[3.0, 5.0, 
3.0], [4.0, 2.0, 7.0], [4.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 6.0], [5.0, 6.0, 4.0], [1.0, 2.0, 7.0]], 
[[4.0, 3.0, 1.0], [7.0, 3.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 3.0, 4.0], [2.0, 2.0, 2.0], [4.0, 1.0, 8.0]]], [[[5.0, 5.0, 5.0], 
[5.0, 2.0, 2.0], [6.0, 1.0, 2.0]], [[3.0, 7.0, 5.0], [7.0, 5.0, 7.0], [5.0, 1.0, 3.0]], [[1.0, 6.0, 7.0], [1.0, 4.0, 7.0], 
[2.0, 3.0, 4.0]]], [[[3.0, 7.0, 3.0], [8.0, 4.0, 8.0], [3.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [2.0, 6.0, 7.0], [6.0, 1.0, 8.0]], 
[[6.0, 7.0, 1.0], [5.0, 5.0, 5.0], [4.0, 2.0, 1.0]]], [[[8.0, 8.0, 1.0], [6.0, 4.0, 8.0], [3.0, 6.0, 4.0]], [[5.0, 7.0, 2.0], 
[8.0, 3.0, 8.0], [2.0, 2.0, 6.0]], [[7.0, 1.0, 2.0], [3.0, 4.0, 8.0], [8.0, 7.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 4.0]], [[2.0, 8.0]], [[1.0, 4.0]]], [[[8.0, 1.0]], [[7.0, 8.0]], [[7.0, 5.0]]], [[[7.0, 
7.0]], [[6.0, 6.0]], [[7.0, 5.0]]], [[[1.0, 3.0]], [[8.0, 3.0]], [[8.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 4.0]], 
[[8.0, 1.0]], [[6.0, 5.0]]], [[[2.0, 5.0]], [[8.0, 8.0]], [[6.0, 8.0]]], [[[8.0, 3.0]], [[5.0, 7.0]], [[3.0, 2.0]]], [[[7.0, 
1.0]], [[8.0, 5.0]], [[7.0, 7.0]]]])
      arg[:,:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 3.0], [6.0, 4.0, 4.0], [2.0, 2.0, 1.0]], [[6.0, 8.0, 7.0], [2.0, 8.0, 4.0], [1.0, 
5.0, 4.0]], [[5.0, 1.0, 6.0], [1.0, 4.0, 4.0], [1.0, 8.0, 2.0]]], [[[2.0, 1.0, 6.0], [8.0, 1.0, 7.0], [1.0, 6.0, 3.0]], [[3.0, 
5.0, 2.0], [7.0, 8.0, 3.0], [6.0, 2.0, 7.0]], [[4.0, 1.0, 4.0], [7.0, 5.0, 4.0], [3.0, 6.0, 4.0]]], [[[4.0, 7.0, 8.0], [7.0, 
7.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 4.0, 4.0], [6.0, 6.0, 4.0], [2.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], [7.0, 5.0, 8.0], [8.0, 4.0, 
8.0]]], [[[8.0, 3.0, 2.0], [1.0, 3.0, 5.0], [6.0, 1.0, 5.0]], [[3.0, 4.0, 4.0], [8.0, 3.0, 5.0], [1.0, 7.0, 3.0]], [[3.0, 5.0, 
3.0], [8.0, 2.0, 7.0], [4.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 6.0], [1.0, 4.0, 4.0], [1.0, 2.0, 7.0]], 
[[4.0, 3.0, 1.0], [8.0, 1.0, 5.0], [3.0, 2.0, 8.0]], [[2.0, 3.0, 4.0], [6.0, 5.0, 2.0], [4.0, 1.0, 8.0]]], [[[5.0, 5.0, 5.0], 
[2.0, 5.0, 2.0], [6.0, 1.0, 2.0]], [[3.0, 7.0, 5.0], [8.0, 8.0, 7.0], [5.0, 1.0, 3.0]], [[1.0, 6.0, 7.0], [6.0, 8.0, 7.0], 
[2.0, 3.0, 4.0]]], [[[3.0, 7.0, 3.0], [8.0, 3.0, 8.0], [3.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [5.0, 7.0, 7.0], [6.0, 1.0, 8.0]], 
[[6.0, 7.0, 1.0], [3.0, 2.0, 5.0], [4.0, 2.0, 1.0]]], [[[8.0, 8.0, 1.0], [7.0, 1.0, 8.0], [3.0, 6.0, 4.0]], [[5.0, 7.0, 2.0], 
[8.0, 5.0, 8.0], [2.0, 2.0, 6.0]], [[7.0, 1.0, 2.0], [7.0, 7.0, 8.0], [8.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 8.0], [4.0, 8.0, 3.0], [8.0, 2.0, 6.0]], [[4.0, 8.0, 8.0], [3.0, 7.0, 1.0], [1.0, 
6.0, 5.0]], [[6.0, 3.0, 6.0], [2.0, 2.0, 1.0], [5.0, 5.0, 1.0]]], [[[6.0, 5.0, 4.0], [6.0, 6.0, 4.0], [2.0, 1.0, 7.0]], [[1.0, 
7.0, 5.0], [8.0, 7.0, 3.0], [4.0, 6.0, 1.0]], [[3.0, 3.0, 1.0], [2.0, 6.0, 7.0], [4.0, 6.0, 7.0]]], [[[1.0, 5.0, 7.0], [3.0, 
1.0, 2.0], [5.0, 8.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 2.0, 7.0], [8.0, 2.0, 7.0]], [[7.0, 5.0, 5.0], [6.0, 1.0, 1.0], [6.0, 4.0, 
8.0]]], [[[4.0, 1.0, 1.0], [1.0, 1.0, 5.0], [1.0, 8.0, 6.0]], [[4.0, 6.0, 2.0], [6.0, 6.0, 1.0], [3.0, 4.0, 6.0]], [[6.0, 3.0, 
6.0], [2.0, 1.0, 4.0], [1.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 1.0], [2.0, 7.0, 8.0], [2.0, 8.0, 5.0]], 
[[3.0, 5.0, 1.0], [2.0, 8.0, 2.0], [3.0, 6.0, 5.0]], [[3.0, 3.0, 3.0], [7.0, 4.0, 2.0], [4.0, 2.0, 1.0]]], [[[8.0, 7.0, 8.0], 
[5.0, 3.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 2.0, 8.0], [5.0, 6.0, 8.0], [7.0, 4.0, 2.0]], [[2.0, 5.0, 4.0], [6.0, 6.0, 1.0], 
[1.0, 5.0, 6.0]]], [[[7.0, 1.0, 6.0], [2.0, 8.0, 2.0], [8.0, 3.0, 5.0]], [[8.0, 4.0, 1.0], [3.0, 3.0, 2.0], [4.0, 1.0, 8.0]], 
[[3.0, 1.0, 2.0], [6.0, 4.0, 3.0], [5.0, 4.0, 1.0]]], [[[7.0, 1.0, 1.0], [7.0, 6.0, 6.0], [8.0, 5.0, 4.0]], [[2.0, 1.0, 2.0], 
[4.0, 1.0, 5.0], [6.0, 2.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 1.0, 1.0], [5.0, 4.0, 3.0]]]])
      val=numpy.array([[[7.0, 4.0, 1.0]], [[2.0, 7.0, 2.0]], [[4.0, 6.0, 5.0]]])
      arg[:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 8.0], [4.0, 8.0, 3.0], [8.0, 2.0, 6.0]], [[4.0, 8.0, 8.0], [3.0, 7.0, 1.0], [7.0, 
4.0, 1.0]], [[6.0, 3.0, 6.0], [2.0, 2.0, 1.0], [5.0, 5.0, 1.0]]], [[[6.0, 5.0, 4.0], [6.0, 6.0, 4.0], [2.0, 1.0, 7.0]], [[1.0, 
7.0, 5.0], [8.0, 7.0, 3.0], [2.0, 7.0, 2.0]], [[3.0, 3.0, 1.0], [2.0, 6.0, 7.0], [4.0, 6.0, 7.0]]], [[[1.0, 5.0, 7.0], [3.0, 
1.0, 2.0], [5.0, 8.0, 8.0]], [[6.0, 2.0, 1.0], [5.0, 2.0, 7.0], [4.0, 6.0, 5.0]], [[7.0, 5.0, 5.0], [6.0, 1.0, 1.0], [6.0, 4.0, 
8.0]]], [[[4.0, 1.0, 1.0], [1.0, 1.0, 5.0], [1.0, 8.0, 6.0]], [[4.0, 6.0, 2.0], [6.0, 6.0, 1.0], [3.0, 4.0, 6.0]], [[6.0, 3.0, 
6.0], [2.0, 1.0, 4.0], [1.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 1.0], [2.0, 7.0, 8.0], [2.0, 8.0, 5.0]], 
[[3.0, 5.0, 1.0], [2.0, 8.0, 2.0], [7.0, 4.0, 1.0]], [[3.0, 3.0, 3.0], [7.0, 4.0, 2.0], [4.0, 2.0, 1.0]]], [[[8.0, 7.0, 8.0], 
[5.0, 3.0, 1.0], [7.0, 4.0, 6.0]], [[6.0, 2.0, 8.0], [5.0, 6.0, 8.0], [2.0, 7.0, 2.0]], [[2.0, 5.0, 4.0], [6.0, 6.0, 1.0], 
[1.0, 5.0, 6.0]]], [[[7.0, 1.0, 6.0], [2.0, 8.0, 2.0], [8.0, 3.0, 5.0]], [[8.0, 4.0, 1.0], [3.0, 3.0, 2.0], [4.0, 6.0, 5.0]], 
[[3.0, 1.0, 2.0], [6.0, 4.0, 3.0], [5.0, 4.0, 1.0]]], [[[7.0, 1.0, 1.0], [7.0, 6.0, 6.0], [8.0, 5.0, 4.0]], [[2.0, 1.0, 2.0], 
[4.0, 1.0, 5.0], [6.0, 2.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 1.0, 1.0], [5.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 4.0], [1.0, 4.0, 1.0], [6.0, 3.0, 6.0]], [[3.0, 6.0, 7.0], [3.0, 2.0, 2.0], [7.0, 
4.0, 1.0]], [[6.0, 3.0, 8.0], [6.0, 2.0, 1.0], [4.0, 4.0, 8.0]]], [[[4.0, 6.0, 8.0], [7.0, 5.0, 3.0], [6.0, 4.0, 8.0]], [[5.0, 
5.0, 6.0], [8.0, 2.0, 8.0], [6.0, 7.0, 8.0]], [[7.0, 5.0, 4.0], [8.0, 2.0, 4.0], [1.0, 1.0, 7.0]]], [[[1.0, 3.0, 6.0], [4.0, 
1.0, 6.0], [3.0, 5.0, 2.0]], [[1.0, 5.0, 7.0], [1.0, 8.0, 2.0], [5.0, 2.0, 3.0]], [[8.0, 6.0, 4.0], [4.0, 7.0, 7.0], [7.0, 8.0, 
2.0]]], [[[4.0, 8.0, 1.0], [5.0, 2.0, 6.0], [2.0, 1.0, 5.0]], [[1.0, 1.0, 2.0], [3.0, 8.0, 2.0], [6.0, 6.0, 4.0]], [[5.0, 5.0, 
1.0], [6.0, 7.0, 2.0], [7.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 6.0], [4.0, 5.0, 4.0], [2.0, 7.0, 6.0]], 
[[1.0, 6.0, 5.0], [5.0, 5.0, 8.0], [3.0, 8.0, 7.0]], [[7.0, 2.0, 6.0], [1.0, 7.0, 4.0], [4.0, 6.0, 7.0]]], [[[8.0, 5.0, 4.0], 
[4.0, 5.0, 6.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 4.0], [8.0, 7.0, 4.0], [2.0, 4.0, 3.0]], [[3.0, 8.0, 2.0], [5.0, 2.0, 2.0], 
[1.0, 1.0, 2.0]]], [[[8.0, 1.0, 6.0], [1.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[7.0, 1.0, 7.0], [2.0, 1.0, 8.0], [2.0, 8.0, 5.0]], 
[[7.0, 6.0, 5.0], [6.0, 6.0, 4.0], [7.0, 6.0, 1.0]]], [[[5.0, 4.0, 5.0], [5.0, 2.0, 5.0], [6.0, 1.0, 3.0]], [[7.0, 5.0, 1.0], 
[4.0, 8.0, 4.0], [6.0, 2.0, 4.0]], [[3.0, 6.0, 6.0], [2.0, 6.0, 4.0], [2.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[1.0, 1.0, 2.0]], [[5.0, 5.0, 4.0]], [[6.0, 3.0, 4.0]]]),self.functionspace)
      arg[:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 4.0], [1.0, 4.0, 1.0], [6.0, 3.0, 6.0]], [[3.0, 6.0, 7.0], [3.0, 2.0, 2.0], [1.0, 
1.0, 2.0]], [[6.0, 3.0, 8.0], [6.0, 2.0, 1.0], [4.0, 4.0, 8.0]]], [[[4.0, 6.0, 8.0], [7.0, 5.0, 3.0], [6.0, 4.0, 8.0]], [[5.0, 
5.0, 6.0], [8.0, 2.0, 8.0], [5.0, 5.0, 4.0]], [[7.0, 5.0, 4.0], [8.0, 2.0, 4.0], [1.0, 1.0, 7.0]]], [[[1.0, 3.0, 6.0], [4.0, 
1.0, 6.0], [3.0, 5.0, 2.0]], [[1.0, 5.0, 7.0], [1.0, 8.0, 2.0], [6.0, 3.0, 4.0]], [[8.0, 6.0, 4.0], [4.0, 7.0, 7.0], [7.0, 8.0, 
2.0]]], [[[4.0, 8.0, 1.0], [5.0, 2.0, 6.0], [2.0, 1.0, 5.0]], [[1.0, 1.0, 2.0], [3.0, 8.0, 2.0], [6.0, 6.0, 4.0]], [[5.0, 5.0, 
1.0], [6.0, 7.0, 2.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 6.0], [4.0, 5.0, 4.0], [2.0, 7.0, 6.0]], 
[[1.0, 6.0, 5.0], [5.0, 5.0, 8.0], [1.0, 1.0, 2.0]], [[7.0, 2.0, 6.0], [1.0, 7.0, 4.0], [4.0, 6.0, 7.0]]], [[[8.0, 5.0, 4.0], 
[4.0, 5.0, 6.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 4.0], [8.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[3.0, 8.0, 2.0], [5.0, 2.0, 2.0], 
[1.0, 1.0, 2.0]]], [[[8.0, 1.0, 6.0], [1.0, 8.0, 6.0], [6.0, 2.0, 1.0]], [[7.0, 1.0, 7.0], [2.0, 1.0, 8.0], [6.0, 3.0, 4.0]], 
[[7.0, 6.0, 5.0], [6.0, 6.0, 4.0], [7.0, 6.0, 1.0]]], [[[5.0, 4.0, 5.0], [5.0, 2.0, 5.0], [6.0, 1.0, 3.0]], [[7.0, 5.0, 1.0], 
[4.0, 8.0, 4.0], [6.0, 2.0, 4.0]], [[3.0, 6.0, 6.0], [2.0, 6.0, 4.0], [2.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 2.0], [5.0, 5.0, 5.0], [7.0, 6.0, 4.0]], [[3.0, 1.0, 7.0], [5.0, 6.0, 4.0], [4.0, 
7.0, 6.0]], [[1.0, 4.0, 1.0], [4.0, 4.0, 6.0], [3.0, 7.0, 7.0]]], [[[5.0, 2.0, 2.0], [8.0, 1.0, 8.0], [2.0, 1.0, 7.0]], [[1.0, 
6.0, 4.0], [4.0, 8.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 4.0, 2.0], [2.0, 8.0, 5.0], [7.0, 1.0, 3.0]]], [[[8.0, 3.0, 2.0], [6.0, 
5.0, 3.0], [7.0, 4.0, 4.0]], [[7.0, 8.0, 1.0], [4.0, 2.0, 7.0], [8.0, 2.0, 8.0]], [[7.0, 1.0, 5.0], [3.0, 8.0, 3.0], [6.0, 5.0, 
7.0]]], [[[2.0, 1.0, 6.0], [3.0, 1.0, 2.0], [8.0, 1.0, 1.0]], [[4.0, 4.0, 6.0], [4.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[2.0, 3.0, 
3.0], [7.0, 2.0, 5.0], [7.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 6.0], [2.0, 3.0, 7.0], [5.0, 4.0, 8.0]], 
[[3.0, 7.0, 4.0], [7.0, 4.0, 4.0], [7.0, 5.0, 6.0]], [[7.0, 6.0, 2.0], [4.0, 6.0, 4.0], [3.0, 5.0, 5.0]]], [[[4.0, 2.0, 1.0], 
[5.0, 2.0, 6.0], [4.0, 7.0, 5.0]], [[8.0, 7.0, 2.0], [5.0, 6.0, 4.0], [2.0, 7.0, 1.0]], [[5.0, 6.0, 8.0], [1.0, 5.0, 8.0], 
[5.0, 8.0, 5.0]]], [[[3.0, 5.0, 7.0], [8.0, 2.0, 4.0], [3.0, 1.0, 5.0]], [[3.0, 4.0, 6.0], [7.0, 5.0, 6.0], [3.0, 5.0, 5.0]], 
[[5.0, 3.0, 2.0], [6.0, 3.0, 2.0], [1.0, 4.0, 2.0]]], [[[5.0, 7.0, 4.0], [1.0, 5.0, 1.0], [5.0, 6.0, 8.0]], [[3.0, 3.0, 4.0], 
[3.0, 6.0, 1.0], [3.0, 6.0, 1.0]], [[3.0, 1.0, 1.0], [3.0, 2.0, 4.0], [6.0, 6.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 8.0, 5.0]], [[2.0, 6.0, 3.0]], [[6.0, 4.0, 5.0]]])+(1.-msk_val)*numpy.array([[[6.0, 3.0, 
2.0]], [[6.0, 1.0, 3.0]], [[2.0, 2.0, 2.0]]])
      arg[:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 2.0], [5.0, 5.0, 5.0], [7.0, 6.0, 4.0]], [[3.0, 1.0, 7.0], [5.0, 6.0, 4.0], [2.0, 
8.0, 5.0]], [[1.0, 4.0, 1.0], [4.0, 4.0, 6.0], [3.0, 7.0, 7.0]]], [[[5.0, 2.0, 2.0], [8.0, 1.0, 8.0], [2.0, 1.0, 7.0]], [[1.0, 
6.0, 4.0], [4.0, 8.0, 5.0], [2.0, 6.0, 3.0]], [[2.0, 4.0, 2.0], [2.0, 8.0, 5.0], [7.0, 1.0, 3.0]]], [[[8.0, 3.0, 2.0], [6.0, 
5.0, 3.0], [7.0, 4.0, 4.0]], [[7.0, 8.0, 1.0], [4.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[7.0, 1.0, 5.0], [3.0, 8.0, 3.0], [6.0, 5.0, 
7.0]]], [[[2.0, 1.0, 6.0], [3.0, 1.0, 2.0], [8.0, 1.0, 1.0]], [[4.0, 4.0, 6.0], [4.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[2.0, 3.0, 
3.0], [7.0, 2.0, 5.0], [7.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 6.0], [2.0, 3.0, 7.0], [5.0, 4.0, 8.0]], 
[[3.0, 7.0, 4.0], [7.0, 4.0, 4.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 2.0], [4.0, 6.0, 4.0], [3.0, 5.0, 5.0]]], [[[4.0, 2.0, 1.0], 
[5.0, 2.0, 6.0], [4.0, 7.0, 5.0]], [[8.0, 7.0, 2.0], [5.0, 6.0, 4.0], [6.0, 1.0, 3.0]], [[5.0, 6.0, 8.0], [1.0, 5.0, 8.0], 
[5.0, 8.0, 5.0]]], [[[3.0, 5.0, 7.0], [8.0, 2.0, 4.0], [3.0, 1.0, 5.0]], [[3.0, 4.0, 6.0], [7.0, 5.0, 6.0], [2.0, 2.0, 2.0]], 
[[5.0, 3.0, 2.0], [6.0, 3.0, 2.0], [1.0, 4.0, 2.0]]], [[[5.0, 7.0, 4.0], [1.0, 5.0, 1.0], [5.0, 6.0, 8.0]], [[3.0, 3.0, 4.0], 
[3.0, 6.0, 1.0], [3.0, 6.0, 1.0]], [[3.0, 1.0, 1.0], [3.0, 2.0, 4.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PUCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 1.0], [3.0, 8.0, 3.0], [6.0, 3.0, 3.0]], [[8.0, 6.0, 2.0], [1.0, 8.0, 1.0], [3.0, 
4.0, 7.0]], [[6.0, 8.0, 3.0], [4.0, 2.0, 7.0], [3.0, 7.0, 6.0]]], [[[8.0, 3.0, 6.0], [2.0, 5.0, 8.0], [7.0, 4.0, 5.0]], [[6.0, 
6.0, 8.0], [2.0, 6.0, 2.0], [7.0, 4.0, 6.0]], [[6.0, 6.0, 3.0], [6.0, 2.0, 2.0], [2.0, 8.0, 4.0]]], [[[4.0, 8.0, 2.0], [2.0, 
1.0, 3.0], [4.0, 1.0, 8.0]], [[3.0, 2.0, 2.0], [7.0, 2.0, 1.0], [6.0, 5.0, 8.0]], [[4.0, 4.0, 8.0], [5.0, 6.0, 5.0], [4.0, 7.0, 
1.0]]], [[[6.0, 4.0, 6.0], [5.0, 1.0, 7.0], [5.0, 6.0, 7.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 1.0], [7.0, 8.0, 7.0]], [[4.0, 5.0, 
2.0], [3.0, 6.0, 1.0], [7.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 4.0], [7.0, 6.0, 8.0], [4.0, 3.0, 4.0]], 
[[2.0, 4.0, 7.0], [4.0, 5.0, 6.0], [6.0, 2.0, 4.0]], [[8.0, 1.0, 7.0], [7.0, 8.0, 7.0], [7.0, 5.0, 2.0]]], [[[4.0, 1.0, 7.0], 
[5.0, 1.0, 7.0], [6.0, 1.0, 6.0]], [[6.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 5.0, 4.0]], [[8.0, 2.0, 7.0], [5.0, 8.0, 4.0], 
[3.0, 5.0, 4.0]]], [[[4.0, 4.0, 8.0], [7.0, 3.0, 1.0], [7.0, 2.0, 4.0]], [[3.0, 3.0, 5.0], [2.0, 5.0, 8.0], [7.0, 6.0, 6.0]], 
[[2.0, 6.0, 4.0], [4.0, 6.0, 8.0], [5.0, 1.0, 1.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 2.0], [7.0, 3.0, 5.0]], [[2.0, 8.0, 8.0], 
[3.0, 3.0, 4.0], [5.0, 3.0, 8.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 6.0], [7.0, 2.0, 1.0]]]])
      val=numpy.array([[[2.0, 1.0], [3.0, 4.0], [2.0, 6.0]], [[8.0, 7.0], [4.0, 1.0], [7.0, 6.0]]])
      arg[2,:2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 1.0], [3.0, 8.0, 3.0], [6.0, 3.0, 3.0]], [[8.0, 6.0, 2.0], [1.0, 8.0, 1.0], [3.0, 
4.0, 7.0]], [[6.0, 8.0, 3.0], [4.0, 2.0, 7.0], [3.0, 7.0, 6.0]]], [[[8.0, 3.0, 6.0], [2.0, 5.0, 8.0], [7.0, 4.0, 5.0]], [[6.0, 
6.0, 8.0], [2.0, 6.0, 2.0], [7.0, 4.0, 6.0]], [[6.0, 6.0, 3.0], [6.0, 2.0, 2.0], [2.0, 8.0, 4.0]]], [[[2.0, 1.0, 2.0], [3.0, 
4.0, 3.0], [2.0, 6.0, 8.0]], [[8.0, 7.0, 2.0], [4.0, 1.0, 1.0], [7.0, 6.0, 8.0]], [[4.0, 4.0, 8.0], [5.0, 6.0, 5.0], [4.0, 7.0, 
1.0]]], [[[6.0, 4.0, 6.0], [5.0, 1.0, 7.0], [5.0, 6.0, 7.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 1.0], [7.0, 8.0, 7.0]], [[4.0, 5.0, 
2.0], [3.0, 6.0, 1.0], [7.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 4.0], [7.0, 6.0, 8.0], [4.0, 3.0, 4.0]], 
[[2.0, 4.0, 7.0], [4.0, 5.0, 6.0], [6.0, 2.0, 4.0]], [[8.0, 1.0, 7.0], [7.0, 8.0, 7.0], [7.0, 5.0, 2.0]]], [[[4.0, 1.0, 7.0], 
[5.0, 1.0, 7.0], [6.0, 1.0, 6.0]], [[6.0, 3.0, 8.0], [7.0, 7.0, 6.0], [1.0, 5.0, 4.0]], [[8.0, 2.0, 7.0], [5.0, 8.0, 4.0], 
[3.0, 5.0, 4.0]]], [[[2.0, 1.0, 8.0], [3.0, 4.0, 1.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 5.0], [4.0, 1.0, 8.0], [7.0, 6.0, 6.0]], 
[[2.0, 6.0, 4.0], [4.0, 6.0, 8.0], [5.0, 1.0, 1.0]]], [[[1.0, 6.0, 5.0], [2.0, 7.0, 2.0], [7.0, 3.0, 5.0]], [[2.0, 8.0, 8.0], 
[3.0, 3.0, 4.0], [5.0, 3.0, 8.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 6.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PUCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 2.0], [1.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[3.0, 4.0, 2.0], [2.0, 8.0, 7.0], [6.0, 
5.0, 2.0]], [[6.0, 6.0, 7.0], [6.0, 2.0, 7.0], [5.0, 6.0, 6.0]]], [[[4.0, 1.0, 7.0], [5.0, 2.0, 5.0], [3.0, 7.0, 7.0]], [[3.0, 
7.0, 1.0], [6.0, 8.0, 8.0], [6.0, 1.0, 6.0]], [[7.0, 6.0, 8.0], [7.0, 8.0, 4.0], [4.0, 4.0, 5.0]]], [[[3.0, 4.0, 5.0], [2.0, 
7.0, 8.0], [7.0, 7.0, 1.0]], [[2.0, 5.0, 5.0], [1.0, 6.0, 3.0], [7.0, 2.0, 1.0]], [[4.0, 4.0, 4.0], [6.0, 3.0, 8.0], [3.0, 4.0, 
8.0]]], [[[1.0, 4.0, 6.0], [5.0, 7.0, 4.0], [8.0, 5.0, 5.0]], [[5.0, 7.0, 1.0], [6.0, 6.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 4.0, 
2.0], [7.0, 3.0, 2.0], [8.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 4.0], [4.0, 8.0, 4.0], [2.0, 3.0, 2.0]], 
[[6.0, 8.0, 4.0], [2.0, 5.0, 2.0], [8.0, 2.0, 5.0]], [[2.0, 2.0, 1.0], [4.0, 4.0, 6.0], [4.0, 3.0, 5.0]]], [[[1.0, 1.0, 5.0], 
[6.0, 6.0, 2.0], [2.0, 1.0, 7.0]], [[1.0, 4.0, 3.0], [6.0, 6.0, 6.0], [4.0, 1.0, 7.0]], [[5.0, 8.0, 8.0], [3.0, 2.0, 7.0], 
[8.0, 2.0, 6.0]]], [[[1.0, 2.0, 2.0], [6.0, 7.0, 3.0], [2.0, 5.0, 8.0]], [[3.0, 8.0, 8.0], [1.0, 6.0, 6.0], [8.0, 7.0, 1.0]], 
[[1.0, 7.0, 5.0], [2.0, 1.0, 3.0], [5.0, 1.0, 2.0]]], [[[1.0, 3.0, 4.0], [6.0, 8.0, 5.0], [7.0, 4.0, 7.0]], [[6.0, 6.0, 8.0], 
[1.0, 4.0, 7.0], [5.0, 1.0, 6.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 7.0], [6.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[3.0, 8.0], [8.0, 6.0], [1.0, 3.0]], [[2.0, 7.0], [2.0, 5.0], [6.0, 7.0]]]),self.functionspace)
      arg[2,:2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 2.0], [1.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[3.0, 4.0, 2.0], [2.0, 8.0, 7.0], [6.0, 
5.0, 2.0]], [[6.0, 6.0, 7.0], [6.0, 2.0, 7.0], [5.0, 6.0, 6.0]]], [[[4.0, 1.0, 7.0], [5.0, 2.0, 5.0], [3.0, 7.0, 7.0]], [[3.0, 
7.0, 1.0], [6.0, 8.0, 8.0], [6.0, 1.0, 6.0]], [[7.0, 6.0, 8.0], [7.0, 8.0, 4.0], [4.0, 4.0, 5.0]]], [[[3.0, 8.0, 5.0], [8.0, 
6.0, 8.0], [1.0, 3.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 5.0, 3.0], [6.0, 7.0, 1.0]], [[4.0, 4.0, 4.0], [6.0, 3.0, 8.0], [3.0, 4.0, 
8.0]]], [[[1.0, 4.0, 6.0], [5.0, 7.0, 4.0], [8.0, 5.0, 5.0]], [[5.0, 7.0, 1.0], [6.0, 6.0, 4.0], [2.0, 1.0, 3.0]], [[1.0, 4.0, 
2.0], [7.0, 3.0, 2.0], [8.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 4.0], [4.0, 8.0, 4.0], [2.0, 3.0, 2.0]], 
[[6.0, 8.0, 4.0], [2.0, 5.0, 2.0], [8.0, 2.0, 5.0]], [[2.0, 2.0, 1.0], [4.0, 4.0, 6.0], [4.0, 3.0, 5.0]]], [[[1.0, 1.0, 5.0], 
[6.0, 6.0, 2.0], [2.0, 1.0, 7.0]], [[1.0, 4.0, 3.0], [6.0, 6.0, 6.0], [4.0, 1.0, 7.0]], [[5.0, 8.0, 8.0], [3.0, 2.0, 7.0], 
[8.0, 2.0, 6.0]]], [[[3.0, 8.0, 2.0], [8.0, 6.0, 3.0], [1.0, 3.0, 8.0]], [[2.0, 7.0, 8.0], [2.0, 5.0, 6.0], [6.0, 7.0, 1.0]], 
[[1.0, 7.0, 5.0], [2.0, 1.0, 3.0], [5.0, 1.0, 2.0]]], [[[1.0, 3.0, 4.0], [6.0, 8.0, 5.0], [7.0, 4.0, 7.0]], [[6.0, 6.0, 8.0], 
[1.0, 4.0, 7.0], [5.0, 1.0, 6.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 7.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PUCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 7.0], [2.0, 2.0, 2.0], [8.0, 1.0, 4.0]], [[7.0, 5.0, 8.0], [8.0, 3.0, 7.0], [7.0, 
6.0, 7.0]], [[1.0, 4.0, 5.0], [4.0, 8.0, 8.0], [5.0, 3.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 1.0, 8.0], [5.0, 3.0, 2.0]], [[4.0, 
8.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 2.0], [1.0, 7.0, 3.0]]], [[[4.0, 1.0, 7.0], [6.0, 
8.0, 3.0], [5.0, 8.0, 6.0]], [[3.0, 3.0, 5.0], [2.0, 7.0, 3.0], [3.0, 2.0, 6.0]], [[6.0, 6.0, 5.0], [6.0, 3.0, 6.0], [3.0, 6.0, 
5.0]]], [[[2.0, 2.0, 7.0], [8.0, 4.0, 8.0], [4.0, 1.0, 5.0]], [[7.0, 3.0, 1.0], [1.0, 8.0, 1.0], [5.0, 5.0, 3.0]], [[5.0, 1.0, 
4.0], [2.0, 1.0, 3.0], [8.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 3.0], [3.0, 4.0, 3.0], [3.0, 1.0, 7.0]], 
[[7.0, 3.0, 6.0], [4.0, 2.0, 2.0], [4.0, 6.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 1.0, 2.0], [8.0, 6.0, 2.0]]], [[[3.0, 1.0, 2.0], 
[3.0, 5.0, 8.0], [7.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [6.0, 6.0, 2.0], [6.0, 2.0, 7.0]], [[6.0, 3.0, 2.0], [1.0, 2.0, 1.0], 
[3.0, 4.0, 8.0]]], [[[2.0, 3.0, 5.0], [6.0, 1.0, 3.0], [4.0, 4.0, 4.0]], [[2.0, 7.0, 8.0], [2.0, 4.0, 4.0], [7.0, 7.0, 7.0]], 
[[2.0, 5.0, 7.0], [8.0, 3.0, 6.0], [8.0, 3.0, 4.0]]], [[[8.0, 7.0, 2.0], [5.0, 3.0, 6.0], [2.0, 8.0, 8.0]], [[1.0, 4.0, 7.0], 
[7.0, 3.0, 2.0], [2.0, 4.0, 8.0]], [[5.0, 2.0, 2.0], [5.0, 8.0, 4.0], [1.0, 8.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 8.0], [7.0, 2.0], [7.0, 4.0]], [[2.0, 1.0], [7.0, 6.0], [8.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[5.0, 4.0], [3.0, 7.0], [1.0, 4.0]], [[3.0, 1.0], [2.0, 6.0], [3.0, 5.0]]])
      arg[2,:2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 7.0], [2.0, 2.0, 2.0], [8.0, 1.0, 4.0]], [[7.0, 5.0, 8.0], [8.0, 3.0, 7.0], [7.0, 
6.0, 7.0]], [[1.0, 4.0, 5.0], [4.0, 8.0, 8.0], [5.0, 3.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 1.0, 8.0], [5.0, 3.0, 2.0]], [[4.0, 
8.0, 8.0], [6.0, 5.0, 6.0], [7.0, 8.0, 6.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 2.0], [1.0, 7.0, 3.0]]], [[[4.0, 8.0, 7.0], [7.0, 
2.0, 3.0], [7.0, 4.0, 6.0]], [[2.0, 1.0, 5.0], [7.0, 6.0, 3.0], [8.0, 6.0, 6.0]], [[6.0, 6.0, 5.0], [6.0, 3.0, 6.0], [3.0, 6.0, 
5.0]]], [[[2.0, 2.0, 7.0], [8.0, 4.0, 8.0], [4.0, 1.0, 5.0]], [[7.0, 3.0, 1.0], [1.0, 8.0, 1.0], [5.0, 5.0, 3.0]], [[5.0, 1.0, 
4.0], [2.0, 1.0, 3.0], [8.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 3.0], [3.0, 4.0, 3.0], [3.0, 1.0, 7.0]], 
[[7.0, 3.0, 6.0], [4.0, 2.0, 2.0], [4.0, 6.0, 7.0]], [[5.0, 7.0, 5.0], [4.0, 1.0, 2.0], [8.0, 6.0, 2.0]]], [[[3.0, 1.0, 2.0], 
[3.0, 5.0, 8.0], [7.0, 6.0, 8.0]], [[7.0, 4.0, 5.0], [6.0, 6.0, 2.0], [6.0, 2.0, 7.0]], [[6.0, 3.0, 2.0], [1.0, 2.0, 1.0], 
[3.0, 4.0, 8.0]]], [[[5.0, 4.0, 5.0], [3.0, 7.0, 3.0], [1.0, 4.0, 4.0]], [[3.0, 1.0, 8.0], [2.0, 6.0, 4.0], [3.0, 5.0, 7.0]], 
[[2.0, 5.0, 7.0], [8.0, 3.0, 6.0], [8.0, 3.0, 4.0]]], [[[8.0, 7.0, 2.0], [5.0, 3.0, 6.0], [2.0, 8.0, 8.0]], [[1.0, 4.0, 7.0], 
[7.0, 3.0, 2.0], [2.0, 4.0, 8.0]], [[5.0, 2.0, 2.0], [5.0, 8.0, 4.0], [1.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 8.0], [7.0, 5.0, 6.0], [2.0, 8.0, 3.0]], [[2.0, 8.0, 2.0], [8.0, 8.0, 2.0], [1.0, 
7.0, 6.0]], [[1.0, 6.0, 4.0], [1.0, 1.0, 7.0], [8.0, 2.0, 7.0]]], [[[1.0, 6.0, 6.0], [5.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[3.0, 
7.0, 8.0], [5.0, 4.0, 2.0], [6.0, 4.0, 3.0]], [[5.0, 4.0, 1.0], [4.0, 6.0, 8.0], [6.0, 1.0, 7.0]]], [[[1.0, 2.0, 2.0], [2.0, 
7.0, 5.0], [1.0, 5.0, 5.0]], [[5.0, 1.0, 4.0], [3.0, 1.0, 6.0], [6.0, 4.0, 6.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 5.0], [6.0, 2.0, 
1.0]]], [[[5.0, 2.0, 2.0], [7.0, 1.0, 3.0], [5.0, 6.0, 4.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 5.0, 
5.0], [6.0, 6.0, 3.0], [4.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 8.0], [1.0, 1.0, 4.0], [2.0, 1.0, 3.0]], 
[[3.0, 7.0, 4.0], [8.0, 1.0, 7.0], [5.0, 5.0, 1.0]], [[8.0, 6.0, 8.0], [6.0, 3.0, 7.0], [8.0, 6.0, 6.0]]], [[[7.0, 6.0, 3.0], 
[4.0, 1.0, 1.0], [1.0, 5.0, 3.0]], [[2.0, 1.0, 6.0], [1.0, 5.0, 7.0], [1.0, 4.0, 6.0]], [[7.0, 6.0, 6.0], [7.0, 4.0, 4.0], 
[8.0, 1.0, 4.0]]], [[[5.0, 3.0, 7.0], [7.0, 3.0, 3.0], [8.0, 2.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 3.0], [1.0, 1.0, 7.0]], 
[[5.0, 1.0, 1.0], [4.0, 7.0, 5.0], [3.0, 8.0, 7.0]]], [[[1.0, 7.0, 5.0], [5.0, 8.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 7.0, 5.0], 
[1.0, 4.0, 5.0], [7.0, 7.0, 6.0]], [[6.0, 3.0, 4.0], [8.0, 2.0, 7.0], [7.0, 2.0, 4.0]]]])
      val=numpy.array([[[[3.0]], [[8.0]], [[5.0]]], [[[3.0]], [[1.0]], [[5.0]]], [[[7.0]], [[5.0]], [[1.0]]]])
      arg[:3,:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 8.0], [7.0, 5.0, 6.0], [2.0, 3.0, 3.0]], [[2.0, 8.0, 2.0], [8.0, 8.0, 2.0], [1.0, 
8.0, 6.0]], [[1.0, 6.0, 4.0], [1.0, 1.0, 7.0], [8.0, 5.0, 7.0]]], [[[1.0, 6.0, 6.0], [5.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[3.0, 
7.0, 8.0], [5.0, 4.0, 2.0], [6.0, 1.0, 3.0]], [[5.0, 4.0, 1.0], [4.0, 6.0, 8.0], [6.0, 5.0, 7.0]]], [[[1.0, 2.0, 2.0], [2.0, 
7.0, 5.0], [1.0, 7.0, 5.0]], [[5.0, 1.0, 4.0], [3.0, 1.0, 6.0], [6.0, 5.0, 6.0]], [[4.0, 6.0, 4.0], [3.0, 1.0, 5.0], [6.0, 1.0, 
1.0]]], [[[5.0, 2.0, 2.0], [7.0, 1.0, 3.0], [5.0, 6.0, 4.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 8.0], [1.0, 4.0, 2.0]], [[5.0, 5.0, 
5.0], [6.0, 6.0, 3.0], [4.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 8.0], [1.0, 1.0, 4.0], [2.0, 3.0, 3.0]], 
[[3.0, 7.0, 4.0], [8.0, 1.0, 7.0], [5.0, 8.0, 1.0]], [[8.0, 6.0, 8.0], [6.0, 3.0, 7.0], [8.0, 5.0, 6.0]]], [[[7.0, 6.0, 3.0], 
[4.0, 1.0, 1.0], [1.0, 3.0, 3.0]], [[2.0, 1.0, 6.0], [1.0, 5.0, 7.0], [1.0, 1.0, 6.0]], [[7.0, 6.0, 6.0], [7.0, 4.0, 4.0], 
[8.0, 5.0, 4.0]]], [[[5.0, 3.0, 7.0], [7.0, 3.0, 3.0], [8.0, 7.0, 1.0]], [[5.0, 7.0, 5.0], [6.0, 5.0, 3.0], [1.0, 5.0, 7.0]], 
[[5.0, 1.0, 1.0], [4.0, 7.0, 5.0], [3.0, 1.0, 7.0]]], [[[1.0, 7.0, 5.0], [5.0, 8.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 7.0, 5.0], 
[1.0, 4.0, 5.0], [7.0, 7.0, 6.0]], [[6.0, 3.0, 4.0], [8.0, 2.0, 7.0], [7.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 5.0], [2.0, 2.0, 3.0], [1.0, 1.0, 6.0]], [[5.0, 2.0, 8.0], [8.0, 7.0, 3.0], [5.0, 
6.0, 1.0]], [[4.0, 3.0, 8.0], [3.0, 6.0, 6.0], [1.0, 3.0, 1.0]]], [[[2.0, 5.0, 6.0], [5.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[4.0, 
6.0, 5.0], [8.0, 2.0, 4.0], [7.0, 6.0, 6.0]], [[6.0, 5.0, 2.0], [3.0, 1.0, 2.0], [6.0, 5.0, 6.0]]], [[[6.0, 6.0, 1.0], [6.0, 
4.0, 2.0], [6.0, 6.0, 7.0]], [[5.0, 1.0, 5.0], [8.0, 4.0, 4.0], [6.0, 6.0, 7.0]], [[2.0, 6.0, 6.0], [1.0, 5.0, 3.0], [8.0, 3.0, 
3.0]]], [[[6.0, 3.0, 1.0], [8.0, 5.0, 4.0], [5.0, 1.0, 3.0]], [[3.0, 3.0, 6.0], [4.0, 4.0, 1.0], [1.0, 1.0, 2.0]], [[6.0, 5.0, 
1.0], [2.0, 1.0, 3.0], [8.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 7.0], [7.0, 6.0, 1.0], [2.0, 7.0, 1.0]], 
[[3.0, 3.0, 7.0], [6.0, 3.0, 3.0], [7.0, 7.0, 7.0]], [[5.0, 3.0, 5.0], [5.0, 8.0, 1.0], [4.0, 7.0, 1.0]]], [[[4.0, 3.0, 8.0], 
[5.0, 4.0, 1.0], [8.0, 1.0, 3.0]], [[8.0, 2.0, 8.0], [7.0, 1.0, 2.0], [2.0, 3.0, 7.0]], [[5.0, 6.0, 6.0], [1.0, 5.0, 2.0], 
[8.0, 6.0, 1.0]]], [[[8.0, 3.0, 1.0], [6.0, 1.0, 1.0], [4.0, 1.0, 1.0]], [[5.0, 5.0, 2.0], [8.0, 8.0, 2.0], [7.0, 6.0, 8.0]], 
[[3.0, 1.0, 3.0], [8.0, 8.0, 2.0], [5.0, 3.0, 7.0]]], [[[3.0, 7.0, 5.0], [8.0, 6.0, 3.0], [5.0, 8.0, 4.0]], [[4.0, 6.0, 2.0], 
[1.0, 1.0, 5.0], [1.0, 7.0, 6.0]], [[3.0, 1.0, 6.0], [4.0, 6.0, 7.0], [1.0, 1.0, 7.0]]]])
      val=Data(numpy.array([[[[7.0]], [[2.0]], [[8.0]]], [[[2.0]], [[3.0]], [[5.0]]], [[[4.0]], [[4.0]], 
[[4.0]]]]),self.functionspace)
      arg[:3,:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 5.0], [2.0, 2.0, 3.0], [1.0, 7.0, 6.0]], [[5.0, 2.0, 8.0], [8.0, 7.0, 3.0], [5.0, 
2.0, 1.0]], [[4.0, 3.0, 8.0], [3.0, 6.0, 6.0], [1.0, 8.0, 1.0]]], [[[2.0, 5.0, 6.0], [5.0, 2.0, 2.0], [6.0, 2.0, 8.0]], [[4.0, 
6.0, 5.0], [8.0, 2.0, 4.0], [7.0, 3.0, 6.0]], [[6.0, 5.0, 2.0], [3.0, 1.0, 2.0], [6.0, 5.0, 6.0]]], [[[6.0, 6.0, 1.0], [6.0, 
4.0, 2.0], [6.0, 4.0, 7.0]], [[5.0, 1.0, 5.0], [8.0, 4.0, 4.0], [6.0, 4.0, 7.0]], [[2.0, 6.0, 6.0], [1.0, 5.0, 3.0], [8.0, 4.0, 
3.0]]], [[[6.0, 3.0, 1.0], [8.0, 5.0, 4.0], [5.0, 1.0, 3.0]], [[3.0, 3.0, 6.0], [4.0, 4.0, 1.0], [1.0, 1.0, 2.0]], [[6.0, 5.0, 
1.0], [2.0, 1.0, 3.0], [8.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [7.0, 6.0, 1.0], [2.0, 7.0, 1.0]], 
[[3.0, 3.0, 7.0], [6.0, 3.0, 3.0], [7.0, 2.0, 7.0]], [[5.0, 3.0, 5.0], [5.0, 8.0, 1.0], [4.0, 8.0, 1.0]]], [[[4.0, 3.0, 8.0], 
[5.0, 4.0, 1.0], [8.0, 2.0, 3.0]], [[8.0, 2.0, 8.0], [7.0, 1.0, 2.0], [2.0, 3.0, 7.0]], [[5.0, 6.0, 6.0], [1.0, 5.0, 2.0], 
[8.0, 5.0, 1.0]]], [[[8.0, 3.0, 1.0], [6.0, 1.0, 1.0], [4.0, 4.0, 1.0]], [[5.0, 5.0, 2.0], [8.0, 8.0, 2.0], [7.0, 4.0, 8.0]], 
[[3.0, 1.0, 3.0], [8.0, 8.0, 2.0], [5.0, 4.0, 7.0]]], [[[3.0, 7.0, 5.0], [8.0, 6.0, 3.0], [5.0, 8.0, 4.0]], [[4.0, 6.0, 2.0], 
[1.0, 1.0, 5.0], [1.0, 7.0, 6.0]], [[3.0, 1.0, 6.0], [4.0, 6.0, 7.0], [1.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 3.0], [7.0, 4.0, 6.0], [7.0, 5.0, 1.0]], [[8.0, 3.0, 4.0], [5.0, 4.0, 2.0], [6.0, 
7.0, 6.0]], [[7.0, 3.0, 6.0], [5.0, 2.0, 8.0], [1.0, 1.0, 8.0]]], [[[7.0, 5.0, 7.0], [1.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 
3.0, 2.0], [7.0, 3.0, 5.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 8.0, 1.0], [4.0, 6.0, 4.0]]], [[[5.0, 6.0, 2.0], [6.0, 
4.0, 8.0], [7.0, 5.0, 7.0]], [[6.0, 6.0, 8.0], [6.0, 7.0, 8.0], [2.0, 7.0, 3.0]], [[3.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 4.0, 
1.0]]], [[[8.0, 5.0, 6.0], [5.0, 6.0, 3.0], [6.0, 8.0, 5.0]], [[3.0, 5.0, 4.0], [7.0, 1.0, 5.0], [6.0, 1.0, 7.0]], [[1.0, 7.0, 
7.0], [2.0, 8.0, 7.0], [2.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 8.0], [6.0, 8.0, 2.0], [6.0, 4.0, 5.0]], 
[[8.0, 4.0, 1.0], [1.0, 5.0, 2.0], [3.0, 2.0, 2.0]], [[2.0, 7.0, 4.0], [1.0, 3.0, 4.0], [1.0, 4.0, 5.0]]], [[[3.0, 8.0, 4.0], 
[1.0, 5.0, 8.0], [2.0, 2.0, 3.0]], [[3.0, 3.0, 6.0], [3.0, 3.0, 2.0], [4.0, 8.0, 4.0]], [[3.0, 7.0, 7.0], [3.0, 7.0, 2.0], 
[2.0, 1.0, 1.0]]], [[[1.0, 4.0, 6.0], [7.0, 6.0, 2.0], [6.0, 1.0, 7.0]], [[5.0, 7.0, 2.0], [7.0, 6.0, 6.0], [4.0, 3.0, 7.0]], 
[[1.0, 8.0, 2.0], [4.0, 6.0, 1.0], [4.0, 5.0, 1.0]]], [[[6.0, 8.0, 6.0], [5.0, 8.0, 2.0], [4.0, 5.0, 2.0]], [[5.0, 3.0, 5.0], 
[6.0, 7.0, 6.0], [5.0, 4.0, 4.0]], [[4.0, 7.0, 6.0], [3.0, 3.0, 1.0], [7.0, 7.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0]], [[2.0]], [[6.0]]], [[[3.0]], [[8.0]], [[2.0]]], [[[3.0]], [[6.0]], 
[[8.0]]]])+(1.-msk_val)*numpy.array([[[[4.0]], [[3.0]], [[1.0]]], [[[8.0]], [[8.0]], [[4.0]]], [[[3.0]], [[8.0]], [[2.0]]]])
      arg[:3,:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 3.0], [7.0, 4.0, 6.0], [7.0, 4.0, 1.0]], [[8.0, 3.0, 4.0], [5.0, 4.0, 2.0], [6.0, 
2.0, 6.0]], [[7.0, 3.0, 6.0], [5.0, 2.0, 8.0], [1.0, 6.0, 8.0]]], [[[7.0, 5.0, 7.0], [1.0, 8.0, 8.0], [2.0, 3.0, 3.0]], [[5.0, 
3.0, 2.0], [7.0, 3.0, 5.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 5.0], [1.0, 8.0, 1.0], [4.0, 2.0, 4.0]]], [[[5.0, 6.0, 2.0], [6.0, 
4.0, 8.0], [7.0, 3.0, 7.0]], [[6.0, 6.0, 8.0], [6.0, 7.0, 8.0], [2.0, 6.0, 3.0]], [[3.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 8.0, 
1.0]]], [[[8.0, 5.0, 6.0], [5.0, 6.0, 3.0], [6.0, 8.0, 5.0]], [[3.0, 5.0, 4.0], [7.0, 1.0, 5.0], [6.0, 1.0, 7.0]], [[1.0, 7.0, 
7.0], [2.0, 8.0, 7.0], [2.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 8.0], [6.0, 8.0, 2.0], [6.0, 4.0, 5.0]], 
[[8.0, 4.0, 1.0], [1.0, 5.0, 2.0], [3.0, 3.0, 2.0]], [[2.0, 7.0, 4.0], [1.0, 3.0, 4.0], [1.0, 1.0, 5.0]]], [[[3.0, 8.0, 4.0], 
[1.0, 5.0, 8.0], [2.0, 8.0, 3.0]], [[3.0, 3.0, 6.0], [3.0, 3.0, 2.0], [4.0, 8.0, 4.0]], [[3.0, 7.0, 7.0], [3.0, 7.0, 2.0], 
[2.0, 4.0, 1.0]]], [[[1.0, 4.0, 6.0], [7.0, 6.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 7.0, 2.0], [7.0, 6.0, 6.0], [4.0, 8.0, 7.0]], 
[[1.0, 8.0, 2.0], [4.0, 6.0, 1.0], [4.0, 2.0, 1.0]]], [[[6.0, 8.0, 6.0], [5.0, 8.0, 2.0], [4.0, 5.0, 2.0]], [[5.0, 3.0, 5.0], 
[6.0, 7.0, 6.0], [5.0, 4.0, 4.0]], [[4.0, 7.0, 6.0], [3.0, 3.0, 1.0], [7.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 1.0], [3.0, 1.0, 3.0], [6.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], [1.0, 2.0, 1.0], [3.0, 
1.0, 6.0]], [[4.0, 6.0, 2.0], [4.0, 7.0, 7.0], [4.0, 7.0, 1.0]]], [[[2.0, 7.0, 1.0], [2.0, 3.0, 2.0], [3.0, 6.0, 5.0]], [[8.0, 
1.0, 6.0], [6.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 8.0], [1.0, 5.0, 3.0]]], [[[8.0, 4.0, 5.0], [2.0, 
5.0, 7.0], [8.0, 2.0, 6.0]], [[3.0, 5.0, 4.0], [3.0, 4.0, 7.0], [8.0, 8.0, 2.0]], [[7.0, 8.0, 7.0], [1.0, 6.0, 6.0], [3.0, 3.0, 
8.0]]], [[[4.0, 4.0, 7.0], [6.0, 3.0, 1.0], [8.0, 6.0, 5.0]], [[4.0, 5.0, 3.0], [8.0, 3.0, 7.0], [5.0, 3.0, 7.0]], [[6.0, 1.0, 
5.0], [1.0, 4.0, 1.0], [2.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 2.0], [5.0, 3.0, 4.0], [7.0, 8.0, 1.0]], 
[[1.0, 4.0, 7.0], [8.0, 7.0, 4.0], [1.0, 8.0, 8.0]], [[8.0, 3.0, 8.0], [7.0, 6.0, 7.0], [4.0, 8.0, 3.0]]], [[[4.0, 6.0, 7.0], 
[7.0, 2.0, 3.0], [5.0, 2.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 8.0, 3.0], [6.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [6.0, 2.0, 5.0], 
[4.0, 6.0, 4.0]]], [[[1.0, 7.0, 7.0], [6.0, 5.0, 5.0], [3.0, 4.0, 4.0]], [[8.0, 8.0, 8.0], [1.0, 5.0, 4.0], [6.0, 6.0, 6.0]], 
[[3.0, 3.0, 8.0], [6.0, 5.0, 5.0], [1.0, 7.0, 1.0]]], [[[3.0, 3.0, 6.0], [6.0, 2.0, 4.0], [6.0, 1.0, 5.0]], [[4.0, 2.0, 3.0], 
[3.0, 7.0, 6.0], [6.0, 5.0, 8.0]], [[5.0, 4.0, 4.0], [5.0, 3.0, 3.0], [1.0, 3.0, 3.0]]]])
      val=numpy.array([[[5.0, 2.0]], [[2.0, 6.0]], [[2.0, 7.0]]])
      arg[2,:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 1.0], [3.0, 1.0, 3.0], [6.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], [1.0, 2.0, 1.0], [3.0, 
1.0, 6.0]], [[4.0, 6.0, 2.0], [4.0, 7.0, 7.0], [4.0, 7.0, 1.0]]], [[[2.0, 7.0, 1.0], [2.0, 3.0, 2.0], [3.0, 6.0, 5.0]], [[8.0, 
1.0, 6.0], [6.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 8.0], [1.0, 5.0, 3.0]]], [[[8.0, 4.0, 5.0], [2.0, 
5.0, 7.0], [5.0, 2.0, 6.0]], [[3.0, 5.0, 4.0], [3.0, 4.0, 7.0], [2.0, 6.0, 2.0]], [[7.0, 8.0, 7.0], [1.0, 6.0, 6.0], [2.0, 7.0, 
8.0]]], [[[4.0, 4.0, 7.0], [6.0, 3.0, 1.0], [8.0, 6.0, 5.0]], [[4.0, 5.0, 3.0], [8.0, 3.0, 7.0], [5.0, 3.0, 7.0]], [[6.0, 1.0, 
5.0], [1.0, 4.0, 1.0], [2.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 2.0], [5.0, 3.0, 4.0], [7.0, 8.0, 1.0]], 
[[1.0, 4.0, 7.0], [8.0, 7.0, 4.0], [1.0, 8.0, 8.0]], [[8.0, 3.0, 8.0], [7.0, 6.0, 7.0], [4.0, 8.0, 3.0]]], [[[4.0, 6.0, 7.0], 
[7.0, 2.0, 3.0], [5.0, 2.0, 4.0]], [[7.0, 2.0, 8.0], [6.0, 8.0, 3.0], [6.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [6.0, 2.0, 5.0], 
[4.0, 6.0, 4.0]]], [[[1.0, 7.0, 7.0], [6.0, 5.0, 5.0], [5.0, 2.0, 4.0]], [[8.0, 8.0, 8.0], [1.0, 5.0, 4.0], [2.0, 6.0, 6.0]], 
[[3.0, 3.0, 8.0], [6.0, 5.0, 5.0], [2.0, 7.0, 1.0]]], [[[3.0, 3.0, 6.0], [6.0, 2.0, 4.0], [6.0, 1.0, 5.0]], [[4.0, 2.0, 3.0], 
[3.0, 7.0, 6.0], [6.0, 5.0, 8.0]], [[5.0, 4.0, 4.0], [5.0, 3.0, 3.0], [1.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 1.0], [4.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[2.0, 5.0, 3.0], [2.0, 3.0, 8.0], [8.0, 
6.0, 8.0]], [[2.0, 1.0, 2.0], [5.0, 6.0, 7.0], [8.0, 1.0, 5.0]]], [[[1.0, 3.0, 1.0], [8.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 
6.0, 1.0], [1.0, 8.0, 2.0], [7.0, 8.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 1.0, 4.0], [7.0, 4.0, 5.0]]], [[[5.0, 8.0, 2.0], [8.0, 
8.0, 4.0], [7.0, 7.0, 6.0]], [[5.0, 8.0, 5.0], [6.0, 6.0, 2.0], [4.0, 5.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 7.0], [8.0, 1.0, 
4.0]]], [[[6.0, 1.0, 6.0], [4.0, 5.0, 8.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 4.0], [3.0, 5.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 8.0, 
6.0], [3.0, 5.0, 8.0], [7.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 3.0, 8.0], [1.0, 4.0, 2.0]], 
[[4.0, 4.0, 4.0], [6.0, 6.0, 7.0], [3.0, 3.0, 8.0]], [[4.0, 3.0, 3.0], [2.0, 4.0, 8.0], [1.0, 5.0, 6.0]]], [[[7.0, 4.0, 8.0], 
[6.0, 8.0, 5.0], [2.0, 3.0, 7.0]], [[6.0, 1.0, 3.0], [2.0, 1.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 1.0], [1.0, 8.0, 2.0], 
[8.0, 8.0, 3.0]]], [[[3.0, 8.0, 7.0], [2.0, 7.0, 4.0], [6.0, 6.0, 8.0]], [[4.0, 3.0, 8.0], [8.0, 2.0, 6.0], [5.0, 4.0, 6.0]], 
[[1.0, 1.0, 3.0], [4.0, 2.0, 4.0], [8.0, 1.0, 7.0]]], [[[3.0, 4.0, 3.0], [4.0, 7.0, 1.0], [4.0, 1.0, 5.0]], [[2.0, 4.0, 7.0], 
[6.0, 8.0, 7.0], [8.0, 4.0, 3.0]], [[3.0, 5.0, 2.0], [2.0, 3.0, 1.0], [4.0, 3.0, 8.0]]]])
      val=Data(numpy.array([[[3.0, 2.0]], [[8.0, 1.0]], [[2.0, 3.0]]]),self.functionspace)
      arg[2,:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [4.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[2.0, 5.0, 3.0], [2.0, 3.0, 8.0], [8.0, 
6.0, 8.0]], [[2.0, 1.0, 2.0], [5.0, 6.0, 7.0], [8.0, 1.0, 5.0]]], [[[1.0, 3.0, 1.0], [8.0, 1.0, 1.0], [5.0, 4.0, 3.0]], [[8.0, 
6.0, 1.0], [1.0, 8.0, 2.0], [7.0, 8.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 1.0, 4.0], [7.0, 4.0, 5.0]]], [[[5.0, 8.0, 2.0], [8.0, 
8.0, 4.0], [3.0, 2.0, 6.0]], [[5.0, 8.0, 5.0], [6.0, 6.0, 2.0], [8.0, 1.0, 3.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 7.0], [2.0, 3.0, 
4.0]]], [[[6.0, 1.0, 6.0], [4.0, 5.0, 8.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 4.0], [3.0, 5.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 8.0, 
6.0], [3.0, 5.0, 8.0], [7.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 3.0, 8.0], [1.0, 4.0, 2.0]], 
[[4.0, 4.0, 4.0], [6.0, 6.0, 7.0], [3.0, 3.0, 8.0]], [[4.0, 3.0, 3.0], [2.0, 4.0, 8.0], [1.0, 5.0, 6.0]]], [[[7.0, 4.0, 8.0], 
[6.0, 8.0, 5.0], [2.0, 3.0, 7.0]], [[6.0, 1.0, 3.0], [2.0, 1.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 1.0], [1.0, 8.0, 2.0], 
[8.0, 8.0, 3.0]]], [[[3.0, 8.0, 7.0], [2.0, 7.0, 4.0], [3.0, 2.0, 8.0]], [[4.0, 3.0, 8.0], [8.0, 2.0, 6.0], [8.0, 1.0, 6.0]], 
[[1.0, 1.0, 3.0], [4.0, 2.0, 4.0], [2.0, 3.0, 7.0]]], [[[3.0, 4.0, 3.0], [4.0, 7.0, 1.0], [4.0, 1.0, 5.0]], [[2.0, 4.0, 7.0], 
[6.0, 8.0, 7.0], [8.0, 4.0, 3.0]], [[3.0, 5.0, 2.0], [2.0, 3.0, 1.0], [4.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 8.0], [6.0, 7.0, 7.0], [4.0, 3.0, 4.0]], [[5.0, 3.0, 4.0], [3.0, 7.0, 3.0], [2.0, 
4.0, 1.0]], [[6.0, 5.0, 6.0], [8.0, 4.0, 6.0], [2.0, 2.0, 4.0]]], [[[4.0, 4.0, 7.0], [1.0, 5.0, 4.0], [6.0, 4.0, 4.0]], [[8.0, 
8.0, 5.0], [2.0, 1.0, 1.0], [7.0, 8.0, 7.0]], [[6.0, 7.0, 8.0], [7.0, 1.0, 6.0], [4.0, 8.0, 8.0]]], [[[2.0, 5.0, 1.0], [4.0, 
5.0, 1.0], [7.0, 7.0, 3.0]], [[2.0, 4.0, 3.0], [1.0, 7.0, 5.0], [7.0, 6.0, 1.0]], [[6.0, 7.0, 7.0], [7.0, 5.0, 7.0], [7.0, 1.0, 
5.0]]], [[[1.0, 6.0, 2.0], [4.0, 1.0, 2.0], [8.0, 5.0, 3.0]], [[3.0, 1.0, 8.0], [7.0, 7.0, 5.0], [5.0, 4.0, 4.0]], [[1.0, 1.0, 
4.0], [4.0, 3.0, 4.0], [5.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 6.0], [6.0, 8.0, 6.0], [2.0, 4.0, 3.0]], 
[[6.0, 1.0, 8.0], [5.0, 3.0, 2.0], [3.0, 5.0, 1.0]], [[8.0, 8.0, 2.0], [3.0, 8.0, 3.0], [5.0, 4.0, 7.0]]], [[[6.0, 5.0, 8.0], 
[4.0, 7.0, 1.0], [1.0, 2.0, 3.0]], [[8.0, 6.0, 5.0], [7.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[2.0, 2.0, 7.0], [7.0, 4.0, 5.0], 
[7.0, 3.0, 7.0]]], [[[3.0, 4.0, 8.0], [1.0, 1.0, 1.0], [7.0, 6.0, 1.0]], [[1.0, 4.0, 4.0], [5.0, 1.0, 7.0], [8.0, 2.0, 4.0]], 
[[8.0, 6.0, 3.0], [1.0, 2.0, 2.0], [2.0, 5.0, 3.0]]], [[[5.0, 8.0, 6.0], [5.0, 4.0, 5.0], [8.0, 2.0, 6.0]], [[5.0, 1.0, 1.0], 
[2.0, 7.0, 6.0], [1.0, 6.0, 2.0]], [[2.0, 2.0, 6.0], [7.0, 3.0, 7.0], [8.0, 3.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 5.0]], [[4.0, 5.0]], [[6.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0]], [[1.0, 3.0]], 
[[6.0, 3.0]]])
      arg[2,:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 8.0], [6.0, 7.0, 7.0], [4.0, 3.0, 4.0]], [[5.0, 3.0, 4.0], [3.0, 7.0, 3.0], [2.0, 
4.0, 1.0]], [[6.0, 5.0, 6.0], [8.0, 4.0, 6.0], [2.0, 2.0, 4.0]]], [[[4.0, 4.0, 7.0], [1.0, 5.0, 4.0], [6.0, 4.0, 4.0]], [[8.0, 
8.0, 5.0], [2.0, 1.0, 1.0], [7.0, 8.0, 7.0]], [[6.0, 7.0, 8.0], [7.0, 1.0, 6.0], [4.0, 8.0, 8.0]]], [[[2.0, 5.0, 1.0], [4.0, 
5.0, 1.0], [7.0, 5.0, 3.0]], [[2.0, 4.0, 3.0], [1.0, 7.0, 5.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 7.0], [7.0, 5.0, 7.0], [6.0, 5.0, 
5.0]]], [[[1.0, 6.0, 2.0], [4.0, 1.0, 2.0], [8.0, 5.0, 3.0]], [[3.0, 1.0, 8.0], [7.0, 7.0, 5.0], [5.0, 4.0, 4.0]], [[1.0, 1.0, 
4.0], [4.0, 3.0, 4.0], [5.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 6.0], [6.0, 8.0, 6.0], [2.0, 4.0, 3.0]], 
[[6.0, 1.0, 8.0], [5.0, 3.0, 2.0], [3.0, 5.0, 1.0]], [[8.0, 8.0, 2.0], [3.0, 8.0, 3.0], [5.0, 4.0, 7.0]]], [[[6.0, 5.0, 8.0], 
[4.0, 7.0, 1.0], [1.0, 2.0, 3.0]], [[8.0, 6.0, 5.0], [7.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[2.0, 2.0, 7.0], [7.0, 4.0, 5.0], 
[7.0, 3.0, 7.0]]], [[[3.0, 4.0, 8.0], [1.0, 1.0, 1.0], [8.0, 1.0, 1.0]], [[1.0, 4.0, 4.0], [5.0, 1.0, 7.0], [1.0, 3.0, 4.0]], 
[[8.0, 6.0, 3.0], [1.0, 2.0, 2.0], [6.0, 3.0, 3.0]]], [[[5.0, 8.0, 6.0], [5.0, 4.0, 5.0], [8.0, 2.0, 6.0]], [[5.0, 1.0, 1.0], 
[2.0, 7.0, 6.0], [1.0, 6.0, 2.0]], [[2.0, 2.0, 6.0], [7.0, 3.0, 7.0], [8.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 1.0], [2.0, 2.0, 8.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [2.0, 6.0, 3.0], [5.0, 
8.0, 3.0]], [[6.0, 4.0, 6.0], [7.0, 3.0, 5.0], [4.0, 6.0, 5.0]]], [[[3.0, 8.0, 2.0], [2.0, 2.0, 6.0], [7.0, 8.0, 8.0]], [[2.0, 
1.0, 6.0], [8.0, 8.0, 5.0], [8.0, 3.0, 3.0]], [[4.0, 8.0, 5.0], [1.0, 2.0, 4.0], [7.0, 4.0, 7.0]]], [[[1.0, 4.0, 6.0], [6.0, 
2.0, 3.0], [2.0, 3.0, 1.0]], [[8.0, 7.0, 2.0], [4.0, 5.0, 4.0], [1.0, 1.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 7.0, 7.0], [6.0, 7.0, 
4.0]]], [[[5.0, 3.0, 7.0], [6.0, 8.0, 5.0], [6.0, 7.0, 3.0]], [[4.0, 1.0, 3.0], [3.0, 4.0, 8.0], [4.0, 4.0, 3.0]], [[6.0, 2.0, 
2.0], [8.0, 6.0, 4.0], [4.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 5.0], [8.0, 1.0, 2.0], [2.0, 7.0, 2.0]], 
[[3.0, 2.0, 8.0], [7.0, 7.0, 3.0], [1.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [7.0, 7.0, 7.0], [3.0, 4.0, 2.0]]], [[[6.0, 4.0, 5.0], 
[6.0, 2.0, 5.0], [5.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [5.0, 5.0, 2.0], [2.0, 6.0, 4.0]], [[6.0, 4.0, 2.0], [7.0, 3.0, 4.0], 
[5.0, 5.0, 6.0]]], [[[7.0, 4.0, 3.0], [5.0, 7.0, 2.0], [6.0, 5.0, 7.0]], [[1.0, 5.0, 7.0], [6.0, 5.0, 6.0], [5.0, 4.0, 8.0]], 
[[7.0, 6.0, 1.0], [4.0, 6.0, 4.0], [1.0, 2.0, 1.0]]], [[[3.0, 2.0, 5.0], [2.0, 5.0, 2.0], [6.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], 
[6.0, 2.0, 8.0], [2.0, 4.0, 4.0]], [[7.0, 8.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0]]]])
      val=numpy.array([[3.0], [1.0], [4.0]])
      arg[:3,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [2.0, 2.0, 8.0], [5.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [2.0, 3.0, 3.0], [5.0, 
8.0, 3.0]], [[6.0, 4.0, 6.0], [7.0, 3.0, 5.0], [4.0, 6.0, 5.0]]], [[[3.0, 8.0, 2.0], [2.0, 2.0, 6.0], [7.0, 8.0, 8.0]], [[2.0, 
1.0, 6.0], [8.0, 1.0, 5.0], [8.0, 3.0, 3.0]], [[4.0, 8.0, 5.0], [1.0, 2.0, 4.0], [7.0, 4.0, 7.0]]], [[[1.0, 4.0, 6.0], [6.0, 
2.0, 3.0], [2.0, 3.0, 1.0]], [[8.0, 7.0, 2.0], [4.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[7.0, 7.0, 3.0], [8.0, 7.0, 7.0], [6.0, 7.0, 
4.0]]], [[[5.0, 3.0, 7.0], [6.0, 8.0, 5.0], [6.0, 7.0, 3.0]], [[4.0, 1.0, 3.0], [3.0, 4.0, 8.0], [4.0, 4.0, 3.0]], [[6.0, 2.0, 
2.0], [8.0, 6.0, 4.0], [4.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 5.0], [8.0, 1.0, 2.0], [2.0, 7.0, 2.0]], 
[[3.0, 2.0, 8.0], [7.0, 3.0, 3.0], [1.0, 5.0, 5.0]], [[3.0, 2.0, 6.0], [7.0, 7.0, 7.0], [3.0, 4.0, 2.0]]], [[[6.0, 4.0, 5.0], 
[6.0, 2.0, 5.0], [5.0, 3.0, 8.0]], [[4.0, 2.0, 6.0], [5.0, 1.0, 2.0], [2.0, 6.0, 4.0]], [[6.0, 4.0, 2.0], [7.0, 3.0, 4.0], 
[5.0, 5.0, 6.0]]], [[[7.0, 4.0, 3.0], [5.0, 7.0, 2.0], [6.0, 5.0, 7.0]], [[1.0, 5.0, 7.0], [6.0, 4.0, 6.0], [5.0, 4.0, 8.0]], 
[[7.0, 6.0, 1.0], [4.0, 6.0, 4.0], [1.0, 2.0, 1.0]]], [[[3.0, 2.0, 5.0], [2.0, 5.0, 2.0], [6.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], 
[6.0, 2.0, 8.0], [2.0, 4.0, 4.0]], [[7.0, 8.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 7.0], [8.0, 2.0, 2.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 5.0], [7.0, 1.0, 8.0], [8.0, 
7.0, 4.0]], [[3.0, 3.0, 6.0], [4.0, 8.0, 7.0], [4.0, 2.0, 3.0]]], [[[8.0, 4.0, 7.0], [2.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[3.0, 
6.0, 3.0], [5.0, 1.0, 3.0], [2.0, 4.0, 2.0]], [[7.0, 6.0, 2.0], [8.0, 8.0, 8.0], [1.0, 4.0, 8.0]]], [[[7.0, 1.0, 1.0], [8.0, 
6.0, 1.0], [8.0, 4.0, 6.0]], [[3.0, 5.0, 5.0], [6.0, 5.0, 2.0], [8.0, 4.0, 6.0]], [[8.0, 2.0, 7.0], [7.0, 8.0, 7.0], [1.0, 3.0, 
2.0]]], [[[1.0, 7.0, 2.0], [3.0, 4.0, 7.0], [4.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [3.0, 4.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 6.0, 
2.0], [3.0, 8.0, 2.0], [1.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 4.0], [5.0, 4.0, 1.0], [4.0, 2.0, 5.0]], 
[[7.0, 3.0, 8.0], [3.0, 3.0, 6.0], [5.0, 7.0, 5.0]], [[3.0, 6.0, 3.0], [3.0, 1.0, 1.0], [6.0, 6.0, 3.0]]], [[[5.0, 4.0, 2.0], 
[4.0, 1.0, 7.0], [8.0, 6.0, 3.0]], [[3.0, 2.0, 4.0], [1.0, 1.0, 3.0], [7.0, 3.0, 5.0]], [[5.0, 7.0, 3.0], [3.0, 5.0, 4.0], 
[6.0, 7.0, 6.0]]], [[[3.0, 7.0, 8.0], [1.0, 4.0, 7.0], [8.0, 1.0, 1.0]], [[3.0, 5.0, 5.0], [7.0, 6.0, 4.0], [5.0, 8.0, 8.0]], 
[[8.0, 5.0, 5.0], [3.0, 1.0, 8.0], [6.0, 6.0, 4.0]]], [[[8.0, 8.0, 2.0], [8.0, 2.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], 
[3.0, 1.0, 6.0], [6.0, 8.0, 7.0]], [[3.0, 1.0, 1.0], [7.0, 5.0, 4.0], [4.0, 5.0, 7.0]]]])
      val=Data(numpy.array([[6.0], [8.0], [3.0]]),self.functionspace)
      arg[:3,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 7.0], [8.0, 2.0, 2.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 5.0], [7.0, 6.0, 8.0], [8.0, 
7.0, 4.0]], [[3.0, 3.0, 6.0], [4.0, 8.0, 7.0], [4.0, 2.0, 3.0]]], [[[8.0, 4.0, 7.0], [2.0, 5.0, 1.0], [4.0, 6.0, 5.0]], [[3.0, 
6.0, 3.0], [5.0, 8.0, 3.0], [2.0, 4.0, 2.0]], [[7.0, 6.0, 2.0], [8.0, 8.0, 8.0], [1.0, 4.0, 8.0]]], [[[7.0, 1.0, 1.0], [8.0, 
6.0, 1.0], [8.0, 4.0, 6.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 2.0], [8.0, 4.0, 6.0]], [[8.0, 2.0, 7.0], [7.0, 8.0, 7.0], [1.0, 3.0, 
2.0]]], [[[1.0, 7.0, 2.0], [3.0, 4.0, 7.0], [4.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [3.0, 4.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, 6.0, 
2.0], [3.0, 8.0, 2.0], [1.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 4.0], [5.0, 4.0, 1.0], [4.0, 2.0, 5.0]], 
[[7.0, 3.0, 8.0], [3.0, 6.0, 6.0], [5.0, 7.0, 5.0]], [[3.0, 6.0, 3.0], [3.0, 1.0, 1.0], [6.0, 6.0, 3.0]]], [[[5.0, 4.0, 2.0], 
[4.0, 1.0, 7.0], [8.0, 6.0, 3.0]], [[3.0, 2.0, 4.0], [1.0, 8.0, 3.0], [7.0, 3.0, 5.0]], [[5.0, 7.0, 3.0], [3.0, 5.0, 4.0], 
[6.0, 7.0, 6.0]]], [[[3.0, 7.0, 8.0], [1.0, 4.0, 7.0], [8.0, 1.0, 1.0]], [[3.0, 5.0, 5.0], [7.0, 3.0, 4.0], [5.0, 8.0, 8.0]], 
[[8.0, 5.0, 5.0], [3.0, 1.0, 8.0], [6.0, 6.0, 4.0]]], [[[8.0, 8.0, 2.0], [8.0, 2.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], 
[3.0, 1.0, 6.0], [6.0, 8.0, 7.0]], [[3.0, 1.0, 1.0], [7.0, 5.0, 4.0], [4.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 7.0], [5.0, 6.0, 4.0], [6.0, 5.0, 6.0]], [[7.0, 3.0, 6.0], [2.0, 3.0, 6.0], [5.0, 
4.0, 4.0]], [[7.0, 8.0, 2.0], [8.0, 5.0, 7.0], [3.0, 8.0, 3.0]]], [[[3.0, 2.0, 8.0], [4.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[3.0, 
3.0, 2.0], [7.0, 2.0, 6.0], [5.0, 1.0, 8.0]], [[2.0, 6.0, 1.0], [1.0, 7.0, 7.0], [1.0, 3.0, 6.0]]], [[[2.0, 7.0, 5.0], [7.0, 
1.0, 5.0], [1.0, 6.0, 3.0]], [[5.0, 8.0, 2.0], [6.0, 5.0, 4.0], [6.0, 4.0, 6.0]], [[3.0, 4.0, 5.0], [1.0, 6.0, 8.0], [6.0, 2.0, 
5.0]]], [[[5.0, 5.0, 6.0], [8.0, 2.0, 6.0], [7.0, 2.0, 7.0]], [[6.0, 4.0, 4.0], [1.0, 7.0, 8.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 
3.0], [2.0, 2.0, 4.0], [5.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 1.0], [7.0, 3.0, 2.0], [2.0, 8.0, 3.0]], 
[[8.0, 6.0, 4.0], [8.0, 7.0, 7.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 3.0], [2.0, 3.0, 2.0], [4.0, 3.0, 4.0]]], [[[3.0, 4.0, 6.0], 
[7.0, 8.0, 2.0], [1.0, 7.0, 3.0]], [[8.0, 5.0, 6.0], [6.0, 2.0, 5.0], [5.0, 8.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 1.0, 5.0], 
[7.0, 4.0, 1.0]]], [[[6.0, 3.0, 2.0], [8.0, 1.0, 2.0], [2.0, 1.0, 2.0]], [[3.0, 2.0, 7.0], [5.0, 1.0, 8.0], [8.0, 6.0, 8.0]], 
[[1.0, 1.0, 3.0], [8.0, 7.0, 6.0], [7.0, 4.0, 6.0]]], [[[6.0, 2.0, 7.0], [4.0, 2.0, 5.0], [2.0, 8.0, 8.0]], [[1.0, 5.0, 5.0], 
[7.0, 6.0, 3.0], [6.0, 6.0, 5.0]], [[5.0, 3.0, 7.0], [5.0, 5.0, 7.0], [7.0, 2.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0], [8.0], [6.0]])+(1.-msk_val)*numpy.array([[3.0], [5.0], [7.0]])
      arg[:3,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [5.0, 6.0, 4.0], [6.0, 5.0, 6.0]], [[7.0, 3.0, 6.0], [2.0, 3.0, 6.0], [5.0, 
4.0, 4.0]], [[7.0, 8.0, 2.0], [8.0, 5.0, 7.0], [3.0, 8.0, 3.0]]], [[[3.0, 2.0, 8.0], [4.0, 1.0, 2.0], [5.0, 5.0, 2.0]], [[3.0, 
3.0, 2.0], [7.0, 8.0, 6.0], [5.0, 1.0, 8.0]], [[2.0, 6.0, 1.0], [1.0, 7.0, 7.0], [1.0, 3.0, 6.0]]], [[[2.0, 7.0, 5.0], [7.0, 
1.0, 5.0], [1.0, 6.0, 3.0]], [[5.0, 8.0, 2.0], [6.0, 6.0, 4.0], [6.0, 4.0, 6.0]], [[3.0, 4.0, 5.0], [1.0, 6.0, 8.0], [6.0, 2.0, 
5.0]]], [[[5.0, 5.0, 6.0], [8.0, 2.0, 6.0], [7.0, 2.0, 7.0]], [[6.0, 4.0, 4.0], [1.0, 7.0, 8.0], [3.0, 8.0, 8.0]], [[5.0, 5.0, 
3.0], [2.0, 2.0, 4.0], [5.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 1.0], [7.0, 3.0, 2.0], [2.0, 8.0, 3.0]], 
[[8.0, 6.0, 4.0], [8.0, 3.0, 7.0], [3.0, 7.0, 8.0]], [[1.0, 6.0, 3.0], [2.0, 3.0, 2.0], [4.0, 3.0, 4.0]]], [[[3.0, 4.0, 6.0], 
[7.0, 8.0, 2.0], [1.0, 7.0, 3.0]], [[8.0, 5.0, 6.0], [6.0, 5.0, 5.0], [5.0, 8.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 1.0, 5.0], 
[7.0, 4.0, 1.0]]], [[[6.0, 3.0, 2.0], [8.0, 1.0, 2.0], [2.0, 1.0, 2.0]], [[3.0, 2.0, 7.0], [5.0, 7.0, 8.0], [8.0, 6.0, 8.0]], 
[[1.0, 1.0, 3.0], [8.0, 7.0, 6.0], [7.0, 4.0, 6.0]]], [[[6.0, 2.0, 7.0], [4.0, 2.0, 5.0], [2.0, 8.0, 8.0]], [[1.0, 5.0, 5.0], 
[7.0, 6.0, 3.0], [6.0, 6.0, 5.0]], [[5.0, 3.0, 7.0], [5.0, 5.0, 7.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 3.0], [2.0, 8.0, 5.0], [3.0, 5.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 6.0, 8.0], [1.0, 
2.0, 7.0]], [[6.0, 8.0, 8.0], [3.0, 5.0, 3.0], [1.0, 7.0, 7.0]]], [[[8.0, 8.0, 1.0], [8.0, 3.0, 2.0], [8.0, 6.0, 4.0]], [[7.0, 
5.0, 5.0], [6.0, 5.0, 2.0], [4.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], [3.0, 2.0, 5.0], [3.0, 6.0, 1.0]]], [[[6.0, 1.0, 3.0], [7.0, 
5.0, 4.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 1.0], [6.0, 3.0, 2.0], [1.0, 3.0, 1.0]], [[3.0, 3.0, 3.0], [4.0, 6.0, 4.0], [6.0, 1.0, 
1.0]]], [[[3.0, 2.0, 2.0], [3.0, 2.0, 8.0], [3.0, 4.0, 5.0]], [[2.0, 6.0, 4.0], [3.0, 6.0, 3.0], [2.0, 3.0, 7.0]], [[1.0, 5.0, 
2.0], [6.0, 7.0, 5.0], [4.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 8.0], [8.0, 5.0, 2.0], [8.0, 6.0, 4.0]], 
[[7.0, 6.0, 3.0], [3.0, 7.0, 5.0], [7.0, 7.0, 7.0]], [[6.0, 7.0, 5.0], [1.0, 4.0, 8.0], [7.0, 7.0, 5.0]]], [[[4.0, 2.0, 7.0], 
[7.0, 2.0, 7.0], [8.0, 7.0, 3.0]], [[1.0, 8.0, 1.0], [8.0, 4.0, 4.0], [1.0, 2.0, 5.0]], [[6.0, 1.0, 2.0], [3.0, 4.0, 6.0], 
[2.0, 6.0, 7.0]]], [[[3.0, 8.0, 4.0], [2.0, 5.0, 1.0], [7.0, 7.0, 3.0]], [[6.0, 1.0, 6.0], [8.0, 8.0, 5.0], [2.0, 1.0, 5.0]], 
[[5.0, 3.0, 4.0], [8.0, 5.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 7.0, 8.0], [5.0, 5.0, 5.0], [2.0, 2.0, 3.0]], [[5.0, 7.0, 1.0], 
[3.0, 2.0, 1.0], [5.0, 7.0, 1.0]], [[6.0, 3.0, 3.0], [6.0, 3.0, 8.0], [8.0, 7.0, 8.0]]]])
      val=numpy.array([[[8.0, 8.0, 8.0]]])
      arg[2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 3.0], [2.0, 8.0, 5.0], [3.0, 5.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 6.0, 8.0], [1.0, 
2.0, 7.0]], [[6.0, 8.0, 8.0], [3.0, 5.0, 3.0], [1.0, 7.0, 7.0]]], [[[8.0, 8.0, 1.0], [8.0, 3.0, 2.0], [8.0, 6.0, 4.0]], [[7.0, 
5.0, 5.0], [6.0, 5.0, 2.0], [4.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], [3.0, 2.0, 5.0], [3.0, 6.0, 1.0]]], [[[6.0, 1.0, 3.0], [7.0, 
5.0, 4.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 1.0], [8.0, 8.0, 8.0], [1.0, 3.0, 1.0]], [[3.0, 3.0, 3.0], [4.0, 6.0, 4.0], [6.0, 1.0, 
1.0]]], [[[3.0, 2.0, 2.0], [3.0, 2.0, 8.0], [3.0, 4.0, 5.0]], [[2.0, 6.0, 4.0], [3.0, 6.0, 3.0], [2.0, 3.0, 7.0]], [[1.0, 5.0, 
2.0], [6.0, 7.0, 5.0], [4.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 8.0], [8.0, 5.0, 2.0], [8.0, 6.0, 4.0]], 
[[7.0, 6.0, 3.0], [3.0, 7.0, 5.0], [7.0, 7.0, 7.0]], [[6.0, 7.0, 5.0], [1.0, 4.0, 8.0], [7.0, 7.0, 5.0]]], [[[4.0, 2.0, 7.0], 
[7.0, 2.0, 7.0], [8.0, 7.0, 3.0]], [[1.0, 8.0, 1.0], [8.0, 4.0, 4.0], [1.0, 2.0, 5.0]], [[6.0, 1.0, 2.0], [3.0, 4.0, 6.0], 
[2.0, 6.0, 7.0]]], [[[3.0, 8.0, 4.0], [2.0, 5.0, 1.0], [7.0, 7.0, 3.0]], [[6.0, 1.0, 6.0], [8.0, 8.0, 8.0], [2.0, 1.0, 5.0]], 
[[5.0, 3.0, 4.0], [8.0, 5.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 7.0, 8.0], [5.0, 5.0, 5.0], [2.0, 2.0, 3.0]], [[5.0, 7.0, 1.0], 
[3.0, 2.0, 1.0], [5.0, 7.0, 1.0]], [[6.0, 3.0, 3.0], [6.0, 3.0, 8.0], [8.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 6.0], [2.0, 6.0, 1.0], [2.0, 8.0, 6.0]], [[2.0, 2.0, 3.0], [7.0, 2.0, 1.0], [3.0, 
2.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 7.0, 8.0], [1.0, 2.0, 6.0]]], [[[3.0, 1.0, 5.0], [4.0, 1.0, 3.0], [2.0, 4.0, 8.0]], [[8.0, 
3.0, 7.0], [1.0, 7.0, 4.0], [5.0, 3.0, 8.0]], [[6.0, 6.0, 6.0], [5.0, 2.0, 7.0], [6.0, 8.0, 8.0]]], [[[7.0, 8.0, 3.0], [5.0, 
3.0, 2.0], [1.0, 6.0, 6.0]], [[2.0, 7.0, 2.0], [2.0, 3.0, 8.0], [2.0, 8.0, 3.0]], [[7.0, 1.0, 6.0], [1.0, 1.0, 8.0], [4.0, 3.0, 
3.0]]], [[[3.0, 2.0, 1.0], [1.0, 6.0, 5.0], [3.0, 7.0, 8.0]], [[8.0, 3.0, 4.0], [3.0, 7.0, 1.0], [7.0, 2.0, 7.0]], [[8.0, 8.0, 
5.0], [2.0, 2.0, 6.0], [8.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 7.0], [6.0, 1.0, 2.0], [8.0, 5.0, 6.0]], 
[[8.0, 2.0, 2.0], [5.0, 2.0, 7.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], [4.0, 2.0, 6.0], [5.0, 2.0, 2.0]]], [[[6.0, 2.0, 7.0], 
[8.0, 5.0, 1.0], [6.0, 3.0, 5.0]], [[1.0, 4.0, 3.0], [7.0, 2.0, 2.0], [1.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 2.0, 2.0], 
[5.0, 8.0, 4.0]]], [[[4.0, 5.0, 6.0], [4.0, 2.0, 6.0], [3.0, 1.0, 5.0]], [[1.0, 5.0, 8.0], [8.0, 2.0, 4.0], [2.0, 5.0, 1.0]], 
[[5.0, 4.0, 8.0], [3.0, 1.0, 7.0], [1.0, 7.0, 1.0]]], [[[1.0, 6.0, 7.0], [8.0, 4.0, 7.0], [7.0, 2.0, 3.0]], [[5.0, 4.0, 4.0], 
[5.0, 4.0, 5.0], [2.0, 2.0, 4.0]], [[8.0, 2.0, 5.0], [4.0, 8.0, 4.0], [7.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[2.0, 4.0, 6.0]]]),self.functionspace)
      arg[2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 6.0], [2.0, 6.0, 1.0], [2.0, 8.0, 6.0]], [[2.0, 2.0, 3.0], [7.0, 2.0, 1.0], [3.0, 
2.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 7.0, 8.0], [1.0, 2.0, 6.0]]], [[[3.0, 1.0, 5.0], [4.0, 1.0, 3.0], [2.0, 4.0, 8.0]], [[8.0, 
3.0, 7.0], [1.0, 7.0, 4.0], [5.0, 3.0, 8.0]], [[6.0, 6.0, 6.0], [5.0, 2.0, 7.0], [6.0, 8.0, 8.0]]], [[[7.0, 8.0, 3.0], [5.0, 
3.0, 2.0], [1.0, 6.0, 6.0]], [[2.0, 7.0, 2.0], [2.0, 4.0, 6.0], [2.0, 8.0, 3.0]], [[7.0, 1.0, 6.0], [1.0, 1.0, 8.0], [4.0, 3.0, 
3.0]]], [[[3.0, 2.0, 1.0], [1.0, 6.0, 5.0], [3.0, 7.0, 8.0]], [[8.0, 3.0, 4.0], [3.0, 7.0, 1.0], [7.0, 2.0, 7.0]], [[8.0, 8.0, 
5.0], [2.0, 2.0, 6.0], [8.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [6.0, 1.0, 2.0], [8.0, 5.0, 6.0]], 
[[8.0, 2.0, 2.0], [5.0, 2.0, 7.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], [4.0, 2.0, 6.0], [5.0, 2.0, 2.0]]], [[[6.0, 2.0, 7.0], 
[8.0, 5.0, 1.0], [6.0, 3.0, 5.0]], [[1.0, 4.0, 3.0], [7.0, 2.0, 2.0], [1.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 2.0, 2.0], 
[5.0, 8.0, 4.0]]], [[[4.0, 5.0, 6.0], [4.0, 2.0, 6.0], [3.0, 1.0, 5.0]], [[1.0, 5.0, 8.0], [2.0, 4.0, 6.0], [2.0, 5.0, 1.0]], 
[[5.0, 4.0, 8.0], [3.0, 1.0, 7.0], [1.0, 7.0, 1.0]]], [[[1.0, 6.0, 7.0], [8.0, 4.0, 7.0], [7.0, 2.0, 3.0]], [[5.0, 4.0, 4.0], 
[5.0, 4.0, 5.0], [2.0, 2.0, 4.0]], [[8.0, 2.0, 5.0], [4.0, 8.0, 4.0], [7.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 3.0], [8.0, 4.0, 2.0], [4.0, 1.0, 3.0]], [[2.0, 8.0, 4.0], [3.0, 7.0, 5.0], [6.0, 
4.0, 8.0]], [[5.0, 8.0, 5.0], [7.0, 7.0, 4.0], [7.0, 5.0, 7.0]]], [[[4.0, 2.0, 1.0], [8.0, 8.0, 2.0], [1.0, 8.0, 1.0]], [[6.0, 
4.0, 1.0], [8.0, 8.0, 4.0], [2.0, 4.0, 6.0]], [[1.0, 1.0, 5.0], [5.0, 4.0, 2.0], [7.0, 5.0, 5.0]]], [[[8.0, 3.0, 3.0], [8.0, 
5.0, 8.0], [6.0, 4.0, 2.0]], [[1.0, 1.0, 7.0], [6.0, 3.0, 4.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], [4.0, 2.0, 6.0], [2.0, 1.0, 
6.0]]], [[[4.0, 2.0, 4.0], [7.0, 2.0, 3.0], [5.0, 7.0, 8.0]], [[8.0, 8.0, 6.0], [2.0, 1.0, 1.0], [6.0, 4.0, 2.0]], [[8.0, 2.0, 
3.0], [3.0, 5.0, 2.0], [1.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 6.0], [7.0, 3.0, 8.0], [1.0, 3.0, 8.0]], 
[[5.0, 3.0, 6.0], [8.0, 1.0, 5.0], [2.0, 1.0, 1.0]], [[2.0, 7.0, 2.0], [5.0, 7.0, 1.0], [2.0, 4.0, 2.0]]], [[[3.0, 2.0, 2.0], 
[2.0, 5.0, 7.0], [3.0, 2.0, 4.0]], [[6.0, 2.0, 1.0], [5.0, 3.0, 8.0], [5.0, 1.0, 6.0]], [[5.0, 5.0, 7.0], [1.0, 7.0, 8.0], 
[6.0, 2.0, 1.0]]], [[[7.0, 6.0, 1.0], [6.0, 2.0, 5.0], [6.0, 5.0, 7.0]], [[2.0, 2.0, 2.0], [8.0, 5.0, 8.0], [4.0, 1.0, 6.0]], 
[[4.0, 2.0, 6.0], [5.0, 5.0, 3.0], [4.0, 8.0, 7.0]]], [[[8.0, 7.0, 7.0], [5.0, 3.0, 6.0], [6.0, 1.0, 1.0]], [[5.0, 7.0, 6.0], 
[5.0, 7.0, 2.0], [2.0, 3.0, 2.0]], [[8.0, 1.0, 3.0], [7.0, 8.0, 1.0], [6.0, 3.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 3.0, 7.0]]])+(1.-msk_val)*numpy.array([[[7.0, 1.0, 4.0]]])
      arg[2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 3.0], [8.0, 4.0, 2.0], [4.0, 1.0, 3.0]], [[2.0, 8.0, 4.0], [3.0, 7.0, 5.0], [6.0, 
4.0, 8.0]], [[5.0, 8.0, 5.0], [7.0, 7.0, 4.0], [7.0, 5.0, 7.0]]], [[[4.0, 2.0, 1.0], [8.0, 8.0, 2.0], [1.0, 8.0, 1.0]], [[6.0, 
4.0, 1.0], [8.0, 8.0, 4.0], [2.0, 4.0, 6.0]], [[1.0, 1.0, 5.0], [5.0, 4.0, 2.0], [7.0, 5.0, 5.0]]], [[[8.0, 3.0, 3.0], [8.0, 
5.0, 8.0], [6.0, 4.0, 2.0]], [[1.0, 1.0, 7.0], [2.0, 3.0, 7.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 8.0], [4.0, 2.0, 6.0], [2.0, 1.0, 
6.0]]], [[[4.0, 2.0, 4.0], [7.0, 2.0, 3.0], [5.0, 7.0, 8.0]], [[8.0, 8.0, 6.0], [2.0, 1.0, 1.0], [6.0, 4.0, 2.0]], [[8.0, 2.0, 
3.0], [3.0, 5.0, 2.0], [1.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 6.0], [7.0, 3.0, 8.0], [1.0, 3.0, 8.0]], 
[[5.0, 3.0, 6.0], [8.0, 1.0, 5.0], [2.0, 1.0, 1.0]], [[2.0, 7.0, 2.0], [5.0, 7.0, 1.0], [2.0, 4.0, 2.0]]], [[[3.0, 2.0, 2.0], 
[2.0, 5.0, 7.0], [3.0, 2.0, 4.0]], [[6.0, 2.0, 1.0], [5.0, 3.0, 8.0], [5.0, 1.0, 6.0]], [[5.0, 5.0, 7.0], [1.0, 7.0, 8.0], 
[6.0, 2.0, 1.0]]], [[[7.0, 6.0, 1.0], [6.0, 2.0, 5.0], [6.0, 5.0, 7.0]], [[2.0, 2.0, 2.0], [7.0, 1.0, 4.0], [4.0, 1.0, 6.0]], 
[[4.0, 2.0, 6.0], [5.0, 5.0, 3.0], [4.0, 8.0, 7.0]]], [[[8.0, 7.0, 7.0], [5.0, 3.0, 6.0], [6.0, 1.0, 1.0]], [[5.0, 7.0, 6.0], 
[5.0, 7.0, 2.0], [2.0, 3.0, 2.0]], [[8.0, 1.0, 3.0], [7.0, 8.0, 1.0], [6.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 2.0], [6.0, 7.0, 8.0], [4.0, 8.0, 5.0]], [[6.0, 2.0, 6.0], [7.0, 5.0, 3.0], [7.0, 
7.0, 4.0]], [[1.0, 1.0, 4.0], [7.0, 4.0, 5.0], [6.0, 2.0, 1.0]]], [[[6.0, 1.0, 2.0], [5.0, 6.0, 6.0], [1.0, 3.0, 8.0]], [[4.0, 
8.0, 7.0], [5.0, 5.0, 8.0], [6.0, 6.0, 7.0]], [[6.0, 7.0, 6.0], [1.0, 4.0, 3.0], [2.0, 1.0, 1.0]]], [[[8.0, 7.0, 5.0], [2.0, 
5.0, 4.0], [8.0, 6.0, 4.0]], [[2.0, 8.0, 7.0], [1.0, 2.0, 4.0], [2.0, 8.0, 2.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 6.0], [8.0, 8.0, 
3.0]]], [[[8.0, 2.0, 7.0], [6.0, 8.0, 2.0], [3.0, 8.0, 7.0]], [[3.0, 8.0, 8.0], [3.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[4.0, 6.0, 
2.0], [5.0, 6.0, 1.0], [5.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 7.0], [3.0, 7.0, 8.0], [4.0, 3.0, 5.0]], 
[[4.0, 3.0, 6.0], [5.0, 8.0, 3.0], [5.0, 2.0, 4.0]], [[4.0, 5.0, 8.0], [2.0, 3.0, 5.0], [7.0, 4.0, 4.0]]], [[[5.0, 5.0, 6.0], 
[3.0, 4.0, 7.0], [6.0, 7.0, 3.0]], [[3.0, 2.0, 2.0], [7.0, 5.0, 5.0], [5.0, 8.0, 1.0]], [[6.0, 3.0, 2.0], [5.0, 1.0, 4.0], 
[4.0, 4.0, 5.0]]], [[[6.0, 1.0, 2.0], [4.0, 1.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 4.0], [5.0, 8.0, 1.0]], 
[[3.0, 1.0, 6.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0]]], [[[8.0, 2.0, 1.0], [2.0, 8.0, 1.0], [1.0, 7.0, 7.0]], [[8.0, 4.0, 1.0], 
[1.0, 3.0, 3.0], [4.0, 4.0, 6.0]], [[6.0, 3.0, 1.0], [8.0, 5.0, 1.0], [7.0, 2.0, 4.0]]]])
      val=numpy.array([[[1.0]], [[4.0]], [[2.0]], [[3.0]]])
      arg[:,1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 2.0], [6.0, 7.0, 8.0], [4.0, 8.0, 5.0]], [[6.0, 2.0, 6.0], [7.0, 5.0, 3.0], [7.0, 
1.0, 4.0]], [[1.0, 1.0, 4.0], [7.0, 4.0, 5.0], [6.0, 2.0, 1.0]]], [[[6.0, 1.0, 2.0], [5.0, 6.0, 6.0], [1.0, 3.0, 8.0]], [[4.0, 
8.0, 7.0], [5.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 7.0, 6.0], [1.0, 4.0, 3.0], [2.0, 1.0, 1.0]]], [[[8.0, 7.0, 5.0], [2.0, 
5.0, 4.0], [8.0, 6.0, 4.0]], [[2.0, 8.0, 7.0], [1.0, 2.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 6.0], [8.0, 8.0, 
3.0]]], [[[8.0, 2.0, 7.0], [6.0, 8.0, 2.0], [3.0, 8.0, 7.0]], [[3.0, 8.0, 8.0], [3.0, 6.0, 3.0], [1.0, 3.0, 5.0]], [[4.0, 6.0, 
2.0], [5.0, 6.0, 1.0], [5.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [3.0, 7.0, 8.0], [4.0, 3.0, 5.0]], 
[[4.0, 3.0, 6.0], [5.0, 8.0, 3.0], [5.0, 1.0, 4.0]], [[4.0, 5.0, 8.0], [2.0, 3.0, 5.0], [7.0, 4.0, 4.0]]], [[[5.0, 5.0, 6.0], 
[3.0, 4.0, 7.0], [6.0, 7.0, 3.0]], [[3.0, 2.0, 2.0], [7.0, 5.0, 5.0], [5.0, 4.0, 1.0]], [[6.0, 3.0, 2.0], [5.0, 1.0, 4.0], 
[4.0, 4.0, 5.0]]], [[[6.0, 1.0, 2.0], [4.0, 1.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [5.0, 5.0, 4.0], [5.0, 2.0, 1.0]], 
[[3.0, 1.0, 6.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0]]], [[[8.0, 2.0, 1.0], [2.0, 8.0, 1.0], [1.0, 7.0, 7.0]], [[8.0, 4.0, 1.0], 
[1.0, 3.0, 3.0], [4.0, 3.0, 6.0]], [[6.0, 3.0, 1.0], [8.0, 5.0, 1.0], [7.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 7.0], [2.0, 7.0, 4.0], [2.0, 7.0, 1.0]], [[5.0, 6.0, 5.0], [5.0, 2.0, 6.0], [2.0, 
8.0, 6.0]], [[2.0, 6.0, 6.0], [8.0, 7.0, 5.0], [8.0, 4.0, 3.0]]], [[[7.0, 1.0, 2.0], [4.0, 3.0, 7.0], [3.0, 4.0, 8.0]], [[6.0, 
5.0, 2.0], [4.0, 3.0, 1.0], [4.0, 4.0, 7.0]], [[8.0, 4.0, 5.0], [6.0, 5.0, 8.0], [5.0, 5.0, 3.0]]], [[[2.0, 7.0, 2.0], [8.0, 
1.0, 7.0], [1.0, 3.0, 8.0]], [[2.0, 4.0, 5.0], [2.0, 2.0, 2.0], [3.0, 5.0, 6.0]], [[2.0, 5.0, 5.0], [8.0, 7.0, 6.0], [8.0, 6.0, 
3.0]]], [[[6.0, 3.0, 3.0], [8.0, 6.0, 1.0], [5.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [4.0, 1.0, 7.0], [4.0, 4.0, 1.0]], [[1.0, 4.0, 
5.0], [3.0, 3.0, 2.0], [3.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 1.0], [3.0, 1.0, 2.0], [1.0, 8.0, 3.0]], 
[[2.0, 8.0, 8.0], [1.0, 6.0, 3.0], [3.0, 4.0, 1.0]], [[3.0, 1.0, 5.0], [2.0, 3.0, 1.0], [5.0, 5.0, 1.0]]], [[[6.0, 5.0, 8.0], 
[7.0, 1.0, 8.0], [8.0, 6.0, 3.0]], [[2.0, 7.0, 4.0], [8.0, 4.0, 5.0], [2.0, 8.0, 1.0]], [[3.0, 4.0, 3.0], [4.0, 5.0, 6.0], 
[8.0, 7.0, 7.0]]], [[[3.0, 7.0, 1.0], [8.0, 5.0, 4.0], [1.0, 5.0, 5.0]], [[6.0, 8.0, 8.0], [6.0, 4.0, 6.0], [5.0, 6.0, 5.0]], 
[[2.0, 7.0, 1.0], [3.0, 7.0, 5.0], [3.0, 7.0, 8.0]]], [[[3.0, 5.0, 6.0], [6.0, 1.0, 3.0], [4.0, 2.0, 6.0]], [[8.0, 7.0, 1.0], 
[1.0, 4.0, 1.0], [7.0, 5.0, 1.0]], [[5.0, 6.0, 4.0], [1.0, 8.0, 7.0], [4.0, 6.0, 5.0]]]])
      val=Data(numpy.array([[[1.0]], [[1.0]], [[2.0]], [[1.0]]]),self.functionspace)
      arg[:,1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [2.0, 7.0, 4.0], [2.0, 7.0, 1.0]], [[5.0, 6.0, 5.0], [5.0, 2.0, 6.0], [2.0, 
1.0, 6.0]], [[2.0, 6.0, 6.0], [8.0, 7.0, 5.0], [8.0, 4.0, 3.0]]], [[[7.0, 1.0, 2.0], [4.0, 3.0, 7.0], [3.0, 4.0, 8.0]], [[6.0, 
5.0, 2.0], [4.0, 3.0, 1.0], [4.0, 1.0, 7.0]], [[8.0, 4.0, 5.0], [6.0, 5.0, 8.0], [5.0, 5.0, 3.0]]], [[[2.0, 7.0, 2.0], [8.0, 
1.0, 7.0], [1.0, 3.0, 8.0]], [[2.0, 4.0, 5.0], [2.0, 2.0, 2.0], [3.0, 2.0, 6.0]], [[2.0, 5.0, 5.0], [8.0, 7.0, 6.0], [8.0, 6.0, 
3.0]]], [[[6.0, 3.0, 3.0], [8.0, 6.0, 1.0], [5.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [4.0, 1.0, 7.0], [4.0, 1.0, 1.0]], [[1.0, 4.0, 
5.0], [3.0, 3.0, 2.0], [3.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [3.0, 1.0, 2.0], [1.0, 8.0, 3.0]], 
[[2.0, 8.0, 8.0], [1.0, 6.0, 3.0], [3.0, 1.0, 1.0]], [[3.0, 1.0, 5.0], [2.0, 3.0, 1.0], [5.0, 5.0, 1.0]]], [[[6.0, 5.0, 8.0], 
[7.0, 1.0, 8.0], [8.0, 6.0, 3.0]], [[2.0, 7.0, 4.0], [8.0, 4.0, 5.0], [2.0, 1.0, 1.0]], [[3.0, 4.0, 3.0], [4.0, 5.0, 6.0], 
[8.0, 7.0, 7.0]]], [[[3.0, 7.0, 1.0], [8.0, 5.0, 4.0], [1.0, 5.0, 5.0]], [[6.0, 8.0, 8.0], [6.0, 4.0, 6.0], [5.0, 2.0, 5.0]], 
[[2.0, 7.0, 1.0], [3.0, 7.0, 5.0], [3.0, 7.0, 8.0]]], [[[3.0, 5.0, 6.0], [6.0, 1.0, 3.0], [4.0, 2.0, 6.0]], [[8.0, 7.0, 1.0], 
[1.0, 4.0, 1.0], [7.0, 1.0, 1.0]], [[5.0, 6.0, 4.0], [1.0, 8.0, 7.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 6.0], [8.0, 4.0, 6.0], [5.0, 3.0, 2.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 6.0], [6.0, 
4.0, 1.0]], [[6.0, 8.0, 2.0], [6.0, 8.0, 3.0], [2.0, 4.0, 6.0]]], [[[4.0, 2.0, 5.0], [5.0, 7.0, 6.0], [5.0, 6.0, 1.0]], [[6.0, 
8.0, 1.0], [7.0, 5.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 7.0, 5.0], [3.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[1.0, 8.0, 8.0], [2.0, 
4.0, 2.0], [3.0, 6.0, 8.0]], [[8.0, 6.0, 6.0], [3.0, 2.0, 5.0], [8.0, 5.0, 1.0]], [[7.0, 5.0, 6.0], [6.0, 1.0, 1.0], [5.0, 2.0, 
8.0]]], [[[3.0, 5.0, 3.0], [3.0, 5.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 1.0, 7.0], [1.0, 2.0, 3.0], [3.0, 8.0, 7.0]], [[2.0, 4.0, 
4.0], [2.0, 1.0, 5.0], [8.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 6.0], [6.0, 4.0, 5.0], [7.0, 8.0, 7.0]], 
[[4.0, 4.0, 7.0], [8.0, 3.0, 2.0], [4.0, 8.0, 8.0]], [[3.0, 8.0, 7.0], [5.0, 6.0, 8.0], [3.0, 5.0, 8.0]]], [[[7.0, 7.0, 5.0], 
[8.0, 7.0, 7.0], [4.0, 6.0, 5.0]], [[8.0, 6.0, 6.0], [4.0, 5.0, 5.0], [1.0, 7.0, 7.0]], [[8.0, 1.0, 5.0], [8.0, 5.0, 1.0], 
[8.0, 6.0, 4.0]]], [[[8.0, 8.0, 1.0], [8.0, 1.0, 5.0], [8.0, 6.0, 8.0]], [[4.0, 8.0, 4.0], [4.0, 6.0, 4.0], [3.0, 1.0, 3.0]], 
[[8.0, 1.0, 3.0], [6.0, 4.0, 2.0], [7.0, 8.0, 6.0]]], [[[8.0, 2.0, 4.0], [6.0, 3.0, 4.0], [5.0, 4.0, 4.0]], [[7.0, 6.0, 3.0], 
[4.0, 8.0, 6.0], [4.0, 4.0, 1.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 4.0], [8.0, 6.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0]], [[3.0]], [[1.0]], [[5.0]]])+(1.-msk_val)*numpy.array([[[6.0]], [[7.0]], [[5.0]], 
[[4.0]]])
      arg[:,1,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 6.0], [8.0, 4.0, 6.0], [5.0, 3.0, 2.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 6.0], [6.0, 
7.0, 1.0]], [[6.0, 8.0, 2.0], [6.0, 8.0, 3.0], [2.0, 4.0, 6.0]]], [[[4.0, 2.0, 5.0], [5.0, 7.0, 6.0], [5.0, 6.0, 1.0]], [[6.0, 
8.0, 1.0], [7.0, 5.0, 4.0], [5.0, 3.0, 8.0]], [[3.0, 7.0, 5.0], [3.0, 8.0, 5.0], [8.0, 1.0, 6.0]]], [[[1.0, 8.0, 8.0], [2.0, 
4.0, 2.0], [3.0, 6.0, 8.0]], [[8.0, 6.0, 6.0], [3.0, 2.0, 5.0], [8.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], [6.0, 1.0, 1.0], [5.0, 2.0, 
8.0]]], [[[3.0, 5.0, 3.0], [3.0, 5.0, 2.0], [1.0, 4.0, 5.0]], [[4.0, 1.0, 7.0], [1.0, 2.0, 3.0], [3.0, 5.0, 7.0]], [[2.0, 4.0, 
4.0], [2.0, 1.0, 5.0], [8.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [6.0, 4.0, 5.0], [7.0, 8.0, 7.0]], 
[[4.0, 4.0, 7.0], [8.0, 3.0, 2.0], [4.0, 6.0, 8.0]], [[3.0, 8.0, 7.0], [5.0, 6.0, 8.0], [3.0, 5.0, 8.0]]], [[[7.0, 7.0, 5.0], 
[8.0, 7.0, 7.0], [4.0, 6.0, 5.0]], [[8.0, 6.0, 6.0], [4.0, 5.0, 5.0], [1.0, 7.0, 7.0]], [[8.0, 1.0, 5.0], [8.0, 5.0, 1.0], 
[8.0, 6.0, 4.0]]], [[[8.0, 8.0, 1.0], [8.0, 1.0, 5.0], [8.0, 6.0, 8.0]], [[4.0, 8.0, 4.0], [4.0, 6.0, 4.0], [3.0, 5.0, 3.0]], 
[[8.0, 1.0, 3.0], [6.0, 4.0, 2.0], [7.0, 8.0, 6.0]]], [[[8.0, 2.0, 4.0], [6.0, 3.0, 4.0], [5.0, 4.0, 4.0]], [[7.0, 6.0, 3.0], 
[4.0, 8.0, 6.0], [4.0, 4.0, 1.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 4.0], [8.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 2.0], [7.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[3.0, 8.0, 6.0], [6.0, 5.0, 5.0], [6.0, 
5.0, 4.0]], [[6.0, 4.0, 3.0], [5.0, 1.0, 7.0], [7.0, 8.0, 1.0]]], [[[2.0, 1.0, 3.0], [5.0, 5.0, 4.0], [3.0, 6.0, 8.0]], [[4.0, 
7.0, 2.0], [6.0, 3.0, 6.0], [8.0, 4.0, 3.0]], [[7.0, 5.0, 8.0], [4.0, 5.0, 1.0], [1.0, 7.0, 1.0]]], [[[8.0, 5.0, 3.0], [5.0, 
4.0, 3.0], [1.0, 4.0, 4.0]], [[7.0, 5.0, 4.0], [1.0, 8.0, 2.0], [2.0, 8.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 4.0, 2.0], [5.0, 8.0, 
6.0]]], [[[5.0, 5.0, 7.0], [5.0, 1.0, 7.0], [8.0, 3.0, 8.0]], [[4.0, 5.0, 5.0], [3.0, 3.0, 4.0], [4.0, 7.0, 1.0]], [[4.0, 2.0, 
6.0], [8.0, 8.0, 4.0], [1.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 7.0], [6.0, 8.0, 2.0], [3.0, 6.0, 3.0]], 
[[5.0, 6.0, 1.0], [6.0, 8.0, 7.0], [1.0, 5.0, 6.0]], [[6.0, 1.0, 5.0], [3.0, 6.0, 4.0], [3.0, 3.0, 3.0]]], [[[5.0, 2.0, 3.0], 
[8.0, 8.0, 4.0], [8.0, 3.0, 4.0]], [[2.0, 8.0, 6.0], [4.0, 5.0, 4.0], [7.0, 1.0, 6.0]], [[8.0, 5.0, 1.0], [1.0, 5.0, 4.0], 
[6.0, 8.0, 3.0]]], [[[7.0, 5.0, 6.0], [5.0, 7.0, 1.0], [2.0, 3.0, 8.0]], [[2.0, 3.0, 2.0], [2.0, 3.0, 7.0], [8.0, 6.0, 7.0]], 
[[6.0, 1.0, 5.0], [2.0, 5.0, 1.0], [8.0, 8.0, 5.0]]], [[[2.0, 5.0, 5.0], [3.0, 3.0, 3.0], [7.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], 
[2.0, 6.0, 7.0], [2.0, 6.0, 5.0]], [[3.0, 1.0, 8.0], [4.0, 8.0, 8.0], [7.0, 1.0, 7.0]]]])
      val=numpy.array([[[[8.0, 2.0, 4.0]]], [[[7.0, 1.0, 2.0]]]])
      arg[1:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 2.0], [7.0, 1.0, 3.0], [2.0, 3.0, 3.0]], [[3.0, 8.0, 6.0], [6.0, 5.0, 5.0], [6.0, 
5.0, 4.0]], [[6.0, 4.0, 3.0], [5.0, 1.0, 7.0], [7.0, 8.0, 1.0]]], [[[2.0, 1.0, 3.0], [5.0, 5.0, 4.0], [3.0, 6.0, 8.0]], [[4.0, 
7.0, 2.0], [6.0, 3.0, 6.0], [8.0, 4.0, 3.0]], [[7.0, 5.0, 8.0], [4.0, 5.0, 1.0], [8.0, 2.0, 4.0]]], [[[8.0, 5.0, 3.0], [5.0, 
4.0, 3.0], [1.0, 4.0, 4.0]], [[7.0, 5.0, 4.0], [1.0, 8.0, 2.0], [2.0, 8.0, 5.0]], [[4.0, 2.0, 4.0], [8.0, 4.0, 2.0], [7.0, 1.0, 
2.0]]], [[[5.0, 5.0, 7.0], [5.0, 1.0, 7.0], [8.0, 3.0, 8.0]], [[4.0, 5.0, 5.0], [3.0, 3.0, 4.0], [4.0, 7.0, 1.0]], [[4.0, 2.0, 
6.0], [8.0, 8.0, 4.0], [1.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [6.0, 8.0, 2.0], [3.0, 6.0, 3.0]], 
[[5.0, 6.0, 1.0], [6.0, 8.0, 7.0], [1.0, 5.0, 6.0]], [[6.0, 1.0, 5.0], [3.0, 6.0, 4.0], [3.0, 3.0, 3.0]]], [[[5.0, 2.0, 3.0], 
[8.0, 8.0, 4.0], [8.0, 3.0, 4.0]], [[2.0, 8.0, 6.0], [4.0, 5.0, 4.0], [7.0, 1.0, 6.0]], [[8.0, 5.0, 1.0], [1.0, 5.0, 4.0], 
[8.0, 2.0, 4.0]]], [[[7.0, 5.0, 6.0], [5.0, 7.0, 1.0], [2.0, 3.0, 8.0]], [[2.0, 3.0, 2.0], [2.0, 3.0, 7.0], [8.0, 6.0, 7.0]], 
[[6.0, 1.0, 5.0], [2.0, 5.0, 1.0], [7.0, 1.0, 2.0]]], [[[2.0, 5.0, 5.0], [3.0, 3.0, 3.0], [7.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], 
[2.0, 6.0, 7.0], [2.0, 6.0, 5.0]], [[3.0, 1.0, 8.0], [4.0, 8.0, 8.0], [7.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 7.0], [7.0, 6.0, 6.0], [5.0, 4.0, 5.0]], [[4.0, 7.0, 3.0], [3.0, 2.0, 3.0], [2.0, 
4.0, 5.0]], [[7.0, 3.0, 4.0], [4.0, 4.0, 7.0], [3.0, 4.0, 5.0]]], [[[1.0, 3.0, 3.0], [2.0, 8.0, 4.0], [6.0, 5.0, 6.0]], [[6.0, 
7.0, 4.0], [6.0, 5.0, 1.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [6.0, 1.0, 6.0], [5.0, 2.0, 1.0]]], [[[8.0, 8.0, 7.0], [7.0, 
8.0, 8.0], [2.0, 6.0, 4.0]], [[1.0, 5.0, 6.0], [7.0, 8.0, 3.0], [7.0, 2.0, 2.0]], [[1.0, 3.0, 7.0], [5.0, 3.0, 8.0], [3.0, 4.0, 
5.0]]], [[[3.0, 3.0, 7.0], [5.0, 1.0, 2.0], [2.0, 5.0, 7.0]], [[4.0, 3.0, 3.0], [4.0, 2.0, 7.0], [5.0, 4.0, 3.0]], [[8.0, 2.0, 
8.0], [7.0, 3.0, 1.0], [1.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 1.0], [8.0, 1.0, 8.0], [3.0, 5.0, 4.0]], 
[[1.0, 1.0, 4.0], [8.0, 5.0, 8.0], [3.0, 5.0, 2.0]], [[7.0, 5.0, 5.0], [8.0, 2.0, 5.0], [2.0, 2.0, 7.0]]], [[[8.0, 3.0, 8.0], 
[8.0, 4.0, 3.0], [3.0, 1.0, 2.0]], [[6.0, 4.0, 1.0], [7.0, 4.0, 2.0], [2.0, 8.0, 5.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 8.0], 
[4.0, 6.0, 6.0]]], [[[7.0, 4.0, 2.0], [4.0, 4.0, 5.0], [7.0, 7.0, 8.0]], [[8.0, 5.0, 6.0], [1.0, 2.0, 1.0], [4.0, 4.0, 3.0]], 
[[6.0, 7.0, 2.0], [6.0, 7.0, 1.0], [3.0, 2.0, 7.0]]], [[[2.0, 8.0, 5.0], [1.0, 5.0, 6.0], [7.0, 6.0, 8.0]], [[7.0, 7.0, 8.0], 
[7.0, 5.0, 2.0], [2.0, 3.0, 2.0]], [[8.0, 3.0, 4.0], [8.0, 5.0, 1.0], [5.0, 6.0, 8.0]]]])
      val=Data(numpy.array([[[[4.0, 1.0, 1.0]]], [[[2.0, 3.0, 3.0]]]]),self.functionspace)
      arg[1:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 7.0], [7.0, 6.0, 6.0], [5.0, 4.0, 5.0]], [[4.0, 7.0, 3.0], [3.0, 2.0, 3.0], [2.0, 
4.0, 5.0]], [[7.0, 3.0, 4.0], [4.0, 4.0, 7.0], [3.0, 4.0, 5.0]]], [[[1.0, 3.0, 3.0], [2.0, 8.0, 4.0], [6.0, 5.0, 6.0]], [[6.0, 
7.0, 4.0], [6.0, 5.0, 1.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 6.0], [6.0, 1.0, 6.0], [4.0, 1.0, 1.0]]], [[[8.0, 8.0, 7.0], [7.0, 
8.0, 8.0], [2.0, 6.0, 4.0]], [[1.0, 5.0, 6.0], [7.0, 8.0, 3.0], [7.0, 2.0, 2.0]], [[1.0, 3.0, 7.0], [5.0, 3.0, 8.0], [2.0, 3.0, 
3.0]]], [[[3.0, 3.0, 7.0], [5.0, 1.0, 2.0], [2.0, 5.0, 7.0]], [[4.0, 3.0, 3.0], [4.0, 2.0, 7.0], [5.0, 4.0, 3.0]], [[8.0, 2.0, 
8.0], [7.0, 3.0, 1.0], [1.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 1.0], [8.0, 1.0, 8.0], [3.0, 5.0, 4.0]], 
[[1.0, 1.0, 4.0], [8.0, 5.0, 8.0], [3.0, 5.0, 2.0]], [[7.0, 5.0, 5.0], [8.0, 2.0, 5.0], [2.0, 2.0, 7.0]]], [[[8.0, 3.0, 8.0], 
[8.0, 4.0, 3.0], [3.0, 1.0, 2.0]], [[6.0, 4.0, 1.0], [7.0, 4.0, 2.0], [2.0, 8.0, 5.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 8.0], 
[4.0, 1.0, 1.0]]], [[[7.0, 4.0, 2.0], [4.0, 4.0, 5.0], [7.0, 7.0, 8.0]], [[8.0, 5.0, 6.0], [1.0, 2.0, 1.0], [4.0, 4.0, 3.0]], 
[[6.0, 7.0, 2.0], [6.0, 7.0, 1.0], [2.0, 3.0, 3.0]]], [[[2.0, 8.0, 5.0], [1.0, 5.0, 6.0], [7.0, 6.0, 8.0]], [[7.0, 7.0, 8.0], 
[7.0, 5.0, 2.0], [2.0, 3.0, 2.0]], [[8.0, 3.0, 4.0], [8.0, 5.0, 1.0], [5.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 5.0], [8.0, 6.0, 1.0], [7.0, 4.0, 4.0]], [[5.0, 1.0, 5.0], [6.0, 2.0, 3.0], [8.0, 
5.0, 7.0]], [[5.0, 6.0, 7.0], [5.0, 7.0, 7.0], [4.0, 4.0, 7.0]]], [[[3.0, 5.0, 4.0], [2.0, 8.0, 6.0], [4.0, 5.0, 5.0]], [[6.0, 
7.0, 6.0], [6.0, 7.0, 4.0], [6.0, 1.0, 8.0]], [[1.0, 4.0, 8.0], [7.0, 2.0, 7.0], [2.0, 1.0, 3.0]]], [[[4.0, 3.0, 7.0], [1.0, 
6.0, 4.0], [4.0, 7.0, 6.0]], [[2.0, 2.0, 7.0], [7.0, 1.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [3.0, 3.0, 6.0], [7.0, 5.0, 
2.0]]], [[[6.0, 6.0, 3.0], [8.0, 7.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 3.0, 5.0], [5.0, 7.0, 2.0], [3.0, 8.0, 2.0]], [[6.0, 6.0, 
2.0], [3.0, 2.0, 8.0], [5.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 7.0], [8.0, 7.0, 2.0], [3.0, 5.0, 3.0]], 
[[6.0, 1.0, 3.0], [3.0, 1.0, 6.0], [8.0, 1.0, 4.0]], [[4.0, 1.0, 2.0], [3.0, 5.0, 4.0], [4.0, 4.0, 4.0]]], [[[6.0, 3.0, 3.0], 
[4.0, 1.0, 1.0], [5.0, 7.0, 1.0]], [[4.0, 2.0, 6.0], [7.0, 8.0, 6.0], [8.0, 6.0, 8.0]], [[8.0, 7.0, 1.0], [5.0, 2.0, 1.0], 
[3.0, 2.0, 1.0]]], [[[6.0, 8.0, 2.0], [1.0, 5.0, 2.0], [2.0, 6.0, 3.0]], [[7.0, 5.0, 5.0], [7.0, 8.0, 2.0], [5.0, 3.0, 5.0]], 
[[6.0, 3.0, 7.0], [1.0, 4.0, 7.0], [3.0, 8.0, 3.0]]], [[[3.0, 4.0, 2.0], [1.0, 3.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 2.0, 2.0], 
[2.0, 2.0, 3.0], [2.0, 7.0, 4.0]], [[4.0, 3.0, 8.0], [3.0, 5.0, 1.0], [2.0, 3.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 6.0, 8.0]]], [[[4.0, 6.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 1.0, 2.0]]], 
[[[1.0, 1.0, 2.0]]]])
      arg[1:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 5.0], [8.0, 6.0, 1.0], [7.0, 4.0, 4.0]], [[5.0, 1.0, 5.0], [6.0, 2.0, 3.0], [8.0, 
5.0, 7.0]], [[5.0, 6.0, 7.0], [5.0, 7.0, 7.0], [4.0, 4.0, 7.0]]], [[[3.0, 5.0, 4.0], [2.0, 8.0, 6.0], [4.0, 5.0, 5.0]], [[6.0, 
7.0, 6.0], [6.0, 7.0, 4.0], [6.0, 1.0, 8.0]], [[1.0, 4.0, 8.0], [7.0, 2.0, 7.0], [4.0, 6.0, 8.0]]], [[[4.0, 3.0, 7.0], [1.0, 
6.0, 4.0], [4.0, 7.0, 6.0]], [[2.0, 2.0, 7.0], [7.0, 1.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [3.0, 3.0, 6.0], [4.0, 6.0, 
3.0]]], [[[6.0, 6.0, 3.0], [8.0, 7.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 3.0, 5.0], [5.0, 7.0, 2.0], [3.0, 8.0, 2.0]], [[6.0, 6.0, 
2.0], [3.0, 2.0, 8.0], [5.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 7.0], [8.0, 7.0, 2.0], [3.0, 5.0, 3.0]], 
[[6.0, 1.0, 3.0], [3.0, 1.0, 6.0], [8.0, 1.0, 4.0]], [[4.0, 1.0, 2.0], [3.0, 5.0, 4.0], [4.0, 4.0, 4.0]]], [[[6.0, 3.0, 3.0], 
[4.0, 1.0, 1.0], [5.0, 7.0, 1.0]], [[4.0, 2.0, 6.0], [7.0, 8.0, 6.0], [8.0, 6.0, 8.0]], [[8.0, 7.0, 1.0], [5.0, 2.0, 1.0], 
[1.0, 1.0, 2.0]]], [[[6.0, 8.0, 2.0], [1.0, 5.0, 2.0], [2.0, 6.0, 3.0]], [[7.0, 5.0, 5.0], [7.0, 8.0, 2.0], [5.0, 3.0, 5.0]], 
[[6.0, 3.0, 7.0], [1.0, 4.0, 7.0], [1.0, 1.0, 2.0]]], [[[3.0, 4.0, 2.0], [1.0, 3.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 2.0, 2.0], 
[2.0, 2.0, 3.0], [2.0, 7.0, 4.0]], [[4.0, 3.0, 8.0], [3.0, 5.0, 1.0], [2.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 5.0], [1.0, 4.0, 2.0], [7.0, 8.0, 4.0]], [[1.0, 8.0, 7.0], [8.0, 6.0, 3.0], [3.0, 
4.0, 2.0]], [[6.0, 3.0, 6.0], [5.0, 7.0, 3.0], [3.0, 5.0, 7.0]]], [[[4.0, 6.0, 2.0], [1.0, 4.0, 7.0], [5.0, 2.0, 5.0]], [[8.0, 
1.0, 4.0], [4.0, 4.0, 4.0], [8.0, 6.0, 6.0]], [[7.0, 1.0, 6.0], [3.0, 8.0, 3.0], [8.0, 7.0, 1.0]]], [[[6.0, 4.0, 4.0], [8.0, 
8.0, 3.0], [1.0, 1.0, 7.0]], [[2.0, 5.0, 4.0], [2.0, 8.0, 8.0], [4.0, 4.0, 2.0]], [[6.0, 7.0, 4.0], [2.0, 4.0, 2.0], [1.0, 7.0, 
1.0]]], [[[5.0, 3.0, 6.0], [3.0, 8.0, 6.0], [5.0, 5.0, 1.0]], [[6.0, 7.0, 8.0], [6.0, 1.0, 2.0], [5.0, 3.0, 2.0]], [[1.0, 4.0, 
6.0], [1.0, 4.0, 5.0], [5.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 1.0], [2.0, 5.0, 4.0], [7.0, 5.0, 8.0]], 
[[4.0, 4.0, 2.0], [1.0, 3.0, 4.0], [5.0, 7.0, 1.0]], [[1.0, 5.0, 8.0], [1.0, 1.0, 2.0], [2.0, 4.0, 8.0]]], [[[7.0, 2.0, 4.0], 
[8.0, 8.0, 8.0], [5.0, 8.0, 6.0]], [[4.0, 4.0, 7.0], [1.0, 1.0, 8.0], [2.0, 8.0, 8.0]], [[3.0, 2.0, 5.0], [2.0, 6.0, 4.0], 
[7.0, 7.0, 4.0]]], [[[5.0, 2.0, 4.0], [5.0, 6.0, 2.0], [7.0, 2.0, 7.0]], [[4.0, 3.0, 1.0], [8.0, 6.0, 2.0], [5.0, 8.0, 6.0]], 
[[6.0, 1.0, 6.0], [5.0, 3.0, 2.0], [3.0, 1.0, 2.0]]], [[[2.0, 1.0, 4.0], [2.0, 5.0, 2.0], [8.0, 2.0, 6.0]], [[7.0, 8.0, 2.0], 
[2.0, 3.0, 4.0], [4.0, 5.0, 8.0]], [[3.0, 5.0, 3.0], [7.0, 2.0, 3.0], [2.0, 8.0, 6.0]]]])
      val=numpy.array([[[[3.0, 7.0, 7.0]]], [[[2.0, 1.0, 8.0]]], [[[3.0, 8.0, 8.0]]], [[[1.0, 4.0, 1.0]]]])
      arg[:,1:2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 5.0], [1.0, 4.0, 2.0], [7.0, 8.0, 4.0]], [[1.0, 8.0, 7.0], [8.0, 6.0, 3.0], [3.0, 
7.0, 7.0]], [[6.0, 3.0, 6.0], [5.0, 7.0, 3.0], [3.0, 5.0, 7.0]]], [[[4.0, 6.0, 2.0], [1.0, 4.0, 7.0], [5.0, 2.0, 5.0]], [[8.0, 
1.0, 4.0], [4.0, 4.0, 4.0], [2.0, 1.0, 8.0]], [[7.0, 1.0, 6.0], [3.0, 8.0, 3.0], [8.0, 7.0, 1.0]]], [[[6.0, 4.0, 4.0], [8.0, 
8.0, 3.0], [1.0, 1.0, 7.0]], [[2.0, 5.0, 4.0], [2.0, 8.0, 8.0], [3.0, 8.0, 8.0]], [[6.0, 7.0, 4.0], [2.0, 4.0, 2.0], [1.0, 7.0, 
1.0]]], [[[5.0, 3.0, 6.0], [3.0, 8.0, 6.0], [5.0, 5.0, 1.0]], [[6.0, 7.0, 8.0], [6.0, 1.0, 2.0], [1.0, 4.0, 1.0]], [[1.0, 4.0, 
6.0], [1.0, 4.0, 5.0], [5.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 1.0], [2.0, 5.0, 4.0], [7.0, 5.0, 8.0]], 
[[4.0, 4.0, 2.0], [1.0, 3.0, 4.0], [3.0, 7.0, 7.0]], [[1.0, 5.0, 8.0], [1.0, 1.0, 2.0], [2.0, 4.0, 8.0]]], [[[7.0, 2.0, 4.0], 
[8.0, 8.0, 8.0], [5.0, 8.0, 6.0]], [[4.0, 4.0, 7.0], [1.0, 1.0, 8.0], [2.0, 1.0, 8.0]], [[3.0, 2.0, 5.0], [2.0, 6.0, 4.0], 
[7.0, 7.0, 4.0]]], [[[5.0, 2.0, 4.0], [5.0, 6.0, 2.0], [7.0, 2.0, 7.0]], [[4.0, 3.0, 1.0], [8.0, 6.0, 2.0], [3.0, 8.0, 8.0]], 
[[6.0, 1.0, 6.0], [5.0, 3.0, 2.0], [3.0, 1.0, 2.0]]], [[[2.0, 1.0, 4.0], [2.0, 5.0, 2.0], [8.0, 2.0, 6.0]], [[7.0, 8.0, 2.0], 
[2.0, 3.0, 4.0], [1.0, 4.0, 1.0]], [[3.0, 5.0, 3.0], [7.0, 2.0, 3.0], [2.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 6.0], [6.0, 8.0, 8.0], [4.0, 8.0, 7.0]], [[4.0, 2.0, 4.0], [7.0, 2.0, 7.0], [1.0, 
8.0, 5.0]], [[7.0, 8.0, 4.0], [5.0, 1.0, 5.0], [1.0, 8.0, 6.0]]], [[[3.0, 7.0, 7.0], [1.0, 6.0, 5.0], [2.0, 8.0, 5.0]], [[6.0, 
1.0, 6.0], [8.0, 5.0, 8.0], [7.0, 2.0, 8.0]], [[1.0, 6.0, 8.0], [2.0, 7.0, 7.0], [1.0, 4.0, 3.0]]], [[[3.0, 6.0, 5.0], [3.0, 
1.0, 1.0], [5.0, 7.0, 7.0]], [[7.0, 1.0, 8.0], [8.0, 7.0, 8.0], [1.0, 4.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 7.0, 2.0], [5.0, 5.0, 
7.0]]], [[[7.0, 4.0, 6.0], [5.0, 6.0, 4.0], [5.0, 8.0, 7.0]], [[6.0, 1.0, 2.0], [6.0, 7.0, 7.0], [5.0, 4.0, 8.0]], [[8.0, 7.0, 
8.0], [6.0, 7.0, 2.0], [1.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 5.0], [6.0, 5.0, 4.0], [2.0, 2.0, 1.0]], 
[[6.0, 8.0, 7.0], [5.0, 5.0, 7.0], [7.0, 4.0, 3.0]], [[6.0, 1.0, 4.0], [6.0, 6.0, 5.0], [8.0, 3.0, 8.0]]], [[[4.0, 2.0, 2.0], 
[8.0, 4.0, 6.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 3.0], [1.0, 3.0, 3.0]], [[8.0, 7.0, 3.0], [3.0, 2.0, 6.0], 
[5.0, 5.0, 2.0]]], [[[2.0, 1.0, 4.0], [3.0, 6.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 8.0, 8.0], [5.0, 5.0, 4.0]], 
[[7.0, 4.0, 3.0], [1.0, 3.0, 6.0], [6.0, 6.0, 1.0]]], [[[8.0, 8.0, 6.0], [2.0, 6.0, 5.0], [6.0, 4.0, 1.0]], [[4.0, 3.0, 2.0], 
[8.0, 3.0, 1.0], [2.0, 4.0, 6.0]], [[4.0, 2.0, 5.0], [2.0, 4.0, 2.0], [4.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[[4.0, 2.0, 4.0]]], [[[4.0, 4.0, 7.0]]], [[[3.0, 3.0, 2.0]]], [[[5.0, 4.0, 
1.0]]]]),self.functionspace)
      arg[:,1:2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 6.0], [6.0, 8.0, 8.0], [4.0, 8.0, 7.0]], [[4.0, 2.0, 4.0], [7.0, 2.0, 7.0], [4.0, 
2.0, 4.0]], [[7.0, 8.0, 4.0], [5.0, 1.0, 5.0], [1.0, 8.0, 6.0]]], [[[3.0, 7.0, 7.0], [1.0, 6.0, 5.0], [2.0, 8.0, 5.0]], [[6.0, 
1.0, 6.0], [8.0, 5.0, 8.0], [4.0, 4.0, 7.0]], [[1.0, 6.0, 8.0], [2.0, 7.0, 7.0], [1.0, 4.0, 3.0]]], [[[3.0, 6.0, 5.0], [3.0, 
1.0, 1.0], [5.0, 7.0, 7.0]], [[7.0, 1.0, 8.0], [8.0, 7.0, 8.0], [3.0, 3.0, 2.0]], [[7.0, 7.0, 7.0], [1.0, 7.0, 2.0], [5.0, 5.0, 
7.0]]], [[[7.0, 4.0, 6.0], [5.0, 6.0, 4.0], [5.0, 8.0, 7.0]], [[6.0, 1.0, 2.0], [6.0, 7.0, 7.0], [5.0, 4.0, 1.0]], [[8.0, 7.0, 
8.0], [6.0, 7.0, 2.0], [1.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [6.0, 5.0, 4.0], [2.0, 2.0, 1.0]], 
[[6.0, 8.0, 7.0], [5.0, 5.0, 7.0], [4.0, 2.0, 4.0]], [[6.0, 1.0, 4.0], [6.0, 6.0, 5.0], [8.0, 3.0, 8.0]]], [[[4.0, 2.0, 2.0], 
[8.0, 4.0, 6.0], [8.0, 4.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 5.0, 3.0], [4.0, 4.0, 7.0]], [[8.0, 7.0, 3.0], [3.0, 2.0, 6.0], 
[5.0, 5.0, 2.0]]], [[[2.0, 1.0, 4.0], [3.0, 6.0, 6.0], [1.0, 3.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 8.0, 8.0], [3.0, 3.0, 2.0]], 
[[7.0, 4.0, 3.0], [1.0, 3.0, 6.0], [6.0, 6.0, 1.0]]], [[[8.0, 8.0, 6.0], [2.0, 6.0, 5.0], [6.0, 4.0, 1.0]], [[4.0, 3.0, 2.0], 
[8.0, 3.0, 1.0], [5.0, 4.0, 1.0]], [[4.0, 2.0, 5.0], [2.0, 4.0, 2.0], [4.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 2.0], [4.0, 1.0, 6.0], [7.0, 8.0, 2.0]], [[6.0, 4.0, 2.0], [6.0, 6.0, 8.0], [3.0, 
6.0, 7.0]], [[2.0, 1.0, 4.0], [4.0, 3.0, 7.0], [1.0, 1.0, 7.0]]], [[[4.0, 3.0, 5.0], [8.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[3.0, 
8.0, 4.0], [7.0, 3.0, 1.0], [1.0, 7.0, 4.0]], [[2.0, 2.0, 6.0], [2.0, 6.0, 1.0], [8.0, 2.0, 7.0]]], [[[6.0, 4.0, 1.0], [8.0, 
5.0, 3.0], [4.0, 4.0, 2.0]], [[6.0, 5.0, 2.0], [7.0, 5.0, 6.0], [3.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [6.0, 4.0, 3.0], [4.0, 7.0, 
3.0]]], [[[2.0, 4.0, 4.0], [8.0, 7.0, 7.0], [1.0, 5.0, 8.0]], [[1.0, 2.0, 6.0], [4.0, 1.0, 8.0], [3.0, 4.0, 3.0]], [[7.0, 2.0, 
5.0], [8.0, 8.0, 4.0], [5.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 6.0], [5.0, 7.0, 2.0], [3.0, 5.0, 8.0]], 
[[6.0, 3.0, 1.0], [8.0, 5.0, 1.0], [3.0, 8.0, 1.0]], [[5.0, 1.0, 6.0], [8.0, 5.0, 2.0], [4.0, 3.0, 5.0]]], [[[6.0, 8.0, 8.0], 
[7.0, 5.0, 2.0], [7.0, 7.0, 2.0]], [[6.0, 6.0, 7.0], [6.0, 6.0, 4.0], [6.0, 2.0, 8.0]], [[1.0, 7.0, 8.0], [6.0, 5.0, 3.0], 
[5.0, 6.0, 5.0]]], [[[7.0, 8.0, 6.0], [7.0, 5.0, 4.0], [7.0, 2.0, 7.0]], [[3.0, 3.0, 5.0], [6.0, 2.0, 6.0], [7.0, 6.0, 8.0]], 
[[1.0, 3.0, 1.0], [4.0, 7.0, 4.0], [8.0, 8.0, 6.0]]], [[[3.0, 8.0, 2.0], [7.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[4.0, 7.0, 4.0], 
[6.0, 6.0, 6.0], [8.0, 8.0, 7.0]], [[1.0, 2.0, 6.0], [3.0, 4.0, 4.0], [1.0, 5.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 6.0, 4.0]]], [[[5.0, 3.0, 6.0]]], [[[2.0, 2.0, 5.0]]], [[[6.0, 2.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 6.0, 3.0]]], [[[3.0, 4.0, 8.0]]], [[[6.0, 7.0, 6.0]]], [[[1.0, 2.0, 8.0]]]])
      arg[:,1:2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 2.0], [4.0, 1.0, 6.0], [7.0, 8.0, 2.0]], [[6.0, 4.0, 2.0], [6.0, 6.0, 8.0], [2.0, 
6.0, 4.0]], [[2.0, 1.0, 4.0], [4.0, 3.0, 7.0], [1.0, 1.0, 7.0]]], [[[4.0, 3.0, 5.0], [8.0, 8.0, 6.0], [3.0, 1.0, 2.0]], [[3.0, 
8.0, 4.0], [7.0, 3.0, 1.0], [5.0, 3.0, 6.0]], [[2.0, 2.0, 6.0], [2.0, 6.0, 1.0], [8.0, 2.0, 7.0]]], [[[6.0, 4.0, 1.0], [8.0, 
5.0, 3.0], [4.0, 4.0, 2.0]], [[6.0, 5.0, 2.0], [7.0, 5.0, 6.0], [2.0, 2.0, 5.0]], [[7.0, 3.0, 5.0], [6.0, 4.0, 3.0], [4.0, 7.0, 
3.0]]], [[[2.0, 4.0, 4.0], [8.0, 7.0, 7.0], [1.0, 5.0, 8.0]], [[1.0, 2.0, 6.0], [4.0, 1.0, 8.0], [6.0, 2.0, 1.0]], [[7.0, 2.0, 
5.0], [8.0, 8.0, 4.0], [5.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 6.0], [5.0, 7.0, 2.0], [3.0, 5.0, 8.0]], 
[[6.0, 3.0, 1.0], [8.0, 5.0, 1.0], [6.0, 6.0, 3.0]], [[5.0, 1.0, 6.0], [8.0, 5.0, 2.0], [4.0, 3.0, 5.0]]], [[[6.0, 8.0, 8.0], 
[7.0, 5.0, 2.0], [7.0, 7.0, 2.0]], [[6.0, 6.0, 7.0], [6.0, 6.0, 4.0], [3.0, 4.0, 8.0]], [[1.0, 7.0, 8.0], [6.0, 5.0, 3.0], 
[5.0, 6.0, 5.0]]], [[[7.0, 8.0, 6.0], [7.0, 5.0, 4.0], [7.0, 2.0, 7.0]], [[3.0, 3.0, 5.0], [6.0, 2.0, 6.0], [6.0, 7.0, 6.0]], 
[[1.0, 3.0, 1.0], [4.0, 7.0, 4.0], [8.0, 8.0, 6.0]]], [[[3.0, 8.0, 2.0], [7.0, 6.0, 7.0], [3.0, 7.0, 2.0]], [[4.0, 7.0, 4.0], 
[6.0, 6.0, 6.0], [1.0, 2.0, 8.0]], [[1.0, 2.0, 6.0], [3.0, 4.0, 4.0], [1.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 7.0], [1.0, 7.0, 1.0], [3.0, 5.0, 1.0]], [[4.0, 3.0, 2.0], [1.0, 7.0, 5.0], [4.0, 
8.0, 1.0]], [[7.0, 6.0, 8.0], [3.0, 7.0, 6.0], [6.0, 5.0, 1.0]]], [[[2.0, 1.0, 1.0], [5.0, 8.0, 1.0], [7.0, 7.0, 5.0]], [[3.0, 
2.0, 2.0], [4.0, 1.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 1.0, 6.0], [6.0, 2.0, 5.0], [7.0, 2.0, 1.0]]], [[[5.0, 7.0, 4.0], [2.0, 
8.0, 8.0], [4.0, 3.0, 3.0]], [[7.0, 6.0, 5.0], [3.0, 6.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 8.0, 1.0], [2.0, 6.0, 5.0], [7.0, 6.0, 
5.0]]], [[[5.0, 2.0, 2.0], [6.0, 5.0, 7.0], [7.0, 7.0, 7.0]], [[6.0, 1.0, 3.0], [3.0, 3.0, 5.0], [5.0, 7.0, 7.0]], [[6.0, 1.0, 
2.0], [7.0, 8.0, 8.0], [8.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 4.0], [3.0, 6.0, 8.0], [2.0, 4.0, 8.0]], 
[[6.0, 1.0, 4.0], [7.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[8.0, 1.0, 2.0], [6.0, 1.0, 8.0], [4.0, 3.0, 5.0]]], [[[5.0, 5.0, 6.0], 
[7.0, 6.0, 6.0], [6.0, 5.0, 5.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 4.0], [4.0, 7.0, 2.0]], [[7.0, 6.0, 2.0], [7.0, 3.0, 8.0], 
[3.0, 4.0, 4.0]]], [[[5.0, 5.0, 8.0], [4.0, 7.0, 4.0], [3.0, 3.0, 6.0]], [[6.0, 6.0, 2.0], [7.0, 3.0, 6.0], [5.0, 3.0, 6.0]], 
[[3.0, 6.0, 2.0], [3.0, 5.0, 3.0], [5.0, 1.0, 6.0]]], [[[6.0, 3.0, 1.0], [5.0, 2.0, 4.0], [8.0, 1.0, 7.0]], [[5.0, 2.0, 6.0], 
[7.0, 5.0, 6.0], [3.0, 5.0, 2.0]], [[4.0, 4.0, 4.0], [3.0, 2.0, 7.0], [4.0, 5.0, 3.0]]]])
      val=numpy.array([[[7.0], [5.0]], [[7.0], [7.0]]])
      arg[2:,1,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 7.0], [1.0, 7.0, 1.0], [3.0, 5.0, 1.0]], [[4.0, 3.0, 2.0], [1.0, 7.0, 5.0], [4.0, 
8.0, 1.0]], [[7.0, 6.0, 8.0], [3.0, 7.0, 6.0], [6.0, 5.0, 1.0]]], [[[2.0, 1.0, 1.0], [5.0, 8.0, 1.0], [7.0, 7.0, 5.0]], [[3.0, 
2.0, 2.0], [4.0, 1.0, 4.0], [7.0, 6.0, 5.0]], [[8.0, 1.0, 6.0], [6.0, 2.0, 5.0], [7.0, 2.0, 1.0]]], [[[5.0, 7.0, 4.0], [2.0, 
8.0, 8.0], [4.0, 3.0, 3.0]], [[7.0, 7.0, 5.0], [3.0, 5.0, 6.0], [2.0, 3.0, 7.0]], [[5.0, 8.0, 1.0], [2.0, 6.0, 5.0], [7.0, 6.0, 
5.0]]], [[[5.0, 2.0, 2.0], [6.0, 5.0, 7.0], [7.0, 7.0, 7.0]], [[6.0, 7.0, 3.0], [3.0, 7.0, 5.0], [5.0, 7.0, 7.0]], [[6.0, 1.0, 
2.0], [7.0, 8.0, 8.0], [8.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 4.0], [3.0, 6.0, 8.0], [2.0, 4.0, 8.0]], 
[[6.0, 1.0, 4.0], [7.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[8.0, 1.0, 2.0], [6.0, 1.0, 8.0], [4.0, 3.0, 5.0]]], [[[5.0, 5.0, 6.0], 
[7.0, 6.0, 6.0], [6.0, 5.0, 5.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 4.0], [4.0, 7.0, 2.0]], [[7.0, 6.0, 2.0], [7.0, 3.0, 8.0], 
[3.0, 4.0, 4.0]]], [[[5.0, 5.0, 8.0], [4.0, 7.0, 4.0], [3.0, 3.0, 6.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 6.0], [5.0, 3.0, 6.0]], 
[[3.0, 6.0, 2.0], [3.0, 5.0, 3.0], [5.0, 1.0, 6.0]]], [[[6.0, 3.0, 1.0], [5.0, 2.0, 4.0], [8.0, 1.0, 7.0]], [[5.0, 7.0, 6.0], 
[7.0, 7.0, 6.0], [3.0, 5.0, 2.0]], [[4.0, 4.0, 4.0], [3.0, 2.0, 7.0], [4.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 7.0], [7.0, 2.0, 4.0], [4.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [8.0, 8.0, 8.0], [4.0, 
2.0, 7.0]], [[4.0, 6.0, 2.0], [2.0, 2.0, 3.0], [2.0, 2.0, 1.0]]], [[[6.0, 4.0, 1.0], [1.0, 5.0, 4.0], [2.0, 6.0, 7.0]], [[1.0, 
3.0, 6.0], [3.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[1.0, 6.0, 7.0], [4.0, 6.0, 4.0], [4.0, 8.0, 6.0]]], [[[8.0, 1.0, 2.0], [4.0, 
6.0, 6.0], [2.0, 6.0, 1.0]], [[7.0, 8.0, 6.0], [5.0, 3.0, 7.0], [8.0, 7.0, 6.0]], [[3.0, 7.0, 6.0], [3.0, 5.0, 8.0], [1.0, 8.0, 
4.0]]], [[[3.0, 1.0, 5.0], [4.0, 4.0, 7.0], [1.0, 7.0, 8.0]], [[7.0, 4.0, 3.0], [6.0, 4.0, 2.0], [4.0, 7.0, 5.0]], [[1.0, 4.0, 
4.0], [8.0, 7.0, 2.0], [7.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 6.0], [6.0, 7.0, 3.0], [1.0, 6.0, 6.0]], 
[[7.0, 2.0, 8.0], [8.0, 4.0, 5.0], [4.0, 5.0, 6.0]], [[8.0, 2.0, 4.0], [3.0, 6.0, 6.0], [1.0, 4.0, 5.0]]], [[[7.0, 1.0, 7.0], 
[4.0, 3.0, 1.0], [3.0, 4.0, 7.0]], [[3.0, 7.0, 7.0], [1.0, 8.0, 7.0], [7.0, 4.0, 7.0]], [[6.0, 4.0, 8.0], [5.0, 8.0, 6.0], 
[3.0, 8.0, 5.0]]], [[[4.0, 6.0, 7.0], [8.0, 8.0, 2.0], [4.0, 6.0, 3.0]], [[5.0, 3.0, 2.0], [5.0, 6.0, 3.0], [1.0, 2.0, 2.0]], 
[[5.0, 1.0, 4.0], [7.0, 4.0, 3.0], [5.0, 7.0, 4.0]]], [[[4.0, 4.0, 8.0], [5.0, 6.0, 1.0], [8.0, 4.0, 2.0]], [[1.0, 3.0, 5.0], 
[3.0, 1.0, 1.0], [6.0, 7.0, 4.0]], [[5.0, 1.0, 5.0], [1.0, 6.0, 7.0], [2.0, 1.0, 4.0]]]])
      val=Data(numpy.array([[[6.0], [8.0]], [[1.0], [2.0]]]),self.functionspace)
      arg[2:,1,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 7.0], [7.0, 2.0, 4.0], [4.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [8.0, 8.0, 8.0], [4.0, 
2.0, 7.0]], [[4.0, 6.0, 2.0], [2.0, 2.0, 3.0], [2.0, 2.0, 1.0]]], [[[6.0, 4.0, 1.0], [1.0, 5.0, 4.0], [2.0, 6.0, 7.0]], [[1.0, 
3.0, 6.0], [3.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[1.0, 6.0, 7.0], [4.0, 6.0, 4.0], [4.0, 8.0, 6.0]]], [[[8.0, 1.0, 2.0], [4.0, 
6.0, 6.0], [2.0, 6.0, 1.0]], [[7.0, 6.0, 6.0], [5.0, 8.0, 7.0], [8.0, 7.0, 6.0]], [[3.0, 7.0, 6.0], [3.0, 5.0, 8.0], [1.0, 8.0, 
4.0]]], [[[3.0, 1.0, 5.0], [4.0, 4.0, 7.0], [1.0, 7.0, 8.0]], [[7.0, 1.0, 3.0], [6.0, 2.0, 2.0], [4.0, 7.0, 5.0]], [[1.0, 4.0, 
4.0], [8.0, 7.0, 2.0], [7.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 6.0], [6.0, 7.0, 3.0], [1.0, 6.0, 6.0]], 
[[7.0, 2.0, 8.0], [8.0, 4.0, 5.0], [4.0, 5.0, 6.0]], [[8.0, 2.0, 4.0], [3.0, 6.0, 6.0], [1.0, 4.0, 5.0]]], [[[7.0, 1.0, 7.0], 
[4.0, 3.0, 1.0], [3.0, 4.0, 7.0]], [[3.0, 7.0, 7.0], [1.0, 8.0, 7.0], [7.0, 4.0, 7.0]], [[6.0, 4.0, 8.0], [5.0, 8.0, 6.0], 
[3.0, 8.0, 5.0]]], [[[4.0, 6.0, 7.0], [8.0, 8.0, 2.0], [4.0, 6.0, 3.0]], [[5.0, 6.0, 2.0], [5.0, 8.0, 3.0], [1.0, 2.0, 2.0]], 
[[5.0, 1.0, 4.0], [7.0, 4.0, 3.0], [5.0, 7.0, 4.0]]], [[[4.0, 4.0, 8.0], [5.0, 6.0, 1.0], [8.0, 4.0, 2.0]], [[1.0, 1.0, 5.0], 
[3.0, 2.0, 1.0], [6.0, 7.0, 4.0]], [[5.0, 1.0, 5.0], [1.0, 6.0, 7.0], [2.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 7.0], [5.0, 1.0, 6.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 2.0, 2.0], [6.0, 
2.0, 2.0]], [[3.0, 6.0, 8.0], [8.0, 4.0, 6.0], [5.0, 7.0, 5.0]]], [[[2.0, 1.0, 6.0], [4.0, 1.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 
6.0, 6.0], [5.0, 3.0, 6.0], [5.0, 6.0, 3.0]], [[1.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 6.0, 2.0]]], [[[8.0, 1.0, 2.0], [3.0, 
8.0, 5.0], [3.0, 2.0, 1.0]], [[3.0, 5.0, 5.0], [2.0, 4.0, 6.0], [6.0, 5.0, 7.0]], [[8.0, 8.0, 3.0], [2.0, 5.0, 6.0], [1.0, 1.0, 
6.0]]], [[[4.0, 5.0, 8.0], [1.0, 8.0, 1.0], [2.0, 8.0, 7.0]], [[6.0, 3.0, 7.0], [6.0, 2.0, 1.0], [4.0, 8.0, 7.0]], [[5.0, 2.0, 
6.0], [8.0, 3.0, 8.0], [5.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 2.0], [2.0, 1.0, 8.0], [8.0, 5.0, 5.0]], 
[[3.0, 8.0, 7.0], [8.0, 7.0, 6.0], [8.0, 1.0, 3.0]], [[2.0, 6.0, 2.0], [3.0, 5.0, 2.0], [7.0, 8.0, 8.0]]], [[[7.0, 4.0, 2.0], 
[1.0, 3.0, 1.0], [2.0, 8.0, 1.0]], [[4.0, 8.0, 7.0], [7.0, 3.0, 6.0], [8.0, 4.0, 7.0]], [[7.0, 6.0, 1.0], [6.0, 8.0, 3.0], 
[8.0, 4.0, 7.0]]], [[[5.0, 1.0, 2.0], [2.0, 6.0, 1.0], [7.0, 2.0, 7.0]], [[5.0, 2.0, 8.0], [3.0, 4.0, 5.0], [6.0, 2.0, 5.0]], 
[[2.0, 2.0, 8.0], [6.0, 4.0, 6.0], [1.0, 2.0, 2.0]]], [[[2.0, 1.0, 3.0], [5.0, 1.0, 8.0], [2.0, 2.0, 2.0]], [[6.0, 4.0, 1.0], 
[5.0, 7.0, 7.0], [8.0, 2.0, 3.0]], [[6.0, 7.0, 8.0], [7.0, 3.0, 8.0], [5.0, 7.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0], [4.0]], [[6.0], [7.0]]])+(1.-msk_val)*numpy.array([[[4.0], [6.0]], [[8.0], [8.0]]])
      arg[2:,1,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 7.0], [5.0, 1.0, 6.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 5.0], [7.0, 2.0, 2.0], [6.0, 
2.0, 2.0]], [[3.0, 6.0, 8.0], [8.0, 4.0, 6.0], [5.0, 7.0, 5.0]]], [[[2.0, 1.0, 6.0], [4.0, 1.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 
6.0, 6.0], [5.0, 3.0, 6.0], [5.0, 6.0, 3.0]], [[1.0, 5.0, 5.0], [7.0, 3.0, 7.0], [1.0, 6.0, 2.0]]], [[[8.0, 1.0, 2.0], [3.0, 
8.0, 5.0], [3.0, 2.0, 1.0]], [[3.0, 8.0, 5.0], [2.0, 4.0, 6.0], [6.0, 5.0, 7.0]], [[8.0, 8.0, 3.0], [2.0, 5.0, 6.0], [1.0, 1.0, 
6.0]]], [[[4.0, 5.0, 8.0], [1.0, 8.0, 1.0], [2.0, 8.0, 7.0]], [[6.0, 6.0, 7.0], [6.0, 7.0, 1.0], [4.0, 8.0, 7.0]], [[5.0, 2.0, 
6.0], [8.0, 3.0, 8.0], [5.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 2.0], [2.0, 1.0, 8.0], [8.0, 5.0, 5.0]], 
[[3.0, 8.0, 7.0], [8.0, 7.0, 6.0], [8.0, 1.0, 3.0]], [[2.0, 6.0, 2.0], [3.0, 5.0, 2.0], [7.0, 8.0, 8.0]]], [[[7.0, 4.0, 2.0], 
[1.0, 3.0, 1.0], [2.0, 8.0, 1.0]], [[4.0, 8.0, 7.0], [7.0, 3.0, 6.0], [8.0, 4.0, 7.0]], [[7.0, 6.0, 1.0], [6.0, 8.0, 3.0], 
[8.0, 4.0, 7.0]]], [[[5.0, 1.0, 2.0], [2.0, 6.0, 1.0], [7.0, 2.0, 7.0]], [[5.0, 4.0, 8.0], [3.0, 6.0, 5.0], [6.0, 2.0, 5.0]], 
[[2.0, 2.0, 8.0], [6.0, 4.0, 6.0], [1.0, 2.0, 2.0]]], [[[2.0, 1.0, 3.0], [5.0, 1.0, 8.0], [2.0, 2.0, 2.0]], [[6.0, 8.0, 1.0], 
[5.0, 8.0, 7.0], [8.0, 2.0, 3.0]], [[6.0, 7.0, 8.0], [7.0, 3.0, 8.0], [5.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 2.0], [8.0, 2.0, 3.0], [7.0, 8.0, 6.0]], [[4.0, 7.0, 5.0], [4.0, 8.0, 1.0], [1.0, 
8.0, 7.0]], [[4.0, 2.0, 7.0], [6.0, 1.0, 3.0], [4.0, 3.0, 3.0]]], [[[2.0, 7.0, 1.0], [1.0, 4.0, 4.0], [3.0, 8.0, 4.0]], [[4.0, 
7.0, 2.0], [2.0, 3.0, 1.0], [7.0, 4.0, 2.0]], [[6.0, 7.0, 7.0], [6.0, 6.0, 1.0], [2.0, 8.0, 7.0]]], [[[3.0, 7.0, 3.0], [6.0, 
3.0, 6.0], [5.0, 2.0, 3.0]], [[5.0, 4.0, 7.0], [5.0, 4.0, 4.0], [4.0, 2.0, 8.0]], [[8.0, 4.0, 3.0], [7.0, 5.0, 7.0], [3.0, 7.0, 
5.0]]], [[[8.0, 2.0, 3.0], [2.0, 8.0, 7.0], [6.0, 4.0, 7.0]], [[8.0, 5.0, 2.0], [3.0, 2.0, 8.0], [3.0, 8.0, 7.0]], [[7.0, 4.0, 
6.0], [4.0, 2.0, 8.0], [5.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 7.0], [2.0, 7.0, 6.0], [5.0, 8.0, 6.0]], 
[[5.0, 8.0, 3.0], [3.0, 3.0, 2.0], [7.0, 8.0, 3.0]], [[8.0, 7.0, 6.0], [3.0, 7.0, 4.0], [3.0, 8.0, 5.0]]], [[[4.0, 4.0, 3.0], 
[8.0, 6.0, 3.0], [7.0, 3.0, 6.0]], [[2.0, 1.0, 4.0], [3.0, 2.0, 2.0], [1.0, 6.0, 2.0]], [[6.0, 4.0, 4.0], [3.0, 7.0, 1.0], 
[7.0, 6.0, 2.0]]], [[[8.0, 5.0, 2.0], [2.0, 5.0, 8.0], [4.0, 2.0, 7.0]], [[3.0, 1.0, 7.0], [7.0, 3.0, 4.0], [5.0, 1.0, 6.0]], 
[[5.0, 6.0, 4.0], [8.0, 4.0, 6.0], [8.0, 4.0, 4.0]]], [[[3.0, 2.0, 1.0], [2.0, 3.0, 4.0], [6.0, 6.0, 4.0]], [[3.0, 5.0, 8.0], 
[6.0, 2.0, 2.0], [1.0, 1.0, 1.0]], [[7.0, 1.0, 1.0], [4.0, 4.0, 6.0], [7.0, 2.0, 8.0]]]])
      val=numpy.array([[[[2.0, 1.0, 7.0], [4.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[8.0, 5.0, 8.0], [4.0, 2.0, 2.0], [8.0, 2.0, 
8.0]], [[6.0, 4.0, 5.0], [1.0, 8.0, 4.0], [8.0, 7.0, 4.0]]], [[[5.0, 8.0, 2.0], [7.0, 3.0, 7.0], [5.0, 1.0, 5.0]], [[2.0, 7.0, 
5.0], [7.0, 5.0, 6.0], [8.0, 8.0, 7.0]], [[1.0, 5.0, 4.0], [1.0, 8.0, 8.0], [6.0, 8.0, 7.0]]], [[[5.0, 7.0, 8.0], [5.0, 8.0, 
4.0], [1.0, 5.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 8.0, 2.0], [8.0, 3.0, 5.0]], [[6.0, 7.0, 7.0], [1.0, 6.0, 7.0], [5.0, 5.0, 
5.0]]]])
      arg[:3,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 7.0], [4.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[8.0, 5.0, 8.0], [4.0, 2.0, 2.0], [8.0, 
2.0, 8.0]], [[6.0, 4.0, 5.0], [1.0, 8.0, 4.0], [8.0, 7.0, 4.0]]], [[[5.0, 8.0, 2.0], [7.0, 3.0, 7.0], [5.0, 1.0, 5.0]], [[2.0, 
7.0, 5.0], [7.0, 5.0, 6.0], [8.0, 8.0, 7.0]], [[1.0, 5.0, 4.0], [1.0, 8.0, 8.0], [6.0, 8.0, 7.0]]], [[[5.0, 7.0, 8.0], [5.0, 
8.0, 4.0], [1.0, 5.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 8.0, 2.0], [8.0, 3.0, 5.0]], [[6.0, 7.0, 7.0], [1.0, 6.0, 7.0], [5.0, 5.0, 
5.0]]], [[[8.0, 2.0, 3.0], [2.0, 8.0, 7.0], [6.0, 4.0, 7.0]], [[8.0, 5.0, 2.0], [3.0, 2.0, 8.0], [3.0, 8.0, 7.0]], [[7.0, 4.0, 
6.0], [4.0, 2.0, 8.0], [5.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [4.0, 7.0, 1.0], [2.0, 1.0, 8.0]], 
[[8.0, 5.0, 8.0], [4.0, 2.0, 2.0], [8.0, 2.0, 8.0]], [[6.0, 4.0, 5.0], [1.0, 8.0, 4.0], [8.0, 7.0, 4.0]]], [[[5.0, 8.0, 2.0], 
[7.0, 3.0, 7.0], [5.0, 1.0, 5.0]], [[2.0, 7.0, 5.0], [7.0, 5.0, 6.0], [8.0, 8.0, 7.0]], [[1.0, 5.0, 4.0], [1.0, 8.0, 8.0], 
[6.0, 8.0, 7.0]]], [[[5.0, 7.0, 8.0], [5.0, 8.0, 4.0], [1.0, 5.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 8.0, 2.0], [8.0, 3.0, 5.0]], 
[[6.0, 7.0, 7.0], [1.0, 6.0, 7.0], [5.0, 5.0, 5.0]]], [[[3.0, 2.0, 1.0], [2.0, 3.0, 4.0], [6.0, 6.0, 4.0]], [[3.0, 5.0, 8.0], 
[6.0, 2.0, 2.0], [1.0, 1.0, 1.0]], [[7.0, 1.0, 1.0], [4.0, 4.0, 6.0], [7.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 2.0], [2.0, 2.0, 7.0], [6.0, 7.0, 6.0]], [[6.0, 4.0, 3.0], [8.0, 5.0, 6.0], [4.0, 
2.0, 3.0]], [[7.0, 2.0, 2.0], [5.0, 8.0, 8.0], [1.0, 3.0, 7.0]]], [[[6.0, 2.0, 4.0], [5.0, 4.0, 5.0], [4.0, 4.0, 7.0]], [[2.0, 
4.0, 6.0], [6.0, 5.0, 8.0], [4.0, 1.0, 6.0]], [[4.0, 7.0, 2.0], [1.0, 7.0, 6.0], [7.0, 6.0, 6.0]]], [[[6.0, 1.0, 6.0], [8.0, 
5.0, 6.0], [3.0, 2.0, 4.0]], [[2.0, 7.0, 5.0], [7.0, 3.0, 5.0], [3.0, 6.0, 5.0]], [[8.0, 7.0, 7.0], [5.0, 6.0, 6.0], [6.0, 6.0, 
8.0]]], [[[8.0, 7.0, 5.0], [7.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[6.0, 6.0, 1.0], [4.0, 6.0, 8.0], [5.0, 7.0, 4.0]], [[3.0, 7.0, 
6.0], [2.0, 1.0, 3.0], [6.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 6.0], [7.0, 5.0, 2.0], [6.0, 4.0, 8.0]], 
[[4.0, 6.0, 5.0], [1.0, 6.0, 4.0], [5.0, 6.0, 3.0]], [[6.0, 8.0, 7.0], [8.0, 4.0, 2.0], [2.0, 8.0, 2.0]]], [[[5.0, 8.0, 2.0], 
[3.0, 2.0, 5.0], [6.0, 4.0, 2.0]], [[4.0, 7.0, 6.0], [6.0, 2.0, 1.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 5.0], [7.0, 8.0, 4.0], 
[6.0, 6.0, 1.0]]], [[[1.0, 3.0, 1.0], [4.0, 7.0, 3.0], [1.0, 3.0, 4.0]], [[8.0, 6.0, 7.0], [7.0, 5.0, 5.0], [1.0, 5.0, 6.0]], 
[[5.0, 3.0, 4.0], [7.0, 2.0, 2.0], [4.0, 7.0, 5.0]]], [[[6.0, 6.0, 8.0], [1.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], 
[6.0, 1.0, 5.0], [6.0, 3.0, 4.0]], [[7.0, 2.0, 7.0], [3.0, 1.0, 2.0], [7.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[[8.0, 8.0, 5.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 4.0], [8.0, 4.0, 4.0], [1.0, 7.0, 
5.0]], [[1.0, 2.0, 7.0], [3.0, 5.0, 5.0], [1.0, 8.0, 6.0]]], [[[4.0, 3.0, 7.0], [1.0, 5.0, 5.0], [8.0, 2.0, 5.0]], [[4.0, 8.0, 
3.0], [7.0, 4.0, 3.0], [3.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [1.0, 1.0, 1.0], [5.0, 5.0, 7.0]]], [[[7.0, 8.0, 5.0], [6.0, 3.0, 
6.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [3.0, 8.0, 4.0], [4.0, 5.0, 3.0]], [[7.0, 2.0, 1.0], [6.0, 4.0, 5.0], [3.0, 1.0, 
3.0]]]]),self.functionspace)
      arg[:3,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 4.0], [8.0, 4.0, 4.0], [1.0, 
7.0, 5.0]], [[1.0, 2.0, 7.0], [3.0, 5.0, 5.0], [1.0, 8.0, 6.0]]], [[[4.0, 3.0, 7.0], [1.0, 5.0, 5.0], [8.0, 2.0, 5.0]], [[4.0, 
8.0, 3.0], [7.0, 4.0, 3.0], [3.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [1.0, 1.0, 1.0], [5.0, 5.0, 7.0]]], [[[7.0, 8.0, 5.0], [6.0, 
3.0, 6.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [3.0, 8.0, 4.0], [4.0, 5.0, 3.0]], [[7.0, 2.0, 1.0], [6.0, 4.0, 5.0], [3.0, 1.0, 
3.0]]], [[[8.0, 7.0, 5.0], [7.0, 2.0, 5.0], [5.0, 6.0, 6.0]], [[6.0, 6.0, 1.0], [4.0, 6.0, 8.0], [5.0, 7.0, 4.0]], [[3.0, 7.0, 
6.0], [2.0, 1.0, 3.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]], 
[[4.0, 5.0, 4.0], [8.0, 4.0, 4.0], [1.0, 7.0, 5.0]], [[1.0, 2.0, 7.0], [3.0, 5.0, 5.0], [1.0, 8.0, 6.0]]], [[[4.0, 3.0, 7.0], 
[1.0, 5.0, 5.0], [8.0, 2.0, 5.0]], [[4.0, 8.0, 3.0], [7.0, 4.0, 3.0], [3.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [1.0, 1.0, 1.0], 
[5.0, 5.0, 7.0]]], [[[7.0, 8.0, 5.0], [6.0, 3.0, 6.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 3.0], [3.0, 8.0, 4.0], [4.0, 5.0, 3.0]], 
[[7.0, 2.0, 1.0], [6.0, 4.0, 5.0], [3.0, 1.0, 3.0]]], [[[6.0, 6.0, 8.0], [1.0, 2.0, 5.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 6.0], 
[6.0, 1.0, 5.0], [6.0, 3.0, 4.0]], [[7.0, 2.0, 7.0], [3.0, 1.0, 2.0], [7.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 5.0], [6.0, 3.0, 6.0], [7.0, 8.0, 4.0]], [[3.0, 4.0, 4.0], [5.0, 3.0, 4.0], [7.0, 
2.0, 4.0]], [[7.0, 7.0, 8.0], [2.0, 4.0, 6.0], [5.0, 4.0, 8.0]]], [[[3.0, 3.0, 2.0], [5.0, 7.0, 3.0], [1.0, 6.0, 3.0]], [[5.0, 
3.0, 8.0], [1.0, 8.0, 7.0], [2.0, 2.0, 1.0]], [[5.0, 7.0, 3.0], [8.0, 3.0, 2.0], [8.0, 3.0, 3.0]]], [[[7.0, 2.0, 8.0], [7.0, 
8.0, 3.0], [4.0, 6.0, 8.0]], [[5.0, 4.0, 3.0], [6.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[6.0, 5.0, 8.0], [8.0, 5.0, 7.0], [5.0, 1.0, 
3.0]]], [[[7.0, 2.0, 1.0], [4.0, 5.0, 6.0], [4.0, 7.0, 6.0]], [[4.0, 8.0, 6.0], [8.0, 3.0, 2.0], [5.0, 6.0, 3.0]], [[4.0, 6.0, 
5.0], [8.0, 7.0, 6.0], [1.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 1.0, 1.0], [6.0, 8.0, 5.0], [1.0, 4.0, 3.0]], 
[[2.0, 3.0, 7.0], [1.0, 6.0, 8.0], [6.0, 3.0, 1.0]], [[6.0, 7.0, 2.0], [7.0, 6.0, 3.0], [2.0, 4.0, 5.0]]], [[[7.0, 7.0, 7.0], 
[6.0, 4.0, 7.0], [2.0, 8.0, 8.0]], [[1.0, 1.0, 5.0], [2.0, 7.0, 5.0], [8.0, 3.0, 7.0]], [[1.0, 1.0, 6.0], [1.0, 4.0, 8.0], 
[4.0, 7.0, 1.0]]], [[[2.0, 4.0, 2.0], [4.0, 4.0, 2.0], [3.0, 5.0, 3.0]], [[3.0, 8.0, 6.0], [2.0, 7.0, 5.0], [4.0, 1.0, 3.0]], 
[[2.0, 4.0, 7.0], [3.0, 6.0, 2.0], [2.0, 8.0, 6.0]]], [[[5.0, 3.0, 8.0], [8.0, 6.0, 4.0], [1.0, 4.0, 3.0]], [[2.0, 5.0, 6.0], 
[7.0, 2.0, 8.0], [7.0, 6.0, 6.0]], [[2.0, 1.0, 1.0], [5.0, 8.0, 8.0], [8.0, 1.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 1.0, 1.0], [6.0, 4.0, 4.0], [2.0, 8.0, 8.0]], [[5.0, 7.0, 1.0], [7.0, 1.0, 8.0], [7.0, 
2.0, 6.0]], [[7.0, 6.0, 5.0], [7.0, 6.0, 2.0], [2.0, 1.0, 8.0]]], [[[3.0, 6.0, 1.0], [1.0, 2.0, 4.0], [4.0, 3.0, 8.0]], [[2.0, 
7.0, 4.0], [1.0, 5.0, 2.0], [4.0, 7.0, 3.0]], [[4.0, 7.0, 2.0], [4.0, 5.0, 7.0], [8.0, 4.0, 1.0]]], [[[3.0, 6.0, 2.0], [1.0, 
8.0, 5.0], [4.0, 5.0, 8.0]], [[5.0, 3.0, 4.0], [4.0, 3.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 7.0, 2.0], [3.0, 5.0, 4.0], [6.0, 7.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 1.0, 2.0], [8.0, 5.0, 3.0], [6.0, 8.0, 5.0]], [[2.0, 1.0, 2.0], [1.0, 1.0, 1.0], 
[3.0, 2.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 2.0], [8.0, 8.0, 1.0]]], [[[1.0, 4.0, 5.0], [7.0, 2.0, 1.0], [5.0, 2.0, 5.0]], 
[[8.0, 3.0, 2.0], [8.0, 4.0, 6.0], [7.0, 4.0, 8.0]], [[6.0, 4.0, 4.0], [5.0, 5.0, 1.0], [4.0, 8.0, 8.0]]], [[[8.0, 5.0, 2.0], 
[8.0, 6.0, 7.0], [6.0, 7.0, 7.0]], [[1.0, 3.0, 7.0], [3.0, 4.0, 1.0], [5.0, 5.0, 1.0]], [[8.0, 6.0, 2.0], [4.0, 8.0, 2.0], 
[5.0, 7.0, 2.0]]]])
      arg[:3,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 1.0], [6.0, 4.0, 4.0], [2.0, 8.0, 8.0]], [[5.0, 7.0, 1.0], [7.0, 1.0, 8.0], [7.0, 
2.0, 6.0]], [[7.0, 6.0, 5.0], [7.0, 6.0, 2.0], [2.0, 1.0, 8.0]]], [[[3.0, 6.0, 1.0], [1.0, 2.0, 4.0], [4.0, 3.0, 8.0]], [[2.0, 
7.0, 4.0], [1.0, 5.0, 2.0], [4.0, 7.0, 3.0]], [[4.0, 7.0, 2.0], [4.0, 5.0, 7.0], [8.0, 4.0, 1.0]]], [[[3.0, 6.0, 2.0], [1.0, 
8.0, 5.0], [4.0, 5.0, 8.0]], [[5.0, 3.0, 4.0], [4.0, 3.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 7.0, 2.0], [3.0, 5.0, 4.0], [6.0, 7.0, 
2.0]]], [[[7.0, 2.0, 1.0], [4.0, 5.0, 6.0], [4.0, 7.0, 6.0]], [[4.0, 8.0, 6.0], [8.0, 3.0, 2.0], [5.0, 6.0, 3.0]], [[4.0, 6.0, 
5.0], [8.0, 7.0, 6.0], [1.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 2.0], [8.0, 5.0, 3.0], [6.0, 8.0, 5.0]], 
[[2.0, 1.0, 2.0], [1.0, 1.0, 1.0], [3.0, 2.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 2.0], [8.0, 8.0, 1.0]]], [[[1.0, 4.0, 5.0], 
[7.0, 2.0, 1.0], [5.0, 2.0, 5.0]], [[8.0, 3.0, 2.0], [8.0, 4.0, 6.0], [7.0, 4.0, 8.0]], [[6.0, 4.0, 4.0], [5.0, 5.0, 1.0], 
[4.0, 8.0, 8.0]]], [[[8.0, 5.0, 2.0], [8.0, 6.0, 7.0], [6.0, 7.0, 7.0]], [[1.0, 3.0, 7.0], [3.0, 4.0, 1.0], [5.0, 5.0, 1.0]], 
[[8.0, 6.0, 2.0], [4.0, 8.0, 2.0], [5.0, 7.0, 2.0]]], [[[5.0, 3.0, 8.0], [8.0, 6.0, 4.0], [1.0, 4.0, 3.0]], [[2.0, 5.0, 6.0], 
[7.0, 2.0, 8.0], [7.0, 6.0, 6.0]], [[2.0, 1.0, 1.0], [5.0, 8.0, 8.0], [8.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 1.0], [2.0, 8.0, 2.0], [3.0, 2.0, 6.0]], [[8.0, 4.0, 7.0], [4.0, 3.0, 2.0], [5.0, 
7.0, 8.0]], [[8.0, 1.0, 2.0], [6.0, 7.0, 3.0], [1.0, 6.0, 4.0]]], [[[3.0, 1.0, 8.0], [4.0, 3.0, 6.0], [2.0, 2.0, 6.0]], [[5.0, 
6.0, 2.0], [5.0, 4.0, 8.0], [5.0, 5.0, 6.0]], [[3.0, 2.0, 8.0], [2.0, 5.0, 8.0], [5.0, 5.0, 2.0]]], [[[2.0, 7.0, 5.0], [2.0, 
5.0, 7.0], [2.0, 3.0, 3.0]], [[1.0, 2.0, 3.0], [5.0, 1.0, 6.0], [3.0, 2.0, 4.0]], [[2.0, 3.0, 4.0], [6.0, 1.0, 4.0], [6.0, 5.0, 
8.0]]], [[[5.0, 2.0, 1.0], [8.0, 5.0, 1.0], [1.0, 2.0, 2.0]], [[6.0, 8.0, 7.0], [2.0, 3.0, 6.0], [1.0, 7.0, 4.0]], [[2.0, 2.0, 
6.0], [5.0, 2.0, 6.0], [8.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 4.0], [4.0, 1.0, 1.0], [7.0, 8.0, 3.0]], 
[[7.0, 7.0, 6.0], [8.0, 7.0, 2.0], [6.0, 2.0, 1.0]], [[7.0, 2.0, 4.0], [7.0, 4.0, 7.0], [1.0, 1.0, 1.0]]], [[[1.0, 7.0, 6.0], 
[6.0, 4.0, 1.0], [6.0, 3.0, 8.0]], [[1.0, 5.0, 3.0], [1.0, 7.0, 8.0], [7.0, 7.0, 6.0]], [[1.0, 2.0, 6.0], [2.0, 3.0, 5.0], 
[4.0, 8.0, 6.0]]], [[[3.0, 4.0, 4.0], [3.0, 5.0, 6.0], [3.0, 8.0, 2.0]], [[6.0, 5.0, 6.0], [5.0, 7.0, 8.0], [3.0, 6.0, 1.0]], 
[[6.0, 5.0, 8.0], [5.0, 6.0, 1.0], [8.0, 2.0, 6.0]]], [[[3.0, 7.0, 1.0], [7.0, 8.0, 1.0], [3.0, 2.0, 4.0]], [[6.0, 8.0, 1.0], 
[8.0, 7.0, 8.0], [8.0, 8.0, 7.0]], [[1.0, 7.0, 3.0], [5.0, 3.0, 3.0], [2.0, 3.0, 2.0]]]])
      val=numpy.array([[[[8.0, 8.0, 4.0], [7.0, 1.0, 1.0], [8.0, 8.0, 5.0]], [[4.0, 3.0, 3.0], [3.0, 4.0, 6.0], [3.0, 2.0, 
8.0]]], [[[1.0, 8.0, 3.0], [7.0, 7.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 3.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0]]]])
      arg[1:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 1.0], [2.0, 8.0, 2.0], [3.0, 2.0, 6.0]], [[8.0, 4.0, 7.0], [4.0, 3.0, 2.0], [5.0, 
7.0, 8.0]], [[8.0, 1.0, 2.0], [6.0, 7.0, 3.0], [1.0, 6.0, 4.0]]], [[[8.0, 8.0, 4.0], [7.0, 1.0, 1.0], [8.0, 8.0, 5.0]], [[4.0, 
3.0, 3.0], [3.0, 4.0, 6.0], [3.0, 2.0, 8.0]], [[3.0, 2.0, 8.0], [2.0, 5.0, 8.0], [5.0, 5.0, 2.0]]], [[[1.0, 8.0, 3.0], [7.0, 
7.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 3.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 4.0], [6.0, 1.0, 4.0], [6.0, 5.0, 
8.0]]], [[[5.0, 2.0, 1.0], [8.0, 5.0, 1.0], [1.0, 2.0, 2.0]], [[6.0, 8.0, 7.0], [2.0, 3.0, 6.0], [1.0, 7.0, 4.0]], [[2.0, 2.0, 
6.0], [5.0, 2.0, 6.0], [8.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 4.0], [4.0, 1.0, 1.0], [7.0, 8.0, 3.0]], 
[[7.0, 7.0, 6.0], [8.0, 7.0, 2.0], [6.0, 2.0, 1.0]], [[7.0, 2.0, 4.0], [7.0, 4.0, 7.0], [1.0, 1.0, 1.0]]], [[[8.0, 8.0, 4.0], 
[7.0, 1.0, 1.0], [8.0, 8.0, 5.0]], [[4.0, 3.0, 3.0], [3.0, 4.0, 6.0], [3.0, 2.0, 8.0]], [[1.0, 2.0, 6.0], [2.0, 3.0, 5.0], 
[4.0, 8.0, 6.0]]], [[[1.0, 8.0, 3.0], [7.0, 7.0, 1.0], [3.0, 7.0, 4.0]], [[2.0, 5.0, 3.0], [1.0, 3.0, 7.0], [2.0, 7.0, 5.0]], 
[[6.0, 5.0, 8.0], [5.0, 6.0, 1.0], [8.0, 2.0, 6.0]]], [[[3.0, 7.0, 1.0], [7.0, 8.0, 1.0], [3.0, 2.0, 4.0]], [[6.0, 8.0, 1.0], 
[8.0, 7.0, 8.0], [8.0, 8.0, 7.0]], [[1.0, 7.0, 3.0], [5.0, 3.0, 3.0], [2.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 1.0], [7.0, 5.0, 7.0], [4.0, 8.0, 2.0]], [[5.0, 1.0, 1.0], [8.0, 1.0, 8.0], [2.0, 
4.0, 4.0]], [[8.0, 1.0, 5.0], [1.0, 6.0, 5.0], [5.0, 1.0, 6.0]]], [[[2.0, 1.0, 7.0], [7.0, 2.0, 2.0], [2.0, 1.0, 1.0]], [[5.0, 
1.0, 2.0], [8.0, 6.0, 5.0], [7.0, 5.0, 1.0]], [[2.0, 2.0, 2.0], [2.0, 4.0, 4.0], [5.0, 8.0, 1.0]]], [[[3.0, 6.0, 5.0], [6.0, 
2.0, 8.0], [4.0, 7.0, 4.0]], [[7.0, 1.0, 6.0], [5.0, 1.0, 1.0], [3.0, 2.0, 4.0]], [[2.0, 4.0, 2.0], [8.0, 5.0, 6.0], [5.0, 7.0, 
6.0]]], [[[7.0, 1.0, 1.0], [5.0, 2.0, 8.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 6.0], [5.0, 8.0, 8.0], [1.0, 4.0, 5.0]], [[6.0, 6.0, 
2.0], [7.0, 8.0, 1.0], [2.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 7.0], [2.0, 8.0, 1.0], [6.0, 1.0, 2.0]], 
[[5.0, 4.0, 4.0], [1.0, 8.0, 4.0], [4.0, 5.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 5.0, 7.0], [4.0, 8.0, 2.0]]], [[[8.0, 1.0, 7.0], 
[1.0, 2.0, 3.0], [2.0, 4.0, 5.0]], [[7.0, 4.0, 2.0], [6.0, 5.0, 4.0], [6.0, 3.0, 3.0]], [[8.0, 3.0, 8.0], [7.0, 7.0, 3.0], 
[7.0, 7.0, 4.0]]], [[[8.0, 2.0, 6.0], [6.0, 3.0, 4.0], [8.0, 1.0, 6.0]], [[7.0, 1.0, 1.0], [8.0, 7.0, 1.0], [2.0, 3.0, 6.0]], 
[[5.0, 1.0, 8.0], [4.0, 2.0, 1.0], [6.0, 8.0, 1.0]]], [[[3.0, 1.0, 4.0], [6.0, 3.0, 1.0], [2.0, 7.0, 4.0]], [[8.0, 6.0, 1.0], 
[2.0, 2.0, 2.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 4.0], [3.0, 1.0, 6.0], [1.0, 6.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0, 5.0, 6.0], [3.0, 7.0, 2.0], [2.0, 2.0, 2.0]], [[3.0, 8.0, 4.0], [1.0, 2.0, 1.0], [5.0, 8.0, 
8.0]]], [[[3.0, 7.0, 4.0], [5.0, 3.0, 7.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 6.0], [8.0, 4.0, 8.0], [6.0, 2.0, 
4.0]]]]),self.functionspace)
      arg[1:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 1.0], [7.0, 5.0, 7.0], [4.0, 8.0, 2.0]], [[5.0, 1.0, 1.0], [8.0, 1.0, 8.0], [2.0, 
4.0, 4.0]], [[8.0, 1.0, 5.0], [1.0, 6.0, 5.0], [5.0, 1.0, 6.0]]], [[[4.0, 5.0, 6.0], [3.0, 7.0, 2.0], [2.0, 2.0, 2.0]], [[3.0, 
8.0, 4.0], [1.0, 2.0, 1.0], [5.0, 8.0, 8.0]], [[2.0, 2.0, 2.0], [2.0, 4.0, 4.0], [5.0, 8.0, 1.0]]], [[[3.0, 7.0, 4.0], [5.0, 
3.0, 7.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 6.0], [8.0, 4.0, 8.0], [6.0, 2.0, 4.0]], [[2.0, 4.0, 2.0], [8.0, 5.0, 6.0], [5.0, 7.0, 
6.0]]], [[[7.0, 1.0, 1.0], [5.0, 2.0, 8.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 6.0], [5.0, 8.0, 8.0], [1.0, 4.0, 5.0]], [[6.0, 6.0, 
2.0], [7.0, 8.0, 1.0], [2.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 7.0], [2.0, 8.0, 1.0], [6.0, 1.0, 2.0]], 
[[5.0, 4.0, 4.0], [1.0, 8.0, 4.0], [4.0, 5.0, 6.0]], [[2.0, 1.0, 5.0], [2.0, 5.0, 7.0], [4.0, 8.0, 2.0]]], [[[4.0, 5.0, 6.0], 
[3.0, 7.0, 2.0], [2.0, 2.0, 2.0]], [[3.0, 8.0, 4.0], [1.0, 2.0, 1.0], [5.0, 8.0, 8.0]], [[8.0, 3.0, 8.0], [7.0, 7.0, 3.0], 
[7.0, 7.0, 4.0]]], [[[3.0, 7.0, 4.0], [5.0, 3.0, 7.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 6.0], [8.0, 4.0, 8.0], [6.0, 2.0, 4.0]], 
[[5.0, 1.0, 8.0], [4.0, 2.0, 1.0], [6.0, 8.0, 1.0]]], [[[3.0, 1.0, 4.0], [6.0, 3.0, 1.0], [2.0, 7.0, 4.0]], [[8.0, 6.0, 1.0], 
[2.0, 2.0, 2.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 4.0], [3.0, 1.0, 6.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 7.0], [5.0, 1.0, 2.0], [5.0, 1.0, 8.0]], [[4.0, 1.0, 2.0], [2.0, 7.0, 2.0], [8.0, 
3.0, 2.0]], [[8.0, 2.0, 7.0], [2.0, 5.0, 8.0], [5.0, 6.0, 8.0]]], [[[2.0, 5.0, 2.0], [4.0, 5.0, 4.0], [7.0, 8.0, 5.0]], [[7.0, 
2.0, 2.0], [1.0, 8.0, 4.0], [3.0, 8.0, 2.0]], [[2.0, 5.0, 2.0], [6.0, 5.0, 5.0], [4.0, 8.0, 5.0]]], [[[7.0, 3.0, 4.0], [2.0, 
6.0, 2.0], [4.0, 8.0, 7.0]], [[6.0, 1.0, 3.0], [5.0, 6.0, 4.0], [7.0, 5.0, 4.0]], [[2.0, 3.0, 3.0], [8.0, 5.0, 7.0], [2.0, 8.0, 
8.0]]], [[[4.0, 6.0, 5.0], [3.0, 1.0, 4.0], [6.0, 6.0, 3.0]], [[2.0, 6.0, 4.0], [3.0, 3.0, 7.0], [3.0, 1.0, 5.0]], [[4.0, 4.0, 
8.0], [1.0, 6.0, 2.0], [1.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 7.0], [5.0, 6.0, 5.0], [3.0, 7.0, 2.0]], 
[[8.0, 2.0, 1.0], [1.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 1.0], [2.0, 4.0, 7.0], [7.0, 6.0, 3.0]]], [[[3.0, 6.0, 6.0], 
[5.0, 6.0, 5.0], [4.0, 2.0, 1.0]], [[6.0, 1.0, 4.0], [8.0, 7.0, 2.0], [5.0, 5.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 7.0, 1.0], 
[4.0, 4.0, 5.0]]], [[[3.0, 7.0, 8.0], [3.0, 2.0, 1.0], [7.0, 2.0, 7.0]], [[4.0, 3.0, 4.0], [2.0, 3.0, 7.0], [1.0, 7.0, 2.0]], 
[[6.0, 1.0, 2.0], [2.0, 3.0, 4.0], [3.0, 2.0, 8.0]]], [[[2.0, 2.0, 6.0], [3.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[1.0, 5.0, 3.0], 
[5.0, 4.0, 7.0], [2.0, 5.0, 3.0]], [[1.0, 1.0, 6.0], [1.0, 5.0, 6.0], [5.0, 2.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 8.0, 8.0], [7.0, 2.0, 1.0], [8.0, 3.0, 3.0]], [[1.0, 4.0, 4.0], [2.0, 1.0, 7.0], [7.0, 
8.0, 2.0]]], [[[2.0, 4.0, 4.0], [8.0, 8.0, 5.0], [4.0, 4.0, 1.0]], [[4.0, 4.0, 4.0], [3.0, 8.0, 3.0], [3.0, 3.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 3.0, 1.0], [3.0, 1.0, 2.0], [6.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [2.0, 3.0, 8.0], 
[6.0, 8.0, 5.0]]], [[[8.0, 2.0, 3.0], [1.0, 6.0, 7.0], [8.0, 6.0, 2.0]], [[6.0, 4.0, 7.0], [7.0, 7.0, 6.0], [3.0, 7.0, 5.0]]]])
      arg[1:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [5.0, 1.0, 2.0], [5.0, 1.0, 8.0]], [[4.0, 1.0, 2.0], [2.0, 7.0, 2.0], [8.0, 
3.0, 2.0]], [[8.0, 2.0, 7.0], [2.0, 5.0, 8.0], [5.0, 6.0, 8.0]]], [[[3.0, 8.0, 8.0], [7.0, 2.0, 1.0], [8.0, 3.0, 3.0]], [[1.0, 
4.0, 4.0], [2.0, 1.0, 7.0], [7.0, 8.0, 2.0]], [[2.0, 5.0, 2.0], [6.0, 5.0, 5.0], [4.0, 8.0, 5.0]]], [[[2.0, 4.0, 4.0], [8.0, 
8.0, 5.0], [4.0, 4.0, 1.0]], [[4.0, 4.0, 4.0], [3.0, 8.0, 3.0], [3.0, 3.0, 1.0]], [[2.0, 3.0, 3.0], [8.0, 5.0, 7.0], [2.0, 8.0, 
8.0]]], [[[4.0, 6.0, 5.0], [3.0, 1.0, 4.0], [6.0, 6.0, 3.0]], [[2.0, 6.0, 4.0], [3.0, 3.0, 7.0], [3.0, 1.0, 5.0]], [[4.0, 4.0, 
8.0], [1.0, 6.0, 2.0], [1.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 7.0], [5.0, 6.0, 5.0], [3.0, 7.0, 2.0]], 
[[8.0, 2.0, 1.0], [1.0, 6.0, 1.0], [5.0, 5.0, 6.0]], [[7.0, 4.0, 1.0], [2.0, 4.0, 7.0], [7.0, 6.0, 3.0]]], [[[7.0, 3.0, 1.0], 
[3.0, 1.0, 2.0], [6.0, 3.0, 3.0]], [[6.0, 6.0, 7.0], [2.0, 3.0, 8.0], [6.0, 8.0, 5.0]], [[8.0, 1.0, 6.0], [1.0, 7.0, 1.0], 
[4.0, 4.0, 5.0]]], [[[8.0, 2.0, 3.0], [1.0, 6.0, 7.0], [8.0, 6.0, 2.0]], [[6.0, 4.0, 7.0], [7.0, 7.0, 6.0], [3.0, 7.0, 5.0]], 
[[6.0, 1.0, 2.0], [2.0, 3.0, 4.0], [3.0, 2.0, 8.0]]], [[[2.0, 2.0, 6.0], [3.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[1.0, 5.0, 3.0], 
[5.0, 4.0, 7.0], [2.0, 5.0, 3.0]], [[1.0, 1.0, 6.0], [1.0, 5.0, 6.0], [5.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PLUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 8.0], [4.0, 4.0, 4.0], [8.0, 7.0, 1.0]], [[5.0, 5.0, 3.0], [8.0, 8.0, 6.0], [4.0, 
1.0, 7.0]], [[5.0, 8.0, 7.0], [2.0, 8.0, 5.0], [4.0, 7.0, 8.0]]], [[[2.0, 2.0, 8.0], [5.0, 6.0, 5.0], [1.0, 2.0, 8.0]], [[8.0, 
2.0, 4.0], [4.0, 4.0, 6.0], [4.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], [7.0, 3.0, 3.0], [2.0, 1.0, 1.0]]], [[[4.0, 2.0, 8.0], [1.0, 
5.0, 8.0], [8.0, 4.0, 6.0]], [[4.0, 6.0, 4.0], [6.0, 1.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 5.0, 8.0], [2.0, 3.0, 1.0], [5.0, 4.0, 
6.0]]], [[[4.0, 8.0, 7.0], [3.0, 3.0, 4.0], [5.0, 8.0, 4.0]], [[7.0, 5.0, 2.0], [6.0, 4.0, 1.0], [7.0, 1.0, 8.0]], [[1.0, 4.0, 
5.0], [7.0, 4.0, 7.0], [7.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 5.0], [2.0, 5.0, 7.0], [3.0, 6.0, 5.0]], 
[[5.0, 6.0, 6.0], [3.0, 8.0, 7.0], [1.0, 5.0, 7.0]], [[7.0, 6.0, 2.0], [4.0, 7.0, 5.0], [7.0, 6.0, 7.0]]], [[[5.0, 1.0, 3.0], 
[2.0, 2.0, 7.0], [8.0, 7.0, 5.0]], [[4.0, 5.0, 8.0], [4.0, 8.0, 7.0], [2.0, 4.0, 2.0]], [[5.0, 6.0, 8.0], [4.0, 3.0, 3.0], 
[3.0, 5.0, 4.0]]], [[[8.0, 2.0, 1.0], [8.0, 6.0, 5.0], [8.0, 4.0, 3.0]], [[1.0, 3.0, 5.0], [8.0, 3.0, 1.0], [1.0, 5.0, 8.0]], 
[[4.0, 2.0, 4.0], [4.0, 7.0, 1.0], [8.0, 8.0, 6.0]]], [[[3.0, 8.0, 8.0], [5.0, 5.0, 5.0], [1.0, 6.0, 2.0]], [[5.0, 5.0, 4.0], 
[2.0, 1.0, 6.0], [4.0, 1.0, 6.0]], [[6.0, 7.0, 6.0], [6.0, 2.0, 2.0], [1.0, 3.0, 7.0]]]])
      val=numpy.array([[8.0, 1.0]])
      arg[2,2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [4.0, 4.0, 4.0], [8.0, 7.0, 1.0]], [[5.0, 5.0, 3.0], [8.0, 8.0, 6.0], [4.0, 
1.0, 7.0]], [[5.0, 8.0, 7.0], [2.0, 8.0, 5.0], [4.0, 7.0, 8.0]]], [[[2.0, 2.0, 8.0], [5.0, 6.0, 5.0], [1.0, 2.0, 8.0]], [[8.0, 
2.0, 4.0], [4.0, 4.0, 6.0], [4.0, 5.0, 5.0]], [[1.0, 8.0, 8.0], [7.0, 3.0, 3.0], [2.0, 1.0, 1.0]]], [[[4.0, 2.0, 8.0], [1.0, 
5.0, 8.0], [8.0, 4.0, 6.0]], [[4.0, 6.0, 4.0], [6.0, 1.0, 2.0], [7.0, 7.0, 4.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 1.0], [5.0, 4.0, 
6.0]]], [[[4.0, 8.0, 7.0], [3.0, 3.0, 4.0], [5.0, 8.0, 4.0]], [[7.0, 5.0, 2.0], [6.0, 4.0, 1.0], [7.0, 1.0, 8.0]], [[1.0, 4.0, 
5.0], [7.0, 4.0, 7.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 5.0], [2.0, 5.0, 7.0], [3.0, 6.0, 5.0]], 
[[5.0, 6.0, 6.0], [3.0, 8.0, 7.0], [1.0, 5.0, 7.0]], [[7.0, 6.0, 2.0], [4.0, 7.0, 5.0], [7.0, 6.0, 7.0]]], [[[5.0, 1.0, 3.0], 
[2.0, 2.0, 7.0], [8.0, 7.0, 5.0]], [[4.0, 5.0, 8.0], [4.0, 8.0, 7.0], [2.0, 4.0, 2.0]], [[5.0, 6.0, 8.0], [4.0, 3.0, 3.0], 
[3.0, 5.0, 4.0]]], [[[8.0, 2.0, 1.0], [8.0, 6.0, 5.0], [8.0, 4.0, 3.0]], [[1.0, 3.0, 5.0], [8.0, 3.0, 1.0], [1.0, 5.0, 8.0]], 
[[4.0, 8.0, 4.0], [4.0, 1.0, 1.0], [8.0, 8.0, 6.0]]], [[[3.0, 8.0, 8.0], [5.0, 5.0, 5.0], [1.0, 6.0, 2.0]], [[5.0, 5.0, 4.0], 
[2.0, 1.0, 6.0], [4.0, 1.0, 6.0]], [[6.0, 7.0, 6.0], [6.0, 2.0, 2.0], [1.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PLUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 3.0], [3.0, 2.0, 5.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 4.0, 6.0], [6.0, 
3.0, 5.0]], [[4.0, 6.0, 2.0], [7.0, 4.0, 2.0], [7.0, 7.0, 2.0]]], [[[3.0, 5.0, 3.0], [3.0, 8.0, 2.0], [3.0, 6.0, 3.0]], [[2.0, 
3.0, 1.0], [5.0, 7.0, 2.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 5.0], [8.0, 3.0, 4.0], [2.0, 4.0, 5.0]]], [[[6.0, 8.0, 2.0], [4.0, 
4.0, 7.0], [7.0, 1.0, 8.0]], [[1.0, 3.0, 4.0], [7.0, 3.0, 5.0], [7.0, 1.0, 5.0]], [[7.0, 8.0, 5.0], [8.0, 2.0, 5.0], [2.0, 2.0, 
6.0]]], [[[2.0, 1.0, 6.0], [1.0, 8.0, 5.0], [8.0, 3.0, 7.0]], [[7.0, 4.0, 4.0], [8.0, 8.0, 8.0], [3.0, 4.0, 8.0]], [[8.0, 8.0, 
7.0], [4.0, 7.0, 6.0], [6.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 5.0], [1.0, 7.0, 3.0], [5.0, 7.0, 2.0]], 
[[6.0, 3.0, 2.0], [3.0, 5.0, 1.0], [5.0, 4.0, 4.0]], [[8.0, 2.0, 1.0], [4.0, 1.0, 8.0], [4.0, 8.0, 5.0]]], [[[6.0, 1.0, 8.0], 
[3.0, 6.0, 4.0], [5.0, 2.0, 6.0]], [[5.0, 2.0, 2.0], [6.0, 8.0, 6.0], [3.0, 8.0, 4.0]], [[1.0, 6.0, 1.0], [7.0, 2.0, 5.0], 
[4.0, 6.0, 2.0]]], [[[3.0, 1.0, 5.0], [6.0, 5.0, 8.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 5.0], [1.0, 7.0, 5.0], [5.0, 8.0, 1.0]], 
[[7.0, 5.0, 1.0], [4.0, 2.0, 7.0], [7.0, 7.0, 7.0]]], [[[8.0, 1.0, 5.0], [2.0, 1.0, 3.0], [8.0, 6.0, 8.0]], [[2.0, 7.0, 8.0], 
[3.0, 1.0, 6.0], [3.0, 3.0, 6.0]], [[5.0, 6.0, 5.0], [7.0, 7.0, 6.0], [8.0, 6.0, 3.0]]]])
      val=Data(numpy.array([[3.0, 4.0]]),self.functionspace)
      arg[2,2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 3.0], [3.0, 2.0, 5.0], [8.0, 4.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 4.0, 6.0], [6.0, 
3.0, 5.0]], [[4.0, 6.0, 2.0], [7.0, 4.0, 2.0], [7.0, 7.0, 2.0]]], [[[3.0, 5.0, 3.0], [3.0, 8.0, 2.0], [3.0, 6.0, 3.0]], [[2.0, 
3.0, 1.0], [5.0, 7.0, 2.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 5.0], [8.0, 3.0, 4.0], [2.0, 4.0, 5.0]]], [[[6.0, 8.0, 2.0], [4.0, 
4.0, 7.0], [7.0, 1.0, 8.0]], [[1.0, 3.0, 4.0], [7.0, 3.0, 5.0], [7.0, 1.0, 5.0]], [[7.0, 3.0, 5.0], [8.0, 4.0, 5.0], [2.0, 2.0, 
6.0]]], [[[2.0, 1.0, 6.0], [1.0, 8.0, 5.0], [8.0, 3.0, 7.0]], [[7.0, 4.0, 4.0], [8.0, 8.0, 8.0], [3.0, 4.0, 8.0]], [[8.0, 8.0, 
7.0], [4.0, 7.0, 6.0], [6.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 5.0], [1.0, 7.0, 3.0], [5.0, 7.0, 2.0]], 
[[6.0, 3.0, 2.0], [3.0, 5.0, 1.0], [5.0, 4.0, 4.0]], [[8.0, 2.0, 1.0], [4.0, 1.0, 8.0], [4.0, 8.0, 5.0]]], [[[6.0, 1.0, 8.0], 
[3.0, 6.0, 4.0], [5.0, 2.0, 6.0]], [[5.0, 2.0, 2.0], [6.0, 8.0, 6.0], [3.0, 8.0, 4.0]], [[1.0, 6.0, 1.0], [7.0, 2.0, 5.0], 
[4.0, 6.0, 2.0]]], [[[3.0, 1.0, 5.0], [6.0, 5.0, 8.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 5.0], [1.0, 7.0, 5.0], [5.0, 8.0, 1.0]], 
[[7.0, 3.0, 1.0], [4.0, 4.0, 7.0], [7.0, 7.0, 7.0]]], [[[8.0, 1.0, 5.0], [2.0, 1.0, 3.0], [8.0, 6.0, 8.0]], [[2.0, 7.0, 8.0], 
[3.0, 1.0, 6.0], [3.0, 3.0, 6.0]], [[5.0, 6.0, 5.0], [7.0, 7.0, 6.0], [8.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PLUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 8.0], [7.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[8.0, 7.0, 6.0], [3.0, 2.0, 6.0], [1.0, 
2.0, 1.0]], [[4.0, 3.0, 5.0], [4.0, 7.0, 8.0], [6.0, 4.0, 4.0]]], [[[8.0, 7.0, 5.0], [6.0, 8.0, 2.0], [7.0, 3.0, 7.0]], [[7.0, 
5.0, 2.0], [8.0, 7.0, 5.0], [4.0, 6.0, 8.0]], [[7.0, 1.0, 8.0], [8.0, 6.0, 1.0], [6.0, 3.0, 6.0]]], [[[8.0, 6.0, 8.0], [7.0, 
5.0, 1.0], [2.0, 3.0, 4.0]], [[5.0, 8.0, 7.0], [8.0, 2.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 8.0, 1.0], [4.0, 6.0, 7.0], [4.0, 3.0, 
3.0]]], [[[1.0, 2.0, 1.0], [2.0, 8.0, 6.0], [6.0, 2.0, 8.0]], [[7.0, 4.0, 8.0], [7.0, 3.0, 1.0], [7.0, 6.0, 7.0]], [[3.0, 2.0, 
3.0], [3.0, 4.0, 7.0], [3.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 5.0], [4.0, 1.0, 1.0], [7.0, 6.0, 3.0]], 
[[1.0, 5.0, 1.0], [1.0, 8.0, 4.0], [7.0, 4.0, 4.0]], [[6.0, 3.0, 2.0], [4.0, 3.0, 2.0], [8.0, 4.0, 1.0]]], [[[4.0, 6.0, 1.0], 
[6.0, 2.0, 1.0], [6.0, 6.0, 5.0]], [[8.0, 3.0, 5.0], [7.0, 7.0, 7.0], [2.0, 4.0, 6.0]], [[6.0, 6.0, 7.0], [6.0, 7.0, 3.0], 
[7.0, 3.0, 3.0]]], [[[3.0, 6.0, 8.0], [4.0, 5.0, 4.0], [2.0, 6.0, 4.0]], [[5.0, 6.0, 1.0], [2.0, 1.0, 7.0], [1.0, 4.0, 8.0]], 
[[4.0, 1.0, 4.0], [3.0, 4.0, 1.0], [1.0, 5.0, 5.0]]], [[[3.0, 2.0, 6.0], [3.0, 8.0, 2.0], [2.0, 8.0, 6.0]], [[6.0, 8.0, 6.0], 
[1.0, 1.0, 3.0], [3.0, 7.0, 8.0]], [[5.0, 8.0, 7.0], [5.0, 2.0, 8.0], [1.0, 2.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 6.0]])+(1.-msk_val)*numpy.array([[3.0, 4.0]])
      arg[2,2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 8.0], [7.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[8.0, 7.0, 6.0], [3.0, 2.0, 6.0], [1.0, 
2.0, 1.0]], [[4.0, 3.0, 5.0], [4.0, 7.0, 8.0], [6.0, 4.0, 4.0]]], [[[8.0, 7.0, 5.0], [6.0, 8.0, 2.0], [7.0, 3.0, 7.0]], [[7.0, 
5.0, 2.0], [8.0, 7.0, 5.0], [4.0, 6.0, 8.0]], [[7.0, 1.0, 8.0], [8.0, 6.0, 1.0], [6.0, 3.0, 6.0]]], [[[8.0, 6.0, 8.0], [7.0, 
5.0, 1.0], [2.0, 3.0, 4.0]], [[5.0, 8.0, 7.0], [8.0, 2.0, 7.0], [7.0, 7.0, 6.0]], [[8.0, 1.0, 1.0], [4.0, 6.0, 7.0], [4.0, 3.0, 
3.0]]], [[[1.0, 2.0, 1.0], [2.0, 8.0, 6.0], [6.0, 2.0, 8.0]], [[7.0, 4.0, 8.0], [7.0, 3.0, 1.0], [7.0, 6.0, 7.0]], [[3.0, 2.0, 
3.0], [3.0, 4.0, 7.0], [3.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 5.0], [4.0, 1.0, 1.0], [7.0, 6.0, 3.0]], 
[[1.0, 5.0, 1.0], [1.0, 8.0, 4.0], [7.0, 4.0, 4.0]], [[6.0, 3.0, 2.0], [4.0, 3.0, 2.0], [8.0, 4.0, 1.0]]], [[[4.0, 6.0, 1.0], 
[6.0, 2.0, 1.0], [6.0, 6.0, 5.0]], [[8.0, 3.0, 5.0], [7.0, 7.0, 7.0], [2.0, 4.0, 6.0]], [[6.0, 6.0, 7.0], [6.0, 7.0, 3.0], 
[7.0, 3.0, 3.0]]], [[[3.0, 6.0, 8.0], [4.0, 5.0, 4.0], [2.0, 6.0, 4.0]], [[5.0, 6.0, 1.0], [2.0, 1.0, 7.0], [1.0, 4.0, 8.0]], 
[[4.0, 3.0, 4.0], [3.0, 4.0, 1.0], [1.0, 5.0, 5.0]]], [[[3.0, 2.0, 6.0], [3.0, 8.0, 2.0], [2.0, 8.0, 6.0]], [[6.0, 8.0, 6.0], 
[1.0, 1.0, 3.0], [3.0, 7.0, 8.0]], [[5.0, 8.0, 7.0], [5.0, 2.0, 8.0], [1.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 6.0], [8.0, 6.0, 1.0], [2.0, 7.0, 1.0]], [[7.0, 2.0, 5.0], [2.0, 7.0, 7.0], [3.0, 
5.0, 1.0]], [[8.0, 6.0, 3.0], [4.0, 2.0, 3.0], [1.0, 4.0, 8.0]]], [[[4.0, 7.0, 8.0], [4.0, 2.0, 8.0], [1.0, 8.0, 2.0]], [[2.0, 
2.0, 8.0], [2.0, 1.0, 3.0], [3.0, 5.0, 6.0]], [[5.0, 7.0, 3.0], [3.0, 3.0, 7.0], [4.0, 6.0, 1.0]]], [[[7.0, 5.0, 2.0], [8.0, 
3.0, 1.0], [7.0, 7.0, 1.0]], [[2.0, 7.0, 7.0], [8.0, 5.0, 1.0], [5.0, 6.0, 7.0]], [[5.0, 6.0, 5.0], [6.0, 8.0, 3.0], [1.0, 3.0, 
4.0]]], [[[6.0, 4.0, 5.0], [6.0, 6.0, 8.0], [3.0, 2.0, 1.0]], [[3.0, 1.0, 7.0], [2.0, 2.0, 6.0], [2.0, 2.0, 3.0]], [[6.0, 8.0, 
2.0], [8.0, 8.0, 8.0], [1.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 3.0], [4.0, 3.0, 5.0], [5.0, 1.0, 3.0]], 
[[1.0, 4.0, 8.0], [5.0, 5.0, 2.0], [5.0, 8.0, 8.0]], [[6.0, 2.0, 3.0], [6.0, 5.0, 3.0], [7.0, 1.0, 5.0]]], [[[8.0, 8.0, 2.0], 
[2.0, 8.0, 4.0], [8.0, 4.0, 6.0]], [[3.0, 7.0, 8.0], [3.0, 6.0, 8.0], [7.0, 8.0, 2.0]], [[7.0, 2.0, 6.0], [3.0, 5.0, 5.0], 
[8.0, 5.0, 8.0]]], [[[5.0, 8.0, 1.0], [6.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[2.0, 8.0, 1.0], [1.0, 1.0, 1.0], [6.0, 7.0, 7.0]], 
[[4.0, 5.0, 1.0], [5.0, 1.0, 1.0], [5.0, 7.0, 2.0]]], [[[5.0, 3.0, 6.0], [5.0, 3.0, 5.0], [2.0, 4.0, 6.0]], [[3.0, 2.0, 8.0], 
[1.0, 7.0, 8.0], [6.0, 7.0, 8.0]], [[6.0, 5.0, 1.0], [8.0, 5.0, 7.0], [1.0, 7.0, 2.0]]]])
      val=numpy.array([[[6.0]], [[1.0]], [[5.0]], [[2.0]]])
      arg[:,1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [8.0, 6.0, 1.0], [2.0, 7.0, 1.0]], [[7.0, 2.0, 5.0], [2.0, 7.0, 7.0], [3.0, 
5.0, 6.0]], [[8.0, 6.0, 3.0], [4.0, 2.0, 3.0], [1.0, 4.0, 8.0]]], [[[4.0, 7.0, 8.0], [4.0, 2.0, 8.0], [1.0, 8.0, 2.0]], [[2.0, 
2.0, 8.0], [2.0, 1.0, 3.0], [3.0, 5.0, 1.0]], [[5.0, 7.0, 3.0], [3.0, 3.0, 7.0], [4.0, 6.0, 1.0]]], [[[7.0, 5.0, 2.0], [8.0, 
3.0, 1.0], [7.0, 7.0, 1.0]], [[2.0, 7.0, 7.0], [8.0, 5.0, 1.0], [5.0, 6.0, 5.0]], [[5.0, 6.0, 5.0], [6.0, 8.0, 3.0], [1.0, 3.0, 
4.0]]], [[[6.0, 4.0, 5.0], [6.0, 6.0, 8.0], [3.0, 2.0, 1.0]], [[3.0, 1.0, 7.0], [2.0, 2.0, 6.0], [2.0, 2.0, 2.0]], [[6.0, 8.0, 
2.0], [8.0, 8.0, 8.0], [1.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 3.0], [4.0, 3.0, 5.0], [5.0, 1.0, 3.0]], 
[[1.0, 4.0, 8.0], [5.0, 5.0, 2.0], [5.0, 8.0, 6.0]], [[6.0, 2.0, 3.0], [6.0, 5.0, 3.0], [7.0, 1.0, 5.0]]], [[[8.0, 8.0, 2.0], 
[2.0, 8.0, 4.0], [8.0, 4.0, 6.0]], [[3.0, 7.0, 8.0], [3.0, 6.0, 8.0], [7.0, 8.0, 1.0]], [[7.0, 2.0, 6.0], [3.0, 5.0, 5.0], 
[8.0, 5.0, 8.0]]], [[[5.0, 8.0, 1.0], [6.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[2.0, 8.0, 1.0], [1.0, 1.0, 1.0], [6.0, 7.0, 5.0]], 
[[4.0, 5.0, 1.0], [5.0, 1.0, 1.0], [5.0, 7.0, 2.0]]], [[[5.0, 3.0, 6.0], [5.0, 3.0, 5.0], [2.0, 4.0, 6.0]], [[3.0, 2.0, 8.0], 
[1.0, 7.0, 8.0], [6.0, 7.0, 2.0]], [[6.0, 5.0, 1.0], [8.0, 5.0, 7.0], [1.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 4.0], [1.0, 8.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 4.0, 5.0], [7.0, 1.0, 6.0], [1.0, 
6.0, 8.0]], [[6.0, 1.0, 2.0], [7.0, 1.0, 8.0], [6.0, 3.0, 6.0]]], [[[2.0, 3.0, 8.0], [6.0, 3.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 
8.0, 1.0], [8.0, 3.0, 7.0], [4.0, 2.0, 8.0]], [[6.0, 4.0, 7.0], [4.0, 7.0, 6.0], [5.0, 7.0, 8.0]]], [[[6.0, 8.0, 4.0], [7.0, 
1.0, 4.0], [5.0, 8.0, 1.0]], [[5.0, 6.0, 5.0], [8.0, 4.0, 6.0], [3.0, 6.0, 4.0]], [[7.0, 7.0, 5.0], [6.0, 4.0, 7.0], [6.0, 8.0, 
8.0]]], [[[5.0, 8.0, 7.0], [7.0, 1.0, 7.0], [5.0, 1.0, 5.0]], [[6.0, 3.0, 7.0], [2.0, 7.0, 4.0], [2.0, 8.0, 6.0]], [[7.0, 1.0, 
5.0], [3.0, 1.0, 1.0], [2.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 7.0, 5.0], [4.0, 2.0, 8.0], [6.0, 7.0, 5.0]], 
[[8.0, 8.0, 2.0], [7.0, 7.0, 2.0], [3.0, 4.0, 4.0]], [[1.0, 6.0, 2.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]]], [[[1.0, 5.0, 6.0], 
[8.0, 1.0, 6.0], [3.0, 3.0, 3.0]], [[8.0, 5.0, 3.0], [1.0, 7.0, 5.0], [4.0, 8.0, 6.0]], [[7.0, 5.0, 2.0], [2.0, 3.0, 1.0], 
[7.0, 4.0, 6.0]]], [[[7.0, 1.0, 4.0], [3.0, 7.0, 5.0], [5.0, 2.0, 4.0]], [[6.0, 3.0, 7.0], [7.0, 7.0, 6.0], [3.0, 3.0, 8.0]], 
[[6.0, 2.0, 1.0], [6.0, 4.0, 8.0], [8.0, 3.0, 8.0]]], [[[5.0, 8.0, 2.0], [1.0, 1.0, 5.0], [7.0, 7.0, 3.0]], [[8.0, 5.0, 1.0], 
[3.0, 3.0, 6.0], [8.0, 8.0, 4.0]], [[5.0, 8.0, 3.0], [1.0, 6.0, 5.0], [7.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[[4.0]], [[2.0]], [[6.0]], [[3.0]]]),self.functionspace)
      arg[:,1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 4.0], [1.0, 8.0, 7.0], [4.0, 2.0, 8.0]], [[1.0, 4.0, 5.0], [7.0, 1.0, 6.0], [1.0, 
6.0, 4.0]], [[6.0, 1.0, 2.0], [7.0, 1.0, 8.0], [6.0, 3.0, 6.0]]], [[[2.0, 3.0, 8.0], [6.0, 3.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 
8.0, 1.0], [8.0, 3.0, 7.0], [4.0, 2.0, 2.0]], [[6.0, 4.0, 7.0], [4.0, 7.0, 6.0], [5.0, 7.0, 8.0]]], [[[6.0, 8.0, 4.0], [7.0, 
1.0, 4.0], [5.0, 8.0, 1.0]], [[5.0, 6.0, 5.0], [8.0, 4.0, 6.0], [3.0, 6.0, 6.0]], [[7.0, 7.0, 5.0], [6.0, 4.0, 7.0], [6.0, 8.0, 
8.0]]], [[[5.0, 8.0, 7.0], [7.0, 1.0, 7.0], [5.0, 1.0, 5.0]], [[6.0, 3.0, 7.0], [2.0, 7.0, 4.0], [2.0, 8.0, 3.0]], [[7.0, 1.0, 
5.0], [3.0, 1.0, 1.0], [2.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 5.0], [4.0, 2.0, 8.0], [6.0, 7.0, 5.0]], 
[[8.0, 8.0, 2.0], [7.0, 7.0, 2.0], [3.0, 4.0, 4.0]], [[1.0, 6.0, 2.0], [6.0, 7.0, 7.0], [1.0, 6.0, 5.0]]], [[[1.0, 5.0, 6.0], 
[8.0, 1.0, 6.0], [3.0, 3.0, 3.0]], [[8.0, 5.0, 3.0], [1.0, 7.0, 5.0], [4.0, 8.0, 2.0]], [[7.0, 5.0, 2.0], [2.0, 3.0, 1.0], 
[7.0, 4.0, 6.0]]], [[[7.0, 1.0, 4.0], [3.0, 7.0, 5.0], [5.0, 2.0, 4.0]], [[6.0, 3.0, 7.0], [7.0, 7.0, 6.0], [3.0, 3.0, 6.0]], 
[[6.0, 2.0, 1.0], [6.0, 4.0, 8.0], [8.0, 3.0, 8.0]]], [[[5.0, 8.0, 2.0], [1.0, 1.0, 5.0], [7.0, 7.0, 3.0]], [[8.0, 5.0, 1.0], 
[3.0, 3.0, 6.0], [8.0, 8.0, 3.0]], [[5.0, 8.0, 3.0], [1.0, 6.0, 5.0], [7.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 5.0], [7.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[2.0, 4.0, 4.0], [7.0, 3.0, 5.0], [2.0, 
1.0, 3.0]], [[8.0, 2.0, 8.0], [8.0, 7.0, 7.0], [3.0, 3.0, 2.0]]], [[[7.0, 5.0, 4.0], [5.0, 7.0, 5.0], [6.0, 6.0, 1.0]], [[8.0, 
5.0, 4.0], [3.0, 8.0, 8.0], [4.0, 8.0, 5.0]], [[4.0, 3.0, 2.0], [5.0, 3.0, 2.0], [6.0, 7.0, 6.0]]], [[[2.0, 1.0, 3.0], [6.0, 
4.0, 3.0], [5.0, 1.0, 1.0]], [[4.0, 3.0, 2.0], [2.0, 4.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 6.0, 6.0], [5.0, 3.0, 4.0], [2.0, 1.0, 
7.0]]], [[[4.0, 2.0, 6.0], [8.0, 2.0, 4.0], [2.0, 5.0, 8.0]], [[5.0, 3.0, 6.0], [4.0, 2.0, 8.0], [4.0, 3.0, 5.0]], [[7.0, 8.0, 
5.0], [1.0, 4.0, 5.0], [7.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 5.0], [1.0, 5.0, 7.0], [4.0, 2.0, 4.0]], 
[[7.0, 2.0, 1.0], [7.0, 7.0, 5.0], [2.0, 4.0, 6.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 7.0], [4.0, 7.0, 5.0]]], [[[4.0, 1.0, 5.0], 
[2.0, 6.0, 2.0], [5.0, 7.0, 5.0]], [[1.0, 5.0, 1.0], [5.0, 5.0, 2.0], [8.0, 4.0, 1.0]], [[1.0, 6.0, 3.0], [6.0, 1.0, 6.0], 
[6.0, 7.0, 5.0]]], [[[5.0, 4.0, 1.0], [5.0, 8.0, 2.0], [6.0, 7.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 1.0, 8.0], [6.0, 6.0, 7.0]], 
[[4.0, 2.0, 8.0], [6.0, 5.0, 8.0], [7.0, 8.0, 3.0]]], [[[5.0, 1.0, 3.0], [2.0, 5.0, 6.0], [5.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], 
[6.0, 3.0, 2.0], [8.0, 5.0, 1.0]], [[8.0, 6.0, 2.0], [4.0, 6.0, 4.0], [8.0, 4.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0]], [[8.0]], [[6.0]], [[1.0]]])+(1.-msk_val)*numpy.array([[[7.0]], [[2.0]], [[8.0]], 
[[2.0]]])
      arg[:,1,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 5.0], [7.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[2.0, 4.0, 4.0], [7.0, 3.0, 5.0], [2.0, 
1.0, 1.0]], [[8.0, 2.0, 8.0], [8.0, 7.0, 7.0], [3.0, 3.0, 2.0]]], [[[7.0, 5.0, 4.0], [5.0, 7.0, 5.0], [6.0, 6.0, 1.0]], [[8.0, 
5.0, 4.0], [3.0, 8.0, 8.0], [4.0, 8.0, 8.0]], [[4.0, 3.0, 2.0], [5.0, 3.0, 2.0], [6.0, 7.0, 6.0]]], [[[2.0, 1.0, 3.0], [6.0, 
4.0, 3.0], [5.0, 1.0, 1.0]], [[4.0, 3.0, 2.0], [2.0, 4.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 6.0, 6.0], [5.0, 3.0, 4.0], [2.0, 1.0, 
7.0]]], [[[4.0, 2.0, 6.0], [8.0, 2.0, 4.0], [2.0, 5.0, 8.0]], [[5.0, 3.0, 6.0], [4.0, 2.0, 8.0], [4.0, 3.0, 1.0]], [[7.0, 8.0, 
5.0], [1.0, 4.0, 5.0], [7.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 5.0], [1.0, 5.0, 7.0], [4.0, 2.0, 4.0]], 
[[7.0, 2.0, 1.0], [7.0, 7.0, 5.0], [2.0, 4.0, 7.0]], [[4.0, 5.0, 3.0], [1.0, 1.0, 7.0], [4.0, 7.0, 5.0]]], [[[4.0, 1.0, 5.0], 
[2.0, 6.0, 2.0], [5.0, 7.0, 5.0]], [[1.0, 5.0, 1.0], [5.0, 5.0, 2.0], [8.0, 4.0, 2.0]], [[1.0, 6.0, 3.0], [6.0, 1.0, 6.0], 
[6.0, 7.0, 5.0]]], [[[5.0, 4.0, 1.0], [5.0, 8.0, 2.0], [6.0, 7.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 1.0, 8.0], [6.0, 6.0, 8.0]], 
[[4.0, 2.0, 8.0], [6.0, 5.0, 8.0], [7.0, 8.0, 3.0]]], [[[5.0, 1.0, 3.0], [2.0, 5.0, 6.0], [5.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], 
[6.0, 3.0, 2.0], [8.0, 5.0, 2.0]], [[8.0, 6.0, 2.0], [4.0, 6.0, 4.0], [8.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 6.0], [7.0, 3.0, 8.0], [8.0, 5.0, 4.0]], [[6.0, 6.0, 5.0], [4.0, 4.0, 3.0], [8.0, 
1.0, 8.0]], [[7.0, 1.0, 6.0], [5.0, 5.0, 8.0], [8.0, 6.0, 6.0]]], [[[4.0, 3.0, 5.0], [2.0, 7.0, 5.0], [1.0, 5.0, 2.0]], [[5.0, 
2.0, 7.0], [5.0, 1.0, 4.0], [7.0, 5.0, 7.0]], [[5.0, 7.0, 3.0], [3.0, 4.0, 7.0], [4.0, 4.0, 4.0]]], [[[1.0, 4.0, 7.0], [2.0, 
8.0, 5.0], [6.0, 3.0, 8.0]], [[1.0, 8.0, 1.0], [3.0, 3.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 7.0], [5.0, 3.0, 
3.0]]], [[[3.0, 3.0, 8.0], [4.0, 1.0, 7.0], [2.0, 4.0, 5.0]], [[8.0, 4.0, 1.0], [1.0, 7.0, 6.0], [8.0, 8.0, 1.0]], [[2.0, 7.0, 
7.0], [7.0, 3.0, 2.0], [2.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 3.0], [1.0, 8.0, 4.0], [2.0, 1.0, 1.0]], 
[[3.0, 7.0, 2.0], [5.0, 6.0, 6.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 1.0], [7.0, 8.0, 1.0], [6.0, 2.0, 2.0]]], [[[5.0, 1.0, 5.0], 
[3.0, 6.0, 1.0], [4.0, 2.0, 8.0]], [[3.0, 3.0, 5.0], [1.0, 4.0, 8.0], [2.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [5.0, 3.0, 2.0], 
[5.0, 5.0, 6.0]]], [[[6.0, 2.0, 2.0], [2.0, 2.0, 5.0], [4.0, 8.0, 5.0]], [[1.0, 4.0, 6.0], [7.0, 7.0, 2.0], [2.0, 4.0, 6.0]], 
[[1.0, 2.0, 5.0], [4.0, 3.0, 3.0], [5.0, 2.0, 3.0]]], [[[6.0, 3.0, 7.0], [5.0, 8.0, 5.0], [2.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], 
[7.0, 7.0, 3.0], [1.0, 4.0, 8.0]], [[3.0, 6.0, 7.0], [5.0, 2.0, 2.0], [5.0, 4.0, 4.0]]]])
      val=numpy.array([7.0])
      arg[2,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 6.0], [7.0, 3.0, 8.0], [8.0, 5.0, 4.0]], [[6.0, 6.0, 5.0], [4.0, 4.0, 3.0], [8.0, 
1.0, 8.0]], [[7.0, 1.0, 6.0], [5.0, 5.0, 8.0], [8.0, 6.0, 6.0]]], [[[4.0, 3.0, 5.0], [2.0, 7.0, 5.0], [1.0, 5.0, 2.0]], [[5.0, 
2.0, 7.0], [5.0, 1.0, 4.0], [7.0, 5.0, 7.0]], [[5.0, 7.0, 3.0], [3.0, 4.0, 7.0], [4.0, 4.0, 4.0]]], [[[1.0, 4.0, 7.0], [2.0, 
8.0, 5.0], [6.0, 3.0, 8.0]], [[1.0, 8.0, 1.0], [3.0, 3.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 7.0], [5.0, 3.0, 
3.0]]], [[[3.0, 3.0, 8.0], [4.0, 1.0, 7.0], [2.0, 4.0, 5.0]], [[8.0, 4.0, 1.0], [1.0, 7.0, 6.0], [8.0, 8.0, 1.0]], [[2.0, 7.0, 
7.0], [7.0, 3.0, 2.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 3.0], [1.0, 8.0, 4.0], [2.0, 1.0, 1.0]], 
[[3.0, 7.0, 2.0], [5.0, 6.0, 6.0], [8.0, 4.0, 3.0]], [[8.0, 8.0, 1.0], [7.0, 8.0, 1.0], [6.0, 2.0, 2.0]]], [[[5.0, 1.0, 5.0], 
[3.0, 6.0, 1.0], [4.0, 2.0, 8.0]], [[3.0, 3.0, 5.0], [1.0, 4.0, 8.0], [2.0, 6.0, 7.0]], [[3.0, 3.0, 7.0], [5.0, 3.0, 2.0], 
[5.0, 5.0, 6.0]]], [[[6.0, 2.0, 2.0], [2.0, 2.0, 5.0], [4.0, 8.0, 5.0]], [[1.0, 4.0, 6.0], [7.0, 7.0, 7.0], [2.0, 4.0, 6.0]], 
[[1.0, 2.0, 5.0], [4.0, 3.0, 3.0], [5.0, 2.0, 3.0]]], [[[6.0, 3.0, 7.0], [5.0, 8.0, 5.0], [2.0, 5.0, 1.0]], [[7.0, 8.0, 8.0], 
[7.0, 7.0, 3.0], [1.0, 4.0, 8.0]], [[3.0, 6.0, 7.0], [5.0, 2.0, 2.0], [5.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 7.0], [7.0, 5.0, 8.0], [5.0, 2.0, 2.0]], [[1.0, 5.0, 7.0], [4.0, 4.0, 3.0], [5.0, 
8.0, 7.0]], [[7.0, 3.0, 4.0], [2.0, 1.0, 6.0], [6.0, 8.0, 2.0]]], [[[7.0, 5.0, 3.0], [5.0, 5.0, 4.0], [3.0, 1.0, 6.0]], [[2.0, 
1.0, 5.0], [2.0, 7.0, 5.0], [5.0, 6.0, 2.0]], [[7.0, 6.0, 6.0], [5.0, 4.0, 6.0], [5.0, 1.0, 4.0]]], [[[7.0, 8.0, 8.0], [6.0, 
8.0, 1.0], [1.0, 7.0, 8.0]], [[6.0, 6.0, 7.0], [3.0, 3.0, 1.0], [7.0, 5.0, 8.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 8.0], [2.0, 3.0, 
8.0]]], [[[2.0, 8.0, 4.0], [1.0, 6.0, 1.0], [6.0, 8.0, 3.0]], [[4.0, 8.0, 4.0], [5.0, 1.0, 2.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 
3.0], [6.0, 2.0, 4.0], [6.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 6.0], [7.0, 3.0, 3.0], [3.0, 6.0, 4.0]], 
[[6.0, 7.0, 2.0], [3.0, 5.0, 3.0], [4.0, 3.0, 6.0]], [[1.0, 5.0, 6.0], [7.0, 5.0, 5.0], [1.0, 4.0, 5.0]]], [[[8.0, 1.0, 3.0], 
[6.0, 5.0, 2.0], [7.0, 4.0, 3.0]], [[7.0, 7.0, 1.0], [4.0, 4.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 2.0, 8.0], [6.0, 7.0, 3.0], 
[6.0, 4.0, 2.0]]], [[[6.0, 1.0, 8.0], [1.0, 6.0, 6.0], [8.0, 3.0, 4.0]], [[7.0, 3.0, 3.0], [5.0, 5.0, 1.0], [7.0, 2.0, 4.0]], 
[[1.0, 6.0, 4.0], [7.0, 6.0, 4.0], [8.0, 6.0, 1.0]]], [[[6.0, 4.0, 4.0], [1.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[3.0, 7.0, 8.0], 
[6.0, 1.0, 7.0], [5.0, 8.0, 4.0]], [[5.0, 2.0, 8.0], [5.0, 2.0, 5.0], [4.0, 5.0, 2.0]]]])
      val=Data(numpy.array([3.0]),self.functionspace)
      arg[2,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [7.0, 5.0, 8.0], [5.0, 2.0, 2.0]], [[1.0, 5.0, 7.0], [4.0, 4.0, 3.0], [5.0, 
8.0, 7.0]], [[7.0, 3.0, 4.0], [2.0, 1.0, 6.0], [6.0, 8.0, 2.0]]], [[[7.0, 5.0, 3.0], [5.0, 5.0, 4.0], [3.0, 1.0, 6.0]], [[2.0, 
1.0, 5.0], [2.0, 7.0, 5.0], [5.0, 6.0, 2.0]], [[7.0, 6.0, 6.0], [5.0, 4.0, 6.0], [5.0, 1.0, 4.0]]], [[[7.0, 8.0, 8.0], [6.0, 
8.0, 1.0], [1.0, 7.0, 8.0]], [[6.0, 6.0, 7.0], [3.0, 3.0, 3.0], [7.0, 5.0, 8.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 8.0], [2.0, 3.0, 
8.0]]], [[[2.0, 8.0, 4.0], [1.0, 6.0, 1.0], [6.0, 8.0, 3.0]], [[4.0, 8.0, 4.0], [5.0, 1.0, 2.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 
3.0], [6.0, 2.0, 4.0], [6.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [7.0, 3.0, 3.0], [3.0, 6.0, 4.0]], 
[[6.0, 7.0, 2.0], [3.0, 5.0, 3.0], [4.0, 3.0, 6.0]], [[1.0, 5.0, 6.0], [7.0, 5.0, 5.0], [1.0, 4.0, 5.0]]], [[[8.0, 1.0, 3.0], 
[6.0, 5.0, 2.0], [7.0, 4.0, 3.0]], [[7.0, 7.0, 1.0], [4.0, 4.0, 2.0], [8.0, 7.0, 4.0]], [[3.0, 2.0, 8.0], [6.0, 7.0, 3.0], 
[6.0, 4.0, 2.0]]], [[[6.0, 1.0, 8.0], [1.0, 6.0, 6.0], [8.0, 3.0, 4.0]], [[7.0, 3.0, 3.0], [5.0, 5.0, 3.0], [7.0, 2.0, 4.0]], 
[[1.0, 6.0, 4.0], [7.0, 6.0, 4.0], [8.0, 6.0, 1.0]]], [[[6.0, 4.0, 4.0], [1.0, 3.0, 5.0], [6.0, 2.0, 8.0]], [[3.0, 7.0, 8.0], 
[6.0, 1.0, 7.0], [5.0, 8.0, 4.0]], [[5.0, 2.0, 8.0], [5.0, 2.0, 5.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 7.0, 7.0], [7.0, 6.0, 5.0]], [[2.0, 8.0, 6.0], [3.0, 7.0, 8.0], [6.0, 
8.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 1.0, 3.0], [5.0, 1.0, 2.0]]], [[[7.0, 4.0, 8.0], [4.0, 6.0, 3.0], [4.0, 6.0, 7.0]], [[5.0, 
7.0, 4.0], [3.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], [6.0, 1.0, 6.0], [8.0, 6.0, 7.0]]], [[[8.0, 4.0, 8.0], [2.0, 
7.0, 4.0], [5.0, 1.0, 6.0]], [[8.0, 4.0, 1.0], [7.0, 5.0, 6.0], [2.0, 4.0, 7.0]], [[3.0, 2.0, 4.0], [1.0, 3.0, 7.0], [8.0, 1.0, 
6.0]]], [[[4.0, 6.0, 7.0], [5.0, 7.0, 7.0], [4.0, 8.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 3.0, 3.0], [6.0, 1.0, 3.0]], [[1.0, 3.0, 
2.0], [8.0, 7.0, 5.0], [6.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 8.0], [4.0, 1.0, 5.0], [4.0, 8.0, 6.0]], 
[[3.0, 5.0, 3.0], [3.0, 1.0, 3.0], [3.0, 1.0, 5.0]], [[2.0, 1.0, 1.0], [7.0, 7.0, 5.0], [2.0, 8.0, 2.0]]], [[[3.0, 1.0, 7.0], 
[4.0, 1.0, 4.0], [2.0, 3.0, 8.0]], [[1.0, 5.0, 8.0], [6.0, 3.0, 5.0], [7.0, 7.0, 1.0]], [[4.0, 2.0, 6.0], [6.0, 5.0, 5.0], 
[3.0, 2.0, 6.0]]], [[[5.0, 3.0, 8.0], [6.0, 8.0, 2.0], [3.0, 4.0, 6.0]], [[8.0, 8.0, 3.0], [3.0, 3.0, 6.0], [1.0, 1.0, 7.0]], 
[[5.0, 3.0, 3.0], [2.0, 7.0, 7.0], [2.0, 1.0, 3.0]]], [[[5.0, 5.0, 7.0], [5.0, 1.0, 4.0], [1.0, 6.0, 4.0]], [[1.0, 1.0, 2.0], 
[5.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [5.0, 7.0, 4.0], [3.0, 3.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([1.0])+(1.-msk_val)*numpy.array([2.0])
      arg[2,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 7.0, 7.0], [7.0, 6.0, 5.0]], [[2.0, 8.0, 6.0], [3.0, 7.0, 8.0], [6.0, 
8.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 1.0, 3.0], [5.0, 1.0, 2.0]]], [[[7.0, 4.0, 8.0], [4.0, 6.0, 3.0], [4.0, 6.0, 7.0]], [[5.0, 
7.0, 4.0], [3.0, 8.0, 5.0], [3.0, 8.0, 2.0]], [[1.0, 6.0, 8.0], [6.0, 1.0, 6.0], [8.0, 6.0, 7.0]]], [[[8.0, 4.0, 8.0], [2.0, 
7.0, 4.0], [5.0, 1.0, 6.0]], [[8.0, 4.0, 1.0], [7.0, 5.0, 1.0], [2.0, 4.0, 7.0]], [[3.0, 2.0, 4.0], [1.0, 3.0, 7.0], [8.0, 1.0, 
6.0]]], [[[4.0, 6.0, 7.0], [5.0, 7.0, 7.0], [4.0, 8.0, 8.0]], [[2.0, 3.0, 3.0], [5.0, 3.0, 3.0], [6.0, 1.0, 3.0]], [[1.0, 3.0, 
2.0], [8.0, 7.0, 5.0], [6.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 8.0], [4.0, 1.0, 5.0], [4.0, 8.0, 6.0]], 
[[3.0, 5.0, 3.0], [3.0, 1.0, 3.0], [3.0, 1.0, 5.0]], [[2.0, 1.0, 1.0], [7.0, 7.0, 5.0], [2.0, 8.0, 2.0]]], [[[3.0, 1.0, 7.0], 
[4.0, 1.0, 4.0], [2.0, 3.0, 8.0]], [[1.0, 5.0, 8.0], [6.0, 3.0, 5.0], [7.0, 7.0, 1.0]], [[4.0, 2.0, 6.0], [6.0, 5.0, 5.0], 
[3.0, 2.0, 6.0]]], [[[5.0, 3.0, 8.0], [6.0, 8.0, 2.0], [3.0, 4.0, 6.0]], [[8.0, 8.0, 3.0], [3.0, 3.0, 2.0], [1.0, 1.0, 7.0]], 
[[5.0, 3.0, 3.0], [2.0, 7.0, 7.0], [2.0, 1.0, 3.0]]], [[[5.0, 5.0, 7.0], [5.0, 1.0, 4.0], [1.0, 6.0, 4.0]], [[1.0, 1.0, 2.0], 
[5.0, 5.0, 6.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [5.0, 7.0, 4.0], [3.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LUPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 4.0], [1.0, 4.0, 2.0], [1.0, 2.0, 5.0]], [[6.0, 1.0, 4.0], [6.0, 3.0, 8.0], [8.0, 
2.0, 1.0]], [[6.0, 7.0, 7.0], [7.0, 5.0, 1.0], [4.0, 1.0, 2.0]]], [[[6.0, 7.0, 1.0], [6.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[1.0, 
1.0, 6.0], [3.0, 4.0, 8.0], [3.0, 4.0, 3.0]], [[8.0, 2.0, 2.0], [8.0, 7.0, 7.0], [7.0, 5.0, 5.0]]], [[[8.0, 4.0, 2.0], [6.0, 
4.0, 2.0], [4.0, 7.0, 1.0]], [[2.0, 7.0, 8.0], [1.0, 2.0, 6.0], [3.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 3.0], [3.0, 6.0, 
4.0]]], [[[7.0, 5.0, 5.0], [3.0, 3.0, 7.0], [3.0, 7.0, 5.0]], [[2.0, 1.0, 6.0], [1.0, 6.0, 5.0], [7.0, 1.0, 4.0]], [[4.0, 1.0, 
1.0], [3.0, 6.0, 6.0], [4.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 1.0], [2.0, 5.0, 3.0], [5.0, 1.0, 4.0]], 
[[2.0, 7.0, 5.0], [5.0, 4.0, 3.0], [5.0, 1.0, 6.0]], [[4.0, 7.0, 1.0], [8.0, 5.0, 4.0], [2.0, 7.0, 1.0]]], [[[1.0, 2.0, 5.0], 
[3.0, 2.0, 3.0], [3.0, 5.0, 5.0]], [[1.0, 7.0, 2.0], [4.0, 6.0, 4.0], [4.0, 3.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 1.0, 5.0], 
[6.0, 2.0, 1.0]]], [[[5.0, 4.0, 6.0], [4.0, 7.0, 7.0], [8.0, 5.0, 7.0]], [[3.0, 1.0, 4.0], [3.0, 3.0, 8.0], [8.0, 4.0, 2.0]], 
[[8.0, 4.0, 6.0], [7.0, 2.0, 8.0], [6.0, 1.0, 6.0]]], [[[3.0, 7.0, 3.0], [5.0, 2.0, 3.0], [2.0, 4.0, 8.0]], [[3.0, 7.0, 2.0], 
[7.0, 3.0, 6.0], [8.0, 8.0, 7.0]], [[3.0, 6.0, 2.0], [1.0, 3.0, 3.0], [4.0, 4.0, 8.0]]]])
      val=numpy.array([[[2.0, 5.0], [3.0, 4.0]], [[1.0, 1.0], [5.0, 1.0]]])
      arg[2:,:2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 4.0], [1.0, 4.0, 2.0], [1.0, 2.0, 5.0]], [[6.0, 1.0, 4.0], [6.0, 3.0, 8.0], [8.0, 
2.0, 1.0]], [[6.0, 7.0, 7.0], [7.0, 5.0, 1.0], [4.0, 1.0, 2.0]]], [[[6.0, 7.0, 1.0], [6.0, 7.0, 3.0], [1.0, 3.0, 8.0]], [[1.0, 
1.0, 6.0], [3.0, 4.0, 8.0], [3.0, 4.0, 3.0]], [[8.0, 2.0, 2.0], [8.0, 7.0, 7.0], [7.0, 5.0, 5.0]]], [[[8.0, 4.0, 2.0], [2.0, 
5.0, 2.0], [4.0, 7.0, 1.0]], [[2.0, 7.0, 8.0], [3.0, 4.0, 6.0], [3.0, 5.0, 3.0]], [[8.0, 6.0, 7.0], [2.0, 2.0, 3.0], [3.0, 6.0, 
4.0]]], [[[7.0, 5.0, 5.0], [1.0, 1.0, 7.0], [3.0, 7.0, 5.0]], [[2.0, 1.0, 6.0], [5.0, 1.0, 5.0], [7.0, 1.0, 4.0]], [[4.0, 1.0, 
1.0], [3.0, 6.0, 6.0], [4.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 1.0], [2.0, 5.0, 3.0], [5.0, 1.0, 4.0]], 
[[2.0, 7.0, 5.0], [5.0, 4.0, 3.0], [5.0, 1.0, 6.0]], [[4.0, 7.0, 1.0], [8.0, 5.0, 4.0], [2.0, 7.0, 1.0]]], [[[1.0, 2.0, 5.0], 
[3.0, 2.0, 3.0], [3.0, 5.0, 5.0]], [[1.0, 7.0, 2.0], [4.0, 6.0, 4.0], [4.0, 3.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 1.0, 5.0], 
[6.0, 2.0, 1.0]]], [[[5.0, 4.0, 6.0], [2.0, 5.0, 7.0], [8.0, 5.0, 7.0]], [[3.0, 1.0, 4.0], [3.0, 4.0, 8.0], [8.0, 4.0, 2.0]], 
[[8.0, 4.0, 6.0], [7.0, 2.0, 8.0], [6.0, 1.0, 6.0]]], [[[3.0, 7.0, 3.0], [1.0, 1.0, 3.0], [2.0, 4.0, 8.0]], [[3.0, 7.0, 2.0], 
[5.0, 1.0, 6.0], [8.0, 8.0, 7.0]], [[3.0, 6.0, 2.0], [1.0, 3.0, 3.0], [4.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LUPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 6.0], [2.0, 5.0, 8.0], [5.0, 2.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 1.0], [4.0, 
1.0, 2.0]], [[7.0, 3.0, 7.0], [1.0, 3.0, 8.0], [1.0, 8.0, 3.0]]], [[[1.0, 2.0, 3.0], [2.0, 2.0, 1.0], [3.0, 4.0, 2.0]], [[8.0, 
1.0, 2.0], [3.0, 5.0, 6.0], [6.0, 2.0, 5.0]], [[5.0, 5.0, 8.0], [4.0, 4.0, 8.0], [1.0, 5.0, 5.0]]], [[[1.0, 5.0, 2.0], [2.0, 
7.0, 2.0], [4.0, 6.0, 2.0]], [[1.0, 2.0, 6.0], [3.0, 1.0, 8.0], [8.0, 5.0, 3.0]], [[8.0, 2.0, 8.0], [6.0, 6.0, 2.0], [8.0, 4.0, 
8.0]]], [[[3.0, 2.0, 1.0], [4.0, 1.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 8.0, 1.0], [2.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 8.0, 
4.0], [1.0, 6.0, 6.0], [1.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 6.0], [2.0, 3.0, 1.0], [2.0, 5.0, 5.0]], 
[[4.0, 7.0, 2.0], [2.0, 5.0, 2.0], [4.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [1.0, 2.0, 2.0], [1.0, 1.0, 5.0]]], [[[1.0, 3.0, 2.0], 
[8.0, 7.0, 1.0], [6.0, 2.0, 6.0]], [[8.0, 3.0, 3.0], [3.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 3.0], [8.0, 1.0, 4.0], 
[1.0, 8.0, 5.0]]], [[[2.0, 3.0, 6.0], [3.0, 1.0, 1.0], [8.0, 2.0, 8.0]], [[6.0, 4.0, 3.0], [1.0, 1.0, 8.0], [2.0, 4.0, 7.0]], 
[[6.0, 8.0, 8.0], [2.0, 4.0, 8.0], [1.0, 5.0, 7.0]]], [[[8.0, 1.0, 5.0], [4.0, 4.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 5.0, 1.0], 
[1.0, 6.0, 6.0], [5.0, 7.0, 6.0]], [[6.0, 6.0, 8.0], [7.0, 8.0, 8.0], [7.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[[7.0, 7.0], [7.0, 1.0]], [[8.0, 2.0], [2.0, 8.0]]]),self.functionspace)
      arg[2:,:2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 6.0], [2.0, 5.0, 8.0], [5.0, 2.0, 7.0]], [[8.0, 8.0, 3.0], [6.0, 1.0, 1.0], [4.0, 
1.0, 2.0]], [[7.0, 3.0, 7.0], [1.0, 3.0, 8.0], [1.0, 8.0, 3.0]]], [[[1.0, 2.0, 3.0], [2.0, 2.0, 1.0], [3.0, 4.0, 2.0]], [[8.0, 
1.0, 2.0], [3.0, 5.0, 6.0], [6.0, 2.0, 5.0]], [[5.0, 5.0, 8.0], [4.0, 4.0, 8.0], [1.0, 5.0, 5.0]]], [[[1.0, 5.0, 2.0], [7.0, 
7.0, 2.0], [4.0, 6.0, 2.0]], [[1.0, 2.0, 6.0], [7.0, 1.0, 8.0], [8.0, 5.0, 3.0]], [[8.0, 2.0, 8.0], [6.0, 6.0, 2.0], [8.0, 4.0, 
8.0]]], [[[3.0, 2.0, 1.0], [8.0, 2.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 8.0, 1.0], [2.0, 8.0, 6.0], [4.0, 7.0, 4.0]], [[4.0, 8.0, 
4.0], [1.0, 6.0, 6.0], [1.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 6.0], [2.0, 3.0, 1.0], [2.0, 5.0, 5.0]], 
[[4.0, 7.0, 2.0], [2.0, 5.0, 2.0], [4.0, 8.0, 7.0]], [[4.0, 8.0, 4.0], [1.0, 2.0, 2.0], [1.0, 1.0, 5.0]]], [[[1.0, 3.0, 2.0], 
[8.0, 7.0, 1.0], [6.0, 2.0, 6.0]], [[8.0, 3.0, 3.0], [3.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 3.0], [8.0, 1.0, 4.0], 
[1.0, 8.0, 5.0]]], [[[2.0, 3.0, 6.0], [7.0, 7.0, 1.0], [8.0, 2.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 1.0, 8.0], [2.0, 4.0, 7.0]], 
[[6.0, 8.0, 8.0], [2.0, 4.0, 8.0], [1.0, 5.0, 7.0]]], [[[8.0, 1.0, 5.0], [8.0, 2.0, 2.0], [8.0, 5.0, 7.0]], [[6.0, 5.0, 1.0], 
[2.0, 8.0, 6.0], [5.0, 7.0, 6.0]], [[6.0, 6.0, 8.0], [7.0, 8.0, 8.0], [7.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LUPU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 7.0, 7.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 3.0], [3.0, 7.0, 1.0], [5.0, 
7.0, 6.0]], [[7.0, 5.0, 4.0], [6.0, 6.0, 8.0], [8.0, 1.0, 1.0]]], [[[7.0, 3.0, 2.0], [6.0, 6.0, 4.0], [1.0, 1.0, 4.0]], [[3.0, 
7.0, 5.0], [6.0, 8.0, 1.0], [5.0, 2.0, 2.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 8.0], [7.0, 8.0, 2.0]]], [[[1.0, 4.0, 5.0], [8.0, 
3.0, 1.0], [2.0, 4.0, 6.0]], [[5.0, 5.0, 4.0], [1.0, 8.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 4.0, 6.0], [4.0, 1.0, 2.0], [4.0, 5.0, 
6.0]]], [[[3.0, 8.0, 4.0], [7.0, 4.0, 2.0], [7.0, 2.0, 2.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 5.0], [3.0, 4.0, 5.0]], [[1.0, 3.0, 
8.0], [3.0, 3.0, 8.0], [6.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 8.0], [7.0, 7.0, 5.0], [8.0, 6.0, 1.0]], 
[[7.0, 4.0, 7.0], [5.0, 5.0, 1.0], [6.0, 2.0, 2.0]], [[3.0, 5.0, 1.0], [8.0, 3.0, 7.0], [5.0, 1.0, 6.0]]], [[[8.0, 1.0, 2.0], 
[5.0, 3.0, 2.0], [4.0, 8.0, 4.0]], [[7.0, 6.0, 7.0], [5.0, 1.0, 7.0], [7.0, 8.0, 6.0]], [[7.0, 4.0, 3.0], [8.0, 7.0, 5.0], 
[7.0, 1.0, 5.0]]], [[[4.0, 3.0, 5.0], [1.0, 4.0, 3.0], [2.0, 1.0, 7.0]], [[2.0, 5.0, 1.0], [4.0, 1.0, 8.0], [6.0, 5.0, 6.0]], 
[[1.0, 6.0, 6.0], [5.0, 5.0, 6.0], [8.0, 3.0, 5.0]]], [[[6.0, 8.0, 8.0], [1.0, 1.0, 2.0], [7.0, 7.0, 6.0]], [[6.0, 4.0, 7.0], 
[3.0, 7.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 3.0, 4.0], [8.0, 5.0, 3.0], [8.0, 4.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 1.0], [6.0, 2.0]], [[8.0, 1.0], [3.0, 4.0]]])+(1.-msk_val)*numpy.array([[[2.0, 3.0], 
[5.0, 7.0]], [[5.0, 1.0], [8.0, 8.0]]])
      arg[2:,:2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 6.0], [8.0, 7.0, 7.0], [4.0, 7.0, 1.0]], [[3.0, 2.0, 3.0], [3.0, 7.0, 1.0], [5.0, 
7.0, 6.0]], [[7.0, 5.0, 4.0], [6.0, 6.0, 8.0], [8.0, 1.0, 1.0]]], [[[7.0, 3.0, 2.0], [6.0, 6.0, 4.0], [1.0, 1.0, 4.0]], [[3.0, 
7.0, 5.0], [6.0, 8.0, 1.0], [5.0, 2.0, 2.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 8.0], [7.0, 8.0, 2.0]]], [[[1.0, 4.0, 5.0], [1.0, 
1.0, 1.0], [2.0, 4.0, 6.0]], [[5.0, 5.0, 4.0], [6.0, 2.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 4.0, 6.0], [4.0, 1.0, 2.0], [4.0, 5.0, 
6.0]]], [[[3.0, 8.0, 4.0], [8.0, 1.0, 2.0], [7.0, 2.0, 2.0]], [[6.0, 3.0, 5.0], [3.0, 4.0, 5.0], [3.0, 4.0, 5.0]], [[1.0, 3.0, 
8.0], [3.0, 3.0, 8.0], [6.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 8.0], [7.0, 7.0, 5.0], [8.0, 6.0, 1.0]], 
[[7.0, 4.0, 7.0], [5.0, 5.0, 1.0], [6.0, 2.0, 2.0]], [[3.0, 5.0, 1.0], [8.0, 3.0, 7.0], [5.0, 1.0, 6.0]]], [[[8.0, 1.0, 2.0], 
[5.0, 3.0, 2.0], [4.0, 8.0, 4.0]], [[7.0, 6.0, 7.0], [5.0, 1.0, 7.0], [7.0, 8.0, 6.0]], [[7.0, 4.0, 3.0], [8.0, 7.0, 5.0], 
[7.0, 1.0, 5.0]]], [[[4.0, 3.0, 5.0], [2.0, 3.0, 3.0], [2.0, 1.0, 7.0]], [[2.0, 5.0, 1.0], [5.0, 7.0, 8.0], [6.0, 5.0, 6.0]], 
[[1.0, 6.0, 6.0], [5.0, 5.0, 6.0], [8.0, 3.0, 5.0]]], [[[6.0, 8.0, 8.0], [5.0, 1.0, 2.0], [7.0, 7.0, 6.0]], [[6.0, 4.0, 7.0], 
[8.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[2.0, 3.0, 4.0], [8.0, 5.0, 3.0], [8.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 7.0], [5.0, 5.0, 6.0], [2.0, 4.0, 2.0]], [[3.0, 2.0, 8.0], [2.0, 6.0, 5.0], [4.0, 
4.0, 5.0]], [[1.0, 8.0, 2.0], [1.0, 2.0, 2.0], [8.0, 1.0, 4.0]]], [[[1.0, 4.0, 5.0], [6.0, 8.0, 1.0], [1.0, 8.0, 2.0]], [[5.0, 
4.0, 7.0], [6.0, 2.0, 8.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 4.0], [8.0, 4.0, 8.0], [5.0, 4.0, 8.0]]], [[[7.0, 7.0, 1.0], [3.0, 
2.0, 5.0], [7.0, 2.0, 4.0]], [[2.0, 8.0, 4.0], [2.0, 6.0, 6.0], [5.0, 5.0, 2.0]], [[5.0, 4.0, 5.0], [7.0, 8.0, 7.0], [3.0, 4.0, 
5.0]]], [[[6.0, 4.0, 3.0], [7.0, 5.0, 6.0], [5.0, 2.0, 4.0]], [[8.0, 5.0, 6.0], [7.0, 1.0, 1.0], [2.0, 6.0, 4.0]], [[1.0, 4.0, 
7.0], [6.0, 7.0, 2.0], [1.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 3.0], [2.0, 2.0, 5.0], [6.0, 3.0, 3.0]], 
[[7.0, 2.0, 3.0], [4.0, 8.0, 1.0], [7.0, 7.0, 2.0]], [[1.0, 3.0, 5.0], [2.0, 5.0, 5.0], [1.0, 4.0, 7.0]]], [[[7.0, 7.0, 1.0], 
[6.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[2.0, 8.0, 2.0], [5.0, 5.0, 4.0], [2.0, 1.0, 5.0]], [[2.0, 3.0, 2.0], [7.0, 4.0, 7.0], 
[4.0, 4.0, 1.0]]], [[[1.0, 1.0, 8.0], [3.0, 4.0, 5.0], [8.0, 8.0, 7.0]], [[3.0, 3.0, 2.0], [5.0, 2.0, 4.0], [2.0, 5.0, 2.0]], 
[[7.0, 7.0, 2.0], [2.0, 8.0, 7.0], [7.0, 5.0, 1.0]]], [[[2.0, 5.0, 1.0], [3.0, 2.0, 3.0], [2.0, 6.0, 2.0]], [[5.0, 6.0, 7.0], 
[5.0, 6.0, 3.0], [1.0, 6.0, 8.0]], [[6.0, 3.0, 1.0], [5.0, 8.0, 8.0], [8.0, 3.0, 7.0]]]])
      val=numpy.array([[2.0], [2.0], [3.0], [2.0]])
      arg[:,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 7.0], [5.0, 5.0, 6.0], [2.0, 4.0, 2.0]], [[3.0, 2.0, 8.0], [2.0, 6.0, 2.0], [4.0, 
4.0, 5.0]], [[1.0, 8.0, 2.0], [1.0, 2.0, 2.0], [8.0, 1.0, 4.0]]], [[[1.0, 4.0, 5.0], [6.0, 8.0, 1.0], [1.0, 8.0, 2.0]], [[5.0, 
4.0, 7.0], [6.0, 2.0, 2.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 4.0], [8.0, 4.0, 8.0], [5.0, 4.0, 8.0]]], [[[7.0, 7.0, 1.0], [3.0, 
2.0, 5.0], [7.0, 2.0, 4.0]], [[2.0, 8.0, 4.0], [2.0, 6.0, 3.0], [5.0, 5.0, 2.0]], [[5.0, 4.0, 5.0], [7.0, 8.0, 7.0], [3.0, 4.0, 
5.0]]], [[[6.0, 4.0, 3.0], [7.0, 5.0, 6.0], [5.0, 2.0, 4.0]], [[8.0, 5.0, 6.0], [7.0, 1.0, 2.0], [2.0, 6.0, 4.0]], [[1.0, 4.0, 
7.0], [6.0, 7.0, 2.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 3.0], [2.0, 2.0, 5.0], [6.0, 3.0, 3.0]], 
[[7.0, 2.0, 3.0], [4.0, 8.0, 2.0], [7.0, 7.0, 2.0]], [[1.0, 3.0, 5.0], [2.0, 5.0, 5.0], [1.0, 4.0, 7.0]]], [[[7.0, 7.0, 1.0], 
[6.0, 1.0, 8.0], [1.0, 8.0, 6.0]], [[2.0, 8.0, 2.0], [5.0, 5.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 3.0, 2.0], [7.0, 4.0, 7.0], 
[4.0, 4.0, 1.0]]], [[[1.0, 1.0, 8.0], [3.0, 4.0, 5.0], [8.0, 8.0, 7.0]], [[3.0, 3.0, 2.0], [5.0, 2.0, 3.0], [2.0, 5.0, 2.0]], 
[[7.0, 7.0, 2.0], [2.0, 8.0, 7.0], [7.0, 5.0, 1.0]]], [[[2.0, 5.0, 1.0], [3.0, 2.0, 3.0], [2.0, 6.0, 2.0]], [[5.0, 6.0, 7.0], 
[5.0, 6.0, 2.0], [1.0, 6.0, 8.0]], [[6.0, 3.0, 1.0], [5.0, 8.0, 8.0], [8.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 4.0], [3.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[8.0, 5.0, 3.0], [3.0, 2.0, 4.0], [7.0, 
7.0, 7.0]], [[1.0, 2.0, 3.0], [5.0, 7.0, 8.0], [4.0, 6.0, 6.0]]], [[[5.0, 7.0, 4.0], [2.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 
2.0, 4.0], [2.0, 6.0, 3.0], [4.0, 2.0, 5.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 4.0], [5.0, 5.0, 4.0]]], [[[8.0, 2.0, 7.0], [7.0, 
3.0, 5.0], [5.0, 2.0, 6.0]], [[5.0, 4.0, 7.0], [1.0, 6.0, 2.0], [8.0, 4.0, 5.0]], [[7.0, 3.0, 6.0], [4.0, 8.0, 2.0], [1.0, 3.0, 
6.0]]], [[[3.0, 8.0, 4.0], [6.0, 5.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 8.0, 8.0], [7.0, 5.0, 1.0], [7.0, 3.0, 5.0]], [[2.0, 7.0, 
8.0], [3.0, 4.0, 7.0], [4.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 1.0], [5.0, 8.0, 2.0], [3.0, 5.0, 4.0]], 
[[6.0, 1.0, 1.0], [4.0, 2.0, 1.0], [4.0, 3.0, 6.0]], [[8.0, 8.0, 4.0], [6.0, 4.0, 6.0], [2.0, 2.0, 4.0]]], [[[5.0, 8.0, 4.0], 
[2.0, 4.0, 8.0], [1.0, 3.0, 3.0]], [[8.0, 2.0, 4.0], [3.0, 1.0, 4.0], [8.0, 4.0, 6.0]], [[4.0, 7.0, 2.0], [1.0, 2.0, 5.0], 
[4.0, 3.0, 1.0]]], [[[5.0, 4.0, 6.0], [2.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 2.0, 8.0], [6.0, 7.0, 6.0], [2.0, 5.0, 5.0]], 
[[7.0, 4.0, 2.0], [2.0, 8.0, 8.0], [8.0, 3.0, 2.0]]], [[[8.0, 3.0, 5.0], [7.0, 6.0, 6.0], [8.0, 5.0, 1.0]], [[3.0, 7.0, 4.0], 
[5.0, 6.0, 5.0], [1.0, 6.0, 4.0]], [[3.0, 7.0, 5.0], [3.0, 8.0, 1.0], [2.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[4.0], [7.0], [3.0], [3.0]]),self.functionspace)
      arg[:,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 4.0], [3.0, 7.0, 6.0], [5.0, 7.0, 4.0]], [[8.0, 5.0, 3.0], [3.0, 2.0, 4.0], [7.0, 
7.0, 7.0]], [[1.0, 2.0, 3.0], [5.0, 7.0, 8.0], [4.0, 6.0, 6.0]]], [[[5.0, 7.0, 4.0], [2.0, 7.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 
2.0, 4.0], [2.0, 6.0, 7.0], [4.0, 2.0, 5.0]], [[8.0, 3.0, 6.0], [3.0, 8.0, 4.0], [5.0, 5.0, 4.0]]], [[[8.0, 2.0, 7.0], [7.0, 
3.0, 5.0], [5.0, 2.0, 6.0]], [[5.0, 4.0, 7.0], [1.0, 6.0, 3.0], [8.0, 4.0, 5.0]], [[7.0, 3.0, 6.0], [4.0, 8.0, 2.0], [1.0, 3.0, 
6.0]]], [[[3.0, 8.0, 4.0], [6.0, 5.0, 5.0], [5.0, 6.0, 3.0]], [[7.0, 8.0, 8.0], [7.0, 5.0, 3.0], [7.0, 3.0, 5.0]], [[2.0, 7.0, 
8.0], [3.0, 4.0, 7.0], [4.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 1.0], [5.0, 8.0, 2.0], [3.0, 5.0, 4.0]], 
[[6.0, 1.0, 1.0], [4.0, 2.0, 4.0], [4.0, 3.0, 6.0]], [[8.0, 8.0, 4.0], [6.0, 4.0, 6.0], [2.0, 2.0, 4.0]]], [[[5.0, 8.0, 4.0], 
[2.0, 4.0, 8.0], [1.0, 3.0, 3.0]], [[8.0, 2.0, 4.0], [3.0, 1.0, 7.0], [8.0, 4.0, 6.0]], [[4.0, 7.0, 2.0], [1.0, 2.0, 5.0], 
[4.0, 3.0, 1.0]]], [[[5.0, 4.0, 6.0], [2.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[4.0, 2.0, 8.0], [6.0, 7.0, 3.0], [2.0, 5.0, 5.0]], 
[[7.0, 4.0, 2.0], [2.0, 8.0, 8.0], [8.0, 3.0, 2.0]]], [[[8.0, 3.0, 5.0], [7.0, 6.0, 6.0], [8.0, 5.0, 1.0]], [[3.0, 7.0, 4.0], 
[5.0, 6.0, 3.0], [1.0, 6.0, 4.0]], [[3.0, 7.0, 5.0], [3.0, 8.0, 1.0], [2.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 5.0], [8.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[3.0, 5.0, 4.0], [7.0, 4.0, 3.0], [1.0, 
8.0, 1.0]], [[8.0, 2.0, 6.0], [4.0, 1.0, 8.0], [6.0, 3.0, 4.0]]], [[[5.0, 8.0, 2.0], [4.0, 3.0, 1.0], [8.0, 7.0, 7.0]], [[2.0, 
4.0, 4.0], [1.0, 1.0, 1.0], [7.0, 6.0, 4.0]], [[8.0, 1.0, 6.0], [4.0, 1.0, 3.0], [2.0, 2.0, 3.0]]], [[[4.0, 4.0, 3.0], [2.0, 
4.0, 7.0], [3.0, 3.0, 3.0]], [[5.0, 6.0, 3.0], [4.0, 1.0, 6.0], [4.0, 3.0, 5.0]], [[7.0, 6.0, 7.0], [3.0, 7.0, 5.0], [3.0, 6.0, 
6.0]]], [[[2.0, 2.0, 3.0], [8.0, 3.0, 2.0], [4.0, 2.0, 7.0]], [[4.0, 7.0, 1.0], [8.0, 4.0, 5.0], [6.0, 3.0, 8.0]], [[4.0, 2.0, 
2.0], [1.0, 1.0, 3.0], [2.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 5.0], [4.0, 6.0, 7.0], [2.0, 3.0, 5.0]], 
[[3.0, 7.0, 8.0], [4.0, 7.0, 4.0], [5.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [3.0, 8.0, 5.0], [1.0, 8.0, 1.0]]], [[[3.0, 8.0, 2.0], 
[3.0, 6.0, 6.0], [3.0, 4.0, 4.0]], [[1.0, 3.0, 8.0], [4.0, 3.0, 3.0], [1.0, 8.0, 6.0]], [[3.0, 5.0, 5.0], [3.0, 7.0, 1.0], 
[4.0, 1.0, 4.0]]], [[[1.0, 5.0, 1.0], [3.0, 7.0, 8.0], [5.0, 5.0, 2.0]], [[7.0, 2.0, 1.0], [2.0, 4.0, 5.0], [1.0, 4.0, 8.0]], 
[[4.0, 5.0, 8.0], [8.0, 5.0, 6.0], [7.0, 7.0, 6.0]]], [[[8.0, 7.0, 8.0], [3.0, 6.0, 5.0], [8.0, 4.0, 8.0]], [[7.0, 6.0, 5.0], 
[2.0, 6.0, 5.0], [3.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [8.0, 3.0, 2.0], [1.0, 6.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0], [3.0], [2.0], [4.0]])+(1.-msk_val)*numpy.array([[6.0], [8.0], [6.0], [8.0]])
      arg[:,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 5.0], [8.0, 6.0, 8.0], [3.0, 1.0, 5.0]], [[3.0, 5.0, 4.0], [7.0, 4.0, 2.0], [1.0, 
8.0, 1.0]], [[8.0, 2.0, 6.0], [4.0, 1.0, 8.0], [6.0, 3.0, 4.0]]], [[[5.0, 8.0, 2.0], [4.0, 3.0, 1.0], [8.0, 7.0, 7.0]], [[2.0, 
4.0, 4.0], [1.0, 1.0, 3.0], [7.0, 6.0, 4.0]], [[8.0, 1.0, 6.0], [4.0, 1.0, 3.0], [2.0, 2.0, 3.0]]], [[[4.0, 4.0, 3.0], [2.0, 
4.0, 7.0], [3.0, 3.0, 3.0]], [[5.0, 6.0, 3.0], [4.0, 1.0, 2.0], [4.0, 3.0, 5.0]], [[7.0, 6.0, 7.0], [3.0, 7.0, 5.0], [3.0, 6.0, 
6.0]]], [[[2.0, 2.0, 3.0], [8.0, 3.0, 2.0], [4.0, 2.0, 7.0]], [[4.0, 7.0, 1.0], [8.0, 4.0, 4.0], [6.0, 3.0, 8.0]], [[4.0, 2.0, 
2.0], [1.0, 1.0, 3.0], [2.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 5.0], [4.0, 6.0, 7.0], [2.0, 3.0, 5.0]], 
[[3.0, 7.0, 8.0], [4.0, 7.0, 6.0], [5.0, 7.0, 6.0]], [[2.0, 3.0, 2.0], [3.0, 8.0, 5.0], [1.0, 8.0, 1.0]]], [[[3.0, 8.0, 2.0], 
[3.0, 6.0, 6.0], [3.0, 4.0, 4.0]], [[1.0, 3.0, 8.0], [4.0, 3.0, 8.0], [1.0, 8.0, 6.0]], [[3.0, 5.0, 5.0], [3.0, 7.0, 1.0], 
[4.0, 1.0, 4.0]]], [[[1.0, 5.0, 1.0], [3.0, 7.0, 8.0], [5.0, 5.0, 2.0]], [[7.0, 2.0, 1.0], [2.0, 4.0, 6.0], [1.0, 4.0, 8.0]], 
[[4.0, 5.0, 8.0], [8.0, 5.0, 6.0], [7.0, 7.0, 6.0]]], [[[8.0, 7.0, 8.0], [3.0, 6.0, 5.0], [8.0, 4.0, 8.0]], [[7.0, 6.0, 5.0], 
[2.0, 6.0, 8.0], [3.0, 2.0, 6.0]], [[1.0, 7.0, 8.0], [8.0, 3.0, 2.0], [1.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 1.0], [4.0, 8.0, 5.0], [7.0, 7.0, 7.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 5.0], [7.0, 
1.0, 7.0]], [[7.0, 8.0, 1.0], [3.0, 2.0, 8.0], [6.0, 7.0, 1.0]]], [[[7.0, 1.0, 1.0], [7.0, 6.0, 1.0], [3.0, 1.0, 4.0]], [[7.0, 
2.0, 5.0], [5.0, 2.0, 3.0], [6.0, 8.0, 4.0]], [[6.0, 5.0, 5.0], [8.0, 6.0, 6.0], [3.0, 6.0, 1.0]]], [[[8.0, 1.0, 2.0], [3.0, 
1.0, 5.0], [1.0, 8.0, 1.0]], [[3.0, 3.0, 5.0], [4.0, 3.0, 5.0], [8.0, 6.0, 2.0]], [[8.0, 7.0, 6.0], [3.0, 8.0, 6.0], [6.0, 2.0, 
1.0]]], [[[6.0, 3.0, 2.0], [5.0, 8.0, 5.0], [2.0, 6.0, 5.0]], [[6.0, 1.0, 4.0], [3.0, 6.0, 1.0], [2.0, 2.0, 7.0]], [[7.0, 8.0, 
6.0], [2.0, 1.0, 7.0], [1.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 5.0], [4.0, 5.0, 2.0], [5.0, 5.0, 5.0]], 
[[4.0, 5.0, 5.0], [2.0, 4.0, 6.0], [8.0, 4.0, 4.0]], [[3.0, 7.0, 4.0], [7.0, 2.0, 7.0], [2.0, 8.0, 1.0]]], [[[5.0, 7.0, 5.0], 
[5.0, 7.0, 2.0], [7.0, 8.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 6.0, 6.0], [3.0, 5.0, 7.0]], [[5.0, 8.0, 4.0], [1.0, 2.0, 5.0], 
[4.0, 7.0, 2.0]]], [[[5.0, 4.0, 6.0], [5.0, 4.0, 7.0], [6.0, 6.0, 4.0]], [[7.0, 3.0, 7.0], [6.0, 6.0, 5.0], [4.0, 3.0, 2.0]], 
[[1.0, 8.0, 1.0], [3.0, 3.0, 5.0], [7.0, 2.0, 8.0]]], [[[4.0, 1.0, 6.0], [5.0, 2.0, 2.0], [6.0, 7.0, 6.0]], [[2.0, 7.0, 1.0], 
[7.0, 7.0, 4.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 2.0], [5.0, 7.0, 3.0], [4.0, 6.0, 4.0]]]])
      val=numpy.array([[[2.0, 5.0, 8.0]], [[1.0, 3.0, 3.0]], [[3.0, 4.0, 4.0]], [[8.0, 8.0, 1.0]]])
      arg[:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 1.0], [4.0, 8.0, 5.0], [7.0, 7.0, 7.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 5.0], [7.0, 
1.0, 7.0]], [[7.0, 8.0, 1.0], [2.0, 5.0, 8.0], [6.0, 7.0, 1.0]]], [[[7.0, 1.0, 1.0], [7.0, 6.0, 1.0], [3.0, 1.0, 4.0]], [[7.0, 
2.0, 5.0], [5.0, 2.0, 3.0], [6.0, 8.0, 4.0]], [[6.0, 5.0, 5.0], [1.0, 3.0, 3.0], [3.0, 6.0, 1.0]]], [[[8.0, 1.0, 2.0], [3.0, 
1.0, 5.0], [1.0, 8.0, 1.0]], [[3.0, 3.0, 5.0], [4.0, 3.0, 5.0], [8.0, 6.0, 2.0]], [[8.0, 7.0, 6.0], [3.0, 4.0, 4.0], [6.0, 2.0, 
1.0]]], [[[6.0, 3.0, 2.0], [5.0, 8.0, 5.0], [2.0, 6.0, 5.0]], [[6.0, 1.0, 4.0], [3.0, 6.0, 1.0], [2.0, 2.0, 7.0]], [[7.0, 8.0, 
6.0], [8.0, 8.0, 1.0], [1.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 5.0], [4.0, 5.0, 2.0], [5.0, 5.0, 5.0]], 
[[4.0, 5.0, 5.0], [2.0, 4.0, 6.0], [8.0, 4.0, 4.0]], [[3.0, 7.0, 4.0], [2.0, 5.0, 8.0], [2.0, 8.0, 1.0]]], [[[5.0, 7.0, 5.0], 
[5.0, 7.0, 2.0], [7.0, 8.0, 7.0]], [[3.0, 7.0, 7.0], [2.0, 6.0, 6.0], [3.0, 5.0, 7.0]], [[5.0, 8.0, 4.0], [1.0, 3.0, 3.0], 
[4.0, 7.0, 2.0]]], [[[5.0, 4.0, 6.0], [5.0, 4.0, 7.0], [6.0, 6.0, 4.0]], [[7.0, 3.0, 7.0], [6.0, 6.0, 5.0], [4.0, 3.0, 2.0]], 
[[1.0, 8.0, 1.0], [3.0, 4.0, 4.0], [7.0, 2.0, 8.0]]], [[[4.0, 1.0, 6.0], [5.0, 2.0, 2.0], [6.0, 7.0, 6.0]], [[2.0, 7.0, 1.0], 
[7.0, 7.0, 4.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 2.0], [8.0, 8.0, 1.0], [4.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 2.0], [4.0, 3.0, 7.0], [3.0, 7.0, 7.0]], [[2.0, 8.0, 7.0], [1.0, 2.0, 7.0], [6.0, 
8.0, 2.0]], [[2.0, 5.0, 2.0], [1.0, 8.0, 6.0], [2.0, 8.0, 4.0]]], [[[5.0, 6.0, 3.0], [5.0, 8.0, 3.0], [1.0, 8.0, 7.0]], [[1.0, 
4.0, 3.0], [3.0, 5.0, 5.0], [6.0, 4.0, 7.0]], [[1.0, 5.0, 6.0], [6.0, 3.0, 1.0], [2.0, 1.0, 4.0]]], [[[5.0, 8.0, 2.0], [7.0, 
4.0, 4.0], [7.0, 5.0, 5.0]], [[3.0, 6.0, 8.0], [5.0, 1.0, 3.0], [2.0, 2.0, 8.0]], [[3.0, 6.0, 8.0], [6.0, 2.0, 6.0], [7.0, 5.0, 
5.0]]], [[[2.0, 8.0, 6.0], [2.0, 2.0, 1.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 6.0], [7.0, 6.0, 1.0], [3.0, 7.0, 4.0]], [[6.0, 2.0, 
4.0], [5.0, 8.0, 5.0], [6.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 4.0], [6.0, 2.0, 1.0], [2.0, 6.0, 6.0]], 
[[6.0, 4.0, 2.0], [7.0, 6.0, 1.0], [4.0, 8.0, 6.0]], [[3.0, 4.0, 8.0], [3.0, 8.0, 8.0], [4.0, 5.0, 2.0]]], [[[6.0, 8.0, 1.0], 
[5.0, 6.0, 2.0], [4.0, 8.0, 7.0]], [[1.0, 4.0, 1.0], [1.0, 2.0, 5.0], [4.0, 5.0, 8.0]], [[4.0, 6.0, 3.0], [2.0, 2.0, 1.0], 
[4.0, 1.0, 8.0]]], [[[5.0, 1.0, 6.0], [3.0, 5.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 5.0], [8.0, 3.0, 4.0]], 
[[6.0, 1.0, 1.0], [8.0, 5.0, 8.0], [1.0, 8.0, 6.0]]], [[[8.0, 5.0, 3.0], [2.0, 5.0, 2.0], [1.0, 3.0, 1.0]], [[2.0, 8.0, 8.0], 
[5.0, 1.0, 2.0], [1.0, 5.0, 1.0]], [[7.0, 2.0, 2.0], [8.0, 6.0, 2.0], [3.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[8.0, 4.0, 7.0]], [[7.0, 2.0, 1.0]], [[3.0, 5.0, 1.0]], [[4.0, 4.0, 3.0]]]),self.functionspace)
      arg[:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 2.0], [4.0, 3.0, 7.0], [3.0, 7.0, 7.0]], [[2.0, 8.0, 7.0], [1.0, 2.0, 7.0], [6.0, 
8.0, 2.0]], [[2.0, 5.0, 2.0], [8.0, 4.0, 7.0], [2.0, 8.0, 4.0]]], [[[5.0, 6.0, 3.0], [5.0, 8.0, 3.0], [1.0, 8.0, 7.0]], [[1.0, 
4.0, 3.0], [3.0, 5.0, 5.0], [6.0, 4.0, 7.0]], [[1.0, 5.0, 6.0], [7.0, 2.0, 1.0], [2.0, 1.0, 4.0]]], [[[5.0, 8.0, 2.0], [7.0, 
4.0, 4.0], [7.0, 5.0, 5.0]], [[3.0, 6.0, 8.0], [5.0, 1.0, 3.0], [2.0, 2.0, 8.0]], [[3.0, 6.0, 8.0], [3.0, 5.0, 1.0], [7.0, 5.0, 
5.0]]], [[[2.0, 8.0, 6.0], [2.0, 2.0, 1.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 6.0], [7.0, 6.0, 1.0], [3.0, 7.0, 4.0]], [[6.0, 2.0, 
4.0], [4.0, 4.0, 3.0], [6.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 4.0], [6.0, 2.0, 1.0], [2.0, 6.0, 6.0]], 
[[6.0, 4.0, 2.0], [7.0, 6.0, 1.0], [4.0, 8.0, 6.0]], [[3.0, 4.0, 8.0], [8.0, 4.0, 7.0], [4.0, 5.0, 2.0]]], [[[6.0, 8.0, 1.0], 
[5.0, 6.0, 2.0], [4.0, 8.0, 7.0]], [[1.0, 4.0, 1.0], [1.0, 2.0, 5.0], [4.0, 5.0, 8.0]], [[4.0, 6.0, 3.0], [7.0, 2.0, 1.0], 
[4.0, 1.0, 8.0]]], [[[5.0, 1.0, 6.0], [3.0, 5.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 6.0], [7.0, 1.0, 5.0], [8.0, 3.0, 4.0]], 
[[6.0, 1.0, 1.0], [3.0, 5.0, 1.0], [1.0, 8.0, 6.0]]], [[[8.0, 5.0, 3.0], [2.0, 5.0, 2.0], [1.0, 3.0, 1.0]], [[2.0, 8.0, 8.0], 
[5.0, 1.0, 2.0], [1.0, 5.0, 1.0]], [[7.0, 2.0, 2.0], [4.0, 4.0, 3.0], [3.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 1.0], [3.0, 2.0, 8.0], [6.0, 4.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 6.0], [4.0, 
4.0, 1.0]], [[7.0, 1.0, 4.0], [5.0, 5.0, 6.0], [5.0, 4.0, 6.0]]], [[[1.0, 5.0, 7.0], [2.0, 4.0, 2.0], [1.0, 8.0, 6.0]], [[8.0, 
8.0, 6.0], [2.0, 4.0, 6.0], [5.0, 3.0, 7.0]], [[1.0, 4.0, 3.0], [7.0, 4.0, 8.0], [1.0, 3.0, 3.0]]], [[[4.0, 3.0, 2.0], [7.0, 
1.0, 6.0], [8.0, 5.0, 3.0]], [[7.0, 5.0, 8.0], [1.0, 4.0, 1.0], [7.0, 1.0, 3.0]], [[7.0, 3.0, 5.0], [3.0, 6.0, 1.0], [3.0, 1.0, 
4.0]]], [[[3.0, 8.0, 5.0], [7.0, 1.0, 6.0], [8.0, 8.0, 3.0]], [[3.0, 5.0, 3.0], [8.0, 7.0, 1.0], [7.0, 7.0, 7.0]], [[8.0, 1.0, 
4.0], [3.0, 6.0, 7.0], [8.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 3.0], [2.0, 6.0, 3.0], [6.0, 7.0, 4.0]], 
[[1.0, 5.0, 3.0], [4.0, 3.0, 5.0], [4.0, 5.0, 3.0]], [[3.0, 6.0, 5.0], [8.0, 2.0, 2.0], [4.0, 3.0, 4.0]]], [[[5.0, 7.0, 7.0], 
[7.0, 6.0, 5.0], [3.0, 8.0, 1.0]], [[1.0, 7.0, 1.0], [8.0, 3.0, 6.0], [6.0, 3.0, 3.0]], [[1.0, 2.0, 5.0], [2.0, 5.0, 1.0], 
[4.0, 2.0, 8.0]]], [[[2.0, 6.0, 2.0], [1.0, 7.0, 4.0], [3.0, 8.0, 6.0]], [[8.0, 7.0, 7.0], [1.0, 2.0, 8.0], [2.0, 2.0, 1.0]], 
[[6.0, 8.0, 3.0], [3.0, 6.0, 1.0], [4.0, 4.0, 4.0]]], [[[7.0, 1.0, 7.0], [5.0, 4.0, 1.0], [3.0, 2.0, 1.0]], [[4.0, 2.0, 5.0], 
[1.0, 2.0, 1.0], [3.0, 3.0, 6.0]], [[4.0, 5.0, 7.0], [7.0, 5.0, 4.0], [5.0, 8.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 4.0, 1.0]], [[7.0, 1.0, 2.0]], [[2.0, 4.0, 1.0]], [[7.0, 3.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[1.0, 4.0, 3.0]], [[2.0, 6.0, 4.0]], [[2.0, 2.0, 3.0]], [[6.0, 3.0, 1.0]]])
      arg[:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 1.0], [3.0, 2.0, 8.0], [6.0, 4.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 6.0], [4.0, 
4.0, 1.0]], [[7.0, 1.0, 4.0], [1.0, 4.0, 1.0], [5.0, 4.0, 6.0]]], [[[1.0, 5.0, 7.0], [2.0, 4.0, 2.0], [1.0, 8.0, 6.0]], [[8.0, 
8.0, 6.0], [2.0, 4.0, 6.0], [5.0, 3.0, 7.0]], [[1.0, 4.0, 3.0], [7.0, 1.0, 2.0], [1.0, 3.0, 3.0]]], [[[4.0, 3.0, 2.0], [7.0, 
1.0, 6.0], [8.0, 5.0, 3.0]], [[7.0, 5.0, 8.0], [1.0, 4.0, 1.0], [7.0, 1.0, 3.0]], [[7.0, 3.0, 5.0], [2.0, 4.0, 1.0], [3.0, 1.0, 
4.0]]], [[[3.0, 8.0, 5.0], [7.0, 1.0, 6.0], [8.0, 8.0, 3.0]], [[3.0, 5.0, 3.0], [8.0, 7.0, 1.0], [7.0, 7.0, 7.0]], [[8.0, 1.0, 
4.0], [7.0, 3.0, 1.0], [8.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [2.0, 6.0, 3.0], [6.0, 7.0, 4.0]], 
[[1.0, 5.0, 3.0], [4.0, 3.0, 5.0], [4.0, 5.0, 3.0]], [[3.0, 6.0, 5.0], [1.0, 4.0, 3.0], [4.0, 3.0, 4.0]]], [[[5.0, 7.0, 7.0], 
[7.0, 6.0, 5.0], [3.0, 8.0, 1.0]], [[1.0, 7.0, 1.0], [8.0, 3.0, 6.0], [6.0, 3.0, 3.0]], [[1.0, 2.0, 5.0], [2.0, 6.0, 4.0], 
[4.0, 2.0, 8.0]]], [[[2.0, 6.0, 2.0], [1.0, 7.0, 4.0], [3.0, 8.0, 6.0]], [[8.0, 7.0, 7.0], [1.0, 2.0, 8.0], [2.0, 2.0, 1.0]], 
[[6.0, 8.0, 3.0], [2.0, 2.0, 3.0], [4.0, 4.0, 4.0]]], [[[7.0, 1.0, 7.0], [5.0, 4.0, 1.0], [3.0, 2.0, 1.0]], [[4.0, 2.0, 5.0], 
[1.0, 2.0, 1.0], [3.0, 3.0, 6.0]], [[4.0, 5.0, 7.0], [6.0, 3.0, 1.0], [5.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 7.0], [1.0, 7.0, 1.0], [5.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [3.0, 6.0, 3.0], [7.0, 
6.0, 1.0]], [[4.0, 8.0, 1.0], [6.0, 8.0, 8.0], [1.0, 6.0, 4.0]]], [[[8.0, 4.0, 7.0], [1.0, 3.0, 2.0], [6.0, 2.0, 7.0]], [[1.0, 
1.0, 1.0], [2.0, 4.0, 4.0], [3.0, 7.0, 7.0]], [[2.0, 6.0, 8.0], [2.0, 4.0, 2.0], [2.0, 3.0, 1.0]]], [[[5.0, 4.0, 8.0], [7.0, 
2.0, 2.0], [2.0, 4.0, 8.0]], [[3.0, 6.0, 5.0], [1.0, 2.0, 4.0], [1.0, 1.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 7.0, 8.0], [5.0, 7.0, 
3.0]]], [[[3.0, 6.0, 4.0], [5.0, 2.0, 5.0], [4.0, 2.0, 1.0]], [[4.0, 3.0, 6.0], [5.0, 4.0, 8.0], [5.0, 8.0, 4.0]], [[7.0, 1.0, 
1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 3.0], [5.0, 1.0, 1.0], [8.0, 2.0, 5.0]], 
[[2.0, 3.0, 5.0], [1.0, 2.0, 8.0], [4.0, 8.0, 2.0]], [[4.0, 2.0, 6.0], [7.0, 7.0, 2.0], [5.0, 8.0, 2.0]]], [[[2.0, 6.0, 3.0], 
[3.0, 3.0, 1.0], [5.0, 3.0, 2.0]], [[4.0, 3.0, 6.0], [6.0, 6.0, 7.0], [1.0, 2.0, 7.0]], [[8.0, 5.0, 5.0], [1.0, 1.0, 8.0], 
[1.0, 1.0, 6.0]]], [[[1.0, 1.0, 5.0], [8.0, 2.0, 4.0], [1.0, 2.0, 7.0]], [[8.0, 6.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 8.0]], 
[[1.0, 7.0, 8.0], [7.0, 8.0, 5.0], [4.0, 1.0, 5.0]]], [[[1.0, 6.0, 4.0], [7.0, 6.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 2.0, 5.0], 
[7.0, 8.0, 5.0], [4.0, 4.0, 8.0]], [[1.0, 3.0, 7.0], [4.0, 8.0, 4.0], [1.0, 2.0, 5.0]]]])
      val=numpy.array([[[[5.0, 3.0, 1.0], [1.0, 4.0, 1.0], [1.0, 6.0, 5.0]]], [[[2.0, 2.0, 4.0], [8.0, 2.0, 8.0], [7.0, 3.0, 
4.0]]], [[[7.0, 2.0, 3.0], [2.0, 4.0, 5.0], [1.0, 3.0, 6.0]]]])
      arg[:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [1.0, 7.0, 1.0], [5.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [3.0, 6.0, 3.0], [7.0, 
6.0, 1.0]], [[5.0, 3.0, 1.0], [1.0, 4.0, 1.0], [1.0, 6.0, 5.0]]], [[[8.0, 4.0, 7.0], [1.0, 3.0, 2.0], [6.0, 2.0, 7.0]], [[1.0, 
1.0, 1.0], [2.0, 4.0, 4.0], [3.0, 7.0, 7.0]], [[2.0, 2.0, 4.0], [8.0, 2.0, 8.0], [7.0, 3.0, 4.0]]], [[[5.0, 4.0, 8.0], [7.0, 
2.0, 2.0], [2.0, 4.0, 8.0]], [[3.0, 6.0, 5.0], [1.0, 2.0, 4.0], [1.0, 1.0, 3.0]], [[7.0, 2.0, 3.0], [2.0, 4.0, 5.0], [1.0, 3.0, 
6.0]]], [[[3.0, 6.0, 4.0], [5.0, 2.0, 5.0], [4.0, 2.0, 1.0]], [[4.0, 3.0, 6.0], [5.0, 4.0, 8.0], [5.0, 8.0, 4.0]], [[7.0, 1.0, 
1.0], [4.0, 3.0, 5.0], [6.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 3.0], [5.0, 1.0, 1.0], [8.0, 2.0, 5.0]], 
[[2.0, 3.0, 5.0], [1.0, 2.0, 8.0], [4.0, 8.0, 2.0]], [[5.0, 3.0, 1.0], [1.0, 4.0, 1.0], [1.0, 6.0, 5.0]]], [[[2.0, 6.0, 3.0], 
[3.0, 3.0, 1.0], [5.0, 3.0, 2.0]], [[4.0, 3.0, 6.0], [6.0, 6.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 2.0, 4.0], [8.0, 2.0, 8.0], 
[7.0, 3.0, 4.0]]], [[[1.0, 1.0, 5.0], [8.0, 2.0, 4.0], [1.0, 2.0, 7.0]], [[8.0, 6.0, 7.0], [6.0, 7.0, 6.0], [8.0, 6.0, 8.0]], 
[[7.0, 2.0, 3.0], [2.0, 4.0, 5.0], [1.0, 3.0, 6.0]]], [[[1.0, 6.0, 4.0], [7.0, 6.0, 5.0], [1.0, 8.0, 1.0]], [[8.0, 2.0, 5.0], 
[7.0, 8.0, 5.0], [4.0, 4.0, 8.0]], [[1.0, 3.0, 7.0], [4.0, 8.0, 4.0], [1.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 6.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 6.0], [7.0, 8.0, 4.0], [6.0, 
5.0, 7.0]], [[5.0, 2.0, 7.0], [8.0, 2.0, 4.0], [5.0, 6.0, 3.0]]], [[[6.0, 2.0, 8.0], [8.0, 3.0, 3.0], [3.0, 1.0, 5.0]], [[7.0, 
8.0, 8.0], [8.0, 4.0, 4.0], [1.0, 4.0, 7.0]], [[6.0, 5.0, 3.0], [4.0, 4.0, 2.0], [3.0, 6.0, 1.0]]], [[[3.0, 3.0, 8.0], [6.0, 
7.0, 8.0], [7.0, 8.0, 1.0]], [[7.0, 8.0, 7.0], [2.0, 6.0, 1.0], [7.0, 6.0, 7.0]], [[8.0, 5.0, 4.0], [2.0, 3.0, 7.0], [7.0, 6.0, 
6.0]]], [[[7.0, 4.0, 1.0], [7.0, 8.0, 4.0], [3.0, 6.0, 2.0]], [[3.0, 6.0, 1.0], [2.0, 5.0, 3.0], [1.0, 4.0, 3.0]], [[8.0, 8.0, 
3.0], [1.0, 6.0, 2.0], [4.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 6.0], [7.0, 5.0, 6.0], [3.0, 4.0, 3.0]], 
[[4.0, 1.0, 3.0], [3.0, 8.0, 7.0], [1.0, 5.0, 3.0]], [[6.0, 3.0, 3.0], [6.0, 8.0, 5.0], [7.0, 8.0, 3.0]]], [[[1.0, 1.0, 6.0], 
[6.0, 7.0, 1.0], [2.0, 3.0, 7.0]], [[1.0, 7.0, 3.0], [7.0, 2.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 8.0, 8.0], [5.0, 5.0, 2.0], 
[6.0, 3.0, 8.0]]], [[[8.0, 2.0, 3.0], [8.0, 2.0, 6.0], [6.0, 2.0, 8.0]], [[1.0, 5.0, 2.0], [8.0, 4.0, 5.0], [3.0, 6.0, 7.0]], 
[[2.0, 2.0, 1.0], [3.0, 3.0, 1.0], [2.0, 6.0, 4.0]]], [[[8.0, 4.0, 1.0], [3.0, 7.0, 5.0], [7.0, 8.0, 1.0]], [[3.0, 4.0, 1.0], 
[2.0, 8.0, 8.0], [5.0, 8.0, 2.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 6.0], [6.0, 2.0, 3.0]]]])
      val=Data(numpy.array([[[[2.0, 8.0, 6.0], [8.0, 1.0, 3.0], [2.0, 7.0, 8.0]]], [[[4.0, 6.0, 8.0], [8.0, 3.0, 4.0], [7.0, 
5.0, 5.0]]], [[[5.0, 4.0, 3.0], [4.0, 1.0, 1.0], [4.0, 5.0, 4.0]]]]),self.functionspace)
      arg[:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 6.0, 5.0], [8.0, 2.0, 1.0]], [[7.0, 1.0, 6.0], [7.0, 8.0, 4.0], [6.0, 
5.0, 7.0]], [[2.0, 8.0, 6.0], [8.0, 1.0, 3.0], [2.0, 7.0, 8.0]]], [[[6.0, 2.0, 8.0], [8.0, 3.0, 3.0], [3.0, 1.0, 5.0]], [[7.0, 
8.0, 8.0], [8.0, 4.0, 4.0], [1.0, 4.0, 7.0]], [[4.0, 6.0, 8.0], [8.0, 3.0, 4.0], [7.0, 5.0, 5.0]]], [[[3.0, 3.0, 8.0], [6.0, 
7.0, 8.0], [7.0, 8.0, 1.0]], [[7.0, 8.0, 7.0], [2.0, 6.0, 1.0], [7.0, 6.0, 7.0]], [[5.0, 4.0, 3.0], [4.0, 1.0, 1.0], [4.0, 5.0, 
4.0]]], [[[7.0, 4.0, 1.0], [7.0, 8.0, 4.0], [3.0, 6.0, 2.0]], [[3.0, 6.0, 1.0], [2.0, 5.0, 3.0], [1.0, 4.0, 3.0]], [[8.0, 8.0, 
3.0], [1.0, 6.0, 2.0], [4.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 6.0], [7.0, 5.0, 6.0], [3.0, 4.0, 3.0]], 
[[4.0, 1.0, 3.0], [3.0, 8.0, 7.0], [1.0, 5.0, 3.0]], [[2.0, 8.0, 6.0], [8.0, 1.0, 3.0], [2.0, 7.0, 8.0]]], [[[1.0, 1.0, 6.0], 
[6.0, 7.0, 1.0], [2.0, 3.0, 7.0]], [[1.0, 7.0, 3.0], [7.0, 2.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 6.0, 8.0], [8.0, 3.0, 4.0], 
[7.0, 5.0, 5.0]]], [[[8.0, 2.0, 3.0], [8.0, 2.0, 6.0], [6.0, 2.0, 8.0]], [[1.0, 5.0, 2.0], [8.0, 4.0, 5.0], [3.0, 6.0, 7.0]], 
[[5.0, 4.0, 3.0], [4.0, 1.0, 1.0], [4.0, 5.0, 4.0]]], [[[8.0, 4.0, 1.0], [3.0, 7.0, 5.0], [7.0, 8.0, 1.0]], [[3.0, 4.0, 1.0], 
[2.0, 8.0, 8.0], [5.0, 8.0, 2.0]], [[8.0, 5.0, 5.0], [7.0, 8.0, 6.0], [6.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 5.0], [2.0, 2.0, 8.0], [3.0, 7.0, 3.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 7.0], [3.0, 
8.0, 4.0]], [[4.0, 4.0, 8.0], [3.0, 2.0, 1.0], [3.0, 1.0, 5.0]]], [[[3.0, 4.0, 1.0], [3.0, 7.0, 6.0], [7.0, 4.0, 6.0]], [[4.0, 
8.0, 7.0], [3.0, 1.0, 1.0], [6.0, 3.0, 8.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 7.0], [5.0, 8.0, 3.0]]], [[[4.0, 8.0, 2.0], [7.0, 
4.0, 5.0], [1.0, 5.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 6.0, 4.0], [8.0, 7.0, 2.0]], [[7.0, 5.0, 8.0], [1.0, 5.0, 4.0], [1.0, 3.0, 
6.0]]], [[[3.0, 8.0, 2.0], [4.0, 5.0, 3.0], [4.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], [7.0, 3.0, 4.0], [1.0, 2.0, 2.0]], [[6.0, 4.0, 
7.0], [1.0, 4.0, 4.0], [4.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 8.0], [2.0, 2.0, 6.0], [4.0, 8.0, 7.0]], 
[[1.0, 5.0, 7.0], [3.0, 3.0, 6.0], [2.0, 6.0, 7.0]], [[6.0, 4.0, 1.0], [3.0, 7.0, 8.0], [7.0, 6.0, 1.0]]], [[[8.0, 3.0, 4.0], 
[2.0, 8.0, 5.0], [8.0, 8.0, 1.0]], [[3.0, 3.0, 5.0], [1.0, 1.0, 2.0], [5.0, 4.0, 8.0]], [[6.0, 5.0, 8.0], [7.0, 8.0, 1.0], 
[8.0, 7.0, 8.0]]], [[[8.0, 4.0, 1.0], [7.0, 5.0, 3.0], [5.0, 1.0, 7.0]], [[4.0, 7.0, 1.0], [1.0, 3.0, 5.0], [6.0, 3.0, 5.0]], 
[[3.0, 4.0, 4.0], [7.0, 1.0, 2.0], [2.0, 5.0, 5.0]]], [[[8.0, 5.0, 7.0], [4.0, 2.0, 1.0], [6.0, 1.0, 1.0]], [[8.0, 7.0, 2.0], 
[3.0, 8.0, 4.0], [3.0, 6.0, 4.0]], [[1.0, 6.0, 2.0], [8.0, 1.0, 3.0], [4.0, 5.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 3.0, 3.0], [7.0, 7.0, 8.0], [5.0, 8.0, 4.0]]], [[[3.0, 8.0, 8.0], [8.0, 6.0, 2.0], [3.0, 
4.0, 3.0]]], [[[4.0, 6.0, 2.0], [6.0, 3.0, 7.0], [6.0, 1.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 7.0, 8.0], [1.0, 1.0, 
3.0], [3.0, 2.0, 5.0]]], [[[2.0, 4.0, 6.0], [1.0, 6.0, 3.0], [7.0, 5.0, 7.0]]], [[[5.0, 2.0, 3.0], [6.0, 8.0, 7.0], [4.0, 2.0, 
2.0]]]])
      arg[:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 5.0], [2.0, 2.0, 8.0], [3.0, 7.0, 3.0]], [[3.0, 4.0, 6.0], [5.0, 1.0, 7.0], [3.0, 
8.0, 4.0]], [[4.0, 3.0, 3.0], [7.0, 7.0, 8.0], [5.0, 8.0, 4.0]]], [[[3.0, 4.0, 1.0], [3.0, 7.0, 6.0], [7.0, 4.0, 6.0]], [[4.0, 
8.0, 7.0], [3.0, 1.0, 1.0], [6.0, 3.0, 8.0]], [[3.0, 8.0, 8.0], [8.0, 6.0, 2.0], [3.0, 4.0, 3.0]]], [[[4.0, 8.0, 2.0], [7.0, 
4.0, 5.0], [1.0, 5.0, 2.0]], [[8.0, 4.0, 7.0], [2.0, 6.0, 4.0], [8.0, 7.0, 2.0]], [[4.0, 6.0, 2.0], [6.0, 3.0, 7.0], [6.0, 1.0, 
6.0]]], [[[3.0, 8.0, 2.0], [4.0, 5.0, 3.0], [4.0, 3.0, 1.0]], [[5.0, 2.0, 7.0], [7.0, 3.0, 4.0], [1.0, 2.0, 2.0]], [[6.0, 4.0, 
7.0], [1.0, 4.0, 4.0], [4.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 8.0], [2.0, 2.0, 6.0], [4.0, 8.0, 7.0]], 
[[1.0, 5.0, 7.0], [3.0, 3.0, 6.0], [2.0, 6.0, 7.0]], [[3.0, 7.0, 8.0], [1.0, 1.0, 3.0], [3.0, 2.0, 5.0]]], [[[8.0, 3.0, 4.0], 
[2.0, 8.0, 5.0], [8.0, 8.0, 1.0]], [[3.0, 3.0, 5.0], [1.0, 1.0, 2.0], [5.0, 4.0, 8.0]], [[2.0, 4.0, 6.0], [1.0, 6.0, 3.0], 
[7.0, 5.0, 7.0]]], [[[8.0, 4.0, 1.0], [7.0, 5.0, 3.0], [5.0, 1.0, 7.0]], [[4.0, 7.0, 1.0], [1.0, 3.0, 5.0], [6.0, 3.0, 5.0]], 
[[5.0, 2.0, 3.0], [6.0, 8.0, 7.0], [4.0, 2.0, 2.0]]], [[[8.0, 5.0, 7.0], [4.0, 2.0, 1.0], [6.0, 1.0, 1.0]], [[8.0, 7.0, 2.0], 
[3.0, 8.0, 4.0], [3.0, 6.0, 4.0]], [[1.0, 6.0, 2.0], [8.0, 1.0, 3.0], [4.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 3.0], [8.0, 5.0, 6.0], [8.0, 2.0, 8.0]], [[6.0, 7.0, 6.0], [4.0, 3.0, 4.0], [2.0, 
8.0, 4.0]], [[5.0, 2.0, 6.0], [4.0, 6.0, 4.0], [8.0, 1.0, 1.0]]], [[[5.0, 8.0, 6.0], [1.0, 2.0, 2.0], [7.0, 4.0, 2.0]], [[7.0, 
6.0, 1.0], [3.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 5.0], [6.0, 1.0, 4.0], [8.0, 5.0, 5.0]]], [[[1.0, 3.0, 8.0], [6.0, 
4.0, 3.0], [7.0, 3.0, 5.0]], [[7.0, 5.0, 8.0], [4.0, 5.0, 8.0], [3.0, 7.0, 4.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 1.0], [6.0, 8.0, 
5.0]]], [[[2.0, 4.0, 1.0], [7.0, 2.0, 7.0], [2.0, 7.0, 7.0]], [[2.0, 6.0, 1.0], [2.0, 2.0, 1.0], [2.0, 6.0, 3.0]], [[4.0, 4.0, 
2.0], [2.0, 7.0, 2.0], [8.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 4.0], [2.0, 2.0, 3.0], [1.0, 1.0, 4.0]], 
[[8.0, 1.0, 8.0], [7.0, 6.0, 3.0], [4.0, 3.0, 2.0]], [[5.0, 7.0, 8.0], [7.0, 8.0, 7.0], [4.0, 4.0, 6.0]]], [[[6.0, 6.0, 3.0], 
[7.0, 3.0, 4.0], [3.0, 6.0, 3.0]], [[6.0, 8.0, 8.0], [7.0, 4.0, 1.0], [8.0, 3.0, 5.0]], [[7.0, 1.0, 5.0], [7.0, 7.0, 8.0], 
[6.0, 8.0, 5.0]]], [[[7.0, 3.0, 7.0], [4.0, 5.0, 8.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 5.0], [1.0, 2.0, 1.0], [1.0, 4.0, 5.0]], 
[[4.0, 2.0, 4.0], [3.0, 6.0, 4.0], [6.0, 2.0, 1.0]]], [[[8.0, 6.0, 4.0], [3.0, 7.0, 3.0], [5.0, 1.0, 7.0]], [[3.0, 1.0, 3.0], 
[5.0, 1.0, 2.0], [5.0, 6.0, 1.0]], [[2.0, 1.0, 2.0], [4.0, 4.0, 7.0], [8.0, 5.0, 3.0]]]])
      val=numpy.array([[[[3.0], [2.0], [1.0]]], [[[1.0], [5.0], [4.0]]]])
      arg[2:,1:2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 3.0], [8.0, 5.0, 6.0], [8.0, 2.0, 8.0]], [[6.0, 7.0, 6.0], [4.0, 3.0, 4.0], [2.0, 
8.0, 4.0]], [[5.0, 2.0, 6.0], [4.0, 6.0, 4.0], [8.0, 1.0, 1.0]]], [[[5.0, 8.0, 6.0], [1.0, 2.0, 2.0], [7.0, 4.0, 2.0]], [[7.0, 
6.0, 1.0], [3.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[5.0, 5.0, 5.0], [6.0, 1.0, 4.0], [8.0, 5.0, 5.0]]], [[[1.0, 3.0, 8.0], [6.0, 
4.0, 3.0], [7.0, 3.0, 5.0]], [[7.0, 5.0, 3.0], [4.0, 5.0, 2.0], [3.0, 7.0, 1.0]], [[4.0, 1.0, 7.0], [5.0, 7.0, 1.0], [6.0, 8.0, 
5.0]]], [[[2.0, 4.0, 1.0], [7.0, 2.0, 7.0], [2.0, 7.0, 7.0]], [[2.0, 6.0, 1.0], [2.0, 2.0, 5.0], [2.0, 6.0, 4.0]], [[4.0, 4.0, 
2.0], [2.0, 7.0, 2.0], [8.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 4.0], [2.0, 2.0, 3.0], [1.0, 1.0, 4.0]], 
[[8.0, 1.0, 8.0], [7.0, 6.0, 3.0], [4.0, 3.0, 2.0]], [[5.0, 7.0, 8.0], [7.0, 8.0, 7.0], [4.0, 4.0, 6.0]]], [[[6.0, 6.0, 3.0], 
[7.0, 3.0, 4.0], [3.0, 6.0, 3.0]], [[6.0, 8.0, 8.0], [7.0, 4.0, 1.0], [8.0, 3.0, 5.0]], [[7.0, 1.0, 5.0], [7.0, 7.0, 8.0], 
[6.0, 8.0, 5.0]]], [[[7.0, 3.0, 7.0], [4.0, 5.0, 8.0], [8.0, 4.0, 4.0]], [[4.0, 5.0, 3.0], [1.0, 2.0, 2.0], [1.0, 4.0, 1.0]], 
[[4.0, 2.0, 4.0], [3.0, 6.0, 4.0], [6.0, 2.0, 1.0]]], [[[8.0, 6.0, 4.0], [3.0, 7.0, 3.0], [5.0, 1.0, 7.0]], [[3.0, 1.0, 1.0], 
[5.0, 1.0, 5.0], [5.0, 6.0, 4.0]], [[2.0, 1.0, 2.0], [4.0, 4.0, 7.0], [8.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 7.0], [7.0, 2.0, 2.0], [5.0, 8.0, 4.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 7.0], [3.0, 
2.0, 3.0]], [[5.0, 3.0, 7.0], [1.0, 4.0, 1.0], [3.0, 5.0, 7.0]]], [[[2.0, 1.0, 8.0], [8.0, 4.0, 8.0], [3.0, 1.0, 6.0]], [[5.0, 
1.0, 4.0], [5.0, 3.0, 1.0], [5.0, 3.0, 5.0]], [[8.0, 8.0, 3.0], [8.0, 2.0, 1.0], [8.0, 3.0, 6.0]]], [[[6.0, 5.0, 8.0], [7.0, 
7.0, 4.0], [2.0, 4.0, 2.0]], [[6.0, 6.0, 4.0], [7.0, 3.0, 5.0], [3.0, 3.0, 3.0]], [[3.0, 1.0, 7.0], [8.0, 1.0, 7.0], [3.0, 2.0, 
8.0]]], [[[6.0, 4.0, 6.0], [2.0, 8.0, 7.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 8.0], [1.0, 7.0, 7.0], [6.0, 2.0, 7.0]], [[5.0, 8.0, 
1.0], [1.0, 8.0, 4.0], [6.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 6.0], [4.0, 3.0, 3.0], [4.0, 1.0, 7.0]], 
[[3.0, 3.0, 8.0], [4.0, 8.0, 1.0], [8.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [7.0, 3.0, 2.0], [4.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], 
[7.0, 1.0, 4.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 2.0], [3.0, 2.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 4.0], [3.0, 3.0, 5.0], 
[4.0, 4.0, 2.0]]], [[[1.0, 1.0, 7.0], [2.0, 7.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 2.0], [6.0, 8.0, 1.0], [8.0, 8.0, 5.0]], 
[[3.0, 5.0, 1.0], [1.0, 1.0, 1.0], [2.0, 3.0, 4.0]]], [[[2.0, 7.0, 2.0], [7.0, 6.0, 5.0], [5.0, 2.0, 2.0]], [[5.0, 1.0, 3.0], 
[8.0, 4.0, 3.0], [1.0, 8.0, 2.0]], [[2.0, 7.0, 7.0], [3.0, 8.0, 3.0], [5.0, 1.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0], [3.0], [7.0]]], [[[8.0], [6.0], [4.0]]]]),self.functionspace)
      arg[2:,1:2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 7.0], [7.0, 2.0, 2.0], [5.0, 8.0, 4.0]], [[8.0, 5.0, 6.0], [8.0, 6.0, 7.0], [3.0, 
2.0, 3.0]], [[5.0, 3.0, 7.0], [1.0, 4.0, 1.0], [3.0, 5.0, 7.0]]], [[[2.0, 1.0, 8.0], [8.0, 4.0, 8.0], [3.0, 1.0, 6.0]], [[5.0, 
1.0, 4.0], [5.0, 3.0, 1.0], [5.0, 3.0, 5.0]], [[8.0, 8.0, 3.0], [8.0, 2.0, 1.0], [8.0, 3.0, 6.0]]], [[[6.0, 5.0, 8.0], [7.0, 
7.0, 4.0], [2.0, 4.0, 2.0]], [[6.0, 6.0, 4.0], [7.0, 3.0, 3.0], [3.0, 3.0, 7.0]], [[3.0, 1.0, 7.0], [8.0, 1.0, 7.0], [3.0, 2.0, 
8.0]]], [[[6.0, 4.0, 6.0], [2.0, 8.0, 7.0], [8.0, 2.0, 7.0]], [[8.0, 7.0, 8.0], [1.0, 7.0, 6.0], [6.0, 2.0, 4.0]], [[5.0, 8.0, 
1.0], [1.0, 8.0, 4.0], [6.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 6.0], [4.0, 3.0, 3.0], [4.0, 1.0, 7.0]], 
[[3.0, 3.0, 8.0], [4.0, 8.0, 1.0], [8.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [7.0, 3.0, 2.0], [4.0, 6.0, 4.0]]], [[[4.0, 8.0, 3.0], 
[7.0, 1.0, 4.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 2.0], [3.0, 2.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 4.0], [3.0, 3.0, 5.0], 
[4.0, 4.0, 2.0]]], [[[1.0, 1.0, 7.0], [2.0, 7.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 4.0], [6.0, 8.0, 3.0], [8.0, 8.0, 7.0]], 
[[3.0, 5.0, 1.0], [1.0, 1.0, 1.0], [2.0, 3.0, 4.0]]], [[[2.0, 7.0, 2.0], [7.0, 6.0, 5.0], [5.0, 2.0, 2.0]], [[5.0, 1.0, 8.0], 
[8.0, 4.0, 6.0], [1.0, 8.0, 4.0]], [[2.0, 7.0, 7.0], [3.0, 8.0, 3.0], [5.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 6.0], [6.0, 8.0, 5.0], [8.0, 6.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 2.0, 5.0], [5.0, 
7.0, 3.0]], [[4.0, 6.0, 7.0], [8.0, 3.0, 2.0], [6.0, 4.0, 8.0]]], [[[2.0, 4.0, 5.0], [8.0, 3.0, 3.0], [4.0, 1.0, 6.0]], [[2.0, 
3.0, 7.0], [2.0, 3.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 1.0, 7.0], [7.0, 6.0, 6.0], [1.0, 1.0, 6.0]]], [[[5.0, 1.0, 8.0], [3.0, 
6.0, 2.0], [7.0, 2.0, 7.0]], [[2.0, 7.0, 6.0], [1.0, 1.0, 6.0], [8.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [1.0, 3.0, 8.0], [3.0, 5.0, 
2.0]]], [[[7.0, 6.0, 3.0], [6.0, 4.0, 6.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 2.0], [2.0, 6.0, 3.0], [7.0, 3.0, 8.0]], [[2.0, 1.0, 
6.0], [2.0, 4.0, 2.0], [3.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 2.0], [7.0, 1.0, 8.0], [2.0, 4.0, 3.0]], 
[[7.0, 1.0, 3.0], [8.0, 8.0, 7.0], [1.0, 6.0, 5.0]], [[1.0, 3.0, 5.0], [7.0, 7.0, 4.0], [5.0, 6.0, 1.0]]], [[[1.0, 1.0, 4.0], 
[4.0, 7.0, 8.0], [5.0, 1.0, 1.0]], [[7.0, 4.0, 8.0], [6.0, 5.0, 1.0], [8.0, 4.0, 6.0]], [[5.0, 3.0, 7.0], [4.0, 6.0, 7.0], 
[3.0, 3.0, 7.0]]], [[[5.0, 7.0, 4.0], [7.0, 2.0, 5.0], [8.0, 6.0, 6.0]], [[6.0, 5.0, 7.0], [8.0, 3.0, 1.0], [8.0, 8.0, 6.0]], 
[[6.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 4.0, 6.0]]], [[[7.0, 7.0, 2.0], [6.0, 7.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 3.0, 6.0], 
[7.0, 2.0, 1.0], [8.0, 1.0, 3.0]], [[4.0, 7.0, 2.0], [7.0, 5.0, 5.0], [6.0, 5.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0], [1.0], [7.0]]], [[[7.0], [2.0], [6.0]]]])+(1.-msk_val)*numpy.array([[[[6.0], [3.0], 
[2.0]]], [[[8.0], [5.0], [2.0]]]])
      arg[2:,1:2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [6.0, 8.0, 5.0], [8.0, 6.0, 6.0]], [[8.0, 5.0, 5.0], [7.0, 2.0, 5.0], [5.0, 
7.0, 3.0]], [[4.0, 6.0, 7.0], [8.0, 3.0, 2.0], [6.0, 4.0, 8.0]]], [[[2.0, 4.0, 5.0], [8.0, 3.0, 3.0], [4.0, 1.0, 6.0]], [[2.0, 
3.0, 7.0], [2.0, 3.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 1.0, 7.0], [7.0, 6.0, 6.0], [1.0, 1.0, 6.0]]], [[[5.0, 1.0, 8.0], [3.0, 
6.0, 2.0], [7.0, 2.0, 7.0]], [[2.0, 7.0, 3.0], [1.0, 1.0, 1.0], [8.0, 8.0, 7.0]], [[8.0, 1.0, 4.0], [1.0, 3.0, 8.0], [3.0, 5.0, 
2.0]]], [[[7.0, 6.0, 3.0], [6.0, 4.0, 6.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 7.0], [2.0, 6.0, 2.0], [7.0, 3.0, 6.0]], [[2.0, 1.0, 
6.0], [2.0, 4.0, 2.0], [3.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 2.0], [7.0, 1.0, 8.0], [2.0, 4.0, 3.0]], 
[[7.0, 1.0, 3.0], [8.0, 8.0, 7.0], [1.0, 6.0, 5.0]], [[1.0, 3.0, 5.0], [7.0, 7.0, 4.0], [5.0, 6.0, 1.0]]], [[[1.0, 1.0, 4.0], 
[4.0, 7.0, 8.0], [5.0, 1.0, 1.0]], [[7.0, 4.0, 8.0], [6.0, 5.0, 1.0], [8.0, 4.0, 6.0]], [[5.0, 3.0, 7.0], [4.0, 6.0, 7.0], 
[3.0, 3.0, 7.0]]], [[[5.0, 7.0, 4.0], [7.0, 2.0, 5.0], [8.0, 6.0, 6.0]], [[6.0, 5.0, 6.0], [8.0, 3.0, 3.0], [8.0, 8.0, 2.0]], 
[[6.0, 3.0, 4.0], [2.0, 5.0, 3.0], [6.0, 4.0, 6.0]]], [[[7.0, 7.0, 2.0], [6.0, 7.0, 2.0], [7.0, 6.0, 8.0]], [[3.0, 3.0, 8.0], 
[7.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[4.0, 7.0, 2.0], [7.0, 5.0, 5.0], [6.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 3.0], [5.0, 4.0, 7.0], [6.0, 1.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 6.0, 3.0], [6.0, 
3.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 6.0, 2.0], [2.0, 1.0, 7.0]]], [[[4.0, 1.0, 6.0], [6.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[4.0, 
4.0, 5.0], [1.0, 6.0, 6.0], [8.0, 6.0, 1.0]], [[3.0, 8.0, 4.0], [7.0, 6.0, 7.0], [1.0, 2.0, 3.0]]], [[[7.0, 1.0, 5.0], [7.0, 
6.0, 6.0], [1.0, 8.0, 7.0]], [[2.0, 6.0, 4.0], [2.0, 5.0, 1.0], [7.0, 5.0, 7.0]], [[5.0, 1.0, 4.0], [7.0, 1.0, 3.0], [8.0, 5.0, 
7.0]]], [[[7.0, 8.0, 7.0], [7.0, 6.0, 1.0], [3.0, 3.0, 7.0]], [[1.0, 8.0, 8.0], [1.0, 1.0, 7.0], [8.0, 7.0, 2.0]], [[5.0, 1.0, 
1.0], [8.0, 2.0, 3.0], [7.0, 2.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 6.0], [4.0, 7.0, 1.0], [7.0, 8.0, 8.0]], 
[[8.0, 4.0, 4.0], [8.0, 1.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 6.0], [7.0, 8.0, 5.0]]], [[[4.0, 5.0, 5.0], 
[3.0, 8.0, 1.0], [8.0, 4.0, 1.0]], [[7.0, 4.0, 5.0], [1.0, 4.0, 1.0], [1.0, 2.0, 3.0]], [[1.0, 7.0, 2.0], [7.0, 1.0, 4.0], 
[3.0, 6.0, 8.0]]], [[[3.0, 4.0, 2.0], [8.0, 1.0, 4.0], [4.0, 1.0, 2.0]], [[5.0, 8.0, 2.0], [6.0, 6.0, 8.0], [4.0, 5.0, 7.0]], 
[[6.0, 1.0, 4.0], [8.0, 1.0, 5.0], [5.0, 2.0, 1.0]]], [[[8.0, 4.0, 2.0], [2.0, 6.0, 3.0], [6.0, 6.0, 6.0]], [[1.0, 2.0, 1.0], 
[6.0, 1.0, 8.0], [2.0, 6.0, 3.0]], [[2.0, 5.0, 6.0], [2.0, 6.0, 2.0], [6.0, 6.0, 4.0]]]])
      val=numpy.array([[[[2.0, 7.0, 2.0]]], [[[3.0, 6.0, 3.0]]], [[[4.0, 6.0, 5.0]]], [[[5.0, 2.0, 1.0]]]])
      arg[:,2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [5.0, 4.0, 7.0], [6.0, 1.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 6.0, 3.0], [6.0, 
3.0, 5.0]], [[6.0, 3.0, 7.0], [4.0, 6.0, 2.0], [2.0, 7.0, 2.0]]], [[[4.0, 1.0, 6.0], [6.0, 3.0, 2.0], [3.0, 3.0, 8.0]], [[4.0, 
4.0, 5.0], [1.0, 6.0, 6.0], [8.0, 6.0, 1.0]], [[3.0, 8.0, 4.0], [7.0, 6.0, 7.0], [3.0, 6.0, 3.0]]], [[[7.0, 1.0, 5.0], [7.0, 
6.0, 6.0], [1.0, 8.0, 7.0]], [[2.0, 6.0, 4.0], [2.0, 5.0, 1.0], [7.0, 5.0, 7.0]], [[5.0, 1.0, 4.0], [7.0, 1.0, 3.0], [4.0, 6.0, 
5.0]]], [[[7.0, 8.0, 7.0], [7.0, 6.0, 1.0], [3.0, 3.0, 7.0]], [[1.0, 8.0, 8.0], [1.0, 1.0, 7.0], [8.0, 7.0, 2.0]], [[5.0, 1.0, 
1.0], [8.0, 2.0, 3.0], [5.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 6.0], [4.0, 7.0, 1.0], [7.0, 8.0, 8.0]], 
[[8.0, 4.0, 4.0], [8.0, 1.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 6.0], [2.0, 7.0, 2.0]]], [[[4.0, 5.0, 5.0], 
[3.0, 8.0, 1.0], [8.0, 4.0, 1.0]], [[7.0, 4.0, 5.0], [1.0, 4.0, 1.0], [1.0, 2.0, 3.0]], [[1.0, 7.0, 2.0], [7.0, 1.0, 4.0], 
[3.0, 6.0, 3.0]]], [[[3.0, 4.0, 2.0], [8.0, 1.0, 4.0], [4.0, 1.0, 2.0]], [[5.0, 8.0, 2.0], [6.0, 6.0, 8.0], [4.0, 5.0, 7.0]], 
[[6.0, 1.0, 4.0], [8.0, 1.0, 5.0], [4.0, 6.0, 5.0]]], [[[8.0, 4.0, 2.0], [2.0, 6.0, 3.0], [6.0, 6.0, 6.0]], [[1.0, 2.0, 1.0], 
[6.0, 1.0, 8.0], [2.0, 6.0, 3.0]], [[2.0, 5.0, 6.0], [2.0, 6.0, 2.0], [5.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 5.0], [3.0, 4.0, 3.0], [2.0, 7.0, 5.0]], [[5.0, 6.0, 5.0], [5.0, 7.0, 4.0], [6.0, 
8.0, 1.0]], [[4.0, 8.0, 4.0], [4.0, 7.0, 4.0], [1.0, 4.0, 5.0]]], [[[2.0, 7.0, 7.0], [8.0, 1.0, 6.0], [6.0, 7.0, 1.0]], [[1.0, 
8.0, 2.0], [4.0, 3.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [4.0, 3.0, 5.0], [2.0, 7.0, 1.0]]], [[[3.0, 2.0, 4.0], [4.0, 
2.0, 3.0], [7.0, 7.0, 4.0]], [[3.0, 8.0, 4.0], [2.0, 4.0, 2.0], [2.0, 1.0, 2.0]], [[3.0, 5.0, 7.0], [2.0, 5.0, 8.0], [2.0, 5.0, 
5.0]]], [[[1.0, 3.0, 4.0], [3.0, 7.0, 7.0], [3.0, 1.0, 6.0]], [[7.0, 7.0, 3.0], [7.0, 5.0, 8.0], [1.0, 2.0, 6.0]], [[7.0, 8.0, 
7.0], [2.0, 3.0, 2.0], [8.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 1.0], [6.0, 8.0, 5.0], [6.0, 3.0, 5.0]], 
[[5.0, 4.0, 6.0], [2.0, 3.0, 3.0], [8.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [4.0, 1.0, 1.0], [5.0, 6.0, 7.0]]], [[[1.0, 4.0, 1.0], 
[1.0, 8.0, 2.0], [6.0, 3.0, 5.0]], [[1.0, 1.0, 5.0], [8.0, 3.0, 5.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 5.0], [8.0, 7.0, 1.0], 
[8.0, 2.0, 3.0]]], [[[2.0, 4.0, 6.0], [1.0, 4.0, 4.0], [3.0, 6.0, 6.0]], [[1.0, 5.0, 5.0], [1.0, 7.0, 2.0], [8.0, 3.0, 2.0]], 
[[6.0, 4.0, 1.0], [7.0, 7.0, 8.0], [6.0, 6.0, 8.0]]], [[[3.0, 5.0, 4.0], [4.0, 1.0, 6.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 1.0], 
[6.0, 5.0, 4.0], [5.0, 7.0, 5.0]], [[5.0, 4.0, 7.0], [5.0, 7.0, 4.0], [4.0, 3.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0, 5.0, 5.0]]], [[[8.0, 2.0, 1.0]]], [[[8.0, 6.0, 8.0]]], [[[2.0, 7.0, 
7.0]]]]),self.functionspace)
      arg[:,2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 5.0], [3.0, 4.0, 3.0], [2.0, 7.0, 5.0]], [[5.0, 6.0, 5.0], [5.0, 7.0, 4.0], [6.0, 
8.0, 1.0]], [[4.0, 8.0, 4.0], [4.0, 7.0, 4.0], [6.0, 5.0, 5.0]]], [[[2.0, 7.0, 7.0], [8.0, 1.0, 6.0], [6.0, 7.0, 1.0]], [[1.0, 
8.0, 2.0], [4.0, 3.0, 5.0], [5.0, 6.0, 1.0]], [[6.0, 8.0, 4.0], [4.0, 3.0, 5.0], [8.0, 2.0, 1.0]]], [[[3.0, 2.0, 4.0], [4.0, 
2.0, 3.0], [7.0, 7.0, 4.0]], [[3.0, 8.0, 4.0], [2.0, 4.0, 2.0], [2.0, 1.0, 2.0]], [[3.0, 5.0, 7.0], [2.0, 5.0, 8.0], [8.0, 6.0, 
8.0]]], [[[1.0, 3.0, 4.0], [3.0, 7.0, 7.0], [3.0, 1.0, 6.0]], [[7.0, 7.0, 3.0], [7.0, 5.0, 8.0], [1.0, 2.0, 6.0]], [[7.0, 8.0, 
7.0], [2.0, 3.0, 2.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 1.0], [6.0, 8.0, 5.0], [6.0, 3.0, 5.0]], 
[[5.0, 4.0, 6.0], [2.0, 3.0, 3.0], [8.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [4.0, 1.0, 1.0], [6.0, 5.0, 5.0]]], [[[1.0, 4.0, 1.0], 
[1.0, 8.0, 2.0], [6.0, 3.0, 5.0]], [[1.0, 1.0, 5.0], [8.0, 3.0, 5.0], [7.0, 1.0, 3.0]], [[4.0, 7.0, 5.0], [8.0, 7.0, 1.0], 
[8.0, 2.0, 1.0]]], [[[2.0, 4.0, 6.0], [1.0, 4.0, 4.0], [3.0, 6.0, 6.0]], [[1.0, 5.0, 5.0], [1.0, 7.0, 2.0], [8.0, 3.0, 2.0]], 
[[6.0, 4.0, 1.0], [7.0, 7.0, 8.0], [8.0, 6.0, 8.0]]], [[[3.0, 5.0, 4.0], [4.0, 1.0, 6.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 1.0], 
[6.0, 5.0, 4.0], [5.0, 7.0, 5.0]], [[5.0, 4.0, 7.0], [5.0, 7.0, 4.0], [2.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLLC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 1.0], [4.0, 8.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 2.0, 4.0], [5.0, 2.0, 5.0], [5.0, 
7.0, 4.0]], [[4.0, 6.0, 4.0], [2.0, 3.0, 7.0], [8.0, 5.0, 5.0]]], [[[1.0, 5.0, 3.0], [5.0, 5.0, 1.0], [5.0, 3.0, 8.0]], [[8.0, 
4.0, 7.0], [1.0, 3.0, 2.0], [7.0, 5.0, 3.0]], [[5.0, 7.0, 2.0], [3.0, 3.0, 1.0], [3.0, 6.0, 4.0]]], [[[6.0, 3.0, 3.0], [3.0, 
1.0, 7.0], [2.0, 3.0, 7.0]], [[4.0, 6.0, 1.0], [7.0, 2.0, 5.0], [5.0, 1.0, 7.0]], [[1.0, 7.0, 1.0], [8.0, 4.0, 2.0], [8.0, 1.0, 
4.0]]], [[[7.0, 3.0, 7.0], [5.0, 5.0, 8.0], [6.0, 3.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 6.0, 8.0], [1.0, 8.0, 5.0]], [[7.0, 5.0, 
6.0], [4.0, 5.0, 3.0], [7.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 5.0], [1.0, 4.0, 6.0], [5.0, 4.0, 4.0]], 
[[5.0, 3.0, 6.0], [5.0, 2.0, 8.0], [4.0, 1.0, 1.0]], [[5.0, 7.0, 5.0], [2.0, 5.0, 6.0], [8.0, 7.0, 3.0]]], [[[4.0, 7.0, 4.0], 
[3.0, 1.0, 7.0], [2.0, 2.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 2.0, 3.0], [3.0, 3.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 5.0, 6.0], 
[4.0, 2.0, 8.0]]], [[[2.0, 1.0, 6.0], [8.0, 8.0, 4.0], [4.0, 1.0, 8.0]], [[5.0, 5.0, 1.0], [6.0, 4.0, 5.0], [1.0, 2.0, 6.0]], 
[[4.0, 3.0, 3.0], [7.0, 8.0, 3.0], [8.0, 5.0, 4.0]]], [[[2.0, 7.0, 2.0], [8.0, 2.0, 7.0], [2.0, 8.0, 1.0]], [[7.0, 2.0, 1.0], 
[7.0, 2.0, 8.0], [7.0, 8.0, 8.0]], [[6.0, 7.0, 6.0], [3.0, 7.0, 4.0], [2.0, 3.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 7.0, 1.0]]], [[[8.0, 8.0, 7.0]]], [[[8.0, 8.0, 8.0]]], [[[4.0, 8.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 6.0, 7.0]]], [[[2.0, 8.0, 6.0]]], [[[4.0, 2.0, 8.0]]], [[[7.0, 2.0, 7.0]]]])
      arg[:,2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 1.0], [4.0, 8.0, 4.0], [5.0, 5.0, 3.0]], [[1.0, 2.0, 4.0], [5.0, 2.0, 5.0], [5.0, 
7.0, 4.0]], [[4.0, 6.0, 4.0], [2.0, 3.0, 7.0], [6.0, 7.0, 1.0]]], [[[1.0, 5.0, 3.0], [5.0, 5.0, 1.0], [5.0, 3.0, 8.0]], [[8.0, 
4.0, 7.0], [1.0, 3.0, 2.0], [7.0, 5.0, 3.0]], [[5.0, 7.0, 2.0], [3.0, 3.0, 1.0], [8.0, 8.0, 7.0]]], [[[6.0, 3.0, 3.0], [3.0, 
1.0, 7.0], [2.0, 3.0, 7.0]], [[4.0, 6.0, 1.0], [7.0, 2.0, 5.0], [5.0, 1.0, 7.0]], [[1.0, 7.0, 1.0], [8.0, 4.0, 2.0], [8.0, 8.0, 
8.0]]], [[[7.0, 3.0, 7.0], [5.0, 5.0, 8.0], [6.0, 3.0, 4.0]], [[1.0, 6.0, 5.0], [1.0, 6.0, 8.0], [1.0, 8.0, 5.0]], [[7.0, 5.0, 
6.0], [4.0, 5.0, 3.0], [4.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 5.0], [1.0, 4.0, 6.0], [5.0, 4.0, 4.0]], 
[[5.0, 3.0, 6.0], [5.0, 2.0, 8.0], [4.0, 1.0, 1.0]], [[5.0, 7.0, 5.0], [2.0, 5.0, 6.0], [1.0, 6.0, 7.0]]], [[[4.0, 7.0, 4.0], 
[3.0, 1.0, 7.0], [2.0, 2.0, 3.0]], [[6.0, 4.0, 3.0], [4.0, 2.0, 3.0], [3.0, 3.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 5.0, 6.0], 
[2.0, 8.0, 6.0]]], [[[2.0, 1.0, 6.0], [8.0, 8.0, 4.0], [4.0, 1.0, 8.0]], [[5.0, 5.0, 1.0], [6.0, 4.0, 5.0], [1.0, 2.0, 6.0]], 
[[4.0, 3.0, 3.0], [7.0, 8.0, 3.0], [4.0, 2.0, 8.0]]], [[[2.0, 7.0, 2.0], [8.0, 2.0, 7.0], [2.0, 8.0, 1.0]], [[7.0, 2.0, 1.0], 
[7.0, 2.0, 8.0], [7.0, 8.0, 8.0]], [[6.0, 7.0, 6.0], [3.0, 7.0, 4.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 7.0], [6.0, 7.0, 5.0], [6.0, 5.0, 7.0]], [[6.0, 4.0, 1.0], [7.0, 6.0, 8.0], [1.0, 
3.0, 1.0]], [[4.0, 8.0, 3.0], [4.0, 6.0, 1.0], [3.0, 6.0, 6.0]]], [[[3.0, 7.0, 2.0], [1.0, 7.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 
6.0, 7.0], [1.0, 7.0, 1.0], [5.0, 3.0, 1.0]], [[3.0, 2.0, 1.0], [5.0, 3.0, 3.0], [1.0, 7.0, 3.0]]], [[[7.0, 1.0, 7.0], [5.0, 
5.0, 7.0], [6.0, 8.0, 7.0]], [[4.0, 4.0, 1.0], [8.0, 8.0, 7.0], [6.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [2.0, 4.0, 6.0], [2.0, 4.0, 
6.0]]], [[[7.0, 7.0, 1.0], [8.0, 4.0, 6.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 6.0], [6.0, 7.0, 5.0], [4.0, 8.0, 8.0]], [[4.0, 4.0, 
2.0], [4.0, 4.0, 7.0], [8.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 5.0], [2.0, 2.0, 8.0], [5.0, 2.0, 1.0]], 
[[4.0, 3.0, 4.0], [6.0, 3.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 7.0], [1.0, 8.0, 5.0], [1.0, 6.0, 3.0]]], [[[8.0, 5.0, 7.0], 
[8.0, 8.0, 3.0], [1.0, 1.0, 3.0]], [[7.0, 4.0, 7.0], [4.0, 1.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 5.0], [5.0, 3.0, 1.0], 
[2.0, 4.0, 5.0]]], [[[3.0, 3.0, 6.0], [8.0, 2.0, 6.0], [4.0, 4.0, 4.0]], [[5.0, 1.0, 2.0], [7.0, 2.0, 6.0], [4.0, 5.0, 4.0]], 
[[2.0, 7.0, 2.0], [1.0, 8.0, 1.0], [3.0, 5.0, 6.0]]], [[[7.0, 1.0, 5.0], [2.0, 1.0, 3.0], [5.0, 8.0, 5.0]], [[8.0, 8.0, 2.0], 
[4.0, 6.0, 3.0], [4.0, 2.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 2.0, 6.0], [2.0, 1.0, 4.0]]]])
      val=numpy.array([[[[7.0, 1.0, 8.0]], [[7.0, 4.0, 6.0]]], [[[5.0, 2.0, 5.0]], [[6.0, 8.0, 2.0]]]])
      arg[2:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 7.0], [6.0, 7.0, 5.0], [6.0, 5.0, 7.0]], [[6.0, 4.0, 1.0], [7.0, 6.0, 8.0], [1.0, 
3.0, 1.0]], [[4.0, 8.0, 3.0], [4.0, 6.0, 1.0], [3.0, 6.0, 6.0]]], [[[3.0, 7.0, 2.0], [1.0, 7.0, 6.0], [4.0, 4.0, 4.0]], [[1.0, 
6.0, 7.0], [1.0, 7.0, 1.0], [5.0, 3.0, 1.0]], [[3.0, 2.0, 1.0], [5.0, 3.0, 3.0], [1.0, 7.0, 3.0]]], [[[7.0, 1.0, 7.0], [7.0, 
1.0, 8.0], [6.0, 8.0, 7.0]], [[4.0, 4.0, 1.0], [7.0, 4.0, 6.0], [6.0, 8.0, 7.0]], [[1.0, 7.0, 7.0], [2.0, 4.0, 6.0], [2.0, 4.0, 
6.0]]], [[[7.0, 7.0, 1.0], [5.0, 2.0, 5.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 6.0], [6.0, 8.0, 2.0], [4.0, 8.0, 8.0]], [[4.0, 4.0, 
2.0], [4.0, 4.0, 7.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 5.0], [2.0, 2.0, 8.0], [5.0, 2.0, 1.0]], 
[[4.0, 3.0, 4.0], [6.0, 3.0, 6.0], [4.0, 8.0, 2.0]], [[2.0, 4.0, 7.0], [1.0, 8.0, 5.0], [1.0, 6.0, 3.0]]], [[[8.0, 5.0, 7.0], 
[8.0, 8.0, 3.0], [1.0, 1.0, 3.0]], [[7.0, 4.0, 7.0], [4.0, 1.0, 8.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 5.0], [5.0, 3.0, 1.0], 
[2.0, 4.0, 5.0]]], [[[3.0, 3.0, 6.0], [7.0, 1.0, 8.0], [4.0, 4.0, 4.0]], [[5.0, 1.0, 2.0], [7.0, 4.0, 6.0], [4.0, 5.0, 4.0]], 
[[2.0, 7.0, 2.0], [1.0, 8.0, 1.0], [3.0, 5.0, 6.0]]], [[[7.0, 1.0, 5.0], [5.0, 2.0, 5.0], [5.0, 8.0, 5.0]], [[8.0, 8.0, 2.0], 
[6.0, 8.0, 2.0], [4.0, 2.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 2.0, 6.0], [2.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 2.0], [3.0, 5.0, 8.0], [7.0, 6.0, 7.0]], [[3.0, 7.0, 4.0], [6.0, 3.0, 1.0], [7.0, 
6.0, 4.0]], [[7.0, 5.0, 1.0], [8.0, 5.0, 3.0], [6.0, 1.0, 1.0]]], [[[8.0, 5.0, 4.0], [6.0, 4.0, 4.0], [4.0, 6.0, 2.0]], [[2.0, 
2.0, 3.0], [5.0, 5.0, 2.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 8.0], [3.0, 5.0, 4.0], [2.0, 4.0, 8.0]]], [[[7.0, 7.0, 6.0], [7.0, 
7.0, 8.0], [6.0, 1.0, 8.0]], [[1.0, 1.0, 7.0], [2.0, 1.0, 6.0], [2.0, 3.0, 6.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 8.0], [4.0, 8.0, 
1.0]]], [[[5.0, 1.0, 2.0], [6.0, 5.0, 6.0], [6.0, 4.0, 7.0]], [[1.0, 5.0, 4.0], [8.0, 8.0, 3.0], [1.0, 8.0, 3.0]], [[1.0, 8.0, 
7.0], [5.0, 2.0, 3.0], [5.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 6.0], [2.0, 7.0, 6.0], [7.0, 6.0, 4.0]], 
[[7.0, 5.0, 8.0], [6.0, 4.0, 1.0], [6.0, 1.0, 8.0]], [[6.0, 6.0, 6.0], [4.0, 8.0, 7.0], [3.0, 5.0, 6.0]]], [[[6.0, 1.0, 7.0], 
[1.0, 2.0, 6.0], [3.0, 6.0, 8.0]], [[8.0, 3.0, 1.0], [3.0, 5.0, 2.0], [8.0, 6.0, 8.0]], [[5.0, 2.0, 2.0], [8.0, 6.0, 4.0], 
[5.0, 2.0, 3.0]]], [[[1.0, 2.0, 7.0], [1.0, 8.0, 4.0], [2.0, 7.0, 7.0]], [[2.0, 3.0, 1.0], [1.0, 1.0, 3.0], [5.0, 2.0, 8.0]], 
[[4.0, 4.0, 2.0], [6.0, 3.0, 6.0], [2.0, 7.0, 4.0]]], [[[4.0, 4.0, 5.0], [4.0, 3.0, 6.0], [1.0, 5.0, 6.0]], [[5.0, 3.0, 1.0], 
[6.0, 2.0, 2.0], [7.0, 7.0, 2.0]], [[8.0, 3.0, 3.0], [6.0, 6.0, 6.0], [7.0, 5.0, 7.0]]]])
      val=Data(numpy.array([[[[3.0, 3.0, 2.0]], [[4.0, 7.0, 6.0]]], [[[8.0, 5.0, 8.0]], [[6.0, 5.0, 5.0]]]]),self.functionspace)
      arg[2:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 2.0], [3.0, 5.0, 8.0], [7.0, 6.0, 7.0]], [[3.0, 7.0, 4.0], [6.0, 3.0, 1.0], [7.0, 
6.0, 4.0]], [[7.0, 5.0, 1.0], [8.0, 5.0, 3.0], [6.0, 1.0, 1.0]]], [[[8.0, 5.0, 4.0], [6.0, 4.0, 4.0], [4.0, 6.0, 2.0]], [[2.0, 
2.0, 3.0], [5.0, 5.0, 2.0], [6.0, 4.0, 3.0]], [[8.0, 1.0, 8.0], [3.0, 5.0, 4.0], [2.0, 4.0, 8.0]]], [[[7.0, 7.0, 6.0], [3.0, 
3.0, 2.0], [6.0, 1.0, 8.0]], [[1.0, 1.0, 7.0], [4.0, 7.0, 6.0], [2.0, 3.0, 6.0]], [[7.0, 2.0, 6.0], [6.0, 1.0, 8.0], [4.0, 8.0, 
1.0]]], [[[5.0, 1.0, 2.0], [8.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[1.0, 5.0, 4.0], [6.0, 5.0, 5.0], [1.0, 8.0, 3.0]], [[1.0, 8.0, 
7.0], [5.0, 2.0, 3.0], [5.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 6.0], [2.0, 7.0, 6.0], [7.0, 6.0, 4.0]], 
[[7.0, 5.0, 8.0], [6.0, 4.0, 1.0], [6.0, 1.0, 8.0]], [[6.0, 6.0, 6.0], [4.0, 8.0, 7.0], [3.0, 5.0, 6.0]]], [[[6.0, 1.0, 7.0], 
[1.0, 2.0, 6.0], [3.0, 6.0, 8.0]], [[8.0, 3.0, 1.0], [3.0, 5.0, 2.0], [8.0, 6.0, 8.0]], [[5.0, 2.0, 2.0], [8.0, 6.0, 4.0], 
[5.0, 2.0, 3.0]]], [[[1.0, 2.0, 7.0], [3.0, 3.0, 2.0], [2.0, 7.0, 7.0]], [[2.0, 3.0, 1.0], [4.0, 7.0, 6.0], [5.0, 2.0, 8.0]], 
[[4.0, 4.0, 2.0], [6.0, 3.0, 6.0], [2.0, 7.0, 4.0]]], [[[4.0, 4.0, 5.0], [8.0, 5.0, 8.0], [1.0, 5.0, 6.0]], [[5.0, 3.0, 1.0], 
[6.0, 5.0, 5.0], [7.0, 7.0, 2.0]], [[8.0, 3.0, 3.0], [6.0, 6.0, 6.0], [7.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 2.0], [7.0, 8.0, 2.0], [8.0, 3.0, 3.0]], [[2.0, 3.0, 4.0], [3.0, 6.0, 7.0], [8.0, 
5.0, 8.0]], [[5.0, 3.0, 8.0], [6.0, 8.0, 8.0], [8.0, 3.0, 7.0]]], [[[2.0, 5.0, 4.0], [8.0, 4.0, 2.0], [1.0, 1.0, 1.0]], [[5.0, 
5.0, 2.0], [2.0, 3.0, 7.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 2.0], [3.0, 1.0, 7.0], [5.0, 7.0, 2.0]]], [[[6.0, 6.0, 6.0], [6.0, 
6.0, 8.0], [6.0, 8.0, 2.0]], [[7.0, 5.0, 8.0], [4.0, 3.0, 5.0], [2.0, 3.0, 7.0]], [[8.0, 8.0, 3.0], [8.0, 4.0, 8.0], [5.0, 2.0, 
4.0]]], [[[3.0, 5.0, 6.0], [1.0, 4.0, 8.0], [2.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [1.0, 6.0, 2.0], [2.0, 5.0, 5.0]], [[3.0, 2.0, 
2.0], [1.0, 3.0, 6.0], [3.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 2.0], [4.0, 2.0, 8.0], [3.0, 5.0, 7.0]], 
[[2.0, 6.0, 2.0], [5.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[7.0, 3.0, 6.0], [1.0, 5.0, 8.0], [3.0, 4.0, 1.0]]], [[[2.0, 3.0, 2.0], 
[7.0, 3.0, 2.0], [4.0, 8.0, 3.0]], [[1.0, 8.0, 6.0], [8.0, 2.0, 1.0], [6.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], [1.0, 6.0, 6.0], 
[1.0, 5.0, 7.0]]], [[[7.0, 2.0, 8.0], [7.0, 3.0, 4.0], [6.0, 7.0, 4.0]], [[6.0, 8.0, 4.0], [2.0, 4.0, 3.0], [3.0, 4.0, 7.0]], 
[[5.0, 1.0, 3.0], [6.0, 3.0, 7.0], [3.0, 1.0, 8.0]]], [[[2.0, 7.0, 7.0], [3.0, 4.0, 6.0], [2.0, 7.0, 5.0]], [[4.0, 8.0, 5.0], 
[8.0, 1.0, 1.0], [2.0, 5.0, 5.0]], [[2.0, 1.0, 4.0], [3.0, 2.0, 6.0], [4.0, 1.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 8.0, 3.0]], [[6.0, 1.0, 7.0]]], [[[5.0, 7.0, 7.0]], [[2.0, 1.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 5.0, 2.0]], [[8.0, 4.0, 5.0]]], [[[2.0, 5.0, 1.0]], [[4.0, 2.0, 7.0]]]])
      arg[2:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 2.0], [7.0, 8.0, 2.0], [8.0, 3.0, 3.0]], [[2.0, 3.0, 4.0], [3.0, 6.0, 7.0], [8.0, 
5.0, 8.0]], [[5.0, 3.0, 8.0], [6.0, 8.0, 8.0], [8.0, 3.0, 7.0]]], [[[2.0, 5.0, 4.0], [8.0, 4.0, 2.0], [1.0, 1.0, 1.0]], [[5.0, 
5.0, 2.0], [2.0, 3.0, 7.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 2.0], [3.0, 1.0, 7.0], [5.0, 7.0, 2.0]]], [[[6.0, 6.0, 6.0], [8.0, 
8.0, 3.0], [6.0, 8.0, 2.0]], [[7.0, 5.0, 8.0], [6.0, 1.0, 7.0], [2.0, 3.0, 7.0]], [[8.0, 8.0, 3.0], [8.0, 4.0, 8.0], [5.0, 2.0, 
4.0]]], [[[3.0, 5.0, 6.0], [5.0, 7.0, 7.0], [2.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 1.0, 5.0], [2.0, 5.0, 5.0]], [[3.0, 2.0, 
2.0], [1.0, 3.0, 6.0], [3.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 2.0], [4.0, 2.0, 8.0], [3.0, 5.0, 7.0]], 
[[2.0, 6.0, 2.0], [5.0, 1.0, 3.0], [6.0, 5.0, 1.0]], [[7.0, 3.0, 6.0], [1.0, 5.0, 8.0], [3.0, 4.0, 1.0]]], [[[2.0, 3.0, 2.0], 
[7.0, 3.0, 2.0], [4.0, 8.0, 3.0]], [[1.0, 8.0, 6.0], [8.0, 2.0, 1.0], [6.0, 2.0, 3.0]], [[6.0, 4.0, 2.0], [1.0, 6.0, 6.0], 
[1.0, 5.0, 7.0]]], [[[7.0, 2.0, 8.0], [5.0, 5.0, 2.0], [6.0, 7.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 4.0, 5.0], [3.0, 4.0, 7.0]], 
[[5.0, 1.0, 3.0], [6.0, 3.0, 7.0], [3.0, 1.0, 8.0]]], [[[2.0, 7.0, 7.0], [2.0, 5.0, 1.0], [2.0, 7.0, 5.0]], [[4.0, 8.0, 5.0], 
[4.0, 2.0, 7.0], [2.0, 5.0, 5.0]], [[2.0, 1.0, 4.0], [3.0, 2.0, 6.0], [4.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 7.0], [2.0, 1.0, 8.0], [2.0, 6.0, 8.0]], [[6.0, 6.0, 4.0], [2.0, 2.0, 4.0], [1.0, 
8.0, 2.0]], [[3.0, 5.0, 2.0], [1.0, 7.0, 7.0], [3.0, 2.0, 7.0]]], [[[6.0, 7.0, 4.0], [4.0, 2.0, 1.0], [7.0, 3.0, 7.0]], [[6.0, 
4.0, 3.0], [8.0, 8.0, 4.0], [6.0, 4.0, 4.0]], [[5.0, 7.0, 8.0], [8.0, 6.0, 1.0], [8.0, 1.0, 7.0]]], [[[2.0, 6.0, 3.0], [5.0, 
5.0, 4.0], [1.0, 5.0, 5.0]], [[8.0, 6.0, 3.0], [3.0, 7.0, 2.0], [2.0, 6.0, 8.0]], [[3.0, 2.0, 5.0], [4.0, 2.0, 5.0], [7.0, 5.0, 
3.0]]], [[[3.0, 6.0, 1.0], [8.0, 7.0, 5.0], [5.0, 5.0, 8.0]], [[5.0, 1.0, 5.0], [3.0, 3.0, 3.0], [3.0, 7.0, 5.0]], [[6.0, 7.0, 
3.0], [4.0, 7.0, 7.0], [7.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 4.0], [3.0, 5.0, 1.0], [1.0, 2.0, 3.0]], 
[[4.0, 8.0, 1.0], [6.0, 4.0, 6.0], [3.0, 6.0, 5.0]], [[8.0, 3.0, 6.0], [1.0, 7.0, 4.0], [3.0, 6.0, 7.0]]], [[[6.0, 6.0, 4.0], 
[3.0, 5.0, 1.0], [2.0, 6.0, 6.0]], [[8.0, 2.0, 8.0], [2.0, 5.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 2.0, 2.0], [2.0, 4.0, 7.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 7.0, 4.0], [4.0, 6.0, 7.0], [5.0, 6.0, 3.0]], [[5.0, 5.0, 7.0], [1.0, 2.0, 4.0], [1.0, 5.0, 7.0]], 
[[4.0, 8.0, 4.0], [3.0, 4.0, 3.0], [7.0, 7.0, 2.0]]], [[[2.0, 5.0, 5.0], [1.0, 6.0, 7.0], [1.0, 2.0, 1.0]], [[3.0, 8.0, 3.0], 
[8.0, 7.0, 8.0], [3.0, 5.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 1.0, 7.0], [1.0, 8.0, 7.0]]]])
      val=numpy.array([[[8.0, 4.0]]])
      arg[2,1:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [2.0, 1.0, 8.0], [2.0, 6.0, 8.0]], [[6.0, 6.0, 4.0], [2.0, 2.0, 4.0], [1.0, 
8.0, 2.0]], [[3.0, 5.0, 2.0], [1.0, 7.0, 7.0], [3.0, 2.0, 7.0]]], [[[6.0, 7.0, 4.0], [4.0, 2.0, 1.0], [7.0, 3.0, 7.0]], [[6.0, 
4.0, 3.0], [8.0, 8.0, 4.0], [6.0, 4.0, 4.0]], [[5.0, 7.0, 8.0], [8.0, 6.0, 1.0], [8.0, 1.0, 7.0]]], [[[2.0, 6.0, 3.0], [5.0, 
5.0, 4.0], [1.0, 5.0, 5.0]], [[8.0, 6.0, 3.0], [8.0, 4.0, 2.0], [2.0, 6.0, 8.0]], [[3.0, 2.0, 5.0], [4.0, 2.0, 5.0], [7.0, 5.0, 
3.0]]], [[[3.0, 6.0, 1.0], [8.0, 7.0, 5.0], [5.0, 5.0, 8.0]], [[5.0, 1.0, 5.0], [3.0, 3.0, 3.0], [3.0, 7.0, 5.0]], [[6.0, 7.0, 
3.0], [4.0, 7.0, 7.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 4.0], [3.0, 5.0, 1.0], [1.0, 2.0, 3.0]], 
[[4.0, 8.0, 1.0], [6.0, 4.0, 6.0], [3.0, 6.0, 5.0]], [[8.0, 3.0, 6.0], [1.0, 7.0, 4.0], [3.0, 6.0, 7.0]]], [[[6.0, 6.0, 4.0], 
[3.0, 5.0, 1.0], [2.0, 6.0, 6.0]], [[8.0, 2.0, 8.0], [2.0, 5.0, 5.0], [3.0, 5.0, 1.0]], [[5.0, 2.0, 2.0], [2.0, 4.0, 7.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 7.0, 4.0], [4.0, 6.0, 7.0], [5.0, 6.0, 3.0]], [[5.0, 5.0, 7.0], [8.0, 4.0, 4.0], [1.0, 5.0, 7.0]], 
[[4.0, 8.0, 4.0], [3.0, 4.0, 3.0], [7.0, 7.0, 2.0]]], [[[2.0, 5.0, 5.0], [1.0, 6.0, 7.0], [1.0, 2.0, 1.0]], [[3.0, 8.0, 3.0], 
[8.0, 7.0, 8.0], [3.0, 5.0, 5.0]], [[1.0, 4.0, 3.0], [6.0, 1.0, 7.0], [1.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 3.0], [3.0, 4.0, 2.0], [8.0, 2.0, 6.0]], [[3.0, 8.0, 3.0], [1.0, 6.0, 4.0], [6.0, 
7.0, 5.0]], [[3.0, 5.0, 3.0], [5.0, 4.0, 3.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 1.0], [3.0, 4.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 
3.0, 4.0], [7.0, 1.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 2.0, 5.0], [6.0, 7.0, 2.0], [3.0, 6.0, 3.0]]], [[[4.0, 8.0, 3.0], [1.0, 
3.0, 8.0], [5.0, 5.0, 5.0]], [[8.0, 1.0, 1.0], [4.0, 3.0, 2.0], [7.0, 4.0, 7.0]], [[1.0, 5.0, 2.0], [1.0, 3.0, 1.0], [1.0, 6.0, 
1.0]]], [[[4.0, 4.0, 7.0], [6.0, 1.0, 2.0], [8.0, 4.0, 3.0]], [[7.0, 4.0, 7.0], [1.0, 6.0, 3.0], [2.0, 6.0, 2.0]], [[7.0, 7.0, 
8.0], [2.0, 4.0, 2.0], [4.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 6.0], [3.0, 6.0, 5.0], [2.0, 6.0, 6.0]], 
[[4.0, 2.0, 4.0], [8.0, 1.0, 7.0], [4.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [7.0, 3.0, 8.0], [3.0, 2.0, 5.0]]], [[[4.0, 1.0, 3.0], 
[2.0, 8.0, 1.0], [3.0, 4.0, 8.0]], [[5.0, 6.0, 1.0], [6.0, 4.0, 6.0], [7.0, 3.0, 5.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 5.0], 
[3.0, 5.0, 5.0]]], [[[5.0, 1.0, 1.0], [4.0, 4.0, 2.0], [8.0, 3.0, 6.0]], [[7.0, 1.0, 6.0], [4.0, 6.0, 8.0], [7.0, 1.0, 6.0]], 
[[4.0, 1.0, 7.0], [1.0, 2.0, 1.0], [5.0, 5.0, 3.0]]], [[[6.0, 2.0, 5.0], [5.0, 1.0, 4.0], [8.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], 
[6.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 6.0, 4.0], [7.0, 8.0, 2.0], [6.0, 6.0, 3.0]]]])
      val=Data(numpy.array([[[6.0, 2.0]]]),self.functionspace)
      arg[2,1:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 3.0], [3.0, 4.0, 2.0], [8.0, 2.0, 6.0]], [[3.0, 8.0, 3.0], [1.0, 6.0, 4.0], [6.0, 
7.0, 5.0]], [[3.0, 5.0, 3.0], [5.0, 4.0, 3.0], [4.0, 7.0, 1.0]]], [[[7.0, 2.0, 1.0], [3.0, 4.0, 4.0], [5.0, 7.0, 3.0]], [[8.0, 
3.0, 4.0], [7.0, 1.0, 4.0], [1.0, 5.0, 8.0]], [[2.0, 2.0, 5.0], [6.0, 7.0, 2.0], [3.0, 6.0, 3.0]]], [[[4.0, 8.0, 3.0], [1.0, 
3.0, 8.0], [5.0, 5.0, 5.0]], [[8.0, 1.0, 1.0], [6.0, 2.0, 2.0], [7.0, 4.0, 7.0]], [[1.0, 5.0, 2.0], [1.0, 3.0, 1.0], [1.0, 6.0, 
1.0]]], [[[4.0, 4.0, 7.0], [6.0, 1.0, 2.0], [8.0, 4.0, 3.0]], [[7.0, 4.0, 7.0], [1.0, 6.0, 3.0], [2.0, 6.0, 2.0]], [[7.0, 7.0, 
8.0], [2.0, 4.0, 2.0], [4.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 6.0], [3.0, 6.0, 5.0], [2.0, 6.0, 6.0]], 
[[4.0, 2.0, 4.0], [8.0, 1.0, 7.0], [4.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [7.0, 3.0, 8.0], [3.0, 2.0, 5.0]]], [[[4.0, 1.0, 3.0], 
[2.0, 8.0, 1.0], [3.0, 4.0, 8.0]], [[5.0, 6.0, 1.0], [6.0, 4.0, 6.0], [7.0, 3.0, 5.0]], [[8.0, 4.0, 3.0], [1.0, 6.0, 5.0], 
[3.0, 5.0, 5.0]]], [[[5.0, 1.0, 1.0], [4.0, 4.0, 2.0], [8.0, 3.0, 6.0]], [[7.0, 1.0, 6.0], [6.0, 2.0, 8.0], [7.0, 1.0, 6.0]], 
[[4.0, 1.0, 7.0], [1.0, 2.0, 1.0], [5.0, 5.0, 3.0]]], [[[6.0, 2.0, 5.0], [5.0, 1.0, 4.0], [8.0, 5.0, 4.0]], [[8.0, 1.0, 3.0], 
[6.0, 1.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 6.0, 4.0], [7.0, 8.0, 2.0], [6.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 5.0], [2.0, 5.0, 7.0], [8.0, 3.0, 8.0]], [[6.0, 1.0, 3.0], [7.0, 8.0, 3.0], [4.0, 
5.0, 4.0]], [[6.0, 6.0, 8.0], [3.0, 1.0, 2.0], [6.0, 3.0, 7.0]]], [[[3.0, 4.0, 1.0], [5.0, 8.0, 6.0], [2.0, 8.0, 4.0]], [[1.0, 
8.0, 2.0], [6.0, 6.0, 8.0], [8.0, 3.0, 4.0]], [[4.0, 4.0, 2.0], [8.0, 5.0, 4.0], [2.0, 5.0, 4.0]]], [[[2.0, 5.0, 4.0], [5.0, 
3.0, 2.0], [4.0, 7.0, 8.0]], [[5.0, 7.0, 6.0], [4.0, 8.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 8.0, 1.0], [6.0, 3.0, 7.0], [6.0, 4.0, 
1.0]]], [[[5.0, 4.0, 4.0], [4.0, 8.0, 6.0], [7.0, 1.0, 1.0]], [[2.0, 2.0, 7.0], [1.0, 7.0, 6.0], [3.0, 3.0, 6.0]], [[2.0, 5.0, 
6.0], [6.0, 8.0, 3.0], [6.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 1.0], [3.0, 8.0, 6.0], [5.0, 6.0, 6.0]], 
[[7.0, 1.0, 3.0], [7.0, 1.0, 1.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 4.0], [2.0, 3.0, 8.0], [4.0, 1.0, 5.0]]], [[[8.0, 7.0, 5.0], 
[5.0, 7.0, 3.0], [4.0, 6.0, 6.0]], [[6.0, 8.0, 2.0], [7.0, 7.0, 7.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 8.0], [2.0, 1.0, 3.0], 
[7.0, 4.0, 8.0]]], [[[6.0, 1.0, 6.0], [2.0, 2.0, 5.0], [8.0, 5.0, 6.0]], [[1.0, 1.0, 1.0], [6.0, 8.0, 3.0], [3.0, 1.0, 8.0]], 
[[2.0, 7.0, 8.0], [1.0, 5.0, 6.0], [5.0, 6.0, 3.0]]], [[[6.0, 3.0, 6.0], [8.0, 1.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 6.0, 1.0], 
[1.0, 4.0, 6.0], [2.0, 5.0, 5.0]], [[8.0, 4.0, 1.0], [5.0, 7.0, 2.0], [3.0, 1.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 8.0]]])+(1.-msk_val)*numpy.array([[[6.0, 4.0]]])
      arg[2,1:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 5.0], [2.0, 5.0, 7.0], [8.0, 3.0, 8.0]], [[6.0, 1.0, 3.0], [7.0, 8.0, 3.0], [4.0, 
5.0, 4.0]], [[6.0, 6.0, 8.0], [3.0, 1.0, 2.0], [6.0, 3.0, 7.0]]], [[[3.0, 4.0, 1.0], [5.0, 8.0, 6.0], [2.0, 8.0, 4.0]], [[1.0, 
8.0, 2.0], [6.0, 6.0, 8.0], [8.0, 3.0, 4.0]], [[4.0, 4.0, 2.0], [8.0, 5.0, 4.0], [2.0, 5.0, 4.0]]], [[[2.0, 5.0, 4.0], [5.0, 
3.0, 2.0], [4.0, 7.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 8.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 8.0, 1.0], [6.0, 3.0, 7.0], [6.0, 4.0, 
1.0]]], [[[5.0, 4.0, 4.0], [4.0, 8.0, 6.0], [7.0, 1.0, 1.0]], [[2.0, 2.0, 7.0], [1.0, 7.0, 6.0], [3.0, 3.0, 6.0]], [[2.0, 5.0, 
6.0], [6.0, 8.0, 3.0], [6.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [3.0, 8.0, 6.0], [5.0, 6.0, 6.0]], 
[[7.0, 1.0, 3.0], [7.0, 1.0, 1.0], [2.0, 2.0, 4.0]], [[6.0, 3.0, 4.0], [2.0, 3.0, 8.0], [4.0, 1.0, 5.0]]], [[[8.0, 7.0, 5.0], 
[5.0, 7.0, 3.0], [4.0, 6.0, 6.0]], [[6.0, 8.0, 2.0], [7.0, 7.0, 7.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 8.0], [2.0, 1.0, 3.0], 
[7.0, 4.0, 8.0]]], [[[6.0, 1.0, 6.0], [2.0, 2.0, 5.0], [8.0, 5.0, 6.0]], [[1.0, 1.0, 1.0], [6.0, 4.0, 3.0], [3.0, 1.0, 8.0]], 
[[2.0, 7.0, 8.0], [1.0, 5.0, 6.0], [5.0, 6.0, 3.0]]], [[[6.0, 3.0, 6.0], [8.0, 1.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 6.0, 1.0], 
[1.0, 4.0, 6.0], [2.0, 5.0, 5.0]], [[8.0, 4.0, 1.0], [5.0, 7.0, 2.0], [3.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 6.0], [8.0, 2.0, 7.0], [5.0, 4.0, 5.0]], [[5.0, 4.0, 2.0], [3.0, 1.0, 1.0], [3.0, 
4.0, 1.0]], [[4.0, 1.0, 6.0], [7.0, 5.0, 8.0], [3.0, 6.0, 7.0]]], [[[5.0, 6.0, 6.0], [6.0, 2.0, 1.0], [1.0, 6.0, 1.0]], [[6.0, 
2.0, 8.0], [7.0, 4.0, 3.0], [7.0, 6.0, 3.0]], [[8.0, 2.0, 7.0], [7.0, 2.0, 5.0], [4.0, 4.0, 5.0]]], [[[6.0, 7.0, 6.0], [1.0, 
8.0, 5.0], [4.0, 4.0, 8.0]], [[2.0, 8.0, 8.0], [8.0, 7.0, 2.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], [7.0, 3.0, 3.0], [8.0, 8.0, 
3.0]]], [[[7.0, 4.0, 3.0], [4.0, 6.0, 3.0], [1.0, 5.0, 2.0]], [[2.0, 5.0, 6.0], [4.0, 1.0, 5.0], [6.0, 1.0, 3.0]], [[7.0, 5.0, 
1.0], [7.0, 8.0, 8.0], [7.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 3.0], [5.0, 1.0, 1.0], [2.0, 5.0, 8.0]], 
[[3.0, 7.0, 5.0], [2.0, 5.0, 6.0], [7.0, 1.0, 3.0]], [[7.0, 2.0, 3.0], [3.0, 6.0, 8.0], [2.0, 5.0, 1.0]]], [[[2.0, 4.0, 5.0], 
[1.0, 1.0, 1.0], [5.0, 8.0, 2.0]], [[6.0, 8.0, 8.0], [1.0, 1.0, 6.0], [4.0, 7.0, 3.0]], [[6.0, 6.0, 3.0], [5.0, 1.0, 2.0], 
[5.0, 6.0, 6.0]]], [[[6.0, 1.0, 2.0], [6.0, 3.0, 3.0], [4.0, 1.0, 8.0]], [[3.0, 5.0, 3.0], [6.0, 4.0, 5.0], [2.0, 1.0, 4.0]], 
[[1.0, 8.0, 5.0], [5.0, 6.0, 6.0], [6.0, 4.0, 7.0]]], [[[8.0, 3.0, 1.0], [5.0, 4.0, 8.0], [4.0, 3.0, 6.0]], [[1.0, 4.0, 5.0], 
[8.0, 5.0, 8.0], [6.0, 6.0, 6.0]], [[6.0, 8.0, 2.0], [6.0, 4.0, 6.0], [3.0, 7.0, 4.0]]]])
      val=numpy.array([[[[3.0, 4.0], [4.0, 8.0]], [[5.0, 1.0], [1.0, 6.0]], [[6.0, 1.0], [6.0, 7.0]]], [[[3.0, 1.0], [1.0, 
7.0]], [[7.0, 7.0], [7.0, 3.0]], [[1.0, 4.0], [6.0, 2.0]]], [[[8.0, 8.0], [7.0, 7.0]], [[8.0, 3.0], [3.0, 5.0]], [[1.0, 1.0], 
[2.0, 3.0]]], [[[3.0, 7.0], [1.0, 4.0]], [[1.0, 8.0], [8.0, 2.0]], [[6.0, 5.0], [4.0, 5.0]]]])
      arg[:,:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 6.0], [4.0, 8.0, 7.0], [5.0, 4.0, 5.0]], [[5.0, 1.0, 2.0], [1.0, 6.0, 1.0], [3.0, 
4.0, 1.0]], [[6.0, 1.0, 6.0], [6.0, 7.0, 8.0], [3.0, 6.0, 7.0]]], [[[3.0, 1.0, 6.0], [1.0, 7.0, 1.0], [1.0, 6.0, 1.0]], [[7.0, 
7.0, 8.0], [7.0, 3.0, 3.0], [7.0, 6.0, 3.0]], [[1.0, 4.0, 7.0], [6.0, 2.0, 5.0], [4.0, 4.0, 5.0]]], [[[8.0, 8.0, 6.0], [7.0, 
7.0, 5.0], [4.0, 4.0, 8.0]], [[8.0, 3.0, 8.0], [3.0, 5.0, 2.0], [8.0, 2.0, 1.0]], [[1.0, 1.0, 1.0], [2.0, 3.0, 3.0], [8.0, 8.0, 
3.0]]], [[[3.0, 7.0, 3.0], [1.0, 4.0, 3.0], [1.0, 5.0, 2.0]], [[1.0, 8.0, 6.0], [8.0, 2.0, 5.0], [6.0, 1.0, 3.0]], [[6.0, 5.0, 
1.0], [4.0, 5.0, 8.0], [7.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 3.0], [4.0, 8.0, 1.0], [2.0, 5.0, 8.0]], 
[[5.0, 1.0, 5.0], [1.0, 6.0, 6.0], [7.0, 1.0, 3.0]], [[6.0, 1.0, 3.0], [6.0, 7.0, 8.0], [2.0, 5.0, 1.0]]], [[[3.0, 1.0, 5.0], 
[1.0, 7.0, 1.0], [5.0, 8.0, 2.0]], [[7.0, 7.0, 8.0], [7.0, 3.0, 6.0], [4.0, 7.0, 3.0]], [[1.0, 4.0, 3.0], [6.0, 2.0, 2.0], 
[5.0, 6.0, 6.0]]], [[[8.0, 8.0, 2.0], [7.0, 7.0, 3.0], [4.0, 1.0, 8.0]], [[8.0, 3.0, 3.0], [3.0, 5.0, 5.0], [2.0, 1.0, 4.0]], 
[[1.0, 1.0, 5.0], [2.0, 3.0, 6.0], [6.0, 4.0, 7.0]]], [[[3.0, 7.0, 1.0], [1.0, 4.0, 8.0], [4.0, 3.0, 6.0]], [[1.0, 8.0, 5.0], 
[8.0, 2.0, 8.0], [6.0, 6.0, 6.0]], [[6.0, 5.0, 2.0], [4.0, 5.0, 6.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 4.0], [3.0, 8.0, 7.0], [1.0, 3.0, 2.0]], [[6.0, 3.0, 7.0], [6.0, 6.0, 7.0], [8.0, 
8.0, 5.0]], [[4.0, 1.0, 8.0], [2.0, 3.0, 8.0], [1.0, 5.0, 4.0]]], [[[2.0, 2.0, 4.0], [2.0, 8.0, 5.0], [4.0, 2.0, 2.0]], [[4.0, 
1.0, 1.0], [5.0, 6.0, 1.0], [7.0, 7.0, 4.0]], [[4.0, 5.0, 6.0], [6.0, 3.0, 3.0], [1.0, 6.0, 7.0]]], [[[5.0, 1.0, 1.0], [1.0, 
5.0, 1.0], [8.0, 1.0, 6.0]], [[4.0, 4.0, 7.0], [7.0, 7.0, 7.0], [3.0, 6.0, 7.0]], [[8.0, 3.0, 6.0], [2.0, 7.0, 3.0], [7.0, 8.0, 
4.0]]], [[[2.0, 6.0, 4.0], [5.0, 4.0, 5.0], [7.0, 6.0, 7.0]], [[4.0, 7.0, 4.0], [7.0, 7.0, 2.0], [8.0, 3.0, 4.0]], [[4.0, 4.0, 
1.0], [4.0, 4.0, 1.0], [5.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 7.0], [7.0, 1.0, 3.0], [6.0, 4.0, 5.0]], 
[[2.0, 5.0, 3.0], [3.0, 4.0, 4.0], [8.0, 5.0, 2.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 2.0], [2.0, 4.0, 8.0]]], [[[4.0, 6.0, 7.0], 
[8.0, 3.0, 3.0], [2.0, 8.0, 8.0]], [[6.0, 1.0, 4.0], [1.0, 8.0, 1.0], [5.0, 7.0, 4.0]], [[7.0, 2.0, 2.0], [7.0, 8.0, 1.0], 
[8.0, 6.0, 7.0]]], [[[2.0, 7.0, 8.0], [2.0, 7.0, 1.0], [1.0, 1.0, 3.0]], [[8.0, 8.0, 8.0], [3.0, 6.0, 1.0], [2.0, 2.0, 2.0]], 
[[1.0, 7.0, 3.0], [1.0, 6.0, 5.0], [3.0, 4.0, 2.0]]], [[[3.0, 4.0, 6.0], [8.0, 4.0, 3.0], [7.0, 1.0, 4.0]], [[6.0, 6.0, 6.0], 
[1.0, 7.0, 2.0], [8.0, 3.0, 5.0]], [[2.0, 4.0, 6.0], [5.0, 4.0, 3.0], [3.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[[1.0, 7.0], [5.0, 8.0]], [[4.0, 7.0], [1.0, 5.0]], [[6.0, 8.0], [5.0, 7.0]]], [[[3.0, 6.0], [3.0, 
4.0]], [[8.0, 2.0], [8.0, 3.0]], [[8.0, 5.0], [7.0, 8.0]]], [[[7.0, 3.0], [8.0, 4.0]], [[2.0, 1.0], [4.0, 6.0]], [[1.0, 8.0], 
[2.0, 1.0]]], [[[1.0, 7.0], [6.0, 8.0]], [[8.0, 3.0], [1.0, 5.0]], [[2.0, 8.0], [4.0, 7.0]]]]),self.functionspace)
      arg[:,:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 4.0], [5.0, 8.0, 7.0], [1.0, 3.0, 2.0]], [[4.0, 7.0, 7.0], [1.0, 5.0, 7.0], [8.0, 
8.0, 5.0]], [[6.0, 8.0, 8.0], [5.0, 7.0, 8.0], [1.0, 5.0, 4.0]]], [[[3.0, 6.0, 4.0], [3.0, 4.0, 5.0], [4.0, 2.0, 2.0]], [[8.0, 
2.0, 1.0], [8.0, 3.0, 1.0], [7.0, 7.0, 4.0]], [[8.0, 5.0, 6.0], [7.0, 8.0, 3.0], [1.0, 6.0, 7.0]]], [[[7.0, 3.0, 1.0], [8.0, 
4.0, 1.0], [8.0, 1.0, 6.0]], [[2.0, 1.0, 7.0], [4.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[1.0, 8.0, 6.0], [2.0, 1.0, 3.0], [7.0, 8.0, 
4.0]]], [[[1.0, 7.0, 4.0], [6.0, 8.0, 5.0], [7.0, 6.0, 7.0]], [[8.0, 3.0, 4.0], [1.0, 5.0, 2.0], [8.0, 3.0, 4.0]], [[2.0, 8.0, 
1.0], [4.0, 7.0, 1.0], [5.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 7.0], [5.0, 8.0, 3.0], [6.0, 4.0, 5.0]], 
[[4.0, 7.0, 3.0], [1.0, 5.0, 4.0], [8.0, 5.0, 2.0]], [[6.0, 8.0, 5.0], [5.0, 7.0, 2.0], [2.0, 4.0, 8.0]]], [[[3.0, 6.0, 7.0], 
[3.0, 4.0, 3.0], [2.0, 8.0, 8.0]], [[8.0, 2.0, 4.0], [8.0, 3.0, 1.0], [5.0, 7.0, 4.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 1.0], 
[8.0, 6.0, 7.0]]], [[[7.0, 3.0, 8.0], [8.0, 4.0, 1.0], [1.0, 1.0, 3.0]], [[2.0, 1.0, 8.0], [4.0, 6.0, 1.0], [2.0, 2.0, 2.0]], 
[[1.0, 8.0, 3.0], [2.0, 1.0, 5.0], [3.0, 4.0, 2.0]]], [[[1.0, 7.0, 6.0], [6.0, 8.0, 3.0], [7.0, 1.0, 4.0]], [[8.0, 3.0, 6.0], 
[1.0, 5.0, 2.0], [8.0, 3.0, 5.0]], [[2.0, 8.0, 6.0], [4.0, 7.0, 3.0], [3.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 5.0], [4.0, 4.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 1.0, 5.0], [8.0, 3.0, 8.0], [3.0, 
6.0, 1.0]], [[4.0, 2.0, 7.0], [7.0, 5.0, 6.0], [7.0, 5.0, 7.0]]], [[[2.0, 8.0, 7.0], [5.0, 1.0, 1.0], [6.0, 5.0, 3.0]], [[1.0, 
2.0, 1.0], [1.0, 7.0, 2.0], [2.0, 1.0, 8.0]], [[2.0, 7.0, 8.0], [5.0, 3.0, 4.0], [2.0, 1.0, 4.0]]], [[[1.0, 8.0, 4.0], [2.0, 
2.0, 1.0], [6.0, 7.0, 4.0]], [[6.0, 6.0, 3.0], [3.0, 4.0, 7.0], [3.0, 6.0, 5.0]], [[6.0, 4.0, 6.0], [2.0, 5.0, 3.0], [3.0, 6.0, 
7.0]]], [[[5.0, 6.0, 8.0], [6.0, 4.0, 2.0], [5.0, 7.0, 4.0]], [[8.0, 1.0, 5.0], [8.0, 6.0, 4.0], [8.0, 3.0, 3.0]], [[2.0, 4.0, 
6.0], [2.0, 7.0, 1.0], [6.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 5.0], [1.0, 5.0, 2.0], [7.0, 7.0, 6.0]], 
[[2.0, 5.0, 3.0], [2.0, 4.0, 3.0], [6.0, 5.0, 2.0]], [[3.0, 8.0, 5.0], [1.0, 4.0, 1.0], [6.0, 6.0, 6.0]]], [[[6.0, 7.0, 5.0], 
[7.0, 1.0, 7.0], [5.0, 3.0, 8.0]], [[1.0, 1.0, 5.0], [4.0, 4.0, 5.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 5.0], [1.0, 6.0, 7.0], 
[5.0, 7.0, 3.0]]], [[[1.0, 3.0, 6.0], [3.0, 1.0, 3.0], [8.0, 2.0, 6.0]], [[8.0, 6.0, 1.0], [3.0, 6.0, 7.0], [3.0, 3.0, 7.0]], 
[[4.0, 1.0, 8.0], [5.0, 2.0, 1.0], [2.0, 8.0, 6.0]]], [[[8.0, 8.0, 6.0], [8.0, 5.0, 5.0], [4.0, 1.0, 3.0]], [[7.0, 4.0, 6.0], 
[5.0, 5.0, 2.0], [6.0, 3.0, 3.0]], [[6.0, 7.0, 2.0], [8.0, 6.0, 5.0], [3.0, 1.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 6.0], [3.0, 5.0]], [[6.0, 8.0], [2.0, 5.0]], [[4.0, 8.0], [5.0, 4.0]]], [[[6.0, 6.0], 
[3.0, 4.0]], [[4.0, 7.0], [6.0, 6.0]], [[8.0, 8.0], [6.0, 6.0]]], [[[6.0, 6.0], [6.0, 3.0]], [[5.0, 5.0], [1.0, 2.0]], [[8.0, 
4.0], [3.0, 6.0]]], [[[1.0, 7.0], [2.0, 1.0]], [[8.0, 8.0], [2.0, 7.0]], [[3.0, 5.0], [8.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0], [6.0, 8.0]], [[6.0, 6.0], [3.0, 7.0]], [[6.0, 6.0], [2.0, 6.0]]], [[[5.0, 
4.0], [3.0, 1.0]], [[2.0, 1.0], [3.0, 8.0]], [[1.0, 2.0], [6.0, 8.0]]], [[[3.0, 6.0], [5.0, 7.0]], [[5.0, 5.0], [2.0, 7.0]], 
[[6.0, 4.0], [3.0, 3.0]]], [[[7.0, 7.0], [1.0, 1.0]], [[6.0, 7.0], [3.0, 1.0]], [[8.0, 4.0], [5.0, 1.0]]]])
      arg[:,:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [3.0, 5.0, 7.0], [2.0, 7.0, 1.0]], [[6.0, 8.0, 5.0], [2.0, 5.0, 8.0], [3.0, 
6.0, 1.0]], [[4.0, 8.0, 7.0], [5.0, 4.0, 6.0], [7.0, 5.0, 7.0]]], [[[6.0, 6.0, 7.0], [3.0, 4.0, 1.0], [6.0, 5.0, 3.0]], [[4.0, 
7.0, 1.0], [6.0, 6.0, 2.0], [2.0, 1.0, 8.0]], [[8.0, 8.0, 8.0], [6.0, 6.0, 4.0], [2.0, 1.0, 4.0]]], [[[6.0, 6.0, 4.0], [6.0, 
3.0, 1.0], [6.0, 7.0, 4.0]], [[5.0, 5.0, 3.0], [1.0, 2.0, 7.0], [3.0, 6.0, 5.0]], [[8.0, 4.0, 6.0], [3.0, 6.0, 3.0], [3.0, 6.0, 
7.0]]], [[[1.0, 7.0, 8.0], [2.0, 1.0, 2.0], [5.0, 7.0, 4.0]], [[8.0, 8.0, 5.0], [2.0, 7.0, 4.0], [8.0, 3.0, 3.0]], [[3.0, 5.0, 
6.0], [8.0, 4.0, 1.0], [6.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 5.0], [6.0, 8.0, 2.0], [7.0, 7.0, 6.0]], 
[[6.0, 6.0, 3.0], [3.0, 7.0, 3.0], [6.0, 5.0, 2.0]], [[6.0, 6.0, 5.0], [2.0, 6.0, 1.0], [6.0, 6.0, 6.0]]], [[[5.0, 4.0, 5.0], 
[3.0, 1.0, 7.0], [5.0, 3.0, 8.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 5.0], [5.0, 5.0, 5.0]], [[1.0, 2.0, 5.0], [6.0, 8.0, 7.0], 
[5.0, 7.0, 3.0]]], [[[3.0, 6.0, 6.0], [5.0, 7.0, 3.0], [8.0, 2.0, 6.0]], [[5.0, 5.0, 1.0], [2.0, 7.0, 7.0], [3.0, 3.0, 7.0]], 
[[6.0, 4.0, 8.0], [3.0, 3.0, 1.0], [2.0, 8.0, 6.0]]], [[[7.0, 7.0, 6.0], [1.0, 1.0, 5.0], [4.0, 1.0, 3.0]], [[6.0, 7.0, 6.0], 
[3.0, 1.0, 2.0], [6.0, 3.0, 3.0]], [[8.0, 4.0, 2.0], [5.0, 1.0, 5.0], [3.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 3.0], [6.0, 7.0, 2.0], [5.0, 5.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 8.0, 7.0], [4.0, 
6.0, 1.0]], [[7.0, 6.0, 4.0], [2.0, 5.0, 2.0], [7.0, 2.0, 8.0]]], [[[7.0, 8.0, 7.0], [4.0, 4.0, 7.0], [3.0, 4.0, 2.0]], [[2.0, 
5.0, 5.0], [8.0, 7.0, 2.0], [3.0, 5.0, 7.0]], [[8.0, 1.0, 3.0], [4.0, 7.0, 3.0], [6.0, 6.0, 3.0]]], [[[7.0, 1.0, 1.0], [2.0, 
7.0, 5.0], [8.0, 6.0, 4.0]], [[4.0, 7.0, 1.0], [7.0, 4.0, 2.0], [8.0, 1.0, 5.0]], [[8.0, 2.0, 5.0], [7.0, 1.0, 6.0], [6.0, 2.0, 
3.0]]], [[[8.0, 7.0, 3.0], [1.0, 7.0, 2.0], [8.0, 7.0, 3.0]], [[8.0, 4.0, 6.0], [5.0, 6.0, 6.0], [2.0, 3.0, 3.0]], [[3.0, 6.0, 
1.0], [6.0, 1.0, 5.0], [2.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 2.0], [6.0, 7.0, 7.0], [1.0, 1.0, 1.0]], 
[[4.0, 8.0, 8.0], [1.0, 3.0, 7.0], [8.0, 7.0, 8.0]], [[5.0, 6.0, 8.0], [7.0, 7.0, 2.0], [5.0, 7.0, 6.0]]], [[[7.0, 6.0, 6.0], 
[6.0, 4.0, 8.0], [8.0, 7.0, 1.0]], [[3.0, 3.0, 3.0], [1.0, 8.0, 7.0], [4.0, 8.0, 3.0]], [[1.0, 2.0, 4.0], [7.0, 6.0, 6.0], 
[2.0, 3.0, 5.0]]], [[[1.0, 5.0, 6.0], [3.0, 4.0, 4.0], [8.0, 2.0, 3.0]], [[5.0, 3.0, 3.0], [7.0, 2.0, 8.0], [6.0, 6.0, 1.0]], 
[[2.0, 2.0, 8.0], [1.0, 7.0, 1.0], [4.0, 3.0, 1.0]]], [[[8.0, 7.0, 7.0], [5.0, 7.0, 1.0], [4.0, 2.0, 5.0]], [[6.0, 4.0, 5.0], 
[7.0, 5.0, 4.0], [6.0, 6.0, 7.0]], [[6.0, 7.0, 2.0], [3.0, 7.0, 8.0], [5.0, 2.0, 2.0]]]])
      val=numpy.array([[8.0], [2.0]])
      arg[1:3,1,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 3.0], [6.0, 7.0, 2.0], [5.0, 5.0, 2.0]], [[6.0, 7.0, 3.0], [2.0, 8.0, 7.0], [4.0, 
6.0, 1.0]], [[7.0, 6.0, 4.0], [2.0, 5.0, 2.0], [7.0, 2.0, 8.0]]], [[[7.0, 8.0, 7.0], [4.0, 4.0, 7.0], [3.0, 4.0, 2.0]], [[2.0, 
5.0, 5.0], [8.0, 8.0, 2.0], [3.0, 5.0, 7.0]], [[8.0, 1.0, 3.0], [4.0, 7.0, 3.0], [6.0, 6.0, 3.0]]], [[[7.0, 1.0, 1.0], [2.0, 
7.0, 5.0], [8.0, 6.0, 4.0]], [[4.0, 7.0, 1.0], [7.0, 2.0, 2.0], [8.0, 1.0, 5.0]], [[8.0, 2.0, 5.0], [7.0, 1.0, 6.0], [6.0, 2.0, 
3.0]]], [[[8.0, 7.0, 3.0], [1.0, 7.0, 2.0], [8.0, 7.0, 3.0]], [[8.0, 4.0, 6.0], [5.0, 6.0, 6.0], [2.0, 3.0, 3.0]], [[3.0, 6.0, 
1.0], [6.0, 1.0, 5.0], [2.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 2.0], [6.0, 7.0, 7.0], [1.0, 1.0, 1.0]], 
[[4.0, 8.0, 8.0], [1.0, 3.0, 7.0], [8.0, 7.0, 8.0]], [[5.0, 6.0, 8.0], [7.0, 7.0, 2.0], [5.0, 7.0, 6.0]]], [[[7.0, 6.0, 6.0], 
[6.0, 4.0, 8.0], [8.0, 7.0, 1.0]], [[3.0, 3.0, 3.0], [1.0, 8.0, 7.0], [4.0, 8.0, 3.0]], [[1.0, 2.0, 4.0], [7.0, 6.0, 6.0], 
[2.0, 3.0, 5.0]]], [[[1.0, 5.0, 6.0], [3.0, 4.0, 4.0], [8.0, 2.0, 3.0]], [[5.0, 3.0, 3.0], [7.0, 2.0, 8.0], [6.0, 6.0, 1.0]], 
[[2.0, 2.0, 8.0], [1.0, 7.0, 1.0], [4.0, 3.0, 1.0]]], [[[8.0, 7.0, 7.0], [5.0, 7.0, 1.0], [4.0, 2.0, 5.0]], [[6.0, 4.0, 5.0], 
[7.0, 5.0, 4.0], [6.0, 6.0, 7.0]], [[6.0, 7.0, 2.0], [3.0, 7.0, 8.0], [5.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 1.0], [2.0, 4.0, 1.0], [5.0, 6.0, 6.0]], [[3.0, 5.0, 3.0], [4.0, 2.0, 3.0], [1.0, 
6.0, 5.0]], [[2.0, 1.0, 7.0], [6.0, 2.0, 7.0], [4.0, 6.0, 5.0]]], [[[4.0, 1.0, 6.0], [8.0, 7.0, 2.0], [6.0, 5.0, 7.0]], [[5.0, 
6.0, 5.0], [2.0, 1.0, 4.0], [3.0, 5.0, 8.0]], [[8.0, 4.0, 7.0], [6.0, 1.0, 8.0], [6.0, 6.0, 6.0]]], [[[4.0, 5.0, 4.0], [2.0, 
1.0, 6.0], [5.0, 5.0, 7.0]], [[7.0, 3.0, 2.0], [8.0, 3.0, 4.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 3.0], [1.0, 8.0, 7.0], [8.0, 6.0, 
7.0]]], [[[6.0, 8.0, 7.0], [7.0, 4.0, 2.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [7.0, 3.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 7.0, 
2.0], [4.0, 5.0, 3.0], [3.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 5.0], [1.0, 6.0, 2.0], [2.0, 7.0, 8.0]], 
[[1.0, 8.0, 5.0], [6.0, 3.0, 1.0], [8.0, 4.0, 8.0]], [[6.0, 2.0, 8.0], [3.0, 4.0, 5.0], [2.0, 3.0, 5.0]]], [[[8.0, 4.0, 4.0], 
[4.0, 5.0, 6.0], [7.0, 1.0, 3.0]], [[5.0, 5.0, 3.0], [3.0, 3.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 5.0, 7.0], [7.0, 8.0, 8.0], 
[7.0, 3.0, 8.0]]], [[[6.0, 8.0, 8.0], [2.0, 1.0, 4.0], [4.0, 7.0, 8.0]], [[8.0, 2.0, 2.0], [8.0, 2.0, 8.0], [7.0, 1.0, 5.0]], 
[[6.0, 2.0, 3.0], [2.0, 7.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 3.0, 5.0], [6.0, 3.0, 5.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 8.0], 
[4.0, 3.0, 1.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [8.0, 1.0, 8.0], [1.0, 2.0, 8.0]]]])
      val=Data(numpy.array([[7.0], [5.0]]),self.functionspace)
      arg[1:3,1,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 1.0], [2.0, 4.0, 1.0], [5.0, 6.0, 6.0]], [[3.0, 5.0, 3.0], [4.0, 2.0, 3.0], [1.0, 
6.0, 5.0]], [[2.0, 1.0, 7.0], [6.0, 2.0, 7.0], [4.0, 6.0, 5.0]]], [[[4.0, 1.0, 6.0], [8.0, 7.0, 2.0], [6.0, 5.0, 7.0]], [[5.0, 
6.0, 5.0], [2.0, 7.0, 4.0], [3.0, 5.0, 8.0]], [[8.0, 4.0, 7.0], [6.0, 1.0, 8.0], [6.0, 6.0, 6.0]]], [[[4.0, 5.0, 4.0], [2.0, 
1.0, 6.0], [5.0, 5.0, 7.0]], [[7.0, 3.0, 2.0], [8.0, 5.0, 4.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 3.0], [1.0, 8.0, 7.0], [8.0, 6.0, 
7.0]]], [[[6.0, 8.0, 7.0], [7.0, 4.0, 2.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 2.0], [7.0, 3.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 7.0, 
2.0], [4.0, 5.0, 3.0], [3.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [1.0, 6.0, 2.0], [2.0, 7.0, 8.0]], 
[[1.0, 8.0, 5.0], [6.0, 3.0, 1.0], [8.0, 4.0, 8.0]], [[6.0, 2.0, 8.0], [3.0, 4.0, 5.0], [2.0, 3.0, 5.0]]], [[[8.0, 4.0, 4.0], 
[4.0, 5.0, 6.0], [7.0, 1.0, 3.0]], [[5.0, 5.0, 3.0], [3.0, 7.0, 1.0], [1.0, 8.0, 8.0]], [[1.0, 5.0, 7.0], [7.0, 8.0, 8.0], 
[7.0, 3.0, 8.0]]], [[[6.0, 8.0, 8.0], [2.0, 1.0, 4.0], [4.0, 7.0, 8.0]], [[8.0, 2.0, 2.0], [8.0, 5.0, 8.0], [7.0, 1.0, 5.0]], 
[[6.0, 2.0, 3.0], [2.0, 7.0, 4.0], [5.0, 2.0, 3.0]]], [[[5.0, 3.0, 5.0], [6.0, 3.0, 5.0], [8.0, 2.0, 7.0]], [[8.0, 1.0, 8.0], 
[4.0, 3.0, 1.0], [8.0, 2.0, 2.0]], [[4.0, 3.0, 4.0], [8.0, 1.0, 8.0], [1.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPRP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 6.0], [2.0, 2.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [2.0, 7.0, 3.0], [3.0, 
6.0, 8.0]], [[5.0, 4.0, 7.0], [5.0, 6.0, 3.0], [6.0, 2.0, 6.0]]], [[[5.0, 4.0, 5.0], [8.0, 7.0, 8.0], [6.0, 4.0, 7.0]], [[8.0, 
8.0, 7.0], [4.0, 2.0, 3.0], [8.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [4.0, 6.0, 1.0], [5.0, 3.0, 7.0]]], [[[2.0, 7.0, 2.0], [2.0, 
8.0, 7.0], [1.0, 4.0, 1.0]], [[7.0, 2.0, 1.0], [6.0, 7.0, 6.0], [5.0, 4.0, 2.0]], [[2.0, 3.0, 6.0], [8.0, 8.0, 8.0], [6.0, 4.0, 
3.0]]], [[[4.0, 3.0, 7.0], [8.0, 4.0, 7.0], [2.0, 1.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 8.0, 1.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 
3.0], [2.0, 5.0, 6.0], [8.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 3.0], [1.0, 7.0, 1.0], [3.0, 1.0, 8.0]], 
[[2.0, 3.0, 1.0], [7.0, 7.0, 2.0], [1.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], [5.0, 4.0, 8.0], [5.0, 6.0, 4.0]]], [[[3.0, 1.0, 2.0], 
[6.0, 4.0, 4.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [5.0, 1.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 4.0, 4.0], [6.0, 4.0, 5.0], 
[1.0, 6.0, 2.0]]], [[[6.0, 1.0, 3.0], [1.0, 8.0, 6.0], [7.0, 5.0, 2.0]], [[1.0, 7.0, 5.0], [4.0, 1.0, 2.0], [3.0, 8.0, 5.0]], 
[[8.0, 3.0, 8.0], [5.0, 7.0, 1.0], [8.0, 8.0, 3.0]]], [[[5.0, 8.0, 3.0], [7.0, 1.0, 1.0], [5.0, 8.0, 8.0]], [[4.0, 1.0, 4.0], 
[6.0, 2.0, 5.0], [7.0, 3.0, 5.0]], [[5.0, 8.0, 7.0], [3.0, 1.0, 2.0], [3.0, 3.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0], [5.0]])+(1.-msk_val)*numpy.array([[6.0], [4.0]])
      arg[1:3,1,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 6.0], [2.0, 2.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [2.0, 7.0, 3.0], [3.0, 
6.0, 8.0]], [[5.0, 4.0, 7.0], [5.0, 6.0, 3.0], [6.0, 2.0, 6.0]]], [[[5.0, 4.0, 5.0], [8.0, 7.0, 8.0], [6.0, 4.0, 7.0]], [[8.0, 
8.0, 7.0], [4.0, 3.0, 3.0], [8.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [4.0, 6.0, 1.0], [5.0, 3.0, 7.0]]], [[[2.0, 7.0, 2.0], [2.0, 
8.0, 7.0], [1.0, 4.0, 1.0]], [[7.0, 2.0, 1.0], [6.0, 5.0, 6.0], [5.0, 4.0, 2.0]], [[2.0, 3.0, 6.0], [8.0, 8.0, 8.0], [6.0, 4.0, 
3.0]]], [[[4.0, 3.0, 7.0], [8.0, 4.0, 7.0], [2.0, 1.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 8.0, 1.0], [3.0, 6.0, 8.0]], [[1.0, 2.0, 
3.0], [2.0, 5.0, 6.0], [8.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 3.0], [1.0, 7.0, 1.0], [3.0, 1.0, 8.0]], 
[[2.0, 3.0, 1.0], [7.0, 7.0, 2.0], [1.0, 2.0, 6.0]], [[4.0, 3.0, 7.0], [5.0, 4.0, 8.0], [5.0, 6.0, 4.0]]], [[[3.0, 1.0, 2.0], 
[6.0, 4.0, 4.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [5.0, 6.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 4.0, 4.0], [6.0, 4.0, 5.0], 
[1.0, 6.0, 2.0]]], [[[6.0, 1.0, 3.0], [1.0, 8.0, 6.0], [7.0, 5.0, 2.0]], [[1.0, 7.0, 5.0], [4.0, 4.0, 2.0], [3.0, 8.0, 5.0]], 
[[8.0, 3.0, 8.0], [5.0, 7.0, 1.0], [8.0, 8.0, 3.0]]], [[[5.0, 8.0, 3.0], [7.0, 1.0, 1.0], [5.0, 8.0, 8.0]], [[4.0, 1.0, 4.0], 
[6.0, 2.0, 5.0], [7.0, 3.0, 5.0]], [[5.0, 8.0, 7.0], [3.0, 1.0, 2.0], [3.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 8.0], [2.0, 4.0, 7.0], [6.0, 6.0, 6.0]], [[8.0, 1.0, 7.0], [4.0, 7.0, 2.0], [6.0, 
8.0, 7.0]], [[4.0, 4.0, 6.0], [5.0, 6.0, 6.0], [7.0, 4.0, 7.0]]], [[[4.0, 8.0, 6.0], [1.0, 2.0, 3.0], [5.0, 3.0, 8.0]], [[8.0, 
3.0, 2.0], [1.0, 4.0, 7.0], [3.0, 2.0, 2.0]], [[8.0, 1.0, 3.0], [3.0, 3.0, 1.0], [1.0, 7.0, 8.0]]], [[[6.0, 3.0, 2.0], [3.0, 
4.0, 3.0], [3.0, 1.0, 3.0]], [[1.0, 6.0, 4.0], [7.0, 7.0, 8.0], [5.0, 7.0, 7.0]], [[1.0, 6.0, 2.0], [1.0, 5.0, 7.0], [8.0, 3.0, 
3.0]]], [[[8.0, 7.0, 8.0], [1.0, 1.0, 8.0], [1.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [4.0, 8.0, 4.0], [8.0, 4.0, 3.0]], [[6.0, 7.0, 
2.0], [7.0, 4.0, 1.0], [3.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 7.0], [6.0, 8.0, 1.0], [2.0, 7.0, 3.0]], 
[[4.0, 5.0, 8.0], [7.0, 1.0, 1.0], [7.0, 5.0, 4.0]], [[7.0, 4.0, 2.0], [7.0, 2.0, 6.0], [8.0, 4.0, 6.0]]], [[[3.0, 4.0, 3.0], 
[2.0, 4.0, 6.0], [8.0, 1.0, 8.0]], [[4.0, 7.0, 5.0], [4.0, 6.0, 8.0], [3.0, 5.0, 7.0]], [[2.0, 1.0, 1.0], [8.0, 4.0, 2.0], 
[8.0, 2.0, 8.0]]], [[[3.0, 4.0, 8.0], [2.0, 8.0, 2.0], [3.0, 6.0, 1.0]], [[6.0, 3.0, 1.0], [2.0, 6.0, 7.0], [4.0, 6.0, 1.0]], 
[[1.0, 1.0, 6.0], [4.0, 3.0, 5.0], [2.0, 2.0, 3.0]]], [[[6.0, 7.0, 2.0], [3.0, 6.0, 6.0], [5.0, 3.0, 6.0]], [[1.0, 4.0, 2.0], 
[6.0, 5.0, 6.0], [8.0, 1.0, 2.0]], [[8.0, 1.0, 7.0], [7.0, 1.0, 1.0], [4.0, 3.0, 7.0]]]])
      val=numpy.array([[[8.0, 8.0, 3.0], [1.0, 2.0, 1.0], [8.0, 4.0, 1.0]], [[4.0, 2.0, 4.0], [3.0, 3.0, 7.0], [3.0, 6.0, 
7.0]]])
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 8.0], [2.0, 4.0, 7.0], [6.0, 6.0, 6.0]], [[8.0, 1.0, 7.0], [4.0, 7.0, 2.0], [6.0, 
8.0, 7.0]], [[4.0, 4.0, 6.0], [5.0, 6.0, 6.0], [7.0, 4.0, 7.0]]], [[[4.0, 8.0, 6.0], [1.0, 2.0, 3.0], [5.0, 3.0, 8.0]], [[8.0, 
3.0, 2.0], [1.0, 4.0, 7.0], [3.0, 2.0, 2.0]], [[8.0, 1.0, 3.0], [3.0, 3.0, 1.0], [1.0, 7.0, 8.0]]], [[[6.0, 3.0, 2.0], [8.0, 
8.0, 3.0], [3.0, 1.0, 3.0]], [[1.0, 6.0, 4.0], [1.0, 2.0, 1.0], [5.0, 7.0, 7.0]], [[1.0, 6.0, 2.0], [8.0, 4.0, 1.0], [8.0, 3.0, 
3.0]]], [[[8.0, 7.0, 8.0], [4.0, 2.0, 4.0], [1.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [3.0, 3.0, 7.0], [8.0, 4.0, 3.0]], [[6.0, 7.0, 
2.0], [3.0, 6.0, 7.0], [3.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 7.0], [6.0, 8.0, 1.0], [2.0, 7.0, 3.0]], 
[[4.0, 5.0, 8.0], [7.0, 1.0, 1.0], [7.0, 5.0, 4.0]], [[7.0, 4.0, 2.0], [7.0, 2.0, 6.0], [8.0, 4.0, 6.0]]], [[[3.0, 4.0, 3.0], 
[2.0, 4.0, 6.0], [8.0, 1.0, 8.0]], [[4.0, 7.0, 5.0], [4.0, 6.0, 8.0], [3.0, 5.0, 7.0]], [[2.0, 1.0, 1.0], [8.0, 4.0, 2.0], 
[8.0, 2.0, 8.0]]], [[[3.0, 4.0, 8.0], [8.0, 8.0, 3.0], [3.0, 6.0, 1.0]], [[6.0, 3.0, 1.0], [1.0, 2.0, 1.0], [4.0, 6.0, 1.0]], 
[[1.0, 1.0, 6.0], [8.0, 4.0, 1.0], [2.0, 2.0, 3.0]]], [[[6.0, 7.0, 2.0], [4.0, 2.0, 4.0], [5.0, 3.0, 6.0]], [[1.0, 4.0, 2.0], 
[3.0, 3.0, 7.0], [8.0, 1.0, 2.0]], [[8.0, 1.0, 7.0], [3.0, 6.0, 7.0], [4.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 3.0], [2.0, 4.0, 4.0], [5.0, 1.0, 8.0]], [[2.0, 8.0, 7.0], [7.0, 3.0, 6.0], [8.0, 
7.0, 1.0]], [[3.0, 2.0, 6.0], [2.0, 8.0, 4.0], [7.0, 3.0, 5.0]]], [[[2.0, 4.0, 7.0], [8.0, 3.0, 6.0], [8.0, 6.0, 7.0]], [[2.0, 
5.0, 3.0], [3.0, 3.0, 4.0], [6.0, 2.0, 5.0]], [[6.0, 2.0, 6.0], [8.0, 3.0, 7.0], [2.0, 1.0, 2.0]]], [[[8.0, 4.0, 1.0], [8.0, 
5.0, 7.0], [3.0, 1.0, 3.0]], [[4.0, 6.0, 7.0], [7.0, 4.0, 8.0], [3.0, 4.0, 6.0]], [[2.0, 8.0, 5.0], [2.0, 3.0, 3.0], [3.0, 5.0, 
8.0]]], [[[5.0, 5.0, 8.0], [4.0, 5.0, 8.0], [2.0, 1.0, 6.0]], [[7.0, 5.0, 4.0], [5.0, 8.0, 5.0], [2.0, 4.0, 1.0]], [[2.0, 2.0, 
7.0], [2.0, 1.0, 4.0], [6.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 5.0], [3.0, 1.0, 7.0], [5.0, 4.0, 7.0]], 
[[8.0, 4.0, 5.0], [4.0, 6.0, 4.0], [5.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [6.0, 3.0, 7.0], [2.0, 8.0, 3.0]]], [[[5.0, 2.0, 7.0], 
[6.0, 3.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 6.0], [1.0, 3.0, 2.0]], [[8.0, 2.0, 2.0], [8.0, 4.0, 7.0], 
[2.0, 4.0, 5.0]]], [[[7.0, 8.0, 3.0], [7.0, 6.0, 2.0], [4.0, 3.0, 5.0]], [[8.0, 2.0, 1.0], [5.0, 8.0, 3.0], [6.0, 2.0, 1.0]], 
[[3.0, 1.0, 8.0], [6.0, 2.0, 1.0], [8.0, 3.0, 7.0]]], [[[4.0, 7.0, 2.0], [1.0, 3.0, 6.0], [1.0, 1.0, 7.0]], [[3.0, 1.0, 4.0], 
[6.0, 6.0, 1.0], [3.0, 5.0, 8.0]], [[8.0, 3.0, 7.0], [2.0, 1.0, 1.0], [7.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[7.0, 1.0, 8.0], [6.0, 3.0, 3.0], [6.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [4.0, 8.0, 6.0], [4.0, 1.0, 
5.0]]]),self.functionspace)
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 3.0], [2.0, 4.0, 4.0], [5.0, 1.0, 8.0]], [[2.0, 8.0, 7.0], [7.0, 3.0, 6.0], [8.0, 
7.0, 1.0]], [[3.0, 2.0, 6.0], [2.0, 8.0, 4.0], [7.0, 3.0, 5.0]]], [[[2.0, 4.0, 7.0], [8.0, 3.0, 6.0], [8.0, 6.0, 7.0]], [[2.0, 
5.0, 3.0], [3.0, 3.0, 4.0], [6.0, 2.0, 5.0]], [[6.0, 2.0, 6.0], [8.0, 3.0, 7.0], [2.0, 1.0, 2.0]]], [[[8.0, 4.0, 1.0], [7.0, 
1.0, 8.0], [3.0, 1.0, 3.0]], [[4.0, 6.0, 7.0], [6.0, 3.0, 3.0], [3.0, 4.0, 6.0]], [[2.0, 8.0, 5.0], [6.0, 7.0, 5.0], [3.0, 5.0, 
8.0]]], [[[5.0, 5.0, 8.0], [2.0, 7.0, 8.0], [2.0, 1.0, 6.0]], [[7.0, 5.0, 4.0], [4.0, 8.0, 6.0], [2.0, 4.0, 1.0]], [[2.0, 2.0, 
7.0], [4.0, 1.0, 5.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 5.0], [3.0, 1.0, 7.0], [5.0, 4.0, 7.0]], 
[[8.0, 4.0, 5.0], [4.0, 6.0, 4.0], [5.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [6.0, 3.0, 7.0], [2.0, 8.0, 3.0]]], [[[5.0, 2.0, 7.0], 
[6.0, 3.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 2.0, 4.0], [2.0, 8.0, 6.0], [1.0, 3.0, 2.0]], [[8.0, 2.0, 2.0], [8.0, 4.0, 7.0], 
[2.0, 4.0, 5.0]]], [[[7.0, 8.0, 3.0], [7.0, 1.0, 8.0], [4.0, 3.0, 5.0]], [[8.0, 2.0, 1.0], [6.0, 3.0, 3.0], [6.0, 2.0, 1.0]], 
[[3.0, 1.0, 8.0], [6.0, 7.0, 5.0], [8.0, 3.0, 7.0]]], [[[4.0, 7.0, 2.0], [2.0, 7.0, 8.0], [1.0, 1.0, 7.0]], [[3.0, 1.0, 4.0], 
[4.0, 8.0, 6.0], [3.0, 5.0, 8.0]], [[8.0, 3.0, 7.0], [4.0, 1.0, 5.0], [7.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 5.0], [7.0, 5.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 7.0, 3.0], [8.0, 1.0, 7.0], [4.0, 
4.0, 8.0]], [[7.0, 6.0, 5.0], [1.0, 8.0, 7.0], [1.0, 6.0, 3.0]]], [[[2.0, 2.0, 8.0], [6.0, 1.0, 8.0], [7.0, 6.0, 4.0]], [[7.0, 
2.0, 6.0], [1.0, 2.0, 2.0], [8.0, 2.0, 4.0]], [[3.0, 4.0, 2.0], [8.0, 4.0, 3.0], [2.0, 7.0, 7.0]]], [[[5.0, 5.0, 2.0], [6.0, 
1.0, 5.0], [7.0, 3.0, 7.0]], [[3.0, 8.0, 7.0], [6.0, 4.0, 8.0], [6.0, 8.0, 6.0]], [[5.0, 2.0, 1.0], [6.0, 4.0, 7.0], [1.0, 4.0, 
1.0]]], [[[8.0, 7.0, 4.0], [3.0, 8.0, 5.0], [2.0, 8.0, 5.0]], [[5.0, 8.0, 3.0], [3.0, 2.0, 7.0], [2.0, 8.0, 6.0]], [[1.0, 3.0, 
7.0], [6.0, 7.0, 5.0], [6.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 6.0], [4.0, 5.0, 5.0], [7.0, 7.0, 3.0]], 
[[5.0, 5.0, 4.0], [8.0, 7.0, 7.0], [3.0, 3.0, 5.0]], [[5.0, 4.0, 5.0], [5.0, 2.0, 5.0], [8.0, 3.0, 2.0]]], [[[3.0, 7.0, 3.0], 
[8.0, 2.0, 8.0], [1.0, 1.0, 5.0]], [[4.0, 8.0, 8.0], [1.0, 1.0, 8.0], [4.0, 6.0, 8.0]], [[1.0, 7.0, 3.0], [1.0, 1.0, 2.0], 
[7.0, 4.0, 8.0]]], [[[8.0, 3.0, 7.0], [5.0, 2.0, 7.0], [3.0, 2.0, 1.0]], [[7.0, 5.0, 6.0], [4.0, 1.0, 6.0], [6.0, 2.0, 1.0]], 
[[1.0, 6.0, 5.0], [2.0, 2.0, 2.0], [7.0, 5.0, 4.0]]], [[[7.0, 5.0, 7.0], [1.0, 3.0, 7.0], [5.0, 3.0, 8.0]], [[1.0, 2.0, 8.0], 
[3.0, 3.0, 5.0], [8.0, 4.0, 8.0]], [[4.0, 1.0, 7.0], [7.0, 8.0, 6.0], [1.0, 1.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 2.0, 5.0], [1.0, 1.0, 4.0], [3.0, 6.0, 5.0]], [[6.0, 2.0, 7.0], [8.0, 7.0, 6.0], [6.0, 
3.0, 2.0]]])+(1.-msk_val)*numpy.array([[[2.0, 6.0, 1.0], [2.0, 3.0, 2.0], [5.0, 5.0, 1.0]], [[1.0, 1.0, 2.0], [2.0, 6.0, 8.0], 
[4.0, 8.0, 1.0]]])
      arg[2:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 5.0], [7.0, 5.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 7.0, 3.0], [8.0, 1.0, 7.0], [4.0, 
4.0, 8.0]], [[7.0, 6.0, 5.0], [1.0, 8.0, 7.0], [1.0, 6.0, 3.0]]], [[[2.0, 2.0, 8.0], [6.0, 1.0, 8.0], [7.0, 6.0, 4.0]], [[7.0, 
2.0, 6.0], [1.0, 2.0, 2.0], [8.0, 2.0, 4.0]], [[3.0, 4.0, 2.0], [8.0, 4.0, 3.0], [2.0, 7.0, 7.0]]], [[[5.0, 5.0, 2.0], [3.0, 
2.0, 5.0], [7.0, 3.0, 7.0]], [[3.0, 8.0, 7.0], [1.0, 1.0, 4.0], [6.0, 8.0, 6.0]], [[5.0, 2.0, 1.0], [3.0, 6.0, 5.0], [1.0, 4.0, 
1.0]]], [[[8.0, 7.0, 4.0], [6.0, 2.0, 7.0], [2.0, 8.0, 5.0]], [[5.0, 8.0, 3.0], [8.0, 7.0, 6.0], [2.0, 8.0, 6.0]], [[1.0, 3.0, 
7.0], [6.0, 3.0, 2.0], [6.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 6.0], [4.0, 5.0, 5.0], [7.0, 7.0, 3.0]], 
[[5.0, 5.0, 4.0], [8.0, 7.0, 7.0], [3.0, 3.0, 5.0]], [[5.0, 4.0, 5.0], [5.0, 2.0, 5.0], [8.0, 3.0, 2.0]]], [[[3.0, 7.0, 3.0], 
[8.0, 2.0, 8.0], [1.0, 1.0, 5.0]], [[4.0, 8.0, 8.0], [1.0, 1.0, 8.0], [4.0, 6.0, 8.0]], [[1.0, 7.0, 3.0], [1.0, 1.0, 2.0], 
[7.0, 4.0, 8.0]]], [[[8.0, 3.0, 7.0], [2.0, 6.0, 1.0], [3.0, 2.0, 1.0]], [[7.0, 5.0, 6.0], [2.0, 3.0, 2.0], [6.0, 2.0, 1.0]], 
[[1.0, 6.0, 5.0], [5.0, 5.0, 1.0], [7.0, 5.0, 4.0]]], [[[7.0, 5.0, 7.0], [1.0, 1.0, 2.0], [5.0, 3.0, 8.0]], [[1.0, 2.0, 8.0], 
[2.0, 6.0, 8.0], [8.0, 4.0, 8.0]], [[4.0, 1.0, 7.0], [4.0, 8.0, 1.0], [1.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 8.0], [4.0, 6.0, 5.0], [8.0, 6.0, 8.0]], [[6.0, 8.0, 1.0], [1.0, 3.0, 5.0], [2.0, 
8.0, 7.0]], [[7.0, 2.0, 6.0], [2.0, 5.0, 5.0], [2.0, 7.0, 4.0]]], [[[8.0, 5.0, 8.0], [5.0, 1.0, 8.0], [3.0, 7.0, 4.0]], [[8.0, 
7.0, 4.0], [1.0, 2.0, 4.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 7.0], [1.0, 2.0, 8.0], [1.0, 7.0, 8.0]]], [[[8.0, 5.0, 8.0], [3.0, 
1.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 6.0, 2.0], [3.0, 5.0, 8.0], [8.0, 1.0, 7.0]], [[2.0, 3.0, 3.0], [6.0, 1.0, 8.0], [3.0, 5.0, 
2.0]]], [[[7.0, 1.0, 2.0], [1.0, 4.0, 3.0], [5.0, 4.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 3.0, 2.0], [1.0, 2.0, 3.0]], [[5.0, 1.0, 
7.0], [6.0, 3.0, 6.0], [1.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 3.0], [3.0, 1.0, 1.0], [8.0, 1.0, 6.0]], 
[[7.0, 6.0, 8.0], [1.0, 4.0, 8.0], [1.0, 6.0, 3.0]], [[3.0, 3.0, 6.0], [8.0, 7.0, 1.0], [4.0, 3.0, 4.0]]], [[[4.0, 8.0, 2.0], 
[1.0, 2.0, 4.0], [2.0, 4.0, 1.0]], [[7.0, 4.0, 2.0], [3.0, 8.0, 6.0], [5.0, 5.0, 4.0]], [[8.0, 3.0, 6.0], [8.0, 8.0, 7.0], 
[4.0, 4.0, 3.0]]], [[[2.0, 4.0, 3.0], [2.0, 5.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [8.0, 3.0, 2.0], [2.0, 4.0, 6.0]], 
[[1.0, 6.0, 6.0], [3.0, 7.0, 2.0], [5.0, 1.0, 8.0]]], [[[7.0, 7.0, 5.0], [4.0, 7.0, 7.0], [4.0, 3.0, 3.0]], [[5.0, 4.0, 3.0], 
[2.0, 1.0, 6.0], [8.0, 4.0, 2.0]], [[2.0, 2.0, 6.0], [5.0, 2.0, 4.0], [4.0, 3.0, 3.0]]]])
      val=numpy.array([[[6.0, 4.0, 2.0]], [[3.0, 8.0, 8.0]], [[7.0, 7.0, 4.0]]])
      arg[:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 8.0], [4.0, 6.0, 5.0], [8.0, 6.0, 8.0]], [[6.0, 8.0, 1.0], [6.0, 4.0, 2.0], [2.0, 
8.0, 7.0]], [[7.0, 2.0, 6.0], [2.0, 5.0, 5.0], [2.0, 7.0, 4.0]]], [[[8.0, 5.0, 8.0], [5.0, 1.0, 8.0], [3.0, 7.0, 4.0]], [[8.0, 
7.0, 4.0], [3.0, 8.0, 8.0], [7.0, 4.0, 3.0]], [[8.0, 7.0, 7.0], [1.0, 2.0, 8.0], [1.0, 7.0, 8.0]]], [[[8.0, 5.0, 8.0], [3.0, 
1.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 6.0, 2.0], [7.0, 7.0, 4.0], [8.0, 1.0, 7.0]], [[2.0, 3.0, 3.0], [6.0, 1.0, 8.0], [3.0, 5.0, 
2.0]]], [[[7.0, 1.0, 2.0], [1.0, 4.0, 3.0], [5.0, 4.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 3.0, 2.0], [1.0, 2.0, 3.0]], [[5.0, 1.0, 
7.0], [6.0, 3.0, 6.0], [1.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [3.0, 1.0, 1.0], [8.0, 1.0, 6.0]], 
[[7.0, 6.0, 8.0], [6.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[3.0, 3.0, 6.0], [8.0, 7.0, 1.0], [4.0, 3.0, 4.0]]], [[[4.0, 8.0, 2.0], 
[1.0, 2.0, 4.0], [2.0, 4.0, 1.0]], [[7.0, 4.0, 2.0], [3.0, 8.0, 8.0], [5.0, 5.0, 4.0]], [[8.0, 3.0, 6.0], [8.0, 8.0, 7.0], 
[4.0, 4.0, 3.0]]], [[[2.0, 4.0, 3.0], [2.0, 5.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 3.0, 3.0], [7.0, 7.0, 4.0], [2.0, 4.0, 6.0]], 
[[1.0, 6.0, 6.0], [3.0, 7.0, 2.0], [5.0, 1.0, 8.0]]], [[[7.0, 7.0, 5.0], [4.0, 7.0, 7.0], [4.0, 3.0, 3.0]], [[5.0, 4.0, 3.0], 
[2.0, 1.0, 6.0], [8.0, 4.0, 2.0]], [[2.0, 2.0, 6.0], [5.0, 2.0, 4.0], [4.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 4.0], [4.0, 1.0, 2.0], [6.0, 3.0, 7.0]], [[1.0, 4.0, 3.0], [1.0, 2.0, 7.0], [3.0, 
5.0, 2.0]], [[8.0, 1.0, 1.0], [7.0, 5.0, 2.0], [4.0, 5.0, 1.0]]], [[[7.0, 8.0, 8.0], [2.0, 5.0, 8.0], [7.0, 8.0, 7.0]], [[5.0, 
3.0, 5.0], [2.0, 2.0, 5.0], [7.0, 1.0, 3.0]], [[4.0, 1.0, 5.0], [6.0, 5.0, 1.0], [2.0, 6.0, 7.0]]], [[[7.0, 5.0, 7.0], [1.0, 
7.0, 5.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 1.0], [1.0, 7.0, 8.0], [5.0, 8.0, 8.0]], [[7.0, 1.0, 1.0], [7.0, 6.0, 5.0], [6.0, 4.0, 
8.0]]], [[[5.0, 4.0, 1.0], [2.0, 4.0, 4.0], [7.0, 6.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 2.0, 5.0], [5.0, 7.0, 8.0]], [[7.0, 2.0, 
5.0], [1.0, 4.0, 8.0], [3.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 1.0], [6.0, 1.0, 1.0]], 
[[5.0, 2.0, 2.0], [4.0, 1.0, 1.0], [1.0, 1.0, 1.0]], [[6.0, 8.0, 8.0], [2.0, 4.0, 2.0], [6.0, 6.0, 6.0]]], [[[1.0, 3.0, 4.0], 
[3.0, 3.0, 2.0], [3.0, 7.0, 8.0]], [[6.0, 6.0, 7.0], [3.0, 7.0, 4.0], [6.0, 6.0, 8.0]], [[1.0, 1.0, 2.0], [4.0, 6.0, 4.0], 
[3.0, 8.0, 6.0]]], [[[7.0, 7.0, 8.0], [3.0, 2.0, 8.0], [2.0, 5.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 4.0, 8.0], [2.0, 7.0, 2.0]], 
[[8.0, 7.0, 8.0], [2.0, 4.0, 3.0], [7.0, 6.0, 4.0]]], [[[2.0, 5.0, 5.0], [6.0, 8.0, 8.0], [2.0, 1.0, 5.0]], [[1.0, 4.0, 5.0], 
[5.0, 3.0, 5.0], [1.0, 6.0, 3.0]], [[4.0, 1.0, 2.0], [3.0, 1.0, 8.0], [1.0, 5.0, 3.0]]]])
      val=Data(numpy.array([[[6.0, 1.0, 7.0]], [[6.0, 6.0, 7.0]], [[6.0, 7.0, 4.0]]]),self.functionspace)
      arg[:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 4.0], [4.0, 1.0, 2.0], [6.0, 3.0, 7.0]], [[1.0, 4.0, 3.0], [6.0, 1.0, 7.0], [3.0, 
5.0, 2.0]], [[8.0, 1.0, 1.0], [7.0, 5.0, 2.0], [4.0, 5.0, 1.0]]], [[[7.0, 8.0, 8.0], [2.0, 5.0, 8.0], [7.0, 8.0, 7.0]], [[5.0, 
3.0, 5.0], [6.0, 6.0, 7.0], [7.0, 1.0, 3.0]], [[4.0, 1.0, 5.0], [6.0, 5.0, 1.0], [2.0, 6.0, 7.0]]], [[[7.0, 5.0, 7.0], [1.0, 
7.0, 5.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 1.0], [6.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 1.0, 1.0], [7.0, 6.0, 5.0], [6.0, 4.0, 
8.0]]], [[[5.0, 4.0, 1.0], [2.0, 4.0, 4.0], [7.0, 6.0, 4.0]], [[5.0, 2.0, 2.0], [1.0, 2.0, 5.0], [5.0, 7.0, 8.0]], [[7.0, 2.0, 
5.0], [1.0, 4.0, 8.0], [3.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 6.0], [4.0, 8.0, 1.0], [6.0, 1.0, 1.0]], 
[[5.0, 2.0, 2.0], [6.0, 1.0, 7.0], [1.0, 1.0, 1.0]], [[6.0, 8.0, 8.0], [2.0, 4.0, 2.0], [6.0, 6.0, 6.0]]], [[[1.0, 3.0, 4.0], 
[3.0, 3.0, 2.0], [3.0, 7.0, 8.0]], [[6.0, 6.0, 7.0], [6.0, 6.0, 7.0], [6.0, 6.0, 8.0]], [[1.0, 1.0, 2.0], [4.0, 6.0, 4.0], 
[3.0, 8.0, 6.0]]], [[[7.0, 7.0, 8.0], [3.0, 2.0, 8.0], [2.0, 5.0, 5.0]], [[8.0, 4.0, 7.0], [6.0, 7.0, 4.0], [2.0, 7.0, 2.0]], 
[[8.0, 7.0, 8.0], [2.0, 4.0, 3.0], [7.0, 6.0, 4.0]]], [[[2.0, 5.0, 5.0], [6.0, 8.0, 8.0], [2.0, 1.0, 5.0]], [[1.0, 4.0, 5.0], 
[5.0, 3.0, 5.0], [1.0, 6.0, 3.0]], [[4.0, 1.0, 2.0], [3.0, 1.0, 8.0], [1.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 8.0], [8.0, 5.0, 6.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 2.0], [4.0, 3.0, 7.0], [5.0, 
6.0, 6.0]], [[1.0, 3.0, 6.0], [6.0, 3.0, 8.0], [8.0, 3.0, 3.0]]], [[[2.0, 2.0, 8.0], [7.0, 7.0, 4.0], [2.0, 3.0, 5.0]], [[4.0, 
7.0, 8.0], [7.0, 4.0, 4.0], [6.0, 7.0, 5.0]], [[2.0, 4.0, 1.0], [2.0, 1.0, 2.0], [1.0, 5.0, 2.0]]], [[[6.0, 3.0, 6.0], [6.0, 
8.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [5.0, 5.0, 8.0], [7.0, 4.0, 8.0]], [[7.0, 5.0, 5.0], [8.0, 4.0, 8.0], [3.0, 3.0, 
7.0]]], [[[5.0, 8.0, 6.0], [5.0, 4.0, 1.0], [3.0, 8.0, 4.0]], [[5.0, 1.0, 1.0], [3.0, 1.0, 2.0], [5.0, 1.0, 7.0]], [[6.0, 8.0, 
3.0], [2.0, 1.0, 6.0], [5.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 8.0], [3.0, 8.0, 3.0], [3.0, 3.0, 2.0]], 
[[1.0, 8.0, 2.0], [1.0, 3.0, 1.0], [7.0, 1.0, 8.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 5.0], [1.0, 4.0, 7.0]]], [[[7.0, 1.0, 5.0], 
[6.0, 1.0, 3.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 3.0], [6.0, 5.0, 4.0], [3.0, 7.0, 4.0]], [[3.0, 6.0, 8.0], [7.0, 2.0, 6.0], 
[4.0, 1.0, 1.0]]], [[[8.0, 5.0, 5.0], [6.0, 6.0, 6.0], [5.0, 5.0, 6.0]], [[5.0, 4.0, 2.0], [8.0, 8.0, 8.0], [6.0, 1.0, 4.0]], 
[[6.0, 8.0, 7.0], [6.0, 5.0, 2.0], [3.0, 2.0, 4.0]]], [[[1.0, 7.0, 4.0], [2.0, 3.0, 2.0], [3.0, 2.0, 2.0]], [[1.0, 7.0, 4.0], 
[6.0, 1.0, 1.0], [3.0, 2.0, 7.0]], [[4.0, 8.0, 3.0], [8.0, 1.0, 6.0], [4.0, 1.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 3.0, 2.0]], [[3.0, 4.0, 7.0]], [[8.0, 2.0, 4.0]]])+(1.-msk_val)*numpy.array([[[3.0, 7.0, 
3.0]], [[7.0, 2.0, 2.0]], [[1.0, 5.0, 4.0]]])
      arg[:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 8.0], [8.0, 5.0, 6.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 2.0], [3.0, 3.0, 2.0], [5.0, 
6.0, 6.0]], [[1.0, 3.0, 6.0], [6.0, 3.0, 8.0], [8.0, 3.0, 3.0]]], [[[2.0, 2.0, 8.0], [7.0, 7.0, 4.0], [2.0, 3.0, 5.0]], [[4.0, 
7.0, 8.0], [3.0, 4.0, 7.0], [6.0, 7.0, 5.0]], [[2.0, 4.0, 1.0], [2.0, 1.0, 2.0], [1.0, 5.0, 2.0]]], [[[6.0, 3.0, 6.0], [6.0, 
8.0, 1.0], [5.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [8.0, 2.0, 4.0], [7.0, 4.0, 8.0]], [[7.0, 5.0, 5.0], [8.0, 4.0, 8.0], [3.0, 3.0, 
7.0]]], [[[5.0, 8.0, 6.0], [5.0, 4.0, 1.0], [3.0, 8.0, 4.0]], [[5.0, 1.0, 1.0], [3.0, 1.0, 2.0], [5.0, 1.0, 7.0]], [[6.0, 8.0, 
3.0], [2.0, 1.0, 6.0], [5.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 8.0], [3.0, 8.0, 3.0], [3.0, 3.0, 2.0]], 
[[1.0, 8.0, 2.0], [3.0, 7.0, 3.0], [7.0, 1.0, 8.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 5.0], [1.0, 4.0, 7.0]]], [[[7.0, 1.0, 5.0], 
[6.0, 1.0, 3.0], [2.0, 6.0, 1.0]], [[1.0, 5.0, 3.0], [7.0, 2.0, 2.0], [3.0, 7.0, 4.0]], [[3.0, 6.0, 8.0], [7.0, 2.0, 6.0], 
[4.0, 1.0, 1.0]]], [[[8.0, 5.0, 5.0], [6.0, 6.0, 6.0], [5.0, 5.0, 6.0]], [[5.0, 4.0, 2.0], [1.0, 5.0, 4.0], [6.0, 1.0, 4.0]], 
[[6.0, 8.0, 7.0], [6.0, 5.0, 2.0], [3.0, 2.0, 4.0]]], [[[1.0, 7.0, 4.0], [2.0, 3.0, 2.0], [3.0, 2.0, 2.0]], [[1.0, 7.0, 4.0], 
[6.0, 1.0, 1.0], [3.0, 2.0, 7.0]], [[4.0, 8.0, 3.0], [8.0, 1.0, 6.0], [4.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 3.0], [4.0, 6.0, 8.0], [4.0, 8.0, 7.0]], [[4.0, 5.0, 2.0], [4.0, 4.0, 3.0], [7.0, 
1.0, 1.0]], [[2.0, 6.0, 1.0], [5.0, 6.0, 2.0], [5.0, 4.0, 4.0]]], [[[3.0, 5.0, 5.0], [5.0, 8.0, 5.0], [8.0, 7.0, 1.0]], [[1.0, 
1.0, 3.0], [8.0, 5.0, 4.0], [6.0, 8.0, 2.0]], [[3.0, 4.0, 8.0], [6.0, 2.0, 3.0], [6.0, 1.0, 7.0]]], [[[8.0, 1.0, 4.0], [5.0, 
3.0, 6.0], [8.0, 3.0, 1.0]], [[7.0, 3.0, 8.0], [6.0, 2.0, 4.0], [3.0, 8.0, 5.0]], [[8.0, 4.0, 8.0], [3.0, 4.0, 7.0], [4.0, 2.0, 
2.0]]], [[[5.0, 1.0, 4.0], [5.0, 8.0, 7.0], [1.0, 8.0, 3.0]], [[1.0, 3.0, 3.0], [8.0, 6.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 2.0, 
5.0], [1.0, 5.0, 4.0], [3.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 2.0], [1.0, 6.0, 3.0], [6.0, 6.0, 4.0]], 
[[4.0, 4.0, 2.0], [6.0, 6.0, 4.0], [4.0, 6.0, 6.0]], [[4.0, 7.0, 2.0], [4.0, 6.0, 8.0], [8.0, 5.0, 8.0]]], [[[8.0, 2.0, 3.0], 
[1.0, 8.0, 5.0], [7.0, 8.0, 8.0]], [[1.0, 8.0, 8.0], [1.0, 4.0, 5.0], [7.0, 1.0, 7.0]], [[6.0, 3.0, 6.0], [3.0, 1.0, 3.0], 
[2.0, 8.0, 7.0]]], [[[5.0, 5.0, 2.0], [6.0, 6.0, 3.0], [2.0, 7.0, 2.0]], [[1.0, 3.0, 4.0], [4.0, 4.0, 5.0], [7.0, 5.0, 5.0]], 
[[7.0, 7.0, 6.0], [8.0, 6.0, 3.0], [5.0, 5.0, 7.0]]], [[[1.0, 4.0, 2.0], [1.0, 7.0, 2.0], [7.0, 7.0, 3.0]], [[4.0, 4.0, 1.0], 
[2.0, 3.0, 6.0], [2.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 8.0], [7.0, 8.0, 6.0]]]])
      val=numpy.array([[[[8.0, 4.0], [2.0, 7.0]]], [[[1.0, 5.0], [3.0, 8.0]]], [[[8.0, 3.0], [6.0, 2.0]]]])
      arg[:3,2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [4.0, 6.0, 8.0], [4.0, 8.0, 7.0]], [[4.0, 5.0, 2.0], [4.0, 4.0, 3.0], [7.0, 
1.0, 1.0]], [[8.0, 4.0, 1.0], [2.0, 7.0, 2.0], [5.0, 4.0, 4.0]]], [[[3.0, 5.0, 5.0], [5.0, 8.0, 5.0], [8.0, 7.0, 1.0]], [[1.0, 
1.0, 3.0], [8.0, 5.0, 4.0], [6.0, 8.0, 2.0]], [[1.0, 5.0, 8.0], [3.0, 8.0, 3.0], [6.0, 1.0, 7.0]]], [[[8.0, 1.0, 4.0], [5.0, 
3.0, 6.0], [8.0, 3.0, 1.0]], [[7.0, 3.0, 8.0], [6.0, 2.0, 4.0], [3.0, 8.0, 5.0]], [[8.0, 3.0, 8.0], [6.0, 2.0, 7.0], [4.0, 2.0, 
2.0]]], [[[5.0, 1.0, 4.0], [5.0, 8.0, 7.0], [1.0, 8.0, 3.0]], [[1.0, 3.0, 3.0], [8.0, 6.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 2.0, 
5.0], [1.0, 5.0, 4.0], [3.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [1.0, 6.0, 3.0], [6.0, 6.0, 4.0]], 
[[4.0, 4.0, 2.0], [6.0, 6.0, 4.0], [4.0, 6.0, 6.0]], [[8.0, 4.0, 2.0], [2.0, 7.0, 8.0], [8.0, 5.0, 8.0]]], [[[8.0, 2.0, 3.0], 
[1.0, 8.0, 5.0], [7.0, 8.0, 8.0]], [[1.0, 8.0, 8.0], [1.0, 4.0, 5.0], [7.0, 1.0, 7.0]], [[1.0, 5.0, 6.0], [3.0, 8.0, 3.0], 
[2.0, 8.0, 7.0]]], [[[5.0, 5.0, 2.0], [6.0, 6.0, 3.0], [2.0, 7.0, 2.0]], [[1.0, 3.0, 4.0], [4.0, 4.0, 5.0], [7.0, 5.0, 5.0]], 
[[8.0, 3.0, 6.0], [6.0, 2.0, 3.0], [5.0, 5.0, 7.0]]], [[[1.0, 4.0, 2.0], [1.0, 7.0, 2.0], [7.0, 7.0, 3.0]], [[4.0, 4.0, 1.0], 
[2.0, 3.0, 6.0], [2.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 8.0], [7.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 8.0], [2.0, 8.0, 4.0], [8.0, 7.0, 5.0]], [[6.0, 1.0, 1.0], [8.0, 2.0, 2.0], [1.0, 
3.0, 7.0]], [[6.0, 7.0, 4.0], [5.0, 6.0, 2.0], [8.0, 1.0, 1.0]]], [[[2.0, 4.0, 8.0], [6.0, 5.0, 3.0], [4.0, 8.0, 5.0]], [[5.0, 
1.0, 3.0], [6.0, 1.0, 4.0], [7.0, 4.0, 8.0]], [[6.0, 5.0, 1.0], [7.0, 2.0, 8.0], [7.0, 3.0, 3.0]]], [[[1.0, 6.0, 6.0], [2.0, 
7.0, 6.0], [5.0, 5.0, 7.0]], [[6.0, 7.0, 4.0], [4.0, 4.0, 1.0], [1.0, 1.0, 6.0]], [[8.0, 7.0, 3.0], [6.0, 5.0, 2.0], [6.0, 4.0, 
4.0]]], [[[2.0, 5.0, 5.0], [3.0, 6.0, 4.0], [3.0, 5.0, 7.0]], [[6.0, 2.0, 1.0], [2.0, 3.0, 3.0], [2.0, 8.0, 5.0]], [[6.0, 4.0, 
7.0], [1.0, 1.0, 1.0], [8.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 3.0], [6.0, 6.0, 6.0], [3.0, 3.0, 6.0]], 
[[7.0, 3.0, 2.0], [7.0, 2.0, 5.0], [1.0, 7.0, 8.0]], [[8.0, 4.0, 3.0], [3.0, 7.0, 8.0], [4.0, 3.0, 8.0]]], [[[5.0, 7.0, 3.0], 
[6.0, 6.0, 4.0], [5.0, 4.0, 1.0]], [[7.0, 4.0, 2.0], [7.0, 2.0, 2.0], [3.0, 1.0, 5.0]], [[4.0, 2.0, 1.0], [7.0, 4.0, 3.0], 
[7.0, 4.0, 3.0]]], [[[4.0, 2.0, 5.0], [6.0, 3.0, 3.0], [4.0, 8.0, 7.0]], [[7.0, 2.0, 5.0], [5.0, 6.0, 6.0], [3.0, 1.0, 4.0]], 
[[3.0, 3.0, 6.0], [7.0, 5.0, 7.0], [1.0, 4.0, 1.0]]], [[[5.0, 2.0, 4.0], [7.0, 6.0, 6.0], [4.0, 2.0, 1.0]], [[4.0, 3.0, 3.0], 
[7.0, 3.0, 5.0], [5.0, 3.0, 6.0]], [[7.0, 2.0, 3.0], [4.0, 1.0, 2.0], [6.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[[7.0, 4.0], [4.0, 7.0]]], [[[3.0, 7.0], [5.0, 1.0]]], [[[5.0, 3.0], [1.0, 
3.0]]]]),self.functionspace)
      arg[:3,2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 8.0], [2.0, 8.0, 4.0], [8.0, 7.0, 5.0]], [[6.0, 1.0, 1.0], [8.0, 2.0, 2.0], [1.0, 
3.0, 7.0]], [[7.0, 4.0, 4.0], [4.0, 7.0, 2.0], [8.0, 1.0, 1.0]]], [[[2.0, 4.0, 8.0], [6.0, 5.0, 3.0], [4.0, 8.0, 5.0]], [[5.0, 
1.0, 3.0], [6.0, 1.0, 4.0], [7.0, 4.0, 8.0]], [[3.0, 7.0, 1.0], [5.0, 1.0, 8.0], [7.0, 3.0, 3.0]]], [[[1.0, 6.0, 6.0], [2.0, 
7.0, 6.0], [5.0, 5.0, 7.0]], [[6.0, 7.0, 4.0], [4.0, 4.0, 1.0], [1.0, 1.0, 6.0]], [[5.0, 3.0, 3.0], [1.0, 3.0, 2.0], [6.0, 4.0, 
4.0]]], [[[2.0, 5.0, 5.0], [3.0, 6.0, 4.0], [3.0, 5.0, 7.0]], [[6.0, 2.0, 1.0], [2.0, 3.0, 3.0], [2.0, 8.0, 5.0]], [[6.0, 4.0, 
7.0], [1.0, 1.0, 1.0], [8.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 3.0], [6.0, 6.0, 6.0], [3.0, 3.0, 6.0]], 
[[7.0, 3.0, 2.0], [7.0, 2.0, 5.0], [1.0, 7.0, 8.0]], [[7.0, 4.0, 3.0], [4.0, 7.0, 8.0], [4.0, 3.0, 8.0]]], [[[5.0, 7.0, 3.0], 
[6.0, 6.0, 4.0], [5.0, 4.0, 1.0]], [[7.0, 4.0, 2.0], [7.0, 2.0, 2.0], [3.0, 1.0, 5.0]], [[3.0, 7.0, 1.0], [5.0, 1.0, 3.0], 
[7.0, 4.0, 3.0]]], [[[4.0, 2.0, 5.0], [6.0, 3.0, 3.0], [4.0, 8.0, 7.0]], [[7.0, 2.0, 5.0], [5.0, 6.0, 6.0], [3.0, 1.0, 4.0]], 
[[5.0, 3.0, 6.0], [1.0, 3.0, 7.0], [1.0, 4.0, 1.0]]], [[[5.0, 2.0, 4.0], [7.0, 6.0, 6.0], [4.0, 2.0, 1.0]], [[4.0, 3.0, 3.0], 
[7.0, 3.0, 5.0], [5.0, 3.0, 6.0]], [[7.0, 2.0, 3.0], [4.0, 1.0, 2.0], [6.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 6.0, 4.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [8.0, 5.0, 3.0], [2.0, 
7.0, 2.0]], [[1.0, 7.0, 8.0], [6.0, 1.0, 4.0], [7.0, 6.0, 5.0]]], [[[6.0, 8.0, 4.0], [5.0, 1.0, 2.0], [5.0, 1.0, 4.0]], [[7.0, 
1.0, 2.0], [1.0, 3.0, 1.0], [6.0, 8.0, 6.0]], [[8.0, 6.0, 8.0], [5.0, 5.0, 7.0], [3.0, 1.0, 7.0]]], [[[5.0, 5.0, 8.0], [6.0, 
1.0, 1.0], [3.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [4.0, 2.0, 8.0], [7.0, 3.0, 8.0]], [[6.0, 3.0, 6.0], [8.0, 7.0, 2.0], [3.0, 5.0, 
6.0]]], [[[4.0, 6.0, 2.0], [2.0, 5.0, 1.0], [5.0, 4.0, 1.0]], [[7.0, 7.0, 4.0], [5.0, 8.0, 4.0], [1.0, 8.0, 1.0]], [[4.0, 7.0, 
6.0], [1.0, 3.0, 7.0], [1.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 3.0], [6.0, 7.0, 4.0], [1.0, 1.0, 4.0]], 
[[3.0, 6.0, 2.0], [5.0, 1.0, 6.0], [4.0, 7.0, 2.0]], [[2.0, 4.0, 6.0], [6.0, 8.0, 7.0], [6.0, 2.0, 5.0]]], [[[6.0, 7.0, 6.0], 
[6.0, 5.0, 2.0], [4.0, 2.0, 7.0]], [[6.0, 6.0, 7.0], [7.0, 5.0, 4.0], [5.0, 5.0, 6.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 8.0], 
[1.0, 2.0, 4.0]]], [[[5.0, 5.0, 5.0], [4.0, 1.0, 4.0], [4.0, 6.0, 5.0]], [[7.0, 6.0, 7.0], [3.0, 1.0, 6.0], [3.0, 8.0, 3.0]], 
[[4.0, 7.0, 3.0], [6.0, 6.0, 6.0], [8.0, 3.0, 6.0]]], [[[8.0, 1.0, 2.0], [8.0, 1.0, 5.0], [7.0, 5.0, 8.0]], [[2.0, 6.0, 4.0], 
[2.0, 7.0, 3.0], [1.0, 7.0, 3.0]], [[7.0, 2.0, 4.0], [7.0, 2.0, 8.0], [8.0, 6.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 2.0], [2.0, 7.0]]], [[[7.0, 7.0], [5.0, 5.0]]], [[[3.0, 4.0], [4.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 7.0], [6.0, 4.0]]], [[[7.0, 8.0], [1.0, 7.0]]], [[[1.0, 6.0], [1.0, 7.0]]]])
      arg[:3,2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 6.0, 4.0], [5.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [8.0, 5.0, 3.0], [2.0, 
7.0, 2.0]], [[1.0, 2.0, 8.0], [2.0, 7.0, 4.0], [7.0, 6.0, 5.0]]], [[[6.0, 8.0, 4.0], [5.0, 1.0, 2.0], [5.0, 1.0, 4.0]], [[7.0, 
1.0, 2.0], [1.0, 3.0, 1.0], [6.0, 8.0, 6.0]], [[7.0, 7.0, 8.0], [5.0, 5.0, 7.0], [3.0, 1.0, 7.0]]], [[[5.0, 5.0, 8.0], [6.0, 
1.0, 1.0], [3.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [4.0, 2.0, 8.0], [7.0, 3.0, 8.0]], [[3.0, 4.0, 6.0], [4.0, 1.0, 2.0], [3.0, 5.0, 
6.0]]], [[[4.0, 6.0, 2.0], [2.0, 5.0, 1.0], [5.0, 4.0, 1.0]], [[7.0, 7.0, 4.0], [5.0, 8.0, 4.0], [1.0, 8.0, 1.0]], [[4.0, 7.0, 
6.0], [1.0, 3.0, 7.0], [1.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 3.0], [6.0, 7.0, 4.0], [1.0, 1.0, 4.0]], 
[[3.0, 6.0, 2.0], [5.0, 1.0, 6.0], [4.0, 7.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 7.0], [6.0, 2.0, 5.0]]], [[[6.0, 7.0, 6.0], 
[6.0, 5.0, 2.0], [4.0, 2.0, 7.0]], [[6.0, 6.0, 7.0], [7.0, 5.0, 4.0], [5.0, 5.0, 6.0]], [[7.0, 8.0, 8.0], [1.0, 7.0, 8.0], 
[1.0, 2.0, 4.0]]], [[[5.0, 5.0, 5.0], [4.0, 1.0, 4.0], [4.0, 6.0, 5.0]], [[7.0, 6.0, 7.0], [3.0, 1.0, 6.0], [3.0, 8.0, 3.0]], 
[[1.0, 6.0, 3.0], [1.0, 7.0, 6.0], [8.0, 3.0, 6.0]]], [[[8.0, 1.0, 2.0], [8.0, 1.0, 5.0], [7.0, 5.0, 8.0]], [[2.0, 6.0, 4.0], 
[2.0, 7.0, 3.0], [1.0, 7.0, 3.0]], [[7.0, 2.0, 4.0], [7.0, 2.0, 8.0], [8.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 2.0], [6.0, 2.0, 1.0], [4.0, 7.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 2.0, 8.0], [8.0, 
1.0, 7.0]], [[7.0, 4.0, 5.0], [2.0, 6.0, 3.0], [2.0, 8.0, 5.0]]], [[[4.0, 8.0, 2.0], [8.0, 8.0, 4.0], [2.0, 2.0, 3.0]], [[5.0, 
1.0, 6.0], [1.0, 2.0, 8.0], [6.0, 2.0, 2.0]], [[8.0, 8.0, 2.0], [2.0, 1.0, 2.0], [6.0, 7.0, 1.0]]], [[[2.0, 4.0, 3.0], [3.0, 
2.0, 6.0], [4.0, 1.0, 1.0]], [[7.0, 1.0, 8.0], [8.0, 1.0, 4.0], [3.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [7.0, 5.0, 8.0], [6.0, 7.0, 
2.0]]], [[[5.0, 4.0, 2.0], [2.0, 8.0, 2.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 8.0], [7.0, 1.0, 7.0], [3.0, 2.0, 4.0]], [[4.0, 4.0, 
3.0], [4.0, 6.0, 6.0], [5.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 4.0], [7.0, 3.0, 8.0], [1.0, 3.0, 7.0]], 
[[7.0, 4.0, 7.0], [8.0, 1.0, 8.0], [5.0, 1.0, 6.0]], [[6.0, 4.0, 2.0], [1.0, 6.0, 5.0], [2.0, 3.0, 8.0]]], [[[4.0, 1.0, 1.0], 
[1.0, 4.0, 3.0], [3.0, 2.0, 4.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 1.0], [1.0, 8.0, 2.0]], [[6.0, 7.0, 3.0], [1.0, 2.0, 4.0], 
[3.0, 2.0, 4.0]]], [[[4.0, 5.0, 4.0], [5.0, 7.0, 5.0], [8.0, 2.0, 8.0]], [[3.0, 3.0, 6.0], [6.0, 6.0, 7.0], [4.0, 2.0, 8.0]], 
[[1.0, 6.0, 5.0], [5.0, 5.0, 8.0], [8.0, 5.0, 2.0]]], [[[1.0, 6.0, 4.0], [4.0, 8.0, 1.0], [4.0, 8.0, 3.0]], [[8.0, 2.0, 3.0], 
[8.0, 8.0, 1.0], [6.0, 6.0, 8.0]], [[2.0, 5.0, 5.0], [2.0, 6.0, 8.0], [2.0, 4.0, 7.0]]]])
      val=numpy.array([[[[2.0, 6.0, 7.0]]], [[[8.0, 4.0, 8.0]]]])
      arg[2:,1:2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 2.0], [6.0, 2.0, 1.0], [4.0, 7.0, 3.0]], [[2.0, 3.0, 2.0], [1.0, 2.0, 8.0], [8.0, 
1.0, 7.0]], [[7.0, 4.0, 5.0], [2.0, 6.0, 3.0], [2.0, 8.0, 5.0]]], [[[4.0, 8.0, 2.0], [8.0, 8.0, 4.0], [2.0, 2.0, 3.0]], [[5.0, 
1.0, 6.0], [1.0, 2.0, 8.0], [6.0, 2.0, 2.0]], [[8.0, 8.0, 2.0], [2.0, 1.0, 2.0], [6.0, 7.0, 1.0]]], [[[2.0, 4.0, 3.0], [3.0, 
2.0, 6.0], [4.0, 1.0, 1.0]], [[7.0, 1.0, 8.0], [2.0, 6.0, 7.0], [3.0, 6.0, 5.0]], [[6.0, 1.0, 3.0], [7.0, 5.0, 8.0], [6.0, 7.0, 
2.0]]], [[[5.0, 4.0, 2.0], [2.0, 8.0, 2.0], [8.0, 1.0, 3.0]], [[1.0, 1.0, 8.0], [8.0, 4.0, 8.0], [3.0, 2.0, 4.0]], [[4.0, 4.0, 
3.0], [4.0, 6.0, 6.0], [5.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 4.0], [7.0, 3.0, 8.0], [1.0, 3.0, 7.0]], 
[[7.0, 4.0, 7.0], [8.0, 1.0, 8.0], [5.0, 1.0, 6.0]], [[6.0, 4.0, 2.0], [1.0, 6.0, 5.0], [2.0, 3.0, 8.0]]], [[[4.0, 1.0, 1.0], 
[1.0, 4.0, 3.0], [3.0, 2.0, 4.0]], [[4.0, 2.0, 4.0], [4.0, 5.0, 1.0], [1.0, 8.0, 2.0]], [[6.0, 7.0, 3.0], [1.0, 2.0, 4.0], 
[3.0, 2.0, 4.0]]], [[[4.0, 5.0, 4.0], [5.0, 7.0, 5.0], [8.0, 2.0, 8.0]], [[3.0, 3.0, 6.0], [2.0, 6.0, 7.0], [4.0, 2.0, 8.0]], 
[[1.0, 6.0, 5.0], [5.0, 5.0, 8.0], [8.0, 5.0, 2.0]]], [[[1.0, 6.0, 4.0], [4.0, 8.0, 1.0], [4.0, 8.0, 3.0]], [[8.0, 2.0, 3.0], 
[8.0, 4.0, 8.0], [6.0, 6.0, 8.0]], [[2.0, 5.0, 5.0], [2.0, 6.0, 8.0], [2.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 1.0], [4.0, 7.0, 3.0], [1.0, 3.0, 7.0]], [[8.0, 7.0, 1.0], [6.0, 8.0, 5.0], [1.0, 
1.0, 2.0]], [[2.0, 8.0, 1.0], [2.0, 1.0, 8.0], [7.0, 6.0, 4.0]]], [[[7.0, 4.0, 8.0], [2.0, 1.0, 7.0], [1.0, 3.0, 3.0]], [[7.0, 
3.0, 6.0], [2.0, 7.0, 1.0], [3.0, 5.0, 3.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 3.0], [6.0, 6.0, 6.0]]], [[[4.0, 6.0, 4.0], [8.0, 
5.0, 4.0], [1.0, 7.0, 3.0]], [[7.0, 3.0, 7.0], [7.0, 7.0, 6.0], [8.0, 8.0, 4.0]], [[1.0, 8.0, 1.0], [8.0, 3.0, 7.0], [4.0, 5.0, 
3.0]]], [[[1.0, 4.0, 5.0], [6.0, 2.0, 8.0], [1.0, 6.0, 4.0]], [[8.0, 6.0, 6.0], [1.0, 1.0, 7.0], [6.0, 7.0, 1.0]], [[7.0, 1.0, 
6.0], [2.0, 1.0, 4.0], [2.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 6.0], [4.0, 2.0, 2.0], [6.0, 5.0, 8.0]], 
[[5.0, 8.0, 1.0], [2.0, 6.0, 2.0], [7.0, 1.0, 6.0]], [[7.0, 6.0, 3.0], [1.0, 6.0, 5.0], [2.0, 5.0, 1.0]]], [[[5.0, 8.0, 7.0], 
[8.0, 1.0, 5.0], [3.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], [7.0, 5.0, 4.0], [7.0, 2.0, 7.0]], [[1.0, 4.0, 7.0], [3.0, 7.0, 8.0], 
[4.0, 3.0, 7.0]]], [[[3.0, 7.0, 2.0], [3.0, 2.0, 7.0], [5.0, 1.0, 2.0]], [[7.0, 2.0, 8.0], [8.0, 3.0, 2.0], [8.0, 5.0, 2.0]], 
[[8.0, 6.0, 1.0], [2.0, 7.0, 2.0], [2.0, 7.0, 8.0]]], [[[8.0, 3.0, 8.0], [7.0, 1.0, 7.0], [8.0, 6.0, 2.0]], [[1.0, 5.0, 2.0], 
[3.0, 3.0, 6.0], [5.0, 2.0, 2.0]], [[2.0, 6.0, 7.0], [7.0, 8.0, 6.0], [6.0, 5.0, 2.0]]]])
      val=Data(numpy.array([[[[6.0, 7.0, 2.0]]], [[[1.0, 3.0, 1.0]]]]),self.functionspace)
      arg[2:,1:2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 1.0], [4.0, 7.0, 3.0], [1.0, 3.0, 7.0]], [[8.0, 7.0, 1.0], [6.0, 8.0, 5.0], [1.0, 
1.0, 2.0]], [[2.0, 8.0, 1.0], [2.0, 1.0, 8.0], [7.0, 6.0, 4.0]]], [[[7.0, 4.0, 8.0], [2.0, 1.0, 7.0], [1.0, 3.0, 3.0]], [[7.0, 
3.0, 6.0], [2.0, 7.0, 1.0], [3.0, 5.0, 3.0]], [[2.0, 6.0, 5.0], [1.0, 3.0, 3.0], [6.0, 6.0, 6.0]]], [[[4.0, 6.0, 4.0], [8.0, 
5.0, 4.0], [1.0, 7.0, 3.0]], [[7.0, 3.0, 7.0], [6.0, 7.0, 2.0], [8.0, 8.0, 4.0]], [[1.0, 8.0, 1.0], [8.0, 3.0, 7.0], [4.0, 5.0, 
3.0]]], [[[1.0, 4.0, 5.0], [6.0, 2.0, 8.0], [1.0, 6.0, 4.0]], [[8.0, 6.0, 6.0], [1.0, 3.0, 1.0], [6.0, 7.0, 1.0]], [[7.0, 1.0, 
6.0], [2.0, 1.0, 4.0], [2.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 6.0], [4.0, 2.0, 2.0], [6.0, 5.0, 8.0]], 
[[5.0, 8.0, 1.0], [2.0, 6.0, 2.0], [7.0, 1.0, 6.0]], [[7.0, 6.0, 3.0], [1.0, 6.0, 5.0], [2.0, 5.0, 1.0]]], [[[5.0, 8.0, 7.0], 
[8.0, 1.0, 5.0], [3.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], [7.0, 5.0, 4.0], [7.0, 2.0, 7.0]], [[1.0, 4.0, 7.0], [3.0, 7.0, 8.0], 
[4.0, 3.0, 7.0]]], [[[3.0, 7.0, 2.0], [3.0, 2.0, 7.0], [5.0, 1.0, 2.0]], [[7.0, 2.0, 8.0], [6.0, 7.0, 2.0], [8.0, 5.0, 2.0]], 
[[8.0, 6.0, 1.0], [2.0, 7.0, 2.0], [2.0, 7.0, 8.0]]], [[[8.0, 3.0, 8.0], [7.0, 1.0, 7.0], [8.0, 6.0, 2.0]], [[1.0, 5.0, 2.0], 
[1.0, 3.0, 1.0], [5.0, 2.0, 2.0]], [[2.0, 6.0, 7.0], [7.0, 8.0, 6.0], [6.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 6.0], [6.0, 7.0, 2.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 2.0], [4.0, 8.0, 3.0], [7.0, 
3.0, 1.0]], [[4.0, 7.0, 6.0], [4.0, 6.0, 1.0], [7.0, 6.0, 8.0]]], [[[1.0, 6.0, 4.0], [2.0, 4.0, 6.0], [5.0, 2.0, 5.0]], [[2.0, 
7.0, 3.0], [8.0, 6.0, 7.0], [3.0, 8.0, 1.0]], [[5.0, 4.0, 8.0], [1.0, 6.0, 8.0], [6.0, 5.0, 6.0]]], [[[8.0, 5.0, 4.0], [3.0, 
5.0, 8.0], [6.0, 2.0, 6.0]], [[6.0, 5.0, 3.0], [4.0, 7.0, 5.0], [5.0, 3.0, 1.0]], [[2.0, 8.0, 8.0], [5.0, 6.0, 3.0], [7.0, 6.0, 
2.0]]], [[[8.0, 6.0, 8.0], [2.0, 8.0, 1.0], [8.0, 6.0, 7.0]], [[8.0, 6.0, 1.0], [5.0, 5.0, 8.0], [8.0, 5.0, 7.0]], [[4.0, 7.0, 
1.0], [6.0, 7.0, 3.0], [1.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 7.0], [6.0, 4.0, 4.0], [4.0, 1.0, 1.0]], 
[[6.0, 6.0, 3.0], [5.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 2.0], [6.0, 6.0, 3.0], [4.0, 6.0, 5.0]]], [[[3.0, 1.0, 1.0], 
[4.0, 4.0, 8.0], [5.0, 7.0, 1.0]], [[3.0, 3.0, 6.0], [2.0, 3.0, 4.0], [6.0, 6.0, 8.0]], [[5.0, 2.0, 3.0], [1.0, 3.0, 3.0], 
[1.0, 3.0, 1.0]]], [[[5.0, 4.0, 5.0], [1.0, 7.0, 2.0], [8.0, 3.0, 5.0]], [[2.0, 6.0, 7.0], [7.0, 2.0, 5.0], [8.0, 4.0, 1.0]], 
[[4.0, 2.0, 4.0], [5.0, 2.0, 3.0], [4.0, 1.0, 8.0]]], [[[3.0, 4.0, 5.0], [1.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[6.0, 5.0, 1.0], 
[6.0, 6.0, 2.0], [8.0, 4.0, 4.0]], [[6.0, 2.0, 4.0], [6.0, 2.0, 4.0], [6.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 6.0, 6.0]]], [[[3.0, 4.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 4.0, 4.0]]], 
[[[5.0, 6.0, 2.0]]]])
      arg[2:,1:2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 6.0], [6.0, 7.0, 2.0], [2.0, 1.0, 5.0]], [[6.0, 1.0, 2.0], [4.0, 8.0, 3.0], [7.0, 
3.0, 1.0]], [[4.0, 7.0, 6.0], [4.0, 6.0, 1.0], [7.0, 6.0, 8.0]]], [[[1.0, 6.0, 4.0], [2.0, 4.0, 6.0], [5.0, 2.0, 5.0]], [[2.0, 
7.0, 3.0], [8.0, 6.0, 7.0], [3.0, 8.0, 1.0]], [[5.0, 4.0, 8.0], [1.0, 6.0, 8.0], [6.0, 5.0, 6.0]]], [[[8.0, 5.0, 4.0], [3.0, 
5.0, 8.0], [6.0, 2.0, 6.0]], [[6.0, 5.0, 3.0], [2.0, 6.0, 6.0], [5.0, 3.0, 1.0]], [[2.0, 8.0, 8.0], [5.0, 6.0, 3.0], [7.0, 6.0, 
2.0]]], [[[8.0, 6.0, 8.0], [2.0, 8.0, 1.0], [8.0, 6.0, 7.0]], [[8.0, 6.0, 1.0], [3.0, 4.0, 6.0], [8.0, 5.0, 7.0]], [[4.0, 7.0, 
1.0], [6.0, 7.0, 3.0], [1.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [6.0, 4.0, 4.0], [4.0, 1.0, 1.0]], 
[[6.0, 6.0, 3.0], [5.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 2.0], [6.0, 6.0, 3.0], [4.0, 6.0, 5.0]]], [[[3.0, 1.0, 1.0], 
[4.0, 4.0, 8.0], [5.0, 7.0, 1.0]], [[3.0, 3.0, 6.0], [2.0, 3.0, 4.0], [6.0, 6.0, 8.0]], [[5.0, 2.0, 3.0], [1.0, 3.0, 3.0], 
[1.0, 3.0, 1.0]]], [[[5.0, 4.0, 5.0], [1.0, 7.0, 2.0], [8.0, 3.0, 5.0]], [[2.0, 6.0, 7.0], [2.0, 4.0, 4.0], [8.0, 4.0, 1.0]], 
[[4.0, 2.0, 4.0], [5.0, 2.0, 3.0], [4.0, 1.0, 8.0]]], [[[3.0, 4.0, 5.0], [1.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[6.0, 5.0, 1.0], 
[5.0, 6.0, 2.0], [8.0, 4.0, 4.0]], [[6.0, 2.0, 4.0], [6.0, 2.0, 4.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 3.0], [2.0, 4.0, 7.0], [6.0, 6.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 5.0, 3.0], [8.0, 
5.0, 1.0]], [[2.0, 8.0, 5.0], [4.0, 7.0, 8.0], [4.0, 4.0, 3.0]]], [[[5.0, 4.0, 4.0], [7.0, 5.0, 3.0], [2.0, 3.0, 3.0]], [[5.0, 
5.0, 6.0], [5.0, 7.0, 1.0], [8.0, 1.0, 6.0]], [[7.0, 2.0, 4.0], [7.0, 7.0, 2.0], [4.0, 7.0, 2.0]]], [[[4.0, 7.0, 7.0], [1.0, 
3.0, 7.0], [8.0, 5.0, 6.0]], [[2.0, 7.0, 4.0], [4.0, 2.0, 6.0], [7.0, 3.0, 7.0]], [[2.0, 5.0, 4.0], [5.0, 3.0, 7.0], [8.0, 4.0, 
7.0]]], [[[1.0, 4.0, 4.0], [6.0, 6.0, 4.0], [4.0, 3.0, 1.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 7.0], [8.0, 5.0, 1.0]], [[7.0, 8.0, 
8.0], [5.0, 4.0, 3.0], [7.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 2.0], [8.0, 5.0, 3.0], [6.0, 6.0, 6.0]], 
[[5.0, 3.0, 2.0], [1.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[1.0, 2.0, 5.0], [8.0, 1.0, 4.0], [3.0, 4.0, 3.0]]], [[[2.0, 2.0, 8.0], 
[6.0, 2.0, 5.0], [6.0, 2.0, 3.0]], [[8.0, 8.0, 3.0], [7.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[6.0, 4.0, 1.0], [2.0, 3.0, 6.0], 
[5.0, 1.0, 2.0]]], [[[2.0, 6.0, 6.0], [2.0, 2.0, 5.0], [7.0, 1.0, 7.0]], [[2.0, 5.0, 2.0], [6.0, 6.0, 5.0], [6.0, 8.0, 3.0]], 
[[2.0, 3.0, 5.0], [6.0, 6.0, 1.0], [3.0, 4.0, 8.0]]], [[[3.0, 7.0, 3.0], [2.0, 2.0, 3.0], [7.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], 
[4.0, 1.0, 8.0], [1.0, 3.0, 1.0]], [[5.0, 4.0, 6.0], [2.0, 7.0, 2.0], [5.0, 5.0, 7.0]]]])
      val=numpy.array([[[[6.0]]], [[[8.0]]]])
      arg[2:,1:2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 3.0], [2.0, 4.0, 7.0], [6.0, 6.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 5.0, 3.0], [8.0, 
5.0, 1.0]], [[2.0, 8.0, 5.0], [4.0, 7.0, 8.0], [4.0, 4.0, 3.0]]], [[[5.0, 4.0, 4.0], [7.0, 5.0, 3.0], [2.0, 3.0, 3.0]], [[5.0, 
5.0, 6.0], [5.0, 7.0, 1.0], [8.0, 1.0, 6.0]], [[7.0, 2.0, 4.0], [7.0, 7.0, 2.0], [4.0, 7.0, 2.0]]], [[[4.0, 7.0, 7.0], [1.0, 
3.0, 7.0], [8.0, 5.0, 6.0]], [[2.0, 7.0, 4.0], [4.0, 2.0, 6.0], [7.0, 6.0, 7.0]], [[2.0, 5.0, 4.0], [5.0, 3.0, 7.0], [8.0, 4.0, 
7.0]]], [[[1.0, 4.0, 4.0], [6.0, 6.0, 4.0], [4.0, 3.0, 1.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 7.0], [8.0, 8.0, 1.0]], [[7.0, 8.0, 
8.0], [5.0, 4.0, 3.0], [7.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 2.0], [8.0, 5.0, 3.0], [6.0, 6.0, 6.0]], 
[[5.0, 3.0, 2.0], [1.0, 2.0, 6.0], [3.0, 3.0, 2.0]], [[1.0, 2.0, 5.0], [8.0, 1.0, 4.0], [3.0, 4.0, 3.0]]], [[[2.0, 2.0, 8.0], 
[6.0, 2.0, 5.0], [6.0, 2.0, 3.0]], [[8.0, 8.0, 3.0], [7.0, 8.0, 5.0], [5.0, 1.0, 6.0]], [[6.0, 4.0, 1.0], [2.0, 3.0, 6.0], 
[5.0, 1.0, 2.0]]], [[[2.0, 6.0, 6.0], [2.0, 2.0, 5.0], [7.0, 1.0, 7.0]], [[2.0, 5.0, 2.0], [6.0, 6.0, 5.0], [6.0, 6.0, 3.0]], 
[[2.0, 3.0, 5.0], [6.0, 6.0, 1.0], [3.0, 4.0, 8.0]]], [[[3.0, 7.0, 3.0], [2.0, 2.0, 3.0], [7.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], 
[4.0, 1.0, 8.0], [1.0, 8.0, 1.0]], [[5.0, 4.0, 6.0], [2.0, 7.0, 2.0], [5.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 5.0], [2.0, 8.0, 5.0], [1.0, 8.0, 5.0]], [[3.0, 8.0, 7.0], [6.0, 5.0, 6.0], [3.0, 
5.0, 2.0]], [[2.0, 5.0, 4.0], [1.0, 2.0, 3.0], [6.0, 1.0, 8.0]]], [[[5.0, 3.0, 5.0], [6.0, 7.0, 3.0], [5.0, 8.0, 4.0]], [[6.0, 
7.0, 7.0], [8.0, 2.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 2.0], [3.0, 5.0, 4.0]]], [[[6.0, 5.0, 7.0], [2.0, 
7.0, 7.0], [3.0, 6.0, 4.0]], [[2.0, 6.0, 2.0], [2.0, 5.0, 3.0], [1.0, 8.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 3.0, 1.0], [2.0, 2.0, 
8.0]]], [[[2.0, 8.0, 6.0], [2.0, 5.0, 4.0], [3.0, 8.0, 3.0]], [[4.0, 6.0, 8.0], [4.0, 3.0, 8.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 
7.0], [3.0, 8.0, 3.0], [4.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 2.0], [7.0, 7.0, 6.0], [1.0, 8.0, 5.0]], 
[[2.0, 4.0, 1.0], [1.0, 3.0, 3.0], [8.0, 2.0, 2.0]], [[8.0, 6.0, 3.0], [2.0, 3.0, 4.0], [5.0, 3.0, 1.0]]], [[[5.0, 4.0, 6.0], 
[6.0, 8.0, 5.0], [4.0, 4.0, 3.0]], [[1.0, 8.0, 7.0], [5.0, 4.0, 6.0], [3.0, 5.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 6.0], 
[5.0, 6.0, 7.0]]], [[[1.0, 8.0, 2.0], [7.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 3.0], [6.0, 3.0, 7.0], [8.0, 7.0, 6.0]], 
[[5.0, 1.0, 2.0], [7.0, 7.0, 4.0], [5.0, 2.0, 8.0]]], [[[4.0, 7.0, 8.0], [1.0, 5.0, 8.0], [1.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], 
[4.0, 2.0, 4.0], [2.0, 2.0, 5.0]], [[6.0, 3.0, 2.0], [6.0, 8.0, 8.0], [1.0, 6.0, 5.0]]]])
      val=Data(numpy.array([[[[5.0]]], [[[1.0]]]]),self.functionspace)
      arg[2:,1:2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 5.0], [2.0, 8.0, 5.0], [1.0, 8.0, 5.0]], [[3.0, 8.0, 7.0], [6.0, 5.0, 6.0], [3.0, 
5.0, 2.0]], [[2.0, 5.0, 4.0], [1.0, 2.0, 3.0], [6.0, 1.0, 8.0]]], [[[5.0, 3.0, 5.0], [6.0, 7.0, 3.0], [5.0, 8.0, 4.0]], [[6.0, 
7.0, 7.0], [8.0, 2.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 5.0, 1.0], [8.0, 7.0, 2.0], [3.0, 5.0, 4.0]]], [[[6.0, 5.0, 7.0], [2.0, 
7.0, 7.0], [3.0, 6.0, 4.0]], [[2.0, 6.0, 2.0], [2.0, 5.0, 3.0], [1.0, 5.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 3.0, 1.0], [2.0, 2.0, 
8.0]]], [[[2.0, 8.0, 6.0], [2.0, 5.0, 4.0], [3.0, 8.0, 3.0]], [[4.0, 6.0, 8.0], [4.0, 3.0, 8.0], [3.0, 1.0, 8.0]], [[1.0, 4.0, 
7.0], [3.0, 8.0, 3.0], [4.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 2.0], [7.0, 7.0, 6.0], [1.0, 8.0, 5.0]], 
[[2.0, 4.0, 1.0], [1.0, 3.0, 3.0], [8.0, 2.0, 2.0]], [[8.0, 6.0, 3.0], [2.0, 3.0, 4.0], [5.0, 3.0, 1.0]]], [[[5.0, 4.0, 6.0], 
[6.0, 8.0, 5.0], [4.0, 4.0, 3.0]], [[1.0, 8.0, 7.0], [5.0, 4.0, 6.0], [3.0, 5.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 1.0, 6.0], 
[5.0, 6.0, 7.0]]], [[[1.0, 8.0, 2.0], [7.0, 4.0, 2.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 3.0], [6.0, 3.0, 7.0], [8.0, 5.0, 6.0]], 
[[5.0, 1.0, 2.0], [7.0, 7.0, 4.0], [5.0, 2.0, 8.0]]], [[[4.0, 7.0, 8.0], [1.0, 5.0, 8.0], [1.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], 
[4.0, 2.0, 4.0], [2.0, 1.0, 5.0]], [[6.0, 3.0, 2.0], [6.0, 8.0, 8.0], [1.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRLR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 4.0], [1.0, 7.0, 4.0], [1.0, 2.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 6.0, 2.0], [4.0, 
8.0, 8.0]], [[1.0, 2.0, 7.0], [4.0, 6.0, 3.0], [2.0, 2.0, 7.0]]], [[[7.0, 1.0, 4.0], [6.0, 2.0, 1.0], [1.0, 8.0, 3.0]], [[5.0, 
1.0, 3.0], [1.0, 8.0, 8.0], [1.0, 6.0, 4.0]], [[4.0, 5.0, 3.0], [8.0, 8.0, 2.0], [3.0, 2.0, 7.0]]], [[[4.0, 8.0, 3.0], [7.0, 
6.0, 3.0], [4.0, 1.0, 1.0]], [[6.0, 8.0, 6.0], [6.0, 6.0, 7.0], [5.0, 6.0, 2.0]], [[3.0, 6.0, 1.0], [2.0, 3.0, 2.0], [8.0, 5.0, 
3.0]]], [[[4.0, 1.0, 3.0], [5.0, 1.0, 6.0], [1.0, 1.0, 7.0]], [[6.0, 5.0, 6.0], [7.0, 1.0, 2.0], [5.0, 3.0, 6.0]], [[3.0, 7.0, 
7.0], [7.0, 6.0, 8.0], [5.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 2.0], [7.0, 7.0, 1.0], [1.0, 2.0, 4.0]], 
[[2.0, 2.0, 6.0], [1.0, 2.0, 8.0], [1.0, 3.0, 2.0]], [[5.0, 4.0, 4.0], [3.0, 7.0, 6.0], [1.0, 7.0, 8.0]]], [[[8.0, 3.0, 4.0], 
[4.0, 4.0, 4.0], [3.0, 3.0, 6.0]], [[8.0, 6.0, 2.0], [6.0, 7.0, 2.0], [8.0, 8.0, 5.0]], [[4.0, 7.0, 4.0], [2.0, 7.0, 6.0], 
[4.0, 7.0, 1.0]]], [[[5.0, 5.0, 6.0], [7.0, 6.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 3.0, 4.0], [5.0, 5.0, 7.0], [4.0, 5.0, 1.0]], 
[[6.0, 2.0, 5.0], [1.0, 2.0, 4.0], [1.0, 4.0, 1.0]]], [[[7.0, 2.0, 6.0], [4.0, 2.0, 2.0], [8.0, 8.0, 3.0]], [[7.0, 2.0, 7.0], 
[8.0, 6.0, 1.0], [6.0, 4.0, 6.0]], [[3.0, 5.0, 8.0], [2.0, 6.0, 1.0], [6.0, 7.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0]]], [[[4.0]]]])+(1.-msk_val)*numpy.array([[[[1.0]]], [[[4.0]]]])
      arg[2:,1:2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 4.0], [1.0, 7.0, 4.0], [1.0, 2.0, 8.0]], [[6.0, 3.0, 2.0], [5.0, 6.0, 2.0], [4.0, 
8.0, 8.0]], [[1.0, 2.0, 7.0], [4.0, 6.0, 3.0], [2.0, 2.0, 7.0]]], [[[7.0, 1.0, 4.0], [6.0, 2.0, 1.0], [1.0, 8.0, 3.0]], [[5.0, 
1.0, 3.0], [1.0, 8.0, 8.0], [1.0, 6.0, 4.0]], [[4.0, 5.0, 3.0], [8.0, 8.0, 2.0], [3.0, 2.0, 7.0]]], [[[4.0, 8.0, 3.0], [7.0, 
6.0, 3.0], [4.0, 1.0, 1.0]], [[6.0, 8.0, 6.0], [6.0, 6.0, 7.0], [5.0, 3.0, 2.0]], [[3.0, 6.0, 1.0], [2.0, 3.0, 2.0], [8.0, 5.0, 
3.0]]], [[[4.0, 1.0, 3.0], [5.0, 1.0, 6.0], [1.0, 1.0, 7.0]], [[6.0, 5.0, 6.0], [7.0, 1.0, 2.0], [5.0, 4.0, 6.0]], [[3.0, 7.0, 
7.0], [7.0, 6.0, 8.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 2.0], [7.0, 7.0, 1.0], [1.0, 2.0, 4.0]], 
[[2.0, 2.0, 6.0], [1.0, 2.0, 8.0], [1.0, 3.0, 2.0]], [[5.0, 4.0, 4.0], [3.0, 7.0, 6.0], [1.0, 7.0, 8.0]]], [[[8.0, 3.0, 4.0], 
[4.0, 4.0, 4.0], [3.0, 3.0, 6.0]], [[8.0, 6.0, 2.0], [6.0, 7.0, 2.0], [8.0, 8.0, 5.0]], [[4.0, 7.0, 4.0], [2.0, 7.0, 6.0], 
[4.0, 7.0, 1.0]]], [[[5.0, 5.0, 6.0], [7.0, 6.0, 6.0], [2.0, 3.0, 6.0]], [[6.0, 3.0, 4.0], [5.0, 5.0, 7.0], [4.0, 1.0, 1.0]], 
[[6.0, 2.0, 5.0], [1.0, 2.0, 4.0], [1.0, 4.0, 1.0]]], [[[7.0, 2.0, 6.0], [4.0, 2.0, 2.0], [8.0, 8.0, 3.0]], [[7.0, 2.0, 7.0], 
[8.0, 6.0, 1.0], [6.0, 4.0, 6.0]], [[3.0, 5.0, 8.0], [2.0, 6.0, 1.0], [6.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 7.0], [1.0, 8.0, 5.0], [3.0, 4.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 7.0, 3.0], [4.0, 
5.0, 3.0]], [[5.0, 1.0, 4.0], [2.0, 7.0, 7.0], [5.0, 6.0, 8.0]]], [[[1.0, 2.0, 8.0], [1.0, 8.0, 4.0], [6.0, 4.0, 2.0]], [[5.0, 
7.0, 8.0], [5.0, 3.0, 6.0], [3.0, 6.0, 5.0]], [[5.0, 3.0, 5.0], [2.0, 7.0, 7.0], [6.0, 8.0, 8.0]]], [[[7.0, 7.0, 5.0], [4.0, 
2.0, 7.0], [7.0, 2.0, 5.0]], [[3.0, 8.0, 6.0], [8.0, 7.0, 1.0], [1.0, 8.0, 1.0]], [[6.0, 2.0, 8.0], [2.0, 2.0, 7.0], [6.0, 6.0, 
6.0]]], [[[5.0, 8.0, 5.0], [7.0, 8.0, 6.0], [8.0, 3.0, 8.0]], [[2.0, 6.0, 1.0], [1.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[2.0, 5.0, 
5.0], [1.0, 7.0, 6.0], [2.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 6.0], [3.0, 1.0, 4.0], [3.0, 2.0, 3.0]], 
[[6.0, 1.0, 1.0], [7.0, 7.0, 6.0], [1.0, 7.0, 5.0]], [[7.0, 6.0, 4.0], [3.0, 6.0, 7.0], [7.0, 4.0, 7.0]]], [[[3.0, 7.0, 2.0], 
[2.0, 5.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 7.0, 7.0], [6.0, 7.0, 6.0], [4.0, 7.0, 5.0]], [[3.0, 5.0, 5.0], [7.0, 7.0, 6.0], 
[8.0, 1.0, 6.0]]], [[[1.0, 6.0, 8.0], [4.0, 5.0, 5.0], [8.0, 3.0, 5.0]], [[6.0, 2.0, 6.0], [5.0, 1.0, 2.0], [3.0, 2.0, 7.0]], 
[[4.0, 7.0, 8.0], [5.0, 2.0, 1.0], [2.0, 3.0, 2.0]]], [[[1.0, 2.0, 4.0], [2.0, 3.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 3.0, 7.0], 
[8.0, 5.0, 5.0], [5.0, 3.0, 6.0]], [[2.0, 3.0, 5.0], [1.0, 8.0, 6.0], [1.0, 3.0, 4.0]]]])
      val=numpy.array([[[4.0, 8.0, 6.0]], [[6.0, 6.0, 6.0]]])
      arg[1:3,1:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 7.0], [1.0, 8.0, 5.0], [3.0, 4.0, 7.0]], [[1.0, 7.0, 8.0], [1.0, 7.0, 3.0], [4.0, 
5.0, 3.0]], [[5.0, 1.0, 4.0], [2.0, 7.0, 7.0], [5.0, 6.0, 8.0]]], [[[1.0, 2.0, 8.0], [1.0, 8.0, 4.0], [6.0, 4.0, 2.0]], [[5.0, 
4.0, 8.0], [5.0, 8.0, 6.0], [3.0, 6.0, 5.0]], [[5.0, 3.0, 5.0], [2.0, 7.0, 7.0], [6.0, 8.0, 8.0]]], [[[7.0, 7.0, 5.0], [4.0, 
2.0, 7.0], [7.0, 2.0, 5.0]], [[3.0, 6.0, 6.0], [8.0, 6.0, 1.0], [1.0, 6.0, 1.0]], [[6.0, 2.0, 8.0], [2.0, 2.0, 7.0], [6.0, 6.0, 
6.0]]], [[[5.0, 8.0, 5.0], [7.0, 8.0, 6.0], [8.0, 3.0, 8.0]], [[2.0, 6.0, 1.0], [1.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[2.0, 5.0, 
5.0], [1.0, 7.0, 6.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 6.0], [3.0, 1.0, 4.0], [3.0, 2.0, 3.0]], 
[[6.0, 1.0, 1.0], [7.0, 7.0, 6.0], [1.0, 7.0, 5.0]], [[7.0, 6.0, 4.0], [3.0, 6.0, 7.0], [7.0, 4.0, 7.0]]], [[[3.0, 7.0, 2.0], 
[2.0, 5.0, 7.0], [5.0, 3.0, 5.0]], [[1.0, 4.0, 7.0], [6.0, 8.0, 6.0], [4.0, 6.0, 5.0]], [[3.0, 5.0, 5.0], [7.0, 7.0, 6.0], 
[8.0, 1.0, 6.0]]], [[[1.0, 6.0, 8.0], [4.0, 5.0, 5.0], [8.0, 3.0, 5.0]], [[6.0, 6.0, 6.0], [5.0, 6.0, 2.0], [3.0, 6.0, 7.0]], 
[[4.0, 7.0, 8.0], [5.0, 2.0, 1.0], [2.0, 3.0, 2.0]]], [[[1.0, 2.0, 4.0], [2.0, 3.0, 5.0], [7.0, 6.0, 4.0]], [[8.0, 3.0, 7.0], 
[8.0, 5.0, 5.0], [5.0, 3.0, 6.0]], [[2.0, 3.0, 5.0], [1.0, 8.0, 6.0], [1.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 8.0], [7.0, 2.0, 7.0], [2.0, 6.0, 4.0]], [[7.0, 2.0, 6.0], [1.0, 6.0, 7.0], [5.0, 
2.0, 3.0]], [[8.0, 6.0, 1.0], [3.0, 2.0, 2.0], [6.0, 1.0, 8.0]]], [[[4.0, 5.0, 7.0], [1.0, 4.0, 8.0], [6.0, 6.0, 2.0]], [[8.0, 
3.0, 1.0], [5.0, 2.0, 7.0], [8.0, 4.0, 6.0]], [[6.0, 2.0, 8.0], [1.0, 7.0, 1.0], [8.0, 5.0, 4.0]]], [[[5.0, 1.0, 2.0], [5.0, 
3.0, 1.0], [3.0, 6.0, 1.0]], [[4.0, 8.0, 5.0], [5.0, 6.0, 1.0], [1.0, 2.0, 5.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 6.0], [7.0, 2.0, 
5.0]]], [[[2.0, 7.0, 1.0], [5.0, 3.0, 7.0], [4.0, 5.0, 8.0]], [[6.0, 2.0, 1.0], [1.0, 4.0, 6.0], [8.0, 8.0, 4.0]], [[1.0, 7.0, 
4.0], [4.0, 4.0, 8.0], [7.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 3.0], [8.0, 6.0, 5.0], [1.0, 8.0, 7.0]], 
[[8.0, 5.0, 2.0], [7.0, 5.0, 1.0], [7.0, 7.0, 5.0]], [[4.0, 3.0, 3.0], [8.0, 1.0, 7.0], [3.0, 3.0, 2.0]]], [[[8.0, 3.0, 3.0], 
[8.0, 3.0, 5.0], [3.0, 7.0, 1.0]], [[3.0, 6.0, 4.0], [7.0, 1.0, 7.0], [8.0, 5.0, 8.0]], [[7.0, 8.0, 4.0], [3.0, 4.0, 6.0], 
[8.0, 4.0, 7.0]]], [[[4.0, 7.0, 5.0], [4.0, 4.0, 2.0], [3.0, 5.0, 2.0]], [[5.0, 2.0, 4.0], [7.0, 2.0, 2.0], [8.0, 1.0, 5.0]], 
[[6.0, 2.0, 8.0], [3.0, 2.0, 8.0], [5.0, 8.0, 1.0]]], [[[7.0, 8.0, 1.0], [4.0, 3.0, 5.0], [8.0, 1.0, 8.0]], [[3.0, 5.0, 1.0], 
[2.0, 6.0, 7.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 2.0], [5.0, 4.0, 6.0], [5.0, 3.0, 3.0]]]])
      val=Data(numpy.array([[[6.0, 2.0, 6.0]], [[6.0, 2.0, 6.0]]]),self.functionspace)
      arg[1:3,1:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 8.0], [7.0, 2.0, 7.0], [2.0, 6.0, 4.0]], [[7.0, 2.0, 6.0], [1.0, 6.0, 7.0], [5.0, 
2.0, 3.0]], [[8.0, 6.0, 1.0], [3.0, 2.0, 2.0], [6.0, 1.0, 8.0]]], [[[4.0, 5.0, 7.0], [1.0, 4.0, 8.0], [6.0, 6.0, 2.0]], [[8.0, 
6.0, 1.0], [5.0, 2.0, 7.0], [8.0, 6.0, 6.0]], [[6.0, 2.0, 8.0], [1.0, 7.0, 1.0], [8.0, 5.0, 4.0]]], [[[5.0, 1.0, 2.0], [5.0, 
3.0, 1.0], [3.0, 6.0, 1.0]], [[4.0, 6.0, 5.0], [5.0, 2.0, 1.0], [1.0, 6.0, 5.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 6.0], [7.0, 2.0, 
5.0]]], [[[2.0, 7.0, 1.0], [5.0, 3.0, 7.0], [4.0, 5.0, 8.0]], [[6.0, 2.0, 1.0], [1.0, 4.0, 6.0], [8.0, 8.0, 4.0]], [[1.0, 7.0, 
4.0], [4.0, 4.0, 8.0], [7.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 3.0], [8.0, 6.0, 5.0], [1.0, 8.0, 7.0]], 
[[8.0, 5.0, 2.0], [7.0, 5.0, 1.0], [7.0, 7.0, 5.0]], [[4.0, 3.0, 3.0], [8.0, 1.0, 7.0], [3.0, 3.0, 2.0]]], [[[8.0, 3.0, 3.0], 
[8.0, 3.0, 5.0], [3.0, 7.0, 1.0]], [[3.0, 6.0, 4.0], [7.0, 2.0, 7.0], [8.0, 6.0, 8.0]], [[7.0, 8.0, 4.0], [3.0, 4.0, 6.0], 
[8.0, 4.0, 7.0]]], [[[4.0, 7.0, 5.0], [4.0, 4.0, 2.0], [3.0, 5.0, 2.0]], [[5.0, 6.0, 4.0], [7.0, 2.0, 2.0], [8.0, 6.0, 5.0]], 
[[6.0, 2.0, 8.0], [3.0, 2.0, 8.0], [5.0, 8.0, 1.0]]], [[[7.0, 8.0, 1.0], [4.0, 3.0, 5.0], [8.0, 1.0, 8.0]], [[3.0, 5.0, 1.0], 
[2.0, 6.0, 7.0], [2.0, 3.0, 5.0]], [[4.0, 5.0, 2.0], [5.0, 4.0, 6.0], [5.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 7.0], [2.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[2.0, 6.0, 4.0], [1.0, 1.0, 2.0], [8.0, 
6.0, 4.0]], [[6.0, 8.0, 4.0], [1.0, 6.0, 8.0], [6.0, 6.0, 4.0]]], [[[5.0, 5.0, 3.0], [2.0, 7.0, 8.0], [1.0, 5.0, 5.0]], [[4.0, 
1.0, 5.0], [2.0, 2.0, 7.0], [5.0, 1.0, 4.0]], [[6.0, 4.0, 8.0], [3.0, 8.0, 6.0], [5.0, 5.0, 2.0]]], [[[6.0, 4.0, 5.0], [3.0, 
2.0, 2.0], [4.0, 4.0, 3.0]], [[1.0, 5.0, 3.0], [5.0, 3.0, 1.0], [8.0, 1.0, 4.0]], [[6.0, 4.0, 8.0], [2.0, 1.0, 5.0], [5.0, 7.0, 
4.0]]], [[[3.0, 4.0, 6.0], [5.0, 6.0, 5.0], [8.0, 8.0, 4.0]], [[8.0, 2.0, 8.0], [8.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[1.0, 2.0, 
3.0], [3.0, 2.0, 1.0], [4.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 1.0], [2.0, 8.0, 5.0], [1.0, 7.0, 1.0]], 
[[2.0, 8.0, 7.0], [2.0, 3.0, 7.0], [2.0, 6.0, 4.0]], [[7.0, 7.0, 4.0], [4.0, 1.0, 3.0], [1.0, 5.0, 7.0]]], [[[6.0, 1.0, 2.0], 
[3.0, 8.0, 2.0], [6.0, 8.0, 4.0]], [[7.0, 1.0, 5.0], [3.0, 1.0, 6.0], [5.0, 4.0, 6.0]], [[2.0, 2.0, 4.0], [6.0, 8.0, 4.0], 
[2.0, 7.0, 2.0]]], [[[4.0, 2.0, 6.0], [4.0, 5.0, 6.0], [6.0, 2.0, 5.0]], [[5.0, 1.0, 8.0], [3.0, 2.0, 2.0], [2.0, 1.0, 4.0]], 
[[8.0, 5.0, 3.0], [5.0, 7.0, 5.0], [3.0, 7.0, 6.0]]], [[[8.0, 6.0, 5.0], [6.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[8.0, 4.0, 3.0], 
[7.0, 1.0, 2.0], [5.0, 7.0, 1.0]], [[5.0, 5.0, 2.0], [2.0, 7.0, 5.0], [8.0, 4.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 3.0, 1.0]], [[1.0, 5.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 2.0, 6.0]], [[8.0, 6.0, 
3.0]]])
      arg[1:3,1:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 7.0], [2.0, 2.0, 7.0], [2.0, 2.0, 1.0]], [[2.0, 6.0, 4.0], [1.0, 1.0, 2.0], [8.0, 
6.0, 4.0]], [[6.0, 8.0, 4.0], [1.0, 6.0, 8.0], [6.0, 6.0, 4.0]]], [[[5.0, 5.0, 3.0], [2.0, 7.0, 8.0], [1.0, 5.0, 5.0]], [[4.0, 
1.0, 5.0], [2.0, 3.0, 7.0], [5.0, 1.0, 4.0]], [[6.0, 4.0, 8.0], [3.0, 8.0, 6.0], [5.0, 5.0, 2.0]]], [[[6.0, 4.0, 5.0], [3.0, 
2.0, 2.0], [4.0, 4.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 5.0, 1.0], [8.0, 5.0, 4.0]], [[6.0, 4.0, 8.0], [2.0, 1.0, 5.0], [5.0, 7.0, 
4.0]]], [[[3.0, 4.0, 6.0], [5.0, 6.0, 5.0], [8.0, 8.0, 4.0]], [[8.0, 2.0, 8.0], [8.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[1.0, 2.0, 
3.0], [3.0, 2.0, 1.0], [4.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 1.0], [2.0, 8.0, 5.0], [1.0, 7.0, 1.0]], 
[[2.0, 8.0, 7.0], [2.0, 3.0, 7.0], [2.0, 6.0, 4.0]], [[7.0, 7.0, 4.0], [4.0, 1.0, 3.0], [1.0, 5.0, 7.0]]], [[[6.0, 1.0, 2.0], 
[3.0, 8.0, 2.0], [6.0, 8.0, 4.0]], [[7.0, 8.0, 5.0], [3.0, 2.0, 6.0], [5.0, 6.0, 6.0]], [[2.0, 2.0, 4.0], [6.0, 8.0, 4.0], 
[2.0, 7.0, 2.0]]], [[[4.0, 2.0, 6.0], [4.0, 5.0, 6.0], [6.0, 2.0, 5.0]], [[5.0, 8.0, 8.0], [3.0, 6.0, 2.0], [2.0, 3.0, 4.0]], 
[[8.0, 5.0, 3.0], [5.0, 7.0, 5.0], [3.0, 7.0, 6.0]]], [[[8.0, 6.0, 5.0], [6.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[8.0, 4.0, 3.0], 
[7.0, 1.0, 2.0], [5.0, 7.0, 1.0]], [[5.0, 5.0, 2.0], [2.0, 7.0, 5.0], [8.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CUCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 1.0], [6.0, 1.0, 6.0], [2.0, 7.0, 2.0]], [[6.0, 4.0, 5.0], [1.0, 6.0, 3.0], [2.0, 
3.0, 1.0]], [[4.0, 4.0, 6.0], [4.0, 2.0, 4.0], [2.0, 4.0, 8.0]]], [[[5.0, 6.0, 3.0], [2.0, 4.0, 4.0], [6.0, 2.0, 5.0]], [[7.0, 
7.0, 3.0], [8.0, 3.0, 6.0], [4.0, 7.0, 8.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 4.0], [6.0, 1.0, 1.0]]], [[[2.0, 8.0, 8.0], [5.0, 
3.0, 5.0], [7.0, 2.0, 5.0]], [[7.0, 2.0, 4.0], [3.0, 5.0, 5.0], [4.0, 6.0, 6.0]], [[5.0, 7.0, 8.0], [2.0, 5.0, 5.0], [8.0, 6.0, 
2.0]]], [[[5.0, 3.0, 3.0], [2.0, 6.0, 7.0], [6.0, 4.0, 2.0]], [[5.0, 3.0, 8.0], [7.0, 3.0, 1.0], [4.0, 3.0, 3.0]], [[1.0, 6.0, 
5.0], [2.0, 1.0, 6.0], [5.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 1.0], [7.0, 3.0, 7.0], [3.0, 8.0, 1.0]], 
[[2.0, 1.0, 5.0], [7.0, 8.0, 5.0], [8.0, 3.0, 3.0]], [[7.0, 4.0, 5.0], [3.0, 8.0, 5.0], [3.0, 8.0, 7.0]]], [[[7.0, 8.0, 7.0], 
[7.0, 7.0, 8.0], [7.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [6.0, 2.0, 4.0], [2.0, 6.0, 4.0]], [[1.0, 7.0, 1.0], [2.0, 8.0, 4.0], 
[4.0, 7.0, 5.0]]], [[[7.0, 7.0, 3.0], [1.0, 7.0, 7.0], [6.0, 6.0, 4.0]], [[2.0, 1.0, 8.0], [8.0, 3.0, 6.0], [2.0, 1.0, 1.0]], 
[[1.0, 1.0, 4.0], [7.0, 3.0, 4.0], [5.0, 2.0, 6.0]]], [[[5.0, 2.0, 3.0], [2.0, 8.0, 6.0], [7.0, 6.0, 1.0]], [[2.0, 6.0, 5.0], 
[7.0, 6.0, 4.0], [3.0, 3.0, 8.0]], [[3.0, 5.0, 8.0], [1.0, 4.0, 8.0], [3.0, 6.0, 1.0]]]])
      val=numpy.array([[[6.0, 4.0, 5.0], [7.0, 5.0, 5.0]], [[3.0, 4.0, 2.0], [8.0, 2.0, 2.0]], [[8.0, 7.0, 1.0], [4.0, 2.0, 
4.0]], [[6.0, 5.0, 6.0], [8.0, 1.0, 1.0]]])
      arg[:,:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 1.0], [6.0, 4.0, 6.0], [2.0, 5.0, 2.0]], [[6.0, 7.0, 5.0], [1.0, 5.0, 3.0], [2.0, 
5.0, 1.0]], [[4.0, 4.0, 6.0], [4.0, 2.0, 4.0], [2.0, 4.0, 8.0]]], [[[5.0, 3.0, 3.0], [2.0, 4.0, 4.0], [6.0, 2.0, 5.0]], [[7.0, 
8.0, 3.0], [8.0, 2.0, 6.0], [4.0, 2.0, 8.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 4.0], [6.0, 1.0, 1.0]]], [[[2.0, 8.0, 8.0], [5.0, 
7.0, 5.0], [7.0, 1.0, 5.0]], [[7.0, 4.0, 4.0], [3.0, 2.0, 5.0], [4.0, 4.0, 6.0]], [[5.0, 7.0, 8.0], [2.0, 5.0, 5.0], [8.0, 6.0, 
2.0]]], [[[5.0, 6.0, 3.0], [2.0, 5.0, 7.0], [6.0, 6.0, 2.0]], [[5.0, 8.0, 8.0], [7.0, 1.0, 1.0], [4.0, 1.0, 3.0]], [[1.0, 6.0, 
5.0], [2.0, 1.0, 6.0], [5.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 1.0], [7.0, 4.0, 7.0], [3.0, 5.0, 1.0]], 
[[2.0, 7.0, 5.0], [7.0, 5.0, 5.0], [8.0, 5.0, 3.0]], [[7.0, 4.0, 5.0], [3.0, 8.0, 5.0], [3.0, 8.0, 7.0]]], [[[7.0, 3.0, 7.0], 
[7.0, 4.0, 8.0], [7.0, 2.0, 6.0]], [[7.0, 8.0, 1.0], [6.0, 2.0, 4.0], [2.0, 2.0, 4.0]], [[1.0, 7.0, 1.0], [2.0, 8.0, 4.0], 
[4.0, 7.0, 5.0]]], [[[7.0, 8.0, 3.0], [1.0, 7.0, 7.0], [6.0, 1.0, 4.0]], [[2.0, 4.0, 8.0], [8.0, 2.0, 6.0], [2.0, 4.0, 1.0]], 
[[1.0, 1.0, 4.0], [7.0, 3.0, 4.0], [5.0, 2.0, 6.0]]], [[[5.0, 6.0, 3.0], [2.0, 5.0, 6.0], [7.0, 6.0, 1.0]], [[2.0, 8.0, 5.0], 
[7.0, 1.0, 4.0], [3.0, 1.0, 8.0]], [[3.0, 5.0, 8.0], [1.0, 4.0, 8.0], [3.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CUCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 7.0], [7.0, 1.0, 3.0], [1.0, 8.0, 4.0]], [[7.0, 6.0, 6.0], [1.0, 2.0, 3.0], [7.0, 
6.0, 1.0]], [[8.0, 3.0, 5.0], [8.0, 2.0, 4.0], [4.0, 4.0, 7.0]]], [[[8.0, 5.0, 1.0], [1.0, 8.0, 4.0], [5.0, 8.0, 3.0]], [[8.0, 
6.0, 6.0], [5.0, 3.0, 5.0], [8.0, 6.0, 3.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 6.0], [8.0, 4.0, 3.0]]], [[[1.0, 3.0, 5.0], [5.0, 
4.0, 8.0], [5.0, 1.0, 5.0]], [[3.0, 6.0, 2.0], [5.0, 6.0, 6.0], [6.0, 4.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 3.0, 3.0], [4.0, 2.0, 
4.0]]], [[[6.0, 4.0, 8.0], [2.0, 5.0, 1.0], [4.0, 4.0, 3.0]], [[4.0, 6.0, 5.0], [8.0, 5.0, 3.0], [8.0, 2.0, 4.0]], [[5.0, 1.0, 
6.0], [5.0, 4.0, 6.0], [8.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 4.0], [5.0, 5.0, 6.0], [4.0, 4.0, 8.0]], 
[[2.0, 2.0, 8.0], [1.0, 4.0, 4.0], [7.0, 7.0, 8.0]], [[6.0, 4.0, 6.0], [7.0, 6.0, 4.0], [2.0, 6.0, 8.0]]], [[[7.0, 3.0, 8.0], 
[4.0, 2.0, 4.0], [8.0, 7.0, 2.0]], [[6.0, 3.0, 2.0], [8.0, 5.0, 4.0], [5.0, 8.0, 7.0]], [[4.0, 5.0, 1.0], [7.0, 4.0, 7.0], 
[5.0, 5.0, 3.0]]], [[[4.0, 7.0, 1.0], [3.0, 1.0, 7.0], [4.0, 5.0, 7.0]], [[2.0, 8.0, 2.0], [3.0, 8.0, 5.0], [4.0, 5.0, 8.0]], 
[[1.0, 4.0, 3.0], [4.0, 8.0, 3.0], [2.0, 5.0, 6.0]]], [[[8.0, 6.0, 8.0], [8.0, 4.0, 5.0], [5.0, 5.0, 1.0]], [[3.0, 6.0, 2.0], 
[7.0, 4.0, 2.0], [6.0, 8.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 4.0, 3.0], [6.0, 3.0, 6.0]]]])
      val=Data(numpy.array([[[5.0, 8.0, 2.0], [5.0, 8.0, 2.0]], [[6.0, 4.0, 3.0], [7.0, 2.0, 4.0]], [[2.0, 8.0, 1.0], [5.0, 
1.0, 3.0]], [[2.0, 2.0, 6.0], [3.0, 2.0, 1.0]]]),self.functionspace)
      arg[:,:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 7.0], [7.0, 8.0, 3.0], [1.0, 2.0, 4.0]], [[7.0, 5.0, 6.0], [1.0, 8.0, 3.0], [7.0, 
2.0, 1.0]], [[8.0, 3.0, 5.0], [8.0, 2.0, 4.0], [4.0, 4.0, 7.0]]], [[[8.0, 6.0, 1.0], [1.0, 4.0, 4.0], [5.0, 3.0, 3.0]], [[8.0, 
7.0, 6.0], [5.0, 2.0, 5.0], [8.0, 4.0, 3.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 6.0], [8.0, 4.0, 3.0]]], [[[1.0, 2.0, 5.0], [5.0, 
8.0, 8.0], [5.0, 1.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 1.0, 6.0], [6.0, 3.0, 2.0]], [[5.0, 2.0, 4.0], [8.0, 3.0, 3.0], [4.0, 2.0, 
4.0]]], [[[6.0, 2.0, 8.0], [2.0, 2.0, 1.0], [4.0, 6.0, 3.0]], [[4.0, 3.0, 5.0], [8.0, 2.0, 3.0], [8.0, 1.0, 4.0]], [[5.0, 1.0, 
6.0], [5.0, 4.0, 6.0], [8.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 8.0, 6.0], [4.0, 2.0, 8.0]], 
[[2.0, 5.0, 8.0], [1.0, 8.0, 4.0], [7.0, 2.0, 8.0]], [[6.0, 4.0, 6.0], [7.0, 6.0, 4.0], [2.0, 6.0, 8.0]]], [[[7.0, 6.0, 8.0], 
[4.0, 4.0, 4.0], [8.0, 3.0, 2.0]], [[6.0, 7.0, 2.0], [8.0, 2.0, 4.0], [5.0, 4.0, 7.0]], [[4.0, 5.0, 1.0], [7.0, 4.0, 7.0], 
[5.0, 5.0, 3.0]]], [[[4.0, 2.0, 1.0], [3.0, 8.0, 7.0], [4.0, 1.0, 7.0]], [[2.0, 5.0, 2.0], [3.0, 1.0, 5.0], [4.0, 3.0, 8.0]], 
[[1.0, 4.0, 3.0], [4.0, 8.0, 3.0], [2.0, 5.0, 6.0]]], [[[8.0, 2.0, 8.0], [8.0, 2.0, 5.0], [5.0, 6.0, 1.0]], [[3.0, 3.0, 2.0], 
[7.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 4.0, 3.0], [6.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CUCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 6.0], [2.0, 3.0, 6.0], [8.0, 1.0, 8.0]], [[1.0, 4.0, 3.0], [6.0, 1.0, 2.0], [3.0, 
7.0, 8.0]], [[5.0, 3.0, 4.0], [3.0, 4.0, 6.0], [1.0, 6.0, 2.0]]], [[[8.0, 3.0, 8.0], [8.0, 7.0, 1.0], [7.0, 3.0, 8.0]], [[1.0, 
2.0, 6.0], [4.0, 4.0, 7.0], [3.0, 3.0, 5.0]], [[2.0, 4.0, 4.0], [5.0, 1.0, 5.0], [2.0, 5.0, 8.0]]], [[[7.0, 1.0, 2.0], [2.0, 
6.0, 3.0], [8.0, 5.0, 7.0]], [[6.0, 3.0, 1.0], [7.0, 8.0, 1.0], [1.0, 6.0, 2.0]], [[5.0, 6.0, 4.0], [8.0, 7.0, 7.0], [3.0, 1.0, 
4.0]]], [[[7.0, 3.0, 4.0], [8.0, 4.0, 1.0], [1.0, 3.0, 8.0]], [[4.0, 1.0, 8.0], [2.0, 1.0, 5.0], [4.0, 7.0, 8.0]], [[6.0, 1.0, 
2.0], [7.0, 7.0, 7.0], [1.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 5.0], [8.0, 5.0, 5.0], [3.0, 5.0, 4.0]], 
[[1.0, 4.0, 4.0], [6.0, 6.0, 4.0], [3.0, 5.0, 1.0]], [[7.0, 6.0, 5.0], [3.0, 1.0, 4.0], [6.0, 3.0, 8.0]]], [[[5.0, 1.0, 8.0], 
[4.0, 5.0, 6.0], [4.0, 3.0, 5.0]], [[1.0, 8.0, 5.0], [2.0, 2.0, 7.0], [4.0, 8.0, 6.0]], [[5.0, 4.0, 2.0], [3.0, 1.0, 5.0], 
[4.0, 6.0, 7.0]]], [[[3.0, 1.0, 7.0], [4.0, 7.0, 2.0], [5.0, 3.0, 8.0]], [[4.0, 4.0, 6.0], [7.0, 8.0, 2.0], [2.0, 5.0, 5.0]], 
[[2.0, 7.0, 4.0], [7.0, 7.0, 6.0], [7.0, 5.0, 2.0]]], [[[7.0, 6.0, 7.0], [4.0, 8.0, 6.0], [8.0, 3.0, 4.0]], [[5.0, 5.0, 8.0], 
[8.0, 3.0, 8.0], [5.0, 2.0, 4.0]], [[8.0, 1.0, 4.0], [2.0, 1.0, 8.0], [2.0, 4.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 6.0, 2.0], [5.0, 3.0, 6.0]], [[2.0, 2.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 1.0, 1.0], [8.0, 
6.0, 1.0]], [[6.0, 3.0, 7.0], [6.0, 1.0, 4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 1.0], [5.0, 8.0, 1.0]], [[8.0, 7.0, 
3.0], [3.0, 3.0, 4.0]], [[6.0, 4.0, 5.0], [4.0, 3.0, 4.0]], [[1.0, 3.0, 1.0], [3.0, 2.0, 8.0]]])
      arg[:,:2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [2.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[1.0, 5.0, 3.0], [6.0, 3.0, 2.0], [3.0, 
6.0, 8.0]], [[5.0, 3.0, 4.0], [3.0, 4.0, 6.0], [1.0, 6.0, 2.0]]], [[[8.0, 2.0, 8.0], [8.0, 2.0, 1.0], [7.0, 7.0, 8.0]], [[1.0, 
4.0, 6.0], [4.0, 1.0, 7.0], [3.0, 4.0, 5.0]], [[2.0, 4.0, 4.0], [5.0, 1.0, 5.0], [2.0, 5.0, 8.0]]], [[[7.0, 3.0, 2.0], [2.0, 
1.0, 3.0], [8.0, 1.0, 7.0]], [[6.0, 8.0, 1.0], [7.0, 6.0, 1.0], [1.0, 1.0, 2.0]], [[5.0, 6.0, 4.0], [8.0, 7.0, 7.0], [3.0, 1.0, 
4.0]]], [[[7.0, 6.0, 4.0], [8.0, 3.0, 1.0], [1.0, 7.0, 8.0]], [[4.0, 6.0, 8.0], [2.0, 1.0, 5.0], [4.0, 4.0, 8.0]], [[6.0, 1.0, 
2.0], [7.0, 7.0, 7.0], [1.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 5.0], [8.0, 4.0, 5.0], [3.0, 1.0, 4.0]], 
[[1.0, 5.0, 4.0], [6.0, 8.0, 4.0], [3.0, 1.0, 1.0]], [[7.0, 6.0, 5.0], [3.0, 1.0, 4.0], [6.0, 3.0, 8.0]]], [[[5.0, 8.0, 8.0], 
[4.0, 7.0, 6.0], [4.0, 3.0, 5.0]], [[1.0, 3.0, 5.0], [2.0, 3.0, 7.0], [4.0, 4.0, 6.0]], [[5.0, 4.0, 2.0], [3.0, 1.0, 5.0], 
[4.0, 6.0, 7.0]]], [[[3.0, 6.0, 7.0], [4.0, 4.0, 2.0], [5.0, 5.0, 8.0]], [[4.0, 4.0, 6.0], [7.0, 3.0, 2.0], [2.0, 4.0, 5.0]], 
[[2.0, 7.0, 4.0], [7.0, 7.0, 6.0], [7.0, 5.0, 2.0]]], [[[7.0, 1.0, 7.0], [4.0, 3.0, 6.0], [8.0, 1.0, 4.0]], [[5.0, 3.0, 8.0], 
[8.0, 2.0, 8.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 4.0], [2.0, 1.0, 8.0], [2.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 3.0], [3.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[5.0, 7.0, 4.0], [7.0, 7.0, 7.0], [8.0, 
5.0, 4.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 3.0], [3.0, 1.0, 6.0]]], [[[4.0, 2.0, 3.0], [2.0, 4.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 
5.0, 5.0], [3.0, 1.0, 5.0], [6.0, 7.0, 5.0]], [[8.0, 2.0, 1.0], [3.0, 3.0, 8.0], [3.0, 5.0, 2.0]]], [[[7.0, 1.0, 2.0], [1.0, 
6.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 3.0], [7.0, 1.0, 4.0]], [[2.0, 7.0, 8.0], [3.0, 7.0, 7.0], [1.0, 5.0, 
1.0]]], [[[2.0, 3.0, 3.0], [6.0, 1.0, 8.0], [3.0, 4.0, 6.0]], [[6.0, 6.0, 2.0], [8.0, 1.0, 5.0], [1.0, 6.0, 8.0]], [[7.0, 7.0, 
1.0], [2.0, 2.0, 2.0], [7.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 5.0], [6.0, 6.0, 5.0], [8.0, 5.0, 1.0]], 
[[7.0, 1.0, 4.0], [8.0, 6.0, 8.0], [1.0, 2.0, 8.0]], [[4.0, 4.0, 3.0], [7.0, 6.0, 5.0], [4.0, 8.0, 8.0]]], [[[8.0, 8.0, 3.0], 
[6.0, 7.0, 3.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 7.0], [1.0, 2.0, 1.0], [1.0, 1.0, 6.0]], [[2.0, 5.0, 7.0], [2.0, 1.0, 5.0], 
[3.0, 6.0, 7.0]]], [[[7.0, 1.0, 5.0], [3.0, 6.0, 3.0], [3.0, 3.0, 3.0]], [[4.0, 4.0, 3.0], [3.0, 3.0, 1.0], [3.0, 4.0, 3.0]], 
[[6.0, 2.0, 2.0], [6.0, 8.0, 5.0], [8.0, 7.0, 7.0]]], [[[7.0, 4.0, 3.0], [1.0, 5.0, 5.0], [6.0, 5.0, 2.0]], [[6.0, 4.0, 4.0], 
[2.0, 1.0, 4.0], [1.0, 5.0, 2.0]], [[3.0, 4.0, 6.0], [4.0, 2.0, 4.0], [3.0, 3.0, 2.0]]]])
      val=numpy.array([[[[2.0, 1.0], [7.0, 8.0], [3.0, 5.0]]], [[[2.0, 5.0], [8.0, 6.0], [6.0, 4.0]]], [[[8.0, 8.0], [5.0, 
7.0], [3.0, 7.0]]], [[[2.0, 1.0], [1.0, 5.0], [1.0, 2.0]]]])
      arg[:,2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 3.0], [3.0, 6.0, 7.0], [7.0, 2.0, 8.0]], [[5.0, 7.0, 4.0], [7.0, 7.0, 7.0], [8.0, 
5.0, 4.0]], [[2.0, 1.0, 7.0], [7.0, 8.0, 3.0], [3.0, 5.0, 6.0]]], [[[4.0, 2.0, 3.0], [2.0, 4.0, 3.0], [2.0, 3.0, 1.0]], [[2.0, 
5.0, 5.0], [3.0, 1.0, 5.0], [6.0, 7.0, 5.0]], [[2.0, 5.0, 1.0], [8.0, 6.0, 8.0], [6.0, 4.0, 2.0]]], [[[7.0, 1.0, 2.0], [1.0, 
6.0, 5.0], [6.0, 8.0, 2.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 3.0], [7.0, 1.0, 4.0]], [[8.0, 8.0, 8.0], [5.0, 7.0, 7.0], [3.0, 7.0, 
1.0]]], [[[2.0, 3.0, 3.0], [6.0, 1.0, 8.0], [3.0, 4.0, 6.0]], [[6.0, 6.0, 2.0], [8.0, 1.0, 5.0], [1.0, 6.0, 8.0]], [[2.0, 1.0, 
1.0], [1.0, 5.0, 2.0], [1.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 5.0], [6.0, 6.0, 5.0], [8.0, 5.0, 1.0]], 
[[7.0, 1.0, 4.0], [8.0, 6.0, 8.0], [1.0, 2.0, 8.0]], [[2.0, 1.0, 3.0], [7.0, 8.0, 5.0], [3.0, 5.0, 8.0]]], [[[8.0, 8.0, 3.0], 
[6.0, 7.0, 3.0], [4.0, 3.0, 2.0]], [[3.0, 7.0, 7.0], [1.0, 2.0, 1.0], [1.0, 1.0, 6.0]], [[2.0, 5.0, 7.0], [8.0, 6.0, 5.0], 
[6.0, 4.0, 7.0]]], [[[7.0, 1.0, 5.0], [3.0, 6.0, 3.0], [3.0, 3.0, 3.0]], [[4.0, 4.0, 3.0], [3.0, 3.0, 1.0], [3.0, 4.0, 3.0]], 
[[8.0, 8.0, 2.0], [5.0, 7.0, 5.0], [3.0, 7.0, 7.0]]], [[[7.0, 4.0, 3.0], [1.0, 5.0, 5.0], [6.0, 5.0, 2.0]], [[6.0, 4.0, 4.0], 
[2.0, 1.0, 4.0], [1.0, 5.0, 2.0]], [[2.0, 1.0, 6.0], [1.0, 5.0, 4.0], [1.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 3.0], [8.0, 2.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 3.0, 5.0], [8.0, 5.0, 5.0], [2.0, 
3.0, 4.0]], [[1.0, 4.0, 4.0], [4.0, 4.0, 7.0], [7.0, 2.0, 1.0]]], [[[3.0, 3.0, 3.0], [3.0, 3.0, 3.0], [1.0, 6.0, 7.0]], [[8.0, 
6.0, 2.0], [2.0, 5.0, 5.0], [8.0, 8.0, 5.0]], [[5.0, 7.0, 7.0], [8.0, 5.0, 6.0], [8.0, 1.0, 3.0]]], [[[3.0, 8.0, 3.0], [2.0, 
7.0, 3.0], [5.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [6.0, 4.0, 3.0], [5.0, 1.0, 4.0]], [[5.0, 3.0, 1.0], [7.0, 6.0, 8.0], [7.0, 7.0, 
7.0]]], [[[1.0, 4.0, 3.0], [2.0, 1.0, 2.0], [4.0, 4.0, 3.0]], [[5.0, 4.0, 5.0], [7.0, 5.0, 7.0], [6.0, 8.0, 1.0]], [[2.0, 3.0, 
8.0], [6.0, 4.0, 5.0], [2.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 7.0], [1.0, 4.0, 8.0], [3.0, 1.0, 1.0]], 
[[7.0, 4.0, 2.0], [2.0, 6.0, 8.0], [2.0, 3.0, 6.0]], [[8.0, 5.0, 8.0], [6.0, 8.0, 8.0], [3.0, 7.0, 4.0]]], [[[3.0, 7.0, 4.0], 
[2.0, 1.0, 5.0], [6.0, 5.0, 3.0]], [[1.0, 8.0, 3.0], [1.0, 4.0, 5.0], [4.0, 4.0, 5.0]], [[4.0, 3.0, 1.0], [6.0, 6.0, 2.0], 
[4.0, 7.0, 7.0]]], [[[7.0, 8.0, 5.0], [4.0, 2.0, 6.0], [8.0, 8.0, 2.0]], [[2.0, 3.0, 2.0], [3.0, 4.0, 6.0], [2.0, 1.0, 7.0]], 
[[5.0, 5.0, 3.0], [2.0, 5.0, 4.0], [1.0, 4.0, 7.0]]], [[[3.0, 2.0, 6.0], [6.0, 1.0, 3.0], [1.0, 3.0, 5.0]], [[2.0, 3.0, 6.0], 
[2.0, 8.0, 8.0], [5.0, 4.0, 5.0]], [[2.0, 5.0, 2.0], [5.0, 7.0, 8.0], [2.0, 6.0, 1.0]]]])
      val=Data(numpy.array([[[[2.0, 8.0], [5.0, 6.0], [7.0, 6.0]]], [[[8.0, 5.0], [6.0, 8.0], [4.0, 2.0]]], [[[1.0, 6.0], [3.0, 
3.0], [4.0, 6.0]]], [[[8.0, 5.0], [4.0, 5.0], [1.0, 4.0]]]]),self.functionspace)
      arg[:,2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 3.0], [8.0, 2.0, 1.0], [4.0, 1.0, 1.0]], [[4.0, 3.0, 5.0], [8.0, 5.0, 5.0], [2.0, 
3.0, 4.0]], [[2.0, 8.0, 4.0], [5.0, 6.0, 7.0], [7.0, 6.0, 1.0]]], [[[3.0, 3.0, 3.0], [3.0, 3.0, 3.0], [1.0, 6.0, 7.0]], [[8.0, 
6.0, 2.0], [2.0, 5.0, 5.0], [8.0, 8.0, 5.0]], [[8.0, 5.0, 7.0], [6.0, 8.0, 6.0], [4.0, 2.0, 3.0]]], [[[3.0, 8.0, 3.0], [2.0, 
7.0, 3.0], [5.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [6.0, 4.0, 3.0], [5.0, 1.0, 4.0]], [[1.0, 6.0, 1.0], [3.0, 3.0, 8.0], [4.0, 6.0, 
7.0]]], [[[1.0, 4.0, 3.0], [2.0, 1.0, 2.0], [4.0, 4.0, 3.0]], [[5.0, 4.0, 5.0], [7.0, 5.0, 7.0], [6.0, 8.0, 1.0]], [[8.0, 5.0, 
8.0], [4.0, 5.0, 5.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 7.0], [1.0, 4.0, 8.0], [3.0, 1.0, 1.0]], 
[[7.0, 4.0, 2.0], [2.0, 6.0, 8.0], [2.0, 3.0, 6.0]], [[2.0, 8.0, 8.0], [5.0, 6.0, 8.0], [7.0, 6.0, 4.0]]], [[[3.0, 7.0, 4.0], 
[2.0, 1.0, 5.0], [6.0, 5.0, 3.0]], [[1.0, 8.0, 3.0], [1.0, 4.0, 5.0], [4.0, 4.0, 5.0]], [[8.0, 5.0, 1.0], [6.0, 8.0, 2.0], 
[4.0, 2.0, 7.0]]], [[[7.0, 8.0, 5.0], [4.0, 2.0, 6.0], [8.0, 8.0, 2.0]], [[2.0, 3.0, 2.0], [3.0, 4.0, 6.0], [2.0, 1.0, 7.0]], 
[[1.0, 6.0, 3.0], [3.0, 3.0, 4.0], [4.0, 6.0, 7.0]]], [[[3.0, 2.0, 6.0], [6.0, 1.0, 3.0], [1.0, 3.0, 5.0]], [[2.0, 3.0, 6.0], 
[2.0, 8.0, 8.0], [5.0, 4.0, 5.0]], [[8.0, 5.0, 2.0], [4.0, 5.0, 8.0], [1.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLCU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 8.0], [2.0, 4.0, 5.0], [6.0, 2.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 8.0], [7.0, 
8.0, 2.0]], [[2.0, 6.0, 4.0], [7.0, 6.0, 5.0], [6.0, 6.0, 7.0]]], [[[7.0, 8.0, 2.0], [3.0, 3.0, 3.0], [8.0, 6.0, 2.0]], [[6.0, 
2.0, 1.0], [7.0, 5.0, 4.0], [8.0, 1.0, 7.0]], [[3.0, 4.0, 6.0], [1.0, 2.0, 3.0], [6.0, 8.0, 6.0]]], [[[2.0, 8.0, 4.0], [2.0, 
5.0, 7.0], [6.0, 7.0, 8.0]], [[8.0, 8.0, 6.0], [3.0, 7.0, 2.0], [6.0, 3.0, 8.0]], [[2.0, 1.0, 6.0], [2.0, 7.0, 3.0], [5.0, 7.0, 
6.0]]], [[[3.0, 2.0, 5.0], [5.0, 5.0, 6.0], [3.0, 1.0, 3.0]], [[3.0, 2.0, 4.0], [2.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[3.0, 7.0, 
6.0], [4.0, 2.0, 8.0], [4.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 1.0], [2.0, 7.0, 7.0], [2.0, 5.0, 8.0]], 
[[3.0, 6.0, 3.0], [7.0, 1.0, 2.0], [4.0, 3.0, 1.0]], [[5.0, 6.0, 2.0], [1.0, 2.0, 7.0], [7.0, 7.0, 3.0]]], [[[1.0, 7.0, 8.0], 
[2.0, 8.0, 4.0], [7.0, 2.0, 4.0]], [[2.0, 7.0, 8.0], [4.0, 4.0, 6.0], [3.0, 6.0, 6.0]], [[3.0, 4.0, 7.0], [6.0, 8.0, 5.0], 
[6.0, 2.0, 8.0]]], [[[3.0, 1.0, 3.0], [3.0, 4.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 7.0], [5.0, 5.0, 4.0]], 
[[1.0, 1.0, 6.0], [7.0, 2.0, 7.0], [3.0, 5.0, 8.0]]], [[[6.0, 2.0, 5.0], [2.0, 6.0, 5.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 4.0], 
[6.0, 2.0, 3.0], [8.0, 8.0, 6.0]], [[7.0, 2.0, 7.0], [7.0, 8.0, 4.0], [1.0, 8.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 4.0], [7.0, 3.0], [1.0, 7.0]]], [[[8.0, 7.0], [3.0, 4.0], [6.0, 3.0]]], [[[4.0, 5.0], 
[5.0, 8.0], [1.0, 5.0]]], [[[8.0, 4.0], [5.0, 6.0], [7.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 1.0], [7.0, 3.0], [8.0, 
6.0]]], [[[3.0, 4.0], [7.0, 4.0], [5.0, 1.0]]], [[[8.0, 6.0], [7.0, 8.0], [4.0, 1.0]]], [[[5.0, 3.0], [8.0, 2.0], [6.0, 4.0]]]])
      arg[:,2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 8.0], [2.0, 4.0, 5.0], [6.0, 2.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 8.0], [7.0, 
8.0, 2.0]], [[2.0, 4.0, 4.0], [7.0, 3.0, 5.0], [1.0, 7.0, 7.0]]], [[[7.0, 8.0, 2.0], [3.0, 3.0, 3.0], [8.0, 6.0, 2.0]], [[6.0, 
2.0, 1.0], [7.0, 5.0, 4.0], [8.0, 1.0, 7.0]], [[8.0, 7.0, 6.0], [3.0, 4.0, 3.0], [6.0, 3.0, 6.0]]], [[[2.0, 8.0, 4.0], [2.0, 
5.0, 7.0], [6.0, 7.0, 8.0]], [[8.0, 8.0, 6.0], [3.0, 7.0, 2.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 6.0], [5.0, 8.0, 3.0], [1.0, 5.0, 
6.0]]], [[[3.0, 2.0, 5.0], [5.0, 5.0, 6.0], [3.0, 1.0, 3.0]], [[3.0, 2.0, 4.0], [2.0, 3.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 4.0, 
6.0], [5.0, 6.0, 8.0], [7.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 1.0], [2.0, 7.0, 7.0], [2.0, 5.0, 8.0]], 
[[3.0, 6.0, 3.0], [7.0, 1.0, 2.0], [4.0, 3.0, 1.0]], [[7.0, 1.0, 2.0], [7.0, 3.0, 7.0], [8.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], 
[2.0, 8.0, 4.0], [7.0, 2.0, 4.0]], [[2.0, 7.0, 8.0], [4.0, 4.0, 6.0], [3.0, 6.0, 6.0]], [[3.0, 4.0, 7.0], [7.0, 4.0, 5.0], 
[5.0, 1.0, 8.0]]], [[[3.0, 1.0, 3.0], [3.0, 4.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 7.0], [5.0, 5.0, 4.0]], 
[[8.0, 6.0, 6.0], [7.0, 8.0, 7.0], [4.0, 1.0, 8.0]]], [[[6.0, 2.0, 5.0], [2.0, 6.0, 5.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 4.0], 
[6.0, 2.0, 3.0], [8.0, 8.0, 6.0]], [[5.0, 3.0, 7.0], [8.0, 2.0, 4.0], [6.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULLP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 1.0], [7.0, 4.0, 7.0], [5.0, 6.0, 1.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [3.0, 
6.0, 3.0]], [[1.0, 5.0, 2.0], [8.0, 8.0, 7.0], [5.0, 4.0, 2.0]]], [[[8.0, 8.0, 2.0], [7.0, 4.0, 6.0], [3.0, 2.0, 3.0]], [[2.0, 
4.0, 2.0], [4.0, 5.0, 5.0], [6.0, 4.0, 7.0]], [[8.0, 6.0, 3.0], [6.0, 3.0, 5.0], [6.0, 4.0, 4.0]]], [[[6.0, 1.0, 2.0], [2.0, 
2.0, 7.0], [6.0, 2.0, 2.0]], [[7.0, 3.0, 5.0], [8.0, 1.0, 3.0], [1.0, 5.0, 7.0]], [[8.0, 1.0, 6.0], [6.0, 6.0, 7.0], [6.0, 7.0, 
5.0]]], [[[2.0, 1.0, 4.0], [3.0, 4.0, 2.0], [2.0, 6.0, 6.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 7.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 
6.0], [4.0, 7.0, 2.0], [5.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 2.0], [4.0, 4.0, 7.0], [1.0, 1.0, 3.0]], 
[[3.0, 7.0, 6.0], [2.0, 6.0, 8.0], [2.0, 5.0, 2.0]], [[4.0, 6.0, 4.0], [6.0, 8.0, 2.0], [5.0, 1.0, 8.0]]], [[[5.0, 6.0, 1.0], 
[7.0, 2.0, 6.0], [5.0, 5.0, 5.0]], [[1.0, 7.0, 3.0], [1.0, 7.0, 4.0], [4.0, 4.0, 6.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 6.0], 
[1.0, 6.0, 3.0]]], [[[6.0, 6.0, 7.0], [7.0, 2.0, 5.0], [8.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [7.0, 8.0, 7.0], [4.0, 1.0, 3.0]], 
[[8.0, 2.0, 8.0], [8.0, 6.0, 3.0], [3.0, 5.0, 4.0]]], [[[7.0, 8.0, 2.0], [8.0, 5.0, 6.0], [8.0, 3.0, 3.0]], [[2.0, 4.0, 8.0], 
[8.0, 4.0, 4.0], [8.0, 3.0, 1.0]], [[1.0, 2.0, 2.0], [6.0, 7.0, 2.0], [1.0, 5.0, 1.0]]]])
      val=numpy.array([[[5.0]], [[3.0]], [[2.0]]])
      arg[:3,2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 1.0], [7.0, 4.0, 7.0], [5.0, 6.0, 1.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 3.0], [3.0, 
6.0, 3.0]], [[1.0, 5.0, 2.0], [8.0, 8.0, 7.0], [5.0, 5.0, 2.0]]], [[[8.0, 8.0, 2.0], [7.0, 4.0, 6.0], [3.0, 2.0, 3.0]], [[2.0, 
4.0, 2.0], [4.0, 5.0, 5.0], [6.0, 4.0, 7.0]], [[8.0, 6.0, 3.0], [6.0, 3.0, 5.0], [6.0, 3.0, 4.0]]], [[[6.0, 1.0, 2.0], [2.0, 
2.0, 7.0], [6.0, 2.0, 2.0]], [[7.0, 3.0, 5.0], [8.0, 1.0, 3.0], [1.0, 5.0, 7.0]], [[8.0, 1.0, 6.0], [6.0, 6.0, 7.0], [6.0, 2.0, 
5.0]]], [[[2.0, 1.0, 4.0], [3.0, 4.0, 2.0], [2.0, 6.0, 6.0]], [[4.0, 6.0, 5.0], [5.0, 7.0, 7.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 
6.0], [4.0, 7.0, 2.0], [5.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [4.0, 4.0, 7.0], [1.0, 1.0, 3.0]], 
[[3.0, 7.0, 6.0], [2.0, 6.0, 8.0], [2.0, 5.0, 2.0]], [[4.0, 6.0, 4.0], [6.0, 8.0, 2.0], [5.0, 5.0, 8.0]]], [[[5.0, 6.0, 1.0], 
[7.0, 2.0, 6.0], [5.0, 5.0, 5.0]], [[1.0, 7.0, 3.0], [1.0, 7.0, 4.0], [4.0, 4.0, 6.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 6.0], 
[1.0, 3.0, 3.0]]], [[[6.0, 6.0, 7.0], [7.0, 2.0, 5.0], [8.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [7.0, 8.0, 7.0], [4.0, 1.0, 3.0]], 
[[8.0, 2.0, 8.0], [8.0, 6.0, 3.0], [3.0, 2.0, 4.0]]], [[[7.0, 8.0, 2.0], [8.0, 5.0, 6.0], [8.0, 3.0, 3.0]], [[2.0, 4.0, 8.0], 
[8.0, 4.0, 4.0], [8.0, 3.0, 1.0]], [[1.0, 2.0, 2.0], [6.0, 7.0, 2.0], [1.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULLP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 3.0], [1.0, 2.0, 1.0], [7.0, 2.0, 4.0]], [[5.0, 4.0, 1.0], [3.0, 8.0, 8.0], [6.0, 
2.0, 3.0]], [[8.0, 8.0, 5.0], [8.0, 5.0, 7.0], [3.0, 7.0, 4.0]]], [[[3.0, 2.0, 6.0], [6.0, 4.0, 7.0], [4.0, 8.0, 7.0]], [[6.0, 
2.0, 3.0], [4.0, 5.0, 3.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 8.0], [7.0, 4.0, 5.0], [3.0, 4.0, 2.0]]], [[[5.0, 8.0, 3.0], [1.0, 
3.0, 7.0], [2.0, 6.0, 5.0]], [[3.0, 2.0, 7.0], [7.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[8.0, 3.0, 1.0], [5.0, 7.0, 1.0], [7.0, 8.0, 
6.0]]], [[[7.0, 6.0, 1.0], [8.0, 2.0, 3.0], [7.0, 6.0, 2.0]], [[2.0, 5.0, 4.0], [6.0, 7.0, 3.0], [8.0, 2.0, 5.0]], [[4.0, 3.0, 
3.0], [8.0, 1.0, 2.0], [1.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 5.0], [5.0, 5.0, 8.0], [1.0, 3.0, 8.0]], 
[[5.0, 4.0, 1.0], [7.0, 8.0, 8.0], [7.0, 3.0, 1.0]], [[2.0, 1.0, 5.0], [3.0, 2.0, 1.0], [4.0, 8.0, 3.0]]], [[[2.0, 8.0, 4.0], 
[8.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 8.0, 8.0], [2.0, 4.0, 4.0], [5.0, 4.0, 6.0]], [[1.0, 4.0, 4.0], [8.0, 8.0, 2.0], 
[6.0, 4.0, 2.0]]], [[[8.0, 4.0, 8.0], [4.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 2.0], [4.0, 3.0, 6.0], [5.0, 7.0, 6.0]], 
[[8.0, 2.0, 7.0], [5.0, 6.0, 3.0], [3.0, 2.0, 1.0]]], [[[6.0, 7.0, 2.0], [3.0, 2.0, 1.0], [3.0, 5.0, 6.0]], [[1.0, 1.0, 6.0], 
[6.0, 7.0, 7.0], [2.0, 5.0, 2.0]], [[1.0, 3.0, 4.0], [8.0, 8.0, 7.0], [7.0, 6.0, 8.0]]]])
      val=Data(numpy.array([[[5.0]], [[3.0]], [[7.0]]]),self.functionspace)
      arg[:3,2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 3.0], [1.0, 2.0, 1.0], [7.0, 2.0, 4.0]], [[5.0, 4.0, 1.0], [3.0, 8.0, 8.0], [6.0, 
2.0, 3.0]], [[8.0, 8.0, 5.0], [8.0, 5.0, 7.0], [3.0, 5.0, 4.0]]], [[[3.0, 2.0, 6.0], [6.0, 4.0, 7.0], [4.0, 8.0, 7.0]], [[6.0, 
2.0, 3.0], [4.0, 5.0, 3.0], [8.0, 1.0, 8.0]], [[1.0, 2.0, 8.0], [7.0, 4.0, 5.0], [3.0, 3.0, 2.0]]], [[[5.0, 8.0, 3.0], [1.0, 
3.0, 7.0], [2.0, 6.0, 5.0]], [[3.0, 2.0, 7.0], [7.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[8.0, 3.0, 1.0], [5.0, 7.0, 1.0], [7.0, 7.0, 
6.0]]], [[[7.0, 6.0, 1.0], [8.0, 2.0, 3.0], [7.0, 6.0, 2.0]], [[2.0, 5.0, 4.0], [6.0, 7.0, 3.0], [8.0, 2.0, 5.0]], [[4.0, 3.0, 
3.0], [8.0, 1.0, 2.0], [1.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 5.0], [5.0, 5.0, 8.0], [1.0, 3.0, 8.0]], 
[[5.0, 4.0, 1.0], [7.0, 8.0, 8.0], [7.0, 3.0, 1.0]], [[2.0, 1.0, 5.0], [3.0, 2.0, 1.0], [4.0, 5.0, 3.0]]], [[[2.0, 8.0, 4.0], 
[8.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 8.0, 8.0], [2.0, 4.0, 4.0], [5.0, 4.0, 6.0]], [[1.0, 4.0, 4.0], [8.0, 8.0, 2.0], 
[6.0, 3.0, 2.0]]], [[[8.0, 4.0, 8.0], [4.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 2.0], [4.0, 3.0, 6.0], [5.0, 7.0, 6.0]], 
[[8.0, 2.0, 7.0], [5.0, 6.0, 3.0], [3.0, 7.0, 1.0]]], [[[6.0, 7.0, 2.0], [3.0, 2.0, 1.0], [3.0, 5.0, 6.0]], [[1.0, 1.0, 6.0], 
[6.0, 7.0, 7.0], [2.0, 5.0, 2.0]], [[1.0, 3.0, 4.0], [8.0, 8.0, 7.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULLP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 6.0], [4.0, 2.0, 1.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 2.0], [5.0, 4.0, 2.0], [3.0, 
5.0, 8.0]], [[4.0, 4.0, 7.0], [5.0, 3.0, 8.0], [7.0, 4.0, 5.0]]], [[[6.0, 3.0, 7.0], [4.0, 7.0, 4.0], [8.0, 5.0, 1.0]], [[5.0, 
4.0, 6.0], [6.0, 2.0, 4.0], [3.0, 6.0, 1.0]], [[2.0, 1.0, 7.0], [4.0, 1.0, 5.0], [1.0, 2.0, 1.0]]], [[[8.0, 4.0, 3.0], [4.0, 
7.0, 3.0], [2.0, 5.0, 4.0]], [[6.0, 1.0, 5.0], [1.0, 3.0, 7.0], [7.0, 8.0, 4.0]], [[6.0, 1.0, 8.0], [7.0, 6.0, 7.0], [4.0, 7.0, 
8.0]]], [[[7.0, 1.0, 1.0], [7.0, 6.0, 7.0], [8.0, 7.0, 7.0]], [[1.0, 1.0, 6.0], [2.0, 1.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 6.0, 
8.0], [1.0, 5.0, 3.0], [8.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 4.0], [5.0, 5.0, 5.0], [4.0, 6.0, 4.0]], 
[[1.0, 4.0, 5.0], [5.0, 5.0, 8.0], [1.0, 5.0, 1.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 7.0], [5.0, 8.0, 1.0]]], [[[5.0, 2.0, 8.0], 
[6.0, 7.0, 8.0], [4.0, 4.0, 8.0]], [[5.0, 8.0, 2.0], [1.0, 7.0, 4.0], [2.0, 1.0, 7.0]], [[8.0, 6.0, 6.0], [7.0, 6.0, 2.0], 
[8.0, 2.0, 6.0]]], [[[6.0, 2.0, 4.0], [7.0, 6.0, 3.0], [8.0, 3.0, 3.0]], [[2.0, 7.0, 6.0], [6.0, 4.0, 1.0], [2.0, 7.0, 1.0]], 
[[3.0, 5.0, 3.0], [8.0, 2.0, 8.0], [3.0, 2.0, 5.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 5.0], [2.0, 4.0, 4.0]], [[3.0, 8.0, 5.0], 
[5.0, 8.0, 8.0], [8.0, 2.0, 6.0]], [[7.0, 3.0, 2.0], [1.0, 7.0, 1.0], [8.0, 7.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0]], [[2.0]], [[8.0]]])+(1.-msk_val)*numpy.array([[[1.0]], [[1.0]], [[2.0]]])
      arg[:3,2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 6.0], [4.0, 2.0, 1.0], [4.0, 6.0, 1.0]], [[7.0, 4.0, 2.0], [5.0, 4.0, 2.0], [3.0, 
5.0, 8.0]], [[4.0, 4.0, 7.0], [5.0, 3.0, 8.0], [7.0, 5.0, 5.0]]], [[[6.0, 3.0, 7.0], [4.0, 7.0, 4.0], [8.0, 5.0, 1.0]], [[5.0, 
4.0, 6.0], [6.0, 2.0, 4.0], [3.0, 6.0, 1.0]], [[2.0, 1.0, 7.0], [4.0, 1.0, 5.0], [1.0, 2.0, 1.0]]], [[[8.0, 4.0, 3.0], [4.0, 
7.0, 3.0], [2.0, 5.0, 4.0]], [[6.0, 1.0, 5.0], [1.0, 3.0, 7.0], [7.0, 8.0, 4.0]], [[6.0, 1.0, 8.0], [7.0, 6.0, 7.0], [4.0, 8.0, 
8.0]]], [[[7.0, 1.0, 1.0], [7.0, 6.0, 7.0], [8.0, 7.0, 7.0]], [[1.0, 1.0, 6.0], [2.0, 1.0, 8.0], [4.0, 3.0, 3.0]], [[2.0, 6.0, 
8.0], [1.0, 5.0, 3.0], [8.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [5.0, 5.0, 5.0], [4.0, 6.0, 4.0]], 
[[1.0, 4.0, 5.0], [5.0, 5.0, 8.0], [1.0, 5.0, 1.0]], [[2.0, 5.0, 8.0], [2.0, 3.0, 7.0], [5.0, 1.0, 1.0]]], [[[5.0, 2.0, 8.0], 
[6.0, 7.0, 8.0], [4.0, 4.0, 8.0]], [[5.0, 8.0, 2.0], [1.0, 7.0, 4.0], [2.0, 1.0, 7.0]], [[8.0, 6.0, 6.0], [7.0, 6.0, 2.0], 
[8.0, 1.0, 6.0]]], [[[6.0, 2.0, 4.0], [7.0, 6.0, 3.0], [8.0, 3.0, 3.0]], [[2.0, 7.0, 6.0], [6.0, 4.0, 1.0], [2.0, 7.0, 1.0]], 
[[3.0, 5.0, 3.0], [8.0, 2.0, 8.0], [3.0, 2.0, 5.0]]], [[[7.0, 6.0, 5.0], [8.0, 3.0, 5.0], [2.0, 4.0, 4.0]], [[3.0, 8.0, 5.0], 
[5.0, 8.0, 8.0], [8.0, 2.0, 6.0]], [[7.0, 3.0, 2.0], [1.0, 7.0, 1.0], [8.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 4.0, 4.0], [5.0, 6.0, 3.0]], [[2.0, 6.0, 6.0], [8.0, 6.0, 7.0], [1.0, 
3.0, 2.0]], [[5.0, 3.0, 4.0], [5.0, 6.0, 3.0], [6.0, 1.0, 7.0]]], [[[8.0, 3.0, 2.0], [7.0, 3.0, 1.0], [2.0, 6.0, 1.0]], [[4.0, 
5.0, 5.0], [8.0, 8.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 3.0, 6.0], [2.0, 7.0, 3.0], [4.0, 6.0, 5.0]]], [[[2.0, 5.0, 2.0], [1.0, 
2.0, 6.0], [1.0, 5.0, 5.0]], [[7.0, 5.0, 5.0], [7.0, 6.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 5.0, 8.0], [3.0, 2.0, 6.0], [5.0, 6.0, 
3.0]]], [[[6.0, 8.0, 6.0], [1.0, 3.0, 8.0], [8.0, 3.0, 3.0]], [[8.0, 2.0, 8.0], [2.0, 3.0, 1.0], [7.0, 1.0, 4.0]], [[4.0, 5.0, 
1.0], [8.0, 4.0, 8.0], [1.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 7.0], [1.0, 2.0, 6.0], [3.0, 7.0, 4.0]], 
[[6.0, 7.0, 1.0], [7.0, 2.0, 2.0], [3.0, 4.0, 6.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 7.0], [7.0, 1.0, 5.0]]], [[[6.0, 4.0, 4.0], 
[3.0, 3.0, 2.0], [5.0, 6.0, 7.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 3.0], [4.0, 3.0, 5.0]], [[4.0, 4.0, 7.0], [7.0, 2.0, 2.0], 
[7.0, 2.0, 7.0]]], [[[7.0, 4.0, 2.0], [5.0, 7.0, 8.0], [3.0, 3.0, 5.0]], [[5.0, 7.0, 4.0], [6.0, 6.0, 1.0], [8.0, 8.0, 5.0]], 
[[5.0, 5.0, 2.0], [4.0, 8.0, 7.0], [5.0, 5.0, 6.0]]], [[[1.0, 1.0, 4.0], [6.0, 2.0, 5.0], [3.0, 8.0, 6.0]], [[3.0, 2.0, 3.0], 
[7.0, 8.0, 4.0], [3.0, 2.0, 5.0]], [[4.0, 5.0, 5.0], [5.0, 6.0, 6.0], [6.0, 6.0, 7.0]]]])
      val=numpy.array([[[[4.0, 5.0, 1.0], [6.0, 6.0, 8.0]], [[7.0, 1.0, 2.0], [7.0, 4.0, 8.0]]], [[[4.0, 7.0, 2.0], [7.0, 7.0, 
4.0]], [[6.0, 2.0, 5.0], [4.0, 6.0, 6.0]]]])
      arg[2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 4.0], [5.0, 4.0, 4.0], [5.0, 6.0, 3.0]], [[2.0, 6.0, 6.0], [8.0, 6.0, 7.0], [1.0, 
3.0, 2.0]], [[5.0, 3.0, 4.0], [5.0, 6.0, 3.0], [6.0, 1.0, 7.0]]], [[[8.0, 3.0, 2.0], [7.0, 3.0, 1.0], [2.0, 6.0, 1.0]], [[4.0, 
5.0, 5.0], [8.0, 8.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 3.0, 6.0], [2.0, 7.0, 3.0], [4.0, 6.0, 5.0]]], [[[4.0, 5.0, 1.0], [6.0, 
6.0, 8.0], [1.0, 5.0, 5.0]], [[7.0, 1.0, 2.0], [7.0, 4.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 5.0, 8.0], [3.0, 2.0, 6.0], [5.0, 6.0, 
3.0]]], [[[4.0, 7.0, 2.0], [7.0, 7.0, 4.0], [8.0, 3.0, 3.0]], [[6.0, 2.0, 5.0], [4.0, 6.0, 6.0], [7.0, 1.0, 4.0]], [[4.0, 5.0, 
1.0], [8.0, 4.0, 8.0], [1.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 7.0], [1.0, 2.0, 6.0], [3.0, 7.0, 4.0]], 
[[6.0, 7.0, 1.0], [7.0, 2.0, 2.0], [3.0, 4.0, 6.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 7.0], [7.0, 1.0, 5.0]]], [[[6.0, 4.0, 4.0], 
[3.0, 3.0, 2.0], [5.0, 6.0, 7.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 3.0], [4.0, 3.0, 5.0]], [[4.0, 4.0, 7.0], [7.0, 2.0, 2.0], 
[7.0, 2.0, 7.0]]], [[[4.0, 5.0, 1.0], [6.0, 6.0, 8.0], [3.0, 3.0, 5.0]], [[7.0, 1.0, 2.0], [7.0, 4.0, 8.0], [8.0, 8.0, 5.0]], 
[[5.0, 5.0, 2.0], [4.0, 8.0, 7.0], [5.0, 5.0, 6.0]]], [[[4.0, 7.0, 2.0], [7.0, 7.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 2.0, 5.0], 
[4.0, 6.0, 6.0], [3.0, 2.0, 5.0]], [[4.0, 5.0, 5.0], [5.0, 6.0, 6.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 5.0], [7.0, 2.0, 1.0], [5.0, 7.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 6.0, 4.0], [7.0, 
2.0, 6.0]], [[8.0, 6.0, 7.0], [4.0, 8.0, 3.0], [4.0, 7.0, 1.0]]], [[[4.0, 3.0, 1.0], [4.0, 5.0, 1.0], [6.0, 1.0, 3.0]], [[7.0, 
5.0, 1.0], [5.0, 7.0, 4.0], [6.0, 7.0, 3.0]], [[3.0, 2.0, 4.0], [4.0, 5.0, 1.0], [3.0, 3.0, 1.0]]], [[[6.0, 2.0, 5.0], [4.0, 
7.0, 2.0], [2.0, 4.0, 3.0]], [[8.0, 3.0, 6.0], [6.0, 3.0, 4.0], [5.0, 8.0, 3.0]], [[2.0, 2.0, 2.0], [2.0, 8.0, 2.0], [1.0, 2.0, 
6.0]]], [[[8.0, 3.0, 7.0], [6.0, 6.0, 7.0], [8.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [3.0, 4.0, 3.0], [1.0, 1.0, 1.0]], [[8.0, 8.0, 
6.0], [3.0, 6.0, 2.0], [1.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 1.0, 5.0], [2.0, 2.0, 3.0], [2.0, 2.0, 8.0]], 
[[8.0, 5.0, 5.0], [6.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[2.0, 3.0, 1.0], [6.0, 2.0, 1.0], [3.0, 7.0, 6.0]]], [[[2.0, 2.0, 6.0], 
[3.0, 8.0, 2.0], [3.0, 4.0, 8.0]], [[3.0, 2.0, 3.0], [5.0, 7.0, 7.0], [6.0, 4.0, 3.0]], [[2.0, 2.0, 3.0], [8.0, 3.0, 4.0], 
[3.0, 1.0, 6.0]]], [[[3.0, 7.0, 3.0], [5.0, 4.0, 3.0], [5.0, 7.0, 6.0]], [[1.0, 1.0, 1.0], [1.0, 6.0, 6.0], [2.0, 2.0, 1.0]], 
[[5.0, 5.0, 1.0], [1.0, 4.0, 2.0], [8.0, 6.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 3.0, 8.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], 
[2.0, 6.0, 2.0], [6.0, 4.0, 4.0]], [[5.0, 2.0, 7.0], [1.0, 2.0, 6.0], [6.0, 1.0, 3.0]]]])
      val=Data(numpy.array([[[[3.0, 5.0, 5.0], [4.0, 8.0, 1.0]], [[6.0, 4.0, 4.0], [1.0, 7.0, 4.0]]], [[[7.0, 2.0, 7.0], [6.0, 
3.0, 5.0]], [[2.0, 4.0, 1.0], [6.0, 7.0, 4.0]]]]),self.functionspace)
      arg[2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 5.0], [7.0, 2.0, 1.0], [5.0, 7.0, 5.0]], [[5.0, 2.0, 8.0], [6.0, 6.0, 4.0], [7.0, 
2.0, 6.0]], [[8.0, 6.0, 7.0], [4.0, 8.0, 3.0], [4.0, 7.0, 1.0]]], [[[4.0, 3.0, 1.0], [4.0, 5.0, 1.0], [6.0, 1.0, 3.0]], [[7.0, 
5.0, 1.0], [5.0, 7.0, 4.0], [6.0, 7.0, 3.0]], [[3.0, 2.0, 4.0], [4.0, 5.0, 1.0], [3.0, 3.0, 1.0]]], [[[3.0, 5.0, 5.0], [4.0, 
8.0, 1.0], [2.0, 4.0, 3.0]], [[6.0, 4.0, 4.0], [1.0, 7.0, 4.0], [5.0, 8.0, 3.0]], [[2.0, 2.0, 2.0], [2.0, 8.0, 2.0], [1.0, 2.0, 
6.0]]], [[[7.0, 2.0, 7.0], [6.0, 3.0, 5.0], [8.0, 3.0, 4.0]], [[2.0, 4.0, 1.0], [6.0, 7.0, 4.0], [1.0, 1.0, 1.0]], [[8.0, 8.0, 
6.0], [3.0, 6.0, 2.0], [1.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 5.0], [2.0, 2.0, 3.0], [2.0, 2.0, 8.0]], 
[[8.0, 5.0, 5.0], [6.0, 2.0, 4.0], [4.0, 7.0, 2.0]], [[2.0, 3.0, 1.0], [6.0, 2.0, 1.0], [3.0, 7.0, 6.0]]], [[[2.0, 2.0, 6.0], 
[3.0, 8.0, 2.0], [3.0, 4.0, 8.0]], [[3.0, 2.0, 3.0], [5.0, 7.0, 7.0], [6.0, 4.0, 3.0]], [[2.0, 2.0, 3.0], [8.0, 3.0, 4.0], 
[3.0, 1.0, 6.0]]], [[[3.0, 5.0, 5.0], [4.0, 8.0, 1.0], [5.0, 7.0, 6.0]], [[6.0, 4.0, 4.0], [1.0, 7.0, 4.0], [2.0, 2.0, 1.0]], 
[[5.0, 5.0, 1.0], [1.0, 4.0, 2.0], [8.0, 6.0, 6.0]]], [[[7.0, 2.0, 7.0], [6.0, 3.0, 5.0], [3.0, 5.0, 6.0]], [[2.0, 4.0, 1.0], 
[6.0, 7.0, 4.0], [6.0, 4.0, 4.0]], [[5.0, 2.0, 7.0], [1.0, 2.0, 6.0], [6.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 8.0], [5.0, 4.0, 3.0], [5.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [8.0, 4.0, 7.0], [7.0, 
5.0, 5.0]], [[5.0, 6.0, 2.0], [8.0, 6.0, 4.0], [3.0, 4.0, 7.0]]], [[[4.0, 5.0, 1.0], [4.0, 8.0, 7.0], [2.0, 1.0, 8.0]], [[6.0, 
5.0, 8.0], [3.0, 5.0, 5.0], [2.0, 2.0, 3.0]], [[7.0, 1.0, 2.0], [4.0, 5.0, 6.0], [7.0, 1.0, 2.0]]], [[[1.0, 1.0, 4.0], [5.0, 
2.0, 1.0], [2.0, 8.0, 6.0]], [[7.0, 1.0, 3.0], [4.0, 1.0, 4.0], [4.0, 5.0, 1.0]], [[5.0, 5.0, 1.0], [8.0, 1.0, 3.0], [5.0, 2.0, 
2.0]]], [[[2.0, 1.0, 8.0], [6.0, 3.0, 6.0], [4.0, 1.0, 7.0]], [[8.0, 7.0, 3.0], [4.0, 5.0, 5.0], [1.0, 5.0, 8.0]], [[1.0, 7.0, 
8.0], [4.0, 4.0, 8.0], [2.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 7.0], [4.0, 5.0, 8.0], [7.0, 3.0, 5.0]], 
[[6.0, 6.0, 8.0], [7.0, 8.0, 7.0], [4.0, 3.0, 3.0]], [[8.0, 1.0, 6.0], [4.0, 1.0, 7.0], [7.0, 5.0, 1.0]]], [[[6.0, 8.0, 3.0], 
[2.0, 1.0, 5.0], [6.0, 8.0, 4.0]], [[3.0, 6.0, 2.0], [8.0, 6.0, 1.0], [7.0, 1.0, 6.0]], [[1.0, 6.0, 5.0], [4.0, 1.0, 4.0], 
[6.0, 1.0, 2.0]]], [[[5.0, 2.0, 3.0], [5.0, 5.0, 7.0], [6.0, 4.0, 1.0]], [[7.0, 6.0, 6.0], [8.0, 7.0, 8.0], [7.0, 7.0, 4.0]], 
[[8.0, 1.0, 6.0], [7.0, 2.0, 7.0], [4.0, 4.0, 3.0]]], [[[8.0, 2.0, 2.0], [5.0, 7.0, 6.0], [7.0, 1.0, 8.0]], [[2.0, 6.0, 3.0], 
[4.0, 5.0, 5.0], [2.0, 2.0, 2.0]], [[5.0, 3.0, 4.0], [4.0, 5.0, 1.0], [7.0, 4.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 4.0, 2.0], [5.0, 4.0, 2.0]], [[4.0, 1.0, 5.0], [6.0, 7.0, 2.0]]], [[[4.0, 2.0, 2.0], 
[5.0, 4.0, 2.0]], [[2.0, 1.0, 6.0], [4.0, 3.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 3.0, 5.0], [5.0, 1.0, 2.0]], [[5.0, 
6.0, 7.0], [7.0, 5.0, 8.0]]], [[[7.0, 8.0, 3.0], [2.0, 5.0, 6.0]], [[5.0, 7.0, 1.0], [6.0, 4.0, 1.0]]]])
      arg[2:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 8.0], [5.0, 4.0, 3.0], [5.0, 7.0, 3.0]], [[2.0, 3.0, 6.0], [8.0, 4.0, 7.0], [7.0, 
5.0, 5.0]], [[5.0, 6.0, 2.0], [8.0, 6.0, 4.0], [3.0, 4.0, 7.0]]], [[[4.0, 5.0, 1.0], [4.0, 8.0, 7.0], [2.0, 1.0, 8.0]], [[6.0, 
5.0, 8.0], [3.0, 5.0, 5.0], [2.0, 2.0, 3.0]], [[7.0, 1.0, 2.0], [4.0, 5.0, 6.0], [7.0, 1.0, 2.0]]], [[[3.0, 4.0, 2.0], [5.0, 
4.0, 2.0], [2.0, 8.0, 6.0]], [[4.0, 1.0, 5.0], [6.0, 7.0, 2.0], [4.0, 5.0, 1.0]], [[5.0, 5.0, 1.0], [8.0, 1.0, 3.0], [5.0, 2.0, 
2.0]]], [[[4.0, 2.0, 2.0], [5.0, 4.0, 2.0], [4.0, 1.0, 7.0]], [[2.0, 1.0, 6.0], [4.0, 3.0, 2.0], [1.0, 5.0, 8.0]], [[1.0, 7.0, 
8.0], [4.0, 4.0, 8.0], [2.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 7.0], [4.0, 5.0, 8.0], [7.0, 3.0, 5.0]], 
[[6.0, 6.0, 8.0], [7.0, 8.0, 7.0], [4.0, 3.0, 3.0]], [[8.0, 1.0, 6.0], [4.0, 1.0, 7.0], [7.0, 5.0, 1.0]]], [[[6.0, 8.0, 3.0], 
[2.0, 1.0, 5.0], [6.0, 8.0, 4.0]], [[3.0, 6.0, 2.0], [8.0, 6.0, 1.0], [7.0, 1.0, 6.0]], [[1.0, 6.0, 5.0], [4.0, 1.0, 4.0], 
[6.0, 1.0, 2.0]]], [[[3.0, 3.0, 5.0], [5.0, 1.0, 2.0], [6.0, 4.0, 1.0]], [[5.0, 6.0, 7.0], [7.0, 5.0, 8.0], [7.0, 7.0, 4.0]], 
[[8.0, 1.0, 6.0], [7.0, 2.0, 7.0], [4.0, 4.0, 3.0]]], [[[7.0, 8.0, 3.0], [2.0, 5.0, 6.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 1.0], 
[6.0, 4.0, 1.0], [2.0, 2.0, 2.0]], [[5.0, 3.0, 4.0], [4.0, 5.0, 1.0], [7.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 1.0], [7.0, 8.0, 7.0], [6.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [4.0, 5.0, 3.0], [2.0, 
8.0, 7.0]], [[2.0, 1.0, 2.0], [2.0, 1.0, 7.0], [6.0, 3.0, 3.0]]], [[[6.0, 5.0, 8.0], [8.0, 6.0, 5.0], [1.0, 3.0, 8.0]], [[2.0, 
1.0, 7.0], [6.0, 1.0, 1.0], [1.0, 6.0, 2.0]], [[5.0, 2.0, 7.0], [1.0, 8.0, 2.0], [4.0, 5.0, 5.0]]], [[[1.0, 7.0, 3.0], [6.0, 
8.0, 1.0], [6.0, 6.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[2.0, 2.0, 6.0], [4.0, 5.0, 5.0], [2.0, 3.0, 
7.0]]], [[[4.0, 7.0, 2.0], [4.0, 8.0, 3.0], [2.0, 4.0, 2.0]], [[2.0, 4.0, 7.0], [6.0, 4.0, 1.0], [5.0, 4.0, 5.0]], [[2.0, 4.0, 
3.0], [5.0, 8.0, 6.0], [5.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 3.0], [1.0, 7.0, 8.0], [1.0, 7.0, 4.0]], 
[[5.0, 3.0, 8.0], [7.0, 1.0, 5.0], [8.0, 4.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 3.0, 6.0], [5.0, 5.0, 3.0]]], [[[2.0, 8.0, 2.0], 
[7.0, 3.0, 5.0], [2.0, 2.0, 8.0]], [[6.0, 1.0, 8.0], [4.0, 8.0, 7.0], [4.0, 6.0, 1.0]], [[5.0, 1.0, 1.0], [6.0, 6.0, 3.0], 
[5.0, 5.0, 3.0]]], [[[8.0, 7.0, 5.0], [8.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[4.0, 3.0, 1.0], [4.0, 1.0, 7.0], [4.0, 7.0, 4.0]], 
[[8.0, 7.0, 7.0], [2.0, 1.0, 2.0], [1.0, 3.0, 6.0]]], [[[3.0, 8.0, 6.0], [8.0, 3.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 2.0, 1.0], 
[4.0, 2.0, 5.0], [5.0, 5.0, 7.0]], [[2.0, 8.0, 2.0], [4.0, 7.0, 4.0], [5.0, 2.0, 4.0]]]])
      val=numpy.array([[[[7.0]]], [[[6.0]]]])
      arg[2:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [7.0, 8.0, 7.0], [6.0, 8.0, 4.0]], [[1.0, 2.0, 1.0], [4.0, 5.0, 3.0], [2.0, 
8.0, 7.0]], [[2.0, 1.0, 2.0], [2.0, 1.0, 7.0], [6.0, 3.0, 3.0]]], [[[6.0, 5.0, 8.0], [8.0, 6.0, 5.0], [1.0, 3.0, 8.0]], [[2.0, 
1.0, 7.0], [6.0, 1.0, 1.0], [1.0, 6.0, 2.0]], [[5.0, 2.0, 7.0], [1.0, 8.0, 2.0], [4.0, 5.0, 5.0]]], [[[1.0, 7.0, 3.0], [6.0, 
8.0, 1.0], [6.0, 6.0, 4.0]], [[5.0, 5.0, 7.0], [3.0, 7.0, 5.0], [6.0, 1.0, 6.0]], [[2.0, 2.0, 6.0], [4.0, 5.0, 5.0], [2.0, 3.0, 
7.0]]], [[[4.0, 7.0, 2.0], [4.0, 8.0, 3.0], [2.0, 4.0, 2.0]], [[2.0, 4.0, 7.0], [6.0, 4.0, 1.0], [5.0, 4.0, 5.0]], [[2.0, 4.0, 
3.0], [5.0, 8.0, 6.0], [5.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 3.0], [1.0, 7.0, 8.0], [1.0, 7.0, 4.0]], 
[[5.0, 3.0, 8.0], [7.0, 1.0, 5.0], [8.0, 4.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 3.0, 6.0], [5.0, 5.0, 3.0]]], [[[2.0, 8.0, 2.0], 
[7.0, 3.0, 5.0], [2.0, 2.0, 8.0]], [[6.0, 1.0, 8.0], [4.0, 8.0, 7.0], [4.0, 6.0, 1.0]], [[5.0, 1.0, 1.0], [6.0, 6.0, 3.0], 
[5.0, 5.0, 3.0]]], [[[8.0, 7.0, 5.0], [8.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[4.0, 3.0, 1.0], [4.0, 1.0, 7.0], [4.0, 7.0, 4.0]], 
[[8.0, 7.0, 7.0], [2.0, 1.0, 2.0], [1.0, 3.0, 7.0]]], [[[3.0, 8.0, 6.0], [8.0, 3.0, 7.0], [1.0, 2.0, 7.0]], [[2.0, 2.0, 1.0], 
[4.0, 2.0, 5.0], [5.0, 5.0, 7.0]], [[2.0, 8.0, 2.0], [4.0, 7.0, 4.0], [5.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 6.0], [4.0, 4.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 3.0, 8.0], [6.0, 4.0, 5.0], [8.0, 
6.0, 5.0]], [[2.0, 4.0, 7.0], [6.0, 8.0, 3.0], [6.0, 3.0, 1.0]]], [[[6.0, 6.0, 1.0], [7.0, 3.0, 2.0], [4.0, 4.0, 7.0]], [[5.0, 
4.0, 7.0], [1.0, 8.0, 7.0], [6.0, 7.0, 2.0]], [[8.0, 4.0, 1.0], [4.0, 2.0, 2.0], [7.0, 6.0, 3.0]]], [[[3.0, 4.0, 5.0], [7.0, 
4.0, 6.0], [5.0, 6.0, 5.0]], [[3.0, 6.0, 5.0], [2.0, 7.0, 7.0], [4.0, 2.0, 6.0]], [[7.0, 8.0, 7.0], [5.0, 6.0, 1.0], [2.0, 5.0, 
1.0]]], [[[1.0, 6.0, 5.0], [7.0, 2.0, 3.0], [2.0, 7.0, 5.0]], [[4.0, 2.0, 7.0], [7.0, 3.0, 1.0], [6.0, 6.0, 3.0]], [[8.0, 1.0, 
4.0], [1.0, 7.0, 4.0], [1.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 7.0], [1.0, 1.0, 8.0], [6.0, 8.0, 7.0]], 
[[1.0, 6.0, 8.0], [5.0, 4.0, 6.0], [3.0, 7.0, 2.0]], [[5.0, 5.0, 8.0], [7.0, 4.0, 1.0], [7.0, 8.0, 7.0]]], [[[3.0, 6.0, 5.0], 
[7.0, 2.0, 6.0], [4.0, 4.0, 3.0]], [[2.0, 3.0, 5.0], [7.0, 1.0, 4.0], [6.0, 3.0, 4.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 2.0], 
[5.0, 3.0, 4.0]]], [[[5.0, 1.0, 1.0], [4.0, 5.0, 6.0], [3.0, 4.0, 2.0]], [[3.0, 2.0, 2.0], [5.0, 1.0, 3.0], [6.0, 4.0, 7.0]], 
[[8.0, 6.0, 4.0], [3.0, 1.0, 1.0], [5.0, 1.0, 8.0]]], [[[1.0, 3.0, 5.0], [2.0, 4.0, 8.0], [2.0, 4.0, 2.0]], [[3.0, 8.0, 8.0], 
[4.0, 8.0, 8.0], [1.0, 3.0, 3.0]], [[3.0, 6.0, 1.0], [1.0, 1.0, 2.0], [6.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[[3.0]]], [[[8.0]]]]),self.functionspace)
      arg[2:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 6.0], [4.0, 4.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 3.0, 8.0], [6.0, 4.0, 5.0], [8.0, 
6.0, 5.0]], [[2.0, 4.0, 7.0], [6.0, 8.0, 3.0], [6.0, 3.0, 1.0]]], [[[6.0, 6.0, 1.0], [7.0, 3.0, 2.0], [4.0, 4.0, 7.0]], [[5.0, 
4.0, 7.0], [1.0, 8.0, 7.0], [6.0, 7.0, 2.0]], [[8.0, 4.0, 1.0], [4.0, 2.0, 2.0], [7.0, 6.0, 3.0]]], [[[3.0, 4.0, 5.0], [7.0, 
4.0, 6.0], [5.0, 6.0, 5.0]], [[3.0, 6.0, 5.0], [2.0, 7.0, 7.0], [4.0, 2.0, 6.0]], [[7.0, 8.0, 7.0], [5.0, 6.0, 1.0], [2.0, 5.0, 
3.0]]], [[[1.0, 6.0, 5.0], [7.0, 2.0, 3.0], [2.0, 7.0, 5.0]], [[4.0, 2.0, 7.0], [7.0, 3.0, 1.0], [6.0, 6.0, 3.0]], [[8.0, 1.0, 
4.0], [1.0, 7.0, 4.0], [1.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 7.0], [1.0, 1.0, 8.0], [6.0, 8.0, 7.0]], 
[[1.0, 6.0, 8.0], [5.0, 4.0, 6.0], [3.0, 7.0, 2.0]], [[5.0, 5.0, 8.0], [7.0, 4.0, 1.0], [7.0, 8.0, 7.0]]], [[[3.0, 6.0, 5.0], 
[7.0, 2.0, 6.0], [4.0, 4.0, 3.0]], [[2.0, 3.0, 5.0], [7.0, 1.0, 4.0], [6.0, 3.0, 4.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 2.0], 
[5.0, 3.0, 4.0]]], [[[5.0, 1.0, 1.0], [4.0, 5.0, 6.0], [3.0, 4.0, 2.0]], [[3.0, 2.0, 2.0], [5.0, 1.0, 3.0], [6.0, 4.0, 7.0]], 
[[8.0, 6.0, 4.0], [3.0, 1.0, 1.0], [5.0, 1.0, 3.0]]], [[[1.0, 3.0, 5.0], [2.0, 4.0, 8.0], [2.0, 4.0, 2.0]], [[3.0, 8.0, 8.0], 
[4.0, 8.0, 8.0], [1.0, 3.0, 3.0]], [[3.0, 6.0, 1.0], [1.0, 1.0, 2.0], [6.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 2.0], [7.0, 1.0, 8.0], [2.0, 1.0, 5.0]], [[4.0, 8.0, 4.0], [8.0, 6.0, 1.0], [2.0, 
4.0, 2.0]], [[2.0, 4.0, 1.0], [8.0, 5.0, 3.0], [3.0, 5.0, 6.0]]], [[[3.0, 8.0, 8.0], [3.0, 2.0, 8.0], [6.0, 5.0, 8.0]], [[3.0, 
5.0, 6.0], [4.0, 1.0, 2.0], [1.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [5.0, 5.0, 4.0], [6.0, 8.0, 1.0]]], [[[7.0, 2.0, 8.0], [7.0, 
4.0, 6.0], [8.0, 1.0, 2.0]], [[1.0, 8.0, 5.0], [3.0, 6.0, 5.0], [8.0, 2.0, 1.0]], [[5.0, 5.0, 1.0], [2.0, 6.0, 4.0], [4.0, 5.0, 
7.0]]], [[[6.0, 1.0, 7.0], [5.0, 3.0, 8.0], [3.0, 4.0, 6.0]], [[8.0, 7.0, 2.0], [3.0, 3.0, 6.0], [1.0, 4.0, 3.0]], [[3.0, 3.0, 
5.0], [3.0, 5.0, 7.0], [1.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 4.0], [4.0, 3.0, 7.0], [2.0, 6.0, 5.0]], 
[[2.0, 6.0, 7.0], [7.0, 5.0, 6.0], [6.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 3.0, 3.0], [3.0, 4.0, 3.0]]], [[[5.0, 2.0, 1.0], 
[2.0, 4.0, 3.0], [3.0, 2.0, 2.0]], [[3.0, 5.0, 4.0], [8.0, 4.0, 3.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 4.0], [3.0, 3.0, 8.0], 
[6.0, 1.0, 6.0]]], [[[1.0, 3.0, 7.0], [3.0, 7.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 3.0], [6.0, 2.0, 2.0]], 
[[8.0, 1.0, 1.0], [5.0, 8.0, 2.0], [2.0, 8.0, 4.0]]], [[[6.0, 4.0, 2.0], [3.0, 8.0, 7.0], [1.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], 
[5.0, 4.0, 1.0], [5.0, 8.0, 6.0]], [[3.0, 1.0, 7.0], [1.0, 5.0, 8.0], [7.0, 7.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0]]], [[[8.0]]]])+(1.-msk_val)*numpy.array([[[[2.0]]], [[[6.0]]]])
      arg[2:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 2.0], [7.0, 1.0, 8.0], [2.0, 1.0, 5.0]], [[4.0, 8.0, 4.0], [8.0, 6.0, 1.0], [2.0, 
4.0, 2.0]], [[2.0, 4.0, 1.0], [8.0, 5.0, 3.0], [3.0, 5.0, 6.0]]], [[[3.0, 8.0, 8.0], [3.0, 2.0, 8.0], [6.0, 5.0, 8.0]], [[3.0, 
5.0, 6.0], [4.0, 1.0, 2.0], [1.0, 7.0, 2.0]], [[4.0, 3.0, 6.0], [5.0, 5.0, 4.0], [6.0, 8.0, 1.0]]], [[[7.0, 2.0, 8.0], [7.0, 
4.0, 6.0], [8.0, 1.0, 2.0]], [[1.0, 8.0, 5.0], [3.0, 6.0, 5.0], [8.0, 2.0, 1.0]], [[5.0, 5.0, 1.0], [2.0, 6.0, 4.0], [4.0, 5.0, 
8.0]]], [[[6.0, 1.0, 7.0], [5.0, 3.0, 8.0], [3.0, 4.0, 6.0]], [[8.0, 7.0, 2.0], [3.0, 3.0, 6.0], [1.0, 4.0, 3.0]], [[3.0, 3.0, 
5.0], [3.0, 5.0, 7.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [4.0, 3.0, 7.0], [2.0, 6.0, 5.0]], 
[[2.0, 6.0, 7.0], [7.0, 5.0, 6.0], [6.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 3.0, 3.0], [3.0, 4.0, 3.0]]], [[[5.0, 2.0, 1.0], 
[2.0, 4.0, 3.0], [3.0, 2.0, 2.0]], [[3.0, 5.0, 4.0], [8.0, 4.0, 3.0], [4.0, 4.0, 5.0]], [[8.0, 2.0, 4.0], [3.0, 3.0, 8.0], 
[6.0, 1.0, 6.0]]], [[[1.0, 3.0, 7.0], [3.0, 7.0, 2.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 3.0], [6.0, 2.0, 2.0]], 
[[8.0, 1.0, 1.0], [5.0, 8.0, 2.0], [2.0, 8.0, 2.0]]], [[[6.0, 4.0, 2.0], [3.0, 8.0, 7.0], [1.0, 6.0, 8.0]], [[1.0, 6.0, 5.0], 
[5.0, 4.0, 1.0], [5.0, 8.0, 6.0]], [[3.0, 1.0, 7.0], [1.0, 5.0, 8.0], [7.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UUPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 8.0], [3.0, 5.0, 1.0], [2.0, 2.0, 5.0]], [[3.0, 8.0, 6.0], [8.0, 6.0, 7.0], [2.0, 
6.0, 5.0]], [[8.0, 8.0, 4.0], [1.0, 4.0, 3.0], [2.0, 2.0, 6.0]]], [[[4.0, 1.0, 2.0], [1.0, 8.0, 5.0], [2.0, 4.0, 5.0]], [[2.0, 
1.0, 4.0], [7.0, 8.0, 8.0], [2.0, 2.0, 3.0]], [[5.0, 6.0, 2.0], [7.0, 4.0, 3.0], [2.0, 2.0, 1.0]]], [[[5.0, 5.0, 7.0], [8.0, 
5.0, 6.0], [2.0, 8.0, 7.0]], [[1.0, 6.0, 7.0], [4.0, 8.0, 2.0], [6.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], [7.0, 1.0, 5.0], [5.0, 4.0, 
7.0]]], [[[3.0, 2.0, 4.0], [3.0, 8.0, 5.0], [8.0, 7.0, 1.0]], [[8.0, 5.0, 3.0], [4.0, 2.0, 2.0], [7.0, 2.0, 5.0]], [[5.0, 4.0, 
1.0], [1.0, 7.0, 8.0], [2.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 5.0], [5.0, 4.0, 6.0], [1.0, 7.0, 6.0]], 
[[7.0, 1.0, 2.0], [4.0, 5.0, 6.0], [2.0, 1.0, 7.0]], [[7.0, 1.0, 7.0], [4.0, 8.0, 1.0], [3.0, 1.0, 3.0]]], [[[4.0, 3.0, 3.0], 
[2.0, 5.0, 7.0], [7.0, 8.0, 1.0]], [[1.0, 2.0, 2.0], [8.0, 7.0, 6.0], [2.0, 5.0, 8.0]], [[8.0, 3.0, 3.0], [6.0, 8.0, 5.0], 
[6.0, 1.0, 8.0]]], [[[1.0, 7.0, 1.0], [6.0, 8.0, 2.0], [2.0, 2.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 3.0, 2.0], [5.0, 8.0, 3.0]], 
[[7.0, 8.0, 5.0], [7.0, 6.0, 6.0], [8.0, 2.0, 1.0]]], [[[2.0, 6.0, 4.0], [4.0, 4.0, 8.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 4.0], 
[1.0, 4.0, 1.0], [6.0, 3.0, 3.0]], [[8.0, 4.0, 4.0], [4.0, 8.0, 6.0], [8.0, 8.0, 4.0]]]])
      val=numpy.array([[3.0, 3.0], [6.0, 5.0], [6.0, 2.0]])
      arg[:3,:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 8.0], [3.0, 3.0, 1.0], [2.0, 2.0, 5.0]], [[3.0, 8.0, 6.0], [8.0, 3.0, 7.0], [2.0, 
6.0, 5.0]], [[8.0, 8.0, 4.0], [1.0, 4.0, 3.0], [2.0, 2.0, 6.0]]], [[[4.0, 1.0, 2.0], [1.0, 6.0, 5.0], [2.0, 4.0, 5.0]], [[2.0, 
1.0, 4.0], [7.0, 5.0, 8.0], [2.0, 2.0, 3.0]], [[5.0, 6.0, 2.0], [7.0, 4.0, 3.0], [2.0, 2.0, 1.0]]], [[[5.0, 5.0, 7.0], [8.0, 
6.0, 6.0], [2.0, 8.0, 7.0]], [[1.0, 6.0, 7.0], [4.0, 2.0, 2.0], [6.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], [7.0, 1.0, 5.0], [5.0, 4.0, 
7.0]]], [[[3.0, 2.0, 4.0], [3.0, 8.0, 5.0], [8.0, 7.0, 1.0]], [[8.0, 5.0, 3.0], [4.0, 2.0, 2.0], [7.0, 2.0, 5.0]], [[5.0, 4.0, 
1.0], [1.0, 7.0, 8.0], [2.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 5.0], [5.0, 3.0, 6.0], [1.0, 7.0, 6.0]], 
[[7.0, 1.0, 2.0], [4.0, 3.0, 6.0], [2.0, 1.0, 7.0]], [[7.0, 1.0, 7.0], [4.0, 8.0, 1.0], [3.0, 1.0, 3.0]]], [[[4.0, 3.0, 3.0], 
[2.0, 6.0, 7.0], [7.0, 8.0, 1.0]], [[1.0, 2.0, 2.0], [8.0, 5.0, 6.0], [2.0, 5.0, 8.0]], [[8.0, 3.0, 3.0], [6.0, 8.0, 5.0], 
[6.0, 1.0, 8.0]]], [[[1.0, 7.0, 1.0], [6.0, 6.0, 2.0], [2.0, 2.0, 4.0]], [[3.0, 6.0, 4.0], [8.0, 2.0, 2.0], [5.0, 8.0, 3.0]], 
[[7.0, 8.0, 5.0], [7.0, 6.0, 6.0], [8.0, 2.0, 1.0]]], [[[2.0, 6.0, 4.0], [4.0, 4.0, 8.0], [1.0, 5.0, 1.0]], [[4.0, 8.0, 4.0], 
[1.0, 4.0, 1.0], [6.0, 3.0, 3.0]], [[8.0, 4.0, 4.0], [4.0, 8.0, 6.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UUPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 6.0], [5.0, 1.0, 7.0], [8.0, 5.0, 3.0]], [[6.0, 4.0, 2.0], [1.0, 8.0, 8.0], [5.0, 
8.0, 2.0]], [[8.0, 1.0, 3.0], [8.0, 8.0, 4.0], [4.0, 2.0, 2.0]]], [[[6.0, 8.0, 6.0], [6.0, 6.0, 2.0], [1.0, 7.0, 3.0]], [[4.0, 
5.0, 8.0], [1.0, 7.0, 2.0], [2.0, 8.0, 1.0]], [[6.0, 4.0, 7.0], [4.0, 8.0, 4.0], [7.0, 3.0, 2.0]]], [[[1.0, 2.0, 4.0], [3.0, 
5.0, 8.0], [3.0, 3.0, 3.0]], [[3.0, 6.0, 3.0], [7.0, 1.0, 3.0], [7.0, 7.0, 7.0]], [[8.0, 5.0, 7.0], [7.0, 4.0, 8.0], [3.0, 5.0, 
8.0]]], [[[8.0, 3.0, 5.0], [8.0, 7.0, 6.0], [7.0, 6.0, 5.0]], [[3.0, 2.0, 2.0], [1.0, 5.0, 2.0], [8.0, 1.0, 8.0]], [[2.0, 4.0, 
8.0], [2.0, 5.0, 7.0], [7.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 2.0], [2.0, 4.0, 5.0], [5.0, 2.0, 4.0]], 
[[4.0, 1.0, 7.0], [7.0, 8.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 5.0, 3.0], [3.0, 7.0, 3.0], [1.0, 4.0, 8.0]]], [[[8.0, 5.0, 2.0], 
[6.0, 7.0, 8.0], [8.0, 6.0, 7.0]], [[6.0, 1.0, 4.0], [3.0, 8.0, 5.0], [5.0, 6.0, 5.0]], [[8.0, 8.0, 7.0], [7.0, 7.0, 2.0], 
[6.0, 7.0, 5.0]]], [[[8.0, 8.0, 4.0], [6.0, 8.0, 2.0], [1.0, 6.0, 1.0]], [[3.0, 6.0, 2.0], [5.0, 5.0, 8.0], [4.0, 3.0, 7.0]], 
[[2.0, 7.0, 4.0], [3.0, 8.0, 3.0], [5.0, 3.0, 3.0]]], [[[3.0, 7.0, 7.0], [3.0, 2.0, 5.0], [1.0, 2.0, 8.0]], [[3.0, 8.0, 4.0], 
[1.0, 8.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [2.0, 3.0, 2.0], [1.0, 3.0, 1.0]]]])
      val=Data(numpy.array([[8.0, 3.0], [7.0, 2.0], [8.0, 5.0]]),self.functionspace)
      arg[:3,:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [5.0, 8.0, 7.0], [8.0, 5.0, 3.0]], [[6.0, 4.0, 2.0], [1.0, 3.0, 8.0], [5.0, 
8.0, 2.0]], [[8.0, 1.0, 3.0], [8.0, 8.0, 4.0], [4.0, 2.0, 2.0]]], [[[6.0, 8.0, 6.0], [6.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[4.0, 
5.0, 8.0], [1.0, 2.0, 2.0], [2.0, 8.0, 1.0]], [[6.0, 4.0, 7.0], [4.0, 8.0, 4.0], [7.0, 3.0, 2.0]]], [[[1.0, 2.0, 4.0], [3.0, 
8.0, 8.0], [3.0, 3.0, 3.0]], [[3.0, 6.0, 3.0], [7.0, 5.0, 3.0], [7.0, 7.0, 7.0]], [[8.0, 5.0, 7.0], [7.0, 4.0, 8.0], [3.0, 5.0, 
8.0]]], [[[8.0, 3.0, 5.0], [8.0, 7.0, 6.0], [7.0, 6.0, 5.0]], [[3.0, 2.0, 2.0], [1.0, 5.0, 2.0], [8.0, 1.0, 8.0]], [[2.0, 4.0, 
8.0], [2.0, 5.0, 7.0], [7.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 2.0], [2.0, 8.0, 5.0], [5.0, 2.0, 4.0]], 
[[4.0, 1.0, 7.0], [7.0, 3.0, 1.0], [6.0, 6.0, 7.0]], [[4.0, 5.0, 3.0], [3.0, 7.0, 3.0], [1.0, 4.0, 8.0]]], [[[8.0, 5.0, 2.0], 
[6.0, 7.0, 8.0], [8.0, 6.0, 7.0]], [[6.0, 1.0, 4.0], [3.0, 2.0, 5.0], [5.0, 6.0, 5.0]], [[8.0, 8.0, 7.0], [7.0, 7.0, 2.0], 
[6.0, 7.0, 5.0]]], [[[8.0, 8.0, 4.0], [6.0, 8.0, 2.0], [1.0, 6.0, 1.0]], [[3.0, 6.0, 2.0], [5.0, 5.0, 8.0], [4.0, 3.0, 7.0]], 
[[2.0, 7.0, 4.0], [3.0, 8.0, 3.0], [5.0, 3.0, 3.0]]], [[[3.0, 7.0, 7.0], [3.0, 2.0, 5.0], [1.0, 2.0, 8.0]], [[3.0, 8.0, 4.0], 
[1.0, 8.0, 6.0], [2.0, 3.0, 1.0]], [[6.0, 3.0, 3.0], [2.0, 3.0, 2.0], [1.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UUPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 8.0], [2.0, 6.0, 3.0], [6.0, 8.0, 4.0]], [[8.0, 6.0, 7.0], [2.0, 5.0, 5.0], [7.0, 
3.0, 8.0]], [[4.0, 7.0, 4.0], [1.0, 7.0, 6.0], [2.0, 6.0, 5.0]]], [[[3.0, 3.0, 7.0], [5.0, 6.0, 7.0], [5.0, 3.0, 2.0]], [[8.0, 
7.0, 7.0], [6.0, 6.0, 2.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 4.0], [2.0, 7.0, 1.0], [7.0, 6.0, 8.0]]], [[[5.0, 1.0, 3.0], [5.0, 
5.0, 7.0], [2.0, 5.0, 1.0]], [[7.0, 6.0, 4.0], [2.0, 6.0, 7.0], [6.0, 8.0, 8.0]], [[2.0, 8.0, 6.0], [3.0, 7.0, 6.0], [6.0, 8.0, 
3.0]]], [[[4.0, 8.0, 2.0], [8.0, 4.0, 2.0], [6.0, 3.0, 2.0]], [[3.0, 4.0, 1.0], [2.0, 3.0, 7.0], [4.0, 7.0, 5.0]], [[1.0, 1.0, 
8.0], [7.0, 1.0, 8.0], [4.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 8.0, 6.0], [3.0, 7.0, 6.0]], 
[[4.0, 2.0, 2.0], [8.0, 3.0, 8.0], [2.0, 1.0, 7.0]], [[5.0, 7.0, 1.0], [3.0, 2.0, 8.0], [1.0, 8.0, 2.0]]], [[[1.0, 5.0, 2.0], 
[7.0, 3.0, 1.0], [4.0, 5.0, 2.0]], [[7.0, 7.0, 2.0], [5.0, 3.0, 7.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [8.0, 1.0, 6.0], 
[7.0, 7.0, 1.0]]], [[[5.0, 5.0, 2.0], [3.0, 7.0, 1.0], [4.0, 7.0, 4.0]], [[1.0, 6.0, 2.0], [5.0, 3.0, 7.0], [7.0, 2.0, 6.0]], 
[[1.0, 1.0, 2.0], [7.0, 1.0, 5.0], [2.0, 2.0, 7.0]]], [[[2.0, 2.0, 7.0], [1.0, 8.0, 5.0], [4.0, 6.0, 7.0]], [[3.0, 7.0, 3.0], 
[6.0, 6.0, 7.0], [8.0, 6.0, 1.0]], [[6.0, 4.0, 3.0], [8.0, 2.0, 6.0], [1.0, 6.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[1.0, 3.0], [7.0, 4.0], [3.0, 6.0]])+(1.-msk_val)*numpy.array([[7.0, 7.0], [3.0, 7.0], [6.0, 
4.0]])
      arg[:3,:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 8.0], [2.0, 1.0, 3.0], [6.0, 8.0, 4.0]], [[8.0, 6.0, 7.0], [2.0, 3.0, 5.0], [7.0, 
3.0, 8.0]], [[4.0, 7.0, 4.0], [1.0, 7.0, 6.0], [2.0, 6.0, 5.0]]], [[[3.0, 3.0, 7.0], [5.0, 7.0, 7.0], [5.0, 3.0, 2.0]], [[8.0, 
7.0, 7.0], [6.0, 4.0, 2.0], [4.0, 5.0, 1.0]], [[6.0, 7.0, 4.0], [2.0, 7.0, 1.0], [7.0, 6.0, 8.0]]], [[[5.0, 1.0, 3.0], [5.0, 
3.0, 7.0], [2.0, 5.0, 1.0]], [[7.0, 6.0, 4.0], [2.0, 6.0, 7.0], [6.0, 8.0, 8.0]], [[2.0, 8.0, 6.0], [3.0, 7.0, 6.0], [6.0, 8.0, 
3.0]]], [[[4.0, 8.0, 2.0], [8.0, 4.0, 2.0], [6.0, 3.0, 2.0]], [[3.0, 4.0, 1.0], [2.0, 3.0, 7.0], [4.0, 7.0, 5.0]], [[1.0, 1.0, 
8.0], [7.0, 1.0, 8.0], [4.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 6.0], [8.0, 7.0, 6.0], [3.0, 7.0, 6.0]], 
[[4.0, 2.0, 2.0], [8.0, 7.0, 8.0], [2.0, 1.0, 7.0]], [[5.0, 7.0, 1.0], [3.0, 2.0, 8.0], [1.0, 8.0, 2.0]]], [[[1.0, 5.0, 2.0], 
[7.0, 3.0, 1.0], [4.0, 5.0, 2.0]], [[7.0, 7.0, 2.0], [5.0, 7.0, 7.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 1.0], [8.0, 1.0, 6.0], 
[7.0, 7.0, 1.0]]], [[[5.0, 5.0, 2.0], [3.0, 6.0, 1.0], [4.0, 7.0, 4.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 7.0], [7.0, 2.0, 6.0]], 
[[1.0, 1.0, 2.0], [7.0, 1.0, 5.0], [2.0, 2.0, 7.0]]], [[[2.0, 2.0, 7.0], [1.0, 8.0, 5.0], [4.0, 6.0, 7.0]], [[3.0, 7.0, 3.0], 
[6.0, 6.0, 7.0], [8.0, 6.0, 1.0]], [[6.0, 4.0, 3.0], [8.0, 2.0, 6.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RLLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 8.0], [7.0, 7.0, 8.0], [5.0, 3.0, 5.0]], [[3.0, 3.0, 5.0], [1.0, 3.0, 2.0], [7.0, 
7.0, 7.0]], [[2.0, 4.0, 8.0], [8.0, 8.0, 6.0], [3.0, 1.0, 4.0]]], [[[3.0, 4.0, 3.0], [1.0, 7.0, 7.0], [8.0, 4.0, 1.0]], [[5.0, 
4.0, 3.0], [2.0, 6.0, 3.0], [4.0, 8.0, 1.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 1.0], [2.0, 1.0, 1.0]]], [[[1.0, 7.0, 1.0], [7.0, 
4.0, 2.0], [6.0, 6.0, 5.0]], [[3.0, 3.0, 2.0], [6.0, 2.0, 6.0], [2.0, 3.0, 6.0]], [[5.0, 7.0, 6.0], [3.0, 7.0, 2.0], [7.0, 3.0, 
3.0]]], [[[1.0, 2.0, 5.0], [8.0, 7.0, 7.0], [3.0, 4.0, 8.0]], [[4.0, 3.0, 7.0], [1.0, 5.0, 8.0], [6.0, 5.0, 7.0]], [[3.0, 5.0, 
2.0], [8.0, 5.0, 4.0], [6.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 6.0], [4.0, 2.0, 7.0], [2.0, 7.0, 2.0]], 
[[2.0, 4.0, 8.0], [1.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[3.0, 1.0, 6.0], [3.0, 1.0, 5.0], [1.0, 1.0, 7.0]]], [[[4.0, 2.0, 1.0], 
[5.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 8.0], [8.0, 7.0, 4.0]], [[4.0, 2.0, 6.0], [2.0, 6.0, 7.0], 
[4.0, 8.0, 6.0]]], [[[5.0, 6.0, 1.0], [1.0, 2.0, 5.0], [6.0, 5.0, 2.0]], [[5.0, 5.0, 2.0], [5.0, 6.0, 4.0], [6.0, 8.0, 4.0]], 
[[2.0, 7.0, 1.0], [4.0, 4.0, 6.0], [6.0, 2.0, 1.0]]], [[[4.0, 4.0, 2.0], [2.0, 7.0, 1.0], [4.0, 1.0, 4.0]], [[2.0, 4.0, 3.0], 
[5.0, 1.0, 8.0], [4.0, 6.0, 6.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 8.0], [3.0, 8.0, 2.0]]]])
      val=numpy.array([[[[4.0, 1.0]]], [[[2.0, 7.0]]]])
      arg[1:3,2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 8.0], [7.0, 7.0, 8.0], [5.0, 3.0, 5.0]], [[3.0, 3.0, 5.0], [1.0, 3.0, 2.0], [7.0, 
7.0, 7.0]], [[2.0, 4.0, 8.0], [8.0, 8.0, 6.0], [3.0, 1.0, 4.0]]], [[[3.0, 4.0, 3.0], [1.0, 7.0, 7.0], [8.0, 4.0, 1.0]], [[5.0, 
4.0, 3.0], [2.0, 6.0, 3.0], [4.0, 8.0, 1.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 1.0], [4.0, 1.0, 1.0]]], [[[1.0, 7.0, 1.0], [7.0, 
4.0, 2.0], [6.0, 6.0, 5.0]], [[3.0, 3.0, 2.0], [6.0, 2.0, 6.0], [2.0, 3.0, 6.0]], [[5.0, 7.0, 6.0], [3.0, 7.0, 2.0], [2.0, 7.0, 
3.0]]], [[[1.0, 2.0, 5.0], [8.0, 7.0, 7.0], [3.0, 4.0, 8.0]], [[4.0, 3.0, 7.0], [1.0, 5.0, 8.0], [6.0, 5.0, 7.0]], [[3.0, 5.0, 
2.0], [8.0, 5.0, 4.0], [6.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 6.0], [4.0, 2.0, 7.0], [2.0, 7.0, 2.0]], 
[[2.0, 4.0, 8.0], [1.0, 6.0, 7.0], [4.0, 4.0, 7.0]], [[3.0, 1.0, 6.0], [3.0, 1.0, 5.0], [1.0, 1.0, 7.0]]], [[[4.0, 2.0, 1.0], 
[5.0, 1.0, 2.0], [5.0, 8.0, 1.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 8.0], [8.0, 7.0, 4.0]], [[4.0, 2.0, 6.0], [2.0, 6.0, 7.0], 
[4.0, 1.0, 6.0]]], [[[5.0, 6.0, 1.0], [1.0, 2.0, 5.0], [6.0, 5.0, 2.0]], [[5.0, 5.0, 2.0], [5.0, 6.0, 4.0], [6.0, 8.0, 4.0]], 
[[2.0, 7.0, 1.0], [4.0, 4.0, 6.0], [2.0, 7.0, 1.0]]], [[[4.0, 4.0, 2.0], [2.0, 7.0, 1.0], [4.0, 1.0, 4.0]], [[2.0, 4.0, 3.0], 
[5.0, 1.0, 8.0], [4.0, 6.0, 6.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 8.0], [3.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RLLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 2.0], [8.0, 6.0, 8.0], [3.0, 2.0, 7.0]], [[5.0, 2.0, 8.0], [3.0, 3.0, 5.0], [5.0, 
1.0, 5.0]], [[2.0, 1.0, 3.0], [5.0, 7.0, 6.0], [4.0, 5.0, 6.0]]], [[[3.0, 7.0, 3.0], [4.0, 3.0, 6.0], [6.0, 2.0, 2.0]], [[5.0, 
6.0, 1.0], [5.0, 4.0, 5.0], [5.0, 2.0, 2.0]], [[1.0, 3.0, 6.0], [6.0, 4.0, 6.0], [5.0, 7.0, 7.0]]], [[[5.0, 3.0, 6.0], [2.0, 
3.0, 6.0], [1.0, 3.0, 4.0]], [[4.0, 1.0, 3.0], [7.0, 3.0, 7.0], [1.0, 7.0, 5.0]], [[1.0, 2.0, 5.0], [3.0, 6.0, 5.0], [8.0, 4.0, 
8.0]]], [[[2.0, 7.0, 3.0], [6.0, 2.0, 6.0], [5.0, 3.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 3.0, 1.0], [6.0, 3.0, 6.0]], [[7.0, 3.0, 
2.0], [7.0, 1.0, 2.0], [5.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 3.0, 3.0], [7.0, 2.0, 8.0]], 
[[8.0, 3.0, 7.0], [2.0, 7.0, 2.0], [3.0, 2.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 2.0, 8.0], [5.0, 7.0, 7.0]]], [[[3.0, 3.0, 4.0], 
[1.0, 1.0, 6.0], [3.0, 3.0, 2.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 4.0], [8.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [5.0, 6.0, 2.0], 
[5.0, 7.0, 4.0]]], [[[3.0, 3.0, 1.0], [3.0, 1.0, 8.0], [5.0, 8.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 4.0, 2.0], [2.0, 8.0, 1.0]], 
[[2.0, 5.0, 8.0], [7.0, 6.0, 6.0], [1.0, 2.0, 7.0]]], [[[4.0, 8.0, 6.0], [7.0, 6.0, 4.0], [7.0, 3.0, 8.0]], [[2.0, 5.0, 6.0], 
[5.0, 8.0, 2.0], [7.0, 4.0, 4.0]], [[6.0, 8.0, 8.0], [1.0, 7.0, 1.0], [4.0, 8.0, 5.0]]]])
      val=Data(numpy.array([[[[2.0, 3.0]]], [[[8.0, 3.0]]]]),self.functionspace)
      arg[1:3,2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 2.0], [8.0, 6.0, 8.0], [3.0, 2.0, 7.0]], [[5.0, 2.0, 8.0], [3.0, 3.0, 5.0], [5.0, 
1.0, 5.0]], [[2.0, 1.0, 3.0], [5.0, 7.0, 6.0], [4.0, 5.0, 6.0]]], [[[3.0, 7.0, 3.0], [4.0, 3.0, 6.0], [6.0, 2.0, 2.0]], [[5.0, 
6.0, 1.0], [5.0, 4.0, 5.0], [5.0, 2.0, 2.0]], [[1.0, 3.0, 6.0], [6.0, 4.0, 6.0], [2.0, 3.0, 7.0]]], [[[5.0, 3.0, 6.0], [2.0, 
3.0, 6.0], [1.0, 3.0, 4.0]], [[4.0, 1.0, 3.0], [7.0, 3.0, 7.0], [1.0, 7.0, 5.0]], [[1.0, 2.0, 5.0], [3.0, 6.0, 5.0], [8.0, 3.0, 
8.0]]], [[[2.0, 7.0, 3.0], [6.0, 2.0, 6.0], [5.0, 3.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 3.0, 1.0], [6.0, 3.0, 6.0]], [[7.0, 3.0, 
2.0], [7.0, 1.0, 2.0], [5.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 3.0, 3.0], [7.0, 2.0, 8.0]], 
[[8.0, 3.0, 7.0], [2.0, 7.0, 2.0], [3.0, 2.0, 4.0]], [[7.0, 4.0, 5.0], [1.0, 2.0, 8.0], [5.0, 7.0, 7.0]]], [[[3.0, 3.0, 4.0], 
[1.0, 1.0, 6.0], [3.0, 3.0, 2.0]], [[3.0, 6.0, 6.0], [8.0, 7.0, 4.0], [8.0, 1.0, 2.0]], [[6.0, 2.0, 3.0], [5.0, 6.0, 2.0], 
[2.0, 3.0, 4.0]]], [[[3.0, 3.0, 1.0], [3.0, 1.0, 8.0], [5.0, 8.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 4.0, 2.0], [2.0, 8.0, 1.0]], 
[[2.0, 5.0, 8.0], [7.0, 6.0, 6.0], [8.0, 3.0, 7.0]]], [[[4.0, 8.0, 6.0], [7.0, 6.0, 4.0], [7.0, 3.0, 8.0]], [[2.0, 5.0, 6.0], 
[5.0, 8.0, 2.0], [7.0, 4.0, 4.0]], [[6.0, 8.0, 8.0], [1.0, 7.0, 1.0], [4.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RLLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 4.0], [2.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[2.0, 3.0, 1.0], [8.0, 4.0, 2.0], [6.0, 
2.0, 5.0]], [[7.0, 1.0, 8.0], [8.0, 6.0, 1.0], [7.0, 5.0, 5.0]]], [[[3.0, 3.0, 1.0], [4.0, 5.0, 4.0], [2.0, 4.0, 6.0]], [[8.0, 
2.0, 5.0], [2.0, 3.0, 5.0], [1.0, 6.0, 2.0]], [[8.0, 2.0, 6.0], [2.0, 4.0, 5.0], [1.0, 7.0, 4.0]]], [[[1.0, 8.0, 5.0], [1.0, 
2.0, 5.0], [7.0, 1.0, 8.0]], [[4.0, 1.0, 5.0], [5.0, 2.0, 7.0], [5.0, 8.0, 2.0]], [[5.0, 1.0, 3.0], [3.0, 5.0, 8.0], [3.0, 4.0, 
2.0]]], [[[7.0, 8.0, 3.0], [4.0, 2.0, 7.0], [5.0, 8.0, 7.0]], [[6.0, 1.0, 6.0], [5.0, 4.0, 1.0], [5.0, 4.0, 8.0]], [[3.0, 8.0, 
3.0], [2.0, 7.0, 3.0], [1.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 5.0], [4.0, 3.0, 5.0], [6.0, 2.0, 2.0]], 
[[4.0, 1.0, 8.0], [2.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[4.0, 6.0, 4.0], [8.0, 1.0, 4.0], [4.0, 7.0, 5.0]]], [[[5.0, 5.0, 5.0], 
[7.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[7.0, 4.0, 4.0], [7.0, 5.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 5.0, 3.0], [8.0, 1.0, 8.0], 
[2.0, 8.0, 2.0]]], [[[4.0, 8.0, 1.0], [2.0, 8.0, 5.0], [3.0, 1.0, 1.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 5.0], [1.0, 6.0, 6.0]], 
[[1.0, 6.0, 8.0], [1.0, 8.0, 8.0], [3.0, 1.0, 7.0]]], [[[7.0, 8.0, 1.0], [6.0, 7.0, 2.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 5.0], 
[1.0, 5.0, 4.0], [5.0, 6.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 6.0, 4.0], [2.0, 2.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 7.0]]], [[[4.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 8.0]]], [[[8.0, 4.0]]]])
      arg[1:3,2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 4.0], [2.0, 2.0, 5.0], [2.0, 4.0, 2.0]], [[2.0, 3.0, 1.0], [8.0, 4.0, 2.0], [6.0, 
2.0, 5.0]], [[7.0, 1.0, 8.0], [8.0, 6.0, 1.0], [7.0, 5.0, 5.0]]], [[[3.0, 3.0, 1.0], [4.0, 5.0, 4.0], [2.0, 4.0, 6.0]], [[8.0, 
2.0, 5.0], [2.0, 3.0, 5.0], [1.0, 6.0, 2.0]], [[8.0, 2.0, 6.0], [2.0, 4.0, 5.0], [8.0, 7.0, 4.0]]], [[[1.0, 8.0, 5.0], [1.0, 
2.0, 5.0], [7.0, 1.0, 8.0]], [[4.0, 1.0, 5.0], [5.0, 2.0, 7.0], [5.0, 8.0, 2.0]], [[5.0, 1.0, 3.0], [3.0, 5.0, 8.0], [4.0, 6.0, 
2.0]]], [[[7.0, 8.0, 3.0], [4.0, 2.0, 7.0], [5.0, 8.0, 7.0]], [[6.0, 1.0, 6.0], [5.0, 4.0, 1.0], [5.0, 4.0, 8.0]], [[3.0, 8.0, 
3.0], [2.0, 7.0, 3.0], [1.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 5.0], [4.0, 3.0, 5.0], [6.0, 2.0, 2.0]], 
[[4.0, 1.0, 8.0], [2.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[4.0, 6.0, 4.0], [8.0, 1.0, 4.0], [4.0, 7.0, 5.0]]], [[[5.0, 5.0, 5.0], 
[7.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[7.0, 4.0, 4.0], [7.0, 5.0, 3.0], [7.0, 8.0, 5.0]], [[8.0, 5.0, 3.0], [8.0, 1.0, 8.0], 
[4.0, 8.0, 2.0]]], [[[4.0, 8.0, 1.0], [2.0, 8.0, 5.0], [3.0, 1.0, 1.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 5.0], [1.0, 6.0, 6.0]], 
[[1.0, 6.0, 8.0], [1.0, 8.0, 8.0], [8.0, 4.0, 7.0]]], [[[7.0, 8.0, 1.0], [6.0, 7.0, 2.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 5.0], 
[1.0, 5.0, 4.0], [5.0, 6.0, 4.0]], [[7.0, 7.0, 6.0], [1.0, 6.0, 4.0], [2.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 3.0], [1.0, 7.0, 5.0], [2.0, 5.0, 2.0]], [[4.0, 7.0, 6.0], [4.0, 8.0, 4.0], [8.0, 
8.0, 2.0]], [[6.0, 5.0, 7.0], [8.0, 7.0, 6.0], [8.0, 7.0, 1.0]]], [[[6.0, 3.0, 3.0], [8.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[1.0, 
7.0, 8.0], [3.0, 6.0, 1.0], [2.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [2.0, 5.0, 6.0], [5.0, 8.0, 7.0]]], [[[3.0, 2.0, 6.0], [6.0, 
1.0, 5.0], [7.0, 8.0, 1.0]], [[2.0, 3.0, 6.0], [4.0, 7.0, 4.0], [4.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [2.0, 7.0, 4.0], [5.0, 8.0, 
4.0]]], [[[6.0, 3.0, 3.0], [8.0, 3.0, 3.0], [3.0, 8.0, 1.0]], [[4.0, 3.0, 4.0], [6.0, 5.0, 3.0], [2.0, 8.0, 3.0]], [[8.0, 5.0, 
6.0], [4.0, 4.0, 6.0], [1.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 1.0, 7.0], [2.0, 6.0, 7.0]], 
[[4.0, 3.0, 3.0], [2.0, 4.0, 5.0], [3.0, 1.0, 2.0]], [[5.0, 5.0, 8.0], [1.0, 7.0, 1.0], [4.0, 3.0, 3.0]]], [[[7.0, 5.0, 4.0], 
[3.0, 7.0, 6.0], [5.0, 7.0, 1.0]], [[2.0, 4.0, 6.0], [3.0, 3.0, 2.0], [2.0, 3.0, 5.0]], [[1.0, 8.0, 3.0], [1.0, 6.0, 6.0], 
[7.0, 1.0, 7.0]]], [[[5.0, 3.0, 6.0], [3.0, 2.0, 7.0], [5.0, 1.0, 2.0]], [[8.0, 4.0, 3.0], [6.0, 1.0, 3.0], [6.0, 5.0, 4.0]], 
[[2.0, 5.0, 3.0], [2.0, 6.0, 3.0], [5.0, 2.0, 7.0]]], [[[1.0, 6.0, 6.0], [2.0, 8.0, 5.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 1.0], 
[2.0, 2.0, 8.0], [8.0, 3.0, 1.0]], [[6.0, 5.0, 5.0], [7.0, 6.0, 6.0], [1.0, 3.0, 8.0]]]])
      val=numpy.array([[[1.0]], [[1.0]]])
      arg[2:,2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 3.0], [1.0, 7.0, 5.0], [2.0, 5.0, 2.0]], [[4.0, 7.0, 6.0], [4.0, 8.0, 4.0], [8.0, 
8.0, 2.0]], [[6.0, 5.0, 7.0], [8.0, 7.0, 6.0], [8.0, 7.0, 1.0]]], [[[6.0, 3.0, 3.0], [8.0, 5.0, 4.0], [1.0, 7.0, 8.0]], [[1.0, 
7.0, 8.0], [3.0, 6.0, 1.0], [2.0, 7.0, 3.0]], [[5.0, 3.0, 3.0], [2.0, 5.0, 6.0], [5.0, 8.0, 7.0]]], [[[3.0, 2.0, 6.0], [6.0, 
1.0, 5.0], [7.0, 8.0, 1.0]], [[2.0, 3.0, 6.0], [4.0, 7.0, 4.0], [4.0, 8.0, 4.0]], [[7.0, 5.0, 5.0], [2.0, 7.0, 1.0], [5.0, 8.0, 
4.0]]], [[[6.0, 3.0, 3.0], [8.0, 3.0, 3.0], [3.0, 8.0, 1.0]], [[4.0, 3.0, 4.0], [6.0, 5.0, 3.0], [2.0, 8.0, 3.0]], [[8.0, 5.0, 
6.0], [4.0, 4.0, 1.0], [1.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [4.0, 1.0, 7.0], [2.0, 6.0, 7.0]], 
[[4.0, 3.0, 3.0], [2.0, 4.0, 5.0], [3.0, 1.0, 2.0]], [[5.0, 5.0, 8.0], [1.0, 7.0, 1.0], [4.0, 3.0, 3.0]]], [[[7.0, 5.0, 4.0], 
[3.0, 7.0, 6.0], [5.0, 7.0, 1.0]], [[2.0, 4.0, 6.0], [3.0, 3.0, 2.0], [2.0, 3.0, 5.0]], [[1.0, 8.0, 3.0], [1.0, 6.0, 6.0], 
[7.0, 1.0, 7.0]]], [[[5.0, 3.0, 6.0], [3.0, 2.0, 7.0], [5.0, 1.0, 2.0]], [[8.0, 4.0, 3.0], [6.0, 1.0, 3.0], [6.0, 5.0, 4.0]], 
[[2.0, 5.0, 3.0], [2.0, 6.0, 1.0], [5.0, 2.0, 7.0]]], [[[1.0, 6.0, 6.0], [2.0, 8.0, 5.0], [1.0, 4.0, 5.0]], [[7.0, 4.0, 1.0], 
[2.0, 2.0, 8.0], [8.0, 3.0, 1.0]], [[6.0, 5.0, 5.0], [7.0, 6.0, 1.0], [1.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 7.0], [5.0, 7.0, 8.0], [1.0, 1.0, 8.0]], [[4.0, 2.0, 5.0], [4.0, 2.0, 6.0], [3.0, 
8.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 4.0, 2.0], [8.0, 7.0, 8.0]]], [[[8.0, 2.0, 6.0], [8.0, 8.0, 6.0], [5.0, 3.0, 3.0]], [[8.0, 
1.0, 5.0], [2.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [7.0, 1.0, 1.0], [2.0, 1.0, 3.0]]], [[[2.0, 5.0, 1.0], [2.0, 
1.0, 2.0], [7.0, 1.0, 3.0]], [[8.0, 6.0, 3.0], [5.0, 8.0, 1.0], [5.0, 7.0, 8.0]], [[8.0, 4.0, 8.0], [7.0, 7.0, 6.0], [3.0, 4.0, 
8.0]]], [[[5.0, 2.0, 7.0], [5.0, 1.0, 7.0], [3.0, 8.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 1.0, 8.0], [1.0, 2.0, 6.0]], [[1.0, 1.0, 
7.0], [8.0, 5.0, 5.0], [6.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 5.0], [3.0, 7.0, 8.0], [6.0, 3.0, 2.0]], 
[[5.0, 2.0, 1.0], [4.0, 4.0, 4.0], [6.0, 8.0, 6.0]], [[8.0, 8.0, 7.0], [5.0, 6.0, 1.0], [4.0, 1.0, 6.0]]], [[[1.0, 2.0, 8.0], 
[8.0, 7.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 1.0, 2.0], [3.0, 4.0, 3.0], [7.0, 4.0, 3.0]], [[5.0, 2.0, 7.0], [4.0, 7.0, 8.0], 
[7.0, 3.0, 8.0]]], [[[7.0, 4.0, 8.0], [6.0, 4.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 5.0], [6.0, 6.0, 3.0]], 
[[3.0, 6.0, 6.0], [5.0, 7.0, 7.0], [7.0, 8.0, 5.0]]], [[[8.0, 1.0, 5.0], [3.0, 5.0, 2.0], [8.0, 5.0, 4.0]], [[7.0, 5.0, 1.0], 
[4.0, 2.0, 1.0], [6.0, 6.0, 7.0]], [[1.0, 1.0, 4.0], [7.0, 4.0, 7.0], [1.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[7.0]], [[7.0]]]),self.functionspace)
      arg[2:,2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 7.0], [5.0, 7.0, 8.0], [1.0, 1.0, 8.0]], [[4.0, 2.0, 5.0], [4.0, 2.0, 6.0], [3.0, 
8.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 4.0, 2.0], [8.0, 7.0, 8.0]]], [[[8.0, 2.0, 6.0], [8.0, 8.0, 6.0], [5.0, 3.0, 3.0]], [[8.0, 
1.0, 5.0], [2.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[7.0, 3.0, 1.0], [7.0, 1.0, 1.0], [2.0, 1.0, 3.0]]], [[[2.0, 5.0, 1.0], [2.0, 
1.0, 2.0], [7.0, 1.0, 3.0]], [[8.0, 6.0, 3.0], [5.0, 8.0, 1.0], [5.0, 7.0, 8.0]], [[8.0, 4.0, 8.0], [7.0, 7.0, 7.0], [3.0, 4.0, 
8.0]]], [[[5.0, 2.0, 7.0], [5.0, 1.0, 7.0], [3.0, 8.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 1.0, 8.0], [1.0, 2.0, 6.0]], [[1.0, 1.0, 
7.0], [8.0, 5.0, 7.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 5.0], [3.0, 7.0, 8.0], [6.0, 3.0, 2.0]], 
[[5.0, 2.0, 1.0], [4.0, 4.0, 4.0], [6.0, 8.0, 6.0]], [[8.0, 8.0, 7.0], [5.0, 6.0, 1.0], [4.0, 1.0, 6.0]]], [[[1.0, 2.0, 8.0], 
[8.0, 7.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 1.0, 2.0], [3.0, 4.0, 3.0], [7.0, 4.0, 3.0]], [[5.0, 2.0, 7.0], [4.0, 7.0, 8.0], 
[7.0, 3.0, 8.0]]], [[[7.0, 4.0, 8.0], [6.0, 4.0, 3.0], [8.0, 8.0, 4.0]], [[8.0, 3.0, 5.0], [3.0, 8.0, 5.0], [6.0, 6.0, 3.0]], 
[[3.0, 6.0, 6.0], [5.0, 7.0, 7.0], [7.0, 8.0, 5.0]]], [[[8.0, 1.0, 5.0], [3.0, 5.0, 2.0], [8.0, 5.0, 4.0]], [[7.0, 5.0, 1.0], 
[4.0, 2.0, 1.0], [6.0, 6.0, 7.0]], [[1.0, 1.0, 4.0], [7.0, 4.0, 7.0], [1.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 7.0], [7.0, 4.0, 4.0], [2.0, 5.0, 5.0]], [[7.0, 7.0, 8.0], [4.0, 8.0, 8.0], [1.0, 
5.0, 1.0]], [[5.0, 7.0, 7.0], [5.0, 1.0, 2.0], [1.0, 5.0, 1.0]]], [[[5.0, 6.0, 7.0], [6.0, 6.0, 4.0], [4.0, 1.0, 6.0]], [[4.0, 
2.0, 3.0], [3.0, 5.0, 6.0], [7.0, 8.0, 4.0]], [[7.0, 5.0, 3.0], [5.0, 6.0, 1.0], [3.0, 1.0, 2.0]]], [[[3.0, 7.0, 3.0], [6.0, 
2.0, 5.0], [3.0, 6.0, 6.0]], [[1.0, 3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 8.0, 1.0]], [[2.0, 6.0, 3.0], [8.0, 1.0, 2.0], [7.0, 4.0, 
4.0]]], [[[8.0, 3.0, 3.0], [8.0, 4.0, 6.0], [4.0, 6.0, 6.0]], [[1.0, 3.0, 4.0], [5.0, 5.0, 3.0], [5.0, 1.0, 3.0]], [[6.0, 7.0, 
8.0], [3.0, 3.0, 2.0], [8.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 1.0], [6.0, 5.0, 4.0], [7.0, 7.0, 3.0]], 
[[6.0, 7.0, 4.0], [3.0, 5.0, 4.0], [8.0, 2.0, 1.0]], [[7.0, 7.0, 2.0], [1.0, 2.0, 3.0], [8.0, 3.0, 8.0]]], [[[7.0, 5.0, 1.0], 
[1.0, 4.0, 5.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 7.0], [2.0, 3.0, 4.0], [8.0, 1.0, 4.0]], [[2.0, 1.0, 1.0], [3.0, 5.0, 7.0], 
[6.0, 5.0, 7.0]]], [[[5.0, 3.0, 6.0], [4.0, 6.0, 4.0], [4.0, 7.0, 8.0]], [[3.0, 5.0, 8.0], [3.0, 5.0, 4.0], [3.0, 3.0, 7.0]], 
[[2.0, 5.0, 8.0], [7.0, 7.0, 3.0], [2.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 6.0], [4.0, 6.0, 4.0]], [[8.0, 1.0, 2.0], 
[4.0, 7.0, 8.0], [3.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [4.0, 1.0, 8.0], [2.0, 2.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0]], [[6.0]]])+(1.-msk_val)*numpy.array([[[1.0]], [[8.0]]])
      arg[2:,2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 7.0], [7.0, 4.0, 4.0], [2.0, 5.0, 5.0]], [[7.0, 7.0, 8.0], [4.0, 8.0, 8.0], [1.0, 
5.0, 1.0]], [[5.0, 7.0, 7.0], [5.0, 1.0, 2.0], [1.0, 5.0, 1.0]]], [[[5.0, 6.0, 7.0], [6.0, 6.0, 4.0], [4.0, 1.0, 6.0]], [[4.0, 
2.0, 3.0], [3.0, 5.0, 6.0], [7.0, 8.0, 4.0]], [[7.0, 5.0, 3.0], [5.0, 6.0, 1.0], [3.0, 1.0, 2.0]]], [[[3.0, 7.0, 3.0], [6.0, 
2.0, 5.0], [3.0, 6.0, 6.0]], [[1.0, 3.0, 3.0], [5.0, 4.0, 8.0], [2.0, 8.0, 1.0]], [[2.0, 6.0, 3.0], [8.0, 1.0, 6.0], [7.0, 4.0, 
4.0]]], [[[8.0, 3.0, 3.0], [8.0, 4.0, 6.0], [4.0, 6.0, 6.0]], [[1.0, 3.0, 4.0], [5.0, 5.0, 3.0], [5.0, 1.0, 3.0]], [[6.0, 7.0, 
8.0], [3.0, 3.0, 6.0], [8.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 1.0], [6.0, 5.0, 4.0], [7.0, 7.0, 3.0]], 
[[6.0, 7.0, 4.0], [3.0, 5.0, 4.0], [8.0, 2.0, 1.0]], [[7.0, 7.0, 2.0], [1.0, 2.0, 3.0], [8.0, 3.0, 8.0]]], [[[7.0, 5.0, 1.0], 
[1.0, 4.0, 5.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 7.0], [2.0, 3.0, 4.0], [8.0, 1.0, 4.0]], [[2.0, 1.0, 1.0], [3.0, 5.0, 7.0], 
[6.0, 5.0, 7.0]]], [[[5.0, 3.0, 6.0], [4.0, 6.0, 4.0], [4.0, 7.0, 8.0]], [[3.0, 5.0, 8.0], [3.0, 5.0, 4.0], [3.0, 3.0, 7.0]], 
[[2.0, 5.0, 8.0], [7.0, 7.0, 1.0], [2.0, 8.0, 4.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 6.0], [4.0, 6.0, 4.0]], [[8.0, 1.0, 2.0], 
[4.0, 7.0, 8.0], [3.0, 2.0, 4.0]], [[1.0, 6.0, 2.0], [4.0, 1.0, 8.0], [2.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 3.0], [5.0, 6.0, 7.0], [6.0, 2.0, 8.0]], [[5.0, 1.0, 4.0], [8.0, 5.0, 1.0], [4.0, 
3.0, 4.0]], [[5.0, 1.0, 3.0], [2.0, 8.0, 8.0], [6.0, 3.0, 5.0]]], [[[1.0, 1.0, 5.0], [1.0, 6.0, 6.0], [2.0, 1.0, 1.0]], [[6.0, 
1.0, 3.0], [2.0, 3.0, 5.0], [5.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], [8.0, 4.0, 3.0], [8.0, 4.0, 5.0]]], [[[8.0, 8.0, 4.0], [6.0, 
6.0, 8.0], [7.0, 1.0, 1.0]], [[1.0, 1.0, 5.0], [8.0, 4.0, 7.0], [5.0, 4.0, 4.0]], [[5.0, 8.0, 5.0], [1.0, 2.0, 8.0], [8.0, 2.0, 
4.0]]], [[[7.0, 3.0, 7.0], [3.0, 1.0, 8.0], [4.0, 2.0, 3.0]], [[3.0, 1.0, 2.0], [4.0, 7.0, 5.0], [6.0, 5.0, 3.0]], [[8.0, 7.0, 
6.0], [5.0, 3.0, 4.0], [7.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 2.0], [1.0, 8.0, 4.0], [3.0, 8.0, 5.0]], 
[[2.0, 7.0, 1.0], [1.0, 1.0, 8.0], [1.0, 8.0, 5.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 5.0], [8.0, 8.0, 3.0]]], [[[2.0, 8.0, 5.0], 
[3.0, 2.0, 3.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 4.0], [5.0, 1.0, 4.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 3.0], [8.0, 5.0, 4.0], 
[8.0, 1.0, 3.0]]], [[[4.0, 4.0, 2.0], [7.0, 6.0, 7.0], [3.0, 7.0, 4.0]], [[8.0, 8.0, 7.0], [4.0, 5.0, 1.0], [8.0, 7.0, 2.0]], 
[[5.0, 8.0, 4.0], [8.0, 8.0, 8.0], [7.0, 5.0, 6.0]]], [[[3.0, 2.0, 3.0], [3.0, 8.0, 4.0], [8.0, 8.0, 4.0]], [[4.0, 5.0, 2.0], 
[4.0, 5.0, 4.0], [5.0, 7.0, 8.0]], [[8.0, 2.0, 5.0], [5.0, 7.0, 6.0], [5.0, 8.0, 6.0]]]])
      val=numpy.array([[4.0, 2.0, 6.0], [4.0, 7.0, 2.0]])
      arg[2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 3.0], [5.0, 6.0, 7.0], [6.0, 2.0, 8.0]], [[5.0, 1.0, 4.0], [8.0, 5.0, 1.0], [4.0, 
3.0, 4.0]], [[5.0, 1.0, 3.0], [2.0, 8.0, 8.0], [6.0, 3.0, 5.0]]], [[[1.0, 1.0, 5.0], [1.0, 6.0, 6.0], [2.0, 1.0, 1.0]], [[6.0, 
1.0, 3.0], [2.0, 3.0, 5.0], [5.0, 3.0, 4.0]], [[3.0, 4.0, 1.0], [8.0, 4.0, 3.0], [8.0, 4.0, 5.0]]], [[[8.0, 8.0, 4.0], [6.0, 
6.0, 8.0], [7.0, 1.0, 1.0]], [[4.0, 2.0, 6.0], [4.0, 7.0, 2.0], [5.0, 4.0, 4.0]], [[5.0, 8.0, 5.0], [1.0, 2.0, 8.0], [8.0, 2.0, 
4.0]]], [[[7.0, 3.0, 7.0], [3.0, 1.0, 8.0], [4.0, 2.0, 3.0]], [[3.0, 1.0, 2.0], [4.0, 7.0, 5.0], [6.0, 5.0, 3.0]], [[8.0, 7.0, 
6.0], [5.0, 3.0, 4.0], [7.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 2.0], [1.0, 8.0, 4.0], [3.0, 8.0, 5.0]], 
[[2.0, 7.0, 1.0], [1.0, 1.0, 8.0], [1.0, 8.0, 5.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 5.0], [8.0, 8.0, 3.0]]], [[[2.0, 8.0, 5.0], 
[3.0, 2.0, 3.0], [7.0, 8.0, 5.0]], [[3.0, 3.0, 4.0], [5.0, 1.0, 4.0], [2.0, 8.0, 6.0]], [[3.0, 6.0, 3.0], [8.0, 5.0, 4.0], 
[8.0, 1.0, 3.0]]], [[[4.0, 4.0, 2.0], [7.0, 6.0, 7.0], [3.0, 7.0, 4.0]], [[4.0, 2.0, 6.0], [4.0, 7.0, 2.0], [8.0, 7.0, 2.0]], 
[[5.0, 8.0, 4.0], [8.0, 8.0, 8.0], [7.0, 5.0, 6.0]]], [[[3.0, 2.0, 3.0], [3.0, 8.0, 4.0], [8.0, 8.0, 4.0]], [[4.0, 5.0, 2.0], 
[4.0, 5.0, 4.0], [5.0, 7.0, 8.0]], [[8.0, 2.0, 5.0], [5.0, 7.0, 6.0], [5.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 2.0], [8.0, 3.0, 3.0], [1.0, 4.0, 2.0]], [[1.0, 1.0, 7.0], [1.0, 8.0, 4.0], [8.0, 
5.0, 3.0]], [[4.0, 7.0, 5.0], [3.0, 7.0, 3.0], [3.0, 2.0, 4.0]]], [[[1.0, 5.0, 3.0], [7.0, 3.0, 4.0], [1.0, 5.0, 6.0]], [[1.0, 
3.0, 8.0], [7.0, 2.0, 7.0], [1.0, 7.0, 2.0]], [[3.0, 1.0, 5.0], [2.0, 8.0, 7.0], [5.0, 2.0, 7.0]]], [[[5.0, 2.0, 7.0], [1.0, 
3.0, 8.0], [6.0, 3.0, 8.0]], [[2.0, 6.0, 6.0], [3.0, 2.0, 7.0], [1.0, 8.0, 1.0]], [[8.0, 2.0, 3.0], [7.0, 8.0, 5.0], [6.0, 1.0, 
4.0]]], [[[2.0, 7.0, 1.0], [7.0, 4.0, 8.0], [3.0, 8.0, 8.0]], [[5.0, 7.0, 4.0], [4.0, 1.0, 6.0], [7.0, 1.0, 8.0]], [[8.0, 4.0, 
1.0], [6.0, 1.0, 2.0], [7.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 6.0], [1.0, 4.0, 4.0], [1.0, 4.0, 3.0]], 
[[4.0, 5.0, 5.0], [7.0, 8.0, 4.0], [7.0, 7.0, 3.0]], [[7.0, 8.0, 2.0], [7.0, 1.0, 8.0], [3.0, 7.0, 2.0]]], [[[6.0, 4.0, 7.0], 
[6.0, 3.0, 4.0], [6.0, 2.0, 6.0]], [[2.0, 7.0, 6.0], [5.0, 5.0, 8.0], [5.0, 1.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 7.0, 3.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 5.0, 4.0], [8.0, 1.0, 6.0], [4.0, 4.0, 3.0]], [[4.0, 4.0, 3.0], [7.0, 8.0, 3.0], [3.0, 3.0, 8.0]], 
[[2.0, 4.0, 6.0], [8.0, 7.0, 1.0], [1.0, 7.0, 6.0]]], [[[6.0, 2.0, 8.0], [3.0, 1.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 6.0, 6.0], 
[1.0, 5.0, 7.0], [1.0, 6.0, 7.0]], [[4.0, 8.0, 8.0], [4.0, 7.0, 3.0], [2.0, 6.0, 2.0]]]])
      val=Data(numpy.array([[1.0, 2.0, 8.0], [1.0, 5.0, 7.0]]),self.functionspace)
      arg[2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 2.0], [8.0, 3.0, 3.0], [1.0, 4.0, 2.0]], [[1.0, 1.0, 7.0], [1.0, 8.0, 4.0], [8.0, 
5.0, 3.0]], [[4.0, 7.0, 5.0], [3.0, 7.0, 3.0], [3.0, 2.0, 4.0]]], [[[1.0, 5.0, 3.0], [7.0, 3.0, 4.0], [1.0, 5.0, 6.0]], [[1.0, 
3.0, 8.0], [7.0, 2.0, 7.0], [1.0, 7.0, 2.0]], [[3.0, 1.0, 5.0], [2.0, 8.0, 7.0], [5.0, 2.0, 7.0]]], [[[5.0, 2.0, 7.0], [1.0, 
3.0, 8.0], [6.0, 3.0, 8.0]], [[1.0, 2.0, 8.0], [1.0, 5.0, 7.0], [1.0, 8.0, 1.0]], [[8.0, 2.0, 3.0], [7.0, 8.0, 5.0], [6.0, 1.0, 
4.0]]], [[[2.0, 7.0, 1.0], [7.0, 4.0, 8.0], [3.0, 8.0, 8.0]], [[5.0, 7.0, 4.0], [4.0, 1.0, 6.0], [7.0, 1.0, 8.0]], [[8.0, 4.0, 
1.0], [6.0, 1.0, 2.0], [7.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [1.0, 4.0, 4.0], [1.0, 4.0, 3.0]], 
[[4.0, 5.0, 5.0], [7.0, 8.0, 4.0], [7.0, 7.0, 3.0]], [[7.0, 8.0, 2.0], [7.0, 1.0, 8.0], [3.0, 7.0, 2.0]]], [[[6.0, 4.0, 7.0], 
[6.0, 3.0, 4.0], [6.0, 2.0, 6.0]], [[2.0, 7.0, 6.0], [5.0, 5.0, 8.0], [5.0, 1.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 7.0, 3.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 5.0, 4.0], [8.0, 1.0, 6.0], [4.0, 4.0, 3.0]], [[1.0, 2.0, 8.0], [1.0, 5.0, 7.0], [3.0, 3.0, 8.0]], 
[[2.0, 4.0, 6.0], [8.0, 7.0, 1.0], [1.0, 7.0, 6.0]]], [[[6.0, 2.0, 8.0], [3.0, 1.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 6.0, 6.0], 
[1.0, 5.0, 7.0], [1.0, 6.0, 7.0]], [[4.0, 8.0, 8.0], [4.0, 7.0, 3.0], [2.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 6.0], [4.0, 1.0, 4.0], [3.0, 3.0, 4.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 6.0], [4.0, 
1.0, 3.0]], [[3.0, 7.0, 8.0], [3.0, 8.0, 4.0], [4.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0], [1.0, 6.0, 2.0], [8.0, 4.0, 4.0]], [[5.0, 
3.0, 2.0], [5.0, 3.0, 3.0], [8.0, 8.0, 5.0]], [[7.0, 3.0, 7.0], [1.0, 4.0, 6.0], [4.0, 8.0, 4.0]]], [[[8.0, 8.0, 7.0], [6.0, 
7.0, 2.0], [5.0, 6.0, 3.0]], [[3.0, 5.0, 5.0], [8.0, 2.0, 5.0], [5.0, 4.0, 5.0]], [[4.0, 4.0, 6.0], [5.0, 4.0, 2.0], [5.0, 3.0, 
8.0]]], [[[1.0, 1.0, 5.0], [2.0, 2.0, 2.0], [3.0, 4.0, 7.0]], [[4.0, 3.0, 1.0], [8.0, 3.0, 2.0], [1.0, 8.0, 1.0]], [[2.0, 1.0, 
4.0], [5.0, 2.0, 2.0], [7.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 7.0], [8.0, 4.0, 5.0], [5.0, 4.0, 7.0]], 
[[8.0, 5.0, 3.0], [6.0, 5.0, 2.0], [1.0, 5.0, 6.0]], [[1.0, 3.0, 8.0], [6.0, 2.0, 2.0], [1.0, 4.0, 2.0]]], [[[7.0, 7.0, 6.0], 
[1.0, 7.0, 2.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 8.0], [7.0, 3.0, 1.0], [8.0, 5.0, 6.0]], [[8.0, 5.0, 4.0], [4.0, 8.0, 7.0], 
[7.0, 3.0, 7.0]]], [[[7.0, 5.0, 3.0], [5.0, 4.0, 6.0], [2.0, 7.0, 5.0]], [[3.0, 4.0, 3.0], [4.0, 2.0, 1.0], [4.0, 6.0, 7.0]], 
[[8.0, 8.0, 8.0], [3.0, 2.0, 6.0], [7.0, 1.0, 7.0]]], [[[1.0, 3.0, 1.0], [6.0, 4.0, 3.0], [7.0, 8.0, 4.0]], [[2.0, 4.0, 7.0], 
[7.0, 2.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 8.0, 1.0], [7.0, 3.0, 5.0], [6.0, 2.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 5.0, 7.0], [2.0, 7.0, 1.0]])+(1.-msk_val)*numpy.array([[5.0, 1.0, 3.0], [2.0, 6.0, 8.0]])
      arg[2,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 6.0], [4.0, 1.0, 4.0], [3.0, 3.0, 4.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 6.0], [4.0, 
1.0, 3.0]], [[3.0, 7.0, 8.0], [3.0, 8.0, 4.0], [4.0, 5.0, 8.0]]], [[[6.0, 2.0, 7.0], [1.0, 6.0, 2.0], [8.0, 4.0, 4.0]], [[5.0, 
3.0, 2.0], [5.0, 3.0, 3.0], [8.0, 8.0, 5.0]], [[7.0, 3.0, 7.0], [1.0, 4.0, 6.0], [4.0, 8.0, 4.0]]], [[[8.0, 8.0, 7.0], [6.0, 
7.0, 2.0], [5.0, 6.0, 3.0]], [[6.0, 5.0, 7.0], [2.0, 7.0, 1.0], [5.0, 4.0, 5.0]], [[4.0, 4.0, 6.0], [5.0, 4.0, 2.0], [5.0, 3.0, 
8.0]]], [[[1.0, 1.0, 5.0], [2.0, 2.0, 2.0], [3.0, 4.0, 7.0]], [[4.0, 3.0, 1.0], [8.0, 3.0, 2.0], [1.0, 8.0, 1.0]], [[2.0, 1.0, 
4.0], [5.0, 2.0, 2.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [8.0, 4.0, 5.0], [5.0, 4.0, 7.0]], 
[[8.0, 5.0, 3.0], [6.0, 5.0, 2.0], [1.0, 5.0, 6.0]], [[1.0, 3.0, 8.0], [6.0, 2.0, 2.0], [1.0, 4.0, 2.0]]], [[[7.0, 7.0, 6.0], 
[1.0, 7.0, 2.0], [4.0, 5.0, 6.0]], [[7.0, 3.0, 8.0], [7.0, 3.0, 1.0], [8.0, 5.0, 6.0]], [[8.0, 5.0, 4.0], [4.0, 8.0, 7.0], 
[7.0, 3.0, 7.0]]], [[[7.0, 5.0, 3.0], [5.0, 4.0, 6.0], [2.0, 7.0, 5.0]], [[5.0, 1.0, 3.0], [2.0, 6.0, 8.0], [4.0, 6.0, 7.0]], 
[[8.0, 8.0, 8.0], [3.0, 2.0, 6.0], [7.0, 1.0, 7.0]]], [[[1.0, 3.0, 1.0], [6.0, 4.0, 3.0], [7.0, 8.0, 4.0]], [[2.0, 4.0, 7.0], 
[7.0, 2.0, 4.0], [1.0, 1.0, 6.0]], [[2.0, 8.0, 1.0], [7.0, 3.0, 5.0], [6.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 8.0], [8.0, 3.0, 8.0], [4.0, 3.0, 6.0]], [[3.0, 5.0, 7.0], [8.0, 1.0, 8.0], [2.0, 
6.0, 1.0]], [[8.0, 8.0, 2.0], [4.0, 8.0, 4.0], [5.0, 7.0, 7.0]]], [[[7.0, 4.0, 7.0], [2.0, 1.0, 5.0], [8.0, 2.0, 6.0]], [[7.0, 
4.0, 6.0], [6.0, 8.0, 5.0], [6.0, 3.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 2.0, 3.0], [8.0, 1.0, 5.0]]], [[[4.0, 4.0, 7.0], [7.0, 
7.0, 8.0], [5.0, 7.0, 2.0]], [[1.0, 8.0, 2.0], [2.0, 2.0, 1.0], [5.0, 4.0, 2.0]], [[3.0, 2.0, 7.0], [8.0, 5.0, 1.0], [3.0, 3.0, 
4.0]]], [[[6.0, 7.0, 2.0], [1.0, 6.0, 5.0], [5.0, 6.0, 1.0]], [[5.0, 2.0, 1.0], [5.0, 5.0, 2.0], [3.0, 1.0, 4.0]], [[1.0, 5.0, 
3.0], [3.0, 3.0, 5.0], [2.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 4.0], [1.0, 8.0, 6.0], [8.0, 8.0, 7.0]], 
[[3.0, 2.0, 2.0], [8.0, 6.0, 4.0], [4.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], [4.0, 7.0, 3.0], [5.0, 6.0, 6.0]]], [[[1.0, 4.0, 5.0], 
[7.0, 7.0, 8.0], [1.0, 5.0, 6.0]], [[4.0, 7.0, 1.0], [5.0, 5.0, 2.0], [7.0, 4.0, 6.0]], [[6.0, 2.0, 7.0], [3.0, 7.0, 5.0], 
[3.0, 4.0, 2.0]]], [[[4.0, 5.0, 7.0], [4.0, 5.0, 1.0], [3.0, 4.0, 6.0]], [[4.0, 1.0, 5.0], [2.0, 6.0, 6.0], [3.0, 1.0, 8.0]], 
[[2.0, 2.0, 1.0], [2.0, 4.0, 1.0], [6.0, 5.0, 1.0]]], [[[3.0, 4.0, 3.0], [4.0, 6.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 3.0, 8.0], 
[2.0, 6.0, 4.0], [7.0, 8.0, 8.0]], [[4.0, 5.0, 1.0], [7.0, 6.0, 1.0], [5.0, 7.0, 4.0]]]])
      val=numpy.array([[[[3.0], [4.0]]], [[[6.0], [3.0]]], [[[7.0], [4.0]]], [[[4.0], [3.0]]]])
      arg[:,1:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 8.0], [8.0, 3.0, 8.0], [4.0, 3.0, 6.0]], [[3.0, 3.0, 7.0], [8.0, 4.0, 8.0], [2.0, 
6.0, 1.0]], [[8.0, 8.0, 2.0], [4.0, 8.0, 4.0], [5.0, 7.0, 7.0]]], [[[7.0, 4.0, 7.0], [2.0, 1.0, 5.0], [8.0, 2.0, 6.0]], [[7.0, 
6.0, 6.0], [6.0, 3.0, 5.0], [6.0, 3.0, 3.0]], [[2.0, 4.0, 7.0], [8.0, 2.0, 3.0], [8.0, 1.0, 5.0]]], [[[4.0, 4.0, 7.0], [7.0, 
7.0, 8.0], [5.0, 7.0, 2.0]], [[1.0, 7.0, 2.0], [2.0, 4.0, 1.0], [5.0, 4.0, 2.0]], [[3.0, 2.0, 7.0], [8.0, 5.0, 1.0], [3.0, 3.0, 
4.0]]], [[[6.0, 7.0, 2.0], [1.0, 6.0, 5.0], [5.0, 6.0, 1.0]], [[5.0, 4.0, 1.0], [5.0, 3.0, 2.0], [3.0, 1.0, 4.0]], [[1.0, 5.0, 
3.0], [3.0, 3.0, 5.0], [2.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 4.0], [1.0, 8.0, 6.0], [8.0, 8.0, 7.0]], 
[[3.0, 3.0, 2.0], [8.0, 4.0, 4.0], [4.0, 1.0, 7.0]], [[1.0, 6.0, 1.0], [4.0, 7.0, 3.0], [5.0, 6.0, 6.0]]], [[[1.0, 4.0, 5.0], 
[7.0, 7.0, 8.0], [1.0, 5.0, 6.0]], [[4.0, 6.0, 1.0], [5.0, 3.0, 2.0], [7.0, 4.0, 6.0]], [[6.0, 2.0, 7.0], [3.0, 7.0, 5.0], 
[3.0, 4.0, 2.0]]], [[[4.0, 5.0, 7.0], [4.0, 5.0, 1.0], [3.0, 4.0, 6.0]], [[4.0, 7.0, 5.0], [2.0, 4.0, 6.0], [3.0, 1.0, 8.0]], 
[[2.0, 2.0, 1.0], [2.0, 4.0, 1.0], [6.0, 5.0, 1.0]]], [[[3.0, 4.0, 3.0], [4.0, 6.0, 3.0], [3.0, 7.0, 2.0]], [[7.0, 4.0, 8.0], 
[2.0, 3.0, 4.0], [7.0, 8.0, 8.0]], [[4.0, 5.0, 1.0], [7.0, 6.0, 1.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 2.0], [1.0, 8.0, 8.0], [1.0, 3.0, 5.0]], [[7.0, 8.0, 1.0], [4.0, 4.0, 6.0], [4.0, 
8.0, 4.0]], [[8.0, 7.0, 6.0], [6.0, 5.0, 2.0], [6.0, 5.0, 3.0]]], [[[3.0, 4.0, 3.0], [5.0, 8.0, 7.0], [5.0, 1.0, 5.0]], [[3.0, 
4.0, 4.0], [8.0, 6.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 4.0, 3.0], [8.0, 2.0, 4.0], [5.0, 6.0, 4.0]]], [[[7.0, 2.0, 8.0], [2.0, 
1.0, 7.0], [7.0, 8.0, 5.0]], [[2.0, 5.0, 5.0], [6.0, 3.0, 5.0], [8.0, 1.0, 2.0]], [[3.0, 7.0, 6.0], [8.0, 1.0, 8.0], [6.0, 6.0, 
7.0]]], [[[4.0, 4.0, 3.0], [5.0, 3.0, 4.0], [5.0, 7.0, 5.0]], [[3.0, 7.0, 7.0], [8.0, 1.0, 7.0], [8.0, 5.0, 3.0]], [[7.0, 4.0, 
1.0], [3.0, 8.0, 2.0], [8.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 6.0], [6.0, 7.0, 8.0], [2.0, 2.0, 7.0]], 
[[3.0, 7.0, 5.0], [7.0, 7.0, 1.0], [4.0, 1.0, 1.0]], [[3.0, 6.0, 8.0], [1.0, 3.0, 6.0], [6.0, 7.0, 3.0]]], [[[4.0, 8.0, 5.0], 
[3.0, 4.0, 2.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 1.0], [3.0, 3.0, 6.0], [1.0, 1.0, 1.0]], [[3.0, 1.0, 3.0], [7.0, 7.0, 4.0], 
[7.0, 7.0, 1.0]]], [[[3.0, 7.0, 8.0], [5.0, 7.0, 1.0], [4.0, 7.0, 2.0]], [[8.0, 4.0, 7.0], [1.0, 4.0, 6.0], [6.0, 6.0, 1.0]], 
[[6.0, 4.0, 5.0], [5.0, 8.0, 8.0], [4.0, 7.0, 4.0]]], [[[2.0, 6.0, 8.0], [4.0, 1.0, 8.0], [8.0, 4.0, 8.0]], [[5.0, 6.0, 4.0], 
[7.0, 5.0, 1.0], [5.0, 1.0, 3.0]], [[2.0, 5.0, 7.0], [5.0, 6.0, 3.0], [3.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0], [4.0]]], [[[1.0], [7.0]]], [[[6.0], [4.0]]], [[[6.0], [4.0]]]]),self.functionspace)
      arg[:,1:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 2.0], [1.0, 8.0, 8.0], [1.0, 3.0, 5.0]], [[7.0, 4.0, 1.0], [4.0, 4.0, 6.0], [4.0, 
8.0, 4.0]], [[8.0, 7.0, 6.0], [6.0, 5.0, 2.0], [6.0, 5.0, 3.0]]], [[[3.0, 4.0, 3.0], [5.0, 8.0, 7.0], [5.0, 1.0, 5.0]], [[3.0, 
1.0, 4.0], [8.0, 7.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 4.0, 3.0], [8.0, 2.0, 4.0], [5.0, 6.0, 4.0]]], [[[7.0, 2.0, 8.0], [2.0, 
1.0, 7.0], [7.0, 8.0, 5.0]], [[2.0, 6.0, 5.0], [6.0, 4.0, 5.0], [8.0, 1.0, 2.0]], [[3.0, 7.0, 6.0], [8.0, 1.0, 8.0], [6.0, 6.0, 
7.0]]], [[[4.0, 4.0, 3.0], [5.0, 3.0, 4.0], [5.0, 7.0, 5.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 7.0], [8.0, 5.0, 3.0]], [[7.0, 4.0, 
1.0], [3.0, 8.0, 2.0], [8.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 6.0], [6.0, 7.0, 8.0], [2.0, 2.0, 7.0]], 
[[3.0, 4.0, 5.0], [7.0, 4.0, 1.0], [4.0, 1.0, 1.0]], [[3.0, 6.0, 8.0], [1.0, 3.0, 6.0], [6.0, 7.0, 3.0]]], [[[4.0, 8.0, 5.0], 
[3.0, 4.0, 2.0], [1.0, 5.0, 4.0]], [[5.0, 1.0, 1.0], [3.0, 7.0, 6.0], [1.0, 1.0, 1.0]], [[3.0, 1.0, 3.0], [7.0, 7.0, 4.0], 
[7.0, 7.0, 1.0]]], [[[3.0, 7.0, 8.0], [5.0, 7.0, 1.0], [4.0, 7.0, 2.0]], [[8.0, 6.0, 7.0], [1.0, 4.0, 6.0], [6.0, 6.0, 1.0]], 
[[6.0, 4.0, 5.0], [5.0, 8.0, 8.0], [4.0, 7.0, 4.0]]], [[[2.0, 6.0, 8.0], [4.0, 1.0, 8.0], [8.0, 4.0, 8.0]], [[5.0, 6.0, 4.0], 
[7.0, 4.0, 1.0], [5.0, 1.0, 3.0]], [[2.0, 5.0, 7.0], [5.0, 6.0, 3.0], [3.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 8.0], [8.0, 6.0, 4.0], [5.0, 2.0, 4.0]], [[3.0, 4.0, 1.0], [6.0, 2.0, 3.0], [8.0, 
5.0, 6.0]], [[7.0, 3.0, 1.0], [1.0, 5.0, 7.0], [4.0, 2.0, 6.0]]], [[[4.0, 6.0, 4.0], [8.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[7.0, 
1.0, 5.0], [1.0, 2.0, 1.0], [5.0, 4.0, 2.0]], [[7.0, 8.0, 1.0], [8.0, 6.0, 1.0], [2.0, 5.0, 3.0]]], [[[3.0, 4.0, 4.0], [2.0, 
2.0, 3.0], [2.0, 8.0, 3.0]], [[1.0, 7.0, 3.0], [8.0, 6.0, 8.0], [4.0, 5.0, 7.0]], [[4.0, 2.0, 4.0], [5.0, 6.0, 4.0], [5.0, 4.0, 
7.0]]], [[[4.0, 7.0, 1.0], [8.0, 5.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 5.0, 3.0], [3.0, 3.0, 1.0], [3.0, 2.0, 4.0]], [[5.0, 3.0, 
4.0], [3.0, 4.0, 7.0], [3.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 7.0], [3.0, 6.0, 2.0], [4.0, 1.0, 1.0]], 
[[5.0, 1.0, 6.0], [4.0, 3.0, 4.0], [3.0, 7.0, 1.0]], [[4.0, 8.0, 2.0], [7.0, 7.0, 4.0], [5.0, 3.0, 5.0]]], [[[3.0, 8.0, 3.0], 
[3.0, 2.0, 2.0], [5.0, 6.0, 7.0]], [[4.0, 1.0, 7.0], [3.0, 4.0, 3.0], [8.0, 6.0, 2.0]], [[5.0, 7.0, 5.0], [7.0, 6.0, 8.0], 
[2.0, 8.0, 6.0]]], [[[8.0, 4.0, 8.0], [1.0, 4.0, 2.0], [3.0, 8.0, 3.0]], [[8.0, 8.0, 7.0], [7.0, 1.0, 6.0], [1.0, 4.0, 3.0]], 
[[3.0, 5.0, 6.0], [5.0, 6.0, 7.0], [4.0, 6.0, 8.0]]], [[[6.0, 4.0, 5.0], [7.0, 4.0, 5.0], [2.0, 5.0, 5.0]], [[6.0, 6.0, 4.0], 
[2.0, 3.0, 2.0], [6.0, 4.0, 8.0]], [[7.0, 1.0, 4.0], [5.0, 4.0, 3.0], [1.0, 1.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0], [7.0]]], [[[7.0], [2.0]]], [[[8.0], [3.0]]], [[[3.0], 
[6.0]]]])+(1.-msk_val)*numpy.array([[[[1.0], [2.0]]], [[[6.0], [2.0]]], [[[7.0], [3.0]]], [[[3.0], [2.0]]]])
      arg[:,1:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 8.0], [8.0, 6.0, 4.0], [5.0, 2.0, 4.0]], [[3.0, 1.0, 1.0], [6.0, 7.0, 3.0], [8.0, 
5.0, 6.0]], [[7.0, 3.0, 1.0], [1.0, 5.0, 7.0], [4.0, 2.0, 6.0]]], [[[4.0, 6.0, 4.0], [8.0, 4.0, 3.0], [2.0, 7.0, 7.0]], [[7.0, 
7.0, 5.0], [1.0, 2.0, 1.0], [5.0, 4.0, 2.0]], [[7.0, 8.0, 1.0], [8.0, 6.0, 1.0], [2.0, 5.0, 3.0]]], [[[3.0, 4.0, 4.0], [2.0, 
2.0, 3.0], [2.0, 8.0, 3.0]], [[1.0, 8.0, 3.0], [8.0, 3.0, 8.0], [4.0, 5.0, 7.0]], [[4.0, 2.0, 4.0], [5.0, 6.0, 4.0], [5.0, 4.0, 
7.0]]], [[[4.0, 7.0, 1.0], [8.0, 5.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 3.0, 3.0], [3.0, 6.0, 1.0], [3.0, 2.0, 4.0]], [[5.0, 3.0, 
4.0], [3.0, 4.0, 7.0], [3.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 7.0], [3.0, 6.0, 2.0], [4.0, 1.0, 1.0]], 
[[5.0, 1.0, 6.0], [4.0, 2.0, 4.0], [3.0, 7.0, 1.0]], [[4.0, 8.0, 2.0], [7.0, 7.0, 4.0], [5.0, 3.0, 5.0]]], [[[3.0, 8.0, 3.0], 
[3.0, 2.0, 2.0], [5.0, 6.0, 7.0]], [[4.0, 6.0, 7.0], [3.0, 2.0, 3.0], [8.0, 6.0, 2.0]], [[5.0, 7.0, 5.0], [7.0, 6.0, 8.0], 
[2.0, 8.0, 6.0]]], [[[8.0, 4.0, 8.0], [1.0, 4.0, 2.0], [3.0, 8.0, 3.0]], [[8.0, 7.0, 7.0], [7.0, 3.0, 6.0], [1.0, 4.0, 3.0]], 
[[3.0, 5.0, 6.0], [5.0, 6.0, 7.0], [4.0, 6.0, 8.0]]], [[[6.0, 4.0, 5.0], [7.0, 4.0, 5.0], [2.0, 5.0, 5.0]], [[6.0, 3.0, 4.0], 
[2.0, 2.0, 2.0], [6.0, 4.0, 8.0]], [[7.0, 1.0, 4.0], [5.0, 4.0, 3.0], [1.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 8.0], [5.0, 8.0, 6.0], [4.0, 8.0, 7.0]], [[2.0, 1.0, 6.0], [3.0, 6.0, 4.0], [6.0, 
4.0, 7.0]], [[3.0, 1.0, 6.0], [2.0, 1.0, 7.0], [2.0, 6.0, 7.0]]], [[[4.0, 1.0, 4.0], [1.0, 3.0, 6.0], [1.0, 3.0, 6.0]], [[6.0, 
2.0, 2.0], [1.0, 5.0, 2.0], [8.0, 4.0, 2.0]], [[4.0, 6.0, 8.0], [3.0, 6.0, 5.0], [3.0, 5.0, 7.0]]], [[[7.0, 2.0, 2.0], [3.0, 
8.0, 7.0], [1.0, 3.0, 7.0]], [[6.0, 1.0, 8.0], [2.0, 2.0, 4.0], [2.0, 8.0, 2.0]], [[2.0, 8.0, 3.0], [8.0, 6.0, 8.0], [2.0, 2.0, 
4.0]]], [[[5.0, 4.0, 3.0], [2.0, 4.0, 8.0], [2.0, 5.0, 7.0]], [[6.0, 4.0, 7.0], [2.0, 2.0, 3.0], [1.0, 3.0, 5.0]], [[6.0, 8.0, 
2.0], [5.0, 7.0, 7.0], [1.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 5.0], [8.0, 8.0, 7.0], [7.0, 5.0, 4.0]], 
[[4.0, 4.0, 7.0], [7.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[6.0, 5.0, 1.0], [2.0, 3.0, 4.0], [2.0, 5.0, 6.0]]], [[[7.0, 1.0, 4.0], 
[2.0, 7.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 1.0], [8.0, 6.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 7.0, 1.0], [4.0, 1.0, 4.0], 
[4.0, 8.0, 2.0]]], [[[3.0, 7.0, 1.0], [6.0, 3.0, 8.0], [4.0, 2.0, 5.0]], [[8.0, 1.0, 4.0], [6.0, 3.0, 6.0], [6.0, 6.0, 6.0]], 
[[1.0, 7.0, 6.0], [3.0, 4.0, 8.0], [8.0, 5.0, 3.0]]], [[[8.0, 1.0, 3.0], [1.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[5.0, 1.0, 3.0], 
[5.0, 4.0, 1.0], [6.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 5.0], [3.0, 5.0, 6.0]]]])
      val=numpy.array([[[[1.0]], [[4.0]]], [[[8.0]], [[3.0]]]])
      arg[1:3,:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 8.0], [5.0, 8.0, 6.0], [4.0, 8.0, 7.0]], [[2.0, 1.0, 6.0], [3.0, 6.0, 4.0], [6.0, 
4.0, 7.0]], [[3.0, 1.0, 6.0], [2.0, 1.0, 7.0], [2.0, 6.0, 7.0]]], [[[4.0, 1.0, 4.0], [1.0, 1.0, 6.0], [1.0, 3.0, 6.0]], [[6.0, 
2.0, 2.0], [1.0, 4.0, 2.0], [8.0, 4.0, 2.0]], [[4.0, 6.0, 8.0], [3.0, 6.0, 5.0], [3.0, 5.0, 7.0]]], [[[7.0, 2.0, 2.0], [3.0, 
8.0, 7.0], [1.0, 3.0, 7.0]], [[6.0, 1.0, 8.0], [2.0, 3.0, 4.0], [2.0, 8.0, 2.0]], [[2.0, 8.0, 3.0], [8.0, 6.0, 8.0], [2.0, 2.0, 
4.0]]], [[[5.0, 4.0, 3.0], [2.0, 4.0, 8.0], [2.0, 5.0, 7.0]], [[6.0, 4.0, 7.0], [2.0, 2.0, 3.0], [1.0, 3.0, 5.0]], [[6.0, 8.0, 
2.0], [5.0, 7.0, 7.0], [1.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [8.0, 8.0, 7.0], [7.0, 5.0, 4.0]], 
[[4.0, 4.0, 7.0], [7.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[6.0, 5.0, 1.0], [2.0, 3.0, 4.0], [2.0, 5.0, 6.0]]], [[[7.0, 1.0, 4.0], 
[2.0, 1.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 8.0, 1.0], [8.0, 4.0, 7.0], [2.0, 8.0, 2.0]], [[6.0, 7.0, 1.0], [4.0, 1.0, 4.0], 
[4.0, 8.0, 2.0]]], [[[3.0, 7.0, 1.0], [6.0, 8.0, 8.0], [4.0, 2.0, 5.0]], [[8.0, 1.0, 4.0], [6.0, 3.0, 6.0], [6.0, 6.0, 6.0]], 
[[1.0, 7.0, 6.0], [3.0, 4.0, 8.0], [8.0, 5.0, 3.0]]], [[[8.0, 1.0, 3.0], [1.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[5.0, 1.0, 3.0], 
[5.0, 4.0, 1.0], [6.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [4.0, 4.0, 5.0], [3.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 7.0], [6.0, 5.0, 3.0], [6.0, 6.0, 7.0]], [[7.0, 8.0, 2.0], [6.0, 7.0, 1.0], [4.0, 
6.0, 5.0]], [[3.0, 1.0, 6.0], [5.0, 6.0, 7.0], [6.0, 3.0, 1.0]]], [[[8.0, 7.0, 5.0], [8.0, 2.0, 3.0], [7.0, 5.0, 5.0]], [[2.0, 
6.0, 1.0], [4.0, 3.0, 1.0], [6.0, 7.0, 7.0]], [[4.0, 6.0, 6.0], [6.0, 6.0, 2.0], [8.0, 6.0, 5.0]]], [[[3.0, 7.0, 1.0], [7.0, 
4.0, 4.0], [2.0, 3.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 6.0, 5.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 4.0], [6.0, 7.0, 1.0], [1.0, 6.0, 
4.0]]], [[[8.0, 1.0, 1.0], [3.0, 2.0, 4.0], [8.0, 3.0, 1.0]], [[1.0, 3.0, 1.0], [3.0, 5.0, 4.0], [3.0, 2.0, 8.0]], [[6.0, 2.0, 
2.0], [1.0, 2.0, 2.0], [7.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 4.0], [2.0, 8.0, 8.0], [4.0, 3.0, 8.0]], 
[[4.0, 3.0, 5.0], [5.0, 1.0, 7.0], [3.0, 1.0, 6.0]], [[2.0, 2.0, 7.0], [1.0, 3.0, 5.0], [6.0, 1.0, 1.0]]], [[[8.0, 5.0, 6.0], 
[4.0, 8.0, 7.0], [8.0, 3.0, 7.0]], [[2.0, 6.0, 6.0], [6.0, 4.0, 5.0], [7.0, 8.0, 8.0]], [[1.0, 2.0, 2.0], [7.0, 2.0, 4.0], 
[4.0, 2.0, 3.0]]], [[[2.0, 3.0, 3.0], [8.0, 8.0, 3.0], [2.0, 1.0, 1.0]], [[7.0, 8.0, 4.0], [2.0, 6.0, 7.0], [4.0, 4.0, 1.0]], 
[[6.0, 6.0, 4.0], [4.0, 5.0, 7.0], [3.0, 4.0, 4.0]]], [[[6.0, 6.0, 1.0], [4.0, 7.0, 6.0], [6.0, 4.0, 4.0]], [[3.0, 4.0, 4.0], 
[3.0, 3.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 6.0, 2.0], [2.0, 4.0, 3.0], [1.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0]], [[3.0]]], [[[5.0]], [[1.0]]]]),self.functionspace)
      arg[1:3,:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 7.0], [6.0, 5.0, 3.0], [6.0, 6.0, 7.0]], [[7.0, 8.0, 2.0], [6.0, 7.0, 1.0], [4.0, 
6.0, 5.0]], [[3.0, 1.0, 6.0], [5.0, 6.0, 7.0], [6.0, 3.0, 1.0]]], [[[8.0, 7.0, 5.0], [8.0, 8.0, 3.0], [7.0, 5.0, 5.0]], [[2.0, 
6.0, 1.0], [4.0, 3.0, 1.0], [6.0, 7.0, 7.0]], [[4.0, 6.0, 6.0], [6.0, 6.0, 2.0], [8.0, 6.0, 5.0]]], [[[3.0, 7.0, 1.0], [7.0, 
5.0, 4.0], [2.0, 3.0, 4.0]], [[4.0, 2.0, 1.0], [3.0, 1.0, 5.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 4.0], [6.0, 7.0, 1.0], [1.0, 6.0, 
4.0]]], [[[8.0, 1.0, 1.0], [3.0, 2.0, 4.0], [8.0, 3.0, 1.0]], [[1.0, 3.0, 1.0], [3.0, 5.0, 4.0], [3.0, 2.0, 8.0]], [[6.0, 2.0, 
2.0], [1.0, 2.0, 2.0], [7.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 4.0], [2.0, 8.0, 8.0], [4.0, 3.0, 8.0]], 
[[4.0, 3.0, 5.0], [5.0, 1.0, 7.0], [3.0, 1.0, 6.0]], [[2.0, 2.0, 7.0], [1.0, 3.0, 5.0], [6.0, 1.0, 1.0]]], [[[8.0, 5.0, 6.0], 
[4.0, 8.0, 7.0], [8.0, 3.0, 7.0]], [[2.0, 6.0, 6.0], [6.0, 3.0, 5.0], [7.0, 8.0, 8.0]], [[1.0, 2.0, 2.0], [7.0, 2.0, 4.0], 
[4.0, 2.0, 3.0]]], [[[2.0, 3.0, 3.0], [8.0, 5.0, 3.0], [2.0, 1.0, 1.0]], [[7.0, 8.0, 4.0], [2.0, 1.0, 7.0], [4.0, 4.0, 1.0]], 
[[6.0, 6.0, 4.0], [4.0, 5.0, 7.0], [3.0, 4.0, 4.0]]], [[[6.0, 6.0, 1.0], [4.0, 7.0, 6.0], [6.0, 4.0, 4.0]], [[3.0, 4.0, 4.0], 
[3.0, 3.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 6.0, 2.0], [2.0, 4.0, 3.0], [1.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 2.0], [1.0, 5.0, 6.0], [2.0, 3.0, 8.0]], [[4.0, 8.0, 7.0], [5.0, 5.0, 3.0], [5.0, 
7.0, 4.0]], [[6.0, 8.0, 1.0], [6.0, 8.0, 6.0], [7.0, 3.0, 1.0]]], [[[4.0, 7.0, 2.0], [6.0, 8.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 
6.0, 7.0], [3.0, 3.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 4.0], [2.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[5.0, 5.0, 1.0], [2.0, 
4.0, 8.0], [5.0, 5.0, 8.0]], [[7.0, 4.0, 4.0], [8.0, 3.0, 4.0], [3.0, 3.0, 5.0]], [[6.0, 2.0, 3.0], [8.0, 1.0, 5.0], [1.0, 8.0, 
8.0]]], [[[5.0, 2.0, 5.0], [4.0, 3.0, 8.0], [2.0, 8.0, 8.0]], [[5.0, 8.0, 5.0], [1.0, 3.0, 2.0], [4.0, 6.0, 6.0]], [[8.0, 7.0, 
6.0], [4.0, 1.0, 5.0], [3.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 7.0, 1.0], [8.0, 8.0, 7.0], [8.0, 1.0, 3.0]], 
[[7.0, 8.0, 2.0], [7.0, 2.0, 4.0], [6.0, 2.0, 6.0]], [[1.0, 2.0, 5.0], [8.0, 6.0, 5.0], [7.0, 1.0, 7.0]]], [[[6.0, 7.0, 7.0], 
[1.0, 7.0, 3.0], [8.0, 5.0, 1.0]], [[4.0, 4.0, 7.0], [1.0, 1.0, 5.0], [8.0, 5.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 6.0, 3.0], 
[6.0, 3.0, 5.0]]], [[[6.0, 6.0, 2.0], [7.0, 6.0, 1.0], [3.0, 2.0, 1.0]], [[6.0, 3.0, 8.0], [1.0, 1.0, 1.0], [5.0, 5.0, 1.0]], 
[[1.0, 5.0, 2.0], [8.0, 5.0, 8.0], [6.0, 5.0, 3.0]]], [[[4.0, 2.0, 4.0], [5.0, 1.0, 3.0], [8.0, 2.0, 6.0]], [[7.0, 1.0, 3.0], 
[2.0, 6.0, 2.0], [3.0, 6.0, 2.0]], [[2.0, 6.0, 6.0], [1.0, 2.0, 6.0], [6.0, 6.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0]], [[1.0]]], [[[1.0]], [[2.0]]]])+(1.-msk_val)*numpy.array([[[[8.0]], [[8.0]]], [[[5.0]], 
[[8.0]]]])
      arg[1:3,:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 2.0], [1.0, 5.0, 6.0], [2.0, 3.0, 8.0]], [[4.0, 8.0, 7.0], [5.0, 5.0, 3.0], [5.0, 
7.0, 4.0]], [[6.0, 8.0, 1.0], [6.0, 8.0, 6.0], [7.0, 3.0, 1.0]]], [[[4.0, 7.0, 2.0], [6.0, 5.0, 1.0], [1.0, 2.0, 4.0]], [[1.0, 
6.0, 7.0], [3.0, 1.0, 4.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 4.0], [2.0, 8.0, 1.0], [4.0, 1.0, 5.0]]], [[[5.0, 5.0, 1.0], [2.0, 
1.0, 8.0], [5.0, 5.0, 8.0]], [[7.0, 4.0, 4.0], [8.0, 2.0, 4.0], [3.0, 3.0, 5.0]], [[6.0, 2.0, 3.0], [8.0, 1.0, 5.0], [1.0, 8.0, 
8.0]]], [[[5.0, 2.0, 5.0], [4.0, 3.0, 8.0], [2.0, 8.0, 8.0]], [[5.0, 8.0, 5.0], [1.0, 3.0, 2.0], [4.0, 6.0, 6.0]], [[8.0, 7.0, 
6.0], [4.0, 1.0, 5.0], [3.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 1.0], [8.0, 8.0, 7.0], [8.0, 1.0, 3.0]], 
[[7.0, 8.0, 2.0], [7.0, 2.0, 4.0], [6.0, 2.0, 6.0]], [[1.0, 2.0, 5.0], [8.0, 6.0, 5.0], [7.0, 1.0, 7.0]]], [[[6.0, 7.0, 7.0], 
[1.0, 8.0, 3.0], [8.0, 5.0, 1.0]], [[4.0, 4.0, 7.0], [1.0, 8.0, 5.0], [8.0, 5.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 6.0, 3.0], 
[6.0, 3.0, 5.0]]], [[[6.0, 6.0, 2.0], [7.0, 5.0, 1.0], [3.0, 2.0, 1.0]], [[6.0, 3.0, 8.0], [1.0, 8.0, 1.0], [5.0, 5.0, 1.0]], 
[[1.0, 5.0, 2.0], [8.0, 5.0, 8.0], [6.0, 5.0, 3.0]]], [[[4.0, 2.0, 4.0], [5.0, 1.0, 3.0], [8.0, 2.0, 6.0]], [[7.0, 1.0, 3.0], 
[2.0, 6.0, 2.0], [3.0, 6.0, 2.0]], [[2.0, 6.0, 6.0], [1.0, 2.0, 6.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 2.0], [2.0, 4.0, 1.0], [8.0, 6.0, 3.0]], [[5.0, 5.0, 1.0], [3.0, 2.0, 4.0], [1.0, 
1.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 3.0, 5.0], [6.0, 4.0, 4.0]]], [[[1.0, 2.0, 4.0], [5.0, 3.0, 6.0], [7.0, 2.0, 2.0]], [[5.0, 
7.0, 2.0], [6.0, 1.0, 8.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 6.0], [2.0, 8.0, 1.0], [2.0, 3.0, 1.0]]], [[[6.0, 2.0, 3.0], [1.0, 
3.0, 2.0], [8.0, 4.0, 6.0]], [[1.0, 8.0, 7.0], [4.0, 6.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 6.0, 8.0], [2.0, 2.0, 4.0], [7.0, 5.0, 
6.0]]], [[[4.0, 7.0, 6.0], [6.0, 8.0, 2.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 1.0], [6.0, 4.0, 6.0], [2.0, 1.0, 4.0]], [[1.0, 8.0, 
8.0], [6.0, 5.0, 8.0], [1.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 6.0], [8.0, 2.0, 8.0], [1.0, 8.0, 7.0]], 
[[5.0, 1.0, 7.0], [5.0, 8.0, 7.0], [3.0, 7.0, 7.0]], [[2.0, 6.0, 1.0], [3.0, 6.0, 1.0], [8.0, 2.0, 4.0]]], [[[3.0, 2.0, 2.0], 
[2.0, 7.0, 3.0], [3.0, 6.0, 3.0]], [[2.0, 2.0, 7.0], [6.0, 1.0, 3.0], [3.0, 3.0, 8.0]], [[8.0, 1.0, 1.0], [3.0, 5.0, 3.0], 
[3.0, 2.0, 8.0]]], [[[2.0, 2.0, 6.0], [1.0, 1.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 2.0, 1.0], [4.0, 2.0, 4.0], [8.0, 1.0, 5.0]], 
[[1.0, 5.0, 1.0], [3.0, 1.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 1.0, 2.0], [2.0, 6.0, 8.0], [4.0, 2.0, 1.0]], [[5.0, 7.0, 1.0], 
[8.0, 2.0, 6.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 1.0], [5.0, 4.0, 3.0], [4.0, 3.0, 2.0]]]])
      val=numpy.array([[[[6.0]], [[4.0]]], [[[8.0]], [[1.0]]], [[[5.0]], [[7.0]]]])
      arg[:3,:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 2.0], [2.0, 6.0, 1.0], [8.0, 6.0, 3.0]], [[5.0, 5.0, 1.0], [3.0, 4.0, 4.0], [1.0, 
1.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 3.0, 5.0], [6.0, 4.0, 4.0]]], [[[1.0, 2.0, 4.0], [5.0, 8.0, 6.0], [7.0, 2.0, 2.0]], [[5.0, 
7.0, 2.0], [6.0, 1.0, 8.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 6.0], [2.0, 8.0, 1.0], [2.0, 3.0, 1.0]]], [[[6.0, 2.0, 3.0], [1.0, 
5.0, 2.0], [8.0, 4.0, 6.0]], [[1.0, 8.0, 7.0], [4.0, 7.0, 6.0], [2.0, 8.0, 7.0]], [[7.0, 6.0, 8.0], [2.0, 2.0, 4.0], [7.0, 5.0, 
6.0]]], [[[4.0, 7.0, 6.0], [6.0, 8.0, 2.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 1.0], [6.0, 4.0, 6.0], [2.0, 1.0, 4.0]], [[1.0, 8.0, 
8.0], [6.0, 5.0, 8.0], [1.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 6.0], [8.0, 6.0, 8.0], [1.0, 8.0, 7.0]], 
[[5.0, 1.0, 7.0], [5.0, 4.0, 7.0], [3.0, 7.0, 7.0]], [[2.0, 6.0, 1.0], [3.0, 6.0, 1.0], [8.0, 2.0, 4.0]]], [[[3.0, 2.0, 2.0], 
[2.0, 8.0, 3.0], [3.0, 6.0, 3.0]], [[2.0, 2.0, 7.0], [6.0, 1.0, 3.0], [3.0, 3.0, 8.0]], [[8.0, 1.0, 1.0], [3.0, 5.0, 3.0], 
[3.0, 2.0, 8.0]]], [[[2.0, 2.0, 6.0], [1.0, 5.0, 4.0], [5.0, 2.0, 8.0]], [[3.0, 2.0, 1.0], [4.0, 7.0, 4.0], [8.0, 1.0, 5.0]], 
[[1.0, 5.0, 1.0], [3.0, 1.0, 1.0], [8.0, 1.0, 3.0]]], [[[1.0, 1.0, 2.0], [2.0, 6.0, 8.0], [4.0, 2.0, 1.0]], [[5.0, 7.0, 1.0], 
[8.0, 2.0, 6.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 1.0], [5.0, 4.0, 3.0], [4.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 8.0], [3.0, 5.0, 6.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [4.0, 6.0, 7.0], [8.0, 
8.0, 1.0]], [[6.0, 8.0, 5.0], [5.0, 4.0, 8.0], [5.0, 4.0, 7.0]]], [[[8.0, 2.0, 2.0], [2.0, 7.0, 8.0], [8.0, 4.0, 6.0]], [[1.0, 
8.0, 8.0], [4.0, 5.0, 5.0], [5.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [7.0, 5.0, 7.0], [3.0, 8.0, 2.0]]], [[[8.0, 3.0, 3.0], [4.0, 
8.0, 7.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 5.0], [6.0, 6.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 5.0, 6.0], [8.0, 4.0, 5.0], [6.0, 4.0, 
2.0]]], [[[6.0, 3.0, 3.0], [8.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[2.0, 2.0, 1.0], [5.0, 2.0, 1.0], [3.0, 8.0, 4.0]], [[8.0, 6.0, 
4.0], [4.0, 6.0, 8.0], [3.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 6.0], [1.0, 1.0, 5.0], [6.0, 7.0, 3.0]], 
[[3.0, 8.0, 1.0], [6.0, 7.0, 3.0], [1.0, 5.0, 6.0]], [[7.0, 5.0, 8.0], [7.0, 2.0, 1.0], [5.0, 4.0, 7.0]]], [[[5.0, 8.0, 7.0], 
[4.0, 2.0, 4.0], [5.0, 7.0, 2.0]], [[2.0, 1.0, 3.0], [4.0, 4.0, 1.0], [6.0, 1.0, 3.0]], [[5.0, 6.0, 8.0], [6.0, 1.0, 7.0], 
[8.0, 7.0, 6.0]]], [[[2.0, 7.0, 2.0], [8.0, 1.0, 1.0], [5.0, 4.0, 6.0]], [[5.0, 5.0, 6.0], [8.0, 2.0, 3.0], [2.0, 1.0, 3.0]], 
[[6.0, 4.0, 6.0], [6.0, 1.0, 2.0], [7.0, 4.0, 7.0]]], [[[3.0, 1.0, 4.0], [1.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[6.0, 2.0, 4.0], 
[1.0, 5.0, 4.0], [1.0, 4.0, 8.0]], [[4.0, 4.0, 8.0], [7.0, 4.0, 8.0], [1.0, 5.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0]], [[6.0]]], [[[1.0]], [[7.0]]], [[[2.0]], [[1.0]]]]),self.functionspace)
      arg[:3,:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 8.0], [3.0, 4.0, 6.0], [1.0, 8.0, 1.0]], [[8.0, 1.0, 4.0], [4.0, 6.0, 7.0], [8.0, 
8.0, 1.0]], [[6.0, 8.0, 5.0], [5.0, 4.0, 8.0], [5.0, 4.0, 7.0]]], [[[8.0, 2.0, 2.0], [2.0, 1.0, 8.0], [8.0, 4.0, 6.0]], [[1.0, 
8.0, 8.0], [4.0, 7.0, 5.0], [5.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [7.0, 5.0, 7.0], [3.0, 8.0, 2.0]]], [[[8.0, 3.0, 3.0], [4.0, 
2.0, 7.0], [5.0, 4.0, 4.0]], [[5.0, 7.0, 5.0], [6.0, 1.0, 7.0], [4.0, 8.0, 3.0]], [[4.0, 5.0, 6.0], [8.0, 4.0, 5.0], [6.0, 4.0, 
2.0]]], [[[6.0, 3.0, 3.0], [8.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[2.0, 2.0, 1.0], [5.0, 2.0, 1.0], [3.0, 8.0, 4.0]], [[8.0, 6.0, 
4.0], [4.0, 6.0, 8.0], [3.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 6.0], [1.0, 4.0, 5.0], [6.0, 7.0, 3.0]], 
[[3.0, 8.0, 1.0], [6.0, 6.0, 3.0], [1.0, 5.0, 6.0]], [[7.0, 5.0, 8.0], [7.0, 2.0, 1.0], [5.0, 4.0, 7.0]]], [[[5.0, 8.0, 7.0], 
[4.0, 1.0, 4.0], [5.0, 7.0, 2.0]], [[2.0, 1.0, 3.0], [4.0, 7.0, 1.0], [6.0, 1.0, 3.0]], [[5.0, 6.0, 8.0], [6.0, 1.0, 7.0], 
[8.0, 7.0, 6.0]]], [[[2.0, 7.0, 2.0], [8.0, 2.0, 1.0], [5.0, 4.0, 6.0]], [[5.0, 5.0, 6.0], [8.0, 1.0, 3.0], [2.0, 1.0, 3.0]], 
[[6.0, 4.0, 6.0], [6.0, 1.0, 2.0], [7.0, 4.0, 7.0]]], [[[3.0, 1.0, 4.0], [1.0, 1.0, 4.0], [2.0, 8.0, 8.0]], [[6.0, 2.0, 4.0], 
[1.0, 5.0, 4.0], [1.0, 4.0, 8.0]], [[4.0, 4.0, 8.0], [7.0, 4.0, 8.0], [1.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UURR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 2.0], [3.0, 2.0, 7.0], [7.0, 3.0, 1.0]], [[7.0, 5.0, 7.0], [8.0, 6.0, 4.0], [3.0, 
5.0, 5.0]], [[5.0, 7.0, 6.0], [2.0, 1.0, 3.0], [8.0, 1.0, 6.0]]], [[[1.0, 2.0, 2.0], [5.0, 5.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 
4.0, 2.0], [6.0, 3.0, 6.0], [3.0, 6.0, 6.0]], [[7.0, 8.0, 2.0], [1.0, 6.0, 5.0], [5.0, 2.0, 1.0]]], [[[1.0, 3.0, 3.0], [6.0, 
7.0, 7.0], [5.0, 1.0, 2.0]], [[3.0, 3.0, 8.0], [2.0, 2.0, 7.0], [1.0, 8.0, 5.0]], [[4.0, 2.0, 4.0], [7.0, 3.0, 8.0], [6.0, 7.0, 
4.0]]], [[[6.0, 2.0, 1.0], [4.0, 3.0, 3.0], [1.0, 6.0, 5.0]], [[3.0, 3.0, 5.0], [2.0, 5.0, 3.0], [7.0, 3.0, 1.0]], [[4.0, 5.0, 
1.0], [4.0, 1.0, 4.0], [2.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 2.0], [5.0, 5.0, 6.0], [2.0, 8.0, 5.0]], 
[[2.0, 3.0, 1.0], [4.0, 3.0, 6.0], [5.0, 6.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 2.0, 6.0], [2.0, 8.0, 7.0]]], [[[6.0, 4.0, 7.0], 
[6.0, 1.0, 2.0], [2.0, 5.0, 5.0]], [[1.0, 3.0, 3.0], [7.0, 1.0, 5.0], [2.0, 4.0, 4.0]], [[7.0, 6.0, 5.0], [6.0, 3.0, 8.0], 
[4.0, 4.0, 5.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 2.0], [8.0, 4.0, 6.0]], [[3.0, 6.0, 7.0], [6.0, 3.0, 3.0], [1.0, 1.0, 4.0]], 
[[8.0, 1.0, 4.0], [2.0, 8.0, 1.0], [3.0, 1.0, 4.0]]], [[[6.0, 1.0, 7.0], [5.0, 6.0, 4.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 6.0], 
[7.0, 6.0, 3.0], [5.0, 3.0, 5.0]], [[3.0, 1.0, 1.0], [8.0, 1.0, 2.0], [4.0, 4.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0]], [[5.0]]], [[[8.0]], [[8.0]]], [[[6.0]], [[4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0]], 
[[6.0]]], [[[5.0]], [[6.0]]], [[[3.0]], [[4.0]]]])
      arg[:3,:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 2.0], [3.0, 3.0, 7.0], [7.0, 3.0, 1.0]], [[7.0, 5.0, 7.0], [8.0, 5.0, 4.0], [3.0, 
5.0, 5.0]], [[5.0, 7.0, 6.0], [2.0, 1.0, 3.0], [8.0, 1.0, 6.0]]], [[[1.0, 2.0, 2.0], [5.0, 8.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 
4.0, 2.0], [6.0, 8.0, 6.0], [3.0, 6.0, 6.0]], [[7.0, 8.0, 2.0], [1.0, 6.0, 5.0], [5.0, 2.0, 1.0]]], [[[1.0, 3.0, 3.0], [6.0, 
6.0, 7.0], [5.0, 1.0, 2.0]], [[3.0, 3.0, 8.0], [2.0, 4.0, 7.0], [1.0, 8.0, 5.0]], [[4.0, 2.0, 4.0], [7.0, 3.0, 8.0], [6.0, 7.0, 
4.0]]], [[[6.0, 2.0, 1.0], [4.0, 3.0, 3.0], [1.0, 6.0, 5.0]], [[3.0, 3.0, 5.0], [2.0, 5.0, 3.0], [7.0, 3.0, 1.0]], [[4.0, 5.0, 
1.0], [4.0, 1.0, 4.0], [2.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 2.0], [5.0, 6.0, 6.0], [2.0, 8.0, 5.0]], 
[[2.0, 3.0, 1.0], [4.0, 6.0, 6.0], [5.0, 6.0, 1.0]], [[3.0, 4.0, 3.0], [3.0, 2.0, 6.0], [2.0, 8.0, 7.0]]], [[[6.0, 4.0, 7.0], 
[6.0, 5.0, 2.0], [2.0, 5.0, 5.0]], [[1.0, 3.0, 3.0], [7.0, 6.0, 5.0], [2.0, 4.0, 4.0]], [[7.0, 6.0, 5.0], [6.0, 3.0, 8.0], 
[4.0, 4.0, 5.0]]], [[[3.0, 8.0, 5.0], [3.0, 3.0, 2.0], [8.0, 4.0, 6.0]], [[3.0, 6.0, 7.0], [6.0, 4.0, 3.0], [1.0, 1.0, 4.0]], 
[[8.0, 1.0, 4.0], [2.0, 8.0, 1.0], [3.0, 1.0, 4.0]]], [[[6.0, 1.0, 7.0], [5.0, 6.0, 4.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 6.0], 
[7.0, 6.0, 3.0], [5.0, 3.0, 5.0]], [[3.0, 1.0, 1.0], [8.0, 1.0, 2.0], [4.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 8.0], [8.0, 4.0, 6.0], [2.0, 4.0, 4.0]], [[3.0, 3.0, 6.0], [5.0, 1.0, 1.0], [6.0, 
3.0, 2.0]], [[4.0, 1.0, 5.0], [7.0, 7.0, 8.0], [1.0, 7.0, 7.0]]], [[[3.0, 1.0, 1.0], [5.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 
3.0, 5.0], [7.0, 6.0, 3.0], [7.0, 2.0, 7.0]], [[5.0, 5.0, 4.0], [4.0, 8.0, 5.0], [2.0, 1.0, 3.0]]], [[[4.0, 3.0, 2.0], [3.0, 
4.0, 8.0], [3.0, 7.0, 5.0]], [[5.0, 7.0, 1.0], [2.0, 8.0, 5.0], [1.0, 5.0, 3.0]], [[2.0, 5.0, 4.0], [3.0, 4.0, 3.0], [5.0, 2.0, 
4.0]]], [[[6.0, 2.0, 1.0], [4.0, 3.0, 7.0], [1.0, 2.0, 6.0]], [[8.0, 8.0, 7.0], [4.0, 3.0, 3.0], [6.0, 7.0, 6.0]], [[1.0, 5.0, 
7.0], [8.0, 5.0, 7.0], [7.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 5.0, 5.0], [2.0, 8.0, 5.0]], 
[[1.0, 3.0, 3.0], [7.0, 3.0, 3.0], [3.0, 2.0, 8.0]], [[6.0, 7.0, 2.0], [4.0, 7.0, 1.0], [4.0, 8.0, 1.0]]], [[[6.0, 7.0, 3.0], 
[6.0, 1.0, 1.0], [7.0, 7.0, 7.0]], [[7.0, 6.0, 7.0], [8.0, 5.0, 3.0], [5.0, 5.0, 4.0]], [[2.0, 3.0, 3.0], [4.0, 4.0, 7.0], 
[8.0, 7.0, 4.0]]], [[[8.0, 2.0, 4.0], [5.0, 1.0, 1.0], [4.0, 4.0, 5.0]], [[7.0, 5.0, 6.0], [7.0, 1.0, 8.0], [2.0, 4.0, 3.0]], 
[[1.0, 8.0, 5.0], [3.0, 6.0, 4.0], [7.0, 3.0, 3.0]]], [[[1.0, 1.0, 2.0], [8.0, 1.0, 3.0], [4.0, 6.0, 8.0]], [[5.0, 1.0, 5.0], 
[4.0, 2.0, 8.0], [7.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [2.0, 5.0, 4.0], [1.0, 1.0, 8.0]]]])
      val=numpy.array([[[8.0, 1.0, 5.0]], [[3.0, 2.0, 2.0]], [[7.0, 1.0, 3.0]], [[8.0, 2.0, 3.0]]])
      arg[:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [8.0, 4.0, 6.0], [2.0, 4.0, 4.0]], [[3.0, 3.0, 6.0], [8.0, 1.0, 5.0], [6.0, 
3.0, 2.0]], [[4.0, 1.0, 5.0], [7.0, 7.0, 8.0], [1.0, 7.0, 7.0]]], [[[3.0, 1.0, 1.0], [5.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[4.0, 
3.0, 5.0], [3.0, 2.0, 2.0], [7.0, 2.0, 7.0]], [[5.0, 5.0, 4.0], [4.0, 8.0, 5.0], [2.0, 1.0, 3.0]]], [[[4.0, 3.0, 2.0], [3.0, 
4.0, 8.0], [3.0, 7.0, 5.0]], [[5.0, 7.0, 1.0], [7.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[2.0, 5.0, 4.0], [3.0, 4.0, 3.0], [5.0, 2.0, 
4.0]]], [[[6.0, 2.0, 1.0], [4.0, 3.0, 7.0], [1.0, 2.0, 6.0]], [[8.0, 8.0, 7.0], [8.0, 2.0, 3.0], [6.0, 7.0, 6.0]], [[1.0, 5.0, 
7.0], [8.0, 5.0, 7.0], [7.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 6.0], [4.0, 5.0, 5.0], [2.0, 8.0, 5.0]], 
[[1.0, 3.0, 3.0], [8.0, 1.0, 5.0], [3.0, 2.0, 8.0]], [[6.0, 7.0, 2.0], [4.0, 7.0, 1.0], [4.0, 8.0, 1.0]]], [[[6.0, 7.0, 3.0], 
[6.0, 1.0, 1.0], [7.0, 7.0, 7.0]], [[7.0, 6.0, 7.0], [3.0, 2.0, 2.0], [5.0, 5.0, 4.0]], [[2.0, 3.0, 3.0], [4.0, 4.0, 7.0], 
[8.0, 7.0, 4.0]]], [[[8.0, 2.0, 4.0], [5.0, 1.0, 1.0], [4.0, 4.0, 5.0]], [[7.0, 5.0, 6.0], [7.0, 1.0, 3.0], [2.0, 4.0, 3.0]], 
[[1.0, 8.0, 5.0], [3.0, 6.0, 4.0], [7.0, 3.0, 3.0]]], [[[1.0, 1.0, 2.0], [8.0, 1.0, 3.0], [4.0, 6.0, 8.0]], [[5.0, 1.0, 5.0], 
[8.0, 2.0, 3.0], [7.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [2.0, 5.0, 4.0], [1.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 2.0], [5.0, 8.0, 5.0], [1.0, 4.0, 7.0]], [[4.0, 4.0, 4.0], [1.0, 4.0, 1.0], [7.0, 
2.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 5.0, 6.0], [4.0, 5.0, 7.0]]], [[[2.0, 1.0, 2.0], [6.0, 2.0, 4.0], [1.0, 4.0, 2.0]], [[5.0, 
5.0, 1.0], [4.0, 1.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 4.0, 3.0], [8.0, 1.0, 2.0], [8.0, 6.0, 8.0]]], [[[1.0, 1.0, 4.0], [4.0, 
3.0, 2.0], [1.0, 4.0, 3.0]], [[3.0, 6.0, 3.0], [2.0, 8.0, 6.0], [8.0, 2.0, 2.0]], [[3.0, 7.0, 5.0], [7.0, 3.0, 3.0], [8.0, 6.0, 
7.0]]], [[[1.0, 2.0, 6.0], [3.0, 7.0, 2.0], [5.0, 7.0, 5.0]], [[2.0, 1.0, 7.0], [3.0, 3.0, 5.0], [6.0, 2.0, 6.0]], [[6.0, 7.0, 
8.0], [6.0, 1.0, 1.0], [2.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 8.0], [5.0, 8.0, 5.0], [7.0, 6.0, 4.0]], 
[[2.0, 4.0, 3.0], [1.0, 5.0, 2.0], [4.0, 1.0, 5.0]], [[1.0, 6.0, 3.0], [2.0, 1.0, 5.0], [4.0, 8.0, 3.0]]], [[[7.0, 2.0, 1.0], 
[2.0, 4.0, 7.0], [3.0, 2.0, 1.0]], [[3.0, 2.0, 5.0], [6.0, 3.0, 3.0], [4.0, 1.0, 3.0]], [[6.0, 3.0, 2.0], [8.0, 8.0, 1.0], 
[4.0, 5.0, 2.0]]], [[[6.0, 7.0, 5.0], [3.0, 1.0, 4.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 2.0], [6.0, 5.0, 1.0], [8.0, 6.0, 4.0]], 
[[5.0, 7.0, 5.0], [4.0, 8.0, 7.0], [8.0, 8.0, 4.0]]], [[[6.0, 3.0, 2.0], [5.0, 7.0, 8.0], [1.0, 2.0, 8.0]], [[6.0, 2.0, 6.0], 
[2.0, 1.0, 1.0], [4.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [2.0, 3.0, 4.0], [2.0, 1.0, 6.0]]]])
      val=Data(numpy.array([[[3.0, 1.0, 8.0]], [[6.0, 8.0, 8.0]], [[1.0, 5.0, 6.0]], [[7.0, 5.0, 7.0]]]),self.functionspace)
      arg[:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 2.0], [5.0, 8.0, 5.0], [1.0, 4.0, 7.0]], [[4.0, 4.0, 4.0], [3.0, 1.0, 8.0], [7.0, 
2.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 5.0, 6.0], [4.0, 5.0, 7.0]]], [[[2.0, 1.0, 2.0], [6.0, 2.0, 4.0], [1.0, 4.0, 2.0]], [[5.0, 
5.0, 1.0], [6.0, 8.0, 8.0], [3.0, 5.0, 8.0]], [[4.0, 4.0, 3.0], [8.0, 1.0, 2.0], [8.0, 6.0, 8.0]]], [[[1.0, 1.0, 4.0], [4.0, 
3.0, 2.0], [1.0, 4.0, 3.0]], [[3.0, 6.0, 3.0], [1.0, 5.0, 6.0], [8.0, 2.0, 2.0]], [[3.0, 7.0, 5.0], [7.0, 3.0, 3.0], [8.0, 6.0, 
7.0]]], [[[1.0, 2.0, 6.0], [3.0, 7.0, 2.0], [5.0, 7.0, 5.0]], [[2.0, 1.0, 7.0], [7.0, 5.0, 7.0], [6.0, 2.0, 6.0]], [[6.0, 7.0, 
8.0], [6.0, 1.0, 1.0], [2.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 8.0], [5.0, 8.0, 5.0], [7.0, 6.0, 4.0]], 
[[2.0, 4.0, 3.0], [3.0, 1.0, 8.0], [4.0, 1.0, 5.0]], [[1.0, 6.0, 3.0], [2.0, 1.0, 5.0], [4.0, 8.0, 3.0]]], [[[7.0, 2.0, 1.0], 
[2.0, 4.0, 7.0], [3.0, 2.0, 1.0]], [[3.0, 2.0, 5.0], [6.0, 8.0, 8.0], [4.0, 1.0, 3.0]], [[6.0, 3.0, 2.0], [8.0, 8.0, 1.0], 
[4.0, 5.0, 2.0]]], [[[6.0, 7.0, 5.0], [3.0, 1.0, 4.0], [3.0, 5.0, 2.0]], [[4.0, 1.0, 2.0], [1.0, 5.0, 6.0], [8.0, 6.0, 4.0]], 
[[5.0, 7.0, 5.0], [4.0, 8.0, 7.0], [8.0, 8.0, 4.0]]], [[[6.0, 3.0, 2.0], [5.0, 7.0, 8.0], [1.0, 2.0, 8.0]], [[6.0, 2.0, 6.0], 
[7.0, 5.0, 7.0], [4.0, 4.0, 8.0]], [[7.0, 5.0, 8.0], [2.0, 3.0, 4.0], [2.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 8.0], [6.0, 7.0, 5.0], [4.0, 3.0, 3.0]], [[2.0, 7.0, 5.0], [2.0, 8.0, 1.0], [4.0, 
8.0, 8.0]], [[3.0, 6.0, 6.0], [3.0, 7.0, 3.0], [6.0, 7.0, 4.0]]], [[[3.0, 4.0, 1.0], [6.0, 6.0, 8.0], [5.0, 5.0, 6.0]], [[6.0, 
1.0, 7.0], [8.0, 6.0, 8.0], [1.0, 5.0, 3.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 4.0], [4.0, 1.0, 2.0]]], [[[2.0, 2.0, 5.0], [1.0, 
4.0, 7.0], [8.0, 4.0, 5.0]], [[2.0, 5.0, 7.0], [1.0, 5.0, 8.0], [1.0, 3.0, 4.0]], [[3.0, 4.0, 3.0], [8.0, 4.0, 4.0], [8.0, 6.0, 
7.0]]], [[[8.0, 3.0, 8.0], [1.0, 4.0, 6.0], [2.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], [1.0, 2.0, 3.0], [7.0, 5.0, 1.0]], [[7.0, 8.0, 
4.0], [5.0, 2.0, 3.0], [4.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 4.0], [8.0, 4.0, 5.0], [3.0, 2.0, 2.0]], 
[[5.0, 8.0, 8.0], [4.0, 3.0, 6.0], [5.0, 5.0, 2.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 6.0], [8.0, 4.0, 4.0]]], [[[7.0, 4.0, 7.0], 
[3.0, 8.0, 7.0], [3.0, 5.0, 8.0]], [[8.0, 4.0, 7.0], [4.0, 2.0, 6.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 3.0], [8.0, 5.0, 7.0], 
[5.0, 8.0, 2.0]]], [[[8.0, 2.0, 2.0], [4.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [4.0, 8.0, 4.0], [6.0, 3.0, 2.0]], 
[[5.0, 5.0, 4.0], [6.0, 1.0, 7.0], [5.0, 2.0, 7.0]]], [[[8.0, 6.0, 7.0], [8.0, 2.0, 2.0], [8.0, 6.0, 7.0]], [[1.0, 3.0, 5.0], 
[2.0, 3.0, 2.0], [1.0, 7.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 6.0, 5.0], [1.0, 7.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 6.0, 7.0]], [[2.0, 5.0, 5.0]], [[1.0, 8.0, 4.0]], [[7.0, 6.0, 
2.0]]])+(1.-msk_val)*numpy.array([[[4.0, 1.0, 8.0]], [[2.0, 5.0, 1.0]], [[1.0, 4.0, 8.0]], [[2.0, 4.0, 3.0]]])
      arg[:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 8.0], [6.0, 7.0, 5.0], [4.0, 3.0, 3.0]], [[2.0, 7.0, 5.0], [7.0, 6.0, 7.0], [4.0, 
8.0, 8.0]], [[3.0, 6.0, 6.0], [3.0, 7.0, 3.0], [6.0, 7.0, 4.0]]], [[[3.0, 4.0, 1.0], [6.0, 6.0, 8.0], [5.0, 5.0, 6.0]], [[6.0, 
1.0, 7.0], [2.0, 5.0, 5.0], [1.0, 5.0, 3.0]], [[1.0, 1.0, 5.0], [5.0, 2.0, 4.0], [4.0, 1.0, 2.0]]], [[[2.0, 2.0, 5.0], [1.0, 
4.0, 7.0], [8.0, 4.0, 5.0]], [[2.0, 5.0, 7.0], [1.0, 8.0, 4.0], [1.0, 3.0, 4.0]], [[3.0, 4.0, 3.0], [8.0, 4.0, 4.0], [8.0, 6.0, 
7.0]]], [[[8.0, 3.0, 8.0], [1.0, 4.0, 6.0], [2.0, 1.0, 5.0]], [[2.0, 6.0, 2.0], [7.0, 6.0, 2.0], [7.0, 5.0, 1.0]], [[7.0, 8.0, 
4.0], [5.0, 2.0, 3.0], [4.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 4.0], [8.0, 4.0, 5.0], [3.0, 2.0, 2.0]], 
[[5.0, 8.0, 8.0], [4.0, 1.0, 8.0], [5.0, 5.0, 2.0]], [[8.0, 4.0, 4.0], [5.0, 5.0, 6.0], [8.0, 4.0, 4.0]]], [[[7.0, 4.0, 7.0], 
[3.0, 8.0, 7.0], [3.0, 5.0, 8.0]], [[8.0, 4.0, 7.0], [2.0, 5.0, 1.0], [2.0, 6.0, 4.0]], [[8.0, 7.0, 3.0], [8.0, 5.0, 7.0], 
[5.0, 8.0, 2.0]]], [[[8.0, 2.0, 2.0], [4.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[1.0, 1.0, 5.0], [1.0, 4.0, 8.0], [6.0, 3.0, 2.0]], 
[[5.0, 5.0, 4.0], [6.0, 1.0, 7.0], [5.0, 2.0, 7.0]]], [[[8.0, 6.0, 7.0], [8.0, 2.0, 2.0], [8.0, 6.0, 7.0]], [[1.0, 3.0, 5.0], 
[2.0, 4.0, 3.0], [1.0, 7.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 6.0, 5.0], [1.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 8.0], [4.0, 7.0, 7.0], [1.0, 3.0, 7.0]], [[3.0, 5.0, 5.0], [7.0, 1.0, 7.0], [4.0, 
2.0, 2.0]], [[7.0, 6.0, 1.0], [7.0, 3.0, 3.0], [8.0, 3.0, 6.0]]], [[[6.0, 1.0, 1.0], [6.0, 5.0, 7.0], [8.0, 1.0, 7.0]], [[4.0, 
5.0, 5.0], [2.0, 4.0, 1.0], [5.0, 3.0, 8.0]], [[3.0, 5.0, 7.0], [2.0, 3.0, 5.0], [4.0, 1.0, 3.0]]], [[[3.0, 5.0, 2.0], [5.0, 
1.0, 5.0], [4.0, 2.0, 7.0]], [[4.0, 6.0, 8.0], [7.0, 6.0, 6.0], [5.0, 1.0, 4.0]], [[6.0, 8.0, 1.0], [3.0, 2.0, 2.0], [1.0, 4.0, 
5.0]]], [[[6.0, 2.0, 7.0], [8.0, 7.0, 1.0], [2.0, 5.0, 1.0]], [[1.0, 2.0, 8.0], [4.0, 7.0, 1.0], [4.0, 7.0, 5.0]], [[3.0, 7.0, 
8.0], [7.0, 4.0, 3.0], [6.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 4.0], [4.0, 4.0, 5.0], [4.0, 6.0, 7.0]], 
[[8.0, 7.0, 6.0], [1.0, 8.0, 8.0], [3.0, 2.0, 1.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 2.0], [5.0, 2.0, 3.0]]], [[[6.0, 3.0, 1.0], 
[3.0, 7.0, 4.0], [3.0, 6.0, 7.0]], [[6.0, 4.0, 8.0], [6.0, 3.0, 3.0], [8.0, 8.0, 2.0]], [[6.0, 5.0, 1.0], [2.0, 3.0, 4.0], 
[2.0, 3.0, 8.0]]], [[[6.0, 6.0, 6.0], [4.0, 6.0, 8.0], [1.0, 1.0, 7.0]], [[6.0, 1.0, 2.0], [5.0, 8.0, 2.0], [4.0, 7.0, 4.0]], 
[[4.0, 1.0, 1.0], [4.0, 6.0, 3.0], [8.0, 6.0, 8.0]]], [[[5.0, 1.0, 4.0], [8.0, 6.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 4.0, 1.0], 
[4.0, 7.0, 5.0], [7.0, 8.0, 4.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 1.0], [7.0, 3.0, 3.0]]]])
      val=numpy.array([[[1.0, 1.0, 8.0], [4.0, 1.0, 2.0]], [[8.0, 6.0, 6.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 5.0], [3.0, 4.0, 
1.0]]])
      arg[:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 8.0], [4.0, 7.0, 7.0], [1.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 1.0, 2.0], [4.0, 
2.0, 2.0]], [[7.0, 6.0, 1.0], [7.0, 3.0, 3.0], [8.0, 3.0, 6.0]]], [[[6.0, 1.0, 1.0], [6.0, 5.0, 7.0], [8.0, 1.0, 7.0]], [[8.0, 
6.0, 6.0], [3.0, 5.0, 4.0], [5.0, 3.0, 8.0]], [[3.0, 5.0, 7.0], [2.0, 3.0, 5.0], [4.0, 1.0, 3.0]]], [[[3.0, 5.0, 2.0], [5.0, 
1.0, 5.0], [4.0, 2.0, 7.0]], [[5.0, 1.0, 5.0], [3.0, 4.0, 1.0], [5.0, 1.0, 4.0]], [[6.0, 8.0, 1.0], [3.0, 2.0, 2.0], [1.0, 4.0, 
5.0]]], [[[6.0, 2.0, 7.0], [8.0, 7.0, 1.0], [2.0, 5.0, 1.0]], [[1.0, 2.0, 8.0], [4.0, 7.0, 1.0], [4.0, 7.0, 5.0]], [[3.0, 7.0, 
8.0], [7.0, 4.0, 3.0], [6.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 4.0], [4.0, 4.0, 5.0], [4.0, 6.0, 7.0]], 
[[1.0, 1.0, 8.0], [4.0, 1.0, 2.0], [3.0, 2.0, 1.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 2.0], [5.0, 2.0, 3.0]]], [[[6.0, 3.0, 1.0], 
[3.0, 7.0, 4.0], [3.0, 6.0, 7.0]], [[8.0, 6.0, 6.0], [3.0, 5.0, 4.0], [8.0, 8.0, 2.0]], [[6.0, 5.0, 1.0], [2.0, 3.0, 4.0], 
[2.0, 3.0, 8.0]]], [[[6.0, 6.0, 6.0], [4.0, 6.0, 8.0], [1.0, 1.0, 7.0]], [[5.0, 1.0, 5.0], [3.0, 4.0, 1.0], [4.0, 7.0, 4.0]], 
[[4.0, 1.0, 1.0], [4.0, 6.0, 3.0], [8.0, 6.0, 8.0]]], [[[5.0, 1.0, 4.0], [8.0, 6.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 4.0, 1.0], 
[4.0, 7.0, 5.0], [7.0, 8.0, 4.0]], [[6.0, 3.0, 5.0], [6.0, 7.0, 1.0], [7.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 5.0], [3.0, 2.0, 5.0], [1.0, 3.0, 6.0]], [[5.0, 4.0, 3.0], [6.0, 1.0, 1.0], [4.0, 
3.0, 8.0]], [[1.0, 8.0, 8.0], [4.0, 3.0, 3.0], [4.0, 5.0, 2.0]]], [[[3.0, 7.0, 6.0], [4.0, 4.0, 4.0], [4.0, 2.0, 3.0]], [[8.0, 
5.0, 8.0], [3.0, 7.0, 1.0], [5.0, 8.0, 8.0]], [[2.0, 2.0, 6.0], [4.0, 2.0, 4.0], [3.0, 3.0, 2.0]]], [[[6.0, 6.0, 3.0], [1.0, 
6.0, 4.0], [5.0, 3.0, 5.0]], [[7.0, 2.0, 6.0], [8.0, 3.0, 1.0], [7.0, 6.0, 6.0]], [[6.0, 2.0, 7.0], [1.0, 3.0, 2.0], [5.0, 5.0, 
3.0]]], [[[8.0, 7.0, 6.0], [5.0, 7.0, 5.0], [1.0, 7.0, 3.0]], [[8.0, 1.0, 4.0], [6.0, 7.0, 1.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 
3.0], [3.0, 6.0, 6.0], [8.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 2.0], [8.0, 2.0, 2.0], [6.0, 3.0, 1.0]], 
[[5.0, 7.0, 5.0], [6.0, 3.0, 3.0], [7.0, 4.0, 1.0]], [[2.0, 8.0, 1.0], [1.0, 7.0, 2.0], [2.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], 
[6.0, 8.0, 2.0], [1.0, 8.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 6.0, 4.0], [8.0, 7.0, 3.0]], [[1.0, 8.0, 3.0], [1.0, 1.0, 2.0], 
[7.0, 1.0, 6.0]]], [[[6.0, 7.0, 5.0], [5.0, 7.0, 4.0], [2.0, 8.0, 3.0]], [[8.0, 5.0, 2.0], [5.0, 8.0, 6.0], [2.0, 1.0, 3.0]], 
[[8.0, 1.0, 1.0], [8.0, 5.0, 8.0], [4.0, 6.0, 2.0]]], [[[7.0, 7.0, 4.0], [5.0, 4.0, 8.0], [5.0, 4.0, 3.0]], [[6.0, 1.0, 5.0], 
[6.0, 3.0, 3.0], [4.0, 5.0, 5.0]], [[7.0, 3.0, 5.0], [7.0, 5.0, 2.0], [4.0, 1.0, 3.0]]]])
      val=Data(numpy.array([[[1.0, 3.0, 7.0], [1.0, 5.0, 2.0]], [[4.0, 7.0, 2.0], [2.0, 5.0, 1.0]], [[4.0, 4.0, 5.0], [4.0, 
5.0, 2.0]]]),self.functionspace)
      arg[:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 5.0], [3.0, 2.0, 5.0], [1.0, 3.0, 6.0]], [[1.0, 3.0, 7.0], [1.0, 5.0, 2.0], [4.0, 
3.0, 8.0]], [[1.0, 8.0, 8.0], [4.0, 3.0, 3.0], [4.0, 5.0, 2.0]]], [[[3.0, 7.0, 6.0], [4.0, 4.0, 4.0], [4.0, 2.0, 3.0]], [[4.0, 
7.0, 2.0], [2.0, 5.0, 1.0], [5.0, 8.0, 8.0]], [[2.0, 2.0, 6.0], [4.0, 2.0, 4.0], [3.0, 3.0, 2.0]]], [[[6.0, 6.0, 3.0], [1.0, 
6.0, 4.0], [5.0, 3.0, 5.0]], [[4.0, 4.0, 5.0], [4.0, 5.0, 2.0], [7.0, 6.0, 6.0]], [[6.0, 2.0, 7.0], [1.0, 3.0, 2.0], [5.0, 5.0, 
3.0]]], [[[8.0, 7.0, 6.0], [5.0, 7.0, 5.0], [1.0, 7.0, 3.0]], [[8.0, 1.0, 4.0], [6.0, 7.0, 1.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 
3.0], [3.0, 6.0, 6.0], [8.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 2.0], [8.0, 2.0, 2.0], [6.0, 3.0, 1.0]], 
[[1.0, 3.0, 7.0], [1.0, 5.0, 2.0], [7.0, 4.0, 1.0]], [[2.0, 8.0, 1.0], [1.0, 7.0, 2.0], [2.0, 2.0, 4.0]]], [[[3.0, 3.0, 1.0], 
[6.0, 8.0, 2.0], [1.0, 8.0, 1.0]], [[4.0, 7.0, 2.0], [2.0, 5.0, 1.0], [8.0, 7.0, 3.0]], [[1.0, 8.0, 3.0], [1.0, 1.0, 2.0], 
[7.0, 1.0, 6.0]]], [[[6.0, 7.0, 5.0], [5.0, 7.0, 4.0], [2.0, 8.0, 3.0]], [[4.0, 4.0, 5.0], [4.0, 5.0, 2.0], [2.0, 1.0, 3.0]], 
[[8.0, 1.0, 1.0], [8.0, 5.0, 8.0], [4.0, 6.0, 2.0]]], [[[7.0, 7.0, 4.0], [5.0, 4.0, 8.0], [5.0, 4.0, 3.0]], [[6.0, 1.0, 5.0], 
[6.0, 3.0, 3.0], [4.0, 5.0, 5.0]], [[7.0, 3.0, 5.0], [7.0, 5.0, 2.0], [4.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 3.0], [1.0, 1.0, 5.0], [8.0, 1.0, 5.0]], [[1.0, 5.0, 2.0], [1.0, 2.0, 2.0], [1.0, 
2.0, 7.0]], [[5.0, 7.0, 2.0], [5.0, 8.0, 7.0], [2.0, 1.0, 4.0]]], [[[8.0, 3.0, 7.0], [5.0, 4.0, 7.0], [6.0, 4.0, 4.0]], [[2.0, 
3.0, 5.0], [3.0, 1.0, 2.0], [3.0, 5.0, 7.0]], [[1.0, 1.0, 5.0], [5.0, 3.0, 1.0], [2.0, 5.0, 5.0]]], [[[5.0, 4.0, 4.0], [1.0, 
3.0, 5.0], [7.0, 1.0, 6.0]], [[7.0, 7.0, 5.0], [7.0, 3.0, 7.0], [1.0, 1.0, 6.0]], [[6.0, 6.0, 5.0], [4.0, 8.0, 6.0], [5.0, 4.0, 
6.0]]], [[[6.0, 6.0, 1.0], [3.0, 1.0, 4.0], [1.0, 7.0, 4.0]], [[5.0, 3.0, 8.0], [6.0, 6.0, 8.0], [6.0, 2.0, 5.0]], [[7.0, 7.0, 
4.0], [4.0, 8.0, 3.0], [8.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 4.0], [1.0, 7.0, 5.0], [7.0, 8.0, 5.0]], 
[[7.0, 7.0, 5.0], [5.0, 4.0, 4.0], [8.0, 3.0, 3.0]], [[8.0, 1.0, 7.0], [4.0, 2.0, 7.0], [7.0, 5.0, 1.0]]], [[[4.0, 7.0, 1.0], 
[2.0, 4.0, 4.0], [1.0, 8.0, 6.0]], [[2.0, 1.0, 5.0], [8.0, 4.0, 7.0], [2.0, 8.0, 8.0]], [[1.0, 7.0, 3.0], [3.0, 5.0, 2.0], 
[8.0, 6.0, 7.0]]], [[[2.0, 3.0, 7.0], [5.0, 7.0, 4.0], [5.0, 1.0, 3.0]], [[8.0, 2.0, 8.0], [8.0, 2.0, 1.0], [5.0, 8.0, 6.0]], 
[[1.0, 4.0, 2.0], [6.0, 5.0, 7.0], [6.0, 2.0, 6.0]]], [[[2.0, 5.0, 3.0], [6.0, 7.0, 1.0], [6.0, 3.0, 6.0]], [[8.0, 2.0, 4.0], 
[7.0, 7.0, 4.0], [8.0, 3.0, 7.0]], [[2.0, 7.0, 6.0], [4.0, 6.0, 7.0], [8.0, 2.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 6.0, 4.0], [8.0, 4.0, 8.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 4.0]], [[7.0, 6.0, 2.0], [1.0, 
4.0, 7.0]]])+(1.-msk_val)*numpy.array([[[3.0, 1.0, 1.0], [3.0, 2.0, 7.0]], [[8.0, 3.0, 5.0], [6.0, 1.0, 1.0]], [[7.0, 5.0, 
6.0], [3.0, 4.0, 3.0]]])
      arg[:3,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 3.0], [1.0, 1.0, 5.0], [8.0, 1.0, 5.0]], [[1.0, 6.0, 4.0], [8.0, 4.0, 8.0], [1.0, 
2.0, 7.0]], [[5.0, 7.0, 2.0], [5.0, 8.0, 7.0], [2.0, 1.0, 4.0]]], [[[8.0, 3.0, 7.0], [5.0, 4.0, 7.0], [6.0, 4.0, 4.0]], [[3.0, 
2.0, 5.0], [8.0, 8.0, 4.0], [3.0, 5.0, 7.0]], [[1.0, 1.0, 5.0], [5.0, 3.0, 1.0], [2.0, 5.0, 5.0]]], [[[5.0, 4.0, 4.0], [1.0, 
3.0, 5.0], [7.0, 1.0, 6.0]], [[7.0, 6.0, 2.0], [1.0, 4.0, 7.0], [1.0, 1.0, 6.0]], [[6.0, 6.0, 5.0], [4.0, 8.0, 6.0], [5.0, 4.0, 
6.0]]], [[[6.0, 6.0, 1.0], [3.0, 1.0, 4.0], [1.0, 7.0, 4.0]], [[5.0, 3.0, 8.0], [6.0, 6.0, 8.0], [6.0, 2.0, 5.0]], [[7.0, 7.0, 
4.0], [4.0, 8.0, 3.0], [8.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 4.0], [1.0, 7.0, 5.0], [7.0, 8.0, 5.0]], 
[[3.0, 1.0, 1.0], [3.0, 2.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 1.0, 7.0], [4.0, 2.0, 7.0], [7.0, 5.0, 1.0]]], [[[4.0, 7.0, 1.0], 
[2.0, 4.0, 4.0], [1.0, 8.0, 6.0]], [[8.0, 3.0, 5.0], [6.0, 1.0, 1.0], [2.0, 8.0, 8.0]], [[1.0, 7.0, 3.0], [3.0, 5.0, 2.0], 
[8.0, 6.0, 7.0]]], [[[2.0, 3.0, 7.0], [5.0, 7.0, 4.0], [5.0, 1.0, 3.0]], [[7.0, 5.0, 6.0], [3.0, 4.0, 3.0], [5.0, 8.0, 6.0]], 
[[1.0, 4.0, 2.0], [6.0, 5.0, 7.0], [6.0, 2.0, 6.0]]], [[[2.0, 5.0, 3.0], [6.0, 7.0, 1.0], [6.0, 3.0, 6.0]], [[8.0, 2.0, 4.0], 
[7.0, 7.0, 4.0], [8.0, 3.0, 7.0]], [[2.0, 7.0, 6.0], [4.0, 6.0, 7.0], [8.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 3.0], [6.0, 7.0, 8.0], [3.0, 6.0, 3.0]], [[7.0, 1.0, 1.0], [5.0, 5.0, 7.0], [5.0, 
1.0, 5.0]], [[2.0, 6.0, 3.0], [7.0, 4.0, 4.0], [2.0, 3.0, 2.0]]], [[[7.0, 3.0, 2.0], [4.0, 3.0, 2.0], [7.0, 2.0, 2.0]], [[4.0, 
6.0, 6.0], [8.0, 8.0, 8.0], [8.0, 3.0, 8.0]], [[8.0, 7.0, 1.0], [3.0, 3.0, 5.0], [1.0, 1.0, 4.0]]], [[[6.0, 8.0, 6.0], [2.0, 
6.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 5.0, 4.0], [5.0, 3.0, 2.0], [1.0, 2.0, 8.0]], [[6.0, 8.0, 5.0], [2.0, 4.0, 2.0], [5.0, 3.0, 
8.0]]], [[[7.0, 2.0, 3.0], [1.0, 1.0, 8.0], [3.0, 3.0, 6.0]], [[1.0, 1.0, 4.0], [5.0, 8.0, 4.0], [4.0, 7.0, 8.0]], [[3.0, 3.0, 
7.0], [2.0, 1.0, 7.0], [5.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 1.0, 1.0], [5.0, 4.0, 5.0], [7.0, 1.0, 6.0]], 
[[3.0, 8.0, 6.0], [2.0, 8.0, 7.0], [5.0, 5.0, 4.0]], [[5.0, 5.0, 1.0], [2.0, 5.0, 6.0], [5.0, 2.0, 2.0]]], [[[5.0, 1.0, 1.0], 
[8.0, 2.0, 4.0], [5.0, 4.0, 3.0]], [[8.0, 3.0, 3.0], [1.0, 8.0, 3.0], [5.0, 2.0, 4.0]], [[2.0, 4.0, 2.0], [2.0, 8.0, 1.0], 
[4.0, 1.0, 5.0]]], [[[8.0, 4.0, 2.0], [6.0, 3.0, 4.0], [8.0, 7.0, 1.0]], [[2.0, 7.0, 8.0], [4.0, 5.0, 4.0], [4.0, 4.0, 7.0]], 
[[1.0, 3.0, 6.0], [1.0, 8.0, 8.0], [3.0, 1.0, 8.0]]], [[[2.0, 8.0, 8.0], [6.0, 7.0, 7.0], [3.0, 4.0, 4.0]], [[5.0, 4.0, 3.0], 
[1.0, 7.0, 2.0], [7.0, 5.0, 7.0]], [[4.0, 2.0, 5.0], [8.0, 7.0, 3.0], [2.0, 6.0, 7.0]]]])
      val=numpy.array([[[[1.0, 5.0, 3.0], [7.0, 2.0, 5.0]]], [[[8.0, 6.0, 4.0], [2.0, 1.0, 6.0]]]])
      arg[1:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 3.0], [6.0, 7.0, 8.0], [3.0, 6.0, 3.0]], [[7.0, 1.0, 1.0], [5.0, 5.0, 7.0], [5.0, 
1.0, 5.0]], [[2.0, 6.0, 3.0], [7.0, 4.0, 4.0], [2.0, 3.0, 2.0]]], [[[7.0, 3.0, 2.0], [4.0, 3.0, 2.0], [7.0, 2.0, 2.0]], [[4.0, 
6.0, 6.0], [8.0, 8.0, 8.0], [8.0, 3.0, 8.0]], [[1.0, 5.0, 3.0], [7.0, 2.0, 5.0], [1.0, 1.0, 4.0]]], [[[6.0, 8.0, 6.0], [2.0, 
6.0, 4.0], [7.0, 4.0, 4.0]], [[3.0, 5.0, 4.0], [5.0, 3.0, 2.0], [1.0, 2.0, 8.0]], [[8.0, 6.0, 4.0], [2.0, 1.0, 6.0], [5.0, 3.0, 
8.0]]], [[[7.0, 2.0, 3.0], [1.0, 1.0, 8.0], [3.0, 3.0, 6.0]], [[1.0, 1.0, 4.0], [5.0, 8.0, 4.0], [4.0, 7.0, 8.0]], [[3.0, 3.0, 
7.0], [2.0, 1.0, 7.0], [5.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 1.0], [5.0, 4.0, 5.0], [7.0, 1.0, 6.0]], 
[[3.0, 8.0, 6.0], [2.0, 8.0, 7.0], [5.0, 5.0, 4.0]], [[5.0, 5.0, 1.0], [2.0, 5.0, 6.0], [5.0, 2.0, 2.0]]], [[[5.0, 1.0, 1.0], 
[8.0, 2.0, 4.0], [5.0, 4.0, 3.0]], [[8.0, 3.0, 3.0], [1.0, 8.0, 3.0], [5.0, 2.0, 4.0]], [[1.0, 5.0, 3.0], [7.0, 2.0, 5.0], 
[4.0, 1.0, 5.0]]], [[[8.0, 4.0, 2.0], [6.0, 3.0, 4.0], [8.0, 7.0, 1.0]], [[2.0, 7.0, 8.0], [4.0, 5.0, 4.0], [4.0, 4.0, 7.0]], 
[[8.0, 6.0, 4.0], [2.0, 1.0, 6.0], [3.0, 1.0, 8.0]]], [[[2.0, 8.0, 8.0], [6.0, 7.0, 7.0], [3.0, 4.0, 4.0]], [[5.0, 4.0, 3.0], 
[1.0, 7.0, 2.0], [7.0, 5.0, 7.0]], [[4.0, 2.0, 5.0], [8.0, 7.0, 3.0], [2.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 2.0], [6.0, 6.0, 2.0], [3.0, 7.0, 1.0]], [[4.0, 5.0, 6.0], [6.0, 8.0, 2.0], [5.0, 
4.0, 6.0]], [[8.0, 5.0, 5.0], [4.0, 8.0, 3.0], [6.0, 7.0, 7.0]]], [[[3.0, 7.0, 5.0], [8.0, 6.0, 5.0], [2.0, 3.0, 4.0]], [[4.0, 
8.0, 4.0], [8.0, 4.0, 3.0], [7.0, 5.0, 2.0]], [[6.0, 1.0, 1.0], [7.0, 2.0, 5.0], [4.0, 4.0, 1.0]]], [[[3.0, 8.0, 1.0], [8.0, 
6.0, 5.0], [5.0, 5.0, 7.0]], [[4.0, 2.0, 7.0], [1.0, 6.0, 7.0], [1.0, 2.0, 6.0]], [[4.0, 1.0, 3.0], [4.0, 5.0, 4.0], [1.0, 6.0, 
2.0]]], [[[7.0, 6.0, 8.0], [4.0, 6.0, 7.0], [3.0, 1.0, 4.0]], [[4.0, 6.0, 7.0], [5.0, 8.0, 4.0], [7.0, 4.0, 5.0]], [[2.0, 7.0, 
4.0], [6.0, 6.0, 3.0], [1.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 6.0], [7.0, 6.0, 1.0], [4.0, 1.0, 3.0]], 
[[2.0, 1.0, 2.0], [7.0, 5.0, 5.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 7.0], [1.0, 8.0, 6.0], [4.0, 5.0, 8.0]]], [[[3.0, 8.0, 4.0], 
[2.0, 2.0, 8.0], [4.0, 4.0, 8.0]], [[7.0, 3.0, 5.0], [8.0, 8.0, 7.0], [4.0, 2.0, 8.0]], [[3.0, 6.0, 4.0], [3.0, 1.0, 2.0], 
[4.0, 7.0, 2.0]]], [[[3.0, 6.0, 6.0], [3.0, 1.0, 3.0], [4.0, 2.0, 5.0]], [[6.0, 4.0, 1.0], [2.0, 7.0, 4.0], [4.0, 1.0, 6.0]], 
[[2.0, 3.0, 1.0], [7.0, 8.0, 3.0], [6.0, 2.0, 2.0]]], [[[7.0, 3.0, 8.0], [4.0, 7.0, 5.0], [4.0, 6.0, 7.0]], [[4.0, 7.0, 1.0], 
[8.0, 3.0, 8.0], [4.0, 4.0, 1.0]], [[1.0, 7.0, 8.0], [5.0, 5.0, 6.0], [6.0, 5.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0, 3.0, 6.0], [6.0, 2.0, 2.0]]], [[[8.0, 4.0, 5.0], [5.0, 4.0, 6.0]]]]),self.functionspace)
      arg[1:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 2.0], [6.0, 6.0, 2.0], [3.0, 7.0, 1.0]], [[4.0, 5.0, 6.0], [6.0, 8.0, 2.0], [5.0, 
4.0, 6.0]], [[8.0, 5.0, 5.0], [4.0, 8.0, 3.0], [6.0, 7.0, 7.0]]], [[[3.0, 7.0, 5.0], [8.0, 6.0, 5.0], [2.0, 3.0, 4.0]], [[4.0, 
8.0, 4.0], [8.0, 4.0, 3.0], [7.0, 5.0, 2.0]], [[4.0, 3.0, 6.0], [6.0, 2.0, 2.0], [4.0, 4.0, 1.0]]], [[[3.0, 8.0, 1.0], [8.0, 
6.0, 5.0], [5.0, 5.0, 7.0]], [[4.0, 2.0, 7.0], [1.0, 6.0, 7.0], [1.0, 2.0, 6.0]], [[8.0, 4.0, 5.0], [5.0, 4.0, 6.0], [1.0, 6.0, 
2.0]]], [[[7.0, 6.0, 8.0], [4.0, 6.0, 7.0], [3.0, 1.0, 4.0]], [[4.0, 6.0, 7.0], [5.0, 8.0, 4.0], [7.0, 4.0, 5.0]], [[2.0, 7.0, 
4.0], [6.0, 6.0, 3.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 6.0], [7.0, 6.0, 1.0], [4.0, 1.0, 3.0]], 
[[2.0, 1.0, 2.0], [7.0, 5.0, 5.0], [6.0, 3.0, 1.0]], [[5.0, 1.0, 7.0], [1.0, 8.0, 6.0], [4.0, 5.0, 8.0]]], [[[3.0, 8.0, 4.0], 
[2.0, 2.0, 8.0], [4.0, 4.0, 8.0]], [[7.0, 3.0, 5.0], [8.0, 8.0, 7.0], [4.0, 2.0, 8.0]], [[4.0, 3.0, 6.0], [6.0, 2.0, 2.0], 
[4.0, 7.0, 2.0]]], [[[3.0, 6.0, 6.0], [3.0, 1.0, 3.0], [4.0, 2.0, 5.0]], [[6.0, 4.0, 1.0], [2.0, 7.0, 4.0], [4.0, 1.0, 6.0]], 
[[8.0, 4.0, 5.0], [5.0, 4.0, 6.0], [6.0, 2.0, 2.0]]], [[[7.0, 3.0, 8.0], [4.0, 7.0, 5.0], [4.0, 6.0, 7.0]], [[4.0, 7.0, 1.0], 
[8.0, 3.0, 8.0], [4.0, 4.0, 1.0]], [[1.0, 7.0, 8.0], [5.0, 5.0, 6.0], [6.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 3.0], [3.0, 3.0, 2.0], [8.0, 2.0, 5.0]], [[8.0, 1.0, 5.0], [7.0, 2.0, 4.0], [3.0, 
1.0, 4.0]], [[8.0, 1.0, 1.0], [7.0, 5.0, 3.0], [6.0, 3.0, 3.0]]], [[[2.0, 7.0, 6.0], [2.0, 6.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 
6.0, 1.0], [5.0, 8.0, 2.0], [3.0, 1.0, 5.0]], [[4.0, 1.0, 3.0], [8.0, 6.0, 2.0], [4.0, 4.0, 5.0]]], [[[7.0, 4.0, 3.0], [1.0, 
6.0, 7.0], [7.0, 1.0, 3.0]], [[2.0, 1.0, 7.0], [1.0, 3.0, 7.0], [7.0, 7.0, 8.0]], [[2.0, 2.0, 6.0], [8.0, 6.0, 3.0], [5.0, 8.0, 
3.0]]], [[[5.0, 3.0, 4.0], [1.0, 2.0, 7.0], [5.0, 6.0, 8.0]], [[2.0, 7.0, 3.0], [6.0, 2.0, 8.0], [2.0, 3.0, 4.0]], [[7.0, 2.0, 
6.0], [2.0, 3.0, 3.0], [4.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 5.0], [6.0, 4.0, 8.0], [3.0, 3.0, 3.0]], 
[[3.0, 3.0, 8.0], [6.0, 7.0, 2.0], [2.0, 6.0, 4.0]], [[4.0, 2.0, 2.0], [5.0, 8.0, 3.0], [2.0, 8.0, 1.0]]], [[[1.0, 2.0, 5.0], 
[8.0, 4.0, 8.0], [7.0, 2.0, 7.0]], [[6.0, 2.0, 1.0], [2.0, 8.0, 6.0], [7.0, 8.0, 4.0]], [[2.0, 4.0, 2.0], [6.0, 4.0, 4.0], 
[2.0, 6.0, 5.0]]], [[[8.0, 5.0, 7.0], [4.0, 4.0, 7.0], [4.0, 7.0, 2.0]], [[4.0, 5.0, 1.0], [6.0, 6.0, 1.0], [4.0, 3.0, 1.0]], 
[[7.0, 6.0, 5.0], [6.0, 6.0, 8.0], [2.0, 7.0, 2.0]]], [[[3.0, 6.0, 7.0], [6.0, 8.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 1.0, 5.0], 
[1.0, 6.0, 1.0], [7.0, 1.0, 6.0]], [[3.0, 1.0, 1.0], [5.0, 6.0, 5.0], [7.0, 5.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 1.0, 8.0], [8.0, 5.0, 3.0]]], [[[2.0, 4.0, 7.0], [6.0, 4.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 8.0, 7.0], [5.0, 1.0, 4.0]]], [[[4.0, 6.0, 6.0], [6.0, 8.0, 2.0]]]])
      arg[1:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 3.0], [3.0, 3.0, 2.0], [8.0, 2.0, 5.0]], [[8.0, 1.0, 5.0], [7.0, 2.0, 4.0], [3.0, 
1.0, 4.0]], [[8.0, 1.0, 1.0], [7.0, 5.0, 3.0], [6.0, 3.0, 3.0]]], [[[2.0, 7.0, 6.0], [2.0, 6.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 
6.0, 1.0], [5.0, 8.0, 2.0], [3.0, 1.0, 5.0]], [[4.0, 1.0, 8.0], [8.0, 5.0, 3.0], [4.0, 4.0, 5.0]]], [[[7.0, 4.0, 3.0], [1.0, 
6.0, 7.0], [7.0, 1.0, 3.0]], [[2.0, 1.0, 7.0], [1.0, 3.0, 7.0], [7.0, 7.0, 8.0]], [[2.0, 4.0, 7.0], [6.0, 4.0, 8.0], [5.0, 8.0, 
3.0]]], [[[5.0, 3.0, 4.0], [1.0, 2.0, 7.0], [5.0, 6.0, 8.0]], [[2.0, 7.0, 3.0], [6.0, 2.0, 8.0], [2.0, 3.0, 4.0]], [[7.0, 2.0, 
6.0], [2.0, 3.0, 3.0], [4.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 5.0], [6.0, 4.0, 8.0], [3.0, 3.0, 3.0]], 
[[3.0, 3.0, 8.0], [6.0, 7.0, 2.0], [2.0, 6.0, 4.0]], [[4.0, 2.0, 2.0], [5.0, 8.0, 3.0], [2.0, 8.0, 1.0]]], [[[1.0, 2.0, 5.0], 
[8.0, 4.0, 8.0], [7.0, 2.0, 7.0]], [[6.0, 2.0, 1.0], [2.0, 8.0, 6.0], [7.0, 8.0, 4.0]], [[5.0, 8.0, 7.0], [5.0, 1.0, 4.0], 
[2.0, 6.0, 5.0]]], [[[8.0, 5.0, 7.0], [4.0, 4.0, 7.0], [4.0, 7.0, 2.0]], [[4.0, 5.0, 1.0], [6.0, 6.0, 1.0], [4.0, 3.0, 1.0]], 
[[4.0, 6.0, 6.0], [6.0, 8.0, 2.0], [2.0, 7.0, 2.0]]], [[[3.0, 6.0, 7.0], [6.0, 8.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 1.0, 5.0], 
[1.0, 6.0, 1.0], [7.0, 1.0, 6.0]], [[3.0, 1.0, 1.0], [5.0, 6.0, 5.0], [7.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 4.0], [7.0, 2.0, 3.0], [8.0, 4.0, 6.0]], [[3.0, 2.0, 8.0], [4.0, 2.0, 3.0], [1.0, 
7.0, 7.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 5.0], [7.0, 8.0, 7.0]]], [[[8.0, 6.0, 1.0], [1.0, 8.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 
7.0, 4.0], [4.0, 8.0, 1.0], [8.0, 6.0, 5.0]], [[6.0, 3.0, 5.0], [6.0, 4.0, 6.0], [6.0, 2.0, 8.0]]], [[[1.0, 5.0, 6.0], [8.0, 
4.0, 5.0], [5.0, 1.0, 4.0]], [[1.0, 5.0, 8.0], [2.0, 6.0, 3.0], [5.0, 4.0, 1.0]], [[5.0, 3.0, 7.0], [2.0, 1.0, 8.0], [6.0, 4.0, 
3.0]]], [[[6.0, 6.0, 1.0], [3.0, 7.0, 6.0], [3.0, 4.0, 1.0]], [[6.0, 5.0, 5.0], [3.0, 6.0, 8.0], [6.0, 2.0, 3.0]], [[6.0, 7.0, 
7.0], [6.0, 6.0, 1.0], [4.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 5.0], [3.0, 3.0, 3.0], [7.0, 2.0, 7.0]], 
[[6.0, 3.0, 5.0], [5.0, 7.0, 5.0], [4.0, 6.0, 3.0]], [[3.0, 1.0, 2.0], [1.0, 3.0, 5.0], [3.0, 1.0, 7.0]]], [[[6.0, 6.0, 8.0], 
[7.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[2.0, 6.0, 4.0], [4.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[1.0, 6.0, 5.0], [5.0, 7.0, 2.0], 
[6.0, 1.0, 4.0]]], [[[6.0, 7.0, 4.0], [3.0, 2.0, 1.0], [6.0, 7.0, 5.0]], [[5.0, 7.0, 5.0], [3.0, 1.0, 8.0], [4.0, 3.0, 5.0]], 
[[4.0, 2.0, 5.0], [3.0, 6.0, 1.0], [5.0, 2.0, 1.0]]], [[[1.0, 4.0, 7.0], [6.0, 3.0, 1.0], [3.0, 1.0, 8.0]], [[3.0, 2.0, 7.0], 
[8.0, 1.0, 8.0], [1.0, 1.0, 1.0]], [[6.0, 8.0, 7.0], [4.0, 2.0, 3.0], [1.0, 6.0, 7.0]]]])
      val=numpy.array([[[[6.0, 4.0, 1.0]], [[5.0, 1.0, 8.0]]], [[[8.0, 8.0, 8.0]], [[3.0, 8.0, 5.0]]]])
      arg[2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 4.0], [7.0, 2.0, 3.0], [8.0, 4.0, 6.0]], [[3.0, 2.0, 8.0], [4.0, 2.0, 3.0], [1.0, 
7.0, 7.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 5.0], [7.0, 8.0, 7.0]]], [[[8.0, 6.0, 1.0], [1.0, 8.0, 8.0], [8.0, 7.0, 3.0]], [[2.0, 
7.0, 4.0], [4.0, 8.0, 1.0], [8.0, 6.0, 5.0]], [[6.0, 3.0, 5.0], [6.0, 4.0, 6.0], [6.0, 2.0, 8.0]]], [[[1.0, 5.0, 6.0], [8.0, 
4.0, 5.0], [6.0, 4.0, 1.0]], [[1.0, 5.0, 8.0], [2.0, 6.0, 3.0], [5.0, 1.0, 8.0]], [[5.0, 3.0, 7.0], [2.0, 1.0, 8.0], [6.0, 4.0, 
3.0]]], [[[6.0, 6.0, 1.0], [3.0, 7.0, 6.0], [8.0, 8.0, 8.0]], [[6.0, 5.0, 5.0], [3.0, 6.0, 8.0], [3.0, 8.0, 5.0]], [[6.0, 7.0, 
7.0], [6.0, 6.0, 1.0], [4.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 5.0], [3.0, 3.0, 3.0], [7.0, 2.0, 7.0]], 
[[6.0, 3.0, 5.0], [5.0, 7.0, 5.0], [4.0, 6.0, 3.0]], [[3.0, 1.0, 2.0], [1.0, 3.0, 5.0], [3.0, 1.0, 7.0]]], [[[6.0, 6.0, 8.0], 
[7.0, 6.0, 5.0], [4.0, 6.0, 6.0]], [[2.0, 6.0, 4.0], [4.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[1.0, 6.0, 5.0], [5.0, 7.0, 2.0], 
[6.0, 1.0, 4.0]]], [[[6.0, 7.0, 4.0], [3.0, 2.0, 1.0], [6.0, 4.0, 1.0]], [[5.0, 7.0, 5.0], [3.0, 1.0, 8.0], [5.0, 1.0, 8.0]], 
[[4.0, 2.0, 5.0], [3.0, 6.0, 1.0], [5.0, 2.0, 1.0]]], [[[1.0, 4.0, 7.0], [6.0, 3.0, 1.0], [8.0, 8.0, 8.0]], [[3.0, 2.0, 7.0], 
[8.0, 1.0, 8.0], [3.0, 8.0, 5.0]], [[6.0, 8.0, 7.0], [4.0, 2.0, 3.0], [1.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 4.0], [8.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 2.0], [7.0, 7.0, 8.0], [1.0, 
7.0, 6.0]], [[5.0, 6.0, 4.0], [4.0, 1.0, 8.0], [8.0, 3.0, 3.0]]], [[[3.0, 3.0, 4.0], [5.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[5.0, 
1.0, 7.0], [1.0, 3.0, 3.0], [4.0, 3.0, 7.0]], [[2.0, 8.0, 5.0], [8.0, 6.0, 5.0], [6.0, 8.0, 7.0]]], [[[7.0, 3.0, 7.0], [1.0, 
4.0, 4.0], [1.0, 5.0, 5.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 6.0], [1.0, 4.0, 5.0]], [[2.0, 7.0, 3.0], [8.0, 8.0, 7.0], [6.0, 3.0, 
3.0]]], [[[3.0, 4.0, 2.0], [5.0, 1.0, 3.0], [4.0, 4.0, 6.0]], [[2.0, 3.0, 1.0], [6.0, 1.0, 1.0], [8.0, 7.0, 4.0]], [[1.0, 5.0, 
2.0], [3.0, 1.0, 7.0], [6.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 7.0], [8.0, 8.0, 2.0], [3.0, 3.0, 4.0]], 
[[6.0, 4.0, 2.0], [1.0, 3.0, 5.0], [4.0, 3.0, 3.0]], [[7.0, 2.0, 8.0], [5.0, 6.0, 1.0], [2.0, 8.0, 2.0]]], [[[4.0, 2.0, 5.0], 
[1.0, 5.0, 8.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 8.0], [4.0, 1.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 5.0, 7.0], [4.0, 8.0, 2.0], 
[7.0, 3.0, 7.0]]], [[[3.0, 8.0, 1.0], [4.0, 2.0, 3.0], [6.0, 6.0, 7.0]], [[6.0, 6.0, 8.0], [8.0, 7.0, 1.0], [8.0, 8.0, 6.0]], 
[[2.0, 7.0, 4.0], [4.0, 6.0, 5.0], [3.0, 4.0, 3.0]]], [[[1.0, 1.0, 7.0], [4.0, 1.0, 4.0], [6.0, 3.0, 2.0]], [[3.0, 7.0, 3.0], 
[3.0, 8.0, 6.0], [6.0, 2.0, 7.0]], [[3.0, 1.0, 1.0], [6.0, 5.0, 8.0], [5.0, 7.0, 3.0]]]])
      val=Data(numpy.array([[[[2.0, 7.0, 5.0]], [[1.0, 7.0, 2.0]]], [[[2.0, 7.0, 5.0]], [[3.0, 7.0, 8.0]]]]),self.functionspace)
      arg[2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [8.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 2.0], [7.0, 7.0, 8.0], [1.0, 
7.0, 6.0]], [[5.0, 6.0, 4.0], [4.0, 1.0, 8.0], [8.0, 3.0, 3.0]]], [[[3.0, 3.0, 4.0], [5.0, 3.0, 5.0], [3.0, 4.0, 3.0]], [[5.0, 
1.0, 7.0], [1.0, 3.0, 3.0], [4.0, 3.0, 7.0]], [[2.0, 8.0, 5.0], [8.0, 6.0, 5.0], [6.0, 8.0, 7.0]]], [[[7.0, 3.0, 7.0], [1.0, 
4.0, 4.0], [2.0, 7.0, 5.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 6.0], [1.0, 7.0, 2.0]], [[2.0, 7.0, 3.0], [8.0, 8.0, 7.0], [6.0, 3.0, 
3.0]]], [[[3.0, 4.0, 2.0], [5.0, 1.0, 3.0], [2.0, 7.0, 5.0]], [[2.0, 3.0, 1.0], [6.0, 1.0, 1.0], [3.0, 7.0, 8.0]], [[1.0, 5.0, 
2.0], [3.0, 1.0, 7.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [8.0, 8.0, 2.0], [3.0, 3.0, 4.0]], 
[[6.0, 4.0, 2.0], [1.0, 3.0, 5.0], [4.0, 3.0, 3.0]], [[7.0, 2.0, 8.0], [5.0, 6.0, 1.0], [2.0, 8.0, 2.0]]], [[[4.0, 2.0, 5.0], 
[1.0, 5.0, 8.0], [3.0, 3.0, 7.0]], [[5.0, 8.0, 8.0], [4.0, 1.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 5.0, 7.0], [4.0, 8.0, 2.0], 
[7.0, 3.0, 7.0]]], [[[3.0, 8.0, 1.0], [4.0, 2.0, 3.0], [2.0, 7.0, 5.0]], [[6.0, 6.0, 8.0], [8.0, 7.0, 1.0], [1.0, 7.0, 2.0]], 
[[2.0, 7.0, 4.0], [4.0, 6.0, 5.0], [3.0, 4.0, 3.0]]], [[[1.0, 1.0, 7.0], [4.0, 1.0, 4.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 3.0], 
[3.0, 8.0, 6.0], [3.0, 7.0, 8.0]], [[3.0, 1.0, 1.0], [6.0, 5.0, 8.0], [5.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 7.0], [4.0, 2.0, 3.0], [7.0, 4.0, 2.0]], [[2.0, 2.0, 1.0], [8.0, 2.0, 1.0], [8.0, 
4.0, 4.0]], [[4.0, 2.0, 1.0], [6.0, 6.0, 4.0], [3.0, 3.0, 6.0]]], [[[2.0, 4.0, 6.0], [5.0, 7.0, 1.0], [7.0, 7.0, 5.0]], [[4.0, 
8.0, 1.0], [2.0, 7.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 4.0, 6.0], [2.0, 7.0, 8.0], [1.0, 4.0, 8.0]]], [[[5.0, 3.0, 7.0], [2.0, 
7.0, 8.0], [1.0, 8.0, 1.0]], [[7.0, 2.0, 8.0], [5.0, 2.0, 8.0], [7.0, 8.0, 1.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 7.0], [6.0, 7.0, 
8.0]]], [[[1.0, 3.0, 6.0], [3.0, 3.0, 6.0], [4.0, 2.0, 1.0]], [[2.0, 8.0, 6.0], [8.0, 1.0, 7.0], [4.0, 4.0, 1.0]], [[1.0, 1.0, 
2.0], [7.0, 1.0, 7.0], [6.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 8.0], [6.0, 6.0, 2.0], [5.0, 8.0, 8.0]], 
[[5.0, 8.0, 2.0], [1.0, 1.0, 7.0], [2.0, 4.0, 2.0]], [[7.0, 8.0, 8.0], [3.0, 2.0, 4.0], [5.0, 3.0, 7.0]]], [[[7.0, 4.0, 1.0], 
[5.0, 3.0, 3.0], [7.0, 1.0, 8.0]], [[5.0, 4.0, 5.0], [1.0, 2.0, 3.0], [1.0, 5.0, 7.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 8.0], 
[5.0, 3.0, 3.0]]], [[[8.0, 3.0, 5.0], [8.0, 5.0, 8.0], [3.0, 1.0, 7.0]], [[1.0, 3.0, 5.0], [3.0, 6.0, 3.0], [1.0, 6.0, 3.0]], 
[[7.0, 7.0, 3.0], [6.0, 1.0, 5.0], [3.0, 1.0, 6.0]]], [[[2.0, 5.0, 5.0], [2.0, 6.0, 1.0], [5.0, 5.0, 2.0]], [[8.0, 6.0, 2.0], 
[6.0, 3.0, 5.0], [5.0, 7.0, 4.0]], [[7.0, 8.0, 3.0], [2.0, 5.0, 7.0], [2.0, 4.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 2.0, 2.0]], [[7.0, 2.0, 3.0]]], [[[6.0, 8.0, 2.0]], [[1.0, 1.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 2.0, 2.0]], [[6.0, 4.0, 2.0]]], [[[6.0, 8.0, 5.0]], [[7.0, 1.0, 2.0]]]])
      arg[2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [4.0, 2.0, 3.0], [7.0, 4.0, 2.0]], [[2.0, 2.0, 1.0], [8.0, 2.0, 1.0], [8.0, 
4.0, 4.0]], [[4.0, 2.0, 1.0], [6.0, 6.0, 4.0], [3.0, 3.0, 6.0]]], [[[2.0, 4.0, 6.0], [5.0, 7.0, 1.0], [7.0, 7.0, 5.0]], [[4.0, 
8.0, 1.0], [2.0, 7.0, 5.0], [6.0, 5.0, 6.0]], [[2.0, 4.0, 6.0], [2.0, 7.0, 8.0], [1.0, 4.0, 8.0]]], [[[5.0, 3.0, 7.0], [2.0, 
7.0, 8.0], [4.0, 2.0, 2.0]], [[7.0, 2.0, 8.0], [5.0, 2.0, 8.0], [7.0, 2.0, 3.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 7.0], [6.0, 7.0, 
8.0]]], [[[1.0, 3.0, 6.0], [3.0, 3.0, 6.0], [6.0, 8.0, 2.0]], [[2.0, 8.0, 6.0], [8.0, 1.0, 7.0], [1.0, 1.0, 4.0]], [[1.0, 1.0, 
2.0], [7.0, 1.0, 7.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 8.0], [6.0, 6.0, 2.0], [5.0, 8.0, 8.0]], 
[[5.0, 8.0, 2.0], [1.0, 1.0, 7.0], [2.0, 4.0, 2.0]], [[7.0, 8.0, 8.0], [3.0, 2.0, 4.0], [5.0, 3.0, 7.0]]], [[[7.0, 4.0, 1.0], 
[5.0, 3.0, 3.0], [7.0, 1.0, 8.0]], [[5.0, 4.0, 5.0], [1.0, 2.0, 3.0], [1.0, 5.0, 7.0]], [[5.0, 4.0, 1.0], [8.0, 3.0, 8.0], 
[5.0, 3.0, 3.0]]], [[[8.0, 3.0, 5.0], [8.0, 5.0, 8.0], [3.0, 2.0, 2.0]], [[1.0, 3.0, 5.0], [3.0, 6.0, 3.0], [6.0, 4.0, 2.0]], 
[[7.0, 7.0, 3.0], [6.0, 1.0, 5.0], [3.0, 1.0, 6.0]]], [[[2.0, 5.0, 5.0], [2.0, 6.0, 1.0], [6.0, 8.0, 5.0]], [[8.0, 6.0, 2.0], 
[6.0, 3.0, 5.0], [7.0, 1.0, 2.0]], [[7.0, 8.0, 3.0], [2.0, 5.0, 7.0], [2.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 7.0], [4.0, 3.0, 8.0], [3.0, 5.0, 5.0]], [[6.0, 6.0, 5.0], [4.0, 4.0, 3.0], [2.0, 
2.0, 4.0]], [[3.0, 1.0, 3.0], [6.0, 8.0, 8.0], [7.0, 6.0, 5.0]]], [[[6.0, 2.0, 3.0], [5.0, 3.0, 8.0], [3.0, 8.0, 5.0]], [[2.0, 
4.0, 5.0], [4.0, 2.0, 5.0], [7.0, 8.0, 8.0]], [[5.0, 1.0, 6.0], [2.0, 5.0, 2.0], [5.0, 2.0, 6.0]]], [[[6.0, 7.0, 5.0], [4.0, 
7.0, 8.0], [7.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [7.0, 4.0, 5.0], [7.0, 2.0, 1.0]], [[5.0, 6.0, 2.0], [6.0, 1.0, 1.0], [3.0, 6.0, 
4.0]]], [[[3.0, 2.0, 5.0], [7.0, 7.0, 3.0], [6.0, 6.0, 2.0]], [[8.0, 5.0, 5.0], [4.0, 3.0, 5.0], [4.0, 5.0, 6.0]], [[4.0, 6.0, 
4.0], [3.0, 8.0, 2.0], [5.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 4.0], [7.0, 8.0, 8.0], [3.0, 8.0, 2.0]], 
[[7.0, 2.0, 7.0], [2.0, 5.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 7.0], [5.0, 7.0, 8.0], [6.0, 2.0, 8.0]]], [[[1.0, 1.0, 1.0], 
[5.0, 1.0, 3.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[8.0, 6.0, 5.0], [4.0, 2.0, 5.0], 
[7.0, 1.0, 5.0]]], [[[4.0, 3.0, 1.0], [5.0, 2.0, 8.0], [1.0, 4.0, 2.0]], [[7.0, 5.0, 5.0], [3.0, 1.0, 6.0], [1.0, 8.0, 3.0]], 
[[2.0, 4.0, 2.0], [4.0, 8.0, 6.0], [4.0, 1.0, 1.0]]], [[[6.0, 2.0, 3.0], [6.0, 8.0, 2.0], [5.0, 1.0, 5.0]], [[4.0, 1.0, 5.0], 
[2.0, 7.0, 1.0], [8.0, 7.0, 4.0]], [[2.0, 5.0, 2.0], [7.0, 1.0, 3.0], [4.0, 4.0, 1.0]]]])
      val=numpy.array([[[[6.0, 4.0]]], [[[4.0, 7.0]]]])
      arg[2:,1:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 7.0], [4.0, 3.0, 8.0], [3.0, 5.0, 5.0]], [[6.0, 6.0, 5.0], [4.0, 4.0, 3.0], [2.0, 
2.0, 4.0]], [[3.0, 1.0, 3.0], [6.0, 8.0, 8.0], [7.0, 6.0, 5.0]]], [[[6.0, 2.0, 3.0], [5.0, 3.0, 8.0], [3.0, 8.0, 5.0]], [[2.0, 
4.0, 5.0], [4.0, 2.0, 5.0], [7.0, 8.0, 8.0]], [[5.0, 1.0, 6.0], [2.0, 5.0, 2.0], [5.0, 2.0, 6.0]]], [[[6.0, 7.0, 5.0], [4.0, 
7.0, 8.0], [7.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [6.0, 4.0, 5.0], [7.0, 2.0, 1.0]], [[5.0, 6.0, 2.0], [6.0, 1.0, 1.0], [3.0, 6.0, 
4.0]]], [[[3.0, 2.0, 5.0], [7.0, 7.0, 3.0], [6.0, 6.0, 2.0]], [[8.0, 5.0, 5.0], [4.0, 7.0, 5.0], [4.0, 5.0, 6.0]], [[4.0, 6.0, 
4.0], [3.0, 8.0, 2.0], [5.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [7.0, 8.0, 8.0], [3.0, 8.0, 2.0]], 
[[7.0, 2.0, 7.0], [2.0, 5.0, 7.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 7.0], [5.0, 7.0, 8.0], [6.0, 2.0, 8.0]]], [[[1.0, 1.0, 1.0], 
[5.0, 1.0, 3.0], [4.0, 8.0, 8.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 4.0], [5.0, 5.0, 2.0]], [[8.0, 6.0, 5.0], [4.0, 2.0, 5.0], 
[7.0, 1.0, 5.0]]], [[[4.0, 3.0, 1.0], [5.0, 2.0, 8.0], [1.0, 4.0, 2.0]], [[7.0, 5.0, 5.0], [6.0, 4.0, 6.0], [1.0, 8.0, 3.0]], 
[[2.0, 4.0, 2.0], [4.0, 8.0, 6.0], [4.0, 1.0, 1.0]]], [[[6.0, 2.0, 3.0], [6.0, 8.0, 2.0], [5.0, 1.0, 5.0]], [[4.0, 1.0, 5.0], 
[4.0, 7.0, 1.0], [8.0, 7.0, 4.0]], [[2.0, 5.0, 2.0], [7.0, 1.0, 3.0], [4.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 4.0], [3.0, 4.0, 2.0], [7.0, 2.0, 1.0]], [[5.0, 7.0, 1.0], [3.0, 7.0, 8.0], [3.0, 
6.0, 2.0]], [[3.0, 6.0, 4.0], [4.0, 7.0, 5.0], [6.0, 7.0, 2.0]]], [[[8.0, 5.0, 8.0], [2.0, 5.0, 8.0], [4.0, 6.0, 1.0]], [[5.0, 
7.0, 4.0], [6.0, 2.0, 6.0], [6.0, 4.0, 5.0]], [[2.0, 8.0, 1.0], [4.0, 1.0, 7.0], [1.0, 1.0, 4.0]]], [[[1.0, 3.0, 2.0], [7.0, 
5.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 8.0, 2.0], [8.0, 3.0, 8.0], [1.0, 2.0, 4.0]], [[7.0, 8.0, 1.0], [1.0, 5.0, 7.0], [7.0, 8.0, 
7.0]]], [[[1.0, 5.0, 1.0], [7.0, 4.0, 3.0], [3.0, 7.0, 4.0]], [[2.0, 8.0, 7.0], [6.0, 8.0, 1.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 
3.0], [4.0, 2.0, 5.0], [2.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 8.0], [2.0, 1.0, 3.0], [2.0, 2.0, 6.0]], 
[[5.0, 4.0, 4.0], [3.0, 7.0, 7.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 2.0], [6.0, 4.0, 4.0], [7.0, 3.0, 7.0]]], [[[7.0, 6.0, 3.0], 
[8.0, 1.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 1.0, 7.0], [6.0, 5.0, 5.0], [6.0, 3.0, 1.0]], [[8.0, 6.0, 8.0], [8.0, 7.0, 1.0], 
[3.0, 8.0, 5.0]]], [[[1.0, 8.0, 1.0], [4.0, 8.0, 1.0], [6.0, 7.0, 3.0]], [[6.0, 8.0, 4.0], [7.0, 7.0, 3.0], [1.0, 4.0, 2.0]], 
[[4.0, 6.0, 2.0], [2.0, 3.0, 8.0], [6.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [3.0, 6.0, 8.0], [2.0, 8.0, 6.0]], [[7.0, 5.0, 5.0], 
[5.0, 1.0, 2.0], [5.0, 7.0, 6.0]], [[4.0, 2.0, 8.0], [6.0, 5.0, 4.0], [8.0, 6.0, 8.0]]]])
      val=Data(numpy.array([[[[3.0, 2.0]]], [[[8.0, 3.0]]]]),self.functionspace)
      arg[2:,1:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 4.0], [3.0, 4.0, 2.0], [7.0, 2.0, 1.0]], [[5.0, 7.0, 1.0], [3.0, 7.0, 8.0], [3.0, 
6.0, 2.0]], [[3.0, 6.0, 4.0], [4.0, 7.0, 5.0], [6.0, 7.0, 2.0]]], [[[8.0, 5.0, 8.0], [2.0, 5.0, 8.0], [4.0, 6.0, 1.0]], [[5.0, 
7.0, 4.0], [6.0, 2.0, 6.0], [6.0, 4.0, 5.0]], [[2.0, 8.0, 1.0], [4.0, 1.0, 7.0], [1.0, 1.0, 4.0]]], [[[1.0, 3.0, 2.0], [7.0, 
5.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 8.0, 2.0], [3.0, 2.0, 8.0], [1.0, 2.0, 4.0]], [[7.0, 8.0, 1.0], [1.0, 5.0, 7.0], [7.0, 8.0, 
7.0]]], [[[1.0, 5.0, 1.0], [7.0, 4.0, 3.0], [3.0, 7.0, 4.0]], [[2.0, 8.0, 7.0], [8.0, 3.0, 1.0], [7.0, 2.0, 6.0]], [[7.0, 1.0, 
3.0], [4.0, 2.0, 5.0], [2.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 8.0], [2.0, 1.0, 3.0], [2.0, 2.0, 6.0]], 
[[5.0, 4.0, 4.0], [3.0, 7.0, 7.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 2.0], [6.0, 4.0, 4.0], [7.0, 3.0, 7.0]]], [[[7.0, 6.0, 3.0], 
[8.0, 1.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 1.0, 7.0], [6.0, 5.0, 5.0], [6.0, 3.0, 1.0]], [[8.0, 6.0, 8.0], [8.0, 7.0, 1.0], 
[3.0, 8.0, 5.0]]], [[[1.0, 8.0, 1.0], [4.0, 8.0, 1.0], [6.0, 7.0, 3.0]], [[6.0, 8.0, 4.0], [3.0, 2.0, 3.0], [1.0, 4.0, 2.0]], 
[[4.0, 6.0, 2.0], [2.0, 3.0, 8.0], [6.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [3.0, 6.0, 8.0], [2.0, 8.0, 6.0]], [[7.0, 5.0, 5.0], 
[8.0, 3.0, 2.0], [5.0, 7.0, 6.0]], [[4.0, 2.0, 8.0], [6.0, 5.0, 4.0], [8.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRRU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 3.0], [8.0, 6.0, 3.0], [8.0, 4.0, 1.0]], [[8.0, 1.0, 1.0], [7.0, 5.0, 5.0], [2.0, 
8.0, 5.0]], [[5.0, 2.0, 7.0], [8.0, 4.0, 4.0], [3.0, 2.0, 6.0]]], [[[1.0, 2.0, 6.0], [6.0, 4.0, 3.0], [5.0, 4.0, 7.0]], [[1.0, 
7.0, 2.0], [8.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[6.0, 1.0, 1.0], [1.0, 2.0, 8.0], [6.0, 5.0, 1.0]]], [[[4.0, 2.0, 6.0], [7.0, 
6.0, 5.0], [7.0, 8.0, 5.0]], [[8.0, 8.0, 1.0], [8.0, 2.0, 4.0], [1.0, 4.0, 5.0]], [[3.0, 5.0, 8.0], [3.0, 2.0, 5.0], [4.0, 5.0, 
5.0]]], [[[5.0, 1.0, 3.0], [3.0, 4.0, 6.0], [5.0, 3.0, 1.0]], [[2.0, 5.0, 1.0], [1.0, 7.0, 5.0], [5.0, 7.0, 4.0]], [[6.0, 7.0, 
5.0], [6.0, 1.0, 2.0], [4.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 4.0], [1.0, 1.0, 8.0], [6.0, 2.0, 8.0]], 
[[1.0, 1.0, 7.0], [4.0, 6.0, 4.0], [6.0, 1.0, 1.0]], [[1.0, 6.0, 2.0], [2.0, 5.0, 5.0], [3.0, 1.0, 7.0]]], [[[7.0, 8.0, 8.0], 
[6.0, 6.0, 8.0], [7.0, 1.0, 3.0]], [[4.0, 6.0, 3.0], [8.0, 8.0, 2.0], [5.0, 1.0, 6.0]], [[8.0, 5.0, 5.0], [1.0, 1.0, 5.0], 
[4.0, 4.0, 3.0]]], [[[7.0, 4.0, 4.0], [7.0, 7.0, 6.0], [4.0, 3.0, 2.0]], [[3.0, 8.0, 8.0], [8.0, 1.0, 8.0], [1.0, 8.0, 7.0]], 
[[2.0, 7.0, 6.0], [1.0, 6.0, 2.0], [4.0, 5.0, 6.0]]], [[[3.0, 1.0, 4.0], [8.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[6.0, 3.0, 4.0], 
[5.0, 5.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 2.0, 7.0], [6.0, 7.0, 5.0], [5.0, 5.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 4.0]]], [[[5.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 2.0]]], [[[8.0, 2.0]]]])
      arg[2:,1:2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [8.0, 6.0, 3.0], [8.0, 4.0, 1.0]], [[8.0, 1.0, 1.0], [7.0, 5.0, 5.0], [2.0, 
8.0, 5.0]], [[5.0, 2.0, 7.0], [8.0, 4.0, 4.0], [3.0, 2.0, 6.0]]], [[[1.0, 2.0, 6.0], [6.0, 4.0, 3.0], [5.0, 4.0, 7.0]], [[1.0, 
7.0, 2.0], [8.0, 8.0, 4.0], [2.0, 8.0, 1.0]], [[6.0, 1.0, 1.0], [1.0, 2.0, 8.0], [6.0, 5.0, 1.0]]], [[[4.0, 2.0, 6.0], [7.0, 
6.0, 5.0], [7.0, 8.0, 5.0]], [[8.0, 8.0, 1.0], [7.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[3.0, 5.0, 8.0], [3.0, 2.0, 5.0], [4.0, 5.0, 
5.0]]], [[[5.0, 1.0, 3.0], [3.0, 4.0, 6.0], [5.0, 3.0, 1.0]], [[2.0, 5.0, 1.0], [5.0, 8.0, 5.0], [5.0, 7.0, 4.0]], [[6.0, 7.0, 
5.0], [6.0, 1.0, 2.0], [4.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 4.0], [1.0, 1.0, 8.0], [6.0, 2.0, 8.0]], 
[[1.0, 1.0, 7.0], [4.0, 6.0, 4.0], [6.0, 1.0, 1.0]], [[1.0, 6.0, 2.0], [2.0, 5.0, 5.0], [3.0, 1.0, 7.0]]], [[[7.0, 8.0, 8.0], 
[6.0, 6.0, 8.0], [7.0, 1.0, 3.0]], [[4.0, 6.0, 3.0], [8.0, 8.0, 2.0], [5.0, 1.0, 6.0]], [[8.0, 5.0, 5.0], [1.0, 1.0, 5.0], 
[4.0, 4.0, 3.0]]], [[[7.0, 4.0, 4.0], [7.0, 7.0, 6.0], [4.0, 3.0, 2.0]], [[3.0, 8.0, 8.0], [8.0, 2.0, 8.0], [1.0, 8.0, 7.0]], 
[[2.0, 7.0, 6.0], [1.0, 6.0, 2.0], [4.0, 5.0, 6.0]]], [[[3.0, 1.0, 4.0], [8.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[6.0, 3.0, 4.0], 
[8.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[1.0, 2.0, 7.0], [6.0, 7.0, 5.0], [5.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 1.0], [2.0, 6.0, 5.0], [1.0, 6.0, 2.0]], [[3.0, 8.0, 8.0], [7.0, 5.0, 1.0], [8.0, 
8.0, 2.0]], [[2.0, 3.0, 7.0], [5.0, 5.0, 5.0], [7.0, 4.0, 2.0]]], [[[2.0, 1.0, 8.0], [5.0, 1.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 
5.0, 2.0], [2.0, 1.0, 7.0], [3.0, 1.0, 5.0]], [[4.0, 3.0, 7.0], [1.0, 3.0, 7.0], [3.0, 1.0, 3.0]]], [[[6.0, 1.0, 7.0], [4.0, 
8.0, 3.0], [2.0, 6.0, 3.0]], [[6.0, 4.0, 8.0], [2.0, 7.0, 8.0], [8.0, 6.0, 5.0]], [[8.0, 7.0, 4.0], [4.0, 3.0, 3.0], [2.0, 4.0, 
5.0]]], [[[1.0, 4.0, 4.0], [5.0, 1.0, 5.0], [8.0, 3.0, 4.0]], [[7.0, 5.0, 7.0], [6.0, 2.0, 5.0], [7.0, 2.0, 8.0]], [[5.0, 6.0, 
4.0], [6.0, 5.0, 3.0], [4.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 2.0], [4.0, 8.0, 4.0], [7.0, 5.0, 2.0]], 
[[5.0, 2.0, 7.0], [4.0, 2.0, 3.0], [3.0, 3.0, 2.0]], [[6.0, 1.0, 7.0], [2.0, 4.0, 3.0], [1.0, 5.0, 3.0]]], [[[2.0, 7.0, 7.0], 
[5.0, 8.0, 6.0], [2.0, 6.0, 6.0]], [[7.0, 3.0, 4.0], [2.0, 8.0, 3.0], [7.0, 3.0, 1.0]], [[8.0, 6.0, 5.0], [4.0, 1.0, 3.0], 
[4.0, 5.0, 1.0]]], [[[2.0, 4.0, 8.0], [6.0, 3.0, 4.0], [7.0, 3.0, 6.0]], [[1.0, 2.0, 7.0], [4.0, 3.0, 2.0], [8.0, 3.0, 2.0]], 
[[1.0, 4.0, 3.0], [1.0, 8.0, 3.0], [8.0, 8.0, 1.0]]], [[[2.0, 1.0, 6.0], [5.0, 3.0, 5.0], [8.0, 1.0, 4.0]], [[5.0, 2.0, 4.0], 
[3.0, 1.0, 2.0], [1.0, 3.0, 6.0]], [[6.0, 5.0, 1.0], [8.0, 5.0, 8.0], [6.0, 8.0, 1.0]]]])
      val=numpy.array([[[[5.0, 5.0, 8.0], [6.0, 8.0, 1.0]]], [[[2.0, 1.0, 7.0], [2.0, 4.0, 5.0]]]])
      arg[1:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 1.0], [2.0, 6.0, 5.0], [1.0, 6.0, 2.0]], [[3.0, 8.0, 8.0], [7.0, 5.0, 1.0], [8.0, 
8.0, 2.0]], [[2.0, 3.0, 7.0], [5.0, 5.0, 5.0], [7.0, 4.0, 2.0]]], [[[2.0, 1.0, 8.0], [5.0, 1.0, 6.0], [3.0, 3.0, 2.0]], [[5.0, 
5.0, 8.0], [6.0, 8.0, 1.0], [3.0, 1.0, 5.0]], [[4.0, 3.0, 7.0], [1.0, 3.0, 7.0], [3.0, 1.0, 3.0]]], [[[6.0, 1.0, 7.0], [4.0, 
8.0, 3.0], [2.0, 6.0, 3.0]], [[2.0, 1.0, 7.0], [2.0, 4.0, 5.0], [8.0, 6.0, 5.0]], [[8.0, 7.0, 4.0], [4.0, 3.0, 3.0], [2.0, 4.0, 
5.0]]], [[[1.0, 4.0, 4.0], [5.0, 1.0, 5.0], [8.0, 3.0, 4.0]], [[7.0, 5.0, 7.0], [6.0, 2.0, 5.0], [7.0, 2.0, 8.0]], [[5.0, 6.0, 
4.0], [6.0, 5.0, 3.0], [4.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 2.0], [4.0, 8.0, 4.0], [7.0, 5.0, 2.0]], 
[[5.0, 2.0, 7.0], [4.0, 2.0, 3.0], [3.0, 3.0, 2.0]], [[6.0, 1.0, 7.0], [2.0, 4.0, 3.0], [1.0, 5.0, 3.0]]], [[[2.0, 7.0, 7.0], 
[5.0, 8.0, 6.0], [2.0, 6.0, 6.0]], [[5.0, 5.0, 8.0], [6.0, 8.0, 1.0], [7.0, 3.0, 1.0]], [[8.0, 6.0, 5.0], [4.0, 1.0, 3.0], 
[4.0, 5.0, 1.0]]], [[[2.0, 4.0, 8.0], [6.0, 3.0, 4.0], [7.0, 3.0, 6.0]], [[2.0, 1.0, 7.0], [2.0, 4.0, 5.0], [8.0, 3.0, 2.0]], 
[[1.0, 4.0, 3.0], [1.0, 8.0, 3.0], [8.0, 8.0, 1.0]]], [[[2.0, 1.0, 6.0], [5.0, 3.0, 5.0], [8.0, 1.0, 4.0]], [[5.0, 2.0, 4.0], 
[3.0, 1.0, 2.0], [1.0, 3.0, 6.0]], [[6.0, 5.0, 1.0], [8.0, 5.0, 8.0], [6.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 3.0], [3.0, 5.0, 8.0], [6.0, 5.0, 1.0]], [[2.0, 4.0, 4.0], [8.0, 2.0, 2.0], [6.0, 
2.0, 1.0]], [[2.0, 6.0, 6.0], [3.0, 5.0, 2.0], [6.0, 6.0, 5.0]]], [[[2.0, 6.0, 4.0], [6.0, 5.0, 4.0], [3.0, 3.0, 4.0]], [[2.0, 
7.0, 2.0], [7.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[8.0, 7.0, 5.0], [6.0, 2.0, 6.0], [3.0, 7.0, 2.0]]], [[[4.0, 7.0, 5.0], [4.0, 
6.0, 3.0], [6.0, 3.0, 8.0]], [[4.0, 3.0, 1.0], [1.0, 3.0, 8.0], [1.0, 2.0, 6.0]], [[1.0, 1.0, 3.0], [7.0, 8.0, 3.0], [5.0, 2.0, 
6.0]]], [[[5.0, 7.0, 6.0], [1.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[7.0, 3.0, 7.0], [8.0, 8.0, 8.0], [7.0, 4.0, 6.0]], [[8.0, 3.0, 
5.0], [6.0, 2.0, 3.0], [4.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 7.0], [5.0, 7.0, 8.0], [5.0, 6.0, 7.0]], 
[[8.0, 1.0, 1.0], [2.0, 1.0, 2.0], [6.0, 4.0, 6.0]], [[5.0, 7.0, 2.0], [1.0, 1.0, 4.0], [5.0, 1.0, 3.0]]], [[[5.0, 3.0, 3.0], 
[4.0, 4.0, 7.0], [8.0, 3.0, 1.0]], [[6.0, 7.0, 7.0], [1.0, 3.0, 6.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 8.0], [2.0, 2.0, 3.0], 
[5.0, 8.0, 6.0]]], [[[3.0, 5.0, 2.0], [5.0, 7.0, 2.0], [4.0, 5.0, 4.0]], [[5.0, 4.0, 8.0], [3.0, 3.0, 1.0], [5.0, 5.0, 4.0]], 
[[6.0, 2.0, 7.0], [7.0, 1.0, 6.0], [2.0, 2.0, 3.0]]], [[[2.0, 3.0, 8.0], [1.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[2.0, 2.0, 4.0], 
[2.0, 4.0, 7.0], [1.0, 5.0, 2.0]], [[6.0, 7.0, 7.0], [7.0, 8.0, 8.0], [3.0, 2.0, 1.0]]]])
      val=Data(numpy.array([[[[1.0, 3.0, 4.0], [5.0, 5.0, 1.0]]], [[[7.0, 7.0, 1.0], [4.0, 1.0, 5.0]]]]),self.functionspace)
      arg[1:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [3.0, 5.0, 8.0], [6.0, 5.0, 1.0]], [[2.0, 4.0, 4.0], [8.0, 2.0, 2.0], [6.0, 
2.0, 1.0]], [[2.0, 6.0, 6.0], [3.0, 5.0, 2.0], [6.0, 6.0, 5.0]]], [[[2.0, 6.0, 4.0], [6.0, 5.0, 4.0], [3.0, 3.0, 4.0]], [[1.0, 
3.0, 4.0], [5.0, 5.0, 1.0], [1.0, 5.0, 2.0]], [[8.0, 7.0, 5.0], [6.0, 2.0, 6.0], [3.0, 7.0, 2.0]]], [[[4.0, 7.0, 5.0], [4.0, 
6.0, 3.0], [6.0, 3.0, 8.0]], [[7.0, 7.0, 1.0], [4.0, 1.0, 5.0], [1.0, 2.0, 6.0]], [[1.0, 1.0, 3.0], [7.0, 8.0, 3.0], [5.0, 2.0, 
6.0]]], [[[5.0, 7.0, 6.0], [1.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[7.0, 3.0, 7.0], [8.0, 8.0, 8.0], [7.0, 4.0, 6.0]], [[8.0, 3.0, 
5.0], [6.0, 2.0, 3.0], [4.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 7.0], [5.0, 7.0, 8.0], [5.0, 6.0, 7.0]], 
[[8.0, 1.0, 1.0], [2.0, 1.0, 2.0], [6.0, 4.0, 6.0]], [[5.0, 7.0, 2.0], [1.0, 1.0, 4.0], [5.0, 1.0, 3.0]]], [[[5.0, 3.0, 3.0], 
[4.0, 4.0, 7.0], [8.0, 3.0, 1.0]], [[1.0, 3.0, 4.0], [5.0, 5.0, 1.0], [5.0, 6.0, 4.0]], [[4.0, 2.0, 8.0], [2.0, 2.0, 3.0], 
[5.0, 8.0, 6.0]]], [[[3.0, 5.0, 2.0], [5.0, 7.0, 2.0], [4.0, 5.0, 4.0]], [[7.0, 7.0, 1.0], [4.0, 1.0, 5.0], [5.0, 5.0, 4.0]], 
[[6.0, 2.0, 7.0], [7.0, 1.0, 6.0], [2.0, 2.0, 3.0]]], [[[2.0, 3.0, 8.0], [1.0, 2.0, 3.0], [6.0, 7.0, 3.0]], [[2.0, 2.0, 4.0], 
[2.0, 4.0, 7.0], [1.0, 5.0, 2.0]], [[6.0, 7.0, 7.0], [7.0, 8.0, 8.0], [3.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 6.0], [8.0, 5.0, 4.0], [8.0, 8.0, 6.0]], [[7.0, 1.0, 8.0], [5.0, 4.0, 7.0], [4.0, 
2.0, 1.0]], [[8.0, 5.0, 2.0], [1.0, 1.0, 7.0], [6.0, 2.0, 2.0]]], [[[7.0, 1.0, 1.0], [1.0, 7.0, 4.0], [2.0, 1.0, 8.0]], [[2.0, 
4.0, 6.0], [4.0, 7.0, 5.0], [8.0, 7.0, 3.0]], [[7.0, 2.0, 1.0], [1.0, 7.0, 5.0], [6.0, 1.0, 6.0]]], [[[4.0, 8.0, 1.0], [1.0, 
2.0, 4.0], [4.0, 3.0, 3.0]], [[1.0, 4.0, 7.0], [1.0, 7.0, 8.0], [3.0, 1.0, 7.0]], [[3.0, 5.0, 4.0], [6.0, 6.0, 7.0], [7.0, 2.0, 
7.0]]], [[[8.0, 6.0, 4.0], [5.0, 8.0, 5.0], [5.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [2.0, 1.0, 4.0], [4.0, 1.0, 1.0]], [[6.0, 1.0, 
3.0], [3.0, 2.0, 3.0], [5.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 7.0], [1.0, 6.0, 8.0], [6.0, 4.0, 7.0]], 
[[6.0, 7.0, 8.0], [7.0, 3.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 6.0, 8.0], [1.0, 7.0, 8.0], [3.0, 3.0, 1.0]]], [[[8.0, 6.0, 7.0], 
[6.0, 6.0, 6.0], [1.0, 6.0, 4.0]], [[1.0, 8.0, 6.0], [4.0, 7.0, 2.0], [4.0, 1.0, 6.0]], [[5.0, 6.0, 1.0], [7.0, 5.0, 3.0], 
[3.0, 5.0, 1.0]]], [[[7.0, 6.0, 1.0], [7.0, 8.0, 8.0], [5.0, 2.0, 1.0]], [[6.0, 5.0, 2.0], [7.0, 4.0, 4.0], [5.0, 7.0, 2.0]], 
[[8.0, 3.0, 6.0], [7.0, 7.0, 7.0], [8.0, 8.0, 3.0]]], [[[1.0, 6.0, 3.0], [8.0, 7.0, 8.0], [2.0, 8.0, 1.0]], [[6.0, 1.0, 2.0], 
[3.0, 4.0, 3.0], [2.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [3.0, 3.0, 5.0], [1.0, 3.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 5.0, 1.0], [4.0, 3.0, 6.0]]], [[[5.0, 7.0, 4.0], [5.0, 7.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 8.0, 7.0], [3.0, 5.0, 1.0]]], [[[3.0, 6.0, 4.0], [7.0, 3.0, 5.0]]]])
      arg[1:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 6.0], [8.0, 5.0, 4.0], [8.0, 8.0, 6.0]], [[7.0, 1.0, 8.0], [5.0, 4.0, 7.0], [4.0, 
2.0, 1.0]], [[8.0, 5.0, 2.0], [1.0, 1.0, 7.0], [6.0, 2.0, 2.0]]], [[[7.0, 1.0, 1.0], [1.0, 7.0, 4.0], [2.0, 1.0, 8.0]], [[5.0, 
5.0, 1.0], [4.0, 3.0, 6.0], [8.0, 7.0, 3.0]], [[7.0, 2.0, 1.0], [1.0, 7.0, 5.0], [6.0, 1.0, 6.0]]], [[[4.0, 8.0, 1.0], [1.0, 
2.0, 4.0], [4.0, 3.0, 3.0]], [[5.0, 7.0, 4.0], [5.0, 7.0, 4.0], [3.0, 1.0, 7.0]], [[3.0, 5.0, 4.0], [6.0, 6.0, 7.0], [7.0, 2.0, 
7.0]]], [[[8.0, 6.0, 4.0], [5.0, 8.0, 5.0], [5.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [2.0, 1.0, 4.0], [4.0, 1.0, 1.0]], [[6.0, 1.0, 
3.0], [3.0, 2.0, 3.0], [5.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 7.0], [1.0, 6.0, 8.0], [6.0, 4.0, 7.0]], 
[[6.0, 7.0, 8.0], [7.0, 3.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 6.0, 8.0], [1.0, 7.0, 8.0], [3.0, 3.0, 1.0]]], [[[8.0, 6.0, 7.0], 
[6.0, 6.0, 6.0], [1.0, 6.0, 4.0]], [[6.0, 8.0, 7.0], [3.0, 5.0, 1.0], [4.0, 1.0, 6.0]], [[5.0, 6.0, 1.0], [7.0, 5.0, 3.0], 
[3.0, 5.0, 1.0]]], [[[7.0, 6.0, 1.0], [7.0, 8.0, 8.0], [5.0, 2.0, 1.0]], [[3.0, 6.0, 4.0], [7.0, 3.0, 5.0], [5.0, 7.0, 2.0]], 
[[8.0, 3.0, 6.0], [7.0, 7.0, 7.0], [8.0, 8.0, 3.0]]], [[[1.0, 6.0, 3.0], [8.0, 7.0, 8.0], [2.0, 8.0, 1.0]], [[6.0, 1.0, 2.0], 
[3.0, 4.0, 3.0], [2.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [3.0, 3.0, 5.0], [1.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LCLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 7.0], [6.0, 3.0, 2.0], [2.0, 3.0, 8.0]], [[7.0, 7.0, 4.0], [7.0, 3.0, 3.0], [4.0, 
4.0, 5.0]], [[1.0, 6.0, 1.0], [5.0, 2.0, 3.0], [7.0, 4.0, 7.0]]], [[[5.0, 5.0, 4.0], [7.0, 1.0, 1.0], [2.0, 2.0, 7.0]], [[4.0, 
3.0, 7.0], [8.0, 3.0, 3.0], [7.0, 3.0, 7.0]], [[4.0, 2.0, 3.0], [3.0, 4.0, 4.0], [2.0, 7.0, 6.0]]], [[[7.0, 2.0, 4.0], [6.0, 
4.0, 2.0], [7.0, 2.0, 5.0]], [[1.0, 7.0, 3.0], [6.0, 4.0, 4.0], [5.0, 5.0, 7.0]], [[7.0, 6.0, 7.0], [7.0, 1.0, 1.0], [4.0, 4.0, 
3.0]]], [[[6.0, 2.0, 2.0], [2.0, 7.0, 2.0], [5.0, 5.0, 5.0]], [[7.0, 2.0, 2.0], [1.0, 5.0, 5.0], [4.0, 6.0, 3.0]], [[6.0, 8.0, 
6.0], [7.0, 8.0, 1.0], [2.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 4.0], [5.0, 5.0, 3.0], [3.0, 4.0, 7.0]], 
[[2.0, 3.0, 6.0], [3.0, 6.0, 3.0], [5.0, 2.0, 6.0]], [[7.0, 7.0, 4.0], [6.0, 4.0, 5.0], [3.0, 7.0, 2.0]]], [[[8.0, 7.0, 8.0], 
[7.0, 8.0, 1.0], [8.0, 4.0, 6.0]], [[6.0, 3.0, 8.0], [6.0, 8.0, 4.0], [6.0, 4.0, 3.0]], [[6.0, 8.0, 4.0], [1.0, 6.0, 7.0], 
[1.0, 5.0, 1.0]]], [[[7.0, 5.0, 2.0], [1.0, 4.0, 7.0], [5.0, 1.0, 2.0]], [[7.0, 3.0, 3.0], [7.0, 7.0, 7.0], [4.0, 4.0, 6.0]], 
[[7.0, 8.0, 7.0], [5.0, 7.0, 8.0], [7.0, 7.0, 7.0]]], [[[2.0, 2.0, 8.0], [4.0, 2.0, 7.0], [6.0, 4.0, 3.0]], [[7.0, 4.0, 5.0], 
[7.0, 5.0, 3.0], [3.0, 3.0, 4.0]], [[7.0, 1.0, 6.0], [4.0, 3.0, 7.0], [1.0, 8.0, 1.0]]]])
      val=numpy.array([[[[2.0, 2.0]], [[8.0, 2.0]], [[2.0, 1.0]]], [[[7.0, 3.0]], [[4.0, 5.0]], [[2.0, 3.0]]]])
      arg[2:,:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 7.0], [6.0, 3.0, 2.0], [2.0, 3.0, 8.0]], [[7.0, 7.0, 4.0], [7.0, 3.0, 3.0], [4.0, 
4.0, 5.0]], [[1.0, 6.0, 1.0], [5.0, 2.0, 3.0], [7.0, 4.0, 7.0]]], [[[5.0, 5.0, 4.0], [7.0, 1.0, 1.0], [2.0, 2.0, 7.0]], [[4.0, 
3.0, 7.0], [8.0, 3.0, 3.0], [7.0, 3.0, 7.0]], [[4.0, 2.0, 3.0], [3.0, 4.0, 4.0], [2.0, 7.0, 6.0]]], [[[7.0, 2.0, 4.0], [6.0, 
4.0, 2.0], [2.0, 2.0, 5.0]], [[1.0, 7.0, 3.0], [6.0, 4.0, 4.0], [8.0, 2.0, 7.0]], [[7.0, 6.0, 7.0], [7.0, 1.0, 1.0], [2.0, 1.0, 
3.0]]], [[[6.0, 2.0, 2.0], [2.0, 7.0, 2.0], [7.0, 3.0, 5.0]], [[7.0, 2.0, 2.0], [1.0, 5.0, 5.0], [4.0, 5.0, 3.0]], [[6.0, 8.0, 
6.0], [7.0, 8.0, 1.0], [2.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 4.0], [5.0, 5.0, 3.0], [3.0, 4.0, 7.0]], 
[[2.0, 3.0, 6.0], [3.0, 6.0, 3.0], [5.0, 2.0, 6.0]], [[7.0, 7.0, 4.0], [6.0, 4.0, 5.0], [3.0, 7.0, 2.0]]], [[[8.0, 7.0, 8.0], 
[7.0, 8.0, 1.0], [8.0, 4.0, 6.0]], [[6.0, 3.0, 8.0], [6.0, 8.0, 4.0], [6.0, 4.0, 3.0]], [[6.0, 8.0, 4.0], [1.0, 6.0, 7.0], 
[1.0, 5.0, 1.0]]], [[[7.0, 5.0, 2.0], [1.0, 4.0, 7.0], [2.0, 2.0, 2.0]], [[7.0, 3.0, 3.0], [7.0, 7.0, 7.0], [8.0, 2.0, 6.0]], 
[[7.0, 8.0, 7.0], [5.0, 7.0, 8.0], [2.0, 1.0, 7.0]]], [[[2.0, 2.0, 8.0], [4.0, 2.0, 7.0], [7.0, 3.0, 3.0]], [[7.0, 4.0, 5.0], 
[7.0, 5.0, 3.0], [4.0, 5.0, 4.0]], [[7.0, 1.0, 6.0], [4.0, 3.0, 7.0], [2.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LCLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 3.0], [3.0, 6.0, 5.0], [8.0, 1.0, 2.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 7.0], [8.0, 
2.0, 2.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 1.0], [7.0, 2.0, 6.0]]], [[[6.0, 6.0, 3.0], [8.0, 7.0, 6.0], [8.0, 5.0, 2.0]], [[3.0, 
6.0, 2.0], [8.0, 6.0, 7.0], [6.0, 1.0, 1.0]], [[4.0, 5.0, 2.0], [5.0, 6.0, 3.0], [3.0, 2.0, 4.0]]], [[[3.0, 6.0, 2.0], [8.0, 
5.0, 5.0], [3.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [6.0, 1.0, 7.0], [6.0, 1.0, 2.0]], [[2.0, 8.0, 8.0], [8.0, 5.0, 6.0], [2.0, 8.0, 
1.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 8.0], [2.0, 2.0, 3.0]], [[8.0, 1.0, 1.0], [3.0, 1.0, 1.0], [5.0, 3.0, 8.0]], [[5.0, 7.0, 
1.0], [6.0, 3.0, 1.0], [6.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 6.0], [8.0, 6.0, 3.0], [2.0, 7.0, 8.0]], 
[[8.0, 5.0, 5.0], [4.0, 2.0, 6.0], [8.0, 6.0, 6.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 2.0], [6.0, 8.0, 3.0]]], [[[6.0, 1.0, 3.0], 
[5.0, 4.0, 2.0], [6.0, 3.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 5.0, 7.0], [3.0, 8.0, 8.0]], [[2.0, 3.0, 4.0], [6.0, 8.0, 5.0], 
[6.0, 3.0, 7.0]]], [[[7.0, 8.0, 5.0], [2.0, 8.0, 3.0], [2.0, 8.0, 7.0]], [[1.0, 2.0, 8.0], [3.0, 5.0, 2.0], [1.0, 6.0, 6.0]], 
[[3.0, 6.0, 5.0], [6.0, 2.0, 8.0], [5.0, 6.0, 3.0]]], [[[8.0, 3.0, 5.0], [3.0, 5.0, 6.0], [8.0, 1.0, 8.0]], [[5.0, 8.0, 3.0], 
[8.0, 5.0, 1.0], [8.0, 7.0, 2.0]], [[1.0, 7.0, 6.0], [3.0, 1.0, 2.0], [3.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[[1.0, 1.0]], [[8.0, 7.0]], [[1.0, 1.0]]], [[[8.0, 8.0]], [[4.0, 7.0]], [[3.0, 
6.0]]]]),self.functionspace)
      arg[2:,:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [3.0, 6.0, 5.0], [8.0, 1.0, 2.0]], [[6.0, 1.0, 7.0], [3.0, 7.0, 7.0], [8.0, 
2.0, 2.0]], [[6.0, 8.0, 2.0], [3.0, 2.0, 1.0], [7.0, 2.0, 6.0]]], [[[6.0, 6.0, 3.0], [8.0, 7.0, 6.0], [8.0, 5.0, 2.0]], [[3.0, 
6.0, 2.0], [8.0, 6.0, 7.0], [6.0, 1.0, 1.0]], [[4.0, 5.0, 2.0], [5.0, 6.0, 3.0], [3.0, 2.0, 4.0]]], [[[3.0, 6.0, 2.0], [8.0, 
5.0, 5.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 3.0], [6.0, 1.0, 7.0], [8.0, 7.0, 2.0]], [[2.0, 8.0, 8.0], [8.0, 5.0, 6.0], [1.0, 1.0, 
1.0]]], [[[7.0, 8.0, 1.0], [4.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[8.0, 1.0, 1.0], [3.0, 1.0, 1.0], [4.0, 7.0, 8.0]], [[5.0, 7.0, 
1.0], [6.0, 3.0, 1.0], [3.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 6.0], [8.0, 6.0, 3.0], [2.0, 7.0, 8.0]], 
[[8.0, 5.0, 5.0], [4.0, 2.0, 6.0], [8.0, 6.0, 6.0]], [[4.0, 6.0, 2.0], [3.0, 5.0, 2.0], [6.0, 8.0, 3.0]]], [[[6.0, 1.0, 3.0], 
[5.0, 4.0, 2.0], [6.0, 3.0, 7.0]], [[2.0, 3.0, 3.0], [2.0, 5.0, 7.0], [3.0, 8.0, 8.0]], [[2.0, 3.0, 4.0], [6.0, 8.0, 5.0], 
[6.0, 3.0, 7.0]]], [[[7.0, 8.0, 5.0], [2.0, 8.0, 3.0], [1.0, 1.0, 7.0]], [[1.0, 2.0, 8.0], [3.0, 5.0, 2.0], [8.0, 7.0, 6.0]], 
[[3.0, 6.0, 5.0], [6.0, 2.0, 8.0], [1.0, 1.0, 3.0]]], [[[8.0, 3.0, 5.0], [3.0, 5.0, 6.0], [8.0, 8.0, 8.0]], [[5.0, 8.0, 3.0], 
[8.0, 5.0, 1.0], [4.0, 7.0, 2.0]], [[1.0, 7.0, 6.0], [3.0, 1.0, 2.0], [3.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LCLU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 8.0], [4.0, 7.0, 8.0], [4.0, 1.0, 4.0]], [[6.0, 1.0, 4.0], [4.0, 3.0, 7.0], [8.0, 
6.0, 3.0]], [[8.0, 1.0, 3.0], [6.0, 2.0, 7.0], [7.0, 5.0, 1.0]]], [[[4.0, 7.0, 4.0], [3.0, 1.0, 4.0], [3.0, 1.0, 2.0]], [[4.0, 
3.0, 8.0], [2.0, 5.0, 2.0], [6.0, 5.0, 6.0]], [[8.0, 4.0, 4.0], [4.0, 6.0, 2.0], [5.0, 2.0, 1.0]]], [[[4.0, 7.0, 6.0], [5.0, 
2.0, 5.0], [2.0, 2.0, 2.0]], [[5.0, 8.0, 6.0], [8.0, 7.0, 8.0], [7.0, 7.0, 5.0]], [[2.0, 6.0, 8.0], [1.0, 4.0, 2.0], [4.0, 4.0, 
1.0]]], [[[5.0, 7.0, 1.0], [5.0, 8.0, 1.0], [8.0, 1.0, 8.0]], [[6.0, 8.0, 1.0], [3.0, 4.0, 3.0], [8.0, 5.0, 5.0]], [[4.0, 7.0, 
6.0], [2.0, 4.0, 1.0], [8.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 6.0], [4.0, 5.0, 4.0], [2.0, 4.0, 8.0]], 
[[4.0, 8.0, 6.0], [1.0, 7.0, 6.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 1.0, 1.0], [6.0, 4.0, 2.0]]], [[[1.0, 6.0, 8.0], 
[2.0, 4.0, 7.0], [4.0, 5.0, 8.0]], [[2.0, 6.0, 7.0], [5.0, 4.0, 7.0], [6.0, 2.0, 2.0]], [[1.0, 7.0, 4.0], [1.0, 2.0, 3.0], 
[1.0, 6.0, 1.0]]], [[[7.0, 4.0, 8.0], [3.0, 7.0, 2.0], [8.0, 4.0, 1.0]], [[3.0, 1.0, 3.0], [4.0, 2.0, 8.0], [8.0, 3.0, 2.0]], 
[[5.0, 8.0, 4.0], [8.0, 4.0, 3.0], [8.0, 8.0, 5.0]]], [[[2.0, 5.0, 5.0], [7.0, 8.0, 3.0], [7.0, 1.0, 6.0]], [[1.0, 6.0, 1.0], 
[6.0, 4.0, 5.0], [6.0, 1.0, 4.0]], [[6.0, 1.0, 1.0], [2.0, 1.0, 3.0], [4.0, 1.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 5.0]], [[3.0, 1.0]], [[1.0, 4.0]]], [[[5.0, 5.0]], [[7.0, 6.0]], [[2.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 7.0]], [[2.0, 4.0]], [[7.0, 1.0]]], [[[2.0, 3.0]], [[3.0, 3.0]], [[3.0, 2.0]]]])
      arg[2:,:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 8.0], [4.0, 7.0, 8.0], [4.0, 1.0, 4.0]], [[6.0, 1.0, 4.0], [4.0, 3.0, 7.0], [8.0, 
6.0, 3.0]], [[8.0, 1.0, 3.0], [6.0, 2.0, 7.0], [7.0, 5.0, 1.0]]], [[[4.0, 7.0, 4.0], [3.0, 1.0, 4.0], [3.0, 1.0, 2.0]], [[4.0, 
3.0, 8.0], [2.0, 5.0, 2.0], [6.0, 5.0, 6.0]], [[8.0, 4.0, 4.0], [4.0, 6.0, 2.0], [5.0, 2.0, 1.0]]], [[[4.0, 7.0, 6.0], [5.0, 
2.0, 5.0], [7.0, 5.0, 2.0]], [[5.0, 8.0, 6.0], [8.0, 7.0, 8.0], [3.0, 1.0, 5.0]], [[2.0, 6.0, 8.0], [1.0, 4.0, 2.0], [1.0, 4.0, 
1.0]]], [[[5.0, 7.0, 1.0], [5.0, 8.0, 1.0], [5.0, 5.0, 8.0]], [[6.0, 8.0, 1.0], [3.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[4.0, 7.0, 
6.0], [2.0, 4.0, 1.0], [2.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 6.0], [4.0, 5.0, 4.0], [2.0, 4.0, 8.0]], 
[[4.0, 8.0, 6.0], [1.0, 7.0, 6.0], [4.0, 5.0, 1.0]], [[3.0, 8.0, 6.0], [2.0, 1.0, 1.0], [6.0, 4.0, 2.0]]], [[[1.0, 6.0, 8.0], 
[2.0, 4.0, 7.0], [4.0, 5.0, 8.0]], [[2.0, 6.0, 7.0], [5.0, 4.0, 7.0], [6.0, 2.0, 2.0]], [[1.0, 7.0, 4.0], [1.0, 2.0, 3.0], 
[1.0, 6.0, 1.0]]], [[[7.0, 4.0, 8.0], [3.0, 7.0, 2.0], [6.0, 7.0, 1.0]], [[3.0, 1.0, 3.0], [4.0, 2.0, 8.0], [2.0, 4.0, 2.0]], 
[[5.0, 8.0, 4.0], [8.0, 4.0, 3.0], [7.0, 1.0, 5.0]]], [[[2.0, 5.0, 5.0], [7.0, 8.0, 3.0], [2.0, 3.0, 6.0]], [[1.0, 6.0, 1.0], 
[6.0, 4.0, 5.0], [3.0, 3.0, 4.0]], [[6.0, 1.0, 1.0], [2.0, 1.0, 3.0], [3.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 7.0], [7.0, 7.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 1.0], [6.0, 8.0, 8.0], [8.0, 
5.0, 2.0]], [[5.0, 4.0, 8.0], [4.0, 4.0, 4.0], [2.0, 4.0, 6.0]]], [[[6.0, 8.0, 2.0], [3.0, 3.0, 8.0], [5.0, 7.0, 1.0]], [[3.0, 
1.0, 8.0], [6.0, 7.0, 6.0], [3.0, 1.0, 3.0]], [[3.0, 5.0, 5.0], [8.0, 2.0, 1.0], [1.0, 8.0, 7.0]]], [[[6.0, 5.0, 7.0], [6.0, 
3.0, 7.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 8.0], [1.0, 7.0, 2.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 4.0], [1.0, 3.0, 4.0], [7.0, 8.0, 
2.0]]], [[[8.0, 7.0, 1.0], [5.0, 8.0, 5.0], [2.0, 6.0, 7.0]], [[4.0, 6.0, 7.0], [3.0, 7.0, 5.0], [5.0, 3.0, 8.0]], [[8.0, 7.0, 
4.0], [4.0, 4.0, 5.0], [7.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 4.0], [8.0, 7.0, 7.0], [1.0, 3.0, 5.0]], 
[[2.0, 3.0, 5.0], [6.0, 7.0, 3.0], [5.0, 6.0, 7.0]], [[5.0, 4.0, 1.0], [4.0, 1.0, 5.0], [3.0, 7.0, 1.0]]], [[[3.0, 5.0, 3.0], 
[3.0, 3.0, 3.0], [5.0, 1.0, 8.0]], [[8.0, 8.0, 5.0], [2.0, 3.0, 8.0], [7.0, 4.0, 1.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 1.0], 
[1.0, 7.0, 5.0]]], [[[4.0, 3.0, 2.0], [1.0, 2.0, 3.0], [8.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [3.0, 6.0, 5.0], [6.0, 8.0, 8.0]], 
[[6.0, 7.0, 3.0], [1.0, 6.0, 3.0], [5.0, 5.0, 1.0]]], [[[4.0, 5.0, 2.0], [3.0, 4.0, 7.0], [2.0, 2.0, 6.0]], [[7.0, 2.0, 5.0], 
[4.0, 6.0, 2.0], [7.0, 4.0, 2.0]], [[7.0, 5.0, 3.0], [3.0, 1.0, 2.0], [7.0, 6.0, 1.0]]]])
      val=numpy.array([[3.0, 3.0, 2.0], [6.0, 4.0, 1.0]])
      arg[1:3,1,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [7.0, 7.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 1.0], [6.0, 8.0, 8.0], [8.0, 
5.0, 2.0]], [[5.0, 4.0, 8.0], [4.0, 4.0, 4.0], [2.0, 4.0, 6.0]]], [[[6.0, 8.0, 2.0], [3.0, 3.0, 8.0], [5.0, 7.0, 1.0]], [[3.0, 
1.0, 8.0], [3.0, 3.0, 2.0], [3.0, 1.0, 3.0]], [[3.0, 5.0, 5.0], [8.0, 2.0, 1.0], [1.0, 8.0, 7.0]]], [[[6.0, 5.0, 7.0], [6.0, 
3.0, 7.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 8.0], [6.0, 4.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 4.0], [1.0, 3.0, 4.0], [7.0, 8.0, 
2.0]]], [[[8.0, 7.0, 1.0], [5.0, 8.0, 5.0], [2.0, 6.0, 7.0]], [[4.0, 6.0, 7.0], [3.0, 7.0, 5.0], [5.0, 3.0, 8.0]], [[8.0, 7.0, 
4.0], [4.0, 4.0, 5.0], [7.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 4.0], [8.0, 7.0, 7.0], [1.0, 3.0, 5.0]], 
[[2.0, 3.0, 5.0], [6.0, 7.0, 3.0], [5.0, 6.0, 7.0]], [[5.0, 4.0, 1.0], [4.0, 1.0, 5.0], [3.0, 7.0, 1.0]]], [[[3.0, 5.0, 3.0], 
[3.0, 3.0, 3.0], [5.0, 1.0, 8.0]], [[8.0, 8.0, 5.0], [3.0, 3.0, 2.0], [7.0, 4.0, 1.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 1.0], 
[1.0, 7.0, 5.0]]], [[[4.0, 3.0, 2.0], [1.0, 2.0, 3.0], [8.0, 7.0, 6.0]], [[5.0, 1.0, 1.0], [6.0, 4.0, 1.0], [6.0, 8.0, 8.0]], 
[[6.0, 7.0, 3.0], [1.0, 6.0, 3.0], [5.0, 5.0, 1.0]]], [[[4.0, 5.0, 2.0], [3.0, 4.0, 7.0], [2.0, 2.0, 6.0]], [[7.0, 2.0, 5.0], 
[4.0, 6.0, 2.0], [7.0, 4.0, 2.0]], [[7.0, 5.0, 3.0], [3.0, 1.0, 2.0], [7.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 6.0], [7.0, 7.0, 2.0], [1.0, 2.0, 6.0]], [[2.0, 4.0, 7.0], [8.0, 5.0, 7.0], [4.0, 
3.0, 8.0]], [[7.0, 2.0, 3.0], [1.0, 1.0, 2.0], [4.0, 6.0, 7.0]]], [[[7.0, 3.0, 3.0], [7.0, 2.0, 6.0], [8.0, 1.0, 5.0]], [[5.0, 
2.0, 7.0], [2.0, 8.0, 5.0], [5.0, 5.0, 5.0]], [[8.0, 5.0, 3.0], [2.0, 7.0, 7.0], [3.0, 7.0, 1.0]]], [[[3.0, 5.0, 1.0], [4.0, 
5.0, 4.0], [6.0, 6.0, 4.0]], [[3.0, 4.0, 1.0], [5.0, 3.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 3.0, 1.0], [2.0, 1.0, 8.0], [1.0, 4.0, 
7.0]]], [[[1.0, 8.0, 8.0], [1.0, 5.0, 5.0], [2.0, 8.0, 8.0]], [[5.0, 3.0, 2.0], [8.0, 1.0, 7.0], [6.0, 7.0, 7.0]], [[4.0, 1.0, 
1.0], [7.0, 7.0, 8.0], [6.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 7.0], [1.0, 4.0, 7.0], [2.0, 3.0, 6.0]], 
[[8.0, 1.0, 2.0], [6.0, 3.0, 4.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 6.0], [7.0, 6.0, 2.0], [8.0, 5.0, 3.0]]], [[[6.0, 8.0, 1.0], 
[4.0, 4.0, 8.0], [6.0, 4.0, 6.0]], [[3.0, 1.0, 2.0], [8.0, 8.0, 6.0], [1.0, 6.0, 1.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 8.0], 
[3.0, 2.0, 5.0]]], [[[4.0, 4.0, 3.0], [5.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[6.0, 4.0, 4.0], [5.0, 4.0, 2.0], [3.0, 6.0, 1.0]], 
[[2.0, 1.0, 6.0], [2.0, 6.0, 7.0], [4.0, 7.0, 1.0]]], [[[3.0, 6.0, 2.0], [3.0, 3.0, 8.0], [1.0, 4.0, 8.0]], [[6.0, 4.0, 2.0], 
[2.0, 5.0, 5.0], [8.0, 1.0, 5.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 1.0], [3.0, 4.0, 3.0]]]])
      val=Data(numpy.array([[3.0, 7.0, 3.0], [4.0, 3.0, 6.0]]),self.functionspace)
      arg[1:3,1,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 6.0], [7.0, 7.0, 2.0], [1.0, 2.0, 6.0]], [[2.0, 4.0, 7.0], [8.0, 5.0, 7.0], [4.0, 
3.0, 8.0]], [[7.0, 2.0, 3.0], [1.0, 1.0, 2.0], [4.0, 6.0, 7.0]]], [[[7.0, 3.0, 3.0], [7.0, 2.0, 6.0], [8.0, 1.0, 5.0]], [[5.0, 
2.0, 7.0], [3.0, 7.0, 3.0], [5.0, 5.0, 5.0]], [[8.0, 5.0, 3.0], [2.0, 7.0, 7.0], [3.0, 7.0, 1.0]]], [[[3.0, 5.0, 1.0], [4.0, 
5.0, 4.0], [6.0, 6.0, 4.0]], [[3.0, 4.0, 1.0], [4.0, 3.0, 6.0], [4.0, 1.0, 2.0]], [[5.0, 3.0, 1.0], [2.0, 1.0, 8.0], [1.0, 4.0, 
7.0]]], [[[1.0, 8.0, 8.0], [1.0, 5.0, 5.0], [2.0, 8.0, 8.0]], [[5.0, 3.0, 2.0], [8.0, 1.0, 7.0], [6.0, 7.0, 7.0]], [[4.0, 1.0, 
1.0], [7.0, 7.0, 8.0], [6.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 7.0], [1.0, 4.0, 7.0], [2.0, 3.0, 6.0]], 
[[8.0, 1.0, 2.0], [6.0, 3.0, 4.0], [8.0, 2.0, 2.0]], [[7.0, 1.0, 6.0], [7.0, 6.0, 2.0], [8.0, 5.0, 3.0]]], [[[6.0, 8.0, 1.0], 
[4.0, 4.0, 8.0], [6.0, 4.0, 6.0]], [[3.0, 1.0, 2.0], [3.0, 7.0, 3.0], [1.0, 6.0, 1.0]], [[4.0, 4.0, 5.0], [6.0, 1.0, 8.0], 
[3.0, 2.0, 5.0]]], [[[4.0, 4.0, 3.0], [5.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[6.0, 4.0, 4.0], [4.0, 3.0, 6.0], [3.0, 6.0, 1.0]], 
[[2.0, 1.0, 6.0], [2.0, 6.0, 7.0], [4.0, 7.0, 1.0]]], [[[3.0, 6.0, 2.0], [3.0, 3.0, 8.0], [1.0, 4.0, 8.0]], [[6.0, 4.0, 2.0], 
[2.0, 5.0, 5.0], [8.0, 1.0, 5.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 1.0], [3.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPPC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 1.0], [8.0, 7.0, 5.0], [6.0, 6.0, 7.0]], [[4.0, 7.0, 7.0], [4.0, 3.0, 1.0], [4.0, 
5.0, 3.0]], [[7.0, 3.0, 8.0], [7.0, 8.0, 1.0], [6.0, 7.0, 1.0]]], [[[2.0, 4.0, 4.0], [5.0, 1.0, 4.0], [8.0, 6.0, 8.0]], [[7.0, 
4.0, 2.0], [8.0, 8.0, 2.0], [1.0, 1.0, 8.0]], [[2.0, 8.0, 6.0], [2.0, 1.0, 1.0], [5.0, 6.0, 6.0]]], [[[5.0, 8.0, 2.0], [6.0, 
5.0, 6.0], [8.0, 7.0, 7.0]], [[3.0, 6.0, 5.0], [1.0, 4.0, 4.0], [3.0, 1.0, 5.0]], [[5.0, 6.0, 8.0], [5.0, 2.0, 4.0], [3.0, 8.0, 
2.0]]], [[[3.0, 7.0, 8.0], [8.0, 3.0, 4.0], [8.0, 2.0, 1.0]], [[1.0, 3.0, 8.0], [2.0, 8.0, 5.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 
3.0], [2.0, 1.0, 4.0], [5.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 6.0], [4.0, 6.0, 2.0], [7.0, 8.0, 7.0]], 
[[4.0, 2.0, 2.0], [5.0, 8.0, 2.0], [2.0, 1.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 7.0, 1.0], [7.0, 8.0, 6.0]]], [[[4.0, 5.0, 3.0], 
[6.0, 5.0, 7.0], [5.0, 1.0, 4.0]], [[2.0, 4.0, 8.0], [1.0, 8.0, 7.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 1.0], [6.0, 8.0, 8.0], 
[7.0, 3.0, 6.0]]], [[[6.0, 2.0, 6.0], [8.0, 8.0, 1.0], [1.0, 6.0, 1.0]], [[5.0, 1.0, 3.0], [2.0, 5.0, 7.0], [6.0, 6.0, 1.0]], 
[[7.0, 5.0, 2.0], [5.0, 3.0, 8.0], [1.0, 5.0, 6.0]]], [[[1.0, 5.0, 7.0], [1.0, 5.0, 5.0], [5.0, 6.0, 7.0]], [[2.0, 4.0, 7.0], 
[8.0, 1.0, 1.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 8.0], [4.0, 8.0, 7.0], [4.0, 7.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[5.0, 8.0, 1.0], [4.0, 6.0, 5.0]])+(1.-msk_val)*numpy.array([[3.0, 3.0, 1.0], [4.0, 5.0, 4.0]])
      arg[1:3,1,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [8.0, 7.0, 5.0], [6.0, 6.0, 7.0]], [[4.0, 7.0, 7.0], [4.0, 3.0, 1.0], [4.0, 
5.0, 3.0]], [[7.0, 3.0, 8.0], [7.0, 8.0, 1.0], [6.0, 7.0, 1.0]]], [[[2.0, 4.0, 4.0], [5.0, 1.0, 4.0], [8.0, 6.0, 8.0]], [[7.0, 
4.0, 2.0], [5.0, 8.0, 1.0], [1.0, 1.0, 8.0]], [[2.0, 8.0, 6.0], [2.0, 1.0, 1.0], [5.0, 6.0, 6.0]]], [[[5.0, 8.0, 2.0], [6.0, 
5.0, 6.0], [8.0, 7.0, 7.0]], [[3.0, 6.0, 5.0], [4.0, 6.0, 5.0], [3.0, 1.0, 5.0]], [[5.0, 6.0, 8.0], [5.0, 2.0, 4.0], [3.0, 8.0, 
2.0]]], [[[3.0, 7.0, 8.0], [8.0, 3.0, 4.0], [8.0, 2.0, 1.0]], [[1.0, 3.0, 8.0], [2.0, 8.0, 5.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 
3.0], [2.0, 1.0, 4.0], [5.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [4.0, 6.0, 2.0], [7.0, 8.0, 7.0]], 
[[4.0, 2.0, 2.0], [5.0, 8.0, 2.0], [2.0, 1.0, 3.0]], [[7.0, 7.0, 1.0], [5.0, 7.0, 1.0], [7.0, 8.0, 6.0]]], [[[4.0, 5.0, 3.0], 
[6.0, 5.0, 7.0], [5.0, 1.0, 4.0]], [[2.0, 4.0, 8.0], [3.0, 3.0, 1.0], [8.0, 4.0, 5.0]], [[1.0, 7.0, 1.0], [6.0, 8.0, 8.0], 
[7.0, 3.0, 6.0]]], [[[6.0, 2.0, 6.0], [8.0, 8.0, 1.0], [1.0, 6.0, 1.0]], [[5.0, 1.0, 3.0], [4.0, 5.0, 4.0], [6.0, 6.0, 1.0]], 
[[7.0, 5.0, 2.0], [5.0, 3.0, 8.0], [1.0, 5.0, 6.0]]], [[[1.0, 5.0, 7.0], [1.0, 5.0, 5.0], [5.0, 6.0, 7.0]], [[2.0, 4.0, 7.0], 
[8.0, 1.0, 1.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 8.0], [4.0, 8.0, 7.0], [4.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 4.0], [1.0, 6.0, 6.0], [7.0, 8.0, 4.0]], [[8.0, 3.0, 7.0], [8.0, 3.0, 8.0], [2.0, 
4.0, 3.0]], [[2.0, 2.0, 3.0], [4.0, 3.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 7.0, 8.0], [1.0, 4.0, 4.0], [7.0, 5.0, 6.0]], [[4.0, 
6.0, 6.0], [7.0, 6.0, 3.0], [6.0, 1.0, 2.0]], [[1.0, 7.0, 4.0], [4.0, 4.0, 8.0], [1.0, 6.0, 1.0]]], [[[4.0, 3.0, 5.0], [3.0, 
3.0, 5.0], [6.0, 3.0, 6.0]], [[4.0, 5.0, 2.0], [2.0, 8.0, 3.0], [2.0, 6.0, 1.0]], [[8.0, 6.0, 1.0], [7.0, 8.0, 3.0], [4.0, 4.0, 
3.0]]], [[[4.0, 6.0, 2.0], [7.0, 3.0, 1.0], [3.0, 4.0, 6.0]], [[2.0, 3.0, 2.0], [1.0, 7.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 8.0, 
8.0], [4.0, 6.0, 2.0], [5.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 4.0], [7.0, 2.0, 5.0], [5.0, 1.0, 2.0]], 
[[6.0, 7.0, 4.0], [3.0, 8.0, 5.0], [4.0, 6.0, 6.0]], [[1.0, 8.0, 2.0], [6.0, 8.0, 8.0], [3.0, 6.0, 4.0]]], [[[1.0, 2.0, 6.0], 
[6.0, 6.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 5.0, 7.0], [8.0, 2.0, 1.0], [5.0, 8.0, 8.0]], [[7.0, 2.0, 4.0], [1.0, 5.0, 6.0], 
[6.0, 5.0, 4.0]]], [[[6.0, 1.0, 5.0], [2.0, 1.0, 4.0], [8.0, 8.0, 6.0]], [[5.0, 2.0, 7.0], [8.0, 5.0, 7.0], [3.0, 7.0, 7.0]], 
[[7.0, 1.0, 6.0], [8.0, 6.0, 1.0], [6.0, 4.0, 3.0]]], [[[2.0, 5.0, 7.0], [5.0, 1.0, 1.0], [2.0, 1.0, 1.0]], [[4.0, 6.0, 4.0], 
[3.0, 5.0, 1.0], [7.0, 6.0, 5.0]], [[1.0, 2.0, 5.0], [6.0, 6.0, 3.0], [4.0, 5.0, 2.0]]]])
      val=numpy.array([[[[1.0, 5.0, 1.0]], [[7.0, 2.0, 4.0]], [[6.0, 1.0, 7.0]]], [[[5.0, 3.0, 7.0]], [[8.0, 6.0, 8.0]], [[3.0, 
2.0, 3.0]]]])
      arg[1:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 4.0], [1.0, 6.0, 6.0], [7.0, 8.0, 4.0]], [[8.0, 3.0, 7.0], [8.0, 3.0, 8.0], [2.0, 
4.0, 3.0]], [[2.0, 2.0, 3.0], [4.0, 3.0, 8.0], [4.0, 3.0, 8.0]]], [[[8.0, 7.0, 8.0], [1.0, 5.0, 1.0], [7.0, 5.0, 6.0]], [[4.0, 
6.0, 6.0], [7.0, 2.0, 4.0], [6.0, 1.0, 2.0]], [[1.0, 7.0, 4.0], [6.0, 1.0, 7.0], [1.0, 6.0, 1.0]]], [[[4.0, 3.0, 5.0], [5.0, 
3.0, 7.0], [6.0, 3.0, 6.0]], [[4.0, 5.0, 2.0], [8.0, 6.0, 8.0], [2.0, 6.0, 1.0]], [[8.0, 6.0, 1.0], [3.0, 2.0, 3.0], [4.0, 4.0, 
3.0]]], [[[4.0, 6.0, 2.0], [7.0, 3.0, 1.0], [3.0, 4.0, 6.0]], [[2.0, 3.0, 2.0], [1.0, 7.0, 1.0], [8.0, 4.0, 7.0]], [[5.0, 8.0, 
8.0], [4.0, 6.0, 2.0], [5.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 4.0], [7.0, 2.0, 5.0], [5.0, 1.0, 2.0]], 
[[6.0, 7.0, 4.0], [3.0, 8.0, 5.0], [4.0, 6.0, 6.0]], [[1.0, 8.0, 2.0], [6.0, 8.0, 8.0], [3.0, 6.0, 4.0]]], [[[1.0, 2.0, 6.0], 
[1.0, 5.0, 1.0], [4.0, 7.0, 6.0]], [[3.0, 5.0, 7.0], [7.0, 2.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 2.0, 4.0], [6.0, 1.0, 7.0], 
[6.0, 5.0, 4.0]]], [[[6.0, 1.0, 5.0], [5.0, 3.0, 7.0], [8.0, 8.0, 6.0]], [[5.0, 2.0, 7.0], [8.0, 6.0, 8.0], [3.0, 7.0, 7.0]], 
[[7.0, 1.0, 6.0], [3.0, 2.0, 3.0], [6.0, 4.0, 3.0]]], [[[2.0, 5.0, 7.0], [5.0, 1.0, 1.0], [2.0, 1.0, 1.0]], [[4.0, 6.0, 4.0], 
[3.0, 5.0, 1.0], [7.0, 6.0, 5.0]], [[1.0, 2.0, 5.0], [6.0, 6.0, 3.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 4.0], [8.0, 7.0, 7.0], [1.0, 5.0, 6.0]], [[1.0, 1.0, 6.0], [6.0, 5.0, 1.0], [2.0, 
4.0, 1.0]], [[6.0, 5.0, 3.0], [6.0, 6.0, 5.0], [2.0, 1.0, 1.0]]], [[[7.0, 4.0, 6.0], [7.0, 8.0, 6.0], [5.0, 8.0, 5.0]], [[1.0, 
5.0, 5.0], [2.0, 2.0, 6.0], [1.0, 8.0, 1.0]], [[1.0, 1.0, 3.0], [3.0, 5.0, 8.0], [3.0, 5.0, 4.0]]], [[[7.0, 3.0, 5.0], [5.0, 
3.0, 2.0], [6.0, 3.0, 3.0]], [[4.0, 3.0, 7.0], [7.0, 1.0, 7.0], [8.0, 2.0, 6.0]], [[5.0, 1.0, 3.0], [6.0, 7.0, 2.0], [8.0, 8.0, 
5.0]]], [[[8.0, 5.0, 8.0], [7.0, 2.0, 1.0], [7.0, 2.0, 2.0]], [[6.0, 6.0, 7.0], [2.0, 3.0, 2.0], [7.0, 2.0, 1.0]], [[6.0, 1.0, 
3.0], [7.0, 8.0, 1.0], [6.0, 2.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 7.0], [6.0, 6.0, 3.0], [5.0, 2.0, 1.0]], 
[[2.0, 2.0, 7.0], [2.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[1.0, 6.0, 6.0], [3.0, 4.0, 6.0], [5.0, 7.0, 4.0]]], [[[6.0, 3.0, 7.0], 
[6.0, 4.0, 1.0], [1.0, 5.0, 3.0]], [[8.0, 8.0, 2.0], [5.0, 5.0, 6.0], [8.0, 2.0, 3.0]], [[5.0, 5.0, 8.0], [5.0, 8.0, 6.0], 
[8.0, 2.0, 5.0]]], [[[2.0, 1.0, 2.0], [5.0, 7.0, 5.0], [8.0, 6.0, 7.0]], [[4.0, 8.0, 2.0], [5.0, 2.0, 7.0], [5.0, 7.0, 3.0]], 
[[2.0, 8.0, 2.0], [2.0, 1.0, 4.0], [2.0, 5.0, 8.0]]], [[[4.0, 1.0, 2.0], [3.0, 8.0, 8.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 5.0], 
[8.0, 3.0, 5.0], [1.0, 8.0, 3.0]], [[4.0, 6.0, 4.0], [7.0, 5.0, 4.0], [5.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[[[3.0, 8.0, 1.0]], [[3.0, 4.0, 8.0]], [[6.0, 6.0, 5.0]]], [[[4.0, 1.0, 2.0]], [[4.0, 4.0, 5.0]], 
[[2.0, 1.0, 7.0]]]]),self.functionspace)
      arg[1:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 4.0], [8.0, 7.0, 7.0], [1.0, 5.0, 6.0]], [[1.0, 1.0, 6.0], [6.0, 5.0, 1.0], [2.0, 
4.0, 1.0]], [[6.0, 5.0, 3.0], [6.0, 6.0, 5.0], [2.0, 1.0, 1.0]]], [[[7.0, 4.0, 6.0], [3.0, 8.0, 1.0], [5.0, 8.0, 5.0]], [[1.0, 
5.0, 5.0], [3.0, 4.0, 8.0], [1.0, 8.0, 1.0]], [[1.0, 1.0, 3.0], [6.0, 6.0, 5.0], [3.0, 5.0, 4.0]]], [[[7.0, 3.0, 5.0], [4.0, 
1.0, 2.0], [6.0, 3.0, 3.0]], [[4.0, 3.0, 7.0], [4.0, 4.0, 5.0], [8.0, 2.0, 6.0]], [[5.0, 1.0, 3.0], [2.0, 1.0, 7.0], [8.0, 8.0, 
5.0]]], [[[8.0, 5.0, 8.0], [7.0, 2.0, 1.0], [7.0, 2.0, 2.0]], [[6.0, 6.0, 7.0], [2.0, 3.0, 2.0], [7.0, 2.0, 1.0]], [[6.0, 1.0, 
3.0], [7.0, 8.0, 1.0], [6.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 7.0], [6.0, 6.0, 3.0], [5.0, 2.0, 1.0]], 
[[2.0, 2.0, 7.0], [2.0, 4.0, 7.0], [5.0, 4.0, 6.0]], [[1.0, 6.0, 6.0], [3.0, 4.0, 6.0], [5.0, 7.0, 4.0]]], [[[6.0, 3.0, 7.0], 
[3.0, 8.0, 1.0], [1.0, 5.0, 3.0]], [[8.0, 8.0, 2.0], [3.0, 4.0, 8.0], [8.0, 2.0, 3.0]], [[5.0, 5.0, 8.0], [6.0, 6.0, 5.0], 
[8.0, 2.0, 5.0]]], [[[2.0, 1.0, 2.0], [4.0, 1.0, 2.0], [8.0, 6.0, 7.0]], [[4.0, 8.0, 2.0], [4.0, 4.0, 5.0], [5.0, 7.0, 3.0]], 
[[2.0, 8.0, 2.0], [2.0, 1.0, 7.0], [2.0, 5.0, 8.0]]], [[[4.0, 1.0, 2.0], [3.0, 8.0, 8.0], [4.0, 2.0, 8.0]], [[6.0, 2.0, 5.0], 
[8.0, 3.0, 5.0], [1.0, 8.0, 3.0]], [[4.0, 6.0, 4.0], [7.0, 5.0, 4.0], [5.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 6.0], [5.0, 6.0, 1.0], [6.0, 3.0, 5.0]], [[6.0, 7.0, 1.0], [5.0, 4.0, 4.0], [7.0, 
3.0, 2.0]], [[4.0, 4.0, 7.0], [5.0, 7.0, 8.0], [2.0, 8.0, 1.0]]], [[[3.0, 8.0, 8.0], [7.0, 1.0, 7.0], [3.0, 1.0, 3.0]], [[3.0, 
8.0, 8.0], [7.0, 4.0, 7.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 4.0], [6.0, 6.0, 7.0], [3.0, 6.0, 5.0]]], [[[4.0, 2.0, 8.0], [4.0, 
4.0, 5.0], [1.0, 3.0, 1.0]], [[4.0, 2.0, 5.0], [5.0, 3.0, 3.0], [6.0, 8.0, 4.0]], [[2.0, 1.0, 3.0], [2.0, 1.0, 6.0], [6.0, 8.0, 
5.0]]], [[[1.0, 5.0, 6.0], [2.0, 2.0, 6.0], [2.0, 6.0, 4.0]], [[8.0, 1.0, 5.0], [4.0, 7.0, 7.0], [1.0, 3.0, 4.0]], [[1.0, 2.0, 
7.0], [6.0, 8.0, 7.0], [2.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 1.0], [6.0, 6.0, 7.0], [3.0, 1.0, 1.0]], 
[[4.0, 7.0, 6.0], [7.0, 5.0, 2.0], [3.0, 5.0, 8.0]], [[5.0, 6.0, 3.0], [2.0, 7.0, 5.0], [2.0, 8.0, 5.0]]], [[[4.0, 3.0, 2.0], 
[2.0, 8.0, 7.0], [8.0, 3.0, 5.0]], [[8.0, 6.0, 2.0], [3.0, 3.0, 5.0], [2.0, 6.0, 1.0]], [[7.0, 7.0, 3.0], [1.0, 2.0, 5.0], 
[8.0, 3.0, 5.0]]], [[[4.0, 8.0, 7.0], [1.0, 8.0, 3.0], [1.0, 7.0, 8.0]], [[1.0, 5.0, 1.0], [1.0, 3.0, 8.0], [4.0, 2.0, 3.0]], 
[[2.0, 8.0, 3.0], [8.0, 2.0, 8.0], [4.0, 5.0, 6.0]]], [[[7.0, 3.0, 2.0], [1.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[3.0, 2.0, 1.0], 
[1.0, 3.0, 5.0], [7.0, 6.0, 2.0]], [[7.0, 1.0, 1.0], [2.0, 4.0, 3.0], [7.0, 1.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 6.0, 5.0]], [[2.0, 6.0, 1.0]], [[1.0, 7.0, 2.0]]], [[[2.0, 1.0, 7.0]], [[2.0, 8.0, 
4.0]], [[5.0, 3.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 7.0, 4.0]], [[5.0, 1.0, 5.0]], [[4.0, 4.0, 2.0]]], [[[1.0, 3.0, 
5.0]], [[4.0, 5.0, 8.0]], [[8.0, 7.0, 2.0]]]])
      arg[1:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 6.0], [5.0, 6.0, 1.0], [6.0, 3.0, 5.0]], [[6.0, 7.0, 1.0], [5.0, 4.0, 4.0], [7.0, 
3.0, 2.0]], [[4.0, 4.0, 7.0], [5.0, 7.0, 8.0], [2.0, 8.0, 1.0]]], [[[3.0, 8.0, 8.0], [1.0, 6.0, 5.0], [3.0, 1.0, 3.0]], [[3.0, 
8.0, 8.0], [2.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 4.0], [1.0, 7.0, 2.0], [3.0, 6.0, 5.0]]], [[[4.0, 2.0, 8.0], [2.0, 
1.0, 7.0], [1.0, 3.0, 1.0]], [[4.0, 2.0, 5.0], [2.0, 8.0, 4.0], [6.0, 8.0, 4.0]], [[2.0, 1.0, 3.0], [5.0, 3.0, 3.0], [6.0, 8.0, 
5.0]]], [[[1.0, 5.0, 6.0], [2.0, 2.0, 6.0], [2.0, 6.0, 4.0]], [[8.0, 1.0, 5.0], [4.0, 7.0, 7.0], [1.0, 3.0, 4.0]], [[1.0, 2.0, 
7.0], [6.0, 8.0, 7.0], [2.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 1.0], [6.0, 6.0, 7.0], [3.0, 1.0, 1.0]], 
[[4.0, 7.0, 6.0], [7.0, 5.0, 2.0], [3.0, 5.0, 8.0]], [[5.0, 6.0, 3.0], [2.0, 7.0, 5.0], [2.0, 8.0, 5.0]]], [[[4.0, 3.0, 2.0], 
[5.0, 7.0, 4.0], [8.0, 3.0, 5.0]], [[8.0, 6.0, 2.0], [5.0, 1.0, 5.0], [2.0, 6.0, 1.0]], [[7.0, 7.0, 3.0], [4.0, 4.0, 2.0], 
[8.0, 3.0, 5.0]]], [[[4.0, 8.0, 7.0], [1.0, 3.0, 5.0], [1.0, 7.0, 8.0]], [[1.0, 5.0, 1.0], [4.0, 5.0, 8.0], [4.0, 2.0, 3.0]], 
[[2.0, 8.0, 3.0], [8.0, 7.0, 2.0], [4.0, 5.0, 6.0]]], [[[7.0, 3.0, 2.0], [1.0, 2.0, 5.0], [3.0, 5.0, 3.0]], [[3.0, 2.0, 1.0], 
[1.0, 3.0, 5.0], [7.0, 6.0, 2.0]], [[7.0, 1.0, 1.0], [2.0, 4.0, 3.0], [7.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 4.0], [5.0, 4.0, 1.0], [2.0, 8.0, 4.0]], [[6.0, 2.0, 7.0], [3.0, 2.0, 5.0], [1.0, 
3.0, 6.0]], [[1.0, 2.0, 4.0], [6.0, 1.0, 2.0], [6.0, 7.0, 2.0]]], [[[8.0, 5.0, 8.0], [7.0, 2.0, 5.0], [7.0, 8.0, 5.0]], [[5.0, 
2.0, 2.0], [6.0, 4.0, 4.0], [4.0, 1.0, 8.0]], [[3.0, 1.0, 8.0], [1.0, 3.0, 3.0], [6.0, 5.0, 4.0]]], [[[6.0, 4.0, 4.0], [1.0, 
6.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 8.0, 5.0], [4.0, 3.0, 6.0], [8.0, 2.0, 3.0]], [[5.0, 6.0, 6.0], [2.0, 3.0, 4.0], [7.0, 6.0, 
8.0]]], [[[2.0, 5.0, 6.0], [3.0, 1.0, 6.0], [3.0, 4.0, 3.0]], [[8.0, 8.0, 8.0], [4.0, 7.0, 2.0], [2.0, 3.0, 4.0]], [[4.0, 1.0, 
7.0], [5.0, 6.0, 8.0], [8.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 6.0], [3.0, 6.0, 5.0], [1.0, 3.0, 8.0]], 
[[3.0, 5.0, 4.0], [5.0, 2.0, 5.0], [4.0, 7.0, 6.0]], [[1.0, 2.0, 8.0], [6.0, 1.0, 8.0], [5.0, 8.0, 6.0]]], [[[1.0, 5.0, 1.0], 
[2.0, 6.0, 7.0], [5.0, 4.0, 3.0]], [[1.0, 2.0, 6.0], [2.0, 4.0, 1.0], [4.0, 5.0, 1.0]], [[6.0, 3.0, 8.0], [5.0, 3.0, 1.0], 
[4.0, 8.0, 5.0]]], [[[5.0, 1.0, 4.0], [8.0, 1.0, 2.0], [3.0, 8.0, 5.0]], [[3.0, 6.0, 6.0], [6.0, 4.0, 2.0], [8.0, 1.0, 8.0]], 
[[2.0, 4.0, 6.0], [7.0, 5.0, 6.0], [8.0, 2.0, 1.0]]], [[[5.0, 4.0, 5.0], [2.0, 1.0, 8.0], [8.0, 1.0, 7.0]], [[1.0, 1.0, 3.0], 
[1.0, 4.0, 3.0], [6.0, 4.0, 4.0]], [[5.0, 5.0, 8.0], [4.0, 3.0, 7.0], [7.0, 2.0, 7.0]]]])
      val=numpy.array([[[[1.0, 5.0, 2.0], [6.0, 3.0, 1.0]], [[2.0, 5.0, 3.0], [7.0, 3.0, 5.0]], [[7.0, 8.0, 1.0], [7.0, 2.0, 
3.0]]], [[[2.0, 7.0, 6.0], [2.0, 3.0, 3.0]], [[2.0, 2.0, 3.0], [7.0, 4.0, 8.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 7.0]]], [[[2.0, 
7.0, 5.0], [1.0, 6.0, 7.0]], [[4.0, 8.0, 1.0], [4.0, 7.0, 8.0]], [[3.0, 3.0, 3.0], [8.0, 2.0, 1.0]]]])
      arg[:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 3.0, 1.0], [2.0, 8.0, 4.0]], [[2.0, 5.0, 3.0], [7.0, 3.0, 5.0], [1.0, 
3.0, 6.0]], [[7.0, 8.0, 1.0], [7.0, 2.0, 3.0], [6.0, 7.0, 2.0]]], [[[2.0, 7.0, 6.0], [2.0, 3.0, 3.0], [7.0, 8.0, 5.0]], [[2.0, 
2.0, 3.0], [7.0, 4.0, 8.0], [4.0, 1.0, 8.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 7.0], [6.0, 5.0, 4.0]]], [[[2.0, 7.0, 5.0], [1.0, 
6.0, 7.0], [7.0, 7.0, 4.0]], [[4.0, 8.0, 1.0], [4.0, 7.0, 8.0], [8.0, 2.0, 3.0]], [[3.0, 3.0, 3.0], [8.0, 2.0, 1.0], [7.0, 6.0, 
8.0]]], [[[2.0, 5.0, 6.0], [3.0, 1.0, 6.0], [3.0, 4.0, 3.0]], [[8.0, 8.0, 8.0], [4.0, 7.0, 2.0], [2.0, 3.0, 4.0]], [[4.0, 1.0, 
7.0], [5.0, 6.0, 8.0], [8.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 2.0], [6.0, 3.0, 1.0], [1.0, 3.0, 8.0]], 
[[2.0, 5.0, 3.0], [7.0, 3.0, 5.0], [4.0, 7.0, 6.0]], [[7.0, 8.0, 1.0], [7.0, 2.0, 3.0], [5.0, 8.0, 6.0]]], [[[2.0, 7.0, 6.0], 
[2.0, 3.0, 3.0], [5.0, 4.0, 3.0]], [[2.0, 2.0, 3.0], [7.0, 4.0, 8.0], [4.0, 5.0, 1.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 7.0], 
[4.0, 8.0, 5.0]]], [[[2.0, 7.0, 5.0], [1.0, 6.0, 7.0], [3.0, 8.0, 5.0]], [[4.0, 8.0, 1.0], [4.0, 7.0, 8.0], [8.0, 1.0, 8.0]], 
[[3.0, 3.0, 3.0], [8.0, 2.0, 1.0], [8.0, 2.0, 1.0]]], [[[5.0, 4.0, 5.0], [2.0, 1.0, 8.0], [8.0, 1.0, 7.0]], [[1.0, 1.0, 3.0], 
[1.0, 4.0, 3.0], [6.0, 4.0, 4.0]], [[5.0, 5.0, 8.0], [4.0, 3.0, 7.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 5.0], [8.0, 6.0, 6.0], [4.0, 4.0, 8.0]], [[4.0, 5.0, 1.0], [2.0, 5.0, 5.0], [8.0, 
6.0, 6.0]], [[3.0, 8.0, 7.0], [1.0, 2.0, 4.0], [5.0, 3.0, 7.0]]], [[[3.0, 5.0, 8.0], [1.0, 6.0, 7.0], [5.0, 4.0, 4.0]], [[8.0, 
8.0, 1.0], [3.0, 8.0, 3.0], [5.0, 6.0, 7.0]], [[3.0, 5.0, 3.0], [4.0, 3.0, 1.0], [2.0, 8.0, 5.0]]], [[[2.0, 8.0, 1.0], [8.0, 
2.0, 3.0], [6.0, 2.0, 5.0]], [[6.0, 5.0, 3.0], [2.0, 1.0, 3.0], [2.0, 1.0, 5.0]], [[4.0, 8.0, 2.0], [3.0, 4.0, 5.0], [4.0, 3.0, 
3.0]]], [[[7.0, 1.0, 7.0], [3.0, 8.0, 7.0], [7.0, 4.0, 6.0]], [[5.0, 5.0, 3.0], [8.0, 4.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 7.0, 
5.0], [1.0, 7.0, 3.0], [7.0, 5.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 1.0], [1.0, 5.0, 7.0], [2.0, 4.0, 6.0]], 
[[1.0, 3.0, 2.0], [6.0, 1.0, 5.0], [4.0, 1.0, 3.0]], [[4.0, 6.0, 3.0], [4.0, 4.0, 3.0], [3.0, 5.0, 5.0]]], [[[3.0, 4.0, 2.0], 
[1.0, 2.0, 8.0], [1.0, 8.0, 3.0]], [[4.0, 8.0, 3.0], [3.0, 7.0, 7.0], [6.0, 3.0, 1.0]], [[4.0, 2.0, 4.0], [4.0, 3.0, 2.0], 
[7.0, 4.0, 1.0]]], [[[3.0, 7.0, 3.0], [5.0, 8.0, 8.0], [7.0, 4.0, 4.0]], [[5.0, 5.0, 6.0], [8.0, 8.0, 3.0], [5.0, 8.0, 2.0]], 
[[4.0, 8.0, 7.0], [5.0, 4.0, 4.0], [2.0, 8.0, 5.0]]], [[[6.0, 5.0, 3.0], [6.0, 6.0, 5.0], [1.0, 7.0, 2.0]], [[1.0, 6.0, 3.0], 
[8.0, 6.0, 5.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 6.0], [6.0, 5.0, 5.0], [4.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[[1.0, 8.0, 5.0], [4.0, 7.0, 6.0]], [[7.0, 8.0, 6.0], [5.0, 3.0, 8.0]], [[2.0, 3.0, 7.0], [3.0, 
2.0, 1.0]]], [[[1.0, 2.0, 5.0], [8.0, 2.0, 3.0]], [[8.0, 8.0, 2.0], [6.0, 1.0, 7.0]], [[8.0, 4.0, 6.0], [2.0, 7.0, 1.0]]], 
[[[5.0, 7.0, 8.0], [2.0, 5.0, 6.0]], [[3.0, 4.0, 5.0], [3.0, 7.0, 3.0]], [[3.0, 8.0, 6.0], [7.0, 7.0, 
1.0]]]]),self.functionspace)
      arg[:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 5.0], [4.0, 7.0, 6.0], [4.0, 4.0, 8.0]], [[7.0, 8.0, 6.0], [5.0, 3.0, 8.0], [8.0, 
6.0, 6.0]], [[2.0, 3.0, 7.0], [3.0, 2.0, 1.0], [5.0, 3.0, 7.0]]], [[[1.0, 2.0, 5.0], [8.0, 2.0, 3.0], [5.0, 4.0, 4.0]], [[8.0, 
8.0, 2.0], [6.0, 1.0, 7.0], [5.0, 6.0, 7.0]], [[8.0, 4.0, 6.0], [2.0, 7.0, 1.0], [2.0, 8.0, 5.0]]], [[[5.0, 7.0, 8.0], [2.0, 
5.0, 6.0], [6.0, 2.0, 5.0]], [[3.0, 4.0, 5.0], [3.0, 7.0, 3.0], [2.0, 1.0, 5.0]], [[3.0, 8.0, 6.0], [7.0, 7.0, 1.0], [4.0, 3.0, 
3.0]]], [[[7.0, 1.0, 7.0], [3.0, 8.0, 7.0], [7.0, 4.0, 6.0]], [[5.0, 5.0, 3.0], [8.0, 4.0, 5.0], [5.0, 2.0, 3.0]], [[2.0, 7.0, 
5.0], [1.0, 7.0, 3.0], [7.0, 5.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 5.0], [4.0, 7.0, 6.0], [2.0, 4.0, 6.0]], 
[[7.0, 8.0, 6.0], [5.0, 3.0, 8.0], [4.0, 1.0, 3.0]], [[2.0, 3.0, 7.0], [3.0, 2.0, 1.0], [3.0, 5.0, 5.0]]], [[[1.0, 2.0, 5.0], 
[8.0, 2.0, 3.0], [1.0, 8.0, 3.0]], [[8.0, 8.0, 2.0], [6.0, 1.0, 7.0], [6.0, 3.0, 1.0]], [[8.0, 4.0, 6.0], [2.0, 7.0, 1.0], 
[7.0, 4.0, 1.0]]], [[[5.0, 7.0, 8.0], [2.0, 5.0, 6.0], [7.0, 4.0, 4.0]], [[3.0, 4.0, 5.0], [3.0, 7.0, 3.0], [5.0, 8.0, 2.0]], 
[[3.0, 8.0, 6.0], [7.0, 7.0, 1.0], [2.0, 8.0, 5.0]]], [[[6.0, 5.0, 3.0], [6.0, 6.0, 5.0], [1.0, 7.0, 2.0]], [[1.0, 6.0, 3.0], 
[8.0, 6.0, 5.0], [6.0, 7.0, 3.0]], [[6.0, 2.0, 6.0], [6.0, 5.0, 5.0], [4.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 4.0], [8.0, 5.0, 3.0], [8.0, 2.0, 6.0]], [[2.0, 1.0, 4.0], [2.0, 7.0, 4.0], [8.0, 
5.0, 6.0]], [[2.0, 4.0, 5.0], [1.0, 5.0, 4.0], [5.0, 1.0, 2.0]]], [[[4.0, 1.0, 8.0], [1.0, 2.0, 7.0], [2.0, 2.0, 5.0]], [[6.0, 
1.0, 4.0], [4.0, 2.0, 5.0], [4.0, 2.0, 2.0]], [[8.0, 4.0, 8.0], [5.0, 3.0, 5.0], [4.0, 3.0, 7.0]]], [[[8.0, 2.0, 1.0], [7.0, 
1.0, 2.0], [2.0, 3.0, 5.0]], [[7.0, 6.0, 4.0], [6.0, 1.0, 7.0], [3.0, 2.0, 3.0]], [[6.0, 7.0, 8.0], [7.0, 4.0, 6.0], [1.0, 7.0, 
5.0]]], [[[1.0, 3.0, 7.0], [7.0, 4.0, 5.0], [3.0, 7.0, 1.0]], [[1.0, 7.0, 2.0], [4.0, 8.0, 7.0], [2.0, 7.0, 2.0]], [[1.0, 4.0, 
8.0], [7.0, 5.0, 5.0], [2.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 8.0], [8.0, 7.0, 4.0], [2.0, 2.0, 1.0]], 
[[6.0, 7.0, 1.0], [8.0, 3.0, 8.0], [1.0, 6.0, 5.0]], [[1.0, 2.0, 1.0], [8.0, 1.0, 6.0], [2.0, 2.0, 3.0]]], [[[1.0, 4.0, 5.0], 
[4.0, 5.0, 8.0], [3.0, 6.0, 1.0]], [[4.0, 1.0, 6.0], [1.0, 6.0, 1.0], [8.0, 2.0, 3.0]], [[5.0, 1.0, 8.0], [2.0, 8.0, 4.0], 
[2.0, 5.0, 3.0]]], [[[3.0, 5.0, 1.0], [3.0, 1.0, 1.0], [5.0, 7.0, 7.0]], [[8.0, 7.0, 4.0], [3.0, 2.0, 3.0], [3.0, 4.0, 4.0]], 
[[3.0, 4.0, 4.0], [2.0, 5.0, 6.0], [8.0, 4.0, 5.0]]], [[[3.0, 5.0, 2.0], [6.0, 1.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 4.0, 6.0], 
[5.0, 8.0, 2.0], [5.0, 4.0, 1.0]], [[2.0, 4.0, 7.0], [7.0, 1.0, 4.0], [4.0, 8.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], [1.0, 8.0, 5.0]], [[5.0, 5.0, 4.0], [4.0, 
4.0, 3.0]]], [[[7.0, 5.0, 2.0], [8.0, 4.0, 7.0]], [[8.0, 3.0, 4.0], [8.0, 2.0, 3.0]], [[4.0, 6.0, 7.0], [8.0, 1.0, 4.0]]], 
[[[1.0, 6.0, 4.0], [3.0, 8.0, 7.0]], [[4.0, 1.0, 4.0], [7.0, 5.0, 4.0]], [[5.0, 2.0, 8.0], [2.0, 4.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 4.0, 5.0], [3.0, 2.0, 1.0]], [[7.0, 4.0, 7.0], [5.0, 2.0, 3.0]], [[1.0, 6.0, 1.0], 
[1.0, 1.0, 5.0]]], [[[7.0, 2.0, 1.0], [2.0, 7.0, 3.0]], [[8.0, 6.0, 5.0], [5.0, 4.0, 7.0]], [[4.0, 8.0, 3.0], [1.0, 1.0, 
1.0]]], [[[1.0, 7.0, 4.0], [2.0, 4.0, 2.0]], [[1.0, 1.0, 7.0], [4.0, 1.0, 6.0]], [[2.0, 1.0, 7.0], [5.0, 2.0, 4.0]]]])
      arg[:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 3.0, 1.0], [8.0, 2.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 8.0, 5.0], [8.0, 
5.0, 6.0]], [[5.0, 5.0, 4.0], [4.0, 4.0, 3.0], [5.0, 1.0, 2.0]]], [[[7.0, 5.0, 2.0], [8.0, 4.0, 7.0], [2.0, 2.0, 5.0]], [[8.0, 
3.0, 4.0], [8.0, 2.0, 3.0], [4.0, 2.0, 2.0]], [[4.0, 6.0, 7.0], [8.0, 1.0, 4.0], [4.0, 3.0, 7.0]]], [[[1.0, 6.0, 4.0], [3.0, 
8.0, 7.0], [2.0, 3.0, 5.0]], [[4.0, 1.0, 4.0], [7.0, 5.0, 4.0], [3.0, 2.0, 3.0]], [[5.0, 2.0, 8.0], [2.0, 4.0, 7.0], [1.0, 7.0, 
5.0]]], [[[1.0, 3.0, 7.0], [7.0, 4.0, 5.0], [3.0, 7.0, 1.0]], [[1.0, 7.0, 2.0], [4.0, 8.0, 7.0], [2.0, 7.0, 2.0]], [[1.0, 4.0, 
8.0], [7.0, 5.0, 5.0], [2.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [3.0, 2.0, 1.0], [2.0, 2.0, 1.0]], 
[[7.0, 4.0, 7.0], [5.0, 2.0, 3.0], [1.0, 6.0, 5.0]], [[1.0, 6.0, 1.0], [1.0, 1.0, 5.0], [2.0, 2.0, 3.0]]], [[[7.0, 2.0, 1.0], 
[2.0, 7.0, 3.0], [3.0, 6.0, 1.0]], [[8.0, 6.0, 5.0], [5.0, 4.0, 7.0], [8.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], [1.0, 1.0, 1.0], 
[2.0, 5.0, 3.0]]], [[[1.0, 7.0, 4.0], [2.0, 4.0, 2.0], [5.0, 7.0, 7.0]], [[1.0, 1.0, 7.0], [4.0, 1.0, 6.0], [3.0, 4.0, 4.0]], 
[[2.0, 1.0, 7.0], [5.0, 2.0, 4.0], [8.0, 4.0, 5.0]]], [[[3.0, 5.0, 2.0], [6.0, 1.0, 2.0], [1.0, 2.0, 7.0]], [[4.0, 4.0, 6.0], 
[5.0, 8.0, 2.0], [5.0, 4.0, 1.0]], [[2.0, 4.0, 7.0], [7.0, 1.0, 4.0], [4.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 6.0], [4.0, 4.0, 5.0], [5.0, 2.0, 5.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 4.0], [6.0, 
2.0, 8.0]], [[5.0, 6.0, 5.0], [1.0, 8.0, 5.0], [1.0, 2.0, 6.0]]], [[[4.0, 8.0, 1.0], [1.0, 1.0, 8.0], [8.0, 5.0, 5.0]], [[2.0, 
6.0, 6.0], [2.0, 1.0, 5.0], [8.0, 3.0, 6.0]], [[5.0, 2.0, 4.0], [8.0, 7.0, 1.0], [1.0, 3.0, 5.0]]], [[[5.0, 1.0, 7.0], [6.0, 
7.0, 7.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 4.0], [1.0, 2.0, 3.0], [2.0, 4.0, 2.0]], [[4.0, 7.0, 8.0], [7.0, 2.0, 2.0], [1.0, 3.0, 
5.0]]], [[[1.0, 8.0, 3.0], [5.0, 4.0, 4.0], [1.0, 4.0, 4.0]], [[2.0, 3.0, 1.0], [1.0, 1.0, 7.0], [4.0, 5.0, 2.0]], [[6.0, 4.0, 
3.0], [1.0, 1.0, 3.0], [8.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 2.0, 2.0], [2.0, 2.0, 2.0], [6.0, 4.0, 8.0]], 
[[1.0, 7.0, 4.0], [5.0, 1.0, 5.0], [4.0, 1.0, 6.0]], [[4.0, 8.0, 1.0], [6.0, 5.0, 4.0], [6.0, 4.0, 2.0]]], [[[6.0, 1.0, 8.0], 
[6.0, 3.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 7.0, 1.0], [1.0, 4.0, 3.0], [2.0, 8.0, 3.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 4.0], 
[1.0, 1.0, 3.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 1.0], [8.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], [6.0, 8.0, 8.0], [4.0, 4.0, 4.0]], 
[[6.0, 4.0, 4.0], [4.0, 5.0, 8.0], [6.0, 7.0, 1.0]]], [[[6.0, 6.0, 3.0], [6.0, 4.0, 5.0], [6.0, 5.0, 1.0]], [[2.0, 2.0, 8.0], 
[4.0, 7.0, 2.0], [1.0, 6.0, 2.0]], [[6.0, 6.0, 1.0], [6.0, 7.0, 4.0], [2.0, 5.0, 1.0]]]])
      val=numpy.array([[2.0, 6.0, 1.0]])
      arg[2,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [4.0, 4.0, 5.0], [5.0, 2.0, 5.0]], [[1.0, 1.0, 1.0], [1.0, 3.0, 4.0], [6.0, 
2.0, 8.0]], [[5.0, 6.0, 5.0], [1.0, 8.0, 5.0], [1.0, 2.0, 6.0]]], [[[4.0, 8.0, 1.0], [1.0, 1.0, 8.0], [8.0, 5.0, 5.0]], [[2.0, 
6.0, 6.0], [2.0, 1.0, 5.0], [8.0, 3.0, 6.0]], [[5.0, 2.0, 4.0], [8.0, 7.0, 1.0], [1.0, 3.0, 5.0]]], [[[5.0, 1.0, 7.0], [6.0, 
7.0, 7.0], [1.0, 4.0, 2.0]], [[5.0, 6.0, 4.0], [1.0, 2.0, 3.0], [2.0, 6.0, 1.0]], [[4.0, 7.0, 8.0], [7.0, 2.0, 2.0], [1.0, 3.0, 
5.0]]], [[[1.0, 8.0, 3.0], [5.0, 4.0, 4.0], [1.0, 4.0, 4.0]], [[2.0, 3.0, 1.0], [1.0, 1.0, 7.0], [4.0, 5.0, 2.0]], [[6.0, 4.0, 
3.0], [1.0, 1.0, 3.0], [8.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 2.0], [2.0, 2.0, 2.0], [6.0, 4.0, 8.0]], 
[[1.0, 7.0, 4.0], [5.0, 1.0, 5.0], [4.0, 1.0, 6.0]], [[4.0, 8.0, 1.0], [6.0, 5.0, 4.0], [6.0, 4.0, 2.0]]], [[[6.0, 1.0, 8.0], 
[6.0, 3.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 7.0, 1.0], [1.0, 4.0, 3.0], [2.0, 8.0, 3.0]], [[7.0, 4.0, 4.0], [6.0, 1.0, 4.0], 
[1.0, 1.0, 3.0]]], [[[3.0, 5.0, 2.0], [2.0, 2.0, 1.0], [8.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], [6.0, 8.0, 8.0], [2.0, 6.0, 1.0]], 
[[6.0, 4.0, 4.0], [4.0, 5.0, 8.0], [6.0, 7.0, 1.0]]], [[[6.0, 6.0, 3.0], [6.0, 4.0, 5.0], [6.0, 5.0, 1.0]], [[2.0, 2.0, 8.0], 
[4.0, 7.0, 2.0], [1.0, 6.0, 2.0]], [[6.0, 6.0, 1.0], [6.0, 7.0, 4.0], [2.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 1.0], [8.0, 6.0, 1.0], [4.0, 6.0, 4.0]], [[7.0, 3.0, 2.0], [2.0, 8.0, 1.0], [7.0, 
4.0, 5.0]], [[1.0, 7.0, 8.0], [6.0, 4.0, 3.0], [6.0, 2.0, 2.0]]], [[[7.0, 4.0, 4.0], [3.0, 2.0, 8.0], [7.0, 3.0, 1.0]], [[4.0, 
7.0, 1.0], [6.0, 5.0, 1.0], [2.0, 1.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 7.0], [1.0, 3.0, 2.0]]], [[[3.0, 5.0, 2.0], [2.0, 
8.0, 6.0], [7.0, 6.0, 4.0]], [[6.0, 2.0, 4.0], [6.0, 8.0, 8.0], [1.0, 8.0, 3.0]], [[5.0, 3.0, 4.0], [3.0, 6.0, 2.0], [3.0, 1.0, 
5.0]]], [[[3.0, 3.0, 8.0], [5.0, 1.0, 3.0], [5.0, 4.0, 1.0]], [[2.0, 7.0, 1.0], [8.0, 3.0, 2.0], [4.0, 1.0, 2.0]], [[8.0, 8.0, 
7.0], [7.0, 5.0, 7.0], [7.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 1.0], [7.0, 2.0, 1.0], [7.0, 4.0, 3.0]], 
[[1.0, 3.0, 6.0], [7.0, 4.0, 2.0], [3.0, 8.0, 6.0]], [[3.0, 5.0, 2.0], [8.0, 6.0, 8.0], [7.0, 2.0, 2.0]]], [[[8.0, 6.0, 1.0], 
[2.0, 2.0, 6.0], [6.0, 6.0, 5.0]], [[2.0, 5.0, 4.0], [1.0, 3.0, 5.0], [4.0, 1.0, 7.0]], [[8.0, 8.0, 8.0], [7.0, 6.0, 3.0], 
[6.0, 4.0, 8.0]]], [[[3.0, 6.0, 4.0], [6.0, 1.0, 3.0], [4.0, 3.0, 1.0]], [[6.0, 2.0, 1.0], [1.0, 1.0, 6.0], [5.0, 5.0, 8.0]], 
[[4.0, 1.0, 3.0], [3.0, 2.0, 8.0], [2.0, 3.0, 8.0]]], [[[3.0, 1.0, 2.0], [8.0, 1.0, 1.0], [8.0, 1.0, 3.0]], [[5.0, 5.0, 3.0], 
[5.0, 7.0, 3.0], [1.0, 7.0, 8.0]], [[2.0, 5.0, 8.0], [5.0, 1.0, 6.0], [1.0, 3.0, 4.0]]]])
      val=Data(numpy.array([[4.0, 1.0, 8.0]]),self.functionspace)
      arg[2,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 1.0], [8.0, 6.0, 1.0], [4.0, 6.0, 4.0]], [[7.0, 3.0, 2.0], [2.0, 8.0, 1.0], [7.0, 
4.0, 5.0]], [[1.0, 7.0, 8.0], [6.0, 4.0, 3.0], [6.0, 2.0, 2.0]]], [[[7.0, 4.0, 4.0], [3.0, 2.0, 8.0], [7.0, 3.0, 1.0]], [[4.0, 
7.0, 1.0], [6.0, 5.0, 1.0], [2.0, 1.0, 3.0]], [[1.0, 1.0, 7.0], [8.0, 8.0, 7.0], [1.0, 3.0, 2.0]]], [[[3.0, 5.0, 2.0], [2.0, 
8.0, 6.0], [7.0, 6.0, 4.0]], [[6.0, 2.0, 4.0], [6.0, 8.0, 8.0], [4.0, 1.0, 8.0]], [[5.0, 3.0, 4.0], [3.0, 6.0, 2.0], [3.0, 1.0, 
5.0]]], [[[3.0, 3.0, 8.0], [5.0, 1.0, 3.0], [5.0, 4.0, 1.0]], [[2.0, 7.0, 1.0], [8.0, 3.0, 2.0], [4.0, 1.0, 2.0]], [[8.0, 8.0, 
7.0], [7.0, 5.0, 7.0], [7.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 1.0], [7.0, 2.0, 1.0], [7.0, 4.0, 3.0]], 
[[1.0, 3.0, 6.0], [7.0, 4.0, 2.0], [3.0, 8.0, 6.0]], [[3.0, 5.0, 2.0], [8.0, 6.0, 8.0], [7.0, 2.0, 2.0]]], [[[8.0, 6.0, 1.0], 
[2.0, 2.0, 6.0], [6.0, 6.0, 5.0]], [[2.0, 5.0, 4.0], [1.0, 3.0, 5.0], [4.0, 1.0, 7.0]], [[8.0, 8.0, 8.0], [7.0, 6.0, 3.0], 
[6.0, 4.0, 8.0]]], [[[3.0, 6.0, 4.0], [6.0, 1.0, 3.0], [4.0, 3.0, 1.0]], [[6.0, 2.0, 1.0], [1.0, 1.0, 6.0], [4.0, 1.0, 8.0]], 
[[4.0, 1.0, 3.0], [3.0, 2.0, 8.0], [2.0, 3.0, 8.0]]], [[[3.0, 1.0, 2.0], [8.0, 1.0, 1.0], [8.0, 1.0, 3.0]], [[5.0, 5.0, 3.0], 
[5.0, 7.0, 3.0], [1.0, 7.0, 8.0]], [[2.0, 5.0, 8.0], [5.0, 1.0, 6.0], [1.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 4.0], [7.0, 6.0, 6.0], [8.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [7.0, 3.0, 2.0], [5.0, 
2.0, 4.0]], [[2.0, 4.0, 4.0], [8.0, 6.0, 3.0], [5.0, 7.0, 8.0]]], [[[3.0, 4.0, 6.0], [1.0, 6.0, 5.0], [4.0, 1.0, 1.0]], [[4.0, 
4.0, 6.0], [6.0, 2.0, 1.0], [7.0, 1.0, 8.0]], [[6.0, 6.0, 4.0], [7.0, 2.0, 8.0], [2.0, 5.0, 3.0]]], [[[6.0, 3.0, 4.0], [5.0, 
4.0, 4.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 1.0], [3.0, 5.0, 2.0], [2.0, 1.0, 7.0]], [[3.0, 1.0, 8.0], [8.0, 4.0, 3.0], [8.0, 7.0, 
5.0]]], [[[3.0, 5.0, 1.0], [7.0, 7.0, 6.0], [1.0, 6.0, 4.0]], [[8.0, 3.0, 2.0], [8.0, 8.0, 1.0], [1.0, 6.0, 1.0]], [[3.0, 4.0, 
8.0], [7.0, 8.0, 1.0], [3.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 1.0], [2.0, 6.0, 6.0], [4.0, 6.0, 5.0]], 
[[2.0, 2.0, 8.0], [6.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[1.0, 8.0, 4.0], [2.0, 7.0, 1.0], [8.0, 8.0, 2.0]]], [[[2.0, 2.0, 2.0], 
[3.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[5.0, 3.0, 6.0], [8.0, 6.0, 6.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 7.0], [5.0, 6.0, 8.0], 
[3.0, 6.0, 2.0]]], [[[4.0, 1.0, 2.0], [8.0, 2.0, 7.0], [3.0, 5.0, 5.0]], [[5.0, 7.0, 6.0], [5.0, 4.0, 6.0], [6.0, 6.0, 5.0]], 
[[4.0, 4.0, 4.0], [2.0, 7.0, 8.0], [5.0, 6.0, 2.0]]], [[[1.0, 6.0, 5.0], [7.0, 5.0, 6.0], [2.0, 2.0, 3.0]], [[5.0, 6.0, 4.0], 
[8.0, 4.0, 2.0], [7.0, 7.0, 6.0]], [[6.0, 1.0, 3.0], [7.0, 3.0, 6.0], [2.0, 6.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 1.0, 4.0]])+(1.-msk_val)*numpy.array([[7.0, 6.0, 6.0]])
      arg[2,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 4.0], [7.0, 6.0, 6.0], [8.0, 1.0, 4.0]], [[7.0, 3.0, 5.0], [7.0, 3.0, 2.0], [5.0, 
2.0, 4.0]], [[2.0, 4.0, 4.0], [8.0, 6.0, 3.0], [5.0, 7.0, 8.0]]], [[[3.0, 4.0, 6.0], [1.0, 6.0, 5.0], [4.0, 1.0, 1.0]], [[4.0, 
4.0, 6.0], [6.0, 2.0, 1.0], [7.0, 1.0, 8.0]], [[6.0, 6.0, 4.0], [7.0, 2.0, 8.0], [2.0, 5.0, 3.0]]], [[[6.0, 3.0, 4.0], [5.0, 
4.0, 4.0], [6.0, 2.0, 3.0]], [[5.0, 8.0, 1.0], [3.0, 5.0, 2.0], [8.0, 1.0, 4.0]], [[3.0, 1.0, 8.0], [8.0, 4.0, 3.0], [8.0, 7.0, 
5.0]]], [[[3.0, 5.0, 1.0], [7.0, 7.0, 6.0], [1.0, 6.0, 4.0]], [[8.0, 3.0, 2.0], [8.0, 8.0, 1.0], [1.0, 6.0, 1.0]], [[3.0, 4.0, 
8.0], [7.0, 8.0, 1.0], [3.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 1.0], [2.0, 6.0, 6.0], [4.0, 6.0, 5.0]], 
[[2.0, 2.0, 8.0], [6.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[1.0, 8.0, 4.0], [2.0, 7.0, 1.0], [8.0, 8.0, 2.0]]], [[[2.0, 2.0, 2.0], 
[3.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[5.0, 3.0, 6.0], [8.0, 6.0, 6.0], [3.0, 2.0, 1.0]], [[3.0, 3.0, 7.0], [5.0, 6.0, 8.0], 
[3.0, 6.0, 2.0]]], [[[4.0, 1.0, 2.0], [8.0, 2.0, 7.0], [3.0, 5.0, 5.0]], [[5.0, 7.0, 6.0], [5.0, 4.0, 6.0], [7.0, 6.0, 6.0]], 
[[4.0, 4.0, 4.0], [2.0, 7.0, 8.0], [5.0, 6.0, 2.0]]], [[[1.0, 6.0, 5.0], [7.0, 5.0, 6.0], [2.0, 2.0, 3.0]], [[5.0, 6.0, 4.0], 
[8.0, 4.0, 2.0], [7.0, 7.0, 6.0]], [[6.0, 1.0, 3.0], [7.0, 3.0, 6.0], [2.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 8.0], [4.0, 4.0, 1.0], [1.0, 1.0, 5.0]], [[3.0, 3.0, 6.0], [6.0, 7.0, 8.0], [1.0, 
8.0, 8.0]], [[4.0, 8.0, 7.0], [8.0, 4.0, 6.0], [8.0, 8.0, 8.0]]], [[[4.0, 4.0, 5.0], [5.0, 6.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 
8.0, 2.0], [1.0, 6.0, 3.0], [1.0, 4.0, 6.0]], [[6.0, 5.0, 3.0], [6.0, 3.0, 8.0], [8.0, 7.0, 1.0]]], [[[8.0, 5.0, 1.0], [3.0, 
1.0, 4.0], [3.0, 5.0, 1.0]], [[6.0, 2.0, 5.0], [4.0, 7.0, 1.0], [5.0, 4.0, 7.0]], [[4.0, 6.0, 8.0], [2.0, 3.0, 7.0], [8.0, 8.0, 
1.0]]], [[[8.0, 8.0, 8.0], [6.0, 3.0, 4.0], [6.0, 2.0, 4.0]], [[4.0, 2.0, 4.0], [8.0, 1.0, 1.0], [8.0, 7.0, 2.0]], [[2.0, 5.0, 
6.0], [3.0, 4.0, 2.0], [7.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 4.0], [4.0, 8.0, 4.0], [8.0, 2.0, 1.0]], 
[[4.0, 3.0, 8.0], [3.0, 8.0, 3.0], [3.0, 6.0, 2.0]], [[7.0, 3.0, 4.0], [6.0, 5.0, 7.0], [8.0, 5.0, 1.0]]], [[[6.0, 6.0, 5.0], 
[1.0, 2.0, 6.0], [4.0, 1.0, 2.0]], [[2.0, 7.0, 8.0], [3.0, 6.0, 2.0], [2.0, 3.0, 6.0]], [[3.0, 8.0, 2.0], [5.0, 3.0, 6.0], 
[4.0, 8.0, 4.0]]], [[[6.0, 6.0, 2.0], [4.0, 2.0, 5.0], [1.0, 6.0, 4.0]], [[5.0, 8.0, 3.0], [6.0, 7.0, 7.0], [6.0, 2.0, 1.0]], 
[[2.0, 3.0, 8.0], [3.0, 5.0, 2.0], [2.0, 7.0, 2.0]]], [[[5.0, 4.0, 7.0], [6.0, 7.0, 7.0], [8.0, 2.0, 5.0]], [[2.0, 4.0, 1.0], 
[8.0, 7.0, 1.0], [4.0, 5.0, 6.0]], [[2.0, 2.0, 7.0], [3.0, 6.0, 7.0], [7.0, 3.0, 2.0]]]])
      val=numpy.array([[[[1.0, 1.0, 4.0], [1.0, 8.0, 5.0], [6.0, 3.0, 2.0]], [[8.0, 7.0, 5.0], [2.0, 5.0, 4.0], [5.0, 3.0, 
1.0]]], [[[7.0, 8.0, 2.0], [1.0, 6.0, 2.0], [5.0, 1.0, 2.0]], [[1.0, 3.0, 5.0], [8.0, 5.0, 6.0], [5.0, 8.0, 8.0]]], [[[6.0, 
8.0, 7.0], [4.0, 5.0, 6.0], [1.0, 8.0, 2.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 3.0], [2.0, 8.0, 3.0]]], [[[7.0, 3.0, 8.0], [8.0, 
8.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 4.0, 7.0], [2.0, 5.0, 4.0], [6.0, 4.0, 4.0]]]])
      arg[:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 4.0], [1.0, 8.0, 5.0], [6.0, 3.0, 2.0]], [[8.0, 7.0, 5.0], [2.0, 5.0, 4.0], [5.0, 
3.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 4.0, 6.0], [8.0, 8.0, 8.0]]], [[[7.0, 8.0, 2.0], [1.0, 6.0, 2.0], [5.0, 1.0, 2.0]], [[1.0, 
3.0, 5.0], [8.0, 5.0, 6.0], [5.0, 8.0, 8.0]], [[6.0, 5.0, 3.0], [6.0, 3.0, 8.0], [8.0, 7.0, 1.0]]], [[[6.0, 8.0, 7.0], [4.0, 
5.0, 6.0], [1.0, 8.0, 2.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 3.0], [2.0, 8.0, 3.0]], [[4.0, 6.0, 8.0], [2.0, 3.0, 7.0], [8.0, 8.0, 
1.0]]], [[[7.0, 3.0, 8.0], [8.0, 8.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 4.0, 7.0], [2.0, 5.0, 4.0], [6.0, 4.0, 4.0]], [[2.0, 5.0, 
6.0], [3.0, 4.0, 2.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [1.0, 8.0, 5.0], [6.0, 3.0, 2.0]], 
[[8.0, 7.0, 5.0], [2.0, 5.0, 4.0], [5.0, 3.0, 1.0]], [[7.0, 3.0, 4.0], [6.0, 5.0, 7.0], [8.0, 5.0, 1.0]]], [[[7.0, 8.0, 2.0], 
[1.0, 6.0, 2.0], [5.0, 1.0, 2.0]], [[1.0, 3.0, 5.0], [8.0, 5.0, 6.0], [5.0, 8.0, 8.0]], [[3.0, 8.0, 2.0], [5.0, 3.0, 6.0], 
[4.0, 8.0, 4.0]]], [[[6.0, 8.0, 7.0], [4.0, 5.0, 6.0], [1.0, 8.0, 2.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 3.0], [2.0, 8.0, 3.0]], 
[[2.0, 3.0, 8.0], [3.0, 5.0, 2.0], [2.0, 7.0, 2.0]]], [[[7.0, 3.0, 8.0], [8.0, 8.0, 1.0], [2.0, 6.0, 1.0]], [[6.0, 4.0, 7.0], 
[2.0, 5.0, 4.0], [6.0, 4.0, 4.0]], [[2.0, 2.0, 7.0], [3.0, 6.0, 7.0], [7.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 6.0], [8.0, 1.0, 8.0], [5.0, 3.0, 8.0]], [[3.0, 6.0, 3.0], [4.0, 8.0, 1.0], [8.0, 
2.0, 2.0]], [[5.0, 8.0, 6.0], [8.0, 2.0, 2.0], [4.0, 1.0, 7.0]]], [[[2.0, 2.0, 2.0], [5.0, 8.0, 6.0], [4.0, 8.0, 7.0]], [[8.0, 
3.0, 5.0], [4.0, 3.0, 6.0], [2.0, 2.0, 1.0]], [[1.0, 3.0, 5.0], [1.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[1.0, 1.0, 2.0], [7.0, 
2.0, 5.0], [2.0, 8.0, 2.0]], [[5.0, 5.0, 2.0], [2.0, 2.0, 3.0], [8.0, 8.0, 5.0]], [[8.0, 4.0, 4.0], [4.0, 2.0, 3.0], [5.0, 7.0, 
3.0]]], [[[1.0, 2.0, 8.0], [4.0, 4.0, 2.0], [3.0, 5.0, 2.0]], [[4.0, 7.0, 8.0], [2.0, 4.0, 3.0], [1.0, 1.0, 3.0]], [[7.0, 3.0, 
8.0], [8.0, 1.0, 7.0], [2.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 8.0], [4.0, 6.0, 8.0], [4.0, 8.0, 2.0]], 
[[7.0, 3.0, 1.0], [5.0, 5.0, 4.0], [8.0, 5.0, 7.0]], [[7.0, 7.0, 3.0], [4.0, 3.0, 2.0], [8.0, 3.0, 8.0]]], [[[4.0, 7.0, 4.0], 
[1.0, 7.0, 6.0], [8.0, 2.0, 3.0]], [[3.0, 4.0, 4.0], [6.0, 4.0, 8.0], [5.0, 6.0, 3.0]], [[5.0, 7.0, 5.0], [8.0, 7.0, 4.0], 
[5.0, 8.0, 8.0]]], [[[3.0, 5.0, 2.0], [8.0, 3.0, 3.0], [7.0, 5.0, 7.0]], [[8.0, 7.0, 6.0], [6.0, 6.0, 3.0], [4.0, 8.0, 3.0]], 
[[3.0, 6.0, 4.0], [3.0, 4.0, 8.0], [7.0, 6.0, 5.0]]], [[[8.0, 6.0, 4.0], [2.0, 8.0, 4.0], [4.0, 4.0, 5.0]], [[1.0, 3.0, 8.0], 
[6.0, 4.0, 6.0], [6.0, 7.0, 4.0]], [[2.0, 1.0, 1.0], [3.0, 4.0, 6.0], [4.0, 7.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0, 4.0, 3.0], [2.0, 3.0, 3.0], [3.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 4.0, 4.0], [3.0, 4.0, 
4.0]]], [[[2.0, 2.0, 2.0], [5.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[4.0, 6.0, 4.0], [2.0, 1.0, 6.0], [1.0, 4.0, 8.0]]], [[[2.0, 
8.0, 7.0], [5.0, 5.0, 3.0], [8.0, 1.0, 3.0]], [[6.0, 8.0, 8.0], [5.0, 2.0, 2.0], [5.0, 5.0, 2.0]]], [[[6.0, 1.0, 8.0], [4.0, 
4.0, 1.0], [7.0, 3.0, 6.0]], [[2.0, 7.0, 7.0], [3.0, 8.0, 6.0], [3.0, 7.0, 6.0]]]]),self.functionspace)
      arg[:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 3.0], [2.0, 3.0, 3.0], [3.0, 5.0, 5.0]], [[7.0, 4.0, 3.0], [4.0, 4.0, 4.0], [3.0, 
4.0, 4.0]], [[5.0, 8.0, 6.0], [8.0, 2.0, 2.0], [4.0, 1.0, 7.0]]], [[[2.0, 2.0, 2.0], [5.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[4.0, 
6.0, 4.0], [2.0, 1.0, 6.0], [1.0, 4.0, 8.0]], [[1.0, 3.0, 5.0], [1.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[2.0, 8.0, 7.0], [5.0, 
5.0, 3.0], [8.0, 1.0, 3.0]], [[6.0, 8.0, 8.0], [5.0, 2.0, 2.0], [5.0, 5.0, 2.0]], [[8.0, 4.0, 4.0], [4.0, 2.0, 3.0], [5.0, 7.0, 
3.0]]], [[[6.0, 1.0, 8.0], [4.0, 4.0, 1.0], [7.0, 3.0, 6.0]], [[2.0, 7.0, 7.0], [3.0, 8.0, 6.0], [3.0, 7.0, 6.0]], [[7.0, 3.0, 
8.0], [8.0, 1.0, 7.0], [2.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 3.0], [2.0, 3.0, 3.0], [3.0, 5.0, 5.0]], 
[[7.0, 4.0, 3.0], [4.0, 4.0, 4.0], [3.0, 4.0, 4.0]], [[7.0, 7.0, 3.0], [4.0, 3.0, 2.0], [8.0, 3.0, 8.0]]], [[[2.0, 2.0, 2.0], 
[5.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[4.0, 6.0, 4.0], [2.0, 1.0, 6.0], [1.0, 4.0, 8.0]], [[5.0, 7.0, 5.0], [8.0, 7.0, 4.0], 
[5.0, 8.0, 8.0]]], [[[2.0, 8.0, 7.0], [5.0, 5.0, 3.0], [8.0, 1.0, 3.0]], [[6.0, 8.0, 8.0], [5.0, 2.0, 2.0], [5.0, 5.0, 2.0]], 
[[3.0, 6.0, 4.0], [3.0, 4.0, 8.0], [7.0, 6.0, 5.0]]], [[[6.0, 1.0, 8.0], [4.0, 4.0, 1.0], [7.0, 3.0, 6.0]], [[2.0, 7.0, 7.0], 
[3.0, 8.0, 6.0], [3.0, 7.0, 6.0]], [[2.0, 1.0, 1.0], [3.0, 4.0, 6.0], [4.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 8.0], [7.0, 3.0, 1.0], [2.0, 5.0, 4.0]], [[4.0, 7.0, 1.0], [2.0, 8.0, 7.0], [4.0, 
3.0, 7.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[6.0, 5.0, 2.0], [7.0, 3.0, 1.0], [1.0, 6.0, 1.0]], [[2.0, 
5.0, 7.0], [3.0, 3.0, 6.0], [1.0, 7.0, 8.0]], [[1.0, 3.0, 2.0], [3.0, 7.0, 6.0], [1.0, 6.0, 4.0]]], [[[1.0, 3.0, 6.0], [3.0, 
7.0, 3.0], [6.0, 5.0, 7.0]], [[8.0, 6.0, 8.0], [1.0, 3.0, 5.0], [6.0, 7.0, 3.0]], [[4.0, 5.0, 8.0], [4.0, 5.0, 6.0], [1.0, 2.0, 
6.0]]], [[[6.0, 1.0, 1.0], [5.0, 2.0, 5.0], [7.0, 7.0, 3.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 8.0], [6.0, 3.0, 2.0]], [[2.0, 8.0, 
6.0], [8.0, 8.0, 6.0], [6.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 5.0], [4.0, 8.0, 1.0], [2.0, 1.0, 4.0]], 
[[8.0, 6.0, 8.0], [6.0, 4.0, 5.0], [3.0, 5.0, 7.0]], [[5.0, 1.0, 4.0], [7.0, 5.0, 2.0], [3.0, 2.0, 1.0]]], [[[6.0, 1.0, 1.0], 
[2.0, 3.0, 8.0], [3.0, 5.0, 6.0]], [[7.0, 8.0, 3.0], [6.0, 3.0, 6.0], [2.0, 3.0, 7.0]], [[7.0, 8.0, 2.0], [4.0, 2.0, 8.0], 
[2.0, 2.0, 5.0]]], [[[7.0, 1.0, 5.0], [3.0, 7.0, 4.0], [1.0, 2.0, 4.0]], [[7.0, 5.0, 4.0], [6.0, 6.0, 3.0], [4.0, 6.0, 8.0]], 
[[3.0, 1.0, 6.0], [7.0, 1.0, 7.0], [7.0, 7.0, 2.0]]], [[[2.0, 3.0, 1.0], [8.0, 4.0, 6.0], [2.0, 3.0, 1.0]], [[5.0, 3.0, 7.0], 
[1.0, 8.0, 6.0], [6.0, 6.0, 1.0]], [[3.0, 2.0, 3.0], [6.0, 7.0, 2.0], [8.0, 4.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 1.0, 5.0], [4.0, 8.0, 1.0], [5.0, 1.0, 4.0]], [[5.0, 4.0, 8.0], [6.0, 7.0, 8.0], [1.0, 
7.0, 6.0]]], [[[3.0, 6.0, 8.0], [3.0, 6.0, 6.0], [5.0, 4.0, 6.0]], [[1.0, 4.0, 3.0], [4.0, 3.0, 7.0], [3.0, 8.0, 3.0]]], 
[[[4.0, 1.0, 7.0], [5.0, 5.0, 6.0], [2.0, 6.0, 3.0]], [[6.0, 5.0, 7.0], [6.0, 5.0, 8.0], [1.0, 3.0, 2.0]]], [[[7.0, 1.0, 8.0], 
[1.0, 8.0, 7.0], [3.0, 3.0, 1.0]], [[2.0, 1.0, 5.0], [7.0, 1.0, 3.0], [7.0, 5.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 
8.0, 5.0], [8.0, 7.0, 2.0], [2.0, 3.0, 2.0]], [[5.0, 4.0, 8.0], [8.0, 1.0, 6.0], [3.0, 2.0, 7.0]]], [[[6.0, 4.0, 8.0], [6.0, 
6.0, 2.0], [7.0, 8.0, 2.0]], [[4.0, 2.0, 2.0], [5.0, 3.0, 2.0], [3.0, 7.0, 8.0]]], [[[2.0, 5.0, 6.0], [5.0, 3.0, 4.0], [4.0, 
6.0, 8.0]], [[4.0, 5.0, 2.0], [5.0, 7.0, 5.0], [5.0, 8.0, 4.0]]], [[[5.0, 1.0, 4.0], [3.0, 6.0, 5.0], [3.0, 5.0, 2.0]], [[2.0, 
7.0, 5.0], [3.0, 4.0, 1.0], [8.0, 3.0, 8.0]]]])
      arg[:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 5.0], [4.0, 8.0, 1.0], [5.0, 1.0, 4.0]], [[5.0, 4.0, 8.0], [6.0, 7.0, 8.0], [1.0, 
7.0, 6.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 8.0], [4.0, 3.0, 8.0]]], [[[3.0, 6.0, 8.0], [3.0, 6.0, 6.0], [5.0, 4.0, 6.0]], [[1.0, 
4.0, 3.0], [4.0, 3.0, 7.0], [3.0, 8.0, 3.0]], [[1.0, 3.0, 2.0], [3.0, 7.0, 6.0], [1.0, 6.0, 4.0]]], [[[4.0, 1.0, 7.0], [5.0, 
5.0, 6.0], [2.0, 6.0, 3.0]], [[6.0, 5.0, 7.0], [6.0, 5.0, 8.0], [1.0, 3.0, 2.0]], [[4.0, 5.0, 8.0], [4.0, 5.0, 6.0], [1.0, 2.0, 
6.0]]], [[[7.0, 1.0, 8.0], [1.0, 8.0, 7.0], [3.0, 3.0, 1.0]], [[2.0, 1.0, 5.0], [7.0, 1.0, 3.0], [7.0, 5.0, 4.0]], [[2.0, 8.0, 
6.0], [8.0, 8.0, 6.0], [6.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 5.0], [8.0, 7.0, 2.0], [2.0, 3.0, 2.0]], 
[[5.0, 4.0, 8.0], [8.0, 1.0, 6.0], [3.0, 2.0, 7.0]], [[5.0, 1.0, 4.0], [7.0, 5.0, 2.0], [3.0, 2.0, 1.0]]], [[[6.0, 4.0, 8.0], 
[6.0, 6.0, 2.0], [7.0, 8.0, 2.0]], [[4.0, 2.0, 2.0], [5.0, 3.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 8.0, 2.0], [4.0, 2.0, 8.0], 
[2.0, 2.0, 5.0]]], [[[2.0, 5.0, 6.0], [5.0, 3.0, 4.0], [4.0, 6.0, 8.0]], [[4.0, 5.0, 2.0], [5.0, 7.0, 5.0], [5.0, 8.0, 4.0]], 
[[3.0, 1.0, 6.0], [7.0, 1.0, 7.0], [7.0, 7.0, 2.0]]], [[[5.0, 1.0, 4.0], [3.0, 6.0, 5.0], [3.0, 5.0, 2.0]], [[2.0, 7.0, 5.0], 
[3.0, 4.0, 1.0], [8.0, 3.0, 8.0]], [[3.0, 2.0, 3.0], [6.0, 7.0, 2.0], [8.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 4.0], [4.0, 3.0, 2.0], [7.0, 4.0, 7.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 5.0], [6.0, 
1.0, 3.0]], [[3.0, 4.0, 6.0], [6.0, 7.0, 3.0], [4.0, 6.0, 7.0]]], [[[2.0, 7.0, 2.0], [7.0, 2.0, 7.0], [8.0, 6.0, 5.0]], [[8.0, 
8.0, 8.0], [7.0, 8.0, 8.0], [6.0, 5.0, 4.0]], [[1.0, 6.0, 6.0], [3.0, 4.0, 7.0], [5.0, 1.0, 5.0]]], [[[3.0, 7.0, 8.0], [2.0, 
7.0, 3.0], [3.0, 1.0, 3.0]], [[1.0, 4.0, 1.0], [5.0, 7.0, 5.0], [5.0, 6.0, 7.0]], [[4.0, 7.0, 7.0], [7.0, 7.0, 2.0], [4.0, 6.0, 
2.0]]], [[[7.0, 1.0, 3.0], [4.0, 6.0, 1.0], [1.0, 2.0, 6.0]], [[7.0, 2.0, 7.0], [5.0, 1.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 
2.0], [3.0, 7.0, 3.0], [3.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 2.0], [1.0, 4.0, 6.0], [6.0, 8.0, 7.0]], 
[[8.0, 6.0, 1.0], [4.0, 6.0, 4.0], [2.0, 4.0, 8.0]], [[6.0, 8.0, 1.0], [7.0, 1.0, 4.0], [3.0, 1.0, 3.0]]], [[[4.0, 3.0, 7.0], 
[7.0, 3.0, 3.0], [6.0, 8.0, 6.0]], [[6.0, 3.0, 3.0], [2.0, 3.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], [1.0, 6.0, 1.0], 
[8.0, 7.0, 1.0]]], [[[8.0, 3.0, 8.0], [2.0, 8.0, 7.0], [7.0, 8.0, 4.0]], [[3.0, 3.0, 6.0], [7.0, 6.0, 2.0], [3.0, 3.0, 3.0]], 
[[4.0, 2.0, 5.0], [6.0, 8.0, 3.0], [8.0, 1.0, 3.0]]], [[[3.0, 5.0, 1.0], [8.0, 7.0, 6.0], [6.0, 5.0, 7.0]], [[2.0, 5.0, 1.0], 
[8.0, 6.0, 8.0], [8.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [5.0, 1.0, 5.0], [2.0, 6.0, 5.0]]]])
      val=numpy.array([[2.0], [5.0]])
      arg[2:,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 4.0], [4.0, 3.0, 2.0], [7.0, 4.0, 7.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 5.0], [6.0, 
1.0, 3.0]], [[3.0, 4.0, 6.0], [6.0, 7.0, 3.0], [4.0, 6.0, 7.0]]], [[[2.0, 7.0, 2.0], [7.0, 2.0, 7.0], [8.0, 6.0, 5.0]], [[8.0, 
8.0, 8.0], [7.0, 8.0, 8.0], [6.0, 5.0, 4.0]], [[1.0, 6.0, 6.0], [3.0, 4.0, 7.0], [5.0, 1.0, 5.0]]], [[[3.0, 7.0, 8.0], [2.0, 
7.0, 3.0], [3.0, 1.0, 3.0]], [[1.0, 4.0, 1.0], [5.0, 2.0, 5.0], [5.0, 6.0, 7.0]], [[4.0, 7.0, 7.0], [7.0, 7.0, 2.0], [4.0, 6.0, 
2.0]]], [[[7.0, 1.0, 3.0], [4.0, 6.0, 1.0], [1.0, 2.0, 6.0]], [[7.0, 2.0, 7.0], [5.0, 5.0, 6.0], [3.0, 5.0, 6.0]], [[1.0, 3.0, 
2.0], [3.0, 7.0, 3.0], [3.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 2.0], [1.0, 4.0, 6.0], [6.0, 8.0, 7.0]], 
[[8.0, 6.0, 1.0], [4.0, 6.0, 4.0], [2.0, 4.0, 8.0]], [[6.0, 8.0, 1.0], [7.0, 1.0, 4.0], [3.0, 1.0, 3.0]]], [[[4.0, 3.0, 7.0], 
[7.0, 3.0, 3.0], [6.0, 8.0, 6.0]], [[6.0, 3.0, 3.0], [2.0, 3.0, 4.0], [2.0, 5.0, 4.0]], [[5.0, 8.0, 2.0], [1.0, 6.0, 1.0], 
[8.0, 7.0, 1.0]]], [[[8.0, 3.0, 8.0], [2.0, 8.0, 7.0], [7.0, 8.0, 4.0]], [[3.0, 3.0, 6.0], [7.0, 2.0, 2.0], [3.0, 3.0, 3.0]], 
[[4.0, 2.0, 5.0], [6.0, 8.0, 3.0], [8.0, 1.0, 3.0]]], [[[3.0, 5.0, 1.0], [8.0, 7.0, 6.0], [6.0, 5.0, 7.0]], [[2.0, 5.0, 1.0], 
[8.0, 5.0, 8.0], [8.0, 3.0, 4.0]], [[5.0, 2.0, 8.0], [5.0, 1.0, 5.0], [2.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 8.0], [1.0, 3.0, 3.0], [4.0, 7.0, 3.0]], [[8.0, 6.0, 6.0], [6.0, 4.0, 3.0], [6.0, 
7.0, 8.0]], [[2.0, 5.0, 8.0], [8.0, 6.0, 6.0], [6.0, 3.0, 2.0]]], [[[4.0, 7.0, 3.0], [7.0, 2.0, 3.0], [3.0, 1.0, 8.0]], [[7.0, 
6.0, 2.0], [3.0, 5.0, 1.0], [2.0, 7.0, 7.0]], [[6.0, 6.0, 5.0], [5.0, 4.0, 2.0], [4.0, 1.0, 2.0]]], [[[2.0, 7.0, 4.0], [6.0, 
3.0, 1.0], [2.0, 6.0, 1.0]], [[8.0, 7.0, 3.0], [1.0, 4.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 7.0, 2.0], [2.0, 8.0, 5.0], [2.0, 8.0, 
1.0]]], [[[7.0, 1.0, 4.0], [8.0, 2.0, 2.0], [6.0, 6.0, 2.0]], [[1.0, 1.0, 3.0], [8.0, 6.0, 7.0], [5.0, 7.0, 1.0]], [[1.0, 5.0, 
3.0], [3.0, 8.0, 5.0], [6.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 5.0, 8.0], [7.0, 6.0, 7.0]], 
[[5.0, 2.0, 1.0], [4.0, 6.0, 6.0], [2.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [4.0, 3.0, 8.0], [4.0, 2.0, 1.0]]], [[[7.0, 6.0, 1.0], 
[1.0, 8.0, 8.0], [8.0, 5.0, 6.0]], [[6.0, 1.0, 5.0], [5.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[4.0, 2.0, 6.0], [4.0, 1.0, 8.0], 
[2.0, 1.0, 6.0]]], [[[5.0, 1.0, 4.0], [3.0, 1.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 5.0, 1.0], [6.0, 8.0, 4.0], [5.0, 5.0, 2.0]], 
[[1.0, 6.0, 4.0], [7.0, 7.0, 6.0], [3.0, 8.0, 3.0]]], [[[2.0, 6.0, 5.0], [1.0, 8.0, 5.0], [8.0, 8.0, 2.0]], [[2.0, 3.0, 4.0], 
[2.0, 6.0, 4.0], [8.0, 4.0, 2.0]], [[7.0, 8.0, 7.0], [1.0, 4.0, 4.0], [8.0, 1.0, 8.0]]]])
      val=Data(numpy.array([[6.0], [5.0]]),self.functionspace)
      arg[2:,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 8.0], [1.0, 3.0, 3.0], [4.0, 7.0, 3.0]], [[8.0, 6.0, 6.0], [6.0, 4.0, 3.0], [6.0, 
7.0, 8.0]], [[2.0, 5.0, 8.0], [8.0, 6.0, 6.0], [6.0, 3.0, 2.0]]], [[[4.0, 7.0, 3.0], [7.0, 2.0, 3.0], [3.0, 1.0, 8.0]], [[7.0, 
6.0, 2.0], [3.0, 5.0, 1.0], [2.0, 7.0, 7.0]], [[6.0, 6.0, 5.0], [5.0, 4.0, 2.0], [4.0, 1.0, 2.0]]], [[[2.0, 7.0, 4.0], [6.0, 
3.0, 1.0], [2.0, 6.0, 1.0]], [[8.0, 7.0, 3.0], [1.0, 6.0, 2.0], [5.0, 2.0, 5.0]], [[5.0, 7.0, 2.0], [2.0, 8.0, 5.0], [2.0, 8.0, 
1.0]]], [[[7.0, 1.0, 4.0], [8.0, 2.0, 2.0], [6.0, 6.0, 2.0]], [[1.0, 1.0, 3.0], [8.0, 5.0, 7.0], [5.0, 7.0, 1.0]], [[1.0, 5.0, 
3.0], [3.0, 8.0, 5.0], [6.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 5.0, 8.0], [7.0, 6.0, 7.0]], 
[[5.0, 2.0, 1.0], [4.0, 6.0, 6.0], [2.0, 8.0, 5.0]], [[6.0, 4.0, 5.0], [4.0, 3.0, 8.0], [4.0, 2.0, 1.0]]], [[[7.0, 6.0, 1.0], 
[1.0, 8.0, 8.0], [8.0, 5.0, 6.0]], [[6.0, 1.0, 5.0], [5.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[4.0, 2.0, 6.0], [4.0, 1.0, 8.0], 
[2.0, 1.0, 6.0]]], [[[5.0, 1.0, 4.0], [3.0, 1.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 5.0, 1.0], [6.0, 6.0, 4.0], [5.0, 5.0, 2.0]], 
[[1.0, 6.0, 4.0], [7.0, 7.0, 6.0], [3.0, 8.0, 3.0]]], [[[2.0, 6.0, 5.0], [1.0, 8.0, 5.0], [8.0, 8.0, 2.0]], [[2.0, 3.0, 4.0], 
[2.0, 5.0, 4.0], [8.0, 4.0, 2.0]], [[7.0, 8.0, 7.0], [1.0, 4.0, 4.0], [8.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 5.0], [8.0, 5.0, 2.0], [2.0, 4.0, 5.0]], [[7.0, 5.0, 4.0], [7.0, 2.0, 3.0], [2.0, 
2.0, 4.0]], [[6.0, 4.0, 7.0], [7.0, 1.0, 2.0], [4.0, 5.0, 8.0]]], [[[3.0, 8.0, 8.0], [1.0, 5.0, 3.0], [6.0, 2.0, 3.0]], [[4.0, 
5.0, 1.0], [8.0, 3.0, 4.0], [7.0, 5.0, 2.0]], [[2.0, 7.0, 5.0], [5.0, 4.0, 6.0], [4.0, 6.0, 8.0]]], [[[6.0, 8.0, 3.0], [2.0, 
4.0, 8.0], [6.0, 5.0, 8.0]], [[2.0, 3.0, 3.0], [4.0, 5.0, 6.0], [2.0, 8.0, 2.0]], [[1.0, 8.0, 1.0], [5.0, 6.0, 7.0], [5.0, 7.0, 
8.0]]], [[[4.0, 7.0, 6.0], [1.0, 2.0, 6.0], [5.0, 7.0, 2.0]], [[5.0, 2.0, 2.0], [8.0, 6.0, 6.0], [6.0, 3.0, 1.0]], [[2.0, 2.0, 
4.0], [1.0, 6.0, 8.0], [4.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 3.0], [7.0, 2.0, 4.0], [1.0, 4.0, 8.0]], 
[[5.0, 4.0, 1.0], [6.0, 2.0, 7.0], [3.0, 2.0, 8.0]], [[7.0, 3.0, 2.0], [5.0, 7.0, 5.0], [6.0, 6.0, 7.0]]], [[[5.0, 4.0, 8.0], 
[1.0, 3.0, 8.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [2.0, 6.0, 6.0], [2.0, 7.0, 7.0]], [[8.0, 2.0, 2.0], [8.0, 8.0, 3.0], 
[4.0, 5.0, 3.0]]], [[[4.0, 3.0, 4.0], [1.0, 7.0, 4.0], [7.0, 7.0, 2.0]], [[4.0, 4.0, 6.0], [6.0, 8.0, 4.0], [6.0, 8.0, 1.0]], 
[[5.0, 4.0, 5.0], [1.0, 3.0, 2.0], [1.0, 3.0, 6.0]]], [[[2.0, 6.0, 7.0], [3.0, 2.0, 5.0], [1.0, 2.0, 1.0]], [[7.0, 3.0, 3.0], 
[8.0, 7.0, 5.0], [2.0, 7.0, 5.0]], [[2.0, 5.0, 1.0], [7.0, 8.0, 6.0], [6.0, 7.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0], [7.0]])+(1.-msk_val)*numpy.array([[8.0], [6.0]])
      arg[2:,1:2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 5.0], [8.0, 5.0, 2.0], [2.0, 4.0, 5.0]], [[7.0, 5.0, 4.0], [7.0, 2.0, 3.0], [2.0, 
2.0, 4.0]], [[6.0, 4.0, 7.0], [7.0, 1.0, 2.0], [4.0, 5.0, 8.0]]], [[[3.0, 8.0, 8.0], [1.0, 5.0, 3.0], [6.0, 2.0, 3.0]], [[4.0, 
5.0, 1.0], [8.0, 3.0, 4.0], [7.0, 5.0, 2.0]], [[2.0, 7.0, 5.0], [5.0, 4.0, 6.0], [4.0, 6.0, 8.0]]], [[[6.0, 8.0, 3.0], [2.0, 
4.0, 8.0], [6.0, 5.0, 8.0]], [[2.0, 3.0, 3.0], [4.0, 4.0, 6.0], [2.0, 8.0, 2.0]], [[1.0, 8.0, 1.0], [5.0, 6.0, 7.0], [5.0, 7.0, 
8.0]]], [[[4.0, 7.0, 6.0], [1.0, 2.0, 6.0], [5.0, 7.0, 2.0]], [[5.0, 2.0, 2.0], [8.0, 7.0, 6.0], [6.0, 3.0, 1.0]], [[2.0, 2.0, 
4.0], [1.0, 6.0, 8.0], [4.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 3.0], [7.0, 2.0, 4.0], [1.0, 4.0, 8.0]], 
[[5.0, 4.0, 1.0], [6.0, 2.0, 7.0], [3.0, 2.0, 8.0]], [[7.0, 3.0, 2.0], [5.0, 7.0, 5.0], [6.0, 6.0, 7.0]]], [[[5.0, 4.0, 8.0], 
[1.0, 3.0, 8.0], [1.0, 1.0, 5.0]], [[5.0, 6.0, 5.0], [2.0, 6.0, 6.0], [2.0, 7.0, 7.0]], [[8.0, 2.0, 2.0], [8.0, 8.0, 3.0], 
[4.0, 5.0, 3.0]]], [[[4.0, 3.0, 4.0], [1.0, 7.0, 4.0], [7.0, 7.0, 2.0]], [[4.0, 4.0, 6.0], [6.0, 8.0, 4.0], [6.0, 8.0, 1.0]], 
[[5.0, 4.0, 5.0], [1.0, 3.0, 2.0], [1.0, 3.0, 6.0]]], [[[2.0, 6.0, 7.0], [3.0, 2.0, 5.0], [1.0, 2.0, 1.0]], [[7.0, 3.0, 3.0], 
[8.0, 6.0, 5.0], [2.0, 7.0, 5.0]], [[2.0, 5.0, 1.0], [7.0, 8.0, 6.0], [6.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 4.0], [3.0, 3.0, 4.0], [1.0, 5.0, 7.0]], [[2.0, 1.0, 4.0], [5.0, 4.0, 4.0], [5.0, 
6.0, 3.0]], [[5.0, 2.0, 1.0], [7.0, 3.0, 5.0], [5.0, 4.0, 2.0]]], [[[3.0, 2.0, 4.0], [6.0, 6.0, 1.0], [1.0, 1.0, 5.0]], [[1.0, 
1.0, 7.0], [5.0, 1.0, 5.0], [3.0, 1.0, 5.0]], [[1.0, 5.0, 3.0], [3.0, 5.0, 2.0], [5.0, 3.0, 3.0]]], [[[6.0, 1.0, 5.0], [2.0, 
6.0, 6.0], [2.0, 6.0, 2.0]], [[7.0, 7.0, 7.0], [6.0, 2.0, 8.0], [5.0, 1.0, 7.0]], [[5.0, 1.0, 3.0], [8.0, 7.0, 8.0], [4.0, 4.0, 
2.0]]], [[[4.0, 1.0, 3.0], [5.0, 3.0, 4.0], [2.0, 8.0, 1.0]], [[2.0, 6.0, 3.0], [6.0, 1.0, 3.0], [7.0, 2.0, 4.0]], [[5.0, 5.0, 
4.0], [3.0, 1.0, 2.0], [6.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 1.0]], 
[[8.0, 5.0, 1.0], [5.0, 3.0, 5.0], [6.0, 4.0, 5.0]], [[7.0, 8.0, 8.0], [2.0, 5.0, 4.0], [1.0, 3.0, 3.0]]], [[[8.0, 3.0, 3.0], 
[8.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[5.0, 5.0, 1.0], [2.0, 2.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 4.0, 5.0], [8.0, 5.0, 2.0], 
[2.0, 3.0, 4.0]]], [[[6.0, 5.0, 2.0], [8.0, 5.0, 7.0], [7.0, 4.0, 4.0]], [[2.0, 2.0, 8.0], [6.0, 5.0, 1.0], [1.0, 4.0, 7.0]], 
[[8.0, 1.0, 1.0], [7.0, 7.0, 7.0], [2.0, 2.0, 4.0]]], [[[4.0, 2.0, 5.0], [7.0, 1.0, 5.0], [7.0, 7.0, 4.0]], [[6.0, 8.0, 1.0], 
[7.0, 6.0, 7.0], [3.0, 5.0, 7.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 5.0], [2.0, 1.0, 8.0]]]])
      val=numpy.array([[[1.0, 2.0, 8.0], [3.0, 6.0, 4.0], [5.0, 4.0, 3.0]], [[8.0, 2.0, 1.0], [3.0, 4.0, 8.0], [5.0, 7.0, 
8.0]]])
      arg[2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 4.0], [3.0, 3.0, 4.0], [1.0, 5.0, 7.0]], [[2.0, 1.0, 4.0], [5.0, 4.0, 4.0], [5.0, 
6.0, 3.0]], [[5.0, 2.0, 1.0], [7.0, 3.0, 5.0], [5.0, 4.0, 2.0]]], [[[3.0, 2.0, 4.0], [6.0, 6.0, 1.0], [1.0, 1.0, 5.0]], [[1.0, 
1.0, 7.0], [5.0, 1.0, 5.0], [3.0, 1.0, 5.0]], [[1.0, 5.0, 3.0], [3.0, 5.0, 2.0], [5.0, 3.0, 3.0]]], [[[1.0, 2.0, 8.0], [3.0, 
6.0, 4.0], [5.0, 4.0, 3.0]], [[8.0, 2.0, 1.0], [3.0, 4.0, 8.0], [5.0, 7.0, 8.0]], [[5.0, 1.0, 3.0], [8.0, 7.0, 8.0], [4.0, 4.0, 
2.0]]], [[[4.0, 1.0, 3.0], [5.0, 3.0, 4.0], [2.0, 8.0, 1.0]], [[2.0, 6.0, 3.0], [6.0, 1.0, 3.0], [7.0, 2.0, 4.0]], [[5.0, 5.0, 
4.0], [3.0, 1.0, 2.0], [6.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 6.0], [4.0, 1.0, 6.0], [2.0, 2.0, 1.0]], 
[[8.0, 5.0, 1.0], [5.0, 3.0, 5.0], [6.0, 4.0, 5.0]], [[7.0, 8.0, 8.0], [2.0, 5.0, 4.0], [1.0, 3.0, 3.0]]], [[[8.0, 3.0, 3.0], 
[8.0, 2.0, 6.0], [8.0, 6.0, 5.0]], [[5.0, 5.0, 1.0], [2.0, 2.0, 7.0], [4.0, 1.0, 4.0]], [[3.0, 4.0, 5.0], [8.0, 5.0, 2.0], 
[2.0, 3.0, 4.0]]], [[[1.0, 2.0, 8.0], [3.0, 6.0, 4.0], [5.0, 4.0, 3.0]], [[8.0, 2.0, 1.0], [3.0, 4.0, 8.0], [5.0, 7.0, 8.0]], 
[[8.0, 1.0, 1.0], [7.0, 7.0, 7.0], [2.0, 2.0, 4.0]]], [[[4.0, 2.0, 5.0], [7.0, 1.0, 5.0], [7.0, 7.0, 4.0]], [[6.0, 8.0, 1.0], 
[7.0, 6.0, 7.0], [3.0, 5.0, 7.0]], [[5.0, 1.0, 2.0], [6.0, 1.0, 5.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 8.0], [1.0, 3.0, 8.0], [3.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], [3.0, 2.0, 3.0], [6.0, 
3.0, 1.0]], [[5.0, 5.0, 4.0], [7.0, 5.0, 4.0], [4.0, 6.0, 2.0]]], [[[2.0, 4.0, 2.0], [6.0, 8.0, 5.0], [8.0, 8.0, 1.0]], [[5.0, 
4.0, 4.0], [1.0, 3.0, 7.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 2.0, 2.0], [4.0, 2.0, 4.0]]], [[[3.0, 7.0, 2.0], [8.0, 
8.0, 4.0], [6.0, 6.0, 1.0]], [[3.0, 2.0, 5.0], [7.0, 7.0, 3.0], [5.0, 2.0, 2.0]], [[2.0, 2.0, 1.0], [7.0, 5.0, 5.0], [5.0, 5.0, 
5.0]]], [[[7.0, 2.0, 2.0], [8.0, 3.0, 2.0], [5.0, 3.0, 6.0]], [[8.0, 2.0, 7.0], [6.0, 6.0, 8.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 
8.0], [6.0, 4.0, 5.0], [5.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 4.0, 7.0], [7.0, 1.0, 4.0]], 
[[4.0, 2.0, 8.0], [5.0, 2.0, 7.0], [6.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [3.0, 8.0, 7.0], [3.0, 5.0, 2.0]]], [[[6.0, 2.0, 1.0], 
[8.0, 8.0, 7.0], [7.0, 8.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 8.0, 6.0], [7.0, 7.0, 3.0]], [[6.0, 8.0, 6.0], [4.0, 2.0, 4.0], 
[2.0, 2.0, 3.0]]], [[[6.0, 6.0, 5.0], [8.0, 7.0, 5.0], [4.0, 1.0, 5.0]], [[3.0, 1.0, 8.0], [7.0, 8.0, 5.0], [3.0, 7.0, 7.0]], 
[[7.0, 3.0, 5.0], [5.0, 4.0, 7.0], [6.0, 8.0, 8.0]]], [[[3.0, 7.0, 5.0], [7.0, 1.0, 2.0], [1.0, 1.0, 8.0]], [[8.0, 5.0, 2.0], 
[5.0, 2.0, 8.0], [8.0, 3.0, 8.0]], [[6.0, 6.0, 8.0], [2.0, 6.0, 7.0], [3.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[1.0, 4.0, 4.0], [1.0, 2.0, 8.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 8.0], [2.0, 4.0, 8.0], [5.0, 3.0, 
6.0]]]),self.functionspace)
      arg[2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 8.0], [1.0, 3.0, 8.0], [3.0, 8.0, 6.0]], [[5.0, 1.0, 8.0], [3.0, 2.0, 3.0], [6.0, 
3.0, 1.0]], [[5.0, 5.0, 4.0], [7.0, 5.0, 4.0], [4.0, 6.0, 2.0]]], [[[2.0, 4.0, 2.0], [6.0, 8.0, 5.0], [8.0, 8.0, 1.0]], [[5.0, 
4.0, 4.0], [1.0, 3.0, 7.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [5.0, 2.0, 2.0], [4.0, 2.0, 4.0]]], [[[1.0, 4.0, 4.0], [1.0, 
2.0, 8.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 8.0], [2.0, 4.0, 8.0], [5.0, 3.0, 6.0]], [[2.0, 2.0, 1.0], [7.0, 5.0, 5.0], [5.0, 5.0, 
5.0]]], [[[7.0, 2.0, 2.0], [8.0, 3.0, 2.0], [5.0, 3.0, 6.0]], [[8.0, 2.0, 7.0], [6.0, 6.0, 8.0], [8.0, 7.0, 6.0]], [[4.0, 3.0, 
8.0], [6.0, 4.0, 5.0], [5.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 5.0], [6.0, 4.0, 7.0], [7.0, 1.0, 4.0]], 
[[4.0, 2.0, 8.0], [5.0, 2.0, 7.0], [6.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [3.0, 8.0, 7.0], [3.0, 5.0, 2.0]]], [[[6.0, 2.0, 1.0], 
[8.0, 8.0, 7.0], [7.0, 8.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 8.0, 6.0], [7.0, 7.0, 3.0]], [[6.0, 8.0, 6.0], [4.0, 2.0, 4.0], 
[2.0, 2.0, 3.0]]], [[[1.0, 4.0, 4.0], [1.0, 2.0, 8.0], [6.0, 7.0, 2.0]], [[2.0, 1.0, 8.0], [2.0, 4.0, 8.0], [5.0, 3.0, 6.0]], 
[[7.0, 3.0, 5.0], [5.0, 4.0, 7.0], [6.0, 8.0, 8.0]]], [[[3.0, 7.0, 5.0], [7.0, 1.0, 2.0], [1.0, 1.0, 8.0]], [[8.0, 5.0, 2.0], 
[5.0, 2.0, 8.0], [8.0, 3.0, 8.0]], [[6.0, 6.0, 8.0], [2.0, 6.0, 7.0], [3.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 4.0], [2.0, 8.0, 3.0], [1.0, 8.0, 3.0]], [[2.0, 3.0, 7.0], [2.0, 6.0, 4.0], [6.0, 
3.0, 4.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 8.0], [5.0, 8.0, 8.0]]], [[[6.0, 6.0, 8.0], [1.0, 4.0, 4.0], [4.0, 5.0, 3.0]], [[5.0, 
8.0, 2.0], [4.0, 2.0, 4.0], [1.0, 3.0, 1.0]], [[3.0, 3.0, 4.0], [7.0, 8.0, 7.0], [3.0, 5.0, 4.0]]], [[[1.0, 8.0, 7.0], [5.0, 
7.0, 7.0], [1.0, 4.0, 3.0]], [[1.0, 6.0, 2.0], [3.0, 6.0, 4.0], [1.0, 8.0, 8.0]], [[8.0, 3.0, 4.0], [1.0, 8.0, 4.0], [2.0, 8.0, 
2.0]]], [[[8.0, 5.0, 8.0], [4.0, 3.0, 7.0], [7.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [2.0, 5.0, 2.0], [1.0, 4.0, 5.0]], [[6.0, 1.0, 
8.0], [4.0, 2.0, 8.0], [3.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 6.0], [2.0, 7.0, 2.0], [6.0, 4.0, 7.0]], 
[[1.0, 6.0, 3.0], [7.0, 7.0, 5.0], [7.0, 5.0, 8.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 7.0], [5.0, 8.0, 3.0]]], [[[7.0, 5.0, 7.0], 
[5.0, 3.0, 6.0], [1.0, 3.0, 4.0]], [[6.0, 3.0, 3.0], [8.0, 8.0, 7.0], [6.0, 5.0, 1.0]], [[3.0, 7.0, 7.0], [7.0, 1.0, 5.0], 
[7.0, 4.0, 1.0]]], [[[4.0, 4.0, 6.0], [6.0, 3.0, 8.0], [3.0, 1.0, 3.0]], [[8.0, 2.0, 7.0], [6.0, 2.0, 6.0], [2.0, 2.0, 6.0]], 
[[4.0, 6.0, 4.0], [4.0, 5.0, 7.0], [5.0, 7.0, 2.0]]], [[[7.0, 4.0, 7.0], [8.0, 5.0, 2.0], [6.0, 5.0, 2.0]], [[2.0, 1.0, 4.0], 
[3.0, 8.0, 2.0], [3.0, 8.0, 8.0]], [[5.0, 4.0, 8.0], [7.0, 3.0, 1.0], [1.0, 5.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 4.0, 4.0], [4.0, 4.0, 1.0], [5.0, 6.0, 5.0]], [[1.0, 2.0, 2.0], [6.0, 7.0, 7.0], [5.0, 
2.0, 4.0]]])+(1.-msk_val)*numpy.array([[[1.0, 6.0, 6.0], [1.0, 2.0, 6.0], [5.0, 8.0, 3.0]], [[2.0, 3.0, 7.0], [5.0, 6.0, 8.0], 
[1.0, 8.0, 3.0]]])
      arg[2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [2.0, 8.0, 3.0], [1.0, 8.0, 3.0]], [[2.0, 3.0, 7.0], [2.0, 6.0, 4.0], [6.0, 
3.0, 4.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 8.0], [5.0, 8.0, 8.0]]], [[[6.0, 6.0, 8.0], [1.0, 4.0, 4.0], [4.0, 5.0, 3.0]], [[5.0, 
8.0, 2.0], [4.0, 2.0, 4.0], [1.0, 3.0, 1.0]], [[3.0, 3.0, 4.0], [7.0, 8.0, 7.0], [3.0, 5.0, 4.0]]], [[[2.0, 4.0, 4.0], [4.0, 
4.0, 1.0], [5.0, 6.0, 5.0]], [[1.0, 2.0, 2.0], [6.0, 7.0, 7.0], [5.0, 2.0, 4.0]], [[8.0, 3.0, 4.0], [1.0, 8.0, 4.0], [2.0, 8.0, 
2.0]]], [[[8.0, 5.0, 8.0], [4.0, 3.0, 7.0], [7.0, 3.0, 8.0]], [[5.0, 5.0, 1.0], [2.0, 5.0, 2.0], [1.0, 4.0, 5.0]], [[6.0, 1.0, 
8.0], [4.0, 2.0, 8.0], [3.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 6.0], [2.0, 7.0, 2.0], [6.0, 4.0, 7.0]], 
[[1.0, 6.0, 3.0], [7.0, 7.0, 5.0], [7.0, 5.0, 8.0]], [[6.0, 3.0, 5.0], [2.0, 7.0, 7.0], [5.0, 8.0, 3.0]]], [[[7.0, 5.0, 7.0], 
[5.0, 3.0, 6.0], [1.0, 3.0, 4.0]], [[6.0, 3.0, 3.0], [8.0, 8.0, 7.0], [6.0, 5.0, 1.0]], [[3.0, 7.0, 7.0], [7.0, 1.0, 5.0], 
[7.0, 4.0, 1.0]]], [[[1.0, 6.0, 6.0], [1.0, 2.0, 6.0], [5.0, 8.0, 3.0]], [[2.0, 3.0, 7.0], [5.0, 6.0, 8.0], [1.0, 8.0, 3.0]], 
[[4.0, 6.0, 4.0], [4.0, 5.0, 7.0], [5.0, 7.0, 2.0]]], [[[7.0, 4.0, 7.0], [8.0, 5.0, 2.0], [6.0, 5.0, 2.0]], [[2.0, 1.0, 4.0], 
[3.0, 8.0, 2.0], [3.0, 8.0, 8.0]], [[5.0, 4.0, 8.0], [7.0, 3.0, 1.0], [1.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 8.0], [2.0, 4.0, 7.0], [2.0, 8.0, 1.0]], [[2.0, 4.0, 7.0], [4.0, 5.0, 4.0], [7.0, 
3.0, 2.0]], [[4.0, 5.0, 6.0], [2.0, 2.0, 8.0], [8.0, 6.0, 4.0]]], [[[7.0, 4.0, 2.0], [1.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 
7.0, 2.0], [6.0, 7.0, 4.0], [1.0, 2.0, 6.0]], [[3.0, 3.0, 1.0], [1.0, 4.0, 3.0], [5.0, 7.0, 2.0]]], [[[8.0, 3.0, 3.0], [2.0, 
6.0, 8.0], [2.0, 6.0, 7.0]], [[7.0, 5.0, 4.0], [2.0, 2.0, 5.0], [8.0, 6.0, 7.0]], [[4.0, 6.0, 8.0], [7.0, 1.0, 5.0], [2.0, 7.0, 
4.0]]], [[[1.0, 2.0, 7.0], [7.0, 7.0, 8.0], [7.0, 4.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 2.0, 5.0], [1.0, 2.0, 4.0]], [[6.0, 8.0, 
6.0], [6.0, 8.0, 8.0], [7.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 5.0], [8.0, 5.0, 8.0], [1.0, 5.0, 7.0]], 
[[8.0, 4.0, 3.0], [1.0, 8.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 2.0, 7.0], [1.0, 7.0, 8.0], [5.0, 4.0, 5.0]]], [[[3.0, 5.0, 1.0], 
[7.0, 4.0, 4.0], [2.0, 4.0, 7.0]], [[5.0, 7.0, 8.0], [1.0, 2.0, 1.0], [6.0, 4.0, 8.0]], [[6.0, 1.0, 2.0], [4.0, 3.0, 5.0], 
[6.0, 5.0, 3.0]]], [[[7.0, 2.0, 8.0], [3.0, 4.0, 3.0], [2.0, 8.0, 6.0]], [[1.0, 4.0, 3.0], [5.0, 6.0, 1.0], [7.0, 3.0, 8.0]], 
[[7.0, 8.0, 7.0], [3.0, 1.0, 2.0], [7.0, 8.0, 7.0]]], [[[7.0, 5.0, 4.0], [2.0, 1.0, 3.0], [6.0, 1.0, 4.0]], [[8.0, 3.0, 3.0], 
[6.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[7.0, 5.0, 3.0], [7.0, 7.0, 7.0], [7.0, 8.0, 8.0]]]])
      val=numpy.array([[7.0, 4.0, 1.0]])
      arg[2,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 8.0], [2.0, 4.0, 7.0], [2.0, 8.0, 1.0]], [[2.0, 4.0, 7.0], [4.0, 5.0, 4.0], [7.0, 
3.0, 2.0]], [[4.0, 5.0, 6.0], [2.0, 2.0, 8.0], [8.0, 6.0, 4.0]]], [[[7.0, 4.0, 2.0], [1.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 
7.0, 2.0], [6.0, 7.0, 4.0], [1.0, 2.0, 6.0]], [[3.0, 3.0, 1.0], [1.0, 4.0, 3.0], [5.0, 7.0, 2.0]]], [[[8.0, 3.0, 3.0], [2.0, 
6.0, 8.0], [2.0, 6.0, 7.0]], [[7.0, 5.0, 4.0], [7.0, 4.0, 1.0], [8.0, 6.0, 7.0]], [[4.0, 6.0, 8.0], [7.0, 1.0, 5.0], [2.0, 7.0, 
4.0]]], [[[1.0, 2.0, 7.0], [7.0, 7.0, 8.0], [7.0, 4.0, 6.0]], [[6.0, 7.0, 4.0], [3.0, 2.0, 5.0], [1.0, 2.0, 4.0]], [[6.0, 8.0, 
6.0], [6.0, 8.0, 8.0], [7.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [8.0, 5.0, 8.0], [1.0, 5.0, 7.0]], 
[[8.0, 4.0, 3.0], [1.0, 8.0, 4.0], [8.0, 8.0, 2.0]], [[3.0, 2.0, 7.0], [1.0, 7.0, 8.0], [5.0, 4.0, 5.0]]], [[[3.0, 5.0, 1.0], 
[7.0, 4.0, 4.0], [2.0, 4.0, 7.0]], [[5.0, 7.0, 8.0], [1.0, 2.0, 1.0], [6.0, 4.0, 8.0]], [[6.0, 1.0, 2.0], [4.0, 3.0, 5.0], 
[6.0, 5.0, 3.0]]], [[[7.0, 2.0, 8.0], [3.0, 4.0, 3.0], [2.0, 8.0, 6.0]], [[1.0, 4.0, 3.0], [7.0, 4.0, 1.0], [7.0, 3.0, 8.0]], 
[[7.0, 8.0, 7.0], [3.0, 1.0, 2.0], [7.0, 8.0, 7.0]]], [[[7.0, 5.0, 4.0], [2.0, 1.0, 3.0], [6.0, 1.0, 4.0]], [[8.0, 3.0, 3.0], 
[6.0, 5.0, 1.0], [7.0, 8.0, 3.0]], [[7.0, 5.0, 3.0], [7.0, 7.0, 7.0], [7.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 6.0], [6.0, 4.0, 3.0], [7.0, 1.0, 1.0]], [[3.0, 5.0, 2.0], [5.0, 4.0, 5.0], [2.0, 
7.0, 1.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 6.0], [1.0, 7.0, 6.0]]], [[[1.0, 5.0, 7.0], [4.0, 4.0, 7.0], [6.0, 5.0, 6.0]], [[4.0, 
6.0, 1.0], [6.0, 7.0, 5.0], [1.0, 3.0, 1.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 1.0], [4.0, 7.0, 4.0]]], [[[5.0, 1.0, 2.0], [5.0, 
3.0, 7.0], [3.0, 8.0, 5.0]], [[2.0, 7.0, 4.0], [4.0, 3.0, 5.0], [8.0, 3.0, 3.0]], [[7.0, 1.0, 7.0], [5.0, 1.0, 4.0], [5.0, 4.0, 
8.0]]], [[[2.0, 8.0, 1.0], [6.0, 6.0, 7.0], [3.0, 7.0, 8.0]], [[5.0, 4.0, 4.0], [7.0, 1.0, 1.0], [1.0, 2.0, 6.0]], [[5.0, 2.0, 
5.0], [4.0, 7.0, 5.0], [3.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 8.0], [2.0, 8.0, 4.0], [2.0, 6.0, 4.0]], 
[[1.0, 2.0, 3.0], [2.0, 8.0, 6.0], [3.0, 3.0, 5.0]], [[5.0, 6.0, 3.0], [2.0, 6.0, 3.0], [3.0, 3.0, 7.0]]], [[[3.0, 5.0, 7.0], 
[8.0, 7.0, 6.0], [1.0, 5.0, 2.0]], [[6.0, 5.0, 3.0], [2.0, 2.0, 3.0], [2.0, 3.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 3.0, 4.0], 
[3.0, 5.0, 7.0]]], [[[3.0, 3.0, 6.0], [2.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[3.0, 2.0, 2.0], [8.0, 8.0, 1.0], [1.0, 1.0, 1.0]], 
[[2.0, 1.0, 2.0], [4.0, 5.0, 2.0], [8.0, 8.0, 7.0]]], [[[8.0, 8.0, 4.0], [4.0, 8.0, 1.0], [5.0, 4.0, 4.0]], [[4.0, 8.0, 7.0], 
[1.0, 2.0, 6.0], [7.0, 6.0, 1.0]], [[6.0, 2.0, 6.0], [2.0, 8.0, 3.0], [3.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[3.0, 3.0, 6.0]]),self.functionspace)
      arg[2,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 6.0], [6.0, 4.0, 3.0], [7.0, 1.0, 1.0]], [[3.0, 5.0, 2.0], [5.0, 4.0, 5.0], [2.0, 
7.0, 1.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 6.0], [1.0, 7.0, 6.0]]], [[[1.0, 5.0, 7.0], [4.0, 4.0, 7.0], [6.0, 5.0, 6.0]], [[4.0, 
6.0, 1.0], [6.0, 7.0, 5.0], [1.0, 3.0, 1.0]], [[6.0, 1.0, 5.0], [5.0, 1.0, 1.0], [4.0, 7.0, 4.0]]], [[[5.0, 1.0, 2.0], [5.0, 
3.0, 7.0], [3.0, 8.0, 5.0]], [[2.0, 7.0, 4.0], [3.0, 3.0, 6.0], [8.0, 3.0, 3.0]], [[7.0, 1.0, 7.0], [5.0, 1.0, 4.0], [5.0, 4.0, 
8.0]]], [[[2.0, 8.0, 1.0], [6.0, 6.0, 7.0], [3.0, 7.0, 8.0]], [[5.0, 4.0, 4.0], [7.0, 1.0, 1.0], [1.0, 2.0, 6.0]], [[5.0, 2.0, 
5.0], [4.0, 7.0, 5.0], [3.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [2.0, 8.0, 4.0], [2.0, 6.0, 4.0]], 
[[1.0, 2.0, 3.0], [2.0, 8.0, 6.0], [3.0, 3.0, 5.0]], [[5.0, 6.0, 3.0], [2.0, 6.0, 3.0], [3.0, 3.0, 7.0]]], [[[3.0, 5.0, 7.0], 
[8.0, 7.0, 6.0], [1.0, 5.0, 2.0]], [[6.0, 5.0, 3.0], [2.0, 2.0, 3.0], [2.0, 3.0, 3.0]], [[8.0, 5.0, 5.0], [7.0, 3.0, 4.0], 
[3.0, 5.0, 7.0]]], [[[3.0, 3.0, 6.0], [2.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[3.0, 2.0, 2.0], [3.0, 3.0, 6.0], [1.0, 1.0, 1.0]], 
[[2.0, 1.0, 2.0], [4.0, 5.0, 2.0], [8.0, 8.0, 7.0]]], [[[8.0, 8.0, 4.0], [4.0, 8.0, 1.0], [5.0, 4.0, 4.0]], [[4.0, 8.0, 7.0], 
[1.0, 2.0, 6.0], [7.0, 6.0, 1.0]], [[6.0, 2.0, 6.0], [2.0, 8.0, 3.0], [3.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 1.0], [7.0, 3.0, 7.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 5.0], [1.0, 8.0, 8.0], [3.0, 
5.0, 6.0]], [[4.0, 5.0, 4.0], [6.0, 8.0, 7.0], [5.0, 8.0, 2.0]]], [[[3.0, 1.0, 6.0], [4.0, 3.0, 6.0], [4.0, 3.0, 1.0]], [[2.0, 
6.0, 6.0], [5.0, 2.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 2.0, 2.0], [8.0, 2.0, 2.0], [6.0, 8.0, 5.0]]], [[[5.0, 5.0, 1.0], [1.0, 
4.0, 1.0], [8.0, 6.0, 6.0]], [[2.0, 8.0, 2.0], [5.0, 5.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 5.0, 1.0], [7.0, 1.0, 
8.0]]], [[[2.0, 4.0, 5.0], [6.0, 4.0, 5.0], [7.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [4.0, 6.0, 2.0], [3.0, 4.0, 5.0]], [[6.0, 4.0, 
3.0], [4.0, 3.0, 2.0], [7.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 4.0], [6.0, 4.0, 3.0], [4.0, 4.0, 3.0]], 
[[4.0, 4.0, 3.0], [5.0, 4.0, 4.0], [1.0, 2.0, 1.0]], [[1.0, 2.0, 6.0], [8.0, 3.0, 7.0], [7.0, 3.0, 4.0]]], [[[4.0, 8.0, 2.0], 
[5.0, 5.0, 1.0], [3.0, 8.0, 2.0]], [[8.0, 4.0, 4.0], [5.0, 7.0, 5.0], [5.0, 2.0, 7.0]], [[6.0, 7.0, 8.0], [8.0, 2.0, 7.0], 
[3.0, 8.0, 8.0]]], [[[2.0, 4.0, 7.0], [3.0, 5.0, 2.0], [8.0, 8.0, 4.0]], [[7.0, 5.0, 7.0], [4.0, 7.0, 1.0], [2.0, 1.0, 6.0]], 
[[4.0, 4.0, 1.0], [3.0, 1.0, 5.0], [2.0, 6.0, 4.0]]], [[[7.0, 6.0, 1.0], [5.0, 3.0, 7.0], [2.0, 6.0, 5.0]], [[3.0, 5.0, 4.0], 
[2.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[8.0, 1.0, 3.0], [8.0, 7.0, 6.0], [1.0, 1.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 5.0, 4.0]])+(1.-msk_val)*numpy.array([[7.0, 5.0, 5.0]])
      arg[2,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 1.0], [7.0, 3.0, 7.0], [1.0, 3.0, 5.0]], [[5.0, 7.0, 5.0], [1.0, 8.0, 8.0], [3.0, 
5.0, 6.0]], [[4.0, 5.0, 4.0], [6.0, 8.0, 7.0], [5.0, 8.0, 2.0]]], [[[3.0, 1.0, 6.0], [4.0, 3.0, 6.0], [4.0, 3.0, 1.0]], [[2.0, 
6.0, 6.0], [5.0, 2.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 2.0, 2.0], [8.0, 2.0, 2.0], [6.0, 8.0, 5.0]]], [[[5.0, 5.0, 1.0], [1.0, 
4.0, 1.0], [8.0, 6.0, 6.0]], [[2.0, 8.0, 2.0], [7.0, 5.0, 4.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 5.0, 1.0], [7.0, 1.0, 
8.0]]], [[[2.0, 4.0, 5.0], [6.0, 4.0, 5.0], [7.0, 1.0, 6.0]], [[1.0, 3.0, 1.0], [4.0, 6.0, 2.0], [3.0, 4.0, 5.0]], [[6.0, 4.0, 
3.0], [4.0, 3.0, 2.0], [7.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 4.0], [6.0, 4.0, 3.0], [4.0, 4.0, 3.0]], 
[[4.0, 4.0, 3.0], [5.0, 4.0, 4.0], [1.0, 2.0, 1.0]], [[1.0, 2.0, 6.0], [8.0, 3.0, 7.0], [7.0, 3.0, 4.0]]], [[[4.0, 8.0, 2.0], 
[5.0, 5.0, 1.0], [3.0, 8.0, 2.0]], [[8.0, 4.0, 4.0], [5.0, 7.0, 5.0], [5.0, 2.0, 7.0]], [[6.0, 7.0, 8.0], [8.0, 2.0, 7.0], 
[3.0, 8.0, 8.0]]], [[[2.0, 4.0, 7.0], [3.0, 5.0, 2.0], [8.0, 8.0, 4.0]], [[7.0, 5.0, 7.0], [7.0, 5.0, 5.0], [2.0, 1.0, 6.0]], 
[[4.0, 4.0, 1.0], [3.0, 1.0, 5.0], [2.0, 6.0, 4.0]]], [[[7.0, 6.0, 1.0], [5.0, 3.0, 7.0], [2.0, 6.0, 5.0]], [[3.0, 5.0, 4.0], 
[2.0, 6.0, 6.0], [8.0, 4.0, 7.0]], [[8.0, 1.0, 3.0], [8.0, 7.0, 6.0], [1.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 3.0], [3.0, 5.0, 7.0], [4.0, 4.0, 8.0]], [[6.0, 6.0, 4.0], [5.0, 6.0, 4.0], [2.0, 
5.0, 5.0]], [[7.0, 6.0, 6.0], [7.0, 5.0, 3.0], [3.0, 5.0, 8.0]]], [[[3.0, 3.0, 5.0], [4.0, 1.0, 7.0], [2.0, 5.0, 7.0]], [[3.0, 
4.0, 4.0], [6.0, 6.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 5.0, 1.0], [3.0, 2.0, 4.0], [5.0, 6.0, 4.0]]], [[[3.0, 3.0, 3.0], [3.0, 
6.0, 5.0], [6.0, 1.0, 8.0]], [[5.0, 8.0, 2.0], [8.0, 7.0, 6.0], [3.0, 1.0, 3.0]], [[4.0, 1.0, 1.0], [5.0, 6.0, 7.0], [2.0, 4.0, 
6.0]]], [[[3.0, 7.0, 8.0], [3.0, 4.0, 6.0], [4.0, 2.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 3.0, 8.0], [7.0, 4.0, 7.0]], [[5.0, 2.0, 
4.0], [1.0, 2.0, 1.0], [7.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 2.0], [3.0, 7.0, 2.0], [5.0, 3.0, 6.0]], 
[[1.0, 7.0, 8.0], [4.0, 2.0, 4.0], [8.0, 7.0, 6.0]], [[1.0, 6.0, 1.0], [1.0, 2.0, 1.0], [2.0, 4.0, 8.0]]], [[[3.0, 2.0, 7.0], 
[1.0, 7.0, 2.0], [7.0, 5.0, 6.0]], [[2.0, 6.0, 6.0], [4.0, 8.0, 6.0], [8.0, 3.0, 3.0]], [[3.0, 4.0, 6.0], [4.0, 4.0, 2.0], 
[6.0, 2.0, 2.0]]], [[[3.0, 5.0, 7.0], [6.0, 5.0, 8.0], [6.0, 3.0, 5.0]], [[1.0, 1.0, 7.0], [7.0, 6.0, 3.0], [3.0, 6.0, 8.0]], 
[[3.0, 8.0, 4.0], [7.0, 1.0, 6.0], [6.0, 2.0, 4.0]]], [[[3.0, 6.0, 1.0], [7.0, 3.0, 2.0], [6.0, 8.0, 8.0]], [[3.0, 6.0, 2.0], 
[3.0, 8.0, 1.0], [2.0, 6.0, 1.0]], [[8.0, 3.0, 8.0], [7.0, 8.0, 1.0], [2.0, 8.0, 1.0]]]])
      val=numpy.array([[[5.0, 3.0, 8.0], [7.0, 1.0, 3.0], [6.0, 2.0, 7.0]]])
      arg[2,1:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 3.0], [3.0, 5.0, 7.0], [4.0, 4.0, 8.0]], [[6.0, 6.0, 4.0], [5.0, 6.0, 4.0], [2.0, 
5.0, 5.0]], [[7.0, 6.0, 6.0], [7.0, 5.0, 3.0], [3.0, 5.0, 8.0]]], [[[3.0, 3.0, 5.0], [4.0, 1.0, 7.0], [2.0, 5.0, 7.0]], [[3.0, 
4.0, 4.0], [6.0, 6.0, 6.0], [2.0, 4.0, 1.0]], [[6.0, 5.0, 1.0], [3.0, 2.0, 4.0], [5.0, 6.0, 4.0]]], [[[3.0, 3.0, 3.0], [3.0, 
6.0, 5.0], [6.0, 1.0, 8.0]], [[5.0, 3.0, 8.0], [7.0, 1.0, 3.0], [6.0, 2.0, 7.0]], [[4.0, 1.0, 1.0], [5.0, 6.0, 7.0], [2.0, 4.0, 
6.0]]], [[[3.0, 7.0, 8.0], [3.0, 4.0, 6.0], [4.0, 2.0, 4.0]], [[8.0, 6.0, 2.0], [5.0, 3.0, 8.0], [7.0, 4.0, 7.0]], [[5.0, 2.0, 
4.0], [1.0, 2.0, 1.0], [7.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 2.0], [3.0, 7.0, 2.0], [5.0, 3.0, 6.0]], 
[[1.0, 7.0, 8.0], [4.0, 2.0, 4.0], [8.0, 7.0, 6.0]], [[1.0, 6.0, 1.0], [1.0, 2.0, 1.0], [2.0, 4.0, 8.0]]], [[[3.0, 2.0, 7.0], 
[1.0, 7.0, 2.0], [7.0, 5.0, 6.0]], [[2.0, 6.0, 6.0], [4.0, 8.0, 6.0], [8.0, 3.0, 3.0]], [[3.0, 4.0, 6.0], [4.0, 4.0, 2.0], 
[6.0, 2.0, 2.0]]], [[[3.0, 5.0, 7.0], [6.0, 5.0, 8.0], [6.0, 3.0, 5.0]], [[5.0, 3.0, 8.0], [7.0, 1.0, 3.0], [6.0, 2.0, 7.0]], 
[[3.0, 8.0, 4.0], [7.0, 1.0, 6.0], [6.0, 2.0, 4.0]]], [[[3.0, 6.0, 1.0], [7.0, 3.0, 2.0], [6.0, 8.0, 8.0]], [[3.0, 6.0, 2.0], 
[3.0, 8.0, 1.0], [2.0, 6.0, 1.0]], [[8.0, 3.0, 8.0], [7.0, 8.0, 1.0], [2.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 7.0], [5.0, 6.0, 3.0], [2.0, 4.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 6.0, 6.0], [7.0, 
3.0, 8.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 4.0], [3.0, 5.0, 7.0]]], [[[1.0, 6.0, 8.0], [4.0, 6.0, 4.0], [4.0, 5.0, 1.0]], [[4.0, 
8.0, 7.0], [2.0, 5.0, 8.0], [4.0, 7.0, 1.0]], [[6.0, 5.0, 3.0], [7.0, 2.0, 2.0], [4.0, 6.0, 2.0]]], [[[2.0, 4.0, 5.0], [3.0, 
3.0, 5.0], [8.0, 4.0, 2.0]], [[6.0, 1.0, 7.0], [5.0, 8.0, 3.0], [5.0, 3.0, 5.0]], [[2.0, 2.0, 3.0], [4.0, 3.0, 5.0], [5.0, 8.0, 
5.0]]], [[[6.0, 2.0, 4.0], [5.0, 5.0, 2.0], [2.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [6.0, 8.0, 8.0], [4.0, 4.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 5.0, 4.0], [3.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 5.0], [2.0, 8.0, 8.0], [5.0, 8.0, 2.0]], 
[[1.0, 7.0, 2.0], [1.0, 2.0, 4.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 2.0], [8.0, 5.0, 1.0], [7.0, 3.0, 2.0]]], [[[2.0, 7.0, 4.0], 
[4.0, 8.0, 3.0], [7.0, 5.0, 7.0]], [[4.0, 4.0, 8.0], [7.0, 6.0, 5.0], [5.0, 5.0, 8.0]], [[8.0, 8.0, 4.0], [6.0, 1.0, 8.0], 
[7.0, 7.0, 4.0]]], [[[6.0, 6.0, 3.0], [5.0, 7.0, 4.0], [6.0, 1.0, 2.0]], [[5.0, 1.0, 8.0], [6.0, 7.0, 4.0], [2.0, 8.0, 8.0]], 
[[5.0, 4.0, 3.0], [4.0, 7.0, 6.0], [4.0, 5.0, 1.0]]], [[[2.0, 4.0, 2.0], [8.0, 6.0, 5.0], [7.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], 
[5.0, 5.0, 2.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 7.0], [3.0, 4.0, 6.0], [7.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[[2.0, 4.0, 2.0], [3.0, 3.0, 5.0], [1.0, 8.0, 6.0]]]),self.functionspace)
      arg[2,1:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 7.0], [5.0, 6.0, 3.0], [2.0, 4.0, 3.0]], [[1.0, 1.0, 3.0], [5.0, 6.0, 6.0], [7.0, 
3.0, 8.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 4.0], [3.0, 5.0, 7.0]]], [[[1.0, 6.0, 8.0], [4.0, 6.0, 4.0], [4.0, 5.0, 1.0]], [[4.0, 
8.0, 7.0], [2.0, 5.0, 8.0], [4.0, 7.0, 1.0]], [[6.0, 5.0, 3.0], [7.0, 2.0, 2.0], [4.0, 6.0, 2.0]]], [[[2.0, 4.0, 5.0], [3.0, 
3.0, 5.0], [8.0, 4.0, 2.0]], [[2.0, 4.0, 2.0], [3.0, 3.0, 5.0], [1.0, 8.0, 6.0]], [[2.0, 2.0, 3.0], [4.0, 3.0, 5.0], [5.0, 8.0, 
5.0]]], [[[6.0, 2.0, 4.0], [5.0, 5.0, 2.0], [2.0, 7.0, 4.0]], [[4.0, 5.0, 2.0], [6.0, 8.0, 8.0], [4.0, 4.0, 3.0]], [[3.0, 8.0, 
5.0], [1.0, 5.0, 4.0], [3.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 5.0], [2.0, 8.0, 8.0], [5.0, 8.0, 2.0]], 
[[1.0, 7.0, 2.0], [1.0, 2.0, 4.0], [6.0, 4.0, 2.0]], [[7.0, 6.0, 2.0], [8.0, 5.0, 1.0], [7.0, 3.0, 2.0]]], [[[2.0, 7.0, 4.0], 
[4.0, 8.0, 3.0], [7.0, 5.0, 7.0]], [[4.0, 4.0, 8.0], [7.0, 6.0, 5.0], [5.0, 5.0, 8.0]], [[8.0, 8.0, 4.0], [6.0, 1.0, 8.0], 
[7.0, 7.0, 4.0]]], [[[6.0, 6.0, 3.0], [5.0, 7.0, 4.0], [6.0, 1.0, 2.0]], [[2.0, 4.0, 2.0], [3.0, 3.0, 5.0], [1.0, 8.0, 6.0]], 
[[5.0, 4.0, 3.0], [4.0, 7.0, 6.0], [4.0, 5.0, 1.0]]], [[[2.0, 4.0, 2.0], [8.0, 6.0, 5.0], [7.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], 
[5.0, 5.0, 2.0], [3.0, 1.0, 3.0]], [[7.0, 5.0, 7.0], [3.0, 4.0, 6.0], [7.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 7.0], [6.0, 5.0, 8.0], [4.0, 4.0, 5.0]], [[4.0, 1.0, 2.0], [8.0, 8.0, 3.0], [3.0, 
4.0, 7.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 6.0], [6.0, 7.0, 4.0]]], [[[4.0, 6.0, 1.0], [1.0, 1.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 
3.0, 7.0], [3.0, 3.0, 6.0], [4.0, 2.0, 3.0]], [[2.0, 2.0, 1.0], [7.0, 3.0, 3.0], [4.0, 8.0, 1.0]]], [[[1.0, 7.0, 4.0], [1.0, 
1.0, 1.0], [5.0, 7.0, 7.0]], [[4.0, 8.0, 8.0], [6.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[2.0, 7.0, 1.0], [1.0, 1.0, 1.0], [4.0, 3.0, 
6.0]]], [[[5.0, 7.0, 3.0], [5.0, 8.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [3.0, 8.0, 4.0], [2.0, 6.0, 8.0]], [[6.0, 2.0, 
3.0], [7.0, 7.0, 2.0], [1.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 1.0], [5.0, 2.0, 4.0], [4.0, 4.0, 5.0]], 
[[4.0, 3.0, 1.0], [7.0, 2.0, 2.0], [6.0, 6.0, 4.0]], [[7.0, 7.0, 4.0], [6.0, 2.0, 6.0], [5.0, 8.0, 4.0]]], [[[2.0, 1.0, 5.0], 
[4.0, 7.0, 1.0], [7.0, 6.0, 8.0]], [[7.0, 2.0, 3.0], [4.0, 3.0, 4.0], [3.0, 3.0, 5.0]], [[1.0, 1.0, 2.0], [1.0, 6.0, 6.0], 
[6.0, 3.0, 8.0]]], [[[1.0, 1.0, 8.0], [7.0, 8.0, 1.0], [4.0, 7.0, 5.0]], [[2.0, 6.0, 5.0], [5.0, 5.0, 1.0], [2.0, 2.0, 4.0]], 
[[8.0, 1.0, 6.0], [1.0, 6.0, 2.0], [4.0, 2.0, 5.0]]], [[[7.0, 7.0, 3.0], [7.0, 4.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 2.0, 5.0], 
[8.0, 6.0, 1.0], [6.0, 5.0, 6.0]], [[3.0, 5.0, 7.0], [3.0, 5.0, 4.0], [8.0, 2.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 5.0, 2.0], [5.0, 6.0, 4.0], [6.0, 5.0, 1.0]]])+(1.-msk_val)*numpy.array([[[4.0, 7.0, 
4.0], [8.0, 2.0, 3.0], [6.0, 6.0, 7.0]]])
      arg[2,1:2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 7.0], [6.0, 5.0, 8.0], [4.0, 4.0, 5.0]], [[4.0, 1.0, 2.0], [8.0, 8.0, 3.0], [3.0, 
4.0, 7.0]], [[7.0, 7.0, 2.0], [3.0, 2.0, 6.0], [6.0, 7.0, 4.0]]], [[[4.0, 6.0, 1.0], [1.0, 1.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 
3.0, 7.0], [3.0, 3.0, 6.0], [4.0, 2.0, 3.0]], [[2.0, 2.0, 1.0], [7.0, 3.0, 3.0], [4.0, 8.0, 1.0]]], [[[1.0, 7.0, 4.0], [1.0, 
1.0, 1.0], [5.0, 7.0, 7.0]], [[3.0, 5.0, 2.0], [5.0, 6.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 7.0, 1.0], [1.0, 1.0, 1.0], [4.0, 3.0, 
6.0]]], [[[5.0, 7.0, 3.0], [5.0, 8.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [3.0, 8.0, 4.0], [2.0, 6.0, 8.0]], [[6.0, 2.0, 
3.0], [7.0, 7.0, 2.0], [1.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 1.0], [5.0, 2.0, 4.0], [4.0, 4.0, 5.0]], 
[[4.0, 3.0, 1.0], [7.0, 2.0, 2.0], [6.0, 6.0, 4.0]], [[7.0, 7.0, 4.0], [6.0, 2.0, 6.0], [5.0, 8.0, 4.0]]], [[[2.0, 1.0, 5.0], 
[4.0, 7.0, 1.0], [7.0, 6.0, 8.0]], [[7.0, 2.0, 3.0], [4.0, 3.0, 4.0], [3.0, 3.0, 5.0]], [[1.0, 1.0, 2.0], [1.0, 6.0, 6.0], 
[6.0, 3.0, 8.0]]], [[[1.0, 1.0, 8.0], [7.0, 8.0, 1.0], [4.0, 7.0, 5.0]], [[4.0, 7.0, 4.0], [8.0, 2.0, 3.0], [6.0, 6.0, 7.0]], 
[[8.0, 1.0, 6.0], [1.0, 6.0, 2.0], [4.0, 2.0, 5.0]]], [[[7.0, 7.0, 3.0], [7.0, 4.0, 8.0], [1.0, 4.0, 5.0]], [[1.0, 2.0, 5.0], 
[8.0, 6.0, 1.0], [6.0, 5.0, 6.0]], [[3.0, 5.0, 7.0], [3.0, 5.0, 4.0], [8.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 5.0], [4.0, 5.0, 5.0], [4.0, 4.0, 8.0]], [[8.0, 7.0, 8.0], [6.0, 2.0, 5.0], [5.0, 
4.0, 8.0]], [[5.0, 8.0, 5.0], [3.0, 2.0, 3.0], [5.0, 7.0, 3.0]]], [[[4.0, 3.0, 5.0], [6.0, 1.0, 4.0], [4.0, 4.0, 5.0]], [[2.0, 
8.0, 6.0], [6.0, 1.0, 8.0], [3.0, 2.0, 8.0]], [[1.0, 7.0, 4.0], [2.0, 5.0, 5.0], [4.0, 3.0, 7.0]]], [[[7.0, 7.0, 2.0], [1.0, 
7.0, 8.0], [1.0, 8.0, 3.0]], [[3.0, 2.0, 6.0], [4.0, 7.0, 2.0], [7.0, 1.0, 8.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 5.0], [8.0, 1.0, 
7.0]]], [[[4.0, 5.0, 7.0], [8.0, 4.0, 5.0], [7.0, 2.0, 3.0]], [[6.0, 4.0, 4.0], [8.0, 3.0, 5.0], [2.0, 3.0, 1.0]], [[7.0, 7.0, 
4.0], [7.0, 1.0, 6.0], [5.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 1.0], [2.0, 1.0, 6.0], [5.0, 1.0, 8.0]], 
[[7.0, 8.0, 4.0], [7.0, 8.0, 3.0], [1.0, 6.0, 4.0]], [[6.0, 2.0, 1.0], [5.0, 5.0, 7.0], [8.0, 1.0, 1.0]]], [[[5.0, 5.0, 7.0], 
[8.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 5.0, 4.0], [4.0, 1.0, 3.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 6.0], [2.0, 1.0, 1.0], 
[1.0, 4.0, 5.0]]], [[[1.0, 3.0, 3.0], [3.0, 3.0, 6.0], [4.0, 2.0, 2.0]], [[7.0, 8.0, 2.0], [2.0, 2.0, 3.0], [2.0, 5.0, 1.0]], 
[[3.0, 4.0, 1.0], [8.0, 2.0, 3.0], [8.0, 4.0, 1.0]]], [[[7.0, 7.0, 7.0], [2.0, 1.0, 5.0], [1.0, 8.0, 3.0]], [[1.0, 2.0, 8.0], 
[7.0, 8.0, 1.0], [7.0, 7.0, 5.0]], [[8.0, 7.0, 2.0], [3.0, 4.0, 8.0], [6.0, 3.0, 6.0]]]])
      val=numpy.array([[[[3.0, 6.0, 1.0]]], [[[6.0, 8.0, 4.0]]], [[[4.0, 1.0, 1.0]]], [[[1.0, 4.0, 2.0]]]])
      arg[:,1:2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 5.0], [4.0, 5.0, 5.0], [4.0, 4.0, 8.0]], [[8.0, 7.0, 8.0], [3.0, 6.0, 1.0], [5.0, 
4.0, 8.0]], [[5.0, 8.0, 5.0], [3.0, 2.0, 3.0], [5.0, 7.0, 3.0]]], [[[4.0, 3.0, 5.0], [6.0, 1.0, 4.0], [4.0, 4.0, 5.0]], [[2.0, 
8.0, 6.0], [6.0, 8.0, 4.0], [3.0, 2.0, 8.0]], [[1.0, 7.0, 4.0], [2.0, 5.0, 5.0], [4.0, 3.0, 7.0]]], [[[7.0, 7.0, 2.0], [1.0, 
7.0, 8.0], [1.0, 8.0, 3.0]], [[3.0, 2.0, 6.0], [4.0, 1.0, 1.0], [7.0, 1.0, 8.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 5.0], [8.0, 1.0, 
7.0]]], [[[4.0, 5.0, 7.0], [8.0, 4.0, 5.0], [7.0, 2.0, 3.0]], [[6.0, 4.0, 4.0], [1.0, 4.0, 2.0], [2.0, 3.0, 1.0]], [[7.0, 7.0, 
4.0], [7.0, 1.0, 6.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 1.0], [2.0, 1.0, 6.0], [5.0, 1.0, 8.0]], 
[[7.0, 8.0, 4.0], [3.0, 6.0, 1.0], [1.0, 6.0, 4.0]], [[6.0, 2.0, 1.0], [5.0, 5.0, 7.0], [8.0, 1.0, 1.0]]], [[[5.0, 5.0, 7.0], 
[8.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 5.0, 4.0], [6.0, 8.0, 4.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 6.0], [2.0, 1.0, 1.0], 
[1.0, 4.0, 5.0]]], [[[1.0, 3.0, 3.0], [3.0, 3.0, 6.0], [4.0, 2.0, 2.0]], [[7.0, 8.0, 2.0], [4.0, 1.0, 1.0], [2.0, 5.0, 1.0]], 
[[3.0, 4.0, 1.0], [8.0, 2.0, 3.0], [8.0, 4.0, 1.0]]], [[[7.0, 7.0, 7.0], [2.0, 1.0, 5.0], [1.0, 8.0, 3.0]], [[1.0, 2.0, 8.0], 
[1.0, 4.0, 2.0], [7.0, 7.0, 5.0]], [[8.0, 7.0, 2.0], [3.0, 4.0, 8.0], [6.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 5.0], [8.0, 7.0, 1.0], [3.0, 4.0, 6.0]], [[6.0, 6.0, 3.0], [2.0, 5.0, 7.0], [4.0, 
2.0, 2.0]], [[1.0, 2.0, 6.0], [8.0, 2.0, 8.0], [3.0, 1.0, 3.0]]], [[[4.0, 8.0, 1.0], [8.0, 2.0, 2.0], [1.0, 3.0, 2.0]], [[3.0, 
3.0, 8.0], [5.0, 6.0, 2.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 5.0], [4.0, 2.0, 2.0], [7.0, 4.0, 5.0]]], [[[2.0, 4.0, 2.0], [6.0, 
3.0, 7.0], [6.0, 4.0, 1.0]], [[8.0, 8.0, 6.0], [1.0, 5.0, 3.0], [1.0, 7.0, 6.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 5.0], [5.0, 7.0, 
5.0]]], [[[3.0, 3.0, 3.0], [2.0, 3.0, 7.0], [8.0, 2.0, 4.0]], [[8.0, 5.0, 7.0], [6.0, 8.0, 5.0], [5.0, 3.0, 5.0]], [[3.0, 1.0, 
3.0], [6.0, 2.0, 3.0], [4.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 6.0], [5.0, 1.0, 7.0], [1.0, 8.0, 4.0]], 
[[5.0, 7.0, 5.0], [3.0, 7.0, 1.0], [3.0, 6.0, 6.0]], [[5.0, 4.0, 2.0], [5.0, 7.0, 4.0], [1.0, 8.0, 5.0]]], [[[4.0, 3.0, 2.0], 
[4.0, 3.0, 7.0], [8.0, 5.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 7.0, 1.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 6.0], [2.0, 8.0, 8.0], 
[6.0, 5.0, 1.0]]], [[[8.0, 3.0, 4.0], [2.0, 1.0, 8.0], [1.0, 6.0, 2.0]], [[2.0, 6.0, 4.0], [4.0, 8.0, 7.0], [8.0, 2.0, 1.0]], 
[[1.0, 2.0, 5.0], [4.0, 8.0, 4.0], [5.0, 8.0, 3.0]]], [[[3.0, 6.0, 4.0], [1.0, 7.0, 2.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], 
[7.0, 4.0, 2.0], [8.0, 3.0, 5.0]], [[4.0, 8.0, 1.0], [5.0, 8.0, 4.0], [8.0, 4.0, 2.0]]]])
      val=Data(numpy.array([[[[1.0, 3.0, 7.0]]], [[[2.0, 5.0, 6.0]]], [[[8.0, 7.0, 4.0]]], [[[3.0, 1.0, 
8.0]]]]),self.functionspace)
      arg[:,1:2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 5.0], [8.0, 7.0, 1.0], [3.0, 4.0, 6.0]], [[6.0, 6.0, 3.0], [1.0, 3.0, 7.0], [4.0, 
2.0, 2.0]], [[1.0, 2.0, 6.0], [8.0, 2.0, 8.0], [3.0, 1.0, 3.0]]], [[[4.0, 8.0, 1.0], [8.0, 2.0, 2.0], [1.0, 3.0, 2.0]], [[3.0, 
3.0, 8.0], [2.0, 5.0, 6.0], [1.0, 5.0, 6.0]], [[2.0, 8.0, 5.0], [4.0, 2.0, 2.0], [7.0, 4.0, 5.0]]], [[[2.0, 4.0, 2.0], [6.0, 
3.0, 7.0], [6.0, 4.0, 1.0]], [[8.0, 8.0, 6.0], [8.0, 7.0, 4.0], [1.0, 7.0, 6.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 5.0], [5.0, 7.0, 
5.0]]], [[[3.0, 3.0, 3.0], [2.0, 3.0, 7.0], [8.0, 2.0, 4.0]], [[8.0, 5.0, 7.0], [3.0, 1.0, 8.0], [5.0, 3.0, 5.0]], [[3.0, 1.0, 
3.0], [6.0, 2.0, 3.0], [4.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 6.0], [5.0, 1.0, 7.0], [1.0, 8.0, 4.0]], 
[[5.0, 7.0, 5.0], [1.0, 3.0, 7.0], [3.0, 6.0, 6.0]], [[5.0, 4.0, 2.0], [5.0, 7.0, 4.0], [1.0, 8.0, 5.0]]], [[[4.0, 3.0, 2.0], 
[4.0, 3.0, 7.0], [8.0, 5.0, 8.0]], [[2.0, 6.0, 8.0], [2.0, 5.0, 6.0], [7.0, 1.0, 8.0]], [[5.0, 7.0, 6.0], [2.0, 8.0, 8.0], 
[6.0, 5.0, 1.0]]], [[[8.0, 3.0, 4.0], [2.0, 1.0, 8.0], [1.0, 6.0, 2.0]], [[2.0, 6.0, 4.0], [8.0, 7.0, 4.0], [8.0, 2.0, 1.0]], 
[[1.0, 2.0, 5.0], [4.0, 8.0, 4.0], [5.0, 8.0, 3.0]]], [[[3.0, 6.0, 4.0], [1.0, 7.0, 2.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], 
[3.0, 1.0, 8.0], [8.0, 3.0, 5.0]], [[4.0, 8.0, 1.0], [5.0, 8.0, 4.0], [8.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 7.0], [5.0, 6.0, 4.0], [6.0, 2.0, 2.0]], [[6.0, 7.0, 6.0], [7.0, 5.0, 4.0], [6.0, 
7.0, 7.0]], [[2.0, 4.0, 8.0], [6.0, 4.0, 6.0], [4.0, 3.0, 2.0]]], [[[7.0, 1.0, 7.0], [8.0, 5.0, 6.0], [1.0, 2.0, 1.0]], [[3.0, 
4.0, 3.0], [3.0, 8.0, 3.0], [6.0, 3.0, 2.0]], [[7.0, 1.0, 6.0], [3.0, 8.0, 4.0], [6.0, 7.0, 8.0]]], [[[3.0, 4.0, 2.0], [8.0, 
5.0, 6.0], [7.0, 8.0, 3.0]], [[4.0, 1.0, 5.0], [6.0, 8.0, 2.0], [1.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 8.0], [6.0, 8.0, 
1.0]]], [[[3.0, 7.0, 5.0], [2.0, 4.0, 5.0], [4.0, 8.0, 7.0]], [[8.0, 8.0, 2.0], [7.0, 1.0, 2.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 
2.0], [2.0, 5.0, 5.0], [8.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 7.0], [2.0, 8.0, 6.0], [1.0, 8.0, 3.0]], 
[[1.0, 7.0, 2.0], [7.0, 8.0, 5.0], [7.0, 1.0, 8.0]], [[7.0, 6.0, 1.0], [4.0, 7.0, 3.0], [3.0, 1.0, 5.0]]], [[[5.0, 6.0, 7.0], 
[4.0, 3.0, 8.0], [1.0, 3.0, 4.0]], [[6.0, 1.0, 7.0], [6.0, 3.0, 1.0], [1.0, 3.0, 6.0]], [[1.0, 8.0, 2.0], [6.0, 3.0, 4.0], 
[5.0, 5.0, 5.0]]], [[[1.0, 3.0, 7.0], [3.0, 4.0, 7.0], [4.0, 5.0, 5.0]], [[7.0, 2.0, 5.0], [8.0, 6.0, 7.0], [1.0, 2.0, 2.0]], 
[[7.0, 5.0, 5.0], [5.0, 6.0, 1.0], [7.0, 3.0, 6.0]]], [[[8.0, 1.0, 5.0], [8.0, 6.0, 2.0], [7.0, 7.0, 3.0]], [[2.0, 4.0, 5.0], 
[5.0, 1.0, 3.0], [1.0, 1.0, 8.0]], [[3.0, 8.0, 7.0], [2.0, 7.0, 7.0], [6.0, 2.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 3.0, 5.0]]], [[[1.0, 2.0, 4.0]]], [[[1.0, 7.0, 8.0]]], [[[3.0, 2.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 1.0, 1.0]]], [[[4.0, 7.0, 2.0]]], [[[4.0, 2.0, 6.0]]], [[[4.0, 6.0, 2.0]]]])
      arg[:,1:2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 7.0], [5.0, 6.0, 4.0], [6.0, 2.0, 2.0]], [[6.0, 7.0, 6.0], [2.0, 3.0, 5.0], [6.0, 
7.0, 7.0]], [[2.0, 4.0, 8.0], [6.0, 4.0, 6.0], [4.0, 3.0, 2.0]]], [[[7.0, 1.0, 7.0], [8.0, 5.0, 6.0], [1.0, 2.0, 1.0]], [[3.0, 
4.0, 3.0], [1.0, 2.0, 4.0], [6.0, 3.0, 2.0]], [[7.0, 1.0, 6.0], [3.0, 8.0, 4.0], [6.0, 7.0, 8.0]]], [[[3.0, 4.0, 2.0], [8.0, 
5.0, 6.0], [7.0, 8.0, 3.0]], [[4.0, 1.0, 5.0], [1.0, 7.0, 8.0], [1.0, 4.0, 6.0]], [[6.0, 1.0, 2.0], [4.0, 7.0, 8.0], [6.0, 8.0, 
1.0]]], [[[3.0, 7.0, 5.0], [2.0, 4.0, 5.0], [4.0, 8.0, 7.0]], [[8.0, 8.0, 2.0], [3.0, 2.0, 3.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 
2.0], [2.0, 5.0, 5.0], [8.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 7.0], [2.0, 8.0, 6.0], [1.0, 8.0, 3.0]], 
[[1.0, 7.0, 2.0], [5.0, 1.0, 1.0], [7.0, 1.0, 8.0]], [[7.0, 6.0, 1.0], [4.0, 7.0, 3.0], [3.0, 1.0, 5.0]]], [[[5.0, 6.0, 7.0], 
[4.0, 3.0, 8.0], [1.0, 3.0, 4.0]], [[6.0, 1.0, 7.0], [4.0, 7.0, 2.0], [1.0, 3.0, 6.0]], [[1.0, 8.0, 2.0], [6.0, 3.0, 4.0], 
[5.0, 5.0, 5.0]]], [[[1.0, 3.0, 7.0], [3.0, 4.0, 7.0], [4.0, 5.0, 5.0]], [[7.0, 2.0, 5.0], [4.0, 2.0, 6.0], [1.0, 2.0, 2.0]], 
[[7.0, 5.0, 5.0], [5.0, 6.0, 1.0], [7.0, 3.0, 6.0]]], [[[8.0, 1.0, 5.0], [8.0, 6.0, 2.0], [7.0, 7.0, 3.0]], [[2.0, 4.0, 5.0], 
[4.0, 6.0, 2.0], [1.0, 1.0, 8.0]], [[3.0, 8.0, 7.0], [2.0, 7.0, 7.0], [6.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 6.0], [1.0, 8.0, 8.0], [8.0, 2.0, 5.0]], [[2.0, 4.0, 4.0], [6.0, 1.0, 3.0], [5.0, 
7.0, 2.0]], [[4.0, 5.0, 7.0], [1.0, 3.0, 7.0], [7.0, 2.0, 8.0]]], [[[7.0, 6.0, 8.0], [8.0, 3.0, 5.0], [2.0, 2.0, 2.0]], [[1.0, 
1.0, 7.0], [5.0, 4.0, 3.0], [1.0, 6.0, 8.0]], [[2.0, 5.0, 1.0], [5.0, 1.0, 8.0], [2.0, 8.0, 6.0]]], [[[2.0, 1.0, 2.0], [3.0, 
3.0, 8.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 8.0], [5.0, 7.0, 7.0]], [[2.0, 8.0, 4.0], [3.0, 5.0, 2.0], [4.0, 3.0, 
7.0]]], [[[7.0, 2.0, 5.0], [5.0, 6.0, 8.0], [2.0, 4.0, 3.0]], [[5.0, 8.0, 1.0], [5.0, 7.0, 7.0], [4.0, 1.0, 5.0]], [[5.0, 3.0, 
5.0], [5.0, 6.0, 7.0], [4.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 3.0], [6.0, 7.0, 2.0], [8.0, 5.0, 2.0]], 
[[2.0, 7.0, 5.0], [5.0, 2.0, 8.0], [4.0, 3.0, 2.0]], [[5.0, 5.0, 4.0], [5.0, 7.0, 3.0], [5.0, 7.0, 4.0]]], [[[5.0, 5.0, 4.0], 
[2.0, 2.0, 5.0], [6.0, 2.0, 3.0]], [[8.0, 4.0, 7.0], [7.0, 5.0, 2.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 4.0], [1.0, 7.0, 1.0], 
[4.0, 1.0, 8.0]]], [[[3.0, 8.0, 1.0], [6.0, 8.0, 3.0], [6.0, 3.0, 5.0]], [[8.0, 2.0, 7.0], [6.0, 6.0, 6.0], [4.0, 8.0, 8.0]], 
[[8.0, 3.0, 4.0], [7.0, 4.0, 5.0], [4.0, 5.0, 5.0]]], [[[4.0, 6.0, 5.0], [2.0, 6.0, 5.0], [4.0, 6.0, 4.0]], [[7.0, 3.0, 8.0], 
[5.0, 2.0, 7.0], [6.0, 2.0, 6.0]], [[1.0, 7.0, 5.0], [1.0, 4.0, 7.0], [3.0, 5.0, 5.0]]]])
      val=numpy.array([[[[7.0, 4.0, 5.0]]], [[[4.0, 8.0, 6.0]]]])
      arg[2:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 6.0], [1.0, 8.0, 8.0], [8.0, 2.0, 5.0]], [[2.0, 4.0, 4.0], [6.0, 1.0, 3.0], [5.0, 
7.0, 2.0]], [[4.0, 5.0, 7.0], [1.0, 3.0, 7.0], [7.0, 2.0, 8.0]]], [[[7.0, 6.0, 8.0], [8.0, 3.0, 5.0], [2.0, 2.0, 2.0]], [[1.0, 
1.0, 7.0], [5.0, 4.0, 3.0], [1.0, 6.0, 8.0]], [[2.0, 5.0, 1.0], [5.0, 1.0, 8.0], [2.0, 8.0, 6.0]]], [[[2.0, 1.0, 2.0], [3.0, 
3.0, 8.0], [4.0, 2.0, 6.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 8.0], [5.0, 7.0, 7.0]], [[2.0, 8.0, 4.0], [7.0, 4.0, 5.0], [4.0, 3.0, 
7.0]]], [[[7.0, 2.0, 5.0], [5.0, 6.0, 8.0], [2.0, 4.0, 3.0]], [[5.0, 8.0, 1.0], [5.0, 7.0, 7.0], [4.0, 1.0, 5.0]], [[5.0, 3.0, 
5.0], [4.0, 8.0, 6.0], [4.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [6.0, 7.0, 2.0], [8.0, 5.0, 2.0]], 
[[2.0, 7.0, 5.0], [5.0, 2.0, 8.0], [4.0, 3.0, 2.0]], [[5.0, 5.0, 4.0], [5.0, 7.0, 3.0], [5.0, 7.0, 4.0]]], [[[5.0, 5.0, 4.0], 
[2.0, 2.0, 5.0], [6.0, 2.0, 3.0]], [[8.0, 4.0, 7.0], [7.0, 5.0, 2.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 4.0], [1.0, 7.0, 1.0], 
[4.0, 1.0, 8.0]]], [[[3.0, 8.0, 1.0], [6.0, 8.0, 3.0], [6.0, 3.0, 5.0]], [[8.0, 2.0, 7.0], [6.0, 6.0, 6.0], [4.0, 8.0, 8.0]], 
[[8.0, 3.0, 4.0], [7.0, 4.0, 5.0], [4.0, 5.0, 5.0]]], [[[4.0, 6.0, 5.0], [2.0, 6.0, 5.0], [4.0, 6.0, 4.0]], [[7.0, 3.0, 8.0], 
[5.0, 2.0, 7.0], [6.0, 2.0, 6.0]], [[1.0, 7.0, 5.0], [4.0, 8.0, 6.0], [3.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 7.0], [5.0, 1.0, 6.0], [8.0, 3.0, 8.0]], [[6.0, 1.0, 5.0], [7.0, 8.0, 7.0], [5.0, 
7.0, 2.0]], [[7.0, 3.0, 7.0], [8.0, 1.0, 7.0], [4.0, 3.0, 8.0]]], [[[1.0, 2.0, 6.0], [7.0, 1.0, 3.0], [6.0, 2.0, 6.0]], [[4.0, 
4.0, 8.0], [5.0, 1.0, 7.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 4.0], [2.0, 2.0, 2.0]]], [[[2.0, 7.0, 3.0], [1.0, 
1.0, 5.0], [4.0, 8.0, 4.0]], [[2.0, 8.0, 6.0], [2.0, 7.0, 2.0], [3.0, 7.0, 8.0]], [[6.0, 5.0, 6.0], [8.0, 7.0, 8.0], [3.0, 4.0, 
1.0]]], [[[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [5.0, 3.0, 2.0]], [[2.0, 4.0, 4.0], [8.0, 2.0, 4.0], [1.0, 4.0, 4.0]], [[1.0, 2.0, 
7.0], [8.0, 7.0, 3.0], [2.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 4.0], [8.0, 4.0, 8.0], [3.0, 1.0, 8.0]], 
[[6.0, 8.0, 6.0], [1.0, 5.0, 8.0], [8.0, 7.0, 6.0]], [[2.0, 2.0, 5.0], [5.0, 7.0, 5.0], [1.0, 1.0, 7.0]]], [[[5.0, 7.0, 6.0], 
[8.0, 3.0, 6.0], [1.0, 8.0, 6.0]], [[7.0, 2.0, 3.0], [8.0, 3.0, 6.0], [4.0, 1.0, 8.0]], [[1.0, 2.0, 3.0], [4.0, 7.0, 3.0], 
[8.0, 7.0, 6.0]]], [[[5.0, 5.0, 4.0], [1.0, 8.0, 7.0], [8.0, 3.0, 5.0]], [[2.0, 5.0, 6.0], [6.0, 7.0, 4.0], [3.0, 6.0, 8.0]], 
[[2.0, 1.0, 3.0], [8.0, 5.0, 5.0], [5.0, 5.0, 5.0]]], [[[8.0, 3.0, 2.0], [6.0, 4.0, 8.0], [2.0, 1.0, 4.0]], [[8.0, 4.0, 4.0], 
[4.0, 6.0, 2.0], [6.0, 8.0, 6.0]], [[3.0, 3.0, 8.0], [6.0, 4.0, 1.0], [4.0, 7.0, 8.0]]]])
      val=Data(numpy.array([[[[3.0, 5.0, 7.0]]], [[[7.0, 7.0, 6.0]]]]),self.functionspace)
      arg[2:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 7.0], [5.0, 1.0, 6.0], [8.0, 3.0, 8.0]], [[6.0, 1.0, 5.0], [7.0, 8.0, 7.0], [5.0, 
7.0, 2.0]], [[7.0, 3.0, 7.0], [8.0, 1.0, 7.0], [4.0, 3.0, 8.0]]], [[[1.0, 2.0, 6.0], [7.0, 1.0, 3.0], [6.0, 2.0, 6.0]], [[4.0, 
4.0, 8.0], [5.0, 1.0, 7.0], [6.0, 8.0, 1.0]], [[3.0, 5.0, 5.0], [6.0, 3.0, 4.0], [2.0, 2.0, 2.0]]], [[[2.0, 7.0, 3.0], [1.0, 
1.0, 5.0], [4.0, 8.0, 4.0]], [[2.0, 8.0, 6.0], [2.0, 7.0, 2.0], [3.0, 7.0, 8.0]], [[6.0, 5.0, 6.0], [3.0, 5.0, 7.0], [3.0, 4.0, 
1.0]]], [[[7.0, 3.0, 5.0], [4.0, 1.0, 7.0], [5.0, 3.0, 2.0]], [[2.0, 4.0, 4.0], [8.0, 2.0, 4.0], [1.0, 4.0, 4.0]], [[1.0, 2.0, 
7.0], [7.0, 7.0, 6.0], [2.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 4.0], [8.0, 4.0, 8.0], [3.0, 1.0, 8.0]], 
[[6.0, 8.0, 6.0], [1.0, 5.0, 8.0], [8.0, 7.0, 6.0]], [[2.0, 2.0, 5.0], [5.0, 7.0, 5.0], [1.0, 1.0, 7.0]]], [[[5.0, 7.0, 6.0], 
[8.0, 3.0, 6.0], [1.0, 8.0, 6.0]], [[7.0, 2.0, 3.0], [8.0, 3.0, 6.0], [4.0, 1.0, 8.0]], [[1.0, 2.0, 3.0], [4.0, 7.0, 3.0], 
[8.0, 7.0, 6.0]]], [[[5.0, 5.0, 4.0], [1.0, 8.0, 7.0], [8.0, 3.0, 5.0]], [[2.0, 5.0, 6.0], [6.0, 7.0, 4.0], [3.0, 6.0, 8.0]], 
[[2.0, 1.0, 3.0], [3.0, 5.0, 7.0], [5.0, 5.0, 5.0]]], [[[8.0, 3.0, 2.0], [6.0, 4.0, 8.0], [2.0, 1.0, 4.0]], [[8.0, 4.0, 4.0], 
[4.0, 6.0, 2.0], [6.0, 8.0, 6.0]], [[3.0, 3.0, 8.0], [7.0, 7.0, 6.0], [4.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 3.0], [3.0, 2.0, 2.0], [7.0, 3.0, 7.0]], [[1.0, 6.0, 6.0], [8.0, 5.0, 2.0], [4.0, 
3.0, 8.0]], [[5.0, 8.0, 7.0], [8.0, 4.0, 1.0], [1.0, 6.0, 8.0]]], [[[1.0, 8.0, 4.0], [2.0, 6.0, 1.0], [2.0, 6.0, 7.0]], [[3.0, 
6.0, 3.0], [2.0, 4.0, 2.0], [6.0, 7.0, 5.0]], [[3.0, 1.0, 7.0], [3.0, 8.0, 2.0], [1.0, 6.0, 3.0]]], [[[6.0, 2.0, 5.0], [6.0, 
8.0, 8.0], [3.0, 4.0, 1.0]], [[8.0, 8.0, 7.0], [1.0, 3.0, 3.0], [6.0, 3.0, 7.0]], [[8.0, 3.0, 6.0], [3.0, 6.0, 8.0], [1.0, 6.0, 
7.0]]], [[[4.0, 8.0, 4.0], [7.0, 3.0, 2.0], [7.0, 6.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 3.0, 5.0], [6.0, 8.0, 1.0]], [[3.0, 3.0, 
4.0], [8.0, 4.0, 4.0], [2.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 8.0], [3.0, 1.0, 6.0], [6.0, 6.0, 2.0]], 
[[5.0, 7.0, 8.0], [6.0, 4.0, 4.0], [1.0, 5.0, 2.0]], [[3.0, 1.0, 7.0], [8.0, 7.0, 4.0], [8.0, 5.0, 8.0]]], [[[8.0, 4.0, 1.0], 
[2.0, 2.0, 1.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 4.0], [3.0, 6.0, 7.0], [5.0, 8.0, 2.0]], [[5.0, 4.0, 2.0], [3.0, 8.0, 1.0], 
[8.0, 4.0, 8.0]]], [[[3.0, 8.0, 4.0], [3.0, 2.0, 7.0], [4.0, 7.0, 5.0]], [[1.0, 3.0, 4.0], [7.0, 2.0, 8.0], [2.0, 4.0, 2.0]], 
[[5.0, 6.0, 8.0], [1.0, 4.0, 7.0], [2.0, 7.0, 8.0]]], [[[1.0, 2.0, 6.0], [1.0, 2.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], 
[4.0, 8.0, 5.0], [7.0, 4.0, 3.0]], [[3.0, 2.0, 6.0], [1.0, 1.0, 3.0], [5.0, 3.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 1.0, 7.0]]], [[[3.0, 3.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 3.0, 2.0]]], 
[[[5.0, 2.0, 4.0]]]])
      arg[2:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 3.0], [3.0, 2.0, 2.0], [7.0, 3.0, 7.0]], [[1.0, 6.0, 6.0], [8.0, 5.0, 2.0], [4.0, 
3.0, 8.0]], [[5.0, 8.0, 7.0], [8.0, 4.0, 1.0], [1.0, 6.0, 8.0]]], [[[1.0, 8.0, 4.0], [2.0, 6.0, 1.0], [2.0, 6.0, 7.0]], [[3.0, 
6.0, 3.0], [2.0, 4.0, 2.0], [6.0, 7.0, 5.0]], [[3.0, 1.0, 7.0], [3.0, 8.0, 2.0], [1.0, 6.0, 3.0]]], [[[6.0, 2.0, 5.0], [6.0, 
8.0, 8.0], [3.0, 4.0, 1.0]], [[8.0, 8.0, 7.0], [1.0, 3.0, 3.0], [6.0, 3.0, 7.0]], [[8.0, 3.0, 6.0], [4.0, 1.0, 7.0], [1.0, 6.0, 
7.0]]], [[[4.0, 8.0, 4.0], [7.0, 3.0, 2.0], [7.0, 6.0, 7.0]], [[5.0, 1.0, 3.0], [2.0, 3.0, 5.0], [6.0, 8.0, 1.0]], [[3.0, 3.0, 
4.0], [3.0, 3.0, 5.0], [2.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 8.0], [3.0, 1.0, 6.0], [6.0, 6.0, 2.0]], 
[[5.0, 7.0, 8.0], [6.0, 4.0, 4.0], [1.0, 5.0, 2.0]], [[3.0, 1.0, 7.0], [8.0, 7.0, 4.0], [8.0, 5.0, 8.0]]], [[[8.0, 4.0, 1.0], 
[2.0, 2.0, 1.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 4.0], [3.0, 6.0, 7.0], [5.0, 8.0, 2.0]], [[5.0, 4.0, 2.0], [3.0, 8.0, 1.0], 
[8.0, 4.0, 8.0]]], [[[3.0, 8.0, 4.0], [3.0, 2.0, 7.0], [4.0, 7.0, 5.0]], [[1.0, 3.0, 4.0], [7.0, 2.0, 8.0], [2.0, 4.0, 2.0]], 
[[5.0, 6.0, 8.0], [3.0, 3.0, 2.0], [2.0, 7.0, 8.0]]], [[[1.0, 2.0, 6.0], [1.0, 2.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], 
[4.0, 8.0, 5.0], [7.0, 4.0, 3.0]], [[3.0, 2.0, 6.0], [5.0, 2.0, 4.0], [5.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 3.0], [8.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[5.0, 5.0, 3.0], [1.0, 6.0, 2.0], [4.0, 
5.0, 7.0]], [[2.0, 7.0, 1.0], [1.0, 2.0, 3.0], [7.0, 8.0, 6.0]]], [[[1.0, 2.0, 8.0], [4.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[3.0, 
3.0, 2.0], [7.0, 3.0, 6.0], [6.0, 6.0, 4.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 2.0], [5.0, 6.0, 2.0]]], [[[6.0, 6.0, 4.0], [2.0, 
8.0, 8.0], [7.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [1.0, 2.0, 2.0], [3.0, 5.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 3.0, 6.0], [5.0, 8.0, 
1.0]]], [[[2.0, 7.0, 8.0], [3.0, 7.0, 3.0], [2.0, 5.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 4.0, 4.0], [3.0, 1.0, 7.0]], [[2.0, 8.0, 
1.0], [3.0, 1.0, 6.0], [1.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 7.0], [4.0, 1.0, 1.0], [6.0, 6.0, 6.0]], 
[[6.0, 7.0, 5.0], [6.0, 3.0, 2.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 6.0], [7.0, 8.0, 8.0], [8.0, 5.0, 7.0]]], [[[2.0, 5.0, 3.0], 
[3.0, 8.0, 6.0], [7.0, 2.0, 5.0]], [[3.0, 5.0, 2.0], [4.0, 1.0, 1.0], [1.0, 5.0, 4.0]], [[1.0, 2.0, 6.0], [2.0, 3.0, 2.0], 
[3.0, 5.0, 2.0]]], [[[3.0, 8.0, 7.0], [3.0, 6.0, 6.0], [6.0, 3.0, 2.0]], [[6.0, 6.0, 5.0], [6.0, 6.0, 7.0], [4.0, 2.0, 3.0]], 
[[2.0, 6.0, 6.0], [2.0, 4.0, 6.0], [2.0, 7.0, 5.0]]], [[[3.0, 3.0, 3.0], [1.0, 5.0, 8.0], [4.0, 3.0, 4.0]], [[3.0, 8.0, 6.0], 
[1.0, 3.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 5.0], [8.0, 7.0, 3.0], [7.0, 8.0, 3.0]]]])
      val=numpy.array([[1.0], [6.0]])
      arg[2:,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 3.0], [8.0, 1.0, 4.0], [7.0, 2.0, 5.0]], [[5.0, 5.0, 3.0], [1.0, 6.0, 2.0], [4.0, 
5.0, 7.0]], [[2.0, 7.0, 1.0], [1.0, 2.0, 3.0], [7.0, 8.0, 6.0]]], [[[1.0, 2.0, 8.0], [4.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[3.0, 
3.0, 2.0], [7.0, 3.0, 6.0], [6.0, 6.0, 4.0]], [[4.0, 2.0, 8.0], [2.0, 3.0, 2.0], [5.0, 6.0, 2.0]]], [[[6.0, 6.0, 4.0], [2.0, 
8.0, 8.0], [7.0, 8.0, 2.0]], [[4.0, 6.0, 3.0], [1.0, 2.0, 1.0], [3.0, 5.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 3.0, 6.0], [5.0, 8.0, 
1.0]]], [[[2.0, 7.0, 8.0], [3.0, 7.0, 3.0], [2.0, 5.0, 3.0]], [[8.0, 6.0, 1.0], [5.0, 4.0, 6.0], [3.0, 1.0, 7.0]], [[2.0, 8.0, 
1.0], [3.0, 1.0, 6.0], [1.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 7.0], [4.0, 1.0, 1.0], [6.0, 6.0, 6.0]], 
[[6.0, 7.0, 5.0], [6.0, 3.0, 2.0], [7.0, 5.0, 4.0]], [[2.0, 7.0, 6.0], [7.0, 8.0, 8.0], [8.0, 5.0, 7.0]]], [[[2.0, 5.0, 3.0], 
[3.0, 8.0, 6.0], [7.0, 2.0, 5.0]], [[3.0, 5.0, 2.0], [4.0, 1.0, 1.0], [1.0, 5.0, 4.0]], [[1.0, 2.0, 6.0], [2.0, 3.0, 2.0], 
[3.0, 5.0, 2.0]]], [[[3.0, 8.0, 7.0], [3.0, 6.0, 6.0], [6.0, 3.0, 2.0]], [[6.0, 6.0, 5.0], [6.0, 6.0, 1.0], [4.0, 2.0, 3.0]], 
[[2.0, 6.0, 6.0], [2.0, 4.0, 6.0], [2.0, 7.0, 5.0]]], [[[3.0, 3.0, 3.0], [1.0, 5.0, 8.0], [4.0, 3.0, 4.0]], [[3.0, 8.0, 6.0], 
[1.0, 3.0, 6.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 5.0], [8.0, 7.0, 3.0], [7.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 5.0], [4.0, 7.0, 3.0], [5.0, 3.0, 6.0]], [[3.0, 7.0, 6.0], [2.0, 2.0, 2.0], [8.0, 
1.0, 8.0]], [[4.0, 4.0, 3.0], [1.0, 1.0, 7.0], [1.0, 8.0, 6.0]]], [[[6.0, 7.0, 1.0], [2.0, 7.0, 2.0], [8.0, 3.0, 1.0]], [[7.0, 
5.0, 6.0], [2.0, 3.0, 3.0], [7.0, 8.0, 4.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 3.0], [3.0, 3.0, 4.0]]], [[[1.0, 1.0, 8.0], [1.0, 
7.0, 8.0], [5.0, 1.0, 4.0]], [[5.0, 3.0, 2.0], [2.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[2.0, 4.0, 1.0], [2.0, 3.0, 6.0], [2.0, 2.0, 
7.0]]], [[[2.0, 1.0, 2.0], [1.0, 4.0, 5.0], [6.0, 4.0, 2.0]], [[3.0, 1.0, 7.0], [3.0, 6.0, 4.0], [3.0, 7.0, 6.0]], [[4.0, 1.0, 
4.0], [8.0, 7.0, 7.0], [3.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 1.0], [5.0, 4.0, 3.0], [8.0, 6.0, 5.0]], 
[[4.0, 3.0, 1.0], [7.0, 8.0, 3.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 4.0], [3.0, 1.0, 8.0], [2.0, 8.0, 8.0]]], [[[8.0, 3.0, 2.0], 
[1.0, 4.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 7.0, 1.0], [4.0, 5.0, 7.0], [1.0, 1.0, 8.0]], [[2.0, 5.0, 2.0], [5.0, 8.0, 4.0], 
[8.0, 8.0, 1.0]]], [[[7.0, 6.0, 8.0], [3.0, 3.0, 5.0], [2.0, 3.0, 4.0]], [[4.0, 3.0, 5.0], [1.0, 5.0, 4.0], [7.0, 2.0, 1.0]], 
[[7.0, 5.0, 6.0], [6.0, 4.0, 3.0], [1.0, 1.0, 1.0]]], [[[6.0, 1.0, 6.0], [1.0, 6.0, 6.0], [8.0, 6.0, 5.0]], [[1.0, 3.0, 4.0], 
[7.0, 3.0, 3.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 8.0], [4.0, 2.0, 7.0]]]])
      val=Data(numpy.array([[4.0], [2.0]]),self.functionspace)
      arg[2:,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 5.0], [4.0, 7.0, 3.0], [5.0, 3.0, 6.0]], [[3.0, 7.0, 6.0], [2.0, 2.0, 2.0], [8.0, 
1.0, 8.0]], [[4.0, 4.0, 3.0], [1.0, 1.0, 7.0], [1.0, 8.0, 6.0]]], [[[6.0, 7.0, 1.0], [2.0, 7.0, 2.0], [8.0, 3.0, 1.0]], [[7.0, 
5.0, 6.0], [2.0, 3.0, 3.0], [7.0, 8.0, 4.0]], [[3.0, 7.0, 6.0], [3.0, 3.0, 3.0], [3.0, 3.0, 4.0]]], [[[1.0, 1.0, 8.0], [1.0, 
7.0, 8.0], [5.0, 1.0, 4.0]], [[5.0, 3.0, 2.0], [2.0, 5.0, 4.0], [2.0, 8.0, 8.0]], [[2.0, 4.0, 1.0], [2.0, 3.0, 6.0], [2.0, 2.0, 
7.0]]], [[[2.0, 1.0, 2.0], [1.0, 4.0, 5.0], [6.0, 4.0, 2.0]], [[3.0, 1.0, 7.0], [3.0, 6.0, 2.0], [3.0, 7.0, 6.0]], [[4.0, 1.0, 
4.0], [8.0, 7.0, 7.0], [3.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 1.0], [5.0, 4.0, 3.0], [8.0, 6.0, 5.0]], 
[[4.0, 3.0, 1.0], [7.0, 8.0, 3.0], [5.0, 6.0, 4.0]], [[3.0, 5.0, 4.0], [3.0, 1.0, 8.0], [2.0, 8.0, 8.0]]], [[[8.0, 3.0, 2.0], 
[1.0, 4.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 7.0, 1.0], [4.0, 5.0, 7.0], [1.0, 1.0, 8.0]], [[2.0, 5.0, 2.0], [5.0, 8.0, 4.0], 
[8.0, 8.0, 1.0]]], [[[7.0, 6.0, 8.0], [3.0, 3.0, 5.0], [2.0, 3.0, 4.0]], [[4.0, 3.0, 5.0], [1.0, 5.0, 4.0], [7.0, 2.0, 1.0]], 
[[7.0, 5.0, 6.0], [6.0, 4.0, 3.0], [1.0, 1.0, 1.0]]], [[[6.0, 1.0, 6.0], [1.0, 6.0, 6.0], [8.0, 6.0, 5.0]], [[1.0, 3.0, 4.0], 
[7.0, 3.0, 2.0], [5.0, 3.0, 2.0]], [[2.0, 2.0, 5.0], [4.0, 3.0, 8.0], [4.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPPL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 2.0], [4.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[7.0, 7.0, 4.0], [6.0, 3.0, 6.0], [5.0, 
7.0, 4.0]], [[5.0, 2.0, 2.0], [3.0, 2.0, 1.0], [7.0, 2.0, 3.0]]], [[[5.0, 3.0, 7.0], [7.0, 1.0, 1.0], [6.0, 6.0, 1.0]], [[2.0, 
4.0, 1.0], [3.0, 3.0, 7.0], [6.0, 4.0, 2.0]], [[6.0, 7.0, 5.0], [3.0, 4.0, 1.0], [2.0, 7.0, 3.0]]], [[[8.0, 5.0, 1.0], [7.0, 
7.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [4.0, 8.0, 2.0], [2.0, 5.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 6.0, 3.0], [2.0, 3.0, 
1.0]]], [[[6.0, 8.0, 5.0], [1.0, 5.0, 3.0], [7.0, 3.0, 4.0]], [[3.0, 2.0, 3.0], [2.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[2.0, 4.0, 
3.0], [3.0, 8.0, 5.0], [8.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 5.0], [7.0, 7.0, 8.0], [4.0, 5.0, 1.0]], 
[[2.0, 3.0, 7.0], [5.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[2.0, 5.0, 6.0], [8.0, 5.0, 1.0], [3.0, 4.0, 1.0]]], [[[8.0, 7.0, 7.0], 
[2.0, 8.0, 4.0], [7.0, 3.0, 7.0]], [[7.0, 8.0, 8.0], [4.0, 2.0, 3.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 8.0], [5.0, 4.0, 2.0], 
[8.0, 2.0, 2.0]]], [[[1.0, 6.0, 2.0], [4.0, 3.0, 1.0], [7.0, 4.0, 2.0]], [[5.0, 4.0, 6.0], [7.0, 5.0, 6.0], [2.0, 8.0, 8.0]], 
[[3.0, 6.0, 1.0], [8.0, 1.0, 8.0], [5.0, 3.0, 2.0]]], [[[4.0, 5.0, 4.0], [2.0, 1.0, 4.0], [1.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], 
[7.0, 2.0, 7.0], [8.0, 3.0, 1.0]], [[4.0, 4.0, 8.0], [5.0, 4.0, 7.0], [3.0, 6.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0], [8.0]])+(1.-msk_val)*numpy.array([[1.0], [3.0]])
      arg[2:,1,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 2.0], [4.0, 8.0, 1.0], [4.0, 5.0, 1.0]], [[7.0, 7.0, 4.0], [6.0, 3.0, 6.0], [5.0, 
7.0, 4.0]], [[5.0, 2.0, 2.0], [3.0, 2.0, 1.0], [7.0, 2.0, 3.0]]], [[[5.0, 3.0, 7.0], [7.0, 1.0, 1.0], [6.0, 6.0, 1.0]], [[2.0, 
4.0, 1.0], [3.0, 3.0, 7.0], [6.0, 4.0, 2.0]], [[6.0, 7.0, 5.0], [3.0, 4.0, 1.0], [2.0, 7.0, 3.0]]], [[[8.0, 5.0, 1.0], [7.0, 
7.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 4.0, 1.0], [4.0, 8.0, 2.0], [2.0, 5.0, 2.0]], [[7.0, 2.0, 8.0], [1.0, 6.0, 3.0], [2.0, 3.0, 
1.0]]], [[[6.0, 8.0, 5.0], [1.0, 5.0, 3.0], [7.0, 3.0, 4.0]], [[3.0, 2.0, 3.0], [2.0, 6.0, 8.0], [4.0, 2.0, 3.0]], [[2.0, 4.0, 
3.0], [3.0, 8.0, 5.0], [8.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 5.0], [7.0, 7.0, 8.0], [4.0, 5.0, 1.0]], 
[[2.0, 3.0, 7.0], [5.0, 6.0, 1.0], [5.0, 1.0, 8.0]], [[2.0, 5.0, 6.0], [8.0, 5.0, 1.0], [3.0, 4.0, 1.0]]], [[[8.0, 7.0, 7.0], 
[2.0, 8.0, 4.0], [7.0, 3.0, 7.0]], [[7.0, 8.0, 8.0], [4.0, 2.0, 3.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 8.0], [5.0, 4.0, 2.0], 
[8.0, 2.0, 2.0]]], [[[1.0, 6.0, 2.0], [4.0, 3.0, 1.0], [7.0, 4.0, 2.0]], [[5.0, 4.0, 6.0], [7.0, 5.0, 1.0], [2.0, 8.0, 8.0]], 
[[3.0, 6.0, 1.0], [8.0, 1.0, 8.0], [5.0, 3.0, 2.0]]], [[[4.0, 5.0, 4.0], [2.0, 1.0, 4.0], [1.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], 
[7.0, 2.0, 3.0], [8.0, 3.0, 1.0]], [[4.0, 4.0, 8.0], [5.0, 4.0, 7.0], [3.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 8.0], [3.0, 2.0, 5.0], [2.0, 6.0, 8.0]], [[8.0, 7.0, 4.0], [3.0, 1.0, 4.0], [7.0, 
2.0, 2.0]], [[6.0, 7.0, 4.0], [4.0, 3.0, 1.0], [2.0, 7.0, 2.0]]], [[[3.0, 2.0, 2.0], [8.0, 7.0, 2.0], [3.0, 3.0, 5.0]], [[8.0, 
6.0, 6.0], [8.0, 4.0, 8.0], [2.0, 2.0, 5.0]], [[3.0, 3.0, 3.0], [5.0, 3.0, 8.0], [5.0, 1.0, 6.0]]], [[[3.0, 8.0, 2.0], [1.0, 
3.0, 1.0], [4.0, 4.0, 5.0]], [[1.0, 2.0, 5.0], [6.0, 3.0, 2.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 3.0], [8.0, 2.0, 2.0], [8.0, 2.0, 
5.0]]], [[[6.0, 2.0, 8.0], [6.0, 6.0, 1.0], [5.0, 6.0, 4.0]], [[3.0, 6.0, 4.0], [1.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[1.0, 5.0, 
6.0], [7.0, 4.0, 7.0], [5.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 4.0], [4.0, 5.0, 5.0], [1.0, 3.0, 6.0]], 
[[4.0, 5.0, 7.0], [3.0, 7.0, 4.0], [8.0, 1.0, 8.0]], [[7.0, 1.0, 5.0], [1.0, 1.0, 2.0], [1.0, 2.0, 3.0]]], [[[5.0, 6.0, 4.0], 
[3.0, 1.0, 5.0], [5.0, 5.0, 3.0]], [[4.0, 2.0, 1.0], [4.0, 2.0, 7.0], [2.0, 1.0, 2.0]], [[2.0, 1.0, 6.0], [7.0, 1.0, 8.0], 
[1.0, 2.0, 3.0]]], [[[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [8.0, 2.0, 5.0]], [[2.0, 7.0, 2.0], [7.0, 4.0, 2.0], [1.0, 3.0, 8.0]], 
[[1.0, 3.0, 4.0], [5.0, 5.0, 7.0], [1.0, 2.0, 5.0]]], [[[8.0, 6.0, 3.0], [1.0, 8.0, 8.0], [7.0, 7.0, 5.0]], [[5.0, 2.0, 7.0], 
[8.0, 8.0, 3.0], [3.0, 8.0, 7.0]], [[3.0, 7.0, 1.0], [4.0, 3.0, 5.0], [4.0, 3.0, 5.0]]]])
      val=numpy.array([[[6.0]], [[6.0]]])
      arg[1:3,1,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 8.0], [3.0, 2.0, 5.0], [2.0, 6.0, 8.0]], [[8.0, 7.0, 4.0], [3.0, 1.0, 4.0], [7.0, 
2.0, 2.0]], [[6.0, 7.0, 4.0], [4.0, 3.0, 1.0], [2.0, 7.0, 2.0]]], [[[3.0, 2.0, 2.0], [8.0, 7.0, 2.0], [3.0, 3.0, 5.0]], [[8.0, 
6.0, 6.0], [8.0, 6.0, 8.0], [2.0, 2.0, 5.0]], [[3.0, 3.0, 3.0], [5.0, 3.0, 8.0], [5.0, 1.0, 6.0]]], [[[3.0, 8.0, 2.0], [1.0, 
3.0, 1.0], [4.0, 4.0, 5.0]], [[1.0, 2.0, 5.0], [6.0, 6.0, 2.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 3.0], [8.0, 2.0, 2.0], [8.0, 2.0, 
5.0]]], [[[6.0, 2.0, 8.0], [6.0, 6.0, 1.0], [5.0, 6.0, 4.0]], [[3.0, 6.0, 4.0], [1.0, 1.0, 1.0], [4.0, 8.0, 4.0]], [[1.0, 5.0, 
6.0], [7.0, 4.0, 7.0], [5.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 4.0], [4.0, 5.0, 5.0], [1.0, 3.0, 6.0]], 
[[4.0, 5.0, 7.0], [3.0, 7.0, 4.0], [8.0, 1.0, 8.0]], [[7.0, 1.0, 5.0], [1.0, 1.0, 2.0], [1.0, 2.0, 3.0]]], [[[5.0, 6.0, 4.0], 
[3.0, 1.0, 5.0], [5.0, 5.0, 3.0]], [[4.0, 2.0, 1.0], [4.0, 6.0, 7.0], [2.0, 1.0, 2.0]], [[2.0, 1.0, 6.0], [7.0, 1.0, 8.0], 
[1.0, 2.0, 3.0]]], [[[1.0, 6.0, 5.0], [2.0, 2.0, 1.0], [8.0, 2.0, 5.0]], [[2.0, 7.0, 2.0], [7.0, 6.0, 2.0], [1.0, 3.0, 8.0]], 
[[1.0, 3.0, 4.0], [5.0, 5.0, 7.0], [1.0, 2.0, 5.0]]], [[[8.0, 6.0, 3.0], [1.0, 8.0, 8.0], [7.0, 7.0, 5.0]], [[5.0, 2.0, 7.0], 
[8.0, 8.0, 3.0], [3.0, 8.0, 7.0]], [[3.0, 7.0, 1.0], [4.0, 3.0, 5.0], [4.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 5.0], [4.0, 3.0, 8.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 2.0], [1.0, 1.0, 2.0], [7.0, 
5.0, 4.0]], [[4.0, 5.0, 7.0], [7.0, 6.0, 8.0], [6.0, 1.0, 2.0]]], [[[8.0, 5.0, 7.0], [4.0, 6.0, 1.0], [4.0, 6.0, 1.0]], [[3.0, 
8.0, 3.0], [8.0, 5.0, 6.0], [7.0, 7.0, 5.0]], [[7.0, 3.0, 5.0], [7.0, 5.0, 1.0], [1.0, 5.0, 8.0]]], [[[3.0, 7.0, 4.0], [6.0, 
4.0, 6.0], [8.0, 5.0, 4.0]], [[5.0, 5.0, 5.0], [7.0, 7.0, 2.0], [2.0, 6.0, 6.0]], [[4.0, 5.0, 7.0], [6.0, 6.0, 2.0], [4.0, 5.0, 
7.0]]], [[[3.0, 5.0, 3.0], [1.0, 1.0, 5.0], [5.0, 3.0, 4.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 1.0], [4.0, 5.0, 8.0]], [[6.0, 1.0, 
8.0], [4.0, 3.0, 2.0], [6.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 4.0, 6.0], [8.0, 7.0, 7.0]], 
[[3.0, 8.0, 1.0], [1.0, 3.0, 2.0], [5.0, 4.0, 1.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 1.0], [7.0, 1.0, 7.0]]], [[[5.0, 3.0, 6.0], 
[2.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[4.0, 4.0, 8.0], [6.0, 4.0, 3.0], [5.0, 2.0, 1.0]], [[4.0, 7.0, 4.0], [1.0, 8.0, 7.0], 
[8.0, 1.0, 6.0]]], [[[5.0, 4.0, 1.0], [7.0, 1.0, 6.0], [3.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [8.0, 6.0, 2.0], [5.0, 8.0, 1.0]], 
[[2.0, 7.0, 4.0], [2.0, 5.0, 5.0], [6.0, 7.0, 3.0]]], [[[3.0, 8.0, 7.0], [7.0, 7.0, 5.0], [3.0, 4.0, 4.0]], [[8.0, 6.0, 7.0], 
[8.0, 8.0, 4.0], [5.0, 2.0, 5.0]], [[7.0, 8.0, 3.0], [4.0, 8.0, 4.0], [6.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[7.0]], [[3.0]]]),self.functionspace)
      arg[1:3,1,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 5.0], [4.0, 3.0, 8.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 2.0], [1.0, 1.0, 2.0], [7.0, 
5.0, 4.0]], [[4.0, 5.0, 7.0], [7.0, 6.0, 8.0], [6.0, 1.0, 2.0]]], [[[8.0, 5.0, 7.0], [4.0, 6.0, 1.0], [4.0, 6.0, 1.0]], [[3.0, 
8.0, 3.0], [8.0, 7.0, 6.0], [7.0, 7.0, 5.0]], [[7.0, 3.0, 5.0], [7.0, 5.0, 1.0], [1.0, 5.0, 8.0]]], [[[3.0, 7.0, 4.0], [6.0, 
4.0, 6.0], [8.0, 5.0, 4.0]], [[5.0, 5.0, 5.0], [7.0, 3.0, 2.0], [2.0, 6.0, 6.0]], [[4.0, 5.0, 7.0], [6.0, 6.0, 2.0], [4.0, 5.0, 
7.0]]], [[[3.0, 5.0, 3.0], [1.0, 1.0, 5.0], [5.0, 3.0, 4.0]], [[3.0, 6.0, 7.0], [7.0, 6.0, 1.0], [4.0, 5.0, 8.0]], [[6.0, 1.0, 
8.0], [4.0, 3.0, 2.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 4.0, 6.0], [8.0, 7.0, 7.0]], 
[[3.0, 8.0, 1.0], [1.0, 3.0, 2.0], [5.0, 4.0, 1.0]], [[3.0, 2.0, 1.0], [1.0, 7.0, 1.0], [7.0, 1.0, 7.0]]], [[[5.0, 3.0, 6.0], 
[2.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[4.0, 4.0, 8.0], [6.0, 7.0, 3.0], [5.0, 2.0, 1.0]], [[4.0, 7.0, 4.0], [1.0, 8.0, 7.0], 
[8.0, 1.0, 6.0]]], [[[5.0, 4.0, 1.0], [7.0, 1.0, 6.0], [3.0, 5.0, 4.0]], [[8.0, 2.0, 5.0], [8.0, 3.0, 2.0], [5.0, 8.0, 1.0]], 
[[2.0, 7.0, 4.0], [2.0, 5.0, 5.0], [6.0, 7.0, 3.0]]], [[[3.0, 8.0, 7.0], [7.0, 7.0, 5.0], [3.0, 4.0, 4.0]], [[8.0, 6.0, 7.0], 
[8.0, 8.0, 4.0], [5.0, 2.0, 5.0]], [[7.0, 8.0, 3.0], [4.0, 8.0, 4.0], [6.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 8.0], [2.0, 8.0, 5.0], [4.0, 7.0, 7.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 7.0], [8.0, 
8.0, 6.0]], [[6.0, 6.0, 5.0], [3.0, 8.0, 6.0], [2.0, 6.0, 4.0]]], [[[1.0, 7.0, 8.0], [7.0, 8.0, 2.0], [6.0, 2.0, 3.0]], [[5.0, 
1.0, 6.0], [6.0, 6.0, 3.0], [5.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [3.0, 7.0, 7.0], [6.0, 1.0, 7.0]]], [[[5.0, 3.0, 3.0], [7.0, 
5.0, 8.0], [2.0, 6.0, 6.0]], [[5.0, 7.0, 7.0], [5.0, 5.0, 4.0], [4.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [2.0, 7.0, 5.0], [3.0, 3.0, 
1.0]]], [[[5.0, 5.0, 2.0], [3.0, 5.0, 5.0], [7.0, 8.0, 5.0]], [[7.0, 1.0, 4.0], [4.0, 7.0, 1.0], [1.0, 2.0, 8.0]], [[1.0, 6.0, 
8.0], [4.0, 3.0, 1.0], [7.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 1.0], [8.0, 1.0, 4.0], [3.0, 8.0, 6.0]], 
[[3.0, 6.0, 3.0], [8.0, 7.0, 2.0], [3.0, 8.0, 5.0]], [[8.0, 7.0, 6.0], [1.0, 7.0, 2.0], [3.0, 8.0, 6.0]]], [[[3.0, 2.0, 3.0], 
[4.0, 8.0, 7.0], [6.0, 7.0, 8.0]], [[1.0, 4.0, 3.0], [2.0, 1.0, 6.0], [3.0, 6.0, 8.0]], [[7.0, 3.0, 5.0], [6.0, 4.0, 8.0], 
[1.0, 2.0, 4.0]]], [[[2.0, 5.0, 5.0], [7.0, 2.0, 4.0], [5.0, 5.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 2.0, 8.0], [1.0, 5.0, 6.0]], 
[[5.0, 5.0, 8.0], [7.0, 1.0, 3.0], [3.0, 4.0, 7.0]]], [[[2.0, 7.0, 6.0], [2.0, 2.0, 5.0], [7.0, 7.0, 2.0]], [[2.0, 2.0, 3.0], 
[4.0, 6.0, 2.0], [3.0, 6.0, 6.0]], [[2.0, 3.0, 5.0], [5.0, 8.0, 4.0], [5.0, 3.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0]], [[7.0]]])+(1.-msk_val)*numpy.array([[[3.0]], [[1.0]]])
      arg[1:3,1,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 8.0], [2.0, 8.0, 5.0], [4.0, 7.0, 7.0]], [[2.0, 5.0, 6.0], [7.0, 2.0, 7.0], [8.0, 
8.0, 6.0]], [[6.0, 6.0, 5.0], [3.0, 8.0, 6.0], [2.0, 6.0, 4.0]]], [[[1.0, 7.0, 8.0], [7.0, 8.0, 2.0], [6.0, 2.0, 3.0]], [[5.0, 
1.0, 6.0], [6.0, 4.0, 3.0], [5.0, 5.0, 7.0]], [[6.0, 1.0, 5.0], [3.0, 7.0, 7.0], [6.0, 1.0, 7.0]]], [[[5.0, 3.0, 3.0], [7.0, 
5.0, 8.0], [2.0, 6.0, 6.0]], [[5.0, 7.0, 7.0], [5.0, 7.0, 4.0], [4.0, 2.0, 5.0]], [[4.0, 6.0, 1.0], [2.0, 7.0, 5.0], [3.0, 3.0, 
1.0]]], [[[5.0, 5.0, 2.0], [3.0, 5.0, 5.0], [7.0, 8.0, 5.0]], [[7.0, 1.0, 4.0], [4.0, 7.0, 1.0], [1.0, 2.0, 8.0]], [[1.0, 6.0, 
8.0], [4.0, 3.0, 1.0], [7.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 1.0], [8.0, 1.0, 4.0], [3.0, 8.0, 6.0]], 
[[3.0, 6.0, 3.0], [8.0, 7.0, 2.0], [3.0, 8.0, 5.0]], [[8.0, 7.0, 6.0], [1.0, 7.0, 2.0], [3.0, 8.0, 6.0]]], [[[3.0, 2.0, 3.0], 
[4.0, 8.0, 7.0], [6.0, 7.0, 8.0]], [[1.0, 4.0, 3.0], [2.0, 3.0, 6.0], [3.0, 6.0, 8.0]], [[7.0, 3.0, 5.0], [6.0, 4.0, 8.0], 
[1.0, 2.0, 4.0]]], [[[2.0, 5.0, 5.0], [7.0, 2.0, 4.0], [5.0, 5.0, 8.0]], [[7.0, 2.0, 2.0], [8.0, 1.0, 8.0], [1.0, 5.0, 6.0]], 
[[5.0, 5.0, 8.0], [7.0, 1.0, 3.0], [3.0, 4.0, 7.0]]], [[[2.0, 7.0, 6.0], [2.0, 2.0, 5.0], [7.0, 7.0, 2.0]], [[2.0, 2.0, 3.0], 
[4.0, 6.0, 2.0], [3.0, 6.0, 6.0]], [[2.0, 3.0, 5.0], [5.0, 8.0, 4.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 7.0], [8.0, 2.0, 1.0], [1.0, 5.0, 3.0]], [[7.0, 2.0, 2.0], [7.0, 5.0, 6.0], [1.0, 
3.0, 2.0]], [[2.0, 7.0, 6.0], [4.0, 3.0, 6.0], [1.0, 7.0, 4.0]]], [[[5.0, 7.0, 3.0], [6.0, 5.0, 6.0], [1.0, 2.0, 5.0]], [[1.0, 
2.0, 8.0], [3.0, 8.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [6.0, 3.0, 4.0], [1.0, 1.0, 7.0]]], [[[2.0, 7.0, 1.0], [8.0, 
3.0, 8.0], [5.0, 2.0, 2.0]], [[1.0, 2.0, 4.0], [3.0, 6.0, 2.0], [5.0, 1.0, 8.0]], [[2.0, 2.0, 6.0], [8.0, 4.0, 6.0], [2.0, 8.0, 
3.0]]], [[[1.0, 6.0, 7.0], [4.0, 6.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 3.0, 3.0], [8.0, 4.0, 2.0], [4.0, 2.0, 7.0]], [[1.0, 2.0, 
2.0], [7.0, 2.0, 4.0], [4.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 4.0, 4.0], [1.0, 5.0, 3.0]], 
[[2.0, 3.0, 8.0], [1.0, 1.0, 7.0], [6.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [6.0, 8.0, 6.0], [5.0, 6.0, 5.0]]], [[[1.0, 8.0, 6.0], 
[4.0, 3.0, 2.0], [8.0, 6.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 5.0, 2.0], [2.0, 2.0, 5.0]], [[8.0, 2.0, 1.0], [1.0, 8.0, 4.0], 
[6.0, 8.0, 8.0]]], [[[8.0, 8.0, 5.0], [7.0, 1.0, 6.0], [7.0, 5.0, 8.0]], [[6.0, 3.0, 6.0], [5.0, 8.0, 7.0], [7.0, 2.0, 8.0]], 
[[8.0, 4.0, 3.0], [2.0, 1.0, 7.0], [4.0, 5.0, 8.0]]], [[[4.0, 6.0, 4.0], [5.0, 6.0, 4.0], [3.0, 1.0, 8.0]], [[6.0, 1.0, 8.0], 
[3.0, 1.0, 1.0], [3.0, 3.0, 4.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 1.0], [4.0, 1.0, 8.0]]]])
      val=numpy.array([[[[7.0, 6.0], [2.0, 7.0]], [[6.0, 2.0], [8.0, 6.0]]], [[[5.0, 1.0], [2.0, 8.0]], [[2.0, 3.0], [2.0, 
5.0]]], [[[1.0, 3.0], [7.0, 5.0]], [[8.0, 2.0], [5.0, 2.0]]]])
      arg[:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 7.0], [2.0, 7.0, 1.0], [1.0, 5.0, 3.0]], [[6.0, 2.0, 2.0], [8.0, 6.0, 6.0], [1.0, 
3.0, 2.0]], [[2.0, 7.0, 6.0], [4.0, 3.0, 6.0], [1.0, 7.0, 4.0]]], [[[5.0, 1.0, 3.0], [2.0, 8.0, 6.0], [1.0, 2.0, 5.0]], [[2.0, 
3.0, 8.0], [2.0, 5.0, 1.0], [8.0, 5.0, 5.0]], [[1.0, 4.0, 1.0], [6.0, 3.0, 4.0], [1.0, 1.0, 7.0]]], [[[1.0, 3.0, 1.0], [7.0, 
5.0, 8.0], [5.0, 2.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 2.0, 2.0], [5.0, 1.0, 8.0]], [[2.0, 2.0, 6.0], [8.0, 4.0, 6.0], [2.0, 8.0, 
3.0]]], [[[1.0, 6.0, 7.0], [4.0, 6.0, 1.0], [6.0, 4.0, 2.0]], [[4.0, 3.0, 3.0], [8.0, 4.0, 2.0], [4.0, 2.0, 7.0]], [[1.0, 2.0, 
2.0], [7.0, 2.0, 4.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 5.0], [2.0, 7.0, 4.0], [1.0, 5.0, 3.0]], 
[[6.0, 2.0, 8.0], [8.0, 6.0, 7.0], [6.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [6.0, 8.0, 6.0], [5.0, 6.0, 5.0]]], [[[5.0, 1.0, 6.0], 
[2.0, 8.0, 2.0], [8.0, 6.0, 8.0]], [[2.0, 3.0, 7.0], [2.0, 5.0, 2.0], [2.0, 2.0, 5.0]], [[8.0, 2.0, 1.0], [1.0, 8.0, 4.0], 
[6.0, 8.0, 8.0]]], [[[1.0, 3.0, 5.0], [7.0, 5.0, 6.0], [7.0, 5.0, 8.0]], [[8.0, 2.0, 6.0], [5.0, 2.0, 7.0], [7.0, 2.0, 8.0]], 
[[8.0, 4.0, 3.0], [2.0, 1.0, 7.0], [4.0, 5.0, 8.0]]], [[[4.0, 6.0, 4.0], [5.0, 6.0, 4.0], [3.0, 1.0, 8.0]], [[6.0, 1.0, 8.0], 
[3.0, 1.0, 1.0], [3.0, 3.0, 4.0]], [[6.0, 4.0, 4.0], [2.0, 5.0, 1.0], [4.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 8.0], [8.0, 8.0, 8.0], [4.0, 7.0, 1.0]], [[4.0, 6.0, 5.0], [1.0, 8.0, 7.0], [8.0, 
8.0, 1.0]], [[3.0, 5.0, 3.0], [1.0, 7.0, 4.0], [6.0, 2.0, 8.0]]], [[[6.0, 8.0, 6.0], [8.0, 7.0, 2.0], [6.0, 4.0, 3.0]], [[7.0, 
2.0, 3.0], [6.0, 8.0, 2.0], [1.0, 6.0, 7.0]], [[5.0, 2.0, 2.0], [8.0, 3.0, 6.0], [3.0, 2.0, 3.0]]], [[[4.0, 2.0, 2.0], [8.0, 
1.0, 7.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 8.0], [1.0, 7.0, 6.0], [4.0, 4.0, 3.0]], [[8.0, 5.0, 5.0], [3.0, 2.0, 7.0], [1.0, 7.0, 
5.0]]], [[[4.0, 8.0, 5.0], [4.0, 1.0, 5.0], [4.0, 6.0, 3.0]], [[7.0, 2.0, 7.0], [8.0, 1.0, 8.0], [8.0, 4.0, 6.0]], [[2.0, 1.0, 
6.0], [5.0, 3.0, 8.0], [6.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 3.0], [8.0, 7.0, 3.0], [7.0, 8.0, 4.0]], 
[[5.0, 8.0, 3.0], [4.0, 5.0, 3.0], [2.0, 6.0, 7.0]], [[6.0, 7.0, 3.0], [5.0, 7.0, 1.0], [8.0, 4.0, 6.0]]], [[[6.0, 5.0, 1.0], 
[5.0, 7.0, 2.0], [6.0, 1.0, 4.0]], [[4.0, 2.0, 5.0], [5.0, 2.0, 3.0], [2.0, 8.0, 4.0]], [[3.0, 8.0, 2.0], [6.0, 1.0, 7.0], 
[1.0, 2.0, 1.0]]], [[[5.0, 4.0, 2.0], [2.0, 2.0, 8.0], [7.0, 3.0, 4.0]], [[8.0, 7.0, 6.0], [4.0, 3.0, 7.0], [5.0, 5.0, 3.0]], 
[[2.0, 3.0, 2.0], [1.0, 1.0, 2.0], [1.0, 6.0, 5.0]]], [[[6.0, 8.0, 4.0], [3.0, 4.0, 5.0], [2.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], 
[6.0, 4.0, 5.0], [2.0, 3.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 2.0, 7.0], [4.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0, 5.0], [4.0, 1.0]], [[1.0, 8.0], [7.0, 2.0]]], [[[6.0, 5.0], [8.0, 3.0]], [[6.0, 8.0], [3.0, 
6.0]]], [[[1.0, 6.0], [1.0, 3.0]], [[7.0, 3.0], [5.0, 5.0]]]]),self.functionspace)
      arg[:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 8.0], [4.0, 1.0, 8.0], [4.0, 7.0, 1.0]], [[1.0, 8.0, 5.0], [7.0, 2.0, 7.0], [8.0, 
8.0, 1.0]], [[3.0, 5.0, 3.0], [1.0, 7.0, 4.0], [6.0, 2.0, 8.0]]], [[[6.0, 5.0, 6.0], [8.0, 3.0, 2.0], [6.0, 4.0, 3.0]], [[6.0, 
8.0, 3.0], [3.0, 6.0, 2.0], [1.0, 6.0, 7.0]], [[5.0, 2.0, 2.0], [8.0, 3.0, 6.0], [3.0, 2.0, 3.0]]], [[[1.0, 6.0, 2.0], [1.0, 
3.0, 7.0], [2.0, 5.0, 1.0]], [[7.0, 3.0, 8.0], [5.0, 5.0, 6.0], [4.0, 4.0, 3.0]], [[8.0, 5.0, 5.0], [3.0, 2.0, 7.0], [1.0, 7.0, 
5.0]]], [[[4.0, 8.0, 5.0], [4.0, 1.0, 5.0], [4.0, 6.0, 3.0]], [[7.0, 2.0, 7.0], [8.0, 1.0, 8.0], [8.0, 4.0, 6.0]], [[2.0, 1.0, 
6.0], [5.0, 3.0, 8.0], [6.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [4.0, 1.0, 3.0], [7.0, 8.0, 4.0]], 
[[1.0, 8.0, 3.0], [7.0, 2.0, 3.0], [2.0, 6.0, 7.0]], [[6.0, 7.0, 3.0], [5.0, 7.0, 1.0], [8.0, 4.0, 6.0]]], [[[6.0, 5.0, 1.0], 
[8.0, 3.0, 2.0], [6.0, 1.0, 4.0]], [[6.0, 8.0, 5.0], [3.0, 6.0, 3.0], [2.0, 8.0, 4.0]], [[3.0, 8.0, 2.0], [6.0, 1.0, 7.0], 
[1.0, 2.0, 1.0]]], [[[1.0, 6.0, 2.0], [1.0, 3.0, 8.0], [7.0, 3.0, 4.0]], [[7.0, 3.0, 6.0], [5.0, 5.0, 7.0], [5.0, 5.0, 3.0]], 
[[2.0, 3.0, 2.0], [1.0, 1.0, 2.0], [1.0, 6.0, 5.0]]], [[[6.0, 8.0, 4.0], [3.0, 4.0, 5.0], [2.0, 2.0, 3.0]], [[8.0, 7.0, 6.0], 
[6.0, 4.0, 5.0], [2.0, 3.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 2.0, 7.0], [4.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 3.0], [8.0, 3.0, 6.0], [7.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [7.0, 2.0, 4.0], [2.0, 
8.0, 5.0]], [[6.0, 1.0, 5.0], [6.0, 5.0, 6.0], [3.0, 5.0, 3.0]]], [[[8.0, 5.0, 3.0], [7.0, 2.0, 3.0], [6.0, 7.0, 8.0]], [[6.0, 
7.0, 4.0], [2.0, 8.0, 2.0], [5.0, 1.0, 7.0]], [[6.0, 7.0, 6.0], [5.0, 2.0, 3.0], [5.0, 6.0, 8.0]]], [[[5.0, 5.0, 8.0], [2.0, 
8.0, 5.0], [3.0, 8.0, 8.0]], [[8.0, 6.0, 1.0], [7.0, 4.0, 3.0], [3.0, 8.0, 4.0]], [[5.0, 4.0, 1.0], [6.0, 8.0, 8.0], [6.0, 6.0, 
6.0]]], [[[1.0, 8.0, 2.0], [8.0, 8.0, 3.0], [7.0, 7.0, 5.0]], [[7.0, 4.0, 8.0], [2.0, 8.0, 4.0], [2.0, 2.0, 2.0]], [[8.0, 1.0, 
5.0], [2.0, 1.0, 1.0], [6.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 3.0], [5.0, 4.0, 3.0], [3.0, 4.0, 2.0]], 
[[8.0, 6.0, 4.0], [1.0, 8.0, 2.0], [2.0, 3.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 2.0], [3.0, 3.0, 5.0]]], [[[7.0, 4.0, 5.0], 
[2.0, 7.0, 4.0], [1.0, 6.0, 5.0]], [[4.0, 7.0, 6.0], [3.0, 8.0, 8.0], [7.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], [6.0, 3.0, 2.0], 
[5.0, 1.0, 4.0]]], [[[3.0, 8.0, 4.0], [1.0, 1.0, 6.0], [7.0, 6.0, 4.0]], [[6.0, 3.0, 1.0], [6.0, 7.0, 8.0], [5.0, 4.0, 8.0]], 
[[1.0, 7.0, 6.0], [7.0, 7.0, 1.0], [2.0, 2.0, 1.0]]], [[[8.0, 5.0, 1.0], [5.0, 3.0, 1.0], [1.0, 5.0, 3.0]], [[1.0, 8.0, 7.0], 
[6.0, 5.0, 5.0], [7.0, 6.0, 2.0]], [[4.0, 8.0, 1.0], [3.0, 3.0, 1.0], [7.0, 4.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 8.0], [7.0, 6.0]], [[3.0, 1.0], [8.0, 8.0]]], [[[5.0, 3.0], [6.0, 4.0]], [[8.0, 1.0], 
[1.0, 1.0]]], [[[5.0, 2.0], [3.0, 8.0]], [[8.0, 4.0], [4.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 6.0], [4.0, 1.0]], 
[[2.0, 6.0], [4.0, 7.0]]], [[[8.0, 4.0], [5.0, 3.0]], [[6.0, 6.0], [1.0, 4.0]]], [[[8.0, 3.0], [7.0, 2.0]], [[8.0, 4.0], [6.0, 
6.0]]]])
      arg[:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 3.0], [7.0, 6.0, 6.0], [7.0, 2.0, 1.0]], [[3.0, 1.0, 1.0], [8.0, 8.0, 4.0], [2.0, 
8.0, 5.0]], [[6.0, 1.0, 5.0], [6.0, 5.0, 6.0], [3.0, 5.0, 3.0]]], [[[5.0, 3.0, 3.0], [6.0, 4.0, 3.0], [6.0, 7.0, 8.0]], [[8.0, 
1.0, 4.0], [1.0, 1.0, 2.0], [5.0, 1.0, 7.0]], [[6.0, 7.0, 6.0], [5.0, 2.0, 3.0], [5.0, 6.0, 8.0]]], [[[5.0, 2.0, 8.0], [3.0, 
8.0, 5.0], [3.0, 8.0, 8.0]], [[8.0, 4.0, 1.0], [4.0, 8.0, 3.0], [3.0, 8.0, 4.0]], [[5.0, 4.0, 1.0], [6.0, 8.0, 8.0], [6.0, 6.0, 
6.0]]], [[[1.0, 8.0, 2.0], [8.0, 8.0, 3.0], [7.0, 7.0, 5.0]], [[7.0, 4.0, 8.0], [2.0, 8.0, 4.0], [2.0, 2.0, 2.0]], [[8.0, 1.0, 
5.0], [2.0, 1.0, 1.0], [6.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 3.0], [4.0, 1.0, 3.0], [3.0, 4.0, 2.0]], 
[[2.0, 6.0, 4.0], [4.0, 7.0, 2.0], [2.0, 3.0, 2.0]], [[1.0, 6.0, 2.0], [5.0, 4.0, 2.0], [3.0, 3.0, 5.0]]], [[[8.0, 4.0, 5.0], 
[5.0, 3.0, 4.0], [1.0, 6.0, 5.0]], [[6.0, 6.0, 6.0], [1.0, 4.0, 8.0], [7.0, 3.0, 2.0]], [[7.0, 8.0, 6.0], [6.0, 3.0, 2.0], 
[5.0, 1.0, 4.0]]], [[[8.0, 3.0, 4.0], [7.0, 2.0, 6.0], [7.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [6.0, 6.0, 8.0], [5.0, 4.0, 8.0]], 
[[1.0, 7.0, 6.0], [7.0, 7.0, 1.0], [2.0, 2.0, 1.0]]], [[[8.0, 5.0, 1.0], [5.0, 3.0, 1.0], [1.0, 5.0, 3.0]], [[1.0, 8.0, 7.0], 
[6.0, 5.0, 5.0], [7.0, 6.0, 2.0]], [[4.0, 8.0, 1.0], [3.0, 3.0, 1.0], [7.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 4.0], [3.0, 7.0, 5.0], [8.0, 7.0, 2.0]], [[2.0, 8.0, 5.0], [2.0, 1.0, 3.0], [3.0, 
4.0, 7.0]], [[1.0, 8.0, 4.0], [1.0, 5.0, 1.0], [2.0, 1.0, 4.0]]], [[[3.0, 8.0, 5.0], [1.0, 3.0, 1.0], [3.0, 7.0, 3.0]], [[3.0, 
2.0, 5.0], [1.0, 8.0, 4.0], [8.0, 4.0, 2.0]], [[7.0, 6.0, 5.0], [1.0, 4.0, 2.0], [5.0, 7.0, 2.0]]], [[[7.0, 6.0, 7.0], [5.0, 
1.0, 5.0], [8.0, 3.0, 7.0]], [[7.0, 6.0, 7.0], [2.0, 1.0, 6.0], [8.0, 7.0, 4.0]], [[1.0, 8.0, 3.0], [7.0, 7.0, 3.0], [6.0, 4.0, 
4.0]]], [[[7.0, 3.0, 6.0], [4.0, 4.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 8.0, 2.0], [7.0, 7.0, 1.0]], [[2.0, 6.0, 
7.0], [2.0, 7.0, 5.0], [7.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 3.0], [4.0, 1.0, 7.0], [8.0, 7.0, 8.0]], 
[[2.0, 1.0, 4.0], [5.0, 7.0, 6.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], [4.0, 7.0, 7.0], [6.0, 5.0, 3.0]]], [[[2.0, 2.0, 4.0], 
[3.0, 1.0, 2.0], [3.0, 8.0, 5.0]], [[1.0, 5.0, 5.0], [7.0, 3.0, 2.0], [7.0, 2.0, 8.0]], [[7.0, 8.0, 1.0], [1.0, 3.0, 8.0], 
[6.0, 1.0, 8.0]]], [[[7.0, 3.0, 7.0], [5.0, 7.0, 1.0], [8.0, 2.0, 2.0]], [[7.0, 3.0, 5.0], [8.0, 7.0, 4.0], [4.0, 7.0, 2.0]], 
[[2.0, 8.0, 8.0], [2.0, 1.0, 6.0], [3.0, 7.0, 7.0]]], [[[3.0, 2.0, 3.0], [2.0, 6.0, 5.0], [1.0, 6.0, 3.0]], [[3.0, 8.0, 1.0], 
[5.0, 5.0, 5.0], [8.0, 7.0, 8.0]], [[7.0, 4.0, 7.0], [4.0, 7.0, 1.0], [3.0, 8.0, 2.0]]]])
      val=numpy.array([[3.0, 7.0, 3.0], [8.0, 2.0, 5.0], [3.0, 3.0, 4.0], [1.0, 2.0, 7.0]])
      arg[:,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 4.0], [3.0, 3.0, 5.0], [8.0, 7.0, 2.0]], [[2.0, 8.0, 5.0], [2.0, 7.0, 3.0], [3.0, 
4.0, 7.0]], [[1.0, 8.0, 4.0], [1.0, 3.0, 1.0], [2.0, 1.0, 4.0]]], [[[3.0, 8.0, 5.0], [1.0, 8.0, 1.0], [3.0, 7.0, 3.0]], [[3.0, 
2.0, 5.0], [1.0, 2.0, 4.0], [8.0, 4.0, 2.0]], [[7.0, 6.0, 5.0], [1.0, 5.0, 2.0], [5.0, 7.0, 2.0]]], [[[7.0, 6.0, 7.0], [5.0, 
3.0, 5.0], [8.0, 3.0, 7.0]], [[7.0, 6.0, 7.0], [2.0, 3.0, 6.0], [8.0, 7.0, 4.0]], [[1.0, 8.0, 3.0], [7.0, 4.0, 3.0], [6.0, 4.0, 
4.0]]], [[[7.0, 3.0, 6.0], [4.0, 1.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 2.0, 2.0], [7.0, 7.0, 1.0]], [[2.0, 6.0, 
7.0], [2.0, 7.0, 5.0], [7.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 3.0], [4.0, 3.0, 7.0], [8.0, 7.0, 8.0]], 
[[2.0, 1.0, 4.0], [5.0, 7.0, 6.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], [4.0, 3.0, 7.0], [6.0, 5.0, 3.0]]], [[[2.0, 2.0, 4.0], 
[3.0, 8.0, 2.0], [3.0, 8.0, 5.0]], [[1.0, 5.0, 5.0], [7.0, 2.0, 2.0], [7.0, 2.0, 8.0]], [[7.0, 8.0, 1.0], [1.0, 5.0, 8.0], 
[6.0, 1.0, 8.0]]], [[[7.0, 3.0, 7.0], [5.0, 3.0, 1.0], [8.0, 2.0, 2.0]], [[7.0, 3.0, 5.0], [8.0, 3.0, 4.0], [4.0, 7.0, 2.0]], 
[[2.0, 8.0, 8.0], [2.0, 4.0, 6.0], [3.0, 7.0, 7.0]]], [[[3.0, 2.0, 3.0], [2.0, 1.0, 5.0], [1.0, 6.0, 3.0]], [[3.0, 8.0, 1.0], 
[5.0, 2.0, 5.0], [8.0, 7.0, 8.0]], [[7.0, 4.0, 7.0], [4.0, 7.0, 1.0], [3.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 6.0], [4.0, 8.0, 8.0], [2.0, 5.0, 8.0]], [[7.0, 7.0, 7.0], [2.0, 6.0, 5.0], [4.0, 
2.0, 2.0]], [[3.0, 3.0, 7.0], [6.0, 2.0, 6.0], [3.0, 4.0, 3.0]]], [[[8.0, 5.0, 7.0], [2.0, 2.0, 2.0], [2.0, 5.0, 6.0]], [[4.0, 
4.0, 5.0], [1.0, 4.0, 7.0], [6.0, 7.0, 1.0]], [[2.0, 3.0, 5.0], [5.0, 3.0, 8.0], [2.0, 7.0, 1.0]]], [[[7.0, 4.0, 4.0], [5.0, 
2.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 8.0, 8.0], [7.0, 8.0, 5.0], [8.0, 4.0, 3.0]], [[8.0, 1.0, 5.0], [8.0, 4.0, 2.0], [4.0, 3.0, 
1.0]]], [[[5.0, 8.0, 3.0], [6.0, 7.0, 4.0], [1.0, 4.0, 3.0]], [[3.0, 7.0, 5.0], [3.0, 3.0, 7.0], [1.0, 1.0, 8.0]], [[5.0, 6.0, 
3.0], [5.0, 3.0, 1.0], [1.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 3.0], [2.0, 3.0, 3.0], [5.0, 6.0, 8.0]], 
[[2.0, 7.0, 3.0], [3.0, 5.0, 3.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 6.0], [4.0, 4.0, 3.0], [6.0, 7.0, 4.0]]], [[[5.0, 3.0, 4.0], 
[5.0, 8.0, 6.0], [5.0, 2.0, 5.0]], [[3.0, 6.0, 4.0], [5.0, 5.0, 8.0], [5.0, 1.0, 3.0]], [[4.0, 7.0, 5.0], [5.0, 4.0, 5.0], 
[1.0, 5.0, 1.0]]], [[[8.0, 1.0, 4.0], [1.0, 1.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 2.0, 1.0], [4.0, 8.0, 5.0], [8.0, 1.0, 4.0]], 
[[6.0, 1.0, 8.0], [8.0, 2.0, 8.0], [4.0, 1.0, 6.0]]], [[[8.0, 2.0, 1.0], [1.0, 7.0, 5.0], [7.0, 3.0, 2.0]], [[7.0, 1.0, 4.0], 
[7.0, 8.0, 3.0], [4.0, 4.0, 4.0]], [[4.0, 4.0, 4.0], [1.0, 1.0, 4.0], [8.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[2.0, 6.0, 4.0], [8.0, 2.0, 4.0], [4.0, 4.0, 2.0], [6.0, 4.0, 5.0]]),self.functionspace)
      arg[:,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 6.0], [4.0, 2.0, 8.0], [2.0, 5.0, 8.0]], [[7.0, 7.0, 7.0], [2.0, 6.0, 5.0], [4.0, 
2.0, 2.0]], [[3.0, 3.0, 7.0], [6.0, 4.0, 6.0], [3.0, 4.0, 3.0]]], [[[8.0, 5.0, 7.0], [2.0, 8.0, 2.0], [2.0, 5.0, 6.0]], [[4.0, 
4.0, 5.0], [1.0, 2.0, 7.0], [6.0, 7.0, 1.0]], [[2.0, 3.0, 5.0], [5.0, 4.0, 8.0], [2.0, 7.0, 1.0]]], [[[7.0, 4.0, 4.0], [5.0, 
4.0, 3.0], [3.0, 2.0, 8.0]], [[8.0, 8.0, 8.0], [7.0, 4.0, 5.0], [8.0, 4.0, 3.0]], [[8.0, 1.0, 5.0], [8.0, 2.0, 2.0], [4.0, 3.0, 
1.0]]], [[[5.0, 8.0, 3.0], [6.0, 6.0, 4.0], [1.0, 4.0, 3.0]], [[3.0, 7.0, 5.0], [3.0, 4.0, 7.0], [1.0, 1.0, 8.0]], [[5.0, 6.0, 
3.0], [5.0, 5.0, 1.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 3.0], [2.0, 2.0, 3.0], [5.0, 6.0, 8.0]], 
[[2.0, 7.0, 3.0], [3.0, 6.0, 3.0], [5.0, 3.0, 1.0]], [[3.0, 5.0, 6.0], [4.0, 4.0, 3.0], [6.0, 7.0, 4.0]]], [[[5.0, 3.0, 4.0], 
[5.0, 8.0, 6.0], [5.0, 2.0, 5.0]], [[3.0, 6.0, 4.0], [5.0, 2.0, 8.0], [5.0, 1.0, 3.0]], [[4.0, 7.0, 5.0], [5.0, 4.0, 5.0], 
[1.0, 5.0, 1.0]]], [[[8.0, 1.0, 4.0], [1.0, 4.0, 2.0], [1.0, 6.0, 1.0]], [[8.0, 2.0, 1.0], [4.0, 4.0, 5.0], [8.0, 1.0, 4.0]], 
[[6.0, 1.0, 8.0], [8.0, 2.0, 8.0], [4.0, 1.0, 6.0]]], [[[8.0, 2.0, 1.0], [1.0, 6.0, 5.0], [7.0, 3.0, 2.0]], [[7.0, 1.0, 4.0], 
[7.0, 4.0, 3.0], [4.0, 4.0, 4.0]], [[4.0, 4.0, 4.0], [1.0, 5.0, 4.0], [8.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 2.0], [6.0, 8.0, 3.0], [6.0, 7.0, 3.0]], [[5.0, 8.0, 1.0], [5.0, 6.0, 7.0], [2.0, 
7.0, 5.0]], [[7.0, 6.0, 5.0], [5.0, 4.0, 4.0], [2.0, 5.0, 2.0]]], [[[7.0, 5.0, 2.0], [7.0, 1.0, 8.0], [5.0, 4.0, 6.0]], [[4.0, 
6.0, 8.0], [6.0, 1.0, 3.0], [1.0, 2.0, 3.0]], [[2.0, 7.0, 6.0], [7.0, 6.0, 3.0], [2.0, 7.0, 1.0]]], [[[3.0, 2.0, 6.0], [6.0, 
1.0, 8.0], [5.0, 3.0, 4.0]], [[6.0, 4.0, 8.0], [4.0, 5.0, 1.0], [4.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 6.0, 3.0], [6.0, 5.0, 
5.0]]], [[[5.0, 5.0, 2.0], [6.0, 1.0, 4.0], [1.0, 5.0, 4.0]], [[1.0, 4.0, 2.0], [7.0, 8.0, 3.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 
1.0], [8.0, 1.0, 2.0], [2.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 1.0], [6.0, 6.0, 6.0], [8.0, 2.0, 7.0]], 
[[3.0, 7.0, 6.0], [3.0, 2.0, 8.0], [3.0, 5.0, 5.0]], [[2.0, 7.0, 2.0], [6.0, 5.0, 2.0], [8.0, 3.0, 6.0]]], [[[2.0, 2.0, 8.0], 
[8.0, 5.0, 5.0], [1.0, 2.0, 3.0]], [[4.0, 4.0, 2.0], [7.0, 1.0, 5.0], [1.0, 5.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 8.0, 1.0], 
[2.0, 4.0, 4.0]]], [[[4.0, 7.0, 6.0], [3.0, 1.0, 7.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 7.0], [1.0, 5.0, 4.0], [1.0, 6.0, 1.0]], 
[[7.0, 3.0, 4.0], [6.0, 7.0, 6.0], [6.0, 4.0, 6.0]]], [[[8.0, 2.0, 8.0], [7.0, 1.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 7.0], 
[1.0, 3.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 3.0, 6.0], [1.0, 4.0, 8.0], [4.0, 1.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 7.0, 6.0], [5.0, 6.0, 3.0], [3.0, 8.0, 6.0], [1.0, 1.0, 
7.0]])+(1.-msk_val)*numpy.array([[7.0, 8.0, 7.0], [5.0, 5.0, 2.0], [1.0, 7.0, 7.0], [7.0, 5.0, 4.0]])
      arg[:,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 2.0], [6.0, 3.0, 3.0], [6.0, 7.0, 3.0]], [[5.0, 8.0, 1.0], [5.0, 7.0, 7.0], [2.0, 
7.0, 5.0]], [[7.0, 6.0, 5.0], [5.0, 6.0, 4.0], [2.0, 5.0, 2.0]]], [[[7.0, 5.0, 2.0], [7.0, 5.0, 8.0], [5.0, 4.0, 6.0]], [[4.0, 
6.0, 8.0], [6.0, 6.0, 3.0], [1.0, 2.0, 3.0]], [[2.0, 7.0, 6.0], [7.0, 3.0, 3.0], [2.0, 7.0, 1.0]]], [[[3.0, 2.0, 6.0], [6.0, 
3.0, 8.0], [5.0, 3.0, 4.0]], [[6.0, 4.0, 8.0], [4.0, 8.0, 1.0], [4.0, 6.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 6.0, 3.0], [6.0, 5.0, 
5.0]]], [[[5.0, 5.0, 2.0], [6.0, 1.0, 4.0], [1.0, 5.0, 4.0]], [[1.0, 4.0, 2.0], [7.0, 1.0, 3.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 
1.0], [8.0, 7.0, 2.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 1.0], [6.0, 7.0, 6.0], [8.0, 2.0, 7.0]], 
[[3.0, 7.0, 6.0], [3.0, 8.0, 8.0], [3.0, 5.0, 5.0]], [[2.0, 7.0, 2.0], [6.0, 7.0, 2.0], [8.0, 3.0, 6.0]]], [[[2.0, 2.0, 8.0], 
[8.0, 5.0, 5.0], [1.0, 2.0, 3.0]], [[4.0, 4.0, 2.0], [7.0, 5.0, 5.0], [1.0, 5.0, 1.0]], [[1.0, 3.0, 1.0], [8.0, 2.0, 1.0], 
[2.0, 4.0, 4.0]]], [[[4.0, 7.0, 6.0], [3.0, 1.0, 7.0], [5.0, 3.0, 2.0]], [[7.0, 1.0, 7.0], [1.0, 7.0, 4.0], [1.0, 6.0, 1.0]], 
[[7.0, 3.0, 4.0], [6.0, 7.0, 6.0], [6.0, 4.0, 6.0]]], [[[8.0, 2.0, 8.0], [7.0, 7.0, 5.0], [3.0, 5.0, 5.0]], [[7.0, 1.0, 7.0], 
[1.0, 5.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 3.0, 6.0], [1.0, 4.0, 8.0], [4.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CUUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 5.0], [2.0, 2.0, 7.0], [5.0, 2.0, 6.0]], [[5.0, 8.0, 4.0], [6.0, 7.0, 4.0], [2.0, 
1.0, 2.0]], [[2.0, 6.0, 6.0], [4.0, 1.0, 7.0], [1.0, 8.0, 3.0]]], [[[3.0, 3.0, 4.0], [4.0, 2.0, 1.0], [8.0, 4.0, 7.0]], [[3.0, 
3.0, 7.0], [3.0, 5.0, 7.0], [3.0, 8.0, 8.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 6.0], [7.0, 1.0, 5.0]]], [[[1.0, 8.0, 4.0], [3.0, 
1.0, 5.0], [6.0, 3.0, 3.0]], [[2.0, 5.0, 7.0], [3.0, 1.0, 4.0], [4.0, 8.0, 8.0]], [[5.0, 5.0, 4.0], [1.0, 2.0, 8.0], [1.0, 8.0, 
2.0]]], [[[6.0, 5.0, 3.0], [4.0, 7.0, 2.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 4.0], [2.0, 2.0, 6.0], [5.0, 5.0, 7.0]], [[5.0, 5.0, 
2.0], [6.0, 2.0, 1.0], [6.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 5.0], [8.0, 2.0, 4.0], [5.0, 4.0, 3.0]], 
[[4.0, 1.0, 4.0], [5.0, 2.0, 3.0], [5.0, 5.0, 5.0]], [[2.0, 6.0, 4.0], [2.0, 3.0, 1.0], [1.0, 3.0, 7.0]]], [[[8.0, 2.0, 3.0], 
[1.0, 1.0, 4.0], [1.0, 6.0, 8.0]], [[4.0, 7.0, 4.0], [6.0, 3.0, 8.0], [6.0, 5.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 4.0], 
[8.0, 2.0, 2.0]]], [[[1.0, 1.0, 3.0], [7.0, 6.0, 6.0], [3.0, 2.0, 8.0]], [[3.0, 6.0, 6.0], [3.0, 8.0, 5.0], [6.0, 6.0, 1.0]], 
[[7.0, 2.0, 7.0], [5.0, 4.0, 8.0], [4.0, 7.0, 5.0]]], [[[3.0, 8.0, 4.0], [7.0, 2.0, 8.0], [8.0, 7.0, 3.0]], [[4.0, 1.0, 2.0], 
[4.0, 6.0, 1.0], [2.0, 4.0, 3.0]], [[1.0, 8.0, 7.0], [8.0, 4.0, 2.0], [3.0, 6.0, 8.0]]]])
      val=numpy.array([[[[5.0], [3.0]], [[1.0], [2.0]]], [[[2.0], [2.0]], [[6.0], [8.0]]], [[[7.0], [8.0]], [[4.0], [2.0]]], 
[[[1.0], [7.0]], [[7.0], [5.0]]]])
      arg[:,:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 5.0], [2.0, 2.0, 3.0], [5.0, 2.0, 6.0]], [[5.0, 8.0, 1.0], [6.0, 7.0, 2.0], [2.0, 
1.0, 2.0]], [[2.0, 6.0, 6.0], [4.0, 1.0, 7.0], [1.0, 8.0, 3.0]]], [[[3.0, 3.0, 2.0], [4.0, 2.0, 2.0], [8.0, 4.0, 7.0]], [[3.0, 
3.0, 6.0], [3.0, 5.0, 8.0], [3.0, 8.0, 8.0]], [[7.0, 4.0, 3.0], [8.0, 1.0, 6.0], [7.0, 1.0, 5.0]]], [[[1.0, 8.0, 7.0], [3.0, 
1.0, 8.0], [6.0, 3.0, 3.0]], [[2.0, 5.0, 4.0], [3.0, 1.0, 2.0], [4.0, 8.0, 8.0]], [[5.0, 5.0, 4.0], [1.0, 2.0, 8.0], [1.0, 8.0, 
2.0]]], [[[6.0, 5.0, 1.0], [4.0, 7.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 2.0, 7.0], [2.0, 2.0, 5.0], [5.0, 5.0, 7.0]], [[5.0, 5.0, 
2.0], [6.0, 2.0, 1.0], [6.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 5.0], [8.0, 2.0, 3.0], [5.0, 4.0, 3.0]], 
[[4.0, 1.0, 1.0], [5.0, 2.0, 2.0], [5.0, 5.0, 5.0]], [[2.0, 6.0, 4.0], [2.0, 3.0, 1.0], [1.0, 3.0, 7.0]]], [[[8.0, 2.0, 2.0], 
[1.0, 1.0, 2.0], [1.0, 6.0, 8.0]], [[4.0, 7.0, 6.0], [6.0, 3.0, 8.0], [6.0, 5.0, 2.0]], [[5.0, 7.0, 6.0], [6.0, 4.0, 4.0], 
[8.0, 2.0, 2.0]]], [[[1.0, 1.0, 7.0], [7.0, 6.0, 8.0], [3.0, 2.0, 8.0]], [[3.0, 6.0, 4.0], [3.0, 8.0, 2.0], [6.0, 6.0, 1.0]], 
[[7.0, 2.0, 7.0], [5.0, 4.0, 8.0], [4.0, 7.0, 5.0]]], [[[3.0, 8.0, 1.0], [7.0, 2.0, 7.0], [8.0, 7.0, 3.0]], [[4.0, 1.0, 7.0], 
[4.0, 6.0, 5.0], [2.0, 4.0, 3.0]], [[1.0, 8.0, 7.0], [8.0, 4.0, 2.0], [3.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CUUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 2.0], [4.0, 3.0, 1.0], [2.0, 5.0, 6.0]], [[8.0, 2.0, 2.0], [5.0, 2.0, 7.0], [1.0, 
5.0, 1.0]], [[2.0, 3.0, 4.0], [7.0, 6.0, 5.0], [7.0, 1.0, 1.0]]], [[[8.0, 4.0, 5.0], [8.0, 6.0, 3.0], [3.0, 7.0, 4.0]], [[6.0, 
5.0, 6.0], [8.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[8.0, 7.0, 6.0], [3.0, 1.0, 7.0], [4.0, 7.0, 4.0]]], [[[8.0, 3.0, 2.0], [5.0, 
8.0, 5.0], [6.0, 6.0, 1.0]], [[6.0, 3.0, 2.0], [3.0, 3.0, 3.0], [6.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [1.0, 8.0, 2.0], [6.0, 5.0, 
2.0]]], [[[8.0, 4.0, 2.0], [6.0, 2.0, 5.0], [8.0, 4.0, 1.0]], [[2.0, 1.0, 4.0], [7.0, 6.0, 8.0], [7.0, 3.0, 1.0]], [[8.0, 8.0, 
8.0], [8.0, 2.0, 3.0], [8.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 8.0], [5.0, 3.0, 6.0], [2.0, 8.0, 3.0]], 
[[8.0, 3.0, 7.0], [4.0, 5.0, 7.0], [5.0, 5.0, 8.0]], [[1.0, 4.0, 3.0], [5.0, 2.0, 4.0], [7.0, 8.0, 7.0]]], [[[6.0, 5.0, 5.0], 
[3.0, 6.0, 3.0], [8.0, 8.0, 6.0]], [[2.0, 6.0, 4.0], [7.0, 1.0, 1.0], [5.0, 4.0, 6.0]], [[4.0, 1.0, 5.0], [6.0, 8.0, 1.0], 
[1.0, 3.0, 5.0]]], [[[7.0, 3.0, 8.0], [4.0, 5.0, 5.0], [6.0, 8.0, 5.0]], [[3.0, 2.0, 7.0], [2.0, 8.0, 5.0], [6.0, 8.0, 6.0]], 
[[1.0, 3.0, 6.0], [8.0, 6.0, 6.0], [7.0, 8.0, 7.0]]], [[[5.0, 3.0, 4.0], [4.0, 2.0, 6.0], [6.0, 3.0, 7.0]], [[6.0, 5.0, 5.0], 
[4.0, 7.0, 5.0], [4.0, 4.0, 2.0]], [[5.0, 4.0, 4.0], [6.0, 4.0, 2.0], [1.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[[2.0], [7.0]], [[1.0], [7.0]]], [[[2.0], [8.0]], [[8.0], [6.0]]], [[[6.0], [2.0]], [[8.0], 
[2.0]]], [[[2.0], [2.0]], [[8.0], [3.0]]]]),self.functionspace)
      arg[:,:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 2.0], [4.0, 3.0, 7.0], [2.0, 5.0, 6.0]], [[8.0, 2.0, 1.0], [5.0, 2.0, 7.0], [1.0, 
5.0, 1.0]], [[2.0, 3.0, 4.0], [7.0, 6.0, 5.0], [7.0, 1.0, 1.0]]], [[[8.0, 4.0, 2.0], [8.0, 6.0, 8.0], [3.0, 7.0, 4.0]], [[6.0, 
5.0, 8.0], [8.0, 7.0, 6.0], [5.0, 5.0, 4.0]], [[8.0, 7.0, 6.0], [3.0, 1.0, 7.0], [4.0, 7.0, 4.0]]], [[[8.0, 3.0, 6.0], [5.0, 
8.0, 2.0], [6.0, 6.0, 1.0]], [[6.0, 3.0, 8.0], [3.0, 3.0, 2.0], [6.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [1.0, 8.0, 2.0], [6.0, 5.0, 
2.0]]], [[[8.0, 4.0, 2.0], [6.0, 2.0, 2.0], [8.0, 4.0, 1.0]], [[2.0, 1.0, 8.0], [7.0, 6.0, 3.0], [7.0, 3.0, 1.0]], [[8.0, 8.0, 
8.0], [8.0, 2.0, 3.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 2.0], [5.0, 3.0, 7.0], [2.0, 8.0, 3.0]], 
[[8.0, 3.0, 1.0], [4.0, 5.0, 7.0], [5.0, 5.0, 8.0]], [[1.0, 4.0, 3.0], [5.0, 2.0, 4.0], [7.0, 8.0, 7.0]]], [[[6.0, 5.0, 2.0], 
[3.0, 6.0, 8.0], [8.0, 8.0, 6.0]], [[2.0, 6.0, 8.0], [7.0, 1.0, 6.0], [5.0, 4.0, 6.0]], [[4.0, 1.0, 5.0], [6.0, 8.0, 1.0], 
[1.0, 3.0, 5.0]]], [[[7.0, 3.0, 6.0], [4.0, 5.0, 2.0], [6.0, 8.0, 5.0]], [[3.0, 2.0, 8.0], [2.0, 8.0, 2.0], [6.0, 8.0, 6.0]], 
[[1.0, 3.0, 6.0], [8.0, 6.0, 6.0], [7.0, 8.0, 7.0]]], [[[5.0, 3.0, 2.0], [4.0, 2.0, 2.0], [6.0, 3.0, 7.0]], [[6.0, 5.0, 8.0], 
[4.0, 7.0, 3.0], [4.0, 4.0, 2.0]], [[5.0, 4.0, 4.0], [6.0, 4.0, 2.0], [1.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CUUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 8.0], [8.0, 8.0, 6.0], [4.0, 8.0, 5.0]], [[6.0, 8.0, 7.0], [6.0, 1.0, 5.0], [4.0, 
2.0, 6.0]], [[2.0, 5.0, 1.0], [2.0, 7.0, 5.0], [3.0, 4.0, 7.0]]], [[[2.0, 5.0, 3.0], [4.0, 5.0, 4.0], [5.0, 2.0, 4.0]], [[3.0, 
8.0, 1.0], [1.0, 8.0, 7.0], [5.0, 5.0, 1.0]], [[5.0, 8.0, 3.0], [4.0, 2.0, 5.0], [6.0, 6.0, 4.0]]], [[[6.0, 1.0, 5.0], [3.0, 
8.0, 8.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 8.0], [1.0, 3.0, 4.0], [1.0, 1.0, 7.0]], [[1.0, 3.0, 8.0], [4.0, 2.0, 7.0], [5.0, 7.0, 
6.0]]], [[[7.0, 7.0, 5.0], [4.0, 6.0, 3.0], [8.0, 3.0, 1.0]], [[3.0, 8.0, 3.0], [5.0, 3.0, 7.0], [4.0, 8.0, 7.0]], [[8.0, 2.0, 
1.0], [4.0, 5.0, 1.0], [1.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 7.0], [2.0, 5.0, 3.0], [1.0, 1.0, 6.0]], 
[[8.0, 8.0, 2.0], [8.0, 4.0, 5.0], [7.0, 4.0, 5.0]], [[5.0, 2.0, 8.0], [4.0, 2.0, 3.0], [2.0, 7.0, 1.0]]], [[[5.0, 7.0, 1.0], 
[6.0, 7.0, 5.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 1.0], [8.0, 3.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 8.0, 2.0], [6.0, 8.0, 4.0], 
[3.0, 7.0, 5.0]]], [[[5.0, 6.0, 2.0], [4.0, 4.0, 1.0], [3.0, 1.0, 2.0]], [[2.0, 5.0, 5.0], [1.0, 1.0, 1.0], [8.0, 4.0, 4.0]], 
[[4.0, 8.0, 5.0], [4.0, 8.0, 4.0], [4.0, 3.0, 1.0]]], [[[6.0, 6.0, 5.0], [6.0, 6.0, 5.0], [1.0, 6.0, 4.0]], [[6.0, 7.0, 7.0], 
[4.0, 4.0, 3.0], [6.0, 5.0, 5.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 2.0], [6.0, 2.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0], [4.0]], [[8.0], [3.0]]], [[[4.0], [6.0]], [[4.0], [1.0]]], [[[3.0], [3.0]], [[8.0], 
[2.0]]], [[[5.0], [4.0]], [[5.0], [8.0]]]])+(1.-msk_val)*numpy.array([[[[1.0], [1.0]], [[2.0], [3.0]]], [[[1.0], [8.0]], 
[[3.0], [8.0]]], [[[6.0], [3.0]], [[3.0], [4.0]]], [[[8.0], [7.0]], [[1.0], [5.0]]]])
      arg[:,:2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 7.0], [8.0, 8.0, 4.0], [4.0, 8.0, 5.0]], [[6.0, 8.0, 8.0], [6.0, 1.0, 3.0], [4.0, 
2.0, 6.0]], [[2.0, 5.0, 1.0], [2.0, 7.0, 5.0], [3.0, 4.0, 7.0]]], [[[2.0, 5.0, 4.0], [4.0, 5.0, 6.0], [5.0, 2.0, 4.0]], [[3.0, 
8.0, 4.0], [1.0, 8.0, 1.0], [5.0, 5.0, 1.0]], [[5.0, 8.0, 3.0], [4.0, 2.0, 5.0], [6.0, 6.0, 4.0]]], [[[6.0, 1.0, 3.0], [3.0, 
8.0, 3.0], [4.0, 5.0, 7.0]], [[8.0, 6.0, 8.0], [1.0, 3.0, 2.0], [1.0, 1.0, 7.0]], [[1.0, 3.0, 8.0], [4.0, 2.0, 7.0], [5.0, 7.0, 
6.0]]], [[[7.0, 7.0, 5.0], [4.0, 6.0, 4.0], [8.0, 3.0, 1.0]], [[3.0, 8.0, 5.0], [5.0, 3.0, 8.0], [4.0, 8.0, 7.0]], [[8.0, 2.0, 
1.0], [4.0, 5.0, 1.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 1.0], [2.0, 5.0, 1.0], [1.0, 1.0, 6.0]], 
[[8.0, 8.0, 2.0], [8.0, 4.0, 3.0], [7.0, 4.0, 5.0]], [[5.0, 2.0, 8.0], [4.0, 2.0, 3.0], [2.0, 7.0, 1.0]]], [[[5.0, 7.0, 1.0], 
[6.0, 7.0, 8.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 3.0], [8.0, 3.0, 8.0], [6.0, 2.0, 8.0]], [[3.0, 8.0, 2.0], [6.0, 8.0, 4.0], 
[3.0, 7.0, 5.0]]], [[[5.0, 6.0, 6.0], [4.0, 4.0, 3.0], [3.0, 1.0, 2.0]], [[2.0, 5.0, 3.0], [1.0, 1.0, 4.0], [8.0, 4.0, 4.0]], 
[[4.0, 8.0, 5.0], [4.0, 8.0, 4.0], [4.0, 3.0, 1.0]]], [[[6.0, 6.0, 8.0], [6.0, 6.0, 7.0], [1.0, 6.0, 4.0]], [[6.0, 7.0, 1.0], 
[4.0, 4.0, 5.0], [6.0, 5.0, 5.0]], [[2.0, 5.0, 2.0], [5.0, 6.0, 2.0], [6.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 3.0], [4.0, 4.0, 5.0], [8.0, 2.0, 1.0]], [[1.0, 7.0, 6.0], [6.0, 3.0, 7.0], [7.0, 
8.0, 2.0]], [[2.0, 4.0, 4.0], [6.0, 6.0, 1.0], [6.0, 5.0, 4.0]]], [[[3.0, 6.0, 4.0], [6.0, 1.0, 7.0], [1.0, 6.0, 5.0]], [[2.0, 
5.0, 5.0], [6.0, 1.0, 8.0], [8.0, 3.0, 2.0]], [[4.0, 3.0, 6.0], [7.0, 6.0, 5.0], [4.0, 3.0, 3.0]]], [[[7.0, 1.0, 3.0], [3.0, 
3.0, 3.0], [7.0, 7.0, 6.0]], [[3.0, 2.0, 7.0], [2.0, 4.0, 8.0], [7.0, 1.0, 3.0]], [[5.0, 4.0, 1.0], [4.0, 4.0, 4.0], [2.0, 4.0, 
3.0]]], [[[8.0, 7.0, 4.0], [8.0, 5.0, 8.0], [5.0, 3.0, 7.0]], [[7.0, 8.0, 1.0], [1.0, 8.0, 8.0], [1.0, 2.0, 2.0]], [[4.0, 4.0, 
5.0], [5.0, 1.0, 8.0], [8.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 8.0], [1.0, 5.0, 6.0], [1.0, 2.0, 8.0]], 
[[3.0, 1.0, 5.0], [7.0, 6.0, 4.0], [6.0, 1.0, 5.0]], [[7.0, 2.0, 5.0], [7.0, 4.0, 6.0], [5.0, 8.0, 1.0]]], [[[8.0, 2.0, 4.0], 
[2.0, 3.0, 3.0], [2.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [8.0, 7.0, 5.0], [2.0, 6.0, 1.0]], [[7.0, 3.0, 8.0], [1.0, 2.0, 6.0], 
[6.0, 6.0, 1.0]]], [[[7.0, 2.0, 2.0], [2.0, 8.0, 3.0], [6.0, 1.0, 8.0]], [[6.0, 8.0, 1.0], [8.0, 4.0, 5.0], [1.0, 7.0, 3.0]], 
[[7.0, 6.0, 2.0], [8.0, 1.0, 4.0], [5.0, 5.0, 1.0]]], [[[4.0, 4.0, 4.0], [3.0, 4.0, 7.0], [7.0, 4.0, 3.0]], [[2.0, 1.0, 3.0], 
[1.0, 6.0, 4.0], [4.0, 5.0, 8.0]], [[2.0, 4.0, 6.0], [8.0, 2.0, 4.0], [3.0, 1.0, 4.0]]]])
      val=numpy.array([[2.0, 1.0, 7.0], [5.0, 8.0, 8.0], [2.0, 4.0, 2.0]])
      arg[2,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 3.0], [4.0, 4.0, 5.0], [8.0, 2.0, 1.0]], [[1.0, 7.0, 6.0], [6.0, 3.0, 7.0], [7.0, 
8.0, 2.0]], [[2.0, 4.0, 4.0], [6.0, 6.0, 1.0], [6.0, 5.0, 4.0]]], [[[3.0, 6.0, 4.0], [6.0, 1.0, 7.0], [1.0, 6.0, 5.0]], [[2.0, 
5.0, 5.0], [6.0, 1.0, 8.0], [8.0, 3.0, 2.0]], [[4.0, 3.0, 6.0], [7.0, 6.0, 5.0], [4.0, 3.0, 3.0]]], [[[7.0, 1.0, 3.0], [3.0, 
3.0, 3.0], [7.0, 7.0, 6.0]], [[2.0, 1.0, 7.0], [5.0, 8.0, 8.0], [2.0, 4.0, 2.0]], [[5.0, 4.0, 1.0], [4.0, 4.0, 4.0], [2.0, 4.0, 
3.0]]], [[[8.0, 7.0, 4.0], [8.0, 5.0, 8.0], [5.0, 3.0, 7.0]], [[7.0, 8.0, 1.0], [1.0, 8.0, 8.0], [1.0, 2.0, 2.0]], [[4.0, 4.0, 
5.0], [5.0, 1.0, 8.0], [8.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 8.0], [1.0, 5.0, 6.0], [1.0, 2.0, 8.0]], 
[[3.0, 1.0, 5.0], [7.0, 6.0, 4.0], [6.0, 1.0, 5.0]], [[7.0, 2.0, 5.0], [7.0, 4.0, 6.0], [5.0, 8.0, 1.0]]], [[[8.0, 2.0, 4.0], 
[2.0, 3.0, 3.0], [2.0, 5.0, 5.0]], [[1.0, 6.0, 8.0], [8.0, 7.0, 5.0], [2.0, 6.0, 1.0]], [[7.0, 3.0, 8.0], [1.0, 2.0, 6.0], 
[6.0, 6.0, 1.0]]], [[[7.0, 2.0, 2.0], [2.0, 8.0, 3.0], [6.0, 1.0, 8.0]], [[2.0, 1.0, 7.0], [5.0, 8.0, 8.0], [2.0, 4.0, 2.0]], 
[[7.0, 6.0, 2.0], [8.0, 1.0, 4.0], [5.0, 5.0, 1.0]]], [[[4.0, 4.0, 4.0], [3.0, 4.0, 7.0], [7.0, 4.0, 3.0]], [[2.0, 1.0, 3.0], 
[1.0, 6.0, 4.0], [4.0, 5.0, 8.0]], [[2.0, 4.0, 6.0], [8.0, 2.0, 4.0], [3.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 4.0], [1.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[5.0, 5.0, 1.0], [3.0, 6.0, 4.0], [1.0, 
3.0, 3.0]], [[5.0, 3.0, 8.0], [8.0, 8.0, 7.0], [1.0, 2.0, 8.0]]], [[[5.0, 8.0, 3.0], [1.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 
6.0, 4.0], [4.0, 5.0, 1.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 7.0], [7.0, 4.0, 7.0], [4.0, 8.0, 1.0]]], [[[3.0, 7.0, 6.0], [2.0, 
3.0, 2.0], [5.0, 7.0, 2.0]], [[5.0, 7.0, 8.0], [5.0, 4.0, 3.0], [3.0, 7.0, 3.0]], [[8.0, 3.0, 3.0], [2.0, 6.0, 2.0], [2.0, 8.0, 
7.0]]], [[[5.0, 5.0, 7.0], [8.0, 4.0, 4.0], [6.0, 1.0, 2.0]], [[8.0, 5.0, 1.0], [6.0, 6.0, 3.0], [4.0, 5.0, 5.0]], [[2.0, 6.0, 
6.0], [5.0, 7.0, 2.0], [8.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 5.0], [4.0, 8.0, 1.0], [7.0, 1.0, 5.0]], 
[[5.0, 8.0, 7.0], [3.0, 3.0, 4.0], [2.0, 4.0, 2.0]], [[7.0, 5.0, 1.0], [3.0, 2.0, 2.0], [5.0, 7.0, 2.0]]], [[[5.0, 7.0, 4.0], 
[1.0, 6.0, 5.0], [8.0, 7.0, 6.0]], [[5.0, 6.0, 2.0], [2.0, 3.0, 4.0], [3.0, 6.0, 1.0]], [[5.0, 2.0, 7.0], [1.0, 2.0, 4.0], 
[6.0, 6.0, 1.0]]], [[[1.0, 5.0, 4.0], [8.0, 2.0, 5.0], [8.0, 3.0, 1.0]], [[4.0, 5.0, 6.0], [6.0, 7.0, 2.0], [7.0, 1.0, 7.0]], 
[[5.0, 5.0, 7.0], [2.0, 2.0, 3.0], [5.0, 7.0, 1.0]]], [[[3.0, 8.0, 8.0], [8.0, 7.0, 5.0], [6.0, 7.0, 4.0]], [[3.0, 7.0, 7.0], 
[8.0, 7.0, 6.0], [3.0, 5.0, 8.0]], [[6.0, 2.0, 3.0], [4.0, 1.0, 3.0], [7.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[8.0, 2.0, 2.0], [1.0, 2.0, 3.0], [4.0, 7.0, 5.0]]),self.functionspace)
      arg[2,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 4.0], [1.0, 7.0, 2.0], [1.0, 7.0, 3.0]], [[5.0, 5.0, 1.0], [3.0, 6.0, 4.0], [1.0, 
3.0, 3.0]], [[5.0, 3.0, 8.0], [8.0, 8.0, 7.0], [1.0, 2.0, 8.0]]], [[[5.0, 8.0, 3.0], [1.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 
6.0, 4.0], [4.0, 5.0, 1.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 7.0], [7.0, 4.0, 7.0], [4.0, 8.0, 1.0]]], [[[3.0, 7.0, 6.0], [2.0, 
3.0, 2.0], [5.0, 7.0, 2.0]], [[8.0, 2.0, 2.0], [1.0, 2.0, 3.0], [4.0, 7.0, 5.0]], [[8.0, 3.0, 3.0], [2.0, 6.0, 2.0], [2.0, 8.0, 
7.0]]], [[[5.0, 5.0, 7.0], [8.0, 4.0, 4.0], [6.0, 1.0, 2.0]], [[8.0, 5.0, 1.0], [6.0, 6.0, 3.0], [4.0, 5.0, 5.0]], [[2.0, 6.0, 
6.0], [5.0, 7.0, 2.0], [8.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 5.0], [4.0, 8.0, 1.0], [7.0, 1.0, 5.0]], 
[[5.0, 8.0, 7.0], [3.0, 3.0, 4.0], [2.0, 4.0, 2.0]], [[7.0, 5.0, 1.0], [3.0, 2.0, 2.0], [5.0, 7.0, 2.0]]], [[[5.0, 7.0, 4.0], 
[1.0, 6.0, 5.0], [8.0, 7.0, 6.0]], [[5.0, 6.0, 2.0], [2.0, 3.0, 4.0], [3.0, 6.0, 1.0]], [[5.0, 2.0, 7.0], [1.0, 2.0, 4.0], 
[6.0, 6.0, 1.0]]], [[[1.0, 5.0, 4.0], [8.0, 2.0, 5.0], [8.0, 3.0, 1.0]], [[8.0, 2.0, 2.0], [1.0, 2.0, 3.0], [4.0, 7.0, 5.0]], 
[[5.0, 5.0, 7.0], [2.0, 2.0, 3.0], [5.0, 7.0, 1.0]]], [[[3.0, 8.0, 8.0], [8.0, 7.0, 5.0], [6.0, 7.0, 4.0]], [[3.0, 7.0, 7.0], 
[8.0, 7.0, 6.0], [3.0, 5.0, 8.0]], [[6.0, 2.0, 3.0], [4.0, 1.0, 3.0], [7.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 7.0], [6.0, 4.0, 5.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 5.0], [3.0, 5.0, 6.0], [4.0, 
1.0, 3.0]], [[2.0, 7.0, 3.0], [4.0, 6.0, 2.0], [1.0, 2.0, 2.0]]], [[[2.0, 8.0, 3.0], [7.0, 1.0, 6.0], [8.0, 7.0, 3.0]], [[6.0, 
7.0, 1.0], [3.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[7.0, 3.0, 1.0], [5.0, 8.0, 3.0], [3.0, 8.0, 1.0]]], [[[2.0, 7.0, 2.0], [6.0, 
7.0, 2.0], [7.0, 6.0, 1.0]], [[7.0, 2.0, 3.0], [7.0, 6.0, 1.0], [2.0, 2.0, 6.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 3.0], [5.0, 6.0, 
1.0]]], [[[2.0, 1.0, 8.0], [8.0, 2.0, 3.0], [5.0, 2.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 6.0, 3.0], [4.0, 4.0, 7.0]], [[6.0, 2.0, 
1.0], [7.0, 5.0, 1.0], [3.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 7.0], [4.0, 8.0, 1.0], [5.0, 5.0, 2.0]], 
[[6.0, 7.0, 2.0], [3.0, 2.0, 8.0], [7.0, 1.0, 8.0]], [[6.0, 4.0, 3.0], [4.0, 4.0, 4.0], [5.0, 1.0, 2.0]]], [[[8.0, 8.0, 8.0], 
[1.0, 6.0, 8.0], [8.0, 2.0, 5.0]], [[8.0, 1.0, 1.0], [8.0, 6.0, 7.0], [4.0, 4.0, 4.0]], [[2.0, 7.0, 7.0], [4.0, 6.0, 2.0], 
[2.0, 5.0, 3.0]]], [[[2.0, 3.0, 4.0], [6.0, 8.0, 1.0], [2.0, 3.0, 2.0]], [[4.0, 1.0, 2.0], [2.0, 6.0, 4.0], [5.0, 2.0, 6.0]], 
[[6.0, 8.0, 8.0], [3.0, 7.0, 3.0], [2.0, 6.0, 5.0]]], [[[3.0, 3.0, 2.0], [7.0, 1.0, 3.0], [6.0, 3.0, 4.0]], [[4.0, 2.0, 7.0], 
[7.0, 1.0, 8.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 3.0], [7.0, 4.0, 6.0], [7.0, 8.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 7.0, 1.0], [4.0, 7.0, 7.0], [7.0, 1.0, 4.0]])+(1.-msk_val)*numpy.array([[8.0, 5.0, 4.0], 
[8.0, 5.0, 5.0], [7.0, 3.0, 2.0]])
      arg[2,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 7.0], [6.0, 4.0, 5.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 5.0], [3.0, 5.0, 6.0], [4.0, 
1.0, 3.0]], [[2.0, 7.0, 3.0], [4.0, 6.0, 2.0], [1.0, 2.0, 2.0]]], [[[2.0, 8.0, 3.0], [7.0, 1.0, 6.0], [8.0, 7.0, 3.0]], [[6.0, 
7.0, 1.0], [3.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[7.0, 3.0, 1.0], [5.0, 8.0, 3.0], [3.0, 8.0, 1.0]]], [[[2.0, 7.0, 2.0], [6.0, 
7.0, 2.0], [7.0, 6.0, 1.0]], [[2.0, 7.0, 1.0], [4.0, 7.0, 7.0], [7.0, 1.0, 4.0]], [[2.0, 8.0, 7.0], [4.0, 5.0, 3.0], [5.0, 6.0, 
1.0]]], [[[2.0, 1.0, 8.0], [8.0, 2.0, 3.0], [5.0, 2.0, 8.0]], [[2.0, 7.0, 3.0], [7.0, 6.0, 3.0], [4.0, 4.0, 7.0]], [[6.0, 2.0, 
1.0], [7.0, 5.0, 1.0], [3.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 7.0], [4.0, 8.0, 1.0], [5.0, 5.0, 2.0]], 
[[6.0, 7.0, 2.0], [3.0, 2.0, 8.0], [7.0, 1.0, 8.0]], [[6.0, 4.0, 3.0], [4.0, 4.0, 4.0], [5.0, 1.0, 2.0]]], [[[8.0, 8.0, 8.0], 
[1.0, 6.0, 8.0], [8.0, 2.0, 5.0]], [[8.0, 1.0, 1.0], [8.0, 6.0, 7.0], [4.0, 4.0, 4.0]], [[2.0, 7.0, 7.0], [4.0, 6.0, 2.0], 
[2.0, 5.0, 3.0]]], [[[2.0, 3.0, 4.0], [6.0, 8.0, 1.0], [2.0, 3.0, 2.0]], [[8.0, 5.0, 4.0], [8.0, 5.0, 5.0], [7.0, 3.0, 2.0]], 
[[6.0, 8.0, 8.0], [3.0, 7.0, 3.0], [2.0, 6.0, 5.0]]], [[[3.0, 3.0, 2.0], [7.0, 1.0, 3.0], [6.0, 3.0, 4.0]], [[4.0, 2.0, 7.0], 
[7.0, 1.0, 8.0], [7.0, 6.0, 5.0]], [[7.0, 7.0, 3.0], [7.0, 4.0, 6.0], [7.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RUUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 4.0, 4.0], [8.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], [8.0, 5.0, 8.0], [1.0, 
2.0, 6.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 6.0], [8.0, 4.0, 2.0]]], [[[5.0, 8.0, 3.0], [6.0, 1.0, 2.0], [8.0, 6.0, 4.0]], [[3.0, 
8.0, 1.0], [3.0, 1.0, 7.0], [4.0, 3.0, 8.0]], [[6.0, 4.0, 5.0], [2.0, 2.0, 5.0], [7.0, 7.0, 6.0]]], [[[2.0, 8.0, 3.0], [6.0, 
3.0, 4.0], [3.0, 7.0, 3.0]], [[6.0, 2.0, 1.0], [6.0, 7.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 5.0], [1.0, 1.0, 7.0], [5.0, 8.0, 
7.0]]], [[[6.0, 1.0, 4.0], [6.0, 2.0, 8.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 3.0], [7.0, 4.0, 4.0], [1.0, 4.0, 1.0]], [[2.0, 7.0, 
8.0], [7.0, 8.0, 3.0], [7.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 8.0], [6.0, 3.0, 6.0], [3.0, 8.0, 1.0]], 
[[8.0, 3.0, 7.0], [8.0, 4.0, 3.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 8.0], [1.0, 4.0, 1.0], [5.0, 2.0, 7.0]]], [[[1.0, 1.0, 5.0], 
[1.0, 6.0, 4.0], [7.0, 4.0, 3.0]], [[7.0, 3.0, 6.0], [1.0, 6.0, 4.0], [5.0, 5.0, 5.0]], [[6.0, 4.0, 2.0], [8.0, 4.0, 1.0], 
[3.0, 7.0, 7.0]]], [[[3.0, 4.0, 3.0], [8.0, 3.0, 5.0], [6.0, 5.0, 7.0]], [[3.0, 1.0, 4.0], [1.0, 6.0, 1.0], [8.0, 7.0, 4.0]], 
[[6.0, 5.0, 5.0], [1.0, 1.0, 2.0], [3.0, 4.0, 1.0]]], [[[2.0, 6.0, 2.0], [7.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[3.0, 2.0, 3.0], 
[4.0, 5.0, 4.0], [1.0, 3.0, 3.0]], [[4.0, 1.0, 5.0], [8.0, 2.0, 7.0], [6.0, 4.0, 5.0]]]])
      val=numpy.array([[[[5.0, 6.0, 2.0], [3.0, 6.0, 6.0]], [[8.0, 2.0, 5.0], [6.0, 6.0, 3.0]]], [[[3.0, 7.0, 3.0], [5.0, 8.0, 
6.0]], [[5.0, 8.0, 1.0], [1.0, 2.0, 1.0]]]])
      arg[1:3,:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 4.0, 4.0], [8.0, 8.0, 8.0]], [[2.0, 4.0, 4.0], [8.0, 5.0, 8.0], [1.0, 
2.0, 6.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 6.0], [8.0, 4.0, 2.0]]], [[[5.0, 6.0, 2.0], [3.0, 6.0, 6.0], [8.0, 6.0, 4.0]], [[8.0, 
2.0, 5.0], [6.0, 6.0, 3.0], [4.0, 3.0, 8.0]], [[6.0, 4.0, 5.0], [2.0, 2.0, 5.0], [7.0, 7.0, 6.0]]], [[[3.0, 7.0, 3.0], [5.0, 
8.0, 6.0], [3.0, 7.0, 3.0]], [[5.0, 8.0, 1.0], [1.0, 2.0, 1.0], [5.0, 3.0, 2.0]], [[6.0, 2.0, 5.0], [1.0, 1.0, 7.0], [5.0, 8.0, 
7.0]]], [[[6.0, 1.0, 4.0], [6.0, 2.0, 8.0], [6.0, 4.0, 6.0]], [[5.0, 3.0, 3.0], [7.0, 4.0, 4.0], [1.0, 4.0, 1.0]], [[2.0, 7.0, 
8.0], [7.0, 8.0, 3.0], [7.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 8.0], [6.0, 3.0, 6.0], [3.0, 8.0, 1.0]], 
[[8.0, 3.0, 7.0], [8.0, 4.0, 3.0], [4.0, 7.0, 5.0]], [[2.0, 1.0, 8.0], [1.0, 4.0, 1.0], [5.0, 2.0, 7.0]]], [[[5.0, 6.0, 2.0], 
[3.0, 6.0, 6.0], [7.0, 4.0, 3.0]], [[8.0, 2.0, 5.0], [6.0, 6.0, 3.0], [5.0, 5.0, 5.0]], [[6.0, 4.0, 2.0], [8.0, 4.0, 1.0], 
[3.0, 7.0, 7.0]]], [[[3.0, 7.0, 3.0], [5.0, 8.0, 6.0], [6.0, 5.0, 7.0]], [[5.0, 8.0, 1.0], [1.0, 2.0, 1.0], [8.0, 7.0, 4.0]], 
[[6.0, 5.0, 5.0], [1.0, 1.0, 2.0], [3.0, 4.0, 1.0]]], [[[2.0, 6.0, 2.0], [7.0, 6.0, 1.0], [3.0, 4.0, 1.0]], [[3.0, 2.0, 3.0], 
[4.0, 5.0, 4.0], [1.0, 3.0, 3.0]], [[4.0, 1.0, 5.0], [8.0, 2.0, 7.0], [6.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RUUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 6.0], [8.0, 6.0, 3.0], [7.0, 8.0, 3.0]], [[3.0, 6.0, 3.0], [5.0, 1.0, 3.0], [2.0, 
5.0, 4.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 8.0], [2.0, 6.0, 7.0]]], [[[8.0, 4.0, 2.0], [3.0, 7.0, 6.0], [6.0, 3.0, 2.0]], [[3.0, 
1.0, 6.0], [5.0, 8.0, 8.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 3.0], [5.0, 8.0, 6.0]]], [[[1.0, 4.0, 2.0], [8.0, 
5.0, 1.0], [7.0, 2.0, 2.0]], [[7.0, 4.0, 2.0], [5.0, 8.0, 2.0], [6.0, 8.0, 5.0]], [[8.0, 7.0, 5.0], [3.0, 5.0, 7.0], [8.0, 7.0, 
4.0]]], [[[4.0, 1.0, 3.0], [7.0, 3.0, 7.0], [6.0, 3.0, 8.0]], [[7.0, 7.0, 5.0], [6.0, 7.0, 8.0], [8.0, 6.0, 3.0]], [[5.0, 3.0, 
6.0], [4.0, 8.0, 3.0], [8.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 6.0], [8.0, 7.0, 5.0], [1.0, 2.0, 3.0]], 
[[6.0, 3.0, 4.0], [8.0, 5.0, 5.0], [6.0, 5.0, 5.0]], [[6.0, 1.0, 4.0], [4.0, 6.0, 2.0], [8.0, 4.0, 5.0]]], [[[5.0, 8.0, 6.0], 
[3.0, 1.0, 5.0], [6.0, 5.0, 3.0]], [[3.0, 1.0, 8.0], [2.0, 5.0, 6.0], [7.0, 1.0, 2.0]], [[3.0, 2.0, 7.0], [2.0, 6.0, 4.0], 
[7.0, 3.0, 1.0]]], [[[5.0, 6.0, 5.0], [1.0, 1.0, 2.0], [2.0, 7.0, 4.0]], [[4.0, 4.0, 6.0], [1.0, 3.0, 1.0], [3.0, 7.0, 2.0]], 
[[8.0, 1.0, 2.0], [4.0, 6.0, 7.0], [5.0, 2.0, 4.0]]], [[[8.0, 3.0, 7.0], [3.0, 6.0, 3.0], [4.0, 3.0, 1.0]], [[6.0, 1.0, 4.0], 
[3.0, 7.0, 6.0], [4.0, 1.0, 8.0]], [[6.0, 3.0, 7.0], [7.0, 7.0, 1.0], [7.0, 2.0, 1.0]]]])
      val=Data(numpy.array([[[[5.0, 4.0, 5.0], [8.0, 4.0, 1.0]], [[6.0, 5.0, 4.0], [7.0, 7.0, 4.0]]], [[[1.0, 5.0, 6.0], [6.0, 
7.0, 3.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 1.0]]]]),self.functionspace)
      arg[1:3,:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 6.0], [8.0, 6.0, 3.0], [7.0, 8.0, 3.0]], [[3.0, 6.0, 3.0], [5.0, 1.0, 3.0], [2.0, 
5.0, 4.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 8.0], [2.0, 6.0, 7.0]]], [[[5.0, 4.0, 5.0], [8.0, 4.0, 1.0], [6.0, 3.0, 2.0]], [[6.0, 
5.0, 4.0], [7.0, 7.0, 4.0], [6.0, 8.0, 7.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 3.0], [5.0, 8.0, 6.0]]], [[[1.0, 5.0, 6.0], [6.0, 
7.0, 3.0], [7.0, 2.0, 2.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 1.0], [6.0, 8.0, 5.0]], [[8.0, 7.0, 5.0], [3.0, 5.0, 7.0], [8.0, 7.0, 
4.0]]], [[[4.0, 1.0, 3.0], [7.0, 3.0, 7.0], [6.0, 3.0, 8.0]], [[7.0, 7.0, 5.0], [6.0, 7.0, 8.0], [8.0, 6.0, 3.0]], [[5.0, 3.0, 
6.0], [4.0, 8.0, 3.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 6.0], [8.0, 7.0, 5.0], [1.0, 2.0, 3.0]], 
[[6.0, 3.0, 4.0], [8.0, 5.0, 5.0], [6.0, 5.0, 5.0]], [[6.0, 1.0, 4.0], [4.0, 6.0, 2.0], [8.0, 4.0, 5.0]]], [[[5.0, 4.0, 5.0], 
[8.0, 4.0, 1.0], [6.0, 5.0, 3.0]], [[6.0, 5.0, 4.0], [7.0, 7.0, 4.0], [7.0, 1.0, 2.0]], [[3.0, 2.0, 7.0], [2.0, 6.0, 4.0], 
[7.0, 3.0, 1.0]]], [[[1.0, 5.0, 6.0], [6.0, 7.0, 3.0], [2.0, 7.0, 4.0]], [[3.0, 8.0, 1.0], [7.0, 4.0, 1.0], [3.0, 7.0, 2.0]], 
[[8.0, 1.0, 2.0], [4.0, 6.0, 7.0], [5.0, 2.0, 4.0]]], [[[8.0, 3.0, 7.0], [3.0, 6.0, 3.0], [4.0, 3.0, 1.0]], [[6.0, 1.0, 4.0], 
[3.0, 7.0, 6.0], [4.0, 1.0, 8.0]], [[6.0, 3.0, 7.0], [7.0, 7.0, 1.0], [7.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RUUC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 5.0], [2.0, 7.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 1.0], [3.0, 
4.0, 5.0]], [[4.0, 2.0, 2.0], [5.0, 2.0, 2.0], [1.0, 3.0, 3.0]]], [[[6.0, 6.0, 1.0], [4.0, 4.0, 6.0], [6.0, 7.0, 2.0]], [[2.0, 
7.0, 7.0], [3.0, 3.0, 4.0], [2.0, 7.0, 1.0]], [[4.0, 5.0, 2.0], [2.0, 1.0, 7.0], [6.0, 2.0, 5.0]]], [[[3.0, 3.0, 5.0], [5.0, 
2.0, 5.0], [5.0, 8.0, 6.0]], [[7.0, 7.0, 4.0], [7.0, 4.0, 1.0], [5.0, 8.0, 3.0]], [[1.0, 7.0, 4.0], [4.0, 4.0, 8.0], [3.0, 8.0, 
1.0]]], [[[7.0, 7.0, 2.0], [5.0, 1.0, 4.0], [5.0, 8.0, 6.0]], [[5.0, 7.0, 2.0], [1.0, 2.0, 8.0], [8.0, 1.0, 4.0]], [[2.0, 1.0, 
8.0], [6.0, 2.0, 7.0], [4.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 5.0], [5.0, 3.0, 3.0], [2.0, 4.0, 7.0]], 
[[6.0, 4.0, 2.0], [5.0, 5.0, 1.0], [1.0, 3.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 6.0, 6.0], [8.0, 2.0, 1.0]]], [[[5.0, 7.0, 1.0], 
[2.0, 8.0, 6.0], [8.0, 8.0, 1.0]], [[5.0, 4.0, 4.0], [2.0, 3.0, 4.0], [2.0, 7.0, 2.0]], [[6.0, 2.0, 5.0], [2.0, 4.0, 2.0], 
[2.0, 3.0, 1.0]]], [[[1.0, 8.0, 1.0], [4.0, 4.0, 5.0], [3.0, 5.0, 2.0]], [[7.0, 1.0, 5.0], [1.0, 1.0, 2.0], [1.0, 6.0, 8.0]], 
[[8.0, 6.0, 4.0], [3.0, 6.0, 8.0], [2.0, 5.0, 1.0]]], [[[5.0, 7.0, 1.0], [4.0, 7.0, 5.0], [2.0, 7.0, 2.0]], [[1.0, 5.0, 1.0], 
[7.0, 4.0, 1.0], [6.0, 3.0, 4.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 5.0], [6.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 2.0, 5.0], [3.0, 5.0, 8.0]], [[8.0, 8.0, 7.0], [2.0, 8.0, 7.0]]], [[[3.0, 8.0, 3.0], 
[1.0, 5.0, 2.0]], [[4.0, 5.0, 1.0], [1.0, 2.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 8.0, 6.0], [7.0, 8.0, 2.0]], [[8.0, 
1.0, 2.0], [6.0, 7.0, 7.0]]], [[[6.0, 1.0, 4.0], [8.0, 5.0, 7.0]], [[4.0, 4.0, 3.0], [7.0, 4.0, 6.0]]]])
      arg[1:3,:2,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 5.0], [2.0, 7.0, 7.0], [7.0, 1.0, 5.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 1.0], [3.0, 
4.0, 5.0]], [[4.0, 2.0, 2.0], [5.0, 2.0, 2.0], [1.0, 3.0, 3.0]]], [[[1.0, 2.0, 5.0], [3.0, 5.0, 8.0], [6.0, 7.0, 2.0]], [[8.0, 
8.0, 7.0], [2.0, 8.0, 7.0], [2.0, 7.0, 1.0]], [[4.0, 5.0, 2.0], [2.0, 1.0, 7.0], [6.0, 2.0, 5.0]]], [[[3.0, 8.0, 3.0], [1.0, 
5.0, 2.0], [5.0, 8.0, 6.0]], [[4.0, 5.0, 1.0], [1.0, 2.0, 6.0], [5.0, 8.0, 3.0]], [[1.0, 7.0, 4.0], [4.0, 4.0, 8.0], [3.0, 8.0, 
1.0]]], [[[7.0, 7.0, 2.0], [5.0, 1.0, 4.0], [5.0, 8.0, 6.0]], [[5.0, 7.0, 2.0], [1.0, 2.0, 8.0], [8.0, 1.0, 4.0]], [[2.0, 1.0, 
8.0], [6.0, 2.0, 7.0], [4.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 5.0], [5.0, 3.0, 3.0], [2.0, 4.0, 7.0]], 
[[6.0, 4.0, 2.0], [5.0, 5.0, 1.0], [1.0, 3.0, 5.0]], [[4.0, 5.0, 4.0], [4.0, 6.0, 6.0], [8.0, 2.0, 1.0]]], [[[8.0, 8.0, 6.0], 
[7.0, 8.0, 2.0], [8.0, 8.0, 1.0]], [[8.0, 1.0, 2.0], [6.0, 7.0, 7.0], [2.0, 7.0, 2.0]], [[6.0, 2.0, 5.0], [2.0, 4.0, 2.0], 
[2.0, 3.0, 1.0]]], [[[6.0, 1.0, 4.0], [8.0, 5.0, 7.0], [3.0, 5.0, 2.0]], [[4.0, 4.0, 3.0], [7.0, 4.0, 6.0], [1.0, 6.0, 8.0]], 
[[8.0, 6.0, 4.0], [3.0, 6.0, 8.0], [2.0, 5.0, 1.0]]], [[[5.0, 7.0, 1.0], [4.0, 7.0, 5.0], [2.0, 7.0, 2.0]], [[1.0, 5.0, 1.0], 
[7.0, 4.0, 1.0], [6.0, 3.0, 4.0]], [[8.0, 4.0, 4.0], [2.0, 8.0, 5.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 8.0], [1.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 4.0], [4.0, 7.0, 7.0], [7.0, 
5.0, 4.0]], [[3.0, 2.0, 4.0], [8.0, 2.0, 6.0], [1.0, 6.0, 6.0]]], [[[8.0, 3.0, 5.0], [7.0, 8.0, 6.0], [1.0, 8.0, 3.0]], [[3.0, 
8.0, 4.0], [5.0, 3.0, 7.0], [7.0, 7.0, 1.0]], [[3.0, 1.0, 1.0], [4.0, 5.0, 2.0], [1.0, 8.0, 6.0]]], [[[2.0, 6.0, 6.0], [3.0, 
4.0, 2.0], [8.0, 6.0, 5.0]], [[5.0, 4.0, 4.0], [8.0, 6.0, 4.0], [6.0, 3.0, 2.0]], [[8.0, 7.0, 4.0], [2.0, 2.0, 6.0], [1.0, 4.0, 
4.0]]], [[[1.0, 1.0, 7.0], [6.0, 2.0, 3.0], [8.0, 7.0, 1.0]], [[1.0, 2.0, 5.0], [8.0, 5.0, 6.0], [7.0, 5.0, 7.0]], [[5.0, 2.0, 
6.0], [1.0, 8.0, 4.0], [8.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 2.0], [6.0, 4.0, 7.0], [8.0, 1.0, 5.0]], 
[[1.0, 1.0, 1.0], [1.0, 5.0, 1.0], [7.0, 4.0, 2.0]], [[6.0, 1.0, 5.0], [5.0, 4.0, 8.0], [8.0, 3.0, 6.0]]], [[[5.0, 4.0, 6.0], 
[7.0, 7.0, 6.0], [6.0, 8.0, 5.0]], [[7.0, 3.0, 8.0], [3.0, 7.0, 1.0], [6.0, 8.0, 8.0]], [[2.0, 7.0, 7.0], [5.0, 5.0, 1.0], 
[5.0, 3.0, 4.0]]], [[[7.0, 5.0, 8.0], [8.0, 1.0, 7.0], [2.0, 5.0, 4.0]], [[4.0, 5.0, 2.0], [7.0, 8.0, 5.0], [3.0, 5.0, 3.0]], 
[[3.0, 3.0, 5.0], [6.0, 8.0, 7.0], [6.0, 3.0, 3.0]]], [[[6.0, 8.0, 7.0], [7.0, 6.0, 7.0], [3.0, 2.0, 4.0]], [[3.0, 1.0, 3.0], 
[8.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[7.0, 8.0, 6.0], [6.0, 5.0, 5.0], [1.0, 2.0, 7.0]]]])
      val=numpy.array([[[7.0, 6.0, 2.0], [3.0, 7.0, 4.0]]])
      arg[2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 8.0], [1.0, 1.0, 2.0], [2.0, 2.0, 5.0]], [[4.0, 6.0, 4.0], [4.0, 7.0, 7.0], [7.0, 
5.0, 4.0]], [[3.0, 2.0, 4.0], [8.0, 2.0, 6.0], [1.0, 6.0, 6.0]]], [[[8.0, 3.0, 5.0], [7.0, 8.0, 6.0], [1.0, 8.0, 3.0]], [[3.0, 
8.0, 4.0], [5.0, 3.0, 7.0], [7.0, 7.0, 1.0]], [[3.0, 1.0, 1.0], [4.0, 5.0, 2.0], [1.0, 8.0, 6.0]]], [[[2.0, 6.0, 6.0], [3.0, 
4.0, 2.0], [8.0, 6.0, 5.0]], [[5.0, 4.0, 4.0], [8.0, 6.0, 4.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 2.0], [3.0, 7.0, 4.0], [1.0, 4.0, 
4.0]]], [[[1.0, 1.0, 7.0], [6.0, 2.0, 3.0], [8.0, 7.0, 1.0]], [[1.0, 2.0, 5.0], [8.0, 5.0, 6.0], [7.0, 5.0, 7.0]], [[5.0, 2.0, 
6.0], [1.0, 8.0, 4.0], [8.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 2.0], [6.0, 4.0, 7.0], [8.0, 1.0, 5.0]], 
[[1.0, 1.0, 1.0], [1.0, 5.0, 1.0], [7.0, 4.0, 2.0]], [[6.0, 1.0, 5.0], [5.0, 4.0, 8.0], [8.0, 3.0, 6.0]]], [[[5.0, 4.0, 6.0], 
[7.0, 7.0, 6.0], [6.0, 8.0, 5.0]], [[7.0, 3.0, 8.0], [3.0, 7.0, 1.0], [6.0, 8.0, 8.0]], [[2.0, 7.0, 7.0], [5.0, 5.0, 1.0], 
[5.0, 3.0, 4.0]]], [[[7.0, 5.0, 8.0], [8.0, 1.0, 7.0], [2.0, 5.0, 4.0]], [[4.0, 5.0, 2.0], [7.0, 8.0, 5.0], [3.0, 5.0, 3.0]], 
[[7.0, 6.0, 2.0], [3.0, 7.0, 4.0], [6.0, 3.0, 3.0]]], [[[6.0, 8.0, 7.0], [7.0, 6.0, 7.0], [3.0, 2.0, 4.0]], [[3.0, 1.0, 3.0], 
[8.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[7.0, 8.0, 6.0], [6.0, 5.0, 5.0], [1.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 2.0], [8.0, 6.0, 8.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 6.0], [1.0, 7.0, 2.0], [2.0, 
4.0, 1.0]], [[4.0, 7.0, 4.0], [4.0, 8.0, 4.0], [4.0, 5.0, 7.0]]], [[[7.0, 1.0, 8.0], [3.0, 2.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 
1.0, 7.0], [8.0, 4.0, 8.0], [1.0, 7.0, 2.0]], [[3.0, 4.0, 6.0], [3.0, 6.0, 2.0], [5.0, 1.0, 7.0]]], [[[7.0, 6.0, 7.0], [7.0, 
6.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 6.0, 3.0], [2.0, 7.0, 7.0], [5.0, 7.0, 3.0]], [[2.0, 5.0, 8.0], [7.0, 7.0, 5.0], [4.0, 4.0, 
7.0]]], [[[3.0, 3.0, 7.0], [3.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[8.0, 6.0, 2.0], [1.0, 1.0, 6.0], [6.0, 2.0, 6.0]], [[8.0, 5.0, 
1.0], [6.0, 8.0, 5.0], [6.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 1.0], [7.0, 2.0, 8.0], [7.0, 5.0, 1.0]], 
[[4.0, 6.0, 8.0], [1.0, 6.0, 4.0], [6.0, 5.0, 5.0]], [[7.0, 6.0, 3.0], [4.0, 3.0, 2.0], [3.0, 8.0, 1.0]]], [[[7.0, 5.0, 5.0], 
[3.0, 8.0, 7.0], [6.0, 6.0, 6.0]], [[1.0, 7.0, 4.0], [2.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 8.0, 3.0], 
[4.0, 7.0, 3.0]]], [[[5.0, 8.0, 2.0], [2.0, 6.0, 5.0], [3.0, 1.0, 3.0]], [[8.0, 6.0, 3.0], [8.0, 4.0, 6.0], [2.0, 2.0, 2.0]], 
[[6.0, 8.0, 7.0], [1.0, 4.0, 1.0], [7.0, 7.0, 1.0]]], [[[7.0, 1.0, 8.0], [7.0, 3.0, 3.0], [5.0, 3.0, 4.0]], [[7.0, 4.0, 3.0], 
[6.0, 6.0, 3.0], [5.0, 4.0, 5.0]], [[1.0, 8.0, 8.0], [4.0, 2.0, 4.0], [7.0, 4.0, 8.0]]]])
      val=Data(numpy.array([[[7.0, 5.0, 3.0], [5.0, 7.0, 4.0]]]),self.functionspace)
      arg[2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 2.0], [8.0, 6.0, 8.0], [2.0, 5.0, 1.0]], [[2.0, 7.0, 6.0], [1.0, 7.0, 2.0], [2.0, 
4.0, 1.0]], [[4.0, 7.0, 4.0], [4.0, 8.0, 4.0], [4.0, 5.0, 7.0]]], [[[7.0, 1.0, 8.0], [3.0, 2.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 
1.0, 7.0], [8.0, 4.0, 8.0], [1.0, 7.0, 2.0]], [[3.0, 4.0, 6.0], [3.0, 6.0, 2.0], [5.0, 1.0, 7.0]]], [[[7.0, 6.0, 7.0], [7.0, 
6.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 6.0, 3.0], [2.0, 7.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 5.0, 3.0], [5.0, 7.0, 4.0], [4.0, 4.0, 
7.0]]], [[[3.0, 3.0, 7.0], [3.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[8.0, 6.0, 2.0], [1.0, 1.0, 6.0], [6.0, 2.0, 6.0]], [[8.0, 5.0, 
1.0], [6.0, 8.0, 5.0], [6.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 1.0], [7.0, 2.0, 8.0], [7.0, 5.0, 1.0]], 
[[4.0, 6.0, 8.0], [1.0, 6.0, 4.0], [6.0, 5.0, 5.0]], [[7.0, 6.0, 3.0], [4.0, 3.0, 2.0], [3.0, 8.0, 1.0]]], [[[7.0, 5.0, 5.0], 
[3.0, 8.0, 7.0], [6.0, 6.0, 6.0]], [[1.0, 7.0, 4.0], [2.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[5.0, 7.0, 3.0], [7.0, 8.0, 3.0], 
[4.0, 7.0, 3.0]]], [[[5.0, 8.0, 2.0], [2.0, 6.0, 5.0], [3.0, 1.0, 3.0]], [[8.0, 6.0, 3.0], [8.0, 4.0, 6.0], [2.0, 2.0, 2.0]], 
[[7.0, 5.0, 3.0], [5.0, 7.0, 4.0], [7.0, 7.0, 1.0]]], [[[7.0, 1.0, 8.0], [7.0, 3.0, 3.0], [5.0, 3.0, 4.0]], [[7.0, 4.0, 3.0], 
[6.0, 6.0, 3.0], [5.0, 4.0, 5.0]], [[1.0, 8.0, 8.0], [4.0, 2.0, 4.0], [7.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 5.0], [8.0, 3.0, 5.0], [3.0, 3.0, 7.0]], [[5.0, 6.0, 7.0], [6.0, 7.0, 8.0], [7.0, 
3.0, 8.0]], [[2.0, 7.0, 1.0], [4.0, 4.0, 3.0], [4.0, 5.0, 1.0]]], [[[3.0, 5.0, 8.0], [3.0, 3.0, 8.0], [3.0, 8.0, 4.0]], [[7.0, 
5.0, 1.0], [7.0, 6.0, 6.0], [2.0, 6.0, 8.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 1.0], [5.0, 3.0, 2.0]]], [[[8.0, 7.0, 3.0], [2.0, 
2.0, 6.0], [4.0, 8.0, 5.0]], [[8.0, 8.0, 2.0], [4.0, 7.0, 8.0], [8.0, 5.0, 4.0]], [[6.0, 8.0, 1.0], [5.0, 3.0, 4.0], [6.0, 8.0, 
7.0]]], [[[1.0, 2.0, 5.0], [4.0, 3.0, 8.0], [3.0, 7.0, 1.0]], [[2.0, 2.0, 5.0], [3.0, 1.0, 1.0], [6.0, 4.0, 8.0]], [[4.0, 6.0, 
4.0], [3.0, 1.0, 4.0], [8.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 7.0], [1.0, 8.0, 1.0], [1.0, 7.0, 4.0]], 
[[1.0, 8.0, 7.0], [3.0, 5.0, 3.0], [2.0, 2.0, 4.0]], [[5.0, 7.0, 3.0], [1.0, 6.0, 3.0], [4.0, 5.0, 8.0]]], [[[4.0, 5.0, 8.0], 
[6.0, 1.0, 7.0], [8.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [4.0, 3.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 1.0, 5.0], [4.0, 5.0, 2.0], 
[2.0, 4.0, 8.0]]], [[[6.0, 6.0, 5.0], [7.0, 8.0, 4.0], [3.0, 2.0, 4.0]], [[6.0, 1.0, 2.0], [8.0, 3.0, 4.0], [7.0, 1.0, 3.0]], 
[[1.0, 6.0, 3.0], [5.0, 3.0, 5.0], [4.0, 4.0, 7.0]]], [[[5.0, 8.0, 5.0], [3.0, 5.0, 7.0], [5.0, 4.0, 6.0]], [[2.0, 3.0, 3.0], 
[3.0, 1.0, 5.0], [2.0, 4.0, 6.0]], [[1.0, 2.0, 7.0], [7.0, 6.0, 1.0], [5.0, 1.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 3.0, 2.0], [3.0, 7.0, 2.0]]])+(1.-msk_val)*numpy.array([[[6.0, 1.0, 4.0], [2.0, 2.0, 
5.0]]])
      arg[2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 5.0], [8.0, 3.0, 5.0], [3.0, 3.0, 7.0]], [[5.0, 6.0, 7.0], [6.0, 7.0, 8.0], [7.0, 
3.0, 8.0]], [[2.0, 7.0, 1.0], [4.0, 4.0, 3.0], [4.0, 5.0, 1.0]]], [[[3.0, 5.0, 8.0], [3.0, 3.0, 8.0], [3.0, 8.0, 4.0]], [[7.0, 
5.0, 1.0], [7.0, 6.0, 6.0], [2.0, 6.0, 8.0]], [[8.0, 5.0, 6.0], [2.0, 7.0, 1.0], [5.0, 3.0, 2.0]]], [[[8.0, 7.0, 3.0], [2.0, 
2.0, 6.0], [4.0, 8.0, 5.0]], [[8.0, 8.0, 2.0], [4.0, 7.0, 8.0], [8.0, 5.0, 4.0]], [[8.0, 3.0, 2.0], [3.0, 7.0, 2.0], [6.0, 8.0, 
7.0]]], [[[1.0, 2.0, 5.0], [4.0, 3.0, 8.0], [3.0, 7.0, 1.0]], [[2.0, 2.0, 5.0], [3.0, 1.0, 1.0], [6.0, 4.0, 8.0]], [[4.0, 6.0, 
4.0], [3.0, 1.0, 4.0], [8.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 7.0], [1.0, 8.0, 1.0], [1.0, 7.0, 4.0]], 
[[1.0, 8.0, 7.0], [3.0, 5.0, 3.0], [2.0, 2.0, 4.0]], [[5.0, 7.0, 3.0], [1.0, 6.0, 3.0], [4.0, 5.0, 8.0]]], [[[4.0, 5.0, 8.0], 
[6.0, 1.0, 7.0], [8.0, 5.0, 2.0]], [[5.0, 3.0, 2.0], [4.0, 3.0, 1.0], [2.0, 8.0, 2.0]], [[7.0, 1.0, 5.0], [4.0, 5.0, 2.0], 
[2.0, 4.0, 8.0]]], [[[6.0, 6.0, 5.0], [7.0, 8.0, 4.0], [3.0, 2.0, 4.0]], [[6.0, 1.0, 2.0], [8.0, 3.0, 4.0], [7.0, 1.0, 3.0]], 
[[6.0, 1.0, 4.0], [2.0, 2.0, 5.0], [4.0, 4.0, 7.0]]], [[[5.0, 8.0, 5.0], [3.0, 5.0, 7.0], [5.0, 4.0, 6.0]], [[2.0, 3.0, 3.0], 
[3.0, 1.0, 5.0], [2.0, 4.0, 6.0]], [[1.0, 2.0, 7.0], [7.0, 6.0, 1.0], [5.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 6.0], [1.0, 3.0, 7.0], [3.0, 3.0, 8.0]], [[6.0, 6.0, 5.0], [7.0, 8.0, 5.0], [6.0, 
8.0, 5.0]], [[1.0, 4.0, 8.0], [2.0, 6.0, 8.0], [6.0, 3.0, 8.0]]], [[[1.0, 3.0, 6.0], [1.0, 6.0, 7.0], [8.0, 3.0, 2.0]], [[7.0, 
6.0, 6.0], [5.0, 3.0, 4.0], [6.0, 1.0, 8.0]], [[7.0, 7.0, 2.0], [7.0, 1.0, 1.0], [2.0, 8.0, 5.0]]], [[[7.0, 6.0, 1.0], [8.0, 
6.0, 6.0], [7.0, 2.0, 6.0]], [[8.0, 4.0, 4.0], [1.0, 1.0, 5.0], [5.0, 5.0, 2.0]], [[4.0, 1.0, 4.0], [2.0, 3.0, 1.0], [5.0, 6.0, 
2.0]]], [[[7.0, 7.0, 2.0], [8.0, 5.0, 2.0], [4.0, 3.0, 7.0]], [[3.0, 3.0, 5.0], [1.0, 7.0, 8.0], [8.0, 5.0, 5.0]], [[7.0, 5.0, 
3.0], [3.0, 6.0, 8.0], [2.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 4.0], [3.0, 1.0, 1.0], [7.0, 2.0, 7.0]], 
[[4.0, 3.0, 3.0], [7.0, 4.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 4.0, 2.0], [3.0, 4.0, 8.0], [7.0, 3.0, 8.0]]], [[[4.0, 2.0, 3.0], 
[2.0, 5.0, 6.0], [2.0, 8.0, 1.0]], [[1.0, 8.0, 3.0], [7.0, 8.0, 3.0], [8.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [8.0, 6.0, 1.0], 
[8.0, 2.0, 6.0]]], [[[6.0, 2.0, 6.0], [6.0, 3.0, 2.0], [8.0, 3.0, 1.0]], [[1.0, 3.0, 1.0], [6.0, 7.0, 1.0], [1.0, 5.0, 7.0]], 
[[5.0, 5.0, 8.0], [8.0, 1.0, 4.0], [2.0, 3.0, 4.0]]], [[[4.0, 2.0, 3.0], [2.0, 3.0, 7.0], [5.0, 2.0, 3.0]], [[2.0, 6.0, 8.0], 
[5.0, 6.0, 1.0], [5.0, 7.0, 1.0]], [[5.0, 1.0, 6.0], [7.0, 6.0, 7.0], [7.0, 5.0, 1.0]]]])
      val=numpy.array([[[4.0, 2.0, 6.0], [2.0, 1.0, 8.0]], [[7.0, 5.0, 2.0], [1.0, 5.0, 8.0]]])
      arg[2:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 6.0], [1.0, 3.0, 7.0], [3.0, 3.0, 8.0]], [[6.0, 6.0, 5.0], [7.0, 8.0, 5.0], [6.0, 
8.0, 5.0]], [[1.0, 4.0, 8.0], [2.0, 6.0, 8.0], [6.0, 3.0, 8.0]]], [[[1.0, 3.0, 6.0], [1.0, 6.0, 7.0], [8.0, 3.0, 2.0]], [[7.0, 
6.0, 6.0], [5.0, 3.0, 4.0], [6.0, 1.0, 8.0]], [[7.0, 7.0, 2.0], [7.0, 1.0, 1.0], [2.0, 8.0, 5.0]]], [[[7.0, 6.0, 1.0], [8.0, 
6.0, 6.0], [7.0, 2.0, 6.0]], [[4.0, 2.0, 6.0], [2.0, 1.0, 8.0], [5.0, 5.0, 2.0]], [[4.0, 1.0, 4.0], [2.0, 3.0, 1.0], [5.0, 6.0, 
2.0]]], [[[7.0, 7.0, 2.0], [8.0, 5.0, 2.0], [4.0, 3.0, 7.0]], [[7.0, 5.0, 2.0], [1.0, 5.0, 8.0], [8.0, 5.0, 5.0]], [[7.0, 5.0, 
3.0], [3.0, 6.0, 8.0], [2.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 4.0], [3.0, 1.0, 1.0], [7.0, 2.0, 7.0]], 
[[4.0, 3.0, 3.0], [7.0, 4.0, 5.0], [4.0, 1.0, 8.0]], [[1.0, 4.0, 2.0], [3.0, 4.0, 8.0], [7.0, 3.0, 8.0]]], [[[4.0, 2.0, 3.0], 
[2.0, 5.0, 6.0], [2.0, 8.0, 1.0]], [[1.0, 8.0, 3.0], [7.0, 8.0, 3.0], [8.0, 3.0, 7.0]], [[1.0, 1.0, 8.0], [8.0, 6.0, 1.0], 
[8.0, 2.0, 6.0]]], [[[6.0, 2.0, 6.0], [6.0, 3.0, 2.0], [8.0, 3.0, 1.0]], [[4.0, 2.0, 6.0], [2.0, 1.0, 8.0], [1.0, 5.0, 7.0]], 
[[5.0, 5.0, 8.0], [8.0, 1.0, 4.0], [2.0, 3.0, 4.0]]], [[[4.0, 2.0, 3.0], [2.0, 3.0, 7.0], [5.0, 2.0, 3.0]], [[7.0, 5.0, 2.0], 
[1.0, 5.0, 8.0], [5.0, 7.0, 1.0]], [[5.0, 1.0, 6.0], [7.0, 6.0, 7.0], [7.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 3.0], [8.0, 7.0, 1.0], [2.0, 8.0, 1.0]], [[2.0, 2.0, 4.0], [1.0, 8.0, 7.0], [2.0, 
8.0, 8.0]], [[6.0, 2.0, 1.0], [4.0, 6.0, 6.0], [2.0, 6.0, 1.0]]], [[[2.0, 3.0, 6.0], [4.0, 3.0, 3.0], [8.0, 6.0, 8.0]], [[6.0, 
5.0, 5.0], [4.0, 7.0, 8.0], [1.0, 6.0, 7.0]], [[7.0, 1.0, 5.0], [6.0, 1.0, 7.0], [6.0, 7.0, 4.0]]], [[[5.0, 8.0, 1.0], [1.0, 
8.0, 7.0], [6.0, 4.0, 5.0]], [[2.0, 7.0, 7.0], [4.0, 8.0, 6.0], [2.0, 5.0, 7.0]], [[7.0, 6.0, 8.0], [3.0, 1.0, 4.0], [3.0, 8.0, 
8.0]]], [[[5.0, 3.0, 7.0], [5.0, 1.0, 7.0], [6.0, 8.0, 4.0]], [[8.0, 3.0, 5.0], [7.0, 1.0, 6.0], [1.0, 7.0, 5.0]], [[3.0, 3.0, 
7.0], [1.0, 8.0, 8.0], [8.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 5.0], [2.0, 5.0, 7.0], [8.0, 1.0, 1.0]], 
[[6.0, 8.0, 2.0], [3.0, 3.0, 3.0], [2.0, 6.0, 3.0]], [[4.0, 7.0, 7.0], [5.0, 2.0, 7.0], [7.0, 8.0, 3.0]]], [[[3.0, 1.0, 4.0], 
[8.0, 8.0, 2.0], [5.0, 7.0, 2.0]], [[6.0, 6.0, 7.0], [5.0, 3.0, 8.0], [6.0, 4.0, 6.0]], [[1.0, 3.0, 5.0], [6.0, 2.0, 8.0], 
[2.0, 2.0, 4.0]]], [[[4.0, 3.0, 6.0], [7.0, 3.0, 1.0], [3.0, 4.0, 2.0]], [[8.0, 3.0, 5.0], [8.0, 3.0, 6.0], [1.0, 5.0, 6.0]], 
[[2.0, 5.0, 6.0], [2.0, 1.0, 5.0], [5.0, 2.0, 6.0]]], [[[5.0, 1.0, 6.0], [2.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[5.0, 1.0, 2.0], 
[5.0, 2.0, 4.0], [8.0, 7.0, 4.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 7.0], [3.0, 4.0, 8.0]]]])
      val=Data(numpy.array([[[5.0, 8.0, 7.0], [3.0, 7.0, 7.0]], [[8.0, 7.0, 6.0], [8.0, 6.0, 2.0]]]),self.functionspace)
      arg[2:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 3.0], [8.0, 7.0, 1.0], [2.0, 8.0, 1.0]], [[2.0, 2.0, 4.0], [1.0, 8.0, 7.0], [2.0, 
8.0, 8.0]], [[6.0, 2.0, 1.0], [4.0, 6.0, 6.0], [2.0, 6.0, 1.0]]], [[[2.0, 3.0, 6.0], [4.0, 3.0, 3.0], [8.0, 6.0, 8.0]], [[6.0, 
5.0, 5.0], [4.0, 7.0, 8.0], [1.0, 6.0, 7.0]], [[7.0, 1.0, 5.0], [6.0, 1.0, 7.0], [6.0, 7.0, 4.0]]], [[[5.0, 8.0, 1.0], [1.0, 
8.0, 7.0], [6.0, 4.0, 5.0]], [[5.0, 8.0, 7.0], [3.0, 7.0, 7.0], [2.0, 5.0, 7.0]], [[7.0, 6.0, 8.0], [3.0, 1.0, 4.0], [3.0, 8.0, 
8.0]]], [[[5.0, 3.0, 7.0], [5.0, 1.0, 7.0], [6.0, 8.0, 4.0]], [[8.0, 7.0, 6.0], [8.0, 6.0, 2.0], [1.0, 7.0, 5.0]], [[3.0, 3.0, 
7.0], [1.0, 8.0, 8.0], [8.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 5.0], [2.0, 5.0, 7.0], [8.0, 1.0, 1.0]], 
[[6.0, 8.0, 2.0], [3.0, 3.0, 3.0], [2.0, 6.0, 3.0]], [[4.0, 7.0, 7.0], [5.0, 2.0, 7.0], [7.0, 8.0, 3.0]]], [[[3.0, 1.0, 4.0], 
[8.0, 8.0, 2.0], [5.0, 7.0, 2.0]], [[6.0, 6.0, 7.0], [5.0, 3.0, 8.0], [6.0, 4.0, 6.0]], [[1.0, 3.0, 5.0], [6.0, 2.0, 8.0], 
[2.0, 2.0, 4.0]]], [[[4.0, 3.0, 6.0], [7.0, 3.0, 1.0], [3.0, 4.0, 2.0]], [[5.0, 8.0, 7.0], [3.0, 7.0, 7.0], [1.0, 5.0, 6.0]], 
[[2.0, 5.0, 6.0], [2.0, 1.0, 5.0], [5.0, 2.0, 6.0]]], [[[5.0, 1.0, 6.0], [2.0, 8.0, 4.0], [3.0, 2.0, 6.0]], [[8.0, 7.0, 6.0], 
[8.0, 6.0, 2.0], [8.0, 7.0, 4.0]], [[5.0, 8.0, 3.0], [7.0, 6.0, 7.0], [3.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 3.0], [2.0, 5.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 8.0], [7.0, 1.0, 6.0], [1.0, 
1.0, 2.0]], [[2.0, 3.0, 3.0], [7.0, 5.0, 3.0], [4.0, 6.0, 8.0]]], [[[5.0, 5.0, 1.0], [6.0, 4.0, 6.0], [6.0, 3.0, 4.0]], [[5.0, 
2.0, 2.0], [4.0, 5.0, 7.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 3.0], [4.0, 6.0, 8.0], [1.0, 7.0, 6.0]]], [[[6.0, 5.0, 5.0], [3.0, 
2.0, 3.0], [7.0, 6.0, 6.0]], [[2.0, 8.0, 5.0], [8.0, 7.0, 1.0], [4.0, 1.0, 5.0]], [[5.0, 2.0, 7.0], [8.0, 5.0, 1.0], [2.0, 3.0, 
3.0]]], [[[2.0, 8.0, 8.0], [5.0, 8.0, 3.0], [3.0, 5.0, 6.0]], [[8.0, 8.0, 3.0], [8.0, 2.0, 3.0], [4.0, 7.0, 8.0]], [[2.0, 4.0, 
4.0], [5.0, 5.0, 4.0], [2.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 3.0], [8.0, 2.0, 6.0], [7.0, 3.0, 7.0]], 
[[1.0, 5.0, 7.0], [5.0, 4.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 3.0, 1.0], [6.0, 8.0, 4.0], [7.0, 1.0, 1.0]]], [[[2.0, 3.0, 6.0], 
[2.0, 8.0, 6.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 3.0], [8.0, 5.0, 1.0], [2.0, 8.0, 8.0]], [[8.0, 7.0, 1.0], [8.0, 8.0, 1.0], 
[4.0, 2.0, 1.0]]], [[[4.0, 2.0, 3.0], [7.0, 3.0, 1.0], [6.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [4.0, 5.0, 1.0], [1.0, 2.0, 7.0]], 
[[2.0, 8.0, 4.0], [3.0, 7.0, 3.0], [6.0, 8.0, 2.0]]], [[[4.0, 8.0, 3.0], [8.0, 5.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 1.0, 1.0], 
[1.0, 6.0, 6.0], [8.0, 8.0, 4.0]], [[4.0, 1.0, 1.0], [1.0, 4.0, 8.0], [1.0, 4.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 3.0, 6.0], [1.0, 7.0, 8.0]], [[7.0, 7.0, 5.0], [5.0, 5.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[1.0, 3.0, 7.0], [1.0, 2.0, 1.0]], [[6.0, 2.0, 4.0], [6.0, 8.0, 5.0]]])
      arg[2:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 3.0], [2.0, 5.0, 5.0], [2.0, 2.0, 4.0]], [[7.0, 7.0, 8.0], [7.0, 1.0, 6.0], [1.0, 
1.0, 2.0]], [[2.0, 3.0, 3.0], [7.0, 5.0, 3.0], [4.0, 6.0, 8.0]]], [[[5.0, 5.0, 1.0], [6.0, 4.0, 6.0], [6.0, 3.0, 4.0]], [[5.0, 
2.0, 2.0], [4.0, 5.0, 7.0], [5.0, 8.0, 2.0]], [[4.0, 5.0, 3.0], [4.0, 6.0, 8.0], [1.0, 7.0, 6.0]]], [[[6.0, 5.0, 5.0], [3.0, 
2.0, 3.0], [7.0, 6.0, 6.0]], [[6.0, 3.0, 6.0], [1.0, 7.0, 8.0], [4.0, 1.0, 5.0]], [[5.0, 2.0, 7.0], [8.0, 5.0, 1.0], [2.0, 3.0, 
3.0]]], [[[2.0, 8.0, 8.0], [5.0, 8.0, 3.0], [3.0, 5.0, 6.0]], [[7.0, 7.0, 5.0], [5.0, 5.0, 6.0], [4.0, 7.0, 8.0]], [[2.0, 4.0, 
4.0], [5.0, 5.0, 4.0], [2.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 3.0], [8.0, 2.0, 6.0], [7.0, 3.0, 7.0]], 
[[1.0, 5.0, 7.0], [5.0, 4.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 3.0, 1.0], [6.0, 8.0, 4.0], [7.0, 1.0, 1.0]]], [[[2.0, 3.0, 6.0], 
[2.0, 8.0, 6.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 3.0], [8.0, 5.0, 1.0], [2.0, 8.0, 8.0]], [[8.0, 7.0, 1.0], [8.0, 8.0, 1.0], 
[4.0, 2.0, 1.0]]], [[[4.0, 2.0, 3.0], [7.0, 3.0, 1.0], [6.0, 5.0, 4.0]], [[1.0, 3.0, 7.0], [1.0, 2.0, 1.0], [1.0, 2.0, 7.0]], 
[[2.0, 8.0, 4.0], [3.0, 7.0, 3.0], [6.0, 8.0, 2.0]]], [[[4.0, 8.0, 3.0], [8.0, 5.0, 7.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], 
[6.0, 8.0, 5.0], [8.0, 8.0, 4.0]], [[4.0, 1.0, 1.0], [1.0, 4.0, 8.0], [1.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LULU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 7.0], [8.0, 7.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 1.0, 4.0], [3.0, 
3.0, 5.0]], [[2.0, 6.0, 5.0], [5.0, 7.0, 3.0], [7.0, 1.0, 8.0]]], [[[8.0, 7.0, 1.0], [4.0, 4.0, 6.0], [4.0, 2.0, 6.0]], [[2.0, 
6.0, 7.0], [7.0, 1.0, 4.0], [1.0, 4.0, 4.0]], [[5.0, 4.0, 8.0], [2.0, 4.0, 8.0], [2.0, 7.0, 4.0]]], [[[8.0, 2.0, 8.0], [6.0, 
6.0, 2.0], [4.0, 2.0, 6.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 3.0], [1.0, 8.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 3.0, 2.0], [3.0, 2.0, 
6.0]]], [[[5.0, 2.0, 5.0], [4.0, 1.0, 7.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 2.0], [8.0, 1.0, 5.0], [1.0, 5.0, 4.0]], [[1.0, 3.0, 
8.0], [7.0, 5.0, 6.0], [4.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 2.0], [2.0, 2.0, 8.0], [5.0, 5.0, 8.0]], 
[[3.0, 4.0, 4.0], [4.0, 4.0, 7.0], [8.0, 5.0, 8.0]], [[3.0, 4.0, 2.0], [6.0, 8.0, 6.0], [5.0, 8.0, 3.0]]], [[[7.0, 2.0, 5.0], 
[8.0, 1.0, 8.0], [5.0, 2.0, 5.0]], [[4.0, 6.0, 7.0], [6.0, 3.0, 2.0], [5.0, 4.0, 2.0]], [[3.0, 3.0, 3.0], [3.0, 3.0, 3.0], 
[8.0, 6.0, 5.0]]], [[[2.0, 1.0, 1.0], [7.0, 3.0, 1.0], [5.0, 5.0, 1.0]], [[4.0, 5.0, 1.0], [7.0, 5.0, 1.0], [7.0, 1.0, 3.0]], 
[[3.0, 4.0, 7.0], [2.0, 1.0, 8.0], [7.0, 5.0, 5.0]]], [[[7.0, 4.0, 7.0], [6.0, 1.0, 8.0], [5.0, 2.0, 6.0]], [[4.0, 4.0, 3.0], 
[2.0, 3.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 5.0, 6.0], [8.0, 8.0, 3.0], [4.0, 4.0, 6.0]]]])
      val=numpy.array([[[[5.0, 4.0]], [[1.0, 3.0]]], [[[2.0, 3.0]], [[7.0, 5.0]]]])
      arg[2:,:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 7.0], [8.0, 7.0, 3.0], [5.0, 4.0, 2.0]], [[8.0, 1.0, 3.0], [1.0, 1.0, 4.0], [3.0, 
3.0, 5.0]], [[2.0, 6.0, 5.0], [5.0, 7.0, 3.0], [7.0, 1.0, 8.0]]], [[[8.0, 7.0, 1.0], [4.0, 4.0, 6.0], [4.0, 2.0, 6.0]], [[2.0, 
6.0, 7.0], [7.0, 1.0, 4.0], [1.0, 4.0, 4.0]], [[5.0, 4.0, 8.0], [2.0, 4.0, 8.0], [2.0, 7.0, 4.0]]], [[[8.0, 2.0, 8.0], [6.0, 
6.0, 2.0], [5.0, 4.0, 6.0]], [[1.0, 5.0, 7.0], [1.0, 2.0, 3.0], [1.0, 3.0, 4.0]], [[2.0, 5.0, 3.0], [3.0, 3.0, 2.0], [3.0, 2.0, 
6.0]]], [[[5.0, 2.0, 5.0], [4.0, 1.0, 7.0], [2.0, 3.0, 8.0]], [[1.0, 8.0, 2.0], [8.0, 1.0, 5.0], [7.0, 5.0, 4.0]], [[1.0, 3.0, 
8.0], [7.0, 5.0, 6.0], [4.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 2.0], [2.0, 2.0, 8.0], [5.0, 5.0, 8.0]], 
[[3.0, 4.0, 4.0], [4.0, 4.0, 7.0], [8.0, 5.0, 8.0]], [[3.0, 4.0, 2.0], [6.0, 8.0, 6.0], [5.0, 8.0, 3.0]]], [[[7.0, 2.0, 5.0], 
[8.0, 1.0, 8.0], [5.0, 2.0, 5.0]], [[4.0, 6.0, 7.0], [6.0, 3.0, 2.0], [5.0, 4.0, 2.0]], [[3.0, 3.0, 3.0], [3.0, 3.0, 3.0], 
[8.0, 6.0, 5.0]]], [[[2.0, 1.0, 1.0], [7.0, 3.0, 1.0], [5.0, 4.0, 1.0]], [[4.0, 5.0, 1.0], [7.0, 5.0, 1.0], [1.0, 3.0, 3.0]], 
[[3.0, 4.0, 7.0], [2.0, 1.0, 8.0], [7.0, 5.0, 5.0]]], [[[7.0, 4.0, 7.0], [6.0, 1.0, 8.0], [2.0, 3.0, 6.0]], [[4.0, 4.0, 3.0], 
[2.0, 3.0, 6.0], [7.0, 5.0, 5.0]], [[3.0, 5.0, 6.0], [8.0, 8.0, 3.0], [4.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LULU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 2.0], [3.0, 6.0, 7.0], [1.0, 4.0, 4.0]], [[4.0, 5.0, 5.0], [2.0, 8.0, 7.0], [2.0, 
7.0, 8.0]], [[2.0, 2.0, 4.0], [4.0, 4.0, 5.0], [4.0, 5.0, 2.0]]], [[[3.0, 7.0, 4.0], [1.0, 7.0, 2.0], [6.0, 7.0, 5.0]], [[8.0, 
7.0, 8.0], [2.0, 6.0, 5.0], [1.0, 5.0, 6.0]], [[3.0, 1.0, 4.0], [1.0, 7.0, 7.0], [6.0, 8.0, 4.0]]], [[[7.0, 6.0, 1.0], [4.0, 
4.0, 7.0], [8.0, 8.0, 6.0]], [[8.0, 7.0, 5.0], [2.0, 4.0, 2.0], [6.0, 2.0, 8.0]], [[2.0, 7.0, 6.0], [3.0, 1.0, 1.0], [6.0, 2.0, 
4.0]]], [[[7.0, 8.0, 2.0], [8.0, 4.0, 8.0], [7.0, 4.0, 8.0]], [[8.0, 2.0, 6.0], [1.0, 1.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 2.0, 
5.0], [7.0, 8.0, 4.0], [7.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 7.0], [6.0, 5.0, 1.0], [7.0, 8.0, 8.0]], 
[[4.0, 3.0, 2.0], [3.0, 6.0, 6.0], [1.0, 4.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 3.0, 2.0], [3.0, 5.0, 8.0]]], [[[7.0, 2.0, 2.0], 
[4.0, 2.0, 1.0], [8.0, 4.0, 6.0]], [[6.0, 2.0, 7.0], [1.0, 6.0, 8.0], [7.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [6.0, 2.0, 1.0], 
[3.0, 2.0, 2.0]]], [[[6.0, 3.0, 2.0], [2.0, 5.0, 3.0], [2.0, 7.0, 8.0]], [[3.0, 5.0, 2.0], [5.0, 4.0, 2.0], [1.0, 6.0, 4.0]], 
[[8.0, 7.0, 1.0], [1.0, 4.0, 5.0], [2.0, 3.0, 1.0]]], [[[7.0, 7.0, 7.0], [1.0, 6.0, 1.0], [6.0, 7.0, 4.0]], [[8.0, 4.0, 8.0], 
[2.0, 7.0, 7.0], [1.0, 4.0, 4.0]], [[4.0, 4.0, 2.0], [1.0, 1.0, 3.0], [7.0, 5.0, 3.0]]]])
      val=Data(numpy.array([[[[1.0, 4.0]], [[8.0, 4.0]]], [[[3.0, 2.0]], [[7.0, 7.0]]]]),self.functionspace)
      arg[2:,:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 2.0], [3.0, 6.0, 7.0], [1.0, 4.0, 4.0]], [[4.0, 5.0, 5.0], [2.0, 8.0, 7.0], [2.0, 
7.0, 8.0]], [[2.0, 2.0, 4.0], [4.0, 4.0, 5.0], [4.0, 5.0, 2.0]]], [[[3.0, 7.0, 4.0], [1.0, 7.0, 2.0], [6.0, 7.0, 5.0]], [[8.0, 
7.0, 8.0], [2.0, 6.0, 5.0], [1.0, 5.0, 6.0]], [[3.0, 1.0, 4.0], [1.0, 7.0, 7.0], [6.0, 8.0, 4.0]]], [[[7.0, 6.0, 1.0], [4.0, 
4.0, 7.0], [1.0, 4.0, 6.0]], [[8.0, 7.0, 5.0], [2.0, 4.0, 2.0], [8.0, 4.0, 8.0]], [[2.0, 7.0, 6.0], [3.0, 1.0, 1.0], [6.0, 2.0, 
4.0]]], [[[7.0, 8.0, 2.0], [8.0, 4.0, 8.0], [3.0, 2.0, 8.0]], [[8.0, 2.0, 6.0], [1.0, 1.0, 3.0], [7.0, 7.0, 4.0]], [[7.0, 2.0, 
5.0], [7.0, 8.0, 4.0], [7.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 7.0], [6.0, 5.0, 1.0], [7.0, 8.0, 8.0]], 
[[4.0, 3.0, 2.0], [3.0, 6.0, 6.0], [1.0, 4.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 3.0, 2.0], [3.0, 5.0, 8.0]]], [[[7.0, 2.0, 2.0], 
[4.0, 2.0, 1.0], [8.0, 4.0, 6.0]], [[6.0, 2.0, 7.0], [1.0, 6.0, 8.0], [7.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [6.0, 2.0, 1.0], 
[3.0, 2.0, 2.0]]], [[[6.0, 3.0, 2.0], [2.0, 5.0, 3.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 2.0], [5.0, 4.0, 2.0], [8.0, 4.0, 4.0]], 
[[8.0, 7.0, 1.0], [1.0, 4.0, 5.0], [2.0, 3.0, 1.0]]], [[[7.0, 7.0, 7.0], [1.0, 6.0, 1.0], [3.0, 2.0, 4.0]], [[8.0, 4.0, 8.0], 
[2.0, 7.0, 7.0], [7.0, 7.0, 4.0]], [[4.0, 4.0, 2.0], [1.0, 1.0, 3.0], [7.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LULU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 1.0], [4.0, 7.0, 1.0], [1.0, 2.0, 6.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 3.0], [6.0, 
6.0, 4.0]], [[3.0, 7.0, 2.0], [8.0, 6.0, 7.0], [3.0, 3.0, 4.0]]], [[[7.0, 5.0, 3.0], [8.0, 8.0, 8.0], [2.0, 7.0, 4.0]], [[6.0, 
5.0, 3.0], [5.0, 1.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 3.0, 4.0], [1.0, 5.0, 7.0], [4.0, 5.0, 7.0]]], [[[5.0, 6.0, 3.0], [5.0, 
2.0, 4.0], [5.0, 1.0, 5.0]], [[7.0, 5.0, 3.0], [1.0, 3.0, 7.0], [6.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], [1.0, 4.0, 3.0], [7.0, 5.0, 
4.0]]], [[[8.0, 7.0, 2.0], [7.0, 6.0, 8.0], [4.0, 1.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 6.0, 7.0], [1.0, 6.0, 2.0]], [[1.0, 6.0, 
4.0], [2.0, 6.0, 6.0], [5.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 6.0], [6.0, 2.0, 7.0], [5.0, 2.0, 4.0]], 
[[2.0, 4.0, 2.0], [2.0, 2.0, 3.0], [3.0, 4.0, 5.0]], [[2.0, 4.0, 5.0], [1.0, 3.0, 7.0], [1.0, 3.0, 1.0]]], [[[7.0, 2.0, 2.0], 
[3.0, 3.0, 2.0], [2.0, 5.0, 5.0]], [[1.0, 2.0, 4.0], [8.0, 8.0, 6.0], [8.0, 1.0, 5.0]], [[6.0, 8.0, 2.0], [4.0, 7.0, 6.0], 
[5.0, 7.0, 1.0]]], [[[5.0, 4.0, 2.0], [3.0, 6.0, 1.0], [6.0, 2.0, 2.0]], [[1.0, 8.0, 3.0], [1.0, 6.0, 5.0], [3.0, 6.0, 6.0]], 
[[6.0, 3.0, 5.0], [2.0, 6.0, 4.0], [2.0, 8.0, 3.0]]], [[[1.0, 1.0, 5.0], [2.0, 7.0, 2.0], [5.0, 1.0, 1.0]], [[3.0, 6.0, 2.0], 
[8.0, 6.0, 7.0], [7.0, 7.0, 1.0]], [[4.0, 4.0, 2.0], [8.0, 2.0, 7.0], [2.0, 4.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 3.0]], [[4.0, 3.0]]], [[[7.0, 3.0]], [[2.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 
5.0]], [[2.0, 4.0]]], [[[4.0, 8.0]], [[3.0, 7.0]]]])
      arg[2:,:2,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [4.0, 7.0, 1.0], [1.0, 2.0, 6.0]], [[5.0, 3.0, 8.0], [7.0, 8.0, 3.0], [6.0, 
6.0, 4.0]], [[3.0, 7.0, 2.0], [8.0, 6.0, 7.0], [3.0, 3.0, 4.0]]], [[[7.0, 5.0, 3.0], [8.0, 8.0, 8.0], [2.0, 7.0, 4.0]], [[6.0, 
5.0, 3.0], [5.0, 1.0, 3.0], [5.0, 8.0, 2.0]], [[6.0, 3.0, 4.0], [1.0, 5.0, 7.0], [4.0, 5.0, 7.0]]], [[[5.0, 6.0, 3.0], [5.0, 
2.0, 4.0], [8.0, 3.0, 5.0]], [[7.0, 5.0, 3.0], [1.0, 3.0, 7.0], [4.0, 3.0, 2.0]], [[1.0, 4.0, 8.0], [1.0, 4.0, 3.0], [7.0, 5.0, 
4.0]]], [[[8.0, 7.0, 2.0], [7.0, 6.0, 8.0], [7.0, 3.0, 6.0]], [[8.0, 2.0, 7.0], [3.0, 6.0, 7.0], [2.0, 4.0, 2.0]], [[1.0, 6.0, 
4.0], [2.0, 6.0, 6.0], [5.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 6.0], [6.0, 2.0, 7.0], [5.0, 2.0, 4.0]], 
[[2.0, 4.0, 2.0], [2.0, 2.0, 3.0], [3.0, 4.0, 5.0]], [[2.0, 4.0, 5.0], [1.0, 3.0, 7.0], [1.0, 3.0, 1.0]]], [[[7.0, 2.0, 2.0], 
[3.0, 3.0, 2.0], [2.0, 5.0, 5.0]], [[1.0, 2.0, 4.0], [8.0, 8.0, 6.0], [8.0, 1.0, 5.0]], [[6.0, 8.0, 2.0], [4.0, 7.0, 6.0], 
[5.0, 7.0, 1.0]]], [[[5.0, 4.0, 2.0], [3.0, 6.0, 1.0], [6.0, 5.0, 2.0]], [[1.0, 8.0, 3.0], [1.0, 6.0, 5.0], [2.0, 4.0, 6.0]], 
[[6.0, 3.0, 5.0], [2.0, 6.0, 4.0], [2.0, 8.0, 3.0]]], [[[1.0, 1.0, 5.0], [2.0, 7.0, 2.0], [4.0, 8.0, 1.0]], [[3.0, 6.0, 2.0], 
[8.0, 6.0, 7.0], [3.0, 7.0, 1.0]], [[4.0, 4.0, 2.0], [8.0, 2.0, 7.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PUUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 2.0], [4.0, 2.0, 7.0], [3.0, 2.0, 1.0]], [[2.0, 6.0, 7.0], [3.0, 8.0, 3.0], [4.0, 
1.0, 1.0]], [[8.0, 1.0, 4.0], [7.0, 2.0, 3.0], [4.0, 7.0, 5.0]]], [[[4.0, 3.0, 5.0], [5.0, 2.0, 4.0], [5.0, 7.0, 6.0]], [[6.0, 
8.0, 5.0], [7.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[1.0, 6.0, 6.0], [2.0, 7.0, 2.0], [8.0, 6.0, 2.0]]], [[[5.0, 7.0, 7.0], [3.0, 
7.0, 4.0], [4.0, 1.0, 3.0]], [[7.0, 6.0, 1.0], [2.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], [2.0, 5.0, 1.0], [3.0, 4.0, 
6.0]]], [[[1.0, 3.0, 7.0], [5.0, 1.0, 2.0], [4.0, 4.0, 1.0]], [[6.0, 8.0, 8.0], [6.0, 3.0, 5.0], [1.0, 7.0, 8.0]], [[7.0, 2.0, 
1.0], [5.0, 5.0, 7.0], [7.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 3.0], [3.0, 7.0, 2.0], [5.0, 4.0, 4.0]], 
[[1.0, 5.0, 7.0], [8.0, 4.0, 1.0], [4.0, 5.0, 6.0]], [[7.0, 2.0, 2.0], [4.0, 1.0, 6.0], [3.0, 3.0, 5.0]]], [[[4.0, 7.0, 6.0], 
[3.0, 1.0, 6.0], [8.0, 8.0, 8.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 1.0], [5.0, 2.0, 4.0]], [[1.0, 1.0, 3.0], [4.0, 6.0, 7.0], 
[2.0, 8.0, 3.0]]], [[[8.0, 8.0, 6.0], [2.0, 2.0, 3.0], [3.0, 4.0, 2.0]], [[4.0, 6.0, 2.0], [2.0, 3.0, 5.0], [1.0, 6.0, 4.0]], 
[[2.0, 6.0, 1.0], [8.0, 8.0, 2.0], [2.0, 7.0, 4.0]]], [[[1.0, 3.0, 3.0], [6.0, 8.0, 3.0], [2.0, 5.0, 8.0]], [[6.0, 6.0, 3.0], 
[7.0, 7.0, 4.0], [7.0, 4.0, 3.0]], [[8.0, 6.0, 4.0], [3.0, 6.0, 3.0], [4.0, 5.0, 5.0]]]])
      val=numpy.array([[[8.0], [4.0]], [[2.0], [5.0]]])
      arg[2,:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 2.0], [4.0, 2.0, 7.0], [3.0, 2.0, 1.0]], [[2.0, 6.0, 7.0], [3.0, 8.0, 3.0], [4.0, 
1.0, 1.0]], [[8.0, 1.0, 4.0], [7.0, 2.0, 3.0], [4.0, 7.0, 5.0]]], [[[4.0, 3.0, 5.0], [5.0, 2.0, 4.0], [5.0, 7.0, 6.0]], [[6.0, 
8.0, 5.0], [7.0, 4.0, 4.0], [1.0, 1.0, 3.0]], [[1.0, 6.0, 6.0], [2.0, 7.0, 2.0], [8.0, 6.0, 2.0]]], [[[5.0, 8.0, 7.0], [3.0, 
4.0, 4.0], [4.0, 1.0, 3.0]], [[7.0, 2.0, 1.0], [2.0, 5.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], [2.0, 5.0, 1.0], [3.0, 4.0, 
6.0]]], [[[1.0, 3.0, 7.0], [5.0, 1.0, 2.0], [4.0, 4.0, 1.0]], [[6.0, 8.0, 8.0], [6.0, 3.0, 5.0], [1.0, 7.0, 8.0]], [[7.0, 2.0, 
1.0], [5.0, 5.0, 7.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 3.0], [3.0, 7.0, 2.0], [5.0, 4.0, 4.0]], 
[[1.0, 5.0, 7.0], [8.0, 4.0, 1.0], [4.0, 5.0, 6.0]], [[7.0, 2.0, 2.0], [4.0, 1.0, 6.0], [3.0, 3.0, 5.0]]], [[[4.0, 7.0, 6.0], 
[3.0, 1.0, 6.0], [8.0, 8.0, 8.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 1.0], [5.0, 2.0, 4.0]], [[1.0, 1.0, 3.0], [4.0, 6.0, 7.0], 
[2.0, 8.0, 3.0]]], [[[8.0, 8.0, 6.0], [2.0, 4.0, 3.0], [3.0, 4.0, 2.0]], [[4.0, 2.0, 2.0], [2.0, 5.0, 5.0], [1.0, 6.0, 4.0]], 
[[2.0, 6.0, 1.0], [8.0, 8.0, 2.0], [2.0, 7.0, 4.0]]], [[[1.0, 3.0, 3.0], [6.0, 8.0, 3.0], [2.0, 5.0, 8.0]], [[6.0, 6.0, 3.0], 
[7.0, 7.0, 4.0], [7.0, 4.0, 3.0]], [[8.0, 6.0, 4.0], [3.0, 6.0, 3.0], [4.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PUUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 4.0], [8.0, 6.0, 2.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 7.0], [6.0, 3.0, 1.0], [8.0, 
2.0, 2.0]], [[8.0, 7.0, 1.0], [3.0, 4.0, 3.0], [6.0, 2.0, 2.0]]], [[[5.0, 2.0, 1.0], [2.0, 6.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 
3.0, 6.0], [5.0, 3.0, 7.0], [7.0, 4.0, 7.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 6.0], [7.0, 3.0, 5.0]]], [[[5.0, 2.0, 2.0], [6.0, 
7.0, 1.0], [2.0, 1.0, 2.0]], [[8.0, 2.0, 7.0], [7.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 6.0, 2.0], [7.0, 5.0, 
2.0]]], [[[8.0, 1.0, 5.0], [7.0, 7.0, 3.0], [2.0, 7.0, 7.0]], [[3.0, 1.0, 7.0], [5.0, 7.0, 8.0], [6.0, 6.0, 8.0]], [[4.0, 3.0, 
4.0], [6.0, 7.0, 6.0], [1.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 2.0, 6.0], [6.0, 4.0, 6.0], [7.0, 2.0, 5.0]], 
[[8.0, 3.0, 1.0], [1.0, 7.0, 8.0], [4.0, 1.0, 3.0]], [[3.0, 6.0, 1.0], [8.0, 6.0, 8.0], [2.0, 8.0, 7.0]]], [[[1.0, 3.0, 8.0], 
[5.0, 6.0, 6.0], [2.0, 8.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 6.0, 4.0], [6.0, 6.0, 4.0]], [[6.0, 7.0, 8.0], [4.0, 8.0, 2.0], 
[8.0, 3.0, 5.0]]], [[[1.0, 3.0, 4.0], [3.0, 3.0, 1.0], [7.0, 1.0, 4.0]], [[3.0, 2.0, 8.0], [5.0, 6.0, 8.0], [3.0, 7.0, 4.0]], 
[[2.0, 1.0, 7.0], [3.0, 3.0, 2.0], [4.0, 7.0, 2.0]]], [[[6.0, 8.0, 6.0], [3.0, 4.0, 6.0], [8.0, 3.0, 8.0]], [[7.0, 2.0, 3.0], 
[4.0, 7.0, 6.0], [3.0, 8.0, 6.0]], [[5.0, 5.0, 1.0], [7.0, 4.0, 1.0], [7.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[1.0], [5.0]], [[1.0], [7.0]]]),self.functionspace)
      arg[2,:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 4.0], [8.0, 6.0, 2.0], [5.0, 4.0, 7.0]], [[3.0, 4.0, 7.0], [6.0, 3.0, 1.0], [8.0, 
2.0, 2.0]], [[8.0, 7.0, 1.0], [3.0, 4.0, 3.0], [6.0, 2.0, 2.0]]], [[[5.0, 2.0, 1.0], [2.0, 6.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 
3.0, 6.0], [5.0, 3.0, 7.0], [7.0, 4.0, 7.0]], [[7.0, 2.0, 5.0], [6.0, 4.0, 6.0], [7.0, 3.0, 5.0]]], [[[5.0, 1.0, 2.0], [6.0, 
5.0, 1.0], [2.0, 1.0, 2.0]], [[8.0, 1.0, 7.0], [7.0, 7.0, 5.0], [4.0, 1.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 6.0, 2.0], [7.0, 5.0, 
2.0]]], [[[8.0, 1.0, 5.0], [7.0, 7.0, 3.0], [2.0, 7.0, 7.0]], [[3.0, 1.0, 7.0], [5.0, 7.0, 8.0], [6.0, 6.0, 8.0]], [[4.0, 3.0, 
4.0], [6.0, 7.0, 6.0], [1.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 6.0], [6.0, 4.0, 6.0], [7.0, 2.0, 5.0]], 
[[8.0, 3.0, 1.0], [1.0, 7.0, 8.0], [4.0, 1.0, 3.0]], [[3.0, 6.0, 1.0], [8.0, 6.0, 8.0], [2.0, 8.0, 7.0]]], [[[1.0, 3.0, 8.0], 
[5.0, 6.0, 6.0], [2.0, 8.0, 7.0]], [[3.0, 3.0, 8.0], [7.0, 6.0, 4.0], [6.0, 6.0, 4.0]], [[6.0, 7.0, 8.0], [4.0, 8.0, 2.0], 
[8.0, 3.0, 5.0]]], [[[1.0, 1.0, 4.0], [3.0, 5.0, 1.0], [7.0, 1.0, 4.0]], [[3.0, 1.0, 8.0], [5.0, 7.0, 8.0], [3.0, 7.0, 4.0]], 
[[2.0, 1.0, 7.0], [3.0, 3.0, 2.0], [4.0, 7.0, 2.0]]], [[[6.0, 8.0, 6.0], [3.0, 4.0, 6.0], [8.0, 3.0, 8.0]], [[7.0, 2.0, 3.0], 
[4.0, 7.0, 6.0], [3.0, 8.0, 6.0]], [[5.0, 5.0, 1.0], [7.0, 4.0, 1.0], [7.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PUUR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 1.0], [3.0, 5.0, 6.0], [6.0, 7.0, 5.0]], [[1.0, 7.0, 4.0], [6.0, 3.0, 4.0], [5.0, 
8.0, 6.0]], [[7.0, 3.0, 6.0], [1.0, 4.0, 6.0], [3.0, 7.0, 3.0]]], [[[8.0, 4.0, 5.0], [4.0, 2.0, 5.0], [6.0, 1.0, 6.0]], [[3.0, 
5.0, 6.0], [7.0, 3.0, 1.0], [7.0, 6.0, 4.0]], [[5.0, 6.0, 2.0], [7.0, 8.0, 5.0], [1.0, 6.0, 6.0]]], [[[4.0, 2.0, 6.0], [1.0, 
8.0, 2.0], [3.0, 4.0, 6.0]], [[3.0, 4.0, 8.0], [6.0, 8.0, 4.0], [3.0, 1.0, 6.0]], [[4.0, 5.0, 5.0], [7.0, 2.0, 2.0], [4.0, 5.0, 
7.0]]], [[[7.0, 1.0, 5.0], [7.0, 5.0, 6.0], [5.0, 2.0, 5.0]], [[3.0, 4.0, 3.0], [3.0, 2.0, 3.0], [8.0, 4.0, 2.0]], [[1.0, 8.0, 
8.0], [3.0, 2.0, 4.0], [5.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 2.0], [8.0, 2.0, 2.0], [4.0, 4.0, 2.0]], 
[[7.0, 1.0, 3.0], [2.0, 7.0, 2.0], [2.0, 4.0, 5.0]], [[8.0, 1.0, 7.0], [5.0, 7.0, 7.0], [3.0, 3.0, 7.0]]], [[[3.0, 5.0, 8.0], 
[6.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 4.0], [7.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [5.0, 3.0, 7.0], 
[2.0, 1.0, 7.0]]], [[[5.0, 7.0, 2.0], [3.0, 5.0, 7.0], [3.0, 7.0, 7.0]], [[2.0, 4.0, 4.0], [8.0, 6.0, 6.0], [5.0, 4.0, 6.0]], 
[[2.0, 7.0, 6.0], [7.0, 5.0, 6.0], [4.0, 6.0, 4.0]]], [[[7.0, 6.0, 7.0], [5.0, 7.0, 4.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 6.0], 
[3.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[7.0, 1.0, 7.0], [6.0, 3.0, 1.0], [1.0, 1.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0], [1.0]], [[8.0], [3.0]]])+(1.-msk_val)*numpy.array([[[6.0], [6.0]], [[7.0], [2.0]]])
      arg[2,:2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 1.0], [3.0, 5.0, 6.0], [6.0, 7.0, 5.0]], [[1.0, 7.0, 4.0], [6.0, 3.0, 4.0], [5.0, 
8.0, 6.0]], [[7.0, 3.0, 6.0], [1.0, 4.0, 6.0], [3.0, 7.0, 3.0]]], [[[8.0, 4.0, 5.0], [4.0, 2.0, 5.0], [6.0, 1.0, 6.0]], [[3.0, 
5.0, 6.0], [7.0, 3.0, 1.0], [7.0, 6.0, 4.0]], [[5.0, 6.0, 2.0], [7.0, 8.0, 5.0], [1.0, 6.0, 6.0]]], [[[4.0, 5.0, 6.0], [1.0, 
1.0, 2.0], [3.0, 4.0, 6.0]], [[3.0, 8.0, 8.0], [6.0, 3.0, 4.0], [3.0, 1.0, 6.0]], [[4.0, 5.0, 5.0], [7.0, 2.0, 2.0], [4.0, 5.0, 
7.0]]], [[[7.0, 1.0, 5.0], [7.0, 5.0, 6.0], [5.0, 2.0, 5.0]], [[3.0, 4.0, 3.0], [3.0, 2.0, 3.0], [8.0, 4.0, 2.0]], [[1.0, 8.0, 
8.0], [3.0, 2.0, 4.0], [5.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 2.0], [8.0, 2.0, 2.0], [4.0, 4.0, 2.0]], 
[[7.0, 1.0, 3.0], [2.0, 7.0, 2.0], [2.0, 4.0, 5.0]], [[8.0, 1.0, 7.0], [5.0, 7.0, 7.0], [3.0, 3.0, 7.0]]], [[[3.0, 5.0, 8.0], 
[6.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 4.0], [7.0, 2.0, 1.0]], [[2.0, 7.0, 1.0], [5.0, 3.0, 7.0], 
[2.0, 1.0, 7.0]]], [[[5.0, 6.0, 2.0], [3.0, 6.0, 7.0], [3.0, 7.0, 7.0]], [[2.0, 7.0, 4.0], [8.0, 2.0, 6.0], [5.0, 4.0, 6.0]], 
[[2.0, 7.0, 6.0], [7.0, 5.0, 6.0], [4.0, 6.0, 4.0]]], [[[7.0, 6.0, 7.0], [5.0, 7.0, 4.0], [8.0, 1.0, 4.0]], [[7.0, 1.0, 6.0], 
[3.0, 7.0, 7.0], [4.0, 7.0, 3.0]], [[7.0, 1.0, 7.0], [6.0, 3.0, 1.0], [1.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 5.0], [6.0, 6.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 1.0], [4.0, 4.0, 4.0], [8.0, 
8.0, 3.0]], [[7.0, 8.0, 7.0], [1.0, 4.0, 8.0], [3.0, 8.0, 2.0]]], [[[4.0, 3.0, 1.0], [2.0, 5.0, 1.0], [8.0, 2.0, 4.0]], [[6.0, 
6.0, 5.0], [7.0, 6.0, 5.0], [7.0, 7.0, 6.0]], [[4.0, 5.0, 8.0], [4.0, 3.0, 4.0], [1.0, 1.0, 6.0]]], [[[4.0, 7.0, 2.0], [6.0, 
5.0, 5.0], [6.0, 6.0, 5.0]], [[8.0, 1.0, 6.0], [6.0, 2.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 8.0, 8.0], [6.0, 7.0, 5.0], [8.0, 3.0, 
6.0]]], [[[7.0, 5.0, 4.0], [3.0, 1.0, 6.0], [3.0, 6.0, 3.0]], [[8.0, 7.0, 6.0], [1.0, 8.0, 4.0], [4.0, 5.0, 6.0]], [[2.0, 6.0, 
4.0], [6.0, 1.0, 3.0], [7.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 8.0], [6.0, 1.0, 6.0], [5.0, 8.0, 1.0]], 
[[3.0, 6.0, 4.0], [4.0, 6.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 3.0, 1.0], [1.0, 6.0, 8.0], [4.0, 2.0, 7.0]]], [[[7.0, 1.0, 4.0], 
[1.0, 3.0, 3.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 5.0], [5.0, 7.0, 8.0], [8.0, 4.0, 7.0]], [[2.0, 7.0, 8.0], [5.0, 1.0, 3.0], 
[5.0, 2.0, 7.0]]], [[[6.0, 3.0, 7.0], [1.0, 4.0, 7.0], [2.0, 3.0, 6.0]], [[7.0, 2.0, 6.0], [3.0, 2.0, 1.0], [2.0, 5.0, 1.0]], 
[[6.0, 4.0, 5.0], [5.0, 5.0, 5.0], [8.0, 3.0, 4.0]]], [[[1.0, 7.0, 1.0], [1.0, 7.0, 1.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 3.0], 
[3.0, 7.0, 8.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 3.0], [6.0, 2.0, 3.0], [3.0, 2.0, 3.0]]]])
      val=numpy.array([[[[6.0]]], [[[1.0]]]])
      arg[2:,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [6.0, 6.0, 6.0], [7.0, 7.0, 4.0]], [[1.0, 6.0, 1.0], [4.0, 4.0, 4.0], [8.0, 
8.0, 3.0]], [[7.0, 8.0, 7.0], [1.0, 4.0, 8.0], [3.0, 8.0, 2.0]]], [[[4.0, 3.0, 1.0], [2.0, 5.0, 1.0], [8.0, 2.0, 4.0]], [[6.0, 
6.0, 5.0], [7.0, 6.0, 5.0], [7.0, 7.0, 6.0]], [[4.0, 5.0, 8.0], [4.0, 3.0, 4.0], [1.0, 1.0, 6.0]]], [[[4.0, 7.0, 2.0], [6.0, 
5.0, 5.0], [6.0, 6.0, 5.0]], [[8.0, 1.0, 6.0], [6.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[8.0, 8.0, 8.0], [6.0, 7.0, 5.0], [8.0, 3.0, 
6.0]]], [[[7.0, 5.0, 4.0], [3.0, 1.0, 6.0], [3.0, 6.0, 3.0]], [[8.0, 7.0, 6.0], [1.0, 1.0, 4.0], [4.0, 5.0, 6.0]], [[2.0, 6.0, 
4.0], [6.0, 1.0, 3.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 8.0], [6.0, 1.0, 6.0], [5.0, 8.0, 1.0]], 
[[3.0, 6.0, 4.0], [4.0, 6.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 3.0, 1.0], [1.0, 6.0, 8.0], [4.0, 2.0, 7.0]]], [[[7.0, 1.0, 4.0], 
[1.0, 3.0, 3.0], [4.0, 2.0, 6.0]], [[3.0, 5.0, 5.0], [5.0, 7.0, 8.0], [8.0, 4.0, 7.0]], [[2.0, 7.0, 8.0], [5.0, 1.0, 3.0], 
[5.0, 2.0, 7.0]]], [[[6.0, 3.0, 7.0], [1.0, 4.0, 7.0], [2.0, 3.0, 6.0]], [[7.0, 2.0, 6.0], [3.0, 6.0, 1.0], [2.0, 5.0, 1.0]], 
[[6.0, 4.0, 5.0], [5.0, 5.0, 5.0], [8.0, 3.0, 4.0]]], [[[1.0, 7.0, 1.0], [1.0, 7.0, 1.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 3.0], 
[3.0, 1.0, 8.0], [8.0, 4.0, 2.0]], [[7.0, 4.0, 3.0], [6.0, 2.0, 3.0], [3.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 3.0], [2.0, 6.0, 3.0], [3.0, 3.0, 5.0]], [[7.0, 4.0, 7.0], [1.0, 3.0, 4.0], [8.0, 
3.0, 4.0]], [[6.0, 8.0, 3.0], [6.0, 7.0, 6.0], [8.0, 3.0, 4.0]]], [[[8.0, 2.0, 6.0], [4.0, 1.0, 3.0], [2.0, 1.0, 7.0]], [[3.0, 
3.0, 8.0], [5.0, 4.0, 6.0], [6.0, 5.0, 4.0]], [[7.0, 7.0, 1.0], [3.0, 8.0, 6.0], [7.0, 7.0, 4.0]]], [[[2.0, 7.0, 4.0], [8.0, 
6.0, 2.0], [4.0, 8.0, 7.0]], [[4.0, 1.0, 5.0], [6.0, 7.0, 2.0], [6.0, 5.0, 8.0]], [[7.0, 6.0, 7.0], [3.0, 4.0, 2.0], [1.0, 1.0, 
5.0]]], [[[4.0, 6.0, 2.0], [2.0, 7.0, 5.0], [4.0, 7.0, 7.0]], [[5.0, 5.0, 5.0], [8.0, 3.0, 8.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 
7.0], [8.0, 1.0, 1.0], [6.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 8.0], [5.0, 7.0, 1.0], [8.0, 6.0, 2.0]], 
[[4.0, 8.0, 8.0], [3.0, 5.0, 5.0], [2.0, 3.0, 8.0]], [[8.0, 5.0, 2.0], [6.0, 1.0, 3.0], [3.0, 3.0, 2.0]]], [[[3.0, 1.0, 3.0], 
[3.0, 2.0, 3.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 8.0], [7.0, 1.0, 4.0], [7.0, 4.0, 4.0]], [[7.0, 1.0, 1.0], [1.0, 7.0, 1.0], 
[2.0, 8.0, 3.0]]], [[[4.0, 7.0, 7.0], [8.0, 4.0, 1.0], [3.0, 4.0, 4.0]], [[6.0, 4.0, 6.0], [3.0, 2.0, 7.0], [5.0, 4.0, 5.0]], 
[[7.0, 2.0, 7.0], [4.0, 6.0, 1.0], [3.0, 4.0, 6.0]]], [[[8.0, 3.0, 7.0], [2.0, 5.0, 1.0], [3.0, 6.0, 3.0]], [[8.0, 7.0, 8.0], 
[2.0, 3.0, 2.0], [2.0, 1.0, 7.0]], [[3.0, 8.0, 2.0], [7.0, 8.0, 6.0], [6.0, 3.0, 2.0]]]])
      val=Data(numpy.array([[[[4.0]]], [[[7.0]]]]),self.functionspace)
      arg[2:,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 3.0], [2.0, 6.0, 3.0], [3.0, 3.0, 5.0]], [[7.0, 4.0, 7.0], [1.0, 3.0, 4.0], [8.0, 
3.0, 4.0]], [[6.0, 8.0, 3.0], [6.0, 7.0, 6.0], [8.0, 3.0, 4.0]]], [[[8.0, 2.0, 6.0], [4.0, 1.0, 3.0], [2.0, 1.0, 7.0]], [[3.0, 
3.0, 8.0], [5.0, 4.0, 6.0], [6.0, 5.0, 4.0]], [[7.0, 7.0, 1.0], [3.0, 8.0, 6.0], [7.0, 7.0, 4.0]]], [[[2.0, 7.0, 4.0], [8.0, 
6.0, 2.0], [4.0, 8.0, 7.0]], [[4.0, 1.0, 5.0], [6.0, 4.0, 2.0], [6.0, 5.0, 8.0]], [[7.0, 6.0, 7.0], [3.0, 4.0, 2.0], [1.0, 1.0, 
5.0]]], [[[4.0, 6.0, 2.0], [2.0, 7.0, 5.0], [4.0, 7.0, 7.0]], [[5.0, 5.0, 5.0], [8.0, 7.0, 8.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 
7.0], [8.0, 1.0, 1.0], [6.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 8.0], [5.0, 7.0, 1.0], [8.0, 6.0, 2.0]], 
[[4.0, 8.0, 8.0], [3.0, 5.0, 5.0], [2.0, 3.0, 8.0]], [[8.0, 5.0, 2.0], [6.0, 1.0, 3.0], [3.0, 3.0, 2.0]]], [[[3.0, 1.0, 3.0], 
[3.0, 2.0, 3.0], [2.0, 6.0, 4.0]], [[5.0, 5.0, 8.0], [7.0, 1.0, 4.0], [7.0, 4.0, 4.0]], [[7.0, 1.0, 1.0], [1.0, 7.0, 1.0], 
[2.0, 8.0, 3.0]]], [[[4.0, 7.0, 7.0], [8.0, 4.0, 1.0], [3.0, 4.0, 4.0]], [[6.0, 4.0, 6.0], [3.0, 4.0, 7.0], [5.0, 4.0, 5.0]], 
[[7.0, 2.0, 7.0], [4.0, 6.0, 1.0], [3.0, 4.0, 6.0]]], [[[8.0, 3.0, 7.0], [2.0, 5.0, 1.0], [3.0, 6.0, 3.0]], [[8.0, 7.0, 8.0], 
[2.0, 7.0, 2.0], [2.0, 1.0, 7.0]], [[3.0, 8.0, 2.0], [7.0, 8.0, 6.0], [6.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 5.0], [6.0, 6.0, 2.0], [2.0, 3.0, 1.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 7.0], [8.0, 
2.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 3.0, 6.0], [2.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [3.0, 7.0, 7.0], [3.0, 3.0, 6.0]], [[6.0, 
8.0, 6.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 1.0], [1.0, 2.0, 3.0]]], [[[8.0, 2.0, 7.0], [7.0, 
3.0, 6.0], [1.0, 4.0, 2.0]], [[1.0, 3.0, 3.0], [6.0, 4.0, 5.0], [1.0, 1.0, 8.0]], [[4.0, 5.0, 8.0], [7.0, 8.0, 4.0], [1.0, 7.0, 
7.0]]], [[[7.0, 1.0, 2.0], [4.0, 3.0, 8.0], [5.0, 3.0, 5.0]], [[3.0, 8.0, 7.0], [1.0, 5.0, 6.0], [7.0, 7.0, 8.0]], [[1.0, 6.0, 
4.0], [4.0, 3.0, 7.0], [1.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 2.0, 3.0], [4.0, 4.0, 7.0], [4.0, 3.0, 6.0]], 
[[2.0, 1.0, 6.0], [1.0, 7.0, 1.0], [5.0, 5.0, 8.0]], [[8.0, 8.0, 5.0], [5.0, 4.0, 2.0], [2.0, 1.0, 4.0]]], [[[8.0, 4.0, 4.0], 
[7.0, 3.0, 2.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 3.0], [3.0, 7.0, 1.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [8.0, 7.0, 2.0], 
[5.0, 1.0, 7.0]]], [[[4.0, 8.0, 6.0], [3.0, 7.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 5.0, 3.0], [8.0, 7.0, 6.0]], 
[[5.0, 7.0, 7.0], [8.0, 5.0, 6.0], [2.0, 8.0, 1.0]]], [[[5.0, 6.0, 3.0], [6.0, 1.0, 1.0], [5.0, 2.0, 5.0]], [[7.0, 4.0, 4.0], 
[7.0, 2.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 1.0, 4.0], [8.0, 5.0, 4.0], [5.0, 5.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[7.0]]], [[[3.0]]]])
      arg[2:,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 5.0], [6.0, 6.0, 2.0], [2.0, 3.0, 1.0]], [[7.0, 8.0, 1.0], [5.0, 7.0, 7.0], [8.0, 
2.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 3.0, 6.0], [2.0, 7.0, 1.0]]], [[[6.0, 1.0, 2.0], [3.0, 7.0, 7.0], [3.0, 3.0, 6.0]], [[6.0, 
8.0, 6.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[1.0, 2.0, 6.0], [8.0, 4.0, 1.0], [1.0, 2.0, 3.0]]], [[[8.0, 2.0, 7.0], [7.0, 
3.0, 6.0], [1.0, 4.0, 2.0]], [[1.0, 3.0, 3.0], [6.0, 6.0, 5.0], [1.0, 1.0, 8.0]], [[4.0, 5.0, 8.0], [7.0, 8.0, 4.0], [1.0, 7.0, 
7.0]]], [[[7.0, 1.0, 2.0], [4.0, 3.0, 8.0], [5.0, 3.0, 5.0]], [[3.0, 8.0, 7.0], [1.0, 2.0, 6.0], [7.0, 7.0, 8.0]], [[1.0, 6.0, 
4.0], [4.0, 3.0, 7.0], [1.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 3.0], [4.0, 4.0, 7.0], [4.0, 3.0, 6.0]], 
[[2.0, 1.0, 6.0], [1.0, 7.0, 1.0], [5.0, 5.0, 8.0]], [[8.0, 8.0, 5.0], [5.0, 4.0, 2.0], [2.0, 1.0, 4.0]]], [[[8.0, 4.0, 4.0], 
[7.0, 3.0, 2.0], [2.0, 5.0, 4.0]], [[8.0, 7.0, 3.0], [3.0, 7.0, 1.0], [8.0, 5.0, 3.0]], [[8.0, 4.0, 6.0], [8.0, 7.0, 2.0], 
[5.0, 1.0, 7.0]]], [[[4.0, 8.0, 6.0], [3.0, 7.0, 1.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 7.0, 3.0], [8.0, 7.0, 6.0]], 
[[5.0, 7.0, 7.0], [8.0, 5.0, 6.0], [2.0, 8.0, 1.0]]], [[[5.0, 6.0, 3.0], [6.0, 1.0, 1.0], [5.0, 2.0, 5.0]], [[7.0, 4.0, 4.0], 
[7.0, 3.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 1.0, 4.0], [8.0, 5.0, 4.0], [5.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 8.0], [5.0, 1.0, 5.0], [1.0, 1.0, 1.0]], [[5.0, 7.0, 3.0], [4.0, 1.0, 5.0], [7.0, 
1.0, 7.0]], [[6.0, 3.0, 3.0], [5.0, 8.0, 3.0], [1.0, 6.0, 6.0]]], [[[6.0, 2.0, 5.0], [4.0, 5.0, 1.0], [3.0, 5.0, 5.0]], [[2.0, 
2.0, 5.0], [4.0, 3.0, 5.0], [7.0, 5.0, 5.0]], [[7.0, 6.0, 2.0], [8.0, 5.0, 5.0], [1.0, 3.0, 3.0]]], [[[3.0, 5.0, 3.0], [3.0, 
6.0, 7.0], [5.0, 5.0, 4.0]], [[3.0, 5.0, 1.0], [3.0, 6.0, 2.0], [4.0, 6.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 6.0, 5.0], [8.0, 1.0, 
8.0]]], [[[3.0, 2.0, 3.0], [4.0, 3.0, 8.0], [7.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [6.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[4.0, 1.0, 
3.0], [4.0, 3.0, 8.0], [3.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 1.0], [6.0, 7.0, 2.0], [3.0, 8.0, 3.0]], 
[[6.0, 5.0, 4.0], [7.0, 7.0, 7.0], [1.0, 8.0, 5.0]], [[5.0, 3.0, 1.0], [2.0, 8.0, 7.0], [2.0, 2.0, 7.0]]], [[[6.0, 5.0, 5.0], 
[7.0, 5.0, 8.0], [5.0, 3.0, 6.0]], [[5.0, 4.0, 5.0], [2.0, 5.0, 8.0], [6.0, 5.0, 3.0]], [[6.0, 2.0, 3.0], [3.0, 3.0, 4.0], 
[3.0, 6.0, 6.0]]], [[[6.0, 2.0, 7.0], [6.0, 1.0, 2.0], [7.0, 5.0, 8.0]], [[7.0, 6.0, 8.0], [6.0, 1.0, 8.0], [7.0, 2.0, 3.0]], 
[[8.0, 5.0, 3.0], [3.0, 3.0, 8.0], [4.0, 6.0, 3.0]]], [[[7.0, 5.0, 5.0], [5.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[4.0, 4.0, 3.0], 
[8.0, 4.0, 3.0], [3.0, 1.0, 3.0]], [[1.0, 2.0, 6.0], [4.0, 1.0, 5.0], [4.0, 8.0, 8.0]]]])
      val=numpy.array([[[7.0, 8.0, 7.0], [2.0, 3.0, 5.0]], [[1.0, 3.0, 1.0], [1.0, 4.0, 5.0]], [[2.0, 2.0, 6.0], [1.0, 3.0, 
1.0]]])
      arg[:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 8.0], [7.0, 8.0, 7.0], [1.0, 1.0, 1.0]], [[5.0, 7.0, 3.0], [2.0, 3.0, 5.0], [7.0, 
1.0, 7.0]], [[6.0, 3.0, 3.0], [5.0, 8.0, 3.0], [1.0, 6.0, 6.0]]], [[[6.0, 2.0, 5.0], [1.0, 3.0, 1.0], [3.0, 5.0, 5.0]], [[2.0, 
2.0, 5.0], [1.0, 4.0, 5.0], [7.0, 5.0, 5.0]], [[7.0, 6.0, 2.0], [8.0, 5.0, 5.0], [1.0, 3.0, 3.0]]], [[[3.0, 5.0, 3.0], [2.0, 
2.0, 6.0], [5.0, 5.0, 4.0]], [[3.0, 5.0, 1.0], [1.0, 3.0, 1.0], [4.0, 6.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 6.0, 5.0], [8.0, 1.0, 
8.0]]], [[[3.0, 2.0, 3.0], [4.0, 3.0, 8.0], [7.0, 8.0, 1.0]], [[8.0, 1.0, 3.0], [6.0, 4.0, 7.0], [6.0, 4.0, 3.0]], [[4.0, 1.0, 
3.0], [4.0, 3.0, 8.0], [3.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 1.0], [7.0, 8.0, 7.0], [3.0, 8.0, 3.0]], 
[[6.0, 5.0, 4.0], [2.0, 3.0, 5.0], [1.0, 8.0, 5.0]], [[5.0, 3.0, 1.0], [2.0, 8.0, 7.0], [2.0, 2.0, 7.0]]], [[[6.0, 5.0, 5.0], 
[1.0, 3.0, 1.0], [5.0, 3.0, 6.0]], [[5.0, 4.0, 5.0], [1.0, 4.0, 5.0], [6.0, 5.0, 3.0]], [[6.0, 2.0, 3.0], [3.0, 3.0, 4.0], 
[3.0, 6.0, 6.0]]], [[[6.0, 2.0, 7.0], [2.0, 2.0, 6.0], [7.0, 5.0, 8.0]], [[7.0, 6.0, 8.0], [1.0, 3.0, 1.0], [7.0, 2.0, 3.0]], 
[[8.0, 5.0, 3.0], [3.0, 3.0, 8.0], [4.0, 6.0, 3.0]]], [[[7.0, 5.0, 5.0], [5.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[4.0, 4.0, 3.0], 
[8.0, 4.0, 3.0], [3.0, 1.0, 3.0]], [[1.0, 2.0, 6.0], [4.0, 1.0, 5.0], [4.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 6.0], [6.0, 6.0, 7.0], [6.0, 5.0, 1.0]], [[5.0, 1.0, 2.0], [5.0, 4.0, 4.0], [2.0, 
7.0, 3.0]], [[4.0, 6.0, 4.0], [6.0, 6.0, 1.0], [6.0, 3.0, 2.0]]], [[[8.0, 4.0, 5.0], [4.0, 1.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 
5.0, 1.0], [1.0, 4.0, 5.0], [5.0, 7.0, 5.0]], [[2.0, 8.0, 7.0], [1.0, 4.0, 6.0], [4.0, 2.0, 5.0]]], [[[5.0, 6.0, 3.0], [3.0, 
3.0, 4.0], [8.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [7.0, 4.0, 5.0], [5.0, 8.0, 2.0]], [[2.0, 2.0, 1.0], [7.0, 4.0, 3.0], [6.0, 1.0, 
1.0]]], [[[1.0, 3.0, 1.0], [8.0, 2.0, 2.0], [2.0, 1.0, 5.0]], [[1.0, 7.0, 7.0], [6.0, 3.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 
1.0], [3.0, 7.0, 7.0], [6.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 5.0], [7.0, 2.0, 6.0], [5.0, 6.0, 8.0]], 
[[2.0, 2.0, 8.0], [6.0, 6.0, 6.0], [1.0, 2.0, 2.0]], [[1.0, 8.0, 3.0], [3.0, 5.0, 7.0], [1.0, 4.0, 1.0]]], [[[2.0, 2.0, 4.0], 
[5.0, 8.0, 2.0], [2.0, 2.0, 5.0]], [[5.0, 8.0, 2.0], [5.0, 5.0, 3.0], [6.0, 2.0, 2.0]], [[4.0, 1.0, 7.0], [3.0, 2.0, 6.0], 
[7.0, 7.0, 7.0]]], [[[6.0, 6.0, 8.0], [2.0, 7.0, 8.0], [3.0, 2.0, 6.0]], [[8.0, 8.0, 6.0], [6.0, 3.0, 4.0], [4.0, 3.0, 8.0]], 
[[4.0, 3.0, 1.0], [8.0, 2.0, 8.0], [5.0, 2.0, 6.0]]], [[[5.0, 1.0, 1.0], [4.0, 2.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 4.0, 1.0], 
[6.0, 3.0, 5.0], [2.0, 8.0, 4.0]], [[3.0, 4.0, 5.0], [8.0, 1.0, 4.0], [1.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[7.0, 4.0, 5.0], [8.0, 2.0, 1.0]], [[2.0, 2.0, 2.0], [6.0, 4.0, 2.0]], [[8.0, 7.0, 4.0], [1.0, 
7.0, 7.0]]]),self.functionspace)
      arg[:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 6.0], [7.0, 4.0, 5.0], [6.0, 5.0, 1.0]], [[5.0, 1.0, 2.0], [8.0, 2.0, 1.0], [2.0, 
7.0, 3.0]], [[4.0, 6.0, 4.0], [6.0, 6.0, 1.0], [6.0, 3.0, 2.0]]], [[[8.0, 4.0, 5.0], [2.0, 2.0, 2.0], [3.0, 3.0, 6.0]], [[2.0, 
5.0, 1.0], [6.0, 4.0, 2.0], [5.0, 7.0, 5.0]], [[2.0, 8.0, 7.0], [1.0, 4.0, 6.0], [4.0, 2.0, 5.0]]], [[[5.0, 6.0, 3.0], [8.0, 
7.0, 4.0], [8.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [1.0, 7.0, 7.0], [5.0, 8.0, 2.0]], [[2.0, 2.0, 1.0], [7.0, 4.0, 3.0], [6.0, 1.0, 
1.0]]], [[[1.0, 3.0, 1.0], [8.0, 2.0, 2.0], [2.0, 1.0, 5.0]], [[1.0, 7.0, 7.0], [6.0, 3.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 8.0, 
1.0], [3.0, 7.0, 7.0], [6.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 5.0], [7.0, 4.0, 5.0], [5.0, 6.0, 8.0]], 
[[2.0, 2.0, 8.0], [8.0, 2.0, 1.0], [1.0, 2.0, 2.0]], [[1.0, 8.0, 3.0], [3.0, 5.0, 7.0], [1.0, 4.0, 1.0]]], [[[2.0, 2.0, 4.0], 
[2.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[5.0, 8.0, 2.0], [6.0, 4.0, 2.0], [6.0, 2.0, 2.0]], [[4.0, 1.0, 7.0], [3.0, 2.0, 6.0], 
[7.0, 7.0, 7.0]]], [[[6.0, 6.0, 8.0], [8.0, 7.0, 4.0], [3.0, 2.0, 6.0]], [[8.0, 8.0, 6.0], [1.0, 7.0, 7.0], [4.0, 3.0, 8.0]], 
[[4.0, 3.0, 1.0], [8.0, 2.0, 8.0], [5.0, 2.0, 6.0]]], [[[5.0, 1.0, 1.0], [4.0, 2.0, 1.0], [8.0, 7.0, 7.0]], [[6.0, 4.0, 1.0], 
[6.0, 3.0, 5.0], [2.0, 8.0, 4.0]], [[3.0, 4.0, 5.0], [8.0, 1.0, 4.0], [1.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 7.0], [4.0, 2.0, 8.0], [5.0, 8.0, 8.0]], [[1.0, 6.0, 6.0], [3.0, 8.0, 5.0], [3.0, 
7.0, 5.0]], [[5.0, 8.0, 3.0], [1.0, 3.0, 6.0], [5.0, 6.0, 7.0]]], [[[7.0, 7.0, 8.0], [2.0, 6.0, 4.0], [1.0, 1.0, 5.0]], [[7.0, 
6.0, 3.0], [4.0, 3.0, 5.0], [7.0, 1.0, 2.0]], [[3.0, 1.0, 1.0], [6.0, 2.0, 1.0], [1.0, 2.0, 7.0]]], [[[4.0, 2.0, 5.0], [6.0, 
1.0, 6.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 8.0], [4.0, 6.0, 3.0], [7.0, 5.0, 5.0]], [[5.0, 3.0, 1.0], [2.0, 3.0, 4.0], [4.0, 8.0, 
2.0]]], [[[3.0, 5.0, 2.0], [3.0, 3.0, 8.0], [6.0, 4.0, 8.0]], [[4.0, 2.0, 7.0], [1.0, 4.0, 1.0], [8.0, 5.0, 2.0]], [[1.0, 1.0, 
5.0], [8.0, 5.0, 8.0], [3.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 7.0], [5.0, 6.0, 2.0], [3.0, 2.0, 5.0]], 
[[4.0, 5.0, 3.0], [6.0, 6.0, 4.0], [1.0, 6.0, 6.0]], [[8.0, 8.0, 3.0], [8.0, 2.0, 8.0], [8.0, 8.0, 4.0]]], [[[8.0, 1.0, 1.0], 
[4.0, 7.0, 8.0], [1.0, 1.0, 1.0]], [[1.0, 7.0, 8.0], [8.0, 5.0, 2.0], [6.0, 8.0, 7.0]], [[7.0, 4.0, 2.0], [8.0, 7.0, 7.0], 
[6.0, 2.0, 3.0]]], [[[7.0, 4.0, 7.0], [4.0, 3.0, 8.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 2.0], [1.0, 6.0, 5.0], [2.0, 4.0, 3.0]], 
[[3.0, 5.0, 4.0], [5.0, 2.0, 1.0], [1.0, 6.0, 2.0]]], [[[5.0, 4.0, 3.0], [3.0, 4.0, 3.0], [3.0, 8.0, 4.0]], [[2.0, 1.0, 2.0], 
[5.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 7.0], [8.0, 4.0, 5.0], [8.0, 4.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 4.0, 2.0], [8.0, 5.0, 7.0]], [[1.0, 7.0, 8.0], [7.0, 3.0, 3.0]], [[2.0, 2.0, 1.0], [5.0, 
8.0, 1.0]]])+(1.-msk_val)*numpy.array([[[3.0, 2.0, 5.0], [5.0, 6.0, 7.0]], [[5.0, 6.0, 8.0], [1.0, 1.0, 1.0]], [[4.0, 1.0, 
4.0], [7.0, 6.0, 2.0]]])
      arg[:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 7.0], [6.0, 4.0, 2.0], [5.0, 8.0, 8.0]], [[1.0, 6.0, 6.0], [8.0, 5.0, 7.0], [3.0, 
7.0, 5.0]], [[5.0, 8.0, 3.0], [1.0, 3.0, 6.0], [5.0, 6.0, 7.0]]], [[[7.0, 7.0, 8.0], [1.0, 7.0, 8.0], [1.0, 1.0, 5.0]], [[7.0, 
6.0, 3.0], [7.0, 3.0, 3.0], [7.0, 1.0, 2.0]], [[3.0, 1.0, 1.0], [6.0, 2.0, 1.0], [1.0, 2.0, 7.0]]], [[[4.0, 2.0, 5.0], [2.0, 
2.0, 1.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 8.0], [5.0, 8.0, 1.0], [7.0, 5.0, 5.0]], [[5.0, 3.0, 1.0], [2.0, 3.0, 4.0], [4.0, 8.0, 
2.0]]], [[[3.0, 5.0, 2.0], [3.0, 3.0, 8.0], [6.0, 4.0, 8.0]], [[4.0, 2.0, 7.0], [1.0, 4.0, 1.0], [8.0, 5.0, 2.0]], [[1.0, 1.0, 
5.0], [8.0, 5.0, 8.0], [3.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 7.0], [3.0, 2.0, 5.0], [3.0, 2.0, 5.0]], 
[[4.0, 5.0, 3.0], [5.0, 6.0, 7.0], [1.0, 6.0, 6.0]], [[8.0, 8.0, 3.0], [8.0, 2.0, 8.0], [8.0, 8.0, 4.0]]], [[[8.0, 1.0, 1.0], 
[5.0, 6.0, 8.0], [1.0, 1.0, 1.0]], [[1.0, 7.0, 8.0], [1.0, 1.0, 1.0], [6.0, 8.0, 7.0]], [[7.0, 4.0, 2.0], [8.0, 7.0, 7.0], 
[6.0, 2.0, 3.0]]], [[[7.0, 4.0, 7.0], [4.0, 1.0, 4.0], [3.0, 4.0, 7.0]], [[3.0, 4.0, 2.0], [7.0, 6.0, 2.0], [2.0, 4.0, 3.0]], 
[[3.0, 5.0, 4.0], [5.0, 2.0, 1.0], [1.0, 6.0, 2.0]]], [[[5.0, 4.0, 3.0], [3.0, 4.0, 3.0], [3.0, 8.0, 4.0]], [[2.0, 1.0, 2.0], 
[5.0, 7.0, 2.0], [2.0, 8.0, 7.0]], [[1.0, 4.0, 7.0], [8.0, 4.0, 5.0], [8.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 8.0], [3.0, 5.0, 2.0], [6.0, 2.0, 3.0]], [[3.0, 7.0, 5.0], [7.0, 7.0, 7.0], [4.0, 
6.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 1.0, 7.0], [5.0, 1.0, 3.0]]], [[[8.0, 3.0, 6.0], [2.0, 8.0, 1.0], [7.0, 1.0, 1.0]], [[1.0, 
1.0, 4.0], [5.0, 1.0, 8.0], [3.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], [7.0, 1.0, 4.0], [4.0, 5.0, 2.0]]], [[[7.0, 4.0, 6.0], [2.0, 
1.0, 3.0], [3.0, 5.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 1.0, 8.0], [6.0, 5.0, 8.0]], [[2.0, 5.0, 3.0], [1.0, 4.0, 5.0], [5.0, 4.0, 
8.0]]], [[[3.0, 5.0, 5.0], [5.0, 4.0, 5.0], [1.0, 7.0, 5.0]], [[7.0, 3.0, 7.0], [7.0, 4.0, 2.0], [7.0, 4.0, 2.0]], [[8.0, 1.0, 
5.0], [6.0, 6.0, 4.0], [2.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 6.0], [8.0, 2.0, 8.0], [4.0, 2.0, 7.0]], 
[[2.0, 8.0, 8.0], [2.0, 2.0, 8.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 7.0], [6.0, 6.0, 8.0], [8.0, 2.0, 5.0]]], [[[8.0, 7.0, 2.0], 
[2.0, 3.0, 8.0], [1.0, 7.0, 3.0]], [[4.0, 8.0, 6.0], [5.0, 6.0, 3.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 2.0], [2.0, 8.0, 6.0], 
[6.0, 4.0, 5.0]]], [[[2.0, 3.0, 4.0], [3.0, 6.0, 8.0], [4.0, 5.0, 6.0]], [[8.0, 4.0, 5.0], [6.0, 7.0, 4.0], [4.0, 5.0, 6.0]], 
[[6.0, 8.0, 7.0], [3.0, 2.0, 8.0], [7.0, 8.0, 8.0]]], [[[6.0, 6.0, 4.0], [8.0, 2.0, 4.0], [1.0, 3.0, 1.0]], [[4.0, 2.0, 1.0], 
[2.0, 7.0, 4.0], [7.0, 5.0, 3.0]], [[6.0, 4.0, 6.0], [6.0, 5.0, 3.0], [4.0, 3.0, 3.0]]]])
      val=numpy.array([[5.0, 4.0, 8.0], [5.0, 4.0, 4.0], [8.0, 7.0, 3.0]])
      arg[:3,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 8.0], [3.0, 5.0, 2.0], [6.0, 2.0, 3.0]], [[3.0, 7.0, 5.0], [7.0, 4.0, 7.0], [4.0, 
6.0, 1.0]], [[6.0, 4.0, 6.0], [6.0, 8.0, 7.0], [5.0, 1.0, 3.0]]], [[[8.0, 3.0, 6.0], [2.0, 5.0, 1.0], [7.0, 1.0, 1.0]], [[1.0, 
1.0, 4.0], [5.0, 4.0, 8.0], [3.0, 3.0, 2.0]], [[6.0, 2.0, 4.0], [7.0, 4.0, 4.0], [4.0, 5.0, 2.0]]], [[[7.0, 4.0, 6.0], [2.0, 
8.0, 3.0], [3.0, 5.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 7.0, 8.0], [6.0, 5.0, 8.0]], [[2.0, 5.0, 3.0], [1.0, 3.0, 5.0], [5.0, 4.0, 
8.0]]], [[[3.0, 5.0, 5.0], [5.0, 4.0, 5.0], [1.0, 7.0, 5.0]], [[7.0, 3.0, 7.0], [7.0, 4.0, 2.0], [7.0, 4.0, 2.0]], [[8.0, 1.0, 
5.0], [6.0, 6.0, 4.0], [2.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 6.0], [8.0, 5.0, 8.0], [4.0, 2.0, 7.0]], 
[[2.0, 8.0, 8.0], [2.0, 4.0, 8.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 7.0], [6.0, 8.0, 8.0], [8.0, 2.0, 5.0]]], [[[8.0, 7.0, 2.0], 
[2.0, 5.0, 8.0], [1.0, 7.0, 3.0]], [[4.0, 8.0, 6.0], [5.0, 4.0, 3.0], [7.0, 1.0, 7.0]], [[6.0, 6.0, 2.0], [2.0, 4.0, 6.0], 
[6.0, 4.0, 5.0]]], [[[2.0, 3.0, 4.0], [3.0, 8.0, 8.0], [4.0, 5.0, 6.0]], [[8.0, 4.0, 5.0], [6.0, 7.0, 4.0], [4.0, 5.0, 6.0]], 
[[6.0, 8.0, 7.0], [3.0, 3.0, 8.0], [7.0, 8.0, 8.0]]], [[[6.0, 6.0, 4.0], [8.0, 2.0, 4.0], [1.0, 3.0, 1.0]], [[4.0, 2.0, 1.0], 
[2.0, 7.0, 4.0], [7.0, 5.0, 3.0]], [[6.0, 4.0, 6.0], [6.0, 5.0, 3.0], [4.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 8.0], [2.0, 4.0, 3.0], [5.0, 4.0, 6.0]], [[3.0, 3.0, 4.0], [5.0, 1.0, 6.0], [5.0, 
6.0, 5.0]], [[5.0, 8.0, 5.0], [1.0, 8.0, 2.0], [6.0, 2.0, 1.0]]], [[[2.0, 2.0, 6.0], [1.0, 2.0, 1.0], [4.0, 5.0, 5.0]], [[4.0, 
1.0, 1.0], [3.0, 8.0, 5.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 2.0], [7.0, 2.0, 3.0], [1.0, 8.0, 2.0]]], [[[4.0, 2.0, 2.0], [1.0, 
3.0, 1.0], [5.0, 6.0, 2.0]], [[6.0, 4.0, 1.0], [2.0, 7.0, 2.0], [1.0, 2.0, 1.0]], [[8.0, 6.0, 1.0], [7.0, 4.0, 8.0], [3.0, 2.0, 
5.0]]], [[[1.0, 7.0, 6.0], [3.0, 8.0, 6.0], [1.0, 4.0, 5.0]], [[6.0, 4.0, 4.0], [5.0, 8.0, 3.0], [1.0, 7.0, 8.0]], [[6.0, 5.0, 
3.0], [3.0, 6.0, 8.0], [2.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 8.0], [4.0, 4.0, 7.0], [4.0, 1.0, 7.0]], 
[[6.0, 3.0, 3.0], [7.0, 6.0, 4.0], [8.0, 1.0, 6.0]], [[3.0, 5.0, 6.0], [3.0, 8.0, 1.0], [7.0, 6.0, 6.0]]], [[[2.0, 8.0, 3.0], 
[6.0, 6.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 7.0, 3.0], [6.0, 4.0, 4.0], [5.0, 4.0, 7.0]], [[1.0, 2.0, 2.0], [4.0, 8.0, 2.0], 
[1.0, 8.0, 3.0]]], [[[1.0, 8.0, 4.0], [4.0, 3.0, 6.0], [1.0, 4.0, 2.0]], [[3.0, 6.0, 6.0], [2.0, 6.0, 4.0], [3.0, 8.0, 8.0]], 
[[1.0, 6.0, 6.0], [1.0, 5.0, 5.0], [2.0, 1.0, 5.0]]], [[[5.0, 3.0, 2.0], [8.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 6.0], 
[7.0, 3.0, 1.0], [2.0, 2.0, 3.0]], [[3.0, 3.0, 4.0], [1.0, 4.0, 8.0], [5.0, 2.0, 7.0]]]])
      val=Data(numpy.array([[8.0, 7.0, 6.0], [8.0, 4.0, 6.0], [1.0, 4.0, 6.0]]),self.functionspace)
      arg[:3,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 8.0], [2.0, 8.0, 3.0], [5.0, 4.0, 6.0]], [[3.0, 3.0, 4.0], [5.0, 7.0, 6.0], [5.0, 
6.0, 5.0]], [[5.0, 8.0, 5.0], [1.0, 6.0, 2.0], [6.0, 2.0, 1.0]]], [[[2.0, 2.0, 6.0], [1.0, 8.0, 1.0], [4.0, 5.0, 5.0]], [[4.0, 
1.0, 1.0], [3.0, 4.0, 5.0], [1.0, 1.0, 8.0]], [[5.0, 5.0, 2.0], [7.0, 6.0, 3.0], [1.0, 8.0, 2.0]]], [[[4.0, 2.0, 2.0], [1.0, 
1.0, 1.0], [5.0, 6.0, 2.0]], [[6.0, 4.0, 1.0], [2.0, 4.0, 2.0], [1.0, 2.0, 1.0]], [[8.0, 6.0, 1.0], [7.0, 6.0, 8.0], [3.0, 2.0, 
5.0]]], [[[1.0, 7.0, 6.0], [3.0, 8.0, 6.0], [1.0, 4.0, 5.0]], [[6.0, 4.0, 4.0], [5.0, 8.0, 3.0], [1.0, 7.0, 8.0]], [[6.0, 5.0, 
3.0], [3.0, 6.0, 8.0], [2.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 8.0], [4.0, 8.0, 7.0], [4.0, 1.0, 7.0]], 
[[6.0, 3.0, 3.0], [7.0, 7.0, 4.0], [8.0, 1.0, 6.0]], [[3.0, 5.0, 6.0], [3.0, 6.0, 1.0], [7.0, 6.0, 6.0]]], [[[2.0, 8.0, 3.0], 
[6.0, 8.0, 6.0], [3.0, 6.0, 6.0]], [[1.0, 7.0, 3.0], [6.0, 4.0, 4.0], [5.0, 4.0, 7.0]], [[1.0, 2.0, 2.0], [4.0, 6.0, 2.0], 
[1.0, 8.0, 3.0]]], [[[1.0, 8.0, 4.0], [4.0, 1.0, 6.0], [1.0, 4.0, 2.0]], [[3.0, 6.0, 6.0], [2.0, 4.0, 4.0], [3.0, 8.0, 8.0]], 
[[1.0, 6.0, 6.0], [1.0, 6.0, 5.0], [2.0, 1.0, 5.0]]], [[[5.0, 3.0, 2.0], [8.0, 8.0, 7.0], [1.0, 2.0, 8.0]], [[6.0, 6.0, 6.0], 
[7.0, 3.0, 1.0], [2.0, 2.0, 3.0]], [[3.0, 3.0, 4.0], [1.0, 4.0, 8.0], [5.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 5.0], [1.0, 3.0, 1.0], [2.0, 2.0, 6.0]], [[8.0, 1.0, 1.0], [5.0, 3.0, 3.0], [1.0, 
4.0, 2.0]], [[2.0, 4.0, 3.0], [6.0, 7.0, 5.0], [7.0, 7.0, 4.0]]], [[[6.0, 5.0, 8.0], [7.0, 3.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 
7.0, 6.0], [6.0, 8.0, 3.0], [4.0, 7.0, 8.0]], [[3.0, 2.0, 4.0], [3.0, 2.0, 8.0], [8.0, 8.0, 6.0]]], [[[8.0, 3.0, 6.0], [6.0, 
5.0, 2.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], [4.0, 4.0, 6.0], [3.0, 3.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 2.0, 3.0], [1.0, 2.0, 
1.0]]], [[[7.0, 7.0, 2.0], [6.0, 4.0, 2.0], [6.0, 1.0, 5.0]], [[7.0, 3.0, 7.0], [7.0, 5.0, 5.0], [2.0, 3.0, 8.0]], [[1.0, 2.0, 
2.0], [1.0, 1.0, 5.0], [7.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 2.0], [4.0, 8.0, 3.0], [5.0, 7.0, 3.0]], 
[[6.0, 2.0, 3.0], [3.0, 2.0, 7.0], [4.0, 6.0, 3.0]], [[7.0, 5.0, 6.0], [4.0, 7.0, 6.0], [1.0, 6.0, 8.0]]], [[[7.0, 5.0, 2.0], 
[3.0, 3.0, 4.0], [7.0, 3.0, 2.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 8.0], [3.0, 7.0, 8.0]], [[8.0, 2.0, 3.0], [1.0, 4.0, 4.0], 
[2.0, 4.0, 2.0]]], [[[4.0, 7.0, 5.0], [5.0, 1.0, 1.0], [4.0, 2.0, 3.0]], [[1.0, 7.0, 7.0], [7.0, 6.0, 1.0], [2.0, 6.0, 2.0]], 
[[8.0, 1.0, 7.0], [1.0, 4.0, 7.0], [3.0, 8.0, 5.0]]], [[[4.0, 2.0, 5.0], [2.0, 3.0, 1.0], [2.0, 5.0, 6.0]], [[7.0, 6.0, 4.0], 
[7.0, 5.0, 1.0], [4.0, 6.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 4.0, 7.0], [4.0, 3.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 8.0, 2.0], [2.0, 2.0, 5.0], [3.0, 8.0, 2.0]])+(1.-msk_val)*numpy.array([[8.0, 4.0, 3.0], 
[2.0, 1.0, 6.0], [8.0, 4.0, 5.0]])
      arg[:3,:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 5.0], [1.0, 7.0, 1.0], [2.0, 2.0, 6.0]], [[8.0, 1.0, 1.0], [5.0, 8.0, 3.0], [1.0, 
4.0, 2.0]], [[2.0, 4.0, 3.0], [6.0, 2.0, 5.0], [7.0, 7.0, 4.0]]], [[[6.0, 5.0, 8.0], [7.0, 2.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 
7.0, 6.0], [6.0, 2.0, 3.0], [4.0, 7.0, 8.0]], [[3.0, 2.0, 4.0], [3.0, 5.0, 8.0], [8.0, 8.0, 6.0]]], [[[8.0, 3.0, 6.0], [6.0, 
3.0, 2.0], [1.0, 1.0, 8.0]], [[3.0, 1.0, 6.0], [4.0, 8.0, 6.0], [3.0, 3.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 2.0, 3.0], [1.0, 2.0, 
1.0]]], [[[7.0, 7.0, 2.0], [6.0, 4.0, 2.0], [6.0, 1.0, 5.0]], [[7.0, 3.0, 7.0], [7.0, 5.0, 5.0], [2.0, 3.0, 8.0]], [[1.0, 2.0, 
2.0], [1.0, 1.0, 5.0], [7.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 2.0], [4.0, 8.0, 3.0], [5.0, 7.0, 3.0]], 
[[6.0, 2.0, 3.0], [3.0, 4.0, 7.0], [4.0, 6.0, 3.0]], [[7.0, 5.0, 6.0], [4.0, 3.0, 6.0], [1.0, 6.0, 8.0]]], [[[7.0, 5.0, 2.0], 
[3.0, 2.0, 4.0], [7.0, 3.0, 2.0]], [[6.0, 8.0, 1.0], [7.0, 1.0, 8.0], [3.0, 7.0, 8.0]], [[8.0, 2.0, 3.0], [1.0, 6.0, 4.0], 
[2.0, 4.0, 2.0]]], [[[4.0, 7.0, 5.0], [5.0, 8.0, 1.0], [4.0, 2.0, 3.0]], [[1.0, 7.0, 7.0], [7.0, 4.0, 1.0], [2.0, 6.0, 2.0]], 
[[8.0, 1.0, 7.0], [1.0, 5.0, 7.0], [3.0, 8.0, 5.0]]], [[[4.0, 2.0, 5.0], [2.0, 3.0, 1.0], [2.0, 5.0, 6.0]], [[7.0, 6.0, 4.0], 
[7.0, 5.0, 1.0], [4.0, 6.0, 6.0]], [[3.0, 6.0, 4.0], [1.0, 4.0, 7.0], [4.0, 3.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 7.0], [5.0, 7.0, 5.0], [2.0, 1.0, 7.0]], [[4.0, 7.0, 8.0], [8.0, 1.0, 7.0], [2.0, 
4.0, 5.0]], [[4.0, 4.0, 6.0], [7.0, 3.0, 4.0], [5.0, 6.0, 8.0]]], [[[3.0, 3.0, 7.0], [2.0, 4.0, 4.0], [3.0, 4.0, 1.0]], [[2.0, 
2.0, 7.0], [8.0, 4.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 8.0], [3.0, 3.0, 5.0], [1.0, 5.0, 8.0]]], [[[3.0, 6.0, 4.0], [8.0, 
6.0, 4.0], [6.0, 4.0, 5.0]], [[6.0, 8.0, 3.0], [6.0, 4.0, 6.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 3.0], [1.0, 8.0, 7.0], [2.0, 5.0, 
1.0]]], [[[2.0, 7.0, 1.0], [3.0, 7.0, 1.0], [7.0, 2.0, 7.0]], [[7.0, 3.0, 3.0], [8.0, 6.0, 2.0], [2.0, 5.0, 5.0]], [[8.0, 7.0, 
4.0], [3.0, 6.0, 2.0], [4.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 4.0], [3.0, 4.0, 6.0], [6.0, 5.0, 8.0]], 
[[7.0, 6.0, 1.0], [2.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 6.0, 8.0], [8.0, 8.0, 7.0], [1.0, 6.0, 2.0]]], [[[5.0, 5.0, 2.0], 
[2.0, 8.0, 3.0], [6.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [6.0, 6.0, 7.0], [1.0, 4.0, 7.0]], [[5.0, 2.0, 8.0], [5.0, 7.0, 5.0], 
[1.0, 6.0, 5.0]]], [[[2.0, 2.0, 8.0], [2.0, 3.0, 2.0], [5.0, 1.0, 8.0]], [[4.0, 6.0, 1.0], [1.0, 8.0, 1.0], [6.0, 3.0, 7.0]], 
[[4.0, 7.0, 8.0], [7.0, 1.0, 5.0], [3.0, 8.0, 4.0]]], [[[2.0, 2.0, 6.0], [2.0, 3.0, 4.0], [5.0, 1.0, 2.0]], [[8.0, 8.0, 2.0], 
[4.0, 5.0, 1.0], [6.0, 4.0, 3.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 6.0], [6.0, 1.0, 5.0]]]])
      val=numpy.array([[[3.0, 2.0, 1.0], [6.0, 4.0, 7.0]], [[7.0, 4.0, 7.0], [6.0, 1.0, 5.0]]])
      arg[2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 7.0], [5.0, 7.0, 5.0], [2.0, 1.0, 7.0]], [[4.0, 7.0, 8.0], [8.0, 1.0, 7.0], [2.0, 
4.0, 5.0]], [[4.0, 4.0, 6.0], [7.0, 3.0, 4.0], [5.0, 6.0, 8.0]]], [[[3.0, 3.0, 7.0], [2.0, 4.0, 4.0], [3.0, 4.0, 1.0]], [[2.0, 
2.0, 7.0], [8.0, 4.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 1.0, 8.0], [3.0, 3.0, 5.0], [1.0, 5.0, 8.0]]], [[[3.0, 2.0, 1.0], [6.0, 
4.0, 7.0], [6.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [6.0, 1.0, 5.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 3.0], [1.0, 8.0, 7.0], [2.0, 5.0, 
1.0]]], [[[2.0, 7.0, 1.0], [3.0, 7.0, 1.0], [7.0, 2.0, 7.0]], [[7.0, 3.0, 3.0], [8.0, 6.0, 2.0], [2.0, 5.0, 5.0]], [[8.0, 7.0, 
4.0], [3.0, 6.0, 2.0], [4.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 4.0], [3.0, 4.0, 6.0], [6.0, 5.0, 8.0]], 
[[7.0, 6.0, 1.0], [2.0, 7.0, 4.0], [5.0, 5.0, 4.0]], [[2.0, 6.0, 8.0], [8.0, 8.0, 7.0], [1.0, 6.0, 2.0]]], [[[5.0, 5.0, 2.0], 
[2.0, 8.0, 3.0], [6.0, 8.0, 4.0]], [[4.0, 7.0, 5.0], [6.0, 6.0, 7.0], [1.0, 4.0, 7.0]], [[5.0, 2.0, 8.0], [5.0, 7.0, 5.0], 
[1.0, 6.0, 5.0]]], [[[3.0, 2.0, 1.0], [6.0, 4.0, 7.0], [5.0, 1.0, 8.0]], [[7.0, 4.0, 7.0], [6.0, 1.0, 5.0], [6.0, 3.0, 7.0]], 
[[4.0, 7.0, 8.0], [7.0, 1.0, 5.0], [3.0, 8.0, 4.0]]], [[[2.0, 2.0, 6.0], [2.0, 3.0, 4.0], [5.0, 1.0, 2.0]], [[8.0, 8.0, 2.0], 
[4.0, 5.0, 1.0], [6.0, 4.0, 3.0]], [[2.0, 4.0, 3.0], [4.0, 3.0, 6.0], [6.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 6.0], [4.0, 4.0, 2.0], [6.0, 6.0, 4.0]], [[1.0, 7.0, 1.0], [5.0, 2.0, 5.0], [5.0, 
1.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 7.0, 1.0], [1.0, 4.0, 1.0]]], [[[7.0, 6.0, 1.0], [4.0, 4.0, 5.0], [7.0, 8.0, 5.0]], [[1.0, 
5.0, 7.0], [6.0, 5.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 7.0, 2.0], [7.0, 2.0, 2.0], [7.0, 4.0, 4.0]]], [[[3.0, 5.0, 2.0], [2.0, 
5.0, 8.0], [2.0, 3.0, 8.0]], [[7.0, 3.0, 5.0], [3.0, 6.0, 6.0], [6.0, 8.0, 2.0]], [[5.0, 8.0, 3.0], [8.0, 6.0, 3.0], [4.0, 8.0, 
8.0]]], [[[2.0, 8.0, 2.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]], [[5.0, 4.0, 3.0], [5.0, 3.0, 2.0], [4.0, 7.0, 2.0]], [[3.0, 4.0, 
6.0], [8.0, 2.0, 3.0], [3.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 8.0], [7.0, 8.0, 6.0], [6.0, 5.0, 5.0]], 
[[2.0, 2.0, 7.0], [2.0, 2.0, 8.0], [7.0, 4.0, 1.0]], [[6.0, 3.0, 5.0], [3.0, 3.0, 2.0], [3.0, 1.0, 4.0]]], [[[5.0, 1.0, 6.0], 
[5.0, 5.0, 6.0], [1.0, 3.0, 1.0]], [[7.0, 4.0, 2.0], [8.0, 7.0, 8.0], [3.0, 3.0, 6.0]], [[7.0, 1.0, 4.0], [8.0, 4.0, 5.0], 
[5.0, 4.0, 5.0]]], [[[2.0, 6.0, 7.0], [5.0, 7.0, 3.0], [4.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [6.0, 3.0, 3.0], [3.0, 4.0, 4.0]], 
[[1.0, 2.0, 2.0], [3.0, 7.0, 2.0], [8.0, 7.0, 3.0]]], [[[2.0, 6.0, 5.0], [6.0, 3.0, 2.0], [7.0, 7.0, 7.0]], [[4.0, 8.0, 3.0], 
[6.0, 1.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 3.0, 2.0], [7.0, 3.0, 6.0], [5.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[4.0, 6.0, 3.0], [3.0, 3.0, 4.0]], [[7.0, 8.0, 2.0], [2.0, 1.0, 6.0]]]),self.functionspace)
      arg[2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 6.0], [4.0, 4.0, 2.0], [6.0, 6.0, 4.0]], [[1.0, 7.0, 1.0], [5.0, 2.0, 5.0], [5.0, 
1.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 7.0, 1.0], [1.0, 4.0, 1.0]]], [[[7.0, 6.0, 1.0], [4.0, 4.0, 5.0], [7.0, 8.0, 5.0]], [[1.0, 
5.0, 7.0], [6.0, 5.0, 8.0], [3.0, 3.0, 2.0]], [[3.0, 7.0, 2.0], [7.0, 2.0, 2.0], [7.0, 4.0, 4.0]]], [[[4.0, 6.0, 3.0], [3.0, 
3.0, 4.0], [2.0, 3.0, 8.0]], [[7.0, 8.0, 2.0], [2.0, 1.0, 6.0], [6.0, 8.0, 2.0]], [[5.0, 8.0, 3.0], [8.0, 6.0, 3.0], [4.0, 8.0, 
8.0]]], [[[2.0, 8.0, 2.0], [8.0, 6.0, 8.0], [8.0, 6.0, 5.0]], [[5.0, 4.0, 3.0], [5.0, 3.0, 2.0], [4.0, 7.0, 2.0]], [[3.0, 4.0, 
6.0], [8.0, 2.0, 3.0], [3.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 8.0], [7.0, 8.0, 6.0], [6.0, 5.0, 5.0]], 
[[2.0, 2.0, 7.0], [2.0, 2.0, 8.0], [7.0, 4.0, 1.0]], [[6.0, 3.0, 5.0], [3.0, 3.0, 2.0], [3.0, 1.0, 4.0]]], [[[5.0, 1.0, 6.0], 
[5.0, 5.0, 6.0], [1.0, 3.0, 1.0]], [[7.0, 4.0, 2.0], [8.0, 7.0, 8.0], [3.0, 3.0, 6.0]], [[7.0, 1.0, 4.0], [8.0, 4.0, 5.0], 
[5.0, 4.0, 5.0]]], [[[4.0, 6.0, 3.0], [3.0, 3.0, 4.0], [4.0, 5.0, 4.0]], [[7.0, 8.0, 2.0], [2.0, 1.0, 6.0], [3.0, 4.0, 4.0]], 
[[1.0, 2.0, 2.0], [3.0, 7.0, 2.0], [8.0, 7.0, 3.0]]], [[[2.0, 6.0, 5.0], [6.0, 3.0, 2.0], [7.0, 7.0, 7.0]], [[4.0, 8.0, 3.0], 
[6.0, 1.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 3.0, 2.0], [7.0, 3.0, 6.0], [5.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 8.0, 8.0], [4.0, 4.0, 3.0]], [[5.0, 3.0, 7.0], [7.0, 2.0, 3.0], [7.0, 
7.0, 4.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 1.0], [6.0, 6.0, 1.0]]], [[[2.0, 8.0, 7.0], [8.0, 6.0, 3.0], [2.0, 4.0, 8.0]], [[4.0, 
6.0, 5.0], [3.0, 3.0, 6.0], [5.0, 3.0, 4.0]], [[3.0, 7.0, 6.0], [1.0, 4.0, 7.0], [2.0, 1.0, 5.0]]], [[[8.0, 7.0, 8.0], [6.0, 
7.0, 6.0], [5.0, 1.0, 7.0]], [[5.0, 7.0, 4.0], [8.0, 3.0, 4.0], [7.0, 6.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 4.0], [2.0, 6.0, 
8.0]]], [[[2.0, 5.0, 3.0], [7.0, 8.0, 7.0], [5.0, 5.0, 3.0]], [[1.0, 3.0, 4.0], [7.0, 5.0, 2.0], [4.0, 6.0, 7.0]], [[3.0, 3.0, 
1.0], [7.0, 5.0, 1.0], [5.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 6.0], [8.0, 6.0, 8.0], [7.0, 7.0, 2.0]], 
[[8.0, 6.0, 3.0], [4.0, 4.0, 3.0], [7.0, 6.0, 2.0]], [[8.0, 2.0, 7.0], [3.0, 8.0, 3.0], [3.0, 3.0, 1.0]]], [[[8.0, 2.0, 8.0], 
[3.0, 5.0, 1.0], [4.0, 7.0, 1.0]], [[8.0, 1.0, 4.0], [2.0, 3.0, 3.0], [2.0, 7.0, 1.0]], [[1.0, 2.0, 6.0], [6.0, 1.0, 8.0], 
[8.0, 2.0, 2.0]]], [[[4.0, 4.0, 5.0], [8.0, 3.0, 5.0], [4.0, 5.0, 8.0]], [[7.0, 6.0, 3.0], [5.0, 6.0, 2.0], [8.0, 8.0, 2.0]], 
[[6.0, 8.0, 7.0], [8.0, 7.0, 4.0], [6.0, 6.0, 4.0]]], [[[7.0, 7.0, 5.0], [1.0, 8.0, 4.0], [2.0, 5.0, 4.0]], [[2.0, 5.0, 7.0], 
[4.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[6.0, 7.0, 5.0], [3.0, 8.0, 1.0], [4.0, 4.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 4.0, 8.0], [2.0, 5.0, 8.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 
5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 5.0, 6.0], [3.0, 1.0, 7.0]], [[2.0, 8.0, 8.0], [8.0, 7.0, 3.0]]])
      arg[2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 8.0, 8.0], [4.0, 4.0, 3.0]], [[5.0, 3.0, 7.0], [7.0, 2.0, 3.0], [7.0, 
7.0, 4.0]], [[4.0, 5.0, 4.0], [5.0, 1.0, 1.0], [6.0, 6.0, 1.0]]], [[[2.0, 8.0, 7.0], [8.0, 6.0, 3.0], [2.0, 4.0, 8.0]], [[4.0, 
6.0, 5.0], [3.0, 3.0, 6.0], [5.0, 3.0, 4.0]], [[3.0, 7.0, 6.0], [1.0, 4.0, 7.0], [2.0, 1.0, 5.0]]], [[[7.0, 4.0, 8.0], [2.0, 
5.0, 8.0], [5.0, 1.0, 7.0]], [[3.0, 8.0, 5.0], [4.0, 3.0, 5.0], [7.0, 6.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 4.0], [2.0, 6.0, 
8.0]]], [[[2.0, 5.0, 3.0], [7.0, 8.0, 7.0], [5.0, 5.0, 3.0]], [[1.0, 3.0, 4.0], [7.0, 5.0, 2.0], [4.0, 6.0, 7.0]], [[3.0, 3.0, 
1.0], [7.0, 5.0, 1.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 6.0], [8.0, 6.0, 8.0], [7.0, 7.0, 2.0]], 
[[8.0, 6.0, 3.0], [4.0, 4.0, 3.0], [7.0, 6.0, 2.0]], [[8.0, 2.0, 7.0], [3.0, 8.0, 3.0], [3.0, 3.0, 1.0]]], [[[8.0, 2.0, 8.0], 
[3.0, 5.0, 1.0], [4.0, 7.0, 1.0]], [[8.0, 1.0, 4.0], [2.0, 3.0, 3.0], [2.0, 7.0, 1.0]], [[1.0, 2.0, 6.0], [6.0, 1.0, 8.0], 
[8.0, 2.0, 2.0]]], [[[8.0, 5.0, 6.0], [3.0, 1.0, 7.0], [4.0, 5.0, 8.0]], [[2.0, 8.0, 8.0], [8.0, 7.0, 3.0], [8.0, 8.0, 2.0]], 
[[6.0, 8.0, 7.0], [8.0, 7.0, 4.0], [6.0, 6.0, 4.0]]], [[[7.0, 7.0, 5.0], [1.0, 8.0, 4.0], [2.0, 5.0, 4.0]], [[2.0, 5.0, 7.0], 
[4.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[6.0, 7.0, 5.0], [3.0, 8.0, 1.0], [4.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 2.0], [8.0, 3.0, 2.0], [3.0, 2.0, 3.0]], [[8.0, 7.0, 2.0], [3.0, 8.0, 5.0], [3.0, 
5.0, 7.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 4.0], [6.0, 8.0, 7.0]]], [[[4.0, 5.0, 4.0], [3.0, 2.0, 8.0], [4.0, 3.0, 4.0]], [[3.0, 
6.0, 3.0], [5.0, 2.0, 7.0], [2.0, 4.0, 6.0]], [[5.0, 6.0, 2.0], [8.0, 4.0, 7.0], [7.0, 7.0, 8.0]]], [[[3.0, 7.0, 8.0], [8.0, 
8.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 3.0, 2.0], [2.0, 4.0, 8.0], [4.0, 8.0, 6.0]], [[6.0, 8.0, 8.0], [5.0, 4.0, 7.0], [8.0, 1.0, 
8.0]]], [[[5.0, 7.0, 1.0], [1.0, 2.0, 8.0], [7.0, 4.0, 5.0]], [[8.0, 5.0, 4.0], [2.0, 5.0, 5.0], [5.0, 7.0, 3.0]], [[5.0, 3.0, 
7.0], [4.0, 4.0, 8.0], [6.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 2.0], [2.0, 5.0, 3.0], [5.0, 4.0, 4.0]], 
[[7.0, 3.0, 6.0], [3.0, 7.0, 6.0], [3.0, 5.0, 1.0]], [[6.0, 4.0, 3.0], [4.0, 8.0, 7.0], [1.0, 6.0, 3.0]]], [[[1.0, 7.0, 6.0], 
[1.0, 6.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 4.0, 6.0], [2.0, 8.0, 1.0], [2.0, 4.0, 8.0]], [[6.0, 5.0, 4.0], [1.0, 3.0, 4.0], 
[5.0, 2.0, 8.0]]], [[[6.0, 6.0, 7.0], [2.0, 3.0, 1.0], [2.0, 8.0, 1.0]], [[3.0, 1.0, 3.0], [3.0, 3.0, 6.0], [4.0, 7.0, 1.0]], 
[[5.0, 3.0, 6.0], [4.0, 7.0, 4.0], [5.0, 6.0, 1.0]]], [[[2.0, 4.0, 3.0], [2.0, 5.0, 8.0], [1.0, 7.0, 8.0]], [[4.0, 8.0, 8.0], 
[1.0, 6.0, 1.0], [4.0, 3.0, 7.0]], [[3.0, 5.0, 4.0], [6.0, 7.0, 5.0], [7.0, 4.0, 6.0]]]])
      val=numpy.array([[[[5.0]]], [[[3.0]]], [[[7.0]]], [[[6.0]]]])
      arg[:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 2.0], [8.0, 3.0, 2.0], [3.0, 2.0, 3.0]], [[8.0, 7.0, 2.0], [3.0, 8.0, 5.0], [3.0, 
5.0, 7.0]], [[7.0, 2.0, 8.0], [3.0, 4.0, 4.0], [6.0, 8.0, 5.0]]], [[[4.0, 5.0, 4.0], [3.0, 2.0, 8.0], [4.0, 3.0, 4.0]], [[3.0, 
6.0, 3.0], [5.0, 2.0, 7.0], [2.0, 4.0, 6.0]], [[5.0, 6.0, 2.0], [8.0, 4.0, 7.0], [7.0, 7.0, 3.0]]], [[[3.0, 7.0, 8.0], [8.0, 
8.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 3.0, 2.0], [2.0, 4.0, 8.0], [4.0, 8.0, 6.0]], [[6.0, 8.0, 8.0], [5.0, 4.0, 7.0], [8.0, 1.0, 
7.0]]], [[[5.0, 7.0, 1.0], [1.0, 2.0, 8.0], [7.0, 4.0, 5.0]], [[8.0, 5.0, 4.0], [2.0, 5.0, 5.0], [5.0, 7.0, 3.0]], [[5.0, 3.0, 
7.0], [4.0, 4.0, 8.0], [6.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 2.0], [2.0, 5.0, 3.0], [5.0, 4.0, 4.0]], 
[[7.0, 3.0, 6.0], [3.0, 7.0, 6.0], [3.0, 5.0, 1.0]], [[6.0, 4.0, 3.0], [4.0, 8.0, 7.0], [1.0, 6.0, 5.0]]], [[[1.0, 7.0, 6.0], 
[1.0, 6.0, 2.0], [7.0, 1.0, 2.0]], [[4.0, 4.0, 6.0], [2.0, 8.0, 1.0], [2.0, 4.0, 8.0]], [[6.0, 5.0, 4.0], [1.0, 3.0, 4.0], 
[5.0, 2.0, 3.0]]], [[[6.0, 6.0, 7.0], [2.0, 3.0, 1.0], [2.0, 8.0, 1.0]], [[3.0, 1.0, 3.0], [3.0, 3.0, 6.0], [4.0, 7.0, 1.0]], 
[[5.0, 3.0, 6.0], [4.0, 7.0, 4.0], [5.0, 6.0, 7.0]]], [[[2.0, 4.0, 3.0], [2.0, 5.0, 8.0], [1.0, 7.0, 8.0]], [[4.0, 8.0, 8.0], 
[1.0, 6.0, 1.0], [4.0, 3.0, 7.0]], [[3.0, 5.0, 4.0], [6.0, 7.0, 5.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 6.0], [7.0, 6.0, 5.0], [2.0, 5.0, 1.0]], [[5.0, 1.0, 3.0], [4.0, 2.0, 7.0], [1.0, 
2.0, 7.0]], [[4.0, 3.0, 7.0], [5.0, 4.0, 4.0], [1.0, 1.0, 4.0]]], [[[1.0, 3.0, 5.0], [7.0, 4.0, 3.0], [7.0, 2.0, 8.0]], [[5.0, 
6.0, 5.0], [7.0, 7.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 8.0], [1.0, 1.0, 3.0]]], [[[5.0, 2.0, 2.0], [2.0, 
7.0, 4.0], [2.0, 4.0, 2.0]], [[6.0, 7.0, 4.0], [7.0, 5.0, 6.0], [3.0, 5.0, 7.0]], [[8.0, 2.0, 2.0], [6.0, 6.0, 1.0], [1.0, 8.0, 
3.0]]], [[[7.0, 3.0, 5.0], [5.0, 8.0, 5.0], [1.0, 5.0, 4.0]], [[3.0, 3.0, 8.0], [5.0, 5.0, 4.0], [5.0, 2.0, 7.0]], [[1.0, 5.0, 
3.0], [6.0, 7.0, 6.0], [6.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 7.0], [4.0, 3.0, 3.0], [6.0, 4.0, 3.0]], 
[[6.0, 7.0, 4.0], [1.0, 4.0, 8.0], [3.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [5.0, 7.0, 7.0], [5.0, 7.0, 8.0]]], [[[7.0, 7.0, 1.0], 
[1.0, 6.0, 4.0], [7.0, 4.0, 1.0]], [[7.0, 4.0, 7.0], [2.0, 4.0, 6.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 5.0], [6.0, 8.0, 3.0], 
[8.0, 5.0, 4.0]]], [[[5.0, 5.0, 6.0], [2.0, 2.0, 3.0], [5.0, 7.0, 5.0]], [[4.0, 5.0, 3.0], [2.0, 1.0, 2.0], [5.0, 4.0, 5.0]], 
[[5.0, 5.0, 4.0], [2.0, 8.0, 4.0], [4.0, 6.0, 6.0]]], [[[7.0, 6.0, 5.0], [7.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[5.0, 5.0, 4.0], 
[1.0, 4.0, 3.0], [7.0, 2.0, 4.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 2.0], [2.0, 2.0, 3.0]]]])
      val=Data(numpy.array([[[[2.0]]], [[[2.0]]], [[[4.0]]], [[[1.0]]]]),self.functionspace)
      arg[:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 6.0], [7.0, 6.0, 5.0], [2.0, 5.0, 1.0]], [[5.0, 1.0, 3.0], [4.0, 2.0, 7.0], [1.0, 
2.0, 7.0]], [[4.0, 3.0, 7.0], [5.0, 4.0, 4.0], [1.0, 1.0, 2.0]]], [[[1.0, 3.0, 5.0], [7.0, 4.0, 3.0], [7.0, 2.0, 8.0]], [[5.0, 
6.0, 5.0], [7.0, 7.0, 2.0], [8.0, 7.0, 8.0]], [[7.0, 5.0, 1.0], [2.0, 3.0, 8.0], [1.0, 1.0, 2.0]]], [[[5.0, 2.0, 2.0], [2.0, 
7.0, 4.0], [2.0, 4.0, 2.0]], [[6.0, 7.0, 4.0], [7.0, 5.0, 6.0], [3.0, 5.0, 7.0]], [[8.0, 2.0, 2.0], [6.0, 6.0, 1.0], [1.0, 8.0, 
4.0]]], [[[7.0, 3.0, 5.0], [5.0, 8.0, 5.0], [1.0, 5.0, 4.0]], [[3.0, 3.0, 8.0], [5.0, 5.0, 4.0], [5.0, 2.0, 7.0]], [[1.0, 5.0, 
3.0], [6.0, 7.0, 6.0], [6.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 7.0], [4.0, 3.0, 3.0], [6.0, 4.0, 3.0]], 
[[6.0, 7.0, 4.0], [1.0, 4.0, 8.0], [3.0, 7.0, 5.0]], [[2.0, 7.0, 8.0], [5.0, 7.0, 7.0], [5.0, 7.0, 2.0]]], [[[7.0, 7.0, 1.0], 
[1.0, 6.0, 4.0], [7.0, 4.0, 1.0]], [[7.0, 4.0, 7.0], [2.0, 4.0, 6.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 5.0], [6.0, 8.0, 3.0], 
[8.0, 5.0, 2.0]]], [[[5.0, 5.0, 6.0], [2.0, 2.0, 3.0], [5.0, 7.0, 5.0]], [[4.0, 5.0, 3.0], [2.0, 1.0, 2.0], [5.0, 4.0, 5.0]], 
[[5.0, 5.0, 4.0], [2.0, 8.0, 4.0], [4.0, 6.0, 4.0]]], [[[7.0, 6.0, 5.0], [7.0, 3.0, 1.0], [8.0, 6.0, 7.0]], [[5.0, 5.0, 4.0], 
[1.0, 4.0, 3.0], [7.0, 2.0, 4.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 2.0], [2.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLLL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 6.0], [2.0, 6.0, 8.0], [4.0, 1.0, 4.0]], [[7.0, 6.0, 2.0], [8.0, 7.0, 6.0], [6.0, 
4.0, 2.0]], [[8.0, 4.0, 2.0], [2.0, 2.0, 7.0], [2.0, 6.0, 6.0]]], [[[1.0, 2.0, 3.0], [7.0, 7.0, 1.0], [5.0, 1.0, 6.0]], [[8.0, 
5.0, 1.0], [3.0, 2.0, 7.0], [8.0, 1.0, 1.0]], [[6.0, 5.0, 6.0], [2.0, 7.0, 8.0], [3.0, 8.0, 5.0]]], [[[8.0, 7.0, 8.0], [4.0, 
6.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 6.0, 8.0], [6.0, 4.0, 1.0], [7.0, 8.0, 6.0]], [[3.0, 2.0, 7.0], [2.0, 2.0, 1.0], [8.0, 5.0, 
6.0]]], [[[6.0, 2.0, 6.0], [1.0, 3.0, 3.0], [8.0, 7.0, 4.0]], [[2.0, 3.0, 1.0], [3.0, 2.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 
1.0], [1.0, 8.0, 2.0], [4.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 1.0], [4.0, 7.0, 8.0], [7.0, 3.0, 1.0]], 
[[6.0, 4.0, 1.0], [5.0, 4.0, 5.0], [3.0, 8.0, 6.0]], [[1.0, 4.0, 6.0], [6.0, 8.0, 2.0], [5.0, 1.0, 7.0]]], [[[4.0, 8.0, 6.0], 
[3.0, 4.0, 6.0], [5.0, 1.0, 4.0]], [[7.0, 1.0, 8.0], [1.0, 1.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 4.0, 3.0], [4.0, 6.0, 2.0], 
[2.0, 3.0, 1.0]]], [[[2.0, 4.0, 3.0], [7.0, 5.0, 5.0], [5.0, 3.0, 3.0]], [[6.0, 1.0, 3.0], [4.0, 2.0, 5.0], [7.0, 3.0, 2.0]], 
[[8.0, 5.0, 1.0], [1.0, 6.0, 7.0], [8.0, 8.0, 3.0]]], [[[4.0, 5.0, 6.0], [7.0, 7.0, 3.0], [1.0, 4.0, 8.0]], [[3.0, 3.0, 4.0], 
[8.0, 8.0, 6.0], [3.0, 7.0, 4.0]], [[6.0, 5.0, 4.0], [8.0, 2.0, 1.0], [6.0, 7.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0]]], [[[4.0]]], [[[3.0]]], [[[6.0]]]])+(1.-msk_val)*numpy.array([[[[1.0]]], [[[3.0]]], 
[[[5.0]]], [[[8.0]]]])
      arg[:,2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 6.0], [2.0, 6.0, 8.0], [4.0, 1.0, 4.0]], [[7.0, 6.0, 2.0], [8.0, 7.0, 6.0], [6.0, 
4.0, 2.0]], [[8.0, 4.0, 2.0], [2.0, 2.0, 7.0], [2.0, 6.0, 1.0]]], [[[1.0, 2.0, 3.0], [7.0, 7.0, 1.0], [5.0, 1.0, 6.0]], [[8.0, 
5.0, 1.0], [3.0, 2.0, 7.0], [8.0, 1.0, 1.0]], [[6.0, 5.0, 6.0], [2.0, 7.0, 8.0], [3.0, 8.0, 4.0]]], [[[8.0, 7.0, 8.0], [4.0, 
6.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 6.0, 8.0], [6.0, 4.0, 1.0], [7.0, 8.0, 6.0]], [[3.0, 2.0, 7.0], [2.0, 2.0, 1.0], [8.0, 5.0, 
3.0]]], [[[6.0, 2.0, 6.0], [1.0, 3.0, 3.0], [8.0, 7.0, 4.0]], [[2.0, 3.0, 1.0], [3.0, 2.0, 3.0], [3.0, 6.0, 4.0]], [[7.0, 8.0, 
1.0], [1.0, 8.0, 2.0], [4.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 1.0], [4.0, 7.0, 8.0], [7.0, 3.0, 1.0]], 
[[6.0, 4.0, 1.0], [5.0, 4.0, 5.0], [3.0, 8.0, 6.0]], [[1.0, 4.0, 6.0], [6.0, 8.0, 2.0], [5.0, 1.0, 1.0]]], [[[4.0, 8.0, 6.0], 
[3.0, 4.0, 6.0], [5.0, 1.0, 4.0]], [[7.0, 1.0, 8.0], [1.0, 1.0, 4.0], [1.0, 2.0, 1.0]], [[3.0, 4.0, 3.0], [4.0, 6.0, 2.0], 
[2.0, 3.0, 3.0]]], [[[2.0, 4.0, 3.0], [7.0, 5.0, 5.0], [5.0, 3.0, 3.0]], [[6.0, 1.0, 3.0], [4.0, 2.0, 5.0], [7.0, 3.0, 2.0]], 
[[8.0, 5.0, 1.0], [1.0, 6.0, 7.0], [8.0, 8.0, 5.0]]], [[[4.0, 5.0, 6.0], [7.0, 7.0, 3.0], [1.0, 4.0, 8.0]], [[3.0, 3.0, 4.0], 
[8.0, 8.0, 6.0], [3.0, 7.0, 4.0]], [[6.0, 5.0, 4.0], [8.0, 2.0, 1.0], [6.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 1.0], [1.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[5.0, 2.0, 5.0], [7.0, 8.0, 2.0], [5.0, 
5.0, 8.0]], [[2.0, 8.0, 3.0], [7.0, 3.0, 4.0], [5.0, 5.0, 4.0]]], [[[8.0, 7.0, 6.0], [7.0, 1.0, 6.0], [8.0, 5.0, 2.0]], [[5.0, 
3.0, 1.0], [7.0, 5.0, 4.0], [2.0, 5.0, 6.0]], [[2.0, 3.0, 4.0], [1.0, 6.0, 2.0], [7.0, 6.0, 4.0]]], [[[5.0, 6.0, 5.0], [3.0, 
4.0, 7.0], [7.0, 1.0, 1.0]], [[7.0, 2.0, 6.0], [2.0, 2.0, 6.0], [1.0, 3.0, 4.0]], [[4.0, 3.0, 5.0], [5.0, 6.0, 6.0], [4.0, 7.0, 
7.0]]], [[[2.0, 7.0, 5.0], [5.0, 4.0, 5.0], [2.0, 4.0, 6.0]], [[7.0, 1.0, 6.0], [7.0, 7.0, 6.0], [6.0, 8.0, 2.0]], [[5.0, 8.0, 
6.0], [3.0, 2.0, 5.0], [7.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 7.0], [7.0, 5.0, 7.0], [3.0, 7.0, 7.0]], 
[[7.0, 3.0, 7.0], [6.0, 8.0, 4.0], [8.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 1.0, 3.0], [6.0, 2.0, 3.0]]], [[[4.0, 5.0, 3.0], 
[5.0, 2.0, 3.0], [5.0, 4.0, 1.0]], [[7.0, 8.0, 7.0], [7.0, 4.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 4.0, 1.0], [3.0, 7.0, 6.0], 
[6.0, 1.0, 8.0]]], [[[5.0, 6.0, 6.0], [5.0, 1.0, 1.0], [1.0, 5.0, 6.0]], [[7.0, 2.0, 2.0], [2.0, 5.0, 4.0], [1.0, 1.0, 7.0]], 
[[8.0, 4.0, 8.0], [7.0, 6.0, 2.0], [7.0, 7.0, 2.0]]], [[[7.0, 1.0, 5.0], [4.0, 7.0, 8.0], [5.0, 8.0, 6.0]], [[7.0, 1.0, 6.0], 
[2.0, 2.0, 8.0], [6.0, 7.0, 6.0]], [[5.0, 8.0, 6.0], [7.0, 6.0, 4.0], [2.0, 4.0, 6.0]]]])
      val=numpy.array([[[4.0, 5.0, 4.0], [4.0, 3.0, 1.0], [1.0, 2.0, 4.0]], [[6.0, 8.0, 6.0], [4.0, 8.0, 5.0], [2.0, 4.0, 
5.0]], [[5.0, 2.0, 6.0], [5.0, 1.0, 6.0], [7.0, 6.0, 2.0]], [[2.0, 4.0, 4.0], [2.0, 2.0, 5.0], [3.0, 1.0, 1.0]]])
      arg[:,1,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 1.0], [1.0, 2.0, 2.0], [6.0, 1.0, 8.0]], [[4.0, 5.0, 4.0], [4.0, 3.0, 1.0], [1.0, 
2.0, 4.0]], [[2.0, 8.0, 3.0], [7.0, 3.0, 4.0], [5.0, 5.0, 4.0]]], [[[8.0, 7.0, 6.0], [7.0, 1.0, 6.0], [8.0, 5.0, 2.0]], [[6.0, 
8.0, 6.0], [4.0, 8.0, 5.0], [2.0, 4.0, 5.0]], [[2.0, 3.0, 4.0], [1.0, 6.0, 2.0], [7.0, 6.0, 4.0]]], [[[5.0, 6.0, 5.0], [3.0, 
4.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 2.0, 6.0], [5.0, 1.0, 6.0], [7.0, 6.0, 2.0]], [[4.0, 3.0, 5.0], [5.0, 6.0, 6.0], [4.0, 7.0, 
7.0]]], [[[2.0, 7.0, 5.0], [5.0, 4.0, 5.0], [2.0, 4.0, 6.0]], [[2.0, 4.0, 4.0], [2.0, 2.0, 5.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 
6.0], [3.0, 2.0, 5.0], [7.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [7.0, 5.0, 7.0], [3.0, 7.0, 7.0]], 
[[4.0, 5.0, 4.0], [4.0, 3.0, 1.0], [1.0, 2.0, 4.0]], [[8.0, 6.0, 7.0], [2.0, 1.0, 3.0], [6.0, 2.0, 3.0]]], [[[4.0, 5.0, 3.0], 
[5.0, 2.0, 3.0], [5.0, 4.0, 1.0]], [[6.0, 8.0, 6.0], [4.0, 8.0, 5.0], [2.0, 4.0, 5.0]], [[2.0, 4.0, 1.0], [3.0, 7.0, 6.0], 
[6.0, 1.0, 8.0]]], [[[5.0, 6.0, 6.0], [5.0, 1.0, 1.0], [1.0, 5.0, 6.0]], [[5.0, 2.0, 6.0], [5.0, 1.0, 6.0], [7.0, 6.0, 2.0]], 
[[8.0, 4.0, 8.0], [7.0, 6.0, 2.0], [7.0, 7.0, 2.0]]], [[[7.0, 1.0, 5.0], [4.0, 7.0, 8.0], [5.0, 8.0, 6.0]], [[2.0, 4.0, 4.0], 
[2.0, 2.0, 5.0], [3.0, 1.0, 1.0]], [[5.0, 8.0, 6.0], [7.0, 6.0, 4.0], [2.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 3.0], [7.0, 3.0, 2.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 4.0], [5.0, 
6.0, 5.0]], [[3.0, 6.0, 7.0], [3.0, 1.0, 4.0], [3.0, 5.0, 5.0]]], [[[2.0, 2.0, 5.0], [5.0, 1.0, 7.0], [5.0, 6.0, 4.0]], [[4.0, 
3.0, 6.0], [7.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[6.0, 6.0, 7.0], [7.0, 1.0, 4.0], [4.0, 5.0, 3.0]]], [[[8.0, 5.0, 5.0], [6.0, 
6.0, 4.0], [4.0, 4.0, 8.0]], [[1.0, 3.0, 6.0], [1.0, 8.0, 8.0], [3.0, 6.0, 3.0]], [[3.0, 6.0, 1.0], [7.0, 1.0, 4.0], [7.0, 2.0, 
8.0]]], [[[2.0, 1.0, 6.0], [4.0, 2.0, 7.0], [1.0, 1.0, 6.0]], [[6.0, 2.0, 6.0], [8.0, 8.0, 6.0], [8.0, 1.0, 6.0]], [[6.0, 7.0, 
5.0], [2.0, 1.0, 6.0], [1.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 1.0], [7.0, 3.0, 4.0], [1.0, 1.0, 8.0]], 
[[4.0, 2.0, 5.0], [5.0, 5.0, 6.0], [2.0, 7.0, 3.0]], [[6.0, 2.0, 8.0], [1.0, 2.0, 5.0], [4.0, 2.0, 1.0]]], [[[4.0, 8.0, 4.0], 
[8.0, 8.0, 2.0], [7.0, 2.0, 3.0]], [[1.0, 3.0, 2.0], [1.0, 2.0, 8.0], [5.0, 8.0, 6.0]], [[5.0, 3.0, 8.0], [3.0, 7.0, 2.0], 
[5.0, 6.0, 4.0]]], [[[8.0, 8.0, 4.0], [2.0, 8.0, 2.0], [1.0, 8.0, 6.0]], [[3.0, 4.0, 1.0], [3.0, 5.0, 6.0], [1.0, 5.0, 6.0]], 
[[6.0, 5.0, 3.0], [4.0, 3.0, 8.0], [5.0, 4.0, 1.0]]], [[[1.0, 4.0, 2.0], [7.0, 3.0, 8.0], [4.0, 5.0, 3.0]], [[8.0, 7.0, 3.0], 
[4.0, 3.0, 2.0], [4.0, 8.0, 5.0]], [[8.0, 7.0, 5.0], [1.0, 5.0, 8.0], [7.0, 7.0, 3.0]]]])
      val=Data(numpy.array([[[1.0, 7.0, 5.0], [2.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[2.0, 1.0, 2.0], [2.0, 1.0, 8.0], [4.0, 8.0, 
2.0]], [[1.0, 4.0, 1.0], [6.0, 7.0, 3.0], [7.0, 4.0, 5.0]], [[2.0, 7.0, 5.0], [4.0, 5.0, 6.0], [4.0, 1.0, 
8.0]]]),self.functionspace)
      arg[:,1,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 3.0], [7.0, 3.0, 2.0], [7.0, 1.0, 2.0]], [[1.0, 7.0, 5.0], [2.0, 5.0, 7.0], [4.0, 
2.0, 7.0]], [[3.0, 6.0, 7.0], [3.0, 1.0, 4.0], [3.0, 5.0, 5.0]]], [[[2.0, 2.0, 5.0], [5.0, 1.0, 7.0], [5.0, 6.0, 4.0]], [[2.0, 
1.0, 2.0], [2.0, 1.0, 8.0], [4.0, 8.0, 2.0]], [[6.0, 6.0, 7.0], [7.0, 1.0, 4.0], [4.0, 5.0, 3.0]]], [[[8.0, 5.0, 5.0], [6.0, 
6.0, 4.0], [4.0, 4.0, 8.0]], [[1.0, 4.0, 1.0], [6.0, 7.0, 3.0], [7.0, 4.0, 5.0]], [[3.0, 6.0, 1.0], [7.0, 1.0, 4.0], [7.0, 2.0, 
8.0]]], [[[2.0, 1.0, 6.0], [4.0, 2.0, 7.0], [1.0, 1.0, 6.0]], [[2.0, 7.0, 5.0], [4.0, 5.0, 6.0], [4.0, 1.0, 8.0]], [[6.0, 7.0, 
5.0], [2.0, 1.0, 6.0], [1.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 1.0], [7.0, 3.0, 4.0], [1.0, 1.0, 8.0]], 
[[1.0, 7.0, 5.0], [2.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[6.0, 2.0, 8.0], [1.0, 2.0, 5.0], [4.0, 2.0, 1.0]]], [[[4.0, 8.0, 4.0], 
[8.0, 8.0, 2.0], [7.0, 2.0, 3.0]], [[2.0, 1.0, 2.0], [2.0, 1.0, 8.0], [4.0, 8.0, 2.0]], [[5.0, 3.0, 8.0], [3.0, 7.0, 2.0], 
[5.0, 6.0, 4.0]]], [[[8.0, 8.0, 4.0], [2.0, 8.0, 2.0], [1.0, 8.0, 6.0]], [[1.0, 4.0, 1.0], [6.0, 7.0, 3.0], [7.0, 4.0, 5.0]], 
[[6.0, 5.0, 3.0], [4.0, 3.0, 8.0], [5.0, 4.0, 1.0]]], [[[1.0, 4.0, 2.0], [7.0, 3.0, 8.0], [4.0, 5.0, 3.0]], [[2.0, 7.0, 5.0], 
[4.0, 5.0, 6.0], [4.0, 1.0, 8.0]], [[8.0, 7.0, 5.0], [1.0, 5.0, 8.0], [7.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 8.0], [4.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[8.0, 6.0, 6.0], [4.0, 3.0, 4.0], [7.0, 
1.0, 1.0]], [[3.0, 4.0, 5.0], [4.0, 7.0, 1.0], [1.0, 2.0, 4.0]]], [[[7.0, 6.0, 5.0], [5.0, 7.0, 2.0], [8.0, 1.0, 6.0]], [[1.0, 
6.0, 6.0], [1.0, 1.0, 4.0], [4.0, 5.0, 4.0]], [[1.0, 7.0, 6.0], [7.0, 6.0, 2.0], [1.0, 2.0, 4.0]]], [[[2.0, 7.0, 2.0], [7.0, 
8.0, 1.0], [5.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], [2.0, 2.0, 6.0], [7.0, 5.0, 4.0]], [[2.0, 6.0, 1.0], [7.0, 8.0, 3.0], [3.0, 6.0, 
1.0]]], [[[4.0, 5.0, 2.0], [8.0, 6.0, 5.0], [7.0, 4.0, 8.0]], [[3.0, 1.0, 6.0], [3.0, 4.0, 2.0], [1.0, 4.0, 2.0]], [[4.0, 8.0, 
7.0], [5.0, 7.0, 5.0], [3.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 6.0], [6.0, 4.0, 2.0], [7.0, 1.0, 3.0]], 
[[8.0, 2.0, 3.0], [5.0, 1.0, 6.0], [4.0, 7.0, 1.0]], [[5.0, 1.0, 6.0], [4.0, 2.0, 8.0], [3.0, 8.0, 3.0]]], [[[2.0, 2.0, 3.0], 
[4.0, 3.0, 2.0], [1.0, 8.0, 2.0]], [[6.0, 3.0, 3.0], [5.0, 4.0, 2.0], [6.0, 3.0, 7.0]], [[5.0, 5.0, 2.0], [8.0, 2.0, 2.0], 
[3.0, 5.0, 4.0]]], [[[8.0, 7.0, 6.0], [8.0, 4.0, 8.0], [8.0, 4.0, 2.0]], [[8.0, 1.0, 3.0], [5.0, 5.0, 2.0], [3.0, 7.0, 2.0]], 
[[7.0, 7.0, 7.0], [7.0, 2.0, 6.0], [3.0, 8.0, 3.0]]], [[[6.0, 3.0, 7.0], [4.0, 8.0, 8.0], [5.0, 8.0, 4.0]], [[8.0, 7.0, 4.0], 
[5.0, 6.0, 6.0], [4.0, 5.0, 2.0]], [[4.0, 4.0, 2.0], [8.0, 7.0, 8.0], [3.0, 8.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 6.0, 8.0], [1.0, 7.0, 8.0], [5.0, 6.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 5.0, 5.0], [7.0, 
2.0, 8.0]], [[2.0, 5.0, 7.0], [5.0, 1.0, 4.0], [5.0, 6.0, 5.0]], [[1.0, 1.0, 6.0], [2.0, 8.0, 1.0], [2.0, 5.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[4.0, 3.0, 2.0], [4.0, 8.0, 4.0], [3.0, 1.0, 5.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 5.0], [4.0, 
8.0, 7.0]], [[3.0, 4.0, 1.0], [7.0, 2.0, 1.0], [1.0, 6.0, 6.0]], [[1.0, 6.0, 7.0], [5.0, 3.0, 7.0], [6.0, 7.0, 4.0]]])
      arg[:,1,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 8.0], [4.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 6.0, 8.0], [1.0, 7.0, 8.0], [5.0, 
6.0, 7.0]], [[3.0, 4.0, 5.0], [4.0, 7.0, 1.0], [1.0, 2.0, 4.0]]], [[[7.0, 6.0, 5.0], [5.0, 7.0, 2.0], [8.0, 1.0, 6.0]], [[1.0, 
4.0, 8.0], [8.0, 5.0, 5.0], [7.0, 2.0, 8.0]], [[1.0, 7.0, 6.0], [7.0, 6.0, 2.0], [1.0, 2.0, 4.0]]], [[[2.0, 7.0, 2.0], [7.0, 
8.0, 1.0], [5.0, 5.0, 3.0]], [[2.0, 5.0, 7.0], [5.0, 1.0, 4.0], [5.0, 6.0, 5.0]], [[2.0, 6.0, 1.0], [7.0, 8.0, 3.0], [3.0, 6.0, 
1.0]]], [[[4.0, 5.0, 2.0], [8.0, 6.0, 5.0], [7.0, 4.0, 8.0]], [[1.0, 1.0, 6.0], [2.0, 8.0, 1.0], [2.0, 5.0, 1.0]], [[4.0, 8.0, 
7.0], [5.0, 7.0, 5.0], [3.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 6.0], [6.0, 4.0, 2.0], [7.0, 1.0, 3.0]], 
[[4.0, 3.0, 2.0], [4.0, 8.0, 4.0], [3.0, 1.0, 5.0]], [[5.0, 1.0, 6.0], [4.0, 2.0, 8.0], [3.0, 8.0, 3.0]]], [[[2.0, 2.0, 3.0], 
[4.0, 3.0, 2.0], [1.0, 8.0, 2.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 5.0], [4.0, 8.0, 7.0]], [[5.0, 5.0, 2.0], [8.0, 2.0, 2.0], 
[3.0, 5.0, 4.0]]], [[[8.0, 7.0, 6.0], [8.0, 4.0, 8.0], [8.0, 4.0, 2.0]], [[3.0, 4.0, 1.0], [7.0, 2.0, 1.0], [1.0, 6.0, 6.0]], 
[[7.0, 7.0, 7.0], [7.0, 2.0, 6.0], [3.0, 8.0, 3.0]]], [[[6.0, 3.0, 7.0], [4.0, 8.0, 8.0], [5.0, 8.0, 4.0]], [[1.0, 6.0, 7.0], 
[5.0, 3.0, 7.0], [6.0, 7.0, 4.0]], [[4.0, 4.0, 2.0], [8.0, 7.0, 8.0], [3.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 7.0], [3.0, 7.0, 2.0], [8.0, 4.0, 3.0]], [[6.0, 6.0, 6.0], [6.0, 3.0, 8.0], [4.0, 
3.0, 2.0]], [[5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [3.0, 4.0, 7.0]]], [[[4.0, 3.0, 1.0], [6.0, 1.0, 6.0], [4.0, 3.0, 1.0]], [[4.0, 
3.0, 3.0], [8.0, 4.0, 4.0], [2.0, 5.0, 5.0]], [[1.0, 8.0, 2.0], [7.0, 7.0, 8.0], [3.0, 5.0, 5.0]]], [[[3.0, 1.0, 2.0], [3.0, 
1.0, 4.0], [3.0, 4.0, 1.0]], [[8.0, 8.0, 1.0], [3.0, 5.0, 1.0], [3.0, 1.0, 1.0]], [[8.0, 2.0, 4.0], [1.0, 6.0, 4.0], [5.0, 8.0, 
5.0]]], [[[3.0, 4.0, 5.0], [6.0, 3.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 8.0], [5.0, 2.0, 8.0], [6.0, 8.0, 4.0]], [[8.0, 5.0, 
1.0], [4.0, 3.0, 7.0], [1.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 3.0], [3.0, 3.0, 3.0], [5.0, 1.0, 8.0]], 
[[8.0, 5.0, 2.0], [1.0, 5.0, 6.0], [3.0, 1.0, 5.0]], [[1.0, 1.0, 4.0], [7.0, 2.0, 6.0], [6.0, 2.0, 1.0]]], [[[1.0, 3.0, 1.0], 
[2.0, 2.0, 5.0], [1.0, 7.0, 7.0]], [[5.0, 2.0, 6.0], [7.0, 3.0, 8.0], [8.0, 1.0, 5.0]], [[6.0, 7.0, 5.0], [8.0, 8.0, 2.0], 
[8.0, 5.0, 6.0]]], [[[7.0, 2.0, 8.0], [8.0, 6.0, 6.0], [1.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [1.0, 3.0, 1.0], [1.0, 2.0, 6.0]], 
[[5.0, 6.0, 1.0], [2.0, 7.0, 2.0], [5.0, 4.0, 7.0]]], [[[7.0, 7.0, 8.0], [7.0, 8.0, 7.0], [4.0, 2.0, 7.0]], [[5.0, 6.0, 2.0], 
[4.0, 1.0, 2.0], [4.0, 4.0, 8.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 2.0], [4.0, 4.0, 3.0]]]])
      val=numpy.array([[8.0, 2.0, 5.0], [1.0, 3.0, 5.0], [8.0, 3.0, 4.0]])
      arg[:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [3.0, 7.0, 2.0], [8.0, 4.0, 3.0]], [[6.0, 6.0, 6.0], [8.0, 2.0, 5.0], [4.0, 
3.0, 2.0]], [[5.0, 8.0, 6.0], [6.0, 6.0, 3.0], [3.0, 4.0, 7.0]]], [[[4.0, 3.0, 1.0], [6.0, 1.0, 6.0], [4.0, 3.0, 1.0]], [[4.0, 
3.0, 3.0], [1.0, 3.0, 5.0], [2.0, 5.0, 5.0]], [[1.0, 8.0, 2.0], [7.0, 7.0, 8.0], [3.0, 5.0, 5.0]]], [[[3.0, 1.0, 2.0], [3.0, 
1.0, 4.0], [3.0, 4.0, 1.0]], [[8.0, 8.0, 1.0], [8.0, 3.0, 4.0], [3.0, 1.0, 1.0]], [[8.0, 2.0, 4.0], [1.0, 6.0, 4.0], [5.0, 8.0, 
5.0]]], [[[3.0, 4.0, 5.0], [6.0, 3.0, 1.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 8.0], [5.0, 2.0, 8.0], [6.0, 8.0, 4.0]], [[8.0, 5.0, 
1.0], [4.0, 3.0, 7.0], [1.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 3.0], [3.0, 3.0, 3.0], [5.0, 1.0, 8.0]], 
[[8.0, 5.0, 2.0], [8.0, 2.0, 5.0], [3.0, 1.0, 5.0]], [[1.0, 1.0, 4.0], [7.0, 2.0, 6.0], [6.0, 2.0, 1.0]]], [[[1.0, 3.0, 1.0], 
[2.0, 2.0, 5.0], [1.0, 7.0, 7.0]], [[5.0, 2.0, 6.0], [1.0, 3.0, 5.0], [8.0, 1.0, 5.0]], [[6.0, 7.0, 5.0], [8.0, 8.0, 2.0], 
[8.0, 5.0, 6.0]]], [[[7.0, 2.0, 8.0], [8.0, 6.0, 6.0], [1.0, 1.0, 1.0]], [[4.0, 5.0, 3.0], [8.0, 3.0, 4.0], [1.0, 2.0, 6.0]], 
[[5.0, 6.0, 1.0], [2.0, 7.0, 2.0], [5.0, 4.0, 7.0]]], [[[7.0, 7.0, 8.0], [7.0, 8.0, 7.0], [4.0, 2.0, 7.0]], [[5.0, 6.0, 2.0], 
[4.0, 1.0, 2.0], [4.0, 4.0, 8.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 2.0], [4.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 4.0], [1.0, 7.0, 2.0], [5.0, 5.0, 3.0]], [[5.0, 2.0, 2.0], [1.0, 2.0, 2.0], [4.0, 
4.0, 2.0]], [[1.0, 1.0, 5.0], [5.0, 6.0, 1.0], [6.0, 3.0, 6.0]]], [[[2.0, 8.0, 3.0], [1.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[4.0, 
8.0, 2.0], [7.0, 2.0, 6.0], [6.0, 6.0, 7.0]], [[8.0, 2.0, 4.0], [3.0, 2.0, 5.0], [2.0, 5.0, 6.0]]], [[[5.0, 8.0, 5.0], [5.0, 
1.0, 2.0], [3.0, 6.0, 5.0]], [[2.0, 3.0, 4.0], [6.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[3.0, 4.0, 6.0], [3.0, 5.0, 8.0], [5.0, 8.0, 
5.0]]], [[[2.0, 8.0, 5.0], [3.0, 5.0, 5.0], [1.0, 4.0, 5.0]], [[3.0, 1.0, 2.0], [8.0, 7.0, 3.0], [1.0, 8.0, 4.0]], [[7.0, 7.0, 
1.0], [4.0, 8.0, 2.0], [6.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 1.0, 2.0], [3.0, 8.0, 7.0]], 
[[1.0, 4.0, 3.0], [5.0, 1.0, 6.0], [2.0, 7.0, 4.0]], [[5.0, 4.0, 6.0], [3.0, 6.0, 4.0], [4.0, 5.0, 7.0]]], [[[6.0, 7.0, 6.0], 
[2.0, 1.0, 1.0], [1.0, 2.0, 8.0]], [[4.0, 8.0, 8.0], [8.0, 6.0, 5.0], [8.0, 4.0, 5.0]], [[6.0, 7.0, 2.0], [6.0, 8.0, 1.0], 
[4.0, 4.0, 6.0]]], [[[4.0, 5.0, 2.0], [4.0, 7.0, 5.0], [7.0, 2.0, 8.0]], [[6.0, 6.0, 3.0], [6.0, 1.0, 7.0], [7.0, 1.0, 5.0]], 
[[7.0, 8.0, 7.0], [6.0, 3.0, 3.0], [1.0, 1.0, 6.0]]], [[[7.0, 4.0, 8.0], [5.0, 7.0, 1.0], [1.0, 5.0, 7.0]], [[4.0, 7.0, 2.0], 
[7.0, 2.0, 3.0], [8.0, 2.0, 2.0]], [[6.0, 6.0, 4.0], [2.0, 5.0, 7.0], [3.0, 7.0, 6.0]]]])
      val=Data(numpy.array([[4.0, 2.0, 5.0], [8.0, 3.0, 5.0], [7.0, 3.0, 4.0]]),self.functionspace)
      arg[:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [1.0, 7.0, 2.0], [5.0, 5.0, 3.0]], [[5.0, 2.0, 2.0], [4.0, 2.0, 5.0], [4.0, 
4.0, 2.0]], [[1.0, 1.0, 5.0], [5.0, 6.0, 1.0], [6.0, 3.0, 6.0]]], [[[2.0, 8.0, 3.0], [1.0, 3.0, 6.0], [6.0, 1.0, 2.0]], [[4.0, 
8.0, 2.0], [8.0, 3.0, 5.0], [6.0, 6.0, 7.0]], [[8.0, 2.0, 4.0], [3.0, 2.0, 5.0], [2.0, 5.0, 6.0]]], [[[5.0, 8.0, 5.0], [5.0, 
1.0, 2.0], [3.0, 6.0, 5.0]], [[2.0, 3.0, 4.0], [7.0, 3.0, 4.0], [7.0, 8.0, 3.0]], [[3.0, 4.0, 6.0], [3.0, 5.0, 8.0], [5.0, 8.0, 
5.0]]], [[[2.0, 8.0, 5.0], [3.0, 5.0, 5.0], [1.0, 4.0, 5.0]], [[3.0, 1.0, 2.0], [8.0, 7.0, 3.0], [1.0, 8.0, 4.0]], [[7.0, 7.0, 
1.0], [4.0, 8.0, 2.0], [6.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 1.0, 2.0], [3.0, 8.0, 7.0]], 
[[1.0, 4.0, 3.0], [4.0, 2.0, 5.0], [2.0, 7.0, 4.0]], [[5.0, 4.0, 6.0], [3.0, 6.0, 4.0], [4.0, 5.0, 7.0]]], [[[6.0, 7.0, 6.0], 
[2.0, 1.0, 1.0], [1.0, 2.0, 8.0]], [[4.0, 8.0, 8.0], [8.0, 3.0, 5.0], [8.0, 4.0, 5.0]], [[6.0, 7.0, 2.0], [6.0, 8.0, 1.0], 
[4.0, 4.0, 6.0]]], [[[4.0, 5.0, 2.0], [4.0, 7.0, 5.0], [7.0, 2.0, 8.0]], [[6.0, 6.0, 3.0], [7.0, 3.0, 4.0], [7.0, 1.0, 5.0]], 
[[7.0, 8.0, 7.0], [6.0, 3.0, 3.0], [1.0, 1.0, 6.0]]], [[[7.0, 4.0, 8.0], [5.0, 7.0, 1.0], [1.0, 5.0, 7.0]], [[4.0, 7.0, 2.0], 
[7.0, 2.0, 3.0], [8.0, 2.0, 2.0]], [[6.0, 6.0, 4.0], [2.0, 5.0, 7.0], [3.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 8.0], [5.0, 8.0, 1.0], [4.0, 2.0, 4.0]], [[3.0, 8.0, 6.0], [7.0, 1.0, 6.0], [6.0, 
4.0, 8.0]], [[4.0, 3.0, 3.0], [8.0, 3.0, 3.0], [4.0, 3.0, 8.0]]], [[[5.0, 7.0, 8.0], [5.0, 4.0, 2.0], [1.0, 1.0, 5.0]], [[1.0, 
8.0, 2.0], [1.0, 3.0, 1.0], [4.0, 2.0, 1.0]], [[8.0, 1.0, 7.0], [7.0, 5.0, 7.0], [7.0, 2.0, 4.0]]], [[[6.0, 2.0, 7.0], [4.0, 
8.0, 6.0], [8.0, 5.0, 1.0]], [[2.0, 8.0, 1.0], [2.0, 3.0, 6.0], [4.0, 4.0, 2.0]], [[6.0, 8.0, 2.0], [5.0, 4.0, 4.0], [5.0, 8.0, 
7.0]]], [[[8.0, 1.0, 3.0], [6.0, 1.0, 8.0], [1.0, 8.0, 1.0]], [[6.0, 1.0, 2.0], [3.0, 5.0, 1.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 
4.0], [2.0, 4.0, 4.0], [3.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 1.0], [2.0, 2.0, 5.0], [3.0, 1.0, 1.0]], 
[[8.0, 5.0, 4.0], [8.0, 3.0, 3.0], [3.0, 8.0, 4.0]], [[2.0, 8.0, 4.0], [8.0, 7.0, 8.0], [3.0, 6.0, 6.0]]], [[[6.0, 5.0, 2.0], 
[5.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[6.0, 1.0, 8.0], [5.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[7.0, 7.0, 8.0], [2.0, 3.0, 5.0], 
[7.0, 3.0, 8.0]]], [[[1.0, 8.0, 4.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], [[5.0, 1.0, 2.0], [3.0, 6.0, 7.0], [1.0, 8.0, 2.0]], 
[[6.0, 6.0, 2.0], [7.0, 3.0, 1.0], [6.0, 6.0, 1.0]]], [[[4.0, 1.0, 7.0], [1.0, 8.0, 7.0], [3.0, 2.0, 1.0]], [[8.0, 7.0, 5.0], 
[3.0, 7.0, 5.0], [1.0, 3.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 1.0], [1.0, 7.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[3.0, 7.0, 1.0], [8.0, 4.0, 5.0], [4.0, 6.0, 5.0]])+(1.-msk_val)*numpy.array([[6.0, 3.0, 4.0], 
[2.0, 7.0, 2.0], [2.0, 8.0, 2.0]])
      arg[:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 8.0], [5.0, 8.0, 1.0], [4.0, 2.0, 4.0]], [[3.0, 8.0, 6.0], [3.0, 7.0, 1.0], [6.0, 
4.0, 8.0]], [[4.0, 3.0, 3.0], [8.0, 3.0, 3.0], [4.0, 3.0, 8.0]]], [[[5.0, 7.0, 8.0], [5.0, 4.0, 2.0], [1.0, 1.0, 5.0]], [[1.0, 
8.0, 2.0], [8.0, 4.0, 5.0], [4.0, 2.0, 1.0]], [[8.0, 1.0, 7.0], [7.0, 5.0, 7.0], [7.0, 2.0, 4.0]]], [[[6.0, 2.0, 7.0], [4.0, 
8.0, 6.0], [8.0, 5.0, 1.0]], [[2.0, 8.0, 1.0], [4.0, 6.0, 5.0], [4.0, 4.0, 2.0]], [[6.0, 8.0, 2.0], [5.0, 4.0, 4.0], [5.0, 8.0, 
7.0]]], [[[8.0, 1.0, 3.0], [6.0, 1.0, 8.0], [1.0, 8.0, 1.0]], [[6.0, 1.0, 2.0], [3.0, 5.0, 1.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 
4.0], [2.0, 4.0, 4.0], [3.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 1.0], [2.0, 2.0, 5.0], [3.0, 1.0, 1.0]], 
[[8.0, 5.0, 4.0], [6.0, 3.0, 4.0], [3.0, 8.0, 4.0]], [[2.0, 8.0, 4.0], [8.0, 7.0, 8.0], [3.0, 6.0, 6.0]]], [[[6.0, 5.0, 2.0], 
[5.0, 8.0, 4.0], [5.0, 6.0, 8.0]], [[6.0, 1.0, 8.0], [2.0, 7.0, 2.0], [7.0, 5.0, 6.0]], [[7.0, 7.0, 8.0], [2.0, 3.0, 5.0], 
[7.0, 3.0, 8.0]]], [[[1.0, 8.0, 4.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]], [[5.0, 1.0, 2.0], [2.0, 8.0, 2.0], [1.0, 8.0, 2.0]], 
[[6.0, 6.0, 2.0], [7.0, 3.0, 1.0], [6.0, 6.0, 1.0]]], [[[4.0, 1.0, 7.0], [1.0, 8.0, 7.0], [3.0, 2.0, 1.0]], [[8.0, 7.0, 5.0], 
[3.0, 7.0, 5.0], [1.0, 3.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 8.0, 1.0], [1.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 8.0], [8.0, 7.0, 3.0], [2.0, 6.0, 3.0]], [[4.0, 4.0, 6.0], [4.0, 3.0, 2.0], [2.0, 
1.0, 6.0]], [[3.0, 7.0, 5.0], [3.0, 6.0, 4.0], [3.0, 2.0, 2.0]]], [[[7.0, 1.0, 8.0], [5.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[6.0, 
7.0, 5.0], [1.0, 1.0, 4.0], [3.0, 7.0, 1.0]], [[2.0, 4.0, 4.0], [4.0, 7.0, 1.0], [8.0, 3.0, 7.0]]], [[[6.0, 1.0, 8.0], [2.0, 
7.0, 8.0], [8.0, 7.0, 6.0]], [[6.0, 7.0, 2.0], [8.0, 1.0, 3.0], [2.0, 1.0, 3.0]], [[2.0, 6.0, 8.0], [4.0, 5.0, 7.0], [3.0, 1.0, 
8.0]]], [[[4.0, 8.0, 3.0], [4.0, 7.0, 4.0], [8.0, 8.0, 1.0]], [[1.0, 8.0, 5.0], [5.0, 5.0, 7.0], [8.0, 1.0, 4.0]], [[6.0, 3.0, 
3.0], [1.0, 1.0, 6.0], [5.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 5.0], [7.0, 8.0, 1.0], [1.0, 6.0, 4.0]], 
[[5.0, 2.0, 1.0], [3.0, 8.0, 5.0], [4.0, 7.0, 8.0]], [[8.0, 7.0, 2.0], [1.0, 4.0, 7.0], [2.0, 8.0, 2.0]]], [[[1.0, 6.0, 1.0], 
[7.0, 3.0, 4.0], [7.0, 4.0, 3.0]], [[1.0, 5.0, 7.0], [4.0, 2.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 5.0, 2.0], [1.0, 7.0, 1.0], 
[8.0, 7.0, 3.0]]], [[[6.0, 3.0, 6.0], [2.0, 2.0, 6.0], [1.0, 1.0, 5.0]], [[6.0, 6.0, 2.0], [2.0, 8.0, 8.0], [7.0, 3.0, 4.0]], 
[[3.0, 3.0, 1.0], [2.0, 2.0, 2.0], [5.0, 6.0, 2.0]]], [[[1.0, 7.0, 3.0], [6.0, 5.0, 7.0], [3.0, 8.0, 1.0]], [[8.0, 5.0, 6.0], 
[7.0, 3.0, 3.0], [7.0, 3.0, 8.0]], [[3.0, 3.0, 1.0], [4.0, 3.0, 7.0], [1.0, 8.0, 7.0]]]])
      val=numpy.array([[[[2.0, 1.0, 1.0], [2.0, 5.0, 4.0], [1.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 2.0], [7.0, 1.0, 
3.0]]], [[[4.0, 2.0, 4.0], [6.0, 3.0, 2.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 5.0], [8.0, 7.0, 7.0], [2.0, 4.0, 7.0]]]])
      arg[2:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 8.0], [8.0, 7.0, 3.0], [2.0, 6.0, 3.0]], [[4.0, 4.0, 6.0], [4.0, 3.0, 2.0], [2.0, 
1.0, 6.0]], [[3.0, 7.0, 5.0], [3.0, 6.0, 4.0], [3.0, 2.0, 2.0]]], [[[7.0, 1.0, 8.0], [5.0, 7.0, 1.0], [2.0, 1.0, 8.0]], [[6.0, 
7.0, 5.0], [1.0, 1.0, 4.0], [3.0, 7.0, 1.0]], [[2.0, 4.0, 4.0], [4.0, 7.0, 1.0], [8.0, 3.0, 7.0]]], [[[2.0, 1.0, 1.0], [2.0, 
5.0, 4.0], [1.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 2.0], [7.0, 1.0, 3.0]], [[2.0, 6.0, 8.0], [4.0, 5.0, 7.0], [3.0, 1.0, 
8.0]]], [[[4.0, 2.0, 4.0], [6.0, 3.0, 2.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 5.0], [8.0, 7.0, 7.0], [2.0, 4.0, 7.0]], [[6.0, 3.0, 
3.0], [1.0, 1.0, 6.0], [5.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [7.0, 8.0, 1.0], [1.0, 6.0, 4.0]], 
[[5.0, 2.0, 1.0], [3.0, 8.0, 5.0], [4.0, 7.0, 8.0]], [[8.0, 7.0, 2.0], [1.0, 4.0, 7.0], [2.0, 8.0, 2.0]]], [[[1.0, 6.0, 1.0], 
[7.0, 3.0, 4.0], [7.0, 4.0, 3.0]], [[1.0, 5.0, 7.0], [4.0, 2.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 5.0, 2.0], [1.0, 7.0, 1.0], 
[8.0, 7.0, 3.0]]], [[[2.0, 1.0, 1.0], [2.0, 5.0, 4.0], [1.0, 6.0, 5.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 2.0], [7.0, 1.0, 3.0]], 
[[3.0, 3.0, 1.0], [2.0, 2.0, 2.0], [5.0, 6.0, 2.0]]], [[[4.0, 2.0, 4.0], [6.0, 3.0, 2.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 5.0], 
[8.0, 7.0, 7.0], [2.0, 4.0, 7.0]], [[3.0, 3.0, 1.0], [4.0, 3.0, 7.0], [1.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 6.0, 6.0], [5.0, 2.0, 5.0], [4.0, 6.0, 7.0]], [[6.0, 8.0, 5.0], [7.0, 3.0, 5.0], [1.0, 
8.0, 8.0]], [[8.0, 5.0, 7.0], [6.0, 3.0, 5.0], [6.0, 2.0, 7.0]]], [[[3.0, 1.0, 2.0], [6.0, 4.0, 6.0], [1.0, 8.0, 4.0]], [[8.0, 
1.0, 6.0], [8.0, 4.0, 5.0], [8.0, 6.0, 7.0]], [[6.0, 2.0, 3.0], [3.0, 2.0, 5.0], [7.0, 4.0, 6.0]]], [[[2.0, 1.0, 7.0], [7.0, 
3.0, 3.0], [1.0, 7.0, 1.0]], [[4.0, 3.0, 7.0], [2.0, 6.0, 1.0], [7.0, 6.0, 5.0]], [[5.0, 2.0, 7.0], [5.0, 1.0, 8.0], [6.0, 7.0, 
2.0]]], [[[2.0, 6.0, 8.0], [5.0, 6.0, 7.0], [5.0, 1.0, 4.0]], [[5.0, 4.0, 5.0], [5.0, 8.0, 8.0], [7.0, 3.0, 1.0]], [[7.0, 1.0, 
6.0], [6.0, 1.0, 6.0], [3.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 4.0], [7.0, 7.0, 4.0], [4.0, 2.0, 3.0]], 
[[3.0, 1.0, 4.0], [3.0, 5.0, 6.0], [6.0, 4.0, 3.0]], [[1.0, 8.0, 6.0], [6.0, 3.0, 4.0], [5.0, 1.0, 6.0]]], [[[6.0, 4.0, 7.0], 
[7.0, 5.0, 5.0], [8.0, 1.0, 4.0]], [[4.0, 3.0, 8.0], [5.0, 3.0, 1.0], [1.0, 2.0, 8.0]], [[8.0, 2.0, 5.0], [2.0, 2.0, 6.0], 
[7.0, 5.0, 2.0]]], [[[6.0, 2.0, 8.0], [8.0, 4.0, 7.0], [3.0, 3.0, 4.0]], [[3.0, 3.0, 8.0], [6.0, 7.0, 1.0], [3.0, 7.0, 2.0]], 
[[3.0, 6.0, 3.0], [4.0, 4.0, 4.0], [1.0, 5.0, 8.0]]], [[[3.0, 2.0, 5.0], [1.0, 3.0, 8.0], [8.0, 3.0, 8.0]], [[8.0, 4.0, 7.0], 
[2.0, 8.0, 6.0], [5.0, 7.0, 5.0]], [[1.0, 2.0, 4.0], [1.0, 8.0, 6.0], [3.0, 1.0, 3.0]]]])
      val=Data(numpy.array([[[[2.0, 1.0, 2.0], [2.0, 8.0, 5.0], [4.0, 2.0, 4.0]], [[2.0, 5.0, 3.0], [7.0, 5.0, 7.0], [7.0, 5.0, 
7.0]]], [[[7.0, 7.0, 1.0], [7.0, 6.0, 3.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 5.0], [5.0, 8.0, 3.0], [3.0, 8.0, 
8.0]]]]),self.functionspace)
      arg[2:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 6.0], [5.0, 2.0, 5.0], [4.0, 6.0, 7.0]], [[6.0, 8.0, 5.0], [7.0, 3.0, 5.0], [1.0, 
8.0, 8.0]], [[8.0, 5.0, 7.0], [6.0, 3.0, 5.0], [6.0, 2.0, 7.0]]], [[[3.0, 1.0, 2.0], [6.0, 4.0, 6.0], [1.0, 8.0, 4.0]], [[8.0, 
1.0, 6.0], [8.0, 4.0, 5.0], [8.0, 6.0, 7.0]], [[6.0, 2.0, 3.0], [3.0, 2.0, 5.0], [7.0, 4.0, 6.0]]], [[[2.0, 1.0, 2.0], [2.0, 
8.0, 5.0], [4.0, 2.0, 4.0]], [[2.0, 5.0, 3.0], [7.0, 5.0, 7.0], [7.0, 5.0, 7.0]], [[5.0, 2.0, 7.0], [5.0, 1.0, 8.0], [6.0, 7.0, 
2.0]]], [[[7.0, 7.0, 1.0], [7.0, 6.0, 3.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 5.0], [5.0, 8.0, 3.0], [3.0, 8.0, 8.0]], [[7.0, 1.0, 
6.0], [6.0, 1.0, 6.0], [3.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 4.0], [7.0, 7.0, 4.0], [4.0, 2.0, 3.0]], 
[[3.0, 1.0, 4.0], [3.0, 5.0, 6.0], [6.0, 4.0, 3.0]], [[1.0, 8.0, 6.0], [6.0, 3.0, 4.0], [5.0, 1.0, 6.0]]], [[[6.0, 4.0, 7.0], 
[7.0, 5.0, 5.0], [8.0, 1.0, 4.0]], [[4.0, 3.0, 8.0], [5.0, 3.0, 1.0], [1.0, 2.0, 8.0]], [[8.0, 2.0, 5.0], [2.0, 2.0, 6.0], 
[7.0, 5.0, 2.0]]], [[[2.0, 1.0, 2.0], [2.0, 8.0, 5.0], [4.0, 2.0, 4.0]], [[2.0, 5.0, 3.0], [7.0, 5.0, 7.0], [7.0, 5.0, 7.0]], 
[[3.0, 6.0, 3.0], [4.0, 4.0, 4.0], [1.0, 5.0, 8.0]]], [[[7.0, 7.0, 1.0], [7.0, 6.0, 3.0], [1.0, 6.0, 4.0]], [[8.0, 2.0, 5.0], 
[5.0, 8.0, 3.0], [3.0, 8.0, 8.0]], [[1.0, 2.0, 4.0], [1.0, 8.0, 6.0], [3.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 1.0], [8.0, 2.0, 1.0], [7.0, 2.0, 6.0]], [[8.0, 4.0, 6.0], [2.0, 1.0, 1.0], [8.0, 
6.0, 8.0]], [[1.0, 4.0, 8.0], [5.0, 1.0, 8.0], [6.0, 7.0, 6.0]]], [[[4.0, 2.0, 7.0], [1.0, 3.0, 8.0], [6.0, 3.0, 7.0]], [[7.0, 
1.0, 7.0], [8.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[5.0, 6.0, 8.0], [2.0, 1.0, 7.0], [5.0, 8.0, 8.0]]], [[[2.0, 5.0, 2.0], [8.0, 
1.0, 7.0], [2.0, 3.0, 8.0]], [[7.0, 3.0, 4.0], [7.0, 4.0, 7.0], [6.0, 1.0, 7.0]], [[6.0, 4.0, 3.0], [3.0, 7.0, 1.0], [2.0, 2.0, 
4.0]]], [[[4.0, 2.0, 8.0], [1.0, 6.0, 3.0], [1.0, 8.0, 5.0]], [[2.0, 4.0, 7.0], [4.0, 5.0, 5.0], [5.0, 7.0, 3.0]], [[8.0, 7.0, 
1.0], [1.0, 3.0, 4.0], [4.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 8.0], [1.0, 7.0, 3.0], [6.0, 3.0, 8.0]], 
[[7.0, 8.0, 3.0], [3.0, 3.0, 1.0], [2.0, 2.0, 2.0]], [[5.0, 8.0, 1.0], [2.0, 4.0, 3.0], [7.0, 4.0, 8.0]]], [[[1.0, 5.0, 6.0], 
[6.0, 1.0, 3.0], [7.0, 6.0, 2.0]], [[5.0, 1.0, 1.0], [2.0, 1.0, 7.0], [1.0, 6.0, 3.0]], [[1.0, 3.0, 4.0], [4.0, 5.0, 8.0], 
[5.0, 3.0, 1.0]]], [[[4.0, 6.0, 8.0], [7.0, 1.0, 2.0], [7.0, 2.0, 7.0]], [[4.0, 7.0, 5.0], [8.0, 5.0, 8.0], [6.0, 7.0, 6.0]], 
[[7.0, 6.0, 8.0], [7.0, 7.0, 6.0], [7.0, 1.0, 3.0]]], [[[7.0, 2.0, 3.0], [3.0, 6.0, 4.0], [6.0, 5.0, 3.0]], [[6.0, 6.0, 1.0], 
[7.0, 2.0, 5.0], [6.0, 6.0, 7.0]], [[2.0, 6.0, 4.0], [7.0, 4.0, 8.0], [6.0, 1.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 4.0, 7.0], [5.0, 5.0, 5.0], [3.0, 3.0, 4.0]], [[8.0, 2.0, 1.0], [1.0, 5.0, 2.0], [2.0, 
2.0, 1.0]]], [[[6.0, 8.0, 7.0], [5.0, 4.0, 8.0], [4.0, 6.0, 8.0]], [[1.0, 6.0, 1.0], [1.0, 6.0, 2.0], [6.0, 1.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 2.0, 2.0], [4.0, 1.0, 4.0], [4.0, 2.0, 3.0]], [[8.0, 8.0, 8.0], [7.0, 7.0, 8.0], 
[8.0, 6.0, 3.0]]], [[[1.0, 5.0, 4.0], [3.0, 3.0, 4.0], [3.0, 4.0, 2.0]], [[3.0, 5.0, 4.0], [4.0, 3.0, 6.0], [8.0, 6.0, 6.0]]]])
      arg[2:,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 1.0], [8.0, 2.0, 1.0], [7.0, 2.0, 6.0]], [[8.0, 4.0, 6.0], [2.0, 1.0, 1.0], [8.0, 
6.0, 8.0]], [[1.0, 4.0, 8.0], [5.0, 1.0, 8.0], [6.0, 7.0, 6.0]]], [[[4.0, 2.0, 7.0], [1.0, 3.0, 8.0], [6.0, 3.0, 7.0]], [[7.0, 
1.0, 7.0], [8.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[5.0, 6.0, 8.0], [2.0, 1.0, 7.0], [5.0, 8.0, 8.0]]], [[[2.0, 4.0, 7.0], [5.0, 
5.0, 5.0], [3.0, 3.0, 4.0]], [[8.0, 2.0, 1.0], [1.0, 5.0, 2.0], [2.0, 2.0, 1.0]], [[6.0, 4.0, 3.0], [3.0, 7.0, 1.0], [2.0, 2.0, 
4.0]]], [[[6.0, 8.0, 7.0], [5.0, 4.0, 8.0], [4.0, 6.0, 8.0]], [[1.0, 6.0, 1.0], [1.0, 6.0, 2.0], [6.0, 1.0, 7.0]], [[8.0, 7.0, 
1.0], [1.0, 3.0, 4.0], [4.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 8.0], [1.0, 7.0, 3.0], [6.0, 3.0, 8.0]], 
[[7.0, 8.0, 3.0], [3.0, 3.0, 1.0], [2.0, 2.0, 2.0]], [[5.0, 8.0, 1.0], [2.0, 4.0, 3.0], [7.0, 4.0, 8.0]]], [[[1.0, 5.0, 6.0], 
[6.0, 1.0, 3.0], [7.0, 6.0, 2.0]], [[5.0, 1.0, 1.0], [2.0, 1.0, 7.0], [1.0, 6.0, 3.0]], [[1.0, 3.0, 4.0], [4.0, 5.0, 8.0], 
[5.0, 3.0, 1.0]]], [[[5.0, 2.0, 2.0], [4.0, 1.0, 4.0], [4.0, 2.0, 3.0]], [[8.0, 8.0, 8.0], [7.0, 7.0, 8.0], [8.0, 6.0, 3.0]], 
[[7.0, 6.0, 8.0], [7.0, 7.0, 6.0], [7.0, 1.0, 3.0]]], [[[1.0, 5.0, 4.0], [3.0, 3.0, 4.0], [3.0, 4.0, 2.0]], [[3.0, 5.0, 4.0], 
[4.0, 3.0, 6.0], [8.0, 6.0, 6.0]], [[2.0, 6.0, 4.0], [7.0, 4.0, 8.0], [6.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 7.0], [1.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[4.0, 4.0, 3.0], [1.0, 4.0, 8.0], [3.0, 
3.0, 2.0]], [[1.0, 2.0, 3.0], [8.0, 4.0, 1.0], [2.0, 5.0, 1.0]]], [[[2.0, 4.0, 7.0], [8.0, 3.0, 3.0], [3.0, 3.0, 2.0]], [[2.0, 
7.0, 8.0], [8.0, 2.0, 1.0], [1.0, 5.0, 2.0]], [[8.0, 7.0, 1.0], [7.0, 8.0, 2.0], [7.0, 6.0, 7.0]]], [[[4.0, 3.0, 8.0], [2.0, 
1.0, 8.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 4.0], [1.0, 6.0, 3.0], [1.0, 8.0, 1.0]], [[7.0, 7.0, 3.0], [6.0, 3.0, 7.0], [3.0, 3.0, 
1.0]]], [[[4.0, 7.0, 7.0], [8.0, 3.0, 5.0], [3.0, 7.0, 6.0]], [[1.0, 6.0, 1.0], [2.0, 3.0, 8.0], [5.0, 3.0, 8.0]], [[1.0, 5.0, 
7.0], [1.0, 1.0, 4.0], [3.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 2.0], [8.0, 1.0, 7.0], [6.0, 6.0, 5.0]], 
[[4.0, 1.0, 8.0], [6.0, 2.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 8.0], [4.0, 2.0, 7.0], [2.0, 1.0, 8.0]]], [[[1.0, 8.0, 7.0], 
[3.0, 5.0, 3.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 7.0], [3.0, 2.0, 3.0], [5.0, 8.0, 3.0]], [[4.0, 8.0, 7.0], [7.0, 1.0, 5.0], 
[5.0, 7.0, 2.0]]], [[[2.0, 8.0, 5.0], [8.0, 4.0, 3.0], [3.0, 1.0, 3.0]], [[2.0, 4.0, 2.0], [1.0, 2.0, 1.0], [8.0, 8.0, 1.0]], 
[[6.0, 1.0, 2.0], [3.0, 6.0, 5.0], [7.0, 3.0, 5.0]]], [[[2.0, 5.0, 7.0], [3.0, 3.0, 1.0], [6.0, 8.0, 7.0]], [[1.0, 3.0, 2.0], 
[3.0, 8.0, 6.0], [7.0, 4.0, 8.0]], [[3.0, 1.0, 1.0], [6.0, 8.0, 2.0], [5.0, 2.0, 5.0]]]])
      val=numpy.array([[[[7.0, 5.0, 6.0]], [[1.0, 6.0, 6.0]]], [[[2.0, 1.0, 2.0]], [[4.0, 7.0, 3.0]]]])
      arg[1:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 7.0], [1.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[4.0, 4.0, 3.0], [1.0, 4.0, 8.0], [3.0, 
3.0, 2.0]], [[1.0, 2.0, 3.0], [8.0, 4.0, 1.0], [2.0, 5.0, 1.0]]], [[[2.0, 4.0, 7.0], [7.0, 5.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 
7.0, 8.0], [1.0, 6.0, 6.0], [1.0, 5.0, 2.0]], [[8.0, 7.0, 1.0], [7.0, 8.0, 2.0], [7.0, 6.0, 7.0]]], [[[4.0, 3.0, 8.0], [2.0, 
1.0, 2.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 4.0], [4.0, 7.0, 3.0], [1.0, 8.0, 1.0]], [[7.0, 7.0, 3.0], [6.0, 3.0, 7.0], [3.0, 3.0, 
1.0]]], [[[4.0, 7.0, 7.0], [8.0, 3.0, 5.0], [3.0, 7.0, 6.0]], [[1.0, 6.0, 1.0], [2.0, 3.0, 8.0], [5.0, 3.0, 8.0]], [[1.0, 5.0, 
7.0], [1.0, 1.0, 4.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 2.0], [8.0, 1.0, 7.0], [6.0, 6.0, 5.0]], 
[[4.0, 1.0, 8.0], [6.0, 2.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 8.0], [4.0, 2.0, 7.0], [2.0, 1.0, 8.0]]], [[[1.0, 8.0, 7.0], 
[7.0, 5.0, 6.0], [1.0, 2.0, 7.0]], [[2.0, 5.0, 7.0], [1.0, 6.0, 6.0], [5.0, 8.0, 3.0]], [[4.0, 8.0, 7.0], [7.0, 1.0, 5.0], 
[5.0, 7.0, 2.0]]], [[[2.0, 8.0, 5.0], [2.0, 1.0, 2.0], [3.0, 1.0, 3.0]], [[2.0, 4.0, 2.0], [4.0, 7.0, 3.0], [8.0, 8.0, 1.0]], 
[[6.0, 1.0, 2.0], [3.0, 6.0, 5.0], [7.0, 3.0, 5.0]]], [[[2.0, 5.0, 7.0], [3.0, 3.0, 1.0], [6.0, 8.0, 7.0]], [[1.0, 3.0, 2.0], 
[3.0, 8.0, 6.0], [7.0, 4.0, 8.0]], [[3.0, 1.0, 1.0], [6.0, 8.0, 2.0], [5.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 6.0], [6.0, 1.0, 3.0], [2.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [8.0, 2.0, 8.0], [8.0, 
1.0, 7.0]], [[1.0, 7.0, 2.0], [4.0, 4.0, 8.0], [5.0, 4.0, 2.0]]], [[[1.0, 4.0, 8.0], [2.0, 7.0, 4.0], [1.0, 5.0, 5.0]], [[8.0, 
7.0, 2.0], [7.0, 4.0, 5.0], [8.0, 4.0, 1.0]], [[4.0, 4.0, 6.0], [2.0, 3.0, 6.0], [7.0, 7.0, 2.0]]], [[[6.0, 3.0, 3.0], [7.0, 
7.0, 7.0], [1.0, 2.0, 4.0]], [[2.0, 3.0, 6.0], [3.0, 1.0, 8.0], [5.0, 4.0, 1.0]], [[7.0, 3.0, 8.0], [3.0, 3.0, 3.0], [4.0, 1.0, 
6.0]]], [[[2.0, 8.0, 6.0], [2.0, 4.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 5.0], [6.0, 1.0, 4.0]], [[7.0, 8.0, 
7.0], [5.0, 4.0, 7.0], [1.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 1.0], [7.0, 1.0, 8.0], [6.0, 2.0, 5.0]], 
[[5.0, 1.0, 5.0], [3.0, 3.0, 4.0], [4.0, 4.0, 6.0]], [[7.0, 8.0, 6.0], [4.0, 8.0, 2.0], [5.0, 7.0, 2.0]]], [[[6.0, 8.0, 6.0], 
[3.0, 8.0, 7.0], [5.0, 5.0, 7.0]], [[1.0, 7.0, 4.0], [5.0, 5.0, 7.0], [5.0, 4.0, 7.0]], [[5.0, 5.0, 7.0], [5.0, 3.0, 7.0], 
[6.0, 2.0, 4.0]]], [[[7.0, 6.0, 5.0], [6.0, 3.0, 3.0], [2.0, 8.0, 4.0]], [[8.0, 4.0, 5.0], [6.0, 1.0, 4.0], [3.0, 6.0, 2.0]], 
[[8.0, 5.0, 1.0], [4.0, 2.0, 7.0], [3.0, 1.0, 6.0]]], [[[3.0, 1.0, 5.0], [3.0, 3.0, 2.0], [1.0, 8.0, 3.0]], [[7.0, 2.0, 6.0], 
[4.0, 3.0, 1.0], [7.0, 8.0, 6.0]], [[4.0, 3.0, 7.0], [8.0, 2.0, 6.0], [3.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[[5.0, 3.0, 5.0]], [[2.0, 6.0, 1.0]]], [[[3.0, 1.0, 7.0]], [[8.0, 5.0, 2.0]]]]),self.functionspace)
      arg[1:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 6.0], [6.0, 1.0, 3.0], [2.0, 4.0, 5.0]], [[7.0, 4.0, 7.0], [8.0, 2.0, 8.0], [8.0, 
1.0, 7.0]], [[1.0, 7.0, 2.0], [4.0, 4.0, 8.0], [5.0, 4.0, 2.0]]], [[[1.0, 4.0, 8.0], [5.0, 3.0, 5.0], [1.0, 5.0, 5.0]], [[8.0, 
7.0, 2.0], [2.0, 6.0, 1.0], [8.0, 4.0, 1.0]], [[4.0, 4.0, 6.0], [2.0, 3.0, 6.0], [7.0, 7.0, 2.0]]], [[[6.0, 3.0, 3.0], [3.0, 
1.0, 7.0], [1.0, 2.0, 4.0]], [[2.0, 3.0, 6.0], [8.0, 5.0, 2.0], [5.0, 4.0, 1.0]], [[7.0, 3.0, 8.0], [3.0, 3.0, 3.0], [4.0, 1.0, 
6.0]]], [[[2.0, 8.0, 6.0], [2.0, 4.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 3.0], [3.0, 7.0, 5.0], [6.0, 1.0, 4.0]], [[7.0, 8.0, 
7.0], [5.0, 4.0, 7.0], [1.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 1.0], [7.0, 1.0, 8.0], [6.0, 2.0, 5.0]], 
[[5.0, 1.0, 5.0], [3.0, 3.0, 4.0], [4.0, 4.0, 6.0]], [[7.0, 8.0, 6.0], [4.0, 8.0, 2.0], [5.0, 7.0, 2.0]]], [[[6.0, 8.0, 6.0], 
[5.0, 3.0, 5.0], [5.0, 5.0, 7.0]], [[1.0, 7.0, 4.0], [2.0, 6.0, 1.0], [5.0, 4.0, 7.0]], [[5.0, 5.0, 7.0], [5.0, 3.0, 7.0], 
[6.0, 2.0, 4.0]]], [[[7.0, 6.0, 5.0], [3.0, 1.0, 7.0], [2.0, 8.0, 4.0]], [[8.0, 4.0, 5.0], [8.0, 5.0, 2.0], [3.0, 6.0, 2.0]], 
[[8.0, 5.0, 1.0], [4.0, 2.0, 7.0], [3.0, 1.0, 6.0]]], [[[3.0, 1.0, 5.0], [3.0, 3.0, 2.0], [1.0, 8.0, 3.0]], [[7.0, 2.0, 6.0], 
[4.0, 3.0, 1.0], [7.0, 8.0, 6.0]], [[4.0, 3.0, 7.0], [8.0, 2.0, 6.0], [3.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 1.0], [2.0, 1.0, 4.0], [7.0, 3.0, 6.0]], [[6.0, 2.0, 3.0], [1.0, 6.0, 3.0], [6.0, 
3.0, 8.0]], [[6.0, 1.0, 2.0], [5.0, 2.0, 3.0], [4.0, 7.0, 7.0]]], [[[5.0, 7.0, 2.0], [1.0, 8.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 
3.0, 8.0], [3.0, 2.0, 1.0], [2.0, 2.0, 8.0]], [[5.0, 4.0, 3.0], [2.0, 7.0, 5.0], [6.0, 4.0, 7.0]]], [[[6.0, 1.0, 7.0], [6.0, 
1.0, 8.0], [4.0, 1.0, 1.0]], [[4.0, 6.0, 5.0], [2.0, 2.0, 3.0], [6.0, 6.0, 7.0]], [[5.0, 6.0, 8.0], [7.0, 7.0, 7.0], [7.0, 5.0, 
2.0]]], [[[5.0, 3.0, 4.0], [1.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[3.0, 4.0, 5.0], [6.0, 5.0, 6.0], [6.0, 4.0, 8.0]], [[5.0, 7.0, 
1.0], [2.0, 7.0, 6.0], [5.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 2.0], [4.0, 2.0, 2.0], [6.0, 1.0, 5.0]], 
[[3.0, 1.0, 2.0], [4.0, 1.0, 5.0], [8.0, 7.0, 4.0]], [[2.0, 1.0, 1.0], [8.0, 5.0, 7.0], [8.0, 1.0, 3.0]]], [[[1.0, 7.0, 4.0], 
[4.0, 8.0, 2.0], [7.0, 8.0, 1.0]], [[3.0, 7.0, 1.0], [6.0, 7.0, 1.0], [5.0, 6.0, 1.0]], [[2.0, 6.0, 7.0], [2.0, 8.0, 8.0], 
[7.0, 1.0, 4.0]]], [[[1.0, 7.0, 4.0], [8.0, 7.0, 1.0], [5.0, 1.0, 4.0]], [[7.0, 8.0, 8.0], [1.0, 1.0, 3.0], [2.0, 6.0, 8.0]], 
[[7.0, 1.0, 1.0], [7.0, 5.0, 8.0], [5.0, 8.0, 7.0]]], [[[6.0, 3.0, 6.0], [4.0, 6.0, 5.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 6.0], 
[4.0, 5.0, 8.0], [7.0, 3.0, 6.0]], [[8.0, 2.0, 8.0], [1.0, 8.0, 6.0], [2.0, 4.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 5.0, 5.0]], [[1.0, 6.0, 3.0]]], [[[1.0, 6.0, 7.0]], [[5.0, 2.0, 
7.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 3.0]], [[5.0, 8.0, 6.0]]], [[[3.0, 5.0, 8.0]], [[7.0, 2.0, 8.0]]]])
      arg[1:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [2.0, 1.0, 4.0], [7.0, 3.0, 6.0]], [[6.0, 2.0, 3.0], [1.0, 6.0, 3.0], [6.0, 
3.0, 8.0]], [[6.0, 1.0, 2.0], [5.0, 2.0, 3.0], [4.0, 7.0, 7.0]]], [[[5.0, 7.0, 2.0], [4.0, 5.0, 5.0], [8.0, 2.0, 3.0]], [[8.0, 
3.0, 8.0], [1.0, 6.0, 3.0], [2.0, 2.0, 8.0]], [[5.0, 4.0, 3.0], [2.0, 7.0, 5.0], [6.0, 4.0, 7.0]]], [[[6.0, 1.0, 7.0], [1.0, 
6.0, 7.0], [4.0, 1.0, 1.0]], [[4.0, 6.0, 5.0], [5.0, 2.0, 7.0], [6.0, 6.0, 7.0]], [[5.0, 6.0, 8.0], [7.0, 7.0, 7.0], [7.0, 5.0, 
2.0]]], [[[5.0, 3.0, 4.0], [1.0, 8.0, 3.0], [1.0, 6.0, 1.0]], [[3.0, 4.0, 5.0], [6.0, 5.0, 6.0], [6.0, 4.0, 8.0]], [[5.0, 7.0, 
1.0], [2.0, 7.0, 6.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 2.0], [4.0, 2.0, 2.0], [6.0, 1.0, 5.0]], 
[[3.0, 1.0, 2.0], [4.0, 1.0, 5.0], [8.0, 7.0, 4.0]], [[2.0, 1.0, 1.0], [8.0, 5.0, 7.0], [8.0, 1.0, 3.0]]], [[[1.0, 7.0, 4.0], 
[2.0, 5.0, 3.0], [7.0, 8.0, 1.0]], [[3.0, 7.0, 1.0], [5.0, 8.0, 6.0], [5.0, 6.0, 1.0]], [[2.0, 6.0, 7.0], [2.0, 8.0, 8.0], 
[7.0, 1.0, 4.0]]], [[[1.0, 7.0, 4.0], [3.0, 5.0, 8.0], [5.0, 1.0, 4.0]], [[7.0, 8.0, 8.0], [7.0, 2.0, 8.0], [2.0, 6.0, 8.0]], 
[[7.0, 1.0, 1.0], [7.0, 5.0, 8.0], [5.0, 8.0, 7.0]]], [[[6.0, 3.0, 6.0], [4.0, 6.0, 5.0], [2.0, 5.0, 5.0]], [[6.0, 5.0, 6.0], 
[4.0, 5.0, 8.0], [7.0, 3.0, 6.0]], [[8.0, 2.0, 8.0], [1.0, 8.0, 6.0], [2.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 5.0], [7.0, 4.0, 2.0], [6.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [8.0, 8.0, 6.0], [4.0, 
8.0, 3.0]], [[8.0, 8.0, 1.0], [2.0, 3.0, 7.0], [4.0, 2.0, 2.0]]], [[[2.0, 7.0, 5.0], [6.0, 5.0, 4.0], [6.0, 1.0, 1.0]], [[1.0, 
4.0, 4.0], [3.0, 5.0, 6.0], [2.0, 4.0, 5.0]], [[3.0, 3.0, 8.0], [7.0, 3.0, 8.0], [7.0, 7.0, 3.0]]], [[[1.0, 8.0, 8.0], [4.0, 
1.0, 1.0], [6.0, 3.0, 2.0]], [[1.0, 1.0, 2.0], [4.0, 5.0, 4.0], [2.0, 3.0, 4.0]], [[4.0, 2.0, 3.0], [3.0, 8.0, 8.0], [5.0, 4.0, 
2.0]]], [[[6.0, 1.0, 8.0], [3.0, 2.0, 7.0], [6.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [3.0, 8.0, 2.0], [2.0, 3.0, 3.0]], [[6.0, 7.0, 
4.0], [4.0, 3.0, 8.0], [5.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 3.0, 8.0], [5.0, 7.0, 5.0], [1.0, 1.0, 6.0]], 
[[3.0, 5.0, 2.0], [7.0, 2.0, 3.0], [2.0, 6.0, 7.0]], [[4.0, 7.0, 3.0], [2.0, 7.0, 5.0], [4.0, 6.0, 7.0]]], [[[6.0, 6.0, 6.0], 
[7.0, 8.0, 8.0], [5.0, 6.0, 7.0]], [[6.0, 5.0, 6.0], [5.0, 6.0, 5.0], [2.0, 5.0, 2.0]], [[7.0, 6.0, 4.0], [8.0, 4.0, 5.0], 
[4.0, 7.0, 8.0]]], [[[5.0, 1.0, 8.0], [2.0, 1.0, 8.0], [6.0, 5.0, 5.0]], [[5.0, 5.0, 7.0], [5.0, 1.0, 7.0], [8.0, 4.0, 7.0]], 
[[4.0, 2.0, 7.0], [6.0, 2.0, 4.0], [1.0, 1.0, 4.0]]], [[[4.0, 2.0, 6.0], [3.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[3.0, 4.0, 5.0], 
[6.0, 1.0, 7.0], [7.0, 7.0, 7.0]], [[4.0, 2.0, 5.0], [1.0, 2.0, 2.0], [8.0, 5.0, 5.0]]]])
      val=numpy.array([[[[4.0, 4.0, 5.0], [1.0, 4.0, 7.0]], [[6.0, 3.0, 8.0], [8.0, 2.0, 4.0]], [[3.0, 2.0, 3.0], [1.0, 8.0, 
3.0]]], [[[6.0, 5.0, 6.0], [4.0, 2.0, 5.0]], [[8.0, 4.0, 5.0], [1.0, 4.0, 1.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 8.0]]]])
      arg[1:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 5.0], [7.0, 4.0, 2.0], [6.0, 5.0, 4.0]], [[2.0, 8.0, 5.0], [8.0, 8.0, 6.0], [4.0, 
8.0, 3.0]], [[8.0, 8.0, 1.0], [2.0, 3.0, 7.0], [4.0, 2.0, 2.0]]], [[[4.0, 4.0, 5.0], [1.0, 4.0, 7.0], [6.0, 1.0, 1.0]], [[6.0, 
3.0, 8.0], [8.0, 2.0, 4.0], [2.0, 4.0, 5.0]], [[3.0, 2.0, 3.0], [1.0, 8.0, 3.0], [7.0, 7.0, 3.0]]], [[[6.0, 5.0, 6.0], [4.0, 
2.0, 5.0], [6.0, 3.0, 2.0]], [[8.0, 4.0, 5.0], [1.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 8.0], [1.0, 3.0, 8.0], [5.0, 4.0, 
2.0]]], [[[6.0, 1.0, 8.0], [3.0, 2.0, 7.0], [6.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [3.0, 8.0, 2.0], [2.0, 3.0, 3.0]], [[6.0, 7.0, 
4.0], [4.0, 3.0, 8.0], [5.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 3.0, 8.0], [5.0, 7.0, 5.0], [1.0, 1.0, 6.0]], 
[[3.0, 5.0, 2.0], [7.0, 2.0, 3.0], [2.0, 6.0, 7.0]], [[4.0, 7.0, 3.0], [2.0, 7.0, 5.0], [4.0, 6.0, 7.0]]], [[[4.0, 4.0, 5.0], 
[1.0, 4.0, 7.0], [5.0, 6.0, 7.0]], [[6.0, 3.0, 8.0], [8.0, 2.0, 4.0], [2.0, 5.0, 2.0]], [[3.0, 2.0, 3.0], [1.0, 8.0, 3.0], 
[4.0, 7.0, 8.0]]], [[[6.0, 5.0, 6.0], [4.0, 2.0, 5.0], [6.0, 5.0, 5.0]], [[8.0, 4.0, 5.0], [1.0, 4.0, 1.0], [8.0, 4.0, 7.0]], 
[[4.0, 8.0, 8.0], [1.0, 3.0, 8.0], [1.0, 1.0, 4.0]]], [[[4.0, 2.0, 6.0], [3.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[3.0, 4.0, 5.0], 
[6.0, 1.0, 7.0], [7.0, 7.0, 7.0]], [[4.0, 2.0, 5.0], [1.0, 2.0, 2.0], [8.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 6.0], [2.0, 8.0, 7.0], [1.0, 3.0, 4.0]], [[7.0, 1.0, 2.0], [5.0, 4.0, 1.0], [2.0, 
3.0, 4.0]], [[4.0, 3.0, 4.0], [2.0, 1.0, 2.0], [7.0, 6.0, 4.0]]], [[[4.0, 7.0, 3.0], [1.0, 8.0, 4.0], [2.0, 6.0, 4.0]], [[4.0, 
5.0, 8.0], [2.0, 8.0, 5.0], [4.0, 5.0, 2.0]], [[5.0, 3.0, 1.0], [5.0, 2.0, 6.0], [6.0, 8.0, 5.0]]], [[[8.0, 3.0, 5.0], [5.0, 
6.0, 8.0], [2.0, 7.0, 3.0]], [[5.0, 7.0, 5.0], [2.0, 6.0, 1.0], [2.0, 3.0, 8.0]], [[4.0, 5.0, 5.0], [4.0, 2.0, 7.0], [1.0, 6.0, 
6.0]]], [[[7.0, 5.0, 4.0], [7.0, 4.0, 7.0], [5.0, 5.0, 3.0]], [[3.0, 1.0, 7.0], [8.0, 5.0, 1.0], [5.0, 2.0, 7.0]], [[4.0, 4.0, 
4.0], [4.0, 1.0, 1.0], [6.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 8.0], [4.0, 2.0, 5.0], [4.0, 1.0, 7.0]], 
[[5.0, 6.0, 5.0], [4.0, 7.0, 8.0], [5.0, 7.0, 8.0]], [[6.0, 7.0, 2.0], [6.0, 7.0, 1.0], [7.0, 4.0, 1.0]]], [[[4.0, 3.0, 3.0], 
[3.0, 7.0, 7.0], [7.0, 6.0, 7.0]], [[7.0, 3.0, 4.0], [3.0, 5.0, 2.0], [4.0, 4.0, 7.0]], [[2.0, 6.0, 2.0], [3.0, 2.0, 4.0], 
[7.0, 1.0, 8.0]]], [[[1.0, 7.0, 1.0], [5.0, 2.0, 2.0], [6.0, 1.0, 1.0]], [[5.0, 5.0, 8.0], [8.0, 2.0, 8.0], [5.0, 5.0, 8.0]], 
[[5.0, 7.0, 1.0], [7.0, 8.0, 6.0], [6.0, 7.0, 8.0]]], [[[8.0, 1.0, 6.0], [2.0, 6.0, 6.0], [7.0, 4.0, 5.0]], [[3.0, 8.0, 1.0], 
[5.0, 3.0, 7.0], [7.0, 5.0, 4.0]], [[8.0, 7.0, 3.0], [7.0, 2.0, 8.0], [1.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0, 3.0, 6.0], [7.0, 7.0, 7.0]], [[3.0, 3.0, 2.0], [8.0, 3.0, 4.0]], [[3.0, 6.0, 7.0], [3.0, 
3.0, 6.0]]], [[[7.0, 1.0, 1.0], [8.0, 2.0, 2.0]], [[3.0, 4.0, 5.0], [5.0, 7.0, 4.0]], [[3.0, 8.0, 8.0], [7.0, 1.0, 
1.0]]]]),self.functionspace)
      arg[1:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 6.0], [2.0, 8.0, 7.0], [1.0, 3.0, 4.0]], [[7.0, 1.0, 2.0], [5.0, 4.0, 1.0], [2.0, 
3.0, 4.0]], [[4.0, 3.0, 4.0], [2.0, 1.0, 2.0], [7.0, 6.0, 4.0]]], [[[8.0, 3.0, 6.0], [7.0, 7.0, 7.0], [2.0, 6.0, 4.0]], [[3.0, 
3.0, 2.0], [8.0, 3.0, 4.0], [4.0, 5.0, 2.0]], [[3.0, 6.0, 7.0], [3.0, 3.0, 6.0], [6.0, 8.0, 5.0]]], [[[7.0, 1.0, 1.0], [8.0, 
2.0, 2.0], [2.0, 7.0, 3.0]], [[3.0, 4.0, 5.0], [5.0, 7.0, 4.0], [2.0, 3.0, 8.0]], [[3.0, 8.0, 8.0], [7.0, 1.0, 1.0], [1.0, 6.0, 
6.0]]], [[[7.0, 5.0, 4.0], [7.0, 4.0, 7.0], [5.0, 5.0, 3.0]], [[3.0, 1.0, 7.0], [8.0, 5.0, 1.0], [5.0, 2.0, 7.0]], [[4.0, 4.0, 
4.0], [4.0, 1.0, 1.0], [6.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 8.0], [4.0, 2.0, 5.0], [4.0, 1.0, 7.0]], 
[[5.0, 6.0, 5.0], [4.0, 7.0, 8.0], [5.0, 7.0, 8.0]], [[6.0, 7.0, 2.0], [6.0, 7.0, 1.0], [7.0, 4.0, 1.0]]], [[[8.0, 3.0, 6.0], 
[7.0, 7.0, 7.0], [7.0, 6.0, 7.0]], [[3.0, 3.0, 2.0], [8.0, 3.0, 4.0], [4.0, 4.0, 7.0]], [[3.0, 6.0, 7.0], [3.0, 3.0, 6.0], 
[7.0, 1.0, 8.0]]], [[[7.0, 1.0, 1.0], [8.0, 2.0, 2.0], [6.0, 1.0, 1.0]], [[3.0, 4.0, 5.0], [5.0, 7.0, 4.0], [5.0, 5.0, 8.0]], 
[[3.0, 8.0, 8.0], [7.0, 1.0, 1.0], [6.0, 7.0, 8.0]]], [[[8.0, 1.0, 6.0], [2.0, 6.0, 6.0], [7.0, 4.0, 5.0]], [[3.0, 8.0, 1.0], 
[5.0, 3.0, 7.0], [7.0, 5.0, 4.0]], [[8.0, 7.0, 3.0], [7.0, 2.0, 8.0], [1.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 4.0], [6.0, 7.0, 6.0], [3.0, 6.0, 3.0]], [[2.0, 2.0, 3.0], [2.0, 1.0, 8.0], [3.0, 
5.0, 2.0]], [[7.0, 4.0, 5.0], [7.0, 6.0, 4.0], [1.0, 5.0, 8.0]]], [[[3.0, 2.0, 3.0], [5.0, 7.0, 3.0], [6.0, 8.0, 1.0]], [[6.0, 
7.0, 5.0], [3.0, 8.0, 2.0], [7.0, 3.0, 1.0]], [[1.0, 3.0, 4.0], [2.0, 3.0, 7.0], [5.0, 2.0, 3.0]]], [[[8.0, 5.0, 5.0], [7.0, 
4.0, 2.0], [5.0, 8.0, 8.0]], [[1.0, 1.0, 8.0], [7.0, 7.0, 2.0], [5.0, 8.0, 2.0]], [[8.0, 3.0, 3.0], [5.0, 6.0, 6.0], [6.0, 1.0, 
3.0]]], [[[8.0, 3.0, 8.0], [7.0, 6.0, 1.0], [7.0, 4.0, 7.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 7.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 
2.0], [2.0, 8.0, 8.0], [4.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 1.0], [8.0, 2.0, 3.0], [4.0, 8.0, 4.0]], 
[[7.0, 1.0, 8.0], [7.0, 5.0, 7.0], [4.0, 8.0, 4.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 4.0], [5.0, 8.0, 2.0]]], [[[6.0, 1.0, 3.0], 
[6.0, 5.0, 8.0], [2.0, 7.0, 5.0]], [[8.0, 6.0, 7.0], [7.0, 2.0, 6.0], [1.0, 8.0, 4.0]], [[4.0, 4.0, 1.0], [4.0, 4.0, 5.0], 
[3.0, 6.0, 5.0]]], [[[4.0, 6.0, 7.0], [5.0, 7.0, 1.0], [4.0, 8.0, 3.0]], [[8.0, 1.0, 4.0], [1.0, 6.0, 1.0], [6.0, 3.0, 7.0]], 
[[7.0, 8.0, 2.0], [7.0, 5.0, 6.0], [5.0, 6.0, 1.0]]], [[[3.0, 3.0, 5.0], [6.0, 1.0, 2.0], [6.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], 
[4.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[8.0, 3.0, 7.0], [2.0, 2.0, 1.0], [6.0, 4.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 5.0, 8.0], [5.0, 5.0, 8.0]], [[6.0, 5.0, 7.0], [4.0, 2.0, 7.0]], [[7.0, 6.0, 7.0], [5.0, 
7.0, 5.0]]], [[[2.0, 4.0, 4.0], [8.0, 2.0, 1.0]], [[3.0, 6.0, 6.0], [5.0, 6.0, 4.0]], [[6.0, 7.0, 7.0], [1.0, 1.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 8.0], [6.0, 8.0, 4.0]], [[7.0, 1.0, 8.0], [3.0, 2.0, 7.0]], [[8.0, 8.0, 1.0], 
[2.0, 4.0, 2.0]]], [[[1.0, 7.0, 6.0], [3.0, 5.0, 7.0]], [[7.0, 5.0, 5.0], [1.0, 3.0, 3.0]], [[3.0, 2.0, 1.0], [4.0, 1.0, 
6.0]]]])
      arg[1:3,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 4.0], [6.0, 7.0, 6.0], [3.0, 6.0, 3.0]], [[2.0, 2.0, 3.0], [2.0, 1.0, 8.0], [3.0, 
5.0, 2.0]], [[7.0, 4.0, 5.0], [7.0, 6.0, 4.0], [1.0, 5.0, 8.0]]], [[[6.0, 5.0, 8.0], [5.0, 5.0, 8.0], [6.0, 8.0, 1.0]], [[6.0, 
5.0, 7.0], [4.0, 2.0, 7.0], [7.0, 3.0, 1.0]], [[7.0, 6.0, 7.0], [5.0, 7.0, 5.0], [5.0, 2.0, 3.0]]], [[[2.0, 4.0, 4.0], [8.0, 
2.0, 1.0], [5.0, 8.0, 8.0]], [[3.0, 6.0, 6.0], [5.0, 6.0, 4.0], [5.0, 8.0, 2.0]], [[6.0, 7.0, 7.0], [1.0, 1.0, 2.0], [6.0, 1.0, 
3.0]]], [[[8.0, 3.0, 8.0], [7.0, 6.0, 1.0], [7.0, 4.0, 7.0]], [[5.0, 7.0, 1.0], [1.0, 6.0, 7.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 
2.0], [2.0, 8.0, 8.0], [4.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 1.0], [8.0, 2.0, 3.0], [4.0, 8.0, 4.0]], 
[[7.0, 1.0, 8.0], [7.0, 5.0, 7.0], [4.0, 8.0, 4.0]], [[2.0, 5.0, 8.0], [6.0, 7.0, 4.0], [5.0, 8.0, 2.0]]], [[[5.0, 4.0, 8.0], 
[6.0, 8.0, 4.0], [2.0, 7.0, 5.0]], [[7.0, 1.0, 8.0], [3.0, 2.0, 7.0], [1.0, 8.0, 4.0]], [[8.0, 8.0, 1.0], [2.0, 4.0, 2.0], 
[3.0, 6.0, 5.0]]], [[[1.0, 7.0, 6.0], [3.0, 5.0, 7.0], [4.0, 8.0, 3.0]], [[7.0, 5.0, 5.0], [1.0, 3.0, 3.0], [6.0, 3.0, 7.0]], 
[[3.0, 2.0, 1.0], [4.0, 1.0, 6.0], [5.0, 6.0, 1.0]]], [[[3.0, 3.0, 5.0], [6.0, 1.0, 2.0], [6.0, 5.0, 7.0]], [[8.0, 6.0, 7.0], 
[4.0, 5.0, 7.0], [7.0, 8.0, 8.0]], [[8.0, 3.0, 7.0], [2.0, 2.0, 1.0], [6.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 6.0, 5.0], [2.0, 8.0, 3.0]], [[4.0, 4.0, 2.0], [1.0, 6.0, 8.0], [1.0, 
6.0, 4.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 3.0], [2.0, 8.0, 5.0]]], [[[2.0, 5.0, 3.0], [4.0, 6.0, 1.0], [7.0, 5.0, 7.0]], [[6.0, 
5.0, 6.0], [4.0, 7.0, 6.0], [7.0, 4.0, 5.0]], [[4.0, 3.0, 1.0], [3.0, 1.0, 6.0], [3.0, 4.0, 3.0]]], [[[6.0, 3.0, 8.0], [1.0, 
1.0, 6.0], [4.0, 7.0, 1.0]], [[1.0, 8.0, 2.0], [7.0, 1.0, 2.0], [1.0, 6.0, 2.0]], [[6.0, 7.0, 7.0], [5.0, 5.0, 7.0], [7.0, 4.0, 
2.0]]], [[[5.0, 5.0, 2.0], [2.0, 1.0, 2.0], [6.0, 2.0, 8.0]], [[1.0, 5.0, 7.0], [1.0, 1.0, 1.0], [6.0, 4.0, 1.0]], [[7.0, 6.0, 
3.0], [3.0, 4.0, 5.0], [1.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 7.0], [7.0, 8.0, 3.0], [3.0, 1.0, 5.0]], 
[[4.0, 4.0, 5.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[4.0, 2.0, 3.0], [6.0, 3.0, 2.0], [7.0, 3.0, 3.0]]], [[[6.0, 4.0, 2.0], 
[5.0, 3.0, 8.0], [8.0, 3.0, 2.0]], [[2.0, 5.0, 6.0], [5.0, 5.0, 1.0], [1.0, 1.0, 5.0]], [[2.0, 2.0, 6.0], [6.0, 8.0, 6.0], 
[3.0, 7.0, 5.0]]], [[[6.0, 8.0, 1.0], [2.0, 2.0, 7.0], [6.0, 2.0, 1.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 6.0], [4.0, 6.0, 7.0]], 
[[7.0, 3.0, 1.0], [6.0, 2.0, 6.0], [4.0, 7.0, 6.0]]], [[[5.0, 2.0, 7.0], [2.0, 5.0, 1.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 6.0], 
[7.0, 6.0, 1.0], [6.0, 1.0, 8.0]], [[2.0, 2.0, 3.0], [8.0, 2.0, 2.0], [1.0, 1.0, 2.0]]]])
      val=numpy.array([[[[1.0, 4.0, 6.0]], [[2.0, 1.0, 3.0]]], [[[4.0, 1.0, 1.0]], [[7.0, 8.0, 2.0]]], [[[6.0, 7.0, 3.0]], 
[[8.0, 2.0, 4.0]]]])
      arg[:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [2.0, 6.0, 5.0], [1.0, 4.0, 6.0]], [[4.0, 4.0, 2.0], [1.0, 6.0, 8.0], [2.0, 
1.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 1.0, 3.0], [2.0, 8.0, 5.0]]], [[[2.0, 5.0, 3.0], [4.0, 6.0, 1.0], [4.0, 1.0, 1.0]], [[6.0, 
5.0, 6.0], [4.0, 7.0, 6.0], [7.0, 8.0, 2.0]], [[4.0, 3.0, 1.0], [3.0, 1.0, 6.0], [3.0, 4.0, 3.0]]], [[[6.0, 3.0, 8.0], [1.0, 
1.0, 6.0], [6.0, 7.0, 3.0]], [[1.0, 8.0, 2.0], [7.0, 1.0, 2.0], [8.0, 2.0, 4.0]], [[6.0, 7.0, 7.0], [5.0, 5.0, 7.0], [7.0, 4.0, 
2.0]]], [[[5.0, 5.0, 2.0], [2.0, 1.0, 2.0], [6.0, 2.0, 8.0]], [[1.0, 5.0, 7.0], [1.0, 1.0, 1.0], [6.0, 4.0, 1.0]], [[7.0, 6.0, 
3.0], [3.0, 4.0, 5.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 7.0], [7.0, 8.0, 3.0], [1.0, 4.0, 6.0]], 
[[4.0, 4.0, 5.0], [3.0, 1.0, 7.0], [2.0, 1.0, 3.0]], [[4.0, 2.0, 3.0], [6.0, 3.0, 2.0], [7.0, 3.0, 3.0]]], [[[6.0, 4.0, 2.0], 
[5.0, 3.0, 8.0], [4.0, 1.0, 1.0]], [[2.0, 5.0, 6.0], [5.0, 5.0, 1.0], [7.0, 8.0, 2.0]], [[2.0, 2.0, 6.0], [6.0, 8.0, 6.0], 
[3.0, 7.0, 5.0]]], [[[6.0, 8.0, 1.0], [2.0, 2.0, 7.0], [6.0, 7.0, 3.0]], [[3.0, 3.0, 3.0], [6.0, 5.0, 6.0], [8.0, 2.0, 4.0]], 
[[7.0, 3.0, 1.0], [6.0, 2.0, 6.0], [4.0, 7.0, 6.0]]], [[[5.0, 2.0, 7.0], [2.0, 5.0, 1.0], [4.0, 5.0, 5.0]], [[5.0, 1.0, 6.0], 
[7.0, 6.0, 1.0], [6.0, 1.0, 8.0]], [[2.0, 2.0, 3.0], [8.0, 2.0, 2.0], [1.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 1.0], [7.0, 4.0, 6.0], [6.0, 5.0, 4.0]], [[3.0, 4.0, 1.0], [6.0, 7.0, 7.0], [8.0, 
6.0, 8.0]], [[2.0, 6.0, 3.0], [7.0, 6.0, 6.0], [5.0, 8.0, 1.0]]], [[[1.0, 5.0, 4.0], [4.0, 8.0, 3.0], [1.0, 3.0, 2.0]], [[2.0, 
7.0, 3.0], [6.0, 3.0, 2.0], [3.0, 4.0, 6.0]], [[5.0, 5.0, 4.0], [6.0, 7.0, 8.0], [8.0, 3.0, 5.0]]], [[[3.0, 6.0, 2.0], [6.0, 
1.0, 1.0], [8.0, 3.0, 5.0]], [[1.0, 1.0, 5.0], [8.0, 6.0, 2.0], [4.0, 5.0, 3.0]], [[4.0, 8.0, 6.0], [5.0, 8.0, 4.0], [4.0, 1.0, 
4.0]]], [[[2.0, 3.0, 4.0], [2.0, 1.0, 6.0], [1.0, 3.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 4.0, 5.0], [1.0, 8.0, 8.0]], [[7.0, 4.0, 
1.0], [3.0, 5.0, 4.0], [1.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 6.0], [4.0, 2.0, 8.0], [5.0, 8.0, 3.0]], 
[[4.0, 6.0, 5.0], [8.0, 3.0, 2.0], [2.0, 7.0, 3.0]], [[5.0, 3.0, 2.0], [3.0, 3.0, 6.0], [1.0, 8.0, 5.0]]], [[[8.0, 2.0, 2.0], 
[3.0, 2.0, 2.0], [4.0, 5.0, 2.0]], [[8.0, 6.0, 6.0], [7.0, 1.0, 1.0], [1.0, 3.0, 4.0]], [[2.0, 2.0, 6.0], [3.0, 1.0, 7.0], 
[1.0, 1.0, 5.0]]], [[[6.0, 1.0, 1.0], [7.0, 6.0, 2.0], [6.0, 5.0, 2.0]], [[5.0, 5.0, 5.0], [5.0, 3.0, 7.0], [2.0, 8.0, 7.0]], 
[[6.0, 7.0, 5.0], [2.0, 8.0, 5.0], [8.0, 6.0, 6.0]]], [[[5.0, 6.0, 6.0], [3.0, 4.0, 3.0], [6.0, 6.0, 2.0]], [[2.0, 5.0, 3.0], 
[8.0, 7.0, 1.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 7.0], [3.0, 7.0, 5.0], [1.0, 3.0, 7.0]]]])
      val=Data(numpy.array([[[[7.0, 3.0, 7.0]], [[8.0, 5.0, 5.0]]], [[[7.0, 4.0, 3.0]], [[4.0, 6.0, 1.0]]], [[[1.0, 7.0, 6.0]], 
[[4.0, 8.0, 5.0]]]]),self.functionspace)
      arg[:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 1.0], [7.0, 4.0, 6.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 1.0], [6.0, 7.0, 7.0], [8.0, 
5.0, 5.0]], [[2.0, 6.0, 3.0], [7.0, 6.0, 6.0], [5.0, 8.0, 1.0]]], [[[1.0, 5.0, 4.0], [4.0, 8.0, 3.0], [7.0, 4.0, 3.0]], [[2.0, 
7.0, 3.0], [6.0, 3.0, 2.0], [4.0, 6.0, 1.0]], [[5.0, 5.0, 4.0], [6.0, 7.0, 8.0], [8.0, 3.0, 5.0]]], [[[3.0, 6.0, 2.0], [6.0, 
1.0, 1.0], [1.0, 7.0, 6.0]], [[1.0, 1.0, 5.0], [8.0, 6.0, 2.0], [4.0, 8.0, 5.0]], [[4.0, 8.0, 6.0], [5.0, 8.0, 4.0], [4.0, 1.0, 
4.0]]], [[[2.0, 3.0, 4.0], [2.0, 1.0, 6.0], [1.0, 3.0, 6.0]], [[7.0, 8.0, 3.0], [8.0, 4.0, 5.0], [1.0, 8.0, 8.0]], [[7.0, 4.0, 
1.0], [3.0, 5.0, 4.0], [1.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 6.0], [4.0, 2.0, 8.0], [7.0, 3.0, 7.0]], 
[[4.0, 6.0, 5.0], [8.0, 3.0, 2.0], [8.0, 5.0, 5.0]], [[5.0, 3.0, 2.0], [3.0, 3.0, 6.0], [1.0, 8.0, 5.0]]], [[[8.0, 2.0, 2.0], 
[3.0, 2.0, 2.0], [7.0, 4.0, 3.0]], [[8.0, 6.0, 6.0], [7.0, 1.0, 1.0], [4.0, 6.0, 1.0]], [[2.0, 2.0, 6.0], [3.0, 1.0, 7.0], 
[1.0, 1.0, 5.0]]], [[[6.0, 1.0, 1.0], [7.0, 6.0, 2.0], [1.0, 7.0, 6.0]], [[5.0, 5.0, 5.0], [5.0, 3.0, 7.0], [4.0, 8.0, 5.0]], 
[[6.0, 7.0, 5.0], [2.0, 8.0, 5.0], [8.0, 6.0, 6.0]]], [[[5.0, 6.0, 6.0], [3.0, 4.0, 3.0], [6.0, 6.0, 2.0]], [[2.0, 5.0, 3.0], 
[8.0, 7.0, 1.0], [8.0, 4.0, 8.0]], [[7.0, 2.0, 7.0], [3.0, 7.0, 5.0], [1.0, 3.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 1.0], [3.0, 5.0, 3.0], [1.0, 8.0, 4.0]], [[7.0, 8.0, 8.0], [6.0, 5.0, 5.0], [4.0, 
2.0, 5.0]], [[2.0, 2.0, 8.0], [1.0, 3.0, 2.0], [8.0, 2.0, 3.0]]], [[[3.0, 5.0, 5.0], [7.0, 3.0, 2.0], [4.0, 5.0, 7.0]], [[3.0, 
6.0, 2.0], [3.0, 4.0, 7.0], [2.0, 2.0, 7.0]], [[6.0, 8.0, 8.0], [5.0, 5.0, 3.0], [6.0, 4.0, 1.0]]], [[[2.0, 2.0, 1.0], [4.0, 
4.0, 4.0], [6.0, 6.0, 6.0]], [[8.0, 6.0, 2.0], [8.0, 5.0, 2.0], [6.0, 5.0, 2.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 3.0], [2.0, 8.0, 
5.0]]], [[[1.0, 3.0, 6.0], [1.0, 7.0, 1.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 3.0], [4.0, 8.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 8.0, 
8.0], [6.0, 5.0, 8.0], [1.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 2.0], [3.0, 2.0, 3.0], [6.0, 8.0, 1.0]], 
[[1.0, 4.0, 2.0], [1.0, 3.0, 2.0], [5.0, 7.0, 5.0]], [[7.0, 1.0, 3.0], [4.0, 8.0, 3.0], [4.0, 6.0, 5.0]]], [[[3.0, 7.0, 3.0], 
[1.0, 6.0, 8.0], [8.0, 3.0, 4.0]], [[2.0, 7.0, 5.0], [7.0, 7.0, 4.0], [7.0, 7.0, 5.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 6.0], 
[7.0, 1.0, 8.0]]], [[[1.0, 6.0, 1.0], [7.0, 2.0, 2.0], [2.0, 2.0, 5.0]], [[6.0, 6.0, 7.0], [1.0, 3.0, 4.0], [5.0, 4.0, 4.0]], 
[[7.0, 5.0, 1.0], [8.0, 6.0, 4.0], [8.0, 1.0, 8.0]]], [[[6.0, 5.0, 6.0], [7.0, 3.0, 5.0], [6.0, 1.0, 3.0]], [[1.0, 6.0, 3.0], 
[4.0, 7.0, 2.0], [3.0, 1.0, 8.0]], [[2.0, 4.0, 8.0], [1.0, 5.0, 3.0], [4.0, 3.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 1.0, 6.0]], [[8.0, 1.0, 4.0]]], [[[7.0, 5.0, 2.0]], [[1.0, 6.0, 7.0]]], [[[1.0, 1.0, 
6.0]], [[7.0, 8.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 6.0, 1.0]], [[1.0, 5.0, 7.0]]], [[[4.0, 4.0, 7.0]], [[5.0, 5.0, 
6.0]]], [[[8.0, 6.0, 3.0]], [[7.0, 3.0, 6.0]]]])
      arg[:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 1.0], [3.0, 5.0, 3.0], [8.0, 1.0, 6.0]], [[7.0, 8.0, 8.0], [6.0, 5.0, 5.0], [8.0, 
1.0, 4.0]], [[2.0, 2.0, 8.0], [1.0, 3.0, 2.0], [8.0, 2.0, 3.0]]], [[[3.0, 5.0, 5.0], [7.0, 3.0, 2.0], [7.0, 5.0, 2.0]], [[3.0, 
6.0, 2.0], [3.0, 4.0, 7.0], [1.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], [5.0, 5.0, 3.0], [6.0, 4.0, 1.0]]], [[[2.0, 2.0, 1.0], [4.0, 
4.0, 4.0], [1.0, 1.0, 6.0]], [[8.0, 6.0, 2.0], [8.0, 5.0, 2.0], [7.0, 8.0, 7.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 3.0], [2.0, 8.0, 
5.0]]], [[[1.0, 3.0, 6.0], [1.0, 7.0, 1.0], [5.0, 8.0, 2.0]], [[1.0, 4.0, 3.0], [4.0, 8.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 8.0, 
8.0], [6.0, 5.0, 8.0], [1.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 2.0], [3.0, 2.0, 3.0], [7.0, 6.0, 1.0]], 
[[1.0, 4.0, 2.0], [1.0, 3.0, 2.0], [1.0, 5.0, 7.0]], [[7.0, 1.0, 3.0], [4.0, 8.0, 3.0], [4.0, 6.0, 5.0]]], [[[3.0, 7.0, 3.0], 
[1.0, 6.0, 8.0], [4.0, 4.0, 7.0]], [[2.0, 7.0, 5.0], [7.0, 7.0, 4.0], [5.0, 5.0, 6.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 6.0], 
[7.0, 1.0, 8.0]]], [[[1.0, 6.0, 1.0], [7.0, 2.0, 2.0], [8.0, 6.0, 3.0]], [[6.0, 6.0, 7.0], [1.0, 3.0, 4.0], [7.0, 3.0, 6.0]], 
[[7.0, 5.0, 1.0], [8.0, 6.0, 4.0], [8.0, 1.0, 8.0]]], [[[6.0, 5.0, 6.0], [7.0, 3.0, 5.0], [6.0, 1.0, 3.0]], [[1.0, 6.0, 3.0], 
[4.0, 7.0, 2.0], [3.0, 1.0, 8.0]], [[2.0, 4.0, 8.0], [1.0, 5.0, 3.0], [4.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 1.0], [1.0, 5.0, 1.0], [4.0, 1.0, 3.0]], [[2.0, 3.0, 7.0], [5.0, 8.0, 8.0], [8.0, 
7.0, 6.0]], [[6.0, 2.0, 6.0], [1.0, 1.0, 5.0], [5.0, 7.0, 8.0]]], [[[3.0, 7.0, 2.0], [7.0, 7.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 
1.0, 5.0], [6.0, 6.0, 4.0], [1.0, 8.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 6.0], [3.0, 4.0, 2.0]]], [[[5.0, 3.0, 7.0], [7.0, 
7.0, 5.0], [1.0, 5.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 3.0, 5.0], [6.0, 7.0, 6.0]], [[8.0, 8.0, 8.0], [3.0, 8.0, 5.0], [8.0, 5.0, 
1.0]]], [[[3.0, 6.0, 8.0], [1.0, 1.0, 5.0], [6.0, 6.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 4.0, 5.0], [4.0, 3.0, 3.0]], [[2.0, 2.0, 
8.0], [8.0, 4.0, 6.0], [5.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 3.0], [1.0, 2.0, 1.0], [1.0, 3.0, 2.0]], 
[[1.0, 2.0, 7.0], [2.0, 4.0, 2.0], [4.0, 3.0, 6.0]], [[7.0, 3.0, 8.0], [3.0, 4.0, 7.0], [7.0, 5.0, 2.0]]], [[[5.0, 2.0, 8.0], 
[3.0, 2.0, 4.0], [7.0, 4.0, 1.0]], [[5.0, 7.0, 6.0], [5.0, 5.0, 1.0], [4.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [1.0, 4.0, 3.0], 
[3.0, 2.0, 7.0]]], [[[7.0, 2.0, 6.0], [1.0, 8.0, 8.0], [3.0, 4.0, 3.0]], [[3.0, 5.0, 3.0], [3.0, 3.0, 5.0], [3.0, 8.0, 6.0]], 
[[1.0, 3.0, 2.0], [7.0, 3.0, 8.0], [3.0, 7.0, 6.0]]], [[[5.0, 8.0, 8.0], [7.0, 4.0, 4.0], [7.0, 1.0, 5.0]], [[7.0, 3.0, 2.0], 
[7.0, 3.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 4.0, 4.0], [1.0, 7.0, 6.0], [6.0, 4.0, 2.0]]]])
      val=numpy.array([[[[2.0], [4.0], [1.0]]], [[[5.0], [5.0], [7.0]]]])
      arg[1:3,1:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 1.0], [1.0, 5.0, 1.0], [4.0, 1.0, 3.0]], [[2.0, 3.0, 7.0], [5.0, 8.0, 8.0], [8.0, 
7.0, 6.0]], [[6.0, 2.0, 6.0], [1.0, 1.0, 5.0], [5.0, 7.0, 8.0]]], [[[3.0, 7.0, 2.0], [7.0, 7.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 
2.0, 5.0], [6.0, 4.0, 4.0], [1.0, 1.0, 6.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 6.0], [3.0, 4.0, 2.0]]], [[[5.0, 3.0, 7.0], [7.0, 
7.0, 5.0], [1.0, 5.0, 3.0]], [[7.0, 5.0, 7.0], [4.0, 5.0, 5.0], [6.0, 7.0, 6.0]], [[8.0, 8.0, 8.0], [3.0, 8.0, 5.0], [8.0, 5.0, 
1.0]]], [[[3.0, 6.0, 8.0], [1.0, 1.0, 5.0], [6.0, 6.0, 8.0]], [[5.0, 7.0, 8.0], [5.0, 4.0, 5.0], [4.0, 3.0, 3.0]], [[2.0, 2.0, 
8.0], [8.0, 4.0, 6.0], [5.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [1.0, 2.0, 1.0], [1.0, 3.0, 2.0]], 
[[1.0, 2.0, 7.0], [2.0, 4.0, 2.0], [4.0, 3.0, 6.0]], [[7.0, 3.0, 8.0], [3.0, 4.0, 7.0], [7.0, 5.0, 2.0]]], [[[5.0, 2.0, 8.0], 
[3.0, 2.0, 4.0], [7.0, 4.0, 1.0]], [[5.0, 2.0, 6.0], [5.0, 4.0, 1.0], [4.0, 1.0, 3.0]], [[7.0, 3.0, 5.0], [1.0, 4.0, 3.0], 
[3.0, 2.0, 7.0]]], [[[7.0, 2.0, 6.0], [1.0, 8.0, 8.0], [3.0, 4.0, 3.0]], [[3.0, 5.0, 3.0], [3.0, 5.0, 5.0], [3.0, 7.0, 6.0]], 
[[1.0, 3.0, 2.0], [7.0, 3.0, 8.0], [3.0, 7.0, 6.0]]], [[[5.0, 8.0, 8.0], [7.0, 4.0, 4.0], [7.0, 1.0, 5.0]], [[7.0, 3.0, 2.0], 
[7.0, 3.0, 3.0], [8.0, 1.0, 3.0]], [[7.0, 4.0, 4.0], [1.0, 7.0, 6.0], [6.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 5.0], [6.0, 6.0, 8.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 2.0], [5.0, 5.0, 1.0], [3.0, 
2.0, 5.0]], [[8.0, 7.0, 5.0], [8.0, 8.0, 1.0], [6.0, 2.0, 7.0]]], [[[6.0, 7.0, 5.0], [2.0, 4.0, 4.0], [4.0, 2.0, 2.0]], [[1.0, 
1.0, 3.0], [7.0, 5.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 4.0, 7.0], [5.0, 8.0, 1.0], [4.0, 3.0, 7.0]]], [[[3.0, 4.0, 1.0], [3.0, 
3.0, 2.0], [7.0, 5.0, 7.0]], [[8.0, 2.0, 6.0], [2.0, 8.0, 5.0], [1.0, 3.0, 2.0]], [[5.0, 2.0, 7.0], [6.0, 2.0, 4.0], [6.0, 2.0, 
8.0]]], [[[6.0, 4.0, 5.0], [3.0, 8.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 1.0, 1.0], [1.0, 2.0, 7.0], [5.0, 6.0, 2.0]], [[5.0, 1.0, 
5.0], [4.0, 2.0, 8.0], [6.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 6.0], [3.0, 8.0, 8.0], [1.0, 3.0, 5.0]], 
[[8.0, 1.0, 4.0], [6.0, 7.0, 1.0], [7.0, 7.0, 6.0]], [[6.0, 7.0, 5.0], [4.0, 7.0, 4.0], [2.0, 2.0, 1.0]]], [[[4.0, 8.0, 7.0], 
[6.0, 7.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 3.0, 8.0], [4.0, 5.0, 6.0], [4.0, 6.0, 6.0]], [[3.0, 1.0, 6.0], [7.0, 6.0, 5.0], 
[7.0, 8.0, 1.0]]], [[[8.0, 1.0, 2.0], [4.0, 4.0, 7.0], [7.0, 1.0, 6.0]], [[3.0, 1.0, 5.0], [6.0, 2.0, 8.0], [5.0, 7.0, 3.0]], 
[[5.0, 4.0, 1.0], [5.0, 5.0, 8.0], [7.0, 3.0, 5.0]]], [[[1.0, 3.0, 4.0], [1.0, 5.0, 7.0], [2.0, 5.0, 7.0]], [[5.0, 8.0, 6.0], 
[7.0, 6.0, 6.0], [3.0, 1.0, 4.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 5.0], [3.0, 7.0, 3.0]]]])
      val=Data(numpy.array([[[[7.0], [8.0], [4.0]]], [[[8.0], [7.0], [7.0]]]]),self.functionspace)
      arg[1:3,1:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 5.0], [6.0, 6.0, 8.0], [4.0, 5.0, 3.0]], [[6.0, 4.0, 2.0], [5.0, 5.0, 1.0], [3.0, 
2.0, 5.0]], [[8.0, 7.0, 5.0], [8.0, 8.0, 1.0], [6.0, 2.0, 7.0]]], [[[6.0, 7.0, 5.0], [2.0, 4.0, 4.0], [4.0, 2.0, 2.0]], [[1.0, 
7.0, 3.0], [7.0, 8.0, 8.0], [6.0, 4.0, 1.0]], [[3.0, 4.0, 7.0], [5.0, 8.0, 1.0], [4.0, 3.0, 7.0]]], [[[3.0, 4.0, 1.0], [3.0, 
3.0, 2.0], [7.0, 5.0, 7.0]], [[8.0, 8.0, 6.0], [2.0, 7.0, 5.0], [1.0, 7.0, 2.0]], [[5.0, 2.0, 7.0], [6.0, 2.0, 4.0], [6.0, 2.0, 
8.0]]], [[[6.0, 4.0, 5.0], [3.0, 8.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 1.0, 1.0], [1.0, 2.0, 7.0], [5.0, 6.0, 2.0]], [[5.0, 1.0, 
5.0], [4.0, 2.0, 8.0], [6.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 6.0], [3.0, 8.0, 8.0], [1.0, 3.0, 5.0]], 
[[8.0, 1.0, 4.0], [6.0, 7.0, 1.0], [7.0, 7.0, 6.0]], [[6.0, 7.0, 5.0], [4.0, 7.0, 4.0], [2.0, 2.0, 1.0]]], [[[4.0, 8.0, 7.0], 
[6.0, 7.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 7.0, 8.0], [4.0, 8.0, 6.0], [4.0, 4.0, 6.0]], [[3.0, 1.0, 6.0], [7.0, 6.0, 5.0], 
[7.0, 8.0, 1.0]]], [[[8.0, 1.0, 2.0], [4.0, 4.0, 7.0], [7.0, 1.0, 6.0]], [[3.0, 8.0, 5.0], [6.0, 7.0, 8.0], [5.0, 7.0, 3.0]], 
[[5.0, 4.0, 1.0], [5.0, 5.0, 8.0], [7.0, 3.0, 5.0]]], [[[1.0, 3.0, 4.0], [1.0, 5.0, 7.0], [2.0, 5.0, 7.0]], [[5.0, 8.0, 6.0], 
[7.0, 6.0, 6.0], [3.0, 1.0, 4.0]], [[6.0, 3.0, 3.0], [6.0, 5.0, 5.0], [3.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRCR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 5.0], [6.0, 7.0, 4.0], [2.0, 2.0, 5.0]], [[3.0, 1.0, 2.0], [6.0, 7.0, 4.0], [3.0, 
3.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 3.0, 3.0], [3.0, 1.0, 6.0]]], [[[4.0, 6.0, 2.0], [3.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[4.0, 
4.0, 5.0], [7.0, 5.0, 5.0], [6.0, 4.0, 4.0]], [[4.0, 1.0, 5.0], [8.0, 3.0, 3.0], [3.0, 1.0, 7.0]]], [[[6.0, 7.0, 8.0], [6.0, 
5.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 8.0, 7.0], [2.0, 5.0, 3.0], [6.0, 4.0, 4.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 7.0], [7.0, 4.0, 
4.0]]], [[[6.0, 3.0, 1.0], [5.0, 2.0, 5.0], [3.0, 7.0, 1.0]], [[5.0, 5.0, 5.0], [5.0, 5.0, 2.0], [4.0, 1.0, 1.0]], [[6.0, 1.0, 
4.0], [5.0, 8.0, 8.0], [7.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 1.0], [8.0, 2.0, 5.0], [3.0, 7.0, 1.0]], 
[[8.0, 1.0, 7.0], [5.0, 6.0, 7.0], [4.0, 2.0, 5.0]], [[1.0, 5.0, 7.0], [3.0, 6.0, 4.0], [8.0, 7.0, 3.0]]], [[[8.0, 6.0, 7.0], 
[8.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[8.0, 2.0, 7.0], [7.0, 3.0, 4.0], [2.0, 2.0, 2.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 2.0], 
[8.0, 2.0, 2.0]]], [[[5.0, 5.0, 2.0], [1.0, 5.0, 1.0], [4.0, 5.0, 1.0]], [[2.0, 6.0, 6.0], [8.0, 3.0, 3.0], [6.0, 2.0, 6.0]], 
[[6.0, 5.0, 6.0], [5.0, 6.0, 1.0], [4.0, 1.0, 2.0]]], [[[4.0, 6.0, 6.0], [3.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[6.0, 2.0, 3.0], 
[1.0, 8.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 4.0, 1.0], [5.0, 4.0, 2.0], [5.0, 8.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0], [7.0], [4.0]]], [[[7.0], [6.0], [2.0]]]])+(1.-msk_val)*numpy.array([[[[4.0], [1.0], 
[2.0]]], [[[4.0], [6.0], [4.0]]]])
      arg[1:3,1:2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 5.0], [6.0, 7.0, 4.0], [2.0, 2.0, 5.0]], [[3.0, 1.0, 2.0], [6.0, 7.0, 4.0], [3.0, 
3.0, 1.0]], [[5.0, 4.0, 4.0], [3.0, 3.0, 3.0], [3.0, 1.0, 6.0]]], [[[4.0, 6.0, 2.0], [3.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[4.0, 
3.0, 5.0], [7.0, 7.0, 5.0], [6.0, 4.0, 4.0]], [[4.0, 1.0, 5.0], [8.0, 3.0, 3.0], [3.0, 1.0, 7.0]]], [[[6.0, 7.0, 8.0], [6.0, 
5.0, 6.0], [3.0, 3.0, 2.0]], [[2.0, 7.0, 7.0], [2.0, 6.0, 3.0], [6.0, 2.0, 4.0]], [[8.0, 7.0, 8.0], [3.0, 7.0, 7.0], [7.0, 4.0, 
4.0]]], [[[6.0, 3.0, 1.0], [5.0, 2.0, 5.0], [3.0, 7.0, 1.0]], [[5.0, 5.0, 5.0], [5.0, 5.0, 2.0], [4.0, 1.0, 1.0]], [[6.0, 1.0, 
4.0], [5.0, 8.0, 8.0], [7.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 1.0], [8.0, 2.0, 5.0], [3.0, 7.0, 1.0]], 
[[8.0, 1.0, 7.0], [5.0, 6.0, 7.0], [4.0, 2.0, 5.0]], [[1.0, 5.0, 7.0], [3.0, 6.0, 4.0], [8.0, 7.0, 3.0]]], [[[8.0, 6.0, 7.0], 
[8.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[8.0, 4.0, 7.0], [7.0, 1.0, 4.0], [2.0, 2.0, 2.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 2.0], 
[8.0, 2.0, 2.0]]], [[[5.0, 5.0, 2.0], [1.0, 5.0, 1.0], [4.0, 5.0, 1.0]], [[2.0, 4.0, 6.0], [8.0, 6.0, 3.0], [6.0, 4.0, 6.0]], 
[[6.0, 5.0, 6.0], [5.0, 6.0, 1.0], [4.0, 1.0, 2.0]]], [[[4.0, 6.0, 6.0], [3.0, 4.0, 4.0], [5.0, 8.0, 5.0]], [[6.0, 2.0, 3.0], 
[1.0, 8.0, 5.0], [5.0, 4.0, 6.0]], [[5.0, 4.0, 1.0], [5.0, 4.0, 2.0], [5.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 7.0, 4.0], [4.0, 7.0, 7.0]], [[5.0, 2.0, 2.0], [7.0, 4.0, 1.0], [4.0, 
5.0, 3.0]], [[4.0, 8.0, 2.0], [4.0, 8.0, 1.0], [5.0, 7.0, 4.0]]], [[[2.0, 1.0, 1.0], [7.0, 5.0, 4.0], [6.0, 6.0, 3.0]], [[2.0, 
5.0, 6.0], [2.0, 7.0, 6.0], [2.0, 8.0, 3.0]], [[5.0, 8.0, 8.0], [8.0, 5.0, 1.0], [1.0, 5.0, 6.0]]], [[[2.0, 3.0, 1.0], [1.0, 
4.0, 8.0], [1.0, 3.0, 2.0]], [[7.0, 2.0, 2.0], [7.0, 1.0, 3.0], [2.0, 8.0, 5.0]], [[1.0, 8.0, 6.0], [1.0, 2.0, 7.0], [3.0, 1.0, 
7.0]]], [[[7.0, 8.0, 5.0], [3.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 1.0, 5.0], [8.0, 6.0, 7.0]], [[3.0, 4.0, 
8.0], [1.0, 5.0, 7.0], [5.0, 7.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 4.0], [5.0, 8.0, 1.0], [3.0, 2.0, 5.0]], 
[[3.0, 7.0, 8.0], [3.0, 5.0, 2.0], [3.0, 1.0, 8.0]], [[4.0, 1.0, 6.0], [5.0, 4.0, 5.0], [4.0, 2.0, 6.0]]], [[[1.0, 4.0, 6.0], 
[5.0, 1.0, 6.0], [8.0, 8.0, 6.0]], [[7.0, 2.0, 5.0], [6.0, 3.0, 6.0], [2.0, 3.0, 8.0]], [[2.0, 1.0, 4.0], [3.0, 1.0, 4.0], 
[8.0, 4.0, 5.0]]], [[[4.0, 2.0, 6.0], [1.0, 6.0, 4.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [4.0, 4.0, 7.0], [3.0, 7.0, 7.0]], 
[[8.0, 2.0, 2.0], [3.0, 8.0, 7.0], [4.0, 8.0, 2.0]]], [[[5.0, 5.0, 1.0], [4.0, 1.0, 7.0], [2.0, 5.0, 2.0]], [[5.0, 6.0, 1.0], 
[2.0, 8.0, 8.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 7.0], [6.0, 5.0, 6.0], [1.0, 1.0, 6.0]]]])
      val=numpy.array([[[1.0, 5.0, 3.0]], [[8.0, 8.0, 3.0]], [[3.0, 8.0, 5.0]]])
      arg[:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 7.0, 4.0], [4.0, 7.0, 7.0]], [[5.0, 2.0, 2.0], [7.0, 4.0, 1.0], [4.0, 
5.0, 3.0]], [[4.0, 8.0, 2.0], [1.0, 5.0, 3.0], [5.0, 7.0, 4.0]]], [[[2.0, 1.0, 1.0], [7.0, 5.0, 4.0], [6.0, 6.0, 3.0]], [[2.0, 
5.0, 6.0], [2.0, 7.0, 6.0], [2.0, 8.0, 3.0]], [[5.0, 8.0, 8.0], [8.0, 8.0, 3.0], [1.0, 5.0, 6.0]]], [[[2.0, 3.0, 1.0], [1.0, 
4.0, 8.0], [1.0, 3.0, 2.0]], [[7.0, 2.0, 2.0], [7.0, 1.0, 3.0], [2.0, 8.0, 5.0]], [[1.0, 8.0, 6.0], [3.0, 8.0, 5.0], [3.0, 1.0, 
7.0]]], [[[7.0, 8.0, 5.0], [3.0, 8.0, 5.0], [2.0, 5.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 1.0, 5.0], [8.0, 6.0, 7.0]], [[3.0, 4.0, 
8.0], [1.0, 5.0, 7.0], [5.0, 7.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 4.0], [5.0, 8.0, 1.0], [3.0, 2.0, 5.0]], 
[[3.0, 7.0, 8.0], [3.0, 5.0, 2.0], [3.0, 1.0, 8.0]], [[4.0, 1.0, 6.0], [1.0, 5.0, 3.0], [4.0, 2.0, 6.0]]], [[[1.0, 4.0, 6.0], 
[5.0, 1.0, 6.0], [8.0, 8.0, 6.0]], [[7.0, 2.0, 5.0], [6.0, 3.0, 6.0], [2.0, 3.0, 8.0]], [[2.0, 1.0, 4.0], [8.0, 8.0, 3.0], 
[8.0, 4.0, 5.0]]], [[[4.0, 2.0, 6.0], [1.0, 6.0, 4.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 8.0], [4.0, 4.0, 7.0], [3.0, 7.0, 7.0]], 
[[8.0, 2.0, 2.0], [3.0, 8.0, 5.0], [4.0, 8.0, 2.0]]], [[[5.0, 5.0, 1.0], [4.0, 1.0, 7.0], [2.0, 5.0, 2.0]], [[5.0, 6.0, 1.0], 
[2.0, 8.0, 8.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 7.0], [6.0, 5.0, 6.0], [1.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 1.0], [7.0, 4.0, 6.0], [5.0, 8.0, 6.0]], [[2.0, 7.0, 4.0], [1.0, 6.0, 1.0], [8.0, 
4.0, 8.0]], [[5.0, 2.0, 5.0], [4.0, 8.0, 3.0], [3.0, 7.0, 2.0]]], [[[3.0, 3.0, 7.0], [4.0, 8.0, 3.0], [5.0, 2.0, 5.0]], [[2.0, 
8.0, 5.0], [1.0, 2.0, 8.0], [3.0, 7.0, 3.0]], [[1.0, 4.0, 2.0], [1.0, 6.0, 7.0], [7.0, 1.0, 2.0]]], [[[7.0, 2.0, 2.0], [5.0, 
1.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 1.0, 6.0], [6.0, 5.0, 8.0], [1.0, 3.0, 4.0]], [[3.0, 6.0, 3.0], [3.0, 2.0, 8.0], [4.0, 5.0, 
5.0]]], [[[5.0, 2.0, 1.0], [4.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[2.0, 1.0, 4.0], [4.0, 2.0, 5.0], [7.0, 2.0, 6.0]], [[4.0, 4.0, 
1.0], [3.0, 4.0, 6.0], [3.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 6.0], [3.0, 6.0, 1.0], [1.0, 3.0, 5.0]], 
[[8.0, 4.0, 6.0], [7.0, 3.0, 4.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 8.0], [7.0, 7.0, 6.0], [3.0, 5.0, 5.0]]], [[[4.0, 8.0, 7.0], 
[2.0, 1.0, 3.0], [8.0, 1.0, 2.0]], [[2.0, 4.0, 2.0], [4.0, 5.0, 8.0], [5.0, 5.0, 2.0]], [[7.0, 1.0, 8.0], [5.0, 7.0, 2.0], 
[6.0, 8.0, 2.0]]], [[[5.0, 4.0, 6.0], [3.0, 7.0, 4.0], [4.0, 7.0, 8.0]], [[7.0, 8.0, 8.0], [6.0, 6.0, 7.0], [5.0, 7.0, 3.0]], 
[[1.0, 5.0, 1.0], [6.0, 7.0, 2.0], [5.0, 5.0, 4.0]]], [[[6.0, 7.0, 5.0], [7.0, 5.0, 1.0], [2.0, 7.0, 8.0]], [[7.0, 2.0, 6.0], 
[5.0, 6.0, 2.0], [3.0, 2.0, 8.0]], [[7.0, 4.0, 1.0], [6.0, 8.0, 1.0], [3.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[5.0, 7.0, 6.0]], [[8.0, 6.0, 2.0]], [[4.0, 7.0, 5.0]]]),self.functionspace)
      arg[:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 1.0], [7.0, 4.0, 6.0], [5.0, 8.0, 6.0]], [[2.0, 7.0, 4.0], [1.0, 6.0, 1.0], [8.0, 
4.0, 8.0]], [[5.0, 2.0, 5.0], [5.0, 7.0, 6.0], [3.0, 7.0, 2.0]]], [[[3.0, 3.0, 7.0], [4.0, 8.0, 3.0], [5.0, 2.0, 5.0]], [[2.0, 
8.0, 5.0], [1.0, 2.0, 8.0], [3.0, 7.0, 3.0]], [[1.0, 4.0, 2.0], [8.0, 6.0, 2.0], [7.0, 1.0, 2.0]]], [[[7.0, 2.0, 2.0], [5.0, 
1.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 1.0, 6.0], [6.0, 5.0, 8.0], [1.0, 3.0, 4.0]], [[3.0, 6.0, 3.0], [4.0, 7.0, 5.0], [4.0, 5.0, 
5.0]]], [[[5.0, 2.0, 1.0], [4.0, 4.0, 8.0], [6.0, 3.0, 1.0]], [[2.0, 1.0, 4.0], [4.0, 2.0, 5.0], [7.0, 2.0, 6.0]], [[4.0, 4.0, 
1.0], [3.0, 4.0, 6.0], [3.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 6.0], [3.0, 6.0, 1.0], [1.0, 3.0, 5.0]], 
[[8.0, 4.0, 6.0], [7.0, 3.0, 4.0], [4.0, 1.0, 5.0]], [[8.0, 1.0, 8.0], [5.0, 7.0, 6.0], [3.0, 5.0, 5.0]]], [[[4.0, 8.0, 7.0], 
[2.0, 1.0, 3.0], [8.0, 1.0, 2.0]], [[2.0, 4.0, 2.0], [4.0, 5.0, 8.0], [5.0, 5.0, 2.0]], [[7.0, 1.0, 8.0], [8.0, 6.0, 2.0], 
[6.0, 8.0, 2.0]]], [[[5.0, 4.0, 6.0], [3.0, 7.0, 4.0], [4.0, 7.0, 8.0]], [[7.0, 8.0, 8.0], [6.0, 6.0, 7.0], [5.0, 7.0, 3.0]], 
[[1.0, 5.0, 1.0], [4.0, 7.0, 5.0], [5.0, 5.0, 4.0]]], [[[6.0, 7.0, 5.0], [7.0, 5.0, 1.0], [2.0, 7.0, 8.0]], [[7.0, 2.0, 6.0], 
[5.0, 6.0, 2.0], [3.0, 2.0, 8.0]], [[7.0, 4.0, 1.0], [6.0, 8.0, 1.0], [3.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 5.0], [4.0, 7.0, 5.0], [3.0, 8.0, 5.0]], [[8.0, 8.0, 2.0], [7.0, 2.0, 7.0], [4.0, 
1.0, 2.0]], [[5.0, 5.0, 1.0], [5.0, 3.0, 5.0], [7.0, 1.0, 3.0]]], [[[7.0, 4.0, 1.0], [2.0, 1.0, 5.0], [3.0, 4.0, 2.0]], [[5.0, 
6.0, 7.0], [3.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], [8.0, 6.0, 5.0], [3.0, 5.0, 7.0]]], [[[1.0, 4.0, 6.0], [6.0, 
4.0, 4.0], [8.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [3.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[2.0, 5.0, 5.0], [7.0, 4.0, 1.0], [8.0, 5.0, 
8.0]]], [[[1.0, 3.0, 3.0], [8.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[6.0, 8.0, 3.0], [3.0, 8.0, 7.0], [7.0, 8.0, 8.0]], [[6.0, 2.0, 
6.0], [7.0, 2.0, 4.0], [6.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 1.0], [6.0, 4.0, 2.0], [5.0, 5.0, 6.0]], 
[[5.0, 1.0, 7.0], [7.0, 4.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 4.0, 8.0], [2.0, 8.0, 8.0], [8.0, 2.0, 4.0]]], [[[6.0, 3.0, 6.0], 
[5.0, 3.0, 6.0], [6.0, 6.0, 1.0]], [[1.0, 7.0, 7.0], [8.0, 2.0, 6.0], [2.0, 2.0, 2.0]], [[4.0, 7.0, 6.0], [1.0, 5.0, 2.0], 
[2.0, 6.0, 3.0]]], [[[6.0, 1.0, 1.0], [1.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 1.0], [4.0, 6.0, 6.0]], 
[[6.0, 8.0, 5.0], [2.0, 8.0, 1.0], [1.0, 7.0, 6.0]]], [[[6.0, 8.0, 5.0], [4.0, 4.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 4.0, 4.0], 
[5.0, 7.0, 6.0], [5.0, 1.0, 3.0]], [[1.0, 8.0, 8.0], [1.0, 2.0, 8.0], [2.0, 2.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 4.0, 7.0]], [[3.0, 7.0, 4.0]], [[6.0, 3.0, 7.0]]])+(1.-msk_val)*numpy.array([[[7.0, 7.0, 
4.0]], [[5.0, 4.0, 6.0]], [[5.0, 7.0, 5.0]]])
      arg[:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 5.0], [4.0, 7.0, 5.0], [3.0, 8.0, 5.0]], [[8.0, 8.0, 2.0], [7.0, 2.0, 7.0], [4.0, 
1.0, 2.0]], [[5.0, 5.0, 1.0], [3.0, 4.0, 7.0], [7.0, 1.0, 3.0]]], [[[7.0, 4.0, 1.0], [2.0, 1.0, 5.0], [3.0, 4.0, 2.0]], [[5.0, 
6.0, 7.0], [3.0, 1.0, 4.0], [8.0, 3.0, 4.0]], [[5.0, 7.0, 4.0], [3.0, 7.0, 4.0], [3.0, 5.0, 7.0]]], [[[1.0, 4.0, 6.0], [6.0, 
4.0, 4.0], [8.0, 4.0, 2.0]], [[8.0, 8.0, 1.0], [3.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[2.0, 5.0, 5.0], [6.0, 3.0, 7.0], [8.0, 5.0, 
8.0]]], [[[1.0, 3.0, 3.0], [8.0, 6.0, 2.0], [2.0, 5.0, 8.0]], [[6.0, 8.0, 3.0], [3.0, 8.0, 7.0], [7.0, 8.0, 8.0]], [[6.0, 2.0, 
6.0], [7.0, 2.0, 4.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [6.0, 4.0, 2.0], [5.0, 5.0, 6.0]], 
[[5.0, 1.0, 7.0], [7.0, 4.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 4.0, 8.0], [7.0, 7.0, 4.0], [8.0, 2.0, 4.0]]], [[[6.0, 3.0, 6.0], 
[5.0, 3.0, 6.0], [6.0, 6.0, 1.0]], [[1.0, 7.0, 7.0], [8.0, 2.0, 6.0], [2.0, 2.0, 2.0]], [[4.0, 7.0, 6.0], [5.0, 4.0, 6.0], 
[2.0, 6.0, 3.0]]], [[[6.0, 1.0, 1.0], [1.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 1.0], [4.0, 6.0, 6.0]], 
[[6.0, 8.0, 5.0], [5.0, 7.0, 5.0], [1.0, 7.0, 6.0]]], [[[6.0, 8.0, 5.0], [4.0, 4.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 4.0, 4.0], 
[5.0, 7.0, 6.0], [5.0, 1.0, 3.0]], [[1.0, 8.0, 8.0], [1.0, 2.0, 8.0], [2.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 5.0], [4.0, 7.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 6.0, 5.0], [5.0, 3.0, 6.0], [1.0, 
2.0, 3.0]], [[1.0, 6.0, 5.0], [3.0, 8.0, 6.0], [8.0, 1.0, 8.0]]], [[[5.0, 5.0, 8.0], [1.0, 1.0, 6.0], [7.0, 8.0, 7.0]], [[4.0, 
1.0, 6.0], [5.0, 8.0, 6.0], [5.0, 6.0, 4.0]], [[5.0, 5.0, 4.0], [6.0, 2.0, 8.0], [2.0, 4.0, 5.0]]], [[[1.0, 3.0, 1.0], [4.0, 
2.0, 8.0], [2.0, 1.0, 2.0]], [[6.0, 7.0, 5.0], [3.0, 8.0, 2.0], [6.0, 4.0, 1.0]], [[6.0, 3.0, 7.0], [5.0, 7.0, 4.0], [6.0, 2.0, 
8.0]]], [[[1.0, 2.0, 3.0], [2.0, 4.0, 1.0], [2.0, 3.0, 1.0]], [[5.0, 2.0, 4.0], [7.0, 2.0, 4.0], [1.0, 4.0, 8.0]], [[1.0, 5.0, 
5.0], [1.0, 7.0, 7.0], [1.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 2.0], [6.0, 5.0, 7.0], [3.0, 7.0, 5.0]], 
[[3.0, 3.0, 8.0], [1.0, 5.0, 5.0], [6.0, 6.0, 3.0]], [[6.0, 6.0, 5.0], [3.0, 5.0, 1.0], [8.0, 5.0, 6.0]]], [[[2.0, 2.0, 8.0], 
[5.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[3.0, 1.0, 4.0], [1.0, 3.0, 8.0], [2.0, 6.0, 7.0]], [[5.0, 5.0, 4.0], [8.0, 7.0, 8.0], 
[4.0, 3.0, 7.0]]], [[[6.0, 3.0, 5.0], [7.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[8.0, 4.0, 8.0], [3.0, 7.0, 4.0], [4.0, 8.0, 5.0]], 
[[1.0, 6.0, 8.0], [5.0, 3.0, 1.0], [7.0, 3.0, 6.0]]], [[[8.0, 4.0, 2.0], [5.0, 2.0, 6.0], [5.0, 8.0, 6.0]], [[7.0, 5.0, 1.0], 
[7.0, 2.0, 4.0], [2.0, 8.0, 5.0]], [[5.0, 5.0, 7.0], [3.0, 4.0, 8.0], [5.0, 2.0, 1.0]]]])
      val=numpy.array([2.0, 5.0, 7.0])
      arg[2,1,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 5.0], [4.0, 7.0, 4.0], [5.0, 5.0, 3.0]], [[5.0, 6.0, 5.0], [5.0, 3.0, 6.0], [1.0, 
2.0, 3.0]], [[1.0, 6.0, 5.0], [3.0, 8.0, 6.0], [8.0, 1.0, 8.0]]], [[[5.0, 5.0, 8.0], [1.0, 1.0, 6.0], [7.0, 8.0, 7.0]], [[4.0, 
1.0, 6.0], [5.0, 8.0, 6.0], [5.0, 6.0, 4.0]], [[5.0, 5.0, 4.0], [6.0, 2.0, 8.0], [2.0, 4.0, 5.0]]], [[[1.0, 3.0, 1.0], [4.0, 
2.0, 8.0], [2.0, 1.0, 2.0]], [[6.0, 2.0, 5.0], [3.0, 5.0, 2.0], [6.0, 7.0, 1.0]], [[6.0, 3.0, 7.0], [5.0, 7.0, 4.0], [6.0, 2.0, 
8.0]]], [[[1.0, 2.0, 3.0], [2.0, 4.0, 1.0], [2.0, 3.0, 1.0]], [[5.0, 2.0, 4.0], [7.0, 2.0, 4.0], [1.0, 4.0, 8.0]], [[1.0, 5.0, 
5.0], [1.0, 7.0, 7.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 2.0], [6.0, 5.0, 7.0], [3.0, 7.0, 5.0]], 
[[3.0, 3.0, 8.0], [1.0, 5.0, 5.0], [6.0, 6.0, 3.0]], [[6.0, 6.0, 5.0], [3.0, 5.0, 1.0], [8.0, 5.0, 6.0]]], [[[2.0, 2.0, 8.0], 
[5.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[3.0, 1.0, 4.0], [1.0, 3.0, 8.0], [2.0, 6.0, 7.0]], [[5.0, 5.0, 4.0], [8.0, 7.0, 8.0], 
[4.0, 3.0, 7.0]]], [[[6.0, 3.0, 5.0], [7.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[8.0, 2.0, 8.0], [3.0, 5.0, 4.0], [4.0, 7.0, 5.0]], 
[[1.0, 6.0, 8.0], [5.0, 3.0, 1.0], [7.0, 3.0, 6.0]]], [[[8.0, 4.0, 2.0], [5.0, 2.0, 6.0], [5.0, 8.0, 6.0]], [[7.0, 5.0, 1.0], 
[7.0, 2.0, 4.0], [2.0, 8.0, 5.0]], [[5.0, 5.0, 7.0], [3.0, 4.0, 8.0], [5.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 3.0], [1.0, 8.0, 6.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 3.0], [1.0, 5.0, 7.0], [6.0, 
8.0, 6.0]], [[5.0, 1.0, 2.0], [7.0, 3.0, 2.0], [7.0, 5.0, 5.0]]], [[[1.0, 4.0, 8.0], [8.0, 1.0, 5.0], [5.0, 4.0, 7.0]], [[5.0, 
5.0, 3.0], [2.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [2.0, 3.0, 5.0], [1.0, 2.0, 7.0]]], [[[4.0, 3.0, 8.0], [3.0, 
8.0, 6.0], [5.0, 1.0, 2.0]], [[1.0, 2.0, 5.0], [6.0, 7.0, 3.0], [7.0, 2.0, 4.0]], [[8.0, 1.0, 8.0], [3.0, 2.0, 1.0], [1.0, 2.0, 
5.0]]], [[[3.0, 6.0, 3.0], [4.0, 4.0, 3.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 4.0], [5.0, 8.0, 2.0], [7.0, 6.0, 5.0]], [[8.0, 1.0, 
5.0], [7.0, 4.0, 3.0], [6.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 6.0], [7.0, 3.0, 4.0], [4.0, 8.0, 4.0]], 
[[4.0, 5.0, 6.0], [4.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[1.0, 5.0, 7.0], [1.0, 5.0, 8.0], [5.0, 2.0, 5.0]]], [[[6.0, 8.0, 1.0], 
[7.0, 3.0, 2.0], [5.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 8.0, 7.0], [1.0, 1.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 3.0, 5.0], 
[3.0, 6.0, 2.0]]], [[[3.0, 2.0, 2.0], [4.0, 4.0, 4.0], [5.0, 7.0, 2.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 3.0], [8.0, 6.0, 2.0]], 
[[7.0, 2.0, 3.0], [1.0, 2.0, 6.0], [1.0, 3.0, 7.0]]], [[[1.0, 5.0, 2.0], [5.0, 7.0, 3.0], [4.0, 8.0, 5.0]], [[1.0, 2.0, 7.0], 
[2.0, 7.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 1.0, 6.0], [3.0, 8.0, 1.0], [2.0, 7.0, 6.0]]]])
      val=Data(numpy.array([3.0, 6.0, 6.0]),self.functionspace)
      arg[2,1,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 3.0], [1.0, 8.0, 6.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 3.0], [1.0, 5.0, 7.0], [6.0, 
8.0, 6.0]], [[5.0, 1.0, 2.0], [7.0, 3.0, 2.0], [7.0, 5.0, 5.0]]], [[[1.0, 4.0, 8.0], [8.0, 1.0, 5.0], [5.0, 4.0, 7.0]], [[5.0, 
5.0, 3.0], [2.0, 2.0, 6.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [2.0, 3.0, 5.0], [1.0, 2.0, 7.0]]], [[[4.0, 3.0, 8.0], [3.0, 
8.0, 6.0], [5.0, 1.0, 2.0]], [[1.0, 3.0, 5.0], [6.0, 6.0, 3.0], [7.0, 6.0, 4.0]], [[8.0, 1.0, 8.0], [3.0, 2.0, 1.0], [1.0, 2.0, 
5.0]]], [[[3.0, 6.0, 3.0], [4.0, 4.0, 3.0], [3.0, 5.0, 3.0]], [[7.0, 1.0, 4.0], [5.0, 8.0, 2.0], [7.0, 6.0, 5.0]], [[8.0, 1.0, 
5.0], [7.0, 4.0, 3.0], [6.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [7.0, 3.0, 4.0], [4.0, 8.0, 4.0]], 
[[4.0, 5.0, 6.0], [4.0, 6.0, 6.0], [3.0, 4.0, 2.0]], [[1.0, 5.0, 7.0], [1.0, 5.0, 8.0], [5.0, 2.0, 5.0]]], [[[6.0, 8.0, 1.0], 
[7.0, 3.0, 2.0], [5.0, 3.0, 7.0]], [[8.0, 6.0, 7.0], [2.0, 8.0, 7.0], [1.0, 1.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 3.0, 5.0], 
[3.0, 6.0, 2.0]]], [[[3.0, 2.0, 2.0], [4.0, 4.0, 4.0], [5.0, 7.0, 2.0]], [[5.0, 3.0, 2.0], [6.0, 6.0, 3.0], [8.0, 6.0, 2.0]], 
[[7.0, 2.0, 3.0], [1.0, 2.0, 6.0], [1.0, 3.0, 7.0]]], [[[1.0, 5.0, 2.0], [5.0, 7.0, 3.0], [4.0, 8.0, 5.0]], [[1.0, 2.0, 7.0], 
[2.0, 7.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 1.0, 6.0], [3.0, 8.0, 1.0], [2.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPCP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 3.0], [3.0, 5.0, 6.0], [3.0, 4.0, 8.0]], [[8.0, 7.0, 6.0], [7.0, 3.0, 6.0], [3.0, 
1.0, 6.0]], [[1.0, 6.0, 5.0], [1.0, 5.0, 2.0], [7.0, 3.0, 6.0]]], [[[3.0, 2.0, 3.0], [7.0, 6.0, 7.0], [6.0, 7.0, 8.0]], [[2.0, 
5.0, 2.0], [2.0, 1.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 7.0, 6.0], [1.0, 3.0, 7.0], [5.0, 7.0, 7.0]]], [[[7.0, 5.0, 1.0], [4.0, 
4.0, 7.0], [2.0, 3.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 1.0, 5.0], [6.0, 2.0, 1.0]], [[5.0, 5.0, 6.0], [7.0, 4.0, 7.0], [6.0, 8.0, 
8.0]]], [[[6.0, 2.0, 2.0], [4.0, 4.0, 4.0], [2.0, 8.0, 5.0]], [[1.0, 3.0, 2.0], [4.0, 7.0, 2.0], [5.0, 1.0, 4.0]], [[2.0, 1.0, 
1.0], [6.0, 4.0, 5.0], [2.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 8.0], [7.0, 2.0, 1.0], [8.0, 5.0, 3.0]], 
[[6.0, 2.0, 2.0], [7.0, 8.0, 8.0], [7.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [8.0, 8.0, 3.0], [2.0, 5.0, 7.0]]], [[[5.0, 8.0, 1.0], 
[2.0, 6.0, 3.0], [5.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [1.0, 7.0, 7.0], [2.0, 6.0, 7.0]], [[1.0, 3.0, 7.0], [8.0, 1.0, 8.0], 
[4.0, 7.0, 3.0]]], [[[5.0, 4.0, 2.0], [1.0, 3.0, 4.0], [4.0, 4.0, 5.0]], [[3.0, 4.0, 4.0], [4.0, 1.0, 2.0], [7.0, 7.0, 4.0]], 
[[2.0, 1.0, 2.0], [5.0, 7.0, 7.0], [2.0, 1.0, 4.0]]], [[[4.0, 3.0, 7.0], [7.0, 2.0, 1.0], [2.0, 3.0, 2.0]], [[8.0, 8.0, 2.0], 
[5.0, 6.0, 5.0], [6.0, 1.0, 5.0]], [[3.0, 2.0, 4.0], [6.0, 3.0, 3.0], [1.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0, 7.0, 5.0])+(1.-msk_val)*numpy.array([7.0, 3.0, 4.0])
      arg[2,1,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 3.0], [3.0, 5.0, 6.0], [3.0, 4.0, 8.0]], [[8.0, 7.0, 6.0], [7.0, 3.0, 6.0], [3.0, 
1.0, 6.0]], [[1.0, 6.0, 5.0], [1.0, 5.0, 2.0], [7.0, 3.0, 6.0]]], [[[3.0, 2.0, 3.0], [7.0, 6.0, 7.0], [6.0, 7.0, 8.0]], [[2.0, 
5.0, 2.0], [2.0, 1.0, 5.0], [7.0, 4.0, 1.0]], [[8.0, 7.0, 6.0], [1.0, 3.0, 7.0], [5.0, 7.0, 7.0]]], [[[7.0, 5.0, 1.0], [4.0, 
4.0, 7.0], [2.0, 3.0, 4.0]], [[7.0, 5.0, 8.0], [1.0, 7.0, 5.0], [6.0, 5.0, 1.0]], [[5.0, 5.0, 6.0], [7.0, 4.0, 7.0], [6.0, 8.0, 
8.0]]], [[[6.0, 2.0, 2.0], [4.0, 4.0, 4.0], [2.0, 8.0, 5.0]], [[1.0, 3.0, 2.0], [4.0, 7.0, 2.0], [5.0, 1.0, 4.0]], [[2.0, 1.0, 
1.0], [6.0, 4.0, 5.0], [2.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 8.0], [7.0, 2.0, 1.0], [8.0, 5.0, 3.0]], 
[[6.0, 2.0, 2.0], [7.0, 8.0, 8.0], [7.0, 3.0, 7.0]], [[5.0, 5.0, 5.0], [8.0, 8.0, 3.0], [2.0, 5.0, 7.0]]], [[[5.0, 8.0, 1.0], 
[2.0, 6.0, 3.0], [5.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [1.0, 7.0, 7.0], [2.0, 6.0, 7.0]], [[1.0, 3.0, 7.0], [8.0, 1.0, 8.0], 
[4.0, 7.0, 3.0]]], [[[5.0, 4.0, 2.0], [1.0, 3.0, 4.0], [4.0, 4.0, 5.0]], [[3.0, 7.0, 4.0], [4.0, 3.0, 2.0], [7.0, 4.0, 4.0]], 
[[2.0, 1.0, 2.0], [5.0, 7.0, 7.0], [2.0, 1.0, 4.0]]], [[[4.0, 3.0, 7.0], [7.0, 2.0, 1.0], [2.0, 3.0, 2.0]], [[8.0, 8.0, 2.0], 
[5.0, 6.0, 5.0], [6.0, 1.0, 5.0]], [[3.0, 2.0, 4.0], [6.0, 3.0, 3.0], [1.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 5.0], [5.0, 6.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 4.0, 7.0], [6.0, 4.0, 8.0], [3.0, 
6.0, 1.0]], [[6.0, 5.0, 6.0], [2.0, 2.0, 8.0], [3.0, 4.0, 5.0]]], [[[2.0, 1.0, 1.0], [5.0, 4.0, 5.0], [1.0, 3.0, 7.0]], [[3.0, 
4.0, 2.0], [4.0, 4.0, 3.0], [7.0, 4.0, 3.0]], [[2.0, 3.0, 2.0], [8.0, 4.0, 7.0], [1.0, 4.0, 5.0]]], [[[5.0, 3.0, 7.0], [6.0, 
8.0, 3.0], [6.0, 5.0, 1.0]], [[4.0, 8.0, 6.0], [1.0, 7.0, 6.0], [8.0, 6.0, 8.0]], [[1.0, 1.0, 6.0], [6.0, 6.0, 2.0], [2.0, 1.0, 
2.0]]], [[[4.0, 4.0, 5.0], [8.0, 1.0, 1.0], [1.0, 4.0, 6.0]], [[7.0, 4.0, 8.0], [4.0, 2.0, 8.0], [5.0, 3.0, 2.0]], [[2.0, 8.0, 
4.0], [6.0, 7.0, 5.0], [1.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 7.0], [3.0, 2.0, 7.0], [4.0, 5.0, 2.0]], 
[[8.0, 7.0, 5.0], [5.0, 2.0, 6.0], [4.0, 4.0, 3.0]], [[1.0, 6.0, 2.0], [2.0, 3.0, 7.0], [2.0, 2.0, 5.0]]], [[[8.0, 4.0, 5.0], 
[2.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[7.0, 3.0, 4.0], [2.0, 3.0, 5.0], [4.0, 6.0, 4.0]], [[7.0, 7.0, 5.0], [3.0, 4.0, 2.0], 
[6.0, 5.0, 4.0]]], [[[1.0, 3.0, 2.0], [3.0, 6.0, 4.0], [1.0, 6.0, 3.0]], [[1.0, 6.0, 3.0], [4.0, 3.0, 3.0], [8.0, 5.0, 3.0]], 
[[6.0, 8.0, 6.0], [4.0, 7.0, 8.0], [5.0, 3.0, 1.0]]], [[[4.0, 6.0, 8.0], [4.0, 8.0, 2.0], [7.0, 7.0, 3.0]], [[6.0, 7.0, 4.0], 
[6.0, 4.0, 1.0], [5.0, 8.0, 8.0]], [[8.0, 7.0, 3.0], [4.0, 3.0, 4.0], [6.0, 1.0, 3.0]]]])
      val=numpy.array([[[[7.0, 2.0, 6.0], [4.0, 3.0, 3.0]]], [[[5.0, 7.0, 7.0], [2.0, 5.0, 7.0]]], [[[3.0, 8.0, 1.0], [2.0, 
8.0, 4.0]]], [[[5.0, 4.0, 5.0], [7.0, 2.0, 3.0]]]])
      arg[:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 5.0], [5.0, 6.0, 5.0], [7.0, 8.0, 3.0]], [[7.0, 2.0, 6.0], [4.0, 3.0, 3.0], [3.0, 
6.0, 1.0]], [[6.0, 5.0, 6.0], [2.0, 2.0, 8.0], [3.0, 4.0, 5.0]]], [[[2.0, 1.0, 1.0], [5.0, 4.0, 5.0], [1.0, 3.0, 7.0]], [[5.0, 
7.0, 7.0], [2.0, 5.0, 7.0], [7.0, 4.0, 3.0]], [[2.0, 3.0, 2.0], [8.0, 4.0, 7.0], [1.0, 4.0, 5.0]]], [[[5.0, 3.0, 7.0], [6.0, 
8.0, 3.0], [6.0, 5.0, 1.0]], [[3.0, 8.0, 1.0], [2.0, 8.0, 4.0], [8.0, 6.0, 8.0]], [[1.0, 1.0, 6.0], [6.0, 6.0, 2.0], [2.0, 1.0, 
2.0]]], [[[4.0, 4.0, 5.0], [8.0, 1.0, 1.0], [1.0, 4.0, 6.0]], [[5.0, 4.0, 5.0], [7.0, 2.0, 3.0], [5.0, 3.0, 2.0]], [[2.0, 8.0, 
4.0], [6.0, 7.0, 5.0], [1.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [3.0, 2.0, 7.0], [4.0, 5.0, 2.0]], 
[[7.0, 2.0, 6.0], [4.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[1.0, 6.0, 2.0], [2.0, 3.0, 7.0], [2.0, 2.0, 5.0]]], [[[8.0, 4.0, 5.0], 
[2.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[5.0, 7.0, 7.0], [2.0, 5.0, 7.0], [4.0, 6.0, 4.0]], [[7.0, 7.0, 5.0], [3.0, 4.0, 2.0], 
[6.0, 5.0, 4.0]]], [[[1.0, 3.0, 2.0], [3.0, 6.0, 4.0], [1.0, 6.0, 3.0]], [[3.0, 8.0, 1.0], [2.0, 8.0, 4.0], [8.0, 5.0, 3.0]], 
[[6.0, 8.0, 6.0], [4.0, 7.0, 8.0], [5.0, 3.0, 1.0]]], [[[4.0, 6.0, 8.0], [4.0, 8.0, 2.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 5.0], 
[7.0, 2.0, 3.0], [5.0, 8.0, 8.0]], [[8.0, 7.0, 3.0], [4.0, 3.0, 4.0], [6.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 4.0], [6.0, 5.0, 8.0], [6.0, 1.0, 3.0]], [[6.0, 7.0, 8.0], [6.0, 2.0, 5.0], [4.0, 
2.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 6.0, 3.0], [2.0, 1.0, 4.0]]], [[[7.0, 4.0, 2.0], [7.0, 6.0, 6.0], [7.0, 4.0, 6.0]], [[2.0, 
1.0, 6.0], [6.0, 1.0, 1.0], [7.0, 6.0, 6.0]], [[6.0, 7.0, 8.0], [2.0, 5.0, 2.0], [2.0, 5.0, 7.0]]], [[[1.0, 3.0, 4.0], [4.0, 
5.0, 4.0], [4.0, 7.0, 4.0]], [[2.0, 7.0, 3.0], [8.0, 1.0, 5.0], [3.0, 8.0, 8.0]], [[6.0, 7.0, 1.0], [1.0, 8.0, 2.0], [5.0, 2.0, 
8.0]]], [[[5.0, 3.0, 3.0], [1.0, 3.0, 6.0], [4.0, 8.0, 1.0]], [[7.0, 4.0, 7.0], [7.0, 7.0, 3.0], [7.0, 2.0, 4.0]], [[5.0, 4.0, 
6.0], [8.0, 6.0, 6.0], [1.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 8.0], [1.0, 2.0, 7.0], [2.0, 4.0, 7.0]], 
[[4.0, 4.0, 8.0], [6.0, 5.0, 7.0], [1.0, 4.0, 5.0]], [[7.0, 1.0, 1.0], [5.0, 4.0, 2.0], [7.0, 8.0, 5.0]]], [[[6.0, 1.0, 8.0], 
[2.0, 7.0, 6.0], [6.0, 7.0, 4.0]], [[4.0, 6.0, 4.0], [2.0, 4.0, 1.0], [2.0, 8.0, 1.0]], [[6.0, 8.0, 4.0], [6.0, 5.0, 2.0], 
[2.0, 5.0, 5.0]]], [[[8.0, 8.0, 8.0], [3.0, 4.0, 6.0], [1.0, 2.0, 8.0]], [[1.0, 8.0, 8.0], [3.0, 6.0, 3.0], [3.0, 3.0, 7.0]], 
[[3.0, 5.0, 6.0], [7.0, 4.0, 1.0], [5.0, 1.0, 4.0]]], [[[7.0, 2.0, 6.0], [7.0, 2.0, 3.0], [6.0, 4.0, 2.0]], [[7.0, 7.0, 2.0], 
[2.0, 1.0, 6.0], [4.0, 2.0, 6.0]], [[5.0, 3.0, 3.0], [3.0, 6.0, 8.0], [5.0, 3.0, 1.0]]]])
      val=Data(numpy.array([[[[8.0, 6.0, 4.0], [4.0, 2.0, 7.0]]], [[[2.0, 5.0, 7.0], [7.0, 7.0, 2.0]]], [[[7.0, 8.0, 8.0], 
[5.0, 2.0, 6.0]]], [[[5.0, 6.0, 3.0], [4.0, 7.0, 2.0]]]]),self.functionspace)
      arg[:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 4.0], [6.0, 5.0, 8.0], [6.0, 1.0, 3.0]], [[8.0, 6.0, 4.0], [4.0, 2.0, 7.0], [4.0, 
2.0, 8.0]], [[6.0, 4.0, 2.0], [4.0, 6.0, 3.0], [2.0, 1.0, 4.0]]], [[[7.0, 4.0, 2.0], [7.0, 6.0, 6.0], [7.0, 4.0, 6.0]], [[2.0, 
5.0, 7.0], [7.0, 7.0, 2.0], [7.0, 6.0, 6.0]], [[6.0, 7.0, 8.0], [2.0, 5.0, 2.0], [2.0, 5.0, 7.0]]], [[[1.0, 3.0, 4.0], [4.0, 
5.0, 4.0], [4.0, 7.0, 4.0]], [[7.0, 8.0, 8.0], [5.0, 2.0, 6.0], [3.0, 8.0, 8.0]], [[6.0, 7.0, 1.0], [1.0, 8.0, 2.0], [5.0, 2.0, 
8.0]]], [[[5.0, 3.0, 3.0], [1.0, 3.0, 6.0], [4.0, 8.0, 1.0]], [[5.0, 6.0, 3.0], [4.0, 7.0, 2.0], [7.0, 2.0, 4.0]], [[5.0, 4.0, 
6.0], [8.0, 6.0, 6.0], [1.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 8.0], [1.0, 2.0, 7.0], [2.0, 4.0, 7.0]], 
[[8.0, 6.0, 4.0], [4.0, 2.0, 7.0], [1.0, 4.0, 5.0]], [[7.0, 1.0, 1.0], [5.0, 4.0, 2.0], [7.0, 8.0, 5.0]]], [[[6.0, 1.0, 8.0], 
[2.0, 7.0, 6.0], [6.0, 7.0, 4.0]], [[2.0, 5.0, 7.0], [7.0, 7.0, 2.0], [2.0, 8.0, 1.0]], [[6.0, 8.0, 4.0], [6.0, 5.0, 2.0], 
[2.0, 5.0, 5.0]]], [[[8.0, 8.0, 8.0], [3.0, 4.0, 6.0], [1.0, 2.0, 8.0]], [[7.0, 8.0, 8.0], [5.0, 2.0, 6.0], [3.0, 3.0, 7.0]], 
[[3.0, 5.0, 6.0], [7.0, 4.0, 1.0], [5.0, 1.0, 4.0]]], [[[7.0, 2.0, 6.0], [7.0, 2.0, 3.0], [6.0, 4.0, 2.0]], [[5.0, 6.0, 3.0], 
[4.0, 7.0, 2.0], [4.0, 2.0, 6.0]], [[5.0, 3.0, 3.0], [3.0, 6.0, 8.0], [5.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 7.0], [2.0, 3.0, 4.0], [7.0, 7.0, 7.0]], [[3.0, 6.0, 2.0], [1.0, 5.0, 6.0], [5.0, 
2.0, 7.0]], [[3.0, 1.0, 5.0], [4.0, 6.0, 2.0], [3.0, 7.0, 4.0]]], [[[3.0, 2.0, 2.0], [4.0, 3.0, 6.0], [2.0, 6.0, 6.0]], [[8.0, 
6.0, 5.0], [8.0, 8.0, 7.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 2.0], [5.0, 3.0, 3.0], [1.0, 1.0, 4.0]]], [[[7.0, 7.0, 2.0], [6.0, 
4.0, 4.0], [6.0, 7.0, 7.0]], [[2.0, 5.0, 3.0], [2.0, 7.0, 1.0], [4.0, 7.0, 1.0]], [[5.0, 7.0, 5.0], [5.0, 5.0, 6.0], [1.0, 2.0, 
7.0]]], [[[2.0, 5.0, 4.0], [3.0, 2.0, 7.0], [5.0, 1.0, 5.0]], [[8.0, 7.0, 1.0], [6.0, 4.0, 1.0], [8.0, 2.0, 3.0]], [[3.0, 6.0, 
2.0], [6.0, 6.0, 2.0], [1.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 3.0], [6.0, 7.0, 4.0], [6.0, 8.0, 8.0]], 
[[2.0, 7.0, 1.0], [5.0, 8.0, 7.0], [8.0, 3.0, 1.0]], [[7.0, 7.0, 6.0], [8.0, 5.0, 4.0], [8.0, 8.0, 7.0]]], [[[8.0, 5.0, 6.0], 
[2.0, 4.0, 7.0], [2.0, 8.0, 5.0]], [[8.0, 2.0, 7.0], [5.0, 5.0, 4.0], [8.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [8.0, 4.0, 6.0], 
[1.0, 8.0, 3.0]]], [[[1.0, 8.0, 4.0], [6.0, 3.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 3.0, 1.0], [2.0, 6.0, 3.0], [7.0, 4.0, 1.0]], 
[[2.0, 1.0, 7.0], [6.0, 8.0, 6.0], [6.0, 2.0, 1.0]]], [[[5.0, 7.0, 8.0], [1.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[8.0, 1.0, 5.0], 
[8.0, 7.0, 7.0], [6.0, 4.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 2.0], [6.0, 4.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 6.0, 7.0], [4.0, 8.0, 2.0]]], [[[1.0, 3.0, 5.0], [7.0, 8.0, 7.0]]], [[[7.0, 5.0, 7.0], 
[2.0, 4.0, 3.0]]], [[[6.0, 6.0, 2.0], [8.0, 6.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 3.0, 6.0], [5.0, 5.0, 7.0]]], 
[[[1.0, 4.0, 1.0], [2.0, 8.0, 5.0]]], [[[8.0, 3.0, 8.0], [2.0, 6.0, 7.0]]], [[[7.0, 1.0, 8.0], [7.0, 5.0, 6.0]]]])
      arg[:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 7.0], [2.0, 3.0, 4.0], [7.0, 7.0, 7.0]], [[4.0, 6.0, 7.0], [4.0, 8.0, 2.0], [5.0, 
2.0, 7.0]], [[3.0, 1.0, 5.0], [4.0, 6.0, 2.0], [3.0, 7.0, 4.0]]], [[[3.0, 2.0, 2.0], [4.0, 3.0, 6.0], [2.0, 6.0, 6.0]], [[1.0, 
3.0, 5.0], [7.0, 8.0, 7.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 2.0], [5.0, 3.0, 3.0], [1.0, 1.0, 4.0]]], [[[7.0, 7.0, 2.0], [6.0, 
4.0, 4.0], [6.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 3.0], [4.0, 7.0, 1.0]], [[5.0, 7.0, 5.0], [5.0, 5.0, 6.0], [1.0, 2.0, 
7.0]]], [[[2.0, 5.0, 4.0], [3.0, 2.0, 7.0], [5.0, 1.0, 5.0]], [[6.0, 6.0, 2.0], [8.0, 6.0, 7.0], [8.0, 2.0, 3.0]], [[3.0, 6.0, 
2.0], [6.0, 6.0, 2.0], [1.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 3.0], [6.0, 7.0, 4.0], [6.0, 8.0, 8.0]], 
[[5.0, 3.0, 6.0], [5.0, 5.0, 7.0], [8.0, 3.0, 1.0]], [[7.0, 7.0, 6.0], [8.0, 5.0, 4.0], [8.0, 8.0, 7.0]]], [[[8.0, 5.0, 6.0], 
[2.0, 4.0, 7.0], [2.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [2.0, 8.0, 5.0], [8.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [8.0, 4.0, 6.0], 
[1.0, 8.0, 3.0]]], [[[1.0, 8.0, 4.0], [6.0, 3.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 3.0, 8.0], [2.0, 6.0, 7.0], [7.0, 4.0, 1.0]], 
[[2.0, 1.0, 7.0], [6.0, 8.0, 6.0], [6.0, 2.0, 1.0]]], [[[5.0, 7.0, 8.0], [1.0, 8.0, 7.0], [4.0, 7.0, 7.0]], [[7.0, 1.0, 8.0], 
[7.0, 5.0, 6.0], [6.0, 4.0, 8.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 2.0], [6.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 5.0], [6.0, 6.0, 4.0], [4.0, 7.0, 6.0]], [[7.0, 6.0, 5.0], [3.0, 3.0, 3.0], [5.0, 
8.0, 3.0]], [[7.0, 3.0, 4.0], [6.0, 3.0, 4.0], [2.0, 5.0, 5.0]]], [[[7.0, 6.0, 7.0], [4.0, 2.0, 6.0], [7.0, 1.0, 1.0]], [[1.0, 
5.0, 3.0], [3.0, 2.0, 6.0], [6.0, 1.0, 4.0]], [[7.0, 6.0, 4.0], [3.0, 3.0, 4.0], [1.0, 4.0, 7.0]]], [[[3.0, 7.0, 2.0], [3.0, 
2.0, 7.0], [5.0, 4.0, 4.0]], [[6.0, 2.0, 1.0], [4.0, 8.0, 7.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 1.0], [4.0, 2.0, 1.0], [5.0, 2.0, 
8.0]]], [[[7.0, 2.0, 6.0], [3.0, 1.0, 8.0], [4.0, 2.0, 1.0]], [[8.0, 5.0, 1.0], [5.0, 3.0, 5.0], [3.0, 7.0, 4.0]], [[6.0, 6.0, 
2.0], [5.0, 5.0, 1.0], [8.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 3.0], [2.0, 1.0, 3.0], [3.0, 6.0, 3.0]], 
[[2.0, 5.0, 4.0], [3.0, 6.0, 2.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 1.0], [8.0, 2.0, 4.0]]], [[[5.0, 3.0, 7.0], 
[6.0, 4.0, 5.0], [1.0, 7.0, 7.0]], [[5.0, 3.0, 8.0], [3.0, 8.0, 3.0], [6.0, 5.0, 2.0]], [[6.0, 5.0, 4.0], [6.0, 1.0, 6.0], 
[5.0, 4.0, 4.0]]], [[[8.0, 8.0, 5.0], [1.0, 1.0, 4.0], [8.0, 5.0, 1.0]], [[3.0, 3.0, 3.0], [8.0, 3.0, 3.0], [8.0, 7.0, 8.0]], 
[[3.0, 3.0, 8.0], [1.0, 8.0, 8.0], [7.0, 4.0, 5.0]]], [[[1.0, 6.0, 4.0], [1.0, 7.0, 7.0], [1.0, 7.0, 2.0]], [[2.0, 7.0, 2.0], 
[5.0, 6.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 2.0, 2.0], [5.0, 5.0, 8.0], [1.0, 1.0, 5.0]]]])
      val=numpy.array([[[[3.0, 6.0, 5.0]]], [[[8.0, 1.0, 8.0]]], [[[6.0, 7.0, 4.0]]]])
      arg[:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 5.0], [6.0, 6.0, 4.0], [4.0, 7.0, 6.0]], [[7.0, 6.0, 5.0], [3.0, 3.0, 3.0], [5.0, 
8.0, 3.0]], [[7.0, 3.0, 4.0], [6.0, 3.0, 4.0], [3.0, 6.0, 5.0]]], [[[7.0, 6.0, 7.0], [4.0, 2.0, 6.0], [7.0, 1.0, 1.0]], [[1.0, 
5.0, 3.0], [3.0, 2.0, 6.0], [6.0, 1.0, 4.0]], [[7.0, 6.0, 4.0], [3.0, 3.0, 4.0], [8.0, 1.0, 8.0]]], [[[3.0, 7.0, 2.0], [3.0, 
2.0, 7.0], [5.0, 4.0, 4.0]], [[6.0, 2.0, 1.0], [4.0, 8.0, 7.0], [4.0, 6.0, 4.0]], [[5.0, 7.0, 1.0], [4.0, 2.0, 1.0], [6.0, 7.0, 
4.0]]], [[[7.0, 2.0, 6.0], [3.0, 1.0, 8.0], [4.0, 2.0, 1.0]], [[8.0, 5.0, 1.0], [5.0, 3.0, 5.0], [3.0, 7.0, 4.0]], [[6.0, 6.0, 
2.0], [5.0, 5.0, 1.0], [8.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 3.0], [2.0, 1.0, 3.0], [3.0, 6.0, 3.0]], 
[[2.0, 5.0, 4.0], [3.0, 6.0, 2.0], [3.0, 2.0, 3.0]], [[1.0, 2.0, 5.0], [7.0, 7.0, 1.0], [3.0, 6.0, 5.0]]], [[[5.0, 3.0, 7.0], 
[6.0, 4.0, 5.0], [1.0, 7.0, 7.0]], [[5.0, 3.0, 8.0], [3.0, 8.0, 3.0], [6.0, 5.0, 2.0]], [[6.0, 5.0, 4.0], [6.0, 1.0, 6.0], 
[8.0, 1.0, 8.0]]], [[[8.0, 8.0, 5.0], [1.0, 1.0, 4.0], [8.0, 5.0, 1.0]], [[3.0, 3.0, 3.0], [8.0, 3.0, 3.0], [8.0, 7.0, 8.0]], 
[[3.0, 3.0, 8.0], [1.0, 8.0, 8.0], [6.0, 7.0, 4.0]]], [[[1.0, 6.0, 4.0], [1.0, 7.0, 7.0], [1.0, 7.0, 2.0]], [[2.0, 7.0, 2.0], 
[5.0, 6.0, 8.0], [6.0, 5.0, 5.0]], [[3.0, 2.0, 2.0], [5.0, 5.0, 8.0], [1.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 1.0], [2.0, 5.0, 7.0], [7.0, 3.0, 1.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 8.0], [3.0, 
6.0, 2.0]], [[1.0, 1.0, 4.0], [3.0, 2.0, 4.0], [3.0, 2.0, 8.0]]], [[[1.0, 6.0, 8.0], [3.0, 6.0, 6.0], [1.0, 8.0, 7.0]], [[7.0, 
5.0, 3.0], [8.0, 8.0, 3.0], [8.0, 7.0, 2.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 8.0], [2.0, 4.0, 7.0]]], [[[2.0, 4.0, 2.0], [2.0, 
7.0, 4.0], [3.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [6.0, 5.0, 4.0], [4.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [8.0, 4.0, 8.0], [3.0, 3.0, 
5.0]]], [[[2.0, 3.0, 2.0], [3.0, 8.0, 1.0], [2.0, 1.0, 1.0]], [[1.0, 1.0, 4.0], [4.0, 3.0, 4.0], [8.0, 8.0, 5.0]], [[4.0, 7.0, 
8.0], [5.0, 1.0, 8.0], [4.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 4.0], [7.0, 5.0, 4.0], [2.0, 2.0, 6.0]], 
[[3.0, 3.0, 6.0], [2.0, 3.0, 4.0], [1.0, 8.0, 5.0]], [[3.0, 5.0, 4.0], [7.0, 5.0, 6.0], [3.0, 7.0, 1.0]]], [[[4.0, 7.0, 2.0], 
[4.0, 2.0, 6.0], [4.0, 4.0, 6.0]], [[1.0, 2.0, 7.0], [7.0, 6.0, 6.0], [3.0, 5.0, 4.0]], [[7.0, 2.0, 7.0], [6.0, 6.0, 3.0], 
[7.0, 1.0, 4.0]]], [[[1.0, 5.0, 7.0], [6.0, 6.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 2.0, 8.0], [7.0, 7.0, 6.0], [3.0, 3.0, 8.0]], 
[[4.0, 6.0, 8.0], [2.0, 8.0, 2.0], [4.0, 8.0, 7.0]]], [[[3.0, 3.0, 3.0], [4.0, 2.0, 6.0], [1.0, 1.0, 6.0]], [[5.0, 1.0, 3.0], 
[8.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 4.0], [2.0, 3.0, 5.0], [7.0, 4.0, 6.0]]]])
      val=Data(numpy.array([[[[8.0, 3.0, 8.0]]], [[[5.0, 6.0, 2.0]]], [[[2.0, 2.0, 6.0]]]]),self.functionspace)
      arg[:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 1.0], [2.0, 5.0, 7.0], [7.0, 3.0, 1.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 8.0], [3.0, 
6.0, 2.0]], [[1.0, 1.0, 4.0], [3.0, 2.0, 4.0], [8.0, 3.0, 8.0]]], [[[1.0, 6.0, 8.0], [3.0, 6.0, 6.0], [1.0, 8.0, 7.0]], [[7.0, 
5.0, 3.0], [8.0, 8.0, 3.0], [8.0, 7.0, 2.0]], [[2.0, 2.0, 4.0], [7.0, 5.0, 8.0], [5.0, 6.0, 2.0]]], [[[2.0, 4.0, 2.0], [2.0, 
7.0, 4.0], [3.0, 2.0, 8.0]], [[4.0, 5.0, 6.0], [6.0, 5.0, 4.0], [4.0, 3.0, 7.0]], [[5.0, 8.0, 2.0], [8.0, 4.0, 8.0], [2.0, 2.0, 
6.0]]], [[[2.0, 3.0, 2.0], [3.0, 8.0, 1.0], [2.0, 1.0, 1.0]], [[1.0, 1.0, 4.0], [4.0, 3.0, 4.0], [8.0, 8.0, 5.0]], [[4.0, 7.0, 
8.0], [5.0, 1.0, 8.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [7.0, 5.0, 4.0], [2.0, 2.0, 6.0]], 
[[3.0, 3.0, 6.0], [2.0, 3.0, 4.0], [1.0, 8.0, 5.0]], [[3.0, 5.0, 4.0], [7.0, 5.0, 6.0], [8.0, 3.0, 8.0]]], [[[4.0, 7.0, 2.0], 
[4.0, 2.0, 6.0], [4.0, 4.0, 6.0]], [[1.0, 2.0, 7.0], [7.0, 6.0, 6.0], [3.0, 5.0, 4.0]], [[7.0, 2.0, 7.0], [6.0, 6.0, 3.0], 
[5.0, 6.0, 2.0]]], [[[1.0, 5.0, 7.0], [6.0, 6.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 2.0, 8.0], [7.0, 7.0, 6.0], [3.0, 3.0, 8.0]], 
[[4.0, 6.0, 8.0], [2.0, 8.0, 2.0], [2.0, 2.0, 6.0]]], [[[3.0, 3.0, 3.0], [4.0, 2.0, 6.0], [1.0, 1.0, 6.0]], [[5.0, 1.0, 3.0], 
[8.0, 2.0, 7.0], [8.0, 5.0, 3.0]], [[8.0, 6.0, 4.0], [2.0, 3.0, 5.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 2.0], [1.0, 8.0, 5.0], [5.0, 6.0, 7.0]], [[5.0, 3.0, 1.0], [2.0, 1.0, 7.0], [4.0, 
1.0, 8.0]], [[6.0, 2.0, 8.0], [2.0, 1.0, 3.0], [6.0, 6.0, 7.0]]], [[[1.0, 5.0, 4.0], [4.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[5.0, 
4.0, 5.0], [3.0, 5.0, 5.0], [2.0, 5.0, 8.0]], [[8.0, 3.0, 6.0], [5.0, 8.0, 2.0], [7.0, 2.0, 8.0]]], [[[8.0, 2.0, 4.0], [2.0, 
2.0, 6.0], [1.0, 6.0, 6.0]], [[2.0, 1.0, 4.0], [1.0, 5.0, 5.0], [6.0, 6.0, 4.0]], [[8.0, 7.0, 8.0], [8.0, 1.0, 7.0], [7.0, 1.0, 
3.0]]], [[[1.0, 2.0, 3.0], [5.0, 7.0, 8.0], [5.0, 3.0, 5.0]], [[4.0, 5.0, 3.0], [5.0, 2.0, 2.0], [6.0, 1.0, 5.0]], [[6.0, 1.0, 
7.0], [7.0, 1.0, 3.0], [1.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 6.0], [2.0, 7.0, 1.0], [5.0, 1.0, 3.0]], 
[[4.0, 8.0, 6.0], [1.0, 5.0, 3.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 5.0], [7.0, 4.0, 1.0], [5.0, 8.0, 3.0]]], [[[8.0, 5.0, 7.0], 
[7.0, 3.0, 4.0], [6.0, 7.0, 2.0]], [[6.0, 8.0, 3.0], [7.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [4.0, 8.0, 2.0], 
[6.0, 8.0, 3.0]]], [[[4.0, 4.0, 5.0], [6.0, 2.0, 6.0], [2.0, 7.0, 1.0]], [[2.0, 4.0, 1.0], [2.0, 7.0, 7.0], [4.0, 7.0, 5.0]], 
[[5.0, 4.0, 7.0], [3.0, 2.0, 3.0], [2.0, 7.0, 2.0]]], [[[8.0, 7.0, 8.0], [2.0, 1.0, 3.0], [8.0, 4.0, 2.0]], [[7.0, 2.0, 8.0], 
[7.0, 4.0, 8.0], [6.0, 4.0, 1.0]], [[5.0, 7.0, 5.0], [7.0, 6.0, 1.0], [8.0, 3.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 1.0, 6.0]]], [[[1.0, 6.0, 1.0]]], [[[1.0, 2.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 4.0, 3.0]]], [[[8.0, 2.0, 3.0]]], [[[4.0, 7.0, 7.0]]]])
      arg[:3,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 2.0], [1.0, 8.0, 5.0], [5.0, 6.0, 7.0]], [[5.0, 3.0, 1.0], [2.0, 1.0, 7.0], [4.0, 
1.0, 8.0]], [[6.0, 2.0, 8.0], [2.0, 1.0, 3.0], [8.0, 1.0, 6.0]]], [[[1.0, 5.0, 4.0], [4.0, 7.0, 3.0], [7.0, 6.0, 8.0]], [[5.0, 
4.0, 5.0], [3.0, 5.0, 5.0], [2.0, 5.0, 8.0]], [[8.0, 3.0, 6.0], [5.0, 8.0, 2.0], [1.0, 6.0, 1.0]]], [[[8.0, 2.0, 4.0], [2.0, 
2.0, 6.0], [1.0, 6.0, 6.0]], [[2.0, 1.0, 4.0], [1.0, 5.0, 5.0], [6.0, 6.0, 4.0]], [[8.0, 7.0, 8.0], [8.0, 1.0, 7.0], [1.0, 2.0, 
5.0]]], [[[1.0, 2.0, 3.0], [5.0, 7.0, 8.0], [5.0, 3.0, 5.0]], [[4.0, 5.0, 3.0], [5.0, 2.0, 2.0], [6.0, 1.0, 5.0]], [[6.0, 1.0, 
7.0], [7.0, 1.0, 3.0], [1.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 6.0], [2.0, 7.0, 1.0], [5.0, 1.0, 3.0]], 
[[4.0, 8.0, 6.0], [1.0, 5.0, 3.0], [3.0, 4.0, 3.0]], [[2.0, 1.0, 5.0], [7.0, 4.0, 1.0], [8.0, 4.0, 3.0]]], [[[8.0, 5.0, 7.0], 
[7.0, 3.0, 4.0], [6.0, 7.0, 2.0]], [[6.0, 8.0, 3.0], [7.0, 1.0, 5.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [4.0, 8.0, 2.0], 
[8.0, 2.0, 3.0]]], [[[4.0, 4.0, 5.0], [6.0, 2.0, 6.0], [2.0, 7.0, 1.0]], [[2.0, 4.0, 1.0], [2.0, 7.0, 7.0], [4.0, 7.0, 5.0]], 
[[5.0, 4.0, 7.0], [3.0, 2.0, 3.0], [4.0, 7.0, 7.0]]], [[[8.0, 7.0, 8.0], [2.0, 1.0, 3.0], [8.0, 4.0, 2.0]], [[7.0, 2.0, 8.0], 
[7.0, 4.0, 8.0], [6.0, 4.0, 1.0]], [[5.0, 7.0, 5.0], [7.0, 6.0, 1.0], [8.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 7.0], [8.0, 8.0, 6.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [5.0, 4.0, 7.0], [8.0, 
4.0, 4.0]], [[4.0, 5.0, 6.0], [5.0, 5.0, 4.0], [5.0, 4.0, 8.0]]], [[[3.0, 6.0, 8.0], [8.0, 3.0, 6.0], [1.0, 1.0, 3.0]], [[5.0, 
4.0, 2.0], [3.0, 2.0, 5.0], [8.0, 5.0, 4.0]], [[8.0, 6.0, 8.0], [8.0, 4.0, 8.0], [1.0, 7.0, 3.0]]], [[[7.0, 2.0, 4.0], [2.0, 
2.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 2.0, 6.0], [8.0, 3.0, 1.0], [7.0, 3.0, 3.0]], [[3.0, 7.0, 4.0], [8.0, 2.0, 3.0], [3.0, 8.0, 
5.0]]], [[[6.0, 3.0, 6.0], [2.0, 8.0, 1.0], [8.0, 1.0, 6.0]], [[3.0, 2.0, 7.0], [1.0, 1.0, 8.0], [3.0, 8.0, 1.0]], [[4.0, 6.0, 
3.0], [5.0, 8.0, 8.0], [1.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 5.0], [8.0, 7.0, 1.0], [8.0, 8.0, 5.0]], 
[[5.0, 6.0, 5.0], [7.0, 1.0, 2.0], [1.0, 7.0, 3.0]], [[6.0, 2.0, 7.0], [6.0, 1.0, 1.0], [4.0, 4.0, 6.0]]], [[[2.0, 8.0, 6.0], 
[6.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[5.0, 8.0, 1.0], [7.0, 1.0, 3.0], [7.0, 2.0, 3.0]], [[6.0, 3.0, 3.0], [6.0, 1.0, 3.0], 
[3.0, 6.0, 8.0]]], [[[8.0, 7.0, 6.0], [4.0, 7.0, 6.0], [1.0, 7.0, 3.0]], [[1.0, 1.0, 5.0], [3.0, 4.0, 8.0], [3.0, 7.0, 1.0]], 
[[1.0, 3.0, 5.0], [5.0, 7.0, 1.0], [5.0, 2.0, 2.0]]], [[[6.0, 5.0, 1.0], [7.0, 4.0, 8.0], [6.0, 7.0, 1.0]], [[4.0, 1.0, 2.0], 
[6.0, 8.0, 5.0], [7.0, 6.0, 7.0]], [[5.0, 7.0, 2.0], [4.0, 4.0, 4.0], [6.0, 8.0, 4.0]]]])
      val=numpy.array([[[8.0, 3.0, 7.0]]])
      arg[2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 7.0], [8.0, 8.0, 6.0], [5.0, 5.0, 5.0]], [[2.0, 2.0, 8.0], [5.0, 4.0, 7.0], [8.0, 
4.0, 4.0]], [[4.0, 5.0, 6.0], [5.0, 5.0, 4.0], [5.0, 4.0, 8.0]]], [[[3.0, 6.0, 8.0], [8.0, 3.0, 6.0], [1.0, 1.0, 3.0]], [[5.0, 
4.0, 2.0], [3.0, 2.0, 5.0], [8.0, 5.0, 4.0]], [[8.0, 6.0, 8.0], [8.0, 4.0, 8.0], [1.0, 7.0, 3.0]]], [[[7.0, 2.0, 4.0], [2.0, 
2.0, 4.0], [1.0, 4.0, 5.0]], [[4.0, 2.0, 6.0], [8.0, 3.0, 1.0], [7.0, 3.0, 3.0]], [[3.0, 7.0, 4.0], [8.0, 2.0, 3.0], [8.0, 3.0, 
7.0]]], [[[6.0, 3.0, 6.0], [2.0, 8.0, 1.0], [8.0, 1.0, 6.0]], [[3.0, 2.0, 7.0], [1.0, 1.0, 8.0], [3.0, 8.0, 1.0]], [[4.0, 6.0, 
3.0], [5.0, 8.0, 8.0], [1.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 5.0], [8.0, 7.0, 1.0], [8.0, 8.0, 5.0]], 
[[5.0, 6.0, 5.0], [7.0, 1.0, 2.0], [1.0, 7.0, 3.0]], [[6.0, 2.0, 7.0], [6.0, 1.0, 1.0], [4.0, 4.0, 6.0]]], [[[2.0, 8.0, 6.0], 
[6.0, 6.0, 2.0], [7.0, 5.0, 3.0]], [[5.0, 8.0, 1.0], [7.0, 1.0, 3.0], [7.0, 2.0, 3.0]], [[6.0, 3.0, 3.0], [6.0, 1.0, 3.0], 
[3.0, 6.0, 8.0]]], [[[8.0, 7.0, 6.0], [4.0, 7.0, 6.0], [1.0, 7.0, 3.0]], [[1.0, 1.0, 5.0], [3.0, 4.0, 8.0], [3.0, 7.0, 1.0]], 
[[1.0, 3.0, 5.0], [5.0, 7.0, 1.0], [8.0, 3.0, 7.0]]], [[[6.0, 5.0, 1.0], [7.0, 4.0, 8.0], [6.0, 7.0, 1.0]], [[4.0, 1.0, 2.0], 
[6.0, 8.0, 5.0], [7.0, 6.0, 7.0]], [[5.0, 7.0, 2.0], [4.0, 4.0, 4.0], [6.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 2.0], [2.0, 4.0, 4.0], [2.0, 1.0, 5.0]], [[2.0, 2.0, 7.0], [2.0, 4.0, 7.0], [6.0, 
3.0, 3.0]], [[7.0, 4.0, 5.0], [8.0, 1.0, 4.0], [2.0, 2.0, 7.0]]], [[[4.0, 2.0, 5.0], [5.0, 7.0, 4.0], [4.0, 6.0, 4.0]], [[4.0, 
3.0, 8.0], [2.0, 2.0, 2.0], [2.0, 3.0, 8.0]], [[4.0, 1.0, 8.0], [5.0, 6.0, 8.0], [6.0, 6.0, 4.0]]], [[[7.0, 2.0, 8.0], [2.0, 
4.0, 3.0], [4.0, 6.0, 8.0]], [[8.0, 3.0, 6.0], [5.0, 6.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 2.0, 4.0], [4.0, 5.0, 2.0], [5.0, 3.0, 
2.0]]], [[[1.0, 5.0, 6.0], [5.0, 2.0, 3.0], [7.0, 5.0, 2.0]], [[3.0, 3.0, 2.0], [4.0, 6.0, 3.0], [5.0, 5.0, 8.0]], [[3.0, 5.0, 
8.0], [1.0, 2.0, 4.0], [7.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 2.0], [3.0, 8.0, 4.0], [6.0, 3.0, 3.0]], 
[[3.0, 8.0, 4.0], [3.0, 6.0, 8.0], [5.0, 1.0, 5.0]], [[7.0, 1.0, 7.0], [8.0, 4.0, 4.0], [8.0, 3.0, 5.0]]], [[[7.0, 7.0, 4.0], 
[1.0, 8.0, 7.0], [4.0, 2.0, 2.0]], [[6.0, 2.0, 4.0], [4.0, 6.0, 4.0], [2.0, 7.0, 5.0]], [[1.0, 8.0, 6.0], [2.0, 8.0, 2.0], 
[6.0, 2.0, 5.0]]], [[[7.0, 1.0, 4.0], [6.0, 8.0, 7.0], [3.0, 5.0, 3.0]], [[3.0, 5.0, 5.0], [3.0, 4.0, 6.0], [2.0, 8.0, 3.0]], 
[[2.0, 5.0, 1.0], [6.0, 1.0, 1.0], [6.0, 3.0, 6.0]]], [[[3.0, 1.0, 1.0], [4.0, 3.0, 5.0], [1.0, 6.0, 1.0]], [[8.0, 5.0, 4.0], 
[7.0, 1.0, 4.0], [2.0, 4.0, 4.0]], [[3.0, 1.0, 1.0], [5.0, 1.0, 4.0], [8.0, 4.0, 8.0]]]])
      val=Data(numpy.array([[[1.0, 6.0, 4.0]]]),self.functionspace)
      arg[2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 2.0], [2.0, 4.0, 4.0], [2.0, 1.0, 5.0]], [[2.0, 2.0, 7.0], [2.0, 4.0, 7.0], [6.0, 
3.0, 3.0]], [[7.0, 4.0, 5.0], [8.0, 1.0, 4.0], [2.0, 2.0, 7.0]]], [[[4.0, 2.0, 5.0], [5.0, 7.0, 4.0], [4.0, 6.0, 4.0]], [[4.0, 
3.0, 8.0], [2.0, 2.0, 2.0], [2.0, 3.0, 8.0]], [[4.0, 1.0, 8.0], [5.0, 6.0, 8.0], [6.0, 6.0, 4.0]]], [[[7.0, 2.0, 8.0], [2.0, 
4.0, 3.0], [4.0, 6.0, 8.0]], [[8.0, 3.0, 6.0], [5.0, 6.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 2.0, 4.0], [4.0, 5.0, 2.0], [1.0, 6.0, 
4.0]]], [[[1.0, 5.0, 6.0], [5.0, 2.0, 3.0], [7.0, 5.0, 2.0]], [[3.0, 3.0, 2.0], [4.0, 6.0, 3.0], [5.0, 5.0, 8.0]], [[3.0, 5.0, 
8.0], [1.0, 2.0, 4.0], [7.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 2.0], [3.0, 8.0, 4.0], [6.0, 3.0, 3.0]], 
[[3.0, 8.0, 4.0], [3.0, 6.0, 8.0], [5.0, 1.0, 5.0]], [[7.0, 1.0, 7.0], [8.0, 4.0, 4.0], [8.0, 3.0, 5.0]]], [[[7.0, 7.0, 4.0], 
[1.0, 8.0, 7.0], [4.0, 2.0, 2.0]], [[6.0, 2.0, 4.0], [4.0, 6.0, 4.0], [2.0, 7.0, 5.0]], [[1.0, 8.0, 6.0], [2.0, 8.0, 2.0], 
[6.0, 2.0, 5.0]]], [[[7.0, 1.0, 4.0], [6.0, 8.0, 7.0], [3.0, 5.0, 3.0]], [[3.0, 5.0, 5.0], [3.0, 4.0, 6.0], [2.0, 8.0, 3.0]], 
[[2.0, 5.0, 1.0], [6.0, 1.0, 1.0], [1.0, 6.0, 4.0]]], [[[3.0, 1.0, 1.0], [4.0, 3.0, 5.0], [1.0, 6.0, 1.0]], [[8.0, 5.0, 4.0], 
[7.0, 1.0, 4.0], [2.0, 4.0, 4.0]], [[3.0, 1.0, 1.0], [5.0, 1.0, 4.0], [8.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 4.0], [5.0, 6.0, 8.0], [4.0, 7.0, 6.0]], [[4.0, 7.0, 5.0], [1.0, 1.0, 4.0], [5.0, 
8.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 5.0, 4.0], [7.0, 4.0, 2.0]]], [[[3.0, 7.0, 3.0], [2.0, 7.0, 3.0], [1.0, 2.0, 2.0]], [[4.0, 
7.0, 5.0], [1.0, 8.0, 5.0], [6.0, 7.0, 2.0]], [[5.0, 1.0, 2.0], [4.0, 6.0, 3.0], [3.0, 7.0, 4.0]]], [[[3.0, 3.0, 1.0], [5.0, 
6.0, 1.0], [5.0, 6.0, 4.0]], [[8.0, 5.0, 7.0], [7.0, 1.0, 5.0], [2.0, 3.0, 7.0]], [[4.0, 6.0, 4.0], [7.0, 3.0, 8.0], [5.0, 5.0, 
5.0]]], [[[8.0, 3.0, 5.0], [6.0, 5.0, 4.0], [5.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], [4.0, 4.0, 3.0], [8.0, 7.0, 4.0]], [[8.0, 7.0, 
6.0], [1.0, 6.0, 5.0], [7.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 4.0], [4.0, 2.0, 3.0], [5.0, 1.0, 8.0]], 
[[8.0, 1.0, 3.0], [7.0, 8.0, 2.0], [3.0, 3.0, 8.0]], [[1.0, 7.0, 6.0], [1.0, 2.0, 3.0], [7.0, 1.0, 4.0]]], [[[4.0, 2.0, 5.0], 
[6.0, 1.0, 5.0], [5.0, 8.0, 1.0]], [[7.0, 6.0, 8.0], [8.0, 4.0, 5.0], [2.0, 1.0, 5.0]], [[1.0, 6.0, 5.0], [6.0, 6.0, 1.0], 
[8.0, 1.0, 6.0]]], [[[2.0, 5.0, 5.0], [6.0, 6.0, 3.0], [7.0, 6.0, 6.0]], [[2.0, 6.0, 6.0], [5.0, 7.0, 8.0], [8.0, 3.0, 3.0]], 
[[8.0, 3.0, 6.0], [2.0, 5.0, 3.0], [3.0, 8.0, 6.0]]], [[[3.0, 7.0, 8.0], [4.0, 1.0, 5.0], [2.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], 
[7.0, 7.0, 8.0], [3.0, 7.0, 2.0]], [[4.0, 2.0, 2.0], [3.0, 3.0, 6.0], [2.0, 2.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 3.0, 8.0]]])+(1.-msk_val)*numpy.array([[[1.0, 6.0, 4.0]]])
      arg[2,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [5.0, 6.0, 8.0], [4.0, 7.0, 6.0]], [[4.0, 7.0, 5.0], [1.0, 1.0, 4.0], [5.0, 
8.0, 1.0]], [[4.0, 8.0, 7.0], [8.0, 5.0, 4.0], [7.0, 4.0, 2.0]]], [[[3.0, 7.0, 3.0], [2.0, 7.0, 3.0], [1.0, 2.0, 2.0]], [[4.0, 
7.0, 5.0], [1.0, 8.0, 5.0], [6.0, 7.0, 2.0]], [[5.0, 1.0, 2.0], [4.0, 6.0, 3.0], [3.0, 7.0, 4.0]]], [[[3.0, 3.0, 1.0], [5.0, 
6.0, 1.0], [5.0, 6.0, 4.0]], [[8.0, 5.0, 7.0], [7.0, 1.0, 5.0], [2.0, 3.0, 7.0]], [[4.0, 6.0, 4.0], [7.0, 3.0, 8.0], [2.0, 3.0, 
8.0]]], [[[8.0, 3.0, 5.0], [6.0, 5.0, 4.0], [5.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], [4.0, 4.0, 3.0], [8.0, 7.0, 4.0]], [[8.0, 7.0, 
6.0], [1.0, 6.0, 5.0], [7.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 4.0], [4.0, 2.0, 3.0], [5.0, 1.0, 8.0]], 
[[8.0, 1.0, 3.0], [7.0, 8.0, 2.0], [3.0, 3.0, 8.0]], [[1.0, 7.0, 6.0], [1.0, 2.0, 3.0], [7.0, 1.0, 4.0]]], [[[4.0, 2.0, 5.0], 
[6.0, 1.0, 5.0], [5.0, 8.0, 1.0]], [[7.0, 6.0, 8.0], [8.0, 4.0, 5.0], [2.0, 1.0, 5.0]], [[1.0, 6.0, 5.0], [6.0, 6.0, 1.0], 
[8.0, 1.0, 6.0]]], [[[2.0, 5.0, 5.0], [6.0, 6.0, 3.0], [7.0, 6.0, 6.0]], [[2.0, 6.0, 6.0], [5.0, 7.0, 8.0], [8.0, 3.0, 3.0]], 
[[8.0, 3.0, 6.0], [2.0, 5.0, 3.0], [1.0, 6.0, 4.0]]], [[[3.0, 7.0, 8.0], [4.0, 1.0, 5.0], [2.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], 
[7.0, 7.0, 8.0], [3.0, 7.0, 2.0]], [[4.0, 2.0, 2.0], [3.0, 3.0, 6.0], [2.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 2.0], [5.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[4.0, 4.0, 8.0], [6.0, 8.0, 8.0], [7.0, 
8.0, 3.0]], [[5.0, 7.0, 3.0], [2.0, 2.0, 5.0], [3.0, 7.0, 3.0]]], [[[6.0, 8.0, 6.0], [1.0, 5.0, 4.0], [4.0, 3.0, 2.0]], [[6.0, 
5.0, 6.0], [6.0, 6.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 5.0, 6.0], [1.0, 3.0, 4.0], [8.0, 8.0, 3.0]]], [[[1.0, 7.0, 2.0], [6.0, 
7.0, 4.0], [5.0, 1.0, 6.0]], [[3.0, 2.0, 5.0], [5.0, 4.0, 8.0], [2.0, 2.0, 2.0]], [[7.0, 7.0, 5.0], [3.0, 8.0, 3.0], [8.0, 5.0, 
2.0]]], [[[1.0, 6.0, 7.0], [1.0, 4.0, 4.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 8.0], [6.0, 3.0, 6.0], [6.0, 1.0, 1.0]], [[2.0, 6.0, 
7.0], [3.0, 4.0, 4.0], [7.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 8.0], [8.0, 6.0, 6.0], [6.0, 3.0, 1.0]], 
[[2.0, 5.0, 4.0], [5.0, 7.0, 1.0], [8.0, 2.0, 7.0]], [[4.0, 5.0, 1.0], [5.0, 8.0, 7.0], [8.0, 6.0, 2.0]]], [[[7.0, 1.0, 7.0], 
[4.0, 4.0, 2.0], [5.0, 7.0, 8.0]], [[4.0, 6.0, 8.0], [4.0, 3.0, 5.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 5.0], [8.0, 4.0, 5.0], 
[3.0, 8.0, 6.0]]], [[[3.0, 7.0, 1.0], [5.0, 1.0, 2.0], [1.0, 8.0, 5.0]], [[3.0, 5.0, 5.0], [6.0, 8.0, 2.0], [8.0, 3.0, 2.0]], 
[[8.0, 1.0, 7.0], [1.0, 5.0, 1.0], [7.0, 5.0, 2.0]]], [[[5.0, 7.0, 1.0], [7.0, 5.0, 8.0], [4.0, 8.0, 3.0]], [[6.0, 4.0, 6.0], 
[6.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 8.0, 5.0], [8.0, 2.0, 3.0]]]])
      val=numpy.array([[[3.0, 6.0, 6.0]], [[7.0, 5.0, 2.0]]])
      arg[1:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 2.0], [5.0, 3.0, 6.0], [3.0, 1.0, 3.0]], [[4.0, 4.0, 8.0], [6.0, 8.0, 8.0], [7.0, 
8.0, 3.0]], [[5.0, 7.0, 3.0], [2.0, 2.0, 5.0], [3.0, 7.0, 3.0]]], [[[6.0, 8.0, 6.0], [1.0, 5.0, 4.0], [4.0, 3.0, 2.0]], [[6.0, 
5.0, 6.0], [6.0, 6.0, 8.0], [1.0, 5.0, 5.0]], [[1.0, 5.0, 6.0], [3.0, 6.0, 6.0], [8.0, 8.0, 3.0]]], [[[1.0, 7.0, 2.0], [6.0, 
7.0, 4.0], [5.0, 1.0, 6.0]], [[3.0, 2.0, 5.0], [5.0, 4.0, 8.0], [2.0, 2.0, 2.0]], [[7.0, 7.0, 5.0], [7.0, 5.0, 2.0], [8.0, 5.0, 
2.0]]], [[[1.0, 6.0, 7.0], [1.0, 4.0, 4.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 8.0], [6.0, 3.0, 6.0], [6.0, 1.0, 1.0]], [[2.0, 6.0, 
7.0], [3.0, 4.0, 4.0], [7.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 8.0], [8.0, 6.0, 6.0], [6.0, 3.0, 1.0]], 
[[2.0, 5.0, 4.0], [5.0, 7.0, 1.0], [8.0, 2.0, 7.0]], [[4.0, 5.0, 1.0], [5.0, 8.0, 7.0], [8.0, 6.0, 2.0]]], [[[7.0, 1.0, 7.0], 
[4.0, 4.0, 2.0], [5.0, 7.0, 8.0]], [[4.0, 6.0, 8.0], [4.0, 3.0, 5.0], [8.0, 4.0, 2.0]], [[4.0, 3.0, 5.0], [3.0, 6.0, 6.0], 
[3.0, 8.0, 6.0]]], [[[3.0, 7.0, 1.0], [5.0, 1.0, 2.0], [1.0, 8.0, 5.0]], [[3.0, 5.0, 5.0], [6.0, 8.0, 2.0], [8.0, 3.0, 2.0]], 
[[8.0, 1.0, 7.0], [7.0, 5.0, 2.0], [7.0, 5.0, 2.0]]], [[[5.0, 7.0, 1.0], [7.0, 5.0, 8.0], [4.0, 8.0, 3.0]], [[6.0, 4.0, 6.0], 
[6.0, 4.0, 7.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 4.0], [5.0, 8.0, 5.0], [8.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 2.0], [6.0, 5.0, 7.0], [4.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], [1.0, 7.0, 5.0], [6.0, 
7.0, 4.0]], [[6.0, 3.0, 8.0], [8.0, 6.0, 8.0], [4.0, 2.0, 5.0]]], [[[1.0, 5.0, 5.0], [1.0, 4.0, 7.0], [3.0, 7.0, 2.0]], [[8.0, 
4.0, 6.0], [6.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 1.0, 2.0], [3.0, 7.0, 1.0], [5.0, 5.0, 1.0]]], [[[8.0, 7.0, 4.0], [7.0, 
6.0, 4.0], [3.0, 1.0, 3.0]], [[1.0, 3.0, 1.0], [7.0, 5.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 6.0], [2.0, 4.0, 4.0], [8.0, 8.0, 
2.0]]], [[[6.0, 6.0, 8.0], [8.0, 2.0, 5.0], [5.0, 1.0, 5.0]], [[6.0, 7.0, 5.0], [3.0, 4.0, 5.0], [2.0, 8.0, 8.0]], [[5.0, 5.0, 
4.0], [4.0, 8.0, 1.0], [5.0, 8.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 7.0], [6.0, 6.0, 6.0], [8.0, 1.0, 7.0]], 
[[2.0, 8.0, 8.0], [8.0, 3.0, 5.0], [2.0, 2.0, 8.0]], [[2.0, 7.0, 8.0], [6.0, 2.0, 2.0], [6.0, 8.0, 2.0]]], [[[2.0, 3.0, 8.0], 
[2.0, 3.0, 6.0], [5.0, 7.0, 2.0]], [[4.0, 4.0, 6.0], [7.0, 3.0, 1.0], [2.0, 2.0, 4.0]], [[6.0, 6.0, 8.0], [3.0, 1.0, 6.0], 
[2.0, 5.0, 2.0]]], [[[1.0, 7.0, 8.0], [6.0, 3.0, 4.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 8.0], [7.0, 4.0, 6.0], [6.0, 5.0, 2.0]], 
[[8.0, 2.0, 3.0], [8.0, 1.0, 6.0], [8.0, 4.0, 2.0]]], [[[2.0, 8.0, 1.0], [2.0, 8.0, 7.0], [5.0, 3.0, 8.0]], [[7.0, 7.0, 2.0], 
[6.0, 4.0, 2.0], [4.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [1.0, 4.0, 7.0], [5.0, 6.0, 5.0]]]])
      val=Data(numpy.array([[[6.0, 6.0, 4.0]], [[8.0, 2.0, 4.0]]]),self.functionspace)
      arg[1:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 2.0], [6.0, 5.0, 7.0], [4.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], [1.0, 7.0, 5.0], [6.0, 
7.0, 4.0]], [[6.0, 3.0, 8.0], [8.0, 6.0, 8.0], [4.0, 2.0, 5.0]]], [[[1.0, 5.0, 5.0], [1.0, 4.0, 7.0], [3.0, 7.0, 2.0]], [[8.0, 
4.0, 6.0], [6.0, 4.0, 4.0], [5.0, 6.0, 4.0]], [[5.0, 1.0, 2.0], [6.0, 6.0, 4.0], [5.0, 5.0, 1.0]]], [[[8.0, 7.0, 4.0], [7.0, 
6.0, 4.0], [3.0, 1.0, 3.0]], [[1.0, 3.0, 1.0], [7.0, 5.0, 6.0], [3.0, 5.0, 2.0]], [[5.0, 3.0, 6.0], [8.0, 2.0, 4.0], [8.0, 8.0, 
2.0]]], [[[6.0, 6.0, 8.0], [8.0, 2.0, 5.0], [5.0, 1.0, 5.0]], [[6.0, 7.0, 5.0], [3.0, 4.0, 5.0], [2.0, 8.0, 8.0]], [[5.0, 5.0, 
4.0], [4.0, 8.0, 1.0], [5.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 7.0], [6.0, 6.0, 6.0], [8.0, 1.0, 7.0]], 
[[2.0, 8.0, 8.0], [8.0, 3.0, 5.0], [2.0, 2.0, 8.0]], [[2.0, 7.0, 8.0], [6.0, 2.0, 2.0], [6.0, 8.0, 2.0]]], [[[2.0, 3.0, 8.0], 
[2.0, 3.0, 6.0], [5.0, 7.0, 2.0]], [[4.0, 4.0, 6.0], [7.0, 3.0, 1.0], [2.0, 2.0, 4.0]], [[6.0, 6.0, 8.0], [6.0, 6.0, 4.0], 
[2.0, 5.0, 2.0]]], [[[1.0, 7.0, 8.0], [6.0, 3.0, 4.0], [7.0, 6.0, 6.0]], [[3.0, 8.0, 8.0], [7.0, 4.0, 6.0], [6.0, 5.0, 2.0]], 
[[8.0, 2.0, 3.0], [8.0, 2.0, 4.0], [8.0, 4.0, 2.0]]], [[[2.0, 8.0, 1.0], [2.0, 8.0, 7.0], [5.0, 3.0, 8.0]], [[7.0, 7.0, 2.0], 
[6.0, 4.0, 2.0], [4.0, 3.0, 4.0]], [[2.0, 2.0, 5.0], [1.0, 4.0, 7.0], [5.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 8.0], [7.0, 7.0, 6.0], [8.0, 8.0, 6.0]], [[7.0, 6.0, 4.0], [4.0, 2.0, 1.0], [8.0, 
5.0, 5.0]], [[8.0, 5.0, 8.0], [3.0, 4.0, 1.0], [5.0, 4.0, 8.0]]], [[[6.0, 1.0, 6.0], [6.0, 7.0, 1.0], [7.0, 2.0, 2.0]], [[2.0, 
8.0, 8.0], [3.0, 2.0, 8.0], [5.0, 2.0, 4.0]], [[3.0, 6.0, 6.0], [4.0, 8.0, 1.0], [4.0, 7.0, 6.0]]], [[[4.0, 8.0, 1.0], [2.0, 
8.0, 6.0], [1.0, 1.0, 6.0]], [[2.0, 6.0, 2.0], [1.0, 3.0, 6.0], [2.0, 6.0, 5.0]], [[2.0, 8.0, 1.0], [7.0, 5.0, 8.0], [4.0, 1.0, 
5.0]]], [[[3.0, 8.0, 5.0], [7.0, 2.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[8.0, 6.0, 
4.0], [5.0, 6.0, 8.0], [8.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 3.0], [5.0, 4.0, 6.0], [5.0, 7.0, 2.0]], 
[[8.0, 5.0, 2.0], [4.0, 6.0, 7.0], [7.0, 6.0, 6.0]], [[2.0, 4.0, 2.0], [5.0, 3.0, 3.0], [8.0, 6.0, 6.0]]], [[[8.0, 7.0, 7.0], 
[7.0, 8.0, 3.0], [7.0, 2.0, 5.0]], [[6.0, 3.0, 1.0], [6.0, 2.0, 1.0], [1.0, 4.0, 8.0]], [[3.0, 8.0, 2.0], [4.0, 5.0, 2.0], 
[3.0, 2.0, 5.0]]], [[[1.0, 4.0, 5.0], [3.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[1.0, 8.0, 3.0], [3.0, 2.0, 5.0], [8.0, 3.0, 4.0]], 
[[7.0, 8.0, 2.0], [1.0, 1.0, 4.0], [8.0, 5.0, 4.0]]], [[[6.0, 5.0, 8.0], [1.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[1.0, 5.0, 5.0], 
[6.0, 1.0, 3.0], [3.0, 5.0, 6.0]], [[5.0, 3.0, 4.0], [4.0, 2.0, 8.0], [8.0, 2.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 2.0, 3.0]], [[1.0, 2.0, 4.0]]])+(1.-msk_val)*numpy.array([[[6.0, 3.0, 6.0]], [[3.0, 1.0, 
3.0]]])
      arg[1:3,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 8.0], [7.0, 7.0, 6.0], [8.0, 8.0, 6.0]], [[7.0, 6.0, 4.0], [4.0, 2.0, 1.0], [8.0, 
5.0, 5.0]], [[8.0, 5.0, 8.0], [3.0, 4.0, 1.0], [5.0, 4.0, 8.0]]], [[[6.0, 1.0, 6.0], [6.0, 7.0, 1.0], [7.0, 2.0, 2.0]], [[2.0, 
8.0, 8.0], [3.0, 2.0, 8.0], [5.0, 2.0, 4.0]], [[3.0, 6.0, 6.0], [3.0, 2.0, 3.0], [4.0, 7.0, 6.0]]], [[[4.0, 8.0, 1.0], [2.0, 
8.0, 6.0], [1.0, 1.0, 6.0]], [[2.0, 6.0, 2.0], [1.0, 3.0, 6.0], [2.0, 6.0, 5.0]], [[2.0, 8.0, 1.0], [1.0, 2.0, 4.0], [4.0, 1.0, 
5.0]]], [[[3.0, 8.0, 5.0], [7.0, 2.0, 4.0], [6.0, 1.0, 1.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 1.0], [3.0, 8.0, 1.0]], [[8.0, 6.0, 
4.0], [5.0, 6.0, 8.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [5.0, 4.0, 6.0], [5.0, 7.0, 2.0]], 
[[8.0, 5.0, 2.0], [4.0, 6.0, 7.0], [7.0, 6.0, 6.0]], [[2.0, 4.0, 2.0], [5.0, 3.0, 3.0], [8.0, 6.0, 6.0]]], [[[8.0, 7.0, 7.0], 
[7.0, 8.0, 3.0], [7.0, 2.0, 5.0]], [[6.0, 3.0, 1.0], [6.0, 2.0, 1.0], [1.0, 4.0, 8.0]], [[3.0, 8.0, 2.0], [6.0, 3.0, 6.0], 
[3.0, 2.0, 5.0]]], [[[1.0, 4.0, 5.0], [3.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[1.0, 8.0, 3.0], [3.0, 2.0, 5.0], [8.0, 3.0, 4.0]], 
[[7.0, 8.0, 2.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]]], [[[6.0, 5.0, 8.0], [1.0, 6.0, 4.0], [4.0, 7.0, 3.0]], [[1.0, 5.0, 5.0], 
[6.0, 1.0, 3.0], [3.0, 5.0, 6.0]], [[5.0, 3.0, 4.0], [4.0, 2.0, 8.0], [8.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 1.0], [5.0, 5.0, 8.0], [3.0, 7.0, 2.0]], [[6.0, 6.0, 4.0], [3.0, 6.0, 8.0], [3.0, 
3.0, 1.0]], [[8.0, 7.0, 7.0], [5.0, 7.0, 4.0], [4.0, 2.0, 6.0]]], [[[5.0, 8.0, 4.0], [1.0, 8.0, 5.0], [3.0, 1.0, 4.0]], [[8.0, 
2.0, 1.0], [3.0, 1.0, 6.0], [1.0, 4.0, 7.0]], [[6.0, 8.0, 7.0], [6.0, 8.0, 5.0], [6.0, 4.0, 5.0]]], [[[3.0, 1.0, 4.0], [4.0, 
7.0, 1.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 7.0], [4.0, 8.0, 2.0], [5.0, 5.0, 4.0]], [[5.0, 3.0, 8.0], [8.0, 6.0, 5.0], [1.0, 8.0, 
1.0]]], [[[2.0, 8.0, 4.0], [1.0, 8.0, 4.0], [6.0, 1.0, 2.0]], [[3.0, 3.0, 7.0], [3.0, 1.0, 4.0], [6.0, 7.0, 6.0]], [[8.0, 4.0, 
2.0], [2.0, 6.0, 8.0], [3.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 3.0], [8.0, 2.0, 7.0], [2.0, 1.0, 6.0]], 
[[1.0, 4.0, 6.0], [2.0, 8.0, 5.0], [8.0, 3.0, 2.0]], [[6.0, 4.0, 1.0], [5.0, 4.0, 2.0], [4.0, 5.0, 1.0]]], [[[8.0, 2.0, 2.0], 
[6.0, 6.0, 1.0], [4.0, 1.0, 7.0]], [[1.0, 3.0, 1.0], [8.0, 5.0, 3.0], [7.0, 7.0, 6.0]], [[4.0, 5.0, 6.0], [5.0, 3.0, 1.0], 
[3.0, 6.0, 4.0]]], [[[7.0, 6.0, 3.0], [6.0, 5.0, 2.0], [1.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [7.0, 4.0, 8.0], [8.0, 6.0, 6.0]], 
[[8.0, 7.0, 4.0], [2.0, 2.0, 5.0], [5.0, 7.0, 4.0]]], [[[7.0, 5.0, 2.0], [4.0, 4.0, 3.0], [3.0, 3.0, 4.0]], [[7.0, 7.0, 2.0], 
[6.0, 5.0, 2.0], [3.0, 8.0, 3.0]], [[2.0, 3.0, 5.0], [1.0, 8.0, 4.0], [3.0, 3.0, 6.0]]]])
      val=numpy.array([[[[1.0, 2.0, 1.0]]], [[[2.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0]]]])
      arg[:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [5.0, 5.0, 8.0], [3.0, 7.0, 2.0]], [[6.0, 6.0, 4.0], [1.0, 2.0, 1.0], [3.0, 
3.0, 1.0]], [[8.0, 7.0, 7.0], [5.0, 7.0, 4.0], [4.0, 2.0, 6.0]]], [[[5.0, 8.0, 4.0], [1.0, 8.0, 5.0], [3.0, 1.0, 4.0]], [[8.0, 
2.0, 1.0], [2.0, 2.0, 4.0], [1.0, 4.0, 7.0]], [[6.0, 8.0, 7.0], [6.0, 8.0, 5.0], [6.0, 4.0, 5.0]]], [[[3.0, 1.0, 4.0], [4.0, 
7.0, 1.0], [3.0, 1.0, 5.0]], [[4.0, 8.0, 7.0], [1.0, 8.0, 4.0], [5.0, 5.0, 4.0]], [[5.0, 3.0, 8.0], [8.0, 6.0, 5.0], [1.0, 8.0, 
1.0]]], [[[2.0, 8.0, 4.0], [1.0, 8.0, 4.0], [6.0, 1.0, 2.0]], [[3.0, 3.0, 7.0], [3.0, 1.0, 4.0], [6.0, 7.0, 6.0]], [[8.0, 4.0, 
2.0], [2.0, 6.0, 8.0], [3.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 3.0], [8.0, 2.0, 7.0], [2.0, 1.0, 6.0]], 
[[1.0, 4.0, 6.0], [1.0, 2.0, 1.0], [8.0, 3.0, 2.0]], [[6.0, 4.0, 1.0], [5.0, 4.0, 2.0], [4.0, 5.0, 1.0]]], [[[8.0, 2.0, 2.0], 
[6.0, 6.0, 1.0], [4.0, 1.0, 7.0]], [[1.0, 3.0, 1.0], [2.0, 2.0, 4.0], [7.0, 7.0, 6.0]], [[4.0, 5.0, 6.0], [5.0, 3.0, 1.0], 
[3.0, 6.0, 4.0]]], [[[7.0, 6.0, 3.0], [6.0, 5.0, 2.0], [1.0, 1.0, 5.0]], [[3.0, 7.0, 6.0], [1.0, 8.0, 4.0], [8.0, 6.0, 6.0]], 
[[8.0, 7.0, 4.0], [2.0, 2.0, 5.0], [5.0, 7.0, 4.0]]], [[[7.0, 5.0, 2.0], [4.0, 4.0, 3.0], [3.0, 3.0, 4.0]], [[7.0, 7.0, 2.0], 
[6.0, 5.0, 2.0], [3.0, 8.0, 3.0]], [[2.0, 3.0, 5.0], [1.0, 8.0, 4.0], [3.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 3.0], [8.0, 8.0, 7.0], [4.0, 4.0, 7.0]], [[4.0, 8.0, 8.0], [8.0, 4.0, 8.0], [6.0, 
4.0, 4.0]], [[1.0, 1.0, 8.0], [6.0, 8.0, 5.0], [5.0, 2.0, 5.0]]], [[[5.0, 1.0, 6.0], [4.0, 2.0, 5.0], [2.0, 8.0, 2.0]], [[4.0, 
4.0, 4.0], [2.0, 2.0, 8.0], [2.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [8.0, 4.0, 2.0], [8.0, 8.0, 6.0]]], [[[2.0, 7.0, 5.0], [1.0, 
4.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 7.0, 4.0], [4.0, 6.0, 7.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 6.0], [4.0, 3.0, 8.0], [4.0, 6.0, 
6.0]]], [[[4.0, 7.0, 8.0], [3.0, 2.0, 4.0], [3.0, 1.0, 4.0]], [[6.0, 8.0, 7.0], [2.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 4.0, 
6.0], [2.0, 5.0, 8.0], [3.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 2.0], [4.0, 5.0, 3.0], [3.0, 5.0, 4.0]], 
[[1.0, 2.0, 7.0], [6.0, 5.0, 8.0], [1.0, 6.0, 3.0]], [[8.0, 8.0, 7.0], [5.0, 3.0, 8.0], [3.0, 1.0, 7.0]]], [[[1.0, 7.0, 2.0], 
[7.0, 5.0, 7.0], [1.0, 7.0, 5.0]], [[6.0, 1.0, 1.0], [3.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[1.0, 1.0, 7.0], [2.0, 4.0, 7.0], 
[3.0, 8.0, 5.0]]], [[[3.0, 4.0, 8.0], [3.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[8.0, 7.0, 6.0], [5.0, 5.0, 4.0], [6.0, 5.0, 1.0]], 
[[7.0, 6.0, 6.0], [5.0, 3.0, 4.0], [6.0, 7.0, 4.0]]], [[[8.0, 1.0, 2.0], [3.0, 2.0, 5.0], [7.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], 
[6.0, 5.0, 7.0], [7.0, 4.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 5.0, 4.0], [8.0, 2.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0, 5.0, 3.0]]], [[[8.0, 3.0, 1.0]]], [[[2.0, 1.0, 6.0]]]]),self.functionspace)
      arg[:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 3.0], [8.0, 8.0, 7.0], [4.0, 4.0, 7.0]], [[4.0, 8.0, 8.0], [6.0, 5.0, 3.0], [6.0, 
4.0, 4.0]], [[1.0, 1.0, 8.0], [6.0, 8.0, 5.0], [5.0, 2.0, 5.0]]], [[[5.0, 1.0, 6.0], [4.0, 2.0, 5.0], [2.0, 8.0, 2.0]], [[4.0, 
4.0, 4.0], [8.0, 3.0, 1.0], [2.0, 7.0, 8.0]], [[4.0, 7.0, 3.0], [8.0, 4.0, 2.0], [8.0, 8.0, 6.0]]], [[[2.0, 7.0, 5.0], [1.0, 
4.0, 7.0], [7.0, 1.0, 1.0]], [[5.0, 7.0, 4.0], [2.0, 1.0, 6.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 6.0], [4.0, 3.0, 8.0], [4.0, 6.0, 
6.0]]], [[[4.0, 7.0, 8.0], [3.0, 2.0, 4.0], [3.0, 1.0, 4.0]], [[6.0, 8.0, 7.0], [2.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 4.0, 
6.0], [2.0, 5.0, 8.0], [3.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 2.0], [4.0, 5.0, 3.0], [3.0, 5.0, 4.0]], 
[[1.0, 2.0, 7.0], [6.0, 5.0, 3.0], [1.0, 6.0, 3.0]], [[8.0, 8.0, 7.0], [5.0, 3.0, 8.0], [3.0, 1.0, 7.0]]], [[[1.0, 7.0, 2.0], 
[7.0, 5.0, 7.0], [1.0, 7.0, 5.0]], [[6.0, 1.0, 1.0], [8.0, 3.0, 1.0], [7.0, 7.0, 3.0]], [[1.0, 1.0, 7.0], [2.0, 4.0, 7.0], 
[3.0, 8.0, 5.0]]], [[[3.0, 4.0, 8.0], [3.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[8.0, 7.0, 6.0], [2.0, 1.0, 6.0], [6.0, 5.0, 1.0]], 
[[7.0, 6.0, 6.0], [5.0, 3.0, 4.0], [6.0, 7.0, 4.0]]], [[[8.0, 1.0, 2.0], [3.0, 2.0, 5.0], [7.0, 8.0, 8.0]], [[7.0, 4.0, 1.0], 
[6.0, 5.0, 7.0], [7.0, 4.0, 7.0]], [[4.0, 4.0, 4.0], [4.0, 5.0, 4.0], [8.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 5.0, 4.0], [1.0, 5.0, 2.0]], [[4.0, 7.0, 2.0], [3.0, 3.0, 6.0], [8.0, 
5.0, 3.0]], [[4.0, 2.0, 8.0], [8.0, 4.0, 6.0], [1.0, 8.0, 2.0]]], [[[4.0, 5.0, 1.0], [8.0, 6.0, 8.0], [5.0, 1.0, 2.0]], [[7.0, 
3.0, 7.0], [4.0, 7.0, 1.0], [4.0, 3.0, 6.0]], [[4.0, 7.0, 7.0], [3.0, 6.0, 3.0], [7.0, 7.0, 5.0]]], [[[4.0, 8.0, 2.0], [6.0, 
6.0, 3.0], [4.0, 7.0, 4.0]], [[7.0, 3.0, 4.0], [2.0, 4.0, 1.0], [2.0, 7.0, 4.0]], [[5.0, 1.0, 4.0], [5.0, 2.0, 4.0], [3.0, 3.0, 
3.0]]], [[[3.0, 4.0, 1.0], [3.0, 2.0, 7.0], [4.0, 7.0, 1.0]], [[4.0, 6.0, 2.0], [7.0, 2.0, 7.0], [2.0, 5.0, 4.0]], [[7.0, 1.0, 
2.0], [1.0, 3.0, 4.0], [8.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 8.0], [7.0, 7.0, 8.0], [2.0, 5.0, 8.0]], 
[[1.0, 4.0, 2.0], [8.0, 3.0, 8.0], [5.0, 8.0, 6.0]], [[6.0, 7.0, 6.0], [1.0, 5.0, 3.0], [7.0, 4.0, 6.0]]], [[[6.0, 7.0, 4.0], 
[3.0, 5.0, 5.0], [3.0, 6.0, 7.0]], [[4.0, 4.0, 7.0], [5.0, 7.0, 6.0], [5.0, 3.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 1.0, 3.0], 
[4.0, 6.0, 2.0]]], [[[3.0, 2.0, 2.0], [6.0, 7.0, 6.0], [1.0, 5.0, 6.0]], [[7.0, 8.0, 3.0], [3.0, 6.0, 5.0], [4.0, 3.0, 2.0]], 
[[4.0, 5.0, 5.0], [3.0, 2.0, 2.0], [4.0, 4.0, 5.0]]], [[[3.0, 6.0, 4.0], [6.0, 2.0, 7.0], [1.0, 1.0, 6.0]], [[1.0, 4.0, 4.0], 
[6.0, 4.0, 5.0], [6.0, 2.0, 3.0]], [[7.0, 2.0, 2.0], [5.0, 7.0, 2.0], [4.0, 5.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 7.0, 1.0]]], [[[6.0, 7.0, 3.0]]], [[[6.0, 2.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 3.0, 7.0]]], [[[6.0, 6.0, 6.0]]], [[[6.0, 3.0, 2.0]]]])
      arg[:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 5.0, 4.0], [1.0, 5.0, 2.0]], [[4.0, 7.0, 2.0], [6.0, 7.0, 1.0], [8.0, 
5.0, 3.0]], [[4.0, 2.0, 8.0], [8.0, 4.0, 6.0], [1.0, 8.0, 2.0]]], [[[4.0, 5.0, 1.0], [8.0, 6.0, 8.0], [5.0, 1.0, 2.0]], [[7.0, 
3.0, 7.0], [6.0, 7.0, 3.0], [4.0, 3.0, 6.0]], [[4.0, 7.0, 7.0], [3.0, 6.0, 3.0], [7.0, 7.0, 5.0]]], [[[4.0, 8.0, 2.0], [6.0, 
6.0, 3.0], [4.0, 7.0, 4.0]], [[7.0, 3.0, 4.0], [6.0, 2.0, 8.0], [2.0, 7.0, 4.0]], [[5.0, 1.0, 4.0], [5.0, 2.0, 4.0], [3.0, 3.0, 
3.0]]], [[[3.0, 4.0, 1.0], [3.0, 2.0, 7.0], [4.0, 7.0, 1.0]], [[4.0, 6.0, 2.0], [7.0, 2.0, 7.0], [2.0, 5.0, 4.0]], [[7.0, 1.0, 
2.0], [1.0, 3.0, 4.0], [8.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 8.0], [7.0, 7.0, 8.0], [2.0, 5.0, 8.0]], 
[[1.0, 4.0, 2.0], [4.0, 3.0, 7.0], [5.0, 8.0, 6.0]], [[6.0, 7.0, 6.0], [1.0, 5.0, 3.0], [7.0, 4.0, 6.0]]], [[[6.0, 7.0, 4.0], 
[3.0, 5.0, 5.0], [3.0, 6.0, 7.0]], [[4.0, 4.0, 7.0], [6.0, 6.0, 6.0], [5.0, 3.0, 8.0]], [[1.0, 7.0, 4.0], [8.0, 1.0, 3.0], 
[4.0, 6.0, 2.0]]], [[[3.0, 2.0, 2.0], [6.0, 7.0, 6.0], [1.0, 5.0, 6.0]], [[7.0, 8.0, 3.0], [6.0, 3.0, 2.0], [4.0, 3.0, 2.0]], 
[[4.0, 5.0, 5.0], [3.0, 2.0, 2.0], [4.0, 4.0, 5.0]]], [[[3.0, 6.0, 4.0], [6.0, 2.0, 7.0], [1.0, 1.0, 6.0]], [[1.0, 4.0, 4.0], 
[6.0, 4.0, 5.0], [6.0, 2.0, 3.0]], [[7.0, 2.0, 2.0], [5.0, 7.0, 2.0], [4.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 8.0], [3.0, 8.0, 1.0], [4.0, 7.0, 5.0]], [[7.0, 7.0, 6.0], [3.0, 4.0, 1.0], [8.0, 
8.0, 7.0]], [[6.0, 4.0, 3.0], [6.0, 3.0, 2.0], [3.0, 5.0, 2.0]]], [[[6.0, 7.0, 1.0], [6.0, 7.0, 3.0], [7.0, 2.0, 6.0]], [[4.0, 
6.0, 2.0], [7.0, 2.0, 5.0], [7.0, 2.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 6.0, 8.0], [2.0, 1.0, 6.0]]], [[[6.0, 3.0, 8.0], [5.0, 
7.0, 2.0], [3.0, 8.0, 7.0]], [[7.0, 5.0, 3.0], [3.0, 5.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 7.0, 8.0], [2.0, 1.0, 3.0], [7.0, 1.0, 
2.0]]], [[[5.0, 3.0, 5.0], [2.0, 6.0, 1.0], [2.0, 1.0, 4.0]], [[3.0, 5.0, 8.0], [2.0, 8.0, 3.0], [6.0, 7.0, 2.0]], [[7.0, 7.0, 
2.0], [1.0, 1.0, 8.0], [2.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 4.0], [7.0, 1.0, 4.0], [5.0, 8.0, 7.0]], 
[[2.0, 5.0, 1.0], [8.0, 5.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 6.0], [7.0, 5.0, 2.0]]], [[[2.0, 4.0, 5.0], 
[8.0, 6.0, 1.0], [8.0, 6.0, 4.0]], [[8.0, 6.0, 1.0], [2.0, 3.0, 4.0], [4.0, 3.0, 6.0]], [[5.0, 7.0, 5.0], [3.0, 8.0, 1.0], 
[4.0, 2.0, 2.0]]], [[[4.0, 1.0, 6.0], [6.0, 8.0, 1.0], [7.0, 3.0, 8.0]], [[2.0, 5.0, 3.0], [4.0, 7.0, 6.0], [3.0, 8.0, 3.0]], 
[[8.0, 8.0, 2.0], [6.0, 7.0, 6.0], [6.0, 8.0, 8.0]]], [[[8.0, 2.0, 8.0], [4.0, 6.0, 1.0], [8.0, 1.0, 2.0]], [[5.0, 4.0, 2.0], 
[5.0, 8.0, 1.0], [7.0, 1.0, 7.0]], [[4.0, 6.0, 4.0], [1.0, 1.0, 4.0], [1.0, 6.0, 1.0]]]])
      val=numpy.array([[[[2.0, 7.0], [5.0, 1.0]], [[4.0, 2.0], [8.0, 4.0]]], [[[2.0, 6.0], [2.0, 1.0]], [[1.0, 3.0], [8.0, 
3.0]]], [[[3.0, 3.0], [8.0, 4.0]], [[4.0, 7.0], [6.0, 2.0]]], [[[4.0, 1.0], [8.0, 7.0]], [[2.0, 6.0], [7.0, 4.0]]]])
      arg[:,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [5.0, 1.0, 1.0], [4.0, 7.0, 5.0]], [[4.0, 2.0, 6.0], [8.0, 4.0, 1.0], [8.0, 
8.0, 7.0]], [[6.0, 4.0, 3.0], [6.0, 3.0, 2.0], [3.0, 5.0, 2.0]]], [[[2.0, 6.0, 1.0], [2.0, 1.0, 3.0], [7.0, 2.0, 6.0]], [[1.0, 
3.0, 2.0], [8.0, 3.0, 5.0], [7.0, 2.0, 5.0]], [[7.0, 4.0, 3.0], [3.0, 6.0, 8.0], [2.0, 1.0, 6.0]]], [[[3.0, 3.0, 8.0], [8.0, 
4.0, 2.0], [3.0, 8.0, 7.0]], [[4.0, 7.0, 3.0], [6.0, 2.0, 1.0], [1.0, 5.0, 7.0]], [[1.0, 7.0, 8.0], [2.0, 1.0, 3.0], [7.0, 1.0, 
2.0]]], [[[4.0, 1.0, 5.0], [8.0, 7.0, 1.0], [2.0, 1.0, 4.0]], [[2.0, 6.0, 8.0], [7.0, 4.0, 3.0], [6.0, 7.0, 2.0]], [[7.0, 7.0, 
2.0], [1.0, 1.0, 8.0], [2.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 4.0], [5.0, 1.0, 4.0], [5.0, 8.0, 7.0]], 
[[4.0, 2.0, 1.0], [8.0, 4.0, 1.0], [8.0, 3.0, 5.0]], [[6.0, 2.0, 5.0], [5.0, 3.0, 6.0], [7.0, 5.0, 2.0]]], [[[2.0, 6.0, 5.0], 
[2.0, 1.0, 1.0], [8.0, 6.0, 4.0]], [[1.0, 3.0, 1.0], [8.0, 3.0, 4.0], [4.0, 3.0, 6.0]], [[5.0, 7.0, 5.0], [3.0, 8.0, 1.0], 
[4.0, 2.0, 2.0]]], [[[3.0, 3.0, 6.0], [8.0, 4.0, 1.0], [7.0, 3.0, 8.0]], [[4.0, 7.0, 3.0], [6.0, 2.0, 6.0], [3.0, 8.0, 3.0]], 
[[8.0, 8.0, 2.0], [6.0, 7.0, 6.0], [6.0, 8.0, 8.0]]], [[[4.0, 1.0, 8.0], [8.0, 7.0, 1.0], [8.0, 1.0, 2.0]], [[2.0, 6.0, 2.0], 
[7.0, 4.0, 1.0], [7.0, 1.0, 7.0]], [[4.0, 6.0, 4.0], [1.0, 1.0, 4.0], [1.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 4.0], [2.0, 6.0, 3.0], [8.0, 1.0, 5.0]], [[1.0, 3.0, 8.0], [3.0, 1.0, 2.0], [2.0, 
3.0, 4.0]], [[6.0, 1.0, 7.0], [7.0, 3.0, 7.0], [3.0, 5.0, 2.0]]], [[[5.0, 4.0, 7.0], [7.0, 6.0, 2.0], [4.0, 5.0, 5.0]], [[1.0, 
5.0, 8.0], [6.0, 3.0, 8.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 7.0], [2.0, 4.0, 6.0], [5.0, 3.0, 2.0]]], [[[7.0, 6.0, 1.0], [1.0, 
7.0, 1.0], [6.0, 3.0, 3.0]], [[3.0, 5.0, 7.0], [5.0, 1.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 5.0, 6.0], [5.0, 1.0, 7.0], [4.0, 1.0, 
7.0]]], [[[2.0, 4.0, 3.0], [7.0, 7.0, 1.0], [1.0, 3.0, 6.0]], [[3.0, 8.0, 3.0], [8.0, 1.0, 4.0], [8.0, 8.0, 2.0]], [[4.0, 4.0, 
7.0], [4.0, 7.0, 8.0], [3.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 6.0], [4.0, 3.0, 5.0], [2.0, 1.0, 2.0]], 
[[8.0, 6.0, 4.0], [8.0, 4.0, 5.0], [8.0, 4.0, 5.0]], [[1.0, 4.0, 8.0], [1.0, 6.0, 7.0], [7.0, 7.0, 2.0]]], [[[2.0, 6.0, 2.0], 
[8.0, 5.0, 7.0], [3.0, 1.0, 8.0]], [[6.0, 2.0, 8.0], [6.0, 8.0, 6.0], [5.0, 1.0, 2.0]], [[2.0, 2.0, 2.0], [4.0, 6.0, 1.0], 
[1.0, 8.0, 6.0]]], [[[2.0, 2.0, 2.0], [1.0, 4.0, 1.0], [7.0, 2.0, 2.0]], [[4.0, 7.0, 6.0], [5.0, 8.0, 8.0], [7.0, 5.0, 3.0]], 
[[7.0, 3.0, 6.0], [5.0, 1.0, 2.0], [5.0, 7.0, 4.0]]], [[[5.0, 5.0, 4.0], [2.0, 7.0, 1.0], [3.0, 3.0, 5.0]], [[3.0, 1.0, 6.0], 
[7.0, 1.0, 5.0], [5.0, 1.0, 6.0]], [[8.0, 6.0, 1.0], [3.0, 4.0, 8.0], [6.0, 7.0, 2.0]]]])
      val=Data(numpy.array([[[[3.0, 4.0], [7.0, 2.0]], [[4.0, 2.0], [1.0, 6.0]]], [[[8.0, 6.0], [1.0, 2.0]], [[8.0, 1.0], [8.0, 
4.0]]], [[[3.0, 4.0], [1.0, 3.0]], [[2.0, 2.0], [4.0, 7.0]]], [[[2.0, 4.0], [8.0, 7.0]], [[5.0, 8.0], [3.0, 
2.0]]]]),self.functionspace)
      arg[:,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 4.0], [7.0, 2.0, 3.0], [8.0, 1.0, 5.0]], [[4.0, 2.0, 8.0], [1.0, 6.0, 2.0], [2.0, 
3.0, 4.0]], [[6.0, 1.0, 7.0], [7.0, 3.0, 7.0], [3.0, 5.0, 2.0]]], [[[8.0, 6.0, 7.0], [1.0, 2.0, 2.0], [4.0, 5.0, 5.0]], [[8.0, 
1.0, 8.0], [8.0, 4.0, 8.0], [2.0, 7.0, 2.0]], [[3.0, 7.0, 7.0], [2.0, 4.0, 6.0], [5.0, 3.0, 2.0]]], [[[3.0, 4.0, 1.0], [1.0, 
3.0, 1.0], [6.0, 3.0, 3.0]], [[2.0, 2.0, 7.0], [4.0, 7.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 5.0, 6.0], [5.0, 1.0, 7.0], [4.0, 1.0, 
7.0]]], [[[2.0, 4.0, 3.0], [8.0, 7.0, 1.0], [1.0, 3.0, 6.0]], [[5.0, 8.0, 3.0], [3.0, 2.0, 4.0], [8.0, 8.0, 2.0]], [[4.0, 4.0, 
7.0], [4.0, 7.0, 8.0], [3.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 6.0], [7.0, 2.0, 5.0], [2.0, 1.0, 2.0]], 
[[4.0, 2.0, 4.0], [1.0, 6.0, 5.0], [8.0, 4.0, 5.0]], [[1.0, 4.0, 8.0], [1.0, 6.0, 7.0], [7.0, 7.0, 2.0]]], [[[8.0, 6.0, 2.0], 
[1.0, 2.0, 7.0], [3.0, 1.0, 8.0]], [[8.0, 1.0, 8.0], [8.0, 4.0, 6.0], [5.0, 1.0, 2.0]], [[2.0, 2.0, 2.0], [4.0, 6.0, 1.0], 
[1.0, 8.0, 6.0]]], [[[3.0, 4.0, 2.0], [1.0, 3.0, 1.0], [7.0, 2.0, 2.0]], [[2.0, 2.0, 6.0], [4.0, 7.0, 8.0], [7.0, 5.0, 3.0]], 
[[7.0, 3.0, 6.0], [5.0, 1.0, 2.0], [5.0, 7.0, 4.0]]], [[[2.0, 4.0, 4.0], [8.0, 7.0, 1.0], [3.0, 3.0, 5.0]], [[5.0, 8.0, 6.0], 
[3.0, 2.0, 5.0], [5.0, 1.0, 6.0]], [[8.0, 6.0, 1.0], [3.0, 4.0, 8.0], [6.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 2.0], [6.0, 2.0, 8.0], [6.0, 6.0, 8.0]], [[6.0, 5.0, 8.0], [2.0, 6.0, 1.0], [2.0, 
2.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 7.0, 6.0], [1.0, 5.0, 7.0]]], [[[2.0, 7.0, 2.0], [3.0, 1.0, 5.0], [6.0, 3.0, 2.0]], [[5.0, 
5.0, 3.0], [2.0, 8.0, 4.0], [1.0, 1.0, 4.0]], [[6.0, 2.0, 6.0], [1.0, 8.0, 4.0], [1.0, 4.0, 7.0]]], [[[7.0, 2.0, 1.0], [2.0, 
8.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 4.0, 5.0], [1.0, 1.0, 2.0], [2.0, 7.0, 8.0]], [[8.0, 8.0, 8.0], [4.0, 1.0, 8.0], [8.0, 4.0, 
3.0]]], [[[4.0, 3.0, 6.0], [1.0, 3.0, 3.0], [1.0, 4.0, 2.0]], [[8.0, 8.0, 7.0], [4.0, 7.0, 6.0], [5.0, 7.0, 8.0]], [[4.0, 4.0, 
6.0], [2.0, 3.0, 8.0], [1.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 8.0], [8.0, 1.0, 2.0], [3.0, 7.0, 5.0]], 
[[1.0, 7.0, 1.0], [6.0, 2.0, 5.0], [8.0, 4.0, 1.0]], [[6.0, 5.0, 3.0], [5.0, 5.0, 1.0], [1.0, 6.0, 8.0]]], [[[2.0, 5.0, 2.0], 
[5.0, 6.0, 7.0], [2.0, 4.0, 2.0]], [[8.0, 3.0, 3.0], [1.0, 4.0, 1.0], [2.0, 6.0, 5.0]], [[8.0, 1.0, 2.0], [4.0, 3.0, 8.0], 
[7.0, 2.0, 5.0]]], [[[4.0, 4.0, 6.0], [6.0, 1.0, 2.0], [6.0, 1.0, 8.0]], [[4.0, 8.0, 5.0], [3.0, 3.0, 2.0], [4.0, 6.0, 2.0]], 
[[2.0, 3.0, 8.0], [5.0, 4.0, 7.0], [6.0, 2.0, 4.0]]], [[[8.0, 8.0, 5.0], [2.0, 4.0, 8.0], [2.0, 5.0, 7.0]], [[5.0, 5.0, 4.0], 
[2.0, 1.0, 5.0], [8.0, 2.0, 6.0]], [[4.0, 1.0, 1.0], [3.0, 3.0, 1.0], [4.0, 8.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 2.0], [1.0, 5.0]], [[7.0, 8.0], [5.0, 7.0]]], [[[8.0, 4.0], [4.0, 8.0]], [[3.0, 4.0], 
[6.0, 5.0]]], [[[8.0, 3.0], [7.0, 3.0]], [[5.0, 4.0], [4.0, 1.0]]], [[[5.0, 3.0], [5.0, 3.0]], [[3.0, 3.0], [4.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 4.0], [8.0, 2.0]], [[6.0, 6.0], [6.0, 4.0]]], [[[4.0, 7.0], [8.0, 1.0]], [[5.0, 
6.0], [6.0, 7.0]]], [[[3.0, 7.0], [3.0, 7.0]], [[5.0, 1.0], [5.0, 1.0]]], [[[8.0, 3.0], [3.0, 6.0]], [[8.0, 5.0], [1.0, 2.0]]]])
      arg[:,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 2.0], [1.0, 5.0, 8.0], [6.0, 6.0, 8.0]], [[7.0, 8.0, 8.0], [5.0, 7.0, 1.0], [2.0, 
2.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 7.0, 6.0], [1.0, 5.0, 7.0]]], [[[8.0, 4.0, 2.0], [4.0, 8.0, 5.0], [6.0, 3.0, 2.0]], [[3.0, 
4.0, 3.0], [6.0, 5.0, 4.0], [1.0, 1.0, 4.0]], [[6.0, 2.0, 6.0], [1.0, 8.0, 4.0], [1.0, 4.0, 7.0]]], [[[8.0, 3.0, 1.0], [7.0, 
3.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 4.0, 5.0], [4.0, 1.0, 2.0], [2.0, 7.0, 8.0]], [[8.0, 8.0, 8.0], [4.0, 1.0, 8.0], [8.0, 4.0, 
3.0]]], [[[5.0, 3.0, 6.0], [5.0, 3.0, 3.0], [1.0, 4.0, 2.0]], [[3.0, 3.0, 7.0], [4.0, 8.0, 6.0], [5.0, 7.0, 8.0]], [[4.0, 4.0, 
6.0], [2.0, 3.0, 8.0], [1.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 8.0], [8.0, 2.0, 2.0], [3.0, 7.0, 5.0]], 
[[6.0, 6.0, 1.0], [6.0, 4.0, 5.0], [8.0, 4.0, 1.0]], [[6.0, 5.0, 3.0], [5.0, 5.0, 1.0], [1.0, 6.0, 8.0]]], [[[4.0, 7.0, 2.0], 
[8.0, 1.0, 7.0], [2.0, 4.0, 2.0]], [[5.0, 6.0, 3.0], [6.0, 7.0, 1.0], [2.0, 6.0, 5.0]], [[8.0, 1.0, 2.0], [4.0, 3.0, 8.0], 
[7.0, 2.0, 5.0]]], [[[3.0, 7.0, 6.0], [3.0, 7.0, 2.0], [6.0, 1.0, 8.0]], [[5.0, 1.0, 5.0], [5.0, 1.0, 2.0], [4.0, 6.0, 2.0]], 
[[2.0, 3.0, 8.0], [5.0, 4.0, 7.0], [6.0, 2.0, 4.0]]], [[[8.0, 3.0, 5.0], [3.0, 6.0, 8.0], [2.0, 5.0, 7.0]], [[8.0, 5.0, 4.0], 
[1.0, 2.0, 5.0], [8.0, 2.0, 6.0]], [[4.0, 1.0, 1.0], [3.0, 3.0, 1.0], [4.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 6.0], [8.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 5.0, 3.0], [3.0, 3.0, 8.0], [8.0, 
1.0, 5.0]], [[1.0, 6.0, 8.0], [1.0, 8.0, 5.0], [4.0, 7.0, 2.0]]], [[[1.0, 1.0, 6.0], [6.0, 8.0, 8.0], [8.0, 7.0, 8.0]], [[2.0, 
4.0, 3.0], [2.0, 6.0, 2.0], [3.0, 6.0, 1.0]], [[6.0, 5.0, 4.0], [1.0, 8.0, 2.0], [5.0, 7.0, 2.0]]], [[[4.0, 6.0, 6.0], [5.0, 
1.0, 7.0], [3.0, 4.0, 5.0]], [[1.0, 8.0, 2.0], [5.0, 2.0, 6.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 6.0], [1.0, 5.0, 7.0], [7.0, 8.0, 
4.0]]], [[[7.0, 8.0, 3.0], [8.0, 7.0, 3.0], [8.0, 2.0, 2.0]], [[3.0, 5.0, 8.0], [8.0, 1.0, 8.0], [6.0, 8.0, 7.0]], [[3.0, 5.0, 
4.0], [4.0, 5.0, 5.0], [5.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 7.0], [1.0, 6.0, 6.0], [1.0, 1.0, 4.0]], 
[[7.0, 2.0, 1.0], [3.0, 1.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 1.0, 6.0], [6.0, 4.0, 8.0], [1.0, 3.0, 1.0]]], [[[7.0, 4.0, 8.0], 
[1.0, 6.0, 7.0], [1.0, 3.0, 3.0]], [[3.0, 5.0, 4.0], [6.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[2.0, 1.0, 6.0], [3.0, 3.0, 2.0], 
[4.0, 8.0, 5.0]]], [[[6.0, 1.0, 6.0], [2.0, 6.0, 3.0], [7.0, 5.0, 3.0]], [[8.0, 6.0, 8.0], [4.0, 1.0, 1.0], [4.0, 8.0, 2.0]], 
[[1.0, 3.0, 4.0], [2.0, 8.0, 4.0], [5.0, 6.0, 4.0]]], [[[4.0, 5.0, 7.0], [8.0, 2.0, 8.0], [2.0, 2.0, 4.0]], [[5.0, 5.0, 6.0], 
[4.0, 1.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 1.0, 2.0], [1.0, 2.0, 7.0], [3.0, 5.0, 1.0]]]])
      val=numpy.array([7.0, 1.0, 1.0])
      arg[:3,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 6.0], [8.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[7.0, 5.0, 3.0], [3.0, 7.0, 8.0], [8.0, 
1.0, 5.0]], [[1.0, 6.0, 8.0], [1.0, 8.0, 5.0], [4.0, 7.0, 2.0]]], [[[1.0, 1.0, 6.0], [6.0, 8.0, 8.0], [8.0, 7.0, 8.0]], [[2.0, 
4.0, 3.0], [2.0, 1.0, 2.0], [3.0, 6.0, 1.0]], [[6.0, 5.0, 4.0], [1.0, 8.0, 2.0], [5.0, 7.0, 2.0]]], [[[4.0, 6.0, 6.0], [5.0, 
1.0, 7.0], [3.0, 4.0, 5.0]], [[1.0, 8.0, 2.0], [5.0, 1.0, 6.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 6.0], [1.0, 5.0, 7.0], [7.0, 8.0, 
4.0]]], [[[7.0, 8.0, 3.0], [8.0, 7.0, 3.0], [8.0, 2.0, 2.0]], [[3.0, 5.0, 8.0], [8.0, 1.0, 8.0], [6.0, 8.0, 7.0]], [[3.0, 5.0, 
4.0], [4.0, 5.0, 5.0], [5.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 7.0], [1.0, 6.0, 6.0], [1.0, 1.0, 4.0]], 
[[7.0, 2.0, 1.0], [3.0, 7.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 1.0, 6.0], [6.0, 4.0, 8.0], [1.0, 3.0, 1.0]]], [[[7.0, 4.0, 8.0], 
[1.0, 6.0, 7.0], [1.0, 3.0, 3.0]], [[3.0, 5.0, 4.0], [6.0, 1.0, 5.0], [2.0, 3.0, 4.0]], [[2.0, 1.0, 6.0], [3.0, 3.0, 2.0], 
[4.0, 8.0, 5.0]]], [[[6.0, 1.0, 6.0], [2.0, 6.0, 3.0], [7.0, 5.0, 3.0]], [[8.0, 6.0, 8.0], [4.0, 1.0, 1.0], [4.0, 8.0, 2.0]], 
[[1.0, 3.0, 4.0], [2.0, 8.0, 4.0], [5.0, 6.0, 4.0]]], [[[4.0, 5.0, 7.0], [8.0, 2.0, 8.0], [2.0, 2.0, 4.0]], [[5.0, 5.0, 6.0], 
[4.0, 1.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 1.0, 2.0], [1.0, 2.0, 7.0], [3.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 4.0], [6.0, 7.0, 1.0], [6.0, 3.0, 3.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 1.0], [6.0, 
3.0, 7.0]], [[3.0, 5.0, 7.0], [2.0, 3.0, 5.0], [7.0, 4.0, 6.0]]], [[[4.0, 8.0, 2.0], [7.0, 8.0, 7.0], [4.0, 4.0, 1.0]], [[6.0, 
3.0, 2.0], [5.0, 3.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 2.0, 3.0], [4.0, 2.0, 3.0]]], [[[8.0, 2.0, 8.0], [6.0, 
8.0, 7.0], [5.0, 3.0, 5.0]], [[3.0, 8.0, 6.0], [2.0, 4.0, 2.0], [5.0, 2.0, 3.0]], [[2.0, 4.0, 4.0], [4.0, 4.0, 6.0], [5.0, 3.0, 
4.0]]], [[[3.0, 8.0, 8.0], [5.0, 3.0, 3.0], [3.0, 8.0, 7.0]], [[2.0, 8.0, 2.0], [1.0, 3.0, 4.0], [8.0, 8.0, 3.0]], [[8.0, 2.0, 
1.0], [1.0, 7.0, 5.0], [2.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 1.0], [2.0, 1.0, 8.0], [1.0, 8.0, 7.0]], 
[[8.0, 8.0, 7.0], [3.0, 2.0, 5.0], [7.0, 1.0, 8.0]], [[4.0, 5.0, 6.0], [5.0, 8.0, 1.0], [1.0, 3.0, 7.0]]], [[[8.0, 4.0, 7.0], 
[3.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[8.0, 5.0, 6.0], [1.0, 6.0, 8.0], [1.0, 4.0, 1.0]], [[8.0, 6.0, 7.0], [6.0, 2.0, 1.0], 
[8.0, 7.0, 4.0]]], [[[5.0, 5.0, 8.0], [3.0, 2.0, 5.0], [2.0, 1.0, 5.0]], [[3.0, 3.0, 1.0], [7.0, 1.0, 4.0], [6.0, 1.0, 2.0]], 
[[1.0, 4.0, 5.0], [5.0, 6.0, 7.0], [5.0, 6.0, 5.0]]], [[[7.0, 4.0, 5.0], [6.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[7.0, 1.0, 2.0], 
[3.0, 7.0, 4.0], [7.0, 4.0, 3.0]], [[2.0, 3.0, 1.0], [2.0, 8.0, 1.0], [5.0, 7.0, 5.0]]]])
      val=Data(numpy.array([5.0, 8.0, 8.0]),self.functionspace)
      arg[:3,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 4.0], [6.0, 7.0, 1.0], [6.0, 3.0, 3.0]], [[8.0, 2.0, 5.0], [4.0, 5.0, 1.0], [6.0, 
3.0, 7.0]], [[3.0, 5.0, 7.0], [2.0, 3.0, 5.0], [7.0, 4.0, 6.0]]], [[[4.0, 8.0, 2.0], [7.0, 8.0, 7.0], [4.0, 4.0, 1.0]], [[6.0, 
3.0, 2.0], [5.0, 8.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 2.0, 3.0], [4.0, 2.0, 3.0]]], [[[8.0, 2.0, 8.0], [6.0, 
8.0, 7.0], [5.0, 3.0, 5.0]], [[3.0, 8.0, 6.0], [2.0, 8.0, 2.0], [5.0, 2.0, 3.0]], [[2.0, 4.0, 4.0], [4.0, 4.0, 6.0], [5.0, 3.0, 
4.0]]], [[[3.0, 8.0, 8.0], [5.0, 3.0, 3.0], [3.0, 8.0, 7.0]], [[2.0, 8.0, 2.0], [1.0, 3.0, 4.0], [8.0, 8.0, 3.0]], [[8.0, 2.0, 
1.0], [1.0, 7.0, 5.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 1.0], [2.0, 1.0, 8.0], [1.0, 8.0, 7.0]], 
[[8.0, 8.0, 7.0], [3.0, 5.0, 5.0], [7.0, 1.0, 8.0]], [[4.0, 5.0, 6.0], [5.0, 8.0, 1.0], [1.0, 3.0, 7.0]]], [[[8.0, 4.0, 7.0], 
[3.0, 1.0, 5.0], [3.0, 8.0, 5.0]], [[8.0, 5.0, 6.0], [1.0, 8.0, 8.0], [1.0, 4.0, 1.0]], [[8.0, 6.0, 7.0], [6.0, 2.0, 1.0], 
[8.0, 7.0, 4.0]]], [[[5.0, 5.0, 8.0], [3.0, 2.0, 5.0], [2.0, 1.0, 5.0]], [[3.0, 3.0, 1.0], [7.0, 8.0, 4.0], [6.0, 1.0, 2.0]], 
[[1.0, 4.0, 5.0], [5.0, 6.0, 7.0], [5.0, 6.0, 5.0]]], [[[7.0, 4.0, 5.0], [6.0, 4.0, 1.0], [6.0, 1.0, 6.0]], [[7.0, 1.0, 2.0], 
[3.0, 7.0, 4.0], [7.0, 4.0, 3.0]], [[2.0, 3.0, 1.0], [2.0, 8.0, 1.0], [5.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 4.0], [6.0, 6.0, 6.0], [3.0, 2.0, 2.0]], [[3.0, 3.0, 5.0], [3.0, 2.0, 6.0], [2.0, 
4.0, 4.0]], [[5.0, 7.0, 2.0], [6.0, 6.0, 3.0], [4.0, 5.0, 1.0]]], [[[8.0, 6.0, 1.0], [8.0, 2.0, 2.0], [7.0, 5.0, 1.0]], [[2.0, 
8.0, 4.0], [2.0, 5.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 2.0, 2.0], [8.0, 5.0, 2.0]]], [[[3.0, 2.0, 8.0], [8.0, 
5.0, 1.0], [7.0, 6.0, 1.0]], [[6.0, 1.0, 5.0], [2.0, 8.0, 7.0], [3.0, 3.0, 6.0]], [[8.0, 3.0, 8.0], [6.0, 6.0, 7.0], [5.0, 8.0, 
5.0]]], [[[8.0, 6.0, 1.0], [6.0, 6.0, 8.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 8.0], [1.0, 7.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 1.0, 
2.0], [1.0, 7.0, 1.0], [7.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 1.0], [3.0, 7.0, 2.0], [3.0, 3.0, 7.0]], 
[[6.0, 2.0, 2.0], [8.0, 6.0, 1.0], [2.0, 3.0, 2.0]], [[6.0, 8.0, 3.0], [6.0, 3.0, 7.0], [2.0, 3.0, 6.0]]], [[[5.0, 3.0, 2.0], 
[2.0, 2.0, 2.0], [1.0, 4.0, 4.0]], [[2.0, 3.0, 4.0], [2.0, 5.0, 6.0], [5.0, 6.0, 3.0]], [[6.0, 3.0, 5.0], [8.0, 6.0, 1.0], 
[7.0, 1.0, 1.0]]], [[[2.0, 7.0, 5.0], [2.0, 5.0, 3.0], [2.0, 8.0, 7.0]], [[5.0, 1.0, 7.0], [2.0, 5.0, 6.0], [8.0, 5.0, 3.0]], 
[[8.0, 7.0, 5.0], [5.0, 2.0, 2.0], [6.0, 5.0, 1.0]]], [[[6.0, 8.0, 6.0], [4.0, 6.0, 2.0], [5.0, 8.0, 3.0]], [[6.0, 2.0, 8.0], 
[4.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[6.0, 3.0, 5.0], [4.0, 1.0, 2.0], [7.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([2.0, 5.0, 2.0])+(1.-msk_val)*numpy.array([3.0, 6.0, 2.0])
      arg[:3,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 4.0], [6.0, 6.0, 6.0], [3.0, 2.0, 2.0]], [[3.0, 3.0, 5.0], [3.0, 2.0, 6.0], [2.0, 
4.0, 4.0]], [[5.0, 7.0, 2.0], [6.0, 6.0, 3.0], [4.0, 5.0, 1.0]]], [[[8.0, 6.0, 1.0], [8.0, 2.0, 2.0], [7.0, 5.0, 1.0]], [[2.0, 
8.0, 4.0], [2.0, 5.0, 3.0], [2.0, 8.0, 8.0]], [[1.0, 2.0, 6.0], [6.0, 2.0, 2.0], [8.0, 5.0, 2.0]]], [[[3.0, 2.0, 8.0], [8.0, 
5.0, 1.0], [7.0, 6.0, 1.0]], [[6.0, 1.0, 5.0], [2.0, 2.0, 7.0], [3.0, 3.0, 6.0]], [[8.0, 3.0, 8.0], [6.0, 6.0, 7.0], [5.0, 8.0, 
5.0]]], [[[8.0, 6.0, 1.0], [6.0, 6.0, 8.0], [4.0, 5.0, 3.0]], [[7.0, 6.0, 8.0], [1.0, 7.0, 2.0], [7.0, 5.0, 2.0]], [[8.0, 1.0, 
2.0], [1.0, 7.0, 1.0], [7.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 1.0], [3.0, 7.0, 2.0], [3.0, 3.0, 7.0]], 
[[6.0, 2.0, 2.0], [8.0, 3.0, 1.0], [2.0, 3.0, 2.0]], [[6.0, 8.0, 3.0], [6.0, 3.0, 7.0], [2.0, 3.0, 6.0]]], [[[5.0, 3.0, 2.0], 
[2.0, 2.0, 2.0], [1.0, 4.0, 4.0]], [[2.0, 3.0, 4.0], [2.0, 6.0, 6.0], [5.0, 6.0, 3.0]], [[6.0, 3.0, 5.0], [8.0, 6.0, 1.0], 
[7.0, 1.0, 1.0]]], [[[2.0, 7.0, 5.0], [2.0, 5.0, 3.0], [2.0, 8.0, 7.0]], [[5.0, 1.0, 7.0], [2.0, 2.0, 6.0], [8.0, 5.0, 3.0]], 
[[8.0, 7.0, 5.0], [5.0, 2.0, 2.0], [6.0, 5.0, 1.0]]], [[[6.0, 8.0, 6.0], [4.0, 6.0, 2.0], [5.0, 8.0, 3.0]], [[6.0, 2.0, 8.0], 
[4.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[6.0, 3.0, 5.0], [4.0, 1.0, 2.0], [7.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 3.0], [7.0, 6.0, 7.0], [5.0, 1.0, 3.0]], [[1.0, 5.0, 4.0], [8.0, 8.0, 6.0], [4.0, 
8.0, 1.0]], [[4.0, 7.0, 2.0], [5.0, 7.0, 1.0], [6.0, 5.0, 3.0]]], [[[5.0, 8.0, 2.0], [1.0, 6.0, 4.0], [4.0, 8.0, 3.0]], [[7.0, 
7.0, 7.0], [1.0, 4.0, 1.0], [6.0, 2.0, 3.0]], [[2.0, 7.0, 1.0], [1.0, 2.0, 2.0], [6.0, 8.0, 3.0]]], [[[6.0, 7.0, 6.0], [3.0, 
3.0, 5.0], [4.0, 8.0, 4.0]], [[8.0, 4.0, 3.0], [3.0, 7.0, 3.0], [6.0, 2.0, 6.0]], [[4.0, 5.0, 1.0], [3.0, 5.0, 4.0], [1.0, 5.0, 
1.0]]], [[[7.0, 5.0, 6.0], [6.0, 2.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 4.0, 5.0], [6.0, 2.0, 1.0], [5.0, 1.0, 4.0]], [[4.0, 5.0, 
4.0], [8.0, 6.0, 2.0], [2.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 5.0], [5.0, 1.0, 6.0], [7.0, 5.0, 8.0]], 
[[4.0, 1.0, 7.0], [7.0, 1.0, 1.0], [1.0, 6.0, 5.0]], [[7.0, 6.0, 8.0], [4.0, 6.0, 4.0], [2.0, 5.0, 5.0]]], [[[3.0, 3.0, 4.0], 
[7.0, 1.0, 8.0], [4.0, 3.0, 5.0]], [[7.0, 5.0, 6.0], [6.0, 4.0, 7.0], [4.0, 3.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 3.0, 8.0], 
[1.0, 7.0, 8.0]]], [[[7.0, 4.0, 4.0], [5.0, 2.0, 5.0], [7.0, 3.0, 8.0]], [[8.0, 7.0, 8.0], [4.0, 8.0, 4.0], [5.0, 7.0, 1.0]], 
[[1.0, 8.0, 2.0], [4.0, 1.0, 7.0], [6.0, 3.0, 2.0]]], [[[4.0, 3.0, 2.0], [2.0, 8.0, 5.0], [2.0, 6.0, 2.0]], [[8.0, 7.0, 5.0], 
[5.0, 6.0, 6.0], [8.0, 2.0, 4.0]], [[5.0, 1.0, 4.0], [8.0, 1.0, 1.0], [2.0, 4.0, 8.0]]]])
      val=numpy.array([[[[7.0, 8.0, 6.0]], [[5.0, 8.0, 4.0]]], [[[1.0, 4.0, 2.0]], [[7.0, 3.0, 3.0]]], [[[2.0, 3.0, 1.0]], 
[[2.0, 6.0, 7.0]]], [[[1.0, 4.0, 5.0]], [[2.0, 6.0, 3.0]]]])
      arg[:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 3.0], [7.0, 8.0, 6.0], [5.0, 1.0, 3.0]], [[1.0, 5.0, 4.0], [5.0, 8.0, 4.0], [4.0, 
8.0, 1.0]], [[4.0, 7.0, 2.0], [5.0, 7.0, 1.0], [6.0, 5.0, 3.0]]], [[[5.0, 8.0, 2.0], [1.0, 4.0, 2.0], [4.0, 8.0, 3.0]], [[7.0, 
7.0, 7.0], [7.0, 3.0, 3.0], [6.0, 2.0, 3.0]], [[2.0, 7.0, 1.0], [1.0, 2.0, 2.0], [6.0, 8.0, 3.0]]], [[[6.0, 7.0, 6.0], [2.0, 
3.0, 1.0], [4.0, 8.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 6.0, 7.0], [6.0, 2.0, 6.0]], [[4.0, 5.0, 1.0], [3.0, 5.0, 4.0], [1.0, 5.0, 
1.0]]], [[[7.0, 5.0, 6.0], [1.0, 4.0, 5.0], [4.0, 7.0, 1.0]], [[8.0, 4.0, 5.0], [2.0, 6.0, 3.0], [5.0, 1.0, 4.0]], [[4.0, 5.0, 
4.0], [8.0, 6.0, 2.0], [2.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 5.0], [7.0, 8.0, 6.0], [7.0, 5.0, 8.0]], 
[[4.0, 1.0, 7.0], [5.0, 8.0, 4.0], [1.0, 6.0, 5.0]], [[7.0, 6.0, 8.0], [4.0, 6.0, 4.0], [2.0, 5.0, 5.0]]], [[[3.0, 3.0, 4.0], 
[1.0, 4.0, 2.0], [4.0, 3.0, 5.0]], [[7.0, 5.0, 6.0], [7.0, 3.0, 3.0], [4.0, 3.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 3.0, 8.0], 
[1.0, 7.0, 8.0]]], [[[7.0, 4.0, 4.0], [2.0, 3.0, 1.0], [7.0, 3.0, 8.0]], [[8.0, 7.0, 8.0], [2.0, 6.0, 7.0], [5.0, 7.0, 1.0]], 
[[1.0, 8.0, 2.0], [4.0, 1.0, 7.0], [6.0, 3.0, 2.0]]], [[[4.0, 3.0, 2.0], [1.0, 4.0, 5.0], [2.0, 6.0, 2.0]], [[8.0, 7.0, 5.0], 
[2.0, 6.0, 3.0], [8.0, 2.0, 4.0]], [[5.0, 1.0, 4.0], [8.0, 1.0, 1.0], [2.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 8.0], [2.0, 2.0, 6.0], [7.0, 1.0, 3.0]], [[5.0, 3.0, 3.0], [2.0, 1.0, 5.0], [7.0, 
6.0, 4.0]], [[1.0, 6.0, 5.0], [7.0, 2.0, 3.0], [8.0, 8.0, 4.0]]], [[[7.0, 4.0, 7.0], [1.0, 6.0, 3.0], [7.0, 4.0, 6.0]], [[7.0, 
7.0, 3.0], [3.0, 5.0, 5.0], [2.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 5.0, 4.0], [3.0, 6.0, 4.0]]], [[[6.0, 4.0, 8.0], [5.0, 
1.0, 8.0], [5.0, 4.0, 8.0]], [[7.0, 7.0, 3.0], [5.0, 2.0, 7.0], [2.0, 6.0, 5.0]], [[5.0, 5.0, 7.0], [2.0, 4.0, 8.0], [5.0, 8.0, 
4.0]]], [[[8.0, 5.0, 3.0], [1.0, 8.0, 1.0], [7.0, 7.0, 7.0]], [[4.0, 6.0, 5.0], [7.0, 8.0, 7.0], [7.0, 3.0, 1.0]], [[6.0, 1.0, 
8.0], [5.0, 4.0, 1.0], [2.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 8.0, 6.0], [5.0, 7.0, 5.0]], 
[[4.0, 5.0, 5.0], [3.0, 1.0, 7.0], [3.0, 6.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 3.0, 5.0], [6.0, 2.0, 2.0]]], [[[7.0, 2.0, 1.0], 
[5.0, 7.0, 4.0], [2.0, 3.0, 1.0]], [[8.0, 6.0, 2.0], [2.0, 5.0, 5.0], [4.0, 7.0, 7.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 7.0], 
[4.0, 1.0, 6.0]]], [[[2.0, 4.0, 4.0], [7.0, 1.0, 1.0], [8.0, 3.0, 2.0]], [[8.0, 4.0, 7.0], [5.0, 2.0, 6.0], [1.0, 6.0, 7.0]], 
[[5.0, 2.0, 5.0], [7.0, 7.0, 7.0], [5.0, 8.0, 6.0]]], [[[4.0, 8.0, 5.0], [5.0, 8.0, 8.0], [8.0, 5.0, 2.0]], [[8.0, 7.0, 4.0], 
[8.0, 3.0, 4.0], [5.0, 4.0, 2.0]], [[7.0, 2.0, 7.0], [2.0, 6.0, 1.0], [3.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[[8.0, 4.0, 8.0]], [[5.0, 6.0, 2.0]]], [[[1.0, 3.0, 5.0]], [[7.0, 7.0, 7.0]]], [[[8.0, 3.0, 2.0]], 
[[4.0, 7.0, 8.0]]], [[[7.0, 5.0, 1.0]], [[3.0, 3.0, 1.0]]]]),self.functionspace)
      arg[:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 8.0], [8.0, 4.0, 8.0], [7.0, 1.0, 3.0]], [[5.0, 3.0, 3.0], [5.0, 6.0, 2.0], [7.0, 
6.0, 4.0]], [[1.0, 6.0, 5.0], [7.0, 2.0, 3.0], [8.0, 8.0, 4.0]]], [[[7.0, 4.0, 7.0], [1.0, 3.0, 5.0], [7.0, 4.0, 6.0]], [[7.0, 
7.0, 3.0], [7.0, 7.0, 7.0], [2.0, 5.0, 4.0]], [[5.0, 6.0, 6.0], [2.0, 5.0, 4.0], [3.0, 6.0, 4.0]]], [[[6.0, 4.0, 8.0], [8.0, 
3.0, 2.0], [5.0, 4.0, 8.0]], [[7.0, 7.0, 3.0], [4.0, 7.0, 8.0], [2.0, 6.0, 5.0]], [[5.0, 5.0, 7.0], [2.0, 4.0, 8.0], [5.0, 8.0, 
4.0]]], [[[8.0, 5.0, 3.0], [7.0, 5.0, 1.0], [7.0, 7.0, 7.0]], [[4.0, 6.0, 5.0], [3.0, 3.0, 1.0], [7.0, 3.0, 1.0]], [[6.0, 1.0, 
8.0], [5.0, 4.0, 1.0], [2.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [8.0, 4.0, 8.0], [5.0, 7.0, 5.0]], 
[[4.0, 5.0, 5.0], [5.0, 6.0, 2.0], [3.0, 6.0, 6.0]], [[6.0, 6.0, 6.0], [6.0, 3.0, 5.0], [6.0, 2.0, 2.0]]], [[[7.0, 2.0, 1.0], 
[1.0, 3.0, 5.0], [2.0, 3.0, 1.0]], [[8.0, 6.0, 2.0], [7.0, 7.0, 7.0], [4.0, 7.0, 7.0]], [[5.0, 1.0, 2.0], [2.0, 1.0, 7.0], 
[4.0, 1.0, 6.0]]], [[[2.0, 4.0, 4.0], [8.0, 3.0, 2.0], [8.0, 3.0, 2.0]], [[8.0, 4.0, 7.0], [4.0, 7.0, 8.0], [1.0, 6.0, 7.0]], 
[[5.0, 2.0, 5.0], [7.0, 7.0, 7.0], [5.0, 8.0, 6.0]]], [[[4.0, 8.0, 5.0], [7.0, 5.0, 1.0], [8.0, 5.0, 2.0]], [[8.0, 7.0, 4.0], 
[3.0, 3.0, 1.0], [5.0, 4.0, 2.0]], [[7.0, 2.0, 7.0], [2.0, 6.0, 1.0], [3.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 8.0], [5.0, 8.0, 7.0], [3.0, 1.0, 8.0]], [[2.0, 6.0, 4.0], [5.0, 6.0, 3.0], [5.0, 
7.0, 5.0]], [[2.0, 7.0, 1.0], [5.0, 7.0, 7.0], [3.0, 1.0, 3.0]]], [[[4.0, 7.0, 2.0], [1.0, 6.0, 5.0], [4.0, 5.0, 1.0]], [[8.0, 
3.0, 5.0], [7.0, 2.0, 6.0], [8.0, 4.0, 4.0]], [[6.0, 7.0, 3.0], [1.0, 2.0, 5.0], [6.0, 8.0, 1.0]]], [[[3.0, 5.0, 7.0], [1.0, 
2.0, 5.0], [8.0, 6.0, 4.0]], [[5.0, 5.0, 3.0], [3.0, 7.0, 3.0], [8.0, 4.0, 8.0]], [[4.0, 3.0, 3.0], [7.0, 5.0, 2.0], [4.0, 5.0, 
5.0]]], [[[7.0, 4.0, 7.0], [2.0, 3.0, 7.0], [5.0, 2.0, 4.0]], [[6.0, 8.0, 4.0], [8.0, 2.0, 8.0], [4.0, 7.0, 8.0]], [[5.0, 6.0, 
7.0], [2.0, 2.0, 5.0], [6.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 5.0], [2.0, 1.0, 3.0], [6.0, 6.0, 7.0]], 
[[1.0, 2.0, 7.0], [5.0, 3.0, 3.0], [8.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [4.0, 1.0, 6.0], [1.0, 7.0, 2.0]]], [[[5.0, 2.0, 3.0], 
[4.0, 3.0, 2.0], [3.0, 5.0, 5.0]], [[4.0, 7.0, 2.0], [8.0, 5.0, 3.0], [5.0, 7.0, 7.0]], [[7.0, 8.0, 6.0], [1.0, 8.0, 8.0], 
[2.0, 8.0, 2.0]]], [[[2.0, 4.0, 7.0], [4.0, 8.0, 6.0], [3.0, 7.0, 6.0]], [[3.0, 2.0, 8.0], [1.0, 1.0, 2.0], [2.0, 2.0, 1.0]], 
[[5.0, 3.0, 2.0], [5.0, 5.0, 8.0], [4.0, 5.0, 1.0]]], [[[4.0, 6.0, 5.0], [2.0, 1.0, 3.0], [6.0, 7.0, 5.0]], [[1.0, 7.0, 7.0], 
[7.0, 1.0, 2.0], [8.0, 1.0, 2.0]], [[2.0, 4.0, 5.0], [6.0, 2.0, 8.0], [8.0, 7.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 1.0, 3.0]], [[4.0, 7.0, 1.0]]], [[[2.0, 1.0, 2.0]], [[6.0, 8.0, 2.0]]], [[[1.0, 2.0, 
6.0]], [[6.0, 5.0, 6.0]]], [[[8.0, 5.0, 6.0]], [[1.0, 5.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 6.0, 4.0]], [[4.0, 3.0, 
5.0]]], [[[3.0, 5.0, 8.0]], [[5.0, 2.0, 7.0]]], [[[3.0, 3.0, 4.0]], [[1.0, 6.0, 8.0]]], [[[2.0, 2.0, 8.0]], [[3.0, 5.0, 4.0]]]])
      arg[:,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [2.0, 1.0, 3.0], [3.0, 1.0, 8.0]], [[2.0, 6.0, 4.0], [4.0, 7.0, 1.0], [5.0, 
7.0, 5.0]], [[2.0, 7.0, 1.0], [5.0, 7.0, 7.0], [3.0, 1.0, 3.0]]], [[[4.0, 7.0, 2.0], [2.0, 1.0, 2.0], [4.0, 5.0, 1.0]], [[8.0, 
3.0, 5.0], [6.0, 8.0, 2.0], [8.0, 4.0, 4.0]], [[6.0, 7.0, 3.0], [1.0, 2.0, 5.0], [6.0, 8.0, 1.0]]], [[[3.0, 5.0, 7.0], [1.0, 
2.0, 6.0], [8.0, 6.0, 4.0]], [[5.0, 5.0, 3.0], [6.0, 5.0, 6.0], [8.0, 4.0, 8.0]], [[4.0, 3.0, 3.0], [7.0, 5.0, 2.0], [4.0, 5.0, 
5.0]]], [[[7.0, 4.0, 7.0], [8.0, 5.0, 6.0], [5.0, 2.0, 4.0]], [[6.0, 8.0, 4.0], [1.0, 5.0, 3.0], [4.0, 7.0, 8.0]], [[5.0, 6.0, 
7.0], [2.0, 2.0, 5.0], [6.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 5.0], [2.0, 6.0, 4.0], [6.0, 6.0, 7.0]], 
[[1.0, 2.0, 7.0], [4.0, 3.0, 5.0], [8.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [4.0, 1.0, 6.0], [1.0, 7.0, 2.0]]], [[[5.0, 2.0, 3.0], 
[3.0, 5.0, 8.0], [3.0, 5.0, 5.0]], [[4.0, 7.0, 2.0], [5.0, 2.0, 7.0], [5.0, 7.0, 7.0]], [[7.0, 8.0, 6.0], [1.0, 8.0, 8.0], 
[2.0, 8.0, 2.0]]], [[[2.0, 4.0, 7.0], [3.0, 3.0, 4.0], [3.0, 7.0, 6.0]], [[3.0, 2.0, 8.0], [1.0, 6.0, 8.0], [2.0, 2.0, 1.0]], 
[[5.0, 3.0, 2.0], [5.0, 5.0, 8.0], [4.0, 5.0, 1.0]]], [[[4.0, 6.0, 5.0], [2.0, 2.0, 8.0], [6.0, 7.0, 5.0]], [[1.0, 7.0, 7.0], 
[3.0, 5.0, 4.0], [8.0, 1.0, 2.0]], [[2.0, 4.0, 5.0], [6.0, 2.0, 8.0], [8.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 8.0], [5.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[8.0, 4.0, 5.0], [5.0, 5.0, 7.0], [2.0, 
8.0, 7.0]], [[6.0, 1.0, 7.0], [1.0, 4.0, 4.0], [8.0, 2.0, 6.0]]], [[[5.0, 8.0, 6.0], [5.0, 6.0, 8.0], [3.0, 4.0, 4.0]], [[2.0, 
8.0, 4.0], [3.0, 5.0, 5.0], [8.0, 8.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 1.0, 2.0], [4.0, 2.0, 7.0]]], [[[7.0, 2.0, 5.0], [4.0, 
7.0, 6.0], [8.0, 6.0, 8.0]], [[1.0, 6.0, 7.0], [7.0, 7.0, 3.0], [6.0, 3.0, 3.0]], [[1.0, 8.0, 3.0], [4.0, 2.0, 4.0], [2.0, 7.0, 
4.0]]], [[[5.0, 3.0, 4.0], [1.0, 4.0, 7.0], [4.0, 4.0, 2.0]], [[2.0, 4.0, 2.0], [1.0, 7.0, 5.0], [2.0, 6.0, 2.0]], [[4.0, 8.0, 
2.0], [7.0, 1.0, 7.0], [3.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 4.0], [8.0, 7.0, 8.0], [1.0, 5.0, 2.0]], 
[[4.0, 1.0, 2.0], [6.0, 4.0, 3.0], [7.0, 5.0, 8.0]], [[7.0, 4.0, 4.0], [4.0, 7.0, 5.0], [5.0, 3.0, 3.0]]], [[[3.0, 4.0, 1.0], 
[6.0, 4.0, 4.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 4.0], [2.0, 8.0, 7.0], [5.0, 7.0, 5.0]], [[5.0, 2.0, 5.0], [6.0, 8.0, 5.0], 
[3.0, 6.0, 5.0]]], [[[7.0, 6.0, 7.0], [2.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[3.0, 3.0, 1.0], [4.0, 1.0, 4.0], [7.0, 1.0, 7.0]], 
[[2.0, 6.0, 6.0], [8.0, 3.0, 2.0], [8.0, 1.0, 1.0]]], [[[5.0, 3.0, 7.0], [7.0, 2.0, 5.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 2.0], 
[4.0, 6.0, 6.0], [5.0, 4.0, 6.0]], [[1.0, 1.0, 8.0], [8.0, 4.0, 6.0], [1.0, 6.0, 1.0]]]])
      val=numpy.array([[[[7.0, 2.0, 6.0]]], [[[5.0, 4.0, 4.0]]]])
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 8.0], [5.0, 8.0, 8.0], [2.0, 5.0, 3.0]], [[8.0, 4.0, 5.0], [5.0, 5.0, 7.0], [2.0, 
8.0, 7.0]], [[6.0, 1.0, 7.0], [1.0, 4.0, 4.0], [8.0, 2.0, 6.0]]], [[[5.0, 8.0, 6.0], [5.0, 6.0, 8.0], [3.0, 4.0, 4.0]], [[2.0, 
8.0, 4.0], [3.0, 5.0, 5.0], [8.0, 8.0, 5.0]], [[7.0, 1.0, 4.0], [6.0, 1.0, 2.0], [4.0, 2.0, 7.0]]], [[[7.0, 2.0, 5.0], [4.0, 
7.0, 6.0], [8.0, 6.0, 8.0]], [[1.0, 6.0, 7.0], [7.0, 7.0, 3.0], [6.0, 3.0, 3.0]], [[1.0, 8.0, 3.0], [4.0, 2.0, 4.0], [7.0, 2.0, 
6.0]]], [[[5.0, 3.0, 4.0], [1.0, 4.0, 7.0], [4.0, 4.0, 2.0]], [[2.0, 4.0, 2.0], [1.0, 7.0, 5.0], [2.0, 6.0, 2.0]], [[4.0, 8.0, 
2.0], [7.0, 1.0, 7.0], [5.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 4.0], [8.0, 7.0, 8.0], [1.0, 5.0, 2.0]], 
[[4.0, 1.0, 2.0], [6.0, 4.0, 3.0], [7.0, 5.0, 8.0]], [[7.0, 4.0, 4.0], [4.0, 7.0, 5.0], [5.0, 3.0, 3.0]]], [[[3.0, 4.0, 1.0], 
[6.0, 4.0, 4.0], [7.0, 2.0, 7.0]], [[6.0, 7.0, 4.0], [2.0, 8.0, 7.0], [5.0, 7.0, 5.0]], [[5.0, 2.0, 5.0], [6.0, 8.0, 5.0], 
[3.0, 6.0, 5.0]]], [[[7.0, 6.0, 7.0], [2.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[3.0, 3.0, 1.0], [4.0, 1.0, 4.0], [7.0, 1.0, 7.0]], 
[[2.0, 6.0, 6.0], [8.0, 3.0, 2.0], [7.0, 2.0, 6.0]]], [[[5.0, 3.0, 7.0], [7.0, 2.0, 5.0], [8.0, 7.0, 8.0]], [[3.0, 5.0, 2.0], 
[4.0, 6.0, 6.0], [5.0, 4.0, 6.0]], [[1.0, 1.0, 8.0], [8.0, 4.0, 6.0], [5.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 3.0, 1.0], [4.0, 5.0, 7.0]], [[3.0, 6.0, 8.0], [4.0, 6.0, 3.0], [8.0, 
2.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 3.0, 3.0], [5.0, 4.0, 5.0]]], [[[7.0, 1.0, 2.0], [3.0, 7.0, 4.0], [3.0, 5.0, 6.0]], [[4.0, 
5.0, 8.0], [8.0, 7.0, 6.0], [4.0, 4.0, 4.0]], [[6.0, 8.0, 4.0], [7.0, 6.0, 2.0], [8.0, 4.0, 3.0]]], [[[4.0, 7.0, 4.0], [1.0, 
3.0, 6.0], [1.0, 1.0, 4.0]], [[3.0, 7.0, 3.0], [3.0, 3.0, 1.0], [5.0, 5.0, 6.0]], [[1.0, 7.0, 7.0], [1.0, 1.0, 4.0], [4.0, 1.0, 
6.0]]], [[[4.0, 3.0, 1.0], [5.0, 5.0, 5.0], [2.0, 7.0, 3.0]], [[6.0, 7.0, 7.0], [5.0, 1.0, 6.0], [1.0, 4.0, 2.0]], [[4.0, 6.0, 
7.0], [2.0, 6.0, 3.0], [8.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 2.0], [1.0, 3.0, 1.0], [2.0, 3.0, 7.0]], 
[[6.0, 7.0, 7.0], [7.0, 2.0, 2.0], [7.0, 2.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 1.0, 8.0], [8.0, 7.0, 2.0]]], [[[5.0, 2.0, 6.0], 
[3.0, 8.0, 5.0], [5.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [2.0, 4.0, 8.0], [1.0, 5.0, 3.0]], [[2.0, 4.0, 2.0], [6.0, 8.0, 2.0], 
[3.0, 4.0, 4.0]]], [[[8.0, 4.0, 8.0], [3.0, 2.0, 3.0], [4.0, 5.0, 4.0]], [[4.0, 1.0, 7.0], [4.0, 8.0, 2.0], [1.0, 1.0, 8.0]], 
[[1.0, 3.0, 8.0], [8.0, 8.0, 8.0], [5.0, 8.0, 6.0]]], [[[4.0, 8.0, 2.0], [1.0, 4.0, 6.0], [5.0, 8.0, 5.0]], [[7.0, 2.0, 3.0], 
[1.0, 2.0, 6.0], [3.0, 7.0, 5.0]], [[4.0, 1.0, 1.0], [1.0, 8.0, 2.0], [5.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[[2.0, 4.0, 7.0]]], [[[3.0, 3.0, 8.0]]]]),self.functionspace)
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 1.0], [1.0, 3.0, 1.0], [4.0, 5.0, 7.0]], [[3.0, 6.0, 8.0], [4.0, 6.0, 3.0], [8.0, 
2.0, 7.0]], [[8.0, 7.0, 5.0], [4.0, 3.0, 3.0], [5.0, 4.0, 5.0]]], [[[7.0, 1.0, 2.0], [3.0, 7.0, 4.0], [3.0, 5.0, 6.0]], [[4.0, 
5.0, 8.0], [8.0, 7.0, 6.0], [4.0, 4.0, 4.0]], [[6.0, 8.0, 4.0], [7.0, 6.0, 2.0], [8.0, 4.0, 3.0]]], [[[4.0, 7.0, 4.0], [1.0, 
3.0, 6.0], [1.0, 1.0, 4.0]], [[3.0, 7.0, 3.0], [3.0, 3.0, 1.0], [5.0, 5.0, 6.0]], [[1.0, 7.0, 7.0], [1.0, 1.0, 4.0], [2.0, 4.0, 
7.0]]], [[[4.0, 3.0, 1.0], [5.0, 5.0, 5.0], [2.0, 7.0, 3.0]], [[6.0, 7.0, 7.0], [5.0, 1.0, 6.0], [1.0, 4.0, 2.0]], [[4.0, 6.0, 
7.0], [2.0, 6.0, 3.0], [3.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 2.0], [1.0, 3.0, 1.0], [2.0, 3.0, 7.0]], 
[[6.0, 7.0, 7.0], [7.0, 2.0, 2.0], [7.0, 2.0, 1.0]], [[7.0, 7.0, 7.0], [1.0, 1.0, 8.0], [8.0, 7.0, 2.0]]], [[[5.0, 2.0, 6.0], 
[3.0, 8.0, 5.0], [5.0, 6.0, 3.0]], [[3.0, 6.0, 6.0], [2.0, 4.0, 8.0], [1.0, 5.0, 3.0]], [[2.0, 4.0, 2.0], [6.0, 8.0, 2.0], 
[3.0, 4.0, 4.0]]], [[[8.0, 4.0, 8.0], [3.0, 2.0, 3.0], [4.0, 5.0, 4.0]], [[4.0, 1.0, 7.0], [4.0, 8.0, 2.0], [1.0, 1.0, 8.0]], 
[[1.0, 3.0, 8.0], [8.0, 8.0, 8.0], [2.0, 4.0, 7.0]]], [[[4.0, 8.0, 2.0], [1.0, 4.0, 6.0], [5.0, 8.0, 5.0]], [[7.0, 2.0, 3.0], 
[1.0, 2.0, 6.0], [3.0, 7.0, 5.0]], [[4.0, 1.0, 1.0], [1.0, 8.0, 2.0], [3.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[4.0, 6.0, 6.0], [7.0, 5.0, 2.0], [5.0, 
2.0, 4.0]], [[5.0, 6.0, 2.0], [4.0, 7.0, 5.0], [5.0, 1.0, 4.0]]], [[[7.0, 5.0, 8.0], [1.0, 7.0, 5.0], [6.0, 4.0, 8.0]], [[3.0, 
2.0, 1.0], [3.0, 3.0, 3.0], [2.0, 5.0, 2.0]], [[5.0, 8.0, 1.0], [8.0, 2.0, 1.0], [4.0, 4.0, 4.0]]], [[[4.0, 4.0, 3.0], [1.0, 
4.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 5.0, 2.0], [1.0, 3.0, 3.0], [4.0, 4.0, 5.0]], [[8.0, 8.0, 6.0], [3.0, 7.0, 6.0], [7.0, 5.0, 
6.0]]], [[[3.0, 7.0, 7.0], [2.0, 8.0, 4.0], [3.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 3.0, 5.0], [8.0, 8.0, 3.0]], [[1.0, 2.0, 
3.0], [1.0, 7.0, 2.0], [7.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 1.0], [8.0, 6.0, 4.0], [7.0, 2.0, 7.0]], 
[[7.0, 3.0, 3.0], [6.0, 8.0, 6.0], [1.0, 8.0, 4.0]], [[3.0, 4.0, 2.0], [3.0, 6.0, 7.0], [4.0, 2.0, 5.0]]], [[[5.0, 3.0, 8.0], 
[5.0, 6.0, 8.0], [5.0, 4.0, 8.0]], [[3.0, 3.0, 1.0], [2.0, 4.0, 1.0], [8.0, 7.0, 5.0]], [[6.0, 8.0, 2.0], [4.0, 5.0, 6.0], 
[8.0, 6.0, 8.0]]], [[[8.0, 5.0, 3.0], [7.0, 6.0, 4.0], [6.0, 1.0, 4.0]], [[3.0, 6.0, 6.0], [8.0, 6.0, 3.0], [4.0, 8.0, 4.0]], 
[[2.0, 3.0, 1.0], [7.0, 1.0, 1.0], [4.0, 3.0, 7.0]]], [[[8.0, 1.0, 8.0], [8.0, 5.0, 5.0], [2.0, 2.0, 1.0]], [[7.0, 7.0, 8.0], 
[1.0, 1.0, 3.0], [3.0, 5.0, 4.0]], [[8.0, 8.0, 5.0], [8.0, 7.0, 8.0], [5.0, 5.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 8.0, 3.0]]], [[[8.0, 7.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 1.0, 4.0]]], 
[[[2.0, 5.0, 4.0]]]])
      arg[2:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[4.0, 6.0, 6.0], [7.0, 5.0, 2.0], [5.0, 
2.0, 4.0]], [[5.0, 6.0, 2.0], [4.0, 7.0, 5.0], [5.0, 1.0, 4.0]]], [[[7.0, 5.0, 8.0], [1.0, 7.0, 5.0], [6.0, 4.0, 8.0]], [[3.0, 
2.0, 1.0], [3.0, 3.0, 3.0], [2.0, 5.0, 2.0]], [[5.0, 8.0, 1.0], [8.0, 2.0, 1.0], [4.0, 4.0, 4.0]]], [[[4.0, 4.0, 3.0], [1.0, 
4.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 5.0, 2.0], [1.0, 3.0, 3.0], [4.0, 4.0, 5.0]], [[8.0, 8.0, 6.0], [3.0, 7.0, 6.0], [3.0, 8.0, 
3.0]]], [[[3.0, 7.0, 7.0], [2.0, 8.0, 4.0], [3.0, 2.0, 7.0]], [[4.0, 7.0, 4.0], [1.0, 3.0, 5.0], [8.0, 8.0, 3.0]], [[1.0, 2.0, 
3.0], [1.0, 7.0, 2.0], [8.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 1.0], [8.0, 6.0, 4.0], [7.0, 2.0, 7.0]], 
[[7.0, 3.0, 3.0], [6.0, 8.0, 6.0], [1.0, 8.0, 4.0]], [[3.0, 4.0, 2.0], [3.0, 6.0, 7.0], [4.0, 2.0, 5.0]]], [[[5.0, 3.0, 8.0], 
[5.0, 6.0, 8.0], [5.0, 4.0, 8.0]], [[3.0, 3.0, 1.0], [2.0, 4.0, 1.0], [8.0, 7.0, 5.0]], [[6.0, 8.0, 2.0], [4.0, 5.0, 6.0], 
[8.0, 6.0, 8.0]]], [[[8.0, 5.0, 3.0], [7.0, 6.0, 4.0], [6.0, 1.0, 4.0]], [[3.0, 6.0, 6.0], [8.0, 6.0, 3.0], [4.0, 8.0, 4.0]], 
[[2.0, 3.0, 1.0], [7.0, 1.0, 1.0], [5.0, 1.0, 4.0]]], [[[8.0, 1.0, 8.0], [8.0, 5.0, 5.0], [2.0, 2.0, 1.0]], [[7.0, 7.0, 8.0], 
[1.0, 1.0, 3.0], [3.0, 5.0, 4.0]], [[8.0, 8.0, 5.0], [8.0, 7.0, 8.0], [2.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 3.0], [6.0, 3.0, 4.0], [1.0, 5.0, 5.0]], [[5.0, 7.0, 3.0], [1.0, 8.0, 1.0], [3.0, 
8.0, 4.0]], [[3.0, 7.0, 4.0], [4.0, 3.0, 8.0], [1.0, 6.0, 6.0]]], [[[3.0, 3.0, 5.0], [3.0, 8.0, 6.0], [3.0, 1.0, 6.0]], [[1.0, 
5.0, 2.0], [5.0, 8.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 2.0, 1.0], [5.0, 8.0, 4.0], [6.0, 2.0, 2.0]]], [[[3.0, 2.0, 8.0], [1.0, 
4.0, 8.0], [8.0, 4.0, 5.0]], [[6.0, 3.0, 5.0], [1.0, 7.0, 1.0], [4.0, 8.0, 3.0]], [[4.0, 2.0, 7.0], [6.0, 6.0, 4.0], [3.0, 3.0, 
7.0]]], [[[3.0, 3.0, 3.0], [8.0, 5.0, 8.0], [4.0, 3.0, 1.0]], [[1.0, 4.0, 6.0], [8.0, 1.0, 5.0], [2.0, 2.0, 2.0]], [[8.0, 2.0, 
1.0], [5.0, 5.0, 2.0], [1.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 5.0], [2.0, 4.0, 8.0], [8.0, 3.0, 7.0]], 
[[2.0, 5.0, 1.0], [8.0, 8.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 8.0, 5.0], [4.0, 6.0, 5.0], [4.0, 5.0, 8.0]]], [[[7.0, 2.0, 6.0], 
[6.0, 4.0, 6.0], [1.0, 4.0, 4.0]], [[6.0, 7.0, 7.0], [4.0, 2.0, 4.0], [1.0, 2.0, 3.0]], [[8.0, 5.0, 8.0], [3.0, 7.0, 1.0], 
[6.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [1.0, 4.0, 6.0], [6.0, 7.0, 6.0]], [[2.0, 4.0, 6.0], [3.0, 2.0, 4.0], [7.0, 8.0, 6.0]], 
[[2.0, 4.0, 4.0], [3.0, 6.0, 7.0], [5.0, 2.0, 2.0]]], [[[5.0, 1.0, 7.0], [1.0, 1.0, 5.0], [1.0, 4.0, 5.0]], [[5.0, 5.0, 8.0], 
[1.0, 4.0, 2.0], [5.0, 1.0, 5.0]], [[7.0, 6.0, 7.0], [5.0, 6.0, 7.0], [6.0, 6.0, 1.0]]]])
      val=numpy.array([[5.0, 7.0, 3.0], [7.0, 4.0, 6.0], [6.0, 7.0, 1.0]])
      arg[2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 3.0], [6.0, 3.0, 4.0], [1.0, 5.0, 5.0]], [[5.0, 7.0, 3.0], [1.0, 8.0, 1.0], [3.0, 
8.0, 4.0]], [[3.0, 7.0, 4.0], [4.0, 3.0, 8.0], [1.0, 6.0, 6.0]]], [[[3.0, 3.0, 5.0], [3.0, 8.0, 6.0], [3.0, 1.0, 6.0]], [[1.0, 
5.0, 2.0], [5.0, 8.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 2.0, 1.0], [5.0, 8.0, 4.0], [6.0, 2.0, 2.0]]], [[[3.0, 2.0, 8.0], [5.0, 
7.0, 3.0], [8.0, 4.0, 5.0]], [[6.0, 3.0, 5.0], [7.0, 4.0, 6.0], [4.0, 8.0, 3.0]], [[4.0, 2.0, 7.0], [6.0, 7.0, 1.0], [3.0, 3.0, 
7.0]]], [[[3.0, 3.0, 3.0], [8.0, 5.0, 8.0], [4.0, 3.0, 1.0]], [[1.0, 4.0, 6.0], [8.0, 1.0, 5.0], [2.0, 2.0, 2.0]], [[8.0, 2.0, 
1.0], [5.0, 5.0, 2.0], [1.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 5.0], [2.0, 4.0, 8.0], [8.0, 3.0, 7.0]], 
[[2.0, 5.0, 1.0], [8.0, 8.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 8.0, 5.0], [4.0, 6.0, 5.0], [4.0, 5.0, 8.0]]], [[[7.0, 2.0, 6.0], 
[6.0, 4.0, 6.0], [1.0, 4.0, 4.0]], [[6.0, 7.0, 7.0], [4.0, 2.0, 4.0], [1.0, 2.0, 3.0]], [[8.0, 5.0, 8.0], [3.0, 7.0, 1.0], 
[6.0, 8.0, 8.0]]], [[[4.0, 8.0, 6.0], [5.0, 7.0, 3.0], [6.0, 7.0, 6.0]], [[2.0, 4.0, 6.0], [7.0, 4.0, 6.0], [7.0, 8.0, 6.0]], 
[[2.0, 4.0, 4.0], [6.0, 7.0, 1.0], [5.0, 2.0, 2.0]]], [[[5.0, 1.0, 7.0], [1.0, 1.0, 5.0], [1.0, 4.0, 5.0]], [[5.0, 5.0, 8.0], 
[1.0, 4.0, 2.0], [5.0, 1.0, 5.0]], [[7.0, 6.0, 7.0], [5.0, 6.0, 7.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 5.0], [4.0, 5.0, 1.0], [3.0, 8.0, 3.0]], [[3.0, 1.0, 3.0], [3.0, 4.0, 2.0], [7.0, 
6.0, 2.0]], [[1.0, 3.0, 5.0], [7.0, 7.0, 7.0], [3.0, 4.0, 2.0]]], [[[8.0, 3.0, 8.0], [2.0, 7.0, 1.0], [3.0, 8.0, 8.0]], [[3.0, 
4.0, 1.0], [2.0, 5.0, 7.0], [2.0, 8.0, 6.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 4.0], [5.0, 5.0, 6.0]]], [[[6.0, 5.0, 5.0], [6.0, 
6.0, 6.0], [7.0, 3.0, 6.0]], [[1.0, 1.0, 8.0], [4.0, 3.0, 3.0], [4.0, 5.0, 3.0]], [[8.0, 6.0, 3.0], [5.0, 4.0, 1.0], [5.0, 4.0, 
1.0]]], [[[7.0, 4.0, 2.0], [5.0, 7.0, 2.0], [2.0, 3.0, 7.0]], [[2.0, 5.0, 5.0], [7.0, 8.0, 4.0], [8.0, 2.0, 8.0]], [[3.0, 8.0, 
3.0], [1.0, 7.0, 2.0], [4.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 1.0], [7.0, 4.0, 4.0], [7.0, 6.0, 7.0]], 
[[8.0, 3.0, 2.0], [5.0, 7.0, 7.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 5.0], [6.0, 5.0, 4.0], [3.0, 5.0, 1.0]]], [[[7.0, 2.0, 4.0], 
[5.0, 3.0, 3.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 3.0], [2.0, 7.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 5.0], [4.0, 8.0, 8.0], 
[3.0, 2.0, 4.0]]], [[[8.0, 3.0, 6.0], [6.0, 4.0, 6.0], [7.0, 3.0, 5.0]], [[1.0, 7.0, 6.0], [5.0, 5.0, 7.0], [7.0, 7.0, 6.0]], 
[[8.0, 4.0, 3.0], [4.0, 6.0, 8.0], [3.0, 6.0, 3.0]]], [[[8.0, 2.0, 1.0], [2.0, 7.0, 7.0], [7.0, 7.0, 5.0]], [[2.0, 3.0, 3.0], 
[6.0, 8.0, 7.0], [6.0, 6.0, 3.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 5.0], [7.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[6.0, 1.0, 5.0], [4.0, 6.0, 1.0], [5.0, 4.0, 7.0]]),self.functionspace)
      arg[2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 5.0], [4.0, 5.0, 1.0], [3.0, 8.0, 3.0]], [[3.0, 1.0, 3.0], [3.0, 4.0, 2.0], [7.0, 
6.0, 2.0]], [[1.0, 3.0, 5.0], [7.0, 7.0, 7.0], [3.0, 4.0, 2.0]]], [[[8.0, 3.0, 8.0], [2.0, 7.0, 1.0], [3.0, 8.0, 8.0]], [[3.0, 
4.0, 1.0], [2.0, 5.0, 7.0], [2.0, 8.0, 6.0]], [[7.0, 4.0, 4.0], [6.0, 2.0, 4.0], [5.0, 5.0, 6.0]]], [[[6.0, 5.0, 5.0], [6.0, 
1.0, 5.0], [7.0, 3.0, 6.0]], [[1.0, 1.0, 8.0], [4.0, 6.0, 1.0], [4.0, 5.0, 3.0]], [[8.0, 6.0, 3.0], [5.0, 4.0, 7.0], [5.0, 4.0, 
1.0]]], [[[7.0, 4.0, 2.0], [5.0, 7.0, 2.0], [2.0, 3.0, 7.0]], [[2.0, 5.0, 5.0], [7.0, 8.0, 4.0], [8.0, 2.0, 8.0]], [[3.0, 8.0, 
3.0], [1.0, 7.0, 2.0], [4.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 1.0], [7.0, 4.0, 4.0], [7.0, 6.0, 7.0]], 
[[8.0, 3.0, 2.0], [5.0, 7.0, 7.0], [2.0, 7.0, 5.0]], [[3.0, 7.0, 5.0], [6.0, 5.0, 4.0], [3.0, 5.0, 1.0]]], [[[7.0, 2.0, 4.0], 
[5.0, 3.0, 3.0], [2.0, 8.0, 8.0]], [[6.0, 7.0, 3.0], [2.0, 7.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 5.0], [4.0, 8.0, 8.0], 
[3.0, 2.0, 4.0]]], [[[8.0, 3.0, 6.0], [6.0, 1.0, 5.0], [7.0, 3.0, 5.0]], [[1.0, 7.0, 6.0], [4.0, 6.0, 1.0], [7.0, 7.0, 6.0]], 
[[8.0, 4.0, 3.0], [5.0, 4.0, 7.0], [3.0, 6.0, 3.0]]], [[[8.0, 2.0, 1.0], [2.0, 7.0, 7.0], [7.0, 7.0, 5.0]], [[2.0, 3.0, 3.0], 
[6.0, 8.0, 7.0], [6.0, 6.0, 3.0]], [[7.0, 6.0, 2.0], [4.0, 2.0, 5.0], [7.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 8.0], [5.0, 8.0, 5.0], [1.0, 3.0, 7.0]], [[6.0, 5.0, 1.0], [4.0, 8.0, 4.0], [4.0, 
3.0, 4.0]], [[6.0, 3.0, 7.0], [7.0, 4.0, 6.0], [1.0, 6.0, 3.0]]], [[[5.0, 2.0, 8.0], [6.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[8.0, 
3.0, 4.0], [6.0, 6.0, 7.0], [2.0, 8.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 7.0, 1.0], [7.0, 8.0, 8.0]]], [[[7.0, 6.0, 1.0], [5.0, 
8.0, 6.0], [2.0, 5.0, 6.0]], [[8.0, 8.0, 6.0], [6.0, 5.0, 2.0], [8.0, 4.0, 6.0]], [[8.0, 1.0, 7.0], [2.0, 7.0, 6.0], [4.0, 2.0, 
4.0]]], [[[4.0, 5.0, 8.0], [1.0, 5.0, 8.0], [7.0, 2.0, 1.0]], [[5.0, 8.0, 1.0], [3.0, 1.0, 6.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 
2.0], [6.0, 7.0, 6.0], [4.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 1.0], [2.0, 2.0, 7.0], [2.0, 2.0, 5.0]], 
[[7.0, 4.0, 1.0], [2.0, 2.0, 2.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 8.0], [7.0, 7.0, 6.0], [6.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], 
[6.0, 7.0, 1.0], [7.0, 7.0, 2.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 8.0], [3.0, 7.0, 4.0]], [[1.0, 2.0, 8.0], [6.0, 1.0, 7.0], 
[2.0, 2.0, 4.0]]], [[[8.0, 8.0, 8.0], [1.0, 6.0, 1.0], [4.0, 2.0, 2.0]], [[2.0, 1.0, 4.0], [2.0, 6.0, 5.0], [1.0, 5.0, 7.0]], 
[[4.0, 4.0, 2.0], [2.0, 4.0, 3.0], [8.0, 7.0, 5.0]]], [[[4.0, 8.0, 3.0], [1.0, 1.0, 5.0], [4.0, 3.0, 7.0]], [[3.0, 4.0, 7.0], 
[2.0, 3.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 8.0], [4.0, 3.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 4.0, 2.0], [6.0, 5.0, 2.0], [2.0, 1.0, 1.0]])+(1.-msk_val)*numpy.array([[2.0, 7.0, 5.0], 
[6.0, 8.0, 5.0], [3.0, 4.0, 1.0]])
      arg[2,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 8.0], [5.0, 8.0, 5.0], [1.0, 3.0, 7.0]], [[6.0, 5.0, 1.0], [4.0, 8.0, 4.0], [4.0, 
3.0, 4.0]], [[6.0, 3.0, 7.0], [7.0, 4.0, 6.0], [1.0, 6.0, 3.0]]], [[[5.0, 2.0, 8.0], [6.0, 6.0, 6.0], [4.0, 7.0, 4.0]], [[8.0, 
3.0, 4.0], [6.0, 6.0, 7.0], [2.0, 8.0, 1.0]], [[3.0, 5.0, 2.0], [6.0, 7.0, 1.0], [7.0, 8.0, 8.0]]], [[[7.0, 6.0, 1.0], [2.0, 
4.0, 2.0], [2.0, 5.0, 6.0]], [[8.0, 8.0, 6.0], [6.0, 5.0, 2.0], [8.0, 4.0, 6.0]], [[8.0, 1.0, 7.0], [2.0, 1.0, 1.0], [4.0, 2.0, 
4.0]]], [[[4.0, 5.0, 8.0], [1.0, 5.0, 8.0], [7.0, 2.0, 1.0]], [[5.0, 8.0, 1.0], [3.0, 1.0, 6.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 
2.0], [6.0, 7.0, 6.0], [4.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 1.0], [2.0, 2.0, 7.0], [2.0, 2.0, 5.0]], 
[[7.0, 4.0, 1.0], [2.0, 2.0, 2.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 8.0], [7.0, 7.0, 6.0], [6.0, 8.0, 3.0]]], [[[1.0, 7.0, 3.0], 
[6.0, 7.0, 1.0], [7.0, 7.0, 2.0]], [[4.0, 2.0, 8.0], [6.0, 1.0, 8.0], [3.0, 7.0, 4.0]], [[1.0, 2.0, 8.0], [6.0, 1.0, 7.0], 
[2.0, 2.0, 4.0]]], [[[8.0, 8.0, 8.0], [2.0, 7.0, 5.0], [4.0, 2.0, 2.0]], [[2.0, 1.0, 4.0], [6.0, 8.0, 5.0], [1.0, 5.0, 7.0]], 
[[4.0, 4.0, 2.0], [3.0, 4.0, 1.0], [8.0, 7.0, 5.0]]], [[[4.0, 8.0, 3.0], [1.0, 1.0, 5.0], [4.0, 3.0, 7.0]], [[3.0, 4.0, 7.0], 
[2.0, 3.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 8.0], [4.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 3.0], [1.0, 5.0, 7.0], [4.0, 6.0, 8.0]], [[5.0, 7.0, 7.0], [4.0, 5.0, 2.0], [5.0, 
4.0, 5.0]], [[6.0, 8.0, 5.0], [7.0, 7.0, 6.0], [5.0, 4.0, 1.0]]], [[[4.0, 7.0, 1.0], [5.0, 2.0, 8.0], [2.0, 8.0, 2.0]], [[4.0, 
5.0, 4.0], [2.0, 2.0, 2.0], [4.0, 7.0, 8.0]], [[7.0, 2.0, 7.0], [6.0, 1.0, 8.0], [7.0, 5.0, 1.0]]], [[[4.0, 6.0, 2.0], [6.0, 
5.0, 1.0], [7.0, 7.0, 8.0]], [[5.0, 5.0, 1.0], [3.0, 4.0, 6.0], [5.0, 4.0, 5.0]], [[3.0, 1.0, 1.0], [4.0, 5.0, 1.0], [6.0, 4.0, 
8.0]]], [[[7.0, 3.0, 5.0], [1.0, 6.0, 8.0], [6.0, 4.0, 2.0]], [[5.0, 6.0, 8.0], [1.0, 3.0, 2.0], [7.0, 6.0, 5.0]], [[3.0, 8.0, 
2.0], [2.0, 3.0, 8.0], [5.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 6.0], [5.0, 8.0, 8.0], [8.0, 2.0, 7.0]], 
[[8.0, 6.0, 2.0], [8.0, 8.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 3.0, 2.0], [5.0, 4.0, 2.0], [7.0, 4.0, 2.0]]], [[[3.0, 3.0, 3.0], 
[8.0, 8.0, 4.0], [7.0, 5.0, 8.0]], [[5.0, 4.0, 3.0], [3.0, 2.0, 8.0], [2.0, 8.0, 2.0]], [[6.0, 4.0, 3.0], [4.0, 4.0, 3.0], 
[5.0, 8.0, 1.0]]], [[[6.0, 7.0, 5.0], [5.0, 2.0, 1.0], [3.0, 1.0, 2.0]], [[5.0, 8.0, 2.0], [4.0, 2.0, 1.0], [6.0, 8.0, 4.0]], 
[[6.0, 8.0, 2.0], [2.0, 1.0, 3.0], [5.0, 3.0, 4.0]]], [[[7.0, 8.0, 2.0], [4.0, 3.0, 8.0], [8.0, 2.0, 6.0]], [[8.0, 8.0, 8.0], 
[8.0, 3.0, 4.0], [3.0, 1.0, 8.0]], [[8.0, 8.0, 4.0], [4.0, 8.0, 7.0], [4.0, 6.0, 5.0]]]])
      val=numpy.array([[[7.0, 1.0, 2.0], [6.0, 5.0, 7.0], [8.0, 4.0, 4.0]], [[6.0, 5.0, 6.0], [2.0, 4.0, 8.0], [7.0, 7.0, 
3.0]]])
      arg[1:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 3.0], [1.0, 5.0, 7.0], [4.0, 6.0, 8.0]], [[5.0, 7.0, 7.0], [4.0, 5.0, 2.0], [5.0, 
4.0, 5.0]], [[6.0, 8.0, 5.0], [7.0, 7.0, 6.0], [5.0, 4.0, 1.0]]], [[[4.0, 7.0, 1.0], [5.0, 2.0, 8.0], [2.0, 8.0, 2.0]], [[7.0, 
1.0, 2.0], [6.0, 5.0, 7.0], [8.0, 4.0, 4.0]], [[7.0, 2.0, 7.0], [6.0, 1.0, 8.0], [7.0, 5.0, 1.0]]], [[[4.0, 6.0, 2.0], [6.0, 
5.0, 1.0], [7.0, 7.0, 8.0]], [[6.0, 5.0, 6.0], [2.0, 4.0, 8.0], [7.0, 7.0, 3.0]], [[3.0, 1.0, 1.0], [4.0, 5.0, 1.0], [6.0, 4.0, 
8.0]]], [[[7.0, 3.0, 5.0], [1.0, 6.0, 8.0], [6.0, 4.0, 2.0]], [[5.0, 6.0, 8.0], [1.0, 3.0, 2.0], [7.0, 6.0, 5.0]], [[3.0, 8.0, 
2.0], [2.0, 3.0, 8.0], [5.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 6.0], [5.0, 8.0, 8.0], [8.0, 2.0, 7.0]], 
[[8.0, 6.0, 2.0], [8.0, 8.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 3.0, 2.0], [5.0, 4.0, 2.0], [7.0, 4.0, 2.0]]], [[[3.0, 3.0, 3.0], 
[8.0, 8.0, 4.0], [7.0, 5.0, 8.0]], [[7.0, 1.0, 2.0], [6.0, 5.0, 7.0], [8.0, 4.0, 4.0]], [[6.0, 4.0, 3.0], [4.0, 4.0, 3.0], 
[5.0, 8.0, 1.0]]], [[[6.0, 7.0, 5.0], [5.0, 2.0, 1.0], [3.0, 1.0, 2.0]], [[6.0, 5.0, 6.0], [2.0, 4.0, 8.0], [7.0, 7.0, 3.0]], 
[[6.0, 8.0, 2.0], [2.0, 1.0, 3.0], [5.0, 3.0, 4.0]]], [[[7.0, 8.0, 2.0], [4.0, 3.0, 8.0], [8.0, 2.0, 6.0]], [[8.0, 8.0, 8.0], 
[8.0, 3.0, 4.0], [3.0, 1.0, 8.0]], [[8.0, 8.0, 4.0], [4.0, 8.0, 7.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 5.0], [1.0, 8.0, 8.0], [8.0, 4.0, 5.0]], [[2.0, 2.0, 3.0], [4.0, 8.0, 8.0], [1.0, 
8.0, 3.0]], [[4.0, 7.0, 3.0], [4.0, 3.0, 1.0], [4.0, 6.0, 6.0]]], [[[5.0, 8.0, 6.0], [5.0, 6.0, 1.0], [6.0, 7.0, 7.0]], [[6.0, 
6.0, 2.0], [1.0, 2.0, 7.0], [8.0, 1.0, 6.0]], [[3.0, 5.0, 1.0], [1.0, 2.0, 4.0], [2.0, 8.0, 3.0]]], [[[2.0, 4.0, 7.0], [6.0, 
8.0, 6.0], [3.0, 6.0, 5.0]], [[5.0, 2.0, 2.0], [3.0, 8.0, 2.0], [6.0, 1.0, 6.0]], [[8.0, 4.0, 2.0], [6.0, 5.0, 6.0], [2.0, 3.0, 
6.0]]], [[[3.0, 8.0, 4.0], [8.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[2.0, 5.0, 4.0], [2.0, 8.0, 5.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 
1.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 4.0], [6.0, 1.0, 6.0], [3.0, 6.0, 4.0]], 
[[5.0, 1.0, 7.0], [4.0, 7.0, 6.0], [4.0, 8.0, 5.0]], [[1.0, 6.0, 6.0], [2.0, 7.0, 4.0], [6.0, 2.0, 4.0]]], [[[5.0, 1.0, 8.0], 
[6.0, 4.0, 1.0], [8.0, 2.0, 4.0]], [[1.0, 6.0, 6.0], [7.0, 7.0, 1.0], [2.0, 1.0, 1.0]], [[7.0, 1.0, 2.0], [2.0, 4.0, 8.0], 
[4.0, 2.0, 7.0]]], [[[3.0, 4.0, 5.0], [2.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[3.0, 8.0, 7.0], [7.0, 4.0, 6.0], [4.0, 5.0, 1.0]], 
[[3.0, 1.0, 8.0], [4.0, 6.0, 3.0], [3.0, 3.0, 3.0]]], [[[6.0, 4.0, 7.0], [4.0, 6.0, 3.0], [2.0, 3.0, 6.0]], [[3.0, 5.0, 5.0], 
[4.0, 6.0, 1.0], [2.0, 3.0, 2.0]], [[8.0, 6.0, 6.0], [8.0, 3.0, 2.0], [3.0, 8.0, 3.0]]]])
      val=Data(numpy.array([[[4.0, 7.0, 8.0], [6.0, 5.0, 1.0], [4.0, 4.0, 7.0]], [[5.0, 7.0, 1.0], [4.0, 6.0, 6.0], [4.0, 2.0, 
7.0]]]),self.functionspace)
      arg[1:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 5.0], [1.0, 8.0, 8.0], [8.0, 4.0, 5.0]], [[2.0, 2.0, 3.0], [4.0, 8.0, 8.0], [1.0, 
8.0, 3.0]], [[4.0, 7.0, 3.0], [4.0, 3.0, 1.0], [4.0, 6.0, 6.0]]], [[[5.0, 8.0, 6.0], [5.0, 6.0, 1.0], [6.0, 7.0, 7.0]], [[4.0, 
7.0, 8.0], [6.0, 5.0, 1.0], [4.0, 4.0, 7.0]], [[3.0, 5.0, 1.0], [1.0, 2.0, 4.0], [2.0, 8.0, 3.0]]], [[[2.0, 4.0, 7.0], [6.0, 
8.0, 6.0], [3.0, 6.0, 5.0]], [[5.0, 7.0, 1.0], [4.0, 6.0, 6.0], [4.0, 2.0, 7.0]], [[8.0, 4.0, 2.0], [6.0, 5.0, 6.0], [2.0, 3.0, 
6.0]]], [[[3.0, 8.0, 4.0], [8.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[2.0, 5.0, 4.0], [2.0, 8.0, 5.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 
1.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 4.0], [6.0, 1.0, 6.0], [3.0, 6.0, 4.0]], 
[[5.0, 1.0, 7.0], [4.0, 7.0, 6.0], [4.0, 8.0, 5.0]], [[1.0, 6.0, 6.0], [2.0, 7.0, 4.0], [6.0, 2.0, 4.0]]], [[[5.0, 1.0, 8.0], 
[6.0, 4.0, 1.0], [8.0, 2.0, 4.0]], [[4.0, 7.0, 8.0], [6.0, 5.0, 1.0], [4.0, 4.0, 7.0]], [[7.0, 1.0, 2.0], [2.0, 4.0, 8.0], 
[4.0, 2.0, 7.0]]], [[[3.0, 4.0, 5.0], [2.0, 5.0, 6.0], [4.0, 2.0, 1.0]], [[5.0, 7.0, 1.0], [4.0, 6.0, 6.0], [4.0, 2.0, 7.0]], 
[[3.0, 1.0, 8.0], [4.0, 6.0, 3.0], [3.0, 3.0, 3.0]]], [[[6.0, 4.0, 7.0], [4.0, 6.0, 3.0], [2.0, 3.0, 6.0]], [[3.0, 5.0, 5.0], 
[4.0, 6.0, 1.0], [2.0, 3.0, 2.0]], [[8.0, 6.0, 6.0], [8.0, 3.0, 2.0], [3.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 5.0], [4.0, 1.0, 4.0], [5.0, 7.0, 5.0]], [[5.0, 4.0, 7.0], [7.0, 6.0, 5.0], [1.0, 
5.0, 7.0]], [[5.0, 5.0, 2.0], [7.0, 3.0, 2.0], [3.0, 4.0, 1.0]]], [[[3.0, 4.0, 6.0], [8.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[4.0, 
4.0, 6.0], [4.0, 4.0, 8.0], [7.0, 1.0, 4.0]], [[6.0, 8.0, 3.0], [8.0, 7.0, 6.0], [3.0, 8.0, 1.0]]], [[[8.0, 4.0, 8.0], [5.0, 
5.0, 7.0], [2.0, 8.0, 2.0]], [[7.0, 2.0, 7.0], [7.0, 7.0, 2.0], [5.0, 6.0, 6.0]], [[3.0, 6.0, 1.0], [1.0, 1.0, 5.0], [4.0, 6.0, 
4.0]]], [[[1.0, 5.0, 7.0], [2.0, 3.0, 5.0], [4.0, 4.0, 1.0]], [[5.0, 1.0, 7.0], [6.0, 3.0, 1.0], [4.0, 7.0, 7.0]], [[2.0, 4.0, 
1.0], [4.0, 1.0, 5.0], [1.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 2.0], [7.0, 4.0, 6.0], [3.0, 5.0, 8.0]], 
[[4.0, 7.0, 4.0], [7.0, 4.0, 7.0], [4.0, 1.0, 4.0]], [[7.0, 6.0, 5.0], [1.0, 5.0, 5.0], [7.0, 4.0, 2.0]]], [[[3.0, 6.0, 3.0], 
[6.0, 2.0, 6.0], [4.0, 2.0, 1.0]], [[3.0, 8.0, 8.0], [1.0, 4.0, 5.0], [2.0, 2.0, 7.0]], [[7.0, 3.0, 8.0], [5.0, 4.0, 1.0], 
[6.0, 7.0, 5.0]]], [[[7.0, 3.0, 1.0], [3.0, 4.0, 3.0], [5.0, 2.0, 8.0]], [[7.0, 7.0, 7.0], [6.0, 5.0, 7.0], [8.0, 4.0, 6.0]], 
[[2.0, 3.0, 6.0], [6.0, 8.0, 3.0], [3.0, 1.0, 7.0]]], [[[7.0, 6.0, 5.0], [7.0, 3.0, 1.0], [6.0, 8.0, 5.0]], [[5.0, 2.0, 7.0], 
[2.0, 2.0, 8.0], [7.0, 8.0, 7.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 6.0], [7.0, 2.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 7.0, 6.0], [5.0, 2.0, 4.0], [4.0, 3.0, 1.0]], [[1.0, 4.0, 8.0], [2.0, 5.0, 4.0], [8.0, 
6.0, 1.0]]])+(1.-msk_val)*numpy.array([[[2.0, 3.0, 1.0], [7.0, 2.0, 8.0], [5.0, 8.0, 2.0]], [[1.0, 7.0, 2.0], [7.0, 4.0, 2.0], 
[6.0, 3.0, 6.0]]])
      arg[1:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 5.0], [4.0, 1.0, 4.0], [5.0, 7.0, 5.0]], [[5.0, 4.0, 7.0], [7.0, 6.0, 5.0], [1.0, 
5.0, 7.0]], [[5.0, 5.0, 2.0], [7.0, 3.0, 2.0], [3.0, 4.0, 1.0]]], [[[3.0, 4.0, 6.0], [8.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[8.0, 
7.0, 6.0], [5.0, 2.0, 4.0], [4.0, 3.0, 1.0]], [[6.0, 8.0, 3.0], [8.0, 7.0, 6.0], [3.0, 8.0, 1.0]]], [[[8.0, 4.0, 8.0], [5.0, 
5.0, 7.0], [2.0, 8.0, 2.0]], [[1.0, 4.0, 8.0], [2.0, 5.0, 4.0], [8.0, 6.0, 1.0]], [[3.0, 6.0, 1.0], [1.0, 1.0, 5.0], [4.0, 6.0, 
4.0]]], [[[1.0, 5.0, 7.0], [2.0, 3.0, 5.0], [4.0, 4.0, 1.0]], [[5.0, 1.0, 7.0], [6.0, 3.0, 1.0], [4.0, 7.0, 7.0]], [[2.0, 4.0, 
1.0], [4.0, 1.0, 5.0], [1.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 2.0], [7.0, 4.0, 6.0], [3.0, 5.0, 8.0]], 
[[4.0, 7.0, 4.0], [7.0, 4.0, 7.0], [4.0, 1.0, 4.0]], [[7.0, 6.0, 5.0], [1.0, 5.0, 5.0], [7.0, 4.0, 2.0]]], [[[3.0, 6.0, 3.0], 
[6.0, 2.0, 6.0], [4.0, 2.0, 1.0]], [[2.0, 3.0, 1.0], [7.0, 2.0, 8.0], [5.0, 8.0, 2.0]], [[7.0, 3.0, 8.0], [5.0, 4.0, 1.0], 
[6.0, 7.0, 5.0]]], [[[7.0, 3.0, 1.0], [3.0, 4.0, 3.0], [5.0, 2.0, 8.0]], [[1.0, 7.0, 2.0], [7.0, 4.0, 2.0], [6.0, 3.0, 6.0]], 
[[2.0, 3.0, 6.0], [6.0, 8.0, 3.0], [3.0, 1.0, 7.0]]], [[[7.0, 6.0, 5.0], [7.0, 3.0, 1.0], [6.0, 8.0, 5.0]], [[5.0, 2.0, 7.0], 
[2.0, 2.0, 8.0], [7.0, 8.0, 7.0]], [[5.0, 2.0, 1.0], [6.0, 6.0, 6.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UUUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 6.0], [5.0, 4.0, 7.0], [4.0, 8.0, 1.0]], [[5.0, 3.0, 8.0], [3.0, 3.0, 2.0], [8.0, 
8.0, 3.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 8.0], [1.0, 8.0, 1.0]]], [[[3.0, 8.0, 1.0], [7.0, 6.0, 1.0], [5.0, 8.0, 3.0]], [[2.0, 
6.0, 8.0], [8.0, 3.0, 2.0], [3.0, 7.0, 7.0]], [[5.0, 8.0, 5.0], [1.0, 3.0, 1.0], [6.0, 3.0, 7.0]]], [[[5.0, 8.0, 8.0], [6.0, 
1.0, 8.0], [6.0, 6.0, 3.0]], [[1.0, 7.0, 6.0], [4.0, 2.0, 1.0], [4.0, 1.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 6.0, 1.0], [6.0, 7.0, 
3.0]]], [[[2.0, 4.0, 6.0], [5.0, 1.0, 8.0], [1.0, 3.0, 4.0]], [[4.0, 8.0, 7.0], [1.0, 1.0, 5.0], [3.0, 2.0, 7.0]], [[4.0, 2.0, 
8.0], [3.0, 3.0, 2.0], [8.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 2.0], [5.0, 1.0, 5.0], [5.0, 6.0, 8.0]], 
[[6.0, 5.0, 1.0], [6.0, 7.0, 4.0], [8.0, 5.0, 4.0]], [[6.0, 4.0, 5.0], [7.0, 1.0, 2.0], [5.0, 5.0, 7.0]]], [[[6.0, 1.0, 6.0], 
[1.0, 5.0, 7.0], [1.0, 6.0, 6.0]], [[3.0, 3.0, 8.0], [6.0, 5.0, 6.0], [4.0, 7.0, 3.0]], [[8.0, 8.0, 8.0], [2.0, 4.0, 4.0], 
[3.0, 1.0, 7.0]]], [[[1.0, 1.0, 7.0], [6.0, 2.0, 2.0], [5.0, 2.0, 2.0]], [[2.0, 2.0, 4.0], [4.0, 2.0, 8.0], [8.0, 7.0, 2.0]], 
[[6.0, 2.0, 8.0], [7.0, 8.0, 4.0], [7.0, 1.0, 4.0]]], [[[2.0, 8.0, 4.0], [2.0, 3.0, 8.0], [3.0, 6.0, 5.0]], [[5.0, 4.0, 4.0], 
[8.0, 8.0, 4.0], [3.0, 8.0, 8.0]], [[6.0, 2.0, 5.0], [1.0, 4.0, 5.0], [2.0, 2.0, 1.0]]]])
      val=numpy.array([[[8.0, 8.0], [7.0, 4.0]], [[4.0, 3.0], [5.0, 7.0]], [[1.0, 3.0], [4.0, 5.0]]])
      arg[:3,:2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 6.0], [5.0, 8.0, 7.0], [4.0, 8.0, 1.0]], [[5.0, 7.0, 8.0], [3.0, 4.0, 2.0], [8.0, 
8.0, 3.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 8.0], [1.0, 8.0, 1.0]]], [[[3.0, 4.0, 1.0], [7.0, 3.0, 1.0], [5.0, 8.0, 3.0]], [[2.0, 
5.0, 8.0], [8.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[5.0, 8.0, 5.0], [1.0, 3.0, 1.0], [6.0, 3.0, 7.0]]], [[[5.0, 1.0, 8.0], [6.0, 
3.0, 8.0], [6.0, 6.0, 3.0]], [[1.0, 4.0, 6.0], [4.0, 5.0, 1.0], [4.0, 1.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 6.0, 1.0], [6.0, 7.0, 
3.0]]], [[[2.0, 4.0, 6.0], [5.0, 1.0, 8.0], [1.0, 3.0, 4.0]], [[4.0, 8.0, 7.0], [1.0, 1.0, 5.0], [3.0, 2.0, 7.0]], [[4.0, 2.0, 
8.0], [3.0, 3.0, 2.0], [8.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [5.0, 8.0, 5.0], [5.0, 6.0, 8.0]], 
[[6.0, 7.0, 1.0], [6.0, 4.0, 4.0], [8.0, 5.0, 4.0]], [[6.0, 4.0, 5.0], [7.0, 1.0, 2.0], [5.0, 5.0, 7.0]]], [[[6.0, 4.0, 6.0], 
[1.0, 3.0, 7.0], [1.0, 6.0, 6.0]], [[3.0, 5.0, 8.0], [6.0, 7.0, 6.0], [4.0, 7.0, 3.0]], [[8.0, 8.0, 8.0], [2.0, 4.0, 4.0], 
[3.0, 1.0, 7.0]]], [[[1.0, 1.0, 7.0], [6.0, 3.0, 2.0], [5.0, 2.0, 2.0]], [[2.0, 4.0, 4.0], [4.0, 5.0, 8.0], [8.0, 7.0, 2.0]], 
[[6.0, 2.0, 8.0], [7.0, 8.0, 4.0], [7.0, 1.0, 4.0]]], [[[2.0, 8.0, 4.0], [2.0, 3.0, 8.0], [3.0, 6.0, 5.0]], [[5.0, 4.0, 4.0], 
[8.0, 8.0, 4.0], [3.0, 8.0, 8.0]], [[6.0, 2.0, 5.0], [1.0, 4.0, 5.0], [2.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UUUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 2.0], [4.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[4.0, 1.0, 5.0], [3.0, 4.0, 8.0], [6.0, 
1.0, 5.0]], [[4.0, 2.0, 7.0], [2.0, 3.0, 5.0], [3.0, 6.0, 4.0]]], [[[5.0, 2.0, 5.0], [4.0, 3.0, 2.0], [6.0, 8.0, 8.0]], [[3.0, 
6.0, 2.0], [8.0, 1.0, 8.0], [7.0, 3.0, 7.0]], [[2.0, 6.0, 5.0], [1.0, 5.0, 2.0], [6.0, 3.0, 3.0]]], [[[2.0, 7.0, 3.0], [5.0, 
8.0, 2.0], [7.0, 8.0, 4.0]], [[5.0, 8.0, 1.0], [7.0, 8.0, 7.0], [5.0, 4.0, 6.0]], [[2.0, 7.0, 2.0], [5.0, 5.0, 3.0], [1.0, 2.0, 
4.0]]], [[[8.0, 5.0, 4.0], [7.0, 8.0, 4.0], [3.0, 8.0, 2.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 5.0], [8.0, 6.0, 8.0]], [[1.0, 5.0, 
5.0], [7.0, 4.0, 1.0], [8.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 4.0], [6.0, 7.0, 2.0], [8.0, 7.0, 4.0]], 
[[2.0, 8.0, 4.0], [3.0, 2.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 5.0, 3.0], [7.0, 2.0, 1.0], [3.0, 4.0, 4.0]]], [[[1.0, 3.0, 3.0], 
[8.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[2.0, 7.0, 4.0], [7.0, 4.0, 6.0], [6.0, 4.0, 5.0]], [[5.0, 1.0, 1.0], [5.0, 3.0, 4.0], 
[4.0, 2.0, 6.0]]], [[[7.0, 8.0, 7.0], [7.0, 2.0, 3.0], [4.0, 1.0, 6.0]], [[6.0, 6.0, 8.0], [2.0, 7.0, 3.0], [3.0, 4.0, 6.0]], 
[[6.0, 5.0, 3.0], [7.0, 8.0, 2.0], [7.0, 7.0, 3.0]]], [[[2.0, 3.0, 2.0], [3.0, 7.0, 5.0], [4.0, 5.0, 6.0]], [[8.0, 5.0, 3.0], 
[6.0, 4.0, 1.0], [8.0, 3.0, 7.0]], [[7.0, 8.0, 8.0], [4.0, 4.0, 5.0], [1.0, 3.0, 2.0]]]])
      val=Data(numpy.array([[[6.0, 2.0], [5.0, 1.0]], [[2.0, 4.0], [5.0, 7.0]], [[3.0, 4.0], [5.0, 5.0]]]),self.functionspace)
      arg[:3,:2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 2.0], [4.0, 2.0, 2.0], [3.0, 3.0, 6.0]], [[4.0, 5.0, 5.0], [3.0, 1.0, 8.0], [6.0, 
1.0, 5.0]], [[4.0, 2.0, 7.0], [2.0, 3.0, 5.0], [3.0, 6.0, 4.0]]], [[[5.0, 2.0, 5.0], [4.0, 4.0, 2.0], [6.0, 8.0, 8.0]], [[3.0, 
5.0, 2.0], [8.0, 7.0, 8.0], [7.0, 3.0, 7.0]], [[2.0, 6.0, 5.0], [1.0, 5.0, 2.0], [6.0, 3.0, 3.0]]], [[[2.0, 3.0, 3.0], [5.0, 
4.0, 2.0], [7.0, 8.0, 4.0]], [[5.0, 5.0, 1.0], [7.0, 5.0, 7.0], [5.0, 4.0, 6.0]], [[2.0, 7.0, 2.0], [5.0, 5.0, 3.0], [1.0, 2.0, 
4.0]]], [[[8.0, 5.0, 4.0], [7.0, 8.0, 4.0], [3.0, 8.0, 2.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 5.0], [8.0, 6.0, 8.0]], [[1.0, 5.0, 
5.0], [7.0, 4.0, 1.0], [8.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 4.0], [6.0, 2.0, 2.0], [8.0, 7.0, 4.0]], 
[[2.0, 5.0, 4.0], [3.0, 1.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 5.0, 3.0], [7.0, 2.0, 1.0], [3.0, 4.0, 4.0]]], [[[1.0, 2.0, 3.0], 
[8.0, 4.0, 7.0], [7.0, 5.0, 6.0]], [[2.0, 5.0, 4.0], [7.0, 7.0, 6.0], [6.0, 4.0, 5.0]], [[5.0, 1.0, 1.0], [5.0, 3.0, 4.0], 
[4.0, 2.0, 6.0]]], [[[7.0, 3.0, 7.0], [7.0, 4.0, 3.0], [4.0, 1.0, 6.0]], [[6.0, 5.0, 8.0], [2.0, 5.0, 3.0], [3.0, 4.0, 6.0]], 
[[6.0, 5.0, 3.0], [7.0, 8.0, 2.0], [7.0, 7.0, 3.0]]], [[[2.0, 3.0, 2.0], [3.0, 7.0, 5.0], [4.0, 5.0, 6.0]], [[8.0, 5.0, 3.0], 
[6.0, 4.0, 1.0], [8.0, 3.0, 7.0]], [[7.0, 8.0, 8.0], [4.0, 4.0, 5.0], [1.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UUUP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 6.0], [8.0, 8.0, 4.0], [2.0, 6.0, 7.0]], [[5.0, 2.0, 2.0], [7.0, 4.0, 4.0], [2.0, 
5.0, 7.0]], [[1.0, 3.0, 6.0], [7.0, 1.0, 8.0], [3.0, 8.0, 1.0]]], [[[2.0, 4.0, 6.0], [2.0, 1.0, 5.0], [8.0, 2.0, 1.0]], [[8.0, 
3.0, 2.0], [6.0, 5.0, 1.0], [8.0, 1.0, 5.0]], [[8.0, 2.0, 7.0], [4.0, 4.0, 3.0], [2.0, 5.0, 8.0]]], [[[1.0, 7.0, 4.0], [3.0, 
4.0, 1.0], [1.0, 1.0, 4.0]], [[8.0, 8.0, 5.0], [3.0, 8.0, 5.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 2.0, 5.0], [3.0, 6.0, 
1.0]]], [[[3.0, 4.0, 1.0], [2.0, 2.0, 6.0], [1.0, 3.0, 3.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 8.0, 
7.0], [4.0, 8.0, 3.0], [5.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 4.0], [4.0, 1.0, 1.0], [1.0, 6.0, 5.0]], 
[[7.0, 5.0, 1.0], [2.0, 3.0, 8.0], [5.0, 5.0, 5.0]], [[2.0, 3.0, 8.0], [1.0, 6.0, 8.0], [6.0, 5.0, 4.0]]], [[[5.0, 3.0, 1.0], 
[1.0, 1.0, 4.0], [5.0, 5.0, 8.0]], [[6.0, 6.0, 4.0], [3.0, 5.0, 4.0], [3.0, 4.0, 5.0]], [[2.0, 7.0, 6.0], [7.0, 3.0, 5.0], 
[4.0, 4.0, 8.0]]], [[[2.0, 1.0, 2.0], [7.0, 8.0, 7.0], [8.0, 3.0, 5.0]], [[5.0, 3.0, 4.0], [4.0, 3.0, 4.0], [1.0, 7.0, 3.0]], 
[[6.0, 5.0, 4.0], [3.0, 4.0, 3.0], [8.0, 3.0, 7.0]]], [[[1.0, 2.0, 4.0], [1.0, 1.0, 1.0], [7.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], 
[7.0, 5.0, 4.0], [1.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], [2.0, 3.0, 6.0], [1.0, 5.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 4.0], [3.0, 3.0]], [[2.0, 1.0], [5.0, 5.0]], [[4.0, 1.0], [6.0, 
5.0]]])+(1.-msk_val)*numpy.array([[[3.0, 2.0], [2.0, 5.0]], [[3.0, 5.0], [2.0, 2.0]], [[5.0, 7.0], [6.0, 3.0]]])
      arg[:3,:2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 6.0], [8.0, 4.0, 4.0], [2.0, 6.0, 7.0]], [[5.0, 3.0, 2.0], [7.0, 3.0, 4.0], [2.0, 
5.0, 7.0]], [[1.0, 3.0, 6.0], [7.0, 1.0, 8.0], [3.0, 8.0, 1.0]]], [[[2.0, 2.0, 6.0], [2.0, 1.0, 5.0], [8.0, 2.0, 1.0]], [[8.0, 
5.0, 2.0], [6.0, 5.0, 1.0], [8.0, 1.0, 5.0]], [[8.0, 2.0, 7.0], [4.0, 4.0, 3.0], [2.0, 5.0, 8.0]]], [[[1.0, 4.0, 4.0], [3.0, 
1.0, 1.0], [1.0, 1.0, 4.0]], [[8.0, 6.0, 5.0], [3.0, 5.0, 5.0], [1.0, 4.0, 8.0]], [[3.0, 5.0, 1.0], [1.0, 2.0, 5.0], [3.0, 6.0, 
1.0]]], [[[3.0, 4.0, 1.0], [2.0, 2.0, 6.0], [1.0, 3.0, 3.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 3.0], [2.0, 5.0, 7.0]], [[7.0, 8.0, 
7.0], [4.0, 8.0, 3.0], [5.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 4.0], [4.0, 2.0, 1.0], [1.0, 6.0, 5.0]], 
[[7.0, 2.0, 1.0], [2.0, 5.0, 8.0], [5.0, 5.0, 5.0]], [[2.0, 3.0, 8.0], [1.0, 6.0, 8.0], [6.0, 5.0, 4.0]]], [[[5.0, 3.0, 1.0], 
[1.0, 5.0, 4.0], [5.0, 5.0, 8.0]], [[6.0, 2.0, 4.0], [3.0, 2.0, 4.0], [3.0, 4.0, 5.0]], [[2.0, 7.0, 6.0], [7.0, 3.0, 5.0], 
[4.0, 4.0, 8.0]]], [[[2.0, 5.0, 2.0], [7.0, 7.0, 7.0], [8.0, 3.0, 5.0]], [[5.0, 6.0, 4.0], [4.0, 3.0, 4.0], [1.0, 7.0, 3.0]], 
[[6.0, 5.0, 4.0], [3.0, 4.0, 3.0], [8.0, 3.0, 7.0]]], [[[1.0, 2.0, 4.0], [1.0, 1.0, 1.0], [7.0, 6.0, 5.0]], [[4.0, 2.0, 3.0], 
[7.0, 5.0, 4.0], [1.0, 6.0, 6.0]], [[3.0, 1.0, 2.0], [2.0, 3.0, 6.0], [1.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 8.0], [6.0, 3.0, 8.0], [7.0, 8.0, 6.0]], [[8.0, 5.0, 7.0], [3.0, 8.0, 6.0], [8.0, 
8.0, 5.0]], [[2.0, 5.0, 2.0], [8.0, 8.0, 6.0], [7.0, 1.0, 4.0]]], [[[7.0, 2.0, 6.0], [7.0, 4.0, 8.0], [1.0, 5.0, 3.0]], [[5.0, 
5.0, 7.0], [6.0, 8.0, 7.0], [1.0, 7.0, 6.0]], [[1.0, 8.0, 4.0], [2.0, 6.0, 8.0], [3.0, 1.0, 6.0]]], [[[5.0, 2.0, 2.0], [5.0, 
8.0, 1.0], [3.0, 5.0, 5.0]], [[5.0, 2.0, 8.0], [5.0, 3.0, 7.0], [2.0, 8.0, 6.0]], [[4.0, 7.0, 6.0], [4.0, 4.0, 7.0], [3.0, 3.0, 
6.0]]], [[[4.0, 7.0, 5.0], [7.0, 4.0, 3.0], [3.0, 1.0, 1.0]], [[7.0, 1.0, 3.0], [4.0, 1.0, 1.0], [2.0, 5.0, 8.0]], [[7.0, 3.0, 
1.0], [7.0, 2.0, 8.0], [3.0, 2.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 7.0], [1.0, 3.0, 8.0], [2.0, 5.0, 7.0]], 
[[7.0, 4.0, 4.0], [2.0, 7.0, 7.0], [5.0, 1.0, 4.0]], [[3.0, 8.0, 6.0], [2.0, 8.0, 7.0], [5.0, 6.0, 5.0]]], [[[8.0, 1.0, 1.0], 
[2.0, 6.0, 3.0], [7.0, 4.0, 1.0]], [[2.0, 3.0, 1.0], [3.0, 6.0, 5.0], [7.0, 3.0, 1.0]], [[5.0, 8.0, 5.0], [8.0, 7.0, 5.0], 
[3.0, 2.0, 8.0]]], [[[5.0, 1.0, 6.0], [8.0, 1.0, 7.0], [8.0, 6.0, 8.0]], [[2.0, 5.0, 6.0], [8.0, 1.0, 7.0], [5.0, 5.0, 2.0]], 
[[5.0, 5.0, 3.0], [6.0, 8.0, 8.0], [7.0, 6.0, 4.0]]], [[[3.0, 5.0, 7.0], [2.0, 3.0, 8.0], [3.0, 8.0, 5.0]], [[5.0, 2.0, 1.0], 
[8.0, 1.0, 5.0], [5.0, 4.0, 3.0]], [[3.0, 5.0, 1.0], [8.0, 7.0, 5.0], [4.0, 5.0, 2.0]]]])
      val=numpy.array([[[[6.0], [1.0], [5.0]], [[2.0], [1.0], [1.0]], [[4.0], [4.0], [1.0]]], [[[6.0], [8.0], [2.0]], [[3.0], 
[8.0], [1.0]], [[8.0], [8.0], [2.0]]], [[[2.0], [7.0], [4.0]], [[7.0], [5.0], [5.0]], [[8.0], [5.0], [7.0]]], [[[7.0], [5.0], 
[6.0]], [[8.0], [2.0], [8.0]], [[5.0], [5.0], [3.0]]]])
      arg[:,:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 6.0], [6.0, 3.0, 1.0], [7.0, 8.0, 5.0]], [[8.0, 5.0, 2.0], [3.0, 8.0, 1.0], [8.0, 
8.0, 1.0]], [[2.0, 5.0, 4.0], [8.0, 8.0, 4.0], [7.0, 1.0, 1.0]]], [[[7.0, 2.0, 6.0], [7.0, 4.0, 8.0], [1.0, 5.0, 2.0]], [[5.0, 
5.0, 3.0], [6.0, 8.0, 8.0], [1.0, 7.0, 1.0]], [[1.0, 8.0, 8.0], [2.0, 6.0, 8.0], [3.0, 1.0, 2.0]]], [[[5.0, 2.0, 2.0], [5.0, 
8.0, 7.0], [3.0, 5.0, 4.0]], [[5.0, 2.0, 7.0], [5.0, 3.0, 5.0], [2.0, 8.0, 5.0]], [[4.0, 7.0, 8.0], [4.0, 4.0, 5.0], [3.0, 3.0, 
7.0]]], [[[4.0, 7.0, 7.0], [7.0, 4.0, 5.0], [3.0, 1.0, 6.0]], [[7.0, 1.0, 8.0], [4.0, 1.0, 2.0], [2.0, 5.0, 8.0]], [[7.0, 3.0, 
5.0], [7.0, 2.0, 5.0], [3.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 6.0], [1.0, 3.0, 1.0], [2.0, 5.0, 5.0]], 
[[7.0, 4.0, 2.0], [2.0, 7.0, 1.0], [5.0, 1.0, 1.0]], [[3.0, 8.0, 4.0], [2.0, 8.0, 4.0], [5.0, 6.0, 1.0]]], [[[8.0, 1.0, 6.0], 
[2.0, 6.0, 8.0], [7.0, 4.0, 2.0]], [[2.0, 3.0, 3.0], [3.0, 6.0, 8.0], [7.0, 3.0, 1.0]], [[5.0, 8.0, 8.0], [8.0, 7.0, 8.0], 
[3.0, 2.0, 2.0]]], [[[5.0, 1.0, 2.0], [8.0, 1.0, 7.0], [8.0, 6.0, 4.0]], [[2.0, 5.0, 7.0], [8.0, 1.0, 5.0], [5.0, 5.0, 5.0]], 
[[5.0, 5.0, 8.0], [6.0, 8.0, 5.0], [7.0, 6.0, 7.0]]], [[[3.0, 5.0, 7.0], [2.0, 3.0, 5.0], [3.0, 8.0, 6.0]], [[5.0, 2.0, 8.0], 
[8.0, 1.0, 2.0], [5.0, 4.0, 8.0]], [[3.0, 5.0, 5.0], [8.0, 7.0, 5.0], [4.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 6.0], [5.0, 1.0, 1.0], [5.0, 1.0, 7.0]], [[8.0, 7.0, 6.0], [3.0, 3.0, 8.0], [2.0, 
4.0, 8.0]], [[4.0, 4.0, 3.0], [7.0, 6.0, 8.0], [8.0, 1.0, 7.0]]], [[[8.0, 6.0, 8.0], [3.0, 3.0, 2.0], [7.0, 1.0, 5.0]], [[6.0, 
7.0, 1.0], [2.0, 1.0, 5.0], [8.0, 2.0, 2.0]], [[5.0, 4.0, 5.0], [1.0, 3.0, 5.0], [6.0, 3.0, 7.0]]], [[[3.0, 1.0, 5.0], [3.0, 
7.0, 4.0], [2.0, 3.0, 1.0]], [[5.0, 2.0, 3.0], [4.0, 7.0, 5.0], [5.0, 3.0, 2.0]], [[3.0, 3.0, 8.0], [1.0, 2.0, 5.0], [7.0, 8.0, 
4.0]]], [[[2.0, 7.0, 3.0], [4.0, 2.0, 1.0], [8.0, 3.0, 7.0]], [[7.0, 8.0, 1.0], [8.0, 2.0, 5.0], [8.0, 7.0, 5.0]], [[6.0, 5.0, 
5.0], [1.0, 2.0, 8.0], [2.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 5.0], [3.0, 7.0, 3.0], [1.0, 7.0, 8.0]], 
[[8.0, 2.0, 7.0], [4.0, 7.0, 6.0], [6.0, 5.0, 8.0]], [[8.0, 2.0, 2.0], [4.0, 3.0, 1.0], [5.0, 8.0, 8.0]]], [[[3.0, 6.0, 7.0], 
[7.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[3.0, 2.0, 3.0], [3.0, 2.0, 8.0], [4.0, 4.0, 4.0]], [[3.0, 8.0, 1.0], [3.0, 8.0, 2.0], 
[2.0, 8.0, 7.0]]], [[[6.0, 4.0, 2.0], [3.0, 2.0, 4.0], [6.0, 7.0, 1.0]], [[4.0, 2.0, 8.0], [1.0, 5.0, 6.0], [3.0, 6.0, 7.0]], 
[[7.0, 8.0, 5.0], [1.0, 7.0, 5.0], [8.0, 5.0, 3.0]]], [[[2.0, 3.0, 2.0], [3.0, 2.0, 3.0], [8.0, 6.0, 8.0]], [[6.0, 1.0, 2.0], 
[6.0, 1.0, 1.0], [7.0, 5.0, 2.0]], [[4.0, 5.0, 7.0], [3.0, 8.0, 2.0], [2.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[[8.0], [2.0], [2.0]], [[4.0], [7.0], [7.0]], [[6.0], [6.0], [6.0]]], [[[4.0], [8.0], [6.0]], 
[[6.0], [7.0], [6.0]], [[3.0], [2.0], [4.0]]], [[[5.0], [7.0], [8.0]], [[3.0], [3.0], [8.0]], [[1.0], [5.0], [8.0]]], [[[3.0], 
[6.0], [8.0]], [[2.0], [3.0], [7.0]], [[6.0], [1.0], [5.0]]]]),self.functionspace)
      arg[:,:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [5.0, 1.0, 2.0], [5.0, 1.0, 2.0]], [[8.0, 7.0, 4.0], [3.0, 3.0, 7.0], [2.0, 
4.0, 7.0]], [[4.0, 4.0, 6.0], [7.0, 6.0, 6.0], [8.0, 1.0, 6.0]]], [[[8.0, 6.0, 4.0], [3.0, 3.0, 8.0], [7.0, 1.0, 6.0]], [[6.0, 
7.0, 6.0], [2.0, 1.0, 7.0], [8.0, 2.0, 6.0]], [[5.0, 4.0, 3.0], [1.0, 3.0, 2.0], [6.0, 3.0, 4.0]]], [[[3.0, 1.0, 5.0], [3.0, 
7.0, 7.0], [2.0, 3.0, 8.0]], [[5.0, 2.0, 3.0], [4.0, 7.0, 3.0], [5.0, 3.0, 8.0]], [[3.0, 3.0, 1.0], [1.0, 2.0, 5.0], [7.0, 8.0, 
8.0]]], [[[2.0, 7.0, 3.0], [4.0, 2.0, 6.0], [8.0, 3.0, 8.0]], [[7.0, 8.0, 2.0], [8.0, 2.0, 3.0], [8.0, 7.0, 7.0]], [[6.0, 5.0, 
6.0], [1.0, 2.0, 1.0], [2.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 8.0], [3.0, 7.0, 2.0], [1.0, 7.0, 2.0]], 
[[8.0, 2.0, 4.0], [4.0, 7.0, 7.0], [6.0, 5.0, 7.0]], [[8.0, 2.0, 6.0], [4.0, 3.0, 6.0], [5.0, 8.0, 6.0]]], [[[3.0, 6.0, 4.0], 
[7.0, 7.0, 8.0], [8.0, 5.0, 6.0]], [[3.0, 2.0, 6.0], [3.0, 2.0, 7.0], [4.0, 4.0, 6.0]], [[3.0, 8.0, 3.0], [3.0, 8.0, 2.0], 
[2.0, 8.0, 4.0]]], [[[6.0, 4.0, 5.0], [3.0, 2.0, 7.0], [6.0, 7.0, 8.0]], [[4.0, 2.0, 3.0], [1.0, 5.0, 3.0], [3.0, 6.0, 8.0]], 
[[7.0, 8.0, 1.0], [1.0, 7.0, 5.0], [8.0, 5.0, 8.0]]], [[[2.0, 3.0, 3.0], [3.0, 2.0, 6.0], [8.0, 6.0, 8.0]], [[6.0, 1.0, 2.0], 
[6.0, 1.0, 3.0], [7.0, 5.0, 7.0]], [[4.0, 5.0, 6.0], [3.0, 8.0, 1.0], [2.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCCL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 5.0], [2.0, 8.0, 6.0], [6.0, 5.0, 3.0]], [[6.0, 4.0, 4.0], [3.0, 3.0, 3.0], [1.0, 
3.0, 6.0]], [[3.0, 1.0, 5.0], [2.0, 3.0, 4.0], [1.0, 2.0, 2.0]]], [[[2.0, 7.0, 7.0], [7.0, 7.0, 6.0], [7.0, 1.0, 7.0]], [[2.0, 
5.0, 7.0], [5.0, 5.0, 5.0], [8.0, 3.0, 6.0]], [[4.0, 4.0, 1.0], [2.0, 7.0, 1.0], [7.0, 3.0, 4.0]]], [[[1.0, 4.0, 7.0], [7.0, 
4.0, 2.0], [3.0, 7.0, 5.0]], [[3.0, 3.0, 1.0], [8.0, 3.0, 1.0], [8.0, 5.0, 6.0]], [[8.0, 1.0, 8.0], [5.0, 3.0, 8.0], [6.0, 8.0, 
7.0]]], [[[5.0, 3.0, 4.0], [3.0, 2.0, 6.0], [6.0, 2.0, 2.0]], [[4.0, 2.0, 8.0], [4.0, 2.0, 1.0], [6.0, 8.0, 8.0]], [[6.0, 8.0, 
5.0], [1.0, 8.0, 3.0], [5.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 7.0], [1.0, 5.0, 8.0], [5.0, 6.0, 4.0]], 
[[4.0, 3.0, 4.0], [4.0, 8.0, 1.0], [7.0, 3.0, 6.0]], [[7.0, 6.0, 5.0], [2.0, 1.0, 5.0], [2.0, 7.0, 4.0]]], [[[7.0, 1.0, 8.0], 
[6.0, 6.0, 8.0], [2.0, 4.0, 5.0]], [[7.0, 2.0, 5.0], [8.0, 5.0, 1.0], [6.0, 2.0, 7.0]], [[2.0, 4.0, 2.0], [5.0, 2.0, 7.0], 
[6.0, 2.0, 4.0]]], [[[8.0, 4.0, 6.0], [7.0, 6.0, 1.0], [2.0, 4.0, 2.0]], [[2.0, 1.0, 6.0], [1.0, 1.0, 1.0], [2.0, 4.0, 8.0]], 
[[3.0, 8.0, 5.0], [2.0, 7.0, 7.0], [7.0, 2.0, 7.0]]], [[[1.0, 4.0, 8.0], [8.0, 3.0, 4.0], [6.0, 2.0, 7.0]], [[2.0, 3.0, 4.0], 
[4.0, 4.0, 5.0], [5.0, 7.0, 5.0]], [[3.0, 5.0, 2.0], [5.0, 5.0, 3.0], [3.0, 4.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0], [8.0], [8.0]], [[4.0], [6.0], [2.0]], [[7.0], [6.0], [3.0]]], [[[6.0], [1.0], [3.0]], 
[[3.0], [6.0], [8.0]], [[2.0], [1.0], [1.0]]], [[[8.0], [3.0], [7.0]], [[6.0], [2.0], [1.0]], [[6.0], [3.0], [3.0]]], [[[1.0], 
[7.0], [2.0]], [[1.0], [7.0], [3.0]], [[8.0], [6.0], [5.0]]]])+(1.-msk_val)*numpy.array([[[[4.0], [1.0], [2.0]], [[5.0], [6.0], 
[2.0]], [[6.0], [2.0], [4.0]]], [[[5.0], [8.0], [5.0]], [[7.0], [2.0], [5.0]], [[7.0], [7.0], [4.0]]], [[[7.0], [5.0], [7.0]], 
[[6.0], [7.0], [7.0]], [[2.0], [6.0], [7.0]]], [[[2.0], [5.0], [7.0]], [[3.0], [3.0], [5.0]], [[3.0], [1.0], [2.0]]]])
      arg[:,:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 3.0], [2.0, 8.0, 8.0], [6.0, 5.0, 8.0]], [[6.0, 4.0, 4.0], [3.0, 3.0, 6.0], [1.0, 
3.0, 2.0]], [[3.0, 1.0, 7.0], [2.0, 3.0, 6.0], [1.0, 2.0, 3.0]]], [[[2.0, 7.0, 6.0], [7.0, 7.0, 1.0], [7.0, 1.0, 3.0]], [[2.0, 
5.0, 3.0], [5.0, 5.0, 6.0], [8.0, 3.0, 8.0]], [[4.0, 4.0, 2.0], [2.0, 7.0, 1.0], [7.0, 3.0, 1.0]]], [[[1.0, 4.0, 8.0], [7.0, 
4.0, 3.0], [3.0, 7.0, 7.0]], [[3.0, 3.0, 6.0], [8.0, 3.0, 2.0], [8.0, 5.0, 1.0]], [[8.0, 1.0, 6.0], [5.0, 3.0, 3.0], [6.0, 8.0, 
3.0]]], [[[5.0, 3.0, 1.0], [3.0, 2.0, 7.0], [6.0, 2.0, 2.0]], [[4.0, 2.0, 1.0], [4.0, 2.0, 7.0], [6.0, 8.0, 3.0]], [[6.0, 8.0, 
8.0], [1.0, 8.0, 6.0], [5.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [1.0, 5.0, 1.0], [5.0, 6.0, 2.0]], 
[[4.0, 3.0, 5.0], [4.0, 8.0, 6.0], [7.0, 3.0, 2.0]], [[7.0, 6.0, 6.0], [2.0, 1.0, 2.0], [2.0, 7.0, 4.0]]], [[[7.0, 1.0, 5.0], 
[6.0, 6.0, 8.0], [2.0, 4.0, 5.0]], [[7.0, 2.0, 7.0], [8.0, 5.0, 2.0], [6.0, 2.0, 5.0]], [[2.0, 4.0, 7.0], [5.0, 2.0, 7.0], 
[6.0, 2.0, 4.0]]], [[[8.0, 4.0, 7.0], [7.0, 6.0, 5.0], [2.0, 4.0, 7.0]], [[2.0, 1.0, 6.0], [1.0, 1.0, 7.0], [2.0, 4.0, 7.0]], 
[[3.0, 8.0, 2.0], [2.0, 7.0, 6.0], [7.0, 2.0, 7.0]]], [[[1.0, 4.0, 2.0], [8.0, 3.0, 5.0], [6.0, 2.0, 7.0]], [[2.0, 3.0, 3.0], 
[4.0, 4.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 1.0], [3.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 4.0], [6.0, 3.0, 4.0], [8.0, 3.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 1.0, 4.0], [3.0, 
4.0, 3.0]], [[2.0, 6.0, 8.0], [4.0, 7.0, 3.0], [7.0, 4.0, 3.0]]], [[[2.0, 7.0, 4.0], [2.0, 3.0, 4.0], [8.0, 6.0, 8.0]], [[8.0, 
6.0, 6.0], [2.0, 6.0, 1.0], [2.0, 5.0, 3.0]], [[4.0, 2.0, 4.0], [6.0, 6.0, 1.0], [6.0, 3.0, 3.0]]], [[[3.0, 2.0, 7.0], [5.0, 
6.0, 5.0], [7.0, 5.0, 4.0]], [[8.0, 5.0, 4.0], [2.0, 7.0, 1.0], [5.0, 2.0, 2.0]], [[6.0, 3.0, 7.0], [8.0, 2.0, 5.0], [8.0, 5.0, 
7.0]]], [[[1.0, 6.0, 5.0], [3.0, 2.0, 3.0], [1.0, 3.0, 2.0]], [[1.0, 8.0, 8.0], [7.0, 4.0, 2.0], [3.0, 7.0, 8.0]], [[8.0, 6.0, 
4.0], [1.0, 2.0, 2.0], [2.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 1.0], [2.0, 5.0, 2.0], [8.0, 2.0, 1.0]], 
[[7.0, 3.0, 4.0], [7.0, 7.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 1.0, 7.0], [4.0, 5.0, 8.0], [2.0, 4.0, 8.0]]], [[[7.0, 2.0, 4.0], 
[5.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[2.0, 6.0, 4.0], [5.0, 5.0, 7.0], [3.0, 4.0, 7.0]], [[6.0, 4.0, 2.0], [4.0, 5.0, 7.0], 
[2.0, 5.0, 4.0]]], [[[4.0, 3.0, 7.0], [7.0, 3.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 8.0, 3.0], [3.0, 3.0, 8.0], [3.0, 5.0, 5.0]], 
[[1.0, 4.0, 7.0], [2.0, 7.0, 4.0], [6.0, 8.0, 5.0]]], [[[1.0, 5.0, 4.0], [4.0, 4.0, 7.0], [6.0, 2.0, 3.0]], [[2.0, 7.0, 7.0], 
[8.0, 4.0, 4.0], [5.0, 5.0, 2.0]], [[6.0, 6.0, 1.0], [2.0, 8.0, 2.0], [6.0, 6.0, 8.0]]]])
      val=numpy.array([[[6.0, 2.0, 6.0]], [[4.0, 3.0, 4.0]]])
      arg[1:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [6.0, 3.0, 4.0], [8.0, 3.0, 4.0]], [[1.0, 8.0, 2.0], [4.0, 1.0, 4.0], [3.0, 
4.0, 3.0]], [[2.0, 6.0, 8.0], [4.0, 7.0, 3.0], [7.0, 4.0, 3.0]]], [[[2.0, 7.0, 4.0], [2.0, 3.0, 4.0], [8.0, 6.0, 8.0]], [[8.0, 
6.0, 6.0], [6.0, 2.0, 6.0], [2.0, 5.0, 3.0]], [[4.0, 2.0, 4.0], [6.0, 6.0, 1.0], [6.0, 3.0, 3.0]]], [[[3.0, 2.0, 7.0], [5.0, 
6.0, 5.0], [7.0, 5.0, 4.0]], [[8.0, 5.0, 4.0], [4.0, 3.0, 4.0], [5.0, 2.0, 2.0]], [[6.0, 3.0, 7.0], [8.0, 2.0, 5.0], [8.0, 5.0, 
7.0]]], [[[1.0, 6.0, 5.0], [3.0, 2.0, 3.0], [1.0, 3.0, 2.0]], [[1.0, 8.0, 8.0], [7.0, 4.0, 2.0], [3.0, 7.0, 8.0]], [[8.0, 6.0, 
4.0], [1.0, 2.0, 2.0], [2.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 1.0], [2.0, 5.0, 2.0], [8.0, 2.0, 1.0]], 
[[7.0, 3.0, 4.0], [7.0, 7.0, 6.0], [7.0, 6.0, 3.0]], [[4.0, 1.0, 7.0], [4.0, 5.0, 8.0], [2.0, 4.0, 8.0]]], [[[7.0, 2.0, 4.0], 
[5.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[2.0, 6.0, 4.0], [6.0, 2.0, 6.0], [3.0, 4.0, 7.0]], [[6.0, 4.0, 2.0], [4.0, 5.0, 7.0], 
[2.0, 5.0, 4.0]]], [[[4.0, 3.0, 7.0], [7.0, 3.0, 3.0], [4.0, 3.0, 2.0]], [[6.0, 8.0, 3.0], [4.0, 3.0, 4.0], [3.0, 5.0, 5.0]], 
[[1.0, 4.0, 7.0], [2.0, 7.0, 4.0], [6.0, 8.0, 5.0]]], [[[1.0, 5.0, 4.0], [4.0, 4.0, 7.0], [6.0, 2.0, 3.0]], [[2.0, 7.0, 7.0], 
[8.0, 4.0, 4.0], [5.0, 5.0, 2.0]], [[6.0, 6.0, 1.0], [2.0, 8.0, 2.0], [6.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 6.0], [5.0, 8.0, 6.0], [2.0, 1.0, 2.0]], [[2.0, 4.0, 7.0], [2.0, 1.0, 5.0], [6.0, 
7.0, 2.0]], [[4.0, 4.0, 6.0], [5.0, 6.0, 8.0], [4.0, 7.0, 4.0]]], [[[1.0, 5.0, 3.0], [2.0, 4.0, 1.0], [7.0, 8.0, 5.0]], [[6.0, 
3.0, 7.0], [1.0, 7.0, 5.0], [5.0, 8.0, 3.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 1.0], [3.0, 3.0, 8.0]]], [[[4.0, 7.0, 3.0], [8.0, 
8.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 3.0, 7.0], [6.0, 2.0, 4.0], [7.0, 5.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 2.0, 8.0], [8.0, 6.0, 
2.0]]], [[[4.0, 2.0, 7.0], [3.0, 7.0, 1.0], [3.0, 1.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 1.0, 7.0], [4.0, 3.0, 1.0]], [[6.0, 2.0, 
5.0], [4.0, 8.0, 7.0], [8.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 5.0], [5.0, 8.0, 8.0], [8.0, 4.0, 7.0]], 
[[4.0, 5.0, 3.0], [5.0, 2.0, 6.0], [7.0, 1.0, 1.0]], [[5.0, 5.0, 2.0], [5.0, 5.0, 5.0], [6.0, 5.0, 2.0]]], [[[3.0, 4.0, 3.0], 
[2.0, 5.0, 4.0], [1.0, 8.0, 8.0]], [[8.0, 8.0, 2.0], [7.0, 7.0, 1.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 1.0], [1.0, 2.0, 4.0], 
[1.0, 1.0, 8.0]]], [[[7.0, 7.0, 2.0], [8.0, 6.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 3.0, 2.0], [2.0, 8.0, 5.0]], 
[[1.0, 5.0, 7.0], [3.0, 4.0, 3.0], [3.0, 5.0, 5.0]]], [[[2.0, 4.0, 4.0], [1.0, 6.0, 7.0], [2.0, 7.0, 7.0]], [[3.0, 1.0, 1.0], 
[4.0, 6.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 6.0, 8.0], [3.0, 4.0, 2.0], [4.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[6.0, 2.0, 7.0]], [[4.0, 1.0, 1.0]]]),self.functionspace)
      arg[1:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 6.0], [5.0, 8.0, 6.0], [2.0, 1.0, 2.0]], [[2.0, 4.0, 7.0], [2.0, 1.0, 5.0], [6.0, 
7.0, 2.0]], [[4.0, 4.0, 6.0], [5.0, 6.0, 8.0], [4.0, 7.0, 4.0]]], [[[1.0, 5.0, 3.0], [2.0, 4.0, 1.0], [7.0, 8.0, 5.0]], [[6.0, 
3.0, 7.0], [6.0, 2.0, 7.0], [5.0, 8.0, 3.0]], [[4.0, 5.0, 7.0], [2.0, 6.0, 1.0], [3.0, 3.0, 8.0]]], [[[4.0, 7.0, 3.0], [8.0, 
8.0, 7.0], [7.0, 7.0, 2.0]], [[3.0, 3.0, 7.0], [4.0, 1.0, 1.0], [7.0, 5.0, 4.0]], [[7.0, 8.0, 5.0], [6.0, 2.0, 8.0], [8.0, 6.0, 
2.0]]], [[[4.0, 2.0, 7.0], [3.0, 7.0, 1.0], [3.0, 1.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 1.0, 7.0], [4.0, 3.0, 1.0]], [[6.0, 2.0, 
5.0], [4.0, 8.0, 7.0], [8.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 5.0], [5.0, 8.0, 8.0], [8.0, 4.0, 7.0]], 
[[4.0, 5.0, 3.0], [5.0, 2.0, 6.0], [7.0, 1.0, 1.0]], [[5.0, 5.0, 2.0], [5.0, 5.0, 5.0], [6.0, 5.0, 2.0]]], [[[3.0, 4.0, 3.0], 
[2.0, 5.0, 4.0], [1.0, 8.0, 8.0]], [[8.0, 8.0, 2.0], [6.0, 2.0, 7.0], [5.0, 7.0, 6.0]], [[8.0, 1.0, 1.0], [1.0, 2.0, 4.0], 
[1.0, 1.0, 8.0]]], [[[7.0, 7.0, 2.0], [8.0, 6.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 8.0, 8.0], [4.0, 1.0, 1.0], [2.0, 8.0, 5.0]], 
[[1.0, 5.0, 7.0], [3.0, 4.0, 3.0], [3.0, 5.0, 5.0]]], [[[2.0, 4.0, 4.0], [1.0, 6.0, 7.0], [2.0, 7.0, 7.0]], [[3.0, 1.0, 1.0], 
[4.0, 6.0, 7.0], [7.0, 2.0, 1.0]], [[8.0, 6.0, 8.0], [3.0, 4.0, 2.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 5.0], [8.0, 4.0, 3.0], [2.0, 3.0, 3.0]], [[3.0, 6.0, 1.0], [3.0, 5.0, 7.0], [6.0, 
2.0, 1.0]], [[6.0, 2.0, 3.0], [4.0, 8.0, 1.0], [3.0, 6.0, 3.0]]], [[[3.0, 1.0, 5.0], [7.0, 8.0, 2.0], [4.0, 7.0, 4.0]], [[5.0, 
7.0, 8.0], [8.0, 5.0, 5.0], [8.0, 3.0, 1.0]], [[7.0, 1.0, 6.0], [1.0, 5.0, 5.0], [1.0, 1.0, 5.0]]], [[[1.0, 7.0, 3.0], [7.0, 
5.0, 4.0], [2.0, 7.0, 8.0]], [[4.0, 2.0, 3.0], [5.0, 6.0, 2.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 1.0], [5.0, 2.0, 1.0], [2.0, 2.0, 
4.0]]], [[[1.0, 7.0, 1.0], [6.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[5.0, 6.0, 1.0], [6.0, 2.0, 3.0], [3.0, 3.0, 6.0]], [[8.0, 1.0, 
7.0], [1.0, 5.0, 1.0], [7.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 1.0], [4.0, 8.0, 8.0], [7.0, 6.0, 2.0]], 
[[1.0, 4.0, 5.0], [4.0, 2.0, 7.0], [1.0, 3.0, 7.0]], [[5.0, 6.0, 5.0], [3.0, 8.0, 1.0], [2.0, 7.0, 6.0]]], [[[2.0, 4.0, 8.0], 
[8.0, 5.0, 5.0], [7.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [1.0, 5.0, 1.0], [1.0, 5.0, 7.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 4.0], 
[5.0, 7.0, 3.0]]], [[[3.0, 3.0, 7.0], [6.0, 4.0, 1.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 8.0], [2.0, 3.0, 6.0], [5.0, 1.0, 6.0]], 
[[3.0, 4.0, 3.0], [6.0, 5.0, 7.0], [1.0, 8.0, 8.0]]], [[[2.0, 7.0, 3.0], [7.0, 7.0, 2.0], [5.0, 7.0, 8.0]], [[2.0, 6.0, 4.0], 
[3.0, 3.0, 5.0], [6.0, 8.0, 6.0]], [[7.0, 3.0, 7.0], [8.0, 4.0, 8.0], [4.0, 6.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 4.0, 4.0]], [[5.0, 2.0, 5.0]]])+(1.-msk_val)*numpy.array([[[5.0, 1.0, 6.0]], [[1.0, 3.0, 
5.0]]])
      arg[1:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 5.0], [8.0, 4.0, 3.0], [2.0, 3.0, 3.0]], [[3.0, 6.0, 1.0], [3.0, 5.0, 7.0], [6.0, 
2.0, 1.0]], [[6.0, 2.0, 3.0], [4.0, 8.0, 1.0], [3.0, 6.0, 3.0]]], [[[3.0, 1.0, 5.0], [7.0, 8.0, 2.0], [4.0, 7.0, 4.0]], [[5.0, 
7.0, 8.0], [8.0, 4.0, 4.0], [8.0, 3.0, 1.0]], [[7.0, 1.0, 6.0], [1.0, 5.0, 5.0], [1.0, 1.0, 5.0]]], [[[1.0, 7.0, 3.0], [7.0, 
5.0, 4.0], [2.0, 7.0, 8.0]], [[4.0, 2.0, 3.0], [5.0, 2.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 1.0, 1.0], [5.0, 2.0, 1.0], [2.0, 2.0, 
4.0]]], [[[1.0, 7.0, 1.0], [6.0, 3.0, 6.0], [3.0, 7.0, 6.0]], [[5.0, 6.0, 1.0], [6.0, 2.0, 3.0], [3.0, 3.0, 6.0]], [[8.0, 1.0, 
7.0], [1.0, 5.0, 1.0], [7.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 1.0], [4.0, 8.0, 8.0], [7.0, 6.0, 2.0]], 
[[1.0, 4.0, 5.0], [4.0, 2.0, 7.0], [1.0, 3.0, 7.0]], [[5.0, 6.0, 5.0], [3.0, 8.0, 1.0], [2.0, 7.0, 6.0]]], [[[2.0, 4.0, 8.0], 
[8.0, 5.0, 5.0], [7.0, 8.0, 7.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 6.0], [1.0, 5.0, 7.0]], [[6.0, 4.0, 1.0], [3.0, 4.0, 4.0], 
[5.0, 7.0, 3.0]]], [[[3.0, 3.0, 7.0], [6.0, 4.0, 1.0], [4.0, 8.0, 2.0]], [[2.0, 8.0, 8.0], [1.0, 3.0, 5.0], [5.0, 1.0, 6.0]], 
[[3.0, 4.0, 3.0], [6.0, 5.0, 7.0], [1.0, 8.0, 8.0]]], [[[2.0, 7.0, 3.0], [7.0, 7.0, 2.0], [5.0, 7.0, 8.0]], [[2.0, 6.0, 4.0], 
[3.0, 3.0, 5.0], [6.0, 8.0, 6.0]], [[7.0, 3.0, 7.0], [8.0, 4.0, 8.0], [4.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 4.0], [4.0, 5.0, 4.0], [8.0, 4.0, 6.0]], [[8.0, 8.0, 4.0], [5.0, 1.0, 6.0], [1.0, 
5.0, 3.0]], [[1.0, 1.0, 6.0], [2.0, 8.0, 8.0], [4.0, 8.0, 8.0]]], [[[6.0, 7.0, 3.0], [2.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[1.0, 
2.0, 4.0], [6.0, 1.0, 5.0], [7.0, 4.0, 8.0]], [[5.0, 1.0, 4.0], [2.0, 3.0, 3.0], [3.0, 6.0, 3.0]]], [[[1.0, 7.0, 3.0], [3.0, 
2.0, 5.0], [4.0, 3.0, 2.0]], [[1.0, 3.0, 4.0], [7.0, 8.0, 6.0], [8.0, 7.0, 3.0]], [[5.0, 7.0, 7.0], [2.0, 3.0, 2.0], [3.0, 7.0, 
2.0]]], [[[7.0, 5.0, 2.0], [2.0, 4.0, 8.0], [3.0, 4.0, 4.0]], [[2.0, 4.0, 7.0], [2.0, 1.0, 4.0], [8.0, 3.0, 8.0]], [[4.0, 5.0, 
2.0], [5.0, 6.0, 8.0], [4.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 8.0], [1.0, 7.0, 4.0], [1.0, 1.0, 1.0]], 
[[2.0, 6.0, 5.0], [4.0, 6.0, 4.0], [7.0, 4.0, 3.0]], [[2.0, 6.0, 7.0], [2.0, 3.0, 5.0], [5.0, 8.0, 5.0]]], [[[8.0, 1.0, 5.0], 
[8.0, 4.0, 7.0], [1.0, 8.0, 3.0]], [[6.0, 1.0, 5.0], [8.0, 3.0, 4.0], [7.0, 8.0, 7.0]], [[6.0, 1.0, 7.0], [3.0, 8.0, 6.0], 
[6.0, 3.0, 3.0]]], [[[7.0, 6.0, 4.0], [1.0, 1.0, 8.0], [3.0, 6.0, 3.0]], [[5.0, 6.0, 1.0], [8.0, 8.0, 7.0], [8.0, 3.0, 6.0]], 
[[2.0, 5.0, 8.0], [5.0, 7.0, 3.0], [7.0, 6.0, 4.0]]], [[[8.0, 4.0, 4.0], [2.0, 3.0, 3.0], [8.0, 8.0, 6.0]], [[5.0, 1.0, 5.0], 
[8.0, 5.0, 3.0], [7.0, 4.0, 8.0]], [[3.0, 7.0, 1.0], [6.0, 1.0, 5.0], [1.0, 1.0, 2.0]]]])
      val=numpy.array([[[[5.0, 6.0, 4.0]]], [[[5.0, 2.0, 6.0]]], [[[6.0, 2.0, 8.0]]], [[[4.0, 4.0, 2.0]]]])
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 4.0], [4.0, 5.0, 4.0], [8.0, 4.0, 6.0]], [[8.0, 8.0, 4.0], [5.0, 1.0, 6.0], [1.0, 
5.0, 3.0]], [[1.0, 1.0, 6.0], [5.0, 6.0, 4.0], [4.0, 8.0, 8.0]]], [[[6.0, 7.0, 3.0], [2.0, 3.0, 1.0], [5.0, 1.0, 4.0]], [[1.0, 
2.0, 4.0], [6.0, 1.0, 5.0], [7.0, 4.0, 8.0]], [[5.0, 1.0, 4.0], [5.0, 2.0, 6.0], [3.0, 6.0, 3.0]]], [[[1.0, 7.0, 3.0], [3.0, 
2.0, 5.0], [4.0, 3.0, 2.0]], [[1.0, 3.0, 4.0], [7.0, 8.0, 6.0], [8.0, 7.0, 3.0]], [[5.0, 7.0, 7.0], [6.0, 2.0, 8.0], [3.0, 7.0, 
2.0]]], [[[7.0, 5.0, 2.0], [2.0, 4.0, 8.0], [3.0, 4.0, 4.0]], [[2.0, 4.0, 7.0], [2.0, 1.0, 4.0], [8.0, 3.0, 8.0]], [[4.0, 5.0, 
2.0], [4.0, 4.0, 2.0], [4.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [1.0, 7.0, 4.0], [1.0, 1.0, 1.0]], 
[[2.0, 6.0, 5.0], [4.0, 6.0, 4.0], [7.0, 4.0, 3.0]], [[2.0, 6.0, 7.0], [5.0, 6.0, 4.0], [5.0, 8.0, 5.0]]], [[[8.0, 1.0, 5.0], 
[8.0, 4.0, 7.0], [1.0, 8.0, 3.0]], [[6.0, 1.0, 5.0], [8.0, 3.0, 4.0], [7.0, 8.0, 7.0]], [[6.0, 1.0, 7.0], [5.0, 2.0, 6.0], 
[6.0, 3.0, 3.0]]], [[[7.0, 6.0, 4.0], [1.0, 1.0, 8.0], [3.0, 6.0, 3.0]], [[5.0, 6.0, 1.0], [8.0, 8.0, 7.0], [8.0, 3.0, 6.0]], 
[[2.0, 5.0, 8.0], [6.0, 2.0, 8.0], [7.0, 6.0, 4.0]]], [[[8.0, 4.0, 4.0], [2.0, 3.0, 3.0], [8.0, 8.0, 6.0]], [[5.0, 1.0, 5.0], 
[8.0, 5.0, 3.0], [7.0, 4.0, 8.0]], [[3.0, 7.0, 1.0], [4.0, 4.0, 2.0], [1.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 8.0], [6.0, 6.0, 4.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 3.0], [4.0, 3.0, 5.0], [8.0, 
2.0, 1.0]], [[4.0, 8.0, 6.0], [5.0, 4.0, 2.0], [5.0, 1.0, 2.0]]], [[[6.0, 7.0, 8.0], [3.0, 4.0, 5.0], [4.0, 2.0, 1.0]], [[3.0, 
7.0, 5.0], [8.0, 7.0, 7.0], [2.0, 3.0, 3.0]], [[1.0, 4.0, 6.0], [7.0, 5.0, 6.0], [1.0, 3.0, 4.0]]], [[[7.0, 7.0, 5.0], [2.0, 
6.0, 2.0], [1.0, 7.0, 6.0]], [[7.0, 1.0, 8.0], [6.0, 5.0, 3.0], [1.0, 5.0, 5.0]], [[1.0, 7.0, 2.0], [5.0, 8.0, 3.0], [1.0, 2.0, 
3.0]]], [[[6.0, 8.0, 7.0], [7.0, 2.0, 7.0], [1.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 7.0], [7.0, 8.0, 8.0]], [[3.0, 8.0, 
6.0], [7.0, 1.0, 2.0], [3.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 3.0], [4.0, 3.0, 1.0], [3.0, 1.0, 7.0]], 
[[6.0, 2.0, 5.0], [7.0, 8.0, 2.0], [5.0, 2.0, 1.0]], [[1.0, 7.0, 8.0], [8.0, 2.0, 1.0], [1.0, 5.0, 6.0]]], [[[4.0, 5.0, 2.0], 
[2.0, 8.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 4.0, 7.0], [5.0, 1.0, 5.0], [2.0, 4.0, 8.0]], [[6.0, 6.0, 3.0], [8.0, 7.0, 7.0], 
[2.0, 1.0, 4.0]]], [[[4.0, 3.0, 8.0], [6.0, 6.0, 1.0], [4.0, 4.0, 6.0]], [[8.0, 8.0, 2.0], [4.0, 5.0, 1.0], [8.0, 8.0, 3.0]], 
[[2.0, 6.0, 2.0], [5.0, 1.0, 8.0], [2.0, 4.0, 4.0]]], [[[6.0, 6.0, 7.0], [2.0, 2.0, 6.0], [6.0, 3.0, 7.0]], [[2.0, 2.0, 8.0], 
[3.0, 7.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 3.0, 6.0], [7.0, 3.0, 3.0], [8.0, 2.0, 8.0]]]])
      val=Data(numpy.array([[[[1.0, 3.0, 6.0]]], [[[3.0, 8.0, 8.0]]], [[[2.0, 3.0, 1.0]]], [[[1.0, 6.0, 
4.0]]]]),self.functionspace)
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 8.0], [6.0, 6.0, 4.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 3.0], [4.0, 3.0, 5.0], [8.0, 
2.0, 1.0]], [[4.0, 8.0, 6.0], [1.0, 3.0, 6.0], [5.0, 1.0, 2.0]]], [[[6.0, 7.0, 8.0], [3.0, 4.0, 5.0], [4.0, 2.0, 1.0]], [[3.0, 
7.0, 5.0], [8.0, 7.0, 7.0], [2.0, 3.0, 3.0]], [[1.0, 4.0, 6.0], [3.0, 8.0, 8.0], [1.0, 3.0, 4.0]]], [[[7.0, 7.0, 5.0], [2.0, 
6.0, 2.0], [1.0, 7.0, 6.0]], [[7.0, 1.0, 8.0], [6.0, 5.0, 3.0], [1.0, 5.0, 5.0]], [[1.0, 7.0, 2.0], [2.0, 3.0, 1.0], [1.0, 2.0, 
3.0]]], [[[6.0, 8.0, 7.0], [7.0, 2.0, 7.0], [1.0, 1.0, 2.0]], [[7.0, 5.0, 7.0], [1.0, 8.0, 7.0], [7.0, 8.0, 8.0]], [[3.0, 8.0, 
6.0], [1.0, 6.0, 4.0], [3.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [4.0, 3.0, 1.0], [3.0, 1.0, 7.0]], 
[[6.0, 2.0, 5.0], [7.0, 8.0, 2.0], [5.0, 2.0, 1.0]], [[1.0, 7.0, 8.0], [1.0, 3.0, 6.0], [1.0, 5.0, 6.0]]], [[[4.0, 5.0, 2.0], 
[2.0, 8.0, 1.0], [3.0, 6.0, 7.0]], [[7.0, 4.0, 7.0], [5.0, 1.0, 5.0], [2.0, 4.0, 8.0]], [[6.0, 6.0, 3.0], [3.0, 8.0, 8.0], 
[2.0, 1.0, 4.0]]], [[[4.0, 3.0, 8.0], [6.0, 6.0, 1.0], [4.0, 4.0, 6.0]], [[8.0, 8.0, 2.0], [4.0, 5.0, 1.0], [8.0, 8.0, 3.0]], 
[[2.0, 6.0, 2.0], [2.0, 3.0, 1.0], [2.0, 4.0, 4.0]]], [[[6.0, 6.0, 7.0], [2.0, 2.0, 6.0], [6.0, 3.0, 7.0]], [[2.0, 2.0, 8.0], 
[3.0, 7.0, 6.0], [4.0, 7.0, 6.0]], [[3.0, 3.0, 6.0], [1.0, 6.0, 4.0], [8.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 5.0], [6.0, 4.0, 2.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 2.0], [8.0, 8.0, 5.0], [3.0, 
1.0, 8.0]], [[4.0, 2.0, 6.0], [3.0, 7.0, 8.0], [2.0, 7.0, 6.0]]], [[[8.0, 6.0, 3.0], [7.0, 7.0, 6.0], [8.0, 5.0, 7.0]], [[4.0, 
2.0, 7.0], [3.0, 6.0, 7.0], [4.0, 3.0, 3.0]], [[3.0, 2.0, 8.0], [3.0, 3.0, 5.0], [1.0, 3.0, 5.0]]], [[[7.0, 1.0, 2.0], [4.0, 
3.0, 6.0], [2.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], [1.0, 4.0, 2.0], [6.0, 7.0, 8.0]], [[2.0, 4.0, 2.0], [2.0, 6.0, 2.0], [5.0, 2.0, 
7.0]]], [[[7.0, 4.0, 6.0], [7.0, 5.0, 3.0], [1.0, 1.0, 2.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 8.0], [4.0, 8.0, 6.0]], [[1.0, 4.0, 
4.0], [3.0, 6.0, 4.0], [2.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 8.0], [5.0, 7.0, 4.0], [4.0, 5.0, 6.0]], 
[[3.0, 5.0, 3.0], [4.0, 8.0, 8.0], [2.0, 3.0, 1.0]], [[8.0, 1.0, 3.0], [7.0, 6.0, 3.0], [2.0, 2.0, 7.0]]], [[[6.0, 4.0, 8.0], 
[6.0, 4.0, 6.0], [7.0, 3.0, 6.0]], [[8.0, 5.0, 7.0], [3.0, 7.0, 2.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 5.0], [1.0, 4.0, 1.0], 
[5.0, 7.0, 3.0]]], [[[5.0, 6.0, 6.0], [7.0, 2.0, 2.0], [3.0, 2.0, 7.0]], [[7.0, 1.0, 4.0], [7.0, 7.0, 8.0], [2.0, 4.0, 7.0]], 
[[7.0, 6.0, 1.0], [8.0, 7.0, 8.0], [1.0, 2.0, 7.0]]], [[[4.0, 5.0, 3.0], [7.0, 6.0, 2.0], [7.0, 2.0, 2.0]], [[2.0, 4.0, 5.0], 
[2.0, 8.0, 1.0], [4.0, 3.0, 3.0]], [[7.0, 6.0, 1.0], [3.0, 7.0, 8.0], [4.0, 7.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 5.0, 1.0]]], [[[5.0, 2.0, 4.0]]], [[[1.0, 7.0, 3.0]]], [[[1.0, 2.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 2.0, 2.0]]], [[[5.0, 8.0, 2.0]]], [[[8.0, 6.0, 2.0]]], [[[1.0, 3.0, 8.0]]]])
      arg[:,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 5.0], [6.0, 4.0, 2.0], [1.0, 5.0, 7.0]], [[5.0, 7.0, 2.0], [8.0, 8.0, 5.0], [3.0, 
1.0, 8.0]], [[4.0, 2.0, 6.0], [8.0, 5.0, 1.0], [2.0, 7.0, 6.0]]], [[[8.0, 6.0, 3.0], [7.0, 7.0, 6.0], [8.0, 5.0, 7.0]], [[4.0, 
2.0, 7.0], [3.0, 6.0, 7.0], [4.0, 3.0, 3.0]], [[3.0, 2.0, 8.0], [5.0, 2.0, 4.0], [1.0, 3.0, 5.0]]], [[[7.0, 1.0, 2.0], [4.0, 
3.0, 6.0], [2.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], [1.0, 4.0, 2.0], [6.0, 7.0, 8.0]], [[2.0, 4.0, 2.0], [1.0, 7.0, 3.0], [5.0, 2.0, 
7.0]]], [[[7.0, 4.0, 6.0], [7.0, 5.0, 3.0], [1.0, 1.0, 2.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 8.0], [4.0, 8.0, 6.0]], [[1.0, 4.0, 
4.0], [1.0, 2.0, 5.0], [2.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 8.0], [5.0, 7.0, 4.0], [4.0, 5.0, 6.0]], 
[[3.0, 5.0, 3.0], [4.0, 8.0, 8.0], [2.0, 3.0, 1.0]], [[8.0, 1.0, 3.0], [8.0, 2.0, 2.0], [2.0, 2.0, 7.0]]], [[[6.0, 4.0, 8.0], 
[6.0, 4.0, 6.0], [7.0, 3.0, 6.0]], [[8.0, 5.0, 7.0], [3.0, 7.0, 2.0], [6.0, 4.0, 6.0]], [[1.0, 6.0, 5.0], [5.0, 8.0, 2.0], 
[5.0, 7.0, 3.0]]], [[[5.0, 6.0, 6.0], [7.0, 2.0, 2.0], [3.0, 2.0, 7.0]], [[7.0, 1.0, 4.0], [7.0, 7.0, 8.0], [2.0, 4.0, 7.0]], 
[[7.0, 6.0, 1.0], [8.0, 6.0, 2.0], [1.0, 2.0, 7.0]]], [[[4.0, 5.0, 3.0], [7.0, 6.0, 2.0], [7.0, 2.0, 2.0]], [[2.0, 4.0, 5.0], 
[2.0, 8.0, 1.0], [4.0, 3.0, 3.0]], [[7.0, 6.0, 1.0], [1.0, 3.0, 8.0], [4.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 6.0, 7.0], [7.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[5.0, 7.0, 1.0], [1.0, 4.0, 2.0], [7.0, 
1.0, 1.0]], [[5.0, 8.0, 1.0], [4.0, 1.0, 8.0], [3.0, 5.0, 7.0]]], [[[3.0, 1.0, 5.0], [7.0, 5.0, 3.0], [5.0, 7.0, 7.0]], [[2.0, 
1.0, 6.0], [4.0, 7.0, 6.0], [8.0, 5.0, 1.0]], [[4.0, 7.0, 1.0], [2.0, 4.0, 6.0], [2.0, 2.0, 6.0]]], [[[6.0, 5.0, 2.0], [6.0, 
3.0, 7.0], [3.0, 1.0, 2.0]], [[5.0, 6.0, 5.0], [4.0, 8.0, 7.0], [1.0, 5.0, 4.0]], [[8.0, 8.0, 7.0], [5.0, 7.0, 7.0], [7.0, 7.0, 
4.0]]], [[[3.0, 6.0, 3.0], [2.0, 8.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 8.0, 1.0], [7.0, 3.0, 4.0], [4.0, 3.0, 3.0]], [[2.0, 2.0, 
4.0], [2.0, 1.0, 8.0], [3.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 1.0], [1.0, 7.0, 5.0], [1.0, 6.0, 2.0]], 
[[6.0, 3.0, 3.0], [4.0, 7.0, 4.0], [7.0, 4.0, 6.0]], [[3.0, 6.0, 2.0], [5.0, 1.0, 3.0], [5.0, 8.0, 7.0]]], [[[2.0, 6.0, 5.0], 
[6.0, 7.0, 7.0], [7.0, 5.0, 1.0]], [[6.0, 1.0, 4.0], [8.0, 1.0, 1.0], [4.0, 7.0, 5.0]], [[8.0, 4.0, 3.0], [8.0, 6.0, 4.0], 
[8.0, 7.0, 1.0]]], [[[7.0, 7.0, 3.0], [8.0, 2.0, 4.0], [2.0, 6.0, 4.0]], [[2.0, 4.0, 2.0], [5.0, 8.0, 7.0], [6.0, 7.0, 3.0]], 
[[7.0, 3.0, 4.0], [4.0, 6.0, 6.0], [2.0, 8.0, 6.0]]], [[[2.0, 6.0, 5.0], [2.0, 7.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 4.0, 7.0], 
[8.0, 4.0, 6.0], [4.0, 8.0, 8.0]], [[7.0, 8.0, 3.0], [5.0, 5.0, 2.0], [2.0, 1.0, 8.0]]]])
      val=numpy.array([[[8.0, 2.0, 7.0]], [[3.0, 5.0, 1.0]]])
      arg[2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 7.0], [7.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[5.0, 7.0, 1.0], [1.0, 4.0, 2.0], [7.0, 
1.0, 1.0]], [[5.0, 8.0, 1.0], [4.0, 1.0, 8.0], [3.0, 5.0, 7.0]]], [[[3.0, 1.0, 5.0], [7.0, 5.0, 3.0], [5.0, 7.0, 7.0]], [[2.0, 
1.0, 6.0], [4.0, 7.0, 6.0], [8.0, 5.0, 1.0]], [[4.0, 7.0, 1.0], [2.0, 4.0, 6.0], [2.0, 2.0, 6.0]]], [[[6.0, 5.0, 2.0], [8.0, 
2.0, 7.0], [3.0, 1.0, 2.0]], [[5.0, 6.0, 5.0], [3.0, 5.0, 1.0], [1.0, 5.0, 4.0]], [[8.0, 8.0, 7.0], [5.0, 7.0, 7.0], [7.0, 7.0, 
4.0]]], [[[3.0, 6.0, 3.0], [2.0, 8.0, 6.0], [8.0, 1.0, 8.0]], [[7.0, 8.0, 1.0], [7.0, 3.0, 4.0], [4.0, 3.0, 3.0]], [[2.0, 2.0, 
4.0], [2.0, 1.0, 8.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 1.0], [1.0, 7.0, 5.0], [1.0, 6.0, 2.0]], 
[[6.0, 3.0, 3.0], [4.0, 7.0, 4.0], [7.0, 4.0, 6.0]], [[3.0, 6.0, 2.0], [5.0, 1.0, 3.0], [5.0, 8.0, 7.0]]], [[[2.0, 6.0, 5.0], 
[6.0, 7.0, 7.0], [7.0, 5.0, 1.0]], [[6.0, 1.0, 4.0], [8.0, 1.0, 1.0], [4.0, 7.0, 5.0]], [[8.0, 4.0, 3.0], [8.0, 6.0, 4.0], 
[8.0, 7.0, 1.0]]], [[[7.0, 7.0, 3.0], [8.0, 2.0, 7.0], [2.0, 6.0, 4.0]], [[2.0, 4.0, 2.0], [3.0, 5.0, 1.0], [6.0, 7.0, 3.0]], 
[[7.0, 3.0, 4.0], [4.0, 6.0, 6.0], [2.0, 8.0, 6.0]]], [[[2.0, 6.0, 5.0], [2.0, 7.0, 6.0], [7.0, 2.0, 4.0]], [[4.0, 4.0, 7.0], 
[8.0, 4.0, 6.0], [4.0, 8.0, 8.0]], [[7.0, 8.0, 3.0], [5.0, 5.0, 2.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 2.0], [7.0, 5.0, 6.0], [4.0, 8.0, 2.0]], [[7.0, 4.0, 2.0], [3.0, 6.0, 7.0], [6.0, 
4.0, 3.0]], [[6.0, 6.0, 5.0], [7.0, 5.0, 7.0], [4.0, 6.0, 7.0]]], [[[7.0, 6.0, 8.0], [5.0, 8.0, 5.0], [1.0, 1.0, 1.0]], [[2.0, 
2.0, 2.0], [6.0, 2.0, 5.0], [6.0, 1.0, 3.0]], [[5.0, 6.0, 1.0], [1.0, 1.0, 6.0], [8.0, 8.0, 1.0]]], [[[4.0, 5.0, 7.0], [4.0, 
1.0, 8.0], [1.0, 6.0, 5.0]], [[4.0, 2.0, 8.0], [4.0, 7.0, 8.0], [5.0, 3.0, 7.0]], [[1.0, 3.0, 1.0], [6.0, 2.0, 7.0], [5.0, 5.0, 
3.0]]], [[[3.0, 7.0, 4.0], [7.0, 8.0, 2.0], [1.0, 1.0, 3.0]], [[8.0, 6.0, 2.0], [8.0, 8.0, 3.0], [6.0, 5.0, 3.0]], [[8.0, 4.0, 
2.0], [3.0, 2.0, 6.0], [7.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 5.0, 3.0], [4.0, 7.0, 6.0]], 
[[6.0, 2.0, 7.0], [2.0, 5.0, 1.0], [1.0, 6.0, 3.0]], [[8.0, 5.0, 2.0], [3.0, 5.0, 3.0], [6.0, 8.0, 7.0]]], [[[4.0, 2.0, 7.0], 
[5.0, 2.0, 7.0], [2.0, 3.0, 8.0]], [[8.0, 1.0, 5.0], [4.0, 4.0, 5.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 6.0], [3.0, 4.0, 4.0], 
[6.0, 8.0, 2.0]]], [[[7.0, 1.0, 6.0], [7.0, 2.0, 4.0], [6.0, 7.0, 6.0]], [[7.0, 5.0, 5.0], [1.0, 5.0, 3.0], [3.0, 3.0, 1.0]], 
[[6.0, 8.0, 3.0], [3.0, 8.0, 8.0], [4.0, 8.0, 1.0]]], [[[4.0, 7.0, 4.0], [1.0, 7.0, 8.0], [3.0, 8.0, 1.0]], [[8.0, 8.0, 6.0], 
[4.0, 3.0, 7.0], [3.0, 3.0, 8.0]], [[2.0, 8.0, 4.0], [1.0, 3.0, 7.0], [7.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[2.0, 4.0, 1.0]], [[2.0, 2.0, 8.0]]]),self.functionspace)
      arg[2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [7.0, 5.0, 6.0], [4.0, 8.0, 2.0]], [[7.0, 4.0, 2.0], [3.0, 6.0, 7.0], [6.0, 
4.0, 3.0]], [[6.0, 6.0, 5.0], [7.0, 5.0, 7.0], [4.0, 6.0, 7.0]]], [[[7.0, 6.0, 8.0], [5.0, 8.0, 5.0], [1.0, 1.0, 1.0]], [[2.0, 
2.0, 2.0], [6.0, 2.0, 5.0], [6.0, 1.0, 3.0]], [[5.0, 6.0, 1.0], [1.0, 1.0, 6.0], [8.0, 8.0, 1.0]]], [[[4.0, 5.0, 7.0], [2.0, 
4.0, 1.0], [1.0, 6.0, 5.0]], [[4.0, 2.0, 8.0], [2.0, 2.0, 8.0], [5.0, 3.0, 7.0]], [[1.0, 3.0, 1.0], [6.0, 2.0, 7.0], [5.0, 5.0, 
3.0]]], [[[3.0, 7.0, 4.0], [7.0, 8.0, 2.0], [1.0, 1.0, 3.0]], [[8.0, 6.0, 2.0], [8.0, 8.0, 3.0], [6.0, 5.0, 3.0]], [[8.0, 4.0, 
2.0], [3.0, 2.0, 6.0], [7.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 8.0], [6.0, 5.0, 3.0], [4.0, 7.0, 6.0]], 
[[6.0, 2.0, 7.0], [2.0, 5.0, 1.0], [1.0, 6.0, 3.0]], [[8.0, 5.0, 2.0], [3.0, 5.0, 3.0], [6.0, 8.0, 7.0]]], [[[4.0, 2.0, 7.0], 
[5.0, 2.0, 7.0], [2.0, 3.0, 8.0]], [[8.0, 1.0, 5.0], [4.0, 4.0, 5.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 6.0], [3.0, 4.0, 4.0], 
[6.0, 8.0, 2.0]]], [[[7.0, 1.0, 6.0], [2.0, 4.0, 1.0], [6.0, 7.0, 6.0]], [[7.0, 5.0, 5.0], [2.0, 2.0, 8.0], [3.0, 3.0, 1.0]], 
[[6.0, 8.0, 3.0], [3.0, 8.0, 8.0], [4.0, 8.0, 1.0]]], [[[4.0, 7.0, 4.0], [1.0, 7.0, 8.0], [3.0, 8.0, 1.0]], [[8.0, 8.0, 6.0], 
[4.0, 3.0, 7.0], [3.0, 3.0, 8.0]], [[2.0, 8.0, 4.0], [1.0, 3.0, 7.0], [7.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 6.0], [2.0, 3.0, 3.0], [7.0, 3.0, 6.0]], [[7.0, 8.0, 1.0], [2.0, 6.0, 2.0], [4.0, 
1.0, 1.0]], [[4.0, 2.0, 8.0], [4.0, 4.0, 1.0], [7.0, 7.0, 8.0]]], [[[5.0, 4.0, 2.0], [3.0, 4.0, 3.0], [4.0, 5.0, 5.0]], [[1.0, 
7.0, 5.0], [1.0, 7.0, 5.0], [3.0, 3.0, 6.0]], [[2.0, 4.0, 4.0], [6.0, 5.0, 1.0], [6.0, 3.0, 6.0]]], [[[5.0, 4.0, 3.0], [4.0, 
4.0, 5.0], [8.0, 5.0, 6.0]], [[8.0, 8.0, 6.0], [5.0, 3.0, 1.0], [3.0, 3.0, 1.0]], [[3.0, 4.0, 5.0], [4.0, 4.0, 2.0], [4.0, 8.0, 
7.0]]], [[[6.0, 6.0, 4.0], [7.0, 1.0, 3.0], [3.0, 1.0, 1.0]], [[8.0, 5.0, 5.0], [2.0, 6.0, 7.0], [7.0, 3.0, 6.0]], [[6.0, 1.0, 
6.0], [4.0, 6.0, 6.0], [1.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 7.0], [5.0, 5.0, 5.0], [1.0, 4.0, 4.0]], 
[[5.0, 8.0, 1.0], [2.0, 5.0, 8.0], [3.0, 2.0, 5.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 1.0], [8.0, 3.0, 2.0]]], [[[2.0, 6.0, 5.0], 
[2.0, 5.0, 3.0], [4.0, 8.0, 3.0]], [[2.0, 1.0, 4.0], [6.0, 4.0, 4.0], [5.0, 7.0, 4.0]], [[7.0, 5.0, 7.0], [6.0, 8.0, 4.0], 
[5.0, 4.0, 3.0]]], [[[4.0, 5.0, 4.0], [6.0, 6.0, 2.0], [5.0, 4.0, 7.0]], [[6.0, 6.0, 7.0], [4.0, 8.0, 3.0], [2.0, 7.0, 1.0]], 
[[1.0, 7.0, 7.0], [7.0, 8.0, 5.0], [8.0, 6.0, 6.0]]], [[[7.0, 8.0, 3.0], [7.0, 6.0, 8.0], [5.0, 7.0, 5.0]], [[1.0, 3.0, 2.0], 
[2.0, 8.0, 2.0], [1.0, 1.0, 3.0]], [[2.0, 7.0, 5.0], [1.0, 6.0, 5.0], [1.0, 5.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 1.0, 7.0]], [[1.0, 1.0, 3.0]]])+(1.-msk_val)*numpy.array([[[5.0, 3.0, 7.0]], [[2.0, 6.0, 
5.0]]])
      arg[2,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 6.0], [2.0, 3.0, 3.0], [7.0, 3.0, 6.0]], [[7.0, 8.0, 1.0], [2.0, 6.0, 2.0], [4.0, 
1.0, 1.0]], [[4.0, 2.0, 8.0], [4.0, 4.0, 1.0], [7.0, 7.0, 8.0]]], [[[5.0, 4.0, 2.0], [3.0, 4.0, 3.0], [4.0, 5.0, 5.0]], [[1.0, 
7.0, 5.0], [1.0, 7.0, 5.0], [3.0, 3.0, 6.0]], [[2.0, 4.0, 4.0], [6.0, 5.0, 1.0], [6.0, 3.0, 6.0]]], [[[5.0, 4.0, 3.0], [4.0, 
1.0, 7.0], [8.0, 5.0, 6.0]], [[8.0, 8.0, 6.0], [1.0, 1.0, 3.0], [3.0, 3.0, 1.0]], [[3.0, 4.0, 5.0], [4.0, 4.0, 2.0], [4.0, 8.0, 
7.0]]], [[[6.0, 6.0, 4.0], [7.0, 1.0, 3.0], [3.0, 1.0, 1.0]], [[8.0, 5.0, 5.0], [2.0, 6.0, 7.0], [7.0, 3.0, 6.0]], [[6.0, 1.0, 
6.0], [4.0, 6.0, 6.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 7.0], [5.0, 5.0, 5.0], [1.0, 4.0, 4.0]], 
[[5.0, 8.0, 1.0], [2.0, 5.0, 8.0], [3.0, 2.0, 5.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 1.0], [8.0, 3.0, 2.0]]], [[[2.0, 6.0, 5.0], 
[2.0, 5.0, 3.0], [4.0, 8.0, 3.0]], [[2.0, 1.0, 4.0], [6.0, 4.0, 4.0], [5.0, 7.0, 4.0]], [[7.0, 5.0, 7.0], [6.0, 8.0, 4.0], 
[5.0, 4.0, 3.0]]], [[[4.0, 5.0, 4.0], [5.0, 3.0, 7.0], [5.0, 4.0, 7.0]], [[6.0, 6.0, 7.0], [2.0, 6.0, 5.0], [2.0, 7.0, 1.0]], 
[[1.0, 7.0, 7.0], [7.0, 8.0, 5.0], [8.0, 6.0, 6.0]]], [[[7.0, 8.0, 3.0], [7.0, 6.0, 8.0], [5.0, 7.0, 5.0]], [[1.0, 3.0, 2.0], 
[2.0, 8.0, 2.0], [1.0, 1.0, 3.0]], [[2.0, 7.0, 5.0], [1.0, 6.0, 5.0], [1.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 2.0], [7.0, 2.0, 3.0], [7.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [2.0, 8.0, 1.0], [2.0, 
7.0, 5.0]], [[6.0, 2.0, 6.0], [4.0, 8.0, 1.0], [8.0, 2.0, 2.0]]], [[[6.0, 7.0, 7.0], [2.0, 8.0, 5.0], [4.0, 4.0, 7.0]], [[5.0, 
3.0, 3.0], [2.0, 6.0, 8.0], [2.0, 5.0, 6.0]], [[3.0, 5.0, 5.0], [2.0, 7.0, 2.0], [4.0, 4.0, 6.0]]], [[[7.0, 6.0, 8.0], [3.0, 
2.0, 5.0], [6.0, 6.0, 4.0]], [[8.0, 2.0, 7.0], [8.0, 6.0, 4.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 2.0], [3.0, 4.0, 6.0], [7.0, 7.0, 
3.0]]], [[[5.0, 1.0, 4.0], [1.0, 4.0, 6.0], [7.0, 2.0, 5.0]], [[3.0, 5.0, 6.0], [1.0, 1.0, 1.0], [7.0, 7.0, 1.0]], [[1.0, 7.0, 
8.0], [3.0, 5.0, 1.0], [4.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 8.0], [3.0, 5.0, 2.0], [4.0, 5.0, 5.0]], 
[[3.0, 3.0, 2.0], [3.0, 1.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 4.0, 4.0], [1.0, 6.0, 5.0], [5.0, 1.0, 7.0]]], [[[2.0, 4.0, 8.0], 
[1.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 4.0], [3.0, 2.0, 6.0]], [[7.0, 1.0, 5.0], [6.0, 8.0, 4.0], 
[7.0, 6.0, 4.0]]], [[[1.0, 5.0, 7.0], [4.0, 3.0, 4.0], [2.0, 3.0, 3.0]], [[2.0, 3.0, 4.0], [7.0, 8.0, 1.0], [7.0, 1.0, 6.0]], 
[[8.0, 5.0, 7.0], [8.0, 7.0, 1.0], [6.0, 2.0, 4.0]]], [[[2.0, 4.0, 6.0], [8.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[5.0, 1.0, 6.0], 
[6.0, 4.0, 1.0], [2.0, 3.0, 8.0]], [[3.0, 3.0, 1.0], [5.0, 5.0, 5.0], [1.0, 4.0, 5.0]]]])
      val=numpy.array([[5.0, 2.0, 3.0], [2.0, 3.0, 8.0]])
      arg[2:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 2.0], [7.0, 2.0, 3.0], [7.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [2.0, 8.0, 1.0], [2.0, 
7.0, 5.0]], [[6.0, 2.0, 6.0], [4.0, 8.0, 1.0], [8.0, 2.0, 2.0]]], [[[6.0, 7.0, 7.0], [2.0, 8.0, 5.0], [4.0, 4.0, 7.0]], [[5.0, 
3.0, 3.0], [2.0, 6.0, 8.0], [2.0, 5.0, 6.0]], [[3.0, 5.0, 5.0], [2.0, 7.0, 2.0], [4.0, 4.0, 6.0]]], [[[7.0, 6.0, 8.0], [3.0, 
2.0, 5.0], [6.0, 6.0, 4.0]], [[8.0, 2.0, 7.0], [5.0, 2.0, 3.0], [4.0, 2.0, 3.0]], [[4.0, 5.0, 2.0], [3.0, 4.0, 6.0], [7.0, 7.0, 
3.0]]], [[[5.0, 1.0, 4.0], [1.0, 4.0, 6.0], [7.0, 2.0, 5.0]], [[3.0, 5.0, 6.0], [2.0, 3.0, 8.0], [7.0, 7.0, 1.0]], [[1.0, 7.0, 
8.0], [3.0, 5.0, 1.0], [4.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 8.0], [3.0, 5.0, 2.0], [4.0, 5.0, 5.0]], 
[[3.0, 3.0, 2.0], [3.0, 1.0, 5.0], [2.0, 6.0, 6.0]], [[5.0, 4.0, 4.0], [1.0, 6.0, 5.0], [5.0, 1.0, 7.0]]], [[[2.0, 4.0, 8.0], 
[1.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[7.0, 7.0, 1.0], [8.0, 2.0, 4.0], [3.0, 2.0, 6.0]], [[7.0, 1.0, 5.0], [6.0, 8.0, 4.0], 
[7.0, 6.0, 4.0]]], [[[1.0, 5.0, 7.0], [4.0, 3.0, 4.0], [2.0, 3.0, 3.0]], [[2.0, 3.0, 4.0], [5.0, 2.0, 3.0], [7.0, 1.0, 6.0]], 
[[8.0, 5.0, 7.0], [8.0, 7.0, 1.0], [6.0, 2.0, 4.0]]], [[[2.0, 4.0, 6.0], [8.0, 6.0, 1.0], [6.0, 3.0, 3.0]], [[5.0, 1.0, 6.0], 
[2.0, 3.0, 8.0], [2.0, 3.0, 8.0]], [[3.0, 3.0, 1.0], [5.0, 5.0, 5.0], [1.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 4.0], [8.0, 4.0, 3.0], [6.0, 2.0, 7.0]], [[7.0, 3.0, 1.0], [5.0, 8.0, 2.0], [4.0, 
8.0, 8.0]], [[8.0, 8.0, 2.0], [6.0, 6.0, 2.0], [1.0, 2.0, 4.0]]], [[[1.0, 4.0, 2.0], [8.0, 8.0, 1.0], [5.0, 8.0, 2.0]], [[1.0, 
3.0, 8.0], [3.0, 7.0, 1.0], [8.0, 8.0, 3.0]], [[4.0, 5.0, 7.0], [2.0, 2.0, 6.0], [1.0, 3.0, 2.0]]], [[[4.0, 7.0, 5.0], [5.0, 
4.0, 1.0], [2.0, 2.0, 7.0]], [[4.0, 7.0, 5.0], [4.0, 2.0, 8.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 5.0], [8.0, 2.0, 4.0], [6.0, 2.0, 
5.0]]], [[[2.0, 5.0, 7.0], [3.0, 2.0, 5.0], [2.0, 5.0, 2.0]], [[5.0, 6.0, 6.0], [2.0, 3.0, 4.0], [7.0, 8.0, 2.0]], [[7.0, 5.0, 
2.0], [1.0, 4.0, 5.0], [7.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 5.0], [7.0, 2.0, 7.0], [1.0, 2.0, 7.0]], 
[[3.0, 3.0, 7.0], [7.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[3.0, 6.0, 7.0], [1.0, 2.0, 6.0], [3.0, 3.0, 8.0]]], [[[7.0, 3.0, 3.0], 
[4.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[7.0, 2.0, 3.0], [5.0, 8.0, 8.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 5.0], [4.0, 4.0, 5.0], 
[7.0, 7.0, 7.0]]], [[[1.0, 3.0, 5.0], [8.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[3.0, 8.0, 6.0], [5.0, 6.0, 5.0], [7.0, 3.0, 2.0]], 
[[7.0, 2.0, 4.0], [1.0, 8.0, 6.0], [7.0, 2.0, 7.0]]], [[[7.0, 3.0, 4.0], [6.0, 5.0, 2.0], [5.0, 2.0, 5.0]], [[4.0, 8.0, 8.0], 
[3.0, 5.0, 8.0], [4.0, 6.0, 8.0]], [[8.0, 4.0, 3.0], [6.0, 5.0, 6.0], [4.0, 7.0, 2.0]]]])
      val=Data(numpy.array([[3.0, 8.0, 2.0], [1.0, 5.0, 2.0]]),self.functionspace)
      arg[2:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 4.0], [8.0, 4.0, 3.0], [6.0, 2.0, 7.0]], [[7.0, 3.0, 1.0], [5.0, 8.0, 2.0], [4.0, 
8.0, 8.0]], [[8.0, 8.0, 2.0], [6.0, 6.0, 2.0], [1.0, 2.0, 4.0]]], [[[1.0, 4.0, 2.0], [8.0, 8.0, 1.0], [5.0, 8.0, 2.0]], [[1.0, 
3.0, 8.0], [3.0, 7.0, 1.0], [8.0, 8.0, 3.0]], [[4.0, 5.0, 7.0], [2.0, 2.0, 6.0], [1.0, 3.0, 2.0]]], [[[4.0, 7.0, 5.0], [5.0, 
4.0, 1.0], [2.0, 2.0, 7.0]], [[4.0, 7.0, 5.0], [3.0, 8.0, 2.0], [4.0, 8.0, 8.0]], [[2.0, 7.0, 5.0], [8.0, 2.0, 4.0], [6.0, 2.0, 
5.0]]], [[[2.0, 5.0, 7.0], [3.0, 2.0, 5.0], [2.0, 5.0, 2.0]], [[5.0, 6.0, 6.0], [1.0, 5.0, 2.0], [7.0, 8.0, 2.0]], [[7.0, 5.0, 
2.0], [1.0, 4.0, 5.0], [7.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 5.0], [7.0, 2.0, 7.0], [1.0, 2.0, 7.0]], 
[[3.0, 3.0, 7.0], [7.0, 8.0, 2.0], [6.0, 6.0, 3.0]], [[3.0, 6.0, 7.0], [1.0, 2.0, 6.0], [3.0, 3.0, 8.0]]], [[[7.0, 3.0, 3.0], 
[4.0, 8.0, 1.0], [7.0, 3.0, 4.0]], [[7.0, 2.0, 3.0], [5.0, 8.0, 8.0], [8.0, 3.0, 2.0]], [[5.0, 7.0, 5.0], [4.0, 4.0, 5.0], 
[7.0, 7.0, 7.0]]], [[[1.0, 3.0, 5.0], [8.0, 3.0, 7.0], [7.0, 8.0, 6.0]], [[3.0, 8.0, 6.0], [3.0, 8.0, 2.0], [7.0, 3.0, 2.0]], 
[[7.0, 2.0, 4.0], [1.0, 8.0, 6.0], [7.0, 2.0, 7.0]]], [[[7.0, 3.0, 4.0], [6.0, 5.0, 2.0], [5.0, 2.0, 5.0]], [[4.0, 8.0, 8.0], 
[1.0, 5.0, 2.0], [4.0, 6.0, 8.0]], [[8.0, 4.0, 3.0], [6.0, 5.0, 6.0], [4.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 3.0], [6.0, 7.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 7.0, 2.0], [6.0, 3.0, 7.0], [1.0, 
3.0, 1.0]], [[7.0, 3.0, 3.0], [1.0, 3.0, 5.0], [5.0, 7.0, 3.0]]], [[[5.0, 7.0, 3.0], [2.0, 7.0, 3.0], [8.0, 1.0, 6.0]], [[4.0, 
6.0, 8.0], [6.0, 6.0, 4.0], [6.0, 1.0, 7.0]], [[7.0, 2.0, 2.0], [8.0, 3.0, 3.0], [1.0, 5.0, 4.0]]], [[[8.0, 6.0, 2.0], [5.0, 
3.0, 5.0], [2.0, 2.0, 8.0]], [[5.0, 3.0, 3.0], [3.0, 1.0, 7.0], [3.0, 8.0, 3.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 3.0], [8.0, 5.0, 
1.0]]], [[[3.0, 3.0, 5.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[5.0, 1.0, 4.0], [4.0, 5.0, 4.0], [1.0, 2.0, 8.0]], [[7.0, 5.0, 
1.0], [4.0, 8.0, 3.0], [6.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 2.0], [5.0, 8.0, 1.0], [3.0, 7.0, 5.0]], 
[[2.0, 5.0, 3.0], [1.0, 6.0, 6.0], [4.0, 1.0, 2.0]], [[6.0, 5.0, 6.0], [6.0, 3.0, 8.0], [8.0, 8.0, 6.0]]], [[[6.0, 4.0, 7.0], 
[2.0, 2.0, 3.0], [5.0, 3.0, 4.0]], [[6.0, 7.0, 8.0], [4.0, 7.0, 2.0], [6.0, 3.0, 2.0]], [[1.0, 3.0, 5.0], [4.0, 1.0, 7.0], 
[3.0, 8.0, 1.0]]], [[[2.0, 1.0, 2.0], [6.0, 3.0, 8.0], [7.0, 7.0, 2.0]], [[1.0, 2.0, 1.0], [7.0, 1.0, 2.0], [3.0, 2.0, 7.0]], 
[[2.0, 3.0, 1.0], [4.0, 8.0, 1.0], [3.0, 8.0, 3.0]]], [[[2.0, 6.0, 7.0], [4.0, 1.0, 5.0], [3.0, 5.0, 1.0]], [[4.0, 1.0, 8.0], 
[8.0, 6.0, 8.0], [6.0, 4.0, 3.0]], [[2.0, 4.0, 1.0], [2.0, 2.0, 5.0], [1.0, 7.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[4.0, 2.0, 3.0], [6.0, 7.0, 7.0]])+(1.-msk_val)*numpy.array([[8.0, 8.0, 4.0], [1.0, 1.0, 3.0]])
      arg[2:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 3.0], [6.0, 7.0, 8.0], [5.0, 6.0, 1.0]], [[1.0, 7.0, 2.0], [6.0, 3.0, 7.0], [1.0, 
3.0, 1.0]], [[7.0, 3.0, 3.0], [1.0, 3.0, 5.0], [5.0, 7.0, 3.0]]], [[[5.0, 7.0, 3.0], [2.0, 7.0, 3.0], [8.0, 1.0, 6.0]], [[4.0, 
6.0, 8.0], [6.0, 6.0, 4.0], [6.0, 1.0, 7.0]], [[7.0, 2.0, 2.0], [8.0, 3.0, 3.0], [1.0, 5.0, 4.0]]], [[[8.0, 6.0, 2.0], [5.0, 
3.0, 5.0], [2.0, 2.0, 8.0]], [[5.0, 3.0, 3.0], [4.0, 2.0, 3.0], [3.0, 8.0, 3.0]], [[4.0, 2.0, 6.0], [8.0, 7.0, 3.0], [8.0, 5.0, 
1.0]]], [[[3.0, 3.0, 5.0], [3.0, 1.0, 7.0], [1.0, 4.0, 3.0]], [[5.0, 1.0, 4.0], [6.0, 7.0, 7.0], [1.0, 2.0, 8.0]], [[7.0, 5.0, 
1.0], [4.0, 8.0, 3.0], [6.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 2.0], [5.0, 8.0, 1.0], [3.0, 7.0, 5.0]], 
[[2.0, 5.0, 3.0], [1.0, 6.0, 6.0], [4.0, 1.0, 2.0]], [[6.0, 5.0, 6.0], [6.0, 3.0, 8.0], [8.0, 8.0, 6.0]]], [[[6.0, 4.0, 7.0], 
[2.0, 2.0, 3.0], [5.0, 3.0, 4.0]], [[6.0, 7.0, 8.0], [4.0, 7.0, 2.0], [6.0, 3.0, 2.0]], [[1.0, 3.0, 5.0], [4.0, 1.0, 7.0], 
[3.0, 8.0, 1.0]]], [[[2.0, 1.0, 2.0], [6.0, 3.0, 8.0], [7.0, 7.0, 2.0]], [[1.0, 2.0, 1.0], [8.0, 8.0, 4.0], [3.0, 2.0, 7.0]], 
[[2.0, 3.0, 1.0], [4.0, 8.0, 1.0], [3.0, 8.0, 3.0]]], [[[2.0, 6.0, 7.0], [4.0, 1.0, 5.0], [3.0, 5.0, 1.0]], [[4.0, 1.0, 8.0], 
[1.0, 1.0, 3.0], [6.0, 4.0, 3.0]], [[2.0, 4.0, 1.0], [2.0, 2.0, 5.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 8.0], [5.0, 6.0, 5.0], [3.0, 6.0, 8.0]], [[8.0, 1.0, 6.0], [5.0, 8.0, 7.0], [7.0, 
8.0, 4.0]], [[1.0, 7.0, 2.0], [3.0, 5.0, 2.0], [4.0, 1.0, 2.0]]], [[[6.0, 5.0, 5.0], [6.0, 3.0, 1.0], [6.0, 3.0, 5.0]], [[6.0, 
7.0, 8.0], [4.0, 8.0, 7.0], [6.0, 7.0, 8.0]], [[7.0, 6.0, 8.0], [7.0, 1.0, 6.0], [7.0, 7.0, 4.0]]], [[[3.0, 2.0, 3.0], [2.0, 
6.0, 1.0], [3.0, 8.0, 4.0]], [[8.0, 6.0, 2.0], [6.0, 4.0, 6.0], [4.0, 5.0, 4.0]], [[4.0, 3.0, 3.0], [8.0, 5.0, 8.0], [2.0, 6.0, 
5.0]]], [[[8.0, 1.0, 4.0], [8.0, 4.0, 8.0], [5.0, 5.0, 6.0]], [[1.0, 7.0, 5.0], [4.0, 3.0, 1.0], [8.0, 1.0, 7.0]], [[1.0, 7.0, 
2.0], [7.0, 8.0, 4.0], [8.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 3.0], [5.0, 4.0, 1.0], [4.0, 8.0, 1.0]], 
[[4.0, 7.0, 4.0], [4.0, 2.0, 5.0], [4.0, 7.0, 4.0]], [[7.0, 2.0, 3.0], [6.0, 2.0, 3.0], [8.0, 8.0, 1.0]]], [[[6.0, 5.0, 5.0], 
[7.0, 7.0, 4.0], [5.0, 7.0, 5.0]], [[1.0, 4.0, 3.0], [1.0, 5.0, 7.0], [6.0, 3.0, 6.0]], [[1.0, 1.0, 8.0], [7.0, 2.0, 8.0], 
[1.0, 1.0, 6.0]]], [[[8.0, 3.0, 7.0], [1.0, 6.0, 6.0], [5.0, 3.0, 4.0]], [[1.0, 5.0, 4.0], [3.0, 6.0, 2.0], [6.0, 4.0, 2.0]], 
[[6.0, 6.0, 2.0], [3.0, 1.0, 8.0], [6.0, 5.0, 1.0]]], [[[7.0, 8.0, 7.0], [7.0, 1.0, 7.0], [8.0, 4.0, 5.0]], [[8.0, 6.0, 2.0], 
[4.0, 7.0, 6.0], [7.0, 1.0, 6.0]], [[8.0, 2.0, 3.0], [3.0, 6.0, 6.0], [5.0, 7.0, 3.0]]]])
      val=numpy.array([[[[3.0, 6.0, 6.0]]], [[[8.0, 1.0, 3.0]]]])
      arg[1:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 8.0], [5.0, 6.0, 5.0], [3.0, 6.0, 8.0]], [[8.0, 1.0, 6.0], [5.0, 8.0, 7.0], [7.0, 
8.0, 4.0]], [[1.0, 7.0, 2.0], [3.0, 5.0, 2.0], [4.0, 1.0, 2.0]]], [[[6.0, 5.0, 5.0], [6.0, 3.0, 1.0], [6.0, 3.0, 5.0]], [[6.0, 
7.0, 8.0], [3.0, 6.0, 6.0], [6.0, 7.0, 8.0]], [[7.0, 6.0, 8.0], [7.0, 1.0, 6.0], [7.0, 7.0, 4.0]]], [[[3.0, 2.0, 3.0], [2.0, 
6.0, 1.0], [3.0, 8.0, 4.0]], [[8.0, 6.0, 2.0], [8.0, 1.0, 3.0], [4.0, 5.0, 4.0]], [[4.0, 3.0, 3.0], [8.0, 5.0, 8.0], [2.0, 6.0, 
5.0]]], [[[8.0, 1.0, 4.0], [8.0, 4.0, 8.0], [5.0, 5.0, 6.0]], [[1.0, 7.0, 5.0], [4.0, 3.0, 1.0], [8.0, 1.0, 7.0]], [[1.0, 7.0, 
2.0], [7.0, 8.0, 4.0], [8.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 3.0], [5.0, 4.0, 1.0], [4.0, 8.0, 1.0]], 
[[4.0, 7.0, 4.0], [4.0, 2.0, 5.0], [4.0, 7.0, 4.0]], [[7.0, 2.0, 3.0], [6.0, 2.0, 3.0], [8.0, 8.0, 1.0]]], [[[6.0, 5.0, 5.0], 
[7.0, 7.0, 4.0], [5.0, 7.0, 5.0]], [[1.0, 4.0, 3.0], [3.0, 6.0, 6.0], [6.0, 3.0, 6.0]], [[1.0, 1.0, 8.0], [7.0, 2.0, 8.0], 
[1.0, 1.0, 6.0]]], [[[8.0, 3.0, 7.0], [1.0, 6.0, 6.0], [5.0, 3.0, 4.0]], [[1.0, 5.0, 4.0], [8.0, 1.0, 3.0], [6.0, 4.0, 2.0]], 
[[6.0, 6.0, 2.0], [3.0, 1.0, 8.0], [6.0, 5.0, 1.0]]], [[[7.0, 8.0, 7.0], [7.0, 1.0, 7.0], [8.0, 4.0, 5.0]], [[8.0, 6.0, 2.0], 
[4.0, 7.0, 6.0], [7.0, 1.0, 6.0]], [[8.0, 2.0, 3.0], [3.0, 6.0, 6.0], [5.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 7.0], [6.0, 6.0, 4.0], [3.0, 2.0, 8.0]], [[6.0, 7.0, 7.0], [4.0, 4.0, 1.0], [6.0, 
4.0, 2.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 1.0], [6.0, 5.0, 6.0]]], [[[6.0, 3.0, 3.0], [1.0, 7.0, 2.0], [4.0, 8.0, 7.0]], [[7.0, 
5.0, 8.0], [1.0, 5.0, 6.0], [4.0, 7.0, 2.0]], [[5.0, 1.0, 5.0], [5.0, 5.0, 8.0], [8.0, 3.0, 2.0]]], [[[4.0, 1.0, 3.0], [8.0, 
4.0, 2.0], [8.0, 3.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 4.0, 3.0], [5.0, 4.0, 3.0]], [[5.0, 6.0, 1.0], [2.0, 4.0, 1.0], [1.0, 7.0, 
6.0]]], [[[6.0, 8.0, 8.0], [2.0, 6.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 3.0, 6.0], [6.0, 6.0, 5.0]], [[4.0, 8.0, 
4.0], [5.0, 1.0, 3.0], [6.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 7.0], [4.0, 8.0, 8.0], [8.0, 6.0, 5.0]], 
[[4.0, 6.0, 2.0], [1.0, 8.0, 6.0], [6.0, 6.0, 3.0]], [[2.0, 6.0, 1.0], [6.0, 8.0, 4.0], [6.0, 2.0, 6.0]]], [[[1.0, 5.0, 5.0], 
[1.0, 1.0, 2.0], [4.0, 8.0, 5.0]], [[3.0, 4.0, 4.0], [3.0, 6.0, 8.0], [8.0, 2.0, 3.0]], [[7.0, 3.0, 8.0], [1.0, 3.0, 4.0], 
[2.0, 2.0, 7.0]]], [[[6.0, 2.0, 8.0], [3.0, 8.0, 1.0], [5.0, 5.0, 5.0]], [[8.0, 5.0, 5.0], [5.0, 6.0, 8.0], [7.0, 6.0, 7.0]], 
[[3.0, 7.0, 5.0], [7.0, 6.0, 7.0], [2.0, 7.0, 5.0]]], [[[6.0, 7.0, 2.0], [5.0, 1.0, 8.0], [5.0, 8.0, 4.0]], [[4.0, 4.0, 8.0], 
[5.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[7.0, 4.0, 7.0], [4.0, 3.0, 3.0], [5.0, 3.0, 2.0]]]])
      val=Data(numpy.array([[[[7.0, 7.0, 4.0]]], [[[5.0, 6.0, 7.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 7.0], [6.0, 6.0, 4.0], [3.0, 2.0, 8.0]], [[6.0, 7.0, 7.0], [4.0, 4.0, 1.0], [6.0, 
4.0, 2.0]], [[1.0, 2.0, 5.0], [7.0, 3.0, 1.0], [6.0, 5.0, 6.0]]], [[[6.0, 3.0, 3.0], [1.0, 7.0, 2.0], [4.0, 8.0, 7.0]], [[7.0, 
5.0, 8.0], [7.0, 7.0, 4.0], [4.0, 7.0, 2.0]], [[5.0, 1.0, 5.0], [5.0, 5.0, 8.0], [8.0, 3.0, 2.0]]], [[[4.0, 1.0, 3.0], [8.0, 
4.0, 2.0], [8.0, 3.0, 5.0]], [[7.0, 3.0, 7.0], [5.0, 6.0, 7.0], [5.0, 4.0, 3.0]], [[5.0, 6.0, 1.0], [2.0, 4.0, 1.0], [1.0, 7.0, 
6.0]]], [[[6.0, 8.0, 8.0], [2.0, 6.0, 3.0], [1.0, 8.0, 1.0]], [[8.0, 5.0, 1.0], [7.0, 3.0, 6.0], [6.0, 6.0, 5.0]], [[4.0, 8.0, 
4.0], [5.0, 1.0, 3.0], [6.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 7.0], [4.0, 8.0, 8.0], [8.0, 6.0, 5.0]], 
[[4.0, 6.0, 2.0], [1.0, 8.0, 6.0], [6.0, 6.0, 3.0]], [[2.0, 6.0, 1.0], [6.0, 8.0, 4.0], [6.0, 2.0, 6.0]]], [[[1.0, 5.0, 5.0], 
[1.0, 1.0, 2.0], [4.0, 8.0, 5.0]], [[3.0, 4.0, 4.0], [7.0, 7.0, 4.0], [8.0, 2.0, 3.0]], [[7.0, 3.0, 8.0], [1.0, 3.0, 4.0], 
[2.0, 2.0, 7.0]]], [[[6.0, 2.0, 8.0], [3.0, 8.0, 1.0], [5.0, 5.0, 5.0]], [[8.0, 5.0, 5.0], [5.0, 6.0, 7.0], [7.0, 6.0, 7.0]], 
[[3.0, 7.0, 5.0], [7.0, 6.0, 7.0], [2.0, 7.0, 5.0]]], [[[6.0, 7.0, 2.0], [5.0, 1.0, 8.0], [5.0, 8.0, 4.0]], [[4.0, 4.0, 8.0], 
[5.0, 7.0, 6.0], [4.0, 1.0, 2.0]], [[7.0, 4.0, 7.0], [4.0, 3.0, 3.0], [5.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 7.0], [3.0, 4.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 7.0, 3.0], [2.0, 5.0, 8.0], [4.0, 
3.0, 4.0]], [[1.0, 1.0, 6.0], [5.0, 4.0, 5.0], [8.0, 2.0, 2.0]]], [[[3.0, 4.0, 4.0], [5.0, 1.0, 6.0], [6.0, 4.0, 4.0]], [[2.0, 
5.0, 6.0], [3.0, 2.0, 7.0], [4.0, 4.0, 6.0]], [[3.0, 3.0, 8.0], [5.0, 2.0, 4.0], [5.0, 2.0, 4.0]]], [[[6.0, 6.0, 7.0], [7.0, 
1.0, 3.0], [5.0, 4.0, 2.0]], [[2.0, 1.0, 5.0], [8.0, 5.0, 1.0], [8.0, 1.0, 4.0]], [[1.0, 3.0, 5.0], [3.0, 1.0, 4.0], [5.0, 1.0, 
8.0]]], [[[1.0, 6.0, 5.0], [7.0, 5.0, 4.0], [7.0, 5.0, 7.0]], [[5.0, 3.0, 1.0], [1.0, 6.0, 8.0], [7.0, 8.0, 1.0]], [[1.0, 1.0, 
3.0], [3.0, 7.0, 2.0], [6.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 4.0], [7.0, 4.0, 4.0], [4.0, 3.0, 8.0]], 
[[4.0, 8.0, 5.0], [6.0, 2.0, 8.0], [7.0, 7.0, 6.0]], [[2.0, 3.0, 7.0], [5.0, 6.0, 4.0], [3.0, 6.0, 8.0]]], [[[6.0, 4.0, 8.0], 
[5.0, 3.0, 5.0], [5.0, 2.0, 6.0]], [[7.0, 5.0, 1.0], [1.0, 8.0, 4.0], [7.0, 5.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 2.0, 1.0], 
[8.0, 1.0, 4.0]]], [[[1.0, 5.0, 2.0], [8.0, 2.0, 1.0], [2.0, 2.0, 1.0]], [[4.0, 6.0, 5.0], [5.0, 2.0, 4.0], [5.0, 4.0, 1.0]], 
[[1.0, 1.0, 5.0], [5.0, 6.0, 2.0], [6.0, 5.0, 7.0]]], [[[4.0, 4.0, 2.0], [6.0, 8.0, 4.0], [3.0, 1.0, 2.0]], [[8.0, 2.0, 7.0], 
[7.0, 7.0, 6.0], [6.0, 2.0, 3.0]], [[3.0, 4.0, 6.0], [6.0, 6.0, 2.0], [2.0, 3.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 6.0, 5.0]]], [[[3.0, 8.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 8.0, 8.0]]], 
[[[8.0, 3.0, 2.0]]]])
      arg[1:3,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 7.0], [3.0, 4.0, 8.0], [5.0, 4.0, 7.0]], [[6.0, 7.0, 3.0], [2.0, 5.0, 8.0], [4.0, 
3.0, 4.0]], [[1.0, 1.0, 6.0], [5.0, 4.0, 5.0], [8.0, 2.0, 2.0]]], [[[3.0, 4.0, 4.0], [5.0, 1.0, 6.0], [6.0, 4.0, 4.0]], [[2.0, 
5.0, 6.0], [3.0, 6.0, 5.0], [4.0, 4.0, 6.0]], [[3.0, 3.0, 8.0], [5.0, 2.0, 4.0], [5.0, 2.0, 4.0]]], [[[6.0, 6.0, 7.0], [7.0, 
1.0, 3.0], [5.0, 4.0, 2.0]], [[2.0, 1.0, 5.0], [3.0, 8.0, 6.0], [8.0, 1.0, 4.0]], [[1.0, 3.0, 5.0], [3.0, 1.0, 4.0], [5.0, 1.0, 
8.0]]], [[[1.0, 6.0, 5.0], [7.0, 5.0, 4.0], [7.0, 5.0, 7.0]], [[5.0, 3.0, 1.0], [1.0, 6.0, 8.0], [7.0, 8.0, 1.0]], [[1.0, 1.0, 
3.0], [3.0, 7.0, 2.0], [6.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 4.0], [7.0, 4.0, 4.0], [4.0, 3.0, 8.0]], 
[[4.0, 8.0, 5.0], [6.0, 2.0, 8.0], [7.0, 7.0, 6.0]], [[2.0, 3.0, 7.0], [5.0, 6.0, 4.0], [3.0, 6.0, 8.0]]], [[[6.0, 4.0, 8.0], 
[5.0, 3.0, 5.0], [5.0, 2.0, 6.0]], [[7.0, 5.0, 1.0], [3.0, 8.0, 8.0], [7.0, 5.0, 6.0]], [[8.0, 2.0, 2.0], [1.0, 2.0, 1.0], 
[8.0, 1.0, 4.0]]], [[[1.0, 5.0, 2.0], [8.0, 2.0, 1.0], [2.0, 2.0, 1.0]], [[4.0, 6.0, 5.0], [8.0, 3.0, 2.0], [5.0, 4.0, 1.0]], 
[[1.0, 1.0, 5.0], [5.0, 6.0, 2.0], [6.0, 5.0, 7.0]]], [[[4.0, 4.0, 2.0], [6.0, 8.0, 4.0], [3.0, 1.0, 2.0]], [[8.0, 2.0, 7.0], 
[7.0, 7.0, 6.0], [6.0, 2.0, 3.0]], [[3.0, 4.0, 6.0], [6.0, 6.0, 2.0], [2.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 3.0], [3.0, 5.0, 7.0], [1.0, 1.0, 8.0]], [[8.0, 4.0, 5.0], [4.0, 2.0, 3.0], [5.0, 
2.0, 5.0]], [[3.0, 3.0, 7.0], [2.0, 5.0, 3.0], [1.0, 2.0, 5.0]]], [[[7.0, 2.0, 5.0], [1.0, 5.0, 8.0], [3.0, 8.0, 4.0]], [[6.0, 
6.0, 5.0], [1.0, 6.0, 6.0], [1.0, 4.0, 3.0]], [[7.0, 1.0, 3.0], [1.0, 6.0, 2.0], [4.0, 1.0, 4.0]]], [[[1.0, 5.0, 4.0], [2.0, 
8.0, 3.0], [3.0, 7.0, 7.0]], [[8.0, 3.0, 3.0], [7.0, 2.0, 7.0], [4.0, 5.0, 2.0]], [[3.0, 6.0, 6.0], [3.0, 6.0, 2.0], [2.0, 8.0, 
8.0]]], [[[2.0, 6.0, 1.0], [1.0, 6.0, 7.0], [5.0, 7.0, 2.0]], [[4.0, 7.0, 1.0], [8.0, 6.0, 3.0], [6.0, 2.0, 4.0]], [[4.0, 4.0, 
6.0], [3.0, 8.0, 6.0], [1.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 3.0], [4.0, 3.0, 1.0], [8.0, 3.0, 8.0]], 
[[7.0, 8.0, 8.0], [1.0, 3.0, 3.0], [5.0, 7.0, 6.0]], [[4.0, 3.0, 3.0], [2.0, 2.0, 1.0], [6.0, 3.0, 2.0]]], [[[4.0, 6.0, 7.0], 
[4.0, 5.0, 7.0], [3.0, 7.0, 2.0]], [[1.0, 8.0, 4.0], [2.0, 3.0, 8.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 7.0], [4.0, 8.0, 4.0], 
[1.0, 6.0, 8.0]]], [[[5.0, 1.0, 4.0], [7.0, 5.0, 8.0], [3.0, 7.0, 4.0]], [[3.0, 4.0, 2.0], [3.0, 6.0, 2.0], [5.0, 8.0, 1.0]], 
[[5.0, 7.0, 5.0], [6.0, 7.0, 8.0], [1.0, 4.0, 2.0]]], [[[2.0, 5.0, 1.0], [2.0, 4.0, 7.0], [2.0, 1.0, 4.0]], [[6.0, 7.0, 1.0], 
[7.0, 8.0, 2.0], [7.0, 5.0, 6.0]], [[4.0, 2.0, 3.0], [1.0, 4.0, 4.0], [6.0, 6.0, 7.0]]]])
      val=numpy.array([[[[8.0, 3.0], [5.0, 7.0]], [[1.0, 7.0], [4.0, 2.0]]], [[[2.0, 7.0], [2.0, 6.0]], [[6.0, 1.0], [6.0, 
5.0]]]])
      arg[1:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 3.0], [3.0, 5.0, 7.0], [1.0, 1.0, 8.0]], [[8.0, 4.0, 5.0], [4.0, 2.0, 3.0], [5.0, 
2.0, 5.0]], [[3.0, 3.0, 7.0], [2.0, 5.0, 3.0], [1.0, 2.0, 5.0]]], [[[8.0, 3.0, 5.0], [5.0, 7.0, 8.0], [3.0, 8.0, 4.0]], [[1.0, 
7.0, 5.0], [4.0, 2.0, 6.0], [1.0, 4.0, 3.0]], [[7.0, 1.0, 3.0], [1.0, 6.0, 2.0], [4.0, 1.0, 4.0]]], [[[2.0, 7.0, 4.0], [2.0, 
6.0, 3.0], [3.0, 7.0, 7.0]], [[6.0, 1.0, 3.0], [6.0, 5.0, 7.0], [4.0, 5.0, 2.0]], [[3.0, 6.0, 6.0], [3.0, 6.0, 2.0], [2.0, 8.0, 
8.0]]], [[[2.0, 6.0, 1.0], [1.0, 6.0, 7.0], [5.0, 7.0, 2.0]], [[4.0, 7.0, 1.0], [8.0, 6.0, 3.0], [6.0, 2.0, 4.0]], [[4.0, 4.0, 
6.0], [3.0, 8.0, 6.0], [1.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 3.0], [4.0, 3.0, 1.0], [8.0, 3.0, 8.0]], 
[[7.0, 8.0, 8.0], [1.0, 3.0, 3.0], [5.0, 7.0, 6.0]], [[4.0, 3.0, 3.0], [2.0, 2.0, 1.0], [6.0, 3.0, 2.0]]], [[[8.0, 3.0, 7.0], 
[5.0, 7.0, 7.0], [3.0, 7.0, 2.0]], [[1.0, 7.0, 4.0], [4.0, 2.0, 8.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 7.0], [4.0, 8.0, 4.0], 
[1.0, 6.0, 8.0]]], [[[2.0, 7.0, 4.0], [2.0, 6.0, 8.0], [3.0, 7.0, 4.0]], [[6.0, 1.0, 2.0], [6.0, 5.0, 2.0], [5.0, 8.0, 1.0]], 
[[5.0, 7.0, 5.0], [6.0, 7.0, 8.0], [1.0, 4.0, 2.0]]], [[[2.0, 5.0, 1.0], [2.0, 4.0, 7.0], [2.0, 1.0, 4.0]], [[6.0, 7.0, 1.0], 
[7.0, 8.0, 2.0], [7.0, 5.0, 6.0]], [[4.0, 2.0, 3.0], [1.0, 4.0, 4.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 4.0], [7.0, 6.0, 7.0], [3.0, 6.0, 1.0]], [[3.0, 8.0, 6.0], [8.0, 2.0, 5.0], [2.0, 
4.0, 2.0]], [[6.0, 7.0, 2.0], [5.0, 5.0, 6.0], [5.0, 8.0, 2.0]]], [[[7.0, 4.0, 3.0], [6.0, 5.0, 2.0], [7.0, 3.0, 2.0]], [[7.0, 
5.0, 8.0], [3.0, 2.0, 6.0], [5.0, 4.0, 1.0]], [[6.0, 1.0, 3.0], [7.0, 4.0, 4.0], [6.0, 1.0, 6.0]]], [[[5.0, 8.0, 7.0], [2.0, 
2.0, 1.0], [2.0, 6.0, 4.0]], [[7.0, 2.0, 6.0], [4.0, 6.0, 1.0], [6.0, 7.0, 4.0]], [[6.0, 5.0, 2.0], [5.0, 4.0, 5.0], [8.0, 6.0, 
8.0]]], [[[5.0, 5.0, 4.0], [5.0, 8.0, 5.0], [2.0, 3.0, 7.0]], [[1.0, 2.0, 2.0], [3.0, 2.0, 8.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 
7.0], [4.0, 8.0, 6.0], [7.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 3.0], [7.0, 1.0, 7.0], [1.0, 8.0, 8.0]], 
[[1.0, 7.0, 1.0], [3.0, 7.0, 5.0], [8.0, 7.0, 3.0]], [[7.0, 3.0, 5.0], [7.0, 7.0, 5.0], [8.0, 6.0, 7.0]]], [[[7.0, 1.0, 4.0], 
[5.0, 1.0, 5.0], [7.0, 8.0, 2.0]], [[3.0, 5.0, 1.0], [8.0, 3.0, 1.0], [2.0, 4.0, 6.0]], [[6.0, 5.0, 8.0], [8.0, 3.0, 4.0], 
[8.0, 3.0, 1.0]]], [[[5.0, 5.0, 8.0], [5.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 1.0, 1.0], [6.0, 7.0, 6.0]], 
[[6.0, 3.0, 1.0], [8.0, 4.0, 2.0], [3.0, 5.0, 1.0]]], [[[5.0, 5.0, 7.0], [8.0, 3.0, 4.0], [8.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], 
[2.0, 6.0, 5.0], [8.0, 4.0, 1.0]], [[5.0, 1.0, 6.0], [4.0, 2.0, 1.0], [7.0, 8.0, 7.0]]]])
      val=Data(numpy.array([[[[8.0, 5.0], [4.0, 8.0]], [[2.0, 4.0], [8.0, 1.0]]], [[[5.0, 6.0], [6.0, 8.0]], [[2.0, 1.0], [7.0, 
1.0]]]]),self.functionspace)
      arg[1:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 4.0], [7.0, 6.0, 7.0], [3.0, 6.0, 1.0]], [[3.0, 8.0, 6.0], [8.0, 2.0, 5.0], [2.0, 
4.0, 2.0]], [[6.0, 7.0, 2.0], [5.0, 5.0, 6.0], [5.0, 8.0, 2.0]]], [[[8.0, 5.0, 3.0], [4.0, 8.0, 2.0], [7.0, 3.0, 2.0]], [[2.0, 
4.0, 8.0], [8.0, 1.0, 6.0], [5.0, 4.0, 1.0]], [[6.0, 1.0, 3.0], [7.0, 4.0, 4.0], [6.0, 1.0, 6.0]]], [[[5.0, 6.0, 7.0], [6.0, 
8.0, 1.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 6.0], [7.0, 1.0, 1.0], [6.0, 7.0, 4.0]], [[6.0, 5.0, 2.0], [5.0, 4.0, 5.0], [8.0, 6.0, 
8.0]]], [[[5.0, 5.0, 4.0], [5.0, 8.0, 5.0], [2.0, 3.0, 7.0]], [[1.0, 2.0, 2.0], [3.0, 2.0, 8.0], [3.0, 4.0, 7.0]], [[5.0, 2.0, 
7.0], [4.0, 8.0, 6.0], [7.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 3.0], [7.0, 1.0, 7.0], [1.0, 8.0, 8.0]], 
[[1.0, 7.0, 1.0], [3.0, 7.0, 5.0], [8.0, 7.0, 3.0]], [[7.0, 3.0, 5.0], [7.0, 7.0, 5.0], [8.0, 6.0, 7.0]]], [[[8.0, 5.0, 4.0], 
[4.0, 8.0, 5.0], [7.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [8.0, 1.0, 1.0], [2.0, 4.0, 6.0]], [[6.0, 5.0, 8.0], [8.0, 3.0, 4.0], 
[8.0, 3.0, 1.0]]], [[[5.0, 6.0, 8.0], [6.0, 8.0, 1.0], [8.0, 5.0, 2.0]], [[2.0, 1.0, 8.0], [7.0, 1.0, 1.0], [6.0, 7.0, 6.0]], 
[[6.0, 3.0, 1.0], [8.0, 4.0, 2.0], [3.0, 5.0, 1.0]]], [[[5.0, 5.0, 7.0], [8.0, 3.0, 4.0], [8.0, 7.0, 4.0]], [[3.0, 3.0, 2.0], 
[2.0, 6.0, 5.0], [8.0, 4.0, 1.0]], [[5.0, 1.0, 6.0], [4.0, 2.0, 1.0], [7.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RUUU(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 1.0], [2.0, 4.0, 2.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [1.0, 4.0, 2.0], [1.0, 
3.0, 1.0]], [[3.0, 6.0, 2.0], [8.0, 1.0, 8.0], [2.0, 1.0, 4.0]]], [[[3.0, 4.0, 3.0], [3.0, 6.0, 8.0], [7.0, 6.0, 6.0]], [[7.0, 
4.0, 6.0], [7.0, 5.0, 6.0], [8.0, 3.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 5.0], [5.0, 5.0, 7.0]]], [[[2.0, 1.0, 7.0], [2.0, 
4.0, 2.0], [8.0, 7.0, 3.0]], [[2.0, 3.0, 4.0], [4.0, 2.0, 4.0], [1.0, 7.0, 7.0]], [[4.0, 2.0, 3.0], [3.0, 6.0, 5.0], [1.0, 8.0, 
7.0]]], [[[8.0, 3.0, 5.0], [5.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 1.0, 5.0], [3.0, 8.0, 1.0], [2.0, 6.0, 4.0]], [[8.0, 3.0, 
3.0], [5.0, 4.0, 4.0], [8.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 2.0], [7.0, 5.0, 3.0], [1.0, 4.0, 7.0]], 
[[3.0, 4.0, 6.0], [6.0, 5.0, 3.0], [1.0, 5.0, 8.0]], [[4.0, 5.0, 4.0], [3.0, 5.0, 3.0], [1.0, 4.0, 3.0]]], [[[1.0, 6.0, 2.0], 
[6.0, 8.0, 2.0], [3.0, 1.0, 1.0]], [[1.0, 2.0, 6.0], [2.0, 2.0, 8.0], [8.0, 4.0, 5.0]], [[6.0, 5.0, 2.0], [7.0, 3.0, 1.0], 
[5.0, 6.0, 7.0]]], [[[3.0, 5.0, 7.0], [6.0, 5.0, 4.0], [5.0, 1.0, 8.0]], [[6.0, 7.0, 6.0], [5.0, 4.0, 2.0], [1.0, 6.0, 1.0]], 
[[2.0, 1.0, 4.0], [2.0, 4.0, 1.0], [8.0, 2.0, 5.0]]], [[[1.0, 6.0, 8.0], [4.0, 4.0, 2.0], [4.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], 
[4.0, 3.0, 4.0], [2.0, 3.0, 2.0]], [[8.0, 8.0, 2.0], [6.0, 3.0, 4.0], [7.0, 5.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 5.0], [7.0, 7.0]], [[2.0, 8.0], [8.0, 5.0]]], [[[5.0, 6.0], [1.0, 4.0]], [[6.0, 2.0], 
[8.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 3.0], [2.0, 1.0]], [[1.0, 8.0], [1.0, 2.0]]], [[[2.0, 8.0], [5.0, 5.0]], 
[[2.0, 4.0], [4.0, 1.0]]]])
      arg[1:3,:2,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 1.0], [2.0, 4.0, 2.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [1.0, 4.0, 2.0], [1.0, 
3.0, 1.0]], [[3.0, 6.0, 2.0], [8.0, 1.0, 8.0], [2.0, 1.0, 4.0]]], [[[1.0, 5.0, 3.0], [7.0, 7.0, 8.0], [7.0, 6.0, 6.0]], [[2.0, 
8.0, 6.0], [8.0, 5.0, 6.0], [8.0, 3.0, 3.0]], [[8.0, 3.0, 7.0], [2.0, 5.0, 5.0], [5.0, 5.0, 7.0]]], [[[5.0, 6.0, 7.0], [1.0, 
4.0, 2.0], [8.0, 7.0, 3.0]], [[6.0, 2.0, 4.0], [8.0, 2.0, 4.0], [1.0, 7.0, 7.0]], [[4.0, 2.0, 3.0], [3.0, 6.0, 5.0], [1.0, 8.0, 
7.0]]], [[[8.0, 3.0, 5.0], [5.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 1.0, 5.0], [3.0, 8.0, 1.0], [2.0, 6.0, 4.0]], [[8.0, 3.0, 
3.0], [5.0, 4.0, 4.0], [8.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 2.0], [7.0, 5.0, 3.0], [1.0, 4.0, 7.0]], 
[[3.0, 4.0, 6.0], [6.0, 5.0, 3.0], [1.0, 5.0, 8.0]], [[4.0, 5.0, 4.0], [3.0, 5.0, 3.0], [1.0, 4.0, 3.0]]], [[[7.0, 3.0, 2.0], 
[2.0, 1.0, 2.0], [3.0, 1.0, 1.0]], [[1.0, 8.0, 6.0], [1.0, 2.0, 8.0], [8.0, 4.0, 5.0]], [[6.0, 5.0, 2.0], [7.0, 3.0, 1.0], 
[5.0, 6.0, 7.0]]], [[[2.0, 8.0, 7.0], [5.0, 5.0, 4.0], [5.0, 1.0, 8.0]], [[2.0, 4.0, 6.0], [4.0, 1.0, 2.0], [1.0, 6.0, 1.0]], 
[[2.0, 1.0, 4.0], [2.0, 4.0, 1.0], [8.0, 2.0, 5.0]]], [[[1.0, 6.0, 8.0], [4.0, 4.0, 2.0], [4.0, 4.0, 8.0]], [[2.0, 8.0, 7.0], 
[4.0, 3.0, 4.0], [2.0, 3.0, 2.0]], [[8.0, 8.0, 2.0], [6.0, 3.0, 4.0], [7.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 5.0], [3.0, 7.0, 3.0], [7.0, 7.0, 8.0]], [[8.0, 4.0, 8.0], [7.0, 8.0, 6.0], [5.0, 
5.0, 5.0]], [[7.0, 2.0, 3.0], [8.0, 7.0, 2.0], [4.0, 4.0, 4.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 8.0], [8.0, 8.0, 4.0]], [[8.0, 
1.0, 1.0], [6.0, 8.0, 6.0], [2.0, 1.0, 2.0]], [[6.0, 5.0, 3.0], [3.0, 8.0, 3.0], [4.0, 1.0, 4.0]]], [[[4.0, 1.0, 6.0], [5.0, 
8.0, 1.0], [2.0, 3.0, 1.0]], [[6.0, 6.0, 6.0], [5.0, 8.0, 3.0], [4.0, 2.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 5.0, 2.0], [8.0, 7.0, 
7.0]]], [[[8.0, 3.0, 8.0], [1.0, 4.0, 3.0], [8.0, 6.0, 1.0]], [[3.0, 7.0, 4.0], [7.0, 3.0, 2.0], [3.0, 3.0, 3.0]], [[8.0, 3.0, 
1.0], [2.0, 4.0, 6.0], [2.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 1.0], [6.0, 7.0, 5.0], [3.0, 1.0, 1.0]], 
[[1.0, 6.0, 6.0], [4.0, 3.0, 6.0], [6.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 6.0], [4.0, 3.0, 1.0]]], [[[5.0, 6.0, 6.0], 
[4.0, 4.0, 4.0], [7.0, 6.0, 3.0]], [[5.0, 3.0, 1.0], [5.0, 8.0, 4.0], [7.0, 5.0, 3.0]], [[2.0, 1.0, 5.0], [8.0, 1.0, 1.0], 
[2.0, 8.0, 7.0]]], [[[2.0, 8.0, 8.0], [1.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[2.0, 4.0, 6.0], [4.0, 6.0, 8.0], [7.0, 2.0, 6.0]], 
[[4.0, 3.0, 5.0], [8.0, 2.0, 5.0], [6.0, 4.0, 6.0]]], [[[8.0, 1.0, 3.0], [3.0, 7.0, 8.0], [6.0, 5.0, 8.0]], [[6.0, 1.0, 8.0], 
[7.0, 6.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 8.0, 2.0], [4.0, 7.0, 2.0], [6.0, 4.0, 6.0]]]])
      val=numpy.array([[[3.0, 1.0, 2.0]], [[8.0, 8.0, 6.0]]])
      arg[2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 5.0], [3.0, 7.0, 3.0], [7.0, 7.0, 8.0]], [[8.0, 4.0, 8.0], [7.0, 8.0, 6.0], [5.0, 
5.0, 5.0]], [[7.0, 2.0, 3.0], [8.0, 7.0, 2.0], [4.0, 4.0, 4.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 8.0], [8.0, 8.0, 4.0]], [[8.0, 
1.0, 1.0], [6.0, 8.0, 6.0], [2.0, 1.0, 2.0]], [[6.0, 5.0, 3.0], [3.0, 8.0, 3.0], [4.0, 1.0, 4.0]]], [[[4.0, 1.0, 6.0], [5.0, 
8.0, 1.0], [2.0, 3.0, 1.0]], [[6.0, 6.0, 6.0], [3.0, 1.0, 2.0], [4.0, 2.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 5.0, 2.0], [8.0, 7.0, 
7.0]]], [[[8.0, 3.0, 8.0], [1.0, 4.0, 3.0], [8.0, 6.0, 1.0]], [[3.0, 7.0, 4.0], [8.0, 8.0, 6.0], [3.0, 3.0, 3.0]], [[8.0, 3.0, 
1.0], [2.0, 4.0, 6.0], [2.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 1.0], [6.0, 7.0, 5.0], [3.0, 1.0, 1.0]], 
[[1.0, 6.0, 6.0], [4.0, 3.0, 6.0], [6.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 6.0], [4.0, 3.0, 1.0]]], [[[5.0, 6.0, 6.0], 
[4.0, 4.0, 4.0], [7.0, 6.0, 3.0]], [[5.0, 3.0, 1.0], [5.0, 8.0, 4.0], [7.0, 5.0, 3.0]], [[2.0, 1.0, 5.0], [8.0, 1.0, 1.0], 
[2.0, 8.0, 7.0]]], [[[2.0, 8.0, 8.0], [1.0, 5.0, 3.0], [6.0, 6.0, 5.0]], [[2.0, 4.0, 6.0], [3.0, 1.0, 2.0], [7.0, 2.0, 6.0]], 
[[4.0, 3.0, 5.0], [8.0, 2.0, 5.0], [6.0, 4.0, 6.0]]], [[[8.0, 1.0, 3.0], [3.0, 7.0, 8.0], [6.0, 5.0, 8.0]], [[6.0, 1.0, 8.0], 
[8.0, 8.0, 6.0], [5.0, 3.0, 6.0]], [[8.0, 8.0, 2.0], [4.0, 7.0, 2.0], [6.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 5.0], [5.0, 1.0, 8.0], [4.0, 2.0, 1.0]], [[6.0, 7.0, 6.0], [7.0, 8.0, 6.0], [5.0, 
4.0, 5.0]], [[5.0, 2.0, 8.0], [5.0, 8.0, 8.0], [1.0, 8.0, 4.0]]], [[[3.0, 1.0, 7.0], [2.0, 6.0, 4.0], [8.0, 4.0, 5.0]], [[2.0, 
4.0, 8.0], [2.0, 2.0, 5.0], [8.0, 3.0, 8.0]], [[2.0, 6.0, 6.0], [4.0, 7.0, 3.0], [5.0, 8.0, 8.0]]], [[[5.0, 4.0, 8.0], [6.0, 
5.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 8.0, 3.0], [3.0, 5.0, 4.0], [3.0, 6.0, 1.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 1.0], [1.0, 4.0, 
3.0]]], [[[6.0, 5.0, 8.0], [3.0, 4.0, 2.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 4.0], [5.0, 8.0, 1.0], [8.0, 1.0, 2.0]], [[3.0, 1.0, 
8.0], [7.0, 1.0, 3.0], [8.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 2.0], [7.0, 8.0, 8.0], [2.0, 4.0, 1.0]], 
[[8.0, 7.0, 4.0], [7.0, 1.0, 8.0], [8.0, 1.0, 2.0]], [[1.0, 3.0, 6.0], [3.0, 8.0, 6.0], [7.0, 5.0, 3.0]]], [[[7.0, 5.0, 4.0], 
[5.0, 1.0, 5.0], [6.0, 1.0, 4.0]], [[2.0, 2.0, 3.0], [7.0, 8.0, 5.0], [4.0, 2.0, 6.0]], [[2.0, 1.0, 6.0], [1.0, 5.0, 3.0], 
[4.0, 7.0, 6.0]]], [[[8.0, 7.0, 7.0], [4.0, 2.0, 1.0], [8.0, 1.0, 6.0]], [[4.0, 7.0, 2.0], [3.0, 4.0, 3.0], [2.0, 8.0, 7.0]], 
[[2.0, 2.0, 1.0], [5.0, 3.0, 3.0], [4.0, 2.0, 3.0]]], [[[7.0, 1.0, 4.0], [7.0, 1.0, 4.0], [8.0, 6.0, 1.0]], [[6.0, 5.0, 5.0], 
[4.0, 8.0, 8.0], [3.0, 7.0, 6.0]], [[3.0, 7.0, 6.0], [1.0, 2.0, 5.0], [8.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[4.0, 5.0, 4.0]], [[2.0, 8.0, 4.0]]]),self.functionspace)
      arg[2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 5.0], [5.0, 1.0, 8.0], [4.0, 2.0, 1.0]], [[6.0, 7.0, 6.0], [7.0, 8.0, 6.0], [5.0, 
4.0, 5.0]], [[5.0, 2.0, 8.0], [5.0, 8.0, 8.0], [1.0, 8.0, 4.0]]], [[[3.0, 1.0, 7.0], [2.0, 6.0, 4.0], [8.0, 4.0, 5.0]], [[2.0, 
4.0, 8.0], [2.0, 2.0, 5.0], [8.0, 3.0, 8.0]], [[2.0, 6.0, 6.0], [4.0, 7.0, 3.0], [5.0, 8.0, 8.0]]], [[[5.0, 4.0, 8.0], [6.0, 
5.0, 1.0], [3.0, 3.0, 2.0]], [[7.0, 8.0, 3.0], [4.0, 5.0, 4.0], [3.0, 6.0, 1.0]], [[3.0, 1.0, 6.0], [7.0, 5.0, 1.0], [1.0, 4.0, 
3.0]]], [[[6.0, 5.0, 8.0], [3.0, 4.0, 2.0], [7.0, 1.0, 3.0]], [[1.0, 5.0, 4.0], [2.0, 8.0, 4.0], [8.0, 1.0, 2.0]], [[3.0, 1.0, 
8.0], [7.0, 1.0, 3.0], [8.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 2.0], [7.0, 8.0, 8.0], [2.0, 4.0, 1.0]], 
[[8.0, 7.0, 4.0], [7.0, 1.0, 8.0], [8.0, 1.0, 2.0]], [[1.0, 3.0, 6.0], [3.0, 8.0, 6.0], [7.0, 5.0, 3.0]]], [[[7.0, 5.0, 4.0], 
[5.0, 1.0, 5.0], [6.0, 1.0, 4.0]], [[2.0, 2.0, 3.0], [7.0, 8.0, 5.0], [4.0, 2.0, 6.0]], [[2.0, 1.0, 6.0], [1.0, 5.0, 3.0], 
[4.0, 7.0, 6.0]]], [[[8.0, 7.0, 7.0], [4.0, 2.0, 1.0], [8.0, 1.0, 6.0]], [[4.0, 7.0, 2.0], [4.0, 5.0, 4.0], [2.0, 8.0, 7.0]], 
[[2.0, 2.0, 1.0], [5.0, 3.0, 3.0], [4.0, 2.0, 3.0]]], [[[7.0, 1.0, 4.0], [7.0, 1.0, 4.0], [8.0, 6.0, 1.0]], [[6.0, 5.0, 5.0], 
[2.0, 8.0, 4.0], [3.0, 7.0, 6.0]], [[3.0, 7.0, 6.0], [1.0, 2.0, 5.0], [8.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 1.0], [6.0, 4.0, 5.0], [6.0, 1.0, 8.0]], [[8.0, 2.0, 7.0], [2.0, 6.0, 7.0], [8.0, 
5.0, 3.0]], [[7.0, 5.0, 6.0], [8.0, 1.0, 6.0], [4.0, 5.0, 1.0]]], [[[5.0, 1.0, 7.0], [4.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[6.0, 
3.0, 3.0], [8.0, 7.0, 3.0], [8.0, 3.0, 4.0]], [[8.0, 4.0, 3.0], [3.0, 4.0, 2.0], [2.0, 6.0, 5.0]]], [[[5.0, 7.0, 6.0], [6.0, 
5.0, 1.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 1.0], [3.0, 4.0, 4.0], [4.0, 1.0, 3.0]], [[4.0, 1.0, 1.0], [5.0, 4.0, 8.0], [2.0, 5.0, 
8.0]]], [[[4.0, 4.0, 6.0], [2.0, 8.0, 1.0], [2.0, 6.0, 4.0]], [[1.0, 4.0, 7.0], [5.0, 7.0, 4.0], [1.0, 3.0, 7.0]], [[7.0, 8.0, 
3.0], [1.0, 1.0, 6.0], [4.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 7.0], [8.0, 1.0, 6.0], [7.0, 7.0, 4.0]], 
[[4.0, 1.0, 7.0], [2.0, 6.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 7.0, 2.0], [7.0, 6.0, 6.0], [7.0, 1.0, 5.0]]], [[[3.0, 7.0, 4.0], 
[3.0, 5.0, 4.0], [1.0, 1.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 3.0, 4.0], [1.0, 5.0, 8.0]], [[8.0, 5.0, 7.0], [8.0, 7.0, 4.0], 
[8.0, 6.0, 8.0]]], [[[7.0, 6.0, 6.0], [8.0, 2.0, 7.0], [8.0, 6.0, 8.0]], [[2.0, 4.0, 1.0], [5.0, 7.0, 5.0], [2.0, 3.0, 2.0]], 
[[5.0, 2.0, 6.0], [8.0, 1.0, 1.0], [3.0, 8.0, 1.0]]], [[[4.0, 3.0, 8.0], [7.0, 4.0, 1.0], [2.0, 3.0, 5.0]], [[1.0, 4.0, 3.0], 
[2.0, 8.0, 5.0], [8.0, 7.0, 3.0]], [[3.0, 7.0, 5.0], [4.0, 3.0, 3.0], [2.0, 2.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 5.0, 7.0]], [[2.0, 7.0, 3.0]]])+(1.-msk_val)*numpy.array([[[1.0, 6.0, 2.0]], [[1.0, 4.0, 
2.0]]])
      arg[2:,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 1.0], [6.0, 4.0, 5.0], [6.0, 1.0, 8.0]], [[8.0, 2.0, 7.0], [2.0, 6.0, 7.0], [8.0, 
5.0, 3.0]], [[7.0, 5.0, 6.0], [8.0, 1.0, 6.0], [4.0, 5.0, 1.0]]], [[[5.0, 1.0, 7.0], [4.0, 1.0, 3.0], [4.0, 8.0, 1.0]], [[6.0, 
3.0, 3.0], [8.0, 7.0, 3.0], [8.0, 3.0, 4.0]], [[8.0, 4.0, 3.0], [3.0, 4.0, 2.0], [2.0, 6.0, 5.0]]], [[[5.0, 7.0, 6.0], [6.0, 
5.0, 1.0], [2.0, 4.0, 3.0]], [[3.0, 5.0, 1.0], [7.0, 5.0, 7.0], [4.0, 1.0, 3.0]], [[4.0, 1.0, 1.0], [5.0, 4.0, 8.0], [2.0, 5.0, 
8.0]]], [[[4.0, 4.0, 6.0], [2.0, 8.0, 1.0], [2.0, 6.0, 4.0]], [[1.0, 4.0, 7.0], [2.0, 7.0, 3.0], [1.0, 3.0, 7.0]], [[7.0, 8.0, 
3.0], [1.0, 1.0, 6.0], [4.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 7.0], [8.0, 1.0, 6.0], [7.0, 7.0, 4.0]], 
[[4.0, 1.0, 7.0], [2.0, 6.0, 2.0], [8.0, 2.0, 8.0]], [[7.0, 7.0, 2.0], [7.0, 6.0, 6.0], [7.0, 1.0, 5.0]]], [[[3.0, 7.0, 4.0], 
[3.0, 5.0, 4.0], [1.0, 1.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 3.0, 4.0], [1.0, 5.0, 8.0]], [[8.0, 5.0, 7.0], [8.0, 7.0, 4.0], 
[8.0, 6.0, 8.0]]], [[[7.0, 6.0, 6.0], [8.0, 2.0, 7.0], [8.0, 6.0, 8.0]], [[2.0, 4.0, 1.0], [1.0, 6.0, 2.0], [2.0, 3.0, 2.0]], 
[[5.0, 2.0, 6.0], [8.0, 1.0, 1.0], [3.0, 8.0, 1.0]]], [[[4.0, 3.0, 8.0], [7.0, 4.0, 1.0], [2.0, 3.0, 5.0]], [[1.0, 4.0, 3.0], 
[1.0, 4.0, 2.0], [8.0, 7.0, 3.0]], [[3.0, 7.0, 5.0], [4.0, 3.0, 3.0], [2.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 7.0], [2.0, 8.0, 7.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 5.0], [4.0, 6.0, 7.0], [1.0, 
2.0, 3.0]], [[5.0, 6.0, 1.0], [8.0, 4.0, 5.0], [3.0, 1.0, 7.0]]], [[[3.0, 6.0, 8.0], [1.0, 3.0, 4.0], [7.0, 8.0, 6.0]], [[6.0, 
2.0, 6.0], [8.0, 7.0, 5.0], [1.0, 5.0, 1.0]], [[2.0, 7.0, 7.0], [7.0, 7.0, 4.0], [8.0, 6.0, 6.0]]], [[[6.0, 4.0, 4.0], [3.0, 
7.0, 3.0], [3.0, 7.0, 6.0]], [[8.0, 1.0, 6.0], [4.0, 3.0, 7.0], [5.0, 7.0, 8.0]], [[5.0, 7.0, 6.0], [5.0, 3.0, 3.0], [1.0, 5.0, 
7.0]]], [[[6.0, 2.0, 4.0], [3.0, 8.0, 8.0], [4.0, 1.0, 4.0]], [[1.0, 8.0, 2.0], [6.0, 7.0, 8.0], [2.0, 7.0, 3.0]], [[3.0, 2.0, 
4.0], [6.0, 2.0, 8.0], [3.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 4.0], [4.0, 5.0, 7.0], [1.0, 2.0, 5.0]], 
[[6.0, 8.0, 7.0], [7.0, 2.0, 3.0], [6.0, 6.0, 3.0]], [[2.0, 8.0, 8.0], [1.0, 8.0, 5.0], [5.0, 2.0, 7.0]]], [[[7.0, 8.0, 5.0], 
[2.0, 3.0, 5.0], [3.0, 6.0, 5.0]], [[8.0, 5.0, 5.0], [4.0, 8.0, 3.0], [6.0, 3.0, 3.0]], [[1.0, 5.0, 5.0], [6.0, 2.0, 1.0], 
[6.0, 6.0, 5.0]]], [[[1.0, 5.0, 6.0], [2.0, 4.0, 6.0], [8.0, 1.0, 6.0]], [[8.0, 6.0, 2.0], [1.0, 3.0, 6.0], [8.0, 5.0, 8.0]], 
[[6.0, 3.0, 8.0], [6.0, 4.0, 7.0], [1.0, 8.0, 2.0]]], [[[1.0, 8.0, 1.0], [3.0, 2.0, 4.0], [2.0, 7.0, 3.0]], [[5.0, 1.0, 2.0], 
[6.0, 6.0, 6.0], [6.0, 2.0, 2.0]], [[1.0, 5.0, 8.0], [5.0, 8.0, 2.0], [4.0, 3.0, 4.0]]]])
      val=numpy.array([[[[3.0, 2.0, 2.0]], [[8.0, 3.0, 3.0]], [[1.0, 5.0, 1.0]]], [[[8.0, 3.0, 8.0]], [[8.0, 1.0, 3.0]], [[3.0, 
3.0, 6.0]]], [[[7.0, 8.0, 1.0]], [[4.0, 2.0, 6.0]], [[6.0, 8.0, 8.0]]], [[[4.0, 2.0, 5.0]], [[3.0, 3.0, 8.0]], [[7.0, 4.0, 
7.0]]]])
      arg[:,:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 7.0], [3.0, 2.0, 2.0], [2.0, 7.0, 7.0]], [[4.0, 2.0, 5.0], [8.0, 3.0, 3.0], [1.0, 
2.0, 3.0]], [[5.0, 6.0, 1.0], [1.0, 5.0, 1.0], [3.0, 1.0, 7.0]]], [[[3.0, 6.0, 8.0], [8.0, 3.0, 8.0], [7.0, 8.0, 6.0]], [[6.0, 
2.0, 6.0], [8.0, 1.0, 3.0], [1.0, 5.0, 1.0]], [[2.0, 7.0, 7.0], [3.0, 3.0, 6.0], [8.0, 6.0, 6.0]]], [[[6.0, 4.0, 4.0], [7.0, 
8.0, 1.0], [3.0, 7.0, 6.0]], [[8.0, 1.0, 6.0], [4.0, 2.0, 6.0], [5.0, 7.0, 8.0]], [[5.0, 7.0, 6.0], [6.0, 8.0, 8.0], [1.0, 5.0, 
7.0]]], [[[6.0, 2.0, 4.0], [4.0, 2.0, 5.0], [4.0, 1.0, 4.0]], [[1.0, 8.0, 2.0], [3.0, 3.0, 8.0], [2.0, 7.0, 3.0]], [[3.0, 2.0, 
4.0], [7.0, 4.0, 7.0], [3.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 4.0], [3.0, 2.0, 2.0], [1.0, 2.0, 5.0]], 
[[6.0, 8.0, 7.0], [8.0, 3.0, 3.0], [6.0, 6.0, 3.0]], [[2.0, 8.0, 8.0], [1.0, 5.0, 1.0], [5.0, 2.0, 7.0]]], [[[7.0, 8.0, 5.0], 
[8.0, 3.0, 8.0], [3.0, 6.0, 5.0]], [[8.0, 5.0, 5.0], [8.0, 1.0, 3.0], [6.0, 3.0, 3.0]], [[1.0, 5.0, 5.0], [3.0, 3.0, 6.0], 
[6.0, 6.0, 5.0]]], [[[1.0, 5.0, 6.0], [7.0, 8.0, 1.0], [8.0, 1.0, 6.0]], [[8.0, 6.0, 2.0], [4.0, 2.0, 6.0], [8.0, 5.0, 8.0]], 
[[6.0, 3.0, 8.0], [6.0, 8.0, 8.0], [1.0, 8.0, 2.0]]], [[[1.0, 8.0, 1.0], [4.0, 2.0, 5.0], [2.0, 7.0, 3.0]], [[5.0, 1.0, 2.0], 
[3.0, 3.0, 8.0], [6.0, 2.0, 2.0]], [[1.0, 5.0, 8.0], [7.0, 4.0, 7.0], [4.0, 3.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 7.0], [3.0, 4.0, 3.0], [2.0, 3.0, 3.0]], [[3.0, 6.0, 7.0], [5.0, 6.0, 4.0], [8.0, 
7.0, 2.0]], [[1.0, 3.0, 4.0], [2.0, 3.0, 1.0], [7.0, 2.0, 7.0]]], [[[2.0, 3.0, 3.0], [5.0, 1.0, 5.0], [3.0, 4.0, 6.0]], [[1.0, 
8.0, 7.0], [4.0, 6.0, 3.0], [4.0, 6.0, 5.0]], [[7.0, 5.0, 4.0], [3.0, 3.0, 7.0], [5.0, 8.0, 2.0]]], [[[1.0, 4.0, 5.0], [1.0, 
4.0, 5.0], [4.0, 3.0, 2.0]], [[4.0, 8.0, 4.0], [7.0, 2.0, 5.0], [1.0, 6.0, 4.0]], [[3.0, 5.0, 1.0], [3.0, 2.0, 1.0], [2.0, 4.0, 
1.0]]], [[[2.0, 2.0, 1.0], [1.0, 6.0, 1.0], [3.0, 2.0, 7.0]], [[3.0, 7.0, 7.0], [1.0, 6.0, 6.0], [6.0, 5.0, 8.0]], [[5.0, 3.0, 
3.0], [2.0, 4.0, 3.0], [5.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 3.0], [4.0, 1.0, 1.0], [6.0, 5.0, 6.0]], 
[[7.0, 7.0, 6.0], [8.0, 7.0, 6.0], [1.0, 2.0, 3.0]], [[6.0, 2.0, 2.0], [7.0, 5.0, 5.0], [4.0, 2.0, 4.0]]], [[[2.0, 5.0, 2.0], 
[3.0, 5.0, 7.0], [5.0, 4.0, 2.0]], [[4.0, 3.0, 4.0], [3.0, 2.0, 2.0], [7.0, 8.0, 5.0]], [[3.0, 8.0, 5.0], [4.0, 2.0, 3.0], 
[3.0, 6.0, 1.0]]], [[[3.0, 3.0, 4.0], [2.0, 4.0, 1.0], [5.0, 5.0, 6.0]], [[7.0, 3.0, 7.0], [6.0, 1.0, 4.0], [8.0, 6.0, 6.0]], 
[[8.0, 1.0, 3.0], [2.0, 4.0, 8.0], [2.0, 4.0, 8.0]]], [[[1.0, 6.0, 8.0], [5.0, 5.0, 6.0], [7.0, 7.0, 8.0]], [[1.0, 1.0, 3.0], 
[6.0, 5.0, 1.0], [1.0, 5.0, 3.0]], [[3.0, 5.0, 2.0], [2.0, 2.0, 5.0], [3.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[[2.0, 2.0, 4.0]], [[8.0, 4.0, 4.0]], [[7.0, 5.0, 1.0]]], [[[5.0, 4.0, 4.0]], [[5.0, 4.0, 6.0]], 
[[4.0, 6.0, 6.0]]], [[[8.0, 2.0, 7.0]], [[4.0, 8.0, 4.0]], [[3.0, 6.0, 8.0]]], [[[1.0, 1.0, 2.0]], [[3.0, 1.0, 1.0]], [[3.0, 
4.0, 3.0]]]]),self.functionspace)
      arg[:,:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 7.0], [2.0, 2.0, 4.0], [2.0, 3.0, 3.0]], [[3.0, 6.0, 7.0], [8.0, 4.0, 4.0], [8.0, 
7.0, 2.0]], [[1.0, 3.0, 4.0], [7.0, 5.0, 1.0], [7.0, 2.0, 7.0]]], [[[2.0, 3.0, 3.0], [5.0, 4.0, 4.0], [3.0, 4.0, 6.0]], [[1.0, 
8.0, 7.0], [5.0, 4.0, 6.0], [4.0, 6.0, 5.0]], [[7.0, 5.0, 4.0], [4.0, 6.0, 6.0], [5.0, 8.0, 2.0]]], [[[1.0, 4.0, 5.0], [8.0, 
2.0, 7.0], [4.0, 3.0, 2.0]], [[4.0, 8.0, 4.0], [4.0, 8.0, 4.0], [1.0, 6.0, 4.0]], [[3.0, 5.0, 1.0], [3.0, 6.0, 8.0], [2.0, 4.0, 
1.0]]], [[[2.0, 2.0, 1.0], [1.0, 1.0, 2.0], [3.0, 2.0, 7.0]], [[3.0, 7.0, 7.0], [3.0, 1.0, 1.0], [6.0, 5.0, 8.0]], [[5.0, 3.0, 
3.0], [3.0, 4.0, 3.0], [5.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 3.0], [2.0, 2.0, 4.0], [6.0, 5.0, 6.0]], 
[[7.0, 7.0, 6.0], [8.0, 4.0, 4.0], [1.0, 2.0, 3.0]], [[6.0, 2.0, 2.0], [7.0, 5.0, 1.0], [4.0, 2.0, 4.0]]], [[[2.0, 5.0, 2.0], 
[5.0, 4.0, 4.0], [5.0, 4.0, 2.0]], [[4.0, 3.0, 4.0], [5.0, 4.0, 6.0], [7.0, 8.0, 5.0]], [[3.0, 8.0, 5.0], [4.0, 6.0, 6.0], 
[3.0, 6.0, 1.0]]], [[[3.0, 3.0, 4.0], [8.0, 2.0, 7.0], [5.0, 5.0, 6.0]], [[7.0, 3.0, 7.0], [4.0, 8.0, 4.0], [8.0, 6.0, 6.0]], 
[[8.0, 1.0, 3.0], [3.0, 6.0, 8.0], [2.0, 4.0, 8.0]]], [[[1.0, 6.0, 8.0], [1.0, 1.0, 2.0], [7.0, 7.0, 8.0]], [[1.0, 1.0, 3.0], 
[3.0, 1.0, 1.0], [1.0, 5.0, 3.0]], [[3.0, 5.0, 2.0], [3.0, 4.0, 3.0], [3.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCRC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 2.0], [4.0, 1.0, 7.0], [1.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [5.0, 1.0, 6.0], [8.0, 
6.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 1.0, 5.0], [4.0, 2.0, 3.0]]], [[[8.0, 7.0, 2.0], [4.0, 7.0, 5.0], [4.0, 6.0, 1.0]], [[7.0, 
8.0, 1.0], [8.0, 7.0, 2.0], [1.0, 1.0, 3.0]], [[2.0, 5.0, 4.0], [2.0, 6.0, 5.0], [8.0, 6.0, 2.0]]], [[[2.0, 4.0, 5.0], [6.0, 
5.0, 2.0], [8.0, 7.0, 6.0]], [[3.0, 7.0, 5.0], [1.0, 1.0, 6.0], [2.0, 3.0, 3.0]], [[1.0, 3.0, 4.0], [4.0, 8.0, 2.0], [8.0, 5.0, 
5.0]]], [[[5.0, 5.0, 6.0], [5.0, 4.0, 5.0], [6.0, 2.0, 1.0]], [[2.0, 7.0, 8.0], [7.0, 1.0, 4.0], [2.0, 8.0, 3.0]], [[5.0, 7.0, 
7.0], [5.0, 4.0, 2.0], [1.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 1.0], [3.0, 5.0, 1.0], [1.0, 2.0, 8.0]], 
[[7.0, 4.0, 4.0], [2.0, 5.0, 7.0], [8.0, 1.0, 4.0]], [[7.0, 4.0, 4.0], [2.0, 1.0, 6.0], [8.0, 7.0, 5.0]]], [[[4.0, 2.0, 4.0], 
[6.0, 2.0, 6.0], [8.0, 8.0, 6.0]], [[8.0, 8.0, 7.0], [2.0, 7.0, 2.0], [8.0, 1.0, 2.0]], [[7.0, 5.0, 4.0], [1.0, 8.0, 1.0], 
[3.0, 5.0, 8.0]]], [[[5.0, 1.0, 3.0], [4.0, 7.0, 8.0], [7.0, 6.0, 6.0]], [[2.0, 2.0, 8.0], [8.0, 3.0, 5.0], [3.0, 3.0, 3.0]], 
[[5.0, 6.0, 7.0], [4.0, 1.0, 4.0], [8.0, 8.0, 2.0]]], [[[8.0, 2.0, 4.0], [7.0, 7.0, 1.0], [3.0, 5.0, 1.0]], [[7.0, 6.0, 7.0], 
[6.0, 7.0, 1.0], [5.0, 6.0, 7.0]], [[5.0, 6.0, 7.0], [7.0, 3.0, 5.0], [2.0, 8.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 5.0, 2.0]], [[8.0, 2.0, 2.0]], [[3.0, 2.0, 4.0]]], [[[4.0, 7.0, 4.0]], [[4.0, 1.0, 
4.0]], [[1.0, 1.0, 7.0]]], [[[5.0, 3.0, 1.0]], [[4.0, 2.0, 1.0]], [[7.0, 2.0, 2.0]]], [[[5.0, 1.0, 6.0]], [[7.0, 8.0, 7.0]], 
[[2.0, 2.0, 6.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 2.0, 8.0]], [[2.0, 6.0, 6.0]], [[2.0, 3.0, 5.0]]], [[[3.0, 4.0, 4.0]], 
[[3.0, 2.0, 8.0]], [[1.0, 7.0, 5.0]]], [[[5.0, 1.0, 5.0]], [[7.0, 6.0, 2.0]], [[7.0, 3.0, 1.0]]], [[[5.0, 4.0, 5.0]], [[1.0, 
7.0, 2.0]], [[4.0, 4.0, 5.0]]]])
      arg[:,:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 2.0], [6.0, 5.0, 2.0], [1.0, 3.0, 2.0]], [[6.0, 1.0, 6.0], [8.0, 2.0, 2.0], [8.0, 
6.0, 4.0]], [[7.0, 1.0, 4.0], [3.0, 2.0, 4.0], [4.0, 2.0, 3.0]]], [[[8.0, 7.0, 2.0], [4.0, 7.0, 4.0], [4.0, 6.0, 1.0]], [[7.0, 
8.0, 1.0], [4.0, 1.0, 4.0], [1.0, 1.0, 3.0]], [[2.0, 5.0, 4.0], [1.0, 1.0, 7.0], [8.0, 6.0, 2.0]]], [[[2.0, 4.0, 5.0], [5.0, 
3.0, 1.0], [8.0, 7.0, 6.0]], [[3.0, 7.0, 5.0], [4.0, 2.0, 1.0], [2.0, 3.0, 3.0]], [[1.0, 3.0, 4.0], [7.0, 2.0, 2.0], [8.0, 5.0, 
5.0]]], [[[5.0, 5.0, 6.0], [5.0, 1.0, 6.0], [6.0, 2.0, 1.0]], [[2.0, 7.0, 8.0], [7.0, 8.0, 7.0], [2.0, 8.0, 3.0]], [[5.0, 7.0, 
7.0], [2.0, 2.0, 6.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [7.0, 2.0, 8.0], [1.0, 2.0, 8.0]], 
[[7.0, 4.0, 4.0], [2.0, 6.0, 6.0], [8.0, 1.0, 4.0]], [[7.0, 4.0, 4.0], [2.0, 3.0, 5.0], [8.0, 7.0, 5.0]]], [[[4.0, 2.0, 4.0], 
[3.0, 4.0, 4.0], [8.0, 8.0, 6.0]], [[8.0, 8.0, 7.0], [3.0, 2.0, 8.0], [8.0, 1.0, 2.0]], [[7.0, 5.0, 4.0], [1.0, 7.0, 5.0], 
[3.0, 5.0, 8.0]]], [[[5.0, 1.0, 3.0], [5.0, 1.0, 5.0], [7.0, 6.0, 6.0]], [[2.0, 2.0, 8.0], [7.0, 6.0, 2.0], [3.0, 3.0, 3.0]], 
[[5.0, 6.0, 7.0], [7.0, 3.0, 1.0], [8.0, 8.0, 2.0]]], [[[8.0, 2.0, 4.0], [5.0, 4.0, 5.0], [3.0, 5.0, 1.0]], [[7.0, 6.0, 7.0], 
[1.0, 7.0, 2.0], [5.0, 6.0, 7.0]], [[5.0, 6.0, 7.0], [4.0, 4.0, 5.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 7.0, 1.0], [8.0, 6.0, 5.0], [3.0, 4.0, 3.0]], [[3.0, 7.0, 8.0], [3.0, 5.0, 1.0], [1.0, 
5.0, 7.0]], [[3.0, 4.0, 4.0], [4.0, 3.0, 4.0], [2.0, 6.0, 5.0]]], [[[7.0, 7.0, 3.0], [5.0, 1.0, 1.0], [1.0, 7.0, 8.0]], [[3.0, 
4.0, 7.0], [8.0, 3.0, 8.0], [2.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [4.0, 8.0, 2.0], [6.0, 3.0, 5.0]]], [[[3.0, 6.0, 3.0], [1.0, 
3.0, 2.0], [3.0, 6.0, 7.0]], [[8.0, 8.0, 2.0], [3.0, 1.0, 4.0], [1.0, 3.0, 1.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 2.0], [1.0, 7.0, 
1.0]]], [[[4.0, 3.0, 7.0], [7.0, 5.0, 4.0], [6.0, 8.0, 2.0]], [[8.0, 4.0, 4.0], [1.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[4.0, 3.0, 
7.0], [7.0, 6.0, 7.0], [7.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 6.0], [6.0, 6.0, 8.0], [8.0, 3.0, 2.0]], 
[[4.0, 4.0, 3.0], [5.0, 3.0, 3.0], [5.0, 7.0, 7.0]], [[6.0, 5.0, 8.0], [2.0, 3.0, 8.0], [8.0, 5.0, 7.0]]], [[[5.0, 1.0, 6.0], 
[2.0, 2.0, 7.0], [5.0, 8.0, 8.0]], [[6.0, 3.0, 1.0], [6.0, 4.0, 4.0], [3.0, 3.0, 2.0]], [[1.0, 5.0, 1.0], [8.0, 2.0, 2.0], 
[3.0, 3.0, 3.0]]], [[[8.0, 8.0, 5.0], [1.0, 6.0, 7.0], [7.0, 2.0, 5.0]], [[2.0, 5.0, 4.0], [5.0, 8.0, 4.0], [3.0, 6.0, 4.0]], 
[[2.0, 4.0, 1.0], [6.0, 1.0, 7.0], [6.0, 3.0, 4.0]]], [[[7.0, 3.0, 1.0], [1.0, 4.0, 2.0], [3.0, 7.0, 3.0]], [[2.0, 5.0, 4.0], 
[1.0, 5.0, 4.0], [3.0, 2.0, 3.0]], [[4.0, 2.0, 7.0], [3.0, 7.0, 2.0], [1.0, 7.0, 1.0]]]])
      val=numpy.array([[[6.0, 8.0, 1.0], [7.0, 7.0, 7.0]]])
      arg[2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 7.0, 1.0], [8.0, 6.0, 5.0], [3.0, 4.0, 3.0]], [[3.0, 7.0, 8.0], [3.0, 5.0, 1.0], [1.0, 
5.0, 7.0]], [[3.0, 4.0, 4.0], [4.0, 3.0, 4.0], [2.0, 6.0, 5.0]]], [[[7.0, 7.0, 3.0], [5.0, 1.0, 1.0], [1.0, 7.0, 8.0]], [[3.0, 
4.0, 7.0], [8.0, 3.0, 8.0], [2.0, 3.0, 4.0]], [[6.0, 1.0, 8.0], [4.0, 8.0, 2.0], [6.0, 3.0, 5.0]]], [[[3.0, 6.0, 3.0], [1.0, 
3.0, 2.0], [3.0, 6.0, 7.0]], [[6.0, 8.0, 1.0], [7.0, 7.0, 7.0], [1.0, 3.0, 1.0]], [[7.0, 8.0, 8.0], [1.0, 3.0, 2.0], [1.0, 7.0, 
1.0]]], [[[4.0, 3.0, 7.0], [7.0, 5.0, 4.0], [6.0, 8.0, 2.0]], [[8.0, 4.0, 4.0], [1.0, 1.0, 3.0], [1.0, 2.0, 4.0]], [[4.0, 3.0, 
7.0], [7.0, 6.0, 7.0], [7.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 6.0], [6.0, 6.0, 8.0], [8.0, 3.0, 2.0]], 
[[4.0, 4.0, 3.0], [5.0, 3.0, 3.0], [5.0, 7.0, 7.0]], [[6.0, 5.0, 8.0], [2.0, 3.0, 8.0], [8.0, 5.0, 7.0]]], [[[5.0, 1.0, 6.0], 
[2.0, 2.0, 7.0], [5.0, 8.0, 8.0]], [[6.0, 3.0, 1.0], [6.0, 4.0, 4.0], [3.0, 3.0, 2.0]], [[1.0, 5.0, 1.0], [8.0, 2.0, 2.0], 
[3.0, 3.0, 3.0]]], [[[8.0, 8.0, 5.0], [1.0, 6.0, 7.0], [7.0, 2.0, 5.0]], [[6.0, 8.0, 1.0], [7.0, 7.0, 7.0], [3.0, 6.0, 4.0]], 
[[2.0, 4.0, 1.0], [6.0, 1.0, 7.0], [6.0, 3.0, 4.0]]], [[[7.0, 3.0, 1.0], [1.0, 4.0, 2.0], [3.0, 7.0, 3.0]], [[2.0, 5.0, 4.0], 
[1.0, 5.0, 4.0], [3.0, 2.0, 3.0]], [[4.0, 2.0, 7.0], [3.0, 7.0, 2.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 2.0], [2.0, 2.0, 1.0], [1.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [4.0, 6.0, 1.0], [3.0, 
1.0, 3.0]], [[3.0, 6.0, 5.0], [2.0, 8.0, 8.0], [4.0, 3.0, 3.0]]], [[[3.0, 4.0, 4.0], [8.0, 6.0, 7.0], [8.0, 7.0, 3.0]], [[6.0, 
8.0, 6.0], [5.0, 8.0, 7.0], [7.0, 1.0, 2.0]], [[3.0, 1.0, 2.0], [3.0, 4.0, 7.0], [4.0, 5.0, 8.0]]], [[[4.0, 1.0, 6.0], [4.0, 
8.0, 6.0], [8.0, 3.0, 8.0]], [[2.0, 2.0, 2.0], [7.0, 5.0, 5.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 1.0], [8.0, 6.0, 6.0], [8.0, 6.0, 
4.0]]], [[[1.0, 8.0, 5.0], [4.0, 5.0, 4.0], [2.0, 3.0, 3.0]], [[7.0, 3.0, 5.0], [6.0, 1.0, 2.0], [1.0, 4.0, 2.0]], [[3.0, 8.0, 
5.0], [4.0, 1.0, 5.0], [8.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 5.0], [5.0, 2.0, 3.0], [8.0, 1.0, 8.0]], 
[[8.0, 6.0, 3.0], [1.0, 5.0, 7.0], [8.0, 2.0, 4.0]], [[3.0, 8.0, 8.0], [3.0, 5.0, 1.0], [5.0, 7.0, 8.0]]], [[[2.0, 8.0, 7.0], 
[8.0, 1.0, 4.0], [4.0, 2.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 8.0, 7.0], [3.0, 5.0, 2.0]], [[4.0, 7.0, 8.0], [1.0, 5.0, 6.0], 
[4.0, 2.0, 8.0]]], [[[2.0, 6.0, 3.0], [4.0, 6.0, 4.0], [8.0, 5.0, 1.0]], [[2.0, 5.0, 5.0], [5.0, 3.0, 5.0], [4.0, 3.0, 6.0]], 
[[1.0, 3.0, 1.0], [1.0, 5.0, 6.0], [7.0, 8.0, 7.0]]], [[[5.0, 6.0, 8.0], [3.0, 3.0, 7.0], [4.0, 4.0, 3.0]], [[3.0, 6.0, 6.0], 
[5.0, 6.0, 6.0], [6.0, 8.0, 7.0]], [[7.0, 6.0, 4.0], [7.0, 5.0, 2.0], [3.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[8.0, 2.0, 1.0], [1.0, 2.0, 5.0]]]),self.functionspace)
      arg[2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 2.0], [2.0, 2.0, 1.0], [1.0, 8.0, 1.0]], [[6.0, 8.0, 5.0], [4.0, 6.0, 1.0], [3.0, 
1.0, 3.0]], [[3.0, 6.0, 5.0], [2.0, 8.0, 8.0], [4.0, 3.0, 3.0]]], [[[3.0, 4.0, 4.0], [8.0, 6.0, 7.0], [8.0, 7.0, 3.0]], [[6.0, 
8.0, 6.0], [5.0, 8.0, 7.0], [7.0, 1.0, 2.0]], [[3.0, 1.0, 2.0], [3.0, 4.0, 7.0], [4.0, 5.0, 8.0]]], [[[4.0, 1.0, 6.0], [4.0, 
8.0, 6.0], [8.0, 3.0, 8.0]], [[8.0, 2.0, 1.0], [1.0, 2.0, 5.0], [2.0, 7.0, 6.0]], [[1.0, 3.0, 1.0], [8.0, 6.0, 6.0], [8.0, 6.0, 
4.0]]], [[[1.0, 8.0, 5.0], [4.0, 5.0, 4.0], [2.0, 3.0, 3.0]], [[7.0, 3.0, 5.0], [6.0, 1.0, 2.0], [1.0, 4.0, 2.0]], [[3.0, 8.0, 
5.0], [4.0, 1.0, 5.0], [8.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [5.0, 2.0, 3.0], [8.0, 1.0, 8.0]], 
[[8.0, 6.0, 3.0], [1.0, 5.0, 7.0], [8.0, 2.0, 4.0]], [[3.0, 8.0, 8.0], [3.0, 5.0, 1.0], [5.0, 7.0, 8.0]]], [[[2.0, 8.0, 7.0], 
[8.0, 1.0, 4.0], [4.0, 2.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 8.0, 7.0], [3.0, 5.0, 2.0]], [[4.0, 7.0, 8.0], [1.0, 5.0, 6.0], 
[4.0, 2.0, 8.0]]], [[[2.0, 6.0, 3.0], [4.0, 6.0, 4.0], [8.0, 5.0, 1.0]], [[8.0, 2.0, 1.0], [1.0, 2.0, 5.0], [4.0, 3.0, 6.0]], 
[[1.0, 3.0, 1.0], [1.0, 5.0, 6.0], [7.0, 8.0, 7.0]]], [[[5.0, 6.0, 8.0], [3.0, 3.0, 7.0], [4.0, 4.0, 3.0]], [[3.0, 6.0, 6.0], 
[5.0, 6.0, 6.0], [6.0, 8.0, 7.0]], [[7.0, 6.0, 4.0], [7.0, 5.0, 2.0], [3.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 3.0], [3.0, 8.0, 2.0], [8.0, 5.0, 5.0]], [[6.0, 8.0, 6.0], [3.0, 2.0, 3.0], [1.0, 
4.0, 2.0]], [[1.0, 4.0, 5.0], [4.0, 4.0, 6.0], [1.0, 1.0, 5.0]]], [[[7.0, 2.0, 5.0], [6.0, 8.0, 7.0], [5.0, 1.0, 3.0]], [[3.0, 
3.0, 1.0], [7.0, 2.0, 8.0], [7.0, 2.0, 1.0]], [[6.0, 4.0, 1.0], [4.0, 3.0, 1.0], [8.0, 6.0, 7.0]]], [[[1.0, 5.0, 3.0], [1.0, 
6.0, 5.0], [4.0, 2.0, 2.0]], [[1.0, 2.0, 4.0], [7.0, 3.0, 7.0], [3.0, 6.0, 4.0]], [[1.0, 1.0, 6.0], [4.0, 7.0, 7.0], [4.0, 8.0, 
1.0]]], [[[5.0, 3.0, 5.0], [4.0, 4.0, 1.0], [5.0, 4.0, 3.0]], [[3.0, 5.0, 7.0], [3.0, 2.0, 2.0], [8.0, 2.0, 1.0]], [[6.0, 2.0, 
1.0], [2.0, 6.0, 5.0], [2.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 1.0], [7.0, 2.0, 3.0], [1.0, 5.0, 8.0]], 
[[6.0, 6.0, 8.0], [1.0, 4.0, 7.0], [2.0, 3.0, 2.0]], [[5.0, 8.0, 8.0], [7.0, 8.0, 5.0], [5.0, 5.0, 5.0]]], [[[4.0, 6.0, 2.0], 
[8.0, 1.0, 6.0], [2.0, 5.0, 6.0]], [[5.0, 1.0, 5.0], [6.0, 6.0, 3.0], [1.0, 6.0, 6.0]], [[3.0, 6.0, 8.0], [4.0, 3.0, 8.0], 
[8.0, 1.0, 8.0]]], [[[5.0, 1.0, 2.0], [3.0, 7.0, 8.0], [8.0, 6.0, 8.0]], [[4.0, 7.0, 7.0], [6.0, 5.0, 8.0], [8.0, 8.0, 6.0]], 
[[4.0, 7.0, 1.0], [2.0, 2.0, 4.0], [1.0, 4.0, 8.0]]], [[[8.0, 6.0, 5.0], [2.0, 2.0, 2.0], [6.0, 2.0, 3.0]], [[5.0, 1.0, 4.0], 
[7.0, 4.0, 1.0], [5.0, 7.0, 1.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 7.0], [5.0, 7.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 7.0, 6.0], [1.0, 8.0, 8.0]]])+(1.-msk_val)*numpy.array([[[8.0, 3.0, 3.0], [8.0, 2.0, 
8.0]]])
      arg[2,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 3.0], [3.0, 8.0, 2.0], [8.0, 5.0, 5.0]], [[6.0, 8.0, 6.0], [3.0, 2.0, 3.0], [1.0, 
4.0, 2.0]], [[1.0, 4.0, 5.0], [4.0, 4.0, 6.0], [1.0, 1.0, 5.0]]], [[[7.0, 2.0, 5.0], [6.0, 8.0, 7.0], [5.0, 1.0, 3.0]], [[3.0, 
3.0, 1.0], [7.0, 2.0, 8.0], [7.0, 2.0, 1.0]], [[6.0, 4.0, 1.0], [4.0, 3.0, 1.0], [8.0, 6.0, 7.0]]], [[[1.0, 5.0, 3.0], [1.0, 
6.0, 5.0], [4.0, 2.0, 2.0]], [[4.0, 7.0, 6.0], [1.0, 8.0, 8.0], [3.0, 6.0, 4.0]], [[1.0, 1.0, 6.0], [4.0, 7.0, 7.0], [4.0, 8.0, 
1.0]]], [[[5.0, 3.0, 5.0], [4.0, 4.0, 1.0], [5.0, 4.0, 3.0]], [[3.0, 5.0, 7.0], [3.0, 2.0, 2.0], [8.0, 2.0, 1.0]], [[6.0, 2.0, 
1.0], [2.0, 6.0, 5.0], [2.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 1.0], [7.0, 2.0, 3.0], [1.0, 5.0, 8.0]], 
[[6.0, 6.0, 8.0], [1.0, 4.0, 7.0], [2.0, 3.0, 2.0]], [[5.0, 8.0, 8.0], [7.0, 8.0, 5.0], [5.0, 5.0, 5.0]]], [[[4.0, 6.0, 2.0], 
[8.0, 1.0, 6.0], [2.0, 5.0, 6.0]], [[5.0, 1.0, 5.0], [6.0, 6.0, 3.0], [1.0, 6.0, 6.0]], [[3.0, 6.0, 8.0], [4.0, 3.0, 8.0], 
[8.0, 1.0, 8.0]]], [[[5.0, 1.0, 2.0], [3.0, 7.0, 8.0], [8.0, 6.0, 8.0]], [[8.0, 3.0, 3.0], [8.0, 2.0, 8.0], [8.0, 8.0, 6.0]], 
[[4.0, 7.0, 1.0], [2.0, 2.0, 4.0], [1.0, 4.0, 8.0]]], [[[8.0, 6.0, 5.0], [2.0, 2.0, 2.0], [6.0, 2.0, 3.0]], [[5.0, 1.0, 4.0], 
[7.0, 4.0, 1.0], [5.0, 7.0, 1.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 7.0], [5.0, 7.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 1.0], [7.0, 1.0, 6.0], [2.0, 3.0, 1.0]], [[8.0, 8.0, 1.0], [7.0, 3.0, 2.0], [1.0, 
2.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 6.0, 3.0], [7.0, 8.0, 7.0]]], [[[6.0, 1.0, 4.0], [4.0, 3.0, 1.0], [1.0, 8.0, 3.0]], [[2.0, 
3.0, 1.0], [5.0, 3.0, 1.0], [2.0, 2.0, 5.0]], [[4.0, 5.0, 5.0], [6.0, 2.0, 6.0], [4.0, 3.0, 3.0]]], [[[1.0, 3.0, 7.0], [2.0, 
1.0, 8.0], [4.0, 3.0, 2.0]], [[1.0, 8.0, 5.0], [6.0, 1.0, 6.0], [1.0, 6.0, 8.0]], [[2.0, 3.0, 8.0], [6.0, 2.0, 1.0], [3.0, 8.0, 
6.0]]], [[[3.0, 3.0, 1.0], [2.0, 5.0, 8.0], [5.0, 1.0, 8.0]], [[7.0, 5.0, 6.0], [4.0, 8.0, 2.0], [5.0, 5.0, 4.0]], [[6.0, 4.0, 
4.0], [8.0, 6.0, 3.0], [2.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 5.0], [4.0, 7.0, 7.0], [6.0, 8.0, 1.0]], 
[[1.0, 3.0, 2.0], [6.0, 6.0, 2.0], [5.0, 8.0, 6.0]], [[5.0, 3.0, 1.0], [3.0, 7.0, 2.0], [8.0, 7.0, 6.0]]], [[[3.0, 4.0, 1.0], 
[6.0, 5.0, 2.0], [3.0, 1.0, 8.0]], [[3.0, 6.0, 8.0], [2.0, 6.0, 2.0], [5.0, 3.0, 7.0]], [[4.0, 8.0, 8.0], [7.0, 8.0, 1.0], 
[1.0, 7.0, 5.0]]], [[[4.0, 7.0, 5.0], [2.0, 4.0, 4.0], [1.0, 6.0, 7.0]], [[1.0, 7.0, 8.0], [5.0, 7.0, 2.0], [8.0, 4.0, 5.0]], 
[[6.0, 1.0, 5.0], [7.0, 8.0, 7.0], [5.0, 7.0, 5.0]]], [[[5.0, 8.0, 2.0], [7.0, 2.0, 8.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 6.0], 
[5.0, 6.0, 8.0], [1.0, 4.0, 1.0]], [[5.0, 5.0, 7.0], [6.0, 4.0, 2.0], [6.0, 8.0, 3.0]]]])
      val=numpy.array([[[[3.0, 6.0, 1.0]]], [[[4.0, 7.0, 1.0]]]])
      arg[1:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 1.0], [7.0, 1.0, 6.0], [2.0, 3.0, 1.0]], [[8.0, 8.0, 1.0], [7.0, 3.0, 2.0], [1.0, 
2.0, 8.0]], [[3.0, 7.0, 8.0], [8.0, 6.0, 3.0], [7.0, 8.0, 7.0]]], [[[6.0, 1.0, 4.0], [4.0, 3.0, 1.0], [1.0, 8.0, 3.0]], [[2.0, 
3.0, 1.0], [5.0, 3.0, 1.0], [3.0, 6.0, 1.0]], [[4.0, 5.0, 5.0], [6.0, 2.0, 6.0], [4.0, 3.0, 3.0]]], [[[1.0, 3.0, 7.0], [2.0, 
1.0, 8.0], [4.0, 3.0, 2.0]], [[1.0, 8.0, 5.0], [6.0, 1.0, 6.0], [4.0, 7.0, 1.0]], [[2.0, 3.0, 8.0], [6.0, 2.0, 1.0], [3.0, 8.0, 
6.0]]], [[[3.0, 3.0, 1.0], [2.0, 5.0, 8.0], [5.0, 1.0, 8.0]], [[7.0, 5.0, 6.0], [4.0, 8.0, 2.0], [5.0, 5.0, 4.0]], [[6.0, 4.0, 
4.0], [8.0, 6.0, 3.0], [2.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 5.0], [4.0, 7.0, 7.0], [6.0, 8.0, 1.0]], 
[[1.0, 3.0, 2.0], [6.0, 6.0, 2.0], [5.0, 8.0, 6.0]], [[5.0, 3.0, 1.0], [3.0, 7.0, 2.0], [8.0, 7.0, 6.0]]], [[[3.0, 4.0, 1.0], 
[6.0, 5.0, 2.0], [3.0, 1.0, 8.0]], [[3.0, 6.0, 8.0], [2.0, 6.0, 2.0], [3.0, 6.0, 1.0]], [[4.0, 8.0, 8.0], [7.0, 8.0, 1.0], 
[1.0, 7.0, 5.0]]], [[[4.0, 7.0, 5.0], [2.0, 4.0, 4.0], [1.0, 6.0, 7.0]], [[1.0, 7.0, 8.0], [5.0, 7.0, 2.0], [4.0, 7.0, 1.0]], 
[[6.0, 1.0, 5.0], [7.0, 8.0, 7.0], [5.0, 7.0, 5.0]]], [[[5.0, 8.0, 2.0], [7.0, 2.0, 8.0], [1.0, 5.0, 8.0]], [[4.0, 7.0, 6.0], 
[5.0, 6.0, 8.0], [1.0, 4.0, 1.0]], [[5.0, 5.0, 7.0], [6.0, 4.0, 2.0], [6.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 4.0], [2.0, 6.0, 7.0], [2.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [1.0, 8.0, 4.0], [3.0, 
7.0, 3.0]], [[3.0, 6.0, 4.0], [1.0, 7.0, 5.0], [8.0, 8.0, 1.0]]], [[[7.0, 4.0, 2.0], [4.0, 1.0, 2.0], [3.0, 5.0, 7.0]], [[8.0, 
7.0, 7.0], [6.0, 3.0, 8.0], [1.0, 4.0, 5.0]], [[6.0, 4.0, 6.0], [1.0, 6.0, 3.0], [6.0, 8.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 
5.0, 6.0], [7.0, 7.0, 1.0]], [[7.0, 4.0, 1.0], [2.0, 3.0, 8.0], [2.0, 1.0, 6.0]], [[3.0, 8.0, 1.0], [4.0, 1.0, 1.0], [6.0, 8.0, 
4.0]]], [[[2.0, 5.0, 2.0], [8.0, 3.0, 2.0], [2.0, 4.0, 4.0]], [[2.0, 8.0, 5.0], [8.0, 8.0, 1.0], [4.0, 3.0, 3.0]], [[1.0, 3.0, 
8.0], [6.0, 5.0, 6.0], [4.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 3.0], [1.0, 6.0, 3.0], [7.0, 1.0, 3.0]], 
[[8.0, 7.0, 7.0], [1.0, 2.0, 4.0], [4.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [3.0, 3.0, 2.0], [8.0, 4.0, 5.0]]], [[[7.0, 7.0, 3.0], 
[8.0, 8.0, 6.0], [5.0, 8.0, 5.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 1.0], [5.0, 5.0, 4.0]], [[6.0, 7.0, 7.0], [4.0, 3.0, 3.0], 
[3.0, 3.0, 6.0]]], [[[6.0, 4.0, 5.0], [6.0, 5.0, 4.0], [5.0, 7.0, 1.0]], [[7.0, 5.0, 1.0], [7.0, 3.0, 2.0], [1.0, 3.0, 7.0]], 
[[4.0, 2.0, 1.0], [8.0, 1.0, 5.0], [7.0, 7.0, 7.0]]], [[[8.0, 5.0, 4.0], [1.0, 6.0, 7.0], [5.0, 4.0, 1.0]], [[6.0, 4.0, 4.0], 
[7.0, 6.0, 5.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 5.0], [2.0, 3.0, 5.0], [3.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[[3.0, 3.0, 6.0]]], [[[3.0, 7.0, 4.0]]]]),self.functionspace)
      arg[1:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 4.0], [2.0, 6.0, 7.0], [2.0, 2.0, 8.0]], [[3.0, 2.0, 4.0], [1.0, 8.0, 4.0], [3.0, 
7.0, 3.0]], [[3.0, 6.0, 4.0], [1.0, 7.0, 5.0], [8.0, 8.0, 1.0]]], [[[7.0, 4.0, 2.0], [4.0, 1.0, 2.0], [3.0, 5.0, 7.0]], [[8.0, 
7.0, 7.0], [6.0, 3.0, 8.0], [3.0, 3.0, 6.0]], [[6.0, 4.0, 6.0], [1.0, 6.0, 3.0], [6.0, 8.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 
5.0, 6.0], [7.0, 7.0, 1.0]], [[7.0, 4.0, 1.0], [2.0, 3.0, 8.0], [3.0, 7.0, 4.0]], [[3.0, 8.0, 1.0], [4.0, 1.0, 1.0], [6.0, 8.0, 
4.0]]], [[[2.0, 5.0, 2.0], [8.0, 3.0, 2.0], [2.0, 4.0, 4.0]], [[2.0, 8.0, 5.0], [8.0, 8.0, 1.0], [4.0, 3.0, 3.0]], [[1.0, 3.0, 
8.0], [6.0, 5.0, 6.0], [4.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [1.0, 6.0, 3.0], [7.0, 1.0, 3.0]], 
[[8.0, 7.0, 7.0], [1.0, 2.0, 4.0], [4.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [3.0, 3.0, 2.0], [8.0, 4.0, 5.0]]], [[[7.0, 7.0, 3.0], 
[8.0, 8.0, 6.0], [5.0, 8.0, 5.0]], [[8.0, 6.0, 8.0], [6.0, 4.0, 1.0], [3.0, 3.0, 6.0]], [[6.0, 7.0, 7.0], [4.0, 3.0, 3.0], 
[3.0, 3.0, 6.0]]], [[[6.0, 4.0, 5.0], [6.0, 5.0, 4.0], [5.0, 7.0, 1.0]], [[7.0, 5.0, 1.0], [7.0, 3.0, 2.0], [3.0, 7.0, 4.0]], 
[[4.0, 2.0, 1.0], [8.0, 1.0, 5.0], [7.0, 7.0, 7.0]]], [[[8.0, 5.0, 4.0], [1.0, 6.0, 7.0], [5.0, 4.0, 1.0]], [[6.0, 4.0, 4.0], 
[7.0, 6.0, 5.0], [7.0, 8.0, 7.0]], [[4.0, 4.0, 5.0], [2.0, 3.0, 5.0], [3.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 7.0], [1.0, 6.0, 8.0], [3.0, 8.0, 1.0]], [[6.0, 4.0, 7.0], [5.0, 2.0, 6.0], [5.0, 
6.0, 4.0]], [[1.0, 7.0, 4.0], [3.0, 5.0, 7.0], [1.0, 2.0, 4.0]]], [[[2.0, 1.0, 2.0], [5.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[7.0, 
5.0, 8.0], [1.0, 6.0, 8.0], [3.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], [3.0, 7.0, 7.0], [8.0, 1.0, 1.0]]], [[[7.0, 4.0, 4.0], [5.0, 
3.0, 5.0], [4.0, 8.0, 2.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 6.0], [5.0, 4.0, 5.0]], [[6.0, 2.0, 4.0], [8.0, 2.0, 1.0], [3.0, 3.0, 
8.0]]], [[[4.0, 2.0, 5.0], [8.0, 7.0, 8.0], [1.0, 5.0, 8.0]], [[7.0, 3.0, 2.0], [4.0, 6.0, 3.0], [3.0, 3.0, 3.0]], [[7.0, 8.0, 
1.0], [1.0, 1.0, 7.0], [7.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 8.0], [4.0, 5.0, 3.0], [6.0, 1.0, 6.0]], 
[[6.0, 4.0, 2.0], [3.0, 1.0, 8.0], [6.0, 1.0, 2.0]], [[4.0, 3.0, 6.0], [7.0, 4.0, 2.0], [7.0, 2.0, 2.0]]], [[[4.0, 1.0, 1.0], 
[5.0, 2.0, 2.0], [7.0, 6.0, 4.0]], [[3.0, 8.0, 3.0], [3.0, 2.0, 7.0], [7.0, 2.0, 5.0]], [[1.0, 8.0, 3.0], [1.0, 1.0, 4.0], 
[3.0, 5.0, 1.0]]], [[[6.0, 1.0, 8.0], [4.0, 5.0, 5.0], [3.0, 4.0, 5.0]], [[8.0, 3.0, 7.0], [7.0, 5.0, 2.0], [1.0, 4.0, 4.0]], 
[[1.0, 1.0, 1.0], [6.0, 6.0, 6.0], [8.0, 2.0, 3.0]]], [[[1.0, 8.0, 1.0], [1.0, 5.0, 2.0], [8.0, 8.0, 4.0]], [[6.0, 5.0, 8.0], 
[1.0, 1.0, 1.0], [8.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [6.0, 4.0, 3.0], [1.0, 7.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 4.0, 5.0]]], [[[5.0, 2.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 7.0, 1.0]]], 
[[[6.0, 1.0, 4.0]]]])
      arg[1:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 7.0], [1.0, 6.0, 8.0], [3.0, 8.0, 1.0]], [[6.0, 4.0, 7.0], [5.0, 2.0, 6.0], [5.0, 
6.0, 4.0]], [[1.0, 7.0, 4.0], [3.0, 5.0, 7.0], [1.0, 2.0, 4.0]]], [[[2.0, 1.0, 2.0], [5.0, 2.0, 1.0], [5.0, 2.0, 1.0]], [[7.0, 
5.0, 8.0], [1.0, 6.0, 8.0], [2.0, 4.0, 5.0]], [[7.0, 5.0, 6.0], [3.0, 7.0, 7.0], [8.0, 1.0, 1.0]]], [[[7.0, 4.0, 4.0], [5.0, 
3.0, 5.0], [4.0, 8.0, 2.0]], [[1.0, 8.0, 4.0], [5.0, 1.0, 6.0], [5.0, 2.0, 2.0]], [[6.0, 2.0, 4.0], [8.0, 2.0, 1.0], [3.0, 3.0, 
8.0]]], [[[4.0, 2.0, 5.0], [8.0, 7.0, 8.0], [1.0, 5.0, 8.0]], [[7.0, 3.0, 2.0], [4.0, 6.0, 3.0], [3.0, 3.0, 3.0]], [[7.0, 8.0, 
1.0], [1.0, 1.0, 7.0], [7.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 8.0], [4.0, 5.0, 3.0], [6.0, 1.0, 6.0]], 
[[6.0, 4.0, 2.0], [3.0, 1.0, 8.0], [6.0, 1.0, 2.0]], [[4.0, 3.0, 6.0], [7.0, 4.0, 2.0], [7.0, 2.0, 2.0]]], [[[4.0, 1.0, 1.0], 
[5.0, 2.0, 2.0], [7.0, 6.0, 4.0]], [[3.0, 8.0, 3.0], [3.0, 2.0, 7.0], [8.0, 7.0, 1.0]], [[1.0, 8.0, 3.0], [1.0, 1.0, 4.0], 
[3.0, 5.0, 1.0]]], [[[6.0, 1.0, 8.0], [4.0, 5.0, 5.0], [3.0, 4.0, 5.0]], [[8.0, 3.0, 7.0], [7.0, 5.0, 2.0], [6.0, 1.0, 4.0]], 
[[1.0, 1.0, 1.0], [6.0, 6.0, 6.0], [8.0, 2.0, 3.0]]], [[[1.0, 8.0, 1.0], [1.0, 5.0, 2.0], [8.0, 8.0, 4.0]], [[6.0, 5.0, 8.0], 
[1.0, 1.0, 1.0], [8.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [6.0, 4.0, 3.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 4.0], [1.0, 2.0, 8.0], [5.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [5.0, 2.0, 1.0], [4.0, 
7.0, 8.0]], [[4.0, 1.0, 2.0], [6.0, 1.0, 3.0], [5.0, 2.0, 3.0]]], [[[6.0, 8.0, 4.0], [7.0, 5.0, 2.0], [8.0, 1.0, 2.0]], [[4.0, 
1.0, 3.0], [1.0, 5.0, 4.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 1.0], [8.0, 2.0, 4.0], [4.0, 4.0, 2.0]]], [[[6.0, 8.0, 4.0], [7.0, 
1.0, 3.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], [4.0, 6.0, 8.0], [4.0, 3.0, 8.0]], [[3.0, 3.0, 4.0], [7.0, 5.0, 8.0], [4.0, 5.0, 
7.0]]], [[[3.0, 8.0, 5.0], [8.0, 1.0, 8.0], [6.0, 5.0, 3.0]], [[6.0, 5.0, 8.0], [5.0, 3.0, 5.0], [3.0, 1.0, 3.0]], [[6.0, 2.0, 
6.0], [5.0, 4.0, 4.0], [4.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 7.0], [5.0, 3.0, 8.0], [2.0, 3.0, 3.0]], 
[[7.0, 7.0, 7.0], [2.0, 8.0, 2.0], [3.0, 1.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 5.0, 7.0], [8.0, 1.0, 7.0]]], [[[7.0, 8.0, 3.0], 
[5.0, 4.0, 7.0], [8.0, 5.0, 8.0]], [[7.0, 1.0, 7.0], [6.0, 5.0, 1.0], [2.0, 4.0, 7.0]], [[1.0, 7.0, 5.0], [5.0, 8.0, 2.0], 
[8.0, 7.0, 5.0]]], [[[5.0, 8.0, 1.0], [4.0, 6.0, 7.0], [5.0, 3.0, 5.0]], [[7.0, 8.0, 6.0], [8.0, 4.0, 1.0], [1.0, 5.0, 2.0]], 
[[2.0, 6.0, 6.0], [4.0, 7.0, 5.0], [3.0, 1.0, 8.0]]], [[[6.0, 8.0, 6.0], [6.0, 4.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 8.0, 6.0], 
[3.0, 6.0, 3.0], [5.0, 7.0, 7.0]], [[5.0, 7.0, 2.0], [6.0, 7.0, 7.0], [2.0, 5.0, 5.0]]]])
      val=numpy.array([[[3.0, 4.0, 3.0]], [[7.0, 5.0, 2.0]], [[1.0, 8.0, 8.0]], [[7.0, 4.0, 2.0]]])
      arg[:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 4.0], [1.0, 2.0, 8.0], [5.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 4.0, 3.0], [4.0, 
7.0, 8.0]], [[4.0, 1.0, 2.0], [6.0, 1.0, 3.0], [5.0, 2.0, 3.0]]], [[[6.0, 8.0, 4.0], [7.0, 5.0, 2.0], [8.0, 1.0, 2.0]], [[4.0, 
1.0, 3.0], [7.0, 5.0, 2.0], [4.0, 4.0, 1.0]], [[8.0, 4.0, 1.0], [8.0, 2.0, 4.0], [4.0, 4.0, 2.0]]], [[[6.0, 8.0, 4.0], [7.0, 
1.0, 3.0], [4.0, 6.0, 8.0]], [[3.0, 7.0, 1.0], [1.0, 8.0, 8.0], [4.0, 3.0, 8.0]], [[3.0, 3.0, 4.0], [7.0, 5.0, 8.0], [4.0, 5.0, 
7.0]]], [[[3.0, 8.0, 5.0], [8.0, 1.0, 8.0], [6.0, 5.0, 3.0]], [[6.0, 5.0, 8.0], [7.0, 4.0, 2.0], [3.0, 1.0, 3.0]], [[6.0, 2.0, 
6.0], [5.0, 4.0, 4.0], [4.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 7.0], [5.0, 3.0, 8.0], [2.0, 3.0, 3.0]], 
[[7.0, 7.0, 7.0], [3.0, 4.0, 3.0], [3.0, 1.0, 6.0]], [[5.0, 2.0, 6.0], [6.0, 5.0, 7.0], [8.0, 1.0, 7.0]]], [[[7.0, 8.0, 3.0], 
[5.0, 4.0, 7.0], [8.0, 5.0, 8.0]], [[7.0, 1.0, 7.0], [7.0, 5.0, 2.0], [2.0, 4.0, 7.0]], [[1.0, 7.0, 5.0], [5.0, 8.0, 2.0], 
[8.0, 7.0, 5.0]]], [[[5.0, 8.0, 1.0], [4.0, 6.0, 7.0], [5.0, 3.0, 5.0]], [[7.0, 8.0, 6.0], [1.0, 8.0, 8.0], [1.0, 5.0, 2.0]], 
[[2.0, 6.0, 6.0], [4.0, 7.0, 5.0], [3.0, 1.0, 8.0]]], [[[6.0, 8.0, 6.0], [6.0, 4.0, 2.0], [2.0, 1.0, 5.0]], [[2.0, 8.0, 6.0], 
[7.0, 4.0, 2.0], [5.0, 7.0, 7.0]], [[5.0, 7.0, 2.0], [6.0, 7.0, 7.0], [2.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 7.0], [2.0, 2.0, 7.0], [7.0, 1.0, 4.0]], [[4.0, 1.0, 1.0], [1.0, 6.0, 7.0], [5.0, 
8.0, 5.0]], [[6.0, 8.0, 2.0], [7.0, 8.0, 5.0], [4.0, 4.0, 6.0]]], [[[1.0, 1.0, 3.0], [1.0, 1.0, 6.0], [5.0, 2.0, 3.0]], [[8.0, 
2.0, 8.0], [1.0, 1.0, 5.0], [8.0, 8.0, 3.0]], [[3.0, 4.0, 7.0], [5.0, 3.0, 8.0], [6.0, 2.0, 4.0]]], [[[7.0, 7.0, 7.0], [2.0, 
8.0, 7.0], [7.0, 2.0, 7.0]], [[8.0, 6.0, 8.0], [1.0, 8.0, 5.0], [7.0, 3.0, 5.0]], [[7.0, 4.0, 8.0], [5.0, 6.0, 8.0], [3.0, 2.0, 
2.0]]], [[[5.0, 7.0, 7.0], [3.0, 1.0, 8.0], [8.0, 4.0, 1.0]], [[3.0, 1.0, 5.0], [6.0, 3.0, 3.0], [8.0, 7.0, 5.0]], [[8.0, 6.0, 
4.0], [3.0, 4.0, 2.0], [4.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 1.0], [1.0, 1.0, 6.0], [7.0, 5.0, 4.0]], 
[[8.0, 4.0, 5.0], [3.0, 8.0, 7.0], [6.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [2.0, 3.0, 8.0], [4.0, 4.0, 2.0]]], [[[3.0, 7.0, 6.0], 
[8.0, 5.0, 6.0], [7.0, 8.0, 1.0]], [[6.0, 4.0, 5.0], [4.0, 6.0, 7.0], [3.0, 5.0, 2.0]], [[7.0, 6.0, 1.0], [7.0, 7.0, 6.0], 
[5.0, 1.0, 7.0]]], [[[1.0, 3.0, 6.0], [6.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[6.0, 5.0, 6.0], [3.0, 1.0, 4.0], [7.0, 7.0, 1.0]], 
[[2.0, 3.0, 6.0], [1.0, 3.0, 1.0], [2.0, 6.0, 8.0]]], [[[3.0, 7.0, 1.0], [4.0, 8.0, 5.0], [8.0, 7.0, 5.0]], [[2.0, 8.0, 2.0], 
[4.0, 2.0, 3.0], [2.0, 5.0, 1.0]], [[5.0, 2.0, 3.0], [1.0, 7.0, 5.0], [3.0, 5.0, 2.0]]]])
      val=Data(numpy.array([[[5.0, 3.0, 4.0]], [[6.0, 4.0, 3.0]], [[2.0, 7.0, 2.0]], [[2.0, 8.0, 6.0]]]),self.functionspace)
      arg[:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 7.0], [2.0, 2.0, 7.0], [7.0, 1.0, 4.0]], [[4.0, 1.0, 1.0], [5.0, 3.0, 4.0], [5.0, 
8.0, 5.0]], [[6.0, 8.0, 2.0], [7.0, 8.0, 5.0], [4.0, 4.0, 6.0]]], [[[1.0, 1.0, 3.0], [1.0, 1.0, 6.0], [5.0, 2.0, 3.0]], [[8.0, 
2.0, 8.0], [6.0, 4.0, 3.0], [8.0, 8.0, 3.0]], [[3.0, 4.0, 7.0], [5.0, 3.0, 8.0], [6.0, 2.0, 4.0]]], [[[7.0, 7.0, 7.0], [2.0, 
8.0, 7.0], [7.0, 2.0, 7.0]], [[8.0, 6.0, 8.0], [2.0, 7.0, 2.0], [7.0, 3.0, 5.0]], [[7.0, 4.0, 8.0], [5.0, 6.0, 8.0], [3.0, 2.0, 
2.0]]], [[[5.0, 7.0, 7.0], [3.0, 1.0, 8.0], [8.0, 4.0, 1.0]], [[3.0, 1.0, 5.0], [2.0, 8.0, 6.0], [8.0, 7.0, 5.0]], [[8.0, 6.0, 
4.0], [3.0, 4.0, 2.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 1.0], [1.0, 1.0, 6.0], [7.0, 5.0, 4.0]], 
[[8.0, 4.0, 5.0], [5.0, 3.0, 4.0], [6.0, 2.0, 3.0]], [[5.0, 4.0, 8.0], [2.0, 3.0, 8.0], [4.0, 4.0, 2.0]]], [[[3.0, 7.0, 6.0], 
[8.0, 5.0, 6.0], [7.0, 8.0, 1.0]], [[6.0, 4.0, 5.0], [6.0, 4.0, 3.0], [3.0, 5.0, 2.0]], [[7.0, 6.0, 1.0], [7.0, 7.0, 6.0], 
[5.0, 1.0, 7.0]]], [[[1.0, 3.0, 6.0], [6.0, 1.0, 4.0], [7.0, 8.0, 2.0]], [[6.0, 5.0, 6.0], [2.0, 7.0, 2.0], [7.0, 7.0, 1.0]], 
[[2.0, 3.0, 6.0], [1.0, 3.0, 1.0], [2.0, 6.0, 8.0]]], [[[3.0, 7.0, 1.0], [4.0, 8.0, 5.0], [8.0, 7.0, 5.0]], [[2.0, 8.0, 2.0], 
[2.0, 8.0, 6.0], [2.0, 5.0, 1.0]], [[5.0, 2.0, 3.0], [1.0, 7.0, 5.0], [3.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 2.0], [3.0, 6.0, 4.0], [8.0, 2.0, 5.0]], [[8.0, 8.0, 7.0], [5.0, 1.0, 7.0], [6.0, 
2.0, 4.0]], [[1.0, 8.0, 5.0], [2.0, 7.0, 7.0], [6.0, 2.0, 3.0]]], [[[4.0, 4.0, 8.0], [7.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[2.0, 
3.0, 4.0], [7.0, 5.0, 5.0], [2.0, 8.0, 6.0]], [[1.0, 5.0, 2.0], [6.0, 8.0, 3.0], [8.0, 5.0, 2.0]]], [[[4.0, 4.0, 2.0], [5.0, 
1.0, 6.0], [3.0, 7.0, 2.0]], [[3.0, 8.0, 2.0], [6.0, 4.0, 1.0], [6.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], [2.0, 4.0, 1.0], [4.0, 3.0, 
1.0]]], [[[5.0, 1.0, 4.0], [5.0, 5.0, 3.0], [2.0, 1.0, 6.0]], [[7.0, 2.0, 4.0], [6.0, 1.0, 6.0], [6.0, 2.0, 6.0]], [[4.0, 8.0, 
5.0], [4.0, 5.0, 2.0], [5.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 8.0], [5.0, 7.0, 1.0], [6.0, 1.0, 6.0]], 
[[1.0, 3.0, 2.0], [2.0, 1.0, 4.0], [3.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], [1.0, 5.0, 5.0], [7.0, 2.0, 8.0]]], [[[8.0, 3.0, 5.0], 
[3.0, 7.0, 3.0], [1.0, 1.0, 2.0]], [[2.0, 7.0, 2.0], [7.0, 4.0, 3.0], [4.0, 8.0, 3.0]], [[5.0, 4.0, 5.0], [8.0, 4.0, 3.0], 
[2.0, 8.0, 7.0]]], [[[6.0, 1.0, 2.0], [4.0, 7.0, 6.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 5.0], [6.0, 4.0, 8.0], [3.0, 7.0, 3.0]], 
[[1.0, 6.0, 5.0], [4.0, 8.0, 8.0], [1.0, 1.0, 5.0]]], [[[3.0, 5.0, 2.0], [1.0, 7.0, 1.0], [4.0, 6.0, 2.0]], [[7.0, 7.0, 7.0], 
[1.0, 2.0, 4.0], [5.0, 3.0, 4.0]], [[3.0, 7.0, 8.0], [6.0, 8.0, 4.0], [8.0, 1.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 3.0, 7.0]], [[2.0, 3.0, 7.0]], [[3.0, 8.0, 8.0]], [[7.0, 8.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[6.0, 7.0, 8.0]], [[2.0, 8.0, 8.0]], [[5.0, 1.0, 1.0]], [[6.0, 1.0, 5.0]]])
      arg[:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 2.0], [3.0, 6.0, 4.0], [8.0, 2.0, 5.0]], [[8.0, 8.0, 7.0], [6.0, 3.0, 7.0], [6.0, 
2.0, 4.0]], [[1.0, 8.0, 5.0], [2.0, 7.0, 7.0], [6.0, 2.0, 3.0]]], [[[4.0, 4.0, 8.0], [7.0, 5.0, 8.0], [2.0, 8.0, 8.0]], [[2.0, 
3.0, 4.0], [2.0, 3.0, 7.0], [2.0, 8.0, 6.0]], [[1.0, 5.0, 2.0], [6.0, 8.0, 3.0], [8.0, 5.0, 2.0]]], [[[4.0, 4.0, 2.0], [5.0, 
1.0, 6.0], [3.0, 7.0, 2.0]], [[3.0, 8.0, 2.0], [3.0, 8.0, 8.0], [6.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], [2.0, 4.0, 1.0], [4.0, 3.0, 
1.0]]], [[[5.0, 1.0, 4.0], [5.0, 5.0, 3.0], [2.0, 1.0, 6.0]], [[7.0, 2.0, 4.0], [7.0, 8.0, 6.0], [6.0, 2.0, 6.0]], [[4.0, 8.0, 
5.0], [4.0, 5.0, 2.0], [5.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 8.0], [5.0, 7.0, 1.0], [6.0, 1.0, 6.0]], 
[[1.0, 3.0, 2.0], [6.0, 7.0, 8.0], [3.0, 2.0, 7.0]], [[3.0, 4.0, 5.0], [1.0, 5.0, 5.0], [7.0, 2.0, 8.0]]], [[[8.0, 3.0, 5.0], 
[3.0, 7.0, 3.0], [1.0, 1.0, 2.0]], [[2.0, 7.0, 2.0], [2.0, 8.0, 8.0], [4.0, 8.0, 3.0]], [[5.0, 4.0, 5.0], [8.0, 4.0, 3.0], 
[2.0, 8.0, 7.0]]], [[[6.0, 1.0, 2.0], [4.0, 7.0, 6.0], [5.0, 6.0, 8.0]], [[1.0, 8.0, 5.0], [5.0, 1.0, 1.0], [3.0, 7.0, 3.0]], 
[[1.0, 6.0, 5.0], [4.0, 8.0, 8.0], [1.0, 1.0, 5.0]]], [[[3.0, 5.0, 2.0], [1.0, 7.0, 1.0], [4.0, 6.0, 2.0]], [[7.0, 7.0, 7.0], 
[6.0, 1.0, 5.0], [5.0, 3.0, 4.0]], [[3.0, 7.0, 8.0], [6.0, 8.0, 4.0], [8.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 4.0], [1.0, 5.0, 3.0], [7.0, 3.0, 6.0]], [[4.0, 1.0, 6.0], [4.0, 4.0, 5.0], [1.0, 
5.0, 3.0]], [[3.0, 2.0, 1.0], [5.0, 5.0, 5.0], [4.0, 2.0, 7.0]]], [[[4.0, 4.0, 8.0], [4.0, 1.0, 8.0], [1.0, 3.0, 6.0]], [[3.0, 
8.0, 2.0], [1.0, 8.0, 7.0], [8.0, 4.0, 3.0]], [[7.0, 8.0, 8.0], [2.0, 2.0, 4.0], [7.0, 3.0, 2.0]]], [[[8.0, 2.0, 1.0], [7.0, 
2.0, 7.0], [4.0, 1.0, 6.0]], [[3.0, 4.0, 7.0], [1.0, 8.0, 8.0], [6.0, 6.0, 5.0]], [[1.0, 5.0, 7.0], [3.0, 4.0, 8.0], [4.0, 4.0, 
1.0]]], [[[1.0, 6.0, 7.0], [6.0, 7.0, 1.0], [1.0, 3.0, 8.0]], [[4.0, 6.0, 7.0], [2.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[3.0, 1.0, 
3.0], [3.0, 3.0, 8.0], [8.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 7.0], [2.0, 8.0, 5.0], [1.0, 7.0, 3.0]], 
[[8.0, 3.0, 3.0], [3.0, 8.0, 5.0], [5.0, 4.0, 2.0]], [[6.0, 2.0, 4.0], [7.0, 2.0, 1.0], [4.0, 2.0, 5.0]]], [[[6.0, 8.0, 7.0], 
[7.0, 2.0, 3.0], [3.0, 4.0, 6.0]], [[5.0, 5.0, 3.0], [8.0, 4.0, 8.0], [1.0, 1.0, 5.0]], [[8.0, 7.0, 4.0], [8.0, 7.0, 3.0], 
[7.0, 4.0, 1.0]]], [[[2.0, 4.0, 8.0], [4.0, 2.0, 1.0], [5.0, 5.0, 8.0]], [[5.0, 4.0, 6.0], [5.0, 6.0, 7.0], [7.0, 4.0, 1.0]], 
[[4.0, 2.0, 4.0], [3.0, 1.0, 3.0], [6.0, 3.0, 7.0]]], [[[8.0, 2.0, 5.0], [8.0, 6.0, 3.0], [1.0, 1.0, 8.0]], [[1.0, 5.0, 4.0], 
[3.0, 8.0, 1.0], [6.0, 6.0, 3.0]], [[7.0, 2.0, 3.0], [1.0, 3.0, 4.0], [5.0, 8.0, 1.0]]]])
      val=numpy.array([[[[3.0, 7.0, 8.0], [3.0, 5.0, 7.0], [3.0, 2.0, 4.0]]], [[[4.0, 8.0, 8.0], [3.0, 3.0, 3.0], [1.0, 8.0, 
2.0]]]])
      arg[1:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 4.0], [1.0, 5.0, 3.0], [7.0, 3.0, 6.0]], [[4.0, 1.0, 6.0], [4.0, 4.0, 5.0], [1.0, 
5.0, 3.0]], [[3.0, 2.0, 1.0], [5.0, 5.0, 5.0], [4.0, 2.0, 7.0]]], [[[4.0, 4.0, 8.0], [4.0, 1.0, 8.0], [1.0, 3.0, 6.0]], [[3.0, 
7.0, 8.0], [3.0, 5.0, 7.0], [3.0, 2.0, 4.0]], [[7.0, 8.0, 8.0], [2.0, 2.0, 4.0], [7.0, 3.0, 2.0]]], [[[8.0, 2.0, 1.0], [7.0, 
2.0, 7.0], [4.0, 1.0, 6.0]], [[4.0, 8.0, 8.0], [3.0, 3.0, 3.0], [1.0, 8.0, 2.0]], [[1.0, 5.0, 7.0], [3.0, 4.0, 8.0], [4.0, 4.0, 
1.0]]], [[[1.0, 6.0, 7.0], [6.0, 7.0, 1.0], [1.0, 3.0, 8.0]], [[4.0, 6.0, 7.0], [2.0, 7.0, 4.0], [7.0, 1.0, 3.0]], [[3.0, 1.0, 
3.0], [3.0, 3.0, 8.0], [8.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 7.0], [2.0, 8.0, 5.0], [1.0, 7.0, 3.0]], 
[[8.0, 3.0, 3.0], [3.0, 8.0, 5.0], [5.0, 4.0, 2.0]], [[6.0, 2.0, 4.0], [7.0, 2.0, 1.0], [4.0, 2.0, 5.0]]], [[[6.0, 8.0, 7.0], 
[7.0, 2.0, 3.0], [3.0, 4.0, 6.0]], [[3.0, 7.0, 8.0], [3.0, 5.0, 7.0], [3.0, 2.0, 4.0]], [[8.0, 7.0, 4.0], [8.0, 7.0, 3.0], 
[7.0, 4.0, 1.0]]], [[[2.0, 4.0, 8.0], [4.0, 2.0, 1.0], [5.0, 5.0, 8.0]], [[4.0, 8.0, 8.0], [3.0, 3.0, 3.0], [1.0, 8.0, 2.0]], 
[[4.0, 2.0, 4.0], [3.0, 1.0, 3.0], [6.0, 3.0, 7.0]]], [[[8.0, 2.0, 5.0], [8.0, 6.0, 3.0], [1.0, 1.0, 8.0]], [[1.0, 5.0, 4.0], 
[3.0, 8.0, 1.0], [6.0, 6.0, 3.0]], [[7.0, 2.0, 3.0], [1.0, 3.0, 4.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 3.0], [5.0, 1.0, 6.0], [3.0, 1.0, 7.0]], [[2.0, 6.0, 3.0], [2.0, 2.0, 6.0], [2.0, 
6.0, 1.0]], [[2.0, 2.0, 1.0], [4.0, 6.0, 1.0], [2.0, 3.0, 4.0]]], [[[2.0, 5.0, 6.0], [5.0, 4.0, 3.0], [4.0, 5.0, 3.0]], [[4.0, 
3.0, 4.0], [3.0, 5.0, 8.0], [7.0, 5.0, 1.0]], [[5.0, 1.0, 5.0], [8.0, 1.0, 3.0], [8.0, 7.0, 1.0]]], [[[7.0, 2.0, 8.0], [4.0, 
8.0, 6.0], [2.0, 7.0, 7.0]], [[1.0, 7.0, 1.0], [3.0, 5.0, 2.0], [5.0, 6.0, 4.0]], [[2.0, 2.0, 3.0], [5.0, 7.0, 1.0], [2.0, 6.0, 
2.0]]], [[[3.0, 7.0, 8.0], [3.0, 3.0, 3.0], [2.0, 5.0, 1.0]], [[6.0, 2.0, 8.0], [2.0, 1.0, 8.0], [1.0, 2.0, 2.0]], [[1.0, 7.0, 
8.0], [3.0, 1.0, 4.0], [6.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 5.0], [2.0, 3.0, 2.0], [1.0, 1.0, 8.0]], 
[[8.0, 1.0, 7.0], [6.0, 4.0, 2.0], [4.0, 1.0, 4.0]], [[6.0, 8.0, 5.0], [5.0, 2.0, 2.0], [8.0, 4.0, 4.0]]], [[[1.0, 2.0, 5.0], 
[1.0, 3.0, 6.0], [2.0, 1.0, 5.0]], [[2.0, 3.0, 1.0], [6.0, 4.0, 8.0], [3.0, 8.0, 2.0]], [[3.0, 8.0, 3.0], [6.0, 8.0, 1.0], 
[5.0, 3.0, 8.0]]], [[[5.0, 4.0, 4.0], [2.0, 1.0, 6.0], [7.0, 6.0, 1.0]], [[3.0, 4.0, 8.0], [2.0, 4.0, 7.0], [5.0, 7.0, 8.0]], 
[[8.0, 6.0, 4.0], [2.0, 3.0, 5.0], [7.0, 5.0, 3.0]]], [[[6.0, 4.0, 5.0], [4.0, 3.0, 5.0], [7.0, 6.0, 2.0]], [[3.0, 7.0, 2.0], 
[7.0, 4.0, 4.0], [7.0, 5.0, 7.0]], [[4.0, 5.0, 1.0], [6.0, 3.0, 3.0], [1.0, 6.0, 2.0]]]])
      val=Data(numpy.array([[[[2.0, 3.0, 8.0], [3.0, 4.0, 7.0], [2.0, 8.0, 7.0]]], [[[6.0, 7.0, 1.0], [6.0, 6.0, 2.0], [6.0, 
3.0, 5.0]]]]),self.functionspace)
      arg[1:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 3.0], [5.0, 1.0, 6.0], [3.0, 1.0, 7.0]], [[2.0, 6.0, 3.0], [2.0, 2.0, 6.0], [2.0, 
6.0, 1.0]], [[2.0, 2.0, 1.0], [4.0, 6.0, 1.0], [2.0, 3.0, 4.0]]], [[[2.0, 5.0, 6.0], [5.0, 4.0, 3.0], [4.0, 5.0, 3.0]], [[2.0, 
3.0, 8.0], [3.0, 4.0, 7.0], [2.0, 8.0, 7.0]], [[5.0, 1.0, 5.0], [8.0, 1.0, 3.0], [8.0, 7.0, 1.0]]], [[[7.0, 2.0, 8.0], [4.0, 
8.0, 6.0], [2.0, 7.0, 7.0]], [[6.0, 7.0, 1.0], [6.0, 6.0, 2.0], [6.0, 3.0, 5.0]], [[2.0, 2.0, 3.0], [5.0, 7.0, 1.0], [2.0, 6.0, 
2.0]]], [[[3.0, 7.0, 8.0], [3.0, 3.0, 3.0], [2.0, 5.0, 1.0]], [[6.0, 2.0, 8.0], [2.0, 1.0, 8.0], [1.0, 2.0, 2.0]], [[1.0, 7.0, 
8.0], [3.0, 1.0, 4.0], [6.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 5.0], [2.0, 3.0, 2.0], [1.0, 1.0, 8.0]], 
[[8.0, 1.0, 7.0], [6.0, 4.0, 2.0], [4.0, 1.0, 4.0]], [[6.0, 8.0, 5.0], [5.0, 2.0, 2.0], [8.0, 4.0, 4.0]]], [[[1.0, 2.0, 5.0], 
[1.0, 3.0, 6.0], [2.0, 1.0, 5.0]], [[2.0, 3.0, 8.0], [3.0, 4.0, 7.0], [2.0, 8.0, 7.0]], [[3.0, 8.0, 3.0], [6.0, 8.0, 1.0], 
[5.0, 3.0, 8.0]]], [[[5.0, 4.0, 4.0], [2.0, 1.0, 6.0], [7.0, 6.0, 1.0]], [[6.0, 7.0, 1.0], [6.0, 6.0, 2.0], [6.0, 3.0, 5.0]], 
[[8.0, 6.0, 4.0], [2.0, 3.0, 5.0], [7.0, 5.0, 3.0]]], [[[6.0, 4.0, 5.0], [4.0, 3.0, 5.0], [7.0, 6.0, 2.0]], [[3.0, 7.0, 2.0], 
[7.0, 4.0, 4.0], [7.0, 5.0, 7.0]], [[4.0, 5.0, 1.0], [6.0, 3.0, 3.0], [1.0, 6.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 3.0], [2.0, 6.0, 5.0], [8.0, 2.0, 4.0]], [[4.0, 4.0, 4.0], [8.0, 1.0, 4.0], [4.0, 
7.0, 2.0]], [[6.0, 7.0, 2.0], [7.0, 2.0, 2.0], [3.0, 4.0, 2.0]]], [[[2.0, 7.0, 6.0], [3.0, 8.0, 1.0], [1.0, 5.0, 8.0]], [[3.0, 
6.0, 4.0], [4.0, 1.0, 7.0], [8.0, 7.0, 7.0]], [[2.0, 3.0, 1.0], [3.0, 8.0, 3.0], [3.0, 4.0, 3.0]]], [[[8.0, 8.0, 5.0], [8.0, 
4.0, 2.0], [1.0, 4.0, 8.0]], [[8.0, 8.0, 3.0], [8.0, 2.0, 2.0], [1.0, 5.0, 2.0]], [[7.0, 7.0, 8.0], [5.0, 4.0, 1.0], [5.0, 7.0, 
2.0]]], [[[2.0, 2.0, 3.0], [2.0, 2.0, 6.0], [1.0, 3.0, 7.0]], [[7.0, 8.0, 7.0], [4.0, 7.0, 2.0], [3.0, 7.0, 4.0]], [[2.0, 7.0, 
7.0], [5.0, 5.0, 2.0], [1.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 2.0, 7.0], [4.0, 5.0, 5.0], [8.0, 1.0, 3.0]], 
[[4.0, 7.0, 3.0], [6.0, 7.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 7.0, 2.0], [2.0, 8.0, 8.0], [3.0, 8.0, 8.0]]], [[[4.0, 4.0, 6.0], 
[2.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[3.0, 8.0, 2.0], [1.0, 4.0, 8.0], [2.0, 1.0, 5.0]], [[3.0, 7.0, 2.0], [4.0, 5.0, 5.0], 
[4.0, 3.0, 3.0]]], [[[1.0, 6.0, 5.0], [1.0, 7.0, 1.0], [6.0, 7.0, 1.0]], [[1.0, 3.0, 2.0], [5.0, 7.0, 4.0], [7.0, 1.0, 4.0]], 
[[1.0, 3.0, 1.0], [8.0, 7.0, 4.0], [7.0, 7.0, 5.0]]], [[[6.0, 4.0, 3.0], [2.0, 2.0, 2.0], [6.0, 4.0, 1.0]], [[6.0, 5.0, 6.0], 
[7.0, 8.0, 8.0], [5.0, 1.0, 7.0]], [[5.0, 8.0, 3.0], [3.0, 1.0, 4.0], [8.0, 6.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 5.0, 3.0], [3.0, 7.0, 6.0], [7.0, 3.0, 3.0]]], [[[6.0, 7.0, 5.0], [8.0, 3.0, 1.0], [1.0, 
1.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 3.0, 1.0], [1.0, 8.0, 2.0], [3.0, 1.0, 6.0]]], [[[4.0, 5.0, 5.0], [3.0, 4.0, 
8.0], [8.0, 8.0, 1.0]]]])
      arg[1:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 3.0], [2.0, 6.0, 5.0], [8.0, 2.0, 4.0]], [[4.0, 4.0, 4.0], [8.0, 1.0, 4.0], [4.0, 
7.0, 2.0]], [[6.0, 7.0, 2.0], [7.0, 2.0, 2.0], [3.0, 4.0, 2.0]]], [[[2.0, 7.0, 6.0], [3.0, 8.0, 1.0], [1.0, 5.0, 8.0]], [[8.0, 
5.0, 3.0], [3.0, 7.0, 6.0], [7.0, 3.0, 3.0]], [[2.0, 3.0, 1.0], [3.0, 8.0, 3.0], [3.0, 4.0, 3.0]]], [[[8.0, 8.0, 5.0], [8.0, 
4.0, 2.0], [1.0, 4.0, 8.0]], [[6.0, 7.0, 5.0], [8.0, 3.0, 1.0], [1.0, 1.0, 8.0]], [[7.0, 7.0, 8.0], [5.0, 4.0, 1.0], [5.0, 7.0, 
2.0]]], [[[2.0, 2.0, 3.0], [2.0, 2.0, 6.0], [1.0, 3.0, 7.0]], [[7.0, 8.0, 7.0], [4.0, 7.0, 2.0], [3.0, 7.0, 4.0]], [[2.0, 7.0, 
7.0], [5.0, 5.0, 2.0], [1.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 7.0], [4.0, 5.0, 5.0], [8.0, 1.0, 3.0]], 
[[4.0, 7.0, 3.0], [6.0, 7.0, 1.0], [5.0, 7.0, 4.0]], [[1.0, 7.0, 2.0], [2.0, 8.0, 8.0], [3.0, 8.0, 8.0]]], [[[4.0, 4.0, 6.0], 
[2.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[6.0, 3.0, 1.0], [1.0, 8.0, 2.0], [3.0, 1.0, 6.0]], [[3.0, 7.0, 2.0], [4.0, 5.0, 5.0], 
[4.0, 3.0, 3.0]]], [[[1.0, 6.0, 5.0], [1.0, 7.0, 1.0], [6.0, 7.0, 1.0]], [[4.0, 5.0, 5.0], [3.0, 4.0, 8.0], [8.0, 8.0, 1.0]], 
[[1.0, 3.0, 1.0], [8.0, 7.0, 4.0], [7.0, 7.0, 5.0]]], [[[6.0, 4.0, 3.0], [2.0, 2.0, 2.0], [6.0, 4.0, 1.0]], [[6.0, 5.0, 6.0], 
[7.0, 8.0, 8.0], [5.0, 1.0, 7.0]], [[5.0, 8.0, 3.0], [3.0, 1.0, 4.0], [8.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 1.0], [4.0, 4.0, 7.0], [4.0, 8.0, 1.0]], [[7.0, 2.0, 7.0], [2.0, 1.0, 1.0], [2.0, 
6.0, 8.0]], [[6.0, 5.0, 1.0], [1.0, 8.0, 5.0], [7.0, 3.0, 2.0]]], [[[5.0, 6.0, 5.0], [2.0, 3.0, 5.0], [5.0, 4.0, 2.0]], [[5.0, 
6.0, 8.0], [7.0, 7.0, 4.0], [6.0, 8.0, 5.0]], [[7.0, 1.0, 7.0], [7.0, 2.0, 4.0], [7.0, 3.0, 2.0]]], [[[8.0, 8.0, 1.0], [5.0, 
2.0, 1.0], [3.0, 2.0, 7.0]], [[3.0, 2.0, 6.0], [5.0, 7.0, 1.0], [8.0, 5.0, 6.0]], [[3.0, 6.0, 4.0], [4.0, 3.0, 8.0], [6.0, 7.0, 
3.0]]], [[[8.0, 4.0, 7.0], [3.0, 6.0, 4.0], [5.0, 1.0, 6.0]], [[1.0, 5.0, 3.0], [8.0, 8.0, 4.0], [8.0, 1.0, 1.0]], [[3.0, 1.0, 
3.0], [5.0, 8.0, 3.0], [1.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 1.0], [6.0, 6.0, 7.0], [1.0, 2.0, 4.0]], 
[[2.0, 5.0, 2.0], [4.0, 4.0, 2.0], [3.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 3.0, 1.0], [7.0, 5.0, 2.0]]], [[[1.0, 3.0, 4.0], 
[6.0, 1.0, 6.0], [7.0, 5.0, 8.0]], [[3.0, 1.0, 8.0], [6.0, 7.0, 7.0], [2.0, 1.0, 5.0]], [[1.0, 5.0, 8.0], [8.0, 6.0, 8.0], 
[1.0, 8.0, 1.0]]], [[[8.0, 1.0, 7.0], [2.0, 1.0, 4.0], [2.0, 5.0, 2.0]], [[3.0, 3.0, 6.0], [5.0, 5.0, 6.0], [3.0, 8.0, 5.0]], 
[[4.0, 8.0, 4.0], [2.0, 3.0, 3.0], [6.0, 6.0, 3.0]]], [[[3.0, 4.0, 3.0], [1.0, 4.0, 8.0], [5.0, 2.0, 4.0]], [[6.0, 4.0, 2.0], 
[2.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[7.0, 8.0, 6.0], [4.0, 5.0, 1.0], [7.0, 5.0, 6.0]]]])
      val=numpy.array([[[4.0, 2.0, 4.0], [1.0, 1.0, 5.0], [3.0, 5.0, 1.0]], [[3.0, 7.0, 2.0], [3.0, 7.0, 3.0], [2.0, 5.0, 
1.0]]])
      arg[1:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 1.0], [4.0, 4.0, 7.0], [4.0, 8.0, 1.0]], [[7.0, 2.0, 7.0], [2.0, 1.0, 1.0], [2.0, 
6.0, 8.0]], [[6.0, 5.0, 1.0], [1.0, 8.0, 5.0], [7.0, 3.0, 2.0]]], [[[5.0, 6.0, 5.0], [4.0, 2.0, 4.0], [5.0, 4.0, 2.0]], [[5.0, 
6.0, 8.0], [1.0, 1.0, 5.0], [6.0, 8.0, 5.0]], [[7.0, 1.0, 7.0], [3.0, 5.0, 1.0], [7.0, 3.0, 2.0]]], [[[8.0, 8.0, 1.0], [3.0, 
7.0, 2.0], [3.0, 2.0, 7.0]], [[3.0, 2.0, 6.0], [3.0, 7.0, 3.0], [8.0, 5.0, 6.0]], [[3.0, 6.0, 4.0], [2.0, 5.0, 1.0], [6.0, 7.0, 
3.0]]], [[[8.0, 4.0, 7.0], [3.0, 6.0, 4.0], [5.0, 1.0, 6.0]], [[1.0, 5.0, 3.0], [8.0, 8.0, 4.0], [8.0, 1.0, 1.0]], [[3.0, 1.0, 
3.0], [5.0, 8.0, 3.0], [1.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 1.0], [6.0, 6.0, 7.0], [1.0, 2.0, 4.0]], 
[[2.0, 5.0, 2.0], [4.0, 4.0, 2.0], [3.0, 4.0, 2.0]], [[4.0, 4.0, 5.0], [5.0, 3.0, 1.0], [7.0, 5.0, 2.0]]], [[[1.0, 3.0, 4.0], 
[4.0, 2.0, 4.0], [7.0, 5.0, 8.0]], [[3.0, 1.0, 8.0], [1.0, 1.0, 5.0], [2.0, 1.0, 5.0]], [[1.0, 5.0, 8.0], [3.0, 5.0, 1.0], 
[1.0, 8.0, 1.0]]], [[[8.0, 1.0, 7.0], [3.0, 7.0, 2.0], [2.0, 5.0, 2.0]], [[3.0, 3.0, 6.0], [3.0, 7.0, 3.0], [3.0, 8.0, 5.0]], 
[[4.0, 8.0, 4.0], [2.0, 5.0, 1.0], [6.0, 6.0, 3.0]]], [[[3.0, 4.0, 3.0], [1.0, 4.0, 8.0], [5.0, 2.0, 4.0]], [[6.0, 4.0, 2.0], 
[2.0, 3.0, 3.0], [5.0, 6.0, 8.0]], [[7.0, 8.0, 6.0], [4.0, 5.0, 1.0], [7.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 6.0, 6.0], [4.0, 2.0, 4.0], [2.0, 5.0, 3.0]], [[8.0, 1.0, 2.0], [2.0, 2.0, 6.0], [1.0, 
5.0, 3.0]], [[5.0, 7.0, 5.0], [4.0, 6.0, 8.0], [7.0, 1.0, 5.0]]], [[[7.0, 3.0, 7.0], [4.0, 5.0, 5.0], [7.0, 7.0, 4.0]], [[8.0, 
1.0, 6.0], [5.0, 5.0, 4.0], [4.0, 2.0, 8.0]], [[2.0, 1.0, 7.0], [8.0, 3.0, 6.0], [4.0, 5.0, 3.0]]], [[[7.0, 8.0, 2.0], [2.0, 
4.0, 6.0], [3.0, 3.0, 6.0]], [[1.0, 5.0, 4.0], [7.0, 3.0, 5.0], [7.0, 8.0, 3.0]], [[8.0, 3.0, 6.0], [1.0, 6.0, 6.0], [3.0, 7.0, 
2.0]]], [[[8.0, 5.0, 8.0], [4.0, 7.0, 8.0], [3.0, 1.0, 8.0]], [[2.0, 7.0, 2.0], [7.0, 7.0, 1.0], [8.0, 6.0, 2.0]], [[2.0, 8.0, 
3.0], [7.0, 7.0, 8.0], [7.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 8.0], [8.0, 2.0, 4.0], [2.0, 6.0, 5.0]], 
[[4.0, 4.0, 5.0], [7.0, 1.0, 7.0], [5.0, 6.0, 1.0]], [[7.0, 8.0, 8.0], [8.0, 7.0, 8.0], [6.0, 3.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[5.0, 4.0, 7.0], [2.0, 7.0, 8.0]], [[7.0, 5.0, 4.0], [4.0, 2.0, 8.0], [8.0, 7.0, 2.0]], [[8.0, 3.0, 1.0], [2.0, 1.0, 2.0], 
[7.0, 6.0, 2.0]]], [[[8.0, 7.0, 8.0], [2.0, 6.0, 6.0], [5.0, 3.0, 8.0]], [[3.0, 7.0, 7.0], [2.0, 6.0, 6.0], [5.0, 2.0, 7.0]], 
[[3.0, 7.0, 4.0], [7.0, 7.0, 7.0], [1.0, 7.0, 6.0]]], [[[3.0, 4.0, 2.0], [1.0, 2.0, 1.0], [2.0, 1.0, 4.0]], [[5.0, 1.0, 1.0], 
[8.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[1.0, 1.0, 8.0], [8.0, 6.0, 8.0], [3.0, 4.0, 3.0]]]])
      val=Data(numpy.array([[[6.0, 8.0, 5.0], [7.0, 1.0, 2.0], [2.0, 6.0, 4.0]], [[2.0, 1.0, 2.0], [1.0, 3.0, 8.0], [5.0, 7.0, 
3.0]]]),self.functionspace)
      arg[1:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 6.0], [4.0, 2.0, 4.0], [2.0, 5.0, 3.0]], [[8.0, 1.0, 2.0], [2.0, 2.0, 6.0], [1.0, 
5.0, 3.0]], [[5.0, 7.0, 5.0], [4.0, 6.0, 8.0], [7.0, 1.0, 5.0]]], [[[7.0, 3.0, 7.0], [6.0, 8.0, 5.0], [7.0, 7.0, 4.0]], [[8.0, 
1.0, 6.0], [7.0, 1.0, 2.0], [4.0, 2.0, 8.0]], [[2.0, 1.0, 7.0], [2.0, 6.0, 4.0], [4.0, 5.0, 3.0]]], [[[7.0, 8.0, 2.0], [2.0, 
1.0, 2.0], [3.0, 3.0, 6.0]], [[1.0, 5.0, 4.0], [1.0, 3.0, 8.0], [7.0, 8.0, 3.0]], [[8.0, 3.0, 6.0], [5.0, 7.0, 3.0], [3.0, 7.0, 
2.0]]], [[[8.0, 5.0, 8.0], [4.0, 7.0, 8.0], [3.0, 1.0, 8.0]], [[2.0, 7.0, 2.0], [7.0, 7.0, 1.0], [8.0, 6.0, 2.0]], [[2.0, 8.0, 
3.0], [7.0, 7.0, 8.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 8.0], [8.0, 2.0, 4.0], [2.0, 6.0, 5.0]], 
[[4.0, 4.0, 5.0], [7.0, 1.0, 7.0], [5.0, 6.0, 1.0]], [[7.0, 8.0, 8.0], [8.0, 7.0, 8.0], [6.0, 3.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[6.0, 8.0, 5.0], [2.0, 7.0, 8.0]], [[7.0, 5.0, 4.0], [7.0, 1.0, 2.0], [8.0, 7.0, 2.0]], [[8.0, 3.0, 1.0], [2.0, 6.0, 4.0], 
[7.0, 6.0, 2.0]]], [[[8.0, 7.0, 8.0], [2.0, 1.0, 2.0], [5.0, 3.0, 8.0]], [[3.0, 7.0, 7.0], [1.0, 3.0, 8.0], [5.0, 2.0, 7.0]], 
[[3.0, 7.0, 4.0], [5.0, 7.0, 3.0], [1.0, 7.0, 6.0]]], [[[3.0, 4.0, 2.0], [1.0, 2.0, 1.0], [2.0, 1.0, 4.0]], [[5.0, 1.0, 1.0], 
[8.0, 3.0, 7.0], [5.0, 5.0, 8.0]], [[1.0, 1.0, 8.0], [8.0, 6.0, 8.0], [3.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 2.0], [5.0, 7.0, 7.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 8.0], [7.0, 1.0, 8.0], [1.0, 
5.0, 4.0]], [[1.0, 8.0, 4.0], [3.0, 3.0, 5.0], [2.0, 8.0, 5.0]]], [[[4.0, 7.0, 1.0], [6.0, 6.0, 1.0], [8.0, 7.0, 5.0]], [[5.0, 
2.0, 2.0], [4.0, 7.0, 4.0], [7.0, 2.0, 6.0]], [[3.0, 2.0, 6.0], [1.0, 8.0, 6.0], [7.0, 8.0, 5.0]]], [[[1.0, 2.0, 5.0], [4.0, 
3.0, 5.0], [2.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [7.0, 3.0, 7.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 2.0], [2.0, 8.0, 1.0], [7.0, 3.0, 
7.0]]], [[[5.0, 4.0, 2.0], [8.0, 6.0, 6.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 5.0], [6.0, 2.0, 8.0], [6.0, 1.0, 3.0]], [[2.0, 1.0, 
8.0], [3.0, 8.0, 7.0], [8.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 8.0], [6.0, 7.0, 5.0], [2.0, 4.0, 5.0]], 
[[3.0, 5.0, 8.0], [5.0, 1.0, 2.0], [6.0, 8.0, 8.0]], [[2.0, 3.0, 1.0], [4.0, 4.0, 1.0], [2.0, 6.0, 7.0]]], [[[8.0, 2.0, 2.0], 
[1.0, 3.0, 2.0], [3.0, 3.0, 4.0]], [[4.0, 6.0, 6.0], [2.0, 8.0, 4.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 7.0], [1.0, 4.0, 5.0], 
[3.0, 8.0, 5.0]]], [[[8.0, 5.0, 5.0], [5.0, 6.0, 4.0], [5.0, 7.0, 8.0]], [[4.0, 2.0, 5.0], [3.0, 3.0, 6.0], [8.0, 5.0, 5.0]], 
[[7.0, 1.0, 2.0], [2.0, 5.0, 7.0], [6.0, 3.0, 1.0]]], [[[1.0, 5.0, 7.0], [4.0, 8.0, 1.0], [1.0, 1.0, 1.0]], [[6.0, 7.0, 8.0], 
[2.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[2.0, 7.0, 1.0], [6.0, 6.0, 3.0], [6.0, 7.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 4.0, 3.0], [6.0, 5.0, 8.0], [5.0, 7.0, 1.0]], [[3.0, 5.0, 2.0], [2.0, 3.0, 8.0], [7.0, 
3.0, 5.0]]])+(1.-msk_val)*numpy.array([[[8.0, 2.0, 1.0], [7.0, 1.0, 5.0], [6.0, 8.0, 3.0]], [[3.0, 3.0, 3.0], [2.0, 6.0, 2.0], 
[3.0, 6.0, 2.0]]])
      arg[1:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 2.0], [5.0, 7.0, 7.0], [5.0, 3.0, 7.0]], [[7.0, 2.0, 8.0], [7.0, 1.0, 8.0], [1.0, 
5.0, 4.0]], [[1.0, 8.0, 4.0], [3.0, 3.0, 5.0], [2.0, 8.0, 5.0]]], [[[4.0, 7.0, 1.0], [8.0, 4.0, 3.0], [8.0, 7.0, 5.0]], [[5.0, 
2.0, 2.0], [6.0, 5.0, 8.0], [7.0, 2.0, 6.0]], [[3.0, 2.0, 6.0], [5.0, 7.0, 1.0], [7.0, 8.0, 5.0]]], [[[1.0, 2.0, 5.0], [3.0, 
5.0, 2.0], [2.0, 3.0, 6.0]], [[5.0, 1.0, 1.0], [2.0, 3.0, 8.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 2.0], [7.0, 3.0, 5.0], [7.0, 3.0, 
7.0]]], [[[5.0, 4.0, 2.0], [8.0, 6.0, 6.0], [5.0, 8.0, 4.0]], [[8.0, 1.0, 5.0], [6.0, 2.0, 8.0], [6.0, 1.0, 3.0]], [[2.0, 1.0, 
8.0], [3.0, 8.0, 7.0], [8.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 8.0], [6.0, 7.0, 5.0], [2.0, 4.0, 5.0]], 
[[3.0, 5.0, 8.0], [5.0, 1.0, 2.0], [6.0, 8.0, 8.0]], [[2.0, 3.0, 1.0], [4.0, 4.0, 1.0], [2.0, 6.0, 7.0]]], [[[8.0, 2.0, 2.0], 
[8.0, 2.0, 1.0], [3.0, 3.0, 4.0]], [[4.0, 6.0, 6.0], [7.0, 1.0, 5.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 7.0], [6.0, 8.0, 3.0], 
[3.0, 8.0, 5.0]]], [[[8.0, 5.0, 5.0], [3.0, 3.0, 3.0], [5.0, 7.0, 8.0]], [[4.0, 2.0, 5.0], [2.0, 6.0, 2.0], [8.0, 5.0, 5.0]], 
[[7.0, 1.0, 2.0], [3.0, 6.0, 2.0], [6.0, 3.0, 1.0]]], [[[1.0, 5.0, 7.0], [4.0, 8.0, 1.0], [1.0, 1.0, 1.0]], [[6.0, 7.0, 8.0], 
[2.0, 6.0, 3.0], [2.0, 5.0, 8.0]], [[2.0, 7.0, 1.0], [6.0, 6.0, 3.0], [6.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 3.0], [4.0, 6.0, 2.0], [6.0, 3.0, 8.0]], [[2.0, 2.0, 2.0], [4.0, 7.0, 7.0], [7.0, 
5.0, 6.0]], [[5.0, 5.0, 8.0], [3.0, 8.0, 3.0], [6.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [7.0, 2.0, 2.0], [4.0, 5.0, 7.0]], [[3.0, 
3.0, 6.0], [3.0, 6.0, 5.0], [1.0, 7.0, 6.0]], [[3.0, 4.0, 4.0], [7.0, 7.0, 2.0], [3.0, 3.0, 4.0]]], [[[1.0, 1.0, 5.0], [7.0, 
4.0, 8.0], [3.0, 6.0, 4.0]], [[5.0, 7.0, 8.0], [7.0, 5.0, 7.0], [8.0, 6.0, 2.0]], [[6.0, 7.0, 1.0], [1.0, 2.0, 3.0], [1.0, 3.0, 
5.0]]], [[[8.0, 8.0, 4.0], [7.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[6.0, 7.0, 8.0], [4.0, 6.0, 7.0], [8.0, 1.0, 1.0]], [[2.0, 5.0, 
1.0], [4.0, 2.0, 5.0], [5.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 4.0], [7.0, 1.0, 8.0], [2.0, 1.0, 3.0]], 
[[1.0, 7.0, 2.0], [8.0, 1.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 1.0, 2.0], [1.0, 5.0, 3.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 3.0], 
[5.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[8.0, 6.0, 3.0], [3.0, 2.0, 6.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 7.0], [6.0, 5.0, 4.0], 
[3.0, 2.0, 8.0]]], [[[1.0, 1.0, 2.0], [4.0, 5.0, 3.0], [7.0, 5.0, 3.0]], [[6.0, 1.0, 3.0], [7.0, 4.0, 3.0], [5.0, 3.0, 7.0]], 
[[8.0, 7.0, 7.0], [7.0, 1.0, 4.0], [1.0, 7.0, 5.0]]], [[[3.0, 4.0, 6.0], [2.0, 3.0, 4.0], [5.0, 2.0, 2.0]], [[5.0, 2.0, 1.0], 
[1.0, 2.0, 2.0], [5.0, 4.0, 8.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 3.0], [8.0, 1.0, 7.0]]]])
      val=numpy.array([[[[4.0, 6.0, 1.0], [2.0, 5.0, 8.0], [1.0, 7.0, 1.0]]], [[[1.0, 6.0, 4.0], [5.0, 7.0, 1.0], [5.0, 1.0, 
4.0]]]])
      arg[2:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 3.0], [4.0, 6.0, 2.0], [6.0, 3.0, 8.0]], [[2.0, 2.0, 2.0], [4.0, 7.0, 7.0], [7.0, 
5.0, 6.0]], [[5.0, 5.0, 8.0], [3.0, 8.0, 3.0], [6.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [7.0, 2.0, 2.0], [4.0, 5.0, 7.0]], [[3.0, 
3.0, 6.0], [3.0, 6.0, 5.0], [1.0, 7.0, 6.0]], [[3.0, 4.0, 4.0], [7.0, 7.0, 2.0], [3.0, 3.0, 4.0]]], [[[1.0, 1.0, 5.0], [7.0, 
4.0, 8.0], [3.0, 6.0, 4.0]], [[4.0, 6.0, 1.0], [2.0, 5.0, 8.0], [1.0, 7.0, 1.0]], [[6.0, 7.0, 1.0], [1.0, 2.0, 3.0], [1.0, 3.0, 
5.0]]], [[[8.0, 8.0, 4.0], [7.0, 5.0, 3.0], [1.0, 6.0, 6.0]], [[1.0, 6.0, 4.0], [5.0, 7.0, 1.0], [5.0, 1.0, 4.0]], [[2.0, 5.0, 
1.0], [4.0, 2.0, 5.0], [5.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 4.0], [7.0, 1.0, 8.0], [2.0, 1.0, 3.0]], 
[[1.0, 7.0, 2.0], [8.0, 1.0, 1.0], [7.0, 8.0, 4.0]], [[6.0, 1.0, 2.0], [1.0, 5.0, 3.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 3.0], 
[5.0, 8.0, 2.0], [7.0, 3.0, 3.0]], [[8.0, 6.0, 3.0], [3.0, 2.0, 6.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 7.0], [6.0, 5.0, 4.0], 
[3.0, 2.0, 8.0]]], [[[1.0, 1.0, 2.0], [4.0, 5.0, 3.0], [7.0, 5.0, 3.0]], [[4.0, 6.0, 1.0], [2.0, 5.0, 8.0], [1.0, 7.0, 1.0]], 
[[8.0, 7.0, 7.0], [7.0, 1.0, 4.0], [1.0, 7.0, 5.0]]], [[[3.0, 4.0, 6.0], [2.0, 3.0, 4.0], [5.0, 2.0, 2.0]], [[1.0, 6.0, 4.0], 
[5.0, 7.0, 1.0], [5.0, 1.0, 4.0]], [[4.0, 7.0, 5.0], [1.0, 7.0, 3.0], [8.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 2.0], [7.0, 4.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 8.0, 6.0], [6.0, 7.0, 5.0], [2.0, 
7.0, 7.0]], [[4.0, 1.0, 5.0], [1.0, 2.0, 2.0], [8.0, 6.0, 8.0]]], [[[2.0, 5.0, 6.0], [4.0, 6.0, 3.0], [7.0, 7.0, 1.0]], [[4.0, 
7.0, 7.0], [7.0, 8.0, 3.0], [2.0, 6.0, 6.0]], [[3.0, 5.0, 4.0], [4.0, 3.0, 7.0], [4.0, 2.0, 7.0]]], [[[4.0, 3.0, 6.0], [7.0, 
8.0, 2.0], [1.0, 2.0, 1.0]], [[3.0, 3.0, 1.0], [8.0, 7.0, 6.0], [1.0, 5.0, 7.0]], [[1.0, 4.0, 6.0], [1.0, 7.0, 3.0], [2.0, 4.0, 
1.0]]], [[[5.0, 2.0, 5.0], [7.0, 2.0, 7.0], [4.0, 5.0, 5.0]], [[5.0, 5.0, 2.0], [7.0, 4.0, 4.0], [8.0, 2.0, 3.0]], [[8.0, 7.0, 
3.0], [4.0, 8.0, 5.0], [8.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 3.0], [3.0, 1.0, 2.0], [2.0, 1.0, 5.0]], 
[[2.0, 3.0, 6.0], [1.0, 7.0, 8.0], [6.0, 4.0, 4.0]], [[2.0, 5.0, 5.0], [1.0, 8.0, 5.0], [2.0, 1.0, 8.0]]], [[[4.0, 8.0, 8.0], 
[8.0, 5.0, 7.0], [2.0, 2.0, 6.0]], [[5.0, 8.0, 5.0], [1.0, 7.0, 2.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 2.0], [3.0, 6.0, 8.0], 
[3.0, 8.0, 7.0]]], [[[1.0, 8.0, 1.0], [2.0, 7.0, 2.0], [7.0, 4.0, 2.0]], [[1.0, 4.0, 6.0], [6.0, 5.0, 5.0], [7.0, 1.0, 6.0]], 
[[3.0, 7.0, 6.0], [1.0, 1.0, 6.0], [2.0, 6.0, 3.0]]], [[[8.0, 6.0, 3.0], [5.0, 2.0, 5.0], [5.0, 5.0, 5.0]], [[7.0, 6.0, 1.0], 
[6.0, 8.0, 2.0], [7.0, 5.0, 8.0]], [[6.0, 3.0, 4.0], [7.0, 1.0, 5.0], [2.0, 2.0, 1.0]]]])
      val=Data(numpy.array([[[[3.0, 7.0, 6.0], [5.0, 1.0, 1.0], [1.0, 3.0, 7.0]]], [[[1.0, 2.0, 6.0], [4.0, 6.0, 1.0], [1.0, 
3.0, 7.0]]]]),self.functionspace)
      arg[2:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 2.0], [7.0, 4.0, 1.0], [2.0, 8.0, 3.0]], [[1.0, 8.0, 6.0], [6.0, 7.0, 5.0], [2.0, 
7.0, 7.0]], [[4.0, 1.0, 5.0], [1.0, 2.0, 2.0], [8.0, 6.0, 8.0]]], [[[2.0, 5.0, 6.0], [4.0, 6.0, 3.0], [7.0, 7.0, 1.0]], [[4.0, 
7.0, 7.0], [7.0, 8.0, 3.0], [2.0, 6.0, 6.0]], [[3.0, 5.0, 4.0], [4.0, 3.0, 7.0], [4.0, 2.0, 7.0]]], [[[4.0, 3.0, 6.0], [7.0, 
8.0, 2.0], [1.0, 2.0, 1.0]], [[3.0, 7.0, 6.0], [5.0, 1.0, 1.0], [1.0, 3.0, 7.0]], [[1.0, 4.0, 6.0], [1.0, 7.0, 3.0], [2.0, 4.0, 
1.0]]], [[[5.0, 2.0, 5.0], [7.0, 2.0, 7.0], [4.0, 5.0, 5.0]], [[1.0, 2.0, 6.0], [4.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[8.0, 7.0, 
3.0], [4.0, 8.0, 5.0], [8.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 3.0], [3.0, 1.0, 2.0], [2.0, 1.0, 5.0]], 
[[2.0, 3.0, 6.0], [1.0, 7.0, 8.0], [6.0, 4.0, 4.0]], [[2.0, 5.0, 5.0], [1.0, 8.0, 5.0], [2.0, 1.0, 8.0]]], [[[4.0, 8.0, 8.0], 
[8.0, 5.0, 7.0], [2.0, 2.0, 6.0]], [[5.0, 8.0, 5.0], [1.0, 7.0, 2.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 2.0], [3.0, 6.0, 8.0], 
[3.0, 8.0, 7.0]]], [[[1.0, 8.0, 1.0], [2.0, 7.0, 2.0], [7.0, 4.0, 2.0]], [[3.0, 7.0, 6.0], [5.0, 1.0, 1.0], [1.0, 3.0, 7.0]], 
[[3.0, 7.0, 6.0], [1.0, 1.0, 6.0], [2.0, 6.0, 3.0]]], [[[8.0, 6.0, 3.0], [5.0, 2.0, 5.0], [5.0, 5.0, 5.0]], [[1.0, 2.0, 6.0], 
[4.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[6.0, 3.0, 4.0], [7.0, 1.0, 5.0], [2.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 8.0], [6.0, 5.0, 4.0], [1.0, 1.0, 2.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 4.0], [8.0, 
3.0, 1.0]], [[6.0, 2.0, 6.0], [8.0, 3.0, 7.0], [6.0, 8.0, 3.0]]], [[[7.0, 5.0, 6.0], [5.0, 7.0, 4.0], [8.0, 8.0, 6.0]], [[2.0, 
1.0, 4.0], [4.0, 2.0, 3.0], [3.0, 6.0, 5.0]], [[4.0, 4.0, 8.0], [2.0, 3.0, 7.0], [3.0, 4.0, 4.0]]], [[[1.0, 3.0, 7.0], [2.0, 
6.0, 2.0], [6.0, 7.0, 1.0]], [[4.0, 5.0, 5.0], [3.0, 6.0, 6.0], [5.0, 1.0, 1.0]], [[4.0, 2.0, 6.0], [2.0, 3.0, 4.0], [7.0, 8.0, 
6.0]]], [[[1.0, 7.0, 4.0], [5.0, 8.0, 2.0], [5.0, 5.0, 7.0]], [[5.0, 1.0, 7.0], [8.0, 4.0, 5.0], [6.0, 5.0, 4.0]], [[7.0, 7.0, 
6.0], [6.0, 1.0, 3.0], [7.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 8.0], [1.0, 7.0, 7.0], [5.0, 5.0, 3.0]], 
[[7.0, 2.0, 2.0], [3.0, 4.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 1.0, 6.0], [7.0, 3.0, 1.0], [8.0, 4.0, 2.0]]], [[[8.0, 4.0, 7.0], 
[3.0, 1.0, 5.0], [5.0, 7.0, 4.0]], [[4.0, 3.0, 6.0], [4.0, 1.0, 1.0], [6.0, 3.0, 1.0]], [[3.0, 4.0, 8.0], [2.0, 4.0, 8.0], 
[6.0, 2.0, 6.0]]], [[[3.0, 8.0, 3.0], [6.0, 1.0, 6.0], [2.0, 1.0, 8.0]], [[6.0, 1.0, 7.0], [1.0, 1.0, 5.0], [2.0, 3.0, 7.0]], 
[[7.0, 1.0, 3.0], [8.0, 6.0, 2.0], [6.0, 1.0, 7.0]]], [[[8.0, 7.0, 6.0], [8.0, 2.0, 4.0], [6.0, 3.0, 4.0]], [[7.0, 2.0, 7.0], 
[7.0, 6.0, 4.0], [8.0, 2.0, 1.0]], [[3.0, 8.0, 7.0], [2.0, 8.0, 6.0], [6.0, 3.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 5.0, 1.0], [5.0, 1.0, 2.0], [6.0, 1.0, 6.0]]], [[[3.0, 8.0, 8.0], [6.0, 2.0, 7.0], [8.0, 
6.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 7.0, 2.0], [8.0, 5.0, 8.0], [4.0, 1.0, 5.0]]], [[[2.0, 8.0, 1.0], [5.0, 8.0, 
2.0], [6.0, 3.0, 5.0]]]])
      arg[2:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 5.0, 8.0], [6.0, 5.0, 4.0], [1.0, 1.0, 2.0]], [[8.0, 5.0, 2.0], [8.0, 8.0, 4.0], [8.0, 
3.0, 1.0]], [[6.0, 2.0, 6.0], [8.0, 3.0, 7.0], [6.0, 8.0, 3.0]]], [[[7.0, 5.0, 6.0], [5.0, 7.0, 4.0], [8.0, 8.0, 6.0]], [[2.0, 
1.0, 4.0], [4.0, 2.0, 3.0], [3.0, 6.0, 5.0]], [[4.0, 4.0, 8.0], [2.0, 3.0, 7.0], [3.0, 4.0, 4.0]]], [[[1.0, 3.0, 7.0], [2.0, 
6.0, 2.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 1.0], [5.0, 1.0, 2.0], [6.0, 1.0, 6.0]], [[4.0, 2.0, 6.0], [2.0, 3.0, 4.0], [7.0, 8.0, 
6.0]]], [[[1.0, 7.0, 4.0], [5.0, 8.0, 2.0], [5.0, 5.0, 7.0]], [[3.0, 8.0, 8.0], [6.0, 2.0, 7.0], [8.0, 6.0, 2.0]], [[7.0, 7.0, 
6.0], [6.0, 1.0, 3.0], [7.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 8.0], [1.0, 7.0, 7.0], [5.0, 5.0, 3.0]], 
[[7.0, 2.0, 2.0], [3.0, 4.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 1.0, 6.0], [7.0, 3.0, 1.0], [8.0, 4.0, 2.0]]], [[[8.0, 4.0, 7.0], 
[3.0, 1.0, 5.0], [5.0, 7.0, 4.0]], [[4.0, 3.0, 6.0], [4.0, 1.0, 1.0], [6.0, 3.0, 1.0]], [[3.0, 4.0, 8.0], [2.0, 4.0, 8.0], 
[6.0, 2.0, 6.0]]], [[[3.0, 8.0, 3.0], [6.0, 1.0, 6.0], [2.0, 1.0, 8.0]], [[1.0, 7.0, 2.0], [8.0, 5.0, 8.0], [4.0, 1.0, 5.0]], 
[[7.0, 1.0, 3.0], [8.0, 6.0, 2.0], [6.0, 1.0, 7.0]]], [[[8.0, 7.0, 6.0], [8.0, 2.0, 4.0], [6.0, 3.0, 4.0]], [[2.0, 8.0, 1.0], 
[5.0, 8.0, 2.0], [6.0, 3.0, 5.0]], [[3.0, 8.0, 7.0], [2.0, 8.0, 6.0], [6.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 7.0], [3.0, 5.0, 3.0], [5.0, 1.0, 6.0]], [[8.0, 7.0, 7.0], [4.0, 8.0, 7.0], [1.0, 
7.0, 7.0]], [[3.0, 3.0, 3.0], [6.0, 3.0, 7.0], [4.0, 7.0, 1.0]]], [[[7.0, 6.0, 5.0], [8.0, 1.0, 5.0], [7.0, 1.0, 1.0]], [[6.0, 
8.0, 1.0], [3.0, 6.0, 8.0], [1.0, 6.0, 7.0]], [[4.0, 2.0, 5.0], [7.0, 6.0, 4.0], [3.0, 3.0, 3.0]]], [[[5.0, 5.0, 7.0], [8.0, 
8.0, 6.0], [2.0, 3.0, 8.0]], [[2.0, 6.0, 6.0], [8.0, 7.0, 2.0], [6.0, 7.0, 2.0]], [[6.0, 7.0, 1.0], [7.0, 8.0, 4.0], [6.0, 5.0, 
2.0]]], [[[1.0, 2.0, 5.0], [8.0, 1.0, 3.0], [3.0, 1.0, 5.0]], [[7.0, 3.0, 3.0], [6.0, 5.0, 6.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 
7.0], [2.0, 6.0, 6.0], [3.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 2.0], [5.0, 5.0, 4.0], [7.0, 8.0, 8.0]], 
[[4.0, 1.0, 1.0], [7.0, 5.0, 3.0], [6.0, 3.0, 8.0]], [[3.0, 4.0, 4.0], [1.0, 3.0, 4.0], [4.0, 5.0, 8.0]]], [[[2.0, 6.0, 2.0], 
[1.0, 2.0, 2.0], [1.0, 8.0, 7.0]], [[4.0, 6.0, 2.0], [6.0, 6.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 6.0, 5.0], [2.0, 1.0, 8.0], 
[2.0, 8.0, 7.0]]], [[[3.0, 8.0, 3.0], [2.0, 2.0, 5.0], [4.0, 6.0, 7.0]], [[7.0, 5.0, 3.0], [1.0, 8.0, 3.0], [7.0, 3.0, 1.0]], 
[[8.0, 3.0, 3.0], [6.0, 5.0, 6.0], [8.0, 6.0, 1.0]]], [[[4.0, 1.0, 7.0], [3.0, 3.0, 7.0], [2.0, 8.0, 2.0]], [[7.0, 8.0, 4.0], 
[4.0, 6.0, 6.0], [7.0, 4.0, 2.0]], [[7.0, 4.0, 4.0], [1.0, 1.0, 7.0], [4.0, 6.0, 7.0]]]])
      val=numpy.array([[[[1.0, 2.0, 4.0], [1.0, 3.0, 6.0], [4.0, 2.0, 3.0]], [[6.0, 4.0, 3.0], [7.0, 4.0, 4.0], [6.0, 4.0, 
7.0]]], [[[5.0, 8.0, 6.0], [1.0, 5.0, 7.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 7.0], [6.0, 3.0, 4.0], [2.0, 1.0, 6.0]]], [[[3.0, 
3.0, 8.0], [4.0, 6.0, 1.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 4.0], [3.0, 7.0, 7.0], [7.0, 7.0, 4.0]]]])
      arg[:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 4.0], [1.0, 3.0, 6.0], [4.0, 2.0, 3.0]], [[6.0, 4.0, 3.0], [7.0, 4.0, 4.0], [6.0, 
4.0, 7.0]], [[3.0, 3.0, 3.0], [6.0, 3.0, 7.0], [4.0, 7.0, 1.0]]], [[[5.0, 8.0, 6.0], [1.0, 5.0, 7.0], [4.0, 6.0, 1.0]], [[5.0, 
4.0, 7.0], [6.0, 3.0, 4.0], [2.0, 1.0, 6.0]], [[4.0, 2.0, 5.0], [7.0, 6.0, 4.0], [3.0, 3.0, 3.0]]], [[[3.0, 3.0, 8.0], [4.0, 
6.0, 1.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 4.0], [3.0, 7.0, 7.0], [7.0, 7.0, 4.0]], [[6.0, 7.0, 1.0], [7.0, 8.0, 4.0], [6.0, 5.0, 
2.0]]], [[[1.0, 2.0, 5.0], [8.0, 1.0, 3.0], [3.0, 1.0, 5.0]], [[7.0, 3.0, 3.0], [6.0, 5.0, 6.0], [5.0, 1.0, 2.0]], [[2.0, 4.0, 
7.0], [2.0, 6.0, 6.0], [3.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 4.0], [1.0, 3.0, 6.0], [4.0, 2.0, 3.0]], 
[[6.0, 4.0, 3.0], [7.0, 4.0, 4.0], [6.0, 4.0, 7.0]], [[3.0, 4.0, 4.0], [1.0, 3.0, 4.0], [4.0, 5.0, 8.0]]], [[[5.0, 8.0, 6.0], 
[1.0, 5.0, 7.0], [4.0, 6.0, 1.0]], [[5.0, 4.0, 7.0], [6.0, 3.0, 4.0], [2.0, 1.0, 6.0]], [[4.0, 6.0, 5.0], [2.0, 1.0, 8.0], 
[2.0, 8.0, 7.0]]], [[[3.0, 3.0, 8.0], [4.0, 6.0, 1.0], [7.0, 3.0, 7.0]], [[3.0, 4.0, 4.0], [3.0, 7.0, 7.0], [7.0, 7.0, 4.0]], 
[[8.0, 3.0, 3.0], [6.0, 5.0, 6.0], [8.0, 6.0, 1.0]]], [[[4.0, 1.0, 7.0], [3.0, 3.0, 7.0], [2.0, 8.0, 2.0]], [[7.0, 8.0, 4.0], 
[4.0, 6.0, 6.0], [7.0, 4.0, 2.0]], [[7.0, 4.0, 4.0], [1.0, 1.0, 7.0], [4.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 8.0], [4.0, 3.0, 7.0], [2.0, 8.0, 1.0]], [[8.0, 2.0, 8.0], [3.0, 6.0, 6.0], [5.0, 
5.0, 5.0]], [[4.0, 7.0, 3.0], [7.0, 4.0, 7.0], [7.0, 8.0, 5.0]]], [[[5.0, 1.0, 6.0], [7.0, 8.0, 5.0], [5.0, 1.0, 7.0]], [[8.0, 
5.0, 8.0], [1.0, 8.0, 4.0], [1.0, 3.0, 3.0]], [[3.0, 8.0, 1.0], [4.0, 7.0, 2.0], [4.0, 2.0, 4.0]]], [[[1.0, 3.0, 3.0], [7.0, 
4.0, 3.0], [3.0, 2.0, 6.0]], [[7.0, 2.0, 8.0], [2.0, 5.0, 3.0], [1.0, 5.0, 4.0]], [[1.0, 3.0, 3.0], [8.0, 2.0, 6.0], [6.0, 4.0, 
3.0]]], [[[7.0, 2.0, 3.0], [1.0, 7.0, 4.0], [5.0, 3.0, 4.0]], [[1.0, 7.0, 8.0], [5.0, 4.0, 3.0], [6.0, 2.0, 7.0]], [[2.0, 3.0, 
1.0], [7.0, 4.0, 1.0], [5.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 7.0], [5.0, 8.0, 2.0], [6.0, 7.0, 6.0]], 
[[5.0, 1.0, 4.0], [8.0, 8.0, 4.0], [4.0, 4.0, 3.0]], [[2.0, 6.0, 7.0], [4.0, 3.0, 2.0], [3.0, 1.0, 8.0]]], [[[7.0, 1.0, 8.0], 
[5.0, 7.0, 8.0], [5.0, 2.0, 3.0]], [[6.0, 6.0, 7.0], [8.0, 8.0, 6.0], [2.0, 2.0, 2.0]], [[4.0, 7.0, 2.0], [4.0, 6.0, 8.0], 
[6.0, 8.0, 8.0]]], [[[4.0, 3.0, 5.0], [5.0, 7.0, 1.0], [5.0, 8.0, 3.0]], [[6.0, 2.0, 8.0], [3.0, 4.0, 6.0], [1.0, 3.0, 3.0]], 
[[8.0, 1.0, 7.0], [5.0, 2.0, 5.0], [7.0, 6.0, 4.0]]], [[[3.0, 8.0, 7.0], [5.0, 2.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 7.0, 3.0], 
[5.0, 5.0, 3.0], [2.0, 1.0, 5.0]], [[6.0, 5.0, 8.0], [1.0, 6.0, 7.0], [7.0, 8.0, 3.0]]]])
      val=Data(numpy.array([[[[5.0, 8.0, 1.0], [4.0, 3.0, 7.0], [8.0, 6.0, 8.0]], [[1.0, 7.0, 5.0], [1.0, 4.0, 5.0], [1.0, 5.0, 
7.0]]], [[[4.0, 8.0, 7.0], [3.0, 7.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 4.0, 8.0]]], [[[3.0, 
3.0, 2.0], [6.0, 3.0, 8.0], [2.0, 1.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 2.0], [4.0, 8.0, 8.0]]]]),self.functionspace)
      arg[:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [4.0, 3.0, 7.0], [8.0, 6.0, 8.0]], [[1.0, 7.0, 5.0], [1.0, 4.0, 5.0], [1.0, 
5.0, 7.0]], [[4.0, 7.0, 3.0], [7.0, 4.0, 7.0], [7.0, 8.0, 5.0]]], [[[4.0, 8.0, 7.0], [3.0, 7.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 
1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 4.0, 8.0]], [[3.0, 8.0, 1.0], [4.0, 7.0, 2.0], [4.0, 2.0, 4.0]]], [[[3.0, 3.0, 2.0], [6.0, 
3.0, 8.0], [2.0, 1.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 2.0], [4.0, 8.0, 8.0]], [[1.0, 3.0, 3.0], [8.0, 2.0, 6.0], [6.0, 4.0, 
3.0]]], [[[7.0, 2.0, 3.0], [1.0, 7.0, 4.0], [5.0, 3.0, 4.0]], [[1.0, 7.0, 8.0], [5.0, 4.0, 3.0], [6.0, 2.0, 7.0]], [[2.0, 3.0, 
1.0], [7.0, 4.0, 1.0], [5.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 1.0], [4.0, 3.0, 7.0], [8.0, 6.0, 8.0]], 
[[1.0, 7.0, 5.0], [1.0, 4.0, 5.0], [1.0, 5.0, 7.0]], [[2.0, 6.0, 7.0], [4.0, 3.0, 2.0], [3.0, 1.0, 8.0]]], [[[4.0, 8.0, 7.0], 
[3.0, 7.0, 7.0], [5.0, 2.0, 3.0]], [[8.0, 1.0, 6.0], [7.0, 4.0, 2.0], [7.0, 4.0, 8.0]], [[4.0, 7.0, 2.0], [4.0, 6.0, 8.0], 
[6.0, 8.0, 8.0]]], [[[3.0, 3.0, 2.0], [6.0, 3.0, 8.0], [2.0, 1.0, 8.0]], [[6.0, 4.0, 3.0], [7.0, 6.0, 2.0], [4.0, 8.0, 8.0]], 
[[8.0, 1.0, 7.0], [5.0, 2.0, 5.0], [7.0, 6.0, 4.0]]], [[[3.0, 8.0, 7.0], [5.0, 2.0, 8.0], [5.0, 5.0, 8.0]], [[1.0, 7.0, 3.0], 
[5.0, 5.0, 3.0], [2.0, 1.0, 5.0]], [[6.0, 5.0, 8.0], [1.0, 6.0, 7.0], [7.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UUCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 3.0], [8.0, 6.0, 8.0], [2.0, 6.0, 3.0]], [[7.0, 1.0, 2.0], [3.0, 8.0, 1.0], [8.0, 
4.0, 6.0]], [[3.0, 7.0, 5.0], [1.0, 7.0, 6.0], [1.0, 5.0, 3.0]]], [[[6.0, 2.0, 4.0], [4.0, 3.0, 6.0], [2.0, 7.0, 1.0]], [[5.0, 
3.0, 5.0], [7.0, 6.0, 8.0], [3.0, 5.0, 3.0]], [[3.0, 2.0, 8.0], [2.0, 3.0, 6.0], [8.0, 3.0, 4.0]]], [[[6.0, 3.0, 2.0], [2.0, 
7.0, 5.0], [8.0, 3.0, 7.0]], [[5.0, 8.0, 1.0], [4.0, 1.0, 3.0], [2.0, 3.0, 5.0]], [[4.0, 1.0, 2.0], [3.0, 4.0, 1.0], [6.0, 1.0, 
4.0]]], [[[2.0, 4.0, 6.0], [7.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[3.0, 5.0, 6.0], [2.0, 8.0, 4.0], [7.0, 7.0, 1.0]], [[2.0, 4.0, 
6.0], [8.0, 5.0, 8.0], [3.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 5.0], [4.0, 8.0, 6.0], [3.0, 7.0, 4.0]], 
[[6.0, 7.0, 5.0], [2.0, 2.0, 6.0], [2.0, 7.0, 1.0]], [[4.0, 6.0, 3.0], [1.0, 5.0, 7.0], [7.0, 7.0, 5.0]]], [[[8.0, 2.0, 7.0], 
[8.0, 8.0, 1.0], [2.0, 8.0, 8.0]], [[1.0, 3.0, 2.0], [5.0, 2.0, 1.0], [8.0, 4.0, 6.0]], [[8.0, 7.0, 2.0], [3.0, 4.0, 6.0], 
[3.0, 1.0, 4.0]]], [[[5.0, 7.0, 3.0], [3.0, 1.0, 2.0], [7.0, 3.0, 7.0]], [[5.0, 7.0, 5.0], [7.0, 4.0, 2.0], [4.0, 2.0, 2.0]], 
[[1.0, 1.0, 5.0], [7.0, 4.0, 4.0], [2.0, 3.0, 2.0]]], [[[8.0, 5.0, 1.0], [2.0, 1.0, 7.0], [7.0, 7.0, 7.0]], [[4.0, 2.0, 7.0], 
[8.0, 5.0, 6.0], [3.0, 3.0, 3.0]], [[3.0, 2.0, 7.0], [3.0, 6.0, 1.0], [6.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 6.0, 2.0], [4.0, 4.0, 4.0], [2.0, 7.0, 2.0]], [[6.0, 1.0, 3.0], [1.0, 7.0, 5.0], [3.0, 
8.0, 7.0]]], [[[4.0, 3.0, 7.0], [3.0, 2.0, 4.0], [5.0, 8.0, 5.0]], [[8.0, 7.0, 7.0], [6.0, 6.0, 6.0], [3.0, 2.0, 1.0]]], 
[[[7.0, 2.0, 7.0], [1.0, 4.0, 5.0], [5.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [5.0, 1.0, 3.0], [2.0, 7.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 8.0, 8.0], [3.0, 8.0, 1.0], [4.0, 5.0, 5.0]], [[5.0, 8.0, 5.0], [2.0, 7.0, 5.0], 
[5.0, 7.0, 6.0]]], [[[4.0, 5.0, 1.0], [2.0, 6.0, 7.0], [7.0, 5.0, 4.0]], [[2.0, 6.0, 8.0], [4.0, 5.0, 2.0], [6.0, 3.0, 6.0]]], 
[[[7.0, 6.0, 8.0], [7.0, 2.0, 7.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 4.0], [3.0, 2.0, 1.0], [7.0, 3.0, 7.0]]]])
      arg[:3,:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 2.0], [4.0, 4.0, 4.0], [2.0, 7.0, 2.0]], [[6.0, 1.0, 3.0], [1.0, 7.0, 5.0], [3.0, 
8.0, 7.0]], [[3.0, 7.0, 5.0], [1.0, 7.0, 6.0], [1.0, 5.0, 3.0]]], [[[4.0, 3.0, 7.0], [3.0, 2.0, 4.0], [5.0, 8.0, 5.0]], [[8.0, 
7.0, 7.0], [6.0, 6.0, 6.0], [3.0, 2.0, 1.0]], [[3.0, 2.0, 8.0], [2.0, 3.0, 6.0], [8.0, 3.0, 4.0]]], [[[7.0, 2.0, 7.0], [1.0, 
4.0, 5.0], [5.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [5.0, 1.0, 3.0], [2.0, 7.0, 5.0]], [[4.0, 1.0, 2.0], [3.0, 4.0, 1.0], [6.0, 1.0, 
4.0]]], [[[2.0, 4.0, 6.0], [7.0, 8.0, 8.0], [4.0, 5.0, 1.0]], [[3.0, 5.0, 6.0], [2.0, 8.0, 4.0], [7.0, 7.0, 1.0]], [[2.0, 4.0, 
6.0], [8.0, 5.0, 8.0], [3.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 8.0], [3.0, 8.0, 1.0], [4.0, 5.0, 5.0]], 
[[5.0, 8.0, 5.0], [2.0, 7.0, 5.0], [5.0, 7.0, 6.0]], [[4.0, 6.0, 3.0], [1.0, 5.0, 7.0], [7.0, 7.0, 5.0]]], [[[4.0, 5.0, 1.0], 
[2.0, 6.0, 7.0], [7.0, 5.0, 4.0]], [[2.0, 6.0, 8.0], [4.0, 5.0, 2.0], [6.0, 3.0, 6.0]], [[8.0, 7.0, 2.0], [3.0, 4.0, 6.0], 
[3.0, 1.0, 4.0]]], [[[7.0, 6.0, 8.0], [7.0, 2.0, 7.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 4.0], [3.0, 2.0, 1.0], [7.0, 3.0, 7.0]], 
[[1.0, 1.0, 5.0], [7.0, 4.0, 4.0], [2.0, 3.0, 2.0]]], [[[8.0, 5.0, 1.0], [2.0, 1.0, 7.0], [7.0, 7.0, 7.0]], [[4.0, 2.0, 7.0], 
[8.0, 5.0, 6.0], [3.0, 3.0, 3.0]], [[3.0, 2.0, 7.0], [3.0, 6.0, 1.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 1.0], [7.0, 2.0, 3.0], [7.0, 1.0, 2.0]], [[6.0, 8.0, 5.0], [2.0, 1.0, 2.0], [7.0, 
5.0, 1.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 6.0], [5.0, 6.0, 7.0]]], [[[7.0, 5.0, 5.0], [7.0, 2.0, 8.0], [2.0, 6.0, 3.0]], [[3.0, 
4.0, 7.0], [7.0, 7.0, 2.0], [2.0, 7.0, 8.0]], [[4.0, 6.0, 1.0], [2.0, 7.0, 4.0], [8.0, 8.0, 8.0]]], [[[2.0, 8.0, 8.0], [5.0, 
6.0, 6.0], [4.0, 6.0, 8.0]], [[1.0, 5.0, 8.0], [6.0, 1.0, 2.0], [4.0, 1.0, 2.0]], [[8.0, 3.0, 1.0], [4.0, 2.0, 3.0], [4.0, 6.0, 
3.0]]], [[[7.0, 5.0, 8.0], [8.0, 2.0, 2.0], [1.0, 8.0, 7.0]], [[6.0, 5.0, 4.0], [4.0, 7.0, 4.0], [5.0, 1.0, 2.0]], [[4.0, 6.0, 
1.0], [8.0, 5.0, 2.0], [3.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 8.0], [2.0, 3.0, 5.0], [2.0, 3.0, 7.0]], 
[[2.0, 4.0, 5.0], [3.0, 7.0, 3.0], [5.0, 4.0, 5.0]], [[6.0, 1.0, 3.0], [5.0, 4.0, 5.0], [4.0, 2.0, 3.0]]], [[[6.0, 8.0, 1.0], 
[4.0, 1.0, 8.0], [6.0, 7.0, 3.0]], [[6.0, 5.0, 1.0], [5.0, 7.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 3.0, 8.0], [8.0, 8.0, 1.0], 
[7.0, 4.0, 4.0]]], [[[4.0, 4.0, 1.0], [3.0, 2.0, 4.0], [2.0, 4.0, 3.0]], [[4.0, 8.0, 5.0], [6.0, 4.0, 8.0], [5.0, 5.0, 3.0]], 
[[1.0, 3.0, 7.0], [7.0, 3.0, 7.0], [4.0, 1.0, 8.0]]], [[[2.0, 3.0, 7.0], [4.0, 3.0, 2.0], [7.0, 2.0, 5.0]], [[6.0, 8.0, 8.0], 
[4.0, 6.0, 5.0], [5.0, 2.0, 2.0]], [[5.0, 1.0, 1.0], [7.0, 1.0, 6.0], [6.0, 6.0, 3.0]]]])
      val=numpy.array([[[[3.0, 7.0, 7.0], [4.0, 7.0, 8.0], [8.0, 6.0, 5.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 1.0], [4.0, 2.0, 
5.0]], [[6.0, 7.0, 2.0], [6.0, 7.0, 7.0], [3.0, 3.0, 1.0]]], [[[1.0, 5.0, 2.0], [3.0, 5.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 5.0, 
6.0], [8.0, 1.0, 3.0], [7.0, 4.0, 8.0]], [[5.0, 2.0, 8.0], [7.0, 2.0, 2.0], [7.0, 4.0, 4.0]]]])
      arg[2:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [7.0, 2.0, 3.0], [7.0, 1.0, 2.0]], [[6.0, 8.0, 5.0], [2.0, 1.0, 2.0], [7.0, 
5.0, 1.0]], [[8.0, 3.0, 1.0], [8.0, 7.0, 6.0], [5.0, 6.0, 7.0]]], [[[7.0, 5.0, 5.0], [7.0, 2.0, 8.0], [2.0, 6.0, 3.0]], [[3.0, 
4.0, 7.0], [7.0, 7.0, 2.0], [2.0, 7.0, 8.0]], [[4.0, 6.0, 1.0], [2.0, 7.0, 4.0], [8.0, 8.0, 8.0]]], [[[3.0, 7.0, 7.0], [4.0, 
7.0, 8.0], [8.0, 6.0, 5.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 1.0], [4.0, 2.0, 5.0]], [[6.0, 7.0, 2.0], [6.0, 7.0, 7.0], [3.0, 3.0, 
1.0]]], [[[1.0, 5.0, 2.0], [3.0, 5.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 5.0, 6.0], [8.0, 1.0, 3.0], [7.0, 4.0, 8.0]], [[5.0, 2.0, 
8.0], [7.0, 2.0, 2.0], [7.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 8.0], [2.0, 3.0, 5.0], [2.0, 3.0, 7.0]], 
[[2.0, 4.0, 5.0], [3.0, 7.0, 3.0], [5.0, 4.0, 5.0]], [[6.0, 1.0, 3.0], [5.0, 4.0, 5.0], [4.0, 2.0, 3.0]]], [[[6.0, 8.0, 1.0], 
[4.0, 1.0, 8.0], [6.0, 7.0, 3.0]], [[6.0, 5.0, 1.0], [5.0, 7.0, 1.0], [5.0, 7.0, 6.0]], [[5.0, 3.0, 8.0], [8.0, 8.0, 1.0], 
[7.0, 4.0, 4.0]]], [[[3.0, 7.0, 7.0], [4.0, 7.0, 8.0], [8.0, 6.0, 5.0]], [[7.0, 2.0, 4.0], [8.0, 8.0, 1.0], [4.0, 2.0, 5.0]], 
[[6.0, 7.0, 2.0], [6.0, 7.0, 7.0], [3.0, 3.0, 1.0]]], [[[1.0, 5.0, 2.0], [3.0, 5.0, 8.0], [2.0, 4.0, 1.0]], [[4.0, 5.0, 6.0], 
[8.0, 1.0, 3.0], [7.0, 4.0, 8.0]], [[5.0, 2.0, 8.0], [7.0, 2.0, 2.0], [7.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 7.0], [7.0, 4.0, 6.0], [7.0, 2.0, 3.0]], [[4.0, 2.0, 5.0], [3.0, 4.0, 8.0], [8.0, 
5.0, 8.0]], [[1.0, 2.0, 6.0], [1.0, 6.0, 6.0], [8.0, 8.0, 4.0]]], [[[3.0, 4.0, 8.0], [2.0, 4.0, 6.0], [6.0, 1.0, 1.0]], [[1.0, 
2.0, 6.0], [3.0, 2.0, 7.0], [1.0, 4.0, 4.0]], [[8.0, 7.0, 7.0], [4.0, 6.0, 8.0], [5.0, 6.0, 7.0]]], [[[6.0, 4.0, 2.0], [6.0, 
8.0, 6.0], [2.0, 3.0, 8.0]], [[2.0, 2.0, 7.0], [2.0, 8.0, 7.0], [8.0, 3.0, 8.0]], [[1.0, 3.0, 2.0], [7.0, 2.0, 4.0], [7.0, 6.0, 
6.0]]], [[[8.0, 6.0, 5.0], [5.0, 1.0, 3.0], [3.0, 8.0, 4.0]], [[1.0, 8.0, 1.0], [1.0, 5.0, 1.0], [8.0, 1.0, 7.0]], [[8.0, 7.0, 
5.0], [6.0, 4.0, 3.0], [1.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 7.0], [4.0, 3.0, 8.0], [2.0, 4.0, 5.0]], 
[[8.0, 6.0, 4.0], [3.0, 6.0, 7.0], [2.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [1.0, 5.0, 3.0], [8.0, 8.0, 4.0]]], [[[4.0, 8.0, 8.0], 
[4.0, 8.0, 6.0], [1.0, 5.0, 2.0]], [[5.0, 7.0, 2.0], [7.0, 7.0, 1.0], [2.0, 2.0, 7.0]], [[2.0, 4.0, 8.0], [5.0, 7.0, 8.0], 
[7.0, 1.0, 7.0]]], [[[4.0, 5.0, 3.0], [1.0, 4.0, 4.0], [1.0, 6.0, 8.0]], [[2.0, 6.0, 2.0], [8.0, 4.0, 6.0], [6.0, 4.0, 1.0]], 
[[1.0, 6.0, 2.0], [6.0, 6.0, 6.0], [3.0, 4.0, 4.0]]], [[[5.0, 3.0, 1.0], [7.0, 4.0, 5.0], [3.0, 8.0, 8.0]], [[3.0, 1.0, 2.0], 
[6.0, 5.0, 3.0], [5.0, 6.0, 6.0]], [[2.0, 1.0, 8.0], [5.0, 8.0, 6.0], [8.0, 1.0, 4.0]]]])
      val=Data(numpy.array([[[[2.0, 4.0, 2.0], [4.0, 5.0, 1.0], [5.0, 8.0, 1.0]], [[3.0, 3.0, 2.0], [4.0, 2.0, 8.0], [6.0, 5.0, 
6.0]], [[5.0, 7.0, 7.0], [6.0, 3.0, 2.0], [4.0, 5.0, 3.0]]], [[[4.0, 5.0, 7.0], [3.0, 1.0, 7.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 
6.0], [2.0, 6.0, 4.0], [7.0, 2.0, 4.0]], [[5.0, 8.0, 5.0], [2.0, 4.0, 3.0], [8.0, 5.0, 2.0]]]]),self.functionspace)
      arg[2:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 7.0], [7.0, 4.0, 6.0], [7.0, 2.0, 3.0]], [[4.0, 2.0, 5.0], [3.0, 4.0, 8.0], [8.0, 
5.0, 8.0]], [[1.0, 2.0, 6.0], [1.0, 6.0, 6.0], [8.0, 8.0, 4.0]]], [[[3.0, 4.0, 8.0], [2.0, 4.0, 6.0], [6.0, 1.0, 1.0]], [[1.0, 
2.0, 6.0], [3.0, 2.0, 7.0], [1.0, 4.0, 4.0]], [[8.0, 7.0, 7.0], [4.0, 6.0, 8.0], [5.0, 6.0, 7.0]]], [[[2.0, 4.0, 2.0], [4.0, 
5.0, 1.0], [5.0, 8.0, 1.0]], [[3.0, 3.0, 2.0], [4.0, 2.0, 8.0], [6.0, 5.0, 6.0]], [[5.0, 7.0, 7.0], [6.0, 3.0, 2.0], [4.0, 5.0, 
3.0]]], [[[4.0, 5.0, 7.0], [3.0, 1.0, 7.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 6.0], [2.0, 6.0, 4.0], [7.0, 2.0, 4.0]], [[5.0, 8.0, 
5.0], [2.0, 4.0, 3.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 7.0], [4.0, 3.0, 8.0], [2.0, 4.0, 5.0]], 
[[8.0, 6.0, 4.0], [3.0, 6.0, 7.0], [2.0, 4.0, 6.0]], [[8.0, 2.0, 6.0], [1.0, 5.0, 3.0], [8.0, 8.0, 4.0]]], [[[4.0, 8.0, 8.0], 
[4.0, 8.0, 6.0], [1.0, 5.0, 2.0]], [[5.0, 7.0, 2.0], [7.0, 7.0, 1.0], [2.0, 2.0, 7.0]], [[2.0, 4.0, 8.0], [5.0, 7.0, 8.0], 
[7.0, 1.0, 7.0]]], [[[2.0, 4.0, 2.0], [4.0, 5.0, 1.0], [5.0, 8.0, 1.0]], [[3.0, 3.0, 2.0], [4.0, 2.0, 8.0], [6.0, 5.0, 6.0]], 
[[5.0, 7.0, 7.0], [6.0, 3.0, 2.0], [4.0, 5.0, 3.0]]], [[[4.0, 5.0, 7.0], [3.0, 1.0, 7.0], [7.0, 3.0, 8.0]], [[7.0, 3.0, 6.0], 
[2.0, 6.0, 4.0], [7.0, 2.0, 4.0]], [[5.0, 8.0, 5.0], [2.0, 4.0, 3.0], [8.0, 5.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 2.0], [2.0, 4.0, 6.0], [3.0, 3.0, 2.0]], [[6.0, 2.0, 2.0], [8.0, 8.0, 1.0], [8.0, 
8.0, 8.0]], [[5.0, 5.0, 7.0], [2.0, 1.0, 1.0], [2.0, 5.0, 6.0]]], [[[1.0, 2.0, 1.0], [7.0, 8.0, 4.0], [1.0, 1.0, 6.0]], [[7.0, 
2.0, 7.0], [3.0, 8.0, 8.0], [6.0, 5.0, 5.0]], [[7.0, 6.0, 4.0], [5.0, 7.0, 4.0], [1.0, 5.0, 4.0]]], [[[1.0, 2.0, 1.0], [2.0, 
3.0, 6.0], [1.0, 6.0, 8.0]], [[6.0, 8.0, 5.0], [3.0, 5.0, 8.0], [4.0, 3.0, 6.0]], [[8.0, 8.0, 1.0], [8.0, 1.0, 3.0], [6.0, 8.0, 
6.0]]], [[[4.0, 2.0, 2.0], [7.0, 4.0, 3.0], [5.0, 2.0, 4.0]], [[8.0, 8.0, 3.0], [4.0, 3.0, 6.0], [7.0, 3.0, 2.0]], [[8.0, 1.0, 
8.0], [6.0, 3.0, 5.0], [7.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 2.0], [4.0, 1.0, 2.0], [1.0, 6.0, 5.0]], 
[[8.0, 8.0, 5.0], [4.0, 2.0, 7.0], [4.0, 8.0, 4.0]], [[7.0, 8.0, 3.0], [3.0, 1.0, 1.0], [8.0, 8.0, 3.0]]], [[[4.0, 4.0, 1.0], 
[1.0, 7.0, 7.0], [8.0, 3.0, 1.0]], [[7.0, 8.0, 6.0], [4.0, 7.0, 6.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [1.0, 5.0, 5.0], 
[7.0, 5.0, 4.0]]], [[[6.0, 6.0, 5.0], [5.0, 3.0, 6.0], [4.0, 4.0, 6.0]], [[3.0, 1.0, 5.0], [7.0, 2.0, 2.0], [1.0, 8.0, 4.0]], 
[[6.0, 4.0, 6.0], [5.0, 8.0, 7.0], [7.0, 6.0, 6.0]]], [[[8.0, 8.0, 8.0], [6.0, 2.0, 3.0], [6.0, 5.0, 1.0]], [[4.0, 7.0, 5.0], 
[4.0, 7.0, 1.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 1.0], [4.0, 2.0, 4.0], [3.0, 3.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 6.0, 8.0], [5.0, 3.0, 6.0], [1.0, 2.0, 2.0]], [[2.0, 5.0, 1.0], [6.0, 7.0, 1.0], [7.0, 
6.0, 1.0]], [[1.0, 7.0, 2.0], [2.0, 4.0, 4.0], [1.0, 6.0, 3.0]]], [[[8.0, 1.0, 4.0], [5.0, 6.0, 6.0], [6.0, 6.0, 4.0]], [[7.0, 
3.0, 6.0], [8.0, 1.0, 6.0], [1.0, 7.0, 3.0]], [[3.0, 3.0, 2.0], [5.0, 4.0, 8.0], [3.0, 5.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 6.0, 5.0], [5.0, 1.0, 4.0], [2.0, 7.0, 7.0]], [[3.0, 3.0, 3.0], [7.0, 5.0, 3.0], 
[5.0, 2.0, 5.0]], [[3.0, 1.0, 8.0], [3.0, 6.0, 7.0], [6.0, 3.0, 4.0]]], [[[7.0, 7.0, 7.0], [4.0, 8.0, 5.0], [5.0, 7.0, 8.0]], 
[[8.0, 6.0, 3.0], [5.0, 3.0, 5.0], [8.0, 3.0, 8.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 7.0], [1.0, 5.0, 4.0]]]])
      arg[2:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 2.0], [2.0, 4.0, 6.0], [3.0, 3.0, 2.0]], [[6.0, 2.0, 2.0], [8.0, 8.0, 1.0], [8.0, 
8.0, 8.0]], [[5.0, 5.0, 7.0], [2.0, 1.0, 1.0], [2.0, 5.0, 6.0]]], [[[1.0, 2.0, 1.0], [7.0, 8.0, 4.0], [1.0, 1.0, 6.0]], [[7.0, 
2.0, 7.0], [3.0, 8.0, 8.0], [6.0, 5.0, 5.0]], [[7.0, 6.0, 4.0], [5.0, 7.0, 4.0], [1.0, 5.0, 4.0]]], [[[1.0, 6.0, 8.0], [5.0, 
3.0, 6.0], [1.0, 2.0, 2.0]], [[2.0, 5.0, 1.0], [6.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[1.0, 7.0, 2.0], [2.0, 4.0, 4.0], [1.0, 6.0, 
3.0]]], [[[8.0, 1.0, 4.0], [5.0, 6.0, 6.0], [6.0, 6.0, 4.0]], [[7.0, 3.0, 6.0], [8.0, 1.0, 6.0], [1.0, 7.0, 3.0]], [[3.0, 3.0, 
2.0], [5.0, 4.0, 8.0], [3.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 2.0], [4.0, 1.0, 2.0], [1.0, 6.0, 5.0]], 
[[8.0, 8.0, 5.0], [4.0, 2.0, 7.0], [4.0, 8.0, 4.0]], [[7.0, 8.0, 3.0], [3.0, 1.0, 1.0], [8.0, 8.0, 3.0]]], [[[4.0, 4.0, 1.0], 
[1.0, 7.0, 7.0], [8.0, 3.0, 1.0]], [[7.0, 8.0, 6.0], [4.0, 7.0, 6.0], [3.0, 2.0, 2.0]], [[4.0, 4.0, 4.0], [1.0, 5.0, 5.0], 
[7.0, 5.0, 4.0]]], [[[1.0, 6.0, 5.0], [5.0, 1.0, 4.0], [2.0, 7.0, 7.0]], [[3.0, 3.0, 3.0], [7.0, 5.0, 3.0], [5.0, 2.0, 5.0]], 
[[3.0, 1.0, 8.0], [3.0, 6.0, 7.0], [6.0, 3.0, 4.0]]], [[[7.0, 7.0, 7.0], [4.0, 8.0, 5.0], [5.0, 7.0, 8.0]], [[8.0, 6.0, 3.0], 
[5.0, 3.0, 5.0], [8.0, 3.0, 8.0]], [[4.0, 1.0, 4.0], [5.0, 2.0, 7.0], [1.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 7.0], [7.0, 3.0, 6.0], [1.0, 2.0, 3.0]], [[7.0, 8.0, 3.0], [2.0, 3.0, 2.0], [2.0, 
2.0, 6.0]], [[4.0, 8.0, 2.0], [4.0, 6.0, 3.0], [8.0, 8.0, 8.0]]], [[[6.0, 5.0, 8.0], [3.0, 4.0, 2.0], [7.0, 3.0, 6.0]], [[2.0, 
8.0, 5.0], [6.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [1.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[6.0, 2.0, 3.0], [5.0, 
1.0, 3.0], [1.0, 2.0, 2.0]], [[3.0, 1.0, 4.0], [8.0, 1.0, 4.0], [3.0, 8.0, 6.0]], [[6.0, 4.0, 5.0], [1.0, 1.0, 4.0], [6.0, 8.0, 
2.0]]], [[[6.0, 4.0, 5.0], [4.0, 7.0, 2.0], [2.0, 2.0, 2.0]], [[1.0, 8.0, 1.0], [4.0, 6.0, 1.0], [5.0, 1.0, 3.0]], [[4.0, 7.0, 
5.0], [8.0, 1.0, 1.0], [5.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 5.0], [7.0, 2.0, 6.0], [1.0, 5.0, 2.0]], 
[[8.0, 6.0, 6.0], [8.0, 3.0, 7.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 6.0], [4.0, 6.0, 4.0], [1.0, 7.0, 5.0]]], [[[7.0, 4.0, 8.0], 
[1.0, 6.0, 4.0], [6.0, 5.0, 7.0]], [[8.0, 2.0, 2.0], [4.0, 3.0, 3.0], [2.0, 4.0, 1.0]], [[4.0, 2.0, 5.0], [8.0, 4.0, 6.0], 
[5.0, 2.0, 3.0]]], [[[4.0, 4.0, 7.0], [3.0, 1.0, 3.0], [5.0, 1.0, 1.0]], [[3.0, 2.0, 1.0], [8.0, 5.0, 6.0], [1.0, 7.0, 6.0]], 
[[2.0, 5.0, 4.0], [4.0, 8.0, 4.0], [2.0, 8.0, 7.0]]], [[[7.0, 1.0, 2.0], [2.0, 5.0, 7.0], [2.0, 7.0, 6.0]], [[4.0, 5.0, 5.0], 
[7.0, 1.0, 4.0], [8.0, 7.0, 8.0]], [[1.0, 3.0, 5.0], [4.0, 4.0, 2.0], [6.0, 4.0, 2.0]]]])
      val=numpy.array([[[[6.0, 1.0, 8.0]]], [[[1.0, 3.0, 7.0]]], [[[7.0, 1.0, 5.0]]], [[[6.0, 3.0, 5.0]]]])
      arg[:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 7.0], [7.0, 3.0, 6.0], [1.0, 2.0, 3.0]], [[7.0, 8.0, 3.0], [6.0, 1.0, 8.0], [2.0, 
2.0, 6.0]], [[4.0, 8.0, 2.0], [4.0, 6.0, 3.0], [8.0, 8.0, 8.0]]], [[[6.0, 5.0, 8.0], [3.0, 4.0, 2.0], [7.0, 3.0, 6.0]], [[2.0, 
8.0, 5.0], [1.0, 3.0, 7.0], [4.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [1.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[6.0, 2.0, 3.0], [5.0, 
1.0, 3.0], [1.0, 2.0, 2.0]], [[3.0, 1.0, 4.0], [7.0, 1.0, 5.0], [3.0, 8.0, 6.0]], [[6.0, 4.0, 5.0], [1.0, 1.0, 4.0], [6.0, 8.0, 
2.0]]], [[[6.0, 4.0, 5.0], [4.0, 7.0, 2.0], [2.0, 2.0, 2.0]], [[1.0, 8.0, 1.0], [6.0, 3.0, 5.0], [5.0, 1.0, 3.0]], [[4.0, 7.0, 
5.0], [8.0, 1.0, 1.0], [5.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [7.0, 2.0, 6.0], [1.0, 5.0, 2.0]], 
[[8.0, 6.0, 6.0], [6.0, 1.0, 8.0], [5.0, 3.0, 5.0]], [[2.0, 1.0, 6.0], [4.0, 6.0, 4.0], [1.0, 7.0, 5.0]]], [[[7.0, 4.0, 8.0], 
[1.0, 6.0, 4.0], [6.0, 5.0, 7.0]], [[8.0, 2.0, 2.0], [1.0, 3.0, 7.0], [2.0, 4.0, 1.0]], [[4.0, 2.0, 5.0], [8.0, 4.0, 6.0], 
[5.0, 2.0, 3.0]]], [[[4.0, 4.0, 7.0], [3.0, 1.0, 3.0], [5.0, 1.0, 1.0]], [[3.0, 2.0, 1.0], [7.0, 1.0, 5.0], [1.0, 7.0, 6.0]], 
[[2.0, 5.0, 4.0], [4.0, 8.0, 4.0], [2.0, 8.0, 7.0]]], [[[7.0, 1.0, 2.0], [2.0, 5.0, 7.0], [2.0, 7.0, 6.0]], [[4.0, 5.0, 5.0], 
[6.0, 3.0, 5.0], [8.0, 7.0, 8.0]], [[1.0, 3.0, 5.0], [4.0, 4.0, 2.0], [6.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 1.0], [3.0, 8.0, 6.0], [4.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [2.0, 6.0, 5.0], [1.0, 
7.0, 5.0]], [[8.0, 8.0, 6.0], [5.0, 8.0, 8.0], [4.0, 1.0, 2.0]]], [[[4.0, 5.0, 7.0], [1.0, 1.0, 5.0], [8.0, 8.0, 2.0]], [[1.0, 
7.0, 4.0], [5.0, 6.0, 5.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 2.0], [3.0, 3.0, 8.0], [5.0, 5.0, 2.0]]], [[[4.0, 6.0, 6.0], [6.0, 
4.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 3.0, 6.0], [3.0, 3.0, 2.0], [5.0, 5.0, 3.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 5.0], [5.0, 4.0, 
8.0]]], [[[7.0, 3.0, 7.0], [8.0, 7.0, 6.0], [6.0, 2.0, 5.0]], [[4.0, 2.0, 1.0], [6.0, 7.0, 2.0], [2.0, 2.0, 4.0]], [[3.0, 1.0, 
8.0], [1.0, 6.0, 3.0], [2.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 2.0, 7.0], [4.0, 7.0, 8.0], [7.0, 3.0, 6.0]], 
[[2.0, 7.0, 3.0], [7.0, 1.0, 8.0], [2.0, 2.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 6.0, 5.0], [5.0, 1.0, 2.0]]], [[[2.0, 1.0, 3.0], 
[8.0, 2.0, 1.0], [3.0, 8.0, 3.0]], [[3.0, 4.0, 7.0], [1.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[2.0, 5.0, 8.0], [5.0, 2.0, 8.0], 
[8.0, 5.0, 6.0]]], [[[5.0, 6.0, 2.0], [7.0, 8.0, 3.0], [1.0, 1.0, 7.0]], [[3.0, 4.0, 5.0], [2.0, 4.0, 1.0], [3.0, 7.0, 8.0]], 
[[3.0, 1.0, 7.0], [3.0, 8.0, 3.0], [6.0, 1.0, 2.0]]], [[[1.0, 3.0, 1.0], [6.0, 5.0, 6.0], [6.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], 
[5.0, 2.0, 5.0], [4.0, 8.0, 2.0]], [[2.0, 3.0, 6.0], [8.0, 6.0, 8.0], [7.0, 4.0, 8.0]]]])
      val=Data(numpy.array([[[[3.0, 4.0, 2.0]]], [[[8.0, 8.0, 6.0]]], [[[6.0, 1.0, 5.0]]], [[[1.0, 1.0, 
7.0]]]]),self.functionspace)
      arg[:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 1.0], [3.0, 8.0, 6.0], [4.0, 2.0, 1.0]], [[1.0, 8.0, 7.0], [3.0, 4.0, 2.0], [1.0, 
7.0, 5.0]], [[8.0, 8.0, 6.0], [5.0, 8.0, 8.0], [4.0, 1.0, 2.0]]], [[[4.0, 5.0, 7.0], [1.0, 1.0, 5.0], [8.0, 8.0, 2.0]], [[1.0, 
7.0, 4.0], [8.0, 8.0, 6.0], [1.0, 6.0, 3.0]], [[7.0, 8.0, 2.0], [3.0, 3.0, 8.0], [5.0, 5.0, 2.0]]], [[[4.0, 6.0, 6.0], [6.0, 
4.0, 8.0], [3.0, 5.0, 7.0]], [[1.0, 3.0, 6.0], [6.0, 1.0, 5.0], [5.0, 5.0, 3.0]], [[6.0, 1.0, 3.0], [7.0, 1.0, 5.0], [5.0, 4.0, 
8.0]]], [[[7.0, 3.0, 7.0], [8.0, 7.0, 6.0], [6.0, 2.0, 5.0]], [[4.0, 2.0, 1.0], [1.0, 1.0, 7.0], [2.0, 2.0, 4.0]], [[3.0, 1.0, 
8.0], [1.0, 6.0, 3.0], [2.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 2.0, 7.0], [4.0, 7.0, 8.0], [7.0, 3.0, 6.0]], 
[[2.0, 7.0, 3.0], [3.0, 4.0, 2.0], [2.0, 2.0, 4.0]], [[3.0, 3.0, 7.0], [8.0, 6.0, 5.0], [5.0, 1.0, 2.0]]], [[[2.0, 1.0, 3.0], 
[8.0, 2.0, 1.0], [3.0, 8.0, 3.0]], [[3.0, 4.0, 7.0], [8.0, 8.0, 6.0], [6.0, 1.0, 5.0]], [[2.0, 5.0, 8.0], [5.0, 2.0, 8.0], 
[8.0, 5.0, 6.0]]], [[[5.0, 6.0, 2.0], [7.0, 8.0, 3.0], [1.0, 1.0, 7.0]], [[3.0, 4.0, 5.0], [6.0, 1.0, 5.0], [3.0, 7.0, 8.0]], 
[[3.0, 1.0, 7.0], [3.0, 8.0, 3.0], [6.0, 1.0, 2.0]]], [[[1.0, 3.0, 1.0], [6.0, 5.0, 6.0], [6.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], 
[1.0, 1.0, 7.0], [4.0, 8.0, 2.0]], [[2.0, 3.0, 6.0], [8.0, 6.0, 8.0], [7.0, 4.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 8.0], [6.0, 6.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 2.0], [7.0, 3.0, 7.0], [1.0, 
8.0, 2.0]], [[7.0, 5.0, 6.0], [4.0, 2.0, 4.0], [5.0, 5.0, 5.0]]], [[[8.0, 8.0, 3.0], [3.0, 2.0, 5.0], [3.0, 7.0, 7.0]], [[2.0, 
2.0, 1.0], [1.0, 8.0, 5.0], [2.0, 1.0, 2.0]], [[2.0, 3.0, 8.0], [3.0, 6.0, 7.0], [3.0, 7.0, 6.0]]], [[[5.0, 5.0, 6.0], [4.0, 
4.0, 8.0], [2.0, 1.0, 8.0]], [[1.0, 3.0, 8.0], [4.0, 8.0, 5.0], [3.0, 2.0, 2.0]], [[7.0, 1.0, 1.0], [6.0, 6.0, 4.0], [8.0, 1.0, 
3.0]]], [[[5.0, 7.0, 6.0], [3.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[1.0, 7.0, 6.0], [6.0, 5.0, 1.0], [3.0, 5.0, 5.0]], [[5.0, 6.0, 
1.0], [5.0, 1.0, 7.0], [2.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 8.0], [2.0, 1.0, 6.0], [1.0, 6.0, 2.0]], 
[[6.0, 2.0, 1.0], [2.0, 6.0, 7.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 7.0], [8.0, 2.0, 7.0], [5.0, 5.0, 3.0]]], [[[4.0, 6.0, 3.0], 
[7.0, 6.0, 3.0], [6.0, 6.0, 4.0]], [[7.0, 6.0, 1.0], [4.0, 7.0, 1.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], [8.0, 2.0, 2.0], 
[8.0, 5.0, 4.0]]], [[[1.0, 2.0, 3.0], [5.0, 7.0, 4.0], [6.0, 8.0, 3.0]], [[8.0, 8.0, 4.0], [3.0, 1.0, 5.0], [1.0, 5.0, 8.0]], 
[[1.0, 1.0, 2.0], [5.0, 3.0, 7.0], [2.0, 3.0, 1.0]]], [[[5.0, 3.0, 6.0], [3.0, 6.0, 2.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 2.0], 
[2.0, 2.0, 5.0], [3.0, 4.0, 3.0]], [[4.0, 6.0, 5.0], [2.0, 7.0, 4.0], [4.0, 8.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 3.0, 4.0]]], [[[1.0, 8.0, 1.0]]], [[[2.0, 6.0, 8.0]]], [[[3.0, 3.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 1.0, 7.0]]], [[[1.0, 1.0, 5.0]]], [[[5.0, 1.0, 3.0]]], [[[1.0, 4.0, 3.0]]]])
      arg[:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [6.0, 6.0, 4.0], [7.0, 8.0, 5.0]], [[6.0, 5.0, 2.0], [2.0, 3.0, 4.0], [1.0, 
8.0, 2.0]], [[7.0, 5.0, 6.0], [4.0, 2.0, 4.0], [5.0, 5.0, 5.0]]], [[[8.0, 8.0, 3.0], [3.0, 2.0, 5.0], [3.0, 7.0, 7.0]], [[2.0, 
2.0, 1.0], [1.0, 8.0, 1.0], [2.0, 1.0, 2.0]], [[2.0, 3.0, 8.0], [3.0, 6.0, 7.0], [3.0, 7.0, 6.0]]], [[[5.0, 5.0, 6.0], [4.0, 
4.0, 8.0], [2.0, 1.0, 8.0]], [[1.0, 3.0, 8.0], [2.0, 6.0, 8.0], [3.0, 2.0, 2.0]], [[7.0, 1.0, 1.0], [6.0, 6.0, 4.0], [8.0, 1.0, 
3.0]]], [[[5.0, 7.0, 6.0], [3.0, 3.0, 3.0], [5.0, 4.0, 7.0]], [[1.0, 7.0, 6.0], [3.0, 3.0, 5.0], [3.0, 5.0, 5.0]], [[5.0, 6.0, 
1.0], [5.0, 1.0, 7.0], [2.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 8.0], [2.0, 1.0, 6.0], [1.0, 6.0, 2.0]], 
[[6.0, 2.0, 1.0], [4.0, 1.0, 7.0], [8.0, 1.0, 4.0]], [[8.0, 5.0, 7.0], [8.0, 2.0, 7.0], [5.0, 5.0, 3.0]]], [[[4.0, 6.0, 3.0], 
[7.0, 6.0, 3.0], [6.0, 6.0, 4.0]], [[7.0, 6.0, 1.0], [1.0, 1.0, 5.0], [8.0, 8.0, 1.0]], [[2.0, 1.0, 7.0], [8.0, 2.0, 2.0], 
[8.0, 5.0, 4.0]]], [[[1.0, 2.0, 3.0], [5.0, 7.0, 4.0], [6.0, 8.0, 3.0]], [[8.0, 8.0, 4.0], [5.0, 1.0, 3.0], [1.0, 5.0, 8.0]], 
[[1.0, 1.0, 2.0], [5.0, 3.0, 7.0], [2.0, 3.0, 1.0]]], [[[5.0, 3.0, 6.0], [3.0, 6.0, 2.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 2.0], 
[1.0, 4.0, 3.0], [3.0, 4.0, 3.0]], [[4.0, 6.0, 5.0], [2.0, 7.0, 4.0], [4.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 6.0, 2.0], [3.0, 5.0, 4.0], [3.0, 8.0, 6.0]], [[2.0, 5.0, 3.0], [8.0, 7.0, 6.0], [8.0, 
6.0, 6.0]], [[2.0, 2.0, 6.0], [5.0, 1.0, 5.0], [5.0, 7.0, 6.0]]], [[[6.0, 7.0, 4.0], [8.0, 5.0, 3.0], [3.0, 4.0, 6.0]], [[4.0, 
4.0, 6.0], [5.0, 5.0, 4.0], [7.0, 3.0, 2.0]], [[6.0, 2.0, 2.0], [7.0, 8.0, 8.0], [6.0, 8.0, 8.0]]], [[[1.0, 5.0, 7.0], [2.0, 
6.0, 8.0], [4.0, 5.0, 1.0]], [[8.0, 3.0, 4.0], [5.0, 5.0, 8.0], [7.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], [8.0, 3.0, 7.0], [3.0, 3.0, 
5.0]]], [[[3.0, 6.0, 4.0], [7.0, 3.0, 7.0], [5.0, 1.0, 6.0]], [[6.0, 3.0, 6.0], [6.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[3.0, 8.0, 
7.0], [7.0, 7.0, 4.0], [6.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 7.0], [2.0, 2.0, 5.0], [6.0, 2.0, 2.0]], 
[[6.0, 1.0, 1.0], [5.0, 4.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 4.0, 2.0], [3.0, 5.0, 8.0], [1.0, 1.0, 5.0]]], [[[2.0, 2.0, 1.0], 
[3.0, 4.0, 7.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 7.0], [4.0, 3.0, 8.0], [8.0, 5.0, 4.0]], [[4.0, 6.0, 2.0], [3.0, 3.0, 6.0], 
[6.0, 4.0, 5.0]]], [[[6.0, 8.0, 3.0], [5.0, 2.0, 4.0], [3.0, 7.0, 4.0]], [[2.0, 4.0, 3.0], [2.0, 3.0, 7.0], [6.0, 3.0, 3.0]], 
[[1.0, 1.0, 5.0], [4.0, 6.0, 5.0], [7.0, 4.0, 2.0]]], [[[5.0, 8.0, 6.0], [3.0, 8.0, 7.0], [5.0, 7.0, 1.0]], [[8.0, 7.0, 5.0], 
[2.0, 4.0, 3.0], [5.0, 2.0, 2.0]], [[6.0, 1.0, 6.0], [1.0, 3.0, 5.0], [2.0, 7.0, 3.0]]]])
      val=numpy.array([[1.0, 5.0, 1.0], [4.0, 3.0, 3.0], [7.0, 6.0, 7.0], [3.0, 1.0, 1.0]])
      arg[:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 6.0, 2.0], [3.0, 5.0, 4.0], [3.0, 8.0, 6.0]], [[2.0, 5.0, 3.0], [1.0, 5.0, 1.0], [8.0, 
6.0, 6.0]], [[2.0, 2.0, 6.0], [5.0, 1.0, 5.0], [5.0, 7.0, 6.0]]], [[[6.0, 7.0, 4.0], [8.0, 5.0, 3.0], [3.0, 4.0, 6.0]], [[4.0, 
4.0, 6.0], [4.0, 3.0, 3.0], [7.0, 3.0, 2.0]], [[6.0, 2.0, 2.0], [7.0, 8.0, 8.0], [6.0, 8.0, 8.0]]], [[[1.0, 5.0, 7.0], [2.0, 
6.0, 8.0], [4.0, 5.0, 1.0]], [[8.0, 3.0, 4.0], [7.0, 6.0, 7.0], [7.0, 7.0, 4.0]], [[5.0, 3.0, 3.0], [8.0, 3.0, 7.0], [3.0, 3.0, 
5.0]]], [[[3.0, 6.0, 4.0], [7.0, 3.0, 7.0], [5.0, 1.0, 6.0]], [[6.0, 3.0, 6.0], [3.0, 1.0, 1.0], [1.0, 4.0, 2.0]], [[3.0, 8.0, 
7.0], [7.0, 7.0, 4.0], [6.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 7.0], [2.0, 2.0, 5.0], [6.0, 2.0, 2.0]], 
[[6.0, 1.0, 1.0], [1.0, 5.0, 1.0], [1.0, 8.0, 2.0]], [[2.0, 4.0, 2.0], [3.0, 5.0, 8.0], [1.0, 1.0, 5.0]]], [[[2.0, 2.0, 1.0], 
[3.0, 4.0, 7.0], [3.0, 6.0, 8.0]], [[1.0, 4.0, 7.0], [4.0, 3.0, 3.0], [8.0, 5.0, 4.0]], [[4.0, 6.0, 2.0], [3.0, 3.0, 6.0], 
[6.0, 4.0, 5.0]]], [[[6.0, 8.0, 3.0], [5.0, 2.0, 4.0], [3.0, 7.0, 4.0]], [[2.0, 4.0, 3.0], [7.0, 6.0, 7.0], [6.0, 3.0, 3.0]], 
[[1.0, 1.0, 5.0], [4.0, 6.0, 5.0], [7.0, 4.0, 2.0]]], [[[5.0, 8.0, 6.0], [3.0, 8.0, 7.0], [5.0, 7.0, 1.0]], [[8.0, 7.0, 5.0], 
[3.0, 1.0, 1.0], [5.0, 2.0, 2.0]], [[6.0, 1.0, 6.0], [1.0, 3.0, 5.0], [2.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 7.0], [8.0, 1.0, 7.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 7.0], [4.0, 2.0, 3.0], [6.0, 
4.0, 2.0]], [[3.0, 6.0, 7.0], [6.0, 2.0, 1.0], [5.0, 3.0, 4.0]]], [[[5.0, 7.0, 1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 1.0]], [[7.0, 
2.0, 6.0], [5.0, 2.0, 3.0], [7.0, 1.0, 6.0]], [[6.0, 1.0, 3.0], [2.0, 3.0, 3.0], [1.0, 7.0, 8.0]]], [[[7.0, 3.0, 1.0], [3.0, 
7.0, 8.0], [1.0, 1.0, 8.0]], [[5.0, 3.0, 6.0], [3.0, 6.0, 8.0], [6.0, 3.0, 8.0]], [[2.0, 3.0, 1.0], [8.0, 1.0, 7.0], [4.0, 8.0, 
1.0]]], [[[7.0, 3.0, 2.0], [7.0, 5.0, 3.0], [3.0, 7.0, 7.0]], [[8.0, 7.0, 3.0], [8.0, 7.0, 4.0], [2.0, 2.0, 8.0]], [[6.0, 5.0, 
6.0], [3.0, 6.0, 7.0], [5.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 5.0], [7.0, 4.0, 7.0], [6.0, 1.0, 1.0]], 
[[6.0, 1.0, 4.0], [3.0, 3.0, 5.0], [3.0, 1.0, 1.0]], [[2.0, 6.0, 7.0], [8.0, 6.0, 1.0], [6.0, 7.0, 5.0]]], [[[6.0, 8.0, 6.0], 
[5.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[7.0, 8.0, 1.0], [4.0, 6.0, 8.0], [7.0, 1.0, 8.0]], [[2.0, 7.0, 2.0], [5.0, 5.0, 2.0], 
[3.0, 8.0, 2.0]]], [[[8.0, 5.0, 1.0], [2.0, 2.0, 1.0], [7.0, 6.0, 7.0]], [[3.0, 3.0, 5.0], [4.0, 2.0, 4.0], [6.0, 5.0, 2.0]], 
[[8.0, 1.0, 7.0], [2.0, 3.0, 6.0], [8.0, 4.0, 2.0]]], [[[3.0, 2.0, 5.0], [4.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 6.0], 
[5.0, 2.0, 2.0], [5.0, 6.0, 4.0]], [[4.0, 5.0, 6.0], [3.0, 4.0, 3.0], [7.0, 4.0, 6.0]]]])
      val=Data(numpy.array([[7.0, 3.0, 4.0], [1.0, 8.0, 7.0], [3.0, 4.0, 7.0], [3.0, 3.0, 7.0]]),self.functionspace)
      arg[:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 7.0], [8.0, 1.0, 7.0], [4.0, 4.0, 5.0]], [[2.0, 6.0, 7.0], [7.0, 3.0, 4.0], [6.0, 
4.0, 2.0]], [[3.0, 6.0, 7.0], [6.0, 2.0, 1.0], [5.0, 3.0, 4.0]]], [[[5.0, 7.0, 1.0], [7.0, 1.0, 8.0], [3.0, 2.0, 1.0]], [[7.0, 
2.0, 6.0], [1.0, 8.0, 7.0], [7.0, 1.0, 6.0]], [[6.0, 1.0, 3.0], [2.0, 3.0, 3.0], [1.0, 7.0, 8.0]]], [[[7.0, 3.0, 1.0], [3.0, 
7.0, 8.0], [1.0, 1.0, 8.0]], [[5.0, 3.0, 6.0], [3.0, 4.0, 7.0], [6.0, 3.0, 8.0]], [[2.0, 3.0, 1.0], [8.0, 1.0, 7.0], [4.0, 8.0, 
1.0]]], [[[7.0, 3.0, 2.0], [7.0, 5.0, 3.0], [3.0, 7.0, 7.0]], [[8.0, 7.0, 3.0], [3.0, 3.0, 7.0], [2.0, 2.0, 8.0]], [[6.0, 5.0, 
6.0], [3.0, 6.0, 7.0], [5.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 5.0], [7.0, 4.0, 7.0], [6.0, 1.0, 1.0]], 
[[6.0, 1.0, 4.0], [7.0, 3.0, 4.0], [3.0, 1.0, 1.0]], [[2.0, 6.0, 7.0], [8.0, 6.0, 1.0], [6.0, 7.0, 5.0]]], [[[6.0, 8.0, 6.0], 
[5.0, 4.0, 4.0], [1.0, 4.0, 5.0]], [[7.0, 8.0, 1.0], [1.0, 8.0, 7.0], [7.0, 1.0, 8.0]], [[2.0, 7.0, 2.0], [5.0, 5.0, 2.0], 
[3.0, 8.0, 2.0]]], [[[8.0, 5.0, 1.0], [2.0, 2.0, 1.0], [7.0, 6.0, 7.0]], [[3.0, 3.0, 5.0], [3.0, 4.0, 7.0], [6.0, 5.0, 2.0]], 
[[8.0, 1.0, 7.0], [2.0, 3.0, 6.0], [8.0, 4.0, 2.0]]], [[[3.0, 2.0, 5.0], [4.0, 3.0, 3.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 6.0], 
[3.0, 3.0, 7.0], [5.0, 6.0, 4.0]], [[4.0, 5.0, 6.0], [3.0, 4.0, 3.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 3.0], [8.0, 6.0, 1.0], [8.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [1.0, 3.0, 5.0], [7.0, 
7.0, 6.0]], [[4.0, 2.0, 8.0], [2.0, 7.0, 7.0], [1.0, 1.0, 1.0]]], [[[3.0, 7.0, 2.0], [2.0, 7.0, 2.0], [4.0, 6.0, 7.0]], [[8.0, 
5.0, 8.0], [6.0, 4.0, 7.0], [8.0, 3.0, 7.0]], [[3.0, 7.0, 5.0], [8.0, 7.0, 2.0], [7.0, 7.0, 6.0]]], [[[8.0, 7.0, 5.0], [6.0, 
8.0, 4.0], [4.0, 2.0, 2.0]], [[1.0, 5.0, 1.0], [2.0, 2.0, 3.0], [2.0, 6.0, 6.0]], [[4.0, 5.0, 7.0], [5.0, 4.0, 7.0], [4.0, 3.0, 
6.0]]], [[[6.0, 8.0, 3.0], [1.0, 4.0, 4.0], [7.0, 4.0, 8.0]], [[6.0, 7.0, 6.0], [2.0, 8.0, 7.0], [3.0, 1.0, 8.0]], [[8.0, 3.0, 
1.0], [7.0, 1.0, 8.0], [3.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 8.0], [5.0, 8.0, 2.0], [3.0, 1.0, 3.0]], 
[[7.0, 1.0, 7.0], [5.0, 3.0, 4.0], [8.0, 4.0, 1.0]], [[7.0, 3.0, 8.0], [7.0, 4.0, 6.0], [6.0, 7.0, 5.0]]], [[[6.0, 7.0, 8.0], 
[6.0, 1.0, 3.0], [4.0, 5.0, 3.0]], [[6.0, 7.0, 2.0], [6.0, 6.0, 4.0], [2.0, 2.0, 7.0]], [[4.0, 7.0, 1.0], [5.0, 1.0, 7.0], 
[1.0, 5.0, 4.0]]], [[[3.0, 2.0, 4.0], [5.0, 8.0, 6.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 5.0], [5.0, 3.0, 7.0], [7.0, 4.0, 1.0]], 
[[1.0, 1.0, 7.0], [3.0, 6.0, 4.0], [5.0, 1.0, 3.0]]], [[[4.0, 1.0, 3.0], [4.0, 5.0, 4.0], [1.0, 6.0, 3.0]], [[1.0, 5.0, 3.0], 
[6.0, 8.0, 7.0], [8.0, 4.0, 8.0]], [[8.0, 4.0, 2.0], [5.0, 3.0, 3.0], [7.0, 7.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[7.0, 4.0, 6.0], [5.0, 7.0, 7.0], [5.0, 4.0, 5.0], [2.0, 8.0, 
5.0]])+(1.-msk_val)*numpy.array([[1.0, 3.0, 4.0], [6.0, 4.0, 3.0], [6.0, 1.0, 6.0], [2.0, 7.0, 4.0]])
      arg[:,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 3.0], [8.0, 6.0, 1.0], [8.0, 3.0, 3.0]], [[5.0, 6.0, 4.0], [7.0, 4.0, 6.0], [7.0, 
7.0, 6.0]], [[4.0, 2.0, 8.0], [2.0, 7.0, 7.0], [1.0, 1.0, 1.0]]], [[[3.0, 7.0, 2.0], [2.0, 7.0, 2.0], [4.0, 6.0, 7.0]], [[8.0, 
5.0, 8.0], [5.0, 7.0, 7.0], [8.0, 3.0, 7.0]], [[3.0, 7.0, 5.0], [8.0, 7.0, 2.0], [7.0, 7.0, 6.0]]], [[[8.0, 7.0, 5.0], [6.0, 
8.0, 4.0], [4.0, 2.0, 2.0]], [[1.0, 5.0, 1.0], [5.0, 4.0, 5.0], [2.0, 6.0, 6.0]], [[4.0, 5.0, 7.0], [5.0, 4.0, 7.0], [4.0, 3.0, 
6.0]]], [[[6.0, 8.0, 3.0], [1.0, 4.0, 4.0], [7.0, 4.0, 8.0]], [[6.0, 7.0, 6.0], [2.0, 8.0, 5.0], [3.0, 1.0, 8.0]], [[8.0, 3.0, 
1.0], [7.0, 1.0, 8.0], [3.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 8.0], [5.0, 8.0, 2.0], [3.0, 1.0, 3.0]], 
[[7.0, 1.0, 7.0], [1.0, 3.0, 4.0], [8.0, 4.0, 1.0]], [[7.0, 3.0, 8.0], [7.0, 4.0, 6.0], [6.0, 7.0, 5.0]]], [[[6.0, 7.0, 8.0], 
[6.0, 1.0, 3.0], [4.0, 5.0, 3.0]], [[6.0, 7.0, 2.0], [6.0, 4.0, 3.0], [2.0, 2.0, 7.0]], [[4.0, 7.0, 1.0], [5.0, 1.0, 7.0], 
[1.0, 5.0, 4.0]]], [[[3.0, 2.0, 4.0], [5.0, 8.0, 6.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 5.0], [6.0, 1.0, 6.0], [7.0, 4.0, 1.0]], 
[[1.0, 1.0, 7.0], [3.0, 6.0, 4.0], [5.0, 1.0, 3.0]]], [[[4.0, 1.0, 3.0], [4.0, 5.0, 4.0], [1.0, 6.0, 3.0]], [[1.0, 5.0, 3.0], 
[2.0, 7.0, 4.0], [8.0, 4.0, 8.0]], [[8.0, 4.0, 2.0], [5.0, 3.0, 3.0], [7.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 6.0], [8.0, 3.0, 3.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 2.0, 4.0], [2.0, 
4.0, 2.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 1.0], [2.0, 8.0, 2.0]]], [[[2.0, 6.0, 4.0], [2.0, 8.0, 2.0], [3.0, 1.0, 4.0]], [[2.0, 
5.0, 1.0], [7.0, 6.0, 1.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 6.0], [8.0, 8.0, 2.0], [8.0, 2.0, 8.0]]], [[[2.0, 5.0, 8.0], [3.0, 
2.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 8.0, 3.0], [5.0, 3.0, 2.0], [7.0, 6.0, 6.0]], [[2.0, 6.0, 8.0], [7.0, 1.0, 2.0], [8.0, 4.0, 
5.0]]], [[[4.0, 5.0, 2.0], [7.0, 1.0, 8.0], [1.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [4.0, 4.0, 8.0], [2.0, 7.0, 5.0]], [[4.0, 8.0, 
7.0], [5.0, 3.0, 7.0], [2.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 1.0], [2.0, 8.0, 7.0], [8.0, 3.0, 1.0]], 
[[7.0, 3.0, 3.0], [1.0, 4.0, 7.0], [8.0, 5.0, 8.0]], [[7.0, 8.0, 2.0], [7.0, 6.0, 4.0], [1.0, 7.0, 1.0]]], [[[2.0, 8.0, 2.0], 
[2.0, 4.0, 7.0], [4.0, 6.0, 1.0]], [[7.0, 2.0, 3.0], [2.0, 7.0, 3.0], [6.0, 1.0, 8.0]], [[6.0, 1.0, 4.0], [5.0, 4.0, 2.0], 
[7.0, 6.0, 8.0]]], [[[6.0, 4.0, 8.0], [4.0, 4.0, 5.0], [8.0, 2.0, 8.0]], [[4.0, 2.0, 4.0], [6.0, 2.0, 5.0], [5.0, 3.0, 4.0]], 
[[1.0, 4.0, 5.0], [1.0, 4.0, 7.0], [2.0, 6.0, 2.0]]], [[[7.0, 5.0, 7.0], [8.0, 3.0, 6.0], [3.0, 7.0, 8.0]], [[4.0, 8.0, 1.0], 
[6.0, 8.0, 6.0], [1.0, 3.0, 4.0]], [[8.0, 6.0, 6.0], [1.0, 8.0, 5.0], [7.0, 3.0, 5.0]]]])
      val=numpy.array([[[6.0, 1.0, 4.0]]])
      arg[2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 6.0], [8.0, 3.0, 3.0], [2.0, 8.0, 5.0]], [[4.0, 4.0, 6.0], [1.0, 2.0, 4.0], [2.0, 
4.0, 2.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 1.0], [2.0, 8.0, 2.0]]], [[[2.0, 6.0, 4.0], [2.0, 8.0, 2.0], [3.0, 1.0, 4.0]], [[2.0, 
5.0, 1.0], [7.0, 6.0, 1.0], [6.0, 2.0, 4.0]], [[7.0, 2.0, 6.0], [8.0, 8.0, 2.0], [8.0, 2.0, 8.0]]], [[[2.0, 5.0, 8.0], [3.0, 
2.0, 7.0], [1.0, 8.0, 6.0]], [[4.0, 8.0, 3.0], [5.0, 3.0, 2.0], [7.0, 6.0, 6.0]], [[2.0, 6.0, 8.0], [6.0, 1.0, 4.0], [8.0, 4.0, 
5.0]]], [[[4.0, 5.0, 2.0], [7.0, 1.0, 8.0], [1.0, 2.0, 1.0]], [[2.0, 5.0, 8.0], [4.0, 4.0, 8.0], [2.0, 7.0, 5.0]], [[4.0, 8.0, 
7.0], [5.0, 3.0, 7.0], [2.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [2.0, 8.0, 7.0], [8.0, 3.0, 1.0]], 
[[7.0, 3.0, 3.0], [1.0, 4.0, 7.0], [8.0, 5.0, 8.0]], [[7.0, 8.0, 2.0], [7.0, 6.0, 4.0], [1.0, 7.0, 1.0]]], [[[2.0, 8.0, 2.0], 
[2.0, 4.0, 7.0], [4.0, 6.0, 1.0]], [[7.0, 2.0, 3.0], [2.0, 7.0, 3.0], [6.0, 1.0, 8.0]], [[6.0, 1.0, 4.0], [5.0, 4.0, 2.0], 
[7.0, 6.0, 8.0]]], [[[6.0, 4.0, 8.0], [4.0, 4.0, 5.0], [8.0, 2.0, 8.0]], [[4.0, 2.0, 4.0], [6.0, 2.0, 5.0], [5.0, 3.0, 4.0]], 
[[1.0, 4.0, 5.0], [6.0, 1.0, 4.0], [2.0, 6.0, 2.0]]], [[[7.0, 5.0, 7.0], [8.0, 3.0, 6.0], [3.0, 7.0, 8.0]], [[4.0, 8.0, 1.0], 
[6.0, 8.0, 6.0], [1.0, 3.0, 4.0]], [[8.0, 6.0, 6.0], [1.0, 8.0, 5.0], [7.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 3.0, 6.0], [2.0, 4.0, 7.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 3.0], [6.0, 5.0, 4.0], [8.0, 
8.0, 8.0]], [[3.0, 2.0, 2.0], [5.0, 3.0, 4.0], [5.0, 5.0, 1.0]]], [[[6.0, 3.0, 1.0], [8.0, 6.0, 2.0], [7.0, 7.0, 8.0]], [[7.0, 
7.0, 2.0], [6.0, 6.0, 1.0], [1.0, 3.0, 3.0]], [[7.0, 3.0, 1.0], [5.0, 6.0, 3.0], [1.0, 5.0, 6.0]]], [[[8.0, 8.0, 6.0], [8.0, 
4.0, 3.0], [8.0, 3.0, 7.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 6.0], [3.0, 5.0, 1.0]], [[3.0, 7.0, 3.0], [1.0, 4.0, 5.0], [8.0, 4.0, 
6.0]]], [[[6.0, 1.0, 2.0], [3.0, 1.0, 6.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 4.0], [5.0, 8.0, 5.0], [2.0, 7.0, 6.0]], [[4.0, 6.0, 
2.0], [5.0, 6.0, 3.0], [5.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 2.0], [3.0, 8.0, 5.0], [1.0, 6.0, 7.0]], 
[[2.0, 5.0, 6.0], [5.0, 4.0, 1.0], [7.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [6.0, 1.0, 7.0], [7.0, 6.0, 2.0]]], [[[3.0, 2.0, 3.0], 
[8.0, 2.0, 4.0], [8.0, 1.0, 3.0]], [[1.0, 6.0, 5.0], [4.0, 3.0, 2.0], [5.0, 2.0, 7.0]], [[8.0, 7.0, 1.0], [4.0, 7.0, 4.0], 
[5.0, 3.0, 8.0]]], [[[7.0, 1.0, 4.0], [7.0, 1.0, 2.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 1.0], [3.0, 3.0, 1.0], [8.0, 8.0, 6.0]], 
[[2.0, 6.0, 1.0], [5.0, 3.0, 6.0], [5.0, 8.0, 1.0]]], [[[5.0, 8.0, 5.0], [7.0, 2.0, 7.0], [2.0, 8.0, 4.0]], [[1.0, 1.0, 7.0], 
[2.0, 2.0, 4.0], [7.0, 5.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 6.0, 6.0], [3.0, 1.0, 7.0]]]])
      val=Data(numpy.array([[[8.0, 8.0, 5.0]]]),self.functionspace)
      arg[2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 6.0], [2.0, 4.0, 7.0], [1.0, 8.0, 6.0]], [[1.0, 5.0, 3.0], [6.0, 5.0, 4.0], [8.0, 
8.0, 8.0]], [[3.0, 2.0, 2.0], [5.0, 3.0, 4.0], [5.0, 5.0, 1.0]]], [[[6.0, 3.0, 1.0], [8.0, 6.0, 2.0], [7.0, 7.0, 8.0]], [[7.0, 
7.0, 2.0], [6.0, 6.0, 1.0], [1.0, 3.0, 3.0]], [[7.0, 3.0, 1.0], [5.0, 6.0, 3.0], [1.0, 5.0, 6.0]]], [[[8.0, 8.0, 6.0], [8.0, 
4.0, 3.0], [8.0, 3.0, 7.0]], [[5.0, 6.0, 6.0], [2.0, 8.0, 6.0], [3.0, 5.0, 1.0]], [[3.0, 7.0, 3.0], [8.0, 8.0, 5.0], [8.0, 4.0, 
6.0]]], [[[6.0, 1.0, 2.0], [3.0, 1.0, 6.0], [6.0, 2.0, 7.0]], [[3.0, 7.0, 4.0], [5.0, 8.0, 5.0], [2.0, 7.0, 6.0]], [[4.0, 6.0, 
2.0], [5.0, 6.0, 3.0], [5.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 2.0], [3.0, 8.0, 5.0], [1.0, 6.0, 7.0]], 
[[2.0, 5.0, 6.0], [5.0, 4.0, 1.0], [7.0, 3.0, 8.0]], [[3.0, 8.0, 2.0], [6.0, 1.0, 7.0], [7.0, 6.0, 2.0]]], [[[3.0, 2.0, 3.0], 
[8.0, 2.0, 4.0], [8.0, 1.0, 3.0]], [[1.0, 6.0, 5.0], [4.0, 3.0, 2.0], [5.0, 2.0, 7.0]], [[8.0, 7.0, 1.0], [4.0, 7.0, 4.0], 
[5.0, 3.0, 8.0]]], [[[7.0, 1.0, 4.0], [7.0, 1.0, 2.0], [5.0, 1.0, 3.0]], [[6.0, 8.0, 1.0], [3.0, 3.0, 1.0], [8.0, 8.0, 6.0]], 
[[2.0, 6.0, 1.0], [8.0, 8.0, 5.0], [5.0, 8.0, 1.0]]], [[[5.0, 8.0, 5.0], [7.0, 2.0, 7.0], [2.0, 8.0, 4.0]], [[1.0, 1.0, 7.0], 
[2.0, 2.0, 4.0], [7.0, 5.0, 8.0]], [[5.0, 5.0, 8.0], [4.0, 6.0, 6.0], [3.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 7.0], [4.0, 7.0, 6.0], [7.0, 7.0, 5.0]], [[8.0, 4.0, 2.0], [6.0, 7.0, 1.0], [8.0, 
8.0, 3.0]], [[7.0, 1.0, 2.0], [3.0, 8.0, 4.0], [6.0, 7.0, 3.0]]], [[[2.0, 7.0, 6.0], [7.0, 2.0, 8.0], [7.0, 7.0, 2.0]], [[7.0, 
6.0, 8.0], [1.0, 2.0, 4.0], [2.0, 1.0, 8.0]], [[7.0, 3.0, 8.0], [8.0, 8.0, 1.0], [7.0, 6.0, 4.0]]], [[[5.0, 1.0, 6.0], [1.0, 
7.0, 1.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 4.0], [8.0, 1.0, 1.0], [5.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [8.0, 4.0, 6.0], [2.0, 4.0, 
3.0]]], [[[6.0, 3.0, 6.0], [7.0, 7.0, 1.0], [1.0, 4.0, 2.0]], [[4.0, 7.0, 8.0], [7.0, 4.0, 8.0], [8.0, 2.0, 6.0]], [[5.0, 8.0, 
3.0], [4.0, 5.0, 7.0], [3.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 8.0, 7.0], [7.0, 1.0, 5.0], [3.0, 4.0, 2.0]], 
[[5.0, 3.0, 4.0], [4.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[6.0, 5.0, 2.0], [4.0, 3.0, 1.0], [1.0, 8.0, 4.0]]], [[[3.0, 2.0, 1.0], 
[6.0, 6.0, 7.0], [7.0, 7.0, 3.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 8.0], [6.0, 4.0, 4.0]], [[7.0, 5.0, 6.0], [1.0, 3.0, 2.0], 
[8.0, 3.0, 1.0]]], [[[4.0, 1.0, 2.0], [5.0, 3.0, 3.0], [1.0, 4.0, 7.0]], [[6.0, 4.0, 8.0], [7.0, 2.0, 5.0], [8.0, 5.0, 5.0]], 
[[4.0, 6.0, 5.0], [5.0, 2.0, 7.0], [4.0, 4.0, 3.0]]], [[[7.0, 4.0, 7.0], [3.0, 7.0, 2.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 5.0], 
[8.0, 4.0, 3.0], [5.0, 7.0, 5.0]], [[4.0, 3.0, 7.0], [6.0, 4.0, 1.0], [6.0, 7.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 6.0, 3.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 3.0]]])
      arg[2,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 7.0], [4.0, 7.0, 6.0], [7.0, 7.0, 5.0]], [[8.0, 4.0, 2.0], [6.0, 7.0, 1.0], [8.0, 
8.0, 3.0]], [[7.0, 1.0, 2.0], [3.0, 8.0, 4.0], [6.0, 7.0, 3.0]]], [[[2.0, 7.0, 6.0], [7.0, 2.0, 8.0], [7.0, 7.0, 2.0]], [[7.0, 
6.0, 8.0], [1.0, 2.0, 4.0], [2.0, 1.0, 8.0]], [[7.0, 3.0, 8.0], [8.0, 8.0, 1.0], [7.0, 6.0, 4.0]]], [[[5.0, 1.0, 6.0], [1.0, 
7.0, 1.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 4.0], [8.0, 1.0, 1.0], [5.0, 2.0, 7.0]], [[6.0, 6.0, 4.0], [5.0, 6.0, 3.0], [2.0, 4.0, 
3.0]]], [[[6.0, 3.0, 6.0], [7.0, 7.0, 1.0], [1.0, 4.0, 2.0]], [[4.0, 7.0, 8.0], [7.0, 4.0, 8.0], [8.0, 2.0, 6.0]], [[5.0, 8.0, 
3.0], [4.0, 5.0, 7.0], [3.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 8.0, 7.0], [7.0, 1.0, 5.0], [3.0, 4.0, 2.0]], 
[[5.0, 3.0, 4.0], [4.0, 5.0, 7.0], [4.0, 5.0, 1.0]], [[6.0, 5.0, 2.0], [4.0, 3.0, 1.0], [1.0, 8.0, 4.0]]], [[[3.0, 2.0, 1.0], 
[6.0, 6.0, 7.0], [7.0, 7.0, 3.0]], [[4.0, 8.0, 3.0], [6.0, 4.0, 8.0], [6.0, 4.0, 4.0]], [[7.0, 5.0, 6.0], [1.0, 3.0, 2.0], 
[8.0, 3.0, 1.0]]], [[[4.0, 1.0, 2.0], [5.0, 3.0, 3.0], [1.0, 4.0, 7.0]], [[6.0, 4.0, 8.0], [7.0, 2.0, 5.0], [8.0, 5.0, 5.0]], 
[[4.0, 6.0, 5.0], [8.0, 1.0, 3.0], [4.0, 4.0, 3.0]]], [[[7.0, 4.0, 7.0], [3.0, 7.0, 2.0], [5.0, 6.0, 8.0]], [[1.0, 1.0, 5.0], 
[8.0, 4.0, 3.0], [5.0, 7.0, 5.0]], [[4.0, 3.0, 7.0], [6.0, 4.0, 1.0], [6.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 5.0], [6.0, 4.0, 1.0], [3.0, 1.0, 5.0]], [[2.0, 7.0, 1.0], [4.0, 8.0, 4.0], [6.0, 
8.0, 5.0]], [[8.0, 7.0, 4.0], [4.0, 2.0, 5.0], [8.0, 2.0, 5.0]]], [[[7.0, 6.0, 4.0], [7.0, 2.0, 6.0], [1.0, 2.0, 6.0]], [[4.0, 
2.0, 3.0], [6.0, 7.0, 7.0], [5.0, 8.0, 5.0]], [[6.0, 2.0, 6.0], [6.0, 2.0, 8.0], [6.0, 4.0, 5.0]]], [[[7.0, 5.0, 4.0], [4.0, 
2.0, 2.0], [4.0, 7.0, 1.0]], [[7.0, 5.0, 4.0], [6.0, 3.0, 7.0], [4.0, 1.0, 8.0]], [[6.0, 1.0, 8.0], [8.0, 8.0, 5.0], [8.0, 5.0, 
5.0]]], [[[3.0, 1.0, 2.0], [6.0, 6.0, 7.0], [4.0, 5.0, 8.0]], [[3.0, 4.0, 6.0], [3.0, 8.0, 2.0], [4.0, 6.0, 1.0]], [[7.0, 5.0, 
7.0], [7.0, 6.0, 6.0], [2.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 2.0], [1.0, 4.0, 3.0], [7.0, 7.0, 8.0]], 
[[1.0, 2.0, 5.0], [5.0, 8.0, 8.0], [6.0, 7.0, 2.0]], [[5.0, 6.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 8.0]]], [[[3.0, 7.0, 8.0], 
[4.0, 6.0, 2.0], [1.0, 7.0, 2.0]], [[5.0, 7.0, 4.0], [8.0, 3.0, 1.0], [5.0, 2.0, 1.0]], [[8.0, 4.0, 2.0], [2.0, 6.0, 5.0], 
[2.0, 7.0, 5.0]]], [[[3.0, 7.0, 5.0], [6.0, 2.0, 6.0], [8.0, 8.0, 3.0]], [[4.0, 5.0, 1.0], [5.0, 1.0, 5.0], [3.0, 4.0, 4.0]], 
[[6.0, 6.0, 6.0], [8.0, 8.0, 5.0], [3.0, 1.0, 1.0]]], [[[6.0, 8.0, 5.0], [2.0, 4.0, 3.0], [5.0, 2.0, 7.0]], [[5.0, 8.0, 3.0], 
[7.0, 1.0, 6.0], [1.0, 5.0, 5.0]], [[3.0, 7.0, 1.0], [8.0, 5.0, 2.0], [8.0, 1.0, 6.0]]]])
      val=numpy.array([[[8.0, 8.0, 2.0], [1.0, 5.0, 5.0], [4.0, 2.0, 8.0]], [[6.0, 3.0, 8.0], [1.0, 8.0, 4.0], [7.0, 1.0, 
1.0]], [[1.0, 4.0, 6.0], [5.0, 1.0, 3.0], [8.0, 2.0, 8.0]]])
      arg[:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 5.0], [6.0, 4.0, 1.0], [3.0, 1.0, 5.0]], [[8.0, 8.0, 2.0], [1.0, 5.0, 5.0], [4.0, 
2.0, 8.0]], [[8.0, 7.0, 4.0], [4.0, 2.0, 5.0], [8.0, 2.0, 5.0]]], [[[7.0, 6.0, 4.0], [7.0, 2.0, 6.0], [1.0, 2.0, 6.0]], [[6.0, 
3.0, 8.0], [1.0, 8.0, 4.0], [7.0, 1.0, 1.0]], [[6.0, 2.0, 6.0], [6.0, 2.0, 8.0], [6.0, 4.0, 5.0]]], [[[7.0, 5.0, 4.0], [4.0, 
2.0, 2.0], [4.0, 7.0, 1.0]], [[1.0, 4.0, 6.0], [5.0, 1.0, 3.0], [8.0, 2.0, 8.0]], [[6.0, 1.0, 8.0], [8.0, 8.0, 5.0], [8.0, 5.0, 
5.0]]], [[[3.0, 1.0, 2.0], [6.0, 6.0, 7.0], [4.0, 5.0, 8.0]], [[3.0, 4.0, 6.0], [3.0, 8.0, 2.0], [4.0, 6.0, 1.0]], [[7.0, 5.0, 
7.0], [7.0, 6.0, 6.0], [2.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 2.0], [1.0, 4.0, 3.0], [7.0, 7.0, 8.0]], 
[[8.0, 8.0, 2.0], [1.0, 5.0, 5.0], [4.0, 2.0, 8.0]], [[5.0, 6.0, 1.0], [1.0, 8.0, 7.0], [3.0, 7.0, 8.0]]], [[[3.0, 7.0, 8.0], 
[4.0, 6.0, 2.0], [1.0, 7.0, 2.0]], [[6.0, 3.0, 8.0], [1.0, 8.0, 4.0], [7.0, 1.0, 1.0]], [[8.0, 4.0, 2.0], [2.0, 6.0, 5.0], 
[2.0, 7.0, 5.0]]], [[[3.0, 7.0, 5.0], [6.0, 2.0, 6.0], [8.0, 8.0, 3.0]], [[1.0, 4.0, 6.0], [5.0, 1.0, 3.0], [8.0, 2.0, 8.0]], 
[[6.0, 6.0, 6.0], [8.0, 8.0, 5.0], [3.0, 1.0, 1.0]]], [[[6.0, 8.0, 5.0], [2.0, 4.0, 3.0], [5.0, 2.0, 7.0]], [[5.0, 8.0, 3.0], 
[7.0, 1.0, 6.0], [1.0, 5.0, 5.0]], [[3.0, 7.0, 1.0], [8.0, 5.0, 2.0], [8.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 7.0], [6.0, 7.0, 3.0], [7.0, 7.0, 5.0]], [[5.0, 7.0, 1.0], [4.0, 4.0, 4.0], [7.0, 
4.0, 2.0]], [[3.0, 7.0, 1.0], [4.0, 5.0, 2.0], [3.0, 5.0, 7.0]]], [[[5.0, 6.0, 8.0], [7.0, 4.0, 3.0], [6.0, 6.0, 2.0]], [[6.0, 
6.0, 4.0], [4.0, 3.0, 5.0], [8.0, 6.0, 2.0]], [[8.0, 3.0, 3.0], [8.0, 8.0, 2.0], [7.0, 4.0, 3.0]]], [[[4.0, 7.0, 6.0], [2.0, 
8.0, 7.0], [3.0, 7.0, 2.0]], [[7.0, 1.0, 3.0], [8.0, 3.0, 8.0], [8.0, 4.0, 5.0]], [[3.0, 6.0, 3.0], [6.0, 1.0, 8.0], [6.0, 8.0, 
1.0]]], [[[3.0, 5.0, 3.0], [6.0, 5.0, 2.0], [6.0, 1.0, 2.0]], [[4.0, 3.0, 2.0], [4.0, 6.0, 8.0], [2.0, 2.0, 3.0]], [[4.0, 3.0, 
8.0], [8.0, 2.0, 6.0], [8.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 1.0, 6.0], [7.0, 2.0, 1.0], [2.0, 3.0, 4.0]], 
[[3.0, 3.0, 6.0], [8.0, 6.0, 2.0], [3.0, 5.0, 5.0]], [[8.0, 2.0, 4.0], [6.0, 1.0, 7.0], [7.0, 6.0, 3.0]]], [[[2.0, 2.0, 1.0], 
[6.0, 1.0, 6.0], [5.0, 6.0, 7.0]], [[8.0, 2.0, 8.0], [3.0, 8.0, 2.0], [2.0, 4.0, 8.0]], [[7.0, 1.0, 1.0], [6.0, 6.0, 1.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 6.0, 6.0], [4.0, 3.0, 8.0], [8.0, 7.0, 1.0]], [[2.0, 2.0, 7.0], [3.0, 5.0, 3.0], [7.0, 7.0, 8.0]], 
[[5.0, 2.0, 1.0], [8.0, 4.0, 5.0], [7.0, 3.0, 7.0]]], [[[2.0, 2.0, 1.0], [7.0, 5.0, 5.0], [8.0, 6.0, 2.0]], [[6.0, 4.0, 2.0], 
[5.0, 3.0, 8.0], [1.0, 6.0, 7.0]], [[7.0, 4.0, 4.0], [6.0, 5.0, 8.0], [2.0, 8.0, 4.0]]]])
      val=Data(numpy.array([[[2.0, 2.0, 6.0], [4.0, 6.0, 3.0], [5.0, 6.0, 6.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 2.0], [1.0, 5.0, 
1.0]], [[7.0, 5.0, 2.0], [7.0, 7.0, 2.0], [6.0, 5.0, 8.0]]]),self.functionspace)
      arg[:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 7.0], [6.0, 7.0, 3.0], [7.0, 7.0, 5.0]], [[2.0, 2.0, 6.0], [4.0, 6.0, 3.0], [5.0, 
6.0, 6.0]], [[3.0, 7.0, 1.0], [4.0, 5.0, 2.0], [3.0, 5.0, 7.0]]], [[[5.0, 6.0, 8.0], [7.0, 4.0, 3.0], [6.0, 6.0, 2.0]], [[1.0, 
7.0, 3.0], [6.0, 2.0, 2.0], [1.0, 5.0, 1.0]], [[8.0, 3.0, 3.0], [8.0, 8.0, 2.0], [7.0, 4.0, 3.0]]], [[[4.0, 7.0, 6.0], [2.0, 
8.0, 7.0], [3.0, 7.0, 2.0]], [[7.0, 5.0, 2.0], [7.0, 7.0, 2.0], [6.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [6.0, 1.0, 8.0], [6.0, 8.0, 
1.0]]], [[[3.0, 5.0, 3.0], [6.0, 5.0, 2.0], [6.0, 1.0, 2.0]], [[4.0, 3.0, 2.0], [4.0, 6.0, 8.0], [2.0, 2.0, 3.0]], [[4.0, 3.0, 
8.0], [8.0, 2.0, 6.0], [8.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 1.0, 6.0], [7.0, 2.0, 1.0], [2.0, 3.0, 4.0]], 
[[2.0, 2.0, 6.0], [4.0, 6.0, 3.0], [5.0, 6.0, 6.0]], [[8.0, 2.0, 4.0], [6.0, 1.0, 7.0], [7.0, 6.0, 3.0]]], [[[2.0, 2.0, 1.0], 
[6.0, 1.0, 6.0], [5.0, 6.0, 7.0]], [[1.0, 7.0, 3.0], [6.0, 2.0, 2.0], [1.0, 5.0, 1.0]], [[7.0, 1.0, 1.0], [6.0, 6.0, 1.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 6.0, 6.0], [4.0, 3.0, 8.0], [8.0, 7.0, 1.0]], [[7.0, 5.0, 2.0], [7.0, 7.0, 2.0], [6.0, 5.0, 8.0]], 
[[5.0, 2.0, 1.0], [8.0, 4.0, 5.0], [7.0, 3.0, 7.0]]], [[[2.0, 2.0, 1.0], [7.0, 5.0, 5.0], [8.0, 6.0, 2.0]], [[6.0, 4.0, 2.0], 
[5.0, 3.0, 8.0], [1.0, 6.0, 7.0]], [[7.0, 4.0, 4.0], [6.0, 5.0, 8.0], [2.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 5.0], [6.0, 7.0, 4.0], [1.0, 4.0, 1.0]], [[6.0, 5.0, 4.0], [5.0, 3.0, 3.0], [1.0, 
6.0, 3.0]], [[8.0, 7.0, 1.0], [8.0, 7.0, 4.0], [4.0, 8.0, 6.0]]], [[[2.0, 4.0, 8.0], [2.0, 5.0, 4.0], [5.0, 1.0, 8.0]], [[1.0, 
5.0, 7.0], [1.0, 4.0, 3.0], [6.0, 7.0, 3.0]], [[7.0, 5.0, 3.0], [5.0, 6.0, 5.0], [2.0, 2.0, 8.0]]], [[[5.0, 8.0, 6.0], [6.0, 
8.0, 3.0], [2.0, 4.0, 7.0]], [[7.0, 8.0, 8.0], [6.0, 1.0, 7.0], [5.0, 6.0, 5.0]], [[8.0, 7.0, 3.0], [6.0, 3.0, 2.0], [1.0, 2.0, 
8.0]]], [[[2.0, 2.0, 7.0], [7.0, 4.0, 5.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 1.0], [1.0, 3.0, 1.0]], [[5.0, 3.0, 
2.0], [3.0, 8.0, 1.0], [7.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 3.0, 7.0], [2.0, 2.0, 5.0], [1.0, 2.0, 2.0]], 
[[8.0, 7.0, 4.0], [7.0, 8.0, 4.0], [4.0, 1.0, 6.0]], [[6.0, 2.0, 8.0], [6.0, 2.0, 4.0], [4.0, 6.0, 1.0]]], [[[1.0, 4.0, 2.0], 
[8.0, 2.0, 7.0], [6.0, 1.0, 4.0]], [[1.0, 7.0, 3.0], [1.0, 1.0, 3.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 1.0], [5.0, 2.0, 8.0], 
[3.0, 8.0, 4.0]]], [[[7.0, 3.0, 7.0], [1.0, 1.0, 6.0], [7.0, 2.0, 7.0]], [[6.0, 4.0, 6.0], [7.0, 6.0, 8.0], [7.0, 5.0, 5.0]], 
[[8.0, 2.0, 5.0], [6.0, 1.0, 7.0], [4.0, 4.0, 3.0]]], [[[8.0, 5.0, 5.0], [4.0, 4.0, 3.0], [3.0, 4.0, 1.0]], [[1.0, 8.0, 1.0], 
[4.0, 6.0, 3.0], [7.0, 7.0, 7.0]], [[8.0, 8.0, 3.0], [5.0, 1.0, 7.0], [4.0, 3.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 4.0, 4.0], [3.0, 3.0, 5.0], [3.0, 2.0, 7.0]], [[1.0, 3.0, 8.0], [3.0, 6.0, 6.0], [1.0, 
1.0, 4.0]], [[2.0, 8.0, 1.0], [3.0, 3.0, 5.0], [7.0, 4.0, 5.0]]])+(1.-msk_val)*numpy.array([[[2.0, 6.0, 8.0], [2.0, 6.0, 2.0], 
[8.0, 4.0, 2.0]], [[1.0, 2.0, 1.0], [6.0, 7.0, 4.0], [2.0, 2.0, 7.0]], [[3.0, 3.0, 2.0], [6.0, 3.0, 3.0], [1.0, 1.0, 6.0]]])
      arg[:3,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 5.0], [6.0, 7.0, 4.0], [1.0, 4.0, 1.0]], [[4.0, 4.0, 4.0], [3.0, 3.0, 5.0], [3.0, 
2.0, 7.0]], [[8.0, 7.0, 1.0], [8.0, 7.0, 4.0], [4.0, 8.0, 6.0]]], [[[2.0, 4.0, 8.0], [2.0, 5.0, 4.0], [5.0, 1.0, 8.0]], [[1.0, 
3.0, 8.0], [3.0, 6.0, 6.0], [1.0, 1.0, 4.0]], [[7.0, 5.0, 3.0], [5.0, 6.0, 5.0], [2.0, 2.0, 8.0]]], [[[5.0, 8.0, 6.0], [6.0, 
8.0, 3.0], [2.0, 4.0, 7.0]], [[2.0, 8.0, 1.0], [3.0, 3.0, 5.0], [7.0, 4.0, 5.0]], [[8.0, 7.0, 3.0], [6.0, 3.0, 2.0], [1.0, 2.0, 
8.0]]], [[[2.0, 2.0, 7.0], [7.0, 4.0, 5.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 1.0], [3.0, 3.0, 1.0], [1.0, 3.0, 1.0]], [[5.0, 3.0, 
2.0], [3.0, 8.0, 1.0], [7.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 3.0, 7.0], [2.0, 2.0, 5.0], [1.0, 2.0, 2.0]], 
[[2.0, 6.0, 8.0], [2.0, 6.0, 2.0], [8.0, 4.0, 2.0]], [[6.0, 2.0, 8.0], [6.0, 2.0, 4.0], [4.0, 6.0, 1.0]]], [[[1.0, 4.0, 2.0], 
[8.0, 2.0, 7.0], [6.0, 1.0, 4.0]], [[1.0, 2.0, 1.0], [6.0, 7.0, 4.0], [2.0, 2.0, 7.0]], [[4.0, 3.0, 1.0], [5.0, 2.0, 8.0], 
[3.0, 8.0, 4.0]]], [[[7.0, 3.0, 7.0], [1.0, 1.0, 6.0], [7.0, 2.0, 7.0]], [[3.0, 3.0, 2.0], [6.0, 3.0, 3.0], [1.0, 1.0, 6.0]], 
[[8.0, 2.0, 5.0], [6.0, 1.0, 7.0], [4.0, 4.0, 3.0]]], [[[8.0, 5.0, 5.0], [4.0, 4.0, 3.0], [3.0, 4.0, 1.0]], [[1.0, 8.0, 1.0], 
[4.0, 6.0, 3.0], [7.0, 7.0, 7.0]], [[8.0, 8.0, 3.0], [5.0, 1.0, 7.0], [4.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 1.0], [1.0, 2.0, 6.0], [7.0, 5.0, 3.0]], [[7.0, 7.0, 5.0], [5.0, 6.0, 6.0], [1.0, 
8.0, 5.0]], [[7.0, 1.0, 4.0], [7.0, 8.0, 1.0], [4.0, 7.0, 2.0]]], [[[2.0, 7.0, 8.0], [3.0, 1.0, 5.0], [5.0, 8.0, 4.0]], [[6.0, 
4.0, 3.0], [8.0, 5.0, 2.0], [8.0, 4.0, 2.0]], [[4.0, 2.0, 7.0], [6.0, 2.0, 2.0], [1.0, 2.0, 5.0]]], [[[1.0, 4.0, 4.0], [5.0, 
8.0, 2.0], [6.0, 7.0, 4.0]], [[8.0, 4.0, 7.0], [8.0, 3.0, 1.0], [8.0, 4.0, 8.0]], [[8.0, 3.0, 6.0], [4.0, 2.0, 3.0], [5.0, 4.0, 
4.0]]], [[[3.0, 2.0, 5.0], [2.0, 4.0, 7.0], [1.0, 7.0, 4.0]], [[6.0, 2.0, 2.0], [8.0, 3.0, 3.0], [6.0, 4.0, 2.0]], [[1.0, 5.0, 
3.0], [7.0, 4.0, 6.0], [4.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 2.0], [5.0, 3.0, 4.0], [8.0, 2.0, 6.0]], 
[[2.0, 5.0, 8.0], [7.0, 7.0, 7.0], [3.0, 7.0, 3.0]], [[5.0, 6.0, 6.0], [6.0, 8.0, 3.0], [3.0, 2.0, 7.0]]], [[[2.0, 4.0, 6.0], 
[4.0, 8.0, 4.0], [8.0, 8.0, 3.0]], [[3.0, 7.0, 8.0], [4.0, 7.0, 8.0], [3.0, 5.0, 6.0]], [[2.0, 8.0, 1.0], [1.0, 3.0, 2.0], 
[2.0, 3.0, 2.0]]], [[[4.0, 8.0, 3.0], [8.0, 5.0, 7.0], [1.0, 4.0, 4.0]], [[5.0, 3.0, 7.0], [3.0, 8.0, 8.0], [7.0, 6.0, 4.0]], 
[[6.0, 7.0, 7.0], [6.0, 8.0, 1.0], [3.0, 1.0, 6.0]]], [[[3.0, 3.0, 6.0], [2.0, 8.0, 4.0], [7.0, 3.0, 8.0]], [[2.0, 4.0, 4.0], 
[7.0, 3.0, 2.0], [7.0, 8.0, 1.0]], [[7.0, 7.0, 7.0], [3.0, 4.0, 7.0], [2.0, 1.0, 8.0]]]])
      val=numpy.array([[[[7.0, 2.0, 1.0], [7.0, 5.0, 7.0]], [[5.0, 5.0, 6.0], [8.0, 5.0, 4.0]], [[5.0, 4.0, 3.0], [3.0, 8.0, 
3.0]]], [[[3.0, 8.0, 5.0], [1.0, 1.0, 4.0]], [[4.0, 6.0, 8.0], [5.0, 1.0, 4.0]], [[4.0, 4.0, 6.0], [3.0, 7.0, 7.0]]], [[[3.0, 
1.0, 8.0], [5.0, 4.0, 8.0]], [[4.0, 5.0, 1.0], [2.0, 2.0, 4.0]], [[3.0, 4.0, 7.0], [3.0, 6.0, 1.0]]], [[[5.0, 3.0, 3.0], [7.0, 
1.0, 1.0]], [[2.0, 5.0, 4.0], [2.0, 4.0, 8.0]], [[4.0, 2.0, 3.0], [4.0, 8.0, 5.0]]]])
      arg[:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 1.0], [7.0, 5.0, 7.0], [7.0, 5.0, 3.0]], [[5.0, 5.0, 6.0], [8.0, 5.0, 4.0], [1.0, 
8.0, 5.0]], [[5.0, 4.0, 3.0], [3.0, 8.0, 3.0], [4.0, 7.0, 2.0]]], [[[3.0, 8.0, 5.0], [1.0, 1.0, 4.0], [5.0, 8.0, 4.0]], [[4.0, 
6.0, 8.0], [5.0, 1.0, 4.0], [8.0, 4.0, 2.0]], [[4.0, 4.0, 6.0], [3.0, 7.0, 7.0], [1.0, 2.0, 5.0]]], [[[3.0, 1.0, 8.0], [5.0, 
4.0, 8.0], [6.0, 7.0, 4.0]], [[4.0, 5.0, 1.0], [2.0, 2.0, 4.0], [8.0, 4.0, 8.0]], [[3.0, 4.0, 7.0], [3.0, 6.0, 1.0], [5.0, 4.0, 
4.0]]], [[[5.0, 3.0, 3.0], [7.0, 1.0, 1.0], [1.0, 7.0, 4.0]], [[2.0, 5.0, 4.0], [2.0, 4.0, 8.0], [6.0, 4.0, 2.0]], [[4.0, 2.0, 
3.0], [4.0, 8.0, 5.0], [4.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 1.0], [7.0, 5.0, 7.0], [8.0, 2.0, 6.0]], 
[[5.0, 5.0, 6.0], [8.0, 5.0, 4.0], [3.0, 7.0, 3.0]], [[5.0, 4.0, 3.0], [3.0, 8.0, 3.0], [3.0, 2.0, 7.0]]], [[[3.0, 8.0, 5.0], 
[1.0, 1.0, 4.0], [8.0, 8.0, 3.0]], [[4.0, 6.0, 8.0], [5.0, 1.0, 4.0], [3.0, 5.0, 6.0]], [[4.0, 4.0, 6.0], [3.0, 7.0, 7.0], 
[2.0, 3.0, 2.0]]], [[[3.0, 1.0, 8.0], [5.0, 4.0, 8.0], [1.0, 4.0, 4.0]], [[4.0, 5.0, 1.0], [2.0, 2.0, 4.0], [7.0, 6.0, 4.0]], 
[[3.0, 4.0, 7.0], [3.0, 6.0, 1.0], [3.0, 1.0, 6.0]]], [[[5.0, 3.0, 3.0], [7.0, 1.0, 1.0], [7.0, 3.0, 8.0]], [[2.0, 5.0, 4.0], 
[2.0, 4.0, 8.0], [7.0, 8.0, 1.0]], [[4.0, 2.0, 3.0], [4.0, 8.0, 5.0], [2.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 1.0], [3.0, 7.0, 7.0], [2.0, 5.0, 2.0]], [[6.0, 6.0, 2.0], [4.0, 5.0, 3.0], [7.0, 
6.0, 5.0]], [[5.0, 1.0, 1.0], [1.0, 6.0, 7.0], [7.0, 1.0, 5.0]]], [[[5.0, 7.0, 2.0], [8.0, 6.0, 7.0], [7.0, 2.0, 3.0]], [[2.0, 
8.0, 4.0], [1.0, 1.0, 4.0], [4.0, 6.0, 3.0]], [[3.0, 3.0, 4.0], [7.0, 6.0, 8.0], [6.0, 2.0, 4.0]]], [[[4.0, 8.0, 2.0], [1.0, 
8.0, 4.0], [3.0, 3.0, 6.0]], [[3.0, 6.0, 5.0], [7.0, 6.0, 3.0], [7.0, 3.0, 4.0]], [[3.0, 4.0, 8.0], [3.0, 5.0, 1.0], [4.0, 6.0, 
8.0]]], [[[3.0, 6.0, 1.0], [7.0, 8.0, 5.0], [2.0, 1.0, 8.0]], [[8.0, 6.0, 5.0], [5.0, 3.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 8.0, 
1.0], [7.0, 3.0, 2.0], [7.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 2.0], [8.0, 6.0, 7.0], [5.0, 7.0, 8.0]], 
[[5.0, 3.0, 2.0], [6.0, 7.0, 2.0], [3.0, 7.0, 1.0]], [[3.0, 4.0, 8.0], [8.0, 3.0, 5.0], [7.0, 1.0, 7.0]]], [[[7.0, 6.0, 8.0], 
[4.0, 5.0, 1.0], [4.0, 3.0, 2.0]], [[1.0, 4.0, 2.0], [4.0, 4.0, 1.0], [8.0, 1.0, 6.0]], [[2.0, 7.0, 5.0], [5.0, 2.0, 8.0], 
[6.0, 4.0, 2.0]]], [[[2.0, 3.0, 1.0], [6.0, 5.0, 2.0], [5.0, 4.0, 8.0]], [[5.0, 8.0, 2.0], [1.0, 1.0, 3.0], [2.0, 5.0, 7.0]], 
[[2.0, 8.0, 3.0], [5.0, 5.0, 8.0], [1.0, 6.0, 6.0]]], [[[8.0, 6.0, 8.0], [3.0, 4.0, 1.0], [8.0, 7.0, 6.0]], [[6.0, 1.0, 6.0], 
[4.0, 6.0, 5.0], [1.0, 7.0, 2.0]], [[2.0, 7.0, 8.0], [8.0, 3.0, 8.0], [1.0, 3.0, 6.0]]]])
      val=Data(numpy.array([[[[2.0, 6.0, 7.0], [3.0, 6.0, 8.0]], [[5.0, 4.0, 1.0], [8.0, 6.0, 1.0]], [[8.0, 4.0, 4.0], [8.0, 
6.0, 1.0]]], [[[7.0, 4.0, 7.0], [2.0, 5.0, 3.0]], [[3.0, 4.0, 5.0], [4.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], [6.0, 8.0, 5.0]]], 
[[[6.0, 6.0, 7.0], [2.0, 5.0, 2.0]], [[6.0, 3.0, 1.0], [4.0, 3.0, 3.0]], [[6.0, 8.0, 2.0], [1.0, 6.0, 5.0]]], [[[3.0, 5.0, 
8.0], [4.0, 2.0, 6.0]], [[8.0, 5.0, 1.0], [6.0, 7.0, 1.0]], [[6.0, 1.0, 8.0], [1.0, 5.0, 7.0]]]]),self.functionspace)
      arg[:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 7.0], [3.0, 6.0, 8.0], [2.0, 5.0, 2.0]], [[5.0, 4.0, 1.0], [8.0, 6.0, 1.0], [7.0, 
6.0, 5.0]], [[8.0, 4.0, 4.0], [8.0, 6.0, 1.0], [7.0, 1.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 5.0, 3.0], [7.0, 2.0, 3.0]], [[3.0, 
4.0, 5.0], [4.0, 1.0, 1.0], [4.0, 6.0, 3.0]], [[3.0, 2.0, 3.0], [6.0, 8.0, 5.0], [6.0, 2.0, 4.0]]], [[[6.0, 6.0, 7.0], [2.0, 
5.0, 2.0], [3.0, 3.0, 6.0]], [[6.0, 3.0, 1.0], [4.0, 3.0, 3.0], [7.0, 3.0, 4.0]], [[6.0, 8.0, 2.0], [1.0, 6.0, 5.0], [4.0, 6.0, 
8.0]]], [[[3.0, 5.0, 8.0], [4.0, 2.0, 6.0], [2.0, 1.0, 8.0]], [[8.0, 5.0, 1.0], [6.0, 7.0, 1.0], [5.0, 6.0, 4.0]], [[6.0, 1.0, 
8.0], [1.0, 5.0, 7.0], [7.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 7.0], [3.0, 6.0, 8.0], [5.0, 7.0, 8.0]], 
[[5.0, 4.0, 1.0], [8.0, 6.0, 1.0], [3.0, 7.0, 1.0]], [[8.0, 4.0, 4.0], [8.0, 6.0, 1.0], [7.0, 1.0, 7.0]]], [[[7.0, 4.0, 7.0], 
[2.0, 5.0, 3.0], [4.0, 3.0, 2.0]], [[3.0, 4.0, 5.0], [4.0, 1.0, 1.0], [8.0, 1.0, 6.0]], [[3.0, 2.0, 3.0], [6.0, 8.0, 5.0], 
[6.0, 4.0, 2.0]]], [[[6.0, 6.0, 7.0], [2.0, 5.0, 2.0], [5.0, 4.0, 8.0]], [[6.0, 3.0, 1.0], [4.0, 3.0, 3.0], [2.0, 5.0, 7.0]], 
[[6.0, 8.0, 2.0], [1.0, 6.0, 5.0], [1.0, 6.0, 6.0]]], [[[3.0, 5.0, 8.0], [4.0, 2.0, 6.0], [8.0, 7.0, 6.0]], [[8.0, 5.0, 1.0], 
[6.0, 7.0, 1.0], [1.0, 7.0, 2.0]], [[6.0, 1.0, 8.0], [1.0, 5.0, 7.0], [1.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 4.0], [6.0, 5.0, 7.0], [6.0, 8.0, 5.0]], [[1.0, 8.0, 3.0], [8.0, 4.0, 6.0], [8.0, 
1.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 3.0, 7.0], [5.0, 8.0, 2.0]]], [[[8.0, 6.0, 8.0], [7.0, 5.0, 2.0], [6.0, 3.0, 1.0]], [[4.0, 
5.0, 2.0], [3.0, 3.0, 4.0], [1.0, 1.0, 5.0]], [[5.0, 3.0, 1.0], [5.0, 2.0, 8.0], [8.0, 8.0, 8.0]]], [[[5.0, 4.0, 1.0], [4.0, 
3.0, 2.0], [4.0, 1.0, 5.0]], [[4.0, 5.0, 5.0], [7.0, 3.0, 6.0], [1.0, 6.0, 4.0]], [[7.0, 2.0, 5.0], [1.0, 5.0, 1.0], [8.0, 1.0, 
1.0]]], [[[3.0, 3.0, 3.0], [8.0, 7.0, 2.0], [2.0, 4.0, 3.0]], [[3.0, 3.0, 8.0], [4.0, 1.0, 6.0], [1.0, 3.0, 3.0]], [[4.0, 1.0, 
1.0], [7.0, 7.0, 2.0], [4.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 7.0], [7.0, 1.0, 2.0], [2.0, 5.0, 8.0]], 
[[5.0, 6.0, 3.0], [2.0, 4.0, 5.0], [3.0, 5.0, 3.0]], [[5.0, 4.0, 7.0], [6.0, 6.0, 8.0], [6.0, 5.0, 2.0]]], [[[2.0, 7.0, 4.0], 
[4.0, 8.0, 7.0], [7.0, 3.0, 1.0]], [[8.0, 1.0, 5.0], [5.0, 7.0, 4.0], [3.0, 3.0, 8.0]], [[6.0, 4.0, 4.0], [5.0, 4.0, 5.0], 
[5.0, 3.0, 4.0]]], [[[1.0, 7.0, 5.0], [7.0, 3.0, 5.0], [4.0, 4.0, 2.0]], [[5.0, 6.0, 5.0], [1.0, 6.0, 5.0], [4.0, 4.0, 8.0]], 
[[4.0, 5.0, 1.0], [6.0, 8.0, 3.0], [1.0, 7.0, 1.0]]], [[[8.0, 4.0, 3.0], [7.0, 2.0, 8.0], [6.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], 
[2.0, 8.0, 5.0], [3.0, 4.0, 4.0]], [[5.0, 2.0, 2.0], [3.0, 7.0, 2.0], [6.0, 8.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 6.0, 7.0], [4.0, 3.0, 1.0]], [[7.0, 4.0, 2.0], [5.0, 5.0, 3.0]], [[4.0, 6.0, 8.0], [2.0, 
5.0, 3.0]]], [[[1.0, 3.0, 7.0], [2.0, 7.0, 4.0]], [[6.0, 2.0, 8.0], [6.0, 3.0, 2.0]], [[5.0, 8.0, 6.0], [8.0, 5.0, 3.0]]], 
[[[1.0, 2.0, 8.0], [5.0, 5.0, 2.0]], [[6.0, 2.0, 3.0], [7.0, 4.0, 5.0]], [[7.0, 8.0, 5.0], [7.0, 6.0, 8.0]]], [[[5.0, 8.0, 
6.0], [7.0, 4.0, 4.0]], [[1.0, 3.0, 3.0], [1.0, 6.0, 6.0]], [[6.0, 8.0, 1.0], [6.0, 4.0, 
1.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 8.0, 2.0], [7.0, 7.0, 8.0]], [[4.0, 8.0, 4.0], [4.0, 4.0, 7.0]], [[1.0, 3.0, 8.0], 
[7.0, 1.0, 4.0]]], [[[3.0, 2.0, 6.0], [4.0, 1.0, 8.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 8.0]], [[1.0, 2.0, 5.0], [2.0, 3.0, 
8.0]]], [[[6.0, 7.0, 3.0], [8.0, 8.0, 4.0]], [[6.0, 3.0, 1.0], [1.0, 7.0, 8.0]], [[7.0, 7.0, 2.0], [2.0, 4.0, 5.0]]], [[[6.0, 
8.0, 6.0], [4.0, 8.0, 8.0]], [[5.0, 6.0, 3.0], [5.0, 8.0, 8.0]], [[5.0, 4.0, 1.0], [5.0, 2.0, 2.0]]]])
      arg[:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 7.0], [4.0, 3.0, 1.0], [6.0, 8.0, 5.0]], [[7.0, 4.0, 2.0], [5.0, 5.0, 3.0], [8.0, 
1.0, 1.0]], [[4.0, 6.0, 8.0], [2.0, 5.0, 3.0], [5.0, 8.0, 2.0]]], [[[1.0, 3.0, 7.0], [2.0, 7.0, 4.0], [6.0, 3.0, 1.0]], [[6.0, 
2.0, 8.0], [6.0, 3.0, 2.0], [1.0, 1.0, 5.0]], [[5.0, 8.0, 6.0], [8.0, 5.0, 3.0], [8.0, 8.0, 8.0]]], [[[1.0, 2.0, 8.0], [5.0, 
5.0, 2.0], [4.0, 1.0, 5.0]], [[6.0, 2.0, 3.0], [7.0, 4.0, 5.0], [1.0, 6.0, 4.0]], [[7.0, 8.0, 5.0], [7.0, 6.0, 8.0], [8.0, 1.0, 
1.0]]], [[[5.0, 8.0, 6.0], [7.0, 4.0, 4.0], [2.0, 4.0, 3.0]], [[1.0, 3.0, 3.0], [1.0, 6.0, 6.0], [1.0, 3.0, 3.0]], [[6.0, 8.0, 
1.0], [6.0, 4.0, 1.0], [4.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 2.0], [7.0, 7.0, 8.0], [2.0, 5.0, 8.0]], 
[[4.0, 8.0, 4.0], [4.0, 4.0, 7.0], [3.0, 5.0, 3.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 4.0], [6.0, 5.0, 2.0]]], [[[3.0, 2.0, 6.0], 
[4.0, 1.0, 8.0], [7.0, 3.0, 1.0]], [[1.0, 2.0, 1.0], [3.0, 7.0, 8.0], [3.0, 3.0, 8.0]], [[1.0, 2.0, 5.0], [2.0, 3.0, 8.0], 
[5.0, 3.0, 4.0]]], [[[6.0, 7.0, 3.0], [8.0, 8.0, 4.0], [4.0, 4.0, 2.0]], [[6.0, 3.0, 1.0], [1.0, 7.0, 8.0], [4.0, 4.0, 8.0]], 
[[7.0, 7.0, 2.0], [2.0, 4.0, 5.0], [1.0, 7.0, 1.0]]], [[[6.0, 8.0, 6.0], [4.0, 8.0, 8.0], [6.0, 6.0, 4.0]], [[5.0, 6.0, 3.0], 
[5.0, 8.0, 8.0], [3.0, 4.0, 4.0]], [[5.0, 4.0, 1.0], [5.0, 2.0, 2.0], [6.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 2.0], [2.0, 3.0, 8.0], [7.0, 2.0, 5.0]], [[8.0, 3.0, 8.0], [8.0, 4.0, 4.0], [8.0, 
7.0, 3.0]], [[5.0, 4.0, 1.0], [3.0, 2.0, 2.0], [3.0, 3.0, 7.0]]], [[[3.0, 8.0, 8.0], [3.0, 6.0, 3.0], [3.0, 3.0, 8.0]], [[2.0, 
4.0, 5.0], [5.0, 8.0, 3.0], [6.0, 5.0, 1.0]], [[6.0, 5.0, 1.0], [2.0, 4.0, 7.0], [1.0, 2.0, 5.0]]], [[[1.0, 1.0, 4.0], [6.0, 
8.0, 8.0], [3.0, 5.0, 1.0]], [[3.0, 2.0, 5.0], [2.0, 5.0, 6.0], [5.0, 8.0, 3.0]], [[6.0, 4.0, 4.0], [7.0, 2.0, 5.0], [7.0, 3.0, 
7.0]]], [[[1.0, 5.0, 4.0], [1.0, 3.0, 8.0], [8.0, 5.0, 3.0]], [[5.0, 2.0, 4.0], [1.0, 5.0, 5.0], [6.0, 8.0, 7.0]], [[4.0, 2.0, 
4.0], [2.0, 6.0, 5.0], [7.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 2.0], [7.0, 5.0, 6.0], [3.0, 1.0, 2.0]], 
[[1.0, 3.0, 4.0], [6.0, 1.0, 6.0], [7.0, 3.0, 4.0]], [[3.0, 1.0, 2.0], [4.0, 7.0, 3.0], [4.0, 7.0, 1.0]]], [[[7.0, 4.0, 1.0], 
[5.0, 8.0, 2.0], [1.0, 3.0, 5.0]], [[7.0, 4.0, 2.0], [1.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[6.0, 8.0, 6.0], [2.0, 8.0, 8.0], 
[1.0, 2.0, 4.0]]], [[[2.0, 8.0, 6.0], [7.0, 3.0, 3.0], [2.0, 1.0, 7.0]], [[5.0, 5.0, 7.0], [7.0, 5.0, 4.0], [7.0, 2.0, 8.0]], 
[[1.0, 1.0, 4.0], [7.0, 8.0, 3.0], [2.0, 1.0, 8.0]]], [[[1.0, 8.0, 1.0], [1.0, 1.0, 2.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 1.0], 
[3.0, 2.0, 1.0], [4.0, 4.0, 8.0]], [[1.0, 2.0, 8.0], [4.0, 4.0, 2.0], [4.0, 5.0, 3.0]]]])
      val=numpy.array([[[[3.0, 4.0, 3.0], [2.0, 7.0, 3.0], [5.0, 5.0, 4.0]], [[8.0, 4.0, 7.0], [7.0, 4.0, 5.0], [2.0, 8.0, 
7.0]], [[2.0, 5.0, 6.0], [8.0, 3.0, 5.0], [7.0, 1.0, 7.0]]], [[[3.0, 6.0, 6.0], [4.0, 2.0, 1.0], [1.0, 4.0, 2.0]], [[7.0, 5.0, 
8.0], [4.0, 7.0, 5.0], [5.0, 1.0, 5.0]], [[3.0, 6.0, 7.0], [3.0, 6.0, 8.0], [8.0, 6.0, 3.0]]], [[[7.0, 2.0, 4.0], [2.0, 5.0, 
1.0], [3.0, 5.0, 5.0]], [[7.0, 7.0, 3.0], [8.0, 8.0, 7.0], [3.0, 5.0, 2.0]], [[1.0, 6.0, 1.0], [3.0, 1.0, 4.0], [2.0, 8.0, 
3.0]]], [[[4.0, 5.0, 4.0], [4.0, 2.0, 5.0], [6.0, 7.0, 5.0]], [[1.0, 3.0, 4.0], [4.0, 4.0, 2.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 
1.0], [6.0, 3.0, 5.0], [6.0, 5.0, 7.0]]]])
      arg[:,:,:,:]=val
      ref=Data(numpy.array([[[[3.0, 4.0, 3.0], [2.0, 7.0, 3.0], [5.0, 5.0, 4.0]], [[8.0, 4.0, 7.0], [7.0, 4.0, 5.0], [2.0, 8.0, 
7.0]], [[2.0, 5.0, 6.0], [8.0, 3.0, 5.0], [7.0, 1.0, 7.0]]], [[[3.0, 6.0, 6.0], [4.0, 2.0, 1.0], [1.0, 4.0, 2.0]], [[7.0, 5.0, 
8.0], [4.0, 7.0, 5.0], [5.0, 1.0, 5.0]], [[3.0, 6.0, 7.0], [3.0, 6.0, 8.0], [8.0, 6.0, 3.0]]], [[[7.0, 2.0, 4.0], [2.0, 5.0, 
1.0], [3.0, 5.0, 5.0]], [[7.0, 7.0, 3.0], [8.0, 8.0, 7.0], [3.0, 5.0, 2.0]], [[1.0, 6.0, 1.0], [3.0, 1.0, 4.0], [2.0, 8.0, 
3.0]]], [[[4.0, 5.0, 4.0], [4.0, 2.0, 5.0], [6.0, 7.0, 5.0]], [[1.0, 3.0, 4.0], [4.0, 4.0, 2.0], [7.0, 1.0, 6.0]], [[1.0, 2.0, 
1.0], [6.0, 3.0, 5.0], [6.0, 5.0, 7.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 4.0], [7.0, 1.0, 7.0], [5.0, 4.0, 6.0]], [[4.0, 2.0, 5.0], [4.0, 8.0, 7.0], [7.0, 
3.0, 7.0]], [[7.0, 1.0, 3.0], [7.0, 1.0, 8.0], [8.0, 8.0, 6.0]]], [[[4.0, 3.0, 4.0], [4.0, 2.0, 4.0], [7.0, 7.0, 8.0]], [[1.0, 
6.0, 8.0], [8.0, 6.0, 8.0], [8.0, 2.0, 1.0]], [[8.0, 3.0, 6.0], [5.0, 4.0, 6.0], [3.0, 3.0, 1.0]]], [[[5.0, 3.0, 1.0], [5.0, 
5.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 3.0, 1.0], [5.0, 4.0, 2.0]], [[4.0, 7.0, 3.0], [4.0, 5.0, 2.0], [7.0, 4.0, 
2.0]]], [[[2.0, 3.0, 1.0], [8.0, 4.0, 6.0], [6.0, 5.0, 5.0]], [[8.0, 8.0, 7.0], [7.0, 4.0, 2.0], [2.0, 5.0, 7.0]], [[2.0, 5.0, 
2.0], [3.0, 8.0, 5.0], [7.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 6.0], [8.0, 5.0, 8.0], [4.0, 8.0, 7.0]], 
[[2.0, 4.0, 3.0], [1.0, 7.0, 6.0], [5.0, 3.0, 6.0]], [[3.0, 5.0, 3.0], [1.0, 1.0, 8.0], [6.0, 5.0, 1.0]]], [[[2.0, 7.0, 4.0], 
[3.0, 4.0, 7.0], [3.0, 5.0, 6.0]], [[5.0, 4.0, 5.0], [3.0, 3.0, 7.0], [8.0, 5.0, 3.0]], [[5.0, 5.0, 2.0], [2.0, 5.0, 2.0], 
[7.0, 3.0, 5.0]]], [[[4.0, 3.0, 4.0], [4.0, 6.0, 3.0], [1.0, 1.0, 1.0]], [[5.0, 5.0, 8.0], [5.0, 7.0, 8.0], [7.0, 1.0, 8.0]], 
[[8.0, 8.0, 7.0], [2.0, 8.0, 7.0], [1.0, 1.0, 7.0]]], [[[3.0, 5.0, 3.0], [3.0, 1.0, 8.0], [3.0, 4.0, 7.0]], [[6.0, 3.0, 5.0], 
[4.0, 2.0, 6.0], [4.0, 8.0, 7.0]], [[1.0, 6.0, 5.0], [2.0, 8.0, 7.0], [8.0, 1.0, 1.0]]]])
      val=Data(numpy.array([[[[7.0, 4.0, 1.0], [7.0, 7.0, 3.0], [7.0, 2.0, 5.0]], [[7.0, 6.0, 1.0], [2.0, 4.0, 4.0], [3.0, 1.0, 
7.0]], [[2.0, 1.0, 6.0], [1.0, 1.0, 3.0], [2.0, 5.0, 6.0]]], [[[8.0, 6.0, 3.0], [6.0, 4.0, 6.0], [5.0, 1.0, 5.0]], [[8.0, 6.0, 
7.0], [1.0, 8.0, 7.0], [1.0, 5.0, 5.0]], [[7.0, 6.0, 6.0], [4.0, 5.0, 8.0], [1.0, 2.0, 8.0]]], [[[7.0, 5.0, 2.0], [2.0, 1.0, 
3.0], [7.0, 4.0, 4.0]], [[5.0, 6.0, 4.0], [1.0, 1.0, 8.0], [4.0, 8.0, 8.0]], [[3.0, 1.0, 1.0], [1.0, 8.0, 8.0], [1.0, 3.0, 
2.0]]], [[[2.0, 5.0, 7.0], [2.0, 5.0, 6.0], [5.0, 5.0, 5.0]], [[4.0, 4.0, 4.0], [1.0, 3.0, 6.0], [8.0, 8.0, 1.0]], [[5.0, 6.0, 
8.0], [5.0, 7.0, 8.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      arg[:,:,:,:]=val
      ref=Data(numpy.array([[[[7.0, 4.0, 1.0], [7.0, 7.0, 3.0], [7.0, 2.0, 5.0]], [[7.0, 6.0, 1.0], [2.0, 4.0, 4.0], [3.0, 1.0, 
7.0]], [[2.0, 1.0, 6.0], [1.0, 1.0, 3.0], [2.0, 5.0, 6.0]]], [[[8.0, 6.0, 3.0], [6.0, 4.0, 6.0], [5.0, 1.0, 5.0]], [[8.0, 6.0, 
7.0], [1.0, 8.0, 7.0], [1.0, 5.0, 5.0]], [[7.0, 6.0, 6.0], [4.0, 5.0, 8.0], [1.0, 2.0, 8.0]]], [[[7.0, 5.0, 2.0], [2.0, 1.0, 
3.0], [7.0, 4.0, 4.0]], [[5.0, 6.0, 4.0], [1.0, 1.0, 8.0], [4.0, 8.0, 8.0]], [[3.0, 1.0, 1.0], [1.0, 8.0, 8.0], [1.0, 3.0, 
2.0]]], [[[2.0, 5.0, 7.0], [2.0, 5.0, 6.0], [5.0, 5.0, 5.0]], [[4.0, 4.0, 4.0], [1.0, 3.0, 6.0], [8.0, 8.0, 1.0]], [[5.0, 6.0, 
8.0], [5.0, 7.0, 8.0], [5.0, 7.0, 5.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCCC(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 6.0], [6.0, 4.0, 3.0], [5.0, 6.0, 8.0]], [[3.0, 4.0, 1.0], [8.0, 8.0, 7.0], [4.0, 
5.0, 2.0]], [[2.0, 7.0, 2.0], [2.0, 2.0, 8.0], [4.0, 7.0, 8.0]]], [[[4.0, 2.0, 4.0], [7.0, 5.0, 6.0], [4.0, 3.0, 6.0]], [[6.0, 
8.0, 1.0], [8.0, 6.0, 8.0], [8.0, 1.0, 2.0]], [[2.0, 4.0, 7.0], [7.0, 5.0, 6.0], [4.0, 3.0, 7.0]]], [[[1.0, 5.0, 4.0], [2.0, 
6.0, 5.0], [5.0, 7.0, 6.0]], [[5.0, 2.0, 3.0], [3.0, 4.0, 1.0], [4.0, 3.0, 6.0]], [[3.0, 2.0, 8.0], [8.0, 2.0, 4.0], [5.0, 2.0, 
8.0]]], [[[4.0, 5.0, 3.0], [4.0, 4.0, 2.0], [4.0, 1.0, 1.0]], [[8.0, 7.0, 4.0], [7.0, 3.0, 8.0], [5.0, 2.0, 2.0]], [[3.0, 6.0, 
2.0], [7.0, 3.0, 4.0], [8.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 5.0], [2.0, 6.0, 8.0], [5.0, 8.0, 7.0]], 
[[7.0, 4.0, 1.0], [2.0, 3.0, 1.0], [5.0, 7.0, 7.0]], [[6.0, 7.0, 4.0], [5.0, 4.0, 8.0], [4.0, 3.0, 7.0]]], [[[1.0, 1.0, 5.0], 
[1.0, 4.0, 7.0], [6.0, 4.0, 7.0]], [[7.0, 6.0, 3.0], [7.0, 2.0, 5.0], [5.0, 5.0, 8.0]], [[3.0, 7.0, 8.0], [3.0, 2.0, 3.0], 
[1.0, 1.0, 7.0]]], [[[5.0, 1.0, 6.0], [4.0, 5.0, 4.0], [4.0, 5.0, 7.0]], [[1.0, 5.0, 7.0], [1.0, 4.0, 5.0], [5.0, 3.0, 5.0]], 
[[5.0, 1.0, 3.0], [8.0, 1.0, 1.0], [1.0, 3.0, 3.0]]], [[[4.0, 1.0, 8.0], [7.0, 7.0, 8.0], [2.0, 1.0, 3.0]], [[5.0, 2.0, 7.0], 
[6.0, 4.0, 1.0], [4.0, 5.0, 3.0]], [[4.0, 7.0, 3.0], [3.0, 4.0, 2.0], [2.0, 5.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 5.0, 1.0], [2.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[3.0, 1.0, 2.0], [6.0, 1.0, 3.0], [1.0, 
4.0, 7.0]], [[7.0, 4.0, 3.0], [6.0, 1.0, 6.0], [8.0, 7.0, 3.0]]], [[[6.0, 8.0, 6.0], [4.0, 2.0, 2.0], [1.0, 8.0, 4.0]], [[8.0, 
5.0, 5.0], [2.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[1.0, 1.0, 7.0], [1.0, 1.0, 5.0], [4.0, 1.0, 3.0]]], [[[5.0, 8.0, 1.0], [5.0, 
8.0, 4.0], [2.0, 1.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 6.0], [8.0, 6.0, 
3.0]]], [[[4.0, 4.0, 6.0], [4.0, 6.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 3.0, 2.0], [3.0, 6.0, 3.0], [3.0, 5.0, 8.0]], [[2.0, 2.0, 
1.0], [6.0, 8.0, 6.0], [2.0, 1.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 6.0, 3.0], [5.0, 4.0, 8.0], [8.0, 3.0, 4.0]], 
[[3.0, 4.0, 8.0], [8.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[2.0, 2.0, 5.0], [2.0, 1.0, 5.0], [4.0, 5.0, 4.0]]], [[[2.0, 8.0, 3.0], 
[4.0, 5.0, 6.0], [5.0, 8.0, 6.0]], [[3.0, 4.0, 8.0], [1.0, 5.0, 3.0], [6.0, 6.0, 3.0]], [[4.0, 7.0, 1.0], [7.0, 8.0, 2.0], 
[8.0, 5.0, 7.0]]], [[[6.0, 3.0, 6.0], [8.0, 1.0, 1.0], [6.0, 2.0, 8.0]], [[1.0, 7.0, 8.0], [7.0, 6.0, 3.0], [6.0, 1.0, 8.0]], 
[[2.0, 3.0, 5.0], [1.0, 5.0, 3.0], [5.0, 6.0, 1.0]]], [[[8.0, 6.0, 5.0], [1.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 2.0, 3.0], 
[7.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[3.0, 5.0, 2.0], [2.0, 4.0, 1.0], [4.0, 8.0, 3.0]]]])
      arg[:,:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 1.0], [2.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[3.0, 1.0, 2.0], [6.0, 1.0, 3.0], [1.0, 
4.0, 7.0]], [[7.0, 4.0, 3.0], [6.0, 1.0, 6.0], [8.0, 7.0, 3.0]]], [[[6.0, 8.0, 6.0], [4.0, 2.0, 2.0], [1.0, 8.0, 4.0]], [[8.0, 
5.0, 5.0], [2.0, 2.0, 3.0], [8.0, 2.0, 7.0]], [[1.0, 1.0, 7.0], [1.0, 1.0, 5.0], [4.0, 1.0, 3.0]]], [[[5.0, 8.0, 1.0], [5.0, 
8.0, 4.0], [2.0, 1.0, 2.0]], [[8.0, 1.0, 6.0], [1.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[4.0, 4.0, 4.0], [4.0, 8.0, 6.0], [8.0, 6.0, 
3.0]]], [[[4.0, 4.0, 6.0], [4.0, 6.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 3.0, 2.0], [3.0, 6.0, 3.0], [3.0, 5.0, 8.0]], [[2.0, 2.0, 
1.0], [6.0, 8.0, 6.0], [2.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 3.0], [5.0, 4.0, 8.0], [8.0, 3.0, 4.0]], 
[[3.0, 4.0, 8.0], [8.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[2.0, 2.0, 5.0], [2.0, 1.0, 5.0], [4.0, 5.0, 4.0]]], [[[2.0, 8.0, 3.0], 
[4.0, 5.0, 6.0], [5.0, 8.0, 6.0]], [[3.0, 4.0, 8.0], [1.0, 5.0, 3.0], [6.0, 6.0, 3.0]], [[4.0, 7.0, 1.0], [7.0, 8.0, 2.0], 
[8.0, 5.0, 7.0]]], [[[6.0, 3.0, 6.0], [8.0, 1.0, 1.0], [6.0, 2.0, 8.0]], [[1.0, 7.0, 8.0], [7.0, 6.0, 3.0], [6.0, 1.0, 8.0]], 
[[2.0, 3.0, 5.0], [1.0, 5.0, 3.0], [5.0, 6.0, 1.0]]], [[[8.0, 6.0, 5.0], [1.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[3.0, 2.0, 3.0], 
[7.0, 3.0, 6.0], [2.0, 3.0, 5.0]], [[3.0, 5.0, 2.0], [2.0, 4.0, 1.0], [4.0, 8.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 7.0], [5.0, 7.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 3.0, 8.0], [2.0, 7.0, 1.0], [4.0, 
6.0, 6.0]], [[3.0, 4.0, 6.0], [2.0, 3.0, 6.0], [7.0, 5.0, 4.0]]], [[[8.0, 2.0, 7.0], [6.0, 7.0, 8.0], [2.0, 4.0, 2.0]], [[7.0, 
5.0, 3.0], [8.0, 6.0, 5.0], [2.0, 7.0, 4.0]], [[1.0, 8.0, 3.0], [6.0, 2.0, 7.0], [5.0, 2.0, 1.0]]], [[[1.0, 4.0, 1.0], [3.0, 
1.0, 1.0], [7.0, 6.0, 2.0]], [[7.0, 5.0, 4.0], [4.0, 7.0, 1.0], [1.0, 6.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 8.0, 1.0], [6.0, 3.0, 
4.0]]], [[[3.0, 7.0, 2.0], [2.0, 7.0, 6.0], [2.0, 4.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 3.0, 7.0], [3.0, 3.0, 4.0]], [[5.0, 7.0, 
4.0], [7.0, 6.0, 1.0], [2.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 6.0], [1.0, 1.0, 6.0], [3.0, 3.0, 7.0]], 
[[5.0, 7.0, 7.0], [5.0, 5.0, 3.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 7.0], [4.0, 4.0, 3.0], [6.0, 8.0, 4.0]]], [[[8.0, 6.0, 3.0], 
[1.0, 6.0, 7.0], [2.0, 4.0, 6.0]], [[5.0, 4.0, 2.0], [7.0, 4.0, 6.0], [3.0, 6.0, 3.0]], [[4.0, 8.0, 2.0], [5.0, 4.0, 4.0], 
[2.0, 3.0, 2.0]]], [[[7.0, 1.0, 3.0], [2.0, 2.0, 6.0], [6.0, 4.0, 4.0]], [[7.0, 1.0, 2.0], [4.0, 5.0, 1.0], [7.0, 4.0, 7.0]], 
[[7.0, 3.0, 1.0], [3.0, 2.0, 5.0], [5.0, 3.0, 4.0]]], [[[7.0, 3.0, 2.0], [2.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[7.0, 3.0, 4.0], 
[6.0, 2.0, 5.0], [3.0, 3.0, 2.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 5.0], [1.0, 5.0, 3.0]]]])
      val=numpy.array(1.0)
      arg[2,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 7.0], [5.0, 7.0, 2.0], [1.0, 5.0, 8.0]], [[5.0, 3.0, 8.0], [2.0, 7.0, 1.0], [4.0, 
6.0, 6.0]], [[3.0, 4.0, 6.0], [2.0, 3.0, 6.0], [7.0, 5.0, 4.0]]], [[[8.0, 2.0, 7.0], [6.0, 7.0, 8.0], [2.0, 4.0, 2.0]], [[7.0, 
5.0, 3.0], [8.0, 6.0, 5.0], [2.0, 7.0, 4.0]], [[1.0, 8.0, 3.0], [6.0, 2.0, 7.0], [5.0, 2.0, 1.0]]], [[[1.0, 4.0, 1.0], [3.0, 
1.0, 1.0], [7.0, 6.0, 2.0]], [[7.0, 5.0, 4.0], [4.0, 1.0, 1.0], [1.0, 6.0, 1.0]], [[3.0, 5.0, 5.0], [1.0, 8.0, 1.0], [6.0, 3.0, 
4.0]]], [[[3.0, 7.0, 2.0], [2.0, 7.0, 6.0], [2.0, 4.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 3.0, 7.0], [3.0, 3.0, 4.0]], [[5.0, 7.0, 
4.0], [7.0, 6.0, 1.0], [2.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 6.0], [1.0, 1.0, 6.0], [3.0, 3.0, 7.0]], 
[[5.0, 7.0, 7.0], [5.0, 5.0, 3.0], [3.0, 3.0, 3.0]], [[1.0, 5.0, 7.0], [4.0, 4.0, 3.0], [6.0, 8.0, 4.0]]], [[[8.0, 6.0, 3.0], 
[1.0, 6.0, 7.0], [2.0, 4.0, 6.0]], [[5.0, 4.0, 2.0], [7.0, 4.0, 6.0], [3.0, 6.0, 3.0]], [[4.0, 8.0, 2.0], [5.0, 4.0, 4.0], 
[2.0, 3.0, 2.0]]], [[[7.0, 1.0, 3.0], [2.0, 2.0, 6.0], [6.0, 4.0, 4.0]], [[7.0, 1.0, 2.0], [4.0, 1.0, 1.0], [7.0, 4.0, 7.0]], 
[[7.0, 3.0, 1.0], [3.0, 2.0, 5.0], [5.0, 3.0, 4.0]]], [[[7.0, 3.0, 2.0], [2.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[7.0, 3.0, 4.0], 
[6.0, 2.0, 5.0], [3.0, 3.0, 2.0]], [[7.0, 1.0, 3.0], [7.0, 3.0, 5.0], [1.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 1.0], [3.0, 1.0, 6.0], [8.0, 7.0, 1.0]], [[5.0, 1.0, 8.0], [7.0, 1.0, 6.0], [2.0, 
5.0, 1.0]], [[4.0, 7.0, 2.0], [4.0, 4.0, 4.0], [8.0, 3.0, 1.0]]], [[[4.0, 5.0, 5.0], [7.0, 3.0, 3.0], [1.0, 4.0, 1.0]], [[7.0, 
2.0, 6.0], [3.0, 8.0, 7.0], [3.0, 6.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 1.0, 7.0], [1.0, 4.0, 8.0]]], [[[1.0, 1.0, 5.0], [5.0, 
3.0, 8.0], [3.0, 5.0, 5.0]], [[3.0, 2.0, 8.0], [2.0, 5.0, 3.0], [7.0, 6.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 2.0, 8.0], [4.0, 5.0, 
1.0]]], [[[6.0, 5.0, 3.0], [6.0, 2.0, 7.0], [2.0, 1.0, 3.0]], [[2.0, 6.0, 6.0], [3.0, 7.0, 3.0], [7.0, 7.0, 2.0]], [[7.0, 6.0, 
2.0], [8.0, 8.0, 8.0], [1.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 6.0], [8.0, 8.0, 4.0], [3.0, 2.0, 6.0]], 
[[3.0, 3.0, 5.0], [5.0, 6.0, 7.0], [3.0, 7.0, 5.0]], [[5.0, 4.0, 4.0], [2.0, 7.0, 5.0], [4.0, 2.0, 6.0]]], [[[3.0, 7.0, 1.0], 
[7.0, 5.0, 4.0], [2.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [7.0, 4.0, 3.0], [7.0, 5.0, 4.0]], [[5.0, 2.0, 2.0], [4.0, 5.0, 4.0], 
[5.0, 5.0, 4.0]]], [[[4.0, 7.0, 7.0], [7.0, 4.0, 2.0], [6.0, 3.0, 7.0]], [[6.0, 7.0, 8.0], [6.0, 6.0, 7.0], [3.0, 7.0, 6.0]], 
[[4.0, 7.0, 6.0], [7.0, 3.0, 3.0], [3.0, 8.0, 1.0]]], [[[3.0, 3.0, 6.0], [7.0, 3.0, 1.0], [2.0, 2.0, 7.0]], [[2.0, 6.0, 7.0], 
[3.0, 4.0, 6.0], [6.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [7.0, 8.0, 3.0], [8.0, 8.0, 8.0]]]])
      val=Data(1.0,self.functionspace)
      arg[2,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 1.0], [3.0, 1.0, 6.0], [8.0, 7.0, 1.0]], [[5.0, 1.0, 8.0], [7.0, 1.0, 6.0], [2.0, 
5.0, 1.0]], [[4.0, 7.0, 2.0], [4.0, 4.0, 4.0], [8.0, 3.0, 1.0]]], [[[4.0, 5.0, 5.0], [7.0, 3.0, 3.0], [1.0, 4.0, 1.0]], [[7.0, 
2.0, 6.0], [3.0, 8.0, 7.0], [3.0, 6.0, 4.0]], [[4.0, 5.0, 8.0], [8.0, 1.0, 7.0], [1.0, 4.0, 8.0]]], [[[1.0, 1.0, 5.0], [5.0, 
3.0, 8.0], [3.0, 5.0, 5.0]], [[3.0, 2.0, 8.0], [2.0, 1.0, 3.0], [7.0, 6.0, 1.0]], [[4.0, 2.0, 2.0], [8.0, 2.0, 8.0], [4.0, 5.0, 
1.0]]], [[[6.0, 5.0, 3.0], [6.0, 2.0, 7.0], [2.0, 1.0, 3.0]], [[2.0, 6.0, 6.0], [3.0, 7.0, 3.0], [7.0, 7.0, 2.0]], [[7.0, 6.0, 
2.0], [8.0, 8.0, 8.0], [1.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 6.0], [8.0, 8.0, 4.0], [3.0, 2.0, 6.0]], 
[[3.0, 3.0, 5.0], [5.0, 6.0, 7.0], [3.0, 7.0, 5.0]], [[5.0, 4.0, 4.0], [2.0, 7.0, 5.0], [4.0, 2.0, 6.0]]], [[[3.0, 7.0, 1.0], 
[7.0, 5.0, 4.0], [2.0, 8.0, 5.0]], [[8.0, 4.0, 5.0], [7.0, 4.0, 3.0], [7.0, 5.0, 4.0]], [[5.0, 2.0, 2.0], [4.0, 5.0, 4.0], 
[5.0, 5.0, 4.0]]], [[[4.0, 7.0, 7.0], [7.0, 4.0, 2.0], [6.0, 3.0, 7.0]], [[6.0, 7.0, 8.0], [6.0, 1.0, 7.0], [3.0, 7.0, 6.0]], 
[[4.0, 7.0, 6.0], [7.0, 3.0, 3.0], [3.0, 8.0, 1.0]]], [[[3.0, 3.0, 6.0], [7.0, 3.0, 1.0], [2.0, 2.0, 7.0]], [[2.0, 6.0, 7.0], 
[3.0, 4.0, 6.0], [6.0, 2.0, 5.0]], [[6.0, 2.0, 7.0], [7.0, 8.0, 3.0], [8.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPPP(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 3.0], [1.0, 7.0, 2.0], [7.0, 6.0, 2.0]], [[5.0, 7.0, 3.0], [4.0, 7.0, 3.0], [2.0, 
3.0, 5.0]], [[5.0, 8.0, 1.0], [3.0, 8.0, 3.0], [8.0, 4.0, 4.0]]], [[[5.0, 4.0, 2.0], [1.0, 6.0, 4.0], [7.0, 8.0, 7.0]], [[6.0, 
7.0, 1.0], [8.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[2.0, 8.0, 7.0], [4.0, 3.0, 5.0], [2.0, 5.0, 1.0]]], [[[5.0, 6.0, 5.0], [3.0, 
2.0, 1.0], [1.0, 2.0, 1.0]], [[8.0, 4.0, 6.0], [1.0, 4.0, 5.0], [3.0, 4.0, 3.0]], [[1.0, 2.0, 8.0], [8.0, 3.0, 7.0], [7.0, 1.0, 
2.0]]], [[[6.0, 2.0, 3.0], [8.0, 7.0, 3.0], [1.0, 5.0, 4.0]], [[7.0, 5.0, 4.0], [5.0, 8.0, 4.0], [2.0, 4.0, 7.0]], [[7.0, 3.0, 
3.0], [2.0, 6.0, 4.0], [8.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 3.0], [3.0, 8.0, 7.0], [2.0, 3.0, 3.0]], 
[[1.0, 1.0, 4.0], [3.0, 4.0, 5.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 2.0], [5.0, 5.0, 3.0], [6.0, 6.0, 2.0]]], [[[3.0, 1.0, 4.0], 
[3.0, 5.0, 5.0], [7.0, 5.0, 3.0]], [[5.0, 2.0, 6.0], [6.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[7.0, 5.0, 5.0], [1.0, 5.0, 1.0], 
[6.0, 1.0, 5.0]]], [[[3.0, 6.0, 8.0], [4.0, 3.0, 1.0], [8.0, 5.0, 4.0]], [[3.0, 5.0, 6.0], [1.0, 3.0, 5.0], [5.0, 6.0, 1.0]], 
[[2.0, 6.0, 4.0], [6.0, 4.0, 2.0], [2.0, 7.0, 7.0]]], [[[3.0, 6.0, 2.0], [3.0, 1.0, 1.0], [4.0, 1.0, 7.0]], [[7.0, 4.0, 5.0], 
[1.0, 4.0, 1.0], [2.0, 2.0, 5.0]], [[7.0, 1.0, 4.0], [8.0, 3.0, 3.0], [4.0, 4.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*(1.0)+(1.-msk_val)*(1.0)
      arg[2,1,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 3.0], [1.0, 7.0, 2.0], [7.0, 6.0, 2.0]], [[5.0, 7.0, 3.0], [4.0, 7.0, 3.0], [2.0, 
3.0, 5.0]], [[5.0, 8.0, 1.0], [3.0, 8.0, 3.0], [8.0, 4.0, 4.0]]], [[[5.0, 4.0, 2.0], [1.0, 6.0, 4.0], [7.0, 8.0, 7.0]], [[6.0, 
7.0, 1.0], [8.0, 1.0, 2.0], [8.0, 8.0, 2.0]], [[2.0, 8.0, 7.0], [4.0, 3.0, 5.0], [2.0, 5.0, 1.0]]], [[[5.0, 6.0, 5.0], [3.0, 
2.0, 1.0], [1.0, 2.0, 1.0]], [[8.0, 4.0, 6.0], [1.0, 1.0, 5.0], [3.0, 4.0, 3.0]], [[1.0, 2.0, 8.0], [8.0, 3.0, 7.0], [7.0, 1.0, 
2.0]]], [[[6.0, 2.0, 3.0], [8.0, 7.0, 3.0], [1.0, 5.0, 4.0]], [[7.0, 5.0, 4.0], [5.0, 8.0, 4.0], [2.0, 4.0, 7.0]], [[7.0, 3.0, 
3.0], [2.0, 6.0, 4.0], [8.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 3.0], [3.0, 8.0, 7.0], [2.0, 3.0, 3.0]], 
[[1.0, 1.0, 4.0], [3.0, 4.0, 5.0], [2.0, 7.0, 1.0]], [[7.0, 4.0, 2.0], [5.0, 5.0, 3.0], [6.0, 6.0, 2.0]]], [[[3.0, 1.0, 4.0], 
[3.0, 5.0, 5.0], [7.0, 5.0, 3.0]], [[5.0, 2.0, 6.0], [6.0, 4.0, 2.0], [7.0, 7.0, 1.0]], [[7.0, 5.0, 5.0], [1.0, 5.0, 1.0], 
[6.0, 1.0, 5.0]]], [[[3.0, 6.0, 8.0], [4.0, 3.0, 1.0], [8.0, 5.0, 4.0]], [[3.0, 5.0, 6.0], [1.0, 1.0, 5.0], [5.0, 6.0, 1.0]], 
[[2.0, 6.0, 4.0], [6.0, 4.0, 2.0], [2.0, 7.0, 7.0]]], [[[3.0, 6.0, 2.0], [3.0, 1.0, 1.0], [4.0, 1.0, 7.0]], [[7.0, 4.0, 5.0], 
[1.0, 4.0, 1.0], [2.0, 2.0, 5.0]], [[7.0, 1.0, 4.0], [8.0, 3.0, 3.0], [4.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 8.0], [8.0, 8.0, 1.0], [4.0, 3.0, 5.0]], [[6.0, 5.0, 4.0], [3.0, 4.0, 5.0], [4.0, 
6.0, 6.0]], [[7.0, 4.0, 6.0], [6.0, 2.0, 3.0], [4.0, 3.0, 2.0]]], [[[2.0, 4.0, 5.0], [5.0, 6.0, 4.0], [6.0, 6.0, 4.0]], [[1.0, 
1.0, 6.0], [2.0, 8.0, 6.0], [5.0, 1.0, 4.0]], [[3.0, 1.0, 6.0], [4.0, 3.0, 4.0], [1.0, 2.0, 7.0]]], [[[1.0, 3.0, 8.0], [4.0, 
4.0, 6.0], [4.0, 1.0, 1.0]], [[8.0, 3.0, 4.0], [6.0, 8.0, 6.0], [3.0, 2.0, 3.0]], [[8.0, 3.0, 6.0], [5.0, 4.0, 8.0], [6.0, 3.0, 
1.0]]], [[[6.0, 1.0, 6.0], [3.0, 3.0, 7.0], [4.0, 4.0, 5.0]], [[8.0, 4.0, 3.0], [6.0, 4.0, 5.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 
2.0], [4.0, 8.0, 1.0], [2.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 7.0], [3.0, 5.0, 5.0], [3.0, 4.0, 1.0]], 
[[4.0, 7.0, 5.0], [6.0, 6.0, 3.0], [5.0, 1.0, 8.0]], [[8.0, 7.0, 2.0], [4.0, 3.0, 3.0], [2.0, 1.0, 8.0]]], [[[1.0, 8.0, 3.0], 
[8.0, 1.0, 7.0], [8.0, 4.0, 4.0]], [[8.0, 4.0, 1.0], [5.0, 6.0, 8.0], [8.0, 5.0, 5.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 2.0], 
[5.0, 5.0, 1.0]]], [[[1.0, 5.0, 5.0], [6.0, 6.0, 1.0], [1.0, 3.0, 3.0]], [[5.0, 4.0, 1.0], [7.0, 2.0, 8.0], [4.0, 8.0, 7.0]], 
[[7.0, 1.0, 1.0], [3.0, 4.0, 1.0], [6.0, 7.0, 4.0]]], [[[7.0, 1.0, 2.0], [7.0, 5.0, 8.0], [7.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], 
[5.0, 6.0, 3.0], [1.0, 4.0, 4.0]], [[6.0, 2.0, 2.0], [6.0, 1.0, 2.0], [7.0, 1.0, 3.0]]]])
      val=numpy.array([[[2.0, 1.0, 5.0], [4.0, 2.0, 3.0]], [[3.0, 7.0, 1.0], [2.0, 4.0, 3.0]]])
      arg[2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 8.0], [8.0, 8.0, 1.0], [4.0, 3.0, 5.0]], [[6.0, 5.0, 4.0], [3.0, 4.0, 5.0], [4.0, 
6.0, 6.0]], [[7.0, 4.0, 6.0], [6.0, 2.0, 3.0], [4.0, 3.0, 2.0]]], [[[2.0, 4.0, 5.0], [5.0, 6.0, 4.0], [6.0, 6.0, 4.0]], [[1.0, 
1.0, 6.0], [2.0, 8.0, 6.0], [5.0, 1.0, 4.0]], [[3.0, 1.0, 6.0], [4.0, 3.0, 4.0], [1.0, 2.0, 7.0]]], [[[1.0, 3.0, 8.0], [2.0, 
1.0, 5.0], [4.0, 1.0, 1.0]], [[8.0, 3.0, 4.0], [4.0, 2.0, 3.0], [3.0, 2.0, 3.0]], [[8.0, 3.0, 6.0], [5.0, 4.0, 8.0], [6.0, 3.0, 
1.0]]], [[[6.0, 1.0, 6.0], [3.0, 7.0, 1.0], [4.0, 4.0, 5.0]], [[8.0, 4.0, 3.0], [2.0, 4.0, 3.0], [4.0, 7.0, 8.0]], [[7.0, 6.0, 
2.0], [4.0, 8.0, 1.0], [2.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 7.0], [3.0, 5.0, 5.0], [3.0, 4.0, 1.0]], 
[[4.0, 7.0, 5.0], [6.0, 6.0, 3.0], [5.0, 1.0, 8.0]], [[8.0, 7.0, 2.0], [4.0, 3.0, 3.0], [2.0, 1.0, 8.0]]], [[[1.0, 8.0, 3.0], 
[8.0, 1.0, 7.0], [8.0, 4.0, 4.0]], [[8.0, 4.0, 1.0], [5.0, 6.0, 8.0], [8.0, 5.0, 5.0]], [[5.0, 5.0, 5.0], [6.0, 3.0, 2.0], 
[5.0, 5.0, 1.0]]], [[[1.0, 5.0, 5.0], [2.0, 1.0, 5.0], [1.0, 3.0, 3.0]], [[5.0, 4.0, 1.0], [4.0, 2.0, 3.0], [4.0, 8.0, 7.0]], 
[[7.0, 1.0, 1.0], [3.0, 4.0, 1.0], [6.0, 7.0, 4.0]]], [[[7.0, 1.0, 2.0], [3.0, 7.0, 1.0], [7.0, 8.0, 8.0]], [[7.0, 4.0, 6.0], 
[2.0, 4.0, 3.0], [1.0, 4.0, 4.0]], [[6.0, 2.0, 2.0], [6.0, 1.0, 2.0], [7.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 1.0], [5.0, 5.0, 4.0], [4.0, 5.0, 2.0]], [[8.0, 1.0, 4.0], [7.0, 3.0, 1.0], [7.0, 
4.0, 7.0]], [[8.0, 7.0, 8.0], [2.0, 2.0, 3.0], [5.0, 4.0, 7.0]]], [[[2.0, 3.0, 1.0], [5.0, 2.0, 7.0], [7.0, 7.0, 1.0]], [[7.0, 
7.0, 1.0], [8.0, 3.0, 2.0], [3.0, 4.0, 5.0]], [[1.0, 2.0, 1.0], [6.0, 8.0, 3.0], [2.0, 6.0, 5.0]]], [[[8.0, 5.0, 3.0], [3.0, 
3.0, 6.0], [5.0, 7.0, 1.0]], [[2.0, 1.0, 2.0], [2.0, 5.0, 8.0], [6.0, 8.0, 7.0]], [[6.0, 5.0, 4.0], [4.0, 3.0, 6.0], [8.0, 2.0, 
5.0]]], [[[5.0, 5.0, 3.0], [2.0, 3.0, 7.0], [5.0, 8.0, 5.0]], [[8.0, 5.0, 1.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]], [[6.0, 1.0, 
1.0], [6.0, 2.0, 4.0], [2.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 7.0, 7.0], [3.0, 3.0, 2.0], [7.0, 8.0, 3.0]], 
[[3.0, 4.0, 3.0], [2.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[2.0, 1.0, 2.0], [8.0, 4.0, 7.0], [3.0, 1.0, 3.0]]], [[[4.0, 1.0, 1.0], 
[7.0, 4.0, 3.0], [4.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [5.0, 2.0, 8.0], [8.0, 7.0, 6.0]], [[3.0, 8.0, 7.0], [5.0, 2.0, 6.0], 
[6.0, 7.0, 8.0]]], [[[2.0, 6.0, 7.0], [6.0, 2.0, 6.0], [6.0, 4.0, 6.0]], [[2.0, 1.0, 4.0], [5.0, 2.0, 6.0], [4.0, 6.0, 1.0]], 
[[5.0, 8.0, 5.0], [5.0, 3.0, 7.0], [6.0, 6.0, 3.0]]], [[[6.0, 2.0, 6.0], [6.0, 4.0, 1.0], [3.0, 6.0, 3.0]], [[6.0, 1.0, 1.0], 
[2.0, 7.0, 4.0], [2.0, 7.0, 3.0]], [[3.0, 3.0, 6.0], [3.0, 6.0, 4.0], [2.0, 2.0, 1.0]]]])
      val=Data(numpy.array([[[7.0, 7.0, 4.0], [3.0, 5.0, 7.0]], [[7.0, 7.0, 1.0], [1.0, 8.0, 7.0]]]),self.functionspace)
      arg[2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 1.0], [5.0, 5.0, 4.0], [4.0, 5.0, 2.0]], [[8.0, 1.0, 4.0], [7.0, 3.0, 1.0], [7.0, 
4.0, 7.0]], [[8.0, 7.0, 8.0], [2.0, 2.0, 3.0], [5.0, 4.0, 7.0]]], [[[2.0, 3.0, 1.0], [5.0, 2.0, 7.0], [7.0, 7.0, 1.0]], [[7.0, 
7.0, 1.0], [8.0, 3.0, 2.0], [3.0, 4.0, 5.0]], [[1.0, 2.0, 1.0], [6.0, 8.0, 3.0], [2.0, 6.0, 5.0]]], [[[8.0, 5.0, 3.0], [7.0, 
7.0, 4.0], [5.0, 7.0, 1.0]], [[2.0, 1.0, 2.0], [3.0, 5.0, 7.0], [6.0, 8.0, 7.0]], [[6.0, 5.0, 4.0], [4.0, 3.0, 6.0], [8.0, 2.0, 
5.0]]], [[[5.0, 5.0, 3.0], [7.0, 7.0, 1.0], [5.0, 8.0, 5.0]], [[8.0, 5.0, 1.0], [1.0, 8.0, 7.0], [8.0, 5.0, 4.0]], [[6.0, 1.0, 
1.0], [6.0, 2.0, 4.0], [2.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 7.0], [3.0, 3.0, 2.0], [7.0, 8.0, 3.0]], 
[[3.0, 4.0, 3.0], [2.0, 5.0, 7.0], [7.0, 5.0, 6.0]], [[2.0, 1.0, 2.0], [8.0, 4.0, 7.0], [3.0, 1.0, 3.0]]], [[[4.0, 1.0, 1.0], 
[7.0, 4.0, 3.0], [4.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [5.0, 2.0, 8.0], [8.0, 7.0, 6.0]], [[3.0, 8.0, 7.0], [5.0, 2.0, 6.0], 
[6.0, 7.0, 8.0]]], [[[2.0, 6.0, 7.0], [7.0, 7.0, 4.0], [6.0, 4.0, 6.0]], [[2.0, 1.0, 4.0], [3.0, 5.0, 7.0], [4.0, 6.0, 1.0]], 
[[5.0, 8.0, 5.0], [5.0, 3.0, 7.0], [6.0, 6.0, 3.0]]], [[[6.0, 2.0, 6.0], [7.0, 7.0, 1.0], [3.0, 6.0, 3.0]], [[6.0, 1.0, 1.0], 
[1.0, 8.0, 7.0], [2.0, 7.0, 3.0]], [[3.0, 3.0, 6.0], [3.0, 6.0, 4.0], [2.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 6.0], [1.0, 1.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 1.0, 6.0], [4.0, 4.0, 4.0], [2.0, 
1.0, 3.0]], [[1.0, 5.0, 4.0], [5.0, 1.0, 1.0], [3.0, 5.0, 4.0]]], [[[7.0, 4.0, 8.0], [6.0, 6.0, 1.0], [5.0, 1.0, 6.0]], [[1.0, 
6.0, 8.0], [1.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[4.0, 2.0, 6.0], [2.0, 8.0, 2.0], [5.0, 3.0, 7.0]]], [[[4.0, 7.0, 7.0], [2.0, 
3.0, 3.0], [1.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [3.0, 6.0, 3.0], [1.0, 5.0, 8.0]], [[7.0, 4.0, 7.0], [8.0, 1.0, 8.0], [3.0, 5.0, 
7.0]]], [[[4.0, 1.0, 8.0], [5.0, 1.0, 2.0], [2.0, 3.0, 2.0]], [[5.0, 5.0, 2.0], [4.0, 2.0, 3.0], [6.0, 2.0, 8.0]], [[6.0, 5.0, 
5.0], [7.0, 2.0, 8.0], [8.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 7.0, 5.0], [5.0, 1.0, 1.0], [6.0, 4.0, 1.0]], 
[[6.0, 2.0, 5.0], [7.0, 8.0, 3.0], [7.0, 2.0, 4.0]], [[1.0, 8.0, 8.0], [3.0, 1.0, 7.0], [2.0, 1.0, 3.0]]], [[[3.0, 7.0, 3.0], 
[1.0, 1.0, 8.0], [7.0, 4.0, 7.0]], [[2.0, 2.0, 6.0], [6.0, 3.0, 4.0], [8.0, 7.0, 2.0]], [[7.0, 5.0, 3.0], [4.0, 2.0, 3.0], 
[7.0, 5.0, 7.0]]], [[[2.0, 4.0, 3.0], [1.0, 1.0, 3.0], [8.0, 5.0, 6.0]], [[3.0, 1.0, 6.0], [6.0, 1.0, 4.0], [1.0, 6.0, 6.0]], 
[[3.0, 3.0, 2.0], [7.0, 2.0, 8.0], [6.0, 7.0, 7.0]]], [[[4.0, 8.0, 1.0], [6.0, 4.0, 3.0], [2.0, 3.0, 8.0]], [[1.0, 8.0, 1.0], 
[3.0, 5.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], [5.0, 1.0, 4.0], [6.0, 3.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 8.0, 3.0], [7.0, 2.0, 2.0]], [[3.0, 7.0, 8.0], [2.0, 4.0, 
4.0]]])+(1.-msk_val)*numpy.array([[[2.0, 1.0, 2.0], [7.0, 4.0, 6.0]], [[5.0, 5.0, 2.0], [8.0, 2.0, 8.0]]])
      arg[2:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 6.0], [1.0, 1.0, 7.0], [2.0, 4.0, 5.0]], [[6.0, 1.0, 6.0], [4.0, 4.0, 4.0], [2.0, 
1.0, 3.0]], [[1.0, 5.0, 4.0], [5.0, 1.0, 1.0], [3.0, 5.0, 4.0]]], [[[7.0, 4.0, 8.0], [6.0, 6.0, 1.0], [5.0, 1.0, 6.0]], [[1.0, 
6.0, 8.0], [1.0, 1.0, 5.0], [5.0, 1.0, 2.0]], [[4.0, 2.0, 6.0], [2.0, 8.0, 2.0], [5.0, 3.0, 7.0]]], [[[4.0, 7.0, 7.0], [4.0, 
8.0, 3.0], [1.0, 6.0, 7.0]], [[3.0, 5.0, 8.0], [7.0, 2.0, 2.0], [1.0, 5.0, 8.0]], [[7.0, 4.0, 7.0], [8.0, 1.0, 8.0], [3.0, 5.0, 
7.0]]], [[[4.0, 1.0, 8.0], [3.0, 7.0, 8.0], [2.0, 3.0, 2.0]], [[5.0, 5.0, 2.0], [2.0, 4.0, 4.0], [6.0, 2.0, 8.0]], [[6.0, 5.0, 
5.0], [7.0, 2.0, 8.0], [8.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, 5.0], [5.0, 1.0, 1.0], [6.0, 4.0, 1.0]], 
[[6.0, 2.0, 5.0], [7.0, 8.0, 3.0], [7.0, 2.0, 4.0]], [[1.0, 8.0, 8.0], [3.0, 1.0, 7.0], [2.0, 1.0, 3.0]]], [[[3.0, 7.0, 3.0], 
[1.0, 1.0, 8.0], [7.0, 4.0, 7.0]], [[2.0, 2.0, 6.0], [6.0, 3.0, 4.0], [8.0, 7.0, 2.0]], [[7.0, 5.0, 3.0], [4.0, 2.0, 3.0], 
[7.0, 5.0, 7.0]]], [[[2.0, 4.0, 3.0], [2.0, 1.0, 2.0], [8.0, 5.0, 6.0]], [[3.0, 1.0, 6.0], [7.0, 4.0, 6.0], [1.0, 6.0, 6.0]], 
[[3.0, 3.0, 2.0], [7.0, 2.0, 8.0], [6.0, 7.0, 7.0]]], [[[4.0, 8.0, 1.0], [5.0, 5.0, 2.0], [2.0, 3.0, 8.0]], [[1.0, 8.0, 1.0], 
[8.0, 2.0, 8.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], [5.0, 1.0, 4.0], [6.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 3.0], [1.0, 3.0, 8.0], [1.0, 7.0, 7.0]], [[6.0, 8.0, 5.0], [6.0, 8.0, 1.0], [7.0, 
6.0, 8.0]], [[3.0, 4.0, 6.0], [2.0, 8.0, 2.0], [5.0, 3.0, 3.0]]], [[[8.0, 7.0, 4.0], [6.0, 7.0, 5.0], [5.0, 3.0, 8.0]], [[1.0, 
2.0, 3.0], [6.0, 2.0, 7.0], [5.0, 4.0, 8.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[7.0, 3.0, 1.0], [8.0, 
3.0, 6.0], [1.0, 3.0, 5.0]], [[2.0, 6.0, 6.0], [1.0, 1.0, 7.0], [7.0, 8.0, 2.0]], [[4.0, 8.0, 2.0], [5.0, 4.0, 1.0], [7.0, 8.0, 
7.0]]], [[[6.0, 8.0, 2.0], [1.0, 2.0, 6.0], [4.0, 1.0, 4.0]], [[8.0, 4.0, 3.0], [5.0, 3.0, 6.0], [5.0, 3.0, 8.0]], [[4.0, 8.0, 
5.0], [6.0, 6.0, 1.0], [7.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 7.0], [7.0, 6.0, 6.0], [1.0, 4.0, 8.0]], 
[[7.0, 4.0, 1.0], [1.0, 2.0, 4.0], [7.0, 7.0, 3.0]], [[6.0, 8.0, 2.0], [4.0, 1.0, 7.0], [1.0, 2.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[4.0, 8.0, 8.0], [2.0, 7.0, 4.0]], [[7.0, 2.0, 5.0], [8.0, 2.0, 1.0], [1.0, 2.0, 1.0]], [[1.0, 7.0, 1.0], [3.0, 8.0, 8.0], 
[3.0, 5.0, 1.0]]], [[[6.0, 2.0, 7.0], [2.0, 8.0, 8.0], [3.0, 3.0, 4.0]], [[5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [6.0, 2.0, 3.0]], 
[[2.0, 2.0, 4.0], [8.0, 8.0, 5.0], [4.0, 2.0, 5.0]]], [[[1.0, 3.0, 4.0], [8.0, 7.0, 7.0], [2.0, 8.0, 1.0]], [[7.0, 5.0, 5.0], 
[1.0, 8.0, 8.0], [6.0, 8.0, 2.0]], [[3.0, 6.0, 7.0], [1.0, 4.0, 2.0], [4.0, 6.0, 3.0]]]])
      val=numpy.array([[[5.0, 5.0, 6.0], [3.0, 6.0, 5.0]], [[3.0, 5.0, 4.0], [6.0, 6.0, 8.0]], [[8.0, 5.0, 5.0], [4.0, 4.0, 
5.0]], [[3.0, 4.0, 1.0], [3.0, 5.0, 4.0]]])
      arg[:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [5.0, 5.0, 6.0], [1.0, 7.0, 7.0]], [[6.0, 8.0, 5.0], [3.0, 6.0, 5.0], [7.0, 
6.0, 8.0]], [[3.0, 4.0, 6.0], [2.0, 8.0, 2.0], [5.0, 3.0, 3.0]]], [[[8.0, 7.0, 4.0], [3.0, 5.0, 4.0], [5.0, 3.0, 8.0]], [[1.0, 
2.0, 3.0], [6.0, 6.0, 8.0], [5.0, 4.0, 8.0]], [[5.0, 5.0, 8.0], [6.0, 5.0, 8.0], [1.0, 5.0, 6.0]]], [[[7.0, 3.0, 1.0], [8.0, 
5.0, 5.0], [1.0, 3.0, 5.0]], [[2.0, 6.0, 6.0], [4.0, 4.0, 5.0], [7.0, 8.0, 2.0]], [[4.0, 8.0, 2.0], [5.0, 4.0, 1.0], [7.0, 8.0, 
7.0]]], [[[6.0, 8.0, 2.0], [3.0, 4.0, 1.0], [4.0, 1.0, 4.0]], [[8.0, 4.0, 3.0], [3.0, 5.0, 4.0], [5.0, 3.0, 8.0]], [[4.0, 8.0, 
5.0], [6.0, 6.0, 1.0], [7.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 7.0], [5.0, 5.0, 6.0], [1.0, 4.0, 8.0]], 
[[7.0, 4.0, 1.0], [3.0, 6.0, 5.0], [7.0, 7.0, 3.0]], [[6.0, 8.0, 2.0], [4.0, 1.0, 7.0], [1.0, 2.0, 5.0]]], [[[6.0, 7.0, 2.0], 
[3.0, 5.0, 4.0], [2.0, 7.0, 4.0]], [[7.0, 2.0, 5.0], [6.0, 6.0, 8.0], [1.0, 2.0, 1.0]], [[1.0, 7.0, 1.0], [3.0, 8.0, 8.0], 
[3.0, 5.0, 1.0]]], [[[6.0, 2.0, 7.0], [8.0, 5.0, 5.0], [3.0, 3.0, 4.0]], [[5.0, 6.0, 3.0], [4.0, 4.0, 5.0], [6.0, 2.0, 3.0]], 
[[2.0, 2.0, 4.0], [8.0, 8.0, 5.0], [4.0, 2.0, 5.0]]], [[[1.0, 3.0, 4.0], [3.0, 4.0, 1.0], [2.0, 8.0, 1.0]], [[7.0, 5.0, 5.0], 
[3.0, 5.0, 4.0], [6.0, 8.0, 2.0]], [[3.0, 6.0, 7.0], [1.0, 4.0, 2.0], [4.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 8.0], [2.0, 2.0, 3.0], [7.0, 8.0, 8.0]], [[2.0, 2.0, 7.0], [1.0, 1.0, 8.0], [7.0, 
4.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 3.0, 6.0], [2.0, 1.0, 3.0]]], [[[5.0, 1.0, 4.0], [7.0, 5.0, 4.0], [6.0, 5.0, 4.0]], [[1.0, 
5.0, 2.0], [6.0, 7.0, 7.0], [4.0, 1.0, 8.0]], [[4.0, 7.0, 1.0], [8.0, 7.0, 5.0], [2.0, 1.0, 6.0]]], [[[2.0, 3.0, 1.0], [6.0, 
2.0, 8.0], [8.0, 2.0, 2.0]], [[2.0, 8.0, 1.0], [6.0, 5.0, 6.0], [5.0, 2.0, 8.0]], [[4.0, 7.0, 6.0], [5.0, 6.0, 1.0], [8.0, 5.0, 
7.0]]], [[[8.0, 3.0, 5.0], [8.0, 8.0, 1.0], [6.0, 2.0, 5.0]], [[3.0, 5.0, 3.0], [6.0, 8.0, 8.0], [4.0, 6.0, 2.0]], [[1.0, 2.0, 
6.0], [3.0, 5.0, 4.0], [3.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 5.0], [4.0, 5.0, 7.0], [5.0, 6.0, 5.0]], 
[[7.0, 4.0, 1.0], [2.0, 2.0, 5.0], [1.0, 8.0, 2.0]], [[5.0, 3.0, 3.0], [2.0, 1.0, 8.0], [3.0, 3.0, 8.0]]], [[[5.0, 3.0, 2.0], 
[3.0, 4.0, 6.0], [7.0, 2.0, 4.0]], [[2.0, 6.0, 6.0], [1.0, 3.0, 7.0], [7.0, 2.0, 6.0]], [[6.0, 7.0, 1.0], [1.0, 7.0, 3.0], 
[3.0, 1.0, 2.0]]], [[[2.0, 2.0, 6.0], [5.0, 3.0, 1.0], [4.0, 1.0, 8.0]], [[4.0, 7.0, 4.0], [8.0, 4.0, 4.0], [4.0, 8.0, 8.0]], 
[[1.0, 7.0, 7.0], [2.0, 4.0, 4.0], [2.0, 4.0, 2.0]]], [[[3.0, 2.0, 6.0], [6.0, 7.0, 5.0], [6.0, 2.0, 1.0]], [[5.0, 6.0, 7.0], 
[6.0, 2.0, 3.0], [6.0, 1.0, 6.0]], [[2.0, 6.0, 6.0], [8.0, 6.0, 1.0], [3.0, 4.0, 4.0]]]])
      val=Data(numpy.array([[[7.0, 7.0, 8.0], [2.0, 6.0, 5.0]], [[7.0, 4.0, 5.0], [2.0, 5.0, 8.0]], [[8.0, 4.0, 4.0], [2.0, 
3.0, 3.0]], [[1.0, 6.0, 5.0], [1.0, 7.0, 3.0]]]),self.functionspace)
      arg[:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 8.0], [7.0, 7.0, 8.0], [7.0, 8.0, 8.0]], [[2.0, 2.0, 7.0], [2.0, 6.0, 5.0], [7.0, 
4.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 3.0, 6.0], [2.0, 1.0, 3.0]]], [[[5.0, 1.0, 4.0], [7.0, 4.0, 5.0], [6.0, 5.0, 4.0]], [[1.0, 
5.0, 2.0], [2.0, 5.0, 8.0], [4.0, 1.0, 8.0]], [[4.0, 7.0, 1.0], [8.0, 7.0, 5.0], [2.0, 1.0, 6.0]]], [[[2.0, 3.0, 1.0], [8.0, 
4.0, 4.0], [8.0, 2.0, 2.0]], [[2.0, 8.0, 1.0], [2.0, 3.0, 3.0], [5.0, 2.0, 8.0]], [[4.0, 7.0, 6.0], [5.0, 6.0, 1.0], [8.0, 5.0, 
7.0]]], [[[8.0, 3.0, 5.0], [1.0, 6.0, 5.0], [6.0, 2.0, 5.0]], [[3.0, 5.0, 3.0], [1.0, 7.0, 3.0], [4.0, 6.0, 2.0]], [[1.0, 2.0, 
6.0], [3.0, 5.0, 4.0], [3.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 5.0], [7.0, 7.0, 8.0], [5.0, 6.0, 5.0]], 
[[7.0, 4.0, 1.0], [2.0, 6.0, 5.0], [1.0, 8.0, 2.0]], [[5.0, 3.0, 3.0], [2.0, 1.0, 8.0], [3.0, 3.0, 8.0]]], [[[5.0, 3.0, 2.0], 
[7.0, 4.0, 5.0], [7.0, 2.0, 4.0]], [[2.0, 6.0, 6.0], [2.0, 5.0, 8.0], [7.0, 2.0, 6.0]], [[6.0, 7.0, 1.0], [1.0, 7.0, 3.0], 
[3.0, 1.0, 2.0]]], [[[2.0, 2.0, 6.0], [8.0, 4.0, 4.0], [4.0, 1.0, 8.0]], [[4.0, 7.0, 4.0], [2.0, 3.0, 3.0], [4.0, 8.0, 8.0]], 
[[1.0, 7.0, 7.0], [2.0, 4.0, 4.0], [2.0, 4.0, 2.0]]], [[[3.0, 2.0, 6.0], [1.0, 6.0, 5.0], [6.0, 2.0, 1.0]], [[5.0, 6.0, 7.0], 
[1.0, 7.0, 3.0], [6.0, 1.0, 6.0]], [[2.0, 6.0, 6.0], [8.0, 6.0, 1.0], [3.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 3.0], [7.0, 6.0, 2.0], [6.0, 1.0, 4.0]], [[3.0, 4.0, 3.0], [5.0, 6.0, 8.0], [7.0, 
8.0, 4.0]], [[8.0, 8.0, 2.0], [4.0, 5.0, 8.0], [2.0, 2.0, 8.0]]], [[[3.0, 2.0, 6.0], [7.0, 8.0, 6.0], [2.0, 2.0, 3.0]], [[1.0, 
2.0, 6.0], [2.0, 7.0, 3.0], [7.0, 7.0, 7.0]], [[2.0, 8.0, 6.0], [7.0, 7.0, 3.0], [4.0, 6.0, 7.0]]], [[[1.0, 1.0, 4.0], [1.0, 
7.0, 6.0], [1.0, 8.0, 2.0]], [[5.0, 5.0, 1.0], [7.0, 3.0, 5.0], [1.0, 8.0, 3.0]], [[4.0, 5.0, 2.0], [3.0, 4.0, 2.0], [2.0, 4.0, 
7.0]]], [[[7.0, 1.0, 3.0], [1.0, 2.0, 4.0], [5.0, 4.0, 3.0]], [[1.0, 1.0, 6.0], [6.0, 3.0, 2.0], [1.0, 1.0, 7.0]], [[7.0, 2.0, 
1.0], [3.0, 8.0, 7.0], [2.0, 8.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 4.0], [5.0, 2.0, 3.0], [1.0, 1.0, 1.0]], 
[[4.0, 7.0, 7.0], [2.0, 3.0, 6.0], [7.0, 2.0, 8.0]], [[8.0, 8.0, 2.0], [1.0, 7.0, 1.0], [8.0, 6.0, 8.0]]], [[[2.0, 7.0, 8.0], 
[3.0, 6.0, 3.0], [4.0, 8.0, 1.0]], [[3.0, 6.0, 4.0], [6.0, 2.0, 4.0], [2.0, 3.0, 3.0]], [[1.0, 8.0, 5.0], [2.0, 4.0, 6.0], 
[4.0, 6.0, 1.0]]], [[[6.0, 5.0, 2.0], [3.0, 8.0, 6.0], [6.0, 4.0, 3.0]], [[1.0, 6.0, 7.0], [8.0, 2.0, 1.0], [3.0, 5.0, 7.0]], 
[[4.0, 2.0, 6.0], [5.0, 1.0, 5.0], [5.0, 3.0, 4.0]]], [[[8.0, 4.0, 8.0], [3.0, 4.0, 8.0], [7.0, 3.0, 3.0]], [[4.0, 3.0, 5.0], 
[8.0, 8.0, 2.0], [3.0, 6.0, 3.0]], [[6.0, 4.0, 5.0], [2.0, 7.0, 8.0], [4.0, 3.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 4.0, 1.0], [6.0, 3.0, 8.0]], [[7.0, 2.0, 8.0], [5.0, 1.0, 1.0]], [[1.0, 7.0, 6.0], [4.0, 
1.0, 1.0]], [[5.0, 2.0, 1.0], [6.0, 1.0, 5.0]]])+(1.-msk_val)*numpy.array([[[2.0, 8.0, 8.0], [5.0, 3.0, 4.0]], [[8.0, 8.0, 
6.0], [7.0, 3.0, 1.0]], [[8.0, 7.0, 1.0], [4.0, 8.0, 8.0]], [[4.0, 3.0, 8.0], [7.0, 6.0, 8.0]]])
      arg[:,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [6.0, 4.0, 1.0], [6.0, 1.0, 4.0]], [[3.0, 4.0, 3.0], [6.0, 3.0, 8.0], [7.0, 
8.0, 4.0]], [[8.0, 8.0, 2.0], [4.0, 5.0, 8.0], [2.0, 2.0, 8.0]]], [[[3.0, 2.0, 6.0], [7.0, 2.0, 8.0], [2.0, 2.0, 3.0]], [[1.0, 
2.0, 6.0], [5.0, 1.0, 1.0], [7.0, 7.0, 7.0]], [[2.0, 8.0, 6.0], [7.0, 7.0, 3.0], [4.0, 6.0, 7.0]]], [[[1.0, 1.0, 4.0], [1.0, 
7.0, 6.0], [1.0, 8.0, 2.0]], [[5.0, 5.0, 1.0], [4.0, 1.0, 1.0], [1.0, 8.0, 3.0]], [[4.0, 5.0, 2.0], [3.0, 4.0, 2.0], [2.0, 4.0, 
7.0]]], [[[7.0, 1.0, 3.0], [5.0, 2.0, 1.0], [5.0, 4.0, 3.0]], [[1.0, 1.0, 6.0], [6.0, 1.0, 5.0], [1.0, 1.0, 7.0]], [[7.0, 2.0, 
1.0], [3.0, 8.0, 7.0], [2.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 4.0], [2.0, 8.0, 8.0], [1.0, 1.0, 1.0]], 
[[4.0, 7.0, 7.0], [5.0, 3.0, 4.0], [7.0, 2.0, 8.0]], [[8.0, 8.0, 2.0], [1.0, 7.0, 1.0], [8.0, 6.0, 8.0]]], [[[2.0, 7.0, 8.0], 
[8.0, 8.0, 6.0], [4.0, 8.0, 1.0]], [[3.0, 6.0, 4.0], [7.0, 3.0, 1.0], [2.0, 3.0, 3.0]], [[1.0, 8.0, 5.0], [2.0, 4.0, 6.0], 
[4.0, 6.0, 1.0]]], [[[6.0, 5.0, 2.0], [8.0, 7.0, 1.0], [6.0, 4.0, 3.0]], [[1.0, 6.0, 7.0], [4.0, 8.0, 8.0], [3.0, 5.0, 7.0]], 
[[4.0, 2.0, 6.0], [5.0, 1.0, 5.0], [5.0, 3.0, 4.0]]], [[[8.0, 4.0, 8.0], [4.0, 3.0, 8.0], [7.0, 3.0, 3.0]], [[4.0, 3.0, 5.0], 
[7.0, 6.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 4.0, 5.0], [2.0, 7.0, 8.0], [4.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 5.0], [5.0, 2.0, 3.0], [8.0, 2.0, 8.0]], [[6.0, 2.0, 2.0], [5.0, 8.0, 8.0], [6.0, 
4.0, 7.0]], [[6.0, 2.0, 1.0], [8.0, 7.0, 7.0], [4.0, 5.0, 6.0]]], [[[3.0, 4.0, 7.0], [5.0, 2.0, 7.0], [2.0, 5.0, 5.0]], [[6.0, 
6.0, 3.0], [6.0, 3.0, 4.0], [6.0, 2.0, 1.0]], [[1.0, 4.0, 4.0], [8.0, 3.0, 5.0], [2.0, 8.0, 6.0]]], [[[2.0, 5.0, 4.0], [2.0, 
2.0, 4.0], [1.0, 1.0, 3.0]], [[8.0, 2.0, 5.0], [7.0, 3.0, 4.0], [6.0, 5.0, 4.0]], [[2.0, 1.0, 2.0], [3.0, 6.0, 8.0], [3.0, 4.0, 
2.0]]], [[[3.0, 4.0, 1.0], [6.0, 4.0, 6.0], [7.0, 7.0, 7.0]], [[5.0, 7.0, 8.0], [3.0, 6.0, 4.0], [7.0, 4.0, 6.0]], [[4.0, 8.0, 
8.0], [1.0, 5.0, 2.0], [8.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 1.0], [1.0, 3.0, 3.0], [4.0, 7.0, 8.0]], 
[[3.0, 6.0, 4.0], [1.0, 6.0, 3.0], [2.0, 7.0, 7.0]], [[8.0, 6.0, 6.0], [5.0, 6.0, 1.0], [8.0, 8.0, 2.0]]], [[[1.0, 6.0, 5.0], 
[6.0, 6.0, 4.0], [5.0, 7.0, 1.0]], [[3.0, 7.0, 5.0], [6.0, 3.0, 5.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 8.0], [7.0, 6.0, 2.0], 
[7.0, 6.0, 3.0]]], [[[8.0, 4.0, 6.0], [7.0, 4.0, 8.0], [6.0, 7.0, 1.0]], [[5.0, 4.0, 7.0], [7.0, 3.0, 1.0], [1.0, 7.0, 6.0]], 
[[6.0, 3.0, 2.0], [4.0, 8.0, 6.0], [2.0, 5.0, 4.0]]], [[[8.0, 4.0, 1.0], [7.0, 5.0, 6.0], [7.0, 6.0, 1.0]], [[7.0, 3.0, 1.0], 
[2.0, 2.0, 2.0], [1.0, 4.0, 1.0]], [[5.0, 8.0, 8.0], [8.0, 7.0, 2.0], [2.0, 1.0, 5.0]]]])
      val=numpy.array([[[[2.0, 6.0, 6.0]], [[6.0, 6.0, 1.0]], [[8.0, 1.0, 3.0]]], [[[3.0, 7.0, 6.0]], [[7.0, 6.0, 1.0]], [[4.0, 
3.0, 6.0]]], [[[6.0, 3.0, 4.0]], [[3.0, 7.0, 4.0]], [[7.0, 6.0, 6.0]]], [[[7.0, 6.0, 4.0]], [[5.0, 2.0, 5.0]], [[8.0, 6.0, 
2.0]]]])
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 5.0], [2.0, 6.0, 6.0], [8.0, 2.0, 8.0]], [[6.0, 2.0, 2.0], [6.0, 6.0, 1.0], [6.0, 
4.0, 7.0]], [[6.0, 2.0, 1.0], [8.0, 1.0, 3.0], [4.0, 5.0, 6.0]]], [[[3.0, 4.0, 7.0], [3.0, 7.0, 6.0], [2.0, 5.0, 5.0]], [[6.0, 
6.0, 3.0], [7.0, 6.0, 1.0], [6.0, 2.0, 1.0]], [[1.0, 4.0, 4.0], [4.0, 3.0, 6.0], [2.0, 8.0, 6.0]]], [[[2.0, 5.0, 4.0], [6.0, 
3.0, 4.0], [1.0, 1.0, 3.0]], [[8.0, 2.0, 5.0], [3.0, 7.0, 4.0], [6.0, 5.0, 4.0]], [[2.0, 1.0, 2.0], [7.0, 6.0, 6.0], [3.0, 4.0, 
2.0]]], [[[3.0, 4.0, 1.0], [7.0, 6.0, 4.0], [7.0, 7.0, 7.0]], [[5.0, 7.0, 8.0], [5.0, 2.0, 5.0], [7.0, 4.0, 6.0]], [[4.0, 8.0, 
8.0], [8.0, 6.0, 2.0], [8.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 1.0], [2.0, 6.0, 6.0], [4.0, 7.0, 8.0]], 
[[3.0, 6.0, 4.0], [6.0, 6.0, 1.0], [2.0, 7.0, 7.0]], [[8.0, 6.0, 6.0], [8.0, 1.0, 3.0], [8.0, 8.0, 2.0]]], [[[1.0, 6.0, 5.0], 
[3.0, 7.0, 6.0], [5.0, 7.0, 1.0]], [[3.0, 7.0, 5.0], [7.0, 6.0, 1.0], [5.0, 1.0, 1.0]], [[2.0, 4.0, 8.0], [4.0, 3.0, 6.0], 
[7.0, 6.0, 3.0]]], [[[8.0, 4.0, 6.0], [6.0, 3.0, 4.0], [6.0, 7.0, 1.0]], [[5.0, 4.0, 7.0], [3.0, 7.0, 4.0], [1.0, 7.0, 6.0]], 
[[6.0, 3.0, 2.0], [7.0, 6.0, 6.0], [2.0, 5.0, 4.0]]], [[[8.0, 4.0, 1.0], [7.0, 6.0, 4.0], [7.0, 6.0, 1.0]], [[7.0, 3.0, 1.0], 
[5.0, 2.0, 5.0], [1.0, 4.0, 1.0]], [[5.0, 8.0, 8.0], [8.0, 6.0, 2.0], [2.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 1.0], [7.0, 8.0, 8.0], [4.0, 3.0, 6.0]], [[2.0, 6.0, 7.0], [6.0, 7.0, 5.0], [5.0, 
6.0, 7.0]], [[7.0, 4.0, 2.0], [3.0, 4.0, 2.0], [5.0, 7.0, 1.0]]], [[[5.0, 6.0, 2.0], [1.0, 8.0, 1.0], [8.0, 1.0, 7.0]], [[2.0, 
8.0, 5.0], [6.0, 1.0, 5.0], [1.0, 6.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 7.0, 8.0], [8.0, 1.0, 6.0]]], [[[8.0, 5.0, 5.0], [2.0, 
2.0, 5.0], [2.0, 7.0, 4.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 2.0], [4.0, 2.0, 5.0]], [[5.0, 3.0, 2.0], [7.0, 2.0, 4.0], [7.0, 6.0, 
1.0]]], [[[5.0, 1.0, 8.0], [3.0, 2.0, 5.0], [5.0, 3.0, 7.0]], [[7.0, 1.0, 2.0], [2.0, 2.0, 7.0], [6.0, 5.0, 7.0]], [[1.0, 1.0, 
3.0], [1.0, 3.0, 8.0], [7.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 6.0], [3.0, 7.0, 2.0], [2.0, 7.0, 7.0]], 
[[1.0, 1.0, 7.0], [8.0, 8.0, 7.0], [1.0, 6.0, 6.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 2.0], [1.0, 1.0, 6.0]]], [[[8.0, 6.0, 2.0], 
[6.0, 6.0, 7.0], [6.0, 3.0, 6.0]], [[5.0, 1.0, 7.0], [8.0, 3.0, 1.0], [8.0, 7.0, 2.0]], [[1.0, 3.0, 3.0], [1.0, 8.0, 3.0], 
[7.0, 8.0, 4.0]]], [[[1.0, 8.0, 7.0], [4.0, 6.0, 1.0], [8.0, 2.0, 6.0]], [[7.0, 1.0, 2.0], [5.0, 3.0, 7.0], [4.0, 4.0, 3.0]], 
[[5.0, 5.0, 6.0], [3.0, 1.0, 2.0], [8.0, 2.0, 3.0]]], [[[3.0, 4.0, 7.0], [7.0, 8.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 3.0, 2.0], 
[7.0, 6.0, 8.0], [7.0, 7.0, 7.0]], [[7.0, 4.0, 5.0], [6.0, 2.0, 8.0], [8.0, 2.0, 7.0]]]])
      val=Data(numpy.array([[[[5.0, 7.0, 5.0]], [[3.0, 7.0, 8.0]], [[6.0, 7.0, 2.0]]], [[[8.0, 2.0, 4.0]], [[3.0, 4.0, 4.0]], 
[[8.0, 4.0, 7.0]]], [[[5.0, 8.0, 2.0]], [[1.0, 8.0, 7.0]], [[3.0, 5.0, 3.0]]], [[[4.0, 6.0, 4.0]], [[7.0, 2.0, 3.0]], [[5.0, 
5.0, 5.0]]]]),self.functionspace)
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 1.0], [5.0, 7.0, 5.0], [4.0, 3.0, 6.0]], [[2.0, 6.0, 7.0], [3.0, 7.0, 8.0], [5.0, 
6.0, 7.0]], [[7.0, 4.0, 2.0], [6.0, 7.0, 2.0], [5.0, 7.0, 1.0]]], [[[5.0, 6.0, 2.0], [8.0, 2.0, 4.0], [8.0, 1.0, 7.0]], [[2.0, 
8.0, 5.0], [3.0, 4.0, 4.0], [1.0, 6.0, 4.0]], [[6.0, 5.0, 6.0], [8.0, 4.0, 7.0], [8.0, 1.0, 6.0]]], [[[8.0, 5.0, 5.0], [5.0, 
8.0, 2.0], [2.0, 7.0, 4.0]], [[2.0, 5.0, 4.0], [1.0, 8.0, 7.0], [4.0, 2.0, 5.0]], [[5.0, 3.0, 2.0], [3.0, 5.0, 3.0], [7.0, 6.0, 
1.0]]], [[[5.0, 1.0, 8.0], [4.0, 6.0, 4.0], [5.0, 3.0, 7.0]], [[7.0, 1.0, 2.0], [7.0, 2.0, 3.0], [6.0, 5.0, 7.0]], [[1.0, 1.0, 
3.0], [5.0, 5.0, 5.0], [7.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 6.0], [5.0, 7.0, 5.0], [2.0, 7.0, 7.0]], 
[[1.0, 1.0, 7.0], [3.0, 7.0, 8.0], [1.0, 6.0, 6.0]], [[8.0, 5.0, 7.0], [6.0, 7.0, 2.0], [1.0, 1.0, 6.0]]], [[[8.0, 6.0, 2.0], 
[8.0, 2.0, 4.0], [6.0, 3.0, 6.0]], [[5.0, 1.0, 7.0], [3.0, 4.0, 4.0], [8.0, 7.0, 2.0]], [[1.0, 3.0, 3.0], [8.0, 4.0, 7.0], 
[7.0, 8.0, 4.0]]], [[[1.0, 8.0, 7.0], [5.0, 8.0, 2.0], [8.0, 2.0, 6.0]], [[7.0, 1.0, 2.0], [1.0, 8.0, 7.0], [4.0, 4.0, 3.0]], 
[[5.0, 5.0, 6.0], [3.0, 5.0, 3.0], [8.0, 2.0, 3.0]]], [[[3.0, 4.0, 7.0], [4.0, 6.0, 4.0], [5.0, 4.0, 4.0]], [[5.0, 3.0, 2.0], 
[7.0, 2.0, 3.0], [7.0, 7.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 5.0, 5.0], [8.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 3.0], [8.0, 4.0, 7.0], [8.0, 6.0, 3.0]], [[6.0, 1.0, 1.0], [7.0, 7.0, 3.0], [8.0, 
6.0, 5.0]], [[1.0, 4.0, 1.0], [8.0, 2.0, 5.0], [2.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [5.0, 3.0, 5.0], [6.0, 6.0, 8.0]], [[1.0, 
3.0, 7.0], [1.0, 4.0, 8.0], [6.0, 4.0, 6.0]], [[6.0, 2.0, 5.0], [3.0, 6.0, 1.0], [8.0, 1.0, 3.0]]], [[[4.0, 5.0, 2.0], [8.0, 
6.0, 7.0], [2.0, 3.0, 5.0]], [[6.0, 1.0, 3.0], [6.0, 2.0, 8.0], [6.0, 4.0, 1.0]], [[2.0, 7.0, 6.0], [6.0, 8.0, 1.0], [2.0, 3.0, 
3.0]]], [[[4.0, 1.0, 3.0], [1.0, 8.0, 3.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 4.0], [5.0, 5.0, 3.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 
6.0], [6.0, 6.0, 6.0], [2.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 8.0], [5.0, 2.0, 2.0], [8.0, 4.0, 4.0]], 
[[3.0, 8.0, 1.0], [1.0, 3.0, 3.0], [6.0, 1.0, 2.0]], [[6.0, 1.0, 6.0], [4.0, 7.0, 7.0], [7.0, 4.0, 2.0]]], [[[3.0, 1.0, 4.0], 
[1.0, 7.0, 8.0], [8.0, 4.0, 2.0]], [[4.0, 2.0, 6.0], [3.0, 2.0, 4.0], [1.0, 4.0, 6.0]], [[2.0, 8.0, 6.0], [1.0, 8.0, 7.0], 
[2.0, 5.0, 2.0]]], [[[5.0, 1.0, 7.0], [7.0, 8.0, 4.0], [3.0, 3.0, 7.0]], [[1.0, 2.0, 8.0], [2.0, 3.0, 5.0], [3.0, 6.0, 5.0]], 
[[7.0, 7.0, 1.0], [8.0, 2.0, 4.0], [8.0, 7.0, 8.0]]], [[[6.0, 4.0, 5.0], [4.0, 5.0, 2.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], 
[7.0, 3.0, 8.0], [4.0, 4.0, 1.0]], [[3.0, 4.0, 4.0], [4.0, 7.0, 6.0], [8.0, 1.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 1.0, 2.0]], [[1.0, 2.0, 6.0]], [[1.0, 8.0, 4.0]]], [[[1.0, 1.0, 4.0]], [[7.0, 2.0, 
7.0]], [[8.0, 8.0, 1.0]]], [[[2.0, 1.0, 1.0]], [[8.0, 8.0, 3.0]], [[7.0, 3.0, 7.0]]], [[[3.0, 4.0, 3.0]], [[2.0, 6.0, 8.0]], 
[[3.0, 8.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 6.0, 3.0]], [[1.0, 8.0, 3.0]], [[7.0, 1.0, 1.0]]], [[[7.0, 6.0, 1.0]], 
[[6.0, 7.0, 4.0]], [[6.0, 5.0, 6.0]]], [[[7.0, 7.0, 1.0]], [[7.0, 2.0, 3.0]], [[4.0, 3.0, 1.0]]], [[[4.0, 2.0, 7.0]], [[3.0, 
6.0, 8.0]], [[2.0, 7.0, 5.0]]]])
      arg[:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 3.0], [6.0, 1.0, 2.0], [8.0, 6.0, 3.0]], [[6.0, 1.0, 1.0], [1.0, 2.0, 6.0], [8.0, 
6.0, 5.0]], [[1.0, 4.0, 1.0], [1.0, 8.0, 4.0], [2.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [1.0, 1.0, 4.0], [6.0, 6.0, 8.0]], [[1.0, 
3.0, 7.0], [7.0, 2.0, 7.0], [6.0, 4.0, 6.0]], [[6.0, 2.0, 5.0], [8.0, 8.0, 1.0], [8.0, 1.0, 3.0]]], [[[4.0, 5.0, 2.0], [2.0, 
1.0, 1.0], [2.0, 3.0, 5.0]], [[6.0, 1.0, 3.0], [8.0, 8.0, 3.0], [6.0, 4.0, 1.0]], [[2.0, 7.0, 6.0], [7.0, 3.0, 7.0], [2.0, 3.0, 
3.0]]], [[[4.0, 1.0, 3.0], [3.0, 4.0, 3.0], [8.0, 8.0, 8.0]], [[7.0, 4.0, 4.0], [2.0, 6.0, 8.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 
6.0], [3.0, 8.0, 4.0], [2.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [5.0, 6.0, 3.0], [8.0, 4.0, 4.0]], 
[[3.0, 8.0, 1.0], [1.0, 8.0, 3.0], [6.0, 1.0, 2.0]], [[6.0, 1.0, 6.0], [7.0, 1.0, 1.0], [7.0, 4.0, 2.0]]], [[[3.0, 1.0, 4.0], 
[7.0, 6.0, 1.0], [8.0, 4.0, 2.0]], [[4.0, 2.0, 6.0], [6.0, 7.0, 4.0], [1.0, 4.0, 6.0]], [[2.0, 8.0, 6.0], [6.0, 5.0, 6.0], 
[2.0, 5.0, 2.0]]], [[[5.0, 1.0, 7.0], [7.0, 7.0, 1.0], [3.0, 3.0, 7.0]], [[1.0, 2.0, 8.0], [7.0, 2.0, 3.0], [3.0, 6.0, 5.0]], 
[[7.0, 7.0, 1.0], [4.0, 3.0, 1.0], [8.0, 7.0, 8.0]]], [[[6.0, 4.0, 5.0], [4.0, 2.0, 7.0], [8.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], 
[3.0, 6.0, 8.0], [4.0, 4.0, 1.0]], [[3.0, 4.0, 4.0], [2.0, 7.0, 5.0], [8.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 4.0], [7.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [3.0, 2.0, 6.0], [4.0, 
6.0, 5.0]], [[2.0, 5.0, 1.0], [6.0, 1.0, 5.0], [5.0, 3.0, 7.0]]], [[[3.0, 3.0, 2.0], [8.0, 5.0, 1.0], [3.0, 7.0, 1.0]], [[5.0, 
2.0, 8.0], [2.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[3.0, 3.0, 1.0], [7.0, 2.0, 3.0], [8.0, 8.0, 6.0]]], [[[3.0, 3.0, 2.0], [7.0, 
1.0, 1.0], [5.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [4.0, 7.0, 5.0], [4.0, 8.0, 4.0]], [[6.0, 8.0, 5.0], [6.0, 2.0, 7.0], [4.0, 1.0, 
2.0]]], [[[7.0, 8.0, 6.0], [3.0, 5.0, 8.0], [2.0, 7.0, 5.0]], [[5.0, 4.0, 5.0], [7.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[6.0, 6.0, 
6.0], [5.0, 6.0, 2.0], [8.0, 5.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 5.0], [5.0, 2.0, 3.0], [2.0, 6.0, 5.0]], 
[[6.0, 8.0, 1.0], [4.0, 8.0, 1.0], [7.0, 6.0, 3.0]], [[1.0, 7.0, 4.0], [1.0, 8.0, 8.0], [3.0, 7.0, 7.0]]], [[[4.0, 4.0, 8.0], 
[2.0, 5.0, 8.0], [6.0, 1.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 6.0, 8.0], [2.0, 1.0, 2.0]], [[5.0, 4.0, 3.0], [6.0, 1.0, 2.0], 
[1.0, 4.0, 2.0]]], [[[4.0, 7.0, 1.0], [3.0, 1.0, 8.0], [6.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [5.0, 5.0, 1.0], [2.0, 6.0, 5.0]], 
[[3.0, 7.0, 2.0], [4.0, 3.0, 7.0], [1.0, 4.0, 4.0]]], [[[4.0, 3.0, 3.0], [7.0, 4.0, 1.0], [4.0, 2.0, 8.0]], [[7.0, 5.0, 2.0], 
[8.0, 7.0, 2.0], [1.0, 8.0, 3.0]], [[6.0, 6.0, 5.0], [1.0, 8.0, 2.0], [5.0, 7.0, 5.0]]]])
      val=numpy.array([[[[4.0, 5.0, 7.0], [7.0, 8.0, 1.0]]], [[[1.0, 5.0, 5.0], [3.0, 3.0, 5.0]]], [[[8.0, 8.0, 4.0], [1.0, 
5.0, 8.0]]]])
      arg[:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [7.0, 8.0, 4.0], [2.0, 5.0, 6.0]], [[3.0, 2.0, 1.0], [3.0, 2.0, 6.0], [4.0, 
6.0, 5.0]], [[4.0, 5.0, 7.0], [7.0, 8.0, 1.0], [5.0, 3.0, 7.0]]], [[[3.0, 3.0, 2.0], [8.0, 5.0, 1.0], [3.0, 7.0, 1.0]], [[5.0, 
2.0, 8.0], [2.0, 4.0, 2.0], [2.0, 2.0, 6.0]], [[1.0, 5.0, 5.0], [3.0, 3.0, 5.0], [8.0, 8.0, 6.0]]], [[[3.0, 3.0, 2.0], [7.0, 
1.0, 1.0], [5.0, 6.0, 3.0]], [[4.0, 2.0, 7.0], [4.0, 7.0, 5.0], [4.0, 8.0, 4.0]], [[8.0, 8.0, 4.0], [1.0, 5.0, 8.0], [4.0, 1.0, 
2.0]]], [[[7.0, 8.0, 6.0], [3.0, 5.0, 8.0], [2.0, 7.0, 5.0]], [[5.0, 4.0, 5.0], [7.0, 1.0, 1.0], [1.0, 8.0, 2.0]], [[6.0, 6.0, 
6.0], [5.0, 6.0, 2.0], [8.0, 5.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [5.0, 2.0, 3.0], [2.0, 6.0, 5.0]], 
[[6.0, 8.0, 1.0], [4.0, 8.0, 1.0], [7.0, 6.0, 3.0]], [[4.0, 5.0, 7.0], [7.0, 8.0, 1.0], [3.0, 7.0, 7.0]]], [[[4.0, 4.0, 8.0], 
[2.0, 5.0, 8.0], [6.0, 1.0, 6.0]], [[3.0, 3.0, 3.0], [4.0, 6.0, 8.0], [2.0, 1.0, 2.0]], [[1.0, 5.0, 5.0], [3.0, 3.0, 5.0], 
[1.0, 4.0, 2.0]]], [[[4.0, 7.0, 1.0], [3.0, 1.0, 8.0], [6.0, 6.0, 5.0]], [[2.0, 6.0, 2.0], [5.0, 5.0, 1.0], [2.0, 6.0, 5.0]], 
[[8.0, 8.0, 4.0], [1.0, 5.0, 8.0], [1.0, 4.0, 4.0]]], [[[4.0, 3.0, 3.0], [7.0, 4.0, 1.0], [4.0, 2.0, 8.0]], [[7.0, 5.0, 2.0], 
[8.0, 7.0, 2.0], [1.0, 8.0, 3.0]], [[6.0, 6.0, 5.0], [1.0, 8.0, 2.0], [5.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 3.0], [7.0, 1.0, 5.0], [4.0, 7.0, 5.0]], [[2.0, 5.0, 8.0], [5.0, 6.0, 7.0], [3.0, 
4.0, 7.0]], [[3.0, 2.0, 6.0], [6.0, 2.0, 6.0], [5.0, 3.0, 2.0]]], [[[8.0, 5.0, 5.0], [3.0, 7.0, 7.0], [7.0, 7.0, 4.0]], [[7.0, 
1.0, 6.0], [6.0, 6.0, 1.0], [2.0, 7.0, 7.0]], [[7.0, 4.0, 1.0], [1.0, 7.0, 4.0], [8.0, 3.0, 3.0]]], [[[1.0, 7.0, 7.0], [6.0, 
2.0, 7.0], [3.0, 6.0, 1.0]], [[8.0, 8.0, 2.0], [2.0, 5.0, 1.0], [7.0, 7.0, 3.0]], [[5.0, 2.0, 7.0], [7.0, 5.0, 4.0], [5.0, 1.0, 
7.0]]], [[[8.0, 8.0, 7.0], [6.0, 8.0, 4.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [4.0, 5.0, 8.0], [3.0, 6.0, 1.0]], [[5.0, 7.0, 
1.0], [2.0, 2.0, 8.0], [8.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 7.0], [4.0, 8.0, 4.0], [5.0, 3.0, 5.0]], 
[[3.0, 7.0, 7.0], [2.0, 1.0, 7.0], [1.0, 5.0, 4.0]], [[4.0, 7.0, 5.0], [2.0, 1.0, 3.0], [3.0, 2.0, 1.0]]], [[[5.0, 6.0, 6.0], 
[8.0, 2.0, 6.0], [7.0, 5.0, 3.0]], [[3.0, 6.0, 1.0], [8.0, 8.0, 1.0], [8.0, 1.0, 6.0]], [[8.0, 7.0, 3.0], [7.0, 5.0, 4.0], 
[2.0, 2.0, 6.0]]], [[[4.0, 4.0, 1.0], [4.0, 6.0, 5.0], [4.0, 3.0, 1.0]], [[7.0, 6.0, 1.0], [1.0, 1.0, 4.0], [2.0, 4.0, 7.0]], 
[[1.0, 7.0, 3.0], [5.0, 2.0, 7.0], [1.0, 2.0, 4.0]]], [[[5.0, 5.0, 3.0], [6.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[7.0, 6.0, 1.0], 
[5.0, 8.0, 5.0], [6.0, 6.0, 2.0]], [[3.0, 3.0, 8.0], [1.0, 8.0, 3.0], [8.0, 2.0, 3.0]]]])
      val=Data(numpy.array([[[[8.0, 4.0, 2.0], [4.0, 8.0, 8.0]]], [[[7.0, 5.0, 7.0], [7.0, 5.0, 5.0]]], [[[3.0, 7.0, 3.0], 
[6.0, 3.0, 5.0]]]]),self.functionspace)
      arg[:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 3.0], [7.0, 1.0, 5.0], [4.0, 7.0, 5.0]], [[2.0, 5.0, 8.0], [5.0, 6.0, 7.0], [3.0, 
4.0, 7.0]], [[8.0, 4.0, 2.0], [4.0, 8.0, 8.0], [5.0, 3.0, 2.0]]], [[[8.0, 5.0, 5.0], [3.0, 7.0, 7.0], [7.0, 7.0, 4.0]], [[7.0, 
1.0, 6.0], [6.0, 6.0, 1.0], [2.0, 7.0, 7.0]], [[7.0, 5.0, 7.0], [7.0, 5.0, 5.0], [8.0, 3.0, 3.0]]], [[[1.0, 7.0, 7.0], [6.0, 
2.0, 7.0], [3.0, 6.0, 1.0]], [[8.0, 8.0, 2.0], [2.0, 5.0, 1.0], [7.0, 7.0, 3.0]], [[3.0, 7.0, 3.0], [6.0, 3.0, 5.0], [5.0, 1.0, 
7.0]]], [[[8.0, 8.0, 7.0], [6.0, 8.0, 4.0], [3.0, 5.0, 4.0]], [[4.0, 2.0, 5.0], [4.0, 5.0, 8.0], [3.0, 6.0, 1.0]], [[5.0, 7.0, 
1.0], [2.0, 2.0, 8.0], [8.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 7.0], [4.0, 8.0, 4.0], [5.0, 3.0, 5.0]], 
[[3.0, 7.0, 7.0], [2.0, 1.0, 7.0], [1.0, 5.0, 4.0]], [[8.0, 4.0, 2.0], [4.0, 8.0, 8.0], [3.0, 2.0, 1.0]]], [[[5.0, 6.0, 6.0], 
[8.0, 2.0, 6.0], [7.0, 5.0, 3.0]], [[3.0, 6.0, 1.0], [8.0, 8.0, 1.0], [8.0, 1.0, 6.0]], [[7.0, 5.0, 7.0], [7.0, 5.0, 5.0], 
[2.0, 2.0, 6.0]]], [[[4.0, 4.0, 1.0], [4.0, 6.0, 5.0], [4.0, 3.0, 1.0]], [[7.0, 6.0, 1.0], [1.0, 1.0, 4.0], [2.0, 4.0, 7.0]], 
[[3.0, 7.0, 3.0], [6.0, 3.0, 5.0], [1.0, 2.0, 4.0]]], [[[5.0, 5.0, 3.0], [6.0, 4.0, 5.0], [7.0, 3.0, 2.0]], [[7.0, 6.0, 1.0], 
[5.0, 8.0, 5.0], [6.0, 6.0, 2.0]], [[3.0, 3.0, 8.0], [1.0, 8.0, 3.0], [8.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 2.0], [2.0, 5.0, 7.0], [1.0, 8.0, 5.0]], [[5.0, 6.0, 7.0], [5.0, 1.0, 3.0], [3.0, 
5.0, 8.0]], [[3.0, 4.0, 7.0], [5.0, 8.0, 5.0], [5.0, 8.0, 7.0]]], [[[7.0, 7.0, 4.0], [8.0, 4.0, 5.0], [3.0, 8.0, 3.0]], [[1.0, 
2.0, 1.0], [4.0, 7.0, 3.0], [5.0, 5.0, 8.0]], [[4.0, 5.0, 7.0], [6.0, 3.0, 8.0], [6.0, 8.0, 8.0]]], [[[1.0, 7.0, 3.0], [6.0, 
8.0, 8.0], [2.0, 3.0, 3.0]], [[3.0, 3.0, 5.0], [2.0, 8.0, 5.0], [2.0, 8.0, 1.0]], [[4.0, 5.0, 8.0], [3.0, 5.0, 6.0], [7.0, 1.0, 
5.0]]], [[[5.0, 6.0, 2.0], [2.0, 1.0, 4.0], [8.0, 8.0, 6.0]], [[8.0, 5.0, 8.0], [1.0, 8.0, 1.0], [7.0, 5.0, 5.0]], [[8.0, 5.0, 
2.0], [8.0, 5.0, 6.0], [6.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 6.0], [6.0, 2.0, 5.0], [5.0, 5.0, 4.0]], 
[[3.0, 2.0, 8.0], [5.0, 7.0, 1.0], [2.0, 8.0, 8.0]], [[7.0, 4.0, 7.0], [7.0, 2.0, 4.0], [3.0, 8.0, 2.0]]], [[[2.0, 5.0, 7.0], 
[7.0, 2.0, 6.0], [4.0, 2.0, 3.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 5.0], [4.0, 7.0, 6.0]], [[4.0, 7.0, 3.0], [3.0, 8.0, 8.0], 
[8.0, 8.0, 3.0]]], [[[4.0, 1.0, 6.0], [7.0, 2.0, 1.0], [5.0, 5.0, 7.0]], [[4.0, 5.0, 2.0], [8.0, 4.0, 2.0], [5.0, 7.0, 3.0]], 
[[8.0, 4.0, 4.0], [7.0, 3.0, 4.0], [3.0, 3.0, 2.0]]], [[[8.0, 2.0, 5.0], [3.0, 3.0, 5.0], [8.0, 8.0, 6.0]], [[2.0, 6.0, 2.0], 
[6.0, 2.0, 2.0], [5.0, 3.0, 3.0]], [[1.0, 2.0, 6.0], [2.0, 6.0, 4.0], [2.0, 5.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 2.0, 6.0], [2.0, 1.0, 7.0]]], [[[6.0, 3.0, 6.0], [2.0, 8.0, 3.0]]], [[[4.0, 6.0, 1.0], 
[2.0, 3.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 2.0, 3.0], [5.0, 6.0, 8.0]]], [[[1.0, 5.0, 8.0], [1.0, 2.0, 7.0]]], 
[[[2.0, 1.0, 5.0], [8.0, 4.0, 7.0]]]])
      arg[:3,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 2.0], [2.0, 5.0, 7.0], [1.0, 8.0, 5.0]], [[5.0, 6.0, 7.0], [5.0, 1.0, 3.0], [3.0, 
5.0, 8.0]], [[3.0, 2.0, 6.0], [2.0, 1.0, 7.0], [5.0, 8.0, 7.0]]], [[[7.0, 7.0, 4.0], [8.0, 4.0, 5.0], [3.0, 8.0, 3.0]], [[1.0, 
2.0, 1.0], [4.0, 7.0, 3.0], [5.0, 5.0, 8.0]], [[6.0, 3.0, 6.0], [2.0, 8.0, 3.0], [6.0, 8.0, 8.0]]], [[[1.0, 7.0, 3.0], [6.0, 
8.0, 8.0], [2.0, 3.0, 3.0]], [[3.0, 3.0, 5.0], [2.0, 8.0, 5.0], [2.0, 8.0, 1.0]], [[4.0, 6.0, 1.0], [2.0, 3.0, 1.0], [7.0, 1.0, 
5.0]]], [[[5.0, 6.0, 2.0], [2.0, 1.0, 4.0], [8.0, 8.0, 6.0]], [[8.0, 5.0, 8.0], [1.0, 8.0, 1.0], [7.0, 5.0, 5.0]], [[8.0, 5.0, 
2.0], [8.0, 5.0, 6.0], [6.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 6.0], [6.0, 2.0, 5.0], [5.0, 5.0, 4.0]], 
[[3.0, 2.0, 8.0], [5.0, 7.0, 1.0], [2.0, 8.0, 8.0]], [[5.0, 2.0, 3.0], [5.0, 6.0, 8.0], [3.0, 8.0, 2.0]]], [[[2.0, 5.0, 7.0], 
[7.0, 2.0, 6.0], [4.0, 2.0, 3.0]], [[1.0, 3.0, 5.0], [3.0, 5.0, 5.0], [4.0, 7.0, 6.0]], [[1.0, 5.0, 8.0], [1.0, 2.0, 7.0], 
[8.0, 8.0, 3.0]]], [[[4.0, 1.0, 6.0], [7.0, 2.0, 1.0], [5.0, 5.0, 7.0]], [[4.0, 5.0, 2.0], [8.0, 4.0, 2.0], [5.0, 7.0, 3.0]], 
[[2.0, 1.0, 5.0], [8.0, 4.0, 7.0], [3.0, 3.0, 2.0]]], [[[8.0, 2.0, 5.0], [3.0, 3.0, 5.0], [8.0, 8.0, 6.0]], [[2.0, 6.0, 2.0], 
[6.0, 2.0, 2.0], [5.0, 3.0, 3.0]], [[1.0, 2.0, 6.0], [2.0, 6.0, 4.0], [2.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 8.0], [6.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 5.0, 1.0], [4.0, 7.0, 2.0], [5.0, 
4.0, 2.0]], [[6.0, 6.0, 4.0], [3.0, 7.0, 8.0], [6.0, 8.0, 6.0]]], [[[7.0, 7.0, 1.0], [5.0, 8.0, 3.0], [5.0, 6.0, 1.0]], [[7.0, 
7.0, 8.0], [2.0, 3.0, 2.0], [5.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [3.0, 2.0, 8.0], [4.0, 7.0, 5.0]]], [[[6.0, 4.0, 1.0], [1.0, 
1.0, 2.0], [3.0, 1.0, 7.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 4.0], [6.0, 7.0, 1.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 1.0], [3.0, 4.0, 
2.0]]], [[[8.0, 7.0, 1.0], [7.0, 1.0, 2.0], [4.0, 8.0, 3.0]], [[8.0, 7.0, 7.0], [6.0, 4.0, 2.0], [1.0, 5.0, 7.0]], [[7.0, 6.0, 
1.0], [2.0, 3.0, 1.0], [6.0, 7.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 1.0], [8.0, 3.0, 6.0], [4.0, 6.0, 3.0]], 
[[2.0, 4.0, 3.0], [2.0, 2.0, 1.0], [2.0, 6.0, 7.0]], [[6.0, 3.0, 2.0], [7.0, 6.0, 6.0], [7.0, 4.0, 3.0]]], [[[1.0, 2.0, 8.0], 
[2.0, 7.0, 5.0], [7.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], [1.0, 1.0, 5.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 6.0], [7.0, 5.0, 1.0], 
[7.0, 7.0, 6.0]]], [[[3.0, 4.0, 3.0], [2.0, 6.0, 3.0], [5.0, 1.0, 2.0]], [[6.0, 8.0, 8.0], [5.0, 4.0, 8.0], [1.0, 4.0, 2.0]], 
[[3.0, 8.0, 2.0], [4.0, 3.0, 1.0], [1.0, 6.0, 2.0]]], [[[8.0, 1.0, 4.0], [6.0, 6.0, 7.0], [5.0, 6.0, 7.0]], [[1.0, 7.0, 8.0], 
[4.0, 5.0, 6.0], [4.0, 8.0, 2.0]], [[6.0, 4.0, 2.0], [7.0, 1.0, 4.0], [7.0, 4.0, 6.0]]]])
      val=numpy.array([[[7.0, 4.0, 5.0]], [[8.0, 7.0, 4.0]]])
      arg[2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 8.0], [6.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 5.0, 1.0], [4.0, 7.0, 2.0], [5.0, 
4.0, 2.0]], [[6.0, 6.0, 4.0], [3.0, 7.0, 8.0], [6.0, 8.0, 6.0]]], [[[7.0, 7.0, 1.0], [5.0, 8.0, 3.0], [5.0, 6.0, 1.0]], [[7.0, 
7.0, 8.0], [2.0, 3.0, 2.0], [5.0, 7.0, 6.0]], [[6.0, 7.0, 3.0], [3.0, 2.0, 8.0], [4.0, 7.0, 5.0]]], [[[6.0, 4.0, 1.0], [1.0, 
1.0, 2.0], [3.0, 1.0, 7.0]], [[6.0, 8.0, 1.0], [8.0, 2.0, 4.0], [7.0, 4.0, 5.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 1.0], [3.0, 4.0, 
2.0]]], [[[8.0, 7.0, 1.0], [7.0, 1.0, 2.0], [4.0, 8.0, 3.0]], [[8.0, 7.0, 7.0], [6.0, 4.0, 2.0], [8.0, 7.0, 4.0]], [[7.0, 6.0, 
1.0], [2.0, 3.0, 1.0], [6.0, 7.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [8.0, 3.0, 6.0], [4.0, 6.0, 3.0]], 
[[2.0, 4.0, 3.0], [2.0, 2.0, 1.0], [2.0, 6.0, 7.0]], [[6.0, 3.0, 2.0], [7.0, 6.0, 6.0], [7.0, 4.0, 3.0]]], [[[1.0, 2.0, 8.0], 
[2.0, 7.0, 5.0], [7.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], [1.0, 1.0, 5.0], [3.0, 2.0, 2.0]], [[4.0, 5.0, 6.0], [7.0, 5.0, 1.0], 
[7.0, 7.0, 6.0]]], [[[3.0, 4.0, 3.0], [2.0, 6.0, 3.0], [5.0, 1.0, 2.0]], [[6.0, 8.0, 8.0], [5.0, 4.0, 8.0], [7.0, 4.0, 5.0]], 
[[3.0, 8.0, 2.0], [4.0, 3.0, 1.0], [1.0, 6.0, 2.0]]], [[[8.0, 1.0, 4.0], [6.0, 6.0, 7.0], [5.0, 6.0, 7.0]], [[1.0, 7.0, 8.0], 
[4.0, 5.0, 6.0], [8.0, 7.0, 4.0]], [[6.0, 4.0, 2.0], [7.0, 1.0, 4.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 1.0], [4.0, 2.0, 4.0], [2.0, 6.0, 6.0]], [[3.0, 6.0, 1.0], [6.0, 8.0, 1.0], [6.0, 
7.0, 5.0]], [[7.0, 3.0, 1.0], [1.0, 3.0, 7.0], [1.0, 4.0, 6.0]]], [[[1.0, 2.0, 1.0], [1.0, 7.0, 3.0], [4.0, 8.0, 7.0]], [[6.0, 
4.0, 4.0], [5.0, 1.0, 5.0], [7.0, 7.0, 6.0]], [[1.0, 8.0, 6.0], [6.0, 2.0, 6.0], [8.0, 1.0, 4.0]]], [[[2.0, 7.0, 8.0], [8.0, 
5.0, 1.0], [4.0, 4.0, 3.0]], [[5.0, 1.0, 4.0], [7.0, 6.0, 3.0], [2.0, 4.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 4.0, 8.0], [1.0, 1.0, 
1.0]]], [[[7.0, 2.0, 7.0], [6.0, 6.0, 6.0], [5.0, 3.0, 7.0]], [[1.0, 4.0, 6.0], [2.0, 5.0, 6.0], [4.0, 7.0, 3.0]], [[6.0, 5.0, 
3.0], [4.0, 6.0, 8.0], [3.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 5.0], [6.0, 7.0, 1.0], [2.0, 4.0, 2.0]], 
[[8.0, 1.0, 1.0], [7.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[7.0, 5.0, 3.0], [3.0, 6.0, 6.0], [8.0, 6.0, 4.0]]], [[[2.0, 7.0, 5.0], 
[7.0, 6.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 5.0, 2.0], [5.0, 2.0, 3.0], [1.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [2.0, 7.0, 4.0], 
[7.0, 2.0, 4.0]]], [[[5.0, 1.0, 2.0], [8.0, 7.0, 8.0], [1.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [6.0, 3.0, 3.0], [1.0, 6.0, 3.0]], 
[[5.0, 3.0, 6.0], [6.0, 1.0, 4.0], [8.0, 6.0, 5.0]]], [[[7.0, 7.0, 1.0], [7.0, 3.0, 6.0], [6.0, 3.0, 4.0]], [[5.0, 3.0, 8.0], 
[1.0, 4.0, 6.0], [3.0, 4.0, 4.0]], [[6.0, 5.0, 7.0], [8.0, 1.0, 7.0], [3.0, 3.0, 2.0]]]])
      val=Data(numpy.array([[[5.0, 3.0, 6.0]], [[8.0, 3.0, 4.0]]]),self.functionspace)
      arg[2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 1.0], [4.0, 2.0, 4.0], [2.0, 6.0, 6.0]], [[3.0, 6.0, 1.0], [6.0, 8.0, 1.0], [6.0, 
7.0, 5.0]], [[7.0, 3.0, 1.0], [1.0, 3.0, 7.0], [1.0, 4.0, 6.0]]], [[[1.0, 2.0, 1.0], [1.0, 7.0, 3.0], [4.0, 8.0, 7.0]], [[6.0, 
4.0, 4.0], [5.0, 1.0, 5.0], [7.0, 7.0, 6.0]], [[1.0, 8.0, 6.0], [6.0, 2.0, 6.0], [8.0, 1.0, 4.0]]], [[[2.0, 7.0, 8.0], [8.0, 
5.0, 1.0], [4.0, 4.0, 3.0]], [[5.0, 1.0, 4.0], [7.0, 6.0, 3.0], [5.0, 3.0, 6.0]], [[7.0, 7.0, 3.0], [5.0, 4.0, 8.0], [1.0, 1.0, 
1.0]]], [[[7.0, 2.0, 7.0], [6.0, 6.0, 6.0], [5.0, 3.0, 7.0]], [[1.0, 4.0, 6.0], [2.0, 5.0, 6.0], [8.0, 3.0, 4.0]], [[6.0, 5.0, 
3.0], [4.0, 6.0, 8.0], [3.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [6.0, 7.0, 1.0], [2.0, 4.0, 2.0]], 
[[8.0, 1.0, 1.0], [7.0, 6.0, 6.0], [1.0, 4.0, 2.0]], [[7.0, 5.0, 3.0], [3.0, 6.0, 6.0], [8.0, 6.0, 4.0]]], [[[2.0, 7.0, 5.0], 
[7.0, 6.0, 2.0], [7.0, 6.0, 5.0]], [[2.0, 5.0, 2.0], [5.0, 2.0, 3.0], [1.0, 5.0, 8.0]], [[3.0, 6.0, 3.0], [2.0, 7.0, 4.0], 
[7.0, 2.0, 4.0]]], [[[5.0, 1.0, 2.0], [8.0, 7.0, 8.0], [1.0, 8.0, 5.0]], [[6.0, 3.0, 4.0], [6.0, 3.0, 3.0], [5.0, 3.0, 6.0]], 
[[5.0, 3.0, 6.0], [6.0, 1.0, 4.0], [8.0, 6.0, 5.0]]], [[[7.0, 7.0, 1.0], [7.0, 3.0, 6.0], [6.0, 3.0, 4.0]], [[5.0, 3.0, 8.0], 
[1.0, 4.0, 6.0], [8.0, 3.0, 4.0]], [[6.0, 5.0, 7.0], [8.0, 1.0, 7.0], [3.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 8.0], [8.0, 4.0, 1.0], [3.0, 1.0, 1.0]], [[3.0, 7.0, 2.0], [5.0, 2.0, 7.0], [6.0, 
1.0, 7.0]], [[2.0, 5.0, 2.0], [8.0, 6.0, 2.0], [1.0, 3.0, 6.0]]], [[[8.0, 8.0, 7.0], [5.0, 2.0, 8.0], [8.0, 7.0, 1.0]], [[2.0, 
5.0, 4.0], [5.0, 8.0, 3.0], [5.0, 5.0, 8.0]], [[3.0, 6.0, 5.0], [6.0, 2.0, 8.0], [7.0, 8.0, 3.0]]], [[[8.0, 4.0, 1.0], [7.0, 
3.0, 6.0], [7.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [7.0, 7.0, 1.0], [6.0, 4.0, 5.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 4.0], [3.0, 1.0, 
4.0]]], [[[7.0, 5.0, 3.0], [2.0, 1.0, 1.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 8.0], [2.0, 5.0, 5.0], [2.0, 6.0, 1.0]], [[5.0, 1.0, 
6.0], [1.0, 5.0, 5.0], [8.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 4.0, 7.0], [2.0, 6.0, 5.0], [7.0, 4.0, 7.0]], 
[[1.0, 2.0, 2.0], [3.0, 7.0, 7.0], [5.0, 1.0, 4.0]], [[8.0, 7.0, 6.0], [2.0, 7.0, 4.0], [2.0, 1.0, 8.0]]], [[[6.0, 2.0, 4.0], 
[4.0, 8.0, 1.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], [4.0, 1.0, 3.0], [2.0, 8.0, 5.0]], [[5.0, 5.0, 3.0], [2.0, 8.0, 3.0], 
[4.0, 4.0, 7.0]]], [[[4.0, 4.0, 8.0], [1.0, 2.0, 7.0], [8.0, 8.0, 5.0]], [[8.0, 1.0, 7.0], [1.0, 3.0, 2.0], [1.0, 1.0, 6.0]], 
[[2.0, 5.0, 2.0], [5.0, 5.0, 6.0], [2.0, 6.0, 6.0]]], [[[5.0, 6.0, 4.0], [7.0, 8.0, 8.0], [1.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], 
[5.0, 7.0, 3.0], [3.0, 2.0, 7.0]], [[4.0, 7.0, 6.0], [4.0, 1.0, 3.0], [2.0, 5.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 7.0, 1.0]], [[3.0, 5.0, 4.0]]])+(1.-msk_val)*numpy.array([[[4.0, 8.0, 8.0]], [[7.0, 2.0, 
6.0]]])
      arg[2:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 8.0], [8.0, 4.0, 1.0], [3.0, 1.0, 1.0]], [[3.0, 7.0, 2.0], [5.0, 2.0, 7.0], [6.0, 
1.0, 7.0]], [[2.0, 5.0, 2.0], [8.0, 6.0, 2.0], [1.0, 3.0, 6.0]]], [[[8.0, 8.0, 7.0], [5.0, 2.0, 8.0], [8.0, 7.0, 1.0]], [[2.0, 
5.0, 4.0], [5.0, 8.0, 3.0], [5.0, 5.0, 8.0]], [[3.0, 6.0, 5.0], [6.0, 2.0, 8.0], [7.0, 8.0, 3.0]]], [[[8.0, 4.0, 1.0], [7.0, 
3.0, 6.0], [7.0, 2.0, 4.0]], [[6.0, 3.0, 2.0], [7.0, 7.0, 1.0], [2.0, 7.0, 1.0]], [[8.0, 2.0, 5.0], [4.0, 6.0, 4.0], [3.0, 1.0, 
4.0]]], [[[7.0, 5.0, 3.0], [2.0, 1.0, 1.0], [7.0, 3.0, 1.0]], [[4.0, 2.0, 8.0], [2.0, 5.0, 5.0], [3.0, 5.0, 4.0]], [[5.0, 1.0, 
6.0], [1.0, 5.0, 5.0], [8.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 4.0, 7.0], [2.0, 6.0, 5.0], [7.0, 4.0, 7.0]], 
[[1.0, 2.0, 2.0], [3.0, 7.0, 7.0], [5.0, 1.0, 4.0]], [[8.0, 7.0, 6.0], [2.0, 7.0, 4.0], [2.0, 1.0, 8.0]]], [[[6.0, 2.0, 4.0], 
[4.0, 8.0, 1.0], [7.0, 6.0, 2.0]], [[6.0, 2.0, 4.0], [4.0, 1.0, 3.0], [2.0, 8.0, 5.0]], [[5.0, 5.0, 3.0], [2.0, 8.0, 3.0], 
[4.0, 4.0, 7.0]]], [[[4.0, 4.0, 8.0], [1.0, 2.0, 7.0], [8.0, 8.0, 5.0]], [[8.0, 1.0, 7.0], [1.0, 3.0, 2.0], [4.0, 8.0, 8.0]], 
[[2.0, 5.0, 2.0], [5.0, 5.0, 6.0], [2.0, 6.0, 6.0]]], [[[5.0, 6.0, 4.0], [7.0, 8.0, 8.0], [1.0, 3.0, 8.0]], [[8.0, 1.0, 8.0], 
[5.0, 7.0, 3.0], [7.0, 2.0, 6.0]], [[4.0, 7.0, 6.0], [4.0, 1.0, 3.0], [2.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 6.0], [2.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[7.0, 2.0, 8.0], [3.0, 5.0, 2.0], [7.0, 
4.0, 7.0]], [[1.0, 8.0, 6.0], [5.0, 1.0, 8.0], [1.0, 4.0, 6.0]]], [[[6.0, 7.0, 6.0], [6.0, 3.0, 7.0], [3.0, 1.0, 2.0]], [[7.0, 
3.0, 8.0], [3.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 3.0, 1.0], [3.0, 4.0, 3.0], [1.0, 4.0, 3.0]]], [[[7.0, 7.0, 4.0], [3.0, 
1.0, 5.0], [6.0, 4.0, 7.0]], [[7.0, 1.0, 8.0], [8.0, 8.0, 3.0], [3.0, 3.0, 4.0]], [[2.0, 1.0, 6.0], [7.0, 4.0, 1.0], [3.0, 5.0, 
6.0]]], [[[3.0, 1.0, 6.0], [8.0, 6.0, 2.0], [7.0, 8.0, 7.0]], [[1.0, 1.0, 5.0], [3.0, 6.0, 7.0], [3.0, 2.0, 4.0]], [[4.0, 8.0, 
2.0], [7.0, 1.0, 7.0], [1.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 8.0], [1.0, 7.0, 6.0], [8.0, 3.0, 4.0]], 
[[5.0, 1.0, 4.0], [2.0, 4.0, 8.0], [7.0, 2.0, 3.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 6.0], [7.0, 1.0, 5.0]]], [[[4.0, 3.0, 3.0], 
[8.0, 2.0, 1.0], [6.0, 2.0, 3.0]], [[1.0, 5.0, 2.0], [5.0, 8.0, 2.0], [2.0, 1.0, 6.0]], [[8.0, 2.0, 6.0], [8.0, 3.0, 2.0], 
[4.0, 6.0, 8.0]]], [[[7.0, 2.0, 6.0], [7.0, 7.0, 7.0], [7.0, 3.0, 5.0]], [[5.0, 7.0, 1.0], [5.0, 7.0, 7.0], [3.0, 4.0, 4.0]], 
[[6.0, 5.0, 6.0], [3.0, 6.0, 5.0], [1.0, 8.0, 7.0]]], [[[3.0, 6.0, 8.0], [1.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[7.0, 5.0, 1.0], 
[7.0, 8.0, 5.0], [4.0, 1.0, 8.0]], [[4.0, 8.0, 3.0], [3.0, 7.0, 3.0], [3.0, 2.0, 2.0]]]])
      val=numpy.array([[[[4.0, 3.0, 7.0], [5.0, 3.0, 1.0]]], [[[3.0, 5.0, 5.0], [1.0, 3.0, 1.0]]], [[[4.0, 3.0, 5.0], [1.0, 
4.0, 6.0]]]])
      arg[:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 6.0], [2.0, 5.0, 3.0], [5.0, 1.0, 1.0]], [[4.0, 3.0, 7.0], [5.0, 3.0, 1.0], [7.0, 
4.0, 7.0]], [[1.0, 8.0, 6.0], [5.0, 1.0, 8.0], [1.0, 4.0, 6.0]]], [[[6.0, 7.0, 6.0], [6.0, 3.0, 7.0], [3.0, 1.0, 2.0]], [[3.0, 
5.0, 5.0], [1.0, 3.0, 1.0], [1.0, 3.0, 7.0]], [[6.0, 3.0, 1.0], [3.0, 4.0, 3.0], [1.0, 4.0, 3.0]]], [[[7.0, 7.0, 4.0], [3.0, 
1.0, 5.0], [6.0, 4.0, 7.0]], [[4.0, 3.0, 5.0], [1.0, 4.0, 6.0], [3.0, 3.0, 4.0]], [[2.0, 1.0, 6.0], [7.0, 4.0, 1.0], [3.0, 5.0, 
6.0]]], [[[3.0, 1.0, 6.0], [8.0, 6.0, 2.0], [7.0, 8.0, 7.0]], [[1.0, 1.0, 5.0], [3.0, 6.0, 7.0], [3.0, 2.0, 4.0]], [[4.0, 8.0, 
2.0], [7.0, 1.0, 7.0], [1.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 8.0], [1.0, 7.0, 6.0], [8.0, 3.0, 4.0]], 
[[4.0, 3.0, 7.0], [5.0, 3.0, 1.0], [7.0, 2.0, 3.0]], [[1.0, 6.0, 7.0], [1.0, 3.0, 6.0], [7.0, 1.0, 5.0]]], [[[4.0, 3.0, 3.0], 
[8.0, 2.0, 1.0], [6.0, 2.0, 3.0]], [[3.0, 5.0, 5.0], [1.0, 3.0, 1.0], [2.0, 1.0, 6.0]], [[8.0, 2.0, 6.0], [8.0, 3.0, 2.0], 
[4.0, 6.0, 8.0]]], [[[7.0, 2.0, 6.0], [7.0, 7.0, 7.0], [7.0, 3.0, 5.0]], [[4.0, 3.0, 5.0], [1.0, 4.0, 6.0], [3.0, 4.0, 4.0]], 
[[6.0, 5.0, 6.0], [3.0, 6.0, 5.0], [1.0, 8.0, 7.0]]], [[[3.0, 6.0, 8.0], [1.0, 7.0, 7.0], [6.0, 8.0, 3.0]], [[7.0, 5.0, 1.0], 
[7.0, 8.0, 5.0], [4.0, 1.0, 8.0]], [[4.0, 8.0, 3.0], [3.0, 7.0, 3.0], [3.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[4.0, 8.0, 1.0], [6.0, 1.0, 5.0], [8.0, 
1.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 7.0, 3.0], [6.0, 2.0, 3.0]]], [[[7.0, 1.0, 4.0], [2.0, 8.0, 7.0], [1.0, 3.0, 3.0]], [[8.0, 
2.0, 8.0], [3.0, 7.0, 6.0], [1.0, 5.0, 1.0]], [[1.0, 3.0, 5.0], [1.0, 5.0, 1.0], [3.0, 4.0, 4.0]]], [[[1.0, 8.0, 1.0], [6.0, 
1.0, 4.0], [7.0, 6.0, 4.0]], [[1.0, 2.0, 5.0], [2.0, 3.0, 1.0], [2.0, 5.0, 8.0]], [[8.0, 5.0, 7.0], [8.0, 8.0, 1.0], [7.0, 2.0, 
7.0]]], [[[3.0, 5.0, 5.0], [5.0, 5.0, 4.0], [2.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 4.0], [5.0, 7.0, 1.0]], [[4.0, 1.0, 
3.0], [5.0, 5.0, 6.0], [6.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 6.0], [6.0, 1.0, 2.0], [3.0, 5.0, 6.0]], 
[[4.0, 1.0, 4.0], [8.0, 1.0, 7.0], [2.0, 8.0, 6.0]], [[8.0, 8.0, 5.0], [3.0, 7.0, 8.0], [5.0, 4.0, 7.0]]], [[[2.0, 6.0, 4.0], 
[4.0, 4.0, 8.0], [1.0, 2.0, 2.0]], [[2.0, 3.0, 8.0], [5.0, 2.0, 2.0], [4.0, 2.0, 3.0]], [[7.0, 8.0, 6.0], [3.0, 5.0, 4.0], 
[1.0, 6.0, 2.0]]], [[[8.0, 5.0, 1.0], [1.0, 8.0, 7.0], [8.0, 8.0, 4.0]], [[4.0, 8.0, 2.0], [1.0, 7.0, 4.0], [3.0, 7.0, 1.0]], 
[[6.0, 2.0, 2.0], [6.0, 3.0, 4.0], [3.0, 2.0, 2.0]]], [[[1.0, 3.0, 4.0], [1.0, 1.0, 8.0], [4.0, 1.0, 3.0]], [[4.0, 1.0, 8.0], 
[7.0, 7.0, 5.0], [2.0, 1.0, 7.0]], [[3.0, 6.0, 2.0], [1.0, 5.0, 8.0], [5.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[[[5.0, 8.0, 4.0], [2.0, 8.0, 8.0]]], [[[8.0, 2.0, 6.0], [1.0, 4.0, 6.0]]], [[[3.0, 8.0, 2.0], 
[1.0, 8.0, 5.0]]]]),self.functionspace)
      arg[:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 6.0, 4.0], [6.0, 7.0, 6.0]], [[5.0, 8.0, 4.0], [2.0, 8.0, 8.0], [8.0, 
1.0, 4.0]], [[2.0, 1.0, 6.0], [8.0, 7.0, 3.0], [6.0, 2.0, 3.0]]], [[[7.0, 1.0, 4.0], [2.0, 8.0, 7.0], [1.0, 3.0, 3.0]], [[8.0, 
2.0, 6.0], [1.0, 4.0, 6.0], [1.0, 5.0, 1.0]], [[1.0, 3.0, 5.0], [1.0, 5.0, 1.0], [3.0, 4.0, 4.0]]], [[[1.0, 8.0, 1.0], [6.0, 
1.0, 4.0], [7.0, 6.0, 4.0]], [[3.0, 8.0, 2.0], [1.0, 8.0, 5.0], [2.0, 5.0, 8.0]], [[8.0, 5.0, 7.0], [8.0, 8.0, 1.0], [7.0, 2.0, 
7.0]]], [[[3.0, 5.0, 5.0], [5.0, 5.0, 4.0], [2.0, 5.0, 8.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 4.0], [5.0, 7.0, 1.0]], [[4.0, 1.0, 
3.0], [5.0, 5.0, 6.0], [6.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 6.0], [6.0, 1.0, 2.0], [3.0, 5.0, 6.0]], 
[[5.0, 8.0, 4.0], [2.0, 8.0, 8.0], [2.0, 8.0, 6.0]], [[8.0, 8.0, 5.0], [3.0, 7.0, 8.0], [5.0, 4.0, 7.0]]], [[[2.0, 6.0, 4.0], 
[4.0, 4.0, 8.0], [1.0, 2.0, 2.0]], [[8.0, 2.0, 6.0], [1.0, 4.0, 6.0], [4.0, 2.0, 3.0]], [[7.0, 8.0, 6.0], [3.0, 5.0, 4.0], 
[1.0, 6.0, 2.0]]], [[[8.0, 5.0, 1.0], [1.0, 8.0, 7.0], [8.0, 8.0, 4.0]], [[3.0, 8.0, 2.0], [1.0, 8.0, 5.0], [3.0, 7.0, 1.0]], 
[[6.0, 2.0, 2.0], [6.0, 3.0, 4.0], [3.0, 2.0, 2.0]]], [[[1.0, 3.0, 4.0], [1.0, 1.0, 8.0], [4.0, 1.0, 3.0]], [[4.0, 1.0, 8.0], 
[7.0, 7.0, 5.0], [2.0, 1.0, 7.0]], [[3.0, 6.0, 2.0], [1.0, 5.0, 8.0], [5.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 3.0], [6.0, 1.0, 2.0], [5.0, 8.0, 4.0]], [[4.0, 6.0, 8.0], [3.0, 4.0, 8.0], [1.0, 
4.0, 5.0]], [[8.0, 5.0, 4.0], [6.0, 6.0, 7.0], [6.0, 5.0, 8.0]]], [[[3.0, 1.0, 6.0], [6.0, 5.0, 7.0], [7.0, 8.0, 1.0]], [[3.0, 
6.0, 8.0], [3.0, 8.0, 5.0], [2.0, 1.0, 4.0]], [[6.0, 4.0, 6.0], [5.0, 1.0, 7.0], [7.0, 1.0, 8.0]]], [[[7.0, 5.0, 5.0], [1.0, 
2.0, 3.0], [1.0, 5.0, 3.0]], [[4.0, 3.0, 4.0], [7.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[2.0, 5.0, 7.0], [5.0, 7.0, 5.0], [5.0, 2.0, 
3.0]]], [[[7.0, 7.0, 4.0], [4.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 3.0], [1.0, 1.0, 2.0], [8.0, 4.0, 5.0]], [[5.0, 5.0, 
5.0], [5.0, 1.0, 5.0], [2.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 5.0], [6.0, 6.0, 3.0], [1.0, 4.0, 3.0]], 
[[5.0, 3.0, 2.0], [7.0, 6.0, 8.0], [5.0, 1.0, 7.0]], [[6.0, 1.0, 6.0], [3.0, 3.0, 4.0], [4.0, 3.0, 8.0]]], [[[6.0, 5.0, 6.0], 
[8.0, 7.0, 3.0], [6.0, 5.0, 1.0]], [[3.0, 3.0, 3.0], [6.0, 8.0, 8.0], [4.0, 4.0, 4.0]], [[5.0, 8.0, 8.0], [4.0, 2.0, 8.0], 
[4.0, 5.0, 4.0]]], [[[3.0, 7.0, 1.0], [5.0, 6.0, 7.0], [4.0, 2.0, 7.0]], [[5.0, 4.0, 6.0], [4.0, 3.0, 1.0], [4.0, 5.0, 5.0]], 
[[3.0, 6.0, 4.0], [1.0, 6.0, 3.0], [4.0, 3.0, 6.0]]], [[[3.0, 6.0, 6.0], [5.0, 1.0, 3.0], [3.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], 
[3.0, 5.0, 2.0], [7.0, 1.0, 7.0]], [[2.0, 2.0, 5.0], [8.0, 8.0, 2.0], [5.0, 6.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 8.0, 5.0], [1.0, 1.0, 1.0]]], [[[7.0, 1.0, 4.0], [1.0, 6.0, 6.0]]], [[[7.0, 7.0, 6.0], 
[7.0, 6.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 3.0, 4.0], [7.0, 3.0, 2.0]]], [[[7.0, 2.0, 7.0], [5.0, 3.0, 4.0]]], 
[[[6.0, 6.0, 5.0], [7.0, 7.0, 3.0]]]])
      arg[:3,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 3.0], [6.0, 1.0, 2.0], [5.0, 8.0, 4.0]], [[8.0, 8.0, 5.0], [1.0, 1.0, 1.0], [1.0, 
4.0, 5.0]], [[8.0, 5.0, 4.0], [6.0, 6.0, 7.0], [6.0, 5.0, 8.0]]], [[[3.0, 1.0, 6.0], [6.0, 5.0, 7.0], [7.0, 8.0, 1.0]], [[7.0, 
1.0, 4.0], [1.0, 6.0, 6.0], [2.0, 1.0, 4.0]], [[6.0, 4.0, 6.0], [5.0, 1.0, 7.0], [7.0, 1.0, 8.0]]], [[[7.0, 5.0, 5.0], [1.0, 
2.0, 3.0], [1.0, 5.0, 3.0]], [[7.0, 7.0, 6.0], [7.0, 6.0, 5.0], [1.0, 4.0, 7.0]], [[2.0, 5.0, 7.0], [5.0, 7.0, 5.0], [5.0, 2.0, 
3.0]]], [[[7.0, 7.0, 4.0], [4.0, 4.0, 4.0], [6.0, 1.0, 5.0]], [[1.0, 5.0, 3.0], [1.0, 1.0, 2.0], [8.0, 4.0, 5.0]], [[5.0, 5.0, 
5.0], [5.0, 1.0, 5.0], [2.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 5.0], [6.0, 6.0, 3.0], [1.0, 4.0, 3.0]], 
[[2.0, 3.0, 4.0], [7.0, 3.0, 2.0], [5.0, 1.0, 7.0]], [[6.0, 1.0, 6.0], [3.0, 3.0, 4.0], [4.0, 3.0, 8.0]]], [[[6.0, 5.0, 6.0], 
[8.0, 7.0, 3.0], [6.0, 5.0, 1.0]], [[7.0, 2.0, 7.0], [5.0, 3.0, 4.0], [4.0, 4.0, 4.0]], [[5.0, 8.0, 8.0], [4.0, 2.0, 8.0], 
[4.0, 5.0, 4.0]]], [[[3.0, 7.0, 1.0], [5.0, 6.0, 7.0], [4.0, 2.0, 7.0]], [[6.0, 6.0, 5.0], [7.0, 7.0, 3.0], [4.0, 5.0, 5.0]], 
[[3.0, 6.0, 4.0], [1.0, 6.0, 3.0], [4.0, 3.0, 6.0]]], [[[3.0, 6.0, 6.0], [5.0, 1.0, 3.0], [3.0, 2.0, 1.0]], [[2.0, 5.0, 2.0], 
[3.0, 5.0, 2.0], [7.0, 1.0, 7.0]], [[2.0, 2.0, 5.0], [8.0, 8.0, 2.0], [5.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 3.0], [7.0, 4.0, 8.0], [1.0, 2.0, 5.0]], [[5.0, 2.0, 3.0], [6.0, 7.0, 3.0], [4.0, 
6.0, 5.0]], [[2.0, 7.0, 3.0], [7.0, 3.0, 4.0], [8.0, 7.0, 4.0]]], [[[5.0, 4.0, 6.0], [6.0, 1.0, 1.0], [3.0, 8.0, 1.0]], [[7.0, 
3.0, 7.0], [8.0, 7.0, 7.0], [7.0, 3.0, 6.0]], [[3.0, 3.0, 1.0], [3.0, 4.0, 2.0], [2.0, 4.0, 7.0]]], [[[6.0, 6.0, 5.0], [4.0, 
6.0, 3.0], [4.0, 6.0, 7.0]], [[1.0, 6.0, 7.0], [8.0, 3.0, 2.0], [4.0, 7.0, 5.0]], [[2.0, 7.0, 4.0], [7.0, 5.0, 7.0], [5.0, 8.0, 
6.0]]], [[[6.0, 3.0, 6.0], [4.0, 8.0, 2.0], [7.0, 4.0, 7.0]], [[8.0, 5.0, 3.0], [3.0, 8.0, 2.0], [3.0, 7.0, 1.0]], [[3.0, 2.0, 
2.0], [7.0, 1.0, 3.0], [5.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 5.0], [3.0, 1.0, 4.0], [2.0, 6.0, 5.0]], 
[[6.0, 2.0, 5.0], [5.0, 1.0, 5.0], [5.0, 4.0, 4.0]], [[6.0, 8.0, 5.0], [8.0, 1.0, 5.0], [3.0, 8.0, 3.0]]], [[[3.0, 2.0, 8.0], 
[3.0, 8.0, 2.0], [5.0, 8.0, 5.0]], [[4.0, 2.0, 5.0], [7.0, 3.0, 2.0], [7.0, 7.0, 4.0]], [[8.0, 3.0, 6.0], [5.0, 4.0, 6.0], 
[2.0, 8.0, 2.0]]], [[[7.0, 3.0, 3.0], [4.0, 3.0, 1.0], [2.0, 3.0, 8.0]], [[4.0, 5.0, 2.0], [7.0, 2.0, 7.0], [2.0, 2.0, 2.0]], 
[[3.0, 6.0, 8.0], [3.0, 2.0, 4.0], [7.0, 4.0, 3.0]]], [[[3.0, 4.0, 7.0], [6.0, 1.0, 7.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 3.0], 
[6.0, 7.0, 2.0], [8.0, 7.0, 7.0]], [[4.0, 2.0, 6.0], [5.0, 7.0, 8.0], [6.0, 2.0, 5.0]]]])
      val=numpy.array([[[[8.0, 8.0, 1.0], [1.0, 7.0, 5.0], [7.0, 3.0, 5.0]], [[1.0, 8.0, 7.0], [6.0, 4.0, 6.0], [4.0, 7.0, 
8.0]], [[6.0, 3.0, 7.0], [3.0, 3.0, 6.0], [3.0, 7.0, 8.0]]], [[[2.0, 1.0, 2.0], [2.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[3.0, 7.0, 
6.0], [2.0, 1.0, 7.0], [5.0, 7.0, 5.0]], [[7.0, 5.0, 2.0], [7.0, 4.0, 4.0], [8.0, 6.0, 2.0]]]])
      arg[1:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 3.0], [7.0, 4.0, 8.0], [1.0, 2.0, 5.0]], [[5.0, 2.0, 3.0], [6.0, 7.0, 3.0], [4.0, 
6.0, 5.0]], [[2.0, 7.0, 3.0], [7.0, 3.0, 4.0], [8.0, 7.0, 4.0]]], [[[8.0, 8.0, 1.0], [1.0, 7.0, 5.0], [7.0, 3.0, 5.0]], [[1.0, 
8.0, 7.0], [6.0, 4.0, 6.0], [4.0, 7.0, 8.0]], [[6.0, 3.0, 7.0], [3.0, 3.0, 6.0], [3.0, 7.0, 8.0]]], [[[2.0, 1.0, 2.0], [2.0, 
1.0, 7.0], [8.0, 8.0, 7.0]], [[3.0, 7.0, 6.0], [2.0, 1.0, 7.0], [5.0, 7.0, 5.0]], [[7.0, 5.0, 2.0], [7.0, 4.0, 4.0], [8.0, 6.0, 
2.0]]], [[[6.0, 3.0, 6.0], [4.0, 8.0, 2.0], [7.0, 4.0, 7.0]], [[8.0, 5.0, 3.0], [3.0, 8.0, 2.0], [3.0, 7.0, 1.0]], [[3.0, 2.0, 
2.0], [7.0, 1.0, 3.0], [5.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 5.0], [3.0, 1.0, 4.0], [2.0, 6.0, 5.0]], 
[[6.0, 2.0, 5.0], [5.0, 1.0, 5.0], [5.0, 4.0, 4.0]], [[6.0, 8.0, 5.0], [8.0, 1.0, 5.0], [3.0, 8.0, 3.0]]], [[[8.0, 8.0, 1.0], 
[1.0, 7.0, 5.0], [7.0, 3.0, 5.0]], [[1.0, 8.0, 7.0], [6.0, 4.0, 6.0], [4.0, 7.0, 8.0]], [[6.0, 3.0, 7.0], [3.0, 3.0, 6.0], 
[3.0, 7.0, 8.0]]], [[[2.0, 1.0, 2.0], [2.0, 1.0, 7.0], [8.0, 8.0, 7.0]], [[3.0, 7.0, 6.0], [2.0, 1.0, 7.0], [5.0, 7.0, 5.0]], 
[[7.0, 5.0, 2.0], [7.0, 4.0, 4.0], [8.0, 6.0, 2.0]]], [[[3.0, 4.0, 7.0], [6.0, 1.0, 7.0], [2.0, 2.0, 5.0]], [[2.0, 2.0, 3.0], 
[6.0, 7.0, 2.0], [8.0, 7.0, 7.0]], [[4.0, 2.0, 6.0], [5.0, 7.0, 8.0], [6.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 8.0], [7.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 6.0], [1.0, 
6.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 8.0, 3.0], [1.0, 6.0, 2.0]]], [[[6.0, 6.0, 1.0], [2.0, 4.0, 6.0], [5.0, 8.0, 6.0]], [[4.0, 
8.0, 5.0], [8.0, 8.0, 3.0], [4.0, 6.0, 7.0]], [[8.0, 2.0, 8.0], [1.0, 5.0, 1.0], [8.0, 3.0, 6.0]]], [[[2.0, 6.0, 4.0], [1.0, 
3.0, 8.0], [2.0, 3.0, 6.0]], [[2.0, 6.0, 1.0], [5.0, 7.0, 7.0], [5.0, 4.0, 8.0]], [[5.0, 5.0, 1.0], [7.0, 8.0, 4.0], [3.0, 5.0, 
2.0]]], [[[1.0, 7.0, 5.0], [7.0, 8.0, 5.0], [2.0, 2.0, 5.0]], [[4.0, 2.0, 3.0], [7.0, 3.0, 6.0], [2.0, 8.0, 6.0]], [[6.0, 1.0, 
3.0], [8.0, 1.0, 3.0], [4.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 4.0], [1.0, 2.0, 5.0], [4.0, 3.0, 7.0]], 
[[6.0, 8.0, 2.0], [3.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[5.0, 5.0, 5.0], [8.0, 8.0, 1.0], [1.0, 8.0, 3.0]]], [[[7.0, 1.0, 7.0], 
[2.0, 5.0, 7.0], [2.0, 1.0, 4.0]], [[4.0, 1.0, 4.0], [1.0, 3.0, 4.0], [4.0, 7.0, 6.0]], [[1.0, 3.0, 5.0], [4.0, 6.0, 7.0], 
[8.0, 4.0, 3.0]]], [[[1.0, 6.0, 4.0], [3.0, 7.0, 8.0], [2.0, 3.0, 1.0]], [[5.0, 7.0, 3.0], [3.0, 4.0, 2.0], [4.0, 6.0, 6.0]], 
[[8.0, 4.0, 6.0], [3.0, 7.0, 1.0], [7.0, 1.0, 8.0]]], [[[2.0, 6.0, 6.0], [1.0, 3.0, 6.0], [2.0, 4.0, 1.0]], [[2.0, 8.0, 6.0], 
[5.0, 1.0, 8.0], [6.0, 6.0, 3.0]], [[4.0, 8.0, 7.0], [2.0, 2.0, 5.0], [7.0, 8.0, 4.0]]]])
      val=Data(numpy.array([[[[8.0, 2.0, 2.0], [6.0, 2.0, 1.0], [2.0, 4.0, 5.0]], [[6.0, 8.0, 1.0], [8.0, 3.0, 4.0], [4.0, 5.0, 
7.0]], [[8.0, 4.0, 6.0], [5.0, 2.0, 1.0], [2.0, 3.0, 3.0]]], [[[4.0, 7.0, 8.0], [5.0, 6.0, 2.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 
6.0], [1.0, 1.0, 5.0], [8.0, 2.0, 8.0]], [[6.0, 4.0, 4.0], [3.0, 3.0, 4.0], [5.0, 7.0, 6.0]]]]),self.functionspace)
      arg[1:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 8.0], [7.0, 7.0, 1.0], [8.0, 5.0, 2.0]], [[6.0, 2.0, 4.0], [8.0, 8.0, 6.0], [1.0, 
6.0, 5.0]], [[8.0, 5.0, 8.0], [8.0, 8.0, 3.0], [1.0, 6.0, 2.0]]], [[[8.0, 2.0, 2.0], [6.0, 2.0, 1.0], [2.0, 4.0, 5.0]], [[6.0, 
8.0, 1.0], [8.0, 3.0, 4.0], [4.0, 5.0, 7.0]], [[8.0, 4.0, 6.0], [5.0, 2.0, 1.0], [2.0, 3.0, 3.0]]], [[[4.0, 7.0, 8.0], [5.0, 
6.0, 2.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 5.0], [8.0, 2.0, 8.0]], [[6.0, 4.0, 4.0], [3.0, 3.0, 4.0], [5.0, 7.0, 
6.0]]], [[[1.0, 7.0, 5.0], [7.0, 8.0, 5.0], [2.0, 2.0, 5.0]], [[4.0, 2.0, 3.0], [7.0, 3.0, 6.0], [2.0, 8.0, 6.0]], [[6.0, 1.0, 
3.0], [8.0, 1.0, 3.0], [4.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [1.0, 2.0, 5.0], [4.0, 3.0, 7.0]], 
[[6.0, 8.0, 2.0], [3.0, 3.0, 8.0], [1.0, 8.0, 7.0]], [[5.0, 5.0, 5.0], [8.0, 8.0, 1.0], [1.0, 8.0, 3.0]]], [[[8.0, 2.0, 2.0], 
[6.0, 2.0, 1.0], [2.0, 4.0, 5.0]], [[6.0, 8.0, 1.0], [8.0, 3.0, 4.0], [4.0, 5.0, 7.0]], [[8.0, 4.0, 6.0], [5.0, 2.0, 1.0], 
[2.0, 3.0, 3.0]]], [[[4.0, 7.0, 8.0], [5.0, 6.0, 2.0], [2.0, 6.0, 4.0]], [[5.0, 2.0, 6.0], [1.0, 1.0, 5.0], [8.0, 2.0, 8.0]], 
[[6.0, 4.0, 4.0], [3.0, 3.0, 4.0], [5.0, 7.0, 6.0]]], [[[2.0, 6.0, 6.0], [1.0, 3.0, 6.0], [2.0, 4.0, 1.0]], [[2.0, 8.0, 6.0], 
[5.0, 1.0, 8.0], [6.0, 6.0, 3.0]], [[4.0, 8.0, 7.0], [2.0, 2.0, 5.0], [7.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 5.0], [3.0, 3.0, 8.0], [2.0, 2.0, 2.0]], [[4.0, 1.0, 2.0], [7.0, 1.0, 7.0], [4.0, 
6.0, 4.0]], [[7.0, 5.0, 7.0], [4.0, 4.0, 2.0], [7.0, 4.0, 2.0]]], [[[1.0, 4.0, 4.0], [6.0, 6.0, 6.0], [4.0, 5.0, 8.0]], [[7.0, 
8.0, 2.0], [6.0, 8.0, 4.0], [2.0, 5.0, 1.0]], [[5.0, 7.0, 6.0], [1.0, 5.0, 7.0], [8.0, 1.0, 8.0]]], [[[8.0, 4.0, 3.0], [1.0, 
8.0, 8.0], [2.0, 5.0, 4.0]], [[8.0, 5.0, 3.0], [4.0, 4.0, 6.0], [8.0, 3.0, 3.0]], [[2.0, 7.0, 3.0], [6.0, 5.0, 1.0], [8.0, 5.0, 
1.0]]], [[[7.0, 7.0, 4.0], [3.0, 4.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 4.0, 1.0], [6.0, 7.0, 6.0]], [[8.0, 1.0, 
2.0], [6.0, 4.0, 2.0], [3.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 4.0], [4.0, 6.0, 7.0], [7.0, 3.0, 6.0]], 
[[7.0, 7.0, 8.0], [5.0, 7.0, 8.0], [1.0, 2.0, 2.0]], [[8.0, 6.0, 5.0], [8.0, 4.0, 1.0], [6.0, 2.0, 7.0]]], [[[8.0, 6.0, 1.0], 
[7.0, 5.0, 6.0], [8.0, 2.0, 6.0]], [[3.0, 1.0, 5.0], [5.0, 7.0, 8.0], [5.0, 7.0, 1.0]], [[8.0, 4.0, 4.0], [3.0, 8.0, 6.0], 
[6.0, 6.0, 6.0]]], [[[7.0, 4.0, 2.0], [6.0, 1.0, 8.0], [4.0, 1.0, 5.0]], [[6.0, 7.0, 7.0], [8.0, 5.0, 7.0], [8.0, 2.0, 5.0]], 
[[8.0, 4.0, 8.0], [8.0, 8.0, 4.0], [8.0, 7.0, 3.0]]], [[[1.0, 7.0, 1.0], [6.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[2.0, 4.0, 3.0], 
[4.0, 2.0, 7.0], [8.0, 3.0, 2.0]], [[4.0, 7.0, 2.0], [3.0, 1.0, 3.0], [4.0, 2.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 1.0, 5.0], [7.0, 3.0, 1.0], [2.0, 2.0, 1.0]], [[8.0, 3.0, 8.0], [1.0, 1.0, 8.0], [4.0, 
7.0, 3.0]], [[7.0, 6.0, 5.0], [5.0, 1.0, 3.0], [8.0, 7.0, 8.0]]], [[[2.0, 2.0, 6.0], [3.0, 4.0, 2.0], [1.0, 1.0, 2.0]], [[1.0, 
2.0, 3.0], [3.0, 2.0, 2.0], [2.0, 7.0, 7.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 6.0], [1.0, 4.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 7.0, 8.0], [3.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 7.0, 5.0], [6.0, 3.0, 8.0], 
[6.0, 2.0, 2.0]], [[6.0, 2.0, 2.0], [6.0, 6.0, 6.0], [7.0, 1.0, 3.0]]], [[[8.0, 6.0, 7.0], [6.0, 8.0, 7.0], [5.0, 2.0, 1.0]], 
[[5.0, 5.0, 7.0], [2.0, 2.0, 4.0], [5.0, 4.0, 8.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 6.0], [4.0, 5.0, 8.0]]]])
      arg[1:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [3.0, 3.0, 8.0], [2.0, 2.0, 2.0]], [[4.0, 1.0, 2.0], [7.0, 1.0, 7.0], [4.0, 
6.0, 4.0]], [[7.0, 5.0, 7.0], [4.0, 4.0, 2.0], [7.0, 4.0, 2.0]]], [[[1.0, 1.0, 5.0], [7.0, 3.0, 1.0], [2.0, 2.0, 1.0]], [[8.0, 
3.0, 8.0], [1.0, 1.0, 8.0], [4.0, 7.0, 3.0]], [[7.0, 6.0, 5.0], [5.0, 1.0, 3.0], [8.0, 7.0, 8.0]]], [[[2.0, 2.0, 6.0], [3.0, 
4.0, 2.0], [1.0, 1.0, 2.0]], [[1.0, 2.0, 3.0], [3.0, 2.0, 2.0], [2.0, 7.0, 7.0]], [[5.0, 4.0, 6.0], [8.0, 6.0, 6.0], [1.0, 4.0, 
4.0]]], [[[7.0, 7.0, 4.0], [3.0, 4.0, 2.0], [8.0, 8.0, 2.0]], [[4.0, 3.0, 8.0], [4.0, 4.0, 1.0], [6.0, 7.0, 6.0]], [[8.0, 1.0, 
2.0], [6.0, 4.0, 2.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 4.0], [4.0, 6.0, 7.0], [7.0, 3.0, 6.0]], 
[[7.0, 7.0, 8.0], [5.0, 7.0, 8.0], [1.0, 2.0, 2.0]], [[8.0, 6.0, 5.0], [8.0, 4.0, 1.0], [6.0, 2.0, 7.0]]], [[[2.0, 7.0, 8.0], 
[3.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[7.0, 7.0, 5.0], [6.0, 3.0, 8.0], [6.0, 2.0, 2.0]], [[6.0, 2.0, 2.0], [6.0, 6.0, 6.0], 
[7.0, 1.0, 3.0]]], [[[8.0, 6.0, 7.0], [6.0, 8.0, 7.0], [5.0, 2.0, 1.0]], [[5.0, 5.0, 7.0], [2.0, 2.0, 4.0], [5.0, 4.0, 8.0]], 
[[6.0, 3.0, 7.0], [4.0, 8.0, 6.0], [4.0, 5.0, 8.0]]], [[[1.0, 7.0, 1.0], [6.0, 7.0, 2.0], [3.0, 7.0, 7.0]], [[2.0, 4.0, 3.0], 
[4.0, 2.0, 7.0], [8.0, 3.0, 2.0]], [[4.0, 7.0, 2.0], [3.0, 1.0, 3.0], [4.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 5.0], [6.0, 4.0, 3.0], [2.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 4.0], [2.0, 
2.0, 7.0]], [[4.0, 5.0, 8.0], [8.0, 8.0, 8.0], [8.0, 2.0, 6.0]]], [[[6.0, 3.0, 8.0], [2.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 
2.0, 6.0], [1.0, 8.0, 4.0], [6.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [3.0, 8.0, 3.0], [6.0, 3.0, 3.0]]], [[[6.0, 3.0, 3.0], [5.0, 
6.0, 4.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 7.0], [8.0, 8.0, 8.0], [8.0, 2.0, 2.0]], [[4.0, 8.0, 3.0], [4.0, 4.0, 1.0], [2.0, 3.0, 
5.0]]], [[[8.0, 2.0, 8.0], [2.0, 2.0, 1.0], [1.0, 1.0, 6.0]], [[5.0, 5.0, 1.0], [3.0, 8.0, 5.0], [2.0, 5.0, 5.0]], [[1.0, 6.0, 
4.0], [2.0, 8.0, 8.0], [5.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 2.0], [4.0, 5.0, 4.0], [3.0, 4.0, 3.0]], 
[[8.0, 4.0, 8.0], [4.0, 5.0, 7.0], [2.0, 7.0, 6.0]], [[5.0, 2.0, 4.0], [5.0, 8.0, 2.0], [7.0, 4.0, 7.0]]], [[[1.0, 4.0, 3.0], 
[3.0, 5.0, 8.0], [6.0, 4.0, 2.0]], [[6.0, 3.0, 6.0], [8.0, 7.0, 7.0], [8.0, 7.0, 8.0]], [[4.0, 1.0, 6.0], [3.0, 8.0, 6.0], 
[6.0, 7.0, 5.0]]], [[[5.0, 8.0, 8.0], [2.0, 2.0, 6.0], [2.0, 7.0, 5.0]], [[5.0, 8.0, 7.0], [5.0, 4.0, 7.0], [4.0, 3.0, 1.0]], 
[[2.0, 3.0, 8.0], [2.0, 2.0, 8.0], [4.0, 2.0, 7.0]]], [[[7.0, 6.0, 3.0], [3.0, 6.0, 3.0], [8.0, 6.0, 7.0]], [[5.0, 2.0, 6.0], 
[6.0, 1.0, 3.0], [5.0, 5.0, 5.0]], [[3.0, 1.0, 5.0], [5.0, 1.0, 4.0], [5.0, 7.0, 2.0]]]])
      val=numpy.array([[[6.0, 6.0, 5.0]], [[7.0, 8.0, 3.0]]])
      arg[2:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 5.0], [6.0, 4.0, 3.0], [2.0, 6.0, 1.0]], [[4.0, 3.0, 7.0], [1.0, 8.0, 4.0], [2.0, 
2.0, 7.0]], [[4.0, 5.0, 8.0], [8.0, 8.0, 8.0], [8.0, 2.0, 6.0]]], [[[6.0, 3.0, 8.0], [2.0, 1.0, 6.0], [3.0, 8.0, 3.0]], [[3.0, 
2.0, 6.0], [1.0, 8.0, 4.0], [6.0, 3.0, 5.0]], [[4.0, 4.0, 4.0], [3.0, 8.0, 3.0], [6.0, 3.0, 3.0]]], [[[6.0, 3.0, 3.0], [5.0, 
6.0, 4.0], [7.0, 7.0, 2.0]], [[8.0, 1.0, 7.0], [6.0, 6.0, 5.0], [8.0, 2.0, 2.0]], [[4.0, 8.0, 3.0], [4.0, 4.0, 1.0], [2.0, 3.0, 
5.0]]], [[[8.0, 2.0, 8.0], [2.0, 2.0, 1.0], [1.0, 1.0, 6.0]], [[5.0, 5.0, 1.0], [7.0, 8.0, 3.0], [2.0, 5.0, 5.0]], [[1.0, 6.0, 
4.0], [2.0, 8.0, 8.0], [5.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 2.0], [4.0, 5.0, 4.0], [3.0, 4.0, 3.0]], 
[[8.0, 4.0, 8.0], [4.0, 5.0, 7.0], [2.0, 7.0, 6.0]], [[5.0, 2.0, 4.0], [5.0, 8.0, 2.0], [7.0, 4.0, 7.0]]], [[[1.0, 4.0, 3.0], 
[3.0, 5.0, 8.0], [6.0, 4.0, 2.0]], [[6.0, 3.0, 6.0], [8.0, 7.0, 7.0], [8.0, 7.0, 8.0]], [[4.0, 1.0, 6.0], [3.0, 8.0, 6.0], 
[6.0, 7.0, 5.0]]], [[[5.0, 8.0, 8.0], [2.0, 2.0, 6.0], [2.0, 7.0, 5.0]], [[5.0, 8.0, 7.0], [6.0, 6.0, 5.0], [4.0, 3.0, 1.0]], 
[[2.0, 3.0, 8.0], [2.0, 2.0, 8.0], [4.0, 2.0, 7.0]]], [[[7.0, 6.0, 3.0], [3.0, 6.0, 3.0], [8.0, 6.0, 7.0]], [[5.0, 2.0, 6.0], 
[7.0, 8.0, 3.0], [5.0, 5.0, 5.0]], [[3.0, 1.0, 5.0], [5.0, 1.0, 4.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 6.0], [1.0, 2.0, 7.0], [1.0, 4.0, 6.0]], [[1.0, 7.0, 6.0], [6.0, 6.0, 8.0], [3.0, 
7.0, 7.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 4.0], [7.0, 7.0, 5.0]]], [[[4.0, 3.0, 7.0], [2.0, 6.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 
2.0, 2.0], [4.0, 2.0, 1.0], [2.0, 8.0, 8.0]], [[5.0, 6.0, 7.0], [7.0, 3.0, 2.0], [7.0, 7.0, 8.0]]], [[[5.0, 6.0, 5.0], [1.0, 
1.0, 6.0], [5.0, 3.0, 2.0]], [[7.0, 7.0, 4.0], [4.0, 8.0, 7.0], [3.0, 6.0, 8.0]], [[3.0, 1.0, 2.0], [2.0, 5.0, 7.0], [3.0, 5.0, 
8.0]]], [[[2.0, 3.0, 8.0], [6.0, 7.0, 7.0], [2.0, 7.0, 4.0]], [[5.0, 5.0, 7.0], [7.0, 4.0, 1.0], [1.0, 2.0, 6.0]], [[1.0, 4.0, 
4.0], [3.0, 8.0, 4.0], [1.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 7.0], [5.0, 5.0, 4.0], [3.0, 5.0, 2.0]], 
[[7.0, 2.0, 4.0], [8.0, 7.0, 3.0], [4.0, 6.0, 8.0]], [[8.0, 3.0, 3.0], [2.0, 7.0, 7.0], [4.0, 6.0, 1.0]]], [[[6.0, 1.0, 5.0], 
[6.0, 8.0, 2.0], [7.0, 1.0, 7.0]], [[8.0, 1.0, 1.0], [8.0, 7.0, 7.0], [5.0, 3.0, 6.0]], [[4.0, 1.0, 4.0], [5.0, 6.0, 8.0], 
[3.0, 6.0, 3.0]]], [[[2.0, 4.0, 8.0], [8.0, 1.0, 8.0], [2.0, 7.0, 6.0]], [[4.0, 4.0, 6.0], [8.0, 4.0, 2.0], [2.0, 1.0, 4.0]], 
[[1.0, 1.0, 8.0], [5.0, 5.0, 6.0], [4.0, 4.0, 7.0]]], [[[1.0, 8.0, 1.0], [5.0, 3.0, 4.0], [4.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], 
[5.0, 7.0, 2.0], [1.0, 6.0, 1.0]], [[3.0, 2.0, 3.0], [6.0, 3.0, 1.0], [7.0, 4.0, 1.0]]]])
      val=Data(numpy.array([[[1.0, 1.0, 4.0]], [[6.0, 6.0, 6.0]]]),self.functionspace)
      arg[2:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 6.0], [1.0, 2.0, 7.0], [1.0, 4.0, 6.0]], [[1.0, 7.0, 6.0], [6.0, 6.0, 8.0], [3.0, 
7.0, 7.0]], [[8.0, 1.0, 7.0], [1.0, 6.0, 4.0], [7.0, 7.0, 5.0]]], [[[4.0, 3.0, 7.0], [2.0, 6.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 
2.0, 2.0], [4.0, 2.0, 1.0], [2.0, 8.0, 8.0]], [[5.0, 6.0, 7.0], [7.0, 3.0, 2.0], [7.0, 7.0, 8.0]]], [[[5.0, 6.0, 5.0], [1.0, 
1.0, 6.0], [5.0, 3.0, 2.0]], [[7.0, 7.0, 4.0], [1.0, 1.0, 4.0], [3.0, 6.0, 8.0]], [[3.0, 1.0, 2.0], [2.0, 5.0, 7.0], [3.0, 5.0, 
8.0]]], [[[2.0, 3.0, 8.0], [6.0, 7.0, 7.0], [2.0, 7.0, 4.0]], [[5.0, 5.0, 7.0], [6.0, 6.0, 6.0], [1.0, 2.0, 6.0]], [[1.0, 4.0, 
4.0], [3.0, 8.0, 4.0], [1.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 7.0], [5.0, 5.0, 4.0], [3.0, 5.0, 2.0]], 
[[7.0, 2.0, 4.0], [8.0, 7.0, 3.0], [4.0, 6.0, 8.0]], [[8.0, 3.0, 3.0], [2.0, 7.0, 7.0], [4.0, 6.0, 1.0]]], [[[6.0, 1.0, 5.0], 
[6.0, 8.0, 2.0], [7.0, 1.0, 7.0]], [[8.0, 1.0, 1.0], [8.0, 7.0, 7.0], [5.0, 3.0, 6.0]], [[4.0, 1.0, 4.0], [5.0, 6.0, 8.0], 
[3.0, 6.0, 3.0]]], [[[2.0, 4.0, 8.0], [8.0, 1.0, 8.0], [2.0, 7.0, 6.0]], [[4.0, 4.0, 6.0], [1.0, 1.0, 4.0], [2.0, 1.0, 4.0]], 
[[1.0, 1.0, 8.0], [5.0, 5.0, 6.0], [4.0, 4.0, 7.0]]], [[[1.0, 8.0, 1.0], [5.0, 3.0, 4.0], [4.0, 2.0, 1.0]], [[1.0, 8.0, 8.0], 
[6.0, 6.0, 6.0], [1.0, 6.0, 1.0]], [[3.0, 2.0, 3.0], [6.0, 3.0, 1.0], [7.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 2.0], [6.0, 5.0, 3.0], [5.0, 1.0, 2.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 8.0], [3.0, 
2.0, 7.0]], [[3.0, 3.0, 1.0], [6.0, 5.0, 7.0], [7.0, 1.0, 5.0]]], [[[2.0, 1.0, 2.0], [7.0, 5.0, 6.0], [8.0, 5.0, 4.0]], [[5.0, 
7.0, 8.0], [1.0, 7.0, 8.0], [2.0, 3.0, 3.0]], [[8.0, 7.0, 8.0], [6.0, 2.0, 3.0], [6.0, 6.0, 8.0]]], [[[6.0, 1.0, 5.0], [5.0, 
7.0, 3.0], [4.0, 4.0, 3.0]], [[3.0, 3.0, 7.0], [3.0, 3.0, 1.0], [3.0, 5.0, 8.0]], [[3.0, 1.0, 6.0], [5.0, 1.0, 4.0], [3.0, 3.0, 
8.0]]], [[[2.0, 5.0, 4.0], [5.0, 6.0, 7.0], [1.0, 3.0, 3.0]], [[4.0, 7.0, 5.0], [7.0, 7.0, 5.0], [3.0, 8.0, 4.0]], [[2.0, 3.0, 
2.0], [7.0, 6.0, 8.0], [7.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 2.0, 7.0], [6.0, 7.0, 7.0], [4.0, 4.0, 3.0]], 
[[3.0, 8.0, 7.0], [2.0, 3.0, 4.0], [3.0, 8.0, 4.0]], [[4.0, 3.0, 2.0], [6.0, 7.0, 6.0], [1.0, 5.0, 4.0]]], [[[4.0, 1.0, 5.0], 
[1.0, 5.0, 3.0], [6.0, 8.0, 4.0]], [[6.0, 6.0, 2.0], [2.0, 6.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 7.0, 1.0], 
[3.0, 1.0, 8.0]]], [[[5.0, 2.0, 8.0], [2.0, 8.0, 6.0], [2.0, 4.0, 7.0]], [[5.0, 4.0, 6.0], [7.0, 8.0, 8.0], [4.0, 1.0, 1.0]], 
[[5.0, 6.0, 6.0], [3.0, 8.0, 7.0], [5.0, 2.0, 5.0]]], [[[7.0, 5.0, 6.0], [6.0, 4.0, 1.0], [6.0, 1.0, 7.0]], [[8.0, 2.0, 3.0], 
[5.0, 4.0, 7.0], [1.0, 3.0, 5.0]], [[4.0, 3.0, 1.0], [5.0, 2.0, 6.0], [5.0, 4.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 3.0, 8.0]], [[1.0, 2.0, 8.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 1.0]], [[6.0, 6.0, 
1.0]]])
      arg[2:,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 2.0], [6.0, 5.0, 3.0], [5.0, 1.0, 2.0]], [[7.0, 4.0, 3.0], [4.0, 1.0, 8.0], [3.0, 
2.0, 7.0]], [[3.0, 3.0, 1.0], [6.0, 5.0, 7.0], [7.0, 1.0, 5.0]]], [[[2.0, 1.0, 2.0], [7.0, 5.0, 6.0], [8.0, 5.0, 4.0]], [[5.0, 
7.0, 8.0], [1.0, 7.0, 8.0], [2.0, 3.0, 3.0]], [[8.0, 7.0, 8.0], [6.0, 2.0, 3.0], [6.0, 6.0, 8.0]]], [[[6.0, 1.0, 5.0], [5.0, 
7.0, 3.0], [4.0, 4.0, 3.0]], [[3.0, 3.0, 7.0], [2.0, 3.0, 8.0], [3.0, 5.0, 8.0]], [[3.0, 1.0, 6.0], [5.0, 1.0, 4.0], [3.0, 3.0, 
8.0]]], [[[2.0, 5.0, 4.0], [5.0, 6.0, 7.0], [1.0, 3.0, 3.0]], [[4.0, 7.0, 5.0], [1.0, 2.0, 8.0], [3.0, 8.0, 4.0]], [[2.0, 3.0, 
2.0], [7.0, 6.0, 8.0], [7.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 2.0, 7.0], [6.0, 7.0, 7.0], [4.0, 4.0, 3.0]], 
[[3.0, 8.0, 7.0], [2.0, 3.0, 4.0], [3.0, 8.0, 4.0]], [[4.0, 3.0, 2.0], [6.0, 7.0, 6.0], [1.0, 5.0, 4.0]]], [[[4.0, 1.0, 5.0], 
[1.0, 5.0, 3.0], [6.0, 8.0, 4.0]], [[6.0, 6.0, 2.0], [2.0, 6.0, 3.0], [2.0, 4.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 7.0, 1.0], 
[3.0, 1.0, 8.0]]], [[[5.0, 2.0, 8.0], [2.0, 8.0, 6.0], [2.0, 4.0, 7.0]], [[5.0, 4.0, 6.0], [4.0, 4.0, 1.0], [4.0, 1.0, 1.0]], 
[[5.0, 6.0, 6.0], [3.0, 8.0, 7.0], [5.0, 2.0, 5.0]]], [[[7.0, 5.0, 6.0], [6.0, 4.0, 1.0], [6.0, 1.0, 7.0]], [[8.0, 2.0, 3.0], 
[6.0, 6.0, 1.0], [1.0, 3.0, 5.0]], [[4.0, 3.0, 1.0], [5.0, 2.0, 6.0], [5.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 3.0], [4.0, 1.0, 8.0], [3.0, 8.0, 6.0]], [[6.0, 6.0, 4.0], [4.0, 4.0, 1.0], [3.0, 
5.0, 8.0]], [[5.0, 1.0, 1.0], [3.0, 7.0, 8.0], [3.0, 4.0, 8.0]]], [[[4.0, 1.0, 5.0], [1.0, 2.0, 3.0], [5.0, 2.0, 5.0]], [[4.0, 
3.0, 3.0], [7.0, 7.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], [7.0, 2.0, 4.0], [2.0, 2.0, 6.0]]], [[[5.0, 5.0, 6.0], [8.0, 
2.0, 6.0], [6.0, 2.0, 8.0]], [[6.0, 3.0, 1.0], [5.0, 6.0, 6.0], [8.0, 5.0, 7.0]], [[5.0, 8.0, 3.0], [7.0, 2.0, 1.0], [6.0, 5.0, 
8.0]]], [[[8.0, 8.0, 4.0], [2.0, 3.0, 7.0], [8.0, 8.0, 5.0]], [[8.0, 3.0, 8.0], [6.0, 4.0, 2.0], [5.0, 8.0, 1.0]], [[7.0, 5.0, 
8.0], [7.0, 1.0, 2.0], [6.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 4.0], [7.0, 5.0, 5.0], [8.0, 5.0, 8.0]], 
[[1.0, 1.0, 3.0], [6.0, 4.0, 1.0], [3.0, 4.0, 3.0]], [[4.0, 5.0, 7.0], [2.0, 2.0, 8.0], [7.0, 1.0, 4.0]]], [[[3.0, 4.0, 7.0], 
[8.0, 4.0, 1.0], [8.0, 7.0, 8.0]], [[7.0, 8.0, 3.0], [4.0, 4.0, 1.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 5.0], [8.0, 1.0, 5.0], 
[5.0, 6.0, 2.0]]], [[[7.0, 3.0, 5.0], [4.0, 5.0, 4.0], [4.0, 5.0, 3.0]], [[2.0, 8.0, 7.0], [6.0, 1.0, 5.0], [7.0, 7.0, 8.0]], 
[[3.0, 2.0, 6.0], [6.0, 8.0, 8.0], [5.0, 3.0, 3.0]]], [[[1.0, 7.0, 8.0], [2.0, 5.0, 4.0], [3.0, 7.0, 1.0]], [[4.0, 8.0, 6.0], 
[4.0, 5.0, 2.0], [2.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [6.0, 3.0, 6.0], [7.0, 6.0, 6.0]]]])
      val=numpy.array([[[[6.0]]], [[[8.0]]]])
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 3.0], [4.0, 1.0, 8.0], [3.0, 8.0, 6.0]], [[6.0, 6.0, 4.0], [4.0, 4.0, 1.0], [3.0, 
5.0, 8.0]], [[5.0, 1.0, 1.0], [3.0, 7.0, 8.0], [3.0, 4.0, 8.0]]], [[[4.0, 1.0, 5.0], [1.0, 2.0, 3.0], [5.0, 2.0, 5.0]], [[4.0, 
3.0, 3.0], [7.0, 6.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], [7.0, 2.0, 4.0], [2.0, 2.0, 6.0]]], [[[5.0, 5.0, 6.0], [8.0, 
2.0, 6.0], [6.0, 2.0, 8.0]], [[6.0, 3.0, 1.0], [5.0, 8.0, 6.0], [8.0, 5.0, 7.0]], [[5.0, 8.0, 3.0], [7.0, 2.0, 1.0], [6.0, 5.0, 
8.0]]], [[[8.0, 8.0, 4.0], [2.0, 3.0, 7.0], [8.0, 8.0, 5.0]], [[8.0, 3.0, 8.0], [6.0, 4.0, 2.0], [5.0, 8.0, 1.0]], [[7.0, 5.0, 
8.0], [7.0, 1.0, 2.0], [6.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 4.0], [7.0, 5.0, 5.0], [8.0, 5.0, 8.0]], 
[[1.0, 1.0, 3.0], [6.0, 4.0, 1.0], [3.0, 4.0, 3.0]], [[4.0, 5.0, 7.0], [2.0, 2.0, 8.0], [7.0, 1.0, 4.0]]], [[[3.0, 4.0, 7.0], 
[8.0, 4.0, 1.0], [8.0, 7.0, 8.0]], [[7.0, 8.0, 3.0], [4.0, 6.0, 1.0], [7.0, 2.0, 4.0]], [[1.0, 1.0, 5.0], [8.0, 1.0, 5.0], 
[5.0, 6.0, 2.0]]], [[[7.0, 3.0, 5.0], [4.0, 5.0, 4.0], [4.0, 5.0, 3.0]], [[2.0, 8.0, 7.0], [6.0, 8.0, 5.0], [7.0, 7.0, 8.0]], 
[[3.0, 2.0, 6.0], [6.0, 8.0, 8.0], [5.0, 3.0, 3.0]]], [[[1.0, 7.0, 8.0], [2.0, 5.0, 4.0], [3.0, 7.0, 1.0]], [[4.0, 8.0, 6.0], 
[4.0, 5.0, 2.0], [2.0, 8.0, 8.0]], [[2.0, 5.0, 2.0], [6.0, 3.0, 6.0], [7.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 5.0], [4.0, 8.0, 1.0], [5.0, 6.0, 1.0]], [[7.0, 6.0, 1.0], [6.0, 3.0, 5.0], [4.0, 
8.0, 3.0]], [[2.0, 8.0, 2.0], [6.0, 5.0, 5.0], [8.0, 7.0, 4.0]]], [[[7.0, 3.0, 6.0], [6.0, 1.0, 6.0], [7.0, 4.0, 6.0]], [[1.0, 
7.0, 1.0], [3.0, 2.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 4.0, 2.0], [7.0, 8.0, 8.0]]], [[[4.0, 5.0, 1.0], [1.0, 
7.0, 2.0], [1.0, 7.0, 8.0]], [[1.0, 6.0, 1.0], [1.0, 5.0, 3.0], [3.0, 5.0, 3.0]], [[5.0, 7.0, 4.0], [4.0, 7.0, 3.0], [7.0, 2.0, 
7.0]]], [[[7.0, 6.0, 3.0], [1.0, 7.0, 3.0], [3.0, 1.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 7.0, 7.0], [2.0, 1.0, 8.0]], [[3.0, 2.0, 
3.0], [5.0, 3.0, 5.0], [1.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 8.0], [7.0, 3.0, 3.0], [8.0, 3.0, 4.0]], 
[[1.0, 7.0, 4.0], [6.0, 7.0, 2.0], [7.0, 6.0, 4.0]], [[1.0, 4.0, 3.0], [7.0, 1.0, 1.0], [2.0, 2.0, 7.0]]], [[[6.0, 7.0, 4.0], 
[1.0, 5.0, 1.0], [6.0, 8.0, 6.0]], [[4.0, 8.0, 3.0], [1.0, 5.0, 3.0], [1.0, 2.0, 7.0]], [[2.0, 1.0, 5.0], [7.0, 6.0, 7.0], 
[7.0, 2.0, 2.0]]], [[[3.0, 7.0, 8.0], [4.0, 5.0, 8.0], [5.0, 2.0, 8.0]], [[6.0, 4.0, 4.0], [7.0, 2.0, 5.0], [7.0, 4.0, 3.0]], 
[[8.0, 8.0, 3.0], [4.0, 8.0, 7.0], [5.0, 4.0, 5.0]]], [[[7.0, 2.0, 2.0], [2.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[3.0, 3.0, 2.0], 
[6.0, 4.0, 5.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 6.0], [2.0, 4.0, 6.0], [1.0, 2.0, 5.0]]]])
      val=Data(numpy.array([[[[6.0]]], [[[3.0]]]]),self.functionspace)
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 5.0], [4.0, 8.0, 1.0], [5.0, 6.0, 1.0]], [[7.0, 6.0, 1.0], [6.0, 3.0, 5.0], [4.0, 
8.0, 3.0]], [[2.0, 8.0, 2.0], [6.0, 5.0, 5.0], [8.0, 7.0, 4.0]]], [[[7.0, 3.0, 6.0], [6.0, 1.0, 6.0], [7.0, 4.0, 6.0]], [[1.0, 
7.0, 1.0], [3.0, 6.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 2.0, 7.0], [4.0, 4.0, 2.0], [7.0, 8.0, 8.0]]], [[[4.0, 5.0, 1.0], [1.0, 
7.0, 2.0], [1.0, 7.0, 8.0]], [[1.0, 6.0, 1.0], [1.0, 3.0, 3.0], [3.0, 5.0, 3.0]], [[5.0, 7.0, 4.0], [4.0, 7.0, 3.0], [7.0, 2.0, 
7.0]]], [[[7.0, 6.0, 3.0], [1.0, 7.0, 3.0], [3.0, 1.0, 8.0]], [[5.0, 8.0, 2.0], [7.0, 7.0, 7.0], [2.0, 1.0, 8.0]], [[3.0, 2.0, 
3.0], [5.0, 3.0, 5.0], [1.0, 1.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 8.0], [7.0, 3.0, 3.0], [8.0, 3.0, 4.0]], 
[[1.0, 7.0, 4.0], [6.0, 7.0, 2.0], [7.0, 6.0, 4.0]], [[1.0, 4.0, 3.0], [7.0, 1.0, 1.0], [2.0, 2.0, 7.0]]], [[[6.0, 7.0, 4.0], 
[1.0, 5.0, 1.0], [6.0, 8.0, 6.0]], [[4.0, 8.0, 3.0], [1.0, 6.0, 3.0], [1.0, 2.0, 7.0]], [[2.0, 1.0, 5.0], [7.0, 6.0, 7.0], 
[7.0, 2.0, 2.0]]], [[[3.0, 7.0, 8.0], [4.0, 5.0, 8.0], [5.0, 2.0, 8.0]], [[6.0, 4.0, 4.0], [7.0, 3.0, 5.0], [7.0, 4.0, 3.0]], 
[[8.0, 8.0, 3.0], [4.0, 8.0, 7.0], [5.0, 4.0, 5.0]]], [[[7.0, 2.0, 2.0], [2.0, 1.0, 7.0], [7.0, 7.0, 3.0]], [[3.0, 3.0, 2.0], 
[6.0, 4.0, 5.0], [5.0, 2.0, 3.0]], [[3.0, 5.0, 6.0], [2.0, 4.0, 6.0], [1.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RRRR(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 1.0], [2.0, 4.0, 3.0], [2.0, 7.0, 2.0]], [[2.0, 4.0, 6.0], [8.0, 3.0, 1.0], [3.0, 
4.0, 8.0]], [[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [6.0, 7.0, 4.0]]], [[[5.0, 4.0, 3.0], [1.0, 7.0, 3.0], [5.0, 7.0, 2.0]], [[7.0, 
1.0, 5.0], [8.0, 3.0, 8.0], [8.0, 3.0, 7.0]], [[5.0, 5.0, 7.0], [4.0, 5.0, 3.0], [6.0, 5.0, 4.0]]], [[[4.0, 7.0, 5.0], [6.0, 
7.0, 1.0], [7.0, 4.0, 3.0]], [[5.0, 2.0, 2.0], [6.0, 3.0, 3.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 6.0], [2.0, 7.0, 1.0], [3.0, 7.0, 
2.0]]], [[[1.0, 3.0, 1.0], [3.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[4.0, 2.0, 7.0], [4.0, 8.0, 2.0], [5.0, 6.0, 8.0]], [[6.0, 3.0, 
7.0], [7.0, 7.0, 2.0], [6.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 2.0], [7.0, 6.0, 1.0], [7.0, 6.0, 6.0]], 
[[8.0, 1.0, 6.0], [7.0, 5.0, 8.0], [4.0, 8.0, 3.0]], [[8.0, 1.0, 1.0], [8.0, 6.0, 3.0], [8.0, 5.0, 3.0]]], [[[8.0, 1.0, 5.0], 
[1.0, 6.0, 6.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 6.0], [5.0, 7.0, 8.0], [6.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [4.0, 5.0, 7.0], 
[6.0, 5.0, 2.0]]], [[[6.0, 6.0, 1.0], [6.0, 1.0, 1.0], [3.0, 8.0, 5.0]], [[4.0, 6.0, 3.0], [4.0, 8.0, 1.0], [4.0, 8.0, 8.0]], 
[[2.0, 3.0, 8.0], [8.0, 4.0, 8.0], [6.0, 4.0, 6.0]]], [[[6.0, 4.0, 2.0], [3.0, 7.0, 4.0], [5.0, 1.0, 4.0]], [[7.0, 6.0, 7.0], 
[4.0, 3.0, 3.0], [1.0, 7.0, 5.0]], [[1.0, 2.0, 8.0], [6.0, 5.0, 2.0], [8.0, 6.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0]]], [[[2.0]]]])+(1.-msk_val)*numpy.array([[[[7.0]]], [[[3.0]]]])
      arg[1:3,1:2,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 1.0], [2.0, 4.0, 3.0], [2.0, 7.0, 2.0]], [[2.0, 4.0, 6.0], [8.0, 3.0, 1.0], [3.0, 
4.0, 8.0]], [[6.0, 8.0, 4.0], [4.0, 5.0, 1.0], [6.0, 7.0, 4.0]]], [[[5.0, 4.0, 3.0], [1.0, 7.0, 3.0], [5.0, 7.0, 2.0]], [[7.0, 
1.0, 5.0], [8.0, 1.0, 8.0], [8.0, 3.0, 7.0]], [[5.0, 5.0, 7.0], [4.0, 5.0, 3.0], [6.0, 5.0, 4.0]]], [[[4.0, 7.0, 5.0], [6.0, 
7.0, 1.0], [7.0, 4.0, 3.0]], [[5.0, 2.0, 2.0], [6.0, 2.0, 3.0], [7.0, 7.0, 8.0]], [[8.0, 6.0, 6.0], [2.0, 7.0, 1.0], [3.0, 7.0, 
2.0]]], [[[1.0, 3.0, 1.0], [3.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[4.0, 2.0, 7.0], [4.0, 8.0, 2.0], [5.0, 6.0, 8.0]], [[6.0, 3.0, 
7.0], [7.0, 7.0, 2.0], [6.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 2.0], [7.0, 6.0, 1.0], [7.0, 6.0, 6.0]], 
[[8.0, 1.0, 6.0], [7.0, 5.0, 8.0], [4.0, 8.0, 3.0]], [[8.0, 1.0, 1.0], [8.0, 6.0, 3.0], [8.0, 5.0, 3.0]]], [[[8.0, 1.0, 5.0], 
[1.0, 6.0, 6.0], [5.0, 1.0, 8.0]], [[6.0, 3.0, 6.0], [5.0, 7.0, 8.0], [6.0, 3.0, 8.0]], [[7.0, 3.0, 2.0], [4.0, 5.0, 7.0], 
[6.0, 5.0, 2.0]]], [[[6.0, 6.0, 1.0], [6.0, 1.0, 1.0], [3.0, 8.0, 5.0]], [[4.0, 6.0, 3.0], [4.0, 3.0, 1.0], [4.0, 8.0, 8.0]], 
[[2.0, 3.0, 8.0], [8.0, 4.0, 8.0], [6.0, 4.0, 6.0]]], [[[6.0, 4.0, 2.0], [3.0, 7.0, 4.0], [5.0, 1.0, 4.0]], [[7.0, 6.0, 7.0], 
[4.0, 3.0, 3.0], [1.0, 7.0, 5.0]], [[1.0, 2.0, 8.0], [6.0, 5.0, 2.0], [8.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 3.0], [8.0, 6.0, 4.0], [4.0, 4.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 4.0, 4.0], [1.0, 
7.0, 6.0]], [[8.0, 1.0, 7.0], [3.0, 3.0, 4.0], [6.0, 2.0, 1.0]]], [[[6.0, 5.0, 1.0], [7.0, 8.0, 8.0], [1.0, 5.0, 1.0]], [[6.0, 
6.0, 2.0], [3.0, 1.0, 2.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 4.0], [6.0, 1.0, 1.0], [7.0, 8.0, 8.0]]], [[[7.0, 1.0, 3.0], [3.0, 
2.0, 4.0], [1.0, 1.0, 2.0]], [[8.0, 5.0, 2.0], [8.0, 7.0, 3.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 4.0], [5.0, 5.0, 2.0], [1.0, 5.0, 
6.0]]], [[[5.0, 6.0, 1.0], [5.0, 2.0, 2.0], [1.0, 8.0, 8.0]], [[5.0, 7.0, 3.0], [2.0, 4.0, 8.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 
1.0], [2.0, 1.0, 7.0], [4.0, 3.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 4.0, 5.0], [8.0, 5.0, 6.0]], 
[[3.0, 4.0, 7.0], [8.0, 1.0, 1.0], [7.0, 2.0, 2.0]], [[7.0, 1.0, 4.0], [8.0, 2.0, 1.0], [5.0, 4.0, 3.0]]], [[[6.0, 3.0, 2.0], 
[7.0, 1.0, 6.0], [5.0, 1.0, 7.0]], [[1.0, 4.0, 8.0], [4.0, 4.0, 2.0], [7.0, 1.0, 5.0]], [[4.0, 3.0, 7.0], [6.0, 3.0, 5.0], 
[2.0, 2.0, 2.0]]], [[[7.0, 3.0, 7.0], [6.0, 8.0, 5.0], [1.0, 8.0, 6.0]], [[8.0, 8.0, 4.0], [5.0, 8.0, 8.0], [6.0, 6.0, 8.0]], 
[[4.0, 3.0, 4.0], [1.0, 6.0, 2.0], [6.0, 4.0, 4.0]]], [[[3.0, 8.0, 7.0], [7.0, 1.0, 1.0], [1.0, 2.0, 7.0]], [[4.0, 1.0, 7.0], 
[1.0, 2.0, 8.0], [6.0, 7.0, 7.0]], [[6.0, 2.0, 2.0], [4.0, 4.0, 3.0], [8.0, 6.0, 3.0]]]])
      val=numpy.array([[[[8.0, 4.0, 7.0]], [[3.0, 6.0, 5.0]], [[7.0, 2.0, 6.0]]], [[[4.0, 5.0, 6.0]], [[7.0, 7.0, 8.0]], [[1.0, 
3.0, 5.0]]]])
      arg[2:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 3.0], [8.0, 6.0, 4.0], [4.0, 4.0, 3.0]], [[4.0, 6.0, 3.0], [2.0, 4.0, 4.0], [1.0, 
7.0, 6.0]], [[8.0, 1.0, 7.0], [3.0, 3.0, 4.0], [6.0, 2.0, 1.0]]], [[[6.0, 5.0, 1.0], [7.0, 8.0, 8.0], [1.0, 5.0, 1.0]], [[6.0, 
6.0, 2.0], [3.0, 1.0, 2.0], [6.0, 8.0, 4.0]], [[1.0, 4.0, 4.0], [6.0, 1.0, 1.0], [7.0, 8.0, 8.0]]], [[[7.0, 1.0, 3.0], [8.0, 
4.0, 7.0], [1.0, 1.0, 2.0]], [[8.0, 5.0, 2.0], [3.0, 6.0, 5.0], [1.0, 5.0, 3.0]], [[6.0, 6.0, 4.0], [7.0, 2.0, 6.0], [1.0, 5.0, 
6.0]]], [[[5.0, 6.0, 1.0], [4.0, 5.0, 6.0], [1.0, 8.0, 8.0]], [[5.0, 7.0, 3.0], [7.0, 7.0, 8.0], [3.0, 8.0, 2.0]], [[3.0, 1.0, 
1.0], [1.0, 3.0, 5.0], [4.0, 3.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 5.0], [4.0, 4.0, 5.0], [8.0, 5.0, 6.0]], 
[[3.0, 4.0, 7.0], [8.0, 1.0, 1.0], [7.0, 2.0, 2.0]], [[7.0, 1.0, 4.0], [8.0, 2.0, 1.0], [5.0, 4.0, 3.0]]], [[[6.0, 3.0, 2.0], 
[7.0, 1.0, 6.0], [5.0, 1.0, 7.0]], [[1.0, 4.0, 8.0], [4.0, 4.0, 2.0], [7.0, 1.0, 5.0]], [[4.0, 3.0, 7.0], [6.0, 3.0, 5.0], 
[2.0, 2.0, 2.0]]], [[[7.0, 3.0, 7.0], [8.0, 4.0, 7.0], [1.0, 8.0, 6.0]], [[8.0, 8.0, 4.0], [3.0, 6.0, 5.0], [6.0, 6.0, 8.0]], 
[[4.0, 3.0, 4.0], [7.0, 2.0, 6.0], [6.0, 4.0, 4.0]]], [[[3.0, 8.0, 7.0], [4.0, 5.0, 6.0], [1.0, 2.0, 7.0]], [[4.0, 1.0, 7.0], 
[7.0, 7.0, 8.0], [6.0, 7.0, 7.0]], [[6.0, 2.0, 2.0], [1.0, 3.0, 5.0], [8.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 4.0], [7.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[5.0, 2.0, 1.0], [4.0, 1.0, 6.0], [3.0, 
8.0, 2.0]], [[3.0, 3.0, 3.0], [2.0, 3.0, 4.0], [3.0, 3.0, 2.0]]], [[[6.0, 2.0, 3.0], [1.0, 3.0, 6.0], [5.0, 7.0, 5.0]], [[5.0, 
1.0, 4.0], [1.0, 1.0, 7.0], [7.0, 4.0, 2.0]], [[8.0, 4.0, 2.0], [4.0, 5.0, 8.0], [7.0, 6.0, 8.0]]], [[[8.0, 7.0, 8.0], [8.0, 
1.0, 1.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 6.0], [4.0, 8.0, 6.0], [5.0, 4.0, 2.0]], [[8.0, 4.0, 2.0], [4.0, 1.0, 2.0], [3.0, 3.0, 
6.0]]], [[[1.0, 7.0, 4.0], [2.0, 5.0, 8.0], [1.0, 2.0, 1.0]], [[4.0, 8.0, 6.0], [5.0, 5.0, 8.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 
5.0], [3.0, 8.0, 2.0], [5.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 7.0, 3.0], [7.0, 8.0, 5.0], [4.0, 8.0, 3.0]], 
[[4.0, 2.0, 2.0], [1.0, 2.0, 8.0], [2.0, 8.0, 2.0]], [[7.0, 4.0, 8.0], [8.0, 5.0, 7.0], [6.0, 1.0, 6.0]]], [[[7.0, 6.0, 8.0], 
[1.0, 3.0, 7.0], [6.0, 5.0, 1.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 6.0], [1.0, 2.0, 8.0]], [[8.0, 1.0, 2.0], [1.0, 4.0, 3.0], 
[7.0, 3.0, 4.0]]], [[[8.0, 3.0, 5.0], [1.0, 2.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 1.0, 4.0], [1.0, 3.0, 2.0], [4.0, 7.0, 5.0]], 
[[1.0, 2.0, 7.0], [6.0, 7.0, 5.0], [7.0, 1.0, 6.0]]], [[[1.0, 1.0, 4.0], [3.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[2.0, 1.0, 4.0], 
[2.0, 5.0, 4.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], [2.0, 4.0, 7.0], [5.0, 3.0, 6.0]]]])
      val=Data(numpy.array([[[[1.0, 5.0, 1.0]], [[7.0, 7.0, 1.0]], [[7.0, 6.0, 3.0]]], [[[7.0, 2.0, 4.0]], [[3.0, 2.0, 3.0]], 
[[7.0, 1.0, 8.0]]]]),self.functionspace)
      arg[2:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [7.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[5.0, 2.0, 1.0], [4.0, 1.0, 6.0], [3.0, 
8.0, 2.0]], [[3.0, 3.0, 3.0], [2.0, 3.0, 4.0], [3.0, 3.0, 2.0]]], [[[6.0, 2.0, 3.0], [1.0, 3.0, 6.0], [5.0, 7.0, 5.0]], [[5.0, 
1.0, 4.0], [1.0, 1.0, 7.0], [7.0, 4.0, 2.0]], [[8.0, 4.0, 2.0], [4.0, 5.0, 8.0], [7.0, 6.0, 8.0]]], [[[8.0, 7.0, 8.0], [1.0, 
5.0, 1.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 6.0], [7.0, 7.0, 1.0], [5.0, 4.0, 2.0]], [[8.0, 4.0, 2.0], [7.0, 6.0, 3.0], [3.0, 3.0, 
6.0]]], [[[1.0, 7.0, 4.0], [7.0, 2.0, 4.0], [1.0, 2.0, 1.0]], [[4.0, 8.0, 6.0], [3.0, 2.0, 3.0], [5.0, 8.0, 8.0]], [[3.0, 3.0, 
5.0], [7.0, 1.0, 8.0], [5.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 7.0, 3.0], [7.0, 8.0, 5.0], [4.0, 8.0, 3.0]], 
[[4.0, 2.0, 2.0], [1.0, 2.0, 8.0], [2.0, 8.0, 2.0]], [[7.0, 4.0, 8.0], [8.0, 5.0, 7.0], [6.0, 1.0, 6.0]]], [[[7.0, 6.0, 8.0], 
[1.0, 3.0, 7.0], [6.0, 5.0, 1.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 6.0], [1.0, 2.0, 8.0]], [[8.0, 1.0, 2.0], [1.0, 4.0, 3.0], 
[7.0, 3.0, 4.0]]], [[[8.0, 3.0, 5.0], [1.0, 5.0, 1.0], [3.0, 2.0, 3.0]], [[1.0, 1.0, 4.0], [7.0, 7.0, 1.0], [4.0, 7.0, 5.0]], 
[[1.0, 2.0, 7.0], [7.0, 6.0, 3.0], [7.0, 1.0, 6.0]]], [[[1.0, 1.0, 4.0], [7.0, 2.0, 4.0], [4.0, 5.0, 3.0]], [[2.0, 1.0, 4.0], 
[3.0, 2.0, 3.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], [7.0, 1.0, 8.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 5.0], [4.0, 3.0, 1.0], [3.0, 5.0, 1.0]], [[1.0, 7.0, 4.0], [8.0, 4.0, 8.0], [1.0, 
5.0, 4.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 5.0], [7.0, 3.0, 6.0]]], [[[4.0, 8.0, 5.0], [4.0, 8.0, 2.0], [8.0, 4.0, 1.0]], [[1.0, 
2.0, 3.0], [5.0, 1.0, 8.0], [5.0, 3.0, 2.0]], [[8.0, 4.0, 3.0], [3.0, 4.0, 4.0], [8.0, 5.0, 2.0]]], [[[3.0, 1.0, 8.0], [8.0, 
5.0, 1.0], [6.0, 7.0, 1.0]], [[3.0, 7.0, 7.0], [3.0, 5.0, 3.0], [3.0, 1.0, 4.0]], [[8.0, 4.0, 8.0], [8.0, 7.0, 2.0], [1.0, 7.0, 
7.0]]], [[[2.0, 7.0, 4.0], [2.0, 6.0, 2.0], [5.0, 2.0, 7.0]], [[5.0, 8.0, 1.0], [6.0, 3.0, 8.0], [7.0, 4.0, 8.0]], [[2.0, 2.0, 
1.0], [8.0, 5.0, 8.0], [1.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 4.0], [1.0, 4.0, 5.0], [4.0, 2.0, 3.0]], 
[[7.0, 5.0, 4.0], [5.0, 6.0, 7.0], [7.0, 5.0, 8.0]], [[2.0, 8.0, 3.0], [1.0, 3.0, 8.0], [1.0, 4.0, 5.0]]], [[[3.0, 8.0, 6.0], 
[2.0, 8.0, 5.0], [5.0, 6.0, 5.0]], [[1.0, 1.0, 7.0], [5.0, 6.0, 5.0], [1.0, 3.0, 3.0]], [[7.0, 2.0, 2.0], [6.0, 6.0, 5.0], 
[4.0, 5.0, 5.0]]], [[[5.0, 8.0, 7.0], [3.0, 1.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], [7.0, 5.0, 7.0], [4.0, 1.0, 8.0]], 
[[3.0, 3.0, 8.0], [6.0, 5.0, 2.0], [4.0, 1.0, 6.0]]], [[[6.0, 7.0, 1.0], [5.0, 8.0, 6.0], [7.0, 1.0, 6.0]], [[8.0, 5.0, 7.0], 
[7.0, 7.0, 4.0], [6.0, 8.0, 6.0]], [[5.0, 7.0, 6.0], [7.0, 2.0, 3.0], [5.0, 7.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 4.0, 3.0]], [[2.0, 6.0, 1.0]], [[7.0, 6.0, 8.0]]], [[[1.0, 4.0, 6.0]], [[1.0, 7.0, 
4.0]], [[5.0, 2.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 1.0, 4.0]], [[2.0, 6.0, 7.0]], [[1.0, 8.0, 7.0]]], [[[8.0, 5.0, 
5.0]], [[3.0, 4.0, 6.0]], [[3.0, 3.0, 5.0]]]])
      arg[2:,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 5.0], [4.0, 3.0, 1.0], [3.0, 5.0, 1.0]], [[1.0, 7.0, 4.0], [8.0, 4.0, 8.0], [1.0, 
5.0, 4.0]], [[8.0, 6.0, 5.0], [5.0, 7.0, 5.0], [7.0, 3.0, 6.0]]], [[[4.0, 8.0, 5.0], [4.0, 8.0, 2.0], [8.0, 4.0, 1.0]], [[1.0, 
2.0, 3.0], [5.0, 1.0, 8.0], [5.0, 3.0, 2.0]], [[8.0, 4.0, 3.0], [3.0, 4.0, 4.0], [8.0, 5.0, 2.0]]], [[[3.0, 1.0, 8.0], [2.0, 
4.0, 3.0], [6.0, 7.0, 1.0]], [[3.0, 7.0, 7.0], [2.0, 6.0, 1.0], [3.0, 1.0, 4.0]], [[8.0, 4.0, 8.0], [7.0, 6.0, 8.0], [1.0, 7.0, 
7.0]]], [[[2.0, 7.0, 4.0], [1.0, 4.0, 6.0], [5.0, 2.0, 7.0]], [[5.0, 8.0, 1.0], [1.0, 7.0, 4.0], [7.0, 4.0, 8.0]], [[2.0, 2.0, 
1.0], [5.0, 2.0, 8.0], [1.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 4.0], [1.0, 4.0, 5.0], [4.0, 2.0, 3.0]], 
[[7.0, 5.0, 4.0], [5.0, 6.0, 7.0], [7.0, 5.0, 8.0]], [[2.0, 8.0, 3.0], [1.0, 3.0, 8.0], [1.0, 4.0, 5.0]]], [[[3.0, 8.0, 6.0], 
[2.0, 8.0, 5.0], [5.0, 6.0, 5.0]], [[1.0, 1.0, 7.0], [5.0, 6.0, 5.0], [1.0, 3.0, 3.0]], [[7.0, 2.0, 2.0], [6.0, 6.0, 5.0], 
[4.0, 5.0, 5.0]]], [[[5.0, 8.0, 7.0], [4.0, 1.0, 4.0], [6.0, 2.0, 5.0]], [[2.0, 8.0, 5.0], [2.0, 6.0, 7.0], [4.0, 1.0, 8.0]], 
[[3.0, 3.0, 8.0], [1.0, 8.0, 7.0], [4.0, 1.0, 6.0]]], [[[6.0, 7.0, 1.0], [8.0, 5.0, 5.0], [7.0, 1.0, 6.0]], [[8.0, 5.0, 7.0], 
[3.0, 4.0, 6.0], [6.0, 8.0, 6.0]], [[5.0, 7.0, 6.0], [3.0, 3.0, 5.0], [5.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 4.0], [7.0, 2.0, 4.0], [1.0, 3.0, 4.0]], [[8.0, 4.0, 5.0], [1.0, 8.0, 2.0], [8.0, 
5.0, 4.0]], [[3.0, 3.0, 8.0], [8.0, 5.0, 8.0], [3.0, 1.0, 8.0]]], [[[3.0, 5.0, 1.0], [6.0, 8.0, 4.0], [4.0, 5.0, 3.0]], [[8.0, 
1.0, 5.0], [4.0, 7.0, 2.0], [6.0, 3.0, 1.0]], [[7.0, 4.0, 3.0], [2.0, 7.0, 1.0], [5.0, 8.0, 8.0]]], [[[1.0, 4.0, 1.0], [7.0, 
3.0, 1.0], [4.0, 4.0, 8.0]], [[4.0, 2.0, 3.0], [1.0, 4.0, 8.0], [5.0, 5.0, 4.0]], [[8.0, 4.0, 4.0], [4.0, 4.0, 6.0], [7.0, 2.0, 
7.0]]], [[[8.0, 4.0, 7.0], [1.0, 3.0, 4.0], [1.0, 4.0, 8.0]], [[8.0, 8.0, 1.0], [5.0, 3.0, 3.0], [3.0, 6.0, 8.0]], [[5.0, 7.0, 
2.0], [5.0, 2.0, 7.0], [6.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 1.0], [6.0, 4.0, 5.0], [2.0, 1.0, 4.0]], 
[[8.0, 8.0, 2.0], [1.0, 8.0, 3.0], [1.0, 7.0, 5.0]], [[4.0, 8.0, 4.0], [1.0, 4.0, 1.0], [8.0, 2.0, 7.0]]], [[[8.0, 5.0, 3.0], 
[7.0, 6.0, 2.0], [5.0, 4.0, 7.0]], [[8.0, 8.0, 2.0], [5.0, 1.0, 4.0], [7.0, 7.0, 7.0]], [[4.0, 7.0, 6.0], [2.0, 4.0, 6.0], 
[7.0, 3.0, 5.0]]], [[[2.0, 6.0, 7.0], [1.0, 2.0, 5.0], [6.0, 3.0, 2.0]], [[2.0, 6.0, 2.0], [2.0, 1.0, 6.0], [2.0, 1.0, 7.0]], 
[[1.0, 6.0, 6.0], [7.0, 3.0, 7.0], [1.0, 5.0, 3.0]]], [[[2.0, 2.0, 1.0], [5.0, 5.0, 1.0], [3.0, 3.0, 2.0]], [[8.0, 7.0, 2.0], 
[5.0, 3.0, 6.0], [4.0, 2.0, 7.0]], [[8.0, 6.0, 8.0], [3.0, 4.0, 2.0], [5.0, 1.0, 3.0]]]])
      val=numpy.array([[[[5.0, 3.0, 3.0]], [[3.0, 4.0, 5.0]], [[8.0, 8.0, 6.0]]], [[[6.0, 7.0, 3.0]], [[7.0, 5.0, 8.0]], [[6.0, 
5.0, 1.0]]]])
      arg[1:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 4.0], [7.0, 2.0, 4.0], [1.0, 3.0, 4.0]], [[8.0, 4.0, 5.0], [1.0, 8.0, 2.0], [8.0, 
5.0, 4.0]], [[3.0, 3.0, 8.0], [8.0, 5.0, 8.0], [3.0, 1.0, 8.0]]], [[[3.0, 5.0, 1.0], [6.0, 8.0, 4.0], [5.0, 3.0, 3.0]], [[8.0, 
1.0, 5.0], [4.0, 7.0, 2.0], [3.0, 4.0, 5.0]], [[7.0, 4.0, 3.0], [2.0, 7.0, 1.0], [8.0, 8.0, 6.0]]], [[[1.0, 4.0, 1.0], [7.0, 
3.0, 1.0], [6.0, 7.0, 3.0]], [[4.0, 2.0, 3.0], [1.0, 4.0, 8.0], [7.0, 5.0, 8.0]], [[8.0, 4.0, 4.0], [4.0, 4.0, 6.0], [6.0, 5.0, 
1.0]]], [[[8.0, 4.0, 7.0], [1.0, 3.0, 4.0], [1.0, 4.0, 8.0]], [[8.0, 8.0, 1.0], [5.0, 3.0, 3.0], [3.0, 6.0, 8.0]], [[5.0, 7.0, 
2.0], [5.0, 2.0, 7.0], [6.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 1.0], [6.0, 4.0, 5.0], [2.0, 1.0, 4.0]], 
[[8.0, 8.0, 2.0], [1.0, 8.0, 3.0], [1.0, 7.0, 5.0]], [[4.0, 8.0, 4.0], [1.0, 4.0, 1.0], [8.0, 2.0, 7.0]]], [[[8.0, 5.0, 3.0], 
[7.0, 6.0, 2.0], [5.0, 3.0, 3.0]], [[8.0, 8.0, 2.0], [5.0, 1.0, 4.0], [3.0, 4.0, 5.0]], [[4.0, 7.0, 6.0], [2.0, 4.0, 6.0], 
[8.0, 8.0, 6.0]]], [[[2.0, 6.0, 7.0], [1.0, 2.0, 5.0], [6.0, 7.0, 3.0]], [[2.0, 6.0, 2.0], [2.0, 1.0, 6.0], [7.0, 5.0, 8.0]], 
[[1.0, 6.0, 6.0], [7.0, 3.0, 7.0], [6.0, 5.0, 1.0]]], [[[2.0, 2.0, 1.0], [5.0, 5.0, 1.0], [3.0, 3.0, 2.0]], [[8.0, 7.0, 2.0], 
[5.0, 3.0, 6.0], [4.0, 2.0, 7.0]], [[8.0, 6.0, 8.0], [3.0, 4.0, 2.0], [5.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 1.0], [1.0, 4.0, 2.0], [4.0, 2.0, 7.0]], [[5.0, 4.0, 6.0], [8.0, 4.0, 5.0], [3.0, 
5.0, 3.0]], [[2.0, 5.0, 4.0], [7.0, 6.0, 3.0], [8.0, 1.0, 4.0]]], [[[8.0, 7.0, 4.0], [5.0, 3.0, 5.0], [5.0, 7.0, 3.0]], [[8.0, 
6.0, 4.0], [6.0, 2.0, 1.0], [5.0, 3.0, 2.0]], [[2.0, 5.0, 1.0], [3.0, 7.0, 1.0], [2.0, 3.0, 8.0]]], [[[1.0, 3.0, 3.0], [2.0, 
2.0, 5.0], [7.0, 3.0, 3.0]], [[3.0, 5.0, 7.0], [1.0, 8.0, 7.0], [5.0, 2.0, 7.0]], [[5.0, 1.0, 2.0], [1.0, 4.0, 8.0], [4.0, 2.0, 
3.0]]], [[[1.0, 2.0, 4.0], [4.0, 1.0, 7.0], [7.0, 5.0, 2.0]], [[2.0, 7.0, 3.0], [7.0, 1.0, 7.0], [4.0, 5.0, 2.0]], [[1.0, 6.0, 
7.0], [8.0, 8.0, 5.0], [3.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 6.0, 1.0], [3.0, 6.0, 5.0], [6.0, 1.0, 6.0]], 
[[7.0, 1.0, 3.0], [5.0, 4.0, 2.0], [2.0, 6.0, 8.0]], [[8.0, 6.0, 2.0], [5.0, 8.0, 7.0], [7.0, 2.0, 4.0]]], [[[1.0, 8.0, 6.0], 
[5.0, 1.0, 3.0], [8.0, 6.0, 1.0]], [[3.0, 6.0, 6.0], [7.0, 2.0, 4.0], [7.0, 6.0, 5.0]], [[2.0, 3.0, 2.0], [6.0, 1.0, 2.0], 
[7.0, 8.0, 8.0]]], [[[2.0, 1.0, 6.0], [3.0, 5.0, 8.0], [7.0, 6.0, 8.0]], [[4.0, 7.0, 8.0], [1.0, 3.0, 7.0], [7.0, 2.0, 8.0]], 
[[5.0, 2.0, 2.0], [8.0, 4.0, 5.0], [3.0, 3.0, 3.0]]], [[[1.0, 6.0, 7.0], [6.0, 7.0, 4.0], [5.0, 4.0, 7.0]], [[7.0, 8.0, 8.0], 
[5.0, 7.0, 3.0], [1.0, 5.0, 6.0]], [[6.0, 5.0, 2.0], [7.0, 3.0, 2.0], [1.0, 2.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0, 7.0, 4.0]], [[3.0, 7.0, 7.0]], [[5.0, 6.0, 8.0]]], [[[1.0, 1.0, 5.0]], [[5.0, 5.0, 4.0]], 
[[3.0, 5.0, 5.0]]]]),self.functionspace)
      arg[1:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 1.0], [1.0, 4.0, 2.0], [4.0, 2.0, 7.0]], [[5.0, 4.0, 6.0], [8.0, 4.0, 5.0], [3.0, 
5.0, 3.0]], [[2.0, 5.0, 4.0], [7.0, 6.0, 3.0], [8.0, 1.0, 4.0]]], [[[8.0, 7.0, 4.0], [5.0, 3.0, 5.0], [4.0, 7.0, 4.0]], [[8.0, 
6.0, 4.0], [6.0, 2.0, 1.0], [3.0, 7.0, 7.0]], [[2.0, 5.0, 1.0], [3.0, 7.0, 1.0], [5.0, 6.0, 8.0]]], [[[1.0, 3.0, 3.0], [2.0, 
2.0, 5.0], [1.0, 1.0, 5.0]], [[3.0, 5.0, 7.0], [1.0, 8.0, 7.0], [5.0, 5.0, 4.0]], [[5.0, 1.0, 2.0], [1.0, 4.0, 8.0], [3.0, 5.0, 
5.0]]], [[[1.0, 2.0, 4.0], [4.0, 1.0, 7.0], [7.0, 5.0, 2.0]], [[2.0, 7.0, 3.0], [7.0, 1.0, 7.0], [4.0, 5.0, 2.0]], [[1.0, 6.0, 
7.0], [8.0, 8.0, 5.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 1.0], [3.0, 6.0, 5.0], [6.0, 1.0, 6.0]], 
[[7.0, 1.0, 3.0], [5.0, 4.0, 2.0], [2.0, 6.0, 8.0]], [[8.0, 6.0, 2.0], [5.0, 8.0, 7.0], [7.0, 2.0, 4.0]]], [[[1.0, 8.0, 6.0], 
[5.0, 1.0, 3.0], [4.0, 7.0, 4.0]], [[3.0, 6.0, 6.0], [7.0, 2.0, 4.0], [3.0, 7.0, 7.0]], [[2.0, 3.0, 2.0], [6.0, 1.0, 2.0], 
[5.0, 6.0, 8.0]]], [[[2.0, 1.0, 6.0], [3.0, 5.0, 8.0], [1.0, 1.0, 5.0]], [[4.0, 7.0, 8.0], [1.0, 3.0, 7.0], [5.0, 5.0, 4.0]], 
[[5.0, 2.0, 2.0], [8.0, 4.0, 5.0], [3.0, 5.0, 5.0]]], [[[1.0, 6.0, 7.0], [6.0, 7.0, 4.0], [5.0, 4.0, 7.0]], [[7.0, 8.0, 8.0], 
[5.0, 7.0, 3.0], [1.0, 5.0, 6.0]], [[6.0, 5.0, 2.0], [7.0, 3.0, 2.0], [1.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 4.0]], [[5.0, 5.0, 3.0], [1.0, 8.0, 2.0], [2.0, 
6.0, 4.0]], [[6.0, 7.0, 6.0], [8.0, 2.0, 5.0], [7.0, 7.0, 5.0]]], [[[6.0, 4.0, 5.0], [2.0, 4.0, 4.0], [1.0, 1.0, 8.0]], [[7.0, 
3.0, 3.0], [1.0, 1.0, 3.0], [1.0, 4.0, 8.0]], [[4.0, 2.0, 5.0], [1.0, 5.0, 2.0], [2.0, 1.0, 6.0]]], [[[8.0, 1.0, 7.0], [3.0, 
7.0, 6.0], [3.0, 1.0, 7.0]], [[5.0, 8.0, 4.0], [2.0, 8.0, 2.0], [8.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], [4.0, 3.0, 8.0], [2.0, 4.0, 
7.0]]], [[[5.0, 4.0, 2.0], [8.0, 8.0, 2.0], [4.0, 5.0, 7.0]], [[1.0, 2.0, 1.0], [6.0, 1.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 7.0, 
7.0], [2.0, 4.0, 7.0], [7.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 4.0], [6.0, 2.0, 4.0], [3.0, 2.0, 2.0]], 
[[3.0, 7.0, 5.0], [8.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 5.0], [3.0, 7.0, 8.0], [7.0, 4.0, 3.0]]], [[[6.0, 7.0, 2.0], 
[6.0, 6.0, 2.0], [4.0, 1.0, 5.0]], [[8.0, 3.0, 5.0], [1.0, 5.0, 2.0], [7.0, 2.0, 6.0]], [[4.0, 4.0, 7.0], [2.0, 8.0, 4.0], 
[3.0, 8.0, 3.0]]], [[[6.0, 8.0, 3.0], [1.0, 5.0, 2.0], [2.0, 8.0, 7.0]], [[3.0, 5.0, 6.0], [2.0, 1.0, 1.0], [5.0, 3.0, 5.0]], 
[[6.0, 2.0, 7.0], [8.0, 3.0, 3.0], [2.0, 7.0, 4.0]]], [[[5.0, 7.0, 6.0], [2.0, 3.0, 4.0], [8.0, 2.0, 6.0]], [[1.0, 3.0, 1.0], 
[6.0, 3.0, 1.0], [5.0, 8.0, 8.0]], [[8.0, 6.0, 6.0], [7.0, 7.0, 2.0], [1.0, 3.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 4.0, 8.0]], [[7.0, 8.0, 8.0]], [[2.0, 8.0, 3.0]]], [[[8.0, 5.0, 1.0]], [[1.0, 5.0, 
2.0]], [[2.0, 4.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 4.0, 4.0]], [[1.0, 4.0, 4.0]], [[1.0, 3.0, 4.0]]], [[[6.0, 1.0, 
5.0]], [[5.0, 8.0, 7.0]], [[1.0, 7.0, 7.0]]]])
      arg[1:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 3.0], [6.0, 7.0, 1.0], [2.0, 1.0, 4.0]], [[5.0, 5.0, 3.0], [1.0, 8.0, 2.0], [2.0, 
6.0, 4.0]], [[6.0, 7.0, 6.0], [8.0, 2.0, 5.0], [7.0, 7.0, 5.0]]], [[[6.0, 4.0, 5.0], [2.0, 4.0, 4.0], [3.0, 4.0, 8.0]], [[7.0, 
3.0, 3.0], [1.0, 1.0, 3.0], [7.0, 8.0, 8.0]], [[4.0, 2.0, 5.0], [1.0, 5.0, 2.0], [2.0, 8.0, 3.0]]], [[[8.0, 1.0, 7.0], [3.0, 
7.0, 6.0], [8.0, 5.0, 1.0]], [[5.0, 8.0, 4.0], [2.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[7.0, 4.0, 1.0], [4.0, 3.0, 8.0], [2.0, 4.0, 
3.0]]], [[[5.0, 4.0, 2.0], [8.0, 8.0, 2.0], [4.0, 5.0, 7.0]], [[1.0, 2.0, 1.0], [6.0, 1.0, 3.0], [8.0, 2.0, 4.0]], [[8.0, 7.0, 
7.0], [2.0, 4.0, 7.0], [7.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 4.0], [6.0, 2.0, 4.0], [3.0, 2.0, 2.0]], 
[[3.0, 7.0, 5.0], [8.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[4.0, 1.0, 5.0], [3.0, 7.0, 8.0], [7.0, 4.0, 3.0]]], [[[6.0, 7.0, 2.0], 
[6.0, 6.0, 2.0], [8.0, 4.0, 4.0]], [[8.0, 3.0, 5.0], [1.0, 5.0, 2.0], [1.0, 4.0, 4.0]], [[4.0, 4.0, 7.0], [2.0, 8.0, 4.0], 
[1.0, 3.0, 4.0]]], [[[6.0, 8.0, 3.0], [1.0, 5.0, 2.0], [6.0, 1.0, 5.0]], [[3.0, 5.0, 6.0], [2.0, 1.0, 1.0], [5.0, 8.0, 7.0]], 
[[6.0, 2.0, 7.0], [8.0, 3.0, 3.0], [1.0, 7.0, 7.0]]], [[[5.0, 7.0, 6.0], [2.0, 3.0, 4.0], [8.0, 2.0, 6.0]], [[1.0, 3.0, 1.0], 
[6.0, 3.0, 1.0], [5.0, 8.0, 8.0]], [[8.0, 6.0, 6.0], [7.0, 7.0, 2.0], [1.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 4.0], [3.0, 7.0, 7.0], [8.0, 1.0, 2.0]], [[4.0, 6.0, 2.0], [3.0, 8.0, 5.0], [4.0, 
8.0, 4.0]], [[8.0, 1.0, 5.0], [2.0, 1.0, 8.0], [1.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [7.0, 4.0, 8.0], [1.0, 3.0, 6.0]], [[3.0, 
5.0, 7.0], [5.0, 7.0, 6.0], [3.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], [4.0, 4.0, 1.0], [7.0, 8.0, 1.0]]], [[[4.0, 8.0, 2.0], [5.0, 
1.0, 5.0], [3.0, 8.0, 7.0]], [[4.0, 5.0, 6.0], [3.0, 6.0, 4.0], [5.0, 5.0, 3.0]], [[7.0, 8.0, 7.0], [8.0, 6.0, 7.0], [4.0, 5.0, 
1.0]]], [[[4.0, 5.0, 2.0], [3.0, 7.0, 7.0], [3.0, 8.0, 8.0]], [[6.0, 7.0, 2.0], [2.0, 7.0, 3.0], [7.0, 8.0, 3.0]], [[2.0, 3.0, 
3.0], [8.0, 8.0, 6.0], [1.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 6.0, 4.0], [5.0, 7.0, 4.0], [3.0, 8.0, 2.0]], 
[[4.0, 6.0, 5.0], [1.0, 3.0, 8.0], [7.0, 2.0, 5.0]], [[3.0, 8.0, 1.0], [2.0, 6.0, 5.0], [5.0, 5.0, 7.0]]], [[[4.0, 5.0, 8.0], 
[1.0, 2.0, 2.0], [3.0, 1.0, 7.0]], [[7.0, 6.0, 2.0], [3.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[7.0, 6.0, 2.0], [7.0, 4.0, 1.0], 
[1.0, 7.0, 5.0]]], [[[4.0, 4.0, 3.0], [8.0, 6.0, 3.0], [2.0, 1.0, 4.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 4.0], [2.0, 7.0, 4.0]], 
[[1.0, 1.0, 5.0], [2.0, 5.0, 2.0], [1.0, 4.0, 5.0]]], [[[1.0, 8.0, 3.0], [3.0, 5.0, 5.0], [4.0, 2.0, 2.0]], [[6.0, 7.0, 1.0], 
[1.0, 1.0, 1.0], [3.0, 3.0, 7.0]], [[6.0, 5.0, 1.0], [1.0, 2.0, 7.0], [2.0, 6.0, 2.0]]]])
      val=numpy.array([[[[2.0, 8.0, 6.0], [1.0, 6.0, 8.0], [4.0, 7.0, 6.0]]], [[[5.0, 2.0, 8.0], [2.0, 8.0, 8.0], [3.0, 5.0, 
3.0]]]])
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 1.0, 4.0], [3.0, 7.0, 7.0], [8.0, 1.0, 2.0]], [[4.0, 6.0, 2.0], [3.0, 8.0, 5.0], [4.0, 
8.0, 4.0]], [[8.0, 1.0, 5.0], [2.0, 1.0, 8.0], [1.0, 2.0, 3.0]]], [[[5.0, 2.0, 1.0], [7.0, 4.0, 8.0], [1.0, 3.0, 6.0]], [[3.0, 
5.0, 7.0], [5.0, 7.0, 6.0], [3.0, 4.0, 8.0]], [[2.0, 5.0, 3.0], [4.0, 4.0, 1.0], [7.0, 8.0, 1.0]]], [[[4.0, 8.0, 2.0], [5.0, 
1.0, 5.0], [3.0, 8.0, 7.0]], [[4.0, 5.0, 6.0], [3.0, 6.0, 4.0], [5.0, 5.0, 3.0]], [[2.0, 8.0, 6.0], [1.0, 6.0, 8.0], [4.0, 7.0, 
6.0]]], [[[4.0, 5.0, 2.0], [3.0, 7.0, 7.0], [3.0, 8.0, 8.0]], [[6.0, 7.0, 2.0], [2.0, 7.0, 3.0], [7.0, 8.0, 3.0]], [[5.0, 2.0, 
8.0], [2.0, 8.0, 8.0], [3.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 6.0, 4.0], [5.0, 7.0, 4.0], [3.0, 8.0, 2.0]], 
[[4.0, 6.0, 5.0], [1.0, 3.0, 8.0], [7.0, 2.0, 5.0]], [[3.0, 8.0, 1.0], [2.0, 6.0, 5.0], [5.0, 5.0, 7.0]]], [[[4.0, 5.0, 8.0], 
[1.0, 2.0, 2.0], [3.0, 1.0, 7.0]], [[7.0, 6.0, 2.0], [3.0, 6.0, 1.0], [2.0, 6.0, 6.0]], [[7.0, 6.0, 2.0], [7.0, 4.0, 1.0], 
[1.0, 7.0, 5.0]]], [[[4.0, 4.0, 3.0], [8.0, 6.0, 3.0], [2.0, 1.0, 4.0]], [[1.0, 5.0, 7.0], [6.0, 8.0, 4.0], [2.0, 7.0, 4.0]], 
[[2.0, 8.0, 6.0], [1.0, 6.0, 8.0], [4.0, 7.0, 6.0]]], [[[1.0, 8.0, 3.0], [3.0, 5.0, 5.0], [4.0, 2.0, 2.0]], [[6.0, 7.0, 1.0], 
[1.0, 1.0, 1.0], [3.0, 3.0, 7.0]], [[5.0, 2.0, 8.0], [2.0, 8.0, 8.0], [3.0, 5.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 5.0], [2.0, 7.0, 1.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 5.0], [1.0, 5.0, 5.0], [8.0, 
3.0, 8.0]], [[5.0, 3.0, 5.0], [7.0, 3.0, 8.0], [7.0, 7.0, 1.0]]], [[[5.0, 4.0, 2.0], [1.0, 5.0, 5.0], [6.0, 2.0, 4.0]], [[5.0, 
5.0, 7.0], [5.0, 3.0, 3.0], [6.0, 2.0, 2.0]], [[8.0, 3.0, 7.0], [6.0, 3.0, 2.0], [3.0, 5.0, 2.0]]], [[[2.0, 4.0, 8.0], [1.0, 
6.0, 1.0], [2.0, 5.0, 2.0]], [[2.0, 2.0, 3.0], [5.0, 8.0, 6.0], [3.0, 6.0, 2.0]], [[6.0, 6.0, 5.0], [7.0, 6.0, 7.0], [3.0, 1.0, 
3.0]]], [[[3.0, 5.0, 1.0], [3.0, 7.0, 2.0], [1.0, 3.0, 8.0]], [[4.0, 2.0, 5.0], [4.0, 5.0, 8.0], [1.0, 1.0, 1.0]], [[7.0, 4.0, 
4.0], [8.0, 2.0, 8.0], [1.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 6.0], [7.0, 4.0, 7.0], [1.0, 1.0, 5.0]], 
[[4.0, 2.0, 6.0], [8.0, 2.0, 7.0], [2.0, 7.0, 4.0]], [[8.0, 4.0, 2.0], [3.0, 8.0, 5.0], [4.0, 5.0, 7.0]]], [[[3.0, 6.0, 3.0], 
[5.0, 7.0, 1.0], [8.0, 2.0, 3.0]], [[4.0, 8.0, 2.0], [3.0, 8.0, 2.0], [6.0, 4.0, 4.0]], [[5.0, 6.0, 7.0], [2.0, 3.0, 7.0], 
[4.0, 2.0, 4.0]]], [[[4.0, 1.0, 3.0], [8.0, 3.0, 7.0], [1.0, 6.0, 7.0]], [[3.0, 8.0, 3.0], [4.0, 1.0, 5.0], [8.0, 5.0, 4.0]], 
[[5.0, 7.0, 3.0], [4.0, 7.0, 7.0], [7.0, 1.0, 1.0]]], [[[8.0, 6.0, 8.0], [7.0, 4.0, 5.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 6.0], 
[4.0, 8.0, 5.0], [2.0, 8.0, 4.0]], [[3.0, 2.0, 1.0], [3.0, 2.0, 8.0], [1.0, 7.0, 8.0]]]])
      val=Data(numpy.array([[[[3.0, 8.0, 7.0], [8.0, 3.0, 6.0], [3.0, 6.0, 8.0]]], [[[4.0, 5.0, 3.0], [3.0, 1.0, 8.0], [3.0, 
8.0, 5.0]]]]),self.functionspace)
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [2.0, 7.0, 1.0], [1.0, 5.0, 4.0]], [[8.0, 1.0, 5.0], [1.0, 5.0, 5.0], [8.0, 
3.0, 8.0]], [[5.0, 3.0, 5.0], [7.0, 3.0, 8.0], [7.0, 7.0, 1.0]]], [[[5.0, 4.0, 2.0], [1.0, 5.0, 5.0], [6.0, 2.0, 4.0]], [[5.0, 
5.0, 7.0], [5.0, 3.0, 3.0], [6.0, 2.0, 2.0]], [[8.0, 3.0, 7.0], [6.0, 3.0, 2.0], [3.0, 5.0, 2.0]]], [[[2.0, 4.0, 8.0], [1.0, 
6.0, 1.0], [2.0, 5.0, 2.0]], [[2.0, 2.0, 3.0], [5.0, 8.0, 6.0], [3.0, 6.0, 2.0]], [[3.0, 8.0, 7.0], [8.0, 3.0, 6.0], [3.0, 6.0, 
8.0]]], [[[3.0, 5.0, 1.0], [3.0, 7.0, 2.0], [1.0, 3.0, 8.0]], [[4.0, 2.0, 5.0], [4.0, 5.0, 8.0], [1.0, 1.0, 1.0]], [[4.0, 5.0, 
3.0], [3.0, 1.0, 8.0], [3.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 6.0], [7.0, 4.0, 7.0], [1.0, 1.0, 5.0]], 
[[4.0, 2.0, 6.0], [8.0, 2.0, 7.0], [2.0, 7.0, 4.0]], [[8.0, 4.0, 2.0], [3.0, 8.0, 5.0], [4.0, 5.0, 7.0]]], [[[3.0, 6.0, 3.0], 
[5.0, 7.0, 1.0], [8.0, 2.0, 3.0]], [[4.0, 8.0, 2.0], [3.0, 8.0, 2.0], [6.0, 4.0, 4.0]], [[5.0, 6.0, 7.0], [2.0, 3.0, 7.0], 
[4.0, 2.0, 4.0]]], [[[4.0, 1.0, 3.0], [8.0, 3.0, 7.0], [1.0, 6.0, 7.0]], [[3.0, 8.0, 3.0], [4.0, 1.0, 5.0], [8.0, 5.0, 4.0]], 
[[3.0, 8.0, 7.0], [8.0, 3.0, 6.0], [3.0, 6.0, 8.0]]], [[[8.0, 6.0, 8.0], [7.0, 4.0, 5.0], [5.0, 7.0, 2.0]], [[7.0, 7.0, 6.0], 
[4.0, 8.0, 5.0], [2.0, 8.0, 4.0]], [[4.0, 5.0, 3.0], [3.0, 1.0, 8.0], [3.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 7.0], [4.0, 8.0, 1.0], [3.0, 6.0, 2.0]], [[1.0, 5.0, 4.0], [3.0, 6.0, 7.0], [1.0, 
2.0, 1.0]], [[5.0, 2.0, 8.0], [1.0, 7.0, 4.0], [3.0, 4.0, 8.0]]], [[[2.0, 6.0, 3.0], [5.0, 3.0, 5.0], [7.0, 6.0, 6.0]], [[5.0, 
3.0, 7.0], [7.0, 7.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 7.0, 1.0], [6.0, 6.0, 4.0], [5.0, 7.0, 8.0]]], [[[6.0, 2.0, 7.0], [6.0, 
5.0, 1.0], [5.0, 7.0, 7.0]], [[1.0, 5.0, 3.0], [6.0, 2.0, 8.0], [2.0, 7.0, 7.0]], [[2.0, 1.0, 4.0], [1.0, 1.0, 6.0], [2.0, 6.0, 
3.0]]], [[[6.0, 3.0, 5.0], [1.0, 3.0, 4.0], [4.0, 6.0, 4.0]], [[1.0, 4.0, 8.0], [1.0, 1.0, 3.0], [6.0, 6.0, 3.0]], [[7.0, 8.0, 
7.0], [1.0, 1.0, 3.0], [1.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 6.0], [7.0, 3.0, 8.0], [1.0, 2.0, 7.0]], 
[[2.0, 6.0, 8.0], [7.0, 8.0, 6.0], [8.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [8.0, 7.0, 1.0], [5.0, 4.0, 3.0]]], [[[3.0, 8.0, 7.0], 
[5.0, 7.0, 3.0], [3.0, 7.0, 1.0]], [[3.0, 2.0, 5.0], [2.0, 2.0, 3.0], [2.0, 4.0, 3.0]], [[7.0, 4.0, 2.0], [1.0, 1.0, 3.0], 
[8.0, 5.0, 5.0]]], [[[4.0, 7.0, 1.0], [1.0, 2.0, 4.0], [1.0, 3.0, 6.0]], [[6.0, 2.0, 4.0], [5.0, 5.0, 5.0], [3.0, 1.0, 2.0]], 
[[1.0, 1.0, 5.0], [3.0, 6.0, 1.0], [7.0, 7.0, 3.0]]], [[[1.0, 7.0, 1.0], [4.0, 4.0, 7.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 5.0], 
[4.0, 3.0, 1.0], [1.0, 3.0, 6.0]], [[5.0, 5.0, 1.0], [1.0, 7.0, 2.0], [1.0, 2.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 7.0, 7.0], [2.0, 6.0, 7.0], [4.0, 3.0, 1.0]]], [[[5.0, 5.0, 4.0], [3.0, 2.0, 5.0], [2.0, 
7.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 1.0, 5.0], [4.0, 5.0, 8.0], [7.0, 4.0, 7.0]]], [[[4.0, 4.0, 4.0], [6.0, 5.0, 
5.0], [1.0, 6.0, 5.0]]]])
      arg[2:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 7.0], [4.0, 8.0, 1.0], [3.0, 6.0, 2.0]], [[1.0, 5.0, 4.0], [3.0, 6.0, 7.0], [1.0, 
2.0, 1.0]], [[5.0, 2.0, 8.0], [1.0, 7.0, 4.0], [3.0, 4.0, 8.0]]], [[[2.0, 6.0, 3.0], [5.0, 3.0, 5.0], [7.0, 6.0, 6.0]], [[5.0, 
3.0, 7.0], [7.0, 7.0, 6.0], [1.0, 2.0, 4.0]], [[1.0, 7.0, 1.0], [6.0, 6.0, 4.0], [5.0, 7.0, 8.0]]], [[[6.0, 2.0, 7.0], [6.0, 
5.0, 1.0], [5.0, 7.0, 7.0]], [[1.0, 5.0, 3.0], [6.0, 2.0, 8.0], [2.0, 7.0, 7.0]], [[2.0, 7.0, 7.0], [2.0, 6.0, 7.0], [4.0, 3.0, 
1.0]]], [[[6.0, 3.0, 5.0], [1.0, 3.0, 4.0], [4.0, 6.0, 4.0]], [[1.0, 4.0, 8.0], [1.0, 1.0, 3.0], [6.0, 6.0, 3.0]], [[5.0, 5.0, 
4.0], [3.0, 2.0, 5.0], [2.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 6.0], [7.0, 3.0, 8.0], [1.0, 2.0, 7.0]], 
[[2.0, 6.0, 8.0], [7.0, 8.0, 6.0], [8.0, 5.0, 6.0]], [[1.0, 6.0, 4.0], [8.0, 7.0, 1.0], [5.0, 4.0, 3.0]]], [[[3.0, 8.0, 7.0], 
[5.0, 7.0, 3.0], [3.0, 7.0, 1.0]], [[3.0, 2.0, 5.0], [2.0, 2.0, 3.0], [2.0, 4.0, 3.0]], [[7.0, 4.0, 2.0], [1.0, 1.0, 3.0], 
[8.0, 5.0, 5.0]]], [[[4.0, 7.0, 1.0], [1.0, 2.0, 4.0], [1.0, 3.0, 6.0]], [[6.0, 2.0, 4.0], [5.0, 5.0, 5.0], [3.0, 1.0, 2.0]], 
[[8.0, 1.0, 5.0], [4.0, 5.0, 8.0], [7.0, 4.0, 7.0]]], [[[1.0, 7.0, 1.0], [4.0, 4.0, 7.0], [5.0, 3.0, 5.0]], [[5.0, 2.0, 5.0], 
[4.0, 3.0, 1.0], [1.0, 3.0, 6.0]], [[4.0, 4.0, 4.0], [6.0, 5.0, 5.0], [1.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 1.0], [3.0, 3.0, 1.0], [3.0, 3.0, 3.0]], [[8.0, 7.0, 1.0], [5.0, 8.0, 8.0], [1.0, 
3.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 8.0, 4.0], [4.0, 4.0, 4.0]]], [[[2.0, 5.0, 1.0], [5.0, 3.0, 4.0], [3.0, 3.0, 3.0]], [[2.0, 
4.0, 4.0], [2.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 4.0, 2.0], [2.0, 1.0, 8.0], [7.0, 1.0, 1.0]]], [[[8.0, 6.0, 2.0], [2.0, 
5.0, 7.0], [3.0, 7.0, 3.0]], [[2.0, 4.0, 6.0], [5.0, 7.0, 2.0], [3.0, 1.0, 3.0]], [[6.0, 5.0, 3.0], [4.0, 4.0, 2.0], [7.0, 3.0, 
8.0]]], [[[2.0, 4.0, 6.0], [8.0, 1.0, 6.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [6.0, 6.0, 1.0], [3.0, 8.0, 1.0]], [[3.0, 7.0, 
7.0], [8.0, 7.0, 1.0], [7.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 7.0, 8.0], [4.0, 2.0, 8.0], [6.0, 8.0, 1.0]], 
[[3.0, 6.0, 3.0], [1.0, 2.0, 3.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 1.0], [6.0, 7.0, 1.0], [6.0, 3.0, 8.0]]], [[[5.0, 2.0, 1.0], 
[1.0, 3.0, 4.0], [8.0, 4.0, 8.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[5.0, 4.0, 8.0], [8.0, 4.0, 7.0], 
[5.0, 3.0, 6.0]]], [[[7.0, 6.0, 1.0], [3.0, 4.0, 4.0], [1.0, 7.0, 2.0]], [[5.0, 8.0, 8.0], [4.0, 7.0, 1.0], [4.0, 6.0, 5.0]], 
[[8.0, 2.0, 3.0], [1.0, 1.0, 3.0], [3.0, 1.0, 7.0]]], [[[7.0, 2.0, 6.0], [2.0, 4.0, 8.0], [7.0, 1.0, 7.0]], [[7.0, 1.0, 5.0], 
[2.0, 3.0, 2.0], [7.0, 6.0, 1.0]], [[6.0, 7.0, 8.0], [1.0, 4.0, 5.0], [6.0, 8.0, 1.0]]]])
      val=numpy.array([[[[1.0, 8.0, 2.0], [6.0, 7.0, 7.0]]], [[[7.0, 1.0, 1.0], [4.0, 4.0, 3.0]]]])
      arg[2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 1.0], [3.0, 3.0, 1.0], [3.0, 3.0, 3.0]], [[8.0, 7.0, 1.0], [5.0, 8.0, 8.0], [1.0, 
3.0, 4.0]], [[5.0, 2.0, 6.0], [2.0, 8.0, 4.0], [4.0, 4.0, 4.0]]], [[[2.0, 5.0, 1.0], [5.0, 3.0, 4.0], [3.0, 3.0, 3.0]], [[2.0, 
4.0, 4.0], [2.0, 7.0, 4.0], [5.0, 8.0, 8.0]], [[7.0, 4.0, 2.0], [2.0, 1.0, 8.0], [7.0, 1.0, 1.0]]], [[[8.0, 6.0, 2.0], [2.0, 
5.0, 7.0], [3.0, 7.0, 3.0]], [[2.0, 4.0, 6.0], [5.0, 7.0, 2.0], [3.0, 1.0, 3.0]], [[1.0, 8.0, 2.0], [6.0, 7.0, 7.0], [7.0, 3.0, 
8.0]]], [[[2.0, 4.0, 6.0], [8.0, 1.0, 6.0], [7.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [6.0, 6.0, 1.0], [3.0, 8.0, 1.0]], [[7.0, 1.0, 
1.0], [4.0, 4.0, 3.0], [7.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 8.0], [4.0, 2.0, 8.0], [6.0, 8.0, 1.0]], 
[[3.0, 6.0, 3.0], [1.0, 2.0, 3.0], [5.0, 1.0, 6.0]], [[6.0, 8.0, 1.0], [6.0, 7.0, 1.0], [6.0, 3.0, 8.0]]], [[[5.0, 2.0, 1.0], 
[1.0, 3.0, 4.0], [8.0, 4.0, 8.0]], [[6.0, 8.0, 4.0], [7.0, 4.0, 4.0], [6.0, 5.0, 1.0]], [[5.0, 4.0, 8.0], [8.0, 4.0, 7.0], 
[5.0, 3.0, 6.0]]], [[[7.0, 6.0, 1.0], [3.0, 4.0, 4.0], [1.0, 7.0, 2.0]], [[5.0, 8.0, 8.0], [4.0, 7.0, 1.0], [4.0, 6.0, 5.0]], 
[[1.0, 8.0, 2.0], [6.0, 7.0, 7.0], [3.0, 1.0, 7.0]]], [[[7.0, 2.0, 6.0], [2.0, 4.0, 8.0], [7.0, 1.0, 7.0]], [[7.0, 1.0, 5.0], 
[2.0, 3.0, 2.0], [7.0, 6.0, 1.0]], [[7.0, 1.0, 1.0], [4.0, 4.0, 3.0], [6.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 1.0], [2.0, 8.0, 4.0], [6.0, 1.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 2.0], [7.0, 
5.0, 2.0]], [[2.0, 7.0, 3.0], [2.0, 8.0, 6.0], [5.0, 4.0, 6.0]]], [[[8.0, 1.0, 4.0], [8.0, 8.0, 6.0], [4.0, 4.0, 8.0]], [[7.0, 
8.0, 2.0], [5.0, 4.0, 7.0], [1.0, 6.0, 7.0]], [[7.0, 5.0, 6.0], [7.0, 3.0, 4.0], [3.0, 6.0, 2.0]]], [[[4.0, 4.0, 7.0], [7.0, 
5.0, 7.0], [1.0, 2.0, 1.0]], [[7.0, 5.0, 8.0], [3.0, 8.0, 6.0], [1.0, 2.0, 1.0]], [[1.0, 4.0, 3.0], [3.0, 6.0, 1.0], [7.0, 8.0, 
4.0]]], [[[3.0, 5.0, 4.0], [4.0, 2.0, 3.0], [7.0, 3.0, 2.0]], [[2.0, 2.0, 8.0], [8.0, 8.0, 5.0], [4.0, 8.0, 8.0]], [[3.0, 5.0, 
1.0], [4.0, 7.0, 1.0], [8.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 1.0], [5.0, 8.0, 4.0], [5.0, 8.0, 3.0]], 
[[3.0, 3.0, 6.0], [5.0, 8.0, 2.0], [7.0, 8.0, 2.0]], [[5.0, 2.0, 7.0], [7.0, 7.0, 7.0], [1.0, 3.0, 1.0]]], [[[3.0, 6.0, 6.0], 
[7.0, 4.0, 3.0], [8.0, 1.0, 1.0]], [[3.0, 3.0, 2.0], [7.0, 4.0, 8.0], [5.0, 7.0, 4.0]], [[2.0, 1.0, 3.0], [4.0, 3.0, 5.0], 
[2.0, 2.0, 1.0]]], [[[4.0, 8.0, 1.0], [8.0, 4.0, 4.0], [4.0, 4.0, 2.0]], [[8.0, 7.0, 3.0], [7.0, 8.0, 6.0], [1.0, 3.0, 6.0]], 
[[8.0, 5.0, 7.0], [5.0, 3.0, 6.0], [4.0, 6.0, 3.0]]], [[[6.0, 6.0, 3.0], [6.0, 5.0, 4.0], [5.0, 2.0, 1.0]], [[3.0, 6.0, 1.0], 
[5.0, 4.0, 1.0], [7.0, 4.0, 8.0]], [[2.0, 4.0, 7.0], [1.0, 7.0, 3.0], [4.0, 5.0, 4.0]]]])
      val=Data(numpy.array([[[[5.0, 1.0, 2.0], [5.0, 5.0, 1.0]]], [[[2.0, 1.0, 4.0], [7.0, 2.0, 8.0]]]]),self.functionspace)
      arg[2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 1.0], [2.0, 8.0, 4.0], [6.0, 1.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 4.0, 2.0], [7.0, 
5.0, 2.0]], [[2.0, 7.0, 3.0], [2.0, 8.0, 6.0], [5.0, 4.0, 6.0]]], [[[8.0, 1.0, 4.0], [8.0, 8.0, 6.0], [4.0, 4.0, 8.0]], [[7.0, 
8.0, 2.0], [5.0, 4.0, 7.0], [1.0, 6.0, 7.0]], [[7.0, 5.0, 6.0], [7.0, 3.0, 4.0], [3.0, 6.0, 2.0]]], [[[4.0, 4.0, 7.0], [7.0, 
5.0, 7.0], [1.0, 2.0, 1.0]], [[7.0, 5.0, 8.0], [3.0, 8.0, 6.0], [1.0, 2.0, 1.0]], [[5.0, 1.0, 2.0], [5.0, 5.0, 1.0], [7.0, 8.0, 
4.0]]], [[[3.0, 5.0, 4.0], [4.0, 2.0, 3.0], [7.0, 3.0, 2.0]], [[2.0, 2.0, 8.0], [8.0, 8.0, 5.0], [4.0, 8.0, 8.0]], [[2.0, 1.0, 
4.0], [7.0, 2.0, 8.0], [8.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 1.0], [5.0, 8.0, 4.0], [5.0, 8.0, 3.0]], 
[[3.0, 3.0, 6.0], [5.0, 8.0, 2.0], [7.0, 8.0, 2.0]], [[5.0, 2.0, 7.0], [7.0, 7.0, 7.0], [1.0, 3.0, 1.0]]], [[[3.0, 6.0, 6.0], 
[7.0, 4.0, 3.0], [8.0, 1.0, 1.0]], [[3.0, 3.0, 2.0], [7.0, 4.0, 8.0], [5.0, 7.0, 4.0]], [[2.0, 1.0, 3.0], [4.0, 3.0, 5.0], 
[2.0, 2.0, 1.0]]], [[[4.0, 8.0, 1.0], [8.0, 4.0, 4.0], [4.0, 4.0, 2.0]], [[8.0, 7.0, 3.0], [7.0, 8.0, 6.0], [1.0, 3.0, 6.0]], 
[[5.0, 1.0, 2.0], [5.0, 5.0, 1.0], [4.0, 6.0, 3.0]]], [[[6.0, 6.0, 3.0], [6.0, 5.0, 4.0], [5.0, 2.0, 1.0]], [[3.0, 6.0, 1.0], 
[5.0, 4.0, 1.0], [7.0, 4.0, 8.0]], [[2.0, 1.0, 4.0], [7.0, 2.0, 8.0], [4.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 6.0], [7.0, 8.0, 8.0], [6.0, 8.0, 3.0]], [[5.0, 2.0, 7.0], [8.0, 3.0, 2.0], [3.0, 
1.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 8.0, 3.0], [5.0, 7.0, 8.0]]], [[[5.0, 8.0, 2.0], [8.0, 5.0, 6.0], [6.0, 3.0, 6.0]], [[2.0, 
8.0, 8.0], [5.0, 2.0, 1.0], [1.0, 2.0, 7.0]], [[6.0, 8.0, 5.0], [4.0, 4.0, 6.0], [3.0, 2.0, 2.0]]], [[[8.0, 3.0, 1.0], [2.0, 
6.0, 4.0], [6.0, 4.0, 3.0]], [[6.0, 2.0, 8.0], [5.0, 1.0, 8.0], [6.0, 1.0, 7.0]], [[5.0, 2.0, 8.0], [8.0, 3.0, 2.0], [6.0, 2.0, 
6.0]]], [[[2.0, 1.0, 5.0], [4.0, 6.0, 2.0], [5.0, 8.0, 5.0]], [[8.0, 8.0, 3.0], [7.0, 6.0, 6.0], [4.0, 5.0, 4.0]], [[1.0, 8.0, 
2.0], [4.0, 1.0, 1.0], [8.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 4.0, 1.0], [1.0, 1.0, 3.0], [8.0, 2.0, 4.0]], 
[[3.0, 1.0, 2.0], [7.0, 1.0, 7.0], [5.0, 4.0, 8.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 1.0], [3.0, 5.0, 4.0]]], [[[4.0, 4.0, 6.0], 
[3.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[8.0, 8.0, 6.0], [3.0, 3.0, 2.0], [4.0, 8.0, 5.0]], [[4.0, 8.0, 4.0], [7.0, 5.0, 1.0], 
[6.0, 2.0, 1.0]]], [[[3.0, 1.0, 8.0], [4.0, 6.0, 4.0], [3.0, 1.0, 4.0]], [[1.0, 6.0, 6.0], [2.0, 5.0, 5.0], [3.0, 5.0, 7.0]], 
[[4.0, 7.0, 7.0], [5.0, 3.0, 1.0], [4.0, 8.0, 4.0]]], [[[7.0, 8.0, 4.0], [4.0, 7.0, 8.0], [6.0, 6.0, 3.0]], [[6.0, 5.0, 2.0], 
[3.0, 7.0, 5.0], [1.0, 7.0, 2.0]], [[3.0, 7.0, 5.0], [5.0, 3.0, 8.0], [5.0, 8.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 5.0, 6.0], [3.0, 3.0, 7.0]]], [[[7.0, 1.0, 1.0], [5.0, 7.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 4.0, 3.0], [8.0, 4.0, 3.0]]], [[[5.0, 6.0, 5.0], [3.0, 5.0, 6.0]]]])
      arg[2:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 6.0], [7.0, 8.0, 8.0], [6.0, 8.0, 3.0]], [[5.0, 2.0, 7.0], [8.0, 3.0, 2.0], [3.0, 
1.0, 4.0]], [[3.0, 2.0, 2.0], [1.0, 8.0, 3.0], [5.0, 7.0, 8.0]]], [[[5.0, 8.0, 2.0], [8.0, 5.0, 6.0], [6.0, 3.0, 6.0]], [[2.0, 
8.0, 8.0], [5.0, 2.0, 1.0], [1.0, 2.0, 7.0]], [[6.0, 8.0, 5.0], [4.0, 4.0, 6.0], [3.0, 2.0, 2.0]]], [[[8.0, 3.0, 1.0], [2.0, 
6.0, 4.0], [6.0, 4.0, 3.0]], [[6.0, 2.0, 8.0], [5.0, 1.0, 8.0], [6.0, 1.0, 7.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 7.0], [6.0, 2.0, 
6.0]]], [[[2.0, 1.0, 5.0], [4.0, 6.0, 2.0], [5.0, 8.0, 5.0]], [[8.0, 8.0, 3.0], [7.0, 6.0, 6.0], [4.0, 5.0, 4.0]], [[7.0, 1.0, 
1.0], [5.0, 7.0, 5.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 4.0, 1.0], [1.0, 1.0, 3.0], [8.0, 2.0, 4.0]], 
[[3.0, 1.0, 2.0], [7.0, 1.0, 7.0], [5.0, 4.0, 8.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 1.0], [3.0, 5.0, 4.0]]], [[[4.0, 4.0, 6.0], 
[3.0, 6.0, 4.0], [7.0, 6.0, 7.0]], [[8.0, 8.0, 6.0], [3.0, 3.0, 2.0], [4.0, 8.0, 5.0]], [[4.0, 8.0, 4.0], [7.0, 5.0, 1.0], 
[6.0, 2.0, 1.0]]], [[[3.0, 1.0, 8.0], [4.0, 6.0, 4.0], [3.0, 1.0, 4.0]], [[1.0, 6.0, 6.0], [2.0, 5.0, 5.0], [3.0, 5.0, 7.0]], 
[[6.0, 4.0, 3.0], [8.0, 4.0, 3.0], [4.0, 8.0, 4.0]]], [[[7.0, 8.0, 4.0], [4.0, 7.0, 8.0], [6.0, 6.0, 3.0]], [[6.0, 5.0, 2.0], 
[3.0, 7.0, 5.0], [1.0, 7.0, 2.0]], [[5.0, 6.0, 5.0], [3.0, 5.0, 6.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 5.0], [4.0, 4.0, 2.0], [1.0, 7.0, 5.0]], [[1.0, 8.0, 3.0], [5.0, 8.0, 3.0], [1.0, 
7.0, 5.0]], [[7.0, 4.0, 8.0], [8.0, 3.0, 5.0], [1.0, 6.0, 4.0]]], [[[1.0, 7.0, 7.0], [3.0, 4.0, 6.0], [7.0, 7.0, 8.0]], [[3.0, 
2.0, 7.0], [8.0, 6.0, 1.0], [4.0, 2.0, 1.0]], [[4.0, 6.0, 3.0], [4.0, 6.0, 6.0], [1.0, 4.0, 6.0]]], [[[7.0, 2.0, 8.0], [3.0, 
2.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 8.0, 1.0], [3.0, 1.0, 1.0], [4.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], [5.0, 1.0, 2.0], [4.0, 7.0, 
6.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 8.0, 7.0], [1.0, 8.0, 7.0], [6.0, 1.0, 8.0]], [[7.0, 3.0, 
7.0], [2.0, 6.0, 5.0], [2.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 4.0, 3.0], [7.0, 3.0, 4.0]], 
[[3.0, 5.0, 6.0], [2.0, 4.0, 6.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 8.0], [8.0, 5.0, 4.0], [3.0, 5.0, 7.0]]], [[[7.0, 4.0, 8.0], 
[8.0, 3.0, 7.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 6.0], [8.0, 2.0, 3.0], [8.0, 5.0, 2.0]], [[7.0, 3.0, 4.0], [8.0, 6.0, 4.0], 
[7.0, 8.0, 5.0]]], [[[1.0, 8.0, 7.0], [4.0, 4.0, 2.0], [2.0, 8.0, 1.0]], [[4.0, 6.0, 6.0], [7.0, 5.0, 3.0], [5.0, 8.0, 2.0]], 
[[5.0, 4.0, 2.0], [4.0, 4.0, 2.0], [6.0, 7.0, 8.0]]], [[[2.0, 3.0, 8.0], [2.0, 1.0, 6.0], [2.0, 4.0, 6.0]], [[3.0, 2.0, 1.0], 
[5.0, 7.0, 1.0], [1.0, 3.0, 7.0]], [[7.0, 4.0, 4.0], [3.0, 1.0, 1.0], [4.0, 5.0, 1.0]]]])
      val=numpy.array([[[7.0, 1.0, 4.0]], [[1.0, 4.0, 8.0]], [[8.0, 1.0, 6.0]]])
      arg[:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 5.0], [4.0, 4.0, 2.0], [1.0, 7.0, 5.0]], [[1.0, 8.0, 3.0], [7.0, 1.0, 4.0], [1.0, 
7.0, 5.0]], [[7.0, 4.0, 8.0], [8.0, 3.0, 5.0], [1.0, 6.0, 4.0]]], [[[1.0, 7.0, 7.0], [3.0, 4.0, 6.0], [7.0, 7.0, 8.0]], [[3.0, 
2.0, 7.0], [1.0, 4.0, 8.0], [4.0, 2.0, 1.0]], [[4.0, 6.0, 3.0], [4.0, 6.0, 6.0], [1.0, 4.0, 6.0]]], [[[7.0, 2.0, 8.0], [3.0, 
2.0, 6.0], [3.0, 2.0, 3.0]], [[1.0, 8.0, 1.0], [8.0, 1.0, 6.0], [4.0, 2.0, 6.0]], [[8.0, 7.0, 4.0], [5.0, 1.0, 2.0], [4.0, 7.0, 
6.0]]], [[[3.0, 8.0, 5.0], [3.0, 7.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 8.0, 7.0], [1.0, 8.0, 7.0], [6.0, 1.0, 8.0]], [[7.0, 3.0, 
7.0], [2.0, 6.0, 5.0], [2.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 4.0, 3.0], [7.0, 3.0, 4.0]], 
[[3.0, 5.0, 6.0], [7.0, 1.0, 4.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 8.0], [8.0, 5.0, 4.0], [3.0, 5.0, 7.0]]], [[[7.0, 4.0, 8.0], 
[8.0, 3.0, 7.0], [4.0, 4.0, 1.0]], [[5.0, 8.0, 6.0], [1.0, 4.0, 8.0], [8.0, 5.0, 2.0]], [[7.0, 3.0, 4.0], [8.0, 6.0, 4.0], 
[7.0, 8.0, 5.0]]], [[[1.0, 8.0, 7.0], [4.0, 4.0, 2.0], [2.0, 8.0, 1.0]], [[4.0, 6.0, 6.0], [8.0, 1.0, 6.0], [5.0, 8.0, 2.0]], 
[[5.0, 4.0, 2.0], [4.0, 4.0, 2.0], [6.0, 7.0, 8.0]]], [[[2.0, 3.0, 8.0], [2.0, 1.0, 6.0], [2.0, 4.0, 6.0]], [[3.0, 2.0, 1.0], 
[5.0, 7.0, 1.0], [1.0, 3.0, 7.0]], [[7.0, 4.0, 4.0], [3.0, 1.0, 1.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 8.0], [5.0, 4.0, 2.0], [7.0, 6.0, 5.0]], [[4.0, 2.0, 5.0], [8.0, 3.0, 7.0], [2.0, 
1.0, 2.0]], [[2.0, 3.0, 7.0], [8.0, 5.0, 6.0], [1.0, 2.0, 8.0]]], [[[7.0, 1.0, 8.0], [5.0, 1.0, 7.0], [8.0, 4.0, 7.0]], [[4.0, 
2.0, 3.0], [7.0, 8.0, 8.0], [8.0, 2.0, 5.0]], [[4.0, 1.0, 3.0], [3.0, 7.0, 4.0], [3.0, 7.0, 5.0]]], [[[4.0, 3.0, 8.0], [6.0, 
1.0, 1.0], [5.0, 2.0, 1.0]], [[8.0, 4.0, 7.0], [2.0, 1.0, 6.0], [4.0, 4.0, 1.0]], [[3.0, 2.0, 6.0], [7.0, 7.0, 4.0], [4.0, 1.0, 
4.0]]], [[[8.0, 7.0, 3.0], [4.0, 7.0, 8.0], [4.0, 6.0, 6.0]], [[1.0, 8.0, 5.0], [6.0, 1.0, 5.0], [3.0, 5.0, 5.0]], [[8.0, 3.0, 
7.0], [2.0, 3.0, 4.0], [7.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 1.0], [5.0, 8.0, 7.0], [7.0, 3.0, 8.0]], 
[[1.0, 7.0, 7.0], [6.0, 5.0, 7.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 3.0], [5.0, 4.0, 6.0], [6.0, 5.0, 8.0]]], [[[3.0, 3.0, 5.0], 
[5.0, 5.0, 2.0], [3.0, 6.0, 2.0]], [[8.0, 4.0, 1.0], [7.0, 5.0, 6.0], [7.0, 6.0, 3.0]], [[1.0, 4.0, 6.0], [3.0, 2.0, 5.0], 
[6.0, 4.0, 5.0]]], [[[2.0, 4.0, 3.0], [4.0, 1.0, 7.0], [8.0, 4.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 1.0, 8.0], [8.0, 5.0, 4.0]], 
[[6.0, 7.0, 7.0], [6.0, 5.0, 2.0], [1.0, 5.0, 3.0]]], [[[6.0, 2.0, 2.0], [3.0, 6.0, 2.0], [5.0, 7.0, 5.0]], [[1.0, 5.0, 4.0], 
[2.0, 1.0, 2.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 4.0], [6.0, 3.0, 5.0], [3.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[[2.0, 6.0, 7.0]], [[8.0, 6.0, 4.0]], [[6.0, 5.0, 6.0]]]),self.functionspace)
      arg[:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 8.0], [5.0, 4.0, 2.0], [7.0, 6.0, 5.0]], [[4.0, 2.0, 5.0], [2.0, 6.0, 7.0], [2.0, 
1.0, 2.0]], [[2.0, 3.0, 7.0], [8.0, 5.0, 6.0], [1.0, 2.0, 8.0]]], [[[7.0, 1.0, 8.0], [5.0, 1.0, 7.0], [8.0, 4.0, 7.0]], [[4.0, 
2.0, 3.0], [8.0, 6.0, 4.0], [8.0, 2.0, 5.0]], [[4.0, 1.0, 3.0], [3.0, 7.0, 4.0], [3.0, 7.0, 5.0]]], [[[4.0, 3.0, 8.0], [6.0, 
1.0, 1.0], [5.0, 2.0, 1.0]], [[8.0, 4.0, 7.0], [6.0, 5.0, 6.0], [4.0, 4.0, 1.0]], [[3.0, 2.0, 6.0], [7.0, 7.0, 4.0], [4.0, 1.0, 
4.0]]], [[[8.0, 7.0, 3.0], [4.0, 7.0, 8.0], [4.0, 6.0, 6.0]], [[1.0, 8.0, 5.0], [6.0, 1.0, 5.0], [3.0, 5.0, 5.0]], [[8.0, 3.0, 
7.0], [2.0, 3.0, 4.0], [7.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 1.0], [5.0, 8.0, 7.0], [7.0, 3.0, 8.0]], 
[[1.0, 7.0, 7.0], [2.0, 6.0, 7.0], [6.0, 3.0, 2.0]], [[7.0, 6.0, 3.0], [5.0, 4.0, 6.0], [6.0, 5.0, 8.0]]], [[[3.0, 3.0, 5.0], 
[5.0, 5.0, 2.0], [3.0, 6.0, 2.0]], [[8.0, 4.0, 1.0], [8.0, 6.0, 4.0], [7.0, 6.0, 3.0]], [[1.0, 4.0, 6.0], [3.0, 2.0, 5.0], 
[6.0, 4.0, 5.0]]], [[[2.0, 4.0, 3.0], [4.0, 1.0, 7.0], [8.0, 4.0, 7.0]], [[6.0, 5.0, 3.0], [6.0, 5.0, 6.0], [8.0, 5.0, 4.0]], 
[[6.0, 7.0, 7.0], [6.0, 5.0, 2.0], [1.0, 5.0, 3.0]]], [[[6.0, 2.0, 2.0], [3.0, 6.0, 2.0], [5.0, 7.0, 5.0]], [[1.0, 5.0, 4.0], 
[2.0, 1.0, 2.0], [2.0, 3.0, 4.0]], [[4.0, 8.0, 4.0], [6.0, 3.0, 5.0], [3.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 5.0], [6.0, 1.0, 3.0], [7.0, 8.0, 7.0]], [[5.0, 4.0, 6.0], [1.0, 5.0, 6.0], [1.0, 
7.0, 8.0]], [[4.0, 4.0, 1.0], [6.0, 6.0, 1.0], [4.0, 8.0, 4.0]]], [[[2.0, 8.0, 3.0], [4.0, 6.0, 3.0], [6.0, 6.0, 7.0]], [[8.0, 
3.0, 4.0], [1.0, 1.0, 3.0], [2.0, 6.0, 4.0]], [[2.0, 4.0, 8.0], [2.0, 7.0, 2.0], [2.0, 1.0, 6.0]]], [[[8.0, 4.0, 5.0], [2.0, 
4.0, 7.0], [4.0, 1.0, 3.0]], [[5.0, 4.0, 3.0], [1.0, 5.0, 3.0], [8.0, 2.0, 1.0]], [[1.0, 3.0, 8.0], [3.0, 3.0, 7.0], [3.0, 8.0, 
8.0]]], [[[1.0, 7.0, 1.0], [5.0, 7.0, 6.0], [2.0, 4.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 5.0, 4.0], [3.0, 2.0, 1.0]], [[2.0, 6.0, 
7.0], [5.0, 3.0, 3.0], [8.0, 3.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 1.0], [1.0, 2.0, 2.0], [2.0, 8.0, 4.0]], 
[[3.0, 4.0, 3.0], [3.0, 2.0, 7.0], [3.0, 2.0, 7.0]], [[7.0, 8.0, 8.0], [5.0, 7.0, 4.0], [6.0, 8.0, 1.0]]], [[[8.0, 3.0, 1.0], 
[5.0, 2.0, 8.0], [7.0, 3.0, 8.0]], [[6.0, 3.0, 2.0], [1.0, 6.0, 5.0], [6.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [2.0, 2.0, 4.0], 
[6.0, 2.0, 1.0]]], [[[6.0, 6.0, 4.0], [4.0, 8.0, 2.0], [2.0, 3.0, 3.0]], [[3.0, 8.0, 5.0], [2.0, 1.0, 1.0], [4.0, 7.0, 2.0]], 
[[1.0, 5.0, 3.0], [2.0, 4.0, 1.0], [7.0, 5.0, 7.0]]], [[[8.0, 3.0, 4.0], [7.0, 6.0, 4.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 5.0], 
[7.0, 7.0, 8.0], [7.0, 6.0, 1.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 7.0], [7.0, 5.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 5.0, 7.0]], [[4.0, 8.0, 2.0]], [[6.0, 8.0, 2.0]]])+(1.-msk_val)*numpy.array([[[1.0, 4.0, 
5.0]], [[3.0, 7.0, 7.0]], [[4.0, 1.0, 2.0]]])
      arg[:3,1:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 5.0], [6.0, 1.0, 3.0], [7.0, 8.0, 7.0]], [[5.0, 4.0, 6.0], [2.0, 5.0, 7.0], [1.0, 
7.0, 8.0]], [[4.0, 4.0, 1.0], [6.0, 6.0, 1.0], [4.0, 8.0, 4.0]]], [[[2.0, 8.0, 3.0], [4.0, 6.0, 3.0], [6.0, 6.0, 7.0]], [[8.0, 
3.0, 4.0], [4.0, 8.0, 2.0], [2.0, 6.0, 4.0]], [[2.0, 4.0, 8.0], [2.0, 7.0, 2.0], [2.0, 1.0, 6.0]]], [[[8.0, 4.0, 5.0], [2.0, 
4.0, 7.0], [4.0, 1.0, 3.0]], [[5.0, 4.0, 3.0], [6.0, 8.0, 2.0], [8.0, 2.0, 1.0]], [[1.0, 3.0, 8.0], [3.0, 3.0, 7.0], [3.0, 8.0, 
8.0]]], [[[1.0, 7.0, 1.0], [5.0, 7.0, 6.0], [2.0, 4.0, 5.0]], [[8.0, 7.0, 2.0], [2.0, 5.0, 4.0], [3.0, 2.0, 1.0]], [[2.0, 6.0, 
7.0], [5.0, 3.0, 3.0], [8.0, 3.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 1.0], [1.0, 2.0, 2.0], [2.0, 8.0, 4.0]], 
[[3.0, 4.0, 3.0], [1.0, 4.0, 5.0], [3.0, 2.0, 7.0]], [[7.0, 8.0, 8.0], [5.0, 7.0, 4.0], [6.0, 8.0, 1.0]]], [[[8.0, 3.0, 1.0], 
[5.0, 2.0, 8.0], [7.0, 3.0, 8.0]], [[6.0, 3.0, 2.0], [3.0, 7.0, 7.0], [6.0, 1.0, 8.0]], [[5.0, 1.0, 8.0], [2.0, 2.0, 4.0], 
[6.0, 2.0, 1.0]]], [[[6.0, 6.0, 4.0], [4.0, 8.0, 2.0], [2.0, 3.0, 3.0]], [[3.0, 8.0, 5.0], [4.0, 1.0, 2.0], [4.0, 7.0, 2.0]], 
[[1.0, 5.0, 3.0], [2.0, 4.0, 1.0], [7.0, 5.0, 7.0]]], [[[8.0, 3.0, 4.0], [7.0, 6.0, 4.0], [7.0, 2.0, 8.0]], [[6.0, 1.0, 5.0], 
[7.0, 7.0, 8.0], [7.0, 6.0, 1.0]], [[4.0, 8.0, 8.0], [2.0, 1.0, 7.0], [7.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 1.0], [7.0, 7.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 7.0, 6.0], [4.0, 2.0, 7.0], [1.0, 
6.0, 4.0]], [[4.0, 3.0, 7.0], [4.0, 7.0, 7.0], [1.0, 5.0, 4.0]]], [[[8.0, 5.0, 1.0], [6.0, 6.0, 8.0], [5.0, 1.0, 8.0]], [[1.0, 
1.0, 7.0], [7.0, 5.0, 7.0], [4.0, 6.0, 3.0]], [[1.0, 5.0, 5.0], [8.0, 7.0, 1.0], [5.0, 1.0, 1.0]]], [[[3.0, 8.0, 7.0], [3.0, 
1.0, 2.0], [5.0, 6.0, 3.0]], [[7.0, 3.0, 3.0], [1.0, 6.0, 4.0], [2.0, 3.0, 3.0]], [[7.0, 7.0, 8.0], [5.0, 4.0, 3.0], [3.0, 5.0, 
8.0]]], [[[5.0, 3.0, 5.0], [4.0, 2.0, 2.0], [2.0, 3.0, 3.0]], [[8.0, 2.0, 4.0], [5.0, 2.0, 5.0], [2.0, 8.0, 2.0]], [[1.0, 6.0, 
1.0], [5.0, 2.0, 1.0], [7.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 3.0], [2.0, 6.0, 6.0], [4.0, 7.0, 4.0]], 
[[8.0, 4.0, 4.0], [2.0, 3.0, 1.0], [4.0, 8.0, 8.0]], [[7.0, 1.0, 6.0], [1.0, 5.0, 8.0], [5.0, 2.0, 3.0]]], [[[3.0, 8.0, 6.0], 
[2.0, 5.0, 6.0], [4.0, 1.0, 3.0]], [[3.0, 6.0, 7.0], [2.0, 8.0, 3.0], [7.0, 4.0, 5.0]], [[5.0, 8.0, 8.0], [8.0, 3.0, 3.0], 
[3.0, 3.0, 3.0]]], [[[1.0, 2.0, 5.0], [4.0, 1.0, 8.0], [1.0, 1.0, 7.0]], [[5.0, 7.0, 8.0], [7.0, 2.0, 4.0], [5.0, 1.0, 7.0]], 
[[3.0, 4.0, 1.0], [8.0, 2.0, 5.0], [5.0, 1.0, 7.0]]], [[[5.0, 3.0, 5.0], [5.0, 1.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 6.0, 7.0], 
[7.0, 8.0, 5.0], [5.0, 8.0, 5.0]], [[4.0, 3.0, 4.0], [6.0, 4.0, 7.0], [8.0, 2.0, 6.0]]]])
      val=numpy.array([[[[8.0, 2.0, 2.0], [1.0, 7.0, 5.0], [7.0, 3.0, 8.0]]], [[[3.0, 8.0, 3.0], [7.0, 6.0, 3.0], [1.0, 4.0, 
5.0]]], [[[1.0, 1.0, 4.0], [7.0, 7.0, 4.0], [2.0, 7.0, 3.0]]], [[[5.0, 4.0, 1.0], [6.0, 8.0, 3.0], [7.0, 2.0, 7.0]]]])
      arg[:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 1.0], [7.0, 7.0, 3.0], [7.0, 1.0, 3.0]], [[8.0, 2.0, 2.0], [1.0, 7.0, 5.0], [7.0, 
3.0, 8.0]], [[4.0, 3.0, 7.0], [4.0, 7.0, 7.0], [1.0, 5.0, 4.0]]], [[[8.0, 5.0, 1.0], [6.0, 6.0, 8.0], [5.0, 1.0, 8.0]], [[3.0, 
8.0, 3.0], [7.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[1.0, 5.0, 5.0], [8.0, 7.0, 1.0], [5.0, 1.0, 1.0]]], [[[3.0, 8.0, 7.0], [3.0, 
1.0, 2.0], [5.0, 6.0, 3.0]], [[1.0, 1.0, 4.0], [7.0, 7.0, 4.0], [2.0, 7.0, 3.0]], [[7.0, 7.0, 8.0], [5.0, 4.0, 3.0], [3.0, 5.0, 
8.0]]], [[[5.0, 3.0, 5.0], [4.0, 2.0, 2.0], [2.0, 3.0, 3.0]], [[5.0, 4.0, 1.0], [6.0, 8.0, 3.0], [7.0, 2.0, 7.0]], [[1.0, 6.0, 
1.0], [5.0, 2.0, 1.0], [7.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [2.0, 6.0, 6.0], [4.0, 7.0, 4.0]], 
[[8.0, 2.0, 2.0], [1.0, 7.0, 5.0], [7.0, 3.0, 8.0]], [[7.0, 1.0, 6.0], [1.0, 5.0, 8.0], [5.0, 2.0, 3.0]]], [[[3.0, 8.0, 6.0], 
[2.0, 5.0, 6.0], [4.0, 1.0, 3.0]], [[3.0, 8.0, 3.0], [7.0, 6.0, 3.0], [1.0, 4.0, 5.0]], [[5.0, 8.0, 8.0], [8.0, 3.0, 3.0], 
[3.0, 3.0, 3.0]]], [[[1.0, 2.0, 5.0], [4.0, 1.0, 8.0], [1.0, 1.0, 7.0]], [[1.0, 1.0, 4.0], [7.0, 7.0, 4.0], [2.0, 7.0, 3.0]], 
[[3.0, 4.0, 1.0], [8.0, 2.0, 5.0], [5.0, 1.0, 7.0]]], [[[5.0, 3.0, 5.0], [5.0, 1.0, 7.0], [3.0, 3.0, 1.0]], [[5.0, 4.0, 1.0], 
[6.0, 8.0, 3.0], [7.0, 2.0, 7.0]], [[4.0, 3.0, 4.0], [6.0, 4.0, 7.0], [8.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 2.0], [7.0, 2.0, 3.0], [1.0, 7.0, 2.0]], [[1.0, 1.0, 3.0], [7.0, 3.0, 3.0], [5.0, 
2.0, 6.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 8.0], [7.0, 1.0, 3.0]]], [[[4.0, 6.0, 6.0], [4.0, 3.0, 4.0], [8.0, 2.0, 4.0]], [[4.0, 
6.0, 3.0], [3.0, 6.0, 5.0], [1.0, 6.0, 8.0]], [[6.0, 7.0, 6.0], [7.0, 1.0, 2.0], [8.0, 4.0, 6.0]]], [[[3.0, 6.0, 6.0], [7.0, 
7.0, 2.0], [7.0, 8.0, 7.0]], [[7.0, 3.0, 7.0], [3.0, 7.0, 3.0], [6.0, 7.0, 7.0]], [[6.0, 8.0, 2.0], [8.0, 7.0, 8.0], [6.0, 3.0, 
7.0]]], [[[1.0, 4.0, 5.0], [3.0, 6.0, 1.0], [1.0, 2.0, 5.0]], [[5.0, 5.0, 6.0], [7.0, 6.0, 5.0], [6.0, 4.0, 2.0]], [[6.0, 6.0, 
3.0], [3.0, 4.0, 7.0], [3.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 2.0], [5.0, 5.0, 3.0], [8.0, 3.0, 5.0]], 
[[8.0, 1.0, 8.0], [2.0, 5.0, 5.0], [7.0, 8.0, 4.0]], [[5.0, 3.0, 2.0], [6.0, 6.0, 2.0], [3.0, 1.0, 4.0]]], [[[3.0, 5.0, 1.0], 
[2.0, 3.0, 7.0], [8.0, 8.0, 4.0]], [[6.0, 6.0, 3.0], [2.0, 5.0, 4.0], [1.0, 5.0, 5.0]], [[4.0, 3.0, 7.0], [3.0, 7.0, 8.0], 
[5.0, 5.0, 3.0]]], [[[8.0, 4.0, 1.0], [2.0, 4.0, 7.0], [7.0, 7.0, 7.0]], [[2.0, 4.0, 1.0], [6.0, 3.0, 7.0], [8.0, 2.0, 3.0]], 
[[4.0, 6.0, 1.0], [5.0, 2.0, 1.0], [5.0, 5.0, 2.0]]], [[[1.0, 2.0, 2.0], [5.0, 4.0, 3.0], [2.0, 7.0, 3.0]], [[8.0, 6.0, 5.0], 
[3.0, 3.0, 3.0], [8.0, 2.0, 6.0]], [[8.0, 3.0, 6.0], [6.0, 2.0, 5.0], [7.0, 7.0, 1.0]]]])
      val=Data(numpy.array([[[[1.0, 7.0, 2.0], [8.0, 7.0, 8.0], [2.0, 3.0, 7.0]]], [[[6.0, 8.0, 4.0], [1.0, 2.0, 6.0], [7.0, 
6.0, 2.0]]], [[[2.0, 4.0, 1.0], [6.0, 2.0, 5.0], [2.0, 2.0, 8.0]]], [[[5.0, 8.0, 4.0], [7.0, 3.0, 1.0], [6.0, 6.0, 
5.0]]]]),self.functionspace)
      arg[:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 2.0], [7.0, 2.0, 3.0], [1.0, 7.0, 2.0]], [[1.0, 7.0, 2.0], [8.0, 7.0, 8.0], [2.0, 
3.0, 7.0]], [[6.0, 8.0, 7.0], [7.0, 3.0, 8.0], [7.0, 1.0, 3.0]]], [[[4.0, 6.0, 6.0], [4.0, 3.0, 4.0], [8.0, 2.0, 4.0]], [[6.0, 
8.0, 4.0], [1.0, 2.0, 6.0], [7.0, 6.0, 2.0]], [[6.0, 7.0, 6.0], [7.0, 1.0, 2.0], [8.0, 4.0, 6.0]]], [[[3.0, 6.0, 6.0], [7.0, 
7.0, 2.0], [7.0, 8.0, 7.0]], [[2.0, 4.0, 1.0], [6.0, 2.0, 5.0], [2.0, 2.0, 8.0]], [[6.0, 8.0, 2.0], [8.0, 7.0, 8.0], [6.0, 3.0, 
7.0]]], [[[1.0, 4.0, 5.0], [3.0, 6.0, 1.0], [1.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [7.0, 3.0, 1.0], [6.0, 6.0, 5.0]], [[6.0, 6.0, 
3.0], [3.0, 4.0, 7.0], [3.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 2.0], [5.0, 5.0, 3.0], [8.0, 3.0, 5.0]], 
[[1.0, 7.0, 2.0], [8.0, 7.0, 8.0], [2.0, 3.0, 7.0]], [[5.0, 3.0, 2.0], [6.0, 6.0, 2.0], [3.0, 1.0, 4.0]]], [[[3.0, 5.0, 1.0], 
[2.0, 3.0, 7.0], [8.0, 8.0, 4.0]], [[6.0, 8.0, 4.0], [1.0, 2.0, 6.0], [7.0, 6.0, 2.0]], [[4.0, 3.0, 7.0], [3.0, 7.0, 8.0], 
[5.0, 5.0, 3.0]]], [[[8.0, 4.0, 1.0], [2.0, 4.0, 7.0], [7.0, 7.0, 7.0]], [[2.0, 4.0, 1.0], [6.0, 2.0, 5.0], [2.0, 2.0, 8.0]], 
[[4.0, 6.0, 1.0], [5.0, 2.0, 1.0], [5.0, 5.0, 2.0]]], [[[1.0, 2.0, 2.0], [5.0, 4.0, 3.0], [2.0, 7.0, 3.0]], [[5.0, 8.0, 4.0], 
[7.0, 3.0, 1.0], [6.0, 6.0, 5.0]], [[8.0, 3.0, 6.0], [6.0, 2.0, 5.0], [7.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 7.0], [2.0, 7.0, 4.0], [1.0, 4.0, 6.0]], [[3.0, 4.0, 6.0], [3.0, 1.0, 2.0], [1.0, 
2.0, 3.0]], [[2.0, 1.0, 7.0], [3.0, 3.0, 4.0], [4.0, 5.0, 2.0]]], [[[5.0, 3.0, 8.0], [3.0, 8.0, 1.0], [4.0, 7.0, 6.0]], [[1.0, 
1.0, 2.0], [2.0, 2.0, 7.0], [8.0, 6.0, 4.0]], [[3.0, 4.0, 4.0], [4.0, 1.0, 6.0], [5.0, 3.0, 3.0]]], [[[8.0, 6.0, 3.0], [2.0, 
7.0, 6.0], [2.0, 7.0, 3.0]], [[7.0, 5.0, 4.0], [1.0, 6.0, 1.0], [8.0, 5.0, 7.0]], [[4.0, 7.0, 2.0], [1.0, 8.0, 3.0], [5.0, 1.0, 
4.0]]], [[[4.0, 8.0, 1.0], [2.0, 4.0, 3.0], [7.0, 7.0, 2.0]], [[4.0, 4.0, 8.0], [1.0, 3.0, 5.0], [8.0, 2.0, 5.0]], [[7.0, 6.0, 
2.0], [2.0, 7.0, 8.0], [4.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 2.0, 2.0], [1.0, 4.0, 6.0], [5.0, 7.0, 1.0]], 
[[4.0, 3.0, 2.0], [8.0, 4.0, 5.0], [2.0, 4.0, 4.0]], [[5.0, 2.0, 3.0], [3.0, 6.0, 1.0], [5.0, 7.0, 5.0]]], [[[7.0, 3.0, 2.0], 
[4.0, 1.0, 4.0], [1.0, 8.0, 5.0]], [[6.0, 8.0, 3.0], [4.0, 1.0, 2.0], [5.0, 7.0, 5.0]], [[4.0, 1.0, 1.0], [4.0, 2.0, 8.0], 
[1.0, 3.0, 6.0]]], [[[6.0, 4.0, 1.0], [2.0, 4.0, 3.0], [8.0, 7.0, 4.0]], [[8.0, 8.0, 3.0], [5.0, 4.0, 3.0], [3.0, 4.0, 8.0]], 
[[2.0, 8.0, 1.0], [8.0, 1.0, 1.0], [5.0, 8.0, 6.0]]], [[[5.0, 7.0, 7.0], [7.0, 7.0, 4.0], [5.0, 6.0, 1.0]], [[6.0, 3.0, 1.0], 
[2.0, 1.0, 6.0], [8.0, 3.0, 7.0]], [[8.0, 8.0, 6.0], [4.0, 4.0, 6.0], [2.0, 2.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 6.0, 1.0], [7.0, 6.0, 5.0], [5.0, 3.0, 5.0]]], [[[3.0, 6.0, 1.0], [3.0, 1.0, 1.0], [4.0, 
4.0, 2.0]]], [[[6.0, 2.0, 4.0], [2.0, 4.0, 2.0], [6.0, 1.0, 5.0]]], [[[1.0, 6.0, 7.0], [5.0, 8.0, 8.0], [1.0, 2.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 5.0, 1.0], [1.0, 3.0, 4.0], [6.0, 2.0, 1.0]]], [[[8.0, 3.0, 5.0], [3.0, 1.0, 8.0], 
[6.0, 3.0, 7.0]]], [[[8.0, 1.0, 8.0], [2.0, 8.0, 5.0], [1.0, 4.0, 8.0]]], [[[4.0, 4.0, 3.0], [7.0, 8.0, 6.0], [4.0, 4.0, 
2.0]]]])
      arg[:,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 7.0], [2.0, 7.0, 4.0], [1.0, 4.0, 6.0]], [[1.0, 6.0, 1.0], [7.0, 6.0, 5.0], [5.0, 
3.0, 5.0]], [[2.0, 1.0, 7.0], [3.0, 3.0, 4.0], [4.0, 5.0, 2.0]]], [[[5.0, 3.0, 8.0], [3.0, 8.0, 1.0], [4.0, 7.0, 6.0]], [[3.0, 
6.0, 1.0], [3.0, 1.0, 1.0], [4.0, 4.0, 2.0]], [[3.0, 4.0, 4.0], [4.0, 1.0, 6.0], [5.0, 3.0, 3.0]]], [[[8.0, 6.0, 3.0], [2.0, 
7.0, 6.0], [2.0, 7.0, 3.0]], [[6.0, 2.0, 4.0], [2.0, 4.0, 2.0], [6.0, 1.0, 5.0]], [[4.0, 7.0, 2.0], [1.0, 8.0, 3.0], [5.0, 1.0, 
4.0]]], [[[4.0, 8.0, 1.0], [2.0, 4.0, 3.0], [7.0, 7.0, 2.0]], [[1.0, 6.0, 7.0], [5.0, 8.0, 8.0], [1.0, 2.0, 2.0]], [[7.0, 6.0, 
2.0], [2.0, 7.0, 8.0], [4.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 2.0, 2.0], [1.0, 4.0, 6.0], [5.0, 7.0, 1.0]], 
[[5.0, 5.0, 1.0], [1.0, 3.0, 4.0], [6.0, 2.0, 1.0]], [[5.0, 2.0, 3.0], [3.0, 6.0, 1.0], [5.0, 7.0, 5.0]]], [[[7.0, 3.0, 2.0], 
[4.0, 1.0, 4.0], [1.0, 8.0, 5.0]], [[8.0, 3.0, 5.0], [3.0, 1.0, 8.0], [6.0, 3.0, 7.0]], [[4.0, 1.0, 1.0], [4.0, 2.0, 8.0], 
[1.0, 3.0, 6.0]]], [[[6.0, 4.0, 1.0], [2.0, 4.0, 3.0], [8.0, 7.0, 4.0]], [[8.0, 1.0, 8.0], [2.0, 8.0, 5.0], [1.0, 4.0, 8.0]], 
[[2.0, 8.0, 1.0], [8.0, 1.0, 1.0], [5.0, 8.0, 6.0]]], [[[5.0, 7.0, 7.0], [7.0, 7.0, 4.0], [5.0, 6.0, 1.0]], [[4.0, 4.0, 3.0], 
[7.0, 8.0, 6.0], [4.0, 4.0, 2.0]], [[8.0, 8.0, 6.0], [4.0, 4.0, 6.0], [2.0, 2.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_ULRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 2.0], [4.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 1.0, 2.0], [8.0, 7.0, 4.0], [6.0, 
4.0, 8.0]], [[3.0, 5.0, 3.0], [1.0, 5.0, 3.0], [4.0, 6.0, 7.0]]], [[[8.0, 4.0, 6.0], [8.0, 5.0, 5.0], [5.0, 8.0, 8.0]], [[1.0, 
4.0, 3.0], [7.0, 3.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 4.0, 7.0], [7.0, 1.0, 3.0], [4.0, 8.0, 7.0]]], [[[7.0, 2.0, 1.0], [4.0, 
5.0, 6.0], [5.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [7.0, 2.0, 7.0], [5.0, 8.0, 1.0]], [[2.0, 2.0, 7.0], [2.0, 1.0, 5.0], [4.0, 7.0, 
2.0]]], [[[2.0, 2.0, 4.0], [8.0, 1.0, 8.0], [8.0, 5.0, 5.0]], [[5.0, 2.0, 2.0], [3.0, 5.0, 2.0], [7.0, 8.0, 8.0]], [[5.0, 4.0, 
7.0], [7.0, 2.0, 4.0], [7.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 5.0], [4.0, 4.0, 6.0], [8.0, 5.0, 5.0]], 
[[7.0, 4.0, 5.0], [4.0, 1.0, 4.0], [4.0, 2.0, 1.0]], [[1.0, 1.0, 6.0], [3.0, 7.0, 6.0], [2.0, 4.0, 7.0]]], [[[3.0, 4.0, 4.0], 
[5.0, 7.0, 4.0], [3.0, 8.0, 4.0]], [[1.0, 3.0, 3.0], [4.0, 3.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 2.0], [5.0, 3.0, 7.0], 
[5.0, 5.0, 8.0]]], [[[2.0, 4.0, 2.0], [4.0, 8.0, 7.0], [2.0, 1.0, 5.0]], [[8.0, 3.0, 7.0], [7.0, 8.0, 2.0], [8.0, 6.0, 4.0]], 
[[4.0, 1.0, 6.0], [1.0, 3.0, 4.0], [7.0, 6.0, 1.0]]], [[[4.0, 7.0, 3.0], [1.0, 2.0, 6.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 7.0], 
[6.0, 1.0, 3.0], [1.0, 3.0, 8.0]], [[5.0, 5.0, 5.0], [8.0, 1.0, 4.0], [7.0, 8.0, 1.0]]]])
      val=numpy.array([[[[7.0, 1.0, 3.0]]], [[[1.0, 7.0, 3.0]]], [[[4.0, 7.0, 6.0]]]])
      arg[:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [4.0, 6.0, 2.0], [1.0, 2.0, 2.0]], [[7.0, 1.0, 2.0], [8.0, 7.0, 4.0], [6.0, 
4.0, 8.0]], [[3.0, 5.0, 3.0], [7.0, 1.0, 3.0], [4.0, 6.0, 7.0]]], [[[8.0, 4.0, 6.0], [8.0, 5.0, 5.0], [5.0, 8.0, 8.0]], [[1.0, 
4.0, 3.0], [7.0, 3.0, 5.0], [7.0, 5.0, 5.0]], [[2.0, 4.0, 7.0], [1.0, 7.0, 3.0], [4.0, 8.0, 7.0]]], [[[7.0, 2.0, 1.0], [4.0, 
5.0, 6.0], [5.0, 7.0, 7.0]], [[5.0, 8.0, 8.0], [7.0, 2.0, 7.0], [5.0, 8.0, 1.0]], [[2.0, 2.0, 7.0], [4.0, 7.0, 6.0], [4.0, 7.0, 
2.0]]], [[[2.0, 2.0, 4.0], [8.0, 1.0, 8.0], [8.0, 5.0, 5.0]], [[5.0, 2.0, 2.0], [3.0, 5.0, 2.0], [7.0, 8.0, 8.0]], [[5.0, 4.0, 
7.0], [7.0, 2.0, 4.0], [7.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 5.0], [4.0, 4.0, 6.0], [8.0, 5.0, 5.0]], 
[[7.0, 4.0, 5.0], [4.0, 1.0, 4.0], [4.0, 2.0, 1.0]], [[1.0, 1.0, 6.0], [7.0, 1.0, 3.0], [2.0, 4.0, 7.0]]], [[[3.0, 4.0, 4.0], 
[5.0, 7.0, 4.0], [3.0, 8.0, 4.0]], [[1.0, 3.0, 3.0], [4.0, 3.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 2.0], [1.0, 7.0, 3.0], 
[5.0, 5.0, 8.0]]], [[[2.0, 4.0, 2.0], [4.0, 8.0, 7.0], [2.0, 1.0, 5.0]], [[8.0, 3.0, 7.0], [7.0, 8.0, 2.0], [8.0, 6.0, 4.0]], 
[[4.0, 1.0, 6.0], [4.0, 7.0, 6.0], [7.0, 6.0, 1.0]]], [[[4.0, 7.0, 3.0], [1.0, 2.0, 6.0], [6.0, 3.0, 8.0]], [[4.0, 5.0, 7.0], 
[6.0, 1.0, 3.0], [1.0, 3.0, 8.0]], [[5.0, 5.0, 5.0], [8.0, 1.0, 4.0], [7.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_ULRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 4.0], [5.0, 6.0, 6.0], [7.0, 5.0, 8.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 2.0], [4.0, 
3.0, 5.0]], [[7.0, 6.0, 8.0], [7.0, 2.0, 5.0], [5.0, 6.0, 5.0]]], [[[5.0, 5.0, 8.0], [5.0, 3.0, 1.0], [1.0, 6.0, 1.0]], [[4.0, 
6.0, 1.0], [2.0, 7.0, 1.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 2.0], [2.0, 1.0, 2.0], [8.0, 4.0, 3.0]]], [[[5.0, 2.0, 8.0], [8.0, 
7.0, 2.0], [3.0, 2.0, 4.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[6.0, 6.0, 6.0], [8.0, 7.0, 8.0], [6.0, 4.0, 
3.0]]], [[[7.0, 8.0, 2.0], [6.0, 6.0, 3.0], [6.0, 6.0, 8.0]], [[3.0, 3.0, 2.0], [3.0, 6.0, 1.0], [6.0, 3.0, 5.0]], [[6.0, 6.0, 
7.0], [2.0, 6.0, 2.0], [3.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 1.0], [5.0, 2.0, 6.0], [1.0, 8.0, 5.0]], 
[[8.0, 2.0, 5.0], [2.0, 2.0, 6.0], [5.0, 7.0, 4.0]], [[7.0, 4.0, 1.0], [5.0, 5.0, 3.0], [4.0, 5.0, 6.0]]], [[[4.0, 8.0, 3.0], 
[4.0, 7.0, 6.0], [5.0, 4.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 7.0], [3.0, 3.0, 3.0]], [[4.0, 8.0, 5.0], [7.0, 5.0, 8.0], 
[4.0, 2.0, 8.0]]], [[[6.0, 5.0, 4.0], [6.0, 7.0, 2.0], [4.0, 7.0, 2.0]], [[5.0, 8.0, 7.0], [8.0, 4.0, 8.0], [3.0, 1.0, 2.0]], 
[[1.0, 5.0, 5.0], [5.0, 3.0, 4.0], [3.0, 5.0, 4.0]]], [[[3.0, 8.0, 6.0], [2.0, 2.0, 3.0], [4.0, 2.0, 2.0]], [[4.0, 8.0, 7.0], 
[8.0, 5.0, 3.0], [3.0, 3.0, 4.0]], [[2.0, 7.0, 3.0], [8.0, 6.0, 7.0], [5.0, 4.0, 7.0]]]])
      val=Data(numpy.array([[[[4.0, 7.0, 1.0]]], [[[4.0, 1.0, 7.0]]], [[[5.0, 6.0, 6.0]]]]),self.functionspace)
      arg[:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [5.0, 6.0, 6.0], [7.0, 5.0, 8.0]], [[7.0, 2.0, 3.0], [4.0, 4.0, 2.0], [4.0, 
3.0, 5.0]], [[7.0, 6.0, 8.0], [4.0, 7.0, 1.0], [5.0, 6.0, 5.0]]], [[[5.0, 5.0, 8.0], [5.0, 3.0, 1.0], [1.0, 6.0, 1.0]], [[4.0, 
6.0, 1.0], [2.0, 7.0, 1.0], [6.0, 4.0, 5.0]], [[4.0, 5.0, 2.0], [4.0, 1.0, 7.0], [8.0, 4.0, 3.0]]], [[[5.0, 2.0, 8.0], [8.0, 
7.0, 2.0], [3.0, 2.0, 4.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 2.0], [5.0, 7.0, 3.0]], [[6.0, 6.0, 6.0], [5.0, 6.0, 6.0], [6.0, 4.0, 
3.0]]], [[[7.0, 8.0, 2.0], [6.0, 6.0, 3.0], [6.0, 6.0, 8.0]], [[3.0, 3.0, 2.0], [3.0, 6.0, 1.0], [6.0, 3.0, 5.0]], [[6.0, 6.0, 
7.0], [2.0, 6.0, 2.0], [3.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 1.0], [5.0, 2.0, 6.0], [1.0, 8.0, 5.0]], 
[[8.0, 2.0, 5.0], [2.0, 2.0, 6.0], [5.0, 7.0, 4.0]], [[7.0, 4.0, 1.0], [4.0, 7.0, 1.0], [4.0, 5.0, 6.0]]], [[[4.0, 8.0, 3.0], 
[4.0, 7.0, 6.0], [5.0, 4.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 6.0, 7.0], [3.0, 3.0, 3.0]], [[4.0, 8.0, 5.0], [4.0, 1.0, 7.0], 
[4.0, 2.0, 8.0]]], [[[6.0, 5.0, 4.0], [6.0, 7.0, 2.0], [4.0, 7.0, 2.0]], [[5.0, 8.0, 7.0], [8.0, 4.0, 8.0], [3.0, 1.0, 2.0]], 
[[1.0, 5.0, 5.0], [5.0, 6.0, 6.0], [3.0, 5.0, 4.0]]], [[[3.0, 8.0, 6.0], [2.0, 2.0, 3.0], [4.0, 2.0, 2.0]], [[4.0, 8.0, 7.0], 
[8.0, 5.0, 3.0], [3.0, 3.0, 4.0]], [[2.0, 7.0, 3.0], [8.0, 6.0, 7.0], [5.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_ULRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 3.0], [6.0, 7.0, 3.0], [6.0, 6.0, 4.0]], [[3.0, 4.0, 4.0], [4.0, 2.0, 4.0], [2.0, 
8.0, 5.0]], [[4.0, 1.0, 3.0], [5.0, 6.0, 2.0], [8.0, 2.0, 1.0]]], [[[5.0, 2.0, 8.0], [2.0, 4.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 
8.0, 6.0], [5.0, 8.0, 7.0], [4.0, 3.0, 2.0]], [[5.0, 7.0, 6.0], [5.0, 3.0, 8.0], [8.0, 1.0, 7.0]]], [[[5.0, 2.0, 6.0], [4.0, 
3.0, 1.0], [8.0, 3.0, 2.0]], [[4.0, 7.0, 4.0], [7.0, 5.0, 1.0], [1.0, 1.0, 6.0]], [[4.0, 7.0, 5.0], [6.0, 5.0, 7.0], [5.0, 8.0, 
7.0]]], [[[6.0, 5.0, 2.0], [2.0, 7.0, 3.0], [7.0, 3.0, 5.0]], [[2.0, 1.0, 5.0], [8.0, 3.0, 3.0], [6.0, 7.0, 4.0]], [[8.0, 3.0, 
6.0], [5.0, 1.0, 8.0], [8.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 5.0], [8.0, 7.0, 1.0], [8.0, 4.0, 3.0]], 
[[1.0, 5.0, 2.0], [1.0, 4.0, 3.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 2.0], [5.0, 3.0, 8.0], [4.0, 8.0, 5.0]]], [[[2.0, 5.0, 6.0], 
[3.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 5.0], [6.0, 5.0, 2.0], [4.0, 2.0, 3.0]], [[2.0, 8.0, 5.0], [4.0, 3.0, 3.0], 
[1.0, 7.0, 2.0]]], [[[2.0, 2.0, 8.0], [6.0, 2.0, 6.0], [8.0, 7.0, 2.0]], [[7.0, 4.0, 7.0], [6.0, 5.0, 1.0], [3.0, 3.0, 5.0]], 
[[5.0, 7.0, 4.0], [4.0, 6.0, 8.0], [5.0, 7.0, 4.0]]], [[[7.0, 8.0, 8.0], [8.0, 5.0, 2.0], [7.0, 5.0, 5.0]], [[6.0, 8.0, 4.0], 
[1.0, 4.0, 3.0], [7.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [6.0, 8.0, 4.0], [5.0, 3.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 3.0, 6.0]]], [[[7.0, 4.0, 1.0]]], [[[3.0, 8.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 5.0, 5.0]]], [[[3.0, 8.0, 6.0]]], [[[2.0, 3.0, 2.0]]]])
      arg[:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [6.0, 7.0, 3.0], [6.0, 6.0, 4.0]], [[3.0, 4.0, 4.0], [4.0, 2.0, 4.0], [2.0, 
8.0, 5.0]], [[4.0, 1.0, 3.0], [4.0, 3.0, 6.0], [8.0, 2.0, 1.0]]], [[[5.0, 2.0, 8.0], [2.0, 4.0, 1.0], [3.0, 4.0, 7.0]], [[7.0, 
8.0, 6.0], [5.0, 8.0, 7.0], [4.0, 3.0, 2.0]], [[5.0, 7.0, 6.0], [7.0, 4.0, 1.0], [8.0, 1.0, 7.0]]], [[[5.0, 2.0, 6.0], [4.0, 
3.0, 1.0], [8.0, 3.0, 2.0]], [[4.0, 7.0, 4.0], [7.0, 5.0, 1.0], [1.0, 1.0, 6.0]], [[4.0, 7.0, 5.0], [3.0, 8.0, 3.0], [5.0, 8.0, 
7.0]]], [[[6.0, 5.0, 2.0], [2.0, 7.0, 3.0], [7.0, 3.0, 5.0]], [[2.0, 1.0, 5.0], [8.0, 3.0, 3.0], [6.0, 7.0, 4.0]], [[8.0, 3.0, 
6.0], [5.0, 1.0, 8.0], [8.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 5.0], [8.0, 7.0, 1.0], [8.0, 4.0, 3.0]], 
[[1.0, 5.0, 2.0], [1.0, 4.0, 3.0], [7.0, 2.0, 4.0]], [[3.0, 8.0, 2.0], [2.0, 5.0, 5.0], [4.0, 8.0, 5.0]]], [[[2.0, 5.0, 6.0], 
[3.0, 6.0, 7.0], [6.0, 5.0, 5.0]], [[1.0, 4.0, 5.0], [6.0, 5.0, 2.0], [4.0, 2.0, 3.0]], [[2.0, 8.0, 5.0], [3.0, 8.0, 6.0], 
[1.0, 7.0, 2.0]]], [[[2.0, 2.0, 8.0], [6.0, 2.0, 6.0], [8.0, 7.0, 2.0]], [[7.0, 4.0, 7.0], [6.0, 5.0, 1.0], [3.0, 3.0, 5.0]], 
[[5.0, 7.0, 4.0], [2.0, 3.0, 2.0], [5.0, 7.0, 4.0]]], [[[7.0, 8.0, 8.0], [8.0, 5.0, 2.0], [7.0, 5.0, 5.0]], [[6.0, 8.0, 4.0], 
[1.0, 4.0, 3.0], [7.0, 2.0, 2.0]], [[3.0, 2.0, 8.0], [6.0, 8.0, 4.0], [5.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 6.0], [1.0, 2.0, 5.0], [6.0, 6.0, 4.0]], [[5.0, 2.0, 5.0], [6.0, 8.0, 3.0], [8.0, 
3.0, 5.0]], [[5.0, 4.0, 3.0], [2.0, 5.0, 6.0], [3.0, 8.0, 4.0]]], [[[4.0, 2.0, 4.0], [8.0, 4.0, 6.0], [8.0, 5.0, 8.0]], [[1.0, 
6.0, 3.0], [7.0, 2.0, 1.0], [5.0, 5.0, 6.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 1.0], [3.0, 7.0, 3.0]]], [[[4.0, 5.0, 8.0], [2.0, 
5.0, 7.0], [7.0, 5.0, 4.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 8.0], [7.0, 4.0, 5.0]], [[3.0, 1.0, 8.0], [3.0, 1.0, 6.0], [1.0, 1.0, 
4.0]]], [[[6.0, 6.0, 3.0], [8.0, 8.0, 2.0], [3.0, 2.0, 1.0]], [[1.0, 6.0, 6.0], [4.0, 1.0, 3.0], [2.0, 1.0, 8.0]], [[3.0, 5.0, 
7.0], [8.0, 4.0, 4.0], [5.0, 2.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 3.0], [1.0, 1.0, 5.0], [1.0, 8.0, 2.0]], 
[[4.0, 2.0, 7.0], [3.0, 5.0, 7.0], [2.0, 5.0, 7.0]], [[7.0, 4.0, 4.0], [5.0, 1.0, 8.0], [7.0, 3.0, 1.0]]], [[[6.0, 8.0, 6.0], 
[8.0, 3.0, 2.0], [8.0, 5.0, 2.0]], [[4.0, 2.0, 7.0], [3.0, 1.0, 1.0], [4.0, 5.0, 3.0]], [[6.0, 3.0, 7.0], [2.0, 5.0, 3.0], 
[1.0, 7.0, 6.0]]], [[[8.0, 5.0, 8.0], [5.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [5.0, 3.0, 3.0], [5.0, 6.0, 1.0]], 
[[4.0, 6.0, 3.0], [2.0, 6.0, 8.0], [4.0, 2.0, 5.0]]], [[[1.0, 1.0, 2.0], [8.0, 5.0, 1.0], [5.0, 8.0, 4.0]], [[2.0, 6.0, 1.0], 
[4.0, 4.0, 4.0], [3.0, 8.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 3.0, 2.0], [2.0, 8.0, 7.0]]]])
      val=numpy.array([[[[7.0, 6.0, 5.0]]], [[[4.0, 4.0, 8.0]]]])
      arg[1:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 6.0], [1.0, 2.0, 5.0], [6.0, 6.0, 4.0]], [[5.0, 2.0, 5.0], [6.0, 8.0, 3.0], [8.0, 
3.0, 5.0]], [[5.0, 4.0, 3.0], [2.0, 5.0, 6.0], [3.0, 8.0, 4.0]]], [[[4.0, 2.0, 4.0], [8.0, 4.0, 6.0], [8.0, 5.0, 8.0]], [[1.0, 
6.0, 3.0], [7.0, 2.0, 1.0], [5.0, 5.0, 6.0]], [[2.0, 2.0, 4.0], [7.0, 6.0, 5.0], [3.0, 7.0, 3.0]]], [[[4.0, 5.0, 8.0], [2.0, 
5.0, 7.0], [7.0, 5.0, 4.0]], [[6.0, 2.0, 4.0], [1.0, 1.0, 8.0], [7.0, 4.0, 5.0]], [[3.0, 1.0, 8.0], [4.0, 4.0, 8.0], [1.0, 1.0, 
4.0]]], [[[6.0, 6.0, 3.0], [8.0, 8.0, 2.0], [3.0, 2.0, 1.0]], [[1.0, 6.0, 6.0], [4.0, 1.0, 3.0], [2.0, 1.0, 8.0]], [[3.0, 5.0, 
7.0], [8.0, 4.0, 4.0], [5.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [1.0, 1.0, 5.0], [1.0, 8.0, 2.0]], 
[[4.0, 2.0, 7.0], [3.0, 5.0, 7.0], [2.0, 5.0, 7.0]], [[7.0, 4.0, 4.0], [5.0, 1.0, 8.0], [7.0, 3.0, 1.0]]], [[[6.0, 8.0, 6.0], 
[8.0, 3.0, 2.0], [8.0, 5.0, 2.0]], [[4.0, 2.0, 7.0], [3.0, 1.0, 1.0], [4.0, 5.0, 3.0]], [[6.0, 3.0, 7.0], [7.0, 6.0, 5.0], 
[1.0, 7.0, 6.0]]], [[[8.0, 5.0, 8.0], [5.0, 7.0, 6.0], [1.0, 3.0, 7.0]], [[6.0, 6.0, 1.0], [5.0, 3.0, 3.0], [5.0, 6.0, 1.0]], 
[[4.0, 6.0, 3.0], [4.0, 4.0, 8.0], [4.0, 2.0, 5.0]]], [[[1.0, 1.0, 2.0], [8.0, 5.0, 1.0], [5.0, 8.0, 4.0]], [[2.0, 6.0, 1.0], 
[4.0, 4.0, 4.0], [3.0, 8.0, 8.0]], [[6.0, 1.0, 4.0], [4.0, 3.0, 2.0], [2.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 7.0], [5.0, 7.0, 3.0], [4.0, 3.0, 7.0]], [[6.0, 8.0, 3.0], [6.0, 3.0, 4.0], [5.0, 
4.0, 7.0]], [[1.0, 3.0, 1.0], [1.0, 3.0, 3.0], [7.0, 2.0, 3.0]]], [[[2.0, 6.0, 2.0], [8.0, 2.0, 4.0], [5.0, 4.0, 4.0]], [[3.0, 
2.0, 5.0], [3.0, 8.0, 3.0], [2.0, 3.0, 1.0]], [[4.0, 3.0, 4.0], [7.0, 1.0, 6.0], [4.0, 8.0, 8.0]]], [[[3.0, 8.0, 7.0], [7.0, 
8.0, 6.0], [6.0, 3.0, 7.0]], [[4.0, 1.0, 4.0], [3.0, 7.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 6.0, 2.0], [8.0, 8.0, 3.0], [4.0, 7.0, 
1.0]]], [[[8.0, 4.0, 8.0], [3.0, 1.0, 1.0], [7.0, 6.0, 4.0]], [[1.0, 4.0, 3.0], [6.0, 7.0, 3.0], [8.0, 7.0, 4.0]], [[5.0, 1.0, 
5.0], [1.0, 5.0, 1.0], [2.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 6.0], [3.0, 5.0, 2.0], [2.0, 1.0, 8.0]], 
[[7.0, 5.0, 2.0], [1.0, 4.0, 7.0], [2.0, 8.0, 1.0]], [[1.0, 7.0, 6.0], [4.0, 2.0, 1.0], [8.0, 3.0, 6.0]]], [[[6.0, 3.0, 1.0], 
[1.0, 6.0, 8.0], [7.0, 5.0, 2.0]], [[6.0, 6.0, 8.0], [8.0, 4.0, 1.0], [8.0, 4.0, 2.0]], [[7.0, 1.0, 3.0], [2.0, 3.0, 3.0], 
[8.0, 6.0, 1.0]]], [[[5.0, 7.0, 1.0], [3.0, 3.0, 2.0], [3.0, 7.0, 4.0]], [[2.0, 2.0, 2.0], [1.0, 2.0, 1.0], [8.0, 3.0, 2.0]], 
[[4.0, 3.0, 5.0], [5.0, 8.0, 1.0], [6.0, 6.0, 3.0]]], [[[5.0, 8.0, 6.0], [4.0, 1.0, 8.0], [2.0, 3.0, 1.0]], [[7.0, 8.0, 3.0], 
[8.0, 4.0, 5.0], [7.0, 8.0, 6.0]], [[8.0, 1.0, 4.0], [7.0, 8.0, 6.0], [7.0, 1.0, 3.0]]]])
      val=Data(numpy.array([[[[6.0, 8.0, 4.0]]], [[[2.0, 1.0, 2.0]]]]),self.functionspace)
      arg[1:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 7.0], [5.0, 7.0, 3.0], [4.0, 3.0, 7.0]], [[6.0, 8.0, 3.0], [6.0, 3.0, 4.0], [5.0, 
4.0, 7.0]], [[1.0, 3.0, 1.0], [1.0, 3.0, 3.0], [7.0, 2.0, 3.0]]], [[[2.0, 6.0, 2.0], [8.0, 2.0, 4.0], [5.0, 4.0, 4.0]], [[3.0, 
2.0, 5.0], [3.0, 8.0, 3.0], [2.0, 3.0, 1.0]], [[4.0, 3.0, 4.0], [6.0, 8.0, 4.0], [4.0, 8.0, 8.0]]], [[[3.0, 8.0, 7.0], [7.0, 
8.0, 6.0], [6.0, 3.0, 7.0]], [[4.0, 1.0, 4.0], [3.0, 7.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 6.0, 2.0], [2.0, 1.0, 2.0], [4.0, 7.0, 
1.0]]], [[[8.0, 4.0, 8.0], [3.0, 1.0, 1.0], [7.0, 6.0, 4.0]], [[1.0, 4.0, 3.0], [6.0, 7.0, 3.0], [8.0, 7.0, 4.0]], [[5.0, 1.0, 
5.0], [1.0, 5.0, 1.0], [2.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 6.0], [3.0, 5.0, 2.0], [2.0, 1.0, 8.0]], 
[[7.0, 5.0, 2.0], [1.0, 4.0, 7.0], [2.0, 8.0, 1.0]], [[1.0, 7.0, 6.0], [4.0, 2.0, 1.0], [8.0, 3.0, 6.0]]], [[[6.0, 3.0, 1.0], 
[1.0, 6.0, 8.0], [7.0, 5.0, 2.0]], [[6.0, 6.0, 8.0], [8.0, 4.0, 1.0], [8.0, 4.0, 2.0]], [[7.0, 1.0, 3.0], [6.0, 8.0, 4.0], 
[8.0, 6.0, 1.0]]], [[[5.0, 7.0, 1.0], [3.0, 3.0, 2.0], [3.0, 7.0, 4.0]], [[2.0, 2.0, 2.0], [1.0, 2.0, 1.0], [8.0, 3.0, 2.0]], 
[[4.0, 3.0, 5.0], [2.0, 1.0, 2.0], [6.0, 6.0, 3.0]]], [[[5.0, 8.0, 6.0], [4.0, 1.0, 8.0], [2.0, 3.0, 1.0]], [[7.0, 8.0, 3.0], 
[8.0, 4.0, 5.0], [7.0, 8.0, 6.0]], [[8.0, 1.0, 4.0], [7.0, 8.0, 6.0], [7.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RLRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 1.0], [2.0, 7.0, 4.0], [6.0, 3.0, 8.0]], [[1.0, 8.0, 7.0], [2.0, 7.0, 2.0], [4.0, 
8.0, 7.0]], [[2.0, 2.0, 6.0], [7.0, 5.0, 5.0], [7.0, 5.0, 2.0]]], [[[7.0, 4.0, 3.0], [7.0, 8.0, 8.0], [6.0, 8.0, 1.0]], [[6.0, 
4.0, 6.0], [5.0, 1.0, 5.0], [2.0, 2.0, 8.0]], [[7.0, 2.0, 2.0], [4.0, 7.0, 2.0], [8.0, 7.0, 1.0]]], [[[5.0, 6.0, 5.0], [1.0, 
6.0, 7.0], [5.0, 1.0, 2.0]], [[8.0, 2.0, 1.0], [1.0, 8.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 7.0, 1.0], [6.0, 5.0, 5.0], [6.0, 6.0, 
3.0]]], [[[2.0, 3.0, 2.0], [3.0, 3.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 6.0, 4.0], [4.0, 1.0, 1.0], [5.0, 6.0, 3.0]], [[6.0, 4.0, 
1.0], [6.0, 8.0, 2.0], [3.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 3.0], [5.0, 4.0, 6.0], [5.0, 1.0, 2.0]], 
[[2.0, 5.0, 7.0], [8.0, 2.0, 5.0], [3.0, 7.0, 8.0]], [[5.0, 1.0, 6.0], [4.0, 8.0, 3.0], [6.0, 5.0, 1.0]]], [[[7.0, 4.0, 2.0], 
[4.0, 7.0, 7.0], [4.0, 3.0, 2.0]], [[1.0, 2.0, 8.0], [6.0, 5.0, 7.0], [5.0, 6.0, 3.0]], [[5.0, 7.0, 8.0], [6.0, 4.0, 6.0], 
[8.0, 3.0, 5.0]]], [[[8.0, 3.0, 5.0], [5.0, 8.0, 6.0], [4.0, 4.0, 5.0]], [[5.0, 8.0, 7.0], [2.0, 2.0, 7.0], [6.0, 8.0, 5.0]], 
[[7.0, 6.0, 8.0], [1.0, 2.0, 3.0], [1.0, 3.0, 4.0]]], [[[6.0, 8.0, 7.0], [1.0, 2.0, 3.0], [2.0, 5.0, 1.0]], [[1.0, 5.0, 5.0], 
[7.0, 5.0, 4.0], [1.0, 4.0, 2.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 3.0], [5.0, 6.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 8.0, 1.0]]], [[[5.0, 2.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 3.0, 6.0]]], 
[[[1.0, 1.0, 6.0]]]])
      arg[1:3,2:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 1.0], [2.0, 7.0, 4.0], [6.0, 3.0, 8.0]], [[1.0, 8.0, 7.0], [2.0, 7.0, 2.0], [4.0, 
8.0, 7.0]], [[2.0, 2.0, 6.0], [7.0, 5.0, 5.0], [7.0, 5.0, 2.0]]], [[[7.0, 4.0, 3.0], [7.0, 8.0, 8.0], [6.0, 8.0, 1.0]], [[6.0, 
4.0, 6.0], [5.0, 1.0, 5.0], [2.0, 2.0, 8.0]], [[7.0, 2.0, 2.0], [6.0, 8.0, 1.0], [8.0, 7.0, 1.0]]], [[[5.0, 6.0, 5.0], [1.0, 
6.0, 7.0], [5.0, 1.0, 2.0]], [[8.0, 2.0, 1.0], [1.0, 8.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 7.0, 1.0], [5.0, 2.0, 3.0], [6.0, 6.0, 
3.0]]], [[[2.0, 3.0, 2.0], [3.0, 3.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 6.0, 4.0], [4.0, 1.0, 1.0], [5.0, 6.0, 3.0]], [[6.0, 4.0, 
1.0], [6.0, 8.0, 2.0], [3.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 3.0], [5.0, 4.0, 6.0], [5.0, 1.0, 2.0]], 
[[2.0, 5.0, 7.0], [8.0, 2.0, 5.0], [3.0, 7.0, 8.0]], [[5.0, 1.0, 6.0], [4.0, 8.0, 3.0], [6.0, 5.0, 1.0]]], [[[7.0, 4.0, 2.0], 
[4.0, 7.0, 7.0], [4.0, 3.0, 2.0]], [[1.0, 2.0, 8.0], [6.0, 5.0, 7.0], [5.0, 6.0, 3.0]], [[5.0, 7.0, 8.0], [8.0, 3.0, 6.0], 
[8.0, 3.0, 5.0]]], [[[8.0, 3.0, 5.0], [5.0, 8.0, 6.0], [4.0, 4.0, 5.0]], [[5.0, 8.0, 7.0], [2.0, 2.0, 7.0], [6.0, 8.0, 5.0]], 
[[7.0, 6.0, 8.0], [1.0, 1.0, 6.0], [1.0, 3.0, 4.0]]], [[[6.0, 8.0, 7.0], [1.0, 2.0, 3.0], [2.0, 5.0, 1.0]], [[1.0, 5.0, 5.0], 
[7.0, 5.0, 4.0], [1.0, 4.0, 2.0]], [[8.0, 2.0, 5.0], [3.0, 6.0, 3.0], [5.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 5.0], [5.0, 6.0, 4.0], [7.0, 5.0, 4.0]], [[4.0, 1.0, 6.0], [4.0, 1.0, 7.0], [3.0, 
4.0, 7.0]], [[6.0, 8.0, 6.0], [3.0, 5.0, 6.0], [3.0, 4.0, 3.0]]], [[[8.0, 8.0, 5.0], [4.0, 5.0, 8.0], [7.0, 7.0, 4.0]], [[7.0, 
3.0, 4.0], [8.0, 6.0, 3.0], [6.0, 4.0, 3.0]], [[7.0, 7.0, 5.0], [4.0, 1.0, 3.0], [6.0, 8.0, 8.0]]], [[[6.0, 4.0, 6.0], [4.0, 
8.0, 1.0], [4.0, 2.0, 2.0]], [[7.0, 3.0, 3.0], [2.0, 1.0, 7.0], [4.0, 4.0, 1.0]], [[2.0, 1.0, 5.0], [1.0, 5.0, 7.0], [5.0, 6.0, 
8.0]]], [[[1.0, 7.0, 2.0], [3.0, 7.0, 5.0], [6.0, 6.0, 4.0]], [[2.0, 5.0, 6.0], [8.0, 4.0, 7.0], [7.0, 3.0, 3.0]], [[3.0, 1.0, 
5.0], [1.0, 7.0, 8.0], [2.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 7.0], [4.0, 1.0, 7.0], [1.0, 2.0, 1.0]], 
[[5.0, 4.0, 6.0], [2.0, 4.0, 6.0], [3.0, 8.0, 7.0]], [[2.0, 6.0, 2.0], [8.0, 4.0, 1.0], [8.0, 6.0, 8.0]]], [[[1.0, 4.0, 8.0], 
[2.0, 5.0, 4.0], [7.0, 5.0, 2.0]], [[2.0, 4.0, 6.0], [2.0, 7.0, 8.0], [3.0, 3.0, 5.0]], [[2.0, 4.0, 1.0], [8.0, 8.0, 4.0], 
[5.0, 3.0, 3.0]]], [[[8.0, 3.0, 7.0], [6.0, 7.0, 2.0], [6.0, 3.0, 6.0]], [[7.0, 6.0, 8.0], [2.0, 7.0, 8.0], [2.0, 6.0, 3.0]], 
[[7.0, 2.0, 1.0], [6.0, 8.0, 4.0], [8.0, 4.0, 8.0]]], [[[4.0, 2.0, 3.0], [7.0, 6.0, 5.0], [3.0, 7.0, 6.0]], [[5.0, 4.0, 8.0], 
[2.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 8.0], [5.0, 4.0, 5.0], [1.0, 8.0, 6.0]]]])
      val=numpy.array([[4.0, 1.0, 1.0]])
      arg[2,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 5.0], [5.0, 6.0, 4.0], [7.0, 5.0, 4.0]], [[4.0, 1.0, 6.0], [4.0, 1.0, 7.0], [3.0, 
4.0, 7.0]], [[6.0, 8.0, 6.0], [3.0, 5.0, 6.0], [3.0, 4.0, 3.0]]], [[[8.0, 8.0, 5.0], [4.0, 5.0, 8.0], [7.0, 7.0, 4.0]], [[7.0, 
3.0, 4.0], [8.0, 6.0, 3.0], [6.0, 4.0, 3.0]], [[7.0, 7.0, 5.0], [4.0, 1.0, 3.0], [6.0, 8.0, 8.0]]], [[[6.0, 4.0, 6.0], [4.0, 
8.0, 1.0], [4.0, 2.0, 2.0]], [[7.0, 3.0, 3.0], [4.0, 1.0, 1.0], [4.0, 4.0, 1.0]], [[2.0, 1.0, 5.0], [1.0, 5.0, 7.0], [5.0, 6.0, 
8.0]]], [[[1.0, 7.0, 2.0], [3.0, 7.0, 5.0], [6.0, 6.0, 4.0]], [[2.0, 5.0, 6.0], [8.0, 4.0, 7.0], [7.0, 3.0, 3.0]], [[3.0, 1.0, 
5.0], [1.0, 7.0, 8.0], [2.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 7.0], [4.0, 1.0, 7.0], [1.0, 2.0, 1.0]], 
[[5.0, 4.0, 6.0], [2.0, 4.0, 6.0], [3.0, 8.0, 7.0]], [[2.0, 6.0, 2.0], [8.0, 4.0, 1.0], [8.0, 6.0, 8.0]]], [[[1.0, 4.0, 8.0], 
[2.0, 5.0, 4.0], [7.0, 5.0, 2.0]], [[2.0, 4.0, 6.0], [2.0, 7.0, 8.0], [3.0, 3.0, 5.0]], [[2.0, 4.0, 1.0], [8.0, 8.0, 4.0], 
[5.0, 3.0, 3.0]]], [[[8.0, 3.0, 7.0], [6.0, 7.0, 2.0], [6.0, 3.0, 6.0]], [[7.0, 6.0, 8.0], [4.0, 1.0, 1.0], [2.0, 6.0, 3.0]], 
[[7.0, 2.0, 1.0], [6.0, 8.0, 4.0], [8.0, 4.0, 8.0]]], [[[4.0, 2.0, 3.0], [7.0, 6.0, 5.0], [3.0, 7.0, 6.0]], [[5.0, 4.0, 8.0], 
[2.0, 3.0, 5.0], [3.0, 6.0, 1.0]], [[8.0, 7.0, 8.0], [5.0, 4.0, 5.0], [1.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 5.0], [7.0, 3.0, 2.0], [1.0, 2.0, 6.0]], [[7.0, 2.0, 2.0], [1.0, 8.0, 1.0], [2.0, 
4.0, 7.0]], [[3.0, 2.0, 3.0], [6.0, 6.0, 1.0], [8.0, 3.0, 3.0]]], [[[4.0, 2.0, 3.0], [7.0, 1.0, 8.0], [4.0, 3.0, 8.0]], [[8.0, 
2.0, 4.0], [8.0, 6.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 3.0, 5.0], [7.0, 4.0, 6.0], [1.0, 2.0, 3.0]]], [[[2.0, 5.0, 8.0], [6.0, 
8.0, 6.0], [2.0, 1.0, 2.0]], [[4.0, 3.0, 6.0], [1.0, 2.0, 6.0], [8.0, 8.0, 8.0]], [[1.0, 1.0, 4.0], [6.0, 1.0, 1.0], [7.0, 3.0, 
1.0]]], [[[5.0, 8.0, 4.0], [7.0, 7.0, 4.0], [1.0, 3.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 7.0, 6.0], [6.0, 8.0, 5.0]], [[8.0, 7.0, 
2.0], [1.0, 1.0, 5.0], [7.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 4.0], [7.0, 4.0, 8.0], [6.0, 3.0, 7.0]], 
[[5.0, 4.0, 7.0], [7.0, 6.0, 2.0], [1.0, 4.0, 2.0]], [[6.0, 3.0, 1.0], [1.0, 5.0, 1.0], [8.0, 3.0, 2.0]]], [[[8.0, 6.0, 4.0], 
[7.0, 7.0, 2.0], [2.0, 7.0, 6.0]], [[4.0, 6.0, 5.0], [1.0, 8.0, 2.0], [2.0, 8.0, 6.0]], [[5.0, 2.0, 4.0], [6.0, 2.0, 5.0], 
[3.0, 7.0, 4.0]]], [[[8.0, 6.0, 3.0], [1.0, 6.0, 6.0], [3.0, 6.0, 1.0]], [[1.0, 5.0, 4.0], [7.0, 8.0, 1.0], [6.0, 5.0, 2.0]], 
[[5.0, 8.0, 2.0], [4.0, 7.0, 4.0], [6.0, 2.0, 3.0]]], [[[4.0, 1.0, 1.0], [6.0, 7.0, 4.0], [4.0, 6.0, 4.0]], [[8.0, 3.0, 2.0], 
[4.0, 3.0, 5.0], [4.0, 5.0, 1.0]], [[5.0, 2.0, 5.0], [1.0, 4.0, 3.0], [7.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[3.0, 8.0, 4.0]]),self.functionspace)
      arg[2,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 5.0], [7.0, 3.0, 2.0], [1.0, 2.0, 6.0]], [[7.0, 2.0, 2.0], [1.0, 8.0, 1.0], [2.0, 
4.0, 7.0]], [[3.0, 2.0, 3.0], [6.0, 6.0, 1.0], [8.0, 3.0, 3.0]]], [[[4.0, 2.0, 3.0], [7.0, 1.0, 8.0], [4.0, 3.0, 8.0]], [[8.0, 
2.0, 4.0], [8.0, 6.0, 8.0], [6.0, 8.0, 1.0]], [[3.0, 3.0, 5.0], [7.0, 4.0, 6.0], [1.0, 2.0, 3.0]]], [[[2.0, 5.0, 8.0], [6.0, 
8.0, 6.0], [2.0, 1.0, 2.0]], [[4.0, 3.0, 6.0], [3.0, 8.0, 4.0], [8.0, 8.0, 8.0]], [[1.0, 1.0, 4.0], [6.0, 1.0, 1.0], [7.0, 3.0, 
1.0]]], [[[5.0, 8.0, 4.0], [7.0, 7.0, 4.0], [1.0, 3.0, 5.0]], [[1.0, 3.0, 3.0], [8.0, 7.0, 6.0], [6.0, 8.0, 5.0]], [[8.0, 7.0, 
2.0], [1.0, 1.0, 5.0], [7.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 4.0], [7.0, 4.0, 8.0], [6.0, 3.0, 7.0]], 
[[5.0, 4.0, 7.0], [7.0, 6.0, 2.0], [1.0, 4.0, 2.0]], [[6.0, 3.0, 1.0], [1.0, 5.0, 1.0], [8.0, 3.0, 2.0]]], [[[8.0, 6.0, 4.0], 
[7.0, 7.0, 2.0], [2.0, 7.0, 6.0]], [[4.0, 6.0, 5.0], [1.0, 8.0, 2.0], [2.0, 8.0, 6.0]], [[5.0, 2.0, 4.0], [6.0, 2.0, 5.0], 
[3.0, 7.0, 4.0]]], [[[8.0, 6.0, 3.0], [1.0, 6.0, 6.0], [3.0, 6.0, 1.0]], [[1.0, 5.0, 4.0], [3.0, 8.0, 4.0], [6.0, 5.0, 2.0]], 
[[5.0, 8.0, 2.0], [4.0, 7.0, 4.0], [6.0, 2.0, 3.0]]], [[[4.0, 1.0, 1.0], [6.0, 7.0, 4.0], [4.0, 6.0, 4.0]], [[8.0, 3.0, 2.0], 
[4.0, 3.0, 5.0], [4.0, 5.0, 1.0]], [[5.0, 2.0, 5.0], [1.0, 4.0, 3.0], [7.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 6.0], [3.0, 7.0, 1.0], [5.0, 8.0, 3.0]], [[1.0, 5.0, 6.0], [7.0, 3.0, 5.0], [6.0, 
2.0, 4.0]], [[5.0, 3.0, 7.0], [5.0, 3.0, 2.0], [5.0, 4.0, 4.0]]], [[[5.0, 1.0, 2.0], [4.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[6.0, 
3.0, 7.0], [8.0, 3.0, 5.0], [2.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 4.0, 3.0], [3.0, 7.0, 1.0]]], [[[8.0, 7.0, 8.0], [5.0, 
7.0, 7.0], [4.0, 8.0, 4.0]], [[5.0, 2.0, 8.0], [2.0, 7.0, 1.0], [5.0, 8.0, 5.0]], [[1.0, 6.0, 6.0], [1.0, 1.0, 8.0], [1.0, 4.0, 
1.0]]], [[[7.0, 4.0, 5.0], [8.0, 7.0, 7.0], [3.0, 7.0, 6.0]], [[1.0, 6.0, 4.0], [2.0, 4.0, 1.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 
2.0], [7.0, 5.0, 8.0], [7.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 7.0], [4.0, 2.0, 4.0], [8.0, 4.0, 2.0]], 
[[4.0, 7.0, 7.0], [1.0, 7.0, 1.0], [2.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], [1.0, 3.0, 1.0], [7.0, 6.0, 8.0]]], [[[6.0, 7.0, 1.0], 
[5.0, 7.0, 5.0], [7.0, 4.0, 4.0]], [[3.0, 8.0, 3.0], [6.0, 3.0, 5.0], [1.0, 5.0, 4.0]], [[4.0, 4.0, 7.0], [6.0, 7.0, 7.0], 
[2.0, 1.0, 5.0]]], [[[4.0, 1.0, 5.0], [8.0, 3.0, 2.0], [2.0, 8.0, 7.0]], [[7.0, 3.0, 1.0], [6.0, 5.0, 6.0], [5.0, 1.0, 2.0]], 
[[2.0, 3.0, 8.0], [4.0, 3.0, 2.0], [6.0, 4.0, 6.0]]], [[[7.0, 7.0, 1.0], [8.0, 3.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 7.0, 8.0], 
[2.0, 6.0, 3.0], [5.0, 5.0, 4.0]], [[2.0, 2.0, 7.0], [5.0, 1.0, 6.0], [5.0, 3.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 3.0, 8.0]])+(1.-msk_val)*numpy.array([[1.0, 6.0, 2.0]])
      arg[2,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 6.0], [3.0, 7.0, 1.0], [5.0, 8.0, 3.0]], [[1.0, 5.0, 6.0], [7.0, 3.0, 5.0], [6.0, 
2.0, 4.0]], [[5.0, 3.0, 7.0], [5.0, 3.0, 2.0], [5.0, 4.0, 4.0]]], [[[5.0, 1.0, 2.0], [4.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[6.0, 
3.0, 7.0], [8.0, 3.0, 5.0], [2.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 4.0, 3.0], [3.0, 7.0, 1.0]]], [[[8.0, 7.0, 8.0], [5.0, 
7.0, 7.0], [4.0, 8.0, 4.0]], [[5.0, 2.0, 8.0], [8.0, 3.0, 8.0], [5.0, 8.0, 5.0]], [[1.0, 6.0, 6.0], [1.0, 1.0, 8.0], [1.0, 4.0, 
1.0]]], [[[7.0, 4.0, 5.0], [8.0, 7.0, 7.0], [3.0, 7.0, 6.0]], [[1.0, 6.0, 4.0], [2.0, 4.0, 1.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 
2.0], [7.0, 5.0, 8.0], [7.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 7.0], [4.0, 2.0, 4.0], [8.0, 4.0, 2.0]], 
[[4.0, 7.0, 7.0], [1.0, 7.0, 1.0], [2.0, 7.0, 5.0]], [[5.0, 4.0, 3.0], [1.0, 3.0, 1.0], [7.0, 6.0, 8.0]]], [[[6.0, 7.0, 1.0], 
[5.0, 7.0, 5.0], [7.0, 4.0, 4.0]], [[3.0, 8.0, 3.0], [6.0, 3.0, 5.0], [1.0, 5.0, 4.0]], [[4.0, 4.0, 7.0], [6.0, 7.0, 7.0], 
[2.0, 1.0, 5.0]]], [[[4.0, 1.0, 5.0], [8.0, 3.0, 2.0], [2.0, 8.0, 7.0]], [[7.0, 3.0, 1.0], [1.0, 6.0, 2.0], [5.0, 1.0, 2.0]], 
[[2.0, 3.0, 8.0], [4.0, 3.0, 2.0], [6.0, 4.0, 6.0]]], [[[7.0, 7.0, 1.0], [8.0, 3.0, 7.0], [8.0, 4.0, 1.0]], [[7.0, 7.0, 8.0], 
[2.0, 6.0, 3.0], [5.0, 5.0, 4.0]], [[2.0, 2.0, 7.0], [5.0, 1.0, 6.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 6.0], [6.0, 2.0, 4.0], [7.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [8.0, 8.0, 8.0], [3.0, 
1.0, 2.0]], [[5.0, 1.0, 5.0], [5.0, 2.0, 4.0], [6.0, 5.0, 7.0]]], [[[3.0, 3.0, 7.0], [3.0, 7.0, 5.0], [5.0, 6.0, 8.0]], [[5.0, 
8.0, 3.0], [2.0, 2.0, 5.0], [2.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [4.0, 3.0, 7.0], [4.0, 5.0, 1.0]]], [[[2.0, 2.0, 7.0], [3.0, 
5.0, 5.0], [1.0, 7.0, 5.0]], [[7.0, 5.0, 2.0], [3.0, 4.0, 5.0], [3.0, 6.0, 2.0]], [[7.0, 1.0, 1.0], [1.0, 4.0, 7.0], [8.0, 2.0, 
8.0]]], [[[7.0, 1.0, 1.0], [6.0, 7.0, 8.0], [7.0, 2.0, 7.0]], [[4.0, 7.0, 7.0], [4.0, 3.0, 3.0], [2.0, 1.0, 8.0]], [[1.0, 3.0, 
5.0], [1.0, 6.0, 7.0], [2.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 7.0], [8.0, 2.0, 6.0], [3.0, 4.0, 2.0]], 
[[5.0, 1.0, 8.0], [7.0, 7.0, 7.0], [6.0, 4.0, 8.0]], [[7.0, 6.0, 7.0], [1.0, 5.0, 3.0], [8.0, 3.0, 2.0]]], [[[2.0, 5.0, 5.0], 
[7.0, 8.0, 6.0], [2.0, 5.0, 5.0]], [[6.0, 7.0, 8.0], [2.0, 4.0, 2.0], [1.0, 8.0, 8.0]], [[7.0, 3.0, 3.0], [2.0, 2.0, 3.0], 
[4.0, 3.0, 4.0]]], [[[8.0, 3.0, 7.0], [6.0, 2.0, 8.0], [1.0, 7.0, 4.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 8.0], [4.0, 1.0, 1.0]], 
[[8.0, 5.0, 3.0], [7.0, 8.0, 6.0], [4.0, 5.0, 1.0]]], [[[4.0, 7.0, 6.0], [5.0, 5.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 6.0, 4.0], 
[5.0, 7.0, 7.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 6.0], [7.0, 8.0, 8.0], [7.0, 4.0, 3.0]]]])
      val=numpy.array([[[5.0, 8.0, 4.0]], [[5.0, 8.0, 7.0]]])
      arg[2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 6.0], [6.0, 2.0, 4.0], [7.0, 6.0, 4.0]], [[1.0, 6.0, 4.0], [8.0, 8.0, 8.0], [3.0, 
1.0, 2.0]], [[5.0, 1.0, 5.0], [5.0, 2.0, 4.0], [6.0, 5.0, 7.0]]], [[[3.0, 3.0, 7.0], [3.0, 7.0, 5.0], [5.0, 6.0, 8.0]], [[5.0, 
8.0, 3.0], [2.0, 2.0, 5.0], [2.0, 3.0, 7.0]], [[8.0, 3.0, 3.0], [4.0, 3.0, 7.0], [4.0, 5.0, 1.0]]], [[[2.0, 2.0, 7.0], [3.0, 
5.0, 5.0], [5.0, 8.0, 4.0]], [[7.0, 5.0, 2.0], [3.0, 4.0, 5.0], [5.0, 8.0, 7.0]], [[7.0, 1.0, 1.0], [1.0, 4.0, 7.0], [8.0, 2.0, 
8.0]]], [[[7.0, 1.0, 1.0], [6.0, 7.0, 8.0], [7.0, 2.0, 7.0]], [[4.0, 7.0, 7.0], [4.0, 3.0, 3.0], [2.0, 1.0, 8.0]], [[1.0, 3.0, 
5.0], [1.0, 6.0, 7.0], [2.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 7.0], [8.0, 2.0, 6.0], [3.0, 4.0, 2.0]], 
[[5.0, 1.0, 8.0], [7.0, 7.0, 7.0], [6.0, 4.0, 8.0]], [[7.0, 6.0, 7.0], [1.0, 5.0, 3.0], [8.0, 3.0, 2.0]]], [[[2.0, 5.0, 5.0], 
[7.0, 8.0, 6.0], [2.0, 5.0, 5.0]], [[6.0, 7.0, 8.0], [2.0, 4.0, 2.0], [1.0, 8.0, 8.0]], [[7.0, 3.0, 3.0], [2.0, 2.0, 3.0], 
[4.0, 3.0, 4.0]]], [[[8.0, 3.0, 7.0], [6.0, 2.0, 8.0], [5.0, 8.0, 4.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 8.0], [5.0, 8.0, 7.0]], 
[[8.0, 5.0, 3.0], [7.0, 8.0, 6.0], [4.0, 5.0, 1.0]]], [[[4.0, 7.0, 6.0], [5.0, 5.0, 8.0], [8.0, 4.0, 1.0]], [[4.0, 6.0, 4.0], 
[5.0, 7.0, 7.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 6.0], [7.0, 8.0, 8.0], [7.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 1.0], [7.0, 7.0, 4.0], [3.0, 5.0, 7.0]], [[5.0, 2.0, 1.0], [8.0, 3.0, 1.0], [6.0, 
6.0, 3.0]], [[8.0, 1.0, 3.0], [4.0, 7.0, 7.0], [1.0, 4.0, 1.0]]], [[[1.0, 8.0, 7.0], [8.0, 6.0, 2.0], [2.0, 2.0, 7.0]], [[8.0, 
1.0, 3.0], [2.0, 4.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 6.0, 1.0], [5.0, 5.0, 7.0], [1.0, 6.0, 6.0]]], [[[5.0, 3.0, 8.0], [4.0, 
2.0, 3.0], [8.0, 6.0, 8.0]], [[8.0, 6.0, 2.0], [5.0, 8.0, 4.0], [1.0, 5.0, 6.0]], [[5.0, 6.0, 1.0], [5.0, 6.0, 2.0], [1.0, 7.0, 
2.0]]], [[[2.0, 7.0, 8.0], [1.0, 2.0, 5.0], [3.0, 1.0, 7.0]], [[5.0, 3.0, 6.0], [8.0, 5.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 
3.0], [7.0, 1.0, 7.0], [8.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 1.0], [7.0, 6.0, 7.0], [6.0, 5.0, 6.0]], 
[[8.0, 2.0, 2.0], [3.0, 4.0, 4.0], [6.0, 6.0, 4.0]], [[4.0, 3.0, 5.0], [4.0, 8.0, 5.0], [5.0, 2.0, 6.0]]], [[[2.0, 1.0, 8.0], 
[8.0, 4.0, 6.0], [8.0, 7.0, 1.0]], [[5.0, 3.0, 1.0], [4.0, 7.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 2.0], [1.0, 2.0, 8.0], 
[4.0, 8.0, 7.0]]], [[[3.0, 3.0, 1.0], [8.0, 7.0, 4.0], [1.0, 8.0, 2.0]], [[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [7.0, 4.0, 5.0]], 
[[4.0, 4.0, 7.0], [1.0, 4.0, 4.0], [5.0, 2.0, 2.0]]], [[[8.0, 2.0, 5.0], [1.0, 3.0, 6.0], [5.0, 2.0, 5.0]], [[1.0, 3.0, 3.0], 
[5.0, 7.0, 1.0], [8.0, 7.0, 5.0]], [[7.0, 6.0, 8.0], [3.0, 4.0, 7.0], [4.0, 4.0, 5.0]]]])
      val=Data(numpy.array([[[6.0, 8.0, 1.0]], [[3.0, 7.0, 5.0]]]),self.functionspace)
      arg[2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 1.0], [7.0, 7.0, 4.0], [3.0, 5.0, 7.0]], [[5.0, 2.0, 1.0], [8.0, 3.0, 1.0], [6.0, 
6.0, 3.0]], [[8.0, 1.0, 3.0], [4.0, 7.0, 7.0], [1.0, 4.0, 1.0]]], [[[1.0, 8.0, 7.0], [8.0, 6.0, 2.0], [2.0, 2.0, 7.0]], [[8.0, 
1.0, 3.0], [2.0, 4.0, 6.0], [3.0, 1.0, 4.0]], [[1.0, 6.0, 1.0], [5.0, 5.0, 7.0], [1.0, 6.0, 6.0]]], [[[5.0, 3.0, 8.0], [4.0, 
2.0, 3.0], [6.0, 8.0, 1.0]], [[8.0, 6.0, 2.0], [5.0, 8.0, 4.0], [3.0, 7.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 6.0, 2.0], [1.0, 7.0, 
2.0]]], [[[2.0, 7.0, 8.0], [1.0, 2.0, 5.0], [3.0, 1.0, 7.0]], [[5.0, 3.0, 6.0], [8.0, 5.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 5.0, 
3.0], [7.0, 1.0, 7.0], [8.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [7.0, 6.0, 7.0], [6.0, 5.0, 6.0]], 
[[8.0, 2.0, 2.0], [3.0, 4.0, 4.0], [6.0, 6.0, 4.0]], [[4.0, 3.0, 5.0], [4.0, 8.0, 5.0], [5.0, 2.0, 6.0]]], [[[2.0, 1.0, 8.0], 
[8.0, 4.0, 6.0], [8.0, 7.0, 1.0]], [[5.0, 3.0, 1.0], [4.0, 7.0, 1.0], [4.0, 8.0, 5.0]], [[4.0, 5.0, 2.0], [1.0, 2.0, 8.0], 
[4.0, 8.0, 7.0]]], [[[3.0, 3.0, 1.0], [8.0, 7.0, 4.0], [6.0, 8.0, 1.0]], [[3.0, 2.0, 2.0], [6.0, 1.0, 4.0], [3.0, 7.0, 5.0]], 
[[4.0, 4.0, 7.0], [1.0, 4.0, 4.0], [5.0, 2.0, 2.0]]], [[[8.0, 2.0, 5.0], [1.0, 3.0, 6.0], [5.0, 2.0, 5.0]], [[1.0, 3.0, 3.0], 
[5.0, 7.0, 1.0], [8.0, 7.0, 5.0]], [[7.0, 6.0, 8.0], [3.0, 4.0, 7.0], [4.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 1.0], [5.0, 3.0, 3.0], [6.0, 1.0, 1.0]], [[5.0, 8.0, 8.0], [2.0, 8.0, 7.0], [5.0, 
4.0, 5.0]], [[2.0, 7.0, 3.0], [5.0, 7.0, 7.0], [8.0, 6.0, 6.0]]], [[[3.0, 8.0, 3.0], [4.0, 2.0, 1.0], [3.0, 5.0, 8.0]], [[1.0, 
3.0, 3.0], [8.0, 3.0, 1.0], [2.0, 5.0, 8.0]], [[7.0, 7.0, 4.0], [6.0, 5.0, 8.0], [5.0, 2.0, 8.0]]], [[[4.0, 6.0, 8.0], [1.0, 
8.0, 6.0], [7.0, 8.0, 7.0]], [[3.0, 4.0, 5.0], [4.0, 3.0, 1.0], [1.0, 6.0, 3.0]], [[4.0, 1.0, 2.0], [6.0, 2.0, 8.0], [3.0, 4.0, 
5.0]]], [[[7.0, 1.0, 1.0], [5.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 6.0], [7.0, 1.0, 2.0]], [[2.0, 8.0, 
8.0], [8.0, 6.0, 2.0], [6.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 4.0], [8.0, 4.0, 2.0], [6.0, 7.0, 7.0]], 
[[4.0, 3.0, 8.0], [8.0, 3.0, 7.0], [2.0, 3.0, 6.0]], [[5.0, 1.0, 5.0], [6.0, 3.0, 8.0], [1.0, 8.0, 5.0]]], [[[2.0, 7.0, 8.0], 
[2.0, 6.0, 4.0], [1.0, 4.0, 6.0]], [[8.0, 1.0, 5.0], [3.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 3.0, 8.0], [4.0, 8.0, 8.0], 
[1.0, 8.0, 6.0]]], [[[5.0, 6.0, 8.0], [7.0, 6.0, 6.0], [1.0, 1.0, 3.0]], [[2.0, 6.0, 5.0], [8.0, 3.0, 2.0], [8.0, 4.0, 7.0]], 
[[4.0, 8.0, 6.0], [2.0, 5.0, 1.0], [2.0, 8.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 2.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 8.0, 4.0], 
[4.0, 4.0, 2.0], [5.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [8.0, 1.0, 8.0], [6.0, 3.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 2.0, 5.0]], [[1.0, 7.0, 7.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 4.0]], [[6.0, 1.0, 
1.0]]])
      arg[2,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 1.0], [5.0, 3.0, 3.0], [6.0, 1.0, 1.0]], [[5.0, 8.0, 8.0], [2.0, 8.0, 7.0], [5.0, 
4.0, 5.0]], [[2.0, 7.0, 3.0], [5.0, 7.0, 7.0], [8.0, 6.0, 6.0]]], [[[3.0, 8.0, 3.0], [4.0, 2.0, 1.0], [3.0, 5.0, 8.0]], [[1.0, 
3.0, 3.0], [8.0, 3.0, 1.0], [2.0, 5.0, 8.0]], [[7.0, 7.0, 4.0], [6.0, 5.0, 8.0], [5.0, 2.0, 8.0]]], [[[4.0, 6.0, 8.0], [1.0, 
8.0, 6.0], [8.0, 2.0, 5.0]], [[3.0, 4.0, 5.0], [4.0, 3.0, 1.0], [1.0, 7.0, 7.0]], [[4.0, 1.0, 2.0], [6.0, 2.0, 8.0], [3.0, 4.0, 
5.0]]], [[[7.0, 1.0, 1.0], [5.0, 4.0, 3.0], [3.0, 3.0, 7.0]], [[1.0, 1.0, 6.0], [5.0, 7.0, 6.0], [7.0, 1.0, 2.0]], [[2.0, 8.0, 
8.0], [8.0, 6.0, 2.0], [6.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 4.0], [8.0, 4.0, 2.0], [6.0, 7.0, 7.0]], 
[[4.0, 3.0, 8.0], [8.0, 3.0, 7.0], [2.0, 3.0, 6.0]], [[5.0, 1.0, 5.0], [6.0, 3.0, 8.0], [1.0, 8.0, 5.0]]], [[[2.0, 7.0, 8.0], 
[2.0, 6.0, 4.0], [1.0, 4.0, 6.0]], [[8.0, 1.0, 5.0], [3.0, 2.0, 7.0], [6.0, 4.0, 5.0]], [[6.0, 3.0, 8.0], [4.0, 8.0, 8.0], 
[1.0, 8.0, 6.0]]], [[[5.0, 6.0, 8.0], [7.0, 6.0, 6.0], [8.0, 1.0, 4.0]], [[2.0, 6.0, 5.0], [8.0, 3.0, 2.0], [6.0, 1.0, 1.0]], 
[[4.0, 8.0, 6.0], [2.0, 5.0, 1.0], [2.0, 8.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 2.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 8.0, 4.0], 
[4.0, 4.0, 2.0], [5.0, 6.0, 2.0]], [[4.0, 1.0, 6.0], [8.0, 1.0, 8.0], [6.0, 3.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 6.0], [1.0, 2.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 4.0, 7.0], [6.0, 4.0, 8.0], [4.0, 
8.0, 1.0]], [[3.0, 3.0, 6.0], [7.0, 5.0, 5.0], [3.0, 8.0, 8.0]]], [[[2.0, 4.0, 8.0], [2.0, 1.0, 4.0], [2.0, 3.0, 8.0]], [[3.0, 
8.0, 4.0], [7.0, 7.0, 1.0], [1.0, 5.0, 5.0]], [[1.0, 5.0, 4.0], [6.0, 2.0, 2.0], [3.0, 8.0, 6.0]]], [[[6.0, 1.0, 5.0], [3.0, 
1.0, 6.0], [5.0, 1.0, 7.0]], [[5.0, 8.0, 6.0], [2.0, 1.0, 7.0], [7.0, 3.0, 5.0]], [[3.0, 8.0, 1.0], [3.0, 8.0, 3.0], [8.0, 6.0, 
1.0]]], [[[7.0, 7.0, 1.0], [5.0, 5.0, 3.0], [3.0, 5.0, 5.0]], [[8.0, 2.0, 8.0], [8.0, 6.0, 8.0], [7.0, 8.0, 3.0]], [[6.0, 7.0, 
7.0], [8.0, 1.0, 3.0], [1.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 3.0], [6.0, 3.0, 1.0], [3.0, 5.0, 7.0]], 
[[2.0, 6.0, 6.0], [4.0, 2.0, 7.0], [6.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [7.0, 5.0, 3.0], [3.0, 2.0, 3.0]]], [[[6.0, 4.0, 1.0], 
[6.0, 6.0, 5.0], [4.0, 8.0, 7.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 4.0], [5.0, 8.0, 3.0]], [[7.0, 5.0, 2.0], [2.0, 8.0, 6.0], 
[2.0, 7.0, 3.0]]], [[[1.0, 3.0, 1.0], [8.0, 2.0, 2.0], [5.0, 8.0, 6.0]], [[3.0, 3.0, 2.0], [1.0, 3.0, 6.0], [8.0, 8.0, 2.0]], 
[[7.0, 7.0, 7.0], [2.0, 7.0, 6.0], [7.0, 6.0, 4.0]]], [[[3.0, 8.0, 5.0], [7.0, 5.0, 2.0], [1.0, 3.0, 7.0]], [[3.0, 8.0, 3.0], 
[7.0, 2.0, 7.0], [8.0, 4.0, 8.0]], [[5.0, 7.0, 8.0], [1.0, 2.0, 4.0], [3.0, 3.0, 7.0]]]])
      val=numpy.array([[[[3.0, 7.0, 2.0]], [[1.0, 1.0, 5.0]], [[6.0, 7.0, 7.0]]], [[[7.0, 2.0, 7.0]], [[2.0, 4.0, 8.0]], [[3.0, 
2.0, 3.0]]]])
      arg[2:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 6.0], [1.0, 2.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 4.0, 7.0], [6.0, 4.0, 8.0], [4.0, 
8.0, 1.0]], [[3.0, 3.0, 6.0], [7.0, 5.0, 5.0], [3.0, 8.0, 8.0]]], [[[2.0, 4.0, 8.0], [2.0, 1.0, 4.0], [2.0, 3.0, 8.0]], [[3.0, 
8.0, 4.0], [7.0, 7.0, 1.0], [1.0, 5.0, 5.0]], [[1.0, 5.0, 4.0], [6.0, 2.0, 2.0], [3.0, 8.0, 6.0]]], [[[6.0, 1.0, 5.0], [3.0, 
1.0, 6.0], [3.0, 7.0, 2.0]], [[5.0, 8.0, 6.0], [2.0, 1.0, 7.0], [1.0, 1.0, 5.0]], [[3.0, 8.0, 1.0], [3.0, 8.0, 3.0], [6.0, 7.0, 
7.0]]], [[[7.0, 7.0, 1.0], [5.0, 5.0, 3.0], [7.0, 2.0, 7.0]], [[8.0, 2.0, 8.0], [8.0, 6.0, 8.0], [2.0, 4.0, 8.0]], [[6.0, 7.0, 
7.0], [8.0, 1.0, 3.0], [3.0, 2.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 3.0], [6.0, 3.0, 1.0], [3.0, 5.0, 7.0]], 
[[2.0, 6.0, 6.0], [4.0, 2.0, 7.0], [6.0, 7.0, 2.0]], [[8.0, 8.0, 7.0], [7.0, 5.0, 3.0], [3.0, 2.0, 3.0]]], [[[6.0, 4.0, 1.0], 
[6.0, 6.0, 5.0], [4.0, 8.0, 7.0]], [[1.0, 1.0, 6.0], [7.0, 7.0, 4.0], [5.0, 8.0, 3.0]], [[7.0, 5.0, 2.0], [2.0, 8.0, 6.0], 
[2.0, 7.0, 3.0]]], [[[1.0, 3.0, 1.0], [8.0, 2.0, 2.0], [3.0, 7.0, 2.0]], [[3.0, 3.0, 2.0], [1.0, 3.0, 6.0], [1.0, 1.0, 5.0]], 
[[7.0, 7.0, 7.0], [2.0, 7.0, 6.0], [6.0, 7.0, 7.0]]], [[[3.0, 8.0, 5.0], [7.0, 5.0, 2.0], [7.0, 2.0, 7.0]], [[3.0, 8.0, 3.0], 
[7.0, 2.0, 7.0], [2.0, 4.0, 8.0]], [[5.0, 7.0, 8.0], [1.0, 2.0, 4.0], [3.0, 2.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 7.0], [6.0, 8.0, 6.0], [1.0, 6.0, 8.0]], [[1.0, 3.0, 3.0], [1.0, 2.0, 6.0], [4.0, 
1.0, 1.0]], [[8.0, 8.0, 8.0], [5.0, 4.0, 4.0], [6.0, 8.0, 6.0]]], [[[6.0, 8.0, 5.0], [7.0, 2.0, 6.0], [6.0, 8.0, 3.0]], [[6.0, 
1.0, 8.0], [8.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 4.0, 4.0], [6.0, 5.0, 8.0], [1.0, 6.0, 5.0]]], [[[5.0, 2.0, 3.0], [8.0, 
5.0, 4.0], [1.0, 3.0, 4.0]], [[7.0, 7.0, 5.0], [1.0, 1.0, 7.0], [2.0, 1.0, 7.0]], [[6.0, 6.0, 6.0], [2.0, 6.0, 5.0], [8.0, 3.0, 
6.0]]], [[[7.0, 2.0, 8.0], [1.0, 2.0, 6.0], [1.0, 5.0, 1.0]], [[7.0, 4.0, 4.0], [4.0, 2.0, 6.0], [7.0, 8.0, 1.0]], [[2.0, 3.0, 
4.0], [8.0, 5.0, 8.0], [7.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 8.0], [1.0, 6.0, 4.0], [8.0, 4.0, 8.0]], 
[[3.0, 4.0, 1.0], [2.0, 8.0, 3.0], [8.0, 6.0, 5.0]], [[4.0, 5.0, 2.0], [8.0, 6.0, 7.0], [8.0, 5.0, 2.0]]], [[[5.0, 2.0, 3.0], 
[8.0, 5.0, 4.0], [6.0, 8.0, 5.0]], [[2.0, 6.0, 2.0], [4.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 3.0], 
[2.0, 1.0, 1.0]]], [[[6.0, 4.0, 4.0], [2.0, 1.0, 3.0], [3.0, 8.0, 3.0]], [[2.0, 3.0, 7.0], [4.0, 8.0, 3.0], [5.0, 6.0, 1.0]], 
[[4.0, 4.0, 1.0], [6.0, 5.0, 7.0], [6.0, 4.0, 6.0]]], [[[4.0, 1.0, 5.0], [1.0, 8.0, 1.0], [8.0, 3.0, 8.0]], [[4.0, 3.0, 3.0], 
[7.0, 1.0, 4.0], [1.0, 7.0, 4.0]], [[5.0, 2.0, 2.0], [8.0, 2.0, 2.0], [3.0, 8.0, 2.0]]]])
      val=Data(numpy.array([[[[5.0, 5.0, 4.0]], [[6.0, 6.0, 8.0]], [[8.0, 5.0, 4.0]]], [[[8.0, 2.0, 8.0]], [[7.0, 5.0, 6.0]], 
[[5.0, 8.0, 7.0]]]]),self.functionspace)
      arg[2:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 7.0], [6.0, 8.0, 6.0], [1.0, 6.0, 8.0]], [[1.0, 3.0, 3.0], [1.0, 2.0, 6.0], [4.0, 
1.0, 1.0]], [[8.0, 8.0, 8.0], [5.0, 4.0, 4.0], [6.0, 8.0, 6.0]]], [[[6.0, 8.0, 5.0], [7.0, 2.0, 6.0], [6.0, 8.0, 3.0]], [[6.0, 
1.0, 8.0], [8.0, 2.0, 8.0], [3.0, 4.0, 8.0]], [[7.0, 4.0, 4.0], [6.0, 5.0, 8.0], [1.0, 6.0, 5.0]]], [[[5.0, 2.0, 3.0], [8.0, 
5.0, 4.0], [5.0, 5.0, 4.0]], [[7.0, 7.0, 5.0], [1.0, 1.0, 7.0], [6.0, 6.0, 8.0]], [[6.0, 6.0, 6.0], [2.0, 6.0, 5.0], [8.0, 5.0, 
4.0]]], [[[7.0, 2.0, 8.0], [1.0, 2.0, 6.0], [8.0, 2.0, 8.0]], [[7.0, 4.0, 4.0], [4.0, 2.0, 6.0], [7.0, 5.0, 6.0]], [[2.0, 3.0, 
4.0], [8.0, 5.0, 8.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 8.0], [1.0, 6.0, 4.0], [8.0, 4.0, 8.0]], 
[[3.0, 4.0, 1.0], [2.0, 8.0, 3.0], [8.0, 6.0, 5.0]], [[4.0, 5.0, 2.0], [8.0, 6.0, 7.0], [8.0, 5.0, 2.0]]], [[[5.0, 2.0, 3.0], 
[8.0, 5.0, 4.0], [6.0, 8.0, 5.0]], [[2.0, 6.0, 2.0], [4.0, 4.0, 8.0], [6.0, 8.0, 1.0]], [[8.0, 8.0, 1.0], [8.0, 8.0, 3.0], 
[2.0, 1.0, 1.0]]], [[[6.0, 4.0, 4.0], [2.0, 1.0, 3.0], [5.0, 5.0, 4.0]], [[2.0, 3.0, 7.0], [4.0, 8.0, 3.0], [6.0, 6.0, 8.0]], 
[[4.0, 4.0, 1.0], [6.0, 5.0, 7.0], [8.0, 5.0, 4.0]]], [[[4.0, 1.0, 5.0], [1.0, 8.0, 1.0], [8.0, 2.0, 8.0]], [[4.0, 3.0, 3.0], 
[7.0, 1.0, 4.0], [7.0, 5.0, 6.0]], [[5.0, 2.0, 2.0], [8.0, 2.0, 2.0], [5.0, 8.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 8.0, 3.0], [6.0, 5.0, 1.0], [6.0, 4.0, 6.0]], [[8.0, 5.0, 3.0], [8.0, 8.0, 4.0], [7.0, 
8.0, 2.0]], [[1.0, 6.0, 1.0], [1.0, 4.0, 7.0], [7.0, 4.0, 3.0]]], [[[6.0, 6.0, 7.0], [6.0, 7.0, 3.0], [6.0, 2.0, 1.0]], [[8.0, 
6.0, 8.0], [8.0, 2.0, 8.0], [8.0, 8.0, 1.0]], [[7.0, 2.0, 7.0], [8.0, 8.0, 3.0], [3.0, 4.0, 4.0]]], [[[8.0, 3.0, 1.0], [7.0, 
8.0, 8.0], [1.0, 4.0, 1.0]], [[8.0, 4.0, 3.0], [3.0, 3.0, 5.0], [6.0, 1.0, 4.0]], [[1.0, 1.0, 7.0], [6.0, 5.0, 2.0], [1.0, 2.0, 
3.0]]], [[[7.0, 5.0, 1.0], [6.0, 2.0, 5.0], [6.0, 4.0, 2.0]], [[2.0, 8.0, 6.0], [4.0, 5.0, 2.0], [6.0, 5.0, 3.0]], [[6.0, 6.0, 
8.0], [4.0, 5.0, 7.0], [5.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 6.0], [2.0, 4.0, 1.0], [2.0, 8.0, 4.0]], 
[[7.0, 3.0, 2.0], [5.0, 2.0, 6.0], [1.0, 5.0, 2.0]], [[5.0, 1.0, 8.0], [7.0, 4.0, 1.0], [4.0, 3.0, 6.0]]], [[[6.0, 3.0, 6.0], 
[4.0, 3.0, 5.0], [6.0, 1.0, 8.0]], [[7.0, 5.0, 1.0], [6.0, 4.0, 2.0], [2.0, 8.0, 3.0]], [[1.0, 1.0, 3.0], [2.0, 2.0, 7.0], 
[7.0, 5.0, 1.0]]], [[[6.0, 1.0, 6.0], [8.0, 3.0, 5.0], [2.0, 2.0, 8.0]], [[7.0, 4.0, 2.0], [4.0, 2.0, 6.0], [1.0, 7.0, 3.0]], 
[[1.0, 5.0, 8.0], [7.0, 7.0, 6.0], [7.0, 3.0, 7.0]]], [[[2.0, 8.0, 8.0], [2.0, 8.0, 4.0], [6.0, 1.0, 6.0]], [[3.0, 1.0, 3.0], 
[8.0, 8.0, 7.0], [1.0, 3.0, 8.0]], [[8.0, 6.0, 6.0], [3.0, 2.0, 7.0], [5.0, 7.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 4.0, 6.0]], [[8.0, 1.0, 5.0]], [[2.0, 3.0, 5.0]]], [[[5.0, 7.0, 8.0]], [[8.0, 5.0, 
3.0]], [[2.0, 8.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 7.0, 8.0]], [[3.0, 4.0, 2.0]], [[2.0, 7.0, 8.0]]], [[[5.0, 7.0, 
8.0]], [[8.0, 3.0, 3.0]], [[3.0, 8.0, 6.0]]]])
      arg[2:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 8.0, 3.0], [6.0, 5.0, 1.0], [6.0, 4.0, 6.0]], [[8.0, 5.0, 3.0], [8.0, 8.0, 4.0], [7.0, 
8.0, 2.0]], [[1.0, 6.0, 1.0], [1.0, 4.0, 7.0], [7.0, 4.0, 3.0]]], [[[6.0, 6.0, 7.0], [6.0, 7.0, 3.0], [6.0, 2.0, 1.0]], [[8.0, 
6.0, 8.0], [8.0, 2.0, 8.0], [8.0, 8.0, 1.0]], [[7.0, 2.0, 7.0], [8.0, 8.0, 3.0], [3.0, 4.0, 4.0]]], [[[8.0, 3.0, 1.0], [7.0, 
8.0, 8.0], [7.0, 4.0, 6.0]], [[8.0, 4.0, 3.0], [3.0, 3.0, 5.0], [8.0, 1.0, 5.0]], [[1.0, 1.0, 7.0], [6.0, 5.0, 2.0], [2.0, 3.0, 
5.0]]], [[[7.0, 5.0, 1.0], [6.0, 2.0, 5.0], [5.0, 7.0, 8.0]], [[2.0, 8.0, 6.0], [4.0, 5.0, 2.0], [8.0, 5.0, 3.0]], [[6.0, 6.0, 
8.0], [4.0, 5.0, 7.0], [2.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 6.0], [2.0, 4.0, 1.0], [2.0, 8.0, 4.0]], 
[[7.0, 3.0, 2.0], [5.0, 2.0, 6.0], [1.0, 5.0, 2.0]], [[5.0, 1.0, 8.0], [7.0, 4.0, 1.0], [4.0, 3.0, 6.0]]], [[[6.0, 3.0, 6.0], 
[4.0, 3.0, 5.0], [6.0, 1.0, 8.0]], [[7.0, 5.0, 1.0], [6.0, 4.0, 2.0], [2.0, 8.0, 3.0]], [[1.0, 1.0, 3.0], [2.0, 2.0, 7.0], 
[7.0, 5.0, 1.0]]], [[[6.0, 1.0, 6.0], [8.0, 3.0, 5.0], [7.0, 7.0, 8.0]], [[7.0, 4.0, 2.0], [4.0, 2.0, 6.0], [3.0, 4.0, 2.0]], 
[[1.0, 5.0, 8.0], [7.0, 7.0, 6.0], [2.0, 7.0, 8.0]]], [[[2.0, 8.0, 8.0], [2.0, 8.0, 4.0], [5.0, 7.0, 8.0]], [[3.0, 1.0, 3.0], 
[8.0, 8.0, 7.0], [8.0, 3.0, 3.0]], [[8.0, 6.0, 6.0], [3.0, 2.0, 7.0], [3.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 2.0, 3.0], [8.0, 8.0, 1.0], [7.0, 1.0, 7.0]], [[8.0, 6.0, 7.0], [5.0, 1.0, 8.0], [2.0, 
8.0, 6.0]], [[1.0, 1.0, 8.0], [4.0, 4.0, 5.0], [1.0, 2.0, 7.0]]], [[[4.0, 5.0, 5.0], [7.0, 5.0, 5.0], [1.0, 2.0, 5.0]], [[6.0, 
1.0, 7.0], [1.0, 5.0, 6.0], [5.0, 5.0, 7.0]], [[5.0, 2.0, 5.0], [1.0, 4.0, 5.0], [5.0, 3.0, 6.0]]], [[[3.0, 5.0, 7.0], [1.0, 
5.0, 2.0], [7.0, 1.0, 3.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 5.0], [1.0, 8.0, 7.0]], [[6.0, 5.0, 2.0], [5.0, 4.0, 5.0], [7.0, 4.0, 
1.0]]], [[[2.0, 2.0, 3.0], [7.0, 5.0, 5.0], [1.0, 2.0, 1.0]], [[4.0, 2.0, 1.0], [2.0, 8.0, 5.0], [6.0, 3.0, 4.0]], [[8.0, 7.0, 
5.0], [4.0, 6.0, 8.0], [8.0, 3.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 7.0], [2.0, 5.0, 1.0], [1.0, 5.0, 1.0]], 
[[4.0, 6.0, 4.0], [8.0, 8.0, 1.0], [6.0, 4.0, 7.0]], [[4.0, 2.0, 5.0], [1.0, 4.0, 6.0], [7.0, 2.0, 7.0]]], [[[7.0, 7.0, 7.0], 
[2.0, 2.0, 3.0], [5.0, 4.0, 1.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 1.0], [2.0, 6.0, 3.0]], [[8.0, 5.0, 6.0], [2.0, 4.0, 7.0], 
[3.0, 2.0, 2.0]]], [[[2.0, 3.0, 8.0], [7.0, 6.0, 1.0], [7.0, 8.0, 5.0]], [[8.0, 4.0, 7.0], [1.0, 6.0, 1.0], [6.0, 2.0, 7.0]], 
[[5.0, 1.0, 3.0], [1.0, 2.0, 3.0], [3.0, 7.0, 6.0]]], [[[3.0, 2.0, 8.0], [6.0, 4.0, 3.0], [1.0, 1.0, 8.0]], [[3.0, 5.0, 8.0], 
[8.0, 6.0, 2.0], [3.0, 7.0, 2.0]], [[1.0, 7.0, 7.0], [8.0, 8.0, 3.0], [2.0, 3.0, 8.0]]]])
      val=numpy.array([[[[3.0, 3.0, 6.0], [4.0, 6.0, 1.0]], [[7.0, 2.0, 4.0], [8.0, 1.0, 6.0]], [[8.0, 3.0, 7.0], [1.0, 2.0, 
1.0]]], [[[3.0, 4.0, 3.0], [6.0, 2.0, 8.0]], [[1.0, 4.0, 3.0], [2.0, 6.0, 8.0]], [[8.0, 1.0, 7.0], [1.0, 1.0, 7.0]]]])
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 3.0], [8.0, 8.0, 1.0], [7.0, 1.0, 7.0]], [[8.0, 6.0, 7.0], [5.0, 1.0, 8.0], [2.0, 
8.0, 6.0]], [[1.0, 1.0, 8.0], [4.0, 4.0, 5.0], [1.0, 2.0, 7.0]]], [[[4.0, 5.0, 5.0], [7.0, 5.0, 5.0], [1.0, 2.0, 5.0]], [[6.0, 
1.0, 7.0], [1.0, 5.0, 6.0], [5.0, 5.0, 7.0]], [[5.0, 2.0, 5.0], [1.0, 4.0, 5.0], [5.0, 3.0, 6.0]]], [[[3.0, 3.0, 6.0], [4.0, 
6.0, 1.0], [7.0, 1.0, 3.0]], [[7.0, 2.0, 4.0], [8.0, 1.0, 6.0], [1.0, 8.0, 7.0]], [[8.0, 3.0, 7.0], [1.0, 2.0, 1.0], [7.0, 4.0, 
1.0]]], [[[3.0, 4.0, 3.0], [6.0, 2.0, 8.0], [1.0, 2.0, 1.0]], [[1.0, 4.0, 3.0], [2.0, 6.0, 8.0], [6.0, 3.0, 4.0]], [[8.0, 1.0, 
7.0], [1.0, 1.0, 7.0], [8.0, 3.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 7.0], [2.0, 5.0, 1.0], [1.0, 5.0, 1.0]], 
[[4.0, 6.0, 4.0], [8.0, 8.0, 1.0], [6.0, 4.0, 7.0]], [[4.0, 2.0, 5.0], [1.0, 4.0, 6.0], [7.0, 2.0, 7.0]]], [[[7.0, 7.0, 7.0], 
[2.0, 2.0, 3.0], [5.0, 4.0, 1.0]], [[6.0, 2.0, 5.0], [2.0, 5.0, 1.0], [2.0, 6.0, 3.0]], [[8.0, 5.0, 6.0], [2.0, 4.0, 7.0], 
[3.0, 2.0, 2.0]]], [[[3.0, 3.0, 6.0], [4.0, 6.0, 1.0], [7.0, 8.0, 5.0]], [[7.0, 2.0, 4.0], [8.0, 1.0, 6.0], [6.0, 2.0, 7.0]], 
[[8.0, 3.0, 7.0], [1.0, 2.0, 1.0], [3.0, 7.0, 6.0]]], [[[3.0, 4.0, 3.0], [6.0, 2.0, 8.0], [1.0, 1.0, 8.0]], [[1.0, 4.0, 3.0], 
[2.0, 6.0, 8.0], [3.0, 7.0, 2.0]], [[8.0, 1.0, 7.0], [1.0, 1.0, 7.0], [2.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 6.0], [1.0, 5.0, 5.0], [5.0, 7.0, 8.0]], [[8.0, 5.0, 7.0], [1.0, 4.0, 1.0], [3.0, 
3.0, 3.0]], [[4.0, 3.0, 3.0], [6.0, 4.0, 1.0], [4.0, 1.0, 4.0]]], [[[6.0, 2.0, 7.0], [5.0, 3.0, 2.0], [2.0, 5.0, 4.0]], [[2.0, 
8.0, 6.0], [1.0, 1.0, 8.0], [2.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [4.0, 2.0, 3.0], [5.0, 6.0, 5.0]]], [[[8.0, 3.0, 5.0], [3.0, 
2.0, 3.0], [2.0, 4.0, 5.0]], [[8.0, 4.0, 5.0], [3.0, 7.0, 6.0], [6.0, 3.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 4.0, 5.0], [5.0, 1.0, 
5.0]]], [[[5.0, 3.0, 5.0], [6.0, 5.0, 4.0], [7.0, 1.0, 5.0]], [[8.0, 5.0, 8.0], [4.0, 3.0, 6.0], [4.0, 7.0, 4.0]], [[6.0, 6.0, 
8.0], [1.0, 7.0, 7.0], [7.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 6.0], [5.0, 5.0, 6.0], [4.0, 8.0, 4.0]], 
[[1.0, 4.0, 2.0], [5.0, 5.0, 7.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 7.0, 4.0], [3.0, 5.0, 3.0]]], [[[5.0, 3.0, 3.0], 
[3.0, 6.0, 4.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 1.0], [5.0, 5.0, 4.0], [7.0, 7.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 1.0, 8.0], 
[4.0, 5.0, 2.0]]], [[[6.0, 5.0, 1.0], [8.0, 2.0, 4.0], [5.0, 4.0, 6.0]], [[3.0, 5.0, 8.0], [1.0, 7.0, 2.0], [2.0, 7.0, 6.0]], 
[[2.0, 1.0, 2.0], [6.0, 6.0, 8.0], [6.0, 2.0, 1.0]]], [[[5.0, 8.0, 6.0], [1.0, 2.0, 6.0], [6.0, 5.0, 4.0]], [[8.0, 2.0, 7.0], 
[5.0, 3.0, 7.0], [2.0, 7.0, 6.0]], [[7.0, 2.0, 3.0], [8.0, 1.0, 6.0], [7.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[[5.0, 3.0, 5.0], [5.0, 7.0, 6.0]], [[2.0, 2.0, 2.0], [1.0, 8.0, 2.0]], [[8.0, 5.0, 7.0], [2.0, 
7.0, 8.0]]], [[[7.0, 3.0, 3.0], [1.0, 2.0, 4.0]], [[1.0, 1.0, 8.0], [7.0, 8.0, 4.0]], [[4.0, 3.0, 8.0], [5.0, 1.0, 
2.0]]]]),self.functionspace)
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 6.0], [1.0, 5.0, 5.0], [5.0, 7.0, 8.0]], [[8.0, 5.0, 7.0], [1.0, 4.0, 1.0], [3.0, 
3.0, 3.0]], [[4.0, 3.0, 3.0], [6.0, 4.0, 1.0], [4.0, 1.0, 4.0]]], [[[6.0, 2.0, 7.0], [5.0, 3.0, 2.0], [2.0, 5.0, 4.0]], [[2.0, 
8.0, 6.0], [1.0, 1.0, 8.0], [2.0, 5.0, 8.0]], [[4.0, 2.0, 2.0], [4.0, 2.0, 3.0], [5.0, 6.0, 5.0]]], [[[5.0, 3.0, 5.0], [5.0, 
7.0, 6.0], [2.0, 4.0, 5.0]], [[2.0, 2.0, 2.0], [1.0, 8.0, 2.0], [6.0, 3.0, 5.0]], [[8.0, 5.0, 7.0], [2.0, 7.0, 8.0], [5.0, 1.0, 
5.0]]], [[[7.0, 3.0, 3.0], [1.0, 2.0, 4.0], [7.0, 1.0, 5.0]], [[1.0, 1.0, 8.0], [7.0, 8.0, 4.0], [4.0, 7.0, 4.0]], [[4.0, 3.0, 
8.0], [5.0, 1.0, 2.0], [7.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 6.0], [5.0, 5.0, 6.0], [4.0, 8.0, 4.0]], 
[[1.0, 4.0, 2.0], [5.0, 5.0, 7.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 5.0], [4.0, 7.0, 4.0], [3.0, 5.0, 3.0]]], [[[5.0, 3.0, 3.0], 
[3.0, 6.0, 4.0], [2.0, 7.0, 8.0]], [[1.0, 7.0, 1.0], [5.0, 5.0, 4.0], [7.0, 7.0, 3.0]], [[4.0, 1.0, 1.0], [4.0, 1.0, 8.0], 
[4.0, 5.0, 2.0]]], [[[5.0, 3.0, 5.0], [5.0, 7.0, 6.0], [5.0, 4.0, 6.0]], [[2.0, 2.0, 2.0], [1.0, 8.0, 2.0], [2.0, 7.0, 6.0]], 
[[8.0, 5.0, 7.0], [2.0, 7.0, 8.0], [6.0, 2.0, 1.0]]], [[[7.0, 3.0, 3.0], [1.0, 2.0, 4.0], [6.0, 5.0, 4.0]], [[1.0, 1.0, 8.0], 
[7.0, 8.0, 4.0], [2.0, 7.0, 6.0]], [[4.0, 3.0, 8.0], [5.0, 1.0, 2.0], [7.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 4.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [7.0, 
3.0, 8.0]], [[8.0, 6.0, 8.0], [8.0, 7.0, 3.0], [2.0, 1.0, 7.0]]], [[[5.0, 7.0, 2.0], [8.0, 4.0, 7.0], [2.0, 2.0, 2.0]], [[2.0, 
8.0, 8.0], [7.0, 7.0, 5.0], [1.0, 4.0, 6.0]], [[4.0, 3.0, 4.0], [5.0, 3.0, 7.0], [7.0, 7.0, 6.0]]], [[[2.0, 3.0, 4.0], [3.0, 
3.0, 2.0], [5.0, 1.0, 2.0]], [[4.0, 1.0, 4.0], [2.0, 5.0, 7.0], [6.0, 5.0, 7.0]], [[6.0, 4.0, 2.0], [6.0, 4.0, 7.0], [3.0, 3.0, 
6.0]]], [[[1.0, 4.0, 4.0], [8.0, 4.0, 2.0], [2.0, 7.0, 1.0]], [[4.0, 6.0, 7.0], [7.0, 2.0, 5.0], [7.0, 5.0, 6.0]], [[5.0, 8.0, 
2.0], [1.0, 4.0, 6.0], [3.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 8.0], [2.0, 3.0, 7.0], [1.0, 4.0, 8.0]], 
[[5.0, 5.0, 1.0], [3.0, 7.0, 4.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 1.0, 2.0], [8.0, 3.0, 5.0]]], [[[8.0, 3.0, 8.0], 
[1.0, 4.0, 5.0], [3.0, 5.0, 6.0]], [[2.0, 4.0, 1.0], [8.0, 1.0, 5.0], [8.0, 3.0, 1.0]], [[1.0, 5.0, 8.0], [7.0, 1.0, 2.0], 
[1.0, 7.0, 5.0]]], [[[5.0, 3.0, 2.0], [8.0, 3.0, 3.0], [8.0, 6.0, 2.0]], [[2.0, 3.0, 6.0], [4.0, 5.0, 7.0], [3.0, 4.0, 7.0]], 
[[2.0, 4.0, 6.0], [4.0, 1.0, 3.0], [1.0, 8.0, 3.0]]], [[[3.0, 7.0, 4.0], [1.0, 5.0, 2.0], [6.0, 2.0, 2.0]], [[3.0, 6.0, 6.0], 
[1.0, 5.0, 7.0], [6.0, 2.0, 4.0]], [[7.0, 7.0, 5.0], [8.0, 2.0, 3.0], [8.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[7.0, 2.0, 8.0], [4.0, 7.0, 6.0]], [[8.0, 5.0, 4.0], [5.0, 
1.0, 4.0]]], [[[1.0, 1.0, 4.0], [7.0, 3.0, 4.0]], [[4.0, 7.0, 5.0], [4.0, 5.0, 4.0]], [[7.0, 7.0, 2.0], [4.0, 4.0, 
4.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 3.0, 4.0], [5.0, 4.0, 2.0]], [[7.0, 3.0, 8.0], [4.0, 8.0, 5.0]], [[6.0, 3.0, 5.0], 
[2.0, 1.0, 5.0]]], [[[2.0, 5.0, 8.0], [4.0, 1.0, 4.0]], [[1.0, 1.0, 1.0], [4.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [1.0, 6.0, 
6.0]]]])
      arg[2:,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 4.0]], [[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [7.0, 
3.0, 8.0]], [[8.0, 6.0, 8.0], [8.0, 7.0, 3.0], [2.0, 1.0, 7.0]]], [[[5.0, 7.0, 2.0], [8.0, 4.0, 7.0], [2.0, 2.0, 2.0]], [[2.0, 
8.0, 8.0], [7.0, 7.0, 5.0], [1.0, 4.0, 6.0]], [[4.0, 3.0, 4.0], [5.0, 3.0, 7.0], [7.0, 7.0, 6.0]]], [[[8.0, 3.0, 3.0], [7.0, 
6.0, 6.0], [5.0, 1.0, 2.0]], [[7.0, 2.0, 8.0], [4.0, 7.0, 6.0], [6.0, 5.0, 7.0]], [[8.0, 5.0, 4.0], [5.0, 1.0, 4.0], [3.0, 3.0, 
6.0]]], [[[1.0, 1.0, 4.0], [7.0, 3.0, 4.0], [2.0, 7.0, 1.0]], [[4.0, 7.0, 5.0], [4.0, 5.0, 4.0], [7.0, 5.0, 6.0]], [[7.0, 7.0, 
2.0], [4.0, 4.0, 4.0], [3.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 8.0], [2.0, 3.0, 7.0], [1.0, 4.0, 8.0]], 
[[5.0, 5.0, 1.0], [3.0, 7.0, 4.0], [6.0, 1.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 1.0, 2.0], [8.0, 3.0, 5.0]]], [[[8.0, 3.0, 8.0], 
[1.0, 4.0, 5.0], [3.0, 5.0, 6.0]], [[2.0, 4.0, 1.0], [8.0, 1.0, 5.0], [8.0, 3.0, 1.0]], [[1.0, 5.0, 8.0], [7.0, 1.0, 2.0], 
[1.0, 7.0, 5.0]]], [[[2.0, 3.0, 4.0], [5.0, 4.0, 2.0], [8.0, 6.0, 2.0]], [[7.0, 3.0, 8.0], [4.0, 8.0, 5.0], [3.0, 4.0, 7.0]], 
[[6.0, 3.0, 5.0], [2.0, 1.0, 5.0], [1.0, 8.0, 3.0]]], [[[2.0, 5.0, 8.0], [4.0, 1.0, 4.0], [6.0, 2.0, 2.0]], [[1.0, 1.0, 1.0], 
[4.0, 3.0, 4.0], [6.0, 2.0, 4.0]], [[7.0, 4.0, 6.0], [1.0, 6.0, 6.0], [8.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 1.0], [7.0, 4.0, 2.0], [8.0, 8.0, 7.0]], [[5.0, 2.0, 2.0], [5.0, 4.0, 1.0], [4.0, 
5.0, 8.0]], [[6.0, 6.0, 4.0], [1.0, 5.0, 7.0], [1.0, 2.0, 1.0]]], [[[8.0, 3.0, 4.0], [2.0, 7.0, 3.0], [8.0, 6.0, 1.0]], [[3.0, 
8.0, 5.0], [6.0, 4.0, 2.0], [3.0, 3.0, 3.0]], [[4.0, 8.0, 8.0], [8.0, 8.0, 7.0], [1.0, 7.0, 5.0]]], [[[2.0, 7.0, 1.0], [8.0, 
2.0, 5.0], [7.0, 5.0, 3.0]], [[2.0, 6.0, 5.0], [6.0, 3.0, 2.0], [3.0, 1.0, 1.0]], [[2.0, 8.0, 8.0], [3.0, 7.0, 5.0], [5.0, 7.0, 
4.0]]], [[[8.0, 3.0, 8.0], [2.0, 3.0, 7.0], [6.0, 1.0, 3.0]], [[7.0, 7.0, 8.0], [3.0, 7.0, 1.0], [4.0, 6.0, 1.0]], [[2.0, 5.0, 
1.0], [8.0, 4.0, 5.0], [7.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 3.0], [1.0, 4.0, 2.0], [6.0, 7.0, 4.0]], 
[[4.0, 5.0, 7.0], [5.0, 6.0, 6.0], [4.0, 4.0, 5.0]], [[3.0, 1.0, 7.0], [2.0, 4.0, 8.0], [1.0, 8.0, 6.0]]], [[[8.0, 2.0, 8.0], 
[8.0, 4.0, 3.0], [6.0, 8.0, 2.0]], [[4.0, 4.0, 4.0], [8.0, 2.0, 1.0], [2.0, 5.0, 6.0]], [[2.0, 8.0, 8.0], [7.0, 1.0, 7.0], 
[6.0, 2.0, 6.0]]], [[[5.0, 3.0, 5.0], [6.0, 7.0, 4.0], [5.0, 7.0, 6.0]], [[4.0, 1.0, 3.0], [2.0, 5.0, 2.0], [7.0, 4.0, 2.0]], 
[[6.0, 7.0, 8.0], [5.0, 6.0, 6.0], [2.0, 5.0, 5.0]]], [[[2.0, 3.0, 7.0], [8.0, 1.0, 5.0], [4.0, 4.0, 7.0]], [[1.0, 6.0, 6.0], 
[3.0, 1.0, 8.0], [1.0, 1.0, 4.0]], [[3.0, 5.0, 1.0], [8.0, 8.0, 6.0], [1.0, 7.0, 1.0]]]])
      val=numpy.array([[[[4.0], [8.0]]], [[[4.0], [5.0]]]])
      arg[2:,2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 1.0], [7.0, 4.0, 2.0], [8.0, 8.0, 7.0]], [[5.0, 2.0, 2.0], [5.0, 4.0, 1.0], [4.0, 
5.0, 8.0]], [[6.0, 6.0, 4.0], [1.0, 5.0, 7.0], [1.0, 2.0, 1.0]]], [[[8.0, 3.0, 4.0], [2.0, 7.0, 3.0], [8.0, 6.0, 1.0]], [[3.0, 
8.0, 5.0], [6.0, 4.0, 2.0], [3.0, 3.0, 3.0]], [[4.0, 8.0, 8.0], [8.0, 8.0, 7.0], [1.0, 7.0, 5.0]]], [[[2.0, 7.0, 1.0], [8.0, 
2.0, 5.0], [7.0, 5.0, 3.0]], [[2.0, 6.0, 5.0], [6.0, 3.0, 2.0], [3.0, 1.0, 1.0]], [[2.0, 8.0, 4.0], [3.0, 7.0, 8.0], [5.0, 7.0, 
4.0]]], [[[8.0, 3.0, 8.0], [2.0, 3.0, 7.0], [6.0, 1.0, 3.0]], [[7.0, 7.0, 8.0], [3.0, 7.0, 1.0], [4.0, 6.0, 1.0]], [[2.0, 5.0, 
4.0], [8.0, 4.0, 5.0], [7.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 3.0], [1.0, 4.0, 2.0], [6.0, 7.0, 4.0]], 
[[4.0, 5.0, 7.0], [5.0, 6.0, 6.0], [4.0, 4.0, 5.0]], [[3.0, 1.0, 7.0], [2.0, 4.0, 8.0], [1.0, 8.0, 6.0]]], [[[8.0, 2.0, 8.0], 
[8.0, 4.0, 3.0], [6.0, 8.0, 2.0]], [[4.0, 4.0, 4.0], [8.0, 2.0, 1.0], [2.0, 5.0, 6.0]], [[2.0, 8.0, 8.0], [7.0, 1.0, 7.0], 
[6.0, 2.0, 6.0]]], [[[5.0, 3.0, 5.0], [6.0, 7.0, 4.0], [5.0, 7.0, 6.0]], [[4.0, 1.0, 3.0], [2.0, 5.0, 2.0], [7.0, 4.0, 2.0]], 
[[6.0, 7.0, 4.0], [5.0, 6.0, 8.0], [2.0, 5.0, 5.0]]], [[[2.0, 3.0, 7.0], [8.0, 1.0, 5.0], [4.0, 4.0, 7.0]], [[1.0, 6.0, 6.0], 
[3.0, 1.0, 8.0], [1.0, 1.0, 4.0]], [[3.0, 5.0, 4.0], [8.0, 8.0, 5.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 2.0, 4.0], [7.0, 5.0, 1.0], [7.0, 6.0, 5.0]], [[1.0, 3.0, 3.0], [1.0, 4.0, 6.0], [3.0, 
4.0, 4.0]], [[2.0, 3.0, 7.0], [6.0, 7.0, 3.0], [7.0, 8.0, 4.0]]], [[[1.0, 2.0, 8.0], [1.0, 7.0, 1.0], [7.0, 2.0, 5.0]], [[2.0, 
8.0, 8.0], [1.0, 7.0, 1.0], [4.0, 8.0, 3.0]], [[1.0, 7.0, 6.0], [1.0, 2.0, 8.0], [6.0, 6.0, 7.0]]], [[[8.0, 5.0, 4.0], [3.0, 
3.0, 2.0], [3.0, 1.0, 4.0]], [[2.0, 8.0, 7.0], [1.0, 5.0, 5.0], [7.0, 2.0, 6.0]], [[3.0, 2.0, 8.0], [3.0, 1.0, 3.0], [6.0, 1.0, 
2.0]]], [[[5.0, 1.0, 5.0], [5.0, 1.0, 7.0], [1.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], [4.0, 4.0, 3.0], [6.0, 1.0, 3.0]], [[2.0, 2.0, 
7.0], [1.0, 4.0, 5.0], [3.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 7.0], [1.0, 7.0, 1.0], [1.0, 3.0, 1.0]], 
[[8.0, 1.0, 5.0], [5.0, 4.0, 7.0], [3.0, 4.0, 8.0]], [[3.0, 5.0, 1.0], [4.0, 4.0, 5.0], [3.0, 8.0, 6.0]]], [[[2.0, 5.0, 5.0], 
[6.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[8.0, 7.0, 7.0], [8.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[1.0, 1.0, 7.0], [6.0, 2.0, 7.0], 
[1.0, 5.0, 7.0]]], [[[5.0, 2.0, 7.0], [5.0, 7.0, 2.0], [7.0, 2.0, 4.0]], [[5.0, 1.0, 1.0], [6.0, 3.0, 4.0], [7.0, 2.0, 3.0]], 
[[1.0, 6.0, 1.0], [5.0, 3.0, 5.0], [3.0, 8.0, 2.0]]], [[[6.0, 8.0, 6.0], [6.0, 2.0, 8.0], [3.0, 1.0, 6.0]], [[8.0, 7.0, 8.0], 
[7.0, 3.0, 1.0], [5.0, 7.0, 7.0]], [[4.0, 5.0, 2.0], [7.0, 7.0, 2.0], [4.0, 5.0, 1.0]]]])
      val=Data(numpy.array([[[[2.0], [5.0]]], [[[3.0], [7.0]]]]),self.functionspace)
      arg[2:,2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 2.0, 4.0], [7.0, 5.0, 1.0], [7.0, 6.0, 5.0]], [[1.0, 3.0, 3.0], [1.0, 4.0, 6.0], [3.0, 
4.0, 4.0]], [[2.0, 3.0, 7.0], [6.0, 7.0, 3.0], [7.0, 8.0, 4.0]]], [[[1.0, 2.0, 8.0], [1.0, 7.0, 1.0], [7.0, 2.0, 5.0]], [[2.0, 
8.0, 8.0], [1.0, 7.0, 1.0], [4.0, 8.0, 3.0]], [[1.0, 7.0, 6.0], [1.0, 2.0, 8.0], [6.0, 6.0, 7.0]]], [[[8.0, 5.0, 4.0], [3.0, 
3.0, 2.0], [3.0, 1.0, 4.0]], [[2.0, 8.0, 7.0], [1.0, 5.0, 5.0], [7.0, 2.0, 6.0]], [[3.0, 2.0, 2.0], [3.0, 1.0, 5.0], [6.0, 1.0, 
2.0]]], [[[5.0, 1.0, 5.0], [5.0, 1.0, 7.0], [1.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], [4.0, 4.0, 3.0], [6.0, 1.0, 3.0]], [[2.0, 2.0, 
3.0], [1.0, 4.0, 7.0], [3.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 7.0], [1.0, 7.0, 1.0], [1.0, 3.0, 1.0]], 
[[8.0, 1.0, 5.0], [5.0, 4.0, 7.0], [3.0, 4.0, 8.0]], [[3.0, 5.0, 1.0], [4.0, 4.0, 5.0], [3.0, 8.0, 6.0]]], [[[2.0, 5.0, 5.0], 
[6.0, 8.0, 2.0], [2.0, 6.0, 2.0]], [[8.0, 7.0, 7.0], [8.0, 7.0, 2.0], [8.0, 8.0, 1.0]], [[1.0, 1.0, 7.0], [6.0, 2.0, 7.0], 
[1.0, 5.0, 7.0]]], [[[5.0, 2.0, 7.0], [5.0, 7.0, 2.0], [7.0, 2.0, 4.0]], [[5.0, 1.0, 1.0], [6.0, 3.0, 4.0], [7.0, 2.0, 3.0]], 
[[1.0, 6.0, 2.0], [5.0, 3.0, 5.0], [3.0, 8.0, 2.0]]], [[[6.0, 8.0, 6.0], [6.0, 2.0, 8.0], [3.0, 1.0, 6.0]], [[8.0, 7.0, 8.0], 
[7.0, 3.0, 1.0], [5.0, 7.0, 7.0]], [[4.0, 5.0, 3.0], [7.0, 7.0, 7.0], [4.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLUL(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 6.0], [8.0, 6.0, 4.0], [8.0, 3.0, 6.0]], [[1.0, 3.0, 6.0], [8.0, 4.0, 5.0], [7.0, 
3.0, 7.0]], [[1.0, 5.0, 5.0], [2.0, 1.0, 2.0], [1.0, 8.0, 7.0]]], [[[7.0, 6.0, 1.0], [5.0, 6.0, 5.0], [6.0, 4.0, 5.0]], [[4.0, 
3.0, 4.0], [1.0, 1.0, 2.0], [3.0, 5.0, 7.0]], [[8.0, 4.0, 4.0], [7.0, 4.0, 5.0], [2.0, 6.0, 2.0]]], [[[6.0, 2.0, 3.0], [5.0, 
2.0, 7.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[8.0, 1.0, 2.0], [1.0, 1.0, 8.0], [4.0, 4.0, 
8.0]]], [[[3.0, 1.0, 2.0], [1.0, 7.0, 7.0], [1.0, 1.0, 5.0]], [[1.0, 3.0, 4.0], [8.0, 8.0, 6.0], [6.0, 3.0, 8.0]], [[7.0, 8.0, 
4.0], [1.0, 3.0, 2.0], [4.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 6.0], [2.0, 3.0, 4.0], [8.0, 8.0, 3.0]], 
[[1.0, 8.0, 8.0], [4.0, 1.0, 7.0], [1.0, 8.0, 1.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 7.0], [2.0, 2.0, 7.0]]], [[[3.0, 4.0, 5.0], 
[1.0, 2.0, 1.0], [3.0, 6.0, 3.0]], [[1.0, 3.0, 5.0], [1.0, 5.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 6.0, 1.0], [8.0, 2.0, 8.0], 
[5.0, 2.0, 1.0]]], [[[1.0, 4.0, 4.0], [5.0, 7.0, 1.0], [4.0, 2.0, 1.0]], [[1.0, 2.0, 3.0], [4.0, 8.0, 1.0], [3.0, 6.0, 6.0]], 
[[8.0, 7.0, 6.0], [7.0, 3.0, 3.0], [3.0, 4.0, 8.0]]], [[[7.0, 6.0, 6.0], [7.0, 6.0, 8.0], [1.0, 7.0, 6.0]], [[6.0, 2.0, 5.0], 
[4.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 4.0, 5.0], [1.0, 7.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[8.0], [3.0]]], [[[6.0], [3.0]]]])+(1.-msk_val)*numpy.array([[[[1.0], [1.0]]], [[[5.0], 
[7.0]]]])
      arg[2:,2:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 6.0], [8.0, 6.0, 4.0], [8.0, 3.0, 6.0]], [[1.0, 3.0, 6.0], [8.0, 4.0, 5.0], [7.0, 
3.0, 7.0]], [[1.0, 5.0, 5.0], [2.0, 1.0, 2.0], [1.0, 8.0, 7.0]]], [[[7.0, 6.0, 1.0], [5.0, 6.0, 5.0], [6.0, 4.0, 5.0]], [[4.0, 
3.0, 4.0], [1.0, 1.0, 2.0], [3.0, 5.0, 7.0]], [[8.0, 4.0, 4.0], [7.0, 4.0, 5.0], [2.0, 6.0, 2.0]]], [[[6.0, 2.0, 3.0], [5.0, 
2.0, 7.0], [8.0, 8.0, 2.0]], [[7.0, 5.0, 8.0], [2.0, 1.0, 1.0], [3.0, 8.0, 6.0]], [[8.0, 1.0, 8.0], [1.0, 1.0, 3.0], [4.0, 4.0, 
8.0]]], [[[3.0, 1.0, 2.0], [1.0, 7.0, 7.0], [1.0, 1.0, 5.0]], [[1.0, 3.0, 4.0], [8.0, 8.0, 6.0], [6.0, 3.0, 8.0]], [[7.0, 8.0, 
6.0], [1.0, 3.0, 3.0], [4.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 6.0], [2.0, 3.0, 4.0], [8.0, 8.0, 3.0]], 
[[1.0, 8.0, 8.0], [4.0, 1.0, 7.0], [1.0, 8.0, 1.0]], [[1.0, 5.0, 2.0], [7.0, 6.0, 7.0], [2.0, 2.0, 7.0]]], [[[3.0, 4.0, 5.0], 
[1.0, 2.0, 1.0], [3.0, 6.0, 3.0]], [[1.0, 3.0, 5.0], [1.0, 5.0, 5.0], [7.0, 8.0, 3.0]], [[1.0, 6.0, 1.0], [8.0, 2.0, 8.0], 
[5.0, 2.0, 1.0]]], [[[1.0, 4.0, 4.0], [5.0, 7.0, 1.0], [4.0, 2.0, 1.0]], [[1.0, 2.0, 3.0], [4.0, 8.0, 1.0], [3.0, 6.0, 6.0]], 
[[8.0, 7.0, 1.0], [7.0, 3.0, 1.0], [3.0, 4.0, 8.0]]], [[[7.0, 6.0, 6.0], [7.0, 6.0, 8.0], [1.0, 7.0, 6.0]], [[6.0, 2.0, 5.0], 
[4.0, 2.0, 6.0], [8.0, 2.0, 1.0]], [[2.0, 7.0, 5.0], [2.0, 4.0, 7.0], [1.0, 7.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 1.0, 6.0], [7.0, 2.0, 1.0], [3.0, 1.0, 1.0]], [[6.0, 8.0, 7.0], [8.0, 3.0, 2.0], [6.0, 
8.0, 2.0]], [[3.0, 8.0, 6.0], [7.0, 5.0, 5.0], [1.0, 1.0, 8.0]]], [[[1.0, 6.0, 5.0], [4.0, 3.0, 1.0], [4.0, 3.0, 4.0]], [[3.0, 
7.0, 6.0], [7.0, 7.0, 5.0], [7.0, 5.0, 6.0]], [[4.0, 4.0, 3.0], [7.0, 4.0, 8.0], [1.0, 1.0, 4.0]]], [[[5.0, 7.0, 5.0], [1.0, 
6.0, 4.0], [8.0, 6.0, 6.0]], [[1.0, 1.0, 5.0], [8.0, 7.0, 7.0], [4.0, 8.0, 7.0]], [[3.0, 4.0, 8.0], [8.0, 3.0, 3.0], [8.0, 2.0, 
8.0]]], [[[2.0, 6.0, 7.0], [2.0, 4.0, 6.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 4.0], [4.0, 8.0, 1.0], [4.0, 2.0, 3.0]], [[6.0, 4.0, 
3.0], [7.0, 7.0, 1.0], [1.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 7.0], [6.0, 1.0, 8.0], [4.0, 3.0, 2.0]], 
[[8.0, 1.0, 5.0], [6.0, 5.0, 5.0], [8.0, 8.0, 5.0]], [[6.0, 2.0, 3.0], [6.0, 3.0, 1.0], [8.0, 8.0, 8.0]]], [[[1.0, 3.0, 8.0], 
[6.0, 4.0, 6.0], [8.0, 5.0, 2.0]], [[2.0, 1.0, 1.0], [6.0, 5.0, 4.0], [1.0, 4.0, 7.0]], [[8.0, 5.0, 7.0], [3.0, 7.0, 3.0], 
[7.0, 2.0, 3.0]]], [[[1.0, 5.0, 7.0], [5.0, 8.0, 2.0], [2.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [1.0, 4.0, 3.0], [2.0, 1.0, 1.0]], 
[[6.0, 8.0, 7.0], [1.0, 1.0, 7.0], [6.0, 6.0, 2.0]]], [[[8.0, 6.0, 2.0], [3.0, 6.0, 3.0], [3.0, 5.0, 1.0]], [[4.0, 3.0, 3.0], 
[5.0, 5.0, 6.0], [7.0, 3.0, 7.0]], [[5.0, 3.0, 4.0], [8.0, 6.0, 3.0], [1.0, 5.0, 5.0]]]])
      val=numpy.array([[3.0, 4.0, 7.0], [7.0, 2.0, 7.0]])
      arg[2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 1.0, 6.0], [7.0, 2.0, 1.0], [3.0, 1.0, 1.0]], [[6.0, 8.0, 7.0], [8.0, 3.0, 2.0], [6.0, 
8.0, 2.0]], [[3.0, 8.0, 6.0], [7.0, 5.0, 5.0], [1.0, 1.0, 8.0]]], [[[1.0, 6.0, 5.0], [4.0, 3.0, 1.0], [4.0, 3.0, 4.0]], [[3.0, 
7.0, 6.0], [7.0, 7.0, 5.0], [7.0, 5.0, 6.0]], [[4.0, 4.0, 3.0], [7.0, 4.0, 8.0], [1.0, 1.0, 4.0]]], [[[5.0, 7.0, 5.0], [3.0, 
4.0, 7.0], [8.0, 6.0, 6.0]], [[1.0, 1.0, 5.0], [7.0, 2.0, 7.0], [4.0, 8.0, 7.0]], [[3.0, 4.0, 8.0], [8.0, 3.0, 3.0], [8.0, 2.0, 
8.0]]], [[[2.0, 6.0, 7.0], [2.0, 4.0, 6.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 4.0], [4.0, 8.0, 1.0], [4.0, 2.0, 3.0]], [[6.0, 4.0, 
3.0], [7.0, 7.0, 1.0], [1.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 7.0], [6.0, 1.0, 8.0], [4.0, 3.0, 2.0]], 
[[8.0, 1.0, 5.0], [6.0, 5.0, 5.0], [8.0, 8.0, 5.0]], [[6.0, 2.0, 3.0], [6.0, 3.0, 1.0], [8.0, 8.0, 8.0]]], [[[1.0, 3.0, 8.0], 
[6.0, 4.0, 6.0], [8.0, 5.0, 2.0]], [[2.0, 1.0, 1.0], [6.0, 5.0, 4.0], [1.0, 4.0, 7.0]], [[8.0, 5.0, 7.0], [3.0, 7.0, 3.0], 
[7.0, 2.0, 3.0]]], [[[1.0, 5.0, 7.0], [3.0, 4.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 2.0, 6.0], [7.0, 2.0, 7.0], [2.0, 1.0, 1.0]], 
[[6.0, 8.0, 7.0], [1.0, 1.0, 7.0], [6.0, 6.0, 2.0]]], [[[8.0, 6.0, 2.0], [3.0, 6.0, 3.0], [3.0, 5.0, 1.0]], [[4.0, 3.0, 3.0], 
[5.0, 5.0, 6.0], [7.0, 3.0, 7.0]], [[5.0, 3.0, 4.0], [8.0, 6.0, 3.0], [1.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 2.0], [7.0, 6.0, 7.0], [1.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [6.0, 2.0, 7.0], [8.0, 
7.0, 1.0]], [[4.0, 4.0, 5.0], [3.0, 2.0, 7.0], [6.0, 2.0, 8.0]]], [[[3.0, 8.0, 5.0], [7.0, 1.0, 7.0], [8.0, 6.0, 3.0]], [[4.0, 
4.0, 3.0], [6.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[4.0, 2.0, 5.0], [6.0, 3.0, 4.0], [6.0, 8.0, 7.0]]], [[[6.0, 3.0, 6.0], [6.0, 
1.0, 7.0], [4.0, 7.0, 4.0]], [[6.0, 4.0, 5.0], [3.0, 4.0, 8.0], [2.0, 2.0, 6.0]], [[6.0, 1.0, 3.0], [5.0, 2.0, 8.0], [3.0, 7.0, 
6.0]]], [[[6.0, 2.0, 4.0], [3.0, 4.0, 2.0], [1.0, 2.0, 4.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 3.0], [2.0, 8.0, 8.0]], [[8.0, 5.0, 
7.0], [3.0, 3.0, 7.0], [1.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 2.0], [3.0, 6.0, 5.0], [5.0, 8.0, 6.0]], 
[[2.0, 8.0, 3.0], [2.0, 4.0, 7.0], [1.0, 4.0, 7.0]], [[8.0, 7.0, 3.0], [8.0, 1.0, 1.0], [7.0, 2.0, 6.0]]], [[[3.0, 2.0, 2.0], 
[8.0, 3.0, 3.0], [7.0, 3.0, 5.0]], [[5.0, 6.0, 8.0], [8.0, 4.0, 8.0], [2.0, 5.0, 8.0]], [[5.0, 1.0, 5.0], [7.0, 5.0, 3.0], 
[6.0, 3.0, 4.0]]], [[[7.0, 3.0, 4.0], [4.0, 1.0, 3.0], [7.0, 8.0, 3.0]], [[2.0, 4.0, 4.0], [7.0, 3.0, 8.0], [2.0, 1.0, 6.0]], 
[[5.0, 1.0, 1.0], [3.0, 6.0, 8.0], [7.0, 8.0, 3.0]]], [[[4.0, 3.0, 1.0], [6.0, 4.0, 1.0], [2.0, 2.0, 4.0]], [[3.0, 5.0, 3.0], 
[4.0, 4.0, 3.0], [3.0, 3.0, 3.0]], [[6.0, 2.0, 7.0], [7.0, 1.0, 3.0], [1.0, 7.0, 5.0]]]])
      val=Data(numpy.array([[2.0, 3.0, 2.0], [6.0, 2.0, 5.0]]),self.functionspace)
      arg[2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 2.0], [7.0, 6.0, 7.0], [1.0, 6.0, 6.0]], [[7.0, 1.0, 1.0], [6.0, 2.0, 7.0], [8.0, 
7.0, 1.0]], [[4.0, 4.0, 5.0], [3.0, 2.0, 7.0], [6.0, 2.0, 8.0]]], [[[3.0, 8.0, 5.0], [7.0, 1.0, 7.0], [8.0, 6.0, 3.0]], [[4.0, 
4.0, 3.0], [6.0, 6.0, 3.0], [1.0, 1.0, 5.0]], [[4.0, 2.0, 5.0], [6.0, 3.0, 4.0], [6.0, 8.0, 7.0]]], [[[6.0, 3.0, 6.0], [2.0, 
3.0, 2.0], [4.0, 7.0, 4.0]], [[6.0, 4.0, 5.0], [6.0, 2.0, 5.0], [2.0, 2.0, 6.0]], [[6.0, 1.0, 3.0], [5.0, 2.0, 8.0], [3.0, 7.0, 
6.0]]], [[[6.0, 2.0, 4.0], [3.0, 4.0, 2.0], [1.0, 2.0, 4.0]], [[5.0, 5.0, 2.0], [6.0, 3.0, 3.0], [2.0, 8.0, 8.0]], [[8.0, 5.0, 
7.0], [3.0, 3.0, 7.0], [1.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 2.0], [3.0, 6.0, 5.0], [5.0, 8.0, 6.0]], 
[[2.0, 8.0, 3.0], [2.0, 4.0, 7.0], [1.0, 4.0, 7.0]], [[8.0, 7.0, 3.0], [8.0, 1.0, 1.0], [7.0, 2.0, 6.0]]], [[[3.0, 2.0, 2.0], 
[8.0, 3.0, 3.0], [7.0, 3.0, 5.0]], [[5.0, 6.0, 8.0], [8.0, 4.0, 8.0], [2.0, 5.0, 8.0]], [[5.0, 1.0, 5.0], [7.0, 5.0, 3.0], 
[6.0, 3.0, 4.0]]], [[[7.0, 3.0, 4.0], [2.0, 3.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 4.0, 4.0], [6.0, 2.0, 5.0], [2.0, 1.0, 6.0]], 
[[5.0, 1.0, 1.0], [3.0, 6.0, 8.0], [7.0, 8.0, 3.0]]], [[[4.0, 3.0, 1.0], [6.0, 4.0, 1.0], [2.0, 2.0, 4.0]], [[3.0, 5.0, 3.0], 
[4.0, 4.0, 3.0], [3.0, 3.0, 3.0]], [[6.0, 2.0, 7.0], [7.0, 1.0, 3.0], [1.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 8.0, 3.0], [2.0, 3.0, 4.0], [2.0, 8.0, 6.0]], [[7.0, 4.0, 1.0], [3.0, 7.0, 3.0], [6.0, 
1.0, 2.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 5.0], [4.0, 6.0, 2.0]]], [[[8.0, 5.0, 2.0], [4.0, 6.0, 6.0], [3.0, 4.0, 3.0]], [[5.0, 
6.0, 8.0], [2.0, 2.0, 1.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 2.0], [6.0, 8.0, 5.0], [4.0, 7.0, 6.0]]], [[[1.0, 4.0, 2.0], [1.0, 
2.0, 8.0], [5.0, 8.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 4.0, 3.0], [5.0, 7.0, 5.0]], [[3.0, 7.0, 1.0], [4.0, 3.0, 7.0], [1.0, 3.0, 
7.0]]], [[[5.0, 8.0, 8.0], [5.0, 7.0, 5.0], [6.0, 6.0, 2.0]], [[4.0, 5.0, 6.0], [4.0, 4.0, 7.0], [5.0, 8.0, 5.0]], [[3.0, 3.0, 
3.0], [3.0, 1.0, 6.0], [3.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 1.0, 4.0], [2.0, 3.0, 4.0]], 
[[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [1.0, 7.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 3.0, 8.0], [3.0, 1.0, 2.0]]], [[[3.0, 8.0, 5.0], 
[1.0, 8.0, 1.0], [7.0, 7.0, 4.0]], [[6.0, 1.0, 1.0], [7.0, 6.0, 5.0], [1.0, 6.0, 5.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 2.0], 
[1.0, 7.0, 5.0]]], [[[1.0, 5.0, 7.0], [8.0, 3.0, 1.0], [4.0, 2.0, 2.0]], [[4.0, 7.0, 2.0], [7.0, 4.0, 8.0], [1.0, 4.0, 2.0]], 
[[4.0, 8.0, 1.0], [1.0, 4.0, 8.0], [7.0, 8.0, 1.0]]], [[[1.0, 5.0, 1.0], [3.0, 3.0, 3.0], [6.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], 
[3.0, 6.0, 3.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 6.0], [6.0, 5.0, 3.0], [2.0, 6.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[2.0, 7.0, 1.0], [1.0, 1.0, 2.0]])+(1.-msk_val)*numpy.array([[7.0, 3.0, 2.0], [3.0, 5.0, 8.0]])
      arg[2,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 8.0, 3.0], [2.0, 3.0, 4.0], [2.0, 8.0, 6.0]], [[7.0, 4.0, 1.0], [3.0, 7.0, 3.0], [6.0, 
1.0, 2.0]], [[4.0, 1.0, 1.0], [3.0, 6.0, 5.0], [4.0, 6.0, 2.0]]], [[[8.0, 5.0, 2.0], [4.0, 6.0, 6.0], [3.0, 4.0, 3.0]], [[5.0, 
6.0, 8.0], [2.0, 2.0, 1.0], [4.0, 8.0, 3.0]], [[4.0, 3.0, 2.0], [6.0, 8.0, 5.0], [4.0, 7.0, 6.0]]], [[[1.0, 4.0, 2.0], [2.0, 
7.0, 1.0], [5.0, 8.0, 7.0]], [[7.0, 4.0, 5.0], [1.0, 1.0, 2.0], [5.0, 7.0, 5.0]], [[3.0, 7.0, 1.0], [4.0, 3.0, 7.0], [1.0, 3.0, 
7.0]]], [[[5.0, 8.0, 8.0], [5.0, 7.0, 5.0], [6.0, 6.0, 2.0]], [[4.0, 5.0, 6.0], [4.0, 4.0, 7.0], [5.0, 8.0, 5.0]], [[3.0, 3.0, 
3.0], [3.0, 1.0, 6.0], [3.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 3.0], [5.0, 1.0, 4.0], [2.0, 3.0, 4.0]], 
[[6.0, 3.0, 4.0], [6.0, 8.0, 6.0], [1.0, 7.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 3.0, 8.0], [3.0, 1.0, 2.0]]], [[[3.0, 8.0, 5.0], 
[1.0, 8.0, 1.0], [7.0, 7.0, 4.0]], [[6.0, 1.0, 1.0], [7.0, 6.0, 5.0], [1.0, 6.0, 5.0]], [[6.0, 3.0, 1.0], [3.0, 5.0, 2.0], 
[1.0, 7.0, 5.0]]], [[[1.0, 5.0, 7.0], [7.0, 3.0, 2.0], [4.0, 2.0, 2.0]], [[4.0, 7.0, 2.0], [3.0, 5.0, 8.0], [1.0, 4.0, 2.0]], 
[[4.0, 8.0, 1.0], [1.0, 4.0, 8.0], [7.0, 8.0, 1.0]]], [[[1.0, 5.0, 1.0], [3.0, 3.0, 3.0], [6.0, 1.0, 2.0]], [[4.0, 1.0, 2.0], 
[3.0, 6.0, 3.0], [2.0, 1.0, 2.0]], [[6.0, 4.0, 6.0], [6.0, 5.0, 3.0], [2.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 8.0], [2.0, 5.0, 3.0], [2.0, 1.0, 7.0]], [[1.0, 2.0, 7.0], [8.0, 5.0, 1.0], [1.0, 
2.0, 3.0]], [[8.0, 7.0, 8.0], [6.0, 8.0, 5.0], [3.0, 5.0, 7.0]]], [[[1.0, 3.0, 3.0], [4.0, 2.0, 3.0], [1.0, 3.0, 7.0]], [[5.0, 
7.0, 6.0], [3.0, 8.0, 2.0], [1.0, 5.0, 8.0]], [[3.0, 5.0, 5.0], [6.0, 1.0, 7.0], [5.0, 5.0, 3.0]]], [[[4.0, 2.0, 3.0], [5.0, 
8.0, 1.0], [2.0, 5.0, 8.0]], [[3.0, 7.0, 8.0], [1.0, 1.0, 1.0], [2.0, 8.0, 5.0]], [[5.0, 8.0, 8.0], [1.0, 7.0, 8.0], [3.0, 2.0, 
8.0]]], [[[8.0, 3.0, 5.0], [2.0, 4.0, 8.0], [3.0, 3.0, 2.0]], [[1.0, 3.0, 1.0], [1.0, 7.0, 4.0], [2.0, 3.0, 1.0]], [[5.0, 5.0, 
5.0], [7.0, 8.0, 8.0], [2.0, 1.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 2.0], [3.0, 1.0, 5.0], [5.0, 8.0, 5.0]], 
[[1.0, 7.0, 3.0], [1.0, 1.0, 8.0], [5.0, 5.0, 7.0]], [[7.0, 8.0, 6.0], [5.0, 1.0, 3.0], [4.0, 6.0, 4.0]]], [[[4.0, 3.0, 4.0], 
[1.0, 7.0, 5.0], [2.0, 2.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 1.0, 5.0], [3.0, 7.0, 8.0]], [[6.0, 1.0, 7.0], [2.0, 2.0, 7.0], 
[8.0, 8.0, 1.0]]], [[[1.0, 3.0, 6.0], [2.0, 5.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 8.0, 5.0], [4.0, 1.0, 7.0], [3.0, 3.0, 2.0]], 
[[7.0, 7.0, 7.0], [3.0, 3.0, 6.0], [2.0, 7.0, 3.0]]], [[[3.0, 8.0, 4.0], [5.0, 3.0, 4.0], [4.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], 
[3.0, 4.0, 6.0], [7.0, 4.0, 8.0]], [[3.0, 6.0, 1.0], [4.0, 1.0, 6.0], [5.0, 2.0, 6.0]]]])
      val=numpy.array([[[6.0, 8.0, 1.0]]])
      arg[2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 8.0], [2.0, 5.0, 3.0], [2.0, 1.0, 7.0]], [[1.0, 2.0, 7.0], [8.0, 5.0, 1.0], [1.0, 
2.0, 3.0]], [[8.0, 7.0, 8.0], [6.0, 8.0, 5.0], [3.0, 5.0, 7.0]]], [[[1.0, 3.0, 3.0], [4.0, 2.0, 3.0], [1.0, 3.0, 7.0]], [[5.0, 
7.0, 6.0], [3.0, 8.0, 2.0], [1.0, 5.0, 8.0]], [[3.0, 5.0, 5.0], [6.0, 1.0, 7.0], [5.0, 5.0, 3.0]]], [[[4.0, 2.0, 3.0], [5.0, 
8.0, 1.0], [2.0, 5.0, 8.0]], [[3.0, 7.0, 8.0], [1.0, 1.0, 1.0], [6.0, 8.0, 1.0]], [[5.0, 8.0, 8.0], [1.0, 7.0, 8.0], [3.0, 2.0, 
8.0]]], [[[8.0, 3.0, 5.0], [2.0, 4.0, 8.0], [3.0, 3.0, 2.0]], [[1.0, 3.0, 1.0], [1.0, 7.0, 4.0], [2.0, 3.0, 1.0]], [[5.0, 5.0, 
5.0], [7.0, 8.0, 8.0], [2.0, 1.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 2.0], [3.0, 1.0, 5.0], [5.0, 8.0, 5.0]], 
[[1.0, 7.0, 3.0], [1.0, 1.0, 8.0], [5.0, 5.0, 7.0]], [[7.0, 8.0, 6.0], [5.0, 1.0, 3.0], [4.0, 6.0, 4.0]]], [[[4.0, 3.0, 4.0], 
[1.0, 7.0, 5.0], [2.0, 2.0, 6.0]], [[4.0, 1.0, 4.0], [1.0, 1.0, 5.0], [3.0, 7.0, 8.0]], [[6.0, 1.0, 7.0], [2.0, 2.0, 7.0], 
[8.0, 8.0, 1.0]]], [[[1.0, 3.0, 6.0], [2.0, 5.0, 2.0], [3.0, 6.0, 7.0]], [[7.0, 8.0, 5.0], [4.0, 1.0, 7.0], [6.0, 8.0, 1.0]], 
[[7.0, 7.0, 7.0], [3.0, 3.0, 6.0], [2.0, 7.0, 3.0]]], [[[3.0, 8.0, 4.0], [5.0, 3.0, 4.0], [4.0, 4.0, 3.0]], [[4.0, 3.0, 1.0], 
[3.0, 4.0, 6.0], [7.0, 4.0, 8.0]], [[3.0, 6.0, 1.0], [4.0, 1.0, 6.0], [5.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 4.0]], [[6.0, 1.0, 4.0], [8.0, 6.0, 2.0], [2.0, 
7.0, 6.0]], [[5.0, 8.0, 5.0], [1.0, 7.0, 3.0], [6.0, 6.0, 8.0]]], [[[7.0, 8.0, 7.0], [7.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[2.0, 
4.0, 1.0], [1.0, 4.0, 1.0], [6.0, 1.0, 3.0]], [[8.0, 3.0, 2.0], [2.0, 5.0, 8.0], [5.0, 4.0, 2.0]]], [[[8.0, 2.0, 8.0], [5.0, 
8.0, 8.0], [4.0, 5.0, 8.0]], [[8.0, 4.0, 7.0], [2.0, 1.0, 5.0], [6.0, 5.0, 4.0]], [[1.0, 1.0, 7.0], [4.0, 1.0, 6.0], [6.0, 5.0, 
2.0]]], [[[8.0, 6.0, 6.0], [4.0, 3.0, 1.0], [2.0, 6.0, 2.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 3.0], [4.0, 5.0, 1.0]], [[6.0, 3.0, 
8.0], [6.0, 5.0, 1.0], [8.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 8.0], [7.0, 3.0, 4.0], [6.0, 4.0, 6.0]], 
[[2.0, 5.0, 5.0], [3.0, 1.0, 3.0], [8.0, 8.0, 8.0]], [[2.0, 1.0, 4.0], [1.0, 5.0, 3.0], [1.0, 1.0, 5.0]]], [[[2.0, 7.0, 2.0], 
[2.0, 4.0, 8.0], [7.0, 3.0, 7.0]], [[1.0, 6.0, 2.0], [3.0, 1.0, 1.0], [2.0, 8.0, 6.0]], [[2.0, 4.0, 6.0], [3.0, 7.0, 2.0], 
[2.0, 2.0, 5.0]]], [[[4.0, 8.0, 8.0], [7.0, 4.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 2.0, 3.0], [2.0, 4.0, 2.0]], 
[[7.0, 1.0, 8.0], [1.0, 1.0, 8.0], [5.0, 7.0, 8.0]]], [[[4.0, 8.0, 8.0], [5.0, 1.0, 1.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], 
[6.0, 7.0, 5.0], [1.0, 7.0, 8.0]], [[2.0, 4.0, 3.0], [3.0, 2.0, 8.0], [7.0, 3.0, 1.0]]]])
      val=Data(numpy.array([[[5.0, 2.0, 6.0]]]),self.functionspace)
      arg[2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 2.0], [7.0, 2.0, 3.0], [4.0, 4.0, 4.0]], [[6.0, 1.0, 4.0], [8.0, 6.0, 2.0], [2.0, 
7.0, 6.0]], [[5.0, 8.0, 5.0], [1.0, 7.0, 3.0], [6.0, 6.0, 8.0]]], [[[7.0, 8.0, 7.0], [7.0, 3.0, 2.0], [1.0, 7.0, 2.0]], [[2.0, 
4.0, 1.0], [1.0, 4.0, 1.0], [6.0, 1.0, 3.0]], [[8.0, 3.0, 2.0], [2.0, 5.0, 8.0], [5.0, 4.0, 2.0]]], [[[8.0, 2.0, 8.0], [5.0, 
8.0, 8.0], [4.0, 5.0, 8.0]], [[8.0, 4.0, 7.0], [2.0, 1.0, 5.0], [5.0, 2.0, 6.0]], [[1.0, 1.0, 7.0], [4.0, 1.0, 6.0], [6.0, 5.0, 
2.0]]], [[[8.0, 6.0, 6.0], [4.0, 3.0, 1.0], [2.0, 6.0, 2.0]], [[7.0, 1.0, 8.0], [8.0, 3.0, 3.0], [4.0, 5.0, 1.0]], [[6.0, 3.0, 
8.0], [6.0, 5.0, 1.0], [8.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 8.0], [7.0, 3.0, 4.0], [6.0, 4.0, 6.0]], 
[[2.0, 5.0, 5.0], [3.0, 1.0, 3.0], [8.0, 8.0, 8.0]], [[2.0, 1.0, 4.0], [1.0, 5.0, 3.0], [1.0, 1.0, 5.0]]], [[[2.0, 7.0, 2.0], 
[2.0, 4.0, 8.0], [7.0, 3.0, 7.0]], [[1.0, 6.0, 2.0], [3.0, 1.0, 1.0], [2.0, 8.0, 6.0]], [[2.0, 4.0, 6.0], [3.0, 7.0, 2.0], 
[2.0, 2.0, 5.0]]], [[[4.0, 8.0, 8.0], [7.0, 4.0, 6.0], [6.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], [1.0, 2.0, 3.0], [5.0, 2.0, 6.0]], 
[[7.0, 1.0, 8.0], [1.0, 1.0, 8.0], [5.0, 7.0, 8.0]]], [[[4.0, 8.0, 8.0], [5.0, 1.0, 1.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], 
[6.0, 7.0, 5.0], [1.0, 7.0, 8.0]], [[2.0, 4.0, 3.0], [3.0, 2.0, 8.0], [7.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 7.0], [3.0, 1.0, 7.0], [3.0, 2.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 8.0, 7.0], [4.0, 
4.0, 2.0]], [[4.0, 3.0, 1.0], [7.0, 3.0, 2.0], [7.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [6.0, 1.0, 4.0], [5.0, 3.0, 6.0]], [[5.0, 
4.0, 4.0], [2.0, 5.0, 5.0], [2.0, 3.0, 3.0]], [[7.0, 2.0, 8.0], [7.0, 3.0, 6.0], [8.0, 7.0, 4.0]]], [[[5.0, 7.0, 6.0], [3.0, 
1.0, 8.0], [2.0, 8.0, 8.0]], [[4.0, 7.0, 6.0], [7.0, 7.0, 1.0], [1.0, 1.0, 3.0]], [[7.0, 8.0, 6.0], [6.0, 1.0, 7.0], [7.0, 1.0, 
6.0]]], [[[4.0, 8.0, 7.0], [4.0, 5.0, 7.0], [2.0, 6.0, 8.0]], [[7.0, 2.0, 3.0], [1.0, 8.0, 4.0], [1.0, 5.0, 5.0]], [[6.0, 5.0, 
8.0], [4.0, 3.0, 2.0], [1.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 2.0], [1.0, 8.0, 3.0], [8.0, 6.0, 6.0]], 
[[6.0, 2.0, 4.0], [8.0, 4.0, 4.0], [1.0, 8.0, 3.0]], [[4.0, 7.0, 5.0], [7.0, 1.0, 1.0], [5.0, 5.0, 6.0]]], [[[7.0, 5.0, 8.0], 
[4.0, 8.0, 6.0], [7.0, 7.0, 5.0]], [[8.0, 6.0, 3.0], [3.0, 1.0, 5.0], [4.0, 4.0, 4.0]], [[6.0, 7.0, 8.0], [3.0, 4.0, 7.0], 
[4.0, 7.0, 2.0]]], [[[3.0, 4.0, 2.0], [1.0, 8.0, 3.0], [2.0, 8.0, 4.0]], [[1.0, 5.0, 2.0], [7.0, 5.0, 4.0], [8.0, 6.0, 4.0]], 
[[5.0, 2.0, 8.0], [3.0, 3.0, 2.0], [6.0, 7.0, 2.0]]], [[[8.0, 1.0, 3.0], [6.0, 6.0, 3.0], [5.0, 4.0, 1.0]], [[2.0, 8.0, 1.0], 
[7.0, 7.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 8.0], [7.0, 4.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 5.0, 3.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 3.0]]])
      arg[2,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 7.0], [3.0, 1.0, 7.0], [3.0, 2.0, 1.0]], [[7.0, 2.0, 1.0], [5.0, 8.0, 7.0], [4.0, 
4.0, 2.0]], [[4.0, 3.0, 1.0], [7.0, 3.0, 2.0], [7.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [6.0, 1.0, 4.0], [5.0, 3.0, 6.0]], [[5.0, 
4.0, 4.0], [2.0, 5.0, 5.0], [2.0, 3.0, 3.0]], [[7.0, 2.0, 8.0], [7.0, 3.0, 6.0], [8.0, 7.0, 4.0]]], [[[5.0, 7.0, 6.0], [3.0, 
1.0, 8.0], [2.0, 8.0, 8.0]], [[4.0, 7.0, 6.0], [7.0, 7.0, 1.0], [3.0, 5.0, 3.0]], [[7.0, 8.0, 6.0], [6.0, 1.0, 7.0], [7.0, 1.0, 
6.0]]], [[[4.0, 8.0, 7.0], [4.0, 5.0, 7.0], [2.0, 6.0, 8.0]], [[7.0, 2.0, 3.0], [1.0, 8.0, 4.0], [1.0, 5.0, 5.0]], [[6.0, 5.0, 
8.0], [4.0, 3.0, 2.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 2.0], [1.0, 8.0, 3.0], [8.0, 6.0, 6.0]], 
[[6.0, 2.0, 4.0], [8.0, 4.0, 4.0], [1.0, 8.0, 3.0]], [[4.0, 7.0, 5.0], [7.0, 1.0, 1.0], [5.0, 5.0, 6.0]]], [[[7.0, 5.0, 8.0], 
[4.0, 8.0, 6.0], [7.0, 7.0, 5.0]], [[8.0, 6.0, 3.0], [3.0, 1.0, 5.0], [4.0, 4.0, 4.0]], [[6.0, 7.0, 8.0], [3.0, 4.0, 7.0], 
[4.0, 7.0, 2.0]]], [[[3.0, 4.0, 2.0], [1.0, 8.0, 3.0], [2.0, 8.0, 4.0]], [[1.0, 5.0, 2.0], [7.0, 5.0, 4.0], [4.0, 4.0, 3.0]], 
[[5.0, 2.0, 8.0], [3.0, 3.0, 2.0], [6.0, 7.0, 2.0]]], [[[8.0, 1.0, 3.0], [6.0, 6.0, 3.0], [5.0, 4.0, 1.0]], [[2.0, 8.0, 1.0], 
[7.0, 7.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 7.0, 3.0], [3.0, 1.0, 8.0], [7.0, 4.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 3.0], [6.0, 2.0, 5.0], [8.0, 1.0, 1.0]], [[2.0, 4.0, 1.0], [2.0, 3.0, 8.0], [6.0, 
3.0, 6.0]], [[6.0, 6.0, 8.0], [5.0, 8.0, 3.0], [1.0, 5.0, 7.0]]], [[[1.0, 2.0, 4.0], [7.0, 5.0, 8.0], [3.0, 1.0, 8.0]], [[5.0, 
5.0, 3.0], [3.0, 8.0, 6.0], [3.0, 2.0, 1.0]], [[6.0, 5.0, 4.0], [5.0, 4.0, 7.0], [8.0, 8.0, 7.0]]], [[[5.0, 2.0, 2.0], [6.0, 
5.0, 3.0], [3.0, 3.0, 7.0]], [[3.0, 7.0, 5.0], [8.0, 1.0, 4.0], [6.0, 8.0, 5.0]], [[2.0, 3.0, 5.0], [3.0, 5.0, 3.0], [5.0, 7.0, 
8.0]]], [[[3.0, 3.0, 3.0], [2.0, 7.0, 4.0], [3.0, 6.0, 3.0]], [[7.0, 7.0, 8.0], [2.0, 2.0, 8.0], [4.0, 2.0, 3.0]], [[4.0, 7.0, 
4.0], [1.0, 7.0, 8.0], [5.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 5.0], [6.0, 7.0, 3.0], [7.0, 4.0, 4.0]], 
[[7.0, 8.0, 1.0], [2.0, 1.0, 5.0], [3.0, 3.0, 4.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 4.0], [5.0, 1.0, 5.0]]], [[[2.0, 5.0, 3.0], 
[2.0, 4.0, 6.0], [1.0, 5.0, 1.0]], [[7.0, 6.0, 4.0], [3.0, 8.0, 8.0], [2.0, 2.0, 5.0]], [[3.0, 8.0, 4.0], [5.0, 2.0, 7.0], 
[4.0, 4.0, 3.0]]], [[[7.0, 1.0, 1.0], [2.0, 5.0, 2.0], [4.0, 5.0, 5.0]], [[7.0, 8.0, 3.0], [5.0, 7.0, 6.0], [4.0, 1.0, 6.0]], 
[[8.0, 3.0, 6.0], [1.0, 4.0, 3.0], [8.0, 7.0, 5.0]]], [[[6.0, 8.0, 4.0], [5.0, 6.0, 3.0], [4.0, 2.0, 5.0]], [[5.0, 2.0, 3.0], 
[5.0, 8.0, 1.0], [1.0, 4.0, 7.0]], [[4.0, 7.0, 3.0], [8.0, 3.0, 1.0], [3.0, 7.0, 5.0]]]])
      val=numpy.array([[[[3.0, 2.0, 5.0]]], [[[6.0, 6.0, 3.0]]], [[[3.0, 1.0, 6.0]]], [[[3.0, 2.0, 3.0]]]])
      arg[:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 3.0], [6.0, 2.0, 5.0], [8.0, 1.0, 1.0]], [[2.0, 4.0, 1.0], [2.0, 3.0, 8.0], [3.0, 
2.0, 5.0]], [[6.0, 6.0, 8.0], [5.0, 8.0, 3.0], [1.0, 5.0, 7.0]]], [[[1.0, 2.0, 4.0], [7.0, 5.0, 8.0], [3.0, 1.0, 8.0]], [[5.0, 
5.0, 3.0], [3.0, 8.0, 6.0], [6.0, 6.0, 3.0]], [[6.0, 5.0, 4.0], [5.0, 4.0, 7.0], [8.0, 8.0, 7.0]]], [[[5.0, 2.0, 2.0], [6.0, 
5.0, 3.0], [3.0, 3.0, 7.0]], [[3.0, 7.0, 5.0], [8.0, 1.0, 4.0], [3.0, 1.0, 6.0]], [[2.0, 3.0, 5.0], [3.0, 5.0, 3.0], [5.0, 7.0, 
8.0]]], [[[3.0, 3.0, 3.0], [2.0, 7.0, 4.0], [3.0, 6.0, 3.0]], [[7.0, 7.0, 8.0], [2.0, 2.0, 8.0], [3.0, 2.0, 3.0]], [[4.0, 7.0, 
4.0], [1.0, 7.0, 8.0], [5.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 5.0], [6.0, 7.0, 3.0], [7.0, 4.0, 4.0]], 
[[7.0, 8.0, 1.0], [2.0, 1.0, 5.0], [3.0, 2.0, 5.0]], [[3.0, 8.0, 5.0], [7.0, 8.0, 4.0], [5.0, 1.0, 5.0]]], [[[2.0, 5.0, 3.0], 
[2.0, 4.0, 6.0], [1.0, 5.0, 1.0]], [[7.0, 6.0, 4.0], [3.0, 8.0, 8.0], [6.0, 6.0, 3.0]], [[3.0, 8.0, 4.0], [5.0, 2.0, 7.0], 
[4.0, 4.0, 3.0]]], [[[7.0, 1.0, 1.0], [2.0, 5.0, 2.0], [4.0, 5.0, 5.0]], [[7.0, 8.0, 3.0], [5.0, 7.0, 6.0], [3.0, 1.0, 6.0]], 
[[8.0, 3.0, 6.0], [1.0, 4.0, 3.0], [8.0, 7.0, 5.0]]], [[[6.0, 8.0, 4.0], [5.0, 6.0, 3.0], [4.0, 2.0, 5.0]], [[5.0, 2.0, 3.0], 
[5.0, 8.0, 1.0], [3.0, 2.0, 3.0]], [[4.0, 7.0, 3.0], [8.0, 3.0, 1.0], [3.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 4.0], [2.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[6.0, 6.0, 4.0], [3.0, 3.0, 1.0], [8.0, 
2.0, 8.0]], [[1.0, 6.0, 3.0], [1.0, 4.0, 6.0], [8.0, 7.0, 2.0]]], [[[1.0, 6.0, 8.0], [6.0, 8.0, 6.0], [4.0, 8.0, 4.0]], [[1.0, 
8.0, 6.0], [1.0, 8.0, 1.0], [8.0, 6.0, 3.0]], [[7.0, 3.0, 5.0], [5.0, 7.0, 1.0], [8.0, 6.0, 4.0]]], [[[3.0, 1.0, 3.0], [1.0, 
8.0, 6.0], [2.0, 4.0, 4.0]], [[4.0, 3.0, 7.0], [2.0, 7.0, 4.0], [1.0, 3.0, 3.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 3.0], [5.0, 6.0, 
8.0]]], [[[2.0, 3.0, 4.0], [1.0, 2.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 3.0, 2.0], [6.0, 4.0, 4.0], [3.0, 2.0, 7.0]], [[8.0, 6.0, 
5.0], [8.0, 8.0, 2.0], [2.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 4.0, 1.0], [6.0, 4.0, 6.0], [8.0, 8.0, 3.0]], 
[[3.0, 7.0, 7.0], [7.0, 7.0, 5.0], [8.0, 1.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 2.0, 5.0], [5.0, 7.0, 6.0]]], [[[8.0, 7.0, 4.0], 
[4.0, 4.0, 7.0], [5.0, 6.0, 7.0]], [[6.0, 6.0, 1.0], [6.0, 6.0, 6.0], [6.0, 1.0, 6.0]], [[5.0, 2.0, 4.0], [7.0, 2.0, 8.0], 
[4.0, 2.0, 1.0]]], [[[5.0, 2.0, 6.0], [4.0, 4.0, 5.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 4.0], [2.0, 2.0, 5.0], [2.0, 6.0, 3.0]], 
[[6.0, 5.0, 7.0], [8.0, 4.0, 6.0], [7.0, 8.0, 8.0]]], [[[8.0, 5.0, 8.0], [5.0, 7.0, 5.0], [3.0, 7.0, 7.0]], [[7.0, 5.0, 5.0], 
[3.0, 2.0, 8.0], [7.0, 6.0, 7.0]], [[8.0, 8.0, 7.0], [5.0, 5.0, 5.0], [6.0, 1.0, 4.0]]]])
      val=Data(numpy.array([[[[3.0, 3.0, 1.0]]], [[[5.0, 5.0, 1.0]]], [[[6.0, 4.0, 7.0]]], [[[5.0, 3.0, 
8.0]]]]),self.functionspace)
      arg[:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 4.0], [2.0, 2.0, 5.0], [4.0, 5.0, 3.0]], [[6.0, 6.0, 4.0], [3.0, 3.0, 1.0], [3.0, 
3.0, 1.0]], [[1.0, 6.0, 3.0], [1.0, 4.0, 6.0], [8.0, 7.0, 2.0]]], [[[1.0, 6.0, 8.0], [6.0, 8.0, 6.0], [4.0, 8.0, 4.0]], [[1.0, 
8.0, 6.0], [1.0, 8.0, 1.0], [5.0, 5.0, 1.0]], [[7.0, 3.0, 5.0], [5.0, 7.0, 1.0], [8.0, 6.0, 4.0]]], [[[3.0, 1.0, 3.0], [1.0, 
8.0, 6.0], [2.0, 4.0, 4.0]], [[4.0, 3.0, 7.0], [2.0, 7.0, 4.0], [6.0, 4.0, 7.0]], [[6.0, 8.0, 1.0], [7.0, 5.0, 3.0], [5.0, 6.0, 
8.0]]], [[[2.0, 3.0, 4.0], [1.0, 2.0, 7.0], [2.0, 7.0, 1.0]], [[7.0, 3.0, 2.0], [6.0, 4.0, 4.0], [5.0, 3.0, 8.0]], [[8.0, 6.0, 
5.0], [8.0, 8.0, 2.0], [2.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 4.0, 1.0], [6.0, 4.0, 6.0], [8.0, 8.0, 3.0]], 
[[3.0, 7.0, 7.0], [7.0, 7.0, 5.0], [3.0, 3.0, 1.0]], [[5.0, 7.0, 4.0], [3.0, 2.0, 5.0], [5.0, 7.0, 6.0]]], [[[8.0, 7.0, 4.0], 
[4.0, 4.0, 7.0], [5.0, 6.0, 7.0]], [[6.0, 6.0, 1.0], [6.0, 6.0, 6.0], [5.0, 5.0, 1.0]], [[5.0, 2.0, 4.0], [7.0, 2.0, 8.0], 
[4.0, 2.0, 1.0]]], [[[5.0, 2.0, 6.0], [4.0, 4.0, 5.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 4.0], [2.0, 2.0, 5.0], [6.0, 4.0, 7.0]], 
[[6.0, 5.0, 7.0], [8.0, 4.0, 6.0], [7.0, 8.0, 8.0]]], [[[8.0, 5.0, 8.0], [5.0, 7.0, 5.0], [3.0, 7.0, 7.0]], [[7.0, 5.0, 5.0], 
[3.0, 2.0, 8.0], [5.0, 3.0, 8.0]], [[8.0, 8.0, 7.0], [5.0, 5.0, 5.0], [6.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 8.0], [5.0, 3.0, 5.0], [4.0, 5.0, 4.0]], [[5.0, 8.0, 1.0], [6.0, 5.0, 6.0], [1.0, 
8.0, 1.0]], [[4.0, 2.0, 2.0], [4.0, 3.0, 4.0], [2.0, 4.0, 3.0]]], [[[4.0, 8.0, 6.0], [6.0, 1.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 
3.0, 7.0], [6.0, 6.0, 3.0], [8.0, 2.0, 3.0]], [[4.0, 6.0, 8.0], [4.0, 2.0, 3.0], [4.0, 8.0, 5.0]]], [[[2.0, 6.0, 5.0], [4.0, 
1.0, 6.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 2.0], [4.0, 7.0, 8.0], [4.0, 6.0, 2.0]], [[3.0, 3.0, 2.0], [4.0, 2.0, 6.0], [2.0, 3.0, 
2.0]]], [[[4.0, 1.0, 7.0], [2.0, 4.0, 6.0], [6.0, 3.0, 3.0]], [[8.0, 4.0, 5.0], [7.0, 5.0, 8.0], [1.0, 6.0, 1.0]], [[3.0, 4.0, 
6.0], [6.0, 4.0, 5.0], [7.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 1.0], [8.0, 7.0, 7.0], [1.0, 1.0, 3.0]], 
[[8.0, 6.0, 3.0], [2.0, 1.0, 2.0], [3.0, 1.0, 1.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 3.0], [3.0, 3.0, 5.0]]], [[[5.0, 6.0, 2.0], 
[2.0, 5.0, 2.0], [8.0, 4.0, 8.0]], [[4.0, 6.0, 4.0], [4.0, 4.0, 6.0], [6.0, 8.0, 2.0]], [[4.0, 5.0, 6.0], [7.0, 8.0, 3.0], 
[8.0, 3.0, 7.0]]], [[[6.0, 5.0, 4.0], [7.0, 5.0, 1.0], [8.0, 3.0, 4.0]], [[6.0, 3.0, 5.0], [6.0, 2.0, 1.0], [4.0, 8.0, 1.0]], 
[[2.0, 6.0, 7.0], [8.0, 8.0, 5.0], [7.0, 2.0, 5.0]]], [[[2.0, 8.0, 2.0], [5.0, 1.0, 6.0], [6.0, 8.0, 5.0]], [[1.0, 3.0, 6.0], 
[2.0, 7.0, 2.0], [8.0, 5.0, 3.0]], [[7.0, 8.0, 7.0], [1.0, 2.0, 5.0], [4.0, 1.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 7.0, 3.0]]], [[[4.0, 6.0, 4.0]]], [[[8.0, 5.0, 8.0]]], [[[3.0, 8.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[4.0, 8.0, 4.0]]], [[[7.0, 6.0, 1.0]]], [[[1.0, 1.0, 6.0]]], [[[4.0, 5.0, 5.0]]]])
      arg[:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 8.0], [5.0, 3.0, 5.0], [4.0, 5.0, 4.0]], [[5.0, 8.0, 1.0], [6.0, 5.0, 6.0], [4.0, 
7.0, 3.0]], [[4.0, 2.0, 2.0], [4.0, 3.0, 4.0], [2.0, 4.0, 3.0]]], [[[4.0, 8.0, 6.0], [6.0, 1.0, 2.0], [2.0, 7.0, 5.0]], [[4.0, 
3.0, 7.0], [6.0, 6.0, 3.0], [4.0, 6.0, 4.0]], [[4.0, 6.0, 8.0], [4.0, 2.0, 3.0], [4.0, 8.0, 5.0]]], [[[2.0, 6.0, 5.0], [4.0, 
1.0, 6.0], [8.0, 5.0, 3.0]], [[1.0, 1.0, 2.0], [4.0, 7.0, 8.0], [8.0, 5.0, 8.0]], [[3.0, 3.0, 2.0], [4.0, 2.0, 6.0], [2.0, 3.0, 
2.0]]], [[[4.0, 1.0, 7.0], [2.0, 4.0, 6.0], [6.0, 3.0, 3.0]], [[8.0, 4.0, 5.0], [7.0, 5.0, 8.0], [3.0, 8.0, 8.0]], [[3.0, 4.0, 
6.0], [6.0, 4.0, 5.0], [7.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 1.0], [8.0, 7.0, 7.0], [1.0, 1.0, 3.0]], 
[[8.0, 6.0, 3.0], [2.0, 1.0, 2.0], [4.0, 8.0, 4.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 3.0], [3.0, 3.0, 5.0]]], [[[5.0, 6.0, 2.0], 
[2.0, 5.0, 2.0], [8.0, 4.0, 8.0]], [[4.0, 6.0, 4.0], [4.0, 4.0, 6.0], [7.0, 6.0, 1.0]], [[4.0, 5.0, 6.0], [7.0, 8.0, 3.0], 
[8.0, 3.0, 7.0]]], [[[6.0, 5.0, 4.0], [7.0, 5.0, 1.0], [8.0, 3.0, 4.0]], [[6.0, 3.0, 5.0], [6.0, 2.0, 1.0], [1.0, 1.0, 6.0]], 
[[2.0, 6.0, 7.0], [8.0, 8.0, 5.0], [7.0, 2.0, 5.0]]], [[[2.0, 8.0, 2.0], [5.0, 1.0, 6.0], [6.0, 8.0, 5.0]], [[1.0, 3.0, 6.0], 
[2.0, 7.0, 2.0], [4.0, 5.0, 5.0]], [[7.0, 8.0, 7.0], [1.0, 2.0, 5.0], [4.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 6.0, 6.0], [6.0, 7.0, 5.0], [5.0, 5.0, 2.0]], [[6.0, 5.0, 4.0], [1.0, 4.0, 5.0], [5.0, 
5.0, 7.0]], [[7.0, 4.0, 5.0], [4.0, 4.0, 1.0], [4.0, 6.0, 7.0]]], [[[8.0, 7.0, 8.0], [7.0, 4.0, 5.0], [7.0, 4.0, 6.0]], [[4.0, 
4.0, 7.0], [5.0, 8.0, 3.0], [6.0, 7.0, 7.0]], [[5.0, 2.0, 7.0], [7.0, 4.0, 2.0], [7.0, 4.0, 6.0]]], [[[5.0, 6.0, 2.0], [5.0, 
8.0, 1.0], [7.0, 8.0, 8.0]], [[6.0, 5.0, 5.0], [8.0, 7.0, 7.0], [7.0, 3.0, 8.0]], [[3.0, 4.0, 1.0], [1.0, 3.0, 5.0], [4.0, 3.0, 
5.0]]], [[[7.0, 8.0, 8.0], [1.0, 6.0, 8.0], [5.0, 3.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[5.0, 8.0, 
5.0], [7.0, 5.0, 5.0], [2.0, 7.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 6.0], [5.0, 8.0, 6.0], [1.0, 8.0, 6.0]], 
[[7.0, 2.0, 2.0], [1.0, 5.0, 6.0], [3.0, 6.0, 1.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 5.0], [1.0, 3.0, 6.0]]], [[[4.0, 5.0, 6.0], 
[1.0, 7.0, 1.0], [1.0, 4.0, 7.0]], [[5.0, 2.0, 7.0], [3.0, 8.0, 3.0], [8.0, 6.0, 1.0]], [[4.0, 4.0, 3.0], [2.0, 4.0, 2.0], 
[1.0, 7.0, 4.0]]], [[[2.0, 3.0, 2.0], [4.0, 5.0, 7.0], [4.0, 2.0, 8.0]], [[4.0, 3.0, 7.0], [6.0, 5.0, 8.0], [2.0, 6.0, 2.0]], 
[[1.0, 7.0, 8.0], [2.0, 7.0, 4.0], [6.0, 7.0, 6.0]]], [[[8.0, 8.0, 5.0], [3.0, 6.0, 3.0], [1.0, 5.0, 3.0]], [[1.0, 4.0, 7.0], 
[2.0, 1.0, 4.0], [4.0, 7.0, 6.0]], [[1.0, 7.0, 4.0], [7.0, 5.0, 1.0], [2.0, 7.0, 3.0]]]])
      val=numpy.array([[[[3.0, 7.0, 7.0], [5.0, 1.0, 1.0]], [[5.0, 6.0, 7.0], [4.0, 5.0, 8.0]]], [[[7.0, 3.0, 3.0], [6.0, 1.0, 
8.0]], [[4.0, 4.0, 1.0], [5.0, 8.0, 8.0]]]])
      arg[1:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 6.0], [6.0, 7.0, 5.0], [5.0, 5.0, 2.0]], [[6.0, 5.0, 4.0], [1.0, 4.0, 5.0], [5.0, 
5.0, 7.0]], [[7.0, 4.0, 5.0], [4.0, 4.0, 1.0], [4.0, 6.0, 7.0]]], [[[3.0, 7.0, 7.0], [5.0, 1.0, 1.0], [7.0, 4.0, 6.0]], [[5.0, 
6.0, 7.0], [4.0, 5.0, 8.0], [6.0, 7.0, 7.0]], [[5.0, 2.0, 7.0], [7.0, 4.0, 2.0], [7.0, 4.0, 6.0]]], [[[7.0, 3.0, 3.0], [6.0, 
1.0, 8.0], [7.0, 8.0, 8.0]], [[4.0, 4.0, 1.0], [5.0, 8.0, 8.0], [7.0, 3.0, 8.0]], [[3.0, 4.0, 1.0], [1.0, 3.0, 5.0], [4.0, 3.0, 
5.0]]], [[[7.0, 8.0, 8.0], [1.0, 6.0, 8.0], [5.0, 3.0, 1.0]], [[4.0, 8.0, 4.0], [2.0, 4.0, 4.0], [5.0, 4.0, 5.0]], [[5.0, 8.0, 
5.0], [7.0, 5.0, 5.0], [2.0, 7.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 6.0], [5.0, 8.0, 6.0], [1.0, 8.0, 6.0]], 
[[7.0, 2.0, 2.0], [1.0, 5.0, 6.0], [3.0, 6.0, 1.0]], [[3.0, 8.0, 6.0], [6.0, 7.0, 5.0], [1.0, 3.0, 6.0]]], [[[3.0, 7.0, 7.0], 
[5.0, 1.0, 1.0], [1.0, 4.0, 7.0]], [[5.0, 6.0, 7.0], [4.0, 5.0, 8.0], [8.0, 6.0, 1.0]], [[4.0, 4.0, 3.0], [2.0, 4.0, 2.0], 
[1.0, 7.0, 4.0]]], [[[7.0, 3.0, 3.0], [6.0, 1.0, 8.0], [4.0, 2.0, 8.0]], [[4.0, 4.0, 1.0], [5.0, 8.0, 8.0], [2.0, 6.0, 2.0]], 
[[1.0, 7.0, 8.0], [2.0, 7.0, 4.0], [6.0, 7.0, 6.0]]], [[[8.0, 8.0, 5.0], [3.0, 6.0, 3.0], [1.0, 5.0, 3.0]], [[1.0, 4.0, 7.0], 
[2.0, 1.0, 4.0], [4.0, 7.0, 6.0]], [[1.0, 7.0, 4.0], [7.0, 5.0, 1.0], [2.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 2.0], [2.0, 6.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 2.0, 7.0], [8.0, 3.0, 1.0], [1.0, 
1.0, 6.0]], [[8.0, 6.0, 2.0], [2.0, 6.0, 5.0], [8.0, 2.0, 3.0]]], [[[8.0, 5.0, 1.0], [3.0, 8.0, 4.0], [7.0, 5.0, 5.0]], [[6.0, 
2.0, 8.0], [7.0, 3.0, 3.0], [2.0, 8.0, 2.0]], [[1.0, 7.0, 5.0], [8.0, 4.0, 7.0], [3.0, 4.0, 8.0]]], [[[6.0, 5.0, 7.0], [8.0, 
5.0, 2.0], [4.0, 1.0, 1.0]], [[8.0, 4.0, 8.0], [7.0, 3.0, 7.0], [6.0, 7.0, 6.0]], [[4.0, 8.0, 7.0], [1.0, 3.0, 2.0], [8.0, 7.0, 
1.0]]], [[[1.0, 2.0, 5.0], [6.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[6.0, 5.0, 1.0], [6.0, 3.0, 3.0], [8.0, 1.0, 4.0]], [[7.0, 8.0, 
8.0], [5.0, 8.0, 6.0], [1.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 5.0], [5.0, 6.0, 7.0], [7.0, 4.0, 7.0]], 
[[3.0, 5.0, 2.0], [7.0, 1.0, 3.0], [6.0, 8.0, 1.0]], [[3.0, 6.0, 8.0], [2.0, 4.0, 8.0], [2.0, 7.0, 5.0]]], [[[1.0, 6.0, 2.0], 
[5.0, 8.0, 4.0], [4.0, 3.0, 1.0]], [[8.0, 3.0, 4.0], [3.0, 7.0, 1.0], [7.0, 5.0, 6.0]], [[8.0, 3.0, 7.0], [6.0, 5.0, 1.0], 
[4.0, 2.0, 5.0]]], [[[5.0, 7.0, 1.0], [5.0, 4.0, 6.0], [5.0, 6.0, 2.0]], [[7.0, 1.0, 6.0], [7.0, 8.0, 8.0], [8.0, 3.0, 3.0]], 
[[5.0, 6.0, 6.0], [7.0, 4.0, 2.0], [1.0, 1.0, 3.0]]], [[[4.0, 8.0, 5.0], [5.0, 6.0, 1.0], [7.0, 5.0, 5.0]], [[2.0, 6.0, 6.0], 
[1.0, 3.0, 8.0], [1.0, 7.0, 7.0]], [[8.0, 4.0, 8.0], [2.0, 7.0, 3.0], [7.0, 4.0, 6.0]]]])
      val=Data(numpy.array([[[[4.0, 8.0, 8.0], [8.0, 3.0, 1.0]], [[5.0, 4.0, 5.0], [1.0, 4.0, 4.0]]], [[[3.0, 4.0, 2.0], [8.0, 
5.0, 5.0]], [[3.0, 4.0, 2.0], [8.0, 3.0, 4.0]]]]),self.functionspace)
      arg[1:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 2.0], [2.0, 6.0, 4.0], [8.0, 6.0, 6.0]], [[3.0, 2.0, 7.0], [8.0, 3.0, 1.0], [1.0, 
1.0, 6.0]], [[8.0, 6.0, 2.0], [2.0, 6.0, 5.0], [8.0, 2.0, 3.0]]], [[[4.0, 8.0, 8.0], [8.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[5.0, 
4.0, 5.0], [1.0, 4.0, 4.0], [2.0, 8.0, 2.0]], [[1.0, 7.0, 5.0], [8.0, 4.0, 7.0], [3.0, 4.0, 8.0]]], [[[3.0, 4.0, 2.0], [8.0, 
5.0, 5.0], [4.0, 1.0, 1.0]], [[3.0, 4.0, 2.0], [8.0, 3.0, 4.0], [6.0, 7.0, 6.0]], [[4.0, 8.0, 7.0], [1.0, 3.0, 2.0], [8.0, 7.0, 
1.0]]], [[[1.0, 2.0, 5.0], [6.0, 4.0, 1.0], [7.0, 6.0, 2.0]], [[6.0, 5.0, 1.0], [6.0, 3.0, 3.0], [8.0, 1.0, 4.0]], [[7.0, 8.0, 
8.0], [5.0, 8.0, 6.0], [1.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 5.0], [5.0, 6.0, 7.0], [7.0, 4.0, 7.0]], 
[[3.0, 5.0, 2.0], [7.0, 1.0, 3.0], [6.0, 8.0, 1.0]], [[3.0, 6.0, 8.0], [2.0, 4.0, 8.0], [2.0, 7.0, 5.0]]], [[[4.0, 8.0, 8.0], 
[8.0, 3.0, 1.0], [4.0, 3.0, 1.0]], [[5.0, 4.0, 5.0], [1.0, 4.0, 4.0], [7.0, 5.0, 6.0]], [[8.0, 3.0, 7.0], [6.0, 5.0, 1.0], 
[4.0, 2.0, 5.0]]], [[[3.0, 4.0, 2.0], [8.0, 5.0, 5.0], [5.0, 6.0, 2.0]], [[3.0, 4.0, 2.0], [8.0, 3.0, 4.0], [8.0, 3.0, 3.0]], 
[[5.0, 6.0, 6.0], [7.0, 4.0, 2.0], [1.0, 1.0, 3.0]]], [[[4.0, 8.0, 5.0], [5.0, 6.0, 1.0], [7.0, 5.0, 5.0]], [[2.0, 6.0, 6.0], 
[1.0, 3.0, 8.0], [1.0, 7.0, 7.0]], [[8.0, 4.0, 8.0], [2.0, 7.0, 3.0], [7.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 2.0, 1.0], [3.0, 8.0, 7.0], [4.0, 8.0, 8.0]], [[1.0, 7.0, 7.0], [6.0, 8.0, 1.0], [8.0, 
8.0, 1.0]], [[6.0, 3.0, 7.0], [3.0, 5.0, 2.0], [5.0, 5.0, 4.0]]], [[[8.0, 1.0, 1.0], [8.0, 1.0, 5.0], [5.0, 8.0, 3.0]], [[7.0, 
6.0, 8.0], [6.0, 6.0, 1.0], [3.0, 6.0, 7.0]], [[2.0, 6.0, 1.0], [2.0, 1.0, 6.0], [3.0, 2.0, 8.0]]], [[[3.0, 3.0, 5.0], [4.0, 
4.0, 8.0], [8.0, 8.0, 2.0]], [[5.0, 7.0, 5.0], [8.0, 1.0, 5.0], [8.0, 6.0, 4.0]], [[5.0, 8.0, 4.0], [8.0, 8.0, 4.0], [3.0, 5.0, 
2.0]]], [[[1.0, 6.0, 6.0], [6.0, 6.0, 1.0], [8.0, 7.0, 6.0]], [[6.0, 2.0, 1.0], [4.0, 3.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 2.0, 
3.0], [1.0, 8.0, 5.0], [5.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 5.0], [7.0, 2.0, 2.0], [5.0, 4.0, 5.0]], 
[[2.0, 1.0, 8.0], [6.0, 3.0, 4.0], [8.0, 2.0, 6.0]], [[6.0, 5.0, 4.0], [1.0, 5.0, 3.0], [4.0, 6.0, 8.0]]], [[[3.0, 7.0, 2.0], 
[3.0, 2.0, 6.0], [6.0, 1.0, 3.0]], [[2.0, 8.0, 2.0], [7.0, 2.0, 4.0], [7.0, 7.0, 5.0]], [[8.0, 3.0, 2.0], [7.0, 2.0, 6.0], 
[6.0, 8.0, 3.0]]], [[[5.0, 2.0, 8.0], [7.0, 5.0, 8.0], [3.0, 8.0, 3.0]], [[5.0, 3.0, 3.0], [8.0, 3.0, 8.0], [4.0, 7.0, 4.0]], 
[[2.0, 2.0, 6.0], [3.0, 7.0, 8.0], [1.0, 5.0, 8.0]]], [[[6.0, 2.0, 1.0], [1.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], 
[8.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[5.0, 4.0, 8.0], [5.0, 6.0, 1.0], [1.0, 1.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 1.0, 7.0], [7.0, 4.0, 6.0]], [[4.0, 8.0, 8.0], [7.0, 8.0, 4.0]]], [[[8.0, 8.0, 6.0], 
[1.0, 5.0, 6.0]], [[6.0, 5.0, 1.0], [4.0, 6.0, 3.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 6.0, 2.0], [5.0, 4.0, 3.0]], [[4.0, 
6.0, 4.0], [6.0, 1.0, 7.0]]], [[[2.0, 3.0, 1.0], [2.0, 5.0, 2.0]], [[7.0, 3.0, 3.0], [4.0, 2.0, 6.0]]]])
      arg[1:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 2.0, 1.0], [3.0, 8.0, 7.0], [4.0, 8.0, 8.0]], [[1.0, 7.0, 7.0], [6.0, 8.0, 1.0], [8.0, 
8.0, 1.0]], [[6.0, 3.0, 7.0], [3.0, 5.0, 2.0], [5.0, 5.0, 4.0]]], [[[2.0, 1.0, 7.0], [7.0, 4.0, 6.0], [5.0, 8.0, 3.0]], [[4.0, 
8.0, 8.0], [7.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[2.0, 6.0, 1.0], [2.0, 1.0, 6.0], [3.0, 2.0, 8.0]]], [[[8.0, 8.0, 6.0], [1.0, 
5.0, 6.0], [8.0, 8.0, 2.0]], [[6.0, 5.0, 1.0], [4.0, 6.0, 3.0], [8.0, 6.0, 4.0]], [[5.0, 8.0, 4.0], [8.0, 8.0, 4.0], [3.0, 5.0, 
2.0]]], [[[1.0, 6.0, 6.0], [6.0, 6.0, 1.0], [8.0, 7.0, 6.0]], [[6.0, 2.0, 1.0], [4.0, 3.0, 3.0], [2.0, 4.0, 1.0]], [[3.0, 2.0, 
3.0], [1.0, 8.0, 5.0], [5.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 5.0], [7.0, 2.0, 2.0], [5.0, 4.0, 5.0]], 
[[2.0, 1.0, 8.0], [6.0, 3.0, 4.0], [8.0, 2.0, 6.0]], [[6.0, 5.0, 4.0], [1.0, 5.0, 3.0], [4.0, 6.0, 8.0]]], [[[8.0, 6.0, 2.0], 
[5.0, 4.0, 3.0], [6.0, 1.0, 3.0]], [[4.0, 6.0, 4.0], [6.0, 1.0, 7.0], [7.0, 7.0, 5.0]], [[8.0, 3.0, 2.0], [7.0, 2.0, 6.0], 
[6.0, 8.0, 3.0]]], [[[2.0, 3.0, 1.0], [2.0, 5.0, 2.0], [3.0, 8.0, 3.0]], [[7.0, 3.0, 3.0], [4.0, 2.0, 6.0], [4.0, 7.0, 4.0]], 
[[2.0, 2.0, 6.0], [3.0, 7.0, 8.0], [1.0, 5.0, 8.0]]], [[[6.0, 2.0, 1.0], [1.0, 2.0, 7.0], [2.0, 1.0, 7.0]], [[7.0, 5.0, 8.0], 
[8.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[5.0, 4.0, 8.0], [5.0, 6.0, 1.0], [1.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 8.0], [5.0, 2.0, 5.0], [7.0, 8.0, 4.0]], [[5.0, 5.0, 3.0], [3.0, 8.0, 5.0], [7.0, 
4.0, 3.0]], [[8.0, 7.0, 6.0], [7.0, 8.0, 3.0], [7.0, 1.0, 3.0]]], [[[8.0, 8.0, 5.0], [1.0, 5.0, 1.0], [2.0, 3.0, 8.0]], [[6.0, 
3.0, 7.0], [7.0, 6.0, 4.0], [6.0, 5.0, 5.0]], [[2.0, 2.0, 7.0], [2.0, 6.0, 6.0], [4.0, 6.0, 1.0]]], [[[2.0, 4.0, 1.0], [6.0, 
8.0, 1.0], [7.0, 4.0, 2.0]], [[7.0, 8.0, 1.0], [7.0, 1.0, 7.0], [6.0, 2.0, 4.0]], [[8.0, 4.0, 3.0], [8.0, 1.0, 2.0], [7.0, 4.0, 
2.0]]], [[[6.0, 4.0, 7.0], [4.0, 2.0, 4.0], [3.0, 1.0, 8.0]], [[4.0, 2.0, 5.0], [2.0, 4.0, 4.0], [3.0, 3.0, 5.0]], [[2.0, 3.0, 
4.0], [7.0, 3.0, 2.0], [1.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 5.0], [5.0, 1.0, 1.0], [6.0, 6.0, 4.0]], 
[[1.0, 1.0, 6.0], [7.0, 8.0, 4.0], [7.0, 4.0, 6.0]], [[7.0, 2.0, 3.0], [5.0, 7.0, 6.0], [4.0, 6.0, 3.0]]], [[[4.0, 3.0, 3.0], 
[8.0, 5.0, 2.0], [1.0, 6.0, 5.0]], [[1.0, 1.0, 4.0], [2.0, 3.0, 1.0], [3.0, 3.0, 7.0]], [[3.0, 3.0, 8.0], [5.0, 8.0, 7.0], 
[2.0, 5.0, 1.0]]], [[[8.0, 4.0, 7.0], [2.0, 2.0, 8.0], [6.0, 8.0, 3.0]], [[7.0, 5.0, 5.0], [1.0, 7.0, 6.0], [2.0, 7.0, 7.0]], 
[[1.0, 6.0, 6.0], [3.0, 1.0, 4.0], [7.0, 3.0, 1.0]]], [[[4.0, 4.0, 5.0], [3.0, 5.0, 3.0], [4.0, 4.0, 7.0]], [[8.0, 1.0, 7.0], 
[2.0, 4.0, 8.0], [6.0, 2.0, 6.0]], [[3.0, 5.0, 7.0], [3.0, 3.0, 8.0], [8.0, 1.0, 8.0]]]])
      val=numpy.array([[[[7.0, 2.0, 3.0], [2.0, 3.0, 4.0], [1.0, 2.0, 5.0]], [[6.0, 6.0, 4.0], [1.0, 2.0, 3.0], [7.0, 6.0, 
8.0]], [[2.0, 1.0, 1.0], [7.0, 6.0, 2.0], [5.0, 5.0, 8.0]]], [[[5.0, 7.0, 2.0], [3.0, 7.0, 1.0], [7.0, 8.0, 6.0]], [[5.0, 6.0, 
7.0], [3.0, 4.0, 4.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 3.0], [6.0, 7.0, 6.0], [7.0, 2.0, 6.0]]], [[[1.0, 8.0, 8.0], [7.0, 8.0, 
8.0], [8.0, 8.0, 1.0]], [[1.0, 7.0, 2.0], [4.0, 3.0, 2.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 1.0], [2.0, 8.0, 3.0], [1.0, 4.0, 
8.0]]], [[[6.0, 6.0, 1.0], [8.0, 2.0, 4.0], [6.0, 5.0, 3.0]], [[6.0, 1.0, 5.0], [8.0, 8.0, 6.0], [4.0, 1.0, 5.0]], [[4.0, 1.0, 
2.0], [3.0, 3.0, 6.0], [2.0, 7.0, 1.0]]]])
      arg[:,:,:]=val
      ref=Data(numpy.array([[[[7.0, 2.0, 3.0], [2.0, 3.0, 4.0], [1.0, 2.0, 5.0]], [[6.0, 6.0, 4.0], [1.0, 2.0, 3.0], [7.0, 6.0, 
8.0]], [[2.0, 1.0, 1.0], [7.0, 6.0, 2.0], [5.0, 5.0, 8.0]]], [[[5.0, 7.0, 2.0], [3.0, 7.0, 1.0], [7.0, 8.0, 6.0]], [[5.0, 6.0, 
7.0], [3.0, 4.0, 4.0], [2.0, 1.0, 6.0]], [[2.0, 4.0, 3.0], [6.0, 7.0, 6.0], [7.0, 2.0, 6.0]]], [[[1.0, 8.0, 8.0], [7.0, 8.0, 
8.0], [8.0, 8.0, 1.0]], [[1.0, 7.0, 2.0], [4.0, 3.0, 2.0], [6.0, 5.0, 2.0]], [[7.0, 2.0, 1.0], [2.0, 8.0, 3.0], [1.0, 4.0, 
8.0]]], [[[6.0, 6.0, 1.0], [8.0, 2.0, 4.0], [6.0, 5.0, 3.0]], [[6.0, 1.0, 5.0], [8.0, 8.0, 6.0], [4.0, 1.0, 5.0]], [[4.0, 1.0, 
2.0], [3.0, 3.0, 6.0], [2.0, 7.0, 1.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 5.0], [5.0, 7.0, 3.0], [7.0, 4.0, 8.0]], [[7.0, 6.0, 4.0], [5.0, 1.0, 5.0], [4.0, 
7.0, 5.0]], [[8.0, 8.0, 3.0], [7.0, 7.0, 6.0], [4.0, 8.0, 1.0]]], [[[6.0, 5.0, 1.0], [4.0, 7.0, 8.0], [7.0, 8.0, 8.0]], [[4.0, 
6.0, 7.0], [3.0, 3.0, 7.0], [4.0, 7.0, 6.0]], [[8.0, 7.0, 7.0], [8.0, 5.0, 1.0], [7.0, 2.0, 6.0]]], [[[8.0, 2.0, 5.0], [6.0, 
1.0, 8.0], [7.0, 6.0, 2.0]], [[1.0, 2.0, 2.0], [8.0, 1.0, 4.0], [2.0, 6.0, 3.0]], [[5.0, 4.0, 3.0], [6.0, 5.0, 2.0], [5.0, 5.0, 
6.0]]], [[[8.0, 6.0, 1.0], [1.0, 2.0, 7.0], [6.0, 2.0, 2.0]], [[4.0, 8.0, 5.0], [5.0, 2.0, 2.0], [8.0, 6.0, 2.0]], [[7.0, 6.0, 
5.0], [2.0, 5.0, 3.0], [3.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 6.0], [1.0, 2.0, 5.0], [1.0, 2.0, 6.0]], 
[[2.0, 8.0, 8.0], [3.0, 5.0, 5.0], [7.0, 6.0, 8.0]], [[6.0, 1.0, 1.0], [3.0, 7.0, 5.0], [8.0, 1.0, 4.0]]], [[[3.0, 6.0, 3.0], 
[8.0, 4.0, 8.0], [4.0, 1.0, 2.0]], [[4.0, 2.0, 7.0], [5.0, 3.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 2.0, 4.0], [6.0, 3.0, 3.0], 
[7.0, 7.0, 7.0]]], [[[7.0, 8.0, 4.0], [2.0, 4.0, 3.0], [5.0, 4.0, 3.0]], [[6.0, 8.0, 3.0], [6.0, 5.0, 2.0], [8.0, 2.0, 1.0]], 
[[7.0, 6.0, 3.0], [3.0, 6.0, 8.0], [2.0, 2.0, 4.0]]], [[[8.0, 6.0, 7.0], [1.0, 2.0, 2.0], [3.0, 5.0, 2.0]], [[7.0, 3.0, 1.0], 
[4.0, 6.0, 2.0], [1.0, 3.0, 5.0]], [[7.0, 1.0, 4.0], [8.0, 2.0, 6.0], [3.0, 3.0, 7.0]]]])
      val=Data(numpy.array([[[[6.0, 8.0, 7.0], [3.0, 1.0, 2.0], [5.0, 3.0, 1.0]], [[4.0, 6.0, 5.0], [1.0, 8.0, 3.0], [4.0, 7.0, 
8.0]], [[5.0, 8.0, 6.0], [4.0, 2.0, 6.0], [6.0, 1.0, 1.0]]], [[[6.0, 3.0, 3.0], [1.0, 2.0, 1.0], [3.0, 4.0, 4.0]], [[2.0, 8.0, 
5.0], [3.0, 5.0, 8.0], [1.0, 6.0, 2.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 5.0], [2.0, 7.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 1.0, 
5.0], [1.0, 3.0, 6.0]], [[3.0, 6.0, 6.0], [5.0, 4.0, 2.0], [4.0, 1.0, 2.0]], [[4.0, 1.0, 3.0], [8.0, 2.0, 8.0], [2.0, 5.0, 
7.0]]], [[[1.0, 2.0, 3.0], [4.0, 7.0, 4.0], [8.0, 5.0, 3.0]], [[3.0, 5.0, 7.0], [6.0, 6.0, 3.0], [8.0, 1.0, 8.0]], [[3.0, 4.0, 
3.0], [4.0, 5.0, 4.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      arg[:,:,:]=val
      ref=Data(numpy.array([[[[6.0, 8.0, 7.0], [3.0, 1.0, 2.0], [5.0, 3.0, 1.0]], [[4.0, 6.0, 5.0], [1.0, 8.0, 3.0], [4.0, 7.0, 
8.0]], [[5.0, 8.0, 6.0], [4.0, 2.0, 6.0], [6.0, 1.0, 1.0]]], [[[6.0, 3.0, 3.0], [1.0, 2.0, 1.0], [3.0, 4.0, 4.0]], [[2.0, 8.0, 
5.0], [3.0, 5.0, 8.0], [1.0, 6.0, 2.0]], [[6.0, 8.0, 6.0], [5.0, 4.0, 5.0], [2.0, 7.0, 7.0]]], [[[6.0, 6.0, 1.0], [2.0, 1.0, 
5.0], [1.0, 3.0, 6.0]], [[3.0, 6.0, 6.0], [5.0, 4.0, 2.0], [4.0, 1.0, 2.0]], [[4.0, 1.0, 3.0], [8.0, 2.0, 8.0], [2.0, 5.0, 
7.0]]], [[[1.0, 2.0, 3.0], [4.0, 7.0, 4.0], [8.0, 5.0, 3.0]], [[3.0, 5.0, 7.0], [6.0, 6.0, 3.0], [8.0, 1.0, 8.0]], [[3.0, 4.0, 
3.0], [4.0, 5.0, 4.0], [5.0, 1.0, 6.0]]]]),self.functionspace)
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 3.0], [6.0, 5.0, 7.0], [8.0, 3.0, 4.0]], [[8.0, 5.0, 6.0], [5.0, 2.0, 4.0], [7.0, 
8.0, 3.0]], [[2.0, 2.0, 2.0], [7.0, 1.0, 1.0], [8.0, 2.0, 6.0]]], [[[5.0, 6.0, 4.0], [6.0, 2.0, 8.0], [8.0, 8.0, 5.0]], [[8.0, 
1.0, 7.0], [7.0, 7.0, 8.0], [3.0, 8.0, 8.0]], [[2.0, 6.0, 8.0], [3.0, 4.0, 5.0], [3.0, 1.0, 3.0]]], [[[5.0, 1.0, 6.0], [1.0, 
8.0, 7.0], [7.0, 2.0, 1.0]], [[7.0, 8.0, 5.0], [6.0, 5.0, 1.0], [3.0, 4.0, 4.0]], [[8.0, 8.0, 7.0], [8.0, 6.0, 6.0], [4.0, 7.0, 
1.0]]], [[[6.0, 2.0, 5.0], [6.0, 3.0, 6.0], [2.0, 6.0, 4.0]], [[2.0, 2.0, 7.0], [4.0, 3.0, 3.0], [4.0, 4.0, 5.0]], [[3.0, 4.0, 
7.0], [1.0, 7.0, 1.0], [7.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 2.0], [4.0, 1.0, 4.0], [6.0, 2.0, 4.0]], 
[[6.0, 8.0, 8.0], [4.0, 2.0, 2.0], [8.0, 7.0, 7.0]], [[1.0, 7.0, 1.0], [1.0, 5.0, 8.0], [3.0, 6.0, 6.0]]], [[[6.0, 1.0, 4.0], 
[8.0, 4.0, 3.0], [4.0, 7.0, 8.0]], [[3.0, 2.0, 1.0], [6.0, 6.0, 1.0], [7.0, 1.0, 4.0]], [[5.0, 6.0, 2.0], [7.0, 2.0, 4.0], 
[1.0, 7.0, 4.0]]], [[[6.0, 8.0, 5.0], [3.0, 2.0, 3.0], [8.0, 4.0, 6.0]], [[3.0, 1.0, 8.0], [6.0, 5.0, 1.0], [6.0, 7.0, 8.0]], 
[[1.0, 4.0, 3.0], [3.0, 3.0, 6.0], [1.0, 1.0, 2.0]]], [[[1.0, 3.0, 6.0], [4.0, 3.0, 1.0], [5.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], 
[1.0, 2.0, 1.0], [5.0, 2.0, 7.0]], [[6.0, 6.0, 8.0], [7.0, 3.0, 7.0], [4.0, 5.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 6.0, 4.0], [3.0, 1.0, 2.0], [1.0, 4.0, 2.0]], [[1.0, 5.0, 1.0], [2.0, 8.0, 4.0], [1.0, 
5.0, 4.0]], [[1.0, 8.0, 7.0], [6.0, 5.0, 2.0], [7.0, 7.0, 8.0]]], [[[3.0, 3.0, 8.0], [1.0, 5.0, 3.0], [2.0, 8.0, 1.0]], [[8.0, 
6.0, 3.0], [4.0, 4.0, 5.0], [5.0, 8.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 3.0, 4.0], [4.0, 4.0, 3.0]]], [[[6.0, 5.0, 5.0], [2.0, 
5.0, 3.0], [3.0, 1.0, 3.0]], [[7.0, 2.0, 4.0], [6.0, 3.0, 1.0], [4.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 8.0, 8.0], [4.0, 4.0, 
1.0]]], [[[1.0, 2.0, 5.0], [6.0, 2.0, 4.0], [7.0, 5.0, 2.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 4.0], [5.0, 1.0, 8.0]], [[7.0, 8.0, 
6.0], [7.0, 4.0, 1.0], [1.0, 4.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 2.0, 8.0], [3.0, 7.0, 3.0], [4.0, 6.0, 2.0]], 
[[8.0, 2.0, 4.0], [2.0, 4.0, 5.0], [1.0, 8.0, 1.0]], [[3.0, 3.0, 1.0], [2.0, 3.0, 6.0], [7.0, 6.0, 2.0]]], [[[7.0, 5.0, 2.0], 
[1.0, 3.0, 6.0], [4.0, 3.0, 3.0]], [[4.0, 3.0, 5.0], [8.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[3.0, 1.0, 8.0], [3.0, 5.0, 8.0], 
[5.0, 3.0, 5.0]]], [[[8.0, 6.0, 7.0], [3.0, 6.0, 6.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 6.0], [2.0, 4.0, 2.0], [7.0, 5.0, 4.0]], 
[[4.0, 7.0, 1.0], [2.0, 4.0, 4.0], [5.0, 8.0, 6.0]]], [[[2.0, 1.0, 3.0], [7.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], 
[5.0, 7.0, 4.0], [3.0, 6.0, 6.0]], [[7.0, 2.0, 8.0], [1.0, 6.0, 7.0], [4.0, 7.0, 7.0]]]])
      arg[:,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 4.0], [3.0, 1.0, 2.0], [1.0, 4.0, 2.0]], [[1.0, 5.0, 1.0], [2.0, 8.0, 4.0], [1.0, 
5.0, 4.0]], [[1.0, 8.0, 7.0], [6.0, 5.0, 2.0], [7.0, 7.0, 8.0]]], [[[3.0, 3.0, 8.0], [1.0, 5.0, 3.0], [2.0, 8.0, 1.0]], [[8.0, 
6.0, 3.0], [4.0, 4.0, 5.0], [5.0, 8.0, 7.0]], [[6.0, 5.0, 3.0], [1.0, 3.0, 4.0], [4.0, 4.0, 3.0]]], [[[6.0, 5.0, 5.0], [2.0, 
5.0, 3.0], [3.0, 1.0, 3.0]], [[7.0, 2.0, 4.0], [6.0, 3.0, 1.0], [4.0, 8.0, 2.0]], [[8.0, 8.0, 6.0], [3.0, 8.0, 8.0], [4.0, 4.0, 
1.0]]], [[[1.0, 2.0, 5.0], [6.0, 2.0, 4.0], [7.0, 5.0, 2.0]], [[5.0, 5.0, 7.0], [2.0, 3.0, 4.0], [5.0, 1.0, 8.0]], [[7.0, 8.0, 
6.0], [7.0, 4.0, 1.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [3.0, 7.0, 3.0], [4.0, 6.0, 2.0]], 
[[8.0, 2.0, 4.0], [2.0, 4.0, 5.0], [1.0, 8.0, 1.0]], [[3.0, 3.0, 1.0], [2.0, 3.0, 6.0], [7.0, 6.0, 2.0]]], [[[7.0, 5.0, 2.0], 
[1.0, 3.0, 6.0], [4.0, 3.0, 3.0]], [[4.0, 3.0, 5.0], [8.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[3.0, 1.0, 8.0], [3.0, 5.0, 8.0], 
[5.0, 3.0, 5.0]]], [[[8.0, 6.0, 7.0], [3.0, 6.0, 6.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 6.0], [2.0, 4.0, 2.0], [7.0, 5.0, 4.0]], 
[[4.0, 7.0, 1.0], [2.0, 4.0, 4.0], [5.0, 8.0, 6.0]]], [[[2.0, 1.0, 3.0], [7.0, 6.0, 6.0], [4.0, 2.0, 3.0]], [[4.0, 8.0, 3.0], 
[5.0, 7.0, 4.0], [3.0, 6.0, 6.0]], [[7.0, 2.0, 8.0], [1.0, 6.0, 7.0], [4.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 4.0], [7.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[5.0, 5.0, 5.0], [4.0, 3.0, 5.0], [2.0, 
4.0, 2.0]], [[5.0, 5.0, 3.0], [3.0, 1.0, 3.0], [1.0, 4.0, 3.0]]], [[[4.0, 4.0, 3.0], [8.0, 4.0, 3.0], [1.0, 4.0, 4.0]], [[7.0, 
4.0, 3.0], [8.0, 5.0, 3.0], [8.0, 7.0, 6.0]], [[7.0, 1.0, 7.0], [2.0, 8.0, 8.0], [5.0, 1.0, 1.0]]], [[[7.0, 2.0, 2.0], [8.0, 
6.0, 1.0], [8.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [8.0, 3.0, 4.0], [8.0, 8.0, 5.0]], [[8.0, 8.0, 8.0], [6.0, 4.0, 8.0], [6.0, 2.0, 
4.0]]], [[[4.0, 8.0, 3.0], [7.0, 6.0, 3.0], [2.0, 6.0, 8.0]], [[2.0, 4.0, 8.0], [1.0, 5.0, 8.0], [3.0, 4.0, 7.0]], [[3.0, 1.0, 
4.0], [8.0, 8.0, 8.0], [2.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 5.0], [2.0, 5.0, 3.0], [4.0, 2.0, 4.0]], 
[[8.0, 2.0, 1.0], [8.0, 3.0, 5.0], [5.0, 1.0, 7.0]], [[1.0, 1.0, 7.0], [7.0, 7.0, 5.0], [8.0, 4.0, 8.0]]], [[[7.0, 1.0, 8.0], 
[3.0, 5.0, 8.0], [7.0, 5.0, 1.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 4.0], [3.0, 2.0, 1.0]], [[1.0, 4.0, 7.0], [3.0, 1.0, 4.0], 
[2.0, 7.0, 4.0]]], [[[1.0, 4.0, 4.0], [5.0, 3.0, 2.0], [8.0, 8.0, 2.0]], [[2.0, 6.0, 2.0], [6.0, 2.0, 8.0], [7.0, 7.0, 2.0]], 
[[5.0, 1.0, 7.0], [3.0, 6.0, 2.0], [4.0, 8.0, 2.0]]], [[[8.0, 5.0, 4.0], [3.0, 6.0, 3.0], [2.0, 5.0, 4.0]], [[5.0, 5.0, 2.0], 
[7.0, 4.0, 1.0], [3.0, 8.0, 8.0]], [[2.0, 4.0, 8.0], [4.0, 5.0, 6.0], [7.0, 6.0, 4.0]]]])
      val=numpy.array([[[[2.0, 7.0, 8.0], [8.0, 3.0, 3.0], [7.0, 7.0, 7.0]]], [[[2.0, 4.0, 2.0], [8.0, 4.0, 7.0], [5.0, 5.0, 
5.0]]], [[[4.0, 3.0, 3.0], [7.0, 7.0, 6.0], [2.0, 3.0, 1.0]]], [[[4.0, 1.0, 8.0], [8.0, 5.0, 3.0], [5.0, 7.0, 4.0]]]])
      arg[:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [7.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[5.0, 5.0, 5.0], [4.0, 3.0, 5.0], [2.0, 
4.0, 2.0]], [[2.0, 7.0, 8.0], [8.0, 3.0, 3.0], [7.0, 7.0, 7.0]]], [[[4.0, 4.0, 3.0], [8.0, 4.0, 3.0], [1.0, 4.0, 4.0]], [[7.0, 
4.0, 3.0], [8.0, 5.0, 3.0], [8.0, 7.0, 6.0]], [[2.0, 4.0, 2.0], [8.0, 4.0, 7.0], [5.0, 5.0, 5.0]]], [[[7.0, 2.0, 2.0], [8.0, 
6.0, 1.0], [8.0, 3.0, 1.0]], [[6.0, 2.0, 3.0], [8.0, 3.0, 4.0], [8.0, 8.0, 5.0]], [[4.0, 3.0, 3.0], [7.0, 7.0, 6.0], [2.0, 3.0, 
1.0]]], [[[4.0, 8.0, 3.0], [7.0, 6.0, 3.0], [2.0, 6.0, 8.0]], [[2.0, 4.0, 8.0], [1.0, 5.0, 8.0], [3.0, 4.0, 7.0]], [[4.0, 1.0, 
8.0], [8.0, 5.0, 3.0], [5.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 5.0], [2.0, 5.0, 3.0], [4.0, 2.0, 4.0]], 
[[8.0, 2.0, 1.0], [8.0, 3.0, 5.0], [5.0, 1.0, 7.0]], [[2.0, 7.0, 8.0], [8.0, 3.0, 3.0], [7.0, 7.0, 7.0]]], [[[7.0, 1.0, 8.0], 
[3.0, 5.0, 8.0], [7.0, 5.0, 1.0]], [[5.0, 8.0, 2.0], [2.0, 5.0, 4.0], [3.0, 2.0, 1.0]], [[2.0, 4.0, 2.0], [8.0, 4.0, 7.0], 
[5.0, 5.0, 5.0]]], [[[1.0, 4.0, 4.0], [5.0, 3.0, 2.0], [8.0, 8.0, 2.0]], [[2.0, 6.0, 2.0], [6.0, 2.0, 8.0], [7.0, 7.0, 2.0]], 
[[4.0, 3.0, 3.0], [7.0, 7.0, 6.0], [2.0, 3.0, 1.0]]], [[[8.0, 5.0, 4.0], [3.0, 6.0, 3.0], [2.0, 5.0, 4.0]], [[5.0, 5.0, 2.0], 
[7.0, 4.0, 1.0], [3.0, 8.0, 8.0]], [[4.0, 1.0, 8.0], [8.0, 5.0, 3.0], [5.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 5.0], [2.0, 8.0, 6.0], [1.0, 5.0, 6.0]], [[7.0, 6.0, 7.0], [2.0, 8.0, 2.0], [4.0, 
3.0, 3.0]], [[4.0, 2.0, 8.0], [7.0, 5.0, 3.0], [8.0, 1.0, 6.0]]], [[[5.0, 2.0, 7.0], [6.0, 3.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 
4.0, 7.0], [2.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[6.0, 6.0, 2.0], [7.0, 7.0, 8.0], [1.0, 4.0, 1.0]]], [[[2.0, 8.0, 6.0], [7.0, 
2.0, 1.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 4.0], [2.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[4.0, 3.0, 8.0], [6.0, 5.0, 4.0], [8.0, 4.0, 
6.0]]], [[[3.0, 5.0, 5.0], [5.0, 2.0, 5.0], [2.0, 4.0, 5.0]], [[5.0, 1.0, 2.0], [1.0, 6.0, 2.0], [2.0, 4.0, 2.0]], [[3.0, 8.0, 
2.0], [3.0, 2.0, 8.0], [1.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 6.0], [4.0, 8.0, 4.0], [7.0, 8.0, 6.0]], 
[[3.0, 4.0, 7.0], [6.0, 7.0, 1.0], [5.0, 1.0, 3.0]], [[6.0, 2.0, 2.0], [4.0, 4.0, 7.0], [7.0, 3.0, 3.0]]], [[[3.0, 4.0, 3.0], 
[5.0, 8.0, 1.0], [6.0, 7.0, 4.0]], [[1.0, 5.0, 6.0], [5.0, 6.0, 6.0], [1.0, 8.0, 5.0]], [[5.0, 8.0, 2.0], [1.0, 2.0, 7.0], 
[1.0, 5.0, 2.0]]], [[[1.0, 8.0, 7.0], [4.0, 3.0, 2.0], [5.0, 5.0, 8.0]], [[3.0, 7.0, 2.0], [2.0, 3.0, 6.0], [4.0, 1.0, 2.0]], 
[[7.0, 3.0, 2.0], [5.0, 2.0, 3.0], [1.0, 7.0, 2.0]]], [[[2.0, 2.0, 8.0], [8.0, 5.0, 7.0], [4.0, 8.0, 4.0]], [[7.0, 2.0, 8.0], 
[5.0, 5.0, 4.0], [6.0, 7.0, 7.0]], [[2.0, 2.0, 6.0], [5.0, 4.0, 7.0], [3.0, 2.0, 2.0]]]])
      val=Data(numpy.array([[[[5.0, 5.0, 5.0], [3.0, 5.0, 5.0], [6.0, 1.0, 1.0]]], [[[6.0, 8.0, 7.0], [2.0, 2.0, 2.0], [7.0, 
3.0, 1.0]]], [[[5.0, 2.0, 8.0], [8.0, 2.0, 7.0], [1.0, 6.0, 5.0]]], [[[5.0, 6.0, 2.0], [1.0, 1.0, 7.0], [7.0, 5.0, 
4.0]]]]),self.functionspace)
      arg[:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 5.0], [2.0, 8.0, 6.0], [1.0, 5.0, 6.0]], [[7.0, 6.0, 7.0], [2.0, 8.0, 2.0], [4.0, 
3.0, 3.0]], [[5.0, 5.0, 5.0], [3.0, 5.0, 5.0], [6.0, 1.0, 1.0]]], [[[5.0, 2.0, 7.0], [6.0, 3.0, 2.0], [6.0, 3.0, 5.0]], [[7.0, 
4.0, 7.0], [2.0, 1.0, 2.0], [2.0, 8.0, 3.0]], [[6.0, 8.0, 7.0], [2.0, 2.0, 2.0], [7.0, 3.0, 1.0]]], [[[2.0, 8.0, 6.0], [7.0, 
2.0, 1.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 4.0], [2.0, 5.0, 2.0], [5.0, 2.0, 4.0]], [[5.0, 2.0, 8.0], [8.0, 2.0, 7.0], [1.0, 6.0, 
5.0]]], [[[3.0, 5.0, 5.0], [5.0, 2.0, 5.0], [2.0, 4.0, 5.0]], [[5.0, 1.0, 2.0], [1.0, 6.0, 2.0], [2.0, 4.0, 2.0]], [[5.0, 6.0, 
2.0], [1.0, 1.0, 7.0], [7.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 6.0], [4.0, 8.0, 4.0], [7.0, 8.0, 6.0]], 
[[3.0, 4.0, 7.0], [6.0, 7.0, 1.0], [5.0, 1.0, 3.0]], [[5.0, 5.0, 5.0], [3.0, 5.0, 5.0], [6.0, 1.0, 1.0]]], [[[3.0, 4.0, 3.0], 
[5.0, 8.0, 1.0], [6.0, 7.0, 4.0]], [[1.0, 5.0, 6.0], [5.0, 6.0, 6.0], [1.0, 8.0, 5.0]], [[6.0, 8.0, 7.0], [2.0, 2.0, 2.0], 
[7.0, 3.0, 1.0]]], [[[1.0, 8.0, 7.0], [4.0, 3.0, 2.0], [5.0, 5.0, 8.0]], [[3.0, 7.0, 2.0], [2.0, 3.0, 6.0], [4.0, 1.0, 2.0]], 
[[5.0, 2.0, 8.0], [8.0, 2.0, 7.0], [1.0, 6.0, 5.0]]], [[[2.0, 2.0, 8.0], [8.0, 5.0, 7.0], [4.0, 8.0, 4.0]], [[7.0, 2.0, 8.0], 
[5.0, 5.0, 4.0], [6.0, 7.0, 7.0]], [[5.0, 6.0, 2.0], [1.0, 1.0, 7.0], [7.0, 5.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 1.0], [2.0, 1.0, 3.0], [8.0, 2.0, 3.0]], [[5.0, 3.0, 5.0], [4.0, 8.0, 6.0], [6.0, 
1.0, 2.0]], [[4.0, 5.0, 8.0], [2.0, 6.0, 4.0], [6.0, 3.0, 6.0]]], [[[1.0, 3.0, 7.0], [8.0, 7.0, 2.0], [8.0, 2.0, 8.0]], [[5.0, 
8.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 5.0]], [[2.0, 4.0, 8.0], [3.0, 4.0, 6.0], [1.0, 5.0, 8.0]]], [[[5.0, 7.0, 1.0], [2.0, 
4.0, 2.0], [4.0, 2.0, 2.0]], [[3.0, 4.0, 4.0], [5.0, 8.0, 4.0], [1.0, 2.0, 1.0]], [[2.0, 8.0, 4.0], [4.0, 5.0, 2.0], [7.0, 8.0, 
3.0]]], [[[8.0, 3.0, 2.0], [3.0, 5.0, 3.0], [3.0, 3.0, 2.0]], [[2.0, 2.0, 2.0], [3.0, 1.0, 5.0], [3.0, 2.0, 6.0]], [[7.0, 3.0, 
2.0], [4.0, 8.0, 1.0], [1.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 3.0], [1.0, 7.0, 7.0], [6.0, 8.0, 1.0]], 
[[5.0, 1.0, 7.0], [1.0, 7.0, 3.0], [8.0, 3.0, 6.0]], [[6.0, 2.0, 2.0], [3.0, 8.0, 8.0], [5.0, 2.0, 6.0]]], [[[3.0, 5.0, 4.0], 
[2.0, 1.0, 4.0], [5.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [2.0, 4.0, 3.0], [1.0, 8.0, 8.0]], [[7.0, 6.0, 8.0], [2.0, 3.0, 5.0], 
[4.0, 4.0, 3.0]]], [[[7.0, 4.0, 1.0], [7.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[8.0, 3.0, 2.0], [2.0, 5.0, 6.0], [7.0, 8.0, 3.0]], 
[[2.0, 8.0, 6.0], [6.0, 6.0, 4.0], [4.0, 8.0, 1.0]]], [[[4.0, 2.0, 7.0], [8.0, 3.0, 3.0], [4.0, 3.0, 4.0]], [[8.0, 3.0, 3.0], 
[5.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[6.0, 4.0, 7.0], [2.0, 7.0, 1.0], [4.0, 2.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 8.0, 8.0], [7.0, 8.0, 6.0], [7.0, 3.0, 1.0]]], [[[6.0, 6.0, 7.0], [8.0, 2.0, 6.0], [4.0, 
4.0, 6.0]]], [[[4.0, 2.0, 3.0], [6.0, 7.0, 2.0], [3.0, 2.0, 8.0]]], [[[2.0, 2.0, 3.0], [2.0, 3.0, 6.0], [1.0, 8.0, 
8.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 3.0], [7.0, 1.0, 5.0], [6.0, 6.0, 5.0]]], [[[4.0, 3.0, 8.0], [3.0, 8.0, 1.0], 
[3.0, 3.0, 5.0]]], [[[6.0, 7.0, 2.0], [6.0, 5.0, 3.0], [5.0, 2.0, 5.0]]], [[[5.0, 6.0, 5.0], [1.0, 1.0, 7.0], [5.0, 3.0, 
6.0]]]])
      arg[:,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 1.0], [2.0, 1.0, 3.0], [8.0, 2.0, 3.0]], [[5.0, 3.0, 5.0], [4.0, 8.0, 6.0], [6.0, 
1.0, 2.0]], [[2.0, 8.0, 8.0], [7.0, 8.0, 6.0], [7.0, 3.0, 1.0]]], [[[1.0, 3.0, 7.0], [8.0, 7.0, 2.0], [8.0, 2.0, 8.0]], [[5.0, 
8.0, 3.0], [2.0, 4.0, 5.0], [1.0, 2.0, 5.0]], [[6.0, 6.0, 7.0], [8.0, 2.0, 6.0], [4.0, 4.0, 6.0]]], [[[5.0, 7.0, 1.0], [2.0, 
4.0, 2.0], [4.0, 2.0, 2.0]], [[3.0, 4.0, 4.0], [5.0, 8.0, 4.0], [1.0, 2.0, 1.0]], [[4.0, 2.0, 3.0], [6.0, 7.0, 2.0], [3.0, 2.0, 
8.0]]], [[[8.0, 3.0, 2.0], [3.0, 5.0, 3.0], [3.0, 3.0, 2.0]], [[2.0, 2.0, 2.0], [3.0, 1.0, 5.0], [3.0, 2.0, 6.0]], [[2.0, 2.0, 
3.0], [2.0, 3.0, 6.0], [1.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 3.0], [1.0, 7.0, 7.0], [6.0, 8.0, 1.0]], 
[[5.0, 1.0, 7.0], [1.0, 7.0, 3.0], [8.0, 3.0, 6.0]], [[6.0, 5.0, 3.0], [7.0, 1.0, 5.0], [6.0, 6.0, 5.0]]], [[[3.0, 5.0, 4.0], 
[2.0, 1.0, 4.0], [5.0, 5.0, 1.0]], [[5.0, 3.0, 4.0], [2.0, 4.0, 3.0], [1.0, 8.0, 8.0]], [[4.0, 3.0, 8.0], [3.0, 8.0, 1.0], 
[3.0, 3.0, 5.0]]], [[[7.0, 4.0, 1.0], [7.0, 1.0, 5.0], [5.0, 2.0, 4.0]], [[8.0, 3.0, 2.0], [2.0, 5.0, 6.0], [7.0, 8.0, 3.0]], 
[[6.0, 7.0, 2.0], [6.0, 5.0, 3.0], [5.0, 2.0, 5.0]]], [[[4.0, 2.0, 7.0], [8.0, 3.0, 3.0], [4.0, 3.0, 4.0]], [[8.0, 3.0, 3.0], 
[5.0, 5.0, 5.0], [4.0, 1.0, 2.0]], [[5.0, 6.0, 5.0], [1.0, 1.0, 7.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 2.0], [8.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 1.0], [7.0, 
8.0, 1.0]], [[7.0, 6.0, 7.0], [6.0, 4.0, 1.0], [1.0, 4.0, 3.0]]], [[[4.0, 7.0, 6.0], [5.0, 5.0, 8.0], [5.0, 8.0, 2.0]], [[6.0, 
7.0, 3.0], [8.0, 6.0, 5.0], [2.0, 7.0, 1.0]], [[6.0, 5.0, 8.0], [4.0, 1.0, 4.0], [2.0, 7.0, 1.0]]], [[[6.0, 7.0, 4.0], [6.0, 
7.0, 6.0], [2.0, 7.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 4.0, 2.0], [1.0, 7.0, 4.0]], [[4.0, 6.0, 8.0], [6.0, 2.0, 5.0], [4.0, 8.0, 
1.0]]], [[[3.0, 8.0, 7.0], [1.0, 4.0, 1.0], [7.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [8.0, 4.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 
1.0], [8.0, 8.0, 3.0], [2.0, 8.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 1.0], [7.0, 3.0, 8.0], [1.0, 7.0, 4.0]], 
[[7.0, 8.0, 7.0], [4.0, 4.0, 7.0], [1.0, 7.0, 8.0]], [[3.0, 2.0, 3.0], [7.0, 2.0, 7.0], [8.0, 4.0, 3.0]]], [[[4.0, 1.0, 3.0], 
[3.0, 5.0, 8.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 3.0], [1.0, 2.0, 8.0], [4.0, 1.0, 5.0]], [[5.0, 1.0, 1.0], [4.0, 2.0, 4.0], 
[4.0, 6.0, 4.0]]], [[[1.0, 6.0, 4.0], [1.0, 4.0, 7.0], [7.0, 1.0, 2.0]], [[1.0, 3.0, 2.0], [7.0, 3.0, 2.0], [4.0, 8.0, 1.0]], 
[[2.0, 8.0, 4.0], [4.0, 3.0, 2.0], [6.0, 5.0, 6.0]]], [[[3.0, 5.0, 7.0], [3.0, 6.0, 3.0], [4.0, 5.0, 5.0]], [[6.0, 5.0, 1.0], 
[2.0, 7.0, 6.0], [2.0, 7.0, 5.0]], [[6.0, 8.0, 3.0], [6.0, 5.0, 7.0], [2.0, 7.0, 5.0]]]])
      val=numpy.array([[[[3.0, 1.0, 3.0]]], [[[1.0, 1.0, 1.0]]], [[[5.0, 8.0, 7.0]]]])
      arg[:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 2.0], [8.0, 2.0, 4.0], [4.0, 4.0, 7.0]], [[3.0, 5.0, 1.0], [5.0, 2.0, 1.0], [3.0, 
1.0, 3.0]], [[7.0, 6.0, 7.0], [6.0, 4.0, 1.0], [1.0, 4.0, 3.0]]], [[[4.0, 7.0, 6.0], [5.0, 5.0, 8.0], [5.0, 8.0, 2.0]], [[6.0, 
7.0, 3.0], [8.0, 6.0, 5.0], [1.0, 1.0, 1.0]], [[6.0, 5.0, 8.0], [4.0, 1.0, 4.0], [2.0, 7.0, 1.0]]], [[[6.0, 7.0, 4.0], [6.0, 
7.0, 6.0], [2.0, 7.0, 4.0]], [[3.0, 3.0, 1.0], [5.0, 4.0, 2.0], [5.0, 8.0, 7.0]], [[4.0, 6.0, 8.0], [6.0, 2.0, 5.0], [4.0, 8.0, 
1.0]]], [[[3.0, 8.0, 7.0], [1.0, 4.0, 1.0], [7.0, 3.0, 1.0]], [[5.0, 5.0, 7.0], [8.0, 4.0, 3.0], [4.0, 5.0, 7.0]], [[2.0, 4.0, 
1.0], [8.0, 8.0, 3.0], [2.0, 8.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 1.0], [7.0, 3.0, 8.0], [1.0, 7.0, 4.0]], 
[[7.0, 8.0, 7.0], [4.0, 4.0, 7.0], [3.0, 1.0, 3.0]], [[3.0, 2.0, 3.0], [7.0, 2.0, 7.0], [8.0, 4.0, 3.0]]], [[[4.0, 1.0, 3.0], 
[3.0, 5.0, 8.0], [2.0, 4.0, 1.0]], [[7.0, 7.0, 3.0], [1.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[5.0, 1.0, 1.0], [4.0, 2.0, 4.0], 
[4.0, 6.0, 4.0]]], [[[1.0, 6.0, 4.0], [1.0, 4.0, 7.0], [7.0, 1.0, 2.0]], [[1.0, 3.0, 2.0], [7.0, 3.0, 2.0], [5.0, 8.0, 7.0]], 
[[2.0, 8.0, 4.0], [4.0, 3.0, 2.0], [6.0, 5.0, 6.0]]], [[[3.0, 5.0, 7.0], [3.0, 6.0, 3.0], [4.0, 5.0, 5.0]], [[6.0, 5.0, 1.0], 
[2.0, 7.0, 6.0], [2.0, 7.0, 5.0]], [[6.0, 8.0, 3.0], [6.0, 5.0, 7.0], [2.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 7.0], [1.0, 2.0, 3.0], [2.0, 6.0, 6.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 7.0], [1.0, 
1.0, 4.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 4.0], [7.0, 1.0, 6.0]]], [[[1.0, 2.0, 7.0], [6.0, 4.0, 2.0], [7.0, 3.0, 7.0]], [[5.0, 
6.0, 4.0], [7.0, 6.0, 4.0], [6.0, 5.0, 2.0]], [[8.0, 4.0, 8.0], [2.0, 4.0, 8.0], [6.0, 6.0, 1.0]]], [[[2.0, 5.0, 8.0], [6.0, 
5.0, 8.0], [2.0, 6.0, 5.0]], [[4.0, 4.0, 6.0], [6.0, 1.0, 7.0], [4.0, 7.0, 1.0]], [[4.0, 5.0, 8.0], [4.0, 2.0, 5.0], [2.0, 8.0, 
7.0]]], [[[6.0, 5.0, 5.0], [3.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[7.0, 4.0, 3.0], [1.0, 1.0, 2.0], [2.0, 7.0, 1.0]], [[2.0, 7.0, 
6.0], [8.0, 3.0, 4.0], [3.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 4.0], [2.0, 5.0, 4.0], [1.0, 1.0, 7.0]], 
[[6.0, 3.0, 5.0], [5.0, 4.0, 6.0], [2.0, 1.0, 6.0]], [[3.0, 2.0, 2.0], [6.0, 6.0, 3.0], [4.0, 8.0, 8.0]]], [[[3.0, 7.0, 6.0], 
[8.0, 6.0, 1.0], [2.0, 1.0, 7.0]], [[1.0, 8.0, 7.0], [6.0, 6.0, 5.0], [1.0, 1.0, 4.0]], [[7.0, 4.0, 1.0], [7.0, 6.0, 5.0], 
[7.0, 5.0, 2.0]]], [[[7.0, 1.0, 7.0], [6.0, 5.0, 3.0], [5.0, 5.0, 7.0]], [[8.0, 3.0, 5.0], [7.0, 7.0, 5.0], [7.0, 7.0, 5.0]], 
[[7.0, 3.0, 5.0], [7.0, 5.0, 2.0], [4.0, 4.0, 1.0]]], [[[6.0, 6.0, 8.0], [3.0, 1.0, 5.0], [6.0, 1.0, 4.0]], [[6.0, 8.0, 7.0], 
[5.0, 5.0, 1.0], [7.0, 5.0, 5.0]], [[2.0, 4.0, 7.0], [2.0, 4.0, 4.0], [6.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[[4.0, 2.0, 4.0]]], [[[5.0, 7.0, 1.0]]], [[[5.0, 6.0, 2.0]]]]),self.functionspace)
      arg[:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 7.0], [1.0, 2.0, 3.0], [2.0, 6.0, 6.0]], [[5.0, 6.0, 4.0], [6.0, 1.0, 7.0], [4.0, 
2.0, 4.0]], [[2.0, 5.0, 4.0], [6.0, 8.0, 4.0], [7.0, 1.0, 6.0]]], [[[1.0, 2.0, 7.0], [6.0, 4.0, 2.0], [7.0, 3.0, 7.0]], [[5.0, 
6.0, 4.0], [7.0, 6.0, 4.0], [5.0, 7.0, 1.0]], [[8.0, 4.0, 8.0], [2.0, 4.0, 8.0], [6.0, 6.0, 1.0]]], [[[2.0, 5.0, 8.0], [6.0, 
5.0, 8.0], [2.0, 6.0, 5.0]], [[4.0, 4.0, 6.0], [6.0, 1.0, 7.0], [5.0, 6.0, 2.0]], [[4.0, 5.0, 8.0], [4.0, 2.0, 5.0], [2.0, 8.0, 
7.0]]], [[[6.0, 5.0, 5.0], [3.0, 8.0, 6.0], [7.0, 5.0, 7.0]], [[7.0, 4.0, 3.0], [1.0, 1.0, 2.0], [2.0, 7.0, 1.0]], [[2.0, 7.0, 
6.0], [8.0, 3.0, 4.0], [3.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 4.0], [2.0, 5.0, 4.0], [1.0, 1.0, 7.0]], 
[[6.0, 3.0, 5.0], [5.0, 4.0, 6.0], [4.0, 2.0, 4.0]], [[3.0, 2.0, 2.0], [6.0, 6.0, 3.0], [4.0, 8.0, 8.0]]], [[[3.0, 7.0, 6.0], 
[8.0, 6.0, 1.0], [2.0, 1.0, 7.0]], [[1.0, 8.0, 7.0], [6.0, 6.0, 5.0], [5.0, 7.0, 1.0]], [[7.0, 4.0, 1.0], [7.0, 6.0, 5.0], 
[7.0, 5.0, 2.0]]], [[[7.0, 1.0, 7.0], [6.0, 5.0, 3.0], [5.0, 5.0, 7.0]], [[8.0, 3.0, 5.0], [7.0, 7.0, 5.0], [5.0, 6.0, 2.0]], 
[[7.0, 3.0, 5.0], [7.0, 5.0, 2.0], [4.0, 4.0, 1.0]]], [[[6.0, 6.0, 8.0], [3.0, 1.0, 5.0], [6.0, 1.0, 4.0]], [[6.0, 8.0, 7.0], 
[5.0, 5.0, 1.0], [7.0, 5.0, 5.0]], [[2.0, 4.0, 7.0], [2.0, 4.0, 4.0], [6.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 4.0, 5.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 4.0], [7.0, 4.0, 5.0], [4.0, 
4.0, 4.0]], [[3.0, 2.0, 3.0], [6.0, 6.0, 3.0], [7.0, 2.0, 7.0]]], [[[2.0, 7.0, 8.0], [5.0, 5.0, 4.0], [3.0, 5.0, 1.0]], [[1.0, 
1.0, 2.0], [1.0, 4.0, 6.0], [8.0, 1.0, 6.0]], [[8.0, 8.0, 1.0], [3.0, 5.0, 1.0], [1.0, 2.0, 2.0]]], [[[7.0, 5.0, 5.0], [7.0, 
1.0, 3.0], [8.0, 5.0, 4.0]], [[6.0, 5.0, 6.0], [7.0, 4.0, 7.0], [7.0, 1.0, 1.0]], [[6.0, 4.0, 7.0], [3.0, 6.0, 3.0], [1.0, 3.0, 
2.0]]], [[[4.0, 4.0, 7.0], [4.0, 4.0, 8.0], [8.0, 6.0, 6.0]], [[4.0, 1.0, 8.0], [4.0, 1.0, 5.0], [4.0, 7.0, 5.0]], [[3.0, 7.0, 
8.0], [2.0, 6.0, 6.0], [2.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 4.0], [7.0, 2.0, 7.0], [3.0, 2.0, 7.0]], 
[[4.0, 5.0, 2.0], [5.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[7.0, 3.0, 5.0], [4.0, 4.0, 5.0], [8.0, 4.0, 4.0]]], [[[6.0, 4.0, 7.0], 
[1.0, 2.0, 1.0], [5.0, 7.0, 1.0]], [[3.0, 4.0, 3.0], [4.0, 1.0, 5.0], [2.0, 6.0, 1.0]], [[4.0, 3.0, 4.0], [1.0, 6.0, 1.0], 
[8.0, 8.0, 5.0]]], [[[7.0, 4.0, 8.0], [5.0, 6.0, 7.0], [7.0, 1.0, 1.0]], [[1.0, 5.0, 3.0], [5.0, 8.0, 4.0], [6.0, 1.0, 6.0]], 
[[8.0, 2.0, 6.0], [1.0, 3.0, 4.0], [1.0, 5.0, 3.0]]], [[[2.0, 6.0, 8.0], [2.0, 6.0, 2.0], [1.0, 8.0, 3.0]], [[4.0, 4.0, 3.0], 
[2.0, 1.0, 4.0], [2.0, 2.0, 8.0]], [[2.0, 3.0, 7.0], [1.0, 4.0, 6.0], [1.0, 5.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[1.0, 1.0, 8.0]]], [[[2.0, 3.0, 3.0]]], [[[2.0, 3.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[3.0, 7.0, 4.0]]], [[[4.0, 3.0, 2.0]]], [[[3.0, 7.0, 4.0]]]])
      arg[:3,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 8.0, 7.0], [8.0, 4.0, 5.0], [6.0, 1.0, 4.0]], [[1.0, 5.0, 4.0], [7.0, 4.0, 5.0], [1.0, 
1.0, 8.0]], [[3.0, 2.0, 3.0], [6.0, 6.0, 3.0], [7.0, 2.0, 7.0]]], [[[2.0, 7.0, 8.0], [5.0, 5.0, 4.0], [3.0, 5.0, 1.0]], [[1.0, 
1.0, 2.0], [1.0, 4.0, 6.0], [2.0, 3.0, 3.0]], [[8.0, 8.0, 1.0], [3.0, 5.0, 1.0], [1.0, 2.0, 2.0]]], [[[7.0, 5.0, 5.0], [7.0, 
1.0, 3.0], [8.0, 5.0, 4.0]], [[6.0, 5.0, 6.0], [7.0, 4.0, 7.0], [2.0, 3.0, 3.0]], [[6.0, 4.0, 7.0], [3.0, 6.0, 3.0], [1.0, 3.0, 
2.0]]], [[[4.0, 4.0, 7.0], [4.0, 4.0, 8.0], [8.0, 6.0, 6.0]], [[4.0, 1.0, 8.0], [4.0, 1.0, 5.0], [4.0, 7.0, 5.0]], [[3.0, 7.0, 
8.0], [2.0, 6.0, 6.0], [2.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 4.0], [7.0, 2.0, 7.0], [3.0, 2.0, 7.0]], 
[[4.0, 5.0, 2.0], [5.0, 7.0, 4.0], [3.0, 7.0, 4.0]], [[7.0, 3.0, 5.0], [4.0, 4.0, 5.0], [8.0, 4.0, 4.0]]], [[[6.0, 4.0, 7.0], 
[1.0, 2.0, 1.0], [5.0, 7.0, 1.0]], [[3.0, 4.0, 3.0], [4.0, 1.0, 5.0], [4.0, 3.0, 2.0]], [[4.0, 3.0, 4.0], [1.0, 6.0, 1.0], 
[8.0, 8.0, 5.0]]], [[[7.0, 4.0, 8.0], [5.0, 6.0, 7.0], [7.0, 1.0, 1.0]], [[1.0, 5.0, 3.0], [5.0, 8.0, 4.0], [3.0, 7.0, 4.0]], 
[[8.0, 2.0, 6.0], [1.0, 3.0, 4.0], [1.0, 5.0, 3.0]]], [[[2.0, 6.0, 8.0], [2.0, 6.0, 2.0], [1.0, 8.0, 3.0]], [[4.0, 4.0, 3.0], 
[2.0, 1.0, 4.0], [2.0, 2.0, 8.0]], [[2.0, 3.0, 7.0], [1.0, 4.0, 6.0], [1.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 1.0], [4.0, 2.0, 7.0], [4.0, 8.0, 4.0]], [[3.0, 4.0, 6.0], [8.0, 4.0, 1.0], [5.0, 
1.0, 7.0]], [[6.0, 2.0, 7.0], [5.0, 5.0, 3.0], [6.0, 8.0, 7.0]]], [[[3.0, 1.0, 7.0], [4.0, 6.0, 3.0], [3.0, 4.0, 5.0]], [[8.0, 
2.0, 3.0], [5.0, 4.0, 5.0], [7.0, 8.0, 4.0]], [[4.0, 3.0, 4.0], [4.0, 2.0, 8.0], [4.0, 7.0, 3.0]]], [[[2.0, 8.0, 5.0], [5.0, 
7.0, 8.0], [5.0, 2.0, 4.0]], [[2.0, 2.0, 1.0], [7.0, 5.0, 8.0], [4.0, 8.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 5.0, 4.0], [2.0, 5.0, 
3.0]]], [[[5.0, 8.0, 2.0], [2.0, 7.0, 7.0], [8.0, 8.0, 7.0]], [[3.0, 6.0, 5.0], [5.0, 6.0, 2.0], [3.0, 4.0, 5.0]], [[7.0, 6.0, 
8.0], [1.0, 6.0, 3.0], [6.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 8.0], [3.0, 3.0, 7.0], [5.0, 7.0, 7.0]], 
[[1.0, 7.0, 1.0], [3.0, 8.0, 7.0], [1.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [7.0, 3.0, 1.0], [1.0, 8.0, 1.0]]], [[[2.0, 2.0, 3.0], 
[7.0, 3.0, 2.0], [1.0, 1.0, 4.0]], [[4.0, 2.0, 2.0], [4.0, 6.0, 3.0], [2.0, 6.0, 5.0]], [[7.0, 4.0, 5.0], [5.0, 8.0, 8.0], 
[4.0, 6.0, 3.0]]], [[[2.0, 6.0, 8.0], [2.0, 4.0, 1.0], [1.0, 3.0, 2.0]], [[2.0, 7.0, 1.0], [5.0, 8.0, 8.0], [3.0, 8.0, 8.0]], 
[[1.0, 4.0, 1.0], [2.0, 2.0, 8.0], [5.0, 7.0, 4.0]]], [[[2.0, 4.0, 5.0], [8.0, 3.0, 6.0], [5.0, 3.0, 2.0]], [[7.0, 8.0, 1.0], 
[8.0, 5.0, 1.0], [5.0, 3.0, 2.0]], [[3.0, 3.0, 6.0], [1.0, 4.0, 1.0], [3.0, 8.0, 1.0]]]])
      val=numpy.array([[[[5.0, 4.0, 1.0]], [[3.0, 5.0, 6.0]], [[2.0, 2.0, 6.0]]], [[[4.0, 4.0, 3.0]], [[7.0, 7.0, 7.0]], [[8.0, 
3.0, 7.0]]], [[[2.0, 6.0, 2.0]], [[2.0, 2.0, 2.0]], [[2.0, 1.0, 5.0]]], [[[5.0, 6.0, 8.0]], [[4.0, 1.0, 1.0]], [[4.0, 4.0, 
6.0]]]])
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 1.0], [4.0, 2.0, 7.0], [5.0, 4.0, 1.0]], [[3.0, 4.0, 6.0], [8.0, 4.0, 1.0], [3.0, 
5.0, 6.0]], [[6.0, 2.0, 7.0], [5.0, 5.0, 3.0], [2.0, 2.0, 6.0]]], [[[3.0, 1.0, 7.0], [4.0, 6.0, 3.0], [4.0, 4.0, 3.0]], [[8.0, 
2.0, 3.0], [5.0, 4.0, 5.0], [7.0, 7.0, 7.0]], [[4.0, 3.0, 4.0], [4.0, 2.0, 8.0], [8.0, 3.0, 7.0]]], [[[2.0, 8.0, 5.0], [5.0, 
7.0, 8.0], [2.0, 6.0, 2.0]], [[2.0, 2.0, 1.0], [7.0, 5.0, 8.0], [2.0, 2.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 5.0, 4.0], [2.0, 1.0, 
5.0]]], [[[5.0, 8.0, 2.0], [2.0, 7.0, 7.0], [5.0, 6.0, 8.0]], [[3.0, 6.0, 5.0], [5.0, 6.0, 2.0], [4.0, 1.0, 1.0]], [[7.0, 6.0, 
8.0], [1.0, 6.0, 3.0], [4.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 8.0], [3.0, 3.0, 7.0], [5.0, 4.0, 1.0]], 
[[1.0, 7.0, 1.0], [3.0, 8.0, 7.0], [3.0, 5.0, 6.0]], [[2.0, 6.0, 8.0], [7.0, 3.0, 1.0], [2.0, 2.0, 6.0]]], [[[2.0, 2.0, 3.0], 
[7.0, 3.0, 2.0], [4.0, 4.0, 3.0]], [[4.0, 2.0, 2.0], [4.0, 6.0, 3.0], [7.0, 7.0, 7.0]], [[7.0, 4.0, 5.0], [5.0, 8.0, 8.0], 
[8.0, 3.0, 7.0]]], [[[2.0, 6.0, 8.0], [2.0, 4.0, 1.0], [2.0, 6.0, 2.0]], [[2.0, 7.0, 1.0], [5.0, 8.0, 8.0], [2.0, 2.0, 2.0]], 
[[1.0, 4.0, 1.0], [2.0, 2.0, 8.0], [2.0, 1.0, 5.0]]], [[[2.0, 4.0, 5.0], [8.0, 3.0, 6.0], [5.0, 6.0, 8.0]], [[7.0, 8.0, 1.0], 
[8.0, 5.0, 1.0], [4.0, 1.0, 1.0]], [[3.0, 3.0, 6.0], [1.0, 4.0, 1.0], [4.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 8.0], [2.0, 3.0, 7.0], [2.0, 4.0, 3.0]], [[4.0, 7.0, 3.0], [1.0, 6.0, 6.0], [6.0, 
4.0, 6.0]], [[4.0, 2.0, 8.0], [5.0, 5.0, 7.0], [5.0, 6.0, 3.0]]], [[[5.0, 6.0, 3.0], [2.0, 8.0, 7.0], [2.0, 1.0, 8.0]], [[1.0, 
4.0, 4.0], [1.0, 7.0, 8.0], [2.0, 7.0, 5.0]], [[5.0, 5.0, 4.0], [8.0, 5.0, 8.0], [6.0, 7.0, 4.0]]], [[[3.0, 5.0, 5.0], [8.0, 
1.0, 8.0], [6.0, 3.0, 5.0]], [[2.0, 1.0, 8.0], [7.0, 4.0, 8.0], [6.0, 7.0, 6.0]], [[6.0, 4.0, 7.0], [7.0, 1.0, 6.0], [2.0, 7.0, 
2.0]]], [[[3.0, 4.0, 3.0], [8.0, 1.0, 3.0], [5.0, 3.0, 6.0]], [[4.0, 2.0, 2.0], [2.0, 5.0, 5.0], [6.0, 4.0, 7.0]], [[3.0, 5.0, 
3.0], [7.0, 3.0, 5.0], [1.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 3.0], [3.0, 2.0, 1.0], [2.0, 6.0, 2.0]], 
[[4.0, 7.0, 6.0], [1.0, 2.0, 4.0], [6.0, 1.0, 8.0]], [[3.0, 8.0, 7.0], [4.0, 7.0, 7.0], [3.0, 6.0, 3.0]]], [[[6.0, 2.0, 4.0], 
[3.0, 1.0, 1.0], [2.0, 8.0, 2.0]], [[1.0, 1.0, 2.0], [5.0, 6.0, 3.0], [7.0, 8.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 5.0, 5.0], 
[8.0, 4.0, 2.0]]], [[[6.0, 7.0, 3.0], [4.0, 5.0, 1.0], [4.0, 3.0, 4.0]], [[3.0, 4.0, 2.0], [6.0, 1.0, 8.0], [6.0, 1.0, 4.0]], 
[[8.0, 5.0, 6.0], [4.0, 8.0, 5.0], [1.0, 4.0, 8.0]]], [[[3.0, 8.0, 3.0], [7.0, 1.0, 7.0], [3.0, 4.0, 8.0]], [[5.0, 5.0, 5.0], 
[5.0, 8.0, 2.0], [5.0, 1.0, 1.0]], [[8.0, 5.0, 4.0], [1.0, 5.0, 4.0], [1.0, 6.0, 1.0]]]])
      val=Data(numpy.array([[[[2.0, 6.0, 4.0]], [[7.0, 2.0, 2.0]], [[8.0, 6.0, 1.0]]], [[[7.0, 7.0, 5.0]], [[3.0, 4.0, 2.0]], 
[[4.0, 8.0, 2.0]]], [[[8.0, 1.0, 3.0]], [[4.0, 3.0, 7.0]], [[5.0, 7.0, 7.0]]], [[[5.0, 3.0, 5.0]], [[7.0, 7.0, 6.0]], [[4.0, 
7.0, 3.0]]]]),self.functionspace)
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 8.0], [2.0, 3.0, 7.0], [2.0, 6.0, 4.0]], [[4.0, 7.0, 3.0], [1.0, 6.0, 6.0], [7.0, 
2.0, 2.0]], [[4.0, 2.0, 8.0], [5.0, 5.0, 7.0], [8.0, 6.0, 1.0]]], [[[5.0, 6.0, 3.0], [2.0, 8.0, 7.0], [7.0, 7.0, 5.0]], [[1.0, 
4.0, 4.0], [1.0, 7.0, 8.0], [3.0, 4.0, 2.0]], [[5.0, 5.0, 4.0], [8.0, 5.0, 8.0], [4.0, 8.0, 2.0]]], [[[3.0, 5.0, 5.0], [8.0, 
1.0, 8.0], [8.0, 1.0, 3.0]], [[2.0, 1.0, 8.0], [7.0, 4.0, 8.0], [4.0, 3.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 1.0, 6.0], [5.0, 7.0, 
7.0]]], [[[3.0, 4.0, 3.0], [8.0, 1.0, 3.0], [5.0, 3.0, 5.0]], [[4.0, 2.0, 2.0], [2.0, 5.0, 5.0], [7.0, 7.0, 6.0]], [[3.0, 5.0, 
3.0], [7.0, 3.0, 5.0], [4.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 3.0], [3.0, 2.0, 1.0], [2.0, 6.0, 4.0]], 
[[4.0, 7.0, 6.0], [1.0, 2.0, 4.0], [7.0, 2.0, 2.0]], [[3.0, 8.0, 7.0], [4.0, 7.0, 7.0], [8.0, 6.0, 1.0]]], [[[6.0, 2.0, 4.0], 
[3.0, 1.0, 1.0], [7.0, 7.0, 5.0]], [[1.0, 1.0, 2.0], [5.0, 6.0, 3.0], [3.0, 4.0, 2.0]], [[1.0, 3.0, 2.0], [7.0, 5.0, 5.0], 
[4.0, 8.0, 2.0]]], [[[6.0, 7.0, 3.0], [4.0, 5.0, 1.0], [8.0, 1.0, 3.0]], [[3.0, 4.0, 2.0], [6.0, 1.0, 8.0], [4.0, 3.0, 7.0]], 
[[8.0, 5.0, 6.0], [4.0, 8.0, 5.0], [5.0, 7.0, 7.0]]], [[[3.0, 8.0, 3.0], [7.0, 1.0, 7.0], [5.0, 3.0, 5.0]], [[5.0, 5.0, 5.0], 
[5.0, 8.0, 2.0], [7.0, 7.0, 6.0]], [[8.0, 5.0, 4.0], [1.0, 5.0, 4.0], [4.0, 7.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 7.0], [6.0, 7.0, 1.0], [7.0, 5.0, 3.0]], [[3.0, 5.0, 7.0], [7.0, 3.0, 1.0], [4.0, 
8.0, 8.0]], [[8.0, 6.0, 1.0], [1.0, 1.0, 3.0], [6.0, 1.0, 4.0]]], [[[2.0, 2.0, 3.0], [2.0, 2.0, 6.0], [6.0, 6.0, 4.0]], [[5.0, 
6.0, 7.0], [7.0, 7.0, 3.0], [1.0, 3.0, 7.0]], [[5.0, 6.0, 3.0], [8.0, 2.0, 6.0], [6.0, 1.0, 5.0]]], [[[3.0, 3.0, 1.0], [8.0, 
6.0, 3.0], [5.0, 4.0, 1.0]], [[8.0, 3.0, 8.0], [1.0, 6.0, 5.0], [5.0, 8.0, 7.0]], [[1.0, 1.0, 8.0], [4.0, 4.0, 8.0], [3.0, 4.0, 
7.0]]], [[[1.0, 8.0, 6.0], [4.0, 8.0, 8.0], [3.0, 1.0, 3.0]], [[4.0, 5.0, 6.0], [5.0, 3.0, 8.0], [5.0, 4.0, 6.0]], [[5.0, 6.0, 
3.0], [7.0, 5.0, 5.0], [4.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 1.0], [7.0, 2.0, 1.0], [6.0, 4.0, 8.0]], 
[[3.0, 8.0, 4.0], [7.0, 3.0, 6.0], [8.0, 8.0, 5.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 1.0], [4.0, 3.0, 4.0]]], [[[4.0, 8.0, 8.0], 
[3.0, 7.0, 1.0], [6.0, 3.0, 3.0]], [[3.0, 1.0, 1.0], [3.0, 3.0, 3.0], [1.0, 4.0, 4.0]], [[5.0, 1.0, 8.0], [3.0, 7.0, 2.0], 
[4.0, 8.0, 4.0]]], [[[6.0, 4.0, 1.0], [3.0, 2.0, 6.0], [2.0, 4.0, 2.0]], [[8.0, 7.0, 7.0], [4.0, 8.0, 7.0], [1.0, 6.0, 2.0]], 
[[2.0, 1.0, 5.0], [4.0, 6.0, 1.0], [6.0, 1.0, 5.0]]], [[[4.0, 6.0, 1.0], [7.0, 8.0, 2.0], [6.0, 3.0, 4.0]], [[6.0, 7.0, 1.0], 
[7.0, 2.0, 3.0], [6.0, 5.0, 5.0]], [[4.0, 5.0, 4.0], [5.0, 3.0, 1.0], [4.0, 5.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[5.0, 5.0, 2.0]], [[5.0, 1.0, 4.0]], [[3.0, 2.0, 7.0]]], [[[1.0, 8.0, 1.0]], [[5.0, 4.0, 
8.0]], [[6.0, 5.0, 4.0]]], [[[4.0, 5.0, 1.0]], [[7.0, 5.0, 1.0]], [[8.0, 6.0, 6.0]]], [[[5.0, 1.0, 4.0]], [[7.0, 7.0, 8.0]], 
[[8.0, 8.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 3.0, 1.0]], [[1.0, 3.0, 8.0]], [[8.0, 4.0, 2.0]]], [[[7.0, 3.0, 3.0]], 
[[2.0, 2.0, 4.0]], [[3.0, 6.0, 3.0]]], [[[6.0, 2.0, 4.0]], [[6.0, 3.0, 1.0]], [[7.0, 3.0, 5.0]]], [[[1.0, 2.0, 7.0]], [[8.0, 
4.0, 8.0]], [[3.0, 2.0, 6.0]]]])
      arg[:,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 7.0], [6.0, 7.0, 1.0], [5.0, 5.0, 2.0]], [[3.0, 5.0, 7.0], [7.0, 3.0, 1.0], [5.0, 
1.0, 4.0]], [[8.0, 6.0, 1.0], [1.0, 1.0, 3.0], [3.0, 2.0, 7.0]]], [[[2.0, 2.0, 3.0], [2.0, 2.0, 6.0], [1.0, 8.0, 1.0]], [[5.0, 
6.0, 7.0], [7.0, 7.0, 3.0], [5.0, 4.0, 8.0]], [[5.0, 6.0, 3.0], [8.0, 2.0, 6.0], [6.0, 5.0, 4.0]]], [[[3.0, 3.0, 1.0], [8.0, 
6.0, 3.0], [4.0, 5.0, 1.0]], [[8.0, 3.0, 8.0], [1.0, 6.0, 5.0], [7.0, 5.0, 1.0]], [[1.0, 1.0, 8.0], [4.0, 4.0, 8.0], [8.0, 6.0, 
6.0]]], [[[1.0, 8.0, 6.0], [4.0, 8.0, 8.0], [5.0, 1.0, 4.0]], [[4.0, 5.0, 6.0], [5.0, 3.0, 8.0], [7.0, 7.0, 8.0]], [[5.0, 6.0, 
3.0], [7.0, 5.0, 5.0], [8.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 1.0], [7.0, 2.0, 1.0], [1.0, 3.0, 1.0]], 
[[3.0, 8.0, 4.0], [7.0, 3.0, 6.0], [1.0, 3.0, 8.0]], [[1.0, 3.0, 8.0], [7.0, 1.0, 1.0], [8.0, 4.0, 2.0]]], [[[4.0, 8.0, 8.0], 
[3.0, 7.0, 1.0], [7.0, 3.0, 3.0]], [[3.0, 1.0, 1.0], [3.0, 3.0, 3.0], [2.0, 2.0, 4.0]], [[5.0, 1.0, 8.0], [3.0, 7.0, 2.0], 
[3.0, 6.0, 3.0]]], [[[6.0, 4.0, 1.0], [3.0, 2.0, 6.0], [6.0, 2.0, 4.0]], [[8.0, 7.0, 7.0], [4.0, 8.0, 7.0], [6.0, 3.0, 1.0]], 
[[2.0, 1.0, 5.0], [4.0, 6.0, 1.0], [7.0, 3.0, 5.0]]], [[[4.0, 6.0, 1.0], [7.0, 8.0, 2.0], [1.0, 2.0, 7.0]], [[6.0, 7.0, 1.0], 
[7.0, 2.0, 3.0], [8.0, 4.0, 8.0]], [[4.0, 5.0, 4.0], [5.0, 3.0, 1.0], [3.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 8.0], [2.0, 8.0, 7.0], [8.0, 8.0, 2.0]], [[4.0, 7.0, 7.0], [3.0, 3.0, 3.0], [6.0, 
3.0, 8.0]], [[5.0, 5.0, 8.0], [6.0, 2.0, 7.0], [6.0, 2.0, 6.0]]], [[[5.0, 5.0, 3.0], [3.0, 7.0, 6.0], [2.0, 6.0, 4.0]], [[7.0, 
5.0, 7.0], [5.0, 4.0, 3.0], [7.0, 8.0, 3.0]], [[4.0, 6.0, 3.0], [4.0, 2.0, 4.0], [3.0, 7.0, 3.0]]], [[[5.0, 6.0, 4.0], [8.0, 
3.0, 6.0], [8.0, 2.0, 2.0]], [[5.0, 2.0, 3.0], [6.0, 5.0, 6.0], [2.0, 4.0, 8.0]], [[8.0, 1.0, 2.0], [5.0, 1.0, 1.0], [2.0, 5.0, 
8.0]]], [[[4.0, 2.0, 8.0], [2.0, 5.0, 5.0], [4.0, 8.0, 1.0]], [[2.0, 6.0, 7.0], [3.0, 2.0, 8.0], [6.0, 5.0, 7.0]], [[7.0, 6.0, 
5.0], [3.0, 5.0, 5.0], [7.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 6.0, 7.0], [8.0, 3.0, 5.0], [3.0, 6.0, 1.0]], 
[[5.0, 2.0, 7.0], [5.0, 3.0, 8.0], [6.0, 3.0, 8.0]], [[1.0, 1.0, 1.0], [3.0, 4.0, 1.0], [8.0, 2.0, 2.0]]], [[[5.0, 7.0, 2.0], 
[8.0, 4.0, 3.0], [3.0, 8.0, 4.0]], [[2.0, 7.0, 1.0], [8.0, 2.0, 5.0], [1.0, 8.0, 2.0]], [[6.0, 5.0, 3.0], [3.0, 8.0, 4.0], 
[5.0, 3.0, 2.0]]], [[[3.0, 2.0, 5.0], [1.0, 8.0, 6.0], [1.0, 5.0, 7.0]], [[5.0, 4.0, 7.0], [4.0, 3.0, 8.0], [1.0, 3.0, 4.0]], 
[[6.0, 8.0, 6.0], [2.0, 1.0, 4.0], [8.0, 7.0, 8.0]]], [[[1.0, 5.0, 3.0], [3.0, 3.0, 6.0], [8.0, 7.0, 1.0]], [[6.0, 2.0, 2.0], 
[7.0, 4.0, 7.0], [8.0, 3.0, 6.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 7.0], [3.0, 6.0, 6.0]]]])
      val=numpy.array([[[[4.0, 2.0, 3.0]], [[8.0, 8.0, 5.0]], [[1.0, 1.0, 7.0]]], [[[4.0, 5.0, 2.0]], [[6.0, 1.0, 5.0]], [[3.0, 
4.0, 7.0]]], [[[5.0, 2.0, 4.0]], [[1.0, 5.0, 8.0]], [[1.0, 7.0, 3.0]]]])
      arg[:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 8.0], [2.0, 8.0, 7.0], [4.0, 2.0, 3.0]], [[4.0, 7.0, 7.0], [3.0, 3.0, 3.0], [8.0, 
8.0, 5.0]], [[5.0, 5.0, 8.0], [6.0, 2.0, 7.0], [1.0, 1.0, 7.0]]], [[[5.0, 5.0, 3.0], [3.0, 7.0, 6.0], [4.0, 5.0, 2.0]], [[7.0, 
5.0, 7.0], [5.0, 4.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 6.0, 3.0], [4.0, 2.0, 4.0], [3.0, 4.0, 7.0]]], [[[5.0, 6.0, 4.0], [8.0, 
3.0, 6.0], [5.0, 2.0, 4.0]], [[5.0, 2.0, 3.0], [6.0, 5.0, 6.0], [1.0, 5.0, 8.0]], [[8.0, 1.0, 2.0], [5.0, 1.0, 1.0], [1.0, 7.0, 
3.0]]], [[[4.0, 2.0, 8.0], [2.0, 5.0, 5.0], [4.0, 8.0, 1.0]], [[2.0, 6.0, 7.0], [3.0, 2.0, 8.0], [6.0, 5.0, 7.0]], [[7.0, 6.0, 
5.0], [3.0, 5.0, 5.0], [7.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 6.0, 7.0], [8.0, 3.0, 5.0], [4.0, 2.0, 3.0]], 
[[5.0, 2.0, 7.0], [5.0, 3.0, 8.0], [8.0, 8.0, 5.0]], [[1.0, 1.0, 1.0], [3.0, 4.0, 1.0], [1.0, 1.0, 7.0]]], [[[5.0, 7.0, 2.0], 
[8.0, 4.0, 3.0], [4.0, 5.0, 2.0]], [[2.0, 7.0, 1.0], [8.0, 2.0, 5.0], [6.0, 1.0, 5.0]], [[6.0, 5.0, 3.0], [3.0, 8.0, 4.0], 
[3.0, 4.0, 7.0]]], [[[3.0, 2.0, 5.0], [1.0, 8.0, 6.0], [5.0, 2.0, 4.0]], [[5.0, 4.0, 7.0], [4.0, 3.0, 8.0], [1.0, 5.0, 8.0]], 
[[6.0, 8.0, 6.0], [2.0, 1.0, 4.0], [1.0, 7.0, 3.0]]], [[[1.0, 5.0, 3.0], [3.0, 3.0, 6.0], [8.0, 7.0, 1.0]], [[6.0, 2.0, 2.0], 
[7.0, 4.0, 7.0], [8.0, 3.0, 6.0]], [[1.0, 1.0, 6.0], [7.0, 5.0, 7.0], [3.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 7.0], [1.0, 7.0, 2.0], [5.0, 8.0, 3.0]], [[3.0, 3.0, 4.0], [2.0, 1.0, 7.0], [1.0, 
3.0, 1.0]], [[2.0, 3.0, 2.0], [6.0, 4.0, 8.0], [1.0, 2.0, 2.0]]], [[[2.0, 7.0, 5.0], [5.0, 4.0, 5.0], [6.0, 6.0, 2.0]], [[6.0, 
2.0, 3.0], [8.0, 6.0, 8.0], [3.0, 3.0, 2.0]], [[1.0, 3.0, 4.0], [7.0, 1.0, 2.0], [6.0, 6.0, 4.0]]], [[[8.0, 5.0, 6.0], [6.0, 
1.0, 7.0], [7.0, 8.0, 1.0]], [[6.0, 7.0, 6.0], [6.0, 8.0, 8.0], [4.0, 8.0, 4.0]], [[5.0, 3.0, 6.0], [7.0, 6.0, 8.0], [6.0, 1.0, 
1.0]]], [[[5.0, 7.0, 6.0], [8.0, 7.0, 6.0], [1.0, 3.0, 1.0]], [[3.0, 1.0, 1.0], [1.0, 3.0, 6.0], [1.0, 1.0, 3.0]], [[1.0, 5.0, 
5.0], [8.0, 1.0, 2.0], [2.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 1.0], [6.0, 7.0, 1.0], [1.0, 2.0, 6.0]], 
[[6.0, 2.0, 4.0], [6.0, 3.0, 5.0], [7.0, 6.0, 6.0]], [[4.0, 2.0, 3.0], [4.0, 5.0, 4.0], [2.0, 3.0, 5.0]]], [[[7.0, 6.0, 3.0], 
[4.0, 3.0, 3.0], [7.0, 4.0, 2.0]], [[7.0, 5.0, 8.0], [6.0, 8.0, 1.0], [6.0, 8.0, 5.0]], [[2.0, 7.0, 5.0], [2.0, 8.0, 4.0], 
[8.0, 2.0, 3.0]]], [[[1.0, 3.0, 7.0], [5.0, 7.0, 5.0], [3.0, 5.0, 5.0]], [[2.0, 5.0, 1.0], [6.0, 3.0, 4.0], [4.0, 3.0, 5.0]], 
[[3.0, 3.0, 1.0], [2.0, 4.0, 4.0], [3.0, 1.0, 3.0]]], [[[3.0, 8.0, 5.0], [1.0, 2.0, 5.0], [7.0, 7.0, 1.0]], [[7.0, 7.0, 6.0], 
[4.0, 5.0, 1.0], [3.0, 8.0, 5.0]], [[4.0, 4.0, 3.0], [1.0, 2.0, 4.0], [8.0, 8.0, 5.0]]]])
      val=Data(numpy.array([[[[1.0, 4.0, 1.0]], [[7.0, 8.0, 3.0]], [[4.0, 6.0, 3.0]]], [[[1.0, 8.0, 6.0]], [[7.0, 6.0, 7.0]], 
[[8.0, 8.0, 3.0]]], [[[8.0, 5.0, 7.0]], [[6.0, 6.0, 4.0]], [[4.0, 7.0, 4.0]]]]),self.functionspace)
      arg[:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 7.0], [1.0, 7.0, 2.0], [1.0, 4.0, 1.0]], [[3.0, 3.0, 4.0], [2.0, 1.0, 7.0], [7.0, 
8.0, 3.0]], [[2.0, 3.0, 2.0], [6.0, 4.0, 8.0], [4.0, 6.0, 3.0]]], [[[2.0, 7.0, 5.0], [5.0, 4.0, 5.0], [1.0, 8.0, 6.0]], [[6.0, 
2.0, 3.0], [8.0, 6.0, 8.0], [7.0, 6.0, 7.0]], [[1.0, 3.0, 4.0], [7.0, 1.0, 2.0], [8.0, 8.0, 3.0]]], [[[8.0, 5.0, 6.0], [6.0, 
1.0, 7.0], [8.0, 5.0, 7.0]], [[6.0, 7.0, 6.0], [6.0, 8.0, 8.0], [6.0, 6.0, 4.0]], [[5.0, 3.0, 6.0], [7.0, 6.0, 8.0], [4.0, 7.0, 
4.0]]], [[[5.0, 7.0, 6.0], [8.0, 7.0, 6.0], [1.0, 3.0, 1.0]], [[3.0, 1.0, 1.0], [1.0, 3.0, 6.0], [1.0, 1.0, 3.0]], [[1.0, 5.0, 
5.0], [8.0, 1.0, 2.0], [2.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 1.0], [6.0, 7.0, 1.0], [1.0, 4.0, 1.0]], 
[[6.0, 2.0, 4.0], [6.0, 3.0, 5.0], [7.0, 8.0, 3.0]], [[4.0, 2.0, 3.0], [4.0, 5.0, 4.0], [4.0, 6.0, 3.0]]], [[[7.0, 6.0, 3.0], 
[4.0, 3.0, 3.0], [1.0, 8.0, 6.0]], [[7.0, 5.0, 8.0], [6.0, 8.0, 1.0], [7.0, 6.0, 7.0]], [[2.0, 7.0, 5.0], [2.0, 8.0, 4.0], 
[8.0, 8.0, 3.0]]], [[[1.0, 3.0, 7.0], [5.0, 7.0, 5.0], [8.0, 5.0, 7.0]], [[2.0, 5.0, 1.0], [6.0, 3.0, 4.0], [6.0, 6.0, 4.0]], 
[[3.0, 3.0, 1.0], [2.0, 4.0, 4.0], [4.0, 7.0, 4.0]]], [[[3.0, 8.0, 5.0], [1.0, 2.0, 5.0], [7.0, 7.0, 1.0]], [[7.0, 7.0, 6.0], 
[4.0, 5.0, 1.0], [3.0, 8.0, 5.0]], [[4.0, 4.0, 3.0], [1.0, 2.0, 4.0], [8.0, 8.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 2.0], [5.0, 2.0, 2.0], [8.0, 7.0, 8.0]], [[6.0, 4.0, 6.0], [8.0, 6.0, 2.0], [2.0, 
4.0, 5.0]], [[3.0, 3.0, 8.0], [3.0, 7.0, 1.0], [5.0, 8.0, 8.0]]], [[[7.0, 1.0, 4.0], [1.0, 7.0, 7.0], [7.0, 4.0, 4.0]], [[7.0, 
3.0, 5.0], [5.0, 6.0, 2.0], [7.0, 4.0, 3.0]], [[2.0, 2.0, 6.0], [2.0, 3.0, 6.0], [3.0, 6.0, 1.0]]], [[[7.0, 5.0, 2.0], [5.0, 
5.0, 3.0], [8.0, 4.0, 8.0]], [[2.0, 3.0, 2.0], [8.0, 8.0, 5.0], [3.0, 4.0, 4.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 3.0], [1.0, 6.0, 
8.0]]], [[[5.0, 1.0, 1.0], [4.0, 5.0, 2.0], [1.0, 2.0, 2.0]], [[6.0, 8.0, 7.0], [1.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[2.0, 3.0, 
7.0], [6.0, 1.0, 5.0], [4.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 5.0], [5.0, 1.0, 6.0], [2.0, 2.0, 1.0]], 
[[4.0, 1.0, 7.0], [1.0, 7.0, 6.0], [2.0, 1.0, 8.0]], [[5.0, 6.0, 3.0], [6.0, 3.0, 3.0], [8.0, 7.0, 6.0]]], [[[7.0, 4.0, 4.0], 
[8.0, 3.0, 2.0], [2.0, 2.0, 2.0]], [[8.0, 8.0, 5.0], [8.0, 6.0, 6.0], [5.0, 6.0, 6.0]], [[4.0, 3.0, 7.0], [8.0, 7.0, 8.0], 
[1.0, 7.0, 4.0]]], [[[1.0, 3.0, 6.0], [7.0, 4.0, 4.0], [1.0, 7.0, 1.0]], [[1.0, 6.0, 5.0], [1.0, 8.0, 5.0], [8.0, 1.0, 8.0]], 
[[2.0, 5.0, 8.0], [2.0, 1.0, 5.0], [4.0, 7.0, 5.0]]], [[[7.0, 2.0, 7.0], [8.0, 7.0, 5.0], [4.0, 2.0, 3.0]], [[5.0, 7.0, 7.0], 
[7.0, 4.0, 5.0], [1.0, 3.0, 6.0]], [[7.0, 8.0, 6.0], [2.0, 8.0, 2.0], [2.0, 4.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 4.0, 6.0]], [[8.0, 3.0, 7.0]], [[3.0, 3.0, 3.0]]], [[[5.0, 7.0, 4.0]], [[5.0, 3.0, 
3.0]], [[2.0, 1.0, 5.0]]], [[[1.0, 5.0, 8.0]], [[5.0, 2.0, 3.0]], [[6.0, 1.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 
1.0]], [[4.0, 3.0, 8.0]], [[2.0, 7.0, 1.0]]], [[[2.0, 1.0, 1.0]], [[2.0, 8.0, 3.0]], [[7.0, 2.0, 4.0]]], [[[5.0, 7.0, 3.0]], 
[[4.0, 5.0, 6.0]], [[7.0, 6.0, 1.0]]]])
      arg[:3,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 2.0], [5.0, 2.0, 2.0], [4.0, 4.0, 6.0]], [[6.0, 4.0, 6.0], [8.0, 6.0, 2.0], [8.0, 
3.0, 7.0]], [[3.0, 3.0, 8.0], [3.0, 7.0, 1.0], [3.0, 3.0, 3.0]]], [[[7.0, 1.0, 4.0], [1.0, 7.0, 7.0], [5.0, 7.0, 4.0]], [[7.0, 
3.0, 5.0], [5.0, 6.0, 2.0], [5.0, 3.0, 3.0]], [[2.0, 2.0, 6.0], [2.0, 3.0, 6.0], [2.0, 1.0, 5.0]]], [[[7.0, 5.0, 2.0], [5.0, 
5.0, 3.0], [1.0, 5.0, 8.0]], [[2.0, 3.0, 2.0], [8.0, 8.0, 5.0], [5.0, 2.0, 3.0]], [[6.0, 7.0, 2.0], [7.0, 5.0, 3.0], [6.0, 1.0, 
7.0]]], [[[5.0, 1.0, 1.0], [4.0, 5.0, 2.0], [1.0, 2.0, 2.0]], [[6.0, 8.0, 7.0], [1.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[2.0, 3.0, 
7.0], [6.0, 1.0, 5.0], [4.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [5.0, 1.0, 6.0], [5.0, 4.0, 1.0]], 
[[4.0, 1.0, 7.0], [1.0, 7.0, 6.0], [4.0, 3.0, 8.0]], [[5.0, 6.0, 3.0], [6.0, 3.0, 3.0], [2.0, 7.0, 1.0]]], [[[7.0, 4.0, 4.0], 
[8.0, 3.0, 2.0], [2.0, 1.0, 1.0]], [[8.0, 8.0, 5.0], [8.0, 6.0, 6.0], [2.0, 8.0, 3.0]], [[4.0, 3.0, 7.0], [8.0, 7.0, 8.0], 
[7.0, 2.0, 4.0]]], [[[1.0, 3.0, 6.0], [7.0, 4.0, 4.0], [5.0, 7.0, 3.0]], [[1.0, 6.0, 5.0], [1.0, 8.0, 5.0], [4.0, 5.0, 6.0]], 
[[2.0, 5.0, 8.0], [2.0, 1.0, 5.0], [7.0, 6.0, 1.0]]], [[[7.0, 2.0, 7.0], [8.0, 7.0, 5.0], [4.0, 2.0, 3.0]], [[5.0, 7.0, 7.0], 
[7.0, 4.0, 5.0], [1.0, 3.0, 6.0]], [[7.0, 8.0, 6.0], [2.0, 8.0, 2.0], [2.0, 4.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 4.0, 4.0], [7.0, 2.0, 2.0], [4.0, 7.0, 1.0]], [[6.0, 1.0, 5.0], [8.0, 4.0, 2.0], [6.0, 
3.0, 8.0]], [[2.0, 5.0, 4.0], [7.0, 8.0, 3.0], [8.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [7.0, 5.0, 5.0], [5.0, 2.0, 5.0]], [[3.0, 
8.0, 7.0], [5.0, 3.0, 3.0], [5.0, 7.0, 7.0]], [[6.0, 2.0, 4.0], [6.0, 8.0, 4.0], [3.0, 7.0, 6.0]]], [[[1.0, 6.0, 4.0], [7.0, 
7.0, 6.0], [5.0, 1.0, 5.0]], [[1.0, 7.0, 1.0], [8.0, 6.0, 8.0], [6.0, 1.0, 8.0]], [[8.0, 8.0, 1.0], [5.0, 4.0, 2.0], [3.0, 3.0, 
5.0]]], [[[6.0, 1.0, 6.0], [5.0, 2.0, 6.0], [6.0, 2.0, 8.0]], [[2.0, 3.0, 3.0], [6.0, 3.0, 1.0], [1.0, 5.0, 5.0]], [[7.0, 1.0, 
1.0], [3.0, 1.0, 7.0], [1.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 6.0], [2.0, 1.0, 8.0], [3.0, 1.0, 4.0]], 
[[7.0, 6.0, 2.0], [4.0, 5.0, 2.0], [3.0, 7.0, 8.0]], [[7.0, 4.0, 5.0], [6.0, 3.0, 7.0], [3.0, 3.0, 6.0]]], [[[7.0, 3.0, 4.0], 
[6.0, 2.0, 2.0], [2.0, 7.0, 6.0]], [[7.0, 1.0, 6.0], [5.0, 5.0, 3.0], [6.0, 1.0, 5.0]], [[4.0, 3.0, 3.0], [6.0, 4.0, 2.0], 
[7.0, 6.0, 4.0]]], [[[1.0, 2.0, 7.0], [8.0, 2.0, 4.0], [5.0, 3.0, 6.0]], [[4.0, 3.0, 5.0], [7.0, 8.0, 1.0], [5.0, 5.0, 7.0]], 
[[4.0, 3.0, 4.0], [4.0, 3.0, 6.0], [8.0, 8.0, 1.0]]], [[[2.0, 7.0, 5.0], [2.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[1.0, 1.0, 7.0], 
[7.0, 5.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 2.0, 8.0], [2.0, 1.0, 2.0], [8.0, 6.0, 3.0]]]])
      val=numpy.array([[[2.0, 5.0, 8.0], [4.0, 6.0, 3.0], [6.0, 4.0, 1.0]], [[4.0, 4.0, 5.0], [4.0, 7.0, 2.0], [8.0, 6.0, 
1.0]], [[3.0, 6.0, 6.0], [3.0, 2.0, 3.0], [1.0, 7.0, 1.0]]])
      arg[:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 4.0, 4.0], [2.0, 5.0, 8.0], [4.0, 7.0, 1.0]], [[6.0, 1.0, 5.0], [4.0, 6.0, 3.0], [6.0, 
3.0, 8.0]], [[2.0, 5.0, 4.0], [6.0, 4.0, 1.0], [8.0, 6.0, 4.0]]], [[[1.0, 6.0, 5.0], [4.0, 4.0, 5.0], [5.0, 2.0, 5.0]], [[3.0, 
8.0, 7.0], [4.0, 7.0, 2.0], [5.0, 7.0, 7.0]], [[6.0, 2.0, 4.0], [8.0, 6.0, 1.0], [3.0, 7.0, 6.0]]], [[[1.0, 6.0, 4.0], [3.0, 
6.0, 6.0], [5.0, 1.0, 5.0]], [[1.0, 7.0, 1.0], [3.0, 2.0, 3.0], [6.0, 1.0, 8.0]], [[8.0, 8.0, 1.0], [1.0, 7.0, 1.0], [3.0, 3.0, 
5.0]]], [[[6.0, 1.0, 6.0], [5.0, 2.0, 6.0], [6.0, 2.0, 8.0]], [[2.0, 3.0, 3.0], [6.0, 3.0, 1.0], [1.0, 5.0, 5.0]], [[7.0, 1.0, 
1.0], [3.0, 1.0, 7.0], [1.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 6.0], [2.0, 5.0, 8.0], [3.0, 1.0, 4.0]], 
[[7.0, 6.0, 2.0], [4.0, 6.0, 3.0], [3.0, 7.0, 8.0]], [[7.0, 4.0, 5.0], [6.0, 4.0, 1.0], [3.0, 3.0, 6.0]]], [[[7.0, 3.0, 4.0], 
[4.0, 4.0, 5.0], [2.0, 7.0, 6.0]], [[7.0, 1.0, 6.0], [4.0, 7.0, 2.0], [6.0, 1.0, 5.0]], [[4.0, 3.0, 3.0], [8.0, 6.0, 1.0], 
[7.0, 6.0, 4.0]]], [[[1.0, 2.0, 7.0], [3.0, 6.0, 6.0], [5.0, 3.0, 6.0]], [[4.0, 3.0, 5.0], [3.0, 2.0, 3.0], [5.0, 5.0, 7.0]], 
[[4.0, 3.0, 4.0], [1.0, 7.0, 1.0], [8.0, 8.0, 1.0]]], [[[2.0, 7.0, 5.0], [2.0, 7.0, 8.0], [1.0, 2.0, 1.0]], [[1.0, 1.0, 7.0], 
[7.0, 5.0, 5.0], [2.0, 7.0, 1.0]], [[4.0, 2.0, 8.0], [2.0, 1.0, 2.0], [8.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 1.0], [2.0, 3.0, 1.0], [2.0, 2.0, 8.0]], [[1.0, 4.0, 8.0], [8.0, 3.0, 4.0], [6.0, 
4.0, 5.0]], [[6.0, 5.0, 4.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]]], [[[5.0, 5.0, 5.0], [8.0, 2.0, 3.0], [5.0, 5.0, 8.0]], [[6.0, 
8.0, 4.0], [6.0, 4.0, 7.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 6.0], [5.0, 7.0, 8.0], [4.0, 4.0, 4.0]]], [[[2.0, 4.0, 6.0], [4.0, 
3.0, 2.0], [8.0, 4.0, 8.0]], [[3.0, 4.0, 6.0], [1.0, 4.0, 6.0], [7.0, 6.0, 1.0]], [[2.0, 1.0, 8.0], [8.0, 4.0, 8.0], [1.0, 1.0, 
3.0]]], [[[6.0, 7.0, 4.0], [1.0, 7.0, 5.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 8.0], [5.0, 1.0, 5.0], [6.0, 4.0, 4.0]], [[3.0, 5.0, 
4.0], [4.0, 4.0, 7.0], [3.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 2.0], [8.0, 4.0, 8.0], [2.0, 7.0, 3.0]], 
[[5.0, 6.0, 3.0], [6.0, 2.0, 8.0], [5.0, 8.0, 4.0]], [[8.0, 5.0, 1.0], [5.0, 5.0, 4.0], [1.0, 2.0, 5.0]]], [[[7.0, 4.0, 8.0], 
[3.0, 7.0, 7.0], [2.0, 1.0, 7.0]], [[2.0, 8.0, 8.0], [4.0, 4.0, 5.0], [7.0, 6.0, 7.0]], [[3.0, 7.0, 1.0], [4.0, 8.0, 3.0], 
[5.0, 8.0, 1.0]]], [[[6.0, 7.0, 4.0], [2.0, 4.0, 5.0], [5.0, 7.0, 2.0]], [[4.0, 7.0, 7.0], [6.0, 1.0, 6.0], [4.0, 5.0, 6.0]], 
[[3.0, 2.0, 1.0], [2.0, 2.0, 7.0], [7.0, 6.0, 1.0]]], [[[6.0, 1.0, 1.0], [1.0, 1.0, 6.0], [7.0, 4.0, 3.0]], [[5.0, 6.0, 4.0], 
[3.0, 2.0, 4.0], [5.0, 1.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 4.0, 1.0], [4.0, 8.0, 6.0]]]])
      val=Data(numpy.array([[[2.0, 1.0, 7.0], [4.0, 1.0, 6.0], [2.0, 6.0, 4.0]], [[3.0, 5.0, 1.0], [5.0, 3.0, 3.0], [4.0, 7.0, 
4.0]], [[6.0, 5.0, 7.0], [4.0, 6.0, 8.0], [1.0, 2.0, 6.0]]]),self.functionspace)
      arg[:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 1.0], [2.0, 1.0, 7.0], [2.0, 2.0, 8.0]], [[1.0, 4.0, 8.0], [4.0, 1.0, 6.0], [6.0, 
4.0, 5.0]], [[6.0, 5.0, 4.0], [2.0, 6.0, 4.0], [8.0, 5.0, 4.0]]], [[[5.0, 5.0, 5.0], [3.0, 5.0, 1.0], [5.0, 5.0, 8.0]], [[6.0, 
8.0, 4.0], [5.0, 3.0, 3.0], [1.0, 8.0, 7.0]], [[7.0, 2.0, 6.0], [4.0, 7.0, 4.0], [4.0, 4.0, 4.0]]], [[[2.0, 4.0, 6.0], [6.0, 
5.0, 7.0], [8.0, 4.0, 8.0]], [[3.0, 4.0, 6.0], [4.0, 6.0, 8.0], [7.0, 6.0, 1.0]], [[2.0, 1.0, 8.0], [1.0, 2.0, 6.0], [1.0, 1.0, 
3.0]]], [[[6.0, 7.0, 4.0], [1.0, 7.0, 5.0], [4.0, 2.0, 1.0]], [[5.0, 2.0, 8.0], [5.0, 1.0, 5.0], [6.0, 4.0, 4.0]], [[3.0, 5.0, 
4.0], [4.0, 4.0, 7.0], [3.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 2.0], [2.0, 1.0, 7.0], [2.0, 7.0, 3.0]], 
[[5.0, 6.0, 3.0], [4.0, 1.0, 6.0], [5.0, 8.0, 4.0]], [[8.0, 5.0, 1.0], [2.0, 6.0, 4.0], [1.0, 2.0, 5.0]]], [[[7.0, 4.0, 8.0], 
[3.0, 5.0, 1.0], [2.0, 1.0, 7.0]], [[2.0, 8.0, 8.0], [5.0, 3.0, 3.0], [7.0, 6.0, 7.0]], [[3.0, 7.0, 1.0], [4.0, 7.0, 4.0], 
[5.0, 8.0, 1.0]]], [[[6.0, 7.0, 4.0], [6.0, 5.0, 7.0], [5.0, 7.0, 2.0]], [[4.0, 7.0, 7.0], [4.0, 6.0, 8.0], [4.0, 5.0, 6.0]], 
[[3.0, 2.0, 1.0], [1.0, 2.0, 6.0], [7.0, 6.0, 1.0]]], [[[6.0, 1.0, 1.0], [1.0, 1.0, 6.0], [7.0, 4.0, 3.0]], [[5.0, 6.0, 4.0], 
[3.0, 2.0, 4.0], [5.0, 1.0, 2.0]], [[8.0, 8.0, 1.0], [8.0, 4.0, 1.0], [4.0, 8.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 6.0], [7.0, 4.0, 3.0], [2.0, 5.0, 3.0]], [[4.0, 4.0, 3.0], [5.0, 8.0, 7.0], [3.0, 
5.0, 5.0]], [[6.0, 6.0, 8.0], [4.0, 1.0, 3.0], [8.0, 2.0, 4.0]]], [[[4.0, 8.0, 5.0], [4.0, 7.0, 8.0], [6.0, 7.0, 5.0]], [[1.0, 
4.0, 1.0], [2.0, 3.0, 8.0], [6.0, 1.0, 3.0]], [[6.0, 2.0, 7.0], [2.0, 2.0, 8.0], [6.0, 2.0, 4.0]]], [[[5.0, 7.0, 3.0], [1.0, 
3.0, 1.0], [2.0, 5.0, 7.0]], [[4.0, 3.0, 6.0], [1.0, 5.0, 7.0], [6.0, 2.0, 8.0]], [[1.0, 4.0, 1.0], [6.0, 4.0, 6.0], [3.0, 4.0, 
4.0]]], [[[1.0, 1.0, 3.0], [1.0, 8.0, 2.0], [1.0, 8.0, 6.0]], [[5.0, 3.0, 5.0], [1.0, 5.0, 2.0], [5.0, 3.0, 7.0]], [[3.0, 8.0, 
4.0], [1.0, 5.0, 3.0], [1.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 1.0], [4.0, 3.0, 7.0], [1.0, 8.0, 1.0]], 
[[3.0, 4.0, 2.0], [2.0, 6.0, 4.0], [6.0, 1.0, 2.0]], [[4.0, 3.0, 7.0], [6.0, 3.0, 5.0], [2.0, 3.0, 6.0]]], [[[5.0, 4.0, 2.0], 
[7.0, 4.0, 3.0], [7.0, 1.0, 2.0]], [[1.0, 5.0, 4.0], [7.0, 2.0, 5.0], [6.0, 8.0, 4.0]], [[7.0, 1.0, 7.0], [2.0, 4.0, 6.0], 
[2.0, 3.0, 8.0]]], [[[4.0, 4.0, 5.0], [7.0, 5.0, 2.0], [5.0, 2.0, 1.0]], [[2.0, 1.0, 2.0], [8.0, 6.0, 4.0], [5.0, 3.0, 4.0]], 
[[1.0, 5.0, 1.0], [7.0, 2.0, 1.0], [3.0, 4.0, 3.0]]], [[[3.0, 1.0, 2.0], [2.0, 8.0, 5.0], [2.0, 7.0, 3.0]], [[5.0, 6.0, 4.0], 
[6.0, 4.0, 6.0], [8.0, 7.0, 3.0]], [[4.0, 8.0, 5.0], [7.0, 6.0, 6.0], [6.0, 8.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 6.0, 2.0], [5.0, 2.0, 2.0], [7.0, 3.0, 7.0]], [[2.0, 5.0, 4.0], [3.0, 1.0, 8.0], [6.0, 
1.0, 1.0]], [[7.0, 5.0, 8.0], [6.0, 8.0, 7.0], [8.0, 5.0, 2.0]]])+(1.-msk_val)*numpy.array([[[3.0, 4.0, 2.0], [5.0, 5.0, 7.0], 
[3.0, 6.0, 4.0]], [[7.0, 1.0, 6.0], [2.0, 1.0, 4.0], [7.0, 7.0, 8.0]], [[5.0, 6.0, 6.0], [5.0, 4.0, 2.0], [7.0, 6.0, 4.0]]])
      arg[:3,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 3.0, 6.0], [2.0, 6.0, 2.0], [2.0, 5.0, 3.0]], [[4.0, 4.0, 3.0], [5.0, 2.0, 2.0], [3.0, 
5.0, 5.0]], [[6.0, 6.0, 8.0], [7.0, 3.0, 7.0], [8.0, 2.0, 4.0]]], [[[4.0, 8.0, 5.0], [2.0, 5.0, 4.0], [6.0, 7.0, 5.0]], [[1.0, 
4.0, 1.0], [3.0, 1.0, 8.0], [6.0, 1.0, 3.0]], [[6.0, 2.0, 7.0], [6.0, 1.0, 1.0], [6.0, 2.0, 4.0]]], [[[5.0, 7.0, 3.0], [7.0, 
5.0, 8.0], [2.0, 5.0, 7.0]], [[4.0, 3.0, 6.0], [6.0, 8.0, 7.0], [6.0, 2.0, 8.0]], [[1.0, 4.0, 1.0], [8.0, 5.0, 2.0], [3.0, 4.0, 
4.0]]], [[[1.0, 1.0, 3.0], [1.0, 8.0, 2.0], [1.0, 8.0, 6.0]], [[5.0, 3.0, 5.0], [1.0, 5.0, 2.0], [5.0, 3.0, 7.0]], [[3.0, 8.0, 
4.0], [1.0, 5.0, 3.0], [1.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 1.0], [3.0, 4.0, 2.0], [1.0, 8.0, 1.0]], 
[[3.0, 4.0, 2.0], [5.0, 5.0, 7.0], [6.0, 1.0, 2.0]], [[4.0, 3.0, 7.0], [3.0, 6.0, 4.0], [2.0, 3.0, 6.0]]], [[[5.0, 4.0, 2.0], 
[7.0, 1.0, 6.0], [7.0, 1.0, 2.0]], [[1.0, 5.0, 4.0], [2.0, 1.0, 4.0], [6.0, 8.0, 4.0]], [[7.0, 1.0, 7.0], [7.0, 7.0, 8.0], 
[2.0, 3.0, 8.0]]], [[[4.0, 4.0, 5.0], [5.0, 6.0, 6.0], [5.0, 2.0, 1.0]], [[2.0, 1.0, 2.0], [5.0, 4.0, 2.0], [5.0, 3.0, 4.0]], 
[[1.0, 5.0, 1.0], [7.0, 6.0, 4.0], [3.0, 4.0, 3.0]]], [[[3.0, 1.0, 2.0], [2.0, 8.0, 5.0], [2.0, 7.0, 3.0]], [[5.0, 6.0, 4.0], 
[6.0, 4.0, 6.0], [8.0, 7.0, 3.0]], [[4.0, 8.0, 5.0], [7.0, 6.0, 6.0], [6.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 8.0, 6.0], [3.0, 3.0, 1.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 1.0], [6.0, 5.0, 3.0], [6.0, 
8.0, 3.0]], [[3.0, 8.0, 5.0], [3.0, 5.0, 3.0], [8.0, 7.0, 4.0]]], [[[2.0, 7.0, 5.0], [7.0, 6.0, 3.0], [2.0, 1.0, 3.0]], [[5.0, 
6.0, 3.0], [6.0, 7.0, 5.0], [2.0, 3.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 5.0, 2.0], [5.0, 3.0, 6.0]]], [[[1.0, 6.0, 4.0], [5.0, 
5.0, 4.0], [5.0, 3.0, 8.0]], [[5.0, 6.0, 4.0], [2.0, 6.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 7.0, 6.0], [4.0, 2.0, 8.0], [5.0, 6.0, 
5.0]]], [[[8.0, 4.0, 3.0], [4.0, 5.0, 5.0], [3.0, 2.0, 5.0]], [[4.0, 8.0, 2.0], [6.0, 2.0, 4.0], [7.0, 8.0, 8.0]], [[7.0, 4.0, 
3.0], [6.0, 1.0, 8.0], [2.0, 2.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 8.0], [4.0, 1.0, 4.0], [4.0, 5.0, 5.0]], 
[[3.0, 2.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 4.0]], [[6.0, 3.0, 3.0], [5.0, 7.0, 5.0], [4.0, 7.0, 3.0]]], [[[2.0, 3.0, 4.0], 
[6.0, 3.0, 3.0], [3.0, 7.0, 8.0]], [[5.0, 3.0, 7.0], [7.0, 3.0, 8.0], [8.0, 6.0, 3.0]], [[6.0, 4.0, 4.0], [1.0, 2.0, 3.0], 
[1.0, 5.0, 4.0]]], [[[2.0, 2.0, 7.0], [1.0, 3.0, 1.0], [2.0, 2.0, 1.0]], [[2.0, 7.0, 2.0], [6.0, 2.0, 1.0], [7.0, 1.0, 4.0]], 
[[6.0, 8.0, 6.0], [3.0, 6.0, 3.0], [1.0, 8.0, 7.0]]], [[[5.0, 5.0, 8.0], [6.0, 6.0, 2.0], [7.0, 3.0, 2.0]], [[2.0, 5.0, 8.0], 
[8.0, 4.0, 2.0], [5.0, 3.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 2.0], [4.0, 3.0, 5.0]]]])
      val=numpy.array([[[[3.0, 8.0, 5.0]]], [[[4.0, 6.0, 7.0]]]])
      arg[2:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 8.0, 6.0], [3.0, 3.0, 1.0], [5.0, 8.0, 1.0]], [[5.0, 1.0, 1.0], [6.0, 5.0, 3.0], [6.0, 
8.0, 3.0]], [[3.0, 8.0, 5.0], [3.0, 5.0, 3.0], [8.0, 7.0, 4.0]]], [[[2.0, 7.0, 5.0], [7.0, 6.0, 3.0], [2.0, 1.0, 3.0]], [[5.0, 
6.0, 3.0], [6.0, 7.0, 5.0], [2.0, 3.0, 8.0]], [[2.0, 7.0, 2.0], [4.0, 5.0, 2.0], [5.0, 3.0, 6.0]]], [[[1.0, 6.0, 4.0], [5.0, 
5.0, 4.0], [5.0, 3.0, 8.0]], [[5.0, 6.0, 4.0], [3.0, 8.0, 5.0], [8.0, 6.0, 8.0]], [[5.0, 7.0, 6.0], [4.0, 2.0, 8.0], [5.0, 6.0, 
5.0]]], [[[8.0, 4.0, 3.0], [4.0, 5.0, 5.0], [3.0, 2.0, 5.0]], [[4.0, 8.0, 2.0], [4.0, 6.0, 7.0], [7.0, 8.0, 8.0]], [[7.0, 4.0, 
3.0], [6.0, 1.0, 8.0], [2.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 8.0], [4.0, 1.0, 4.0], [4.0, 5.0, 5.0]], 
[[3.0, 2.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 4.0]], [[6.0, 3.0, 3.0], [5.0, 7.0, 5.0], [4.0, 7.0, 3.0]]], [[[2.0, 3.0, 4.0], 
[6.0, 3.0, 3.0], [3.0, 7.0, 8.0]], [[5.0, 3.0, 7.0], [7.0, 3.0, 8.0], [8.0, 6.0, 3.0]], [[6.0, 4.0, 4.0], [1.0, 2.0, 3.0], 
[1.0, 5.0, 4.0]]], [[[2.0, 2.0, 7.0], [1.0, 3.0, 1.0], [2.0, 2.0, 1.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 5.0], [7.0, 1.0, 4.0]], 
[[6.0, 8.0, 6.0], [3.0, 6.0, 3.0], [1.0, 8.0, 7.0]]], [[[5.0, 5.0, 8.0], [6.0, 6.0, 2.0], [7.0, 3.0, 2.0]], [[2.0, 5.0, 8.0], 
[4.0, 6.0, 7.0], [5.0, 3.0, 2.0]], [[7.0, 6.0, 8.0], [8.0, 6.0, 2.0], [4.0, 3.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 2.0], [3.0, 3.0, 5.0], [4.0, 7.0, 6.0]], [[3.0, 8.0, 7.0], [8.0, 8.0, 7.0], [8.0, 
5.0, 5.0]], [[1.0, 6.0, 8.0], [7.0, 6.0, 6.0], [3.0, 3.0, 4.0]]], [[[1.0, 8.0, 6.0], [2.0, 6.0, 8.0], [3.0, 2.0, 1.0]], [[6.0, 
4.0, 4.0], [5.0, 4.0, 1.0], [8.0, 2.0, 6.0]], [[2.0, 5.0, 5.0], [3.0, 1.0, 6.0], [3.0, 6.0, 5.0]]], [[[2.0, 4.0, 2.0], [7.0, 
2.0, 7.0], [6.0, 5.0, 3.0]], [[2.0, 6.0, 6.0], [6.0, 2.0, 1.0], [5.0, 8.0, 5.0]], [[3.0, 1.0, 5.0], [7.0, 8.0, 4.0], [3.0, 2.0, 
3.0]]], [[[6.0, 4.0, 7.0], [1.0, 3.0, 2.0], [3.0, 8.0, 1.0]], [[4.0, 3.0, 3.0], [8.0, 5.0, 6.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 
6.0], [4.0, 5.0, 2.0], [3.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 3.0, 4.0], [4.0, 5.0, 8.0]], 
[[2.0, 4.0, 1.0], [6.0, 1.0, 2.0], [2.0, 1.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 1.0, 6.0], [4.0, 1.0, 5.0]]], [[[1.0, 3.0, 2.0], 
[6.0, 7.0, 7.0], [5.0, 2.0, 2.0]], [[8.0, 1.0, 7.0], [7.0, 3.0, 3.0], [2.0, 2.0, 2.0]], [[2.0, 5.0, 4.0], [6.0, 4.0, 4.0], 
[1.0, 8.0, 8.0]]], [[[5.0, 5.0, 1.0], [1.0, 1.0, 1.0], [1.0, 4.0, 8.0]], [[1.0, 2.0, 2.0], [8.0, 3.0, 8.0], [6.0, 2.0, 3.0]], 
[[3.0, 1.0, 7.0], [7.0, 6.0, 3.0], [1.0, 2.0, 8.0]]], [[[4.0, 4.0, 7.0], [3.0, 2.0, 4.0], [3.0, 4.0, 1.0]], [[2.0, 7.0, 6.0], 
[8.0, 5.0, 2.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 7.0], [8.0, 3.0, 5.0], [6.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[[4.0, 6.0, 4.0]]], [[[4.0, 2.0, 7.0]]]]),self.functionspace)
      arg[2:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 2.0], [3.0, 3.0, 5.0], [4.0, 7.0, 6.0]], [[3.0, 8.0, 7.0], [8.0, 8.0, 7.0], [8.0, 
5.0, 5.0]], [[1.0, 6.0, 8.0], [7.0, 6.0, 6.0], [3.0, 3.0, 4.0]]], [[[1.0, 8.0, 6.0], [2.0, 6.0, 8.0], [3.0, 2.0, 1.0]], [[6.0, 
4.0, 4.0], [5.0, 4.0, 1.0], [8.0, 2.0, 6.0]], [[2.0, 5.0, 5.0], [3.0, 1.0, 6.0], [3.0, 6.0, 5.0]]], [[[2.0, 4.0, 2.0], [7.0, 
2.0, 7.0], [6.0, 5.0, 3.0]], [[2.0, 6.0, 6.0], [4.0, 6.0, 4.0], [5.0, 8.0, 5.0]], [[3.0, 1.0, 5.0], [7.0, 8.0, 4.0], [3.0, 2.0, 
3.0]]], [[[6.0, 4.0, 7.0], [1.0, 3.0, 2.0], [3.0, 8.0, 1.0]], [[4.0, 3.0, 3.0], [4.0, 2.0, 7.0], [6.0, 4.0, 2.0]], [[1.0, 8.0, 
6.0], [4.0, 5.0, 2.0], [3.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 7.0, 2.0], [6.0, 3.0, 4.0], [4.0, 5.0, 8.0]], 
[[2.0, 4.0, 1.0], [6.0, 1.0, 2.0], [2.0, 1.0, 2.0]], [[2.0, 3.0, 4.0], [4.0, 1.0, 6.0], [4.0, 1.0, 5.0]]], [[[1.0, 3.0, 2.0], 
[6.0, 7.0, 7.0], [5.0, 2.0, 2.0]], [[8.0, 1.0, 7.0], [7.0, 3.0, 3.0], [2.0, 2.0, 2.0]], [[2.0, 5.0, 4.0], [6.0, 4.0, 4.0], 
[1.0, 8.0, 8.0]]], [[[5.0, 5.0, 1.0], [1.0, 1.0, 1.0], [1.0, 4.0, 8.0]], [[1.0, 2.0, 2.0], [4.0, 6.0, 4.0], [6.0, 2.0, 3.0]], 
[[3.0, 1.0, 7.0], [7.0, 6.0, 3.0], [1.0, 2.0, 8.0]]], [[[4.0, 4.0, 7.0], [3.0, 2.0, 4.0], [3.0, 4.0, 1.0]], [[2.0, 7.0, 6.0], 
[4.0, 2.0, 7.0], [4.0, 3.0, 7.0]], [[1.0, 6.0, 7.0], [8.0, 3.0, 5.0], [6.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 1.0], [8.0, 8.0, 4.0], [6.0, 2.0, 3.0]], [[6.0, 1.0, 2.0], [7.0, 8.0, 7.0], [7.0, 
6.0, 5.0]], [[2.0, 3.0, 8.0], [1.0, 6.0, 5.0], [1.0, 7.0, 4.0]]], [[[8.0, 6.0, 4.0], [1.0, 2.0, 2.0], [5.0, 6.0, 7.0]], [[3.0, 
1.0, 8.0], [4.0, 1.0, 7.0], [8.0, 2.0, 7.0]], [[8.0, 6.0, 6.0], [4.0, 2.0, 6.0], [5.0, 3.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 
3.0, 7.0], [8.0, 8.0, 4.0]], [[6.0, 2.0, 6.0], [3.0, 8.0, 5.0], [8.0, 6.0, 1.0]], [[1.0, 8.0, 2.0], [6.0, 6.0, 6.0], [5.0, 1.0, 
4.0]]], [[[7.0, 8.0, 6.0], [2.0, 2.0, 5.0], [7.0, 5.0, 7.0]], [[2.0, 6.0, 2.0], [4.0, 4.0, 6.0], [1.0, 1.0, 8.0]], [[6.0, 5.0, 
4.0], [2.0, 2.0, 7.0], [4.0, 1.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 1.0], [5.0, 7.0, 8.0], [6.0, 7.0, 6.0]], 
[[8.0, 6.0, 5.0], [1.0, 7.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 2.0], [2.0, 5.0, 4.0], [8.0, 1.0, 1.0]]], [[[4.0, 6.0, 5.0], 
[1.0, 4.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 4.0, 4.0], [8.0, 7.0, 4.0], [2.0, 6.0, 1.0]], [[8.0, 2.0, 1.0], [6.0, 2.0, 2.0], 
[7.0, 5.0, 4.0]]], [[[2.0, 4.0, 3.0], [2.0, 4.0, 1.0], [5.0, 2.0, 7.0]], [[1.0, 8.0, 2.0], [4.0, 4.0, 3.0], [6.0, 1.0, 5.0]], 
[[1.0, 2.0, 2.0], [4.0, 3.0, 3.0], [4.0, 7.0, 2.0]]], [[[8.0, 1.0, 8.0], [3.0, 2.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 2.0], 
[2.0, 1.0, 1.0], [1.0, 8.0, 8.0]], [[8.0, 3.0, 8.0], [1.0, 7.0, 2.0], [8.0, 3.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 2.0, 2.0]]], [[[6.0, 3.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 6.0, 4.0]]], 
[[[4.0, 8.0, 4.0]]]])
      arg[2:,1:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 1.0], [8.0, 8.0, 4.0], [6.0, 2.0, 3.0]], [[6.0, 1.0, 2.0], [7.0, 8.0, 7.0], [7.0, 
6.0, 5.0]], [[2.0, 3.0, 8.0], [1.0, 6.0, 5.0], [1.0, 7.0, 4.0]]], [[[8.0, 6.0, 4.0], [1.0, 2.0, 2.0], [5.0, 6.0, 7.0]], [[3.0, 
1.0, 8.0], [4.0, 1.0, 7.0], [8.0, 2.0, 7.0]], [[8.0, 6.0, 6.0], [4.0, 2.0, 6.0], [5.0, 3.0, 5.0]]], [[[7.0, 4.0, 7.0], [2.0, 
3.0, 7.0], [8.0, 8.0, 4.0]], [[6.0, 2.0, 6.0], [3.0, 2.0, 2.0], [8.0, 6.0, 1.0]], [[1.0, 8.0, 2.0], [6.0, 6.0, 6.0], [5.0, 1.0, 
4.0]]], [[[7.0, 8.0, 6.0], [2.0, 2.0, 5.0], [7.0, 5.0, 7.0]], [[2.0, 6.0, 2.0], [6.0, 3.0, 1.0], [1.0, 1.0, 8.0]], [[6.0, 5.0, 
4.0], [2.0, 2.0, 7.0], [4.0, 1.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 1.0], [5.0, 7.0, 8.0], [6.0, 7.0, 6.0]], 
[[8.0, 6.0, 5.0], [1.0, 7.0, 8.0], [3.0, 5.0, 8.0]], [[6.0, 1.0, 2.0], [2.0, 5.0, 4.0], [8.0, 1.0, 1.0]]], [[[4.0, 6.0, 5.0], 
[1.0, 4.0, 7.0], [5.0, 6.0, 1.0]], [[8.0, 4.0, 4.0], [8.0, 7.0, 4.0], [2.0, 6.0, 1.0]], [[8.0, 2.0, 1.0], [6.0, 2.0, 2.0], 
[7.0, 5.0, 4.0]]], [[[2.0, 4.0, 3.0], [2.0, 4.0, 1.0], [5.0, 2.0, 7.0]], [[1.0, 8.0, 2.0], [7.0, 6.0, 4.0], [6.0, 1.0, 5.0]], 
[[1.0, 2.0, 2.0], [4.0, 3.0, 3.0], [4.0, 7.0, 2.0]]], [[[8.0, 1.0, 8.0], [3.0, 2.0, 6.0], [2.0, 1.0, 3.0]], [[4.0, 6.0, 2.0], 
[4.0, 8.0, 4.0], [1.0, 8.0, 8.0]], [[8.0, 3.0, 8.0], [1.0, 7.0, 2.0], [8.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 8.0], [5.0, 7.0, 4.0], [1.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], [1.0, 4.0, 3.0], [8.0, 
8.0, 6.0]], [[5.0, 2.0, 2.0], [1.0, 2.0, 8.0], [1.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [6.0, 1.0, 7.0], [1.0, 2.0, 2.0]], [[3.0, 
6.0, 2.0], [4.0, 4.0, 5.0], [5.0, 5.0, 8.0]], [[1.0, 1.0, 7.0], [1.0, 7.0, 3.0], [3.0, 6.0, 4.0]]], [[[7.0, 8.0, 7.0], [7.0, 
3.0, 1.0], [4.0, 7.0, 7.0]], [[1.0, 7.0, 2.0], [4.0, 2.0, 5.0], [7.0, 6.0, 3.0]], [[6.0, 1.0, 3.0], [5.0, 8.0, 8.0], [1.0, 2.0, 
6.0]]], [[[7.0, 3.0, 3.0], [6.0, 8.0, 4.0], [6.0, 7.0, 3.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 5.0], [5.0, 6.0, 7.0]], [[8.0, 3.0, 
3.0], [5.0, 7.0, 7.0], [5.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 7.0, 6.0], [8.0, 7.0, 6.0], [5.0, 2.0, 5.0]], 
[[8.0, 5.0, 6.0], [4.0, 6.0, 8.0], [2.0, 7.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 3.0, 3.0], [1.0, 2.0, 1.0]]], [[[6.0, 5.0, 4.0], 
[7.0, 8.0, 4.0], [2.0, 3.0, 6.0]], [[8.0, 7.0, 8.0], [7.0, 7.0, 2.0], [8.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [5.0, 7.0, 6.0], 
[4.0, 4.0, 3.0]]], [[[6.0, 4.0, 1.0], [2.0, 4.0, 5.0], [2.0, 3.0, 7.0]], [[6.0, 2.0, 3.0], [1.0, 6.0, 4.0], [4.0, 5.0, 2.0]], 
[[8.0, 2.0, 6.0], [6.0, 6.0, 7.0], [3.0, 7.0, 5.0]]], [[[3.0, 3.0, 7.0], [2.0, 5.0, 2.0], [1.0, 5.0, 1.0]], [[5.0, 3.0, 1.0], 
[7.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[7.0, 6.0, 4.0], [1.0, 8.0, 3.0], [3.0, 6.0, 3.0]]]])
      val=numpy.array([[[[3.0, 1.0, 6.0]], [[1.0, 3.0, 4.0]]], [[[7.0, 5.0, 1.0]], [[4.0, 7.0, 2.0]]], [[[3.0, 4.0, 5.0]], 
[[7.0, 2.0, 3.0]]]])
      arg[:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 8.0], [3.0, 1.0, 6.0], [1.0, 7.0, 7.0]], [[2.0, 2.0, 7.0], [1.0, 3.0, 4.0], [8.0, 
8.0, 6.0]], [[5.0, 2.0, 2.0], [1.0, 2.0, 8.0], [1.0, 3.0, 1.0]]], [[[7.0, 8.0, 1.0], [7.0, 5.0, 1.0], [1.0, 2.0, 2.0]], [[3.0, 
6.0, 2.0], [4.0, 7.0, 2.0], [5.0, 5.0, 8.0]], [[1.0, 1.0, 7.0], [1.0, 7.0, 3.0], [3.0, 6.0, 4.0]]], [[[7.0, 8.0, 7.0], [3.0, 
4.0, 5.0], [4.0, 7.0, 7.0]], [[1.0, 7.0, 2.0], [7.0, 2.0, 3.0], [7.0, 6.0, 3.0]], [[6.0, 1.0, 3.0], [5.0, 8.0, 8.0], [1.0, 2.0, 
6.0]]], [[[7.0, 3.0, 3.0], [6.0, 8.0, 4.0], [6.0, 7.0, 3.0]], [[8.0, 2.0, 5.0], [7.0, 5.0, 5.0], [5.0, 6.0, 7.0]], [[8.0, 3.0, 
3.0], [5.0, 7.0, 7.0], [5.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 7.0, 6.0], [3.0, 1.0, 6.0], [5.0, 2.0, 5.0]], 
[[8.0, 5.0, 6.0], [1.0, 3.0, 4.0], [2.0, 7.0, 6.0]], [[7.0, 3.0, 2.0], [8.0, 3.0, 3.0], [1.0, 2.0, 1.0]]], [[[6.0, 5.0, 4.0], 
[7.0, 5.0, 1.0], [2.0, 3.0, 6.0]], [[8.0, 7.0, 8.0], [4.0, 7.0, 2.0], [8.0, 6.0, 1.0]], [[7.0, 2.0, 7.0], [5.0, 7.0, 6.0], 
[4.0, 4.0, 3.0]]], [[[6.0, 4.0, 1.0], [3.0, 4.0, 5.0], [2.0, 3.0, 7.0]], [[6.0, 2.0, 3.0], [7.0, 2.0, 3.0], [4.0, 5.0, 2.0]], 
[[8.0, 2.0, 6.0], [6.0, 6.0, 7.0], [3.0, 7.0, 5.0]]], [[[3.0, 3.0, 7.0], [2.0, 5.0, 2.0], [1.0, 5.0, 1.0]], [[5.0, 3.0, 1.0], 
[7.0, 7.0, 1.0], [3.0, 3.0, 3.0]], [[7.0, 6.0, 4.0], [1.0, 8.0, 3.0], [3.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 4.0, 2.0], [8.0, 6.0, 3.0], [2.0, 2.0, 6.0]], [[5.0, 4.0, 3.0], [6.0, 4.0, 4.0], [3.0, 
2.0, 5.0]], [[4.0, 6.0, 2.0], [8.0, 3.0, 5.0], [4.0, 2.0, 8.0]]], [[[4.0, 7.0, 2.0], [7.0, 1.0, 1.0], [8.0, 1.0, 1.0]], [[1.0, 
1.0, 4.0], [7.0, 5.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], [4.0, 5.0, 1.0], [2.0, 3.0, 3.0]]], [[[5.0, 4.0, 2.0], [2.0, 
2.0, 7.0], [3.0, 4.0, 6.0]], [[7.0, 2.0, 1.0], [5.0, 6.0, 3.0], [2.0, 8.0, 4.0]], [[3.0, 7.0, 1.0], [2.0, 7.0, 5.0], [2.0, 1.0, 
8.0]]], [[[3.0, 3.0, 7.0], [7.0, 7.0, 3.0], [5.0, 7.0, 5.0]], [[5.0, 3.0, 4.0], [6.0, 6.0, 6.0], [6.0, 3.0, 4.0]], [[7.0, 8.0, 
5.0], [4.0, 4.0, 1.0], [1.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 3.0, 5.0], [8.0, 8.0, 1.0], [5.0, 7.0, 6.0]], 
[[7.0, 5.0, 3.0], [2.0, 7.0, 3.0], [3.0, 4.0, 4.0]], [[5.0, 4.0, 4.0], [2.0, 8.0, 8.0], [4.0, 1.0, 3.0]]], [[[8.0, 2.0, 3.0], 
[8.0, 3.0, 2.0], [2.0, 2.0, 8.0]], [[1.0, 8.0, 4.0], [7.0, 1.0, 4.0], [7.0, 7.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 1.0, 1.0], 
[3.0, 5.0, 4.0]]], [[[5.0, 3.0, 5.0], [3.0, 6.0, 1.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 7.0], [1.0, 8.0, 7.0], [1.0, 3.0, 6.0]], 
[[3.0, 1.0, 6.0], [6.0, 7.0, 6.0], [1.0, 6.0, 7.0]]], [[[4.0, 1.0, 2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], 
[5.0, 4.0, 4.0], [6.0, 3.0, 6.0]], [[3.0, 5.0, 7.0], [2.0, 3.0, 2.0], [7.0, 5.0, 1.0]]]])
      val=Data(numpy.array([[[[3.0, 4.0, 1.0]], [[3.0, 3.0, 7.0]]], [[[6.0, 4.0, 3.0]], [[7.0, 1.0, 7.0]]], [[[2.0, 1.0, 1.0]], 
[[5.0, 1.0, 5.0]]]]),self.functionspace)
      arg[:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 4.0, 2.0], [3.0, 4.0, 1.0], [2.0, 2.0, 6.0]], [[5.0, 4.0, 3.0], [3.0, 3.0, 7.0], [3.0, 
2.0, 5.0]], [[4.0, 6.0, 2.0], [8.0, 3.0, 5.0], [4.0, 2.0, 8.0]]], [[[4.0, 7.0, 2.0], [6.0, 4.0, 3.0], [8.0, 1.0, 1.0]], [[1.0, 
1.0, 4.0], [7.0, 1.0, 7.0], [3.0, 6.0, 6.0]], [[3.0, 8.0, 4.0], [4.0, 5.0, 1.0], [2.0, 3.0, 3.0]]], [[[5.0, 4.0, 2.0], [2.0, 
1.0, 1.0], [3.0, 4.0, 6.0]], [[7.0, 2.0, 1.0], [5.0, 1.0, 5.0], [2.0, 8.0, 4.0]], [[3.0, 7.0, 1.0], [2.0, 7.0, 5.0], [2.0, 1.0, 
8.0]]], [[[3.0, 3.0, 7.0], [7.0, 7.0, 3.0], [5.0, 7.0, 5.0]], [[5.0, 3.0, 4.0], [6.0, 6.0, 6.0], [6.0, 3.0, 4.0]], [[7.0, 8.0, 
5.0], [4.0, 4.0, 1.0], [1.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 3.0, 5.0], [3.0, 4.0, 1.0], [5.0, 7.0, 6.0]], 
[[7.0, 5.0, 3.0], [3.0, 3.0, 7.0], [3.0, 4.0, 4.0]], [[5.0, 4.0, 4.0], [2.0, 8.0, 8.0], [4.0, 1.0, 3.0]]], [[[8.0, 2.0, 3.0], 
[6.0, 4.0, 3.0], [2.0, 2.0, 8.0]], [[1.0, 8.0, 4.0], [7.0, 1.0, 7.0], [7.0, 7.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 1.0, 1.0], 
[3.0, 5.0, 4.0]]], [[[5.0, 3.0, 5.0], [2.0, 1.0, 1.0], [8.0, 3.0, 7.0]], [[4.0, 6.0, 7.0], [5.0, 1.0, 5.0], [1.0, 3.0, 6.0]], 
[[3.0, 1.0, 6.0], [6.0, 7.0, 6.0], [1.0, 6.0, 7.0]]], [[[4.0, 1.0, 2.0], [1.0, 7.0, 1.0], [3.0, 2.0, 1.0]], [[2.0, 8.0, 1.0], 
[5.0, 4.0, 4.0], [6.0, 3.0, 6.0]], [[3.0, 5.0, 7.0], [2.0, 3.0, 2.0], [7.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UURN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 3.0], [2.0, 1.0, 2.0], [6.0, 1.0, 7.0]], [[3.0, 5.0, 5.0], [3.0, 1.0, 6.0], [5.0, 
3.0, 7.0]], [[4.0, 3.0, 2.0], [3.0, 2.0, 6.0], [8.0, 1.0, 6.0]]], [[[2.0, 1.0, 4.0], [2.0, 8.0, 8.0], [8.0, 1.0, 7.0]], [[6.0, 
2.0, 7.0], [1.0, 6.0, 1.0], [3.0, 1.0, 1.0]], [[6.0, 7.0, 2.0], [1.0, 2.0, 7.0], [4.0, 3.0, 7.0]]], [[[8.0, 6.0, 1.0], [3.0, 
3.0, 6.0], [2.0, 7.0, 7.0]], [[8.0, 5.0, 2.0], [7.0, 8.0, 3.0], [4.0, 6.0, 1.0]], [[4.0, 1.0, 3.0], [3.0, 7.0, 5.0], [4.0, 7.0, 
8.0]]], [[[6.0, 3.0, 4.0], [8.0, 3.0, 2.0], [6.0, 1.0, 2.0]], [[8.0, 7.0, 1.0], [7.0, 1.0, 4.0], [2.0, 5.0, 7.0]], [[6.0, 8.0, 
8.0], [4.0, 1.0, 3.0], [1.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 7.0], [6.0, 5.0, 8.0], [7.0, 4.0, 3.0]], 
[[6.0, 8.0, 5.0], [3.0, 8.0, 5.0], [3.0, 1.0, 4.0]], [[3.0, 3.0, 5.0], [1.0, 2.0, 6.0], [6.0, 3.0, 3.0]]], [[[8.0, 1.0, 4.0], 
[4.0, 6.0, 2.0], [2.0, 6.0, 1.0]], [[2.0, 2.0, 7.0], [1.0, 2.0, 8.0], [4.0, 6.0, 8.0]], [[7.0, 2.0, 7.0], [5.0, 8.0, 1.0], 
[6.0, 8.0, 7.0]]], [[[6.0, 8.0, 1.0], [2.0, 6.0, 3.0], [8.0, 4.0, 6.0]], [[8.0, 4.0, 4.0], [7.0, 4.0, 6.0], [5.0, 2.0, 2.0]], 
[[2.0, 1.0, 8.0], [1.0, 7.0, 3.0], [5.0, 3.0, 4.0]]], [[[3.0, 5.0, 3.0], [8.0, 1.0, 4.0], [2.0, 3.0, 7.0]], [[6.0, 1.0, 1.0], 
[1.0, 2.0, 8.0], [6.0, 8.0, 4.0]], [[2.0, 7.0, 1.0], [4.0, 5.0, 4.0], [7.0, 7.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 1.0, 3.0]], [[8.0, 8.0, 4.0]]], [[[6.0, 5.0, 6.0]], [[4.0, 6.0, 5.0]]], [[[8.0, 5.0, 
5.0]], [[8.0, 6.0, 5.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 4.0, 7.0]], [[3.0, 7.0, 3.0]]], [[[2.0, 5.0, 6.0]], [[8.0, 8.0, 
1.0]]], [[[1.0, 6.0, 5.0]], [[4.0, 5.0, 7.0]]]])
      arg[:3,:2,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 3.0], [7.0, 1.0, 3.0], [6.0, 1.0, 7.0]], [[3.0, 5.0, 5.0], [8.0, 8.0, 4.0], [5.0, 
3.0, 7.0]], [[4.0, 3.0, 2.0], [3.0, 2.0, 6.0], [8.0, 1.0, 6.0]]], [[[2.0, 1.0, 4.0], [6.0, 5.0, 6.0], [8.0, 1.0, 7.0]], [[6.0, 
2.0, 7.0], [4.0, 6.0, 5.0], [3.0, 1.0, 1.0]], [[6.0, 7.0, 2.0], [1.0, 2.0, 7.0], [4.0, 3.0, 7.0]]], [[[8.0, 6.0, 1.0], [8.0, 
5.0, 5.0], [2.0, 7.0, 7.0]], [[8.0, 5.0, 2.0], [8.0, 6.0, 5.0], [4.0, 6.0, 1.0]], [[4.0, 1.0, 3.0], [3.0, 7.0, 5.0], [4.0, 7.0, 
8.0]]], [[[6.0, 3.0, 4.0], [8.0, 3.0, 2.0], [6.0, 1.0, 2.0]], [[8.0, 7.0, 1.0], [7.0, 1.0, 4.0], [2.0, 5.0, 7.0]], [[6.0, 8.0, 
8.0], [4.0, 1.0, 3.0], [1.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 7.0], [5.0, 4.0, 7.0], [7.0, 4.0, 3.0]], 
[[6.0, 8.0, 5.0], [3.0, 7.0, 3.0], [3.0, 1.0, 4.0]], [[3.0, 3.0, 5.0], [1.0, 2.0, 6.0], [6.0, 3.0, 3.0]]], [[[8.0, 1.0, 4.0], 
[2.0, 5.0, 6.0], [2.0, 6.0, 1.0]], [[2.0, 2.0, 7.0], [8.0, 8.0, 1.0], [4.0, 6.0, 8.0]], [[7.0, 2.0, 7.0], [5.0, 8.0, 1.0], 
[6.0, 8.0, 7.0]]], [[[6.0, 8.0, 1.0], [1.0, 6.0, 5.0], [8.0, 4.0, 6.0]], [[8.0, 4.0, 4.0], [4.0, 5.0, 7.0], [5.0, 2.0, 2.0]], 
[[2.0, 1.0, 8.0], [1.0, 7.0, 3.0], [5.0, 3.0, 4.0]]], [[[3.0, 5.0, 3.0], [8.0, 1.0, 4.0], [2.0, 3.0, 7.0]], [[6.0, 1.0, 1.0], 
[1.0, 2.0, 8.0], [6.0, 8.0, 4.0]], [[2.0, 7.0, 1.0], [4.0, 5.0, 4.0], [7.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 4.0], [6.0, 6.0, 1.0], [2.0, 1.0, 4.0]], [[7.0, 5.0, 7.0], [8.0, 3.0, 3.0], [3.0, 
5.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 6.0, 6.0], [4.0, 7.0, 3.0]]], [[[6.0, 6.0, 2.0], [6.0, 4.0, 4.0], [4.0, 7.0, 3.0]], [[7.0, 
4.0, 6.0], [2.0, 1.0, 3.0], [3.0, 7.0, 3.0]], [[2.0, 2.0, 2.0], [8.0, 4.0, 8.0], [2.0, 6.0, 2.0]]], [[[2.0, 8.0, 7.0], [8.0, 
4.0, 5.0], [1.0, 1.0, 2.0]], [[3.0, 4.0, 7.0], [2.0, 4.0, 6.0], [3.0, 1.0, 6.0]], [[2.0, 2.0, 6.0], [8.0, 5.0, 8.0], [7.0, 8.0, 
7.0]]], [[[3.0, 6.0, 3.0], [3.0, 5.0, 7.0], [6.0, 7.0, 4.0]], [[3.0, 5.0, 6.0], [6.0, 5.0, 6.0], [1.0, 1.0, 2.0]], [[2.0, 4.0, 
8.0], [5.0, 5.0, 8.0], [5.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 3.0], [6.0, 5.0, 7.0], [8.0, 1.0, 3.0]], 
[[1.0, 4.0, 7.0], [7.0, 3.0, 2.0], [1.0, 8.0, 2.0]], [[3.0, 7.0, 6.0], [2.0, 7.0, 7.0], [3.0, 2.0, 5.0]]], [[[6.0, 4.0, 4.0], 
[3.0, 2.0, 8.0], [5.0, 2.0, 5.0]], [[7.0, 8.0, 2.0], [7.0, 5.0, 5.0], [2.0, 1.0, 2.0]], [[6.0, 3.0, 2.0], [6.0, 1.0, 8.0], 
[3.0, 8.0, 3.0]]], [[[4.0, 2.0, 6.0], [6.0, 3.0, 2.0], [1.0, 3.0, 2.0]], [[8.0, 4.0, 4.0], [4.0, 2.0, 7.0], [7.0, 1.0, 3.0]], 
[[4.0, 5.0, 1.0], [5.0, 1.0, 1.0], [8.0, 4.0, 8.0]]], [[[3.0, 4.0, 6.0], [8.0, 6.0, 2.0], [3.0, 5.0, 4.0]], [[6.0, 7.0, 6.0], 
[1.0, 5.0, 7.0], [3.0, 4.0, 3.0]], [[3.0, 8.0, 3.0], [8.0, 5.0, 7.0], [2.0, 5.0, 1.0]]]])
      val=numpy.array([[[7.0, 4.0, 2.0], [4.0, 4.0, 5.0]], [[2.0, 2.0, 4.0], [6.0, 2.0, 3.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 
7.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 8.0]]])
      arg[:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 4.0], [6.0, 6.0, 1.0], [2.0, 1.0, 4.0]], [[7.0, 4.0, 2.0], [4.0, 4.0, 5.0], [3.0, 
5.0, 3.0]], [[2.0, 3.0, 4.0], [8.0, 6.0, 6.0], [4.0, 7.0, 3.0]]], [[[6.0, 6.0, 2.0], [6.0, 4.0, 4.0], [4.0, 7.0, 3.0]], [[2.0, 
2.0, 4.0], [6.0, 2.0, 3.0], [3.0, 7.0, 3.0]], [[2.0, 2.0, 2.0], [8.0, 4.0, 8.0], [2.0, 6.0, 2.0]]], [[[2.0, 8.0, 7.0], [8.0, 
4.0, 5.0], [1.0, 1.0, 2.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 7.0], [3.0, 1.0, 6.0]], [[2.0, 2.0, 6.0], [8.0, 5.0, 8.0], [7.0, 8.0, 
7.0]]], [[[3.0, 6.0, 3.0], [3.0, 5.0, 7.0], [6.0, 7.0, 4.0]], [[2.0, 2.0, 5.0], [3.0, 8.0, 8.0], [1.0, 1.0, 2.0]], [[2.0, 4.0, 
8.0], [5.0, 5.0, 8.0], [5.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 4.0, 3.0], [6.0, 5.0, 7.0], [8.0, 1.0, 3.0]], 
[[7.0, 4.0, 2.0], [4.0, 4.0, 5.0], [1.0, 8.0, 2.0]], [[3.0, 7.0, 6.0], [2.0, 7.0, 7.0], [3.0, 2.0, 5.0]]], [[[6.0, 4.0, 4.0], 
[3.0, 2.0, 8.0], [5.0, 2.0, 5.0]], [[2.0, 2.0, 4.0], [6.0, 2.0, 3.0], [2.0, 1.0, 2.0]], [[6.0, 3.0, 2.0], [6.0, 1.0, 8.0], 
[3.0, 8.0, 3.0]]], [[[4.0, 2.0, 6.0], [6.0, 3.0, 2.0], [1.0, 3.0, 2.0]], [[6.0, 3.0, 5.0], [1.0, 4.0, 7.0], [7.0, 1.0, 3.0]], 
[[4.0, 5.0, 1.0], [5.0, 1.0, 1.0], [8.0, 4.0, 8.0]]], [[[3.0, 4.0, 6.0], [8.0, 6.0, 2.0], [3.0, 5.0, 4.0]], [[2.0, 2.0, 5.0], 
[3.0, 8.0, 8.0], [3.0, 4.0, 3.0]], [[3.0, 8.0, 3.0], [8.0, 5.0, 7.0], [2.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 3.0], [3.0, 4.0, 1.0], [2.0, 2.0, 7.0]], [[6.0, 4.0, 7.0], [7.0, 2.0, 3.0], [3.0, 
7.0, 6.0]], [[1.0, 3.0, 2.0], [5.0, 6.0, 5.0], [4.0, 6.0, 6.0]]], [[[6.0, 5.0, 1.0], [5.0, 1.0, 1.0], [6.0, 8.0, 1.0]], [[6.0, 
4.0, 8.0], [4.0, 7.0, 5.0], [4.0, 8.0, 4.0]], [[8.0, 7.0, 6.0], [3.0, 2.0, 6.0], [6.0, 2.0, 1.0]]], [[[1.0, 8.0, 8.0], [1.0, 
8.0, 7.0], [4.0, 5.0, 5.0]], [[8.0, 3.0, 7.0], [1.0, 3.0, 5.0], [8.0, 3.0, 3.0]], [[8.0, 4.0, 1.0], [7.0, 1.0, 2.0], [4.0, 4.0, 
7.0]]], [[[6.0, 4.0, 7.0], [6.0, 4.0, 6.0], [8.0, 2.0, 8.0]], [[6.0, 8.0, 2.0], [8.0, 4.0, 6.0], [2.0, 3.0, 6.0]], [[7.0, 2.0, 
6.0], [3.0, 2.0, 5.0], [3.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 7.0, 8.0], [8.0, 1.0, 8.0], [6.0, 1.0, 5.0]], 
[[3.0, 3.0, 7.0], [6.0, 6.0, 7.0], [4.0, 4.0, 6.0]], [[3.0, 5.0, 6.0], [2.0, 4.0, 3.0], [5.0, 6.0, 5.0]]], [[[1.0, 5.0, 2.0], 
[5.0, 1.0, 4.0], [4.0, 5.0, 4.0]], [[3.0, 1.0, 5.0], [6.0, 5.0, 1.0], [7.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 4.0], 
[2.0, 2.0, 7.0]]], [[[2.0, 1.0, 8.0], [4.0, 5.0, 7.0], [3.0, 2.0, 5.0]], [[3.0, 2.0, 6.0], [8.0, 1.0, 6.0], [5.0, 3.0, 4.0]], 
[[1.0, 3.0, 1.0], [4.0, 1.0, 8.0], [1.0, 6.0, 8.0]]], [[[2.0, 1.0, 1.0], [7.0, 1.0, 6.0], [8.0, 6.0, 5.0]], [[7.0, 1.0, 7.0], 
[6.0, 1.0, 4.0], [1.0, 5.0, 1.0]], [[6.0, 1.0, 6.0], [1.0, 1.0, 6.0], [6.0, 2.0, 1.0]]]])
      val=Data(numpy.array([[[7.0, 3.0, 4.0], [6.0, 7.0, 3.0]], [[1.0, 2.0, 5.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 7.0], [8.0, 
4.0, 6.0]], [[7.0, 6.0, 7.0], [7.0, 6.0, 3.0]]]),self.functionspace)
      arg[:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 3.0], [3.0, 4.0, 1.0], [2.0, 2.0, 7.0]], [[7.0, 3.0, 4.0], [6.0, 7.0, 3.0], [3.0, 
7.0, 6.0]], [[1.0, 3.0, 2.0], [5.0, 6.0, 5.0], [4.0, 6.0, 6.0]]], [[[6.0, 5.0, 1.0], [5.0, 1.0, 1.0], [6.0, 8.0, 1.0]], [[1.0, 
2.0, 5.0], [5.0, 6.0, 4.0], [4.0, 8.0, 4.0]], [[8.0, 7.0, 6.0], [3.0, 2.0, 6.0], [6.0, 2.0, 1.0]]], [[[1.0, 8.0, 8.0], [1.0, 
8.0, 7.0], [4.0, 5.0, 5.0]], [[2.0, 3.0, 7.0], [8.0, 4.0, 6.0], [8.0, 3.0, 3.0]], [[8.0, 4.0, 1.0], [7.0, 1.0, 2.0], [4.0, 4.0, 
7.0]]], [[[6.0, 4.0, 7.0], [6.0, 4.0, 6.0], [8.0, 2.0, 8.0]], [[7.0, 6.0, 7.0], [7.0, 6.0, 3.0], [2.0, 3.0, 6.0]], [[7.0, 2.0, 
6.0], [3.0, 2.0, 5.0], [3.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 7.0, 8.0], [8.0, 1.0, 8.0], [6.0, 1.0, 5.0]], 
[[7.0, 3.0, 4.0], [6.0, 7.0, 3.0], [4.0, 4.0, 6.0]], [[3.0, 5.0, 6.0], [2.0, 4.0, 3.0], [5.0, 6.0, 5.0]]], [[[1.0, 5.0, 2.0], 
[5.0, 1.0, 4.0], [4.0, 5.0, 4.0]], [[1.0, 2.0, 5.0], [5.0, 6.0, 4.0], [7.0, 8.0, 8.0]], [[4.0, 8.0, 4.0], [5.0, 7.0, 4.0], 
[2.0, 2.0, 7.0]]], [[[2.0, 1.0, 8.0], [4.0, 5.0, 7.0], [3.0, 2.0, 5.0]], [[2.0, 3.0, 7.0], [8.0, 4.0, 6.0], [5.0, 3.0, 4.0]], 
[[1.0, 3.0, 1.0], [4.0, 1.0, 8.0], [1.0, 6.0, 8.0]]], [[[2.0, 1.0, 1.0], [7.0, 1.0, 6.0], [8.0, 6.0, 5.0]], [[7.0, 6.0, 7.0], 
[7.0, 6.0, 3.0], [1.0, 5.0, 1.0]], [[6.0, 1.0, 6.0], [1.0, 1.0, 6.0], [6.0, 2.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 8.0], [5.0, 7.0, 5.0], [6.0, 7.0, 3.0]], [[8.0, 6.0, 2.0], [3.0, 4.0, 6.0], [7.0, 
8.0, 6.0]], [[4.0, 1.0, 3.0], [4.0, 1.0, 6.0], [3.0, 6.0, 8.0]]], [[[1.0, 6.0, 3.0], [6.0, 5.0, 7.0], [6.0, 4.0, 4.0]], [[2.0, 
2.0, 7.0], [6.0, 5.0, 5.0], [6.0, 5.0, 2.0]], [[4.0, 5.0, 1.0], [6.0, 1.0, 3.0], [5.0, 4.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 
4.0, 7.0], [2.0, 4.0, 5.0]], [[7.0, 8.0, 2.0], [8.0, 8.0, 3.0], [6.0, 7.0, 6.0]], [[1.0, 2.0, 1.0], [7.0, 6.0, 8.0], [1.0, 5.0, 
7.0]]], [[[3.0, 5.0, 8.0], [2.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[4.0, 1.0, 2.0], [4.0, 7.0, 4.0], [6.0, 3.0, 7.0]], [[7.0, 6.0, 
2.0], [1.0, 6.0, 4.0], [6.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 4.0, 1.0], [2.0, 3.0, 1.0], [8.0, 4.0, 3.0]], 
[[1.0, 1.0, 3.0], [7.0, 3.0, 8.0], [3.0, 4.0, 2.0]], [[8.0, 4.0, 5.0], [6.0, 5.0, 6.0], [4.0, 3.0, 4.0]]], [[[2.0, 1.0, 2.0], 
[7.0, 4.0, 5.0], [8.0, 2.0, 3.0]], [[6.0, 4.0, 7.0], [2.0, 4.0, 3.0], [5.0, 7.0, 2.0]], [[1.0, 2.0, 2.0], [8.0, 6.0, 2.0], 
[6.0, 3.0, 1.0]]], [[[6.0, 3.0, 8.0], [7.0, 4.0, 5.0], [6.0, 5.0, 8.0]], [[1.0, 4.0, 5.0], [2.0, 4.0, 6.0], [5.0, 5.0, 6.0]], 
[[1.0, 8.0, 7.0], [5.0, 4.0, 5.0], [8.0, 1.0, 7.0]]], [[[4.0, 5.0, 8.0], [8.0, 4.0, 3.0], [2.0, 5.0, 1.0]], [[7.0, 3.0, 6.0], 
[7.0, 3.0, 8.0], [7.0, 7.0, 1.0]], [[8.0, 6.0, 3.0], [1.0, 5.0, 1.0], [3.0, 6.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 5.0, 5.0], [2.0, 5.0, 4.0]], [[5.0, 5.0, 2.0], [1.0, 6.0, 4.0]], [[2.0, 6.0, 5.0], [1.0, 
8.0, 8.0]], [[3.0, 7.0, 8.0], [6.0, 8.0, 7.0]]])+(1.-msk_val)*numpy.array([[[4.0, 8.0, 4.0], [8.0, 6.0, 2.0]], [[2.0, 1.0, 
7.0], [7.0, 8.0, 4.0]], [[6.0, 7.0, 7.0], [2.0, 6.0, 3.0]], [[3.0, 4.0, 1.0], [4.0, 3.0, 1.0]]])
      arg[:,1,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 8.0], [5.0, 7.0, 5.0], [6.0, 7.0, 3.0]], [[3.0, 5.0, 5.0], [2.0, 5.0, 4.0], [7.0, 
8.0, 6.0]], [[4.0, 1.0, 3.0], [4.0, 1.0, 6.0], [3.0, 6.0, 8.0]]], [[[1.0, 6.0, 3.0], [6.0, 5.0, 7.0], [6.0, 4.0, 4.0]], [[5.0, 
5.0, 2.0], [1.0, 6.0, 4.0], [6.0, 5.0, 2.0]], [[4.0, 5.0, 1.0], [6.0, 1.0, 3.0], [5.0, 4.0, 6.0]]], [[[4.0, 5.0, 3.0], [5.0, 
4.0, 7.0], [2.0, 4.0, 5.0]], [[2.0, 6.0, 5.0], [1.0, 8.0, 8.0], [6.0, 7.0, 6.0]], [[1.0, 2.0, 1.0], [7.0, 6.0, 8.0], [1.0, 5.0, 
7.0]]], [[[3.0, 5.0, 8.0], [2.0, 3.0, 3.0], [4.0, 4.0, 3.0]], [[3.0, 7.0, 8.0], [6.0, 8.0, 7.0], [6.0, 3.0, 7.0]], [[7.0, 6.0, 
2.0], [1.0, 6.0, 4.0], [6.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 4.0, 1.0], [2.0, 3.0, 1.0], [8.0, 4.0, 3.0]], 
[[4.0, 8.0, 4.0], [8.0, 6.0, 2.0], [3.0, 4.0, 2.0]], [[8.0, 4.0, 5.0], [6.0, 5.0, 6.0], [4.0, 3.0, 4.0]]], [[[2.0, 1.0, 2.0], 
[7.0, 4.0, 5.0], [8.0, 2.0, 3.0]], [[2.0, 1.0, 7.0], [7.0, 8.0, 4.0], [5.0, 7.0, 2.0]], [[1.0, 2.0, 2.0], [8.0, 6.0, 2.0], 
[6.0, 3.0, 1.0]]], [[[6.0, 3.0, 8.0], [7.0, 4.0, 5.0], [6.0, 5.0, 8.0]], [[6.0, 7.0, 7.0], [2.0, 6.0, 3.0], [5.0, 5.0, 6.0]], 
[[1.0, 8.0, 7.0], [5.0, 4.0, 5.0], [8.0, 1.0, 7.0]]], [[[4.0, 5.0, 8.0], [8.0, 4.0, 3.0], [2.0, 5.0, 1.0]], [[3.0, 4.0, 1.0], 
[4.0, 3.0, 1.0], [7.0, 7.0, 1.0]], [[8.0, 6.0, 3.0], [1.0, 5.0, 1.0], [3.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 6.0], [3.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[4.0, 2.0, 4.0], [6.0, 2.0, 2.0], [5.0, 
2.0, 7.0]], [[3.0, 6.0, 5.0], [7.0, 6.0, 8.0], [7.0, 6.0, 1.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 6.0], [8.0, 2.0, 1.0]], [[1.0, 
1.0, 1.0], [2.0, 8.0, 4.0], [8.0, 5.0, 3.0]], [[1.0, 5.0, 2.0], [1.0, 6.0, 8.0], [8.0, 5.0, 1.0]]], [[[8.0, 4.0, 8.0], [6.0, 
5.0, 4.0], [7.0, 2.0, 1.0]], [[5.0, 6.0, 8.0], [2.0, 5.0, 8.0], [6.0, 3.0, 2.0]], [[8.0, 8.0, 3.0], [5.0, 7.0, 4.0], [5.0, 4.0, 
2.0]]], [[[1.0, 3.0, 1.0], [2.0, 6.0, 1.0], [4.0, 7.0, 6.0]], [[4.0, 4.0, 2.0], [1.0, 6.0, 3.0], [7.0, 4.0, 2.0]], [[7.0, 5.0, 
3.0], [6.0, 4.0, 5.0], [7.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 7.0], [3.0, 7.0, 8.0], [3.0, 5.0, 1.0]], 
[[1.0, 1.0, 1.0], [2.0, 5.0, 8.0], [3.0, 3.0, 4.0]], [[2.0, 2.0, 3.0], [6.0, 3.0, 5.0], [3.0, 7.0, 4.0]]], [[[4.0, 6.0, 7.0], 
[5.0, 6.0, 3.0], [1.0, 5.0, 6.0]], [[4.0, 3.0, 3.0], [5.0, 1.0, 7.0], [6.0, 8.0, 5.0]], [[6.0, 3.0, 2.0], [2.0, 3.0, 5.0], 
[4.0, 2.0, 3.0]]], [[[1.0, 2.0, 5.0], [3.0, 6.0, 8.0], [6.0, 5.0, 2.0]], [[8.0, 2.0, 4.0], [3.0, 5.0, 1.0], [2.0, 6.0, 1.0]], 
[[5.0, 8.0, 8.0], [6.0, 2.0, 8.0], [4.0, 3.0, 4.0]]], [[[1.0, 4.0, 8.0], [5.0, 7.0, 7.0], [7.0, 4.0, 5.0]], [[8.0, 8.0, 4.0], 
[1.0, 3.0, 8.0], [2.0, 3.0, 8.0]], [[5.0, 2.0, 5.0], [6.0, 7.0, 8.0], [1.0, 7.0, 6.0]]]])
      val=numpy.array([[[[6.0, 1.0, 1.0]], [[3.0, 6.0, 7.0]]], [[[4.0, 2.0, 5.0]], [[4.0, 2.0, 1.0]]], [[[1.0, 8.0, 4.0]], 
[[2.0, 2.0, 4.0]]], [[[7.0, 4.0, 4.0]], [[4.0, 5.0, 4.0]]]])
      arg[:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 6.0], [3.0, 2.0, 5.0], [6.0, 1.0, 1.0]], [[4.0, 2.0, 4.0], [6.0, 2.0, 2.0], [3.0, 
6.0, 7.0]], [[3.0, 6.0, 5.0], [7.0, 6.0, 8.0], [7.0, 6.0, 1.0]]], [[[2.0, 2.0, 4.0], [2.0, 1.0, 6.0], [4.0, 2.0, 5.0]], [[1.0, 
1.0, 1.0], [2.0, 8.0, 4.0], [4.0, 2.0, 1.0]], [[1.0, 5.0, 2.0], [1.0, 6.0, 8.0], [8.0, 5.0, 1.0]]], [[[8.0, 4.0, 8.0], [6.0, 
5.0, 4.0], [1.0, 8.0, 4.0]], [[5.0, 6.0, 8.0], [2.0, 5.0, 8.0], [2.0, 2.0, 4.0]], [[8.0, 8.0, 3.0], [5.0, 7.0, 4.0], [5.0, 4.0, 
2.0]]], [[[1.0, 3.0, 1.0], [2.0, 6.0, 1.0], [7.0, 4.0, 4.0]], [[4.0, 4.0, 2.0], [1.0, 6.0, 3.0], [4.0, 5.0, 4.0]], [[7.0, 5.0, 
3.0], [6.0, 4.0, 5.0], [7.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 7.0], [3.0, 7.0, 8.0], [6.0, 1.0, 1.0]], 
[[1.0, 1.0, 1.0], [2.0, 5.0, 8.0], [3.0, 6.0, 7.0]], [[2.0, 2.0, 3.0], [6.0, 3.0, 5.0], [3.0, 7.0, 4.0]]], [[[4.0, 6.0, 7.0], 
[5.0, 6.0, 3.0], [4.0, 2.0, 5.0]], [[4.0, 3.0, 3.0], [5.0, 1.0, 7.0], [4.0, 2.0, 1.0]], [[6.0, 3.0, 2.0], [2.0, 3.0, 5.0], 
[4.0, 2.0, 3.0]]], [[[1.0, 2.0, 5.0], [3.0, 6.0, 8.0], [1.0, 8.0, 4.0]], [[8.0, 2.0, 4.0], [3.0, 5.0, 1.0], [2.0, 2.0, 4.0]], 
[[5.0, 8.0, 8.0], [6.0, 2.0, 8.0], [4.0, 3.0, 4.0]]], [[[1.0, 4.0, 8.0], [5.0, 7.0, 7.0], [7.0, 4.0, 4.0]], [[8.0, 8.0, 4.0], 
[1.0, 3.0, 8.0], [4.0, 5.0, 4.0]], [[5.0, 2.0, 5.0], [6.0, 7.0, 8.0], [1.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 6.0], [4.0, 4.0, 4.0], [6.0, 2.0, 8.0]], [[3.0, 5.0, 4.0], [6.0, 6.0, 6.0], [8.0, 
4.0, 1.0]], [[7.0, 5.0, 8.0], [8.0, 1.0, 6.0], [8.0, 4.0, 2.0]]], [[[8.0, 3.0, 5.0], [5.0, 5.0, 5.0], [3.0, 5.0, 5.0]], [[3.0, 
2.0, 7.0], [4.0, 4.0, 7.0], [4.0, 4.0, 4.0]], [[1.0, 6.0, 2.0], [1.0, 3.0, 3.0], [8.0, 2.0, 3.0]]], [[[7.0, 3.0, 3.0], [8.0, 
7.0, 8.0], [7.0, 2.0, 3.0]], [[3.0, 7.0, 5.0], [1.0, 5.0, 3.0], [2.0, 4.0, 7.0]], [[4.0, 2.0, 2.0], [6.0, 3.0, 7.0], [3.0, 7.0, 
4.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 5.0], [2.0, 6.0, 3.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 4.0], [4.0, 6.0, 4.0]], [[2.0, 7.0, 
2.0], [5.0, 8.0, 5.0], [8.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 6.0], [2.0, 8.0, 2.0], [5.0, 4.0, 4.0]], 
[[1.0, 1.0, 7.0], [4.0, 5.0, 1.0], [2.0, 6.0, 8.0]], [[4.0, 6.0, 2.0], [1.0, 6.0, 8.0], [2.0, 5.0, 5.0]]], [[[7.0, 6.0, 8.0], 
[1.0, 8.0, 3.0], [1.0, 5.0, 4.0]], [[1.0, 1.0, 7.0], [4.0, 5.0, 7.0], [1.0, 6.0, 2.0]], [[7.0, 8.0, 8.0], [1.0, 8.0, 2.0], 
[6.0, 1.0, 3.0]]], [[[1.0, 5.0, 3.0], [2.0, 5.0, 8.0], [4.0, 1.0, 4.0]], [[6.0, 8.0, 2.0], [3.0, 3.0, 1.0], [5.0, 8.0, 3.0]], 
[[3.0, 2.0, 4.0], [1.0, 5.0, 3.0], [6.0, 7.0, 2.0]]], [[[6.0, 2.0, 8.0], [6.0, 2.0, 1.0], [4.0, 4.0, 5.0]], [[1.0, 8.0, 7.0], 
[1.0, 8.0, 5.0], [2.0, 8.0, 5.0]], [[6.0, 3.0, 5.0], [1.0, 8.0, 5.0], [5.0, 3.0, 8.0]]]])
      val=Data(numpy.array([[[[5.0, 2.0, 4.0]], [[1.0, 8.0, 6.0]]], [[[5.0, 3.0, 1.0]], [[8.0, 7.0, 1.0]]], [[[5.0, 4.0, 4.0]], 
[[2.0, 6.0, 7.0]]], [[[5.0, 5.0, 4.0]], [[4.0, 7.0, 7.0]]]]),self.functionspace)
      arg[:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 6.0], [4.0, 4.0, 4.0], [5.0, 2.0, 4.0]], [[3.0, 5.0, 4.0], [6.0, 6.0, 6.0], [1.0, 
8.0, 6.0]], [[7.0, 5.0, 8.0], [8.0, 1.0, 6.0], [8.0, 4.0, 2.0]]], [[[8.0, 3.0, 5.0], [5.0, 5.0, 5.0], [5.0, 3.0, 1.0]], [[3.0, 
2.0, 7.0], [4.0, 4.0, 7.0], [8.0, 7.0, 1.0]], [[1.0, 6.0, 2.0], [1.0, 3.0, 3.0], [8.0, 2.0, 3.0]]], [[[7.0, 3.0, 3.0], [8.0, 
7.0, 8.0], [5.0, 4.0, 4.0]], [[3.0, 7.0, 5.0], [1.0, 5.0, 3.0], [2.0, 6.0, 7.0]], [[4.0, 2.0, 2.0], [6.0, 3.0, 7.0], [3.0, 7.0, 
4.0]]], [[[8.0, 6.0, 8.0], [7.0, 1.0, 5.0], [5.0, 5.0, 4.0]], [[2.0, 1.0, 7.0], [2.0, 8.0, 4.0], [4.0, 7.0, 7.0]], [[2.0, 7.0, 
2.0], [5.0, 8.0, 5.0], [8.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 6.0], [2.0, 8.0, 2.0], [5.0, 2.0, 4.0]], 
[[1.0, 1.0, 7.0], [4.0, 5.0, 1.0], [1.0, 8.0, 6.0]], [[4.0, 6.0, 2.0], [1.0, 6.0, 8.0], [2.0, 5.0, 5.0]]], [[[7.0, 6.0, 8.0], 
[1.0, 8.0, 3.0], [5.0, 3.0, 1.0]], [[1.0, 1.0, 7.0], [4.0, 5.0, 7.0], [8.0, 7.0, 1.0]], [[7.0, 8.0, 8.0], [1.0, 8.0, 2.0], 
[6.0, 1.0, 3.0]]], [[[1.0, 5.0, 3.0], [2.0, 5.0, 8.0], [5.0, 4.0, 4.0]], [[6.0, 8.0, 2.0], [3.0, 3.0, 1.0], [2.0, 6.0, 7.0]], 
[[3.0, 2.0, 4.0], [1.0, 5.0, 3.0], [6.0, 7.0, 2.0]]], [[[6.0, 2.0, 8.0], [6.0, 2.0, 1.0], [5.0, 5.0, 4.0]], [[1.0, 8.0, 7.0], 
[1.0, 8.0, 5.0], [4.0, 7.0, 7.0]], [[6.0, 3.0, 5.0], [1.0, 8.0, 5.0], [5.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 7.0], [2.0, 3.0, 4.0], [7.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [8.0, 7.0, 6.0], [4.0, 
6.0, 8.0]], [[7.0, 2.0, 1.0], [1.0, 3.0, 1.0], [8.0, 5.0, 2.0]]], [[[5.0, 7.0, 3.0], [3.0, 8.0, 8.0], [4.0, 8.0, 2.0]], [[7.0, 
6.0, 2.0], [3.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[2.0, 7.0, 2.0], [1.0, 2.0, 8.0], [2.0, 6.0, 7.0]]], [[[3.0, 5.0, 1.0], [1.0, 
3.0, 4.0], [6.0, 2.0, 2.0]], [[3.0, 7.0, 6.0], [6.0, 3.0, 2.0], [5.0, 6.0, 5.0]], [[7.0, 6.0, 5.0], [3.0, 3.0, 6.0], [8.0, 8.0, 
3.0]]], [[[2.0, 4.0, 1.0], [6.0, 4.0, 2.0], [5.0, 5.0, 7.0]], [[3.0, 8.0, 4.0], [1.0, 1.0, 4.0], [8.0, 5.0, 6.0]], [[6.0, 4.0, 
6.0], [7.0, 1.0, 4.0], [3.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 1.0], [1.0, 6.0, 6.0], [6.0, 4.0, 4.0]], 
[[4.0, 3.0, 7.0], [8.0, 6.0, 2.0], [3.0, 4.0, 8.0]], [[4.0, 8.0, 3.0], [7.0, 5.0, 4.0], [8.0, 4.0, 3.0]]], [[[5.0, 3.0, 8.0], 
[3.0, 1.0, 7.0], [8.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [2.0, 2.0, 5.0], [6.0, 6.0, 2.0]], [[8.0, 6.0, 3.0], [3.0, 2.0, 6.0], 
[8.0, 1.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 7.0, 8.0], [6.0, 3.0, 7.0]], [[3.0, 8.0, 5.0], [7.0, 5.0, 5.0], [4.0, 6.0, 8.0]], 
[[2.0, 5.0, 1.0], [6.0, 5.0, 2.0], [5.0, 4.0, 2.0]]], [[[2.0, 2.0, 2.0], [6.0, 4.0, 7.0], [3.0, 5.0, 2.0]], [[7.0, 4.0, 3.0], 
[1.0, 5.0, 1.0], [6.0, 8.0, 1.0]], [[7.0, 8.0, 4.0], [3.0, 5.0, 7.0], [7.0, 1.0, 3.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 3.0, 4.0]], [[2.0, 6.0, 6.0]]], [[[4.0, 6.0, 7.0]], [[4.0, 5.0, 5.0]]], [[[7.0, 4.0, 
7.0]], [[4.0, 2.0, 5.0]]], [[[1.0, 7.0, 4.0]], [[5.0, 2.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 6.0, 2.0]], [[2.0, 3.0, 
8.0]]], [[[3.0, 4.0, 1.0]], [[1.0, 5.0, 1.0]]], [[[2.0, 4.0, 5.0]], [[4.0, 3.0, 2.0]]], [[[1.0, 7.0, 6.0]], [[5.0, 8.0, 5.0]]]])
      arg[:,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 7.0], [2.0, 3.0, 4.0], [2.0, 3.0, 4.0]], [[5.0, 7.0, 6.0], [8.0, 7.0, 6.0], [2.0, 
6.0, 6.0]], [[7.0, 2.0, 1.0], [1.0, 3.0, 1.0], [8.0, 5.0, 2.0]]], [[[5.0, 7.0, 3.0], [3.0, 8.0, 8.0], [4.0, 6.0, 7.0]], [[7.0, 
6.0, 2.0], [3.0, 4.0, 6.0], [4.0, 5.0, 5.0]], [[2.0, 7.0, 2.0], [1.0, 2.0, 8.0], [2.0, 6.0, 7.0]]], [[[3.0, 5.0, 1.0], [1.0, 
3.0, 4.0], [7.0, 4.0, 7.0]], [[3.0, 7.0, 6.0], [6.0, 3.0, 2.0], [4.0, 2.0, 5.0]], [[7.0, 6.0, 5.0], [3.0, 3.0, 6.0], [8.0, 8.0, 
3.0]]], [[[2.0, 4.0, 1.0], [6.0, 4.0, 2.0], [1.0, 7.0, 4.0]], [[3.0, 8.0, 4.0], [1.0, 1.0, 4.0], [5.0, 2.0, 1.0]], [[6.0, 4.0, 
6.0], [7.0, 1.0, 4.0], [3.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 1.0], [1.0, 6.0, 6.0], [6.0, 6.0, 2.0]], 
[[4.0, 3.0, 7.0], [8.0, 6.0, 2.0], [2.0, 3.0, 8.0]], [[4.0, 8.0, 3.0], [7.0, 5.0, 4.0], [8.0, 4.0, 3.0]]], [[[5.0, 3.0, 8.0], 
[3.0, 1.0, 7.0], [3.0, 4.0, 1.0]], [[4.0, 3.0, 1.0], [2.0, 2.0, 5.0], [1.0, 5.0, 1.0]], [[8.0, 6.0, 3.0], [3.0, 2.0, 6.0], 
[8.0, 1.0, 1.0]]], [[[3.0, 1.0, 8.0], [3.0, 7.0, 8.0], [2.0, 4.0, 5.0]], [[3.0, 8.0, 5.0], [7.0, 5.0, 5.0], [4.0, 3.0, 2.0]], 
[[2.0, 5.0, 1.0], [6.0, 5.0, 2.0], [5.0, 4.0, 2.0]]], [[[2.0, 2.0, 2.0], [6.0, 4.0, 7.0], [1.0, 7.0, 6.0]], [[7.0, 4.0, 3.0], 
[1.0, 5.0, 1.0], [5.0, 8.0, 5.0]], [[7.0, 8.0, 4.0], [3.0, 5.0, 7.0], [7.0, 1.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 4.0], [3.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 2.0], [5.0, 
5.0, 5.0]], [[8.0, 1.0, 1.0], [6.0, 1.0, 6.0], [7.0, 4.0, 5.0]]], [[[5.0, 3.0, 1.0], [7.0, 3.0, 4.0], [5.0, 3.0, 2.0]], [[4.0, 
7.0, 3.0], [8.0, 6.0, 3.0], [7.0, 6.0, 8.0]], [[7.0, 7.0, 5.0], [4.0, 8.0, 3.0], [3.0, 4.0, 6.0]]], [[[1.0, 5.0, 7.0], [8.0, 
5.0, 8.0], [1.0, 8.0, 6.0]], [[3.0, 8.0, 7.0], [5.0, 7.0, 5.0], [7.0, 2.0, 2.0]], [[6.0, 8.0, 2.0], [3.0, 8.0, 1.0], [2.0, 1.0, 
4.0]]], [[[4.0, 7.0, 5.0], [6.0, 2.0, 7.0], [1.0, 7.0, 8.0]], [[6.0, 5.0, 8.0], [8.0, 6.0, 6.0], [8.0, 8.0, 3.0]], [[8.0, 8.0, 
4.0], [8.0, 4.0, 7.0], [6.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 6.0], [3.0, 1.0, 8.0], [1.0, 3.0, 1.0]], 
[[3.0, 7.0, 3.0], [1.0, 6.0, 2.0], [2.0, 6.0, 6.0]], [[5.0, 1.0, 5.0], [3.0, 1.0, 3.0], [3.0, 6.0, 8.0]]], [[[3.0, 4.0, 2.0], 
[1.0, 2.0, 3.0], [7.0, 8.0, 2.0]], [[5.0, 7.0, 2.0], [1.0, 8.0, 5.0], [5.0, 6.0, 2.0]], [[7.0, 4.0, 8.0], [1.0, 2.0, 6.0], 
[2.0, 8.0, 3.0]]], [[[6.0, 5.0, 1.0], [5.0, 5.0, 5.0], [8.0, 4.0, 4.0]], [[5.0, 6.0, 1.0], [3.0, 6.0, 8.0], [8.0, 7.0, 7.0]], 
[[3.0, 3.0, 4.0], [5.0, 4.0, 1.0], [8.0, 6.0, 3.0]]], [[[1.0, 5.0, 4.0], [1.0, 2.0, 8.0], [6.0, 1.0, 7.0]], [[5.0, 8.0, 1.0], 
[1.0, 3.0, 6.0], [7.0, 5.0, 6.0]], [[6.0, 8.0, 5.0], [1.0, 6.0, 5.0], [1.0, 6.0, 5.0]]]])
      val=numpy.array([[[7.0, 8.0, 3.0], [3.0, 4.0, 1.0], [8.0, 2.0, 4.0]], [[3.0, 3.0, 8.0], [8.0, 1.0, 3.0], [5.0, 5.0, 
7.0]]])
      arg[2:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 4.0], [3.0, 1.0, 4.0], [4.0, 7.0, 4.0]], [[7.0, 7.0, 6.0], [3.0, 6.0, 2.0], [5.0, 
5.0, 5.0]], [[8.0, 1.0, 1.0], [6.0, 1.0, 6.0], [7.0, 4.0, 5.0]]], [[[5.0, 3.0, 1.0], [7.0, 3.0, 4.0], [5.0, 3.0, 2.0]], [[4.0, 
7.0, 3.0], [8.0, 6.0, 3.0], [7.0, 6.0, 8.0]], [[7.0, 7.0, 5.0], [4.0, 8.0, 3.0], [3.0, 4.0, 6.0]]], [[[1.0, 5.0, 7.0], [8.0, 
5.0, 8.0], [1.0, 8.0, 6.0]], [[7.0, 8.0, 3.0], [3.0, 4.0, 1.0], [8.0, 2.0, 4.0]], [[6.0, 8.0, 2.0], [3.0, 8.0, 1.0], [2.0, 1.0, 
4.0]]], [[[4.0, 7.0, 5.0], [6.0, 2.0, 7.0], [1.0, 7.0, 8.0]], [[3.0, 3.0, 8.0], [8.0, 1.0, 3.0], [5.0, 5.0, 7.0]], [[8.0, 8.0, 
4.0], [8.0, 4.0, 7.0], [6.0, 1.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 6.0], [3.0, 1.0, 8.0], [1.0, 3.0, 1.0]], 
[[3.0, 7.0, 3.0], [1.0, 6.0, 2.0], [2.0, 6.0, 6.0]], [[5.0, 1.0, 5.0], [3.0, 1.0, 3.0], [3.0, 6.0, 8.0]]], [[[3.0, 4.0, 2.0], 
[1.0, 2.0, 3.0], [7.0, 8.0, 2.0]], [[5.0, 7.0, 2.0], [1.0, 8.0, 5.0], [5.0, 6.0, 2.0]], [[7.0, 4.0, 8.0], [1.0, 2.0, 6.0], 
[2.0, 8.0, 3.0]]], [[[6.0, 5.0, 1.0], [5.0, 5.0, 5.0], [8.0, 4.0, 4.0]], [[7.0, 8.0, 3.0], [3.0, 4.0, 1.0], [8.0, 2.0, 4.0]], 
[[3.0, 3.0, 4.0], [5.0, 4.0, 1.0], [8.0, 6.0, 3.0]]], [[[1.0, 5.0, 4.0], [1.0, 2.0, 8.0], [6.0, 1.0, 7.0]], [[3.0, 3.0, 8.0], 
[8.0, 1.0, 3.0], [5.0, 5.0, 7.0]], [[6.0, 8.0, 5.0], [1.0, 6.0, 5.0], [1.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 7.0], [6.0, 3.0, 6.0], [2.0, 5.0, 7.0]], [[4.0, 8.0, 2.0], [4.0, 2.0, 7.0], [4.0, 
2.0, 1.0]], [[6.0, 7.0, 7.0], [3.0, 8.0, 2.0], [6.0, 1.0, 3.0]]], [[[2.0, 5.0, 5.0], [1.0, 8.0, 4.0], [8.0, 1.0, 6.0]], [[7.0, 
6.0, 5.0], [4.0, 5.0, 5.0], [7.0, 4.0, 7.0]], [[8.0, 1.0, 8.0], [8.0, 8.0, 4.0], [7.0, 6.0, 1.0]]], [[[3.0, 1.0, 6.0], [1.0, 
3.0, 1.0], [4.0, 8.0, 6.0]], [[4.0, 7.0, 2.0], [3.0, 7.0, 5.0], [7.0, 6.0, 2.0]], [[8.0, 5.0, 2.0], [4.0, 4.0, 8.0], [8.0, 6.0, 
2.0]]], [[[4.0, 1.0, 3.0], [3.0, 4.0, 8.0], [5.0, 4.0, 5.0]], [[8.0, 1.0, 6.0], [6.0, 3.0, 3.0], [3.0, 8.0, 3.0]], [[4.0, 1.0, 
3.0], [5.0, 5.0, 5.0], [8.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 4.0], [1.0, 3.0, 8.0], [4.0, 4.0, 2.0]], 
[[5.0, 3.0, 3.0], [3.0, 4.0, 7.0], [7.0, 2.0, 3.0]], [[4.0, 6.0, 4.0], [3.0, 6.0, 7.0], [7.0, 5.0, 8.0]]], [[[8.0, 4.0, 6.0], 
[4.0, 5.0, 6.0], [2.0, 7.0, 5.0]], [[2.0, 6.0, 1.0], [4.0, 7.0, 4.0], [6.0, 3.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 6.0, 1.0], 
[7.0, 3.0, 5.0]]], [[[2.0, 2.0, 7.0], [1.0, 6.0, 6.0], [4.0, 6.0, 8.0]], [[7.0, 1.0, 7.0], [7.0, 3.0, 7.0], [1.0, 3.0, 5.0]], 
[[4.0, 1.0, 6.0], [5.0, 2.0, 8.0], [8.0, 6.0, 6.0]]], [[[8.0, 5.0, 1.0], [7.0, 1.0, 7.0], [4.0, 8.0, 6.0]], [[4.0, 5.0, 2.0], 
[4.0, 4.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 6.0, 5.0], [2.0, 6.0, 8.0], [7.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[6.0, 5.0, 3.0], [6.0, 4.0, 4.0], [5.0, 4.0, 2.0]], [[5.0, 8.0, 8.0], [4.0, 7.0, 5.0], [4.0, 3.0, 
2.0]]]),self.functionspace)
      arg[2:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [6.0, 3.0, 6.0], [2.0, 5.0, 7.0]], [[4.0, 8.0, 2.0], [4.0, 2.0, 7.0], [4.0, 
2.0, 1.0]], [[6.0, 7.0, 7.0], [3.0, 8.0, 2.0], [6.0, 1.0, 3.0]]], [[[2.0, 5.0, 5.0], [1.0, 8.0, 4.0], [8.0, 1.0, 6.0]], [[7.0, 
6.0, 5.0], [4.0, 5.0, 5.0], [7.0, 4.0, 7.0]], [[8.0, 1.0, 8.0], [8.0, 8.0, 4.0], [7.0, 6.0, 1.0]]], [[[3.0, 1.0, 6.0], [1.0, 
3.0, 1.0], [4.0, 8.0, 6.0]], [[6.0, 5.0, 3.0], [6.0, 4.0, 4.0], [5.0, 4.0, 2.0]], [[8.0, 5.0, 2.0], [4.0, 4.0, 8.0], [8.0, 6.0, 
2.0]]], [[[4.0, 1.0, 3.0], [3.0, 4.0, 8.0], [5.0, 4.0, 5.0]], [[5.0, 8.0, 8.0], [4.0, 7.0, 5.0], [4.0, 3.0, 2.0]], [[4.0, 1.0, 
3.0], [5.0, 5.0, 5.0], [8.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 4.0], [1.0, 3.0, 8.0], [4.0, 4.0, 2.0]], 
[[5.0, 3.0, 3.0], [3.0, 4.0, 7.0], [7.0, 2.0, 3.0]], [[4.0, 6.0, 4.0], [3.0, 6.0, 7.0], [7.0, 5.0, 8.0]]], [[[8.0, 4.0, 6.0], 
[4.0, 5.0, 6.0], [2.0, 7.0, 5.0]], [[2.0, 6.0, 1.0], [4.0, 7.0, 4.0], [6.0, 3.0, 4.0]], [[8.0, 4.0, 3.0], [2.0, 6.0, 1.0], 
[7.0, 3.0, 5.0]]], [[[2.0, 2.0, 7.0], [1.0, 6.0, 6.0], [4.0, 6.0, 8.0]], [[6.0, 5.0, 3.0], [6.0, 4.0, 4.0], [5.0, 4.0, 2.0]], 
[[4.0, 1.0, 6.0], [5.0, 2.0, 8.0], [8.0, 6.0, 6.0]]], [[[8.0, 5.0, 1.0], [7.0, 1.0, 7.0], [4.0, 8.0, 6.0]], [[5.0, 8.0, 8.0], 
[4.0, 7.0, 5.0], [4.0, 3.0, 2.0]], [[6.0, 6.0, 5.0], [2.0, 6.0, 8.0], [7.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 3.0], [4.0, 3.0, 1.0], [8.0, 8.0, 7.0]], [[8.0, 1.0, 5.0], [2.0, 3.0, 6.0], [7.0, 
5.0, 6.0]], [[4.0, 1.0, 4.0], [8.0, 6.0, 7.0], [6.0, 7.0, 1.0]]], [[[5.0, 3.0, 2.0], [6.0, 1.0, 1.0], [4.0, 3.0, 3.0]], [[8.0, 
5.0, 8.0], [2.0, 1.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 5.0], [6.0, 3.0, 1.0]]], [[[5.0, 7.0, 8.0], [1.0, 
6.0, 2.0], [1.0, 8.0, 7.0]], [[3.0, 1.0, 7.0], [1.0, 5.0, 4.0], [6.0, 6.0, 8.0]], [[6.0, 3.0, 4.0], [3.0, 2.0, 8.0], [4.0, 4.0, 
2.0]]], [[[3.0, 8.0, 2.0], [4.0, 1.0, 2.0], [5.0, 2.0, 6.0]], [[4.0, 2.0, 1.0], [6.0, 4.0, 2.0], [3.0, 8.0, 2.0]], [[5.0, 6.0, 
2.0], [6.0, 3.0, 3.0], [6.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 1.0], [1.0, 3.0, 2.0], [5.0, 2.0, 3.0]], 
[[2.0, 4.0, 8.0], [3.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[3.0, 7.0, 2.0], [2.0, 5.0, 5.0], [8.0, 6.0, 6.0]]], [[[3.0, 2.0, 7.0], 
[7.0, 2.0, 5.0], [3.0, 1.0, 7.0]], [[4.0, 6.0, 1.0], [7.0, 3.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 8.0, 5.0], [4.0, 1.0, 6.0], 
[5.0, 1.0, 3.0]]], [[[2.0, 2.0, 7.0], [5.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[1.0, 8.0, 1.0], [1.0, 6.0, 5.0], [6.0, 3.0, 8.0]], 
[[1.0, 6.0, 2.0], [5.0, 5.0, 1.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 3.0], [5.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[2.0, 5.0, 1.0], 
[2.0, 7.0, 6.0], [3.0, 6.0, 8.0]], [[1.0, 7.0, 2.0], [1.0, 5.0, 5.0], [2.0, 2.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 8.0, 4.0], [7.0, 6.0, 8.0], [8.0, 3.0, 5.0]], [[6.0, 5.0, 7.0], [6.0, 2.0, 1.0], [3.0, 
8.0, 4.0]]])+(1.-msk_val)*numpy.array([[[7.0, 2.0, 7.0], [5.0, 2.0, 1.0], [7.0, 8.0, 8.0]], [[8.0, 5.0, 6.0], [5.0, 4.0, 3.0], 
[4.0, 8.0, 1.0]]])
      arg[2:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 3.0], [4.0, 3.0, 1.0], [8.0, 8.0, 7.0]], [[8.0, 1.0, 5.0], [2.0, 3.0, 6.0], [7.0, 
5.0, 6.0]], [[4.0, 1.0, 4.0], [8.0, 6.0, 7.0], [6.0, 7.0, 1.0]]], [[[5.0, 3.0, 2.0], [6.0, 1.0, 1.0], [4.0, 3.0, 3.0]], [[8.0, 
5.0, 8.0], [2.0, 1.0, 6.0], [1.0, 2.0, 5.0]], [[6.0, 6.0, 2.0], [7.0, 5.0, 5.0], [6.0, 3.0, 1.0]]], [[[5.0, 7.0, 8.0], [1.0, 
6.0, 2.0], [1.0, 8.0, 7.0]], [[6.0, 8.0, 4.0], [7.0, 6.0, 8.0], [8.0, 3.0, 5.0]], [[6.0, 3.0, 4.0], [3.0, 2.0, 8.0], [4.0, 4.0, 
2.0]]], [[[3.0, 8.0, 2.0], [4.0, 1.0, 2.0], [5.0, 2.0, 6.0]], [[6.0, 5.0, 7.0], [6.0, 2.0, 1.0], [3.0, 8.0, 4.0]], [[5.0, 6.0, 
2.0], [6.0, 3.0, 3.0], [6.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 1.0, 1.0], [1.0, 3.0, 2.0], [5.0, 2.0, 3.0]], 
[[2.0, 4.0, 8.0], [3.0, 8.0, 4.0], [3.0, 6.0, 7.0]], [[3.0, 7.0, 2.0], [2.0, 5.0, 5.0], [8.0, 6.0, 6.0]]], [[[3.0, 2.0, 7.0], 
[7.0, 2.0, 5.0], [3.0, 1.0, 7.0]], [[4.0, 6.0, 1.0], [7.0, 3.0, 1.0], [4.0, 4.0, 6.0]], [[5.0, 8.0, 5.0], [4.0, 1.0, 6.0], 
[5.0, 1.0, 3.0]]], [[[2.0, 2.0, 7.0], [5.0, 2.0, 8.0], [1.0, 1.0, 1.0]], [[7.0, 2.0, 7.0], [5.0, 2.0, 1.0], [7.0, 8.0, 8.0]], 
[[1.0, 6.0, 2.0], [5.0, 5.0, 1.0], [3.0, 6.0, 8.0]]], [[[1.0, 5.0, 3.0], [5.0, 7.0, 3.0], [4.0, 5.0, 6.0]], [[8.0, 5.0, 6.0], 
[5.0, 4.0, 3.0], [4.0, 8.0, 1.0]], [[1.0, 7.0, 2.0], [1.0, 5.0, 5.0], [2.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 4.0], [3.0, 7.0, 3.0], [6.0, 6.0, 7.0]], [[8.0, 7.0, 4.0], [3.0, 1.0, 7.0], [3.0, 
5.0, 7.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 1.0], [5.0, 8.0, 5.0]]], [[[5.0, 3.0, 1.0], [3.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[6.0, 
1.0, 2.0], [1.0, 7.0, 1.0], [7.0, 1.0, 7.0]], [[6.0, 7.0, 5.0], [4.0, 4.0, 7.0], [1.0, 4.0, 8.0]]], [[[1.0, 4.0, 1.0], [4.0, 
2.0, 6.0], [3.0, 4.0, 5.0]], [[8.0, 4.0, 6.0], [1.0, 3.0, 7.0], [2.0, 4.0, 8.0]], [[4.0, 6.0, 6.0], [4.0, 2.0, 4.0], [8.0, 5.0, 
7.0]]], [[[8.0, 5.0, 7.0], [5.0, 7.0, 8.0], [6.0, 3.0, 7.0]], [[4.0, 8.0, 6.0], [1.0, 4.0, 7.0], [2.0, 5.0, 6.0]], [[7.0, 3.0, 
2.0], [6.0, 4.0, 2.0], [3.0, 7.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 5.0], [6.0, 2.0, 5.0], [8.0, 8.0, 6.0]], 
[[7.0, 4.0, 4.0], [8.0, 2.0, 3.0], [7.0, 8.0, 7.0]], [[5.0, 5.0, 5.0], [7.0, 3.0, 2.0], [1.0, 6.0, 3.0]]], [[[8.0, 3.0, 8.0], 
[8.0, 4.0, 7.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 5.0], [8.0, 2.0, 3.0], [3.0, 8.0, 3.0]], [[7.0, 5.0, 8.0], [6.0, 3.0, 7.0], 
[1.0, 4.0, 8.0]]], [[[2.0, 1.0, 7.0], [3.0, 4.0, 4.0], [1.0, 3.0, 2.0]], [[1.0, 6.0, 6.0], [7.0, 2.0, 4.0], [4.0, 6.0, 7.0]], 
[[7.0, 2.0, 1.0], [4.0, 6.0, 5.0], [1.0, 7.0, 1.0]]], [[[4.0, 8.0, 7.0], [5.0, 5.0, 5.0], [7.0, 4.0, 4.0]], [[2.0, 5.0, 5.0], 
[5.0, 3.0, 7.0], [5.0, 1.0, 2.0]], [[3.0, 8.0, 3.0], [2.0, 2.0, 2.0], [1.0, 8.0, 1.0]]]])
      val=numpy.array([[[6.0, 5.0, 5.0]], [[8.0, 2.0, 5.0]]])
      arg[1:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 4.0], [3.0, 7.0, 3.0], [6.0, 6.0, 7.0]], [[8.0, 7.0, 4.0], [3.0, 1.0, 7.0], [3.0, 
5.0, 7.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 1.0], [5.0, 8.0, 5.0]]], [[[5.0, 3.0, 1.0], [3.0, 3.0, 5.0], [2.0, 7.0, 6.0]], [[6.0, 
1.0, 2.0], [1.0, 7.0, 1.0], [6.0, 5.0, 5.0]], [[6.0, 7.0, 5.0], [4.0, 4.0, 7.0], [1.0, 4.0, 8.0]]], [[[1.0, 4.0, 1.0], [4.0, 
2.0, 6.0], [3.0, 4.0, 5.0]], [[8.0, 4.0, 6.0], [1.0, 3.0, 7.0], [8.0, 2.0, 5.0]], [[4.0, 6.0, 6.0], [4.0, 2.0, 4.0], [8.0, 5.0, 
7.0]]], [[[8.0, 5.0, 7.0], [5.0, 7.0, 8.0], [6.0, 3.0, 7.0]], [[4.0, 8.0, 6.0], [1.0, 4.0, 7.0], [2.0, 5.0, 6.0]], [[7.0, 3.0, 
2.0], [6.0, 4.0, 2.0], [3.0, 7.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 5.0], [6.0, 2.0, 5.0], [8.0, 8.0, 6.0]], 
[[7.0, 4.0, 4.0], [8.0, 2.0, 3.0], [7.0, 8.0, 7.0]], [[5.0, 5.0, 5.0], [7.0, 3.0, 2.0], [1.0, 6.0, 3.0]]], [[[8.0, 3.0, 8.0], 
[8.0, 4.0, 7.0], [6.0, 8.0, 3.0]], [[3.0, 7.0, 5.0], [8.0, 2.0, 3.0], [6.0, 5.0, 5.0]], [[7.0, 5.0, 8.0], [6.0, 3.0, 7.0], 
[1.0, 4.0, 8.0]]], [[[2.0, 1.0, 7.0], [3.0, 4.0, 4.0], [1.0, 3.0, 2.0]], [[1.0, 6.0, 6.0], [7.0, 2.0, 4.0], [8.0, 2.0, 5.0]], 
[[7.0, 2.0, 1.0], [4.0, 6.0, 5.0], [1.0, 7.0, 1.0]]], [[[4.0, 8.0, 7.0], [5.0, 5.0, 5.0], [7.0, 4.0, 4.0]], [[2.0, 5.0, 5.0], 
[5.0, 3.0, 7.0], [5.0, 1.0, 2.0]], [[3.0, 8.0, 3.0], [2.0, 2.0, 2.0], [1.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 3.0, 3.0], [7.0, 1.0, 3.0], [8.0, 4.0, 4.0]], [[1.0, 2.0, 3.0], [1.0, 6.0, 3.0], [8.0, 
2.0, 3.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 2.0], [5.0, 6.0, 7.0]]], [[[1.0, 5.0, 2.0], [8.0, 8.0, 5.0], [8.0, 4.0, 1.0]], [[7.0, 
7.0, 8.0], [6.0, 3.0, 1.0], [3.0, 3.0, 3.0]], [[6.0, 3.0, 7.0], [7.0, 1.0, 4.0], [5.0, 6.0, 2.0]]], [[[6.0, 2.0, 8.0], [3.0, 
8.0, 1.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], [4.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[3.0, 3.0, 7.0], [4.0, 4.0, 2.0], [1.0, 8.0, 
1.0]]], [[[3.0, 8.0, 2.0], [1.0, 2.0, 3.0], [2.0, 8.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[7.0, 5.0, 
7.0], [8.0, 4.0, 1.0], [8.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 7.0], [3.0, 4.0, 8.0], [2.0, 4.0, 1.0]], 
[[8.0, 6.0, 1.0], [7.0, 4.0, 8.0], [3.0, 3.0, 2.0]], [[6.0, 8.0, 8.0], [2.0, 5.0, 1.0], [6.0, 6.0, 1.0]]], [[[3.0, 4.0, 8.0], 
[6.0, 7.0, 2.0], [2.0, 6.0, 5.0]], [[3.0, 4.0, 6.0], [7.0, 8.0, 3.0], [5.0, 7.0, 2.0]], [[3.0, 7.0, 7.0], [5.0, 7.0, 6.0], 
[3.0, 5.0, 1.0]]], [[[7.0, 6.0, 5.0], [2.0, 7.0, 6.0], [4.0, 1.0, 7.0]], [[1.0, 2.0, 7.0], [7.0, 5.0, 5.0], [7.0, 3.0, 1.0]], 
[[6.0, 4.0, 6.0], [4.0, 6.0, 8.0], [2.0, 7.0, 5.0]]], [[[7.0, 5.0, 8.0], [2.0, 4.0, 3.0], [6.0, 3.0, 6.0]], [[2.0, 6.0, 1.0], 
[1.0, 2.0, 5.0], [3.0, 7.0, 6.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 3.0], [1.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[[7.0, 5.0, 8.0]], [[5.0, 7.0, 6.0]]]),self.functionspace)
      arg[1:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 3.0], [7.0, 1.0, 3.0], [8.0, 4.0, 4.0]], [[1.0, 2.0, 3.0], [1.0, 6.0, 3.0], [8.0, 
2.0, 3.0]], [[3.0, 5.0, 3.0], [5.0, 5.0, 2.0], [5.0, 6.0, 7.0]]], [[[1.0, 5.0, 2.0], [8.0, 8.0, 5.0], [8.0, 4.0, 1.0]], [[7.0, 
7.0, 8.0], [6.0, 3.0, 1.0], [7.0, 5.0, 8.0]], [[6.0, 3.0, 7.0], [7.0, 1.0, 4.0], [5.0, 6.0, 2.0]]], [[[6.0, 2.0, 8.0], [3.0, 
8.0, 1.0], [8.0, 2.0, 1.0]], [[2.0, 8.0, 2.0], [4.0, 7.0, 1.0], [5.0, 7.0, 6.0]], [[3.0, 3.0, 7.0], [4.0, 4.0, 2.0], [1.0, 8.0, 
1.0]]], [[[3.0, 8.0, 2.0], [1.0, 2.0, 3.0], [2.0, 8.0, 3.0]], [[8.0, 6.0, 4.0], [8.0, 4.0, 5.0], [2.0, 8.0, 6.0]], [[7.0, 5.0, 
7.0], [8.0, 4.0, 1.0], [8.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 7.0], [3.0, 4.0, 8.0], [2.0, 4.0, 1.0]], 
[[8.0, 6.0, 1.0], [7.0, 4.0, 8.0], [3.0, 3.0, 2.0]], [[6.0, 8.0, 8.0], [2.0, 5.0, 1.0], [6.0, 6.0, 1.0]]], [[[3.0, 4.0, 8.0], 
[6.0, 7.0, 2.0], [2.0, 6.0, 5.0]], [[3.0, 4.0, 6.0], [7.0, 8.0, 3.0], [7.0, 5.0, 8.0]], [[3.0, 7.0, 7.0], [5.0, 7.0, 6.0], 
[3.0, 5.0, 1.0]]], [[[7.0, 6.0, 5.0], [2.0, 7.0, 6.0], [4.0, 1.0, 7.0]], [[1.0, 2.0, 7.0], [7.0, 5.0, 5.0], [5.0, 7.0, 6.0]], 
[[6.0, 4.0, 6.0], [4.0, 6.0, 8.0], [2.0, 7.0, 5.0]]], [[[7.0, 5.0, 8.0], [2.0, 4.0, 3.0], [6.0, 3.0, 6.0]], [[2.0, 6.0, 1.0], 
[1.0, 2.0, 5.0], [3.0, 7.0, 6.0]], [[2.0, 8.0, 2.0], [5.0, 6.0, 3.0], [1.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 5.0, 7.0], [4.0, 1.0, 6.0], [5.0, 2.0, 6.0]], [[2.0, 7.0, 4.0], [6.0, 6.0, 7.0], [7.0, 
5.0, 1.0]], [[3.0, 4.0, 3.0], [6.0, 1.0, 2.0], [7.0, 6.0, 2.0]]], [[[1.0, 8.0, 5.0], [5.0, 1.0, 3.0], [6.0, 3.0, 8.0]], [[5.0, 
8.0, 7.0], [8.0, 6.0, 4.0], [6.0, 3.0, 8.0]], [[6.0, 7.0, 5.0], [4.0, 5.0, 8.0], [1.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [5.0, 
5.0, 1.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 2.0], [7.0, 3.0, 8.0], [2.0, 8.0, 4.0]], [[2.0, 3.0, 5.0], [3.0, 7.0, 2.0], [4.0, 2.0, 
2.0]]], [[[1.0, 2.0, 6.0], [2.0, 6.0, 1.0], [8.0, 8.0, 8.0]], [[5.0, 4.0, 1.0], [2.0, 2.0, 4.0], [5.0, 3.0, 7.0]], [[4.0, 1.0, 
8.0], [7.0, 6.0, 6.0], [1.0, 5.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 3.0], [5.0, 1.0, 5.0], [5.0, 4.0, 6.0]], 
[[1.0, 6.0, 1.0], [5.0, 8.0, 3.0], [5.0, 5.0, 4.0]], [[3.0, 7.0, 1.0], [5.0, 3.0, 4.0], [6.0, 7.0, 1.0]]], [[[7.0, 8.0, 8.0], 
[3.0, 2.0, 5.0], [7.0, 4.0, 2.0]], [[7.0, 8.0, 1.0], [2.0, 7.0, 6.0], [6.0, 6.0, 3.0]], [[1.0, 7.0, 6.0], [5.0, 4.0, 4.0], 
[6.0, 3.0, 5.0]]], [[[7.0, 3.0, 5.0], [7.0, 6.0, 6.0], [1.0, 5.0, 6.0]], [[1.0, 6.0, 3.0], [1.0, 5.0, 5.0], [4.0, 5.0, 6.0]], 
[[5.0, 3.0, 3.0], [8.0, 7.0, 7.0], [5.0, 6.0, 6.0]]], [[[6.0, 4.0, 7.0], [6.0, 6.0, 7.0], [7.0, 5.0, 6.0]], [[4.0, 4.0, 5.0], 
[8.0, 8.0, 3.0], [2.0, 7.0, 3.0]], [[4.0, 4.0, 2.0], [4.0, 1.0, 3.0], [6.0, 6.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 1.0, 2.0]], [[5.0, 6.0, 4.0]]])+(1.-msk_val)*numpy.array([[[1.0, 6.0, 7.0]], [[2.0, 3.0, 
8.0]]])
      arg[1:3,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, 7.0], [4.0, 1.0, 6.0], [5.0, 2.0, 6.0]], [[2.0, 7.0, 4.0], [6.0, 6.0, 7.0], [7.0, 
5.0, 1.0]], [[3.0, 4.0, 3.0], [6.0, 1.0, 2.0], [7.0, 6.0, 2.0]]], [[[1.0, 8.0, 5.0], [5.0, 1.0, 3.0], [6.0, 3.0, 8.0]], [[5.0, 
8.0, 7.0], [8.0, 6.0, 4.0], [3.0, 1.0, 2.0]], [[6.0, 7.0, 5.0], [4.0, 5.0, 8.0], [1.0, 7.0, 8.0]]], [[[1.0, 6.0, 1.0], [5.0, 
5.0, 1.0], [7.0, 2.0, 4.0]], [[8.0, 5.0, 2.0], [7.0, 3.0, 8.0], [5.0, 6.0, 4.0]], [[2.0, 3.0, 5.0], [3.0, 7.0, 2.0], [4.0, 2.0, 
2.0]]], [[[1.0, 2.0, 6.0], [2.0, 6.0, 1.0], [8.0, 8.0, 8.0]], [[5.0, 4.0, 1.0], [2.0, 2.0, 4.0], [5.0, 3.0, 7.0]], [[4.0, 1.0, 
8.0], [7.0, 6.0, 6.0], [1.0, 5.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 3.0], [5.0, 1.0, 5.0], [5.0, 4.0, 6.0]], 
[[1.0, 6.0, 1.0], [5.0, 8.0, 3.0], [5.0, 5.0, 4.0]], [[3.0, 7.0, 1.0], [5.0, 3.0, 4.0], [6.0, 7.0, 1.0]]], [[[7.0, 8.0, 8.0], 
[3.0, 2.0, 5.0], [7.0, 4.0, 2.0]], [[7.0, 8.0, 1.0], [2.0, 7.0, 6.0], [1.0, 6.0, 7.0]], [[1.0, 7.0, 6.0], [5.0, 4.0, 4.0], 
[6.0, 3.0, 5.0]]], [[[7.0, 3.0, 5.0], [7.0, 6.0, 6.0], [1.0, 5.0, 6.0]], [[1.0, 6.0, 3.0], [1.0, 5.0, 5.0], [2.0, 3.0, 8.0]], 
[[5.0, 3.0, 3.0], [8.0, 7.0, 7.0], [5.0, 6.0, 6.0]]], [[[6.0, 4.0, 7.0], [6.0, 6.0, 7.0], [7.0, 5.0, 6.0]], [[4.0, 4.0, 5.0], 
[8.0, 8.0, 3.0], [2.0, 7.0, 3.0]], [[4.0, 4.0, 2.0], [4.0, 1.0, 3.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 4.0, 8.0], [2.0, 5.0, 1.0], [6.0, 8.0, 2.0]], [[4.0, 6.0, 7.0], [6.0, 7.0, 8.0], [3.0, 
3.0, 6.0]], [[8.0, 1.0, 4.0], [5.0, 2.0, 4.0], [4.0, 5.0, 1.0]]], [[[1.0, 7.0, 8.0], [3.0, 7.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 
2.0, 2.0], [1.0, 5.0, 6.0], [8.0, 8.0, 6.0]], [[1.0, 3.0, 6.0], [2.0, 8.0, 8.0], [3.0, 3.0, 5.0]]], [[[7.0, 7.0, 1.0], [7.0, 
6.0, 1.0], [6.0, 8.0, 2.0]], [[6.0, 7.0, 2.0], [4.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[2.0, 8.0, 5.0], [4.0, 7.0, 8.0], [7.0, 3.0, 
8.0]]], [[[8.0, 8.0, 5.0], [6.0, 2.0, 5.0], [5.0, 6.0, 1.0]], [[4.0, 7.0, 3.0], [6.0, 7.0, 7.0], [1.0, 2.0, 4.0]], [[6.0, 8.0, 
8.0], [6.0, 5.0, 2.0], [4.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 3.0], [1.0, 2.0, 1.0], [5.0, 1.0, 2.0]], 
[[1.0, 8.0, 8.0], [1.0, 2.0, 8.0], [6.0, 2.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 8.0, 7.0], [2.0, 5.0, 8.0]]], [[[2.0, 1.0, 2.0], 
[7.0, 7.0, 5.0], [7.0, 3.0, 8.0]], [[6.0, 8.0, 7.0], [7.0, 2.0, 4.0], [6.0, 2.0, 4.0]], [[4.0, 7.0, 5.0], [5.0, 1.0, 4.0], 
[4.0, 5.0, 3.0]]], [[[8.0, 7.0, 2.0], [6.0, 8.0, 4.0], [7.0, 4.0, 3.0]], [[2.0, 6.0, 6.0], [5.0, 7.0, 1.0], [3.0, 6.0, 7.0]], 
[[2.0, 7.0, 6.0], [5.0, 2.0, 6.0], [3.0, 5.0, 6.0]]], [[[4.0, 8.0, 8.0], [8.0, 8.0, 7.0], [2.0, 1.0, 3.0]], [[5.0, 7.0, 4.0], 
[3.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 6.0, 6.0], [3.0, 5.0, 8.0]]]])
      val=numpy.array([[[4.0, 1.0, 1.0], [2.0, 6.0, 8.0], [8.0, 2.0, 6.0]]])
      arg[2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 4.0, 8.0], [2.0, 5.0, 1.0], [6.0, 8.0, 2.0]], [[4.0, 6.0, 7.0], [6.0, 7.0, 8.0], [3.0, 
3.0, 6.0]], [[8.0, 1.0, 4.0], [5.0, 2.0, 4.0], [4.0, 5.0, 1.0]]], [[[1.0, 7.0, 8.0], [3.0, 7.0, 4.0], [4.0, 1.0, 5.0]], [[5.0, 
2.0, 2.0], [1.0, 5.0, 6.0], [8.0, 8.0, 6.0]], [[1.0, 3.0, 6.0], [2.0, 8.0, 8.0], [3.0, 3.0, 5.0]]], [[[7.0, 7.0, 1.0], [7.0, 
6.0, 1.0], [6.0, 8.0, 2.0]], [[6.0, 7.0, 2.0], [4.0, 8.0, 2.0], [1.0, 5.0, 2.0]], [[4.0, 1.0, 1.0], [2.0, 6.0, 8.0], [8.0, 2.0, 
6.0]]], [[[8.0, 8.0, 5.0], [6.0, 2.0, 5.0], [5.0, 6.0, 1.0]], [[4.0, 7.0, 3.0], [6.0, 7.0, 7.0], [1.0, 2.0, 4.0]], [[6.0, 8.0, 
8.0], [6.0, 5.0, 2.0], [4.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 3.0], [1.0, 2.0, 1.0], [5.0, 1.0, 2.0]], 
[[1.0, 8.0, 8.0], [1.0, 2.0, 8.0], [6.0, 2.0, 6.0]], [[1.0, 7.0, 7.0], [8.0, 8.0, 7.0], [2.0, 5.0, 8.0]]], [[[2.0, 1.0, 2.0], 
[7.0, 7.0, 5.0], [7.0, 3.0, 8.0]], [[6.0, 8.0, 7.0], [7.0, 2.0, 4.0], [6.0, 2.0, 4.0]], [[4.0, 7.0, 5.0], [5.0, 1.0, 4.0], 
[4.0, 5.0, 3.0]]], [[[8.0, 7.0, 2.0], [6.0, 8.0, 4.0], [7.0, 4.0, 3.0]], [[2.0, 6.0, 6.0], [5.0, 7.0, 1.0], [3.0, 6.0, 7.0]], 
[[4.0, 1.0, 1.0], [2.0, 6.0, 8.0], [8.0, 2.0, 6.0]]], [[[4.0, 8.0, 8.0], [8.0, 8.0, 7.0], [2.0, 1.0, 3.0]], [[5.0, 7.0, 4.0], 
[3.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[1.0, 8.0, 1.0], [6.0, 6.0, 6.0], [3.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 6.0], [4.0, 5.0, 8.0], [3.0, 2.0, 4.0]], [[7.0, 6.0, 5.0], [6.0, 7.0, 4.0], [4.0, 
2.0, 7.0]], [[6.0, 7.0, 5.0], [5.0, 1.0, 6.0], [8.0, 2.0, 2.0]]], [[[6.0, 7.0, 6.0], [1.0, 5.0, 5.0], [1.0, 7.0, 1.0]], [[6.0, 
4.0, 8.0], [7.0, 1.0, 6.0], [1.0, 4.0, 8.0]], [[3.0, 1.0, 7.0], [3.0, 5.0, 1.0], [5.0, 5.0, 1.0]]], [[[4.0, 5.0, 7.0], [3.0, 
7.0, 1.0], [5.0, 8.0, 7.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 1.0], [2.0, 6.0, 2.0]], [[2.0, 8.0, 8.0], [7.0, 6.0, 6.0], [8.0, 6.0, 
1.0]]], [[[3.0, 1.0, 5.0], [6.0, 3.0, 6.0], [2.0, 1.0, 6.0]], [[8.0, 4.0, 3.0], [6.0, 2.0, 8.0], [8.0, 1.0, 1.0]], [[7.0, 4.0, 
1.0], [6.0, 3.0, 4.0], [1.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 2.0, 8.0], [8.0, 6.0, 4.0], [8.0, 1.0, 1.0]], 
[[7.0, 7.0, 2.0], [5.0, 6.0, 1.0], [6.0, 4.0, 4.0]], [[3.0, 3.0, 5.0], [5.0, 6.0, 8.0], [4.0, 8.0, 4.0]]], [[[5.0, 2.0, 4.0], 
[8.0, 7.0, 3.0], [8.0, 8.0, 5.0]], [[7.0, 7.0, 2.0], [7.0, 4.0, 7.0], [5.0, 2.0, 6.0]], [[7.0, 4.0, 7.0], [3.0, 4.0, 5.0], 
[2.0, 4.0, 3.0]]], [[[4.0, 6.0, 4.0], [3.0, 2.0, 2.0], [2.0, 1.0, 1.0]], [[1.0, 8.0, 6.0], [6.0, 4.0, 3.0], [3.0, 3.0, 3.0]], 
[[6.0, 2.0, 5.0], [5.0, 8.0, 2.0], [5.0, 1.0, 8.0]]], [[[5.0, 2.0, 2.0], [1.0, 8.0, 6.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 6.0], 
[6.0, 4.0, 5.0], [3.0, 8.0, 4.0]], [[5.0, 6.0, 4.0], [3.0, 3.0, 6.0], [1.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[3.0, 2.0, 3.0], [1.0, 3.0, 8.0], [7.0, 5.0, 8.0]]]),self.functionspace)
      arg[2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 6.0], [4.0, 5.0, 8.0], [3.0, 2.0, 4.0]], [[7.0, 6.0, 5.0], [6.0, 7.0, 4.0], [4.0, 
2.0, 7.0]], [[6.0, 7.0, 5.0], [5.0, 1.0, 6.0], [8.0, 2.0, 2.0]]], [[[6.0, 7.0, 6.0], [1.0, 5.0, 5.0], [1.0, 7.0, 1.0]], [[6.0, 
4.0, 8.0], [7.0, 1.0, 6.0], [1.0, 4.0, 8.0]], [[3.0, 1.0, 7.0], [3.0, 5.0, 1.0], [5.0, 5.0, 1.0]]], [[[4.0, 5.0, 7.0], [3.0, 
7.0, 1.0], [5.0, 8.0, 7.0]], [[5.0, 6.0, 4.0], [7.0, 1.0, 1.0], [2.0, 6.0, 2.0]], [[3.0, 2.0, 3.0], [1.0, 3.0, 8.0], [7.0, 5.0, 
8.0]]], [[[3.0, 1.0, 5.0], [6.0, 3.0, 6.0], [2.0, 1.0, 6.0]], [[8.0, 4.0, 3.0], [6.0, 2.0, 8.0], [8.0, 1.0, 1.0]], [[7.0, 4.0, 
1.0], [6.0, 3.0, 4.0], [1.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 2.0, 8.0], [8.0, 6.0, 4.0], [8.0, 1.0, 1.0]], 
[[7.0, 7.0, 2.0], [5.0, 6.0, 1.0], [6.0, 4.0, 4.0]], [[3.0, 3.0, 5.0], [5.0, 6.0, 8.0], [4.0, 8.0, 4.0]]], [[[5.0, 2.0, 4.0], 
[8.0, 7.0, 3.0], [8.0, 8.0, 5.0]], [[7.0, 7.0, 2.0], [7.0, 4.0, 7.0], [5.0, 2.0, 6.0]], [[7.0, 4.0, 7.0], [3.0, 4.0, 5.0], 
[2.0, 4.0, 3.0]]], [[[4.0, 6.0, 4.0], [3.0, 2.0, 2.0], [2.0, 1.0, 1.0]], [[1.0, 8.0, 6.0], [6.0, 4.0, 3.0], [3.0, 3.0, 3.0]], 
[[3.0, 2.0, 3.0], [1.0, 3.0, 8.0], [7.0, 5.0, 8.0]]], [[[5.0, 2.0, 2.0], [1.0, 8.0, 6.0], [8.0, 2.0, 3.0]], [[6.0, 8.0, 6.0], 
[6.0, 4.0, 5.0], [3.0, 8.0, 4.0]], [[5.0, 6.0, 4.0], [3.0, 3.0, 6.0], [1.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 8.0], [5.0, 8.0, 8.0], [8.0, 1.0, 4.0]], [[2.0, 8.0, 5.0], [5.0, 2.0, 7.0], [6.0, 
6.0, 4.0]], [[7.0, 2.0, 6.0], [2.0, 7.0, 4.0], [5.0, 5.0, 3.0]]], [[[8.0, 1.0, 1.0], [8.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[5.0, 
1.0, 1.0], [1.0, 8.0, 3.0], [7.0, 2.0, 3.0]], [[6.0, 6.0, 7.0], [4.0, 5.0, 3.0], [1.0, 2.0, 2.0]]], [[[6.0, 3.0, 2.0], [2.0, 
5.0, 1.0], [1.0, 7.0, 5.0]], [[8.0, 1.0, 6.0], [4.0, 8.0, 3.0], [5.0, 6.0, 3.0]], [[4.0, 6.0, 3.0], [5.0, 3.0, 6.0], [7.0, 4.0, 
3.0]]], [[[7.0, 2.0, 7.0], [8.0, 4.0, 2.0], [8.0, 6.0, 1.0]], [[2.0, 2.0, 2.0], [4.0, 6.0, 8.0], [8.0, 4.0, 5.0]], [[6.0, 1.0, 
7.0], [5.0, 2.0, 6.0], [4.0, 5.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 8.0], [3.0, 6.0, 5.0], [5.0, 2.0, 6.0]], 
[[5.0, 8.0, 8.0], [5.0, 8.0, 2.0], [1.0, 8.0, 2.0]], [[4.0, 3.0, 8.0], [2.0, 5.0, 2.0], [3.0, 7.0, 8.0]]], [[[2.0, 2.0, 6.0], 
[7.0, 6.0, 2.0], [7.0, 7.0, 8.0]], [[5.0, 6.0, 8.0], [8.0, 8.0, 6.0], [7.0, 3.0, 8.0]], [[7.0, 4.0, 7.0], [2.0, 8.0, 6.0], 
[3.0, 6.0, 3.0]]], [[[5.0, 8.0, 5.0], [5.0, 4.0, 5.0], [3.0, 7.0, 1.0]], [[2.0, 8.0, 6.0], [2.0, 8.0, 6.0], [2.0, 6.0, 4.0]], 
[[6.0, 8.0, 1.0], [4.0, 5.0, 4.0], [4.0, 1.0, 4.0]]], [[[2.0, 4.0, 7.0], [3.0, 6.0, 6.0], [2.0, 7.0, 4.0]], [[4.0, 2.0, 7.0], 
[4.0, 7.0, 3.0], [2.0, 2.0, 4.0]], [[1.0, 4.0, 1.0], [2.0, 1.0, 6.0], [6.0, 1.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 7.0, 5.0], [5.0, 6.0, 3.0], [3.0, 7.0, 5.0]]])+(1.-msk_val)*numpy.array([[[6.0, 1.0, 
6.0], [2.0, 4.0, 5.0], [8.0, 2.0, 4.0]]])
      arg[2,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 8.0], [5.0, 8.0, 8.0], [8.0, 1.0, 4.0]], [[2.0, 8.0, 5.0], [5.0, 2.0, 7.0], [6.0, 
6.0, 4.0]], [[7.0, 2.0, 6.0], [2.0, 7.0, 4.0], [5.0, 5.0, 3.0]]], [[[8.0, 1.0, 1.0], [8.0, 8.0, 8.0], [1.0, 4.0, 7.0]], [[5.0, 
1.0, 1.0], [1.0, 8.0, 3.0], [7.0, 2.0, 3.0]], [[6.0, 6.0, 7.0], [4.0, 5.0, 3.0], [1.0, 2.0, 2.0]]], [[[6.0, 3.0, 2.0], [2.0, 
5.0, 1.0], [1.0, 7.0, 5.0]], [[8.0, 1.0, 6.0], [4.0, 8.0, 3.0], [5.0, 6.0, 3.0]], [[4.0, 7.0, 5.0], [5.0, 6.0, 3.0], [3.0, 7.0, 
5.0]]], [[[7.0, 2.0, 7.0], [8.0, 4.0, 2.0], [8.0, 6.0, 1.0]], [[2.0, 2.0, 2.0], [4.0, 6.0, 8.0], [8.0, 4.0, 5.0]], [[6.0, 1.0, 
7.0], [5.0, 2.0, 6.0], [4.0, 5.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [3.0, 6.0, 5.0], [5.0, 2.0, 6.0]], 
[[5.0, 8.0, 8.0], [5.0, 8.0, 2.0], [1.0, 8.0, 2.0]], [[4.0, 3.0, 8.0], [2.0, 5.0, 2.0], [3.0, 7.0, 8.0]]], [[[2.0, 2.0, 6.0], 
[7.0, 6.0, 2.0], [7.0, 7.0, 8.0]], [[5.0, 6.0, 8.0], [8.0, 8.0, 6.0], [7.0, 3.0, 8.0]], [[7.0, 4.0, 7.0], [2.0, 8.0, 6.0], 
[3.0, 6.0, 3.0]]], [[[5.0, 8.0, 5.0], [5.0, 4.0, 5.0], [3.0, 7.0, 1.0]], [[2.0, 8.0, 6.0], [2.0, 8.0, 6.0], [2.0, 6.0, 4.0]], 
[[6.0, 1.0, 6.0], [2.0, 4.0, 5.0], [8.0, 2.0, 4.0]]], [[[2.0, 4.0, 7.0], [3.0, 6.0, 6.0], [2.0, 7.0, 4.0]], [[4.0, 2.0, 7.0], 
[4.0, 7.0, 3.0], [2.0, 2.0, 4.0]], [[1.0, 4.0, 1.0], [2.0, 1.0, 6.0], [6.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 7.0, 5.0], [3.0, 5.0, 4.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 5.0, 3.0], [3.0, 
6.0, 8.0]], [[7.0, 6.0, 3.0], [2.0, 5.0, 5.0], [7.0, 3.0, 4.0]]], [[[4.0, 7.0, 1.0], [6.0, 6.0, 8.0], [3.0, 7.0, 2.0]], [[3.0, 
7.0, 1.0], [6.0, 4.0, 8.0], [6.0, 8.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 6.0, 3.0], [3.0, 6.0, 5.0]]], [[[4.0, 5.0, 5.0], [4.0, 
4.0, 6.0], [2.0, 4.0, 3.0]], [[5.0, 7.0, 6.0], [2.0, 8.0, 2.0], [7.0, 5.0, 2.0]], [[4.0, 4.0, 2.0], [2.0, 6.0, 6.0], [7.0, 1.0, 
3.0]]], [[[4.0, 7.0, 8.0], [4.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[2.0, 6.0, 5.0], [8.0, 1.0, 5.0], [6.0, 1.0, 1.0]], [[1.0, 5.0, 
4.0], [4.0, 8.0, 1.0], [5.0, 8.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 6.0], [8.0, 2.0, 7.0], [7.0, 4.0, 6.0]], 
[[6.0, 1.0, 8.0], [6.0, 8.0, 8.0], [1.0, 2.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 1.0, 5.0], [3.0, 4.0, 5.0]]], [[[4.0, 2.0, 8.0], 
[5.0, 2.0, 6.0], [3.0, 2.0, 8.0]], [[3.0, 4.0, 4.0], [3.0, 1.0, 6.0], [5.0, 5.0, 2.0]], [[1.0, 8.0, 2.0], [1.0, 6.0, 1.0], 
[3.0, 2.0, 6.0]]], [[[3.0, 2.0, 3.0], [5.0, 3.0, 4.0], [4.0, 2.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 3.0, 4.0], [2.0, 2.0, 6.0]], 
[[2.0, 4.0, 4.0], [6.0, 3.0, 4.0], [5.0, 4.0, 7.0]]], [[[8.0, 2.0, 2.0], [6.0, 4.0, 2.0], [8.0, 7.0, 6.0]], [[6.0, 2.0, 7.0], 
[1.0, 7.0, 7.0], [8.0, 1.0, 2.0]], [[2.0, 8.0, 3.0], [7.0, 7.0, 3.0], [8.0, 5.0, 5.0]]]])
      val=numpy.array([[[3.0, 6.0, 7.0]], [[6.0, 8.0, 3.0]]])
      arg[1:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 7.0, 5.0], [3.0, 5.0, 4.0], [3.0, 1.0, 5.0]], [[6.0, 4.0, 8.0], [2.0, 5.0, 3.0], [3.0, 
6.0, 8.0]], [[7.0, 6.0, 3.0], [2.0, 5.0, 5.0], [7.0, 3.0, 4.0]]], [[[4.0, 7.0, 1.0], [6.0, 6.0, 8.0], [3.0, 7.0, 2.0]], [[3.0, 
7.0, 1.0], [3.0, 6.0, 7.0], [6.0, 8.0, 8.0]], [[7.0, 6.0, 3.0], [3.0, 6.0, 3.0], [3.0, 6.0, 5.0]]], [[[4.0, 5.0, 5.0], [4.0, 
4.0, 6.0], [2.0, 4.0, 3.0]], [[5.0, 7.0, 6.0], [6.0, 8.0, 3.0], [7.0, 5.0, 2.0]], [[4.0, 4.0, 2.0], [2.0, 6.0, 6.0], [7.0, 1.0, 
3.0]]], [[[4.0, 7.0, 8.0], [4.0, 6.0, 2.0], [8.0, 4.0, 7.0]], [[2.0, 6.0, 5.0], [8.0, 1.0, 5.0], [6.0, 1.0, 1.0]], [[1.0, 5.0, 
4.0], [4.0, 8.0, 1.0], [5.0, 8.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 6.0], [8.0, 2.0, 7.0], [7.0, 4.0, 6.0]], 
[[6.0, 1.0, 8.0], [6.0, 8.0, 8.0], [1.0, 2.0, 5.0]], [[5.0, 6.0, 1.0], [5.0, 1.0, 5.0], [3.0, 4.0, 5.0]]], [[[4.0, 2.0, 8.0], 
[5.0, 2.0, 6.0], [3.0, 2.0, 8.0]], [[3.0, 4.0, 4.0], [3.0, 6.0, 7.0], [5.0, 5.0, 2.0]], [[1.0, 8.0, 2.0], [1.0, 6.0, 1.0], 
[3.0, 2.0, 6.0]]], [[[3.0, 2.0, 3.0], [5.0, 3.0, 4.0], [4.0, 2.0, 1.0]], [[3.0, 5.0, 1.0], [6.0, 8.0, 3.0], [2.0, 2.0, 6.0]], 
[[2.0, 4.0, 4.0], [6.0, 3.0, 4.0], [5.0, 4.0, 7.0]]], [[[8.0, 2.0, 2.0], [6.0, 4.0, 2.0], [8.0, 7.0, 6.0]], [[6.0, 2.0, 7.0], 
[1.0, 7.0, 7.0], [8.0, 1.0, 2.0]], [[2.0, 8.0, 3.0], [7.0, 7.0, 3.0], [8.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 1.0], [4.0, 7.0, 5.0], [1.0, 2.0, 3.0]], [[1.0, 1.0, 7.0], [3.0, 4.0, 5.0], [1.0, 
4.0, 6.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 4.0], [4.0, 4.0, 1.0]]], [[[8.0, 1.0, 6.0], [1.0, 3.0, 2.0], [2.0, 4.0, 4.0]], [[7.0, 
8.0, 6.0], [1.0, 3.0, 3.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 4.0], [3.0, 4.0, 8.0], [3.0, 4.0, 6.0]]], [[[3.0, 2.0, 4.0], [7.0, 
7.0, 1.0], [7.0, 6.0, 5.0]], [[2.0, 2.0, 5.0], [2.0, 2.0, 5.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 6.0], [7.0, 1.0, 5.0], [1.0, 2.0, 
5.0]]], [[[2.0, 5.0, 4.0], [2.0, 8.0, 4.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 1.0], [4.0, 5.0, 1.0], [3.0, 8.0, 3.0]], [[8.0, 3.0, 
2.0], [1.0, 2.0, 5.0], [3.0, 3.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 3.0], [6.0, 8.0, 1.0], [1.0, 4.0, 7.0]], 
[[1.0, 6.0, 8.0], [2.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 4.0], [7.0, 5.0, 7.0]]], [[[2.0, 8.0, 1.0], 
[2.0, 1.0, 3.0], [2.0, 4.0, 5.0]], [[8.0, 4.0, 7.0], [3.0, 4.0, 6.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 4.0], [3.0, 4.0, 5.0], 
[5.0, 6.0, 7.0]]], [[[2.0, 2.0, 1.0], [6.0, 2.0, 6.0], [8.0, 2.0, 6.0]], [[5.0, 7.0, 7.0], [1.0, 7.0, 4.0], [8.0, 6.0, 6.0]], 
[[7.0, 8.0, 2.0], [4.0, 5.0, 4.0], [4.0, 1.0, 4.0]]], [[[7.0, 5.0, 2.0], [4.0, 1.0, 4.0], [6.0, 2.0, 1.0]], [[4.0, 7.0, 8.0], 
[7.0, 1.0, 8.0], [3.0, 7.0, 8.0]], [[2.0, 5.0, 8.0], [3.0, 4.0, 6.0], [3.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[6.0, 1.0, 5.0]], [[3.0, 7.0, 5.0]]]),self.functionspace)
      arg[1:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 1.0], [4.0, 7.0, 5.0], [1.0, 2.0, 3.0]], [[1.0, 1.0, 7.0], [3.0, 4.0, 5.0], [1.0, 
4.0, 6.0]], [[6.0, 1.0, 2.0], [4.0, 5.0, 4.0], [4.0, 4.0, 1.0]]], [[[8.0, 1.0, 6.0], [1.0, 3.0, 2.0], [2.0, 4.0, 4.0]], [[7.0, 
8.0, 6.0], [6.0, 1.0, 5.0], [3.0, 5.0, 4.0]], [[1.0, 7.0, 4.0], [3.0, 4.0, 8.0], [3.0, 4.0, 6.0]]], [[[3.0, 2.0, 4.0], [7.0, 
7.0, 1.0], [7.0, 6.0, 5.0]], [[2.0, 2.0, 5.0], [3.0, 7.0, 5.0], [4.0, 8.0, 7.0]], [[7.0, 5.0, 6.0], [7.0, 1.0, 5.0], [1.0, 2.0, 
5.0]]], [[[2.0, 5.0, 4.0], [2.0, 8.0, 4.0], [5.0, 3.0, 4.0]], [[8.0, 5.0, 1.0], [4.0, 5.0, 1.0], [3.0, 8.0, 3.0]], [[8.0, 3.0, 
2.0], [1.0, 2.0, 5.0], [3.0, 3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 3.0], [6.0, 8.0, 1.0], [1.0, 4.0, 7.0]], 
[[1.0, 6.0, 8.0], [2.0, 1.0, 7.0], [5.0, 7.0, 4.0]], [[3.0, 2.0, 5.0], [2.0, 4.0, 4.0], [7.0, 5.0, 7.0]]], [[[2.0, 8.0, 1.0], 
[2.0, 1.0, 3.0], [2.0, 4.0, 5.0]], [[8.0, 4.0, 7.0], [6.0, 1.0, 5.0], [7.0, 1.0, 4.0]], [[7.0, 3.0, 4.0], [3.0, 4.0, 5.0], 
[5.0, 6.0, 7.0]]], [[[2.0, 2.0, 1.0], [6.0, 2.0, 6.0], [8.0, 2.0, 6.0]], [[5.0, 7.0, 7.0], [3.0, 7.0, 5.0], [8.0, 6.0, 6.0]], 
[[7.0, 8.0, 2.0], [4.0, 5.0, 4.0], [4.0, 1.0, 4.0]]], [[[7.0, 5.0, 2.0], [4.0, 1.0, 4.0], [6.0, 2.0, 1.0]], [[4.0, 7.0, 8.0], 
[7.0, 1.0, 8.0], [3.0, 7.0, 8.0]], [[2.0, 5.0, 8.0], [3.0, 4.0, 6.0], [3.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 6.0, 6.0], [2.0, 3.0, 8.0], [3.0, 6.0, 4.0]], [[3.0, 2.0, 1.0], [3.0, 7.0, 7.0], [5.0, 
2.0, 6.0]], [[8.0, 2.0, 6.0], [1.0, 4.0, 4.0], [5.0, 5.0, 5.0]]], [[[3.0, 2.0, 5.0], [1.0, 7.0, 5.0], [2.0, 5.0, 2.0]], [[6.0, 
8.0, 8.0], [4.0, 7.0, 3.0], [3.0, 1.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 6.0, 1.0], [7.0, 7.0, 4.0]]], [[[1.0, 6.0, 6.0], [8.0, 
7.0, 2.0], [5.0, 5.0, 3.0]], [[8.0, 2.0, 5.0], [3.0, 7.0, 3.0], [6.0, 2.0, 8.0]], [[3.0, 8.0, 5.0], [6.0, 1.0, 8.0], [4.0, 5.0, 
3.0]]], [[[6.0, 7.0, 7.0], [1.0, 6.0, 2.0], [8.0, 2.0, 1.0]], [[8.0, 2.0, 5.0], [2.0, 8.0, 1.0], [8.0, 4.0, 3.0]], [[2.0, 3.0, 
5.0], [7.0, 7.0, 7.0], [1.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 4.0], [4.0, 6.0, 5.0], [7.0, 4.0, 5.0]], 
[[6.0, 4.0, 1.0], [5.0, 1.0, 4.0], [1.0, 6.0, 7.0]], [[3.0, 1.0, 8.0], [4.0, 1.0, 2.0], [7.0, 8.0, 2.0]]], [[[5.0, 4.0, 8.0], 
[2.0, 1.0, 6.0], [5.0, 8.0, 2.0]], [[8.0, 7.0, 4.0], [5.0, 3.0, 1.0], [7.0, 6.0, 7.0]], [[2.0, 2.0, 7.0], [6.0, 3.0, 5.0], 
[7.0, 6.0, 8.0]]], [[[4.0, 7.0, 3.0], [6.0, 6.0, 1.0], [4.0, 1.0, 5.0]], [[1.0, 1.0, 7.0], [6.0, 6.0, 1.0], [2.0, 2.0, 7.0]], 
[[2.0, 1.0, 7.0], [1.0, 4.0, 8.0], [3.0, 2.0, 7.0]]], [[[5.0, 8.0, 2.0], [2.0, 1.0, 6.0], [8.0, 2.0, 8.0]], [[3.0, 5.0, 6.0], 
[6.0, 6.0, 1.0], [3.0, 5.0, 3.0]], [[4.0, 5.0, 5.0], [8.0, 4.0, 3.0], [7.0, 7.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 5.0, 1.0]], [[6.0, 7.0, 7.0]]])+(1.-msk_val)*numpy.array([[[8.0, 6.0, 5.0]], [[4.0, 2.0, 
3.0]]])
      arg[1:3,1,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 6.0, 6.0], [2.0, 3.0, 8.0], [3.0, 6.0, 4.0]], [[3.0, 2.0, 1.0], [3.0, 7.0, 7.0], [5.0, 
2.0, 6.0]], [[8.0, 2.0, 6.0], [1.0, 4.0, 4.0], [5.0, 5.0, 5.0]]], [[[3.0, 2.0, 5.0], [1.0, 7.0, 5.0], [2.0, 5.0, 2.0]], [[6.0, 
8.0, 8.0], [6.0, 5.0, 1.0], [3.0, 1.0, 2.0]], [[7.0, 2.0, 1.0], [5.0, 6.0, 1.0], [7.0, 7.0, 4.0]]], [[[1.0, 6.0, 6.0], [8.0, 
7.0, 2.0], [5.0, 5.0, 3.0]], [[8.0, 2.0, 5.0], [6.0, 7.0, 7.0], [6.0, 2.0, 8.0]], [[3.0, 8.0, 5.0], [6.0, 1.0, 8.0], [4.0, 5.0, 
3.0]]], [[[6.0, 7.0, 7.0], [1.0, 6.0, 2.0], [8.0, 2.0, 1.0]], [[8.0, 2.0, 5.0], [2.0, 8.0, 1.0], [8.0, 4.0, 3.0]], [[2.0, 3.0, 
5.0], [7.0, 7.0, 7.0], [1.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 4.0], [4.0, 6.0, 5.0], [7.0, 4.0, 5.0]], 
[[6.0, 4.0, 1.0], [5.0, 1.0, 4.0], [1.0, 6.0, 7.0]], [[3.0, 1.0, 8.0], [4.0, 1.0, 2.0], [7.0, 8.0, 2.0]]], [[[5.0, 4.0, 8.0], 
[2.0, 1.0, 6.0], [5.0, 8.0, 2.0]], [[8.0, 7.0, 4.0], [8.0, 6.0, 5.0], [7.0, 6.0, 7.0]], [[2.0, 2.0, 7.0], [6.0, 3.0, 5.0], 
[7.0, 6.0, 8.0]]], [[[4.0, 7.0, 3.0], [6.0, 6.0, 1.0], [4.0, 1.0, 5.0]], [[1.0, 1.0, 7.0], [4.0, 2.0, 3.0], [2.0, 2.0, 7.0]], 
[[2.0, 1.0, 7.0], [1.0, 4.0, 8.0], [3.0, 2.0, 7.0]]], [[[5.0, 8.0, 2.0], [2.0, 1.0, 6.0], [8.0, 2.0, 8.0]], [[3.0, 5.0, 6.0], 
[6.0, 6.0, 1.0], [3.0, 5.0, 3.0]], [[4.0, 5.0, 5.0], [8.0, 4.0, 3.0], [7.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 4.0], [8.0, 8.0, 3.0], [7.0, 6.0, 4.0]], [[7.0, 8.0, 1.0], [8.0, 4.0, 7.0], [3.0, 
1.0, 7.0]], [[6.0, 7.0, 3.0], [1.0, 5.0, 8.0], [4.0, 6.0, 7.0]]], [[[2.0, 5.0, 4.0], [5.0, 2.0, 6.0], [5.0, 8.0, 8.0]], [[1.0, 
7.0, 8.0], [8.0, 8.0, 2.0], [5.0, 8.0, 6.0]], [[4.0, 6.0, 5.0], [7.0, 8.0, 6.0], [8.0, 4.0, 8.0]]], [[[6.0, 1.0, 4.0], [4.0, 
7.0, 5.0], [5.0, 6.0, 3.0]], [[2.0, 8.0, 8.0], [1.0, 2.0, 1.0], [7.0, 7.0, 4.0]], [[1.0, 5.0, 7.0], [8.0, 3.0, 6.0], [5.0, 5.0, 
4.0]]], [[[4.0, 7.0, 6.0], [8.0, 1.0, 5.0], [8.0, 6.0, 6.0]], [[8.0, 1.0, 1.0], [8.0, 5.0, 8.0], [1.0, 3.0, 8.0]], [[3.0, 5.0, 
3.0], [4.0, 8.0, 5.0], [8.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 7.0], [7.0, 4.0, 4.0], [8.0, 7.0, 4.0]], 
[[1.0, 1.0, 1.0], [1.0, 3.0, 5.0], [8.0, 1.0, 1.0]], [[6.0, 6.0, 5.0], [8.0, 6.0, 2.0], [1.0, 5.0, 3.0]]], [[[7.0, 7.0, 2.0], 
[4.0, 2.0, 4.0], [7.0, 4.0, 3.0]], [[5.0, 3.0, 3.0], [8.0, 8.0, 8.0], [5.0, 1.0, 7.0]], [[6.0, 1.0, 1.0], [4.0, 1.0, 3.0], 
[4.0, 1.0, 1.0]]], [[[2.0, 1.0, 4.0], [6.0, 3.0, 7.0], [5.0, 4.0, 7.0]], [[6.0, 2.0, 3.0], [1.0, 5.0, 6.0], [3.0, 3.0, 4.0]], 
[[3.0, 6.0, 1.0], [6.0, 8.0, 8.0], [6.0, 8.0, 4.0]]], [[[8.0, 8.0, 8.0], [7.0, 4.0, 1.0], [3.0, 8.0, 3.0]], [[5.0, 1.0, 5.0], 
[7.0, 6.0, 1.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 4.0], [1.0, 4.0, 2.0], [1.0, 6.0, 3.0]]]])
      val=numpy.array([[[3.0, 8.0, 6.0], [3.0, 4.0, 7.0], [4.0, 3.0, 4.0]], [[1.0, 2.0, 6.0], [1.0, 3.0, 8.0], [4.0, 5.0, 
6.0]], [[2.0, 3.0, 3.0], [2.0, 1.0, 4.0], [1.0, 7.0, 2.0]]])
      arg[2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 4.0], [8.0, 8.0, 3.0], [7.0, 6.0, 4.0]], [[7.0, 8.0, 1.0], [8.0, 4.0, 7.0], [3.0, 
1.0, 7.0]], [[6.0, 7.0, 3.0], [1.0, 5.0, 8.0], [4.0, 6.0, 7.0]]], [[[2.0, 5.0, 4.0], [5.0, 2.0, 6.0], [5.0, 8.0, 8.0]], [[1.0, 
7.0, 8.0], [8.0, 8.0, 2.0], [5.0, 8.0, 6.0]], [[4.0, 6.0, 5.0], [7.0, 8.0, 6.0], [8.0, 4.0, 8.0]]], [[[3.0, 8.0, 6.0], [3.0, 
4.0, 7.0], [4.0, 3.0, 4.0]], [[1.0, 2.0, 6.0], [1.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[2.0, 3.0, 3.0], [2.0, 1.0, 4.0], [1.0, 7.0, 
2.0]]], [[[4.0, 7.0, 6.0], [8.0, 1.0, 5.0], [8.0, 6.0, 6.0]], [[8.0, 1.0, 1.0], [8.0, 5.0, 8.0], [1.0, 3.0, 8.0]], [[3.0, 5.0, 
3.0], [4.0, 8.0, 5.0], [8.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 7.0], [7.0, 4.0, 4.0], [8.0, 7.0, 4.0]], 
[[1.0, 1.0, 1.0], [1.0, 3.0, 5.0], [8.0, 1.0, 1.0]], [[6.0, 6.0, 5.0], [8.0, 6.0, 2.0], [1.0, 5.0, 3.0]]], [[[7.0, 7.0, 2.0], 
[4.0, 2.0, 4.0], [7.0, 4.0, 3.0]], [[5.0, 3.0, 3.0], [8.0, 8.0, 8.0], [5.0, 1.0, 7.0]], [[6.0, 1.0, 1.0], [4.0, 1.0, 3.0], 
[4.0, 1.0, 1.0]]], [[[3.0, 8.0, 6.0], [3.0, 4.0, 7.0], [4.0, 3.0, 4.0]], [[1.0, 2.0, 6.0], [1.0, 3.0, 8.0], [4.0, 5.0, 6.0]], 
[[2.0, 3.0, 3.0], [2.0, 1.0, 4.0], [1.0, 7.0, 2.0]]], [[[8.0, 8.0, 8.0], [7.0, 4.0, 1.0], [3.0, 8.0, 3.0]], [[5.0, 1.0, 5.0], 
[7.0, 6.0, 1.0], [8.0, 8.0, 2.0]], [[4.0, 6.0, 4.0], [1.0, 4.0, 2.0], [1.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 8.0], [2.0, 8.0, 3.0], [2.0, 2.0, 7.0]], [[3.0, 2.0, 4.0], [8.0, 5.0, 4.0], [4.0, 
4.0, 3.0]], [[5.0, 6.0, 5.0], [3.0, 1.0, 8.0], [1.0, 8.0, 3.0]]], [[[5.0, 8.0, 8.0], [7.0, 2.0, 6.0], [2.0, 8.0, 1.0]], [[1.0, 
4.0, 7.0], [2.0, 4.0, 6.0], [7.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 2.0, 2.0], [6.0, 6.0, 7.0]]], [[[1.0, 8.0, 8.0], [5.0, 
8.0, 2.0], [7.0, 4.0, 5.0]], [[2.0, 1.0, 2.0], [2.0, 3.0, 8.0], [1.0, 8.0, 4.0]], [[8.0, 5.0, 4.0], [2.0, 2.0, 6.0], [1.0, 6.0, 
4.0]]], [[[8.0, 4.0, 4.0], [1.0, 7.0, 5.0], [7.0, 2.0, 8.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 2.0], [6.0, 4.0, 5.0]], [[3.0, 8.0, 
2.0], [2.0, 1.0, 8.0], [3.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 3.0], [4.0, 4.0, 2.0], [1.0, 6.0, 1.0]], 
[[7.0, 8.0, 7.0], [1.0, 2.0, 3.0], [1.0, 7.0, 7.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 4.0], [5.0, 3.0, 3.0]]], [[[3.0, 3.0, 6.0], 
[2.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 5.0, 1.0], [4.0, 6.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 8.0], [5.0, 2.0, 6.0], 
[6.0, 6.0, 6.0]]], [[[4.0, 5.0, 7.0], [8.0, 2.0, 1.0], [5.0, 7.0, 8.0]], [[8.0, 2.0, 4.0], [7.0, 5.0, 4.0], [6.0, 8.0, 6.0]], 
[[2.0, 3.0, 2.0], [7.0, 7.0, 3.0], [4.0, 5.0, 4.0]]], [[[5.0, 8.0, 5.0], [4.0, 4.0, 7.0], [8.0, 4.0, 7.0]], [[2.0, 4.0, 8.0], 
[5.0, 7.0, 3.0], [7.0, 3.0, 4.0]], [[5.0, 4.0, 3.0], [5.0, 6.0, 1.0], [8.0, 6.0, 7.0]]]])
      val=Data(numpy.array([[[7.0, 5.0, 3.0], [4.0, 3.0, 2.0], [4.0, 2.0, 3.0]], [[7.0, 7.0, 7.0], [3.0, 2.0, 3.0], [8.0, 7.0, 
7.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 8.0], [3.0, 3.0, 8.0]]]),self.functionspace)
      arg[2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 8.0], [2.0, 8.0, 3.0], [2.0, 2.0, 7.0]], [[3.0, 2.0, 4.0], [8.0, 5.0, 4.0], [4.0, 
4.0, 3.0]], [[5.0, 6.0, 5.0], [3.0, 1.0, 8.0], [1.0, 8.0, 3.0]]], [[[5.0, 8.0, 8.0], [7.0, 2.0, 6.0], [2.0, 8.0, 1.0]], [[1.0, 
4.0, 7.0], [2.0, 4.0, 6.0], [7.0, 2.0, 1.0]], [[8.0, 5.0, 2.0], [2.0, 2.0, 2.0], [6.0, 6.0, 7.0]]], [[[7.0, 5.0, 3.0], [4.0, 
3.0, 2.0], [4.0, 2.0, 3.0]], [[7.0, 7.0, 7.0], [3.0, 2.0, 3.0], [8.0, 7.0, 7.0]], [[6.0, 8.0, 4.0], [2.0, 6.0, 8.0], [3.0, 3.0, 
8.0]]], [[[8.0, 4.0, 4.0], [1.0, 7.0, 5.0], [7.0, 2.0, 8.0]], [[4.0, 6.0, 6.0], [3.0, 6.0, 2.0], [6.0, 4.0, 5.0]], [[3.0, 8.0, 
2.0], [2.0, 1.0, 8.0], [3.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 3.0], [4.0, 4.0, 2.0], [1.0, 6.0, 1.0]], 
[[7.0, 8.0, 7.0], [1.0, 2.0, 3.0], [1.0, 7.0, 7.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 4.0], [5.0, 3.0, 3.0]]], [[[3.0, 3.0, 6.0], 
[2.0, 3.0, 2.0], [8.0, 7.0, 4.0]], [[8.0, 5.0, 1.0], [4.0, 6.0, 8.0], [7.0, 5.0, 4.0]], [[3.0, 6.0, 8.0], [5.0, 2.0, 6.0], 
[6.0, 6.0, 6.0]]], [[[7.0, 5.0, 3.0], [4.0, 3.0, 2.0], [4.0, 2.0, 3.0]], [[7.0, 7.0, 7.0], [3.0, 2.0, 3.0], [8.0, 7.0, 7.0]], 
[[6.0, 8.0, 4.0], [2.0, 6.0, 8.0], [3.0, 3.0, 8.0]]], [[[5.0, 8.0, 5.0], [4.0, 4.0, 7.0], [8.0, 4.0, 7.0]], [[2.0, 4.0, 8.0], 
[5.0, 7.0, 3.0], [7.0, 3.0, 4.0]], [[5.0, 4.0, 3.0], [5.0, 6.0, 1.0], [8.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 2.0], [4.0, 8.0, 3.0], [4.0, 7.0, 2.0]], [[6.0, 5.0, 6.0], [2.0, 7.0, 6.0], [6.0, 
7.0, 3.0]], [[4.0, 4.0, 7.0], [6.0, 2.0, 8.0], [7.0, 8.0, 8.0]]], [[[6.0, 2.0, 6.0], [1.0, 2.0, 5.0], [7.0, 5.0, 4.0]], [[7.0, 
6.0, 8.0], [5.0, 1.0, 7.0], [4.0, 7.0, 5.0]], [[6.0, 5.0, 6.0], [3.0, 4.0, 7.0], [7.0, 8.0, 4.0]]], [[[8.0, 2.0, 7.0], [8.0, 
7.0, 6.0], [3.0, 7.0, 5.0]], [[3.0, 1.0, 3.0], [4.0, 3.0, 1.0], [7.0, 3.0, 5.0]], [[6.0, 6.0, 3.0], [5.0, 3.0, 6.0], [6.0, 8.0, 
8.0]]], [[[2.0, 4.0, 5.0], [5.0, 1.0, 6.0], [4.0, 6.0, 5.0]], [[4.0, 6.0, 3.0], [4.0, 3.0, 8.0], [8.0, 4.0, 1.0]], [[8.0, 2.0, 
8.0], [4.0, 8.0, 8.0], [7.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 5.0], [3.0, 5.0, 5.0], [5.0, 7.0, 1.0]], 
[[5.0, 2.0, 3.0], [6.0, 6.0, 8.0], [7.0, 8.0, 2.0]], [[5.0, 5.0, 3.0], [3.0, 1.0, 3.0], [5.0, 6.0, 5.0]]], [[[4.0, 3.0, 1.0], 
[2.0, 4.0, 6.0], [6.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [6.0, 5.0, 4.0], [7.0, 6.0, 7.0]], [[4.0, 7.0, 7.0], [5.0, 5.0, 5.0], 
[4.0, 3.0, 2.0]]], [[[5.0, 7.0, 6.0], [8.0, 7.0, 6.0], [4.0, 4.0, 5.0]], [[2.0, 5.0, 3.0], [1.0, 4.0, 4.0], [1.0, 1.0, 1.0]], 
[[8.0, 4.0, 3.0], [4.0, 8.0, 5.0], [6.0, 7.0, 6.0]]], [[[7.0, 7.0, 4.0], [4.0, 2.0, 4.0], [5.0, 1.0, 2.0]], [[2.0, 6.0, 6.0], 
[2.0, 4.0, 4.0], [8.0, 8.0, 5.0]], [[4.0, 6.0, 7.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[7.0, 2.0, 8.0], [8.0, 1.0, 1.0], [6.0, 3.0, 7.0]], [[1.0, 4.0, 3.0], [7.0, 6.0, 2.0], [3.0, 
8.0, 7.0]], [[7.0, 4.0, 1.0], [4.0, 4.0, 4.0], [2.0, 3.0, 3.0]]])+(1.-msk_val)*numpy.array([[[7.0, 3.0, 5.0], [8.0, 3.0, 5.0], 
[8.0, 7.0, 8.0]], [[4.0, 3.0, 6.0], [1.0, 5.0, 8.0], [1.0, 7.0, 7.0]], [[8.0, 2.0, 5.0], [7.0, 3.0, 8.0], [5.0, 1.0, 1.0]]])
      arg[2,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 2.0], [4.0, 8.0, 3.0], [4.0, 7.0, 2.0]], [[6.0, 5.0, 6.0], [2.0, 7.0, 6.0], [6.0, 
7.0, 3.0]], [[4.0, 4.0, 7.0], [6.0, 2.0, 8.0], [7.0, 8.0, 8.0]]], [[[6.0, 2.0, 6.0], [1.0, 2.0, 5.0], [7.0, 5.0, 4.0]], [[7.0, 
6.0, 8.0], [5.0, 1.0, 7.0], [4.0, 7.0, 5.0]], [[6.0, 5.0, 6.0], [3.0, 4.0, 7.0], [7.0, 8.0, 4.0]]], [[[7.0, 2.0, 8.0], [8.0, 
1.0, 1.0], [6.0, 3.0, 7.0]], [[1.0, 4.0, 3.0], [7.0, 6.0, 2.0], [3.0, 8.0, 7.0]], [[7.0, 4.0, 1.0], [4.0, 4.0, 4.0], [2.0, 3.0, 
3.0]]], [[[2.0, 4.0, 5.0], [5.0, 1.0, 6.0], [4.0, 6.0, 5.0]], [[4.0, 6.0, 3.0], [4.0, 3.0, 8.0], [8.0, 4.0, 1.0]], [[8.0, 2.0, 
8.0], [4.0, 8.0, 8.0], [7.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 5.0], [3.0, 5.0, 5.0], [5.0, 7.0, 1.0]], 
[[5.0, 2.0, 3.0], [6.0, 6.0, 8.0], [7.0, 8.0, 2.0]], [[5.0, 5.0, 3.0], [3.0, 1.0, 3.0], [5.0, 6.0, 5.0]]], [[[4.0, 3.0, 1.0], 
[2.0, 4.0, 6.0], [6.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [6.0, 5.0, 4.0], [7.0, 6.0, 7.0]], [[4.0, 7.0, 7.0], [5.0, 5.0, 5.0], 
[4.0, 3.0, 2.0]]], [[[7.0, 3.0, 5.0], [8.0, 3.0, 5.0], [8.0, 7.0, 8.0]], [[4.0, 3.0, 6.0], [1.0, 5.0, 8.0], [1.0, 7.0, 7.0]], 
[[8.0, 2.0, 5.0], [7.0, 3.0, 8.0], [5.0, 1.0, 1.0]]], [[[7.0, 7.0, 4.0], [4.0, 2.0, 4.0], [5.0, 1.0, 2.0]], [[2.0, 6.0, 6.0], 
[2.0, 4.0, 4.0], [8.0, 8.0, 5.0]], [[4.0, 6.0, 7.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 7.0], [7.0, 5.0, 1.0], [6.0, 2.0, 4.0]], [[6.0, 2.0, 8.0], [4.0, 8.0, 4.0], [2.0, 
4.0, 4.0]], [[6.0, 4.0, 6.0], [1.0, 4.0, 2.0], [8.0, 5.0, 7.0]]], [[[1.0, 8.0, 8.0], [3.0, 6.0, 2.0], [4.0, 5.0, 3.0]], [[5.0, 
6.0, 3.0], [5.0, 6.0, 2.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 5.0], [7.0, 5.0, 7.0], [3.0, 8.0, 8.0]]], [[[1.0, 2.0, 4.0], [8.0, 
1.0, 8.0], [7.0, 4.0, 8.0]], [[5.0, 4.0, 6.0], [4.0, 7.0, 7.0], [6.0, 7.0, 1.0]], [[1.0, 2.0, 2.0], [4.0, 3.0, 1.0], [8.0, 3.0, 
8.0]]], [[[3.0, 3.0, 7.0], [5.0, 7.0, 7.0], [3.0, 8.0, 6.0]], [[3.0, 1.0, 6.0], [4.0, 3.0, 2.0], [3.0, 1.0, 7.0]], [[8.0, 1.0, 
3.0], [1.0, 7.0, 5.0], [3.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 2.0], [6.0, 4.0, 5.0], [2.0, 4.0, 5.0]], 
[[5.0, 1.0, 8.0], [7.0, 4.0, 5.0], [2.0, 8.0, 3.0]], [[7.0, 7.0, 3.0], [4.0, 3.0, 8.0], [1.0, 4.0, 8.0]]], [[[3.0, 1.0, 7.0], 
[4.0, 5.0, 1.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 6.0], [6.0, 3.0, 8.0], [3.0, 6.0, 1.0]], [[2.0, 1.0, 7.0], [6.0, 5.0, 5.0], 
[7.0, 1.0, 2.0]]], [[[6.0, 4.0, 1.0], [4.0, 5.0, 1.0], [1.0, 3.0, 2.0]], [[5.0, 2.0, 5.0], [3.0, 6.0, 6.0], [2.0, 3.0, 3.0]], 
[[3.0, 8.0, 1.0], [8.0, 3.0, 7.0], [1.0, 8.0, 5.0]]], [[[8.0, 7.0, 4.0], [7.0, 6.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 6.0, 5.0], 
[1.0, 2.0, 8.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 3.0], [7.0, 4.0, 7.0], [3.0, 6.0, 3.0]]]])
      val=numpy.array([[[3.0, 8.0, 5.0], [8.0, 5.0, 3.0]], [[1.0, 7.0, 3.0], [5.0, 3.0, 3.0]], [[3.0, 1.0, 5.0], [6.0, 8.0, 
6.0]]])
      arg[2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 7.0], [7.0, 5.0, 1.0], [6.0, 2.0, 4.0]], [[6.0, 2.0, 8.0], [4.0, 8.0, 4.0], [2.0, 
4.0, 4.0]], [[6.0, 4.0, 6.0], [1.0, 4.0, 2.0], [8.0, 5.0, 7.0]]], [[[1.0, 8.0, 8.0], [3.0, 6.0, 2.0], [4.0, 5.0, 3.0]], [[5.0, 
6.0, 3.0], [5.0, 6.0, 2.0], [4.0, 7.0, 3.0]], [[7.0, 5.0, 5.0], [7.0, 5.0, 7.0], [3.0, 8.0, 8.0]]], [[[3.0, 8.0, 5.0], [8.0, 
5.0, 3.0], [7.0, 4.0, 8.0]], [[1.0, 7.0, 3.0], [5.0, 3.0, 3.0], [6.0, 7.0, 1.0]], [[3.0, 1.0, 5.0], [6.0, 8.0, 6.0], [8.0, 3.0, 
8.0]]], [[[3.0, 3.0, 7.0], [5.0, 7.0, 7.0], [3.0, 8.0, 6.0]], [[3.0, 1.0, 6.0], [4.0, 3.0, 2.0], [3.0, 1.0, 7.0]], [[8.0, 1.0, 
3.0], [1.0, 7.0, 5.0], [3.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 2.0], [6.0, 4.0, 5.0], [2.0, 4.0, 5.0]], 
[[5.0, 1.0, 8.0], [7.0, 4.0, 5.0], [2.0, 8.0, 3.0]], [[7.0, 7.0, 3.0], [4.0, 3.0, 8.0], [1.0, 4.0, 8.0]]], [[[3.0, 1.0, 7.0], 
[4.0, 5.0, 1.0], [3.0, 5.0, 6.0]], [[6.0, 3.0, 6.0], [6.0, 3.0, 8.0], [3.0, 6.0, 1.0]], [[2.0, 1.0, 7.0], [6.0, 5.0, 5.0], 
[7.0, 1.0, 2.0]]], [[[3.0, 8.0, 5.0], [8.0, 5.0, 3.0], [1.0, 3.0, 2.0]], [[1.0, 7.0, 3.0], [5.0, 3.0, 3.0], [2.0, 3.0, 3.0]], 
[[3.0, 1.0, 5.0], [6.0, 8.0, 6.0], [1.0, 8.0, 5.0]]], [[[8.0, 7.0, 4.0], [7.0, 6.0, 6.0], [4.0, 7.0, 1.0]], [[8.0, 6.0, 5.0], 
[1.0, 2.0, 8.0], [1.0, 5.0, 8.0]], [[5.0, 4.0, 3.0], [7.0, 4.0, 7.0], [3.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 5.0], [1.0, 3.0, 1.0], [5.0, 4.0, 5.0]], [[5.0, 3.0, 3.0], [6.0, 7.0, 6.0], [7.0, 
6.0, 2.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 2.0], [6.0, 2.0, 3.0]]], [[[3.0, 7.0, 7.0], [3.0, 5.0, 5.0], [1.0, 3.0, 8.0]], [[5.0, 
3.0, 4.0], [8.0, 1.0, 1.0], [4.0, 1.0, 7.0]], [[6.0, 2.0, 5.0], [1.0, 7.0, 5.0], [8.0, 7.0, 5.0]]], [[[7.0, 2.0, 3.0], [6.0, 
3.0, 8.0], [4.0, 3.0, 1.0]], [[5.0, 5.0, 3.0], [6.0, 4.0, 7.0], [8.0, 8.0, 6.0]], [[1.0, 7.0, 5.0], [1.0, 4.0, 5.0], [7.0, 4.0, 
4.0]]], [[[1.0, 5.0, 5.0], [3.0, 5.0, 6.0], [3.0, 5.0, 5.0]], [[6.0, 2.0, 6.0], [4.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[4.0, 3.0, 
4.0], [8.0, 8.0, 3.0], [7.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 8.0, 4.0], [2.0, 3.0, 6.0], [3.0, 1.0, 8.0]], 
[[4.0, 8.0, 1.0], [4.0, 7.0, 2.0], [2.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [2.0, 8.0, 2.0], [4.0, 5.0, 7.0]]], [[[6.0, 6.0, 3.0], 
[3.0, 4.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 6.0, 4.0], [4.0, 6.0, 5.0], [1.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [5.0, 4.0, 7.0], 
[5.0, 1.0, 8.0]]], [[[5.0, 2.0, 5.0], [7.0, 1.0, 8.0], [1.0, 8.0, 8.0]], [[5.0, 3.0, 1.0], [1.0, 5.0, 5.0], [2.0, 7.0, 1.0]], 
[[7.0, 2.0, 1.0], [5.0, 8.0, 2.0], [3.0, 3.0, 1.0]]], [[[2.0, 3.0, 4.0], [1.0, 3.0, 6.0], [1.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], 
[5.0, 8.0, 7.0], [6.0, 2.0, 5.0]], [[4.0, 2.0, 8.0], [7.0, 5.0, 7.0], [3.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[8.0, 8.0, 7.0], [3.0, 4.0, 4.0]], [[4.0, 5.0, 8.0], [7.0, 1.0, 5.0]], [[1.0, 6.0, 8.0], [2.0, 
6.0, 1.0]]]),self.functionspace)
      arg[2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 5.0], [1.0, 3.0, 1.0], [5.0, 4.0, 5.0]], [[5.0, 3.0, 3.0], [6.0, 7.0, 6.0], [7.0, 
6.0, 2.0]], [[2.0, 3.0, 3.0], [5.0, 5.0, 2.0], [6.0, 2.0, 3.0]]], [[[3.0, 7.0, 7.0], [3.0, 5.0, 5.0], [1.0, 3.0, 8.0]], [[5.0, 
3.0, 4.0], [8.0, 1.0, 1.0], [4.0, 1.0, 7.0]], [[6.0, 2.0, 5.0], [1.0, 7.0, 5.0], [8.0, 7.0, 5.0]]], [[[8.0, 8.0, 7.0], [3.0, 
4.0, 4.0], [4.0, 3.0, 1.0]], [[4.0, 5.0, 8.0], [7.0, 1.0, 5.0], [8.0, 8.0, 6.0]], [[1.0, 6.0, 8.0], [2.0, 6.0, 1.0], [7.0, 4.0, 
4.0]]], [[[1.0, 5.0, 5.0], [3.0, 5.0, 6.0], [3.0, 5.0, 5.0]], [[6.0, 2.0, 6.0], [4.0, 6.0, 4.0], [4.0, 6.0, 2.0]], [[4.0, 3.0, 
4.0], [8.0, 8.0, 3.0], [7.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 8.0, 4.0], [2.0, 3.0, 6.0], [3.0, 1.0, 8.0]], 
[[4.0, 8.0, 1.0], [4.0, 7.0, 2.0], [2.0, 7.0, 3.0]], [[5.0, 8.0, 7.0], [2.0, 8.0, 2.0], [4.0, 5.0, 7.0]]], [[[6.0, 6.0, 3.0], 
[3.0, 4.0, 2.0], [4.0, 2.0, 1.0]], [[7.0, 6.0, 4.0], [4.0, 6.0, 5.0], [1.0, 2.0, 5.0]], [[5.0, 8.0, 4.0], [5.0, 4.0, 7.0], 
[5.0, 1.0, 8.0]]], [[[8.0, 8.0, 7.0], [3.0, 4.0, 4.0], [1.0, 8.0, 8.0]], [[4.0, 5.0, 8.0], [7.0, 1.0, 5.0], [2.0, 7.0, 1.0]], 
[[1.0, 6.0, 8.0], [2.0, 6.0, 1.0], [3.0, 3.0, 1.0]]], [[[2.0, 3.0, 4.0], [1.0, 3.0, 6.0], [1.0, 3.0, 1.0]], [[7.0, 2.0, 2.0], 
[5.0, 8.0, 7.0], [6.0, 2.0, 5.0]], [[4.0, 2.0, 8.0], [7.0, 5.0, 7.0], [3.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 3.0, 2.0], [3.0, 7.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 8.0, 8.0], [4.0, 8.0, 3.0], [2.0, 
4.0, 8.0]], [[2.0, 6.0, 1.0], [4.0, 1.0, 8.0], [3.0, 2.0, 8.0]]], [[[6.0, 4.0, 6.0], [3.0, 7.0, 1.0], [7.0, 5.0, 1.0]], [[7.0, 
6.0, 5.0], [8.0, 2.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 1.0, 2.0], [2.0, 4.0, 8.0], [1.0, 4.0, 3.0]]], [[[4.0, 5.0, 4.0], [5.0, 
5.0, 3.0], [1.0, 4.0, 8.0]], [[6.0, 4.0, 3.0], [5.0, 4.0, 7.0], [7.0, 8.0, 5.0]], [[2.0, 8.0, 2.0], [7.0, 4.0, 7.0], [7.0, 3.0, 
1.0]]], [[[8.0, 5.0, 4.0], [4.0, 8.0, 1.0], [1.0, 2.0, 7.0]], [[1.0, 6.0, 7.0], [6.0, 1.0, 3.0], [3.0, 6.0, 6.0]], [[5.0, 5.0, 
6.0], [7.0, 5.0, 5.0], [5.0, 2.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 7.0], [1.0, 1.0, 6.0], [7.0, 6.0, 4.0]], 
[[7.0, 7.0, 6.0], [2.0, 5.0, 4.0], [4.0, 1.0, 3.0]], [[1.0, 6.0, 8.0], [1.0, 8.0, 8.0], [5.0, 5.0, 2.0]]], [[[4.0, 2.0, 4.0], 
[5.0, 5.0, 1.0], [8.0, 2.0, 7.0]], [[7.0, 8.0, 1.0], [8.0, 2.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 3.0], 
[4.0, 7.0, 7.0]]], [[[3.0, 6.0, 6.0], [3.0, 5.0, 7.0], [7.0, 2.0, 6.0]], [[2.0, 5.0, 7.0], [8.0, 5.0, 7.0], [6.0, 2.0, 4.0]], 
[[6.0, 3.0, 1.0], [6.0, 1.0, 5.0], [8.0, 6.0, 4.0]]], [[[7.0, 1.0, 6.0], [5.0, 6.0, 4.0], [2.0, 3.0, 7.0]], [[1.0, 7.0, 6.0], 
[4.0, 8.0, 5.0], [2.0, 3.0, 2.0]], [[3.0, 8.0, 1.0], [1.0, 8.0, 7.0], [5.0, 6.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[6.0, 5.0, 7.0], [8.0, 8.0, 1.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 3.0]], [[3.0, 1.0, 5.0], [7.0, 
3.0, 3.0]]])+(1.-msk_val)*numpy.array([[[4.0, 4.0, 7.0], [4.0, 3.0, 6.0]], [[1.0, 5.0, 6.0], [5.0, 8.0, 2.0]], [[5.0, 2.0, 
6.0], [7.0, 4.0, 5.0]]])
      arg[2,:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 3.0, 2.0], [3.0, 7.0, 7.0], [5.0, 7.0, 3.0]], [[7.0, 8.0, 8.0], [4.0, 8.0, 3.0], [2.0, 
4.0, 8.0]], [[2.0, 6.0, 1.0], [4.0, 1.0, 8.0], [3.0, 2.0, 8.0]]], [[[6.0, 4.0, 6.0], [3.0, 7.0, 1.0], [7.0, 5.0, 1.0]], [[7.0, 
6.0, 5.0], [8.0, 2.0, 3.0], [2.0, 8.0, 2.0]], [[5.0, 1.0, 2.0], [2.0, 4.0, 8.0], [1.0, 4.0, 3.0]]], [[[6.0, 5.0, 7.0], [8.0, 
8.0, 1.0], [1.0, 4.0, 8.0]], [[8.0, 7.0, 4.0], [4.0, 7.0, 3.0], [7.0, 8.0, 5.0]], [[3.0, 1.0, 5.0], [7.0, 3.0, 3.0], [7.0, 3.0, 
1.0]]], [[[8.0, 5.0, 4.0], [4.0, 8.0, 1.0], [1.0, 2.0, 7.0]], [[1.0, 6.0, 7.0], [6.0, 1.0, 3.0], [3.0, 6.0, 6.0]], [[5.0, 5.0, 
6.0], [7.0, 5.0, 5.0], [5.0, 2.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [1.0, 1.0, 6.0], [7.0, 6.0, 4.0]], 
[[7.0, 7.0, 6.0], [2.0, 5.0, 4.0], [4.0, 1.0, 3.0]], [[1.0, 6.0, 8.0], [1.0, 8.0, 8.0], [5.0, 5.0, 2.0]]], [[[4.0, 2.0, 4.0], 
[5.0, 5.0, 1.0], [8.0, 2.0, 7.0]], [[7.0, 8.0, 1.0], [8.0, 2.0, 7.0], [2.0, 6.0, 3.0]], [[7.0, 5.0, 2.0], [5.0, 6.0, 3.0], 
[4.0, 7.0, 7.0]]], [[[4.0, 4.0, 7.0], [4.0, 3.0, 6.0], [7.0, 2.0, 6.0]], [[1.0, 5.0, 6.0], [5.0, 8.0, 2.0], [6.0, 2.0, 4.0]], 
[[5.0, 2.0, 6.0], [7.0, 4.0, 5.0], [8.0, 6.0, 4.0]]], [[[7.0, 1.0, 6.0], [5.0, 6.0, 4.0], [2.0, 3.0, 7.0]], [[1.0, 7.0, 6.0], 
[4.0, 8.0, 5.0], [2.0, 3.0, 2.0]], [[3.0, 8.0, 1.0], [1.0, 8.0, 7.0], [5.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 5.0], [7.0, 2.0, 8.0], [6.0, 5.0, 5.0]], [[8.0, 8.0, 3.0], [8.0, 8.0, 3.0], [2.0, 
2.0, 1.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 3.0], [8.0, 1.0, 4.0]]], [[[1.0, 1.0, 1.0], [6.0, 8.0, 7.0], [7.0, 7.0, 2.0]], [[2.0, 
6.0, 6.0], [1.0, 1.0, 4.0], [6.0, 4.0, 4.0]], [[3.0, 1.0, 1.0], [5.0, 5.0, 3.0], [4.0, 5.0, 2.0]]], [[[2.0, 8.0, 3.0], [7.0, 
6.0, 7.0], [3.0, 4.0, 4.0]], [[8.0, 1.0, 8.0], [8.0, 3.0, 2.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 7.0], [4.0, 3.0, 3.0], [8.0, 7.0, 
4.0]]], [[[2.0, 3.0, 6.0], [8.0, 8.0, 3.0], [8.0, 2.0, 6.0]], [[8.0, 1.0, 7.0], [5.0, 3.0, 8.0], [1.0, 5.0, 3.0]], [[2.0, 1.0, 
1.0], [2.0, 2.0, 5.0], [5.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 3.0], [2.0, 8.0, 2.0], [6.0, 5.0, 4.0]], 
[[7.0, 7.0, 2.0], [4.0, 2.0, 3.0], [6.0, 1.0, 3.0]], [[7.0, 8.0, 7.0], [7.0, 7.0, 1.0], [8.0, 7.0, 4.0]]], [[[4.0, 7.0, 5.0], 
[2.0, 5.0, 2.0], [1.0, 5.0, 6.0]], [[6.0, 3.0, 7.0], [5.0, 5.0, 3.0], [2.0, 1.0, 7.0]], [[8.0, 3.0, 3.0], [4.0, 3.0, 3.0], 
[1.0, 2.0, 7.0]]], [[[8.0, 2.0, 8.0], [2.0, 3.0, 4.0], [5.0, 2.0, 6.0]], [[4.0, 5.0, 4.0], [6.0, 6.0, 8.0], [8.0, 6.0, 6.0]], 
[[7.0, 2.0, 3.0], [4.0, 2.0, 6.0], [6.0, 3.0, 8.0]]], [[[2.0, 8.0, 2.0], [7.0, 8.0, 2.0], [7.0, 7.0, 1.0]], [[4.0, 8.0, 3.0], 
[2.0, 5.0, 8.0], [1.0, 7.0, 8.0]], [[6.0, 2.0, 5.0], [2.0, 8.0, 7.0], [7.0, 1.0, 8.0]]]])
      val=numpy.array([[[[2.0, 5.0, 6.0], [5.0, 8.0, 3.0]], [[2.0, 7.0, 5.0], [8.0, 4.0, 4.0]]], [[[2.0, 1.0, 2.0], [5.0, 5.0, 
1.0]], [[4.0, 8.0, 6.0], [8.0, 5.0, 8.0]]], [[[5.0, 8.0, 3.0], [7.0, 6.0, 6.0]], [[4.0, 2.0, 5.0], [6.0, 3.0, 5.0]]], [[[6.0, 
7.0, 3.0], [2.0, 1.0, 8.0]], [[1.0, 7.0, 4.0], [7.0, 5.0, 7.0]]]])
      arg[:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 6.0], [5.0, 8.0, 3.0], [6.0, 5.0, 5.0]], [[2.0, 7.0, 5.0], [8.0, 4.0, 4.0], [2.0, 
2.0, 1.0]], [[3.0, 6.0, 6.0], [8.0, 1.0, 3.0], [8.0, 1.0, 4.0]]], [[[2.0, 1.0, 2.0], [5.0, 5.0, 1.0], [7.0, 7.0, 2.0]], [[4.0, 
8.0, 6.0], [8.0, 5.0, 8.0], [6.0, 4.0, 4.0]], [[3.0, 1.0, 1.0], [5.0, 5.0, 3.0], [4.0, 5.0, 2.0]]], [[[5.0, 8.0, 3.0], [7.0, 
6.0, 6.0], [3.0, 4.0, 4.0]], [[4.0, 2.0, 5.0], [6.0, 3.0, 5.0], [1.0, 3.0, 6.0]], [[4.0, 6.0, 7.0], [4.0, 3.0, 3.0], [8.0, 7.0, 
4.0]]], [[[6.0, 7.0, 3.0], [2.0, 1.0, 8.0], [8.0, 2.0, 6.0]], [[1.0, 7.0, 4.0], [7.0, 5.0, 7.0], [1.0, 5.0, 3.0]], [[2.0, 1.0, 
1.0], [2.0, 2.0, 5.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 6.0], [5.0, 8.0, 3.0], [6.0, 5.0, 4.0]], 
[[2.0, 7.0, 5.0], [8.0, 4.0, 4.0], [6.0, 1.0, 3.0]], [[7.0, 8.0, 7.0], [7.0, 7.0, 1.0], [8.0, 7.0, 4.0]]], [[[2.0, 1.0, 2.0], 
[5.0, 5.0, 1.0], [1.0, 5.0, 6.0]], [[4.0, 8.0, 6.0], [8.0, 5.0, 8.0], [2.0, 1.0, 7.0]], [[8.0, 3.0, 3.0], [4.0, 3.0, 3.0], 
[1.0, 2.0, 7.0]]], [[[5.0, 8.0, 3.0], [7.0, 6.0, 6.0], [5.0, 2.0, 6.0]], [[4.0, 2.0, 5.0], [6.0, 3.0, 5.0], [8.0, 6.0, 6.0]], 
[[7.0, 2.0, 3.0], [4.0, 2.0, 6.0], [6.0, 3.0, 8.0]]], [[[6.0, 7.0, 3.0], [2.0, 1.0, 8.0], [7.0, 7.0, 1.0]], [[1.0, 7.0, 4.0], 
[7.0, 5.0, 7.0], [1.0, 7.0, 8.0]], [[6.0, 2.0, 5.0], [2.0, 8.0, 7.0], [7.0, 1.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 3.0, 7.0], [4.0, 1.0, 4.0], [7.0, 5.0, 8.0]], [[7.0, 2.0, 7.0], [1.0, 3.0, 3.0], [6.0, 
3.0, 4.0]], [[8.0, 5.0, 5.0], [8.0, 8.0, 7.0], [7.0, 6.0, 7.0]]], [[[5.0, 2.0, 2.0], [7.0, 5.0, 6.0], [5.0, 6.0, 5.0]], [[8.0, 
3.0, 7.0], [3.0, 3.0, 4.0], [4.0, 3.0, 1.0]], [[6.0, 4.0, 5.0], [6.0, 5.0, 5.0], [3.0, 7.0, 4.0]]], [[[6.0, 1.0, 8.0], [6.0, 
2.0, 1.0], [5.0, 1.0, 3.0]], [[8.0, 3.0, 2.0], [4.0, 2.0, 2.0], [8.0, 2.0, 4.0]], [[4.0, 7.0, 6.0], [3.0, 2.0, 2.0], [6.0, 7.0, 
4.0]]], [[[8.0, 7.0, 3.0], [5.0, 6.0, 3.0], [5.0, 2.0, 8.0]], [[5.0, 2.0, 7.0], [5.0, 4.0, 2.0], [7.0, 8.0, 7.0]], [[2.0, 2.0, 
2.0], [1.0, 8.0, 4.0], [6.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 4.0, 5.0], [2.0, 6.0, 4.0], [5.0, 7.0, 3.0]], 
[[6.0, 1.0, 2.0], [6.0, 3.0, 8.0], [4.0, 5.0, 2.0]], [[5.0, 1.0, 7.0], [6.0, 5.0, 8.0], [7.0, 5.0, 5.0]]], [[[1.0, 7.0, 5.0], 
[3.0, 7.0, 3.0], [4.0, 8.0, 4.0]], [[3.0, 6.0, 1.0], [5.0, 6.0, 5.0], [1.0, 7.0, 6.0]], [[6.0, 4.0, 3.0], [1.0, 8.0, 7.0], 
[8.0, 6.0, 3.0]]], [[[3.0, 8.0, 2.0], [7.0, 7.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 8.0, 3.0], [4.0, 3.0, 3.0], [2.0, 5.0, 2.0]], 
[[5.0, 8.0, 8.0], [5.0, 2.0, 6.0], [2.0, 6.0, 7.0]]], [[[5.0, 5.0, 6.0], [8.0, 8.0, 2.0], [4.0, 8.0, 6.0]], [[1.0, 5.0, 4.0], 
[4.0, 4.0, 6.0], [7.0, 4.0, 8.0]], [[6.0, 3.0, 6.0], [8.0, 5.0, 8.0], [1.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 2.0]], [[1.0, 2.0, 2.0], [3.0, 4.0, 6.0]]], [[[8.0, 4.0, 1.0], [4.0, 
2.0, 8.0]], [[3.0, 8.0, 8.0], [8.0, 6.0, 8.0]]], [[[2.0, 7.0, 1.0], [4.0, 4.0, 8.0]], [[6.0, 3.0, 2.0], [8.0, 4.0, 4.0]]], 
[[[6.0, 3.0, 3.0], [8.0, 4.0, 2.0]], [[3.0, 4.0, 8.0], [2.0, 4.0, 1.0]]]]),self.functionspace)
      arg[:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 2.0], [7.0, 5.0, 8.0]], [[1.0, 2.0, 2.0], [3.0, 4.0, 6.0], [6.0, 
3.0, 4.0]], [[8.0, 5.0, 5.0], [8.0, 8.0, 7.0], [7.0, 6.0, 7.0]]], [[[8.0, 4.0, 1.0], [4.0, 2.0, 8.0], [5.0, 6.0, 5.0]], [[3.0, 
8.0, 8.0], [8.0, 6.0, 8.0], [4.0, 3.0, 1.0]], [[6.0, 4.0, 5.0], [6.0, 5.0, 5.0], [3.0, 7.0, 4.0]]], [[[2.0, 7.0, 1.0], [4.0, 
4.0, 8.0], [5.0, 1.0, 3.0]], [[6.0, 3.0, 2.0], [8.0, 4.0, 4.0], [8.0, 2.0, 4.0]], [[4.0, 7.0, 6.0], [3.0, 2.0, 2.0], [6.0, 7.0, 
4.0]]], [[[6.0, 3.0, 3.0], [8.0, 4.0, 2.0], [5.0, 2.0, 8.0]], [[3.0, 4.0, 8.0], [2.0, 4.0, 1.0], [7.0, 8.0, 7.0]], [[2.0, 2.0, 
2.0], [1.0, 8.0, 4.0], [6.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 1.0], [1.0, 7.0, 2.0], [5.0, 7.0, 3.0]], 
[[1.0, 2.0, 2.0], [3.0, 4.0, 6.0], [4.0, 5.0, 2.0]], [[5.0, 1.0, 7.0], [6.0, 5.0, 8.0], [7.0, 5.0, 5.0]]], [[[8.0, 4.0, 1.0], 
[4.0, 2.0, 8.0], [4.0, 8.0, 4.0]], [[3.0, 8.0, 8.0], [8.0, 6.0, 8.0], [1.0, 7.0, 6.0]], [[6.0, 4.0, 3.0], [1.0, 8.0, 7.0], 
[8.0, 6.0, 3.0]]], [[[2.0, 7.0, 1.0], [4.0, 4.0, 8.0], [3.0, 6.0, 3.0]], [[6.0, 3.0, 2.0], [8.0, 4.0, 4.0], [2.0, 5.0, 2.0]], 
[[5.0, 8.0, 8.0], [5.0, 2.0, 6.0], [2.0, 6.0, 7.0]]], [[[6.0, 3.0, 3.0], [8.0, 4.0, 2.0], [4.0, 8.0, 6.0]], [[3.0, 4.0, 8.0], 
[2.0, 4.0, 1.0], [7.0, 4.0, 8.0]], [[6.0, 3.0, 6.0], [8.0, 5.0, 8.0], [1.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 5.0], [2.0, 7.0, 8.0], [7.0, 5.0, 6.0]], [[2.0, 4.0, 2.0], [7.0, 8.0, 1.0], [3.0, 
4.0, 2.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 1.0], [1.0, 5.0, 1.0]]], [[[2.0, 6.0, 4.0], [8.0, 8.0, 7.0], [2.0, 5.0, 6.0]], [[7.0, 
3.0, 3.0], [1.0, 1.0, 6.0], [5.0, 2.0, 2.0]], [[8.0, 5.0, 2.0], [6.0, 8.0, 3.0], [4.0, 4.0, 1.0]]], [[[8.0, 1.0, 2.0], [2.0, 
3.0, 5.0], [6.0, 6.0, 6.0]], [[4.0, 5.0, 7.0], [4.0, 4.0, 3.0], [7.0, 7.0, 5.0]], [[4.0, 7.0, 1.0], [5.0, 5.0, 3.0], [7.0, 4.0, 
3.0]]], [[[7.0, 5.0, 5.0], [5.0, 1.0, 1.0], [3.0, 4.0, 3.0]], [[1.0, 8.0, 1.0], [2.0, 8.0, 5.0], [3.0, 4.0, 1.0]], [[5.0, 7.0, 
8.0], [4.0, 1.0, 4.0], [5.0, 6.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 4.0], [7.0, 4.0, 3.0], [5.0, 1.0, 3.0]], 
[[3.0, 4.0, 4.0], [8.0, 8.0, 1.0], [4.0, 1.0, 5.0]], [[5.0, 8.0, 6.0], [2.0, 3.0, 1.0], [8.0, 5.0, 2.0]]], [[[5.0, 6.0, 4.0], 
[3.0, 3.0, 6.0], [1.0, 4.0, 5.0]], [[4.0, 2.0, 2.0], [2.0, 6.0, 1.0], [6.0, 5.0, 8.0]], [[1.0, 4.0, 5.0], [6.0, 8.0, 1.0], 
[6.0, 8.0, 2.0]]], [[[1.0, 3.0, 5.0], [3.0, 3.0, 5.0], [4.0, 7.0, 6.0]], [[2.0, 7.0, 2.0], [8.0, 7.0, 8.0], [6.0, 3.0, 5.0]], 
[[5.0, 6.0, 7.0], [7.0, 1.0, 1.0], [4.0, 7.0, 6.0]]], [[[5.0, 6.0, 7.0], [4.0, 3.0, 3.0], [3.0, 4.0, 6.0]], [[3.0, 2.0, 2.0], 
[8.0, 2.0, 7.0], [7.0, 8.0, 7.0]], [[5.0, 1.0, 3.0], [6.0, 6.0, 4.0], [8.0, 4.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 3.0, 8.0], [3.0, 8.0, 4.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 5.0]]], [[[2.0, 6.0, 5.0], 
[3.0, 3.0, 3.0]], [[4.0, 6.0, 8.0], [7.0, 7.0, 4.0]]], [[[5.0, 2.0, 3.0], [4.0, 2.0, 3.0]], [[2.0, 8.0, 8.0], [4.0, 2.0, 
5.0]]], [[[7.0, 3.0, 8.0], [4.0, 7.0, 8.0]], [[6.0, 2.0, 8.0], [7.0, 7.0, 1.0]]]])+(1.-msk_val)*numpy.array([[[[5.0, 2.0, 2.0], 
[5.0, 2.0, 8.0]], [[6.0, 1.0, 2.0], [8.0, 6.0, 7.0]]], [[[8.0, 7.0, 1.0], [7.0, 7.0, 3.0]], [[5.0, 4.0, 6.0], [4.0, 4.0, 
3.0]]], [[[8.0, 4.0, 5.0], [3.0, 4.0, 1.0]], [[2.0, 8.0, 8.0], [1.0, 7.0, 3.0]]], [[[8.0, 2.0, 7.0], [7.0, 5.0, 6.0]], [[2.0, 
5.0, 8.0], [4.0, 2.0, 8.0]]]])
      arg[:,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 8.0], [3.0, 8.0, 4.0], [7.0, 5.0, 6.0]], [[2.0, 7.0, 6.0], [1.0, 5.0, 5.0], [3.0, 
4.0, 2.0]], [[7.0, 6.0, 5.0], [5.0, 3.0, 1.0], [1.0, 5.0, 1.0]]], [[[2.0, 6.0, 5.0], [3.0, 3.0, 3.0], [2.0, 5.0, 6.0]], [[4.0, 
6.0, 8.0], [7.0, 7.0, 4.0], [5.0, 2.0, 2.0]], [[8.0, 5.0, 2.0], [6.0, 8.0, 3.0], [4.0, 4.0, 1.0]]], [[[5.0, 2.0, 3.0], [4.0, 
2.0, 3.0], [6.0, 6.0, 6.0]], [[2.0, 8.0, 8.0], [4.0, 2.0, 5.0], [7.0, 7.0, 5.0]], [[4.0, 7.0, 1.0], [5.0, 5.0, 3.0], [7.0, 4.0, 
3.0]]], [[[7.0, 3.0, 8.0], [4.0, 7.0, 8.0], [3.0, 4.0, 3.0]], [[6.0, 2.0, 8.0], [7.0, 7.0, 1.0], [3.0, 4.0, 1.0]], [[5.0, 7.0, 
8.0], [4.0, 1.0, 4.0], [5.0, 6.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 2.0], [5.0, 2.0, 8.0], [5.0, 1.0, 3.0]], 
[[6.0, 1.0, 2.0], [8.0, 6.0, 7.0], [4.0, 1.0, 5.0]], [[5.0, 8.0, 6.0], [2.0, 3.0, 1.0], [8.0, 5.0, 2.0]]], [[[8.0, 7.0, 1.0], 
[7.0, 7.0, 3.0], [1.0, 4.0, 5.0]], [[5.0, 4.0, 6.0], [4.0, 4.0, 3.0], [6.0, 5.0, 8.0]], [[1.0, 4.0, 5.0], [6.0, 8.0, 1.0], 
[6.0, 8.0, 2.0]]], [[[8.0, 4.0, 5.0], [3.0, 4.0, 1.0], [4.0, 7.0, 6.0]], [[2.0, 8.0, 8.0], [1.0, 7.0, 3.0], [6.0, 3.0, 5.0]], 
[[5.0, 6.0, 7.0], [7.0, 1.0, 1.0], [4.0, 7.0, 6.0]]], [[[8.0, 2.0, 7.0], [7.0, 5.0, 6.0], [3.0, 4.0, 6.0]], [[2.0, 5.0, 8.0], 
[4.0, 2.0, 8.0], [7.0, 8.0, 7.0]], [[5.0, 1.0, 3.0], [6.0, 6.0, 4.0], [8.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 1.0], [7.0, 2.0, 4.0], [2.0, 7.0, 1.0]], [[4.0, 2.0, 5.0], [7.0, 8.0, 5.0], [8.0, 
5.0, 3.0]], [[1.0, 8.0, 6.0], [1.0, 8.0, 3.0], [4.0, 4.0, 8.0]]], [[[6.0, 4.0, 6.0], [3.0, 1.0, 1.0], [3.0, 1.0, 4.0]], [[7.0, 
1.0, 5.0], [1.0, 2.0, 1.0], [1.0, 7.0, 8.0]], [[4.0, 7.0, 1.0], [1.0, 3.0, 1.0], [3.0, 3.0, 3.0]]], [[[5.0, 5.0, 3.0], [1.0, 
7.0, 6.0], [8.0, 4.0, 8.0]], [[1.0, 1.0, 4.0], [2.0, 6.0, 5.0], [3.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [2.0, 1.0, 1.0], [3.0, 4.0, 
4.0]]], [[[8.0, 2.0, 5.0], [1.0, 4.0, 2.0], [3.0, 6.0, 4.0]], [[5.0, 5.0, 5.0], [8.0, 5.0, 2.0], [6.0, 7.0, 5.0]], [[1.0, 1.0, 
7.0], [7.0, 1.0, 7.0], [5.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 6.0], [3.0, 8.0, 2.0], [8.0, 7.0, 1.0]], 
[[7.0, 7.0, 4.0], [5.0, 3.0, 6.0], [1.0, 8.0, 3.0]], [[6.0, 2.0, 3.0], [2.0, 5.0, 8.0], [5.0, 4.0, 1.0]]], [[[1.0, 5.0, 1.0], 
[6.0, 6.0, 8.0], [4.0, 8.0, 1.0]], [[6.0, 1.0, 5.0], [6.0, 6.0, 1.0], [8.0, 4.0, 3.0]], [[6.0, 2.0, 5.0], [8.0, 6.0, 8.0], 
[8.0, 3.0, 6.0]]], [[[3.0, 6.0, 8.0], [1.0, 7.0, 1.0], [3.0, 1.0, 4.0]], [[6.0, 1.0, 7.0], [5.0, 6.0, 8.0], [3.0, 5.0, 6.0]], 
[[2.0, 4.0, 6.0], [8.0, 1.0, 3.0], [3.0, 3.0, 5.0]]], [[[6.0, 6.0, 3.0], [1.0, 1.0, 5.0], [1.0, 2.0, 4.0]], [[3.0, 4.0, 2.0], 
[5.0, 6.0, 7.0], [2.0, 3.0, 5.0]], [[6.0, 7.0, 3.0], [6.0, 2.0, 8.0], [6.0, 3.0, 8.0]]]])
      val=numpy.array([8.0, 4.0, 2.0])
      arg[2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 1.0], [7.0, 2.0, 4.0], [2.0, 7.0, 1.0]], [[4.0, 2.0, 5.0], [7.0, 8.0, 5.0], [8.0, 
5.0, 3.0]], [[1.0, 8.0, 6.0], [1.0, 8.0, 3.0], [4.0, 4.0, 8.0]]], [[[6.0, 4.0, 6.0], [3.0, 1.0, 1.0], [3.0, 1.0, 4.0]], [[7.0, 
1.0, 5.0], [1.0, 2.0, 1.0], [1.0, 7.0, 8.0]], [[4.0, 7.0, 1.0], [1.0, 3.0, 1.0], [3.0, 3.0, 3.0]]], [[[5.0, 5.0, 3.0], [1.0, 
7.0, 6.0], [8.0, 4.0, 8.0]], [[1.0, 1.0, 4.0], [8.0, 4.0, 2.0], [3.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [2.0, 1.0, 1.0], [3.0, 4.0, 
4.0]]], [[[8.0, 2.0, 5.0], [1.0, 4.0, 2.0], [3.0, 6.0, 4.0]], [[5.0, 5.0, 5.0], [8.0, 5.0, 2.0], [6.0, 7.0, 5.0]], [[1.0, 1.0, 
7.0], [7.0, 1.0, 7.0], [5.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 6.0], [3.0, 8.0, 2.0], [8.0, 7.0, 1.0]], 
[[7.0, 7.0, 4.0], [5.0, 3.0, 6.0], [1.0, 8.0, 3.0]], [[6.0, 2.0, 3.0], [2.0, 5.0, 8.0], [5.0, 4.0, 1.0]]], [[[1.0, 5.0, 1.0], 
[6.0, 6.0, 8.0], [4.0, 8.0, 1.0]], [[6.0, 1.0, 5.0], [6.0, 6.0, 1.0], [8.0, 4.0, 3.0]], [[6.0, 2.0, 5.0], [8.0, 6.0, 8.0], 
[8.0, 3.0, 6.0]]], [[[3.0, 6.0, 8.0], [1.0, 7.0, 1.0], [3.0, 1.0, 4.0]], [[6.0, 1.0, 7.0], [8.0, 4.0, 2.0], [3.0, 5.0, 6.0]], 
[[2.0, 4.0, 6.0], [8.0, 1.0, 3.0], [3.0, 3.0, 5.0]]], [[[6.0, 6.0, 3.0], [1.0, 1.0, 5.0], [1.0, 2.0, 4.0]], [[3.0, 4.0, 2.0], 
[5.0, 6.0, 7.0], [2.0, 3.0, 5.0]], [[6.0, 7.0, 3.0], [6.0, 2.0, 8.0], [6.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 2.0, 6.0], [2.0, 5.0, 8.0]], [[2.0, 5.0, 1.0], [4.0, 7.0, 4.0], [3.0, 
7.0, 5.0]], [[5.0, 5.0, 6.0], [7.0, 5.0, 6.0], [1.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], [1.0, 3.0, 4.0], [4.0, 1.0, 3.0]], [[2.0, 
8.0, 8.0], [3.0, 7.0, 5.0], [4.0, 3.0, 1.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 1.0], [4.0, 7.0, 3.0]]], [[[2.0, 3.0, 7.0], [5.0, 
3.0, 1.0], [6.0, 7.0, 5.0]], [[4.0, 6.0, 6.0], [6.0, 7.0, 4.0], [7.0, 6.0, 3.0]], [[3.0, 4.0, 4.0], [4.0, 7.0, 1.0], [3.0, 5.0, 
2.0]]], [[[4.0, 8.0, 4.0], [6.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[6.0, 6.0, 3.0], [3.0, 6.0, 3.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 
8.0], [8.0, 7.0, 1.0], [5.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 1.0, 4.0], [6.0, 6.0, 4.0], [7.0, 1.0, 2.0]], 
[[4.0, 8.0, 8.0], [4.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 6.0, 1.0], [5.0, 8.0, 1.0], [6.0, 7.0, 2.0]]], [[[4.0, 3.0, 3.0], 
[5.0, 2.0, 7.0], [7.0, 1.0, 8.0]], [[4.0, 1.0, 7.0], [4.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[3.0, 3.0, 5.0], [2.0, 4.0, 6.0], 
[5.0, 8.0, 4.0]]], [[[3.0, 4.0, 5.0], [3.0, 3.0, 8.0], [2.0, 8.0, 5.0]], [[1.0, 6.0, 5.0], [7.0, 8.0, 3.0], [2.0, 3.0, 4.0]], 
[[5.0, 6.0, 6.0], [6.0, 1.0, 4.0], [1.0, 8.0, 2.0]]], [[[1.0, 2.0, 2.0], [7.0, 4.0, 7.0], [6.0, 4.0, 5.0]], [[7.0, 2.0, 5.0], 
[8.0, 2.0, 7.0], [7.0, 8.0, 4.0]], [[4.0, 8.0, 7.0], [5.0, 5.0, 1.0], [5.0, 5.0, 6.0]]]])
      val=Data(numpy.array([8.0, 7.0, 1.0]),self.functionspace)
      arg[2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 2.0, 6.0], [2.0, 5.0, 8.0]], [[2.0, 5.0, 1.0], [4.0, 7.0, 4.0], [3.0, 
7.0, 5.0]], [[5.0, 5.0, 6.0], [7.0, 5.0, 6.0], [1.0, 6.0, 3.0]]], [[[1.0, 7.0, 8.0], [1.0, 3.0, 4.0], [4.0, 1.0, 3.0]], [[2.0, 
8.0, 8.0], [3.0, 7.0, 5.0], [4.0, 3.0, 1.0]], [[2.0, 2.0, 4.0], [8.0, 4.0, 1.0], [4.0, 7.0, 3.0]]], [[[2.0, 3.0, 7.0], [5.0, 
3.0, 1.0], [6.0, 7.0, 5.0]], [[4.0, 6.0, 6.0], [8.0, 7.0, 1.0], [7.0, 6.0, 3.0]], [[3.0, 4.0, 4.0], [4.0, 7.0, 1.0], [3.0, 5.0, 
2.0]]], [[[4.0, 8.0, 4.0], [6.0, 7.0, 5.0], [8.0, 4.0, 8.0]], [[6.0, 6.0, 3.0], [3.0, 6.0, 3.0], [6.0, 7.0, 1.0]], [[7.0, 5.0, 
8.0], [8.0, 7.0, 1.0], [5.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 1.0, 4.0], [6.0, 6.0, 4.0], [7.0, 1.0, 2.0]], 
[[4.0, 8.0, 8.0], [4.0, 5.0, 1.0], [7.0, 8.0, 5.0]], [[3.0, 6.0, 1.0], [5.0, 8.0, 1.0], [6.0, 7.0, 2.0]]], [[[4.0, 3.0, 3.0], 
[5.0, 2.0, 7.0], [7.0, 1.0, 8.0]], [[4.0, 1.0, 7.0], [4.0, 8.0, 4.0], [5.0, 7.0, 7.0]], [[3.0, 3.0, 5.0], [2.0, 4.0, 6.0], 
[5.0, 8.0, 4.0]]], [[[3.0, 4.0, 5.0], [3.0, 3.0, 8.0], [2.0, 8.0, 5.0]], [[1.0, 6.0, 5.0], [8.0, 7.0, 1.0], [2.0, 3.0, 4.0]], 
[[5.0, 6.0, 6.0], [6.0, 1.0, 4.0], [1.0, 8.0, 2.0]]], [[[1.0, 2.0, 2.0], [7.0, 4.0, 7.0], [6.0, 4.0, 5.0]], [[7.0, 2.0, 5.0], 
[8.0, 2.0, 7.0], [7.0, 8.0, 4.0]], [[4.0, 8.0, 7.0], [5.0, 5.0, 1.0], [5.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 1.0, 8.0], [1.0, 5.0, 2.0], [4.0, 8.0, 3.0]], [[3.0, 4.0, 4.0], [1.0, 8.0, 4.0], [2.0, 
2.0, 3.0]], [[8.0, 2.0, 5.0], [5.0, 1.0, 2.0], [6.0, 2.0, 8.0]]], [[[7.0, 5.0, 1.0], [2.0, 2.0, 3.0], [4.0, 5.0, 2.0]], [[5.0, 
8.0, 7.0], [5.0, 4.0, 2.0], [2.0, 4.0, 5.0]], [[7.0, 7.0, 8.0], [7.0, 2.0, 3.0], [4.0, 5.0, 7.0]]], [[[6.0, 8.0, 1.0], [8.0, 
2.0, 6.0], [1.0, 2.0, 6.0]], [[2.0, 7.0, 4.0], [8.0, 1.0, 6.0], [1.0, 7.0, 4.0]], [[7.0, 5.0, 4.0], [7.0, 5.0, 4.0], [5.0, 3.0, 
2.0]]], [[[6.0, 2.0, 5.0], [7.0, 5.0, 4.0], [7.0, 5.0, 6.0]], [[8.0, 1.0, 3.0], [2.0, 7.0, 1.0], [4.0, 3.0, 4.0]], [[2.0, 5.0, 
1.0], [5.0, 6.0, 3.0], [3.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 1.0], [3.0, 7.0, 3.0], [7.0, 4.0, 7.0]], 
[[4.0, 4.0, 7.0], [4.0, 3.0, 8.0], [2.0, 8.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 6.0, 7.0], [7.0, 1.0, 1.0]]], [[[2.0, 2.0, 8.0], 
[7.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 1.0], [7.0, 1.0, 7.0]], [[8.0, 6.0, 7.0], [8.0, 6.0, 4.0], 
[7.0, 8.0, 4.0]]], [[[8.0, 7.0, 3.0], [2.0, 5.0, 4.0], [7.0, 4.0, 6.0]], [[2.0, 5.0, 1.0], [7.0, 6.0, 6.0], [7.0, 5.0, 5.0]], 
[[3.0, 1.0, 8.0], [7.0, 6.0, 6.0], [7.0, 3.0, 7.0]]], [[[6.0, 8.0, 7.0], [6.0, 1.0, 2.0], [3.0, 6.0, 6.0]], [[8.0, 2.0, 3.0], 
[5.0, 1.0, 1.0], [6.0, 6.0, 5.0]], [[2.0, 8.0, 1.0], [7.0, 8.0, 3.0], [5.0, 6.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([5.0, 7.0, 4.0])+(1.-msk_val)*numpy.array([3.0, 7.0, 6.0])
      arg[2,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 1.0, 8.0], [1.0, 5.0, 2.0], [4.0, 8.0, 3.0]], [[3.0, 4.0, 4.0], [1.0, 8.0, 4.0], [2.0, 
2.0, 3.0]], [[8.0, 2.0, 5.0], [5.0, 1.0, 2.0], [6.0, 2.0, 8.0]]], [[[7.0, 5.0, 1.0], [2.0, 2.0, 3.0], [4.0, 5.0, 2.0]], [[5.0, 
8.0, 7.0], [5.0, 4.0, 2.0], [2.0, 4.0, 5.0]], [[7.0, 7.0, 8.0], [7.0, 2.0, 3.0], [4.0, 5.0, 7.0]]], [[[6.0, 8.0, 1.0], [8.0, 
2.0, 6.0], [1.0, 2.0, 6.0]], [[2.0, 7.0, 4.0], [5.0, 7.0, 4.0], [1.0, 7.0, 4.0]], [[7.0, 5.0, 4.0], [7.0, 5.0, 4.0], [5.0, 3.0, 
2.0]]], [[[6.0, 2.0, 5.0], [7.0, 5.0, 4.0], [7.0, 5.0, 6.0]], [[8.0, 1.0, 3.0], [2.0, 7.0, 1.0], [4.0, 3.0, 4.0]], [[2.0, 5.0, 
1.0], [5.0, 6.0, 3.0], [3.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 1.0], [3.0, 7.0, 3.0], [7.0, 4.0, 7.0]], 
[[4.0, 4.0, 7.0], [4.0, 3.0, 8.0], [2.0, 8.0, 7.0]], [[6.0, 6.0, 7.0], [1.0, 6.0, 7.0], [7.0, 1.0, 1.0]]], [[[2.0, 2.0, 8.0], 
[7.0, 6.0, 5.0], [4.0, 2.0, 4.0]], [[2.0, 2.0, 7.0], [3.0, 8.0, 1.0], [7.0, 1.0, 7.0]], [[8.0, 6.0, 7.0], [8.0, 6.0, 4.0], 
[7.0, 8.0, 4.0]]], [[[8.0, 7.0, 3.0], [2.0, 5.0, 4.0], [7.0, 4.0, 6.0]], [[2.0, 5.0, 1.0], [3.0, 7.0, 6.0], [7.0, 5.0, 5.0]], 
[[3.0, 1.0, 8.0], [7.0, 6.0, 6.0], [7.0, 3.0, 7.0]]], [[[6.0, 8.0, 7.0], [6.0, 1.0, 2.0], [3.0, 6.0, 6.0]], [[8.0, 2.0, 3.0], 
[5.0, 1.0, 1.0], [6.0, 6.0, 5.0]], [[2.0, 8.0, 1.0], [7.0, 8.0, 3.0], [5.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 7.0, 4.0], [8.0, 2.0, 1.0], [4.0, 6.0, 6.0]], [[2.0, 5.0, 5.0], [2.0, 2.0, 5.0], [1.0, 
8.0, 3.0]], [[2.0, 4.0, 7.0], [6.0, 6.0, 1.0], [3.0, 4.0, 5.0]]], [[[1.0, 1.0, 8.0], [7.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[3.0, 
3.0, 7.0], [2.0, 7.0, 7.0], [8.0, 6.0, 4.0]], [[2.0, 1.0, 6.0], [7.0, 4.0, 8.0], [1.0, 3.0, 7.0]]], [[[5.0, 1.0, 4.0], [1.0, 
1.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 7.0, 5.0], [6.0, 3.0, 5.0], [4.0, 8.0, 3.0]], [[7.0, 7.0, 8.0], [1.0, 4.0, 1.0], [3.0, 8.0, 
8.0]]], [[[2.0, 8.0, 2.0], [1.0, 1.0, 7.0], [8.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [3.0, 7.0, 3.0], [7.0, 7.0, 8.0]], [[2.0, 3.0, 
4.0], [1.0, 2.0, 2.0], [5.0, 1.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 6.0], [5.0, 8.0, 6.0], [8.0, 5.0, 6.0]], 
[[2.0, 8.0, 1.0], [4.0, 6.0, 7.0], [6.0, 6.0, 2.0]], [[7.0, 3.0, 5.0], [6.0, 2.0, 2.0], [4.0, 1.0, 4.0]]], [[[3.0, 7.0, 2.0], 
[2.0, 7.0, 8.0], [8.0, 6.0, 8.0]], [[6.0, 3.0, 5.0], [2.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[1.0, 6.0, 3.0], [8.0, 7.0, 1.0], 
[3.0, 6.0, 7.0]]], [[[6.0, 2.0, 7.0], [3.0, 8.0, 5.0], [4.0, 3.0, 3.0]], [[3.0, 4.0, 5.0], [5.0, 2.0, 4.0], [2.0, 6.0, 4.0]], 
[[8.0, 1.0, 1.0], [5.0, 4.0, 5.0], [8.0, 7.0, 5.0]]], [[[6.0, 7.0, 1.0], [8.0, 4.0, 4.0], [8.0, 8.0, 1.0]], [[2.0, 2.0, 2.0], 
[8.0, 6.0, 8.0], [4.0, 3.0, 7.0]], [[5.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 6.0, 1.0]]]])
      val=numpy.array([[4.0, 5.0, 4.0]])
      arg[2,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 7.0, 4.0], [8.0, 2.0, 1.0], [4.0, 6.0, 6.0]], [[2.0, 5.0, 5.0], [2.0, 2.0, 5.0], [1.0, 
8.0, 3.0]], [[2.0, 4.0, 7.0], [6.0, 6.0, 1.0], [3.0, 4.0, 5.0]]], [[[1.0, 1.0, 8.0], [7.0, 7.0, 5.0], [2.0, 7.0, 1.0]], [[3.0, 
3.0, 7.0], [2.0, 7.0, 7.0], [8.0, 6.0, 4.0]], [[2.0, 1.0, 6.0], [7.0, 4.0, 8.0], [1.0, 3.0, 7.0]]], [[[5.0, 1.0, 4.0], [1.0, 
1.0, 4.0], [2.0, 2.0, 2.0]], [[3.0, 7.0, 5.0], [6.0, 3.0, 5.0], [4.0, 8.0, 3.0]], [[7.0, 7.0, 8.0], [4.0, 5.0, 4.0], [3.0, 8.0, 
8.0]]], [[[2.0, 8.0, 2.0], [1.0, 1.0, 7.0], [8.0, 1.0, 8.0]], [[5.0, 1.0, 6.0], [3.0, 7.0, 3.0], [7.0, 7.0, 8.0]], [[2.0, 3.0, 
4.0], [1.0, 2.0, 2.0], [5.0, 1.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 6.0], [5.0, 8.0, 6.0], [8.0, 5.0, 6.0]], 
[[2.0, 8.0, 1.0], [4.0, 6.0, 7.0], [6.0, 6.0, 2.0]], [[7.0, 3.0, 5.0], [6.0, 2.0, 2.0], [4.0, 1.0, 4.0]]], [[[3.0, 7.0, 2.0], 
[2.0, 7.0, 8.0], [8.0, 6.0, 8.0]], [[6.0, 3.0, 5.0], [2.0, 6.0, 7.0], [3.0, 6.0, 7.0]], [[1.0, 6.0, 3.0], [8.0, 7.0, 1.0], 
[3.0, 6.0, 7.0]]], [[[6.0, 2.0, 7.0], [3.0, 8.0, 5.0], [4.0, 3.0, 3.0]], [[3.0, 4.0, 5.0], [5.0, 2.0, 4.0], [2.0, 6.0, 4.0]], 
[[8.0, 1.0, 1.0], [4.0, 5.0, 4.0], [8.0, 7.0, 5.0]]], [[[6.0, 7.0, 1.0], [8.0, 4.0, 4.0], [8.0, 8.0, 1.0]], [[2.0, 2.0, 2.0], 
[8.0, 6.0, 8.0], [4.0, 3.0, 7.0]], [[5.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 6.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 2.0, 2.0], [4.0, 6.0, 2.0], [2.0, 7.0, 3.0]], [[1.0, 8.0, 5.0], [3.0, 7.0, 7.0], [1.0, 
2.0, 7.0]], [[5.0, 8.0, 8.0], [4.0, 7.0, 3.0], [6.0, 3.0, 1.0]]], [[[7.0, 7.0, 4.0], [4.0, 2.0, 3.0], [8.0, 3.0, 3.0]], [[4.0, 
2.0, 7.0], [5.0, 4.0, 4.0], [6.0, 7.0, 5.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 4.0], [8.0, 5.0, 3.0]]], [[[1.0, 4.0, 6.0], [4.0, 
3.0, 4.0], [4.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], [6.0, 3.0, 6.0], [4.0, 8.0, 8.0]], [[3.0, 1.0, 7.0], [5.0, 3.0, 3.0], [2.0, 8.0, 
6.0]]], [[[7.0, 7.0, 1.0], [6.0, 5.0, 4.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 6.0], [2.0, 2.0, 6.0], [1.0, 3.0, 1.0]], [[4.0, 1.0, 
8.0], [8.0, 6.0, 3.0], [1.0, 2.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 6.0, 2.0], [3.0, 2.0, 6.0], [4.0, 6.0, 3.0]], 
[[5.0, 7.0, 8.0], [2.0, 5.0, 5.0], [7.0, 6.0, 2.0]], [[8.0, 1.0, 4.0], [1.0, 5.0, 2.0], [4.0, 6.0, 2.0]]], [[[7.0, 8.0, 1.0], 
[6.0, 3.0, 2.0], [6.0, 4.0, 2.0]], [[5.0, 3.0, 2.0], [3.0, 2.0, 4.0], [4.0, 5.0, 1.0]], [[3.0, 2.0, 4.0], [3.0, 6.0, 2.0], 
[2.0, 5.0, 7.0]]], [[[5.0, 3.0, 1.0], [2.0, 2.0, 3.0], [5.0, 5.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 6.0, 8.0], [7.0, 3.0, 1.0]], 
[[3.0, 7.0, 6.0], [3.0, 7.0, 1.0], [4.0, 2.0, 8.0]]], [[[7.0, 3.0, 7.0], [3.0, 2.0, 2.0], [6.0, 3.0, 7.0]], [[7.0, 7.0, 6.0], 
[7.0, 8.0, 8.0], [8.0, 2.0, 3.0]], [[4.0, 1.0, 2.0], [2.0, 4.0, 3.0], [5.0, 3.0, 1.0]]]])
      val=Data(numpy.array([[6.0, 7.0, 2.0]]),self.functionspace)
      arg[2,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 2.0, 2.0], [4.0, 6.0, 2.0], [2.0, 7.0, 3.0]], [[1.0, 8.0, 5.0], [3.0, 7.0, 7.0], [1.0, 
2.0, 7.0]], [[5.0, 8.0, 8.0], [4.0, 7.0, 3.0], [6.0, 3.0, 1.0]]], [[[7.0, 7.0, 4.0], [4.0, 2.0, 3.0], [8.0, 3.0, 3.0]], [[4.0, 
2.0, 7.0], [5.0, 4.0, 4.0], [6.0, 7.0, 5.0]], [[3.0, 8.0, 8.0], [4.0, 6.0, 4.0], [8.0, 5.0, 3.0]]], [[[1.0, 4.0, 6.0], [4.0, 
3.0, 4.0], [4.0, 3.0, 1.0]], [[4.0, 1.0, 4.0], [6.0, 3.0, 6.0], [4.0, 8.0, 8.0]], [[3.0, 1.0, 7.0], [6.0, 7.0, 2.0], [2.0, 8.0, 
6.0]]], [[[7.0, 7.0, 1.0], [6.0, 5.0, 4.0], [7.0, 6.0, 8.0]], [[3.0, 5.0, 6.0], [2.0, 2.0, 6.0], [1.0, 3.0, 1.0]], [[4.0, 1.0, 
8.0], [8.0, 6.0, 3.0], [1.0, 2.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 6.0, 2.0], [3.0, 2.0, 6.0], [4.0, 6.0, 3.0]], 
[[5.0, 7.0, 8.0], [2.0, 5.0, 5.0], [7.0, 6.0, 2.0]], [[8.0, 1.0, 4.0], [1.0, 5.0, 2.0], [4.0, 6.0, 2.0]]], [[[7.0, 8.0, 1.0], 
[6.0, 3.0, 2.0], [6.0, 4.0, 2.0]], [[5.0, 3.0, 2.0], [3.0, 2.0, 4.0], [4.0, 5.0, 1.0]], [[3.0, 2.0, 4.0], [3.0, 6.0, 2.0], 
[2.0, 5.0, 7.0]]], [[[5.0, 3.0, 1.0], [2.0, 2.0, 3.0], [5.0, 5.0, 3.0]], [[8.0, 5.0, 8.0], [5.0, 6.0, 8.0], [7.0, 3.0, 1.0]], 
[[3.0, 7.0, 6.0], [6.0, 7.0, 2.0], [4.0, 2.0, 8.0]]], [[[7.0, 3.0, 7.0], [3.0, 2.0, 2.0], [6.0, 3.0, 7.0]], [[7.0, 7.0, 6.0], 
[7.0, 8.0, 8.0], [8.0, 2.0, 3.0]], [[4.0, 1.0, 2.0], [2.0, 4.0, 3.0], [5.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 1.0], [7.0, 4.0, 3.0], [7.0, 7.0, 5.0]], [[6.0, 3.0, 8.0], [1.0, 4.0, 2.0], [3.0, 
2.0, 5.0]], [[2.0, 2.0, 3.0], [6.0, 2.0, 8.0], [8.0, 1.0, 5.0]]], [[[5.0, 3.0, 7.0], [7.0, 8.0, 5.0], [3.0, 5.0, 8.0]], [[8.0, 
8.0, 6.0], [1.0, 4.0, 7.0], [4.0, 2.0, 8.0]], [[7.0, 5.0, 1.0], [1.0, 8.0, 8.0], [3.0, 4.0, 1.0]]], [[[7.0, 8.0, 6.0], [1.0, 
3.0, 4.0], [1.0, 2.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 7.0, 7.0], [1.0, 6.0, 7.0]], [[4.0, 3.0, 2.0], [8.0, 5.0, 1.0], [8.0, 6.0, 
5.0]]], [[[7.0, 3.0, 1.0], [5.0, 3.0, 1.0], [7.0, 6.0, 6.0]], [[8.0, 6.0, 2.0], [5.0, 7.0, 5.0], [2.0, 5.0, 7.0]], [[1.0, 5.0, 
7.0], [8.0, 2.0, 3.0], [4.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 5.0], [8.0, 8.0, 7.0], [1.0, 8.0, 8.0]], 
[[1.0, 4.0, 2.0], [7.0, 5.0, 7.0], [5.0, 4.0, 2.0]], [[2.0, 6.0, 8.0], [4.0, 5.0, 2.0], [8.0, 7.0, 4.0]]], [[[4.0, 1.0, 6.0], 
[1.0, 1.0, 2.0], [1.0, 3.0, 7.0]], [[4.0, 7.0, 5.0], [2.0, 7.0, 6.0], [7.0, 3.0, 7.0]], [[4.0, 8.0, 8.0], [8.0, 6.0, 3.0], 
[1.0, 3.0, 6.0]]], [[[4.0, 1.0, 6.0], [7.0, 7.0, 7.0], [7.0, 2.0, 6.0]], [[6.0, 6.0, 7.0], [1.0, 7.0, 5.0], [1.0, 2.0, 4.0]], 
[[8.0, 7.0, 5.0], [8.0, 7.0, 2.0], [7.0, 5.0, 2.0]]], [[[1.0, 1.0, 5.0], [3.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[8.0, 7.0, 3.0], 
[8.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[4.0, 6.0, 2.0], [2.0, 1.0, 8.0], [6.0, 3.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[8.0, 4.0, 3.0]])+(1.-msk_val)*numpy.array([[3.0, 8.0, 7.0]])
      arg[2,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 1.0], [7.0, 4.0, 3.0], [7.0, 7.0, 5.0]], [[6.0, 3.0, 8.0], [1.0, 4.0, 2.0], [3.0, 
2.0, 5.0]], [[2.0, 2.0, 3.0], [6.0, 2.0, 8.0], [8.0, 1.0, 5.0]]], [[[5.0, 3.0, 7.0], [7.0, 8.0, 5.0], [3.0, 5.0, 8.0]], [[8.0, 
8.0, 6.0], [1.0, 4.0, 7.0], [4.0, 2.0, 8.0]], [[7.0, 5.0, 1.0], [1.0, 8.0, 8.0], [3.0, 4.0, 1.0]]], [[[7.0, 8.0, 6.0], [1.0, 
3.0, 4.0], [1.0, 2.0, 6.0]], [[2.0, 5.0, 7.0], [1.0, 7.0, 7.0], [1.0, 6.0, 7.0]], [[4.0, 3.0, 2.0], [8.0, 4.0, 3.0], [8.0, 6.0, 
5.0]]], [[[7.0, 3.0, 1.0], [5.0, 3.0, 1.0], [7.0, 6.0, 6.0]], [[8.0, 6.0, 2.0], [5.0, 7.0, 5.0], [2.0, 5.0, 7.0]], [[1.0, 5.0, 
7.0], [8.0, 2.0, 3.0], [4.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 5.0], [8.0, 8.0, 7.0], [1.0, 8.0, 8.0]], 
[[1.0, 4.0, 2.0], [7.0, 5.0, 7.0], [5.0, 4.0, 2.0]], [[2.0, 6.0, 8.0], [4.0, 5.0, 2.0], [8.0, 7.0, 4.0]]], [[[4.0, 1.0, 6.0], 
[1.0, 1.0, 2.0], [1.0, 3.0, 7.0]], [[4.0, 7.0, 5.0], [2.0, 7.0, 6.0], [7.0, 3.0, 7.0]], [[4.0, 8.0, 8.0], [8.0, 6.0, 3.0], 
[1.0, 3.0, 6.0]]], [[[4.0, 1.0, 6.0], [7.0, 7.0, 7.0], [7.0, 2.0, 6.0]], [[6.0, 6.0, 7.0], [1.0, 7.0, 5.0], [1.0, 2.0, 4.0]], 
[[8.0, 7.0, 5.0], [3.0, 8.0, 7.0], [7.0, 5.0, 2.0]]], [[[1.0, 1.0, 5.0], [3.0, 4.0, 2.0], [3.0, 3.0, 6.0]], [[8.0, 7.0, 3.0], 
[8.0, 2.0, 2.0], [5.0, 4.0, 4.0]], [[4.0, 6.0, 2.0], [2.0, 1.0, 8.0], [6.0, 3.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 5.0, 4.0], [3.0, 6.0, 2.0], [7.0, 1.0, 1.0]], [[2.0, 1.0, 7.0], [6.0, 1.0, 3.0], [3.0, 
4.0, 7.0]], [[1.0, 8.0, 3.0], [1.0, 4.0, 8.0], [7.0, 7.0, 4.0]]], [[[5.0, 6.0, 5.0], [6.0, 1.0, 8.0], [4.0, 6.0, 4.0]], [[6.0, 
4.0, 4.0], [5.0, 2.0, 4.0], [2.0, 6.0, 8.0]], [[7.0, 3.0, 8.0], [6.0, 5.0, 7.0], [5.0, 5.0, 5.0]]], [[[5.0, 4.0, 3.0], [7.0, 
1.0, 6.0], [2.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [8.0, 7.0, 6.0], [5.0, 7.0, 1.0]], [[5.0, 8.0, 2.0], [6.0, 2.0, 5.0], [5.0, 3.0, 
7.0]]], [[[1.0, 3.0, 7.0], [7.0, 3.0, 7.0], [5.0, 1.0, 2.0]], [[5.0, 6.0, 6.0], [2.0, 6.0, 6.0], [2.0, 7.0, 7.0]], [[6.0, 2.0, 
3.0], [4.0, 2.0, 1.0], [8.0, 8.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 1.0], [7.0, 5.0, 1.0], [8.0, 1.0, 3.0]], 
[[4.0, 1.0, 8.0], [2.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 1.0, 4.0], [5.0, 6.0, 5.0], [3.0, 3.0, 6.0]]], [[[4.0, 3.0, 2.0], 
[8.0, 5.0, 7.0], [8.0, 6.0, 8.0]], [[1.0, 2.0, 4.0], [6.0, 4.0, 7.0], [6.0, 1.0, 4.0]], [[4.0, 4.0, 2.0], [6.0, 6.0, 3.0], 
[6.0, 3.0, 3.0]]], [[[3.0, 3.0, 8.0], [5.0, 2.0, 4.0], [3.0, 8.0, 7.0]], [[8.0, 2.0, 1.0], [4.0, 7.0, 5.0], [4.0, 8.0, 2.0]], 
[[4.0, 6.0, 7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 1.0]]], [[[8.0, 7.0, 8.0], [3.0, 5.0, 4.0], [3.0, 6.0, 1.0]], [[6.0, 6.0, 5.0], 
[8.0, 6.0, 8.0], [7.0, 5.0, 4.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 3.0], [2.0, 5.0, 5.0]]]])
      val=numpy.array([[3.0, 1.0, 6.0], [8.0, 6.0, 5.0]])
      arg[1:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 5.0, 4.0], [3.0, 6.0, 2.0], [7.0, 1.0, 1.0]], [[2.0, 1.0, 7.0], [6.0, 1.0, 3.0], [3.0, 
4.0, 7.0]], [[1.0, 8.0, 3.0], [1.0, 4.0, 8.0], [7.0, 7.0, 4.0]]], [[[5.0, 6.0, 5.0], [6.0, 1.0, 8.0], [4.0, 6.0, 4.0]], [[6.0, 
4.0, 4.0], [3.0, 1.0, 6.0], [2.0, 6.0, 8.0]], [[7.0, 3.0, 8.0], [6.0, 5.0, 7.0], [5.0, 5.0, 5.0]]], [[[5.0, 4.0, 3.0], [7.0, 
1.0, 6.0], [2.0, 3.0, 6.0]], [[1.0, 6.0, 4.0], [8.0, 6.0, 5.0], [5.0, 7.0, 1.0]], [[5.0, 8.0, 2.0], [6.0, 2.0, 5.0], [5.0, 3.0, 
7.0]]], [[[1.0, 3.0, 7.0], [7.0, 3.0, 7.0], [5.0, 1.0, 2.0]], [[5.0, 6.0, 6.0], [2.0, 6.0, 6.0], [2.0, 7.0, 7.0]], [[6.0, 2.0, 
3.0], [4.0, 2.0, 1.0], [8.0, 8.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 1.0], [7.0, 5.0, 1.0], [8.0, 1.0, 3.0]], 
[[4.0, 1.0, 8.0], [2.0, 4.0, 3.0], [4.0, 4.0, 6.0]], [[5.0, 1.0, 4.0], [5.0, 6.0, 5.0], [3.0, 3.0, 6.0]]], [[[4.0, 3.0, 2.0], 
[8.0, 5.0, 7.0], [8.0, 6.0, 8.0]], [[1.0, 2.0, 4.0], [3.0, 1.0, 6.0], [6.0, 1.0, 4.0]], [[4.0, 4.0, 2.0], [6.0, 6.0, 3.0], 
[6.0, 3.0, 3.0]]], [[[3.0, 3.0, 8.0], [5.0, 2.0, 4.0], [3.0, 8.0, 7.0]], [[8.0, 2.0, 1.0], [8.0, 6.0, 5.0], [4.0, 8.0, 2.0]], 
[[4.0, 6.0, 7.0], [4.0, 7.0, 3.0], [3.0, 4.0, 1.0]]], [[[8.0, 7.0, 8.0], [3.0, 5.0, 4.0], [3.0, 6.0, 1.0]], [[6.0, 6.0, 5.0], 
[8.0, 6.0, 8.0], [7.0, 5.0, 4.0]], [[6.0, 5.0, 6.0], [1.0, 8.0, 3.0], [2.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 4.0], [2.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[5.0, 2.0, 7.0], [1.0, 7.0, 5.0], [5.0, 
6.0, 5.0]], [[3.0, 4.0, 1.0], [2.0, 1.0, 3.0], [4.0, 5.0, 2.0]]], [[[5.0, 6.0, 5.0], [4.0, 5.0, 4.0], [5.0, 4.0, 4.0]], [[8.0, 
5.0, 6.0], [3.0, 7.0, 4.0], [5.0, 2.0, 7.0]], [[1.0, 3.0, 6.0], [7.0, 4.0, 2.0], [4.0, 8.0, 4.0]]], [[[2.0, 4.0, 5.0], [8.0, 
5.0, 1.0], [2.0, 2.0, 1.0]], [[4.0, 7.0, 2.0], [2.0, 2.0, 3.0], [3.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], [6.0, 7.0, 4.0], [8.0, 8.0, 
5.0]]], [[[1.0, 2.0, 5.0], [8.0, 6.0, 1.0], [4.0, 5.0, 3.0]], [[6.0, 3.0, 6.0], [4.0, 2.0, 6.0], [2.0, 3.0, 1.0]], [[5.0, 7.0, 
6.0], [8.0, 7.0, 5.0], [4.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 1.0, 1.0], [8.0, 5.0, 4.0], [7.0, 7.0, 7.0]], 
[[5.0, 3.0, 6.0], [1.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[5.0, 5.0, 4.0], [4.0, 8.0, 3.0], [7.0, 5.0, 6.0]]], [[[8.0, 8.0, 8.0], 
[6.0, 5.0, 7.0], [6.0, 8.0, 3.0]], [[6.0, 7.0, 1.0], [8.0, 8.0, 3.0], [7.0, 6.0, 1.0]], [[4.0, 7.0, 2.0], [5.0, 7.0, 6.0], 
[7.0, 6.0, 8.0]]], [[[5.0, 4.0, 6.0], [2.0, 3.0, 6.0], [8.0, 8.0, 3.0]], [[8.0, 8.0, 3.0], [7.0, 6.0, 1.0], [2.0, 4.0, 5.0]], 
[[1.0, 7.0, 5.0], [2.0, 3.0, 4.0], [7.0, 2.0, 1.0]]], [[[1.0, 3.0, 2.0], [2.0, 8.0, 4.0], [2.0, 4.0, 8.0]], [[5.0, 6.0, 1.0], 
[1.0, 6.0, 2.0], [8.0, 8.0, 1.0]], [[5.0, 4.0, 5.0], [2.0, 8.0, 4.0], [8.0, 5.0, 6.0]]]])
      val=Data(numpy.array([[6.0, 7.0, 1.0], [8.0, 6.0, 5.0]]),self.functionspace)
      arg[1:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 4.0], [2.0, 3.0, 2.0], [1.0, 3.0, 4.0]], [[5.0, 2.0, 7.0], [1.0, 7.0, 5.0], [5.0, 
6.0, 5.0]], [[3.0, 4.0, 1.0], [2.0, 1.0, 3.0], [4.0, 5.0, 2.0]]], [[[5.0, 6.0, 5.0], [4.0, 5.0, 4.0], [5.0, 4.0, 4.0]], [[8.0, 
5.0, 6.0], [6.0, 7.0, 1.0], [5.0, 2.0, 7.0]], [[1.0, 3.0, 6.0], [7.0, 4.0, 2.0], [4.0, 8.0, 4.0]]], [[[2.0, 4.0, 5.0], [8.0, 
5.0, 1.0], [2.0, 2.0, 1.0]], [[4.0, 7.0, 2.0], [8.0, 6.0, 5.0], [3.0, 1.0, 1.0]], [[7.0, 5.0, 6.0], [6.0, 7.0, 4.0], [8.0, 8.0, 
5.0]]], [[[1.0, 2.0, 5.0], [8.0, 6.0, 1.0], [4.0, 5.0, 3.0]], [[6.0, 3.0, 6.0], [4.0, 2.0, 6.0], [2.0, 3.0, 1.0]], [[5.0, 7.0, 
6.0], [8.0, 7.0, 5.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 1.0, 1.0], [8.0, 5.0, 4.0], [7.0, 7.0, 7.0]], 
[[5.0, 3.0, 6.0], [1.0, 1.0, 8.0], [5.0, 2.0, 1.0]], [[5.0, 5.0, 4.0], [4.0, 8.0, 3.0], [7.0, 5.0, 6.0]]], [[[8.0, 8.0, 8.0], 
[6.0, 5.0, 7.0], [6.0, 8.0, 3.0]], [[6.0, 7.0, 1.0], [6.0, 7.0, 1.0], [7.0, 6.0, 1.0]], [[4.0, 7.0, 2.0], [5.0, 7.0, 6.0], 
[7.0, 6.0, 8.0]]], [[[5.0, 4.0, 6.0], [2.0, 3.0, 6.0], [8.0, 8.0, 3.0]], [[8.0, 8.0, 3.0], [8.0, 6.0, 5.0], [2.0, 4.0, 5.0]], 
[[1.0, 7.0, 5.0], [2.0, 3.0, 4.0], [7.0, 2.0, 1.0]]], [[[1.0, 3.0, 2.0], [2.0, 8.0, 4.0], [2.0, 4.0, 8.0]], [[5.0, 6.0, 1.0], 
[1.0, 6.0, 2.0], [8.0, 8.0, 1.0]], [[5.0, 4.0, 5.0], [2.0, 8.0, 4.0], [8.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RPPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 5.0, 5.0], [5.0, 4.0, 6.0], [8.0, 8.0, 8.0]], [[6.0, 7.0, 8.0], [8.0, 1.0, 2.0], [8.0, 
3.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 5.0, 1.0], [5.0, 4.0, 3.0]]], [[[5.0, 3.0, 5.0], [2.0, 2.0, 4.0], [5.0, 3.0, 8.0]], [[2.0, 
8.0, 5.0], [5.0, 3.0, 3.0], [1.0, 5.0, 6.0]], [[7.0, 6.0, 7.0], [5.0, 2.0, 4.0], [8.0, 4.0, 4.0]]], [[[4.0, 6.0, 6.0], [7.0, 
6.0, 7.0], [8.0, 8.0, 8.0]], [[5.0, 2.0, 8.0], [6.0, 3.0, 8.0], [6.0, 3.0, 4.0]], [[6.0, 2.0, 2.0], [1.0, 7.0, 1.0], [6.0, 3.0, 
6.0]]], [[[7.0, 4.0, 2.0], [8.0, 2.0, 8.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 5.0], [7.0, 8.0, 2.0], [6.0, 5.0, 4.0]], [[4.0, 3.0, 
3.0], [7.0, 8.0, 5.0], [7.0, 2.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 6.0, 1.0], [6.0, 5.0, 8.0], [2.0, 3.0, 2.0]], 
[[5.0, 3.0, 1.0], [8.0, 8.0, 3.0], [5.0, 1.0, 4.0]], [[7.0, 8.0, 7.0], [4.0, 2.0, 5.0], [7.0, 3.0, 3.0]]], [[[3.0, 6.0, 5.0], 
[8.0, 1.0, 2.0], [5.0, 5.0, 1.0]], [[2.0, 8.0, 4.0], [5.0, 6.0, 1.0], [8.0, 6.0, 6.0]], [[1.0, 5.0, 8.0], [7.0, 2.0, 8.0], 
[8.0, 3.0, 4.0]]], [[[2.0, 7.0, 7.0], [7.0, 6.0, 2.0], [7.0, 2.0, 7.0]], [[5.0, 7.0, 7.0], [8.0, 4.0, 1.0], [2.0, 6.0, 2.0]], 
[[8.0, 7.0, 5.0], [3.0, 1.0, 8.0], [3.0, 4.0, 7.0]]], [[[1.0, 6.0, 5.0], [3.0, 1.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 5.0, 7.0], 
[7.0, 1.0, 6.0], [5.0, 3.0, 1.0]], [[4.0, 4.0, 3.0], [8.0, 4.0, 5.0], [2.0, 2.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[6.0, 1.0, 2.0], [5.0, 4.0, 3.0]])+(1.-msk_val)*numpy.array([[7.0, 2.0, 5.0], [6.0, 5.0, 5.0]])
      arg[1:3,1,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 5.0, 5.0], [5.0, 4.0, 6.0], [8.0, 8.0, 8.0]], [[6.0, 7.0, 8.0], [8.0, 1.0, 2.0], [8.0, 
3.0, 7.0]], [[1.0, 3.0, 2.0], [7.0, 5.0, 1.0], [5.0, 4.0, 3.0]]], [[[5.0, 3.0, 5.0], [2.0, 2.0, 4.0], [5.0, 3.0, 8.0]], [[2.0, 
8.0, 5.0], [6.0, 1.0, 2.0], [1.0, 5.0, 6.0]], [[7.0, 6.0, 7.0], [5.0, 2.0, 4.0], [8.0, 4.0, 4.0]]], [[[4.0, 6.0, 6.0], [7.0, 
6.0, 7.0], [8.0, 8.0, 8.0]], [[5.0, 2.0, 8.0], [5.0, 4.0, 3.0], [6.0, 3.0, 4.0]], [[6.0, 2.0, 2.0], [1.0, 7.0, 1.0], [6.0, 3.0, 
6.0]]], [[[7.0, 4.0, 2.0], [8.0, 2.0, 8.0], [3.0, 3.0, 5.0]], [[3.0, 5.0, 5.0], [7.0, 8.0, 2.0], [6.0, 5.0, 4.0]], [[4.0, 3.0, 
3.0], [7.0, 8.0, 5.0], [7.0, 2.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 6.0, 1.0], [6.0, 5.0, 8.0], [2.0, 3.0, 2.0]], 
[[5.0, 3.0, 1.0], [8.0, 8.0, 3.0], [5.0, 1.0, 4.0]], [[7.0, 8.0, 7.0], [4.0, 2.0, 5.0], [7.0, 3.0, 3.0]]], [[[3.0, 6.0, 5.0], 
[8.0, 1.0, 2.0], [5.0, 5.0, 1.0]], [[2.0, 8.0, 4.0], [7.0, 2.0, 5.0], [8.0, 6.0, 6.0]], [[1.0, 5.0, 8.0], [7.0, 2.0, 8.0], 
[8.0, 3.0, 4.0]]], [[[2.0, 7.0, 7.0], [7.0, 6.0, 2.0], [7.0, 2.0, 7.0]], [[5.0, 7.0, 7.0], [6.0, 5.0, 5.0], [2.0, 6.0, 2.0]], 
[[8.0, 7.0, 5.0], [3.0, 1.0, 8.0], [3.0, 4.0, 7.0]]], [[[1.0, 6.0, 5.0], [3.0, 1.0, 8.0], [3.0, 7.0, 3.0]], [[4.0, 5.0, 7.0], 
[7.0, 1.0, 6.0], [5.0, 3.0, 1.0]], [[4.0, 4.0, 3.0], [8.0, 4.0, 5.0], [2.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 2.0], [8.0, 7.0, 5.0], [7.0, 2.0, 2.0]], [[7.0, 8.0, 2.0], [6.0, 6.0, 6.0], [3.0, 
7.0, 1.0]], [[4.0, 4.0, 4.0], [5.0, 7.0, 7.0], [6.0, 1.0, 2.0]]], [[[1.0, 8.0, 2.0], [6.0, 3.0, 1.0], [8.0, 1.0, 4.0]], [[2.0, 
3.0, 7.0], [3.0, 4.0, 1.0], [8.0, 4.0, 5.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 4.0], [8.0, 7.0, 7.0]]], [[[5.0, 1.0, 2.0], [4.0, 
5.0, 6.0], [5.0, 7.0, 1.0]], [[6.0, 2.0, 7.0], [5.0, 4.0, 3.0], [4.0, 6.0, 8.0]], [[2.0, 3.0, 2.0], [8.0, 3.0, 6.0], [3.0, 7.0, 
3.0]]], [[[3.0, 5.0, 4.0], [1.0, 1.0, 1.0], [1.0, 7.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 8.0, 5.0], [4.0, 5.0, 5.0]], [[8.0, 8.0, 
2.0], [8.0, 2.0, 6.0], [3.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 4.0, 1.0], [6.0, 4.0, 7.0], [3.0, 4.0, 2.0]], 
[[2.0, 6.0, 3.0], [2.0, 7.0, 8.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 5.0], [1.0, 2.0, 3.0]]], [[[4.0, 4.0, 3.0], 
[6.0, 1.0, 1.0], [1.0, 3.0, 7.0]], [[3.0, 2.0, 1.0], [1.0, 1.0, 8.0], [1.0, 4.0, 1.0]], [[5.0, 4.0, 4.0], [8.0, 3.0, 8.0], 
[1.0, 2.0, 1.0]]], [[[7.0, 5.0, 2.0], [4.0, 2.0, 6.0], [8.0, 5.0, 3.0]], [[2.0, 1.0, 7.0], [1.0, 6.0, 7.0], [3.0, 1.0, 5.0]], 
[[4.0, 7.0, 2.0], [6.0, 8.0, 7.0], [5.0, 4.0, 7.0]]], [[[8.0, 6.0, 6.0], [1.0, 1.0, 6.0], [1.0, 1.0, 1.0]], [[2.0, 2.0, 4.0], 
[8.0, 1.0, 5.0], [7.0, 5.0, 6.0]], [[1.0, 8.0, 3.0], [6.0, 5.0, 6.0], [6.0, 6.0, 6.0]]]])
      val=numpy.array([[[2.0, 4.0, 2.0], [1.0, 6.0, 3.0]], [[5.0, 5.0, 6.0], [3.0, 8.0, 2.0]]])
      arg[1:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 2.0], [8.0, 7.0, 5.0], [7.0, 2.0, 2.0]], [[7.0, 8.0, 2.0], [6.0, 6.0, 6.0], [3.0, 
7.0, 1.0]], [[4.0, 4.0, 4.0], [5.0, 7.0, 7.0], [6.0, 1.0, 2.0]]], [[[1.0, 8.0, 2.0], [2.0, 4.0, 2.0], [8.0, 1.0, 4.0]], [[2.0, 
3.0, 7.0], [1.0, 6.0, 3.0], [8.0, 4.0, 5.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 4.0], [8.0, 7.0, 7.0]]], [[[5.0, 1.0, 2.0], [5.0, 
5.0, 6.0], [5.0, 7.0, 1.0]], [[6.0, 2.0, 7.0], [3.0, 8.0, 2.0], [4.0, 6.0, 8.0]], [[2.0, 3.0, 2.0], [8.0, 3.0, 6.0], [3.0, 7.0, 
3.0]]], [[[3.0, 5.0, 4.0], [1.0, 1.0, 1.0], [1.0, 7.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 8.0, 5.0], [4.0, 5.0, 5.0]], [[8.0, 8.0, 
2.0], [8.0, 2.0, 6.0], [3.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 4.0, 1.0], [6.0, 4.0, 7.0], [3.0, 4.0, 2.0]], 
[[2.0, 6.0, 3.0], [2.0, 7.0, 8.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 5.0], [2.0, 4.0, 5.0], [1.0, 2.0, 3.0]]], [[[4.0, 4.0, 3.0], 
[2.0, 4.0, 2.0], [1.0, 3.0, 7.0]], [[3.0, 2.0, 1.0], [1.0, 6.0, 3.0], [1.0, 4.0, 1.0]], [[5.0, 4.0, 4.0], [8.0, 3.0, 8.0], 
[1.0, 2.0, 1.0]]], [[[7.0, 5.0, 2.0], [5.0, 5.0, 6.0], [8.0, 5.0, 3.0]], [[2.0, 1.0, 7.0], [3.0, 8.0, 2.0], [3.0, 1.0, 5.0]], 
[[4.0, 7.0, 2.0], [6.0, 8.0, 7.0], [5.0, 4.0, 7.0]]], [[[8.0, 6.0, 6.0], [1.0, 1.0, 6.0], [1.0, 1.0, 1.0]], [[2.0, 2.0, 4.0], 
[8.0, 1.0, 5.0], [7.0, 5.0, 6.0]], [[1.0, 8.0, 3.0], [6.0, 5.0, 6.0], [6.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 8.0], [5.0, 6.0, 5.0], [8.0, 1.0, 3.0]], [[5.0, 6.0, 4.0], [6.0, 5.0, 3.0], [4.0, 
3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 4.0, 3.0], [8.0, 3.0, 6.0]]], [[[3.0, 7.0, 4.0], [8.0, 6.0, 1.0], [1.0, 2.0, 7.0]], [[3.0, 
8.0, 2.0], [2.0, 2.0, 7.0], [4.0, 1.0, 8.0]], [[6.0, 1.0, 3.0], [3.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[7.0, 5.0, 4.0], [2.0, 
5.0, 1.0], [8.0, 5.0, 5.0]], [[6.0, 6.0, 4.0], [6.0, 4.0, 2.0], [2.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 8.0, 4.0], [2.0, 7.0, 
5.0]]], [[[7.0, 1.0, 1.0], [8.0, 5.0, 4.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 3.0], [4.0, 1.0, 6.0]], [[7.0, 1.0, 
1.0], [2.0, 8.0, 8.0], [5.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 3.0], [8.0, 7.0, 8.0], [6.0, 7.0, 6.0]], 
[[3.0, 5.0, 8.0], [6.0, 8.0, 1.0], [3.0, 8.0, 6.0]], [[7.0, 6.0, 5.0], [1.0, 4.0, 7.0], [6.0, 3.0, 3.0]]], [[[4.0, 8.0, 6.0], 
[6.0, 1.0, 3.0], [4.0, 4.0, 7.0]], [[6.0, 2.0, 8.0], [1.0, 3.0, 6.0], [8.0, 2.0, 8.0]], [[2.0, 1.0, 8.0], [4.0, 2.0, 8.0], 
[8.0, 8.0, 7.0]]], [[[4.0, 1.0, 3.0], [3.0, 1.0, 6.0], [2.0, 7.0, 7.0]], [[4.0, 8.0, 5.0], [8.0, 8.0, 8.0], [1.0, 4.0, 8.0]], 
[[3.0, 2.0, 6.0], [8.0, 4.0, 5.0], [7.0, 4.0, 8.0]]], [[[3.0, 3.0, 2.0], [8.0, 3.0, 5.0], [6.0, 6.0, 7.0]], [[8.0, 2.0, 6.0], 
[7.0, 7.0, 4.0], [8.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [6.0, 8.0, 3.0], [7.0, 1.0, 5.0]]]])
      val=Data(numpy.array([[[1.0, 7.0, 7.0], [7.0, 6.0, 1.0]], [[8.0, 5.0, 6.0], [4.0, 8.0, 3.0]]]),self.functionspace)
      arg[1:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 8.0], [5.0, 6.0, 5.0], [8.0, 1.0, 3.0]], [[5.0, 6.0, 4.0], [6.0, 5.0, 3.0], [4.0, 
3.0, 2.0]], [[3.0, 2.0, 5.0], [4.0, 4.0, 3.0], [8.0, 3.0, 6.0]]], [[[3.0, 7.0, 4.0], [1.0, 7.0, 7.0], [1.0, 2.0, 7.0]], [[3.0, 
8.0, 2.0], [7.0, 6.0, 1.0], [4.0, 1.0, 8.0]], [[6.0, 1.0, 3.0], [3.0, 8.0, 1.0], [1.0, 8.0, 6.0]]], [[[7.0, 5.0, 4.0], [8.0, 
5.0, 6.0], [8.0, 5.0, 5.0]], [[6.0, 6.0, 4.0], [4.0, 8.0, 3.0], [2.0, 8.0, 3.0]], [[5.0, 5.0, 5.0], [5.0, 8.0, 4.0], [2.0, 7.0, 
5.0]]], [[[7.0, 1.0, 1.0], [8.0, 5.0, 4.0], [5.0, 8.0, 2.0]], [[2.0, 4.0, 8.0], [5.0, 4.0, 3.0], [4.0, 1.0, 6.0]], [[7.0, 1.0, 
1.0], [2.0, 8.0, 8.0], [5.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [8.0, 7.0, 8.0], [6.0, 7.0, 6.0]], 
[[3.0, 5.0, 8.0], [6.0, 8.0, 1.0], [3.0, 8.0, 6.0]], [[7.0, 6.0, 5.0], [1.0, 4.0, 7.0], [6.0, 3.0, 3.0]]], [[[4.0, 8.0, 6.0], 
[1.0, 7.0, 7.0], [4.0, 4.0, 7.0]], [[6.0, 2.0, 8.0], [7.0, 6.0, 1.0], [8.0, 2.0, 8.0]], [[2.0, 1.0, 8.0], [4.0, 2.0, 8.0], 
[8.0, 8.0, 7.0]]], [[[4.0, 1.0, 3.0], [8.0, 5.0, 6.0], [2.0, 7.0, 7.0]], [[4.0, 8.0, 5.0], [4.0, 8.0, 3.0], [1.0, 4.0, 8.0]], 
[[3.0, 2.0, 6.0], [8.0, 4.0, 5.0], [7.0, 4.0, 8.0]]], [[[3.0, 3.0, 2.0], [8.0, 3.0, 5.0], [6.0, 6.0, 7.0]], [[8.0, 2.0, 6.0], 
[7.0, 7.0, 4.0], [8.0, 5.0, 4.0]], [[3.0, 2.0, 6.0], [6.0, 8.0, 3.0], [7.0, 1.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RUPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 7.0], [6.0, 2.0, 4.0], [6.0, 3.0, 2.0]], [[1.0, 6.0, 3.0], [8.0, 8.0, 1.0], [8.0, 
7.0, 4.0]], [[6.0, 5.0, 2.0], [4.0, 6.0, 6.0], [5.0, 4.0, 6.0]]], [[[5.0, 8.0, 3.0], [3.0, 4.0, 8.0], [4.0, 5.0, 3.0]], [[6.0, 
4.0, 7.0], [6.0, 2.0, 1.0], [8.0, 6.0, 3.0]], [[1.0, 2.0, 2.0], [8.0, 2.0, 4.0], [8.0, 6.0, 3.0]]], [[[6.0, 5.0, 4.0], [5.0, 
3.0, 8.0], [2.0, 7.0, 4.0]], [[4.0, 6.0, 1.0], [2.0, 3.0, 1.0], [7.0, 5.0, 7.0]], [[2.0, 3.0, 4.0], [8.0, 3.0, 7.0], [5.0, 2.0, 
8.0]]], [[[3.0, 6.0, 6.0], [8.0, 3.0, 4.0], [5.0, 8.0, 2.0]], [[6.0, 4.0, 6.0], [8.0, 7.0, 7.0], [1.0, 8.0, 1.0]], [[3.0, 1.0, 
6.0], [6.0, 7.0, 1.0], [5.0, 4.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 6.0, 7.0], [2.0, 8.0, 5.0], [4.0, 1.0, 3.0]], 
[[8.0, 3.0, 2.0], [5.0, 7.0, 7.0], [2.0, 6.0, 5.0]], [[6.0, 8.0, 3.0], [7.0, 8.0, 5.0], [4.0, 8.0, 3.0]]], [[[4.0, 4.0, 2.0], 
[8.0, 1.0, 6.0], [2.0, 4.0, 3.0]], [[2.0, 5.0, 4.0], [8.0, 6.0, 1.0], [6.0, 1.0, 7.0]], [[8.0, 4.0, 1.0], [2.0, 2.0, 5.0], 
[5.0, 5.0, 2.0]]], [[[1.0, 7.0, 8.0], [3.0, 6.0, 2.0], [1.0, 5.0, 8.0]], [[4.0, 6.0, 8.0], [3.0, 6.0, 8.0], [2.0, 8.0, 8.0]], 
[[2.0, 3.0, 1.0], [6.0, 2.0, 5.0], [2.0, 1.0, 1.0]]], [[[3.0, 8.0, 8.0], [4.0, 1.0, 2.0], [2.0, 3.0, 7.0]], [[5.0, 4.0, 4.0], 
[1.0, 7.0, 2.0], [3.0, 3.0, 1.0]], [[4.0, 5.0, 7.0], [7.0, 5.0, 2.0], [1.0, 6.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[5.0, 2.0, 7.0], [3.0, 5.0, 3.0]], [[3.0, 6.0, 3.0], [6.0, 8.0, 
8.0]]])+(1.-msk_val)*numpy.array([[[8.0, 1.0, 4.0], [2.0, 4.0, 5.0]], [[4.0, 3.0, 3.0], [1.0, 5.0, 6.0]]])
      arg[1:3,:2,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 7.0], [6.0, 2.0, 4.0], [6.0, 3.0, 2.0]], [[1.0, 6.0, 3.0], [8.0, 8.0, 1.0], [8.0, 
7.0, 4.0]], [[6.0, 5.0, 2.0], [4.0, 6.0, 6.0], [5.0, 4.0, 6.0]]], [[[5.0, 8.0, 3.0], [5.0, 2.0, 7.0], [4.0, 5.0, 3.0]], [[6.0, 
4.0, 7.0], [3.0, 5.0, 3.0], [8.0, 6.0, 3.0]], [[1.0, 2.0, 2.0], [8.0, 2.0, 4.0], [8.0, 6.0, 3.0]]], [[[6.0, 5.0, 4.0], [3.0, 
6.0, 3.0], [2.0, 7.0, 4.0]], [[4.0, 6.0, 1.0], [6.0, 8.0, 8.0], [7.0, 5.0, 7.0]], [[2.0, 3.0, 4.0], [8.0, 3.0, 7.0], [5.0, 2.0, 
8.0]]], [[[3.0, 6.0, 6.0], [8.0, 3.0, 4.0], [5.0, 8.0, 2.0]], [[6.0, 4.0, 6.0], [8.0, 7.0, 7.0], [1.0, 8.0, 1.0]], [[3.0, 1.0, 
6.0], [6.0, 7.0, 1.0], [5.0, 4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 6.0, 7.0], [2.0, 8.0, 5.0], [4.0, 1.0, 3.0]], 
[[8.0, 3.0, 2.0], [5.0, 7.0, 7.0], [2.0, 6.0, 5.0]], [[6.0, 8.0, 3.0], [7.0, 8.0, 5.0], [4.0, 8.0, 3.0]]], [[[4.0, 4.0, 2.0], 
[8.0, 1.0, 4.0], [2.0, 4.0, 3.0]], [[2.0, 5.0, 4.0], [2.0, 4.0, 5.0], [6.0, 1.0, 7.0]], [[8.0, 4.0, 1.0], [2.0, 2.0, 5.0], 
[5.0, 5.0, 2.0]]], [[[1.0, 7.0, 8.0], [4.0, 3.0, 3.0], [1.0, 5.0, 8.0]], [[4.0, 6.0, 8.0], [1.0, 5.0, 6.0], [2.0, 8.0, 8.0]], 
[[2.0, 3.0, 1.0], [6.0, 2.0, 5.0], [2.0, 1.0, 1.0]]], [[[3.0, 8.0, 8.0], [4.0, 1.0, 2.0], [2.0, 3.0, 7.0]], [[5.0, 4.0, 4.0], 
[1.0, 7.0, 2.0], [3.0, 3.0, 1.0]], [[4.0, 5.0, 7.0], [7.0, 5.0, 2.0], [1.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 5.0], [7.0, 5.0, 4.0], [7.0, 5.0, 8.0]], [[5.0, 2.0, 7.0], [4.0, 4.0, 5.0], [7.0, 
8.0, 7.0]], [[5.0, 4.0, 8.0], [7.0, 3.0, 3.0], [2.0, 1.0, 2.0]]], [[[8.0, 1.0, 3.0], [1.0, 6.0, 1.0], [2.0, 1.0, 2.0]], [[5.0, 
4.0, 4.0], [8.0, 5.0, 1.0], [6.0, 6.0, 5.0]], [[3.0, 7.0, 3.0], [6.0, 6.0, 1.0], [8.0, 4.0, 6.0]]], [[[8.0, 7.0, 4.0], [4.0, 
7.0, 1.0], [8.0, 5.0, 6.0]], [[4.0, 4.0, 3.0], [8.0, 7.0, 1.0], [8.0, 3.0, 5.0]], [[1.0, 3.0, 6.0], [4.0, 2.0, 8.0], [7.0, 1.0, 
6.0]]], [[[8.0, 2.0, 7.0], [5.0, 6.0, 7.0], [7.0, 7.0, 8.0]], [[3.0, 4.0, 3.0], [1.0, 3.0, 5.0], [4.0, 7.0, 7.0]], [[6.0, 6.0, 
8.0], [8.0, 5.0, 4.0], [5.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 1.0, 2.0], [8.0, 1.0, 2.0], [2.0, 8.0, 2.0]], 
[[5.0, 1.0, 3.0], [2.0, 4.0, 3.0], [5.0, 4.0, 3.0]], [[6.0, 4.0, 5.0], [8.0, 6.0, 5.0], [1.0, 5.0, 4.0]]], [[[4.0, 2.0, 1.0], 
[5.0, 3.0, 2.0], [5.0, 8.0, 4.0]], [[4.0, 3.0, 3.0], [2.0, 1.0, 3.0], [3.0, 4.0, 5.0]], [[3.0, 5.0, 8.0], [3.0, 2.0, 3.0], 
[8.0, 1.0, 4.0]]], [[[6.0, 2.0, 2.0], [4.0, 4.0, 2.0], [5.0, 6.0, 8.0]], [[8.0, 2.0, 5.0], [3.0, 7.0, 2.0], [1.0, 7.0, 2.0]], 
[[5.0, 4.0, 1.0], [6.0, 3.0, 5.0], [5.0, 4.0, 8.0]]], [[[1.0, 3.0, 4.0], [4.0, 7.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 5.0, 7.0], 
[3.0, 2.0, 2.0], [8.0, 2.0, 4.0]], [[1.0, 5.0, 2.0], [7.0, 8.0, 6.0], [7.0, 2.0, 7.0]]]])
      val=numpy.array([[[[1.0, 3.0, 7.0], [2.0, 8.0, 2.0], [3.0, 5.0, 5.0]], [[3.0, 5.0, 1.0], [5.0, 4.0, 5.0], [1.0, 6.0, 
5.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 8.0], [5.0, 4.0, 1.0]]], [[[2.0, 8.0, 7.0], [4.0, 8.0, 2.0], [7.0, 7.0, 7.0]], [[4.0, 6.0, 
3.0], [7.0, 1.0, 3.0], [1.0, 6.0, 3.0]], [[6.0, 8.0, 8.0], [2.0, 5.0, 7.0], [1.0, 8.0, 1.0]]], [[[1.0, 2.0, 2.0], [4.0, 4.0, 
7.0], [2.0, 5.0, 8.0]], [[6.0, 4.0, 5.0], [3.0, 8.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 7.0, 2.0], [6.0, 2.0, 1.0], [1.0, 7.0, 
2.0]]]])
      arg[:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, 7.0], [2.0, 8.0, 2.0], [3.0, 5.0, 5.0]], [[3.0, 5.0, 1.0], [5.0, 4.0, 5.0], [1.0, 
6.0, 5.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 8.0], [5.0, 4.0, 1.0]]], [[[2.0, 8.0, 7.0], [4.0, 8.0, 2.0], [7.0, 7.0, 7.0]], [[4.0, 
6.0, 3.0], [7.0, 1.0, 3.0], [1.0, 6.0, 3.0]], [[6.0, 8.0, 8.0], [2.0, 5.0, 7.0], [1.0, 8.0, 1.0]]], [[[1.0, 2.0, 2.0], [4.0, 
4.0, 7.0], [2.0, 5.0, 8.0]], [[6.0, 4.0, 5.0], [3.0, 8.0, 4.0], [6.0, 5.0, 1.0]], [[2.0, 7.0, 2.0], [6.0, 2.0, 1.0], [1.0, 7.0, 
2.0]]], [[[8.0, 2.0, 7.0], [5.0, 6.0, 7.0], [7.0, 7.0, 8.0]], [[3.0, 4.0, 3.0], [1.0, 3.0, 5.0], [4.0, 7.0, 7.0]], [[6.0, 6.0, 
8.0], [8.0, 5.0, 4.0], [5.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 7.0], [2.0, 8.0, 2.0], [3.0, 5.0, 5.0]], 
[[3.0, 5.0, 1.0], [5.0, 4.0, 5.0], [1.0, 6.0, 5.0]], [[4.0, 7.0, 5.0], [2.0, 6.0, 8.0], [5.0, 4.0, 1.0]]], [[[2.0, 8.0, 7.0], 
[4.0, 8.0, 2.0], [7.0, 7.0, 7.0]], [[4.0, 6.0, 3.0], [7.0, 1.0, 3.0], [1.0, 6.0, 3.0]], [[6.0, 8.0, 8.0], [2.0, 5.0, 7.0], 
[1.0, 8.0, 1.0]]], [[[1.0, 2.0, 2.0], [4.0, 4.0, 7.0], [2.0, 5.0, 8.0]], [[6.0, 4.0, 5.0], [3.0, 8.0, 4.0], [6.0, 5.0, 1.0]], 
[[2.0, 7.0, 2.0], [6.0, 2.0, 1.0], [1.0, 7.0, 2.0]]], [[[1.0, 3.0, 4.0], [4.0, 7.0, 4.0], [7.0, 1.0, 4.0]], [[5.0, 5.0, 7.0], 
[3.0, 2.0, 2.0], [8.0, 2.0, 4.0]], [[1.0, 5.0, 2.0], [7.0, 8.0, 6.0], [7.0, 2.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 6.0], [5.0, 5.0, 3.0], [7.0, 1.0, 2.0]], [[5.0, 6.0, 6.0], [3.0, 3.0, 6.0], [4.0, 
4.0, 5.0]], [[5.0, 1.0, 3.0], [7.0, 5.0, 1.0], [1.0, 4.0, 4.0]]], [[[4.0, 5.0, 4.0], [5.0, 4.0, 6.0], [2.0, 1.0, 2.0]], [[2.0, 
4.0, 5.0], [3.0, 6.0, 1.0], [5.0, 8.0, 4.0]], [[3.0, 5.0, 3.0], [6.0, 1.0, 1.0], [5.0, 8.0, 7.0]]], [[[4.0, 7.0, 4.0], [3.0, 
8.0, 5.0], [3.0, 3.0, 5.0]], [[8.0, 8.0, 4.0], [1.0, 5.0, 5.0], [3.0, 3.0, 2.0]], [[5.0, 8.0, 4.0], [2.0, 1.0, 2.0], [5.0, 3.0, 
6.0]]], [[[3.0, 4.0, 6.0], [7.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[5.0, 3.0, 4.0], [5.0, 4.0, 4.0], [3.0, 7.0, 2.0]], [[3.0, 1.0, 
1.0], [1.0, 3.0, 7.0], [3.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 8.0], [7.0, 3.0, 8.0], [3.0, 5.0, 4.0]], 
[[1.0, 6.0, 3.0], [8.0, 2.0, 4.0], [3.0, 5.0, 7.0]], [[8.0, 3.0, 7.0], [5.0, 1.0, 8.0], [2.0, 7.0, 3.0]]], [[[7.0, 8.0, 6.0], 
[3.0, 6.0, 5.0], [1.0, 6.0, 4.0]], [[1.0, 8.0, 1.0], [3.0, 1.0, 7.0], [3.0, 7.0, 8.0]], [[3.0, 7.0, 3.0], [6.0, 1.0, 7.0], 
[5.0, 1.0, 2.0]]], [[[1.0, 6.0, 5.0], [1.0, 3.0, 8.0], [5.0, 8.0, 5.0]], [[8.0, 1.0, 4.0], [3.0, 3.0, 1.0], [6.0, 5.0, 1.0]], 
[[6.0, 7.0, 1.0], [3.0, 8.0, 2.0], [5.0, 4.0, 7.0]]], [[[2.0, 8.0, 4.0], [3.0, 3.0, 2.0], [4.0, 5.0, 6.0]], [[7.0, 6.0, 5.0], 
[2.0, 6.0, 8.0], [7.0, 3.0, 7.0]], [[8.0, 4.0, 3.0], [8.0, 4.0, 4.0], [1.0, 1.0, 7.0]]]])
      val=Data(numpy.array([[[[6.0, 5.0, 3.0], [5.0, 1.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 7.0, 5.0], [2.0, 4.0, 7.0], [6.0, 8.0, 
2.0]], [[8.0, 6.0, 1.0], [6.0, 4.0, 8.0], [7.0, 3.0, 7.0]]], [[[6.0, 1.0, 3.0], [6.0, 4.0, 7.0], [5.0, 2.0, 3.0]], [[5.0, 1.0, 
1.0], [4.0, 2.0, 1.0], [1.0, 5.0, 6.0]], [[8.0, 5.0, 3.0], [4.0, 1.0, 8.0], [8.0, 2.0, 5.0]]], [[[8.0, 1.0, 7.0], [4.0, 4.0, 
4.0], [1.0, 7.0, 1.0]], [[1.0, 7.0, 7.0], [1.0, 7.0, 5.0], [1.0, 3.0, 4.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 6.0], [6.0, 2.0, 
8.0]]]]),self.functionspace)
      arg[:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 3.0], [5.0, 1.0, 6.0], [7.0, 2.0, 6.0]], [[3.0, 7.0, 5.0], [2.0, 4.0, 7.0], [6.0, 
8.0, 2.0]], [[8.0, 6.0, 1.0], [6.0, 4.0, 8.0], [7.0, 3.0, 7.0]]], [[[6.0, 1.0, 3.0], [6.0, 4.0, 7.0], [5.0, 2.0, 3.0]], [[5.0, 
1.0, 1.0], [4.0, 2.0, 1.0], [1.0, 5.0, 6.0]], [[8.0, 5.0, 3.0], [4.0, 1.0, 8.0], [8.0, 2.0, 5.0]]], [[[8.0, 1.0, 7.0], [4.0, 
4.0, 4.0], [1.0, 7.0, 1.0]], [[1.0, 7.0, 7.0], [1.0, 7.0, 5.0], [1.0, 3.0, 4.0]], [[3.0, 8.0, 3.0], [8.0, 3.0, 6.0], [6.0, 2.0, 
8.0]]], [[[3.0, 4.0, 6.0], [7.0, 8.0, 8.0], [3.0, 8.0, 5.0]], [[5.0, 3.0, 4.0], [5.0, 4.0, 4.0], [3.0, 7.0, 2.0]], [[3.0, 1.0, 
1.0], [1.0, 3.0, 7.0], [3.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 3.0], [5.0, 1.0, 6.0], [7.0, 2.0, 6.0]], 
[[3.0, 7.0, 5.0], [2.0, 4.0, 7.0], [6.0, 8.0, 2.0]], [[8.0, 6.0, 1.0], [6.0, 4.0, 8.0], [7.0, 3.0, 7.0]]], [[[6.0, 1.0, 3.0], 
[6.0, 4.0, 7.0], [5.0, 2.0, 3.0]], [[5.0, 1.0, 1.0], [4.0, 2.0, 1.0], [1.0, 5.0, 6.0]], [[8.0, 5.0, 3.0], [4.0, 1.0, 8.0], 
[8.0, 2.0, 5.0]]], [[[8.0, 1.0, 7.0], [4.0, 4.0, 4.0], [1.0, 7.0, 1.0]], [[1.0, 7.0, 7.0], [1.0, 7.0, 5.0], [1.0, 3.0, 4.0]], 
[[3.0, 8.0, 3.0], [8.0, 3.0, 6.0], [6.0, 2.0, 8.0]]], [[[2.0, 8.0, 4.0], [3.0, 3.0, 2.0], [4.0, 5.0, 6.0]], [[7.0, 6.0, 5.0], 
[2.0, 6.0, 8.0], [7.0, 3.0, 7.0]], [[8.0, 4.0, 3.0], [8.0, 4.0, 4.0], [1.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 5.0], [8.0, 7.0, 2.0], [3.0, 7.0, 3.0]], [[3.0, 2.0, 3.0], [7.0, 4.0, 2.0], [6.0, 
5.0, 4.0]], [[8.0, 2.0, 8.0], [3.0, 4.0, 6.0], [6.0, 8.0, 6.0]]], [[[6.0, 1.0, 1.0], [3.0, 1.0, 8.0], [4.0, 1.0, 5.0]], [[1.0, 
6.0, 7.0], [4.0, 5.0, 5.0], [2.0, 6.0, 7.0]], [[5.0, 7.0, 1.0], [7.0, 5.0, 7.0], [8.0, 2.0, 4.0]]], [[[7.0, 3.0, 4.0], [6.0, 
5.0, 8.0], [4.0, 7.0, 2.0]], [[8.0, 7.0, 4.0], [4.0, 8.0, 7.0], [8.0, 8.0, 8.0]], [[3.0, 5.0, 2.0], [8.0, 2.0, 1.0], [7.0, 2.0, 
2.0]]], [[[3.0, 2.0, 6.0], [1.0, 2.0, 6.0], [8.0, 3.0, 4.0]], [[2.0, 4.0, 1.0], [2.0, 2.0, 3.0], [1.0, 4.0, 5.0]], [[8.0, 3.0, 
5.0], [8.0, 6.0, 3.0], [6.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 6.0], [8.0, 7.0, 4.0], [3.0, 1.0, 8.0]], 
[[6.0, 5.0, 1.0], [4.0, 4.0, 2.0], [3.0, 6.0, 8.0]], [[8.0, 8.0, 3.0], [6.0, 4.0, 2.0], [1.0, 5.0, 3.0]]], [[[5.0, 4.0, 5.0], 
[8.0, 7.0, 2.0], [5.0, 4.0, 6.0]], [[3.0, 2.0, 2.0], [4.0, 5.0, 1.0], [1.0, 1.0, 5.0]], [[7.0, 7.0, 5.0], [2.0, 3.0, 5.0], 
[5.0, 5.0, 4.0]]], [[[1.0, 4.0, 3.0], [7.0, 6.0, 8.0], [1.0, 2.0, 1.0]], [[5.0, 2.0, 7.0], [6.0, 1.0, 1.0], [2.0, 6.0, 5.0]], 
[[6.0, 3.0, 3.0], [2.0, 8.0, 5.0], [8.0, 7.0, 2.0]]], [[[4.0, 8.0, 5.0], [1.0, 1.0, 4.0], [3.0, 3.0, 2.0]], [[5.0, 3.0, 2.0], 
[5.0, 3.0, 6.0], [6.0, 7.0, 3.0]], [[7.0, 2.0, 2.0], [2.0, 3.0, 6.0], [5.0, 5.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 6.0], [7.0, 
7.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 7.0, 3.0], [4.0, 6.0, 8.0]]], [[[6.0, 8.0, 5.0], [5.0, 2.0, 1.0], [1.0, 8.0, 1.0]], [[4.0, 
2.0, 1.0], [6.0, 8.0, 5.0], [7.0, 4.0, 2.0]], [[5.0, 2.0, 4.0], [5.0, 3.0, 3.0], [2.0, 8.0, 8.0]]], [[[3.0, 7.0, 1.0], [5.0, 
5.0, 1.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 1.0, 7.0], [5.0, 2.0, 4.0]], [[5.0, 7.0, 7.0], [1.0, 3.0, 6.0], [8.0, 2.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[1.0, 7.0, 7.0], [8.0, 1.0, 5.0], [4.0, 4.0, 7.0]], [[3.0, 2.0, 7.0], [5.0, 7.0, 4.0], 
[1.0, 5.0, 7.0]], [[3.0, 2.0, 1.0], [3.0, 5.0, 5.0], [6.0, 1.0, 7.0]]], [[[1.0, 2.0, 8.0], [7.0, 2.0, 5.0], [3.0, 4.0, 4.0]], 
[[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [4.0, 2.0, 2.0]], [[3.0, 6.0, 8.0], [8.0, 8.0, 8.0], [4.0, 7.0, 7.0]]], [[[4.0, 2.0, 5.0], 
[1.0, 5.0, 5.0], [5.0, 4.0, 8.0]], [[1.0, 4.0, 5.0], [8.0, 5.0, 8.0], [8.0, 5.0, 2.0]], [[7.0, 3.0, 3.0], [5.0, 4.0, 6.0], 
[8.0, 8.0, 8.0]]]])
      arg[:3,:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 6.0], [3.0, 5.0, 3.0], [8.0, 6.0, 4.0]], [[8.0, 5.0, 7.0], [3.0, 6.0, 6.0], [7.0, 
7.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 7.0, 3.0], [4.0, 6.0, 8.0]]], [[[6.0, 8.0, 5.0], [5.0, 2.0, 1.0], [1.0, 8.0, 1.0]], [[4.0, 
2.0, 1.0], [6.0, 8.0, 5.0], [7.0, 4.0, 2.0]], [[5.0, 2.0, 4.0], [5.0, 3.0, 3.0], [2.0, 8.0, 8.0]]], [[[3.0, 7.0, 1.0], [5.0, 
5.0, 1.0], [1.0, 6.0, 3.0]], [[3.0, 5.0, 8.0], [4.0, 1.0, 7.0], [5.0, 2.0, 4.0]], [[5.0, 7.0, 7.0], [1.0, 3.0, 6.0], [8.0, 2.0, 
5.0]]], [[[3.0, 2.0, 6.0], [1.0, 2.0, 6.0], [8.0, 3.0, 4.0]], [[2.0, 4.0, 1.0], [2.0, 2.0, 3.0], [1.0, 4.0, 5.0]], [[8.0, 3.0, 
5.0], [8.0, 6.0, 3.0], [6.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 7.0], [8.0, 1.0, 5.0], [4.0, 4.0, 7.0]], 
[[3.0, 2.0, 7.0], [5.0, 7.0, 4.0], [1.0, 5.0, 7.0]], [[3.0, 2.0, 1.0], [3.0, 5.0, 5.0], [6.0, 1.0, 7.0]]], [[[1.0, 2.0, 8.0], 
[7.0, 2.0, 5.0], [3.0, 4.0, 4.0]], [[2.0, 7.0, 2.0], [3.0, 8.0, 6.0], [4.0, 2.0, 2.0]], [[3.0, 6.0, 8.0], [8.0, 8.0, 8.0], 
[4.0, 7.0, 7.0]]], [[[4.0, 2.0, 5.0], [1.0, 5.0, 5.0], [5.0, 4.0, 8.0]], [[1.0, 4.0, 5.0], [8.0, 5.0, 8.0], [8.0, 5.0, 2.0]], 
[[7.0, 3.0, 3.0], [5.0, 4.0, 6.0], [8.0, 8.0, 8.0]]], [[[4.0, 8.0, 5.0], [1.0, 1.0, 4.0], [3.0, 3.0, 2.0]], [[5.0, 3.0, 2.0], 
[5.0, 3.0, 6.0], [6.0, 7.0, 3.0]], [[7.0, 2.0, 2.0], [2.0, 3.0, 6.0], [5.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 1.0], [8.0, 5.0, 7.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 4.0], [8.0, 
1.0, 8.0]], [[1.0, 2.0, 1.0], [8.0, 5.0, 6.0], [7.0, 8.0, 7.0]]], [[[3.0, 4.0, 5.0], [3.0, 4.0, 6.0], [7.0, 1.0, 3.0]], [[6.0, 
6.0, 2.0], [4.0, 3.0, 8.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 1.0], [3.0, 6.0, 6.0], [8.0, 7.0, 6.0]]], [[[5.0, 2.0, 7.0], [2.0, 
1.0, 2.0], [8.0, 6.0, 3.0]], [[8.0, 8.0, 2.0], [1.0, 8.0, 2.0], [2.0, 8.0, 7.0]], [[3.0, 5.0, 8.0], [4.0, 7.0, 8.0], [1.0, 1.0, 
4.0]]], [[[7.0, 8.0, 6.0], [3.0, 2.0, 7.0], [1.0, 6.0, 1.0]], [[2.0, 3.0, 6.0], [6.0, 5.0, 8.0], [8.0, 7.0, 1.0]], [[4.0, 8.0, 
8.0], [4.0, 5.0, 7.0], [1.0, 2.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 7.0, 3.0], [5.0, 6.0, 4.0]], 
[[6.0, 6.0, 3.0], [6.0, 7.0, 2.0], [2.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], [3.0, 5.0, 6.0], [5.0, 5.0, 1.0]]], [[[7.0, 4.0, 1.0], 
[7.0, 8.0, 1.0], [4.0, 8.0, 2.0]], [[7.0, 5.0, 7.0], [4.0, 4.0, 3.0], [3.0, 5.0, 6.0]], [[1.0, 8.0, 6.0], [4.0, 1.0, 4.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 1.0, 3.0], [4.0, 3.0, 8.0], [7.0, 3.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 7.0, 3.0], [1.0, 8.0, 1.0]], 
[[5.0, 4.0, 4.0], [5.0, 1.0, 1.0], [2.0, 8.0, 8.0]]], [[[6.0, 2.0, 7.0], [4.0, 3.0, 4.0], [5.0, 5.0, 6.0]], [[5.0, 8.0, 6.0], 
[3.0, 5.0, 5.0], [4.0, 8.0, 6.0]], [[6.0, 4.0, 3.0], [4.0, 5.0, 7.0], [3.0, 7.0, 6.0]]]])
      val=numpy.array([[[[7.0, 8.0, 6.0]]], [[[5.0, 7.0, 4.0]]]])
      arg[2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 1.0], [8.0, 5.0, 7.0], [2.0, 7.0, 5.0]], [[6.0, 5.0, 5.0], [5.0, 4.0, 4.0], [8.0, 
1.0, 8.0]], [[1.0, 2.0, 1.0], [8.0, 5.0, 6.0], [7.0, 8.0, 7.0]]], [[[3.0, 4.0, 5.0], [3.0, 4.0, 6.0], [7.0, 1.0, 3.0]], [[6.0, 
6.0, 2.0], [4.0, 3.0, 8.0], [5.0, 8.0, 2.0]], [[4.0, 2.0, 1.0], [3.0, 6.0, 6.0], [8.0, 7.0, 6.0]]], [[[5.0, 2.0, 7.0], [2.0, 
1.0, 2.0], [8.0, 6.0, 3.0]], [[8.0, 8.0, 2.0], [1.0, 8.0, 2.0], [7.0, 8.0, 6.0]], [[3.0, 5.0, 8.0], [4.0, 7.0, 8.0], [1.0, 1.0, 
4.0]]], [[[7.0, 8.0, 6.0], [3.0, 2.0, 7.0], [1.0, 6.0, 1.0]], [[2.0, 3.0, 6.0], [6.0, 5.0, 8.0], [5.0, 7.0, 4.0]], [[4.0, 8.0, 
8.0], [4.0, 5.0, 7.0], [1.0, 2.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 5.0, 5.0], [2.0, 7.0, 3.0], [5.0, 6.0, 4.0]], 
[[6.0, 6.0, 3.0], [6.0, 7.0, 2.0], [2.0, 1.0, 1.0]], [[3.0, 2.0, 3.0], [3.0, 5.0, 6.0], [5.0, 5.0, 1.0]]], [[[7.0, 4.0, 1.0], 
[7.0, 8.0, 1.0], [4.0, 8.0, 2.0]], [[7.0, 5.0, 7.0], [4.0, 4.0, 3.0], [3.0, 5.0, 6.0]], [[1.0, 8.0, 6.0], [4.0, 1.0, 4.0], 
[2.0, 3.0, 6.0]]], [[[3.0, 1.0, 3.0], [4.0, 3.0, 8.0], [7.0, 3.0, 6.0]], [[3.0, 1.0, 2.0], [6.0, 7.0, 3.0], [7.0, 8.0, 6.0]], 
[[5.0, 4.0, 4.0], [5.0, 1.0, 1.0], [2.0, 8.0, 8.0]]], [[[6.0, 2.0, 7.0], [4.0, 3.0, 4.0], [5.0, 5.0, 6.0]], [[5.0, 8.0, 6.0], 
[3.0, 5.0, 5.0], [5.0, 7.0, 4.0]], [[6.0, 4.0, 3.0], [4.0, 5.0, 7.0], [3.0, 7.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 7.0], [5.0, 6.0, 8.0], [8.0, 8.0, 5.0]], [[2.0, 1.0, 8.0], [6.0, 5.0, 4.0], [7.0, 
1.0, 6.0]], [[2.0, 7.0, 5.0], [4.0, 2.0, 7.0], [4.0, 6.0, 5.0]]], [[[6.0, 8.0, 2.0], [2.0, 3.0, 2.0], [8.0, 4.0, 6.0]], [[2.0, 
8.0, 8.0], [8.0, 1.0, 3.0], [3.0, 4.0, 6.0]], [[3.0, 1.0, 6.0], [8.0, 5.0, 5.0], [7.0, 6.0, 1.0]]], [[[4.0, 5.0, 5.0], [7.0, 
6.0, 2.0], [7.0, 5.0, 6.0]], [[1.0, 8.0, 5.0], [3.0, 6.0, 5.0], [1.0, 3.0, 3.0]], [[7.0, 6.0, 7.0], [5.0, 1.0, 1.0], [7.0, 3.0, 
4.0]]], [[[2.0, 7.0, 3.0], [3.0, 1.0, 4.0], [2.0, 8.0, 4.0]], [[7.0, 2.0, 5.0], [5.0, 4.0, 8.0], [5.0, 2.0, 4.0]], [[4.0, 6.0, 
4.0], [2.0, 6.0, 1.0], [7.0, 6.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 5.0], [7.0, 2.0, 3.0], [8.0, 2.0, 2.0]], 
[[6.0, 7.0, 1.0], [4.0, 6.0, 7.0], [3.0, 2.0, 8.0]], [[6.0, 1.0, 3.0], [5.0, 6.0, 2.0], [6.0, 7.0, 1.0]]], [[[5.0, 5.0, 4.0], 
[8.0, 1.0, 6.0], [7.0, 5.0, 2.0]], [[8.0, 7.0, 4.0], [1.0, 6.0, 7.0], [3.0, 1.0, 5.0]], [[7.0, 8.0, 4.0], [1.0, 4.0, 5.0], 
[2.0, 1.0, 8.0]]], [[[5.0, 5.0, 7.0], [3.0, 8.0, 3.0], [8.0, 2.0, 5.0]], [[4.0, 1.0, 6.0], [1.0, 8.0, 4.0], [4.0, 5.0, 5.0]], 
[[4.0, 4.0, 3.0], [8.0, 6.0, 6.0], [8.0, 1.0, 7.0]]], [[[5.0, 1.0, 5.0], [1.0, 4.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 2.0], 
[5.0, 8.0, 1.0], [5.0, 4.0, 6.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 1.0], [6.0, 5.0, 8.0]]]])
      val=Data(numpy.array([[[[1.0, 1.0, 2.0]]], [[[7.0, 7.0, 4.0]]]]),self.functionspace)
      arg[2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 7.0], [5.0, 6.0, 8.0], [8.0, 8.0, 5.0]], [[2.0, 1.0, 8.0], [6.0, 5.0, 4.0], [7.0, 
1.0, 6.0]], [[2.0, 7.0, 5.0], [4.0, 2.0, 7.0], [4.0, 6.0, 5.0]]], [[[6.0, 8.0, 2.0], [2.0, 3.0, 2.0], [8.0, 4.0, 6.0]], [[2.0, 
8.0, 8.0], [8.0, 1.0, 3.0], [3.0, 4.0, 6.0]], [[3.0, 1.0, 6.0], [8.0, 5.0, 5.0], [7.0, 6.0, 1.0]]], [[[4.0, 5.0, 5.0], [7.0, 
6.0, 2.0], [7.0, 5.0, 6.0]], [[1.0, 8.0, 5.0], [3.0, 6.0, 5.0], [1.0, 1.0, 2.0]], [[7.0, 6.0, 7.0], [5.0, 1.0, 1.0], [7.0, 3.0, 
4.0]]], [[[2.0, 7.0, 3.0], [3.0, 1.0, 4.0], [2.0, 8.0, 4.0]], [[7.0, 2.0, 5.0], [5.0, 4.0, 8.0], [7.0, 7.0, 4.0]], [[4.0, 6.0, 
4.0], [2.0, 6.0, 1.0], [7.0, 6.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 8.0, 5.0], [7.0, 2.0, 3.0], [8.0, 2.0, 2.0]], 
[[6.0, 7.0, 1.0], [4.0, 6.0, 7.0], [3.0, 2.0, 8.0]], [[6.0, 1.0, 3.0], [5.0, 6.0, 2.0], [6.0, 7.0, 1.0]]], [[[5.0, 5.0, 4.0], 
[8.0, 1.0, 6.0], [7.0, 5.0, 2.0]], [[8.0, 7.0, 4.0], [1.0, 6.0, 7.0], [3.0, 1.0, 5.0]], [[7.0, 8.0, 4.0], [1.0, 4.0, 5.0], 
[2.0, 1.0, 8.0]]], [[[5.0, 5.0, 7.0], [3.0, 8.0, 3.0], [8.0, 2.0, 5.0]], [[4.0, 1.0, 6.0], [1.0, 8.0, 4.0], [1.0, 1.0, 2.0]], 
[[4.0, 4.0, 3.0], [8.0, 6.0, 6.0], [8.0, 1.0, 7.0]]], [[[5.0, 1.0, 5.0], [1.0, 4.0, 8.0], [2.0, 5.0, 3.0]], [[5.0, 8.0, 2.0], 
[5.0, 8.0, 1.0], [7.0, 7.0, 4.0]], [[7.0, 1.0, 2.0], [6.0, 4.0, 1.0], [6.0, 5.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 4.0, 8.0], [6.0, 2.0, 2.0], [4.0, 7.0, 6.0]], [[5.0, 1.0, 4.0], [2.0, 2.0, 4.0], [8.0, 
1.0, 6.0]], [[5.0, 3.0, 7.0], [7.0, 7.0, 7.0], [6.0, 6.0, 3.0]]], [[[7.0, 2.0, 5.0], [2.0, 6.0, 2.0], [6.0, 7.0, 2.0]], [[8.0, 
6.0, 7.0], [4.0, 6.0, 1.0], [7.0, 6.0, 7.0]], [[8.0, 1.0, 5.0], [6.0, 6.0, 1.0], [6.0, 7.0, 2.0]]], [[[7.0, 1.0, 5.0], [3.0, 
6.0, 7.0], [5.0, 3.0, 2.0]], [[4.0, 7.0, 3.0], [7.0, 4.0, 7.0], [5.0, 6.0, 2.0]], [[8.0, 1.0, 4.0], [4.0, 2.0, 7.0], [6.0, 5.0, 
4.0]]], [[[6.0, 6.0, 8.0], [3.0, 8.0, 5.0], [6.0, 6.0, 5.0]], [[6.0, 2.0, 1.0], [1.0, 8.0, 5.0], [8.0, 7.0, 6.0]], [[5.0, 7.0, 
1.0], [8.0, 7.0, 8.0], [3.0, 3.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 1.0, 5.0], [6.0, 6.0, 8.0]], 
[[6.0, 4.0, 6.0], [5.0, 1.0, 6.0], [5.0, 8.0, 6.0]], [[8.0, 6.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 4.0]]], [[[6.0, 4.0, 8.0], 
[2.0, 2.0, 8.0], [5.0, 4.0, 7.0]], [[8.0, 5.0, 5.0], [5.0, 6.0, 4.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 6.0], [3.0, 1.0, 5.0], 
[6.0, 1.0, 5.0]]], [[[6.0, 6.0, 5.0], [7.0, 8.0, 7.0], [8.0, 7.0, 6.0]], [[6.0, 8.0, 3.0], [3.0, 2.0, 1.0], [3.0, 8.0, 6.0]], 
[[8.0, 2.0, 8.0], [1.0, 4.0, 7.0], [6.0, 5.0, 1.0]]], [[[6.0, 7.0, 6.0], [1.0, 5.0, 8.0], [8.0, 2.0, 2.0]], [[4.0, 7.0, 7.0], 
[6.0, 5.0, 2.0], [1.0, 2.0, 6.0]], [[6.0, 2.0, 8.0], [8.0, 4.0, 1.0], [5.0, 2.0, 2.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 5.0, 8.0]]], 
[[[4.0, 5.0, 4.0]]]])
      arg[2:,1:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 4.0, 8.0], [6.0, 2.0, 2.0], [4.0, 7.0, 6.0]], [[5.0, 1.0, 4.0], [2.0, 2.0, 4.0], [8.0, 
1.0, 6.0]], [[5.0, 3.0, 7.0], [7.0, 7.0, 7.0], [6.0, 6.0, 3.0]]], [[[7.0, 2.0, 5.0], [2.0, 6.0, 2.0], [6.0, 7.0, 2.0]], [[8.0, 
6.0, 7.0], [4.0, 6.0, 1.0], [7.0, 6.0, 7.0]], [[8.0, 1.0, 5.0], [6.0, 6.0, 1.0], [6.0, 7.0, 2.0]]], [[[7.0, 1.0, 5.0], [3.0, 
6.0, 7.0], [5.0, 3.0, 2.0]], [[4.0, 7.0, 3.0], [7.0, 4.0, 7.0], [4.0, 2.0, 4.0]], [[8.0, 1.0, 4.0], [4.0, 2.0, 7.0], [6.0, 5.0, 
4.0]]], [[[6.0, 6.0, 8.0], [3.0, 8.0, 5.0], [6.0, 6.0, 5.0]], [[6.0, 2.0, 1.0], [1.0, 8.0, 5.0], [2.0, 1.0, 4.0]], [[5.0, 7.0, 
1.0], [8.0, 7.0, 8.0], [3.0, 3.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 2.0], [2.0, 1.0, 5.0], [6.0, 6.0, 8.0]], 
[[6.0, 4.0, 6.0], [5.0, 1.0, 6.0], [5.0, 8.0, 6.0]], [[8.0, 6.0, 5.0], [1.0, 7.0, 6.0], [1.0, 8.0, 4.0]]], [[[6.0, 4.0, 8.0], 
[2.0, 2.0, 8.0], [5.0, 4.0, 7.0]], [[8.0, 5.0, 5.0], [5.0, 6.0, 4.0], [4.0, 8.0, 6.0]], [[7.0, 3.0, 6.0], [3.0, 1.0, 5.0], 
[6.0, 1.0, 5.0]]], [[[6.0, 6.0, 5.0], [7.0, 8.0, 7.0], [8.0, 7.0, 6.0]], [[6.0, 8.0, 3.0], [3.0, 2.0, 1.0], [7.0, 5.0, 8.0]], 
[[8.0, 2.0, 8.0], [1.0, 4.0, 7.0], [6.0, 5.0, 1.0]]], [[[6.0, 7.0, 6.0], [1.0, 5.0, 8.0], [8.0, 2.0, 2.0]], [[4.0, 7.0, 7.0], 
[6.0, 5.0, 2.0], [4.0, 5.0, 4.0]], [[6.0, 2.0, 8.0], [8.0, 4.0, 1.0], [5.0, 2.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 8.0, 4.0], [8.0, 1.0, 1.0], [4.0, 3.0, 2.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 8.0], [3.0, 
1.0, 1.0]], [[5.0, 8.0, 2.0], [1.0, 1.0, 3.0], [5.0, 8.0, 6.0]]], [[[3.0, 4.0, 5.0], [1.0, 8.0, 4.0], [6.0, 8.0, 6.0]], [[1.0, 
2.0, 4.0], [4.0, 5.0, 2.0], [1.0, 1.0, 3.0]], [[6.0, 4.0, 5.0], [1.0, 6.0, 7.0], [1.0, 5.0, 3.0]]], [[[7.0, 5.0, 8.0], [7.0, 
4.0, 1.0], [1.0, 4.0, 2.0]], [[4.0, 2.0, 6.0], [8.0, 3.0, 5.0], [7.0, 4.0, 1.0]], [[5.0, 5.0, 1.0], [3.0, 4.0, 1.0], [1.0, 4.0, 
8.0]]], [[[5.0, 1.0, 6.0], [8.0, 3.0, 2.0], [8.0, 1.0, 7.0]], [[1.0, 2.0, 3.0], [3.0, 7.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 
6.0], [7.0, 3.0, 7.0], [1.0, 4.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 5.0, 7.0], [3.0, 6.0, 5.0], [5.0, 8.0, 8.0]], 
[[5.0, 2.0, 5.0], [2.0, 6.0, 4.0], [8.0, 4.0, 3.0]], [[8.0, 6.0, 6.0], [5.0, 5.0, 8.0], [8.0, 5.0, 4.0]]], [[[4.0, 5.0, 6.0], 
[3.0, 1.0, 1.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 8.0], [7.0, 1.0, 6.0], [6.0, 3.0, 3.0]], [[3.0, 3.0, 2.0], [2.0, 8.0, 4.0], 
[6.0, 8.0, 7.0]]], [[[7.0, 6.0, 6.0], [4.0, 2.0, 8.0], [4.0, 1.0, 3.0]], [[3.0, 2.0, 1.0], [8.0, 7.0, 1.0], [3.0, 6.0, 6.0]], 
[[5.0, 5.0, 8.0], [8.0, 2.0, 3.0], [8.0, 2.0, 1.0]]], [[[1.0, 5.0, 2.0], [4.0, 5.0, 7.0], [8.0, 1.0, 2.0]], [[3.0, 2.0, 7.0], 
[3.0, 1.0, 8.0], [1.0, 4.0, 4.0]], [[6.0, 5.0, 1.0], [3.0, 1.0, 7.0], [7.0, 6.0, 8.0]]]])
      val=numpy.array([[[3.0, 2.0, 8.0]], [[4.0, 7.0, 5.0]]])
      arg[2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 8.0, 4.0], [8.0, 1.0, 1.0], [4.0, 3.0, 2.0]], [[5.0, 1.0, 3.0], [6.0, 3.0, 8.0], [3.0, 
1.0, 1.0]], [[5.0, 8.0, 2.0], [1.0, 1.0, 3.0], [5.0, 8.0, 6.0]]], [[[3.0, 4.0, 5.0], [1.0, 8.0, 4.0], [6.0, 8.0, 6.0]], [[1.0, 
2.0, 4.0], [4.0, 5.0, 2.0], [1.0, 1.0, 3.0]], [[6.0, 4.0, 5.0], [1.0, 6.0, 7.0], [1.0, 5.0, 3.0]]], [[[7.0, 5.0, 8.0], [7.0, 
4.0, 1.0], [1.0, 4.0, 2.0]], [[4.0, 2.0, 6.0], [8.0, 3.0, 5.0], [7.0, 4.0, 1.0]], [[5.0, 5.0, 1.0], [3.0, 2.0, 8.0], [1.0, 4.0, 
8.0]]], [[[5.0, 1.0, 6.0], [8.0, 3.0, 2.0], [8.0, 1.0, 7.0]], [[1.0, 2.0, 3.0], [3.0, 7.0, 2.0], [2.0, 2.0, 3.0]], [[6.0, 7.0, 
6.0], [4.0, 7.0, 5.0], [1.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 5.0, 7.0], [3.0, 6.0, 5.0], [5.0, 8.0, 8.0]], 
[[5.0, 2.0, 5.0], [2.0, 6.0, 4.0], [8.0, 4.0, 3.0]], [[8.0, 6.0, 6.0], [5.0, 5.0, 8.0], [8.0, 5.0, 4.0]]], [[[4.0, 5.0, 6.0], 
[3.0, 1.0, 1.0], [3.0, 8.0, 2.0]], [[5.0, 2.0, 8.0], [7.0, 1.0, 6.0], [6.0, 3.0, 3.0]], [[3.0, 3.0, 2.0], [2.0, 8.0, 4.0], 
[6.0, 8.0, 7.0]]], [[[7.0, 6.0, 6.0], [4.0, 2.0, 8.0], [4.0, 1.0, 3.0]], [[3.0, 2.0, 1.0], [8.0, 7.0, 1.0], [3.0, 6.0, 6.0]], 
[[5.0, 5.0, 8.0], [3.0, 2.0, 8.0], [8.0, 2.0, 1.0]]], [[[1.0, 5.0, 2.0], [4.0, 5.0, 7.0], [8.0, 1.0, 2.0]], [[3.0, 2.0, 7.0], 
[3.0, 1.0, 8.0], [1.0, 4.0, 4.0]], [[6.0, 5.0, 1.0], [4.0, 7.0, 5.0], [7.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 1.0, 1.0], [1.0, 4.0, 7.0], [6.0, 2.0, 2.0]], [[4.0, 5.0, 2.0], [3.0, 6.0, 8.0], [6.0, 
7.0, 2.0]], [[5.0, 3.0, 2.0], [1.0, 8.0, 3.0], [5.0, 7.0, 3.0]]], [[[3.0, 4.0, 4.0], [3.0, 8.0, 5.0], [7.0, 5.0, 6.0]], [[5.0, 
8.0, 8.0], [4.0, 8.0, 6.0], [7.0, 2.0, 7.0]], [[4.0, 2.0, 8.0], [8.0, 5.0, 7.0], [7.0, 2.0, 1.0]]], [[[2.0, 5.0, 2.0], [2.0, 
2.0, 3.0], [2.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 4.0, 2.0], [6.0, 2.0, 2.0]], [[6.0, 5.0, 6.0], [7.0, 1.0, 8.0], [8.0, 4.0, 
1.0]]], [[[3.0, 3.0, 3.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 6.0, 8.0], [1.0, 2.0, 4.0]], [[6.0, 4.0, 
2.0], [4.0, 3.0, 3.0], [2.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 1.0], [8.0, 3.0, 1.0], [1.0, 7.0, 5.0]], 
[[5.0, 6.0, 3.0], [7.0, 4.0, 5.0], [1.0, 3.0, 2.0]], [[5.0, 6.0, 3.0], [1.0, 6.0, 3.0], [4.0, 5.0, 6.0]]], [[[4.0, 7.0, 7.0], 
[2.0, 3.0, 7.0], [2.0, 5.0, 5.0]], [[1.0, 8.0, 6.0], [6.0, 5.0, 7.0], [2.0, 2.0, 7.0]], [[2.0, 8.0, 1.0], [8.0, 6.0, 8.0], 
[5.0, 3.0, 1.0]]], [[[4.0, 6.0, 4.0], [3.0, 7.0, 8.0], [5.0, 6.0, 6.0]], [[4.0, 1.0, 8.0], [2.0, 8.0, 4.0], [3.0, 2.0, 8.0]], 
[[6.0, 4.0, 5.0], [1.0, 6.0, 3.0], [4.0, 3.0, 8.0]]], [[[4.0, 2.0, 1.0], [8.0, 4.0, 8.0], [4.0, 5.0, 7.0]], [[7.0, 3.0, 8.0], 
[4.0, 1.0, 8.0], [1.0, 2.0, 7.0]], [[6.0, 8.0, 2.0], [1.0, 5.0, 7.0], [1.0, 8.0, 4.0]]]])
      val=Data(numpy.array([[[3.0, 6.0, 5.0]], [[6.0, 2.0, 8.0]]]),self.functionspace)
      arg[2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 1.0, 1.0], [1.0, 4.0, 7.0], [6.0, 2.0, 2.0]], [[4.0, 5.0, 2.0], [3.0, 6.0, 8.0], [6.0, 
7.0, 2.0]], [[5.0, 3.0, 2.0], [1.0, 8.0, 3.0], [5.0, 7.0, 3.0]]], [[[3.0, 4.0, 4.0], [3.0, 8.0, 5.0], [7.0, 5.0, 6.0]], [[5.0, 
8.0, 8.0], [4.0, 8.0, 6.0], [7.0, 2.0, 7.0]], [[4.0, 2.0, 8.0], [8.0, 5.0, 7.0], [7.0, 2.0, 1.0]]], [[[2.0, 5.0, 2.0], [2.0, 
2.0, 3.0], [2.0, 8.0, 2.0]], [[2.0, 4.0, 1.0], [3.0, 4.0, 2.0], [6.0, 2.0, 2.0]], [[6.0, 5.0, 6.0], [3.0, 6.0, 5.0], [8.0, 4.0, 
1.0]]], [[[3.0, 3.0, 3.0], [7.0, 4.0, 7.0], [1.0, 2.0, 1.0]], [[5.0, 3.0, 5.0], [5.0, 6.0, 8.0], [1.0, 2.0, 4.0]], [[6.0, 4.0, 
2.0], [6.0, 2.0, 8.0], [2.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 1.0], [8.0, 3.0, 1.0], [1.0, 7.0, 5.0]], 
[[5.0, 6.0, 3.0], [7.0, 4.0, 5.0], [1.0, 3.0, 2.0]], [[5.0, 6.0, 3.0], [1.0, 6.0, 3.0], [4.0, 5.0, 6.0]]], [[[4.0, 7.0, 7.0], 
[2.0, 3.0, 7.0], [2.0, 5.0, 5.0]], [[1.0, 8.0, 6.0], [6.0, 5.0, 7.0], [2.0, 2.0, 7.0]], [[2.0, 8.0, 1.0], [8.0, 6.0, 8.0], 
[5.0, 3.0, 1.0]]], [[[4.0, 6.0, 4.0], [3.0, 7.0, 8.0], [5.0, 6.0, 6.0]], [[4.0, 1.0, 8.0], [2.0, 8.0, 4.0], [3.0, 2.0, 8.0]], 
[[6.0, 4.0, 5.0], [3.0, 6.0, 5.0], [4.0, 3.0, 8.0]]], [[[4.0, 2.0, 1.0], [8.0, 4.0, 8.0], [4.0, 5.0, 7.0]], [[7.0, 3.0, 8.0], 
[4.0, 1.0, 8.0], [1.0, 2.0, 7.0]], [[6.0, 8.0, 2.0], [6.0, 2.0, 8.0], [1.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LLPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 3.0], [8.0, 6.0, 3.0], [7.0, 3.0, 8.0]], [[8.0, 3.0, 4.0], [8.0, 3.0, 2.0], [3.0, 
3.0, 7.0]], [[1.0, 4.0, 4.0], [2.0, 4.0, 1.0], [3.0, 8.0, 3.0]]], [[[4.0, 5.0, 1.0], [1.0, 4.0, 4.0], [1.0, 5.0, 7.0]], [[4.0, 
4.0, 1.0], [3.0, 1.0, 3.0], [1.0, 6.0, 2.0]], [[2.0, 7.0, 8.0], [1.0, 5.0, 4.0], [3.0, 7.0, 2.0]]], [[[3.0, 8.0, 1.0], [3.0, 
1.0, 2.0], [2.0, 7.0, 4.0]], [[3.0, 6.0, 7.0], [6.0, 2.0, 3.0], [1.0, 1.0, 7.0]], [[7.0, 7.0, 2.0], [8.0, 8.0, 7.0], [5.0, 1.0, 
2.0]]], [[[2.0, 3.0, 5.0], [2.0, 5.0, 6.0], [3.0, 5.0, 6.0]], [[4.0, 2.0, 1.0], [8.0, 4.0, 4.0], [1.0, 6.0, 5.0]], [[5.0, 6.0, 
6.0], [6.0, 5.0, 8.0], [1.0, 4.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 8.0, 7.0], [3.0, 6.0, 7.0]], 
[[5.0, 8.0, 3.0], [1.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[2.0, 4.0, 6.0], [7.0, 5.0, 4.0], [6.0, 2.0, 7.0]]], [[[4.0, 3.0, 8.0], 
[8.0, 7.0, 6.0], [1.0, 8.0, 8.0]], [[3.0, 7.0, 5.0], [8.0, 2.0, 1.0], [8.0, 5.0, 1.0]], [[5.0, 5.0, 6.0], [7.0, 8.0, 1.0], 
[6.0, 6.0, 5.0]]], [[[6.0, 4.0, 2.0], [4.0, 3.0, 1.0], [1.0, 1.0, 7.0]], [[6.0, 4.0, 4.0], [3.0, 4.0, 7.0], [3.0, 2.0, 1.0]], 
[[2.0, 1.0, 2.0], [8.0, 1.0, 1.0], [1.0, 5.0, 2.0]]], [[[7.0, 7.0, 2.0], [3.0, 5.0, 8.0], [1.0, 4.0, 4.0]], [[5.0, 6.0, 5.0], 
[1.0, 8.0, 5.0], [8.0, 4.0, 2.0]], [[5.0, 3.0, 1.0], [7.0, 8.0, 6.0], [4.0, 6.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[3.0, 8.0, 7.0]], [[1.0, 4.0, 5.0]]])+(1.-msk_val)*numpy.array([[[4.0, 6.0, 1.0]], [[1.0, 7.0, 
4.0]]])
      arg[2:,2:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 3.0], [8.0, 6.0, 3.0], [7.0, 3.0, 8.0]], [[8.0, 3.0, 4.0], [8.0, 3.0, 2.0], [3.0, 
3.0, 7.0]], [[1.0, 4.0, 4.0], [2.0, 4.0, 1.0], [3.0, 8.0, 3.0]]], [[[4.0, 5.0, 1.0], [1.0, 4.0, 4.0], [1.0, 5.0, 7.0]], [[4.0, 
4.0, 1.0], [3.0, 1.0, 3.0], [1.0, 6.0, 2.0]], [[2.0, 7.0, 8.0], [1.0, 5.0, 4.0], [3.0, 7.0, 2.0]]], [[[3.0, 8.0, 1.0], [3.0, 
1.0, 2.0], [2.0, 7.0, 4.0]], [[3.0, 6.0, 7.0], [6.0, 2.0, 3.0], [1.0, 1.0, 7.0]], [[7.0, 7.0, 2.0], [3.0, 8.0, 7.0], [5.0, 1.0, 
2.0]]], [[[2.0, 3.0, 5.0], [2.0, 5.0, 6.0], [3.0, 5.0, 6.0]], [[4.0, 2.0, 1.0], [8.0, 4.0, 4.0], [1.0, 6.0, 5.0]], [[5.0, 6.0, 
6.0], [1.0, 4.0, 5.0], [1.0, 4.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 8.0, 6.0], [2.0, 8.0, 7.0], [3.0, 6.0, 7.0]], 
[[5.0, 8.0, 3.0], [1.0, 6.0, 5.0], [1.0, 5.0, 1.0]], [[2.0, 4.0, 6.0], [7.0, 5.0, 4.0], [6.0, 2.0, 7.0]]], [[[4.0, 3.0, 8.0], 
[8.0, 7.0, 6.0], [1.0, 8.0, 8.0]], [[3.0, 7.0, 5.0], [8.0, 2.0, 1.0], [8.0, 5.0, 1.0]], [[5.0, 5.0, 6.0], [7.0, 8.0, 1.0], 
[6.0, 6.0, 5.0]]], [[[6.0, 4.0, 2.0], [4.0, 3.0, 1.0], [1.0, 1.0, 7.0]], [[6.0, 4.0, 4.0], [3.0, 4.0, 7.0], [3.0, 2.0, 1.0]], 
[[2.0, 1.0, 2.0], [4.0, 6.0, 1.0], [1.0, 5.0, 2.0]]], [[[7.0, 7.0, 2.0], [3.0, 5.0, 8.0], [1.0, 4.0, 4.0]], [[5.0, 6.0, 5.0], 
[1.0, 8.0, 5.0], [8.0, 4.0, 2.0]], [[5.0, 3.0, 1.0], [1.0, 7.0, 4.0], [4.0, 6.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 5.0, 5.0], [5.0, 7.0, 1.0], [2.0, 4.0, 1.0]], [[8.0, 5.0, 1.0], [5.0, 6.0, 7.0], [3.0, 
1.0, 2.0]], [[2.0, 3.0, 6.0], [3.0, 5.0, 7.0], [7.0, 4.0, 8.0]]], [[[5.0, 1.0, 5.0], [4.0, 4.0, 1.0], [2.0, 7.0, 2.0]], [[5.0, 
3.0, 8.0], [3.0, 4.0, 2.0], [2.0, 2.0, 2.0]], [[4.0, 4.0, 7.0], [3.0, 3.0, 8.0], [3.0, 4.0, 4.0]]], [[[4.0, 6.0, 2.0], [8.0, 
5.0, 5.0], [4.0, 1.0, 7.0]], [[6.0, 1.0, 3.0], [8.0, 5.0, 8.0], [6.0, 6.0, 8.0]], [[3.0, 1.0, 1.0], [3.0, 5.0, 3.0], [6.0, 7.0, 
1.0]]], [[[6.0, 8.0, 6.0], [7.0, 8.0, 5.0], [2.0, 2.0, 1.0]], [[1.0, 7.0, 2.0], [2.0, 4.0, 4.0], [2.0, 6.0, 1.0]], [[7.0, 6.0, 
4.0], [5.0, 7.0, 1.0], [3.0, 6.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 8.0], [4.0, 6.0, 1.0], [1.0, 4.0, 2.0]], 
[[5.0, 5.0, 2.0], [3.0, 3.0, 5.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 8.0], [1.0, 6.0, 6.0]]], [[[5.0, 7.0, 6.0], 
[1.0, 5.0, 3.0], [2.0, 3.0, 2.0]], [[5.0, 5.0, 4.0], [3.0, 1.0, 2.0], [4.0, 1.0, 7.0]], [[6.0, 2.0, 4.0], [8.0, 3.0, 7.0], 
[2.0, 4.0, 4.0]]], [[[6.0, 2.0, 3.0], [7.0, 8.0, 8.0], [3.0, 8.0, 2.0]], [[1.0, 3.0, 7.0], [4.0, 1.0, 3.0], [6.0, 3.0, 1.0]], 
[[1.0, 5.0, 6.0], [7.0, 8.0, 2.0], [7.0, 2.0, 1.0]]], [[[6.0, 2.0, 4.0], [8.0, 2.0, 7.0], [6.0, 8.0, 4.0]], [[8.0, 2.0, 8.0], 
[1.0, 1.0, 3.0], [5.0, 4.0, 3.0]], [[1.0, 1.0, 6.0], [6.0, 4.0, 1.0], [6.0, 7.0, 2.0]]]])
      val=numpy.array([[[[6.0, 7.0, 8.0]], [[3.0, 1.0, 1.0]]], [[[6.0, 1.0, 3.0]], [[8.0, 2.0, 2.0]]]])
      arg[1:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 5.0, 5.0], [5.0, 7.0, 1.0], [2.0, 4.0, 1.0]], [[8.0, 5.0, 1.0], [5.0, 6.0, 7.0], [3.0, 
1.0, 2.0]], [[2.0, 3.0, 6.0], [3.0, 5.0, 7.0], [7.0, 4.0, 8.0]]], [[[5.0, 1.0, 5.0], [4.0, 4.0, 1.0], [6.0, 7.0, 8.0]], [[5.0, 
3.0, 8.0], [3.0, 4.0, 2.0], [3.0, 1.0, 1.0]], [[4.0, 4.0, 7.0], [3.0, 3.0, 8.0], [3.0, 4.0, 4.0]]], [[[4.0, 6.0, 2.0], [8.0, 
5.0, 5.0], [6.0, 1.0, 3.0]], [[6.0, 1.0, 3.0], [8.0, 5.0, 8.0], [8.0, 2.0, 2.0]], [[3.0, 1.0, 1.0], [3.0, 5.0, 3.0], [6.0, 7.0, 
1.0]]], [[[6.0, 8.0, 6.0], [7.0, 8.0, 5.0], [2.0, 2.0, 1.0]], [[1.0, 7.0, 2.0], [2.0, 4.0, 4.0], [2.0, 6.0, 1.0]], [[7.0, 6.0, 
4.0], [5.0, 7.0, 1.0], [3.0, 6.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 8.0], [4.0, 6.0, 1.0], [1.0, 4.0, 2.0]], 
[[5.0, 5.0, 2.0], [3.0, 3.0, 5.0], [8.0, 7.0, 5.0]], [[6.0, 6.0, 6.0], [2.0, 2.0, 8.0], [1.0, 6.0, 6.0]]], [[[5.0, 7.0, 6.0], 
[1.0, 5.0, 3.0], [6.0, 7.0, 8.0]], [[5.0, 5.0, 4.0], [3.0, 1.0, 2.0], [3.0, 1.0, 1.0]], [[6.0, 2.0, 4.0], [8.0, 3.0, 7.0], 
[2.0, 4.0, 4.0]]], [[[6.0, 2.0, 3.0], [7.0, 8.0, 8.0], [6.0, 1.0, 3.0]], [[1.0, 3.0, 7.0], [4.0, 1.0, 3.0], [8.0, 2.0, 2.0]], 
[[1.0, 5.0, 6.0], [7.0, 8.0, 2.0], [7.0, 2.0, 1.0]]], [[[6.0, 2.0, 4.0], [8.0, 2.0, 7.0], [6.0, 8.0, 4.0]], [[8.0, 2.0, 8.0], 
[1.0, 1.0, 3.0], [5.0, 4.0, 3.0]], [[1.0, 1.0, 6.0], [6.0, 4.0, 1.0], [6.0, 7.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 1.0], [4.0, 7.0, 8.0], [3.0, 1.0, 2.0]], [[5.0, 7.0, 8.0], [5.0, 5.0, 8.0], [8.0, 
2.0, 8.0]], [[3.0, 2.0, 6.0], [1.0, 3.0, 7.0], [3.0, 7.0, 1.0]]], [[[7.0, 1.0, 5.0], [7.0, 7.0, 3.0], [6.0, 3.0, 5.0]], [[5.0, 
1.0, 5.0], [8.0, 3.0, 6.0], [8.0, 8.0, 5.0]], [[5.0, 3.0, 5.0], [3.0, 7.0, 2.0], [4.0, 7.0, 3.0]]], [[[6.0, 1.0, 5.0], [6.0, 
3.0, 5.0], [5.0, 5.0, 1.0]], [[3.0, 6.0, 6.0], [6.0, 7.0, 3.0], [1.0, 4.0, 6.0]], [[7.0, 4.0, 6.0], [2.0, 5.0, 6.0], [4.0, 7.0, 
1.0]]], [[[8.0, 4.0, 4.0], [6.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 7.0], [4.0, 2.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 4.0, 
1.0], [4.0, 2.0, 8.0], [2.0, 4.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 5.0, 3.0], [7.0, 5.0, 6.0], [2.0, 5.0, 8.0]], 
[[4.0, 8.0, 7.0], [6.0, 5.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 4.0, 5.0], [7.0, 2.0, 3.0]]], [[[8.0, 8.0, 3.0], 
[3.0, 6.0, 5.0], [4.0, 4.0, 5.0]], [[8.0, 3.0, 4.0], [8.0, 5.0, 1.0], [8.0, 1.0, 5.0]], [[7.0, 6.0, 4.0], [8.0, 3.0, 1.0], 
[6.0, 2.0, 2.0]]], [[[5.0, 1.0, 5.0], [6.0, 3.0, 4.0], [6.0, 6.0, 1.0]], [[3.0, 5.0, 1.0], [1.0, 8.0, 7.0], [4.0, 8.0, 7.0]], 
[[6.0, 6.0, 2.0], [8.0, 4.0, 5.0], [1.0, 7.0, 7.0]]], [[[5.0, 5.0, 1.0], [8.0, 4.0, 5.0], [4.0, 1.0, 1.0]], [[1.0, 7.0, 7.0], 
[5.0, 5.0, 5.0], [1.0, 2.0, 1.0]], [[8.0, 8.0, 2.0], [1.0, 4.0, 5.0], [8.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[[4.0, 1.0, 7.0]], [[7.0, 6.0, 6.0]]], [[[1.0, 8.0, 7.0]], [[1.0, 1.0, 2.0]]]]),self.functionspace)
      arg[1:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 1.0], [4.0, 7.0, 8.0], [3.0, 1.0, 2.0]], [[5.0, 7.0, 8.0], [5.0, 5.0, 8.0], [8.0, 
2.0, 8.0]], [[3.0, 2.0, 6.0], [1.0, 3.0, 7.0], [3.0, 7.0, 1.0]]], [[[7.0, 1.0, 5.0], [7.0, 7.0, 3.0], [4.0, 1.0, 7.0]], [[5.0, 
1.0, 5.0], [8.0, 3.0, 6.0], [7.0, 6.0, 6.0]], [[5.0, 3.0, 5.0], [3.0, 7.0, 2.0], [4.0, 7.0, 3.0]]], [[[6.0, 1.0, 5.0], [6.0, 
3.0, 5.0], [1.0, 8.0, 7.0]], [[3.0, 6.0, 6.0], [6.0, 7.0, 3.0], [1.0, 1.0, 2.0]], [[7.0, 4.0, 6.0], [2.0, 5.0, 6.0], [4.0, 7.0, 
1.0]]], [[[8.0, 4.0, 4.0], [6.0, 4.0, 7.0], [5.0, 2.0, 7.0]], [[6.0, 3.0, 7.0], [4.0, 2.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 4.0, 
1.0], [4.0, 2.0, 8.0], [2.0, 4.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 5.0, 3.0], [7.0, 5.0, 6.0], [2.0, 5.0, 8.0]], 
[[4.0, 8.0, 7.0], [6.0, 5.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 2.0, 4.0], [5.0, 4.0, 5.0], [7.0, 2.0, 3.0]]], [[[8.0, 8.0, 3.0], 
[3.0, 6.0, 5.0], [4.0, 1.0, 7.0]], [[8.0, 3.0, 4.0], [8.0, 5.0, 1.0], [7.0, 6.0, 6.0]], [[7.0, 6.0, 4.0], [8.0, 3.0, 1.0], 
[6.0, 2.0, 2.0]]], [[[5.0, 1.0, 5.0], [6.0, 3.0, 4.0], [1.0, 8.0, 7.0]], [[3.0, 5.0, 1.0], [1.0, 8.0, 7.0], [1.0, 1.0, 2.0]], 
[[6.0, 6.0, 2.0], [8.0, 4.0, 5.0], [1.0, 7.0, 7.0]]], [[[5.0, 5.0, 1.0], [8.0, 4.0, 5.0], [4.0, 1.0, 1.0]], [[1.0, 7.0, 7.0], 
[5.0, 5.0, 5.0], [1.0, 2.0, 1.0]], [[8.0, 8.0, 2.0], [1.0, 4.0, 5.0], [8.0, 7.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RULN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 3.0, 1.0], [3.0, 3.0, 8.0], [5.0, 5.0, 5.0]], [[5.0, 7.0, 7.0], [2.0, 7.0, 2.0], [3.0, 
8.0, 4.0]], [[3.0, 4.0, 4.0], [1.0, 5.0, 7.0], [3.0, 3.0, 4.0]]], [[[2.0, 1.0, 8.0], [6.0, 7.0, 5.0], [3.0, 7.0, 4.0]], [[6.0, 
2.0, 4.0], [5.0, 2.0, 7.0], [6.0, 6.0, 6.0]], [[5.0, 7.0, 2.0], [2.0, 6.0, 6.0], [4.0, 4.0, 5.0]]], [[[4.0, 1.0, 1.0], [6.0, 
1.0, 1.0], [8.0, 5.0, 3.0]], [[7.0, 4.0, 5.0], [3.0, 5.0, 3.0], [1.0, 7.0, 8.0]], [[2.0, 8.0, 8.0], [2.0, 2.0, 6.0], [8.0, 4.0, 
2.0]]], [[[3.0, 5.0, 8.0], [1.0, 2.0, 5.0], [1.0, 2.0, 8.0]], [[4.0, 3.0, 8.0], [6.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[8.0, 5.0, 
8.0], [8.0, 3.0, 1.0], [6.0, 2.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 1.0], [1.0, 2.0, 1.0], [8.0, 8.0, 5.0]], 
[[2.0, 8.0, 7.0], [1.0, 8.0, 4.0], [6.0, 4.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 1.0, 7.0], [5.0, 7.0, 5.0]]], [[[2.0, 5.0, 1.0], 
[7.0, 2.0, 3.0], [7.0, 7.0, 8.0]], [[7.0, 8.0, 8.0], [6.0, 7.0, 1.0], [4.0, 8.0, 1.0]], [[6.0, 8.0, 8.0], [4.0, 2.0, 5.0], 
[8.0, 6.0, 7.0]]], [[[3.0, 1.0, 5.0], [5.0, 3.0, 6.0], [1.0, 3.0, 3.0]], [[4.0, 7.0, 2.0], [4.0, 1.0, 3.0], [8.0, 8.0, 6.0]], 
[[5.0, 6.0, 3.0], [4.0, 4.0, 3.0], [1.0, 8.0, 6.0]]], [[[2.0, 4.0, 7.0], [8.0, 4.0, 1.0], [5.0, 5.0, 7.0]], [[6.0, 3.0, 4.0], 
[7.0, 6.0, 3.0], [4.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [7.0, 7.0, 2.0], [1.0, 8.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 8.0, 6.0]], [[7.0, 1.0, 5.0]]], [[[1.0, 6.0, 2.0]], [[5.0, 7.0, 
6.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 7.0, 6.0]], [[6.0, 6.0, 7.0]]], [[[8.0, 8.0, 1.0]], [[1.0, 5.0, 8.0]]]])
      arg[1:3,:2,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 3.0, 1.0], [3.0, 3.0, 8.0], [5.0, 5.0, 5.0]], [[5.0, 7.0, 7.0], [2.0, 7.0, 2.0], [3.0, 
8.0, 4.0]], [[3.0, 4.0, 4.0], [1.0, 5.0, 7.0], [3.0, 3.0, 4.0]]], [[[2.0, 1.0, 8.0], [6.0, 7.0, 5.0], [3.0, 8.0, 6.0]], [[6.0, 
2.0, 4.0], [5.0, 2.0, 7.0], [7.0, 1.0, 5.0]], [[5.0, 7.0, 2.0], [2.0, 6.0, 6.0], [4.0, 4.0, 5.0]]], [[[4.0, 1.0, 1.0], [6.0, 
1.0, 1.0], [1.0, 6.0, 2.0]], [[7.0, 4.0, 5.0], [3.0, 5.0, 3.0], [5.0, 7.0, 6.0]], [[2.0, 8.0, 8.0], [2.0, 2.0, 6.0], [8.0, 4.0, 
2.0]]], [[[3.0, 5.0, 8.0], [1.0, 2.0, 5.0], [1.0, 2.0, 8.0]], [[4.0, 3.0, 8.0], [6.0, 3.0, 5.0], [2.0, 7.0, 5.0]], [[8.0, 5.0, 
8.0], [8.0, 3.0, 1.0], [6.0, 2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 1.0], [1.0, 2.0, 1.0], [8.0, 8.0, 5.0]], 
[[2.0, 8.0, 7.0], [1.0, 8.0, 4.0], [6.0, 4.0, 4.0]], [[6.0, 6.0, 4.0], [2.0, 1.0, 7.0], [5.0, 7.0, 5.0]]], [[[2.0, 5.0, 1.0], 
[7.0, 2.0, 3.0], [2.0, 7.0, 6.0]], [[7.0, 8.0, 8.0], [6.0, 7.0, 1.0], [6.0, 6.0, 7.0]], [[6.0, 8.0, 8.0], [4.0, 2.0, 5.0], 
[8.0, 6.0, 7.0]]], [[[3.0, 1.0, 5.0], [5.0, 3.0, 6.0], [8.0, 8.0, 1.0]], [[4.0, 7.0, 2.0], [4.0, 1.0, 3.0], [1.0, 5.0, 8.0]], 
[[5.0, 6.0, 3.0], [4.0, 4.0, 3.0], [1.0, 8.0, 6.0]]], [[[2.0, 4.0, 7.0], [8.0, 4.0, 1.0], [5.0, 5.0, 7.0]], [[6.0, 3.0, 4.0], 
[7.0, 6.0, 3.0], [4.0, 5.0, 7.0]], [[5.0, 4.0, 3.0], [7.0, 7.0, 2.0], [1.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 2.0], [6.0, 8.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 7.0, 3.0], [1.0, 1.0, 6.0], [6.0, 
8.0, 1.0]], [[1.0, 8.0, 1.0], [1.0, 4.0, 6.0], [6.0, 4.0, 2.0]]], [[[8.0, 4.0, 5.0], [1.0, 4.0, 2.0], [4.0, 5.0, 2.0]], [[8.0, 
3.0, 1.0], [5.0, 7.0, 7.0], [4.0, 7.0, 7.0]], [[8.0, 3.0, 2.0], [3.0, 4.0, 3.0], [7.0, 4.0, 3.0]]], [[[4.0, 2.0, 2.0], [8.0, 
7.0, 2.0], [7.0, 4.0, 3.0]], [[4.0, 3.0, 5.0], [6.0, 5.0, 3.0], [6.0, 1.0, 3.0]], [[7.0, 7.0, 3.0], [7.0, 1.0, 6.0], [4.0, 2.0, 
2.0]]], [[[4.0, 4.0, 4.0], [2.0, 8.0, 2.0], [5.0, 7.0, 2.0]], [[6.0, 6.0, 4.0], [5.0, 7.0, 1.0], [6.0, 4.0, 8.0]], [[8.0, 7.0, 
2.0], [2.0, 1.0, 5.0], [2.0, 6.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 4.0], [7.0, 1.0, 8.0], [3.0, 2.0, 6.0]], 
[[8.0, 8.0, 3.0], [6.0, 5.0, 7.0], [4.0, 7.0, 1.0]], [[2.0, 3.0, 7.0], [4.0, 3.0, 1.0], [4.0, 3.0, 3.0]]], [[[1.0, 2.0, 6.0], 
[5.0, 1.0, 4.0], [8.0, 1.0, 5.0]], [[1.0, 7.0, 2.0], [3.0, 6.0, 3.0], [1.0, 4.0, 7.0]], [[8.0, 2.0, 7.0], [6.0, 2.0, 2.0], 
[5.0, 4.0, 7.0]]], [[[4.0, 2.0, 1.0], [5.0, 1.0, 6.0], [8.0, 7.0, 2.0]], [[5.0, 4.0, 4.0], [2.0, 2.0, 2.0], [1.0, 4.0, 5.0]], 
[[2.0, 2.0, 3.0], [5.0, 1.0, 7.0], [1.0, 7.0, 4.0]]], [[[2.0, 5.0, 8.0], [2.0, 3.0, 4.0], [6.0, 4.0, 6.0]], [[5.0, 4.0, 4.0], 
[8.0, 2.0, 3.0], [8.0, 5.0, 7.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 8.0], [1.0, 5.0, 1.0]]]])
      val=numpy.array([[[1.0, 6.0, 1.0]], [[5.0, 5.0, 7.0]], [[8.0, 5.0, 6.0]], [[6.0, 7.0, 7.0]]])
      arg[:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 2.0], [6.0, 8.0, 4.0], [5.0, 8.0, 7.0]], [[3.0, 7.0, 3.0], [1.0, 1.0, 6.0], [1.0, 
6.0, 1.0]], [[1.0, 8.0, 1.0], [1.0, 4.0, 6.0], [6.0, 4.0, 2.0]]], [[[8.0, 4.0, 5.0], [1.0, 4.0, 2.0], [4.0, 5.0, 2.0]], [[8.0, 
3.0, 1.0], [5.0, 7.0, 7.0], [5.0, 5.0, 7.0]], [[8.0, 3.0, 2.0], [3.0, 4.0, 3.0], [7.0, 4.0, 3.0]]], [[[4.0, 2.0, 2.0], [8.0, 
7.0, 2.0], [7.0, 4.0, 3.0]], [[4.0, 3.0, 5.0], [6.0, 5.0, 3.0], [8.0, 5.0, 6.0]], [[7.0, 7.0, 3.0], [7.0, 1.0, 6.0], [4.0, 2.0, 
2.0]]], [[[4.0, 4.0, 4.0], [2.0, 8.0, 2.0], [5.0, 7.0, 2.0]], [[6.0, 6.0, 4.0], [5.0, 7.0, 1.0], [6.0, 7.0, 7.0]], [[8.0, 7.0, 
2.0], [2.0, 1.0, 5.0], [2.0, 6.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 4.0], [7.0, 1.0, 8.0], [3.0, 2.0, 6.0]], 
[[8.0, 8.0, 3.0], [6.0, 5.0, 7.0], [1.0, 6.0, 1.0]], [[2.0, 3.0, 7.0], [4.0, 3.0, 1.0], [4.0, 3.0, 3.0]]], [[[1.0, 2.0, 6.0], 
[5.0, 1.0, 4.0], [8.0, 1.0, 5.0]], [[1.0, 7.0, 2.0], [3.0, 6.0, 3.0], [5.0, 5.0, 7.0]], [[8.0, 2.0, 7.0], [6.0, 2.0, 2.0], 
[5.0, 4.0, 7.0]]], [[[4.0, 2.0, 1.0], [5.0, 1.0, 6.0], [8.0, 7.0, 2.0]], [[5.0, 4.0, 4.0], [2.0, 2.0, 2.0], [8.0, 5.0, 6.0]], 
[[2.0, 2.0, 3.0], [5.0, 1.0, 7.0], [1.0, 7.0, 4.0]]], [[[2.0, 5.0, 8.0], [2.0, 3.0, 4.0], [6.0, 4.0, 6.0]], [[5.0, 4.0, 4.0], 
[8.0, 2.0, 3.0], [6.0, 7.0, 7.0]], [[7.0, 8.0, 7.0], [3.0, 8.0, 8.0], [1.0, 5.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 1.0], [1.0, 8.0, 8.0], [6.0, 5.0, 8.0]], [[5.0, 6.0, 7.0], [8.0, 7.0, 7.0], [8.0, 
1.0, 4.0]], [[8.0, 5.0, 3.0], [2.0, 3.0, 1.0], [5.0, 3.0, 6.0]]], [[[3.0, 8.0, 6.0], [4.0, 2.0, 2.0], [4.0, 3.0, 1.0]], [[7.0, 
6.0, 3.0], [8.0, 5.0, 1.0], [6.0, 8.0, 7.0]], [[1.0, 6.0, 6.0], [6.0, 2.0, 4.0], [8.0, 8.0, 6.0]]], [[[2.0, 3.0, 1.0], [8.0, 
8.0, 5.0], [6.0, 6.0, 8.0]], [[6.0, 4.0, 5.0], [1.0, 1.0, 5.0], [8.0, 2.0, 4.0]], [[4.0, 4.0, 6.0], [4.0, 2.0, 3.0], [4.0, 8.0, 
5.0]]], [[[3.0, 6.0, 6.0], [8.0, 3.0, 2.0], [2.0, 4.0, 1.0]], [[1.0, 3.0, 3.0], [2.0, 5.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, 2.0, 
1.0], [5.0, 6.0, 4.0], [2.0, 8.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 4.0, 5.0], [4.0, 5.0, 4.0], [7.0, 4.0, 7.0]], 
[[1.0, 5.0, 7.0], [5.0, 7.0, 7.0], [3.0, 4.0, 6.0]], [[8.0, 6.0, 1.0], [1.0, 5.0, 6.0], [3.0, 6.0, 8.0]]], [[[2.0, 7.0, 5.0], 
[2.0, 3.0, 8.0], [3.0, 3.0, 4.0]], [[5.0, 5.0, 5.0], [8.0, 1.0, 4.0], [5.0, 4.0, 2.0]], [[5.0, 7.0, 7.0], [8.0, 3.0, 4.0], 
[5.0, 7.0, 4.0]]], [[[2.0, 1.0, 8.0], [3.0, 8.0, 6.0], [4.0, 2.0, 4.0]], [[6.0, 1.0, 6.0], [8.0, 1.0, 8.0], [2.0, 3.0, 2.0]], 
[[5.0, 7.0, 8.0], [4.0, 4.0, 2.0], [8.0, 4.0, 2.0]]], [[[5.0, 7.0, 6.0], [1.0, 5.0, 5.0], [7.0, 3.0, 7.0]], [[1.0, 5.0, 7.0], 
[6.0, 8.0, 7.0], [4.0, 7.0, 3.0]], [[7.0, 6.0, 6.0], [4.0, 4.0, 2.0], [5.0, 4.0, 5.0]]]])
      val=Data(numpy.array([[[7.0, 7.0, 2.0]], [[3.0, 2.0, 1.0]], [[1.0, 4.0, 3.0]], [[3.0, 7.0, 3.0]]]),self.functionspace)
      arg[:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 1.0], [1.0, 8.0, 8.0], [6.0, 5.0, 8.0]], [[5.0, 6.0, 7.0], [8.0, 7.0, 7.0], [7.0, 
7.0, 2.0]], [[8.0, 5.0, 3.0], [2.0, 3.0, 1.0], [5.0, 3.0, 6.0]]], [[[3.0, 8.0, 6.0], [4.0, 2.0, 2.0], [4.0, 3.0, 1.0]], [[7.0, 
6.0, 3.0], [8.0, 5.0, 1.0], [3.0, 2.0, 1.0]], [[1.0, 6.0, 6.0], [6.0, 2.0, 4.0], [8.0, 8.0, 6.0]]], [[[2.0, 3.0, 1.0], [8.0, 
8.0, 5.0], [6.0, 6.0, 8.0]], [[6.0, 4.0, 5.0], [1.0, 1.0, 5.0], [1.0, 4.0, 3.0]], [[4.0, 4.0, 6.0], [4.0, 2.0, 3.0], [4.0, 8.0, 
5.0]]], [[[3.0, 6.0, 6.0], [8.0, 3.0, 2.0], [2.0, 4.0, 1.0]], [[1.0, 3.0, 3.0], [2.0, 5.0, 6.0], [3.0, 7.0, 3.0]], [[1.0, 2.0, 
1.0], [5.0, 6.0, 4.0], [2.0, 8.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 4.0, 5.0], [4.0, 5.0, 4.0], [7.0, 4.0, 7.0]], 
[[1.0, 5.0, 7.0], [5.0, 7.0, 7.0], [7.0, 7.0, 2.0]], [[8.0, 6.0, 1.0], [1.0, 5.0, 6.0], [3.0, 6.0, 8.0]]], [[[2.0, 7.0, 5.0], 
[2.0, 3.0, 8.0], [3.0, 3.0, 4.0]], [[5.0, 5.0, 5.0], [8.0, 1.0, 4.0], [3.0, 2.0, 1.0]], [[5.0, 7.0, 7.0], [8.0, 3.0, 4.0], 
[5.0, 7.0, 4.0]]], [[[2.0, 1.0, 8.0], [3.0, 8.0, 6.0], [4.0, 2.0, 4.0]], [[6.0, 1.0, 6.0], [8.0, 1.0, 8.0], [1.0, 4.0, 3.0]], 
[[5.0, 7.0, 8.0], [4.0, 4.0, 2.0], [8.0, 4.0, 2.0]]], [[[5.0, 7.0, 6.0], [1.0, 5.0, 5.0], [7.0, 3.0, 7.0]], [[1.0, 5.0, 7.0], 
[6.0, 8.0, 7.0], [3.0, 7.0, 3.0]], [[7.0, 6.0, 6.0], [4.0, 4.0, 2.0], [5.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 2.0, 7.0], [1.0, 2.0, 7.0], [6.0, 6.0, 2.0]], [[2.0, 3.0, 7.0], [6.0, 1.0, 1.0], [3.0, 
4.0, 5.0]], [[3.0, 2.0, 2.0], [7.0, 8.0, 1.0], [3.0, 8.0, 4.0]]], [[[2.0, 2.0, 1.0], [6.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[1.0, 
6.0, 3.0], [4.0, 2.0, 3.0], [6.0, 7.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 1.0, 8.0], [2.0, 3.0, 5.0]]], [[[8.0, 2.0, 5.0], [3.0, 
6.0, 1.0], [5.0, 5.0, 2.0]], [[7.0, 4.0, 4.0], [8.0, 1.0, 4.0], [5.0, 8.0, 6.0]], [[7.0, 5.0, 8.0], [7.0, 4.0, 7.0], [3.0, 8.0, 
8.0]]], [[[6.0, 2.0, 1.0], [2.0, 6.0, 4.0], [4.0, 3.0, 5.0]], [[4.0, 7.0, 3.0], [1.0, 8.0, 2.0], [7.0, 2.0, 5.0]], [[4.0, 6.0, 
8.0], [6.0, 7.0, 5.0], [6.0, 3.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 8.0, 3.0], [2.0, 3.0, 2.0], [1.0, 4.0, 4.0]], 
[[3.0, 7.0, 3.0], [3.0, 4.0, 7.0], [2.0, 5.0, 1.0]], [[3.0, 3.0, 8.0], [6.0, 3.0, 5.0], [6.0, 2.0, 8.0]]], [[[6.0, 3.0, 8.0], 
[5.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[7.0, 8.0, 7.0], [5.0, 2.0, 3.0], [4.0, 1.0, 8.0]], [[6.0, 3.0, 5.0], [7.0, 7.0, 3.0], 
[7.0, 4.0, 1.0]]], [[[8.0, 8.0, 4.0], [5.0, 2.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 7.0, 8.0], [3.0, 2.0, 3.0], [5.0, 8.0, 7.0]], 
[[4.0, 5.0, 1.0], [4.0, 7.0, 1.0], [6.0, 2.0, 1.0]]], [[[8.0, 8.0, 1.0], [8.0, 1.0, 3.0], [3.0, 6.0, 5.0]], [[8.0, 6.0, 5.0], 
[2.0, 1.0, 3.0], [7.0, 7.0, 4.0]], [[2.0, 7.0, 3.0], [5.0, 5.0, 6.0], [3.0, 1.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[8.0, 7.0, 6.0]], [[2.0, 7.0, 4.0]], [[6.0, 6.0, 5.0]], [[8.0, 8.0, 
6.0]]])+(1.-msk_val)*numpy.array([[[8.0, 7.0, 7.0]], [[5.0, 5.0, 1.0]], [[8.0, 6.0, 8.0]], [[6.0, 1.0, 7.0]]])
      arg[:,1,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 2.0, 7.0], [1.0, 2.0, 7.0], [6.0, 6.0, 2.0]], [[2.0, 3.0, 7.0], [6.0, 1.0, 1.0], [8.0, 
7.0, 6.0]], [[3.0, 2.0, 2.0], [7.0, 8.0, 1.0], [3.0, 8.0, 4.0]]], [[[2.0, 2.0, 1.0], [6.0, 7.0, 2.0], [5.0, 8.0, 1.0]], [[1.0, 
6.0, 3.0], [4.0, 2.0, 3.0], [2.0, 7.0, 4.0]], [[6.0, 4.0, 2.0], [3.0, 1.0, 8.0], [2.0, 3.0, 5.0]]], [[[8.0, 2.0, 5.0], [3.0, 
6.0, 1.0], [5.0, 5.0, 2.0]], [[7.0, 4.0, 4.0], [8.0, 1.0, 4.0], [6.0, 6.0, 5.0]], [[7.0, 5.0, 8.0], [7.0, 4.0, 7.0], [3.0, 8.0, 
8.0]]], [[[6.0, 2.0, 1.0], [2.0, 6.0, 4.0], [4.0, 3.0, 5.0]], [[4.0, 7.0, 3.0], [1.0, 8.0, 2.0], [8.0, 8.0, 6.0]], [[4.0, 6.0, 
8.0], [6.0, 7.0, 5.0], [6.0, 3.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 8.0, 3.0], [2.0, 3.0, 2.0], [1.0, 4.0, 4.0]], 
[[3.0, 7.0, 3.0], [3.0, 4.0, 7.0], [8.0, 7.0, 7.0]], [[3.0, 3.0, 8.0], [6.0, 3.0, 5.0], [6.0, 2.0, 8.0]]], [[[6.0, 3.0, 8.0], 
[5.0, 6.0, 8.0], [5.0, 8.0, 4.0]], [[7.0, 8.0, 7.0], [5.0, 2.0, 3.0], [5.0, 5.0, 1.0]], [[6.0, 3.0, 5.0], [7.0, 7.0, 3.0], 
[7.0, 4.0, 1.0]]], [[[8.0, 8.0, 4.0], [5.0, 2.0, 4.0], [2.0, 4.0, 4.0]], [[8.0, 7.0, 8.0], [3.0, 2.0, 3.0], [8.0, 6.0, 8.0]], 
[[4.0, 5.0, 1.0], [4.0, 7.0, 1.0], [6.0, 2.0, 1.0]]], [[[8.0, 8.0, 1.0], [8.0, 1.0, 3.0], [3.0, 6.0, 5.0]], [[8.0, 6.0, 5.0], 
[2.0, 1.0, 3.0], [6.0, 1.0, 7.0]], [[2.0, 7.0, 3.0], [5.0, 5.0, 6.0], [3.0, 1.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 5.0, 5.0], [4.0, 6.0, 8.0]], [[5.0, 1.0, 2.0], [5.0, 8.0, 4.0], [6.0, 
3.0, 5.0]], [[6.0, 6.0, 1.0], [6.0, 2.0, 4.0], [7.0, 1.0, 7.0]]], [[[1.0, 6.0, 2.0], [4.0, 8.0, 6.0], [6.0, 7.0, 7.0]], [[3.0, 
3.0, 1.0], [5.0, 4.0, 3.0], [1.0, 7.0, 8.0]], [[8.0, 5.0, 8.0], [7.0, 4.0, 5.0], [4.0, 1.0, 5.0]]], [[[8.0, 4.0, 4.0], [5.0, 
4.0, 8.0], [4.0, 7.0, 8.0]], [[6.0, 4.0, 5.0], [4.0, 7.0, 3.0], [6.0, 7.0, 1.0]], [[7.0, 3.0, 3.0], [7.0, 8.0, 1.0], [4.0, 3.0, 
1.0]]], [[[4.0, 1.0, 7.0], [6.0, 6.0, 3.0], [1.0, 2.0, 2.0]], [[7.0, 7.0, 7.0], [6.0, 3.0, 4.0], [5.0, 7.0, 6.0]], [[2.0, 8.0, 
3.0], [3.0, 7.0, 6.0], [3.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 2.0], [8.0, 1.0, 6.0], [1.0, 3.0, 5.0]], 
[[8.0, 5.0, 7.0], [7.0, 2.0, 1.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 2.0], [8.0, 2.0, 3.0], [4.0, 2.0, 2.0]]], [[[5.0, 2.0, 2.0], 
[1.0, 2.0, 8.0], [3.0, 3.0, 2.0]], [[4.0, 3.0, 6.0], [3.0, 8.0, 2.0], [2.0, 7.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 4.0, 4.0], 
[2.0, 8.0, 5.0]]], [[[7.0, 2.0, 7.0], [8.0, 6.0, 3.0], [6.0, 6.0, 1.0]], [[7.0, 5.0, 7.0], [7.0, 3.0, 2.0], [1.0, 2.0, 5.0]], 
[[6.0, 5.0, 7.0], [3.0, 1.0, 6.0], [4.0, 2.0, 2.0]]], [[[5.0, 8.0, 3.0], [5.0, 2.0, 5.0], [2.0, 6.0, 3.0]], [[1.0, 5.0, 1.0], 
[5.0, 1.0, 1.0], [4.0, 6.0, 4.0]], [[7.0, 2.0, 5.0], [7.0, 5.0, 4.0], [4.0, 4.0, 7.0]]]])
      val=numpy.array([[[6.0, 3.0, 5.0]], [[2.0, 4.0, 6.0]], [[5.0, 1.0, 2.0]]])
      arg[2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 5.0, 5.0], [4.0, 6.0, 8.0]], [[5.0, 1.0, 2.0], [5.0, 8.0, 4.0], [6.0, 
3.0, 5.0]], [[6.0, 6.0, 1.0], [6.0, 2.0, 4.0], [7.0, 1.0, 7.0]]], [[[1.0, 6.0, 2.0], [4.0, 8.0, 6.0], [6.0, 7.0, 7.0]], [[3.0, 
3.0, 1.0], [5.0, 4.0, 3.0], [1.0, 7.0, 8.0]], [[8.0, 5.0, 8.0], [7.0, 4.0, 5.0], [4.0, 1.0, 5.0]]], [[[8.0, 4.0, 4.0], [6.0, 
3.0, 5.0], [4.0, 7.0, 8.0]], [[6.0, 4.0, 5.0], [2.0, 4.0, 6.0], [6.0, 7.0, 1.0]], [[7.0, 3.0, 3.0], [5.0, 1.0, 2.0], [4.0, 3.0, 
1.0]]], [[[4.0, 1.0, 7.0], [6.0, 6.0, 3.0], [1.0, 2.0, 2.0]], [[7.0, 7.0, 7.0], [6.0, 3.0, 4.0], [5.0, 7.0, 6.0]], [[2.0, 8.0, 
3.0], [3.0, 7.0, 6.0], [3.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 2.0], [8.0, 1.0, 6.0], [1.0, 3.0, 5.0]], 
[[8.0, 5.0, 7.0], [7.0, 2.0, 1.0], [4.0, 2.0, 6.0]], [[4.0, 3.0, 2.0], [8.0, 2.0, 3.0], [4.0, 2.0, 2.0]]], [[[5.0, 2.0, 2.0], 
[1.0, 2.0, 8.0], [3.0, 3.0, 2.0]], [[4.0, 3.0, 6.0], [3.0, 8.0, 2.0], [2.0, 7.0, 1.0]], [[7.0, 1.0, 4.0], [4.0, 4.0, 4.0], 
[2.0, 8.0, 5.0]]], [[[7.0, 2.0, 7.0], [6.0, 3.0, 5.0], [6.0, 6.0, 1.0]], [[7.0, 5.0, 7.0], [2.0, 4.0, 6.0], [1.0, 2.0, 5.0]], 
[[6.0, 5.0, 7.0], [5.0, 1.0, 2.0], [4.0, 2.0, 2.0]]], [[[5.0, 8.0, 3.0], [5.0, 2.0, 5.0], [2.0, 6.0, 3.0]], [[1.0, 5.0, 1.0], 
[5.0, 1.0, 1.0], [4.0, 6.0, 4.0]], [[7.0, 2.0, 5.0], [7.0, 5.0, 4.0], [4.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 7.0, 2.0], [3.0, 6.0, 6.0], [1.0, 4.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 7.0, 4.0], [6.0, 
6.0, 7.0]], [[8.0, 7.0, 1.0], [5.0, 7.0, 5.0], [3.0, 2.0, 2.0]]], [[[8.0, 7.0, 2.0], [8.0, 4.0, 4.0], [4.0, 8.0, 3.0]], [[7.0, 
2.0, 3.0], [2.0, 4.0, 2.0], [3.0, 6.0, 5.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [2.0, 7.0, 2.0]]], [[[3.0, 1.0, 6.0], [7.0, 
1.0, 3.0], [6.0, 5.0, 2.0]], [[7.0, 5.0, 6.0], [3.0, 7.0, 8.0], [6.0, 1.0, 7.0]], [[7.0, 7.0, 3.0], [5.0, 7.0, 6.0], [1.0, 4.0, 
4.0]]], [[[6.0, 6.0, 3.0], [1.0, 4.0, 2.0], [8.0, 6.0, 2.0]], [[1.0, 8.0, 5.0], [1.0, 4.0, 4.0], [5.0, 5.0, 7.0]], [[2.0, 6.0, 
5.0], [3.0, 8.0, 2.0], [4.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 4.0], [6.0, 7.0, 6.0], [8.0, 2.0, 3.0]], 
[[5.0, 7.0, 3.0], [8.0, 3.0, 3.0], [2.0, 4.0, 4.0]], [[8.0, 8.0, 1.0], [7.0, 8.0, 1.0], [6.0, 4.0, 4.0]]], [[[5.0, 1.0, 3.0], 
[7.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[4.0, 2.0, 3.0], [3.0, 3.0, 2.0], [6.0, 3.0, 5.0]], [[8.0, 4.0, 1.0], [3.0, 2.0, 2.0], 
[5.0, 2.0, 1.0]]], [[[4.0, 6.0, 2.0], [4.0, 6.0, 3.0], [7.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [8.0, 6.0, 3.0], [6.0, 4.0, 2.0]], 
[[7.0, 5.0, 2.0], [7.0, 3.0, 7.0], [1.0, 6.0, 4.0]]], [[[5.0, 7.0, 4.0], [6.0, 3.0, 7.0], [1.0, 7.0, 3.0]], [[7.0, 7.0, 5.0], 
[4.0, 8.0, 4.0], [5.0, 8.0, 4.0]], [[1.0, 4.0, 7.0], [4.0, 6.0, 2.0], [5.0, 8.0, 1.0]]]])
      val=Data(numpy.array([[[1.0, 4.0, 6.0]], [[4.0, 6.0, 3.0]], [[8.0, 3.0, 1.0]]]),self.functionspace)
      arg[2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 7.0, 2.0], [3.0, 6.0, 6.0], [1.0, 4.0, 1.0]], [[4.0, 1.0, 1.0], [1.0, 7.0, 4.0], [6.0, 
6.0, 7.0]], [[8.0, 7.0, 1.0], [5.0, 7.0, 5.0], [3.0, 2.0, 2.0]]], [[[8.0, 7.0, 2.0], [8.0, 4.0, 4.0], [4.0, 8.0, 3.0]], [[7.0, 
2.0, 3.0], [2.0, 4.0, 2.0], [3.0, 6.0, 5.0]], [[3.0, 3.0, 7.0], [4.0, 2.0, 6.0], [2.0, 7.0, 2.0]]], [[[3.0, 1.0, 6.0], [1.0, 
4.0, 6.0], [6.0, 5.0, 2.0]], [[7.0, 5.0, 6.0], [4.0, 6.0, 3.0], [6.0, 1.0, 7.0]], [[7.0, 7.0, 3.0], [8.0, 3.0, 1.0], [1.0, 4.0, 
4.0]]], [[[6.0, 6.0, 3.0], [1.0, 4.0, 2.0], [8.0, 6.0, 2.0]], [[1.0, 8.0, 5.0], [1.0, 4.0, 4.0], [5.0, 5.0, 7.0]], [[2.0, 6.0, 
5.0], [3.0, 8.0, 2.0], [4.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 4.0], [6.0, 7.0, 6.0], [8.0, 2.0, 3.0]], 
[[5.0, 7.0, 3.0], [8.0, 3.0, 3.0], [2.0, 4.0, 4.0]], [[8.0, 8.0, 1.0], [7.0, 8.0, 1.0], [6.0, 4.0, 4.0]]], [[[5.0, 1.0, 3.0], 
[7.0, 8.0, 8.0], [1.0, 4.0, 3.0]], [[4.0, 2.0, 3.0], [3.0, 3.0, 2.0], [6.0, 3.0, 5.0]], [[8.0, 4.0, 1.0], [3.0, 2.0, 2.0], 
[5.0, 2.0, 1.0]]], [[[4.0, 6.0, 2.0], [1.0, 4.0, 6.0], [7.0, 1.0, 7.0]], [[1.0, 8.0, 1.0], [4.0, 6.0, 3.0], [6.0, 4.0, 2.0]], 
[[7.0, 5.0, 2.0], [8.0, 3.0, 1.0], [1.0, 6.0, 4.0]]], [[[5.0, 7.0, 4.0], [6.0, 3.0, 7.0], [1.0, 7.0, 3.0]], [[7.0, 7.0, 5.0], 
[4.0, 8.0, 4.0], [5.0, 8.0, 4.0]], [[1.0, 4.0, 7.0], [4.0, 6.0, 2.0], [5.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 7.0, 3.0], [7.0, 8.0, 7.0], [2.0, 6.0, 2.0]], [[8.0, 3.0, 3.0], [2.0, 1.0, 2.0], [6.0, 
1.0, 1.0]], [[2.0, 6.0, 3.0], [4.0, 2.0, 7.0], [8.0, 2.0, 7.0]]], [[[4.0, 5.0, 7.0], [4.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[8.0, 
7.0, 4.0], [3.0, 6.0, 1.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 1.0], [5.0, 7.0, 3.0], [2.0, 4.0, 6.0]]], [[[3.0, 8.0, 3.0], [2.0, 
6.0, 6.0], [2.0, 6.0, 2.0]], [[3.0, 8.0, 5.0], [4.0, 2.0, 7.0], [4.0, 4.0, 4.0]], [[3.0, 2.0, 7.0], [8.0, 7.0, 3.0], [8.0, 1.0, 
2.0]]], [[[7.0, 2.0, 5.0], [6.0, 5.0, 5.0], [8.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [7.0, 2.0, 2.0], [7.0, 3.0, 4.0]], [[4.0, 8.0, 
5.0], [1.0, 8.0, 1.0], [8.0, 4.0, 6.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 6.0, 2.0], [5.0, 4.0, 3.0]], 
[[2.0, 4.0, 1.0], [7.0, 6.0, 7.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 8.0], [2.0, 5.0, 5.0], [2.0, 3.0, 1.0]]], [[[1.0, 2.0, 3.0], 
[1.0, 7.0, 1.0], [5.0, 2.0, 5.0]], [[6.0, 5.0, 5.0], [6.0, 4.0, 2.0], [8.0, 8.0, 5.0]], [[2.0, 7.0, 8.0], [8.0, 2.0, 7.0], 
[6.0, 7.0, 2.0]]], [[[3.0, 8.0, 7.0], [7.0, 7.0, 6.0], [7.0, 3.0, 8.0]], [[2.0, 2.0, 5.0], [4.0, 8.0, 1.0], [5.0, 3.0, 4.0]], 
[[4.0, 1.0, 1.0], [1.0, 5.0, 8.0], [7.0, 3.0, 2.0]]], [[[2.0, 1.0, 1.0], [4.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[5.0, 8.0, 8.0], 
[1.0, 7.0, 4.0], [6.0, 3.0, 1.0]], [[3.0, 5.0, 8.0], [6.0, 7.0, 1.0], [1.0, 4.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[2.0, 8.0, 2.0]], [[1.0, 5.0, 6.0]], [[2.0, 8.0, 3.0]]])+(1.-msk_val)*numpy.array([[[6.0, 7.0, 
8.0]], [[4.0, 8.0, 3.0]], [[7.0, 8.0, 5.0]]])
      arg[2,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 3.0], [7.0, 8.0, 7.0], [2.0, 6.0, 2.0]], [[8.0, 3.0, 3.0], [2.0, 1.0, 2.0], [6.0, 
1.0, 1.0]], [[2.0, 6.0, 3.0], [4.0, 2.0, 7.0], [8.0, 2.0, 7.0]]], [[[4.0, 5.0, 7.0], [4.0, 6.0, 3.0], [3.0, 5.0, 7.0]], [[8.0, 
7.0, 4.0], [3.0, 6.0, 1.0], [2.0, 5.0, 5.0]], [[8.0, 5.0, 1.0], [5.0, 7.0, 3.0], [2.0, 4.0, 6.0]]], [[[3.0, 8.0, 3.0], [2.0, 
8.0, 2.0], [2.0, 6.0, 2.0]], [[3.0, 8.0, 5.0], [1.0, 5.0, 6.0], [4.0, 4.0, 4.0]], [[3.0, 2.0, 7.0], [2.0, 8.0, 3.0], [8.0, 1.0, 
2.0]]], [[[7.0, 2.0, 5.0], [6.0, 5.0, 5.0], [8.0, 3.0, 4.0]], [[7.0, 4.0, 6.0], [7.0, 2.0, 2.0], [7.0, 3.0, 4.0]], [[4.0, 8.0, 
5.0], [1.0, 8.0, 1.0], [8.0, 4.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 1.0], [7.0, 6.0, 2.0], [5.0, 4.0, 3.0]], 
[[2.0, 4.0, 1.0], [7.0, 6.0, 7.0], [4.0, 3.0, 5.0]], [[3.0, 4.0, 8.0], [2.0, 5.0, 5.0], [2.0, 3.0, 1.0]]], [[[1.0, 2.0, 3.0], 
[1.0, 7.0, 1.0], [5.0, 2.0, 5.0]], [[6.0, 5.0, 5.0], [6.0, 4.0, 2.0], [8.0, 8.0, 5.0]], [[2.0, 7.0, 8.0], [8.0, 2.0, 7.0], 
[6.0, 7.0, 2.0]]], [[[3.0, 8.0, 7.0], [6.0, 7.0, 8.0], [7.0, 3.0, 8.0]], [[2.0, 2.0, 5.0], [4.0, 8.0, 3.0], [5.0, 3.0, 4.0]], 
[[4.0, 1.0, 1.0], [7.0, 8.0, 5.0], [7.0, 3.0, 2.0]]], [[[2.0, 1.0, 1.0], [4.0, 4.0, 3.0], [7.0, 6.0, 5.0]], [[5.0, 8.0, 8.0], 
[1.0, 7.0, 4.0], [6.0, 3.0, 1.0]], [[3.0, 5.0, 8.0], [6.0, 7.0, 1.0], [1.0, 4.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 3.0], [6.0, 1.0, 6.0], [6.0, 6.0, 5.0]], [[4.0, 2.0, 6.0], [5.0, 2.0, 3.0], [8.0, 
4.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 7.0, 2.0], [7.0, 3.0, 3.0]]], [[[5.0, 1.0, 7.0], [7.0, 4.0, 2.0], [1.0, 7.0, 1.0]], [[6.0, 
4.0, 4.0], [2.0, 1.0, 2.0], [4.0, 4.0, 2.0]], [[2.0, 5.0, 7.0], [7.0, 6.0, 3.0], [6.0, 6.0, 3.0]]], [[[3.0, 8.0, 8.0], [7.0, 
3.0, 6.0], [6.0, 5.0, 1.0]], [[1.0, 5.0, 5.0], [7.0, 5.0, 4.0], [4.0, 2.0, 1.0]], [[4.0, 2.0, 5.0], [8.0, 7.0, 1.0], [1.0, 2.0, 
7.0]]], [[[7.0, 8.0, 3.0], [5.0, 2.0, 3.0], [4.0, 6.0, 1.0]], [[5.0, 1.0, 3.0], [8.0, 3.0, 8.0], [3.0, 1.0, 8.0]], [[6.0, 2.0, 
4.0], [8.0, 5.0, 2.0], [3.0, 1.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 7.0, 4.0], [3.0, 8.0, 8.0]], 
[[5.0, 8.0, 3.0], [6.0, 4.0, 7.0], [3.0, 3.0, 8.0]], [[4.0, 4.0, 6.0], [6.0, 4.0, 1.0], [7.0, 2.0, 1.0]]], [[[1.0, 7.0, 6.0], 
[1.0, 3.0, 7.0], [1.0, 1.0, 6.0]], [[3.0, 5.0, 5.0], [4.0, 2.0, 5.0], [6.0, 5.0, 2.0]], [[3.0, 1.0, 8.0], [2.0, 4.0, 7.0], 
[2.0, 8.0, 1.0]]], [[[3.0, 2.0, 8.0], [4.0, 8.0, 8.0], [1.0, 8.0, 1.0]], [[1.0, 7.0, 5.0], [8.0, 7.0, 7.0], [4.0, 1.0, 7.0]], 
[[1.0, 5.0, 3.0], [1.0, 4.0, 4.0], [5.0, 7.0, 6.0]]], [[[1.0, 6.0, 3.0], [4.0, 3.0, 7.0], [7.0, 5.0, 2.0]], [[5.0, 8.0, 7.0], 
[6.0, 6.0, 4.0], [2.0, 3.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 7.0, 8.0], [1.0, 1.0, 2.0]]]])
      val=numpy.array([[[[7.0, 1.0, 4.0]]], [[[1.0, 4.0, 4.0]]], [[[1.0, 2.0, 2.0]]], [[[6.0, 2.0, 5.0]]]])
      arg[:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 3.0], [6.0, 1.0, 6.0], [6.0, 6.0, 5.0]], [[4.0, 2.0, 6.0], [5.0, 2.0, 3.0], [8.0, 
4.0, 7.0]], [[3.0, 4.0, 7.0], [3.0, 7.0, 2.0], [7.0, 1.0, 4.0]]], [[[5.0, 1.0, 7.0], [7.0, 4.0, 2.0], [1.0, 7.0, 1.0]], [[6.0, 
4.0, 4.0], [2.0, 1.0, 2.0], [4.0, 4.0, 2.0]], [[2.0, 5.0, 7.0], [7.0, 6.0, 3.0], [1.0, 4.0, 4.0]]], [[[3.0, 8.0, 8.0], [7.0, 
3.0, 6.0], [6.0, 5.0, 1.0]], [[1.0, 5.0, 5.0], [7.0, 5.0, 4.0], [4.0, 2.0, 1.0]], [[4.0, 2.0, 5.0], [8.0, 7.0, 1.0], [1.0, 2.0, 
2.0]]], [[[7.0, 8.0, 3.0], [5.0, 2.0, 3.0], [4.0, 6.0, 1.0]], [[5.0, 1.0, 3.0], [8.0, 3.0, 8.0], [3.0, 1.0, 8.0]], [[6.0, 2.0, 
4.0], [8.0, 5.0, 2.0], [6.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 4.0], [4.0, 7.0, 4.0], [3.0, 8.0, 8.0]], 
[[5.0, 8.0, 3.0], [6.0, 4.0, 7.0], [3.0, 3.0, 8.0]], [[4.0, 4.0, 6.0], [6.0, 4.0, 1.0], [7.0, 1.0, 4.0]]], [[[1.0, 7.0, 6.0], 
[1.0, 3.0, 7.0], [1.0, 1.0, 6.0]], [[3.0, 5.0, 5.0], [4.0, 2.0, 5.0], [6.0, 5.0, 2.0]], [[3.0, 1.0, 8.0], [2.0, 4.0, 7.0], 
[1.0, 4.0, 4.0]]], [[[3.0, 2.0, 8.0], [4.0, 8.0, 8.0], [1.0, 8.0, 1.0]], [[1.0, 7.0, 5.0], [8.0, 7.0, 7.0], [4.0, 1.0, 7.0]], 
[[1.0, 5.0, 3.0], [1.0, 4.0, 4.0], [1.0, 2.0, 2.0]]], [[[1.0, 6.0, 3.0], [4.0, 3.0, 7.0], [7.0, 5.0, 2.0]], [[5.0, 8.0, 7.0], 
[6.0, 6.0, 4.0], [2.0, 3.0, 7.0]], [[1.0, 4.0, 8.0], [8.0, 7.0, 8.0], [6.0, 2.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 2.0], [2.0, 2.0, 2.0], [1.0, 8.0, 1.0]], [[3.0, 6.0, 2.0], [6.0, 4.0, 2.0], [4.0, 
3.0, 1.0]], [[1.0, 7.0, 8.0], [1.0, 3.0, 4.0], [3.0, 5.0, 7.0]]], [[[3.0, 1.0, 7.0], [6.0, 7.0, 3.0], [6.0, 8.0, 8.0]], [[6.0, 
7.0, 1.0], [5.0, 3.0, 8.0], [5.0, 2.0, 4.0]], [[5.0, 4.0, 3.0], [1.0, 7.0, 8.0], [8.0, 1.0, 8.0]]], [[[5.0, 1.0, 1.0], [1.0, 
6.0, 4.0], [6.0, 5.0, 8.0]], [[8.0, 5.0, 3.0], [8.0, 4.0, 8.0], [7.0, 6.0, 2.0]], [[7.0, 4.0, 7.0], [2.0, 6.0, 5.0], [3.0, 1.0, 
7.0]]], [[[1.0, 7.0, 8.0], [7.0, 2.0, 8.0], [7.0, 6.0, 4.0]], [[5.0, 5.0, 7.0], [4.0, 2.0, 7.0], [3.0, 2.0, 3.0]], [[1.0, 5.0, 
6.0], [4.0, 2.0, 3.0], [1.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 3.0, 7.0], [7.0, 2.0, 2.0], [8.0, 5.0, 4.0]], 
[[6.0, 4.0, 7.0], [2.0, 3.0, 1.0], [1.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [6.0, 3.0, 8.0], [3.0, 3.0, 6.0]]], [[[4.0, 6.0, 3.0], 
[7.0, 3.0, 5.0], [4.0, 5.0, 4.0]], [[7.0, 2.0, 1.0], [5.0, 3.0, 6.0], [1.0, 4.0, 7.0]], [[7.0, 4.0, 4.0], [2.0, 3.0, 7.0], 
[2.0, 1.0, 4.0]]], [[[5.0, 5.0, 4.0], [7.0, 4.0, 1.0], [8.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], [3.0, 6.0, 4.0], [4.0, 4.0, 1.0]], 
[[6.0, 1.0, 3.0], [3.0, 3.0, 3.0], [2.0, 1.0, 8.0]]], [[[5.0, 4.0, 1.0], [5.0, 2.0, 1.0], [3.0, 2.0, 5.0]], [[7.0, 6.0, 1.0], 
[8.0, 7.0, 5.0], [1.0, 3.0, 8.0]], [[5.0, 1.0, 3.0], [2.0, 7.0, 6.0], [8.0, 7.0, 4.0]]]])
      val=Data(numpy.array([[[[8.0, 3.0, 7.0]]], [[[2.0, 1.0, 5.0]]], [[[1.0, 8.0, 8.0]]], [[[7.0, 4.0, 
1.0]]]]),self.functionspace)
      arg[:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 2.0], [2.0, 2.0, 2.0], [1.0, 8.0, 1.0]], [[3.0, 6.0, 2.0], [6.0, 4.0, 2.0], [4.0, 
3.0, 1.0]], [[1.0, 7.0, 8.0], [1.0, 3.0, 4.0], [8.0, 3.0, 7.0]]], [[[3.0, 1.0, 7.0], [6.0, 7.0, 3.0], [6.0, 8.0, 8.0]], [[6.0, 
7.0, 1.0], [5.0, 3.0, 8.0], [5.0, 2.0, 4.0]], [[5.0, 4.0, 3.0], [1.0, 7.0, 8.0], [2.0, 1.0, 5.0]]], [[[5.0, 1.0, 1.0], [1.0, 
6.0, 4.0], [6.0, 5.0, 8.0]], [[8.0, 5.0, 3.0], [8.0, 4.0, 8.0], [7.0, 6.0, 2.0]], [[7.0, 4.0, 7.0], [2.0, 6.0, 5.0], [1.0, 8.0, 
8.0]]], [[[1.0, 7.0, 8.0], [7.0, 2.0, 8.0], [7.0, 6.0, 4.0]], [[5.0, 5.0, 7.0], [4.0, 2.0, 7.0], [3.0, 2.0, 3.0]], [[1.0, 5.0, 
6.0], [4.0, 2.0, 3.0], [7.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 3.0, 7.0], [7.0, 2.0, 2.0], [8.0, 5.0, 4.0]], 
[[6.0, 4.0, 7.0], [2.0, 3.0, 1.0], [1.0, 8.0, 6.0]], [[4.0, 1.0, 6.0], [6.0, 3.0, 8.0], [8.0, 3.0, 7.0]]], [[[4.0, 6.0, 3.0], 
[7.0, 3.0, 5.0], [4.0, 5.0, 4.0]], [[7.0, 2.0, 1.0], [5.0, 3.0, 6.0], [1.0, 4.0, 7.0]], [[7.0, 4.0, 4.0], [2.0, 3.0, 7.0], 
[2.0, 1.0, 5.0]]], [[[5.0, 5.0, 4.0], [7.0, 4.0, 1.0], [8.0, 4.0, 8.0]], [[5.0, 3.0, 4.0], [3.0, 6.0, 4.0], [4.0, 4.0, 1.0]], 
[[6.0, 1.0, 3.0], [3.0, 3.0, 3.0], [1.0, 8.0, 8.0]]], [[[5.0, 4.0, 1.0], [5.0, 2.0, 1.0], [3.0, 2.0, 5.0]], [[7.0, 6.0, 1.0], 
[8.0, 7.0, 5.0], [1.0, 3.0, 8.0]], [[5.0, 1.0, 3.0], [2.0, 7.0, 6.0], [7.0, 4.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 2.0, 5.0], [3.0, 5.0, 3.0], [5.0, 7.0, 2.0]], [[2.0, 2.0, 1.0], [7.0, 4.0, 7.0], [1.0, 
7.0, 6.0]], [[8.0, 7.0, 6.0], [5.0, 7.0, 5.0], [4.0, 7.0, 3.0]]], [[[5.0, 4.0, 2.0], [4.0, 3.0, 1.0], [8.0, 2.0, 6.0]], [[3.0, 
7.0, 3.0], [7.0, 3.0, 3.0], [3.0, 4.0, 3.0]], [[8.0, 4.0, 8.0], [3.0, 3.0, 8.0], [1.0, 2.0, 8.0]]], [[[1.0, 4.0, 3.0], [5.0, 
3.0, 6.0], [5.0, 5.0, 1.0]], [[2.0, 3.0, 5.0], [5.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 1.0, 3.0], [6.0, 6.0, 7.0], [7.0, 8.0, 
6.0]]], [[[6.0, 1.0, 2.0], [7.0, 8.0, 1.0], [7.0, 7.0, 5.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 3.0], [1.0, 8.0, 5.0]], [[5.0, 6.0, 
6.0], [3.0, 8.0, 7.0], [1.0, 1.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 3.0, 3.0], [5.0, 2.0, 8.0], [1.0, 6.0, 3.0]], 
[[8.0, 6.0, 5.0], [6.0, 4.0, 7.0], [7.0, 4.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 2.0, 5.0], [4.0, 5.0, 2.0]]], [[[2.0, 5.0, 5.0], 
[1.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 7.0, 6.0], [2.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 3.0], [7.0, 3.0, 4.0], 
[7.0, 8.0, 1.0]]], [[[8.0, 1.0, 8.0], [2.0, 5.0, 8.0], [1.0, 5.0, 5.0]], [[2.0, 3.0, 6.0], [8.0, 4.0, 3.0], [7.0, 7.0, 6.0]], 
[[7.0, 4.0, 5.0], [4.0, 5.0, 2.0], [7.0, 1.0, 6.0]]], [[[3.0, 4.0, 5.0], [1.0, 3.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 8.0, 7.0], 
[4.0, 5.0, 3.0], [3.0, 5.0, 5.0]], [[6.0, 7.0, 3.0], [5.0, 2.0, 3.0], [7.0, 1.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[2.0, 3.0, 3.0]]], [[[5.0, 1.0, 2.0]]], [[[4.0, 7.0, 4.0]]], [[[3.0, 1.0, 
2.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 4.0, 2.0]]], [[[4.0, 6.0, 8.0]]], [[[3.0, 3.0, 5.0]]], [[[2.0, 6.0, 3.0]]]])
      arg[:,2:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 2.0, 5.0], [3.0, 5.0, 3.0], [5.0, 7.0, 2.0]], [[2.0, 2.0, 1.0], [7.0, 4.0, 7.0], [1.0, 
7.0, 6.0]], [[8.0, 7.0, 6.0], [5.0, 7.0, 5.0], [2.0, 3.0, 3.0]]], [[[5.0, 4.0, 2.0], [4.0, 3.0, 1.0], [8.0, 2.0, 6.0]], [[3.0, 
7.0, 3.0], [7.0, 3.0, 3.0], [3.0, 4.0, 3.0]], [[8.0, 4.0, 8.0], [3.0, 3.0, 8.0], [5.0, 1.0, 2.0]]], [[[1.0, 4.0, 3.0], [5.0, 
3.0, 6.0], [5.0, 5.0, 1.0]], [[2.0, 3.0, 5.0], [5.0, 5.0, 8.0], [6.0, 4.0, 7.0]], [[6.0, 1.0, 3.0], [6.0, 6.0, 7.0], [4.0, 7.0, 
4.0]]], [[[6.0, 1.0, 2.0], [7.0, 8.0, 1.0], [7.0, 7.0, 5.0]], [[5.0, 8.0, 7.0], [2.0, 1.0, 3.0], [1.0, 8.0, 5.0]], [[5.0, 6.0, 
6.0], [3.0, 8.0, 7.0], [3.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 3.0, 3.0], [5.0, 2.0, 8.0], [1.0, 6.0, 3.0]], 
[[8.0, 6.0, 5.0], [6.0, 4.0, 7.0], [7.0, 4.0, 5.0]], [[2.0, 4.0, 8.0], [4.0, 2.0, 5.0], [2.0, 4.0, 2.0]]], [[[2.0, 5.0, 5.0], 
[1.0, 2.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 7.0, 6.0], [2.0, 4.0, 1.0], [2.0, 3.0, 4.0]], [[7.0, 8.0, 3.0], [7.0, 3.0, 4.0], 
[4.0, 6.0, 8.0]]], [[[8.0, 1.0, 8.0], [2.0, 5.0, 8.0], [1.0, 5.0, 5.0]], [[2.0, 3.0, 6.0], [8.0, 4.0, 3.0], [7.0, 7.0, 6.0]], 
[[7.0, 4.0, 5.0], [4.0, 5.0, 2.0], [3.0, 3.0, 5.0]]], [[[3.0, 4.0, 5.0], [1.0, 3.0, 3.0], [8.0, 1.0, 1.0]], [[2.0, 8.0, 7.0], 
[4.0, 5.0, 3.0], [3.0, 5.0, 5.0]], [[6.0, 7.0, 3.0], [5.0, 2.0, 3.0], [2.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 8.0, 8.0], [6.0, 8.0, 1.0], [5.0, 7.0, 8.0]], [[6.0, 4.0, 8.0], [6.0, 2.0, 4.0], [4.0, 
4.0, 8.0]], [[6.0, 7.0, 2.0], [7.0, 1.0, 2.0], [3.0, 2.0, 8.0]]], [[[1.0, 6.0, 8.0], [8.0, 6.0, 5.0], [7.0, 1.0, 8.0]], [[3.0, 
4.0, 8.0], [6.0, 7.0, 5.0], [7.0, 5.0, 7.0]], [[7.0, 5.0, 4.0], [4.0, 2.0, 5.0], [6.0, 6.0, 1.0]]], [[[7.0, 2.0, 3.0], [6.0, 
6.0, 8.0], [3.0, 8.0, 4.0]], [[8.0, 8.0, 2.0], [4.0, 6.0, 1.0], [8.0, 5.0, 3.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 6.0], [5.0, 6.0, 
3.0]]], [[[4.0, 8.0, 1.0], [1.0, 3.0, 7.0], [5.0, 7.0, 3.0]], [[3.0, 6.0, 3.0], [2.0, 6.0, 4.0], [8.0, 6.0, 8.0]], [[3.0, 7.0, 
3.0], [2.0, 3.0, 7.0], [4.0, 6.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 1.0, 5.0], [5.0, 2.0, 8.0], [5.0, 2.0, 4.0]], 
[[3.0, 6.0, 2.0], [8.0, 3.0, 4.0], [5.0, 7.0, 2.0]], [[6.0, 5.0, 7.0], [8.0, 1.0, 7.0], [6.0, 2.0, 4.0]]], [[[7.0, 8.0, 2.0], 
[8.0, 6.0, 3.0], [2.0, 2.0, 1.0]], [[6.0, 1.0, 8.0], [4.0, 7.0, 6.0], [5.0, 6.0, 3.0]], [[3.0, 8.0, 7.0], [3.0, 1.0, 1.0], 
[1.0, 8.0, 8.0]]], [[[7.0, 1.0, 7.0], [6.0, 4.0, 1.0], [7.0, 8.0, 2.0]], [[7.0, 7.0, 3.0], [8.0, 1.0, 5.0], [2.0, 7.0, 2.0]], 
[[5.0, 5.0, 5.0], [5.0, 4.0, 3.0], [8.0, 7.0, 3.0]]], [[[3.0, 5.0, 1.0], [2.0, 2.0, 7.0], [3.0, 6.0, 1.0]], [[7.0, 3.0, 8.0], 
[4.0, 7.0, 7.0], [3.0, 2.0, 4.0]], [[8.0, 4.0, 1.0], [1.0, 6.0, 5.0], [8.0, 4.0, 6.0]]]])
      val=numpy.array([[[5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [1.0, 1.0, 5.0]], [[8.0, 2.0, 7.0], [3.0, 2.0, 2.0], [6.0, 1.0, 
3.0]], [[1.0, 4.0, 8.0], [5.0, 4.0, 2.0], [8.0, 1.0, 1.0]], [[1.0, 2.0, 8.0], [2.0, 8.0, 6.0], [3.0, 6.0, 4.0]]])
      arg[:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 8.0, 8.0], [6.0, 8.0, 1.0], [5.0, 7.0, 8.0]], [[5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [1.0, 
1.0, 5.0]], [[6.0, 7.0, 2.0], [7.0, 1.0, 2.0], [3.0, 2.0, 8.0]]], [[[1.0, 6.0, 8.0], [8.0, 6.0, 5.0], [7.0, 1.0, 8.0]], [[8.0, 
2.0, 7.0], [3.0, 2.0, 2.0], [6.0, 1.0, 3.0]], [[7.0, 5.0, 4.0], [4.0, 2.0, 5.0], [6.0, 6.0, 1.0]]], [[[7.0, 2.0, 3.0], [6.0, 
6.0, 8.0], [3.0, 8.0, 4.0]], [[1.0, 4.0, 8.0], [5.0, 4.0, 2.0], [8.0, 1.0, 1.0]], [[3.0, 8.0, 5.0], [8.0, 3.0, 6.0], [5.0, 6.0, 
3.0]]], [[[4.0, 8.0, 1.0], [1.0, 3.0, 7.0], [5.0, 7.0, 3.0]], [[1.0, 2.0, 8.0], [2.0, 8.0, 6.0], [3.0, 6.0, 4.0]], [[3.0, 7.0, 
3.0], [2.0, 3.0, 7.0], [4.0, 6.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 1.0, 5.0], [5.0, 2.0, 8.0], [5.0, 2.0, 4.0]], 
[[5.0, 6.0, 3.0], [8.0, 4.0, 3.0], [1.0, 1.0, 5.0]], [[6.0, 5.0, 7.0], [8.0, 1.0, 7.0], [6.0, 2.0, 4.0]]], [[[7.0, 8.0, 2.0], 
[8.0, 6.0, 3.0], [2.0, 2.0, 1.0]], [[8.0, 2.0, 7.0], [3.0, 2.0, 2.0], [6.0, 1.0, 3.0]], [[3.0, 8.0, 7.0], [3.0, 1.0, 1.0], 
[1.0, 8.0, 8.0]]], [[[7.0, 1.0, 7.0], [6.0, 4.0, 1.0], [7.0, 8.0, 2.0]], [[1.0, 4.0, 8.0], [5.0, 4.0, 2.0], [8.0, 1.0, 1.0]], 
[[5.0, 5.0, 5.0], [5.0, 4.0, 3.0], [8.0, 7.0, 3.0]]], [[[3.0, 5.0, 1.0], [2.0, 2.0, 7.0], [3.0, 6.0, 1.0]], [[1.0, 2.0, 8.0], 
[2.0, 8.0, 6.0], [3.0, 6.0, 4.0]], [[8.0, 4.0, 1.0], [1.0, 6.0, 5.0], [8.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 7.0, 2.0], [4.0, 6.0, 6.0], [7.0, 2.0, 7.0]], [[3.0, 5.0, 7.0], [4.0, 1.0, 3.0], [6.0, 
4.0, 8.0]], [[5.0, 2.0, 7.0], [6.0, 1.0, 6.0], [3.0, 8.0, 5.0]]], [[[4.0, 5.0, 6.0], [2.0, 7.0, 6.0], [3.0, 6.0, 4.0]], [[7.0, 
7.0, 3.0], [8.0, 3.0, 5.0], [1.0, 5.0, 8.0]], [[2.0, 7.0, 6.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]]], [[[7.0, 6.0, 7.0], [3.0, 
3.0, 2.0], [7.0, 6.0, 1.0]], [[1.0, 7.0, 8.0], [2.0, 1.0, 6.0], [5.0, 3.0, 5.0]], [[6.0, 4.0, 8.0], [8.0, 7.0, 7.0], [3.0, 8.0, 
5.0]]], [[[3.0, 4.0, 5.0], [6.0, 1.0, 4.0], [8.0, 5.0, 8.0]], [[2.0, 6.0, 2.0], [5.0, 4.0, 1.0], [5.0, 4.0, 6.0]], [[7.0, 8.0, 
7.0], [6.0, 6.0, 6.0], [5.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 1.0, 8.0], [8.0, 3.0, 5.0], [6.0, 2.0, 8.0]], 
[[5.0, 8.0, 3.0], [1.0, 3.0, 8.0], [6.0, 7.0, 8.0]], [[5.0, 2.0, 6.0], [7.0, 7.0, 8.0], [2.0, 2.0, 4.0]]], [[[1.0, 5.0, 1.0], 
[5.0, 7.0, 6.0], [1.0, 3.0, 6.0]], [[7.0, 5.0, 4.0], [7.0, 6.0, 1.0], [3.0, 4.0, 8.0]], [[2.0, 3.0, 7.0], [5.0, 2.0, 5.0], 
[7.0, 5.0, 4.0]]], [[[6.0, 3.0, 4.0], [5.0, 6.0, 2.0], [8.0, 6.0, 4.0]], [[8.0, 6.0, 6.0], [5.0, 2.0, 8.0], [5.0, 4.0, 4.0]], 
[[1.0, 7.0, 2.0], [3.0, 1.0, 7.0], [1.0, 7.0, 4.0]]], [[[2.0, 3.0, 4.0], [7.0, 5.0, 2.0], [7.0, 5.0, 4.0]], [[7.0, 2.0, 4.0], 
[7.0, 6.0, 6.0], [7.0, 3.0, 6.0]], [[7.0, 7.0, 7.0], [1.0, 7.0, 7.0], [7.0, 6.0, 3.0]]]])
      val=Data(numpy.array([[[3.0, 6.0, 3.0], [5.0, 2.0, 5.0], [8.0, 7.0, 6.0]], [[6.0, 1.0, 6.0], [1.0, 2.0, 5.0], [3.0, 6.0, 
2.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 4.0], [4.0, 6.0, 6.0]], [[4.0, 8.0, 7.0], [6.0, 2.0, 3.0], [2.0, 8.0, 
5.0]]]),self.functionspace)
      arg[:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 7.0, 2.0], [4.0, 6.0, 6.0], [7.0, 2.0, 7.0]], [[3.0, 6.0, 3.0], [5.0, 2.0, 5.0], [8.0, 
7.0, 6.0]], [[5.0, 2.0, 7.0], [6.0, 1.0, 6.0], [3.0, 8.0, 5.0]]], [[[4.0, 5.0, 6.0], [2.0, 7.0, 6.0], [3.0, 6.0, 4.0]], [[6.0, 
1.0, 6.0], [1.0, 2.0, 5.0], [3.0, 6.0, 2.0]], [[2.0, 7.0, 6.0], [6.0, 5.0, 2.0], [6.0, 4.0, 5.0]]], [[[7.0, 6.0, 7.0], [3.0, 
3.0, 2.0], [7.0, 6.0, 1.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 4.0], [4.0, 6.0, 6.0]], [[6.0, 4.0, 8.0], [8.0, 7.0, 7.0], [3.0, 8.0, 
5.0]]], [[[3.0, 4.0, 5.0], [6.0, 1.0, 4.0], [8.0, 5.0, 8.0]], [[4.0, 8.0, 7.0], [6.0, 2.0, 3.0], [2.0, 8.0, 5.0]], [[7.0, 8.0, 
7.0], [6.0, 6.0, 6.0], [5.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 1.0, 8.0], [8.0, 3.0, 5.0], [6.0, 2.0, 8.0]], 
[[3.0, 6.0, 3.0], [5.0, 2.0, 5.0], [8.0, 7.0, 6.0]], [[5.0, 2.0, 6.0], [7.0, 7.0, 8.0], [2.0, 2.0, 4.0]]], [[[1.0, 5.0, 1.0], 
[5.0, 7.0, 6.0], [1.0, 3.0, 6.0]], [[6.0, 1.0, 6.0], [1.0, 2.0, 5.0], [3.0, 6.0, 2.0]], [[2.0, 3.0, 7.0], [5.0, 2.0, 5.0], 
[7.0, 5.0, 4.0]]], [[[6.0, 3.0, 4.0], [5.0, 6.0, 2.0], [8.0, 6.0, 4.0]], [[3.0, 5.0, 8.0], [5.0, 1.0, 4.0], [4.0, 6.0, 6.0]], 
[[1.0, 7.0, 2.0], [3.0, 1.0, 7.0], [1.0, 7.0, 4.0]]], [[[2.0, 3.0, 4.0], [7.0, 5.0, 2.0], [7.0, 5.0, 4.0]], [[4.0, 8.0, 7.0], 
[6.0, 2.0, 3.0], [2.0, 8.0, 5.0]], [[7.0, 7.0, 7.0], [1.0, 7.0, 7.0], [7.0, 6.0, 3.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CPCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 1.0, 3.0], [6.0, 5.0, 4.0], [5.0, 2.0, 3.0]], [[3.0, 4.0, 7.0], [4.0, 1.0, 4.0], [1.0, 
6.0, 8.0]], [[4.0, 4.0, 8.0], [3.0, 2.0, 4.0], [6.0, 7.0, 6.0]]], [[[6.0, 3.0, 6.0], [2.0, 7.0, 8.0], [6.0, 2.0, 1.0]], [[1.0, 
5.0, 7.0], [2.0, 4.0, 2.0], [3.0, 6.0, 2.0]], [[4.0, 8.0, 6.0], [7.0, 2.0, 8.0], [3.0, 2.0, 4.0]]], [[[1.0, 4.0, 8.0], [7.0, 
3.0, 1.0], [1.0, 4.0, 7.0]], [[2.0, 6.0, 1.0], [3.0, 8.0, 6.0], [8.0, 8.0, 4.0]], [[7.0, 3.0, 2.0], [3.0, 4.0, 8.0], [4.0, 5.0, 
1.0]]], [[[1.0, 1.0, 6.0], [4.0, 6.0, 6.0], [8.0, 4.0, 6.0]], [[3.0, 6.0, 4.0], [5.0, 1.0, 4.0], [2.0, 4.0, 5.0]], [[7.0, 4.0, 
3.0], [6.0, 4.0, 6.0], [1.0, 4.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 2.0], [4.0, 5.0, 3.0], [1.0, 3.0, 4.0]], 
[[4.0, 5.0, 3.0], [5.0, 4.0, 6.0], [3.0, 1.0, 1.0]], [[8.0, 4.0, 3.0], [2.0, 2.0, 6.0], [2.0, 8.0, 3.0]]], [[[7.0, 7.0, 3.0], 
[3.0, 2.0, 4.0], [7.0, 7.0, 5.0]], [[5.0, 7.0, 1.0], [3.0, 2.0, 2.0], [8.0, 5.0, 2.0]], [[3.0, 6.0, 5.0], [4.0, 7.0, 1.0], 
[6.0, 2.0, 6.0]]], [[[2.0, 4.0, 4.0], [8.0, 5.0, 1.0], [2.0, 8.0, 2.0]], [[4.0, 8.0, 8.0], [4.0, 2.0, 8.0], [3.0, 3.0, 7.0]], 
[[7.0, 7.0, 3.0], [5.0, 8.0, 6.0], [5.0, 2.0, 3.0]]], [[[3.0, 2.0, 4.0], [8.0, 4.0, 6.0], [7.0, 7.0, 7.0]], [[8.0, 4.0, 4.0], 
[8.0, 1.0, 3.0], [7.0, 6.0, 7.0]], [[8.0, 7.0, 8.0], [5.0, 1.0, 3.0], [3.0, 4.0, 6.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 3.0, 5.0], [8.0, 2.0, 7.0], [4.0, 8.0, 3.0]], [[7.0, 7.0, 6.0], [2.0, 5.0, 2.0], [3.0, 
1.0, 8.0]], [[7.0, 6.0, 8.0], [2.0, 3.0, 2.0], [6.0, 7.0, 1.0]], [[3.0, 3.0, 6.0], [7.0, 1.0, 2.0], [1.0, 4.0, 
1.0]]])+(1.-msk_val)*numpy.array([[[3.0, 1.0, 2.0], [4.0, 4.0, 1.0], [8.0, 5.0, 2.0]], [[5.0, 3.0, 3.0], [7.0, 7.0, 8.0], [3.0, 
1.0, 7.0]], [[6.0, 1.0, 2.0], [1.0, 2.0, 5.0], [8.0, 1.0, 2.0]], [[7.0, 4.0, 3.0], [2.0, 5.0, 7.0], [2.0, 8.0, 3.0]]])
      arg[:,1,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 1.0, 3.0], [6.0, 5.0, 4.0], [5.0, 2.0, 3.0]], [[4.0, 3.0, 5.0], [8.0, 2.0, 7.0], [4.0, 
8.0, 3.0]], [[4.0, 4.0, 8.0], [3.0, 2.0, 4.0], [6.0, 7.0, 6.0]]], [[[6.0, 3.0, 6.0], [2.0, 7.0, 8.0], [6.0, 2.0, 1.0]], [[7.0, 
7.0, 6.0], [2.0, 5.0, 2.0], [3.0, 1.0, 8.0]], [[4.0, 8.0, 6.0], [7.0, 2.0, 8.0], [3.0, 2.0, 4.0]]], [[[1.0, 4.0, 8.0], [7.0, 
3.0, 1.0], [1.0, 4.0, 7.0]], [[7.0, 6.0, 8.0], [2.0, 3.0, 2.0], [6.0, 7.0, 1.0]], [[7.0, 3.0, 2.0], [3.0, 4.0, 8.0], [4.0, 5.0, 
1.0]]], [[[1.0, 1.0, 6.0], [4.0, 6.0, 6.0], [8.0, 4.0, 6.0]], [[3.0, 3.0, 6.0], [7.0, 1.0, 2.0], [1.0, 4.0, 1.0]], [[7.0, 4.0, 
3.0], [6.0, 4.0, 6.0], [1.0, 4.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 2.0], [4.0, 5.0, 3.0], [1.0, 3.0, 4.0]], 
[[3.0, 1.0, 2.0], [4.0, 4.0, 1.0], [8.0, 5.0, 2.0]], [[8.0, 4.0, 3.0], [2.0, 2.0, 6.0], [2.0, 8.0, 3.0]]], [[[7.0, 7.0, 3.0], 
[3.0, 2.0, 4.0], [7.0, 7.0, 5.0]], [[5.0, 3.0, 3.0], [7.0, 7.0, 8.0], [3.0, 1.0, 7.0]], [[3.0, 6.0, 5.0], [4.0, 7.0, 1.0], 
[6.0, 2.0, 6.0]]], [[[2.0, 4.0, 4.0], [8.0, 5.0, 1.0], [2.0, 8.0, 2.0]], [[6.0, 1.0, 2.0], [1.0, 2.0, 5.0], [8.0, 1.0, 2.0]], 
[[7.0, 7.0, 3.0], [5.0, 8.0, 6.0], [5.0, 2.0, 3.0]]], [[[3.0, 2.0, 4.0], [8.0, 4.0, 6.0], [7.0, 7.0, 7.0]], [[7.0, 4.0, 3.0], 
[2.0, 5.0, 7.0], [2.0, 8.0, 3.0]], [[8.0, 7.0, 8.0], [5.0, 1.0, 3.0], [3.0, 4.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_URCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 1.0, 8.0], [2.0, 5.0, 2.0], [4.0, 3.0, 1.0]], [[3.0, 8.0, 4.0], [6.0, 6.0, 7.0], [5.0, 
5.0, 3.0]], [[1.0, 1.0, 1.0], [5.0, 6.0, 4.0], [2.0, 4.0, 3.0]]], [[[8.0, 7.0, 7.0], [5.0, 3.0, 8.0], [3.0, 7.0, 1.0]], [[8.0, 
4.0, 6.0], [3.0, 8.0, 2.0], [5.0, 4.0, 4.0]], [[5.0, 2.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 5.0]]], [[[7.0, 4.0, 3.0], [8.0, 
1.0, 8.0], [8.0, 5.0, 6.0]], [[2.0, 7.0, 3.0], [6.0, 1.0, 6.0], [6.0, 1.0, 7.0]], [[1.0, 7.0, 3.0], [5.0, 7.0, 6.0], [1.0, 6.0, 
1.0]]], [[[4.0, 2.0, 8.0], [3.0, 5.0, 3.0], [4.0, 6.0, 5.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 5.0], [6.0, 1.0, 5.0]], [[7.0, 6.0, 
6.0], [2.0, 3.0, 6.0], [2.0, 1.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 6.0, 6.0], [8.0, 5.0, 5.0], [7.0, 3.0, 7.0]], 
[[4.0, 5.0, 2.0], [7.0, 1.0, 5.0], [2.0, 6.0, 1.0]], [[8.0, 5.0, 1.0], [4.0, 5.0, 1.0], [5.0, 2.0, 4.0]]], [[[6.0, 1.0, 8.0], 
[2.0, 1.0, 7.0], [5.0, 8.0, 7.0]], [[6.0, 8.0, 5.0], [5.0, 1.0, 1.0], [7.0, 7.0, 4.0]], [[7.0, 8.0, 4.0], [1.0, 3.0, 2.0], 
[6.0, 3.0, 5.0]]], [[[6.0, 7.0, 4.0], [4.0, 5.0, 2.0], [4.0, 5.0, 5.0]], [[2.0, 7.0, 5.0], [4.0, 3.0, 2.0], [5.0, 7.0, 2.0]], 
[[3.0, 6.0, 6.0], [2.0, 7.0, 2.0], [4.0, 2.0, 6.0]]], [[[6.0, 5.0, 6.0], [5.0, 2.0, 6.0], [5.0, 6.0, 7.0]], [[7.0, 6.0, 3.0], 
[5.0, 6.0, 7.0], [5.0, 5.0, 1.0]], [[7.0, 3.0, 8.0], [8.0, 6.0, 8.0], [7.0, 5.0, 7.0]]]])
      val=numpy.array([[[[6.0, 2.0, 6.0], [8.0, 5.0, 7.0], [3.0, 1.0, 5.0]]], [[[2.0, 3.0, 2.0], [7.0, 7.0, 6.0], [8.0, 8.0, 
2.0]]], [[[4.0, 4.0, 2.0], [3.0, 3.0, 7.0], [4.0, 4.0, 4.0]]]])
      arg[:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 1.0, 8.0], [2.0, 5.0, 2.0], [4.0, 3.0, 1.0]], [[6.0, 2.0, 6.0], [8.0, 5.0, 7.0], [3.0, 
1.0, 5.0]], [[1.0, 1.0, 1.0], [5.0, 6.0, 4.0], [2.0, 4.0, 3.0]]], [[[8.0, 7.0, 7.0], [5.0, 3.0, 8.0], [3.0, 7.0, 1.0]], [[2.0, 
3.0, 2.0], [7.0, 7.0, 6.0], [8.0, 8.0, 2.0]], [[5.0, 2.0, 8.0], [3.0, 3.0, 8.0], [6.0, 8.0, 5.0]]], [[[7.0, 4.0, 3.0], [8.0, 
1.0, 8.0], [8.0, 5.0, 6.0]], [[4.0, 4.0, 2.0], [3.0, 3.0, 7.0], [4.0, 4.0, 4.0]], [[1.0, 7.0, 3.0], [5.0, 7.0, 6.0], [1.0, 6.0, 
1.0]]], [[[4.0, 2.0, 8.0], [3.0, 5.0, 3.0], [4.0, 6.0, 5.0]], [[7.0, 8.0, 3.0], [6.0, 5.0, 5.0], [6.0, 1.0, 5.0]], [[7.0, 6.0, 
6.0], [2.0, 3.0, 6.0], [2.0, 1.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 6.0, 6.0], [8.0, 5.0, 5.0], [7.0, 3.0, 7.0]], 
[[6.0, 2.0, 6.0], [8.0, 5.0, 7.0], [3.0, 1.0, 5.0]], [[8.0, 5.0, 1.0], [4.0, 5.0, 1.0], [5.0, 2.0, 4.0]]], [[[6.0, 1.0, 8.0], 
[2.0, 1.0, 7.0], [5.0, 8.0, 7.0]], [[2.0, 3.0, 2.0], [7.0, 7.0, 6.0], [8.0, 8.0, 2.0]], [[7.0, 8.0, 4.0], [1.0, 3.0, 2.0], 
[6.0, 3.0, 5.0]]], [[[6.0, 7.0, 4.0], [4.0, 5.0, 2.0], [4.0, 5.0, 5.0]], [[4.0, 4.0, 2.0], [3.0, 3.0, 7.0], [4.0, 4.0, 4.0]], 
[[3.0, 6.0, 6.0], [2.0, 7.0, 2.0], [4.0, 2.0, 6.0]]], [[[6.0, 5.0, 6.0], [5.0, 2.0, 6.0], [5.0, 6.0, 7.0]], [[7.0, 6.0, 3.0], 
[5.0, 6.0, 7.0], [5.0, 5.0, 1.0]], [[7.0, 3.0, 8.0], [8.0, 6.0, 8.0], [7.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_URCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 8.0, 4.0], [7.0, 5.0, 1.0], [6.0, 5.0, 6.0]], [[2.0, 3.0, 3.0], [6.0, 8.0, 2.0], [6.0, 
6.0, 4.0]], [[6.0, 7.0, 4.0], [2.0, 1.0, 6.0], [4.0, 8.0, 4.0]]], [[[3.0, 5.0, 2.0], [2.0, 4.0, 6.0], [1.0, 7.0, 8.0]], [[6.0, 
3.0, 2.0], [7.0, 2.0, 6.0], [4.0, 7.0, 5.0]], [[5.0, 8.0, 1.0], [5.0, 7.0, 1.0], [5.0, 3.0, 3.0]]], [[[2.0, 3.0, 5.0], [3.0, 
1.0, 8.0], [7.0, 1.0, 1.0]], [[8.0, 6.0, 7.0], [3.0, 5.0, 3.0], [8.0, 1.0, 5.0]], [[4.0, 2.0, 7.0], [7.0, 4.0, 4.0], [5.0, 6.0, 
7.0]]], [[[1.0, 6.0, 6.0], [4.0, 2.0, 6.0], [1.0, 4.0, 4.0]], [[1.0, 3.0, 1.0], [2.0, 8.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 5.0, 
5.0], [8.0, 2.0, 8.0], [1.0, 5.0, 1.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 8.0], [6.0, 6.0, 4.0], [7.0, 8.0, 6.0]], 
[[3.0, 1.0, 6.0], [5.0, 1.0, 7.0], [6.0, 3.0, 2.0]], [[4.0, 7.0, 1.0], [5.0, 3.0, 5.0], [8.0, 2.0, 8.0]]], [[[7.0, 3.0, 6.0], 
[8.0, 6.0, 7.0], [4.0, 1.0, 7.0]], [[1.0, 7.0, 8.0], [2.0, 4.0, 8.0], [7.0, 7.0, 8.0]], [[6.0, 4.0, 6.0], [7.0, 7.0, 5.0], 
[2.0, 5.0, 3.0]]], [[[5.0, 1.0, 3.0], [8.0, 6.0, 1.0], [7.0, 8.0, 6.0]], [[1.0, 3.0, 2.0], [7.0, 5.0, 2.0], [4.0, 1.0, 3.0]], 
[[8.0, 7.0, 7.0], [1.0, 8.0, 1.0], [8.0, 6.0, 4.0]]], [[[5.0, 7.0, 3.0], [6.0, 3.0, 5.0], [2.0, 1.0, 2.0]], [[4.0, 6.0, 3.0], 
[5.0, 4.0, 5.0], [5.0, 5.0, 4.0]], [[5.0, 3.0, 7.0], [2.0, 6.0, 7.0], [5.0, 3.0, 6.0]]]])
      val=Data(numpy.array([[[[5.0, 3.0, 6.0], [8.0, 4.0, 2.0], [3.0, 3.0, 4.0]]], [[[2.0, 3.0, 8.0], [3.0, 6.0, 5.0], [6.0, 
3.0, 7.0]]], [[[5.0, 5.0, 1.0], [4.0, 3.0, 5.0], [7.0, 7.0, 3.0]]]]),self.functionspace)
      arg[:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 8.0, 4.0], [7.0, 5.0, 1.0], [6.0, 5.0, 6.0]], [[5.0, 3.0, 6.0], [8.0, 4.0, 2.0], [3.0, 
3.0, 4.0]], [[6.0, 7.0, 4.0], [2.0, 1.0, 6.0], [4.0, 8.0, 4.0]]], [[[3.0, 5.0, 2.0], [2.0, 4.0, 6.0], [1.0, 7.0, 8.0]], [[2.0, 
3.0, 8.0], [3.0, 6.0, 5.0], [6.0, 3.0, 7.0]], [[5.0, 8.0, 1.0], [5.0, 7.0, 1.0], [5.0, 3.0, 3.0]]], [[[2.0, 3.0, 5.0], [3.0, 
1.0, 8.0], [7.0, 1.0, 1.0]], [[5.0, 5.0, 1.0], [4.0, 3.0, 5.0], [7.0, 7.0, 3.0]], [[4.0, 2.0, 7.0], [7.0, 4.0, 4.0], [5.0, 6.0, 
7.0]]], [[[1.0, 6.0, 6.0], [4.0, 2.0, 6.0], [1.0, 4.0, 4.0]], [[1.0, 3.0, 1.0], [2.0, 8.0, 2.0], [4.0, 5.0, 2.0]], [[1.0, 5.0, 
5.0], [8.0, 2.0, 8.0], [1.0, 5.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 8.0], [6.0, 6.0, 4.0], [7.0, 8.0, 6.0]], 
[[5.0, 3.0, 6.0], [8.0, 4.0, 2.0], [3.0, 3.0, 4.0]], [[4.0, 7.0, 1.0], [5.0, 3.0, 5.0], [8.0, 2.0, 8.0]]], [[[7.0, 3.0, 6.0], 
[8.0, 6.0, 7.0], [4.0, 1.0, 7.0]], [[2.0, 3.0, 8.0], [3.0, 6.0, 5.0], [6.0, 3.0, 7.0]], [[6.0, 4.0, 6.0], [7.0, 7.0, 5.0], 
[2.0, 5.0, 3.0]]], [[[5.0, 1.0, 3.0], [8.0, 6.0, 1.0], [7.0, 8.0, 6.0]], [[5.0, 5.0, 1.0], [4.0, 3.0, 5.0], [7.0, 7.0, 3.0]], 
[[8.0, 7.0, 7.0], [1.0, 8.0, 1.0], [8.0, 6.0, 4.0]]], [[[5.0, 7.0, 3.0], [6.0, 3.0, 5.0], [2.0, 1.0, 2.0]], [[4.0, 6.0, 3.0], 
[5.0, 4.0, 5.0], [5.0, 5.0, 4.0]], [[5.0, 3.0, 7.0], [2.0, 6.0, 7.0], [5.0, 3.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_URCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 4.0, 1.0], [8.0, 1.0, 1.0], [3.0, 3.0, 1.0]], [[7.0, 3.0, 4.0], [8.0, 7.0, 5.0], [3.0, 
2.0, 3.0]], [[2.0, 2.0, 3.0], [3.0, 4.0, 8.0], [5.0, 1.0, 3.0]]], [[[3.0, 8.0, 7.0], [5.0, 7.0, 1.0], [1.0, 3.0, 3.0]], [[1.0, 
6.0, 7.0], [4.0, 8.0, 8.0], [5.0, 5.0, 4.0]], [[4.0, 2.0, 7.0], [4.0, 3.0, 4.0], [1.0, 8.0, 3.0]]], [[[1.0, 6.0, 3.0], [2.0, 
1.0, 6.0], [1.0, 8.0, 8.0]], [[5.0, 2.0, 5.0], [8.0, 4.0, 6.0], [5.0, 8.0, 4.0]], [[4.0, 8.0, 1.0], [5.0, 5.0, 1.0], [1.0, 5.0, 
6.0]]], [[[2.0, 3.0, 4.0], [8.0, 6.0, 2.0], [1.0, 7.0, 2.0]], [[7.0, 1.0, 6.0], [4.0, 8.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 7.0, 
4.0], [2.0, 8.0, 8.0], [4.0, 6.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 3.0], [2.0, 8.0, 4.0], [1.0, 6.0, 6.0]], 
[[5.0, 2.0, 5.0], [7.0, 3.0, 8.0], [5.0, 1.0, 8.0]], [[8.0, 6.0, 1.0], [2.0, 5.0, 2.0], [7.0, 6.0, 8.0]]], [[[2.0, 7.0, 8.0], 
[6.0, 7.0, 7.0], [4.0, 1.0, 5.0]], [[7.0, 7.0, 4.0], [5.0, 7.0, 6.0], [8.0, 6.0, 2.0]], [[8.0, 5.0, 6.0], [7.0, 7.0, 6.0], 
[1.0, 1.0, 3.0]]], [[[6.0, 1.0, 3.0], [6.0, 7.0, 7.0], [8.0, 7.0, 1.0]], [[2.0, 6.0, 4.0], [8.0, 8.0, 7.0], [2.0, 6.0, 2.0]], 
[[7.0, 7.0, 6.0], [7.0, 3.0, 8.0], [8.0, 3.0, 3.0]]], [[[6.0, 8.0, 6.0], [8.0, 5.0, 5.0], [6.0, 7.0, 3.0]], [[4.0, 2.0, 4.0], 
[7.0, 2.0, 7.0], [4.0, 5.0, 5.0]], [[3.0, 8.0, 6.0], [4.0, 5.0, 3.0], [7.0, 1.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[4.0, 3.0, 5.0], [4.0, 4.0, 7.0], [7.0, 7.0, 6.0]]], [[[2.0, 8.0, 6.0], [6.0, 3.0, 4.0], [3.0, 
4.0, 5.0]]], [[[8.0, 5.0, 6.0], [1.0, 1.0, 5.0], [2.0, 4.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[6.0, 5.0, 7.0], [6.0, 1.0, 
3.0], [6.0, 2.0, 5.0]]], [[[4.0, 6.0, 5.0], [4.0, 1.0, 3.0], [8.0, 2.0, 3.0]]], [[[5.0, 1.0, 4.0], [4.0, 5.0, 4.0], [5.0, 8.0, 
8.0]]]])
      arg[:3,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 4.0, 1.0], [8.0, 1.0, 1.0], [3.0, 3.0, 1.0]], [[4.0, 3.0, 5.0], [4.0, 4.0, 7.0], [7.0, 
7.0, 6.0]], [[2.0, 2.0, 3.0], [3.0, 4.0, 8.0], [5.0, 1.0, 3.0]]], [[[3.0, 8.0, 7.0], [5.0, 7.0, 1.0], [1.0, 3.0, 3.0]], [[2.0, 
8.0, 6.0], [6.0, 3.0, 4.0], [3.0, 4.0, 5.0]], [[4.0, 2.0, 7.0], [4.0, 3.0, 4.0], [1.0, 8.0, 3.0]]], [[[1.0, 6.0, 3.0], [2.0, 
1.0, 6.0], [1.0, 8.0, 8.0]], [[8.0, 5.0, 6.0], [1.0, 1.0, 5.0], [2.0, 4.0, 2.0]], [[4.0, 8.0, 1.0], [5.0, 5.0, 1.0], [1.0, 5.0, 
6.0]]], [[[2.0, 3.0, 4.0], [8.0, 6.0, 2.0], [1.0, 7.0, 2.0]], [[7.0, 1.0, 6.0], [4.0, 8.0, 7.0], [3.0, 7.0, 4.0]], [[7.0, 7.0, 
4.0], [2.0, 8.0, 8.0], [4.0, 6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 3.0], [2.0, 8.0, 4.0], [1.0, 6.0, 6.0]], 
[[6.0, 5.0, 7.0], [6.0, 1.0, 3.0], [6.0, 2.0, 5.0]], [[8.0, 6.0, 1.0], [2.0, 5.0, 2.0], [7.0, 6.0, 8.0]]], [[[2.0, 7.0, 8.0], 
[6.0, 7.0, 7.0], [4.0, 1.0, 5.0]], [[4.0, 6.0, 5.0], [4.0, 1.0, 3.0], [8.0, 2.0, 3.0]], [[8.0, 5.0, 6.0], [7.0, 7.0, 6.0], 
[1.0, 1.0, 3.0]]], [[[6.0, 1.0, 3.0], [6.0, 7.0, 7.0], [8.0, 7.0, 1.0]], [[5.0, 1.0, 4.0], [4.0, 5.0, 4.0], [5.0, 8.0, 8.0]], 
[[7.0, 7.0, 6.0], [7.0, 3.0, 8.0], [8.0, 3.0, 3.0]]], [[[6.0, 8.0, 6.0], [8.0, 5.0, 5.0], [6.0, 7.0, 3.0]], [[4.0, 2.0, 4.0], 
[7.0, 2.0, 7.0], [4.0, 5.0, 5.0]], [[3.0, 8.0, 6.0], [4.0, 5.0, 3.0], [7.0, 1.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 5.0, 1.0], [4.0, 6.0, 5.0], [4.0, 2.0, 2.0]], [[7.0, 3.0, 4.0], [7.0, 7.0, 3.0], [8.0, 
7.0, 2.0]], [[5.0, 7.0, 2.0], [3.0, 3.0, 5.0], [7.0, 7.0, 2.0]]], [[[3.0, 5.0, 8.0], [5.0, 6.0, 6.0], [6.0, 2.0, 2.0]], [[7.0, 
2.0, 8.0], [2.0, 2.0, 3.0], [4.0, 6.0, 7.0]], [[5.0, 8.0, 5.0], [5.0, 7.0, 2.0], [5.0, 7.0, 8.0]]], [[[5.0, 6.0, 7.0], [6.0, 
4.0, 6.0], [5.0, 6.0, 3.0]], [[6.0, 1.0, 5.0], [2.0, 8.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 7.0, 1.0], [7.0, 1.0, 2.0], [7.0, 5.0, 
6.0]]], [[[2.0, 6.0, 1.0], [6.0, 2.0, 8.0], [4.0, 6.0, 3.0]], [[5.0, 6.0, 7.0], [5.0, 1.0, 8.0], [6.0, 8.0, 7.0]], [[7.0, 7.0, 
8.0], [2.0, 6.0, 6.0], [6.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 7.0, 4.0], [1.0, 1.0, 2.0], [8.0, 7.0, 5.0]], 
[[5.0, 7.0, 5.0], [3.0, 8.0, 2.0], [6.0, 1.0, 5.0]], [[2.0, 5.0, 1.0], [3.0, 2.0, 7.0], [6.0, 8.0, 2.0]]], [[[2.0, 6.0, 1.0], 
[5.0, 1.0, 5.0], [5.0, 8.0, 5.0]], [[2.0, 5.0, 6.0], [6.0, 7.0, 3.0], [6.0, 5.0, 1.0]], [[4.0, 2.0, 3.0], [7.0, 2.0, 3.0], 
[5.0, 1.0, 4.0]]], [[[8.0, 2.0, 6.0], [5.0, 2.0, 1.0], [3.0, 8.0, 7.0]], [[4.0, 6.0, 3.0], [6.0, 8.0, 5.0], [1.0, 5.0, 8.0]], 
[[2.0, 5.0, 4.0], [8.0, 3.0, 1.0], [4.0, 2.0, 2.0]]], [[[8.0, 3.0, 7.0], [8.0, 5.0, 1.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 4.0], 
[4.0, 1.0, 3.0], [6.0, 5.0, 4.0]], [[2.0, 6.0, 3.0], [7.0, 7.0, 2.0], [7.0, 4.0, 5.0]]]])
      val=numpy.array([[[3.0, 8.0, 2.0], [7.0, 8.0, 6.0], [8.0, 8.0, 8.0]], [[3.0, 2.0, 2.0], [7.0, 4.0, 8.0], [4.0, 4.0, 
7.0]], [[2.0, 8.0, 3.0], [5.0, 5.0, 8.0], [2.0, 4.0, 2.0]], [[4.0, 8.0, 5.0], [1.0, 1.0, 4.0], [2.0, 7.0, 3.0]]])
      arg[:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 5.0, 1.0], [3.0, 8.0, 2.0], [4.0, 2.0, 2.0]], [[7.0, 3.0, 4.0], [7.0, 8.0, 6.0], [8.0, 
7.0, 2.0]], [[5.0, 7.0, 2.0], [8.0, 8.0, 8.0], [7.0, 7.0, 2.0]]], [[[3.0, 5.0, 8.0], [3.0, 2.0, 2.0], [6.0, 2.0, 2.0]], [[7.0, 
2.0, 8.0], [7.0, 4.0, 8.0], [4.0, 6.0, 7.0]], [[5.0, 8.0, 5.0], [4.0, 4.0, 7.0], [5.0, 7.0, 8.0]]], [[[5.0, 6.0, 7.0], [2.0, 
8.0, 3.0], [5.0, 6.0, 3.0]], [[6.0, 1.0, 5.0], [5.0, 5.0, 8.0], [8.0, 7.0, 7.0]], [[8.0, 7.0, 1.0], [2.0, 4.0, 2.0], [7.0, 5.0, 
6.0]]], [[[2.0, 6.0, 1.0], [4.0, 8.0, 5.0], [4.0, 6.0, 3.0]], [[5.0, 6.0, 7.0], [1.0, 1.0, 4.0], [6.0, 8.0, 7.0]], [[7.0, 7.0, 
8.0], [2.0, 7.0, 3.0], [6.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 7.0, 4.0], [3.0, 8.0, 2.0], [8.0, 7.0, 5.0]], 
[[5.0, 7.0, 5.0], [7.0, 8.0, 6.0], [6.0, 1.0, 5.0]], [[2.0, 5.0, 1.0], [8.0, 8.0, 8.0], [6.0, 8.0, 2.0]]], [[[2.0, 6.0, 1.0], 
[3.0, 2.0, 2.0], [5.0, 8.0, 5.0]], [[2.0, 5.0, 6.0], [7.0, 4.0, 8.0], [6.0, 5.0, 1.0]], [[4.0, 2.0, 3.0], [4.0, 4.0, 7.0], 
[5.0, 1.0, 4.0]]], [[[8.0, 2.0, 6.0], [2.0, 8.0, 3.0], [3.0, 8.0, 7.0]], [[4.0, 6.0, 3.0], [5.0, 5.0, 8.0], [1.0, 5.0, 8.0]], 
[[2.0, 5.0, 4.0], [2.0, 4.0, 2.0], [4.0, 2.0, 2.0]]], [[[8.0, 3.0, 7.0], [4.0, 8.0, 5.0], [5.0, 4.0, 4.0]], [[6.0, 3.0, 4.0], 
[1.0, 1.0, 4.0], [6.0, 5.0, 4.0]], [[2.0, 6.0, 3.0], [2.0, 7.0, 3.0], [7.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 5.0, 8.0], [1.0, 6.0, 1.0], [5.0, 3.0, 5.0]], [[8.0, 1.0, 1.0], [4.0, 7.0, 8.0], [7.0, 
5.0, 6.0]], [[5.0, 5.0, 6.0], [5.0, 2.0, 7.0], [5.0, 6.0, 6.0]]], [[[5.0, 1.0, 5.0], [8.0, 1.0, 8.0], [1.0, 5.0, 4.0]], [[3.0, 
8.0, 5.0], [4.0, 6.0, 6.0], [1.0, 3.0, 6.0]], [[1.0, 7.0, 5.0], [3.0, 2.0, 7.0], [8.0, 2.0, 8.0]]], [[[3.0, 7.0, 1.0], [5.0, 
8.0, 5.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [2.0, 2.0, 2.0], [7.0, 4.0, 3.0]], [[8.0, 1.0, 6.0], [3.0, 6.0, 2.0], [5.0, 1.0, 
4.0]]], [[[1.0, 4.0, 4.0], [4.0, 2.0, 3.0], [3.0, 7.0, 8.0]], [[3.0, 6.0, 1.0], [1.0, 6.0, 8.0], [4.0, 6.0, 3.0]], [[5.0, 3.0, 
8.0], [4.0, 3.0, 3.0], [3.0, 4.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 8.0, 3.0], [5.0, 6.0, 8.0], [2.0, 8.0, 4.0]], 
[[6.0, 1.0, 1.0], [7.0, 3.0, 4.0], [8.0, 4.0, 7.0]], [[5.0, 3.0, 6.0], [4.0, 4.0, 2.0], [5.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0], 
[4.0, 4.0, 3.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 5.0], [1.0, 3.0, 6.0], [7.0, 5.0, 6.0]], [[5.0, 7.0, 1.0], [3.0, 4.0, 6.0], 
[6.0, 3.0, 5.0]]], [[[4.0, 4.0, 3.0], [4.0, 7.0, 6.0], [2.0, 5.0, 6.0]], [[1.0, 8.0, 1.0], [8.0, 8.0, 4.0], [3.0, 2.0, 6.0]], 
[[1.0, 4.0, 1.0], [7.0, 3.0, 3.0], [7.0, 6.0, 1.0]]], [[[3.0, 4.0, 2.0], [7.0, 2.0, 1.0], [5.0, 8.0, 7.0]], [[7.0, 4.0, 3.0], 
[7.0, 2.0, 6.0], [8.0, 6.0, 6.0]], [[6.0, 7.0, 3.0], [7.0, 6.0, 5.0], [4.0, 2.0, 6.0]]]])
      val=Data(numpy.array([[[1.0, 7.0, 5.0], [6.0, 7.0, 4.0], [2.0, 3.0, 4.0]], [[2.0, 2.0, 3.0], [3.0, 7.0, 8.0], [7.0, 7.0, 
7.0]], [[6.0, 4.0, 1.0], [3.0, 7.0, 4.0], [8.0, 4.0, 6.0]], [[3.0, 4.0, 6.0], [1.0, 4.0, 1.0], [7.0, 1.0, 
4.0]]]),self.functionspace)
      arg[:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 5.0, 8.0], [1.0, 7.0, 5.0], [5.0, 3.0, 5.0]], [[8.0, 1.0, 1.0], [6.0, 7.0, 4.0], [7.0, 
5.0, 6.0]], [[5.0, 5.0, 6.0], [2.0, 3.0, 4.0], [5.0, 6.0, 6.0]]], [[[5.0, 1.0, 5.0], [2.0, 2.0, 3.0], [1.0, 5.0, 4.0]], [[3.0, 
8.0, 5.0], [3.0, 7.0, 8.0], [1.0, 3.0, 6.0]], [[1.0, 7.0, 5.0], [7.0, 7.0, 7.0], [8.0, 2.0, 8.0]]], [[[3.0, 7.0, 1.0], [6.0, 
4.0, 1.0], [8.0, 8.0, 5.0]], [[1.0, 4.0, 1.0], [3.0, 7.0, 4.0], [7.0, 4.0, 3.0]], [[8.0, 1.0, 6.0], [8.0, 4.0, 6.0], [5.0, 1.0, 
4.0]]], [[[1.0, 4.0, 4.0], [3.0, 4.0, 6.0], [3.0, 7.0, 8.0]], [[3.0, 6.0, 1.0], [1.0, 4.0, 1.0], [4.0, 6.0, 3.0]], [[5.0, 3.0, 
8.0], [7.0, 1.0, 4.0], [3.0, 4.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 8.0, 3.0], [1.0, 7.0, 5.0], [2.0, 8.0, 4.0]], 
[[6.0, 1.0, 1.0], [6.0, 7.0, 4.0], [8.0, 4.0, 7.0]], [[5.0, 3.0, 6.0], [2.0, 3.0, 4.0], [5.0, 2.0, 4.0]]], [[[1.0, 8.0, 4.0], 
[2.0, 2.0, 3.0], [7.0, 5.0, 1.0]], [[8.0, 5.0, 5.0], [3.0, 7.0, 8.0], [7.0, 5.0, 6.0]], [[5.0, 7.0, 1.0], [7.0, 7.0, 7.0], 
[6.0, 3.0, 5.0]]], [[[4.0, 4.0, 3.0], [6.0, 4.0, 1.0], [2.0, 5.0, 6.0]], [[1.0, 8.0, 1.0], [3.0, 7.0, 4.0], [3.0, 2.0, 6.0]], 
[[1.0, 4.0, 1.0], [8.0, 4.0, 6.0], [7.0, 6.0, 1.0]]], [[[3.0, 4.0, 2.0], [3.0, 4.0, 6.0], [5.0, 8.0, 7.0]], [[7.0, 4.0, 3.0], 
[1.0, 4.0, 1.0], [8.0, 6.0, 6.0]], [[6.0, 7.0, 3.0], [7.0, 1.0, 4.0], [4.0, 2.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CCPN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 1.0, 5.0], [1.0, 4.0, 3.0], [8.0, 8.0, 1.0]], [[1.0, 5.0, 6.0], [3.0, 5.0, 3.0], [1.0, 
4.0, 7.0]], [[3.0, 8.0, 2.0], [6.0, 5.0, 2.0], [3.0, 4.0, 7.0]]], [[[3.0, 4.0, 4.0], [7.0, 3.0, 1.0], [6.0, 3.0, 8.0]], [[8.0, 
4.0, 6.0], [6.0, 3.0, 5.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 2.0], [8.0, 5.0, 4.0], [1.0, 4.0, 1.0]]], [[[6.0, 3.0, 2.0], [6.0, 
7.0, 6.0], [6.0, 4.0, 6.0]], [[5.0, 8.0, 5.0], [3.0, 6.0, 4.0], [5.0, 2.0, 5.0]], [[7.0, 3.0, 2.0], [4.0, 7.0, 6.0], [7.0, 5.0, 
1.0]]], [[[3.0, 6.0, 1.0], [1.0, 7.0, 4.0], [8.0, 4.0, 8.0]], [[2.0, 8.0, 5.0], [2.0, 6.0, 5.0], [4.0, 5.0, 6.0]], [[7.0, 1.0, 
3.0], [6.0, 6.0, 8.0], [3.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 5.0, 3.0], [1.0, 2.0, 6.0], [6.0, 6.0, 3.0]], 
[[8.0, 8.0, 7.0], [5.0, 4.0, 5.0], [6.0, 6.0, 8.0]], [[4.0, 1.0, 6.0], [6.0, 7.0, 3.0], [4.0, 7.0, 7.0]]], [[[5.0, 3.0, 4.0], 
[2.0, 2.0, 8.0], [1.0, 4.0, 4.0]], [[4.0, 3.0, 4.0], [2.0, 7.0, 5.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 4.0], [6.0, 2.0, 2.0], 
[6.0, 4.0, 2.0]]], [[[2.0, 1.0, 2.0], [7.0, 6.0, 7.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 2.0], [6.0, 1.0, 4.0], [8.0, 4.0, 8.0]], 
[[7.0, 5.0, 7.0], [4.0, 6.0, 5.0], [4.0, 8.0, 4.0]]], [[[2.0, 4.0, 8.0], [3.0, 3.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 8.0, 7.0], 
[4.0, 5.0, 3.0], [3.0, 4.0, 8.0]], [[8.0, 8.0, 6.0], [5.0, 5.0, 5.0], [6.0, 5.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[1.0, 6.0, 4.0], [1.0, 3.0, 7.0], [7.0, 8.0, 4.0]], [[4.0, 5.0, 6.0], [7.0, 3.0, 8.0], [8.0, 
6.0, 7.0]], [[6.0, 6.0, 1.0], [7.0, 7.0, 1.0], [1.0, 8.0, 3.0]], [[2.0, 6.0, 3.0], [6.0, 3.0, 8.0], [6.0, 8.0, 
8.0]]])+(1.-msk_val)*numpy.array([[[7.0, 5.0, 6.0], [8.0, 7.0, 8.0], [4.0, 8.0, 8.0]], [[7.0, 4.0, 8.0], [5.0, 4.0, 5.0], [5.0, 
1.0, 1.0]], [[2.0, 6.0, 8.0], [2.0, 7.0, 7.0], [1.0, 5.0, 1.0]], [[7.0, 2.0, 5.0], [2.0, 2.0, 1.0], [2.0, 2.0, 2.0]]])
      arg[:,:,1]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 1.0, 5.0], [1.0, 6.0, 4.0], [8.0, 8.0, 1.0]], [[1.0, 5.0, 6.0], [1.0, 3.0, 7.0], [1.0, 
4.0, 7.0]], [[3.0, 8.0, 2.0], [7.0, 8.0, 4.0], [3.0, 4.0, 7.0]]], [[[3.0, 4.0, 4.0], [4.0, 5.0, 6.0], [6.0, 3.0, 8.0]], [[8.0, 
4.0, 6.0], [7.0, 3.0, 8.0], [4.0, 5.0, 8.0]], [[5.0, 1.0, 2.0], [8.0, 6.0, 7.0], [1.0, 4.0, 1.0]]], [[[6.0, 3.0, 2.0], [6.0, 
6.0, 1.0], [6.0, 4.0, 6.0]], [[5.0, 8.0, 5.0], [7.0, 7.0, 1.0], [5.0, 2.0, 5.0]], [[7.0, 3.0, 2.0], [1.0, 8.0, 3.0], [7.0, 5.0, 
1.0]]], [[[3.0, 6.0, 1.0], [2.0, 6.0, 3.0], [8.0, 4.0, 8.0]], [[2.0, 8.0, 5.0], [6.0, 3.0, 8.0], [4.0, 5.0, 6.0]], [[7.0, 1.0, 
3.0], [6.0, 8.0, 8.0], [3.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 5.0, 3.0], [7.0, 5.0, 6.0], [6.0, 6.0, 3.0]], 
[[8.0, 8.0, 7.0], [8.0, 7.0, 8.0], [6.0, 6.0, 8.0]], [[4.0, 1.0, 6.0], [4.0, 8.0, 8.0], [4.0, 7.0, 7.0]]], [[[5.0, 3.0, 4.0], 
[7.0, 4.0, 8.0], [1.0, 4.0, 4.0]], [[4.0, 3.0, 4.0], [5.0, 4.0, 5.0], [7.0, 6.0, 1.0]], [[6.0, 4.0, 4.0], [5.0, 1.0, 1.0], 
[6.0, 4.0, 2.0]]], [[[2.0, 1.0, 2.0], [2.0, 6.0, 8.0], [6.0, 4.0, 8.0]], [[8.0, 3.0, 2.0], [2.0, 7.0, 7.0], [8.0, 4.0, 8.0]], 
[[7.0, 5.0, 7.0], [1.0, 5.0, 1.0], [4.0, 8.0, 4.0]]], [[[2.0, 4.0, 8.0], [7.0, 2.0, 5.0], [7.0, 2.0, 6.0]], [[1.0, 8.0, 7.0], 
[2.0, 2.0, 1.0], [3.0, 4.0, 8.0]], [[8.0, 8.0, 6.0], [2.0, 2.0, 2.0], [6.0, 5.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[1.0, 6.0, 8.0], [8.0, 3.0, 3.0], [7.0, 1.0, 3.0]], [[4.0, 5.0, 1.0], [3.0, 6.0, 7.0], [8.0, 
8.0, 7.0]], [[6.0, 4.0, 3.0], [1.0, 4.0, 5.0], [4.0, 5.0, 1.0]]], [[[4.0, 8.0, 3.0], [5.0, 1.0, 3.0], [5.0, 7.0, 2.0]], [[4.0, 
3.0, 7.0], [3.0, 8.0, 2.0], [3.0, 2.0, 1.0]], [[6.0, 7.0, 4.0], [8.0, 3.0, 3.0], [3.0, 6.0, 2.0]]], [[[2.0, 8.0, 5.0], [4.0, 
2.0, 3.0], [6.0, 3.0, 1.0]], [[1.0, 4.0, 4.0], [6.0, 4.0, 5.0], [4.0, 1.0, 7.0]], [[4.0, 7.0, 4.0], [8.0, 2.0, 2.0], [5.0, 4.0, 
4.0]]], [[[1.0, 8.0, 6.0], [6.0, 1.0, 7.0], [3.0, 7.0, 1.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 7.0], [3.0, 1.0, 8.0]], [[1.0, 3.0, 
5.0], [6.0, 4.0, 2.0], [7.0, 3.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[4.0, 5.0, 2.0], [8.0, 6.0, 4.0], [7.0, 2.0, 2.0]], 
[[4.0, 3.0, 6.0], [8.0, 2.0, 5.0], [2.0, 5.0, 4.0]], [[1.0, 7.0, 3.0], [2.0, 4.0, 8.0], [5.0, 3.0, 3.0]]], [[[6.0, 7.0, 7.0], 
[7.0, 6.0, 8.0], [2.0, 1.0, 4.0]], [[5.0, 4.0, 2.0], [2.0, 1.0, 5.0], [6.0, 1.0, 2.0]], [[4.0, 4.0, 8.0], [6.0, 8.0, 4.0], 
[5.0, 7.0, 7.0]]], [[[8.0, 7.0, 6.0], [7.0, 4.0, 7.0], [3.0, 4.0, 7.0]], [[6.0, 4.0, 1.0], [1.0, 8.0, 8.0], [4.0, 4.0, 8.0]], 
[[8.0, 2.0, 5.0], [8.0, 7.0, 6.0], [4.0, 8.0, 1.0]]], [[[7.0, 5.0, 4.0], [6.0, 7.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 7.0, 3.0], 
[2.0, 8.0, 4.0], [6.0, 3.0, 1.0]], [[2.0, 4.0, 6.0], [3.0, 5.0, 4.0], [2.0, 1.0, 7.0]]]])
      val=numpy.array([[[[8.0, 1.0, 6.0]], [[1.0, 5.0, 4.0]], [[1.0, 2.0, 2.0]]], [[[5.0, 8.0, 6.0]], [[3.0, 3.0, 2.0]], [[2.0, 
7.0, 4.0]]], [[[8.0, 4.0, 5.0]], [[1.0, 2.0, 2.0]], [[7.0, 3.0, 2.0]]]])
      arg[:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 8.0], [8.0, 1.0, 6.0], [7.0, 1.0, 3.0]], [[4.0, 5.0, 1.0], [1.0, 5.0, 4.0], [8.0, 
8.0, 7.0]], [[6.0, 4.0, 3.0], [1.0, 2.0, 2.0], [4.0, 5.0, 1.0]]], [[[4.0, 8.0, 3.0], [5.0, 8.0, 6.0], [5.0, 7.0, 2.0]], [[4.0, 
3.0, 7.0], [3.0, 3.0, 2.0], [3.0, 2.0, 1.0]], [[6.0, 7.0, 4.0], [2.0, 7.0, 4.0], [3.0, 6.0, 2.0]]], [[[2.0, 8.0, 5.0], [8.0, 
4.0, 5.0], [6.0, 3.0, 1.0]], [[1.0, 4.0, 4.0], [1.0, 2.0, 2.0], [4.0, 1.0, 7.0]], [[4.0, 7.0, 4.0], [7.0, 3.0, 2.0], [5.0, 4.0, 
4.0]]], [[[1.0, 8.0, 6.0], [6.0, 1.0, 7.0], [3.0, 7.0, 1.0]], [[4.0, 7.0, 6.0], [1.0, 1.0, 7.0], [3.0, 1.0, 8.0]], [[1.0, 3.0, 
5.0], [6.0, 4.0, 2.0], [7.0, 3.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 5.0, 2.0], [8.0, 1.0, 6.0], [7.0, 2.0, 2.0]], 
[[4.0, 3.0, 6.0], [1.0, 5.0, 4.0], [2.0, 5.0, 4.0]], [[1.0, 7.0, 3.0], [1.0, 2.0, 2.0], [5.0, 3.0, 3.0]]], [[[6.0, 7.0, 7.0], 
[5.0, 8.0, 6.0], [2.0, 1.0, 4.0]], [[5.0, 4.0, 2.0], [3.0, 3.0, 2.0], [6.0, 1.0, 2.0]], [[4.0, 4.0, 8.0], [2.0, 7.0, 4.0], 
[5.0, 7.0, 7.0]]], [[[8.0, 7.0, 6.0], [8.0, 4.0, 5.0], [3.0, 4.0, 7.0]], [[6.0, 4.0, 1.0], [1.0, 2.0, 2.0], [4.0, 4.0, 8.0]], 
[[8.0, 2.0, 5.0], [7.0, 3.0, 2.0], [4.0, 8.0, 1.0]]], [[[7.0, 5.0, 4.0], [6.0, 7.0, 5.0], [2.0, 3.0, 6.0]], [[6.0, 7.0, 3.0], 
[2.0, 8.0, 4.0], [6.0, 3.0, 1.0]], [[2.0, 4.0, 6.0], [3.0, 5.0, 4.0], [2.0, 1.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 1.0, 4.0], [2.0, 8.0, 4.0], [3.0, 2.0, 5.0]], [[6.0, 7.0, 4.0], [5.0, 4.0, 6.0], [4.0, 
8.0, 4.0]], [[4.0, 7.0, 4.0], [4.0, 6.0, 8.0], [7.0, 8.0, 5.0]]], [[[3.0, 3.0, 2.0], [5.0, 5.0, 7.0], [6.0, 8.0, 8.0]], [[3.0, 
6.0, 8.0], [5.0, 6.0, 7.0], [3.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 5.0, 2.0], [2.0, 2.0, 4.0]]], [[[3.0, 6.0, 3.0], [1.0, 
4.0, 3.0], [2.0, 6.0, 4.0]], [[6.0, 2.0, 6.0], [4.0, 3.0, 1.0], [1.0, 2.0, 8.0]], [[3.0, 4.0, 2.0], [6.0, 6.0, 8.0], [4.0, 2.0, 
2.0]]], [[[7.0, 2.0, 3.0], [7.0, 5.0, 5.0], [4.0, 3.0, 5.0]], [[8.0, 2.0, 1.0], [5.0, 1.0, 5.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 
6.0], [1.0, 3.0, 6.0], [1.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 8.0], [5.0, 6.0, 2.0], [3.0, 3.0, 2.0]], 
[[2.0, 3.0, 5.0], [8.0, 6.0, 2.0], [5.0, 7.0, 1.0]], [[6.0, 7.0, 3.0], [5.0, 4.0, 3.0], [5.0, 4.0, 7.0]]], [[[1.0, 5.0, 7.0], 
[7.0, 7.0, 7.0], [1.0, 5.0, 1.0]], [[1.0, 5.0, 2.0], [1.0, 5.0, 5.0], [1.0, 6.0, 1.0]], [[5.0, 6.0, 7.0], [8.0, 6.0, 3.0], 
[6.0, 5.0, 8.0]]], [[[8.0, 4.0, 7.0], [5.0, 8.0, 8.0], [1.0, 6.0, 2.0]], [[2.0, 4.0, 7.0], [6.0, 1.0, 1.0], [4.0, 5.0, 1.0]], 
[[4.0, 1.0, 5.0], [4.0, 2.0, 6.0], [7.0, 4.0, 5.0]]], [[[2.0, 3.0, 3.0], [8.0, 2.0, 8.0], [6.0, 5.0, 5.0]], [[6.0, 4.0, 7.0], 
[5.0, 2.0, 6.0], [8.0, 7.0, 4.0]], [[4.0, 8.0, 6.0], [7.0, 7.0, 3.0], [5.0, 5.0, 5.0]]]])
      val=Data(numpy.array([[[[8.0, 5.0, 7.0]], [[2.0, 1.0, 8.0]], [[2.0, 5.0, 6.0]]], [[[8.0, 3.0, 3.0]], [[3.0, 3.0, 7.0]], 
[[6.0, 8.0, 4.0]]], [[[3.0, 5.0, 4.0]], [[7.0, 2.0, 3.0]], [[7.0, 6.0, 4.0]]]]),self.functionspace)
      arg[:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 1.0, 4.0], [8.0, 5.0, 7.0], [3.0, 2.0, 5.0]], [[6.0, 7.0, 4.0], [2.0, 1.0, 8.0], [4.0, 
8.0, 4.0]], [[4.0, 7.0, 4.0], [2.0, 5.0, 6.0], [7.0, 8.0, 5.0]]], [[[3.0, 3.0, 2.0], [8.0, 3.0, 3.0], [6.0, 8.0, 8.0]], [[3.0, 
6.0, 8.0], [3.0, 3.0, 7.0], [3.0, 5.0, 3.0]], [[7.0, 3.0, 2.0], [6.0, 8.0, 4.0], [2.0, 2.0, 4.0]]], [[[3.0, 6.0, 3.0], [3.0, 
5.0, 4.0], [2.0, 6.0, 4.0]], [[6.0, 2.0, 6.0], [7.0, 2.0, 3.0], [1.0, 2.0, 8.0]], [[3.0, 4.0, 2.0], [7.0, 6.0, 4.0], [4.0, 2.0, 
2.0]]], [[[7.0, 2.0, 3.0], [7.0, 5.0, 5.0], [4.0, 3.0, 5.0]], [[8.0, 2.0, 1.0], [5.0, 1.0, 5.0], [3.0, 5.0, 5.0]], [[2.0, 2.0, 
6.0], [1.0, 3.0, 6.0], [1.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 2.0, 8.0], [8.0, 5.0, 7.0], [3.0, 3.0, 2.0]], 
[[2.0, 3.0, 5.0], [2.0, 1.0, 8.0], [5.0, 7.0, 1.0]], [[6.0, 7.0, 3.0], [2.0, 5.0, 6.0], [5.0, 4.0, 7.0]]], [[[1.0, 5.0, 7.0], 
[8.0, 3.0, 3.0], [1.0, 5.0, 1.0]], [[1.0, 5.0, 2.0], [3.0, 3.0, 7.0], [1.0, 6.0, 1.0]], [[5.0, 6.0, 7.0], [6.0, 8.0, 4.0], 
[6.0, 5.0, 8.0]]], [[[8.0, 4.0, 7.0], [3.0, 5.0, 4.0], [1.0, 6.0, 2.0]], [[2.0, 4.0, 7.0], [7.0, 2.0, 3.0], [4.0, 5.0, 1.0]], 
[[4.0, 1.0, 5.0], [7.0, 6.0, 4.0], [7.0, 4.0, 5.0]]], [[[2.0, 3.0, 3.0], [8.0, 2.0, 8.0], [6.0, 5.0, 5.0]], [[6.0, 4.0, 7.0], 
[5.0, 2.0, 6.0], [8.0, 7.0, 4.0]], [[4.0, 8.0, 6.0], [7.0, 7.0, 3.0], [5.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UCRN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 5.0, 1.0], [4.0, 3.0, 3.0], [4.0, 3.0, 8.0]], [[6.0, 5.0, 7.0], [3.0, 1.0, 4.0], [8.0, 
8.0, 7.0]], [[7.0, 8.0, 2.0], [7.0, 6.0, 3.0], [4.0, 1.0, 6.0]]], [[[4.0, 3.0, 2.0], [3.0, 6.0, 2.0], [1.0, 2.0, 1.0]], [[8.0, 
8.0, 1.0], [6.0, 8.0, 5.0], [6.0, 5.0, 4.0]], [[4.0, 4.0, 2.0], [1.0, 1.0, 7.0], [8.0, 2.0, 3.0]]], [[[2.0, 1.0, 8.0], [4.0, 
3.0, 5.0], [2.0, 1.0, 8.0]], [[3.0, 5.0, 3.0], [3.0, 8.0, 3.0], [4.0, 7.0, 2.0]], [[5.0, 7.0, 5.0], [6.0, 7.0, 6.0], [8.0, 6.0, 
1.0]]], [[[8.0, 2.0, 1.0], [6.0, 6.0, 6.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 3.0], [7.0, 7.0, 5.0], [8.0, 8.0, 7.0]], [[2.0, 1.0, 
5.0], [3.0, 4.0, 5.0], [7.0, 2.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 3.0, 1.0], [3.0, 2.0, 3.0], [7.0, 1.0, 5.0]], 
[[4.0, 2.0, 1.0], [6.0, 1.0, 8.0], [4.0, 4.0, 8.0]], [[8.0, 1.0, 1.0], [1.0, 8.0, 1.0], [6.0, 7.0, 3.0]]], [[[7.0, 4.0, 6.0], 
[3.0, 7.0, 3.0], [3.0, 6.0, 7.0]], [[1.0, 1.0, 4.0], [1.0, 4.0, 2.0], [7.0, 2.0, 3.0]], [[6.0, 6.0, 8.0], [1.0, 2.0, 1.0], 
[7.0, 6.0, 3.0]]], [[[4.0, 3.0, 7.0], [1.0, 3.0, 8.0], [1.0, 7.0, 6.0]], [[3.0, 7.0, 8.0], [2.0, 3.0, 6.0], [7.0, 3.0, 4.0]], 
[[7.0, 1.0, 7.0], [6.0, 1.0, 3.0], [7.0, 3.0, 1.0]]], [[[2.0, 5.0, 2.0], [4.0, 5.0, 4.0], [5.0, 3.0, 7.0]], [[7.0, 1.0, 3.0], 
[7.0, 7.0, 8.0], [7.0, 6.0, 3.0]], [[6.0, 1.0, 2.0], [8.0, 3.0, 4.0], [1.0, 5.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[6.0, 6.0, 8.0]], [[8.0, 7.0, 6.0]], [[8.0, 4.0, 5.0]]], [[[6.0, 2.0, 8.0]], [[3.0, 8.0, 
3.0]], [[6.0, 5.0, 8.0]]], [[[6.0, 1.0, 6.0]], [[5.0, 3.0, 7.0]], [[3.0, 2.0, 2.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 1.0, 
1.0]], [[1.0, 7.0, 2.0]], [[4.0, 6.0, 7.0]]], [[[3.0, 4.0, 3.0]], [[1.0, 2.0, 1.0]], [[2.0, 5.0, 8.0]]], [[[7.0, 6.0, 8.0]], 
[[6.0, 1.0, 4.0]], [[1.0, 6.0, 5.0]]]])
      arg[:3,:,1:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 5.0, 1.0], [6.0, 6.0, 8.0], [4.0, 3.0, 8.0]], [[6.0, 5.0, 7.0], [8.0, 7.0, 6.0], [8.0, 
8.0, 7.0]], [[7.0, 8.0, 2.0], [8.0, 4.0, 5.0], [4.0, 1.0, 6.0]]], [[[4.0, 3.0, 2.0], [6.0, 2.0, 8.0], [1.0, 2.0, 1.0]], [[8.0, 
8.0, 1.0], [3.0, 8.0, 3.0], [6.0, 5.0, 4.0]], [[4.0, 4.0, 2.0], [6.0, 5.0, 8.0], [8.0, 2.0, 3.0]]], [[[2.0, 1.0, 8.0], [6.0, 
1.0, 6.0], [2.0, 1.0, 8.0]], [[3.0, 5.0, 3.0], [5.0, 3.0, 7.0], [4.0, 7.0, 2.0]], [[5.0, 7.0, 5.0], [3.0, 2.0, 2.0], [8.0, 6.0, 
1.0]]], [[[8.0, 2.0, 1.0], [6.0, 6.0, 6.0], [5.0, 5.0, 1.0]], [[5.0, 1.0, 3.0], [7.0, 7.0, 5.0], [8.0, 8.0, 7.0]], [[2.0, 1.0, 
5.0], [3.0, 4.0, 5.0], [7.0, 2.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 3.0, 1.0], [7.0, 1.0, 1.0], [7.0, 1.0, 5.0]], 
[[4.0, 2.0, 1.0], [1.0, 7.0, 2.0], [4.0, 4.0, 8.0]], [[8.0, 1.0, 1.0], [4.0, 6.0, 7.0], [6.0, 7.0, 3.0]]], [[[7.0, 4.0, 6.0], 
[3.0, 4.0, 3.0], [3.0, 6.0, 7.0]], [[1.0, 1.0, 4.0], [1.0, 2.0, 1.0], [7.0, 2.0, 3.0]], [[6.0, 6.0, 8.0], [2.0, 5.0, 8.0], 
[7.0, 6.0, 3.0]]], [[[4.0, 3.0, 7.0], [7.0, 6.0, 8.0], [1.0, 7.0, 6.0]], [[3.0, 7.0, 8.0], [6.0, 1.0, 4.0], [7.0, 3.0, 4.0]], 
[[7.0, 1.0, 7.0], [1.0, 6.0, 5.0], [7.0, 3.0, 1.0]]], [[[2.0, 5.0, 2.0], [4.0, 5.0, 4.0], [5.0, 3.0, 7.0]], [[7.0, 1.0, 3.0], 
[7.0, 7.0, 8.0], [7.0, 6.0, 3.0]], [[6.0, 1.0, 2.0], [8.0, 3.0, 4.0], [1.0, 5.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 3.0, 6.0], [5.0, 4.0, 7.0], [8.0, 1.0, 8.0]], [[4.0, 5.0, 7.0], [7.0, 2.0, 4.0], [8.0, 
5.0, 1.0]], [[2.0, 1.0, 7.0], [4.0, 3.0, 8.0], [6.0, 5.0, 8.0]]], [[[1.0, 2.0, 8.0], [1.0, 8.0, 5.0], [4.0, 3.0, 5.0]], [[8.0, 
8.0, 4.0], [3.0, 1.0, 2.0], [5.0, 3.0, 5.0]], [[3.0, 2.0, 2.0], [5.0, 7.0, 2.0], [6.0, 2.0, 4.0]]], [[[5.0, 6.0, 1.0], [1.0, 
3.0, 2.0], [7.0, 8.0, 3.0]], [[2.0, 1.0, 8.0], [6.0, 2.0, 2.0], [5.0, 1.0, 7.0]], [[3.0, 8.0, 2.0], [2.0, 4.0, 2.0], [1.0, 2.0, 
2.0]]], [[[5.0, 3.0, 5.0], [1.0, 4.0, 5.0], [6.0, 3.0, 7.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 
5.0], [2.0, 8.0, 1.0], [7.0, 8.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 1.0], [5.0, 6.0, 8.0], [7.0, 2.0, 7.0]], 
[[1.0, 4.0, 2.0], [3.0, 5.0, 6.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 6.0], [5.0, 5.0, 1.0]]], [[[2.0, 6.0, 2.0], 
[8.0, 8.0, 8.0], [7.0, 3.0, 7.0]], [[1.0, 6.0, 3.0], [3.0, 3.0, 5.0], [4.0, 2.0, 3.0]], [[6.0, 8.0, 5.0], [3.0, 5.0, 7.0], 
[6.0, 6.0, 2.0]]], [[[5.0, 7.0, 6.0], [5.0, 1.0, 2.0], [7.0, 8.0, 6.0]], [[6.0, 4.0, 6.0], [1.0, 7.0, 8.0], [5.0, 2.0, 5.0]], 
[[5.0, 2.0, 6.0], [3.0, 3.0, 6.0], [6.0, 7.0, 4.0]]], [[[4.0, 3.0, 7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 8.0], 
[8.0, 5.0, 2.0], [7.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [4.0, 2.0, 4.0], [3.0, 4.0, 5.0]]]])
      val=numpy.array([[[8.0, 1.0, 4.0]], [[8.0, 1.0, 3.0]], [[4.0, 1.0, 4.0]]])
      arg[2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 3.0, 6.0], [5.0, 4.0, 7.0], [8.0, 1.0, 8.0]], [[4.0, 5.0, 7.0], [7.0, 2.0, 4.0], [8.0, 
5.0, 1.0]], [[2.0, 1.0, 7.0], [4.0, 3.0, 8.0], [6.0, 5.0, 8.0]]], [[[1.0, 2.0, 8.0], [1.0, 8.0, 5.0], [4.0, 3.0, 5.0]], [[8.0, 
8.0, 4.0], [3.0, 1.0, 2.0], [5.0, 3.0, 5.0]], [[3.0, 2.0, 2.0], [5.0, 7.0, 2.0], [6.0, 2.0, 4.0]]], [[[5.0, 6.0, 1.0], [1.0, 
3.0, 2.0], [8.0, 1.0, 4.0]], [[2.0, 1.0, 8.0], [6.0, 2.0, 2.0], [8.0, 1.0, 3.0]], [[3.0, 8.0, 2.0], [2.0, 4.0, 2.0], [4.0, 1.0, 
4.0]]], [[[5.0, 3.0, 5.0], [1.0, 4.0, 5.0], [6.0, 3.0, 7.0]], [[1.0, 5.0, 8.0], [5.0, 6.0, 5.0], [8.0, 1.0, 8.0]], [[1.0, 8.0, 
5.0], [2.0, 8.0, 1.0], [7.0, 8.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 1.0], [5.0, 6.0, 8.0], [7.0, 2.0, 7.0]], 
[[1.0, 4.0, 2.0], [3.0, 5.0, 6.0], [6.0, 6.0, 1.0]], [[2.0, 1.0, 3.0], [3.0, 4.0, 6.0], [5.0, 5.0, 1.0]]], [[[2.0, 6.0, 2.0], 
[8.0, 8.0, 8.0], [7.0, 3.0, 7.0]], [[1.0, 6.0, 3.0], [3.0, 3.0, 5.0], [4.0, 2.0, 3.0]], [[6.0, 8.0, 5.0], [3.0, 5.0, 7.0], 
[6.0, 6.0, 2.0]]], [[[5.0, 7.0, 6.0], [5.0, 1.0, 2.0], [8.0, 1.0, 4.0]], [[6.0, 4.0, 6.0], [1.0, 7.0, 8.0], [8.0, 1.0, 3.0]], 
[[5.0, 2.0, 6.0], [3.0, 3.0, 6.0], [4.0, 1.0, 4.0]]], [[[4.0, 3.0, 7.0], [6.0, 1.0, 7.0], [8.0, 8.0, 6.0]], [[2.0, 3.0, 8.0], 
[8.0, 5.0, 2.0], [7.0, 4.0, 8.0]], [[3.0, 5.0, 3.0], [4.0, 2.0, 4.0], [3.0, 4.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 7.0, 4.0], [4.0, 5.0, 5.0], [1.0, 8.0, 5.0]], [[1.0, 2.0, 7.0], [1.0, 6.0, 1.0], [7.0, 
2.0, 8.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 4.0], [5.0, 5.0, 7.0]]], [[[6.0, 6.0, 8.0], [1.0, 6.0, 4.0], [6.0, 1.0, 4.0]], [[2.0, 
4.0, 4.0], [5.0, 7.0, 2.0], [1.0, 5.0, 4.0]], [[2.0, 4.0, 4.0], [6.0, 3.0, 7.0], [3.0, 8.0, 6.0]]], [[[1.0, 2.0, 1.0], [2.0, 
1.0, 7.0], [4.0, 3.0, 8.0]], [[5.0, 6.0, 6.0], [2.0, 5.0, 6.0], [4.0, 3.0, 4.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 7.0], [6.0, 3.0, 
3.0]]], [[[8.0, 1.0, 4.0], [3.0, 6.0, 3.0], [8.0, 4.0, 6.0]], [[4.0, 4.0, 1.0], [3.0, 6.0, 5.0], [5.0, 6.0, 8.0]], [[2.0, 5.0, 
3.0], [3.0, 7.0, 4.0], [3.0, 8.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 1.0, 5.0], [5.0, 5.0, 2.0], [2.0, 1.0, 2.0]], 
[[8.0, 8.0, 4.0], [8.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[7.0, 2.0, 4.0], [3.0, 3.0, 5.0], [8.0, 4.0, 7.0]]], [[[3.0, 4.0, 4.0], 
[1.0, 2.0, 3.0], [4.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [1.0, 2.0, 4.0], [8.0, 2.0, 7.0]], [[1.0, 2.0, 2.0], [2.0, 8.0, 4.0], 
[8.0, 8.0, 1.0]]], [[[8.0, 6.0, 4.0], [8.0, 7.0, 3.0], [1.0, 3.0, 6.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 3.0], [4.0, 2.0, 4.0]], 
[[6.0, 4.0, 1.0], [1.0, 1.0, 4.0], [1.0, 4.0, 5.0]]], [[[8.0, 8.0, 3.0], [3.0, 7.0, 5.0], [3.0, 6.0, 5.0]], [[8.0, 8.0, 7.0], 
[3.0, 2.0, 2.0], [3.0, 6.0, 3.0]], [[8.0, 1.0, 8.0], [7.0, 7.0, 8.0], [5.0, 8.0, 8.0]]]])
      val=Data(numpy.array([[[5.0, 6.0, 5.0]], [[4.0, 2.0, 6.0]], [[2.0, 4.0, 5.0]]]),self.functionspace)
      arg[2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 7.0, 4.0], [4.0, 5.0, 5.0], [1.0, 8.0, 5.0]], [[1.0, 2.0, 7.0], [1.0, 6.0, 1.0], [7.0, 
2.0, 8.0]], [[4.0, 6.0, 6.0], [7.0, 6.0, 4.0], [5.0, 5.0, 7.0]]], [[[6.0, 6.0, 8.0], [1.0, 6.0, 4.0], [6.0, 1.0, 4.0]], [[2.0, 
4.0, 4.0], [5.0, 7.0, 2.0], [1.0, 5.0, 4.0]], [[2.0, 4.0, 4.0], [6.0, 3.0, 7.0], [3.0, 8.0, 6.0]]], [[[1.0, 2.0, 1.0], [2.0, 
1.0, 7.0], [5.0, 6.0, 5.0]], [[5.0, 6.0, 6.0], [2.0, 5.0, 6.0], [4.0, 2.0, 6.0]], [[3.0, 6.0, 5.0], [4.0, 8.0, 7.0], [2.0, 4.0, 
5.0]]], [[[8.0, 1.0, 4.0], [3.0, 6.0, 3.0], [8.0, 4.0, 6.0]], [[4.0, 4.0, 1.0], [3.0, 6.0, 5.0], [5.0, 6.0, 8.0]], [[2.0, 5.0, 
3.0], [3.0, 7.0, 4.0], [3.0, 8.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 1.0, 5.0], [5.0, 5.0, 2.0], [2.0, 1.0, 2.0]], 
[[8.0, 8.0, 4.0], [8.0, 6.0, 5.0], [5.0, 3.0, 6.0]], [[7.0, 2.0, 4.0], [3.0, 3.0, 5.0], [8.0, 4.0, 7.0]]], [[[3.0, 4.0, 4.0], 
[1.0, 2.0, 3.0], [4.0, 1.0, 3.0]], [[6.0, 4.0, 8.0], [1.0, 2.0, 4.0], [8.0, 2.0, 7.0]], [[1.0, 2.0, 2.0], [2.0, 8.0, 4.0], 
[8.0, 8.0, 1.0]]], [[[8.0, 6.0, 4.0], [8.0, 7.0, 3.0], [5.0, 6.0, 5.0]], [[5.0, 3.0, 7.0], [6.0, 7.0, 3.0], [4.0, 2.0, 6.0]], 
[[6.0, 4.0, 1.0], [1.0, 1.0, 4.0], [2.0, 4.0, 5.0]]], [[[8.0, 8.0, 3.0], [3.0, 7.0, 5.0], [3.0, 6.0, 5.0]], [[8.0, 8.0, 7.0], 
[3.0, 2.0, 2.0], [3.0, 6.0, 3.0]], [[8.0, 1.0, 8.0], [7.0, 7.0, 8.0], [5.0, 8.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PCLN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 4.0, 2.0], [4.0, 2.0, 3.0], [4.0, 4.0, 6.0]], [[1.0, 3.0, 8.0], [8.0, 5.0, 2.0], [6.0, 
1.0, 2.0]], [[6.0, 2.0, 2.0], [6.0, 7.0, 3.0], [7.0, 6.0, 8.0]]], [[[1.0, 2.0, 4.0], [3.0, 6.0, 5.0], [2.0, 1.0, 5.0]], [[7.0, 
2.0, 4.0], [6.0, 5.0, 5.0], [3.0, 4.0, 3.0]], [[6.0, 4.0, 5.0], [2.0, 6.0, 7.0], [5.0, 2.0, 3.0]]], [[[7.0, 4.0, 1.0], [8.0, 
6.0, 3.0], [4.0, 7.0, 5.0]], [[4.0, 1.0, 3.0], [1.0, 6.0, 7.0], [5.0, 1.0, 5.0]], [[7.0, 4.0, 1.0], [8.0, 8.0, 1.0], [4.0, 7.0, 
7.0]]], [[[1.0, 1.0, 4.0], [6.0, 4.0, 5.0], [1.0, 5.0, 4.0]], [[5.0, 3.0, 6.0], [4.0, 3.0, 1.0], [1.0, 4.0, 6.0]], [[6.0, 4.0, 
2.0], [7.0, 3.0, 3.0], [3.0, 8.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 6.0, 1.0], [7.0, 3.0, 3.0]], 
[[1.0, 8.0, 7.0], [2.0, 3.0, 7.0], [4.0, 5.0, 7.0]], [[2.0, 7.0, 3.0], [1.0, 6.0, 3.0], [4.0, 3.0, 1.0]]], [[[2.0, 2.0, 7.0], 
[7.0, 4.0, 2.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 3.0], [1.0, 7.0, 3.0], [3.0, 5.0, 7.0]], [[4.0, 2.0, 1.0], [1.0, 7.0, 2.0], 
[2.0, 4.0, 7.0]]], [[[1.0, 4.0, 3.0], [8.0, 4.0, 1.0], [1.0, 2.0, 6.0]], [[3.0, 4.0, 5.0], [6.0, 2.0, 7.0], [8.0, 1.0, 4.0]], 
[[5.0, 5.0, 1.0], [5.0, 5.0, 6.0], [2.0, 5.0, 4.0]]], [[[3.0, 3.0, 5.0], [8.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[4.0, 7.0, 2.0], 
[1.0, 3.0, 2.0], [5.0, 4.0, 3.0]], [[7.0, 3.0, 4.0], [3.0, 6.0, 7.0], [6.0, 6.0, 7.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 8.0, 6.0]], [[4.0, 7.0, 4.0]], [[1.0, 1.0, 6.0]]])+(1.-msk_val)*numpy.array([[[7.0, 6.0, 
5.0]], [[4.0, 4.0, 2.0]], [[5.0, 2.0, 7.0]]])
      arg[2,:,2:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 4.0, 2.0], [4.0, 2.0, 3.0], [4.0, 4.0, 6.0]], [[1.0, 3.0, 8.0], [8.0, 5.0, 2.0], [6.0, 
1.0, 2.0]], [[6.0, 2.0, 2.0], [6.0, 7.0, 3.0], [7.0, 6.0, 8.0]]], [[[1.0, 2.0, 4.0], [3.0, 6.0, 5.0], [2.0, 1.0, 5.0]], [[7.0, 
2.0, 4.0], [6.0, 5.0, 5.0], [3.0, 4.0, 3.0]], [[6.0, 4.0, 5.0], [2.0, 6.0, 7.0], [5.0, 2.0, 3.0]]], [[[7.0, 4.0, 1.0], [8.0, 
6.0, 3.0], [4.0, 8.0, 6.0]], [[4.0, 1.0, 3.0], [1.0, 6.0, 7.0], [4.0, 7.0, 4.0]], [[7.0, 4.0, 1.0], [8.0, 8.0, 1.0], [1.0, 1.0, 
6.0]]], [[[1.0, 1.0, 4.0], [6.0, 4.0, 5.0], [1.0, 5.0, 4.0]], [[5.0, 3.0, 6.0], [4.0, 3.0, 1.0], [1.0, 4.0, 6.0]], [[6.0, 4.0, 
2.0], [7.0, 3.0, 3.0], [3.0, 8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 4.0, 3.0], [8.0, 6.0, 1.0], [7.0, 3.0, 3.0]], 
[[1.0, 8.0, 7.0], [2.0, 3.0, 7.0], [4.0, 5.0, 7.0]], [[2.0, 7.0, 3.0], [1.0, 6.0, 3.0], [4.0, 3.0, 1.0]]], [[[2.0, 2.0, 7.0], 
[7.0, 4.0, 2.0], [4.0, 4.0, 8.0]], [[7.0, 7.0, 3.0], [1.0, 7.0, 3.0], [3.0, 5.0, 7.0]], [[4.0, 2.0, 1.0], [1.0, 7.0, 2.0], 
[2.0, 4.0, 7.0]]], [[[1.0, 4.0, 3.0], [8.0, 4.0, 1.0], [7.0, 6.0, 5.0]], [[3.0, 4.0, 5.0], [6.0, 2.0, 7.0], [4.0, 4.0, 2.0]], 
[[5.0, 5.0, 1.0], [5.0, 5.0, 6.0], [5.0, 2.0, 7.0]]], [[[3.0, 3.0, 5.0], [8.0, 7.0, 1.0], [5.0, 3.0, 7.0]], [[4.0, 7.0, 2.0], 
[1.0, 3.0, 2.0], [5.0, 4.0, 3.0]], [[7.0, 3.0, 4.0], [3.0, 6.0, 7.0], [6.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_LRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 6.0, 7.0], [2.0, 4.0, 5.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [5.0, 5.0, 6.0], [4.0, 
2.0, 5.0]], [[4.0, 8.0, 1.0], [7.0, 7.0, 8.0], [4.0, 2.0, 2.0]]], [[[5.0, 7.0, 8.0], [4.0, 8.0, 7.0], [8.0, 4.0, 2.0]], [[8.0, 
7.0, 7.0], [5.0, 8.0, 3.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 1.0], [5.0, 8.0, 6.0], [2.0, 3.0, 8.0]]], [[[2.0, 8.0, 6.0], [6.0, 
1.0, 1.0], [6.0, 4.0, 2.0]], [[2.0, 4.0, 1.0], [1.0, 1.0, 4.0], [5.0, 1.0, 6.0]], [[6.0, 4.0, 8.0], [6.0, 7.0, 2.0], [5.0, 4.0, 
2.0]]], [[[8.0, 4.0, 6.0], [2.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[1.0, 1.0, 5.0], [5.0, 3.0, 5.0], [2.0, 8.0, 4.0]], [[6.0, 1.0, 
8.0], [1.0, 6.0, 8.0], [3.0, 2.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 1.0, 6.0], [7.0, 3.0, 8.0], [7.0, 7.0, 3.0]], 
[[5.0, 2.0, 4.0], [2.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[2.0, 3.0, 3.0], [2.0, 7.0, 5.0], [5.0, 3.0, 6.0]]], [[[1.0, 8.0, 8.0], 
[3.0, 5.0, 8.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 8.0], [7.0, 1.0, 6.0], [2.0, 7.0, 4.0]], [[3.0, 8.0, 8.0], [5.0, 2.0, 2.0], 
[2.0, 5.0, 3.0]]], [[[2.0, 1.0, 4.0], [8.0, 5.0, 8.0], [8.0, 1.0, 3.0]], [[8.0, 7.0, 7.0], [6.0, 2.0, 7.0], [4.0, 8.0, 4.0]], 
[[2.0, 3.0, 3.0], [4.0, 6.0, 8.0], [4.0, 5.0, 3.0]]], [[[6.0, 5.0, 6.0], [3.0, 5.0, 1.0], [1.0, 6.0, 6.0]], [[7.0, 7.0, 3.0], 
[2.0, 3.0, 3.0], [6.0, 7.0, 7.0]], [[7.0, 1.0, 8.0], [1.0, 4.0, 5.0], [8.0, 8.0, 4.0]]]])
      val=numpy.array([[[[7.0, 1.0, 7.0], [1.0, 6.0, 6.0]]], [[[7.0, 2.0, 3.0], [1.0, 5.0, 8.0]]]])
      arg[2:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 6.0, 7.0], [2.0, 4.0, 5.0], [3.0, 5.0, 7.0]], [[4.0, 3.0, 2.0], [5.0, 5.0, 6.0], [4.0, 
2.0, 5.0]], [[4.0, 8.0, 1.0], [7.0, 7.0, 8.0], [4.0, 2.0, 2.0]]], [[[5.0, 7.0, 8.0], [4.0, 8.0, 7.0], [8.0, 4.0, 2.0]], [[8.0, 
7.0, 7.0], [5.0, 8.0, 3.0], [2.0, 3.0, 2.0]], [[2.0, 8.0, 1.0], [5.0, 8.0, 6.0], [2.0, 3.0, 8.0]]], [[[2.0, 8.0, 6.0], [6.0, 
1.0, 1.0], [6.0, 4.0, 2.0]], [[7.0, 1.0, 7.0], [1.0, 6.0, 6.0], [5.0, 1.0, 6.0]], [[6.0, 4.0, 8.0], [6.0, 7.0, 2.0], [5.0, 4.0, 
2.0]]], [[[8.0, 4.0, 6.0], [2.0, 4.0, 7.0], [4.0, 5.0, 6.0]], [[7.0, 2.0, 3.0], [1.0, 5.0, 8.0], [2.0, 8.0, 4.0]], [[6.0, 1.0, 
8.0], [1.0, 6.0, 8.0], [3.0, 2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 1.0, 6.0], [7.0, 3.0, 8.0], [7.0, 7.0, 3.0]], 
[[5.0, 2.0, 4.0], [2.0, 8.0, 5.0], [5.0, 8.0, 4.0]], [[2.0, 3.0, 3.0], [2.0, 7.0, 5.0], [5.0, 3.0, 6.0]]], [[[1.0, 8.0, 8.0], 
[3.0, 5.0, 8.0], [5.0, 4.0, 2.0]], [[6.0, 3.0, 8.0], [7.0, 1.0, 6.0], [2.0, 7.0, 4.0]], [[3.0, 8.0, 8.0], [5.0, 2.0, 2.0], 
[2.0, 5.0, 3.0]]], [[[2.0, 1.0, 4.0], [8.0, 5.0, 8.0], [8.0, 1.0, 3.0]], [[7.0, 1.0, 7.0], [1.0, 6.0, 6.0], [4.0, 8.0, 4.0]], 
[[2.0, 3.0, 3.0], [4.0, 6.0, 8.0], [4.0, 5.0, 3.0]]], [[[6.0, 5.0, 6.0], [3.0, 5.0, 1.0], [1.0, 6.0, 6.0]], [[7.0, 2.0, 3.0], 
[1.0, 5.0, 8.0], [6.0, 7.0, 7.0]], [[7.0, 1.0, 8.0], [1.0, 4.0, 5.0], [8.0, 8.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_LRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 2.0, 1.0], [5.0, 8.0, 6.0], [3.0, 6.0, 5.0]], [[5.0, 2.0, 2.0], [1.0, 1.0, 8.0], [8.0, 
5.0, 5.0]], [[7.0, 2.0, 5.0], [7.0, 3.0, 1.0], [1.0, 3.0, 4.0]]], [[[6.0, 6.0, 1.0], [4.0, 6.0, 8.0], [4.0, 6.0, 2.0]], [[3.0, 
1.0, 2.0], [7.0, 6.0, 5.0], [4.0, 1.0, 6.0]], [[8.0, 2.0, 2.0], [8.0, 1.0, 1.0], [4.0, 7.0, 8.0]]], [[[6.0, 4.0, 7.0], [5.0, 
8.0, 8.0], [5.0, 4.0, 2.0]], [[7.0, 5.0, 3.0], [1.0, 2.0, 2.0], [8.0, 7.0, 7.0]], [[4.0, 1.0, 6.0], [1.0, 1.0, 7.0], [8.0, 3.0, 
8.0]]], [[[6.0, 8.0, 5.0], [1.0, 2.0, 2.0], [5.0, 3.0, 6.0]], [[4.0, 2.0, 4.0], [7.0, 8.0, 1.0], [8.0, 1.0, 1.0]], [[7.0, 7.0, 
1.0], [7.0, 1.0, 1.0], [5.0, 7.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 2.0, 2.0], [1.0, 4.0, 4.0]], 
[[5.0, 8.0, 2.0], [7.0, 8.0, 5.0], [6.0, 5.0, 4.0]], [[6.0, 3.0, 5.0], [2.0, 8.0, 7.0], [4.0, 6.0, 3.0]]], [[[5.0, 5.0, 6.0], 
[7.0, 6.0, 5.0], [5.0, 7.0, 8.0]], [[5.0, 2.0, 5.0], [5.0, 8.0, 3.0], [5.0, 5.0, 3.0]], [[4.0, 8.0, 3.0], [2.0, 2.0, 5.0], 
[1.0, 4.0, 8.0]]], [[[6.0, 8.0, 3.0], [4.0, 1.0, 5.0], [8.0, 2.0, 1.0]], [[5.0, 1.0, 4.0], [4.0, 5.0, 5.0], [4.0, 1.0, 7.0]], 
[[4.0, 5.0, 8.0], [2.0, 1.0, 7.0], [2.0, 7.0, 3.0]]], [[[8.0, 5.0, 7.0], [1.0, 6.0, 6.0], [5.0, 5.0, 7.0]], [[4.0, 7.0, 7.0], 
[3.0, 6.0, 8.0], [4.0, 5.0, 2.0]], [[3.0, 3.0, 5.0], [5.0, 4.0, 4.0], [2.0, 7.0, 7.0]]]])
      val=Data(numpy.array([[[[4.0, 8.0, 8.0], [5.0, 8.0, 3.0]]], [[[2.0, 3.0, 1.0], [1.0, 7.0, 3.0]]]]),self.functionspace)
      arg[2:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 2.0, 1.0], [5.0, 8.0, 6.0], [3.0, 6.0, 5.0]], [[5.0, 2.0, 2.0], [1.0, 1.0, 8.0], [8.0, 
5.0, 5.0]], [[7.0, 2.0, 5.0], [7.0, 3.0, 1.0], [1.0, 3.0, 4.0]]], [[[6.0, 6.0, 1.0], [4.0, 6.0, 8.0], [4.0, 6.0, 2.0]], [[3.0, 
1.0, 2.0], [7.0, 6.0, 5.0], [4.0, 1.0, 6.0]], [[8.0, 2.0, 2.0], [8.0, 1.0, 1.0], [4.0, 7.0, 8.0]]], [[[6.0, 4.0, 7.0], [5.0, 
8.0, 8.0], [5.0, 4.0, 2.0]], [[4.0, 8.0, 8.0], [5.0, 8.0, 3.0], [8.0, 7.0, 7.0]], [[4.0, 1.0, 6.0], [1.0, 1.0, 7.0], [8.0, 3.0, 
8.0]]], [[[6.0, 8.0, 5.0], [1.0, 2.0, 2.0], [5.0, 3.0, 6.0]], [[2.0, 3.0, 1.0], [1.0, 7.0, 3.0], [8.0, 1.0, 1.0]], [[7.0, 7.0, 
1.0], [7.0, 1.0, 1.0], [5.0, 7.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 7.0, 8.0], [8.0, 2.0, 2.0], [1.0, 4.0, 4.0]], 
[[5.0, 8.0, 2.0], [7.0, 8.0, 5.0], [6.0, 5.0, 4.0]], [[6.0, 3.0, 5.0], [2.0, 8.0, 7.0], [4.0, 6.0, 3.0]]], [[[5.0, 5.0, 6.0], 
[7.0, 6.0, 5.0], [5.0, 7.0, 8.0]], [[5.0, 2.0, 5.0], [5.0, 8.0, 3.0], [5.0, 5.0, 3.0]], [[4.0, 8.0, 3.0], [2.0, 2.0, 5.0], 
[1.0, 4.0, 8.0]]], [[[6.0, 8.0, 3.0], [4.0, 1.0, 5.0], [8.0, 2.0, 1.0]], [[4.0, 8.0, 8.0], [5.0, 8.0, 3.0], [4.0, 1.0, 7.0]], 
[[4.0, 5.0, 8.0], [2.0, 1.0, 7.0], [2.0, 7.0, 3.0]]], [[[8.0, 5.0, 7.0], [1.0, 6.0, 6.0], [5.0, 5.0, 7.0]], [[2.0, 3.0, 1.0], 
[1.0, 7.0, 3.0], [4.0, 5.0, 2.0]], [[3.0, 3.0, 5.0], [5.0, 4.0, 4.0], [2.0, 7.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_LRUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 6.0, 7.0], [4.0, 1.0, 1.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 2.0], [7.0, 1.0, 3.0], [1.0, 
1.0, 3.0]], [[5.0, 2.0, 7.0], [1.0, 8.0, 4.0], [2.0, 6.0, 2.0]]], [[[4.0, 5.0, 7.0], [8.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[3.0, 
5.0, 3.0], [8.0, 2.0, 4.0], [1.0, 2.0, 6.0]], [[7.0, 4.0, 7.0], [6.0, 1.0, 7.0], [7.0, 5.0, 4.0]]], [[[2.0, 7.0, 7.0], [4.0, 
4.0, 4.0], [6.0, 6.0, 8.0]], [[6.0, 1.0, 5.0], [2.0, 2.0, 8.0], [2.0, 7.0, 5.0]], [[3.0, 6.0, 6.0], [3.0, 4.0, 1.0], [7.0, 7.0, 
4.0]]], [[[2.0, 2.0, 3.0], [3.0, 6.0, 2.0], [5.0, 4.0, 2.0]], [[2.0, 8.0, 5.0], [6.0, 7.0, 3.0], [6.0, 5.0, 2.0]], [[5.0, 7.0, 
5.0], [8.0, 5.0, 3.0], [3.0, 4.0, 5.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 3.0, 7.0], [5.0, 5.0, 1.0], [2.0, 2.0, 1.0]], 
[[8.0, 8.0, 4.0], [8.0, 8.0, 2.0], [8.0, 3.0, 8.0]], [[1.0, 4.0, 7.0], [6.0, 4.0, 4.0], [3.0, 4.0, 3.0]]], [[[8.0, 4.0, 5.0], 
[3.0, 3.0, 8.0], [4.0, 4.0, 6.0]], [[6.0, 5.0, 5.0], [1.0, 4.0, 1.0], [3.0, 3.0, 6.0]], [[2.0, 3.0, 3.0], [8.0, 1.0, 1.0], 
[8.0, 6.0, 3.0]]], [[[1.0, 7.0, 3.0], [7.0, 3.0, 1.0], [1.0, 7.0, 7.0]], [[4.0, 1.0, 7.0], [5.0, 4.0, 3.0], [2.0, 6.0, 6.0]], 
[[1.0, 3.0, 1.0], [5.0, 5.0, 1.0], [2.0, 8.0, 8.0]]], [[[8.0, 3.0, 1.0], [3.0, 5.0, 4.0], [5.0, 7.0, 8.0]], [[3.0, 8.0, 2.0], 
[5.0, 6.0, 7.0], [1.0, 4.0, 8.0]], [[6.0, 6.0, 8.0], [6.0, 2.0, 5.0], [8.0, 8.0, 1.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 1.0, 6.0], [5.0, 5.0, 8.0]]], [[[8.0, 7.0, 4.0], [5.0, 5.0, 
5.0]]]])+(1.-msk_val)*numpy.array([[[[2.0, 8.0, 4.0], [6.0, 5.0, 4.0]]], [[[2.0, 3.0, 1.0], [6.0, 4.0, 2.0]]]])
      arg[2:,1:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 6.0, 7.0], [4.0, 1.0, 1.0], [2.0, 1.0, 3.0]], [[7.0, 2.0, 2.0], [7.0, 1.0, 3.0], [1.0, 
1.0, 3.0]], [[5.0, 2.0, 7.0], [1.0, 8.0, 4.0], [2.0, 6.0, 2.0]]], [[[4.0, 5.0, 7.0], [8.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[3.0, 
5.0, 3.0], [8.0, 2.0, 4.0], [1.0, 2.0, 6.0]], [[7.0, 4.0, 7.0], [6.0, 1.0, 7.0], [7.0, 5.0, 4.0]]], [[[2.0, 7.0, 7.0], [4.0, 
4.0, 4.0], [6.0, 6.0, 8.0]], [[3.0, 1.0, 6.0], [5.0, 5.0, 8.0], [2.0, 7.0, 5.0]], [[3.0, 6.0, 6.0], [3.0, 4.0, 1.0], [7.0, 7.0, 
4.0]]], [[[2.0, 2.0, 3.0], [3.0, 6.0, 2.0], [5.0, 4.0, 2.0]], [[8.0, 7.0, 4.0], [5.0, 5.0, 5.0], [6.0, 5.0, 2.0]], [[5.0, 7.0, 
5.0], [8.0, 5.0, 3.0], [3.0, 4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 3.0, 7.0], [5.0, 5.0, 1.0], [2.0, 2.0, 1.0]], 
[[8.0, 8.0, 4.0], [8.0, 8.0, 2.0], [8.0, 3.0, 8.0]], [[1.0, 4.0, 7.0], [6.0, 4.0, 4.0], [3.0, 4.0, 3.0]]], [[[8.0, 4.0, 5.0], 
[3.0, 3.0, 8.0], [4.0, 4.0, 6.0]], [[6.0, 5.0, 5.0], [1.0, 4.0, 1.0], [3.0, 3.0, 6.0]], [[2.0, 3.0, 3.0], [8.0, 1.0, 1.0], 
[8.0, 6.0, 3.0]]], [[[1.0, 7.0, 3.0], [7.0, 3.0, 1.0], [1.0, 7.0, 7.0]], [[2.0, 8.0, 4.0], [6.0, 5.0, 4.0], [2.0, 6.0, 6.0]], 
[[1.0, 3.0, 1.0], [5.0, 5.0, 1.0], [2.0, 8.0, 8.0]]], [[[8.0, 3.0, 1.0], [3.0, 5.0, 4.0], [5.0, 7.0, 8.0]], [[2.0, 3.0, 1.0], 
[6.0, 4.0, 2.0], [1.0, 4.0, 8.0]], [[6.0, 6.0, 8.0], [6.0, 2.0, 5.0], [8.0, 8.0, 1.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_UUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[3.0, 1.0, 7.0], [4.0, 2.0, 5.0], [6.0, 6.0, 6.0]], [[3.0, 3.0, 1.0], [2.0, 2.0, 8.0], [8.0, 
3.0, 4.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 5.0], [8.0, 1.0, 1.0]]], [[[7.0, 6.0, 2.0], [7.0, 1.0, 7.0], [4.0, 5.0, 6.0]], [[6.0, 
4.0, 1.0], [3.0, 8.0, 6.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [8.0, 2.0, 5.0], [7.0, 3.0, 5.0]]], [[[5.0, 3.0, 7.0], [8.0, 
6.0, 5.0], [7.0, 1.0, 6.0]], [[3.0, 8.0, 2.0], [3.0, 5.0, 1.0], [5.0, 2.0, 4.0]], [[3.0, 4.0, 2.0], [8.0, 3.0, 8.0], [1.0, 2.0, 
6.0]]], [[[8.0, 6.0, 2.0], [7.0, 2.0, 4.0], [7.0, 5.0, 7.0]], [[1.0, 6.0, 1.0], [8.0, 4.0, 6.0], [8.0, 8.0, 3.0]], [[2.0, 7.0, 
5.0], [5.0, 1.0, 2.0], [5.0, 3.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 8.0, 7.0], [5.0, 2.0, 3.0], [6.0, 4.0, 3.0]], 
[[6.0, 5.0, 2.0], [5.0, 3.0, 6.0], [8.0, 5.0, 8.0]], [[4.0, 5.0, 8.0], [8.0, 3.0, 8.0], [5.0, 6.0, 2.0]]], [[[1.0, 8.0, 6.0], 
[7.0, 1.0, 2.0], [1.0, 7.0, 7.0]], [[1.0, 2.0, 8.0], [7.0, 8.0, 6.0], [8.0, 4.0, 5.0]], [[4.0, 8.0, 1.0], [4.0, 1.0, 1.0], 
[3.0, 8.0, 2.0]]], [[[1.0, 2.0, 2.0], [6.0, 1.0, 2.0], [6.0, 1.0, 8.0]], [[7.0, 2.0, 5.0], [2.0, 8.0, 1.0], [5.0, 5.0, 3.0]], 
[[6.0, 7.0, 7.0], [3.0, 1.0, 6.0], [3.0, 2.0, 7.0]]], [[[3.0, 1.0, 2.0], [6.0, 7.0, 3.0], [8.0, 4.0, 1.0]], [[6.0, 2.0, 4.0], 
[1.0, 4.0, 2.0], [4.0, 3.0, 5.0]], [[5.0, 3.0, 5.0], [5.0, 7.0, 8.0], [4.0, 6.0, 7.0]]]])
      val=numpy.array([[[[1.0, 6.0, 2.0], [4.0, 6.0, 3.0]], [[8.0, 6.0, 7.0], [1.0, 5.0, 3.0]]], [[[5.0, 6.0, 8.0], [7.0, 8.0, 
6.0]], [[4.0, 8.0, 6.0], [7.0, 6.0, 3.0]]], [[[4.0, 1.0, 2.0], [7.0, 1.0, 7.0]], [[6.0, 8.0, 8.0], [6.0, 4.0, 7.0]]]])
      arg[:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0, 6.0, 2.0], [4.0, 6.0, 3.0], [6.0, 6.0, 6.0]], [[8.0, 6.0, 7.0], [1.0, 5.0, 3.0], [8.0, 
3.0, 4.0]], [[6.0, 3.0, 7.0], [4.0, 8.0, 5.0], [8.0, 1.0, 1.0]]], [[[5.0, 6.0, 8.0], [7.0, 8.0, 6.0], [4.0, 5.0, 6.0]], [[4.0, 
8.0, 6.0], [7.0, 6.0, 3.0], [5.0, 7.0, 6.0]], [[2.0, 6.0, 8.0], [8.0, 2.0, 5.0], [7.0, 3.0, 5.0]]], [[[4.0, 1.0, 2.0], [7.0, 
1.0, 7.0], [7.0, 1.0, 6.0]], [[6.0, 8.0, 8.0], [6.0, 4.0, 7.0], [5.0, 2.0, 4.0]], [[3.0, 4.0, 2.0], [8.0, 3.0, 8.0], [1.0, 2.0, 
6.0]]], [[[8.0, 6.0, 2.0], [7.0, 2.0, 4.0], [7.0, 5.0, 7.0]], [[1.0, 6.0, 1.0], [8.0, 4.0, 6.0], [8.0, 8.0, 3.0]], [[2.0, 7.0, 
5.0], [5.0, 1.0, 2.0], [5.0, 3.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 6.0, 2.0], [4.0, 6.0, 3.0], [6.0, 4.0, 3.0]], 
[[8.0, 6.0, 7.0], [1.0, 5.0, 3.0], [8.0, 5.0, 8.0]], [[4.0, 5.0, 8.0], [8.0, 3.0, 8.0], [5.0, 6.0, 2.0]]], [[[5.0, 6.0, 8.0], 
[7.0, 8.0, 6.0], [1.0, 7.0, 7.0]], [[4.0, 8.0, 6.0], [7.0, 6.0, 3.0], [8.0, 4.0, 5.0]], [[4.0, 8.0, 1.0], [4.0, 1.0, 1.0], 
[3.0, 8.0, 2.0]]], [[[4.0, 1.0, 2.0], [7.0, 1.0, 7.0], [6.0, 1.0, 8.0]], [[6.0, 8.0, 8.0], [6.0, 4.0, 7.0], [5.0, 5.0, 3.0]], 
[[6.0, 7.0, 7.0], [3.0, 1.0, 6.0], [3.0, 2.0, 7.0]]], [[[3.0, 1.0, 2.0], [6.0, 7.0, 3.0], [8.0, 4.0, 1.0]], [[6.0, 2.0, 4.0], 
[1.0, 4.0, 2.0], [4.0, 3.0, 5.0]], [[5.0, 3.0, 5.0], [5.0, 7.0, 8.0], [4.0, 6.0, 7.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_UUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 4.0, 3.0], [1.0, 8.0, 6.0], [7.0, 6.0, 2.0]], [[5.0, 7.0, 4.0], [5.0, 4.0, 8.0], [8.0, 
2.0, 2.0]], [[8.0, 4.0, 5.0], [7.0, 1.0, 8.0], [4.0, 2.0, 2.0]]], [[[4.0, 4.0, 7.0], [4.0, 6.0, 1.0], [1.0, 3.0, 7.0]], [[7.0, 
6.0, 1.0], [3.0, 2.0, 8.0], [6.0, 8.0, 1.0]], [[8.0, 5.0, 4.0], [4.0, 8.0, 1.0], [1.0, 7.0, 3.0]]], [[[7.0, 7.0, 1.0], [8.0, 
2.0, 3.0], [3.0, 5.0, 5.0]], [[8.0, 1.0, 2.0], [7.0, 7.0, 7.0], [6.0, 5.0, 5.0]], [[6.0, 8.0, 6.0], [1.0, 2.0, 8.0], [6.0, 7.0, 
4.0]]], [[[6.0, 2.0, 3.0], [5.0, 3.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 8.0], [2.0, 4.0, 2.0], [7.0, 3.0, 1.0]], [[7.0, 1.0, 
1.0], [1.0, 8.0, 2.0], [8.0, 7.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[2.0, 2.0, 5.0], [2.0, 8.0, 6.0], [1.0, 1.0, 6.0]], 
[[8.0, 2.0, 7.0], [1.0, 8.0, 8.0], [6.0, 2.0, 3.0]], [[5.0, 2.0, 2.0], [3.0, 5.0, 2.0], [1.0, 3.0, 2.0]]], [[[2.0, 8.0, 6.0], 
[6.0, 3.0, 6.0], [4.0, 3.0, 7.0]], [[5.0, 3.0, 2.0], [7.0, 7.0, 1.0], [2.0, 4.0, 5.0]], [[5.0, 2.0, 8.0], [8.0, 4.0, 2.0], 
[7.0, 7.0, 8.0]]], [[[1.0, 7.0, 2.0], [2.0, 5.0, 2.0], [8.0, 8.0, 3.0]], [[7.0, 5.0, 3.0], [8.0, 5.0, 2.0], [3.0, 1.0, 3.0]], 
[[7.0, 4.0, 6.0], [4.0, 4.0, 1.0], [4.0, 1.0, 7.0]]], [[[1.0, 6.0, 6.0], [8.0, 8.0, 6.0], [6.0, 7.0, 2.0]], [[6.0, 8.0, 1.0], 
[1.0, 1.0, 8.0], [6.0, 6.0, 7.0]], [[6.0, 6.0, 4.0], [1.0, 7.0, 6.0], [7.0, 4.0, 2.0]]]])
      val=Data(numpy.array([[[[5.0, 2.0, 2.0], [5.0, 1.0, 5.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 7.0]]], [[[8.0, 2.0, 2.0], [3.0, 
2.0, 6.0]], [[1.0, 8.0, 3.0], [5.0, 4.0, 1.0]]], [[[3.0, 6.0, 3.0], [4.0, 1.0, 8.0]], [[5.0, 2.0, 4.0], [4.0, 1.0, 
4.0]]]]),self.functionspace)
      arg[:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 2.0, 2.0], [5.0, 1.0, 5.0], [7.0, 6.0, 2.0]], [[1.0, 7.0, 8.0], [2.0, 8.0, 7.0], [8.0, 
2.0, 2.0]], [[8.0, 4.0, 5.0], [7.0, 1.0, 8.0], [4.0, 2.0, 2.0]]], [[[8.0, 2.0, 2.0], [3.0, 2.0, 6.0], [1.0, 3.0, 7.0]], [[1.0, 
8.0, 3.0], [5.0, 4.0, 1.0], [6.0, 8.0, 1.0]], [[8.0, 5.0, 4.0], [4.0, 8.0, 1.0], [1.0, 7.0, 3.0]]], [[[3.0, 6.0, 3.0], [4.0, 
1.0, 8.0], [3.0, 5.0, 5.0]], [[5.0, 2.0, 4.0], [4.0, 1.0, 4.0], [6.0, 5.0, 5.0]], [[6.0, 8.0, 6.0], [1.0, 2.0, 8.0], [6.0, 7.0, 
4.0]]], [[[6.0, 2.0, 3.0], [5.0, 3.0, 6.0], [2.0, 5.0, 6.0]], [[6.0, 1.0, 8.0], [2.0, 4.0, 2.0], [7.0, 3.0, 1.0]], [[7.0, 1.0, 
1.0], [1.0, 8.0, 2.0], [8.0, 7.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 2.0, 2.0], [5.0, 1.0, 5.0], [1.0, 1.0, 6.0]], 
[[1.0, 7.0, 8.0], [2.0, 8.0, 7.0], [6.0, 2.0, 3.0]], [[5.0, 2.0, 2.0], [3.0, 5.0, 2.0], [1.0, 3.0, 2.0]]], [[[8.0, 2.0, 2.0], 
[3.0, 2.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, 8.0, 3.0], [5.0, 4.0, 1.0], [2.0, 4.0, 5.0]], [[5.0, 2.0, 8.0], [8.0, 4.0, 2.0], 
[7.0, 7.0, 8.0]]], [[[3.0, 6.0, 3.0], [4.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[5.0, 2.0, 4.0], [4.0, 1.0, 4.0], [3.0, 1.0, 3.0]], 
[[7.0, 4.0, 6.0], [4.0, 4.0, 1.0], [4.0, 1.0, 7.0]]], [[[1.0, 6.0, 6.0], [8.0, 8.0, 6.0], [6.0, 7.0, 2.0]], [[6.0, 8.0, 1.0], 
[1.0, 1.0, 8.0], [6.0, 6.0, 7.0]], [[6.0, 6.0, 4.0], [1.0, 7.0, 6.0], [7.0, 4.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_UUUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 5.0, 1.0], [4.0, 7.0, 4.0], [5.0, 2.0, 3.0]], [[3.0, 3.0, 6.0], [6.0, 4.0, 2.0], [4.0, 
3.0, 4.0]], [[2.0, 6.0, 7.0], [7.0, 2.0, 7.0], [7.0, 5.0, 3.0]]], [[[6.0, 6.0, 3.0], [7.0, 7.0, 7.0], [7.0, 1.0, 4.0]], [[7.0, 
2.0, 3.0], [2.0, 2.0, 6.0], [3.0, 6.0, 1.0]], [[7.0, 5.0, 3.0], [2.0, 8.0, 3.0], [1.0, 6.0, 6.0]]], [[[2.0, 1.0, 8.0], [5.0, 
3.0, 4.0], [4.0, 8.0, 6.0]], [[2.0, 7.0, 5.0], [3.0, 6.0, 4.0], [6.0, 7.0, 5.0]], [[8.0, 3.0, 6.0], [4.0, 7.0, 4.0], [8.0, 5.0, 
1.0]]], [[[3.0, 2.0, 5.0], [4.0, 1.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 4.0, 1.0], [1.0, 5.0, 6.0], [4.0, 4.0, 1.0]], [[1.0, 7.0, 
6.0], [1.0, 7.0, 1.0], [6.0, 1.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 3.0], [5.0, 2.0, 4.0], [7.0, 8.0, 5.0]], 
[[6.0, 8.0, 1.0], [5.0, 7.0, 5.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 3.0], [4.0, 3.0, 8.0], [3.0, 1.0, 1.0]]], [[[2.0, 3.0, 3.0], 
[4.0, 8.0, 7.0], [5.0, 6.0, 2.0]], [[1.0, 6.0, 7.0], [4.0, 3.0, 8.0], [2.0, 3.0, 6.0]], [[8.0, 7.0, 6.0], [8.0, 2.0, 8.0], 
[2.0, 7.0, 3.0]]], [[[2.0, 1.0, 3.0], [2.0, 6.0, 7.0], [5.0, 7.0, 7.0]], [[8.0, 6.0, 5.0], [2.0, 7.0, 4.0], [5.0, 5.0, 6.0]], 
[[6.0, 8.0, 8.0], [8.0, 4.0, 7.0], [5.0, 8.0, 2.0]]], [[[1.0, 6.0, 4.0], [6.0, 6.0, 8.0], [6.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], 
[6.0, 3.0, 8.0], [7.0, 5.0, 7.0]], [[6.0, 5.0, 6.0], [2.0, 3.0, 4.0], [2.0, 2.0, 4.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 7.0, 1.0], [5.0, 2.0, 6.0]], [[4.0, 7.0, 3.0], [1.0, 7.0, 3.0]]], [[[5.0, 3.0, 8.0], 
[6.0, 5.0, 7.0]], [[8.0, 6.0, 8.0], [2.0, 3.0, 1.0]]], [[[8.0, 5.0, 1.0], [6.0, 3.0, 6.0]], [[7.0, 5.0, 2.0], [2.0, 2.0, 
3.0]]]])+(1.-msk_val)*numpy.array([[[[7.0, 7.0, 8.0], [7.0, 5.0, 3.0]], [[1.0, 1.0, 5.0], [1.0, 8.0, 8.0]]], [[[1.0, 3.0, 8.0], 
[4.0, 5.0, 8.0]], [[2.0, 2.0, 5.0], [8.0, 4.0, 4.0]]], [[[8.0, 8.0, 5.0], [6.0, 5.0, 2.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 
2.0]]]])
      arg[:3,:2,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.0, 7.0, 1.0], [5.0, 2.0, 6.0], [5.0, 2.0, 3.0]], [[4.0, 7.0, 3.0], [1.0, 7.0, 3.0], [4.0, 
3.0, 4.0]], [[2.0, 6.0, 7.0], [7.0, 2.0, 7.0], [7.0, 5.0, 3.0]]], [[[5.0, 3.0, 8.0], [6.0, 5.0, 7.0], [7.0, 1.0, 4.0]], [[8.0, 
6.0, 8.0], [2.0, 3.0, 1.0], [3.0, 6.0, 1.0]], [[7.0, 5.0, 3.0], [2.0, 8.0, 3.0], [1.0, 6.0, 6.0]]], [[[8.0, 5.0, 1.0], [6.0, 
3.0, 6.0], [4.0, 8.0, 6.0]], [[7.0, 5.0, 2.0], [2.0, 2.0, 3.0], [6.0, 7.0, 5.0]], [[8.0, 3.0, 6.0], [4.0, 7.0, 4.0], [8.0, 5.0, 
1.0]]], [[[3.0, 2.0, 5.0], [4.0, 1.0, 6.0], [1.0, 2.0, 2.0]], [[5.0, 4.0, 1.0], [1.0, 5.0, 6.0], [4.0, 4.0, 1.0]], [[1.0, 7.0, 
6.0], [1.0, 7.0, 1.0], [6.0, 1.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 8.0], [7.0, 5.0, 3.0], [7.0, 8.0, 5.0]], 
[[1.0, 1.0, 5.0], [1.0, 8.0, 8.0], [5.0, 6.0, 5.0]], [[3.0, 4.0, 3.0], [4.0, 3.0, 8.0], [3.0, 1.0, 1.0]]], [[[1.0, 3.0, 8.0], 
[4.0, 5.0, 8.0], [5.0, 6.0, 2.0]], [[2.0, 2.0, 5.0], [8.0, 4.0, 4.0], [2.0, 3.0, 6.0]], [[8.0, 7.0, 6.0], [8.0, 2.0, 8.0], 
[2.0, 7.0, 3.0]]], [[[8.0, 8.0, 5.0], [6.0, 5.0, 2.0], [5.0, 7.0, 7.0]], [[3.0, 2.0, 5.0], [8.0, 8.0, 2.0], [5.0, 5.0, 6.0]], 
[[6.0, 8.0, 8.0], [8.0, 4.0, 7.0], [5.0, 8.0, 2.0]]], [[[1.0, 6.0, 4.0], [6.0, 6.0, 8.0], [6.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], 
[6.0, 3.0, 8.0], [7.0, 5.0, 7.0]], [[6.0, 5.0, 6.0], [2.0, 3.0, 4.0], [2.0, 2.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_CLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[6.0, 3.0, 3.0], [7.0, 4.0, 5.0], [7.0, 5.0, 4.0]], [[2.0, 3.0, 3.0], [1.0, 7.0, 1.0], [1.0, 
1.0, 4.0]], [[4.0, 2.0, 1.0], [7.0, 2.0, 7.0], [2.0, 8.0, 7.0]]], [[[6.0, 2.0, 4.0], [3.0, 6.0, 3.0], [2.0, 6.0, 3.0]], [[5.0, 
5.0, 1.0], [2.0, 7.0, 5.0], [8.0, 3.0, 1.0]], [[1.0, 2.0, 4.0], [1.0, 6.0, 6.0], [2.0, 6.0, 5.0]]], [[[7.0, 5.0, 7.0], [1.0, 
8.0, 1.0], [2.0, 1.0, 3.0]], [[8.0, 7.0, 3.0], [2.0, 5.0, 5.0], [4.0, 5.0, 2.0]], [[4.0, 7.0, 6.0], [8.0, 1.0, 7.0], [1.0, 3.0, 
6.0]]], [[[8.0, 6.0, 1.0], [5.0, 2.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 3.0, 7.0], [5.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[3.0, 7.0, 
6.0], [7.0, 5.0, 6.0], [6.0, 1.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[3.0, 5.0, 3.0], [2.0, 2.0, 2.0], [2.0, 6.0, 6.0]], 
[[4.0, 3.0, 1.0], [1.0, 5.0, 4.0], [7.0, 1.0, 5.0]], [[8.0, 2.0, 2.0], [2.0, 3.0, 8.0], [1.0, 5.0, 4.0]]], [[[8.0, 1.0, 1.0], 
[2.0, 8.0, 5.0], [8.0, 4.0, 3.0]], [[4.0, 2.0, 8.0], [8.0, 2.0, 6.0], [3.0, 2.0, 1.0]], [[1.0, 3.0, 1.0], [3.0, 7.0, 5.0], 
[5.0, 6.0, 6.0]]], [[[3.0, 8.0, 1.0], [1.0, 8.0, 5.0], [3.0, 3.0, 4.0]], [[3.0, 6.0, 7.0], [5.0, 5.0, 7.0], [8.0, 7.0, 1.0]], 
[[3.0, 7.0, 1.0], [5.0, 1.0, 1.0], [8.0, 8.0, 8.0]]], [[[2.0, 8.0, 3.0], [7.0, 8.0, 8.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 7.0], 
[4.0, 1.0, 6.0], [3.0, 7.0, 7.0]], [[5.0, 6.0, 8.0], [6.0, 8.0, 4.0], [8.0, 6.0, 4.0]]]])
      val=numpy.array([[[[8.0, 8.0, 4.0], [5.0, 4.0, 4.0]]], [[[6.0, 5.0, 5.0], [3.0, 2.0, 4.0]]], [[[2.0, 1.0, 4.0], [2.0, 
1.0, 1.0]]], [[[8.0, 5.0, 2.0], [2.0, 2.0, 2.0]]]])
      arg[:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0, 3.0, 3.0], [7.0, 4.0, 5.0], [7.0, 5.0, 4.0]], [[2.0, 3.0, 3.0], [1.0, 7.0, 1.0], [1.0, 
1.0, 4.0]], [[8.0, 8.0, 4.0], [5.0, 4.0, 4.0], [2.0, 8.0, 7.0]]], [[[6.0, 2.0, 4.0], [3.0, 6.0, 3.0], [2.0, 6.0, 3.0]], [[5.0, 
5.0, 1.0], [2.0, 7.0, 5.0], [8.0, 3.0, 1.0]], [[6.0, 5.0, 5.0], [3.0, 2.0, 4.0], [2.0, 6.0, 5.0]]], [[[7.0, 5.0, 7.0], [1.0, 
8.0, 1.0], [2.0, 1.0, 3.0]], [[8.0, 7.0, 3.0], [2.0, 5.0, 5.0], [4.0, 5.0, 2.0]], [[2.0, 1.0, 4.0], [2.0, 1.0, 1.0], [1.0, 3.0, 
6.0]]], [[[8.0, 6.0, 1.0], [5.0, 2.0, 8.0], [6.0, 5.0, 7.0]], [[4.0, 3.0, 7.0], [5.0, 3.0, 8.0], [3.0, 7.0, 7.0]], [[8.0, 5.0, 
2.0], [2.0, 2.0, 2.0], [6.0, 1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 5.0, 3.0], [2.0, 2.0, 2.0], [2.0, 6.0, 6.0]], 
[[4.0, 3.0, 1.0], [1.0, 5.0, 4.0], [7.0, 1.0, 5.0]], [[8.0, 8.0, 4.0], [5.0, 4.0, 4.0], [1.0, 5.0, 4.0]]], [[[8.0, 1.0, 1.0], 
[2.0, 8.0, 5.0], [8.0, 4.0, 3.0]], [[4.0, 2.0, 8.0], [8.0, 2.0, 6.0], [3.0, 2.0, 1.0]], [[6.0, 5.0, 5.0], [3.0, 2.0, 4.0], 
[5.0, 6.0, 6.0]]], [[[3.0, 8.0, 1.0], [1.0, 8.0, 5.0], [3.0, 3.0, 4.0]], [[3.0, 6.0, 7.0], [5.0, 5.0, 7.0], [8.0, 7.0, 1.0]], 
[[2.0, 1.0, 4.0], [2.0, 1.0, 1.0], [8.0, 8.0, 8.0]]], [[[2.0, 8.0, 3.0], [7.0, 8.0, 8.0], [7.0, 1.0, 2.0]], [[6.0, 4.0, 7.0], 
[4.0, 1.0, 6.0], [3.0, 7.0, 7.0]], [[8.0, 5.0, 2.0], [2.0, 2.0, 2.0], [8.0, 6.0, 4.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_CLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 7.0, 2.0], [6.0, 1.0, 7.0], [1.0, 5.0, 7.0]], [[4.0, 5.0, 7.0], [6.0, 1.0, 4.0], [2.0, 
3.0, 7.0]], [[2.0, 5.0, 4.0], [1.0, 2.0, 4.0], [3.0, 8.0, 2.0]]], [[[8.0, 8.0, 2.0], [5.0, 6.0, 6.0], [8.0, 8.0, 1.0]], [[2.0, 
2.0, 4.0], [2.0, 7.0, 5.0], [3.0, 8.0, 4.0]], [[2.0, 4.0, 5.0], [8.0, 8.0, 8.0], [5.0, 6.0, 2.0]]], [[[5.0, 6.0, 2.0], [5.0, 
2.0, 5.0], [8.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], [4.0, 3.0, 6.0], [6.0, 3.0, 1.0]], [[4.0, 7.0, 3.0], [6.0, 8.0, 3.0], [1.0, 5.0, 
8.0]]], [[[7.0, 2.0, 2.0], [8.0, 8.0, 8.0], [2.0, 1.0, 5.0]], [[7.0, 6.0, 1.0], [2.0, 5.0, 3.0], [5.0, 3.0, 7.0]], [[2.0, 1.0, 
4.0], [4.0, 2.0, 5.0], [4.0, 7.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 3.0, 2.0], [6.0, 1.0, 6.0], [5.0, 3.0, 1.0]], 
[[8.0, 6.0, 2.0], [4.0, 6.0, 4.0], [2.0, 1.0, 8.0]], [[5.0, 4.0, 6.0], [4.0, 4.0, 1.0], [5.0, 1.0, 6.0]]], [[[1.0, 6.0, 2.0], 
[4.0, 8.0, 7.0], [7.0, 6.0, 7.0]], [[8.0, 8.0, 4.0], [7.0, 6.0, 7.0], [1.0, 1.0, 1.0]], [[8.0, 7.0, 5.0], [6.0, 5.0, 4.0], 
[3.0, 2.0, 2.0]]], [[[8.0, 7.0, 6.0], [1.0, 4.0, 7.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 7.0], [6.0, 6.0, 1.0], [4.0, 7.0, 2.0]], 
[[7.0, 5.0, 3.0], [6.0, 7.0, 2.0], [8.0, 2.0, 4.0]]], [[[6.0, 5.0, 3.0], [5.0, 8.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 7.0], 
[2.0, 3.0, 5.0], [1.0, 2.0, 5.0]], [[3.0, 2.0, 1.0], [7.0, 7.0, 4.0], [7.0, 1.0, 2.0]]]])
      val=Data(numpy.array([[[[5.0, 7.0, 4.0], [3.0, 4.0, 5.0]]], [[[4.0, 2.0, 1.0], [6.0, 3.0, 7.0]]], [[[6.0, 7.0, 6.0], 
[1.0, 6.0, 7.0]]], [[[1.0, 8.0, 5.0], [2.0, 4.0, 2.0]]]]),self.functionspace)
      arg[:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, 2.0], [6.0, 1.0, 7.0], [1.0, 5.0, 7.0]], [[4.0, 5.0, 7.0], [6.0, 1.0, 4.0], [2.0, 
3.0, 7.0]], [[5.0, 7.0, 4.0], [3.0, 4.0, 5.0], [3.0, 8.0, 2.0]]], [[[8.0, 8.0, 2.0], [5.0, 6.0, 6.0], [8.0, 8.0, 1.0]], [[2.0, 
2.0, 4.0], [2.0, 7.0, 5.0], [3.0, 8.0, 4.0]], [[4.0, 2.0, 1.0], [6.0, 3.0, 7.0], [5.0, 6.0, 2.0]]], [[[5.0, 6.0, 2.0], [5.0, 
2.0, 5.0], [8.0, 6.0, 4.0]], [[2.0, 5.0, 1.0], [4.0, 3.0, 6.0], [6.0, 3.0, 1.0]], [[6.0, 7.0, 6.0], [1.0, 6.0, 7.0], [1.0, 5.0, 
8.0]]], [[[7.0, 2.0, 2.0], [8.0, 8.0, 8.0], [2.0, 1.0, 5.0]], [[7.0, 6.0, 1.0], [2.0, 5.0, 3.0], [5.0, 3.0, 7.0]], [[1.0, 8.0, 
5.0], [2.0, 4.0, 2.0], [4.0, 7.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 3.0, 2.0], [6.0, 1.0, 6.0], [5.0, 3.0, 1.0]], 
[[8.0, 6.0, 2.0], [4.0, 6.0, 4.0], [2.0, 1.0, 8.0]], [[5.0, 7.0, 4.0], [3.0, 4.0, 5.0], [5.0, 1.0, 6.0]]], [[[1.0, 6.0, 2.0], 
[4.0, 8.0, 7.0], [7.0, 6.0, 7.0]], [[8.0, 8.0, 4.0], [7.0, 6.0, 7.0], [1.0, 1.0, 1.0]], [[4.0, 2.0, 1.0], [6.0, 3.0, 7.0], 
[3.0, 2.0, 2.0]]], [[[8.0, 7.0, 6.0], [1.0, 4.0, 7.0], [3.0, 7.0, 6.0]], [[2.0, 2.0, 7.0], [6.0, 6.0, 1.0], [4.0, 7.0, 2.0]], 
[[6.0, 7.0, 6.0], [1.0, 6.0, 7.0], [8.0, 2.0, 4.0]]], [[[6.0, 5.0, 3.0], [5.0, 8.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 4.0, 7.0], 
[2.0, 3.0, 5.0], [1.0, 2.0, 5.0]], [[1.0, 8.0, 5.0], [2.0, 4.0, 2.0], [7.0, 1.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_CLUN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[2.0, 6.0, 6.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 3.0, 6.0], [3.0, 7.0, 7.0], [8.0, 
5.0, 5.0]], [[1.0, 3.0, 1.0], [6.0, 2.0, 2.0], [1.0, 3.0, 5.0]]], [[[2.0, 1.0, 8.0], [6.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[8.0, 
4.0, 7.0], [7.0, 6.0, 7.0], [2.0, 1.0, 6.0]], [[6.0, 3.0, 7.0], [3.0, 3.0, 7.0], [2.0, 3.0, 7.0]]], [[[8.0, 7.0, 2.0], [6.0, 
3.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 1.0, 7.0], [2.0, 4.0, 6.0], [6.0, 5.0, 4.0]], [[4.0, 8.0, 3.0], [5.0, 2.0, 1.0], [7.0, 8.0, 
3.0]]], [[[7.0, 8.0, 6.0], [4.0, 5.0, 6.0], [5.0, 3.0, 5.0]], [[5.0, 6.0, 6.0], [6.0, 7.0, 4.0], [2.0, 4.0, 3.0]], [[4.0, 5.0, 
7.0], [6.0, 8.0, 2.0], [1.0, 5.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 5.0, 5.0], [4.0, 5.0, 4.0], [2.0, 8.0, 6.0]], 
[[5.0, 3.0, 5.0], [7.0, 4.0, 2.0], [8.0, 1.0, 4.0]], [[7.0, 5.0, 7.0], [8.0, 4.0, 5.0], [5.0, 8.0, 2.0]]], [[[7.0, 7.0, 4.0], 
[7.0, 7.0, 3.0], [1.0, 4.0, 7.0]], [[6.0, 1.0, 4.0], [4.0, 8.0, 8.0], [5.0, 6.0, 6.0]], [[1.0, 2.0, 7.0], [8.0, 6.0, 3.0], 
[8.0, 8.0, 7.0]]], [[[2.0, 5.0, 4.0], [3.0, 1.0, 7.0], [1.0, 1.0, 2.0]], [[2.0, 4.0, 3.0], [7.0, 8.0, 5.0], [3.0, 6.0, 7.0]], 
[[3.0, 7.0, 6.0], [6.0, 1.0, 3.0], [2.0, 3.0, 1.0]]], [[[1.0, 4.0, 3.0], [4.0, 6.0, 2.0], [8.0, 3.0, 2.0]], [[6.0, 1.0, 4.0], 
[7.0, 3.0, 3.0], [6.0, 6.0, 7.0]], [[6.0, 3.0, 1.0], [4.0, 2.0, 8.0], [1.0, 3.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 7.0, 5.0], [5.0, 4.0, 4.0]]], [[[1.0, 2.0, 3.0], [8.0, 5.0, 3.0]]], [[[4.0, 2.0, 4.0], 
[3.0, 2.0, 8.0]]], [[[3.0, 6.0, 8.0], [8.0, 4.0, 8.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 3.0, 5.0], [2.0, 7.0, 6.0]]], 
[[[4.0, 8.0, 7.0], [7.0, 8.0, 1.0]]], [[[1.0, 5.0, 2.0], [2.0, 8.0, 7.0]]], [[[4.0, 5.0, 3.0], [1.0, 5.0, 3.0]]]])
      arg[:,2:,:2]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0, 6.0, 6.0], [3.0, 1.0, 3.0], [8.0, 5.0, 4.0]], [[2.0, 3.0, 6.0], [3.0, 7.0, 7.0], [8.0, 
5.0, 5.0]], [[3.0, 7.0, 5.0], [5.0, 4.0, 4.0], [1.0, 3.0, 5.0]]], [[[2.0, 1.0, 8.0], [6.0, 5.0, 7.0], [2.0, 3.0, 8.0]], [[8.0, 
4.0, 7.0], [7.0, 6.0, 7.0], [2.0, 1.0, 6.0]], [[1.0, 2.0, 3.0], [8.0, 5.0, 3.0], [2.0, 3.0, 7.0]]], [[[8.0, 7.0, 2.0], [6.0, 
3.0, 3.0], [2.0, 5.0, 5.0]], [[6.0, 1.0, 7.0], [2.0, 4.0, 6.0], [6.0, 5.0, 4.0]], [[4.0, 2.0, 4.0], [3.0, 2.0, 8.0], [7.0, 8.0, 
3.0]]], [[[7.0, 8.0, 6.0], [4.0, 5.0, 6.0], [5.0, 3.0, 5.0]], [[5.0, 6.0, 6.0], [6.0, 7.0, 4.0], [2.0, 4.0, 3.0]], [[3.0, 6.0, 
8.0], [8.0, 4.0, 8.0], [1.0, 5.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 5.0, 5.0], [4.0, 5.0, 4.0], [2.0, 8.0, 6.0]], 
[[5.0, 3.0, 5.0], [7.0, 4.0, 2.0], [8.0, 1.0, 4.0]], [[8.0, 3.0, 5.0], [2.0, 7.0, 6.0], [5.0, 8.0, 2.0]]], [[[7.0, 7.0, 4.0], 
[7.0, 7.0, 3.0], [1.0, 4.0, 7.0]], [[6.0, 1.0, 4.0], [4.0, 8.0, 8.0], [5.0, 6.0, 6.0]], [[4.0, 8.0, 7.0], [7.0, 8.0, 1.0], 
[8.0, 8.0, 7.0]]], [[[2.0, 5.0, 4.0], [3.0, 1.0, 7.0], [1.0, 1.0, 2.0]], [[2.0, 4.0, 3.0], [7.0, 8.0, 5.0], [3.0, 6.0, 7.0]], 
[[1.0, 5.0, 2.0], [2.0, 8.0, 7.0], [2.0, 3.0, 1.0]]], [[[1.0, 4.0, 3.0], [4.0, 6.0, 2.0], [8.0, 3.0, 2.0]], [[6.0, 1.0, 4.0], 
[7.0, 3.0, 3.0], [6.0, 6.0, 7.0]], [[4.0, 5.0, 3.0], [1.0, 5.0, 3.0], [1.0, 3.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_RLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[7.0, 6.0, 4.0], [6.0, 4.0, 1.0], [2.0, 1.0, 7.0]], [[8.0, 2.0, 4.0], [8.0, 7.0, 4.0], [1.0, 
8.0, 3.0]], [[5.0, 7.0, 8.0], [5.0, 2.0, 4.0], [1.0, 7.0, 6.0]]], [[[3.0, 3.0, 8.0], [4.0, 5.0, 8.0], [5.0, 4.0, 5.0]], [[5.0, 
8.0, 3.0], [7.0, 1.0, 1.0], [8.0, 5.0, 3.0]], [[2.0, 6.0, 2.0], [7.0, 1.0, 8.0], [8.0, 4.0, 2.0]]], [[[1.0, 3.0, 6.0], [6.0, 
6.0, 8.0], [1.0, 3.0, 5.0]], [[8.0, 5.0, 4.0], [3.0, 1.0, 1.0], [1.0, 3.0, 7.0]], [[3.0, 4.0, 5.0], [7.0, 3.0, 5.0], [4.0, 2.0, 
3.0]]], [[[6.0, 1.0, 3.0], [1.0, 2.0, 8.0], [6.0, 2.0, 4.0]], [[1.0, 6.0, 3.0], [6.0, 6.0, 1.0], [6.0, 4.0, 3.0]], [[3.0, 8.0, 
6.0], [6.0, 3.0, 6.0], [2.0, 6.0, 4.0]]]])+(1.-msk_arg)*numpy.array([[[[6.0, 2.0, 7.0], [1.0, 8.0, 3.0], [7.0, 4.0, 4.0]], 
[[5.0, 3.0, 5.0], [6.0, 3.0, 8.0], [8.0, 8.0, 5.0]], [[5.0, 4.0, 2.0], [7.0, 6.0, 8.0], [2.0, 8.0, 3.0]]], [[[5.0, 5.0, 1.0], 
[6.0, 3.0, 5.0], [6.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 4.0, 2.0], [4.0, 8.0, 2.0]], [[8.0, 4.0, 1.0], [1.0, 4.0, 3.0], 
[2.0, 3.0, 6.0]]], [[[5.0, 2.0, 4.0], [8.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 2.0], [5.0, 2.0, 4.0]], 
[[6.0, 8.0, 2.0], [8.0, 6.0, 7.0], [6.0, 1.0, 7.0]]], [[[3.0, 3.0, 1.0], [1.0, 7.0, 2.0], [1.0, 1.0, 5.0]], [[5.0, 3.0, 7.0], 
[2.0, 4.0, 1.0], [4.0, 1.0, 8.0]], [[1.0, 6.0, 1.0], [8.0, 1.0, 6.0], [2.0, 8.0, 2.0]]]])
      val=numpy.array([[[[4.0, 1.0, 3.0], [8.0, 6.0, 3.0], [8.0, 8.0, 3.0]]], [[[8.0, 3.0, 4.0], [7.0, 8.0, 7.0], [4.0, 3.0, 
2.0]]]])
      arg[1:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.0, 6.0, 4.0], [6.0, 4.0, 1.0], [2.0, 1.0, 7.0]], [[8.0, 2.0, 4.0], [8.0, 7.0, 4.0], [1.0, 
8.0, 3.0]], [[5.0, 7.0, 8.0], [5.0, 2.0, 4.0], [1.0, 7.0, 6.0]]], [[[3.0, 3.0, 8.0], [4.0, 5.0, 8.0], [5.0, 4.0, 5.0]], [[5.0, 
8.0, 3.0], [7.0, 1.0, 1.0], [8.0, 5.0, 3.0]], [[4.0, 1.0, 3.0], [8.0, 6.0, 3.0], [8.0, 8.0, 3.0]]], [[[1.0, 3.0, 6.0], [6.0, 
6.0, 8.0], [1.0, 3.0, 5.0]], [[8.0, 5.0, 4.0], [3.0, 1.0, 1.0], [1.0, 3.0, 7.0]], [[8.0, 3.0, 4.0], [7.0, 8.0, 7.0], [4.0, 3.0, 
2.0]]], [[[6.0, 1.0, 3.0], [1.0, 2.0, 8.0], [6.0, 2.0, 4.0]], [[1.0, 6.0, 3.0], [6.0, 6.0, 1.0], [6.0, 4.0, 3.0]], [[3.0, 8.0, 
6.0], [6.0, 3.0, 6.0], [2.0, 6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, 2.0, 7.0], [1.0, 8.0, 3.0], [7.0, 4.0, 4.0]], 
[[5.0, 3.0, 5.0], [6.0, 3.0, 8.0], [8.0, 8.0, 5.0]], [[5.0, 4.0, 2.0], [7.0, 6.0, 8.0], [2.0, 8.0, 3.0]]], [[[5.0, 5.0, 1.0], 
[6.0, 3.0, 5.0], [6.0, 8.0, 1.0]], [[4.0, 3.0, 1.0], [6.0, 4.0, 2.0], [4.0, 8.0, 2.0]], [[4.0, 1.0, 3.0], [8.0, 6.0, 3.0], 
[8.0, 8.0, 3.0]]], [[[5.0, 2.0, 4.0], [8.0, 3.0, 3.0], [7.0, 6.0, 6.0]], [[6.0, 2.0, 8.0], [7.0, 4.0, 2.0], [5.0, 2.0, 4.0]], 
[[8.0, 3.0, 4.0], [7.0, 8.0, 7.0], [4.0, 3.0, 2.0]]], [[[3.0, 3.0, 1.0], [1.0, 7.0, 2.0], [1.0, 1.0, 5.0]], [[5.0, 3.0, 7.0], 
[2.0, 4.0, 1.0], [4.0, 1.0, 8.0]], [[1.0, 6.0, 1.0], [8.0, 1.0, 6.0], [2.0, 8.0, 2.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_RLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 7.0], [6.0, 4.0, 2.0], [6.0, 2.0, 4.0]], [[3.0, 1.0, 5.0], [3.0, 6.0, 6.0], [7.0, 
7.0, 1.0]], [[2.0, 5.0, 3.0], [5.0, 4.0, 7.0], [1.0, 5.0, 6.0]]], [[[4.0, 7.0, 6.0], [5.0, 4.0, 3.0], [2.0, 5.0, 6.0]], [[8.0, 
3.0, 7.0], [3.0, 8.0, 7.0], [5.0, 1.0, 4.0]], [[5.0, 7.0, 7.0], [8.0, 4.0, 8.0], [8.0, 8.0, 3.0]]], [[[4.0, 7.0, 5.0], [5.0, 
6.0, 8.0], [6.0, 6.0, 7.0]], [[6.0, 8.0, 1.0], [4.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[4.0, 4.0, 6.0], [8.0, 1.0, 6.0], [3.0, 1.0, 
5.0]]], [[[5.0, 1.0, 3.0], [7.0, 3.0, 6.0], [8.0, 2.0, 5.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 3.0], [3.0, 8.0, 5.0]], [[8.0, 4.0, 
3.0], [5.0, 4.0, 8.0], [5.0, 6.0, 8.0]]]])+(1.-msk_arg)*numpy.array([[[[1.0, 2.0, 7.0], [5.0, 1.0, 5.0], [4.0, 8.0, 7.0]], 
[[3.0, 5.0, 6.0], [8.0, 6.0, 3.0], [7.0, 7.0, 5.0]], [[2.0, 3.0, 8.0], [7.0, 7.0, 4.0], [5.0, 6.0, 7.0]]], [[[8.0, 6.0, 4.0], 
[5.0, 1.0, 4.0], [1.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [5.0, 1.0, 7.0], [8.0, 8.0, 6.0]], [[2.0, 5.0, 4.0], [1.0, 7.0, 4.0], 
[7.0, 6.0, 6.0]]], [[[2.0, 1.0, 5.0], [5.0, 3.0, 5.0], [7.0, 5.0, 3.0]], [[8.0, 3.0, 8.0], [6.0, 1.0, 5.0], [6.0, 5.0, 3.0]], 
[[3.0, 8.0, 4.0], [5.0, 5.0, 6.0], [7.0, 3.0, 3.0]]], [[[3.0, 6.0, 5.0], [8.0, 4.0, 8.0], [3.0, 4.0, 1.0]], [[7.0, 4.0, 7.0], 
[4.0, 3.0, 6.0], [2.0, 4.0, 7.0]], [[2.0, 7.0, 8.0], [6.0, 1.0, 8.0], [4.0, 7.0, 5.0]]]])
      val=Data(numpy.array([[[[3.0, 6.0, 7.0], [3.0, 8.0, 3.0], [6.0, 3.0, 4.0]]], [[[1.0, 2.0, 5.0], [8.0, 8.0, 4.0], [7.0, 
2.0, 8.0]]]]),self.functionspace)
      arg[1:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 7.0], [6.0, 4.0, 2.0], [6.0, 2.0, 4.0]], [[3.0, 1.0, 5.0], [3.0, 6.0, 6.0], [7.0, 
7.0, 1.0]], [[2.0, 5.0, 3.0], [5.0, 4.0, 7.0], [1.0, 5.0, 6.0]]], [[[4.0, 7.0, 6.0], [5.0, 4.0, 3.0], [2.0, 5.0, 6.0]], [[8.0, 
3.0, 7.0], [3.0, 8.0, 7.0], [5.0, 1.0, 4.0]], [[3.0, 6.0, 7.0], [3.0, 8.0, 3.0], [6.0, 3.0, 4.0]]], [[[4.0, 7.0, 5.0], [5.0, 
6.0, 8.0], [6.0, 6.0, 7.0]], [[6.0, 8.0, 1.0], [4.0, 1.0, 3.0], [6.0, 8.0, 6.0]], [[1.0, 2.0, 5.0], [8.0, 8.0, 4.0], [7.0, 2.0, 
8.0]]], [[[5.0, 1.0, 3.0], [7.0, 3.0, 6.0], [8.0, 2.0, 5.0]], [[7.0, 5.0, 6.0], [3.0, 3.0, 3.0], [3.0, 8.0, 5.0]], [[8.0, 4.0, 
3.0], [5.0, 4.0, 8.0], [5.0, 6.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, 2.0, 7.0], [5.0, 1.0, 5.0], [4.0, 8.0, 7.0]], 
[[3.0, 5.0, 6.0], [8.0, 6.0, 3.0], [7.0, 7.0, 5.0]], [[2.0, 3.0, 8.0], [7.0, 7.0, 4.0], [5.0, 6.0, 7.0]]], [[[8.0, 6.0, 4.0], 
[5.0, 1.0, 4.0], [1.0, 4.0, 8.0]], [[1.0, 5.0, 1.0], [5.0, 1.0, 7.0], [8.0, 8.0, 6.0]], [[3.0, 6.0, 7.0], [3.0, 8.0, 3.0], 
[6.0, 3.0, 4.0]]], [[[2.0, 1.0, 5.0], [5.0, 3.0, 5.0], [7.0, 5.0, 3.0]], [[8.0, 3.0, 8.0], [6.0, 1.0, 5.0], [6.0, 5.0, 3.0]], 
[[1.0, 2.0, 5.0], [8.0, 8.0, 4.0], [7.0, 2.0, 8.0]]], [[[3.0, 6.0, 5.0], [8.0, 4.0, 8.0], [3.0, 4.0, 1.0]], [[7.0, 4.0, 7.0], 
[4.0, 3.0, 6.0], [2.0, 4.0, 7.0]], [[2.0, 7.0, 8.0], [6.0, 1.0, 8.0], [4.0, 7.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_RLCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[4.0, 6.0, 2.0], [1.0, 3.0, 4.0], [4.0, 6.0, 5.0]], [[8.0, 1.0, 1.0], [1.0, 5.0, 7.0], [4.0, 
8.0, 6.0]], [[7.0, 3.0, 5.0], [2.0, 4.0, 3.0], [8.0, 3.0, 8.0]]], [[[4.0, 4.0, 7.0], [5.0, 1.0, 5.0], [3.0, 1.0, 3.0]], [[6.0, 
3.0, 1.0], [1.0, 5.0, 5.0], [6.0, 6.0, 8.0]], [[3.0, 5.0, 7.0], [8.0, 8.0, 1.0], [4.0, 7.0, 4.0]]], [[[3.0, 6.0, 1.0], [3.0, 
7.0, 5.0], [2.0, 8.0, 7.0]], [[2.0, 8.0, 4.0], [3.0, 1.0, 2.0], [2.0, 4.0, 3.0]], [[1.0, 2.0, 3.0], [2.0, 1.0, 1.0], [2.0, 5.0, 
1.0]]], [[[7.0, 4.0, 6.0], [7.0, 4.0, 8.0], [6.0, 8.0, 5.0]], [[2.0, 4.0, 1.0], [3.0, 2.0, 3.0], [1.0, 6.0, 7.0]], [[6.0, 3.0, 
7.0], [7.0, 4.0, 2.0], [6.0, 5.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[7.0, 7.0, 8.0], [6.0, 8.0, 6.0], [3.0, 5.0, 7.0]], 
[[8.0, 6.0, 1.0], [4.0, 8.0, 2.0], [3.0, 5.0, 5.0]], [[5.0, 8.0, 6.0], [1.0, 1.0, 5.0], [4.0, 8.0, 7.0]]], [[[7.0, 6.0, 1.0], 
[2.0, 5.0, 7.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 3.0], [1.0, 2.0, 3.0], [5.0, 8.0, 3.0]], [[4.0, 7.0, 1.0], [5.0, 3.0, 6.0], 
[5.0, 3.0, 4.0]]], [[[7.0, 4.0, 5.0], [7.0, 4.0, 8.0], [4.0, 6.0, 6.0]], [[4.0, 3.0, 4.0], [8.0, 6.0, 6.0], [3.0, 2.0, 3.0]], 
[[7.0, 3.0, 5.0], [8.0, 1.0, 1.0], [4.0, 2.0, 2.0]]], [[[1.0, 1.0, 5.0], [7.0, 6.0, 6.0], [1.0, 3.0, 3.0]], [[8.0, 6.0, 7.0], 
[4.0, 5.0, 5.0], [7.0, 8.0, 1.0]], [[7.0, 7.0, 7.0], [7.0, 8.0, 1.0], [5.0, 6.0, 8.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[[3.0, 8.0, 3.0], [2.0, 4.0, 3.0], [6.0, 6.0, 5.0]]], [[[7.0, 4.0, 5.0], [1.0, 7.0, 6.0], [2.0, 
2.0, 7.0]]]])+(1.-msk_val)*numpy.array([[[[8.0, 7.0, 3.0], [4.0, 8.0, 4.0], [3.0, 7.0, 6.0]]], [[[7.0, 1.0, 5.0], [4.0, 3.0, 
7.0], [8.0, 5.0, 1.0]]]])
      arg[1:3,2:,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0, 6.0, 2.0], [1.0, 3.0, 4.0], [4.0, 6.0, 5.0]], [[8.0, 1.0, 1.0], [1.0, 5.0, 7.0], [4.0, 
8.0, 6.0]], [[7.0, 3.0, 5.0], [2.0, 4.0, 3.0], [8.0, 3.0, 8.0]]], [[[4.0, 4.0, 7.0], [5.0, 1.0, 5.0], [3.0, 1.0, 3.0]], [[6.0, 
3.0, 1.0], [1.0, 5.0, 5.0], [6.0, 6.0, 8.0]], [[3.0, 8.0, 3.0], [2.0, 4.0, 3.0], [6.0, 6.0, 5.0]]], [[[3.0, 6.0, 1.0], [3.0, 
7.0, 5.0], [2.0, 8.0, 7.0]], [[2.0, 8.0, 4.0], [3.0, 1.0, 2.0], [2.0, 4.0, 3.0]], [[7.0, 4.0, 5.0], [1.0, 7.0, 6.0], [2.0, 2.0, 
7.0]]], [[[7.0, 4.0, 6.0], [7.0, 4.0, 8.0], [6.0, 8.0, 5.0]], [[2.0, 4.0, 1.0], [3.0, 2.0, 3.0], [1.0, 6.0, 7.0]], [[6.0, 3.0, 
7.0], [7.0, 4.0, 2.0], [6.0, 5.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 7.0, 8.0], [6.0, 8.0, 6.0], [3.0, 5.0, 7.0]], 
[[8.0, 6.0, 1.0], [4.0, 8.0, 2.0], [3.0, 5.0, 5.0]], [[5.0, 8.0, 6.0], [1.0, 1.0, 5.0], [4.0, 8.0, 7.0]]], [[[7.0, 6.0, 1.0], 
[2.0, 5.0, 7.0], [5.0, 2.0, 8.0]], [[1.0, 3.0, 3.0], [1.0, 2.0, 3.0], [5.0, 8.0, 3.0]], [[8.0, 7.0, 3.0], [4.0, 8.0, 4.0], 
[3.0, 7.0, 6.0]]], [[[7.0, 4.0, 5.0], [7.0, 4.0, 8.0], [4.0, 6.0, 6.0]], [[4.0, 3.0, 4.0], [8.0, 6.0, 6.0], [3.0, 2.0, 3.0]], 
[[7.0, 1.0, 5.0], [4.0, 3.0, 7.0], [8.0, 5.0, 1.0]]], [[[1.0, 1.0, 5.0], [7.0, 6.0, 6.0], [1.0, 3.0, 3.0]], [[8.0, 6.0, 7.0], 
[4.0, 5.0, 5.0], [7.0, 8.0, 1.0]], [[7.0, 7.0, 7.0], [7.0, 8.0, 1.0], [5.0, 6.0, 8.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_array_PRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[8.0, 2.0, 2.0], [6.0, 1.0, 6.0], [3.0, 4.0, 1.0]], [[1.0, 5.0, 4.0], [1.0, 6.0, 5.0], [7.0, 
7.0, 8.0]], [[4.0, 4.0, 5.0], [3.0, 4.0, 6.0], [5.0, 4.0, 4.0]]], [[[5.0, 8.0, 7.0], [6.0, 1.0, 7.0], [8.0, 1.0, 5.0]], [[6.0, 
6.0, 7.0], [6.0, 1.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 2.0, 2.0], [1.0, 3.0, 4.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 8.0], [6.0, 
8.0, 8.0], [4.0, 4.0, 8.0]], [[8.0, 8.0, 3.0], [6.0, 3.0, 1.0], [1.0, 7.0, 5.0]], [[6.0, 3.0, 7.0], [6.0, 1.0, 6.0], [4.0, 6.0, 
5.0]]], [[[2.0, 5.0, 1.0], [8.0, 6.0, 5.0], [7.0, 2.0, 5.0]], [[1.0, 4.0, 7.0], [6.0, 7.0, 2.0], [1.0, 2.0, 8.0]], [[6.0, 4.0, 
6.0], [5.0, 8.0, 7.0], [1.0, 7.0, 7.0]]]])+(1.-msk_arg)*numpy.array([[[[5.0, 6.0, 5.0], [7.0, 5.0, 4.0], [3.0, 3.0, 3.0]], 
[[8.0, 3.0, 2.0], [1.0, 5.0, 4.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 4.0], [1.0, 3.0, 5.0], [6.0, 6.0, 7.0]]], [[[7.0, 5.0, 1.0], 
[8.0, 6.0, 1.0], [4.0, 4.0, 2.0]], [[2.0, 7.0, 6.0], [2.0, 3.0, 1.0], [7.0, 7.0, 1.0]], [[2.0, 7.0, 8.0], [3.0, 6.0, 4.0], 
[6.0, 6.0, 6.0]]], [[[8.0, 8.0, 7.0], [4.0, 3.0, 3.0], [5.0, 4.0, 8.0]], [[5.0, 3.0, 5.0], [5.0, 1.0, 4.0], [2.0, 6.0, 5.0]], 
[[2.0, 2.0, 3.0], [6.0, 2.0, 4.0], [8.0, 3.0, 6.0]]], [[[4.0, 6.0, 6.0], [8.0, 3.0, 5.0], [5.0, 5.0, 1.0]], [[3.0, 3.0, 8.0], 
[2.0, 8.0, 3.0], [7.0, 8.0, 6.0]], [[2.0, 4.0, 3.0], [2.0, 4.0, 8.0], [5.0, 5.0, 6.0]]]])
      val=numpy.array([[[3.0, 7.0, 5.0], [5.0, 4.0, 6.0], [8.0, 6.0, 2.0]]])
      arg[2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0, 2.0, 2.0], [6.0, 1.0, 6.0], [3.0, 4.0, 1.0]], [[1.0, 5.0, 4.0], [1.0, 6.0, 5.0], [7.0, 
7.0, 8.0]], [[4.0, 4.0, 5.0], [3.0, 4.0, 6.0], [5.0, 4.0, 4.0]]], [[[5.0, 8.0, 7.0], [6.0, 1.0, 7.0], [8.0, 1.0, 5.0]], [[6.0, 
6.0, 7.0], [6.0, 1.0, 1.0], [8.0, 6.0, 7.0]], [[3.0, 2.0, 2.0], [1.0, 3.0, 4.0], [6.0, 1.0, 6.0]]], [[[8.0, 6.0, 8.0], [6.0, 
8.0, 8.0], [4.0, 4.0, 8.0]], [[3.0, 7.0, 5.0], [5.0, 4.0, 6.0], [8.0, 6.0, 2.0]], [[6.0, 3.0, 7.0], [6.0, 1.0, 6.0], [4.0, 6.0, 
5.0]]], [[[2.0, 5.0, 1.0], [8.0, 6.0, 5.0], [7.0, 2.0, 5.0]], [[1.0, 4.0, 7.0], [6.0, 7.0, 2.0], [1.0, 2.0, 8.0]], [[6.0, 4.0, 
6.0], [5.0, 8.0, 7.0], [1.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 6.0, 5.0], [7.0, 5.0, 4.0], [3.0, 3.0, 3.0]], 
[[8.0, 3.0, 2.0], [1.0, 5.0, 4.0], [1.0, 5.0, 4.0]], [[5.0, 7.0, 4.0], [1.0, 3.0, 5.0], [6.0, 6.0, 7.0]]], [[[7.0, 5.0, 1.0], 
[8.0, 6.0, 1.0], [4.0, 4.0, 2.0]], [[2.0, 7.0, 6.0], [2.0, 3.0, 1.0], [7.0, 7.0, 1.0]], [[2.0, 7.0, 8.0], [3.0, 6.0, 4.0], 
[6.0, 6.0, 6.0]]], [[[8.0, 8.0, 7.0], [4.0, 3.0, 3.0], [5.0, 4.0, 8.0]], [[3.0, 7.0, 5.0], [5.0, 4.0, 6.0], [8.0, 6.0, 2.0]], 
[[2.0, 2.0, 3.0], [6.0, 2.0, 4.0], [8.0, 3.0, 6.0]]], [[[4.0, 6.0, 6.0], [8.0, 3.0, 5.0], [5.0, 5.0, 1.0]], [[3.0, 3.0, 8.0], 
[2.0, 8.0, 3.0], [7.0, 8.0, 6.0]], [[2.0, 4.0, 3.0], [2.0, 4.0, 8.0], [5.0, 5.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_constData_PRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 3.0, 8.0], [7.0, 6.0, 8.0], [3.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [1.0, 4.0, 4.0], [2.0, 
3.0, 6.0]], [[6.0, 5.0, 1.0], [5.0, 3.0, 5.0], [8.0, 2.0, 7.0]]], [[[2.0, 7.0, 5.0], [6.0, 7.0, 6.0], [1.0, 7.0, 7.0]], [[3.0, 
3.0, 5.0], [3.0, 1.0, 5.0], [6.0, 5.0, 4.0]], [[3.0, 6.0, 8.0], [2.0, 8.0, 5.0], [7.0, 1.0, 8.0]]], [[[6.0, 6.0, 2.0], [1.0, 
5.0, 7.0], [6.0, 2.0, 6.0]], [[6.0, 6.0, 1.0], [5.0, 8.0, 2.0], [3.0, 4.0, 3.0]], [[3.0, 4.0, 3.0], [8.0, 1.0, 1.0], [3.0, 6.0, 
5.0]]], [[[3.0, 4.0, 8.0], [4.0, 1.0, 8.0], [8.0, 3.0, 1.0]], [[1.0, 1.0, 1.0], [6.0, 1.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 6.0, 
1.0], [6.0, 2.0, 5.0], [8.0, 8.0, 3.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 5.0, 1.0], [1.0, 4.0, 7.0], [3.0, 5.0, 1.0]], 
[[2.0, 2.0, 8.0], [7.0, 2.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 3.0, 2.0], [1.0, 5.0, 4.0], [3.0, 8.0, 8.0]]], [[[8.0, 8.0, 6.0], 
[2.0, 2.0, 1.0], [3.0, 2.0, 4.0]], [[2.0, 6.0, 6.0], [3.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 6.0, 4.0], 
[1.0, 6.0, 5.0]]], [[[5.0, 2.0, 6.0], [6.0, 6.0, 8.0], [3.0, 8.0, 1.0]], [[3.0, 6.0, 8.0], [6.0, 6.0, 7.0], [6.0, 5.0, 7.0]], 
[[8.0, 8.0, 8.0], [4.0, 5.0, 5.0], [3.0, 3.0, 5.0]]], [[[5.0, 1.0, 4.0], [5.0, 7.0, 3.0], [5.0, 6.0, 7.0]], [[8.0, 3.0, 5.0], 
[3.0, 1.0, 4.0], [8.0, 6.0, 4.0]], [[3.0, 6.0, 5.0], [8.0, 6.0, 2.0], [3.0, 1.0, 6.0]]]])
      val=Data(numpy.array([[[3.0, 7.0, 1.0], [1.0, 8.0, 6.0], [8.0, 6.0, 5.0]]]),self.functionspace)
      arg[2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 3.0, 8.0], [7.0, 6.0, 8.0], [3.0, 3.0, 3.0]], [[5.0, 7.0, 6.0], [1.0, 4.0, 4.0], [2.0, 
3.0, 6.0]], [[6.0, 5.0, 1.0], [5.0, 3.0, 5.0], [8.0, 2.0, 7.0]]], [[[2.0, 7.0, 5.0], [6.0, 7.0, 6.0], [1.0, 7.0, 7.0]], [[3.0, 
3.0, 5.0], [3.0, 1.0, 5.0], [6.0, 5.0, 4.0]], [[3.0, 6.0, 8.0], [2.0, 8.0, 5.0], [7.0, 1.0, 8.0]]], [[[6.0, 6.0, 2.0], [1.0, 
5.0, 7.0], [6.0, 2.0, 6.0]], [[3.0, 7.0, 1.0], [1.0, 8.0, 6.0], [8.0, 6.0, 5.0]], [[3.0, 4.0, 3.0], [8.0, 1.0, 1.0], [3.0, 6.0, 
5.0]]], [[[3.0, 4.0, 8.0], [4.0, 1.0, 8.0], [8.0, 3.0, 1.0]], [[1.0, 1.0, 1.0], [6.0, 1.0, 1.0], [5.0, 3.0, 7.0]], [[5.0, 6.0, 
1.0], [6.0, 2.0, 5.0], [8.0, 8.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 5.0, 1.0], [1.0, 4.0, 7.0], [3.0, 5.0, 1.0]], 
[[2.0, 2.0, 8.0], [7.0, 2.0, 6.0], [8.0, 5.0, 5.0]], [[3.0, 3.0, 2.0], [1.0, 5.0, 4.0], [3.0, 8.0, 8.0]]], [[[8.0, 8.0, 6.0], 
[2.0, 2.0, 1.0], [3.0, 2.0, 4.0]], [[2.0, 6.0, 6.0], [3.0, 4.0, 6.0], [1.0, 6.0, 8.0]], [[2.0, 6.0, 8.0], [1.0, 6.0, 4.0], 
[1.0, 6.0, 5.0]]], [[[5.0, 2.0, 6.0], [6.0, 6.0, 8.0], [3.0, 8.0, 1.0]], [[3.0, 7.0, 1.0], [1.0, 8.0, 6.0], [8.0, 6.0, 5.0]], 
[[8.0, 8.0, 8.0], [4.0, 5.0, 5.0], [3.0, 3.0, 5.0]]], [[[5.0, 1.0, 4.0], [5.0, 7.0, 3.0], [5.0, 6.0, 7.0]], [[8.0, 3.0, 5.0], 
[3.0, 1.0, 4.0], [8.0, 6.0, 4.0]], [[3.0, 6.0, 5.0], [8.0, 6.0, 2.0], [3.0, 1.0, 6.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_setslice_expandedData_rank4_expandedData_PRCN(self):
      msk_arg=whereNegative(self.functionspace.getX()[0]-0.5)
      arg=msk_arg*numpy.array([[[[5.0, 4.0, 6.0], [6.0, 5.0, 8.0], [7.0, 4.0, 3.0]], [[7.0, 1.0, 7.0], [7.0, 8.0, 8.0], [2.0, 
7.0, 2.0]], [[2.0, 1.0, 6.0], [2.0, 1.0, 4.0], [7.0, 8.0, 2.0]]], [[[2.0, 4.0, 4.0], [5.0, 4.0, 8.0], [5.0, 2.0, 7.0]], [[2.0, 
2.0, 7.0], [1.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[7.0, 3.0, 8.0], [5.0, 3.0, 8.0], [8.0, 1.0, 6.0]]], [[[1.0, 7.0, 8.0], [1.0, 
7.0, 6.0], [7.0, 4.0, 3.0]], [[8.0, 2.0, 7.0], [6.0, 5.0, 1.0], [3.0, 6.0, 8.0]], [[6.0, 6.0, 6.0], [6.0, 7.0, 6.0], [2.0, 1.0, 
4.0]]], [[[1.0, 8.0, 3.0], [5.0, 3.0, 4.0], [5.0, 8.0, 4.0]], [[6.0, 8.0, 6.0], [7.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[3.0, 5.0, 
1.0], [7.0, 3.0, 2.0], [2.0, 5.0, 2.0]]]])+(1.-msk_arg)*numpy.array([[[[8.0, 8.0, 7.0], [1.0, 2.0, 7.0], [3.0, 3.0, 4.0]], 
[[7.0, 4.0, 6.0], [6.0, 3.0, 3.0], [4.0, 2.0, 2.0]], [[5.0, 5.0, 3.0], [5.0, 6.0, 8.0], [4.0, 4.0, 1.0]]], [[[2.0, 2.0, 8.0], 
[5.0, 5.0, 8.0], [2.0, 4.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 6.0, 4.0], [6.0, 3.0, 5.0]], [[3.0, 7.0, 8.0], [3.0, 7.0, 7.0], 
[3.0, 1.0, 2.0]]], [[[2.0, 3.0, 7.0], [1.0, 7.0, 1.0], [8.0, 3.0, 2.0]], [[7.0, 1.0, 5.0], [7.0, 2.0, 3.0], [7.0, 3.0, 7.0]], 
[[7.0, 6.0, 4.0], [7.0, 6.0, 4.0], [5.0, 2.0, 1.0]]], [[[3.0, 6.0, 3.0], [6.0, 2.0, 6.0], [4.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], 
[6.0, 6.0, 7.0], [2.0, 7.0, 3.0]], [[1.0, 1.0, 2.0], [4.0, 6.0, 8.0], [4.0, 6.0, 5.0]]]])
      msk_val=whereNegative(self.functionspace.getX()[0]-0.5)
      val=msk_val*numpy.array([[[4.0, 4.0, 5.0], [5.0, 8.0, 8.0], [7.0, 2.0, 8.0]]])+(1.-msk_val)*numpy.array([[[2.0, 6.0, 
3.0], [2.0, 1.0, 4.0], [7.0, 7.0, 7.0]]])
      arg[2,1:2,:]=val
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0, 4.0, 6.0], [6.0, 5.0, 8.0], [7.0, 4.0, 3.0]], [[7.0, 1.0, 7.0], [7.0, 8.0, 8.0], [2.0, 
7.0, 2.0]], [[2.0, 1.0, 6.0], [2.0, 1.0, 4.0], [7.0, 8.0, 2.0]]], [[[2.0, 4.0, 4.0], [5.0, 4.0, 8.0], [5.0, 2.0, 7.0]], [[2.0, 
2.0, 7.0], [1.0, 1.0, 8.0], [8.0, 8.0, 3.0]], [[7.0, 3.0, 8.0], [5.0, 3.0, 8.0], [8.0, 1.0, 6.0]]], [[[1.0, 7.0, 8.0], [1.0, 
7.0, 6.0], [7.0, 4.0, 3.0]], [[4.0, 4.0, 5.0], [5.0, 8.0, 8.0], [7.0, 2.0, 8.0]], [[6.0, 6.0, 6.0], [6.0, 7.0, 6.0], [2.0, 1.0, 
4.0]]], [[[1.0, 8.0, 3.0], [5.0, 3.0, 4.0], [5.0, 8.0, 4.0]], [[6.0, 8.0, 6.0], [7.0, 3.0, 1.0], [7.0, 5.0, 5.0]], [[3.0, 5.0, 
1.0], [7.0, 3.0, 2.0], [2.0, 5.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 8.0, 7.0], [1.0, 2.0, 7.0], [3.0, 3.0, 4.0]], 
[[7.0, 4.0, 6.0], [6.0, 3.0, 3.0], [4.0, 2.0, 2.0]], [[5.0, 5.0, 3.0], [5.0, 6.0, 8.0], [4.0, 4.0, 1.0]]], [[[2.0, 2.0, 8.0], 
[5.0, 5.0, 8.0], [2.0, 4.0, 5.0]], [[1.0, 7.0, 7.0], [3.0, 6.0, 4.0], [6.0, 3.0, 5.0]], [[3.0, 7.0, 8.0], [3.0, 7.0, 7.0], 
[3.0, 1.0, 2.0]]], [[[2.0, 3.0, 7.0], [1.0, 7.0, 1.0], [8.0, 3.0, 2.0]], [[2.0, 6.0, 3.0], [2.0, 1.0, 4.0], [7.0, 7.0, 7.0]], 
[[7.0, 6.0, 4.0], [7.0, 6.0, 4.0], [5.0, 2.0, 1.0]]], [[[3.0, 6.0, 3.0], [6.0, 2.0, 6.0], [4.0, 5.0, 3.0]], [[1.0, 7.0, 8.0], 
[6.0, 6.0, 7.0], [2.0, 7.0, 3.0]], [[1.0, 1.0, 2.0], [4.0, 6.0, 8.0], [4.0, 6.0, 5.0]]]])
      self.assertTrue(Lsup(arg-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
